
indexing
	
	description: "[
		List of windows that are candidates for the window
		relative to which error, warning and information
		dialogs are displayed.  First window in list is always
		used for this purpose.
		]"
	author: "David Hollenberg"
	date: "November 23, 2004"

class ACTIVE_WINDOW_LIST
inherit
	ARRAYED_LIST [EV_WINDOW]

create
	make
create {ACTIVE_WINDOW_LIST}
	make_filled

feature -- Properties

	current_window: EV_WINDOW
			-- Window relative to which dialogs should
			-- be shown
		local
			env: EV_ENVIRONMENT
			list: LINEAR [EV_WINDOW]
		do
			from
				start
			until
				after
			loop
				if item.is_destroyed then
					remove
				else
					forth
				end
			end
			if not is_empty then
				Result := first
			end
			if Result = Void then
				create env
				list := env.application.windows
				list.start
				Result := list.item
			end
		ensure
			result_exists: Result /= Void
		end

feature -- Operations

	set_current (w: EV_WINDOW)
			-- Make `current_window' to `w'
		require
			window_exists: w /= Void
		do
			deregister (w)
			put_front (w)
		end

	register (w: EV_WINDOW)
			-- Add `w' to end of list, if not already there
		require
			window_exists: w /= Void
		do
			if not has (w) then
				extend (w)
			end
		end

	deregister (w: EV_WINDOW)
			-- Remove `w' from list, if in list
		require
			window_exists: w /= Void
		do
			search (w)
			if not exhausted then
				remove
			end
		end

end
