
indexing
	
	description: "A run closing step operation that adjusts %
		%charges for the run, if necessary (e.g., may reverse %
		%charges for an educational run or dedicated run)"
	author: "David Hollenberg"
	date: "February 21, 2004"

class ADJUST_CHARGES_OP
inherit
	RC_PRIMITIVE_STEP_OP
	NO_USER_INTERFACE
	NO_EXTERNAL_STATE
	RC_UTILITIES_ACCESS
		export
			{NONE} all
		end

create
	make

feature -- Properties

	input_data_items: LIST [STRING]
			-- Data items required before step can be executed
		do
			Result := array_as_list (<<
				data_item.Production_job
				>>)
		end

feature {NONE} -- Implementation

	is_applicable: BOOLEAN
			-- Is step applicable to run for session?
		do
			Result := cache.is_educational_run or cache.is_dedicated_run
		end

	do_step
			-- Execute of step
		do
			adjust_charges (Plsql_to_reverse_charges)
			add_commit_action (agent publish_charge_adjustment (True))
			call_completion_actions
		end

	undo_step
			-- Undo execution of step
		do
			adjust_charges (Plsql_to_restore_charges)
			add_commit_action (agent publish_charge_adjustment (False))
			call_completion_actions
		end

	set_state
			-- Set `state'
		do
			state := Void
		end

	adjust_charges (text: STRING)
			-- Reverse or restore charges for a run,
			-- based on the PL/SQL block `text'
		require
			text_exists: text /= Void
		local
			b: SQL_BLOCK
			a_reason: STRING
		do
			if cache.is_dedicated_run then
				a_reason := Ded_adjustment_reason
			elseif cache.is_educational_run then
				a_reason := Edu_adjustment_reason
			else
				handle_internal_error ("cannot choose reason for adjustment because%Nrun is neither dedicated nor educational")
			end
			connection.discard_package_state
				-- Avoid "existing state of packages has
				-- been discarded" Oracle error
			b := connection.sql_block (text)
			b.set_named_bind_capacity (2)
			b.bind_name_to_string (Ph_run_id, session.run_id, False)
			b.bind_name_to_string (Ph_reason, a_reason, False)
			b.execute
			b.release
		end
	
	publish_charge_adjustment (rev: BOOLEAN)
			-- Publish info about adjustment of charges, where
			-- `rev' is true if charges were reversed and false
			-- if charges were restored
		local
			event: CHARGE_ADJUSTMENT_EVENT
		do
			create event
			event.set_count (charges_processed_count)
			event.set_amount (adjustment_amount)
			event.set_reverse (rev)
			event.publish
		end
	
	charges_processed_count: INTEGER
			-- Number of charges processed by last call to
			-- `adjust_charges'
		do
			Result := connection.selected_integer ("m_charge_utilities.charges_processed", "dual", Void, Void)
		end
	
	adjustment_amount: DOUBLE
			-- Total amount adjusted by last call to
			-- `adjust_charges'
		do
			Result := connection.selected_double ("m_charge_utilities.adjustment_amount", "dual", Void, Void)
		end
	
	Plsql_to_reverse_charges: STRING = "begin m_charge_utilities.adjust_run_amount_to_zero(:run_id, :reason, True); end;"
	
	Plsql_to_restore_charges: STRING = "begin m_charge_utilities.remove_run_adjustments(:run_id, :reason, True); end;"
	
	Ph_run_id: STRING = "run_id"
			-- Placeholder for run id
	
	Ph_reason: STRING = "reason"
			-- Placeholder for adjustment reason

	Edu_adjustment_reason: STRING = "Educational charges reversed by Delta"
			-- Reason for adjusting charges for educational run

	Ded_adjustment_reason: STRING = "Dedicated run charges reversed by Delta"
			-- Reason for adjusting charges for dedicated run

end
