
indexing
	
	description: "AMI_ABN wafer techn family"
	author: "David Hollenberg";
	date: "June 18, 2003"

class AMI_ABN
inherit
	WAFER_TECHN_FAMILY
	redefine
		is_dynamic_vendor_geometry,
		build_vendor_geometry,
		add_explicit_cats_statements
	end;

	LAYOUT_SUPPORT

feature -- Properties

	name: STRING is "AMI_ABN"

	native_design_techn: STRING is
			-- name of the "native" design_techn
	do  Result := dtv.ami_abn
	end;

feature	-- dynamic vendor geometry support (none unless redefined)

	is_dynamic_vendor_geometry(d: DIES_XROW): BOOLEAN is
		-- indicates whether this die's geometry item is static (files 
		--  already in place) or dynamic (files generated per run)
	do
	    Result := equal(d.geometry_id, gids.ami_abn_partno)
	end;

	build_vendor_geometry(	g: GEOMETRIES_ROW;
				dt: DESIGN_TECHNOLOGY;
				lim: CONST_RECTANGLE): LAYOUT is
		-- only dynamic is ami_abn_partno
	local bx: RECTANGLE
	do  if equal(g.geometry_id, gids.ami_abn_partno) then
		-- the AMI part number as text, rotated 90 degrees and applied to all levels
		create bx.make_copy(lim);
		bx.size_by(-1, -1);		-- shrink 0.5/side
		Result := text (cache.wafer_part_id)
		Result := move (Result, rot90)
		Result := Result.nicely_fitting_in (bx)
		Result := multi_layer (Result, dt.non_cmnt_layers)
	    else
		handle_hard_error("AMI_ABN: no geometry build for " + g.geometry_id)
	    end;
	end;

	add_explicit_cats_statements (d: DIES_XROW) is
		-- compute all of the explicitly-specified Cats statements required to fracture d
		--  and add them onto all_stmts.
	local dt: DESIGN_TECHNOLOGY; dcode: STRING
	do  dt := d.design_techn;
	    dcode := dt.design_techn_id
	    if is_scmos(dcode) then
		active_dogbones(d);
		add_clevel_with_merge(leaf (lyr.pbase), lvl.pbase, 1.0, d)
		if d.is_present (lyr.expl_field_implant) then
			add_clevel_with_bias(leaf (lyr.expl_field_implant) #xor#
					     leaf (source_layer_name(d, lvl.n_well)),
						lvl.n_channel_implant, d)
		end;
		if equal(dcode, dtid.scga) then
			scga_special_selects (d)
		else
			scmos_selects (d)
		end
		add_clevel_with_merge(leaf (lyr.via), lvl.via, 2.2, d);
	    elseif equal(dcode, dtid.ami_abn) then
		if not d.is_present (lyr.n_channel_implant) then
			add_clevel_with_bias(leaf (lyr.n_well), lvl.n_channel_implant, d)
		end
	    elseif equal(dcode, dtid.ami_abn_mebes) then
		-- nada
	    else
		unknown_design_techn(dcode)
	    end
	end;

feature {NONE} -- implementation

	active_dogbones (d: DIES_XROW) is
	local act_src: CATS_EXPRESSION
	do	-- build big dogbone around active contacts
		act_src := (leaf (lyr.active) #sized# nominal_bias(d, lvl.active)) #or#
			   ((leaf (lyr.active) #and# leaf (lyr.contact)) #sized# 
				(3.0 * d.design_techn.lambda + 0.2));
		-- add in the pbase (which is pbase_active, remember??)
		if d.is_present (lyr.pbase) then
			act_src := act_src #or# (#layout_only# leaf (lyr.pbase))
		end;
		-- that's it
		add_clevel (act_src, lvl.active, d)
	end;

	scga_special_selects (d: DIES_XROW) is
	do  -- SCGA special: active inside pbase marks the regions subject to a
	    --  select implant (outside pbase, all regions are select implanted)
	    add_ctemp ((leaf (lyr.pbase) #sized# 4) #minus# leaf (lyr.active), 
						tmp_unselected, 0.1);
	    add_clevel_with_bias (leaf (lyr.select_) #minus# leaf (tmp_unselected),
						lvl.n_plus_select, d);
	    add_clevel_with_bias (#not# (leaf (lyr.select_) #or# leaf (tmp_unselected)),
						lvl.p_plus_select, d);
	end;

	tmp_unselected: STRING is "UNSELECTED"

	scmos_selects (d: DIES_XROW) is
	local hn, hp: BOOLEAN;
	do  -- check for presence
	    hn := d.is_present (lyr.n_plus_select);
	    hp := d.is_present (lyr.p_plus_select);
	    -- fill-in from other select when just one is defined
	    if hp and not hn then
		add_clevel_with_bias (#not# leaf (lyr.p_plus_select),
					lvl.n_plus_select, d);
	    end;
	    if hn and not hp then
		add_clevel_with_bias (#not# leaf (lyr.n_plus_select),
					lvl.p_plus_select, d)
	    end;
	end;

end
