
indexing
	
	description: "AMI_C4 wafer techn family"
	author: "David Hollenberg";
	date: "December 22, 2004"

class AMI_C4
inherit
	WAFER_TECHN_FAMILY
		redefine
			add_explicit_cats_statements
		end;

feature -- Properties

	name: STRING is "AMI_C4"

	native_design_techn: STRING is
			-- name of the "native" design_techn
	do  Result := dtid.ami_c4
	end;

feature	-- statements for Cats fracture script

	add_explicit_cats_statements (d: DIES_XROW) is
	local dtech: STRING
	do  dtech := d.wafer_design_techn;
	    if equal(dtech, dtid.ami_c5) then
		selects_ala_ami(d);
		well_merge(d);
	    elseif is_scmos(dtech) then
		active_dogbone(d)
		selects_ala_mosis(d)
		well_merge(d);
	    elseif equal(dtech, dtid.ami_c5_mebes) then
		-- nada
	    else unknown_design_techn(dtech)
	    end
	end;

feature {NONE} -- details

	active_dogbone(d: DIES_XROW) is
		-- SCMOS only
	local mask: CATS_EXPRESSION
	do  mask := leaf (lyr.active) #or#
		    ((leaf (lyr.active) #and# leaf (lyr.contact))
			#sized# 0.9 #sized# -0.2)
	    add_clevel_with_bias(mask, lvl.active, d)
	end;

	selects_ala_ami(d: DIES_XROW) is
		-- add the selects, which are drawn ami-style
		-- (ami draws an n_plus_block layer, 
		--  which is usually a copy of p_plus_select)
	local hn, hp: BOOLEAN; nmask, pmask: CATS_EXPRESSION
	do  -- check for presence
	    hn := d.is_present (lyr.n_plus_block);
	    hp := d.is_present (lyr.p_plus_select);
	    -- basic expressions for the select masks
	    if hp and not hn then
		nmask := leaf (lyr.p_plus_select)
	    else
		nmask := leaf (lyr.n_plus_block)
	    end;
	    if hn and not hp then
		pmask := leaf (lyr.n_plus_block)
	    else
		pmask := leaf (lyr.p_plus_select)
	    end;
	    -- and put it all out
	    select_cfinal(d, nmask, pmask)
	end;

	selects_ala_mosis(d: DIES_XROW) is
		-- add the selects, which are drawn mosis-style
		-- (i.e., n_plus_select must be reversed
		--   to be an n_plus_block equivalent)
	local hn, hp: BOOLEAN; nmask, pmask: CATS_EXPRESSION
	do  -- check for presence
	    hn := d.is_present (lyr.n_plus_select);
	    hp := d.is_present (lyr.p_plus_select);
	    -- basic expressions for the N/P select masks
	    if hp and not hn then
		nmask := leaf (lyr.p_plus_select)
	    else
		nmask := #not# leaf (lyr.n_plus_select)
	    end;
	    if hn and not hp then
		pmask := #not# leaf (lyr.n_plus_select)
	    else
		pmask := leaf (lyr.p_plus_select)
	    end;
	    -- and put it all out
	    select_cfinal(d, nmask, pmask)
	end;

	select_cfinal(d: DIES_XROW; nmask, pmask: CATS_EXPRESSION) is
		-- just do it
	do  add_clevel_with_merge_X2 (nmask, lvl.n_plus_block, 0.5, d)
	    add_clevel_with_merge_X2 (pmask, lvl.p_plus_select, 0.5, d)
	end;

	well_merge(d: DIES_XROW) is
		-- process well and field implant, with merging
	local nwl, imp: STRING; impmask: CATS_EXPRESSION
	do  -- get some layer names
	    nwl := source_layer_name (d, lvl.n_well);
	    imp := source_layer_name (d, lvl.n_channel_fld);
	    -- compute n_channel_fld source expression
	    if d.is_present (imp) then
			-- if source layer drawn, use it
		impmask := leaf (imp)
	    else	-- if source layer not drawn, copy n_well
		impmask := leaf (nwl)
	    end;
	    if d.is_present (lyr.expl_field_implant) then 
		-- (SCMOS only)
		impmask := impmask #xor# leaf (lyr.expl_field_implant)
	    end;
	    -- and go
	    add_clevel_with_merge (leaf (nwl), lvl.n_well, 3, d)
	    add_clevel_with_merge (impmask, lvl.n_channel_fld, 3, d)
	end;

end
