
indexing
	
	description: "AMI_C5 wafer techn family"
	author: "David Hollenberg";
	date: "June 18, 2003"

class AMI_C5
inherit
	WAFER_TECHN_FAMILY
		redefine add_explicit_cats_statements
		end;

feature -- Properties

	name: STRING is "AMI_C5"

	native_design_techn: STRING is
			-- name of the "native" design_techn
	do  Result := dtid.ami_c5
	end;

	add_explicit_cats_statements (d: DIES_XROW) is
		-- create all of the explicitly-specified Cats statements required to fracture d
		--  and add them onto all_stmts.
	local dt: DESIGN_TECHNOLOGY; dtech: STRING
	do  dt := d.design_techn;
	    dtech := dt.design_techn_id
	    active_src := leaf (source_layer_name (d, lvl.active))
			-- (unless modified)
	    if is_scmos(dtech) or 
	       equal(dtech, dtid.cmosh) or equal(dtech, dtid.cs050n) or
	       equal(dtech, dtid.ami_c5) then
		if equal(dtech, dtid.scn3me_subm) then
			active_dogbone(d)
		end;
		well_merge(d);
		if equal(dtech, dtid.ami_c5) then
			selects_ala_ami(d)
		else
			selects_ala_mosis(d)
		end;
		poly2_dogbone(d);
		thick_gate_mask(d);
		c5t_masks(d)
	    elseif equal(dtech, dtid.ami_c5_mebes) then
		-- nada
	    else unknown_design_techn(dtech)
	    end
	end;

feature {NONE} -- details

	tmp_tgate_src: STRING is "THK_GATE_SRC"	
	tmp_plo_or_pi: STRING is "PLO_OR_PI"
		-- names for explicit temps

	active_src: CATS_EXPRESSION
		-- saved expression to pass around

	have_plo_or_pi(d: DIES_XROW): BOOLEAN is
	do  Result := d.is_present (lyr.plo) or d.is_present (lyr.pi)
	end;

	active_dogbone(d: DIES_XROW) is
		-- add active dogbone around contacts (SCN3ME_SUBM only)
	do  active_src := leaf (lyr.active) #or#
			  ((leaf (lyr.active) #and# leaf (lyr.contact) ) #sized# 0.7 )
	    add_clevel_with_bias (active_src, lvl.active, d)
	end;

	well_merge(d: DIES_XROW) is
		-- process well and field implant, with merging
	local nwl, imp: STRING; impsrc: CATS_EXPRESSION
	do  -- get some layer names
	    nwl := source_layer_name (d, lvl.n_well);
	    imp := source_layer_name (d, lvl.n_channel_fld);
	    -- compute n_channel_fld source expression
	    if d.is_present (imp) then
		impsrc := leaf (imp)
	    else	-- if source layer not drawn, copy n_well
		impsrc := leaf (nwl)
	    end;
	    if d.is_present (lyr.ni) then	-- (AMI_C5 only)
		impsrc := impsrc #or# (active_src #and# leaf (lyr.poly) #and# leaf (lyr.ni))
	    end;
	    if d.is_present (lyr.expl_field_implant) then -- (SCMOS only)
		impsrc := impsrc #xor# leaf (lyr.expl_field_implant)
	    end;
	    -- and go
	    add_clevel_with_merge (leaf (nwl), lvl.n_well, 3, d)
	    add_clevel_with_merge (impsrc, lvl.n_channel_fld, 3, d)
	end;

	selects_ala_ami(d: DIES_XROW) is
		-- add the selects, which are drawn ami-style
		-- (ami uses an n_plus_block, which is usually a copy of p_plus_select)
	local hn, hp: BOOLEAN; nsrc, psrc, nmsrc, pmsrc: CATS_EXPRESSION
	do  -- check for presence
	    hn := d.is_present (lyr.n_plus_block);	-- if (either) one is missing,
	    hp := d.is_present (lyr.p_plus_select);	-- we synthesize from the other
	    -- basic expressions for the N/P minus masks
	    if hp and not hn then
		nmsrc := leaf (lyr.p_plus_select)
	    else
		nmsrc := leaf (lyr.n_plus_block)
	    end;
	    if hn and not hp then
		pmsrc := leaf (lyr.n_plus_block)
	    else
		pmsrc := leaf (lyr.p_plus_select)
	    end;
	    -- added steps if plo or pi are present
	    if have_plo_or_pi (d) then
		add_ctemp (leaf (lyr.plo) #or# leaf (lyr.pi), tmp_plo_or_pi, 0.05);
			-- (this temp is also used for the c5t masks
		nmsrc := nmsrc #or# leaf (tmp_plo_or_pi)
		pmsrc := pmsrc #or# leaf (tmp_plo_or_pi)
	    end;
	    -- now derive N/P select (block) from N/P minus
	    if d.is_present (lyr.n_minus_implant) then
		nsrc := nmsrc #or# leaf (lyr.n_minus_implant)
	    else
		nsrc := nmsrc
	    end;
	    if d.is_present (lyr.p_minus_implant) then
		psrc := pmsrc #minus# leaf (lyr.p_minus_implant)
	    else
		psrc := pmsrc
	    end;
	    -- and put it all out
	    select_cfinal(d, nsrc, psrc, nmsrc, pmsrc)
	end;

	selects_ala_mosis(d: DIES_XROW) is
		-- add the selects, which are drawn mosis-style
		-- (i.e., n_plus_select must be flipped to be an n_plus_block equivalent)
	local hn, hp: BOOLEAN; nsrc, psrc: CATS_EXPRESSION
	do  -- check for presence
	    hn := d.is_present (lyr.n_plus_select);
	    hp := d.is_present (lyr.p_plus_select);
	    -- basic expressions for the N/P select masks
	    if hp and not hn then
		nsrc := leaf (lyr.p_plus_select)
	    else
		nsrc := #not# leaf (lyr.n_plus_select)
	    end;
	    if hn and not hp then
		psrc := #not# leaf (lyr.n_plus_select)
	    else
		psrc := leaf (lyr.p_plus_select)
	    end;
	    -- and put it all out
	    select_cfinal(d, nsrc, psrc, nsrc, psrc)
	end;

	select_cfinal(d: DIES_XROW; nsrc, psrc, nmsrc, pmsrc: CATS_EXPRESSION) is
		-- just do it
	do  add_clevel_with_merge_X2 (nsrc, lvl.n_plus_block, 0.5, d)
	    add_clevel_with_merge_X2 (psrc, lvl.p_plus_select, 0.5, d)
	    if option_is_present (oid.thkgate) then
		add_clevel_with_merge_X2 (nmsrc, lvl.n_minus_implant, 0.5, d);
		add_clevel_with_merge_X2 (pmsrc, lvl.p_minus_implant, 0.5, d)
	    end
	end;

	poly2_dogbone(d: DIES_XROW) is
		-- add poly2 dogbone around contacts (per AMI RICE of 13-apr-2005)
	local p2lyr: STRING; p2src: CATS_EXPRESSION
	do  p2lyr := source_layer_name (d, lvl.cap_poly)
	    if su.not_empty(p2lyr) then
		p2src := leaf (p2lyr) #or#

		    -- was ((leaf (p2lyr) #and# leaf (lyr.contact)) #sized# 1.2 #sized# -.05)
		    -- corrected, 27-May-2008 to
			   ((leaf (p2lyr) #and# leaf (lyr.contact)) #sized# 1.1 #sized# -0.5)
		add_clevel_with_bias (p2src, lvl.cap_poly, d)
	    end
	end;

	thick_gate_mask(d: DIES_XROW) is
		-- process thick gate mask
		-- (the N/P minus masks, which are part of this option, were generated
		--  with the selects)
	local thklyr: STRING; thksrc: CATS_EXPRESSION
	do  if option_is_present(oid.thkgate) then
		thklyr := source_layer_name (d, lvl.thkgate)
		thksrc := #not# (active_src #sized# nominal_bias (d, lvl.active));	-- for all
		if d.is_present (thklyr) then
			thksrc := (thksrc #or# leaf (thklyr)) #sized# 0.5 #sized# -0.5
		end;
		add_ctemp (thksrc, tmp_tgate_src, 0.05)	-- (temp used for c5t too)
		add_clevel_with_bias (leaf (tmp_tgate_src), lvl.thkgate, d)
	    end
	end;

	c5t_masks(d: DIES_XROW) is
		-- process additional c5t masks: plyblk, vtplo, vt2blk
	local plyblksrc, vtplosrc, vt2blksrc: CATS_EXPRESSION
	do  if option_is_present(oid.c5t) then
		vt2blksrc := leaf (tmp_tgate_src);	-- for all
		-- rest is just for c5t layout that includes plo, pi, and ni
		if have_plo_or_pi (d) then
			vt2blksrc := vt2blksrc #or# leaf (tmp_plo_or_pi);
			vtplosrc  := leaf (lyr.plo);
			plyblksrc := leaf (tmp_plo_or_pi)
		else	vtplosrc  := emptiness
			plyblksrc := emptiness
		end;
		if d.is_present (lyr.ni) then
			vt2blksrc := vt2blksrc #or# leaf (lyr.ni)
		end;
		-- add the statements
		add_clevel (vt2blksrc, lvl.vt2blk, d);
		add_clevel (plyblksrc, lvl.plyblk, d);
		add_clevel (vtplosrc,  lvl.vtplo,  d);
	    end
	end;

end -- AMI_C5
