class APPLICATION_COLORS_ROW

inherit
	APPLICATION_COLORS_RAW
		rename	red_value as direct_red_value,
			green_value as direct_green_value,
			blue_value as direct_blue_value
		end;

	RGB_COLOR

	DB_OPTIONS_ACCESS
		export
			{NONE} all
		end
	ROW_COUNT_CHECK_ACCESS
		export
			{NONE} all
		end


create
	make_template, make_null, make_detached, make_by_key

feature -- Collections

	colors_for_application (app: STRING conn: DB_CONNECTION range: ITEM_COUNT_RANGE): LIST [like Current] is
			-- All rows for application `app', ordered by
			-- color role
		require
			app_not_void: app /= Void
			connection_exists: conn /= Void
			connection_open: conn.is_open
			range_exists: range /= Void
		local
			b: DB_BIND_S
		do
			create b.make (app)
			Result := all_rows_filtered_ordered (conn, "application = :1", b, "color_role")
			check_row_count (Result, range)
		ensure
			result_exists: Result /= Void
		end

feature -- status

	has_color_name: BOOLEAN is
	do  Result := not is_null_color_name
	end;

	red_value: DOUBLE is
	do  if has_color_name then
		Result := named_color.red_value
	    else
		Result := direct_red_value
	    end
	end;

	green_value: DOUBLE is
	do  if has_color_name then
		Result := named_color.green_value
	    else
		Result := direct_green_value
	    end
	end;

	blue_value: DOUBLE is
	do  if has_color_name then
		Result := named_color.blue_value
	    else
		Result := direct_blue_value
	    end
	end;

	named_color: NAMED_COLORS_ROW is
		-- my named color
	require has_color_name: has_color_name
	do
		if cached_named_color = Void then
			create cached_named_color.make_by_key (connection, option.Old_row, color_name)
		end;
		Result := cached_named_color		
	end;

feature {NONE} -- cached stuff

	cached_named_color: NAMED_COLORS_ROW

end -- APPLICATION_COLORS_ROW
