
indexing
	
	description: "A run closing step operation that assigns die IDs and %
		%mask site number to a completed packing, populates %
		%dies and die locations and updates fab instances"
	author: "David Hollenberg"
	date: "February 27, 2007"

class ASSIGN_DIE_IDS_OP
inherit
	RC_PRIMITIVE_STEP_OP
		redefine
			is_currently_undoable, not_undoable_reason
		end
	RC_APPLICABLE_STEP
	NO_USER_INTERFACE
	NO_EXTERNAL_STATE
	PACKING_ACCESSOR
		export
			{NONE} all
		undefine
			default_create
		end

create
	make

feature -- Properties

	input_data_items: LIST [STRING]
			-- Data items required before step can be executed
		do
			Result := array_as_list (<<
				data_item.Completed_packing
				 >>)
		end

feature {NONE} -- Implementation

	is_currently_undoable: BOOLEAN
			-- Is `Current' currently in a state where it can be
			-- undone?
		local
			n: INTEGER
			row: IBM_MAP_DATA_ROW
		do
			create row.make_template
			n := row.map_data_for_run_die_locations (session.run_id, connection, expected_count.At_least_zero).count
			Result := n = 0
		end

	not_undoable_reason: STRING
			-- Reason step cannot currently be undone
		local
			n: INTEGER
			row: IBM_MAP_DATA_ROW
		do
			create row.make_template
			n := row.map_data_for_run_die_locations (session.run_id, connection, expected_count.At_least_zero).count
			Result := "there " + su.number (n, "is", "are") + " " + n.out + " " + su.number (n, "row", "rows") + " in table " + row.table_name.as_upper + " for die locations for this run.%NYou must delete these rows manually or set their MASK_SITE_NO to null to%Nundo this step."
		end

	do_step
			-- Execute of step
		do
			cache.uncache_all
			fetch_packing_selected_version
			packing_version.select_external_vendor_geometry
				-- Add external vendor geometry - not
				-- stored with packing version in DB
			initialize
			clear_non_project_die_ids
			assign_die_ids
			assign_mask_site_numbers
			populate_dies_and_locations
			populate_cut_patterns
			update_fab_instances
			
			-- Post changes to database so that we can
			-- update die IDs in all versions of
			-- packing_dies with a single SQL statement
			
			connection.post
			
			save_die_ids
			call_completion_actions
		end

	undo_step
			-- Undo execution of step
		do
			save_die_ids
			remove_data
			cache.uncache_all
			call_completion_actions
		end

	set_state
			-- Set `state'
		do
			state := Void
		end

	remove_data
			-- Remove data inserted when step was done
		local
			l_dies: LIST [DIES_MROW]
			l_die_locations: LIST [DIE_LOCATIONS_MROW]
			l_cut_patterns: LIST [RUN_CUT_PATTERNS_MROW]
			l_cut_locations: LIST [CUT_LOCATIONS_MROW]
			l_cut_die_sites: LIST [CUT_DIE_SITES_MROW]
			l_fab_instances: LIST [IC_FAB_INSTANCES_MROW]
			ifi: IC_FAB_INSTANCES_MROW
			lock_op: ROW_LOCK_OP
		do
			l_dies := cache.die_mrows
			l_die_locations := cache.die_location_mrows
			l_cut_patterns := cache.cut_pattern_mrows
			l_cut_locations := cache.cut_location_mrows
			l_cut_die_sites := cache.cut_die_site_mrows
			create ifi.make_template
			l_fab_instances := ifi.fab_instances_for_run (session.run_id, connection, expected_count.At_least_zero)
			create lock_op
			lock_op.lock_list_nowait (l_fab_instances)
			lock_op.lock_list_nowait (l_dies)
			lock_op.lock_list_nowait (l_die_locations)
			lock_op.lock_list_nowait (l_cut_patterns)
			lock_op.lock_list_nowait (l_cut_locations)
			lock_op.lock_list_nowait (l_cut_die_sites)
			l_fab_instances.do_all (agent {IC_FAB_INSTANCES_MROW}.clear_die_id)
			l_cut_die_sites.do_all (agent {CUT_DIE_SITES_MROW}.delete)
			l_cut_locations.do_all (agent {CUT_LOCATIONS_MROW}.delete)
			l_cut_patterns.do_all (agent {RUN_CUT_PATTERNS_MROW}.delete)
			l_die_locations.do_all (agent {DIE_LOCATIONS_MROW}.delete)
			l_dies.do_all (agent {DIES_MROW}.delete)
		end

	initialize
			-- Set attributes from current packing version
		do
			dies := packing_version.dies
			cut_patterns := packing_version.cut_patterns
		end

	clear_non_project_die_ids
			-- Clear the die ID for every die that is not
			-- a project, since we don't want to preserve any
			-- previously assigned die ID for non-projects
		do
			dies.do_if (agent {PACKING_DIES_MROW}.set_null_die_id, agent {PACKING_DIES_MROW}.is_non_project)
		end

	assign_die_ids
			-- Assign a die ID to every die that does not
			-- already have one assigned, in current packing
			-- version.  Fill in "holes".
		do
			fetch_die_id_first_characters
			fetch_all_die_types
			set_second_characters_for_scribes
			set_second_characters_for_non_scribes
			set_second_characters
			set_assigned_die_ids
			check_assigned_die_ids
			set_available_die_ids
			dies.do_all (agent assign_die_id)
		end

	set_second_characters
			-- Fill in table `second_characters_table'
		local
			type: DIE_TYPE_VALUES
			dt: STRING
		do
			create second_characters_table.make (11)
			from
				all_die_types.start
			until
				all_die_types.after
			loop
				dt := all_die_types.item
				if dt.is_equal (type.Scribe) then
					second_characters_table.put (second_characters_for_scribes, dt)
				else
					second_characters_table.put (second_characters_for_non_scribes, dt)
				end
				all_die_types.forth
			end
		end

	set_second_characters_for_non_scribes
			-- Set `second_characters_for_non_scribes'
		local
			list: like second_characters_for_non_scribes
		do
			create {ARRAYED_LIST [CHARACTER]} second_characters_for_non_scribes.make (40)
			list := second_characters_for_non_scribes
			list.extend ('A')
			list.extend ('B')
			list.extend ('C')
			list.extend ('D')
			list.extend ('E')
			list.extend ('F')
			list.extend ('G')
			list.extend ('H')
			list.extend ('J')
			list.extend ('K')
			list.extend ('L')
			list.extend ('M')
			list.extend ('N')
			list.extend ('P')
			list.extend ('Q')
			list.extend ('R')
			list.extend ('S')
			list.extend ('T')
			list.extend ('U')
			list.extend ('V')
			list.extend ('W')
			list.extend ('X')
			list.extend ('Y')
			list.extend ('Z')
		end

	set_second_characters_for_scribes
			-- Set `second_characters_for_scribes'
		local
			list: like second_characters_for_scribes
		do
			create {ARRAYED_LIST [CHARACTER]} second_characters_for_scribes.make (40)
			list := second_characters_for_scribes
			list.extend ('A')
			list.extend ('B')
			list.extend ('C')
			list.extend ('D')
			list.extend ('E')
			list.extend ('F')
			list.extend ('G')
			list.extend ('H')
			list.extend ('I')
			list.extend ('J')
			list.extend ('K')
			list.extend ('L')
			list.extend ('M')
			list.extend ('N')
			list.extend ('O')
			list.extend ('P')
			list.extend ('Q')
			list.extend ('R')
			list.extend ('S')
			list.extend ('T')
			list.extend ('U')
			list.extend ('V')
			list.extend ('W')
			list.extend ('X')
			list.extend ('Y')
			list.extend ('Z')
			list.extend ('0')
			list.extend ('1')
			list.extend ('2')
			list.extend ('3')
			list.extend ('4')
			list.extend ('5')
			list.extend ('6')
			list.extend ('7')
			list.extend ('8')
			list.extend ('9')
		end

	set_assigned_die_ids
			-- Fetch all design numbers there were ever
			-- assigned a die ID on any packing for this
			-- run, along with the assigned die ID.
			-- Use this list to build tables `assigned_die_ids'
			-- and `design_die_ids'
		local
			q: SQL_QUERY
			text, die_id: STRING
			pd_row: PACKING_DIES_ROW
			design_no: INTEGER
			design_no_list: ARRAYED_LIST [INTEGER]
			die_id_list: ARRAYED_LIST [STRING]
		do
			create assigned_die_ids.make (dies.count)
			create design_die_ids.make (dies.count)

			create pd_row.make_template
			text := "[
				select distinct pp.design_no, pd.die_id
				from packing_dies pd, packing_projects pp, packings p
				where p.packing_id = pd.packing_id
				  and pd.packing_id = pp.packing_id
				  and pd.project_no = pp.project_no
				  and pd.project_version_no = pp.version_no
				  and (p.packing_id = :1 or p.intended_run = :2)
				  and (p.wafer_techn = :3
				       or p.wafer_techn in (
					-- Superset wafer techns
					select 	wt2.wafer_techn
					from	wafer_techns_ wt1,
						wafer_techns_ wt2,
						subset_wafer_techns wts
					where	wts.wafer_techn_family = wt1.wafer_techn_family
					  and	wts.wafer_techn_id = wt1.wafer_techn_id
					  and	wts.wafer_techn_family = wt2.wafer_techn_family
					  and	wts.superset_wafer_techn_id = wt2.wafer_techn_id
					  and	wt1.wafer_techn = :3
					union
					-- Subset wafer techns
					select 	wt2.wafer_techn
					from	wafer_techns_ wt1,
						wafer_techns_ wt2,
						subset_wafer_techns wts
					where	wts.wafer_techn_family = wt1.wafer_techn_family
					  and	wts.superset_wafer_techn_id = wt1.wafer_techn_id
					  and	wts.wafer_techn_family = wt2.wafer_techn_family
					  and	wts.wafer_techn_id = wt2.wafer_techn_id
					  and	wt1.wafer_techn = :3
					))
				  and pd.die_id is not null
				order by pd.die_id, pp.design_no
				]"

			q := connection.sql_query (text)
			q.set_numeric_bind_capacity (3)
			q.bind_num_to_string (1, packing_id, False)
			q.bind_num_to_string (2, session.run_id, False)
			q.bind_num_to_string (3, cache.wafer_techn, False)
			q.set_select_count (2)
			q.select_integer (1)
			q.select_string (2, pd_row.maxlen_die_id)
			q.execute
			from
				q.fetch
			until
				q.end_of_data
			loop
				design_no := q.integer_value (1)
				die_id := q.string_value (2)
				assigned_die_ids.search (die_id)
				if assigned_die_ids.found then
					assigned_die_ids.found_item.extend (design_no)
				else
					create design_no_list.make (1)
					design_no_list.extend (design_no)
					assigned_die_ids.put (design_no_list, die_id)
				end
				design_die_ids.search (design_no)
				if design_die_ids.found then
					design_die_ids.found_item.extend (die_id)
				else
					create die_id_list.make (1)
					die_id_list.extend (die_id)
					design_die_ids.put (die_id_list, design_no)
				end
				q.fetch
			end
			q.release
		end

	check_assigned_die_ids
			-- Check that every design number has at most one
			-- die ID assigned to it, considering design numbers
			-- from every version of every packing associated with run.
			-- Check that every die ID has at most one
			-- design number assigned to it, considering design numbers
			-- from every version of every packing associated with run.
		local
			die_id, design_no_errors, die_id_errors, err_msg: STRING
			design_no: INTEGER
			design_no_list: LIST [INTEGER]
			die_id_list: LIST [STRING]
			had_error: BOOLEAN
			first: BOOLEAN
		do
			had_error := False
			create die_id_errors.make (0)
			from
				assigned_die_ids.start
			until
				assigned_die_ids.off
			loop
				die_id := assigned_die_ids.key_for_iteration
				design_no_list := assigned_die_ids.item_for_iteration
				if design_no_list.count > 1 then
					had_error := True
					die_id_errors.append ("   " + die_id + "   ")
					from
						first := True
						design_no_list.start
					until
						design_no_list.after
					loop
						if not first then
							die_id_errors.append (", ")
						end
						die_id_errors.append_integer (design_no_list.item)
						first := False
						design_no_list.forth
					end
					die_id_errors.extend ('%N')
				end
				assigned_die_ids.forth
			end
			if had_error then
				die_id_errors.prepend ("Die IDs with more than one associated design number:%N%N")
			end
			
			had_error := False
			create design_no_errors.make (0)
			from
				design_die_ids.start
			until
				design_die_ids.off
			loop
				design_no := design_die_ids.key_for_iteration
				die_id_list := design_die_ids.item_for_iteration
				if die_id_list.count > 1 then
					had_error := True
					design_no_errors.append ("   " + design_no.out + "   " + su.joined_by (die_id_list, ", ") + "%N")
				end
				design_die_ids.forth
			end
			if had_error then
				design_no_errors.prepend ("Design numbers with more than one assigned die ID:%N%N")
			end
			
			if not die_id_errors.is_empty or not design_no_errors.is_empty then
				create err_msg.make (die_id_errors.count + 2 + design_no_errors.count)
				err_msg.append ("die ID assignment conflict%N%N")
				err_msg.append (die_id_errors)
				if not err_msg.is_empty and not design_no_errors.is_empty then
					err_msg.append ("%N")
				end
				err_msg.append (design_no_errors)
				handle_hard_error (err_msg)
			end
		end

	set_available_die_ids
			-- Initialize table with list of available
			-- (unassigned) die IDs for each die type
		local
			die_type: DIE_TYPE_VALUES
		do
			create available_die_ids.make (10)
			from
				all_die_types.start
			until
				all_die_types.after
			loop
				add_available_die_ids (all_die_types.item)
				all_die_types.forth
			end
			
			-- Make vendor geometry and vendor ghost share
			-- same die ID list

			available_die_ids.search (die_type.Vendor_geometry)
			if available_die_ids.found then
				available_die_ids.force (available_die_ids.found_item, die_type.Vendor_ghost)
			end
		end

	add_available_die_ids (dt: STRING)
			-- Add list of available die IDs for die type `dt'
		local
			list: LIST [STRING]
			first_chars, second_chars: LIST [CHARACTER]
			first_char, second_char: CHARACTER
			die_id: STRING
		do
			first_characters_table.search (dt)
			if first_characters_table.found then
				first_chars := first_characters_table.found_item
				second_chars := second_characters_table.item (dt)
				create {LINKED_LIST [STRING]} list.make
				from
					first_chars.start
				until
					first_chars.after
				loop
					first_char := first_chars.item
					from
						second_chars.start
					until
						second_chars.after
					loop
						second_char := second_chars.item
						create die_id.make (2)
						die_id.extend (first_char)
						die_id.extend (second_char)
						if is_available_die_id (die_id) then
							list.extend (die_id)
						end
						second_chars.forth
					end
					first_chars.forth
				end
			else
				create {ARRAYED_LIST [STRING]} list.make (0)
			end
			list.start
			available_die_ids.put (list, dt)
		end

	assign_die_id (die: PACKING_DIES_MROW)
			-- Assign a die ID to `die' if it does not already
			-- have one assigned
		local
			die_id: STRING
		do
			if die.is_null_die_id then
				if die.has_project then
					design_die_ids.search (die.design_no)
					if design_die_ids.found then
						die_id := design_die_ids.found_item.first
					else
						fetch_next_die_id (die.die_type)
						die_id := next_die_id
					end
				else
					fetch_next_die_id (die.die_type)
					die_id := next_die_id
				end
				die.set_die_id (die_id)
			end
		end

	assign_mask_site_numbers
			-- Assign a mask site number to every die
			-- location in current packing version.
			-- Ignore any previous assignment
		do
			last_mask_site_number := 0
			sorted_die_locations.do_all (agent assign_location_mask_site_number)
		end

	sorted_die_locations: LIST [PACKING_DIE_LOCATIONS_MROW]
			-- All locations for all dies in an order suitable
			-- for mask site number assignment
		local
			loc_sorter: LIST_QUICK_SORTER [PACKING_DIE_LOCATIONS_MROW]
			loc_comparator: FIELD_POSITION_COMPARATOR [PACKING_DIE_LOCATIONS_MROW]
		do
			create {ARRAYED_LIST [PACKING_DIE_LOCATIONS_MROW]} non_scribe_locations.make (50)
			create {ARRAYED_LIST [PACKING_DIE_LOCATIONS_MROW]} scribe_locations.make (50)
			create {ARRAYED_LIST [PACKING_DIE_LOCATIONS_MROW]} external_locations.make (50)
			dies.do_all (agent classify_die_locations)
			create loc_sorter
			create loc_comparator
			non_scribe_locations := loc_sorter.sorted (non_scribe_locations, loc_comparator)
			scribe_locations := loc_sorter.sorted (scribe_locations, loc_comparator)
			external_locations := loc_sorter.sorted (external_locations, loc_comparator)
			create {ARRAYED_LIST [PACKING_DIE_LOCATIONS_MROW]} Result.make (200)
			Result.append (non_scribe_locations)
			Result.append (scribe_locations)
			Result.append (external_locations)
		end

	classify_die_locations (die: PACKING_DIES_MROW)
			-- Append locations for `die' to appropriate list,
			-- depending on its properties.  Used to assign
			-- mask site numbers to non-scribes, then scribes,
			-- then external dies
		local
			locs: LIST [PACKING_DIE_LOCATIONS_MROW]
		do
			locs := die.die_locations
			if die.is_external_to_payload then
				external_locations.append (locs)
			elseif die.is_scribe then
				scribe_locations.append (locs)
			else
				non_scribe_locations.append (locs)
			end
		end

	append_die_locations (die: PACKING_DIES_MROW list: LIST [PACKING_DIE_LOCATIONS_MROW])
			-- Append locations for `die' to `list'
		do
			list.append (die.die_locations)
		end

	assign_die_mask_site_numbers (die: PACKING_DIES_MROW)
			-- Assign a mask site number to every die
			-- location for `die'.  Ignore any previous
			-- assignment
		do
			die.die_locations.do_all (agent assign_location_mask_site_number)
		end

	assign_location_mask_site_number (dl: PACKING_DIE_LOCATIONS_MROW)
			-- Assign a mask site number to die location `dl'
		do
			last_mask_site_number := last_mask_site_number + 1
			dl.set_mask_site_no (last_mask_site_number)
		end

	populate_dies_and_locations
			-- Populate tables DIES and DIE_LOCATIONS from
			-- the current version of packing for run
		do
			dies.do_all (agent populate_die)
		end

	populate_die (die: PACKING_DIES_MROW)
			-- Populate DIES and DIE_LOCATIONS for `die'
		local
			row: DIES_MROW
			status: JOB_STATUS_VALUES
			cs: STRING
		do
			create row.make_null (connection)
			row.set_run_id (cache.run_id)
			row.set_die_id (die.die_id)
			row.set_die_type (die.die_type)
			row.set_die_size_printed_x (die.die_size_printed_x)
			row.set_die_size_printed_y (die.die_size_printed_y)
			row.set_die_wafer_rotation (die.die_wafer_rotation)
			if die.is_test_vehicle then
				row.set_test_vehicle_design_no (die.project.design_no)
			end
			row.set_geometry_id (die.geometry_id)
			row.set_scribe_orientation (die.scribe_orientation)
			row.set_filler_name (die.filler_name)
			if die.is_vendor_ghost then
				cs := status.Na
			else
				cs := status.Not_created
			end
			row.set_job_status (cs)
			die.die_locations.do_all (agent populate_die_location (die, ?))
		end

	populate_die_location (die: PACKING_DIES_MROW dl: PACKING_DIE_LOCATIONS_MROW)
			-- Populate DIE_LOCATIONS from die location `dl' for
			-- die `die'
		local
			row: DIE_LOCATIONS_MROW
			status: JOB_STATUS_VALUES
		do
			create row.make_null (connection)
			row.set_run_id (cache.run_id)
			row.set_mask_site_no (dl.mask_site_no)
			row.set_die_id (die.die_id)
			row.set_field_name (dl.field_name)
			row.set_location_x (dl.location_x)
			row.set_location_y (dl.location_y)
			row.set_job_status (status.Na)
		end

	populate_cut_patterns
			-- Populate tables RUN_CUT_PATTERNS,
			-- CUT_LOCATIONS and CUT_DIE_SITES from the
			-- current version of packing for run
		do
			cut_patterns.do_all (agent populate_cut_pattern)
		end

	populate_cut_pattern (pat: PACKING_CUT_PATTERNS_ROW)
			-- Populate tables RUN_CUT_PATTERNS,
			-- CUT_LOCATIONS and CUT_DIE_SITES for `pat'
		local
			row: RUN_CUT_PATTERNS_MROW
		do
			create row.make_null (connection)
			row.set_run_id (cache.run_id)
			row.set_cut_pattern_no (pat.cut_pattern_no)
			row.set_field_name (pat.field_name)
			row.set_null_packaging_lot_no
			row.set_quantity_wafers_needed (pat.quantity_wafers_needed)
			row.set_quantity_wafers_needed_wo_dfc (pat.quantity_wafers_needed)
			pat.cut_locations.do_all (agent populate_cut_location)
			pat.cut_die_sites.do_all (agent populate_cut_die_site)
		end

	populate_cut_location (loc: PACKING_CUT_LOCATIONS_ROW)
			-- Populate table CUT_LOCATIONS from `loc'
		local
			row: CUT_LOCATIONS_MROW
		do
			create row.make_null (connection)
			row.set_run_id (cache.run_id)
			row.set_cut_pattern_no (loc.cut_pattern_no)
			row.set_cut_no (loc.cut_no)
			row.set_orientation (loc.orientation)
			row.set_location (loc.location)
		end

	populate_cut_die_site (site: PACKING_CUT_DIE_SITES_ROW)
			-- Populate table CUT_DIE_SITES from `site'
		local
			row: CUT_DIE_SITES_MROW
		do
			create row.make_null (connection)
			row.set_run_id (cache.run_id)
			row.set_cut_pattern_no (site.cut_pattern_no)
			row.set_mask_site_no (site.die_location.mask_site_no)
			row.set_location_x (site.location_x)
			row.set_location_y (site.location_y)
			row.set_die_size_cut_x (site.die_size_cut_x)
			row.set_die_size_cut_y (site.die_size_cut_y)
		end

	update_fab_instances
			-- Update fab instances with the die IDs in the
			-- current version of packing for run.  Also set
			-- project on die
		do
			dies.do_if (agent update_fab_instance, agent {PACKING_DIES_MROW}.is_cust_die)
		end

	update_fab_instance (die: PACKING_DIES_MROW)
			-- Update fab instance for `die' with die ID and
			-- project on die
		local
			ifi: IC_FAB_INSTANCES_MROW
		do
			ifi := die.project.fab_instance
			ifi.set_die_id (die.die_id)
			ifi.set_project_on_die (1)
		end

	save_die_ids
			-- Save the die IDs for customer project and
			-- test vehicle dies in all versions of all
			-- packings intedned for run into the
			-- database.
		local
			change: SQL_CHANGE
		do
			change := connection.sql_change (cust_die_update_statement)
			change.bind_num_to_string (1, packing_id, False)
			change.bind_num_to_string (2, session.run_id, False)
			change.execute
			change.release

			change := connection.sql_change (test_vehicle_update_statement)
			change.bind_num_to_string (1, packing_id, False)
			change.bind_num_to_string (2, session.run_id, False)
			change.execute
			change.release
		end

	fetch_all_die_types
			-- Fetch all die types and put in `all_die_types'
		do
			all_die_types := connection.selected_strings ("die_type", "die_types", Void, Void, False, Void)
		end
	
	fetch_die_id_first_characters
			-- Fetch rows that indicate which first characters
			-- can be used for die IDs for which die type
		local
			row: DIE_ID_FIRST_LETTERS_ROW
			rows: LIST [DIE_ID_FIRST_LETTERS_ROW]
			dt: STRING
			c: CHARACTER
			new_list: ARRAYED_LIST [CHARACTER]
		do
			create row.make_template
			rows := row.rows_for_litho_type (cache.litho_type, connection, expected_count.At_least_one)
			create first_characters_table.make (rows.count)
			from
				rows.start
			until
				rows.after
			loop
				row := rows.item
				dt := row.die_type
				c := row.die_id_first_letter
				first_characters_table.search (dt)
				if first_characters_table.found then
					first_characters_table.found_item.extend (c)
				else
					create new_list.make (10)
					new_list.extend (c)
					first_characters_table.put (new_list, dt)
				end
				rows.forth
			end
		end

	fetch_next_die_id (dt: STRING)
			-- Put next available die ID for die type `dt'
			-- in `next_die_id'
		local
			list: LIST [STRING]
		do
			available_die_ids.search (dt)
			if available_die_ids.found then
				list := available_die_ids.found_item
				if not list.is_empty then
					next_die_id := list.first
					list.remove
				else
					handle_hard_error ("no more die IDs available for die type " + dt)
				end
			else
				handle_hard_error ("no die IDs available for die type " + dt)
			end
		end

	is_available_die_id (id: STRING): BOOLEAN
			-- Is `id' an available (unassigned) die ID?
		do
			assigned_die_ids.search (id)
			Result := not assigned_die_ids.found
		end

	cust_die_update_statement: STRING = "[
		update packing_dies pd set die_id =
			(select die_id 
			 from   ic_fab_instances
			 where  run_id = :2
			   and  m_fab_no in
				(select m_fab_no from packing_projects pp
				  where	pp.packing_id = pd.packing_id
				    and	pp.project_no = pd.project_no))
		where (packing_id = :1 or packing_id in 
			(select packing_id from packings where intended_run = :2))
		  and exists
			(select die_id 
			 from   ic_fab_instances
			 where  run_id = :2
			   and  m_fab_no in
				(select m_fab_no from packing_projects pp
				  where	pp.packing_id = pd.packing_id
				    and	pp.project_no = pd.project_no))
		  and (die_id is null or
		       die_id <> 
			(select die_id 
			 from   ic_fab_instances
			 where  run_id = :2
			   and  m_fab_no in
				(select m_fab_no from packing_projects pp
				  where	pp.packing_id = pd.packing_id
				    and	pp.project_no = pd.project_no)))
		]"

	test_vehicle_update_statement: STRING = "[
		update packing_dies pd set die_id =
			(select die_id 
			 from   dies
			 where  run_id = :2
			   and  design_no in
				(select test_vehicle_design_no from packing_projects pp
				  where	pp.packing_id = pd.packing_id
				    and	pp.project_no = pd.project_no))
		where (packing_id = :1 or packing_id in 
			(select packing_id from packings where intended_run = :2))
		  and exists
			(select die_id 
			 from   dies
			 where  run_id = :2
			   and  design_no in
				(select test_vehicle_design_no from packing_projects pp
				  where	pp.packing_id = pd.packing_id
				    and	pp.project_no = pd.project_no))
		  and (die_id is null or
		       die_id <> 
			(select die_id 
			 from   dies
			 where  run_id = :2
			   and  design_no in
				(select test_vehicle_design_no from packing_projects pp
				  where	pp.packing_id = pd.packing_id
				    and	pp.project_no = pd.project_no)))
		]"

	dies: LIST [PACKING_DIES_MROW]
			-- List of dies in current packing version for run

	cut_patterns: LIST [PACKING_CUT_PATTERNS_ROW]
			-- List of cut patterns in current packing version 
			-- for run

	design_die_ids: HASH_TABLE [LIST [STRING], INTEGER]
			-- Lists of previously assigned die IDS, indexed by
			-- design number to which they were assigned

	assigned_die_ids: HASH_TABLE [LIST [INTEGER], STRING]
			-- Lists of design numbers that share the
			-- same die ID, indexed by die ID

	available_die_ids: HASH_TABLE [LIST [STRING], STRING]
			-- Lists of available (unassigned) die IDs, indexed
			-- by die type

	next_die_id: STRING
			-- Next available die ID for given die type

	all_die_types: LIST [STRING]
			-- List of all possible die types
	
	first_characters_table: HASH_TABLE [ARRAYED_LIST [CHARACTER], STRING]
			-- Ordered lists of first characters for die ID,
			-- indexed by die type

	second_characters_table: HASH_TABLE [LIST [CHARACTER], STRING]
			-- Ordered lists of second characters for die ID,
			-- indexed by die type

	second_characters_for_scribes: LIST [CHARACTER]
			-- List of valid second characters for die ID
			-- for scribes

	second_characters_for_non_scribes: LIST [CHARACTER]
			-- List of valid second characters for die ID
			-- for non-scribes

	last_mask_site_number: INTEGER
			-- Last mask site number used so far

	non_scribe_locations: LIST [PACKING_DIE_LOCATIONS_MROW]
			-- Die locations for internal non-scribes

	scribe_locations: LIST [PACKING_DIE_LOCATIONS_MROW]
			-- Die locations for internal scribes

	external_locations: LIST [PACKING_DIE_LOCATIONS_MROW]
			-- Die locations for external dies

end
