
indexing
	
	description: "A run closing step operation does initial assignment %
		%of packaging line items to packaging lots"
	author: "David Hollenberg"
	date: "June 3, 2005"

class ASSIGN_PACKAGING_LINE_ITEMS_OP
inherit
	RC_PRIMITIVE_STEP_OP
	RC_APPLICABLE_STEP
	NO_USER_INTERFACE
	NO_EXTERNAL_STATE

create
	make

feature -- Properties

	input_data_items: LIST [STRING]
			-- Data items required before step can be executed
		do
			Result := array_as_list (<<
				data_item.Assigned_wafer_processes,
				data_item.Initial_packaging_lots,
				data_item.Fab_run_order
				>>)
		end

feature {NONE} -- Implementation

	do_step
		local
			row: CUST_WOS_MROW
			wos: LIST [CUST_WOS_MROW]
		do
			create {ARRAYED_LIST [STRING]} no_lots_list.make (20)
			create {ARRAYED_LIST [STRING]} multiple_lots_list.make (20)
			create row.make_template
			wos := row.work_orders_for_production (session.run_id, connection, expected_count.At_least_zero)
			wos.do_all (agent set_order_packaging_lots)
			report_problems
			call_completion_actions
		end

	set_order_packaging_lots (wo: CUST_WOS_MROW)
			-- Assign each packaging line item on `wo' to
			-- the appropriate packaging lot (best guess)
			-- if not already assigned
		require
			wo_not_void: wo /= Void
		do
			wo.line_items.do_if (agent set_line_item_packaging_lot, agent {CUST_WO_LINE_ITEMS_MROW}.needs_packaging_lot)
		end

	set_line_item_packaging_lot (woli: CUST_WO_LINE_ITEMS_MROW)
			-- Assign packaging line item `woli' to
			-- the appropriate packaging lot (best guess)
		require
			woli_not_void: woli /= Void
			needs_packaging_lot: woli.needs_packaging_lot
		local
			ord_type, proc: STRING
			lot: RUN_PACKAGING_LOTS_ROW
			lots: LIST [RUN_PACKAGING_LOTS_ROW]
			lock_op: ROW_LOCK_OP
		do
			ord_type := woli.best_guess_packaging_order_type
			proc := woli.the_wo.fab_instance.ordered_wafer_process
			create lot.make_template
			lots := lot.lots_for_order_type (session.run_id, proc, ord_type, connection, expected_count.At_least_zero)
			if not lots.is_empty then
				if lots.count > 1 then
					multiple_lots_list.extend (woli.packaging_description)
				end
				lot := lots.first
				create lock_op
				lock_op.lock_nowait (woli)
				woli.set_packaging_lot (lot.run_id, lot.packaging_lot_no)
			else
				no_lots_list.extend (woli.packaging_description)
			end
		end

	undo_step
			-- Undo execution of step
		do
			-- Leave packaging line items assigned to same
			-- lots.  The step that creates initial packaging
			-- lots will remove the packaging line items from
			-- their packaging lots, if user goes back that far.
			call_completion_actions
		end

	report_problems
			-- Report potential problems with line items
			-- that need to be assigned to a packaging lot,
			-- but for which no packaging lots (or multiple lots)
			-- were found
		do
			report_problem (no_lots_list, "for which no appropriate packaging lot was found")
			report_problem (multiple_lots_list, "with more than one appropriate packaging lot found")
		end

	report_problem (list: LIST [STRING] descr: STRING)
			-- Report potential problem with line items in
			-- `list', with problem description `descr'
		local
			msg: STRING
			n: INTEGER
			comp: STRING_COMPARATOR
			sorter: LIST_BUBBLE_SORTER [STRING]
			sorted_list: LIST [STRING]
		do
			if not list.is_empty then
				n := list.count
				create msg.make (50 * n)
				msg.append (n.out + " " + su.number (n, "line item", "line items") + " " + descr + "%N%N")
				create comp
				create sorter
				sorted_list := sorter.sorted (list, comp)
				msg.append (su.joined_by (sorted_list, "%N"))
				msg.extend ('%N')
				warn_with_notify (msg)
			end
		end

	set_state
			-- Set `state'
		do
			state := Void
		end

	no_lots_list: LIST [STRING]
			-- Descriptions of line items that need to be
			-- assigned to a packaging lot, but for which
			-- no appropriate lot can be found

	multiple_lots_list: LIST [STRING]
			-- Descriptions of line items that need to be
			-- assigned to a packaging lot, but for where
			-- there are multiple possible lots

end
