class BARCODE_GENERATOR
	-- generator of LAYOUT that represents a readable barcode

inherit
	LAYOUT_SUPPORT

	DB_ACCESS

	DB_QUERY_OPTIONS

	STRING_UTILITIES_ACCESS

	EXCEPTIONS

create
	make

feature	-- attributes

	bar_code_class: STRING is
	do  Result := bc_class.bar_code_class
	end;

	pixel_height: DOUBLE
		-- pixel_width is 1.0; this is the height
		-- (and therefore the aspect ratio too)
		-- scale resulting layout to fit your situation

	with_frame: BOOLEAN
		-- put a 0 width box around the bar code layout to
		-- establish its bounding box
		-- (without the frame, the true width is not obvious)

feature	-- setting attributes

	set_pixel_height(ht: DOUBLE) is
	require ht > 1;
	do  pixel_height := ht
	end;

	set_with_frame(wf: BOOLEAN) is
	do  with_frame := wf
	ensure with_frame = wf
	end;

feature	-- the generator

	barcode(bc_text: STRING): LAYOUT is
	require have_ht: pixel_height > 0;
		have_text: bc_text /= Void
	local char, bc_bits: STRING; bcch: BAR_CODE_CHARACTERS_ROW;
		j: INTEGER; bc_bars: LAYOUT_COLLECTION;
		fram: LAYOUT_PATH
	do  -- build bc_bits as a string of 0s and 1s
	    create bc_bits.make(bc_class.bar_code_bits * (bc_text.count+2));
	    create char.make(1);
	    bcch := bar_codes.item(bc_values.begin_sync)
	    if bcch /= Void then bc_bits.append(bcch.bar_code) end;
	    from  j := 1
	    until j > bc_text.count
	    loop  -- turn char into a string that is the j-th char
		  char.wipe_out; char.extend(bc_text @ j);
		  bcch := bar_codes.item(char);
		  if bcch /= Void
		   then bc_bits.append(bcch.bar_code)
		   else raise("No char in " + bar_code_class + 
				" for %'" + char + "%'")
		  end;
		  j := j + 1
	    end;
	    bcch := bar_codes.item(bc_values.end_sync)
	    if bcch /= Void then bc_bits.append(bcch.bar_code) end;
	    -- build a layout that embodies bc_bits
	    create bc_bars.make_unlayered(bc_bits.count)
	    from  j := 1;
	    until j > bc_bits.count
	    loop  if (bc_bits @ j) = '1'
		   then bc_bars.extend(box(j-1, 0, j, pixel_height))
		  end;
		  j := j + 1
	    end;
	    -- frame it if desired
	    if with_frame
	     then create fram.make_frame(box(0, 0, j-1,pixel_height),
					 0)
		  bc_bars.extend(fram)
	    end;
	    -- give it up
	    Result := bc_bars
	end;

feature {NONE}

	make(bc_cls: STRING) is
		-- set up to create bar code layout using the bc_cls
		--  bar_code_class.
		-- if bc_cls is empty (or Void), a default
		--  bar_code_class is used.
	local bccl: STRING; bcch: BAR_CODE_CHARACTERS_ROW;
		bcchs: LIST[BAR_CODE_CHARACTERS_ROW];
		b: DB_BIND_S;
	do  -- use default name if one is not provided
	    if su.is_empty(bc_cls)
	     then bccl := bc_values.default_class
	     else bccl := bc_cls
	    end;
	    -- fetch the class and set default attribute values
	    create bc_class.make_by_key(db, old_row, bccl);
	    pixel_height := bc_values.default_height;
	    with_frame := true;
	    -- now fetch all the characters and insert into bar_codes
	    create bcch.make_template;
	    create b.make(bar_code_class);
	    bcchs := bcch.all_rows_filtered
				(db, "bar_code_class = :1", b)
	    create bar_codes.make(2*bcchs.count);
	    from  bcchs.start
	    until bcchs.after
	    loop  bcch := bcchs.item
		  bar_codes.put(bcch, bcch.bar_code_character)
		  bcchs.forth
	    end
	end

	bc_values: BARCODE_VALUES

	bc_class: BAR_CODE_CLASSES_ROW

	bar_codes: HASH_TABLE[BAR_CODE_CHARACTERS_ROW, STRING]

end -- BARCODE_GENERATOR
