class BASIC_LIBRARY_SUPPORT
		-- common stuff for library classes

inherit
	GEOMETRY_SUPPORT

	DB_ACCESS
		export
			{NONE} all
		end

	DB_UTILITIES
		export
			{NONE} all
		end

feature -- common constants

	one_micron: DOUBLE is 0.000001;		-- in meters

	one_nanometer: DOUBLE is 0.000000001;	-- in meters

feature	-- instant structure

	structure(nam: STRING; body: LAYOUT): LIBRARY_STRUCTURE is
	do  create Result.make(nam);
	    Result.set_body(body);
	    Result.set_date_created(now)
	end;

feature {NONE} -- details

	now: DB_DATE is
	do  if db.is_open
	     then Result := db.transaction_date
	     else Result := db_date_from_Unix_time(
				unix_os.current_time_in_seconds)
	    end;
	end;

	make_note(l: LIST[STRING]; nam: STRING) is
		-- add nam to the list if it's not already there
	require l.object_comparison;
	do  if not l.has(nam)
	     then l.extend(nam)
	    end;
	ensure l.has(nam)
	end;

	make_bunch_of_notes(l: LIST[STRING]; src: LIST[STRING]) is
		-- add each src string to l if not already there
	do  if not l.object_comparison then l.compare_objects end;
	    from  src.start
	    until src.after
	    loop  make_note(l, src.item)
		  src.forth
	    end;
	end;

feature -- for gds structures and structure references

	valid_non_alpha_characters: STRING is "1234567890_?$";
	-- the characters, other than letters, legal in gds structure names

	valid_structure_name_character(c: CHARACTER): BOOLEAN is
		-- is c a valid structure name character?
	do  Result := c.is_alpha or valid_non_alpha_characters.has(c)
	end;

	former_max_structure_name_length: INTEGER is 32;
			-- the name length limit "has been removed"

	valid_structure_name(s: STRING): BOOLEAN is
		-- is s a valid structure name?
	require s /= Void
	local i, cnt: INTEGER;
	do  cnt := s.count;
	    from  i := 1;
		  Result := cnt > 0;
	    until i > cnt or not Result
	    loop  Result := valid_structure_name_character(s @ i);
		  i := i + 1
	    end
	end;

end -- BASIC_LIBRARY_SUPPORT
