
indexing

	author: "Generated"
	date: "Wed Nov 26 07:55:35 PST 2003"

class BATCH_JOBS_ROW
inherit
	BATCH_JOBS_RAW
	ROW_COUNT_CHECK_ACCESS
create
	make_template, make_null, make_by_key

feature

	gds_merge_jobs_for_run (run: STRING conn: DB_CONNECTION range: ITEM_COUNT_RANGE): LIST [like Current] is
			-- All batch jobs for creating merged GDS files
			-- (one per die or one per field)
			-- for run `run'
		require
			run_not_voiod: run /= Void
			connection_exists: conn /= Void
			connection_open: conn.is_open
			range_exists: range /= Void
		local
			b: DB_BIND_S
		do
			create b.make (run)
			Result := all_rows_filtered (conn, "batch_job_no in (select batch_job_no from run_gds_files where run_id = :1)", b)
			check_row_count (Result, range)
		ensure
			result_exists: Result /= Void
		end

	die_jobs_for_run (run: STRING conn: DB_CONNECTION range: ITEM_COUNT_RANGE): LIST [like Current] is
			-- All batch jobs for fracturing individual dies
			-- for run `run'
		require
			run_exists: run /= Void
			connection_exists: conn /= Void
			connection_open: conn.is_open
			range_exists: range /= Void
		local
			b: DB_BIND_S
		do
			create b.make (run)
			Result := all_rows_filtered (conn, "batch_job_no in (select batch_job_no from dies where run_id = :1)", b)
			check_row_count (Result, range)
		ensure
			result_exists: Result /= Void
		end

	level_jobs_for_run_and_type (run, a_job_type: STRING conn: DB_CONNECTION range: ITEM_COUNT_RANGE): LIST [like Current] is
			-- All batch jobs for level jobs with type `a_job_type' 
			-- for `run'
		require
			run_exists: run /= Void
			a_job_type_exists: a_job_type /= Void
			connection_exists: conn /= Void
			connection_open: conn.is_open
			range_exists: range /= Void
		local
			b: DB_BIND_SS
		do
			create b.make (run, a_job_type)
			Result := all_rows_filtered (conn, "batch_job_no in (select batch_job_no from run_wafer_level_jobs where run_id = :1 and cats_job_type = :2)", b)
			check_row_count (Result, range)
		ensure
			result_exists: Result /= Void
		end

	fill_selector_jobs_for_run (run: STRING conn: DB_CONNECTION range: ITEM_COUNT_RANGE): LIST [like Current] is
			-- All batch jobs for creating fill selector file
			-- for run `run'
		require
			run_exists: run /= Void
			connection_exists: conn /= Void
			connection_open: conn.is_open
			range_exists: range /= Void
		local
			b: DB_BIND_SS
			job_type: CATS_JOB_TYPE_VALUES
		do
			create b.make (run, job_type.Fill_selector)
			Result := all_rows_filtered (conn, "batch_job_no in (select batch_job_no from run_jobs where run_id = :1 and cats_job_type = :2)", b)
			check_row_count (Result, range)
		ensure
			result_exists: Result /= Void
		end

	is_null_elapsed_job_time: BOOLEAN is
			-- Is elapsed time of job null?
		local
			state: BATCH_JOB_STATE_VALUES
		do
			Result := is_null_date_state_changed or is_null_date_child_started or batch_job_state.is_equal (state.Waiting) or batch_job_state.is_equal (state.Assigned)
		end

	elapsed_job_time: INTEGER is
			-- Elapsed time of job, in seconds
		require
			time_not_null: not is_null_elapsed_job_time
		local
			state: BATCH_JOB_STATE_VALUES
			end_date: DB_DATE
		do
			if not is_known_job_interval then
				if batch_job_state.is_equal (state.Executing) then
					end_date := connection.system_date
				else
					end_date := date_state_changed
				end
				computed_job_interval := (connection.date_difference (end_date, date_child_started) * 86400).rounded
				is_known_job_interval := True
			end
			Result := computed_job_interval
		end

feature {NONE} -- Implementation

	computed_job_interval: INTEGER
			-- Last computed number of seconds job took
			-- (elapsed time, not CPU time)

	is_known_job_interval: BOOLEAN
			-- Is `computed_job_interval' meaningful?

end
