
indexing
	
	description: "Multi-column list row that has additional information %
		%about the entity it represents that uniquely identifies %
		%the entity and associates it with a key or pointer binding %
		%for an operation"
	author: "David Hollenberg"
	date: "March 16, 2007"

class BINDING_MULTI_COLUMN_LIST_ROW
inherit	
	MULTI_COLUMN_LIST_ROW
		rename
			make as mcl_make
		end
	HAS_BINDING_TYPE
		undefine
			copy, is_equal, default_create
		end
	HAS_BINDING
		undefine
			copy, is_equal, default_create
		end
	HAS_OPERATION
		undefine
			copy, is_equal, default_create
		end
	HAS_CATEGORY
		undefine
			copy, is_equal, default_create
		end

create
	make

create {BINDING_MULTI_COLUMN_LIST_ROW}
	make_filled

feature {NONE} -- Creation

	make (a_binding: MODIFIED_BINDING)
		require
			a_binding_not_void: a_binding /= Void
		do
			default_create
			operation_binding := a_binding
			enable_visible
		ensure
			is_visible: is_visible
		end

feature -- Properties

	operation_binding: MODIFIED_BINDING
			-- Operation and its binding

	binding: STRING
			-- Binding for operation
		do
			if cached_binding = Void then
				cached_binding := operation_binding.description
			end
			Result := cached_binding
		end

	has_binding: BOOLEAN
			-- Is there an associated binding for operation?
		do
			Result := operation_binding.is_bound
		end

	binding_type: STRING
			-- Type of binding for operation
		do
			Result := operation_binding.type
		end

	has_binding_type: BOOLEAN = True
			-- Is there an associated binding type for operation?

	category: STRING
			-- Category of operation
		do
			Result := operation_binding.category
		end

	has_category: BOOLEAN = True
			-- Is there an associated category?

	operation: STRING
			-- Category of operation
		do
			Result := operation_binding.operation
		end

	has_operation: BOOLEAN = True
			-- Is there an associated operation?

	matches_type_category_and_op (bt, cat, op: STRING): BOOLEAN
			-- Does `Current' have binding type `bt', category
			-- `cat' and does operation contain `op' as a substring
			-- (case independent)?  Empty arguments always match
			-- (no restriction)
		do
			Result := True
			if Result and not bt.is_empty then
				Result := binding_type.is_equal (bt)
			end
			if Result and not cat.is_empty then
				Result := category.is_equal (cat)
			end
			if Result and not op.is_empty then
				Result := operation.as_lower.has_substring (op)
			end
		end

	unique_identifier: STRING
			-- Value that uniquely identifies row
		do
			Result := operation_binding.number.out
		end

feature {NONE} -- Implementation

	new_filled_list (n: INTEGER): like Current
			-- New list with `n' elements.
		do
			create Result.make_filled (n)
		end
		
	cached_binding: STRING
			-- Value of `binding' cached to speed up
			-- sorting

end
