
indexing
	
	description: "Window to allow printing of a list of bonding diagrams"
	author: "David Hollenberg"
	date: "April 20, 2005"

class BOND_DIAGRAM_LIST_PRINT_WINDOW

inherit
	RC_BOND_PRINT_IFC
	RC_MAP_PRINT_WINDOW
		
create
	make

feature -- Properties

	has_file_name_items: BOOLEAN = False
			-- Is the data associated with each list item
			-- that can be printed a file name?  If not,
			-- the associated data is a data source that
			-- provides the data to be printed

feature {NONE} -- Implementation

	add_extra_select_menu_items (select_menu: EV_MENU)
		do
		end

	populate_maps
			-- Populate list of maps that can be printed
		do
			maps.wipe_out
			missing_map_count := 0
			packagings.do_all (agent add_packaging)
			set_missing_files_status
		end

	add_packaging (ipw: IC_PACKAGING_AND_WO)
			-- Add bonding diagram for `ipw' to `maps'
		require
			ipw_not_void: ipw /= Void
		local
			list_item: EV_LIST_ITEM
			src: BOND_DIAGRAM_SOURCE
		do
			create list_item.make_with_text (packaging_identifier (ipw))
			create src.make (ipw)
			list_item.set_data (src)
			maps.extend (list_item)
		end

	packaging_identifier (ipw: IC_PACKAGING_AND_WO) : STRING
			-- Text that uniquely identifies bonding diagram
			-- for `ipw'
		require
			ipw_not_void: ipw /= Void
		local
			icp: IC_PACKAGINGS_ROW
			ifi: IC_FAB_INSTANCES_ROW
		do
			icp := ipw.ic_packagings_row
			ifi := icp.fab_instance
			create Result.make (40)
			if ifi.run_id.is_empty and ifi.die_id.is_empty then
				Result.append_integer (ifi.design_no)
			else
				if not ipw.production_id.is_empty then
					Result.append (ipw.production_id)
				else
					Result.append (ifi.run_id)
				end
				if not ifi.die_id.is_empty then
					Result.extend ('-')
					Result.append (ifi.die_id)
				end
			end
			if not Result.is_empty then
				Result.append ("   ")
			end
			Result.append (icp.bonding_name)
		ensure
			result_not_void: Result /= Void
		end
	
	job_name: STRING
			-- Name to use for print job
		do
			Result :=  "bond_diagrams"
		end
	
end
