
indexing
	
	description: "Window to allow preparation of final bonding diagrams %
		%for a specific run and make them available to customers"
	author: "David Hollenberg"
	date: "May 31, 2005"

class BOND_DIAGRAM_PREP_WINDOW

inherit
	INTERNAL_FE_REQUEST_WINDOW
		redefine
			process_cancel
		end
		
create
	make

feature {NONE} -- Implementation

	fetch_cust_wos
			-- Fetch customer work orders for projects that
			-- may need final bonding diagrams prepared
		local
			cwo: CUST_WOS_ROW
			list: like cust_wos
		do
			create cwo.make_template
			list := cwo.work_orders_for_production (session.run_id, connection, expected_count.At_least_zero)
			cust_wos := list.twin
			cust_wos.wipe_out
			list.do_if (agent cust_wos.extend, agent {CUST_WOS_ROW}.needs_final_bonding_diagrams)
		end

	set_visible_columns
			-- Set visible columns in `projects'
		local
			cols: ARRAYED_LIST [MULTI_COLUMN_LIST_COLUMN [ORDER_MULTI_COLUMN_LIST_ROW]]
		do
			create cols.make (10)
			cols.extend (create {RUN_ID_COLUMN [ORDER_MULTI_COLUMN_LIST_ROW]})
			cols.extend (create {DIE_ID_COLUMN [ORDER_MULTI_COLUMN_LIST_ROW]})
			cols.extend (create {DESIGN_NO_COLUMN [ORDER_MULTI_COLUMN_LIST_ROW]})
			cols.extend (create {ACCOUNT_STRING_COLUMN [ORDER_MULTI_COLUMN_LIST_ROW]})
			cols.extend (create {PACKAGE_ID_COLUMN [ORDER_MULTI_COLUMN_LIST_ROW]})
			cols.extend (create {DATES_SENT_COLUMN [ORDER_MULTI_COLUMN_LIST_ROW]})
			projects.set_visible_columns (cols)
		end
			
	internal_request_type: STRING
			-- Type of internal requests that can be submitted
			-- and monitored
		local
			int_req: INTERNAL_REQUEST_TYPE_VALUES
		do
			Result := int_req.Send_bonding_diagram
		end

	request_description: STRING = "final bonding diagram generation"
			-- Description of internal requests that can be
			-- submitted and monitored, suitable for inclusion
			-- in messages to operator

	progress_window_title: STRING = "Final Bonding Diagram Generation"
			-- Title for request progress window (without
			-- run id)
	
	check_rows_valid
			-- Check that all selected rows are valid for
			-- this internal request
		do
		end

	confirm_before_submission (n: INTEGER)
			-- Tell operator we're about to submit `n' internal
			-- requests and get confirmation before proceeding
		do
			confirm ("Final bonding diagrams will be prepared for " + n.out + " " + su.number (n, "project", "projects") + " and made available%Nin the project document server.  Customers will be notified by e-mail.%N%NProceed")
		end

	warn_prev_processed_requests (n: INTEGER)
			-- Warn operator about `n' orders for which an
			-- internal request of this type has already been
			-- processed
		do
			warn ("Final bonding diagrams have already been prepared before for " + n.out + " " + su.number (n, "project", "projects") + ".%NNew diagrams will be generated for these projects and customers will be notified again.%N%NProceed anyway?")
		end

	process_cancel
			-- Process cancel request
		do
			if not session.connection.user_name.is_equal (session.cache.runs_row.closer) then
				-- Current user is not closer for this run,
				-- so treat Cancel like Finish to prevent
				-- leaving step unfinished
				process_finish
			else
				precursor
			end
		end

end
