
indexing
	
	description: "A bonding diagram for a specific packaging"
	author: "David Hollenberg and Barden E. Smith"
	date: "February 24, 2005"

class BONDING_DIAGRAM
inherit
	MOSIS_MAP
		rename
			microns_label as micrometers_label
		redefine
			append_mosis_logo,
			set_margins,
			set_sizes
		end

create
	make_with_date

feature -- Properties

	is_for_customer: BOOLEAN
			-- Include information for customer (when true)

	chip_only: BOOLEAN
			-- Create drawing of chip only

	packaging: IC_PACKAGINGS_ROW
			-- Information about packaging

	package: PACKAGES_ROW
			-- Information about package named in
			-- `packaging' or Void if no package

	fab_instance: IC_FAB_INSTANCES_ROW
			-- Information about fab instance or Void if
			-- no fab instance (can only happen when a refab
			-- is cancelled)

	cut_dies: LIST [CUT_DIE_SITES_ROW]
			-- Cut Die Sites for this design on just the
			-- packagings lots for `packaging' or all
			-- Cut Die Sites for this design or Void

	cust_wo_line_items: LIST [CUST_WO_LINE_ITEMS_ROW]
			-- Customer WO Line items for this `packaging'

	cust_ic_design: CUST_IC_DESIGNS_ROW
			-- Information about the Customer IC Design or Void
			-- (can only happen when the fab instance disapears)

	cust_acct: CUST_ACCTS_ROW
			-- Customer Account Row for this design or order

	pads: LIST [CUST_IC_DESIGN_PADS_ROW]
			-- List of pads in design for packaging

	wires: LIST [IC_PACKAGING_WIRES_ROW]
			-- List of wires for packaging
		do
			Result := packaging.my_wiring
		end

	package_connections: LIST [PACKAGE_CONNECTIONS_ROW]
			-- List of connections for package named in `package'
		do
			if package /= Void then
				Result := package.my_connections
			else
				create {ARRAYED_LIST [PACKAGE_CONNECTIONS_ROW]} Result.make (0)
			end
		end

	fab_id: STRING
			-- Fab ID to include on diagram
		do
			if fab_instance /= Void and not fab_instance.is_null_die_id then
				Result := fab_instance.run_id + "-" + fab_instance.die_id
			end
		end
	
	title: STRING
			-- Title to include on map
		local
			s: STRING
		do
			Result := "Bonding_diagram: "
			if fab_instance /= Void then
				if fab_instance.is_null_die_id then
					Result.append (fab_instance.design_no.out)
				else
					Result.append (fab_id)
				end
				s := packaging.bonding_name
				if s /= Void and then not s.is_empty then
					Result.append ("-")
					Result.append (s)
				end
			end
		end
	
feature -- Modification

	set_date (d: like date)
			-- Set `date' to `d'
		require
			date_not_void: d /= Void
		do
			date := d
		ensure
			value_set: date = d
		end

	set_is_for_customer (f: BOOLEAN)
			-- Set `is_for_customer' to `f'
		do
			is_for_customer := f
		ensure
			value_set: is_for_customer = f
		end

	set_chip_only (b: BOOLEAN)
			-- Set `chip_only' to `b'
		do
			chip_only := b
		ensure
			value_set: chip_only = b
		end

	set_packaging (r: like packaging)
			-- Set `packaging' to `r'
		require
			row_not_void: r /= Void
		do
			packaging := r
		ensure
			value_set: packaging = r
		end

	set_package (r: like package)
			-- Set `package' to `r'
		do
			package := r
		ensure
			value_set: package = r
		end

	set_fab_instance (r: like fab_instance)
			-- Set `fab_instance' to `r'
		do
			fab_instance := r
		ensure
			value_set: fab_instance = r
		end

	set_cut_dies (r: like cut_dies)
			-- Set `die' to `r'
		do
			cut_dies := r
		ensure
			value_set: cut_dies = r
		end

	set_cust_acct (r: like cust_acct)
			-- Set `cust_acct' to `r'
		do
			cust_acct := r
		ensure
			value_set: cust_acct = r
		end

	set_cust_wo_line_items (r: like cust_wo_line_items)
			-- Set `cust_wo_line_items' to `r'
		do
			cust_wo_line_items := r
		ensure
			value_set: cust_wo_line_items = r
		end

	set_cust_ic_design (r: like cust_ic_design)
			-- Set `cust_ic_design' to `r'
		do
			cust_ic_design := r
		ensure
			value_set: cust_ic_design = r
		end

	set_pads (list: like pads)
			-- Set `pad' to `list'
		require
			list_not_void: list /= Void
		do
			pads := list
		ensure
			value_set: pads = list
		end

feature -- Operations

	generate_whole_document
			-- Generate entire document representing diagram
			-- in `last_document'
		do
			generate_page
			last_document.add_page (last_page)
		end

feature {NONE} -- Implementation

	generate_drawing_common_part
			-- Generate part of drawing that is common
			-- to a group of maps
		do
			-- FIXME: implement if appropriate
			-- Probably not appropriate for bonding diagrams
		end

	generate_drawing_specific_part
			-- Generate part of drawing that is specific to a particular map.
			-- In this case the entire bonding diagram: IC, Package and wires
		local
			box: DRW_BASIC_RECTANGLE
			cid: CUST_IC_DESIGNS_ROW
			drw: DRW_GROUP
		do
			cid := cust_ic_design
			if cid /= Void and then cid.design_size_x * cid.design_size_y > 0 then
				drw := design_drawing
				create design_package_transform.make_translation_xy (-cid.center_x, -cid.center_y)
				if not packaging.is_null_design_package_rotation and
					packaging.design_package_rotation /= 0 then
						design_package_transform := design_package_transform.and_then
							(create {DRW_MATRIX}.make_rotation_degrees
								( - packaging.design_package_rotation))
				end
				drw.set_transform (design_package_transform)
				last_drawing.extend (drw)
				if package /= Void then
					last_drawing.extend (package_drawing)
					if not packaging.ignore_the_wires then
						last_drawing.extend (wires_drawing)
					end
				else
					-- use a scale based on the existing drawing figures
					if cut_dies /= Void then
						box := cut_die_size
					else
						create box.make (cid.design_size_x, cid.design_size_y, cid.center_x, cid.center_y)
					end
					create box.make (0.9 * box.width, 0.1 * box.height,
							0, 0.57 * box.height)
					last_drawing.extend (text_in_box ("No Package", box))
				end
			else
				create box.make (available_paper_width, available_drawing_height, 0, 0)
				if package /= Void then
					last_drawing.extend (package_drawing)
				elseif cid = Void then
					last_drawing.extend (text_in_box ("No Design!!", box))
				else
					last_drawing.extend (text_in_box ("No Design size information!", box))
				end
			end
		end

	generate_whole_title
			-- Generate entire title and put in `last_title'
		local
			box: DRW_RECTANGLE
			b_size: DRW_COORDINATE
			cwli: CUST_WO_LINE_ITEMS_ROW
			w: DOUBLE
			b, d: DRW_BASIC_RECTANGLE
			gg: DRW_GROUP_GENERATOR
			pv: DRW_GROUP
			s: STRING
			t: DRW_TEXT
			q: INTEGER
		do
			-- overall title box
			create box.make (available_paper_width, title_height, 0.5 * available_paper_width, 0.5 * title_height)
			last_title.extend (box)

			-- main title area
			w := available_paper_width - (0.72 * title_height)
			create box.make (0.5 * w, 0.18 * title_height, 0.25 * w, 0.91 * title_height)
			if fab_instance /= Void then
				box.set_stroke_width (thin_width)
				b := box.bounding_box
				shrink_box (b)
				s := "Qty: "
				if cust_wo_line_items /= Void and then not cust_wo_line_items.is_empty then
					from
						cust_wo_line_items.start
					until
						cust_wo_line_items.after
					loop
						cwli := cust_wo_line_items.item
						if cwli.part.packaged then
							q := q + cwli.number_of_parts
						end
						cust_wo_line_items.forth
					end
					s.append (q.out)
				elseif packaging.quantity_packaged_requested > 0 then
					s.append (packaging.quantity_packaged_requested.out + " req'd")
				else
					s.extend ('?')
				end
				last_title.extend (text_in_box (s, b))
			end
			last_title.extend (box)

			create box.make (0.5 * w, box.height, box.right + 0.25 * w, box.y)
			if fab_id /= Void then
				box.set_stroke_width (thin_width)
				b := box.bounding_box
				shrink_box (b)
				last_title.extend (text_in_box (fab_id, b))
			end
			last_title.extend (box)

			create box.make (0.72 * title_height, box.height,
						available_paper_width - 0.36 * title_height, box.y)
			if package /= Void then
				box.set_stroke_width (thin_width)
				b := box.bounding_box
				shrink_box (b)
				last_title.extend (text_in_box (package.package_id, b))
			end
			last_title.extend (box)

			-- package view area
			-- square area of size 0.72 * title_height (and is already in box.width)
			create box.make (box.width, box.width, box.x, box.bottom - 0.5 * box.width)
			last_title.extend (box)
			if package /= Void and then not package.outline_drawing.is_empty then
				box.set_stroke_width (thin_width)
				b := box.bounding_box
				shrink_box (b)
				create gg.make (package.outline_drawing)
				gg.set_font (font_name)
				gg.scan_path
				pv := gg.group
				pv.set_view_box (b)
				if pv.count > 0 then
					last_title.extend (pv)
				end
			end

			-- drawing date
			create box.make (box.width, 0.1 * title_height, box.x, 0.05 * title_height)
			box.set_stroke_width (thin_width)
			b := box.bounding_box
			shrink_wide_box (b)
			last_title.extend (text_in_box (date.to_date_and_time_string, b))
			last_title.extend (box)

			-- second line (two segments)
			w := box.left
			create box.make (0.5 * w, 0.18 * title_height, 0.25 * w, 0.73 * title_height)
			if fab_instance /= Void and not packaging.is_null_bonding_id then
				box.set_stroke_width (thin_width)
				b := box.bounding_box
				shrink_box (b)
				last_title.extend (text_in_box (packaging.bonding_id, b))
			else
			end
			last_title.extend (box)
			--
			create box.make (0.5 * w, 0.18 * title_height, 0.75 * w, 0.73 * title_height)
			if fab_instance /= Void then
				box.set_stroke_width (thin_width)
				b := box.bounding_box
				shrink_box (b)
				last_title.extend (text_in_box ("(" + fab_instance.design_no.out + ")", b))
			end
			last_title.extend (box)

			-- third line
			w := box.right
			create box.make (w, 0.08 * title_height, 0.5 * w, 0.60 * title_height)
			b := box.bounding_box
			shrink_wide_box (b)
			if packaging.customer_providing_diagram then
				s := "Customer Providing Diagram"
				if not packaging.is_null_date_diagram_approved then
					s := s + " (approved)"
				end
			elseif not (wires.is_empty or packaging.ignore_the_wires or is_for_customer) then
				s := wire_stats_string
			else	s := Void
			end
			if s /= Void then
				box.set_stroke_width (thin_width)
				t := text_in_box (s, b)
				t.set_mid_x (-1) -- left adjust in box
				last_title.extend (t)
			end
			last_title.extend (box)

			-- forth line (box.y has previous line center)
			w := box.width
			create box.make (w, box.height, 0.5 * w, box.y - box.height)
			b := box.bounding_box
			shrink_wide_box (b)
			s := pads_stats_string
			if s /= Void then
				box.set_stroke_width (thin_width)
				t := text_in_box (s, b)
				t.set_mid_x (-1) -- left adjust in box
				last_title.extend (t)
			end
			last_title.extend (box)

			-- fifth line (box.y has previous line center)
			w := box.width
			create box.make (w, box.height, 0.5 * w, box.y - box.height)
			last_title.extend (box)

			-- sixth line (box.y has previous line center)
			w := box.width - title_logo_width
			create box.make (w, box.height, title_logo_width + 0.5 * w, box.y - box.height)
			b := box.bounding_box
			shrink_wide_box (b)
			if cust_ic_design /= Void and then not cust_ic_design.is_null_design_name then
				box.set_stroke_width (thin_width)
				s := cust_ic_design.design_name
				if s.count > 50 then
					s := s.substring (1,47) + "..."
				end
				t := text_in_box ("Design_name: " + s, b)
				t.set_mid_x (-1) -- left adjust in box
				last_title.extend (t)
			end
			last_title.extend (box)

			-- seventh line (box.y has previous line center)
			create box.make (w, box.height, box.x, box.y - box.height)
			b := box.bounding_box
			shrink_wide_box (b)
			if cust_acct /= Void then
				box.set_stroke_width (thin_width)
				s := "Customer Account: "
				if is_for_customer then
					s := s + cust_acct.display_name
				else
					s := s + cust_acct.cust_acct_no.out
				end
				t := text_in_box (s, b)
				t.set_mid_x (-1) -- left adjust in box
				last_title.extend (t)
			end
			last_title.extend (box)

			-- eighth line (box.y has previous line center)
			create box.make (w, box.height, box.x, box.y - box.height)
			if cut_dies /= Void then
				box.set_stroke_width (thin_width)
				d := cut_die_size
				b := box.bounding_box
				shrink_wide_box (b)
				s := "Die Size:  " +
					die_size_text(d.width,
						      largest_die_size_x,
						      smallest_die_size_x) +
					" x " +
					die_size_text(d.height,
						      largest_die_size_y,
						      smallest_die_size_y) +
					Micrometers_label
				t := text_in_box (s, b)
				t.set_mid_x (-1)
				last_title.extend (t)
			end
			last_title.extend (box)

			-- ninth line (box.y has previous line center)
			create box.make (w, box.height, box.x, box.y - box.height)
			box.set_stroke_width (thin_width)
			b := box.bounding_box
			shrink_wide_box (b)
			s := "Die Rotation in Cavity: " + packaging.rotation_text
			t := text_in_box (s, b)
			t.set_mid_x (-1)
			last_title.extend (t)
			last_title.extend (box)

			-- tenth line (box.y has previous line center)
			create box.make (w, box.height, box.x, box.y - box.height)
			if package /= Void then
				create b_size.set (package.package_cavity_size_x_mil * 25.4,
						package.package_cavity_size_y_mil * 25.4)
				box.set_stroke_width (thin_width)
				b := box.bounding_box
				shrink_wide_box (b)
				t := text_in_box ("Cavity Size:  " +
						b_size.x.rounded.out +
						Micrometers_label + " x " +
						b_size.y.rounded.out +
						Micrometers_label, b)
				t.set_mid_x (-1)
				last_title.extend (t)
			end
			last_title.extend (box)

			-- and the logo
			append_mosis_logo
		end

	current_page_label: STRING
			-- Descriptive label for current page of map
			-- (alternative to page number) or Void if none
		do
		end

	set_stroke_and_fill_colors
			-- Set colors used for stroke and fill
		do
			-- FIXME: implement, if needed.
			-- Declare attributes of type DRW_COLOR and
			-- initialize them appropriately here.
			-- They can be used in routines that generate
			-- title and drawing.
		end

	approximate_drawing_width: DOUBLE
			-- Approximate width of drawing, in unspecified units.
			-- Used only to determine initial stroke width
			-- for various line thicknesses, so that stroke
			-- width will be a small fraction of drawing width
		do
			-- FIXME: implement
			Result := 20000
		end

	design_package_transform: DRW_MATRIX

	pad_loc_in_package(a_pad: CUST_IC_DESIGN_PADS_ROW): POINT
			-- a_pad center point in package drawing coordinates
	require have_pad: a_pad /= Void
	do  Result := design_package_transform.transform_xy (a_pad.center_x, a_pad.center_y)
	end

	design_drawing: DRW_GROUP
			-- The design bounding box and pads
		require
			have_design: cust_ic_design /= Void
		local
			dbb: DRW_RECTANGLE
			cid: CUST_IC_DESIGNS_ROW
			b: DRW_BASIC_RECTANGLE
			ts: STRING
			t: DRW_TEXT
		do
			cid := cust_ic_design
			create Result.make (pads.count + 1)
			create dbb.make (cid.design_size_x, cid.design_size_y, cid.center_x, cid.center_y)
			Result.extend (dbb)
			pads.do_all (Agent draw_pad (?, Result))
			if cut_dies /= Void then
				dbb := cut_die_size
				Result.extend (dbb)
			end
			create b.make (0.3 * dbb.width, 0.1 * dbb.height,
					dbb.right - 0.15 * dbb.width, dbb.top + 0.05 * dbb.height)
			if fab_instance.is_null_die_id then
				ts := "Top"
			else
				ts := fab_instance.die_id
			end
			t := text_in_box (ts, b)
			t.set_mid_x (1)
			t.set_mid_y (1)
			Result.extend (t)
		end

	largest_die_size_x,
	largest_die_size_y,
	smallest_die_size_x,
	smallest_die_size_y: DOUBLE
		-- 4 added values computed by last call to cut_die_size

	cut_die_size: DRW_RECTANGLE
		require
			cid_not_void: cust_ic_design /= Void
			cut_dies_not_void: cut_dies /= Void
		do
			if not cut_dies.is_empty then
				create Result.make (0, 0, cust_ic_design.center_x, cust_ic_design.center_y)
				largest_die_size_x := 0
				largest_die_size_y := 0
				smallest_die_size_x := 1000000000
				smallest_die_size_y := 1000000000
				cut_dies.do_all (Agent stretch_die (?, Result))
			end
		end

	stretch_die (die_row: CUT_DIE_SITES_ROW b: DRW_RECTANGLE)
		local
			bx: DRW_BASIC_RECTANGLE
			drx, dry: DOUBLE
		do
			drx := die_row.die_size_cut_x
			dry := die_row.die_size_cut_y
			create bx.make(drx, dry, b.x, b.y)	-- keep center
			b.merge (bx)
			-- and the 4 side calculations
			largest_die_size_x := largest_die_size_x.max (drx + die_row.die_oversizing_x)
			largest_die_size_y := largest_die_size_y.max (dry + die_row.die_oversizing_y)
			smallest_die_size_x := smallest_die_size_x.min (drx - die_row.die_undersizing_x)
			smallest_die_size_y := smallest_die_size_y.min (dry - die_row.die_undersizing_y)
		end

	die_size_text(nominal, largest, smallest: DOUBLE): STRING
		-- "nominal (+over / -under)"
	do  Result := nominal.rounded.out +
			" (+" + (largest-nominal).rounded.out +
			" / -" + (nominal-smallest).rounded.out + ")"
	end
			
	draw_pad (pad: CUST_IC_DESIGN_PADS_ROW drawing: DRW_GROUP)
			-- draw a pad in a drawing group
		local
			box: DRW_RECTANGLE
		do
			create box.make (pad.size_x, pad.size_y, pad.center_x, pad.center_y)
			if pad.bonded then
				box.set_stroke_width (thin_width)
			end
			drawing.extend (box)
		end

	wires_drawing: DRW_GROUP
		do
			create Result.make (wires.count)
			wires.do_all (Agent draw_wire (?, Result))
		end

	draw_wire (w: IC_PACKAGING_WIRES_ROW drawing: DRW_GROUP)
			-- draw a wire in package space from pad to bonding finger
		local
			ppkg, ppad: VIRTUAL_POINT
			wire: DRW_LINE
		do
			ppad := pad_loc_in_package(w.design_pad)
			ppkg := w.package_connection
			create wire.make (ppad.x, ppad.y, ppkg.x, ppkg.y)
			wire.set_stroke_width (thick_width)
			drawing.extend (wire)
		end

	package_drawing: DRW_GROUP
			-- The fingers and cavity drawn and collected into a drawing group
		require
			have_package: package /= Void
		local
			cavity: DRW_SOFT_RECTANGLE
			pkg: PACKAGES_ROW
			p: DRW_PATH
			t: DRW_TEXT
			f: PACKAGE_CONNECTIONS_ROW
			m: DRW_MATRIX
			b, b2: DRW_BASIC_RECTANGLE
			conn_ring: RECTANGLE
			conn_pitch: DOUBLE
		do
			create Result.make (package_connections.count + 2)
			pkg := package
			create cavity.make_by_size_center_corners (
					pkg.package_cavity_size_x_mil * 25.4, pkg.package_cavity_size_y_mil * 25.4,
					0, 0, pkg.cavity_corner_radius_mil * 25.4, pkg.cavity_corner_radius_mil * 25.4)
			cavity.set_stroke_width (thick_width)
			Result.add_figure (cavity)
			if package_connections.is_empty then
				if not chip_only then
					create b.make (0.8 * cavity.width, (0.6 * cavity.height).min (508.0),
							0, 0.5 * cavity.height + 254.0)
					Result.extend (text_in_box ("No Package Connections", b))
				end
			else
				-- conn_ring and conn_pitch are used for labelling box
				create conn_ring.make_by_size_and_center (0,0,0,0)
				from	package_connections.start
				until	package_connections.after
				loop	conn_ring.extend_to_enclose(package_connections.item)
					package_connections.forth
				end
				conn_pitch := (pkg.package_cavity_size_x_mil + pkg.package_cavity_size_y_mil) * 25.4 * 2 / pkg.bonding_finger_count
				-- now draw the connections
				from
					package_connections.start
				until
					package_connections.after
				loop
					f := package_connections.item
					p := finger_path (f.bonding_finger_layout)
					if p /= Void then
						create m.make_from_matrix (f.orientation)
						p.set_transform (m)
						-- Now add the finger path to the drawing
					else
						-- FIXME:  put a marker instead of empty path
						create p.make (0)
					end
					-- if the pad is to be fill, add just a filled version
					-- at this point and add the outline after the text
					--Result.add_figure (p)
					-- get the pad's box (and trim to reduce outsized labels)
					b := p.bounding_box
					if is_left_or_right_connection(f, conn_ring) then
						b.set_height (b.height.min (conn_pitch))
					else	b.set_width (b.width.min (conn_pitch))
					end
					if f.is_null_pin_id then
						shrink_box (b)
						t := text_in_box (f.bonding_finger_id, b)
					elseif b.height < 1.5 * b.width then
						shrink_box (b)
						t := text_in_box (f.bonding_finger_id + " " + f.pin_id, b)
					else
						b2 := b.twin
						b.move_and_resize (b.x, b.y + 0.25 * b.height, b.width, 0.5 * b.height)
						shrink_box (b)
						t := text_in_box (f.bonding_finger_id, b)
						Result.add_figure (t)
						b2.move_and_resize (b2.x, b2.y - 0.25 * b2.height, b2.width, 0.5 * b2.height)
						shrink_box (b2)
						t := text_in_box (f.pin_id, b2)
					end
					Result.add_figure (t)
					Result.add_figure (p) -- now add the pad
					package_connections.forth
				end
			end
		end

	is_left_or_right_connection(f: PACKAGE_CONNECTIONS_ROW conn_ring: RECTANGLE): BOOLEAN
		-- using conn_ring, decide whether f is l/r (vs t/b)
	do  Result := (f.bonding_finger_location_x - conn_ring.low_x).min
			(conn_ring.high_x - f.bonding_finger_location_x)
		      <
		      (f.bonding_finger_location_y - conn_ring.low_y).min
			(conn_ring.high_y - f.bonding_finger_location_y)
	end

	wire_stats_string: STRING
		-- printable string with wiring stats
	local	ws: like wires
		w: IC_PACKAGING_WIRES_ROW
		long, shrt, len: DOUBLE
		pt: POINT
	do	long := 0
		shrt := 1000000000
		ws := wires
		from  ws.start
		until ws.after
		loop  w := ws.item
		      pt := pad_loc_in_package (w.design_pad)
		      len := pt |-| w.package_connection
		      long := long.max(len)
		      shrt := shrt.min(len)
		      ws.forth
		end
		Result := ws.count.out + " wires; longest is " +
			  long.rounded.out + "; shortest is " +
			  shrt.rounded.out + Micrometers_label
	end

	pads_stats_string: STRING
		-- printable test re pad size and pitch
	local	ppad, pad: CUST_IC_DESIGN_PADS_ROW
		sdim, ldim, pitch: DOUBLE
	do
		if pads /= Void and then pads.count > 1 then
			sdim := 1000000  -- min of shorter pad dimensions
			ldim := 1000000  -- min of longer pad dimensions
			pitch:= 1000000  -- min of pad pitches
			from	ppad := pads.last
				pads.start
			until	pads.after
			loop	pad := pads.item
				if pad.bonded then
					-- add pad into the minima
					sdim := sdim.min(pad.size_min)
					ldim := ldim.min(pad.size_max)
					pitch := pitch.min(pad |-| ppad)
					ppad := pad	-- and advance
				end
				pads.forth
			end
			Result := "Minimum pad size: " + ldim.rounded.out +
				" x " + sdim.rounded.out +
				"; minimum pad pitch: " +
				pitch.rounded.out + Micrometers_label
		end
	end

	finger_path (l: BONDING_FINGER_LAYOUTS_ROW): DRW_PATH
		local
			g: DRW_PATH_GENERATOR
		do
			if not l.is_null_path then
				create g.make (l.path)
				g.scan_path
				Result := g.last_drawing_path
				Result.set_stroke_width (thin_width)
			end				
		end

	append_mosis_logo
			-- Generate MOSIS logo and append to `last_title'
		local
			box: DRW_RECTANGLE
			logo: DRW_MOSIS_LOGO
			logo_group: DRW_GROUP
		do
			-- Add logo box (at bottom left)
			create box.make (title_logo_width, 0.4 * title_height,
					0.5 * title_logo_width, 0.2 * title_height)
			box.set_stroke_width (thin_width)
			last_title.extend (box)
			
			-- Add the logo itself
			create logo
			logo_group := logo.full_color
			logo_group.set_view_box (box.bounding_box)
			shrink_box (logo_group.view_box)	-- Adjust the view box to provide margin
			last_title.extend (logo_group)
		end

	set_margins
			-- Set margins
		do
			precursor
			top_margin := 0.25
			bottom_margin := 0.25
		end

	set_sizes
		do
			precursor
			title_logo_width := 7.0 / 8.0
			title_aux_width := 1.75
			title_drawing_separation := 0.25
			title_fraction := 0.25
		end

end
