
indexing
	
	description: "A generator for objects that produce bonding diagrams"
	author: "David Hollenberg"
	date: "February 24, 2005"

class BONDING_DIAGRAM_GENERATOR
inherit
	DB_OPTIONS_ACCESS

create
	make

feature {NONE} -- Creation

	make (conn: DB_CONNECTION)
			-- Create generator with `conn' as database connection
		require
			conn_exists: conn /= Void
		do
			connection := conn
		end

feature -- Properties

	customer_bonding_diagram (a_packaging: IC_PACKAGINGS_ROW prod: STRING): BONDING_DIAGRAM
			-- Diagram that shows bonding for `a_packaging'
			-- on production `prod' with customer
			-- information included
		do
			Result := bonding_diagram (a_packaging, prod)
			Result.set_is_for_customer (True)
		end

	vendor_bonding_diagram (a_packaging: IC_PACKAGINGS_ROW prod: STRING): BONDING_DIAGRAM
			-- Diagram that shows bonding for `a_packaging'
			-- on production `prod' with vendor
			-- information included
		do
			Result := bonding_diagram (a_packaging, prod)
			Result.set_is_for_customer (False)
		end


feature {NONE} -- Implementation

	bonding_diagram (a_packaging: IC_PACKAGINGS_ROW prod: STRING): BONDING_DIAGRAM
			-- Diagram that shows bonding for `a_packaging'
			-- on production `prod'
		local
			pkg: PACKAGES_ROW
			ifi: IC_FAB_INSTANCES_ROW
			cut_die: CUT_DIE_SITES_ROW
			cut_dies: LIST [CUT_DIE_SITES_ROW]
			ca: CUST_ACCTS_ROW
			cas: LIST [CUST_ACCTS_ROW]
			cid: CUST_IC_DESIGNS_ROW
			wires: LIST [IC_PACKAGING_WIRES_ROW]
			cwli: CUST_WO_LINE_ITEMS_ROW
			cwlis: LIST [CUST_WO_LINE_ITEMS_ROW]
			pkg_conns: LIST [PACKAGE_CONNECTIONS_ROW]
			pkg_conn_table: HASH_TABLE [PACKAGE_CONNECTIONS_ROW, STRING]
			bond_finger: BONDING_FINGER_LAYOUTS_ROW
			bond_fingers: LIST [BONDING_FINGER_LAYOUTS_ROW]
			table: HASH_TABLE [BONDING_FINGER_LAYOUTS_ROW, INTEGER]
			pad: CUST_IC_DESIGN_PADS_ROW
			pads: LIST [CUST_IC_DESIGN_PADS_ROW]
			pad_table: HASH_TABLE [CUST_IC_DESIGN_PADS_ROW, STRING]
		do
			ifi := a_packaging.possible_fab_instance
			if ifi.is_empty then
				ifi := Void
				create {ARRAYED_LIST [CUST_IC_DESIGN_PADS_ROW]} pads.make (0)
					-- No ifi, so can't get design_no
			else
				cid := ifi.cust_ic_design
				create pad.make_template
				pads := pad.rows_for_design_no (ifi.design_no, connection, expected_count.At_least_zero)
				if ifi.is_null_die_id or ifi.die_id.is_empty then
					-- cut_dies := Void
				else
					create cut_die.make_template
					cut_dies := cut_die.rows_for_ic_packaging (a_packaging.ic_packaging_no, connection, expected_count.At_least_zero)
					if cut_dies.is_empty then
						cut_dies := cut_die.rows_for_die (ifi.run_id, ifi.die_id, connection, expected_count.At_least_zero)
						if cut_dies.is_empty then
							cut_dies := Void
						end
					end
				end
			end
			create cwli.make_template
			cwlis := cwli.rows_for_ic_packaging (a_packaging.ic_packaging_no, prod, connection, expected_count.At_least_zero)
			if cwlis.is_empty then
				cwlis := Void
			end
			create ca.make_template
			cas := ca.rows_for_ic_packaging (a_packaging.ic_packaging_no, connection, expected_count.At_least_zero)
			if cas.is_empty or cas.count > 1 then
				-- no or multiple accounts for this packaging, use the one from the design (below)
				create ca.make_by_key (connection, option.Old_no_exception_row, cid.cust_acct_no)
				if ca.is_empty then
					ca := Void
				end
			else
				ca := cas.first
			end


			pkg := a_packaging.my_package
			if pkg /= Void then
				wires := a_packaging.my_wiring
				pkg_conns := pkg.my_connections
				create pkg_conn_table.make (pkg_conns.count)
				pkg_conns.do_all (agent put_in_pkg_conn_table (?, pkg_conn_table))
				create pad_table.make (pads.count)
				pads.do_all (agent put_in_pad_table (?, pad_table))
				wires.do_all (agent set_wire_connection_and_pad (?, pkg_conn_table, pad_table))
				create bond_finger.make_template
				bond_fingers := bond_finger.rows_for_package_id (a_packaging.package_id, connection, expected_count.At_least_zero)
				create table.make (bond_fingers.count)
				bond_fingers.do_all (agent put_in_table (?, table))
				pkg_conns.do_all (agent set_finger_layout (?, table))
			end

			create Result.make_with_date (connection.system_date)
			Result.set_packaging (a_packaging)
			Result.set_package (pkg)
			Result.set_fab_instance (ifi)
			Result.set_cut_dies (cut_dies)
			Result.set_cust_ic_design (cid)
			Result.set_cust_acct (ca)
			Result.set_pads (pads)
			Result.set_cust_wo_line_items (cwlis)
		ensure
			result_exists: Result /= Void
		end

	connection: DB_CONNECTION
			-- Database connection to use

	put_in_table (row: BONDING_FINGER_LAYOUTS_ROW table: HASH_TABLE [BONDING_FINGER_LAYOUTS_ROW, INTEGER])
			-- Put `row' in `table' under bonding finger layout
			-- number
		do
			table.put (row, row.bonding_finger_layout_no)
		end

	set_finger_layout (row: PACKAGE_CONNECTIONS_ROW table: HASH_TABLE [BONDING_FINGER_LAYOUTS_ROW, INTEGER])
			-- Retrieve `bonding_finger_layout' for `row'
			-- from `table' and put in `row'
		do
			row.set_bonding_finger_layout (table.item (row.bonding_finger_layout_no))
		end

	put_in_pad_table (row: CUST_IC_DESIGN_PADS_ROW table: HASH_TABLE [CUST_IC_DESIGN_PADS_ROW, STRING])
			-- Put `row' in `table' under pad name
			--  this assumes all wires are for a single design
		do
			table.put (row, row.pad_name)
		end

	put_in_pkg_conn_table (row: PACKAGE_CONNECTIONS_ROW table: HASH_TABLE [PACKAGE_CONNECTIONS_ROW, STRING])
			-- Put `row' in `table' under package connection bonding finger ID
			--  this assumes all connections are for a single package ID
		do
			table.put (row, row.bonding_finger_id)
		end


	set_wire_connection_and_pad (row: IC_PACKAGING_WIRES_ROW
				c_table: HASH_TABLE [PACKAGE_CONNECTIONS_ROW, STRING]
				dp_table: HASH_TABLE [CUST_IC_DESIGN_PADS_ROW, STRING])
			-- Retrieve `package_connection' for `row' from `c_table'
			-- Retrieve `design_pad' for `row' from `dp_table'
			-- and put both in `row'
		do
			row.set_package_connection (c_table.item (row.bonding_finger_id))
			row.set_design_pad (dp_table.item (row.pad_name))
		end

invariant
	connection_exists: connection /= Void
end
