deferred class BONDING_POINT
		-- a named point that sits somewhere on a ring around
		-- a rectangle (or maybe not!)

inherit
	NAMED_POINT

feature	-- status

	interior_box: CONST_RECTANGLE
	-- defines the "interior", 
	--  and thereby, also left, top, right, and bottom
	-- NOTE that a "corner" point will be on two (adjacent) sides
	-- When used for a package, the box should be the cavity
	--  and there should not be any interior points!
	-- When used for a project, the box reflects an arbitrary
	--  dividing line between "edge" and "internal" pads.

	have_interior: BOOLEAN is
		-- has the interior been defined yet?
	do  Result := interior_box /= Void
	end;

	is_on_top: BOOLEAN is
	require have_interior
	do  Result := y > interior_box.high_y
	end;

	is_on_bottom: BOOLEAN is
	require have_interior
	do  Result := y < interior_box.low_y
	end;

	is_on_right: BOOLEAN is
	require have_interior
	do  Result := x > interior_box.high_x
	end;

	is_on_left: BOOLEAN is
	require have_interior
	do  Result := x < interior_box.low_x
	end;

	is_internal: BOOLEAN is
	require have_interior
	do  Result :=	interior_box.contains(Current)
	end;

feature	-- modification

	set_interior_box(b: CONST_RECTANGLE) is
	do  interior_box := b
	end;

end -- BONDING_POINT
