
indexing
	
	author: "David Hollenberg"
	date:	"April 25, 2001"
	description: "A chunk of C memory which will be freed when the %
		%the Eiffel object that points to it is garbage collected"

class C_MEMORY_CHUNK
inherit
	MEMORY
		redefine
			dispose
		end
	C_MEMORY_ALLOCATION
		rename
			store_pointer as c_memory_store_pointer
		export
			{NONE} all
		end

create
	make, make_pointer

feature {NONE} -- Creation

	make (n: INTEGER)
			-- Allocate a chunk of C memory not subject to
			-- garbage collection that is `n' bytes long
			-- and put its address in `address'
		do
			capacity := n
			address := allocate_nongc_memory (n)
		end

	make_pointer (n: INTEGER)
			-- Allocate a chunk of C memory not subject to
			-- garbage collection that is large enough to
			-- hold `n' C pointers and put its address in
			-- `address'
		do
			capacity := n * pointer_size
			address := allocate_nongc_memory (capacity)
		end

feature -- Properties

	address: POINTER
			-- Address of C memory not subject to garbage
			-- collection

	capacity: INTEGER	
			-- Number of bytes allocated for memory at `address'
	
	pointer_capacity: INTEGER
			-- Number of pointers that can be stored in
			-- memory at `address'
		do
			Result := capacity // pointer_size
		end
	

feature -- Modification

	copy_to_address (src: ANY len: INTEGER)
			-- Copy `len' bytes from `src' to memory pointed
			-- to by `address'
		do
			copy_nongc_memory ($src, address, len)
		end
	
	store_pointer (p: POINTER n: INTEGER)
			-- Store `p' in `n'th pointer slot (relative to 0)
			-- of memory pointed to by `address'
		require
			offset_large_enough: n >= 0
			offset_small_enough: n <= pointer_capacity - 1
		do
			c_memory_store_pointer (p, address, n)
		end
	
	clear
			-- Set memory pointed to by `address' to zeroes
		do
			set_nongc_memory (address, '%U', capacity)
		end
	
	clear_address
			-- Set `address' to null pointer, so that
			-- memory it points to won't be freed when
			-- this object is garbage collected.  Caller
			-- must assume responsibility for freeing
			-- memory when appropriate
		do
			address := default_pointer
		end
	
	dispose
			-- Free allocated memory
		local
			null_ptr: POINTER
		do
			if address /= null_ptr then
				free_nongc_memory (address)
				address := null_ptr
			end
		end

			
end
