deferred class CACHED_BOUNDING_BOX
	-- implement the LAYOUT bounding_box as a either
	--  a cached value that is calculated upon first request, or
	--  an uncached value that is recomputed upon every request

inherit
	ANY
	undefine	out
	end;

feature	-- the caching control

	is_caching_bb: BOOLEAN
		-- are we calculating the bb just once and caching its value?
		-- default is false (calculated each call)

	set_is_caching_bb(a_val: BOOLEAN)
	do  is_caching_bb := a_val
	    if not is_caching_bb then
		cached_bb := Void
	    end
	end;

feature	-- the deferred LAYOUT features

	have_bounding_box: BOOLEAN is
		-- do I currently have a cached bounding box?
		-- (false does NOT mean that bounding_box will return Void)
	do  Result := cached_bb /= Void
	end;

	bounding_box: CONST_RECTANGLE is
		-- a Void box means the item has NO layout
		-- (e.g., an empty LAYOUT_COLLECTION)
	do  if is_caching_bb then
		if cached_bb = Void then
			cached_bb := calculate_bb
		end;
		Result := cached_bb
	    else
		Result := calculate_bb
	    end
	end;

feature {NONE} -- bounding_box implementation

	cached_bb: CONST_RECTANGLE
		-- hold it here to avoid repeating ourselves
		-- Note to descendents: whenever your layout changes,
		--  you also need to Void out your cached_bb

	calculate_bb: CONST_RECTANGLE
		-- calculate my bounding box now
	deferred
	end;

end -- CACHED_BOUNDING_BOX
