class CATS_CONSTANTS
	-- constants and validation routines
	-- and a few other simple items

inherit
	GEOMETRY_SUPPORT

	NUMBER_UTILITIES_ACCESS

feature -- the Formats (the few we use)

	frmt_is_flat:		STRING is "FLAT"
	frmt_is_mebes:		STRING is "MEBES"
	frmt_is_mebes4:		STRING is "MEBES/4"

feature	-- the Units (stick to microns, please!!)

	unit_is_micron:		STRING is "MICRON"

feature	-- the Functions (Boolean operators)

	cats_op_and:		STRING is "AND"
	cats_op_or:		STRING is "OR"
	cats_op_xor:		STRING is "XOR"
	cats_op_minus:		STRING is "MINUS"

feature	-- the Justify(cations)

	just_is_center:		STRING is "CENTER"
	just_is_origin:		STRING is "ORIGIN"

feature	-- the Produce option

	produce_cflt:		STRING is "CFLT"

feature	-- the (sizing) Rule option

	rule_is_square:		STRING is "SQUARE"
	rule_is_paragon:	STRING is "PARAGON"
	rule_is_extend:		STRING is "EXTEND"
	rule_is_octagon:	STRING is "OCTAGON"

feature	-- Border option

	border_is_fit:		STRING is "FIT"
	border_is_inside:	STRING is "INSIDE"
	border_is_outside:	STRING is "OUTSIDE"

feature	-- Orientation options

	orient_is_0:		STRING is "0"
	orient_is_90:		STRING is "90"
	orient_is_180:		STRING is "180"
	orient_is_270:		STRING is "270"
	orient_is_Mir0:		STRING is "MIRROR0"
	orient_is_Mir45:	STRING is "MIRROR45"
	orient_is_Mir90:	STRING is "MIRROR90"
	orient_is_Mir135:	STRING is "MIRROR135"

feature	-- CIF root structure name that Cats uses

	cif_root_structure_name: STRING is "TOP"

feature	-- Commands to perform

	cmd_is_do:		STRING is "DO"
	cmd_is_area:		STRING is "AREA"
	cmd_is_extent:		STRING is "EXTENT"

feature	-- standard Cats file extensions (with the '.')

	dot_cflt: STRING is ".cflt"
	dot_clib: STRING is ".clib"
	dot_clay: STRING is ".clay"
	dot_cinc: STRING is ".cinc"

feature	-- the standard working directory for Cats

	ted_path: STRING is "$TED/"

feature	-- validity tests

	is_valid_Boolean_op(op: STRING): BOOLEAN is
	do  Result := equal(op, cats_op_and) or equal(op, cats_op_or) or
		      equal(op, cats_op_xor) or equal(op, cats_op_minus)
	end;

	is_valid_rule(rul: STRING): BOOLEAN is
	do  Result :=	equal(rul, rule_is_square) or
			equal(rul, rule_is_paragon) or
			equal(rul, rule_is_extend) or
			equal(rul, rule_is_octagon)
	end;

	is_valid_border(brdr: STRING): BOOLEAN is
	do  Result :=	equal(brdr, border_is_fit) or
			equal(brdr, border_is_inside) or
			equal(brdr, border_is_outside)
	end;

	is_valid_cats_orientation(m: MATRIX): BOOLEAN is
		-- is this one of the 8 Cats orientations, exactly
	require m /= Void
	do  Result := m.is_equal(identity) or m.is_equal(rot90)  or
		      m.is_equal(rot180)   or m.is_equal(rot270) or
		      m.is_equal(mirror0)  or m.is_equal(mirror45) or
		      m.is_equal(mirror90) or m.is_equal(mirror135)
	end;

	Cats_orientation_name(m: MATRIX): STRING is
	require m /= Void
	do  if     m.is_equal(identity) then Result := orient_is_0
	    elseif m.is_equal(rot90)    then Result := orient_is_90
	    elseif m.is_equal(rot180)   then Result := orient_is_180
	    elseif m.is_equal(rot270)   then Result := orient_is_270
	    elseif m.is_equal(mirror0)  then Result := orient_is_Mir0
	    elseif m.is_equal(mirror45) then Result := orient_is_Mir45
	    elseif m.is_equal(mirror90) then Result := orient_is_Mir90
	    elseif m.is_equal(mirror135) then Result := orient_is_Mir135
					else Result := "????"
	    end
	end;

feature	-- spot size stuff

	spot_size_grain: INTEGER is 40000;	-- 25 pico-meters
		-- we will assume that all spot sizes are integral
		-- multiples of 1/spot_size_grain microns
	
	common_spot(s1, s2: DOUBLE): DOUBLE is
		-- greatest spot size that can handle both s1 and s2
		-- (i.e., the "gcd" of s1 and s2)
	do  Result := nu.gcd((s1*spot_size_grain).rounded,
			     (s2*spot_size_grain).rounded);
	    Result := Result / spot_size_grain
	end;

feature	-- our "standard" formatter for micron-ish numbers: to nearest 100 pico-meters

	fd4: FORMAT_DOUBLE is
		-- a formatter for 4 decimal places
	once create Result.make(20,4);
	     Result.no_justify
	end;

end -- CATS_CONSTANTS
