deferred class CATS_EXPR_HOLDER
	-- an object that "holds" (refers to) a CATS_EXPRESSION
	-- used by the optimizer to locate and replace
	--  common subexpressions

feature	-- status

	holds(ce: CATS_EXPRESSION): BOOLEAN is
		-- do I hold a CATS_EXPRESSION that is_equal to ce?
	require ce /= Void
	deferred
	end;

feature	-- registration

	register_all_exprs(ce_opt: CATS_EXPR_OPTIMIZER; stmt_no: INTEGER) is
		-- register (recursively downward) my held
		-- expression(s) with the optimizer
		-- stmt_no is the statement number in the list
	require stmt_no > 0
	do  if first_statement_no = 0 or stmt_no < first_statement_no then
		first_statement_no := stmt_no
	    end;
	    register_all_exprs2(ce_opt, stmt_no)
	end;

feature {NONE} -- the actual registrar

	register_all_exprs2(ce_opt: CATS_EXPR_OPTIMIZER; stmt_no: INTEGER) is
		-- register (recursively downward) my held
		-- expression(s) with the optimizer
		-- stmt_no is the statement number in the list
	deferred
	end;

feature	-- replacement

	replace(olde, newe: CATS_EXPRESSION) is
		-- replace olde with newe
		-- (if the holder is a shared object that appears more
		--  than once in the statement set, then the replacing
		--  may have already taken place)
	require hold_old_or_new: holds(olde) or holds(newe)
		newe_exists: newe /= Void
	deferred
	ensure holds(newe)
	end

feature	-- position indicator

	first_statement_no: INTEGER
		-- index of the first statement in which this holder appears
		-- (only one appearance unless a shared object)

	compare_statement_no(other: CATS_EXPR_HOLDER): INTEGER is
		-- a sorter
	require other /= Void
	do  Result := first_statement_no.three_way_comparison(other.first_statement_no)
	end;

end -- CATS_EXPR_HOLDER
