deferred class CATS_EXPR_OP [G -> ANY]

	-- a CATS_EXPRESSION that is an operation on one
	-- input CATS_EXPRESSION modified by a second operand of type G
	-- (details in the descendants)

inherit
	CATS_EXPRESSION

feature	-- data

	operand1: CATS_EXPRESSION
		-- the first operand

	function: STRING is
		-- the function name
	deferred
	end;

	operand2: G 
		-- the modifier or second operand

feature	-- compilation

	compile(env: CATS_COMPILATION_ENVIRONMENT): CATS_OPERATION is
	local inp: CATS_OP_FILE
	do  compile_prolog(env);
	    Result := operand1.compile(env);
	    if not can_merge_into_op(Result)
	     then inp := env.fracture_to_temp(Result)
		  Result := env.new_operation(inp)
	    end;
	    compile_op2(env, Result)
	end;

	compile_prolog(env: CATS_COMPILATION_ENVIRONMENT) is
		-- whatever needs to be done before op1 is compiled
	deferred
	end;

	can_merge_into_op(cop: CATS_OPERATION): BOOLEAN is
		-- can I add myself into cop?
		-- (if not, a temporary is generated
		--   and we get a clean start)
	deferred
	end;

	compile_op2(env:CATS_COMPILATION_ENVIRONMENT;
			cop:CATS_OPERATION) is
		-- compile function and op2 into cop
	require can_merge_into_op(cop)
	deferred
	end;

feature	-- equality, display and complexity

	is_equal(other: like Current): BOOLEAN is
		-- really have other: ANY
	do  Result := same_type(other) and then
			(equal(operand1, other.operand1) and
			 equal(operand2, other.operand2))
	end;

	out: STRING is
	do  Result := su.string_of_7
			("(", operand1.out, " #", function, "# ", op2_out, ")")
	end;

	op2_out: STRING is
		-- (separated out for redefinition ease)
	do  Result := operand2.out
	end;

	complexity: INTEGER is
	do Result := operand1.complexity + 1
	end;

feature	-- hash_code

	hash_code: INTEGER is
	local h: HASHABLE; op1h, funh, op2h: INTEGER
	do  op1h := operand1.hash_code \\ 100000;
	    funh := function.hash_code \\ 100000;
	    h ?= operand2;
	    if h /= Void then op2h := h.hash_code \\ 100000 end;
	    Result := 5 * op1h + 3 * funh + 2 * op2h
	end;

feature	-- cats_expr_holder stuff

	holds(ce: CATS_EXPRESSION): BOOLEAN is
	do  Result := operand1.is_equal(ce)
	end;

	register_all_exprs2(ce_opt: CATS_EXPR_OPTIMIZER; stmt_no: INTEGER) is
	do  ce_opt.add_holder(operand1, Current);	-- first me
	    operand1.register_all_exprs(ce_opt, stmt_no)	-- then him
	end;

	replace(olde, newe: CATS_EXPRESSION) is
	do  if operand1.is_equal(olde)
	     then operand1 := newe
	    end;
	end;

feature {NONE} -- creation (for most descendants)

	make(op1: CATS_EXPRESSION; op2: G) is
	require	two_operands: op1 /= Void and op2 /= Void;
	do  operand1 := op1;
	    operand2 := op2
	end;

end -- CATS_EXPR_OP
