deferred class CATS_EXPRESSION
	-- parent for each of the forms of a CATS expression
	-- (an expression involving the transformation of layout thru
	--  BOOLEAN binary operations and unary manipulations)

inherit
	CATS_CONSTANTS
	undefine  is_equal, out  end;

	STRING_UTILITIES_ACCESS
	undefine  is_equal, out  end;

	CATS_ERROR
	undefine  is_equal, out  end;

	CATS_EXPR_HOLDER
	undefine  is_equal, out  end;

	HASHABLE
	undefine  is_equal, out  end;

feature	-- cats boolean expression constructors

	infix "#and#" (other: CATS_EXPRESSION): CATS_EXPRESSION is
	do  create {CATS_EXPR_BOOL}
		Result.make3(cats_op_and, Current, other)
	end;

	infix "#or#" (other: CATS_EXPRESSION): CATS_EXPRESSION is
	do  create {CATS_EXPR_BOOL}
		Result.make3(cats_op_or, Current, other)
	end;

	infix "#xor#" (other: CATS_EXPRESSION): CATS_EXPRESSION is
	do  create {CATS_EXPR_BOOL}
		Result.make3(cats_op_xor, Current, other)
	end;

	infix "#minus#" (other: CATS_EXPRESSION): CATS_EXPRESSION is
	do  create {CATS_EXPR_BOOL}
		Result.make3(cats_op_minus, Current, other)
	end;

feature	-- cats "unary" expression constructors

	infix "#sized#", infix "#sizing#" (amt: DOUBLE): CATS_EXPRESSION is
	do  if amt = 0 then
		Result := Current
	    else
		create {CATS_EXPR_SIZED} result.make(Current, amt)
	    end
	end;

	infix "#scaled#" (amt: DOUBLE): CATS_EXPRESSION is
	require amt > 0
	do  if amt = 1.0 then
		Result := Current
	    else
		create {CATS_EXPR_SCALED} result.make(Current, amt)
	    end
	end;

	prefix "#not#": CATS_EXPRESSION is
	do  create {CATS_EXPR_REVERSED} result.make(Current, true)
	end;

	infix "#selected#" (fltr: STRING): CATS_EXPRESSION is
		-- one filter item
		-- successive filters are merged 
	do  create {CATS_EXPR_SELECTED} result.make(Current, fltr)
	end;

	infix "#tagged#" (tag: STRING): CATS_EXPRESSION is
		-- one tag
	do  create {CATS_EXPR_TAGGED} result.make(Current, tag)
	end;

feature	-- "environmental" operators
	-- these operators modify the expression evaluation environment
	--  for the contained expression by altering the operation_template

	infix "#ruled#" (styl: STRING): CATS_EXPRESSION is
	require is_valid_rule(styl);
	do  create {CATS_EXPR_RULED} result.make(Current, styl)
	end;

	infix "#bordered#" (styl: STRING): CATS_EXPRESSION is
	require is_valid_border(styl);
	do  create {CATS_EXPR_BORDERED} result.make(Current, styl)
	end;

feature	-- input-limiting operators
	--  limit input to either the layout (primary input) or overhead
	--	(secondary input) 

	prefix "#layout_only#": CATS_EXPRESSION is
	do  create {CATS_EXPR_ONLY} result.make(Current, 'L')
	end;

	prefix "#overhead_only#": CATS_EXPRESSION is
	do  create {CATS_EXPR_ONLY} result.make(Current, 'O')
	end;

	infix "#synthesized_from#" (other: CATS_EXPRESSION):
				CATS_EXPRESSION is
	do  Result := (#overhead_only# (Current)) #or#
			(#layout_only# (other))
	end;

feature -- common expression attributes

	complexity: INTEGER is
		-- a measure of the complexity of this expression
		-- (larger value means more work to evaluate)
	deferred
	ensure Result > 0
	end;

	compile(env: CATS_COMPILATION_ENVIRONMENT): CATS_OPERATION is
		-- compile the expression to a single final CATS_OPERATION
		-- (with varying side effects in the env)
	deferred
	end;

end -- CATS_EXPRESSION
