class CATS_LAYER_SOURCES
	-- encodes all the known cases (for a particular DESIGN_TECHNOLOGY)
	-- where the source CATS_EXPRESSION for a layer is something
	-- more complicated than just the layer itself

inherit
	CATS_EXPRESSION_SUPPORT

	CONDITION_UTILITIES

create
	make

feature	-- the interface

	source_expr(lnam: STRING): CATS_EXPRESSION is
		-- the full source expression for lnam as used in both
		--  density checks and layer fractures
		-- (usually just lnam, but sometimes more)
	local foo: CATS_EXPR_LEAF
	do  if my_source_exprs.has(lnam)
	     then Result := my_source_exprs.item(lnam)
	     else foo := lnam		-- convert it
		  Result := foo
	    end;
	ensure Result /= Void
	end;

	my_dtid: STRING is
		-- the design_techn_id for which I stand
	do  Result := my_dt.design_techn_id
	end;

feature {NONE} -- creation

	make(dt: DESIGN_TECHNOLOGY) is
	require dt /= Void
	do  my_dt := dt;
	    build_my_source_exprs
	end

feature {NONE} -- implementation

	my_dt: DESIGN_TECHNOLOGY
		-- as set at creation

	my_source_exprs: HASH_TABLE[CATS_EXPRESSION, STRING]
		-- store all the non-trivial expressions
		-- the key is the layer_name; the value is the
		--  complex expression that should be used.

	build_my_source_exprs is
	local dtid: DESIGN_TECHN_ID_VALUES
	do  create my_source_exprs.make(69);
	    -- TSMC metal slot and fill expressions
	    if equal(my_dtid, dtid.tsmc13) then
		add_metal_fill_and_slot(8, "DUMMY", "SLOT")
	    elseif equal(my_dtid, dtid.tsmc18) then
		add_metal_fill_and_slot(6, "_FILL", "_SLOT")
	    elseif equal(my_dtid, dtid.tsmc25) then
		add_metal_fill_and_slot(5, "_FILL", "_SLOT")
	    end;
	end;

	leaf(alyr: DESIGN_TECHN_LAYER): CATS_EXPR_LEAF is
	do  Result := alyr.layer_name	-- convert it
	end;

	add_metal_fill_and_slot(mcnt: INTEGER;
				fill_suffix, slot_suffix: STRING) is
		-- look for METAL<n>, METAL<n><fill_suffix>,
		--   and METAL<n><slot_suffix>
		-- for mcnt metal layers (1 thru mcnt)
	local mno: INTEGER; mnam: STRING;
		mlyr, slyr, flyr: DESIGN_TECHN_LAYER
	do  from  mno := 1
	    until mno > mcnt
	    loop  mnam := metal_prefix + mno.out;
		  mlyr := my_dt.layer_by_name(mnam)
		  flyr := my_dt.layer_by_name(mnam + fill_suffix);
		  slyr := my_dt.layer_by_name(mnam + slot_suffix)
		  if mlyr = Void or flyr = Void or slyr = Void then
			handle_hard_error("add_metal_fill_and_slot failure for "
					  + my_dtid + " " + mnam)
		  end;
		  my_source_exprs.put
			((leaf(mlyr) #or# leaf(flyr)) #minus# leaf(slyr), mnam)
		  mno := mno + 1;
	    end
	end;

	metal_prefix: STRING is "METAL"

	layer_names: LAYER_NAME_VALUES
		-- (not used, but it "should be" so I want the relationship)

end -- CATS_LAYER_SOURCES
