indexing
	
	description: "String constants used in writing Cats scripts and reading Cats log files"
	author: "Joel Goldberg"
	date: "September 22, 2004"

class CATS_LOG_CONSTANTS

inherit
	ANY
	STRING_UTILITIES_ACCESS
		export
			{NONE} all
		end

feature	-- log section tagging

	section_tag_prefix: STRING = "$!"
			-- Prefix that must occur before every tag

	is_section_tag (s: STRING): BOOLEAN
			-- Is `s' a tag?
		do
			Result := su.is_prefix (section_tag_prefix, s)
		end

	section_name_in_tag (s: STRING): STRING
			-- Name portion of tag `s'
		require
			is_section_tag: is_section_tag (s)
		do
			Result := s.substring (section_tag_prefix.count + 1, s.count)
		ensure
			result_exists: Result /= Void
		end

feature	-- specific section tags

	section_beg_cats_ovrhd: STRING = "BEGIN_CATS_OVRHD"

	section_end_cats_ovrhd: STRING = "END_CATS_OVRHD"

	section_beg_read_acctng: STRING = "READ_ACCTNG_START"

	section_end_read_acctng: STRING = "READ_ACCTNG_DONE"

	section_beg_source_acctng: STRING = "SOURCE_ACCTNG_START"

	section_end_source_acctng: STRING = "SOURCE_ACCTNG_DONE"

	section_beg_cats_project: STRING = "BEGIN_CATS_PROJECT"

	section_end_cats_project: STRING = "END_CATS_PROJECT"

	section_beg_cats_op: STRING = "BEGIN_CATS_OP"

	section_end_cats_op: STRING = "END_CATS_OP"

	section_beg_writefile: STRING = "BEGIN_WRITEFILE"

	section_end_writefile: STRING = "END_WRITEFILE"

	section_beg_is_done: STRING = "BEGIN_IS_DONE"

	section_end_is_done: STRING = "END_IS_DONE"

	section_beg_readfile: STRING = "BEGIN_READFILE"

	section_end_readfile: STRING = "END_READFILE"

	section_beg_merge: STRING = "BEGIN_MERGE"

	section_end_merge: STRING = "END_MERGE"

	section_beg_area_measure: STRING = "BEGIN_AREA_MEASURE"

	section_end_area_measure: STRING = "END_AREA_MEASURE"

	section_beg_user_readfile: STRING = "BEGIN_USER_READFILE"

	section_end_user_readfile: STRING = "END_USER_READFILE"

	section_beg_dc_check: STRING = "BEGIN_DC_CHECK"

	section_end_dc_check: STRING = "END_DC_CHECK"

feature	-- specific sub-section tags for dc_check

	section_beg_root: STRING = "<<<BEGIN_ROOT>>>"

	section_end_root: STRING = "<<<END_ROOT>>>"

	section_beg_structure: STRING = "<<<BEGIN_STRUCTURE>>>"

	section_end_structure: STRING = "<<<END_STRUCTURE>>>"

	section_beg_datalayers: STRING = "<<<BEGIN_DATALAYERS>>>"

	section_end_datalayers: STRING = "<<<END_DATALAYERS>>>"

	section_beg_extent: STRING = "<<<BEGIN_EXTENT>>>"

	section_end_extent: STRING = "<<<END_EXTENT>>>"

	section_beg_checkfile: STRING = "<<<BEGIN_CHECKFILE>>>"

	section_end_checkfile: STRING = "<<<END_CHECKFILE>>>"

feature	-- specific shell script text that gets echoed into log

	structure_real_name_header: STRING = "And the real name is"

end
