class CATS_OP_SUPPORT
	-- common stuff for CATS_SCRIPT_STEPs and friends

inherit
	STRING_UTILITIES_ACCESS

	CATS_CONSTANTS

feature	-- bracket the input lines to a process (shell " << " )

	begin_piped_input(csh: LARGE_PLAIN_TEXT_FILE; cmd: STRING) is
		-- execute cmd; divert subsequent lines to a temp input
	do  csh.putstring(cmd);
	    csh.putstring(" << ");
	    csh.putstring(EOFword);
	    csh.putchar('%N')
	end;

	end_piped_input(csh: LARGE_PLAIN_TEXT_FILE) is
		-- end the input diversion
	do  csh.putstring(EOFword);
	    csh.putchar('%N')
	end;

	EOFword: STRING is "%'EOFEOFEOF%'"

feature -- writing out of (cats) command lines

	add_a_line(csh: LARGE_PLAIN_TEXT_FILE; cmd, val: STRING) is
		-- add a command line to a command file
		-- format so that val (almost) always lines up
	require su.not_empty(cmd)
	local spcs: INTEGER;
	do  csh.putstring(cmd);
	    if su.not_empty(val)
	     then from  spcs := (11 - cmd.count).max(1);
		  until spcs = 0
		  loop  csh.putchar(' ');
			spcs := spcs - 1
		  end;
		  csh.putstring(val)
	    end;
	    csh.putchar('%N');
	end;

	maybe_add_a_line(csh: LARGE_PLAIN_TEXT_FILE; cmd, val: STRING) is
		-- add a line unless val is Void
	do  if val /= Void
	     then add_a_line(csh, cmd, val)
	    end
	end;

feature	-- add a line whose val must be converted to a string

	add_a_line_yn(csh: LARGE_PLAIN_TEXT_FILE; cmd: STRING; val: BOOLEAN) is
	do  if val
	     then add_a_line(csh, cmd, say_yes)
	     else add_a_line(csh, cmd, say_no)
	    end
	end;

	add_a_line_int(csh: LARGE_PLAIN_TEXT_FILE; cmd: STRING; val: INTEGER) is
	do  add_a_line(csh, cmd, val.out)
	end;

	add_a_line_dbl4(csh: LARGE_PLAIN_TEXT_FILE; cmd: STRING; val: DOUBLE) is
		-- to four decimal places
	do  add_a_line(csh, cmd, fd4.formatted(val))
	end;

	add_a_line_bi(csh: LARGE_PLAIN_TEXT_FILE; cmd: STRING; hw: INTEGER) is
	do  add_a_line(csh, cmd, su.string_of_3("[", hw.out, "]"))
	end;

feature	-- log file tagging

	place_a_tag(csh: LARGE_PLAIN_TEXT_FILE; tag: STRING) is
		-- if tag is present, put the ancient echo command into csh
	do  if su.not_empty(tag)
	     then add_a_line(csh, say_echo, "%'%'");
		  add_a_line(csh, say_echo, su.string_of_3
				("%'%D%H!", tag, "%'"));
		  add_a_line(csh, say_echo, "%'%'")
	    end
	end;

	begin_a_section(csh: LARGE_PLAIN_TEXT_FILE; cmd: STRING) is
		-- 
	do  
	    add_a_line(csh, say_say,
			su.string_of_3("<<<BEGIN_", cmd.as_upper, ">>>"))
	end;

	end_a_section(csh: LARGE_PLAIN_TEXT_FILE; cmd: STRING) is
		-- 
	do  
	    add_a_line(csh, say_say,
			su.string_of_3("<<<END_", cmd.as_upper, ">>>"))
	end;

	add_a_section(csh: LARGE_PLAIN_TEXT_FILE; cmd, val: STRING) is 
		-- add a complete section that has just one command line
		--  plus an opening marker (for log searching),
		--  plus a closing marker
	do  begin_a_section(csh, cmd);
	    add_a_line(csh, cmd, val);
	    end_a_section(csh, cmd)
	end;

feature --

	say_echo: STRING is "echo";
	say_yes: STRING is "Yes";
	say_no:  STRING is "No";
	say_say: STRING is "say";

end -- CATS_OP_SUPPORT
