class CATS_OPERATION
	-- a holder for all the attributes of a single cats operation
	-- (i.e., the entire setup for a single fracture
	--  or discovery operation)

inherit
	CATS_CONSTANTS

	CATS_SCRIPT_STEP

	CONDITION_UTILITIES

create
	make_null, clear, m_standard, copy_from_template

feature	-- the operational parameters (other than the 2 inputs)

	format:   STRING
  --	units:	  STRING	-- MICRON always
	compact:  BOOLEAN
  --	pregrid:  BOOLEAN	-- set to constant default
  --	precut:   BOOLEAN	--  "
  --	join:     BOOLEAN	--  "

	function: STRING
	justify:  STRING
	reverse:  BOOLEAN
  --	overlaps: BOOLEAN	--  "
  --	produce:  STRING	-- CFLT is all we know
  --	keep:     BOOLEAN	--  "

	resolution: DOUBLE
	height:   INTEGER	-- in resolution units
	width:    INTEGER	-- in resolution units
	grow:	  DOUBLE
  --	twosided: BOOLEAN	-- NO always
	rule:	  STRING
	border:	  STRING
	tag:      STRING

  --	output:	  STRING	-- inherited from CATS_SCRIPT_STEP

feature	-- the two input files

	primary,
	secondary: CATS_OP_FILE

feature	-- additional stuff we add

	final_comment: STRING	-- a possible tag just before the command
	command: STRING		-- usually, do; sometimes area
	descr: STRING		-- usable for script file comments
	layer_level_name: STRING
		-- (only used by CATS_OPERATION)
		-- for a explicit temp or a merge temp, the layer
		-- for a final, the level_name
		-- for a generated temp1, not used

feature	-- deferred stuff from CATS_SCRIPT_STEP

	step_type: STRING is "CATS_OP"

	command_line: STRING is
	do  Result := su.string_of_2("cats ", initial_cinc_file)
	end;

	input1: STRING is
		-- name of the primary input file (or Void)
	do  if primary /= Void
	     then Result := primary.input
	    end
	end;

	input2: STRING is
		-- name of the secondary input file (or Void)
	do  if secondary /= Void
	     then Result := secondary.input
	    end
	end;

feature	-- output file types

	is_final: BOOLEAN is
		-- this file is the final result of a clevel
	do  Result := output_type = type_is_final
	end;

	is_explicit: BOOLEAN is
		-- this file is the explicit result of a ctemp
	do  Result := output_type = type_is_explicit
	end;

	is_merge: BOOLEAN is
		-- this file is the merge of the primary and secondary
		-- inputs
	do  Result := output_type = type_is_merge
	end;

	is_generated: BOOLEAN is
		-- this file is a generated (single-use) temporary
	do  Result := output_type = type_is_generated
	end;

feature {NONE} -- details of output_type

	output_type: CHARACTER

	type_is_final: CHARACTER is 'F';	-- final (mask) file
	type_is_explicit: CHARACTER is 'X';	-- explicit temporary
	type_is_merge: CHARACTER is 'M';	-- layer merge temporary
	type_is_generated: CHARACTER is 'G'	-- generated temporary

	is_valid_output_type(otyp: CHARACTER): BOOLEAN is
	do  Result := (otyp = type_is_final) or 
			(otyp = type_is_explicit) or
			(otyp = type_is_merge) or
			(otyp = type_is_generated)
	end;
 
feature	-- modification

	set_primary(var: CATS_OP_FILE) is
	require var /= Void
	do  primary := var
	    var.set_my_step(Current)
	end;

	set_format(var: STRING) is
	do  format := var
	end;

	set_compact(var: BOOLEAN) is
	do  compact := var
	end;

	set_function(var: STRING; op2: CATS_OP_FILE) is
	require unary: var = Void implies op2 = Void;
		binary: var /= Void implies (is_valid_boolean_op(var) and
						op2 /= Void)
	do  function := var;
	    secondary := op2;
	    if op2 /= Void then op2.set_my_step(Current) end;
	end;

	set_justify(var: STRING) is
	do  justify := var
	end;

	set_reverse(var: BOOLEAN) is
	do  reverse := var
	end;

	set_resolution(var: DOUBLE) is
	do  resolution := var
	end;

	set_height(var: INTEGER) is
	do  height := var
	end;

	set_width(var: INTEGER) is
	do  width := var
	end;

	set_grow(var: DOUBLE) is
	do  grow := var
	end;

	set_rule(var: STRING) is
	do  rule := var
	end;

	set_border(var: STRING) is
	do  border := var
	end;

	set_tag(var: STRING) is
	do  tag := var
	end;

	set_output_final(var, lvl: STRING) is
	do  output := var;
	    layer_level_name := lvl
	    output_type := type_is_final
	end;

	set_output_merge(var, lyr: STRING) is
	do  output := var;
	    layer_level_name := lyr
	    output_type := type_is_merge
	end;

	set_output_explicit(var, lyr: STRING) is
	do  output := var;
	    layer_level_name := lyr
	    output_type := type_is_explicit
	end;

	set_output_generated(var: STRING) is
	do  output := var;
	    output_type := type_is_generated
	end;

	set_final_comment(var: STRING) is
	do  final_comment := var
	end;

	set_command(var: STRING) is
	do  command := var
	end;

	set_descr(var: STRING) is
	do  descr := var
	end;

	set_layer_level_name(var: STRING) is
	do  layer_level_name := var
	end;

feature	-- initial cinc file

	initial_cinc_file: STRING

	default_initial_cinc_file: STRING is ""

	set_initial_cinc_file(cfil: STRING) is
	do  initial_cinc_file := cfil
	end;

feature	-- creation and/or reset

	m_standard is
		-- initializes to the Mosis "standard" defaults
	do  clear;
	    set_format(frmt_is_flat);
	    set_compact(true);
	    set_height(8192);
	    set_border(border_is_outside)
	end;

	clear is
		-- initializes (or returns) to the state after a Cats
		-- clear command (as of Cats v. 23:02)
	do  make_null;
	    set_format(frmt_is_mebes);
	--  set_units(unit_is_micron);
	    set_compact(false);
	--  set_pregrid(true);
	--  set_precut(true);
	--  set_join(true);
	    set_function(Void, Void);
	    set_justify(just_is_center);
	    set_reverse(false);
	--  set_overlaps(false);
	--  set_produce(produce_cflt);
	--  set_keep(false);
	    set_resolution(1.0);
	    set_height(1024);
	    set_width(32768);
	    set_grow(0);
	--  set_twosided(false);
	    set_rule(rule_is_square);
	    set_border(border_is_fit);
	    set_tag(Void);
	    output := Void;
	    primary := Void;
	    set_command(Void);
	end;

feature {NONE} -- creation

	make_null is
	do  output_is_temp := true
	    initial_cinc_file := default_initial_cinc_file
	end;

	copy_from_template(tmplt: CATS_OPERATION) is
	require tmplt /= Void
	do  copy(tmplt)	-- (no further fix_up required)
	end;

feature	-- status

	modifies_primary: BOOLEAN is
		-- is there anything here that modifies the primary layout?
		-- NOTE: does NOT include factors in the primary itself!!
		-- tag is used to force fracturing without the need to
		--  create an explicit temp, as before an area measurement
	do  Result :=  (function /= Void) or reverse or (grow /= 0) or
			(tag /= Void)
	end;

	output_scale_factor: DOUBLE
		-- the scale factor (relative to 1X drawn of the output
	do  Result := primary.output_scale_factor
	    if secondary /= Void and then
	       Result /= secondary.output_scale_factor then
		handle_hard_error("scale factor mismatch between " +
				  input1 + " and " + input2)
	    end
	end;

feature	-- output

	write_input_to_script(csh: LARGE_PLAIN_TEXT_FILE) is
		-- the list of cats commands (one command per list element)
		-- to set up this very operation in Cats
	do  add_a_line(csh, "say CATS_OP:", 
			su.nicely_truncated_string(descr, 1000));
	    add_a_line_dbl4(csh, "resolution", resolution);
	    -- function (and secondary input)
	    if function /= Void
	     then add_a_line(csh, "function", function);
		  secondary.add_cats_commands(csh);
		  add_a_line(csh, "switch", Void);
	    end;
	    -- now primary input
	    primary.add_cats_commands(csh);
	    -- and now everything else
	    if not equal(format, frmt_is_flat) then
		add_a_line(csh, "format", format)
	    end;
	    add_a_line_yn(csh, "compact", compact);
	    add_a_line(csh, "justify", justify);
	    if reverse
	     then add_a_line_yn(csh, "reverse", reverse) end;
	    add_a_line_dbl4(csh, "resolution", resolution);
	    add_a_line_bi(csh, "height", height);
	    add_a_line_bi(csh, "width", width);
	    if grow /= 0
	     then add_a_line_dbl4(csh, "grow", grow) end;
	    if rule /= rule_is_square
	     then add_a_line(csh, "rule", rule) end;
	    add_a_line(csh, "border", border);
	    maybe_add_a_line(csh, "tag", tag);
	    maybe_add_a_line(csh, "output", output);
	    -- anything special to add?
	    maybe_add_a_line(csh, "say", final_comment);
	    -- the command itself and out
	    add_a_line(csh, command, Void);
	    add_a_line(csh, "exit", Void);
	end;

end -- CATS_OPERATION
