deferred class CATS_SCRIPT_STEP
	-- internal representation of one step in a Cats script.
	-- each step reads 0, 1, or 2 input files and produces
	-- 0 or 1 output file.

inherit
	CATS_OP_SUPPORT

feature	-- the mandatory attributes of each step

	step_type: STRING is
		-- the (fixed) name of this type of step
	deferred
	end;

	command_line: STRING is
		-- the (shell) command line to run/start this process
	deferred
	end;

	processed: BOOLEAN
		-- set when this step is processed (written out)

feature	-- my (up to) three files, by name

	input1: STRING is
		-- the first input file (by name), or Void if no input
	deferred
	end;

	input2: STRING is
		-- the second input file (by name), or Void if none
	deferred
	end;

	output: STRING
		-- the output file (by name), or Void if none created

feature -- output management

	output_is_temp: BOOLEAN
		-- the output file is a temp that should be removed by us

	output_removed: BOOLEAN
		-- the output file has been removed

	output_is_exhausted: BOOLEAN is
		-- have all of my consumers been processed?
	do  if consumers = Void
	     then Result := true
	     else Result := consumers.for_all
				(agent {CATS_FILE_CONSUMER}.processed)
	    end
	end;

	output_is_removable: BOOLEAN is
		-- output can now be removed
	do  Result := su.not_empty(output) and
			output_is_temp and
			(not output_removed) and
			output_is_exhausted
	end;

	consumers: LIST[CATS_FILE_CONSUMER]
		-- all of my output consumers

feature	-- change of mode

	set_output_is_temp(b: BOOLEAN) is
		-- (any descendent that doesn't want clients to change his
		-- opinion of his output's temp-hood needs to redefine
		-- this routine)
	do  output_is_temp := b
	end;

	add_consumer(cons: CATS_FILE_CONSUMER) is
	do  if consumers = Void
	     then create {LINKED_LIST[CATS_FILE_CONSUMER]} consumers.make
	    end;
	    consumers.extend(cons)
	end;

	set_output_removed is
	do  output_removed := true
	end;

feature	-- my opening and/or closing tags

	opening_tag: STRING
	closing_tag: STRING

	set_a_tag(tag: STRING; open: BOOLEAN) is
		-- these are totally passive; we just do as we are told
	do  if open
	     then opening_tag := tag
	     else closing_tag := tag
	    end
	end;

feature	-- output to shell script

	write_to_script(sfil: LARGE_PLAIN_TEXT_FILE) is
		-- the standard script body generator for a step
	do  -- leading tag, if any
	    place_a_tag(sfil, opening_tag);
	    -- command line, with piped input
	    begin_piped_input(sfil, command_line);
	    -- the input line for this command
	    write_input_to_script(sfil);
	    -- EOF for the piped input
	    end_piped_input(sfil);
	    -- and the trailing tag, if any
	    place_a_tag(sfil, closing_tag);
	    -- done
	    processed := true
	end;

	write_input_to_script(sfil: LARGE_PLAIN_TEXT_FILE) is
		-- write out the input lines (if any) 
		--  to run this step into sfil
	deferred
	end;

end -- CATS_SCRIPT_STEP
