deferred class CATS_STATEMENT
	-- an operation that evaluates a CATS_EXPRESSION and then does
	-- something with the results

inherit
	CATS_CONSTANTS

	CATS_EXPR_HOLDER

	STRING_UTILITIES_ACCESS

feature	-- the minimal stuff

	source: CATS_EXPRESSION

	resolution: DOUBLE
		-- resolution for this statement (i.e., for the
		--  evaluation of the source expression)

	mebes_ext: STRING
		-- the 2 character mebes extension (if relevant)

	layer_level_name: STRING
		-- the longer name of the layer or level being produced

	statement_name: STRING
		-- short handle for identification

feature	-- NOTE: my_env and my_op_template are only available at
	--	compile time, not at statement construction time!

	my_env: CATS_COMPILATION_ENVIRONMENT
		-- from compile

	my_op_template: CATS_OPERATION

feature	-- modification

	is_valid_mebes_ext(x: STRING): BOOLEAN is
	do  Result := x = Void or else x.count = 2
	end;

	set_mebes_ext(x: STRING) is
	require is_valid_mebes_ext(x)
	do  mebes_ext := x
	end;

  --	set_layer_level_name(lnam: STRING) is
		-- only done at creation !!

feature	-- compilation

	compile(env: CATS_COMPILATION_ENVIRONMENT) is
	local cop: CATS_OPERATION
	do  -- things to do before compiling source
	    my_env := env;
	    my_op_template := my_env.operation_template;
	    compile_prolog;
	    my_op_template.set_descr(su.string_of_2
			("Part of ", statement_name));
	    my_op_template.set_resolution(resolution);
	    -- compile the source expression
	    cop := source.compile(env);
	    -- now do something with the source expression
	    cop := compile_statement(cop)
	    cop.set_descr(su.string_of_4
			("Last of ", statement_name, " from ", source.out));
	    -- and register it
	    my_env.add_a_step(cop);
	end;

feature -- deferred compilation operators

	compile_prolog is
	deferred
	end;

	compile_statement(cop: CATS_OPERATION): CATS_OPERATION is
	deferred
	end;

feature	-- cats_expr_holder stuff

	holds(ce: CATS_EXPRESSION): BOOLEAN is 
	do  Result := source.is_equal(ce)
	end;

	register_all_exprs2(ce_opt: CATS_EXPR_OPTIMIZER; stmt_no: INTEGER) is
	do  ce_opt.add_holder(source, Current)	-- the full expression
	    source.register_all_exprs(ce_opt, stmt_no)	-- and its subexpressions
	end;

	replace(olde, newe: CATS_EXPRESSION) is
	do  source := newe
	end;

end -- CATS_STATEMENT
