
indexing
	
	description: "Action to take for elements that are scribes %
		%during validity checking"
	author: "David Hollenberg"
	date: "October 1, 2006"

class CHECK_SCRIBE_WIDTH_ACTION
inherit
	VIEW_ELEMENT_ACTION
	STRING_UTILITIES_ACCESS
		export
			{NONE} all
		end
		
feature -- Properties

	multiple_widths_warning: STRING
			-- Warning if there are multiple widths
			-- among all scribes or empty string if
			-- only one distinct scribe width
feature -- Operations

	initialize
			-- Initialize `Current'
		do
			default_width_count := 0
			custom_width_count := 0
			create custom_widths.make (10)
			create all_widths.make (10)
		end

	finish
			-- Add warnings if there are any custom width scribes
		local
			sorter: LIST_BUBBLE_SORTER [DOUBLE]
			comparator: DOUBLE_COMPARATOR
			widths: LIST [DOUBLE]
			array: ARRAY [DOUBLE]
			w: DOUBLE
			k, n: INTEGER
			all_count, horiz_count, vert_count, tot_count: INTEGER
			rec: ELEMENT_WIDTH_RECORD
			msg: STRING
		do
			if custom_width_count > 0 then
				create sorter
				create comparator
				create {ARRAYED_LIST [DOUBLE]} widths.make (custom_widths.count)
				from
					array := custom_widths.current_keys
					k := array.lower
				until
					k > array.upper
				loop
					widths.extend (array.item (k))
					k := k + 1
				end
				widths := sorter.sorted (widths, comparator)
				
				create msg.make (100)
				msg.append ("Found " + custom_width_count.out + " custom width " + su.number (custom_width_count, "scribe", "scribes") + " and " + default_width_count.out + " default width " + su.number (default_width_count, "scribe", "scribes"))
				msg.extend ('%N')
				msg.append ("    " + default_width_count.out + " " + su.number (n, "scribe", "scribes") + " with width " + last_element.grid.formatted_number (last_element.default_scribe_width) + " (default)")
				from
					widths.start
				until
					widths.after
				loop
					w := widths.item
					n := custom_widths.item (w)
					msg.extend ('%N')
					msg.append ("    " + n.out + " " + su.number (n, "scribe", "scribes") + " with width " + last_element.grid.formatted_number (w))
					widths.forth
				end
				last_element.add_warning (msg)
			end
		
			all_count := all_widths.count
			if all_count > 1 then
				create sorter
				create comparator
				create {ARRAYED_LIST [DOUBLE]} widths.make (all_widths.count)
				from
					array := all_widths.current_keys
					k := array.lower
				until
					k > array.upper
				loop
					widths.extend (array.item (k))
					k := k + 1
				end
				widths := sorter.sorted (widths, comparator)
				create msg.make (100)
				msg.append ("Found " + all_count.out + " different scribe widths.  Scribe width distribution:%N")
				from
					widths.start
				until
					widths.after
				loop
					w := widths.item
					rec := all_widths.item (w)
					horiz_count := rec.horizontal_count
					vert_count := rec.vertical_count
					tot_count := horiz_count + vert_count
					msg.extend ('%N')
					msg.append ("    " + tot_count.out + " " + su.number (tot_count, "scribe", "scribes") + " with width " + last_element.grid.formatted_number (w) + " (" + rec.count_summary + ")")
					widths.forth
				end
			else
				msg := ""
			end
			multiple_widths_warning := msg

		end

	apply (an_element: RETICLE_VIEW_ELEMENT)
			-- Apply action to `an_element'
		local
			w: DOUBLE
			rec: ELEMENT_WIDTH_RECORD
		do
			last_element := an_element
			if an_element.is_scribe then
				w := an_element.grid.rounded_size (an_element.scribe_width)
				if an_element.is_default_width_scribe then
					default_width_count := default_width_count + 1
				elseif an_element.is_custom_width_scribe then
					custom_width_count := custom_width_count + 1
					custom_widths.search (w)
					if custom_widths.found then
						custom_widths.force (custom_widths.found_item + 1, w)
					else
						custom_widths.put (1, w)
					end
				end
				
				-- Find scribe width record for this
				-- width 
				
				all_widths.search (w)
				if all_widths.found then
					rec := all_widths.found_item
				else
					create rec.make (w)
					all_widths.put (rec, w)
				end
				if an_element.is_horizontal_scribe then
					rec.increment_horizontal_count
				else
					rec.increment_vertical_count
				end
			end
		end

feature {NONE} -- Implementation

	all_widths: HASH_TABLE [ELEMENT_WIDTH_RECORD, DOUBLE]
			-- Table of scribe width records,
			-- indexed by width

	custom_widths: HASH_TABLE [INTEGER, DOUBLE]
			-- Table of counts of scribes with a particular
			-- width, indexed by width

	default_width_count: INTEGER
			-- Number of scribes with default width

	custom_width_count: INTEGER
			-- Number of scribes with custom width

	last_element: RETICLE_VIEW_ELEMENT
			-- Last scribe processed

end
