class CIF_FILE_WRITER

inherit

	LAYOUT_IO_SUPPORT

	LC_OUTPUT_TERMINAL
	export  {NONE} all;
				-- we are our own layout_consumer
				--   (see put_structure)
	redefine output_text
	end;

	LC_SIMPLE_REFERENCE	-- we have no array support, so
				-- we must expand them

	GEOMETRY_SUPPORT
	export {NONE} all
	end;

	CIF_FILE_OPTIONS

create
	make

feature {NONE} -- the output manager

	my_file: DATA_CONSUMER

	cif_write_scale: DOUBLE
		-- we write out cif_write_scale x the library numbers

	file_position: STRING is "N/A";

	stru_position: STRING is
	do  if curr_structure = Void
	     then Result := "not in symbol"
	     else Result := su.string_of_2("in symbol ",
					   curr_structure.name)
	    end
	end;

	put_string(s: STRING) is
		-- (we may decide to buffer here to reduce overhead)
	do  my_file.consume(s)
	end;

	put_integer(i: INTEGER) is
	do  put_string(i.out)
	end;

	put_character(c: CHARACTER) is
	local s: STRING
	do  create s.make(1);
	    s.extend(c);
	    put_string(s)
	end;

feature {NONE} -- the layout writers (deferred from LC_OUTPUT_TERMINAL)

	semi_eol: STRING is ";%N";

	do_simple_reference(nam: STRING; bdy: LAYOUT; at: MATRIX) is
	local mat: MATRIX
	do  if at.is_cif_legal
	     then mat := at
	     else -- issue error and preserve just the translation part
		  log_io_error(error_is_illmat, at.out);
		  create mat.make_translation_xy(at.tx, at.ty)
	    end;
	    write_call_line(nam, mat)
	end;

	output_rectangle(r: CONST_RECTANGLE) is
	do  update_layer;
	    put_string("B ");
	    write_scaled_point(r.size_x, r.size_y);
	    put_string(" at ");
	    write_scaled_point(r.center_x, r.center_y);
	    put_string(semi_eol);
	end;

	output_polygon(p: POLYGON) is
	do  update_layer;
	    put_string("P ");
	    write_list_of_points(p);
	    put_string(semi_eol);
	end;

	output_path(w: PATH) is
	do  update_layer;
	    -- FIX_ME: cif wires are square-ended.
	    -- if w.ends is not square (or round), we really should
	    --  adjust the first and last points
	    put_string("W ");
	    write_scaled_number(w.width);
	    write_list_of_points(w);
	    put_string(semi_eol);
	end;

	output_port(p: CONST_RECTANGLE) is
	local lay: LAYER; arg: STRING;
	do  if su.not_empty(port_name_command) and
	       su.not_empty(current_name)
	     then create arg.make(80);
		  arg.append(current_name);
		  arg.extend(' ');
		  arg.append_integer(to_scaled_number(p.center_X));
		  arg.extend(' ');
		  arg.append_integer(to_scaled_number(p.center_Y));
		  lay := the_techn.layer_by_name(current_layer);
		  if lay /= Void and then
		     lay.have_cif_name
		   then arg.extend(' ');
			arg.append(lay.a_cif_name)
		  end;
		  put_user_command(port_name_command, arg);
	    end;
	end;

	output_text(t: LAYOUT_TEXT; at: MATRIX)
	do  if lioopt.preserve_gds_text then
		put_comment(t.text)
	    end
	end;

	terminate_traversal: BOOLEAN is false

feature {NONE} -- layer management for output

	last_layer: STRING
		-- last L command (in this structure)

	update_layer is
		-- make sure that the last L command is current
	local lay: LAYER;
	do  if not equal(last_layer, current_layer)
	     then lay := layer_by_name(current_layer);
		  put_string("L ");
		  if lay.have_cif_name
		   then put_string(lay.a_cif_name)
		   else log_io_error(error_is_notrns, current_layer);
			put_string("XX")
		  end;
		  put_string(semi_eol);
		  last_layer := current_layer
	    end;
	end;

feature {NONE} -- writers of the other cif commands

	write_call_line(nam: STRING; at: MATRIX) is
	require is_present: the_lib.has_structure(nam);
		is_cif_legal: at.is_cif_legal
	local  stru: LIBRARY_STRUCTURE;
	do  stru := the_lib.get_structure(nam);
	    put_string("C ");
	    put_integer(stru.cif_number);
	    if not at.is_identity
	     then if at.is_mirrored_in_x
		   then put_string(" MY") end;
		  write_rotation_clause(at.rotation_x, at.rotation_y);
		  write_translation_clause(at.tx, at.ty);
	    end;
	    put_string(semi_eol);
	end;

feature {NONE} -- writers of the bits and pieces

	write_rotation_clause(x, y: DOUBLE) is
		-- R clause of a call (if needed)
	require x /= 0 or y /= 0;
	local zz: DOUBLE;
	do  if x < 0 or y /= 0		-- i.e., angle not 0 
	     then put_string(" R ");
		  -- scale up x, y to large integer range
		  zz := 100000000.0 / (x.abs.max(y.abs))
		  put_integer((x*zz).rounded);
		  put_string(su.comma_space);
		  put_integer((y*zz).rounded);
	    end
	end;

	write_translation_clause(x, y: DOUBLE) is
		-- T clause of a call (if needed)
	do  if x /= 0 or y /= 0
	     then put_string(" T ");
		  write_scaled_point(x, y)
	    end;
	end;

	write_list_of_points(lst: LIST_OF_POINTS) is
	do  from  lst.start
	    until lst.after
	    loop  put_string(" / ");
		  write_scaled_point(lst.item_X, lst.item_Y)
		  lst.forth
	    end;
	end;

	write_scaled_point(x, y: DOUBLE) is
	do  write_scaled_number(x);
	    put_string(su.comma_space);
	    write_scaled_number(y)
	end;

	write_scaled_number(x: DOUBLE) is
		-- write a number that is subject to ds_mul, ds_div scaling
	do  put_integer(to_scaled_number(x))
	end;

	to_scaled_number(x: DOUBLE): INTEGER is
		-- NOTE: rest of scaling is in the ds
	do  Result := (x * cif_write_scale).rounded
	end;

feature {NONE}

	assign_cif_numbers is
		-- assign a cif_number to every structure in the_lib
	local stru: LIBRARY_STRUCTURE; nam: STRING;
		cif, maxcif: INTEGER; missing: BOOLEAN
	do  -- for the numerically named, use the name
	    from  the_lib.start
	    until the_lib.off
	    loop  stru := the_lib.item;  nam := stru.name
		  if nam.is_integer
		   then cif := nam.to_integer;
			stru.set_cif_number(cif);
			maxcif := maxcif.max(cif);
		   else missing := true
		  end;
		  the_lib.forth
	    end;
	    -- for the rest (if any), assign from maxcif on up
	    if missing
	     then  from  the_lib.start
		   until the_lib.off
		   loop  stru := the_lib.item;  nam := stru.name
			 if not nam.is_integer
			  then	maxcif := maxcif + 1;
				stru.set_cif_number(maxcif)
			 end
			 the_lib.forth
		   end
	    end
	end;

feature -- the cif writer

	put_comment(text: STRING) is
		-- write a cif comment;
	do  put_string("( ");
	    put_string(text);
	    put_string(" );%N");
	end;

	put_user_command(cmd, bdy: STRING) is
		-- write a cif user command
	require cmd.is_integer;
	do  put_string(cmd);
	    if su.not_empty(bdy)
	     then put_character(' ');
		  put_string(bdy);
	    end
	    put_string(semi_eol);
	end;

	put_structure(stru: LIBRARY_STRUCTURE; a, b: INTEGER) is
		-- write stru to the file, with a / b as its scale factor
		-- NOTE: all dimensional numbers are written scaled by
		--   cif_write_scale, so (a / b) should equal
		--   (cif_write_scale) structure units in centimicrons
	require exists: stru /= Void
	do  curr_structure := stru;
	    -- DS line
	    put_string("DS ");
	    put_integer(stru.cif_number);
	    put_character(' ');
	    put_integer(a);
	    put_character('/');
	    put_integer(b);
	    put_string(semi_eol);
	    -- name line
	    if su.not_empty(symbol_name_command)
	     then put_user_command(symbol_name_command, stru.name)
	    end;
	    -- the body
	    last_layer := Void;
	    stru.body.traverse(Current, identity);
	    -- DF line
	    put_string("DF;%N");
	    curr_structure := Void
	end;

	put_library(lib: LAYOUT_LIBRARY) is
		-- we set each structure's scale factor to a / b
		-- to get us from lib.unit_size to centimicrons
		-- and then write out all the dimensional values as is
	require exists: lib /= Void
		has_root: su.not_empty(lib.root_structure)
	local   foo: RATIONAL; a, b: INTEGER; cws: DOUBLE;
		stru, root: LIBRARY_STRUCTURE
	do  set_the_lib(lib);
	    cif_write_scale := lioopt.output_write_scale;
	    filtering_techn := lioopt.filtering_techn;
	    traversed_layers := lioopt.traversed_layers;
	    create foo.make_from_double(
		(lib.unit_size/cif_write_scale)/one_centimicron, 10);
	    foo.reduce
	    a := foo.numerator;
	    b := foo.denominator;
	    -- say hello
	    put_comment("written by Eiffel Cif_Writer 1.0");
	    -- write out all the structures in the library
	    assign_cif_numbers;
	    from  lib.start
	    until lib.off
	    loop  stru := lib.item;
		  if lioopt.expand_root_structure and 
		     equal(stru.name, lib.root_structure)
		   then root := stru
		   else put_structure(stru, a, b)
		  end;
		  lib.forth
	    end;
	    -- Call or expand the root structure
	    if root = Void
	     then write_call_line(lib.root_structure, identity)
	     else cws := cif_write_scale;
		  cif_write_scale := (cif_write_scale * a) / b;
		  root.body.traverse(Current, identity)
		  cif_write_scale := cws
	    end;
	    -- write End and be done
	    put_string("End%N");
	end;

feature {NONE} -- creation

	make(sink: DATA_CONSUMER) is
	do  my_file := sink
	    create the_messages.make(writer_application)
	    unknown_layers := lioopt.init_unknown_layers;
	end;

end -- CIF_FILE_WRITER
