class COMPARATOR_SORTED_LIST[G]
		-- a list of items of type G, sorted according to a
		-- provided agent that compares pairs of G

inherit
	LINKED_LIST[G]
	rename make as make_linked
	export {COMPARATOR_SORTED_LIST}	-- disable all Element change routines that
			-- add or modify list elements
			-- (element removal routines are OK)
			make_linked, append, extend, fill, force,
			merge_left, merge_right,
			put, put_front, put_i_th, put_left, put_right,
			replace, swap
	end

create
	make
create {COMPARATOR_SORTED_LIST} 
	make_linked

feature	-- The comparator for this list
	-- It must function like {INTEGER}.three_way_comparison,
	--  always returning one of -1, 0, or 1

	my_comparator: FUNCTION[ANY, TUPLE[G, G], INTEGER];

feature	-- Sort direction

	sort_direction: INTEGER
		-- +1 for increasing sort (default)
		-- -1 for decreasing sort

	set_sort_increasing is
		-- least value (per the comparator) is first
	require is_empty
	do  sort_direction := 1
	end

	set_sort_decreasing is
		-- greatest value (per the comparator) is first
	require is_empty
	do  sort_direction := -1
	end;

feature	-- Element change

	insert(v: like item) is
		-- insert v in its proper place (per sort_direction)
		-- in a tie, new element goes after existing
	require v /= Void
	local l, r, new: LINKABLE[G]
	do  -- find our spot between l and r 
	    from  r := first_element
	    until r = Void or else
		  my_comparator.item([r.item, v]) = sort_direction
	    loop  l := r; r := l.right
	    end;
	    -- build new element for v and slip it between l and r
	    new := new_cell(v);
	    if l = Void
	     then first_element := new
	     else l.put_right(new)
	    end
	    new.put_right(r)	-- (r may be Void)
	    -- fix up count and active (to maintain invariants)
	    count := count + 1
	    if (before and l = Void) or (after and r = Void)
	     then active := new		
	    end
	end;

	insert_list(s: LIST[G]) is
		-- insert all the (non-Void) elements of s
	require s /= Void;
	local v: G
	do  from  s.start
	    until s.after
	    loop  v := s.item;
		  if v /= Void then insert(v) end;
		  s.forth
	    end
	end;

feature {NONE} -- creation

	make(comp: like my_comparator) is
	require comp /= Void
	do  make_linked;
	    my_comparator := comp
	    set_sort_increasing
	end

end -- COMPARATOR_SORTED_LIST
