
indexing
	
	description: "A run closing step operation that completes the %
		%assembly of a run by marking the fab instances as officially %
		%on the run and setting the run state to one that indicates %
		%that the run is reportable."
	author: "David Hollenberg"
	date: "March 15, 2004"

class COMPLETE_RUN_ASSEMBLY_OP
inherit
	RC_PRIMITIVE_STEP_OP
	RC_APPLICABLE_STEP
	NO_USER_INTERFACE
	CREATES_EXTERNAL_STATE

create
	make

feature -- Properties

	input_data_items: LIST [STRING]
			-- Data items required before step can be executed
		do
		    Result := array_as_list (<<
				data_item.Close_date,
				data_item.Assigned_die_ids,
				data_item.Cut_patterns,
				data_item.Initial_wafer_lots,
				data_item.Field_step_pattern,
				data_item.Initial_packaging_lots,
				data_item.Test_vehicle_orders,
				data_item.Unresolved_issue_reports,
				data_item.Edu_charge_adjustments,
				data_item.Production_job,
				data_item.Wafer_techn_survey,
				data_item.Merged_gds_files,
				data_item.Die_mebes_files,
				data_item.Fill_exclusion_file,
				data_item.Fill_area_selector_file,
				data_item.Merged_mebes_files,
				data_item.Filled_mebes_files,
				data_item.Renamed_mebes_files,
				data_item.Final_job_file,
				data_item.Level_density_report
			 >>)
		end

feature {NONE} -- Implementation

	do_step
			-- Execute step
		local
			fab_instances: LIST [IC_FAB_INSTANCES_MROW]
		do
			fab_instances := all_fab_instances_locked
			fab_instances.do_all (agent {IC_FAB_INSTANCES_MROW}.set_pending_for_run (False))
			call_completion_actions
		end

	undo_step
			-- Undo execution of step
		local
			fab_instances: LIST [IC_FAB_INSTANCES_MROW]
		do
			fab_instances := all_fab_instances_locked
			fab_instances.do_all (agent {IC_FAB_INSTANCES_MROW}.set_pending_for_run (True))
			call_completion_actions
		end

	all_fab_instances_locked: LIST [IC_FAB_INSTANCES_MROW]
			-- All fab instance rows for run, already locked
		local
			row: IC_FAB_INSTANCES_MROW
		do
			create row.make_template
			Result := row.locked_fab_instances_for_run (session.run_id, connection, busy_action.Return, expected_count.At_least_zero)
		ensure
			result_exists: Result /= Void
		end

	set_state
			-- Set `state'
		do
			state := Void
		end

end
