
indexing
	
	description: "Action that computes list of projects which have all %
		%copies selected and list of dies for non-projects that have %
		%all copies selected"
	author: "David Hollenberg"
	date: "November 30, 2006"

class COMPUTE_DELETE_LISTS_ACTION
inherit
	VIEW_ELEMENT_ACTION

feature -- Properties

	project_list: LIST [PACKING_PROJECTS_MROW]
			-- List to which to add projects with all copies
			-- selected, regardless of whether any copies are
			-- delete-locked (delete-locked projects will be
			-- removed by project management interface)

	non_project_list: LIST [PACKING_DIES_MROW]
			-- List to which to add dies for non-projects with all
			-- copies selected and none of the copies are
			-- delete-locked

	scribes_only: BOOLEAN
			-- Should selected objects which are not scribes
			-- be ignored?

feature -- Modification

	set_project_list (a_list: like project_list)
			-- Set `project_list' to `a_list'
		require
			a_list_not_void: a_list /= Void
		do
			project_list := a_list
			create projects_table.make (a_list.count)
		ensure
			project_list_set: project_list = a_list
		end

	set_non_project_list (a_list: like non_project_list)
			-- Set `non_project_list' to `a_list'
		require
			a_list_not_void: a_list /= Void
		do
			non_project_list := a_list
			create dies_table.make (a_list.count)
		ensure
			non_project_list_set: non_project_list = a_list
		end

	set_scribes_only (a_value: like scribes_only)
			-- Set `scribes_only' to `a_value'
		do
			scribes_only := a_value
		ensure
			scribes_only_set: scribes_only = a_value
		end

feature -- Operations

	apply (an_element: RETICLE_VIEW_ELEMENT)
			-- Apply action to `an_element'
		local
			p: PACKING_PROJECTS_MROW
			d: PACKING_DIES_MROW
		do
			if not scribes_only or an_element.is_scribe then
				if an_element.is_project then
					p := an_element.project
					projects_table.search (p)
					if not projects_table.found then
						projects_table.put (1, p)
						project_list.extend (an_element.project)
					end
				elseif an_element.is_deletable then
					d := an_element.die
					dies_table.search (d)
					if not dies_table.found then
						dies_table.put (1, d)
						non_project_list.extend (an_element.die)
					end
				end
			end
		end

feature {NONE} -- Implementation

	projects_table: HASH_TABLE [INTEGER, PACKING_PROJECTS_MROW]
			-- Table which records projects that have already
			-- been added to `project_list'

	dies_table: HASH_TABLE [INTEGER, PACKING_DIES_MROW]
			-- Table which records dies that have already
			-- been added to `non_project_list'

end
