
indexing
	
	description: "Set of condition handlers needed by an %
		%application.  Includes handlers for hard errors, %
		%soft errors, warnings, confirmations and informational %
		%messages"
	author: "David Hollenberg"
	date: "October 26, 2003"

deferred class CONDITION_HANDLER_SET

feature -- Properties

	hard_error_handler: CONDITION_HANDLER
			-- Handler for errors that cannot be
			-- retried and cannot be ignored
		deferred
		ensure
			result_exists: Result /= Void
		end

	retryable_hard_error_handler: CONDITION_HANDLER
			-- Handler for errors that can be retried
			-- but cannot be ignored
		deferred
		ensure
			result_exists: Result /= Void
		end

	soft_error_handler: CONDITION_HANDLER
			-- Handler for errors that cannot be
			-- retried but can be ignored
		deferred
		ensure
			result_exists: Result /= Void
		end

	retryable_soft_error_handler: CONDITION_HANDLER
			-- Handler for errors that can be retried
			-- and can be ignored
		deferred
		ensure
			result_exists: Result /= Void
		end

	warning_handler: CONDITION_HANDLER
			-- Handler for warnings that cannot be
			-- retried but can be ignored
		deferred
		ensure
			result_exists: Result /= Void
		end

	retryable_warning_handler: CONDITION_HANDLER
			-- Handler for warnings that can be retried
			-- and can be ignored
		deferred
		ensure
			result_exists: Result /= Void
		end

	question_handler: CONDITION_HANDLER
			-- Handler that asks users a question and gets
			-- a Yes or No answer or allows user to cancel
		deferred
		ensure
			result_exists: Result /= Void
		end

	confirmation_handler: CONDITION_HANDLER
			-- Handler that tells user something and
			-- allows them to either confirm that they want
			-- to continue or to cancel
		deferred
		ensure
			result_exists: Result /= Void
		end

	information_handler: CONDITION_HANDLER
			-- Handler for informational conditions
		deferred
		ensure
			result_exists: Result /= Void
		end

feature -- Modification

	add_notify_action (a: PROCEDURE [ANY, TUPLE [STRING]])
			-- Add action `a' to notify actions of all handlers
			-- in the set
		do
			hard_error_handler.notify_actions.extend (a)
			retryable_hard_error_handler.notify_actions.extend (a)
			soft_error_handler.notify_actions.extend (a)
			retryable_soft_error_handler.notify_actions.extend (a)
			warning_handler.notify_actions.extend (a)
			retryable_warning_handler.notify_actions.extend (a)
			confirmation_handler.notify_actions.extend (a)
			information_handler.notify_actions.extend (a)
		end

invariant
	hard_handler_exists: hard_error_handler /= Void
	hard_retry_handler_exists: retryable_hard_error_handler /= Void
	soft_handler_exists: soft_error_handler /= Void
	soft_retry_handler_exists: retryable_soft_error_handler /= Void
	warning_handler_exists: warning_handler /= Void
	warning_retry_handler_exists: retryable_warning_handler /= Void
	confirmation_handler_exists: confirmation_handler /= Void
	information_handler_exists: information_handler /= Void
end
