deferred class CONFIGURATION_VARS
	-- input of CONFIGURATION vars for the execution environment
	-- from the arguments, and possibly from (recursively)
	--  referenced files

inherit
	ANY
	STRING_UTILITIES_ACCESS
		export
			{NONE} all
		end
	FILE_UTILITIES_ACCESS
		export
			{NONE} all
		end
	EXECUTION_ENVIRONMENT_ACCESS
		export
			{NONE} all
		end

feature	-- constants

	parm_file_key: STRING is "PARM_FILE";
		-- the special 'parameter' that causes us to read a file

	comment_marker: CHARACTER is '#';
		-- if first non-blank, marks a comment line

feature	-- processing error notice (Void = nothing to report)

	configuration_errors: STRING
		-- error(s) encountered in configuring

feature -- argument processing

	configure_from_args(args: ARRAY[STRING]) is
		-- args is (usually) the command line argument array
	local i: INTEGER;
	do  from  i := 1;	-- skip arg[0]
	    until i > args.upper
	    loop  configuration_line(args @ i);
		  i := i + 1
	    end;
	end;

feature {NONE} -- implementation

	configuration_line(lin: STRING) is
		-- look for "key = value" or "key : value"
		--  or "$key = value" or "$key : value"
		--  ($ is trigger to put an environment variable value too)
	local  foo: INTEGER; ln, key, val: STRING; put_env: BOOLEAN;
	do  ln := lin.twin;
	    ln.left_adjust;
	    if ln.is_empty or else (ln @ 1) = comment_marker
	     then	-- empty or comment
	     else foo := ln.index_of('=', 1);
		  if foo = 0 then foo := ln.index_of(':', 1) end;
		  if foo = 0		-- no key = value pair
		   then	add_configuration_error("Bad line: ", lin)
		   else key := ln.substring(1, foo-1);
			key.right_adjust;
			val := ln.substring(foo+1, ln.count);
			val.left_adjust; val.right_adjust;
			put_env := (key.count > 1) and then (key @ 1) = '%D';
			if put_env then  -- remove '$' now; define later
			    key.remove(1)
			    end
			key.to_upper;
			-- now do it (or report error)
			if equal(key, parm_file_key)
			 then configuration_file(val)
			elseif su.is_identifier(key)
			 then set_configuration_variable(key, val);
			      if put_env then
				exe.put(val, key)
			      end
			 else add_configuration_error("Bad key: ", lin)
			end
		  end
	    end
	end;

	configuration_file(nam: STRING) is
		-- add all the parameters in the file
	local  fil: LARGE_PLAIN_TEXT_FILE;
	do  fil := fu.opened_for_read(fu.shell_tilde_expanded(nam));
	    if fil /= Void
	     then  from  fil.read_line;
		   until fil.end_of_file
		   loop  configuration_line(fil.last_string);
			 fil.read_line
		   end
		   fil.close;
	     else  add_configuration_error("Missing parameter file: ",
						nam);
	    end
	end;

	add_configuration_error(str1, str2: STRING) is
	do  if configuration_errors = Void
	     then create configuration_errors.make(0)
	     else configuration_errors.extend('%N')
	    end;
	    configuration_errors.append(str1);
	    configuration_errors.append(str2)
	end;

feature {NONE} -- the deferred argument consumer

	set_configuration_variable(key, val: STRING) is
		-- the ultimate consumer;
		-- another key/value pair to accept
		-- all keys are UPPER case and trimmed
		-- NOTE: both strings are new and can be safely stored
	deferred
	end;

end  -- CONFIGURATION_VARS
