class CONTACTS_ROW

inherit
	CONTACTS_RAW

	STRING_UTILITIES_ACCESS

	MOSIS_ADDR_VALUES
	ROW_COUNT_CHECK_ACCESS
		export
			{NONE} all
		end

create
	make_template, make_null, make_by_key

feature
	name: STRING is
	do  
		create Result.make (40)
		Result.append (first_name)
		if not mid_name.is_empty then
			if not Result.is_empty then
				Result.extend (' ')
			end
			Result.append (mid_name)
		end
		if not last_name.is_empty then
			if not Result.is_empty then
				Result.extend (' ')
			end
			Result.append (last_name)
		end
	end;

	full_name: STRING is
	do  Result := name;
	    if not appellation.is_empty
	     then Result := su.string_of_3(appellation, su.space, Result) end;
	end;

	address(typ: CHARACTER): ADDRESSES_ROW is
		-- one of my addresses (M, B, or S)
	local  a: ADDRESSES_ROW; q: ANONYMOUS_QUERY[ADDRESSES_ROW];
		b: DB_BIND_IC;
	do  create a.make_template;
	    create q.make(connection, a);
	    create b.make(contact_no, typ)
	    q.set_f_and_b("adr.contact_no = :1 and address_type = :2", b);
	    q.execute;
	    q.fetch;
	    if not q.end_of_data then Result := q.row end;
	    q.release
	end;

	billing_address: ADDRESSES_ROW is
	do result := address(addr_is_billing)
	end

	mailing_address: ADDRESSES_ROW is
	do result := address(addr_is_mailing)
	end

	shipping_address: ADDRESSES_ROW is
	do result := address(addr_is_shipping)
	end

	org_full_name: STRING is
		-- name of my organization
	local b: DB_BIND_I;
	do  create b.make(org_no);
	    Result := connection.safe_selected_string("full_name",
				"organizations",
				"org_no = :1", b)
	end;

	full_mail_address(typ: CHARACTER): STRING is
		-- the whole megilla
	local addr: ADDRESSES_ROW;
	do  addr := address(typ);
	    if addr /= Void
	     then Result := addr.full_label
	    end;
	end;

feature -- Queries

	admin_liaisons_for_account (n: INTEGER conn: DB_CONNECTION range: ITEM_COUNT_RANGE): LIST [like Current] is
			-- All contacts for cust account number `n' that
			-- are administrative liaisons
		require
			connection_exists: conn /= Void
			connection_open: conn.is_open
			range_exists: range /= Void
		local
			b: DB_BIND_IS
			role: CONTACT_ROLE_VALUES
		do
			create b.make (n, role.Admin_lias)
			Result := all_rows_filtered (conn, "contact_no in (select contact_no from cust_acct_contacts where cust_acct_no = :1 and contact_role = :2)", b)
			check_row_count (Result, range)
		ensure
			result_exists: Result /= Void
		end

end -- CONTACTS_ROW
