
indexing
	
	description: "A run closing step operation that creates %
		%a fab run order for dedicated and trusted foundry runs"
	author: "David Hollenberg"
	date: "June 3, 2005"

class CREATE_FAB_RUN_ORDER_OP
inherit
	RC_PRIMITIVE_STEP_OP
		redefine
			is_currently_undoable, not_undoable_reason
		end
	HAS_USER_INTERFACE
	NO_EXTERNAL_STATE

create
	make

feature -- Properties

	input_data_items: LIST [STRING]
			-- Data items required before step can be executed
		do
			Result := array_as_list (<<
				data_item.Production_job
				 >>)
		end

feature {NONE} -- Implementation

	is_applicable: BOOLEAN
			-- Is step applicable to run for session?
		do
			Result := cache.is_dedicated_run or cache.is_trusted_foundry_run
		end

	is_currently_undoable: BOOLEAN
			-- Is `Current' currently in a state where it can be
			-- undone?
		do
			Result := not cache.productions_row.production_charges_locked
		end

	not_undoable_reason: STRING
			-- Reason step cannot currently be undone
		do
			Result := "production charges are locked"
		end

	do_step
		local
			ifc: FAB_RUN_ORDER_IFC
		do
			ifc := factory.fab_run_order_interface (session.run_id, session.title_with_run_id ("Create FAB RUN Order"))
			ifc.set_session (session)
			ifc.success_actions.extend (agent call_completion_actions)
			ifc.abort_actions.append (abort_actions)
			ifc.build
		end

	undo_step
			-- Undo execution of step
		do
			inform ("Undoing step " + su.double_quoted (step.presentation_text) + " does not remove any of the line items%Nadded when the step was done nor does it remove their charges.")
			call_completion_actions
		end

	set_state
			-- Set `state'
		do
			state := Void
		end

end
