
indexing
	
	description: "A run closing step operation that creates a field step pattern, %
		%when needed"
	author: "David Hollenberg"
	date: "September 23, 2008"

class CREATE_FIELD_STEP_PATTERN_OP
inherit
	RC_PRIMITIVE_STEP_OP
	NO_USER_INTERFACE
	NO_EXTERNAL_STATE
	DB_OPTIONS_ACCESS
		export
			{NONE} all
		undefine
			default_create
		end

create
	make

feature -- Properties

	input_data_items: LIST [STRING]
			-- Data items required before step can be executed
		do
			Result := array_as_list (<<
				data_item.Completed_packing
				 >>)
		end

feature {NONE} -- Implementation

	is_applicable: BOOLEAN
			-- Is step applicable to run for session?
		do
			Result := cache.is_custom_packing_target and cache.is_reticle_packing_target
		end

	do_step
		do
			insert_field_step_pattern
			insert_field_step_pattern_locations
			call_completion_actions
		end

	undo_step
			-- Undo execution of step
		do
			delete_field_step_pattern_locations
			delete_field_step_pattern
			call_completion_actions
		end

	insert_field_step_pattern
			-- Insert field step patterns row into database
		local
			fsp: FIELD_STEP_PATTERNS_MROW
			name: STRING
		do
			name := cache.field_set_name
			create fsp.make_null (connection)
			fsp.set_field_step_pattern (name)
			fsp.set_field_set_name (name)
			fsp.set_is_preliminary (True)
			fsp.set_descr ("Step plan for " + session.run_id)
			fsp.set_available (True)
			field_step_patterns_row := fsp
		end

	insert_field_step_pattern_locations
			-- Insert field step pattern locations into database
		local
			builder: FIELD_STEP_PATTERN_BUILDER
		do
			create builder
			builder.build_computed_field_locations (field_step_patterns_row, cache.tweezer_margin, 1)
		end

	delete_field_step_pattern
			-- Delete field step patterns row if present
		local
			fsp: FIELD_STEP_PATTERNS_MROW
		do
			create fsp.make_by_key (connection, option.Old_no_exception_row, cache.field_set_name)
			if not fsp.is_empty then
				session.lock_row (fsp)
				fsp.delete
			end
		end

	delete_field_step_pattern_locations
			-- Delete field step pattern locations from database
		local
			floc: FIELD_STEP_PATTERNXFIELD_LOCS_MROW
			list: LIST [DB_DELETABLE_ROW]
			lock_op: ROW_LOCK_OP
		do
			create floc.make_template
			list := floc.locations_for_step_pattern (cache.field_set_name, connection, expected_count.At_least_zero)
			create lock_op
			lock_op.lock_list_nowait (list)
			list.do_all (agent {DB_DELETABLE_ROW}.delete)
		end

	set_state
			-- Set `state'
		do
			state := Void
		end

	field_step_patterns_row: FIELD_STEP_PATTERNS_MROW

end
