
indexing
	
	description: "A run closing step operation that creates a %
		%final job file that will be sent to the mask shop"
	author: "David Hollenberg"
	date: "July 15, 2004"

class CREATE_FINAL_JOB_FILE_OP
inherit
	CREATE_JOB_FILE_OP
		redefine
			job_deck
		end

create
	make

feature -- Properties

	input_data_items: LIST [STRING]
			-- Data items required before step can be executed
		do
			Result := array_as_list (<<
				data_item.Mask_techn,
				data_item.Field_set,
				data_item.Initial_wafer_lots,
				data_item.Test_vehicle_fab_instances
				>>)
		end

feature {NONE} -- Implementation

	is_applicable: BOOLEAN
			-- Is step applicable to run for session?
		do
			Result := cache.run_needs_mebes_files and cache.is_reticle_packing_target
		end

	job_file: RC_FILE
			-- Job file to be written
		do
			Result := file_generator.final_job_file
		end

	create_job_deck
			-- Create `job_deck' with appropriate type of job deck
		do
			create {FINAL_JOB_FILE} job_deck
		end

	set_job_titles_and_orientations
			-- Set any need titles and title orientations in
			-- `job_deck'
		do
			job_deck.set_title_orientations (cache.mask_title_orientations)
			job_deck.set_titles (cache.mask_title_rows)
		end

	set_job_canned_lines
			-- Set any needed prolog or epilog canned lines in
			-- `job_deck'
		do
			job_deck.set_prolog_canned_lines (cache.mask_techn_prolog_lines)
			job_deck.set_epilog_canned_lines (cache.mask_techn_epilog_lines)
		end

	add_merged_chips
			-- Add chips that refer to merged pattern files
		local
			fields: LIST [STRING]
		do
			if needs_merge then
				fields := cache.field_names
				field_count := fields.count
				field_number := 0
				fields.do_all (agent add_merged_chip)
			end
		end

	add_merged_chip (fld_name: STRING)
			-- Add a chip for merged pattern file for field
			-- `fld_name'
		require
			fld_name_exists: fld_name /= Void
		local
			chip: JOB_FILE_CHIP
			chip_levels: LIST [JOB_FILE_CHIP_LEVEL]
			level: JOB_FILE_CHIP_LEVEL
			fname: STRING
			n: INTEGER
			run_level: RUN_LEVELS_ROW
			sorter: LIST_BUBBLE_SORTER [JOB_FILE_CHIP_LEVEL]
			comparator: CHIP_LEVEL_COMPARATOR
		do
			field_number := field_number + 1
			chip_locations := cache.all_merged_mask_locations_for_field (fld_name)
			create chip
			chip_number := chip_number + 1
			chip.set_number (chip_number)
			chip.set_comment ("PRIMARY DIE FOR FIELD " + fld_name.as_upper)
			create {ARRAYED_LIST [JOB_FILE_CHIP_LEVEL]} chip_levels.make (run_levels.count)
			from
				run_levels.start
			until
				run_levels.after
			loop
				run_level := run_levels.item
				n := run_level.level_no
				create level
				level.set_level_number (n)
				level.set_reverse_tone (reverse_tone (run_level.polarity))
				level.set_spot_size (run_level.merged_pattern_file_spot_size)
				if run_level.filled_layer then
					fname := file_generator.level_fill_pattern_file (1, 1, run_level.non_null_mebes_extension).file_name
				else
					fname := file_generator.level_merge_pattern_file (field_number, field_count, run_level.non_null_mebes_extension).file_name
				end
				fname.to_upper
				level.set_file_name (fname)
				chip_levels.extend (level)
				run_levels.forth
			end

			create sorter
			create comparator
			if mask_techn.increasing_spot_size then
				chip_levels := sorter.sorted (chip_levels, comparator)
			else
				chip_levels := sorter.reverse_sorted (chip_levels, comparator)
			end
			chip.set_levels (chip_levels)
			chip.set_locations (chip_locations)
			chips.extend (chip)
		end

	set_chip_locations (row: DIE_ATTRIBUTES_ROW)
			-- Set `chip_locations' for `row'
		do
			if needs_merge then
				chip_locations := cache.all_unmerged_mask_locations_for_die (row)
			else
				chip_locations := cache.all_mask_locations_for_die (row)
			end
		end

	reverse_tone (polarity: STRING): BOOLEAN
			-- Should mask level with `polarity' be written
			-- in reverse tone?
		do
			Result := mask_techn.reverse_tone (polarity)
		end

	job_deck: FINAL_JOB_FILE
			-- Job deck to be created

	field_count: INTEGER
			-- Total number of fields

	field_number: INTEGER
			-- Number of field for which we are currently
			-- writing merged chip

end
