
indexing
	
	description: "A run closing step operation that creates the initial %
		%packaging lots for the run"
	author: "David Hollenberg"
	date: "February 27, 2007"

class CREATE_INITIAL_PACKAGING_LOTS_OP
inherit
	RC_PRIMITIVE_STEP_OP
		redefine
			state
		end
	RC_APPLICABLE_STEP
	HAS_USER_INTERFACE
	NO_EXTERNAL_STATE

create
	make

feature -- Properties

	input_data_items: LIST [STRING]
			-- Data items required before step can be executed
		do
			Result := array_as_list (<<
				data_item.Initial_wafer_lots
				 >>)
		end

	state: PACKAGING_LOTS_STATE
			-- State saved by soft undo

feature {NONE} -- Implementation

	do_step
			-- Execute of step
		local
			ifc: PACKAGING_LOT_CREATION_IFC
		do
			cache.uncache_cut_patterns
			cache.uncache_packaging_lots
			ifc := factory.packaging_lot_creation_interface (session.run_id, session.title_with_run_id ("Create Initial Packaging Lots"))
			ifc.set_session (session)
			ifc.set_state (state)
			ifc.success_actions.extend (agent call_completion_actions)
			ifc.abort_actions.append (abort_actions)
			ifc.build
		end

	undo_step
			-- Undo execution of step
		do
			remove_data
			cache.uncache_cut_patterns
			cache.uncache_packaging_lots
			call_completion_actions
		end

	set_state
			-- Set `state'
		do
			state := Void
		end

	remove_data
			-- Remove data added to database by step execution
		local
			cut_patterns: LIST [RUN_CUT_PATTERNS_MROW]
			packaging_line_items: LIST [CUST_WO_LINE_ITEMS_MROW]
			packaging_lots: LIST [RUN_PACKAGING_LOTS_MROW]
			line_item: CUST_WO_LINE_ITEMS_MROW
			lock_op: ROW_LOCK_OP
		do
			cut_patterns := cache.cut_pattern_mrows
			create line_item.make_template
			packaging_line_items := line_item.rows_for_packaging_run_id (session.run_id, connection, expected_count.At_least_zero)
			packaging_lots := cache.packaging_lot_mrows
			create lock_op
			lock_op.lock_list_nowait (cut_patterns)
			lock_op.lock_list_nowait (packaging_line_items)
			lock_op.lock_list_nowait (packaging_lots)
			cut_patterns.do_all (agent {RUN_CUT_PATTERNS_MROW}.set_null_packaging_lot_no)
			packaging_line_items.do_all (agent {CUST_WO_LINE_ITEMS_MROW}.clear_packaging_lot)
			packaging_lots.do_all (agent {RUN_PACKAGING_LOTS_MROW}.delete)
		end

end
