
indexing
	
	description: "A run closing step operation that creates initial wafer %
		%lots and quantity to order for each lot"
	author: "David Hollenberg"
	date: "February 27, 2007"

class CREATE_INITIAL_WAFER_LOTS_OP
inherit
	RC_PRIMITIVE_STEP_OP
		redefine
			state
		end
	RC_APPLICABLE_STEP
	HAS_USER_INTERFACE
	NO_EXTERNAL_STATE

create
	make

feature -- Properties

	input_data_items: LIST [STRING]
			-- Data items required before step can be executed
		do
			Result := array_as_list (<<
				data_item.Field_step_pattern_existence,
				data_item.Assigned_wafer_processes
				 >>)
		end

	state: WAFER_LOTS_STATE
			-- State saved by soft undo

feature {NONE} -- Implementation

	do_step
		local
			ifc: WAFER_LOT_CREATION_IFC
		do
			ifc := factory.wafer_lot_creation_interface (session.run_id, session.title_with_run_id ("Create Initial Wafer Lots"))
			ifc.set_session (session)
			ifc.set_state (state)
			ifc.success_actions.extend (agent call_completion_actions)
			ifc.abort_actions.append (abort_actions)
			ifc.build
		end

	undo_step
			-- Undo execution of step
		do
			remove_data
			cache.uncache_wafer_lots
			call_completion_actions
		end

	remove_data
			-- Remove data added to database by step execution
		local
			wafer_lots: LIST [RUN_WAFER_LOTS_MROW]
			lock_op: ROW_LOCK_OP
		do
			wafer_lots := cache.wafer_lot_mrows
			create lock_op
			lock_op.lock_list_nowait (wafer_lots)
			wafer_lots.do_all (agent {RUN_WAFER_LOTS_MROW}.delete)
		end

	set_state
			-- Set `state'
		local
			wafer_lots: LIST [RUN_WAFER_LOTS_ROW]
			waf_lots: ARRAYED_LIST [WAFER_PROCESS_QUANTITY_STATE]
		do
			wafer_lots := cache.db_wafer_lot_rows
			create waf_lots.make (wafer_lots.count)
			wafer_lots.do_all (agent add_wafer_lot (waf_lots, ?))
			create state
			state.set_wafer_lots (waf_lots)
		end

	add_wafer_lot (list: LIST [WAFER_PROCESS_QUANTITY_STATE] lot: RUN_WAFER_LOTS_ROW)
			-- Add `lot' to `list'
		require
			list_exists: list /= Void
			lot_exists: lot /= Void
		local
			new_lot: WAFER_PROCESS_QUANTITY_STATE
		do
			create new_lot
			new_lot.set_wafer_process (lot.wafer_process)
			new_lot.set_quantity (lot.quantity_wafers_ordered)
			new_lot.set_step_pattern (lot.field_step_pattern)
			list.extend (new_lot)
		end

end
