
indexing
	
	description: "A run closing step operation that creates a %
		%job file"
	author: "David Hollenberg"
	date: "July 15, 2004"

deferred class CREATE_JOB_FILE_OP
inherit
	RC_PRIMITIVE_STEP_OP
	NO_USER_INTERFACE
	CREATES_EXTERNAL_STATE
	POLARITY_VALUES
		export
			{NONE} all
		undefine
			default_create
		end

feature {NONE} -- Implementation

	do_step
			-- Execute of step
		do
			cache.check_valid_ebeam_type
			cache.check_valid_litho_type
			cache.check_one_scale_factor
			write_job_file
			call_completion_actions
		end

	undo_step
			-- Undo execution of step
		do
			job_file.delete_if_exists
			call_completion_actions
		end

	set_state
			-- Set `state'
		do
			state := Void
		end

	job_file: RC_FILE
			-- Job file to be written
		deferred
		ensure
			result_exists: Result /= Void
		end

	write_job_file
			-- Write job file
		local
			file: LARGE_PLAIN_TEXT_FILE
			f: RC_FILE
		do
			create_job_deck
			job_deck.set_slice_id (session.run_id)
			if cache.mask_size_id.is_empty then
				-- No masks, so use virtual slice code
				job_deck.set_slice_code (cache.virtual_slice_code)
			else
				job_deck.set_slice_code (cache.slice_code)
			end
			set_job_titles_and_orientations
			job_deck.set_parameters (cache.substitution_parameter_set)
			job_deck.set_mask_levels (cache.run_level_rows)
			job_deck.set_mask_techn (cache.mask_techns_row)
			set_job_canned_lines
			set_job_chips
			
			f := job_file
			create file.make (f.file_pathname)
			session.open_file_for_write (file, f)
			job_deck.write (file)
			session.close_file (file)
		end

	create_job_deck
			-- Create `job_deck' with appropriate type of job deck
		deferred
		end

	set_job_titles_and_orientations
			-- Set any need titles and title orientations in
			-- `job_deck'
		deferred
		end
	
	set_job_canned_lines
			-- Set any needed prolog or epilog canned lines in
			-- `job_deck'
		deferred
		end

	set_job_chips
			-- Set `chips' in `job_deck'
		local
			dies: LIST [DIE_ATTRIBUTES_ROW]
		do
			dies := cache.non_ghost_die_attribute_rows
			create {ARRAYED_LIST [JOB_FILE_CHIP]} chips.make (dies.count)
			chip_number := 0
			run_levels := cache.run_level_rows
			mask_techn := cache.mask_techns_row
			needs_merge := cache.run_needs_merge
			add_merged_chips
			dies.do_all (agent add_job_chip)
			job_deck.set_chips (chips)
		end

	add_merged_chips
			-- Add chips that refer to merged pattern files
		deferred
		end

	add_job_chip (row: DIE_ATTRIBUTES_ROW)
			-- Add chip for `row' to `chips' if it has any
			-- locations on mask
		local
			chip: JOB_FILE_CHIP
			chip_levels: LIST [JOB_FILE_CHIP_LEVEL]
			fname: STRING
			level: JOB_FILE_CHIP_LEVEL
			n: INTEGER
			run_level: RUN_LEVELS_ROW
			sorter: LIST_BUBBLE_SORTER [JOB_FILE_CHIP_LEVEL]
			comparator: CHIP_LEVEL_COMPARATOR
		do
			set_chip_locations (row)
			if not chip_locations.is_empty then
				create chip
				chip_number := chip_number + 1
				chip.set_number (chip_number)
				create {ARRAYED_LIST [JOB_FILE_CHIP_LEVEL]} chip_levels.make (run_levels.count)
				from
					run_levels.start
				until
					run_levels.after
				loop
					run_level := run_levels.item
					n := run_level.level_no
					create level
					level.set_level_number (n)
					level.set_reverse_tone (reverse_tone (run_level.polarity))
					level.set_spot_size (cache.xform_level_row_for_dt_level (row.non_null_wafer_design_techn, n).mebes_spot_size)
					fname := file_generator.die_fracture_pattern_file (row.die_id, row.die_wafer_rotation, mask_techn.overetch, run_level.non_null_mebes_extension).file_name
					fname.to_upper
					level.set_file_name (fname)
					chip_levels.extend (level)
					run_levels.forth
				end

				create sorter
				create comparator
				if mask_techn.increasing_spot_size then
					chip_levels := sorter.sorted (chip_levels, comparator)
				else
					chip_levels := sorter.reverse_sorted (chip_levels, comparator)
				end
				chip.set_levels (chip_levels)
				chip.set_locations (chip_locations)
				chips.extend (chip)
			end
		end

	set_chip_locations (row: DIE_ATTRIBUTES_ROW)
			-- Set `chip_locations' for `row'
		require
			row_exists: row /= Void
		deferred
		end

	reverse_tone (polarity: STRING): BOOLEAN
			-- Should mask level with `polarity' be written
			-- in reverse tone?
		require
			polarity_exists: polarity /= Void
			polarity_valid: is_valid_polarity (polarity)
		deferred
		end

	job_deck: JOB_FILE
			-- Job deck to be created

	chip_locations: LIST [CONST_POINT]
			-- Locations on mask where chip is to be written

	chips: LIST [JOB_FILE_CHIP]
			-- List of chips to go in job deck

	chip_number: INTEGER
			-- Last chip number used

	run_levels: LIST [RUN_LEVELS_ROW]
			-- Mask levels for run

	mask_techn: MASK_TECHNS_ROW
			-- Mask techn for run

	needs_merge: BOOLEAN
			-- Do pattern files need to be merged into
			-- one pattern field per field/level combination?

end
