
indexing
	
	description: "A run closing step operation that creates a %
		%production job for the run and sets production id %
		%and final revision no in cust work orders %
		%of all fab instances on run"
	author: "David Hollenberg"
	date: "February 21, 2004"

class CREATE_PRODUCTION_JOB_OP
inherit
	RC_PRIMITIVE_STEP_OP
		redefine
			is_currently_undoable, not_undoable_reason
		end
	RC_APPLICABLE_STEP
	NO_USER_INTERFACE
	NO_EXTERNAL_STATE

create
	make

feature -- Properties

	input_data_items: LIST [STRING]
			-- Data items required before step can be executed
		do
			Result := array_as_list (<<
					data_item.Completed_packing,
					data_item.Test_vehicle_fab_instances
				>>)
		end

feature {NONE} -- Implementation

	is_currently_undoable: BOOLEAN
			-- Is `Current' currently in a state where it can be
			-- undone?
		do
			Result := line_items_with_shipments_count = 0
		end

	not_undoable_reason: STRING
			-- Reason step cannot currently be undone
		local
			m, n: INTEGER
		do
			m := projects_with_shipments_count
			n := line_items_with_shipments_count
			Result := "run has " + n.out + " " + su.number (n, "line item", "line items") + " with shipments (spread over "+ m.out + " " + su.number (m, "project", "projects") + ")"
		end

	line_items_with_shipments_count: INTEGER
			-- Number of line items for projects on run which
			-- have a shipment
		local
			b: DB_BIND_S
		do
			create b.make (cache.production_id)
			Result := connection.selected_count ("cust_wo_line_items", "m_order_no in (select m_order_no from cust_wos where production_id = :1) and m_shipment_no is not null", b)
		end

	projects_with_shipments_count: INTEGER
			-- Number of projects on run which have a line
			-- item with a shipment
		local
			b: DB_BIND_S
		do
			create b.make (cache.production_id)
			Result := connection.selected_count ("ic_fab_instances", "m_fab_no in (select ifi.m_fab_no from ic_fab_instances ifi, cust_wos cw, cust_wo_line_items cwli where ifi.m_fab_no = cw.m_fab_no and cw.m_order_no = cwli.m_order_no and cw.production_id = :1 and m_shipment_no is not null)", b)
		end

	do_step
			-- Execute of step
		do
			update_work_orders (agent do_one_cwo(?))
			call_completion_actions
		end

	do_one_cwo(cwo: CUST_WOS_MROW)
			-- Execute of one CWO of the step
		local b: DB_BIND_I; final_rno: INTEGER
		do
			cwo.set_production_id (session.run_id)
		    if not cwo.is_null_m_fab_no then
			create b.make(cwo.m_fab_no)
			final_rno := connection.selected_integer
					("final_revision_no",
					 "cust_ic_design_attributes cida, ic_fab_instances ifi",
					 "cida.design_no = ifi.design_no and ifi.m_fab_no = :1", b)
			cwo.set_revision_no (final_rno)
		    end
		end

	undo_step
			-- Undo execution of step
		do
			update_work_orders (agent undo_one_cwo(?))
			call_completion_actions
		end

	undo_one_cwo(cwo: CUST_WOS_MROW)
			-- Undo execution of one cwo in the list
		do
			cwo.set_null_production_id
			cwo.set_null_revision_no
		end

	set_state
			-- Set `state'
		do
			state := Void
		end

	update_work_orders (action: PROCEDURE [ANY, TUPLE [CUST_WOS_MROW]])
			-- Update all cust work orders for run by applying
			-- `action' to them
		local
			row: CUST_WOS_MROW
			list: LIST [CUST_WOS_MROW]
		do
			create row.make_template
			list := row.locked_work_orders_for_run (session.run_id, connection, busy_action.return, expected_count.at_least_zero)
			list.do_all (action)
			add_commit_action (agent publish_updated_row_count (row.table_name, list.count))
		end
	
	publish_updated_row_count (table_name: STRING n: INTEGER)
			-- Publish count of changed rows
		require
			table_name_exists: table_name /= Void
			table_name_nonempty: not table_name.is_empty
			number_nonnegative: n >= 0
		local
			e: ROWS_UPDATED_EVENT
		do
			create e
			e.set_table (table_name)
			e.set_count (n)
			e.publish
		end

end
