
indexing
	
	description: "A run closing step operation that creates data for a %
		%test vehicle (fab instance, work order, charges, etc)"
	author: "David Hollenberg"
	date: "March 27, 2007"

deferred class CREATE_TEST_VEHICLE_DATA_OP

inherit
	RC_PRIMITIVE_STEP_OP
	RC_APPLICABLE_STEP
	NO_USER_INTERFACE
	NO_EXTERNAL_STATE

feature {NONE} -- Implementation

	set_state
			-- Set `state'
		do
			state := Void
		end

	fetch_test_vehicle (tv: DIES_ROW)
			-- Fetch test vehicle for `tv' and put in
			-- `test_vehicle'
		require
			test_vehicle_exists: tv /= Void
			is_test_vehicle: tv.is_test_vehicle
		do
			create test_vehicle.make_template
			test_vehicle := test_vehicle.test_vehicle_with_design_no (tv.test_vehicle_design_no, cache.wafer_techn, connection, expected_count.Exactly_one).first

		end

	fetch_cust_acct_no (tv: DIES_ROW)
			-- Fetch account number to use for creating
			-- test vehicle work order and put in `cust_acct_no'
		require
			test_vehicle_exists: tv /= Void
			is_test_vehicle: tv.is_test_vehicle
		local
			ic_design: CUST_IC_DESIGNS_ROW
		do
			create ic_design.make_template
			ic_design := ic_design.rows_with_design_no (tv.test_vehicle_design_no, connection, expected_count.exactly_one).first
			cust_acct_no := ic_design.cust_acct_no
		end

	fetch_funding_info (acct_no: INTEGER)
			-- Fetch funding and pricing information for
			-- account with number `acct_no'
		local
			q: SQL_QUERY
			n: INTEGER
			row_count_checker: ROW_COUNT_CHECK_OP
			bindings: STRING
		do
			q := connection.sql_query (
				"select fa.m_funding_alloc_no, %N%
				%       fau.m_funding_auth_no, %N%
				%       fd.distribution_rank, %N%
				%       pricing_schedule %N%
				%from funding_allocs fa, funding_distributions fd, funding_auths fau %N%
				%where fd.m_funding_alloc_no = fa.m_funding_alloc_no %N%
				%  and fd.m_funding_auth_no = fau.m_funding_auth_no %N%
				%  and fa.cust_acct_no = :1 %N%
				%  and fau.date_effective <= :2 %N%
				%  and nvl(fau.date_expires, :2) >= :2 %N%
				%  and 1 = (select count(*) from funding_distributions f where f.m_funding_alloc_no = fd.m_funding_alloc_no) %N")
			q.set_numeric_bind_capacity (2)
			q.bind_num_to_integer (1, acct_no, False)
			q.bind_num_to_date (2, order_date, False)
			q.set_select_count (4)
			q.select_integer (1)
			q.select_integer (2)
			q.select_integer (3)
			q.select_string (4, 255)
			q.execute
			from
				q.fetch
			until
				q.end_of_data
			loop
				n := n + 1
				m_funding_alloc_no := q.integer_value (1)
				m_funding_auth_no := q.integer_value (2)
				distribution_rank := q.integer_value (3)
				pricing_schedule := q.string_value (4)
				q.fetch
			end
			create row_count_checker
			create bindings.make (0)
			q.append_bindings_description (bindings)
			row_count_checker.check_actual_count_within_range (n, q.text, bindings, expected_count.Exactly_one)
			q.release
		end

	order_date: DB_DATE
			-- Date to use for order creation

	test_vehicle: M_TEST_VEHICLES_ROW
			-- Test vehicle for die

	cust_acct_no: INTEGER
			-- Number of cust acct to use for creating order
			-- and for funding

	m_funding_alloc_no: INTEGER
			-- Number of funding allocation for test vehicle

	m_funding_auth_no: INTEGER
			-- Number of funding authorization for test vehicle

	distribution_rank: INTEGER
			-- Distribution rank selected for test vehicle

	pricing_schedule: STRING
			-- Pricing schedule selected for test vehicle

	m_order_no: INTEGER
			-- Number of last work order

	m_fab_no: INTEGER
			-- Number of last fab instance

end
