
indexing
	
	description: "A run closing step operation that creates a %
		%fab instance and an empty work order (with no line items %
		%for each test vehicle on run"
	author: "David Hollenberg"
	date: "March 27, 2007"

class CREATE_TEST_VEHICLE_FABS_OP
inherit
	CREATE_TEST_VEHICLE_DATA_OP

create
	make

feature -- Properties

	input_data_items: LIST [STRING]
			-- Data items required before step can be executed
		do
			Result := array_as_list (<<
				data_item.Assigned_die_ids
				>>)
		end

feature {NONE} -- Implementation

	do_step
			-- Execute of step
		do
			cache.uncache_dies
			fetch_test_vehicle_dies
			create_test_vehicle_fabs
			session.commit_actions.extend (agent publish_test_vehicles)
			call_completion_actions
		end

	undo_step
			-- Undo execution of step
		do
			cache.uncache_dies
			delete_test_vehicle_fabs
			call_completion_actions
		end

	fetch_test_vehicle_dies
			-- Fetch dies for test vehicles and put in
			-- `test_vehicle_dies'
		do
			test_vehicle_dies := cache.test_vehicle_die_rows
		end

	create_test_vehicle_fabs
			-- Create a fab instance and an empty work
			-- order for each test vehicle on for run
		local
			list: LIST [DIES_ROW]
			tv: DIES_ROW
		do
			order_date := connection.transaction_date
			list := test_vehicle_dies

			from
				list.start
			until
				list.after
			loop
				tv := list.item
				fetch_test_vehicle (tv)
				create_fab_instance (tv)
				create_test_vehicle_order (tv)
				list.forth
			end
			
		end

	create_test_vehicle_order (tv: DIES_ROW)
			-- Create a fab instance and an order for `tv'
		require
			test_vehicle_exists: tv /= Void
			is_test_vehicle: tv.is_test_vehicle
		do
			fetch_cust_acct_no (tv)
			fetch_funding_info (cust_acct_no)
			create_work_order (tv)
		end

	create_fab_instance (tv: DIES_ROW)
			-- Create a fab instance for `tv'.  Leave fab instance
			-- number in `m_fab_no'
		require
			test_vehicle_exists: tv /= Void
			is_test_vehicle: tv.is_test_vehicle
		local
			ifi: IC_FAB_INSTANCES_MROW
			sequence: SEQUENCE_NAMES
			fab_instance_type: IC_FAB_INSTANCE_TYPE_VALUES
			disposition: INTENDED_DISPOSITION_VALUES
			style: OVERHEAD_STYLE_VALUES
			l_style: STRING
			cid: CUST_IC_DESIGNS_ROW
		do
			m_fab_no := connection.sequence_next_value (sequence.M_fab_no)
			cid := tv.cust_ic_design
			tv.uncache_fab_instance
				-- Make sure next call to get fab instance
				-- won't return Void, since fab instance does
				-- not exist yet
			if not cid.is_null_label_box_size_x then
				l_style := style.Internal
			else
				l_style := style.External_
			end
			create ifi.make_null (connection)
			ifi.set_m_fab_no (m_fab_no)
			ifi.set_design_no (tv.test_vehicle_design_no)
			ifi.set_ic_fab_instance_type (fab_instance_type.Test_vehicle)
			ifi.set_pending_for_run (True)
			ifi.set_run_id (session.run_id)
			ifi.set_die_id (tv.die_id)
			ifi.set_project_on_die (1)
			ifi.set_is_reworkable (False)
			ifi.set_fill_allowed (True)
			ifi.set_overhead_style (l_style)
			ifi.set_run_type_requested (cache.run_type)
			ifi.set_date_fab_requested (order_date)
			if test_vehicle.is_null_design_wafer_rotation_reqd then
				ifi.set_null_design_wafer_rotation_reqd
			else
				ifi.set_design_wafer_rotation_reqd (test_vehicle.design_wafer_rotation_reqd)
			end
			ifi.set_design_die_rotation (0)
			ifi.set_intended_disposition (disposition.Research)
		end

	create_work_order (tv: DIES_ROW)
			-- Create a work order for `tv'.  Leave order number
			-- in `m_order_no'
		require
			test_vehicle_exists: tv /= Void
			is_test_vehicle: tv.is_test_vehicle
		local
			sequence: SEQUENCE_NAMES
			wo: CUST_WOS_MROW
		do
			m_order_no := connection.sequence_next_value (sequence.M_order_no)
			create wo.make_null (connection)
			wo.set_m_order_no (m_order_no)
			wo.set_cust_acct_no (cust_acct_no)
			wo.set_m_funding_alloc_no (m_funding_alloc_no)
			wo.set_pricing_schedule (pricing_schedule)
			wo.set_discount_fraction (0)
			wo.set_date_for_pricing (order_date)
			wo.set_created_by ("RC")
			wo.set_modified_by (connection.user_name)
			wo.set_cust_wo_comment ("Test vehicles for run " + session.run_id)
			wo.set_date_cust_wo (order_date)
			wo.set_m_fab_no (m_fab_no)
			wo.set_has_eccn (False)
			wo.set_has_bxa711 (False)
		end

	delete_test_vehicle_fabs
			-- Delete all fab instances and orders for test
			-- vehicles for run.  Orders should not have any
			-- line items at this point
		local
			work_order_row: CUST_WOS_MROW
			fab_instance_row: IC_FAB_INSTANCES_MROW
			list: LIST [DB_DELETABLE_ROW]
		do
			-- Delete work orders

			create work_order_row.make_template
			list := work_order_row.locked_test_vehicle_work_orders_for_run (session.run_id, connection, busy_action.return, expected_count.at_least_zero)
			list.do_all (agent {DB_DELETABLE_ROW}.delete)
		
			-- Delete fab instances

			create fab_instance_row.make_template
			list := fab_instance_row.locked_test_vehicle_fab_instances_for_run (session.run_id, connection, busy_action.return, expected_count.at_least_zero)
			list.do_all (agent {DB_DELETABLE_ROW}.delete)
		
		end
	
	publish_test_vehicles
			-- Publish a "final test vehicles determined" event
		local
			e: TEST_VEHICLES_EVENT
		do
			create e
			e.set_subject ("final test vehicles")
			e.set_message ("Final test vehicles for run " + session.run_id + "%N%N" + final_test_vehicles_text)
			e.publish
		end

	final_test_vehicles_text: STRING is
			-- Text for body of message reporting final test
			-- vehicles on run
		local
			list: LIST [DIES_ROW]
			l_die: DIES_ROW
			hdr: BOOLEAN
		do
			create Result.make (500)
			list := test_vehicle_dies
			from
				list.start
			until
				list.after
			loop
				l_die := list.item
				if not hdr then
					Result.append (test_vehicle_header)
					hdr := True
				end
				Result.append (test_vehicle_description (l_die))
				list.forth
			end
			if Result.is_empty then
				Result := "    (None)%N"
			end
		end

	test_vehicle_header: STRING is
			-- Header lines for test vehicle notification
			-- message
		do
			create Result.make (200)
			Result.append ("    Design  Design Name                     Rotation  Project Type%N")
			Result.append ("    ------  ------------------------------  --------  ------------%N%N")
		end

	test_vehicle_description (a_die: DIES_ROW): STRING is
			-- Description of `a_die' for test vehicle
			-- notification message
		local
			cid: CUST_IC_DESIGNS_ROW
		do
			cid := a_die.cust_ic_design
			create Result.make (100)
			Result.append ("    ")
			Result.append (su.left_padded (cid.design_no.out, 6))
			Result.append ("  ")
			Result.append (su.right_padded (cid.design_name, 30))
			Result.append ("  ")
			Result.append (su.left_padded (a_die.die_wafer_rotation.out, 8))
			Result.append ("  ")
			Result.append ("TEST VEHICLE")
			Result.extend ('%N')
		end

	test_vehicle_dies: LIST [DIES_ROW]
			-- Dies for test vehicles

end
