indexing
	description:
		"Augmented Customer Accounts DB Row."
	author: "Barden E. Smith"
	date:	"March 1, 2005"

class CUST_ACCTS_ROW

inherit
	CUST_ACCTS_RAW
	ROW_COUNT_CHECK_ACCESS
	DB_QUERY_OPTIONS
	export {NONE} all end;
	MOSIS_CACT_VALUES
	export {NONE} all end;
	STRING_UTILITIES_ACCESS
	export {NONE} all end;

create

	make_template, make_null, make_by_key

feature -- status

	is_educational: BOOLEAN is
		-- is this account an educational account?
	do  Result := equal(acct_sponsor_id, sponsor_is_nsf) or
			equal(acct_sponsor_id, sponsor_is_edu) or
			equal(acct_sponsor_id, sponsor_is_mep)
	end;

	is_NSA_sponsored: BOOLEAN is
		-- is this account NSA-sponsored?
	do  Result := equal(acct_sponsor_id, sponsor_is_nsa)
	end;

	is_commercial: BOOLEAN is
		-- does he pay his own way? (not sponsored)
	do  Result := equal(acct_type, accttyp_is_com)
	end;

	is_nda_acct: BOOLEAN is
		-- did he execute just an NDA
	do  Result := equal(acct_type, accttyp_is_nda)
	end;

	is_research: BOOLEAN is
		-- is this a research account?
	do  Result := equal(acct_type, accttyp_is_res)
	end;

	is_class_acct: BOOLEAN is
		-- is this a class account?
	do  Result := equal(acct_type, accttyp_is_class) or
			equal(acct_type, accttyp_is_aclas)
	end;

	is_overhead: BOOLEAN is
		-- is this an overhead account
	do  Result := equal(acct_sponsor_id, sponsor_is_oh)
	end;

	contact_no: INTEGER is
		-- the contact no of the admin_liaison
	do  Result := a_contact_of_role(cr_is_admin_liaison)
	end;

	ITAR_admin_contact: INTEGER is
		-- the contact no of the ITAR liaison
		-- (0 if there isn't one)
	do  Result := a_contact_of_role(cr_is_ITAR_liaison)
	end;

	is_expired: BOOLEAN is
		-- is account currently expired (as of this transaction_date)
	do  Result := (not is_null_date_acct_expires) and then
			date_acct_expires <
			   connection.transaction_date
	end;

	expiration_date: STRING is
		-- printable expiration date
	require not is_null_date_acct_expires
	do  Result := date_acct_expires.to_date_string
        end;

	watermarking_required: BOOLEAN is
		-- do we need to watermark documents 
		--  that we deliver to this account?
	do  if not is_null_watermark_documents
	     then Result := watermark_documents
	     else Result := my_acct_type.watermark_documents
	    end
	end;

	inda_required: BOOLEAN is
		-- do we need a valid iNDA?
	do  Result := my_acct_type.inda_required
	end;

	a_contact_of_role(role: STRING): INTEGER is
		-- a contact_no for this contact_role
		--  (arbitrary selection if more than one),
		-- or 0 if there are none
	local b: DB_BIND_SI;
	do  create b.make(role, cust_acct_no);
	    Result := connection.safe_selected_integer(
				"contact_no",
				"cust_acct_contacts",
				"cust_acct_no = :2 and contact_role = :1",
				b)
	end;

	always_uses_po: BOOLEAN is
		-- are all account allocations PO-based?
	do  Result := is_commercial
	end;

	sometimes_uses_po: BOOLEAN is
		-- are some (but not necessarily all) allocations PO-based?
	do  Result := is_educational and is_research
	end;

	are_dollars_reportable: BOOLEAN is
		-- do we tell him about $$$ issues in status?
	do  Result := always_uses_po or sometimes_uses_po
	end;

feature	-- legal agreements stuff

	commercial_customer_agr: CUST_ACCT_LEGAL_AGRS_ROW is
		-- agreement or Void (if none)
	do  Result := cust_acct_agr(la_type_is_cst)
	end;

	nda_customer_agr: CUST_ACCT_LEGAL_AGRS_ROW is
		-- agreement or Void (if none)
	do  Result := cust_acct_agr(la_type_is_nda)
	end;

	gov_customer_agr: CUST_ACCT_LEGAL_AGRS_ROW is
		-- agreement or Void (if none)
	do  Result := cust_acct_agr(la_type_is_gov)
	end;

	invnda_customer_agr: CUST_ACCT_LEGAL_AGRS_ROW is
		-- agreement or Void (if none)
	do  Result := cust_acct_agr(la_type_is_invnda)
	end;

	doc_access_enabling_customer_agreements: 
			LIST[CUST_ACCT_LEGAL_AGRS_ROW] is
		-- all those agreements that grant doc access
	local r: CUST_ACCT_LEGAL_AGRS_ROW; b: DB_BIND_I;
	do  create r.make_template;
	    create b.make(cust_acct_no);
	    Result := r.all_rows_filtered
			(connection, enabling_cust_acct_agrs_filter, b);
	end;

	a_customer_agr: CUST_ACCT_LEGAL_AGRS_ROW is
		-- commercial or gov or nda will do
	do  Result := commercial_customer_agr;
	    if Result = Void
	     then Result := gov_customer_agr
	    end;
	    if Result = Void
	     then Result := nda_customer_agr
	    end
	end;
	
feature {NONE} -- legal agreements implementation

	cust_acct_agr(typ: STRING): CUST_ACCT_LEGAL_AGRS_ROW is
		-- account's legal agr of given legal_agr_type
	local r: CUST_ACCT_LEGAL_AGRS_ROW;
		agrs: LIST[CUST_ACCT_LEGAL_AGRS_ROW];
		b: DB_BIND_IS;
	do  create r.make_template;
	    create b.make(cust_acct_no, typ);
	    agrs := r.all_rows_filtered
			(connection, cust_acct_agr_filter, b);
	    if not agrs.is_empty
	     then Result := agrs.first
	    end;
	end;

	cust_acct_agr_filter: STRING is
	"cust_acct_no = :1 and la.legal_agr_type = :2%
	% and nvl(date_expires,sysdate) >= sysdate"

	all_cust_acct_agrs_filter: STRING is
	"cust_acct_no = :1%
	% and nvl(date_expires,sysdate) >= sysdate"

	enabling_cust_acct_agrs_filter: STRING is
	"cust_acct_no = :1 and document_server_access = %'Y%'%
	% and nvl(date_expires,sysdate) >= sysdate"

feature	-- account items of interest to the document server 

	is_in_agreement: BOOLEAN is
		-- is a valid legal agreement (in place) or (not required)
	local agrs: LIST[CUST_ACCT_LEGAL_AGRS_ROW];
		agr: CUST_ACCT_LEGAL_AGRS_ROW
	do  agrs := doc_access_enabling_customer_agreements;
	    from  agrs.start;
		  Result := is_overhead
	    until agrs.after or Result
	    loop  agr := agrs.item;
		  Result := agr.legal_agr_complete;
		  agrs.forth
	    end
	end;

feature -- account items of interest to the standard quote server

	is_eligible_for_quotes: BOOLEAN is
		-- is this account eligible to receive standard quotes?
	do  Result :=	(not is_expired) and
			is_null_date_acct_delinquent and
			is_null_date_acct_on_hold and
			terms_payment_days > 0
	end;

feature	-- password access
		-- Void  == no password row in cust_acct_passwords
		-- Empty == null password in the entry

	basic_password: STRING is
		-- the BASIC password (for project creation)
	local pwd: CUST_ACCT_PASSWORDS_ROW
	do  pwd := password_of_type(pwt_is_basic);
	    if pwd /= Void
	     then Result := pwd.password
	    end
	end;

	accounting_password: STRING is
		-- the accounting password (for financial report access)
	local pwd: CUST_ACCT_PASSWORDS_ROW
	do  pwd := password_of_type(pwt_is_accounting);
	    if pwd /= Void
	     then Result := pwd.password
	    end
	end;

	document_password: STRING is
		-- the documentation password (for doc server access)
	local pwd: CUST_ACCT_PASSWORDS_ROW
	do  pwd := password_of_type(pwt_is_documentation);
	    if pwd = Void
	     then Result := basic_password
	     else Result := pwd.password
	    end
	end;

	document_password_expiry: DB_DATE is
		-- null if no expiry
	local pwd: CUST_ACCT_PASSWORDS_ROW
	do  pwd := password_of_type(pwt_is_documentation);
	    if pwd = Void
	     then pwd := password_of_type(pwt_is_basic)
	    end;
	    if pwd /= Void
	     then Result := pwd.date_password_expires
	    end
	end;

feature {NONE} -- password implementation

	my_passwords: LIST[CUST_ACCT_PASSWORDS_ROW] is
	local b: DB_BIND_I; r: CUST_ACCT_PASSWORDS_ROW;
	do  if cached_passwords = Void
	     then create r.make_template;
		  create b.make(cust_acct_no);
		  cached_passwords := r.all_rows_filtered
				(connection, "cust_acct_no = :1", b);
	    end
	    Result := cached_passwords
	end;

	password_of_type(ptyp: STRING): CUST_ACCT_PASSWORDS_ROW is
	local pswds: like my_passwords
	do  pswds := my_passwords;
	    from  pswds.start
	    until pswds.after or Result /= Void
	    loop  if equal(pswds.item.password_type, ptyp)
		   then Result := pswds.item
		  end
		  pswds.forth
	    end
	end;

	cached_passwords: LIST[CUST_ACCT_PASSWORDS_ROW]

feature -- Collections

	rows_for_ic_packaging (packaging_no: INTEGER conn: DB_CONNECTION range: ITEM_COUNT_RANGE): LIST [like Current] is
			-- All customer accounts for an IC Packaging
		require
			connection_exists: conn /= Void
			connection_open: conn.is_open
			range_exists: range /= Void
		local
			b: DB_BIND_I
		do
			create b.make (packaging_no)
			Result := all_rows_filtered (conn, "cust_acct_no in (select cwo.cust_acct_no from cust_wos cwo, cust_wo_line_items cwli where cwli.m_order_no=cwo.m_order_no and cwli.ic_packaging_no = :1)", b)
			check_row_count (Result, range)
		end

	cust_ic_rows_for_production (prod: STRING conn: DB_CONNECTION range: ITEM_COUNT_RANGE): LIST [like Current] is
			-- All customer accounts for IC orders for customer
			-- projects on production `prod', ordered by 
			-- account number
		require
			prod_not_void: prod /= Void
			connection_exists: conn /= Void
			connection_open: conn.is_open
			range_exists: range /= Void
		local
			b: DB_BIND_SS
			type: IC_FAB_INSTANCE_TYPE_VALUES
		do
			create b.make (prod, type.Customer_fab)
			Result := all_rows_filtered_ordered (conn, "cust_acct_no in (select cwo.cust_acct_no from cust_wos cwo, ic_fab_instances ifi where cwo.m_fab_no = ifi.m_fab_no and cwo.production_id = :1 and ifi.ic_fab_instance_type = :2)", b, "cust_acct_no")
			check_row_count (Result, range)
		end

	rows_quoted_part_number (part_no: INTEGER conn: DB_CONNECTION range: ITEM_COUNT_RANGE): LIST [like Current] is
			-- All customer accounts for which there is an
			-- unexpired quote with a line item for `part_no'
			-- ordered by account number
		require
			connection_exists: conn /= Void
			connection_open: conn.is_open
			range_exists: range /= Void
		local
			b: DB_BIND_I
		do
			create b.make (part_no)
			Result := all_rows_filtered_ordered (conn, "cust_acct_no in (select distinct cust_acct_no from cust_quotes cq, cust_quote_line_items cqli where cq.m_cust_quote_no = cqli.m_cust_quote_no and nvl(cq.date_expires, sysdate) >= trunc(sysdate) and cqli.m_part_no = :1)", b, "cust_acct_no")
			check_row_count (Result, range)
		end

feature -- Added attributes

	display_name: STRING is
		require
			have_connection: connection /= Void
		local
			b: DB_BIND_I
		do
			if cached_display_name = Void then
				create b.make (cust_acct_no)
				cached_display_name := connection.safe_selected_string (
					"m_lookup.cust_acct_display_name (:1)",
					"dual",
					Void,
					b)
			end
			Result := cached_display_name
		end

	organization: like anchored_organization is
			-- Organization for `Current'
		do  
			if not is_cached_organization then 
	     			create cached_organization.make_by_key (connection, old_row, org_no)
	    		end
	    		Result := cached_organization
		ensure
			result_not_void: Result /= Void
		end;

	uncache_organization is
			-- Uncache organization for `Current'
		do  
			cached_organization := Void
		ensure
			organization_not_cached: not is_cached_organization
		end;

	is_cached_organization: BOOLEAN is
			-- Is organization cached?
		do  
			Result := cached_organization /= Void and then cached_organization.org_no = org_no
		end;

	is_exempt_from_eccn_requirements: BOOLEAN
			-- Are work orders on this account Is account exempt 
			-- from ECCN requirements?
		local
			sponsor: ACCT_SPONSOR_VALUES
		do  
			Result := not is_null_acct_sponsor_id and then acct_sponsor_id.is_equal (sponsor.Oh)
		end;

feature {NONE} -- Implementation

	cached_display_name: STRING

	cached_organization: like organization

	anchored_organization: ORGANIZATIONS_ROW

feature -- other account items of limited interest

	liaison_name: STRING is
		-- name of the contact for this account
	local b: DB_BIND_I;
	do  create b.make(contact_no)
	    Result := connection.selected_string(
	        "first_name||' '||last_name",
		"contacts", "contact_no = :1", b)
	end;

	liaison_net_address: STRING is
		-- net_address for the contact for this account
	local b: DB_BIND_I;
	do  create b.make(contact_no)
	    Result := connection.safe_selected_string(
	        "net_address", "contacts", "contact_no = :1", b)
	end;

	org_full_name: STRING is
		-- full name of the organization
	do  Result := organization.full_name
	end;

	sales_rep_net_address: STRING is
		-- net_address for the sales rep for this account
	local b: DB_BIND_I;
	      reps: LIST[STRING];
	do      create b.make(cust_acct_no);
		-- DON'T FORGET to fill in this code when we have the details
		reps := connection.selected_strings(
		     "c.net_address",
		     "contacts c, cust_acct#sales_reps casr, sales_reps sr",
		     "c.contact_no = sr.contact_no and %
		      %casr.cust_acct_no = :1 and %
		      %sr.sales_rep_no = casr.sales_rep_no and %
		      %trunc(casr.date_effective) <= trunc(sysdate) and %
		      %nvl(casr.date_expires,trunc(sysdate)) >= trunc(sysdate)",
		     Void,
		     True,
		     b)
		Result := su.joined_by(reps, su.comma_space);
	end;	     

feature {NONE} -- acct_type or acct_sub_type caching

	cached_acct_type: ESSENCE_OF_ACCT_TYPE

	my_acct_type: ESSENCE_OF_ACCT_TYPE is
		-- either my acct_sub_type or my acct_type
	do  if cached_acct_type /= Void
	     then -- already have it
	    elseif is_null_acct_sub_type
	     then create {ACCT_TYPES_ROW} cached_acct_type.make_by_key
				(connection, old_row, acct_type)
	     else create {ACCT_SUB_TYPES_ROW} cached_acct_type.make_by_key
				(connection, old_row,
				 acct_type, acct_sub_type)
	    end;
	    Result := cached_acct_type
	end;

end -- class CUST_ACCTS_ROW
