class CUST_IC_DESIGN_FILES_MROW

inherit
	CUST_IC_DESIGN_FILES_MRAW
		redefine
			delete
		end;

	CUST_IC_DESIGN_FILES_ROW
		undefine
			initialize_new,
			initialize_new_by_key
		redefine
			anchored_layer
		end;

	MOSIS_PROD_VALUES
		export {NONE} all
		end;

	MOSIS_DOCS_VALUES
		export {NONE} all
		end;

	FILE_UTILITIES_ACCESS
		export {NONE} all
		end;

	THREE_CHECKSUM_FLAVORS
	rename	cif_checksum		as cif_checksum_1,
		cif_byte_count		as cif_checksum_2,
		is_null_cif_checksum	as is_null_cif_checksum_1,
		set_cif_checksum	as set_cif_checksum_1,
		set_cif_byte_count	as set_cif_checksum_2,
		set_null_cif_checksum	as set_null_cif_checksum_1,
		set_null_cif_byte_count	as set_null_cif_checksum_2
	end;

	DB_UTILITIES
		export {NONE} all
		end;

create

	make_template, make_null, make_by_key,
	make_revision_layout, make_cust_layout, make_final_layout

feature -- default deliverable

	set_default_deliverable is
		-- copy deliverable from my file_role
	do  set_deliverable(file_role_row.deliverable)
	end;

feature	-- file_no selection

	set_next_file_no is
		-- set file_no to next available value.
		-- to prevent conflict with other processes, all users
		-- MUST first lock the cid row for this design.
		-- NOTE: can only be used once per transaction!!
	local fno: INTEGER; b: DB_BIND_I;
	do  create b.make(design_no);
	    fno := connection.selected_integer(
			"nvl(max(file_no),0)", table_and_alias_names,
			"design_no = :1", b);
	    set_file_no(fno+1)
	end;

feature	-- filename_prefix construction

	set_standard_filename_prefix(sfx: STRING) is
		-- set filename_prefix to standard_filename_prefix(sfx)
	do  set_filename_prefix(standard_filename_prefix(sfx))
	end;

feature	-- delete associated file and mark it deleted

	delete_file is
		-- delete this file (and make undeliverable)
	do  if not is_null_filename_prefix then
		fu.delete_if_exists(full_filename)
		set_date_file_deleted(connection.transaction_date)
	    end;
	    set_deliverable(false)
	ensure not is_null_date_file_deleted
	end;

feature	-- Run archiving

	set_archived_mask_set (a_run_id: STRING a_mask_revision: CHARACTER)
			-- Set mask set on which `Current' was
			-- archived to the one indicated by
			-- `a_run_id' and `a_mask_revision'
		do
			set_archived_run_id (a_run_id)
			set_archived_mask_revision (a_mask_revision)
		end

feature	-- bonding diagram creation

	generate_customer_diagram(icp: IC_PACKAGINGS_ROW;
				  gs_prog: STRING) is
		-- generate a customer bonding diagram for icp and
		-- become the cidf row for that file
		-- sets: file_role, content_type_id, ic_packaging_no,
		--	 filename_prefix, and date_posted
		-- gs_prog is the name of the ghostscript program that
		--  we need to convert postscript to pdf
	require have_rev: not (is_null_design_no or is_null_revision_no)
	do  generate_bonding_diagram(icp, gs_prog, "C")
	end;

	generate_vendor_diagram(icp: IC_PACKAGINGS_ROW;
				gs_prog: STRING) is
		-- generate a vendor bonding diagram for icp and
		-- become the cidf row for that file
		-- sets: file_role, content_type_id, ic_packaging_no,
		--	 filename_prefix, and date_posted
		-- gs_prog is the name of the ghostscript program that
		--  we need to convert postscript to pdf
	require have_rev: not (is_null_design_no or is_null_revision_no)
	do  generate_bonding_diagram(icp, gs_prog, "V")
	end;

feature {NONE} -- bonding diagram creation

	generate_bonding_diagram(icp: IC_PACKAGINGS_ROW;
				 gs_prog, typ: STRING) is
	local	ps_nam, pdf_nam: STRING;
		map_gen: BONDING_DIAGRAM_GENERATOR
		diagram: BONDING_DIAGRAM
		ps_conv: POSTSCRIPT_CONVERTER
	do  -- fill in my data
	    set_ic_packaging_no(icp.ic_packaging_no);
	    set_content_type_id(ctype_is_pdf);
	    set_standard_filename_prefix(su.string_of_3
			(icp.bonding_name, "-", typ));
	    set_date_file_posted(connection.transaction_date);
	    -- get a pair of file names
	    pdf_nam := full_filename;
	    ps_nam := pdf_nam.twin;
	    ps_nam.remove_tail(2);	-- replace "[p]df"
	    ps_nam.extend('s');		--  with "[p]s"
	    -- create a diagram
	    create map_gen.make(connection);
	    if equal(typ, "C")
	     then diagram := map_gen.customer_bonding_diagram(icp, Void)
		  set_file_role(fir_is_p_bnd_diagram);
	     else diagram := map_gen.vendor_bonding_diagram(icp, Void)
		  set_file_role(fir_is_f_bnd_diagram);
	    end;
	    set_default_deliverable;	-- (needs file_role)
	    diagram.set_drawing_rotatable(false)
	    -- turn it into a document and then a ps file
	    diagram.generate_document
	    diagram.write_postscript_file(ps_nam)
	    -- convert ps to pdf, and delete the ps file
	    create ps_conv.make (gs_prog);
	    ps_conv.convert_named_file_to_pdf(ps_nam, pdf_nam)
	    fu.delete_if_exists(ps_nam);
	end;

feature	-- cust_ic_design_file_layers management

	delete
		-- first delete the layers
	do  wipe_out_layers;
	    precursor;
	    reregister_last
	end;

        wipe_out_layers
                -- go to a state with no known layers
        do  if have_any_layers then
                -- (force fetch if cache was Void)
                delete_and_wipe_out(cached_layers)
                -- remove from db (at next post) and from the list
                -- (cached list is of MROWs, which are deletable)
            end
        end;

        add_layer(lyr_name: STRING; dens: DOUBLE; is_null_dens: BOOLEAN)
                -- add a new layer to layers
        require have_name: su.not_empty(lyr_name);
                new_name: not is_layer_present(lyr_name);
                valid_dens: is_null_dens or (0 <= dens and dens <= 1.0)
        local lyr: CUST_IC_DESIGN_FILE_LAYERS_MROW
        do  create lyr.make_null(connection);
            lyr.set_design_no(design_no);
	    lyr.set_file_no(file_no);
            lyr.set_layer_name(lyr_name);
            if is_null_dens then
                lyr.set_null_density
            else
                lyr.set_density(dens)
            end
            my_layers.extend(lyr)
        end;

feature {NONE}

	anchored_layer: CUST_IC_DESIGN_FILE_LAYERS_MROW
	do
	end;

end -- CUST_IC_DESIGN_FILES_MROW
