
indexing

	author: "Generated"
	date: "Fri Feb 27 14:00:36 PST 2004"

class CUST_IC_DESIGNS_ROW
inherit
	CUST_IC_DESIGNS_RAW

	DB_QUERY_OPTIONS
	    export
		{NONE} all
	    end;

	ROW_COUNT_CHECK_ACCESS
		export
			{NONE} all
		end

	ITEM_COUNT_RANGES_ACCESS
		export
			{NONE} all
		end

	CONDITION_UTILITIES
	    export
		{NONE} all
	    end;

	UNIX_OS_ACCESS
	    export
		{NONE} all
	    end;

	STRING_UTILITIES_ACCESS
	    export
		{NONE} all
	    end;

	OS_ACCESS
	    export
		{NONE} all
	    end;

	MOSIS_PROD_VALUES
	    export
		{NONE} all
	    end;

	ACTION_ITEM
		rename
			do_it as repair_layout_symbolic_links,
			descr as repair_links_descr,
			done  as repair_links_done
		end;

create
	make_template, make_null, make_by_key

feature {NONE} -- the two distinguished cust_ic_design_files

	cust_layout: like anchored_cidf_row
	do  if cached_cust_layout = Void or else
	       cached_cust_layout.file_no /= cust_file_no then
		create cached_cust_layout.make_by_key
			(connection, old_row, design_no, cust_file_no)
	    end;
	    Result := cached_cust_layout
	end;

	final_layout: like anchored_cidf_row
	do  if cached_final_layout = Void or else
	       cached_final_layout.file_no /= final_file_no then
		if final_file_no = cust_file_no then
			-- share the single cidf_row
			cached_final_layout := cust_layout
		else
			create cached_final_layout.make_by_key
			(connection, old_row, design_no, final_file_no)
		end
	    end;
	    Result := cached_final_layout
	end;

feature	-- selection of the "layout of interest" (cust or final)

	is_using_final_layout: BOOLEAN
		-- is the layout_of_interest the final_layout or the cust_layout?

	is_using_cust_layout: BOOLEAN
		-- is layout_of_interest = cust_layout?
		-- (even if is_using_final_layout)
	do  Result := layout_of_interest = cust_layout
	end;

	layout_of_interest: like anchored_cidf_row
		-- the one or the other
	do  if is_using_final_layout then
		Result := final_layout
	    else
		Result := cust_layout
	    end
	end;

	use_cust_layout
		-- make the cust_layout be the layout_of_interest
	do  is_using_final_layout := false
	end;

	use_final_layout
		-- make the final_layout be the layout_of_interest
	do  is_using_final_layout := true
	end;

feature	-- various attributes of the layout_of_interest 
	--   (things that used to be in the cid itself)

	design_techn: DESIGN_TECHNS_ROW
		-- this is a db row, not a string
	do  Result := layout_of_interest.design_techns_row
	end;

	design_techn_id: STRING
		-- this is a string, not a db row!!
	do  Result := design_techn.design_techn_id
	end;

	geometry_format: STRING
	do  Result := layout_of_interest.content_type_id
	end;

	is_gds: BOOLEAN
	local ctv: CONTENT_TYPE_VALUES
	do  Result := equal(geometry_format, ctv.gds)
	end;

	is_cif: BOOLEAN
	local ctv: CONTENT_TYPE_VALUES
	do  Result := equal(geometry_format, ctv.cif)
	end;

	source_filename: STRING
	do  Result := layout_of_interest.filename_prefix
	end;
	
	is_null_source_filename: BOOLEAN
	do  Result := layout_of_interest.is_null_filename_prefix
	end;

	layout_file_exists: BOOLEAN
	do  Result := layout_of_interest.exists
	end;

feature	-- full source directory filename
	
	full_source_directory_filename(fnam: STRING): STRING is
		-- fully qualified filename for fnam
		--  in our source_directory as a NEW string
	do  Result := os.full_file_name(source_directory, fnam)
	end

feature -- status

	design_center: POINT is
			-- center as a point
	do  create Result.make_xy(center_x, center_y)
	end;

	design_size: POINT is
			-- size as a point
	do  create Result.make_xy(design_size_x, design_size_y)
	end;

	layout_area: DOUBLE
	    		-- Area of layout
	    	do
			Result := design_size_x * design_size_y
		end

	design_bounding_box: RECTANGLE 
			-- bounding box for the layout
	do  create Result.make_by_size_and_center
				(design_size_x, design_size_y,
				 center_x, center_y)
	end;

	bonded_pad_count: INTEGER
			-- number of bonded cust_ic_design_pads
	do  Result := count_cid_pads("design_no = :1 and bonded = %'Y%'")
	end;

	unbonded_pad_count: INTEGER
			-- number of unbonded cust_ic_design_pads
	do  Result := count_cid_pads("design_no = :1 and bonded = %'N%'")
	end;

	total_pad_count: INTEGER
			-- number of cust_ic_design_pads
	do  Result := count_cid_pads(by_design_no)
	end;

	has_multiple_fab_instances: BOOLEAN
			-- Are there multiple fab instances for `design_no'?
	local	b: DB_BIND_I
	do
		create b.make (design_no);
		Result := any_connection.selected_count
				("ic_fab_instances", by_design_no, b) > 1
	end;

	is_final_layout_derived: BOOLEAN
		-- Is the final_layout different from the cust_layout?
	do  Result := final_file_no /= cust_file_no
	end;

	is_frozen: BOOLEAN
		-- is the layout frozen (cannot be changed by customer)?
	do  Result := design_frozen or design_cancelled or
			(not is_null_date_layout_locked) or
			is_final_layout_derived
	end;

feature -- Associated rows lookup

	admin_liaison_contact: CONTACTS_ROW is
			-- Administrative liaison contact (for `cust_acct_no)
		do
			if cached_admin_liaison_contact = Void then
				fetch_admin_liaison_contact
			end
			Result := cached_admin_liaison_contact
		ensure
			result_not_void: Result /= Void
		end

	uncache_admin_liaison_contact is
			-- Remove adminstrative liaison contact from cache, 
			-- if present
		do
			cached_admin_liaison_contact := Void
		ensure
			admin_liaison_contact_not_cached: not is_cached_admin_liaison_contact
		end

	is_cached_admin_liaison_contact: BOOLEAN is
			-- Is `admin_liaison_contact' currently cached?
		do
			Result := cached_admin_liaison_contact /= Void
		end

feature	{NONE}

	count_cid_pads(where: STRING): INTEGER is
	local b: DB_BIND_I
	do  create b.make(design_no);
	    Result := connection.selected_count
					("cust_ic_design_pads", where, b)
	end;

feature -- Queries

	rows_with_design_no (n: INTEGER conn: DB_CONNECTION range: ITEM_COUNT_RANGE): LIST [like Current] is
			-- All IC designs with design number `n'
		require
			design_no_nonegative: n >= 0
			connection_exists: conn /= Void
			connection_open: conn.is_open
			range_exists: range /= Void
		local
			b: DB_BIND_I
		do
			create b.make (n)
			Result := all_rows_filtered (conn, by_design_no, b)
			check_row_count (Result, range)
		ensure
			result_exists: Result /= Void
		end

feature -- stuff related to chamfer reconstuction in the overhead
	--  that we generate

	is_overhead_viable(a_style: STRING): BOOLEAN
		-- can we generate overhead a la a_style?
	require	valid_style: a_style /= Void;
	local styl: OVERHEAD_STYLE_VALUES
	do  Result := (is_chamfer_reconstruction_required(a_style)
			implies (not is_null_chamfer_edge))
			-- if we need to perform chamfer reconstruction,
			-- then we better have a discovered chamfer_edge
		and   (equal(a_style, styl.internal)
			implies (not is_null_label_box_size_x))
	end;

	is_chamfer_reconstruction_required(a_style: STRING): BOOLEAN
		-- given that our overhead is in a_style, will
		-- chamfer reconstruction be necessary?
	local ostyl: OVERHEAD_STYLE_VALUES
	do  Result := equal(a_style, ostyl.external_) and
		      chamfer_layer /= Void
	end;

	chamfer_layer: STRING
		-- layer for the 4 corner chamfer triangles
		-- (Void if chamfer reconstruction does not apply)
		-- this value is strictly a function of the design techn
	local b: DB_BIND_S;
	do  create b.make(design_techn_id);
	    Result := connection.safe_selected_string
			("layer_name", chamfer_layer_tables,
			 chamfer_layer_filter, b);
	    if Result.is_empty then Result := Void end
	end;

	is_null_chamfer_edge: BOOLEAN 
		-- do we lack a measured chamfer edge length?
	local inv: ITEM_NAME_VALUES
	do  Result := not have_measurement(inv.chamfer_edge)
	end;

	chamfer_edge: DOUBLE
		-- the measured chamfer edge length
		-- (or 0 if we don't have one and you didn't check)
	local inv: ITEM_NAME_VALUES
	do  Result := get_measurement_value(inv.chamfer_edge)
	end;

feature {NONE} -- details for overhead generation

	chamfer_layer_tables: STRING =
	"design_techn_id_layers dtil, layer_roles lr"

	chamfer_layer_filter: STRING =
	"dtil.layer_role = lr.layer_role and dtil.design_techn_id = :1%
	% and lr.is_corner_chamfer = %'Y%'"

feature	-- cust_ic_design_measurements
	-- (the routines to set or remove a measurement are in
	--   CUST_IC_DESIGNS_MROW)

	have_measurement(iname: STRING): BOOLEAN 
		-- do we have a measurement for item_name iname?
	do  Result := a_measurement(iname) /= Void
	end;

	get_measurement_value(iname: STRING): DOUBLE 
		-- get the value for a measurement
		--  (0 if not present and you didn't check first)
	local foo: like a_measurement
	do  foo := a_measurement(iname)
	    if foo /= Void then Result := foo.numeric_value end
	end;

feature {NONE} -- standard symbolic link maintenance

	repair_links_descr: STRING

	repair_layout_symbolic_links
		-- maintain the links <dno>_[cust/final].<format>
		-- this is my action_item do_it routine; it is queued
		-- for commit action when a new layout cidf is created
	local cust_fnam, final_fnam, dnos: STRING;
		cust_cif_lnam, cust_gds_lnam, final_cif_lnam, final_gds_lnam: STRING
	do  repair_links_descr := "begin symbolic link fixup"
	    -- the two target names
	    cust_fnam := cust_layout.standard_full_filename(Void);
	    final_fnam := final_layout.standard_full_filename(Void);
	    -- the 4 canonical symbolic link names
	    dnos := design_no.out
	    cust_cif_lnam := full_source_directory_filename(dnos + "_cust.cif")
	    cust_gds_lnam := full_source_directory_filename(dnos + "_cust.gds")
	    final_cif_lnam := full_source_directory_filename(dnos + "_final.cif")
	    final_gds_lnam := full_source_directory_filename(dnos + "_final.gds")
	    -- now repair each of the 4 names
	    repair_one_symbolic_link(cust_fnam, cust_cif_lnam)
	    repair_one_symbolic_link(cust_fnam, cust_gds_lnam)
	    repair_one_symbolic_link(final_fnam, final_cif_lnam)
	    repair_one_symbolic_link(final_fnam, final_gds_lnam)
	    -- and sign off
	    repair_links_done := true
	end;

	repair_one_symbolic_link(target, symlink: STRING)
		-- arguments are full paths
		-- if target's extension matches symlink's, link is created,
		--  otherwise, old link is just removed
	require have_target: target /= Void
		have_symlink: symlink /= Void
	local foo: LARGE_RAW_FILE
	do  create foo.make(symlink)
	    if foo.path_exists then
		if unix_os.is_symbolic_link(symlink) then
			repair_links_descr := "removing link " + symlink;
			unix_os.unlink(symlink)
		else
			handle_hard_error("file is NOT a symbolic link: " + symlink)
		end
	    end;
	    if equal(target.substring(target.count-3, target.count),
		     symlink.substring(symlink.count-3, symlink.count)) then
		repair_links_descr := su.string_of_4("linking ", symlink, " to ", target);
		unix_os.make_symbolic_link(target, symlink)
	    end;
	end;

feature {NONE} -- implementation stuff

	by_design_no: STRING = "design_no = :1";

	anchored_cidf_row: CUST_IC_DESIGN_FILES_ROW
	do
	end;

	cached_cust_layout,
	cached_final_layout: like anchored_cidf_row;

	a_measurement(iname: STRING): CUST_IC_DESIGN_MEASUREMENTS_MROW
		-- measurement with item_name iname or Void
		-- NOTE: clients never see or handle any
		--   CUST_IC_DESIGN_MEASUREMENTS_MROWs
	local r: CUST_IC_DESIGN_MEASUREMENTS_MROW; b: DB_BIND_I
	do  if cached_measurements = Void
	     then create r.make_template;
		  create b.make(design_no)
		  cached_measurements := 
			r.all_rows_filtered(connection, by_design_no, b)
	    end;
	    from  cached_measurements.start
	    until cached_measurements.after or Result /= Void
	    loop  if equal(cached_measurements.item.item_name, iname)
		   then Result := cached_measurements.item
		   else cached_measurements.forth
		  end;
	    end;
	ensure Result_is_item: 
		(Result = Void) or else (Result = cached_measurements.item)
	end;

	fetch_admin_liaison_contact is
		local
			row: CONTACTS_ROW
		do
			create row.make_template
			cached_admin_liaison_contact := row.admin_liaisons_for_account (cust_acct_no, connection, expected_count.Exactly_one).first
		end

	cached_admin_liaison_contact: CONTACTS_ROW

	cached_layers: LIST[DESIGN_NOXLAYER]

	cached_measurements: DYNAMIC_LIST[CUST_IC_DESIGN_MEASUREMENTS_MROW]

end -- CUST_IC_DESIGNS_ROW
