
indexing

	author: "Generated"
	date: "Fri Feb 27 10:32:55 PST 2004"

class CUST_WO_LINE_ITEMS_ROW
inherit
	CUST_WO_LINE_ITEMS_RAW
	ROW_COUNT_CHECK_ACCESS
	DB_QUERY_OPTIONS
		export {NONE} all end;

create
	make_template, make_null, make_by_key

feature	-- back link to my work order

	the_wo: CUST_WOS_ROW
			-- my parent work order

	set_the_wo(wo: like the_wo) is
			-- cant find our part_and_price without it
		do
			the_wo := wo
		end

feature	-- my part

	part: like cached_part is
		-- my part
	do  if cached_part = Void or else 
		not (cached_part.m_part_no = m_part_no)
	     then create cached_part.make_by_key
				(connection, old_row, m_part_no)
	    end
	    Result := cached_part
	end;

	select_part(master: LIST[like cached_part]) is
		-- pick out my part from this list
	require master /= Void
	do  from  master.start;
		  cached_part := Void
	    until master.after or cached_part /= Void
	    loop  if master.item.m_part_no = m_part_no
		   then cached_part := master.item
		  end;
		  master.forth
	    end
	end;

feature -- attributes about my part

	is_ic_part: BOOLEAN is
		-- is this line item for an IC part?
	do  Result := part.is_IC
	end;

	is_ic_prep_part: BOOLEAN is
		-- is this line item for an IC_prep part?
	do  Result := part.is_IC_prep
	end;

	is_ic_or_prep_part: BOOLEAN is
		-- is this line item for an IC part or an IC_prep part?
	do  Result := part.is_IC_or_prep
	end;

	is_package_part: BOOLEAN is
		-- is this line item for a package part?
	do  Result := part.is_PACKAGE
	end;

	is_fab_run_part: BOOLEAN is
		-- is this line item for a fab run part?
	do  Result := part.is_fab_run
	end;

	is_spare_package_part: BOOLEAN is
		-- is this line item for a spare package part?
	do  Result := part.is_SPARE_PACKAGE
	end;

	is_spare: BOOLEAN is
		-- is the part increment 'spare'?
	do  Result := part.is_spare
	end;

	number_of_parts: INTEGER is
		-- total number of parts in line item
	do  Result := part_quantity_ordered * part.quantity_in_part
	end

	is_shippable: BOOLEAN is
		-- is this a line item that is REALLY shipped?
	do  Result := part.is_shippable
	end;

	is_fe_reportable: BOOLEAN is
		-- is this a line item that the front end deals in?
	do  Result := part.is_fe_reportable
	end;

	part_type: STRING is
		-- the part_type of my part
	do  Result := part.part_type
	end;

	is_correct_tg_family(tgf: STRING): BOOLEAN is
		-- for ic(prep) parts, is tgf my part's techn_group_family?
		-- for non-ic(prep) parts, always true
	do  Result := is_IC_or_prep_part implies
			equal(tgf, part.techn_group_family)
	end;

feature	-- Packaging

	ic_packaging: like cached_ic_packaging is
			-- IC packaging for `Current'
		require
			ic_packaging_no_not_null: not is_null_ic_packaging_no
		do  
			if not is_cached_ic_packaging then 
	     			create cached_ic_packaging.make_by_key (connection, old_row, ic_packaging_no)
	    		end
	    		Result := cached_ic_packaging
		ensure
			result_not_void: Result /= Void
		end;

	uncache_ic_packaging is
			-- Uncache IC packaging for `Current'
		do  
			cached_ic_packaging := Void
		ensure
			packaging_not_cached: cached_ic_packaging = Void
		end;

	is_cached_ic_packaging: BOOLEAN is
			-- Is IC packaging cached?
		do  
			Result := cached_ic_packaging /= Void and then cached_ic_packaging.ic_packaging_no = ic_packaging_no
		end;

feature {NONE} --

	cached_part: M_PARTS_ROW
		-- (should always match m_part_no, but we do check in part)

	cached_ic_packaging: IC_PACKAGINGS_ROW
			-- Packaging for `Current' cached in memory

feature -- Properties

	needs_final_bonding_diagram: BOOLEAN is
			-- Does line item require preparation of a
			-- final bonding diagram?
		do
			Result := is_package_part and part.packaged and not is_null_ic_packaging_no and then (not ic_packaging.customer_providing_diagram or else ic_packaging.needs_final_diagram)
		end 

	needs_packaging_lot: BOOLEAN is
			-- Does line item need to be assigned to a packaging
			-- lot?  False if already assigned to one
		do
			Result := is_package_part and is_null_packaging_run_id
		end 

	best_guess_packaging_order_type: STRING is
			-- Order type of packaging lot to which line item
			-- should be assigned initially
		require
			is_package_part: is_package_part
		local
			type: PACKAGING_ORDER_TYPE_VALUES
		do
			if part.packaged and ic_packaging.my_package.leadframe then
				Result := type.Plastic
			else
				Result := type.Ceramic
			end
		end 

	packaging_description: STRING is
			-- Description of packaging line item suitable
			-- for use in error messages to operator when
			-- assigning "best guess" packaging lot
		local
			pkg_id: STRING
		do
			pkg_id := part.package_id
			if pkg_id.is_empty then
				pkg_id := "unpackaged"
			elseif not part.packaged then
				pkg_id := pkg_id + "*"
			end
			Result := the_wo.fab_instance.die_id + "   " + best_guess_packaging_order_type + "   " + pkg_id + "   " + the_wo.fab_instance.ordered_wafer_process
		ensure
			result_not_void: Result /= Void
		end 

feature -- Collections

	rows_for_ic_packaging (packaging_no: INTEGER prod: STRING conn: DB_CONNECTION range: ITEM_COUNT_RANGE): LIST [like Current] is
			-- All line items for an IC Packaging on production
			-- `prod'
		require
			connection_exists: conn /= Void
			connection_open: conn.is_open
			range_exists: range /= Void
		local
			b: DB_BIND_IS
			where: STRING
		do
			create b.make (packaging_no, prod)
			where := "ic_packaging_no = :1"
			if prod /= Void and then not prod.is_empty then
				where.append (" and :2 = (select production_id from cust_wos where m_order_no = cwli.m_order_no)")
			else
				where.append (" and :2 is null")
			end
			Result := all_rows_filtered (conn, where, b)
			check_row_count (Result, range)
		end

	rows_for_packaging_run_id (pkg_run_id: STRING conn: DB_CONNECTION range: ITEM_COUNT_RANGE): LIST [like Current] is
			-- All line items to be packaged on lot for 
			-- `pkg_run_id'
		require
			pkg_run_id_not_void:  pkg_run_id /= Void
			connection_not_void: conn /= Void
			connection_open: conn.is_open
			range_not_void: range /= Void
		local
			b: DB_BIND_S
		do
			create b.make (pkg_run_id)
			Result := all_rows_filtered (conn, "packaging_run_id = :1", b)
			check_row_count (Result, range)
		end

end
