
indexing
	
	description: "Target which indicates that packing is destined for a custom reticle"
	author: "David Hollenberg"
	date: "August 28, 2008"

deferred class CUSTOM_RETICLE_PACKING_TARGET
inherit
	PACKING_TARGET

feature -- Properties

	is_custom: BOOLEAN = True
			-- Is target a custom target?

	has_enough_data_to_build_interface: BOOLEAN
			-- Is there enough data from related
			-- field sets to build the interface for `Current'?
		do
			Result := not field_sets.is_empty or 
				(related_field_sets.there_exists (agent {FIELD_SETS_ROW}.has_fab_location) and
				 related_field_sets.there_exists (agent {FIELD_SETS_ROW}.has_wafer_type) and
				 related_field_sets.there_exists (agent {FIELD_SETS_ROW}.has_mask_size) and
				 related_field_sets.there_exists (agent {FIELD_SETS_ROW}.has_tweezer_margin) and
				 related_field_sets.there_exists (agent {FIELD_SETS_ROW}.has_primary_flat_location))
		end

	selected_field_set_name: STRING
			-- Name of currently selected field set
		do
			Result := field_set_field.text
		end

feature -- Modification

	create_text_fields
			-- Create text fields for `Current'
		local
			wtpt: like wafer_techn_packing_target
		do
			field_set_field := new_custom_field_set_field (Field_set_width)
			payload_size_x_field := new_writable_text_field ("", Size_width)
			payload_size_y_field := new_writable_text_field ("", Size_width)
			external_kerf_size_x_field := new_writable_text_field ("", Size_width)
			external_kerf_size_y_field := new_writable_text_field ("", Size_width)
			field_size_x_field := new_read_only_text_field ("", Size_width)
			field_size_y_field := new_read_only_text_field ("", Size_width)
			renaming_style_field := new_custom_renaming_style_field
			fab_location_field := new_custom_fab_location_field
			wafer_type_field := new_custom_wafer_type_field
			mask_size_field := new_custom_mask_size_field
			tweezer_margin_field := new_custom_tweezer_margin_field
			primary_flat_field := new_custom_flat_location_field
			wtpt := wafer_techn_packing_target
			if has_existing_field_set then
				set_text (payload_size_x_field, formatted_number (only_field_set.payload_size_x))
				set_text (payload_size_y_field, formatted_number (only_field_set.payload_size_y))
				set_text (external_kerf_size_x_field, formatted_number (only_field_set.external_kerf_x))
				set_text (external_kerf_size_y_field, formatted_number (only_field_set.external_kerf_y))
				set_text (field_size_x_field, formatted_number (only_field_set.field_size_x))
				set_text (field_size_y_field, formatted_number (only_field_set.field_size_y))
			elseif wtpt /= Void then 
				if not wtpt.is_null_default_external_kerf_x then
					set_text (external_kerf_size_x_field, formatted_number (wtpt.default_external_kerf_x))
				end
				if not wtpt.is_null_default_external_kerf_y then
					set_text (external_kerf_size_y_field, formatted_number (wtpt.default_external_kerf_y))
				end
			end
			add_change_actions
		end

feature -- Operations

	validate_input
			-- Validate user input.  Leave any validation
			-- errors in `validation_errors' and set `is_input_valid'
			-- to indicate whether input is valid
		do
			clear_validation_errors
			validate_payload_and_kerf_fields
			validate_field (renaming_style_field, Pattern_file_renaming_style_text)
			validate_field (fab_location_field, Fab_location_text)
			validate_field (wafer_type_field, Wafer_type_text)
			validate_field (mask_size_field, Mask_size_text)
			validate_field (primary_flat_field, Primary_flat_location_text)
			validate_numeric_field (tweezer_margin_field, Tweezer_margin_text)
		end

	update_database
			-- Do any needed database inserts or updates
			-- prior to setting field set to one user
			-- has selected
		local
			fs: FIELD_SETS_MROW
			ls: LITHO_SIZES_MROW
			fl: FIELD_LOCATIONS_MROW
			val, msize_id: STRING
			num_val: DOUBLE
		do
			ls := new_or_existing_litho_size
			
			val := field_data (mask_size_field)
			if not ls.mask_size_id.is_equal (val) then
				ls.set_mask_size_id (val)
			end
			msize_id := val
			
			val := field_data (wafer_type_field)
			if not ls.wafer_type_id.is_equal (val) then
				ls.set_wafer_type_id (val)
			end
			
			val := field_data (primary_flat_field)
			if not ls.primary_flat_location.is_equal (val) then
				ls.set_primary_flat_location (val)
			end
			
			num_val := grid.rounded_size (tweezer_margin_field.text.to_double)
			if ls.is_null_tweezer_margin or else ls.tweezer_margin /= num_val then
				ls.set_tweezer_margin (num_val)
			end
			
			num_val := grid.rounded_size (field_size_x_field.text.to_double)
			if ls.is_null_field_size_x or else ls.field_size_x /= num_val then
				ls.set_field_size_x (num_val)
			end
			num_val := grid.rounded_size (field_size_y_field.text.to_double)
			if ls.is_null_field_size_y or else ls.field_size_y /= num_val then
				ls.set_field_size_y (num_val)
			end
			
			fs := new_or_existing_field_set
			val := "Custom reticle for " + run_id
			if not fs.descr.is_equal (val) then
				fs.set_descr (val)
			end
			
			fl := new_or_existing_field_location (ls, msize_id)
		end

feature {NONE} -- Implementation

	Estimated_field_overhead: DOUBLE = 200.0
			-- Initial guess at amount of overhead added
			-- by field over payload size

end
