
indexing
	
	description: "Target which indicates that packing is destined for a custom tile"
	author: "David Hollenberg"
	date: "August 28, 2008"

class CUSTOM_TILE_PACKING_TARGET
inherit
	PACKING_TARGET
		redefine
			renaming_style_field,
			fab_location_field,
			standard_name
		end

create
	make

feature -- Properties

	is_custom: BOOLEAN = True
			-- Is target a custom target?

	has_enough_data_to_build_interface: BOOLEAN
			-- Is there enough data from related
			-- field sets to build the interface for `Current'?
		do
			Result := not field_sets.is_empty or 
				related_field_sets.there_exists (agent {FIELD_SETS_ROW}.has_fab_location)
		end

	selected_field_set_name: STRING
			-- Name of currently selected field set
		do
			Result := field_set_field.text
		end

	standard_name: STRING
			-- Standard name for field set and litho size
		local
			target: PACKING_TARGET_VALUES
		do
			Result := target.custom_tile_name (run_id)
		end

	renaming_style_field: EV_COMBO_BOX
			-- Field for selecting/displaying pattern file
			-- renaming style

	fab_location_field: EV_COMBO_BOX
			-- Field for selecting/displaying fab location

feature -- Modification

	create_text_fields
			-- Create text fields for `Current'
		do
			field_set_field := new_custom_field_set_field (Field_set_width)
			payload_size_x_field := new_writable_text_field ("", Size_width)
			payload_size_y_field := new_writable_text_field ("", Size_width)
			renaming_style_field := new_custom_renaming_style_field
			fab_location_field := new_custom_fab_location_field
			if has_existing_field_set then
				set_text (payload_size_x_field, formatted_number (only_field_set.payload_size_x))
				set_text (payload_size_y_field, formatted_number (only_field_set.payload_size_y))
			end
		end

feature -- Operations

	validate_input
			-- Validate user input.  Leave any validation
			-- errors in `validation_errors' and set `is_input_valid'
			-- to indicate whether input is valid
		do
			clear_validation_errors
			validate_numeric_field (payload_size_x_field, initial_text (Payload_size_x_text))
			validate_numeric_field (payload_size_y_field, initial_text (Payload_size_y_text))
			validate_field (renaming_style_field, Pattern_file_renaming_style_text)
			validate_field (fab_location_field, Fab_location_text)
		end

	update_database
			-- Do any needed database inserts or updates
			-- prior to setting field set to one user
			-- has selected
		local
			fs: FIELD_SETS_MROW
			ls: LITHO_SIZES_MROW
			fl: FIELD_LOCATIONS_MROW
			val, msize_id: STRING
		do
			ls := new_or_existing_litho_size
			fs := new_or_existing_field_set
			
			val := "Custom tile for " + run_id
			if not fs.descr.is_equal (val) then
				fs.set_descr (val)
			end
			
			msize_id := session.cache.virtual_mask_size_id
			fl := new_or_existing_field_location (ls, msize_id)
		end

end
