
indexing

	author: "Generated"
	date: "Thu Mar 11 15:52:47 PST 2004"

class CUT_DIE_SITES_ROW
inherit
	CUT_DIE_SITES_RAW
	ROW_COUNT_CHECK_ACCESS
	DB_QUERY_OPTIONS
		export {NONE} all
		end

create
	make_template, make_null, make_by_key

feature -- Properties

	rows_for_die (a_run_id, a_die_id: STRING conn: DB_CONNECTION range: ITEM_COUNT_RANGE): LIST [like Current] is
			-- All rows for `a_run_id' and `a_die_id'
		require
			a_run_id_exists: a_run_id /= Void
			a_die_id_exists: a_die_id /= Void
			connection_exists: conn /= Void
			connection_open: conn.is_open
			range_exists: range /= Void
		local
			b: DB_BIND_SS
		do
			create b.make (a_run_id, a_die_id)
			Result := all_rows_filtered (conn, die_filter, b)
			check_row_count (Result, range)
		ensure
			result_exists: Result /= Void
		end

	rows_for_run (a_run_id: STRING conn: DB_CONNECTION range: ITEM_COUNT_RANGE): LIST [like Current] is
			-- All rows for `a_run_id'
		require
			a_run_id_exists: a_run_id /= Void
			connection_exists: conn /= Void
			connection_open: conn.is_open
			range_exists: range /= Void
		local
			b: DB_BIND_S
		do
			create b.make (a_run_id)
			Result := all_rows_filtered (conn, "run_id = :1", b)
			check_row_count (Result, range)
		ensure
			result_exists: Result /= Void
		end

	rows_for_run_and_pattern (a_run_id: STRING pat_no: INTEGER conn: DB_CONNECTION range: ITEM_COUNT_RANGE): LIST [like Current] is
			-- All rows for `a_run_id' and `pat_no'
		require
			a_run_id_exists: a_run_id /= Void
			connection_exists: conn /= Void
			connection_open: conn.is_open
			range_exists: range /= Void
		local
			b: DB_BIND_SI
		do
			create b.make (a_run_id, pat_no)
			Result := all_rows_filtered (conn, "run_id = :1 and cut_pattern_no = :2", b)
			check_row_count (Result, range)
		ensure
			result_exists: Result /= Void
		end

	rows_for_ic_packaging (packaging_no: INTEGER conn: DB_CONNECTION range: ITEM_COUNT_RANGE): LIST [like Current] is
			-- All cut die sites for an IC Packaging
		require
			connection_exists: conn /= Void
			connection_open: conn.is_open
			range_exists: range /= Void
		local
			b: DB_BIND_I
		do
			create b.make (packaging_no)
			Result := all_rows_filtered (conn, ic_packaging_filter, b)
			check_row_count (Result, range)
		end

feature {NONE} -- Implementation

	die_filter: STRING is "(run_id, mask_site_no) in (%
		%select distinct dl.run_id, dl.mask_site_no %
		%from die_locations dl %
		%where dl.run_id = :1 and dl.die_id = :2)"

	ic_packaging_filter: STRING is "(run_id, cut_pattern_no, mask_site_no) in (%
		%select distinct rcp.run_id, rcp.cut_pattern_no, dl.mask_site_no %
		%from ic_packagings icp, ic_fab_instances ifi, die_locations dl, cust_wo_line_items cwli, run_cut_patterns rcp, run_packaging_lots rpl %
		%where rcp.run_id=rpl.run_id and rcp.packaging_lot_no=rpl.packaging_lot_no %
		%and rpl.run_id=cwli.packaging_run_id and rpl.packaging_lot_no=cwli.packaging_lot_no %
		%and cwli.ic_packaging_no = icp.ic_packaging_no %
		%and dl.run_id = ifi.run_id and dl.die_id = ifi.die_id %
		%and ifi.m_fab_no = icp.m_fab_no %
		%and icp.ic_packaging_no = :1)"

feature	-- cut die size error amounts

	reticle_size_x: DOUBLE is
		-- size of cut die in reticle coordinates
	do  if die.is_rotated_sideways then
		Result := die_size_cut_y
	    else
		Result := die_size_cut_x
	    end
	end;

	reticle_size_y: DOUBLE is
		-- size of cut die in reticle coordinates
	do  if die.is_rotated_sideways then
		Result := die_size_cut_x
	    else
		Result := die_size_cut_y
	    end
	end;

	is_at_left_edge: BOOLEAN is
		-- is this site at the left edge of the reticle?
	local
	do  Result := (location_x - reticle_size_x/2) <=
			-(litho_size.payload_size_x/2)
	end;

	is_at_right_edge: BOOLEAN is
		-- is this site at the right edge of the reticle?
	local
	do  Result := (location_x + reticle_size_x/2) >=
			(litho_size.payload_size_x/2)
	end;

	is_at_top_edge: BOOLEAN is
		-- is this site at the top edge of the reticle?
	local
	do  Result := (location_y + reticle_size_y/2) >=
			(litho_size.payload_size_y/2)
	end;

	is_at_bottom_edge: BOOLEAN is
		-- is this site at the bottom edge of the reticle?
	local
	do  Result := (location_y - reticle_size_y/2) <=
			-(litho_size.payload_size_y/2)
	end;

	die_oversizing_x: DOUBLE is
		-- approx upper limit on how much larger than
		--  die_size_cut_x the cut die might be
	do  if is_at_left_edge or is_at_right_edge then
		Result := litho_size.external_kerf_x
		Result := (Result - die_undersizing_x).max(0);
	    else
		Result := 0
	    end;
	ensure Result >= 0
	end;

	die_oversizing_y: DOUBLE is
		-- approx upper limit on how much larger than
		--  die_size_cut_y the cut die might be
	do  if is_at_top_edge or is_at_bottom_edge then
		Result := litho_size.external_kerf_y
		Result := (Result - die_undersizing_y).max(0);
	    else
		Result := 0
	    end;
	ensure Result >= 0
	end;

	die_undersizing_x: DOUBLE is
		-- approx upper limit on how much smaller than
		--  die_size_cut_x the cut die might be
	do  Result := 72
	ensure Result >= 0
	end;

	die_undersizing_y: DOUBLE is
		-- approx upper limit on how much smaller than
		--  die_size_cut_y the cut die might be
	do  Result := 72
	ensure Result >= 0
	end;

feature	-- access to dies_row and litho_sizes_row

	die_location: DIE_LOCATIONS_ROW is
		-- my die_location
	do  if cached_die_location = Void then
		create cached_die_location.make_by_key
				(connection, old_row, run_id, mask_site_no)
	    end
	    Result := cached_die_location
	end;

	die: DIES_ROW is
		-- my die
	local dloc: DIE_LOCATIONS_ROW
	do  if cached_die = Void then
		dloc := die_location;
		create cached_die.make_by_key
				(connection, old_row, dloc.run_id, dloc.die_id)
	    end
	    Result := cached_die
	end;

	litho_size: LITHO_SIZES_ROW is
		-- my litho_size
	local lsid: STRING; b: DB_BIND_S;
	do  if cached_litho_size = Void then
		create b.make(run_id)
		lsid := connection.selected_string
				("litho_size_id",
				 "field_sets fs, runs",
				 "fs.field_set_name = runs.field_set_name and runs.run_id = :1",
				 b);
		create cached_litho_size.make_by_key
				(connection, old_row, lsid)
	    end
	    Result := cached_litho_size
	end;

feature {NONE} -- caches for above

	cached_die_location: DIE_LOCATIONS_ROW

	cached_die: DIES_ROW

	cached_litho_size: LITHO_SIZES_ROW

end
