
indexing
	
	author: "David Hollenberg"
	date: "July 9, 2005"
	description: "A consumer of data that writes it to a file, with %
		%error checking on open and close that allows interactive %
		%program to present a nice error message and with logging."

class DATA_CONSUMER_CHECKED_FILE

inherit
	DATA_CONSUMER_ONLY
		redefine
			initialize, reset, terminate
		end
	UNIX_OS_ACCESS
		export
			{NONE} all
		end


create
	make

feature {NONE} -- Creation

	make (a_file: LARGE_FILE)
			-- Create `Current' with `file' of `a_file'
		require
			a_file_not_void: a_file /= Void
		do
			set_file (a_file)
			reset
		ensure
			file_set: file = a_file
		end

feature -- Operations

	initialize
			-- Initialize `Current'.  Must be called before
			-- any data is passed to it.
		local
			op: FILE_OPEN_WRITE_OP
		do
			precursor
			create op
			op.open (file)
		end

	consume (the_data: STRING)
			-- Consume some more data in `the_data'.
			-- Just write it to `file'
		do
			in_initial_state := False
			file.put_string (the_data)
		end

	consume_eof
			-- Consume an "end of file" indication.
		do
			in_initial_state := False
			close_file
		end

	reset
			-- Reset `Current' to its initial state so
			-- that it may be reused
		do
			precursor
			if file /= Void and then not file.is_closed then
				close_file
			end
			in_initial_state := True
		end

	terminate
			-- Terminate operation prematurely
		do
			if file /= Void and then not file.is_closed then
				close_file
			end
		end

feature -- Modification

	set_file (a_file: LARGE_FILE)
			-- Set `file' to `a_file'
		require
			a_file_not_void: a_file /= Void
		do
			file := a_file
		ensure
			file_set: file = a_file
		end

feature -- Properties

	file: LARGE_FILE
			-- File to which to write consumed data

	in_initial_state: BOOLEAN
			-- Is `Current' in its initial state?

	Default_name: STRING = "File"
			-- Default name of `Current'

	always_ok: BOOLEAN = True
			-- Is this consumer always OK? (Answer: yes)


feature {NONE} -- Implementation

	close_file
			-- Close `file'
		
		local
			op: FILE_CLOSE_OP
		do
			file.flush
			unix_os.flush_descriptor_to_disk (file.descriptor)
			create op
			op.close (file)
		end

end

