
indexing
	
	author: "David Hollenberg"
	date: "July 13, 1999"
	description: "A consumer of data that writes it to a file"

class DATA_CONSUMER_FILE

inherit
	DATA_CONSUMER_ONLY
		redefine
			initialize, reset, terminate
		end
	UNIX_OS_ACCESS
		export
			{NONE} all
		end


create
	make

feature {NONE} -- Creation

	make (s: STRING)
			-- Create `Current' with `file_name' of `s'
		require
			name_exists: s /= Void
		do
			set_file_name (s)
			reset
		ensure
			name_set: equal (file_name, s)
		end

feature -- Operations

	initialize
			-- Initialize `Current'.  Must be called before
			-- any data is passed to it.
		do
			precursor
			create file.make_open_write (file_name)
		end

	consume (the_data: STRING)
			-- Consume some more data in `the_data'.
			-- Just write it to `file'
		do
			in_initial_state := False
			file.put_string (the_data)
		end

	consume_eof
			-- Consume an "end of file" indication.
		do
			in_initial_state := False
			file.flush
			unix_os.flush_descriptor_to_disk (file.descriptor)
			file.close
		end

	reset
			-- Reset `Current' to its initial state so
			-- that it may be reused
		do
			precursor
			if file /= Void and then not file.is_closed then
				file.flush
				unix_os.flush_descriptor_to_disk (file.descriptor)
				file.close
			end
			in_initial_state := True
		end

	terminate
			-- Terminate operation prematurely
		do
			if file /= Void and then not file.is_closed then
				file.flush
				unix_os.flush_descriptor_to_disk (file.descriptor)
				file.close
			end
		end

feature -- Modification

	set_file_name (s: STRING)
			-- Set `file_name' to `s'
		require
			name_exists: s /= Void
		do
			file_name := s.twin
		ensure
			name_set: equal (file_name, s)
		end

feature -- Properties

	file: LARGE_RAW_FILE
			-- File to which to write consumed data

	file_name: STRING
			-- Name of file to which to write consumed data

	in_initial_state: BOOLEAN
			-- Is `Current' in its initial state?

	Default_name: STRING = "File"
			-- Default name of `Current'

	always_ok: BOOLEAN = True
			-- Is this consumer always OK? (Answer: yes)

end

