
indexing
	
	author: "Joel Goldberg (reworked by David Hollenberg)"
	date: "January 18, 2000"
	description: "Source of data which is empty (contains no data)"

class DATA_SOURCE_EMPTY

inherit
	DATA_SOURCE

feature -- Status

	after: BOOLEAN
			-- Is there no more data available in data source?

	Position: INTEGER_64 = 0
			-- Position of next character to read from source,
			-- relative to 0

	Count: INTEGER_64 = 0
			-- Total number of characters in data source

feature -- Modification

	go (pos: INTEGER_64)
			-- Reset `position' to `pos'
		do
			after := pos > 0
		end

	read_char
			-- Read next character of source.  No more
			-- characters, so `after' becomes true and
			-- value of `last_char' is not meaningful
		do
			after := true
		end

	read_line
			-- Read characters until newline encountered or no
			-- more data.  Never any more data, so `after' 
			-- becomes true and `last_string' is empty
		do
			after := true
			last_string := ""
		end

	read_stream (n: INTEGER)
			-- Read characters until `n' have been read or
			-- no more data and put in `last_string'.
			-- Never any more data, so `after' becomes
			-- true and `last_string' is empty
		do
			read_line
		end

feature -- Segments

	segment (pos, cnt: INTEGER_64): DATA_SOURCE_EMPTY
			-- Segment of current source consisting of 
			-- `cnt' characters starting at position `pos'
		do
			Result := Current
		end

feature -- Operations

	close
			-- Close data source and release all resources
		do
		end
	
end
