
indexing
	
	description: "A database login operation"
	author: "David Hollenberg"
	date: "March 12, 2003"

class DATABASE_LOGIN_OP
inherit
	OPERATION
		rename
			try_operation as try_login
		end
	OCI_CONSTANTS
		export
			{NONE} all
		undefine
			default_create
		end

create
	default_create

feature -- Operations

	open (conn: DB_CONNECTION user, passwd, connstr: STRING)
			-- Open `conn' using `user', `passwd' and `connstr'
		require
			connection_exists: conn /= Void
			connection_closed: not conn.is_open
			user_exists: user /= Void
			passwd_exists: passwd /= Void
			connstr_exists: connstr /= Void
		do
			connection := conn
			user_name := user
			password := passwd
			connect_string := connstr
			execute
		end

feature {NONE} -- Implementation

	try_login
			-- Try database login.  If login fails,
			-- set `error_message'
		local
			error: DATABASE_LOGIN_ERROR
			code: INTEGER
		do
			error_message := Void
			connection.set_display_suppressed (True)
			publish_try_event
			connection.try_to_open (user_name, password, connect_string, 3)
			if connection.is_open then
				publish_ok_event
				connection.clear_last_error_description
			else
				code := connection.last_error_code
				error_message := connection.error_message_for_code (code)
				error_message.right_adjust
				publish_failed_event
				create error
				error.set_hard_error
				error.disable_retry
				if code = Oci_invalid_username_code or code = Oci_no_default_username_code then
					error.disable_notify
				end
				error.set_user_name (user_name)
				error.set_password (password)
				error.set_connect_string (connect_string)
				error.set_error_message (error_message)
				error.handle
			end
		end

	connection: DB_CONNECTION
			-- Database connection to try to open

	user_name: STRING

	password: STRING

	connect_string: STRING

	error_message: STRING
			-- Error message if login fails

	publish_try_event
		local
			try_event: DATABASE_LOGIN_TRY_EVENT
		do
			create try_event
			try_event.set_user_name (user_name)
			try_event.set_password (password)
			try_event.set_connect_string (connect_string)
			try_event.publish
		end

	publish_ok_event
		local
			ok_event: DATABASE_LOGIN_OK_EVENT
		do
			create ok_event
			ok_event.set_user_name (user_name)
			ok_event.set_password (password)
			ok_event.set_connect_string (connect_string)
			ok_event.publish
		end

	publish_failed_event
		local
			failed_event: DATABASE_LOGIN_FAILED_EVENT
		do
			create failed_event
			failed_event.set_user_name (user_name)
			failed_event.set_password (password)
			failed_event.set_connect_string (connect_string)
			failed_event.set_error_message (error_message)
			failed_event.publish
		end

end
