
indexing
	
	description: "Window for database login"
	author: "David Hollenberg"
	date: "November 23, 2003"

class DATABASE_LOGIN_WINDOW

inherit
	DATABASE_LOGIN_IFC
	WINDOW_INTERFACE
	MOSIS_GRAPHICAL_UTILITIES
		export
			{NONE} all
		undefine
			default_create
		end
		
create
	make

feature {NONE} -- Creation

	make (w: EV_TITLED_WINDOW init_user, conn_str: STRING)
			-- Create interface in window `w' for database
			-- login with initial username `init_user'
			-- (changeable) and connect string `conn_str'
		require
			window_exists: w /= Void
			window_valid: not w.is_destroyed
			init_user_exists: init_user /= Void
			connect_string_exists: conn_str /= Void
		do
			make_interface (init_user, conn_str)
			make_proxy (w)
		end

feature -- Operations

	reset
			-- Set focus in password field and move
			-- pointer into it
		do
			enable_active
			password_field.set_focus
			password_field.center_pointer
		end

feature {NONE} -- Implementation

	build_interface
			-- Build interface in `window'
		local
			main_vbox: EV_VERTICAL_BOX
			box: EV_BOX
			cols: WIDGET_COLUMN_SET
			buttons: HORIZONTAL_BUTTON_SET
		do
			set_title ("Database Login (" + initial_connect_string + ")")
			create cols.make (2)
			cols.set_padding (user_interface.padding)
			
			create user_field.make_with_text (initial_user)
			user_field.enable_uppercase_only
			user_field.return_actions.extend (agent process_accept)
			cols.append_row (<< gui.label_with_text (Username_text), user_field >>)
			
			create password_field
			password_field.return_actions.extend (agent process_accept)
			cols.append_row (<< gui.label_with_text (Password_text), password_field >>)
			
			create database_field.make_with_text (initial_connect_string)
			database_field.disable_sensitive
			cols.append_row (<< gui.label_with_text (Database_text), database_field >>)
			
			cols.disable_column_expand (1)
			cols.align_column_right (1)
			box := cols.box
			
			create main_vbox
			main_vbox.set_padding (user_interface.padding)
			main_vbox.set_border_width (user_interface.border_width)
			main_vbox.extend (box)
			main_vbox.disable_item_expand (box)
			
			create buttons.make_with_text_and_actions (<< gui.ev_ok, gui.ev_cancel >>, << agent process_accept, agent process_decline >>)
			box := buttons.column_set.box
			
			main_vbox.extend (box)
			main_vbox.disable_item_expand (box)
			

			gui.set_box_background_recursive (main_vbox, background_color)
			extend (main_vbox)
			gui.add_cancel_accelerator (window, agent process_decline)
			gui.do_once_on_idle (agent post_display_actions)
		end
		
	post_display_actions
		do
			reset
		end

	process_accept
			-- Take actions needed when user accepts
			-- current operation
		local
			old_style: EV_POINTER_STYLE
		do
			if is_active then
				disable_active
				old_style := pointer_style
				set_pointer_style (gui.Wait_cursor)
				accept_actions.call ([user_field.text.to_string_8, password_field.text.to_string_8, database_field.text.to_string_8])
				set_pointer_style (old_style)
			end
		rescue
			if old_style /= Void then
				set_pointer_style (old_style)
			end
			enable_active
		end

	process_decline
			-- Take actions needed when user declines
			-- current operation
		do
			if is_active then
				disable_active
				decline_actions.call ([])
			end
		end

feature {NONE} -- Implementation

	Username_text: STRING = "Username"

	Password_text: STRING = "Password"
	
	Database_text: STRING = "Database"

	user_field: TEXT_FIELD
	
	password_field: EV_PASSWORD_FIELD
	
	database_field: EV_TEXT_FIELD
	
invariant
	initial_user_exists: initial_user /= Void
	connect_string_exists: initial_connect_string /= Void
end
