indexing
	
	description: "Date utilities"
	author: "David Hollenberg";
	date: "March 30, 2004"

class DATE_UTILITIES
inherit
	DATE_VALIDITY
	CONDITION_UTILITIES
		export
			{NONE} all
		end
	UNIX_OS_ACCESS
		export
			{NONE} all
		end
	STRING_UTILITIES_ACCESS
		export
			{NONE} all
		end

feature -- Queries

	aligned_to_work_day (conn: DB_CONNECTION date: DB_DATE incr: INTEGER): DB_DATE is
			-- `date' + `incr' days, aligned to a work day
			-- using `conn'
		require
			connection_exists: conn /= Void
			connection_open: conn.is_open
			date_exists: date /= Void
			date_not_null: not date.is_null
		local
			b: DB_BIND_DI
		do
			create b.make (date, incr)
			Result := conn.selected_date ("m_date_utilities.align_to_work_day (:1 + :2)", "dual", Void, b)
		end
	
feature	-- Conversion between Unix date (seconds since epoch) and DB_DATE

	Unix_to_DB_DATE(u_date: INTEGER): DB_DATE is
		-- u_date is in seconds since start of epoch
	local ustr: STRING; uparts: LIST[STRING]
	do  
	    ustr := unix_os.integer_to_date_string(u_date);
			-- format is SUN JAN 31 23:59:59 1999
	    uparts := su.broken_into_words(ustr)
			-- word #     1   2  3     4      5
	    create {ORACLE_DATE} Result.set_date_and_time
			(su.string_of_7
			     ((uparts @ 3), "-", (uparts @ 2), "-",
			      (uparts @ 5), " ", (uparts @ 4)))
	end;

feature -- Operations

	check_date_in_range (date: STRING conn: DB_CONNECTION start_days, end_days: INTEGER) is
			-- Check that `date' is in the range 
			-- [today + `start_days', today + `end_days']
			-- Issue warning if not
		require
			date_exists: date /= Void
			valid_date: valid_date (date)
			connection_exists: conn /= Void
			connection_open: conn.is_open
		local
			diff: INTEGER
		do
			diff := conn.date_difference (conn.date (date), conn.system_date.truncated_to_midnight).rounded
			if diff < start_days or diff > end_days then
				-- Outside expected range
				if diff > 0 then
					warn ("Date %"" + date + "%" is " + diff.out + " days in the future.%N" + out_of_range_explanation (start_days, end_days))
				else
					warn ("Date %"" + date + "%" was " + (-diff).out + " days ago.%N" + out_of_range_explanation (start_days, end_days))
				end
			end
		end

	out_of_range_explanation (start_days, end_days: INTEGER): STRING is
		do
			Result := "Date falls outside expected range (" + date_phrase (start_days) + ", " + date_phrase (end_days) + ")"
		end
		
	date_phrase (offset_days: INTEGER): STRING is
		do
			if offset_days = 0 then
				Result := "today"
			elseif offset_days > 0 then
				Result := "today + " + offset_days.out + " days"
			else
				Result := "today - " + (-offset_days).out + " days"
			end
		end
		
end
