
indexing
	
	author: "David Hollenberg"
	date:	"February 8, 1996"
	description: "Manager of all the database connections"

class DB_CONNECTION_MANAGER

inherit
	ANY
	EXCEPTIONS
		export
			{NONE} all
		end
	DB_EXCEPTION_ACCESS
		export
			{NONE} all
		end
create
	make

feature -- Creation

	make
			-- Initialize newly created object
		do
			create connections.make (1)
		end

feature -- Registration

	register (conn: DB_CONNECTION)
			-- Register database connection `conn'
		require
			connection_exists: conn /= Void
			not_already_registered: not is_registered (conn)
		do
			connections.extend (conn)
		ensure
			registered: is_registered (conn)
		end

	deregister (conn: DB_CONNECTION)
			-- Deregister database connection `conn'
		require
			connection_exists: conn /= Void
			registered: is_registered (conn)
		do
			connections.start
			connections.prune (conn)
		ensure
			not_registered: not is_registered (conn)
		end

feature -- Operations

	abort_operations
			-- Abort database operation in progress, if any,
			-- on all registered connections which are open
		local
			conn: DB_CONNECTION
		do
			from
				connections.start
			until
				connections.exhausted
			loop
				conn := connections.item
				if conn.is_open then
					conn.abort_operation
				end
				connections.forth
			end
		end

feature {DB_CONNECTION} -- Modification

	set_error_connection (conn: DB_CONNECTION)
			-- Set `error_connection' to `conn'
		require
			connection_exists: conn /= Void
		do
			error_connection := conn
		end

feature -- Properties

	is_registered (conn: DB_CONNECTION): BOOLEAN
			-- Is connection `conn' registered?
		require
			connection_exists: conn /= Void
		do
			Result := connections.has (conn)
		end

	is_last_exception_retryable: BOOLEAN
			-- Does last exception indicate a retryable error on
			-- a database connection?
		local
			name: STRING
			conn: DB_CONNECTION
		do
			if is_developer_exception then
				name := developer_exception_name
				conn := error_connection
				Result := equal (name, dbex.Db_transaction_failure) or
					(equal (name, dbex.Db_call_interface_error)
					 and (conn /= Void and then conn.is_retryable_error))
			end
		end

	error_connection: DB_CONNECTION
			-- Connection on which last database error
			-- occurred.  Void if no database errors have
			-- occurred

feature {NONE} -- Implementation

	connections: ARRAYED_LIST [DB_CONNECTION]
			-- Registered database connections

invariant
	connections_exists: connections /= Void

end
