
indexing
	
	description: "Action that deletes view elements"
	author: "David Hollenberg"
	date: "November 30, 2006"

class DELETE_ACTION
inherit
	VIEW_ELEMENT_ACTION

feature -- Properties

	dies: LIST [PACKING_DIES_MROW]
			-- Dies all of whose copies are selected and
			-- which have already been deleted from packing

	partially_deleted_elements: LIST [RETICLE_VIEW_ELEMENT]
			-- View elements which were deleted and which
			-- did not have all copies selected

	fully_deleted_elements: LIST [RETICLE_VIEW_ELEMENT]
			-- View elements which were deleted and which
			-- had all copies selected

	scribes_only: BOOLEAN
			-- Should selected objects which are not scribes
			-- be ignored?

feature -- Modification

	set_dies (a_list: like dies)
			-- Set `dies' to `a_list'
		require
			a_list_not_void: a_list /= Void
		local
			die: PACKING_DIES_MROW
		do
			dies := a_list
			create dies_table.make (a_list.count)
			from
				a_list.start
			until
				a_list.after
			loop
				die := a_list.item
				dies_table.put (1, die)
				a_list.forth
			end
		ensure
			dies_set: dies = a_list
		end

	set_partially_deleted_elements (a_list: like partially_deleted_elements)
			-- Set `partially_deleted_elements' to `a_list'
		require
			a_list_not_void: a_list /= Void
		do
			partially_deleted_elements := a_list
		ensure
			partially_deleted_elements_set: partially_deleted_elements = a_list
		end

	set_fully_deleted_elements (a_list: like fully_deleted_elements)
			-- Set `fully_deleted_elements' to `a_list'
		require
			a_list_not_void: a_list /= Void
		do
			fully_deleted_elements := a_list
		ensure
			fully_deleted_elements_set: fully_deleted_elements = a_list
		end

	set_scribes_only (a_value: like scribes_only)
			-- Set `scribes_only' to `a_value'
		do
			scribes_only := a_value
		ensure
			scribes_only_set: scribes_only = a_value
		end

feature -- Operations

	apply (an_element: RETICLE_VIEW_ELEMENT)
			-- Apply action to `an_element'
		local
			die: PACKING_DIES_MROW
			keep: BOOLEAN
		do
			if an_element.is_selected and an_element.is_die and (not scribes_only or else an_element.is_scribe) then
				if an_element.all_copies_selected then
					die := an_element.die
					dies_table.search (die)
					keep := not dies_table.found
					if not keep then
						fully_deleted_elements.extend (an_element)
						an_element.add_change (create {FULL_DELETION_CHANGE}.make (an_element))
					end
				else
					keep := not an_element.is_deletable
					if not keep then
						an_element.delete
						partially_deleted_elements.extend (an_element)
						an_element.add_change (create {PARTIAL_DELETION_CHANGE}.make (an_element))
					end
				end
				if keep then
					an_element.add_delete_error
				end
			else
				keep := True
			end
			if keep then
				an_element.set_selected (an_element.is_selected, True)
			else
				an_element.set_is_selected (False)
				an_element.add_to_changed_region
			end
		end

feature {NONE} -- Implementation

	dies_table: HASH_TABLE [INTEGER, PACKING_DIES_MROW]

end
