
indexing
	
	description: "Action that deletes all elements whose die is in a list"
	author: "David Hollenberg"
	date: "October 31, 2006"

class DELETE_DIES_ACTION
inherit
	VIEW_ELEMENT_ACTION

feature -- Properties

	dies: LIST [PACKING_DIES_MROW]
			-- Dies whose view element is to be deleted

	deleted_elements: LIST [RETICLE_VIEW_ELEMENT]
			-- View elements which were deleted

feature -- Modification

	set_dies (a_list: like dies)
			-- Set `dies' to `a_list'
		require
			a_list_not_void: a_list /= Void
		local
			die: PACKING_DIES_MROW
		do
			dies := a_list
			create dies_table.make (a_list.count)
			from
				a_list.start
			until
				a_list.after
			loop
				die := a_list.item
				dies_table.put (1, die)
				a_list.forth
			end
		ensure
			dies_set: dies = a_list
		end

	set_deleted_elements (a_list: like deleted_elements)
			-- Set `deleted_elements' to `a_list'
		require
			a_list_not_void: a_list /= Void
		do
			deleted_elements := a_list
		ensure
			deleted_elements_set: deleted_elements = a_list
		end

feature -- Operations

	apply (an_element: RETICLE_VIEW_ELEMENT)
			-- Apply action to `an_element'
		local
			die: PACKING_DIES_MROW
			keep: BOOLEAN
		do
			if an_element.is_die then
				die := an_element.die
				dies_table.search (die)
				keep := not dies_table.found
			else
				keep := True
			end
			if keep then
				an_element.set_selected (an_element.is_selected, True)
			else
				deleted_elements.extend (an_element)
				an_element.add_to_changed_region
			end
		end

feature {NONE} -- Implementation

	dies_table: HASH_TABLE [INTEGER, PACKING_DIES_MROW]

end
