
indexing
	
	description: "Delta run closing application"
	author: "David Hollenberg"
	date: "September 14, 2002"

deferred class DELTA_APPLICATION

inherit
	RC_INTERFACE_FACTORY
		undefine
			default_create, copy
		end
	RC_APPLICATION
		undefine
			default_create, copy
		end
	RC_ENVIRONMENT_ACCESS
		export
			{NONE} all
		undefine
			default_create, copy
		end
	DB_OPTIONS_ACCESS
		export
			{NONE} all
		undefine
			default_create, copy
		end
	RC_SESSION_MANAGER_ACCESS
		export
			{NONE} all
		undefine
			default_create, copy
		end
	RC_UTILITIES_ACCESS
		export
			{NONE} all
		undefine
			copy
		end
	EXCEPTIONS_ACCESS
		export
			{NONE} all
		undefine
			default_create, copy
		end
	DELTA_VERSION_CHECKER_ACCESS
		export
			{NONE} all
		undefine
			default_create, copy
		end
	
feature -- Initialization

	make (d: DELTA)
			-- Create the application and set up `main_window'
		require
			delta_exists: d /= Void
		do
			delta := d
			initialize
		end

	initialize
			-- Initialize the application
		deferred
		end

	initialization_errors: STRING
			-- Errors that occurred during initialization

	build
			-- Connect the pieces to build the application
		local
			conn: DB_CONNECTION
			login_ifc: DATABASE_LOGIN_IFC
			login_op: DATABASE_LOGIN_OP
		do
			conn := session_manager.current_session.connection
			create login_op
			login_ifc := database_login_interface (rc_env.upper_user_name, rc_env.db_conn_str)
			login_ifc.build
			show_current_window
			login_ifc.accept_actions.extend (agent login_op.open (conn, ?, ?, ?))
			login_ifc.decline_actions.extend (agent exit_application)
			login_op.abort_actions.extend (agent login_ifc.reset)
			
			login_op.enable_abort_caught

			login_op.success_actions.extend (agent initialize_connection (conn))
			login_op.success_actions.extend (agent select_session)
		end

	launch
			-- Launch application
		deferred
		end
	
	initialize_connection (conn: DB_CONNECTION)
			-- Initialize open database connection
		require
			conn_exists: conn /= Void
			conn_open: conn.is_open
		local
			op: OPERATOR_VALIDATION_OP
		do
			conn.set_write_and_block_access
			create op
			op.validate_user (conn, conn.user_name)
			rc_env.set_user_email_address (op.email_address)
		rescue
			conn.close
		end
	
	select_session
			-- Select session type
		local
			selection_ifc: SINGLE_SELECTION_IFC
		do
			set_default_role
			selection_ifc := session_selection_interface ("Select Session Type")
			selection_ifc.add_default_choice_and_action ("Close existing run", agent select_run_id_for_close)
			selection_ifc.add_choice_and_action ("View existing run (read-only)", agent select_run_id_for_view)
			
			selection_ifc.add_choice_and_action ("Create new real run", agent select_run_id_for_create (False))
			-- FIXME: next line removed until new Delta
			-- supports trial runs
			-- selection_ifc.add_choice_and_action ("Create new trial run", agent select_run_id_for_create (True))
			selection_ifc.add_choice_and_action ("Delete existing run", agent select_run_id_for_delete)
			
			-- FIXME: add back in when release project selection
			if not rc_env.is_production then
				selection_ifc.add_choice_and_action ("Create new packing", agent create_new_packing)
				selection_ifc.add_choice_and_action ("Edit existing packing", agent edit_existing_packing)
			end
			-- FIXME: add back in when release standalone cut
			-- pattern editor for productions
			-- selection_ifc.add_choice_and_action ("Edit cut patterns", agent select_production_id_for_cut_patterns)
			selection_ifc.add_choice_and_action ("Generate bonding diagrams", agent select_run_id_for_bonding)
			selection_ifc.add_choice_and_action ("View bonding diagrams", agent view_bonding_diagrams (selection_ifc))
			selection_ifc.add_choice_and_action ("Lock out all run closing", agent lock_out_run_closing (selection_ifc))
			selection_ifc.build
			selection_ifc.accept_actions.extend (agent check_for_new_version (selection_ifc))
			selection_ifc.decline_actions.extend (agent exit_application)
		end
			
	check_for_new_version (ifc: SINGLE_SELECTION_IFC)
		do
			delta_version_checker.check_for_new_version
			if not delta_version_checker.abort_requested then
				ifc.execute_selected_action
			else
				ifc.enable_active
			end
		end

	set_role (role: STRING password: STRING)
			-- Set database role
		local
			role_setter: ROLE_SET_OP
		do
			create role_setter
			role_setter.set_role (session_manager.current_session.connection, role, password)
		end
			
	set_default_role
			-- Set default database role
		do
			set_role (rc_env.db_default_role, Void)
		end
			
	set_run_closer_role
			-- Set role to allow run close
		do
			set_role (rc_env.db_run_closer_role, rc_env.db_access_control)
		end
			
	select_run_id_for_close
			-- Obtain run id and close existing run
		local
			run_id_ifc: SINGLE_FREE_VALUE_IFC
			close_op: RUN_CLOSE_OP
		do
			run_id_ifc := single_free_value_interface ("Select Run ID to Close")
			run_id_ifc.set_label ("Run ID")
			run_id_ifc.enable_uppercase_only
			run_id_ifc.build
			
			create close_op
			close_op.enable_abort_caught
			close_op.success_actions.extend (agent select_run_closing_steps ("Select Run Closing Step"))
			close_op.abort_actions.extend (agent set_default_role)
			close_op.abort_actions.extend (agent run_id_ifc.reset)
			run_id_ifc.accept_actions.extend (agent run_id_ifc.execute_action (agent close_run (close_op, ?)))
			run_id_ifc.decline_actions.extend (agent select_session)
		end
			
	close_run (close_op: RUN_CLOSE_OP run_id: STRING)
			-- Attempt to close run `run_id' using `close_op'
		require
			close_op_exists: close_op /= Void
			run_id_exists: run_id /= Void
		local
			sess: RC_RUN_CLOSING_SESSION
		do
			sess := new_run_closing_session (run_id)
			session_manager.add_session (sess)
			session_manager.set_current_session (sess)
			close_op.close_run (run_id, sess)
		end

	new_run_closing_session (run_id: STRING): RC_RUN_CLOSING_SESSION
			-- New run closing session of appropriate dynamic
			-- type for `run_id'
		require
			run_id_exists: run_id /= Void
		do
			if rc.looks_like_trial_run_id (run_id) then
				create {RC_TRIAL_RUN_SESSION} Result.make_run_session (session_manager.current_session, run_id)
			else
				create {RC_REAL_RUN_SESSION} Result.make_run_session (session_manager.current_session, run_id)
			end
		end

	select_run_id_for_view
			-- Obtain run id and view existing run
		local
			run_id_ifc: SINGLE_FREE_VALUE_IFC
			view_op: RUN_CLOSE_OP
		do
			run_id_ifc := single_free_value_interface ("Select Run ID to View")
			run_id_ifc.set_label ("Run ID")
			run_id_ifc.enable_uppercase_only
			run_id_ifc.build
			
			create view_op
			view_op.enable_abort_caught
			view_op.success_actions.extend (agent select_run_closing_steps ("View Run Closing Steps"))
			view_op.abort_actions.extend (agent set_default_role)
			view_op.abort_actions.extend (agent run_id_ifc.reset)
			run_id_ifc.accept_actions.extend (agent run_id_ifc.execute_action (agent view_run (view_op, ?)))
			run_id_ifc.decline_actions.extend (agent select_session)
		end
			
	view_run (view_op: RUN_CLOSE_OP run_id: STRING)
			-- Attempt to view run `run_id' using `view_op'
		require
			view_op_exists: view_op /= Void
			run_id_exists: run_id /= Void
		local
			sess: RC_RUN_CLOSING_SESSION
		do
			sess := new_run_closing_session (run_id)
			sess.set_read_only
			session_manager.add_session (sess)
			session_manager.set_current_session (sess)
			view_op.close_run (run_id, sess)
		end

	select_run_closing_steps (title: STRING)
			-- Select run closing steps user wants and
			-- what action to take on them (execute, undo, redo)
		require
			title_exists: title /= Void
		local
			selection_ifc: RC_STEP_SELECTION_IFC
			status_ifc: RC_STATUS_IFC
			print_ifc: RC_MAP_PRINT_IFC
			prefs_ifc: EDIT_PREFERENCES_IFC
			session: RC_RUN_CLOSING_SESSION
		do
			session ?= session_manager.current_session
			check
				session_exists: session /= Void
			end
			selection_ifc := step_selection_interface (session.run_id, session.title_with_run_id (title))
			selection_ifc.set_primitive_steps (session.primitive_steps)
			selection_ifc.set_composite_steps (session.composite_steps)
			selection_ifc.set_session (session)
			status_ifc := run_status_interface (session.run_id, session.title_with_run_id ("Status"))
			status_ifc.set_session (session)
			status_ifc.set_primitive_steps (session.primitive_steps)
			selection_ifc.set_status_interface (status_ifc)
			
			print_ifc := wafer_map_print_interface (session.run_id, session.title_with_run_id ("Print Wafer Maps"))
			print_ifc.set_session (session)
			selection_ifc.set_wafer_map_print_interface (print_ifc)
			
			print_ifc := bond_diagram_print_interface (session.run_id, session.title_with_run_id ("Print Bonding Diagrams"))
			print_ifc.set_session (session)
			selection_ifc.set_bond_diagram_print_interface (print_ifc)
			print_ifc := job_file_print_interface (session.run_id, session.title_with_run_id ("Print Job Files"))
			print_ifc.set_session (session)
			selection_ifc.set_job_file_print_interface (print_ifc)
			
			prefs_ifc := edit_preferences_interface (session.run_id, session.title_with_run_id ("Preferences"))
			prefs_ifc.set_session (session)
			selection_ifc.set_preferences_interface (prefs_ifc)
			selection_ifc.subscribe_events
			selection_ifc.build
			selection_ifc.do_actions.extend (agent process_steps (selection_ifc, session, Execute_requested, ?))
			selection_ifc.modify_actions.extend (agent process_steps (selection_ifc, session, Modify_requested, ?))
			selection_ifc.redo_actions.extend (agent process_steps (selection_ifc, session, Redo_requested, ?))
			selection_ifc.undo_actions.extend (agent process_steps (selection_ifc, session, Undo_and_save_requested, ?))
			selection_ifc.hard_undo_actions.extend (agent process_steps (selection_ifc, session, Undo_and_discard_requested, ?))
			selection_ifc.cancel_actions.extend (agent selection_ifc.unsubscribe_events)
			selection_ifc.cancel_actions.extend (agent terminate_current_session)
			selection_ifc.cancel_actions.extend (agent select_session)
		end
			
	process_steps (ifc: RC_STEP_SELECTION_IFC sess: RC_RUN_CLOSING_SESSION type: INTEGER step: RC_STEP)
			-- Process run closing step user selected
		require
			interface_exists: ifc /= Void
			session_exists: sess /= Void
			step_exists: step /= Void
		local
			op: RC_PRIMITIVE_STEPS_OP
			steps: LIST [RC_PRIMITIVE_STEP]
		do
			delta_version_checker.check_for_new_version
			if not delta_version_checker.abort_requested then
				ifc.disable_active
				sess.auto_flush_cache_if_needed
				sess.clear_cache_auto_flush_time
				inspect type
				when Execute_requested then
					steps := sess.forward_steps (step)
				when Undo_and_discard_requested then
					steps := sess.backward_steps (step, False)
				when Modify_requested, Redo_requested, Undo_and_save_requested then
					steps := sess.backward_steps (step, True)
				end
				op := sess.step_list_implementation (steps)
				op.set_selected_step (step)
				inspect type
				when Execute_requested then
					op.set_do
				when Modify_requested then
					op.set_modify
				when Redo_requested then
					op.set_redo
				when Undo_and_discard_requested then
					op.set_undo_and_discard_state
				when Undo_and_save_requested then
					op.set_undo_and_save_state
				end
				op.enable_abort_caught
				op.abort_actions.extend (agent switch_to_main_window)
				op.abort_actions.extend (agent ifc.reset)
				op.abort_actions.extend (agent (sess.cache).uncache_all)
				op.completion_actions.extend (agent switch_to_main_window)
				op.completion_actions.extend (agent ifc.reset)
				op.completion_actions.extend (agent sess.set_cache_auto_flush_time)
				op.execute
			else
				ifc.reset
			end
		end

	select_run_id_for_create (is_trial: BOOLEAN)
			-- Obtain run id and create new run.  If `is_trial'
			-- create a trial run, otherwise create a real run
		local
			run_id_ifc: SINGLE_FREE_VALUE_IFC
			creation_op: RUN_CREATION_OP
			run_type: STRING
		do
			if is_trial then
				run_type := "Trial"
			else
				run_type := "Real"
			end
			run_id_ifc := single_free_value_interface ("Select " + run_type + " Run ID to Create")
			run_id_ifc.set_label ("New " + run_type + " Run ID")
			run_id_ifc.enable_uppercase_only
			run_id_ifc.build
			
			create creation_op
			creation_op.enable_abort_caught
			creation_op.success_actions.extend (agent select_session)
			creation_op.abort_actions.extend (agent set_default_role)
			creation_op.abort_actions.extend (agent run_id_ifc.reset)
			creation_op.abort_actions.extend (agent terminate_current_session)
			run_id_ifc.accept_actions.extend (agent run_id_ifc.execute_action (agent create_run (creation_op, ?, is_trial)))
			run_id_ifc.decline_actions.extend (agent select_session)
		end
			
	create_run (creation_op: RUN_CREATION_OP run_id: STRING is_trial: BOOLEAN)
			-- Attempt to create run `run_id' using `creation_op'.
			-- Create a trial run if `is_trial', otherwise create
			-- a real run
		require
			creation_op_exists: creation_op /= Void
			run_id_exists: run_id /= Void
		local
			sess: RC_RUN_CLOSING_SESSION
		do
			if is_trial then
				create {RC_TRIAL_RUN_SESSION} sess.make_run_session (session_manager.current_session, run_id)
			else
				create {RC_REAL_RUN_SESSION} sess.make_run_session (session_manager.current_session, run_id)
			end
			session_manager.add_session (sess)
			session_manager.set_current_session (sess)
			creation_op.create_run (run_id, sess, rc_env.db_run_closer_role)
		end

	create_new_packing
			-- Present interface to create a new packing
		local
			new_packing_ifc: RC_PACKING_IFC
			sess: RC_SESSION
		do
			new_packing_ifc := packing_creation_interface ("Create New Packing")
			create {RC_PACKING_CREATION_SESSION} sess.make_from_session (session_manager.current_session)
			new_packing_ifc.set_session (sess)
			new_packing_ifc.set_role (rc_env.db_run_closer_role)
			new_packing_ifc.cancel_actions.extend (agent select_session)
			new_packing_ifc.build
		end
			
	edit_existing_packing
			-- Present interface to edit existing packing
		local
			packing_sel_ifc: RC_PACKING_IFC
			sel_sess: RC_SESSION
			pack_sess: RC_PACKING_SESSION
		do
			packing_sel_ifc := packing_selection_interface ("Edit Existing Packing")
			create {RC_PACKING_SELECTION_SESSION} sel_sess.make_from_session (session_manager.current_session)
			create pack_sess.make_from_session (session_manager.current_session)
			packing_sel_ifc.set_role (rc_env.db_run_closer_role)
			packing_sel_ifc.set_session (sel_sess)
			packing_sel_ifc.set_packing_session (pack_sess)
			packing_sel_ifc.cancel_actions.extend (agent select_session)
			packing_sel_ifc.build
		end
			
	terminate_current_session
		do
			session_manager.current_session.terminate
		end

	select_production_id_for_cut_patterns
			-- Obtain production ID and allow user to edit
			-- cut patterns for production
		local
			prod_id_ifc: SINGLE_FREE_VALUE_IFC
			edit_op: EDIT_CUT_PATTERNS_OP
		do
			prod_id_ifc := single_free_value_interface ("Select Production ID for Cut Pattern Editing")
			prod_id_ifc.set_label ("Production ID")
			prod_id_ifc.enable_uppercase_only
			prod_id_ifc.build
			
			create edit_op
			edit_op.enable_abort_caught
			edit_op.success_actions.extend (agent select_session)
			edit_op.abort_actions.extend (agent edit_op.cleanup_after_abort)
			edit_op.abort_actions.extend (agent set_default_role)
			edit_op.abort_actions.extend (agent prod_id_ifc.reset)
			edit_op.abort_actions.extend (agent terminate_current_session)
			prod_id_ifc.accept_actions.extend (agent prod_id_ifc.execute_action (agent edit_cut_patterns (edit_op, ?)))
			prod_id_ifc.decline_actions.extend (agent select_session)
		end
			
	edit_cut_patterns (edit_op: EDIT_CUT_PATTERNS_OP prod_id: STRING)
			-- Attempt to edit cut patterns for production 
			-- `prod_id' using `edit_op'
		require
			edit_op_exists: edit_op /= Void
			prod_id_exists: prod_id /= Void
		local
			sess: RC_RUN_CLOSING_SESSION
		do
			sess := new_run_closing_session ("")
				-- Run ID not known yet - session will be 
				-- switched to correct run if production is
				-- valid
			session_manager.add_session (sess)
			session_manager.set_current_session (sess)
			edit_op.edit_cut_patterns (prod_id, sess)
		end

	select_run_id_for_bonding
			-- Obtain run id and generate bonding diagrams
			-- for run
		local
			run_id_ifc: SINGLE_FREE_VALUE_IFC
			generation_op: BONDING_GENERATION_OP
		do
			run_id_ifc := single_free_value_interface ("Select Run ID for Diagram Generation")
			run_id_ifc.set_label ("Run ID")
			run_id_ifc.enable_uppercase_only
			run_id_ifc.build
			
			create generation_op
			generation_op.enable_abort_caught
			generation_op.success_actions.extend (agent select_session)
			generation_op.abort_actions.extend (agent generation_op.cleanup_after_abort)
			generation_op.abort_actions.extend (agent set_default_role)
			generation_op.abort_actions.extend (agent run_id_ifc.reset)
			generation_op.abort_actions.extend (agent terminate_current_session)
			run_id_ifc.accept_actions.extend (agent run_id_ifc.execute_action (agent generate_bonding_diagrams (generation_op, ?)))
			run_id_ifc.decline_actions.extend (agent select_session)
		end
			
	generate_bonding_diagrams (generation_op: BONDING_GENERATION_OP run_id: STRING)
			-- Attempt to generate bonding diagrams for `run_id'
			-- using `generation_op'
		require
			generation_op_exists: generation_op /= Void
			run_id_exists: run_id /= Void
		local
			sess: RC_RUN_CLOSING_SESSION
		do
			sess := new_run_closing_session (run_id)
			session_manager.add_session (sess)
			session_manager.set_current_session (sess)
			generation_op.generate_diagrams (run_id, sess)
		end

	select_run_id_for_delete
			-- Obtain run id and delete existing run
		local
			run_id_ifc: SINGLE_FREE_VALUE_IFC
			deletion_op: RUN_DELETION_OP
		do
			run_id_ifc := single_free_value_interface ("Select Run ID to Delete")
			run_id_ifc.set_label ("Run ID")
			run_id_ifc.enable_uppercase_only
			run_id_ifc.build
			
			create deletion_op
			deletion_op.enable_abort_caught
			deletion_op.success_actions.extend (agent select_session)
			deletion_op.abort_actions.extend (agent set_default_role)
			deletion_op.abort_actions.extend (agent run_id_ifc.reset)
			deletion_op.abort_actions.extend (agent terminate_current_session)
			run_id_ifc.accept_actions.extend (agent run_id_ifc.execute_action (agent delete_run (deletion_op, ?)))
			run_id_ifc.decline_actions.extend (agent select_session)
		end
			
	delete_run (deletion_op: RUN_DELETION_OP run_id: STRING)
			-- Attempt to delete run `run_id' using `deletion_op'
		require
			deletion_op_exists: deletion_op /= Void
			run_id_exists: run_id /= Void
		local
			sess: RC_RUN_CLOSING_SESSION
		do
			sess := new_run_closing_session (run_id)
			session_manager.add_session (sess)
			session_manager.set_current_session (sess)
			deletion_op.delete_run (run_id, sess, rc_env.db_run_closer_role)
		end

	view_bonding_diagrams (ifc: ABSTRACT_INTERFACE)
			-- View bonding diagrams
		require
			interface_exists: ifc /= Void
		local
			bonding_ifc: RC_BONDING_IFC
			print_ifc: RC_MAP_PRINT_IFC
			sess: RC_BONDING_SESSION
		do
			create sess.make_from_session (session_manager.current_session)
			session_manager.add_session (sess)
			session_manager.set_current_session (sess)
			bonding_ifc := bonding_interface (Void, "View bonding diagrams")
			bonding_ifc.set_session (sess)
			print_ifc := bond_diagram_list_print_interface ("Print bonding diagrams")
			print_ifc.set_session (sess)
			bonding_ifc.set_print_interface (print_ifc)
			bonding_ifc.build
			bonding_ifc.cancel_actions.extend (agent terminate_current_session)
			bonding_ifc.cancel_actions.extend (agent select_session)
		end

	lock_out_run_closing (ifc: ABSTRACT_INTERFACE)
			-- Try to get exclusive lock on shared lock file
			-- to lock out all run closers
		require
			interface_exists: ifc /= Void
		local
			lock_out_op: RUN_LOCK_OUT_OP
		do
			create lock_out_op
			lock_out_op.enable_abort_caught
			lock_out_op.lock_out (session_manager.current_session)
			ifc.enable_active
		end
			
	report_fatal_exception (a_trace: STRING)
			-- Report a fatal exception to user, using
			-- `a_trace' as the exception trace if not Void
		deferred
		end
			
	fatal_exception_text (a_trace: STRING): STRING
			-- Text describing fatal exception, using
			-- `a_trace' as the exception trace if not Void
		local
			db_error: STRING
			conn: DB_CONNECTION
		do
			create Result.make (0)
			conn := session_manager.current_session.connection
			if conn /= Void then
				db_error := conn.last_error_description
				if not db_error.is_empty then
					Result.extend ('%N')
					Result.append (db_error)
					Result.extend ('%N')
				end
			end
			if a_trace /= Void then
				Result.append (a_trace)
			else
				Result.append (exc.exception_trace)
			end
		end
	
feature {NONE} -- Implementation

	delta: DELTA

	Execute_requested: INTEGER = 1
			-- Code to indicate that step should be executed

	Modify_requested: INTEGER = 2
			-- Code to indicate that step should be modified
			-- (undo all steps that depend on it, then mark
			-- step as not done but don't undo any work it did)

	Redo_requested: INTEGER = 3
			-- Code to indicate that step should be redone

	Undo_and_discard_requested: INTEGER = 4
			-- Code to indicate that step should be undone and
			-- no state saved

	Undo_and_save_requested: INTEGER = 5
			-- Code to indicate that step should be undone but
			-- state should be saved to make it easier to redo

end
