
indexing
	
	description: "Delta run closing system as a graphical application"
	author: "David Hollenberg"
	date: "September 14, 2002"

class DELTA_GRAPHIC_APPLICATION

inherit
	DELTA_APPLICATION
	EV_APPLICATION
		rename
			initialize as initialize_app
		end
	RC_SESSION_MANAGER_ACCESS
		export
			{NONE} all
		undefine
			default_create, copy
		end
	ACTIVE_WINDOW_LIST_ACCESS
		export
			{NONE} all
		undefine
			default_create, copy
		end
	GUI_UTILITIES_ACCESS
		export
			{NONE} all
		undefine
			default_create, copy
		end
	EXCEPTIONS_ACCESS
		export
			{NONE} all
		undefine
			default_create, copy
		end
	FILE_UTILITIES_ACCESS
		export
			{NONE} all
		undefine
			default_create, copy
		end
	GUI_UTILITIES_ACCESS
		export
			{NONE} all
		undefine
			default_create, copy
		end
	USER_INTERFACE_ACCESS
		export
			{NONE} all
		undefine
			default_create, copy
		end
	CONDITION_HANDLER_ACCESS
		export
			{NONE} all
		undefine
			default_create, copy
		end
	
create
	make

feature -- Properties

	main_window: EV_TITLED_WINDOW
			-- Main window for Delta

	aux_window: EV_TITLED_WINDOW
			-- Auxiliary window for Delta

	run_status_window: EV_TITLED_WINDOW
			-- Window to show status of run

	wafer_map_print_window: EV_TITLED_WINDOW
			-- Window to print wafer maps for run

	bond_diagram_print_window: EV_TITLED_WINDOW
			-- Window to print bonding diagrams for run

	bond_diagram_list_print_window: EV_TITLED_WINDOW
			-- Window to print list of bonding diagrams
			-- (not necessarily ones on a run)

	job_file_print_window: EV_TITLED_WINDOW
			-- Window to print job files for run

	edit_preferences_window: EV_TITLED_WINDOW
			-- Window to edit preferences

	current_window_cell: CELL [EV_TITLED_WINDOW]
			-- Reference to current "main" window (needed for
			-- error reporting)

	current_window: EV_TITLED_WINDOW
			-- Current "main" window
		do
			Result := current_window_cell.item
		end

	background_color: EV_COLOR
			-- Background color for most windows

feature -- Initialization

	initialize
			-- Initialize application
		do
			default_create
			create initialization_errors.make (0)
			set_screen_size
			read_icon_files
			if initialization_errors.is_empty then
				set_tooltip_delay (1000)
				create main_window
				active_window_list.register (main_window)
				create current_window_cell.put (main_window)
				gui.destroy_on_close_request (main_window)
				setup_condition_handlers
				main_window.set_position (screen_size.x // 3, screen_size.y // 3)
			end
		end

	set_background_color
			-- Set `background_color'
		do
			background_color := user_interface.background_color
		end

	show_current_window
			-- Make sure current window is displayed
		do
			current_window.show
		end
			
	hide_current_window
			-- Make sure current window is not displayed
		do
			current_window.hide
		end
			
	switch_to_auxiliary_window
			-- Make auxiliary window the current window
		do
			if aux_window = Void then
				create aux_window
				active_window_list.register (aux_window)
				aux_window.set_position (screen_size.x * 1 // 5, screen_size.y * 2 // 5)
			end
			current_window_cell.put (aux_window)
		end
			
	switch_to_main_window
			-- Make main window the current window
		do
			current_window_cell.put (main_window)
			if aux_window /= Void then
				aux_window.destroy
				aux_window := Void
			end
		end
			
	raise_auxiliary_window
			-- Raise auxiliary window so that it is on top
		do
			raise_window (aux_window)
		end
			
	raise_main_window
			-- Raise main window so that it is on top
		do
			raise_window (main_window)
			if main_window /= Void then
				main_window.show
				main_window.raise
			end
		end
			
	raise_window (window: EV_TITLED_WINDOW)
			-- Raise `window' so that it is on top.  Restore it
			-- first if it is minimized
		do
			if window /= Void then
				if window.is_minimized then
					window.restore
				end
				window.raise
			end
		end
			
	report_fatal_exception (a_trace: STRING)
			-- Report a fatal exception to user, using
			-- `a_trace' as the exception trace if not Void
		local
			w: EV_TITLED_WINDOW
		do
			publish_fatal_exception
			set_existing_windows_insensitive (windows)
			create w
			create error_window.make (w, fatal_exception_text (a_trace))
			error_window.reset_to_default
			error_window.notify_actions.extend (gui.destroy_application_agent)
			error_window.exit_actions.extend (agent delta.disable_error_notification)
			error_window.exit_actions.extend (gui.destroy_application_agent)
			error_window.set_title ("Delta Fatal Exception")
			error_window.set_background_color (gui.Red)
			error_window.build
			error_window.show
		end
			
feature {NONE} -- Implementation

	error_window: FATAL_ERROR_WINDOW
	
	screen_size: EV_COORDINATE

	set_screen_size
			-- Set `screen_size'
		local
			screen: EV_SCREEN
		do
			create screen
			create screen_size.set (screen.width, screen.height)
		end

	read_icon_files
			-- Read icon files named by parameters and
			-- create a pixmap for each one
		do
			read_icon_file (rc_env.step_done_icon)
			step_done_icon := last_icon
			read_icon_file (rc_env.step_inapplicable_icon)
			step_inapplicable_icon := last_icon
			read_icon_file (rc_env.step_undone_icon)
			step_undone_icon := last_icon
			read_icon_file (rc_env.doing_step_icon)
			doing_step_icon := last_icon
			read_icon_file (rc_env.undoing_step_icon)
			undoing_step_icon := last_icon
			read_icon_file (rc_env.step_saved_state_icon)
			step_saved_state_icon := last_icon
		end

	read_icon_file (fname: STRING)
			-- Read icon file named `fname' and put resulting
			-- pixmap in `last_icon'.  If error reading file,
			-- set `last_icon' to Void and append error message
			-- to `initialization_errors'
		require
			file_name_exists: fname /= Void
		local
			tried: BOOLEAN
			pixmap: EV_PIXMAP
		do
			if not tried then
				if fu.file_exists (fname) then
					create pixmap
					pixmap.set_with_named_file (fname)
					last_icon := pixmap
				else
					pixmap := Void
					initialization_errors.append ("File not found: " + fname + "%N")
				
				end
			else
				pixmap := Void
				initialization_errors.append ("Exception %"" + exc.developer_exception_name + "%" raised while reading file " + fname + "%N")
			end
		rescue
			if not exc.assertion_violation then
				tried := True
				retry
			end
		end

	setup_condition_handlers
			-- Setup condition handlers for conditions such as
			-- error, warning, etc
		local
			handler: STD_CONDITION_HANDLER
			handlers: GRAPHICAL_CONDITION_HANDLER_SET
		do
			create handler
			create handlers.make (current_window_cell)
			handlers.add_notify_action (agent send_error_notification)
			handler.set_handlers (handlers)
			condition_handler_cell.put (handler)
		end

	send_error_notification (msg: STRING)
			-- Send error notification for current session
		require
			message_exists: msg /= Void
		local
			session: RC_SESSION
		do
			session := session_manager.current_session
			if session /= Void then
				session.send_error_notification (msg)
			end
		end

	database_login_interface (user, conn_str: STRING): DATABASE_LOGIN_WINDOW
		do
			create Result.make (current_window, user, conn_str)
			Result.reset_to_default
			Result.set_background_color (background_color)
			set_active (Result)
		end
	
	session_selection_interface (title: STRING): SESSION_SELECTION_WINDOW
			-- Interface to select session type
		do
			create Result.make (current_window)
			Result.reset_to_default
			Result.set_background_color (background_color)
			Result.set_title (title)
			set_active (Result)
		end
	
	single_fixed_selection_interface (title: STRING): SINGLE_FIXED_SELECTION_WINDOW
		do
			create Result.make (current_window)
			Result.reset_to_default
			Result.set_background_color (background_color)
			Result.set_title (title)
			set_active (Result)
		end
	
	single_constrained_value_interface (title: STRING): SINGLE_CONSTRAINED_VALUE_WINDOW
		do
			create Result.make (current_window)
			Result.reset_to_default
			Result.set_background_color (background_color)
			Result.set_title (title)
			set_active (Result)
		end
	
	single_free_value_interface (title: STRING): SINGLE_FREE_VALUE_WINDOW
			-- Interface to enter a single unconstrained value
		do
			create Result.make (current_window)
			Result.reset_to_default
			Result.set_background_color (background_color)
			Result.set_title (title)
			set_active (Result)
		end
	
	step_selection_interface (run_id, title: STRING): RC_STEP_SELECTION_WINDOW
			-- Interface with `title' to select run closing steps
			-- for run `run_id' and what actions to take on them
		do
			create Result.make (current_window)
			Result.reset_to_default
			Result.set_background_color (background_color)
			Result.set_title (title)
			Result.set_step_done_icon (step_done_icon)
			Result.set_step_inapplicable_icon (step_inapplicable_icon)
			Result.set_step_undone_icon (step_undone_icon)
			Result.set_doing_step_icon (doing_step_icon)
			Result.set_undoing_step_icon (undoing_step_icon)
			Result.set_step_saved_state_icon (step_saved_state_icon)
			set_active (Result)
		end
	
	progress_interface (run_id, title: STRING): PROGRESS_WINDOW
			-- Interface with `title' to display progress of
			-- application for `run_id', with Abort button
		do
			create Result.make (current_window)
			Result.reset_to_default
			Result.set_background_color (background_color)
			Result.set_title (title)
			set_active (Result)
		end
	
	field_set_interface (run_id, title: STRING): FIELD_SET_WINDOW
			-- Interface with `title' to allow user to
			-- select field set for `run_id'
		do
			create Result.make (current_window)
			Result.reset_to_default
			Result.set_background_color (background_color)
			Result.set_title (title)
			set_active (Result)
		end
	
	run_status_interface (run_id, title: STRING): RC_STATUS_WINDOW
			-- Interface with `title' to display status of
			-- run `run_id'
		do
			create run_status_window
			active_window_list.register (run_status_window)
			create Result.make (run_status_window)
			Result.reset_to_default
			Result.set_background_color (background_color)
			Result.set_title (title)
			set_active (Result)
		end
	
	merged_gds_field_control_interface (run_id, title: STRING): MERGED_GDS_FIELD_CONTROL_WINDOW
			-- Interface with `title' to allow user to submit,
			-- monitor and control jobs to create merged GDS
			-- files (one per field) for run `run_id'
		do
			create Result.make (current_window)
			Result.reset_to_default
			Result.set_background_color (background_color)
			Result.set_title (title)
			set_active (Result)
		end
	
	merged_gds_die_control_interface (run_id, title: STRING): MERGED_GDS_DIE_CONTROL_WINDOW
			-- Interface with `title' to allow user to submit,
			-- monitor and control jobs to create merged GDS
			-- files (one per die) for run `run_id'
		do
			create Result.make (current_window)
			Result.reset_to_default
			Result.set_background_color (background_color)
			Result.set_title (title)
			set_active (Result)
		end
	
	die_fracture_control_interface (run_id, title: STRING): DIE_FRACTURE_CONTROL_WINDOW
			-- Interface with `title' to allow user to submit,
			-- monitor and control jobs to fracture individual
			-- dies of run `run_id'
		do
			create Result.make (current_window)
			Result.reset_to_default
			Result.set_background_color (background_color)
			Result.set_title (title)
			set_active (Result)
		end
	
	level_merge_control_interface (run_id, title: STRING): LEVEL_MERGE_CONTROL_WINDOW
			-- Interface with `title' to allow user to submit,
			-- monitor and control jobs to merge pattern files
			-- for each levelf of run `run_id'
		do
			create Result.make (current_window)
			Result.reset_to_default
			Result.set_background_color (background_color)
			Result.set_title (title)
			set_active (Result)
		end
	
	level_fill_control_interface (run_id, title: STRING): LEVEL_FILL_CONTROL_WINDOW
			-- Interface with `title' to allow user to submit,
			-- monitor and control jobs to fill pattern files
			-- for each level of run `run_id' that needs to be
			-- filled
		do
			create Result.make (current_window)
			Result.reset_to_default
			Result.set_background_color (background_color)
			Result.set_title (title)
			set_active (Result)
		end
	
	fill_selector_control_interface (run_id, title: STRING): FILL_SELECTOR_CONTROL_WINDOW
			-- Interface with `title' to allow user to submit,
			-- monitor and control jobs to produce the fill
			-- selector file for run `run_id'
		do
			create Result.make (current_window)
			Result.reset_to_default
			Result.set_background_color (background_color)
			Result.set_title (title)
			set_active (Result)
		end
	
	edit_preferences_interface (run_id, title: STRING): EDIT_PREFERENCES_WINDOW
			-- Interface with `title' to allow user to edit
			-- preferences for session for `run_id'
		do
			create edit_preferences_window
			active_window_list.register (edit_preferences_window)
			create Result.make (edit_preferences_window)
			Result.reset_to_default
			Result.set_background_color (background_color)
			Result.set_title (title)
			set_active (Result)
		end
	
	turnaround_dates_interface (run_id, title: STRING): TURNAROUND_DATES_WINDOW
			-- Interface with `title' to allow user to set
			-- tapeout date and estimated turnaround dates
			-- for `run_id'
		do
			create Result.make (current_window)
			Result.reset_to_default
			Result.set_background_color (background_color)
			Result.set_title (title)
			set_active (Result)
		end
	
	fab_run_order_interface (run_id, title: STRING): FAB_RUN_ORDER_WINDOW
			-- Interface with `title' to allow user to build
			-- order for fabrication of dedicated or trusted
			-- foundry run `run_id'
		do
			create Result.make (current_window)
			Result.reset_to_default
			Result.set_background_color (background_color)
			Result.set_title (title)
			set_active (Result)
		end
	
	die_overhead_parameters_interface (run_id, title: STRING): RC_DIE_OVERHEAD_PARAMETERS_WINDOW
			-- Interface with `title' to allow user to set
			-- die overhead parameters
		do
			create Result.make (current_window)
			Result.reset_to_default
			Result.set_background_color (background_color)
			Result.set_title (title)
			set_active (Result)
		end
	
	packing_creation_interface (title: STRING): RC_PACKING_CREATION_WINDOW
			-- Interface with `title' to create new packings
		do
			create Result.make (current_window)
			Result.reset_to_default
			Result.set_background_color (background_color)
			Result.set_title (title)
			set_active (Result)
		end
	
	packing_selection_interface (title: STRING): RC_PACKING_SELECTION_WINDOW
			-- Interface with `title' to select existing packing
			-- to  edited
		do
			create Result.make (current_window)
			Result.reset_to_default
			Result.set_background_color (background_color)
			Result.set_title (title)
			set_active (Result)
		end
	
	packing_confirmation_interface (a_window: EV_TITLED_WINDOW run_id, packing_id, title: STRING): RC_PACKING_CONFIRMATION_WINDOW
			-- Interface with `title' to confirm packing selection
			-- for run `run_id' (or for packing identified by
			-- `packing_id').  Display interface in `a_window'
			-- if not Void, otherwise in current window
		do
			if a_window /= Void then
				create Result.make (a_window)
				active_window_list.register (a_window)
			else
				create Result.make (current_window)
			end
			Result.reset_to_default
			Result.set_background_color (background_color)
			Result.set_title (title)
			Result.set_run_id (run_id)
			Result.set_packing_id (packing_id)
			set_active (Result)
		end
	
	packing_interface (a_window: EV_TITLED_WINDOW run_id, packing_id, title: STRING): RC_PACKING_WINDOW
			-- Interface with `title' to do packing
			-- activities for run `run_id' (or for packing
			-- identified by `packing_id').  Display
			-- interface in `a_window' if not Void,
			-- otherwise in current window
		local
			w: like current_window
		do
			if a_window /= Void then
				w := a_window
			else
				w := current_window
			end
			create Result.make (w)
			Result.reset_to_default
			Result.set_background_color (background_color)
			Result.set_title (title)
			Result.set_run_id (run_id)
			Result.set_packing_id (packing_id)
			set_active (Result)
		end
	
	packing_versions_interface (a_window: EV_TITLED_WINDOW run_id, packing_id, title: STRING): RC_PACKING_VERSIONS_WINDOW
			-- Interface with `title' to select version packing
			-- for run `run_id' (or for packing identified by
			-- `packing_id')
		do
			if a_window /= Void then
				create Result.make (a_window)
				active_window_list.register (a_window)
			else
				create Result.make (current_window)
			end
			Result.reset_to_default
			Result.set_background_color (background_color)
			Result.set_title (title)
			set_active (Result)
		end
	
	packing_projects_interface (a_window: EV_TITLED_WINDOW run_id, packing_id, title: STRING): RC_PACKING_PROJECTS_WINDOW
			-- Interface with `title' to manage projects
			-- while packing for run `run_id' (or for
			-- packing identified by `packing_id')
		do
			if a_window /= Void then
				create Result.make (a_window)
				active_window_list.register (a_window)
			else
				create Result.make (current_window)
			end
			Result.reset_to_default
			Result.set_background_color (background_color)
			Result.set_title (title)
			set_active (Result)
		end
	
	packing_payload_sizes_interface (a_window: EV_TITLED_WINDOW run_id, packing_id, title: STRING): RC_PACKING_PAYLOAD_SIZES_WINDOW
			-- Interface with `title' to view/set payload
			-- and field sizes while packing for run
			-- `run_id' (or for packing identified by
			-- `packing_id')
		do
			if a_window /= Void then
				create Result.make (a_window)
				active_window_list.register (a_window)
			else
				create Result.make (current_window)
			end
			Result.reset_to_default
			Result.set_background_color (background_color)
			Result.set_title (title)
			set_active (Result)
		end
	
	packing_scribes_interface (a_window: EV_TITLED_WINDOW run_id, packing_id, title: STRING): RC_PACKING_SCRIBES_WINDOW
			-- Interface with `title' to set properties of
			-- scribes while packing for run `run_id' (or
			-- for packing identified by `packing_id')
		do
			if a_window /= Void then
				create Result.make (a_window)
				active_window_list.register (a_window)
			else
				create Result.make (current_window)
			end
			Result.reset_to_default
			Result.set_background_color (background_color)
			Result.set_title (title)
			set_active (Result)
		end
	
	packing_filler_interface (a_window: EV_TITLED_WINDOW run_id, packing_id, title: STRING): RC_PACKING_FILLER_WINDOW
			-- Interface with `title' to set properties of
			-- a new filler while packing for run `run_id' (or for
			-- packing identified by `packing_id')
		do
			if a_window /= Void then
				create Result.make (a_window)
				active_window_list.register (a_window)
			else
				create Result.make (current_window)
			end
			Result.reset_to_default
			Result.set_background_color (background_color)
			Result.set_title (title)
			set_active (Result)
		end
	
	packing_delete_interface (a_window: EV_TITLED_WINDOW run_id, packing_id, title: STRING): RC_PACKING_DELETE_WINDOW
			-- Interface with `title' to allow user to control
			-- disposition of customer projects whose last copy
			-- is deleted while packing for run `run_id' (or for
			-- packing identified by `packing_id')
		do
			if a_window /= Void then
				create Result.make (a_window)
				active_window_list.register (a_window)
			else
				create Result.make (current_window)
			end
			Result.reset_to_default
			Result.set_background_color (background_color)
			Result.set_title (title)
			set_active (Result)
		end
	
	packing_move_absolute_interface (a_window: EV_TITLED_WINDOW run_id, packing_id, title: STRING): RC_PACKING_MOVE_ABS_WINDOW
			-- Interface with `title' to allow user to
			-- specify field and location for absolute
			-- move while packing for run `run_id' (or for
			-- packing identified by `packing_id')
		do
			if a_window /= Void then
				create Result.make (a_window)
				active_window_list.register (a_window)
			else
				create Result.make (current_window)
			end
			Result.reset_to_default
			Result.set_background_color (background_color)
			Result.set_title (title)
			set_active (Result)
		end
	
	packing_move_relative_interface (a_window: EV_TITLED_WINDOW run_id, packing_id, title: STRING): RC_PACKING_MOVE_REL_WINDOW
			-- Interface with `title' to allow user to
			-- specify distance for relative
			-- move while packing for run `run_id' (or for
			-- packing identified by `packing_id')
		do
			if a_window /= Void then
				create Result.make (a_window)
				active_window_list.register (a_window)
			else
				create Result.make (current_window)
			end
			Result.reset_to_default
			Result.set_background_color (background_color)
			Result.set_title (title)
			set_active (Result)
		end
	
	packing_output_interface (a_window: EV_TITLED_WINDOW run_id, packing_id, title: STRING): RC_PACKING_OUTPUT_WINDOW
			-- Interface with `title' to display output
			-- from packer operations while packing for
			-- run `run_id' (or for packing identified by
			-- `packing_id')
		do
			if a_window /= Void then
				create Result.make (a_window)
				active_window_list.register (a_window)
			else
				create Result.make (current_window)
			end
			Result.reset_to_default
			Result.set_background_color (background_color)
			Result.set_title (title)
			set_active (Result)
		end
	
	packing_info_interface (a_window: EV_TITLED_WINDOW run_id, packing_id, title: STRING): RC_PACKING_INFO_WINDOW
			-- Interface with `title' to display information
			-- about selected projects while packing for
			-- run `run_id' (or for packing identified by
			-- `packing_id')
		do
			if a_window /= Void then
				create Result.make (a_window)
				active_window_list.register (a_window)
			else
				create Result.make (current_window)
			end
			Result.reset_to_default
			Result.set_background_color (background_color)
			Result.set_title (title)
			set_active (Result)
		end
	
	packing_lot_split_colors_interface (a_window: EV_TITLED_WINDOW run_id, packing_id, title: STRING): RC_PACKING_SPLIT_COLORS_WINDOW
			-- Interface with `title' to allow user to view and
			-- set colors for lot splits  packing for
			-- run `run_id' (or for packing identified by
			-- `packing_id')
		do
			if a_window /= Void then
				create Result.make (a_window)
				active_window_list.register (a_window)
			else
				create Result.make (current_window)
			end
			Result.reset_to_default
			Result.set_background_color (background_color)
			Result.set_title (title)
			set_active (Result)
		end
	
	packing_application_colors_interface (a_window: EV_TITLED_WINDOW run_id, packing_id, title: STRING): RC_PACKING_APP_COLORS_WINDOW
			-- Interface with `title' to allow user to view and
			-- set application colors for packing for
			-- run `run_id' (or for packing identified by
			-- `packing_id')
		do
			if a_window /= Void then
				create Result.make (a_window)
				active_window_list.register (a_window)
			else
				create Result.make (current_window)
			end
			Result.reset_to_default
			Result.set_background_color (background_color)
			Result.set_title (title)
			set_active (Result)
		end
	
	packing_lot_assign_interface (a_window: EV_TITLED_WINDOW run_id, packing_id, title: STRING): RC_PACKING_LOT_ASSIGN_WINDOW
			-- Interface with `title' to allow user to assign
			-- a lot to projects for packing for
			-- run `run_id' (or for packing identified by
			-- `packing_id')
		do
			if a_window /= Void then
				create Result.make (a_window)
				active_window_list.register (a_window)
			else
				create Result.make (current_window)
			end
			Result.reset_to_default
			Result.set_background_color (background_color)
			Result.set_title (title)
			set_active (Result)
		end
	
	packing_lot_summary_interface (a_window: EV_TITLED_WINDOW run_id, packing_id, title: STRING): RC_PACKING_LOT_SUMMARY_WINDOW
			-- Interface with `title' to display summary of
			-- project lot compatibility for packing for
			-- run `run_id' (or for packing identified by
			-- `packing_id')
		do
			if a_window /= Void then
				create Result.make (a_window)
				active_window_list.register (a_window)
			else
				create Result.make (current_window)
			end
			Result.reset_to_default
			Result.set_background_color (background_color)
			Result.set_title (title)
			set_active (Result)
		end
	
	packing_search_interface (a_window: EV_TITLED_WINDOW run_id, packing_id, title: STRING): RC_PACKING_SEARCH_WINDOW
			-- Interface with `title' to allow searching
			-- for selected projects while packing for
			-- run `run_id' (or for packing identified by
			-- `packing_id')
		do
			if a_window /= Void then
				create Result.make (a_window)
				active_window_list.register (a_window)
			else
				create Result.make (current_window)
			end
			Result.reset_to_default
			Result.set_background_color (background_color)
			Result.set_title (title)
			set_active (Result)
		end
	
	packing_help_interface (a_window: EV_TITLED_WINDOW run_id, packing_id, title: STRING): RC_PACKING_HELP_WINDOW
			-- Interface with `title' to provide help for
			-- packer while packing for run `run_id' (or
			-- for packing identified by `packing_id')
		do
			if a_window /= Void then
				create Result.make (a_window)
				active_window_list.register (a_window)
			else
				create Result.make (current_window)
			end
			Result.reset_to_default
			Result.set_background_color (background_color)
			Result.set_title (title)
			set_active (Result)
		end
	
	cut_pattern_properties_interface (a_window: EV_TITLED_WINDOW run_id, packing_id, title: STRING): RC_CUT_PATTERN_PROPERTIES_WINDOW
			-- Interface with `title' to set properties of
			-- a cut pattern while packing for run
			-- `run_id' (or for packing identified by
			-- `packing_id')
		do
			if a_window /= Void then
				create Result.make (a_window)
				active_window_list.register (a_window)
			else
				create Result.make (current_window)
			end
			Result.reset_to_default
			Result.set_background_color (background_color)
			Result.set_title (title)
			set_active (Result)
		end
	
	wafer_lot_creation_interface (run_id, title: STRING): WAFER_LOT_CREATION_WINDOW
			-- Interface with `title' to allow user to create
			-- initial wafer lots for `run_id'
		do
			create Result.make (current_window)
			Result.reset_to_default
			Result.set_background_color (background_color)
			Result.set_title (title)
			set_active (Result)
		end
	
	packaging_lot_creation_interface (run_id, title: STRING): PACKAGING_LOT_CREATION_WINDOW
			-- Interface with `title' to allow user to create
			-- initial packaging lots for `run_id'
		do
			create Result.make (current_window)
			Result.reset_to_default
			Result.set_background_color (background_color)
			Result.set_title (title)
			set_active (Result)
		end
	
	package_info_interface (a_window: EV_TITLED_WINDOW run_id, title: STRING): PACKAGE_INFO_WINDOW
			-- Interface with `title' to display information
			-- about packaging for projects on
			-- run `run_id'
		do
			if a_window /= Void then
				create Result.make (a_window)
				active_window_list.register (a_window)
			else
				create Result.make (current_window)
			end
			Result.reset_to_default
			Result.set_background_color (background_color)
			Result.set_title (title)
			set_active (Result)
		end
	
	bonding_interface (run_id, title: STRING): RC_BONDING_WINDOW
			-- Interface with `title' to do bonding activities
			-- for run `run_id' (not constrained to a particular
			-- run if Void)
		do
			create Result.make (current_window)
			Result.reset_to_default
			Result.set_background_color (background_color)
			Result.set_title (title)
			Result.set_run_id (run_id)
			set_active (Result)
		end
	
	wafer_map_print_interface (run_id, title: STRING): WAFER_MAP_PRINT_WINDOW
			-- Interface with `title' to print maps
			-- for run `run_id'
		do
			create wafer_map_print_window
			active_window_list.register (wafer_map_print_window)
			create Result.make (wafer_map_print_window)
			Result.reset_to_default
			Result.set_background_color (background_color)
			Result.set_title (title)
			set_active (Result)
		end
	
	bond_diagram_print_interface (run_id, title: STRING): BOND_DIAGRAM_PRINT_WINDOW
			-- Interface with `title' to print maps
			-- for run `run_id'
		do
			create bond_diagram_print_window
			active_window_list.register (bond_diagram_print_window)
			create Result.make (bond_diagram_print_window)
			Result.reset_to_default
			Result.set_background_color (background_color)
			Result.set_title (title)
			set_active (Result)
		end
	
	bond_diagram_list_print_interface (title: STRING): BOND_DIAGRAM_LIST_PRINT_WINDOW
			-- Interface with `title' to print list of bonding
			-- diagrams
		do
			create bond_diagram_list_print_window
			active_window_list.register (bond_diagram_list_print_window)
			create Result.make (bond_diagram_list_print_window)
			Result.reset_to_default
			Result.set_background_color (background_color)
			Result.set_title (title)
			set_active (Result)
		end
	
	job_file_print_interface (run_id, title: STRING): JOB_FILE_PRINT_WINDOW
			-- Interface with `title' to print job files
			-- for run `run_id'
		do
			create job_file_print_window
			active_window_list.register (job_file_print_window)
			create Result.make (job_file_print_window)
			Result.reset_to_default
			Result.set_background_color (background_color)
			Result.set_title (title)
			set_active (Result)
		end
	
	bond_diagram_prep_interface (run_id, title: STRING): BOND_DIAGRAM_PREP_WINDOW
			-- Interface with `title' to prepare final bonding
			-- diagrams for run `run_id' and make them available
			-- to customers
		do
			create Result.make (current_window)
			Result.reset_to_default
			Result.set_background_color (background_color)
			Result.set_title (title)
			set_active (Result)
		end
	
	posted_notification_interface (run_id, title: STRING): SUBM_MESSAGE_WINDOW
			-- Interface with `title' to notify customers
			-- for run `run_id' that their projects have been
			-- sent to foundry for review
		do
			create Result.make (current_window)
			Result.reset_to_default
			Result.set_background_color (background_color)
			Result.set_title (title)
			set_active (Result)
		end
	
	rtm_notification_interface (run_id, title: STRING): RTM_MESSAGE_WINDOW
			-- Interface with `title' to notify customers
			-- for run `run_id' that their projects have been
			-- released to manufacturing
		do
			create Result.make (current_window)
			Result.reset_to_default
			Result.set_background_color (background_color)
			Result.set_title (title)
			set_active (Result)
		end
	
	handle_unimplemented_function (s: STRING)
			-- Handle unimplemented function described by `s'
		do
			show_not_implemented_dialog (current_window, s)
		end
	
	show_not_implemented_dialog (w: EV_TITLED_WINDOW msg_text: STRING)
			-- Display modal relative to `w' a "not implemented"
			-- dialog with error message `msg_text'
		require
			window_exists: w /= Void
			window_active: not w.is_destroyed
			message_text_exists: msg_text /= Void
		local
			not_imp_dialog: NOT_IMPLEMENTED_DIALOG
			ok_button: EV_BUTTON
		do
			create not_imp_dialog.make_with_text (msg_text)
			not_imp_dialog.set_background_color (background_color)
			not_imp_dialog.set_buttons (<< gui.ev_ok >>)
			ok_button := not_imp_dialog.button (gui.ev_ok)
			not_imp_dialog.set_default_push_button (ok_button)
			-- not_imp_dialog.show_actions.extend (agent ok_button.center_pointer)
			not_imp_dialog.show_modal_to_window (w)
		end

	exit_application
			-- Exit the application
		do
			gui.application.destroy
		end
	
	set_active (w: TITLED_WINDOW_PROXY)
			-- Make window for `w' become the current window
			-- relative to which dialogs are shown when
			-- pointer enters window
		local
			win: EV_TITLED_WINDOW
		do
			win := w.window
			win.pointer_enter_actions.extend (agent active_window_list.set_current (win))
		end
	
	set_existing_windows_insensitive (window_list: LINEAR [EV_WINDOW])
			-- Make any windows that exist at time of error
			-- insensitive to user input
		do
			from
				window_list.start
			until
				window_list.after
			loop
				window_list.item.disable_sensitive
				window_list.item.close_request_actions.wipe_out
				window_list.forth
			end
		end
	
	publish_fatal_exception
			-- Publish a fatal exception event
		local
			fatal_event: FATAL_EXCEPTION_EVENT
		do
			create fatal_event
			fatal_event.publish
		end

	last_icon: EV_PIXMAP
			-- Last pixmap created by `read_icon_file'

	step_done_icon: EV_PIXMAP
			-- Icon that indicates step has been completed

	step_inapplicable_icon: EV_PIXMAP
			-- Icon that indicates step was not applicable to run

	step_undone_icon: EV_PIXMAP
			-- Icon that indicates step is not even started

	doing_step_icon: EV_PIXMAP
			-- Icon that indicates we have started doing step

	undoing_step_icon: EV_PIXMAP
			-- Icon that indicates we have started undoing step

	step_saved_state_icon: EV_PIXMAP
			-- Icon that indicates that step has been completely
			-- undone, but we have saved state from the last
			-- time we did it

end
