
indexing
	
	description: "Checker for new Delta version"
	author: "David Hollenberg"
	date: "April 15, 2009"

class DELTA_VERSION_CHECKER
inherit
	ANY
		redefine
			default_create
		end
	CONDITION_UTILITIES
		redefine
			default_create
		end
	EXCEPTIONS
		redefine
			default_create
		end
	UNIX_OS_ACCESS
		redefine
			default_create
		end
	RC_ENVIRONMENT_ACCESS
		redefine
			default_create
		end

feature {NONE} -- Creation

	default_create
		do
			create {DELTA_CURRENT_VERSION} current_version
			create {DELTA_LATEST_VERSION} latest_version
		end

feature -- Operations

	check_for_new_version
			-- Check whether there is a new
			-- version of Delta available.  If so,
			-- inform user and offer to restart with
			-- new version.  Set `abort_requested' to
			-- indicate whether user wants to abort
			-- operation that is about to be started
		local
			operation: ANONYMOUS_OPERATION
			now: like last_refresh_time
		do
			abort_requested := False
			if rc_env.is_production and not user_notified then
				now := unix_os.current_time_in_seconds
				if now - latest_version.refresh_time  >= Refresh_interval then
					latest_version.refresh
					
				end
				if not current_version.number.is_equal (latest_version.number) then
					user_notified := True
					create operation.make (agent ask_about_version_switch)
					operation.enable_abort_caught
					operation.execute
					abort_requested := operation.aborted
					if not abort_requested and wants_to_switch then
						try_to_restart_delta
					end
				end
			end
		end
	
feature -- Properties

	abort_requested: BOOLEAN
			-- Did user request that operation we're
			-- about to start be aborted?

feature {NONE} -- Implementation

	ask_about_version_switch
		do
			wants_to_switch := False
				-- Initialize in case user clicks Cancel
			wants_to_switch := question ("A new version of Delta (rev " + latest_version.number + ") is available.  Switch to it now?")
		end

	try_to_restart_delta
		local
			process: UNIX_PROCESS
			fname: STRING
			f: LARGE_RAW_FILE
		do
			fname := rc_env.delta_program
			create f.make (fname)
			if not f.exists then
				inform ("Cannot switch to new Delta version - file " + fname + " not found")
			elseif not f.is_executable then
				inform ("Cannot switch to new Delta version - file " + fname + " is not executable")
			else
				process := unix_os.new_process (fname)
				process.set_close_nonstandard_files (True)
				process.spawn_nowait
				die (0)
			end
		end

	current_version: DELTA_VERSION
			-- Version of Delta currently being run

	latest_version: DELTA_VERSION
			-- Latest available Version of Delta

	user_notified: BOOLEAN
			-- Has user been notified about availability
			-- of a new version?

	wants_to_switch: BOOLEAN
			-- Does user want to switch to new version?

	last_refresh_time: INTEGER
			-- Time in seconds since start of epoch
			-- when `latest_version' was last refreshed

	Refresh_interval: INTEGER = 600
			-- Interval in seconds between refreshes of
			-- `latest_version'

end
