class DESIGN_TECHN_LAYER
	-- a DESIGN_TECHN_ID_LAYERS_ROW and everything else about that
	--  layer that is in the database

inherit
	DESIGN_TECHN_ID_LAYERS_ROW
		-- the basic lambda-independent stuff

	LAYER
		-- the generic LAYOUT layer idiom

	STRING_UTILITIES_ACCESS

	MOSIS_TECH_VALUES

create
	make_template, make_null

feature -- the other database rows about us

	cif: LIST[CIFXDESIGN_TECHN_ID_LAYERS_ROW]

	gds: LIST[GDSXDESIGN_TECHN_ID_LAYERS_ROW]

	assoc_layers: LIST[DESIGN_TECHN_LAYERXLAYERS_ROW]

	dimensions: DESIGN_TECHN_LAYERS_ROW

feature -- cif and gds attributes (deferred from LAYER)

	have_cif_name: BOOLEAN is
		-- do I have any cif names?
	do  Result := not cif.is_empty
	end;

	is_my_cif_name(c: STRING): BOOLEAN is
		-- is c one of my cif names?
	do  from  cif.start
	    until Result or cif.after
	    loop  Result := equal(c, cif.item.cif_layer)
		  cif.forth
	    end;
	end;

	a_cif_name: STRING is
		-- (any one of) my cif name(s)
	do  Result := cif.first.cif_layer
	end;

	have_gds_name: BOOLEAN is
	do  Result := not gds.is_empty
	end;

	is_my_gds_name(layer, datatype: INTEGER): BOOLEAN is
		-- is (layer, datatype) one of my gds names?
		-- datatype is an actual type value
	local  git: GDSXDESIGN_TECHN_ID_LAYERS_ROW
	do  from  gds.start
	    until Result or gds.after
	    loop  git := gds.item;
		  Result := layer = git.gds_layer and
			    (git.is_null_gds_datatype or else
				datatype = git.gds_datatype)
		  gds.forth
	    end;
	end;

	a_gds_layer: INTEGER is
		-- with a_gds_datatype, (one of my) gds names
	do  Result := gds.first.gds_layer
	end;

	a_gds_datatype: INTEGER is
		-- with a_gds_layer, (one of my) gds names
	do  -- we depend on is_null_gds_datatype having a zero gds_datatype
	    Result := gds.first.gds_datatype
	end;

feature -- stuff for working with Cats

	layer_list_for_Cats(src: LIST[VIRTUAL_GDS_PAIR]): STRING is
	do  create Result.make(8*src.count);
	    from  src.start;
	    until src.after
	    loop  Result.append(src.item.out)
		  Result.extend(' ');
		  src.forth
	    end
	end;

	gds_layer_list_for_Cats: STRING is
		-- a string in Cats syntax for the selection of all my gds
		-- no attempt is made to optimize the presentation
	do  Result := layer_list_for_Cats (gds)
	end;

	cif_as_pseudo_gds: LIST[VIRTUAL_GDS_PAIR] is
		-- list (1 item long) of the "gds_pair" for cif
	local gpr: GDS_PAIR;
	do  create gpr.make_integer(layer_order);
	    create {LINKED_LIST[VIRTUAL_GDS_PAIR]}Result.make
	    Result.extend(gpr)
	end;

	cif_layer_list_for_Cats: STRING is
		-- a string in Cats syntax for my layer_order as a
		-- pseudo-gds layer/datatype
	do  Result := layer_list_for_Cats (cif_as_pseudo_gds)
	end;

	write_clay_for_layer(cfil: LARGE_PLAIN_TEXT_FILE) is
		-- add line(s) to this clay file for this layer's names
		-- (our "gds" is derived from our layer_order; no real
		--  gds values are used)
	local gnam: STRING; crow: CIFXDESIGN_TECHN_ID_LAYERS_ROW
	do  gnam := cif_layer_list_for_Cats;	-- use same for all cif
	    from  cif.start
	    until cif.after
	    loop  crow := cif.item;
		  -- write one line for crow to cfil
		  cfil.putstring(crow.cif_layer);
		  cfil.putchar('%T');
		  cfil.putstring(gnam);
		  cfil.putchar('%T');
		  cfil.putstring(layer_name);
		  cfil.putchar('%N');
		  cif.forth
	    end
	end;

feature	-- access to the design_techn_layers data

	have_dimensional_data: BOOLEAN is
		-- do we have our design_techn_layers row?
	do  Result := dimensions /= Void
	end;

	layer_id: CHARACTER is
		-- our layer_id
	do  if have_dimensional_data
	     then Result := dimensions.layer_id
	    end
	end;

	min_width: DOUBLE is
		-- our min_width
	do  if have_dimensional_data
	     then Result := dimensions.min_width
	    end
	end;

	min_spacing: DOUBLE is
		-- our min_spacing
	do  if have_dimensional_data
	     then Result := dimensions.min_spacing
	    end
	end;

	min_pitch: DOUBLE is
		-- our minimum pitch
	do  Result := min_width + min_spacing
	end;

	is_scribe_element: BOOLEAN is
	do  Result := have_dimensional_data and then
			not (dimensions.is_null_scribe_style and
			     dimensions.is_null_near_edge)
	end;

	near_edge: DOUBLE is
		-- our near_edge
	do  if have_dimensional_data
	     then Result := dimensions.near_edge
	    end
	end;

	far_edge: DOUBLE is
		-- our far_edge
	do  if have_dimensional_data
	     then Result := dimensions.far_edge
	    end
	end;

	scribe_style: STRING is
		-- our scribe_style
	do  if have_dimensional_data
	     then Result := dimensions.scribe_style
	    end
	end;

	cd_style: STRING is
		-- our cd_style
	do  if have_dimensional_data
	     then Result := dimensions.cd_style
	    end
	end;

	is_box_cd_style: BOOLEAN is
		-- (assume not if no data)
	do  Result := equal(cd_style, cd_style_is_box)
	end;

	fill_role: STRING is
		-- our fill_role
	do  if have_dimensional_data
	     then Result := dimensions.fill_role
	     else Result := frole_is_none
	    end
	end;

feature {BUILD_SEAL_RING} -- edge shifter for build_seal_ring

	shift_scribe_dimensions(shift: DOUBLE) is
		-- replace dimensions with a local copy tuned to the
		-- needs of build_seal_ring, which needs to move both 
		-- edges by shift (which may "de-scribe" some layers)
	local ndim: DESIGN_TECHN_LAYERS_MROW
	do  if is_scribe_element then
		ndim := hidden_dimensions.detached_twin
		-- and shift the scribe parameters
		dimensions := hidden_dimensions	-- (restore true values)
		if far_edge > shift then
			ndim.set_near_edge ((near_edge - shift).max(0));
			ndim.set_far_edge (far_edge - shift)
		else	-- shifted out of seal_ring
			ndim.set_null_scribe_style
			ndim.set_null_near_edge
			ndim.set_null_far_edge
		end;
		-- now make this our dimensions!!
		dimensions := ndim
	    end;
	end;

feature -- access to the associated_layer data

	cd_background_layers: LIST[STRING] is
	do  Result := associated_layers(assoc_is_bkgrnd)
	end;

	alignment_target_layers: LIST[STRING] is
	do  Result := associated_layers(assoc_is_target)
	end;

feature {NONE} -- implementation

	associated_layers(typ: STRING): LIST[STRING] is
	local alyr: DESIGN_TECHN_LAYERXLAYERS_ROW
	do  create {LINKED_LIST[STRING]} Result.make
	    from  assoc_layers.start
	    until assoc_layers.after
	    loop  alyr := assoc_layers.item;
		  if equal(alyr.association_type, typ)
		   then Result.extend(alyr.associated_layer_name)
		  end;
		  assoc_layers.forth
	    end
	end;

feature	{DESIGN_TECHNOLOGY} -- get everything else

	establish(dt: STRING) is
		-- get everything else for this layer
		-- (including design_techn_layers_row, which needs a specific design_techn)
	local	b: DB_BIND_SS;
		rc: CIFXDESIGN_TECHN_ID_LAYERS_ROW;
		rg: GDSXDESIGN_TECHN_ID_LAYERS_ROW;
		ra: DESIGN_TECHN_LAYERXLAYERS_ROW;
		rd: DESIGN_TECHN_LAYERS_MROW;
		ld: LIST[DESIGN_TECHN_LAYERS_MROW];
	do  -- binder for almost everything
	    create b.make(design_techn_id, layer_name);
	    -- cif names
	    create rc.make_template
	    cif := rc.all_rows_filtered_ordered(connection, f, b, "cif_layer");
	    -- gds names
	    create rg.make_template
	    gds := rg.all_rows_filtered_ordered(connection, f, b, "gds_layer,gds_datatype");
	    -- associated layers
	    create ra.make_template
	    assoc_layers := ra.all_rows_filtered(connection, f, b);
	    -- design_techn_layers
	    create b.make(dt, layer_name);
	    create rd.make_template
	    ld := rd.all_rows_filtered(connection, f2, b);
	    if ld.count = 1
	     then hidden_dimensions := ld.first;
		  dimensions := hidden_dimensions	-- visible copy
	    end;
	end;

feature {NONE} -- stupid details

	hidden_dimensions: DESIGN_TECHN_LAYERS_MROW;
		-- need another copy and it needs to be an MROW
		-- for copy_from_row. DO NOT USE!

	f: STRING = "design_techn_id = :1 and layer_name = :2";

	f2: STRING = "design_techn = :1 and layer_name = :2";

end -- DESIGN_TECHN_LAYER
