
indexing

	author: "Generated"
	date: "Wed Feb 11 12:52:51 PST 2004"

class DESIGN_TECHN_XFORM_LEVELS_ROW
inherit
	DESIGN_TECHN_XFORM_LEVELS_RAW
	ROW_COUNT_CHECK_ACCESS
create
	make_template, make_null

feature	-- bias

	total_bias (overetch: BOOLEAN): DOUBLE is
		-- total bias from data to mebes considering the overetch state
	do  if overetch then
		Result := total_nominal_bias + overetch_bias
	    else
		Result := total_nominal_bias
	    end
	end;

feature	-- filtering tests

	is_my_level_no(a_no: INTEGER): BOOLEAN is
	do  Result := level_no = a_no
	end;

feature -- access

	levels_for_wtf_and_dt (wtf, dt: STRING conn: DB_CONNECTION range: ITEM_COUNT_RANGE): LIST [like Current] is
			-- All transform levels for wafer techn family `wtf', 
			-- design techn `dt', ordered by level number
		require
			family_exists: wtf /= Void
			design_techn_exists: dt /= Void
			connection_open: conn.is_open
			range_exists: range /= Void
		do
			Result := levels_for_two_strings(wtf_filter, wtf, dt, conn)
			check_row_count (Result, range)
		ensure
			result_exists: Result /= Void
		end

	levels_for_wt_and_dt (wt, dt: STRING conn: DB_CONNECTION): LIST [like Current] is
			-- All transform levels for wafer techn `wt', 
			-- design techn `dt', ordered by level number
			-- NOTE: dt MUST be a wafer_design_techn for this wt
		require
			wt_exists: wt /= Void
			design_techn_exists: dt /= Void
			connection_open: conn.is_open
		do
			Result := levels_for_two_strings(wt_filter, wt, dt, conn)
		ensure
			result_exists: Result /= Void
		end

	levels_for_wp_and_dt (wp, dt: STRING conn: DB_CONNECTION): LIST [like Current] is
			-- All transform levels for wafer process `wp', 
			-- design techn `dt', ordered by level number
		require
			wp_exists: wp /= Void
			design_techn_exists: dt /= Void
			connection_open: conn.is_open
		do
			Result := levels_for_two_strings(wp_filter, wp, dt, conn)
		ensure
			result_exists: Result /= Void
		end

	levels_for_wt_and_dt_any_lambda (wt, dt: STRING conn: DB_CONNECTION): LIST [like Current] is
			-- All transform levels for wafer techn `wt' and
			-- design techn `dt', and all of its fellow design_techn_ids,
			-- ordered by level number
			-- NOTE: dt MUST be a wafer_design_techn_id for this wt
		require
			wt_exists: wt /= Void
			design_techn_exists: dt /= Void
			connection_open: conn.is_open
		do
			Result := levels_for_two_strings (wt_all_dts_filter, wt, dt, conn)
		ensure
			result_exists: Result /= Void
		end

	derived_when_absent_layers_for_wt_and_dt (wt, dt: STRING conn: DB_CONNECTION): LIST [STRING] is
			-- List of distinct source layer names for
			-- levels for wafer techn `wt' and design
			-- techn `dt' which are marked "derived when
			-- absent", ordered by layer name
			-- NOTE: dt MUST be a wafer_design_techn_id for this wt
		require
			wt_exists: wt /= Void
			design_techn_exists: dt /= Void
			connection_open: conn.is_open
		local
			list: like levels_for_wt_and_dt_any_lambda
			level: like Current
			layer: STRING
		do
			list := levels_for_wt_and_dt_any_lambda (wt, dt, conn)
			create {SORTED_TWO_WAY_LIST [STRING]} Result.make
			Result.compare_objects
			from
				list.start
			until
				list.after
			loop
				level := list.item
				if level.derived_when_absent then
					layer := level.source_layer_name
					if not Result.has (layer) then
						Result.extend (layer)
					end
				end
				list.forth
			end
		ensure
			result_exists: Result /= Void
		end

feature {NONE} -- common implementation 

	wtf_filter: STRING is
	"wafer_techn_family = :1 and design_techn = :2";

	wt_filter: STRING is
	"(wafer_techn_family, level_no) in (select wafer_techn_family, level_no from wafer_techn_levels where wafer_techn = :1) and design_techn = :2"
	
	wp_filter: STRING is
	"(wafer_techn_family, level_no) in (select wafer_techn_family, level_no from wafer_process_levels where wafer_process = :1) and design_techn = :2"

	wt_all_dts_filter: STRING is
	"(wafer_techn_family, level_no) in (select wafer_techn_family, level_no from wafer_techn_levels where wafer_techn = :1) and design_techn in (select design_techn from design_techns dt2, design_techns dt1 where dt2.design_techn_id = dt1.design_techn_id and dt1.design_techn = :2)"

	levels_for_two_strings (filt, str1, str2: STRING conn: DB_CONNECTION): LIST [like Current]
		local
			b: DB_BIND_SS
		do
			create b.make (str1, str2)
			Result := all_rows_filtered_ordered (conn, filt, b, "level_no")
		ensure
			result_exists: Result /= Void
		end

end
