class DESIGN_TECHNOLOGY
	-- a LAYER_SET as a 
	--  DESIGN TECHNOLOGY (DESIGN_TECHNS_ROW) plus all its baggage
	--  (everything is read-only)

inherit
	LAYER_SET [DESIGN_TECHN_LAYER]

	DESIGN_TECHNS_ROW
	undefine is_equal
	end;

	STRING_UTILITIES_ACCESS
	undefine is_equal
	end;

	DB_EXCEPTION_ACCESS
	undefine is_equal
	end;

	MOSIS_TECH_VALUES
	undefine is_equal
	end;

create
	make_template, make_null, make_detached, make_by_key,
	make_by_dtid_and_lambda

feature -- status

	name: STRING is
	do  if is_scalable
	     then Result := su.string_of_3(design_techn_id,"@",design_lambda)
	     else Result := design_techn_id
	    end
	end;

	lambda: DOUBLE is
		-- lambda as a number
	do  Result := connection.string_to_double(design_lambda)
	end;

	is_my_name(dt: STRING): BOOLEAN is
		-- is dt my design_techn?
	do  Result := equal(dt, design_techn)
	end;

feature -- my layers (deferred from LAYER_SET)

	layer_set: LIST[DESIGN_TECHN_LAYER] is
		-- everything one can know about my layers
	do  if cached_layer_set = Void then get_my_layers end;
	    Result := cached_layer_set
	end;

feature	-- layer locators

	layer_by_layer_order(lord: INTEGER): DESIGN_TECHN_LAYER is
		-- return layer of this layer_order or Void
	local lyr: DESIGN_TECHN_LAYER; lset: like layer_set; pos: CURSOR
	do  lset := layer_set;
	    from  pos := lset.cursor; lset.start
	    until lset.after or Result /= Void
	    loop  lyr := lset.item;
		  if lyr.layer_order = lord
		   then Result := lyr
		  end;
		  lset.forth
	    end;
	    lset.go_to(pos)
	ensure Result = Void or else Result.layer_order = lord
	end;

	layers_of_role(f: FUNCTION [ANY, TUPLE [DESIGN_TECHN_LAYER], BOOLEAN]): LIST[DESIGN_TECHN_LAYER] is
		-- all of my layers that pass f
	do  Result := dtllu.items_passing_filter (layer_set, f)
	end;

	glass_layers: LIST[DESIGN_TECHN_LAYER] is
		-- return list of all layers with is_glass_layer
	do  Result := layers_of_role (agent {DESIGN_TECHN_LAYER}.is_glass_layer)
	end;

	non_cmnt_layers: LIST[DESIGN_TECHN_LAYER] is
		-- return list of all layers that are NOT CMNT layers
	local lyr: DESIGN_TECHN_LAYER; lset: like layer_set; pos: CURSOR
	do  create {LINKED_LIST[DESIGN_TECHN_LAYER]} Result.make;
	    lset := layer_set;
	    from  pos := lset.cursor; lset.start
	    until lset.after
	    loop  lyr := lset.item;
		  if not lyr.is_non_mask_layer
		   then Result.extend(lyr)
		  end;
		  lset.forth
	    end;
	    lset.go_to(pos)
	end;

	layers_for_scribe: LIST[DESIGN_TECHN_LAYER] is
		-- return list of all layers that are scribe lane elements
	local lyr: DESIGN_TECHN_LAYER; lset: like layer_set; pos: CURSOR
	do  create {LINKED_LIST[DESIGN_TECHN_LAYER]} Result.make;
	    lset := layer_set;
	    from  pos := lset.cursor; lset.start
	    until lset.after
	    loop  lyr := lset.item;
		  if lyr.is_scribe_element
		   then Result.extend(lyr)
		  end;
		  lset.forth
	    end;
	    lset.go_to(pos)
	end;

	layer_names_for_scribe: LIST [STRING] is
		-- List of all layer names that are scribe lane elements
	local lyr: DESIGN_TECHN_LAYER; lset: like layer_set; pos: CURSOR
	do  lset := layer_set;
	    create {ARRAYED_LIST [STRING]} Result.make (lset.count)
	    from  pos := lset.cursor; lset.start
	    until lset.after
	    loop  lyr := lset.item;
		  if lyr.is_scribe_element
		   then Result.extend(lyr.layer_name)
		  end;
		  lset.forth
	    end;
	    lset.go_to(pos)
	end;

	throwaway_layer: DESIGN_TECHN_LAYER is
		-- a layer that is likely to be of low density, for
		-- those times where we need to have a layer for no
		-- good reason
	local glass: LIST[DESIGN_TECHN_LAYER]
	do  glass := glass_layers;
	    if glass.is_empty
	     then Result := layer_set.first	-- punt!
	     else Result := glass.first
	    end
	end;

feature	-- highest bond_layer_priority

	highest_bond_layer_priority(lyrs: LIST[STRING]): INTEGER is
		-- for the given list of layer_names, return the
		-- highest (smallest) bond_layer_priority amongst them, 
		-- or 0 if none have any priority
	require lyrs /= Void
	local itm: DESIGN_TECHN_LAYER
	do  from  lyrs.start
	    until lyrs.after
	    loop  itm := layer_by_name(lyrs.item)
		  if itm /= Void and then
		     ((not itm.is_null_bond_layer_priority) and 
		      (Result = 0 or Result > itm.bond_layer_priority))
			 then Result := itm.bond_layer_priority
		  end;
		  lyrs.forth
	    end;
	end;

feature -- list for Cats of all the layers used for taking extent

	all_mbb_layers_for_Cats(use_cif: BOOLEAN): STRING is
		-- use_cif: should we use the real gds or the cif pseudo-gds?
	local lyrs: like layer_set; lyr: DESIGN_TECHN_LAYER; pos: CURSOR
		coll: COMPARATOR_SORTED_LIST[VIRTUAL_GDS_PAIR]
	do  lyrs := layer_set;
	    pos := lyrs.cursor; 
	    create coll.make (agent {VIRTUAL_GDS_PAIR}.three_way_comparison);
	    from  lyrs.start
	    until lyrs.after
	    loop  lyr := lyrs.item;
		  if not lyr.is_mbb_layer
		   then	-- skip this one
		  elseif use_cif
		   then coll.insert_list(lyr.cif_as_pseudo_gds)
		   else coll.insert_list(lyr.gds)
		  end;
		  lyrs.forth
	    end;
	    lyrs.go_to(pos)
	    Result := coll.first.list_as_Cats_string(coll)
	end;

feature	-- clay file output and creation

	write_clay_file_body(cfil: LARGE_PLAIN_TEXT_FILE) is
		-- write clay file body to cfil 
		-- (which we neither open nor close)
	local pos: CURSOR
	do  cfil.putstring("CLAY 1.0%N");
	    pos := layer_set.cursor;
	    layer_set.do_all
		(agent {DESIGN_TECHN_LAYER}.write_clay_for_layer(cfil));
	    layer_set.go_to(pos)
	end;

	write_clay_file(cnam: STRING) is
		-- create a (Cats) .clay file for this technology.
		-- every defined cif name is translated to its
		--  own layer_order
	local cfil: LARGE_PLAIN_TEXT_FILE;
	do  create cfil.make_open_write(cnam);
	    write_clay_file_body(cfil);
	    cfil.close
	end;

feature -- my layer_errors

	dt_layer_errors: LIST[DESIGN_TECHNXLAYER_ERRORS_ROW] is
	local r: DESIGN_TECHNXLAYER_ERRORS_ROW; b: DB_BIND_S
	do  if cached_errors = Void
	     then create r.make_template;
		  create b.make(design_techn_id);
		  cached_errors := r.all_rows_filtered
					(connection, 
					 "design_techn_id = :1", b)
	    end;
	    Result := cached_errors
	end;

feature {NONE}

	cached_errors: like dt_layer_errors

feature {NONE} -- implementation

	cached_layer_set: LIST[DESIGN_TECHN_LAYER]

	get_my_layers is
	require cached_layer_set = Void
	local b: DB_BIND_S;  r: DESIGN_TECHN_LAYER;
		role: LAYER_ROLES_ROW; roles: LIST[LAYER_ROLES_ROW];
	do  create b.make(design_techn_id);
	    create r.make_template;
	    cached_layer_set := r.all_rows_filtered_ordered
				(connection,
				 "design_techn_id = :1", b,
				 "layer_order");
	    cached_layer_set.do_all
				(agent {DESIGN_TECHN_LAYER}.establish (design_techn));
	    -- now fetch all the roles and let them each pick one
	    create role.make_template
	    roles := role.all_rows(connection)
	    cached_layer_set.do_all
				(agent {DESIGN_TECHN_LAYER}.select_roles_row
						(roles));
	ensure cached_layer_set /= Void
	end;

	dtllu: LIST_UTILITIES[DESIGN_TECHN_LAYER] is
	once create Result
	end

end -- DESIGN_TECHNOLOGY
