class DESIGN_TECHNXWAFER_TECHNS_ROW
	-- this class combines a design_techn#wafer_techns row
	-- with its associated wafer_techns (the view) row

inherit
	DESIGN_TECHNXWAFER_TECHNS_RAW
	ROW_COUNT_CHECK_ACCESS
	NFE_SUPPORT

create
	make_template, make_null

feature	-- my design_techns and wafer_design_techns rows

	dt_row: DESIGN_TECHNS_ROW
	do  if cached_dt_row = Void then
		create cached_dt_row.make_by_key
				(connection, old_row, design_techn)
	    end;
	    Result := cached_dt_row
	end;

	wdt_row: DESIGN_TECHNS_ROW
	do  if cached_wdt_row = Void then
		create cached_wdt_row.make_by_key
				(connection, old_row, wafer_design_techn)
	    end;
	    Result := cached_wdt_row
	end;

	design_techn_id: STRING
	do  Result := dt_row.design_techn_id
	end;

feature -- Retrieval

	row_for_dt_and_wt (dt, wt: STRING conn: DB_CONNECTION range: ITEM_COUNT_RANGE): like Current is
			-- Row for design techn `dt' and wafer
			-- techn `wt'
		require
			dt_not_void: dt /= Void
			wt_not_void: wt /= Void
			connection_exists: conn /= Void
			connection_open: conn.is_open
			range_exists: range /= Void
		local
			list: LIST [like Current]
			b: DB_BIND_SS
			filter: STRING
		do
			filter := "dtwt.wafer_techn = :1 and design_techn = :2"
			create {DB_BIND_SS} b.make (wt, dt)
			list := all_rows_filtered (conn, filter, b)
			check_row_count (list, range)
			if not list.is_empty then
				Result := list.first
			end
		end

feature	-- is_native_design_techn

	is_native_design_techn: BOOLEAN
		-- is this design techn "native" to this wafer techn?
	do  Result := wdt_row.org_no = org_no
	end;

feature	-- my own wafer_techn_layer_errors results

	is_wt_error: BOOLEAN is
	do  Result := my_layer_error /= Void
	end;

	is_reportable_wt_error: BOOLEAN is
	do  Result := is_wt_error and then my_layer_error.reportable
	end;

	wt_error_message: STRING is
	do  if is_wt_error
	     then Result := my_layer_error.descr
	    end
	end;

	my_layer_error: WT_LAYER_ERRORS_ROW

feature -- options_list stuff

	wtfo_list: DB_OPTIONS_LIST[WAFER_TECHN_FAMILY_OPTIONS_ROW] is
		-- all of the options in my wafer_techn_family
	do  if wtfo_cached_list = Void then
	    create wtfo_cached_list.make_by_query(wtfo_list_query);
	    end;
	    Result := wtfo_cached_list;
	ensure Result /= Void
	end;

	is_valid_family_option_name(option_nam: STRING): BOOLEAN is
	do  Result := wtfo_list.has_name(option_nam)
	end;

	is_valid_family_option_id(option_id: STRING): BOOLEAN is
	do  Result := wtfo_list.has_id(option_id)
	end;

	is_valid_family_option_list
		(options: LIST[IC_FAB_INSTANCE_OPTIONS_ROW]): BOOLEAN is
	do
    	    from  options.start; Result := true
	    until options.after or not Result
	    loop  Result := is_valid_family_option_id(options.item.option_id);
		  options.forth;
	    end;
	end;

feature {NONE}

	wtfo_cached_list: DB_OPTIONS_LIST[WAFER_TECHN_FAMILY_OPTIONS_ROW];

	wtfo_list_query: ANONYMOUS_QUERY[WAFER_TECHN_FAMILY_OPTIONS_ROW] is
	local r: WAFER_TECHN_FAMILY_OPTIONS_ROW; b: DB_BIND_S;
	do  create r.make_template;
	    create Result.make(connection,r);
	    create b.make(wafer_techn_family);
	    Result.set_f_and_b(
	      "WTFO.wafer_techn_family=:1", b);
	end;

feature	-- support for wafer_processes in this wafer_techn

	cheapest_process: WAFER_TECHN_PROCESSES_ROW is
		-- the cheapest (least relative_cost) process for this design
	do  if qualified_processes /= Void and then 
	       not qualified_processes.is_empty then
		Result := qualified_processes.first
	    end;
	end;

	all_etest_ids: LIST[STRING] is
		-- list of all the distinct wafer_etest_ids in qualified_processes
		-- (usually just one)
	local eid: STRING
	do  create {LINKED_LIST[STRING]} Result.make
	    Result.compare_objects
	    if qualified_processes /= Void then
		from  qualified_processes.start
		until qualified_processes.after
		loop  eid := qualified_processes.item.wafer_etest_id
		      if not Result.has(eid) then Result.extend(eid) end;
		      qualified_processes.forth
		end
	    end
	end;

	qualified_processes: DYNAMIC_LIST[WAFER_TECHN_PROCESSES_ROW]
		-- all of the qualified processes for this design

	process_selection_hint: STRING
		-- a human readable hint as to the cause of process
		-- selection failure; probably not meaningful unless
		-- qualified_processes is empty

feature	-- set all of the above, given a list of layers and required options

	find_techn_and_process_compatibility
			(lyrs: LIST[ABSTRACT_LAYER];
			 opts: LIST[IC_FAB_INSTANCE_OPTIONS_ROW]) is
		-- someone has to call here first with a layer set
		-- and a required option set
	local	wtles: LIST[WT_LAYER_ERRORS_ROW]; wtle: WT_LAYER_ERRORS_ROW;
		worqs: LIST[WAFER_OPTION_REQUIREMENTS_ROW];
		proc_opts: ARRAYED_LIST[ABSTRACT_PROCESS_OPTION];
	do  create process_selection_hint.make(2000);
	    -- first, see if I have any wafer_techn_layer_errors
	    wtles := wt_layer_errors;
	    from  wtles.start; my_layer_error := Void
	    until wtles.after or is_wt_error
	    loop  wtle := wtles.item
		  if not wtle.is_kosher(lyrs)
		   then my_layer_error := wtle;
			process_selection_hint.append(my_layer_error.descr);
			process_selection_hint.extend('%N');
		  end;
		  wtles.forth
	    end;
	    -- second, find our required (and prohibited) options
	    create proc_opts.make(50);	
	    proc_opts.append(opts);	-- explicit options
	    worqs := fetch_our_worqs;
	    from  worqs.start		-- plus all worqs that trigger
	    until worqs.after
	    loop  if worqs.item.is_kosher(lyrs)
		   then	-- doesn't have layer_error, so doesn't have
			-- a requirement re this option
		   else	-- has layer_error; needs (or not) the option
			proc_opts.extend(worqs.item)
		  end;
		  worqs.forth
	    end;
	    -- third, look into wafer_techn_offered_processes
	    if not is_wt_error
	     then -- get all of them
		  qualified_processes := fetch_our_processes;
		  -- then remove the explicitly disqualified
		  prune_qualified_processes(proc_opts);
		  my_pricing_sched := Void;	-- reset
	    end;
	end;

feature {NONE} -- help for the above

	fetch_our_worqs: LIST[WAFER_OPTION_REQUIREMENTS_ROW] is
	local r: WAFER_OPTION_REQUIREMENTS_ROW; b: DB_BIND_SS;
	do  create r.make_template
	    create b.make(wafer_techn_family, design_techn_id);
	    Result := r.all_rows_filtered
				(connection, required_options_filter, b)
	end;

	required_options_filter: STRING is
	"wafer_techn_family = :1 and design_techn_id = :2"

	fetch_our_processes: DYNAMIC_LIST[WAFER_TECHN_PROCESSES_ROW] is
	local r: WAFER_TECHN_PROCESSES_ROW; b: DB_BIND_SS;
	do  create r.make_template
	    create b.make(wafer_techn_family, wafer_techn_id);
	    Result := r.all_rows_filtered_ordered
				(connection, offered_process_filter, b,
				 "relative_cost")
	end;

	offered_process_filter: STRING is
	"wtofproc.wafer_techn_family = :1 and wafer_techn_id = :2";

	wt_layer_errors: LIST[WT_LAYER_ERRORS_ROW] is
	local r: WT_LAYER_ERRORS_ROW;
		b: DB_BIND_SS;
	do  create r.make_template;
	    create b.make(wafer_techn, design_techn_id);
	    Result := r.all_rows_filtered
			(connection,
			 "wafer_techn = :1 and design_techn_id = :2",
			 b)
	end;

	prune_qualified_processes
			(proc_opts: LIST[ABSTRACT_PROCESS_OPTION]) is
		-- remove from qualified processes all processes
		-- that are not qualified by the proc_opts
	local  wtproc: ESSENCE_OF_PROCESS;
		unqual: COMPARATOR_SORTED_LIST[ESSENCE_OF_PROCESS];
		lid, dscr: STRING;
	do  create unqual.make(agent {ESSENCE_OF_PROCESS}.compare_incompatibility);
	    process_selection_hint := Void
	    from  qualified_processes.start
	    until qualified_processes.after
	    loop  wtproc := qualified_processes.item
		  if wtproc.is_qualified_process(proc_opts)
		   then qualified_processes.forth	-- a keeper
		   else qualified_processes.remove;	-- a loser
			unqual.insert(wtproc)
		  end
	    end
	    -- if we took them all out, provide some clues
	    if qualified_processes.is_empty then
		-- create the text hint
		create process_selection_hint.make(400)
		from  unqual.start
		until unqual.after or else unqual.index > 15
		loop  wtproc := unqual.item
		      process_selection_hint.append (" - wafer_process_id ");
		      process_selection_hint.append (wtproc.wafer_process_id);
		      process_selection_hint.append (" has the following option_id conflicts: ")
		      process_selection_hint.append
					(su.joined_by (wtproc.incompatibilities, su.comma_space))
		      process_selection_hint.extend ('%N');
		      unqual.forth
		end;
		if not unqual.after then
		      process_selection_hint.append (" - ...%N")
		end;
		-- and create my_layer_error
		lid := "INCOMPATIBLE_OPTIONS"
		dscr := best_reason_for_failure(proc_opts)
		create my_layer_error.make_forged(lid, dscr)
	    end;
	end;

	best_reason_for_failure(proc_opts: LIST[ABSTRACT_PROCESS_OPTION]): STRING is
		-- look for individual options that are both in and out
		-- and for option categories with more than one in option
	local i, j, cnt: INTEGER; opti, optj: ABSTRACT_PROCESS_OPTION;
		oid, cat, oids: STRING; allopt: LINKED_LIST[STRING]
	do  create allopt.make; allopt.compare_objects	-- for generic msg
	    from  i := 1; cnt := proc_opts.count
	    until i > cnt or Result /= Void
	    loop  opti := proc_opts @ i;
		  oid := opti.option_id
		  cat := opti.option_category;
		  oids := opti.to_short_string;
		  if not (allopt.has(oids) or suppressed_oids.has(oids)) then
			allopt.extend(oids)	-- avoid repetition
		  end;
		  from  j := i + 1
		  until j > cnt or Result /= Void
		  loop  optj := proc_opts @ j;
			if equal(optj.option_id, oid) then
				-- same option; compare polarities
				if opti.option_in_process /= optj.option_in_process then
					Result := oid + " is both required and prohibited"
				end
			elseif (not cat.is_empty) and
			       equal(cat, optj.option_category) and
			       opti.option_in_process and optj.option_in_process then
				-- category conflict
				Result := su.string_of_6
						("the project uses both ",
						 opti.option_name, " and ",
						 optj.option_name,
						 " for ", cat)
			end;
			j := j + 1
		  end
		  i := i + 1
	    end;
	    -- didn't find any; give generic message
	    if Result = Void then
		Result := "layer set plus explicit options are either incompatible or not currently supported %(" + su.joined_by(allopt,",") + "%)"
	    end
	end;

	suppressed_oids: LIST[STRING]
		-- items to suppress from the generic message
	once Result := su.broken_at(policy.suppressed_option_marks, ',')
	     Result.compare_objects
	end;

feature -- support for techn_group_families access

	techn_group_family(psch: STRING; pdat: DB_DATE):
				TECHN_GROUP_FAMILIES_ROW is
		-- via cheapest_process and wafer_process#tg_families
	require su.not_empty(psch); pdat /= Void
	do  if not (equal(psch, my_pricing_sched) and 
		    equal(pdat, my_pricing_date))
	     then fetch_tg_family(psch, pdat)
	    end;
	    Result := my_tg_family
	    -- Void --> there is no tg_family for this pricing_schedule
	end;

feature {NONE} -- cache for techn_group_families access

	my_tg_family: TECHN_GROUP_FAMILIES_ROW
	my_pricing_sched: STRING	-- look for change and refetch
	my_pricing_date: DB_DATE	-- likewise

	fetch_tg_family(psch: STRING; pdat: DB_DATE) is
	local b: DB_BIND_SSD; tfam: STRING
	do  if cheapest_process /= Void then
		create b.make(cheapest_process.wafer_process, psch, pdat);
		tfam := connection.selected_string
			("part_selection.techn_group_family(:1, :3, :2)",
			 "dual", Void, b);
	    end;
	    if tfam = Void or else tfam.is_empty
	     then my_tg_family := Void
	     else create my_tg_family.make_by_key
			(connection, old_row, tfam)
	    end;
	    my_pricing_sched := psch	-- dont get this one again
	    my_pricing_date := pdat
	end;

feature	-- access to wafer_techn#run_types

	is_offered_run_type(rtyp: STRING): BOOLEAN is
	local b: DB_BIND_SS;
	do  create b.make(wafer_techn, rtyp);
	    Result := connection.selected_count(
				"wafer_techn#run_types",
				"wafer_techn = :1 and run_type = :2",
				b) = 1;
	end;

feature	-- access to next and previous runs
	-- these first two routines always return adjacent dates

	my_next_scheduled_date: DB_DATE is
		-- first date in SCHEDULED_RUNS for this wafer_techn
		-- that is either in the future or lacks a run
		-- Void for a wafer_techn w/out scheduled runs
	local b: DB_BIND_S;
	do  create b.make(wafer_techn);
	    Result := connection.safe_selected_date
			("min(sr.date_scheduled)",
			 "scheduled_runs sr, runs r",
			 next_run_date_filter, b)
	end;

	my_latest_scheduled_date: DB_DATE is
		-- latest date in SCHEDULED_RUNS for this wafer_techn
		-- that is in the past and has a run
		-- Void for a wafer_techn w/out scheduled runs
	local b: DB_BIND_S;
	do  create b.make(wafer_techn);
	    Result := connection.safe_selected_date
			("max(sr.date_scheduled)",
			 "scheduled_runs sr, runs r",
			 last_run_date_filter, b);
	end;
	
	my_latest_scheduled_runs: LIST[STRING] is
		-- run_id(s) for all runs (almost always one) that
		-- are associated with the last SCHEDULED_RUNS row
		-- that is in the past and has (at least one) run
		-- Void for a wafer_techn w/out scheduled runs
	local ldat: DB_DATE; b: DB_BIND_SD
	do  ldat := my_latest_scheduled_date;
	    if ldat /= Void
	     then create b.make(wafer_techn, ldat);
		  Result := connection.selected_strings
			("r.run_id",
			 "scheduled_runs sr, runs r",
			 ldat_runs_filter, Void, false, b)
	    end
	end;

	latest_runs_are_posted: BOOLEAN is
		-- are all the runs in my_latest_scheduled_runs posted?
	local rids: LIST[STRING]; run: MOSIS_RUN
	do  rids := my_latest_scheduled_runs;
	    Result := true;	-- until disproven
	    if rids /= Void
	     then from  rids.start;
		  until rids.after or not Result
		  loop  create run.make(rids.item);
			Result := run.is_posted
			rids.forth
		  end
	    end
	end;

feature	{NONE} -- implementation for the above

	next_run_date_filter: STRING is
		-- finding min(date_scheduled)
	"r.sched_run_no (+) = sr.sched_run_no and %
	%sr.wafer_techn = :1 and %
	%sr.date_canceled is null and %
	%(sr.date_scheduled > sysdate or r.run_id is null)"

	last_run_date_filter: STRING is
		-- finding max(date_scheduled)
	"r.sched_run_no = sr.sched_run_no and %
	%sr.wafer_techn = :1 and %
	%sr.date_scheduled <= sysdate";

	ldat_runs_filter: STRING is
		-- finding runs for given date
	"r.sched_run_no = sr.sched_run_no and %
	%sr.wafer_techn = :1 and %
	%sr.date_scheduled = :2"

feature	-- access to subset_wafer_techns

	is_a_subset_id(an_id: STRING): BOOLEAN is
		-- is an_id the wafer_techn_id of a subset of mine?
	do  Result := subset_wafer_techn_ids.has(an_id)
	end;

	subset_wafer_techn_ids: LIST[STRING] is
		-- list of all wafer_techn_ids in subset_wafer_techns
		-- with me as the superset
	local b: DB_BIND_SS
	do  if cached_subset_ids = Void then
		create b.make(wafer_techn_family, wafer_techn_id);
		cached_subset_ids := connection.selected_strings
				("wafer_techn_id",
				 "subset_wafer_techns",
				 "wafer_techn_family = :1 and superset_wafer_techn_id = :2",
				 Void, false, b)
		cached_subset_ids.compare_objects	
	    end;
	    Result := cached_subset_ids
	end;

feature {NONE} -- caches for stuff above

	cached_dt_row: DESIGN_TECHNS_ROW

	cached_wdt_row: DESIGN_TECHNS_ROW

	cached_subset_ids: like subset_wafer_techn_ids

end -- class DESIGN_TECHNXWAFER_TECHNS_ROW
