
indexing
	
	description: "Map that shows fields of view stepped across wafer %
		%with dies shown inside each field"
	author: "David Hollenberg"
	date: "January 21, 2005"

class DETAILED_STEP_PATTERN_MAP
inherit
	WAFER_MAP
		export
			{ANY} generate_title, generate_drawing, generate_page
		redefine
			set_stroke_widths,
			set_stroke_and_fill_colors
		end
create
	make

feature -- Properties

	title: STRING = "Detailed Step Pattern"
			-- Title to include on map
	
feature {NONE} -- Implementation

	add_stepped_field (fsize, psize: CONST_POINT step_loc: FIELD_STEP_PATTERNXFIELD_LOCS_ROW)
			-- <Precursor>
		local
			step_center: CONST_POINT
			box: DRW_RECTANGLE
			left_width, left_height: DOUBLE
			top_width, top_height: DOUBLE
		do
			step_center := step_loc.center + step_pattern_offset
			left_width := fsize.x - psize.x
			left_height := fsize.y + (fsize.y - psize.y)
			top_width := psize.x
			top_height := fsize.y - psize.y
			-- Add left and right boxes
			add_outside_payload_area (left_width, left_height, step_center.x - fsize.x / 2.0, step_center.y)
			add_outside_payload_area (left_width, left_height, step_center.x + fsize.x / 2.0, step_center.y)
			-- Add bottom and top boxes
			add_outside_payload_area (top_width, top_height, step_center.x, step_center.y - fsize.y / 2.0)
			add_outside_payload_area (top_width, top_height, step_center.x, step_center.y + fsize.y / 2.0)
			
			-- Add outline box around payload and
			-- fill if partially processed
			if step_loc.partially_processed and partially_processed_or_edge_fill /= Void then
				create box.make (psize.x, psize.y, step_center.x, step_center.y) 
				box.disable_stroke
				box.set_fill (partially_processed_or_edge_fill)
				last_drawing.extend (box)
			end
		end

	add_outside_payload_area (a_size_x, a_size_y, a_center_x, a_center_y: DOUBLE)
			-- <Precursor>
		local
			box: DRW_RECTANGLE
		do
			create box.make (a_size_x, a_size_y, a_center_x, a_center_y)
			box.disable_stroke
			box.set_fill (outside_payload_fill)
			last_drawing.extend (box)
		end

	generate_drawing_specific_part
			-- Generate part of drawing that is specific
			-- to a particular map
			
		local
			step_loc: FIELD_STEP_PATTERNXFIELD_LOCS_ROW
			step_center: CONST_POINT
			contents: DRW_GROUP
			m: DRW_MATRIX
		do
			generate_field_contents
			from
				step_locations.start
			until
				step_locations.after
			loop
				step_loc := step_locations.item
				step_center := step_loc.center + step_pattern_offset
				contents := field_contents.item (step_loc.field_name)
				if contents /= Void then
					contents := contents.twin
					create m.make_translation_xy (step_center.x, step_center.y)
					contents.set_transform (m)
					last_drawing.extend (contents)
				end
				step_locations.forth
			end
		end

	generate_field_contents
			-- Generate `field_contents'
		local
			fname: STRING
		do
			create field_contents.make (field_names.count)
			from
				field_names.start
			until
				field_names.after
			loop
				fname := field_names.item
				field_contents.put (drawing_for_field (fname), fname)
				field_names.forth
			end
		end

	drawing_for_field (fname: STRING): DRW_GROUP
			-- Drawing for contents of field `fname', with
			-- dies labeled by die id.  Void if field is empty
		local
			locs: LIST [DIE_LOCATIONS_EROW]
			loc: DIE_LOCATIONS_EROW
			group: DRW_GROUP
		do
			locs := die_locations.item (fname)
			if locs /= Void then
				create group.make (locs.count)
				from
					locs.start
				until
					locs.after
				loop
					loc := locs.item
					if loc.included_in_payload_maps then
						group.append (drawing_for_location (loc))
					end
					locs.forth
				end
			end
			if group /= Void and then not group.is_empty then
				Result := group
			end
		end

	drawing_for_location (loc: DIE_LOCATIONS_EROW): LIST [DRW_FIGURE]
			-- List of figures for drawing for `loc'
		require
			loc_exists: loc /= Void
		local
			box: DRW_RECTANGLE
			size: CONST_POINT
		do
			create {ARRAYED_LIST [DRW_FIGURE]} Result.make (2)
			size := loc.printed_size_on_wafer
			create box.make (size.x, size.y, loc.location_x, loc.location_y)
			if loc.is_scribe and scribe_fill /= Void then
				box.set_fill (scribe_fill)
			end
			Result.extend (box)
			if not loc.is_scribe then
				Result.extend (optional_text_pair_in_box (loc.die_id, "", box.bounding_box, loc.non_null_die_wafer_rotation))
			end
		end

	set_stroke_widths
			-- Set stroke values to use for various line
			-- thicknesses (relative numbers only - widths
			-- will be scaled later for each group)
		do
			precursor {WAFER_MAP}
			field_stroke_width := thick_width
		end

	set_stroke_and_fill_colors
			-- Set colors used for stroke and fill
		do
			precursor
			create field_step_stroke.make_with_rgb (1.0, 0.5, 0.5)
			create outside_payload_fill.make_with_rgb (1.0, 0.0, 0.0)
			-- Do not fill scribes
			-- create scribe_fill.make_with_rgb (1.0, 0.5, 0.5)
		end

	field_contents: HASH_TABLE [DRW_GROUP, STRING]
			-- Groups with contents of fields, indexed by
			-- field name
	
	scribe_fill: DRW_COLOR
			-- Fill for scribes

	outside_payload_fill: DRW_COLOR
			-- Fill for areas inside field but outside payload

end
