
indexing
	
	description: "Dictionary in which items can be looked up"
	author: "Joel Goldberg (reworked by David Hollenberg)"
	date: "February 11, 2000"

deferred class DICTIONARY

feature

	has (key: STRING): BOOLEAN
			-- Is there an entry for `key'?
		require
			key_exists: key /= Void
			key_not_empty: not key.is_empty
		deferred
		end

	item (key: STRING): STRING
			-- Value associated with `key' or Void if none
		require
			key_exists: key /= Void
			key_not_empty: not key.is_empty
		deferred
		end
	
	item_or_empty (key: STRING): STRING
			-- Value associated with `key' or empty string if none
		require
			key_exists: key /= Void
			key_not_empty: not key.is_empty
		do
			Result := item (key)
		    	if Result = Void then
				create Result.make(0)
			end
		ensure	
			result_exists: Result /= Void
		end
	
feature -- string to Boolean or Integer or Double, with no exceptions

	as_boolean(s: STRING): BOOLEAN
		-- accept ON/OFF, TRUE/FALSE, YES/NO
		-- (actually, anything except ON, TRUE, or YES is false)
	local ss: STRING
	do  if s /= Void
	     then ss := s.as_upper
		  Result := equal(ss, "ON") or equal(ss, "TRUE") or
			    equal(ss, "YES")
	    end
	end

	as_integer(s: STRING): INTEGER
		-- (return 0 if it's not an integer string)
	do  if s /= Void and then s.is_integer
	     then Result := s.to_integer
	    end
	end

	as_integer_64 (s: STRING): INTEGER_64
		-- (return 0 if it's not an integer string)
	do  if s /= Void and then s.is_integer_64
	     then Result := s.to_integer_64
	    end
	end

	as_double(s: STRING): DOUBLE
		-- (return 0 if it's not a real string)
	do  if s /= Void and then s.is_double
	     then Result := s.to_double
	    end
	end

end
