
indexing
	
	description: "Window to control fracturing of individual dies"
	author: "David Hollenberg"
	date: "November 17, 2003"

class DIE_FRACTURE_CONTROL_WINDOW

inherit
	JOB_CONTROL_WINDOW [DIE_JOB]
		
create
	make

feature {NONE} -- Implementation

	add_extra_select_menu_items (select_menu: EV_MENU)
			-- Add any application-specific items to `select_menu'
		do	
			add_separator (select_menu)
			
			add_menu_item (select_menu, Customer_dies_text, agent process_select_die_type (die_type.Cust_die))
			add_menu_item (select_menu, Test_vehicles_text, agent process_select_die_type (die_type.Test_vehicle))
			add_menu_item (select_menu, Vendor_geometry_text, agent process_select_die_type (die_type.Vendor_geometry))
			add_menu_item (select_menu, Scribes_text, agent process_select_die_type (die_type.Scribe))
			add_menu_item (select_menu, Fillers_text, agent process_select_die_type (die_type.Filler))
			
		end

	add_extra_view_row_menu_items (row_menu: EV_MENU)
			-- Add any application-specific items to `row_menu'
		do	
			add_separator (row_menu)
			
			add_menu_item (row_menu, Customer_dies_text, agent process_view_die_type (die_type.Cust_die))
			add_menu_item (row_menu, Test_vehicles_text, agent process_view_die_type (die_type.Test_vehicle))
			add_menu_item (row_menu, Vendor_geometry_text, agent process_view_die_type (die_type.Vendor_geometry))
			add_menu_item (row_menu, Scribes_text, agent process_view_die_type (die_type.Scribe))
			add_menu_item (row_menu, Fillers_text, agent process_view_die_type (die_type.Filler))
		end

	fetch_needed_data
			-- Fetch needed data when first building interface
		do
			fetch_requestor_list
			fetch_fab_instance_list
			fetch_batch_jobs_list
		end

	fetch_requestor_list
			-- Fetch list of rows requesting batch job
			-- (list of dies for run) from database and
			-- put in `die_rows'
		do
			die_rows := cache.die_mrows
		end
			
	batch_jobs_for_run: LIST [BATCH_JOBS_MROW]
			-- List of batch jobs for this run
		local
			batch_jobs_row: BATCH_JOBS_MROW
		do
			create batch_jobs_row.make_template
			Result := batch_jobs_row.die_jobs_for_run (session.run_id, session.connection, expected_count.At_least_zero)
		end
			
	initialize_total_count
			-- Initialize `total_count'
		do
			total_count := die_rows.count
		end

	build_all_rows
			-- Build list `all_rows'
		local
			row: DIE_JOB
			die: DIES_MROW
			fab: IC_FAB_INSTANCES_EROW
			mebes_files: ARRAYED_LIST [STRING]
			files: LIST [RC_FILE]
			pattern_type: PATTERN_TYPE_VALUES
		do
			from
				die_rows.start
			until
				die_rows.after
			loop
				die := die_rows.item
				fab_instances.search (die.die_id)
				if fab_instances.found then
					fab := fab_instances.found_item
				else
					fab := Void
				end
				create row.make (die, fab)
				row.set_run_id (session.run_id)
				row.set_levels (cache.run_level_rows)
				row.set_pattern_type (pattern_type.Die)
				row.set_job_script_file (cache.die_fracture_script_file (die).file_pathname)
				row.set_job_log_file (cache.die_fracture_log_file (die).file_pathname)
				row.set_job_done_file (cache.die_fracture_done_file (die).file_pathname)
				files := cache.die_fracture_pattern_files (die)
				create mebes_files.make (files.count)
				from
					files.start
				until
					files.after
				loop
					mebes_files.extend (files.item.file_pathname)
					files.forth
				end
				row.set_job_output_files (mebes_files)
				row.select_actions.extend (agent process_select_one (row))
				row.deselect_actions.extend (agent process_deselect_one (row))
				all_rows.extend (row)
				die_rows.forth
			end
		end
			
	add_visible_columns
			-- Add columns to `visible_columns'
		do
			visible_columns.extend (create {DIE_ID_COLUMN [DIE_JOB]})
			visible_columns.extend (create {JOB_STATUS_COLUMN [DIE_JOB]})
			visible_columns.extend (create {DIE_TYPE_COLUMN [DIE_JOB]})
			visible_columns.extend (create {BATCH_JOB_NO_COLUMN [DIE_JOB]})
			visible_columns.extend (create {BATCH_JOB_STATE_COLUMN [DIE_JOB]})
			visible_columns.extend (create {BATCH_JOB_HOST_COLUMN [DIE_JOB]})
			visible_columns.extend (create {JOB_TIME_COLUMN [DIE_JOB]})
			visible_columns.extend (create {DESIGN_NO_COLUMN [DIE_JOB]})
			visible_columns.extend (create {ACCT_NO_COLUMN [DIE_JOB]})
			visible_columns.extend (create {NAME_COLUMN [DIE_JOB]})
			visible_columns.extend (create {DESIGN_TECHN_ID_COLUMN [DIE_JOB]})
			visible_columns.extend (create {TELEPHONE_COLUMN [DIE_JOB]})
			visible_columns.extend (create {NET_ADDRESS_COLUMN [DIE_JOB]})
			visible_columns.extend (create {ERROR_DESCRIPTION_COLUMN [DIE_JOB]})
		end
			
	process_select_die_type (s: STRING)
			-- Take actions needed when user chooses to replace
			-- selection with all dies of type `s'
		require
			die_type_exists: s /= Void
		do
			process_select_rows_filtered (agent {DIE_JOB}.has_die_type (s))
		end

	has_job_status_agent (s: STRING): FUNCTION [DIE_JOB, TUPLE [DIE_JOB], BOOLEAN]
			-- Agent that tells whether row has job status `s'
		do
			Result := agent {DIE_JOB}.has_job_status (s)
		end

	is_stopable_agent: FUNCTION [DIE_JOB, TUPLE [DIE_JOB], BOOLEAN]
			-- Agent that tells whether row has job that
			-- is stopable
		do
			Result := agent {DIE_JOB}.is_stopable
		end

	is_unstopable_agent: FUNCTION [DIE_JOB, TUPLE [DIE_JOB], BOOLEAN]
			-- Agent that tells whether row does not have a
			-- job that is stopable
		do
			Result := agent {DIE_JOB}.is_unstopable
		end

	is_resettable_agent: FUNCTION [DIE_JOB, TUPLE [DIE_JOB], BOOLEAN]
			-- Agent that tells whether row has job that
			-- is resettable
		do
			Result := agent {DIE_JOB}.is_resettable
		end

	is_unresettable_agent: FUNCTION [DIE_JOB, TUPLE [DIE_JOB], BOOLEAN]
			-- Agent that tells whether row does not have
			-- a job that is resettable
		do
			Result := agent {DIE_JOB}.is_unresettable
		end

	is_unsuccessful_agent: FUNCTION [DIE_JOB, TUPLE [DIE_JOB], BOOLEAN]
			-- Agent that tells whether row has job that
			-- was unsuccessful
		do
			Result := agent {DIE_JOB}.is_unsuccessful
		end

	is_submittable_agent: FUNCTION [DIE_JOB, TUPLE [DIE_JOB], BOOLEAN]
			-- Agent that tells whether row has job that
			-- is submittable
		do
			Result := agent {DIE_JOB}.is_submittable
		end

	is_unsubmittable_agent: FUNCTION [DIE_JOB, TUPLE [DIE_JOB], BOOLEAN]
			-- Agent that tells whether row is not in a state
			-- where job can be submitted
		do
			Result := agent {DIE_JOB}.is_unsubmittable
		end

	is_unreleasable_agent: FUNCTION [DIE_JOB, TUPLE [DIE_JOB], BOOLEAN]
			-- Agent that tells whether row is not in a state
			-- where batch job can be released
		do
			Result := agent {DIE_JOB}.is_unreleasable
		end

	is_passed_agent: FUNCTION [DIE_JOB, TUPLE [DIE_JOB], BOOLEAN]
			-- Agent that tells whether row has job that
			-- is passed
		do
			Result := agent {DIE_JOB}.is_passed
		end

	job_arguments (row: DIE_JOB): STRING
			-- Arguments to be passed to batch job for `row'
		do
		end

	process_view_die_type (s: STRING)
			-- Take actions needed when user chooses to view
			-- only dies of type `s'
		require
			die_type_exists: s /= Void
		do
			process_view_rows_filtered (agent {DIE_JOB}.has_die_type (s))
		end

	handle_submit_event (submitted_rows: LIST [DIE_JOB])
			-- Handle job submission event
		local
			submit_event: DIE_FRACTURE_SUBMITTED_EVENT
		do
			create submit_event
			submit_event.set_dies (submitted_rows)
			submit_event.publish
		end

	handle_release_event (released_rows: LIST [DIE_JOB])
			-- Handle job release event
		local
			release_event: DIE_FRACTURE_RELEASED_EVENT
		do
			create release_event
			release_event.set_dies (released_rows)
			release_event.publish
		end

	refresh_corrupted_rows
			-- Refresh requestor for elements of `all_rows'
			-- that were modified and became invalid due
			-- to a rollback
		local
			die: DIE_JOB
			row: DIES_MROW
		do
			-- Populate hash table of DIES rows, indexed
			-- by die id
			
			create dies_table.make (die_rows.count)
			from
				die_rows.start
			until
				die_rows.after
			loop
				row := die_rows.item
				dies_table.put (row, row.die_id)
				die_rows.forth
			end

			from
				all_rows.start
			until
				all_rows.after
			loop
				die := all_rows.item
				if die.dies_row.is_corrupted then
					dies_table.search (die.die_id)
					if dies_table.found then
						row := dies_table.found_item
					else
						handle_internal_error ("cannot find die id " + die.die_id + " in internal table")
					end
					die.set_dies_row (row)
				end
				
				all_rows.forth
			end
		
		end

	handle_changed_event (changed_rows: LIST [DIE_JOB])
			-- Handle job status change event
		local
			changed_event: DIE_FRACTURE_CHANGED_EVENT
		do
			create changed_event
			changed_event.set_dies (changed_rows)
			changed_event.publish
		end

	handle_all_passed_event
			-- Handle "all rows passed" event
		local
			done_event: DIE_FRACTURES_DONE_EVENT
		do
			create done_event
			done_event.publish
		end

	handle_failure_event (failed_rows, aborted_rows: LIST [DIE_JOB])
			-- Handle job failure event
		local
			failed_event: DIE_FRACTURE_FAILED_EVENT
		do
			create failed_event
			failed_event.set_failed_dies (failed_rows)
			failed_event.set_aborted_dies (aborted_rows)
			failed_event.publish
		end

	handle_reset_event (reset_rows: LIST [DIE_JOB])
			-- Handle job reset event
		local
			reset_event: DIE_FRACTURE_RESET_EVENT
		do
			create reset_event
			reset_event.set_dies (reset_rows)
			reset_event.publish
		end

	row_to_generic (row: EV_MULTI_COLUMN_LIST_ROW): DIE_JOB
			-- Value of `row' which is of type DIE_JOB
		do
			Result ?= row
			check
				result_not_void: Result /= Void
			end
		end

	die_rows: LIST [DIES_MROW]
			-- List of database rows for all dies for run

	dies_table: HASH_TABLE [DIES_MROW, STRING]
			-- Rows from DIES table for this run,
			-- indexed by die id

	Customer_dies_text: STRING = "Customer dies"
	Test_vehicles_text: STRING = "Test vehicles"
	Vendor_geometry_text: STRING = "Vendor geometry"
	Scribes_text: STRING = "Scribes"
	Fillers_text: STRING = "Fillers"
	
	die_type: DIE_TYPE_VALUES

end
