
indexing
	
	description: "Multi-column list row for a die and its associated %
		%batch job"
	author: "David Hollenberg"
	date: "November 18, 2003"

class DIE_JOB
inherit	
	JOB_MULTI_COLUMN_LIST_ROW
		rename
			make as mcl_make,
			requestor as dies_row
		end
	HAS_ACCOUNT_NUMBER
		rename
			account_number as cust_acct_no,
			has_account_number as has_acct_no
		undefine
			copy, is_equal, default_create
		end
	HAS_DESIGN_NUMBER
		rename
			design_number as design_no,
			has_design_number as has_design_no
		undefine
			copy, is_equal, default_create
		end
	HAS_DIE_ID
		undefine
			copy, is_equal, default_create
		end
	HAS_DIE_TYPE
		undefine
			copy, is_equal, default_create
		end
	HAS_DESIGN_TECHN
		undefine
			copy, is_equal, default_create
		end
	HAS_NAME
		undefine
			copy, is_equal, default_create
		end
	HAS_NET_ADDRESS
		undefine
			copy, is_equal, default_create
		end
	HAS_TELEPHONE_NUMBER
		rename
			telephone_number as phone_number
		undefine
			copy, is_equal, default_create
		end

create
	make

create {DIE_JOB}
	make_filled

feature {NONE} -- Creation

	make (d: DIES_MROW f: IC_FAB_INSTANCES_EROW)
		require
			die_exists: d /= Void
		do
			default_create
			set_dies_row (d)
			set_fab_instance (f)
			enable_visible
		ensure
			is_visible: is_visible
		end

feature -- Properties

	dies_row: DIES_MROW
			-- Row from dies table

	fab_instance: IC_FAB_INSTANCES_EROW
			-- Information about the fab instance, including
			-- extra information from other tables

	levels: LIST [RUN_LEVELS_ROW]
			-- Mask levels for run.  Used only to translate
			-- MEBES extension to level number for rows
			-- inserted into pattern files table

	die_id: STRING
			-- Die id
		do
			Result := dies_row.die_id
		end

	unique_identifier: STRING
			-- Value that uniquely identifies row
		do
			Result := die_id
		end

	die_type: STRING
			-- Die type
		do
			Result := dies_row.die_type
		end

	has_die_type (s: STRING): BOOLEAN
			-- Does `Current' have die type `s'?
		require
			string_exists: s /= Void
		do
			Result := dies_row.has_die_type (s)
		end

	has_design_no: BOOLEAN
			-- Is there an associated design number?
		do
			Result := fab_instance /= Void
		end

	design_no: INTEGER
			-- Design number
		do
			Result := fab_instance.design_no
		end

	has_acct_no: BOOLEAN
			-- Is there an associated account number?
		do
			Result := fab_instance /= Void
		end

	cust_acct_no: INTEGER
			-- Cust account number
		do
			Result := fab_instance.cust_acct_no
		end

	name: STRING
			-- Identifying name
		do
			if fab_instance /= Void then
				Result := fab_instance.design_name
			elseif dies_row.is_vendor_geometry or dies_row.is_vendor_ghost then
				Result := dies_row.geometry_id
			elseif dies_row.is_scribe then
				Result := dies_row.scribe_orientation + " " + dies_row.die_type
			elseif dies_row.is_filler then
				Result := dies_row.filler_name
			else
				Result := ""
			end
		end

	design_techn: STRING
			-- Design technology
		do
			if fab_instance /= Void then
				Result := dies_row.design_techn
			else
				Result := ""
			end
		end

	phone_number: STRING
			-- Telephone number
		do
			if fab_instance /= Void then
				Result := fab_instance.phone_number
			else
				Result := ""
			end
		end

	net_address: STRING
			-- Net address
		do
			if fab_instance /= Void then
				Result := fab_instance.net_address
			else
				Result := ""
			end
		end

	job_name (a_run_id: STRING): STRING
			-- Name of job for `a_run_id'
		do
			Result := a_run_id + "_" + die_id
		end

	job_type: STRING
			-- Type of job to submit
		local
			type: BATCH_JOB_TYPE_VALUES
		do
			Result := type.Die_fracture
		end

	batch_queue_name: STRING
			-- Name of batch queue to use
		local
			queue: BATCH_JOB_QUEUE_VALUES
		do
			Result := queue.Die_fracture_batch
		end

feature -- Modification

	set_dies_row (r: DIES_MROW)
			-- Set `dies_row' to `r'
		do
			dies_row := r
		ensure
			dies_row_set: dies_row = r
		end

	set_fab_instance (f: IC_FAB_INSTANCES_EROW)
			-- Set `fab_instance' to `f'
		do
			fab_instance := f
		ensure
			fab_instance_set: fab_instance = f
		end

	set_levels (list: LIST [RUN_LEVELS_ROW])
			-- Set `levels' to `list'
		do
			levels := list
		ensure
			levels_set: levels = list
		end

feature {NONE} -- Implementation

	need_pattern_file_stats: BOOLEAN = False
			-- Should information about pattern files be
			-- recorded in database?

	fill_in_pattern_file_row (row: PATTERN_FILES_MROW)
			-- Set rest of columns in `row' before it
			-- is inserted
		local
			fname, ext: STRING
			pos: INTEGER
		do
			fname := row.pattern_filename
			pos := fname.index_of ('.', 1)
			if pos > 0 then
				ext := fname.substring (pos + 1, fname.count)
			else
				ext := ""
			end
			row.set_die_id (die_id)
			row.set_level_no (level_no_for_ext (ext))
		end

	level_no_for_ext (ext: STRING): INTEGER
			-- Level number for MEBES extension `ext'
		require
			extension_exists: ext /= Void
		local
			found: BOOLEAN
		do
			from
				levels.start
			until
				levels.after
			loop
				if levels.item.mebes_extension.is_equal (ext) then
					found := True
					Result := levels.item.level_no
				end
				levels.forth
			end
			if not found then
				handle_hard_error ("cannot find level with MEBES extension " + ext)
			end
		end

	existing_pattern_file_rows: LIST [PATTERN_FILES_MROW]
			-- List of pattern file rows in database
			-- created for `Current'
		local
			row: PATTERN_FILES_MROW
		do
			create row.make_template
			Result := row.rows_for_die_and_type (run_id, die_id, pattern_type, dies_row.connection, expected_count.At_least_zero)
		end

	new_filled_list (n: INTEGER): like Current
			-- New list with `n' elements.
		do
			create Result.make_filled (n)
		end
		
end
