
indexing

	author: "David Hollenberg"
	date: "January 24, 2005"

class DIE_LOCATIONS_EROW
inherit
	DIE_LOCATIONS_ERAW
	ABSTRACT_DIE
	ROW_COUNT_CHECK_ACCESS
create
	make_template, make_null

feature -- Properties

	die_locations_for_run (run: STRING conn: DB_CONNECTION range: ITEM_COUNT_RANGE): LIST [like Current] is
			-- All die locations for `run', ordered 
			-- by die id
		require
			run_exists: run /= Void
			connection_exists: conn /= Void
			connection_open: conn.is_open
			range_exists: range /= Void
		local
			b: DB_BIND_S
		do
			create b.make (run)
			Result := all_rows_filtered_ordered (conn, "dl.run_id = :1", b, "die_id")
			check_row_count (Result, range)
		ensure
			result_exists: Result /= Void
		end

	design_name_for_map: STRING is
			-- Possibly shortened design name to include
			-- on maps
		local
			len: INTEGER
			maxlen: like map_design_name_length
			tail: like map_design_name_tail
		do
			len := design_name.count
			maxlen := map_design_name_length
			if is_vendor_geometry or is_vendor_ghost then
				Result := vendor_geometry_description
			elseif is_filler then
				Result := filler_description
			elseif len <= maxlen then
				Result := design_name
			else
				tail := map_design_name_tail
				Result := design_name.substring (1, maxlen - (ellipsis.count + tail))
				Result.append (ellipsis)
				Result.append (design_name.substring (len - tail + 1, len))
			end
		ensure
			result_exists: Result /= Void
		end
	
	design_number_for_map: STRING is
			-- Design number to include on maps
		do
			if is_vendor_geometry or is_vendor_ghost then
				Result := vendor_geometry_description
			elseif is_filler then
				Result := filler_description
			elseif is_null_design_no then
				Result := ""
			else
				Result := design_no.out
			end
		ensure
			result_exists: Result /= Void
		end
	
	sponsor_for_map: STRING is
			-- Sponsor to include on maps
		do
			if is_vendor_geometry or is_vendor_ghost then
				Result := overhead_sponsor
			elseif is_filler then
				Result := filler_description
			else
				Result := acct_sponsor_id
				if Result.is_empty then
					Result := acct_type
				end
			end
		ensure
			result_exists: Result /= Void
		end
	
	org_short_id_for_map: STRING is
			-- Short id to include on maps
		require
			not_vendor_die: not is_vendor_geometry and not is_vendor_ghost
		do
			if is_test_vehicle then
				Result := overhead_org_short_id
			else
				Result := short_id
			end
		ensure
			result_exists: Result /= Void
		end
	
	is_inside_payload: BOOLEAN is
			-- Is die location inside payload?
		do
			Result := is_null_external_to_payload or not external_to_payload
		end
	
	included_in_payload_maps: BOOLEAN is
			-- Should die location be included in maps of
			-- the payload area?
		do
			Result := not is_macrocell and is_inside_payload
		end
	
feature {NONE} -- Implementation

	map_design_name_length: INTEGER is 15
			-- Maximum length of design name to show on map
	
	map_design_name_tail: INTEGER is 3
			-- Number of characters at end of design name to retain
	
	ellipsis: STRING is "..."
			-- String that replaces omitted text from design name
	
	overhead_sponsor: STRING is "OH"
			-- Name of sponsor for vendor geometry/ghost
	
	filler_description: STRING
			-- Descriptive name to use for fillers (don't have
			-- design number, design name, etc)
		do
			Result := filler_name
		end
	
	vendor_geometry_description: STRING is "(Vendor)"
			-- Descriptive name to use for vendor geometry and
			-- ghosts (don't have design number, design name, etc)
	
	overhead_org_short_id: STRING is "MOSIS"
			-- Short organization id for test vehicles
	
end
