class DIES_XROW
	-- an extension of DIES_ROW intended for Delta
	--  (especially in wafer_techn_family analysis)
	-- drags in stuff that other systems may not want

inherit
	DIES_ROW
	rename
		design_techn as design_techn_name
	end;

create
	make_template, make_null, make_by_key

feature	-- extended data

	design_techn: DESIGN_TECHNOLOGY
		-- design_technology for THIS RUN
		-- i.e., the wafer_design_techn, not the one in the cid

	xform_levels: LIST[DESIGN_TECHN_XFORM_LEVELS_ROW]
		-- design_techn_xform_levels for THIS RUN's wafer_techn

	layers: LIST[STRING]
		-- layers_present (for dies with a design_no)
		-- Void for other dies (all layers assumed present)

	fracture_limits: CONST_RECTANGLE
		-- the fracture limits for this die

feature	-- extension setup

	extend_xrow_data(cache: RUN_CACHE) is
	require	have_cache: cache /= Void;
		same_db: cache.connection = connection;
		same_run: equal(cache.run_id, run_id)
	local ifi: IC_FAB_INSTANCES_ROW; b: DB_BIND_I;
		dcen: CONST_POINT
	do  -- set up the items above, from the cache
	    design_techn := cache.design_technology_for_dt (wafer_design_techn);
	    xform_levels := cache.xform_level_rows_for_dt (wafer_design_techn);
	    ifi := ic_fab_instance
	    if ifi /= Void then		-- cust_die or test_vehicle
		create b.make(ifi.design_no)
		layers := connection.selected_strings
				("layer_name", "design_no#final_layers",
				 "design_no = :1", Void, true, b)
		layers.compare_objects
		-- place layout into die and create fracture limits
		dcen := cust_ic_design.design_center
		dcen := dcen.snapped_to_grid (attributes.die_fracture_grid)
		dcen := dcen - design_die_offset;
		if die_wafer_rotation = 90 or die_wafer_rotation = 270 then
		    -- revert die size from wafer to layout orientation
		    create fracture_limits.make_by_size_and_center
				(die_size_printed_y, die_size_printed_x,
				 dcen.x, dcen.y)
		else
		    create fracture_limits.make_by_size_and_center
				(die_size_printed_x, die_size_printed_y,
				 dcen.x, dcen.y)
		end
	    else		-- anything else
		-- all non-cid dies are presumed to have all layers present
		layers := Void
		-- all non-cid dies follow the mebes convention re
		-- limits: (0,0) to (printed_die_size)
		create fracture_limits.make_by_corners
				(0, 0, die_size_printed_x, die_size_printed_y)
	    end
	ensure	have_dt: design_techn /= Void
		have_xl: xform_levels /= Void
	end;

	clear_xrow_data is
		-- clear out the xrow extended data so that we
		-- are not competing with the cache
	do
		design_techn := Void
		xform_levels := Void
		layers := Void
		fracture_limits := Void
	end

feature	-- functions using the extended data

	is_present(lnam: STRING): BOOLEAN is
		-- is lnam (reputedly) present in this die?
	do
		if layers = Void then
			Result := design_techn.layer_by_name(lnam) /= Void
				-- all known layers presumed present
		else
			Result := layers.has(lnam)
				-- actually present in layout
		end
	end;

	derived_when_absent_xform_layer_names: LIST [STRING]
			-- List of source layer names for xform levels
			-- marked "derived when absent"
		local
			pos: CURSOR
			list: like xform_levels
			lev: DESIGN_TECHN_XFORM_LEVELS_ROW
		do
			create {ARRAYED_LIST [STRING]} Result.make (xform_levels.count)
			list := xform_levels
			pos := list.cursor
			from
				list.start
			until	
				list.after
			loop	
				lev := list.item
				if lev.derived_when_absent then
					Result.extend (lev.source_layer_name)
				end
				list.forth
			end	
			xform_levels.go_to (pos)
		end

end -- DIES_XROW
