
indexing
	
	description: "An operation that opens a directory for read"
	author: "David Hollenberg"
	date: "March 12, 2003"

class DIR_OPEN_READ_OP
inherit
	OPERATION
		rename
			try_operation as try_open
		end
	EXCEPTIONS_ACCESS
		export
			{NONE} all
		undefine
			default_create
		end

create
	default_create

feature -- Operations

	open (d: LARGE_DIRECTORY)
			-- Open `d'
		require
			dir_exists: d /= Void
			dir_closed: d.is_closed
		do
			directory := d
			execute
		end

feature {NONE} -- Implementation

	try_open
			-- Try directory open open
		local
			error: DIR_OPEN_ERROR
		do
			try_to_open_dir_safely
			if not directory.is_closed then
				publish_ok_event
			else
				publish_failed_event
				create error
				error.set_hard_error
				error.set_directory_name (directory.name)
				error.handle
			end
		end

	try_to_open_dir_safely
			-- Try to open directory.  Do not raise exception if
			-- open fails, unless the exception is due to
			-- an assertion violation
		local
			tried: BOOLEAN
		do
			if not tried then
				try_to_open_dir
			end
		rescue
			if not exc.assertion_violation then
				tried := True
				retry
			end
		end

	try_to_open_dir
		do
			directory.open_read
		end

	directory: LARGE_DIRECTORY
			-- Directory to try to open

	publish_ok_event
		local
			ok_event: DIR_OPEN_OK_EVENT
		do
			create ok_event
			ok_event.set_directory_name (directory.name)
			ok_event.publish
		end

	publish_failed_event
		local
			failed_event: DIR_OPEN_FAILED_EVENT
		do
			create failed_event
			failed_event.set_directory_name (directory.name)
			failed_event.publish
		end

end
