
indexing
	
	description: "Class that can make a list of files in directory tree"
	author: "David Hollenberg"
	date: "February 5, 2002"

class DIR_TREE_LISTER

inherit
	DIRECTORY_TREE_TRAVERSAL
		rename
			traverse_postorder as make_pathname_list
		export
			{NONE} traverse_preorder
		redefine
			make, make_pathname_list
		end
	STRING_UTILITIES_ACCESS
		export
			{NONE} all
		end

create
	make

feature {NONE} -- Creation

	make (root: STRING)
			-- Create `Current' with root directory `root'
		do
			precursor (root)
		end


feature -- Properties

	path_names: LIST [STRING]
			-- List of path names from last call to `make_list'
	
feature -- Operations
	
	make_pathname_list
			-- Make list of entire directory tree and
			-- put in `path_names'
		do
			create {ARRAYED_LIST [STRING]} path_names.make (200)
			precursor
		end

feature {NONE} -- Implementation

	visit_directory (parent, entry, full_path: STRING)
			-- Visit directory with full pathname `full_path',
			-- whose parent directory name is `parent' and
			-- whose entry in parent directory is `entry'.
		do
			path_names.extend (full_path)
		end

	visit_file (parent, entry, full_path: STRING)
			-- Visit file with full pathname `full_path',
			-- whose parent directory name is `parent' and
			-- whose entry in parent directory is `entry'.
		do
			path_names.extend (full_path)
		end

	process_error (full_path, err_text: STRING)
			-- Handle error that occurred on `full_path',
			-- with error description `err_text'.  Only called
			-- if `errors_displayed' is true
		do
			io.put_string (full_path)
			io.put_string (su.colon_space)
			io.put_string (err_text)
			io.new_line
		end

end
