
indexing
	
	description: "Remover for directory trees (deletes entire tree)"
	author: "David Hollenberg"
	date: "July 13, 2000"

class DIR_TREE_REMOVER

inherit
	DIRECTORY_TREE_TRAVERSAL
		rename
			traverse_postorder as remove
		export
			{NONE} traverse_preorder
		redefine
			make, remove, handle_error
		end
	STRING_UTILITIES_ACCESS
		export
			{NONE} all
		end

create
	make

feature {NONE} -- Creation

	make (root: STRING)
			-- Create `Current' with root directory `root'
		do
			precursor (root)
			create dir.make ("")
		end


feature -- Operations
	
	remove
			-- Remove entire directory tree
		do
			precursor
		end

feature {NONE} -- Implementation

	dir: LARGE_DIRECTORY

	visit_directory (parent, entry, full_path: STRING)
			-- Visit directory with full pathname `full_path',
			-- whose parent directory name is `parent' and
			-- whose entry in parent directory is `entry'.
		do
			if unix_os.is_symbolic_link (full_path) then
				unix_os.unlink (full_path)
			else
				dir.make (full_path)
				if dir.is_empty then
					dir.delete
				end
			end
		end

	visit_file (parent, entry, full_path: STRING)
			-- Visit file with full pathname `full_path',
			-- whose parent directory name is `parent' and
			-- whose entry in parent directory is `entry'.
		do
			unix_os.unlink (full_path)
		end

	process_error (full_path, err_text: STRING)
			-- Handle error that occurred on `full_path',
			-- with error description `err_text'.  Only called
			-- if `errors_displayed' is true
		do
			io.put_string (full_path)
			io.put_string (su.colon_space)
			io.put_string (err_text)
			io.new_line
		end

	handle_error (full_path: STRING)
			-- Handle error that occurred on `full_path'
		local
			tried: BOOLEAN
		do
			if not tried then
				precursor (full_path)
			end
		rescue
			tried := True
			retry
		end

end
