
indexing
	
	description: "An operation that creates a directory"
	author: "David Hollenberg"
	date: "April 23, 2003"

class DIRECTORY_CREATE_OP
inherit
	OPERATION
		rename
			try_operation as try_directory_create
		end
	EXCEPTIONS_ACCESS
		export
			{NONE} all
		undefine
			default_create
		end

create
	default_create

feature -- Operations

	create_dir (dir_name: STRING)
			-- Create directory named `dir_name'
		require
			dir_name_exists: dir_name /= Void
			dir_name_nonempty: not dir_name.is_empty
		do
			directory_name := dir_name
			execute
		end

feature {NONE} -- Implementation

	try_directory_create
			-- Try directory creation
		local
			error: DIRECTORY_CREATE_ERROR
		do
			publish_try_event
			try_to_create_dir_safely
			if is_created then
				publish_ok_event
			else
				publish_failed_event
				create error
				error.set_hard_error
				error.set_directory_name (directory_name)
				error.handle
			end
		end

	try_to_create_dir_safely
			-- Try to create directory.  Do not raise exception if
			-- creation fails, unless the exception is due to
			-- an assertion violation
		local
			tried: BOOLEAN
		do
			if not tried then
				try_to_create_dir
				is_created := True
			else
				is_created := False
			end
		rescue
			if not exc.assertion_violation then
				tried := True
				retry
			end
		end

	try_to_create_dir
		local
			dir: LARGE_DIRECTORY
		do
			create dir.make (directory_name)
			dir.create_dir
		end

	directory_name: STRING
			-- Name of directory to try to create

	is_created: BOOLEAN
			-- Was directory successfully created?

	publish_try_event
		local
			try_event: DIR_CREATE_TRY_EVENT
		do
			create try_event
			try_event.set_directory_name (directory_name)
			try_event.publish
		end

	publish_ok_event
		local
			ok_event: DIR_CREATE_OK_EVENT
		do
			create ok_event
			ok_event.set_directory_name (directory_name)
			ok_event.publish
		end

	publish_failed_event
		local
			failed_event: DIR_CREATE_FAILED_EVENT
		do
			create failed_event
			failed_event.set_directory_name (directory_name)
			failed_event.publish
		end

end
