indexing
	description:
		"A drawing command to produce an eliptical arc."
	author: "Barden E. Smith"
	date:	"August 13, 2004"

class
	DRW_ARC

inherit
	DRW_COMMAND
		redefine
			move_by, transform_by
		end


create
	make_abs,
	make_rel

feature {NONE} -- Creation

	make_abs (r_x, r_y, rot:DOUBLE large_arc, sweep: BOOLEAN a_x, a_y: DOUBLE) is
			-- Assign parameters.
		do
			set (a_x, a_y)
			set_radius (r_x, r_y)
			set_rotation (rot)
			is_large_arc := large_arc
			is_sweep := sweep
			set_code ({DRW_COMMAND_TYPE}.drw_pathseg_arc_abs)
			is_absolute := True
		ensure
			x_assigned: x = a_x
			y_assigned: y = a_y
		end

	make_rel (r_x, r_y, rot:DOUBLE large_arc, sweep: BOOLEAN a_x, a_y: DOUBLE) is
			-- Assign parameters.
		do
			set (a_x, a_y)
			set_radius (r_x, r_y)
			set_rotation (rot)
			is_large_arc := large_arc
			is_sweep := sweep
			set_code ({DRW_COMMAND_TYPE}.drw_pathseg_arc_rel)
		ensure
			x_assigned: x = a_x
			y_assigned: y = a_y
		end

	
feature -- Access

	rx: DOUBLE
			-- radius along x-axis.

	ry: DOUBLE
			-- Radius along y-axis.

	rotation: DOUBLE
			-- Rotation of elipse (degrees ccw)

	is_large_arc: BOOLEAN
			--  set if large arc

	is_sweep: BOOLEAN
			--  set if sweep


feature -- Element change

	move_by (dx, dy: DOUBLE) is
		do
			if is_absolute then
				precursor (dx, dy)
			end
		end

	transform_by (m: MATRIX) is
		local
			p: VIRTUAL_POINT
		do
			p := m.dtransform_xy (rx, ry)
			rx := p.x
			ry := p.y
			if is_absolute then
				precursor (m)
			else
				p := m.dtransform_xy (x, y)
				x := p.x
				y := p.y
			end
		end

	set_radius (r_x, r_y: DOUBLE) is
			-- Assign size of the elipse.
		do
			rx := r_x
			ry := r_y
		ensure
			rx_assigned: rx = r_x
			ry_assigned: ry = r_y
		end


	set_rotation (rot: DOUBLE) is
			-- Assign axis offset of the elipse.
		do
			rotation := rot
		ensure
			rotation_assigned: rotation = rot
		end


feature -- Display

	svg_path: STRING is
		do
			if is_absolute then
				Result := "A"
			else
				Result := "a"
			end
			Result := Result + rx.out + " " + ry.out + " " + rotation.out
			if is_large_arc then Result.append (" 1") end
			if is_sweep then Result.append (" 1") end
			Result := Result + " " + x.out + "," + y.out
		end

feature {DRW_PATH} -- Size estimating

	expand_bounding_box (context: DRW_PATH) is
			-- Expands the bonding_box info in `context' include command points
		local
			bb: DRW_BASIC_RECTANGLE
		do
			bb := context.expanding_box
			if is_absolute then
				bb.include_coordinate (current)
				context.current_point.set (x, y)
			else
				bb.include (context.current_point.x + x, context.current_point.y + y)
				context.current_point.set (context.current_point.x + x, context.current_point.y + y)
			end
		end

feature {NONE} -- Implementation

	key_values: TUPLE [DOUBLE, DOUBLE, DOUBLE, DOUBLE, DOUBLE, BOOLEAN, BOOLEAN] is
		do
			Result := [x, y, rx, ry, rotation, is_large_arc, is_sweep]
		end

end -- class DRW_COMMAND
