indexing
	description:
		"Color attributes for drawing shapes."
	author: "Barden E. Smith"
	date:	"August 13, 2004"

class
	DRW_COLOR

create
	default_create,
	make_with_rgb,
	make_named_rgb

feature {NONE} -- Creation

	make_named_rgb (c_name: STRING a_red, a_green, a_blue: REAL) is
			-- Create with `a_red', `a_green', `a_blue', and name `c_name')
		require
			c_name_not_void: c_name /= Void
			a_red_within_range: a_red >= 0 and then a_red <= 1
			a_green_within_range: a_green >= 0 and then a_green <= 1
			a_blue_within_range: a_blue >= 0 and then a_blue <= 1
		do
			make_with_rgb (a_red, a_green, a_blue)
			set_name (c_name)
		end


	make_with_rgb (a_red, a_green, a_blue: REAL) is
			-- Create with `a_red', `a_green', `a_blue', and default name.
		require
			a_red_within_range: a_red >= 0 and then a_red <= 1
			a_green_within_range: a_green >= 0 and then a_green <= 1
			a_blue_within_range: a_blue >= 0 and then a_blue <= 1
		do
			default_create
			set_red (a_red)
			set_green (a_green)
			set_blue (a_blue)
		end


feature -- Status

	name: STRING
		-- The name of the color

	red: REAL
		-- The value of the red component

	green: REAL
		-- The value of the green component

	blue: REAL
		-- The value of the blue component

	is_black: BOOLEAN is
			-- Is this the color "Black"
		do
			Result := red = 0.0 and green = 0.0 and blue = 0.0
		end

	is_white: BOOLEAN is
			-- Is this the color "White"
		do
			Result := red = 1.0 and green = 1.0 and blue = 1.0
		end

feature -- Modification

	set_name (new_value: STRING) is
		do
			name := new_value
		end

	set_red (a_red: REAL) is
		require
			a_red_within_range: a_red >= 0 and then a_red <= 1
		do
			red := a_red
		end

	set_green (a_green: REAL) is
		require
			a_green_within_range: a_green >= 0 and then a_green <= 1
		do
			green := a_green
		end

	set_blue (a_blue: REAL) is
		require
			a_blue_within_range: a_blue >= 0 and then a_blue <= 1
		do
			blue := a_blue
		end

invariant
	red_non_negative: red >= 0
	green_non_negative: green >= 0
	blue_non_negative: blue >= 0

end -- class DRW_COLOR
