indexing
	description: "A set of parameters to produce a cubic Bzier curve."
	author: "Barden E. Smith"
	date:	"March 21, 2005"

class
	DRW_CUBIC_CURVE

inherit
	ANY
		export
			{ANY} is_equal, twin
			{NONE} all
		end

	DRW_COORDINATE
		undefine
			is_equal, out
		redefine
			move_by, transform_by
		end

create
	make


feature {NONE} -- Creation

	make (a_x0, a_y0, c_x1, c_y1, c_x2, c_y2, a_x, a_y: DOUBLE) is
			-- Assign parameters.
		do
			set_initial_point (a_x0, a_y0)
			set (a_x, a_y)
			set_control (c_x1, c_y1, c_x2, c_y2)
		ensure
			x_assigned: x = a_x
			y_assigned: y = a_y
		end

feature -- Access

	has_initial_point: BOOLEAN is
			-- Indicate if initial point exists
		do
			Result := initial_point /= Void
		end

	x0: DOUBLE is
			-- Initial point x-axis value
		require
			initial_point_set: has_initial_point
		do
			Result := initial_point.x
		end

	y0: DOUBLE is
			-- Initial point y-axis value
		require
			initial_point_set: has_initial_point
		do
			Result := initial_point.y
		end

	cx1: DOUBLE
			-- Control point 1 x-axis value.

	cy1: DOUBLE
			-- Control point 1 y-axis value.

	cx2: DOUBLE
			-- Control point 2 x-axis value.

	cy2: DOUBLE
			-- Control point 2 y-axis value.

feature -- Element change

	move_by (dx, dy: DOUBLE) is
		do
			if has_initial_point then
				initial_point.move_by (dx, dy)
			end
			cx1 := cx1 + dx
			cy1 := cy1 + dy
			cx2 := cx2 + dx
			cy2 := cy2 + dx
			precursor (dx, dy)
		end

	transform_by (m: MATRIX) is
		local
			p: VIRTUAL_POINT
		do
			if has_initial_point then
				initial_point.transform_by (m)
			end
			p := m.transform_xy (cx1, cy1)
			cx1 := p.x
			cy1 := p.y
			p := m.transform_xy (cx2, cy2)
			cx2 := p.x
			cy2 := p.y
			precursor (m)
		end

	set_initial_point (a_x, a_y: DOUBLE) is
		do
			if initial_point = Void then
				create initial_point.set (a_x, a_y)
			else
				initial_point.set (a_x, a_y)
			end
		end


	set_control (c_x1, c_y1, c_x2, c_y2: DOUBLE) is
			-- Assign control points.
		do
			cx1 := c_x1
			cy1 := c_y1
			cx2 := c_x2
			cy2 := c_y2
		ensure
			cx_assigned: cx1 = c_x1 and cx2 = c_x2
			cy_assigned: cy1 = c_y1 and cy2 = c_y2
		end


feature {NONE} -- Implementation

	initial_point: DRW_COORDINATE

end -- class DRW_CUBIC_CURVE
