indexing
	description:
		"A drawing command to produce a quadradic Bzier curve."
	author: "Barden E. Smith"
	date:	"August 13, 2004"

class
	DRW_CURVE2

inherit
	DRW_COMMAND
		redefine
			move_by, transform_by
		end

create
	make_abs,
	make_rel,
	make_smooth_abs,
	make_smooth_rel


feature {NONE} -- Creation

	make_abs (c_x, c_y, a_x, a_y: DOUBLE) is
			-- Assign parameters.
		do
			set (a_x, a_y)
			set_control (c_x, c_y)
			set_code ({DRW_COMMAND_TYPE}.drw_pathseg_curveto_quadratic_abs)
			is_absolute := True
		ensure
			x_assigned: x = a_x
			y_assigned: y = a_y
		end

	make_rel (c_x, c_y, a_x, a_y: DOUBLE) is
			-- Assign parameters.
		do
			set (a_x, a_y)
			set_control (c_x, c_y)
			set_code ({DRW_COMMAND_TYPE}.drw_pathseg_curveto_quadratic_rel)
		ensure
			x_assigned: x = a_x
			y_assigned: y = a_y
		end

	make_smooth_abs (a_x, a_y: DOUBLE) is
			-- Assign parameters.
		do
			set (a_x, a_y)
			set_control (0, 0)
			set_code ({DRW_COMMAND_TYPE}.drw_pathseg_curveto_quadratic_smooth_abs)
			is_absolute := True
			is_smooth := True
		ensure
			x_assigned: x = a_x
			y_assigned: y = a_y
		end

	make_smooth_rel (a_x, a_y: DOUBLE) is
			-- Assign parameters.
		do
			set (a_x, a_y)
			set_control (0, 0)
			set_code ({DRW_COMMAND_TYPE}.drw_pathseg_curveto_quadratic_smooth_rel)
			is_smooth := True
		ensure
			x_assigned: x = a_x
			y_assigned: y = a_y
		end

	
feature -- Access

	cx: DOUBLE
			-- Control point along x-axis.

	cy: DOUBLE
			-- Control along y-axis.

feature -- Element change

	move_by (dx, dy: DOUBLE) is
		do
			if is_absolute then
				cx := cx + dx
				cy := cy + dy
				precursor (dx, dy)
			end
		end

	transform_by (m: MATRIX) is
		local
			p: VIRTUAL_POINT
		do
			if is_absolute then
				p := m.transform_xy (cx, cy)
				cx := p.x
				cy := p.y
				precursor (m)
			else
				p := m.dtransform_xy (cx, cy)
				cx := p.x
				cy := p.y
				p := m.dtransform_xy (x, y)
				x := p.x
				y := p.y
			end
		end

	set_control (c_x, c_y: DOUBLE) is
			-- Assign control points.
		do
			cx := c_x
			cy := c_y
		ensure
			cx_assigned: cx = c_x
			cy_assigned: cy = c_y
		end


feature -- Display

	svg_path: STRING is
		do
			inspect type_code
			when {DRW_COMMAND_TYPE}.drw_pathseg_curveto_quadratic_abs then
				Result := "C"
				Result := Result + cx.out + "," + cy.out + " "
			when {DRW_COMMAND_TYPE}.drw_pathseg_curveto_quadratic_rel then
				Result := "c"
				Result := Result + cx.out + "," + cy.out + " "
			when {DRW_COMMAND_TYPE}.drw_pathseg_curveto_quadratic_smooth_abs then
				Result := "S"
			when {DRW_COMMAND_TYPE}.drw_pathseg_curveto_quadratic_smooth_rel then
				Result := "s"
			else
				Result := "?"
			end
			Result := Result + x.out + "," + y.out
		end

feature {DRW_PATH} -- Size estimating

	expand_bounding_box (context: DRW_PATH) is
			-- Expands the bonding_box info in `context' include command points
		local
			bb: DRW_BASIC_RECTANGLE
		do
			bb := context.expanding_box
			-- here we should drawout the curve???
			if is_absolute then
				bb.include (x, y)
				if not is_smooth then
					bb.include (cx, cy)
				end
				context.current_point.set (x, y)
			else
				bb.include (context.current_point.x + x, context.current_point.y + y)
				if not is_smooth then
					bb.include (context.current_point.x + cx, context.current_point.y + cy)
				end
				context.current_point.set (context.current_point.x + x, context.current_point.y + y)
			end
		end


feature {NONE} -- Implementation

	key_values: TUPLE [DOUBLE, DOUBLE, DOUBLE, DOUBLE] is
		do
			Result := [x, y, cx, cy]
		end

end -- class DRW_CURVE2
