indexing
	description:
		"A drawing command to produce a cubic Bzier curve."
	author: "Barden E. Smith"
	date:	"August 13, 2004"

class
	DRW_CURVE3

inherit
	DRW_COMMAND
		undefine
			move_by, transform_by
		end

	DRW_CUBIC_CURVE
		export
			{NONE} has_initial_point, set_initial_point, x0, y0
		redefine
			move_by, transform_by
		end

create
	make_abs,
	make_rel,
	make_smooth_abs,
	make_smooth_rel


feature {NONE} -- Creation

	make_abs (c_x1, c_y1, c_x2, c_y2, a_x, a_y: DOUBLE) is
			-- Assign parameters.
		do
			set (a_x, a_y)
			set_control (c_x1, c_y1, c_x2, c_y2)
			set_code ({DRW_COMMAND_TYPE}.drw_pathseg_curveto_cubic_abs)
			is_absolute := True
		ensure
			x_assigned: x = a_x
			y_assigned: y = a_y
		end

	make_rel (c_x1, c_y1, c_x2, c_y2, a_x, a_y: DOUBLE) is
			-- Assign parameters.
		do
			set (a_x, a_y)
			set_control (c_x1, c_y1, c_x2, c_y2)
			set_code ({DRW_COMMAND_TYPE}.drw_pathseg_curveto_cubic_rel)
		ensure
			x_assigned: x = a_x
			y_assigned: y = a_y
		end

	make_smooth_abs (c_x2, c_y2, a_x, a_y: DOUBLE) is
			-- Assign parameters.
		do
			set (a_x, a_y)
			set_control (0, 0, c_x2, c_y2)
			set_code ({DRW_COMMAND_TYPE}.drw_pathseg_curveto_cubic_smooth_abs)
			is_absolute := True
			is_smooth := True
		ensure
			x_assigned: x = a_x
			y_assigned: y = a_y
		end

	make_smooth_rel (c_x2, c_y2, a_x, a_y: DOUBLE) is
			-- Assign parameters.
		do
			set (a_x, a_y)
			set_control (0, 0, c_x2, c_y2)
			set_code ({DRW_COMMAND_TYPE}.drw_pathseg_curveto_cubic_smooth_rel)
			is_smooth := True
		ensure
			x_assigned: x = a_x
			y_assigned: y = a_y
		end

	
feature -- Element change

	move_by (dx, dy: DOUBLE) is
		do
			if is_absolute then
				precursor {DRW_CUBIC_CURVE} (dx, dy)
			end
		end

	transform_by (m: MATRIX) is
		local
			m1: MATRIX
		do
			if is_absolute then
				precursor {DRW_CUBIC_CURVE} (m)
			else
				create m1.make_arbitrary (m.a, m.b, m.c, m.d, 0, 0)
				precursor {DRW_CUBIC_CURVE} (m1)
			end
		end

feature -- Display

	svg_path: STRING is
		do
			inspect type_code
			when {DRW_COMMAND_TYPE}.drw_pathseg_curveto_cubic_abs then
				Result := "C"
				Result := Result + cx1.out + "," + cy1.out + " " + cx2.out + "," + cy2.out + " "
			when {DRW_COMMAND_TYPE}.drw_pathseg_curveto_cubic_rel then
				Result := "c"
				Result := Result + cx1.out + "," + cy1.out + " " + cx2.out + "," + cy2.out + " "
			when {DRW_COMMAND_TYPE}.drw_pathseg_curveto_cubic_smooth_abs then
				Result := "S"
				Result := Result + cx2.out + "," + cy2.out + " "
			when {DRW_COMMAND_TYPE}.drw_pathseg_curveto_cubic_smooth_rel then
				Result := "s"
				Result := Result + cx2.out + "," + cy2.out + " "
			else
				Result := "?"
			end
			Result := Result + x.out + "," + y.out
		end

feature {DRW_PATH} -- Size estimating

	expand_bounding_box (context: DRW_PATH) is
			-- Expands the bonding_box info in `context' include command points
		local
			bb: DRW_BASIC_RECTANGLE
		do
			bb := context.expanding_box
			-- here we should drawout the curve???
			if is_absolute then
				bb.include_coordinate (current)
				if not is_smooth then
					bb.include (cx1, cy1)
				end
				bb.include (cx2, cy2)
				context.current_point.set (x, y)
			else
				bb.include (context.current_point.x + x, context.current_point.y + y)
				if not is_smooth then
					bb.include (context.current_point.x + cx1, context.current_point.y + cy1)
				end
				bb.include (context.current_point.x + cx2, context.current_point.y + cy2)
				context.current_point.set (context.current_point.x + x, context.current_point.y + y)
			end
		end


feature {NONE} -- Implementation

	key_values: TUPLE [DOUBLE, DOUBLE, DOUBLE, DOUBLE, DOUBLE, DOUBLE] is
		do
			Result := [x, y, cx1, cy1, cx2, cy2]
		end

end -- class DRW_CURVE3
