indexing
	description:
		"A drawing document."
	author: "Barden E. Smith"
	date:	"August 13, 2004"

class
	DRW_DOCUMENT

inherit
	ANY
		export
			{NONE} default_create
		redefine
			default_create
		end
	DRW_TRANSFORMABLE
		export
			{DRW_PAGE, DRW_PROJECTOR} transform
			{NONE} all
		redefine
			default_create
		end

	DRW_VIEW_BOX
		export
			{ANY} has_view_box
			{DRW_PAGE, DRW_PROJECTOR} view_box
			{NONE} all
		redefine
			default_create
		end

create
	default_create,
	make_sized


feature {NONE} -- Creation

	default_create is
		do
			create pages.make (0)
		end

	make_sized (p: DRW_PAPER_SIZE) is
		do
			default_create
			set_paper_size (p)
		end

feature -- Access

	common_page: DRW_PAGE

	pages: ARRAYED_LIST [DRW_PAGE]
		-- The pages of the document

	paper: DRW_PAPER_SIZE

feature -- Modification

	set_common (cp: DRW_PAGE) is
		do
			if has_view_box and not cp.has_view_box then
				cp.set_view_box (view_box)
			end
			common_page := cp
		end


	add_pages (list: LIST [DRW_PAGE]) is
		do
			from
				list.start
			until
				list.after
			loop
				add_page (list.item)
				list.forth
			end
		end

	add_page (p: DRW_PAGE) is
		do
			if has_view_box and not p.has_view_box then
				p.set_view_box (view_box)
			end
			pages.extend (p)
			p.set_parent_document (Current)
		end

	set_paper_size (p: DRW_PAPER_SIZE) is
		local
			p0, p1: DRW_COORDINATE
		do
			paper := p
			create p0.set (p.left_margin, p.bottom_margin)
			create p1.set (p.paper_width - p.right_margin, p.paper_height - p.top_margin)
			create view_box.make_by_coordinates (p0, p1)
		end

feature -- Size estimating

	bounding_box: DRW_BASIC_RECTANGLE is
				-- Void if no pages or view_box
		do
			if has_view_box then
				Result := view_box.twin
			elseif pages /= Void and then not pages.is_empty then
				Result := bounding_box_pages
			end
		end

	bounding_box_pages: DRW_BASIC_RECTANGLE is
				-- Void if no pages
		local
			b: DRW_BASIC_RECTANGLE
			p: DRW_PAGE
			i: INTEGER
		do
			if common_page /= Void then
				b := common_page.bounding_box
				if b /= Void then
					create Result.make (b.width, b.height, b.x, b.y)
				end
			end
			if pages /= Void and then not pages.is_empty then
				from
					i := 1
				until
					i > pages.count
				loop
					p := pages.i_th (i)
					b := p.bounding_box
					if Result /= Void then
						Result.merge (b)
					elseif b /= Void then
						create Result.make (b.width, b.height, b.x, b.y)
					end
					i := i + 1
				end
			end
		end


feature -- plotting routine

	project (plotter: DRW_FIGURE_DRAWING_ROUTINES) is
		do
			from
				pages.start
			until
				pages.after
			loop
				pages.item.project (plotter)
				pages.forth
			end
		end

end -- class DRW_DOCUMENT
