indexing
	description:
		"A drawing command to set or draw-to a new coordinate."
	author: "Barden E. Smith"
	date:	"August 13, 2004"

class
	DRW_DRAWTO

inherit
	DRW_COMMAND
		redefine
			move_by, transform_by
		end


create
	make_lineto_abs,
	make_lineto_rel,
	make_lineto_h_abs,
	make_lineto_h_rel,
	make_lineto_v_abs,
	make_lineto_v_rel,
	make_moveto_abs,
	make_moveto_rel


feature -- Access

	is_invisible: BOOLEAN

	is_visible: BOOLEAN is
		do
			Result := not is_invisible
		end

feature {NONE} -- Creation

	make_lineto_abs (a_x, a_y: DOUBLE) is
			-- Assign parameters.
		do
			set (a_x, a_y)
			set_code ({DRW_COMMAND_TYPE}.drw_pathseg_lineto_abs)
			is_absolute := True
		ensure
			x_assigned: x = a_x
			y_assigned: y = a_y
		end

	make_lineto_rel (a_x, a_y: DOUBLE) is
			-- Assign parameters.
		do
			set (a_x, a_y)
			set_code ({DRW_COMMAND_TYPE}.drw_pathseg_lineto_rel)
		ensure
			x_assigned: x = a_x
			y_assigned: y = a_y
		end

	make_lineto_h_abs (a_x: DOUBLE) is
			-- Assign parameters.
		do
			set (a_x, 0)
			set_code ({DRW_COMMAND_TYPE}.drw_pathseg_lineto_horizontal_abs)
			is_absolute := True
			is_horizontal := True
		ensure
			x_assigned: x = a_x
		end

	make_lineto_h_rel (a_x: DOUBLE) is
			-- Assign parameters.
		do
			set (a_x, 0)
			set_code ({DRW_COMMAND_TYPE}.drw_pathseg_lineto_horizontal_rel)
			is_horizontal := True
		ensure
			x_assigned: x = a_x
		end

	make_lineto_v_abs (a_y: DOUBLE) is
			-- Assign parameters.
		do
			set (0, a_y)
			set_code ({DRW_COMMAND_TYPE}.drw_pathseg_lineto_vertical_abs)
			is_absolute := True
			is_vertical := True
		ensure
			y_assigned: y = a_y
		end

	make_lineto_v_rel (a_y: DOUBLE) is
			-- Assign parameters.
		do
			set (0, a_y)
			set_code ({DRW_COMMAND_TYPE}.drw_pathseg_lineto_vertical_rel)
			is_vertical := True
		ensure
			y_assigned: y = a_y
		end

	make_moveto_abs (a_x, a_y: DOUBLE) is
			-- Assign parameters.
		do
			set (a_x, a_y)
			set_code ({DRW_COMMAND_TYPE}.drw_pathseg_moveto_abs)
			is_invisible := True
			is_absolute := True
		ensure
			x_assigned: x = a_x
			y_assigned: y = a_y
		end

	make_moveto_rel (a_x, a_y: DOUBLE) is
			-- Assign parameters.
		do
			set (a_x, a_y)
			set_code ({DRW_COMMAND_TYPE}.drw_pathseg_moveto_rel)
			is_invisible := True
		ensure
			x_assigned: x = a_x
			y_assigned: y = a_y
		end

	
feature -- Element change

	move_by (dx, dy: DOUBLE) is
		do
			if is_absolute then
				precursor (dx, dy)
			end
		end

	transform_by (m: MATRIX) is
		local
			p: VIRTUAL_POINT
		do
			if is_absolute then
				precursor (m)
			else
				p := m.dtransform_xy (x, y)
				x := p.x
				y := p.y
			end
		end

feature -- Display

	svg_path: STRING is
		do
			inspect type_code
			when {DRW_COMMAND_TYPE}.drw_pathseg_lineto_abs then
				Result := "L"
			when {DRW_COMMAND_TYPE}.drw_pathseg_lineto_rel then
				Result := "l"
			when {DRW_COMMAND_TYPE}.drw_pathseg_lineto_horizontal_abs then
				Result := "H"
			when {DRW_COMMAND_TYPE}.drw_pathseg_lineto_horizontal_rel then
				Result := "h"
			when {DRW_COMMAND_TYPE}.drw_pathseg_lineto_vertical_abs then
				Result := "V"
			when {DRW_COMMAND_TYPE}.drw_pathseg_lineto_vertical_rel then
				Result := "v"
			when {DRW_COMMAND_TYPE}.drw_pathseg_moveto_abs then
				Result := "M"
			when {DRW_COMMAND_TYPE}.drw_pathseg_moveto_rel then
				Result := "m"
			end
			Result := Result + x.out + "," + y.out
		end

feature {DRW_PATH} -- Size estimating

	expand_bounding_box (context: DRW_PATH) is
			-- Expands the bonding_box info in `context' include command points
		local
			bb: DRW_BASIC_RECTANGLE
		do
			bb := context.expanding_box
			if is_absolute then
				if is_vertical then
					context.current_point.set_y (y)
				elseif is_horizontal then
					context.current_point.set_x (x)
				else
					context.current_point.set (x, y)
				end
			else
				if is_vertical then
					context.current_point.set_y (context.current_point.y + y)
				elseif is_horizontal then
					context.current_point.set_x (context.current_point.x + x)
				else
					context.current_point.set (context.current_point.x + x, context.current_point.y + y)
				end
			end
			bb.include_coordinate (context.current_point)
		end

feature {NONE} -- Implementation

	is_horizontal: BOOLEAN

	is_vertical: BOOLEAN

	key_values: TUPLE [DOUBLE, DOUBLE] is
		do
			Result := [x, y]
		end

end -- class DRW_DRAWTO
