indexing
	description:
		"A drawing ellipse."
	author: "Barden E. Smith"
	date:	"August 13, 2004"

class
	DRW_ELLIPSE

inherit
	DRW_SHAPE

	DRW_FILLABLE
		undefine
			out
		end

	DRW_COORDINATE
		undefine
			is_equal,
			out
		end

create
	make,
	make_abs

feature {NONE} -- Creation

	make (r_x, r_y, a_x, a_y: DOUBLE) is
			-- Assign parameters.
		do
			set (a_x, a_y)
			set_radius (r_x, r_y)
		ensure
			radius_assigned: rx = r_x and ry = r_y
			x_assigned: x = a_x
			y_assigned: y = a_y
		end

	make_abs (r_x, r_y, rot:DOUBLE large_arc, sweep: BOOLEAN a_x, a_y: DOUBLE) is
			-- Assign parameters.
		do
			set (a_x, a_y)
			set_radius (r_x, r_y)
			is_large_arc := large_arc
			is_sweep := sweep
		ensure
			radius_assigned: rx = r_x and ry = r_y
			x_assigned: x = a_x
			y_assigned: y = a_y
		end

	
feature -- Access

	rx: DOUBLE
			-- radius along x-axis.

	ry: DOUBLE
			-- Radius along y-axis.

	rotation: DOUBLE
			-- Rotation of elipse (degrees ccw)

	is_large_arc: BOOLEAN
			--  set if large arc

	is_sweep: BOOLEAN
			--  set if sweep

feature -- Element change

	set_radius (r_x, r_y: DOUBLE) is
			-- Assign size of the elipse.
		do
			rx := r_x
			ry := r_y
		ensure
			rx_assigned: rx = r_x
			ry_assigned: ry = r_y
		end


feature -- Size estimating

	bounding_box:DRW_BASIC_RECTANGLE is
		do
			Result := box_transformed (create {DRW_BASIC_RECTANGLE}.make (2.0 * rx, 2.0 * ry, x, y))
		end

feature {DRW_PROJECTOR} -- plotting routine

	project (plotter: DRW_FIGURE_DRAWING_ROUTINES) is
		do
			if is_drawable then
				plotter.draw_figure_ellipse (current)
			end
		end

end -- class DRW_ELLIPSE
