indexing
	description:
		"An Eiffel Vision drawing command processor."
	author: "Barden E. Smith"
	date:	"October 26, 2004"


class
	DRW_EV_COMMAND_PROCESSOR

inherit
	DRW_COMMAND_PROCESSOR

	EV_FIGURE_MATH
		rename
			modulo as buggy_modulo
		export
			{NONE} all
		end

feature {NONE} -- Implementation state

	drawable: EV_DRAWABLE

	m_stack: ARRAYED_LIST [DRW_MATRIX]

	ctm: DRW_MATRIX is
		require
			initialized: m_stack /= Void
		do
			Result := m_stack.last
		end

	cur_x: DOUBLE

	cur_y: DOUBLE

	first_x: DOUBLE

	first_y: DOUBLE

	next_control: DRW_COORDINATE

feature {NONE} -- Implementation

	arcto (code: INTEGER x, y, rrx, rry, rot: DOUBLE fa, fs: BOOLEAN) is
		local
			xx1,yy1,xx2,yy2, rx, ry, xp, yp, ss, aa, th, dth: DOUBLE
			rr: BOOLEAN
			cxp, cyp: DOUBLE
			cx, cy: INTEGER
			pt: POINT
		do
			rr := ctm.discriminant < 0
			pt := ctm.transform_xy (cur_x, cur_y)
			xx1 := pt.x
			yy1 := pt.y
			pt := ctm.transform_xy (cur_x + rrx, cur_y + rry)
			rx := (pt.x - xx1).abs
			ry := (pt.y - yy1).abs
			if rx = 0 or ry = 0 then
				drawto (code - 6, x, y)
			else
				if code = 10 then
					cur_x := x
					cur_y := y
				else
					cur_x := cur_x + x
					cur_y := cur_y + y
				end
				pt := ctm.transform_xy (cur_x, cur_y)
				xx2 := pt.x
				yy2 := pt.y
				if rot /= 0 or not ctm.is_manhattan then
					-- FIXME:  Implement this case
					-- until then, just draw a straight line
					drawable.draw_segment(xx1.truncated_to_integer,
								yy1.truncated_to_integer,
								xx2.truncated_to_integer,
								yy2.truncated_to_integer)
				else
					xp := (xx2 - xx1) / 2.0
					yp := (yy2 - yy1) / 2.0
					aa := (( xp*xp / (rx*rx)) + (yp*yp / (ry*ry)))
					-- if aa is greater than 1 then we need to increase the radii
					if aa > 1.0 then
						aa := sqrt(aa)
						rx := aa * rx
						ry := aa * ry
						ss := 0.0
					else
						ss := (rx*rx*ry*ry - rx*rx*yp*yp - ry*ry*xp*xp) /
								(rx*rx*yp*yp + ry*ry*xp*xp)
						if ss > 0 then
							ss := sqrt (ss)
							if (fa xor fs) xor rr then
								ss := - ss
							end
						else
							ss := 0
						end
					end
					cxp := ((  ss * (rx*yp/ry)) + ((xx1 + xx2) / 2.0) )
					cyp := ((- ss * (ry*xp/rx)) + ((yy1 + yy2) / 2.0) )
					cx := cxp.truncated_to_integer
					cy := cyp.truncated_to_integer
					th := line_angle (cx,
							- cy,
							xx1.rounded,
							- yy1.rounded)
					dth := line_angle (cx,
							- cy,
							xx2.rounded,
							- yy2.rounded) - th
					if (fs xor rr) then
						if dth > 0 then
							dth := dth - (2.0 * pi)
						end
					elseif dth < 0 then
						dth := dth + (2.0 * pi)
					end
					drawable.draw_arc ((cx.to_double - rx).truncated_to_integer, (cy.to_double - ry).truncated_to_integer,
							(2.0*rx).rounded,
							(2.0*ry).rounded,
							th, dth)
				end
			end
		end

	close (code: INTEGER) is
		local
			ix1, iy1, ix2, iy2: INTEGER
			pt: POINT
		do
			pt := ctm.transform_xy (cur_x, cur_y)
			ix1 := pt.x.truncated_to_integer
			iy1 := pt.y.truncated_to_integer
			pt := ctm.transform_xy (first_x, first_y)
			ix2 := pt.x.truncated_to_integer
			iy2 := pt.y.truncated_to_integer
			drawable.draw_segment(ix1, iy1, ix2, iy2)
			cur_x := first_x
			cur_y := first_y
		end

	curveto2 (code: INTEGER x, y, x1, y1: DOUBLE) is
		local
			xa, ya, xb, yb, xn, yn: DOUBLE
		do
			if code < 10 then
				xa := (2.0 * x1 + x) / 3.0
				ya := (2.0 * y1 + y) / 3.0
				xn := x - x1
				yn := y - y1
			elseif code = 18 then
				xa := (2.0 * next_control.x) / 3.0
				ya := (2.0 * next_control.y) / 3.0
				xn := x - next_control.x
				yn := y - next_control.y
			elseif code = 19 then
				xa := x + ((2.0 * next_control.x + x) / 3.0)
				ya := x + ((2.0 * next_control.y + y) / 3.0)
				xn := x - next_control.x
				yn := y - next_control.y
			else -- FIXME: raise error
			end
			xb := (2.0 * x1 + x) / 3.0
			yb := (2.0 * y1 + y) / 3.0
			if code < 10 then
				curveto3 (code - 2, x, y, xa, ya, xb, yb)
				next_control.set (x - x1, y - y1)
			else
				curveto3 (code - 12, x, y, xa, ya, xb, yb)
				next_control.set (xn, yn)
			end
		end

	curveto3 (code: INTEGER x, y, x1, y1, x2 ,y2: DOUBLE) is
		local
			p1, p2, p3, p4: POINT
			v1x, v1y, v2x, v2y: DOUBLE
			deltastep, deltastep2, deltastep3: DOUBLE
			xdelta, xdelta2, xdelta3: DOUBLE
			ydelta, ydelta2, ydelta3: DOUBLE
			ax, ay, bx, by, cx, cy: DOUBLE
			i, nsteps: INTEGER
		do
			inspect code
			when 6 then
				p1 := ctm.transform_xy (cur_x, cur_y)
				p2 := ctm.transform_xy (x1, y1)
				p3 := ctm.transform_xy (x2, y2)
				p4 := ctm.transform_xy (x, y)
				cur_x := x
				cur_y := y
			when 7 then
				p1 := ctm.transform_xy (cur_x, cur_y)
				p2 := ctm.transform_xy (cur_x + x1, cur_y + y1)
				p3 := ctm.transform_xy (cur_x + x2, cur_y + y2)
				p4 := ctm.transform_xy (cur_x + x, cur_y + y)
				cur_x := cur_x + x
				cur_y := cur_y + y
			when 16 then
				p1 := ctm.transform_xy (cur_x, cur_y)
				p2 := ctm.transform_xy (next_control.x + cur_x, next_control.y + cur_y)
				p3 := ctm.transform_xy (x2, y2)
				p4 := ctm.transform_xy (x, y)
				cur_x := x
				cur_y := y
			when 17 then
				p1 := ctm.transform_xy (cur_x, cur_y)
				p2 := ctm.transform_xy (next_control.x + cur_x, next_control.y + cur_y)
				p3 := ctm.transform_xy (cur_x + x2, cur_y + y2)
				p4 := ctm.transform_xy (cur_x + x, cur_y + y)
				cur_x := cur_x + x
				cur_y := cur_y + y
			else -- FIXME: raise error
			end
			next_control.set (x - x2, y - y2)
			nsteps := 32;
			deltastep := 1.0 / nsteps
			deltastep2 := deltastep*deltastep
			deltastep3 := deltastep2*deltastep

			ax := -p1.x + 3.0*p2.x - 3.0*p3.x + p4.x
			ay := -p1.y + 3.0*p2.y - 3.0*p3.y + p4.y
			bx := 3.0*p1.x - 6.0*p2.x + 3.0*p3.x
			by := 3.0*p1.y - 6.0*p2.y + 3.0*p3.y
			cx := -3.0*p1.x + 3.0*p2.x
			cy := -3.0*p1.y + 3.0*p2.y

			xdelta := ax*deltastep3 + bx*deltastep2 + cx*deltastep
			ydelta := ay*deltastep3 + by*deltastep2 + cy*deltastep
			xdelta2 := 6.0*ax*deltastep3 + 2.0*bx*deltastep2
			ydelta2 := 6.0*ay*deltastep3 + 2.0*by*deltastep2
			xdelta3 := 6.0*ax*deltastep3
			ydelta3 := 6.0*ay*deltastep3

			v1x := p1.x
			v1y := p1.y

			from
				i := 0
			until
				i >= nsteps
			loop
				v2x := v1x + xdelta
				xdelta := xdelta + xdelta2
				xdelta2 := xdelta2 + xdelta3
				v2y := v1y + ydelta
				ydelta := ydelta + ydelta2
				ydelta2 := ydelta2 + ydelta3
				drawable.draw_segment (
					v1x.truncated_to_integer,
					v1y.truncated_to_integer,
					v2x.truncated_to_integer,
					v2y.truncated_to_integer)
				v1x := v2x
				v1y := v2y
				i := i + 1
			end
		end

	drawto (code: INTEGER x, y: DOUBLE) is
		local
			ix1, iy1, ix2, iy2: INTEGER
			pt: POINT
		do
			inspect code
				when 2 then
					cur_x := x
					cur_y := y
					first_x := cur_x
					first_y := cur_y
				when 3 then
					cur_x := cur_x + x
					cur_y := cur_y + y
					first_x := cur_x
					first_y := cur_y
				when 4 then
					pt := ctm.transform_xy (cur_x, cur_y)
					ix1 := pt.x.truncated_to_integer
					iy1 := pt.y.truncated_to_integer
					cur_x := x
					cur_y := y
					pt := ctm.transform_xy (cur_x, cur_y)
					ix2 := pt.x.truncated_to_integer
					iy2 := pt.y.truncated_to_integer
					drawable.draw_segment(ix1, iy1, ix2, iy2)
				when 5 then
					pt := ctm.transform_xy (cur_x, cur_y)
					ix1 := pt.x.truncated_to_integer
					iy1 := pt.y.truncated_to_integer
					cur_x := cur_x + x
					cur_y := cur_y + y
					pt := ctm.transform_xy (cur_x, cur_y)
					ix2 := pt.x.truncated_to_integer
					iy2 := pt.y.truncated_to_integer
					drawable.draw_segment(ix1, iy1, ix2, iy2)
				when 12 then
					pt := ctm.transform_xy (cur_x, cur_y)
					ix1 := pt.x.truncated_to_integer
					iy1 := pt.y.truncated_to_integer
					cur_x := x
					-- cur_y := y
					pt := ctm.transform_xy (cur_x, cur_y)
					ix2 := pt.x.truncated_to_integer
					iy2 := pt.y.truncated_to_integer
					drawable.draw_segment(ix1, iy1, ix2, iy2)
				when 13 then
					pt := ctm.transform_xy (cur_x, cur_y)
					ix1 := pt.x.truncated_to_integer
					iy1 := pt.y.truncated_to_integer
					cur_x := cur_x + x
					-- cur_y := cur_y + y
					pt := ctm.transform_xy (cur_x, cur_y)
					ix2 := pt.x.truncated_to_integer
					iy2 := pt.y.truncated_to_integer
					drawable.draw_segment(ix1, iy1, ix2, iy2)
				when 14 then
					pt := ctm.transform_xy (cur_x, cur_y)
					ix1 := pt.x.truncated_to_integer
					iy1 := pt.y.truncated_to_integer
					-- cur_x := x
					cur_y := y
					pt := ctm.transform_xy (cur_x, cur_y)
					ix2 := pt.x.truncated_to_integer
					iy2 := pt.y.truncated_to_integer
					drawable.draw_segment(ix1, iy1, ix2, iy2)
				when 15 then
					pt := ctm.transform_xy (cur_x, cur_y)
					ix1 := pt.x.truncated_to_integer
					iy1 := pt.y.truncated_to_integer
					-- cur_x := cur_x + x
					cur_y := cur_y + y
					pt := ctm.transform_xy (cur_x, cur_y)
					ix2 := pt.x.truncated_to_integer
					iy2 := pt.y.truncated_to_integer
					drawable.draw_segment(ix1, iy1, ix2, iy2)
			end		
		end

feature -- Testing

	ellipse_as_curves (cx, cy, rot, rx, ry, th, dth: DOUBLE): LIST [DRW_CUBIC_CURVE] is
		local
			m: DRW_MATRIX
			sections: LIST [DRW_CUBIC_CURVE]
		do
			create m.make_rotation_degrees (rot)
			sections := arc_as_curves (th, dth, m)
		end

feature {NONE} -- Implementation support

	arc_as_curves (th, dth: DOUBLE at: DRW_MATRIX): LIST [DRW_CUBIC_CURVE] is
			-- for a starting `th' in degrees in [-360, 360]
			-- and a sweep angle `dth' in [-360, 360] degrees.
			-- produce a sequence of cubic bezier curves for the arc
			-- transformed by `at'
		require
			dth_in_range: -360.0 <= dth and dth <= 360.0
		local
			m: DRW_MATRIX
			section: DRW_CUBIC_CURVE
			th0, dth0: DOUBLE
			section_count: INTEGER
		do
			if dth = 0 then
				create {ARRAYED_LIST [DRW_CUBIC_CURVE]} Result.make (0)
			elseif dth < 0 then
				create m.make_arbitrary (1.0, 0.0, 0.0, -1.0, 0.0, 0.0)
				if at /= void then
					m := m.and_then (at)
				end
				Result := arc_as_curves (-th, -dth, m)
			else
				from
					section_count :=  (dth / 90).truncated_to_integer
					if 90.0 * section_count < dth then
						-- usual case (unless multiple of 90 degrees)
						section_count := section_count + 1
					end
					create {ARRAYED_LIST [DRW_CUBIC_CURVE]} Result.make (section_count)
					dth0 := dth / section_count
					th0 := th
				until
					section_count <= 0
				loop
					create m.make_rotation_degrees (modulo (th0 + (0.5 * dth0), 360.0))
					if at /= Void then
						m := m.and_then (at)
					end
					section := small_arc_as_curve (dth0)
					section.transform_by (m)
					Result.extend (section)
					section_count := section_count - 1
					th0 := th0 + dth0
				end
			end
		end

	small_arc_as_curve (ang: DOUBLE): DRW_CUBIC_CURVE is
			-- create arc centered at (0,0) with unit radius
			-- and a span of `ang' degrees.  Midpoint at (1,0)
		require
			positive_arc: ang > 0.0
			small_arc: ang <= 90.0
		local
			half_arc, x0, y0, x1, y1, x2, y2, x3, y3: DOUBLE
		do
			-- P0 is starting point. A positive `ang' requires
			-- P0 at (cos (-ang/2), sin (-ang/2))
			half_arc := pi * ang / -360.0
			x0 := cosine (half_arc)
			y0 := sine (half_arc)
			-- check ang != 0 implies y0 != 0.0
			x3 := x0
			y3 := -y0
			x1 := (4.0 - x0) / 3.0
			y1 := (1.0 - x0) * (3.0 - x0) / (3.0 * y0)
			x2 := x1
			y2 := -y1
			create Result.make (x0, y0, x1, y1, x2, y2, x3, y3)
		end

	modulo (a, b: DOUBLE): DOUBLE is
			-- `a' modulo `b'.
			--| Should be in DOUBLE_REF.
		require
			divisible: b /= 0.0
		do
			if b < 0 then
				Result := - buggy_modulo (a, -b)
			else
				Result := buggy_modulo (a, b)
			end
		ensure
			in_interval: b.sign * Result >= 0.0 and Result.abs < b.abs
		end

end -- class DRW_EV_COMMAND_PROCESSOR
