indexing
	description:
		"Plotter to output a drawing to a Eiffel Vision 2 Drawable."
	author: "Barden E. Smith"
	date:	"September 15, 2004"


class
	DRW_EV_PROJECTOR

inherit
	DRW_PROJECTOR_ROUTINES

	DRW_VIEW_BOX
		export
			{NONE} all
		end

	DRW_EV_COMMAND_PROCESSOR

create
	make

feature {NONE} -- Initialization

	make (a_drawable: EV_DRAWABLE) is
			-- Create a projector for  `a_drawable'.
		require
			a_drawable_not_void: a_drawable /= Void
		local
			h, w: INTEGER
			cx, cy: DOUBLE
		do
			drawable := a_drawable
			create m_stack.make (1)
			create text_pixmap
			create font
			create unit_rectangle
			w := a_drawable.width - 1
			h := a_drawable.height - 1
			cx := w / 2.0
			cy := h / 2.0
			create view_box.make (w, h, cx, cy)
			mirror_y := True
			make_processor
		end

feature -- Figure drawing

	start_page (v: DRW_BASIC_RECTANGLE page_label:STRING) is
		do
			drawable.clear
			m_stack.extend (matrix_map (v))
		end

	finish_page is
		do
			m_stack.wipe_out
		end

	alter_current_transform (m: DRW_MATRIX) is
		do
			m_stack.extend (m.and_then (ctm))
		end

	restore_previous_transform is
		do
			m_stack.finish
			m_stack.remove
		end

	draw_figure_circle (circle: DRW_CIRCLE) is
			-- Draw standard representation of `circle' to canvas.
		local
			x, y, rx, ry: DOUBLE
			ix, iy, idx, idy: INTEGER
			lw: INTEGER
			pt: POINT
		do
			lw := drawable.line_width	-- save it
			if circle.is_transformed then
				alter_current_transform (circle.transform)
			end
			if circle.stroke_width > 0 then
				drawable.set_line_width ((circle.transformed_stroke_width (ctm)).truncated_to_integer)
			end
			pt := ctm.transform_xy (circle.x, circle.y)
			x := pt.x
			y := pt.y
			pt := ctm.transform_xy (circle.x + circle.r, circle.y + circle.r)
			rx := (pt.x - x).abs
			ry := (pt.y - y).abs
			ix := (x - rx).truncated_to_integer
			idx := (2.0 * rx).truncated_to_integer
			iy := (y - ry).truncated_to_integer
			idy := (2.0 * ry).truncated_to_integer
			drawable.draw_ellipse (ix, iy, idx, idy)
			if circle.is_transformed then
				restore_previous_transform
			end
			drawable.set_line_width (lw)
		end

	draw_figure_ellipse (ellipse: DRW_ELLIPSE) is
			-- Draw standard representation of `ellipse' to canvas.
		local
			x, y, rx, ry: DOUBLE
			ix, iy, idx, idy: INTEGER
			lw: INTEGER
			pt: POINT
		do
			lw := drawable.line_width	-- save it
			if ellipse.is_transformed then
				alter_current_transform (ellipse.transform)
			end
			if ellipse.stroke_width > 0 then
				drawable.set_line_width ((ellipse.transformed_stroke_width (ctm)).truncated_to_integer)
			end
			pt := ctm.transform_xy (ellipse.x, ellipse.y)
			x := pt.x
			y := pt.y
			pt := ctm.transform_xy (ellipse.x + ellipse.rx, ellipse.y + ellipse.ry)
			rx := (pt.x - x).abs
			ry := (pt.y - y).abs
			ix := (x - rx).truncated_to_integer
			idx := (2.0 * rx).truncated_to_integer
			iy := (y - ry).truncated_to_integer
			idy := (2.0 * ry).truncated_to_integer
			drawable.draw_ellipse (ix, iy, idx, idy)
			if ellipse.is_transformed then
				restore_previous_transform
			end
			drawable.set_line_width (lw)
		end

	draw_figure_line (line: DRW_LINE) is
			-- Draw a standard representation of `line' to the canvas.
		local
			ix1, iy1, ix2, iy2: INTEGER
			m: DRW_MATRIX
			lw: INTEGER
			pt: POINT
		do
			lw := drawable.line_width	-- save it
			if line.is_transformed then
				m := line.transform.and_then (ctm)
			else
				m := ctm
			end
			if line.stroke_width > 0 then
				drawable.set_line_width ((line.transformed_stroke_width (ctm)).truncated_to_integer)
			end
			pt := m.transform_xy (line.x1, line.y1)
			ix1 := pt.x.truncated_to_integer
			iy1 := pt.y.truncated_to_integer
			pt := m.transform_xy (line.x2, line.y2)
			ix2 := pt.x.truncated_to_integer
			iy2 := pt.y.truncated_to_integer
			drawable.draw_segment(ix1, iy1, ix2, iy2)
			drawable.set_line_width (lw)
		end

--	draw_figure_image (image: DRW_IMAGE) is
			-- Draw standard representation of `image' to canvas.
--		do
--		end

	draw_figure_polygon (polygon: DRW_POLYGON) is
			-- Draw standard representation of `polygon' to canvas.
		local
			dc: DRW_COLOR
			ec: EV_COLOR
		do
			if polygon.has_fill then
				dc := polygon.fill
				create ec.make_with_rgb (dc.red, dc.green, dc.blue)
			end
			draw_list_of_points (polygon.as_transformed_points (ctm), True, polygon.is_stroked, ec)
		end

	draw_figure_polyline (lines: DRW_POLYLINE) is
			-- Draw standard representation of `lines' to canvas.
		do
			draw_list_of_points (lines.as_transformed_points (ctm), False, lines.is_stroked, Void)
		end

	draw_figure_rectangle (rectangle: DRW_RECTANGLE) is
			-- Draw standard representation of `rectangle' to canvas.
		local
			b: DRW_BASIC_RECTANGLE
			x0, x1, y0, y1, lw: INTEGER
			m: DRW_MATRIX
			pt: POINT
		do
			lw := drawable.line_width	-- save it
			if rectangle.is_transformed then
				m := rectangle.transform.and_then (ctm)
			else
				m := ctm
			end
			if rectangle.stroke_width > 0 then
				drawable.set_line_width ((rectangle.transformed_stroke_width (m)).truncated_to_integer)
			end
			if m.is_manhattan then
				b := rectangle.bounding_box -- rectangle transformed by local transform
				pt := ctm.transform_xy (b.left, b.bottom)
				x0 := pt.x.truncated_to_integer
				y0 := pt.y.truncated_to_integer
				pt := ctm.transform_xy (b.right, b.top)
				x1 := pt.x.truncated_to_integer
				y1 := pt.y.truncated_to_integer
				drawable.draw_rectangle (x0.min (x1), y1.min (y0), (x1 - x0).abs.max(1), (y1 - y0).abs.max(1))
			else
				draw_list_of_points (rectangle.as_transformed_points (m), True, rectangle.is_stroked, Void)
			end
			drawable.set_line_width (lw)
		end

	draw_figure_soft_rectangle (rectangle: DRW_SOFT_RECTANGLE) is
			-- Draw standard representation of `rectangle' to canvas.
		local
			lp: SIMPLE_LIST_OF_POINTS
			i, lw, x0, x1, x2, y0, y1, y2, w, h: INTEGER
			m: DRW_MATRIX
		do
			lw := drawable.line_width	-- save it
			if rectangle.is_transformed then
				m := rectangle.transform.and_then (ctm)
			else
				m := ctm
			end
			if rectangle.stroke_width > 0 then
				drawable.set_line_width ((rectangle.transformed_stroke_width (m)).truncated_to_integer)
			end
			lp := rectangle.as_transformed_points (m)
			if lp.count > 4 then
				from
					i := lp.count
					x2 := lp.i_th_x (i).truncated_to_integer
					y2 := lp.i_th_y (i).truncated_to_integer
					i := 1
				until
					i > lp.count
				loop
					x0 := lp.i_th_x (i).truncated_to_integer
					y0 := lp.i_th_y (i).truncated_to_integer
					drawable.draw_segment (x2, y2, x0, y0)
					i := i + 1
					x1 := lp.i_th_x (i).truncated_to_integer
					y1 := lp.i_th_y (i).truncated_to_integer
					i := i + 1
					x2 := lp.i_th_x (i).truncated_to_integer
					y2 := lp.i_th_y (i).truncated_to_integer
					if x0 = x1 then
						w := 2 * (x2 - x1)
					else
						w := 2 * (x1 - x0)
					end
					if y0 = y1 then
						h := 2 * (y2 - y1)
					else
						h := 2 * (y1 - y0)
					end
					if h > 0 and w > 0 then
						x0 := x1
						y0 := y1 - h
					elseif h < 0 and w > 0 then
						x0 := x1 - w
						y0 := y1 + h
					elseif h < 0 and w < 0 then
						x0 := x1 + w
						y0 := y1
					else
						x0 := x1
						y0 := y1
					end
					i := i + 1
					drawable.draw_arc (x0, y0, w.abs, h.abs,
							((2 + ((i - 1) // 3)) \\ 4) * pi / 2 + 0.00001,
							pi / 2)
				end
			else
				draw_list_of_points (lp, True, rectangle.is_stroked, Void)
			end
			drawable.set_line_width (lw)
		end

	draw_figure_path (path: DRW_PATH) is
			-- Draw standard representation of `path' to canvas.
		local
			lw: INTEGER
		do
			lw := drawable.line_width	-- save it
			if path.is_transformed then
				alter_current_transform (path.transform)
			end
			if path.stroke_width > 0 then
				drawable.set_line_width ((path.transformed_stroke_width (ctm)).truncated_to_integer)
			end
			from
				path.start
				create next_control.set (0.0, 0.0)
			until
				path.after
			loop
				path.item.execute_command (Current)
				path.forth
			end
			if path.is_transformed then
				restore_previous_transform
			end
			drawable.set_line_width (lw)
		end

	draw_figure_text (text_figure: DRW_TEXT) is
			-- Draw standard representation of `text_figure' to canvas.
		local
			size: TUPLE --[INTEGER, INTEGER]
			b: DRW_BASIC_RECTANGLE
			t_size, f_height: DOUBLE
			x1, y1, x2, y2, w, h: INTEGER
			text: STRING
			m: DRW_MATRIX
			pt: POINT
		do
			t_size := text_figure.size
			if text_figure.is_transformed then
				m := text_figure.transform
				m := m.and_then (ctm)
			else
				m := ctm
			end
			text := text_figure.full_text
			pt :=  m.transform_xy (text_figure.x, text_figure.y)
			x1 := pt.x.truncated_to_integer
			y1 := pt.y.truncated_to_integer
			if text_figure.has_view_box then
				t_size := 30.0
				font.set_height (30)
				text_pixmap.set_font (font)
				size := font.string_size (text)
				w := size.integer_item (1)
				h := size.integer_item (2)
				create b.make (
					w,
					h,
					text_figure.x + (w / 2.0),
					text_figure.y + (h / 2.0))
				m := (text_figure.matrix_map (b)).and_then (m)
				pt := m.transform_xy (text_figure.x, text_figure.y)
				x1 := pt.x.truncated_to_integer
				y1 := pt.y.truncated_to_integer
			end
			if m.is_manhattan then
				pt := m.transform_xy (text_figure.x + t_size, text_figure.y + t_size)
				x2 := pt.x.truncated_to_integer
				y2 := pt.y.truncated_to_integer
				if m.a = 0 then -- rotated (odd multiple of 90)
					f_height := (x2 - x1).abs
				else
					f_height := (y2 - y1).abs
				end
				if f_height > 2 then
					font.set_height (f_height.truncated_to_integer - 1)
					text_pixmap.set_font (font)
					size := font.string_size (text)
					w := size.integer_item (1)
					h := size.integer_item (2)
					text_pixmap.set_size(w, h + font.descent)
					text_pixmap.clear
					text_pixmap.draw_text (0, h, text)
					if m.a > 0 and m.b = 0 and m.c = 0 and m.d < 0 then
						drawable.draw_pixmap (x1, y1 - text_pixmap.height, text_pixmap)
					else
						draw_sub_pixmap_rotated (drawable, x1, y1, text_pixmap, m)
					end
				end
			else
				-- FIXME: add error reporting (undrawable)
			end
		end

feature {NONE} -- Implementation

	text_pixmap: EV_PIXMAP

	font: EV_FONT
	
	establish_document (a_document: DRW_DOCUMENT) is
			-- Finish establishing document
		do
			document := a_document
		end

	unit_rectangle: EV_RECTANGLE

	draw_list_of_points (d: SIMPLE_LIST_OF_POINTS closed, stroked: BOOLEAN fill_color: EV_COLOR) is
			-- Draw the polygon from already transformed list of points
		local
			p: ARRAY [EV_COORDINATE]
			c: EV_COORDINATE
			i: INTEGER
			oc: EV_COLOR
		do
			create p.make (1, d.count)
			from
				i := 1
			until
				i > d.count
			loop
				create c.set (d.i_th_x (i).truncated_to_integer,
						d.i_th_y (i).truncated_to_integer)
				p.put (c, i)
				i := i + 1
			end
			if fill_color /= Void then
				oc := drawable.foreground_color
				drawable.set_foreground_color (fill_color)
				drawable.fill_polygon (p)
				drawable.set_foreground_color (oc)
			end
			if stroked then
				drawable.draw_polyline (p, closed)
			end
		end

	draw_sub_pixmap_rotated (dest: EV_DRAWABLE x, y: INTEGER src_pixmap: EV_PIXMAP orient: DRW_MATRIX) is
			-- Draw `area' of `src_pixmap' rotated using `orient' to define rotation and 
			-- mirroring
		require
			orient_is_manhattan: orient.is_manhattan
		local
			i, j, k, n: INTEGER
			ia, ib, ic, id: INTEGER
			width, height: INTEGER
		do
			width := src_pixmap.width
			height := src_pixmap.height
			ia := orient.a.sign
			ib := orient.b.sign
			ic := orient.c.sign
			id := orient.d.sign
			from
				k := 0
			until
				k > width
			loop
				from
					n := 0
				until
					n > height
				loop
	    				i := ia * k + ib * n + x
	    				j := ic * k + id * n + y
					unit_rectangle.move_and_resize (k, height - n, 1, 1)
					dest.draw_sub_pixmap (i, j, src_pixmap, unit_rectangle)
					n := n + 1
				end
				k := k + 1
			end
		end

end -- class DRW_EV_PROJECTOR
