indexing
	description:
		"A drawing group generator."
	status: "See instruction at end of class."
	author: "Barden E. Smith"
	date:	"November 24, 2004"

class
	DRW_GROUP_GENERATOR

inherit
	DRW_PATH_SCANNER
		redefine
			start_scan,
			dispatch
		end

create
	default_create,
	make

feature {NONE} -- Creation

	make (path: STRING) is
		require
			path_not_void: path /= Void
		do
			set_path (path)
		end

feature -- Status report

	group: DRW_GROUP is
		do
			add_commands
			if shapes /= Void and then not shapes.is_empty then
				create Result.make_from_list (shapes)
			else
				create Result.make (0)
			end
		end

	font: STRING

feature -- Element change

	set_font (f: STRING) is
			-- Set name of the font to use when creating
			-- DRW_TEXT elements
		do
			font := f
		end


feature {NONE} -- Implementation

	shapes: ARRAYED_LIST [DRW_SHAPE]
			-- the shapes from the last scan

	commands_for_array: LIST [DRW_COMMAND]

	add_shape (s: DRW_SHAPE) is
		require
			s_not_void: s /= Void
		do
			shapes.extend (s)
		end

	add_commands is
		do
			if not commands.is_empty then
				add_shape (create {DRW_PATH}.make_from_command_array (commands.twin))
				commands.wipe_out
			end
			commands_for_array := Void
		end

	start_scan is
		do
			create shapes.make (0)
			precursor
		end

	dispatch (c: CHARACTER) is
		do
			inspect c
			when '*' then
				modify_current_path
			when '@' then
				array_commands
			when '|','^' then
				add_commands
				modify_last_shape (c)
			when 'X','x' then
				add_commands
				add_shape (text_in_box (c))
			else
				if commands_for_array /= Void then
					add_commands
				end
				precursor (c)
			end
		end

	modify_last_shape (cmd: CHARACTER) is
		local
			p: ARRAY [DOUBLE]
			l: DRW_SHAPE
			m: DRW_MATRIX
			t: DRW_TEXT
			p1, p2: DRW_COORDINATE
		do
			l := shapes.last
			p := scan_numbers (6)
			if l /= Void then
				t ?= l
				if cmd = '|' then
					if p.count > 0 then
						l.set_stroke_width (p.item (1))
					end
				elseif cmd = '^' then
					if p.count = 6 then
						create m.make_arbitrary (
								p.item (1), p.item (3),
								p.item (2), p.item (4),
								p.item (5), p.item (6))
					elseif p.count = 2 then
						create m.make_translation_xy (
							p.item (1), p.item (2))
					elseif p.count = 1 then
						create m.make_magnification (
							p.item (1))
					end
					if t = Void then
						l.set_transform (m)
					elseif t.has_view_box then
						p1 := t.view_box.lower_left
						p2 := t.view_box.upper_right
						p1.transform_by (m)
						p2.transform_by (m)
						t.set_view_box (create {DRW_BASIC_RECTANGLE}.make_by_coordinates (p1, p2))
					end
				end
			end
		end

	array_commands is
		local
			ax, ay, dx, dy, x0, y0: DOUBLE
			nx, ny, ix, iy: INTEGER
			p: ARRAY [DOUBLE]
			move1: DRW_DRAWTO
		do
			p := scan_numbers (6)
			if p.count = 6 and (not commands.is_empty or commands_for_array /= Void) then
				ax := p.item (1) ay := p.item (2)
				nx := p.item (3).truncated_to_integer
				ny := p.item (4).truncated_to_integer
				dx := p.item (5) dy := p.item (6)
				if commands_for_array = Void then
					commands_for_array := commands	-- handle on current commands
					create commands.make (0) -- recreate commands
				--else
				--  add to commands from existing commands_for_array
				end
				from
					ix := 1
					x0 := ax
				until
					ix > nx
				loop
					from
						iy := 1
						y0 := ay
					until
						iy > ny
					loop
						create move1.make_moveto_abs (x0, y0)
						commands.extend (move1)
						commands.append (commands_for_array)
						y0 := y0 + dy
						iy := iy + 1
					end
					x0 := x0 + dx
					ix := ix + 1
				end
			end
		end

	modify_current_path is
		local
			m: DRW_MATRIX
			p: ARRAY [DOUBLE]
		do
			p := scan_numbers (6)
			if p.count = 6 then
				create m.make_arbitrary (
						p.item (1), p.item (3),
						p.item (2), p.item (4),
						p.item (5), p.item (6))
			elseif p.count = 2 then
				create m.make_translation_xy (
					p.item (1), p.item (2))
			elseif p.count = 1 then
				create m.make_magnification (
					p.item (1))
			end
			if m /= Void and (not commands.is_empty or commands_for_array /= Void) then
				if commands_for_array = Void then
					commands_for_array := commands	-- handle on current commands
					create commands.make (0) -- recreate commands
				--else
				--  add to commands from existing commands_for_array
				end
				from
					commands_for_array.start
				until
					commands_for_array.after
				loop
					commands_for_array.item.transform_by (m)
					commands_for_array.forth
				end
			end
		end

	text_in_box (cmd: CHARACTER): DRW_TEXT is
		local
			bx, by, bw, bh: DOUBLE
			p: ARRAY [DOUBLE]
			b: DRW_BASIC_RECTANGLE
			m: DRW_MATRIX
			s: STRING
			t: DRW_TEXT
		do
			p := scan_numbers (4)
			s := scan_text
			if p.count = 4 and s /= Void then
				bw := p.item (1) bh := p.item (2) bx := p.item (3) by := p.item (4)
				p := scan_numbers (3)
				create t.make (s)
				if font /= Void then
					t.set_font (font)
				end
				if p.count = 3 then
					create m.make_from_matrix (
						create {MATRIX}.make_mx_my_rot90_tx_ty (
							p.item (1) /= 0.0,
							p.item (2) /= 0.0,
							p.item (3) /= 0.0,
							bx,
							by)
						)
					create b.make (bw, bh, 0.0, 0.0)
					t.set_transform (m)
				else
					create b.make (bw, bh, bx, by)
				end
				t.set_size (100)
				t.set_view_box (b)
				Result := t
			elseif s /= Void then
				create Result.make (s)
			else
				create Result.make ("%U")
			end
		end

	scan_text: STRING is
		local
			c, ct: CHARACTER
		do
			skip_space
			if position <= path_value.count then
				c := path_value.item (position)
				inspect c
				when '%"' then
					ct := '%"'
				when '%'' then
					ct := '%''
				when '<' then
					ct := '>'
				else
					ct := '%U'
				end
				if ct /= '%U' then
					from
						position := position + 1
						c := ' '
						create Result.make (0)
					until
						position > path_value.count or
						c = ct
					loop
						c := path_value.item (position)
						Result.extend (c)
						position := position + 1
					end
					if Result.count > 1 then
						Result.remove_tail (1)
					end
				end
			end
		end

indexing
	instructions: "[
	This class is designed to create a drawing group from a string that is a
	compact representation of the graphic.  The syntax is a series of
	DRW_PATH_SCANNER paths followed by one or more operators (special
	characters and values) to affect the path.  The operators are the
	following:

		@ x y nx ny dx dy       expands the (relative) path into an array
		* a b c d tx ty         directly transforms the last path.
		^ a b c d tx ty         transforms the last path.
		| n                     sets the linewidth of the last path.
		X w h cx cy "text" <m>  text-in-box element

	The transform operator may be followed by 1, 2, or 6 numbers; six
	transforms the path by a standard matrix; two scales the path by a
	vector; and one scales the path by a scalar.  The text-in-box element
	will close any prior path.  The text may be in single quotes, in double
	quotes or may be bracketed by less-than and greater-than characters.
	The optional modifier (<m>) consists of three numbers for each of
	mirror-x, mirror-y, rotate-90ccw; non-zero is true.
  ]"

end -- class DRW_GROUP_GENERATOR
