indexing
	description:
		"A drawing line segment."
	author: "Barden E. Smith"
	date:	"March 18, 2005"

class
	DRW_LINE_SEGMENT

create
	make,
	make_by_coordinates

feature -- Status

	x1: DOUBLE
		-- First x value

	y1: DOUBLE
		-- First y value

	x2: DOUBLE
		-- Second x value

	y2: DOUBLE
		-- Second y value

feature {NONE} -- Creation

	make (a_x1, a_y1, a_x2, a_y2: DOUBLE) is
		do
			x1 := a_x1
			y1 := a_y1
			x2 := a_x2
			y2 := a_y2
		end

	make_by_coordinates (c1, c2: DRW_COORDINATE) is
			-- Create a line from two points (`c1', `c2')
		require
			c1_exists: c1 /= Void
			c2_exists: c2 /= Void
		do
			make (c1.x, c1.y, c2.x, c2.y)
		end

feature -- Properties

	p1: DRW_COORDINATE is
			-- Origin of segment
		do
			create Result.set (x1, y1)
		end

	p2: DRW_COORDINATE is
			-- Terminal of segment
		do
			create Result.set (x2, y2)
		end

	direction: DRW_COORDINATE is
			-- direction vector
		do
			create Result.set (x2 - x1, y2 - y1)
		end

	point_on_line (t: DOUBLE): DRW_COORDINATE is
			-- Return a point on the line for parameter `t'.
			-- When `t' = 0 return `p1' when `t' = 1 return `p2'
			-- `Result'  is on the segment for `t' in [0.0, 1.0]
		do
			create Result.set ((1.0 - t) * x1 + t * x2,
						(1.0 - t) * y1 + t * y2)
		end

	closest_point_on_line (p: VIRTUAL_POINT): DRW_COORDINATE is
		local
			pj, vsq, wx, wy: DOUBLE
			md: DRW_COORDINATE
		do
			wx := p.x - x1
			wy := p.y - y1
			md := direction
			vsq := md.dot (md)
			pj := (md.x * wx + md.y * wy) / vsq -- md.dot(w) / vsq
			create Result.set (x1 + pj*md.x, y1 + pj*md.y)
		end


end -- class DRW_LINE_SEGMENT
