indexing
	description:
		"Support for drawing MOSIS logos."
	author: "Barden E. Smith"
	date:	"February 11, 2005"

class
	DRW_MOSIS_LOGO

inherit
	ANY
	DRW_STANDARD_COLOR_ACCESS
		export
			{NONE} all
		end

feature -- Drawings

	simple: DRW_GROUP is
		local
			ggen: DRW_GROUP_GENERATOR
			pgen: DRW_PATH_GENERATOR
			p1: DRW_PATH
		do
			create pgen.make (face + wafer)
			pgen.scan_path
			p1 :=  pgen.last_drawing_path
			p1.set_stroke_width (5.0e-03)
			create ggen.make (grid + mosis)
			ggen.scan_path
			result := ggen.group
			result.extend (p1)
		end

	full_color: DRW_GROUP is
		local
			pgen: DRW_PATH_GENERATOR
			logo: DRW_GROUP
			p1: DRW_PATH
		do
			create logo.make (5)
			create pgen.make (left)
			pgen.scan_path
			p1 :=  pgen.last_drawing_path
			p1.disable_stroke
			p1.set_fill (mosis_blue)
			logo.extend (p1)
			pgen.set_path (right)
			pgen.scan_path
			p1 := pgen.last_drawing_path
			p1.disable_stroke
			p1.set_fill (mosis_red)
			logo.extend (p1)
			pgen.set_path (right_grid)
			pgen.scan_path
			p1 := pgen.last_drawing_path
			p1.set_stroke_width (0.1)
			p1.set_stroke (mosis_white)
			p1.set_stroke_line_cap_square
			logo.extend (p1)
			pgen.set_path (right_grid_accent)
			pgen.scan_path
			p1 := pgen.last_drawing_path
			p1.disable_stroke
			p1.set_fill (mosis_white)
			logo.extend (p1)
			pgen.set_path (l1 + l2 + l3 + l4 + l5)
			pgen.scan_path
			p1 := pgen.last_drawing_path
			p1.disable_stroke
			p1.set_fill (mosis_blue)
			logo.extend (p1)
			logo.set_transform (create {DRW_MATRIX}.make_arbitrary (0.02, 0, 0, -0.02, -0.5, 0.5))
			create Result.make (1)
			Result.extend (logo)
		end

feature {NONE} -- Implementation

	face: STRING is "%
		%M -0.529919264,-0.848048096 A 1,1 0 0,0 -0.207911691,0.978147601 L -0.11,0.68%
		%Q -0.07,0.58 -0.17,0.46%
		%Q -0.21,0.4 -0.17,0.34 L 0.03,0%
		%Q 0.07,-0.06 -0.03,-0.07 T -0.12,-0.18%
		%Q -0.08,-0.28 -0.12,-0.3 T -0.10,-0.35%
		%Q -0.12,-0.39 -0.11,-0.41%
		%C -0.06,-0.68 -0.17,-0.68 -0.32,-0.68 S -0.51,-0.68 -0.53,-0.84 z"

	wafer: STRING is "%
		%M 0.5,-0.8660254 A 1,1 0 0,1 -0.173648178,0.984807753 L -0.08,0.68%
		%Q -0.04,0.58 -0.14,0.46%
		%Q -0.18,0.4 -0.14,0.34 L 0.06,0%
		%Q 0.10,-0.08 0,-0.09 T -0.09,-0.20%
		%Q -0.05,-0.27 -0.08,-0.3 T -0.06,-0.34%
		%Q -0.09,-0.38 -0.08,-0.40%
		%C -0.04,-0.71 -0.15,-0.71 -0.3,-0.71 S -0.48,-0.71 -0.5,-0.86%
		%L -0.5,-0.8660254 z"

	mosis: STRING is "%
		%X 2.0,0.5 0.0,-1.375 %"MOSIS%""

	grid: STRING is "%
		%M-0.85 5 H1 M-0.55 4 H3 M-0.4 3 H4 M-0.85 2 H4 M-0.25 1 H5 M0.3 0 H5%
		%M-0.45-1 H5 M-0.4-2 H4 M-0.4-3 H4 M-2.5-4 H3%
		%M-2-4v0.25 M-1-4v0.275 M0-0.5V-4 M0,0.5V5 M1,5v-9 M2,4v-8 M3,4v-8 M4,3v-6 M5,1v-2%
		%|65.0e-03^0.19"

feature {NONE} -- Implementation imports

	mosis_blue: DRW_COLOR is
		once
			create Result.make_named_rgb ("mosis_blue", 0x0007 / 255.0, 0x0037 / 255.0, 0x0071 / 255.0)
		end

	mosis_red: DRW_COLOR is
		once
			create Result.make_named_rgb ("mosis_red", 0x00B2 / 255.0, 0x0018 / 255.0, 0x0011 / 255.0)
		end

	mosis_white: DRW_COLOR is
		once
			color_generator.colors.search ("white")
			if color_generator.colors.found then
				Result := color_generator.colors.found_item
			else
				io.put_string ("No color white in standard_colors")
			end
		end


	left: STRING is "M15.565,65.969c0.378-2.214,0.446-3.533,1.755-4.699c1.364-1.215,2.971-1.782,5.51-1.674%
		%s7.006,0.802,8.318-1.729c0.756-1.458,0.54-2.323,0.27-3.673c-0.27-1.351-0.397-3.38-0.216-4.267%
		%c0.27-1.323,1.38-1.423,0.27-2.269c-0.705-0.537-0.486-0.756,0-1.242c0.972-0.972,1.35-1.134,0.918-1.999%
		%c-0.486-0.972-1.728-4.159,1.783-5.293l1.729-0.648c0.81-0.378,0.918-0.864,0.378-1.891l-6.698-12.261%
		%c-0.81-1.512-1.188-2.917,0.054-4.969c1.242-2.052,2.322-2.269,1.404-6.536c-0.918-4.267-1.824-6.489-4.093-11.674%
		%C11.431,5.232,0,19.407,0,36.208C0,48.586,6.088,59.439,15.565,65.969z"
		-- MOSIS Blue

	right: STRING is "M17.551,67.127l37.431-0.024c10.461-6.348,17.433-17.765,17.433-30.896%
		%C72.415,16.211,56.204,0,36.208,0c-2.5,0-4.982,0.219-7.339,0.701c1.134,2.592,3.842,8.883,4.279,13.361%
		%c0.216,2.214,0,3.457-0.81,4.753c-0.811,1.296-2.106,2.809-1.026,4.591l6.158,11.505c1.62,2.403,1.647,4.294-0.027,4.915%
		%c-2.116,0.785-4.139,1.179-3.754,3.295c0.216,1.188,1.242,1.566,0.756,2.431c-0.605,1.076-1.583,1.312-1.134,1.782%
		%c0.567,0.594,0.945,0.648,0.297,1.594c-0.451,0.658-0.783,1.161-0.513,2.89c0.27,1.728,1.458,5.779-1.026,8.048%
		%c-2.233,2.039-4.321,1.836-6.104,1.836l-3.889-0.054C19.914,61.702,17.604,62.05,17.551,67.127z"
		-- MOSIS Red

	right_grid: STRING is "%
		%M17.105,63.491h39.123%
		%M28.509,2.456h14.163%
		%M22.327,63.478v-2.485%
		%M29.109,63.478v-2.701%
		%M35.89,63.488V39.664%
		%M35.89,32.75V2.449%
		%M42.672,63.478V2.449%
		%M49.454,63.478V9.248%
		%M56.236,63.478V9.243%
		%M63.017,56.702V16.01%
		%M69.799,43.146V29.591%
		%M31.425,9.238l24.811,0.005%
		%M32.073,16.02h30.943%
		%M33.91,29.583h35.889%
		%M37.583,36.364h32.158%
		%M33.154,43.146h36.645%
		%M32.397,49.927h30.616%
		%M32.722,56.709h30.292%
		%M30.021,22.8h32.992"
		-- White stroke_line_cap=square

	right_grid_accent: STRING is "%
		%M49.405,22.704v-6.698H42.6v6.698H49.405z%
		%M63.017,36.315V29.51h-6.806v6.806H63.017z%
		%M56.238,49.927v-6.806h-6.806v6.806H56.238z"
		-- White

	l1: STRING is "%
		%M18.685,93.111h2.313c0.257,0.02-0.054-0.143-0.139-0.285%
		%c-0.273-0.428-0.283-1.049-0.351-1.457l-1.458-14.017c-0.065-1.093-0.05-1.416,0.202-1.709c0.135-0.081,0.122-0.195-0.202-0.195%
		%h-2.255c-0.284,0-0.155,0.061-0.317,0.175l-5.912,14.381c-0.055,0.113-0.073,0.197-0.104,0.195c-0.013,0-0.052,0.002-0.09-0.127%
		%L4.256,75.705c-0.175-0.216-0.067-0.25-0.351-0.25l-1.789-0.006c-0.324,0-0.544,0.001-0.247,0.244%
		%c0.203,0.269,0.291,0.677,0.207,1.174L0.82,91.369c-0.068,0.411-0.021,1.112-0.274,1.539c-0.105,0.084-0.306,0.203-0.05,0.203%
		%H2.44c0.216,0,0.155-0.035-0.018-0.336c-0.122-0.266-0.104-0.555-0.074-0.96L3.21,78.851c0.081-0.972,0.101-0.709,0.284-0.243%
		%l6.319,14.219c0.243,0.365,0.324,0.729,0.486,0.729s0.243-0.364,0.405-0.729l6.239-14.705c0.243-0.608,0.365-0.689,0.365,0.243%
		%l1.377,13.45c0.058,0.403,0.095,0.766-0.005,1.028C18.629,92.944,18.29,93.121,18.685,93.111z"
		-- MOSIS Blue

	l2: STRING is "%
		%M22.634,84.268c0-4.958,3.949-8.977,8.819-8.977c4.871,0,8.82,4.019,8.82,8.977%
		%c0,4.958-3.949,8.976-8.82,8.976C26.583,93.244,22.634,89.225,22.634,84.268z%
		%M31.45,91.891c3.822,0,6.921-3.426,6.921-7.651c0-4.226-3.099-7.651-6.921-7.651%
		%c-3.823,0-6.921,3.425-6.921,7.651C24.529,88.465,27.627,91.891,31.45,91.891z"
		-- MOSIS Blue

	l3: STRING is "M50.978,77.177v-1.621c0-0.067-0.033-0.099-0.154-0.139%
		%c-5.573-1.115-8.63,0.465-8.94,3.758c-0.264,2.795,2.045,4.01,4.638,5.428c2.02,1.104,3.849,2.228,3.808,3.97%
		%c-0.055,2.349-1.256,3.484-4.335,3.524c-2.611,0.035-3.535-0.675-3.895-1.053c-0.271-0.392-0.264-0.216-0.237-0.081v1.729%
		%c0,0.081,0.033,0.099,0.127,0.125c5.377,1.289,10.169,0.189,10.527-3.839c0.324-3.646-2.714-4.659-4.821-5.955%
		%s-4.203-1.859-4.031-4.071c0.206-2.649,4.392-3.658,7.116-1.708C50.89,77.339,51.005,77.514,50.978,77.177z"
		-- MOSIS Blue

	l4: STRING is "M58.071,75.448h-2.289c-0.273,0-0.199,0.097-0.017,0.238%
		%c0.157,0.185,0.229,0.331,0.229,0.572v15.718c0,0.375-0.024,0.654-0.166,0.911c-0.061,0.091-0.392,0.224-0.108,0.224h2.37%
		%c0.274,0-0.059-0.114-0.119-0.246c-0.131-0.251-0.154-0.524-0.154-0.889V76.258c0-0.224,0.097-0.409,0.236-0.583%
		%C58.216,75.544,58.395,75.448,58.071,75.448z"
		-- MOSIS Blue

	l5: STRING is "%
		%M70.639,77.177v-1.621c0-0.067-0.033-0.099-0.154-0.139%
		%c-5.574-1.115-8.63,0.465-8.94,3.758c-0.263,2.795,2.046,4.01,4.639,5.428c2.02,1.104,3.848,2.228,3.808,3.97%
		%c-0.055,2.349-1.256,3.484-4.335,3.524c-2.612,0.035-3.535-0.675-3.896-1.053c-0.27-0.392-0.264-0.216-0.236-0.081v1.729%
		%c0,0.081,0.033,0.099,0.127,0.125c5.376,1.289,10.168,0.189,10.527-3.839c0.324-3.646-2.714-4.659-4.821-5.955%
		%s-4.203-1.859-4.031-4.071c0.206-2.649,4.392-3.658,7.116-1.708C70.551,77.339,70.666,77.514,70.639,77.177z"
		-- MOSIS Blue

end