indexing
	description:
		"A drawing page."
	author: "Barden E. Smith"
	date:	"August 13, 2004"

class
	DRW_PAGE

inherit
	DRW_TRANSFORMABLE_GROUP
		rename
			project as project_page
		export
			{ANY} add_figure, bounding_box, extend, is_empty, extendible
			{DRW_PAGE} all
		end

	DRW_VIEW_BOX
		export
			{ANY} has_view_box, set_view_box
			{DRW_DOCUMENT, DRW_PROJECTOR} view_box
			{NONE} all
		undefine
			copy, default_create, is_equal
		end

create
	make,
	make_in_document

create {DRW_PAGE}
	make_filled


feature -- Attributes

	page_label: STRING

feature -- Modification

	set_page_label (l: STRING) is
		do
			page_label := l
		ensure
			page_label_set: page_label = l
		end

feature {DRW_DOCUMENT, DRW_PROJECTOR} -- plotting routine

	project (plotter: DRW_FIGURE_DRAWING_ROUTINES) is
		local
			m: DRW_MATRIX
			b: DRW_BASIC_RECTANGLE
		do
			if has_view_box then
				m := transform
				b := untransformed_bounding_box
				if b /= Void then
					transform := matrix_map (b)
				end
				create b.make (view_box.width, view_box.height,
							view_box.x, view_box.y)
			else
				b := bounding_box
			end
			plotter.start_page (b, page_label)
			if parent_document /= Void and then parent_document.common_page /= Void then
				parent_document.common_page.project_page (plotter)
			end
			project_page (plotter)
			plotter.finish_page
			if has_view_box then
				transform := m
			end
		end

feature {DRW_PAGE, DRW_DOCUMENT} -- document projection

	parent_document: DRW_DOCUMENT

	set_parent_document (d: DRW_DOCUMENT) is
		do
			parent_document := d
		end

feature {NONE} -- Creation

	make_in_document (d: DRW_DOCUMENT s: INTEGER) is
		require
			valid_number_of_items: s >= 0
			have_doc: d /= Void
		do
			make_array (s)
			d.add_page (Current)
		end


	make (s: INTEGER) is
		require
			valid_number_of_items: s >= 0
		do
			make_array (s)
		end


end -- class DRW_PAGE
