indexing
	description:
		"A drawing path scanner."
	author: "Barden E. Smith"
	date:	"October 19, 2004"
	instructions: "[
	]"

deferred class
	DRW_PATH_SCANNER

feature -- Function

	set_path (path: STRING) is
		require
			path_not_void: path /= Void
		do
			path_value := path
		end

feature -- Status report

	has_path: BOOLEAN is
		do
			Result := path_value /= Void and then not path_value.is_empty
		end

	error_count: INTEGER

	scan_path is
			-- scan the path
		require
			has_path: has_path
		local
			c: CHARACTER
		do
			start_scan
			from
				position := 1
			until
				position > path_value.count
			loop
				c := path_value.item (position)
				position := position + 1
				dispatch (c)
			end
		end

feature {NONE} -- Implementation

	path_value: STRING

	position: INTEGER

	commands: ARRAYED_LIST [DRW_COMMAND]

	next_number: DOUBLE

	found_next_number: BOOLEAN

	start_scan is
		require
			has_path: has_path
		do
			create commands.make (0) 
			error_count := 0
		ensure
			no_errors: error_count = 0
			commands_available: commands /= Void
		end

	dispatch (c: CHARACTER) is
		do
			inspect c
			when ' ','%T','%N','%R' then
			when 'A','a' then
				add_arc_commands (c)
			when 'C','c' then
				add_curve3_commands (c)
			when 'H','L','M','V','h','l','m','v' then
				add_draw_commands (c)
			when 'Q','q' then
				add_curve2_commands (c)
			when 'S','s' then
				add_smooth_curve3_commands (c)
			when 'T','t' then
				add_smooth_curve2_commands (c)
			when 'Z','z' then
				add_close_command
			else
				error_count := error_count + 1
			end
		end

	add_arc_commands (cmd: CHARACTER) is
		local
			p: ARRAY [DOUBLE]
		do
			p := scan_numbers (7)
			if p.count = 7 then
				if cmd = 'A' then
					commands.extend (create {DRW_ARC}.make_abs (
						p.item (1), p.item (2), p.item (3),
						p.item (4) /= 0,  p.item (5) /= 0,
						p.item (6), p.item (7) ))
				else
					commands.extend (create {DRW_ARC}.make_rel (
						p.item (1), p.item (2), p.item (3),
						p.item (4) /= 0,  p.item (5) /= 0,
						p.item (6), p.item (7) ))
				end
				add_arc_commands (cmd)
			end
		end

	add_curve3_commands (cmd: CHARACTER) is
		local
			p: ARRAY [DOUBLE]
		do
			p := scan_numbers (6)
			if p.count = 6 then
				if cmd = 'C' then
					commands.extend (create {DRW_CURVE3}.make_abs (
						p.item (1), p.item (2), p.item (3),
						p.item (4),  p.item (5), p.item (6) ))
				else
					commands.extend (create {DRW_CURVE3}.make_rel (
						p.item (1), p.item (2), p.item (3),
						p.item (4),  p.item (5), p.item (6) ))
				end
				add_curve3_commands (cmd)
			end
		end

	add_smooth_curve3_commands (cmd: CHARACTER) is
		local
			p: ARRAY [DOUBLE]
		do
			p := scan_numbers (4)
			if p.count = 4 then
				if cmd = 'S' then
					commands.extend (create {DRW_CURVE3}.make_smooth_abs (
						p.item (1), p.item (2), p.item (3), p.item (4) ))
				else
					commands.extend (create {DRW_CURVE3}.make_smooth_rel (
						p.item (1), p.item (2), p.item (3), p.item (4) ))
				end
				add_smooth_curve3_commands (cmd)
			end
	end

	add_draw_commands (cmd: CHARACTER) is
		local
			x, y: DOUBLE
			p: ARRAY [DOUBLE]
		do
			inspect cmd
			when 'M','m' then
				p := scan_numbers (2)
				if p.count = 2 then
					x := p.item (1)
					y := p.item (2)
					if cmd = 'M' then
						commands.extend (create {DRW_DRAWTO}.make_moveto_abs (x, y))
						add_draw_commands ('L')
					else
						commands.extend (create {DRW_DRAWTO}.make_moveto_rel (x, y))
						add_draw_commands ('l')
					end
				end
			when 'L','l' then
				p := scan_numbers (2)
				if p.count = 2 then
					x := p.item (1)
					y := p.item (2)
					if cmd = 'L' then
						commands.extend (create {DRW_DRAWTO}.make_lineto_abs (x, y))
						add_draw_commands ('L')
					else
						commands.extend (create {DRW_DRAWTO}.make_lineto_rel (x, y))
						add_draw_commands ('l')
					end
				end
			when 'H','h' then
				scan_next_number
				if found_next_number then
					x := next_number
					if cmd = 'H' then
						commands.extend (create {DRW_DRAWTO}.make_lineto_h_abs (x))
					else
						commands.extend (create {DRW_DRAWTO}.make_lineto_h_rel (x))
					end
				end
			when 'V','v' then
				scan_next_number
				if found_next_number then
					y := next_number
					if cmd = 'V' then
						commands.extend (create {DRW_DRAWTO}.make_lineto_v_abs (y))
					else
						commands.extend (create {DRW_DRAWTO}.make_lineto_v_rel (y))
					end
				end
			end
		end

	add_curve2_commands (cmd: CHARACTER) is
		local
			p: ARRAY [DOUBLE]
		do
			p := scan_numbers (4)
			if p.count = 4 then
				if cmd = 'Q' then
					commands.extend (create {DRW_CURVE2}.make_abs (
						p.item (1), p.item (2), p.item (3), p.item (4) ))
				else
					commands.extend (create {DRW_CURVE2}.make_rel (
						p.item (1), p.item (2), p.item (3), p.item (4) ))
				end
				add_curve2_commands (cmd)
			end
		end

	add_smooth_curve2_commands (cmd: CHARACTER) is
		local
			p: ARRAY [DOUBLE]
		do
			p := scan_numbers (2)
			if p.count = 2 then
				if cmd = 'T' then
					commands.extend (create {DRW_CURVE2}.make_smooth_abs (
						p.item (1), p.item (2) ))
				else
					commands.extend (create {DRW_CURVE2}.make_smooth_rel (
						p.item (1), p.item (2) ))
				end
				add_smooth_curve2_commands (cmd)
			end
		end

	add_close_command is
		do
			commands.extend (create {DRW_CLOSE})
		end

	skip_space is
			-- skip spaces and commas
		local
			c: CHARACTER
			len: INTEGER
			done: BOOLEAN
		do
			from
				len := path_value.count
			until
				position > len or done
			loop
				c := path_value.item (position)
				if (c = '%T' or c = '%N' or c = ' ' or c = ',') then
					position := position + 1
				else
					done := True
				end
			end
		end

	scan_numbers (len: INTEGER): ARRAYED_LIST [DOUBLE] is
		require
			positive_len: len > 0
		do
			from
				create Result.make (len)
				scan_next_number
			until
				not found_next_number or
				Result.count >= len
			loop
				Result.extend (next_number)
				if Result.count < len then
					scan_next_number
				end
			end
		end
		
	scan_next_number is
		local
			c: CHARACTER
			p: INTEGER
			done, seen_dot, seen_e: boolean
			token: STRING
		do
			found_next_number := False
			skip_space
			if position <= path_value.count then
				c := path_value.item (position)
				if c = '-' or c.is_digit then
					from
						p := position + 1
						done := p > path_value.count
					until
						done or p > path_value.count
					loop
						c := path_value.item (p)
						if c.is_digit then
						elseif c = '.' and not seen_dot then
							seen_dot := True
						elseif not seen_e and (c = 'E' or c = 'e') then
							seen_e := True
							seen_dot := True
							if p < path_value.count then
								p := p + 1
								if path_value.item (p) /= '-' then
									p := p - 1
								end
							end
						else
							done := True
							p := p - 1
						end
						p := p + 1
					end
					token := path_value.substring (position, p - 1)
					found_next_number := token.is_double
					if found_next_number then
						next_number := token.to_double
						position := p
					end
				end
			end
		end

end -- class DRW_PATH_SCANNER
