indexing
	description:
		"A drawing polyline (open polygon)."
	author: "Barden E. Smith"
	date:	"August 13, 2004"

class
	DRW_POLYLINE

inherit
	SIMPLE_LIST_OF_POINTS

	DRW_SHAPE
		undefine
			copy,
			is_equal
		end

create
	make,
	make_copy

feature -- Shape attributes

	is_drawable: BOOLEAN is
			-- true if stroked
		do
			Result := is_stroked
		end

feature -- Transformed figure

	as_transformed_points (xform: DRW_MATRIX): SIMPLE_LIST_OF_POINTS is
			-- return points as transformed by local transform and
			-- then by `xform'
		require
			xform_not_void: xform /= Void
		local
			i: INTEGER
			m: DRW_MATRIX
			x, y: DOUBLE
		do
			if is_transformed then
				m := transform.and_then (xform)
			else
				m := xform
			end
			create Result.make (count)
			from
				i := 1
			until
				i > count
			loop	x := i_th_x (i); 
				y := i_th_y (i)
				Result.extend(m.transform_xy(x,y))
				i := i + 1
			end
		end
	
feature -- Size estimating

	bounding_box: DRW_BASIC_RECTANGLE is
			-- Void if the list is empty
		local
			i: INTEGER
			c: DRW_COORDINATE
		do
			if not is_empty then
				create c.set (first_x, first_y)
				c := coordinate_transformed (c)
				create result.make (0, 0, c.x, c.y)
			 	from
			  		i := 2
			  	until
			  		i > count
			  	loop
			  		c.set (i_th_x (i), i_th_y (i))
			  		Result.include_coordinate (coordinate_transformed (c))
					i := i + 1
			  	end
			end
		end


feature {DRW_PROJECTOR} -- plotting routine

	project (plotter: DRW_FIGURE_DRAWING_ROUTINES) is
		do
			if is_drawable then
				plotter.draw_figure_polyline (current)
			end
		end

end -- class DRW_POLYLINE
