indexing
	description:
		"A Postscript drawing command processor."
	author: "Barden E. Smith"
	date:	"October 28, 2004"


class
	DRW_POSTSCRIPT_COMMAND_PROCESSOR

inherit
	DRW_COMMAND_PROCESSOR

	MATH_CONST
		export
			{NONE} all
		end

feature -- Inspection

	postscript_result: STRING

feature {NONE} -- Implementation

	add_ps (a_code: STRING) is
			-- Add `a_code' to postscript data.
		do
			postscript_result.append (a_code + "%N")
		end

	add_ps_no_comments (a_code: STRING) is
			-- Add `a_code', to postscript data, line by line, removing local comments.
			-- Local comments start with percent followed by a space
			-- character (optionally preceeded by white space)
		local
			i, j: INTEGER
		do
			if preserve_comments then
				postscript_result.append (a_code)
				postscript_result.extend ('%N')
			else
				from
					i := 1
				until
					i > a_code.count
				loop
					j := a_code.index_of ('%N', i)
					if j >= i then
						add_ps_line_no_comments (a_code.substring (i,j))
						i := j
					else
						add_ps_line_no_comments
							(a_code.substring (i,a_code.count) + "%N")
					end
					i := i + 1
				end
			end
		end

	add_ps_line_no_comments (a_line: STRING) is
			-- Scan and remove comments from `a_line'
		do
			postscript_result.append (a_line)
		end

	translate_to (a_x, a_y: DOUBLE) is
		do
			add_ps (a_x.out + " " + a_y.out + " " + "translate")
		end

	arcto (code: INTEGER x, y, rrx, rry, rot: DOUBLE fa, fs: BOOLEAN) is
		local
			fa_s, fs_s, point_s, t, f: STRING
		do
			if rrx = 0 or rry = 0 then
				drawto (code - 6, x, y)
			else
				point_s := x.out + " " + y.out
				t := " true"
				f := " false"
				if fa then
					fa_s := t
				else
					fa_s := f
				end
				if fs then
					fs_s := t
				else
					fs_s := f
				end
				add_ps (rrx.out + " " +
					rry.out + " " +
					rot.out + fa_s + fs_s)
				if code = 10 then
					add_ps (point_s + " drwarcto")
				else
					add_ps (point_s + " rpoint drwarcto")
				end
			end
		end

	close (code: INTEGER) is
		do
			add_ps ("closepath")
		end

	curveto2 (code: INTEGER x, y, x1, y1: DOUBLE) is
		do
			inspect code
			when 8 then
				add_ps (x1.out + " " +
					y1.out + " " +
					x.out + " " +
					y.out + " drwcurve2")
			when 9 then
				add_ps (x1.out + " " +
					y1.out + " " +
					x.out + " " +
					y.out + " drwrcurve2")
			when 18 then
				add_ps ("getnextctrl rpoint " +
					x.out + " " +
					y.out + " drwcurve2")
			when 19 then
				add_ps ("getnextctrl " +
					x.out + " " +
					y.out + " drwrcurve2")
			else
				add_ps ("-- ERROR: bad curve2")
			end
		end

	curveto3 (code: INTEGER x, y, x1, y1,x2 ,y2: DOUBLE) is
		do
			inspect code
			when 6 then
				add_ps (x1.out + " " +
					y1.out + " " +
					x2.out + " " +
					y2.out + " " +
					x.out + " " +
					y.out + " drwcurve3")
			when 7 then
				add_ps (x1.out + " " +
					y1.out + " " +
					x2.out + " " +
					y2.out + " " +
					x.out + " " +
					y.out + " drwrcurve3")
			when 16 then
				add_ps ("getnextctrl rpoint " +
					x2.out + " " +
					y2.out + " " +
					x.out + " " +
					y.out + " drwcurve3")
			when 17 then
				add_ps ("getnextctrl " +
					x2.out + " " +
					y2.out + " " +
					x.out + " " +
					y.out + " drwrcurve3")
			else
				add_ps ("-- ERROR: bad curve3")
			end
		end

	drawto (code: INTEGER x, y: DOUBLE) is
		do
			inspect code
				when 2 then
					-- moveto abs
					add_ps (x.out + " " + y.out + " moveto")
				when 3 then
					-- moveto rel
					add_ps (x.out + " " + y.out + " rmoveto")
				when 4 then
					-- lineto abs
					add_ps (x.out + " " + y.out + " lineto")
				when 5 then
					-- lineto rel
					add_ps (x.out + " " + y.out + " rlineto")
				when 12 then
					-- horizontal lineto abs
					add_ps (x.out + " currentpoint exch pop lineto")
				when 13 then
					-- horizontal lineto rel
					add_ps ("currentpoint exch " + x.out + " add exch lineto")
				when 14 then
					-- vertical lineto abs
					add_ps ("currentpoint pop " + y.out + " lineto")
				when 15 then
					-- vertical lineto rel
					add_ps ("currentpoint " + y.out + " add lineto")
			end		
		end

	preserve_comments: BOOLEAN is True

end -- class DRW_POSTSCRIPT_COMMAND_PROCESSOR
