indexing
	description:
		"Routines to output a drawing as Postscript."
	author: "Barden E. Smith"
	date:	"September 30, 2004"


deferred class
	DRW_POSTSCRIPT_FIGURE_DRAWER

inherit
	DRW_FIGURE_DRAWING_ROUTINES
		rename
			restore_previous_transform as restore_previous_gstate
		end

	DRW_POSTSCRIPT_COMMAND_PROCESSOR

feature -- Figure drawing

	start_page (v: DRW_BASIC_RECTANGLE page_label:STRING) is
		local
			m: DRW_MATRIX
			l: STRING
		do
			current_page := current_page + 1
			if page_label /= Void and then not page_label.is_empty then
				l := postscript_text (page_label)
			else
				l := current_page.out
			end
			add_ps ("%%%%Page: " + l + " " + current_page.out)
			-- map the document space to coordinate space
			m := device_view_box.matrix_map (v)
			alter_current_transform (m)
			add_ps ("0 setlinewidth")
		end

	finish_page is
		do
			add_ps ("grestore showpage%N%%%%PageTrailer")
		end

	alter_current_transform (m: DRW_MATRIX) is
		do
			save_gstate
			alter_ctm_unsaved (m)
		end

	alter_ctm_unsaved (m: DRW_MATRIX) is
		do
			add_ps (m.a.out + " " +
				m.c.out + " " +
				m.b.out + " " +
				m.d.out + " " +
				m.tx.out + " " +
				m.ty.out + " 6 array astore concat"
				)
		end

	save_gstate is
		do
			add_ps ("gsave")
		end

	restore_previous_gstate is
		do
			add_ps ("grestore")
		end

	push_matrix (m: DRW_MATRIX) is
		do
			add_ps (m.a.out + " " +
				m.c.out + " " +
				m.b.out + " " +
				m.d.out + " " +
				m.tx.out + " " +
				m.ty.out + " 6 array astore"
				)
		end

	draw_figure_circle (circle: DRW_CIRCLE) is
			-- Draw standard representation of `circle'.
		local
			sw: DOUBLE
		do
			sw := circle.stroke_width
			save_gstate
			apply_fill_values (circle)
			if circle.is_transformed then
				if sw > 0 then
					alter_ctm_unsaved (circle.transform)
					add_ps (sw.out + " setlinewidth")
				else
					add_ps ("currentlinewidth dup dtransform")
					alter_ctm_unsaved (circle.transform)
					add_ps ("idtransform drwabs_v setlinewidth")
				end
				draw_ellipse (circle.x, circle.y, circle.r, circle.r)
			elseif sw > 0 then
				add_ps (sw.out + " setlinewidth")
				draw_ellipse (circle.x, circle.y, circle.r, circle.r)
			else
				draw_ellipse (circle.x, circle.y, circle.r, circle.r)
			end
			reset_fill_values
			restore_previous_gstate
		end

	draw_figure_ellipse (ellipse: DRW_ELLIPSE) is
			-- Draw standard representation of `ellipse' to canvas.
		local
			sw: DOUBLE
		do
			sw := ellipse.stroke_width
			save_gstate
			apply_fill_values (ellipse)
			if ellipse.is_transformed then
				if sw > 0 then
					alter_ctm_unsaved (ellipse.transform)
					add_ps (sw.out + " setlinewidth")
				else
					add_ps ("currentlinewidth dup dtransform")
					alter_ctm_unsaved (ellipse.transform)
					add_ps ("idtransform drwabs_v setlinewidth")
				end
				draw_ellipse (ellipse.x, ellipse.y, ellipse.rx, ellipse.ry)
			elseif sw > 0 then
				add_ps (sw.out + " setlinewidth")
				draw_ellipse (ellipse.x, ellipse.y, ellipse.rx, ellipse.ry)
			else
				draw_ellipse (ellipse.x, ellipse.y, ellipse.rx, ellipse.ry)
			end
			reset_fill_values
			restore_previous_gstate
		end

	draw_figure_line (line: DRW_LINE) is
			-- Draw a standard representation of `line' to the canvas.
		local
			sw: DOUBLE
		do
			sw := line.stroke_width
			save_gstate
			apply_stroke_values (line)
			if line.is_transformed then
				if sw > 0 then
					alter_ctm_unsaved (line.transform)
					add_ps (sw.out + " setlinewidth")
				else
					add_ps ("currentlinewidth dup dtransform")
					alter_ctm_unsaved (line.transform)
					add_ps ("idtransform drwabs_v setlinewidth")
				end
				draw_line (line.x1, line.y1, line.x2, line.y2)
			elseif sw > 0 then
				add_ps (sw.out + " setlinewidth")
				draw_line (line.x1, line.y1, line.x2, line.y2)
			else
				draw_line (line.x1, line.y1, line.x2, line.y2)
			end
			restore_previous_gstate
		end

--	draw_figure_image (image: DRW_IMAGE) is
			-- Draw standard representation of `image' to canvas.
--		do
--		end

	draw_figure_polygon (polygon: DRW_POLYGON) is
			-- Draw standard representation of `polygon' to canvas.
		do
			apply_fill_values (polygon)
			draw_polyfigure (polygon, True)
			reset_fill_values
		end

	draw_figure_polyline (lines: DRW_POLYLINE) is
		do
			add_ps ("drw_set_paintstroke")
			draw_polyfigure (lines, False)
			reset_fill_values
		end

	draw_figure_rectangle (rectangle: DRW_RECTANGLE) is
			-- Draw standard representation of `rectangle' to canvas.
		local
			sw: DOUBLE
		do
			sw := rectangle.stroke_width
			save_gstate
			if rectangle.is_transformed then
				if sw > 0 then
					alter_ctm_unsaved (rectangle.transform)
					add_ps (sw.out + " setlinewidth")
				else
					add_ps ("currentlinewidth dup dtransform")
					alter_ctm_unsaved (rectangle.transform)
					add_ps ("idtransform drwabs_v setlinewidth")
				end
				draw_rectangle (rectangle)
			elseif sw > 0 then
				add_ps (sw.out + " setlinewidth")
				draw_rectangle (rectangle)
			else
				draw_rectangle (rectangle)
			end
			restore_previous_gstate
		end

	draw_figure_soft_rectangle (rectangle: DRW_SOFT_RECTANGLE) is
			-- Draw standard representation of `rectangle' to canvas.
		local
			sw: DOUBLE
		do
			sw := rectangle.stroke_width
			save_gstate
			if rectangle.is_transformed then
				if sw > 0 then
					alter_ctm_unsaved (rectangle.transform)
					add_ps (sw.out + " setlinewidth")
				else
					add_ps ("currentlinewidth dup dtransform")
					alter_ctm_unsaved (rectangle.transform)
					add_ps ("idtransform drwabs_v setlinewidth")
				end
				draw_soft_rectangle (rectangle, rectangle.rx, rectangle.ry)
			elseif sw > 0 then
				add_ps (sw.out + " setlinewidth")
				draw_soft_rectangle (rectangle, rectangle.rx, rectangle.ry)
			else
				draw_soft_rectangle (rectangle, rectangle.rx, rectangle.ry)
			end
			restore_previous_gstate
		end


	draw_figure_path (path: DRW_PATH) is
			-- Draw standard representation of `path' to canvas.
		local
			sw: DOUBLE
		do
			sw := path.stroke_width
			save_gstate
			if path.is_transformed then
				if sw > 0 then
					alter_ctm_unsaved (path.transform)
					add_ps (sw.out + " setlinewidth")
				else
					add_ps ("currentlinewidth dup dtransform")
					alter_ctm_unsaved (path.transform)
					add_ps ("idtransform drwabs_v setlinewidth")
				end
				draw_path (path)
			elseif sw > 0 then
				add_ps (sw.out + " setlinewidth")
				draw_path (path)
			else
				draw_path (path)
			end
			restore_previous_gstate
		end

	draw_figure_text (text_figure: DRW_TEXT) is
			-- Draw standard representation of `text_figure' to canvas.
		local
			text: STRING
			b: DRW_BASIC_RECTANGLE
		do
			text := postscript_text(text_figure.full_text)
			save_gstate
			if text_figure.has_font then
				apply_font (text_figure.font)
			end
			if text_figure.has_stroke then
				set_stroke_color (text_figure.stroke)
			end
			if text_figure.has_view_box then
				b := text_figure.view_box
				if text_figure.mid_x /= 0 or text_figure.mid_y /= 0 then
					add_ps (text_figure.mid_x.out + " drw_set_mid_x " +
						text_figure.mid_y.out + " drw_set_mid_y")
				end
				add_ps (text)
				if text_figure.is_transformed then
					alter_ctm_unsaved (text_figure.transform)
					push_matrix (text_figure.transform)
					add_ps (b.left.out + " " +
						b.top.out + " " +
						b.right.out + " " +
						b.bottom.out + " drw_last_box text_in_box")
				else
					add_ps (b.width.out + " " +
						b.height.out + " " +
						b.x.out + " " +
						b.y.out + " text_in_box")
				end
				if text_figure.mid_x /= 0 or text_figure.mid_y /= 0 then
					add_ps ("0 drw_set_mid_x 0 drw_set_mid_y")
				end
			elseif text_figure.is_transformed then
				alter_ctm_unsaved (text_figure.transform)
				add_ps ("currentfont " + text_figure.size.out + " scalefont setfont")
				add_ps (text)
				add_ps (text_figure.x.out + " " + text_figure.y.out + " moveto show")
			else
				add_ps ("currentfont " + text_figure.size.out + " scalefont setfont")
				add_ps (text)
				add_ps (text_figure.x.out + " " + text_figure.y.out + " moveto show")
			end
			restore_previous_gstate
		end

feature {NONE} -- Implementation


	current_page: INTEGER

	device_view_box: DRW_VIEW_BOX

	Identity: DRW_MATRIX is
		once
			create Result.make_identity
		end


	apply_font (f: STRING) is
		require
			f_not_void: f /= Void
		do
			if not f.is_empty then
				add_ps ("/" + f + " findfont setfont")
			end
		end


	octal_out (c: CHARACTER): STRING is
			-- put out the three character string for `c'
		local
			i,z: INTEGER
		do
			create Result.make(3)
			i := c.code
			z := ('0').code
			if i > 255 then
				-- return empty string
			else
				Result.append_character ((z + (i // 64)).to_character_8)
				i := i - 64 * (i // 64)
				Result.append_character ((z + (i // 8)).to_character_8)
				i := i - 8 * (i // 8)
				Result.append_character ((z + i).to_character_8)
			end
		end
	postscript_text (s: STRING): STRING is
		require
			have_s: s /= Void
		local
			c: CHARACTER
			i: INTEGER
		do
			create Result.make (s.count)
			Result.append_character ('(')
			from
				i := 1
			until
				i > s.count
			loop
				c := s.item (i)
				inspect c
				when '(', ')', '\' then
					Result.append_character ('\')
					Result.append_character (c)
				when '%R' then
					if i + 1 <= s.count and s.item (i + 1) = '%N' then
						Result.append_character ('\')
						Result.append_character ('n')
						i := i + 1
					else
						Result.append_character ('\')
						Result.append_character ('r')
					end
				when '%N' then
					Result.append_character ('\')
					Result.append_character ('n')
				when '%B' then
					Result.append_character ('\')
					Result.append_character ('b')
				when '%F' then
					Result.append_character ('\')
					Result.append_character ('f')
				when '%T' then
					Result.append_character ('\')
					Result.append_character ('t')
				else
					if c.code >= 127 or c.code < 32 then
						Result.append_character ('\')
						Result.append (octal_out(c))
					else
						Result.append_character (c)
					end
				end
				i := i + 1
			end
			Result.append_character (')')
		end

	preamble: STRING is "%
	%%%%%BeginProlog%N%
	%/myhandler {%N%
	%	initgraphics%N%
	%	/Helvetica findfont 25 scalefont setfont%N%
	%	newpath%N%
	%	100 360 moveto%N%
	%	/str 100 string def%N%
	%	(Error: ) show%N%
	%	$error /errorname get str cvs show%N%
	%	showpage%N%
	%  	} def%N%
	%%N%
	%/rpoint { %% dx dy = > x y%N%
	%	%% Add (dx, dy) to currentpoint%N%
	%	exch%N%
	%	currentpoint%N%
	%	3 1 roll add%N%
	%	3 1 roll add%N%
	%	} def%N%
	%%N%
	%/setnextctrl { %% x0 y0 x1 y1 = > x0 y0 x1 y1%N%
	%	%% compute next smooth control point%N%
	%	4 copy exch 3 1 roll%N%
	%	sub neg 3 1 roll%N%
	%	sub neg exch%N%
	%	2 array astore%N%
	%	/nextctrl exch def%N%
	%	} def%N%
	%%N%
	%/clearnextctrl {%N%
	%	/nextctrl 0.0 0.0 2 array astore def%N%
	%	} def%N%
	%%N%
	%/getnextctrl {%N%
	%	nextctrl aload pop%N%
	%	} def %N%
	%%N%
	%/drw_paintpath {%N%
	%	stroke%N%
	%	} def%N%
	%%N%
	%/drw_fill_color {%N%
	%	0 0 0 setrgbcolor%N%
	%	} def%N%
	%%N%
	%/drw_set_paintstroke {%N%
	%	/drw_paintpath { stroke } def%N%
	%	} def%N%
	%%N%
	%/drw_set_paintfill {%N%
	%	/drw_paintpath { gsave drw_fill_color fill grestore newpath} def%N%
	%	} def%N%
	%%N%
	%/drw_set_paintfillstroke {%N%
	%	/drw_paintpath { gsave drw_fill_color fill grestore stroke } def%N%
	%	} def%N%
	%%N%
	%/drw_restore_paintpath {%N%
	%	/drw_paintpath { stroke } def%N%
	%	} def%N%
	%%N%
	%/drwabs_v { %% dx dy => abs_d%N%
	%	%% abs_d is sqrt( ( dx^2 + dy^2 ) / 2 )%N%
	%	2 copy eq %N%
	%	{ pop } { dup mul exch dup mul add 2 div sqrt } ifelse%N%
	%	} def%N%
	%/drwfixrxry { %% x' y' rx ry = > rx' ry'%N%
	%	2 copy 6 2 roll%N%
	%	dup mul 4 1 roll%N%
	%	dup mul 4 1 roll%N%
	%	dup mul 4 1 roll%N%
	%	dup mul 4 1 roll%N%
	%	%% rx  ry  x'^2  y'^2  rx^2  ry^2%N%
	%	exch 3 1 roll%N%
	%	%% rx  ry  x'^2  rx^2  y'^2  ry^2%N%
	%	div 3 1 roll%N%
	%	div add%N%
	%	dup 1.0 lt%N%
	%	{ pop } { sqrt dup 4 -1 roll mul 3 1 roll mul } ifelse%N%
	%	} def%N%
	%%N%
	%/drwss {%% fs.xor.fa x' y' rx' ry' = > ss%N%
	%	dup mul 4 1 roll%N%
	%	dup mul 4 1 roll%N%
	%	dup mul 4 1 roll%N%
	%	dup mul 4 1 roll%N%
	%	2 copy mul%N%
	%	%% fx.xor.fa x'^2  y'^2 rx'^2 ry'^2  rx'^2*ry'^2%N%
	%	5 1 roll 4 1 roll mul 3 1 roll mul%N%
	%	add dup 3 1 roll sub exch div%N%
	%	dup 0 le { pop 0 } { sqrt } ifelse%N%
	%	exch { } { neg } ifelse%N%
	%	} def%N%
	%%N%
	%/drwarc { %% ang1 ang2 fs rx ry rot x y%N%
	%	matrix currentmatrix 0 0 1 12 4 roll%N%
	%	translate%N%
	%	rotate%N%
	%	scale%N%
	%	{ arc } { arcn } ifelse%N%
	%	setmatrix%N%
	%	} def%N%
	%%N%
	%/drwarcto { %% rx ry rot fa fs x y%N%
	%	currentpoint exch%N%
	%	3 index sub 2 div exch%N%
	%	2 index sub 2 div%N%
	%	6 index neg matrix rotate transform%N%
	%	%% rx ry rot fa fs x y x' y'%N%
	%	2 copy 11 -2 roll drwfixrxry%N%
	%	%% rot fa fs x y x' y' rx' ry'%N%
	%	8 -2 roll dup 10 1 roll xor%N%
	%	%% fs rot x y x' y' rx' ry' fa.xor.fs%N%
	%	5 copy pop drwss%N%
	%	%% fs rot x y x' y' rx' ry' ss%N%
	%	dup neg 5 index mul 2 index mul 3 index div exch%N%
	%	4 index mul 3 index mul 2 index div exch%N%
	%	%% fs rot x y x' y' rx' ry' cx' cy'%N%
	%	2 copy 10 2 roll 8 -2 roll 4 2 roll%N%
	%	%% fs rot cx' cy' x' y' rx' ry' x y cx' cy'%N%
	%	10 index matrix rotate transform%N%
	%	%% fs rot cx' cy' x' y' rx' ry' x y cx'' cy''%N%
	%	4 2 roll currentpoint%N%
	%	%% fs rot cx' cy' x' y' rx' ry' cx'' cy'' x y x0 y0%N%
	%	3 -1 roll add 2 div 4 -1 roll add%N%
	%	%% fs rot cx' cy' x' y' rx' ry' cx'' x x0 cy%N%
	%	4 1 roll add 2 div add exch%N%
	%	%% fs rot cx' cy' x' y' rx' ry' cx cy%N%
	%	9 -1 roll 3 1 roll%N%
	%	%% (fs cx' cy' x' y' rx' ry' rot cx cy\n) print stack%N%
	%	5 index 8 index sub 4 index div%N%
	%	7 index 10 index sub 6 index div atan 11 1 roll%N%
	%	%% (a1 fs cx' cy' x' y' rx' ry' rot cx cy\n) print stack%N%
	%	6 -1 roll neg 8 -1 roll sub 4 index div%N%
	%	7 -1 roll neg 8 -1 roll sub 6 index div atan%N%
	%	%% a1 fs rx' ry' rot cx cy a2%N%
	%	7 1 roll%N%
	%	%% (a1 a2 fs rot rx' ry' cx cy\n) print stack%N%
	%	drwarc%N%
	%	} def%N%
	%%N%
	%/drwrcurve3 { %% x1 y1 x2 y2 x3 y3%N%
	%	setnextctrl%N%
	%	rcurveto%N%
	%	} def%N%
	%/drwcurve3 { %% x1 y1 x2 y2 x3 y3%N%
	%	setnextctrl%N%
	%	curveto%N%
	%	} def%N%
	%/drwrcurve2 { %% dx1 dy1 dx2 dy2%N%
	%	setnextctrl%N%
	%	drwcurve2x%N%
	%	rcurveto%N%
	%	} def%N%
	%%N%
	%/drwcurve2 { %% x1 y1 x2 y2%N%
	%	setnextctrl%N%
	%	drwcurve2x%N%
	%	curveto%N%
	%	} def%N%
	%%N%
	%/drwcurve2x { %% x1 y1 x2 y2%N%
	%	%% Convert quadratic curve to cubic curve%N%
	%	4 copy 4 2 roll%N%
	%	3 -1 roll exch%N%
	%	2 mul add 3 div 3 1 roll%N%
	%	2 mul add 3 div exch%N%
	%	4 2 roll 6 -2 roll%N%
	%	currentpoint%N%
	%	exch 4 1 roll exch%N%
	%	2 mul add 3 div 3 1 roll%N%
	%	2 mul add 3 div exch%N%
	%	6 2 roll%N%
	%	} def%N%
	%%N%
	%/drwellipse { %% r_x r_y center_x center_y%N%
	%	gsave%N%
	%	translate%N%
	%	2 copy scale%N%
	%	1 exch div exch 1 exch div exch%N%
	%	newpath%N%
	%	0 0 1 0 360 arc%N%
	%	%%closepath%N%
	%	scale%N%
	%	drw_paintpath%N%
	%	grestore%N%
	%	} def%N%
	%%N%
	%/drwbox { %% size_x size_y center_x center_y%N%
	%	gsave%N%
	%	newpath %N%
	%	exch 3 index 2 div sub%N%
	%	exch 2 index 2 div sub%N%
	%	translate%N%
	%	0 0 moveto%N%
	%	dup 0 exch lineto%N%
	%	1 index exch lineto%N%
	%	0 lineto%N%
	%	closepath%N%
	%	drw_paintpath%N%
	%	grestore%N%
	%	} def%N%
	%%N%
	%/drwsoftbox { %% size_x size_y rx ry center_x center_y%N%
	%	gsave%N%
	%	translate%N%
	%	2 copy 1 exch div exch 1 exch div exch 6 2 roll%N%
	%	2 copy scale%N%
	%	newpath%N%
	%	exch 3 1 roll div 2 div%N%
	%	3 1 roll div 2 div exch%N%
	%	/h2 exch def%N%
	%	/w2 exch def%N%
	%	0 h2 neg moveto%N%
	%	w2 h2 neg w2 0 1 arcto 4 { pop } repeat%N%
	%	w2 h2 0 h2 1 arcto 4 { pop } repeat%N%
	%	w2 neg h2 w2 neg 0 1 arcto 4 { pop } repeat%N%
	%	w2 neg h2 neg 0 h2 neg 1 arcto 4 { pop } repeat%N%
	%	closepath%N%
	%	scale%N%
	%	drw_paintpath%N%
	%	grestore%N%
	%	} def%N%
	%%N%
	%/drw_set_mid_x { %% integer%N%
	%	/mid_x exch def%N%
	%	} def%N%
	%%N%
	%/drw_set_mid_y { %% integer%N%
	%	/mid_y exch def%N%
	%	} def%N%
	%%N%
	%/drw_last_box { %% last_matrix  min_x min_y max_x max_y%N%
	%	%% =>  size_x size_y center_x center_y%N%
	%	4 index itransform 5 2 roll%N%
	%	3 -1 roll itransform%N%
	%	exch 3 1 roll 4 copy%N%
	%	sub abs 3 1 roll sub abs exch%N%
	%	6 2 roll%N%
	%	add 2 div 3 1 roll add 2 div exch%N%
	%	} def%N%
	%%N%
	%/set_font_size { %% size%N%
	%	currentfont exch scalefont setfont%N%
	%	} def%N%
	%%N%
	%/mbb {	%% string => lower_left_x lower_left_y upper_right_x upper_right_y%N%
	%	%% Minimum bounding box of a string%N%
	%	gsave%N%
	%	newpath%N%
	%	0 0 moveto%N%
	%	false charpath flattenpath pathbbox%N%
	%	grestore%N%
	%	} def%N%
	%%N%
	%/mbb_size { %% string => width height%N%
	%	%% Size of the minimum bounding box of a string%N%
	%	mbb			%% low_x low_y high_x high_y%N%
	%	exch 4 1 roll		%% high_x low_x low_y high_y%N%
	%	exch sub		%% high_x low_x t_size_y%N%
	%	3 1 roll		%% t_size_y high_x low_x%N%
	%	sub exch		%% t_size_x t_size_y%N%
	%	} def%N%
	%%N%
	%/text_in_box {	%% string size_x size_y center_x center_y =>%N%
	%		%% Place text in box and show it%N%
	%	gsave%N%
	%	translate%N%
	%	dup set_font_size%N%
	%	/size_y exch def	%% string size_x%N%
	%	/size_x exch def	%% string%N%
	%	dup			%% string string%N%
	%	mbb			%% string low_x low_y high_x high_y%N%
	%	exch 4 1 roll exch	%% string high_x low_x high_y low_y%N%
	%	dup /low_y exch def	%% string high_x low_x high_y low_y%N%
	%	sub			%% string high_x low_x t_size_y%N%
	%	/t_size_y exch def	%% string high_x low_x%N%
	%	dup /low_x exch def	%% string high_x low_x%N%
	%	sub			%% string t_size_x%N%
	%	/t_size_x exch def	%% string%N%
	%	size_x t_size_x div	%% string ratio_x%N%
	%	size_y t_size_y div	%% string ratio_x ratio_y%N%
	%	2 copy lt { pop } { exch pop } ifelse%N%
	%				%% string min-of-ratio-x-and-ratio-y%N%
	%	/ratio exch def%N%
	%	t_size_x ratio mul 2 div neg%N%
	%	size_x t_size_x ratio mul sub mid_x mul 2 div add%N%
	%	t_size_y ratio mul 2 div neg%N%
	%	size_y t_size_y ratio mul sub mid_y mul 2 div add%N%
	%	translate%N%
	%	low_x ratio mul neg %N%
	%	low_y ratio mul neg %N%
	%	translate%N%
	%	ratio dup scale%N%
	%	newpath%N%
	%	0 0 moveto%N%
	%	show%N%
	%	grestore%N%
	%	} def%N%
	%%N%
	%%%%%EndProlog%N%
	%%N%
	%%%%%BeginSetup%N%
	%	errordict /handleerror { myhandler } put%N%
	%%N%
	%	0 drw_set_mid_x	0 drw_set_mid_y%N%
	%	clearnextctrl%N%
	%%N%
	%/Helvetica findfont setfont%N%
	%%%%%EndSetup"

	define_fill_color (c: DRW_COLOR) is
		require
			c_not_void: c /= Void
		do
			add_ps ("/drw_fill_color { " +
				c.red.out + " " +
				c.green.out + " " +
				c.blue.out + " setrgbcolor } def")
		end

	set_stroke_color (c: DRW_COLOR) is
		require
			c_not_void: c /= Void
		do
			add_ps (c.red.out + " " +
				c.green.out + " " +
				c.blue.out + " setrgbcolor")
		end

	apply_fill_values (f: DRW_FILLABLE) is
		do
			if f.is_stroked then
				if f.stroke_line_cap > 0 then
					add_ps (f.stroke_line_cap.out + " setlinecap")
				end
				if f.has_stroke then
					set_stroke_color (f.stroke)
				end
			end
			if f.has_fill then
				define_fill_color (f.fill)
			end
			if f.is_stroked and f.has_fill then
				add_ps ("drw_set_paintfillstroke")
			elseif f.has_fill then
				add_ps ("drw_set_paintfill")
			else
				add_ps ("drw_set_paintstroke")
			end
		end


	apply_stroke_values (f: DRW_SHAPE) is
		do
			if f.is_stroked then
				if f.stroke_line_cap > 0 then
					add_ps (f.stroke_line_cap.out + " setlinecap")
				end
				if f.has_stroke then
					set_stroke_color (f.stroke)
				end
			end
		end

	reset_fill_values is
		do
			add_ps ("drw_restore_paintpath")
		end


	draw_line (x1, y1, x2, y2: DOUBLE) is
		do
			add_ps ("newpath")
			add_ps (x1.out + " " +
				y1.out + " moveto " +
				x2.out + " " +
				y2.out + " lineto")
			add_ps ("drw_paintpath")
		end

	draw_ellipse (x, y, rx, ry: DOUBLE) is
			-- Generate postscript to draw an ellipse centered at
			-- (`x',`y') with `rx' and `ry' as the radii
		do
			add_ps (rx.out + " " +
				ry.out + " " +
				x.out + " " +
				y.out + " drwellipse")
		end

	draw_path (path: DRW_PATH) is
			-- Draw standard representation of `path' to canvas.
			-- Local transform already applied
		do
			apply_fill_values (path)
			from
				add_ps ("clearnextctrl newpath")
				path.start
			until
				path.after
			loop
				path.item.execute_command (Current)
				path.forth
			end	
			add_ps ("drw_paintpath")
			reset_fill_values
		end

	draw_rectangle (rectangle: DRW_RECTANGLE) is
			-- Draw standard representation of `rectangle' to canvas.
			-- Local transform already applied
		do
			apply_fill_values (rectangle)
			add_ps (rectangle.width.out + " " +
				rectangle.height.out + " " +
				rectangle.x.out + " " +
				rectangle.y.out + " drwbox")
			reset_fill_values
		end

	draw_soft_rectangle (rectangle: DRW_RECTANGLE  rx, ry: DOUBLE) is
			-- Draw standard representation of `rectangle' to canvas.
			-- Local transform already applied
		do
			if rx = 0 and ry = 0 then
				draw_rectangle (rectangle)
			else
				apply_fill_values (rectangle)
				add_ps (rectangle.width.out + " " +
					rectangle.height.out + " " +
					rx.out + " " +
					ry.out + " " +
					rectangle.x.out + " " +
					rectangle.y.out + " drwsoftbox")
				reset_fill_values
			end
		end

	draw_polyfigure (lines: DRW_POLYLINE closed: BOOLEAN) is
			-- Draw standard representation of `lines' to canvas.
		local
			sw: DOUBLE
		do
			sw := lines.stroke_width
			save_gstate
			if lines.is_transformed then
				if sw > 0 then
					-- save state again and apply transform
					alter_current_transform (lines.transform)
					add_ps (sw.out + " dup dtransform")
					restore_previous_gstate
					add_ps ("idtransform drwabs_v setlinewidth")
					draw_list_of_points (lines.as_transformed_points (Identity), closed)
				else
					draw_list_of_points (lines.as_transformed_points (Identity), closed)
				end
			elseif sw > 0 then
				add_ps (sw.out + " setlinewidth")
				draw_list_of_points (lines.as_transformed_points (Identity), closed)
			else
				draw_list_of_points (lines.as_transformed_points (Identity), closed)
			end
			restore_previous_gstate
		end

	draw_list_of_points (d: SIMPLE_LIST_OF_POINTS  closed: BOOLEAN) is
			-- Draw the polygon from already transformed list of points
		local
			i: INTEGER
		do
			from
				add_ps ("newpath")
				i := 1
				add_ps (d.i_th_x (i).out + " " + d.i_th_y (i).out + " moveto")
				i := i + 1
			until
				i > d.count
			loop
				add_ps (d.i_th_x (i).out + " " + d.i_th_y (i).out + " lineto")
				i := i + 1
			end
			if closed then
				add_ps ("closepath drw_paintpath")
			else
				add_ps ("drw_paintpath")
			end
		end

end -- class DRW_POSTSCRIPT_FIGURE_DRAWER
