indexing
	description:
		"Projector to output a drawing to a Postscript file."
	author: "Barden E. Smith"
	date:	"September 27, 2004"


class
	DRW_POSTSCRIPT_PROJECTOR

inherit
	DRW_PROJECTOR_ROUTINES
		rename
			restore_previous_transform as restore_previous_gstate
		redefine
			default_create,
			initialize_projection,
			finish_projection
		end

	DRW_POSTSCRIPT_FIGURE_DRAWER
		redefine
			default_create
		end

create
	default_create,
	make_with_file,
	make_with_filename

feature {NONE} -- Initialization

	make_with_filename (a_filename: FILE_NAME) is
			-- Create projector using filename `a_filename'.
		require
			a_filename_not_void: a_filename /= Void
		do
			filename := a_filename
			file := Void		-- insure that no file is created yet.
			make_processor
		end

	make_with_file (a_file: LARGE_PLAIN_TEXT_FILE) is
			-- Create projector using file `a_file'.
		require
			file_not_void: a_file /= Void
			file_is_open_write: a_file.is_open_write
		do
			file := a_file
			make_processor
		end

	default_create is
		do
			make_processor
		end


feature {NONE} -- Implementation

	file: LARGE_PLAIN_TEXT_FILE

	filename: FILE_NAME

	establish_document (a_document: DRW_DOCUMENT) is
			-- Finish establishing document
		local
			r: DRW_BASIC_RECTANGLE
			p: DRW_PAPER_SIZE
			x, y: DOUBLE
		do
			document := a_document
			p := document.paper
			if p /= void then
				x := p.as_pts (p.left_margin) + 0.5 * p.width_pts
				y := p.as_pts (p.bottom_margin) + 0.5 * p.height_pts
				create r.make (p.width_pts, p.height_pts, x, y)
			else
				create r.make (612, 792, 306, 396)
			end
			create device_view_box.make_normal (r)
		end

	add_headers is
			-- Output standard projection to postscript.
		do
			add_ps ("%%!PS-Adobe-3.0")
			add_ps ("%%%%Pages: " + document.pages.count.out)
			add_mosis_header
			add_ps_no_comments (preamble)
		end

	add_mosis_header is
		do
			add_ps ("%%%%Copyright: (C) 2005 University of Southern California.")
			add_ps ("%%Generated by:%N%
			%%%The MOSIS Service. A service of the University of Southern California.%N%
			%%%Copyright (C) 2005 University of Southern California.%N%
			%%%All rights reserved. Duplication and distribution prohibited.%N%
			%%%Customer support: http://www.mosis.com/%N")
		end

	add_footer is
		do
			add_ps ("%%%%Trailer")
			add_ps ("%%%%EOF")
		end

	initialize_projection is
		do
			precursor
			current_page := 0
			create postscript_result.make (5000)
			add_headers
		end
			
	finish_projection is
		do
			add_footer
			if file = Void and filename /= Void then
				create file.make_open_write (filename)
			end
			if file /= Void then
				file.put_string (postscript_result)
			end
			if filename /= Void then
				file.close
				file := Void
				filename := Void
			end
			precursor
		end

end -- class DRW_POSTSCRIPT_PROJECTOR
