indexing
	description:
		"A drawing shape (subset of DRW_FIGURE)."
	author: "Barden E. Smith"
	date:	"August 13, 2004"

deferred class
	DRW_SHAPE

inherit
	DRW_FIGURE

	DRW_TRANSFORMABLE


feature -- Shape attributes

	stroke: DRW_COLOR

	stroke_width: DOUBLE

	transformed_stroke_width (m:MATRIX): DOUBLE is
		local
			pt, pt0: POINT
		do
			pt := m.transform_xy (stroke_width, stroke_width)
			pt0 := m.transform_xy (0.0, 0.0)
			Result := radius (pt.x - pt0.x, pt.y - pt0.y)
		end

	is_drawable: BOOLEAN is
			-- true if stroked
		deferred
		end

feature -- Status

	has_stroke: BOOLEAN is
		do
			Result := stroke /= Void
		end

	is_stroked: BOOLEAN is
		do
			Result := not stroke_disabled
		end

	is_stroke_line_cap_butt: BOOLEAN is
		do
			Result := stroke_line_cap = 0
		end

	is_stroke_line_cap_round: BOOLEAN is
		do
			Result := stroke_line_cap = 1
		end

	is_stroke_line_cap_square: BOOLEAN is
		do
			Result := stroke_line_cap = 2
		end

	is_stroke_line_join_miter: BOOLEAN is
		do
			Result := stroke_line_join = 0
		end

	is_stroke_line_join_round: BOOLEAN is
		do
			Result := stroke_line_join = 1
		end

	is_stroke_line_join_bevel: BOOLEAN is
		do
			Result := stroke_line_join = 2
		end

feature -- Modification

	set_stroke (s_color: DRW_COLOR) is
			-- sets the stroke color to `s_color'
			-- reenable stroking with previous width
		do
			stroke := s_color
			stroke_disabled := False
		end

	set_stroke_width (s_width: DOUBLE) is
			-- sets the stroke width to `s_width'
			-- reenable stroking with previous color
		require
			width_abs: s_width >= 0
		do
			stroke_width := s_width
			stroke_disabled := False
		end

	disable_stroke is
			-- disables stroking for this figure
		do
			stroke_disabled := True
		end

	set_stroke_line_cap_butt is
			-- set the stroke line cap to butt
		do
			stroke_line_cap := 0
		end

	set_stroke_line_cap_round is
			-- set the stroke line cap to butt
		do
			stroke_line_cap := 1
		end

	set_stroke_line_cap_square is
			-- set the stroke line cap to butt
		do
			stroke_line_cap := 2
		end


	set_stroke_line_join_miter is
			-- set the stroke line cap to butt
		do
			stroke_line_join := 0
		end

	set_stroke_line_join_round is
			-- set the stroke line cap to butt
		do
			stroke_line_join := 1
		end

	set_stroke_line_join_bevel is
			-- set the stroke line cap to butt
		do
			stroke_line_join := 2
		end


feature {DRW_COMMAND_PROCESSOR} -- Implementation and projection data

	stroke_line_cap: INTEGER

	stroke_line_join: INTEGER


feature {NONE} -- Implementation

	stroke_disabled: BOOLEAN


invariant
	stroke_width_non_negative: stroke_width >= 0

end -- class DRW_SHAPE
