indexing
	description:
		"A drawing path."
	author: "Barden E. Smith"
	date:	"August 13, 2004"

class
	DRW_TEXT

inherit
	DRW_SHAPE

	DRW_COORDINATE
		undefine
			is_equal, out
		end

	DRW_VIEW_BOX
		undefine
			copy, default_create, is_equal
		end

create
	make

feature -- creation

	make (s: STRING) is
			-- Make drawing text from string
		require
			s_not_void: s /= Void
		do
			full_text := s
		ensure
			has_text: full_text = s
		end

feature -- access

	full_text: STRING

	font: STRING

	size: DOUBLE

feature -- Shape attributes

	is_drawable: BOOLEAN is
			-- true if stroked
		do
			Result := size > 0 and not full_text.is_empty
		end

	has_font: BOOLEAN is
		do
			Result := font /= Void and then not font.is_empty
		end

feature -- Element change

	set_size (sz: DOUBLE) is
			-- Set nominal text height (in user units)
			--  zero disables display
		require
			sz_positive: sz >= 0
		do
			size := sz
		end


	set_font (f: STRING) is
			-- Set name of the font to pass to the projector
			-- Void or empty string is projector dependent
		do
			font := f
		end


feature -- Size estimating


	bounding_box: DRW_BASIC_RECTANGLE is
		do
			if has_view_box then
				Result := view_box.twin
			elseif size > 0 then
				Result := text_bounding_box
			end
		end

	text_bounding_box: DRW_BASIC_RECTANGLE is
		do
			Result := box_transformed (untransformed_bounding_box)
		end

	untransformed_bounding_box: DRW_BASIC_RECTANGLE is
		local
			w, h: DOUBLE
		do
			h := 0.85 * size
			w := 1.15 * full_text.count.max (1) * size	-- just guessing
			create Result.make (w, h, x + 0.5 * w, y + 0.5 * h)
		end

feature {DRW_PROJECTOR} -- plotting routine

	project (plotter: DRW_FIGURE_DRAWING_ROUTINES) is
		do
			if is_drawable then
				plotter.draw_figure_text (current)
			end
		end

end -- class DRW_TEXT
