indexing
	description:
		"Attribute and routine for transforming figures."
	author: "Barden E. Smith"
	date:	"September 9, 2004"

class
	DRW_TRANSFORMABLE

create

feature -- Group attributes

	transform: DRW_MATRIX


	is_transformed: BOOLEAN is
		do
			Result := transform /= Void and then not transform.is_identity
		end

	box_transformed (b: DRW_BASIC_RECTANGLE): DRW_BASIC_RECTANGLE is
		local
			c0, c1: DRW_COORDINATE
			p: POINT
		do
			if b = Void then
			elseif is_transformed then
				p := transform.transform_xy (b.left, b.bottom)
				create c0.set_point (p)
				p := transform.transform_xy (b.right, b.top)
				create c1.set_point (p)
				create Result.make_by_coordinates (c0, c1)
				if not transform.is_manhattan then
					p := transform.transform_xy (b.left, b.top)
					create c0.set_point (p)
					p := transform.transform_xy (b.right, b.bottom)
					create c1.set_point (p)
					Result.merge (create {DRW_BASIC_RECTANGLE}.make_by_coordinates (c0,c1))
				end
			else
				create Result.make (b.width, b.height, b.x, b.y)
			end
		end

	coordinate_transformed (c: DRW_COORDINATE): DRW_COORDINATE is
		do
			if is_transformed then
				Result := coordinate_transformed_xy (c.x, c.y)
			else
				Result := c
			end
		end

	coordinate_transformed_xy (a_x, a_y: DOUBLE): DRW_COORDINATE is
		local
			p: POINT
		do
			if is_transformed then
				p := transform.transform_xy (a_x, a_y)
				create Result.set_point (p)
			else
				create Result.set (a_x, a_y)
			end
		end
	
feature -- Figure modification

	set_transform (m: DRW_MATRIX) is
		-- set a transform that applies to the figure (or all figures)
		do
			transform := m
		end


end -- class DRW_TRANSFORMABLE
