indexing
	description:
		"Attributes and routines for establishing a view box."
	author: "Barden E. Smith"
	date:	"September 9, 2004"

class
	DRW_VIEW_BOX

create
	make_normal,
	make_mirror_y

feature {NONE} -- Creation

	make_normal (b: DRW_BASIC_RECTANGLE) is
		require
			box_exists: b /= Void
		do
			view_box := b
		end

	make_mirror_y (b: DRW_BASIC_RECTANGLE) is
		require
			box_exists: b /= Void
		do
			view_box := b
			mirror_y := True
		end

feature -- Group attributes

	view_box: DRW_BASIC_RECTANGLE

	distort_aspect: BOOLEAN

	preserve_aspect_ratio: BOOLEAN is
			-- Preserve aspect ratio if true
		do
			Result := not distort_aspect
		end
		

	mid_x: INTEGER
			-- negative, zero, or positve to position in view_box (along x-axis)
			-- zero if centered

	mid_y: INTEGER
			-- negative, zero, or positve to position in view_box (along y-axis)
			-- zero if centered

	slice: BOOLEAN
			-- slice (clip) source image when mapping

	meet: BOOLEAN is
			-- source images meets the edge of the box when mapping
			--  (oposite of slice)
		do
			Result := not slice
		end


	mirror_y: BOOLEAN
			-- Increasing y values in drawing should map to decreasing y values in view box.

feature -- Status

	has_view_box: BOOLEAN is
		do
			Result := view_box /= Void
		end


feature -- Coordinate mapping

	matrix_map (src_box: DRW_BASIC_RECTANGLE): DRW_MATRIX is
			-- Create a Matrix that will transform coordinates
		require
			have_src_box: src_box /= Void
		local
			a, a0, b, b0, tx, ty: DOUBLE
		do
			a0 := view_box.width / src_box.width
			b0 := view_box.height / src_box.height
			if distort_aspect then
				a := a0
				b := b0
			elseif slice then
				a := a0.max(b0)
				b := a
			else
				a := a0.min(b0)
				b := a
			end
			if mirror_y then
				b := -b
			end
			tx := view_box.x - a * src_box.x
			ty := view_box.y - b * src_box.y
			if (a /= a0) and (mid_x /= 0) and not distort_aspect then
				tx := tx + 0.5 * mid_x * (view_box.width - a * src_box.width)
			end
			if (b /= b0) and (mid_y /= 0) and not distort_aspect then
				ty := ty + 0.5 * mid_y * (view_box.height - b.abs * src_box.height)
			end
			create Result.make_arbitrary (a, 0, 0, b, tx, ty)
		end

feature -- Group modification

	set_view_box (v: DRW_BASIC_RECTANGLE) is
			-- set a view_box that applies to all figures.
		do
			view_box := v
		end

	set_mid_x (v: INTEGER) is
			-- set flag for adjustment direction.
		do
			mid_x := v.sign
		end

	set_mid_y (v: INTEGER) is
			-- set flag for adjustment direction.
		do
			mid_y := v.sign
		end

	set_preserve_aspect_ratio (v: BOOLEAN) is
			-- set to true (default) if mapping preserves aspect ratio
			--  i.e. square objects remain square if aspect is preserved.
		do
			distort_aspect := not v
		end

	set_slice (v: BOOLEAN) is
			-- set to true to fill the box clipping the larger side
			--  set to false to size to fill box without clipping.
		do
			slice := v
		end

end -- class DRW_VIEW_BOX
