
indexing
	
	description: "An operation that edit cut patterns for a run production"
	author: "David Hollenberg"
	date: "August 22, 2007"

class EDIT_CUT_PATTERNS_OP
inherit
	OPERATION
		rename
			try_operation as try_to_edit_cut_patterns
		end
	DB_QUERY_OPTIONS
		export
			{NONE} all
		undefine
			default_create
		end
	DB_OPTIONS_ACCESS
		export
			{NONE} all
		undefine
			default_create
		end
	STRING_UTILITIES_ACCESS
		export
			{NONE} all
		undefine
			default_create
		end
	ITEM_COUNT_RANGES_ACCESS
		export
			{NONE} all
		undefine
			default_create
		end
	RC_UTILITIES_ACCESS
		export
			{NONE} all
		end
	GUI_UTILITIES_ACCESS
		export
			{NONE} all
		undefine
			default_create
		end
	USER_INTERFACE_ACCESS
		export
			{NONE} all
		undefine
			default_create
		end

create
	default_create

feature -- Operations

	edit_cut_patterns (prod_id: STRING sess: RC_RUN_CLOSING_SESSION)
			-- Generate bonding diagrams for production `prod_id',
			-- using `sess'
		require
			prod_id_not_void: prod_id /= Void
			session_not_void: sess /= Void
			session_has_connection: sess.connection /= Void
			session_connection_open: sess.connection.is_open
		do
			session := sess
			production_id := prod_id
			connection := sess.connection
			cache := sess.cache
			execute
		end

	cleanup_after_abort
			-- Do cleanup needed after operation is aborted
		do
			-- FIXME: implement if necessary
		end

feature {NONE} -- Implementation

	session: RC_RUN_CLOSING_SESSION
			-- Session to use

	connection: DB_CONNECTION
			-- Database connection to use

	cache: RUN_CACHE
			-- Database cache

	production_id: STRING
			-- Production id for which to try to edit cut
			-- patterns

	run_id: STRING
			-- Run id for production, if production exists

	packing_id: STRING
			-- Packing ID assigned to run

	runs_row: RUNS_MROW
			-- Row for run, if run exists
			
	packing: RC_PACKING
			-- Packing synthesized from official database tables
			
	packings_row: PACKINGS_MROW
			-- Detached row for packing synthesized from
			-- official database tables

	packing_version: PACKING_VERSIONS_MROW
			-- Packing version synthesized from official database 
			-- tables
			
	try_to_edit_cut_patterns
			-- Try to edit cut patterns for production
		do
			do_validation
			session.lock
			session.open_log
			build_cut_pattern_interface
		end

	do_validation
			-- Do validation before generating bonding diagrams
			-- for run (authorization checks, etc)
		local
			role_setter: ROLE_SET_OP
			prods_row: PRODUCTIONS_ROW
			dies: LIST [DIES_ROW]
			cut_patterns: LIST [RUN_CUT_PATTERNS_ROW]
			n: INTEGER
			env: RC_ENVIRONMENT
			type: LITHO_TYPE_VALUES
			litho_type: STRING
			ifi_row: IC_FAB_INSTANCES_ROW
			multiple_orders: LIST [IC_FAB_INSTANCES_ROW]
		do
			-- Verify that user is authorized to edit cut patterns

			env := session.environment
			create role_setter
			role_setter.set_role (connection, env.db_run_closer_role, env.db_access_control)
			
			-- Check whether production exists

			create prods_row.make_by_key (connection, Old_no_exception_row, production_id)
			if prods_row.is_empty then
				handle_user_error ("Production does not exist: " + production_id)
			elseif prods_row.is_null_run_id then
				handle_user_error ("Production " + production_id + " does not have an associated run ID (is not run-oriented)")
			else
				create runs_row.make_by_key (connection, Old_no_exception_row, prods_row.run_id)
				if runs_row.is_empty then
					handle_user_error ("Production " + production_id + " has an associated run " + prods_row.run_id + " but that run does not exist")
				else
					run_id := prods_row.run_id
				end
			end
			
			-- Switch session to correct run

			session.make_run_session (session, run_id)
			cache := session.cache

			-- Check whether there are any dies for run yet

			dies := cache.die_rows
			if dies.is_empty then
				handle_user_error ("There are no dies for run " + run_id + " yet.%NCannot edit cut patterns.")
			end

			-- Check whether litho type is valid

			litho_type := cache.litho_type
			if not litho_type.is_equal (type.Reticle) then
				handle_user_error ("Cannot edit cut patterns for " + production_id + " because its run has the wrong litho type (" + litho_type + ")")
			end

			-- Check whether every cut pattern is assigned to
			-- a packaging lot (if not, can't tell which cut
			-- patterns are part of a production)

			cut_patterns := cache.cut_pattern_rows
			create {ARRAYED_LIST [RUN_CUT_PATTERNS_ROW]} invalid_cut_patterns.make (cut_patterns.count)
			cut_patterns.do_all (agent check_cut_pattern)
			if not invalid_cut_patterns.is_empty then
				n := invalid_cut_patterns.count
				handle_user_error ("There " + su.number (n, "is", "are") + " " + n.out + " cut " + su.number (n, "pattern", "patterns") + " for run " + run_id + " that " + su.number (n, "is", "are") + " not yet assigned to a packaging lot.%NCannot edit cut patterns.")
			end

			-- Check for fab instances with more than one
			-- ordered process and warn user that these projects
			-- won't be displayed accurately (will only show
			-- one of the processes)

			create ifi_row.make_template
			multiple_orders := ifi_row.multiple_process_rows_for_run (run_id, connection, expected_count.At_least_zero)
			if not multiple_orders.is_empty then
				warn ("The following projects will not be displayed accurately because they have%Nmore than one ordered wafer process (or no ordered processes).%NFor each project, the first lot split name in its list will be displayed:%N%N" + multiple_orders_description (multiple_orders))
			end

		end

	check_cut_pattern (a_pattern: RUN_CUT_PATTERNS_ROW)
			-- Check that `a_pattern' is assigned to a packaging
			-- lot and add it to `invalid_cut_patterns' if not
		do
			if a_pattern.is_null_packaging_lot_no then
				invalid_cut_patterns.extend (a_pattern)
			end
		end

	build_cut_pattern_interface
			-- Build interface for cut pattern editor
		local
			ifc: RC_PACKING_IFC
		do
			if runs_row.is_null_packing_id then
				-- No packing ID set (old run).
				-- Create packing with run ID and put in
				-- database so that colors can be assigned 
				-- to lot splits
				packing_id := run_id
				add_packing_to_database
			end
			packing_id := runs_row.packing_id
			synthesize_packing
					
			session.set_packing_id (packing_id)
			ifc := session.factory.packing_interface (Void, run_id, packing_id, session.title_with_run_and_packing_id ("Edit"))
			ifc.set_session (session)
			ifc.set_packing_session (session)
			ifc.set_run_id (run_id)
			ifc.set_production_id (production_id)
			ifc.set_packing_id (packing_id)
			ifc.set_is_read_only (False)
			ifc.set_is_cut_editor_only (True)
			ifc.set_packing (packing)
			ifc.set_packings_row (packings_row)
			ifc.set_official_packing_version (packing_version)
			ifc.success_actions.append (success_actions)
			ifc.cancel_actions.append (success_actions)
			success_actions.wipe_out
			abort_actions.wipe_out
			ifc.build
		end

	add_packing_to_database
			-- Create a new PACKINGS row for run, point run
			-- to it and insert it in the database
		local
			row: PACKINGS_MROW
		do
			create row.make_null (connection)
			row.set_packing_id (packing_id)
			row.initialize_from_run (runs_row)
			row.set_packed_with_new_packer (False)
			row.set_packing_id (packing_id)
			session.commit
		end

	synthesize_packing
			-- Synthesize a packing that looks like it was
			-- fetched from the PACKINGS table, with a
			-- current version that has all dependent rows
			-- filled in from official (non-packing)
			-- database tables and put it in `packing'.
			-- This ensures that all data will be up to
			-- date, in case anything was modified since
			-- the run was packed (packing tables might
			-- have data that is out-of-date).
		local
			packing_creator: PACKING_CREATOR
		do
			create packing_creator.make (production_id, packing_id, runs_row)
			packing_creator.set_dies (cache.die_rows)
			packing_creator.set_die_locations (cache.die_location_rows)
			packing_creator.set_cut_patterns (cut_patterns_for_production)
			packing_creator.set_cut_locations (cache.cut_location_rows)
			packing_creator.set_cut_die_sites (cache.cut_die_site_rows)
			packing_creator.synthesize
			packing := packing_creator.packing
			packings_row := packing_creator.packings_row
			packing_version := packing_creator.packing_version
		end

	cut_patterns_for_production: LIST [RUN_CUT_PATTERNS_ROW]
			-- All cut patterns that are for `production_id'
		local
			row: RUN_CUT_PATTERNS_ROW
		do
			create row.make_template
			Result := row.rows_for_production (production_id, connection, expected_count.At_least_zero)
		end

	multiple_orders_description (list: LIST [IC_FAB_INSTANCES_ROW]): STRING
			-- Description of projects with multiple processes
			-- ordered
		require
			list_not_void: list /= Void
		local
			ifi: IC_FAB_INSTANCES_ROW
		do
			create Result.make (100)
			from
				list.start
			until
				list.after
			loop
				ifi := list.item
				Result.append ("    ")
				Result.append_integer (ifi.design_no)
				Result.append ("    ")
				Result.append (su.joined_by (ifi.ordered_wafer_processes, ", "))
				Result.extend ('%N')
				list.forth
			end
		end

	invalid_cut_patterns: LIST [RUN_CUT_PATTERNS_ROW]
			-- Cut patterns that are invalid because they are
			-- not assigned to a packaging lot

end
