
indexing
	
	description: "Window to allow user to edit preferences for run %
		%closing session"
	author: "David Hollenberg"
	date: "February 26, 2004"

class EDIT_PREFERENCES_WINDOW

inherit
	EDIT_PREFERENCES_IFC
	WINDOW_INTERFACE
	RC_ENVIRONMENT_ACCESS
		rename
			rc_env as environment
		export
			{NONE} all
		undefine
			default_create
		end
	MOSIS_GRAPHICAL_UTILITIES
		export
			{NONE} all
		undefine
			default_create
		end
		
create
	make

feature {NONE} -- Creation

	make (w: EV_TITLED_WINDOW)
			-- Create interface in window `w'
		require
			window_exists: w /= Void
			window_valid: not w.is_destroyed
		do
			make_interface
			make_proxy (w)
		end

feature -- Operations

	reset
			-- Reset interface
		do
		end

	refresh
			-- Refresh interface from current preferences
		do
			if preferences.is_autosubmit_enabled then
				auto_submit_button.enable_select
			else
				auto_submit_button.disable_select
			end
			if preferences.is_cache_auto_flush_enabled then
				cache_auto_flush_button.enable_select
			else
				cache_auto_flush_button.disable_select
			end
			if preferences.is_unattended_mode_enabled then
				unattended_mode_button.enable_select
			else
				unattended_mode_button.disable_select
			end
		end
		
feature {NONE} -- Implementation

	build_interface
			-- Build interface in `window'
		local
			vbox, main_vbox: EV_VERTICAL_BOX
			buttons: HORIZONTAL_BUTTON_SET
			box: EV_HORIZONTAL_BOX
			frame: EV_FRAME
		do
			create auto_submit_button.make_with_text ("Submit jobs automatically")
			create cache_auto_flush_button.make_with_text ("Automatically flush cache after " + environment.cache_auto_flush_interval.out + " seconds of inactivity")
			create unattended_mode_button.make_with_text ("Unattended mode")
			create vbox
			create main_vbox
			main_vbox.set_border_width (user_interface.border_width)
			vbox.set_border_width (user_interface.border_width)
			vbox.extend (auto_submit_button)
			vbox.disable_item_expand (auto_submit_button)
			vbox.extend (cache_auto_flush_button)
			vbox.disable_item_expand (cache_auto_flush_button)
			vbox.extend (unattended_mode_button)
			vbox.disable_item_expand (unattended_mode_button)
			
			create buttons.make_with_text_and_actions (<< gui.ev_ok, reset_text, gui.ev_cancel >>, << agent process_accept, agent process_reset, agent process_decline >>)
			box := buttons.column_set.box
			
			vbox.extend (box)
			vbox.disable_item_expand (box)
			
			create frame
			frame.extend (vbox)
			
			main_vbox.extend (frame)
			main_vbox.disable_item_expand (frame)
			gui.set_box_background_recursive (main_vbox, background_color)
			extend (main_vbox)

			close_request_actions.extend (agent process_decline)
			gui.add_cancel_accelerator (window, agent process_decline)
			refresh
			enable_active
		end
		
	preferences: RC_PREFERENCE_SET
			-- User preferences
		do
			Result := session.preferences
		end

	process_accept
			-- Take actions needed when user accepts
			-- current operation
		do
			if is_active then
				disable_active
				set_preferences
				hide
			end
		end

	process_reset
			-- Take actions needed when user clicks on Reset
		do
			if is_active then
				preferences.set_defaults
				refresh
			end
		end

	process_decline
			-- Take actions needed when user declines
			-- current operation
		do
			if is_active then
				disable_active
				hide
			end
		end

	set_preferences
			-- Set preferences from current values in interface
		do
			if auto_submit_button.is_selected then
				preferences.enable_autosubmit
			else
				preferences.disable_autosubmit
			end
			if cache_auto_flush_button.is_selected then
				preferences.enable_cache_auto_flush
			else
				preferences.disable_cache_auto_flush
			end
			if unattended_mode_button.is_selected then
				preferences.enable_unattended_mode
			else
				preferences.disable_unattended_mode
			end
		end
		
	auto_submit_button: EV_CHECK_BUTTON
	
	unattended_mode_button: EV_CHECK_BUTTON
	
	cache_auto_flush_button: EV_CHECK_BUTTON
	
	Reset_text: STRING = "Reset"

end
