deferred class ESSENCE_OF_PROCESS

inherit

	MOSIS_TECH_VALUES

	STRING_UTILITIES_ACCESS

feature	-- the deferred stuff

	connection: DB_CONNECTION is deferred end;
	wafer_techn_family: STRING is deferred end;
	wafer_process_id: STRING is deferred end;

feature -- options_list stuff

	my_options: DB_OPTIONS_LIST[WAFER_PROCESS_OPTIONS_ROW] is
	do  if wpo_cached_list = Void then
		fetch_wpo_cached_list
	    end;
	    Result := wpo_cached_list;
	ensure Result /= Void
	end;

feature {NONE}

	wpo_cached_list: DB_OPTIONS_LIST[WAFER_PROCESS_OPTIONS_ROW];

	fetch_wpo_cached_list is
	local r: WAFER_PROCESS_OPTIONS_ROW; b: DB_BIND_SS;
		lst: LIST[WAFER_PROCESS_OPTIONS_ROW]
	do  create r.make_template;
	    create b.make(wafer_techn_family, wafer_process_id);
	    lst := r.all_rows_filtered(connection, fetch_wpo_filter, b);
	    create wpo_cached_list.make(lst.count);
	    wpo_cached_list.append(lst)
	end;

	fetch_wpo_filter: STRING is
	"wafer_techn_family=:1 and wafer_process_id=:2"

feature	-- option presence

	is_epi: BOOLEAN is
		-- lookup the epi option for this process, true if present
	do  Result := has_option(option_id_is_epi);
	end;

	is_non_epi: BOOLEAN is
		-- lookup the non_epi option for this process, true if present
	do  Result := has_option(option_id_is_bulk);
	end;

	epi_option: STRING is
		-- decide on which to report depending on options available
	do
		if is_epi then
		    check not_both_epi_and_non_epi: not is_non_epi; end;
		    Result := option_id_is_epi;
		elseif is_non_epi then Result := option_id_is_bulk;
		end;
	end;

	has_option(option_id: STRING): BOOLEAN is
	do
		Result := my_options.has_id(option_id);
	end;

feature	-- process qualification

	incompatibilities: ARRAYED_LIST[STRING];

	is_qualified_process(
		proc_opts: LIST[ABSTRACT_PROCESS_OPTION]): BOOLEAN is
		-- do I have all the options demanded 
		--  and lack all the options prohibited
		-- by the set of proc_opts?
		-- the incompatible options are left in incompatibilities
	require have_proc_opts: proc_opts /= Void
	local proc_opt: ABSTRACT_PROCESS_OPTION;
	do  if incompatibilities = Void
	     then create incompatibilities.make(16);
		  incompatibilities.compare_objects
	     else incompatibilities.wipe_out
	    end;
	    from  proc_opts.start
	    until proc_opts.after
	    loop  proc_opt := proc_opts.item;
		  if has_option(proc_opt.option_id) /=
			proc_opt.option_in_process
		   then incompatibilities.extend(proc_opt.to_short_string)
		  end;
		  proc_opts.forth
	    end
	    Result := incompatibilities.is_empty
	    ensure Result = incompatibilities.is_empty
	end;

	compare_incompatibility(other: like Current): INTEGER is
		-- just compare size of incompatibilities; less is closer
	do  Result := incompatibilities.count.three_way_comparison
					(other.incompatibilities.count)
	end;

end -- ESSENCE_OF_PROCESS
