
indexing

	author: "David Hollenberg"
	date: "December 20, 2005"

class FAB_POOL_W_WAFER_TECHNS_ROW
inherit
	FAB_POOL_W_WAFER_TECHNS_RAW
	ROW_COUNT_CHECK_ACCESS
		export
			{NONE} all
		end
	NULL_VALUE_CHECK_ACCESS
		export
			{NONE} all
		end
	ITEM_COUNT_RANGES_ACCESS
		export
			{NONE} all
		end
	STRING_UTILITIES_ACCESS
		export
			{NONE} all
		end
	WAFER_OPTION_UTILITIES_INTERFACE
		rename
			compatible_processes as cached_compatible_processes
		export
			{ANY} compute_design_wt_layer_errors,
				design_wt_layer_errors
			{NONE} all
		end
	DB_OPTIONS_ACCESS
		export
			{NONE} all
		end
create
	make_template, make_null

feature -- Collections

	passed_projects_for_wafer_techn (wt_name: STRING conn: DB_CONNECTION range: ITEM_COUNT_RANGE): LIST [like Current] is
			-- Projects in state "passed" which can be
			-- fabbed on wafer techn `wt_name'
		require
			wt_name_not_void: wt_name /= Void
			connection_exists: conn /= Void
			connection_open: conn.is_open
			range_exists: range /= Void
		local
			b: DB_BIND_SS
			state: DESIGN_ACCEPT_STATE_VALUES
		do
			create b.make (wt_name, state.Passed)
			Result := all_rows_filtered (conn, "wafer_techn = :1 and design_accept_state = :2", b)
			check_row_count (Result, range)
		ensure
			result_exists: Result /= Void
		end

	row_for_fab_no_and_wafer_techn (mfn: INTEGER wt_name: STRING conn: DB_CONNECTION): like Current is
			-- Project with fab number `mfn' which can be
			-- fabbed on wafer techn `wt_name'
		require
			wt_name_not_void: wt_name /= Void
			connection_exists: conn /= Void
			connection_open: conn.is_open
		local
			b: DB_BIND_IS
			list: LIST [like Current]
		do
			create b.make (mfn, wt_name)
			list := all_rows_filtered (conn, "m_fab_no = :1 and wafer_techn = :2", b)
			if not list.is_empty then
				Result := list.first
			end
		end

feature -- Operations

	check_policy_selectability (p: PACKINGS_ROW run: STRING wants_wrong_type, wants_wrong_foundry, wants_no_access, wants_assigned, wants_uncharged: BOOLEAN) is
			-- Check whether `Current' is selectable for
			-- `p' taking into account only desired user
			-- overrides of policies that normally
			-- prevent project from being selected.
			-- Do not check for layer incompatilibities,
			-- lack of compatible processes or other technical
			-- reasons why project cannot be selected.  Set
			-- `is_selectable' to indicate answer
		require
			packings_row_not_void: p /= Void
		do
			is_selectable := True
			reason_unselectable := ""
			if not run_type_requested.is_equal (p.run_type) and not wants_wrong_type then
				set_unselectable ("Wrong run type " + run_type_requested)
			end
			if not is_null_sched_run_no and sched_run_no /= p.sched_run_no then
				set_unselectable ("Coupon for different sched_run_no " + sched_run_no.out)
			end
			if not foundry_compatible and not wants_wrong_foundry then
				set_unselectable ("Foundry restricted to " + su.joined_by (wafer_vendor_constraint_codes, ", "))
			end
			if not access_granted and not wants_no_access then
				set_unselectable ("Fab access to " + p.wafer_techn + " not granted")
			end
			if not is_null_run_id and (run = Void or else not run.is_equal (run_id)) and not wants_assigned then
				set_unselectable ("Assigned to run " + run_id)
			end
			if not has_charges and not wants_uncharged then
				set_unselectable ("Not yet charged")
			end
			if not p.allowed_pricing_schedules.has (pricing_schedule) then
				set_unselectable ("Incompatible pricing schedule " + pricing_schedule)
			end

		end

	check_technical_selectability (p: PACKINGS_ROW) is
			-- Check whether `Current' is selectable for
			-- `p' taking into account only technical issues
			-- like layer incompatilibities and
			-- lack of compatible processes.  Set
			-- `is_selectable' to indicate answer
		local
			das: DESIGN_ACCEPT_STATE_VALUES
			wt_layer_errors: LIST [STRING]
		do
			is_selectable := True
			reason_unselectable := ""
			if not wafer_techn.is_equal (p.wafer_techn) then
				set_unselectable ("Wrong wafer techn " + wafer_techn)
			end
			if not design_accept_state.is_equal (das.Passed) then
				set_unselectable ("Invalid design accept state " + design_accept_state)
			end
			wt_layer_errors := wafer_techn_layer_errors
			if not wt_layer_errors.is_empty then
				set_unselectable ("Wafer techn layer incompatibilities: " + su.joined_by (wt_layer_errors, ", "))
			end

			if is_selectable then
				if compatible_processes.is_empty then
					set_unselectable ("No compatible processes")
				end
			end

		end

	check_full_selectability (p: PACKINGS_ROW run: STRING wants_wrong_type, wants_wrong_foundry, wants_no_access, wants_assigned, wants_uncharged: BOOLEAN) is
			-- Check whether `Current' is selectable for
			-- `p' taking into account desired user
			-- overrides of conditions that normally
			-- prevent project from being selected,
			-- layer incompatilibities and
			-- lack of compatible processes.  Set
			-- `is_selectable' to indicate answer
		local
			is_policy_sel, is_tech_sel: BOOLEAN
			pol_reason, tech_reason: STRING
		do
			check_policy_selectability (p, run, wants_wrong_type, wants_wrong_foundry, wants_no_access, wants_assigned, wants_uncharged)
			is_policy_sel := is_selectable
			pol_reason := reason_unselectable
			check_technical_selectability (p)
			is_tech_sel := is_selectable
			tech_reason := reason_unselectable
			is_selectable := is_policy_sel and is_tech_sel
			reason_unselectable := pol_reason + tech_reason
		end

feature -- Properties

	is_selectable: BOOLEAN
			-- May project be selected for inclusion on run,
			-- taking into account user overrides of conditions
			-- that usually prevent selection?

	reason_unselectable: STRING
			-- Reason(s) project cannot be selected for inclusion
			-- on run

	wafer_vendor_constraint_codes: LIST [STRING] is
			-- List of codes for wafer vendors to which
			-- `Current' is constrained
		do
			if cached_wafer_vendor_constraint_codes = Void then
				fetch_wafer_vendor_constraint_codes
			end
			Result := cached_wafer_vendor_constraint_codes
		end

	is_cached_wafer_vendor_constraint_codes: BOOLEAN is
			-- Are wafer vendor constraint codes currently cached?
		do
			Result := cached_wafer_vendor_constraint_codes /= Void
		end

	fab_instance: IC_FAB_INSTANCES_ROW is
			-- Fab instance for `Current'
		do
			if cached_fab_instance = Void then
				fetch_fab_instance
			end
			Result := cached_fab_instance
		end

	is_cached_fab_instance: BOOLEAN is
			-- Are fab instance currently cached?
		do
			Result := cached_fab_instance /= Void
		end

	wafer_techn_layer_errors: LIST [STRING] is
			-- Wafer techn layer errors for `Current'
		do  
			if cached_wafer_techn_layer_errors = Void then
				fetch_wafer_techn_layer_errors
			end
			Result := cached_wafer_techn_layer_errors
		ensure
			result_not_void: Result /= Void
		end

	is_cached_wafer_techn_layer_errors: BOOLEAN is
			-- Are wafer techn layer errors currently cached?
		do  
			Result := cached_wafer_techn_layer_errors /= Void
		end

	compatible_processes: LIST [STRING] is
			-- Processes with which `Current' is compatible
		do  
			if cached_compatible_processes = Void then
				compute_compatible_processes
			end
			Result := cached_compatible_processes
		ensure
			result_not_void: Result /= Void
		end

	is_cached_compatible_processes: BOOLEAN is
			-- Is lisst of compatible processes currently cached?
		do  
			Result := cached_compatible_processes /= Void
		end

feature -- Modification

	uncache_wafer_vendor_constraint_codes is
			-- Clear wafer vendor constraint code from cache
		do
			cached_wafer_vendor_constraint_codes := Void
		ensure
			constraints_not_cached: not is_cached_wafer_vendor_constraint_codes
		end

	uncache_fab_instance is
			-- Clear fab instance from cache
		do
			cached_fab_instance := Void
		ensure
			fab_instance_not_cached: not is_cached_fab_instance
		end

	uncache_wafer_techn_layer_errors is
			-- Remove wafer techn layer errors from cache
		do  
			cached_wafer_techn_layer_errors := Void
		end

	uncache_compatible_processes is
			-- Remove list of compatible processes from cache
		do  
			cached_compatible_processes := Void
		end

	set_unselectable (s: STRING) is
			-- Append `s' to `reason_unselectable' and set
			-- `is_selectable' to False
		require
			s_not_void: s /= Void
		do
			is_selectable := False
			if reason_unselectable = Void then
				create reason_unselectable.make (s.count)
			end
			if not reason_unselectable.is_empty then
				reason_unselectable.append ("; ")
			end
			reason_unselectable.append (s)
		end

feature {NONE} -- Implementation

	fetch_wafer_vendor_constraint_codes is
			-- Fetch wafer vendor constraint codes for `Current'
		local
			row: IC_FAB_INSTANCEXWAFER_VENDORS_ROW
			list: LIST [IC_FAB_INSTANCEXWAFER_VENDORS_ROW]
		do
			create row.make_template
			list := row.rows_for_fab_no (m_fab_no, connection, expected_count.At_least_zero)
			create {ARRAYED_LIST [STRING]} cached_wafer_vendor_constraint_codes.make (list.count)
			list.do_all (agent append_constraint)
		end
		
	append_constraint (r: IC_FAB_INSTANCEXWAFER_VENDORS_ROW) is
			-- Append constraint code in `r' to 
			-- `cached_wafer_vendor_constraint_codes'
		do
			cached_wafer_vendor_constraint_codes.extend (r.wafer_vendor_constraint_code)
		end
		
	fetch_fab_instance is
			-- Fetch fab instance
		do
			create cached_fab_instance.make_by_key (connection, option.Old_row, m_fab_no)
		end
		
	design_techn_id: STRING
		local   b: DB_BIND_S;
		do
			create b.make(design_techn);
			Result := connection.selected_string
					("design_techn_id", "design_techns", "design_techn = :1", b)
		end;

	fetch_wafer_techn_layer_errors is
			-- Fetch wafer techn layer error
		do
			compute_design_wt_layer_errors
					(connection, design_no, design_techn_id, wafer_techn)
			cached_wafer_techn_layer_errors := design_wt_layer_errors
		end
		
	compute_compatible_processes is
			-- Compute processes on which `Current' can be
			-- fabricated and put in `cached_compatible_processes'
		do
			compute_layer_options (connection, design_no, design_techn_id, wafer_techn)
			compute_processes_for_options (connection, wafer_techn, computed_layer_options, fab_instance.explicit_options)
		end

	cached_wafer_vendor_constraint_codes: like wafer_vendor_constraint_codes
			-- List of wafer vendor constraint codes for
			-- `Current', cached in memory
		
	cached_fab_instance: like fab_instance
			-- Fab instance for `Current', cached in memory
		
	cached_wafer_techn_layer_errors: LIST [STRING]
			-- Wafer techn layer errors for `Current', cached
			-- in memory

end
