
indexing
	
	description: "Window to allow creation of a FAB RUN order %
		%for a dedicated or trusted foundry run"
	author: "David Hollenberg"
	date: "June 16, 2005"

class FAB_RUN_ORDER_WINDOW

inherit
	FAB_RUN_ORDER_IFC
	WINDOW_INTERFACE
		redefine
			enable_active,
			disable_active
		end
	MOSIS_GRAPHICAL_UTILITIES
		export
			{NONE} all
		undefine
			default_create
		end
	NUMBER_UTILITIES_ACCESS
		export
			{NONE} all
		undefine
			default_create
		end
	RC_POLICIES_ACCESS
		export
			{NONE} all
		undefine
			default_create
		end
	UNIX_OS_ACCESS
		export
			{NONE} all
		undefine
			default_create
		end
		
create
	make

feature {NONE} -- Creation

	make (w: EV_TITLED_WINDOW)
			-- Create interface in window `w'
		require
			window_exists: w /= Void
			window_valid: not w.is_destroyed
		do
			make_interface
			make_proxy (w)
		end

feature -- Operations

	reset
			-- Move focus back into window
		do
			enable_active
		end

	refresh
			-- Refresh status
		do
		end

feature -- Modification

	enable_active
			-- Make interface active
		do
			precursor
		end

	disable_active
			-- Make interface inactive
			-- (can look but cannot initiate most actions)
		do
			precursor
		end

feature {NONE} -- Implementation

	build_interface
			-- Build interface
		local
			buttons: HORIZONTAL_BUTTON_SET
			main_vbox: EV_VERTICAL_BOX
			box: EV_BOX
			cols: WIDGET_COLUMN_SET
			label: EV_LABEL
		do
			fetch_data
			add_menu_bar
			create main_vbox
			main_vbox.set_border_width (user_interface.border_width)
			create cols.make (4)
			cols.set_padding (user_interface.padding)
			cols.set_border_width (user_interface.border_width)
			cols.disable_expand
			
			create account_field
			init_field (account_field, Void)
			account_field.select_actions.extend (agent process_account_selection)
			create pricing_date_field
			init_field (pricing_date_field, agent process_pricing_date_change)
			pricing_date_field.enable_edit
			pricing_date_field.disable_sensitive
			cols.append_row (<< gui.label_with_text ("Account"), account_field, gui.label_with_text ("Pricing Date"), pricing_date_field >>)
			
			create quote_id_field
			init_field (quote_id_field, agent process_quote_id_change)
			quote_id_field.enable_edit
			create po_field
			init_field (po_field, Void)
			po_field.disable_sensitive
			po_field.select_actions.extend (agent process_po_selection)
			cols.append_row (<< gui.label_with_text ("Quote ID"), quote_id_field, gui.label_with_text ("PO"), po_field >>)
			
			create order_field
			init_field (order_field, Void)
			order_field.disable_sensitive
			order_field.select_actions.extend (agent process_order_selection)
			cols.append_row (<< gui.label_with_text ("Order"), order_field, gui.label_with_text (""), gui.label_with_text ("") >>)
			
			create amount_field
			init_field (amount_field, Void)
			cols.append_row (<< gui.label_with_text ("Total Amount"), amount_field, gui.label_with_text (""), gui.label_with_text ("") >>)
			
			cols.align_column_right (1)
			cols.align_column_right (3)
			box := cols.box
			main_vbox.extend (box)
			main_vbox.disable_item_expand (box)
			
			label := gui.label_with_text ("Quote Line Items")
			main_vbox.extend (label)
			main_vbox.disable_item_expand (label)
			
			create quote_item_box
			create quote_item_area
			quote_item_area.extend (quote_item_box)
			quote_item_area.resize_actions.extend (agent process_quote_items_resize)
			main_vbox.extend (quote_item_area)
			
			build_pulldown_lists
			
			create buttons.make_with_text_and_actions (<< gui.Ev_ok, "Finish", gui.Ev_cancel >>, << agent process_ok, agent process_finish, agent process_cancel >>)
			box := buttons.column_set.box
			
			main_vbox.extend (box)
			main_vbox.disable_item_expand (box)
			
			gui.set_box_background_recursive (main_vbox, background_color)
			extend (main_vbox)
			close_request_actions.extend (agent process_cancel)
			gui.add_cancel_accelerator (window, agent process_cancel)
			gui.do_once_on_idle (agent post_display_actions)
			refresh
			is_resize_to_minimum_requested := False
		end
		
	fetch_data
			-- Fetch needed data
		do
			fetch_all_accounts
			fetch_all_quotes
			fetch_quotes
		end

	fetch_all_accounts
			-- Fetch list of all allowed accounts for quote
		local
			row: CUST_ACCTS_ROW
			part_no: INTEGER
		do
			create row.make_template
			required_account := Void
			if cache.is_dedicated_run then
				all_accounts := row.cust_ic_rows_for_production (session.run_id, connection, expected_count.At_least_one)
			elseif cache.is_trusted_foundry_run then
				part_no := policies.trusted_foundry_fab_part
				all_accounts := row.rows_quoted_part_number (part_no, connection, expected_count.At_least_zero)
				if all_accounts.is_empty then
					handle_soft_error ("list of valid customer accounts is empty.%NThere are no customer accounts with an unexpired quote that has a line item for part " + part_no.out)
				end
				all_accounts := row.rows_quoted_part_number (policies.trusted_foundry_fab_part, connection, expected_count.At_least_one)
			else
				handle_hard_error ("don't know how to handle run of type " + cache.run_type)
			end
			create account_table.make (all_accounts.count)
			account_table.compare_objects
			all_accounts.do_all (agent add_account (?, account_table))
		end

	add_account (a: CUST_ACCTS_ROW table: like account_table)
			-- Add `a' to `table'
		do
			table.put (a, a.cust_acct_no)
		end

	fetch_all_quotes
			-- Fetch list of all possible quotes
		local
			row: CUST_QUOTES_ROW
		do
			create row.make_template
			if cache.is_trusted_foundry_run then
				all_quotes := row.unexpired_fab_quotes_for_part_number (policies.trusted_foundry_fab_part, connection, expected_count.At_least_zero)
			else
				all_quotes := row.unexpired_fab_quotes_for_production (session.run_id, connection, expected_count.At_least_zero)
			end
			
			create quote_table.make (all_quotes.count)
			quote_table.compare_objects
			all_quotes.do_all (agent add_quote (?, quote_table))
		end

	add_quote (q: CUST_QUOTES_ROW table: like quote_table)
			-- Add `q' to `table'
		do
			table.put (q, q.full_cust_quote_id)
		end

	fetch_quotes
			-- Fetch list of allowed quotes, based on
			-- `required_account'
		local
			row: CUST_QUOTES_ROW
		do
			create row.make_template
			if required_account /= Void then
				quotes := row.unexpired_fab_quotes_for_account (required_account.cust_acct_no, connection, expected_count.At_least_zero)
			elseif cache.is_trusted_foundry_run then
				quotes := row.unexpired_fab_quotes_for_part_number (policies.trusted_foundry_fab_part, connection, expected_count.At_least_zero)
			else
				quotes := row.unexpired_fab_quotes_for_production (session.run_id, connection, expected_count.At_least_zero)
			end
		end

	add_menu_bar
			-- Add menu bar to window
		local
			file_menu: EV_MENU
			exit_menu_item: EV_MENU_ITEM
			close_menu_item: EV_MENU_ITEM
			a_menu_bar: EV_MENU_BAR
		do	
			create file_menu.make_with_text ("File")
			
			create close_menu_item.make_with_text ("Close")
			close_menu_item.select_actions.extend (agent process_cancel)
			file_menu.extend (close_menu_item)
			
			create exit_menu_item.make_with_text ("Exit")
			exit_menu_item.select_actions.extend (gui.destroy_application_agent)
			file_menu.extend (exit_menu_item)
			
			create a_menu_bar
			a_menu_bar.extend (file_menu)
			set_menu_bar (a_menu_bar)
		end

	init_field (f: EV_TEXT_FIELD action: PROCEDURE [ANY, TUPLE])
			-- Initialize field `f' with change action `action'
			-- if non-Void
		do
			f.disable_edit
			f.set_minimum_width_in_characters (Min_width)
			if action /= Void then
				-- f.return_actions.extend (action)
				-- f.focus_out_actions.extend (action)
				f.change_actions.extend (action)
			end
		end

	post_display_actions
		do
			reset
			gui.center_window_with_margin (window, 100, 100)
		end

	build_pulldown_lists
			-- Build pulldown lists for combo boxes
		do
			build_account_list
			build_quote_list
		end

	build_account_list
			-- Build list of accounts in `account_field'
		local
			acct: CUST_ACCTS_ROW
			list_item: EV_LIST_ITEM
		do
			account_field.select_actions.block
			create list_item.make_with_text ("No restriction")
			list_item.set_data (Void)
			account_field.extend (list_item)
			from
				all_accounts.start
			until
				all_accounts.after
			loop
				acct := all_accounts.item
				create list_item.make_with_text (acct.display_name)
				list_item.set_data (acct)
				account_field.extend (list_item)
				all_accounts.forth
			end
			account_field.select_actions.resume
		end

	build_quote_list
			-- Build list of quotes in `quote_id_field'
		local
			quote: CUST_QUOTES_ROW
			list_item: EV_LIST_ITEM
			saved_text: STRING
		do
			quote_id_field.change_actions.block
			saved_text := quote_id_field.text
			quote_id_field.wipe_out
			from
				quotes.start
			until
				quotes.after
			loop
				quote := quotes.item
				create list_item.make_with_text (quote.full_cust_quote_id)
				list_item.set_data (quote)
				quote_id_field.extend (list_item)
				quotes.forth
			end
			quote_id_field.set_text (saved_text)
			quote_id_field.change_actions.resume
		end

	process_account_selection
			-- Take actions needed when user selects an account
		local
			old_text, mod_text: STRING
			q: CUST_QUOTES_ROW
			clear_quote: BOOLEAN
		do
			required_account ?= account_field.selected_item.data
			old_text := quote_id_field.text
			fetch_quotes
			build_quote_list
			if required_account /= Void then
				mod_text := old_text.twin
				mod_text.left_adjust
				mod_text.right_adjust
				quote_table.search (mod_text)
				if quote_table.found then
					q := quote_table.found_item
					if q.cust_acct_no = required_account.cust_acct_no then
						clear_quote := False
					else
						clear_quote := True
					end
				else
					clear_quote := True
				end
				if clear_quote then
					selected_quote := Void
					build_quote_details
					set_quote_id ("")
				end
			end
		end

	process_order_selection
			-- Take actions needed when user selects an order
		do
			selected_order ?= order_field.selected_item.data
			is_new_order := selected_order = Void
			if is_new_order then
				po_field.enable_sensitive
				pricing_date_field.enable_sensitive
				pricing_date_field.set_text (connection.system_date.to_date_string)
			else
				pricing_date_field.remove_text
				po_field.disable_sensitive
				pricing_date_field.disable_sensitive
			end
		end

	process_po_selection
			-- Take actions needed when user selects a PO
		do
			selected_po ?= po_field.selected_item.data
		end

	process_pricing_date_change
			-- Take actions needed when date for pricing
			-- changes
		local
			s: STRING
		do
			s := pricing_date_field.text
			selected_po := Void
			if valid_date (s) then
				build_po_list
			else
				po_field.enable_edit
				po_field.wipe_out
				po_field.set_text ("")
				po_field.disable_edit
			end
		end

	process_quote_id_change
			-- Take actions needed when quote ID changes
		local
			text, new_text: STRING
			the_quote: like selected_quote
		do
			text := quote_id_field.text.twin
			text.left_adjust
			text.right_adjust
			text.to_upper
			quote_table.search (text)
			if quote_table.found then
				the_quote := quote_table.found_item
				account_table.search (the_quote.cust_acct_no)
				if account_table.found then
					required_account := account_table.found_item
					selected_quote := the_quote
					new_text := required_account.display_name
				else
					selected_quote := Void
					new_text := the_quote.account.display_name
					gui.do_once_on_idle (agent inform ("Quote ID " + text + " exists but its account (" + new_text + ") is not valid for this run"))
				end
			else
				selected_quote := Void
				new_text := "Not determined yet"
			end
			set_quote_id (text)
			set_account (new_text)
			build_quote_details
		end

	set_quote_id (s: STRING)
			-- Set quote ID text to `s'
		do
			quote_id_field.change_actions.block
			quote_id_field.set_text (s)
			quote_id_field.change_actions.resume
		end

	set_account (s: STRING)
			-- Set account text to `s'
		do
			account_field.select_actions.block
			account_field.enable_edit
			account_field.set_text (s)
			account_field.disable_edit
			account_field.select_actions.resume
		end

	build_quote_details
			-- Build details of quote (list of orders compatible
			-- with quote and list of quote line items)
		do
			build_order_list
			build_quote_line_items
		end

	build_order_list
			-- Build list of possible orders for `order_field',
			-- based on `selected_quote'
		local
			acct: CUST_ACCTS_ROW
			row: CUST_WOS_MROW
			all_wos, valid_wos: LIST [CUST_WOS_ROW]
			wo: CUST_WOS_ROW
			wo_text: STRING
			list_item: EV_LIST_ITEM
			sorter: LIST_BUBBLE_SORTER [CUST_WOS_ROW]
			comp: CUST_WO_AREA_COMPARATOR
			cid: CUST_IC_DESIGNS_ROW
			filter: FUNCTION [ANY, TUPLE [CUST_WOS_MROW], BOOLEAN]
		do
			order_field.wipe_out
			selected_order := Void
			if selected_quote = Void then
				order_field.disable_sensitive
			else
				acct := selected_quote.account
				if acct.is_commercial then
					order_field.enable_sensitive
					create row.make_template
					all_wos := row.work_orders_for_production (session.run_id, connection, expected_count.At_least_zero)
					valid_wos := all_wos.twin
					valid_wos.wipe_out
					if cache.is_dedicated_run then
						filter := agent {CUST_WOS_MROW}.is_on_account_and_quote (acct.cust_acct_no, selected_quote.m_cust_quote_no)
					else
						-- Trusted foundry run
						filter := agent {CUST_WOS_MROW}.is_non_ic_on_account_and_quote (acct.cust_acct_no, selected_quote.m_cust_quote_no)
					end
					all_wos.do_if (agent valid_wos.extend, filter)
					create sorter
					create comp
					valid_wos := sorter.sorted (valid_wos, comp)
					from
						valid_wos.start
					until
						valid_wos.after
					loop
						wo := valid_wos.item
						if not wo.is_null_m_fab_no then
							cid := wo.fab_instance.cust_ic_design
							wo_text := wo.fab_instance.die_id + "   " + cid.design_size_x.rounded.out + " x " + cid.design_size_y.rounded.out
						else
							wo_text := wo.cust_wo_id
						end
						create list_item.make_with_text (wo_text)
						list_item.set_data (wo)
						order_field.extend (list_item)
						valid_wos.forth
					end
					create list_item.make_with_text ("New non-IC order")
					list_item.set_data (Void)
					order_field.extend (list_item)
					
				else
					order_field.disable_sensitive
				end
			end
		end

	build_po_list
			-- Build list of possible POs for `po_field',
			-- based on `selected_quote' and `pricing_date_field'
		local
			po, row: FUNDING_TRIPLE_ROW
			all_pos: LIST [FUNDING_TRIPLE_ROW]
			list_item: EV_LIST_ITEM
			acct: CUST_ACCTS_ROW
		do
			po_field.enable_edit
			po_field.wipe_out
			acct := selected_quote.account
			create row.make_template
			all_pos := row.usable_rows_for_account (acct.cust_acct_no, pricing_date_field.text, connection, expected_count.At_least_zero)
			from
				all_pos.start
			until
				all_pos.after
			loop
				po := all_pos.item
				create list_item.make_with_text (po.auth_id)
				list_item.set_data (po)
				po_field.extend (list_item)
				all_pos.forth
			end
			po_field.disable_edit
		end

	build_quote_line_items
			-- Build quote line items in `quote_item_box'
			-- based on `selected_quote'
		local
			quote_items: LIST [CUST_QUOTE_LINE_ITEMS_ROW]
			q: CUST_QUOTE_LINE_ITEMS_ROW
			sorter: LIST_BUBBLE_SORTER [CUST_QUOTE_LINE_ITEMS_ROW]
			comp: CUST_QUOTE_LINE_ITEM_COMPARATOR
			cols: WIDGET_COLUMN_SET
			label: EV_LABEL
			descr_box: EV_HORIZONTAL_BOX
			box: EV_BOX
		do
			quote_item_box.wipe_out
			if selected_quote /= Void then
				quote_items := selected_quote.the_line_items
				create sorter
				create comp
				quote_items := sorter.sorted (quote_items, comp)
				create {ARRAYED_LIST [EV_TEXT_FIELD]} line_item_quantities.make (quote_items.count)
				
				create cols.make (7)
				cols.align_none
				cols.set_padding (user_interface.padding)
				cols.set_border_width (user_interface.border_width)
				cols.disable_expand
				
				label := gui.label_with_text ("Description")
				label.align_text_left
				create descr_box
				descr_box.extend (label)
				descr_box.disable_item_expand (label)
				descr_box.extend (create {EV_HORIZONTAL_BOX})
				cols.append_row (<< gui.label_with_text ("Line"), gui.label_with_text ("Quantity"), gui.label_with_text ("Part #"), gui.label_with_text ("Area"), gui.label_with_text ("Flat Amt"), gui.label_with_text ("Area Amt"), descr_box >>)
				
				from
					quote_items.start
				until
					quote_items.after
				loop
					q := quote_items.item
					if q.m_part_no /= Excluded_part_no then
						add_quote_line_item (q, cols)
					end
					quote_items.forth
				end
				cols.align_columns_right
				cols.enable_column_expand (7)
				box := cols.box
				quote_item_box.extend (box)
				quote_item_box.extend (create {EV_VERTICAL_BOX})
			end
			refresh_quote_item_area
		end

	add_quote_line_item (q: CUST_QUOTE_LINE_ITEMS_ROW cols: WIDGET_COLUMN_SET)
			-- Add row for `q' to `cols'
		local
			descr: STRING
			text_area: EV_TEXT
			flat_amt, area_amt, assumed_area: STRING
		do
			descr := q.descr.twin
			descr.prune_all ('%R')

			create text_area.make_with_text (descr)
			text_area.set_minimum_height (Quote_line_item_height)
			text_area.disable_word_wrapping
			text_area.disable_edit
			
			if not q.is_null_assumed_area then
				assumed_area := nu.std_formatted_number (q.assumed_area)
			else
				assumed_area := ""
			end
			
			if not q.is_null_flat_amount then
				flat_amt := nu.money_formatted_number (q.flat_amount)
			else
				flat_amt := ""
			end
			
			if not q.is_null_area_amount then
				area_amt := nu.money_formatted_number (q.area_amount)
			else
				area_amt := ""
			end
			
			cols.append_row (<< field_with_text (q.cust_quote_line_no.out), editable_quantity_field (q), field_with_text (q.m_part_no.out), field_with_text (assumed_area), field_with_text (flat_amt), field_with_text (area_amt), text_area >>)
		end

	field_with_text (s: STRING): EV_VERTICAL_BOX
			-- New non-editable text field with text `s'
		local
			label: EV_LABEL
			box: EV_HORIZONTAL_BOX
		do
			create label.make_with_text (s)
			label.align_text_right
			create box
			box.extend (label)
			create Result
			Result.extend (box)
			Result.disable_item_expand (box)
			Result.set_minimum_height (Quote_line_item_height)
		end

	editable_quantity_field (d: ANY): EV_VERTICAL_BOX
			-- New editable text field for quantity, with
			-- associated data `d'.  Field is also added
			-- to `line_item_quantities'
		local
			field: EV_TEXT_FIELD
		do
			create field
			field.set_minimum_width (Quote_line_item_field_width)
			field.set_data (d)
			line_item_quantities.extend (field)
			create Result
			Result.extend (field)
			Result.disable_item_expand (field)
			Result.set_minimum_height (Quote_line_item_height)
		end

	process_quote_items_resize (x, y, w, h: INTEGER)
			-- Take actions needed when area for quote items
			-- is resized
		do
			refresh_quote_item_area
		end

	refresh_quote_item_area
			-- Take actions needed when area for quote items
			-- needs to be refreshed
		do
			quote_item_area.set_item_size (quote_item_area.client_width.max (quote_item_area.item.minimum_width), quote_item_area.client_height.max (quote_item_area.item.minimum_height))
			quote_item_area.set_offset (0, 0)
		end

	process_ok
			-- Take actions needed when user clicks OK button
		do
			if is_active then
				disable_active
				create operation.make (agent process_ok_request)
				operation.enable_abort_caught
				operation.abort_actions.extend (agent handle_ok_abort)
				operation.execute
				enable_active
			end
		end

	process_ok_request
			-- Process OK request
		do
			validate_fields
			get_confirmation
			connection.discard_package_state
				-- Avoid "existing state of packages has
				-- been discarded" Oracle error
			if is_new_order then
				build_work_order
			end
			build_wo_line_items
			connection.post
			charge_wo_line_items
			selected_order.resequence_line_items
			session.commit
			inform (quote_line_item_count.out + " work order line " + su.number (quote_line_item_count, "item", "items") + " built and charged")
		end

	validate_fields
			-- Validate all needed fields before trying to
			-- create and charge line items
		local
			quote_id: STRING
			q: CUST_QUOTES_ROW
		do
			if selected_quote = Void then
				quote_id := quote_id_field.text
				create q.make_by_full_cust_quote_id (connection, option.Old_no_exception_row, quote_id)
				if q.is_empty then
					handle_user_error ("Quote ID " + su.double_quoted (quote_id) + " not found")
				else
					handle_user_error ("Quote ID " + su.double_quoted (quote_id) + " exists but its account (" + q.account.display_name + ") is not valid for this run")
				end
			end
			if order_field.is_empty then
				handle_user_error ("No order selected")
			end
			if is_new_order then
				if not valid_date (pricing_date_field.text) then
					handle_user_error ("date for pricing " + su.double_quoted (pricing_date_field.text) + " is not a valid date")
				end
				if selected_po = Void then
					handle_user_error ("no PO selected")
				end
			end
			quote_line_item_count := 0
			line_item_quantities.do_all (agent validate_quantity)
			if quote_line_item_count = 0 then
				handle_user_error ("No quote line items selected (none have quantity field with positive value)")
			end
		end

	validate_quantity (f: EV_TEXT_FIELD)
			-- Validate value in `f' (must be nonnegative integer
			-- or empty)
		local
			s: STRING
			q: CUST_QUOTE_LINE_ITEMS_ROW
			n: INTEGER
		do
			s := f.text
			s.left_adjust
			s.right_adjust
			if not s.is_empty then
				q ?= f.data
				check
					quote_line_item_not_void: q /= Void
				end
				if s.is_integer then
					n := s.to_integer
					if n < 0 then
						handle_user_error ("Quantity " + su.double_quoted (s) + " for quote line item " + q.cust_quote_line_no.out + " is negative")
					elseif n > 0 then
						quote_line_item_count := quote_line_item_count + 1
					end
				else
					handle_user_error ("Quantity " + su.double_quoted (s) + " for quote line item " + q.cust_quote_line_no.out + " is not an integer")
				end
			end
		end

	get_confirmation
			-- Issue warnings about things that might be
			-- errors and confirm that user wants to proceed
		local
			wos: LIST [CUST_WOS_ROW]
			part_type: PART_TYPE_VALUES
		do
			if cache.productions_row.production_charges_locked then
				handle_user_error ("Cannot create and charge line items because production charges are locked")
			end
			if not selected_quote.is_current_revision then
				warn ("selected quote " + selected_quote.full_cust_quote_id + " has a later revision (" + selected_quote.latest_revision + ")")
			end
			if is_new_order and then selected_po.charges_taxable then
				warn ("sales tax must be set manually in the new work order after this step completes")
			end
			wos := fab_run_orders
			if wos.count = 1 then
				warn ("there is already a work order with a " + part_type.Fab_run + " line item.%NOrder may have already been created.%NDo not proceed unless this is one of those rare cases that requires two orders.%N")
			elseif wos.count > 1 then
				handle_user_error ("there are already " + wos.count.out + " work orders with a " + part_type.Fab_run + " line item.%NCannot create and charge any more line items.%NUse Omega to adjust orders if necessary.")
			end
			confirm ("Work order line items will be built and charged for " + quote_line_item_count.out + " quote line items.%NThey will be put on " + order_description + ".%N%NProceed")
		end

	fab_run_orders: LIST [CUST_WOS_ROW]
			-- Work orders for this production that have
			-- at least one line item whose part is of
			-- type FAB RUN
		local
			wo: CUST_WOS_ROW
		do
			create wo.make_template
			Result := wo.fab_run_orders_for_production (session.run_id, connection, expected_count.At_least_zero)
		end

	build_work_order
			-- Build new non-IC work order
		local
			wo: CUST_WOS_MROW
			seq: SEQUENCE_NAMES
			d: DB_DATE
		do
			create wo.make_null (connection)
			wo.set_m_order_no (connection.sequence_next_value (seq.m_order_no))
			wo.set_cust_acct_no (selected_quote.cust_acct_no)
			wo.set_m_funding_alloc_no (selected_po.m_funding_alloc_no)
			wo.set_pricing_schedule (selected_po.pricing_schedule)
			d := connection.date (pricing_date_field.text)
			wo.set_date_for_pricing (d)
			wo.set_m_cust_quote_no (selected_quote.m_cust_quote_no)
			wo.set_created_by ("RC")
			wo.set_cust_wo_id (session.run_id)
			wo.set_date_cust_wo (connection.transaction_date)
			wo.set_production_id (session.run_id)
			selected_order := wo
		end

	build_wo_line_items
			-- Build one line item on the selected order
			-- for each quote line item whose quantity field
			-- is not empty or 0.  Leave line items in
			-- `new_line_items'
		local
			wo: CUST_WOS_MROW
		do
			wo := selected_order
			if wo.is_null_m_cust_quote_no then
				wo.set_m_cust_quote_no (selected_quote.m_cust_quote_no)
			end
			wo.uncache_line_items
			next_wo_line_no := wo.next_wo_line_no
			create {ARRAYED_LIST [CUST_WO_LINE_ITEMS_ROW]} new_line_items.make (line_item_quantities.count)
			line_item_quantities.do_all (agent build_wo_line_item (?, wo))
		end

	build_wo_line_item (f: EV_TEXT_FIELD wo: CUST_WOS_ROW)
			-- Build one line item on `wo'
			-- for `f' if its value is not empty or 0
		local
			val: STRING
			n: INTEGER
			q: CUST_QUOTE_LINE_ITEMS_ROW
			li: CUST_WO_LINE_ITEMS_MROW
			seq: SEQUENCE_NAMES
		do
			val := f.text
			if not val.is_empty then
				n := val.to_integer
				if n > 0 then
					q ?= f.data
					check
						quote_line_item_not_void: q /= Void
					end
					create li.make_null (connection)
					li.set_m_job_no (connection.sequence_next_value (seq.M_job_no_seq))
					li.set_m_order_no (wo.m_order_no)
					li.set_cust_wo_line_no (next_wo_line_no)
					li.set_m_part_no (q.m_part_no)
					li.set_part_quantity_ordered (n)
					next_wo_line_no := next_wo_line_no + 1
					new_line_items.extend (li)
				end
			end
		end

	charge_wo_line_items
			-- Charge each line item in `new_line_items'.
			-- Uses database package, so you must post before
			-- calling this routine
		do
			create {ARRAYED_LIST [STRING]} charge_descriptions.make (new_line_items.count)
			new_line_items.do_all (agent charge_wo_line_item)
		end

	charge_wo_line_item (li: CUST_WO_LINE_ITEMS_ROW)
			-- Charge `li'
		local
			b: SQL_BLOCK
			err, s, unit: STRING
			amt, tax, total: DOUBLE
			part_unit: PART_UNIT_VALUES
		do
			b := connection.sql_block (charge_block)
			b.set_named_bind_capacity (2)
			b.bind_name_to_integer (Ph_job_no, li.m_job_no, False)
			create err.make_filled (' ', 4000)
			b.bind_name_to_string (Ph_error, err, False)
			b.bind_name_to_double (Ph_total_without_tax, amt, False)
			b.bind_name_to_double (Ph_tax, tax, False)
			b.execute
			if not b.is_bound_null_name (Ph_error) then
				err := b.string_bound_to_name (Ph_error)
			else
				err := Void
			end
			amt := b.double_bound_to_name (Ph_total_without_tax)
			tax := b.double_bound_to_name (Ph_tax)
			b.release
			total := amt + tax
			unit := li.part.part_unit.twin
			if unit.is_equal (Part_unit.Each) then
				unit.to_lower
			elseif unit.is_equal (Part_unit.Lot) then
				unit.to_lower
				unit := su.number (li.part_quantity_ordered, unit, unit + "s")
			else
			end
			
			s := "Charged " + nu.money_formatted_number (total) + " (" + nu.money_formatted_number (amt) + " plus " + nu.money_formatted_number (tax) + " tax) for " + li.part_quantity_ordered.out + " " + unit + "%N" + "    " + li.part.descr
			charge_descriptions.extend (s)
			if err /= Void then
				handle_hard_error ("error charging last line item in the following list.%N" + err + "%N%N" + su.joined_by (charge_descriptions, "%N"))
			end
		end

	order_description: STRING
			-- Description of selected order
		local
			wo: CUST_WOS_ROW
		do
			wo := selected_order
			if wo /= Void then
				if wo.is_null_m_fab_no then
					Result := "the non-IC order " + wo.cust_wo_id
				else
					Result := "the IC order for die " + wo.fab_instance.die_id
				end
			else
				Result := "a new non-IC order"
			end
		end

	handle_ok_abort
			-- Handle abort of OK request
		do
			session.rollback
		end

	process_finish
			-- Process Finish request to try to finish step
		do
			if is_active then
				disable_active
				if fab_run_orders.is_empty then
					inform ("Cannot finish step because there are no orders with a FAB RUN line item.")
				else
					success_actions.call ([])
				end
				enable_active
			end
		end

	process_cancel
			-- Process cancel request
		do
			if is_active then
				disable_active
				abort_actions.call ([])
			end
		end

	cust_wos: LIST [CUST_WOS_MROW]
			-- Work orders for projects on run that
			-- need to have an `internal_request_type' request
			-- processed at least once

	operation: ANONYMOUS_OPERATION
			-- Current operation in progress, if any

	account_field: EV_COMBO_BOX
			-- Field for customer account

	quote_id_field: EV_COMBO_BOX
			-- Field for quote ID

	order_field: EV_COMBO_BOX
			-- Field for cust work order

	po_field: EV_COMBO_BOX
			-- Field for purchase order, if creating a new
			-- customer work order

	pricing_date_field: EV_TEXT_FIELD
			-- Field for date for pricing, if purchase order
			-- needed

	amount_field: EV_TEXT_FIELD
			-- Field for total amount of order

	quote_item_area: EV_SCROLLABLE_AREA
			-- Scrollable area for quote line items

	quote_item_box: EV_VERTICAL_BOX
			-- Box with individual quote line items

	Min_width: INTEGER = 20
			-- Minimum width of fields in characters

	all_accounts: LIST [CUST_ACCTS_ROW]
			-- List of all allowed accounts on which quote may
			-- be issued

	required_account: CUST_ACCTS_ROW
			-- Account quote is required to be on
			-- (no restriction if Void)

	account_table: HASH_TABLE [CUST_ACCTS_ROW, INTEGER]
			-- All allowed accounts, indexed by full cust account
			-- number

	all_quotes: LIST [CUST_QUOTES_ROW]
			-- List of all possible quotes

	quotes: LIST [CUST_QUOTES_ROW]
			-- List of allowed quotes (restricted to
			-- `required_account' if non-Void)

	selected_quote: CUST_QUOTES_ROW
			-- Quote user selected

	selected_order: CUST_WOS_MROW
			-- Work order user selected

	selected_po: FUNDING_TRIPLE_ROW
			-- Purchase order user selected

	is_new_order: BOOLEAN
			-- Did user ask to create a new non-IC work order?

	line_item_quantities: LIST [EV_TEXT_FIELD]
			-- Text fields with quantity for each line item

	new_line_items: LIST [CUST_WO_LINE_ITEMS_ROW]
			-- New line items to be charged

	charge_descriptions: LIST [STRING]
			-- Description of each line item charged so far

	quote_line_item_count: INTEGER
			-- Number of quote line items that have non-empty
			-- positive integer values in their quantity field

	next_wo_line_no: INTEGER
			-- Next line number to use when creating cust wo
			-- line items

	quote_table: HASH_TABLE [CUST_QUOTES_ROW, STRING]
			-- All possible quotes, indexed by full cust quote ID

	Excluded_part_no: INTEGER
			-- Exclude quote line items with this part number
		do
			Result := policies.cust_quote_t_and_c_part
				-- Exclude Terms and Conditions pseudo-part
		end

	Quote_line_item_height: INTEGER = 100
			-- Height of box for each quote line item

	Quote_line_item_field_width: INTEGER = 80
			-- Width of box for each quote line item field

	charge_block: STRING = "[
begin
   cust_wo_charge_utilities.build_job_charges (:job_no);
   :error := cust_wo_charge_utilities.last_error_message;
   :total_without_tax := cust_wo_charge_utilities.last_amount_total_no_tax;
   :tax := cust_wo_charge_utilities.last_amount_tax;
exception
   when others then
   :error := 'Unhandled exception:' || chr(10) || sqlerrm;
end;
]"
			-- PL/SQL block to charge a line item
	
	Ph_job_no: STRING = "job_no"
			-- Name of placeholder for m_job_no
   	
	Ph_error: STRING = "error"
			-- Name of placeholder for error indicator
   	
	Ph_total_without_tax: STRING = "total_without_tax"
			-- Name of placeholder for total amount charged
			-- for last line item, without tax
   	
	Ph_tax: STRING = "tax"
			-- Name of placeholder for tax charged
			-- for last line item
   	
end
