
indexing
	
	description: "Window for reporting fatal error"
	author: "David Hollenberg"
	date: "November 11, 2003"

class FATAL_ERROR_WINDOW

inherit
	FATAL_ERROR_IFC
	WINDOW_INTERFACE
	MOSIS_GRAPHICAL_UTILITIES
		export
			{NONE} all
		undefine
			default_create
		end
		
create
	make

feature {NONE} -- Creation

	make (w: EV_TITLED_WINDOW a_text: STRING)
			-- Create interface in window `w' to report
			-- fatal error described by `a_text'
		require
			window_exists: w /= Void
			window_valid: not w.is_destroyed
			a_text_exists: a_text /= Void
		do
			make_interface (a_text)
			make_proxy (w)
		end

feature -- Operations

	reset
			-- Reset window
		do
			enable_active
			-- notify_button.center_pointer
		end

feature {NONE} -- Implementation

	build_interface
			-- Build interface in `window'
		local
			main_vbox: EV_VERTICAL_BOX
			box: EV_BOX
			buttons: HORIZONTAL_BUTTON_SET
			font: EV_FONT
			size: TUPLE [INTEGER, INTEGER]
			w, h: INTEGER
			extra, sb_width, sb_height: INTEGER
			screen: EV_SCREEN
		do
			create main_vbox
			main_vbox.set_padding (user_interface.padding)
			main_vbox.set_border_width (user_interface.border_width)
			create font.make_with_values (gui.Family_screen,
				gui.Weight_regular, gui.Shape_regular, 14)
			font.preferred_families.extend ("Courier")
			size := font.string_size (text)
			w := size.integer_item (1)
			h := size.integer_item (2)
			create screen
			w := w.min (screen.width - 200)
			h := h.min (screen.height - 200)
			extra := 10
			sb_width := (create {EV_VERTICAL_SCROLL_BAR}).width
			sb_height := (create {EV_HORIZONTAL_SCROLL_BAR}).height
			w := w + extra + sb_width
			h := h + extra + sb_height
			
			create text_area.make_with_text (text)
			text_area.set_font (font)
			text_area.disable_word_wrapping
			text_area.disable_edit
			text_area.set_minimum_width (w)
			text_area.set_minimum_height (h)
			
			main_vbox.extend (text_area)
			
			create buttons.make_with_text_and_actions (<< "Notify", "Exit" >>, << agent process_notify, agent process_exit >>)
			notify_button := buttons.i_th (1)
			box := buttons.column_set.box
			
			main_vbox.extend (box)
			main_vbox.disable_item_expand (box)
			

			gui.set_box_background_recursive (main_vbox, background_color)
			extend (main_vbox)
			gui.add_cancel_accelerator (window, agent process_exit)
			gui.do_once_on_idle (agent post_display_actions)
			show
		end
		
	post_display_actions
		do
			reset
		end

	process_notify
		do
			if is_active then
				disable_active
				notify_actions.call ([])
			end
		end

	process_exit
		do
			if is_active then
				disable_active
				exit_actions.call ([])
			end
		end

	text_area: EV_TEXT
			-- Area for error message
	
	notify_button: EV_BUTTON
	
end
