
indexing
	
	author: "David Hollenberg"
	date: "May 3, 2002"
	description: "A file descriptor which is to be set from another %
		%file descriptor in spawned process"

class FD_DESCRIPTOR

inherit
	PROCESS_FD
		redefine
			is_descriptor, descriptor
		end

create
	make

feature {NONE} -- Creation

	make (fd, other_fd: INTEGER)
			-- Create `Current' for file descriptor `fd',
			-- which is to be set from `other_fd' in child process
		require
			valid_descriptor: valid_file_descriptor (fd)
			valid_other_descriptor: valid_file_descriptor (other_fd)
		do
			number := fd
			descriptor := other_fd
		ensure
			number_set: number = fd
		end

feature -- Properties

	is_descriptor: BOOLEAN = True
			-- Is file descriptor setup from another file
			-- descriptor?

	descriptor: INTEGER
			-- Descriptor from which `Current' is to be set
			-- in child process

feature -- Operations

	prepare_for_fork
			-- Do any preparation needed prior to a fork
			-- to setup this file descriptor in spawned
			-- process
		do
			source_descriptor := descriptor
		end

	setup_for_parent
			-- Take any actions needed to setup this file
			-- descriptor in parent process after a fork
		do
		end

	setup_for_child (fd_set: PROCESS_FD_SET)
			-- Take any actions needed to setup this file
			-- descriptor in child process after a fork,
			-- where `fd_set' is the entire set of file
			-- descriptors to be setup
		do
			if source_descriptor /= number then
				unix_os.duplicate_file_descriptor (source_descriptor, number)
			end
			close_source_descriptor_if_ok (fd_set)
		end

end


