
indexing
	
	author: "David Hollenberg"
	date: "May 3, 2002"
	description: "A file descriptor which is to be duplicated from %
		%another file descriptor in spawned process, after that %
		%other descriptor has been setup"

class FD_DUPLICATE

inherit
	PROCESS_FD
		redefine
			is_duplicate, dup_descriptor, setup_for_child_late
		end

create
	make

feature {NONE} -- Creation

	make (fd, dup_fd: INTEGER)
			-- Create `Current' for file descriptor `fd',
			-- which is to be duplicated from `dup_fd'
			-- in spawned process
		require
			valid_descriptor: valid_file_descriptor (fd)
			valid_descriptor: valid_file_descriptor (dup_fd)
		do
			number := fd
			dup_descriptor := dup_fd
		ensure
			number_set: number = fd
			dup_desc_set: dup_descriptor = dup_fd
		end

feature -- Properties

	is_duplicate: BOOLEAN = True
			-- Is file descriptor duplicated from another file
			-- descriptor?

	dup_descriptor: INTEGER
			-- Descriptor from which `Current' is to be duplicated
			-- in child process

feature -- Operations

	prepare_for_fork
			-- Do any preparation needed prior to a fork
			-- to setup this file descriptor in spawned
			-- process
		do
			source_descriptor := dup_descriptor
		end

	setup_for_parent
			-- Take any actions needed to setup this file
			-- descriptor in parent process after a fork
		do
		end

	setup_for_child (fd_set: PROCESS_FD_SET)
			-- Take any actions needed to setup this file
			-- descriptor in child process after a fork,
			-- where `fd_set' is the entire set of file
			-- descriptors to be setup
		do
		end

	setup_for_child_late
			-- Take any late actions needed to setup this file
			-- descriptor in child process after a fork
		do
			unix_os.duplicate_file_descriptor (source_descriptor, number)
		end

end
