
indexing
	
	description: "A reticle field of view and the horizontal and %
		%vertical edges of objects that intersect that field"
	author: "David Hollenberg"
	date: "January 17, 2007"

class FIELD_ELEMENT_EDGES
inherit
	GRIDABLE
create
	make

feature {NONE} -- Creation

	make (a_field: like field n: INTEGER a_grid: like grid)
			-- Create with space for `n' horizontal and `n'
			-- vertical edges
		do
			field := a_field
			grid := a_grid
			left_limit := grid.rounded_edge (field.left_x)
			right_limit := grid.rounded_edge (field.right_x)
			bottom_limit := grid.rounded_edge (field.bottom_y)
			top_limit := grid.rounded_edge (field.top_y)
			create {ARRAYED_LIST [MODEL_ELEMENT_EDGE]} horizontal_edges.make (n)
			create {ARRAYED_LIST [MODEL_ELEMENT_EDGE]} vertical_edges.make (n)
		end

feature -- Properties

	field: RETICLE_FIELD
			-- Field of view

	left_limit: DOUBLE
			-- Gridded coordinate of left side of field

	right_limit: DOUBLE
			-- Gridded coordinate of right side of field

	bottom_limit: DOUBLE
			-- Gridded coordinate of bottom side of field

	top_limit: DOUBLE
			-- Gridded coordinate of top side of field

	horizontal_edges: LIST [MODEL_ELEMENT_EDGE]
			-- Horizontal edges of objects that intersect field

	vertical_edges: LIST [MODEL_ELEMENT_EDGE]
			-- Vertical edges of objects that intersect field

feature -- Operations

	sort_edges
			-- Sort `horizontal_edges' and `vertical_edges'
		local
			sorter: LIST_QUICK_SORTER [MODEL_ELEMENT_EDGE]
			comparator: POSITION_COMPARATOR [MODEL_ELEMENT_EDGE]
		do
			create sorter
			create comparator
			horizontal_edges := sorter.sorted (horizontal_edges, comparator)
			vertical_edges := sorter.sorted (vertical_edges, comparator)
		end

end
