
indexing
	
	description: "Window to allow user to set (and possibly create) field set %
		%for a run"
	author: "David Hollenberg"
	date: "August 26, 2008"

class FIELD_SET_WINDOW

inherit
	FIELD_SET_IFC
	WINDOW_INTERFACE
	MOSIS_GRAPHICAL_UTILITIES
		export
			{NONE} all
		undefine
			default_create
		end
	STRING_UTILITIES_ACCESS
		export
			{NONE} all
		undefine
			default_create
		end
		
create
	make

feature {NONE} -- Creation

	make (w: EV_TITLED_WINDOW)
			-- Create interface in window `w'
		require
			window_exists: w /= Void
			window_valid: not w.is_destroyed
		do
			make_interface
			make_proxy (w)
		end

feature -- Operations

	reset
			-- Reset interface
		do
		end

feature {NONE} -- Implementation

	build_interface
			-- Build interface in `window'
		local
			buttons: HORIZONTAL_BUTTON_SET
			cols: WIDGET_COLUMN_SET
			box: EV_HORIZONTAL_BOX
			frame: EV_FRAME
		do
			fetch_data
			create main_vbox
			main_vbox.set_border_width (user_interface.border_width)
			
			build_packing_target_field

			build_unavailable_targets_label
			if unavailable_targets_label /= Void then
				create frame
				frame.put (unavailable_targets_label)
				create box
				box.set_border_width (user_interface.border_width)
				box.extend (frame)
				main_vbox.extend (box)
				main_vbox.disable_item_expand (box)
			end
			
			build_usage_label
			
			create box
			box.set_border_width (user_interface.border_width)
			box.extend (usage_label)
			main_vbox.extend (box)
			main_vbox.disable_item_expand (box)
			
			create cols.make (2)
			cols.set_padding (user_interface.padding)
			cols.set_border_width (user_interface.border_width)
			cols.append_row (<< gui.label_with_text ("Packing Target"), packing_target_field >>)
			cols.disable_column_expand (1)
			cols.align_column_right (1)
			box := cols.box
			
			main_vbox.extend (box)
			main_vbox.disable_item_expand (box)
			
			create packing_target_specific_box
			main_vbox.extend (packing_target_specific_box)
			main_vbox.disable_item_expand (packing_target_specific_box)
			
			create buttons.make_with_text_and_actions (<< gui.ev_ok, gui.ev_cancel >>, << agent process_accept, agent process_decline >>)
			box := buttons.column_set.box
			
			main_vbox.extend (box)
			main_vbox.disable_item_expand (box)
			gui.set_box_background_recursive (main_vbox, background_color)
			extend (main_vbox)

			close_request_actions.extend (agent process_decline)
			gui.add_cancel_accelerator (window, agent process_decline)
			
			-- Initialize fields for currently selected packing_target
			gui.do_once_on_idle (agent process_new_packing_target)
			gui.do_once_on_idle (agent gui.center_window (window))
			
			enable_active
		end
		
	fetch_data
			-- Fetch needed data from database
		local
			pt_row: PACKING_TARGETS_ROW
		do
			create {ARRAYED_LIST [FIELD_SETS_ROW]} related_field_sets.make (5)
				-- Create first so we can pass reference
				-- to each created packing target 
			create pt_row.make_template
			packing_target_rows := pt_row.all_rows_ordered (connection, "packing_target")
			create_packing_targets_table
			fetch_wafer_techn_packing_targets
			fetch_field_sets
			fetch_default_field_set
		end

	create_packing_targets_table
			-- Create `packing_targets'
		local
			pt_row: PACKING_TARGETS_ROW
			pt_name: STRING
		do
			create packing_targets.make (5)
			from
				packing_target_rows.start
			until
				packing_target_rows.after
			loop
				pt_row := packing_target_rows.item
				pt_name := pt_row.packing_target
				packing_targets.put (packing_target_from_row (pt_row), pt_name)
				packing_target_rows.forth
			end
		end

	packing_target_from_row (pt_row: PACKING_TARGETS_ROW): PACKING_TARGET
			-- Packing target for `pt_row'
		local
			target: PACKING_TARGET_VALUES
			pt_name: STRING
		do
			pt_name := pt_row.packing_target
			if pt_name.is_equal (target.Standard_reticle) then
				create {STANDARD_RETICLE_NORMAL_PACKING_TARGET} Result.make (pt_row, packing_targets, related_field_sets, session)
			elseif pt_name.is_equal (target.Standard_reticle_die) then
				create {STANDARD_RETICLE_DIE_PACKING_TARGET} Result.make (pt_row, packing_targets, related_field_sets, session)
			elseif pt_name.is_equal (target.Custom_reticle) then
				create {CUSTOM_RETICLE_NORMAL_PACKING_TARGET} Result.make (pt_row, packing_targets, related_field_sets, session)
			elseif pt_name.is_equal (target.Custom_reticle_die) then
				create {CUSTOM_RETICLE_DIE_PACKING_TARGET} Result.make (pt_row, packing_targets, related_field_sets, session)
			elseif pt_name.is_equal (target.Custom_tile) then
				create {CUSTOM_TILE_PACKING_TARGET} Result.make (pt_row, packing_targets, related_field_sets, session)
			elseif pt_name.is_equal (target.Unpacked_die) then
				create {UNPACKED_DIE_PACKING_TARGET} Result.make (pt_row, packing_targets, related_field_sets, session)
			else
				handle_hard_error ("unknown packing target " + pt_name)
			end
		end

	fetch_wafer_techn_packing_targets
			-- Fetch packing targets for run's wafer techn
			-- and put each row in the corresponding
			-- packing target object
		local
			wtpt_row: WAFER_TECHN_PACKING_TARGETS_ROW
			list: LIST [WAFER_TECHN_PACKING_TARGETS_ROW]
			pt_name: STRING
			has_avail_pt: BOOLEAN
		do
			create wtpt_row.make_template
			wafer_techn_packing_targets := wtpt_row.targets_for_wafer_techn (cache.wafer_techn, connection, expected_count.At_least_zero)
			list := wafer_techn_packing_targets
			from
				list.start
			until
				list.after
			loop
				wtpt_row := list.item
				has_avail_pt := has_avail_pt or wtpt_row.available
				pt_name := wtpt_row.packing_target
				packing_targets.search (pt_name)
				if packing_targets.found then
					packing_targets.found_item.set_wafer_techn_packing_target (wtpt_row)
				else
					handle_internal_error ("no table entry for packing target " + pt_name)
				end
				list.forth
			end
			if not has_avail_pt then
				handle_hard_error ("no available packing targets for wafer techn " + cache.wafer_techn)
			end
		end

	fetch_field_sets
			-- Fetch possible field sets for each packing target
		do
			field_sets := all_possible_field_sets
			field_sets.do_all (agent add_field_set)
			related_field_sets.append (all_related_field_sets)
		end

	add_field_set (f: FIELD_SETS_MROW)
			-- Add `f' to field set list for
			-- packing target of its litho size
		local
			pt_name: STRING
		do
			pt_name := f.packing_target
			packing_targets.search (pt_name)
			if packing_targets.found then
				packing_targets.found_item.add_field_set (f)
			else
				handle_internal_error ("no table entry for packing target " + pt_name)
			end
		end

	fetch_default_field_set
			-- Fetch default field set, if have saved state, and
			-- put in `default_field_set'
		do
			if state /= Void and then field_sets.there_exists (agent matches_saved_value) then
				create default_field_set.make_by_key (connection, option.Old_row, state)
			else
				default_field_set := Void
			end
		end
		
	matches_saved_value (r: FIELD_SETS_ROW): BOOLEAN
			-- Does `r' have same field set name as one saved in
			-- `state'?
		require
			row_exists: r /= Void
		do
			Result := equal (r.field_set_name, state)
		end

	all_possible_field_sets: DYNAMIC_LIST [FIELD_SETS_MROW]
			-- All possible field sets for this run
		local
			template_row: FIELD_SETS_MROW
		do
			create template_row.make_template
			Result := template_row.available_field_sets_ordered (session.run_id, cache.wafer_techn, connection, expected_count.at_least_zero)
		end

	all_related_field_sets: DYNAMIC_LIST [FIELD_SETS_ROW]
			-- All field sets ever associated with this
			-- run or its wafer techn
		local
			template_row: FIELD_SETS_ROW
		do
			create template_row.make_template
			Result := template_row.field_sets_ordered (session.run_id, cache.wafer_techn, connection, expected_count.at_least_zero)
		end

	is_target_name_allowed (pt_name: STRING): BOOLEAN
			-- Is packing target `pt_name' allowed to be
			-- selected for this run?
		do
			Result := packing_targets.item (pt_name).is_allowed
		end

	is_target_name_available (pt_name: STRING): BOOLEAN
			-- Is packing target `pt_name' available for this
			-- run's wafer techn?  Ignores whether there is enough
			-- data to build interface
		do
			Result := packing_targets.item (pt_name).is_available
		end

	build_usage_label
			-- Build `usage_label'
		do
			usage_label := gui.label_with_text (usage_label_text)
			usage_label.align_text_left
		end

	usage_label_text: STRING
		local
			target: PACKING_TARGET_VALUES
		do
			Result := "Select a packing target.  If you select " + target.Standard_reticle + 
				" or " + target.Standard_reticle_die + " or " + target.Unpacked_die +
				",%Nthen just select the desired field set.  If you select a CUSTOM target, then there is only one possible%N" +
				"field set and you must fill in the rest of the fields."
		end

	build_unavailable_targets_label
			-- Build `unavailable_targets_label'
		local
			l_text: STRING
		do
			l_text := unavailable_targets_text
			if l_text /= Void then
				unavailable_targets_label := gui.label_with_text (l_text)
				unavailable_targets_label.align_text_left
				unavailable_targets_label.set_background_color (gui.White)
				unavailable_targets_label.set_foreground_color (gui.Red)
			end
		end

	unavailable_targets_text: STRING
			-- Text noting which packing targets are not available.
			-- A target can be unavailable either because it is
			-- not marked available in wafer_techn_packing_targets or
			-- because we do not have enough data to build the
			-- interface (or both)
		local
			pt: PACKING_TARGET
			unavailable, not_enough_data, missing_field_set: BOOLEAN
		do
			if not disallowed_packing_targets.is_empty then
				Result := "%N"
				Result.append ("    Unavailable packing targets (manual intervention required to make them available):%N%N")
				from
					disallowed_packing_targets.start
				until
					disallowed_packing_targets.after
				loop
					pt := disallowed_packing_targets.item
					unavailable := not pt.is_available
					not_enough_data := not pt.has_enough_data_to_build_interface
					missing_field_set := not pt.is_custom and not pt.has_existing_field_set
					Result.append ("        " + pt.target_name)
					if unavailable or not_enough_data then
						Result.append ("   (")
						if unavailable then
							Result.append ("Not available")
						end
						if not_enough_data then
							if unavailable then
								Result.append (", ")
							end
							if missing_field_set then
								Result.append ("Missing field set")
							else
								Result.append ("Not enough data to build interface")
							end
						end
						Result.extend (')')
					end
					Result.extend ('%N')
					disallowed_packing_targets.forth
				end
			end
		end

	build_packing_target_field
			-- Build field for selecting packing target.
			-- Also, set `disallowed_packing_targets'
		local
			fact: SINGLE_CONSTRAINED_VALUE_FACTORY
			pt_row: PACKING_TARGETS_ROW
			target: PACKING_TARGET_VALUES
			pt: PACKING_TARGET
			pt_name, default_choice: STRING
			avail_targets: LIST [STRING]
			has_choice: BOOLEAN
		do
			create {ARRAYED_LIST [PACKING_TARGET]} disallowed_packing_targets.make (5)
			create {ARRAYED_LIST [STRING]} avail_targets.make (5)
			create fact
			fact.enable_choices_prepended
			from
				packing_target_rows.start
			until
				packing_target_rows.after
			loop
				pt_row := packing_target_rows.item
				pt_name := pt_row.packing_target
				pt := packing_targets.item (pt_name)
				if is_target_name_allowed (pt_name) then
					has_choice := True
					pt.create_text_fields
					fact.add_choice_and_text (pt_row.packing_target, pt_row.descr)
				else
					disallowed_packing_targets.extend (pt)
					if is_target_name_available (pt_name) then
						avail_targets.extend (pt_name)
					end
				end
				packing_target_rows.forth
			end
			if default_field_set /= Void then
				default_choice := default_field_set.packing_target
			else
				default_choice := target.Standard_reticle
			end
			if not is_target_name_allowed (default_choice) then
				-- Preferred default not available 
				if has_choice then
					default_choice := fact.choices.first
				else
					handle_hard_error ("no available packing target has enough data to build interface - manual intervention required%N" + "Available packing targets:  " + su.joined_by (avail_targets, ", "))
				end
			end
			fact.set_default_choice (default_choice)
			fact.generate_combo_box
			packing_target_field := fact.combo_box
			packing_target_field.disable_edit
			packing_target_field.change_actions.extend (agent process_new_packing_target)
		end
		
	process_new_packing_target
			-- Process change of packing target
		do
			populate_packing_target_specific_box
		end

	populate_packing_target_specific_box
			-- Rebuild contents of `packing_target_specific_box'
			-- based on currently selected packing target
		local
			pt: PACKING_TARGET
			pt_name: STRING
		do
			pt_name ?= packing_target_field.selected_item.data
			pt := packing_targets.item (pt_name)
			packing_target_specific_box.wipe_out
			packing_target_specific_box.extend (pt.target_specific_box)
			gui.set_box_background_recursive (main_vbox, background_color)
		end

	process_accept
			-- Take actions needed when user accepts
			-- current operation
		local
			operation: ANONYMOUS_OPERATION
		do
			if is_active then
				disable_active
				create operation.make (agent process_accept_request)
				operation.enable_abort_caught
				operation.execute
				if operation.aborted then
					session.rollback
					enable_active
				else
					success_actions.call ([])
				end
			end
		end

	process_accept_request
			-- Process an accept request
		local
			pt: PACKING_TARGET
			pt_name: STRING
		do
			pt_name ?= packing_target_field.selected_item.data
			pt := packing_targets.item (pt_name)
			pt.validate_input
			if not pt.is_input_valid then
				handle_user_error ("some input values are missing or invalid.%N%N" + pt.validation_errors)
			end
			pt.update_database
			field_set_name := pt.selected_field_set_name
		end

	process_decline
			-- Take actions needed when user declines
			-- current operation
		do
			if is_active then
				disable_active
				abort_actions.call ([])
			end
		end

	packing_target_field: EV_COMBO_BOX
			-- Field for selecting packing target

	usage_label: EV_LABEL
			-- Label that explains use of interface

	unavailable_targets_label: EV_LABEL
			-- Label that tells user about packing
			-- targets that are not available

	field_sets: DYNAMIC_LIST [FIELD_SETS_ROW]
			-- All possible field sets for run and wafer techn

	related_field_sets: DYNAMIC_LIST [FIELD_SETS_ROW]
			-- List of field sets ever associated with `run_id'
			-- or its wafer techn

	default_field_set: FIELD_SETS_ROW
			-- Default field set determined by saved state
			-- from Undo

	packing_target_rows: DYNAMIC_LIST [PACKING_TARGETS_ROW]
			-- All possible packing targets

	wafer_techn_packing_targets: LIST [WAFER_TECHN_PACKING_TARGETS_ROW]
			-- All rows for run's wafer techn, used to determine
			-- which packing targets are available for run

	packing_targets: HASH_TABLE [PACKING_TARGET, STRING]
			-- All known packing targets, indexed by
			-- target name

	disallowed_packing_targets: DYNAMIC_LIST [PACKING_TARGET]
			-- List of all known packing targets that are 
			-- not allowed to be used for this run

	selected_packing_target: STRING
			-- Name of currently selected packing target

	selected_field_set: STRING
			-- Name of currently selected field set
			-- (may not exist in database yet)

	main_vbox: EV_VERTICAL_BOX
			-- Main box for interface (holds all widgets)

	packing_target_specific_box: EV_VERTICAL_BOX
			-- Box whose contents changes depending on
			-- which packing target is selected

	last_box: EV_HORIZONTAL_BOX
			-- Last box built

end
