class	FIELD_STEP_PATTERN_BUILDER

feature {NONE} -- data

	fldspat: FIELD_STEP_PATTERNS_MROW;

	fldset: FIELD_SETS_MROW;

feature	{NONE} -- aspects of the data

	field_set_name: STRING is
	do if fldset /= Void
	    then Result := fldset.field_set_name
	   end
	end;

	field_step_pattern: STRING is
	do if fldspat /= Void
	    then Result := fldspat.field_step_pattern
	   end
	end;
	
	litho_size_id: STRING is
	do if fldset /= Void
	    then Result := fldset.litho_size_id
	   end
	end;
	
	lithsz: LITHO_SIZES_MROW is
	do if fldset /= Void
	    then Result := fldset.litho_size
	   end
	end;

	packing_target: STRING 
	local ls: LITHO_SIZES_ROW
	do  ls := lithsz;
	    if ls /= Void 
	     then Result := ls.packing_target
	    end
	end;

	wafer_diameter: DOUBLE is
	local ls: LITHO_SIZES_ROW
	do  ls := lithsz;
	    if ls /= Void and then ls.wafer_type /= Void
	     then Result := ls.wafer_type.wafer_diameter
	    end
	end;

	fldsteps: LIST[FIELD_STEP_PATTERNXFIELD_LOCS_MROW] is
	require fldspat /= Void
	do  Result := fldspat.field_locations
	end;

feature	-- builder of computed field locs

	build_computed_field_locations
			(a_fsp: FIELD_STEP_PATTERNS_MROW;
			 twzr_margin: DOUBLE;
			 an_alignment: INTEGER)
		-- build field locations for a_fsp 
		-- twzr_margin is the "tweezer margin" for determining the
		--	limits of the steps (a negative margin allows
		--	steps that fall that much off the wafer)
		-- an_alignment determines the filed placement at the
		--	center of the wafer
	require	have_fsp: a_fsp /= Void
		have_no_steps: a_fsp.field_locations.is_empty
		valid_align: 1 <= an_alignment and an_alignment <= 4
	do  fldspat := a_fsp;
	    fldset  := a_fsp.field_set
	    tweezer_margin := twzr_margin
	    alignmnt := an_alignment
	    compute_new_step_locations
	end;

feature {NONE} -- worker bees

	tweezer_margin: DOUBLE;
	alignmnt: INTEGER	-- 1 through 4

	compute_new_step_locations is
	require	have_fldspat: fldspat /= Void
		have_no_steps: fldsteps.is_empty
	local r, c, rmax, cmax: INTEGER;
		rad, sx, sy, cx, cy, tx, ty: DOUBLE;
		h_offset, v_offset: DOUBLE;
	do  -- turn alignmnt into pair of offsets
	    if alignmnt = 2 or alignmnt = 3
	     then v_offset := 0.5 
	     else v_offset := 0 end;
	    if alignmnt = 2 or alignmnt = 4
	     then h_offset := 0.5
	     else h_offset := 0 end;
	    -- determine row limits
	    rad := (wafer_diameter / 2.0) - tweezer_margin;   -- radius
	    sx := lithsz.field_size_x;
	    sy := lithsz.field_size_y;
	    rmax := 1 + (rad / sy).ceiling;
	    cmax := 1 + (rad / sx).ceiling;
	    from  r := rmax
	    until r < (-rmax)
	    loop  cy := sy * (r + v_offset);
		  if cy > 0
		   then ty := cy + sy/2
		   else ty := cy - sy/2
		  end;
		  from  c := -cmax
		  until c > cmax
		  loop  cx := sx * (c + h_offset);
			-- see if step centered at cx, cy is inside rad
			if cx > 0
			 then tx := cx + sx/2
			 else tx := cx - sx/2
			 end;
			if ty*ty + tx*tx <= rad*rad
			 then add_one_location(cx, cy, false);
			end;
			c := c + 1
		  end;
		  r := r - 1
	    end;
	end;

feature -- add one location

	add_one_location(x, y: DOUBLE; pp: BOOLEAN) is
	local loc: FIELD_STEP_PATTERNXFIELD_LOCS_MROW
	do  create loc.make_null(fldspat.connection);
	    loc.set_field_step_pattern(field_step_pattern);
	    loc.set_field_name("A");
	    loc.set_copy_no(fldsteps.count+1)
	    loc.set_partially_processed(pp);
	    loc.set_location_x(x);
	    loc.set_location_y(y);
	    fldsteps.extend(loc)
	end;

end -- FIELD_STEP_PATTERN_BUILDER
