
indexing
	
	description: "A map for a specific run which shows each field of view"
	author: "David Hollenberg"
	date: "January 21, 2005"

deferred class FIELDS_OF_VIEW_MAP
inherit
	DETAILED_PAYLOAD_MAP

feature -- Operations

	generate_page_for_field (fld_name: STRING)
			-- Generate page for field `fld_name' with
			-- both drawing and title and put in
			-- `last_page'
		do
			field_name := fld_name
			generate_page
		end

	generate_drawing_for_field (fld_name: STRING)
			-- Generate drawing for field `fld_name' and put
			-- in `last_drawing'
		do
			field_name := fld_name
			generate_drawing
		end

	generate_title_for_field (fld_name: STRING)
			-- Generate title for field `fld_name' and put
			-- in `last_drawing'
		do
			field_name := fld_name
			generate_title
		end

feature {NONE} -- Implementation

	generate_whole_document
			-- Generate entire document representing map
			-- and put in `last_document'
		do
			from
				field_names.start
			until
				field_names.after
			loop
				generate_page_for_field (field_names.item)
				last_document.add_page (last_page)
				field_names.forth
			end
		end

	generate_drawing_for_location (loc: DIE_LOCATIONS_EROW)
			-- Generate part of drawing for `loc' that is specific
			-- to a particular map
		local
			box: DRW_BASIC_RECTANGLE
			size: CONST_POINT
			desc1, desc2: STRING
		do
			if loc.is_scribe then
			else
				size := loc.printed_size_on_wafer
				create box.make (size.x, size.y, loc.location_x, loc.location_y)
				desc1 := primary_description (loc)
				desc2 := secondary_description (loc)
				last_drawing.extend (optional_text_pair_in_box (desc1, desc2, box, loc.non_null_die_wafer_rotation))
			end
		end

	primary_description (loc: DIE_LOCATIONS_EROW): STRING
			-- Primary description of `loc' to display on map
		require
			loc_exists: loc /= Void
		do
			Result := loc.die_id
		ensure
			result_exists: Result /= Void
		end

	secondary_description (loc: DIE_LOCATIONS_EROW): STRING
			-- Secondary description of `loc' to display on map
		require
			loc_exists: loc /= Void
		deferred
		ensure
			result_exists: Result /= Void
		end

	generate_whole_title
			-- Generate entire title and put in `last_title'
		local
			b: DRW_BASIC_RECTANGLE
			box: DRW_RECTANGLE
			w: DOUBLE
			t: STRING
		do
			-- Add title

			w := available_paper_width - (title_aux_width + title_logo_width)
			create box.make (w, title_height, w / 2.0 + title_logo_width, 0)
			box.set_stroke_width (thick_width)
			last_title.extend (box)
			
			b := box.bounding_box
			shrink_box (b)
			t := run_id + " " + title
			if has_multiple_fields then
				t.append (" - Field " + field_name)
			end
			last_title.extend (text_in_box (t, b))
			
			append_mosis_logo
			append_std_title_info
		end

	location_fill_color (loc: DIE_LOCATIONS_EROW): DRW_COLOR
			-- Fill color to be used for interior of `loc'
		do
			if loc.is_scribe then
				Result := scribe_fill
			elseif loc.is_test_vehicle then
				Result := test_vehicle_fill
			elseif loc.is_vendor_geometry or loc.is_vendor_ghost then
				Result := vendor_geometry_fill
			end
		end

end
