
indexing
	
	description: "An operation that deletes a file"
	author: "David Hollenberg"
	date: "April 23, 2003"

class FILE_DELETE_OP
inherit
	OPERATION
		rename
			try_operation as try_delete
		end
	EXCEPTIONS_ACCESS
		export
			{NONE} all
		undefine
			default_create
		end

create
	default_create

feature -- Operations

	delete (fname: STRING)
			-- Delete file named `fname'
		require
			file_name_exists: fname /= Void
			file_name_nonempty: not fname.is_empty
		do
			file_name := fname
			execute
		end

	delete_if_exists (fname: STRING)
			-- Delete file named `fname' if it exists
		require
			file_name_exists: fname /= Void
			file_name_nonempty: not fname.is_empty
		local
			checker: FILE_EXISTENCE_CHECK_OP
		do
			create checker
			checker.check_existence (fname)
			if checker.path_exists then
				delete (fname)
			end
		end

feature {NONE} -- Implementation

	try_delete
			-- Try file delete
		local
			error: FILE_DELETE_ERROR
		do
			publish_try_event
			try_to_delete_file_safely
			if is_deleted then
				publish_ok_event
			else
				publish_failed_event
				create error
				error.set_hard_error
				error.set_file_name (file_name)
				error.handle
			end
		end

	try_to_delete_file_safely
			-- Try to delete file.  Do not raise exception if
			-- delete fails, unless the exception is due to
			-- an assertion violation
		local
			tried: BOOLEAN
		do
			if not tried then
				try_to_delete_file
				is_deleted := True
			else
				is_deleted := False
			end
		rescue
			if not exc.assertion_violation then
				tried := True
				retry
			end
		end

	try_to_delete_file
		local
			file: LARGE_PLAIN_TEXT_FILE
		do
			create file.make (file_name)
			file.delete
		end

	file_name: STRING
			-- Name of file to try to delete

	is_deleted: BOOLEAN
			-- Was file successfully deleted?

	publish_try_event
		local
			try_event: FILE_DELETE_TRY_EVENT
		do
			create try_event
			try_event.set_file_name (file_name)
			try_event.publish
		end

	publish_ok_event
		local
			ok_event: FILE_DELETE_OK_EVENT
		do
			create ok_event
			ok_event.set_file_name (file_name)
			ok_event.publish
		end

	publish_failed_event
		local
			failed_event: FILE_DELETE_FAILED_EVENT
		do
			create failed_event
			failed_event.set_file_name (file_name)
			failed_event.publish
		end

end
