
indexing
	
	description: "An operation that checks whether a file exists, is a %
		%plain file and is executable"
	author: "David Hollenberg"
	date: "June 2, 2003"

class FILE_EXECUTABLE_CHECK_OP
inherit
	OPERATION
		rename
			try_operation as try_to_check_executable
		end
	OPERATING_ENVIRONMENT
		export
			{NONE} all
		undefine
			default_create
		end
	CONDITION_UTILITIES
		export
			{NONE} all
		undefine
			default_create
		end

create
	default_create

feature -- Operations

	check_executable (fname, descr: STRING)
			-- Check whether path named `fname' exists, is
			-- a plain file and is executable, where `descr'
			-- is a description of the file if not void
		require
			file_name_exists: fname /= Void
			file_name_nonempty: not fname.is_empty
		do
			file_name := fname
			file_description := descr
			execute
		end

feature {NONE} -- Implementation

	try_to_check_executable
			-- Try to check whether file is executable
		local
			checker: FILE_EXISTENCE_CHECK_OP
			f: LARGE_RAW_FILE
		do
			create checker
			checker.check_existence (file_name)
			if not checker.path_exists then
				handle_hard_error (program_description + " not found")
			end
			create f.make (file_name)
			if not f.is_plain then
				handle_hard_error (program_description + " is not a plain file")
			elseif not f.is_executable then
				handle_hard_error (program_description + " is not executable")
			end
		end

	file_name: STRING
			-- Name of file whose executability should be checked

	file_description: STRING
			-- Description file whose executability
			-- should be checked, if not Void

	program_description: STRING
			-- Full description of file whose executability
			-- should be checked, including both file name
			-- and file description (if available)
		do
			if file_description /= Void then
				Result := file_description + " file " + file_name
			else
				Result := "File " + file_name
			end
		ensure
			result_not_void: Result /= Void
		end

end
