
indexing
	
	description: "An operation that checks whether a file or directory exists"
	author: "David Hollenberg"
	date: "April 23, 2003"

class FILE_EXISTENCE_CHECK_OP
inherit
	OPERATION
		rename
			try_operation as try_to_check_existence
		end
	OPERATING_ENVIRONMENT
		export
			{NONE} all
		undefine
			default_create
		end

create
	default_create

feature -- Properties

	path_exists: BOOLEAN
			-- Does file exist?

feature -- Operations

	check_existence (fname: STRING)
			-- Check whether path named `fname' exists
			-- and set `exists' appropriately
		require
			file_name_exists: fname /= Void
			file_name_nonempty: not fname.is_empty
		do
			file_name := fname
			execute
		end

feature {NONE} -- Implementation

	try_to_check_existence
			-- Try to check file existence
		local
			sep: CHARACTER
			count, n: INTEGER
			path: STRING
			file: LARGE_PLAIN_TEXT_FILE
			done: BOOLEAN
			error: FILE_EXISTS_ERROR
		do
			path_exists := False
			existence_check_failed := False
			sep := Directory_separator
			count := file_name.count
			create file.make ("/")
			from
				n := file_name.index_of (sep, 1)
			until
				n = 0 or done
			loop
				path := file_name.substring (1, n - 1)
				if path.is_empty then
					path.extend (Directory_separator)
				else
					file.make (path)
				end
				if not file.exists then
					done := True
				elseif not file.is_executable then
					existence_check_failed := True
					done := True
				end
				n := file_name.index_of (sep, n + 1)
			end
			if not done then
				file.make (file_name)
				path_exists := file.exists
			end
			if existence_check_failed then
				create error
				error.set_hard_error
				error.set_path_name (file_name)
				error.handle
			end
		end

	file_name: STRING
			-- Name of file whose existence should be checked

	existence_check_failed: BOOLEAN
			-- Did file existence check fail?

end
