
indexing
	
	description: "An operation that changes the group of a file"
	author: "David Hollenberg"
	date: "April 23, 2003"

class FILE_GROUP_CHANGE_OP
inherit
	OPERATION
		rename
			try_operation as try_group_change
		end
	EXCEPTIONS_ACCESS
		export
			{NONE} all
		undefine
			default_create
		end
	UNIX_OS_ACCESS
		export
			{NONE} all
		undefine
			default_create
		end

create
	default_create

feature -- Operations

	change_group (fname, grp_name: STRING)
			-- Change group of file named `fname' to `grp_name'
		require
			file_name_exists: fname /= Void
			file_name_nonempty: not fname.is_empty
			group_name_exists: grp_name /= Void
			group_name_nonempty: not grp_name.is_empty
		do
			file_name := fname
			group_name := grp_name
			execute
		end

feature {NONE} -- Implementation

	try_group_change
			-- Try to change group of file
		local
			file: LARGE_RAW_FILE
			error: FILE_GROUP_CHANGE_ERROR
		do
			group_id := unix_os.group_id_for_group_name (group_name)
			create file.make (file_name)
			if file.group_id /= group_id then
				is_changed := False
				if group_id /= -1 then
					try_to_change_group_safely
				end
				if is_changed then
					publish_ok_event
				else
					publish_failed_event
					create error
					error.set_hard_error
					error.set_file_name (file_name)
					error.set_group_name (group_name)
					error.handle
				end
			end
		end

	try_to_change_group_safely
			-- Try to change group file.  Do not raise exception if
			-- operation fails, unless the exception is due to
			-- an assertion violation
		local
			tried: BOOLEAN
		do
			if not tried then
				try_to_change_group
				is_changed := True
			else
				is_changed := False
			end
		rescue
			if not exc.assertion_violation then
				tried := True
				retry
			end
		end

	try_to_change_group
		local
			file: PLAIN_TEXT_FILE
		do
			create file.make (file_name)
			file.change_group (group_id)
		end

	file_name: STRING
			-- Name of file for which to change group

	group_name: STRING
			-- New group name

	group_id: INTEGER
			-- New group id
	
	is_changed: BOOLEAN
			-- Was group successfully changed?

	publish_ok_event
		local
			ok_event: FILE_GROUP_CHANGE_OK_EVENT
		do
			create ok_event
			ok_event.set_file_name (file_name)
			ok_event.set_group_name (group_name)
			ok_event.publish
		end

	publish_failed_event
		local
			failed_event: FILE_GROUP_CHANGE_FAILED_EVENT
		do
			create failed_event
			failed_event.set_file_name (file_name)
			failed_event.set_group_name (group_name)
			failed_event.publish
		end

end
