
indexing
	
	description: "An operation that does something with a file"
	author: "David Hollenberg"
	date: "April 23, 2003"

class FILE_OP
inherit
	DYNAMIC_OPERATION
	FILE_UTILITIES_ACCESS
		export
			{NONE} all
		undefine
			default_create
		end

create
	default_create

feature -- Properties

	is_valid_permission_string (s: STRING): BOOLEAN
			-- Is `s' a valid file permission string?
			-- A valid string is one constructed from the
			-- string "rwxrwxrwx", where each character can
			-- be replaced by a '-'.  Also the first and second "x"
			-- can be replaced by "s" to indicate set user id and
			-- set group id
		require
			string_exists: s /= Void
		do
			Result := fu.is_valid_permission_string (s)
		end

feature -- Operations

	verify_existence (f: LARGE_FILE)
			-- Verify that `f' exists and is a plain file
		require
			file_not_void: f /= Void
		do
			file := f
			set_implementation (agent try_to_verify_existence)
			execute
		end

	verify_permissions (f: LARGE_FILE perm: STRING)
			-- Verify that `f' has at least the permissions
			-- indicated by `perm'
		require
			file_not_void: f /= Void
			perm_not_void: perm /= Void
			perm_valid: is_valid_permission_string (perm)
		do
			file := f
			permissions := perm
			set_implementation (agent try_to_verify_permissions)
			execute
		end

feature {NONE} -- Implementation

	try_to_verify_existence
			-- Try to verify file existence
		local
			op: FILE_EXISTENCE_CHECK_OP
			verify_error: FILE_EXISTENCE_VERIFICATION_ERROR
			plain_error: FILE_NOT_PLAIN_ERROR
		do
			create op
			op.check_existence (file.name)
			if not op.path_exists then
				create verify_error
				verify_error.set_file_name (file.name)
				verify_error.set_hard_error
				verify_error.handle
			elseif not file.is_plain then
				create plain_error
				plain_error.set_file_name (file.name)
				plain_error.set_hard_error
				plain_error.handle
			end
		end

	try_to_verify_permissions
			-- Try to verify file permissions
		local
			op: FILE_EXISTENCE_CHECK_OP
			verify_error: FILE_EXISTENCE_VERIFICATION_ERROR
			perm_error: FILE_PERMISSION_ERROR
			perm: INTEGER
		do
			perm := fu.permission_string_to_mask (permissions)
			create op
			op.check_existence (file.name)
			if not op.path_exists then
				create verify_error
				verify_error.set_file_name (file.name)
				verify_error.set_hard_error
				verify_error.handle
			elseif file.protection & perm /= perm then
				create perm_error
				perm_error.set_file_name (file.name)
				perm_error.set_permissions (permissions)
				perm_error.set_hard_error
				perm_error.handle
			end
		end

	file: LARGE_FILE
			-- File on which operation will be performed

	permissions: STRING
			-- Required permissions (to be verified)

end
