
indexing
	
	description: "An operation that opens a file"
	author: "David Hollenberg"
	date: "March 12, 2003"

deferred class FILE_OPEN_OP
inherit
	OPERATION
		rename
			try_operation as try_open
		end
	EXCEPTIONS_ACCESS
		export
			{NONE} all
		undefine
			default_create
		end
	MISC_UTILITIES_ACCESS
		export
			{NONE} all
		undefine
			default_create
		end

feature -- Operations

	open (f: LARGE_FILE)
			-- Open `f'
		require
			file_exists: f /= Void
			file_closed: f.is_closed
		do
			file := f
			execute
		end

feature {NONE} -- Implementation

	try_open
			-- Try file open
		local
			error: FILE_OPEN_ERROR
		do
			publish_try_event
			try_to_open_file_safely
			if not file.is_closed then
				publish_ok_event
			else
				publish_failed_event
				create error
				error.set_hard_error
				error.set_file_name (file.name)
				error.set_mode (mode)
				error.set_reason (reason)
				error.handle
			end
		end

	try_to_open_file_safely
			-- Try to open file.  Do not raise exception if
			-- open fails, unless the exception is due to
			-- an assertion violation
		local
			tried: BOOLEAN
		do
			if not tried then
				try_to_open_file
			end
		rescue
			if not exc.assertion_violation then
				reason := mu.exception_explanation
				tried := True
				retry
			end
		end

	try_to_open_file
		deferred
		end

	file: LARGE_FILE
			-- File to try to open

	mode: STRING
			-- Description of mode for file open
		deferred
		end

	reason: STRING
			-- Reason for last failure

	publish_try_event
		local
			try_event: FILE_OPEN_TRY_EVENT
		do
			create try_event
			try_event.set_file_name (file.name)
			try_event.set_mode (mode)
			try_event.publish
		end

	publish_ok_event
		local
			ok_event: FILE_OPEN_OK_EVENT
		do
			create ok_event
			ok_event.set_file_name (file.name)
			ok_event.set_mode (mode)
			ok_event.publish
		end

	publish_failed_event
		local
			failed_event: FILE_OPEN_FAILED_EVENT
		do
			create failed_event
			failed_event.set_file_name (file.name)
			failed_event.set_mode (mode)
			failed_event.set_reason (reason)
			failed_event.publish
		end

invariant
	mode_exists: mode /= Void
end
