
indexing
	
	description: "An operation that does a retrieve from a file that previously had an Eiffel object structure stored into it"
	author: "David Hollenberg"
	date: "April 23, 2003"

class FILE_RETRIEVE_OP
inherit
	OPERATION
		rename
			try_operation as try_retrieve
		end
	EXCEPTIONS_ACCESS
		export
			{NONE} all
		undefine
			default_create
		end

create
	default_create

feature -- Properties

	item: ANY
			-- Object structure retrieve from file

feature -- Operations

	retrieve (f: LARGE_RAW_FILE)
			-- Retrieve file named `fname' and put in `item'
		require
			file_exists: f /= Void
			file_open_read: f.is_open_read
		do
			file := f
			execute
		end

feature {NONE} -- Implementation

	try_retrieve
			-- Try file retrieve
		local
			error: FILE_RETRIEVE_ERROR
		do
			try_to_retrieve_file_safely
			if item /= Void then
				publish_ok_event
			else
				publish_failed_event
				create error
				error.set_hard_error
				error.disable_retry
				error.set_file_name (file.name)
				error.handle
			end
		end

	try_to_retrieve_file_safely
			-- Try to retrieve object structure from file.
			-- Do not raise exception if
			-- operation fails, unless the exception is due to
			-- an assertion violation
		local
			tried: BOOLEAN
		do
			if not tried then
				item := Void
				try_to_retrieve_file
			end
		rescue
			if not exc.assertion_violation then
				tried := True
				retry
			end
		end

	try_to_retrieve_file
		do
			item := file.retrieved
		end

	file: LARGE_RAW_FILE
			-- File from which to retrieve object structure

	publish_ok_event
		local
			ok_event: FILE_RETRIEVE_OK_EVENT
		do
			create ok_event
			ok_event.set_file_name (file.name)
			ok_event.publish
		end

	publish_failed_event
		local
			failed_event: FILE_RETRIEVE_FAILED_EVENT
		do
			create failed_event
			failed_event.set_file_name (file.name)
			failed_event.publish
		end

end
