
indexing
	
	description: "A run closing step operation that generates %
		%filled MEBES files for levels that require fill to be added"
	author: "David Hollenberg"
	date: "February 29, 2004"

class FILL_MEBES_OP
inherit
	RC_PRIMITIVE_STEP_OP
		redefine
			is_currently_undoable, not_undoable_reason
		end
	HAS_USER_INTERFACE
	CREATES_EXTERNAL_STATE
	GUI_UTILITIES_ACCESS
		export
			{NONE} all
		undefine
			default_create
		end

create
	make

feature -- Properties

	input_data_items: LIST [STRING]
			-- Data items required before step can be executed
		do
			Result := array_as_list (<<
				data_item.Fill_area_selector_file
				>>)
		end

feature {NONE} -- Implementation

	is_applicable: BOOLEAN
			-- Is step applicable to run for session?
		do
			Result := cache.run_needs_fill
		end

	is_currently_undoable: BOOLEAN
			-- Is `Current' currently in a state where it can be
			-- undone?
		do
			Result := not cache.run_wafer_level_job_fill_rows.there_exists (agent {RUN_WAFER_LEVEL_JOBS_ROW}.is_status_stopable)
		end

	not_undoable_reason: STRING
			-- Reason step cannot currently be undone
		do
			Result := "one or more level fill jobs is running or pending.%NPlease select all stoppable jobs and stop them first."
		end

	do_step
		local
			ifc: JOB_CONTROL_IFC
		do
			cache.check_one_scale_factor
			cache.check_one_field
			if cache.run_wafer_level_job_fill_rows.is_empty then
				insert_level_jobs
			end
			ifc := factory.level_fill_control_interface (session.run_id, session.title_with_run_id ("Fill Levels"))
			ifc.set_session (session)
			ifc.success_actions.extend (agent call_completion_actions)
			ifc.abort_actions.append (abort_actions)
			ifc.build
			if preferences.is_autosubmit_enabled then
				gui.do_once_on_idle (agent ifc.autosubmit_jobs)
			end
		end

	undo_step
			-- Undo execution of step
		local
			lock_op: ROW_LOCK_OP
			pf_mrows: LIST [PATTERN_FILES_MROW]
		do
			level_job_mrows := cache.run_wafer_level_job_fill_mrows
			add_commit_action (agent delete_files (generated_files))
			pf_mrows := pattern_file_rows
			create lock_op
			lock_op.lock_list_nowait (level_job_mrows)
			lock_op.lock_list_nowait (pf_mrows)
			level_job_mrows.do_all (agent {RUN_WAFER_LEVEL_JOBS_MROW}.delete)
			pf_mrows.do_all (agent {PATTERN_FILES_MROW}.delete)
			call_completion_actions
		end

	set_state
			-- Set `state'
		do
			state := Void
		end

	insert_level_jobs
			-- Insert one row in RUN_WAFER_LEVEL_JOBS for
			-- each level for run's wafer techn that needs
			-- to have fill added
		local
			levels: LIST [RUN_LEVELS_ROW]
			level: RUN_LEVELS_ROW
			job: RUN_WAFER_LEVEL_JOBS_MROW
			jobs: ARRAYED_LIST [RUN_WAFER_LEVEL_JOBS_MROW]
			cats_job_type: CATS_JOB_TYPE_VALUES
			job_status: JOB_STATUS_VALUES
		do
			levels := cache.run_level_rows
			create jobs.make (levels.count)
			from
				levels.start
			until
				levels.after
			loop
				level := levels.item
				if level.filled_layer then
					create job.make_null (session.connection)
					job.set_run_id (session.run_id)
					job.set_wafer_techn_family (cache.wafer_techn_family)
					job.set_level_no (level.level_no)
					job.set_cats_job_type (cats_job_type.Fill)
					job.set_job_status (job_status.Created)
					jobs.extend (job)
				end
				levels.forth
			end
			session.commit
			cache.cache_run_wafer_level_fill_jobs (jobs)
		end

	pattern_file_rows: LIST [PATTERN_FILES_MROW]
			-- Rows in pattern files table created by step
		local
			row: PATTERN_FILES_MROW
			pattern_type: PATTERN_TYPE_VALUES
		do
			create row.make_template
			Result := row.rows_for_run_and_type (session.run_id, Pattern_type.Fill, connection, expected_count.At_least_zero)
		end

	generated_files: LIST [RC_FILE]
			-- Files which may have been generated during
			-- execution of step (were generated if step
			-- finished)
		local
			level_job: RUN_WAFER_LEVEL_JOBS_MROW
			level: ABSTRACT_LEVEL
		do
			create {ARRAYED_LIST [RC_FILE]} Result.make (20)
			from
				level_job_mrows.start
			until
				level_job_mrows.after
			loop
				level_job := level_job_mrows.item
				level := cache.wafer_techn_level_row_for_level (level_job.level_no)
				if level_job.is_status_resettable then
					-- Files may exist
					Result.append (cache.level_fill_pattern_files (level))
					Result.extend (cache.level_fill_log_file (level))
					Result.extend (cache.level_fill_done_file (level))
				end
				level_job_mrows.forth
			end
		end

	delete_files (list: LIST [RC_FILE])
			-- Delete all files in `list' that exist
		do
			from
				list.start
			until
				list.after
			loop
				list.item.delete_if_exists
				list.forth
			end
		end

	level_job_mrows: LIST [RUN_WAFER_LEVEL_JOBS_MROW]
			-- Rows for wafer level jobs

end
