
indexing
	
	description: "Window to control creation of fill selector file for %
		%a run"
	author: "David Hollenberg"
	date: "March 4, 2004"

class FILL_SELECTOR_CONTROL_WINDOW

inherit
	JOB_CONTROL_WINDOW [FILL_SELECTOR_JOB]
	NUMBER_UTILITIES_ACCESS
		export
			{NONE} all
		undefine
			default_create
		end
		
create
	make

feature {NONE} -- Implementation

	add_extra_select_menu_items (select_menu: EV_MENU)
			-- Add any application-specific items to `select_menu'
		do	
		end

	add_extra_view_row_menu_items (row_menu: EV_MENU)
			-- Add any application-specific items to `row_menu'
		do	
		end

	fetch_needed_data
			-- Fetch needed data when first building interface
		do
			fetch_requestor_list
			fetch_batch_jobs_list
		end

	fetch_requestor_list
			-- Fetch list of rows requesting batch job
			-- (currently one row) from database and
			-- put in `fill_selector_rows'
		do
			fill_selector_rows := cache.run_fill_selector_job_mrows
		end
			
	batch_jobs_for_run: LIST [BATCH_JOBS_MROW]
			-- List of batch jobs for this run
		local
			batch_jobs_row: BATCH_JOBS_MROW
		do
			create batch_jobs_row.make_template
			Result := batch_jobs_row.fill_selector_jobs_for_run (session.run_id, session.connection, expected_count.At_most_one)
		end
			
	initialize_total_count
			-- Initialize `total_count'
		do
			total_count := fill_selector_rows.count
		end

	build_all_rows
			-- Build list `all_rows'
		local
			row: FILL_SELECTOR_JOB
			selector_job: RUN_JOBS_MROW
			mebes_files: ARRAYED_LIST [STRING]
			files: LIST [RC_FILE]
		do
			from
				fill_selector_rows.start
			until
				fill_selector_rows.after
			loop
				selector_job := fill_selector_rows.item
				create row.make (selector_job)
				row.set_run_id (session.run_id)
				row.set_job_script_file (session.environment.fill_selector_program)
				row.set_job_log_file (cache.fill_selector_log_file.file_pathname)
				row.set_job_done_file (cache.fill_selector_done_file.file_pathname)
				files := cache.fill_selector_created_files
				create mebes_files.make (files.count)
				from
					files.start
				until
					files.after
				loop
					mebes_files.extend (files.item.file_pathname)
					files.forth
				end
				row.set_job_output_files (mebes_files)
				row.select_actions.extend (agent process_select_one (row))
				row.deselect_actions.extend (agent process_deselect_one (row))
				all_rows.extend (row)
				fill_selector_rows.forth
			end
		end
			
	add_visible_columns
			-- Add columns to `visible_columns'
		do
			visible_columns.extend (create {JOB_TYPE_COLUMN [FILL_SELECTOR_JOB]})
			visible_columns.extend (create {JOB_STATUS_COLUMN [FILL_SELECTOR_JOB]})
			visible_columns.extend (create {BATCH_JOB_NO_COLUMN [FILL_SELECTOR_JOB]})
			visible_columns.extend (create {BATCH_JOB_STATE_COLUMN [FILL_SELECTOR_JOB]})
			visible_columns.extend (create {BATCH_JOB_HOST_COLUMN [FILL_SELECTOR_JOB]})
			visible_columns.extend (create {JOB_TIME_COLUMN [FILL_SELECTOR_JOB]})
			visible_columns.extend (create {ERROR_DESCRIPTION_COLUMN [FILL_SELECTOR_JOB]})
		end
			
	has_job_status_agent (s: STRING): FUNCTION [FILL_SELECTOR_JOB, TUPLE [FILL_SELECTOR_JOB], BOOLEAN]
			-- Agent that tells whether row has job status `s'
		do
			Result := agent {FILL_SELECTOR_JOB}.has_job_status (s)
		end

	is_stopable_agent: FUNCTION [FILL_SELECTOR_JOB, TUPLE [FILL_SELECTOR_JOB], BOOLEAN]
			-- Agent that tells whether row has job that
			-- is stopable
		do
			Result := agent {FILL_SELECTOR_JOB}.is_stopable
		end

	is_unstopable_agent: FUNCTION [FILL_SELECTOR_JOB, TUPLE [FILL_SELECTOR_JOB], BOOLEAN]
			-- Agent that tells whether row does not have a
			-- job that is stopable
		do
			Result := agent {FILL_SELECTOR_JOB}.is_unstopable
		end

	is_resettable_agent: FUNCTION [FILL_SELECTOR_JOB, TUPLE [FILL_SELECTOR_JOB], BOOLEAN]
			-- Agent that tells whether row has job that
			-- is resettable
		do
			Result := agent {FILL_SELECTOR_JOB}.is_resettable
		end

	is_unresettable_agent: FUNCTION [FILL_SELECTOR_JOB, TUPLE [FILL_SELECTOR_JOB], BOOLEAN]
			-- Agent that tells whether row does not have
			-- a job that is resettable
		do
			Result := agent {FILL_SELECTOR_JOB}.is_unresettable
		end

	is_unsuccessful_agent: FUNCTION [FILL_SELECTOR_JOB, TUPLE [FILL_SELECTOR_JOB], BOOLEAN]
			-- Agent that tells whether row has job that
			-- was unsuccessful
		do
			Result := agent {FILL_SELECTOR_JOB}.is_unsuccessful
		end

	is_submittable_agent: FUNCTION [FILL_SELECTOR_JOB, TUPLE [FILL_SELECTOR_JOB], BOOLEAN]
			-- Agent that tells whether row has job that
			-- is submittable
		do
			Result := agent {FILL_SELECTOR_JOB}.is_submittable
		end

	is_unsubmittable_agent: FUNCTION [FILL_SELECTOR_JOB, TUPLE [FILL_SELECTOR_JOB], BOOLEAN]
			-- Agent that tells whether row is not in a state
			-- where job can be submitted
		do
			Result := agent {FILL_SELECTOR_JOB}.is_unsubmittable
		end

	is_unreleasable_agent: FUNCTION [FILL_SELECTOR_JOB, TUPLE [FILL_SELECTOR_JOB], BOOLEAN]
			-- Agent that tells whether row is not in a state
			-- where batch job can be released
		do
			Result := agent {FILL_SELECTOR_JOB}.is_unreleasable
		end

	is_passed_agent: FUNCTION [FILL_SELECTOR_JOB, TUPLE [FILL_SELECTOR_JOB], BOOLEAN]
			-- Agent that tells whether row has job that
			-- is passed
		do
			Result := agent {FILL_SELECTOR_JOB}.is_passed
		end

	job_arguments (row: FILL_SELECTOR_JOB): STRING
			-- Arguments to be passed to batch job for `row'
		local
			compact, format, fill_scale_factor, space: STRING
			gen: RC_FILE_GENERATOR
			fp: FILL_PATTERNS_ROW
			mt: MASK_TECHNS_ROW
			wt: WAFER_TECHNS_ROW
			levels: LIST [RUN_LEVELS_ROW]
			level, first_level: RUN_LEVELS_ROW
			scale_factor, res: DOUBLE
			exclusion_levels: STRING
		do
			gen := session.file_generator
			fp := cache.fill_patterns_row
			mt := cache.mask_techns_row
			wt := cache.wafer_techns_row
			space := " "
			compact := su.boolean_as_yes_no (mt.compaction_allowed)
			-- Assuming one scale factor for all levels
			levels := cache.run_level_rows
			first_level := cache.run_level_rows.first
			scale_factor := first_level.scale_factor
			fill_scale_factor := nu.std_formatted_number (scale_factor / fp.scale_factor)
			create exclusion_levels.make (100)
			from
				levels.start
			until
				levels.after
			loop
				level := levels.item
				if level.exclusion_layer then
					if not exclusion_levels.is_empty then
						exclusion_levels.extend (';')
					end
					exclusion_levels.append (level.level_name)
					exclusion_levels.extend (',')
					exclusion_levels.append (level.non_null_mebes_extension.as_lower)
					exclusion_levels.extend (',')
					exclusion_levels.append (nu.std_formatted_number (level.non_null_merged_pattern_file_spot_size))
				end
				levels.forth
			end
			res := cache.level_lcm_resolution
			format := cache.level_mebes_format (first_level.level_no, res)
			Result := session.run_id + space +
				session.run_directory + space +
				gen.level_merge_root_name (1, 1).file_name  + space +
					-- Assuming one field
				gen.fill_exclusion_file.file_name + space +
				fp.file_name + space +
				gen.fill_reset_file.file_name + space +
				gen.fill_exclusion_zone_file.file_name + space +
				gen.fill_selector_file.file_name + space +
				gen.fill_selector_done_file.file_name + space +
				compact + space +
				format + space +
				fill_scale_factor + space +
				fp.cell_selection_gds_layer.out + space +
				nu.std_formatted_number (res) + space +
				nu.std_formatted_number (wt.fill_gap * scale_factor * 2) + space +
					-- Fill gap is per side at 1X
				nu.std_formatted_number (fp.cell_remove_sizing * scale_factor) + space +
				nu.std_formatted_number (fp.cell_merge_sizing * scale_factor) + space +
				su.boolean_as_yes_no (session.environment.is_production) + space +
				exclusion_levels
				
		end

	handle_submit_event (submitted_rows: LIST [FILL_SELECTOR_JOB])
			-- Handle job submission event
		local
			submit_event: FILL_SELECTOR_SUBMITTED_EVENT
		do
			create submit_event
			submit_event.set_jobs (submitted_rows)
			submit_event.publish
		end

	handle_release_event (released_rows: LIST [FILL_SELECTOR_JOB])
			-- Handle job release event
		local
			release_event: FILL_SELECTOR_RELEASED_EVENT
		do
			create release_event
			release_event.set_jobs (released_rows)
			release_event.publish
		end

	refresh_corrupted_rows
			-- Refresh requestor for elements of `all_rows'
			-- that were modified and became invalid due
			-- to a rollback
		local
			selector_job: FILL_SELECTOR_JOB
		do
			from
				all_rows.start
			until
				all_rows.after
			loop
				selector_job := all_rows.item
				if selector_job.fill_selector_jobs_row.is_corrupted then
					selector_job.set_fill_selector_jobs_row (fill_selector_rows.first)
				end
				
				all_rows.forth
			end
		
		end

	handle_changed_event (changed_rows: LIST [FILL_SELECTOR_JOB])
			-- Handle job status change event
		local
			changed_event: FILL_SELECTOR_CHANGED_EVENT
		do
			create changed_event
			changed_event.set_jobs (changed_rows)
			changed_event.publish
		end

	handle_all_passed_event
			-- Handle "all rows passed" event
		local
			done_event: FILL_SELECTOR_DONE_EVENT
		do
			create done_event
			done_event.publish
		end

	handle_failure_event (failed_rows, aborted_rows: LIST [FILL_SELECTOR_JOB])
			-- Handle job failure event
		local
			failed_event: FILL_SELECTOR_FAILED_EVENT
		do
			create failed_event
			failed_event.set_failed_jobs (failed_rows)
			failed_event.set_aborted_jobs (aborted_rows)
			failed_event.publish
		end

	handle_reset_event (reset_rows: LIST [FILL_SELECTOR_JOB])
			-- Handle job reset event
		local
			reset_event: FILL_SELECTOR_RESET_EVENT
		do
			create reset_event
			reset_event.set_jobs (reset_rows)
			reset_event.publish
		end

	row_to_generic (row: EV_MULTI_COLUMN_LIST_ROW): FILL_SELECTOR_JOB
			-- Value of `row' which is of type FILL_SELECTOR_JOB
		do
			Result ?= row
			check
				result_not_void: Result /= Void
			end
		end

	fill_selector_rows: LIST [RUN_JOBS_MROW]
			-- List of database rows for all fill selector
			-- jobs for run

end
