
indexing
	
	description: "Multi-column list row for a run and its %
		%associated fill selector file creation batch job"
	author: "David Hollenberg"
	date: "March 4, 2004"

class FILL_SELECTOR_JOB
inherit	
	JOB_MULTI_COLUMN_LIST_ROW
		rename
			make as mcl_make,
			requestor as fill_selector_jobs_row
		end
	HAS_JOB_TYPE
		rename
			job_type as cats_job_type
		undefine
			copy, is_equal, default_create
		end

create
	make

create {FILL_SELECTOR_JOB}
	make_filled

feature {NONE} -- Creation

	make (row: RUN_JOBS_MROW)
		require
			row_exists: row /= Void
		do
			default_create
			set_fill_selector_jobs_row (row)
			enable_visible
		ensure
			is_visible: is_visible
		end

feature -- Properties

	fill_selector_jobs_row: RUN_JOBS_MROW
			-- Row from run jobs table

	unique_identifier: STRING
			-- Value that uniquely identifies row
		do
			Result := cats_job_type
		end

	cats_job_type: STRING
			-- Descriptive name of level
		do
			Result := fill_selector_jobs_row.cats_job_type
		end

	job_type: STRING
			-- Type of job to submit
		local
			type: BATCH_JOB_TYPE_VALUES
		do
			Result := type.Fill_selector
		end

	job_name (a_run_id: STRING): STRING
			-- Name of job for `a_run_id'
		do
			Result := a_run_id + "_FILL_SELECTOR"
		end

	batch_queue_name: STRING
			-- Name of batch queue to use
		local
			queue: BATCH_JOB_QUEUE_VALUES
		do
			Result := queue.Die_fracture_batch
		end

feature -- Modification

	set_fill_selector_jobs_row (r: RUN_JOBS_MROW)
			-- Set `fill_selector_jobs_row' to `r'
		do
			fill_selector_jobs_row := r
		ensure
			row_set: fill_selector_jobs_row = r
		end

feature {NONE} -- Implementation

	need_pattern_file_stats: BOOLEAN = False
			-- Should information about pattern files be
			-- recorded in database?

	fill_in_pattern_file_row (row: PATTERN_FILES_MROW)
			-- Set rest of columns in `row' before it
			-- is inserted.  Not applicable
		do
		end

	existing_pattern_file_rows: LIST [PATTERN_FILES_MROW]
			-- List of pattern file rows in database
			-- created for `Current'
		do
			create {ARRAYED_LIST [PATTERN_FILES_MROW]} Result.make (0)
		end

	new_filled_list (n: INTEGER): like Current
			-- New list with `n' elements.
		do
			create Result.make_filled (n)
		end
		
end
