
indexing
	
	description: "A run closing step operation that deals with %
		%the Export Control report to be sent to the foundry."
	author: "David Hollenberg"
	date: "May 27, 2005"

deferred class FOUNDRY_ECCN_REPORT_OP
inherit
	RC_PRIMITIVE_STEP_OP
	NO_USER_INTERFACE

feature {NONE} -- Implementation

	is_applicable: BOOLEAN
			-- Is step applicable to run for session?
		do
			Result := cache.wafer_techn_families_row.is_eccn_required
		end

	set_state
			-- Set `state'
		do
			state := Void
		end

	eccn_report_output_file: RC_FILE
			-- Output file from running the foundry ECCN report
		do
			Result := file_generator.eccn_report_output_file
		ensure
			result_exists: Result /= Void
		end

	fetch_eccn_reported_event
			-- Fetch the ECCN REPORTED event, if any
		local
			list: LIST [RUN_EVENTS_MROW]
			row: RUN_EVENTS_MROW
			event_name: EVENT_NAME_VALUES
		do
			create row.make_template
			list := row.run_events_with_name (session.run_id, event_name.Eccn_reported, connection, expected_count.At_most_one)
			if not list.is_empty then
				eccn_reported_event := list.first
			else
				eccn_reported_event := Void
			end
		end

	delete_eccn_reported_event
			-- Delete the ECCN REPORTED event, if any
		local
			lock_op: ROW_LOCK_OP
		do
			if eccn_reported_event /= Void then
				create lock_op
				lock_op.lock_nowait (eccn_reported_event)
				eccn_reported_event.delete
			end
		end

	eccn_reported_event: RUN_EVENTS_MROW
			-- Row for ECCN REPORTED event

end
