
indexing
	
	description: "Section of fracture log file"
	author: "David Hollenberg"
	date: "December 9, 2003"

class FRACTURE_LOG_SECTION
inherit
	CATS_LOG_CONSTANTS

	STRING_UTILITIES_ACCESS
		export
			{NONE} all
		end

create
	make

feature {NONE} -- Creation

	make (a_start_tag, an_end_tag: STRING)
		require
			a_start_tag_exists: a_start_tag /= Void
			a_start_tag_nonempty: not a_start_tag.is_empty
			an_end_tag_exists: an_end_tag /= Void
			an_end_tag_nonempty: not an_end_tag.is_empty
		do
			start_tag := a_start_tag.as_upper
			end_tag := an_end_tag.as_upper
			disable_checked
			disable_density_present
			disable_figure_count_present
			disable_user_readfile_present
			disable_dc_check_present
		end

feature -- Properties

	start_tag: STRING
			-- Tag which indicates start of section

	end_tag: STRING
			-- Tag which indicates end of section

	is_checked: BOOLEAN
			-- Should section be checked for errors?

	is_density_present: BOOLEAN
			-- Is pattern file density information present
			-- in section?

	is_figure_count_present: BOOLEAN
			-- Is pattern file figure count information present
			-- in section?

	is_user_readfile_present: BOOLEAN
			-- Is this the initial readfile (or ciflib) pass
			-- over a submitted layout?

	is_dc_check_present: BOOLEAN
			-- Is design_check specific information present
			-- in section?

feature -- Modification

	enable_checked
			-- Enable checking of section
		do
			is_checked := True
		ensure
			checked: is_checked
		end

	disable_checked
			-- Disable checking of section
		do
			is_checked := False
		ensure
			not_checked: not is_checked
		end

	enable_density_present
			-- Set `is_density_present' to True
		do
			is_density_present := True
		ensure
			present: is_density_present
		end

	disable_density_present
			-- Set `is_density_present' to False
		do
			is_density_present := False
		ensure
			not_present: not is_density_present
		end

	enable_figure_count_present
			-- Set `is_figure_count_present' to True
		do
			is_figure_count_present := True
		ensure
			present: is_figure_count_present
		end

	disable_figure_count_present
			-- Set `is_figure_count_present' to False
		do
			is_figure_count_present := False
		ensure
			not_present: not is_figure_count_present
		end

	enable_user_readfile_present
			-- Set `is_user_readfile_present' to True
		do
			is_user_readfile_present := True
		ensure
			present: is_user_readfile_present
		end

	disable_user_readfile_present
			-- Set `is_user_readfile_present' to False
		do
			is_user_readfile_present := False
		ensure
			not_present: not is_user_readfile_present
		end

	enable_dc_check_present
			-- Set `is_dc_check_present' to True
		do
			is_dc_check_present := True
		ensure
			present: is_dc_check_present
		end

	disable_dc_check_present
			-- Set `is_dc_check_present' to False
		do
			is_dc_check_present := False
		ensure
			not_present: not is_dc_check_present
		end

end
