class FTP_TRANSFER
	-- an FTP_TRANSFERS_MROW with stuff that is probably only of
	--  interest to the Front End

inherit
	FTP_TRANSFERS_MROW
	undefine delete			-- take delete from MULTI_LINE_TEXT_ROW
	end;

	MULTI_LINE_TEXT_ROW[FTP_RETRIEVAL_MESSAGES_MROW]

	STRING_UTILITIES_ACCESS
		export
			{NONE} all
		end

	NFE_SUPPORT
		export
			{NONE} all
		end

	THREE_CHECKSUM_FLAVORS
	rename	cif_byte_count as cif_checksum_byte_count,
		set_cif_byte_count as set_cif_checksum_byte_count,
		set_null_cif_byte_count as set_null_cif_checksum_byte_count
	end;

create
	make_template, make_null, make_by_key, make_by_lock_filename

feature -- effectors for MULTI_LINE_TEXT_ROW

	create_numbered_row: FTP_RETRIEVAL_MESSAGES_MROW is
	do  create Result.make_null(connection);
	    Result.set_design_no(design_no);
	    Result.set_revision_no(revision_no);
	    Result.set_fragment_no(fragment_no);
	end;

	numbered_rows_filter: STRING is
	"design_no = :1 and revision_no = :2 and fragment_no = :3";

	numbered_rows_binder: DB_BIND_III is
	do  create Result.make(design_no, revision_no, fragment_no)
	end;

feature -- alternate creation

	make_by_lock_filename(conn: DB_CONNECTION; nam: STRING) is
		-- if row not found, you get an empty object
	local q: IDENTIFIED_QUERY[FTP_TRANSFER];
	do  create q.make(conn, Current);
	    q.set_filter_condition(su.string_of_3(
		"lock_filename = %'",nam,"%'"));
	    q.execute;
	    q.fetch_into(Current);
	    q.release
	end;

feature -- Properties

	handle: STRING is
		-- my primary key fields
	do  create Result.make(24);
	    Result.append("Dno:");
	    Result.append_integer(design_no);
	    Result.append(";Rev:");
	    Result.append_integer(revision_no);
	    Result.append(";Frag:");
	    Result.append_integer(fragment_no)
	end;

	transfer_lock_file_name: STRING is
			-- Name of lock file which must be locked 
			-- to get exclusive access to this transfer.
			-- Used to prevent independent ftp sessions from
			-- simultaneously working on the same "send" ftp
		do
			Result := su.string_of_2 (local_filename, Lock_suffix)
		end

	transfer_succeeded: BOOLEAN is
			-- Was transfer successful?  Always true for
			-- remotely initiated transfer, since this is
			-- only used by ftp accept request
		do
			Result := initiated_remotely or 
				equal (ftp_transfer_state, glo.frs_is_passed)
		end

	transfer_is_pending: BOOLEAN is
		-- is transfer currently in the pending state?
	do  Result := equal(ftp_transfer_state, glo.frs_is_pending)
	end;

	checksum_text: STRING is
			-- String with checksum values for presentation
			-- to user
		local
			pair, text_crc, bin_crc, cif: CHECKSUM_PAIR
		do
			text_crc := text_crc_sum
			bin_crc := binary_crc_sum
			cif := cif_sum
			if text_crc /= Void then
				pair := text_crc
			elseif bin_crc /= Void then
				pair := bin_crc
			elseif cif /= Void then
				pair := cif
			else
				pair := Void
			end
			if pair /= Void then
				Result := pair.out
			else
				Result := "<NONE>"
			end
		end

	checksum_type: STRING is
			-- String with type of checksum (text CRC, binary
			-- CRC or old-style "CIF") for presentation
			-- to user
		local
			text_crc, bin_crc, cif: CHECKSUM_PAIR
		do
			text_crc := text_crc_sum
			bin_crc := binary_crc_sum
			cif := cif_sum
			if text_crc /= Void then
				Result := "text CRC"
			elseif bin_crc /= Void then
				Result := "binary CRC"
			elseif cif /= Void then
				Result := "old-style %"CIF%""
			else
				Result := "unknown type"
			end
		end

	expiration_interval_text: STRING is
			-- Text indicating how long until remotely
			-- initiated transfer expires
		do
			Result := expiration_interval_text_for_seconds (expiration_interval)
		end

	expiration_interval_text_for_seconds (n: INTEGER): STRING is
			-- Text indicating how long until remotely
			-- initiated transfer will expire, for transfer
			-- that expires in `n' seconds
		local
			interval, hours, minutes, seconds: INTEGER
			
		do
			interval := n
			if interval < 0 then
				Result := "EXPIRED"
			else
				create Result.make (16)
				hours := interval // 3600
				minutes := (interval - hours * 3600) // 60
				seconds := interval - hours * 3600 - minutes * 60
				Result := su.string_of_3 (su.string_of_4 (hours.out, " h ", minutes.out, " min "), seconds.out, " s")
			end
		end

feature {NONE} -- Implementation

	Lock_suffix: STRING is "_lock"
			-- Suffix added to local filename to derive name
			-- of transfer lock file

end -- FTP_TRANSFER
