class FUNDING_TRIPLE_ROW
	-- every funding distribution, joined with its
	-- funding_alloc and its funding_auth

inherit
	FUNDING_TRIPLE_RAW
	ROW_COUNT_CHECK_ACCESS
		export
			{NONE} all
		end

	MOSIS_CACT_VALUES
		export {NONE} all end;

create
	make_template, make_null, make_dummy

feature -- status

	is_po_funded: BOOLEAN is
	do  Result := equal(sub_type, subtype_is_cust_po)
	end;

	is_usc_auth: BOOLEAN is
	do  Result := equal(sub_type, subtype_is_uscauth)
	end;

	is_blanket_po: BOOLEAN is
	do  Result := equal(po_type, po_type_is_blanket)
	end;

	is_own_money: BOOLEAN is
		-- is this pot of money (auth) reporting to the
		-- same cust_acct_no as the alloc?
	do  Result := (cust_acct_no = cust_acct_no_reporting)
		--     |              |
		--     |              + from funding auth
		--     + from funding_alloc
	end;

feature {NONE} -- creation

	make_dummy(alloc: INTEGER; sched: STRING) is
		-- special for project_order.find_funding_alloc
	do  make_template;
	    m_funding_alloc_no := alloc;
	    pricing_schedule := sched;
	end;

feature -- Collections

	usable_rows_for_account (acct_no: INTEGER a_date: STRING conn: DB_CONNECTION range: ITEM_COUNT_RANGE): LIST [like Current] is
			-- All funding triples of sub-type CUST_PO or
			-- USCAUTH for account number `acct_no' on
			-- distribution rank 1 whose funding auth is
			-- usable (has a positive amount authorized
			-- and a positive amount available) and whose
			-- alloc has only one distribution rank, and
			-- which were effective on `a_date', ordered
			-- by auth ID
		require
			a_date_not_void: a_date /= Void
			connection_exists: conn /= Void
			connection_open: conn.is_open
			range_exists: range /= Void
		local
			b: DB_BIND_ISSS
			type: FUNDING_AUTH_SUB_TYPE_VALUES
		do
			create b.make (acct_no, a_date, type.Cust_po, type.Uscauth)
			Result := all_rows_filtered_ordered (conn, "fal.cust_acct_no = :1 and fau.date_effective <= :2 and :2 <= nvl(fau.date_expires, :2) and fau.sub_type in (:3, :4) and fds.distribution_rank = 1 and fau.amount_authorized > 0 and fau.amount_authorized - fau.amount_unavailable >= 1 and 1 = (select count(*) from funding_distributions where m_funding_alloc_no = fal.m_funding_alloc_no)", b, "auth_id")
			check_row_count (Result, range)
		ensure
			result_exists: Result /= Void
		end


end
