class GDS_FILE_VALUES

inherit
	ANY
	STRING_UTILITIES_ACCESS
	export {NONE} all
	end;

feature	-- gds record types

	rec_HEADER: INTEGER is		 0
	rec_BGNLIB: INTEGER is		 1
	rec_LIBNAME: INTEGER is		 2
	rec_UNITS: INTEGER is		 3
	rec_ENDLIB: INTEGER is		 4
	rec_BGNSTR: INTEGER is		 5
	rec_STRNAME: INTEGER is		 6
	rec_ENDSTR: INTEGER is		 7
	rec_BOUNDARY: INTEGER is	 8
	rec_PATH: INTEGER is		 9
	rec_SREF: INTEGER is		10
	rec_AREF: INTEGER is		11
	rec_TEXT: INTEGER is		12
	rec_LAYER: INTEGER is		13
	rec_DATATYPE: INTEGER is	14
	rec_WIDTH: INTEGER is		15
	rec_XY: INTEGER is		16
	rec_ENDEL: INTEGER is		17
	rec_SNAME: INTEGER is		18
	rec_COLROW: INTEGER is		19
	rec_TEXTNODE: INTEGER is	20
	rec_NODE: INTEGER is		21
	rec_TEXTTYPE: INTEGER is	22
	rec_PRESENTATION: INTEGER is	23
	rec_SPACING: INTEGER is		24	-- discontinued
	rec_STRING: INTEGER is		25
	rec_STRANS: INTEGER is		26
	rec_MAG: INTEGER is		27
	rec_ANGLE: INTEGER is		28
	rec_UINTEGER: INTEGER is	29	-- no longer used
	rec_USTRING: INTEGER is		30	-- no longer used
	rec_REFLIBS: INTEGER is		31
	rec_FONTS: INTEGER is		32
	rec_PATHTYPE: INTEGER is	33
	rec_GENERATIONS: INTEGER is	34
	rec_ATTRTABLE: INTEGER is	35
	rec_STYPTABLE: INTEGER is	36	-- unreleased feature
	rec_STRTYPE: INTEGER is		37	-- unreleased feature
	rec_ELFLAGS: INTEGER is		38
	rec_ELKEY: INTEGER is		39	-- unreleased feature
	rec_LINKTYPE: INTEGER is	40	-- unreleased feature
	rec_LINKKEYS: INTEGER is	41	-- unreleased feature
	rec_NODETYPE: INTEGER is	42
	rec_PROPATTR: INTEGER is	43
	rec_PROPVALUE: INTEGER is	44
	rec_BOX: INTEGER is		45
	rec_BOXTYPE: INTEGER is		46
	rec_PLEX: INTEGER is		47
	rec_BGNEXTN: INTEGER is		48
	rec_ENDEXTN: INTEGER is		49
	rec_TAPENUM: INTEGER is		50
	rec_TAPECODE: INTEGER is	51
	rec_STRCLASS: INTEGER is	52
	rec_RESERVED: INTEGER is	53	-- reserved for future use
	rec_FORMAT: INTEGER is		54
	rec_MASK: INTEGER is		55
	rec_ENDMASKS: INTEGER is	56
	rec_LIBDIRSIZE: INTEGER is	57
	rec_SRFNAME: INTEGER is		58
	rec_LIBSECUR: INTEGER is	59

feature	-- gds data types (each record type "knows" its data type)

	dta_NONE: INTEGER is		 0
	dta_BITS: INTEGER is		 1
	dta_B2INT: INTEGER is		 2
	dta_B4INT: INTEGER is		 3
	dta_B4REAL: INTEGER is		 4	-- not used
	dta_B8REAL: INTEGER is		 5
	dta_ASCII: INTEGER is		 6

feature	-- gds version 

	version_early: INTEGER is	 0
	version_3: INTEGER is		 3
	version_4: INTEGER is		 4
	version_5: INTEGER is		 5
	version_6: INTEGER is		600	-- for 6.00

	is_known_version(vno: INTEGER): BOOLEAN is
	do  Result := vno = version_early or vno = version_3 or
			vno = version_4 or vno = version_5 or
			vno = version_6
	end;

feature	-- other gds constants and constructs

	gds_record_length: INTEGER is	2048	-- bytes

	gds_layer_limit: INTEGER is	255	-- officially, 63

	gds_datatype_limit: INTEGER is	255

	gds_path_limit: INTEGER is	200	-- max points in path

	gds_year_zero: INTEGER =	1900	-- year 0 in BGNLIB, BGNSTR

feature -- the bits of an STRANS bit array

	strans_reflection: INTEGER is	32768	-- 2 ^ 15
	strans_abs_mag: INTEGER is	    4	-- 2 ^ 2; not supported
	strans_abs_angle: INTEGER is	    2	-- 2 ^ 1; not supported

feature	-- a few powers of 2

	two_up_6: INTEGER is		   64
	two_up_7: INTEGER is		  128
	two_up_8: INTEGER is		  256
	two_up_15: INTEGER is		32768
	two_up_16: INTEGER is		65536

feature	-- names of the GDS record types (for publication purposes)

	GDS_record_type_name(rtyp: INTEGER): STRING is
	require reasonable_rtyp: 0 <= rtyp and rtyp <= 255;
	do  Result := GDS_record_names @ rtyp;
	    if Result = Void
	     then Result := su.string_of_3("UNKNOWN (", rtyp.out, ")")
	    end
	end;

feature	{NONE} -- names saved in an array for fast access

	GDS_record_names: ARRAY[STRING] is
	once create Result.make(0,255);
	     Result.put( "HEADER", rec_HEADER);
	     Result.put( "BGNLIB", rec_BGNLIB);
	     Result.put( "LIBNAME", rec_LIBNAME);
	     Result.put( "UNITS", rec_UNITS);
	     Result.put( "ENDLIB", rec_ENDLIB);
	     Result.put( "BGNSTR", rec_BGNSTR);
	     Result.put( "STRNAME", rec_STRNAME);
	     Result.put( "ENDSTR", rec_ENDSTR);
	     Result.put( "BOUNDARY", rec_BOUNDARY);
	     Result.put( "PATH", rec_PATH);
	     Result.put( "SREF", rec_SREF);
	     Result.put( "AREF", rec_AREF);
	     Result.put( "TEXT", rec_TEXT);
	     Result.put( "LAYER", rec_LAYER);
	     Result.put( "DATATYPE", rec_DATATYPE);
	     Result.put( "WIDTH", rec_WIDTH);
	     Result.put( "XY", rec_XY);
	     Result.put( "ENDEL", rec_ENDEL);
	     Result.put( "SNAME", rec_SNAME);
	     Result.put( "COLROW", rec_COLROW);
	     Result.put( "TEXTNODE", rec_TEXTNODE);
	     Result.put( "NODE", rec_NODE);
	     Result.put( "TEXTTYPE", rec_TEXTTYPE);
	     Result.put( "PRESENTATION", rec_PRESENTATION);
	     Result.put( "SPACING", rec_SPACING);
	     Result.put( "STRING", rec_STRING);
	     Result.put( "STRANS", rec_STRANS);
	     Result.put( "MAG", rec_MAG);
	     Result.put( "ANGLE", rec_ANGLE);
	     Result.put( "UINTEGER", rec_UINTEGER);
	     Result.put( "USTRING", rec_USTRING);
	     Result.put( "REFLIBS", rec_REFLIBS);
	     Result.put( "FONTS", rec_FONTS);
	     Result.put( "PATHTYPE", rec_PATHTYPE);
	     Result.put( "GENERATIONS", rec_GENERATIONS);
	     Result.put( "ATTRTABLE", rec_ATTRTABLE);
	     Result.put( "STYPTABLE", rec_STYPTABLE);
	     Result.put( "STRTYPE", rec_STRTYPE);
	     Result.put( "ELFLAGS", rec_ELFLAGS);
	     Result.put( "ELKEY", rec_ELKEY);
	     Result.put( "LINKTYPE", rec_LINKTYPE);
	     Result.put( "LINKKEYS", rec_LINKKEYS);
	     Result.put( "NODETYPE", rec_NODETYPE);
	     Result.put( "PROPATTR", rec_PROPATTR);
	     Result.put( "PROPVALUE", rec_PROPVALUE);
	     Result.put( "BOX", rec_BOX);
	     Result.put( "BOXTYPE", rec_BOXTYPE);
	     Result.put( "PLEX", rec_PLEX);
	     Result.put( "BGNEXTN", rec_BGNEXTN);
	     Result.put( "ENDEXTN", rec_ENDEXTN);
	     Result.put( "TAPENUM", rec_TAPENUM);
	     Result.put( "TAPECODE", rec_TAPECODE);
	     Result.put( "STRCLASS", rec_STRCLASS);
	     Result.put( "RESERVED", rec_RESERVED);
	     Result.put( "FORMAT", rec_FORMAT);
	     Result.put( "MASK", rec_MASK);
	     Result.put( "ENDMASKS", rec_ENDMASKS);
	     Result.put( "LIBDIRSIZE", rec_LIBDIRSIZE);
	     Result.put( "SRFNAME", rec_SRFNAME);
	     Result.put( "LIBSECUR", rec_LIBSECUR);
	end;

end -- GDS_FILE_VALUES
