
indexing
	
	description: "Multi-column list row for a die or field and%
		%associated GDS merge batch job"
	author: "David Hollenberg"
	date: "October 9, 2008"

deferred class GDS_MERGE_JOB
inherit	
	JOB_MULTI_COLUMN_LIST_ROW
		rename
			make as mcl_make,
			requestor as run_gds_files_row
		redefine
			set_job_status_created, submit_job, has_job_done_file
		end
	HAS_DIE_ID
		undefine
			copy, is_equal, default_create
		end
	HAS_FIELD_NAME
		undefine
			copy, is_equal, default_create
		end

feature {NONE} -- Creation

	make (gds_row: RUN_GDS_FILES_MROW)
		require
			gds_row_not_void: gds_row /= Void
		do
			default_create
			set_run_gds_files_row (gds_row)
			enable_visible
		ensure
			is_visible: is_visible
		end

feature -- Properties

	run_gds_files_row: RUN_GDS_FILES_MROW
			-- Row from run GDS files table

	unique_identifier: STRING
			-- Value that uniquely identifies row
		do
			Result := run_gds_files_row.file_no.out
		end

	is_die_merge: BOOLEAN
			-- Is this a per-die merge rather than
			-- a per-field merge?
		do
			Result := not run_gds_files_row.is_null_die_id
		end

	is_field_merge: BOOLEAN
			-- Is this a per-field merge rather than
			-- a per-die merge?
		do
			Result := not is_die_merge
		end

	file_no: INTEGER
			-- File number
		do
			Result := run_gds_files_row.file_no
		end

	die_id: STRING
			-- Die id
		do
			Result := run_gds_files_row.die_id
		end

	field_name: STRING
			-- Name of field to be merged
		do
			Result := run_gds_files_row.field_name
		end

	has_job_done_file: BOOLEAN = False
			-- Does job create a "done" file to indicate
			-- that it completed?

	job_type: STRING
			-- Type of job to submit
		local
			type: BATCH_JOB_TYPE_VALUES
		do
			Result := type.Gds_merge
		end

	batch_queue_name: STRING
			-- Name of batch queue to use
		local
			queue: BATCH_JOB_QUEUE_VALUES
		do
			Result := queue.Gds_merge_batch
		end

feature -- Modification

	set_run_gds_files_row (r: like run_gds_files_row)
			-- Set `run_gds_files_row' to `r'
		do
			run_gds_files_row := r
		ensure
			run_gds_files_row_set: run_gds_files_row = r
		end

	set_job_status_created
			-- Set job status to CREATED for `requestor'
		do
			precursor
			initialize_job_and_results
		end

	submit_job (a_run_id, a_state: STRING script_path, log_path, args: STRING)
			-- Submit fracture job as part of run
			-- `a_run_id' and put it in
			-- `a_state'.  Job will execute `script_path'
			-- with `args' and put output in `log_path'
		do
			precursor (a_run_id, a_state, script_path, log_path, args)
			initialize_job_and_results
		end

feature {NONE} -- Implementation

	initialize_job_and_results
			-- Clear fields whose value is determined after
			-- the GDS merge job runs.  Delete merge result
			-- row and merge layer rows if they exist
		local
			merge_result: RUN_GDS_MERGE_RESULTS_MROW
			merge_layers: LIST [RUN_GDS_FILE_LAYERS_MROW]
			lock_op: ROW_LOCK_OP
		do
			run_gds_files_row.set_null_top_structure
			run_gds_files_row.set_null_size_x
			run_gds_files_row.set_null_size_y
			run_gds_files_row.set_null_center_x
			run_gds_files_row.set_null_center_y
			create lock_op
			merge_result := run_gds_files_row.merge_result
			if merge_result /= Void then
				lock_op.lock_nowait (merge_result)
				merge_result.delete
			end
			merge_layers := run_gds_files_row.merge_layers
			lock_op.lock_list_nowait (merge_layers)
			merge_layers.do_all (agent {RUN_GDS_FILE_LAYERS_MROW}.delete)
		end

	gds_merge_job_base: STRING = "_GDS_MERGE_"
			-- Base part of job name

	need_pattern_file_stats: BOOLEAN = False
			-- Should information about pattern files be
			-- recorded in database?  (No pattern files
			-- will be generated)

	fill_in_pattern_file_row (row: PATTERN_FILES_MROW)
			-- Set rest of columns in `row' before it
			-- is inserted
		do
			-- Inapplicable
		end

	existing_pattern_file_rows: LIST [PATTERN_FILES_MROW]
			-- List of pattern file rows in database
			-- created for `Current'
		do
			create {ARRAYED_LIST [PATTERN_FILES_MROW]} Result.make (0)
		end

end
