class GDS_TO_LAYOUT
	-- read a gds DATA_SOURCE (usually a file) to
	-- produce a LAYOUT_LIBRARY of its contents
	--
	-- Usage is	make(data_source)
	--		set_the_techn(layer_set)
	--		optionally, create the_lib
	--	either	read_gds_library
	--	or	read_gds_header
	--		if header_is_OK		-- and you want the body
	--		 then read_gds_body
	--		end
	-- Output is	the_messages: RESPONSE_COLLECTOR
	--	and	the_lib: LAYOUT_LIBRARY
		
inherit

	GDS_FILE_READER

create --
	make		-- in GDS_FILE_READER

feature	-- scale_factor

	scale_factor: DOUBLE
		-- scale dimensional data (XY and WIDTH) by this factor
		-- which is ratio of file's dbu to the_lib.unit_size
		-- (always 1.0 unless the_lib is externally created)

feature	{NONE} -- additional (semantic) errors 

	error_is_igntxt: STRING is "IGNTXT"
	error_is_ignnod: STRING is "IGNNOD"
	error_is_ignbox: STRING is "IGNBOX"
	error_is_mvaref: STRING is "MVAREF"
	error_is_xparef: STRING is "XPAREF"
	error_is_xxaref: STRING is "XXAREF"

feature {NONE} -- helpers

	add_to_current_layout(lay: LAYOUT) is
		-- add an item to the current_body (if legal)
	do  curr_body.extend(lay)
	end;

feature {NONE} -- the deferred routines to process the library structures

	begin_gds_library
	local dbu: DOUBLE
	do -- make sure we have a library
	   if the_lib = Void then
		create the_lib.make(library_name, 0);
	   else	
		the_lib.set_name(library_name);
	   end;
	   -- make sure it has a valid unit size
	   if the_lib.unit_size = 0 then
		if lioopt.input_library_unit = 0 then
			dbu := database_unit	-- use file's own unit
		else
			dbu := lioopt.input_library_unit
						-- use requested unit
		end;
		the_lib.resize(dbu)
	   end;
	   scale_factor := database_unit / the_lib.unit_size;
		-- multiply file dimensional numbers by scale_factor
		--  to get library numbers
	   the_lib.set_user_units(user_unit / scale_factor)
	   the_lib.set_date_modified(modification_date);
	   the_lib.set_date_accessed(access_date);
	   set_other;
	   create test_polygon.make(4)
	end;

	end_gds_library is
		-- it's all over
	do
	end;

	begin_gds_structure(snam: STRING; cdat, mdat: DB_DATE) is
		-- snam is the structure name
		-- cdat and mdat re dates of creation and
		--  last modification, respectively
	do  -- create new structure
	    create curr_structure.make(snam);
	    curr_structure.set_date_created(cdat);
	    curr_structure.set_date_modified(mdat);
	    create curr_body.make(40)
	    curr_body.set_is_caching_bb(true)
	end;

	end_gds_structure is
		-- marks end of a structure
	do  curr_body.trim(100)
	    curr_structure.set_body(curr_body)
	    the_lib.add_structure(curr_structure);
	    curr_body := Void;
	    curr_structure := Void
	end;

	gds_layer(layer, datatype, rectype: INTEGER) is
		-- layer and datatype for whatever follows
	local lyr: LAYER
	do  -- FIX_ME: check rectype to see if this is a saved item
	    lyr := layer_by_gds(layer, datatype);
	    curr_body.set_current_layer(lyr.layer_name)
	end;

	gds_boundary(pts: INTEGER) is
		-- actual XY values are still in curr_int_array
	local p: LAYOUT_POLYGON
	do  if pts = 4 then
		test_polygon.wipe_out;
		XY_to_lop(test_polygon, 4)
		if test_polygon.is_rectangular then
			add_to_current_layout
					(test_polygon.to_layout_rectangle)
		else	-- not a rectangle; copy to new polygon
			create p.make_copy(test_polygon)
		end
	    else	-- not a rectangle; create new polygon
		create p.make(pts);
		XY_to_lop(p, pts)
	    end;
	    if p /= Void then	-- p = Void --> we found and saved a rectangle
		add_to_current_layout(p)
	    end
	end;

	gds_path(wid: INTEGER; e: PATH_OPTIONS; pts: INTEGER) is
		-- actual XY values are still in curr_int_array
	local w: LAYOUT_PATH
	do  create w.make(scale_factor * wid, e.scaled_by(scale_factor))
	    XY_to_lop(w, pts)
	    add_to_current_layout(w)
	end;

	gds_sref(snam: STRING; strans: MATRIX; pt1: POINT) is
		-- pt1 NOT in strans
	local r: LAYOUT_REFERENCE;
	do  create r.make_sref(snam);
	    final_reference_location(r, strans, pt1);
	end;

	gds_aref(snam: STRING; strans: MATRIX; ncol, nrow: INTEGER;
			pt1, dpt2, dpt3: POINT) is
	local r: LAYOUT_REFERENCE; mag: DOUBLE; sew: BOOLEAN;
		colstp, rowstp, newpt1: POINT; cl, rw: INTEGER;
	do  mag := strans.magnification / scale_factor;
	    sew := same_either_way(strans, ncol, nrow, dpt2, dpt3);
	    if lioopt.te_gds2_arrays or sew
	     then create r.make_aref(snam, ncol, nrow,
				dpt2.rho / mag / ncol,
				dpt3.rho / mag / nrow)
		  final_reference_location(r, strans, pt1)
		  if not sew
		   then log_io_error(error_is_dfaref, snam)
		  end;
	    elseif translatable_to_Calma(strans, ncol, nrow)
	     then if switch_cols_and_rows
		   then create r.make_aref(snam, nrow, ncol,
				dpt3.rho / mag / nrow,
				dpt2.rho / mag / ncol)
		   else create r.make_aref(snam, ncol, nrow,
				dpt2.rho / mag / ncol,
				dpt3.rho / mag / nrow)
		  end;
		  final_reference_location(r, strans, pt1);
		  log_io_error(error_is_mvaref, snam)
	    elseif ncol*nrow <= lioopt.aref_expansion_limit
	     then colstp := dpt2 * (scale_factor / ncol);
		  rowstp := dpt3 * (scale_factor / nrow);
		  from  cl := 0
		  until cl >= ncol
		  loop	newpt1 := (pt1 * scale_factor) + (colstp * cl)
			from  rw := 0
			until rw >= nrow
			loop  gds_sref(snam, strans, newpt1);
			      newpt1.move_by(rowstp.x, rowstp.y)
			      rw := rw + 1
			end;
			cl := cl + 1
		  end;
		  log_io_error(error_is_xparef, snam)
	     else log_io_error(error_is_xxaref, snam)
	    end;
	end;

	gds_text(pres, ptyp, wid: INTEGER; strans: MATRIX;
		 xy: VIRTUAL_POINT; text: STRING) is
	local ltxt: LAYOUT_TEXT; 
	do  if lioopt.preserve_gds_text then
		create ltxt.make(pres, pathtype_ends(ptyp), wid, text);
		final_reference_location(ltxt, strans, xy)
	    else
		log_io_error(error_is_igntxt, Void)
	    end
	end;

	gds_node(pts: INTEGER) is
		-- actual XY values are still in curr_int_array
	do  log_io_error(error_is_ignnod, Void)
	end;

	gds_box(pts: INTEGER) is
		-- actual XY values are still in curr_int_array
	do  if lioopt.boxes_are_layout then
		gds_boundary(pts)
	    else
		log_io_error(error_is_ignbox, Void)
	    end
	end;

feature	{NONE} -- helpers (for AREF, mainly)

	test_polygon: LAYOUT_POLYGON

	XY_to_lop(lop: LIST_OF_POINTS; cnt: INTEGER) is
		-- XY is in curr_int_array; lop is wire or rectangle
	local i: INTEGER;
	do  from  i := 1
	    until i > cnt
	    loop  lop.extend_XY(scale_factor * (curr_int_array @ (2*i-1)),
				scale_factor * (curr_int_array @ (2*i)));
		  i := i + 1
	    end
	end;

	final_reference_location(r: LAYOUT; strans: MATRIX;
				 pt1: VIRTUAL_POINT) is
		-- relocate r by (strans and_then pt1)
	local at: MATRIX; loc: LAYOUT_RELOCATION;
	do  create at.make_translation(pt1 * scale_factor);
	    at := strans.and_then(at);
	    if at.is_identity
	     then add_to_current_layout(r)
	     else create loc.make(r, at);
		  add_to_current_layout(loc)
	    end
	end;

	translatable_to_Calma(strans: MATRIX; ncol, nrow: INTEGER)
					: BOOLEAN is
		-- if true, we can represent this guy by switching his
		-- rows and columns and/or by moving his origin point to
		-- a different lattice point.
		-- if we return true, then switch_cols_and_rows and an
		-- updated ref_pt1 are correct.
	local
	do
	end;

	switch_cols_and_rows: BOOLEAN;

end -- GDS_TO_LAYOUT
