
indexing
	
	description: "A run closing step operation that generates PostScript %
		%and PDF files for the cut patterns on a run, with %
		%one page for each cut pattern"
	author: "David Hollenberg"
	date: "January 20, 2005"

class GEN_CUT_PATTERNS_MAP_OP
inherit
	GEN_MAP_OP
		redefine
			do_step, undo_step
		end
	OS_ACCESS
		export
			{NONE} all
		undefine
			default_create
		end
	CUT_MAP_UTILITIES
		export
			{NONE} all
		undefine
			default_create
		end

create
	make

feature -- Properties

	input_data_items: LIST [STRING]
			-- Data items required before step can be executed
		do
			Result := array_as_list (<<
				data_item.Field_set,
				data_item.Cut_patterns,
				data_item.Production_job
				>>)
		end

feature {NONE} -- Implementation

	do_step
			-- Execute step
		do
			fetch_cut_pattern_data
			precursor
		end

	undo_step
			-- Undo execution of step
		do
			fetch_cut_pattern_data
			precursor
		end

	generate_maps
			-- Generate PostScript and PDF files for map(s)
		do
			cache.packaging_lot_rows.do_all (agent generate_map_for_packaging_lot)
			if cache.cut_pattern_rows.there_exists (agent {RUN_CUT_PATTERNS_ROW}.is_unassigned) then
				-- Generate map for cut patterns not assigned
				-- to any packaging lot
				generate_map_for_packaging_lot (Void)
			end
		end

	generate_map_for_packaging_lot (lot: RUN_PACKAGING_LOTS_ROW)
			-- Generate PostScript and PDF files for cut patterns
			-- for `lot' (or for unassigned cut patterns if `lot'
			-- is Void) if it has any associated cut patterns
		local
			ps, pdf: RC_FILE
			cut_map: CUT_PATTERNS_MAP
		do
			process_pending_abort
			cut_map := map_generator.cut_patterns_map (lot)
			if not cut_map.cut_patterns.is_empty then
				map := cut_map
				ps := postscript_file_for_lot (lot)
				pdf := pdf_file_for_lot (lot)
				generate_postscript_file (cut_map, ps.file_pathname)
				convert_postscript_to_pdf (ps.file_pathname,
					pdf.file_pathname)
			end
		end

	delete_maps
			-- Delete map files
		do
			cache.packaging_lot_rows.do_all (agent delete_maps_for_packaging_lot)
			if cache.cut_pattern_rows.there_exists (agent {RUN_CUT_PATTERNS_ROW}.is_unassigned) then
				-- Delete maps for cut patterns not assigned
				-- to any packaging lot
				delete_maps_for_packaging_lot (Void)
			end
			delete_all_cut_maps
		end

	delete_maps_for_packaging_lot (lot: RUN_PACKAGING_LOTS_ROW)
			-- Delete PostScript and PDF files for cut patterns
			-- for `lot' (or for unassigned cut patterns if `lot'
			-- is Void)
		do
			postscript_file_for_lot (lot).delete_if_exists
			pdf_file_for_lot (lot).delete_if_exists
		end

	delete_all_cut_maps
			-- Delete all cut pattern maps that are left
			-- (files that match a particular wildcard string).
			-- There won't be any unless packaging lots
			-- were deleted from database or were changed in such
			-- a way that map file name changed.
		local
			open_op: DIR_OPEN_READ_OP
			close_op: DIR_CLOSE_OP
			delete_op: FILE_DELETE_OP
			dir: LARGE_DIRECTORY
			ps_file, pdf_file: RC_FILE
			fname, pat1, pat2: STRING
		do
			ps_file := file_generator.cut_patterns_ps_wildcard
			pdf_file := file_generator.cut_patterns_pdf_wildcard
			pat1 := ps_file.file_name
			pat2 := pdf_file.file_name
			create open_op
			create close_op
			create delete_op
			create dir.make (ps_file.directory_pathname)
				-- Assume PostScript and PDF files in same dir
			open_op.open (dir)
			from
                                dir.start
                                dir.readentry
                        until
                                dir.lastentry = Void
                        loop
				fname := dir.lastentry
				if su.wildcard_matches (fname, pat1) or su.wildcard_matches (fname, pat2) then
					delete_op.delete (os.full_file_name (dir.name, fname))
				end
                                dir.readentry
                        end

			close_op.close (dir)
		end

	packaging_lots: LIST [RUN_PACKAGING_LOTS_ROW]
			-- List of packaging lots for run

end
