
indexing
	
	description: "A run closing step operation that generates %
		%files for dies (geometry files or scripts) and takes %
		%sufficiently long that we want to display a progress window"
	author: "David Hollenberg"
	date: "May 22, 2006"

deferred class GEN_DIE_FILES_OP
inherit
	RC_PRIMITIVE_STEP_OP
	NO_USER_INTERFACE
	CREATES_EXTERNAL_STATE
	GUI_UTILITIES_ACCESS
		export
			{NONE} all
		undefine
			default_create
		end
	USER_INTERFACE_ACCESS
		export
			{NONE} all
		undefine
			default_create
		end

feature {NONE} -- Implementation

	is_applicable: BOOLEAN
			-- Is step applicable to run for session?
		do
			Result := cache.run_needs_mebes_files
		end
		
	do_step
			-- Execute step
		local
			dies: LIST [like anchored_die]
		do
			dies := dies_needing_file_generation (cache.die_mxrows)
			total_item_count := dies.count
			finished_item_count := 0
			if total_item_count > 0 then
				generate_all_die_files (dies)
			end
			call_completion_actions
		end

	undo_step
			-- Undo execution of step
		local
			dies: LIST [like anchored_die]
		do
			dies := dies_needing_file_deletion (cache.die_mxrows)
			dies.do_all (agent delete_generated_file)
			call_completion_actions
		end

	generate_all_die_files (dies: LIST [like anchored_die])
			-- Generate files for `dies'
		do
			show_progress_window
			dies.do_all (agent do_one_die)
			update_progress_window ("")
			gui.application.sleep (300)
			progress_window.destroy
		end

	do_one_die (r: like anchored_die)
			-- Process die `r'
		do
			update_progress_window (r.die_id)
			generate_file_for_die (r)
			finished_item_count := finished_item_count + 1
		end

	generate_file_for_die (r: like anchored_die)
			-- Generate file for `r'
		deferred
		end

	show_progress_window
			-- Create and display progress window
		local
			vbox: EV_VERTICAL_BOX
			box: EV_BOX
			cols: WIDGET_COLUMN_SET
			buttons: HORIZONTAL_BUTTON_SET
		do
			create progress_window
			progress_window.set_title (progress_window_title + " for " + session.run_id)
			progress_window.set_width (450)
			create vbox
			vbox.set_padding (user_interface.padding)
			create cols.make (2)
			cols.set_columns_padding (5)
			cols.set_row_padding (user_interface.padding)
			create die_id_box
			create remaining_box
			cols.append_row (<< gui.label_with_text ("Die ID:"), die_id_box >>)
			cols.append_row (<< gui.label_with_text ("Remaining:"), remaining_box >>)
			cols.disable_column_expand (1)
			cols.align_column_left (1)
			cols.align_column_left (2)
			box := cols.box
			vbox.extend (box)
			vbox.disable_item_expand (box)
			create progress_bar
			progress_bar.value_range.resize_exactly (0, total_item_count)
			progress_bar.set_value (0)
			progress_bar.disable_segmentation
			vbox.extend (progress_bar)
			vbox.disable_item_expand (progress_bar)
			
			create buttons.make_with_text_and_actions (<< gui.ev_cancel >>, << agent request_cancel >>)
			box := buttons.column_set.box
			vbox.extend (box)
			vbox.disable_item_expand (box)
			
			vbox.set_border_width (user_interface.border_width)
			progress_window.extend (vbox)
			progress_window.show_relative_to_window (session.application.current_window)
			gui.process_events
		end

	update_progress_window (die: STRING)
			-- Update progress window to show current info
		do
			die_id_box.wipe_out
			die_id_box.extend (gui.label_with_text (die))
			remaining_box.wipe_out
			remaining_box.extend (gui.label_with_text (remaining_item_count.out))
			progress_bar.set_value (finished_item_count)
			gui.process_events
			if progress_window = Void then
				-- User cancelled generation of files
				abort
			end
		end

	request_cancel
			-- Process user cancellation request
		do
			if progress_window /= Void then
				progress_window.destroy
			end
			progress_window := Void
		end

	set_state
			-- Set `state'
		do
			state := Void
		end

	progress_window: EV_DIALOG
			-- Window to show progress of item processing

	progress_bar: EV_HORIZONTAL_PROGRESS_BAR
			-- Bar to show progress of item processing
			-- graphically

	die_id_box: EV_HORIZONTAL_BOX
			-- Box for displaying die id

	remaining_box: EV_HORIZONTAL_BOX
			-- Box for displaying number of items remaining

	total_item_count: INTEGER
			-- Total number of items to be processed

	finished_item_count: INTEGER
			-- Number of items finished

	remaining_item_count: INTEGER
			-- Remaining number of items to be processed
		do
			Result := total_item_count - finished_item_count
		end

	is_cancel_requested: BOOLEAN
			-- Did user request cancellation of operation?

	anchored_die: DIES_XROW
			-- For anchoring only
		do
		ensure
			result_void: Result = Void
		end

feature {NONE} -- Deferred implementation

	dies_needing_file_generation (dies: LIST [like anchored_die]): LIST [like anchored_die]
			-- Dies in `dies' for which file needs
			-- to be generated
		deferred
		ensure
			result_not_void: Result /= Void
		end

	dies_needing_file_deletion (dies: LIST [like anchored_die]): LIST [like anchored_die]
			-- Dies in `dies' for which file needs
			-- to be deleted
		deferred
		ensure
			result_not_void: Result /= Void
		end

	has_valid_type (r: like anchored_die): BOOLEAN
			-- Does `r' have a valid type for the
			-- particular file generation operation to be
			-- performed?
		deferred
		end

	delete_generated_file (die: like anchored_die)
			-- Generate geometry file generated for `die',
			-- if it exists
		require
			die_not_void: die /= Void
			valid_type: has_valid_type (die)
		deferred
		end

	progress_window_title: STRING
			-- Title for progress window (without run id)
		deferred
		ensure
			result_not_void: Result /= Void
		end

end
