
indexing
	
	description: "A run closing step operation that generates script %
		%files for dies and takes sufficiently long that we want to %
		%display a progress window"
	author: "David Hollenberg"
	date: "May 22, 2006"

class GEN_DIE_FRACTURE_SCRIPTS_OP
inherit
	GEN_DIE_FILES_OP
		redefine
			anchored_die
		end

create
	make
feature -- Properties

	input_data_items: LIST [STRING]
			-- Data items required before step can be executed
		do
			Result := array_as_list (<<
				data_item.Mask_techn,
				data_item.Assigned_die_ids,
				data_item.Initial_wafer_lots,
				data_item.Test_vehicle_fab_instances
					-- Depends on fab instances so that
					-- we can get effective overhead style
					-- and design die offset from 
					-- die_attributes
				>>)
		end

feature {NONE} -- Implementation

	dies_needing_file_generation (dies: LIST [like anchored_die]): LIST [like anchored_die]
			-- Dies in `dies' for which geometry file needs
			-- to be generated
		do
			create {ARRAYED_LIST [like anchored_die]} Result.make (dies.count)
			dies.do_if (agent Result.extend, agent {like anchored_die}.still_needs_fracture_script)
		end

	dies_needing_file_deletion (dies: LIST [like anchored_die]): LIST [like anchored_die]
			-- Dies in `dies' for which geometry file needs
			-- to be generated
		do
			create {ARRAYED_LIST [like anchored_die]} Result.make (dies.count)
			dies.do_if (agent Result.extend, agent {like anchored_die}.needs_fracture_script)
		end

	has_valid_type (r: like anchored_die): BOOLEAN
			-- Does `r' have a valid type for the
			-- particular file generation operation to be
			-- performed?
		do
			Result := r.needs_fracture_script
		end

	generate_file_for_die (r: like anchored_die)
			-- Generate fracture file for `r', then mark it
			-- as CREATED
		local
			wtf: WAFER_TECHN_FAMILY
			status: JOB_STATUS_VALUES
		do
			wtf := session.wafer_techn_family_for_wtf (cache.wafer_techn_family)
			wtf.write_fracture_script (r)
			r.set_job_status (status.Created)
			session.commit
		end

	delete_generated_file (die: like anchored_die)
			-- Generate geometry file generated for `die',
			-- if it exists
		local
			status: JOB_STATUS_VALUES
		do
			file_generator.die_fracture_script_file (die.die_id).delete_if_exists
			if not die.job_status.is_equal (status.Not_created) then
				die.set_job_status (status.Not_created)
			end
		end

	progress_window_title: STRING = "Die Fracture Script Generation"
			-- Title for progress window (without run id)

	anchored_die: DIES_MXROW
			-- For anchoring only
		do
		end

end
