
indexing
	
	description: "A run closing step operation that creates one vendor %
		%geometry CIF or GDS file for each vendor geometry whose %
		%layout must be generated dynamically"
	author: "David Hollenberg"
	date: "July 18, 2006"

class GEN_DYNAMIC_VEND_GEOM_FILES_OP
inherit
	GEN_DIE_FILES_OP
		redefine
			is_applicable
		end

create
	make

feature -- Properties

	input_data_items: LIST [STRING]
			-- Data items required before step can be executed
		do
			Result := array_as_list (<<
				data_item.Assigned_die_ids
				>>)
		end

feature {NONE} -- Implementation

	is_applicable: BOOLEAN = True
			-- Is step applicable to run for session?
		
	dies_needing_file_generation (dies: LIST [like anchored_die]): LIST [like anchored_die]
			-- Dies in `dies' for which geometry file needs
			-- to be generated
		local
			wtf: WAFER_TECHN_FAMILY
		do
			wtf := session.wafer_techn_family_for_wtf (cache.wafer_techn_family)
			create {ARRAYED_LIST [like anchored_die]} Result.make (10)
			dies.do_if (agent Result.extend, agent needs_dynamic_vendor_geometry (?, wtf))
		end

	dies_needing_file_deletion (dies: LIST [like anchored_die]): LIST [like anchored_die]
			-- Dies in `dies' for which geometry file needs
			-- to be deleted
		do
			Result := dies_needing_file_generation (dies)
		end

	has_valid_type (r: like anchored_die): BOOLEAN
			-- Does `r' have a valid type for the
			-- particular file generation operation to be
			-- performed?
		do
			Result := r.is_vendor_geometry
		end

	generate_file_for_die (r: like anchored_die)
			-- Generate vendor geometry file for `r'
		local
			wtf: WAFER_TECHN_FAMILY
		do
			wtf := session.wafer_techn_family_for_wtf (cache.wafer_techn_family)
			wtf.generate_dynamic_vendor_geometry (r)
		end

	delete_generated_file (die: like anchored_die)
			-- Generate geometry file generated for `die',
			-- if it exists
		do
			file_generator.dynamic_vendor_geometry_file (die.geometries_row, cache.substitution_parameter_set).delete_if_exists
		end

	progress_window_title: STRING = "Dynamic Vendor Geometry Generation"
			-- Title for progress window (without run id)

feature {NONE} -- Implementation

	needs_dynamic_vendor_geometry (d: like anchored_die wtf: WAFER_TECHN_FAMILY): BOOLEAN
			-- Does `d' need dynamically generated vendor
			-- geometry file?  Dynamic generation may be
			-- need because layout contents cannot be
			-- determined until some earlier steps are
			-- completed, such as vendor part number
			-- selection
		require
			d_not_void: d /= Void
			wtf_not_void: wtf /= Void
		do
			Result := d.is_vendor_geometry and then wtf.is_dynamic_vendor_geometry (d)
		end

end
