
indexing
	
	description: "A run closing step operation that generates %
		%a fill area selector file, which is a .cflt file that %
		%represents areas that will be filled"
	author: "David Hollenberg"
	date: "March 3, 2004"

class GEN_FILL_AREA_SELECTOR_OP
inherit
	RC_PRIMITIVE_STEP_OP
		redefine
			is_currently_undoable, not_undoable_reason
		end
	HAS_USER_INTERFACE
	CREATES_EXTERNAL_STATE
	GUI_UTILITIES_ACCESS
		export
			{NONE} all
		undefine
			default_create
		end

create
	make

feature -- Properties

	input_data_items: LIST [STRING]
			-- Data items required before step can be executed
		do
			Result := array_as_list (<<
				data_item.Fill_exclusion_file,
				data_item.Merged_mebes_files
				>>)
		end

feature {NONE} -- Implementation

	is_applicable: BOOLEAN
			-- Is step applicable to run for session?
		do
			Result := cache.run_needs_fill
		end

	is_currently_undoable: BOOLEAN
			-- Is `Current' currently in a state where it can be
			-- undone?
		do
			Result := not cache.run_fill_selector_job_rows.there_exists (agent {RUN_JOBS_ROW}.is_status_stopable)
		end

	not_undoable_reason: STRING
			-- Reason step cannot currently be undone
		do
			Result := "one or more fill selector jobs is running or pending.%NPlease select all stoppable jobs and stop them first."
		end

	do_step
		local
			ifc: JOB_CONTROL_IFC
		do
			cache.check_one_field
			cache.check_one_scale_factor
			cache.check_fill_pattern_sizings
			if cache.run_fill_selector_job_rows.is_empty then
				insert_run_job
			end
			ifc := factory.fill_selector_control_interface (session.run_id, session.title_with_run_id ("Generate Fill Selector"))
			ifc.set_session (session)
			ifc.success_actions.extend (agent call_completion_actions)
			ifc.abort_actions.append (abort_actions)
			ifc.build
			if preferences.is_autosubmit_enabled then
				gui.do_once_on_idle (agent ifc.autosubmit_jobs)
			end
		end

	undo_step
			-- Undo execution of step
		local
			lock_op: ROW_LOCK_OP
		do
			fill_selector_job_mrows := cache.run_fill_selector_job_mrows
			add_commit_action (agent delete_files (generated_files))
			create lock_op
			lock_op.lock_list_nowait (fill_selector_job_mrows)
			delete_job_rows
			call_completion_actions
		end

	set_state
			-- Set `state'
		do
			state := Void
		end

	insert_run_job
			-- Insert one row in RUN_JOBS for the FILL_SELECTOR
			-- job
		local
			job: RUN_JOBS_MROW
			jobs: ARRAYED_LIST [RUN_JOBS_MROW]
			job_type: CATS_JOB_TYPE_VALUES
			job_status: JOB_STATUS_VALUES
		do
			create jobs.make (1)
			create job.make_null (session.connection)
			job.set_run_id (session.run_id)
			job.set_cats_job_type (job_type.Fill_selector)
			job.set_job_status (job_status.Created)
			jobs.extend (job)
			session.commit
			cache.cache_run_fill_selector_jobs (jobs)
		end

	delete_job_rows
			-- Delete each row in `fill_selector_job_mrows' for run
		local
			job: RUN_JOBS_MROW
		do
			from
				fill_selector_job_mrows.start
			until
				fill_selector_job_mrows.after
			loop
				job := fill_selector_job_mrows.item
				job.delete
				fill_selector_job_mrows.forth
			end
		end

	generated_files: LIST [RC_FILE]
			-- Files which may have been generated during
			-- execution of step (were generated if step
			-- finished)
		local
			job: RUN_JOBS_MROW
		do
			create {ARRAYED_LIST [RC_FILE]} Result.make (20)
			from
				fill_selector_job_mrows.start
			until
				fill_selector_job_mrows.after
			loop
				job := fill_selector_job_mrows.item
				if job.is_status_resettable then
					-- Files may exist
					Result.append (cache.fill_selector_created_files)
					Result.extend (cache.fill_selector_log_file)
					Result.extend (cache.fill_selector_done_file)
				end
				fill_selector_job_mrows.forth
			end
		end

	delete_files (list: LIST [RC_FILE])
			-- Delete all files in `list' that exist
		do
			from
				list.start
			until
				list.after
			loop
				list.item.delete_if_exists
				list.forth
			end
		end

	fill_selector_job_mrows: LIST [RUN_JOBS_MROW]
			-- Rows for fill selector jobs for run

end
