
indexing
	
	description: "A run closing step operation that generates %
		%the Export Control report to be sent to the foundry."
	author: "David Hollenberg"
	date: "May 4, 2005"

class GEN_FOUNDRY_ECCN_REPORT_OP
inherit
	FOUNDRY_ECCN_REPORT_OP
	CREATES_EXTERNAL_STATE

create
	make

feature -- Properties

	input_data_items: LIST [STRING]
			-- Data items required before step can be executed
		do
			Result := array_as_list (<<
					data_item.Production_job
				 >>)
		end

feature {NONE} -- Implementation

	do_step
			-- Execute step
		local
			missing_count: INTEGER
			msg: STRING
		do
			check_orders_eccn_status
			run_one_report (environment.foundry_eccn_report, "p_production_id=" + session.run_id + " background=yes", eccn_report_output_file.file_pathname)
			if question ("Send ECCN report to Export Control mailing list") then
				publish_foundry_eccn_gen_event
			end
			missing_count := missing_non_itar_eccn_count
			if missing_count > 0 then
				msg := missing_count.out + " non-ITAR " + su.number (missing_count, "project", "projects") + " with missing ECCN"
				if cache.is_trusted_foundry_run then
					warn (msg)
				else
					msg := msg + ".%NCannot finish step due to missing ECCNs."
					inform (msg)
					abort
				end
			end
			call_completion_actions
		end

	undo_step
			-- Undo execution of step
		do
			fetch_eccn_reported_event
			delete_eccn_reported_event
			eccn_report_output_file.delete_if_exists
			call_completion_actions
		end

	check_orders_eccn_status
			-- Check that every work order on production
			-- that has a fab instance for a customer project
			-- has an ECCN.  Issue warning if not
		local
			cust_wo: CUST_WOS_ROW
			cust_wos: LIST [CUST_WOS_ROW]
		do
			create cust_wo.make_template
			cust_wos := cust_wo.work_orders_for_production (session.run_id, connection, expected_count.At_least_zero)
			create {ARRAYED_LIST [CUST_WOS_ROW]} orders_missing_eccn.make (20)
			cust_wos.do_all (agent check_order_eccn_status)
			if not all_eccns_present then
				warn_about_missing_eccns
			end
		end

	check_order_eccn_status (wo: CUST_WOS_ROW)
			-- If `wo' has a fab instance for a customer project
			-- but does not have an ECCN, add it to
			-- `orders_missing_eccn'
		require
			wo_not_void: wo /= Void
		do
			if not wo.has_eccn and then wo.needs_eccn (cache.wafer_techn_families_row) then
				orders_missing_eccn.extend (wo)
			end
		end

	warn_about_missing_eccns
			-- Warn user about missing ECCNs and ask for
			-- confirmation before running report
		local
			msg, ec_type: STRING
			f: IC_FAB_INSTANCES_ROW
		do
			create msg.make (100)
			msg.append (orders_missing_eccn.count.out + " customer projects with missing ECCNs:%N%N")
			from
				orders_missing_eccn.start
			until
				orders_missing_eccn.after
			loop
				f := orders_missing_eccn.item.fab_instance
				ec_type := f.cust_ic_design.export_control_type
				msg.append ("    ")
				msg.append (f.die_id)
				msg.append ("    ")
				msg.append_integer (f.design_no)
				if not ec_type.is_empty then
					msg.append ("    (")
					msg.append (ec_type)
					msg.extend (')')
				end
				msg.extend ('%N')
				orders_missing_eccn.forth
			end
			msg.append ("%N")
			msg.append ("Run ECCN report anyway")
			confirm (msg)
		end

	publish_foundry_eccn_gen_event
			-- Publish a "foundry ECCN generated" event
		local
			event: FOUNDRY_ECCN_GENERATED_EVENT
			body: STRING
			open_op: FILE_OPEN_READ_OP
			close_op: FILE_CLOSE_OP
			f: LARGE_PLAIN_TEXT_FILE
		do
			create f.make (eccn_report_output_file.file_pathname)
			create open_op
			open_op.open (f)
			f.read_stream (f.count.to_integer)
			body := f.last_string
			create close_op
			close_op.close (f)
			
			create event
			event.set_subject ("foundry ECCN report")
			event.set_message (body)
			event.publish
		end

	all_eccns_present: BOOLEAN
			-- Does every customer project have an ECCN?
		do
			Result := orders_missing_eccn.is_empty
		end

	missing_non_itar_eccn_count: INTEGER
			-- Number of customer projects that are not ITAR
			-- and do not have an ECCN, but require one
		local
			f: IC_FAB_INSTANCES_ROW
		do
			Result := 0
			from
				orders_missing_eccn.start
			until
				orders_missing_eccn.after
			loop
				f := orders_missing_eccn.item.fab_instance
				if f.cust_ic_design.is_null_export_control_type then
					-- Not ITAR
					Result := Result + 1
				end
				orders_missing_eccn.forth
			end
		end

	orders_missing_eccn: LIST [CUST_WOS_ROW]
			-- Orders for production that have a fab instance
			-- that is a customer fab, but that do not have
			-- an ECCN

end
