
indexing
	
	description: "A run closing step operation that generates geometry %
		%files for dies and takes sufficiently long that we want to %
		%display a progress window"
	author: "David Hollenberg"
	date: "July 9, 2005"

deferred class GEN_GEOMETRY_FILES_OP
inherit
	GEN_DIE_FILES_OP
		rename
			dies_needing_file_generation as dies_needing_file,
			dies_needing_file_deletion as dies_needing_file
		redefine
			is_applicable
		end

feature {NONE} -- Implementation

	is_applicable: BOOLEAN = True
			-- Is step applicable to run for session?
			-- Always true for now, since we may need to
			-- manually merge overhead, scribes and
			-- fillers for TSMC Cybershuttle runs and
			-- others and we don't have enough information
			-- in the database to known when this step
			-- should be inapplicable
		
	generate_file_for_die (r: like anchored_die)
			-- Generate geometry file for `r'
		local
			gen: MOSIS_OVERHEAD_GENERATOR
			errors: RESPONSE_COLLECTOR
			fname: STRING
			file: LARGE_PLAIN_TEXT_FILE
			dcf: DATA_CONSUMER_CHECKED_FILE
		do
			fname := generated_file_name (r)
			create file.make (fname)
			create dcf.make (file)
			create gen.make (cache.runs_row)
			gen.available_design_technology
				(cache.design_technology_for_dt
					(r.wafer_design_techn))
			-- FIXME: move `initialize' and `consume_eof'
			-- calls into Joel's code?
			dcf.initialize
			generate_file (gen, r, dcf)
			dcf.consume_eof
			errors := gen.the_messages
			if not errors.is_empty then
				report_errors (r, errors)
			end
		end

	report_errors (r: like anchored_die errors: RESPONSE_COLLECTOR)
			-- Report errors/warnings on `r' in `errors'
		local
			msg, kind: STRING
			item: RESPONSE_ITEM
			n: INTEGER
		do
			from
				create msg.make (512)
				msg.append (errors.count.out + " errors/warnings while generating " + file_description + " for die " + r.die_id + "%N%N")
				n := 0
				errors.start
			until
				errors.after or n >= Max_displayed_errors
			loop
				item := errors.item
				if item.is_in_error then
					kind := "Error"
				else
					kind := "Warning"
				end
				msg.append (kind + ": " + item.text + "%N")
				n := n + 1
				errors.forth
			end
			if errors.count > Max_displayed_errors then
				msg.append ("(Rest of messages not shown)%N")
			end
			handle_hard_error (msg)
		end

	Max_displayed_errors: INTEGER = 5
			-- Maximum number of error/warning messages
			-- to display

feature {NONE} -- Deferred implementation
	
	generated_file_name (r: like anchored_die): STRING
			-- Name of geometry file to be generated for `r'
		deferred
		end

	generate_file (gen: MOSIS_OVERHEAD_GENERATOR r: like anchored_die dcf: DATA_CONSUMER_CHECKED_FILE)
			-- Generate layout file for `r' into `dcf'
			-- using `gen' to do the work
		deferred
		end

	file_description: STRING
			-- Description of type of files being generated,
			-- for error messages
		deferred
		ensure
			result_not_void: Result /= Void
		end

end
