
indexing
	
	description: "A run closing step operation that generates PostScript %
		%and PDF files for a map of the wafer or a field of view"
	author: "David Hollenberg"
	date: "January 21, 2005"

deferred class GEN_MAP_OP
inherit
	RC_PRIMITIVE_STEP_OP
	NO_USER_INTERFACE
	CREATES_EXTERNAL_STATE

feature {NONE} -- Implementation

	is_applicable: BOOLEAN
			-- Is step applicable to run for session?
		do
			Result := True
		end

	do_step
			-- Execute step
		do
			create map_generator.make (session)
			generate_maps
			call_completion_actions
		end

	undo_step
			-- Undo execution of step
		do
			delete_maps
			call_completion_actions
		end

	set_state
			-- Set `state'
		do
			state := Void
		end

	
	generate_maps
			-- Generate PostScript and PDF files for map(s)
		deferred
		end

	delete_maps
			-- Delete map files
		deferred
		end

	generate_postscript_file (m: MOSIS_MAP fname: STRING)
			-- Generate PostScript file for map `m' and
			-- write it into file `fname'
		require
			map_not_void: m /= Void
			fname_not_void: fname /= Void
		do
			m.set_drawing_rotatable (drawing_rotatable)
			m.generate_document
			m.write_postscript_file (fname)
		end

	convert_postscript_to_pdf (in_fname, out_fname: STRING)
			-- Convert PostScript map file to PDF
		require
			in_fname_not_void: in_fname /= Void
			out_fname_not_void: out_fname /= Void
		local
			converter: POSTSCRIPT_CONVERTER
		do
			create converter.make (environment.ghostscript_program)
			converter.convert_named_file_to_pdf (in_fname, out_fname)
		end

	drawing_rotatable: BOOLEAN
			-- PostScript file to write
			-- May drawing be rotated to make it appear
			-- larger on the paper, if appropriate?
		do
			Result := True
		end

	map_generator: RC_MAP_GENERATOR
			-- Object that knows how to create map objects

	map: RUN_MAP
			-- Map to be generated

end
