
indexing
	
	description: "A run closing step operation that runs %
		%creates PostScript and PDF files for the project summary %
		%report for the run"
	author: "David Hollenberg"
	date: "February 16, 2005"

class GEN_PROJECT_SUMMARY_OP
inherit
	RC_PRIMITIVE_STEP_OP
	RC_APPLICABLE_STEP
	NO_USER_INTERFACE
	CREATES_EXTERNAL_STATE

create
	make

feature -- Properties

	input_data_items: LIST [STRING]
			-- Data items required before step can be executed
		do
			Result := array_as_list (<<
					data_item.Assigned_die_ids,
					data_item.Production_job
				>>)
		end

feature {NONE} -- Implementation

	do_step
			-- Execute step
		do
			run_one_report (environment.project_summary_report, "runid=" + session.run_id + " p_run_id=" + session.run_id, project_summary_pdf_file.file_pathname)
			convert_pdf_to_postscript (project_summary_pdf_file, project_summary_ps_file)
			call_completion_actions
		end

	undo_step
			-- Undo execution of step
		do
			project_summary_ps_file.delete_if_exists
			project_summary_pdf_file.delete_if_exists
			call_completion_actions
		end

	set_state
			-- Set `state'
		do
			state := Void
		end

	convert_pdf_to_postscript (pdf_file, ps_file: RC_FILE)
			-- Convert PDF file to PostScript
		require
			pdf_file_not_void: pdf_file /= Void
			ps_file_not_void: ps_file /= Void
		local
			converter: PDF_CONVERTER
		do
			create converter.make (environment.acroread_program)
			converter.convert_named_file_to_postscript (pdf_file.file_pathname, ps_file.file_pathname)
		end

	project_summary_ps_file: RC_FILE
			-- PostScript file for project summary report
		do
			Result := file_generator.project_summary_ps_file
		ensure
			result_exists: Result /= Void
		end

	project_summary_pdf_file: RC_FILE
			-- PDF file for project summary report
		do
			Result := file_generator.project_summary_pdf_file
		ensure
			result_exists: Result /= Void
		end

end
