
indexing
	
	description: "A run closing step operation that generates %
		%MEBES files for each individual die"
	author: "David Hollenberg"
	date: "November 17, 2003"

class GENERATE_MEBES_OP
inherit
	RC_PRIMITIVE_STEP_OP
		redefine
			is_currently_undoable, not_undoable_reason
		end
	HAS_USER_INTERFACE
	CREATES_EXTERNAL_STATE
	GUI_UTILITIES_ACCESS
		export
			{NONE} all
		undefine
			default_create
		end

create
	make

feature -- Properties

	input_data_items: LIST [STRING]
			-- Data items required before step can be executed
		do
			Result := array_as_list (<<
				data_item.Die_fracture_scripts,
				data_item.Dynamic_vendor_geometry,
				data_item.Scribe_files,
				data_item.Filler_files,
				data_item.Die_overhead_files
				>>)
		end

feature {NONE} -- Implementation

	is_applicable: BOOLEAN
			-- Is step applicable to run for session?
		do
			Result := cache.run_needs_mebes_files
		end

	is_currently_undoable: BOOLEAN
			-- Is `Current' currently in a state where it can be
			-- undone?
		do
			Result := not cache.die_rows.there_exists (agent {DIES_ROW}.is_status_stopable)
		end

	not_undoable_reason: STRING
			-- Reason step cannot currently be undone
		do
			Result := "one or more die fracture jobs is running or pending.%NPlease select all stoppable jobs and stop them first."
		end

	do_step
		local
			ifc: JOB_CONTROL_IFC
		do
			ifc := factory.die_fracture_control_interface (session.run_id, session.title_with_run_id ("Fracture Dies"))
			ifc.set_session (session)
			ifc.success_actions.extend (agent call_completion_actions)
			ifc.abort_actions.append (abort_actions)
			ifc.build
			if preferences.is_autosubmit_enabled then
				gui.do_once_on_idle (agent ifc.autosubmit_jobs)
			end
		end

	undo_step
			-- Undo execution of step
		local
			lock_op: ROW_LOCK_OP
			pf_mrows: LIST [PATTERN_FILES_MROW]
		do
			die_mrows := cache.die_mrows
			add_commit_action (agent delete_files (generated_files))
			pf_mrows := pattern_file_rows
			create lock_op
			lock_op.lock_list_nowait (die_mrows)
			lock_op.lock_list_nowait (pf_mrows)
			die_mrows.do_all (agent {DIES_MROW}.set_status_created)
			pf_mrows.do_all (agent {PATTERN_FILES_MROW}.delete)
			call_completion_actions
		end

	set_state
			-- Set `state'
		do
			state := Void
		end

	pattern_file_rows: LIST [PATTERN_FILES_MROW]
			-- Rows in pattern files table created by step
		local
			row: PATTERN_FILES_MROW
			pattern_type: PATTERN_TYPE_VALUES
		do
			create row.make_template
			Result := row.rows_for_run_and_type (session.run_id, Pattern_type.Die, connection, expected_count.At_least_zero)
		end

	generated_files: LIST [RC_FILE]
			-- Files which may have been generated during
			-- execution of step (were generated if step
			-- finished)
		local
			die: DIES_MROW
		do
			create {ARRAYED_LIST [RC_FILE]} Result.make (200)
			from
				die_mrows.start
			until
				die_mrows.after
			loop
				die := die_mrows.item
				if die.is_status_resettable then
					-- Files may exist
					Result.append (cache.die_fracture_pattern_files (die))
					Result.extend (cache.die_fracture_log_file (die))
					Result.extend (cache.die_fracture_done_file (die))
				end
				die_mrows.forth
			end
		end

	delete_files (list: LIST [RC_FILE])
			-- Delete all files in `list' that exist
		do
			list.do_all (agent {RC_FILE}.delete_if_exists)
		end

	die_mrows: LIST [DIES_MROW]
			-- Rows for run in DIES table

end
