
indexing
	
	description: "A run closing step operation that generates files %
		%needed by probing software (wafer files and PTS files)"
	author: "David Hollenberg"
	date: "August 30, 2004"

class GENERATE_PROBING_FILES_OP
inherit
	RC_PRIMITIVE_STEP_OP
	NO_USER_INTERFACE
	CREATES_EXTERNAL_STATE
	RC_POLICIES_ACCESS
		export
			{NONE} all
		undefine
			default_create
		end

create
	make

feature -- Properties

	input_data_items: LIST [STRING]
			-- Data items required before step can be executed
		do
			Result := array_as_list (<<
				data_item.Mask_techn,
				data_item.Assigned_die_ids,
				data_item.Field_step_pattern,
				data_item.Test_vehicle_fab_instances
					-- Depends on fab instances so that
					-- we can get effective overhead style
					-- from die_attributes
				>>)
		end

feature {NONE} -- Implementation

	is_applicable: BOOLEAN
			-- Is step applicable to run for session?
		do
			Result := cache.is_reticle_packing_target and not cache.probeable_die_rows.is_empty
		end

	do_step
			-- Execute of step
		do
			fetch_cached_data
			fetch_all_probers
			check_test_vehicle_status
			set_rotation_counts
			select_main_wafer_rotation
			write_wafer_files
			publish_probing_files_event
			execute_wafer_file_hook
			call_completion_actions
		end

	undo_step
			-- Undo execution of step
		do
			fetch_all_probers
			probers.do_all (agent delete_wafer_files)
			call_completion_actions
		end

	set_state
			-- Set `state'
		do
			state := Void
		end

	fetch_cached_data
			-- Fetch needed data into attributes
		local
			first_lot: RUN_WAFER_LOTS_ROW
			rect: CONST_RECTANGLE
			margin: CONST_POINT
		do
			test_vehicles := cache.test_vehicle_die_rows
			cust_dies := cache.cust_die_die_rows
			probeable_dies := cache.probeable_die_rows
			create all_dies.make (test_vehicles.count + cust_dies.count)
			all_dies.append (test_vehicles)
			all_dies.append (cust_dies)
			wafer_radius := cache.wafer_radius
			
			create classifier
			classifier.set_wafer_radius (cache.wafer_radius)
			classifier.set_tweezer_radius (cache.tweezer_radius)
			classifier.set_reference_tweezer_radius (cache.tweezer_radius - extra_reference_margin)
			classifier.set_wafer_primary_flat_box (cache.wafer_primary_flat_box)
			classifier.set_tweezer_primary_flat_box (cache.tweezer_primary_flat_box)
			rect := cache.tweezer_primary_flat_box
			create margin.make_xy (extra_reference_margin, extra_reference_margin)
			classifier.set_reference_tweezer_primary_flat_box (create {CONST_RECTANGLE}.make_corners (rect.ll + margin, rect.ur - margin))
			
			field_size := cache.field_size
			process_vdd := cache.process_vdd

			-- FIXME: just use step pattern and offset for
			-- wafer lot 1 for now
			
			first_lot := cache.first_wafer_lot_row
			step_pattern_offset := first_lot.field_step_pattern_offset
			step_locations := cache.field_step_location_rows (first_lot.field_step_pattern)
		end

	fetch_all_probers
			-- Fetch list of all probers
		local
			prober: WAFER_PROBERS_ROW
		do
			create prober.make_template
			probers := prober.all_possible_probers (connection, expected_count.At_least_one)
		end

	check_test_vehicle_status
			-- Check that every element of `probeable_dies'
			-- has a valid status
		do
			probeable_dies.do_all (agent check_valid_status)
		end

	check_valid_status (die: DIES_ROW)
			-- Check that `die' has a valid status
		require
			die_exists: die /= Void
			is_probeable: die.is_probeable
		local
			status: STRING
			tv_status: TEST_VEHICLE_STATUS_VALUES
		do
			status := connection.selected_string ("test_vehicle_status", "m_test_vehicles_", "design_no = :1", create {DB_BIND_I}.make (die.design_no))
			if not status.is_equal (tv_status.Fab_and_test) and not status.is_equal (tv_status.Test_only) then
				handle_hard_error ("probeable die " + die.design_no.out + " (die " + die.die_id + ") has invalid test vehicle status " + status)
			end
		end

	set_rotation_counts
			-- Set counts of number of probeable dies with each
			-- rotation
		local
			rot: INTEGER
			tv: DIES_ROW
		do
			rotated_0_count := 0
			rotated_90_count := 0
			rotated_180_count := 0
			rotated_270_count := 0
			from
				probeable_dies.start
			until
				probeable_dies.after
			loop
				tv := probeable_dies.item
				rot := tv.die_wafer_rotation
				inspect rot
				when 0 then
					rotated_0_count := rotated_0_count + 1
				when 90 then
					rotated_90_count := rotated_90_count + 1
				when 180 then
					rotated_180_count := rotated_180_count + 1
				when 270 then
					rotated_270_count := rotated_270_count + 1
				
				else
					handle_hard_error ("probeable die " + tv.die_id + " has unsupported rotation " + rot.out)
				end
				probeable_dies.forth
			end
		end

	select_main_wafer_rotation
			-- Set `main_file_rotation' to indicate the rotation
			-- of the probeable dies in the main wafer file
		local
			l_count: INTEGER
		do
			main_file_rotation := 0
			l_count := rotated_0_count
			if rotated_90_count > l_count then
				l_count := rotated_90_count
				main_file_rotation := 90
			end
			if rotated_180_count > l_count then
				l_count := rotated_180_count
				main_file_rotation := 180
			end
			if rotated_270_count > l_count then
				l_count := rotated_270_count
				main_file_rotation := 270
			end
		end

	write_wafer_files
			-- Write main wafer file for each prober.  If there
			-- are both rotated and unrotated probeable dies,
			-- write auxiliary wafer file also
		do
			create {ARRAYED_LIST [WAFER_FILE]} wafer_files.make (4)
			probers.do_all (agent write_prober_wafer_files)
		end

	write_prober_wafer_files (prober: WAFER_PROBERS_ROW)
			-- Write main wafer file `prober'.  If there
			-- are both rotated and unrotated probeable dies,
			-- write auxiliary wafer file also
		require
			prober_exists: prober /= Void
		local
			file: RC_FILE
		do
			classifier.set_chuck_travel_box (prober.chuck_travel_box)
			classifier.set_obstacle_distance (prober.obstacle_distance)
			current_prober := prober
			file := file_generator.main_wafer_file (prober.short_id)
			session.ensure_directory_exists (file)
			write_prober_wafer_file (file, main_file_rotation, True)
			write_prober_aux_wafer_file (prober, 0, rotated_0_count)
			write_prober_aux_wafer_file (prober, 90, rotated_90_count)
			write_prober_aux_wafer_file (prober, 180, rotated_180_count)
			write_prober_aux_wafer_file (prober, 270, rotated_270_count)
		end

	write_prober_aux_wafer_file (prober: WAFER_PROBERS_ROW rot, a_count: INTEGER)
			-- Write auxiliary wafer file for `prober' which
			-- includes only probeable dies with rotation `rot',
			-- but only if `a_count' is positive
		require
			prober_exists: prober /= Void
		local
			file: RC_FILE
		do
			if a_count > 0 then
				file := file_generator.aux_wafer_file (prober.short_id, rot)
				session.ensure_directory_exists (file)
				write_prober_wafer_file (file, rot, False)
			end
		end

	write_prober_wafer_file (f: RC_FILE rot: INTEGER main: BOOLEAN)
			-- Write one wafer file to `f' for
			-- `current_prober', where `rot' indicates
			-- the rotation of probeable dies to
			-- be probed and `main' indicates whether this
			-- is a "main" wafer file rather than auxiliary
		require
			file_exists: f /= Void
		do
			rotation := rot
			compute_near_edge_sites
			compute_probe_sites
			select_reference_site
			order_near_edge_sites
			assign_wafer_site_numbers
			write_wafer_file (f, main)
		end

	write_wafer_file (f: RC_FILE main: BOOLEAN)
			-- Write wafer file to `f', where `main' indicates
			-- whether this is a main wafer file rather than
			-- an auxiliary file
		require
			file_exists: f /= Void
		local
			file: LARGE_PLAIN_TEXT_FILE
			wf: WAFER_FILE
			fsize: CONST_POINT
		do
			create file.make (f.file_pathname)
			session.open_file_for_write (file, f)
			create wf
			wf.set_prober_id (current_prober.short_id)
			wf.set_main (main)
			wf.set_run_id (session.run_id)
			wf.set_run_type (cache.run_type)
			wf.set_rotation (rotation)
			wf.set_run_techn (cache.run_techn)
			wf.set_feature_size (0.0)
				-- Not required by report generator any more
			wf.set_process_vdd (process_vdd)
			wf.set_wafer_vendor (cache.wt_organizations_row.short_id)
			wf.set_mask_vendor (cache.mt_organizations_row.short_id)
			wf.set_wafer_techn (cache.wafer_techn)
			wf.set_mask_techn (cache.mask_techn)
			if cache.field_count = 1 then
				wf.set_mask_type ("SINGLE_FIELD_RETICLE")
			else
				wf.set_mask_type ("MULTIPLE_FIELD_RETICLE")
			end
			wf.set_mask_size (cache.mask_sizes_row.mask_size)
			if is_rotated then
				create fsize.make_xy (field_size.y, field_size.x)
			else
				fsize := field_size
			end
			wf.set_field_step_size (fsize)
			wf.set_wafer_diameter (cache.wafer_diameter)
			wf.set_probe_sites (probe_sites)
			wf.set_near_edge_sites (near_edge_sites)
			wf.set_reference_site (reference_site)
			wf.set_reference_pad_location (reference_pad_location)
			wf.write (file)
			session.close_file (file)
			wafer_files.extend (wf)
		end

	compute_probe_sites
			-- Compute `probe_sites' and `reference_probe_sites'
		local
			sorter: LIST_QUICK_SORTER [PROBE_SITE]
			comparator: PROBE_SITE_COMPARATOR
		do
			create {ARRAYED_LIST [PROBE_SITE]} probe_sites.make (100)
			create {ARRAYED_LIST [PROBE_SITE]} reference_probe_sites.make (100)
			probeable_dies.do_all (agent compute_die_probe_sites)
			create sorter
			create comparator
			probe_sites := sorter.sorted (probe_sites, comparator)
			reference_probe_sites := sorter.sorted (reference_probe_sites, comparator)
		end

	compute_die_probe_sites (die: DIES_ROW)
			-- Compute `probe_sites' and `reference_probe_sites'
			-- for `die'
		require
			die_exists: die /= Void
		do
			if die.die_wafer_rotation = rotation then
				cache.die_location_rows_for_die_id (die.die_id).do_all (agent compute_die_loc_probe_sites (die, ?))
			end
		end

	compute_die_loc_probe_sites (die: DIES_ROW die_loc: DIE_LOCATIONS_ROW)
			-- Compute `probe_sites' and `reference_probe_sites'
			-- for `die' and `die_loc'
		require
			die_exists: die /= Void
			die_loc_exists: die_loc /= Void
		local
			step_loc: FIELD_STEP_PATTERNXFIELD_LOCS_ROW
			die_center, step_center, offset: CONST_POINT
			size, center, rot_size, rot_center: CONST_POINT
			site: PROBE_SITE
			objects:  LIST [M_TEST_OBJECTS_EROW]
			object: M_TEST_OBJECTS_EROW
			ok_for_reference: BOOLEAN
			l_design_die_offset: CONST_POINT
		do
			l_design_die_offset := die.design_die_offset
			from
				step_locations.start
			until
				step_locations.after
			loop
				step_loc := step_locations.item
				if die_loc.field_name.is_equal (step_loc.field_name) then
					step_center := step_loc.center + step_pattern_offset
					die_center := die_loc.center + step_center
					size := die.printed_size_on_wafer
					center := die_center
					rot_size := die.printed_size_with_rotation (is_rotated)
					rot_center := rotated_ccw (die_center, rotation)
					
					if not step_loc.partially_processed and classifier.reachable_die (rot_size, rot_center) and classifier.safe_die (rot_size, rot_center) and classifier.good_die (size, center) then
						ok_for_reference := classifier.good_reference_die (size, center)
						from
							objects := die.test_objects
							objects.start
						until
							objects.after
						loop
							object := objects.item
							offset := l_design_die_offset + object.probe_origin
							site := new_site (die_loc, step_loc, die_center, step_center, offset, object.test_object, object.test_object_type)
							probe_sites.extend (site)
							if ok_for_reference then
								reference_probe_sites.extend (site)
							end
							objects.forth
						end
					end
				end
				step_locations.forth
			end
		end

	select_reference_site
			-- Choose reference site and put in `reference_site'.
			-- Also set `reference_pad_location'
		local
			site: PROBE_SITE
			field_min_x, field_max_y, site_max_x, site_max_y: DOUBLE
			step_x, step_y, loc_x, loc_y: DOUBLE
			infinity: DOUBLE
			ref_pad_loc: CONST_POINT
		do
			set_preferred_reference_sites
			if preferred_reference_sites.is_empty then
				handle_hard_error ("list of preferred reference sites is empty")
			end

			reference_site := Void
			infinity := 1.0E100
			field_min_x := infinity
			field_max_y := -infinity
			site_max_x := -infinity
			site_max_y := -infinity
			from
				preferred_reference_sites.start
			until
				preferred_reference_sites.after
			loop
				site := preferred_reference_sites.item
				step_x := site.step_location.x
				step_y := site.step_location.y
				loc_x := site.location.x
				loc_y := site.location.y
				if step_y > field_max_y or
				   (step_y = field_max_y and step_x < field_min_x) or
				   (step_y = field_max_y and step_x = field_min_x and (loc_y > site_max_y or (loc_y = site_max_y and loc_x > site_max_x))) then
					reference_site := site
					field_min_x := step_x
					field_max_y := step_y
					site_max_x  := loc_x
					site_max_y  := loc_y
				end
				preferred_reference_sites.forth
			end
			
			if reference_site = Void then
				handle_hard_error ("no valid reference site for wafer file")
			end
			ref_pad_loc := reference_site.location - (prober_coords (reference_site.step_location))
			create reference_pad_location.make_xy (ref_pad_loc.x, -ref_pad_loc.y)
		end

	set_preferred_reference_sites
			-- Put list of preferred sites for reference
			-- in `preferred_reference_sites'
		local
			site: PROBE_SITE
			list: LIST [PROBE_SITE]
			param_sites, func_sites: ARRAYED_LIST [PROBE_SITE]
			nsa_sites, non_nsa_sites: ARRAYED_LIST [PROBE_SITE]
		do
			create param_sites.make (100)
			create func_sites.make (100)
			from
				reference_probe_sites.start
			until
				reference_probe_sites.after
			loop
				site := reference_probe_sites.item
				if site.is_parametric then
					param_sites.extend (site)
				elseif site.is_functional then
					func_sites.extend (site)
				end
				reference_probe_sites.forth
			end
			if not param_sites.is_empty then
				list := param_sites
			elseif not func_sites.is_empty then
				list := func_sites
			else
				list := probe_sites
			end
			
			create nsa_sites.make (list.count)
			create non_nsa_sites.make (list.count)
			from
				list.start
			until
				list.after
			loop
				site := list.item
				if site.is_nsa_object then
					nsa_sites.extend (site)
				else
					non_nsa_sites.extend (site)
				end
				list.forth
			end
			if not non_nsa_sites.is_empty then
				list := non_nsa_sites
			end
			
			preferred_reference_sites := list
		end

	order_near_edge_sites
			-- Put `near_edge_sites' in an order
			-- that will reduce the amount of stage movement
			-- of the prober.  Use a greedy algorithm
			-- that start from the reference site as the
			-- current site and picks as the next current
			-- site the one closest to the current site
		local
			k, count: INTEGER
			list: like near_edge_sites
			curr_site: PROBE_SITE
		do
			count := near_edge_sites.count
			list := near_edge_sites.twin
			list.wipe_out
			curr_site := reference_site
			from
				k := 1
			until
				k > count
			loop
				add_closest_unvisited_site (near_edge_sites, curr_site, list)
				curr_site := list.last
				k := k + 1
			end
			near_edge_sites := list
		end

	add_closest_unvisited_site (src_list: LIST [PROBE_SITE] curr_site: PROBE_SITE dest_list: LIST [PROBE_SITE])
			-- Add unvisted site in `src_list' that is closest
			-- to `curr_site' to end of `dest_list' and mark the
			-- site as visited
		local
			site, closest_site: PROBE_SITE
			dist, closest_dist: DOUBLE
		do
			closest_site := Void
			closest_dist := 1.0E100
			from
				src_list.start
			until
				src_list.after
			loop
				site := src_list.item
				if not site.is_visited then
					dist := (site.location - curr_site.location).abs
					if dist < closest_dist then
						closest_site := site
						closest_dist := dist
					end
				end
				src_list.forth
			end
			closest_site.set_visited (True)
			dest_list.extend (closest_site)
		end

	assign_wafer_site_numbers
			-- Assign wafer site numbers to `probe_sites'
			-- and then to `near_edge_sites'
		do
			last_site_number := 0
			probe_sites.do_all (agent assign_wafer_site_number)
			near_edge_sites.do_all (agent assign_wafer_site_number)
		end

	assign_wafer_site_number (site: PROBE_SITE)
			-- Assign next wafer site number to `site'
		do
			last_site_number := last_site_number + 1
			site.set_wafer_site_number (last_site_number)
		end

	compute_near_edge_sites
			-- Compute `near_edge_sites'
		do
			create {ARRAYED_LIST [PROBE_SITE]} near_edge_sites.make (100)
			all_dies.do_all (agent compute_die_near_edge_sites)
				-- Include sites near edge in both main
				-- and auxiliary wafer files (if have both)
		end

	compute_die_near_edge_sites (die: DIES_ROW)
			-- Compute `near_edge_sites' for `die'
		require
			die_exists: die /= Void
		do
			cache.die_location_rows_for_die_id (die.die_id).do_all (agent compute_die_loc_near_edge_sites (die, ?))
		end

	compute_die_loc_near_edge_sites (die: DIES_ROW die_loc: DIE_LOCATIONS_ROW)
			-- Compute `near_edge_sites' for `die' and `die_loc'
		require
			die_exists: die /= Void
			die_loc_exists: die_loc /= Void
		local
			step_loc: FIELD_STEP_PATTERNXFIELD_LOCS_ROW
			die_center, step_center, offset: CONST_POINT
			size, center, rot_size, rot_center: CONST_POINT
			site: PROBE_SITE
		do
			create offset.make_xy (0.0, 0.0)
				-- No offset since want to ink center of die
			from
				step_locations.start
			until
				step_locations.after
			loop
				step_loc := step_locations.item
				if die_loc.field_name.is_equal (step_loc.field_name) then
					step_center := step_loc.center + step_pattern_offset
					die_center := die_loc.center + step_center
					size := die.printed_size_on_wafer
					center := die_center
					rot_size := die.printed_size_with_rotation (is_rotated)
					rot_center := rotated_ccw (die_center, rotation)
					
					if classifier.reachable_die (rot_size, rot_center) and classifier.safe_die (rot_size, rot_center) and (classifier.marginal_die (size, center) or (step_loc.partially_processed and classifier.good_die (size, center))) then
						site := new_site (die_loc, step_loc, die_center, step_center, offset, Near_edge_name, Ink_type)
						near_edge_sites.extend (site)
					end
				end
				step_locations.forth
			end
		end

	new_site (die_loc: DIE_LOCATIONS_ROW step_loc: FIELD_STEP_PATTERNXFIELD_LOCS_ROW die_center, step_center, offset: CONST_POINT obj_name, obj_type: STRING): PROBE_SITE
			-- New object constructed from information in arguments
		do
			create Result
			Result.set_run_id (die_loc.run_id)
			Result.set_field_name (die_loc.field_name)
			Result.set_copy_number (step_loc.copy_no)
			Result.set_mask_site_number (die_loc.mask_site_no)
			Result.set_die_id (die_loc.die_id)
			Result.set_step_location (rotated_ccw (step_center, rotation))
			Result.set_location (prober_coords (rotated_ccw (die_center, rotation) + offset))
			Result.set_object_name (obj_name)
			Result.set_object_type (obj_type)
		end

	is_rotated: BOOLEAN
			-- Is wafer file currently being generated for
			-- rotated probeable dies?
		do
			Result := rotation = 90 or rotation = 270
		end

	rotated_ccw (p: CONST_POINT rot: INTEGER): CONST_POINT
			-- `p' rotated `rot' degrees counter-clockwise
		do
			inspect rot
			when 0 then
				Result := p
			when 90 then
				create Result.make_xy (-p.y, p.x)
			when 180 then
				create Result.make_xy (-p.x, -p.y)
			when 270 then
				create Result.make_xy (p.y, -p.x)
			else
				handle_hard_error ("unsupported rotation " + rot.out)
			end
		end

	prober_coords (p: CONST_POINT): CONST_POINT
			-- Given `p' with coordinates relative to center
			-- of wafer, convert it to prober coordinate system
			-- where origin is at upper left corner of box
			-- enclosing wafer circle and Y axis goes down
		do
			create Result.make_xy (p.x + wafer_radius, wafer_radius - p.y)
		end

	publish_probing_files_event
			-- Publish "probing files generated" event
		local
			event: PROBING_FILES_EVENT
		do
			create event
			event.set_subject ("wafer files generated (" + wafer_files.count.out + ")")
			event.set_message (message_body)
			event.publish
		end

	message_body: STRING
			-- Body of message to be sent when probing files
			-- generated
		local
			wf: WAFER_FILE
			list: LIST [PROBE_SITE]
			site: PROBE_SITE
			count: INTEGER
			ref_name: STRING
		do
			count := wafer_files.count
			Result := count.out + " " + su.number (count, "wafer file", "wafer files") + " generated for run " + session.run_id + "%N"
			from
				wafer_files.start
			until
				wafer_files.after
			loop
				wf := wafer_files.item
				ref_name := wf.reference_site.object_name
				Result.extend ('%N')
				if wf.is_main then
					Result.append ("Main")
				else
					Result.append ("Auxiliary")
				end
				Result.extend (' ')
				Result.append (wf.prober_id)
				Result.append (" wafer file with rotation = ")
				Result.append_integer (wf.rotation)
				Result.append (" has the following test objects:%N%N")
				from
					list := wf.distinct_object_sites
					list.start
				until
					list.after
				loop
					site := list.item
					Result.append ("    ")
					Result.append (su.right_padded (site.object_name, 24))
					Result.append ("  ")
					Result.append (su.right_padded (site.object_type, 10))
					if site.object_name.is_equal (ref_name) then
						Result.append ("  (reference object)")
					end
					Result.extend ('%N')
					list.forth
				end
				wafer_files.forth
			end
		end

	execute_wafer_file_hook
			-- Execute program to generate .pts files and
			-- reduce number of sites in wafer file
		local
			fname: STRING
			f: LARGE_RAW_FILE
			process: UNIX_PROCESS
		do
			fname := environment.probing_files_program
			create f.make (fname)
			if not f.exists then
				handle_hard_error ("File " + fname + " not found")
			elseif not f.is_executable then
				handle_hard_error ("File " + fname + " is not executable")
			else
				create process.make (fname)
				process.set_arguments (<< session.run_id, environment.real_probing_dir_base, session.probing_run_directory, session.probing_run_directory >>)
		    		process.spawn_wait
			end
		end

	delete_wafer_files (prober: WAFER_PROBERS_ROW)
			-- Delete wafer files for `prober' that exist
		require
			prober_exists: prober /= Void
		local
			short_id: STRING
		do
			short_id := prober.short_id
			file_generator.main_wafer_file (short_id).delete_if_exists
			file_generator.aux_wafer_file (short_id, 0).delete_if_exists
			file_generator.aux_wafer_file (short_id, 90).delete_if_exists
			file_generator.aux_wafer_file (short_id, 180).delete_if_exists
			file_generator.aux_wafer_file (short_id, 270).delete_if_exists
		end

	test_vehicles: LIST [DIES_ROW]
			-- Dies for test vehicles

	cust_dies: LIST [DIES_ROW]
			-- Dies for customer projects

	probeable_dies: LIST [DIES_ROW]
			-- Dies that can be probed

	all_dies: ARRAYED_LIST [DIES_ROW]
			-- Dies for both customer projects and test vehicles

	main_file_rotation: INTEGER
			-- Rotation of probeable dies in "main" wafer file

	rotated_0_count: INTEGER
			-- Number of probeable dies rotated 0 degrees
	
	rotated_90_count: INTEGER
			-- Number of probeable dies rotated 90 degrees
	
	rotated_180_count: INTEGER
			-- Number of probeable dies rotated 180 degrees
	
	rotated_270_count: INTEGER
			-- Number of probeable dies rotated 270 degrees
	
	rotation: INTEGER
			-- Rotation of probeable dies in wafer file
			-- currently being written

	step_locations: LIST [FIELD_STEP_PATTERNXFIELD_LOCS_ROW]
			-- Locations where pattern is stepped on wafer,
			-- before any offset

	step_pattern_offset: CONST_POINT
			-- Amount by which step pattern should be "moved"
			-- for this lot

	classifier: WAFER_SITE_CLASSIFIER
			-- Object used to classify wafer sites as good, bad,
			-- marginal, etc

	wafer_radius: DOUBLE
			-- Radius of wafer

	extra_reference_margin: DOUBLE
			-- Number of microns by which to expand the tweezer
			-- margin for purposes of selecting the reference site
		do
			Result := policies.reference_extra_tweezer_margin
		end

	field_size: CONST_POINT
			-- Size of field for run, in wafer coordinates

	process_vdd: DOUBLE
			-- Vdd for all processes offered for wafer techn

	probers: LIST [WAFER_PROBERS_ROW]
			-- All possible wafer probers

	current_prober: WAFER_PROBERS_ROW
			-- Prober for which wafer file is currently being
			-- generated

	probe_sites: LIST [PROBE_SITE]
			-- Sites that can be probed

	reference_probe_sites: LIST [PROBE_SITE]
			-- Sites that are valid candidate reference sites
			-- (the subset of `probe_sites' that are within
			-- a bigger tweezer margin, to solve problem with
			-- reference site moving when step pattern is
			-- shifted to match wafer)

	near_edge_sites: LIST [PROBE_SITE]
			-- Sites that need inking because they are near
			-- the edge of the wafer

	preferred_reference_sites: LIST [PROBE_SITE]
			-- Sites that are preferred for selection as
			-- reference site

	reference_site: PROBE_SITE
			-- Sites that serves as reference point

	reference_pad_location: CONST_POINT
			-- Location of lower rightmost probe pad of
			-- reference site relative to center of field

	last_site_number: INTEGER
			-- Last wafer site number used

	wafer_files: LIST [WAFER_FILE]
			-- List of wafer files generated

	Near_edge_name: STRING = "INK_NEAR_EDGE"
	
	Ink_type: STRING = "INK"

end
