deferred class GENERIC_SYSTEM
		-- wrapper for a generic Eiffel system, supporting
		--  configuration variables from the arg line,
		--  database access (db.) and database actions
		--  a punt mechanism that notifies others of crashes
		--  a simple logging mechanism for recording actions

inherit
	ANY
	GENERIC_SYSTEM_SUPPORT
		export	{NONE} all
		end

	EXECUTION_ENVIRONMENT_ACCESS
		export	{NONE} all
		end

	OS_ACCESS
		export	{NONE} all
		end

	MISC_UTILITIES_ACCESS
		export	{NONE} all
		end

	MEMORY
		export	{NONE} all
		end

-- creation
--	make

feature	-- output redirection

	redirected_output_file: PLAIN_TEXT_FILE;

	redirect_standard_output(nam: STRING) is
	do  if su.not_empty(nam)
	     then create redirected_output_file.make_open_append(nam);
		  io.set_file_default(redirected_output_file)
	    end
	end;

feature	-- the system being run

	run_a_system is
		-- normal operation of a system
		-- most everything else in here takes effect only if the
		--  system crashes (untrapped exception)
	deferred
	end;

feature	-- info to use when we punt. make these guys foolproof!!

	mode: STRING is
		-- if the system has various operating modes, the name
		-- can be left here for inclusion in messages
	deferred
	end;

	current_activity: STRING is
		-- some words on what we were processing at crash time
	deferred
	end;

	crash_msg_file_name: STRING is
		-- full name of file to receive crash message
		-- (empty means don't file)
	deferred
	end;

	crash_msg_mailer_options: STRING is
		-- options to sendmail
	deferred
	end;

	crash_msg_to_list: STRING is
		-- address list for crash message
		-- (empty means dont send mail)
	deferred
	end;

	rerun_on_transient_error: BOOLEAN is
		-- shall we re-run the system on a "transient" error?
	deferred
	end;

	rerun_sleep_interval_in_seconds: INTEGER is
		-- if we do re-run, how long to sleep
	deferred
	end;

	report_web_error is
		-- last chance message to web browser (if we suspect one)
	deferred
	end;

feature	-- the notifier that the system has died

	kick_a_punt is
		-- we are about to die; compose and send our
		-- suicide note, explaining everything.
	local  subj, body, foo: STRING; is_exc: BOOLEAN
	do  
	--  dl.log("kap: entering")
	    is_exc := mu.have_exception_explanation;
	    create body.make(80);
	    if is_exc
	     then subj := mu.exception_explanation
	     else subj := "Error Detected"
	    end;
	--  dl.log("kap: subject = " + subj);
	    -- died on a punt: pick up the why
	    if glo.system_error_message /= Void
	     then body.append(glo.system_error_message);
		  body.extend('%N');
	    end;
	    if su.not_empty(mode)
	     then subj := su.string_of_3(subj, " in ", mode)
	    end
	--  dl.log("kap: body = " + body);
	    -- add info on configuration_vars trouble
	    foo := vars.configuration_errors
	    if su.not_empty(foo)
	     then body.append(foo)
		  body.extend('%N');
	    end;
	--  dl.log("kap: body1 = " + body);
	    -- add info on the current system activity
	    foo := current_activity
	    if su.not_empty(foo)
	     then body.append(foo)
		  body.extend('%N');
	    end;
	--  dl.log("kap: body2 = " + body);
	    -- add info on the (last) database error
	    foo := db.last_error_description;
	    if su.not_empty(foo)
	     then body.append(foo)
		  body.extend('%N');
	    end;
	--  dl.log("kap: body3 = " + body);
	    -- try to publish in the log
	    attempt_to_log(subj, body);
	    -- add info on the Eiffel exception trace
	    if is_exc
	     then body.append(exc.exception_trace)
		  body.extend('%N');
	    end;
	--  dl.log("kap: body4 = " + body);
	    -- try to mail it and/or file it
	    attempt_to_file(subj, body);
	    attempt_to_send_punt_msg(subj, body);
	--  dl.log("kap: exitting")
	end;

	attempt_to_log(subj, body: STRING) is
		-- if logging to standard output, let user know too
	do  if glo.is_logging and redirected_output_file = Void
	     then io.putstring(subj);
		  io.putchar('%N'); io.putchar('%N');
		  io.putstring(body);
		  io.putchar('%N');
	    end;
	end;

	attempt_to_file(subj, body: STRING) is
		-- (attempt to) store subj and body in our crash file;
		-- go quietly on failure
	local in_retry: BOOLEAN; foo: LARGE_PLAIN_TEXT_FILE; fnam: STRING
	do  if not in_retry
	     then fnam := crash_msg_file_name;
		  if su.not_empty(fnam)
		   then create foo.make_open_append(fnam);
			foo.put_string(subj);
			foo.put_string(su.eol_eol);
			foo.put_string(body);
			foo.put_character('%N');
			os.flush_file_to_disk (foo)
			foo.close;
		  end
	    end;
	rescue
	    in_retry := true;  retry;
	end;

	attempt_to_send_punt_msg(subj, body: STRING) is
		-- (attempt to) send a punt message
		-- go quietly on failure
	do  attempt_to_send_msg(crash_msg_mailer_options,
				crash_msg_to_list, subj, body)
	end;

feature	-- support for re-running the whole system on a transient error
	-- (if desired)

	is_transient_error(e: INTEGER; tag: STRING): BOOLEAN is
	do  Result := e = exc.Operating_system_exception and then
		      (	equal (tag, "Connection timed out") or
			equal (tag, "Stale NFS file handle") or
			equal (tag, "Operation would block"))
	end;

	is_user_interrupt(e: INTEGER; tag: STRING): BOOLEAN is
	do  Result := e = exc.Signal_exception and then
			-- (unix_os.signal = unix_os.Sigint)
			   (equal (tag, "Interrupt"))
	end;

	system_ran: STRING is "Ran";
	system_punted: STRING is "Punted";

	attempt_to_run_system: STRING is
		-- return system_ran or system_punted or original_tag_name
		-- latter represents a transient system error
		-- NOTE: our return clears the exception
	local in_retry: BOOLEAN; e: INTEGER; tag: STRING
	do  if not in_retry
	     then 
	--	  dl.log("atrs: before");
		  run_a_system
		  Result := system_ran			-- << ran
	--	  dl.log("atrs: after");
	     else e := exc.original_exception;
		  tag := exc.original_tag_name;
		  if rerun_on_transient_error and
		     is_transient_error(e, tag)
		   then 
	--		dl.log("atrs: transient rollback")
			if db.is_open then glo.rollback end;
			full_collect;
			Result := tag			-- << transient failure
		   else 
	--		dl.log("atrs: ready to punt");
			if not (unix_os.is_interrupt or unix_os.is_hangup)
			 then kick_a_punt;
			      report_web_error
			end;
			Result := system_punted		-- << punted
		  end
	    end;
	rescue
	    if not in_retry
	     then in_retry := true; retry
	    end
	end;

	run_system_transient_safe is
		-- Run system with protection from Operating System
		-- transient failures (we don't think these happen any more)
	local retries: INTEGER; res: STRING
	do
	    from  res := attempt_to_run_system;
	    until res = system_ran or res = system_punted
	    loop  retries := retries + 1
		  if retries = 1
		   then attempt_to_send_punt_msg(
				"Operating System Transient", res);
		  end;
	--	  dl.log("rsts: retry #" + retries.out);
		  os.sleep_seconds(rerun_sleep_interval_in_seconds);
		  res := attempt_to_run_system;
	    end;
	    if retries > 0 and res = system_ran
	     then attempt_to_send_punt_msg("Operating System OK",
			 su.string_of_3("(on attempt #", retries.out,
					"  :-|)"));
	    end;
	end;

feature --

	run_configured_system(args: ARRAY[STRING]) is
		-- run the system, with punting and transient retry
	do  -- initialize from args
	    vars.initialize_from_args(args);
	    create dl.make(vars.item("EMERGENCY_DEBUG_LOG_FILE"))
	    if su.not_empty(vars.configuration_errors)
	     then attempt_to_send_punt_msg("Configuration Errors",
					   vars.configuration_errors)
	    end;
	    run_system_transient_safe
	    if db.is_open
	     then 
	--	  dl.log("rcs: rolling back and closing db");
		  glo.rollback;
		  db.close
	    end;
	    if redirected_output_file /= Void and then 
		redirected_output_file.is_open_append
	     then unix_os.flush_descriptor_to_disk (redirected_output_file.descriptor)
		  redirected_output_file.close
	    end;
	    dl.close
	end;

feature -- extra round of safety for web processes

	make(args: ARRAY[STRING]) is
		-- run our system, as protected as we can get
	local tries: INTEGER
	do  inspect tries
	    when 0 then run_configured_system(args)
	    when 1 then report_web_error
	    else
	    end
	rescue
	    tries := tries + 1;
	    retry
	end;

feature	-- debugging log, for when we need it

	dl: DEBUG_LOGGING

end -- GENERIC_SYSTEM
