class GENERIC_SYSTEM_SUPPORT
	-- routines for everyone to share; no attributes here

inherit

	DB_ACCESS		-- db.xxxx is our single db_connection
	export {NONE} all end;

	DB_QUERY_OPTIONS
        export {NONE} all end;

	DB_UTILITIES
	export {NONE} all end;

	STRING_UTILITIES_ACCESS	-- su.xxxx

	FILE_UTILITIES_ACCESS	-- fu.xxxx
		export	{NONE} all
		end

	EXCEPTIONS_ACCESS	-- exc.xxxx
		export	{NONE} all
		end

	UNIX_OS_ACCESS		-- unix_os.xxxx
		export	{NONE} all
		end
	
feature -- globals and environment_vars access

	glo: GENERIC_SYSTEM_GLOBALS is
		-- some globals
        once  create Result  end;

	vars: HASHED_CONFIGURATION_VARS is
		-- access to all of our configuration variables
		-- as vars.nfe_xxxx
	once  create Result.make (223)  end;

feature -- internal error punt

	system_punt(msg: STRING) is
		-- send message to maintainers and raise exception
	do  glo.system_punt(msg);
	end;

	system_punt_int(msg: STRING; ndx: INTEGER) is
		-- punt with an indicator number
	do  system_punt(su.string_of_3(msg, " = ", ndx.out))
	end;

	system_punt_unk(whr, what, val: STRING) is
		-- message is: whr: unknown what = val
	local msg: STRING;
	do  create msg.make(80);
	    msg.append(whr);  msg.append(": unknown ");
	    msg.append(what); msg.append(" = ");
	    msg.append(val);
	    system_punt(msg)
	end;

feature	-- failsafe message sender

	attempt_to_send_msg(opt, tto, subj, body: STRING) is
		-- (attempt to) send a  message
		-- go quietly on failure
		-- opt are the OUTGOING_MESSAGE mailer options
	local in_retry: BOOLEAN; foo: OUTGOING_MESSAGE;
	do  if (not in_retry) and su.not_empty(tto)
	     then create foo.make_alt_mailer(opt);
		  foo.set_header_field(foo.msg_to, tto);
		  foo.set_header_field(foo.msg_reply_to, tto);
		  foo.set_header_field(foo.msg_subject, subj);
		  foo.set_message_body(body);
		  foo.send;
	    end;
	rescue
	    in_retry := true;  retry;
	end;

end -- GENERIC_SYSTEM_SUPPORT
