
indexing

	author: "David Hollenberg"

class GEOMETRIES_ROW
inherit
	GEOMETRIES_RAW
	ROW_COUNT_CHECK_ACCESS
		export
			{NONE} all
		end
	NULL_VALUE_CHECK_ACCESS
		export
			{NONE} all
		end
	ITEM_COUNT_RANGES_ACCESS
		export
			{NONE} all
		end
	CONDITION_UTILITIES
		export
			{NONE} all
		end
	MOSIS_PROD_VALUES
		export	{NONE} all	end

create
	make_template, make_null, make_by_key, make_detached

feature	-- filename substitution

	final_geometry_filename(sps: SUBSTITUTION_PARAMETER_SET): STRING is
		-- return the substituted geometry_filename
		-- plus extension (unless mebes)
	require have_sps: sps /= Void
	do  Result := sps.substituted_text (geometry_filename)
	    Result.to_lower	
		-- Original or substituted text may be upper case, so
		-- lower case
	    if equal(geometry_format, fmt_is_cif) then
		Result := Result + ".cif"
	    elseif equal(geometry_format, fmt_is_gds) then
		Result := Result + ".gds"
	    end
	end;

feature -- Queries

	payload_rows (wt, fs: STRING conn: DB_CONNECTION range: ITEM_COUNT_RANGE): LIST [like Current] is
			-- Rows for wafer techn `wt' and field set
			-- `fs' that are internal to payload, ordered
			-- by geometry_id
		require
			wt_not_void: wt /= Void
			fs_not_void: fs /= Void
			connection_exists: conn /= Void
			range_exists: range /= Void
		local
			b: DB_BIND_SSB
		do
			create b.make (wt, fs, False)
			Result := all_rows_filtered_ordered (conn, 
				wt_fs_ext_filter, b, "geometry_id")
			check_row_count (Result, range)
		ensure
			result_exists: Result /= Void
		end

	nonpayload_rows (wt, fs: STRING conn: DB_CONNECTION range: ITEM_COUNT_RANGE): LIST [like Current] is
			-- Rows for wafer techn `wt' and field set
			-- `fs' that are external to payload, ordered
			-- by geometry_id
		require
			wt_not_void: wt /= Void
			fs_not_void: fs /= Void
			connection_exists: conn /= Void
			range_exists: range /= Void
		local
			b: DB_BIND_SSB
		do
			create b.make (wt, fs, True)
			Result := all_rows_filtered_ordered (conn, 
				wt_fs_ext_filter, b, "geometry_id")
			check_row_count (Result, range)
		ensure
			result_exists: Result /= Void
		end

feature -- Properties

	fixed_locations (fs: STRING): LIST [GEOMETRYXFIXED_LOCATIONS_ROW] is
			-- Fixed locations for `Current' and
			-- field set `fs'
		require
			fs_not_void: fs /= Void
		do
			if cached_fixed_locations = Void then
				create cached_fixed_locations.make (10)
			end
			cached_fixed_locations.search (fs)
			if cached_fixed_locations.found then
				Result := cached_fixed_locations.found_item
			else
				fetch_fixed_locations (fs)
				cached_fixed_locations.search (fs)
				if cached_fixed_locations.found then
					Result := cached_fixed_locations.found_item
				else
					handle_hard_error ("internal error in fixed_locations")
				end
			end
		end

	has_fixed_locations (fs: STRING): BOOLEAN is
			-- Does `Current' have any fixed locations for
			-- field set `fs'?
		require
			fs_not_void: fs /= Void	
		do
			Result := not fixed_locations (fs).is_empty
		end

	is_cached_fixed_locations (fs: STRING): BOOLEAN is
			-- Are fixed locations for field set
			-- `fs' currently cached?
		require
			fs_not_void: fs /= Void	
		do
			if cached_fixed_locations /= Void then
				cached_fixed_locations.search (fs)
				Result := cached_fixed_locations.found
			end
		end

	quantities (fs: STRING): LIST [GEOMETRY_QUANTITIES_ROW] is
			-- Quantities for `Current' and field set `fs'
		require
			fs_not_void: fs /= Void
		do
			if cached_quantities = Void then
				create cached_quantities.make (10)
			end
			cached_quantities.search (fs)
			if cached_quantities.found then
				Result := cached_quantities.found_item
			else
				fetch_quantities (fs)
				cached_quantities.search (fs)
				if cached_quantities.found then
					Result := cached_quantities.found_item
				else
					handle_hard_error ("internal error in quantities")
				end
			end
		end

	has_quantities (fs: STRING): BOOLEAN is
			-- Does `Current' have any quantities for
			-- field set `fs'?
		require
			fs_not_void: fs /= Void	
		do
			Result := not quantities (fs).is_empty
		end

	is_cached_quantities (fs: STRING): BOOLEAN is
			-- Are quantities for field set `fs' currently cached?
		require
			fs_not_void: fs /= Void	
		do
			if cached_quantities /= Void then
				cached_quantities.search (fs)
				Result := cached_quantities.found
			end
		end

	is_rotatable (fs: STRING): BOOLEAN is
			-- Is vendor geometry represented by `Current'
			-- rotatable for field set `fs'?
		require
			fs_not_void: fs /= Void	
		do
			Result := has_quantities (fs) and then quantities (fs).first.is_rotatable
		end
	
	die_type: STRING is
			-- Die type for `Current'
		local
			type: DIE_TYPE_VALUES
		do
			if external_files then
				Result := type.Vendor_ghost
			else
				Result := type.Vendor_geometry
			end
		end

	non_null_size_x: like size_x is
			-- Value of `size_x', with error check 
			-- for nullness
		do
			check_not_null ("size_x", is_null_size_x, Current)
			Result := size_x
		end

	non_null_size_y: like size_y is
			-- Value of `size_y', with error check 
			-- for nullness
		do
			check_not_null ("size_y", is_null_size_y, Current)
			Result := size_y
		end

	non_null_design_techn: like design_techn is
			-- Value of `design_techn', with error check 
			-- for nullness
		do
			check_not_null ("design_techn", is_null_design_techn, Current)
			Result := design_techn
		end

feature -- Modification

	uncache_fixed_locations is
			-- Clear all fixed locations from cache
		do
			cached_fixed_locations := Void
		end

	uncache_quantities is
			-- Clear all quantities from cache
		do
			cached_quantities := Void
		end

feature {NONE} -- Implementation

	fetch_fixed_locations (fs: STRING) is
			-- Fetch fixed_locations for `Current'
			-- and field set `fs'
		require
			fs_not_void: fs /= Void
		local
			row: GEOMETRYXFIXED_LOCATIONS_ROW
			list: LIST [GEOMETRYXFIXED_LOCATIONS_ROW]
		do
			create row.make_template
			list := row.rows_for_id_and_field_set (geometry_id, fs, connection, expected_count.At_least_zero)
			cached_fixed_locations.put (list, fs)
		end
		
	fetch_quantities (fs: STRING) is
			-- Fetch quantities for `Current'
			-- and field set `fs'
		require
			fs_not_void: fs /= Void
		local
			row: GEOMETRY_QUANTITIES_ROW
			list: LIST [GEOMETRY_QUANTITIES_ROW]
		do
			create row.make_template
			list := row.rows_for_id_and_field_set (geometry_id, fs, connection, expected_count.At_most_one)
			cached_quantities.put (list, fs)
		end
		
	cached_fixed_locations: HASH_TABLE [LIST [GEOMETRYXFIXED_LOCATIONS_ROW], STRING]
			-- Table of cached fixed locations, indexed
			-- by field set name
		
	cached_quantities: HASH_TABLE [LIST [GEOMETRY_QUANTITIES_ROW], STRING]
			-- Table of cached quantities, indexed
			-- by field set name
		
	wt_fs_ext_filter: STRING is
			-- Filter on wafer techn, field set and "external
			-- to payload"
		do
			Result := 
				"wafer_techn = :1 and %N%
				% external_to_payload = :3 and %N%
				% (geometry_id in (select geometry_id from geometry#fixed_locations where field_set_name = :2) or %N%
				%  geometry_id in (select geometry_id from geometry_quantities where field_set_name = :2))"
		ensure
			result_not_void: Result /= Void
		end

end
