
indexing
	
	description: "Object that can handle a condition (error, warning, %
		%information, etc) in a graphical environment"
	author: "David Hollenberg"
	date: "October 26, 2003"

deferred class GRAPHICAL_CONDITION_HANDLER
inherit
	CONDITION_HANDLER
	GUI_UTILITIES_ACCESS
                export
                        {NONE} all
		undefine
			default_create
                end
	ACTIVE_WINDOW_LIST_ACCESS
                export
                        {NONE} all
		undefine
			default_create
                end
	CONTROL_FLOW_EXCEPTIONS_ACCESS
		export
			{NONE} all
		undefine
			default_create
		end
	EV_DIALOG_CONSTANTS
		export
			{NONE} all
		undefine
			default_create
		end
	USER_INTERFACE_ACCESS
		export
			{NONE} all
		undefine
			default_create
		end

feature {NONE} -- Creation

	make (ref: CELL [EV_WINDOW])
		require
			window_ref_exists: ref /= Void
		do
			default_create
			window_ref := ref
		end

feature -- Properties

	is_notify_allowed (cond: APPLICATION_CONDITION): BOOLEAN
			-- Should handler allow user to notify others
			-- of `cond'?
		require
			condition_exists: cond /= Void
		do
			Result := cond.is_notifiable
		end

feature -- Operations

	handle (cond: APPLICATION_CONDITION)
			-- Handle condition `cond'
		local
			done, first, notify: BOOLEAN
			event: CONDITION_EVENT
			sel_text: STRING_32
		do
			condition := cond
			message_text := cond.text_description
			create event
			event.set_condition (cond)
			event.set_message (message_text)
			event.publish
			needs_column_alignment := cond.needs_column_alignment
			is_retry_requested := False
			is_abort_requested := False
			from
				first := True
			until
				done
			loop
				if first then
					notify := is_notify_allowed (cond)
					first := False
				else
					notify := False
				end
				show_dialog (active_window_list.current_window, message_text, notify)
				sel_text := dialog.selected_button
				done := sel_text = Void or else not sel_text.as_string_8.is_equal (Notify_button_text)
			end
			if is_retry_requested then
				cfe.raise_retry_exception
			elseif is_abort_requested then
				cfe.raise_abort_exception
			end
		end

feature {NONE} -- Implementation

	condition: APPLICATION_CONDITION
			-- Condition being handle

	window_ref: CELL [EV_WINDOW]
			-- Reference to current "main" window

	dialog: EV_MESSAGE_DIALOG
			-- Dialog used to report message

	title: STRING
			-- Dialog title

	message_text: STRING
			-- Text of message to be reported

	notify_button_text: STRING = "Notify"
			-- Text on Notify button

	needs_column_alignment: BOOLEAN
			-- Does message contain columns that need to
			-- be aligned properly?

	is_retry_requested: BOOLEAN
			-- Did user request retry of operation that
			-- caused condition?

	is_abort_requested: BOOLEAN
			-- Did user request abort of operation that
			-- caused condition?

	show_dialog (w: EV_WINDOW msg_text: STRING notify: BOOLEAN)
			-- Display modal relative to `w' a
			-- dialog with message `msg_text'.
			-- Provide a Notify button if `notify' is true
		require
			window_exists: w /= Void
			window_active: not w.is_destroyed
			message_text_exists: msg_text /= Void
		deferred
		end

	handle_abort
			-- Handle "abort" response to message
		local
			abort_event: ABORT_EVENT
		do
			create abort_event
			abort_event.set_label (dialog.selected_button)
			abort_event.set_title (title)
			abort_event.publish
			is_abort_requested := True
		end
	
	handle_retry
			-- Handle "retry" response to message
		local
			retry_event: RETRY_EVENT
		do
			create retry_event
			retry_event.set_label (dialog.selected_button)
			retry_event.set_title (title)
			retry_event.publish
			is_retry_requested := True
		end
	
	handle_ignore
			-- Handle "ignore error and continue" response to
			-- message
		local
			ignore_event: IGNORE_EVENT
		do
			create ignore_event
			ignore_event.set_label (dialog.selected_button)
			ignore_event.set_title (title)
			ignore_event.publish
		end
	
	handle_continue
			-- Handle "continue" response
		local
			continue_event: CONTINUE_EVENT
		do
			create continue_event
			continue_event.set_label (dialog.selected_button)
			continue_event.set_title (title)
			continue_event.publish
		end
	
	handle_yes
			-- Handle "yes" response
		local
			boolean_event: BOOLEAN_ANSWER_EVENT
		do
			condition.set_answer (True)
			create boolean_event
			boolean_event.set_label (dialog.selected_button)
			boolean_event.set_title (title)
			boolean_event.publish
		end
	
	handle_no
			-- Handle "no" response
		local
			boolean_event: BOOLEAN_ANSWER_EVENT
		do
			condition.set_answer (False)
			create boolean_event
			boolean_event.set_label (dialog.selected_button)
			boolean_event.set_title (title)
			boolean_event.publish
		end
	
	handle_notify
			-- Handle "notify" response to message
		local
			notify_event: NOTIFY_EVENT
		do
			create notify_event
			notify_event.set_label (dialog.selected_button)
			notify_event.set_title (title)
			notify_event.publish
			notify_actions.call ([message_text])
		end
end
