
indexing
	
	description: "Set of condition handlers for a graphical application"
	author: "David Hollenberg"
	date: "October 26, 2003"

class GRAPHICAL_CONDITION_HANDLER_SET
inherit
	CONDITION_HANDLER_SET
	
create
	make

feature {NONE} -- Creation

	make (win_ref: CELL [EV_WINDOW])
		require
			window_ref_exists: win_ref /= Void
		do
			current_window_ref := win_ref
		end

feature -- Properties

	current_window_ref: CELL [EV_WINDOW]
			-- Reference to current main window

	hard_error_handler: GRAPHICAL_CONDITION_HANDLER
			-- Handler for errors that cannot be
			-- retried and cannot be ignored
		once
			create {GUI_HARD_ERROR_HANDLER} Result.make (current_window_ref)
		end

	retryable_hard_error_handler: GRAPHICAL_CONDITION_HANDLER
			-- Handler for errors that can be retried
			-- but cannot be ignored
		once
			create {GUI_HARD_RETRY_ERROR_HANDLER} Result.make (current_window_ref)
		end

	soft_error_handler: GRAPHICAL_CONDITION_HANDLER
			-- Handler for errors that cannot be
			-- retried but can be ignored
		once
			create {GUI_SOFT_ERROR_HANDLER} Result.make (current_window_ref)
		end

	retryable_soft_error_handler: GRAPHICAL_CONDITION_HANDLER
			-- Handler for errors that can be retried
			-- and can be ignored
		once
			create {GUI_SOFT_RETRY_ERROR_HANDLER} Result.make (current_window_ref)
		end

	warning_handler: GRAPHICAL_CONDITION_HANDLER
			-- Handler for warnings that cannot be
			-- retried but can be ignored
		once
			create {GUI_WARNING_HANDLER} Result.make (current_window_ref)
		end

	retryable_warning_handler: GRAPHICAL_CONDITION_HANDLER
			-- Handler for awrnings that can be retried
			-- and can be ignored
		once
			create {GUI_RETRY_WARNING_HANDLER} Result.make (current_window_ref)
		end

	question_handler: GRAPHICAL_CONDITION_HANDLER
			-- Handler that asks users a question and gets
			-- a Yes or No answer or allows user to cancel
		once
			create {GUI_QUESTION_HANDLER} Result.make (current_window_ref)
		end

	confirmation_handler: GRAPHICAL_CONDITION_HANDLER
			-- Handler that tells user something and
			-- allows them to either confirm that they want
			-- to continue or to cancel
		once
			create {GUI_CONFIRMATION_HANDLER} Result.make (current_window_ref)
		end

	information_handler: GRAPHICAL_CONDITION_HANDLER
			-- Handler that tells user something
		once
			create {GUI_INFORMATION_HANDLER} Result.make (current_window_ref)
		end

invariant
	window_ref_exists: current_window_ref /= Void
end
