
indexing
	
	description: "Facilitities useful in a graphical user interface %
		%environment";
	author: "David Hollenberg";
	date: "November 5, 2002"

class GUI_UTILITIES
inherit
	EV_STOCK_COLORS
	EV_STOCK_PIXMAPS
		rename
			implementation as pixmap_imp
		end
	EV_FONT_CONSTANTS
	EV_DIALOG_CONSTANTS
	EV_KEY_CONSTANTS
	STRING_UTILITIES_ACCESS
		export
			{NONE} all
		end
	EXCEPTION_MANAGER
		export
			{NONE} all
		end

feature -- Operations

	process_events is
			-- Process application events if application
			-- has not been destroyed
		do
			if application.is_launched and not application.is_destroyed then
				application.process_events
			end
		end

	do_once_on_idle (an_action: PROCEDURE [ANY, TUPLE []]) is
			-- Do `an_action' once when application is idle
		do
			application.do_once_on_idle (agent protected_action (an_action))
		end

	set_box_background_recursive (a_box: EV_BOX a_color: EV_COLOR) is
			-- Set background of `a_box' and recursively all
			-- boxes and labels it contains to `a_color'
		require
			a_box_exists: a_box /= Void
			a_color_exists: a_color /= Void
		local
			b: EV_BOX
			label: EV_LABEL
			chk_box: EV_CHECK_BUTTON
		do
			a_box.set_background_color (a_color)
			from
				a_box.start
			until
				a_box.after
			loop
				b ?= a_box.item
				label ?= a_box.item
				chk_box ?= a_box.item
				if b /= Void then
					set_box_background_recursive (b, a_color)
				elseif label /= Void then
					label.set_background_color (a_color)
				elseif chk_box /= Void then
					chk_box.set_background_color (a_color)
				end
				a_box.forth
			end
			
		end

	ring_bell (a_count, a_delay: INTEGER) is
			-- Ring bell `a_count' times on default
			-- display, sleeping `a_delay' milliseconds
			-- between rings
		require
			a_delay_nonnegative: a_delay >= 0
		local
			n: INTEGER
		do
			from
				n := 1
			until
				n > a_count
			loop
				ring_bell_once
				
				environment.application.sleep (a_delay)
				n := n + 1
			end
		end

	ring_bell_once is
			-- Ring bell once on default display
		do
			beep.exclamation
		end

feature -- Modification

	add_accelerator (w: EV_TITLED_WINDOW a_code: INTEGER require_control, require_alt, require_shift: BOOLEAN an_action: PROCEDURE [ANY, TUPLE]) is
			-- Add accelerator for key with code `a_code'
			-- that invokes `an_action' to `w'
		require
			window_exists: w /= Void
			window_active: not w.is_destroyed
			valid_code: valid_key_code (a_code)
			action_exists: an_action /= Void
		local
			key: EV_KEY
			accelerator: EV_ACCELERATOR
		do
			create key.make_with_code (a_code)
			create accelerator.make_with_key_combination (key, require_control, require_alt, require_shift)
			accelerator.actions.extend (an_action)
			w.accelerators.extend (accelerator)
		end

	add_ok_accelerator (w: EV_TITLED_WINDOW an_action: PROCEDURE [ANY, TUPLE]) is
			-- Add to `w' an accelerator for the Enter key 
			-- that invokes `an_action'
		require
			window_exists: w /= Void
			window_active: not w.is_destroyed
			action_exists: an_action /= Void
		do
			add_accelerator (w, Key_enter, False, False, False, an_action)
		end

	add_cancel_accelerator (w: EV_TITLED_WINDOW an_action: PROCEDURE [ANY, TUPLE]) is
			-- Add to `w' an accelerator for the Escape key 
			-- that invokes `an_action'
		require
			window_exists: w /= Void
			window_active: not w.is_destroyed
			action_exists: an_action /= Void
		do
			add_accelerator (w, Key_escape, False, False, False, an_action)
		end

	destroy_on_close_request (w: EV_TITLED_WINDOW) is
			-- Add to `w' a close request action to destroy the
			-- application
		require
			window_exists: w /= Void
			window_active: not w.is_destroyed
		do
			w.close_request_actions.extend (destroy_application_agent)
		end

	center_window (w: EV_TITLED_WINDOW) is
			-- Center `w' on screen
		require
			window_exists: w /= Void
			window_active: not w.is_destroyed
		do
			center_window_with_size (w, w.width, w.height)
		end

	center_window_with_size (w: EV_TITLED_WINDOW a_width, a_height: INTEGER) is
		require
			window_exists: w /= Void
			window_active: not w.is_destroyed
			a_width_nonnegative: a_width >= 0
			a_height_nonnegative: a_height >= 0
		local
			screen: EV_SCREEN
		do
			w.set_size (a_width, a_height)
			create screen
			w.set_position ((screen.width - a_width) // 2, (screen.height - a_height) // 2)
		end

	center_window_with_margin (w: EV_TITLED_WINDOW margin_x, margin_y: INTEGER) is
		require
			window_exists: w /= Void
			window_active: not w.is_destroyed
			margin_x_nonnegative: margin_x >= 0
			margin_y_nonnegative: margin_y >= 0
		local
			a_width, a_height: INTEGER
			screen: EV_SCREEN
		do
			create screen
			a_width := (screen.width - 2 * margin_x).max (0)
			a_height := (screen.height - 2 * margin_y).max (0)
			center_window_with_size (w, a_width, a_height)
		end

	center_window_with_vertical_margin (w: EV_TITLED_WINDOW margin_y: INTEGER) is
		require
			window_exists: w /= Void
			window_active: not w.is_destroyed
			margin_y_nonnegative: margin_y >= 0
		local
			a_width, a_height: INTEGER
			screen: EV_SCREEN
		do
			create screen
			a_width := w.width
			a_height := (screen.height - 2 * margin_y).max (0)
			center_window_with_size (w, a_width, a_height)
		end

	center_window_with_horizontal_margin (w: EV_TITLED_WINDOW margin_x: INTEGER) is
		require
			window_exists: w /= Void
			window_active: not w.is_destroyed
			margin_y_nonnegative: margin_x >= 0
		local
			a_width, a_height: INTEGER
			screen: EV_SCREEN
		do
			create screen
			a_width := (screen.width - 2 * margin_x).max (0)
			a_height := w.height
			center_window_with_size (w, a_width, a_height)
		end

	abut_window_bottom_left_with_size (w: EV_TITLED_WINDOW a_width, a_height: INTEGER) is
		require
			window_exists: w /= Void
			window_active: not w.is_destroyed
			a_width_nonnegative: a_width >= 0
			a_height_nonnegative: a_height >= 0
		local
			screen: EV_SCREEN
		do
			w.set_size (a_width, a_height)
			create screen
			-- FIXME: title bar height not available in Vision2 
			-- yet so use a constant value for now
			w.set_position (0, screen.height - (a_height + title_bar_height))
		end

feature -- Properties

	destroy_application_agent: PROCEDURE [ANY, TUPLE]  is
			-- Agent which destroys application
		once
			Result := agent (environment.application).destroy
		ensure
			result_exists: Result /= Void
		end

	application: EV_APPLICATION is
			-- Access to the application
		once
			Result := environment.application
		end
	
	environment: EV_ENVIRONMENT is
			-- Access to the global environment in which
			-- application executes
		once
			create Result
		end
	

feature -- Generation

	label_with_text (s: STRING): EV_LABEL is
			-- Color corresponding to `s' or void if not
			-- a valid color specification
		require
			text_exists: s /= Void
		do
			create Result.make_with_text (s)
		end

	rgb_string_to_color (s: STRING): EV_COLOR is
			-- Color corresponding to `s' or void if not
			-- a valid color specification
		local
			intensities: ARRAY [STRING]
			rs, gs, bs: STRING
			r, g, b: REAL
		do
			if s /= Void then
				intensities := su.as_array (s)
				if intensities.count = 3 then
					rs := intensities.item (1)
					gs := intensities.item (2)
					bs := intensities.item (3)
					if rs.is_real and gs.is_real and bs.is_real then
						r := rs.to_real
						g := gs.to_real
						b := bs.to_real
						if r >= 0 and r <= 1 and g >= 0 and g <= 1 and b >= 0 and b <= 1 then
							create Result.make_with_rgb (r, g, b)
						end
					end
				end
			end
		end

	to_ev_color (a_color: RGB_COLOR): EV_COLOR is
			-- Convert `a_color' to an EV_COLOR
			-- expand, with `a_text' and `a_tooltip' that
			-- calls `an_action' when clicked.  Leave new
			-- button in `last_button'
		require
			a_color_not_void: a_color /= Void
		do
			create Result.make_with_rgb (a_color.red_value, a_color.green_value, a_color.blue_value)
		ensure
			result_not_void: Result /= Void
		end

	color_name_8_bit (a_color: EV_COLOR): STRING  is
			-- Name for  `a_color' as 8-bit name
		require
			a_color_not_void: a_color /= Void
		do
			create Result.make (15)
			Result.extend ('(')
			Result.append_integer (a_color.red_8_bit)
			Result.append (", ")
			Result.append_integer (a_color.green_8_bit)
			Result.append (", ")
			Result.append_integer (a_color.blue_8_bit)
			Result.extend (')')
		ensure
			result_not_void: Result /= Void
		end

feature {NONE} -- Implementation

	protected_action (an_action: PROCEDURE [ANY, TUPLE []]) is
			-- Call `an_action' with protection against
			-- exceptions.  If exception occurs, call
			-- `application'.`uncaught_exception_actions'
		require
			an_action_not_void: an_action /= Void
		local
			tried: BOOLEAN
			e: EXCEPTION
		do
			if not tried then
				an_action.call ([])
			elseif not uncaught_exception_actions_called then
				e := last_exception
				uncaught_exception_actions_called := True
				application.uncaught_exception_actions.call ([e])
				uncaught_exception_actions_called := False
			else
				uncaught_exception_actions_called := False
			end
		rescue
			tried := True
			retry
		end

	beep: EV_BEEP is
		once
			create Result
		end

	uncaught_exception_actions_called: BOOLEAN
			-- Are the `uncaught_exceptions_actions' 
			-- currently being called?  Used to prevent
			-- infinite looping if an exception is raised as
			-- part of calling `uncaught_exception_actions'

	title_bar_height: INTEGER is 36
			-- FIXME: This is a hack.
			-- Approximate height of window title bar in pixels, 
			-- since not available in Vision2 yet
end
