
indexing
	
	description: "Set of horizontal push buttons, with associated %
		%select actions"
	author: "David Hollenberg";
	date: "November y, 2003"

class HORIZONTAL_BUTTON_SET
inherit
	ANY
	GUI_UTILITIES_ACCESS
		export
			{NONE} all
		end
	USER_INTERFACE_ACCESS
		export
			{NONE} all
		end

create
	make_with_text_and_actions

feature {NONE} -- Creation

	make_with_text_and_actions (labels: ARRAY [STRING] actions: ARRAY [PROCEDURE [ANY, TUPLE]]) is
		require
			label_list_exists: labels /= Void
			action_list_exists: actions /= Void
			same_list_lengths: labels.count = actions.count
			label_list_nonempty: labels.count > 0
		local
			text: STRING
			action: PROCEDURE [ANY, TUPLE]
			button: EV_BUTTON
			k: INTEGER
			buttons: like column_set
		do
			create button_list.make (1, labels.count)
			from
				k := labels.lower
			until
				k > labels.upper
			loop
				text := labels.item (k)
				action := actions.item (k + actions.lower - labels.lower)
				create button.make_with_text (text)
				button.set_minimum_width (user_interface.Minimum_button_width)
				button.select_actions.extend (action)
				button_list.put (button, k - labels.lower + 1)
				k := k + 1
			end
			create buttons.make (labels.count)
			buttons.set_padding (user_interface.padding)
			buttons.set_border_width (30)
			buttons.append_row (button_list)
			from
				k := 1
			until
				k > button_list.count
			loop
				buttons.disable_column_expand (k)
				k := k + 1
			end
			buttons.align_center
			column_set := buttons
		end

feature -- Properties

	column_set: WIDGET_COLUMN_SET
			-- Buttons with requested layout

	i_th (n: INTEGER): EV_BUTTON is
			-- Button `n'
		require
			valid_index: is_valid_index (n)
		do
			Result := button_list.item (n)
		ensure
			result_exists: Result /= Void
		end

	is_valid_index (n: INTEGER): BOOLEAN is
			-- Is `n' a valid index into button set?
		do
			Result := n >= button_list.lower and n <= button_list.upper
		end

			
feature {NONE} -- Implementation

	button_list: ARRAY [EV_BUTTON]
	
end
