
indexing
	
	description: "HP_AMOS14TB wafer techn family"
	author: "David Hollenberg";
	date: "June 18, 2003"

class HP_AMOS14TB

inherit
	WAFER_TECHN_FAMILY
	redefine
		is_dynamic_vendor_geometry,
		build_vendor_geometry,
		add_explicit_cats_statements
	end;

	LAYOUT_SUPPORT

feature -- Properties

	name: STRING is "HP_AMOS14TB"

	native_design_techn: STRING is
			-- name of the "native" design_techn
	do  Result := dtv.hp_cmos14
	end;

feature	-- dynamic vendor geometry support (none unless redefined)

	is_dynamic_vendor_geometry(d: DIES_XROW): BOOLEAN is
		-- indicates whether this die's geometry item is static (files 
		--  already in place) or dynamic (files generated per run)
	do
	    Result := equal(d.geometry_id, gids.avago_part_id)
	end;

	build_vendor_geometry(	g: GEOMETRIES_ROW;
				dt: DESIGN_TECHNOLOGY;
				lim: CONST_RECTANGLE): LAYOUT is
		-- only dynamic is avago_part_id
	local bx: RECTANGLE; wpid: STRING
	do  if equal(g.geometry_id, gids.avago_part_id) then
		-- the AVAGO part id as text, applied to all levels
		wpid := cache.wafer_part_id.twin;
		if wpid.count > 5 and then (wpid @ 6) = '-' then
			-- 1RR8W-1234 --> just 1RR8W
			wpid.keep_head(5)
		end;
		create bx.make_copy(lim);
		bx.size_by(-2, -2);		-- shrink 1.0/side
		Result := text (wpid)
		Result := Result.nicely_fitting_in (bx)
		Result := multi_layer (Result, dt.non_cmnt_layers)
	    else
		handle_hard_error("HP_AMOS14TB: no geometry build for " + g.geometry_id)
	    end;
	end;

feature	-- Cats fracture recipe stuff

	add_explicit_cats_statements (d: DIES_XROW)
		-- create all of the explicitly-specified
		--  Cats statements required to fracture d,
		--  and add them onto all_stmts.
	local techn: STRING
	do  techn := d.wafer_design_techn;
	    if equal(techn, dtv.scn3mlc_035) or	-- lambda = 0.35
	       equal(techn, dtv.scn3mlc_subm_03)	-- lambda = 0.30
	     then
		shrink_pfet_poly(d);
		well_and_nlcs(d);
		active_dogbone(d);
		selects_ala_mosis(d);
		cap_implant_ala_mosis(d);
	    elseif equal(techn, dtid.hp_cmos14) then
		shrink_pfet_poly(d);
		well_and_nlcs(d);
		island_ala_hp(d)
		selects_ala_hp(d);
		cap_implant_ala_hp(d);
	    elseif equal(techn, dtid.hp_cmos14_mebes) then
		-- nada
	     else
		unknown_design_techn(techn)
	    end;
	end;

feature {NONE} -- details of the fracturing

	shrink_pfet_poly(d: DIES_XROW)
		-- shrink pfet poly by an extra 0.1 um
		-- we actually shrink ALL poly, then or back in the 
		--  regions not being shrunk to avoid gapping
	local allpoly, psd, ppoly: CATS_EXPRESSION
	do  allpoly := leaf(lyr.poly)
	    if is_scmos_die(d) then
		psd := leaf(lyr.active) #and# leaf(lyr.n_well)
	    else
		psd := leaf(lyr.p_island);
		if d.is_present(lyr.int_island) then
			psd := psd #or#
				(leaf(lyr.int_island) #and# leaf(lyr.n_well))
		end
	    end;
	    ppoly := (allpoly #and# psd) #sized#
			(0.4 + bias_differential(d, lvl.poly) +
				bias_differential(d, lvl.island));

	    add_clevel_with_bias(
			(allpoly #sized# -0.1) #or# (allpoly #minus# ppoly), 
			lvl.poly, d)
	end;

	well_and_nlcs(d: DIES_XROW)
		-- do the well and nlcs masks
	local well, nlcs: CATS_EXPRESSION
	do  well := leaf(source_layer_name(d, lvl.n_well))
	    if d.is_present (lyr.cap_well) then		-- SCMOS only
		well := well #or# leaf(lyr.cap_well)
	    end;
	    nlcs := well;
	    if d.is_present(lyr.explicit_nlcs) then	-- AMOS14 only
		nlcs := nlcs #or# leaf(lyr.explicit_nlcs)
	    elseif d.is_present(lyr.expl_field_implant) then	-- SCMOS
		nlcs := nlcs #xor# leaf(lyr.expl_field_implant)
	    end;
	    if d.is_present(lyr.well_only) then		-- AMOS14 only
		well := well #or# leaf(lyr.well_only)
	    end;
	    add_clevel_with_bias(well, lvl.n_well, d)
	    add_clevel_with_merge(nlcs, lvl.nlcs, 0.9, d)
	end;

	island_ala_hp(d: DIES_XROW)
	local isl: CATS_EXPRESSION
	do  isl := leaf(lyr.p_island) #or# leaf(lyr.n_island);
	    if d.is_present(lyr.int_island) then
		isl := isl #or# leaf(lyr.int_island)
	    end;
	    if d.is_present(lyr.sub_island) then
		isl := isl #or# leaf(lyr.sub_island)
	    end;
	    if d.is_present(lyr.cap_island) then
		isl := isl #or# leaf(lyr.cap_island)
	    end;
	    add_clevel_with_bias(isl, lvl.island, d)
	end;

	selects_ala_hp(d: DIES_XROW)
	local mask, well, npls, nmns, ppls, pmns: CATS_EXPRESSION
	do  well := leaf(lyr.n_well);
	    -- the positive factors
	    npls := leaf(lyr.n_island);
	    ppls := leaf(lyr.p_island);
	    -- and the negative erasers
	    nmns := ppls;
	    pmns := npls;
	    if d.is_present(lyr.cap_island) then
		npls := npls #or# leaf(lyr.cap_island)
	    end;
	    if d.is_present(lyr.sub_island) then
		npls := npls #or# leaf(lyr.sub_island);
		ppls := ppls #or# leaf(lyr.sub_island)
	    end;
	    if d.is_present(lyr.int_island) then
		nmns := nmns #or# leaf(lyr.int_island);
		pmns := pmns #or# leaf(lyr.int_island)
	    end;
	    -- remove the erasers from their erasees
	    npls := (npls #minus# (nmns #sized# 0.8)) #sized# 0.8;
	    ppls := (ppls #minus# (pmns #sized# 0.8)) #sized# 0.8;
	    -- now apply the in-well vs out-well distinction
	    mask := ((well #and# npls) #or# leaf(lyr.explicit_nplus))
			#or# (#not#
		    ((well #or# ppls) #or# leaf(lyr.explicit_pplus)));
	    mask := mask #sized# 0.8 #sized# -1.6 #sized# 0.8;
	    -- mask is now the mask n_plus
	    add_ctemp(mask, select_temp, 0.05)
	    add_clevel_with_bias(      leaf(select_temp), lvl.n_plus_select, d)
	    add_clevel_with_bias(#not# leaf(select_temp), lvl.p_plus_select, d)
	end;

	select_temp: STRING = "SELECT_TEMP"

	cap_implant_ala_hp(d: DIES_XROW)
	local mask: CATS_EXPRESSION
	do  mask := (leaf(lyr.cap_well) #minus#
		     ((leaf(lyr.cap_island) #and# leaf(lyr.contact)) #sized# 0.6)
		    ) #sized# -1.0 #sized# 1.2;
	    add_clevel(mask, lvl.cap_implant, d)
	end;

	active_dogbone(d: DIES_XROW)
		-- extra contact overlap needed for subm only
	local actv, mask: CATS_EXPRESSION
	do  if equal(d.wafer_design_techn, dtv.scn3mlc_subm_03) then
		actv := leaf(lyr.active)
		mask := actv #or#
			((actv #and# leaf(lyr.contact)) #sized#
			  (0.4 + bias_differential(d, lvl.contact)
				- bias_differential(d, lvl.island)))
		add_clevel_with_merge(mask, lvl.island, 0.4, d)
	    end
	end;

	selects_ala_mosis(d: DIES_XROW)
	local hn, hp: BOOLEAN; npls, ppls: CATS_EXPRESSION
	do  -- optional one-select synthesis from the other select
	    hn := d.is_present(lyr.n_plus_select);
	    hp := d.is_present(lyr.p_plus_select);
	    if hp and not hn then
		npls := #not# leaf(lyr.p_plus_select)
	    else
		npls := leaf(lyr.n_plus_select)
		if d.is_present(lyr.cap_well) then
		    npls := npls #or#
			    ((leaf(lyr.active) #and# leaf(lyr.cap_well))
				#sized# 0.8)
		end
	    end
	    if hn and not hp then
		ppls := #not# npls
	    else
		ppls := leaf(lyr.p_plus_select)
	    end;
	    -- and go
	    add_clevel_with_merge_X2(npls, lvl.n_plus_select, 0.8, d);
	    add_clevel_with_merge_X2(ppls, lvl.p_plus_select, 0.8, d);
	end;

	cap_implant_ala_mosis(d: DIES_XROW)
	local mask: CATS_EXPRESSION
	do  mask := leaf(lyr.cap_well) #minus#
		    ((leaf(lyr.active) #and# leaf(lyr.contact)) #sized# 0.8);
	    if d.is_present(lyr.cap_implant) then
		mask := mask #or# leaf(lyr.cap_implant)
	    end;
	    mask := mask #sized# -1.0 #sized# 1.2;
	    add_clevel(mask, lvl.cap_implant, d)
	end;

end
