class HTML_STRING_MODIFIER
		-- the STRING_MODIFIER to quote various funny chars for HTML

inherit
	OBJECT_MODIFIER[STRING]

feature -- the OBJECT_MODIFIER features

	needs_modification(s: STRING): BOOLEAN is
		-- determine whether or not s needs to be fixed
	do  Result := s.has(char_is_gt) or s.has(char_is_lt) or
		      s.has(char_is_quot) or s.has(char_is_amp) or
		      s.has(char_is_nl) or 
		      s.substring_index (Http_prefix, 1) > 0 or
		      s.substring_index (Https_prefix, 1) > 0
	end;

	make_modified_object(s: STRING): STRING is
		-- make a NEW string with the appropriate modifications
	local i, cnt: INTEGER; c: CHARACTER
	do  from  i := 1;
		  cnt := s.count;
		  create Result.make(cnt+4);		-- size is a guess
	    until i > cnt
	    loop  c := s.item(i);
		  if     c = char_is_gt   then Result.append(str_for_gt)
		  elseif c = char_is_lt   then Result.append(str_for_lt)
		  elseif c = char_is_quot then Result.append(str_for_quot)
		  elseif c = char_is_amp  then Result.append(str_for_amp)
		  elseif c = char_is_nl   then Result.append(str_for_nl)
		  else Result.extend(c)
		  end;
		  i := i + 1
	    end;
	    anchor_urls (Result)
	end;

	make_preformatted_object(s: STRING style: STRING): STRING is
		-- make a NEW string with the appropriate modifications
		-- quote most special characters preserve typeface adjustments
	local
	    temp_str: STRING;
	    i, cnt: INTEGER;
	    c: CHARACTER;
	do
	    create Result.make(s.count+4);		-- size is a guess
	    if style /= Void then
		Result.extend (char_is_lt);
		Result.append (style);
		Result.extend (char_is_gt)
	    end;
	    from
		i := 1;
		cnt := s.count;
	    until
		i > cnt
	    loop
		c := s.item(i);
		if c = char_is_lt and i + 1 < cnt then
		    -- may start or end a sequence
		    if s.item (i+1) = '/' then
			temp_str := select_special(s,i+2);
			if temp_str = Void then
			    Result.append (str_for_lt)
			else
			    Result.extend (c);
			    Result.extend ('/');
			    Result.append (temp_str);
			    i := i + temp_str.count + 1;
			end
		    else
			temp_str := select_special(s,i+1);
			if temp_str = Void then
			    Result.append (str_for_lt)
			else
			    Result.extend (c);
			    Result.append (temp_str);
			    i := i + temp_str.count;
			end
		    end
		elseif c = char_is_lt then	-- lt at end of string
		    Result.append(str_for_lt)
		elseif c = char_is_gt then
		    Result.append(str_for_gt)
		elseif c = char_is_quot then
		    Result.append(str_for_quot)
		elseif c = char_is_amp then
		    Result.append(str_for_amp)
		elseif Acceptable_controls.has(c) then
		    Result.extend(c)
		elseif is_control(c) then
		    Result.append(character_reference_string(c)) -- the best we can do
		elseif c.code = 127 then
		    Result.append(character_reference_string(c)) -- the best we can do
		else
		    Result.extend(c)
		end;
		i := i + 1
	    end;
	    if style /= Void then
		Result.extend (char_is_lt);
		Result.extend ('/');
		Result.append (style);
		Result.extend (char_is_gt)
	    end;
	end;


feature {NONE} -- Implementation constants

	char_is_gt:   CHARACTER is '>'
	char_is_lt:   CHARACTER is '<'
	char_is_quot: CHARACTER is '%"'
	char_is_amp:  CHARACTER is '&'
	char_is_nl:   CHARACTER is '%N'

	str_for_gt:   STRING is "&gt;"
	str_for_lt:   STRING is "&lt;"
	str_for_quot: STRING is "&quot;"
	str_for_amp:  STRING is "&amp;"
	str_for_nl:   STRING is "<BR>"

	Http_prefix: STRING is "http://"
	Https_prefix: STRING is "https://"
	Url_specials: STRING is "#-_./:?="
	Ref_prefix:  STRING is "&#"

	Acceptable_controls: STRING is "%R%N%T"

Feature {NONE} -- Implementation

	is_control (c: CHARACTER): BOOLEAN is
	    local
		ccode: INTEGER
	    do
		ccode := c.code
		-- Result := (ccode // 32) in (0,4)
		Result := ccode < 32 or ((ccode >= 128) and (ccode  < 160))
	    end

	character_reference_string (c: CHARACTER): STRING is
	    do
		create Result.make(6)
		Result.append(Ref_prefix)
		Result.append(c.code.out)
		Result.extend(';')
	    end

	anchor_urls (s: STRING) is
			-- Change all URLs in `s' so that they are
			-- anchored, thus making them links in the
			-- browser
		do
			anchor_some_urls (s, Http_prefix)
			anchor_some_urls (s, Https_prefix)
		end

	anchor_some_urls (s, url_prefix: STRING) is
			-- Change all URLs in `s' which start with
			-- `url_prefix' so that they are anchored,
			-- thus making them links in the browser
		local
			pos, start_pos: INTEGER
			next_url, anchored_url: STRING
		do
			from
				pos := s.substring_index (url_prefix, 1)
			until
				pos = 0
			loop
				next_url := url_at_position (s, pos, url_prefix.count)
				anchored_url := "<A HREF=%"" + next_url + "%">" + next_url + "</A>"
				s.replace_substring (anchored_url, pos, pos + next_url.count - 1)
				start_pos := pos + anchored_url.count
				if start_pos <= s.count then
					pos := s.substring_index (url_prefix, start_pos)
				else
					pos := 0
				end
			end
		end

	url_at_position (s: STRING start_pos, prefix_len: INTEGER): STRING is
			-- URL in `s' which starts at position `start_pos'
			-- and has prefix that is `prefix_len' characters
			-- long
		local
			pos, count: INTEGER
		do
			from
				count := s.count
				pos := start_pos + prefix_len
			until
				pos > count or else not mosis_url_character (s.item (pos))
			loop
				pos := pos + 1
			end
			if s.item (pos-1) = '.' then pos := pos - 1 end;
			Result := s.substring (start_pos, pos - 1)
		end

	mosis_url_character (c: CHARACTER): BOOLEAN is
			-- Is `c' a valid character in a MOSIS URL?
		do
			Result := c.is_alpha or c.is_digit or Url_specials.has (c)
		end

	select_special(s: STRING i: INTEGER): STRING is
	    local
		istr: STRING;
	    do
		istr := s.substring(i,i+10);
		istr.to_upper;
		if Bold_text.is_equal (istr.substring(1,Bold_text.count)) then
		    Result := Bold_text
		elseif Emphasize_text.is_equal (istr.substring(1,Emphasize_text.count)) then
		    Result := Emphasize_text
		elseif Italics_text.is_equal (istr.substring(1,Italics_text.count)) then
		    Result := Italics_text
		elseif S_text.is_equal (istr.substring(1,S_text.count)) then
		    Result := S_text
		elseif Strike_text.is_equal (istr.substring(1,Strike_text.count)) then
		    Result := Strike_text
		elseif Strong_text.is_equal (istr.substring(1,Strong_text.count)) then
		    Result := Strong_text
		elseif Tt_text.is_equal (istr.substring(1,Tt_text.count)) then
		    Result := Tt_text
		elseif U_text.is_equal (istr.substring(1,U_text.count)) then
		    Result := U_text
		end
	    end

feature {NONE} -- strings to preserve in preformatted html

	Bold_text:	STRING is "B>"
	Emphasize_text:	STRING is "EM>"
	Italics_text:	STRING is "I>"
	S_text:		STRING is "S>"
	Strike_text:	STRING is "STRIKE>"
	Strong_text:	STRING is "STRONG>"
	Tt_text:	STRING is "TT>"
	U_text:		STRING is "U>"

end -- HTML_STRING_MODIFIER
