
indexing

	author: "Generated"
	date: "Fri Feb 27 10:32:55 PST 2004"

class IC_FAB_INSTANCES_ROW
inherit
	IC_FAB_INSTANCES_RAW

	DB_OPTIONS_ACCESS
		export
			{NONE} all
		end

	MOSIS_PROD_VALUES
		export {NONE} all end;

	ROW_COUNT_CHECK_ACCESS

	STRING_UTILITIES_ACCESS
		export
			{NONE} all
		end
	ITEM_COUNT_RANGES_ACCESS
		export
			{NONE} all
		end
create
	make_template, make_null, make_by_key

feature

	is_on_run: BOOLEAN is
		-- am I actually on a run yet (even if not visible)?
	do  Result := not is_null_run_id
	end;

	can_be_assigned_to_run (s: STRING): BOOLEAN is
			-- Can `Current' be assigned to run `s'?
			-- True if unassigned or if already assigned to `s'
		do
			Result := is_null_run_id or run_id.is_equal (s)
		end

	is_frozen: BOOLEAN is
		-- am I frozen (no changes allowed)
	do  Result := pending_for_run or is_on_run
	end;

	fab_id: STRING is
	require is_on_run	-- (weaker than just has_fab_id)
	do  Result := su.string_of_2(run_id, die_id)
	end;

	funding_po_id: STRING is
		-- the auth_id to use in find_funding_alloc
	do  if is_null_approved_funding_po_id
	     then Result := submitted_funding_po_id
	     else Result := approved_funding_po_id
	    end
	end;

	ordered_wafer_process: STRING is
			-- The ordered wafer process (empty string if none,
			-- first process name in alphabetical order if
			-- multiple ordered processes)
		require 
			is_on_run: is_on_run
		local
			procs: LIST [STRING]
		do 
			procs := ordered_wafer_processes
			if not procs.is_empty then
				Result := procs.first
			else
				Result := ""
			end
		end

	ordered_wafer_processes: LIST [STRING] is
			-- List of wafer processes for `Current'
		require 
			is_on_run: is_on_run
		local 
			b: DB_BIND_I
		do  
			create b.make (m_fab_no);
	    		Result := connection.selected_strings (
				"wafer_process",
				"ic_fab_instance_processes",
				"m_fab_no = :1",
				"wafer_process", False, b)
		end

	is_customer_project: BOOLEAN is
			-- Is `Current' a customer project?
		local
			type: IC_FAB_INSTANCE_TYPE_VALUES
		do  
			Result := ic_fab_instance_type.is_equal (type.Customer_fab)
		end

feature -- run_type_requested stuff

	rtr_word: STRING is
		-- the run_type_word for the run_type_requested
	local b: DB_BIND_S
	do  create b.make(run_type_requested)
	    Result := connection.safe_selected_string
				("run_type_word", "run_types",
				 "run_type = :1", b)
	end;

	rtr_is_scheduled: BOOLEAN is
	do  Result := equal(run_type_requested, rtype_is_scheduled)
	end;

	rtr_is_dedicated: BOOLEAN is
	do  Result := equal(run_type_requested, rtype_is_dedicated)
	end;

	rtr_is_trusted_foundry: BOOLEAN is
	do  Result := equal(run_type_requested, rtype_is_trusted_foundry)
	end;

	rtr_is_development: BOOLEAN is
	do  Result := equal(run_type_requested, rtype_is_development)
	end;

	is_run_date_required: BOOLEAN is
		-- does this run_type require a run_date_requested?
	local b: DB_BIND_S
	do  create b.make(run_type_requested)
	    Result := connection.selected_boolean
				("is_run_date_required", "run_types",
				 "run_type = :1", b)
	end;

feature -- Queries

	fab_instances_for_run (run: STRING conn: DB_CONNECTION range: ITEM_COUNT_RANGE): LIST [like Current] is
			-- All fab instances for run `run'
		require
			run_exists: run /= Void
			connection_exists: conn /= Void
			connection_open: conn.is_open
			range_exists: range /= Void
		local
			b: DB_BIND_S
		do
			create b.make (run)
			Result := all_rows_filtered (conn, "run_id = :1", b)
			check_row_count (Result, range)
		ensure
			result_exists: Result /= Void
		end

	row_for_run_and_die (run, die: STRING conn: DB_CONNECTION range: ITEM_COUNT_RANGE): like Current is
			-- Fab instance for run `run' and die `die' or
			-- Void if none
		require
			run_exists: run /= Void
			die_exists: die /= Void
			connection_exists: conn /= Void
			connection_open: conn.is_open
			range_exists: range /= Void
		local
			b: DB_BIND_SS
			list: LIST [like Current]
		do
			create b.make (run, die)
			list := all_rows_filtered (conn, "run_id = :1 and die_id = :2", b)
			check_row_count (list, range)
			if not list.is_empty then
				Result := list.first
			end
		end

	rows_for_design_no (dno: INTEGER conn: DB_CONNECTION range: ITEM_COUNT_RANGE): LIST [like Current] is
			-- Fab instances for design number `dno'
		require
			connection_exists: conn /= Void
			connection_open: conn.is_open
			range_exists: range /= Void
		local
			b: DB_BIND_I
		do
			create b.make (dno)
			Result := all_rows_filtered (conn, "design_no = :1", b)
			check_row_count (Result, range)
		end

	multiple_process_rows_for_run (run: STRING conn: DB_CONNECTION range: ITEM_COUNT_RANGE): LIST [like Current] is
			-- Fab instances for customer projects for run
			-- `run' that have more than one (or that have
			-- zero) processes ordered, ordered by design
			-- number
		require
			run_not_void: run /= Void
			connection_not_void: conn /= Void
			connection_open: conn.is_open
			range_exists: range /= Void
		local
			b: DB_BIND_SS
			type: IC_FAB_INSTANCE_TYPE_VALUES
		do
			create b.make (run, type.Customer_fab)
			Result := all_rows_filtered_ordered (conn, "run_id = :1 and ic_fab_instance_type = :2 and 1 <> (select count(*) from ic_fab_instance_processes where m_fab_no = ifi.m_fab_no)", b, "design_no")
			check_row_count (Result, range)
		end

feature -- Associated rows lookup

	cust_ic_design: CUST_IC_DESIGNS_ROW is
			-- Customer IC Design for this Fabrication Instance
		do
			if cached_cust_ic_design = Void then
				create cached_cust_ic_design.make_by_key (connection, option.Old_row, design_no)
			end
			Result := cached_cust_ic_design
		ensure
			result_not_void: Result /= Void
		end

	uncache_cust_ic_design is
			-- Remove customer IC design from cache, if present
		do
			cached_cust_ic_design := Void
		ensure
			cust_ic_design_not_cached: not is_cached_cust_ic_design
		end

	is_cached_cust_ic_design: BOOLEAN is
			-- Is `cust_ic_design' currently cached?
		do
			Result := cached_cust_ic_design /= Void
		end

	explicit_options: LIST [IC_FAB_INSTANCE_OPTIONS_ROW] is
			-- Explicitly requested options for fab instance
			-- (does not include options determined by layers
			-- present)
		do
			if cached_explicit_options = Void then
				fetch_explicit_options
			end
			Result := cached_explicit_options
		ensure
			result_not_void: Result /= Void
		end

	uncache_explicit_options is
			-- Remove explicit options from cache, if present
		do
			cached_explicit_options := Void
		ensure
			explicit_options_not_cached: not is_cached_explicit_options
		end

	is_cached_explicit_options: BOOLEAN is
			-- Is `explicit_options' currently cached?
		do
			Result := cached_explicit_options /= Void
		end

	work_order: like anchored_work_order is
			-- Work order for this fab instance
			-- (error if more than one)
		do
			if cached_work_order = Void then
				cached_work_order := fetch_work_orders (expected_count.exactly_one).first
			end
			Result := cached_work_order
		ensure
			result_not_void: Result /= Void
		end

	uncache_work_order is
			-- Remove work order from cache, if present
		do
			cached_work_order := Void
		ensure
			work_order_not_cached: not is_cached_work_order
		end

	is_cached_work_order: BOOLEAN is
			-- Is `work_order' currently cached?
		do
			Result := cached_work_order /= Void
		end

	work_orders: DYNAMIC_LIST[like anchored_work_order]
			-- All work orders for this fab instance (not cached)
		do
			Result := fetch_work_orders (expected_count.at_least_one)
		ensure
			result_not_empty: not Result.is_empty
		end

	cust_acct_nos: LIST[INTEGER]
			-- All distinct cust_acct_no values in our cust_wos
		local b: DB_BIND_I
		do
			create b.make(m_fab_no)
			Result := connection.selected_integers
					("cust_acct_no", "cust_wos", "m_fab_no = :1",
					 "cust_acct_no", true, b)
		end

	replies: LIST [M_REPLIES_ROW] is
			-- Replies to requests for this fab instance,
			-- in reverse order of date processed
		local
			row: M_REPLIES_ROW
		do
			if cached_replies = Void then
				create row.make_template
				cached_replies := row.replies_for_fab_no (m_fab_no, connection, expected_count.At_least_zero)
			end
			Result := cached_replies
		ensure
			result_not_void: Result /= Void
		end

	uncache_replies is
			-- Remove replies from cache, if present
		do
			cached_replies := Void
		ensure
			replies_not_cached: not is_cached_replies
		end

	is_cached_replies: BOOLEAN is
			-- Is `replies' currently cached?
		do
			Result := cached_replies /= Void
		end

	the_die: DIES_ROW is
			-- die for this fab instance
			-- (sorry about the name, but I couldn't call this routine 'die')
		do
			if is_null_die_id then
				cached_die := Void
			elseif cached_die = Void then
				create cached_die.make_by_key(connection, option.old_row, run_id, die_id)
			end;
			Result := cached_die
		end;

	uncache_die is
		do
			cached_die := Void
		end

	is_cached_die: BOOLEAN is
		do
			Result := cached_die /= Void
		end;

feature	-- layer and option testing

	is_layer_present(a_layer: STRING): BOOLEAN is
		-- is this layer present (in my cust_ic_design)?
	do  Result := cust_ic_design.layout_of_interest.is_layer_present(a_layer)
	end;

	are_2_layers_present(a_layer, b_layer: STRING): BOOLEAN is
		-- are these layers both present?
	do  Result := is_layer_present(a_layer) and
		      is_layer_present(b_layer)
	end;

	is_option_present(an_option: STRING): BOOLEAN is
		-- is an_option present in explicit_options?
	local xopt_list: like explicit_options
	do  xopt_list := explicit_options
	    from  xopt_list.start
	    until xopt_list.after or Result
	    loop  Result := equal(an_option, xopt_list.item.option_id)
		  xopt_list.forth
	    end
	end;

feature {NONE} -- Implementation of associated rows lookup

	cached_cust_ic_design: like cust_ic_design

	cached_replies: like replies

	anchored_work_order: CUST_WOS_ROW
	do
	ensure Result = Void
	end;

	cached_work_order: like anchored_work_order

	cached_explicit_options: like explicit_options
			-- Explicitly requested options for fab instance,
			-- cached in memory

	cached_die: like the_die

	fetch_work_orders(a_range: ITEM_COUNT_RANGE): DYNAMIC_LIST[like anchored_work_order]
		local
			row: like anchored_work_order
		do
			create row.make_template
			Result := row.orders_for_fab_no (m_fab_no, connection, a_range)
		end

	fetch_explicit_options is
		local
			row: IC_FAB_INSTANCE_OPTIONS_ROW
		do
			create row.make_template
			cached_explicit_options := row.options_for_fab_no (m_fab_no, connection, expected_count.At_least_zero)
		end

end
