class INCOMING_COMMUNICATIONS_ROW

inherit
	INCOMING_COMMUNICATIONS_RAW
		undefine
			is_equal
		end

	STORED_HUNK_OF_TEXT
		redefine
			get_me_back
		end

	AUTO_DB_DELETABLE_ROW_LIST[M_REQUESTS_ROW]
		rename
			the_list	as the_requests,
			the_hidden_list as the_hidden_requests,
			fetch_the_list  as fetch_the_requests
		undefine
			is_equal
		end
	DB_ACCESS
		export
			{NONE} all
		undefine
			is_equal
		end

create

	make_template, make_null, make_by_key

feature -- 

	is_email: BOOLEAN is
	do  Result := equal(communication_medium, glo.medium_is_email)
	end;

	is_ftp: BOOLEAN is
	do  Result := equal(communication_medium, glo.medium_is_ftp)
	end;

	is_web: BOOLEAN is
	do  Result := equal(communication_medium, glo.medium_is_web)
	end;

	is_operator: BOOLEAN is
	do  Result := equal(communication_medium, glo.medium_is_operator)
	end;

	is_secure: BOOLEAN is
	do  Result := is_operator or (not is_null_security_type)
			or
		      (is_email and 
		       header_from.substring_index("@mosis.com", 1) > 0)
	end;

	security_handle: STRING is
	do  if is_null_security_type
	     then Result := "Non-secure"
	     else Result := "Secure"
	    end;
	end;

	standard_reply_list: STRING is
		-- either reply_to or from (as in incoming_message)
	require is_email
	do  Result := header_reply_to;
	    if Result.is_empty
	     then Result := header_from
	    end;
	end;

	id_for_in_reply_to: STRING is
		--  in_reply_to: <identify ME>
	do  if is_email
	     then Result := su.string_of_2("Message ", header_message_id)
	    elseif is_web
	     then Result := su.string_of_3(security_handle,
				" web request from ", header_from)
	     else Result := "Operator action"
	    end
	    Result.append(" on ");
	    Result.append(header_date);
	end;

feature

	fetch_the_requests: IDENTIFIED_QUERY[M_REQUESTS_ROW] is
	local  b: DB_BIND_I; r: M_REQUESTS_ROW;
	do  if is_old
	     then create b.make(m_communication_no);
		  create r.make_template;
		  create Result.make(connection, r);
		  Result.set_f_and_b("m_communication_no = :1", b);
		  Result.set_order_by("request_no")
	    end;
	end;

	next_request_no: INTEGER is
		-- next number up
	do  Result := the_requests.count+1
	end;

	make_next_request: M_REQUESTS_ROW is
	do  create Result.make_null(connection);
	    the_requests.extend(Result);
	    Result.set_m_communication_no(m_communication_no);
	    Result.set_request_no(the_requests.count);
	end;

	wipe_out_requests is
		-- make us go back to the database (after rollback only!)
	do  the_hidden_requests := Void
	end;

feature {INCOMING_COMMUNICATIONS_ROW} -- artifices solely for STORED_HUNK_OF_TEXT

	request_no: INTEGER is 0;
		-- my own request no

	reply_no: INTEGER is 0;
		-- my own reply no

	date_processed: DB_DATE is
	do  if the_requests.is_empty
	     then 
		  if connection /= Void then
		  	Result := connection.default_date;
		  else
		  	Result := db.default_date;
		  end
		  Result.set_date("1-JAN-1900")
	     else Result := the_requests.first.date_processed
	    end;
	end;

feature {NONE} -- artifices solely for STORED_HUNK_OF_TEXT

	first_response_line: STRING is
	do
	end;

	sent_to: STRING is
		-- empty sent_to for incoming_communications
	do
	end;

	content_type_id: STRING is
	once  Result := glo.content_is_text
	end;

	get_me_back: DATA_SOURCE is
		-- build a new text producer to replay my archive
		-- Void if not is_stored (see exceptions)
	do
	    if is_stored then
		Result := precursor
	    elseif is_web then
		create {DATA_SOURCE_STRING} Result.make(
			su.string_of_4(
				"From: ",header_from,
				"%NSender: ",header_sender))
            elseif is_ftp then
		create {DATA_SOURCE_STRING} Result.make(
			su.string_of_2(
				"From: ",header_from))
	    elseif is_operator then
		create {DATA_SOURCE_STRING} Result.make(
			su.string_of_3(
				"Operator Communication from: ",
				header_from, su.eol))
	    end
	end;

feature -- viewer support

	viewer_row: ARRAY [STRING] is
	    -- array of strings for display in the communication viewer.
	    -- viewer_row_select is either viewer_table_row or viewer_status_row.
	    -- viewer__anchor_base may be set for inclusion in anchor if needed.
	    local
		str: STRING
	    do
		if viewer_row_select = viewer_table_row then
		    create Result.make(0,4);
    
		    str := "<A HREF=%"";
		    if viewer_anchor_base /= Void then
			str.append (viewer_anchor_base)
		    end;
		    str.extend ('#')
		    str.append (my_handle)
		    str.append ("%">")
    
		    if equal (communication_medium,glo.medium_is_email) then
			str.append ("Message");
		    elseif equal (communication_medium,glo.medium_is_web) then
			if security_type /= Void and then not security_type.is_empty then
			    str.append("Secure ")
			end
			str.append ("Form");
		    elseif equal (communication_medium,glo.medium_is_operator) then
			str.append ("Operator");
		    else
			str.append (communication_medium);
		    end;
		    str.append ("</A>");
		    Result.put(str,0);
		    Result.put("",1);
		    Result.put("",2);
		    str := "<B>From: </B>";
		    str.append (std_html_fixer.modify(header_from));
		    if not su.is_empty(header_sender) then
			str.append ("<BR><B>Sender: </B>");
			str.append (std_html_fixer.modify(header_sender));
		    end;
		    if not su.is_empty(header_reply_to) then
			str.append ("<BR><B>Reply to: </B>");
			str.append (std_html_fixer.modify(header_reply_to));
		    end;
		    if  not header_date.is_empty then
			str.append("<BR><B>Dated: </B>");
			str.append(std_html_fixer.modify(header_date));
		    end
		    Result.put(str,3);
		    Result.put(date_processed.out,4);
		elseif viewer_row_select = viewer_status_row then
		else
		end
	    end


end -- INCOMING_COMMUNICATIONS_ROW
