
indexing
	
	description: "Window to allow submission and monitoring of internal %
		%front end requests for a specific run"
	author: "David Hollenberg"
	date: "May 31, 2005"

deferred class INTERNAL_FE_REQUEST_WINDOW

inherit
	INTERNAL_FE_REQUEST_IFC
	WINDOW_INTERFACE
		redefine
			enable_active,
			disable_active
		end
	MOSIS_GRAPHICAL_UTILITIES
		export
			{NONE} all
		undefine
			default_create
		end
	UNIX_OS_ACCESS
		export
			{NONE} all
		undefine
			default_create
		end
		
feature {NONE} -- Creation

	make (w: EV_TITLED_WINDOW)
			-- Create interface in window `w'
		require
			window_exists: w /= Void
			window_valid: not w.is_destroyed
		do
			make_interface
			make_proxy (w)
		end

feature -- Operations

	reset
			-- Move focus back into window
		do
			enable_active
		end

	refresh
			-- Refresh status
		do
		end

feature -- Modification

	enable_active
			-- Make interface active
		do
			precursor
		end

	disable_active
			-- Make interface inactive
			-- (can look but cannot initiate most actions)
		do
			precursor
		end

feature {NONE} -- Implementation

	build_interface
			-- Build interface
		local
			buttons: HORIZONTAL_BUTTON_SET
			main_vbox: EV_VERTICAL_BOX
			box: EV_BOX
		do
			fetch_data
			add_menu_bar
			create projects
			projects.set_row_to_generic_function (agent row_to_record)
			projects.enable_multiple_selection
			projects.build_selection_status_box
			set_visible_columns
			build_all_rows
			projects.sort_on_column (2)
			create main_vbox
			main_vbox.set_border_width (user_interface.border_width)
			main_vbox.extend (projects)
			box := projects.selection_status_box
			main_vbox.extend (box)
			main_vbox.disable_item_expand (box)

			create buttons.make_with_text_and_actions (<< "All", "None", "Invert" >>, << agent process_select_all, agent process_select_none, agent process_select_invert >>)
			box := buttons.column_set.box
			
			main_vbox.extend (box)
			main_vbox.disable_item_expand (box)
			
			create buttons.make_with_text_and_actions (<< gui.Ev_ok, "Finish", gui.Ev_cancel >>, << agent process_ok, agent process_finish, agent process_cancel >>)
			box := buttons.column_set.box
			
			main_vbox.extend (box)
			main_vbox.disable_item_expand (box)
			
			gui.set_box_background_recursive (main_vbox, background_color)
			extend (main_vbox)
			close_request_actions.extend (agent process_cancel)
			gui.add_cancel_accelerator (window, agent process_cancel)
			gui.do_once_on_idle (agent post_display_actions)
			refresh
		end
		
	fetch_data
			-- Fetch needed data
		do
			fetch_cust_wos
			fetch_internal_requests
		end

	fetch_internal_requests
			-- Fetch internal requests for this run which
			-- are of type `internal_request_type' and
			-- put in `internal_requests'
		local
			row: INTERNAL_FE_REQUESTS_ROW
		do
			create row.make_template
			internal_requests := row.rows_for_production_and_type (session.run_id, internal_request_type, connection, expected_count.At_least_zero)
		ensure
			internal_requests_not_void: internal_requests /= Void
		end

	row_to_record (row: EV_MULTI_COLUMN_LIST_ROW): ORDER_MULTI_COLUMN_LIST_ROW
			-- Function which translates `row' to object of
			-- type ORDER_MULTI_COLUMN_LIST_ROW
		do
			Result ?= row
			check
				result_not_void: Result /= Void
			end
		end

	build_all_rows
			-- Build all rows and put in `projects'
		local
			list: ARRAYED_LIST [ORDER_MULTI_COLUMN_LIST_ROW]
		do
			create list.make (cust_wos.count)
			cust_wos.do_all (agent add_row (?, list))
			projects.set_all_rows (list)
		end
			
	add_row (wo: CUST_WOS_ROW list: LIST [ORDER_MULTI_COLUMN_LIST_ROW])
			-- Add a row for `wo' to `list'
		local
			row: ORDER_MULTI_COLUMN_LIST_ROW
		do
			create row.make (wo, reply_mode)
			list.extend (row)
			row.select_actions.extend (agent projects.process_select_one (row))
			row.deselect_actions.extend (agent projects.process_deselect_one (row))
		end
			
	add_menu_bar
			-- Add menu bar to window
		local
			file_menu, select_menu: EV_MENU
			menu_item, exit_menu_item: EV_MENU_ITEM
			close_menu_item: EV_MENU_ITEM
			a_menu_bar: EV_MENU_BAR
		do	
			create file_menu.make_with_text ("File")
			
			create close_menu_item.make_with_text ("Close")
			close_menu_item.select_actions.extend (agent process_cancel)
			file_menu.extend (close_menu_item)
			
			create exit_menu_item.make_with_text ("Exit")
			exit_menu_item.select_actions.extend (gui.destroy_application_agent)
			file_menu.extend (exit_menu_item)
			
			create select_menu.make_with_text ("Select")
			
			create menu_item.make_with_text ("All")
			menu_item.select_actions.extend (agent process_select_all)
			select_menu.extend (menu_item)
			
			create menu_item.make_with_text ("None")
			menu_item.select_actions.extend (agent process_select_none)
			select_menu.extend (menu_item)
			
			create menu_item.make_with_text ("Invert")
			menu_item.select_actions.extend (agent process_select_invert)
			select_menu.extend (menu_item)
			create menu_item.make_with_text ("Never notified")
			menu_item.select_actions.extend (agent process_select_never_notified)
			select_menu.extend (menu_item)
			create menu_item.make_with_text ("Previously notified")
			menu_item.select_actions.extend (agent process_select_previously_notified)
			select_menu.extend (menu_item)
			
			create a_menu_bar
			a_menu_bar.extend (file_menu)
			a_menu_bar.extend (select_menu)
			set_menu_bar (a_menu_bar)
		end

	post_display_actions
		do
			reset
			gui.do_once_on_idle (agent gui.center_window_with_margin (window, 200, 200))
		end

	process_select_all
			-- Process "select all" request
		do
			if is_active then
				disable_active
				projects.process_select_all
				enable_active
			end
		end

	process_select_none
			-- Process "select none" request
		do
			if is_active then
				disable_active
				projects.process_select_none
				enable_active
			end
		end

	process_select_invert
			-- Process "invert selection" request
		do
			if is_active then
				disable_active
				projects.process_select_invert
				enable_active
			end
		end

	process_select_never_notified
			-- Process "select never notified projects" request
		do
			if is_active then
				disable_active
				projects.process_select_rows_filtered (agent {ORDER_MULTI_COLUMN_LIST_ROW}.has_no_dates_sent)
				enable_active
			end
		end

	process_select_previously_notified
			-- Process "select previously notified projects"
			-- request
		do
			if is_active then
				disable_active
				projects.process_select_rows_filtered (agent {ORDER_MULTI_COLUMN_LIST_ROW}.has_dates_sent)
				enable_active
			end
		end

	process_refresh
			-- Process "refresh" request
		do
			if is_active then
				disable_active
				refresh
				enable_active
			end
		end

	process_ok
			-- Take actions needed when user clicks OK button
		do
			if is_active then
				disable_active
				create operation.make (agent process_ok_request)
				operation.enable_abort_caught
				operation.abort_actions.extend (agent handle_ok_abort)
				operation.success_actions.extend (agent refresh_selected_rows)
				operation.execute
				enable_active
			end
		end

	process_ok_request
			-- Process OK request to submit internal front
			-- end requests for processing
		local
			n: INTEGER
		do
			projects.rebuild_selected_rows
			fetch_internal_requests
			if not internal_requests.is_empty then
				n := internal_requests.count
				confirm ("Found " + n.out + " pending front end " + su.number (n, "request", "requests") + " for " + request_description + ".%NCannot submit new requests until the existing ones have been processed.%N%NStart front end to process these requests")
				start_front_end
				compute_selected_rows
				monitor_request_completion (computed_selected_rows)
			elseif selected_rows.is_empty then
				report_no_rows_selected
			else
				check_rows_valid
				confirm_ok_request
				insert_internal_requests
				start_front_end
				computed_selected_rows := selected_rows
				monitor_request_completion (computed_selected_rows)
			end
		end

	confirm_ok_request
			-- Get confirmation from user before processing
			-- OK request
		local
			n: INTEGER
			prev_sent: LIST [ORDER_MULTI_COLUMN_LIST_ROW]
		do
			n := selected_rows.count
			confirm_before_submission (n)
			prev_sent := selected_rows.twin
			prev_sent.wipe_out
			selected_rows.do_if (agent prev_sent.extend, agent {ORDER_MULTI_COLUMN_LIST_ROW}.has_dates_sent)
			if not prev_sent.is_empty then
				n := prev_sent.count
				warn_prev_processed_requests (n)
			end
		end

	insert_internal_requests
			-- Insert into the database an internal front
			-- end request for each selected row
		local
			rows: LIST [ORDER_MULTI_COLUMN_LIST_ROW]
			sorter: LIST_BUBBLE_SORTER [ORDER_MULTI_COLUMN_LIST_ROW]
			comp: DIE_ID_COMPARATOR [ORDER_MULTI_COLUMN_LIST_ROW]
		do
			create sorter
			create comp
			rows := sorter.sorted (projects.selected_rows, comp)
			rows.do_all (agent insert_internal_request)
			session.commit
		end

	insert_internal_request (row: ORDER_MULTI_COLUMN_LIST_ROW)
			-- Insert into the database an internal front
			-- end request for `row'
		local
			req: INTERNAL_FE_REQUESTS_MROW
			sequence: SEQUENCE_NAMES
		do
			create req.make_null (connection)
			req.set_internal_request_no (connection.sequence_next_value (sequence.Internal_request_no_seq))
			req.set_internal_request_type (internal_request_type)
			req.set_m_order_no (row.cust_wo.m_order_no)
			req.set_date_requested (connection.transaction_date)
			req.set_is_production_specific (True)
			session.commit_actions.extend (agent publish_internal_request (req, row.design_no, row.die_id))
		end

	publish_internal_request (req: INTERNAL_FE_REQUESTS_ROW dno: INTEGER die: STRING)
			-- Publish insertion of internal front end request
		require
			req_not_void: req /= Void
			die_not_void: die /= Void
		local
			e: INTERNAL_REQUEST_SUBMITTED_EVENT
		do
			create e
			e.set_request (req)
			e.set_design_no (dno)
			e.set_die_id (die)
			e.publish
		end

	start_front_end
			-- Spawn front end process to handle internal requests
		local
			checker: FILE_EXECUTABLE_CHECK_OP
			prog_file: STRING
			process: UNIX_PROCESS
			func_name: RUN_AS_USER_FUNCTION_NAMES
		do
			prog_file := session.environment.run_as_user_program
			create checker
			checker.check_executable (prog_file, "Run as user program")
			create process.make (prog_file)
			process.set_arguments (<< func_name.Process_internal_request, session.environment.front_end_user, "production_id=" + session.run_id >>)
			process.set_input_file_name (unix_os.null_file_name)
			process.set_output_file_name (unix_os.null_file_name)
			process.set_error_same_as_output
			process.spawn_nowait
		end

	compute_selected_rows
			-- Compute list of selected rows from
			-- `internal_requests' and put in `computed_selected_rows'.
		local
			table: HASH_TABLE [ORDER_MULTI_COLUMN_LIST_ROW, INTEGER]
			all_rows: LIST [ORDER_MULTI_COLUMN_LIST_ROW]
		do
			all_rows:= projects.all_rows
			create table.make (all_rows.count)
			all_rows.do_all (agent add_order_to_table (?, table))
			computed_selected_rows := selected_rows.twin
			computed_selected_rows.wipe_out
			internal_requests.do_all (agent add_computed_selected_row (?, table))
		end

	add_computed_selected_row (r: INTERNAL_FE_REQUESTS_ROW table: HASH_TABLE [ORDER_MULTI_COLUMN_LIST_ROW, INTEGER])
			-- Add row with m_order_no in `r' to
			-- `computed_selected_rows'.
		do
			computed_selected_rows.extend (table.item (r.m_order_no))
		end

	monitor_request_completion (sel_rows: LIST [ORDER_MULTI_COLUMN_LIST_ROW])
			-- Check database periodically so we can show
			-- progress of internal front end request processing,
			-- where `sel_rows' is the list of rows corresponding
			-- to internal requests
		local
			done, aborted: BOOLEAN
			die_id: STRING
			table: HASH_TABLE [ORDER_MULTI_COLUMN_LIST_ROW, INTEGER]
			rows_left: LIST [ORDER_MULTI_COLUMN_LIST_ROW]
			prev_count: INTEGER
			date_changed: INTEGER
		do
			create table.make (sel_rows.count)
			sel_rows.do_all (agent add_order_to_table (?, table))
			rows_left := sel_rows.twin
			rows_left.wipe_out
			rows_left.append (sel_rows)
			show_progress_window
			from
				prev_count := sel_rows.count
				date_changed := unix_os.current_time_in_seconds
			until
				done or aborted
			loop
				fetch_internal_requests
				if internal_requests.count /= prev_count then
					date_changed := unix_os.current_time_in_seconds
					prev_count := internal_requests.count
				end
				rows_left.do_all (agent {ORDER_MULTI_COLUMN_LIST_ROW}.set_is_pending (False))
				internal_requests.do_all (agent mark_pending (?, table))
				rows_left.do_if (agent refresh_row, agent {ORDER_MULTI_COLUMN_LIST_ROW}.is_not_pending)
				if not internal_requests.is_empty then
					die_id := table.item (internal_requests.first.m_order_no).die_id
				else
					done := True
					die_id := ""
				end
				update_progress_window (die_id)
				if unix_os.current_time_in_seconds - date_changed > Max_time_per_project then
					if not question ("Front end has not completed any requests in the last " + Max_time_per_project.out  + " seconds.%NYou may continue monitoring front end progress, but if the problem persists%Nplease contact software development.%N%NContinue monitoring?") then
						operation.abort
					end
					date_changed := unix_os.current_time_in_seconds
				end
				gui.application.sleep (500)
			end
			projects.refresh_all_rows
				-- Be sure we have up to date status
			gui.application.sleep (300)
			progress_window.destroy
		end

	add_order_to_table (ord: ORDER_MULTI_COLUMN_LIST_ROW table: HASH_TABLE [ORDER_MULTI_COLUMN_LIST_ROW, INTEGER])
			-- Add `ord' to `table' under its m_order_no
		do
			table.put (ord, ord.cust_wo.m_order_no)
		end
				
	mark_pending (req: INTERNAL_FE_REQUESTS_ROW table: HASH_TABLE [ORDER_MULTI_COLUMN_LIST_ROW, INTEGER])
			-- Mark row for order corresponding to `req' as
			-- still pending
		do
			table.item (req.m_order_no).set_is_pending (True)
		end
				
	refresh_row (row: ORDER_MULTI_COLUMN_LIST_ROW)
			-- Refresh data for `row'
		do
			row.uncache_dates_sent
			projects.refresh_row (row)
		end
				
	refresh_selected_rows
		do
			projects.selected_rows.do_all (agent {ORDER_MULTI_COLUMN_LIST_ROW}.uncache_dates_sent)
			projects.refresh_after_selected_rows_operation
		end

	refresh_all_rows
		do
			projects.all_rows.do_all (agent {ORDER_MULTI_COLUMN_LIST_ROW}.uncache_dates_sent)
			projects.refresh_all_rows
		end

	handle_ok_abort
			-- Handle abort of OK operation
		do
			if progress_window /= Void then
				progress_window.destroy
			end
			projects.refresh_after_selected_rows_operation
		end

	process_finish
			-- Take actions needed when user clicks Finish button
		do
			if is_active then
				disable_active
				create operation.make (agent process_finish_request)
				operation.enable_abort_caught
				operation.abort_actions.extend (agent handle_ok_abort)
				operation.execute
				enable_active
			end
		end

	process_finish_request
			-- Process Finish request to try to finish step
			-- (only allowed to finish if request of type
			-- `internal_request_type' has been processed
			-- at least once for every project that
			-- needs one)
		local
			list: ARRAYED_LIST [ORDER_MULTI_COLUMN_LIST_ROW]
			all_rows: LIST [ORDER_MULTI_COLUMN_LIST_ROW]
			n: INTEGER
		do
			all_rows := projects.all_rows
			create list.make (all_rows.count)
			all_rows.do_if (agent list.extend, agent {ORDER_MULTI_COLUMN_LIST_ROW}.has_no_dates_sent)
			if not list.is_empty then
				n := list.count
				warn (request_description + " not completed for " + n.out + " " + su.number (n, "project", "projects") + ".%NSkip " + su.number (n, "notification", "notifications") + " and force step to complete anyway?")
			end
			success_actions.call ([])
		end

	handle_finish_abort
			-- Handle abort of Finish operation
		do
		end

	process_cancel
			-- Process cancel request
		do
			if is_active then
				disable_active
				abort_actions.call ([])
			end
		end

	report_no_rows_selected
			-- Tell user no rows were selected
		do
			operation.inform ("You must select one or more rows first")
			operation.abort
		end

	show_progress_window
			-- Create and display progress window
		local
			vbox: EV_VERTICAL_BOX
			box: EV_BOX
			cols: WIDGET_COLUMN_SET
		do
			create progress_window
			progress_window.set_title (progress_window_title + " for " + session.run_id)
			progress_window.set_size (500, 250)
			create vbox
			vbox.set_padding (user_interface.padding)
			create cols.make (2)
			cols.set_columns_padding (5)
			cols.set_row_padding (user_interface.padding)
			create die_id_box
			create remaining_box
			cols.append_row (<< gui.label_with_text ("Die ID:"), die_id_box >>)
			cols.append_row (<< gui.label_with_text ("Remaining:"), remaining_box >>)
			cols.disable_column_expand (1)
			cols.align_column_left (1)
			cols.align_column_left (2)
			box := cols.box
			vbox.extend (box)
			vbox.disable_item_expand (box)
			create progress_bar
			progress_bar.value_range.resize_exactly (0, total_item_count)
			progress_bar.set_value (0)
			progress_bar.disable_segmentation
			vbox.extend (progress_bar)
			vbox.disable_item_expand (progress_bar)
			
			vbox.set_border_width (user_interface.border_width)
			progress_window.extend (vbox)
			progress_window.show_relative_to_window (session.application.current_window)
			gui.process_events
		end

	update_progress_window (die: STRING)
			-- Update progress window to show current info
		do
			die_id_box.wipe_out
			die_id_box.extend (gui.label_with_text (die))
			remaining_box.wipe_out
			remaining_box.extend (gui.label_with_text (remaining_item_count.out))
			progress_bar.set_value (finished_item_count)
			gui.process_events
		end

	cust_wos: LIST [CUST_WOS_ROW]
			-- Work orders for projects on run that
			-- need to have an `internal_request_type' request
			-- processed at least once

	projects: MULTI_COLUMN_LIST [ORDER_MULTI_COLUMN_LIST_ROW]
			-- List of projects that need an
			-- `internal_request_type' request processed
			-- at least once (whether it has been done
			-- previously or not)

	internal_requests: LIST [INTERNAL_FE_REQUESTS_ROW]
			-- Internal requests for this production
			-- for requests of type `internal_request_type'

	reply_mode: STRING
			-- Reply mode for reply inserted into M_REPLIES table
		local
			req_type: INTERNAL_REQUEST_TYPES_ROW
		do
			create req_type.make_by_key (connection, option.Old_row, internal_request_type)
			Result := req_type.reply_mode
		end

	operation: ANONYMOUS_OPERATION
			-- Current operation in progress, if any

	progress_window: EV_DIALOG
			-- Window to show progress of request processing

	progress_bar: EV_HORIZONTAL_PROGRESS_BAR
			-- Bar to show progress of request processing
			-- graphically

	die_id_box: EV_HORIZONTAL_BOX
			-- Box for displaying die id

	remaining_box: EV_HORIZONTAL_BOX
			-- Box for displaying number of requests remaining

	total_item_count: INTEGER
			-- Total number of requests to be processed
		do
			Result := computed_selected_rows.count
		end

	finished_item_count: INTEGER
			-- Number of requests finished
		do
			Result := total_item_count - remaining_item_count
		end

	remaining_item_count: INTEGER
			-- Remaining number of requests to be processed
		do
			Result := internal_requests.count
		end

	selected_rows: LIST [ORDER_MULTI_COLUMN_LIST_ROW]
			-- List of currently selected rows
		do
			Result := projects.selected_rows
		end

	computed_selected_rows: LIST [ORDER_MULTI_COLUMN_LIST_ROW]
			-- List of rows that would have had to have
			-- been selected to produce list of requests
			-- in `internal_requests'

	Max_time_per_project: INTEGER = 30
			-- Maximum number of seconds we expect it to take
			-- to process a request for one project

feature {NONE} -- Deferred implementation

	fetch_cust_wos
			-- Fetch customer work orders for projects to
			-- which requests of type `internal_request_type'
			-- may apply and put in `cust_wos'
		deferred
		ensure
			cust_wos_not_void: cust_wos /= Void
		end

	internal_request_type: STRING
			-- Type of internal requests that can be submitted
			-- and monitored
		deferred
		ensure
			result_not_void: Result /= Void
		end

	request_description: STRING
			-- Description of internal requests that can be
			-- submitted and monitored, suitable for inclusion
			-- in messages to operator
		deferred
		ensure
			result_not_void: Result /= Void
		end

	progress_window_title: STRING
			-- Title for request progress window (without
			-- run id)
		deferred
		ensure
			result_not_void: Result /= Void
		end

	set_visible_columns
			-- Set visible columns in `projects'
		deferred
		end
			
	check_rows_valid
			-- Check that all selected rows are valid for
			-- this internal request
		deferred
		end

	confirm_before_submission (n: INTEGER)
			-- Tell operator we're about to submit `n' internal
			-- requests and get confirmation before proceeding
		deferred
		end

	warn_prev_processed_requests (n: INTEGER)
			-- Warn operator about `n' orders for which an
			-- internal request of this type has already been
			-- processed
		deferred
		end

end
