
indexing
	
	description: "Chip in a job file written on just one level"
	author: "David Hollenberg"
	date: "July 21, 2004"

class JOB_FILE_CHIP_LEVEL

create
	default_create

feature -- Properties

	level_number: INTEGER
			-- Number of level on which to write chip

	reverse_tone: BOOLEAN
			-- Should chip be written in reverse tone?

	file_name: STRING
			-- Name of file to write

	formatted_file_name: STRING
			-- Name of file to write, formatted for inclusion
			-- in job file
		do
			create Result.make (13)
			Result.append (file_name.substring (1, 7))
			Result.extend ('-')
			Result.append (file_name.substring (8, 9))
			Result.extend ('-')
			Result.append (file_name.substring (11, 12))
		end

	spot_size: DOUBLE
			-- Spot size at which chip should be written

feature -- Contract support

	is_valid_file_name (s: STRING): BOOLEAN
		require
			string_exists: s /= Void
		do
			Result := s.count = 12 and s.item (10) = '.'
		end

feature -- Modification

	set_level_number (n: INTEGER)
			-- Set `level_number' to `n'
		require
			valid_level: n >= 0
		do
			level_number := n
		ensure
			value_set: level_number = n
		end

	set_reverse_tone (b: BOOLEAN)
			-- Set `reverse_tone' to `b'
		do
			reverse_tone := b
		ensure
			value_set: reverse_tone = b
		end

	set_file_name (s: STRING)
			-- Set `file_name' to `s'
		require
			name_exists: s /= Void
			name_valid: is_valid_file_name (s)
		do
			file_name := s
		ensure
			value_set: file_name = s
		end

	set_spot_size (d: DOUBLE)
			-- Set `spot_size' to `d'
		require
			value_positive: d > 0
		do
			spot_size := d
		ensure
			value_set: spot_size = d
		end

end
