
indexing
	
	description: "Multi-column list row that has additional information %
		%about the entity it represents that uniquely identifies %
		%the entity and associates it with a batch job"
	author: "David Hollenberg"
	date: "November 18, 2003"

deferred class JOB_MULTI_COLUMN_LIST_ROW
inherit	
	EV_MULTI_COLUMN_LIST_ROW
		undefine
			new_filled_list
		end
	HAS_BATCH_JOB_NUMBER
		rename
			batch_job_number as batch_job_no
		undefine
			copy, is_equal, default_create
		end
	HAS_BATCH_JOB_HOST
		undefine
			copy, is_equal, default_create
		end
	HAS_BATCH_JOB_STATE
		undefine
			copy, is_equal, default_create
		end
	HAS_ELAPSED_JOB_TIME
		undefine
			copy, is_equal, default_create
		end
	HAS_ERROR_DESCRIPTION
		undefine
			copy, is_equal, default_create
		end
	HAS_JOB_STATUS
		undefine
			copy, is_equal, default_create
		end
	DB_OPTIONS_ACCESS
		export
			{NONE} all
		undefine
			copy, is_equal, default_create
		end
	FILE_UTILITIES_ACCESS
		export
			{NONE} all
		undefine
			default_create, copy, is_equal
		end
	CONDITION_UTILITIES
		export
			{NONE} all
		undefine
			default_create, copy, is_equal
		end


feature -- Properties

	run_id: STRING
			-- Run to which job applies

	pattern_type: STRING
			-- Type of pattern files generated

	is_visible: BOOLEAN
			-- Is row currently visible in multi-column list?

	is_locked: BOOLEAN
			-- Is database row for job requestor currently locked?
		do
			Result := requestor.is_locked
		end

	unique_identifier: STRING
			-- Value that uniquely identifies row among its peers
		deferred
		ensure
			result_exists: Result /= Void
		end

	requestor: BATCH_JOB_REQUESTOR
			-- Entity that "requested" batch job - i.e., entity
			-- for which batch job needs to be run (die,
			-- wafer level, etc).  Usually, this is a
			-- database row that has a foreign key to batch
			-- jobs table
		deferred
		ensure
			result_exists: Result /= Void
		end

	job_status: STRING
			-- Status of job
		do
			Result := requestor.job_status
		end

	is_null_batch_job_no: BOOLEAN
			-- Is batch job number null in row?
		do
			Result := requestor.is_null_batch_job_no
		end

	requestor_batch_job_no: INTEGER
			-- Batch job number associated with `requestor'
		require
			has_batch_job: not is_null_batch_job_no
		do
			Result := requestor.batch_job_no
		end

	batch_job: BATCH_JOBS_MROW
			-- Row from batch jobs table, if not Void

	has_batch_job: BOOLEAN
			-- Is there an associated batch job?
		do
			Result := batch_job /= Void
		end

	batch_job_no: INTEGER
			-- Batch job number
		do
			Result := batch_job.batch_job_no
		end

	batch_job_state: STRING
			-- State of batch job
		do
			if batch_job /= Void then
				Result := batch_job.batch_job_state
			else
				Result := ""
			end
		end

	batch_job_host: STRING
			-- Host on which batch job is executing
		do
			if batch_job /= Void then
				Result := batch_job.execution_host
			else
				Result := ""
			end
		end

	is_job_stop_requested: BOOLEAN
			-- Is batch job in "stop requested" state?
		require
			has_batch_job: has_batch_job
		local
			state: BATCH_JOB_STATE_VALUES
		do
			Result := batch_job_state.is_equal (state.Stop_requested)
		end

	has_elapsed_job_time: BOOLEAN
			-- Is the job elapsed time known?
		do
			Result := not requestor.is_null_elapsed_job_time or (batch_job /= Void and then not batch_job.is_null_elapsed_job_time)
		end

	elapsed_job_time: INTEGER
			-- Elapsed job time
		do
			if not requestor.is_null_elapsed_job_time then
				Result := requestor.elapsed_job_time
			else
				Result := batch_job.elapsed_job_time
			end
		end

	has_job_status (s: STRING): BOOLEAN
			-- Is `job_status' equal to `s'?
		require
			string_exists: s /= Void
		do
			Result := job_status.is_equal (s)
		end

	is_monitorable: BOOLEAN
			-- Does `job_status' indicate that there is a
			-- job which can be monitored?
		do
			Result := is_stopable
		end

	is_stopable: BOOLEAN
			-- Does `job_status' indicate that there is a
			-- job which can be stopped?
		do
			Result := job_status.is_equal (status_values.Pending) or job_status.is_equal (status_values.Executing)
		end

	is_unstopable: BOOLEAN
			-- Does `job_status' indicate that there is no
			-- job which can be stopped?
		do
			Result := not is_stopable
		end

	is_resettable: BOOLEAN
			-- Does `job_status' indicate that there is a
			-- job which can be reset (a reset undoes
			-- whatever the job did)
		do
			Result := job_status.is_equal (status_values.Passed) or job_status.is_equal (status_values.Failed) or job_status.is_equal (status_values.Aborted) or job_status.is_equal (status_values.Stopped)
		end

	is_unresettable: BOOLEAN
			-- Does `job_status' indicate that there is no
			-- job which can be reset?
		do
			Result := not is_resettable
		end

	is_submittable: BOOLEAN
			-- Does `job_status' indicate that there is a
			-- job which can be submitted?
		do
			Result := job_status.is_equal (status_values.Created)
		end

	is_unsubmittable: BOOLEAN
			-- Does `job_status' indicate that there is no
			-- job which can be submitted?
		do
			Result := not is_submittable
		end

	is_releasable: BOOLEAN
			-- Does `Current' have a holding batch job which
			-- can be released?
		do
			Result := batch_job_state.is_equal (state_values.Holding)
		end

	is_unreleasable: BOOLEAN
			-- Does `Current' have a holding batch job which
			-- can be released?
		do
			Result := not is_releasable
		end

	is_passed: BOOLEAN
			-- Does `job_status' indicate that job
			-- has completed successfully?
		do
			Result := job_status.is_equal (status_values.Passed)
		end

	is_failed: BOOLEAN
			-- Does `job_status' indicate that job
			-- completed unsuccessfully?
		do
			Result := job_status.is_equal (status_values.Failed)
		end

	is_aborted: BOOLEAN
			-- Does `job_status' indicate that job
			-- aborted before completion?
		do
			Result := job_status.is_equal (status_values.Aborted)
		end

	is_unsuccessful: BOOLEAN
			-- Does `job_status' indicate that job
			-- did not succeed?
		do
			Result := is_failed or is_aborted
		end

	error_description: STRING
			-- Description of error encountered during job
		do
			Result := requestor.error_explanation.twin
			Result.replace_substring_all ("%N", ".  ")
		end

	status_changed: BOOLEAN
			-- Did status of row change on last call to
			-- `update_status'?

	job_script_file: STRING
			-- Name of file that contains script to run job

	job_log_file: STRING
			-- Name of file that contains output of job

	has_job_done_file: BOOLEAN
			-- Does job create a "done" file to indicate
			-- that it completed?
		do
			Result := True
		end

	job_done_file: STRING
			-- Name of file created by script to indicate script
			-- completed

	job_output_files: LIST [STRING]
			-- List of output file names created by script,
			-- not including log file

	job_name (a_run_id: STRING): STRING
			-- Name of job for `a_run_id'
		require
			a_run_id_exists: a_run_id /= Void
		deferred
		ensure
			result_exists: Result /= Void
		end

	job_type: STRING
			-- Type of job to submit
		deferred
		ensure
			result_exists: Result /= Void
		end

	batch_queue_name: STRING
			-- Name of batch queue to use
		deferred
		ensure
			result_exists: Result /= Void
		end

feature -- Modification

	enable_visible
			-- Make row visible
		do
			is_visible := True
		ensure
			is_visible: is_visible
		end

	disable_visible
			-- Make row invisible
		do
			is_visible := False
		ensure
			not_visible: not is_visible
		end

	set_run_id (r: STRING)
			-- Set `run_id' to `r'
		require
			new_run_id_exists: r /= Void
		do
			run_id := r
		ensure
			run_id_set: run_id = r
		end

	set_pattern_type (s: STRING)
			-- Set `pattern_type' to `s'
		require
			value_exists: s /= Void
		do
			pattern_type := s
		ensure
			pattern_type_set: pattern_type = s
		end

	set_batch_job (r: BATCH_JOBS_MROW)
			-- Set `batch_job' to `r'
		do
			batch_job := r
		ensure
			batch_job_set: batch_job = r
		end

	set_job_script_file (s: STRING)
			-- Set `job_script_file' to `s'
		do
			job_script_file := s
		end

	set_job_log_file (s: STRING)
			-- Set `job_log_file' to `s'
		do
			job_log_file := s
		end

	set_job_done_file (s: STRING)
			-- Set `job_done_file' to `s'
		do
			job_done_file := s
		end

	set_job_output_files (list: LIST [STRING])
			-- Set `job_output_files' to `list'
		do
			job_output_files := list
		end

	set_job_status_created
			-- Set job status to CREATED for `requestor'
		local
			was_passed: BOOLEAN
			rows: LIST [PATTERN_FILES_MROW]
			lock_op: ROW_LOCK_OP
		do
			was_passed := requestor.is_status_passed
			requestor.set_status_created
			if was_passed then
				rows := existing_pattern_file_rows
				create lock_op
				lock_op.lock_list_nowait (rows)
				rows.do_all (agent {PATTERN_FILES_MROW}.delete)
			end
		end

	submit_job (a_run_id, a_state: STRING script_path, log_path, args: STRING)
			-- Submit fracture job as part of run
			-- `a_run_id' and put it in
			-- `a_state'.  Job will execute `script_path'
			-- with `args' and put output in `log_path'
		require
			a_run_id_exists: a_run_id /= Void
			a_run_id_nonempty: not a_run_id.is_empty
			a_state_exists: a_state /= Void
			a_state_nonempty: not a_state.is_empty
			script_path_exists: script_path /= Void
			script_path_nonempty: not script_path.is_empty
			log_path_exists: log_path /= Void
			log_path_nonempty: not log_path.is_empty
		local
			r: BATCH_JOBS_MROW
			job_no: INTEGER
			status: JOB_STATUS_VALUES
			sequence: SEQUENCE_NAMES
			conn: DB_CONNECTION
		do
			conn := requestor.connection
			job_no := conn.sequence_next_value (sequence.Batch_job_no_seq)
			create r.make_by_key (conn, option.New_row, job_no)
			r.set_job_name (job_name (a_run_id))
			r.set_job_set_name (a_run_id + "_" + job_type)
			r.set_batch_job_type (job_type)
			r.set_batch_job_state (a_state)
			r.set_date_state_changed (conn.transaction_date)
			r.set_date_submitted (conn.transaction_date)
			r.set_batch_job_queue (batch_queue_name)
			r.set_priority (40)
			r.set_restartable (False)
			r.set_notify_on_failure (False)
			
			r.set_program_filename (script_path)
			r.set_program_arguments (args)
			r.set_output_filename (log_path)
			r.set_error_filename (log_path)
			requestor.set_job_status (status.Pending)
			requestor.set_batch_job_no (job_no)
		end

	release_job
			-- Release job for this row
		require
			has_batch_job: has_batch_job
			is_releasable: is_releasable
		local
			state: BATCH_JOB_STATE_VALUES
		do
			lock_job
			batch_job.set_batch_job_state (state.Waiting)
		end

	try_to_stop_job
			-- Attempt to stop job for this row, if executing,
			-- and then update row status accordingly
		require
			has_batch_job: has_batch_job
			is_stopable: is_stopable
			is_locked: is_locked
		local
			the_state: STRING
			state: BATCH_JOB_STATE_VALUES
		do
			status_changed := False
			batch_job.try_to_lock_nowait
			if batch_job.is_locked_and_unchanged then
				-- If row is busy or it changed since last
				-- fetch, don't do anything
				-- Row cannot be gone due to foreign key
				-- constraint from requestor's table
			
				the_state := batch_job_state
				if the_state.is_equal (state.Waiting) or the_state.is_equal (state.Holding) then
					set_status_created
				elseif the_state.is_equal (state.Assigned) or the_state.is_equal (state.Executing) then
					set_status_stop_requested
				elseif the_state.is_equal (state.Aborted) then
					set_status_aborted
				elseif the_state.is_equal (state.Failed) or the_state.is_equal (state.Internal_error) then
					set_status_failed
				elseif the_state.is_equal (state.Passed) then
					set_status_passed
				elseif the_state.is_equal (state.Stopped) then
					set_status_stopped
				elseif the_state.is_equal (state.Stop_requested) then
					-- Don't do anything
				else
					report_unknown_state
				end
			end
		end


	update_status
			-- Update status of job for this row
		require
			has_batch_job: has_batch_job
		local
			status: JOB_STATUS_VALUES
			the_state: STRING
			state: BATCH_JOB_STATE_VALUES
		do
			status_changed := False
			the_state := batch_job_state
			if the_state.is_equal (state.Waiting) or the_state.is_equal (state.Assigned) or the_state.is_equal (state.Holding) then
				if not job_status.is_equal (status.Pending) then
					set_status_pending
				end
			elseif the_state.is_equal (state.Executing) or the_state.is_equal (state.Stop_requested) then
				if not job_status.is_equal (status.Executing) then
					set_status_executing
				end
			elseif the_state.is_equal (state.Aborted) then
				set_status_aborted
			elseif the_state.is_equal (state.Failed) or the_state.is_equal (state.Internal_error) then
				set_status_failed
			elseif the_state.is_equal (state.Passed) then
				set_status_passed
			elseif the_state.is_equal (state.Stopped) then
				set_status_stopped
			else
				report_unknown_state
			end
		end

	reanalyze_failure
			-- Redo analysis for row in state failed or aborted
		require
			is_locked: is_locked
			failed_or_aborted: is_failed or is_aborted
		local
			status: JOB_STATUS_VALUES
		do
			set_job_status
			if not aborted then
				if error_text.is_empty then
					requestor.set_job_status (status.Passed)
				else
					requestor.set_job_status (status.Failed)
				end
			else
				requestor.set_job_status (status.Aborted)
			end
			requestor.set_error_explanation (error_text)
			status_changed := True
		end

feature {NONE} -- Implementation

	set_status_created
		do
			requestor.set_status_created
			propagate_info_and_delete_job
			status_changed := True
		end

	set_status_stop_requested
		local
			status: JOB_STATUS_VALUES
		do
			if not job_status.is_equal (status.Executing) then
				requestor.set_status_executing
			end
			batch_job.set_stop_requested
			status_changed := True
		end

	set_status_pending
		do
			lock_requestor
			requestor.set_status_pending
			status_changed := True
		end

	set_status_executing
		do
			lock_requestor
			requestor.set_status_executing
			status_changed := True
		end

	set_status_aborted
		local
			status: JOB_STATUS_VALUES
		do
			lock_requestor_and_job
			requestor.set_job_status (status.Aborted)
			propagate_info_and_delete_job
			status_changed := True
		end

	set_status_stopped
		local
			status: JOB_STATUS_VALUES
		do
			lock_requestor_and_job
			requestor.set_job_status (status.Stopped)
			propagate_info_and_delete_job
			requestor.set_null_error_explanation
			status_changed := True
		end

	set_status_failed
		local
			status: JOB_STATUS_VALUES
		do
			lock_requestor_and_job
			requestor.set_job_status (status.Failed)
			propagate_info_and_delete_job
			status_changed := True
		end

	set_status_passed
			-- Set die status when job passed
		local
			status: JOB_STATUS_VALUES
		do
			lock_requestor_and_job
			set_job_status
			if not aborted then
				if error_text.is_empty then
					requestor.set_job_status (status.Passed)
					if need_pattern_file_stats then
						insert_pattern_file_rows
					end
				else
					requestor.set_job_status (status.Failed)
				end
			else
				requestor.set_job_status (status.Aborted)
			end
			propagate_info_and_delete_job
			requestor.set_error_explanation (error_text)
			status_changed := True
		end

	set_job_status
			-- Set `error_text' to the empty string if job
			-- was successful or to text of error messages
			-- if it failed.  Also, set `aborted' to distinguish
			-- aborted jobs from failed jobs.  Set
			-- `pattern_file_rows' to list of rows to be
			-- inserted into pattern files table if job passed
		local
			file_op: FILE_EXISTENCE_CHECK_OP
			file_opener: FILE_OPEN_READ_OP
			files: LIST [STRING]
			fname, err_desc: STRING
			log: LARGE_PLAIN_TEXT_FILE
			log_file: CATS_LOG_FILE
			errors: LIST [STRING]
			done: BOOLEAN
		do
			pattern_file_rows := Void
			aborted := False
			create file_op
			if has_job_done_file then
				file_op.check_existence (job_done_file)
				done := file_op.path_exists
			else
				-- FIXME?  Check database to see if job done
				done := True
			end
			if done then
				err_desc := ""
				create log.make (job_log_file)
				create file_opener
				file_opener.open (log)
				create log_file.make (log)
				log_file.analyze
				log.close
				if log_file.is_ok then
					pattern_file_rows := log_file.pattern_files
				else
					from
						errors := log_file.errors
						errors.start
					until
						errors.after
					loop
						if not err_desc.is_empty then
							err_desc.extend ('%N')
						end
						err_desc.append (errors.item)
						errors.forth
					end
				end
				
				files := job_output_files
				from
					files.start
				until
					files.after
				loop
					fname := files.item
					file_op.check_existence (fname)
					if not file_op.path_exists then
						if not err_desc.is_empty then
							err_desc.extend ('%N')
						end
						err_desc.append ("Output file " + fu.file_name_part (fname) + " not found")
					end
					files.forth
				end
			else
				err_desc := "File " + job_done_file  + " not found"
				aborted := True
			end
			error_text := err_desc
		end

	lock_requestor_and_job
			-- Lock `requestor' and row in BATCH_JOBS table
		do
			lock_requestor
			lock_job
		end

	lock_job
			-- Lock row in BATCH_JOBS table
		local
			lock_op: ROW_LOCK_OP
		do
			create lock_op
			lock_op.lock_nowait (batch_job)
		end

	lock_requestor
			-- Lock `requestor'
		local
			lock_op: ROW_LOCK_OP
		do
			create lock_op
			lock_op.lock_nowait (requestor)
		end

	propagate_info_and_delete_job
			-- Propagate information from batch jobs row
			-- to requestor row and delete batch jobs row
		local
			s: STRING
		do
			requestor.set_date_job_started (batch_job.date_child_started)
			requestor.set_date_job_ended (batch_job.date_state_changed)
			s := batch_job.error_explanation.twin
			if not s.is_empty then
				s.put (s.item (1).upper, 1)
			end
			
			requestor.set_error_explanation (s)
			requestor.set_null_batch_job_no
			batch_job.delete
			batch_job := Void
		end

	need_pattern_file_stats: BOOLEAN
			-- Should information about pattern files be
			-- recorded in database?
		deferred
		end

	insert_pattern_file_rows
			-- Register rows in `pattern_file_rows' to
			-- be inserted on commit
		local
			row: PATTERN_FILES_MROW
			conn: DB_CONNECTION
		do
			conn := requestor.connection
			from
				pattern_file_rows.start
			until
				pattern_file_rows.after
			loop
				row := pattern_file_rows.item
				row.set_run_id (run_id)
				row.set_pattern_type (pattern_type)
				fill_in_pattern_file_row (row)
				row.register (conn)
				pattern_file_rows.forth
			end
		end

	fill_in_pattern_file_row (row: PATTERN_FILES_MROW)
			-- Set rest of columns in `row' before it
			-- is inserted
		require
			row_exists: row /= Void
		deferred
		end

	existing_pattern_file_rows: LIST [PATTERN_FILES_MROW]
			-- List of pattern file rows in database
			-- created for `Current'
		deferred
		ensure
			result_exists: Result /= Void
		end

	report_unknown_state
			-- Report unknown job state
		local
			error: UNKNOWN_JOB_STATE_ERROR
		do
			create error
			error.set_state (batch_job_state)
			error.set_job_number (batch_job_no)
			error.set_hard_error
			error.disable_retry
			error.handle
		end

	status_values: JOB_STATUS_VALUES
		once
			create Result
		end

	state_values: BATCH_JOB_STATE_VALUES
		once
			create Result
		end

	error_text: STRING
			-- Text of error message describing job failure
			-- or abort

	aborted: BOOLEAN
			-- Did job abort?

	pattern_file_rows: LIST [PATTERN_FILES_MROW]
			-- Rows to be inserted into pattern files table
			-- if job passed

end
