deferred class LAYER_SET [G -> LAYER]
	-- a generic set of layers from an unknown source

inherit
	STRING_UTILITIES_ACCESS
	redefine is_equal	-- name match
	end;

feature -- name

	name: STRING is
		-- used for name match in is_equal
	deferred
	end;

	is_equal(other: like Current): BOOLEAN is
		-- consider name match to be sufficient
	do  Result := equal(name, other.name)
	end;

feature	-- the set of layers

	layer_set: LIST[G] is
	deferred
	ensure Result /= Void
	end;

	number_of_layers: INTEGER is
	do  Result := layer_set.count
	end;

feature	-- lookup of layers by name or cif or gds
	-- NOTE: index/item of layer_set is not affected.

	layer_by_name(nam: STRING): G is
		-- return this layer or Void
	local  lyr: G; lset: like layer_set; pos: CURSOR
	do  lset := layer_set;
	    from  pos := lset.cursor; lset.start
	    until lset.after or Result /= Void
	    loop  lyr := lset.item
		  if equal(nam, lyr.layer_name) then Result := lyr end
		  lset.forth
	    end;
	    lset.go_to(pos)
	ensure Result = Void or else equal(Result.layer_name, nam)
	end;

	layer_by_cif(nam: STRING): G is
		-- return layer with this cif name or Void
	local  lyr: G; lset: like layer_set; pos: CURSOR
	do  lset := layer_set;
	    from  pos := lset.cursor; lset.start
	    until lset.after or Result /= Void
	    loop  lyr := lset.item
		  if lyr.is_my_cif_name(nam) then Result := lyr end
		  lset.forth
	    end;
	    lset.go_to(pos)
	ensure Result = Void or else Result.is_my_cif_name(nam)
	end;

	layer_by_gds_pair(gp: GDS_PAIR): G is
	require specific_datatype: not gp.is_null_datatype
	do  Result := layer_by_gds(gp.layer, gp.datatype)
	end;

	layer_by_gds(layer, datatype: INTEGER): G is
		-- return layer with this gds name or Void
	local  lyr: G; lset: like layer_set; pos: CURSOR
	do  lset := layer_set;
	    from  pos := lset.cursor; lset.start
	    until lset.after or Result /= Void
	    loop  lyr := lset.item
		  if lyr.is_my_gds_name(layer, datatype)
		   then Result := lyr end
		  lset.forth
	    end;
	    lset.go_to(pos)
	ensure Result = Void or else Result.is_my_gds_name(layer, datatype)
	end;

end -- LAYER_SET
