class LAYOUT_COLLECTION
		-- union of arbitrary layout items
		-- (except that all must match my value of is_unlayered)

inherit
	LAYOUT
	undefine is_equal, copy
	end;

	CACHED_BOUNDING_BOX
	undefine is_equal, copy
	end;

	R_ARRAYED_LIST[LAYOUT]
	rename extend as list_extend
	export	{NONE} all;
		{ANY} count, is_empty, trim,
			start, forth, after, item, readable, off, index,
			valid_index, valid_cursor
		{LAYOUT_COLLECTION} 
			object_comparison, cursor, go_to, area, lower, upper,
			i_th, subarray, subcopy, full, before
	undefine out
	end;

create
	make_layered, make_unlayered, make_same_as

create {LAYOUT_COLLECTION}
	make_filled

feature -- the deferred LAYOUT features

	out: STRING is
	do  Result := su.string_of_2("Union of ", count.out)
	end;

	is_unlayered: BOOLEAN
		-- set at creation

	traverse(c: LAYOUT_CONSUMER; m: MATRIX) is
		-- "walk" this layout, (no consumer interaction)
	do  from  start
	    until after or c.terminate_traversal
	    loop  item.traverse(c, m)
		  forth
	    end;
	end;

feature	-- the deferred feature of CACHED_BOUNDING_BOX

	calculate_bb: CONST_RECTANGLE is
		-- calculate my bounding box
	local  foo: CONST_RECTANGLE; mbb: RECTANGLE
	do  from  start
	    until after
	    loop  foo := item.bounding_box;
		  if foo = Void
		   then		-- we have nothing to enclose
		  elseif mbb = Void
		   then create mbb.make_copy(foo)
		   else mbb.extend_to_include(foo)
		  end;
		  forth
	    end;
	    Result := mbb	-- (even if Void)
	end;

	change_layers(remapping: HASH_TABLE[STRING, STRING];
			notifier: PROCEDURE[ANY, TUPLE[LAYOUT, STRING]])
		-- pass it down to all the elements
	do  from  start
	    until after
	    loop  item.change_layers(remapping, notifier)
		  forth
	    end
	end;

feature -- add to the collection

	is_compatible(l: LAYOUT): BOOLEAN is
		-- is this layout compatible with this collection?
	do  Result := l = Void or else (l.is_unlayered = is_unlayered)
	end;

	extend(l: LAYOUT) is
		-- (if l is Void, it is discarded quietly)
	require compatible: is_compatible(l)
	do  if l /= Void then
		list_extend(l);
		cached_bb := Void
	    end
	end;

feature {NONE}

	make_layered(n: INTEGER) is
		-- n is initial collection capacity
	do  is_unlayered := false;
	    make(n)
	end;

	make_unlayered(n: INTEGER) is
		-- n is initial collection capacity
	do  is_unlayered := true
	    make(n)
	end;

	make_same_as(sample: LAYOUT; n: INTEGER) is
		-- sample is used solely to determine our layered-ness
		-- n is initial collection capacity
	require sample /= Void
	do  if sample.is_unlayered
	     then make_unlayered(n)
	     else make_layered(n)
	    end
	ensure is_empty
	end;

end -- LAYOUT_COLLECTION
