class LAYOUT_LAYERING
		-- unlayered layout placed on a layer (by layer name)

inherit
	LAYOUT

create
	make

feature	-- my parts

	layer: STRING

	body: LAYOUT
		-- (may be Void)

feature -- the deferred LAYOUT features

	out: STRING is
	do  Result := su.string_of_2("Layer: ", layer)
	end;

	is_unlayered: BOOLEAN is false
		-- is this unlayered ("monochromatic") layout?

	traverse(c: LAYOUT_CONSUMER; m: MATRIX) is
		-- "walk" this layout, passing interesting parts to 
		--  the designated consumer
	do  if body /= Void then
		c.do_layer(layer, body, m)
	    end
	end;

	have_bounding_box: BOOLEAN
	do  if body /= Void then
		Result := body.have_bounding_box
	    end
	end;

	bounding_box: CONST_RECTANGLE
	do  if body /= Void then
		Result := body.bounding_box
	    end
	end;

	change_layers(remapping: HASH_TABLE[STRING, STRING];
			notifier: PROCEDURE[ANY, TUPLE[LAYOUT, STRING]])
		-- remapping defines the layers to be remapped (the keys)
		--   and the new layer for each (the values)
		-- notifier is called once for each item that is remapped,
		--   with the second argument being the NEW layer name
	do  remapping.search(layer);
	    if remapping.found then
		layer := remapping.found_item;
		if notifier /= Void then
			notifier.call([body, layer])
		end
	    end;
	end;

feature {NONE} -- creation

	make(b: LAYOUT; l: STRING) is
	require have_layer: l /= Void
		unlayered: b /= Void implies b.is_unlayered
	do  layer := l
	    body := b
	end;

end -- LAYOUT_LAYERING
