class LAYOUT_REFERENCE
		-- symbolic reference to another structure, with (optional)
		-- simple arraying in a (first quadrant Cartesian) lattice

inherit
	LAYOUT

	CACHED_BOUNDING_BOX

	LIBRARY_SUPPORT
	undefine out
	end;

create
	make_SREF, make_AREF

feature -- attributes of an SREF or an AREF

	structure_name: STRING

	copies_in_X, copies_in_Y: INTEGER
		-- number of copies in X and Y (both 1 for an SREF)

	is_sref: BOOLEAN is
		-- is this an AREF or an SREF?
	do  Result := (copies_in_X = 1) and (copies_in_Y = 1)
	end;

	is_aref: BOOLEAN is
		-- is this an AREF or an SREF?
	do  Result := (copies_in_X > 1) or (copies_in_Y > 1)
	end;

	step_in_X, step_in_Y: DOUBLE
		-- if is_aref, distance from copy to copy
		-- Calma orthogonality ONLY

feature -- structure renaming (needed for library merging)

	rename_structure(nam: STRING) is
  --	require valid_structure_name(nam);
	do  structure_name := nam
	ensure equal(structure_name, nam)
	end;
	
feature	-- body (which we may not have initially)

	body: LAYOUT
		-- layout of referenced cell

	set_body(b: LAYOUT) is
	require b = Void or else not b.is_unlayered
	do  body := b
	    cached_bb := Void
	end;

feature -- the deferred LAYOUT features

	out: STRING is
	do  if is_aref
	     then Result := su.string_of_6("AREF (", copies_in_x.out,
				" x ", copies_in_y.out,
				") to ", structure_name)
	     else Result := su.string_of_2("SREF to ", structure_name)
	    end
	end;

	is_unlayered: BOOLEAN is false
		-- structures are always layered

	traverse(c: LAYOUT_CONSUMER; m: MATRIX) is
		-- "walk" this layout, passing interesting parts to 
		--  the designated consumer
	do  c.do_reference(Current, m);
	end;

	calculate_bb: CONST_RECTANGLE
		-- (returns Void unless instantiation has occurred)
	do  if body /= Void
	     then Result := body.bounding_box;
		  if is_aref and Result /= Void
		   then create Result.make_by_corners(
				Result.low_x, 
				Result.low_y,
				Result.high_x +
				 (copies_in_X-1)*step_in_x,
				Result.high_y +
				 (copies_in_Y-1)*step_in_y)
		  end;
	    end
	end;

feature {NONE} -- creation

	make_SREF(nam: STRING) is
  --	require valid_structure_name(nam);
	do  make_AREF(nam, 1, 1, 0, 0);	-- sref values
	end;

	make_AREF(nam: STRING; nx, ny: INTEGER; stepX, stepY: DOUBLE) is
	require
  --		valid_structure_name(nam);
		nx >= 1; ny >= 1;
		nx > 1 implies stepX > 0; ny > 1 implies stepY > 0;
	do  structure_name := nam
	    copies_in_x := nx;
	    copies_in_y := ny
	    step_in_X   := stepX;
	    step_in_Y   := stepY;
	end;

	change_layers(remapping: HASH_TABLE[STRING, STRING];
			notifier: PROCEDURE[ANY, TUPLE[LAYOUT, STRING]])
		-- no operation here
		-- (NOT our job to find other structures!!)
	do  
	end;

end -- LAYOUT_REFERENCE
