class LAYOUT_SUPPORT

inherit
	GEOMETRY_SUPPORT

	STRING_UTILITIES_ACCESS

	FILE_UTILITIES_ACCESS

	DB_ACCESS

	DB_QUERY_OPTIONS

feature -- the operational design_technology

	the_technology: LAYER_SET [LAYER]

feature -- some standard (SCMOS) layers

	n_well: LAYER is
	do  Result := the_technology.layer_by_name("N_WELL")
	end;

	p_well: LAYER is
	do  Result := the_technology.layer_by_name("P_WELL")
	end;

	active: LAYER is
	do  Result := the_technology.layer_by_name("ACTIVE")
	end;

	poly: LAYER is
	do  Result := the_technology.layer_by_name("POLY")
	end;

	poly2: LAYER is
	do  Result := the_technology.layer_by_name("POLY2")
	end;

	n_plus_select: LAYER is
	do  Result := the_technology.layer_by_name("N_PLUS_SELECT")
	end;

	p_plus_select: LAYER is
	do  Result := the_technology.layer_by_name("P_PLUS_SELECT")
	end;

	contact: LAYER is
	do  Result := the_technology.layer_by_name("CONTACT")
	end;

	metal1: LAYER is
	do  Result := the_technology.layer_by_name("METAL1")
	end;

	via: LAYER is
	do  Result := the_technology.layer_by_name("VIA")
	end;

	metal2: LAYER is
	do  Result := the_technology.layer_by_name("METAL2")
	end;

	via2: LAYER is
	do  Result := the_technology.layer_by_name("VIA2")
	end;

	metal3: LAYER is
	do  Result := the_technology.layer_by_name("METAL3")
	end;

	via3: LAYER is
	do  Result := the_technology.layer_by_name("VIA3")
	end;

	metal4: LAYER is
	do  Result := the_technology.layer_by_name("METAL4")
	end;

	via4: LAYER is
	do  Result := the_technology.layer_by_name("VIA4")
	end;

	metal5: LAYER is
	do  Result := the_technology.layer_by_name("METAL5")
	end;

	via5: LAYER is
	do  Result := the_technology.layer_by_name("VIA5")
	end;

	metal6: LAYER is
	do  Result := the_technology.layer_by_name("METAL6")
	end;

	glass: LAYER is
	do  Result := the_technology.layer_by_name("GLASS")
	end;

feature -- basic layout constructors

	box(lx, ly, hx, hy: DOUBLE): LAYOUT_RECTANGLE is
		-- rectangle by the edges
	do  create Result.make_by_corners(lx, ly, hx, hy)
	end;

	box_SC(sx, sy, cx, cy: DOUBLE): LAYOUT_RECTANGLE is
		-- rectangle by size and center
	do  create Result.make_by_size_and_center(sx, sy, cx, cy)
	end;

	triangle(x1, y1, x2, y2, x3, y3: DOUBLE): LAYOUT_POLYGON is
		-- a triangle
	do  create Result.make(3);
	    Result.extend_xy(x1, y1);
	    Result.extend_xy(x2, y2);
	    Result.extend_xy(x3, y3);
	end;

	polygon(src: LIST_OF_POINTS): LAYOUT_POLYGON is
		-- an arbitrary polygon
	do  create Result.make_copy(src)
	end;

	default_path_ends: PATH_OPTIONS
		-- used by path1, path2, et al.

	path1(w, x1, y1, x2, y2: DOUBLE): LAYOUT_PATH is
		-- one segment (two point) path
	require w >= 0;
	do  if default_path_ends = Void		-- default the default
	     then default_path_ends := half_square_ends
	    end;
	    create Result.make(w, default_path_ends);
	    Result.extend_xy(x1, y1);
	    Result.extend_xy(x2, y2)
	end;

	path2(w, x1, y1, x2, y2, x3, y3: DOUBLE): LAYOUT_PATH is
		-- two segment (three pont) path
	require w >= 0;
	do  Result := path1(w, x1, y1, x2, y2);
	    Result.extend_xy(x3, y3);
	end;

feature	-- simple text composer

	simple_font: ZACHS_FONT is
	once  create Result.make
	end;

	text(str: STRING): LAYOUT is
		-- NOTE: size is arbitrary; scale as needed
	do  Result := simple_font.text_as_layout(str)
	end;

feature -- layout composers

	layer(body: LAYOUT; l: LAYER): LAYOUT is
	do  if body = Void or l = Void
	     then Result := body
	     else create {LAYOUT_LAYERING} Result.make(body, l.layer_name)
	    end
	end;

	multi_layer(body: LAYOUT; lyrs: LIST[LAYER]): LAYOUT is
	local col: LAYOUT_COLLECTION
	do  if body = Void or lyrs = Void
	     then Result := body
	     else create col.make_layered(lyrs.count);
		  from  lyrs.start
		  until lyrs.after
		  loop  col.extend(layer(body, lyrs.item))
			lyrs.forth
		  end;
		  Result := col
	    end
	end;

	move(body: LAYOUT; by: matrix): LAYOUT is
	local brel: LAYOUT_RELOCATION;
	do  if body = Void or (by = Void or else by.is_identity)
	     then Result := body	-- Void matrix acts as identity
	    else brel ?= body;		-- if relocating a relocation
		 if brel /= Void	--  then just combine matrices
		  then Result := move(brel.body, brel.mat.and_then(by))
		  else create {LAYOUT_RELOCATION} Result.make(body, by)
		 end
	    end
	end;

	slide(body: LAYOUT; x, y: DOUBLE): LAYOUT is
	local at: MATRIX;
	do  if body = Void or (x = 0 and y = 0)
	     then Result := body
	     else create at.make_translation_XY(x, y);
		  Result := move(body, at)
	    end;
	end;

	named(body: LAYOUT; nam: STRING): LAYOUT is
	do  if body = Void or su.is_empty(nam)
	     then Result := body
	     else create {LAYOUT_NAMING} Result.make(body, nam)
	    end
	end;

	port(b: CONST_RECTANGLE; l: LAYER; nam: STRING): LAYOUT is
		-- rectangle is required; layer and name are optional
	require b /= Void
	do  create {LAYOUT_PORT} Result.make_copy(b);
	    if l /= Void
	     then Result := layer(Result, l)
	    end;
	    if su.not_empty(nam)
	     then Result := named(Result, nam)
	    end;
	end;

	call(stru: STRING; at: MATRIX): LAYOUT is
  --	require valid_structure_name(stru);
	do  create {LAYOUT_REFERENCE} Result.make_SREF(stru);
	    Result := move(Result, at)
	end;

	u2(b1, b2: LAYOUT): LAYOUT_COLLECTION is
		-- union of 2 items
	require b1.is_unlayered = b2.is_unlayered
	do  create Result.make_same_as(b1, 8)
	    Result.extend(b1);
	    Result.extend(b2)
	end;

	u3(b1, b2, b3: LAYOUT): LAYOUT_COLLECTION is
		-- union of three items
	require b1.is_unlayered = b2.is_unlayered and
		b1.is_unlayered = b3.is_unlayered
	do  Result := u2(b1, b2);
	    Result.extend(b3)
	end;

	u4(b1, b2, b3, b4: LAYOUT): LAYOUT_COLLECTION is
		-- union of 4 items
	require b1.is_unlayered = b2.is_unlayered and
		b1.is_unlayered = b3.is_unlayered and
		b1.is_unlayered = b4.is_unlayered
	do  Result := u2(b1, b2);
	    Result.extend(b3);
	    Result.extend(b4)
	end;

	stepped_copies(body: LAYOUT; step: VIRTUAL_POINT;
			count: INTEGER): LAYOUT_COLLECTION is
		-- count copies of body at step*n 
		--  for n between 0 and count-1
	require body /= Void; step /= Void; 
	local n: INTEGER
	do  create Result.make_same_as(body, count);
	    from  n := 0
	    until n >= count
	    loop  Result.extend(slide(body, step.x * n, step.y * n))
		  n := n + 1
	    end
	end;

feature -- some layout movers

	center_origin(bdy: LAYOUT): LAYOUT is
		-- slide bdy so its origin is at its center
	local bb: CONST_RECTANGLE
	do  if bdy /= Void
	     then bb := bdy.bounding_box;
		  if bb /= Void
		   then Result := slide(bdy, -bb.center_X, -bb.center_Y)
		  end;
	    end;
	end;

	ll_origin(bdy: LAYOUT): LAYOUT is
		-- slide bdy so its origin is at its lower left corner
	local bb: CONST_RECTANGLE
	do  if bdy /= Void
	     then bb := bdy.bounding_box;
		  if bb /= Void
		   then Result := slide(bdy, -bb.low_X, -bb.low_Y)
		  end;
	    end;
	end;

	column(bdies: LIST[LAYOUT]; sep: DOUBLE): LAYOUT is
		-- stack the elements of bdies vertically into a single
		-- LAYOUT, separating them by sep 
		-- with each centered in x
	require sep >= 0;
	local  bdy: LAYOUT; bb: CONST_RECTANGLE; ly: DOUBLE; 
		col: LAYOUT_COLLECTION;
	do  if bdies /= Void and then not bdies.is_empty
	     then from  bdies.start; ly := 0;
			if bdies.first.is_unlayered
			 then create col.make_unlayered(bdies.count)
			 else create col.make_layered(bdies.count)
			end
		  until bdies.after
		  loop  bdy := bdies.item;
			bb := bdy.bounding_box;
			if bb /= Void		-- move into place and add
			 then bdy := slide(bdy, -bb.center_x, ly-bb.low_y)
			      col.extend(bdy);
			      ly := ly + bb.size_y + sep
			end;
			bdies.forth
		  end;
		  Result := col;
	    end;
	end;

end -- LAYOUT_SUPPORT
