class LC_INSTANTIATE_REFERENCES
		-- instantiate references for one structure
		--  (the one being traversed)
		-- (at least, we do not traverse substructures)

inherit
	LC_IGNORE_TERMINALS
		-- we care not about actual layout

	LC_PURSUE_NAMING
		-- we care not about names

	LC_IGNORE_LAYERINGS
		-- since references are never inside layerings

	LIBRARY_SUPPORT

create
	make

feature	-- our library of sources (set at creation)

	library: LAYOUT_LIBRARY

feature	-- our feedback on completion

	instantiated_references: LIST[STRING]
		-- the ones we found and substituted

	missing_references: LIST[STRING]
		-- the ones we couldn't do

feature -- our worker bee

	do_reference(r: LAYOUT_REFERENCE; at: MATRIX) is
		-- NOTE: if r.body /= Void, we do not alter it
	local nam: STRING; it: LIBRARY_STRUCTURE
	do  if r.body = Void
	     then nam := r.structure_name
		  if library.has_structure(nam)
		   then it := library.get_structure(nam);
			r.set_body(it.body);
			make_note(instantiated_references, nam)
		   else make_note(missing_references, nam)
		  end
	    end;
	end;

	terminate_traversal: BOOLEAN is false

feature -- creation (or re-use)

	make(lib: LAYOUT_LIBRARY) is
	require lib /= Void
	do  library := lib
	    if instantiated_references = Void
	     then create {ARRAYED_LIST[STRING]} instantiated_references.make(0)
		  instantiated_references.compare_objects;
	     else instantiated_references.wipe_out
	    end
	    if missing_references = Void
	     then create {ARRAYED_LIST[STRING]} missing_references.make(0)
		  missing_references.compare_objects;
	     else missing_references.wipe_out
	    end;
	end

end -- LC_INSTANTIATE_REFERENCES

