class LC_PRETTY_PRINT

inherit
	LAYOUT_CONSUMER

	STRING_UTILITIES_ACCESS

	GEOMETRY_SUPPORT		-- for identity

create
	make

feature -- attributes of the output

	text: STRING
		-- the whole banana

	indent: STRING;
		-- the line prefix

	indent_unit: STRING
		-- the string to add to indent for each level

feature -- worker bees

	add_line(t: STRING) is
	do  text.append(indent);
	    text.append(t);
	    text.extend('%N');
	end;

	add_transformed_line(t: STRING; m: MATRIX) is
	do  text.append(indent);
	    text.append(t);
	    if not m.is_identity
	     then text.append(" at ");
		  text.append(m.out)
	    end;
	    text.extend('%N');
	end;

	increase_indent is
	do  indent.append(indent_unit)
	end;

	decrease_indent is
	require indent.count >= indent_unit.count
	do  indent.keep_head(indent.count-indent_unit.count)
	end;

feature -- deferred attributes of LAYOUT_CONSUMER

	do_rectangle(r: LAYOUT_RECTANGLE; at: MATRIX) is
	do  add_transformed_line(r.out, at);
	end;

	do_polygon(p: LAYOUT_POLYGON; at: MATRIX) is
	do  add_transformed_line(p.out, at);
	end;

	do_path(w: LAYOUT_PATH; at: MATRIX) is
	do  add_transformed_line(w.out, at);
	end;

	do_port(p: LAYOUT_PORT; at: MATRIX) is
	do  add_transformed_line(p.out, at);
	end;

	do_text(t: LAYOUT_TEXT; at: MATRIX)
	do  add_transformed_line(t.out, at)
	end;

feature -- consumer for the interesting non-terminals
	-- NOTE: to continue the traversal in depth, the consumer must 
	--	explicitly traverse the contained layout (body)

	do_layer(l: STRING; body: LAYOUT; at: MATRIX) is
	do  add_transformed_line(su.string_of_2("[Layer: ", l), at)
	    increase_indent;
	    body.traverse(Current, identity);
	    decrease_indent
	    add_line("]")
	end;

	do_named(n: STRING; body: LAYOUT; at: MATRIX) is
	do  add_transformed_line(su.string_of_2("[Name: ", n), at)
	    increase_indent;
	    body.traverse(Current, identity);
	    decrease_indent
	    add_line("]")
	end;

	do_reference(r: LAYOUT_REFERENCE; at: MATRIX) is
		-- NOTE: DO NOT continue traversal with r.traverse,
		--  you will generate an infinite recursion
		-- use r.traverse_all
	do  add_transformed_line(su.string_of_2(
					"Calling: ", r.structure_name), at)
	    increase_indent;
	    if r.copies_in_x > 1
	     then add_line(su.string_of_4("X: ", r.copies_in_x.out,
					  " copies stepped ", r.step_in_x.out))
	    end;
	    if r.copies_in_y > 1
	     then add_line(su.string_of_4("Y: ", r.copies_in_y.out,
					  " copies stepped ", r.step_in_y.out))
	    end;
	    decrease_indent
	end;

	terminate_traversal: BOOLEAN is false

feature -- creation

	make(ind_unit: STRING) is
	require ind_unit /= Void
	do  create text.make(5000);
	    create indent.make(20)
	    indent_unit := ind_unit
	end;

end -- LC_PRETTY_PRINT
