deferred class LC_PURSUE_LAYERING
	-- LAYOUT_CONSUMER instantiating do_layer to
	--  keep track of the current layer and keep on trucking
	--  IFF the layer is to be pursued.
	-- There are two potential layer filters:
	--	1. set filtering_techn to cause layers unknown to
	--	   this techn to be skipped;
	--	2. set traversed_layers to a specific list of layers
	--	   that are to be traversed; all others are skipped
	-- (if neither is set, all layers are traversed)

inherit
	LAYOUT_CONSUMER

feature	-- the traversal routine

	do_layer(l: STRING; body: LAYOUT; at: MATRIX)
		-- traverse the body if l is being pursued
		-- hang onto this layer until we return
	do  if doing_layer(l)
	     then current_layer := l;
		  body.traverse(Current, at)
		  current_layer := Void
	    end;
	end;

feature	-- the layer tester against the filters

	doing_layer(l: STRING): BOOLEAN
		-- does l pass both of the traversal tests?
	do  Result :=
	       ((filtering_techn = Void) or else 
		(filtering_techn.layer_by_name(l) /= Void))
	       and
	       ((traversed_layers = Void) or else
		(traversed_layers.has(l)))
	end;

feature	-- filters (filters can only be set directly by descendants)

	filtering_techn: LAYER_SET[LAYER]

	traversed_layers: LIST[STRING]
		-- make sure the list is in compare_objects mode!!

feature	-- the current layer that we are traversing

	current_layer: STRING
		-- Void if not within a layering

end -- LC_PURSUE_LAYERING
