
indexing
	
	description: "Window to control jobs for each level of a run that %
		%requires a job (all levels or possibly just a subset)"
	author: "David Hollenberg"
	date: "March 6, 2004"

deferred class LEVEL_CONTROL_WINDOW

inherit
	JOB_CONTROL_WINDOW [LEVEL_JOB]
	RC_UTILITIES_ACCESS
		export
			{NONE} all
		end
		
feature {NONE} -- Implementation

	add_extra_select_menu_items (select_menu: EV_MENU)
			-- Add any application-specific items to `select_menu'
		do	
		end

	add_extra_view_row_menu_items (row_menu: EV_MENU)
			-- Add any application-specific items to `row_menu'
		do	
		end

	fetch_needed_data
			-- Fetch needed data when first building interface
		do
			fetch_requestor_list
			fetch_batch_jobs_list
		end

	batch_jobs_for_run: LIST [BATCH_JOBS_MROW]
			-- List of batch jobs for this run
		local
			batch_jobs_row: BATCH_JOBS_MROW
		do
			create batch_jobs_row.make_template
			Result := batch_jobs_row.level_jobs_for_run_and_type (session.run_id, job_type, session.connection, expected_count.At_least_zero)
		end
			
	job_type: STRING
			-- Type of jobs to be controlled
		deferred
		ensure
			result_exists: Result /= Void
		end
			
	initialize_total_count
			-- Initialize `total_count'
		do
			total_count := level_rows.count
		end

	add_visible_columns
			-- Add columns to `visible_columns'
		do
			visible_columns.extend (create {LEVEL_NO_COLUMN [LEVEL_JOB]})
			visible_columns.extend (create {LEVEL_ID_COLUMN [LEVEL_JOB]})
			visible_columns.extend (create {LEVEL_NAME_COLUMN [LEVEL_JOB]})
			visible_columns.extend (create {MEBES_EXTENSION_COLUMN [LEVEL_JOB]})
			add_fill_visible_columns
			visible_columns.extend (create {JOB_STATUS_COLUMN [LEVEL_JOB]})
			visible_columns.extend (create {BATCH_JOB_NO_COLUMN [LEVEL_JOB]})
			visible_columns.extend (create {BATCH_JOB_STATE_COLUMN [LEVEL_JOB]})
			visible_columns.extend (create {BATCH_JOB_HOST_COLUMN [LEVEL_JOB]})
			visible_columns.extend (create {JOB_TIME_COLUMN [LEVEL_JOB]})
			visible_columns.extend (create {ERROR_DESCRIPTION_COLUMN [LEVEL_JOB]})
		end
	
	add_fill_visible_columns
			-- Add columns for fill attributes to `visible_columns'
		deferred
		end

	has_job_status_agent (s: STRING): FUNCTION [LEVEL_JOB, TUPLE [LEVEL_JOB], BOOLEAN]
			-- Agent that tells whether row has CATS status `s'
		do
			Result := agent {LEVEL_JOB}.has_job_status (s)
		end

	is_stopable_agent: FUNCTION [LEVEL_JOB, TUPLE [LEVEL_JOB], BOOLEAN]
			-- Agent that tells whether row has CATS job that
			-- is stopable
		do
			Result := agent {LEVEL_JOB}.is_stopable
		end

	is_unstopable_agent: FUNCTION [LEVEL_JOB, TUPLE [LEVEL_JOB], BOOLEAN]
			-- Agent that tells whether row does not have a
			-- CATS job that is stopable
		do
			Result := agent {LEVEL_JOB}.is_unstopable
		end

	is_resettable_agent: FUNCTION [LEVEL_JOB, TUPLE [LEVEL_JOB], BOOLEAN]
			-- Agent that tells whether row has CATS job that
			-- is resettable
		do
			Result := agent {LEVEL_JOB}.is_resettable
		end

	is_unresettable_agent: FUNCTION [LEVEL_JOB, TUPLE [LEVEL_JOB], BOOLEAN]
			-- Agent that tells whether row does not have
			-- a CATS job that is resettable
		do
			Result := agent {LEVEL_JOB}.is_unresettable
		end

	is_unsuccessful_agent: FUNCTION [LEVEL_JOB, TUPLE [LEVEL_JOB], BOOLEAN]
			-- Agent that tells whether row has CATS job that
			-- was unsuccessful
		do
			Result := agent {LEVEL_JOB}.is_unsuccessful
		end

	is_submittable_agent: FUNCTION [LEVEL_JOB, TUPLE [LEVEL_JOB], BOOLEAN]
			-- Agent that tells whether row has CATS job that
			-- is submittable
		do
			Result := agent {LEVEL_JOB}.is_submittable
		end

	is_unsubmittable_agent: FUNCTION [LEVEL_JOB, TUPLE [LEVEL_JOB], BOOLEAN]
			-- Agent that tells whether row is not in a state
			-- where CATS job can be submitted
		do
			Result := agent {LEVEL_JOB}.is_unsubmittable
		end

	is_unreleasable_agent: FUNCTION [LEVEL_JOB, TUPLE [LEVEL_JOB], BOOLEAN]
			-- Agent that tells whether row is not in a state
			-- where batch job can be released
		do
			Result := agent {LEVEL_JOB}.is_unreleasable
		end

	is_passed_agent: FUNCTION [LEVEL_JOB, TUPLE [LEVEL_JOB], BOOLEAN]
			-- Agent that tells whether row has CATS job that
			-- is passed
		do
			Result := agent {LEVEL_JOB}.is_passed
		end

	refresh_corrupted_rows
			-- Refresh requestor for elements of `all_rows'
			-- that were modified and became invalid due
			-- to a rollback
		local
			level_job: LEVEL_JOB
			row: RUN_WAFER_LEVEL_JOBS_MROW
		do
			-- Populate hash table of RUN_WAFER_LEVEL_JOBS rows,
			-- indexed by level number
			
			create levels_table.make (level_rows.count)
			from
				level_rows.start
			until
				level_rows.after
			loop
				row := level_rows.item
				levels_table.put (row, row.level_no)
				level_rows.forth
			end

			from
				all_rows.start
			until
				all_rows.after
			loop
				level_job := all_rows.item
				if level_job.level_jobs_row.is_corrupted then
					levels_table.search (level_job.level_no)
					if levels_table.found then
						row := levels_table.found_item
					else
						handle_internal_error ("cannot find wafer techn level " + level_job.level_no.out + " in internal table")
					end
					level_job.set_level_jobs_row (row)
				end
				
				all_rows.forth
			end
		
		end

	row_to_generic (row: EV_MULTI_COLUMN_LIST_ROW): LEVEL_JOB
			-- Value of `row' which is of type LEVEL_JOB
		do
			Result ?= row
			check
				result_not_void: Result /= Void
			end
		end

	level_rows: LIST [RUN_WAFER_LEVEL_JOBS_MROW]
			-- List of database rows for wafer level
			-- jobs for run whose job type is `job_type'

	levels_table: HASH_TABLE [RUN_WAFER_LEVEL_JOBS_MROW, INTEGER]
			-- Rows from run wafer level jobs table for this run
			-- with `job_type', indexed by level number

end
