
indexing
	
	description: "Window to control fill of pattern files for each %
		%level that needs to be filled for a run"
	author: "David Hollenberg"
	date: "March 6, 2004"

class LEVEL_FILL_CONTROL_WINDOW

inherit
	LEVEL_CONTROL_WINDOW
	NUMBER_UTILITIES_ACCESS
		export
			{NONE} all
		undefine
			default_create
		end
		
create
	make

feature {NONE} -- Implementation

	fetch_requestor_list
			-- Fetch list of rows requesting batch job
			-- (list of wafer levels for run) from database and
			-- put in `level_rows'
		do
			level_rows := cache.run_wafer_level_job_fill_mrows
		end
			
	add_fill_visible_columns
			-- Add columns for fill attributes to `visible_columns'
			-- (not applicable for merge)
		do
			-- Removed for now
			-- visible_columns.extend (create {FILL_REQUIRED_COLUMN [LEVEL_JOB]})
		end
			
	job_type: STRING
			-- Type of jobs to be controlled
		local
			cats_job_type: CATS_JOB_TYPE_VALUES
		do
			Result := cats_job_type.Fill
		end
			
	build_all_rows
			-- Build list `all_rows'
		local
			row: LEVEL_JOB
			level_job: RUN_WAFER_LEVEL_JOBS_MROW
			wtl: RUN_LEVELS_ROW
			mebes_files: ARRAYED_LIST [STRING]
			files: LIST [RC_FILE]
			pattern_type: PATTERN_TYPE_VALUES
		do
			from
				level_rows.start
			until
				level_rows.after
			loop
				level_job := level_rows.item
				wtl := cache.run_level_row_for_level (level_job.level_no)
				create row.make (level_job, wtl)
				row.set_run_id (session.run_id)
				row.set_pattern_type (pattern_type.Fill)
				row.set_job_script_file (session.environment.level_fill_program)
				row.set_job_log_file (cache.level_fill_log_file (wtl).file_pathname)
				row.set_job_done_file (cache.level_fill_done_file (wtl).file_pathname)
				files := cache.level_fill_pattern_files (wtl)
				create mebes_files.make (files.count)
				from
					files.start
				until
					files.after
				loop
					mebes_files.extend (files.item.file_pathname)
					files.forth
				end
				row.set_job_output_files (mebes_files)
				row.select_actions.extend (agent process_select_one (row))
				row.deselect_actions.extend (agent process_deselect_one (row))
				all_rows.extend (row)
				level_rows.forth
			end
		end
			
	job_arguments (row: LEVEL_JOB): STRING
			-- Arguments to be passed to batch job for `row'
		local
			fill_scale_factor, format, space: STRING
			gen: RC_FILE_GENERATOR
			fp: FILL_PATTERNS_ROW
			mt: MASK_TECHNS_ROW
			wt: WAFER_TECHNS_ROW
			levels: LIST [RUN_LEVELS_ROW]
			level, first_level: RUN_LEVELS_ROW
			scale_factor, res: DOUBLE
			fill_levels: STRING
			fields: LIST [FIELD_NAMES_ROW]
		do
			gen := session.file_generator
			fp := cache.fill_patterns_row
			mt := cache.mask_techns_row
			wt := cache.wafer_techns_row
			fields := cache.field_name_rows
			space := " "
			-- Assuming one scale factor for all levels
			levels := cache.run_level_rows
			first_level := cache.run_level_rows.first
			scale_factor := first_level.scale_factor
			fill_scale_factor := nu.std_formatted_number (scale_factor / fp.scale_factor)
			create fill_levels.make (100)
			level := row.wafer_techn_level
			check
				is_filled_level: level.filled_layer
			end
				
			fill_levels.append (level.level_name)
			fill_levels.extend (',')
			fill_levels.append (level.non_null_mebes_extension.as_lower)
			fill_levels.extend (',')
			fill_levels.append (nu.std_formatted_number (level.non_null_merged_pattern_file_spot_size))
			fill_levels.extend (',')
			fill_levels.append_integer (level.non_null_fill_pattern_gds_layer)
			fill_levels.extend (',')
			fill_levels.append (nu.std_formatted_number (level.non_null_fill_pattern_sizing * scale_factor))
			format := cache.mebes_format_for_merged_level (level.level_no)
			fill_levels.extend (',')
			fill_levels.append (format)
			
			res := cache.level_lcm_resolution
			Result := session.run_id + space +
				session.run_directory + space +
				gen.level_merge_root_name (1, 1).file_name  + space +
				gen.filled_mebes_root_name (1, 1).file_name  + space +
					-- Assuming only one field
				fp.file_name + space +
				gen.fill_reset_file.file_name + space +
				gen.fill_selector_file.file_name + space +
				cache.level_fill_done_file (level).file_name + space +
				fill_scale_factor + space +
				fields.first.field_name + space +
					-- Assuming only one field
				nu.std_formatted_number (res) + space +
				su.boolean_as_yes_no (session.environment.is_production) + space +
				fill_levels
		end

	handle_submit_event (submitted_rows: LIST [LEVEL_JOB])
			-- Handle job submission event
		local
			submit_event: LEVEL_FILL_SUBMITTED_EVENT
		do
			create submit_event
			submit_event.set_levels (submitted_rows)
			submit_event.publish
		end

	handle_release_event (released_rows: LIST [LEVEL_JOB])
			-- Handle job release event
		local
			release_event: LEVEL_FILL_RELEASED_EVENT
		do
			create release_event
			release_event.set_levels (released_rows)
			release_event.publish
		end

	handle_changed_event (changed_rows: LIST [LEVEL_JOB])
			-- Handle job status change event
		local
			changed_event: LEVEL_FILL_CHANGED_EVENT
		do
			create changed_event
			changed_event.set_levels (changed_rows)
			changed_event.publish
		end

	handle_all_passed_event
			-- Handle "all rows passed" event
		local
			done_event: LEVEL_FILLS_DONE_EVENT
		do
			create done_event
			done_event.publish
		end

	handle_failure_event (failed_rows, aborted_rows: LIST [LEVEL_JOB])
			-- Handle job failure event
		local
			failed_event: LEVEL_FILL_FAILED_EVENT
		do
			create failed_event
			failed_event.set_failed_levels (failed_rows)
			failed_event.set_aborted_levels (aborted_rows)
			failed_event.publish
		end

	handle_reset_event (reset_rows: LIST [LEVEL_JOB])
			-- Handle job reset event
		local
			reset_event: LEVEL_FILL_RESET_EVENT
		do
			create reset_event
			reset_event.set_levels (reset_rows)
			reset_event.publish
		end

end
