
indexing
	
	description: "Multi-column list row for a wafer techn level and its %
		%associated merge or fill batch job"
	author: "David Hollenberg"
	date: "February 4, 2004"

class LEVEL_JOB
inherit	
	JOB_MULTI_COLUMN_LIST_ROW
		rename
			make as mcl_make,
			requestor as level_jobs_row
		end
	HAS_LEVEL_NUMBER
		rename
			level_number as level_no
		undefine
			copy, is_equal, default_create
		end
	HAS_LEVEL_NAME
		undefine
			copy, is_equal, default_create
		end
	HAS_LEVEL_ID
		rename
			level_id as wafer_vendor_level_id
		undefine
			copy, is_equal, default_create
		end
	HAS_MEBES_EXTENSION
		undefine
			copy, is_equal, default_create
		end
	HAS_DENSITY_REQUIRED_BOOLEAN
		undefine
			copy, is_equal, default_create
		end
	HAS_FILL_REQUIRED_BOOLEAN
		undefine
			copy, is_equal, default_create
		end

create
	make

create {LEVEL_JOB}
	make_filled

feature {NONE} -- Creation

	make (lev_row: RUN_WAFER_LEVEL_JOBS_MROW wtl_row: RUN_LEVELS_ROW)
		require
			level_row_exists: lev_row /= Void
			levels_row_exists: wtl_row /= Void
		do
			default_create
			set_level_jobs_row (lev_row)
			set_wafer_techn_level (wtl_row)
			enable_visible
		ensure
			is_visible: is_visible
		end

feature -- Properties

	level_jobs_row: RUN_WAFER_LEVEL_JOBS_MROW
			-- Row from run wafer level jobs table

	wafer_techn_level: RUN_LEVELS_ROW
			-- Information about the wafer techn level for job

	level_no: INTEGER
			-- Artificial level number
		do
			Result := level_jobs_row.level_no
		end

	wafer_vendor_level_id: STRING
			-- Wafer vendor's name for level
		do
			Result := wafer_techn_level.non_null_wafer_vendor_level_id
		end

	unique_identifier: STRING
			-- Value that uniquely identifies row
		do
			Result := wafer_vendor_level_id
		end

	level_identifier: STRING
			-- Value that identifies row for logging purposes
		do
			Result := level_name + " (" + wafer_vendor_level_id
			if not wafer_vendor_level_id.is_equal (mebes_extension) then
				Result := Result + ", ." + mebes_extension
			end
			Result := Result + ")"
		end

	level_name: STRING
			-- Descriptive name of level
		do
			Result := wafer_techn_level.level_name
		end

	mebes_extension: STRING
			-- MEBES extension for pattern file(s) for level
		do
			Result := wafer_techn_level.non_null_mebes_extension
		end

	density_measurement_required: BOOLEAN
			-- Is a density measurement required for pattern file
			-- for level?
		do
			Result := wafer_techn_level.density_measurement_required
		end

	fill_required: BOOLEAN
			-- Should fill be added to pattern file
			-- for level?
		do
			Result := wafer_techn_level.filled_layer
		end

	job_type: STRING
			-- Type of job to submit
		local
			type: BATCH_JOB_TYPE_VALUES
		do
			Result := type.Merge
		end

	job_name (a_run_id: STRING): STRING
			-- Name of job for `a_run_id'
		do
			Result := a_run_id + "_MERGE_" + wafer_vendor_level_id
		end

	batch_queue_name: STRING
			-- Name of batch queue to use
		local
			queue: BATCH_JOB_QUEUE_VALUES
		do
			Result := queue.Level_merge_batch
		end

feature -- Modification

	set_level_jobs_row (r: RUN_WAFER_LEVEL_JOBS_MROW)
			-- Set `level_jobs_row' to `r'
		do
			level_jobs_row := r
		ensure
			level_jobs_row_set: level_jobs_row = r
		end

	set_wafer_techn_level (wtl: RUN_LEVELS_ROW)
			-- Set `wafer_techn_level' to `wtl'
		do
			wafer_techn_level := wtl
		ensure
			level_set: wafer_techn_level = wtl
		end

feature {NONE} -- Implementation

	need_pattern_file_stats: BOOLEAN = True
			-- Should information about pattern files be
			-- recorded in database?

	fill_in_pattern_file_row (row: PATTERN_FILES_MROW)
			-- Set rest of columns in `row' before it
			-- is inserted
		do
			row.set_level_no (level_no)
		end

	existing_pattern_file_rows: LIST [PATTERN_FILES_MROW]
			-- List of pattern file rows in database
			-- created for `Current'
		local
			row: PATTERN_FILES_MROW
		do
			create row.make_template
			Result := row.rows_for_level_and_type (run_id, level_no, pattern_type, level_jobs_row.connection, expected_count.At_least_zero)
		end

	new_filled_list (n: INTEGER): like Current
			-- New list with `n' elements.
		do
			create Result.make_filled (n)
		end
		
end
