
indexing
	
	description: "Window to control merging of pattern files for each %
		%level for a run"
	author: "David Hollenberg"
	date: "February 2, 2004"

class LEVEL_MERGE_CONTROL_WINDOW

inherit
	LEVEL_CONTROL_WINDOW
	NUMBER_UTILITIES_ACCESS
		export
			{NONE} all
		undefine
			default_create
		end
		
create
	make

feature {NONE} -- Implementation

	fetch_requestor_list
			-- Fetch list of rows requesting batch job
			-- (list of wafer levels for run) from database and
			-- put in `level_rows'
		do
			level_rows := cache.run_wafer_level_job_merge_mrows
		end
			
	add_fill_visible_columns
			-- Add columns for fill attributes to `visible_columns'
			-- (not applicable for merge)
		do
		end
			
	job_type: STRING
			-- Type of jobs to be controlled
		local
			cats_job_type: CATS_JOB_TYPE_VALUES
		do
			Result := cats_job_type.Merge
		end
			
	build_all_rows
			-- Build list `all_rows'
		local
			row: LEVEL_JOB
			level_job: RUN_WAFER_LEVEL_JOBS_MROW
			wtl: RUN_LEVELS_ROW
			mebes_files: ARRAYED_LIST [STRING]
			files: LIST [RC_FILE]
			pattern_type: PATTERN_TYPE_VALUES
		do
			from
				level_rows.start
			until
				level_rows.after
			loop
				level_job := level_rows.item
				wtl := cache.run_level_row_for_level (level_job.level_no)
				create row.make (level_job, wtl)
				row.set_run_id (session.run_id)
				row.set_pattern_type (pattern_type.Merge)
				row.set_job_script_file (session.environment.level_merge_program)
				row.set_job_log_file (cache.level_merge_log_file (wtl).file_pathname)
				row.set_job_done_file (cache.level_merge_done_file (wtl).file_pathname)
				
				files := cache.level_merge_pattern_files (wtl)
				create mebes_files.make (files.count)
				from
					files.start
				until
					files.after
				loop
					mebes_files.extend (files.item.file_pathname)
					files.forth
				end
				
				row.set_job_output_files (mebes_files)
				row.select_actions.extend (agent process_select_one (row))
				row.deselect_actions.extend (agent process_deselect_one (row))
				all_rows.extend (row)
				level_rows.forth
			end
		end
			
	job_arguments (row: LEVEL_JOB): STRING
			-- Arguments to be passed to batch job for `row'
		local
			wtl: RUN_LEVELS_ROW
			mt: MASK_TECHNS_ROW
			fields: LIST [FIELD_NAMES_ROW]
			compact, wild, field_names, field_fnames: STRING
			limits, format, prod: STRING
			field: STRING
			field_name_list, field_fname_list: ARRAYED_LIST [STRING]
			k, fcount, level_no: INTEGER
			size, loc, merged_center: CONST_POINT
		do
			wtl := row.wafer_techn_level
			mt := cache.mask_techns_row
			fields := cache.field_name_rows
			fcount := cache.field_count
			merged_center := cache.merged_center
			level_no := wtl.level_no
			compact := su.boolean_as_yes_no (mt.compaction_allowed)
			wild := "??0??"
			if fcount = 1 then
				field_fnames := "fld"
			else
				create field_fname_list.make (fcount)
				create field_fnames.make (20)
				from
					k := 1
				until
					k > fcount
				loop
					create field.make (3)
					field.extend ('f')
					if k < 10 then
						field.extend ('0')
					end
					field.append (k.out)
					field_fname_list.extend (field)
					k := k + 1
				end
				field_fnames := su.joined_by (field_fname_list, ";")
			end
			
			create field_name_list.make (3)
			from
				fields.start
			until
				fields.after
			loop
				field_name_list.extend (fields.item.field_name)
				fields.forth
			end
			field_names := su.joined_by (field_name_list, ";")
			
			create limits.make (fcount * 40)
			from
				fields.start
			until
				fields.after
			loop
				size := cache.merge_limit_size_for_level (level_no)
				loc := cache.field_location_rows_for_field (fields.item.field_name).first.center
				loc := loc + merged_center
				if not limits.is_empty then
					limits.extend (';')
				end
				limits.append (fracture_limit_string (size, loc))
				fields.forth
			end
			format := cache.mebes_format_for_merged_level (level_no)
			prod := su.boolean_as_yes_no (session.environment.is_production)
			Result := session.run_id + " " +
				wtl.non_null_mebes_extension + " " +
				compact + " " +
				wild + " " +
				wtl.level_no.out + " " +
				nu.std_formatted_number (wtl.non_null_merged_pattern_file_spot_size) + " " +
				field_fnames + " " +
				field_names + " " +
				limits + " " +
				format + " " +
				session.run_directory + " " +
				prod
		end

	fracture_limit_string (size, center: CONST_POINT): STRING
			-- String describing fracture limits (for CATS
			-- limit command) for box with `size' and `center'
		local
			box: CONST_RECTANGLE
		do
			create box.make_size_and_center (size, center)
			create Result.make (40)
			Result.append (nu.std_formatted_number (box.ll.x))
			Result.extend (',')
			Result.append (nu.std_formatted_number (box.ll.y))
			Result.extend (',')
			Result.append (nu.std_formatted_number (box.ur.x))
			Result.extend (',')
			Result.append (nu.std_formatted_number (box.ur.y))

		end

	handle_submit_event (submitted_rows: LIST [LEVEL_JOB])
			-- Handle job submission event
		local
			submit_event: LEVEL_MERGE_SUBMITTED_EVENT
		do
			create submit_event
			submit_event.set_levels (submitted_rows)
			submit_event.publish
		end

	handle_release_event (released_rows: LIST [LEVEL_JOB])
			-- Handle job release event
		local
			release_event: LEVEL_MERGE_RELEASED_EVENT
		do
			create release_event
			release_event.set_levels (released_rows)
			release_event.publish
		end

	handle_changed_event (changed_rows: LIST [LEVEL_JOB])
			-- Handle job status change event
		local
			changed_event: LEVEL_MERGE_CHANGED_EVENT
		do
			create changed_event
			changed_event.set_levels (changed_rows)
			changed_event.publish
		end

	handle_all_passed_event
			-- Handle "all rows passed" event
		local
			done_event: LEVEL_MERGES_DONE_EVENT
		do
			create done_event
			done_event.publish
		end

	handle_failure_event (failed_rows, aborted_rows: LIST [LEVEL_JOB])
			-- Handle job failure event
		local
			failed_event: LEVEL_MERGE_FAILED_EVENT
		do
			create failed_event
			failed_event.set_failed_levels (failed_rows)
			failed_event.set_aborted_levels (aborted_rows)
			failed_event.publish
		end

	handle_reset_event (reset_rows: LIST [LEVEL_JOB])
			-- Handle job reset event
		local
			reset_event: LEVEL_MERGE_RESET_EVENT
		do
			create reset_event
			reset_event.set_levels (reset_rows)
			reset_event.publish
		end

end
