
indexing
	
	description: "An operation that removes a link"
	author: "David Hollenberg"
	date: "April 23, 2003"

class LINK_DELETE_OP
inherit
	OPERATION
		rename
			try_operation as try_link_delete
		end
	EXCEPTIONS_ACCESS
		export
			{NONE} all
		undefine
			default_create
		end
	UNIX_OS_ACCESS
		export
			{NONE} all
		undefine
			default_create
		end

create
	default_create

feature -- Operations

	delete_link (path: STRING)
			-- Delete existing link `path'
		require
			path_not_void: path /= Void
			path_not_empty: not path.is_empty
		do
			path_name := path
			execute
		end

feature {NONE} -- Implementation

	try_link_delete
			-- Try link deletion
		local
			error: LINK_DELETE_ERROR
		do
			try_to_delete_link_safely
			if is_deleted then
				publish_ok_event
			else
				publish_failed_event
				create error
				error.set_hard_error
				error.set_path_name (path_name)
				error.handle
			end
		end

	try_to_delete_link_safely
			-- Try to delete link.  Do not raise
			-- exception if deltion fails, unless the
			-- exception is due to an assertion violation
		local
			tried: BOOLEAN
		do
			if not tried then
				try_to_delete_link
				is_deleted := True
			else
				is_deleted := False
			end
		rescue
			if not exc.assertion_violation then
				tried := True
				retry
			end
		end

	try_to_delete_link
		do
			unix_os.unlink (path_name)
		end

	path_name: STRING
			-- Name of existing link

	is_deleted: BOOLEAN
			-- Was link successfully deleted?

	publish_ok_event
		local
			ok_event: LINK_DELETE_OK_EVENT
		do
			create ok_event
			ok_event.set_path_name (path_name)
			ok_event.publish
		end

	publish_failed_event
		local
			failed_event: LINK_DELETE_FAILED_EVENT
		do
			create failed_event
			failed_event.set_path_name (path_name)
			failed_event.publish
		end

end
