
indexing
	
	description: "Sorter for lists"
	author: "David Hollenberg"
	date: "January 17, 2007"

deferred class LIST_SORTER [G]

feature -- Operations

	sorted (list: LIST [G] a_comparator: KL_COMPARATOR [G]): LIST [G]
			-- `list' sorted by `a_comparator'
		require
			list_exists: list /= Void
			a_comparator_exists: a_comparator /= Void
		do
			initialize (list, a_comparator)
			sorter.sort (items)
			populate
			Result := sorted_list
			clear
		ensure
			result_exists: Result /= Void
		end

	reverse_sorted (list: LIST [G] a_comparator: KL_COMPARATOR [G]): LIST [G]
			-- `list' sorted in reverse order by `a_comparator'
		require
			list_exists: list /= Void
			a_comparator_exists: a_comparator /= Void
		do
			initialize (list, a_comparator)
			sorter.reverse_sort (items)
			populate
			Result := sorted_list
			clear
		ensure
			result_exists: Result /= Void
		end

feature {NONE} -- Implementation

	initialize (list: LIST [G] a_comparator: KL_COMPARATOR [G])
		require
			list_exists: list /= Void
			a_comparator_exists: a_comparator /= Void
		local
			k: INTEGER
		do
			create_sorter (a_comparator)
			sorted_list := list.twin
			sorted_list.wipe_out
			create items.make (1, list.count)
			from
				list.start
				k := 1
			until
				list.after
			loop
				items.put (list.item, k)
				list.forth
				k := k + 1
			end
		end

	create_sorter (a_comparator: KL_COMPARATOR [G])
		require
			a_comparator_exists: a_comparator /= Void
		deferred
		end

	populate
			-- Populate `sorted_list' from `items'
		require
			sorted_list_exists: sorted_list /= Void
		local
			k: INTEGER
		do
			from
				k := items.lower
			until
				k > items.upper
			loop
				sorted_list.extend (items.item (k))
				k := k + 1
			end
		end

	clear
			-- Clear attributes
		do
			sorter := Void
			items := Void
			sorted_list := Void
		end

	sorter: DS_ARRAY_SORTER [G]
			-- Object that will do sorting

	items: ARRAY [G]
			-- Items to be sorted

	sorted_list: LIST [G]
			-- List with items sorted

end
