
indexing

	author: "Generated"
	date: "Wed Feb 11 14:31:07 PST 2004"

class LITHO_SIZES_ROW
inherit
	LITHO_SIZES_RAW
	ROW_COUNT_CHECK_ACCESS
	NULL_VALUE_CHECK_ACCESS
	ITEM_COUNT_RANGES_ACCESS
	DB_QUERY_OPTIONS
		export {NONE} all end

create
	make_template, make_null, make_by_key

feature -- Properties

	external_kerf_x: DOUBLE is
			-- Size of the external kerf in x
		do  
			if not is_null_field_size_x then
				Result := field_size_x - payload_size_x
			else
				Result := 0.0
			end
		end

	external_kerf_y: DOUBLE is
			-- Size of the external kerf in y
		do  
			if not is_null_field_size_y then
				Result := field_size_y - payload_size_y
			else
				Result := 0.0
			end
		end

	litho_sizes_with_name (name: STRING conn: DB_CONNECTION range: ITEM_COUNT_RANGE): LIST [like Current] is
			-- All litho sizes named `name'
		require
			name_exists: name /= Void
			connection_exists: conn /= Void
			connection_open: conn.is_open
			range_exists: range /= Void
		local
			b: DB_BIND_S
		do
			create b.make (name)
			Result := all_rows_filtered (conn, "litho_size_id = :1", b)
			check_row_count (Result, range)
		end

	custom_litho_sizes_for_run (run: STRING conn: DB_CONNECTION range: ITEM_COUNT_RANGE): DYNAMIC_LIST [like Current]
			-- All litho sizes which are custom
			-- ones for `run'
		require
			run_not_void: run /= Void
			connection_exists: conn /= Void
			connection_open: conn.is_open
			range_exists: range /= Void
		local
			b: DB_BIND_S
		do
			create b.make (run)
			Result := all_rows_filtered_ordered (conn, "run_id = :1", b, "litho_size_id")
			check_row_count (Result, range)
		ensure
			result_exists: Result /= Void
		end

	field_size: CONST_POINT is
			-- Size of field in microns, in wafer coordinates
		do
			check_not_null ("field_size_x", is_null_field_size_x, Current)
			check_not_null ("field_size_y", is_null_field_size_y, Current)
			create Result.make_xy (field_size_x, field_size_y)
		ensure
			result_exists: Result /= Void
		end

	payload_size: CONST_POINT is
			-- Size of payload in microns, in wafer coordinates
		do
			check_not_null ("payload_size_x", is_null_payload_size_x, Current)
			check_not_null ("payload_size_y", is_null_payload_size_y, Current)
			create Result.make_xy (payload_size_x, payload_size_y)
		ensure
			result_exists: Result /= Void
		end

	payload_center: CONST_POINT is
			-- Location of center of payload in microns relative to
			-- center of field, in wafer coordinates
		do
			check_not_null ("payload_location_x", is_null_payload_location_x, Current)
			check_not_null ("payload_location_y", is_null_payload_location_y, Current)
			create Result.make_xy (payload_location_x, payload_location_y)
		ensure
			result_exists: Result /= Void
		end

	payload_box: CONST_RECTANGLE is
			-- Rectangle that defines extent of payload in 
			-- microns relative to center of field, in 
			-- wafer coordinates
		do
			create Result.make_size_and_center (payload_size, payload_center)
		ensure
			result_exists: Result /= Void
		end

	merged_size: CONST_POINT is
			-- Size of merged pattern file in microns, in mask
			-- coordinates
		do
			check_not_null ("merged_pattern_file_size_x", is_null_merged_pattern_file_size_x, Current)
			check_not_null ("merged_pattern_file_size_y", is_null_merged_pattern_file_size_y, Current)
			create Result.make_xy (merged_pattern_file_size_x, merged_pattern_file_size_y)
		ensure
			result_exists: Result /= Void
		end

	merged_center: CONST_POINT is
			-- Location of center of merged pattern file
			-- in microns, relative to center of field in mask
			-- coordinates
		do
			check_not_null ("merged_pattern_file_location_x", is_null_merged_pattern_file_location_x, Current)
			check_not_null ("merged_pattern_file_location_y", is_null_merged_pattern_file_location_y, Current)
			create Result.make_xy (merged_pattern_file_location_x, merged_pattern_file_location_y)
		ensure
			result_exists: Result /= Void
		end

	merged_box: CONST_RECTANGLE is
			-- Rectangle that defines extent of merged
			-- pattern files in microns, relative to
			-- center of field in mask coordinates
		do
			create Result.make_size_and_center (merged_size, merged_center)
		ensure
			result_exists: Result /= Void
		end


	has_left_flat: BOOLEAN is
			-- Does `Current' have a flat or notch on left
			-- side of wafer?
		local
			flat: WAFER_FLAT_LOCATION_VALUES
			main_flat: STRING
		do
			main_flat := non_null_primary_flat_location
			Result := main_flat.is_equal (flat.Left) or main_flat.is_equal (flat.Unknown)
		end

	has_right_flat: BOOLEAN is
			-- Does `Current' have a flat or notch on
			-- right side of wafer?
		local
			flat: WAFER_FLAT_LOCATION_VALUES
			main_flat: STRING
		do
			main_flat := non_null_primary_flat_location
			Result := main_flat.is_equal (flat.Right) or main_flat.is_equal (flat.Unknown)
		end

	has_top_flat: BOOLEAN is
			-- Does `Current' have a flat or notch on top
			-- side of wafer?
		local
			flat: WAFER_FLAT_LOCATION_VALUES
			main_flat: STRING
		do
			main_flat := non_null_primary_flat_location
			Result := main_flat.is_equal (flat.Top) or main_flat.is_equal (flat.Unknown)
		end

	has_bottom_flat: BOOLEAN is
			-- Does `Current' have a flat or notch on
			-- bottom side of wafer?
		local
			flat: WAFER_FLAT_LOCATION_VALUES
			main_flat: STRING
		do
			main_flat := non_null_primary_flat_location
			Result := main_flat.is_equal (flat.Bottom) or main_flat.is_equal (flat.Unknown)
		end

	non_null_wafer_type_id: like wafer_type_id is
			-- Value of `wafer_type_id', with error 
			-- check for nullness
		do
			check_not_null ("wafer_type_id", is_null_wafer_type_id, Current)
			Result := wafer_type_id
		end

	non_null_primary_flat_location: like primary_flat_location is
			-- Value of `primary_flat_location', with error 
			-- check for nullness
		do
			check_not_null ("primary_flat_location", is_null_primary_flat_location, Current)
			Result := primary_flat_location
		end

feature	-- decrypting the packing target

	is_standard_reticle: BOOLEAN
	local  ptv: PACKING_TARGET_VALUES
	do  Result := equal (packing_target, ptv.standard_reticle)
	end;

	is_standard_reticle_die: BOOLEAN
	local  ptv: PACKING_TARGET_VALUES
	do  Result := equal (packing_target, ptv.standard_reticle_die)
	end;

	is_custom_reticle: BOOLEAN
	local  ptv: PACKING_TARGET_VALUES
	do  Result := equal (packing_target, ptv.custom_reticle)
	end;

	is_custom_reticle_die: BOOLEAN
	local  ptv: PACKING_TARGET_VALUES
	do  Result := equal (packing_target, ptv.custom_reticle_die)
	end;

	is_reticle: BOOLEAN 
		-- standard or custom
	do  Result := is_standard_reticle or is_standard_reticle_die 
		or is_custom_reticle or is_custom_reticle_die
	end;

	is_non_reticle: BOOLEAN 
		-- Not a standard or custom reticle
	do  Result := not is_reticle
	end;

	is_custom_tile: BOOLEAN
	local  ptv: PACKING_TARGET_VALUES
	do  Result := equal (packing_target, ptv.custom_tile)
	end;

	is_custom_target: BOOLEAN
		-- reticle or tile
	do  Result := is_custom_reticle or is_custom_reticle_die or is_custom_tile
	end;

	is_unpacked_die: BOOLEAN
	local  ptv: PACKING_TARGET_VALUES
	do  Result := equal (packing_target, ptv.unpacked_die)
	end;

	is_die_merge: BOOLEAN
		do
			Result := is_unpacked_die or is_standard_reticle_die or is_custom_reticle_die
		end

	is_field_merge: BOOLEAN
		do
			Result := not is_die_merge
		end

feature	-- mask_sizes_row and wafer_types_row

	mask_size: MASK_SIZES_ROW is
	do  if is_null_mask_size_id
	     then cached_mask_size := Void
	    elseif cached_mask_size = Void
	     then create cached_mask_size.make_by_key
				(connection, old_row, mask_size_id)
	    end;
	    Result := cached_mask_size
	end;

	wafer_type: WAFER_TYPES_ROW is
	do  if is_null_wafer_type_id
	     then cached_wafer_type := Void
	    elseif cached_wafer_type = Void
	      then create cached_wafer_type.make_template;
			cached_wafer_type := 
				cached_wafer_type.wafer_types_with_name
					(wafer_type_id, connection,
					 expected_count.exactly_one).first
	    end;
	    Result := cached_wafer_type
	end;

feature	{NONE} -- cached data

	cached_mask_size: MASK_SIZES_ROW

	cached_wafer_type: WAFER_TYPES_ROW

end
