
indexing
	
	description: "An operation that locks a lock"
	author: "David Hollenberg"
	date: "March 12, 2003"

class LOCK_OP
inherit
	OPERATION
		rename
			try_operation as try_lock
		end

create
	default_create

feature -- Properties

feature -- Operations

	lock_nowait (l: LOCK)
		require
			lock_exists: l /= Void
			lock_not_locked: not l.is_locked_by_me
		do
			lock := l
			execute
		end

feature {NONE} -- Implementation

	try_lock
			-- Try file lock
		local
			file: LARGE_RAW_FILE
			file_opener: FILE_OPEN_READ_APPEND_OP
			file_closer: FILE_CLOSE_OP
			error: LOCK_ERROR
		do
			create file.make (lock.lock_name)
			create file_opener
			file_opener.open (file)
			create file_closer
			file_closer.close (file)
			publish_try_event
			lock.lock_nowait
			if lock.is_locked_by_me then
				publish_ok_event
			else
				publish_failed_event
				create error
				error.set_hard_error
				error.disable_notify
				error.set_lock (lock)
				error.handle
			end
		end

	lock: LOCK
			-- Lock to try to lock

	publish_try_event
		local
			try_event: LOCK_TRY_EVENT
		do
			create try_event
			try_event.set_lock (lock)
			try_event.publish
		end

	publish_ok_event
		local
			ok_event: LOCK_OK_EVENT
		do
			create ok_event
			ok_event.set_lock (lock)
			ok_event.publish
		end

	publish_failed_event
		local
			failed_event: LOCK_FAILED_EVENT
		do
			create failed_event
			failed_event.set_lock (lock)
			failed_event.publish
		end

end
