
indexing
	
	description: "Multi-column list row that has additional information %
		%about the entity it represents that uniquely identifies %
		%the entity and associates it with a wafer etest name (lot)"
	author: "David Hollenberg"
	date: "December 6, 2005"

class LOT_SUMMARY_MULTI_COLUMN_LIST_ROW
inherit	
	MULTI_COLUMN_LIST_ROW
		rename
			make as mcl_make
		end
	HAS_WAFER_ETEST_ID
		undefine
			copy, is_equal, default_create
		end
	HAS_REQUIRED_COUNT
		undefine
			copy, is_equal, default_create
		end
	HAS_INCOMPATIBLE_COUNT
		undefine
			copy, is_equal, default_create
		end
	HAS_ASSIGNED_COUNT
		undefine
			copy, is_equal, default_create
		end
	HAS_TOTAL_ASSIGNED_COUNT
		undefine
			copy, is_equal, default_create
		end
	HAS_ASSIGNABLE_COUNT
		undefine
			copy, is_equal, default_create
		end
	HAS_TOTAL_COUNT
		undefine
			copy, is_equal, default_create
		end
	MOSIS_UTILITIES
		export
			{NONE} all
		undefine
			copy, is_equal, default_create
		end

create
	make

create {LOT_SUMMARY_MULTI_COLUMN_LIST_ROW}
	make_filled

feature {NONE} -- Creation

	make (a_wafer_etest_id: STRING)
		require
			a_wafer_etest_id_not_void: a_wafer_etest_id /= Void
		do
			default_create
			wafer_etest_id := a_wafer_etest_id
			enable_visible
		ensure
			is_visible: is_visible
		end

feature -- Properties

	wafer_etest_id: STRING
			-- Wafer etest ID for lot

	has_wafer_etest_id: BOOLEAN = True
			-- Is there an associated wafer etest ID?

	required_count: INTEGER
			-- Number of projects that require this wafer etest ID

	has_required_count: BOOLEAN = True
			-- Is there an associated required count?

	incompatible_count: INTEGER
			-- Number of projects that are incompatible with
			-- this wafer etest ID

	has_incompatible_count: BOOLEAN = True
			-- Is there an associated incompatible count?

	assigned_count: INTEGER
			-- Number of projects that are currently assigned to
			-- this wafer etest ID but do not require it

	has_assigned_count: BOOLEAN = True
			-- Is there an associated assigned count?

	assignable_count: INTEGER
			-- Number of projects that are assignable to
			-- this wafer etest ID but are not currently assigned
			-- to it

	has_assignable_count: BOOLEAN = True
			-- Is there an associated assignable count?

	total_assigned_count: INTEGER
			-- Number of projects that are currently assigned to
			-- this wafer etest ID (including those that require
			-- it)

	has_total_assigned_count: BOOLEAN = True
			-- Is there an associated total assigned count?

	total_count: INTEGER
			-- Total number of projects

	has_total_count: BOOLEAN = True
			-- Is there an associated total count?

	unique_identifier: STRING
			-- Value that uniquely identifies row
		do
			Result := wafer_etest_id
		end

feature -- Properties

	set_required_count (a_value: like required_count)
			-- Set `required_count' to `a_value'
		do
			required_count := a_value
		end

	set_incompatible_count (a_value: like incompatible_count)
			-- Set `incompatible_count' to `a_value'
		do
			incompatible_count := a_value
		end

	set_assigned_count (a_value: like assigned_count)
			-- Set `assigned_count' to `a_value'
		do
			assigned_count := a_value
		end

	set_assignable_count (a_value: like assignable_count)
			-- Set `assignable_count' to `a_value'
		do
			assignable_count := a_value
		end

	set_total_assigned_count (a_value: like total_assigned_count)
			-- Set `total_assigned_count' to `a_value'
		do
			total_assigned_count := a_value
		end

	set_total_count (a_value: like total_count)
			-- Set `total_count' to `a_value'
		do
			total_count := a_value
		end

feature {NONE} -- Implementation

	new_filled_list (n: INTEGER): like Current
			-- New list with `n' elements.
		do
			create Result.make_filled (n)
		end
		
end
