class M_PARTS_ROW

inherit
	M_PARTS_RAW
		export
			{M_PARTS_ROW} shippable,	-- use is_shippable
			fe_reportable,		-- use is_fe_reportable
			discountable		-- use is_discountable
		end

	ROW_COUNT_CHECK_ACCESS
		export
			{NONE} all
		end

	MOSIS_MPRO_VALUES
		export
			{NONE} all
		end

	DB_QUERY_OPTIONS
		export
			{NONE} all
		end

create
	make_by_key, make_template, make_null


feature -- Properties

	parts_with_number (n: INTEGER conn: DB_CONNECTION range: ITEM_COUNT_RANGE): LIST [like Current] is
			-- All parts with part number `n'
		require
			connection_exists: conn /= Void
			connection_open: conn.is_open
			range_exists: range /= Void
		local
			b: DB_BIND_I
		do
			create b.make (n)
			Result := all_rows_filtered (conn, "m_part_no = :1", b)
			check_row_count (Result, range)
		ensure
			result_exists: Result /= Void
		end

feature -- nature of the part

	is_IC: BOOLEAN is
	do  Result := equal(part_type, part_is_IC)
	end;

	is_IC_prep: BOOLEAN is
	do  Result := equal(part_type, part_is_IC_prep)
	end;

	is_IC_or_prep: BOOLEAN is
	do  Result := is_IC or is_IC_prep
	end;

	is_package: BOOLEAN is
	do  Result := equal(part_type, part_is_pkg)
	end;

	is_spare_package: BOOLEAN is
	do  Result := equal(part_type, part_is_spare_pkg)
	end;

	is_fab_run: BOOLEAN is
	do  Result := equal(part_type, part_is_fab_run)
	end;

	is_DRC: BOOLEAN is
	do  Result := equal(part_type, part_is_DRC)
	end;

	is_initial: BOOLEAN is
	do  Result := part_increment.is_empty	-- meaning initial & addl
			or else
			equal(part_increment, increment_is_init)
	end;

	is_additional: BOOLEAN is
	do  Result := part_increment.is_empty or else
			equal(part_increment, increment_is_addl)
	end;

	is_spare: BOOLEAN is
	do  Result := equal(part_increment, increment_is_spare)
	end;

	is_short_lot: BOOLEAN is
	do  Result := equal(part_increment, increment_is_short_lot)
	end;

	is_sold_in_lots: BOOLEAN is
	do  Result := equal(part_unit, unit_is_lot)
	end;

	is_sold_singly: BOOLEAN is
	do  Result := equal(part_unit, unit_is_each)
	end;

feature	-- compound nature

	is_initial_lot: BOOLEAN is
		-- is an initial part that is sold in lots
	do  Result := is_initial and is_sold_in_lots
	end;

	is_additional_lot: BOOLEAN is
		-- is an additionalpart that is sold in lots
	do  Result := is_additional and is_sold_in_lots
	end;

	is_quantity_quotable_part: BOOLEAN is
		-- is the (IC) part used for quantity quoted prices
	do  Result := is_sold_singly and is_null_tg_family_tier_no
	end;

	is_shippable: BOOLEAN is
		-- is this a part that we actually ship??
	do  if is_null_shippable
	     then Result := my_part_type.shippable
	     else Result := shippable
	    end
	end;

	is_fe_reportable: BOOLEAN is
		-- is this a part that the front end orders / reports
	do  if is_null_fe_reportable
	     then Result := my_part_type.fe_reportable
	     else Result := fe_reportable
	    end
	end;

	is_discountable: BOOLEAN is
		-- is this a part to which discounts apply?
	do  if is_null_discountable
	     then Result := my_part_type.discountable
	     else Result := discountable
	    end
	end;

	my_part_type: like cached_part_type is
	do  if cached_part_type = Void
	     then create cached_part_type.make_by_key(
			connection, old_row, part_type)
	    end;
	    Result := cached_part_type
	end;

	select_part_type(master: LIST[like cached_part_type]) is
		-- pick out of cache rather than query
	do  from  master.start
		  cached_part_type := Void
	    until master.after or cached_part_type /= Void
	    loop  if equal(part_type, master.item.part_type)
		   then cached_part_type := master.item
		  end;
		  master.forth
	    end
	end;

feature {NONE} -- cache

	cached_part_type: PART_TYPES_ROW

end -- M_PARTS_ROW
