
indexing

	author: "David Hollenberg"
	date: "November 29, 2004"

class MASK_SETS_ROW

inherit
	MASK_SETS_RAW
	ROW_COUNT_CHECK_ACCESS

create
	make_template, make_null, make_by_key

feature

	mask_sets_for_run (run: STRING conn: DB_CONNECTION range: ITEM_COUNT_RANGE): LIST [like Current] is
			-- All rows for mask sets for `run'
		require
			run_exists: run /= Void
			connection_exists: conn /= Void
			connection_open: conn.is_open
			range_exists: range /= Void
		local
			b: DB_BIND_S
		do
			create b.make (run)
			Result := all_rows_filtered (conn, "run_id = :1", b)
			check_row_count (Result, range)
		ensure
			result_exists: Result /= Void
		end

	mask_sets_for_initial_run_prod (run: STRING conn: DB_CONNECTION range: ITEM_COUNT_RANGE): LIST [like Current] is
			-- All rows for mask sets for `run'
			-- whose first production is an INITIAL_RUN
			-- production, ordered by mask revision
		require
			run_not_void: run /= Void
			connection_not_void: conn /= Void
			connection_open: conn.is_open
			range_exists: range /= Void
		local
			b: DB_BIND_SS
			prod_type: PRODUCTION_TYPE_VALUES
		do
			create b.make (run, prod_type.Initial_run)
			Result := all_rows_filtered_ordered (conn, "run_id = :1 and production_id in (select production_id from productions where production_type = :2 and run_id = mset.run_id)", b, "mask_revision")
			check_row_count (Result, range)
		ensure
			result_exists: Result /= Void
		end

	mask_sets_for_run_and_revision (run: STRING rev: CHARACTER conn: DB_CONNECTION range: ITEM_COUNT_RANGE): LIST [like Current] is
			-- All rows for mask sets for `run' with revision `rev'
		require
			run_exists: run /= Void
			connection_exists: conn /= Void
			connection_open: conn.is_open
			range_exists: range /= Void
		local
			b: DB_BIND_SC
		do
			create b.make (run, rev)
			Result := all_rows_filtered (conn, "run_id = :1 and mask_revision = :2", b)
			check_row_count (Result, range)
		ensure
			result_exists: Result /= Void
		end

end
