
indexing
	
	description: "Object that can generate colors for an %
		%element of a reticle view according to whether it has a %
		%maximum die size"
	author: "David Hollenberg"
	date: "October 1, 2006"

class MAX_DIE_SIZE_COLOR_GENERATOR
inherit
	RETICLE_VIEW_ELEMENT_COLOR_GENERATOR

create
	make

feature -- Properties

	tight_packing_fit: INTEGER
			-- Maximum difference between minimum and
			-- maximum die size for die to be considered a
			-- tight fit for packing purposes

feature -- Modification

	set_tight_packing_fit (a_value: like tight_packing_fit)
			-- Set `tight_packing_fit' to `a_value'
		do
			tight_packing_fit := a_value
		ensure
			value_set: tight_packing_fit = a_value
		end

feature -- Access

	border_color (an_element: RETICLE_VIEW_ELEMENT): EV_COLOR
			-- Foreground color for border (usually a
			-- rectangle) of `an_element'
		do
			Result := standard_color (an_element)
		end
	
	label_color (an_element: RETICLE_VIEW_ELEMENT): EV_COLOR
			-- Foreground color for label
		do
			Result := standard_color (an_element)
		end
	
feature {NONE} -- Implementation

	standard_color (an_element: RETICLE_VIEW_ELEMENT): EV_COLOR
			-- Standard color for `an_element', based on
			-- whether it has a maximum die size
		do
			if not an_element.is_selectable then
				Result := palette.unselectable_color
			elseif an_element.is_die and then an_element.max_die_size /= Void then
					if an_element.slop_x > tight_packing_fit and an_element.slop_y > tight_packing_fit then
						Result := palette.has_max_die_size_color
					else
						Result := palette.tight_fit_color
					end
			else
				Result := palette.no_max_die_size_color
			end
		end
	
end
