
indexing
	
	description: "Window to control creation of merged GDS files for each %
		%field or each project on a run"
	author: "David Hollenberg"
	date: "October 9, 2008"

deferred class MERGED_GDS_CONTROL_WINDOW [reference G -> GDS_MERGE_JOB]

inherit
	JOB_CONTROL_WINDOW [G]
		
feature {NONE} -- Implementation

	add_extra_select_menu_items (select_menu: EV_MENU)
			-- Add any application-specific items to `select_menu'
		do	
		end

	add_extra_view_row_menu_items (row_menu: EV_MENU)
			-- Add any application-specific items to `row_menu'
		do	
		end

	fetch_needed_data
			-- Fetch needed data when first building interface
		do
			fetch_requestor_list
			fetch_batch_jobs_list
		end

	fetch_requestor_list
			-- Fetch list of rows requesting batch job
			-- fer levels for run) from database and
			-- put in `gds_merge_rows'
		do
			gds_merge_rows := cache.run_gds_file_mrows
		end
			
	batch_jobs_for_run: LIST [BATCH_JOBS_MROW]
			-- List of batch jobs for this run
		local
			batch_jobs_row: BATCH_JOBS_MROW
		do
			create batch_jobs_row.make_template
			Result := batch_jobs_row.gds_merge_jobs_for_run (session.run_id, session.connection, expected_count.At_least_zero)
		end
			
	initialize_total_count
			-- Initialize `total_count'
		do
			total_count := gds_merge_rows.count
		end

	add_visible_columns
			-- Add columns to `visible_columns'
		do
			add_descendant_visible_columns
			visible_columns.extend (create {JOB_STATUS_COLUMN [G]})
			visible_columns.extend (create {BATCH_JOB_NO_COLUMN [G]})
			visible_columns.extend (create {BATCH_JOB_STATE_COLUMN [G]})
			visible_columns.extend (create {BATCH_JOB_HOST_COLUMN [G]})
			visible_columns.extend (create {JOB_TIME_COLUMN [G]})
			visible_columns.extend (create {ERROR_DESCRIPTION_COLUMN [G]})
		end
	
	add_descendant_visible_columns
			-- Add columns specific to descendant to `visible_columns'
		deferred
		end

	build_all_rows
			-- Build list `all_rows'
		local
			row: G
			gds_merge_job: RUN_GDS_FILES_MROW
		do
			from
				gds_merge_rows.start
			until
				gds_merge_rows.after
			loop
				gds_merge_job := gds_merge_rows.item
				row := new_gds_merge_row (gds_merge_job)
				row.set_run_id (session.run_id)
				row.set_job_script_file (session.environment.gds_merge_program)
				row.set_job_done_file (Void)
				
				row.select_actions.extend (agent process_select_one (row))
				row.deselect_actions.extend (agent process_deselect_one (row))
				all_rows.extend (row)
				gds_merge_rows.forth
			end
		end
			
			
	new_gds_merge_row (job: RUN_GDS_FILES_MROW): G
			-- New row to be added to interface for
			-- `job'
		deferred
		end

	job_arguments (row: G): STRING
			-- Arguments to be passed to batch job for `row'
		do
			Result := "parm_file=" + session.environment.gds_merge_parm_file + " run_id=" + session.run_id + " file_no=" + row.file_no.out
		end

	refresh_corrupted_rows
			-- Refresh requestor for elements of `all_rows'
			-- that were modified and became invalid due
			-- to a rollback
		local
			job: G
			row: RUN_GDS_FILES_MROW
		do
			-- Populate hash table of RUN_GDS_FILES rows,
			-- indexed by file_no
			
			create files_table.make (gds_merge_rows.count)
			from
				gds_merge_rows.start
			until
				gds_merge_rows.after
			loop
				row := gds_merge_rows.item
				files_table.put (row, row.file_no)
				gds_merge_rows.forth
			end

			from
				all_rows.start
			until
				all_rows.after
			loop
				job := all_rows.item
				if job.run_gds_files_row.is_corrupted then
					files_table.search (job.file_no)
					if files_table.found then
						row := files_table.found_item
					else
						handle_internal_error ("cannot find file_no " + job.file_no.out + " in internal table")
					end
					job.set_run_gds_files_row (row)
				end
				
				all_rows.forth
			end
		
		end

	handle_submit_event (submitted_rows: LIST [G])
			-- Handle job submission event
		local
			submit_event: GDS_MERGE_SUBMITTED_EVENT
		do
			create submit_event
			submit_event.set_rows (submitted_rows)
			submit_event.publish
		end

	handle_release_event (released_rows: LIST [G])
			-- Handle job release event
		local
			release_event: GDS_MERGE_RELEASED_EVENT
		do
			create release_event
			release_event.set_rows (released_rows)
			release_event.publish
		end

	handle_changed_event (changed_rows: LIST [G])
			-- Handle job status change event
		local
			changed_event: GDS_MERGE_CHANGED_EVENT
		do
			create changed_event
			changed_event.set_rows (changed_rows)
			changed_event.publish
		end

	handle_all_passed_event
			-- Handle "all rows passed" event
		local
			done_event: GDS_MERGES_DONE_EVENT
		do
			create done_event
			done_event.publish
		end

	handle_failure_event (failed_rows, aborted_rows: LIST [G])
			-- Handle job failure event
		local
			failed_event: GDS_MERGE_FAILED_EVENT
		do
			create failed_event
			failed_event.set_failed_rows (failed_rows)
			failed_event.set_aborted_rows (aborted_rows)
			failed_event.publish
		end

	handle_reset_event (reset_rows: LIST [G])
			-- Handle job reset event
		local
			reset_event: GDS_MERGE_RESET_EVENT
		do
			create reset_event
			reset_event.set_rows (reset_rows)
			reset_event.publish
		end

	gds_merge_rows: LIST [RUN_GDS_FILES_MROW]
			-- List of database rows for GDS merge
			-- jobs for run

	files_table: HASH_TABLE [RUN_GDS_FILES_MROW, INTEGER]
			-- Rows from run GDS files table for this run,
			-- indexed by file number

end
