
indexing
	
	description: "Window to control creation of merged GDS files for each %
		%die on a run"
	author: "David Hollenberg"
	date: "October 9, 2008"

class MERGED_GDS_DIE_CONTROL_WINDOW 

inherit
	MERGED_GDS_CONTROL_WINDOW [GDS_MERGE_DIE_JOB]
		redefine
			fetch_needed_data
		end
		
create
	make

feature {NONE} -- Implementation

	fetch_needed_data
			-- Fetch needed data when first building interface
		do
			precursor
			fetch_fab_instance_list
		end

	add_descendant_visible_columns
			-- Add columns specific to descendant to `visible_columns'
		do
			visible_columns.extend (create {DIE_ID_COLUMN [GDS_MERGE_DIE_JOB]})
			visible_columns.extend (create {DIE_TYPE_COLUMN [GDS_MERGE_DIE_JOB]})
			visible_columns.extend (create {DESIGN_NO_COLUMN [GDS_MERGE_DIE_JOB]})
			visible_columns.extend (create {NAME_COLUMN [GDS_MERGE_DIE_JOB]})
		end

	new_gds_merge_row (job: RUN_GDS_FILES_MROW): GDS_MERGE_DIE_JOB
			-- New row to be added to interface for
			-- `job'
		local
			ifi: IC_FAB_INSTANCES_EROW
			files: ARRAYED_LIST [STRING]
		do
			fab_instances.search (job.die_id)
			if fab_instances.found then
				ifi := fab_instances.found_item
			else
				handle_hard_error ("cannot find fab instance for die ID " + job.die_id + " in internal table")
			end
			create Result.make (job, ifi)
			create files.make (1)
			files.extend (cache.die_merged_gds_layout_file (job).file_pathname)
			Result.set_job_output_files (files)
			Result.set_job_log_file (cache.die_merged_gds_log_file (job).file_pathname)
		end

	row_to_generic (row: EV_MULTI_COLUMN_LIST_ROW): GDS_MERGE_DIE_JOB
			-- Value of `row' which is of type GDS_MERGE_DIE_JOB
		do
			Result ?= row
			check
				result_not_void: Result /= Void
			end
		end

	has_job_status_agent (s: STRING): FUNCTION [GDS_MERGE_DIE_JOB, TUPLE [GDS_MERGE_DIE_JOB], BOOLEAN]
			-- Agent that tells whether row has job status `s'
		do
			Result := agent {GDS_MERGE_DIE_JOB}.has_job_status (s)
		end

	is_stopable_agent: FUNCTION [GDS_MERGE_DIE_JOB, TUPLE [GDS_MERGE_DIE_JOB], BOOLEAN]
			-- Agent that tells whether row has job that
			-- is stopable
		do
			Result := agent {GDS_MERGE_DIE_JOB}.is_stopable
		end

	is_unstopable_agent: FUNCTION [GDS_MERGE_DIE_JOB, TUPLE [GDS_MERGE_DIE_JOB], BOOLEAN]
			-- Agent that tells whether row does not have a
			-- job that is stopable
		do
			Result := agent {GDS_MERGE_DIE_JOB}.is_unstopable
		end

	is_resettable_agent: FUNCTION [GDS_MERGE_DIE_JOB, TUPLE [GDS_MERGE_DIE_JOB], BOOLEAN]
			-- Agent that tells whether row has job that
			-- is resettable
		do
			Result := agent {GDS_MERGE_DIE_JOB}.is_resettable
		end

	is_unresettable_agent: FUNCTION [GDS_MERGE_DIE_JOB, TUPLE [GDS_MERGE_DIE_JOB], BOOLEAN]
			-- Agent that tells whether row does not have
			-- a job that is resettable
		do
			Result := agent {GDS_MERGE_DIE_JOB}.is_unresettable
		end

	is_unsuccessful_agent: FUNCTION [GDS_MERGE_DIE_JOB, TUPLE [GDS_MERGE_DIE_JOB], BOOLEAN]
			-- Agent that tells whether row has job that
			-- was unsuccessful
		do
			Result := agent {GDS_MERGE_DIE_JOB}.is_unsuccessful
		end

	is_submittable_agent: FUNCTION [GDS_MERGE_DIE_JOB, TUPLE [GDS_MERGE_DIE_JOB], BOOLEAN]
			-- Agent that tells whether row has job that
			-- is submittable
		do
			Result := agent {GDS_MERGE_DIE_JOB}.is_submittable
		end

	is_unsubmittable_agent: FUNCTION [GDS_MERGE_DIE_JOB, TUPLE [GDS_MERGE_DIE_JOB], BOOLEAN]
			-- Agent that tells whether row is not in a state
			-- where job can be submitted
		do
			Result := agent {GDS_MERGE_DIE_JOB}.is_unsubmittable
		end

	is_unreleasable_agent: FUNCTION [GDS_MERGE_DIE_JOB, TUPLE [GDS_MERGE_DIE_JOB], BOOLEAN]
			-- Agent that tells whether row is not in a state
			-- where batch job can be released
		do
			Result := agent {GDS_MERGE_DIE_JOB}.is_unreleasable
		end

	is_passed_agent: FUNCTION [GDS_MERGE_DIE_JOB, TUPLE [GDS_MERGE_DIE_JOB], BOOLEAN]
			-- Agent that tells whether row has job that
			-- is passed
		do
			Result := agent {GDS_MERGE_DIE_JOB}.is_passed
		end

end
