class MESSAGE_SUPPORT

inherit
	STRING_UTILITIES_ACCESS		-- su.

feature  -- address extraction modes
	-- the addr_mode determines the treatment of groups in an
	--  address list that is being parsed and listed

	ignore_groups: INTEGER is 1
			-- Ignore the entire group and its mailboxes

	expand_groups: INTEGER is 2
			-- Add all the group mailboxes (the group phrase is
			-- not saved and the mailboxes cannot be distinguished
			-- from those outside the group)

	retain_groups: INTEGER is 3
			-- Retain the entire group clause as a single entry
			-- in the addrs list
	
	save_addr_specs: INTEGER is 4
			-- Save just addr-spec rather than mailbox for each
			-- entry; implies expand_groups too

feature  -- standard header fields

	msg_to:			STRING is "TO"
	msg_cc:			STRING is "CC"
	msg_bcc:		STRING is "BCC"
	msg_from:		STRING is "FROM"
	msg_sender:		STRING is "SENDER"
	msg_reply_to:		STRING is "REPLY-TO"
	msg_resent_to:		STRING is "RESENT-TO"
	msg_resent_cc:		STRING is "RESENT-CC"
	msg_resent_bcc:		STRING is "RESENT-BCC"
	msg_resent_from:	STRING is "RESENT-FROM"
	msg_resent_sender:	STRING is "RESENT-SENDER"
	msg_resent_reply_to:	STRING is "RESENT-REPLY-TO"

	msg_message_id:		STRING is "MESSAGE-ID"
	msg_resent_message_id:	STRING is "RESENT-MESSAGE-ID"
	msg_in_reply_to:	STRING is "IN-REPLY-TO"
	msg_references:		STRING is "REFERENCES"

	msg_date:		STRING is "DATE"
	msg_resent_date:	STRING is "RESENT-DATE"
	msg_subject:		STRING is "SUBJECT"
	msg_keywords:		STRING is "KEYWORDS"
	msg_comments:		STRING is "COMMENTS"

	msg_received:		STRING is "RECEIVED"

feature  -- names of days and months

	days_of_the_week: ARRAY[STRING] is
	once  Result :=
		<<"MON", "TUE", "WED", "THU", "FRI", "SAT", "SUN">>
	end;

	months_of_the_year: ARRAY[STRING] is
	once  Result :=
		<<"JAN", "FEB", "MAR", "APR", "MAY", "JUN",
		  "JUL", "AUG", "SEP", "OCT", "NOV", "DEC">>
	end;

feature -- build a message header line

	header_line(hdr, val: STRING): STRING is
		-- hdr: val <newline>
	do  Result := su.string_of_4
			(capitalized(hdr), su.colon_space, val, su.eol)
	end;

	capitalized(wrd: STRING): STRING is
		-- lower all but the first character
	require not wrd.is_empty;
	do  Result := wrd.as_lower
	    Result.put((wrd @ 1), 1)
	end;
	
feature	-- merge_address_lists

	merge_address_lists(lst1, lst2: STRING): STRING is
		-- returns a new string unless one argument is empty
	do  if su.is_empty(lst2)
	     then Result := lst1
	    elseif su.is_empty(lst1)
	     then Result := lst2
	     else Result := su.string_of_3(lst1, su.comma_space, lst2)
	    end
	end;

end -- MESSAGE_SUPPORT
