class MIME_MULTIPART

inherit
	MIME_PART

create
	make_mixed, make_alt

feature	-- MIME_PART status

	count: INTEGER is
		-- number of parts
	do  Result := my_parts.count
	end;

feature	-- implementation

	copy_body_to_consumer(c: DATA_CONSUMER) is
	do  c.consume("This space intentionally left blank.%N")
	    from  my_parts.start
	    until my_parts.after
	    loop  c.consume("%N--"); c.consume(my_boundary); c.consume(su.eol)
		  my_parts.item.copy_to_consumer(c)
		  my_parts.forth
	    end
	    c.consume("%N--"); c.consume(my_boundary); c.consume("--%N")
	end;

	close is
	do  from  my_parts.start
	    until my_parts.after
	    loop  my_parts.item.close
		  my_parts.forth
	    end
	end

feature	-- add another part

	extend(a_part: MIME_PART) is
		-- add another part to my list
	require exists: a_part /= Void
	do  my_parts.extend(a_part)
	end;

feature	{NONE} -- my details

	my_boundary: STRING

	my_parts: LINKED_LIST[MIME_PART]

feature {NONE} -- creation

	make_mixed is
		-- multipart / mixed
	do  make_common(mpart_is_mixed)
	end;

	make_alt is
		-- multipart / alternative
	do  make_common(mpart_is_alt)
	end;

	make_common(an_mpart: STRING) is
	do  content_type := mtype_is_multipart;
	    content_subtype := an_mpart;
	    my_boundary := su.string_of_3
			("====Eiffel_Mime_MultiPart::",
			 boundary_value_seq.out, "====")
	    add_content_type_addendum(
			key_is_quoted_val("boundary", my_boundary));
	    create my_parts.make
	end;

	boundary_value_seq: INTEGER is
		-- return a new value with every call
	do  Result := boundary_value_cache.item
	    boundary_value_cache.replace(Result + 1)
	end;

	boundary_value_cache: CELL[INTEGER] is
		-- place to keep a global counter
	once  create Result.put(1)
	end;

end -- MIME_MULTIPART
