deferred class MIME_PART
	-- one MIME part of a message
	-- either a composite or a simple part

inherit
	MESSAGE_SUPPORT

	MIME_SUPPORT

	FILABLE

feature	-- type of part

	is_composite: BOOLEAN is
		-- i.e., this part consists of contained parts
	do  Result := equal(content_type, mtype_is_multipart)
	end;

feature	-- MIME type info

	content_type: STRING
		-- the "media type"

	content_subtype: STRING
		-- the "media subtype"

	content_type_addendum: STRING
		-- the part after the "; " on the content_type line

	content_type_value: STRING is
		-- content_type / content_subtype ; content_type_addendum
	local addend: STRING
	do  addend := content_type_addendum
	    if su.not_empty(addend) 
	     then Result := su.string_of_5
				(content_type, "/", content_subtype,
				 su.semi_space, addend)
	     else Result := su.string_of_3
				(content_type, "/", content_subtype)
	    end
	end;

feature	-- values for other (optional) MIME header lines

	content_encoding: STRING
		-- Void/empty implies none

	content_ID: string

	content_description: STRING

	content_disposition: STRING

feature	-- add_content_type_addendum

	add_content_type_addendum(param: STRING) is
		-- add a key=value clause to end of content_type
	do  if su.is_empty(content_type_addendum)
	     then content_type_addendum := param
	    elseif su.not_empty(param)
	     then content_type_addendum := su.string_of_3
			(content_type_addendum, su.semi_space, param)
	    end
	end;

feature	-- encoding selection

	set_no_encoding is
		-- (default)
	do  content_encoding := Void
	end;

	set_quoted_printable_encoding is
	require is_simple: not is_composite
	do  content_encoding := encoding_is_qp
	end;

	set_base64_encoding is
	require is_simple: not is_composite
	do  content_encoding := encoding_is_b64
	end;

feature	-- set other (arbitrary) header fields

	set_content_id(an_id: STRING) is
	do  content_id := an_id
	end;

	set_content_description(a_description: STRING) is
	do  content_description := a_description
	end;

	set_content_disposition(a_disposition: STRING) is
	do  content_disposition := a_disposition
	end;

feature	-- deferred features of FILABLE

	copy_to_consumer(c: DATA_CONSUMER) is
	do  add_nonempty_header_line
		(c, msg_content_type, content_type_value)
	    add_nonempty_header_line
		(c, msg_content_transfer_encoding, content_encoding)
	    add_nonempty_header_line
		(c, msg_content_description, content_description)
	    add_nonempty_header_line
		(c, msg_content_ID, content_ID)
	    add_nonempty_header_line
		(c, msg_content_disposition, content_disposition)
	    c.consume(su.eol);		-- header / body separator
	    copy_body_to_consumer(c)
	end;

  --	close is	(still)
  --	deferred
  --	end;

feature {NONE} -- stuff

	copy_body_to_consumer(c: DATA_CONSUMER) is
	deferred
	end;

	add_nonempty_header_line(c: DATA_CONSUMER; hdr, val: STRING) is
		-- add a (header) line if val is not empty
	do  if su.not_empty(val)
	     then c.consume(header_line(hdr, val))
	    end
	end;

end -- MIME_PART
