
indexing
	
	description: "Object that can manipulate a model and display a view %
		%of it"
	author: "David Hollenberg"
	date: "July 28, 2006"

deferred class MODEL_VIEW_CONTROLLER

feature {NONE} -- Creation

	make (a_model: like model a_view: like view)
			-- Create with `a_model' and `a_view'
		require
			a_model_not_void: a_model /= Void
			a_view_not_void: a_view /= Void
		do
			model := a_model
			view := a_view
		ensure
			model_set: model = a_model
			view_set: view = a_view
		end

feature -- Properties

	model: MODEL
			-- Model that can be manipulated and displayed

	view: MODEL_VIEW
			-- View on which model is displayed

feature -- Operations

	initialize
			-- Initialize controller
		do
			view.initialize
		end

	set_initial_model_elements (list: LIST [like anchored_model_element] with_redraw: BOOLEAN)
			-- Handle setting of initial list of model elements
			-- to `list'.  Redraw model if `with_redraw'
		require
			list_not_void: list /= Void
		deferred
		end

	add_model_elements (list: LIST [like anchored_model_element])
			-- Handle addition of model elements in `list'
		require
			list_not_void: list /= Void
		deferred
		end

	redraw_rectangle (a_x, a_y, a_width, a_height: INTEGER)
			-- Redraw the area of the canvas whose upper left
			-- corner is at (`a_x', `a_y') and whose size
			-- is `a_width' by `a_height'
		require
			a_width_nonnegative: a_width >= 0
			a_height_nonnegative: a_height >= 0
		deferred
		end

	handle_canvas_resize (a_x, a_y, a_width, a_height: INTEGER)
			-- Handle "canvas resized" event, where new upper left
			-- corner is at (`a_x', `a_y') and new size
			-- is `a_width' by `a_height'
		require
			a_width_nonnegative: a_width >= 0
			a_height_nonnegative: a_height >= 0
		deferred
		end

	anchored_model_element: MODEL_ELEMENT
			-- Type of elements in model
		do
		ensure
			result_void: Result = Void
		end

invariant
	model_not_void: model /= Void
	view_not_void: view /= Void
end
