
indexing
	
	description: "Binding of key or pointer button with modifiers %
		%(Ctrl, Alt, Shift) to an operation"
	author: "David Hollenberg"
	date: "July 28, 2006"

class MODIFIED_BINDING

create
	make_with_key, make_with_keys, make_with_button, make_unbound

feature {NONE} -- Creation

	make_unbound (a_category, an_operation: STRING)
			-- Create with no binding
		require
			a_category_not_void: a_category /= void
			an_operation_not_void: an_operation /= void
		do
			make_general (Void, Void, Void, a_category, an_operation)
		end

	make_with_key (a_key: MODIFIED_KEY a_category, an_operation: STRING)
			-- Create with single key `a_key'
		require
			a_key_not_void: a_key /= void
			a_category_not_void: a_category /= void
			an_operation_not_void: an_operation /= void
		do
			make_general (a_key, Void, Void, a_category, an_operation)
		end

	make_with_keys (a_first_key, a_second_key: MODIFIED_KEY a_category, an_operation: STRING)
			-- Create with two key sequence `a_first_key' followed
			-- by `a_second_key'
		require
			a_first_key_not_void: a_first_key /= void
			a_second_key_not_void: a_second_key /= void
			a_category_not_void: a_category /= void
			an_operation_not_void: an_operation /= void
		do
			make_general (a_first_key, a_second_key, Void, a_category, an_operation)
		end

	make_with_button (a_button: MODIFIED_POINTER_BUTTON a_category, an_operation: STRING)
			-- Create with pointer button `a_button'
		require
			a_button_not_void: a_button /= void
			a_category_not_void: a_category /= void
			an_operation_not_void: an_operation /= void
		do
			make_general (Void, Void, a_button, a_category, an_operation)
		end

	make_general (a_first_key, a_second_key: MODIFIED_KEY a_button: MODIFIED_POINTER_BUTTON a_category, an_operation: STRING)
		do
			first_key := a_first_key
			second_key := a_second_key
			pointer_button := a_button
			category := a_category
			operation := an_operation
		end

feature -- Properties

	number: INTEGER
			-- Number of this binding, for unique identification

	first_key: MODIFIED_KEY
			-- First key in sequence to which operation is bound,
			-- if not Void

	second_key: MODIFIED_KEY
			-- Second key in sequence to which operation is bound,
			-- if not Void

	pointer_button: MODIFIED_POINTER_BUTTON
			-- Pointer button to which operation is bound,
			-- if not Void

	type: STRING
			-- Type of binding for operation
		do
			
			if not is_bound then
				Result := None_binding_type
			elseif is_key_binding then
				Result := Key_binding_type
			elseif is_button_binding then
				Result := Button_binding_type
			else
				Result := Unknown_binding_type
			end
		end

	category: STRING
			-- Category of operation, for classifying related
			-- operations together

	operation: STRING
			-- Name of operation bound to either`first_key' or
			-- to`first_key' followed by `second_key' or to
			-- `pointer_button'

	is_bound: BOOLEAN
			-- Is operation bound to anything?
		do
			Result := first_key /= Void or pointer_button /= Void
		end

	is_key_binding: BOOLEAN
			-- Is binding a key binding?
		require
			is_bound: is_bound
		do
			Result := first_key /= Void
		end

	is_button_binding: BOOLEAN
			-- Is binding a pointer button binding?
		require
			is_bound: is_bound
		do
			Result := pointer_button /= Void
		end

	description: STRING
			-- Description of key or pointer binding
		do
			if pointer_button /= Void then
				Result := pointer_button.description
			elseif first_key = Void and second_key = Void then
				Result := Unbound
			elseif second_key = Void then
				Result := first_key.description
			else
				Result := first_key.description + " " + second_key.description
			end
		end

feature -- Modification

	set_number (a_value: like number)
			-- Set `number' to `a_value'
		do
			number := a_value
		ensure
			number_set: number = a_value
		end

feature {NONE} -- Implementation

	Key_binding_type: STRING = "KEY"

	Button_binding_type: STRING = "BUTTON"

	None_binding_type: STRING = "NONE"

	Unknown_binding_type: STRING = "UNKNOWN"

	Unbound: STRING = ""
			-- Description of binding when operation not bound
			-- to anything

invariant
	second_key_valid: second_key /= Void implies first_key /= Void
	pointer_button_valid: first_key /= Void implies pointer_button = Void
	first_key_valid: pointer_button /= Void implies first_key = Void
	
end
