
indexing
	
	description: "EV_KEY plus state of modifiers (Ctrl, Alt, Shift)"
	author: "David Hollenberg"
	date: "July 28, 2006"

class MODIFIED_KEY
inherit
	EV_KEY
	EV_KEY_CONSTANTS
		export
			{NONE} all
		undefine
			default_create, out
		end
	MODIFIER_KEY_STATE
		undefine
			default_create, out
		redefine
			description
		end

create
	default_create, make_with_code, make_with_code_and_application,
	make_unmodified, make_control, make_alt, make_shift

feature {NONE} -- Creation

	make_with_code_and_application (a_code: INTEGER an_application: EV_APPLICATION)
			-- Create with `a_code', with Ctrl, Alt and Shift
			-- modifiers determined from `an_application'
		require
			a_code_valid: valid_key_code (a_code)
			an_application_not_void: an_application /= Void
		do
			make_with_code (a_code)
			set_modifiers (an_application)
		end

	make_unmodified (a_code: INTEGER)
			-- Create with no modifiers from `a_code'
		require
			a_code_valid: valid_key_code (a_code)
		do
			make_with_code (a_code)
			is_alt_pressed := False
			is_ctrl_pressed := False
			is_shift_pressed := False
		end

	make_alt (a_code: INTEGER)
			-- Create with Alt modifier and `a_code'
		require
			a_code_valid: valid_key_code (a_code)
		do
			make_with_code (a_code)
			is_alt_pressed := True
			is_ctrl_pressed := False
			is_shift_pressed := False
		end

	make_control (a_code: INTEGER)
			-- Create with Ctrl modifier and `a_code'
		require
			a_code_valid: valid_key_code (a_code)
		do
			make_with_code (a_code)
			is_alt_pressed := False
			is_ctrl_pressed := True
			is_shift_pressed := False
		end

	make_shift (a_code: INTEGER)
			-- Create with Shift modifier and `a_code'
		require
			a_code_valid: valid_key_code (a_code)
		do
			make_with_code (a_code)
			is_alt_pressed := False
			is_ctrl_pressed := False
			is_shift_pressed := True
		end

feature -- Properties

	is_unmodified_escape: BOOLEAN
			-- Is this the Escape key with no modifier keys?
		do
			Result := code = Key_escape and modifier_count = 0
		end

	is_caps_lock: BOOLEAN
			-- Is this the Caps Lock key?
		do
			Result := code = Key_caps_lock
		end

	is_modifier: BOOLEAN
			-- Is this a modifier key?
		do
			inspect code
			when Key_Alt, Key_Ctrl, Key_shift then
				Result := True
			else
				Result := False
			end
		end

	description: STRING
			-- Description of key with modifiers
		do
			create Result.make (30)
			inspect code
			when Key_Alt, Key_Ctrl, Key_shift then
			else
				Result.append (precursor)
			end
			Result.append (unmodified_description_as_upper)
		end

	quoted_description: STRING
			-- Description of key with modifiers, in quotes
		do
			create Result.make (30)
			Result.extend ('%"')
			Result.append (description)
			Result.extend ('%"')
		end

	unmodified_description: STRING
			-- Description of key without modifiers
		do
			Result := unmodified_description_as_upper
		end

	quoted_unmodified_description: STRING
			-- Description of key without modifiers, in quotes
		do
			create Result.make (17)
			Result.extend ('%"')
			Result.append (unmodified_description)
			Result.extend ('%"')
		end

feature {NONE} -- Implementation

	unmodified_description_as_upper: STRING
			-- Description of key without modifiers, with
			-- lowercase alphabetic characters converted to
			-- uppercase
		do
			if is_alpha then
				Result := out.as_upper
			else
				Result := out
			end
		end

end
