
indexing
	
	description: "Pointer button number with modifiers (Ctrl, Alt, Shift)"
	author: "David Hollenberg"
	date: "July 28, 2006"

class MODIFIED_POINTER_BUTTON
inherit
	MODIFIER_KEY_STATE
		redefine
			description, out
		end

create
	default_create, make_with_number_and_application, make_unmodified,
	make_control, make_alt, make_shift, make_control_shift,
	make_control_alt, make_alt_shift, make_control_alt_shift

feature {NONE} -- Creation

	make_with_number_and_application (a_button_number: INTEGER an_application: EV_APPLICATION)
			-- Create with `a_button_number', with Ctrl, Alt
			-- and Shift modifiers determined from
			-- `an_application'
		require
			an_application_not_void: an_application /= Void
		do
			set_number (a_button_number)
			set_modifiers (an_application)
		end

	make_unmodified (a_button_number: INTEGER)
			-- Create with no modifiers from `a_button_number'
		do
			set_number (a_button_number)
			is_alt_pressed := False
			is_ctrl_pressed := False
			is_shift_pressed := False
		end

	make_alt (a_button_number: INTEGER)
			-- Create with Alt modifier and `a_button_number'
		do
			set_number (a_button_number)
			is_alt_pressed := True
			is_ctrl_pressed := False
			is_shift_pressed := False
		end

	make_control (a_button_number: INTEGER)
			-- Create with Ctrl modifier and `a_button_number'
		do
			set_number (a_button_number)
			is_alt_pressed := False
			is_ctrl_pressed := True
			is_shift_pressed := False
		end

	make_shift (a_button_number: INTEGER)
			-- Create with Shift modifier and `a_button_number'
		do
			set_number (a_button_number)
			is_alt_pressed := False
			is_ctrl_pressed := False
			is_shift_pressed := True
		end

	make_control_shift (a_button_number: INTEGER)
			-- Create with Control and Shift modifiers and
			-- `a_button_number'
		do
			set_number (a_button_number)
			is_alt_pressed := False
			is_ctrl_pressed := True
			is_shift_pressed := True
		end

	make_control_alt (a_button_number: INTEGER)
			-- Create with Control and Alt modifiers and
			-- `a_button_number'
		do
			set_number (a_button_number)
			is_alt_pressed := True
			is_ctrl_pressed := True
			is_shift_pressed := False
		end

	make_alt_shift (a_button_number: INTEGER)
			-- Create with Alt and Shift modifiers and
			-- `a_button_number'
		do
			set_number (a_button_number)
			is_alt_pressed := True
			is_ctrl_pressed := False
			is_shift_pressed := True
		end

	make_control_alt_shift (a_button_number: INTEGER)
			-- Create with Control, Alt and Shift modifiers and
			-- `a_button_number'
		do
			set_number (a_button_number)
			is_alt_pressed := True
			is_ctrl_pressed := True
			is_shift_pressed := True
		end

feature -- Properties

	description: STRING
			-- Description of button with modifiers
		do
			create Result.make (30)
			Result.append (precursor)
			Result.append (unmodified_description)
		end

	unmodified_description: STRING
			-- Description of button without modifiers
		do
			inspect number
			when 1 then
				Result := "Left Button"
			when 2 then
				Result := "Middle Button"
			when 3 then
				Result := "Right Button"
			else
				Result := "Button_" + number.out
			end
		end

	out: STRING
			-- Description of button without modifiers
		do
			Result := description
		end

	number: INTEGER
			-- Number of pointer button

feature -- Modification
	
	set_number (a_button_number: INTEGER)
		do
			number := a_button_number
		ensure
			number_set: number = a_button_number
		end

end
