
indexing
	
	description: "State of modifier keys (Ctrl, Alt, Shift)"
	author: "David Hollenberg"
	date: "July 28, 2006"

class MODIFIER_KEY_STATE
inherit
	MODIFIER_KEY_CONSTANTS
		rename
			modifier_count as std_modifier_count,
			modifier_code as std_modifier_code,
			description as std_description
		end

feature -- Properties

	is_alt_pressed: BOOLEAN
			-- Was Alt key pressed?

	is_ctrl_pressed: BOOLEAN
			-- Was Ctrl key pressed?

	is_shift_pressed: BOOLEAN
			-- Was Shift key pressed?

	is_modified: BOOLEAN
			-- Were any modifier keys (Ctrl, Alt, Shift) pressed?
		do
			Result := is_alt_pressed or is_ctrl_pressed or is_shift_pressed
		end

	modifier_count: INTEGER
			-- Number of modifier keys pressed
		do
			Result := std_modifier_count (is_alt_pressed, is_ctrl_pressed, is_shift_pressed)
		end

	modifier_code: INTEGER
			-- Code encoding which modifier keys are pressed
		do
			Result := std_modifier_code (is_alt_pressed, is_ctrl_pressed, is_shift_pressed)
		end

	description: STRING
			-- Description of modifier state
		do
			Result := std_description (is_alt_pressed, is_ctrl_pressed, is_shift_pressed)
		end

feature -- Modification

	set_modifiers (app: EV_APPLICATION)
			-- Set modifiers from current values returned
			-- by `app'
		require
			app_not_void: app /= Void
		do
			is_alt_pressed := app.alt_pressed
			is_ctrl_pressed := app.ctrl_pressed
			is_shift_pressed := app.shift_pressed
		end

end
