
indexing
	
	description: "Object that knows how to produce text for a set of row %
		%and set of columns of a multi-column list"
	author: "David Hollenberg";
	date: "March 30, 2007"

class MULTI_COLUMN_LIST_TEXT_FORMATTER [G -> MULTI_COLUMN_LIST_ROW]
inherit
	ANY
	STRING_UTILITIES_ACCESS
		export
			{NONE} all
		end

feature -- Operations

	formatted_text (a_row_list: LIST [G] a_column_list: LIST [MULTI_COLUMN_LIST_COLUMN [G]] a_separator: STRING pad, include_titles: BOOLEAN): STRING is
			-- Text corresponding to `a_row_list' with only
			-- columns of `a_column_list' included.  Values
			-- in a row are separated by `a_separator' in the
			-- output.  If `pad', then each item is padded
			-- to match the length of the longest item in
			-- its column, with padding applied on left, right
			-- or both sides, depending on column alignment.
			-- If `include_titles', then column titles are
			-- included in the output before the first row
		local
			col: MULTI_COLUMN_LIST_COLUMN [G]
			row: G
			k, n, len, col_count: INTEGER
			val: STRING
		do
			create padded_temp.make (100)
			empty := ""
			create max_lengths.make (1, a_column_list.count)
			from
				n := 1
			until
				n > a_column_list.count
			loop
				max_lengths.put (0, n)
				n := n + 1
			end

			from
				a_row_list.start
			until
				a_row_list.after
			loop
				row := a_row_list.item
				from
					k := 1
					a_column_list.start
				until
					a_column_list.after
				loop
					col := a_column_list.item
					val := col.value (row)
					len := len + val.count
					if val.count > max_lengths.item (k) then
						max_lengths.put (val.count, k)
					end
					k := k + 1
					a_column_list.forth
				end
				a_row_list.forth
			end
			
			if include_titles then
				from
					k := 1
					a_column_list.start
				until
					a_column_list.after
				loop
					col := a_column_list.item
					val := col.title
					len := len + val.count
					if val.count > max_lengths.item (k) then
						max_lengths.put (val.count, k)
					end
					k := k + 1
					a_column_list.forth
				end
			end
				
			-- Now create formatted text

			create Result.make (len)
			Result.extend ('%N')
			col_count := a_column_list.count
			
			if include_titles then
				-- Append titles
				from
					k := 1
					a_column_list.start
				until
					a_column_list.after
				loop
					col := a_column_list.item
					val := col.title
					if pad then
						Result.append (padded_value (val, k, col.alignment))
					else
						Result.append (val)
					end
					if k < col_count then
						Result.append (a_separator)
					end
					k := k + 1
					a_column_list.forth
				end
				Result.right_adjust
				Result.extend ('%N')
			end
				

			from
				a_row_list.start
			until
				a_row_list.after
			loop
				row := a_row_list.item
				from
					k := 1
					a_column_list.start
				until
					a_column_list.after
				loop
					col := a_column_list.item
					val := col.value (row)
					if pad then
						Result.append (padded_value (val, k, col.alignment))
					else
						Result.append (val)
					end
					if k < col_count then
						Result.append (a_separator)
					end
					k := k + 1
					a_column_list.forth
				end
				Result.right_adjust
				Result.extend ('%N')
				a_row_list.forth
			end
			
		end

feature {NONE} -- Implementation

	padded_value (val: STRING pos: INTEGER alignment: EV_TEXT_ALIGNMENT): STRING is
		local
			pad_len: INTEGER
		do
			pad_len := max_lengths.item (pos)
			Result := padded_temp
			if alignment.is_left_aligned then
				Result.copy (val)
				su.right_pad (Result, pad_len)
			elseif alignment.is_right_aligned then
				Result.copy (empty)
				su.right_pad (Result, pad_len - val.count)
				Result.append (val)
			else -- Center aligned
				Result.copy (empty)
				su.right_pad (Result, pad_len - val.count // 2)
				Result.append (val)
				su.right_pad (Result, pad_len)
			end
		end

	max_lengths: ARRAY [INTEGER]
			-- Length of longest value in each column

	padded_temp: STRING
			-- Temporary for padding items

	empty: STRING
			-- Temporary for padding items

end
