class NFE_CONFIGURATION_VARS
	-- lookup of CONFIGURATION vars that control NFE execution.

inherit
	DICTIONARY

	STRING_UTILITIES_ACCESS

	FILE_UTILITIES_ACCESS

create
	make

feature -- explicit parameter values (see also NFE_PROCESSOR_MODE)

	nfe_production: BOOLEAN is
		-- are we running a "production" system?
	once  Result := as_boolean(item("PRODUCTION"))
	end;

	nfe_archival_dir: STRING is
		-- directory for communication archives
	once  Result := item("FE_ARCH_DIR");
	end;

	nfe_max_storage_segment_length: INTEGER is
		-- max storage_segment_length for stored hunks of text
	once  Result := as_integer(item("MAX_STORAGE_SEGMENT_LENGTH"));
	      if Result <= 0 then Result := 2000000000
	      end
	end;

	nfe_crash_dir: STRING is
		-- directory for crash messge files
	once  Result := item("FE_CRASH_DIR");
	end;

	auto_drc_dir: STRING is
		-- working directory for running the drc_requests
	once  Result := item("AUTO_DRC_DIR")
	end;

	nfe_db_retry_limit: INTEGER is
		-- transaction retries (for db_retryable_errors) before
		--  we give up
	once Result := as_integer(item("DB_RETRY_COUNT"))
	end;

	nfe_db_optimizer_mode: STRING is
		-- optimizer mode (empty means dont set it)
	once Result := item("DB_OPTIMIZER_MODE")
	end;

	nfe_default_net_address: STRING is
		-- net_address value to use when forgotten
	once  Result := item("FE_DEFAULT_NET_ADDRESS")
	end;

	in_mail_temp_dir: STRING is
		-- directory for request messages
	once  Result := item("IN_MAIL_TEMP_DIR")
	end;

	in_mail_curr_dir: STRING is
		-- directory for request messages
	once  Result := item("IN_MAIL_CURR_DIR")
	end;

	in_mail_done_dir: STRING is
		-- directory for request messages
	once  Result := item("IN_MAIL_DONE_DIR")
	end;

	nfe_mailer_command: STRING is
		-- command to run the mailer for outgoing messages
		-- (only when nfe_reply_dir is NONE)
	once  Result := item("FE_MAILER_COMMAND")
	end;

	nfe_minfo_path: STRING is
		-- info search path
	once  Result := item("FE_MINFO_PATH");
	end;

	nfe_proc_file: STRING is
		-- actual executable for the front end
	once  Result := item("FE_PROC_FILE");
	end;

	nfe_operating_mode: STRING is
		-- operating mode: MAIL, FTP, DC
	once Result := item("FE_PROC_TYPE");
	     if Result = Void
		then Result := ""
		else Result.to_upper
	     end
	end;

	nfe_sender_file: STRING is
		-- actual executable for the message sender
	once  Result := item("SENDER_FILE");
	end;

	nfe_pad_extractor_file: STRING is
		-- actual executable for the pad_extractor
	once  Result := item("PAD_EXTRACTOR_FILE");
	end;

	design_check_processor_file: STRING is
		-- executable for the design_check_processor
		-- (Void/empty if we are still using the (ICL) dc server)
	once  Result := item("DESIGN_CHECK_PROCESSOR_FILE")
	end;

	ghostscript_program: STRING is
	once Result := item("GHOSTSCRIPT_PROGRAM")
	end;

	out_mail_temp_dir: STRING is
		-- directory for reply messages
	once  Result := item("OUT_MAIL_TEMP_DIR")
	end;

	out_mail_curr_dir: STRING is
		-- directory for reply messages
	once  Result := item("OUT_MAIL_CURR_DIR")
	end;

	out_mail_done_dir: STRING is
		-- directory for reply messages
	once  Result := item("OUT_MAIL_DONE_DIR")
	end;

	nfe_proj_dir: STRING is
		-- project directory to store layout files
	once  Result := item("FE_PROJ_DIR");
	end;

	nfe_del_proj_dir: STRING is
		-- directory to store "deleted" layout files
	once  Result := item("FE_DEL_PROJ_DIR");
	end;

	nfe_punt_to_list: STRING is
		-- to: list for maintainers
        once  Result := item("FE_PUNT_TO_LIST")
	end;

	nfe_black_hole_to_list: STRING is
		-- to: list for various black holes
	once  Result := item("FE_BLACK_HOLE_TO_LIST")
	      if su.is_empty(Result)
	       then Result := nfe_punt_to_list
	      end
	end;

	nfe_from_line: STRING is
		-- from: line contents for external messages
	once  Result := item("FE_FROM_LINE")
	end;

	nfe_reply_cc_list: STRING is
		-- additional cc list for all request replies
	once  Result := item("FE_REPLY_CC")
	end;

	nfe_reply_sig: STRING is
		-- .sig file CONTENTS for all request replies
	once  Result := nfe_scarf_file("FE_SIG_FILE");
	end;

	nfe_no_req_text: STRING is
		-- text for no request found
	once  Result := nfe_scarf_file("FE_NO_REQ_FOUND_FILE")
	end;

	nfe_report_to_list(acct_type: STRING): STRING is
		-- to: list for project reports
        do  Result := item(su.string_of_3("FE_REPORT_",acct_type,"_TO_LIST"))
	    if su.is_empty(Result)
	     then Result := item("FE_REPORT_OTHER_TO_LIST")
	    end;
	end;

	nfe_eccn_questionnaire_notify_to_list: STRING is
		-- to: list for nfe questionnaires
	once  Result := item("FE_ECCQ_TO_LIST")
	end;

	nfe_special_handling_to_list: STRING is
		-- to: list for maintainers
        once  Result := item("FE_SPH_TO_LIST")
	end;

	nfe_unassigned_quote_to_list: STRING is
		-- to: list for handlers of unassigned quotes
	once  Result := item("FE_UNASSIGNED_QUOTE_TO_LIST")
	end;

	nfe_unkpo_to_list: STRING is
		-- to: list for the_funding_alloc_from_hell charged
        once  Result := item("FE_UNKPO_TO_LIST")
	end;

	nfe_missing_po_data_to_list: STRING is
		-- to: list for missing data
	once  Result := item("FE_MISSING_PO_DATA_TO_LIST")
	end;

	nfe_missing_acct_data_to_list: STRING is
		-- to: list for missing data
	once  Result := item("FE_MISSING_ACCT_DATA_TO_LIST")
	end;

	nfe_exp_acct_to_list: STRING is
		-- to: list for COM project in fab queue
	once  Result := item("FE_EXP_ACCT_TO_LIST")
	end;

	nfe_com_in_que_to_list: STRING is
		-- to: list for COM project in fab queue
	once  Result := item("FE_COM_IN_QUE_TO_LIST")
	end;

	nfe_nsa_in_que_to_list: STRING is
		-- to: list for NSA project in fab queue
	once  Result := item("FE_NSA_IN_QUE_TO_LIST")
	end;

	nfe_mepres_in_que_to_list: STRING is
		-- to: list for MEP RES project in fab queue
	once  Result := item("FE_MEPRES_IN_QUE_TO_LIST")
	end;

	nfe_oh_in_que_to_list: STRING is
		-- to: list for OH project in fab queue
	once  Result := item("FE_OH_IN_QUE_TO_LIST")
	end

	nfe_coupon_in_que_to_list: STRING is
		-- to: list for couponed (CyberShuttle) projects
	once  Result := item("FE_COUPON_IN_QUE_TO_LIST")
	end;

	nfe_design_file_offline_to_list: STRING is
		-- to: list for restoration of offline files
	once  Result := item("FE_DESIGN_FILE_OFFLINE_TO_LIST")
	end;

	nfe_bond_verify_to_list: STRING is
		-- to: list for bonding problems
	once  Result := item("FE_BOND_VERIFY_TO_LIST")
	      if su.is_empty(Result)
	       then Result := nfe_special_handling_to_list
	      end
	end;

	nfe_wafer_files_to_list: STRING is
		-- to: list for waking up the probers to a run closing
	once  Result := item("FE_WAFER_FILES_TO_LIST")
	end;

	nfe_mask_invoice_to_list: STRING is
		-- to: list for invoicing customer for ded run mask cost
	once  Result := item("FE_MASK_INVOICE_TO_LIST");
	end;

	nfe_payload_and_charges_to_list: STRING is
		-- to: list of payload and charges recipients from post
	once  Result := item("FE_PAYLOAD_AND_CHARGES_TO_LIST");
	end;

	nfe_layer_changes_to_list: STRING
		-- to: Layer Changes List
	once  Result := item("FE_LAYER_CHANGES_TO_LIST")
	end;

	nfe_nobody_to_list: STRING is
		-- to: no one (i.e., just log for viewing)
	once  Result := item("FE_NOBODY_TO_LIST");
	end;

	nfe_respond_with_ids: BOOLEAN is
		-- should we include "ID: " in our response strings?
		-- (default is false)
	once  Result := as_boolean(item("FE_ID_MODE"));
	end;

	nfe_is_logging: BOOLEAN is
		-- are we logging to standard output?
	once  Result := as_boolean(item("FE_LOGGING"))
	end;

	nfe_new_file_perm: STRING is
		-- permission string for created files
	once  Result := item("FE_NEW_FILE_PERM")
	end;

	nfe_new_dir_perm: STRING is
		-- permission string for created directories
	once  Result := item("FE_NEW_DIR_PERM")
	end;

	param_nfe_host_name: STRING is "FE_HOST_NAME";

	nfe_host_name: STRING is
		-- target host for other FE processes
	once  Result := item(param_nfe_host_name)
	end;

	nfe_remote_prog_local: STRING is
		-- name of local program to create remote processes
	once  Result := item("FE_REMOTE_PROG_LOCAL")
	end;

	nfe_remote_prog_remote: STRING is
		-- name of program (on FE_HOST) that creates processes
	once  Result := item("FE_REMOTE_PROG_REMOTE")
	end;

	nfe_scarf_file(key: STRING): STRING is
		-- item value is a file name; return its contents
	local nam: STRING;  fil: LARGE_PLAIN_TEXT_FILE
	do  nam := item(key);
	    if su.not_empty(nam)
		then fil := fu.opened_for_read(nam)
	    end;
	    if fil = Void
		then create Result.make(0)
		else fil.read_stream(8192);
		     Result := fil.last_string;
		     fil.close
	    end;
	ensure Result /= Void
	end;

	ftp_mgr_lock_dir: STRING is
		-- Directory with ftp retrieval manager lock files
	once  Result := item("FTP_MGR_LOCK_DIR");
	end;

	ftp_mgr_lock_template: STRING is
		-- Template for ftp retrieval manager lock files
		-- (identifies which files in directory are his lock files)
	once  Result := item("FTP_MGR_LOCK_TEMPLATE");
	end;

	ftp_srv_lock_dir: STRING is
		-- Directory with ftp server lock files
	once  Result := item("FTP_SRV_LOCK_DIR");
	end;

	ftp_srv_lock_template: STRING is
		-- Template for ftp server lock files
		-- (identifies which files in directory are his lock files)
	once  Result := item("FTP_SRV_LOCK_TEMPLATE");
	end;

	ftp_srv_seq_file: STRING is
		-- Name of ftp server sequence file.  Sequences are used
		-- to uniquely identify each ftp session
	once  Result := item("FTP_SRV_SEQ_FILE");
	end;

	ftp_srv_event_file: STRING is
		-- Name of ftp server significant events file
	once  Result := item("FTP_SRV_EVENT_FILE");
	end;

	ftp_srv_block_file: STRING is
		-- Name of ftp server file with list of
		-- blocked IP addresses
	once  Result := item("FTP_SRV_BLOCK_FILE");
	end;

	ftp_srv_session_dir: STRING is
		-- Directory for ftp server log files for individual
		-- ftp sessions
	once  Result := item("FTP_SRV_SESSION_DIR");
	end;

	bind_priv_sock_program: STRING is
		-- Name of program that can be run to bind a privileged
		-- socket
	once  Result := item("BIND_PRIV_SOCK_PROGRAM");
	end;

	source_location_dir: STRING is
		-- mandatory directory for SOURCE_LOCATION: files
	once  Result := item("SOURCE_LOCATION_DIR");
	end;

	new_design_check_time_limit: INTEGER is
		-- time limit on dc daemon for NEW design_checks
	once  Result := as_integer(item("NEW_DESIGN_CHECK_TIME_LIMIT"));
	end;

	pending_design_check_time_limit: INTEGER is
		-- time limit on dc daemon for PENDING design_checks
	once  Result := as_integer(item("PENDING_DESIGN_CHECK_TIME_LIMIT"));
	end;

	retry_design_check_time_limit: INTEGER is
		-- time limit on dc daemon for RETRY design_checks
	once  Result := as_integer(item("RETRY_DESIGN_CHECK_TIME_LIMIT"));
	end;

	nfe_is_automatic_mode: BOOLEAN is
		-- are we in automatic mode?
	once  Result := as_boolean(item("AUTOMATIC"))
	end;

	oracle_reports_home: STRING is
		-- orahome for running Oracle reports
	once Result := item("ORACLE_REPORTS_HOME")
	end;

	report_exe_file: STRING is
		-- orahome for running Oracle reports
	once Result := item("REPORT_EXE_FILE")
	end;

	report_display: STRING is
		-- orahome for running Oracle reports
	once Result := item("REPORT_DISPLAY")
	end;

	report_temp_dir: STRING is
		-- orahome for running Oracle reports
	once Result := item("REPORT_TEMP_DIR")
	end;

feature	-- pass-thru to the real HASHED_CONFIGURATION_VARS

	has(key: STRING): BOOLEAN is
	do  Result := vars.has(key)
	end;

	item(key: STRING): STRING is
	do  Result := vars.item(key)
	end;

	original_args: ARRAY[STRING] is
	do  Result := vars.original_args
	end;

feature {NONE} -- creation

	vars: HASHED_CONFIGURATION_VARS

	make(a_var: HASHED_CONFIGURATION_VARS) is
	do  vars := a_var
	end;

end  -- NFE_CONFIGURATION_VARS
