class NFE_GLOBALS
		-- global stuff that is accessed thru glo.xxxx

inherit

	NFE_STRINGS

create

	make

feature	-- the current incoming_communication
	--  and the current request number (within that i_comm)

	i_comm: INCOMING_COMMUNICATIONS_ROW
	request_no: INTEGER
		-- sometimes assigned by the request processor;
		-- sometimes calculated by the request itself

	set_i_comm(ic: like i_comm) is
	do  i_comm := ic
	end;

	set_request_no(rno: INTEGER) is
	do  request_no := rno
	end;

	-- and some stuff about the pair, FWIW

	m_communication_no: INTEGER is
	require i_comm /= Void
	do  Result := i_comm.m_communication_no
	end;

	request_handle: STRING is
		-- a printable handle for this request
	do  Result := m_request.my_handle
	end;

	last_reply_handle: STRING is
		-- printable handle of last reply to this request
	do  Result := m_request.the_replies.last.my_handle
	end;

	m_request: M_REQUESTS_ROW is
		-- the request_no request in i_comm
		-- (create on demand)
	require have_i_comm: i_comm /= Void;
		valid_i_comm: not i_comm.is_corrupted;
		req_no_OK: 1 <= request_no and 
				request_no <= i_comm.next_request_no
	do  if request_no = i_comm.next_request_no
	     then	-- create on demand
		Result := i_comm.make_next_request
		Result.set_request_type(the_request_name);
	     else
		Result := i_comm.the_requests.i_th(request_no);
	    end
	ensure  Result.request_no = request_no
	end;

feature	-- the name of the current request (MOSIS_REQUEST)
	-- (NOTE: not necessarily the same as the m_request.request_type)

	the_request_name: STRING

	set_the_request_name(nam: STRING) is
	do  the_request_name := nam
	end;

feature -- the_new_design_no

	the_new_design_no: INTEGER
			-- for use by id: * in later requests

	set_the_new_design_no(dno: INTEGER) is
			-- set by last successful new_project request
	do  the_new_design_no := dno
	end;

feature	-- has_restricted_access

	has_restricted_access: BOOLEAN
		-- does current user have restricted access?
		-- (determined by the nfe_request_processor)

feature {NFE_REQUEST_PROCESSOR} -- set_has_restricted_access

	set_has_restricted_access(hra: BOOLEAN) is
	do
	    has_restricted_access := hra
	ensure has_restricted_access = hra
	end;

feature	-- a commit action and list that is nfe-specific in implementation

	commit_proc_actions: ARRAYED_LIST[NFE_PROCESS_CREATION_ACTION];

	nfe_commit is
	do  if commit_proc_actions /= Void
	     then from  commit_proc_actions.start
		  until commit_proc_actions.after
		  loop  extend_commit_actions(commit_proc_actions.item)
			commit_proc_actions.forth
		  end;
		  commit_proc_actions.wipe_out
	    end;
	    glo.commit
	end;

	nfe_rollback is
	do  if i_comm /= Void
	     then -- fetch anew
		  i_comm.wipe_out_requests
	    end
	    glo.rollback
	end;

	reset_actions is
	do  glo.reset_actions
	    if commit_proc_actions /= Void
	     then commit_proc_actions.wipe_out
	    end;
	end;

	fork_on_commit(mode: STRING): NFE_PROCESS_CREATION_ACTION is
		-- start up a FE process of type mode (but just once)
		-- these guys use a separate list so we can avoid repetition
		-- Returns the new NFE_PROCESS_CREATION_ACTION (may be Void)
	local  found: BOOLEAN; a: NFE_PROCESS_CREATION_ACTION;
	do  if commit_proc_actions = Void
	     then create commit_proc_actions.make(4)
	    end;
	    -- see if we already have a (lockable) action of this mode
	    from  commit_proc_actions.start
	    until commit_proc_actions.after or found
	    loop  a := commit_proc_actions.item;
		  found := equal(a.proc_mode, mode) and
			   a.proc_lock /= Void;
		  commit_proc_actions.forth
	    end;
	    -- build one if dont have one
	    if not found
	     then create Result.make(mode);
		  commit_proc_actions.extend(Result)
	    end;
	end;

feature	-- pass-thru routines to GENERIC_SYSTEM_GLOBALS

	log_line_2(s1, s2: STRING) is
	do  glo.log_line_2(s1, s2)
	end;

	log_pid_line(v, n: STRING; pid: INTEGER) is
	do  glo.log_pid_line(v, n, pid)
	end;

	extend_commit_actions(itm: ACTION_ITEM) is
	do  glo.extend_commit_actions(itm)
	end;

	delete_on_rollback(fn: STRING) is
	do  glo.delete_on_rollback(fn)
	end;

	delete_on_commit(fn: STRING) is
	do  glo.delete_on_commit(fn)
	end;

	rename_on_commit(ofn, nfn: STRING) is
	do  glo.rename_on_commit(ofn, nfn)
	end;

	set_is_logging(b: BOOLEAN) is
	do  glo.set_is_logging(b)
	end;

	set_system_internal_error(s: STRING) is
	do  glo.set_system_internal_error(s)
	end;

feature {NONE} -- creation

	glo: GENERIC_SYSTEM_GLOBALS

	make(a_glo: GENERIC_SYSTEM_GLOBALS) is
	do
	    glo := a_glo
	end;

end -- NFE_GLOBALS
