class NFE_M_REPLIES_MROW
		-- one M_REPLIES row,
		-- with a lot of front end baggage

inherit

	M_REPLIES_MROW
		undefine
			is_equal
		end

	STORED_HUNK_OF_TEXT

	NFE_SUPPORT
		export
			{NONE} all
		undefine
			is_equal
		end

	MESSAGE_SUPPORT
		export
			{NONE} all
		undefine
			is_equal
		end

	OS_ACCESS
		export
			{NONE} all
		undefine
			is_equal
		end

	NFE_PROCESSOR_MODE
		export
			{NONE} all
		undefine
			is_equal
		end

create

	make_template, make_null, make_by_key

feature -- store_me_away from a string

	set_archive_from_string(t: STRING) is
		-- make contents of t be the archived body of this reply
		-- (NOTE: send, below, directly archives the_msg)
	local  foo: DATA_SOURCE_STRING
	do  if su.not_empty(t)
	     then create foo.make(t);
		  store_me_away(foo)
	    end
	end;

feature	-- support for an associated e-mail message

	the_msg: OUTGOING_MESSAGE
		-- the message associated with this reply

	archive_the_message is
		-- archive from the_msg (but kill the attachment(s))
	require the_msg /= Void
	do  if the_msg.MIME_message_body = Void
	     then store_me_away(the_msg)
	     else set_archive_from_string(su.string_of_3
				(the_msg.message_body, su.eol_eol,
				 "<<MIME attachments not archived>>"))
	    end
	end;

	make_msg is
		-- we just make it; filling it in is up to you
	require the_msg = Void
	do  create the_msg.make_alt_mailer(vars.nfe_mailer_command)
	ensure the_msg /= Void
	end;

	send_msg_now is
		-- add reply_id, record header info, archive, and send
	require the_msg /= Void and then not the_msg.sent
	local  fil: LARGE_PLAIN_TEXT_FILE; junk: ACTION_ITEM;
	do  reply_cross_link;
	    if is_reply_sent_directly
	     then the_msg.send
	     else create fil.make_open_write(reply_build_name);
		  set_new_file_perm(fil);
		  the_msg.copy_to_file(fil);
	    	  os.flush_file_to_disk (fil)
		  fil.close;
		  fil.change_name(reply_final_name);
		  junk := glo.fork_on_commit(mode_is_sender)
	    end;
	end;

	send_msg_on_commit is
		-- as above, but queue for sending at next commit
	require the_msg /= Void and then not the_msg.sent
	local  fil: LARGE_PLAIN_TEXT_FILE; junk: ACTION_ITEM;
	do  reply_cross_link;
	    if is_reply_sent_directly
	     then generic_glo.mail_on_commit(the_msg);
	     else create fil.make_open_write(reply_build_name);
		  set_new_file_perm(fil);
		  the_msg.copy_to_file(fil);
	    	  os.flush_file_to_disk (fil)
		  fil.close;
		  generic_glo.rename_or_delete(fil.name, reply_final_name);
		  junk := glo.fork_on_commit(mode_is_sender)
	    end;
	end;

	mosis_reply_id: STRING is "MOSIS-REPLY-ID"

	dont_add_status: BOOLEAN;

	set_dont_add_status(val:BOOLEAN) is
	do
	    dont_add_status := val;
	end;



feature -- viewer support

	viewer_row: ARRAY [STRING] is
	    -- array of strings for display in the communication viewer.
	    -- viewer_row_select' is either viewer_table_row or viewer_status_row.
	    -- view_anchor_base may be set for inclusion in anchor if needed.
	    local
		str: STRING
	    do
		if viewer_row_select = viewer_table_row then
		    create Result.make(0,4);
    
		    str := "<A HREF=%"";
		    if viewer_anchor_base /= Void then
			str.append (viewer_anchor_base)
		    end;
		    str.extend ('#')
		    str.append (my_handle)
		    str.append ("%">")
    
		    if equal (communication_medium,glo.medium_is_email) then
			str.append ("EMail");
		    elseif equal (communication_medium,glo.medium_is_web) then
			str.append ("Page");
		    elseif not su.is_empty (communication_medium) then
			str.append (communication_medium);
		    else
			str.append ("Reply ");
			str.append (reply_no.out)
		    end;
		    str.append ("</A>");
		    Result.put(str,0);
		    if su.is_empty (design_state) then
			Result.put("",1)
		    else
			Result.put(su.string_of_2("State: ",design_state),1);
		    end
		    Result.put(colspan,2);	    -- (design)
		    if sent_to.is_empty then
			Result.put("",3);	    -- net_address
		    else
			Result.put(su.string_of_2 (
				    "<B>To: </B>",
				    std_html_fixer.modify (sent_to)),
			    3);			    -- net_address;
		    end
		    Result.put(date_processed.out,4);
		elseif viewer_row_select = viewer_status_row then
		    create Result.make(0,4);
		    Result.put("",0);
		    if is_mode_immediate then
			Result.put(su.string_of_2 ("Severity: ",
			    severity_name(severity_no)),1)
		    elseif is_mode_asynchronous then
			Result.put(su.string_of_3 ("Severity: ",
			    severity_name(severity_no),
			    " (async)"),1)
		    elseif is_mode_info then
			Result.put(su.string_of_3 ("Severity: ",
			    severity_name(severity_no),
			    " (info)"),1)
		    elseif is_mode_aside then
			Result.put(su.string_of_3 ("Severity: ",
			    severity_name(severity_no),
			    " (aside)"),1)
		    else
			Result.put(su.string_of_4 ("Reply Mode: ",
			    reply_mode,
			    ". Severity: ",
			    severity_name(severity_no)),1)
		    end;
		    Result.put(colspan,2);
		    if su.is_empty (first_response_line) then
			Result.put("&nbsp",3);
		    else
			Result.put(std_html_fixer.modify(first_response_line),3);
		    end
		    Result.put(colspan,4);
		else
		end
	    end


feature {NONE} -- support

	reply_cross_link is
		-- add mosis_reply_id to message and medium and sent_to to repl
	do  -- add mosis_reply_id to msg
	    the_msg.set_header_field(mosis_reply_id, my_handle);
	    -- enter msg info into self and archive it
	    set_communication_medium(glo.medium_is_email);
	    set_content_type_id(glo.content_is_text);
	    set_sent_to(su.nicely_truncated_string
				(the_msg.message_header1(msg_to),
				 maxlen_sent_to));
	    set_header_subject(the_msg.message_header1(msg_subject));
	end;

	is_reply_sent_directly: BOOLEAN is
		-- are we sending it or just filing it
		-- (this is not the do_not_send option, but rather the 
		--  method we are using: sendmail vs. remote agent)
	do  Result := is_none(vars.out_mail_curr_dir)
	end;

	reply_build_name: STRING is
	require not is_reply_sent_directly;
	do  Result := os.full_file_name(vars.out_mail_temp_dir, my_handle)
	end;

	reply_final_name: STRING is
	require not is_reply_sent_directly;
	do  Result := os.full_file_name(vars.out_mail_curr_dir, my_handle)
	end;

end -- NFE_M_REPLIES_MROW
