class NFE_POLICY
	-- fetch our application_parameters (which are read-only),
	--   returning in the form desired.
	-- fetch our application_events (which are updateable) as rows

inherit
	GENERIC_APPLICATION_PARAMETERS

create
	make		-- in GENERIC_APPLICATION_PARAMETERS

feature -- specific policy items

	create_zero_breakdowns: BOOLEAN is
		-- do we create charge_breakdowns for zero amounts?
	once  Result := as_boolean("CREATE_ZERO_BREAKDOWNS")
	end;

	min_ppass_length: INTEGER is
		-- minimum length for a valid ppassword
	once  Result := as_integer("MIN_PPASS_LENGTH")
	end;

	max_user_text_lines: INTEGER is
		-- maximum length of a multi-line item
	once  Result := as_integer("MAX_USER_TEXT_LINES")
	end;

	black_hole_report_interval: INTEGER is
		-- (min) interval in days between black hole reports
	once  Result := as_integer("BLACK_HOLE_REPORT_INTERVAL")
	end;

	show_next_scheduled_run: BOOLEAN is
		-- do we show user date of the next run in status?
	once  Result := as_boolean("SHOW_NEXT_SCHEDULED_RUN")
	end;

	incomplete_NSF_report_message: STRING is
		-- PROJECT_REPLY message to send if incomplete (if any)
	once  Result := as_string("INCOMPLETE_NSF_REPORT_MESSAGE")
	end;

	use_acct_address_for_po: BOOLEAN is
		-- if a po_funded auth has no address, do we use the acct?
	once  Result := as_boolean("USE_ACCT_ADDRESS_FOR_PO")
	end;

	minimum_project_dimension: INTEGER is
		-- minimum acceptable value for size_x or size_y
	once  Result := as_integer("MINIMUM_PROJECT_DIMENSION")
	end;

	run_is_visible: STRING is
		-- point in cycle when a project's run becomes visible
	once  Result := as_string("RUN_IS_VISIBLE")
	end;

	minimum_density_grace: DOUBLE is
		-- slop in density comparison before we complain
	once  Result := as_double("MINIMUM_DENSITY_GRACE")
	end;

	fill_density_factor: DOUBLE is
		-- factor by which our fill cell exceeds minimum density
	once  Result := as_double("FILL_DENSITY_FACTOR")
	end;

	allowed_overdraft: DOUBLE is
		-- $ amount by which we allow an overdrawn account
	once  Result := as_double("ALLOWED_OVERDRAFT")
	end;

	info_is_retired: BOOLEAN is
		-- have the INFORMATION and LIBRARY requests been retired?
	once  Result := as_boolean("INFORMATION_REQUESTS_ARE_RETIRED")
	end;

	limited_distribution_packages: STRING is
		-- SQL condition selecting packages in limited distribution
	once  Result := as_string("LIMITED_DISTRIBUTION_PACKAGES")
	end;

	limited_distribution_accounts: STRING is
		-- SQL condition selecting accounts allowed to order
		-- packages that are in limited distribution
	once  Result := as_string("LIMITED_DISTRIBUTION_ACCOUNTS")
	end;

	res_submission_requires_proposal: BOOLEAN is
		-- does a MEP_RES account submission need to provide a
		-- valid proposal_id beforee it can be queued?
	once  result := as_boolean("RES_SUBMISSION_REQUIRES_PROPOSAL")
	end;

	res_project_linear_overage: DOUBLE is
		-- for projects governed by a research proposal, the
		-- factor by which a project dimension may exceed the
		-- limit in the proposal
	once  result := as_double("RES_PROJECT_LINEAR_OVERAGE")
	end;

	res_project_areal_overage: DOUBLE is
		-- for projects governed by a research proposal, the
		-- factor by which the project area may exceed the
		-- limit in the proposal
	once  result := as_double("RES_PROJECT_AREAL_OVERAGE")
	end;

	fund_nsa_from_hell: BOOLEAN is
		-- should the fe put all NSA sponsored submissions on
		-- the infamous -1 allocation automatically?
	once  result := as_boolean("FUND_NSA_FROM_HELL")
	end;

	auto_drc_limit: INTEGER is
		-- max number of auto_drc jobs per design
	once  result := as_integer("AUTO_DRC_LIMIT")
	end;

	auto_drc_cust_acct_nos: STRING is
		-- list of cust_acct_nos that are entitled
		-- if empty, then all commercial
	once  result := as_string("AUTO_DRC_CUST_ACCT_NOS")
	end;

	auto_drc_leadtime: DOUBLE is
		-- min number of days before run to order DRC
	once  result := as_double("AUTO_DRC_LEADTIME")
	end;

	auto_drc_content_type: STRING is
		-- CONTENT_TYPE of files being generated by auto_drc
	once  result := as_string("AUTO_DRC_CONTENT_TYPE")
	end;

	ITAR_requires_liaison: BOOLEAN is
		-- must ITAR liaison be established before new-project?
	once  result := as_boolean("ITAR_REQUIRES_LIAISON")
	end;

	ITAR_requires_encryption: BOOLEAN is
		-- must ITAR projects be encrypted when sent to us?
	once  result := as_boolean("ITAR_REQUIRES_ENCRYPTION")
	end;

	null_export_control_value: STRING is
		-- the input/display for export_control_type is null
	once  result := as_string("NULL_EXPORT_CONTROL_VALUE")
	end;

	send_bond_diagram_announcement: BOOLEAN is
		-- should bonding diagrams be announced in a separate msg?
	once Result := as_boolean("SEND_BOND_DIAGRAM_ANNOUNCEMENT")
	end;

	reasonable_run_date_requested: STRING is
		-- SQL condition for a "reasonable" run_date_requested
	once Result := as_string("REASONABLE_RUN_DATE_REQUESTED")
	end;

	shipment_notice_window: INTEGER is
		-- days prior to search for "recent" shipments
	once Result := as_integer("SHIPMENT_NOTICE_WINDOW")
	end;

	org_no_for_ibmmdr_note: INTEGER is
		-- comparator to design_techns.org_no for IBMMDR note
	once Result := as_integer("ORG_NO_FOR_IBMMDR_NOTE")
	end;

	tight_die_fit: INTEGER 
		-- threshold between minimum die size possible and
		-- specified max_allowed_die_size to trigger warning
	once Result := as_integer("TIGHT_DIE_FIT")
	end;
		
	date_scheduled_gap: INTEGER 
		-- limit, in days, to locate scheduled_run closest
		-- to his run_date_requested
	once Result := as_integer("DATE_SCHEDULED_GAP")
	end;

	suppressed_option_marks: STRING
		-- comma-separated list of (signed) option_ids to
		-- omit from the generic "no process found" message
	once Result := as_string("SUPPRESSED_OPTION_MARKS")
	end;

feature -- database policy values that denote points in the run cycle

	visible_when_closed: STRING is "CLOSED"
	visible_when_posted: STRING is "POSTED"
	visible_when_rtm:    STRING is "RTM"
	visible_when_shipped: STRING is "SHIPPED"

feature -- plain text response formatting parameters

	response_line_length: INTEGER is
		-- length of response lines (for folding)
	once  Result := as_integer("RESPONSE_LINE_LENGTH")
	end;

	response_line_prefix: STRING is
		-- prefix at head of each response line
	once  Result := as_fixed_length_string("RESPONSE_LINE_PREFIX")
	end;

	continuation_line_prefix: STRING is
		-- prefix at head of each continuation line
	once  Result := as_fixed_length_string("CONTINUATION_LINE_PREFIX")
	end;

	response_line_suffix: STRING is
		-- suffix at head of each response line
	once  Result := as_fixed_length_string("RESPONSE_LINE_SUFFIX")
	end;

	nfe_plain_text_modifier: PRETTY_PRINT_STRING_MODIFIER is
	once  create result.make(response_line_length,
			response_line_prefix, continuation_line_prefix,
			response_line_suffix)
	end;

feature -- ok_message report and filename

	nfe_ok_msg_rep: STRING is
	once Result := as_string("PAYLOAD_AND_CHARGES_REPORT_NAME")
	end;

	nfe_ok_msg_suffix: STRING is
	once Result := as_string("PAYLOAD_AND_CHARGES_FILENAME_SUFFIX")
	end;

feature	-- internal mailing lists

	drc_notify_list: STRING is
	once Result := as_string_or_Void("DRC_NOTIFY_TO_LIST");
	end;

	ip_included_notify_list: STRING is
	once Result := as_string_or_Void("IP_INCLUDED_NOTIFY_TO_LIST");
	end;

feature	-- standard quote generator

	cust_quote_id_base: INTEGER is
	once Result := as_integer("CUST_QUOTE_ID_BASE")
	end;

	cust_quote_approved_by: STRING is
	once Result := as_string("CUST_QUOTE_APPROVED_BY")
	end;

	cust_quote_approved_by_title: STRING is
	once Result := as_string("CUST_QUOTE_APPROVED_BY_TITLE")
	end;

	cust_quote_terms: STRING is
	once Result := as_string("CUST_QUOTE_TERMS")
	end;

	cust_quote_t_and_c_part: INTEGER is
	once Result := as_integer("CUST_QUOTE_T_AND_C_PART")
	end;

	cust_quote_optional_parts: STRING is
		-- a SQL where clause to fetch optional parts for every quote
	once Result := as_string("CUST_QUOTE_OPTIONAL_PARTS")
	end;

	export_order_handling_part: INTEGER is
		-- m_part_no for the Export Order Handling Charge
	once Result := as_integer("EXPORT_ORDER_HANDLING_PART")
	end;

	ITAR_processing_part: INTEGER is
		-- m_part_no for the ITAR processing lot charge
	once Result := as_integer("ITAR_PROCESSING_PART")
	end;

feature	-- URLs

	custom_quote_url: STRING is
		-- URL for the custom quote request form
	once Result := as_string("CUSTOM_QUOTE_URL")
	end;

	mosis_pricing_url: STRING is
		-- URL for the page discussing Mosis pricing
	once Result := as_string("MOSIS_PRICING_URL")
	end;

	proj_doc_download_url: STRING is
		-- URL for the project document server login
	once Result := as_string("PROJ_DOC_DOWNLOAD_URL")
	end;

	bonding_diagram_info_url: STRING is
		-- URL for the section discussing bonding diagrams
	once Result := as_string("BONDING_DIAGRAM_INFO_URL")
	end;

	proj_status_req_url: STRING is
		-- URL for the project status request form
	once Result := as_string("PROJ_STATUS_REQ_URL")
	end;

	max_die_size_form_url: STRING is
		-- URL for the max-die-size-update form
	once Result := as_string("MAX_DIE_SIZE_FORM_URL")
	end;

	layer_change_notification: STRING
		-- ASIDE and/or REPLY (as substrings of value)
	once Result := as_string("LAYER_CHANGE_NOTIFICATION")
	end;

feature -- "Send" ftp policy items

	ftp_max_bytes: DOUBLE is
		-- Maximum size of design file we accept via ftp, in bytes
	once Result := as_double("FTP_MAX_BYTES")
	end;

	send_ftp_server_host: STRING is
		-- Name of host to which users ftp to send us a design file
	once Result := as_string("SEND_FTP_SERVER_HOST")
	end;

	send_ftp_def_expire_hours: DOUBLE is
		-- Default number of hours before send ftp authorization 
		-- expires
	once Result := as_double("SEND_FTP_DEF_EXPIRE_HOURS")
	end;

	send_ftp_max_expire_hours: DOUBLE is
		-- Maximum allowed number of hours before send 
		-- ftp authorization -- expires
	once Result := as_double("SEND_FTP_MAX_EXPIRE_HOURS")
	end;

	send_ftp_base_dir: STRING is
		-- Base directory for response to PWD command
	once Result := as_string("SEND_FTP_BASE_DIR")
	end;

	send_ftp_max_login_tries: INTEGER is
		-- Maximum number of login attempts to ftp server before
		-- server closes control connection
	once Result := as_integer("SEND_FTP_MAX_LOGIN_TRIES")
	end;

	send_ftp_bad_login_delay: DOUBLE is
		-- Number of seconds to sleep after each failed ftp server
		-- login attempt, to slow down automated break-in attempts
	once Result := as_double("SEND_FTP_BAD_LOGIN_DELAY")
	end;

	send_ftp_prelogin_idle_timeout: DOUBLE is
		-- Send ftp inactivity timeout interval before login, 
		-- in seconds
	once Result := as_double("SEND_FTP_PRELOGIN_IDLE_TIMEOUT")
	end;

	send_ftp_prestore_idle_timeout: DOUBLE is
		-- Send ftp inactivity timeout interval after login but 
		-- before store started, in seconds
	once Result := as_double("SEND_FTP_PRESTORE_IDLE_TIMEOUT")
	end;

	send_ftp_store_idle_timeout: DOUBLE is
		-- Send ftp inactivity timeout interval during store, 
		-- in seconds
	once Result := as_double("SEND_FTP_STORE_IDLE_TIMEOUT")
	end;

	send_ftp_poststore_idle_timeout: DOUBLE is
		-- Send ftp inactivity timeout interval after successful 
		-- store, in seconds
	once Result := as_double("SEND_FTP_POSTSTORE_IDLE_TIMEOUT")
	end;

	send_ftp_prelogin_done_timeout: DOUBLE is
		-- Send ftp maximum time in state Pre-Login, in seconds
	once Result := as_double("SEND_FTP_PRELOGIN_DONE_TIMEOUT")
	end;

	send_ftp_prestore_done_timeout: DOUBLE is
		-- Send ftp maximum time in state Pre-Store (after login 
		-- but before store started), in seconds
	once Result := as_double("SEND_FTP_PRESTORE_DONE_TIMEOUT")
	end;

	send_ftp_store_done_timeout: DOUBLE is
		-- Send ftp maximum time for store, in seconds
	once Result := as_double("SEND_FTP_STORE_DONE_TIMEOUT")
	end;

	send_ftp_poststore_done_timeout: DOUBLE is
		-- Send ftp maximum time after successful store, in seconds
	once Result := as_double("SEND_FTP_POSTSTORE_DONE_TIMEOUT")
	end;

end -- NFE_POLICY
