class NFE_PROCESSOR_MODE

inherit
	NFE_SUPPORT

feature -- the parameters that determine who we are

	param_is_file:	STRING is "FE_PROC_FILE";
	param_is_name:	STRING is "FE_PROC_NAME";
	param_is_mode:	STRING is "FE_PROC_TYPE";
	param_is_ftp_lock: STRING is "FTP_CLIENT_LOCK"	-- for ftp and client

feature -- builder of a config file line

	string_is_equal: STRING   is "=";

	configuration_line(param, val: STRING): STRING is
		-- just put the two together
	do  Result := su.string_of_3(param, string_is_equal, val)
	end;

feature -- the allowed VALUES of fe_proc_type (i.e., the operating modes
	--  of the FE system)

	mode_is_email:	STRING is "EMAIL";
	mode_is_cgi:	STRING is "CGI";
	mode_is_ftp:	STRING is "FTP";
	mode_is_dchk:	STRING is "DCHK";
	mode_is_lchk:	STRING is "LCHK";	-- validate completed layout
	mode_is_ipxlat:	STRING is "IPXLAT";	-- IP address translator
	mode_is_mon:	STRING is "MON";
	mode_is_arch:	STRING is "ARCH";	-- archive manager
	mode_is_dummy:	STRING is "DUMMY"	-- (for development)
	mode_is_util:	STRING is "UTIL"	-- collection of utilities
	mode_is_intrq:	STRING is "INTRQ"	-- internal_fe_requests
 	mode_is_sender:	STRING is "SENDER"	-- message sender (not in nfe)
  --	mode_is_padx:	STRING is "PAD_EX"	-- pad extractor (not in nfe)
	mode_is_ftp_cl: STRING is "FTP_CL";	-- ftp child to retrieve
	mode_is_ftp_srv:STRING is "FTP_SRV";	-- ftp server process

	log_key_suffix:  STRING is "_LOG_FILE"
	lock_key_suffix: STRING is "_LOCK_FILE"
	shutd_key_suffix:STRING is "_SHUTD_FILE"

feature -- values that are per mode

	fe_proc_name(mode: STRING): STRING is
		-- the fe_proc_name for this mode
	do  if   equal(mode, mode_is_email)then Result := "mail_processor"
	  elseif equal(mode, mode_is_ftp)  then Result := "ftp_daemon"
	  elseif equal(mode, mode_is_dchk) then Result := "dchk_daemon"
	  elseif equal(mode, mode_is_lchk) then Result := "layout_daemon"
	  elseif equal(mode, mode_is_cgi)  then Result := "cgi_processor"
	  elseif equal(mode, mode_is_ipxlat)  then Result := "ip_translator"
	  elseif equal(mode, mode_is_mon)  then Result := "fe_monitor"
	  elseif equal(mode, mode_is_arch) then Result := "archive_manager"
	  elseif equal(mode, mode_is_util) then Result := "fe_utilities"
	  elseif equal(mode, mode_is_intrq) then Result := "internal_fe_reqs"
	  elseif equal(mode, mode_is_sender) then Result := "message_sender"
  --	  elseif equal(mode, mode_is_padx) then Result := "pad_extractor"
	  elseif equal(mode, mode_is_dummy)then Result := "dummy"
	  elseif equal(mode, mode_is_ftp_cl)then Result := "ftp_client"
	  elseif equal(mode, mode_is_ftp_srv)then Result := "ftp_server"
	    end
	end;

	log_file_key(mode: STRING): STRING is
		-- name of log_file KEY for this mode
	do  Result := su.string_of_2(mode, log_key_suffix)
	end;

	lock_file_key(mode: STRING): STRING is
		-- name of lock_file KEY for this mode
	do  Result := su.string_of_2(mode, lock_key_suffix)
	end;

	shutd_file_key(mode: STRING): STRING is
		-- name of shutd_file KEY for this mode
	do  Result := su.string_of_2(mode, shutd_key_suffix)
	end;

feature -- shutdown test (also done in fe_processor_loop)

	is_shut_down(mode: STRING): BOOLEAN is
	local sfn: STRING
	do  sfn := vars.item(shutd_file_key(mode));	-- file name, if any
	    Result := su.not_empty(sfn) and then fu.file_exists(sfn)
	end;

feature -- send a punt message (if you can)

	attempt_to_send_punt_msg(subj, body: STRING) is
		-- (attempt to) send a punt message
		-- go quietly on failure
	local in_retry: BOOLEAN; foo: OUTGOING_MESSAGE; tto: STRING
	do  if not in_retry
	     then create foo.make_alt_mailer(vars.nfe_mailer_command);
		  tto := vars.nfe_punt_to_list;
		  if su.is_empty(tto) then tto := "mfe-default@mosis.com" end;
		  foo.set_header_field(foo.msg_to, tto);
		  foo.set_header_field(foo.msg_subject, subj);
		  foo.set_message_body(body);
		  foo.send;
	    end;
	rescue
	    in_retry := true;  retry;
	end;

end -- NFE_PROCESSOR_MODE
