class NFE_SUPPORT
	-- routines for everyone to share; no attributes here

inherit

	GENERIC_SYSTEM_SUPPORT
	rename	system_punt	as nfe_punt,
		system_punt_int as nfe_punt_int,
		system_punt_unk as nfe_punt_unk,
		glo		as generic_glo,
		vars		as generic_vars
	end;

	EXECUTION_ENVIRONMENT_ACCESS

	REPORT_EXECUTION

	DB_SIGNAL_NAMES		-- just until transition
	
feature -- additional globals for nfe

	glo: NFE_GLOBALS is
		-- some globals (including NFE_STRINGS)
        once  create Result.make(generic_glo)
	end;

	vars: NFE_CONFIGURATION_VARS is
		-- access to all of our configuration variables
		-- as vars.nfe_xxxx
	once  create Result.make (generic_vars)
	end;

feature -- more globals just for nfe

	policy: NFE_POLICY is
		-- access to our application_parameters and events
	once  create Result.make(glo.application)  end;

	minfo: SEARCH_PATH is
		-- minfo:
	once  create Result.make_from_string(vars.nfe_minfo_path, vars);
	      if not Result.all_exist
	       then nfe_punt_unk("MINFO", "directory", minfo.item)
	      end;
	end;

feature -- misc functions

	none_string: STRING is "NONE"

	empty_string: STRING is ""

	is_none(str: STRING): BOOLEAN is
		-- is the string equivalent to "none"?
	require str /= Void
	do  Result := su.fuzzy_leading_alpha_match(str, none_string, 2) > 0
		-- "NO", "NON", or "NONE", with punctuation
	end;

	set_new_file_perm(f: LARGE_FILE) is
		-- apply the standard new file permission to f
	require  f /= Void;
	local  p: STRING
	do  p := vars.nfe_new_file_perm;
	    if su.not_empty(p)
	     then fu.change_file_mode(f, p) end
	end;

	integer_size_string(x, y: INTEGER): STRING is
		-- "<x> x <y>"
	do  Result := su.string_of_3(x.out, " x ", y.out)
	end;

	integer_size_string_d(x, y: DOUBLE): STRING is
		-- as above, but using doubles as input
	do  Result := integer_size_string(x.ceiling, y.ceiling)
	end;

	create_batch_job(job_name: STRING): BATCH_JOBS_MROW is
		-- common part of building a new BATCH_JOBS_ROW
		-- job_name becomes both job_name and process_name
	do  create Result.make_null(db);
	    Result.set_batch_job_no(
			db.sequence_next_value("BATCH_JOB_NO_SEQ"));
	    Result.set_job_name(job_name);
	    Result.set_process_name(job_name);
	    Result.set_batch_job_state(glo.bjstate_is_waiting);
	    Result.set_date_submitted(db.transaction_date);
	    Result.set_date_state_changed(db.transaction_date);
	    Result.set_priority(glo.bjpriority_is_normal);
	    Result.set_restartable(true);
	    Result.set_notify_on_failure(true);
	    db.queue_signal(glo.Job_queue_changed_signal, Void);
	end;

	delete_idle_ftp_transfer(dno, rno, fno: INTEGER): BOOLEAN is
		-- if there is a remotely initiated ftp_transfer for this
		-- guy (dno, rno, fno), and it is not locked right now,
		-- delete it (and return true)
	local stor: FTP_TRANSFER; tfil: STRING; tlock: LOCK;
	do  create stor.make_by_key(db, old_no_exception_row, dno, rno, fno);
	    if stor.is_old and stor.initiated_remotely
	     then tfil := stor.transfer_lock_file_name;
		  tlock := unix_os.new_lock(tfil);
		  tlock.lock_nowait;
		  if tlock.is_locked_by_me
		   then -- it's ours; get rid of it now
			stor.delete;
			glo.delete_on_commit(tfil);
			Result := true
		  end
	    end;
	end;

	db_login_parameter: STRING is
		-- db login string to pass to other (spawned) tools
	once  create Result.make(40);
	      Result.append(vars.item("DB_USER"));
	      Result.extend('/');
	      Result.append(vars.item("DB_PASS"));
	      Result.extend('@');
	      Result.append(vars.item("DB_TOOLS_CONN_STR"));
	end;

	execute_system_call(cmd: STRING) is
		-- we need to run csh if we want aliases to work
	local foo: STRING;
	do  foo := su.string_of_3("csh -c %'", cmd, "%'")
	    exe.system(foo)
	end;

	execute_SQL_report(rnam, fnam, xtraparam: STRING; 
			   log_it: BOOLEAN) is
		-- execute report rnam to produce file fnam
		-- add xtraparam to the standard parameter list
	local disp: STRING
	do  if log_it
	     then glo.log_line_2("running report ", rnam)
	    end;
  --	    if su.is_empty(exe.get("DISPLAY")) then
		disp := vars.report_display
  --	    end;
	    run_report_with_umask
			(db, 6,
			 vars.report_exe_file,
			 vars.oracle_reports_home,
			 vars.report_temp_dir,
			 rnam, xtraparam,
			 fnam, false, disp)
	end;

end -- NFE_SUPPORT
