
indexing
	
	description: "Generic operating system services"
	author: "David Hollenberg"
	date: "October 7, 1997"

deferred class OPERATING_SYSTEM

inherit
	OPERATING_ENVIRONMENT

feature -- Path name operations

	full_file_name (dir_name, f_name: STRING): STRING
			-- Full name of file in directory `dir_name'
			-- with name `f_name'.
		require
			dir_name_not_void: dir_name /= Void
			file_name_not_void: f_name /= Void
		deferred
		ensure
			result_exists: Result /= Void
		end

	full_directory_name (dir_name, subdir: STRING): STRING
			-- Full name of subdirectory `subdir' of directory
			-- `dir_name'
		require
			dir_name_not_void: dir_name /= Void
			subdir_not_void: subdir /= Void
		deferred
		ensure
			result_exists: Result /= Void
		end

feature -- File operations
	
	delete_directory_tree (dir_name: STRING)
			-- Try to delete the directory tree rooted at
			-- `dir_name'.  Ignore any errors
		require
			directory_not_void: dir_name /= Void
		deferred
		end

	flush_file_to_disk (f: LARGE_FILE)
			-- Flush all changes made to `f' to the operating
			-- system and then to disk.  Does not return until
			-- all changes are on disk
		require
			file_exists: f /= Void
			file_open: not f.is_closed
		deferred
		end

	truncate_file (f: LARGE_FILE)
			-- Truncate open file `f' to zero length
		require
			is_open: not f.is_closed
		deferred
		end

feature -- Process operations
	
	my_process_id: INTEGER
			-- Process id of currently executing process
		deferred
		ensure
			valid_process_id: Result > 0
		end

	local_host_name: STRING
			-- New string with name of host on which
			-- current process is executing
		deferred
		ensure
			result_exists: Result /= Void
		end

	is_sunos: BOOLEAN
			-- Is process executing on a host running SunOS?
		deferred
		end

	is_solaris: BOOLEAN
			-- Is process executing on a host running Solaris?
		deferred
		end

	die_without_handlers (code: INTEGER)
			-- Terminate execution with exit status `code',
			-- without calling any termination handlers
		deferred
		end

feature -- New objects

	new_lock (name: STRING): LOCK
			-- New lock object for lock named `name'
		require
			name_exists: name /= Void
		deferred
		ensure
			result_exists: Result /= Void
		end

feature -- Interval timer

	system_interval_timer: SYSTEM_INTERVAL_TIMER
			-- Operating system's interval timer
		deferred
		end

feature -- Date and time

	current_date_string: STRING
			-- String representing current date and time
			-- in unspecified format.  Intended only for
			-- display purposes where the format is not important.
		deferred
		ensure
			result_exists: Result /= Void
		end
	
	append_current_date_string (dest: STRING)
			-- Append to `dest' a string representing current
			-- date and time in unspecified format.  Intended
			-- only for display purposes where the format
			-- is not important.
		require
			destination_exists: dest /= Void
		deferred
		end
	
feature -- Sleeping

	sleep_microseconds (n: DOUBLE)
			-- Suspend execution for `n' microseconds.
			-- Actual time could be longer or shorter
		require
			nonnegative_time: n >= 0
		deferred
		end

	sleep_seconds (n: DOUBLE)
			-- Suspend execution for `n' seconds.
			-- Actual time could be longer or shorter
		require
			nonnegative_time: n >= 0.0
		deferred
		end

end
