
indexing
	
	description: "An operation that validates a username to see whether user is allowed to be an operator"
	author: "David Hollenberg"
	date: "March 12, 2003"

class OPERATOR_VALIDATION_OP
inherit
	DYNAMIC_OPERATION
		rename
			try_operation as try_to_validate_operator
		end
	DB_OPTIONS_ACCESS
		export
			{NONE} all
		undefine
			default_create
		end

create
	default_create

feature -- Operations

	validate_user (conn: DB_CONNECTION a_user: STRING)
			-- Check that `a_user' is allowed to run Delta
		require
			connection_exists: conn /= Void
			connection_open: conn.is_open
			connection_readable: conn.has_query_access
			user_exists: a_user /= Void
		do
			connection := conn
			user := a_user
			set_implementation (agent try_to_validate_user)
			execute
		end

feature -- Properties

	email_address: STRING
			-- E-mail address of last valid user?

	is_run_closer: BOOLEAN
			-- Is last valid user a run closer?

feature {NONE} -- Implementation

	connection: DB_CONNECTION
			-- Connection to use for validation

	user: STRING
			-- User name to validate

	operator: M_OPERATORS_ROW
			-- Operator row corresponding to `user', if any

	reason: STRING
			-- Reason for validation failure, if not Void

	try_to_validate_user
		local
			error: OPERATOR_VALIDATION_ERROR
		do
			fetch_operator_row
			if operator.is_empty then
				set_reason ("no row for user")
			elseif operator.is_null_net_address then
				set_reason ("user has no net address")
			else
				reason := Void
			end
			
			if reason = Void then
				email_address := operator.net_address
				is_run_closer := operator.is_closer
				publish_ok_event
			else
				email_address := Void
				publish_failed_event
				create error
				error.set_hard_error
				error.set_user_name (user)
				error.set_reason (reason)
				error.handle
			end
		end

	fetch_operator_row
			-- Fetch operator row for `user', if any
		do
			create operator.make_by_key (connection, option.Old_no_exception_row, user)
		end

	set_reason (s: STRING)
			-- Set `reason' based on `s'
		do
			reason := s + " in table " + operator.table_name.as_upper
		end

	publish_ok_event
		local
			ok_event: OPERATOR_OK_EVENT
		do
			create ok_event
			ok_event.set_user_name (user)
			ok_event.publish
		end

	publish_failed_event
		local
			invalid_event: OPERATOR_INVALID_EVENT
		do
			create invalid_event
			invalid_event.set_user_name (user)
			invalid_event.publish
		end

end
