
indexing
	
	author: "David Hollenberg"
	date:	"November 22, 1995"
	description: "Eiffel interface to Oracle Call Interface routines %
			%(OCI routines).  Intended to be inherited by classes %
			%which need its facilities"
		
class ORACLE_CALL_INTERFACE

feature -- OCI function return codes
	
	Oci_success_code: INTEGER = 0
			-- Return code which indicates success (no errors
			-- or warnings).

feature -- Externals which operate on a database connection
	-- These functions all return an integer return code
	-- which indicates whether the operation was successful.

	ora_logon_concurrent (lda_address, hda_address, user_id_address: POINTER
		user_id_length: INTEGER password_address: POINTER
		password_length: INTEGER): INTEGER
			-- Open a new Oracle database connection for the
			-- given LDA and HDA.  This call allows multiple
			-- concurrent connections.
		external
			"C macro use %"oracle.h%""
		end

	ora_logoff (lda_address: POINTER): INTEGER
			-- Close the Oracle database connection indicated
			-- by the LDA at `lda_address'
		external
			"C macro use %"oracle.h%""
		end

	ora_commit (lda_address: POINTER): INTEGER
			-- Commit the current transaction for the LDA
			-- at address `lda_address'
		external
			"C macro use %"oracle.h%""
		end

	ora_rollback (lda_address: POINTER): INTEGER
			-- Rollback the current transaction for the LDA
			-- at address `lda_address'
		external
			"C macro use %"oracle.h%""
		end

	ora_abort_oci_function (lda_address: POINTER): INTEGER
			-- Abort any currently executing OCI function
			-- for the LDA at address `lda_address'
		external
			"C macro use %"oracle.h%""
		end

	ora_enable_autocommit (lda_address: POINTER): INTEGER
			-- Enable automatic commit of every SQL data
			-- manipulation statement for the LDA at address
			-- `lda_address'
		external
			"C macro use %"oracle.h%""
		end

	ora_disable_autocommit (lda_address: POINTER): INTEGER
			-- Disable automatic commit of every SQL data
			-- manipulation statement for the LDA at address
			-- `lda_address'
		external
			"C macro use %"oracle.h%""
		end

	ora_get_error_message (lda_address: POINTER return_code: INTEGER
		message_buffer_address: POINTER
		message_buffer_length: INTEGER): INTEGER
			-- Put the error message corresponding to error code
			-- `return_code' truncated to `message_buffer_length'
			-- characters into the buffer at
			-- `message_buffer_address'.  Return number of
			-- characters placed in buffer.
		external
			"C macro use %"oracle.h%""
		end

feature -- Externals which operate on an SQL statement

	ora_open_cursor (cda_address, lda_address: POINTER): INTEGER
		external
			"C macro use %"oracle.h%""
		end

	ora_close_cursor (cda_address: POINTER): INTEGER
		external
			"C macro use %"oracle.h%""
		end

	ora_parse_sql_statement (cda_address, sql_statement_address: POINTER
		sql_statement_length: INTEGER deferred_flag: INTEGER): INTEGER
		external
			"C macro use %"oracle.h%""
		end

	ora_bind_placeholder_name (cda_address: POINTER placeholder_name: POINTER
		placeholder_length: INTEGER var_address: POINTER
		var_length: INTEGER external_datatype: INTEGER
		null_indicator_address: POINTER): INTEGER
		external
			"C macro use %"oracle.h%""
		end

	ora_bind_placeholder_number (cda_address: POINTER placeholder_num: INTEGER
		var_address: POINTER var_length: INTEGER
		external_datatype: INTEGER null_indicator_address: POINTER): INTEGER
		external
			"C macro use %"oracle.h%""
		end

	ora_describe_select_item (cda: POINTER select_list_position: INTEGER
		max_column_size_address, internal_datatype_address,
		select_list_name_address, select_list_name_length_address,
		max_display_size_address, precision_address, scale_address, null_allowed_address: POINTER): INTEGER
		external
			"C macro use %"oracle.h%""
		end

	ora_bind_select_item (cda_address: POINTER select_list_position: INTEGER
		var_address: POINTER var_length: INTEGER
		external_datatype: INTEGER value_indicator_address: POINTER
		return_length_address, column_return_code_address: POINTER): INTEGER
		external
			"C macro use %"oracle.h%""
		end

	ora_execute_sql_statement (cda_address: POINTER): INTEGER
		external
			"C macro use %"oracle.h%""
		end

	ora_execute_multiple_sql_statements (cda_address: POINTER count, offset: INTEGER): INTEGER
		external
			"C macro use %"oracle.h%""
		end

	ora_fetch_row (cda_address: POINTER): INTEGER
		external
			"C macro use %"oracle.h%""
		end

	ora_fetch_multiple_rows (cda_address: POINTER count: INTEGER): INTEGER
		external
			"C macro use %"oracle.h%""
		end

	ora_cancel_query (cda_address: POINTER): INTEGER
		external
			"C macro use %"oracle.h%""
		end

	ora_set_wait_for_resources (cda_address: POINTER wait_for_resources: BOOLEAN): INTEGER
		external
			"C macro use %"oracle.h%""
		end

feature -- Externals which the integer value at an address

	signed_short_value (addr: POINTER): INTEGER
			-- The signed short integer at address `addr' as
			-- an integer
		external
			"C macro use %"oracle.h%""
		end

	unsigned_short_value (addr: POINTER): INTEGER
			-- The unsigned short integer at address `addr' as
			-- an integer
		external
			"C macro use %"oracle.h%""
		end

	signed_integer_value (addr: POINTER): INTEGER
			-- The signed integer at address `addr' as
			-- an integer
		external
			"C macro use %"oracle.h%""
		end

	unsigned_integer_value (addr: POINTER): INTEGER
			-- The unsigned integer at address `addr' as
			-- an integer
		external
			"C macro use %"oracle.h%""
		end

end
