
indexing
	
	author: "David Hollenberg"
	date:	"November 28, 1995"
	description: "A connection to an Oracle database"

class ORACLE_CONNECTION

inherit
	DB_CONNECTION
		rename
			Db_success_code as Oci_success_code,
			handle_db_user_error as handle_oracle_user_error,
			abort_operation as abort_oci_function,
			sql_statement as parsed_cursor,
			sql_change as parsed_dml_cursor,
			sql_block as parsed_block_cursor,
			sql_query as parsed_query_cursor,
			max_cached_sql_statements as max_parsed_cursors,
			set_max_cached_sql_statements as set_max_parsed_cursors,
			cached_statement_age_limit as parsed_cursor_age_limit,
			set_cached_statement_age_limit as set_parsed_cursor_age_limit,
			flush_sql_statement_cache as flush_cursor_cache,
			in_use_statement_count as reserved_cursor_count,
			in_use_statements_kept as keep_reserved_cursor_list,
			in_use_statements_description as reserved_cursors_description,
			set_in_use_statements_kept as set_keep_reserved_cursor_list
		redefine
			-- error_handler, parsed_cursor, date
			-- parsed_dml_cursor, parsed_query_cursor
		end
	ORACLE_CALL_INTERFACE
		export
			{NONE} all
		end
	ORACLE_HDA
		rename
			make as hda_make,
			address as hda_address,
			length as hda_length
		export
			{NONE} all
		end
	ORACLE_LDA
		rename
			make as lda_make,
			address as lda_address,
			length as lda_length
		export
			{ANY} lda_address, return_code, os_error_code
			{NONE} all
		end
	ORACLE_CURSOR_STATES
		export
			{NONE} all
		end
	ORACLE_STATEMENT_TYPES
		export
			{NONE} all
		end
	ORACLE_EXCEPTIONS
		export
			{NONE} all
		end
	STRING_FILLER_ACCESS
		export
			{NONE} all
		end
	DB_CONNECTION_ACCESS
		export
			{NONE} all
		end
	UNIX_OS
		rename
			send_signal as unix_os_send_signal,
			catch as unix_os_catch,
			ignore as unix_os_ignore,
			return_code as exenv_return_code,
			Infinity as unix_os_Infinity
		export
			{NONE} all
		end
	EXECUTION_ENVIRONMENT
		rename
			return_code as exenv_return_code
		export
			{NONE} all
		end
	MEMORY
		export
			{NONE} all
		redefine
			dispose
		end
	ASCII
		export
			{NONE} all
		end

create
	make

feature {NONE} -- Creation

	make
		do
			hda_make
			lda_make
			create {ORACLE_STD_ERROR_HANDLER} error_handler
			error_checking_enabled := True
			create cursor_server.make (Current)
			create last_error_description.make (0)
			create transaction.make (Current)
			transaction.start
			create {ARRAYED_LIST [STRING]} registered_signals.make (5)
			registered_signals.compare_objects
			create {SORTED_TWO_WAY_LIST [DB_SIGNAL]} queued_signals.make
			queued_signals.compare_objects
			query_buffer_size_limit := Default_query_buffer_size_limit
			query_rows_per_fetch := Default_query_rows_per_fetch
			has_query_access := True
			create temp_buffer.make (0)
			set_lock_fail_defaults
			dbcm.register (Current)
			override_interrupt_handler
		end
			
feature {NONE}	-- Implementation

	Connection_env_variable: STRING = "TWO_TASK"
			-- Name of environment variable with connect
			-- string or alias for connect string

	Default_max_open_attempts: INTEGER = 5
			-- Default number of times to attempt to open
			-- database connection before giving up, if
			-- retryable errors occur

	Sequence_curr_pseudo_col: STRING = "currval"
			-- Oracle pseudo column name for current value of
			-- a sequence

	Sequence_next_pseudo_col: STRING = "nextval"
			-- Oracle pseudo column name for next value of
			-- a sequence

	Select_keyword: STRING = "select "
			-- SQL select keyword

	From_dual_keyword: STRING = " from dual"
			-- SQL string to select from table DUAL

	System_date_query: STRING = "select sysdate from dual"
			-- SQL string to select system date

	User_name_query: STRING = "select user from dual"
			-- SQL string to select connected user name

	Max_user_name_length: INTEGER = 30
			-- Maximum length of user name

	cursor_server: ORACLE_CURSOR_SERVER
			-- Cursor server for this connection, which
			-- allocates a parsed cursor for an SQL statement

	transaction: ORACLE_TRANSACTION
			-- Current transaction for this connection

	increment_transaction_number (n: INTEGER)
			-- Increment transaction number to indicate
			-- the start of a new transaction
		do
			transaction.start
		end

	Connect_string_delimiter: CHARACTER is '@'
			-- Character which separates connect string
			-- from user id

	Default_query_buffer_size_limit: INTEGER = 65536
			-- Default size limit for query buffer for
			-- each query

	Default_query_rows_per_fetch: INTEGER = 32
			-- Default number of rows to be fetched at a time
			-- for a query

	Db_access_error_message: STRING = "attempt to access database violates current access level"
			-- String for error message when caller attempts
			-- to access connection in manner which is not allowed
			-- by current access level

	Db_access_change_message: STRING = "attempt to change database access level with uncommitted changes"
			-- String for error message when caller attempts
			-- to change allowed database access level, but has
			-- pending posted or unposted changes in current
			-- transaction

	Db_uncommitted_changes_message: STRING = "attempt to logoff or exec DDL statement with uncommitted changes"
			-- String for error message when caller attempts
			-- to logoff or execute a DDL statement, but has
			-- pending posted or unposted changes in current
			-- transaction

	Db_illegal_statement_error_message: STRING = "statement type is known but is not allowed"
			-- String for error message when type of potential
			-- SQL statement can be determined, but that type
			-- of statement has been disallowed (made illegal)

	Db_unknown_statement_error_message: STRING = "statement type cannot be determined"
			-- String for error message when type of potential
			-- SQL statement cannot be determined, but the
			-- statement is accepted by the Oracle engine

	temp_buffer: STRING
			-- Buffer used for various purposes.  Allocated
			-- once per connection to avoid extra object
			-- allocations.

	keyword_table: HASH_TABLE [INTEGER, STRING]
			-- Table which maps first word of potential
			-- SQL statement to its integer type
		once
			create Result.make (0)
			Result.put (Ddl_statement,       "ALTER")
			Result.put (Ddl_statement,       "AUDIT")
			Result.put (Block_statement,     "BEGIN")
			Result.put (Ddl_statement,       "COMMENT")
			Result.put (Illegal_statement,   "COMMIT")
			Result.put (Ddl_statement,       "CREATE")
			Result.put (Block_statement,     "DECLARE")
			Result.put (Dml_statement,       "DELETE")
			Result.put (Ddl_statement,       "DROP")
			Result.put (Dml_statement,       "EXPLAIN")
			Result.put (Ddl_statement,       "GRANT")
			Result.put (Dml_statement,       "INSERT")
			Result.put (Dml_statement,       "LOCK")
			Result.put (Ddl_statement,       "NOAUDIT")
			Result.put (Ddl_statement,       "RENAME")
			Result.put (Ddl_statement,       "REVOKE")
			Result.put (Illegal_statement,   "ROLLBACK")
			Result.put (Illegal_statement,   "SAVEPOINT")
			Result.put (Query_statement,     "SELECT")
			Result.put (Illegal_statement,   "SET")
			Result.put (Dml_statement,       "UPDATE")
			Result.put (Ddl_statement,       "VALIDATE")
		end

	Normal: INTEGER = 1
			-- Code for normal state

	Starting_comment: INTEGER = 2
			-- Code for "saw start of comment" state

	In_comment: INTEGER = 3
			-- Code for "in comment" state

	Ending_comment: INTEGER = 4
			-- Code for "saw first part of comment end" state

	get_statement_first_word (stmt, buffer: STRING)
			-- Parse potential SQL statement to get its first
			-- word and put it in `buffer'
		require
			statement_exists: stmt /= Void
			buffer_exists: buffer /= Void
		local
			k, count, state, start_pos, end_pos: INTEGER
			c: INTEGER
			found: BOOLEAN
		do
			buffer.keep_head (0)
			from
				k := 1
				count := stmt.count
				state := Normal
			until
				k > count or found
			loop
				c := stmt.item_code (k)
				inspect state
				when Normal then
					inspect c
					when Blank, Tabulation, Line_feed, Vt, Np, Carriage_return then
					when Slash then
						state := Starting_comment
						start_pos := k
					else
						found := True
						start_pos := k
					end
				when Starting_comment then
					If c = Star then
						state := In_comment
					else
						state := Normal
						found := True
					end
				when In_comment then
					If c = Star then
						state := Ending_comment
					end
				when Ending_comment then
					If c = Slash then
						state := Normal
						start_pos := 0
					elseif c /= Star then
						state := In_comment
					end
				end
				k := k + 1
			end
			
			if start_pos > 0 then
				from
					k := start_pos
				until
					k > count or else not (stmt.item (k).is_alpha or stmt.item (k).is_digit)
				loop
					k := k + 1
				end
				if k > count then
					end_pos := count
				else
					end_pos := k - 1
				end
				buffer.set (stmt, start_pos, end_pos)
			end
		end

	statement_type (sql_stmt: STRING): INTEGER
			-- Type of statement `sql_stmt' represents
		do
			get_statement_first_word (sql_stmt, temp_buffer)
			temp_buffer.to_upper
			Result := keyword_table.item (temp_buffer)
			if Result = 0 then
				Result := Unknown_statement
			end
		end
	
	handle_access_error (cursor: ORACLE_CURSOR operation, stmt: STRING)
			-- Handle an Oracle error which involves an
			-- attempt to access the database in a manner
			-- which is not allowed by current access
			-- level.  `cursor' is the cursor involved if
			-- non-void.  `operation' is the operation
			-- that was attempted and `stmt' is the SQL
			-- statement involved (if any)
		require
			operation_exists: operation /= Void
		local
			message: STRING
		do
			create message.make (0)
			message.append (Db_access_error_message)
			message.append ("%N")
			message.append (operation)
			message.append (" not allowed with access level %"")
			message.append (access_level)
			message.append ("%"")
			if stmt /= Void then
				message.append ("%NSQL statement: ")
				message.append (stmt)
			end
			handle_oracle_user_error (cursor, dbex.Db_access_error,
				message)
		end

	handle_access_change_error (level: STRING)
			-- Handle an error which involves an
			-- attempt to change the database access level `level'
			-- when there are posted or unposted changes
			-- for the current transaction
		require
			level_exists: level /= Void
		local
			message: STRING
		do
			create message.make (0)
			message.append (Db_access_change_message)
			message.append ("%NAccess level change to %"")
			message.append (level)
			message.append (" access%" not allowed with pending changes")
			handle_oracle_user_error (Void, dbex.Db_uncommitted_changes,
				message)
		end

	handle_uncommitted_changes_error (operation, stmt: STRING)
			-- Handle an error which involves an attempt
			-- to do operation `operation' via SQL
			-- statement `stmt' when there are posted or
			-- unposted changes for the current
			-- transaction
		require
			operation_exists: operation /= Void
		local
			message: STRING
		do
			create message.make (0)
			message.append (Db_uncommitted_changes_message)
			message.append ("%NOperation is ")
			message.append (operation)
			if stmt /= Void then
				message.append ("%NSQL statement: ")
				message.append (stmt)
			end
			handle_oracle_user_error (Void, dbex.Db_uncommitted_changes,
				message)
		end

	handle_wrong_statement_type (stmt: STRING expected_type: INTEGER
	   actual_type: INTEGER)
			-- Handle an Oracle error which involves an
			-- inconsistency between the type of statement
			-- indicated by the first word of the statement
			-- and the type of statement indicated in the
			-- cursor data area of `cursor' after parsing the
			-- statement.
		require
			statement_exists: stmt /= Void
			valid_expected_type: valid_statement_type (expected_type)
			valid_actual_type: valid_statement_type (actual_type)
		local
			message: STRING
		do
			create message.make (0)
			message.append ("SQL statement should be a ")
			message.append (statement_type_string (expected_type))
			message.append (" but is a ")
			message.append (statement_type_string (actual_type))
			message.append ("%NSQL statement: ")
			message.append (stmt)
			handle_oracle_user_error (Void, dbex.Db_wrong_statement_type,
				message)
		end

	handle_sql_code_inconsistency (cursor: ORACLE_CURSOR operation: STRING)
			-- Handle an Oracle error which involves an
			-- inconsistency between the type of statement
			-- indicated by the first word of the statement
			-- and the type of statement indicated in the
			-- cursor data area of `cursor' after parsing the
			-- statement.
		require
			cursor_exists: cursor /= Void
			operation_exists: operation /= Void
		local
			message: STRING
		do
			create message.make (0)
			message.append ("SQL function code should be consistent with a ")
			message.append (operation)
			message.append (" but is not")
			handle_oracle_user_error (cursor, dbex.Db_inconsistent_type,
				message)
		end

	handle_illegal_statement_error (stmt: STRING)
			-- Handle an illegal statement error on
			-- statement `stmt'.  This error is signalled
			-- for a statement whose type is known but
			-- which is prohibited because it causes problems
			-- for our model.  For example, "commit" is illegal
			-- because it starts a new transaction behind our back.
		require
			statement_exists: stmt /= Void
		local
			message: STRING
		do
			create message.make (0)
			message.append (Db_illegal_statement_error_message)
			message.append ("%N")
			message.append ("%NSQL statement: ")
			message.append (stmt)
			handle_oracle_user_error (Void, dbex.Db_illegal_statement_type,
				message)
		end

	handle_unknown_statement_error (stmt: STRING)
			-- Handle an unknown statement error on
			-- statement `stmt'.  This error is signalled
			-- when a statement whose type is unknown is detected.
		require
			statement_exists: stmt /= Void
		local
			message: STRING
		do
			create message.make (0)
			message.append (Db_unknown_statement_error_message)
			message.append ("%N")
			message.append ("%NSQL statement: ")
			message.append (stmt)
			handle_oracle_user_error (Void, dbex.Db_unknown_statement_type,
				message)
		end

	set_lock_fail_defaults
			-- Set default values for parameters which
			-- specify whether and how often to retry "nowait"
			-- selects that fail with "resource busy"
		do
			wait_for_locks := False
			lock_fail_max_retries := 3
			lock_fail_max_wait := Infinity
			lock_fail_max_total_wait := Infinity
			lock_fail_notify_time := Infinity
			lock_fail_constant_coeff := 0.5
			lock_fail_linear_coeff := 0.0
			lock_fail_exponential_coeff := 0.0
			lock_fail_exponential_base := 1.0
		end

feature -- Properties
	
	is_open: BOOLEAN
			-- Is the connection currently open?
	
	user_id: STRING
			-- Oracle user id of current connection.

	password: STRING
			-- Password used to open current connection.

	connect_string: STRING
			-- Connect string used to open current connection.

	printable_connect_string: STRING
			-- Implicit or explicit connect string used to
			-- open current connection, in a format suitable
			-- only for display
		local
			conn_string, str: STRING
			implicit: BOOLEAN
		do
			conn_string := connect_string
			if conn_string /= Void and then conn_string.count > 0 then
				str := conn_string
			else
				implicit := True
				str := get (Connection_env_variable)
			end
			create Result.make (30)
			if str /= Void then
				Result.append_character ('%"')
				Result.append (str)
				Result.append_character ('%"')
				If implicit then
					Result.append (" (implicit)")
				end
			else
				Result.append ("(none)")
			end
		end

	last_error_code: INTEGER
			-- Code of last error which occurred on this connection
	
	last_error_description: STRING
			-- Description of last error which occurred on
			-- this connection.  Empty string if
			-- no error has occurred since last call to
			-- `clear_last_error_description' (or since
			-- connection was created if no such calls)
	
	is_retryable_error: BOOLEAN
			-- Was last error which occurred on this connection
			-- an error which may succeed if it is retried
			-- (e.g., "resource busy" or "invalid rowid")?
		local
			oci: OCI_CONSTANTS
			code: INTEGER
		do
			create oci
			code := last_error_code
			Result := code = oci.Oci_invalid_rowid or
				code = oci.Oci_resource_busy_code
		end
	
	is_unique_constraint_error: BOOLEAN
			-- Was last error which occurred on this connection
			-- a unique constraint violation error?
		
		local
			oci: OCI_CONSTANTS
			code: INTEGER
		do
			create oci
			code := last_error_code
			Result := code = oci.Oci_unique_constraint_code
		end
	
	error_handler: ORACLE_ERROR_HANDLER
			-- Error handler for errors on this connection

	error_checking_enabled: BOOLEAN
			-- Should status returned by database server
			-- for operations on this connection be
			-- checked for errors?
	
	error_display_suppressed: BOOLEAN
			-- Should display of error messages be suppressed
			-- for errors on this connection?  Has no effect
			-- if handler does not display errors.
		do
			Result := error_handler.error_display_suppressed
		end

	status_display_suppressed: BOOLEAN
			-- Should display of status messages be suppressed
			-- for this connection?

feature -- Modification

	set_last_error_code (code: INTEGER)
			-- Set last error code to `code'
		do
			last_error_code := code
		ensure
			code_set: last_error_code = code
		end

	clear_last_error_code
			-- Clear last error code
		do
			last_error_code := Oci_success_code
		end

	clear_last_error_description
			-- Set `last_error_description' to the empty string
		do
			create last_error_description.make (0)
		end

	set_last_error_description (desc: STRING)
			-- Set `last_error_description' to `desc'
		do
			last_error_description := desc
		end

feature {ORACLE_CURSOR} -- Cursor operations

	release_cursor (cursor: ORACLE_CURSOR)
			-- Notify cursor server that cursor `cursor' is
			-- no longer in use.  Cursor server may leave
			-- cursor open and parsed or close it or free it
			-- now or later, at its discretion.
		require
			cursor_exists: cursor /= Void
		do
			cursor_server.release_cursor (cursor)
		end
	
	close_cursor (cursor: ORACLE_CURSOR)
			-- Notify cursor server that cursor `cursor' has
			-- been closed.  Cursor server may keep cursor
			-- around and reopen it later or free it now or later,
			-- at its discretion.
		require
			cursor_exists: cursor /= Void
		do
			cursor_server.close_cursor (cursor)
		end
	
	free_cursor (cursor: ORACLE_CURSOR)
			-- Notify cursor server that cursor `cursor' has
			-- been closed and must be freed now.
		require
			cursor_exists: cursor /= Void
		do
			cursor_server.free_cursor (cursor)
		end
	
feature -- Error handling
	
	set_error_handler (handler: DB_ERROR_HANDLER)
			-- Set error handler for `Current' to `handler'
		local
			h: ORACLE_ERROR_HANDLER
		do
			h ?= handler
			if h /= Void then
				error_handler := h
			else
				raise (dbex.Db_conformance_error)
			end
		end

	set_error_checking_enabled (b: BOOLEAN)
			-- Set `error_checking_enabled' to `b'
		do
			error_checking_enabled := b
		end

	set_error_display_suppressed (b: BOOLEAN)
			-- Set error display suppression for `Current' to `b'
		do
			error_handler.set_error_display_suppressed (b)
		end

	set_status_display_suppressed (b: BOOLEAN)
			-- Set status display suppression for `Current' to `b'
		do
			status_display_suppressed := b
		end

	handle_oci_error (cursor: ORACLE_CURSOR)
			-- Handle an Oracle Call Interface error which
			-- occurred for this connection,
			-- using `cursor' as the cursor involved if non-void
		do
			cursor_server.process_error_event
			dbcm.set_error_connection (Current)
			error_handler.handle_oci_error (Current, cursor)
		end

	handle_oci_error_with_code (cursor: ORACLE_CURSOR code: INTEGER)
			-- Handle an Oracle Call Interface error which
			-- occurred for this connection,
			-- using `cursor' as the cursor involved if non-void
			-- and `code' as the code of the error
		do
			cursor_server.process_error_event
			dbcm.set_error_connection (Current)
			error_handler.handle_oci_error_with_code (Current, cursor, code)
		end

	handle_oracle_user_error (stmt: SQL_STATEMENT except, message: STRING)
			-- Handle an Oracle error which is detected by
			-- us rather than by the Oracle Call Interface,
			-- which is described by `message'.  Raise exception
			-- with tag `except', if appropriate.  Use
			-- `stmt' as the SQL statement involved if non-void.
		local
			cursor: ORACLE_CURSOR
		do
			cursor ?= stmt
			if stmt = Void or cursor /= Void then
				cursor_server.process_error_event
				dbcm.set_error_connection (Current)
				error_handler.handle_oracle_user_error (
					Current, cursor, except, message)
			else
				raise (dbex.Db_conformance_error)
			end
		end

	error_message_for_code (retcode: INTEGER): STRING
			-- Error message corresponding to error code
			-- `retcode'.  If no message corresponds to a code,
			-- Oracle returns a "Message not found" message.
		local
			size, char_count: INTEGER
			not_truncated: BOOLEAN
			area: ANY
		do
			from
				create Result.make (Initial_max_error_message_length)
				size := Result.capacity
			until
				not_truncated
			loop
				Result.resize(size)
				area := Result.area
				enter_oci
				char_count := ora_get_error_message (lda_address, retcode, $area, Result.capacity)
				leave_oci
				strfill.set_count (Result, char_count)
				-- Assume truncation if count = capacity or
				-- count = (capacity - 1)
				not_truncated := Result.count < Result.capacity - 1
				size := size + Error_message_len_incr
			end
		end

feature {NONE} -- Error message lengths
	
	Initial_max_error_message_length: INTEGER = 512
			-- Initial maximum length of error message.

	Error_message_len_incr: INTEGER = 512
			-- Amount to increment max error message length
			-- by if error message is truncated.

feature -- Options
	
	enable_autocommit
			-- Enable automatic commit of every SQL data
			-- manipulation statement.
		local
			code: INTEGER
		do
			if has_dml_access then
				enter_oci
				code := ora_enable_autocommit (lda_address)
				leave_oci
				check_status (code)
			else
				handle_access_error (Void, "Enable autocommit",
					Void)
			end
		end

	disable_autocommit
			-- Disable automatic commit of every SQL data
			-- manipulation statement.  Autocommit is already
			-- disabled at the start of an OCI program,
			-- so this only needs to be used if `enable_autocommit'
			-- has been called for this connection.
		local
			code: INTEGER
		do
			enter_oci
			code := ora_disable_autocommit (lda_address)
			leave_oci
			check_status (code)
		end

feature {NONE} -- Connection control primitives
	
	raw_open (the_user_id, the_password: STRING last: BOOLEAN)
			-- Open connection to Oracle database using
			-- communication via SQL*Net with user id
			-- `the_user_id' and password `the_password'.
			-- The user id should include the connect
			-- string, if any.  If `last' is true, then
			-- raise an exception if open fails.  Set
			-- `is_open' to indicate whether open succeeded
		require
			user_id_exists: the_user_id /= Void
			password_exists: the_password /= Void
			connection_not_already_open: not is_open
		local
			userid_area, password_area: ANY
			code: INTEGER
		do
			userid_area := the_user_id.area
			password_area := the_password.area
			block_interrupts
			code := ora_logon_concurrent (lda_address, hda_address, $userid_area, the_user_id.count, $password_area, the_password.count)
			if code = Oci_success_code then
				is_open := True
				last_error_code := Oci_success_code
			end
			unblock_interrupts
			if last or (code /= Oci_success_code and then not is_retryable_open_error) then
				check_status (code)
			end
		end

	try_to_raw_open (the_user_id, the_password: STRING)
			-- Try to open connection to Oracle database
			-- using communication via SQL*Net with user
			-- id `the_user_id' and password
			-- `the_password'.  The user id should include
			-- the connect string, if any.  Do not raise
			-- an exception if open fails.  Set `is_open'
			-- to indicate whether open succeeded
		require
			user_id_exists: the_user_id /= Void
			password_exists: the_password /= Void
			connection_not_already_open: not is_open
		local
			userid_area, password_area: ANY
			code: INTEGER
		do
			userid_area := the_user_id.area
			password_area := the_password.area
			block_interrupts
			code := ora_logon_concurrent (lda_address, hda_address, $userid_area, the_user_id.count, $password_area, the_password.count)
			if code = Oci_success_code then
				is_open := True
				last_error_code := Oci_success_code
			else
				last_error_code := return_code
			end
			unblock_interrupts
			if code /= Oci_success_code then
				set_last_error_description (error_handler.oci_error_description (Current, Void))
			end
		end

	is_retryable_open_error: BOOLEAN
			-- Was last error which occurred on this
			-- connection a connection open error which
			-- may succeed if it is retried (e.g., not
			-- "invalid username/password")?
		local
			oci: OCI_CONSTANTS
			code: INTEGER
		do
			create oci
			code := return_code
			Result := code /= oci.Oci_invalid_username_code and
				code /= oci.Oci_no_default_username_code and
				code /= oci.Oci_null_password_code
		end
	
	raw_close
			-- Do a rollback and then close the Oracle database
			-- connection
		require
			connection_open: is_open
		local
			code: INTEGER
		do
			rollback
			enter_oci
			code := ora_logoff (lda_address)
			if code = Oci_success_code then
				is_open := False
			end
			leave_oci
			check_status (code)
			is_open := False
		ensure
			connection_closed: not is_open
		end

	send_logoff_notification
			-- Notify cursor server that this connection to
			-- database is closed
		do
			cursor_server.process_logoff_event
		end

feature -- Connection control
	
	open (a_user_id, a_password, a_connect_string: STRING max_tries: INTEGER)
			-- Open a new connection using communication
			-- via SQL*Net.  Connect to Oracle with user
			-- id `a_user_id', password `a_password' and
			-- connect string `a_connect_string'.  The
			-- connect string must not include the leading
			-- delimiting "@" character or the open will
			-- fail.  If open fails with a retryable
			-- error, keep trying to connect up to
			-- `max_tries' times
		local
			both: STRING
			k, max_attempts: INTEGER
		do
			create both.make (a_user_id.count + a_connect_string.count
				+ 1)
			both.append (a_user_id)
			if a_connect_string.count > 0 then
				both.append_character (Connect_string_delimiter)
				both.append (a_connect_string)
			end
			user_id := a_user_id.twin
			password := a_password.twin
			connect_string := a_connect_string.twin
			if max_tries = 0 then
				max_attempts := Default_max_open_attempts
			else
				max_attempts := max_tries
			end
			from
				k := 1
			until
				is_open or k > max_attempts
			loop
				if k > 1 then
					sleep_seconds (1)
				end
				raw_open (both, a_password, k = max_attempts)
				k := k + 1
			end
		end

	try_to_open (a_user_id, a_password, a_connect_string: STRING max_tries: INTEGER)
			-- Try to open a new connection using communication
			-- via SQL*Net.  Connect to Oracle with user
			-- id `a_user_id', password `a_password' and
			-- connect string `a_connect_string'.  The
			-- connect string must not include the leading
			-- delimiting "@" character or the open will
			-- fail.  If open fails with a retryable
			-- error, keep trying to connect up to
			-- `max_tries' times.  Do not raise an exception
			-- if open fails.  Set `is_open' to indicate
			-- whether open was successful
		local
			both: STRING
			k, max_attempts: INTEGER
		do
			create both.make (a_user_id.count + a_connect_string.count
				+ 1)
			both.append (a_user_id)
			if a_connect_string.count > 0 then
				both.append_character (Connect_string_delimiter)
				both.append (a_connect_string)
			end
			user_id := a_user_id.twin
			password := a_password.twin
			connect_string := a_connect_string.twin
			if max_tries = 0 then
				max_attempts := Default_max_open_attempts
			else
				max_attempts := max_tries
			end
			from
				k := 1
			until
				is_open or k > max_attempts or (k > 1 and then not is_retryable_open_error)
			loop
				if k > 1 then
					sleep_seconds (1)
				end
				try_to_raw_open (both, a_password)
				k := k + 1
			end
		end

	close
			-- Close the current connection.
		do
			if is_changed then
				handle_uncommitted_changes_error ("logoff", Void)
			end
			send_logoff_notification
			registered_signals.wipe_out
			raw_close
			user_id := Void
			password := Void
			connect_string := Void
		end

	abort_oci_function
			-- Abort any currently executing OCI function.
			-- Do not use when connection opening is in progress.
			-- Not available on all operating systems.
		local
			code: INTEGER
		do
			enter_oci
			code := ora_abort_oci_function (lda_address)
				-- Ignore return code
			leave_oci
		end

feature -- Access control
		
	set_query_access (value: BOOLEAN)
		-- Allow queries if `value' is true; disallow them if it
		-- is false.
		do
			if is_changed then
				handle_access_change_error ("query")
			end
			has_query_access := value
		end
	
	set_dml_access (value: BOOLEAN)
		-- Allow DML statements if `value' is true; disallow them
		-- if it is false
		do
			if is_changed then
				handle_access_change_error ("DML")
			end
			has_dml_access := value
		end
	
	set_ddl_access (value: BOOLEAN)
		-- Allow DDL statements if `value' is true; disallow
		-- them if it is false
		do
			if is_changed then
				handle_access_change_error ("DDL")
			end
			has_ddl_access := value
		end
	
	set_block_access (value: BOOLEAN)
		-- Allow blocks of statements (e.g., PL/SQL blocks) if
		-- `value' is true; disallow them if it is false
		do
			if is_changed then
				handle_access_change_error ("PL/SQL block")
			end
			has_block_access := value
		end
	
	
	set_no_access
		-- Allow no access at all
		do
			if is_changed then
				handle_access_change_error ("no")
			end
			has_query_access := False
			has_dml_access := False
			has_ddl_access := False
			has_block_access := False
		end
	
	set_read_access
		-- Allow queries only
		do
			if is_changed then
				handle_access_change_error ("read")
			end
			has_query_access := True
			has_dml_access := False
			has_ddl_access := False
			has_block_access := False
		end
	
	set_write_access
		-- Allow queries and DML statements only
		do
			if is_changed then
				handle_access_change_error ("write")
			end
			has_query_access := True
			has_dml_access := True
			has_ddl_access := False
			has_block_access := False
		end
	
	set_write_and_block_access
		-- Allow queries, DML statements and blocks of statements
		-- (e.g., PL/SQL blocks) only
		do
			if is_changed then
				handle_access_change_error ("write and block")
			end
			has_query_access := True
			has_dml_access := True
			has_block_access := True
			has_ddl_access := False
		end
	
	set_alter_access
		-- Allow queries, DML statements and DDL statements only
		do
			if is_changed then
				handle_access_change_error ("alter")
			end
			has_query_access := True
			has_dml_access := True
			has_ddl_access := True
			has_block_access := False
		end
	
	set_all_access
		-- Allow queries, DML statements, DDL statements and
		-- blocks of statements (no access restrictions)
		do
			if is_changed then
				handle_access_change_error ("all")
			end
			has_query_access := True
			has_dml_access := True
			has_ddl_access := True
			has_block_access := True
		end
	
    	has_query_access: BOOLEAN
		-- Are query statements allowed?

    	has_dml_access: BOOLEAN
		-- Are DML statements allowed?
	
    	has_block_access: BOOLEAN
		-- Are blocks of statements (e.g. PL/SQL blocks) allowed?
	
    	has_ddl_access: BOOLEAN
		-- Are DDL statements allowed?

	access_level: STRING
		-- Access allowed for this connection
		do
			create Result.make (0)
			if has_query_access then
				Result.append ("query ")
			end
			if has_dml_access then
				Result.append ("DML ")
			end
			if has_block_access then
				Result.append ("PL/SQL ")
			end
			if has_ddl_access then
				Result.append ("DDL ")
			end
			Result.right_adjust
			if Result.count = 0 then
				Result.append ("none")
			end
		end

	check_access_restrictions (sql_stmt: STRING stmt_type: INTEGER)
			-- Check that the type of `sql_stmt' (to be
			-- parsed), which is `stmt_type', is allowed
			-- by current access level.  If not, raise an
			-- exception.
		do
			inspect stmt_type
			when Query_statement then
				if not has_query_access then
					handle_access_error (Void, "Query", sql_stmt)
				end
			when Dml_statement then
				if not has_dml_access then
					handle_access_error (Void, "DML statement", sql_stmt)
				end
			when Ddl_statement then
				if not has_ddl_access then
					handle_access_error (Void, "DDL statement", sql_stmt)
				end
			when Block_statement then
				if not has_block_access then
					handle_access_error (Void, "PL/SQL block", sql_stmt)
				end
			when Illegal_statement then
				handle_illegal_statement_error (sql_stmt)
			when Unknown_statement then
				handle_unknown_statement_error (sql_stmt)
			end
		end
	
feature -- Event notification

	signal_elapsed_time
			-- Notify connection that some time has elapsed,
			-- allowing it to take any appropriate time-dependent
			-- actions
		do
			cursor_server.process_elapsed_time_event
		end

feature -- Validity

	is_query_statement (stmt: STRING): BOOLEAN
			-- Does `stmt' represent a query or query-related
			-- operation?
		do
			Result := statement_type (stmt) = Query_statement
		end

	is_dml_statement (stmt: STRING): BOOLEAN
			-- Does `stmt' represent a DML statement?
		do
			Result := statement_type (stmt) = Dml_statement
		end

	is_ddl_statement (stmt: STRING): BOOLEAN
			-- Does `stmt' represent a DDL statement?
		do
			Result := statement_type (stmt) = Ddl_statement
		end

	is_block_statement (stmt: STRING): BOOLEAN
			-- Does `stmt' represent a block of statements (e.g.,
			-- PL/SQL block)?
		do
			Result := statement_type (stmt) = Block_statement
		end

feature {NONE} -- Allocation

	raw_parsed_cursor (sql_stmt: STRING stmt_type: INTEGER): ORACLE_CURSOR
			-- An open and parsed Oracle cursor for SQL statement
			-- `sql_stmt', which is of type `stmt_type'
		require
			statement_exists: sql_stmt /= Void
			valid_statement_type: valid_statement_type (stmt_type)
		do
			if stmt_type = Ddl_statement then
				start_transaction
			end
			Result := cursor_server.parsed_cursor (sql_stmt)
			if stmt_type = Ddl_statement then
				start_transaction
			end
			Result.set_statement_type (stmt_type)
		end
	
feature -- Allocation

	parsed_cursor (sql_stmt: STRING): ORACLE_CURSOR
			-- An open and parsed Oracle cursor for SQL statement
			-- `sql_stmt'
		local
			stmt_type: INTEGER
		do
			stmt_type := statement_type (sql_stmt)
			check_access_restrictions (sql_stmt, stmt_type)
			if stmt_type = Ddl_statement and is_changed then
				handle_uncommitted_changes_error ("DDL statement", sql_stmt)
			end
			Result := raw_parsed_cursor (sql_stmt, stmt_type)
		end
	
	parsed_dml_cursor (sql_stmt: STRING): ORACLE_CURSOR
			-- An open and parsed Oracle cursor for SQL statement
			-- `sql_stmt'.  Raises an exception if `sql_stmt'
			-- is not a Data Manipulation Language (DML) statement
		local
			stmt_type: INTEGER
		do
			stmt_type := statement_type (sql_stmt)
			if stmt_type /= Dml_statement then
				handle_wrong_statement_type (sql_stmt, Dml_statement, stmt_type)
			end
			if not has_dml_access then
				handle_access_error (Void, "DML statement",
					sql_stmt)
			end
			Result := raw_parsed_cursor (sql_stmt, stmt_type)
			if not Result.consistent_with_dml_statement then
				handle_sql_code_inconsistency (Result, "DML statement")
			end
		end
	
	parsed_block_cursor (sql_stmt: STRING): ORACLE_CURSOR
			-- An open and parsed Oracle cursor for PL/SQL block
			-- `sql_stmt'.  Raises an exception if `sql_stmt'
			-- is not a PL/SQL block
		local
			stmt_type: INTEGER
		do
			stmt_type := statement_type (sql_stmt)
			if stmt_type /= Block_statement then
				handle_wrong_statement_type (sql_stmt, Block_statement, stmt_type)
			end
			if not has_block_access then
				handle_access_error (Void, "PL/SQL block",
					sql_stmt)
			end
			Result := raw_parsed_cursor (sql_stmt, stmt_type)
			if not Result.consistent_with_block_statement then
				handle_sql_code_inconsistency (Result, "PL/SQL block")
			end
		end
	
	parsed_query_cursor (sql_stmt: STRING): ORACLE_CURSOR
			-- An open and parsed Oracle cursor for SQL statement
			-- `sql_stmt'.  Raises an exception if `sql_stmt'
			-- is not a query
		local
			stmt_type: INTEGER
		do
			stmt_type := statement_type (sql_stmt)
			if stmt_type /= Query_statement then
				handle_wrong_statement_type (sql_stmt, Query_statement, stmt_type)
			end
			if not has_query_access then
				handle_access_error (Void, "Query", sql_stmt)
			end
			Result := raw_parsed_cursor (sql_stmt, stmt_type)
			if not Result.consistent_with_query then
				handle_sql_code_inconsistency (Result, "query")
			end
		end
	
	sequence_current_value (seq_name: STRING): INTEGER
			-- Current value of sequence `seq_name'
		do
			Result := sequence_value (Sequence_curr_pseudo_col,
				seq_name)
		end

	sequence_next_value (seq_name: STRING): INTEGER
			-- Next value of sequence `seq_name'.  Each call
			-- allocates a new sequence number and changes
			-- sequence's current value
		do
			if not has_dml_access then
				handle_access_error (Void,
					"Getting sequence nextval", Void)
			end
			Result := sequence_value (Sequence_next_pseudo_col,
				seq_name)
		end
	
	sequence_value (which, seq_name: STRING): INTEGER
			-- The `which' value of sequence `seq_name'
			-- (either "currval" or "nextval")
		require
			value_type_exists: which /= Void
			sequence_name_exists: seq_name /= Void
		local
			c: ORACLE_CURSOR
			stmt: STRING
		do
			create stmt.make (60)
			stmt.append (Select_keyword)
			stmt.append (seq_name)
			stmt.extend ('.')
			stmt.append (which)
			stmt.append (From_dual_keyword)
			c := parsed_query_cursor (stmt)
			c.set_select_count (1)
			c.select_integer (1)
			c.execute
			c.fetch
			Result := c.integer_value (1)
			c.release
		end

	user_name: STRING
			-- The user name under which this connection
			-- is open, according to the database.
			-- May differ from `user_id' in the case of
			-- default logons
		local
			c: ORACLE_CURSOR
		do
			c := parsed_query_cursor (User_name_query)
			c.set_select_count (1)
			c.select_string (1, Max_user_name_length)
			c.execute
			c.fetch
			Result := c.string_value (1)
			c.release
		end
	
	system_date: ORACLE_DATE
			-- The current system date on the database server
			-- for this connection
		local
			c: ORACLE_CURSOR
		do
			c := parsed_query_cursor (System_date_query)
			c.set_select_count (1)
			c.select_date (1)
			c.execute
			c.fetch
			Result := c.date_value (1)
			c.release
		end
	
	date (s: STRING): ORACLE_DATE
			-- The date represented by `s', which must be
			-- in the format "DD-MON-YYYY" (e.g. "03-MAR-1995").
			-- The DD and YYYY fields may contain fewer digits
			-- than the number indicated in this format.
			-- The time is set to midnight (00:00:00).
		do
			create Result.set_date (s)
		end
	
	date_and_time (s: STRING): ORACLE_DATE
			-- The date and time represented by `s', which
			-- must be in the format "DD-MON-YYYY
			-- HH:MM:SS" (e.g. "03-MAR-1995 15:37:42").
			-- The DD, YYYY, HH, MM and SS fields may
			-- contain fewer digits than the number
			-- indicated in this format.
		do
			create Result.set_date_and_time (s)
		end
	
	default_date: ORACLE_DATE
			-- A database date of the proper dynamic type for
			-- this connection.  Useful for doing queries
			-- and filling in the same date variable repeatedly
		do
			create Result.set_zero
		end
	
	null_date: ORACLE_DATE
			-- A database date which is null
		do
			create Result.set_null
		end
	
	default_rowid: ORACLE_ROWID
			-- A database rowid of the proper dynamic type for
			-- this connection.  Useful for doing queries
			-- and filling in the same rowid variable repeatedly
		do
			create Result.set_null
		end
	
feature -- Execution

	execute_sql_define (stmt: STRING)
			-- Execute SQL string `stmt',
			-- which is a Data Definition Language (DDL) statement.
		local
			cursor: ORACLE_CURSOR
			stmt_type: INTEGER
		do
			stmt_type := statement_type (stmt)
			if stmt_type /= Ddl_statement then
				handle_wrong_statement_type (stmt, Ddl_statement, stmt_type)
			end
			if not has_ddl_access then
				handle_access_error (Void, "DDL statement",
					stmt)
			end
			if is_changed then
				handle_uncommitted_changes_error ("DDL statement", stmt)
			end
			cursor := raw_parsed_cursor (stmt, stmt_type)
			if not cursor.consistent_with_ddl_statement then
				handle_sql_code_inconsistency (cursor, "DDL statement")
			end
			cursor.close
		end
	
feature -- Database signals (alerts)

	send_signals
			-- Send any queued signals (alerts).  Note that
			-- the signals won't actually be sent until
			-- a commit is done
		local
			sig: DB_SIGNAL
			signals: LIST [DB_SIGNAL]
		do
			from
				signals := queued_signals
				signals.start
			until
				signals.after
			loop
				sig := signals.item
				send_signal (sig.name, sig.message)
				signals.forth
			end
		end

feature -- Transaction properties
	
	is_changed: BOOLEAN
			-- Have any changes been made to the database
			-- through this connection (posted or unposted)
			-- since the start of the current transaction?
		do
			Result := transaction.is_changed
		end
	
	has_unposted_changes: BOOLEAN
			-- Have any changes been made to the database
			-- through this connection since the start of the
			-- current transaction which have not yet
			-- been posted?
		do
			Result := transaction.has_registered_items
		end
	
	transaction_number: INTEGER
			-- Number of current transaction
		do
			Result := transaction.number
		end

	transaction_date: ORACLE_DATE
			-- The date associated with current transaction
			-- (obtained from the database server once
			-- per transaction)
		do
			if transaction.date = Void then
				transaction.set_date (system_date)
			end
			Result := transaction.date
		end
	
feature -- Transaction control
	
	start_transaction
			-- Start a new transaction
		do
			transaction.start
		end

	register (item: POSTABLE)
			-- Register `item' to be posted when
			-- `post' is called on this connection
		do
			if not has_dml_access then
				handle_access_error (Void, "Item registration",
					Void)
			end
			transaction.register (item)
		end

	reregister_first (item: POSTABLE)
			-- Re-register already registered `item' as first
			-- item in registration list
		do
			if not has_dml_access then
				handle_access_error (Void, "Item re-registration",
					Void)
			end
			transaction.reregister_first (item)
		end

	reregister_last (item: POSTABLE)
			-- Re-register already registered `item' as last
			-- item in registration list
		do
			if not has_dml_access then
				handle_access_error (Void, "Item re-registration",
					Void)
			end
			transaction.reregister_last (item)
		end

	set_item_rowid (item: POSTABLE)
			-- Associate rowid with `item' after insert
		do
			transaction.set_item_rowid (item)
		end

	deregister (item: POSTABLE)
			-- De-register already registered `item'
		do
			transaction.deregister (item)
		end

	rollback_transaction
			-- Roll back transaction
		do
			transaction.rollback
		ensure
			no_unposted_changes: not has_unposted_changes
		end

	commit_transaction
			-- Commit transaction
		do
			transaction.commit
		end

	post
			-- Post all registered items
		do
			if not has_dml_access then
				handle_access_error (Void, "Post", Void)
			end
			transaction.post
		end

	commit
			-- Commit the current transaction.
		local
			code: INTEGER
		do
			if not has_dml_access then
				handle_access_error (Void, "Commit", Void)
			end
			post
			send_signals
			enter_oci
			code := ora_commit (lda_address)
			if code = Oci_success_code then
				queued_signals.wipe_out
				commit_transaction
				start_transaction
				if flush_cache_after_transaction then
					flush_cursor_cache
				end
			end
			leave_oci
			check_status (code)
		end

	savepoint (name: STRING)
			-- Create a savepoint named `name'
		local
			c: ORACLE_CURSOR
			stmt: STRING
		do
			if not has_dml_access then
				handle_access_error (Void, "Savepoint", Void)
			end
			post
			stmt := "savepoint "
			stmt.append (name)
			c := raw_parsed_cursor (stmt, Dml_statement)
			if not c.consistent_with_dml_statement then
				handle_sql_code_inconsistency (c, "DML statement")
			end
			c.execute
			c.close
			start_transaction
		end

	rollback
			-- Rollback the current transaction.
		local
			code: INTEGER
		do
			queued_signals.wipe_out
			enter_oci
			code := ora_rollback (lda_address)
			if code = Oci_success_code then
				rollback_transaction
				start_transaction
				if flush_cache_after_transaction then
					flush_cursor_cache
				end
			end
			leave_oci
			check_status (code)
		end

	rollback_to_savepoint (name: STRING)
			-- Rollback to savepoint named `name', which must
			-- have been previously created in this transaction
		local
			c: ORACLE_CURSOR
			stmt: STRING
		do
			if not has_dml_access then
				handle_access_error (Void, "Rollback", Void)
			end
			rollback_transaction
			stmt := "rollback to savepoint "
			stmt.append (name)
			c := raw_parsed_cursor (stmt, Dml_statement)
			if not c.consistent_with_dml_statement then
				handle_sql_code_inconsistency (c, "DML statement")
			end
			c.execute
			c.close
			start_transaction
		end

	set_transaction_read_only
			-- Set the current transaction to be read only,
			-- so that all queries see a consistent view
			-- of table contents
		local
			c: ORACLE_CURSOR
			stmt: STRING
		do
			stmt := "set transaction read only"
			if not has_query_access then
				handle_access_error (Void, "Set transaction read only", Void)
			end
			if is_changed then
				handle_uncommitted_changes_error ("DML statement", stmt)
			end
			c := raw_parsed_cursor (stmt, Dml_statement)
			if not c.consistent_with_dml_statement then
				handle_sql_code_inconsistency (c, "DML statement")
			end
			c.execute
			c.close
			start_transaction
		end

feature -- Session control

	set_role (roles: STRING)
			-- Set roles for current session to `roles'
		local
			c: ORACLE_CURSOR
			stmt: STRING
		do
			stmt := "set role "
			stmt.append (roles)
			if not has_query_access then
				handle_access_error (Void, "Set role", stmt)
			end
			c := raw_parsed_cursor (stmt, Dml_statement)
			if not c.consistent_with_dml_statement then
				handle_sql_code_inconsistency (c, "DML statement")
			end
			c.execute
			c.close
		end

	alter_session (cmd: STRING)
			-- Alter session as indicated by `cmd'
		local
			c: ORACLE_CURSOR
			stmt: STRING
		do
			stmt := "alter session "
			stmt.append (cmd)
			if not has_query_access then
				handle_access_error (Void, "Alter session", stmt)
			end
			c := raw_parsed_cursor (stmt, Dml_statement)
			if not c.consistent_with_dml_statement then
				handle_sql_code_inconsistency (c, "DML statement")
			end
			c.execute
			c.close
		end

feature -- Tunable properties
	
	max_parsed_cursors: INTEGER
			-- Maximum number of open and parsed cursors for SQL
			-- statements to keep
		do
			Result := cursor_server.max_parsed_cursors
		end

	parsed_cursor_age_limit: INTEGER
			-- Open and parsed cursor with age in seconds at
			-- or above this limit will be closed next time
			-- cursor server has control
		do
			Result := cursor_server.parsed_age_limit
		end

	flush_cache_after_transaction: BOOLEAN
			-- Should the SQL statement cache be flushed at
			-- the end of each transaction?  If yes, the
			-- cache is flushed after the commit or rollback
			-- operation completes.
	
	query_buffer_size_limit: INTEGER
			-- Requested maximum size of buffer to fetch
			-- query results.  This is only a hint, so
			-- the buffer size may be larger.
			-- Zero means no limit.
	
	query_rows_per_fetch: INTEGER
			-- Requested number of rows to fetch from database
			-- engine each time a fetch is needed.  Zero means
			-- use `query_buffer_size_limit' to determine
			-- the number of rows

feature -- Performance tuning
	
	set_max_parsed_cursors (num: INTEGER)
			-- Set `max_parsed_cursors' to `num'
		do
			cursor_server.set_max_parsed_cursors (num)
		end

	set_parsed_cursor_age_limit (num: INTEGER)
			-- Set `parsed_cursor_age_limit' to `num'
		do
			cursor_server.set_parsed_age_limit (num)
		end

	set_flush_cache_after_transaction (value: BOOLEAN)
			-- Set `flush_cache_after_transaction' to `value'
		do
			flush_cache_after_transaction := value
		end

	flush_cursor_cache
			-- Flush from the cursor cache any cursors which
			-- are not reserved
		do
			cursor_server.flush_cursor_cache
		end

	set_query_buffer_size_limit (value: INTEGER)
			-- Set `query_buffer_size_limit' to `value'
		do
			query_buffer_size_limit := value
		end
	
	set_query_rows_per_fetch (value: INTEGER)
			-- Set `query_rows_per_fetch' to `value'
		do
			query_rows_per_fetch := value
		end
	
feature -- Debugging

	cached_open_cursor_count: INTEGER
			-- Number of cursors that are open and cached
			-- within cursor server for future use
		do
			Result := cursor_server.cached_open_cursor_count
		end

	cached_parsed_cursor_count: INTEGER
			-- Number of cursors that are open and parsed
			-- and cached within cursor server for future use
		do
			Result := cursor_server.cached_parsed_cursor_count
		end

	available_cursor_count: INTEGER
			-- Number of additional cursors that could be
			-- opened before running into database
			-- server's limit
		do
			Result := cursor_server.available_cursor_count
		end

	reserved_cursor_count: INTEGER
			-- Number of cursors that have been reserved
			-- but not yet released, closed or freed
			-- (i.e., the number of cursors still in use).
		do
			Result := cursor_server.reserved_cursor_count
		end
	
	keep_reserved_cursor_list: BOOLEAN
			-- Should reserved cursors be kept on a list
			-- until they are released, closed or freed?
			-- If so, reserved cursors can never become
			-- garbage so they won't be closed if client
			-- loses track of them
		do
			Result := cursor_server.keep_reserved_cursor_list
		end

	reserved_cursors_description: STRING
			-- Text of the SQL statements of all reserved
			-- cursors.  If `keep_reserved_cursor_list'
			-- is false, will only tell number of reserved
			-- statements
		do
			Result := cursor_server.reserved_cursors_description
		end

	set_keep_reserved_cursor_list (b: BOOLEAN)
			-- Set `keep_reserved_cursor_list' to `b'
		do
			cursor_server.set_keep_reserved_cursor_list (b)
		end

feature -- Status check
	
	check_status (code: INTEGER)
			-- Check that return status `code' from OCI routine is
			-- OK and call error handler if not
		 do
			if code /= Oci_success_code and then error_checking_enabled then
				last_error_code := return_code
				handle_oci_error (Void)
			end
		 end
	
feature -- Interrupt handling
	
	enter_oci
			-- Take actions needed when about to enter
			-- an Oracle Call Interface routine
		do
			-- FIXME: this won't be necessary in 5.2
			-- since ISE will then no longer re-establish
			-- its own interrupt handler and override ours
			-- set_interrupt_handler
			set_lda (lda_address)
		end

	leave_oci
			-- Take actions needed when we have just left
			-- an Oracle Call Interface routine
		do
			clear_lda
			if interrupt_received then
				clear_interrupt_received
				unix_os_send_signal (Sigint, my_process_id)
			end
		end

feature {NONE} -- Cleanup

	dispose
			-- Return memory allocated for LDA and HDA
		local
			null_ptr: POINTER
			code: INTEGER
		do
			if is_open then
				enter_oci
				code := ora_abort_oci_function (lda_address)
				code := ora_rollback (lda_address)
				remove_oracle_lda (lda_address)
				code := ora_logoff (lda_address)
					-- Attempt logoff even if rollback
					-- fails.  Do not raise an exception,
					-- which would mess up the garbage
					-- collector
				is_open := False
				leave_oci
			end
			if lda_address /= null_ptr then
				free_nongc_memory (lda_address)
				lda_address := null_ptr
			end
			if hda_address /= null_ptr then
				free_nongc_memory (hda_address)
				hda_address := null_ptr
			end
		end

	Connection_close_failed_exception: POINTER
			-- Address of null terminated string which serves
			-- as tag for a "Connection_close_failed" exception
		external
			"C macro use %"oracle.h%""
		end
	
feature {NONE} -- Externals

	override_interrupt_handler
			-- Override interrupt handler provided by ISE
			-- with our own handler that will work
			-- properly with OCI, if not already done
		once
			set_interrupt_handler
		end

	set_interrupt_handler
			-- Set our own interrupt handler which overrides
			-- the one provided by ISE.  Our handler will
			-- work properly with OCI
		external
			"C"
		end

	set_lda (addr: POINTER)
			-- Set address of LDA for connection for which
			-- we're about to enter an OCI routine
		external
			"C"
		end

	clear_lda
			-- Clear address of LDA for connection for which
			-- we're in an OCI routine, to indicate that we
			-- are no longer in an OCI routine
		external
			"C"
		end

	interrupt_received: BOOLEAN
			-- Has an interrupt signal been received
			-- while in an OCI routine?
		external
			"C"
		end

	clear_interrupt_received
			-- Clear "interrupt signal received" indicator
		external
			"C"
		end

	block_interrupts
			-- Block delivery of interrupts
		external
			"C"
		end
	
	unblock_interrupts
			-- Unblock delivery of interrupts
		external
			"C"
		end
	
invariant
	cursor_server_exists: cursor_server /= Void
	temp_buffer_exists: temp_buffer /= Void

end
