
indexing
	
	author: "David Hollenberg"
	date:	"January 25, 1996"
	description: "An Oracle describe item for a selection item a query"

class ORACLE_DESCRIBE_ITEM

inherit
	QUERY_SELECT_ITEM
	ORACLE_DATATYPE_CODES
		export
			{NONE} all
		end
	ORACLE_VALIDITY
		export
			{ANY} is_valid_select_position
			{NONE} all
		end
	ORACLE_CALL_INTERFACE
		export
			{NONE} all
		end
	C_MEMORY_ALLOCATION
		export
			{NONE} all
		end
	MEMORY
		export
			{NONE} all
		redefine
			dispose
		end
	STRING_UTILITIES_ACCESS
		export
			{NONE} all
		end
	STRING_FILLER_ACCESS
		export
			{NONE} all
		end
	ANY

create
	make

feature -- Creation

	make
			-- Allocate memory for Cursor Data Area and
			-- initialize it
		do
			address := allocate_nongc_memory (length)
		end
	
	set_address (addr: POINTER)
			-- Set address of structure describing describe item
			-- to `addr'
		require
			valid_address: addr /= default_pointer
		do
			address := addr
		ensure
			address_set: address = addr
		end

feature -- Properties

	address: POINTER
			-- Address of structure describing select item

	position: INTEGER
			-- Position of select item for which `Current'
			-- is a description
		do
			Result := des_position (address)
		end
	
	max_dict_size: INTEGER
			-- Maximum size of select item as stored in
			-- Oracle data dictionary
		do
			Result := des_max_dict_size (address)
		end
	
	max_display_size: INTEGER
			-- Maximum display size of select item if it is
			-- returned as a character string
		do
			Result := des_max_display_size (address)
		end
	
	internal_datatype: INTEGER
			-- Internal datatype code of select item
		do
			Result := des_internal_datatype (address)
		ensure
			valid_datatype: is_valid_internal_datatype (Result)
		end
	
	precision: INTEGER
			-- Precision of numeric select item
		do
			Result := des_precision (address)
		end
	
	scale: INTEGER
			-- Scale of numeric select item
		do
			Result := des_scale (address)
		end
	
	null_allowed: BOOLEAN
			-- Is null possible for select item?
		do
			Result := des_null_allowed (address) /= 0
		end
	
	name_length: INTEGER
			-- Length of buffer for name for select item on input
			-- Length of name for select item on output
		do
			Result := des_name_length (address)
		ensure
			nonnegative_result: Result >= 0
		end
	
	position_address: POINTER
			-- Address of select item position in `Current'
		do
			Result := des_position_addr (address)
		end
	
	max_dict_size_address: POINTER
			-- Address of maximum dictionary size field
		do
			Result := des_max_dict_size_addr (address)
		end
	
	max_display_size_address: POINTER
			-- Address of maximum display size field
		do
			Result := des_max_display_size_addr (address)
		end
	
	internal_datatype_address: POINTER
			-- Address of internal datatype field
		do
			Result := des_internal_datatype_addr (address)
		end
	
	precision_address: POINTER
			-- Address of precision field
		do
			Result := des_precision_addr (address)
		end
	
	scale_address: POINTER
			-- Address of scale field
		do
			Result := des_scale_addr (address)
		end
	
	null_allowed_address: POINTER
			-- Address of null allowed field
		do
			Result := des_null_allowed_addr (address)
		end
	
	name_length_address: POINTER
			-- Address of name length field
		do
			Result := des_name_length_addr (address)
		end
	
	name_address: POINTER
			-- Address of buffer for select item name
		do
			Result := des_name_addr (address)
		end
	
	name: STRING
			-- New string with select item name
		local
			len: INTEGER
		do
			len := name_length
			create Result.make (len)
			strfill.fill_string (Result, name_address, len)
		end

	return_code: INTEGER
			-- OCI return code from last call to
			-- `describe_without_error'

	is_number: BOOLEAN
			-- Is item internal datatype NUMBER?
		do
			Result := internal_datatype = number_internal_type
		end
	
	is_date: BOOLEAN
			-- Is item internal datatype DATE?
		do
			Result := internal_datatype = date_internal_type
		end
	
	is_rowid: BOOLEAN
			-- Is item internal datatype ROWID?
		do
			Result := internal_datatype = rowid_internal_type
		end
	
	is_mlslabel: BOOLEAN
			-- Is item internal datatype MLSLABEL?
		do
			Result := internal_datatype = mlslabel_internal_type
		end
	
	is_varchar2: BOOLEAN
			-- Is item internal datatype VARCHAR2?
		do
			Result := internal_datatype = varchar2_internal_type
		end
	
	is_long: BOOLEAN
			-- Is item internal datatype LONG?
		do
			Result := internal_datatype = long_internal_type
		end
	
	is_raw: BOOLEAN
			-- Is item internal datatype RAW?
		do
			Result := internal_datatype = raw_internal_type
		end
	
	is_long_raw: BOOLEAN
			-- Is item internal datatype LONG RAW?
		do
			Result := internal_datatype = long_raw_internal_type
		end
	
	is_char: BOOLEAN
			-- Is item internal datatype CHAR?
		do
			Result := internal_datatype = char_internal_type
		end
	
	is_string_type: BOOLEAN
			-- Is item internal datatype a "string" datatype
			-- (fixed length or variable length, long or not,
			-- raw or not)?
		do
			Result := is_varchar2 or is_long or is_raw or is_long_raw or is_char
		end
	
	is_long_type: BOOLEAN
			-- Is item internal datatype a "long string" datatype
			-- (raw or not)?
		do
			Result := is_long or is_long_raw
		end

	is_raw_type: BOOLEAN
			-- Is item internal datatype a "raw string" datatype
			-- (long or not)?
		do
			Result := is_raw or is_long_raw
		end
	
feature -- Modification

	set_position (k: INTEGER)
			-- Set position of `Current' in select clause to `k'
		require
			valid_select_position: is_valid_select_position (k)
		do
			des_set_position (address, k)
		ensure
			position_set: position = k
		end
	
	set_name_length (len: INTEGER)
			-- Set length of name buffer to `len'
		require
			valid_length: len >= 0
		do
			des_set_name_length (address, len)
		ensure
			name_length_set: name_length = len
		end
	
	describe (c: ORACLE_CURSOR k: INTEGER)
			-- Update `Current' with the description of
			-- the select item at position `k' for cursor `c'
		require
			cursor_exists: c /= Void
			valid_select_position: is_valid_select_position (k)
		do
			describe_without_error (c, k)
			c.check_status (return_code)
		ensure
			position_set: position = k
		end
	
	describe_without_error (c: ORACLE_CURSOR k: INTEGER)
			-- Update `Current' with the description of
			-- the select item at position `k' for cursor `c'.
			-- Do not call the error handler if there is
			-- an error.  Leave the return code in `return_code'.
		require
			cursor_exists: c /= Void
			valid_select_position: is_valid_select_position (k)
		do
			set_position (k)
			set_name_length (max_name_length)
			c.enter_oci
			return_code := ora_describe_select_item (c.address, k,
				max_dict_size_address,
				internal_datatype_address,
				name_address,
				name_length_address,
				max_display_size_address,
				precision_address,
				scale_address,
				null_allowed_address)
			c.leave_oci
		ensure
			position_set: position = k
		end
	
feature -- Display

	display
			-- Display `Current'
		local
			s: STRING
			temp: STRING
		do
			create s.make (100)
			s.append ("    ")
			create temp.make (10)
			temp.append_integer (position)
			su.right_pad (temp, 3)
			s.append (temp)
			s.append ("   ")
			temp := name
			su.right_pad (temp, 30)
			s.append (temp)
			s.append ("   ")
			temp := name_for_internal_code (internal_datatype)
			temp.append_character ('(')
			temp.append_integer (max_dict_size)
			temp.append_character (')')
			su.right_pad (temp, 14)
			s.append (temp)
			s.append ("   ")
			s.append_integer (max_display_size)
			if is_number then
				s.append ("  prec = ")
				s.append_integer (precision)
				s.append ("  scale = ")
				s.append_integer (scale)
			end
			io.put_string (s)
		end

feature {NONE} -- Cleanup

	dispose
			-- Return memory allocated for `Current'
		local
			null_ptr: POINTER
		do
			if address /= null_ptr then
				free_nongc_memory (address)
				address := null_ptr
			end
		end

feature {NONE}	-- Externals

	length: INTEGER
			-- Length of representation in bytes
		external
			"C macro use %"oracle.h%""
		alias
			"des_length"
		end

	max_name_length: INTEGER
			-- Maximum length of select item name
		external
			"C macro use %"oracle.h%""
		alias
			"des_max_name_length"
		end

	des_position (des_address: POINTER): INTEGER
			-- Position of select item
		external
			"C macro use %"oracle.h%""
		end
	
	des_max_dict_size (des_address: POINTER): INTEGER
			-- Maximum size of select item
		external
			"C macro use %"oracle.h%""
		end
	
	des_max_display_size (des_address: POINTER): INTEGER
			-- Maximum display size of select item
		external
			"C macro use %"oracle.h%""
		end
	
	des_internal_datatype (des_address: POINTER): INTEGER
			-- Internal datatype code of select item
		external
			"C macro use %"oracle.h%""
		end
	
	des_precision (des_address: POINTER): INTEGER
			-- Precision of numeric select item
		external
			"C macro use %"oracle.h%""
		end
	
	des_scale (des_address: POINTER): INTEGER
			-- Scale of numeric select item
		external
			"C macro use %"oracle.h%""
		end
	
	des_null_allowed (des_address: POINTER): INTEGER
			-- Null allowed indicator for select item
		external
			"C macro use %"oracle.h%""
		end
	
	des_name_length (des_address: POINTER): INTEGER
			-- Length of buffer for name for select item
		external
			"C macro use %"oracle.h%""
		end
	
	des_position_addr (des_address: POINTER): POINTER
			-- Address of select item position in `Current'
		external
			"C macro use %"oracle.h%""
		end
	
	des_max_dict_size_addr (des_address: POINTER): POINTER
			-- Address of maximum dictionary size field
		external
			"C macro use %"oracle.h%""
		end
	
	des_max_display_size_addr (des_address: POINTER): POINTER
			-- Address of maximum display size field
		external
			"C macro use %"oracle.h%""
		end
	
	des_internal_datatype_addr (des_address: POINTER): POINTER
			-- Address of internal datatype field
		external
			"C macro use %"oracle.h%""
		end
	
	des_precision_addr (des_address: POINTER): POINTER
			-- Address of precision field
		external
			"C macro use %"oracle.h%""
		end
	
	des_scale_addr (des_address: POINTER): POINTER
			-- Address of scale field
		external
			"C macro use %"oracle.h%""
		end
	
	des_null_allowed_addr (des_address: POINTER): POINTER
			-- Address of null allowed field
		external
			"C macro use %"oracle.h%""
		end
	
	des_name_length_addr (des_address: POINTER): POINTER
			-- Address of name length field
		external
			"C macro use %"oracle.h%""
		end
	
	des_name_addr (des_address: POINTER): POINTER
			-- Address of buffer for select item name
		external
			"C macro use %"oracle.h%""
		end
	
	des_set_position (des_address: POINTER k: INTEGER)
			-- Set position of `Current' in select clause to `k'
		external
			"C macro use %"oracle.h%""
		end
	
	des_set_name_length (des_address: POINTER len: INTEGER)
			-- Set length of name buffer to `len'
		external
			"C macro use %"oracle.h%""
		end
	
invariant
	address_exists: address /= default_pointer

end
