
indexing
	
	author: "David Hollenberg"
	date:	"November 28, 1995"
	description: "An error handler for errors related to the Oracle %
			%database"

deferred class ORACLE_ERROR_HANDLER
inherit
	DB_ERROR_HANDLER
	ORACLE_EXCEPTIONS
		export
			{NONE} all
		end

feature

	handle_db_error (connection: DB_CONNECTION stmt: SQL_STATEMENT)
			-- Handle an Oracle Call Interface error which
			-- occurred on connection `connection' and which
			-- involves statement `stmt' if non-void
		local
			conn: ORACLE_CONNECTION
			cursor: ORACLE_CURSOR
		do
			conn ?= connection
			cursor ?= stmt
			if conn /= Void and (stmt = Void or cursor /= Void) then
				handle_oci_error (conn, cursor)
			else
				raise (dbex.Db_conformance_error)
			end
		end

	handle_oci_error (connection: ORACLE_CONNECTION cursor: ORACLE_CURSOR)
			-- Handle an Oracle Call Interface error which
			-- occurred on connection `connection' and which
			-- involves cursor `cursor' if non-void
		require
			connection_exists: connection /= Void
		deferred
		end

	handle_oci_error_with_code (connection: ORACLE_CONNECTION cursor: ORACLE_CURSOR code: INTEGER)
			-- Handle an Oracle Call Interface error which
			-- occurred on connection `connection' and which
			-- involves cursor `cursor' if non-void.  Use
			-- `code' as the error code instead of getting it
			-- from the return code field of the CDA (if
			-- `cursor' non-Void) or from the LDA
		require
			connection_exists: connection /= Void
		deferred
		end

	handle_db_user_error (connection: DB_CONNECTION stmt: SQL_STATEMENT except, message: STRING)
			-- Handle a database error which is detected by
			-- us rather than by the database call interface.
			-- The error occurred on connection `connection' and
			-- involves cursor `cursor' if non-void.
			-- `except' is the exception tag to use if
			-- an exception is raised and `message' is the
			-- message to display.
		local
			conn: ORACLE_CONNECTION
			cursor: ORACLE_CURSOR
		do
			conn ?= connection
			cursor ?= stmt
			if conn /= Void and (stmt = Void or cursor /= Void) then
				handle_oracle_user_error (conn, cursor, except, message)
			else
				raise (dbex.Db_conformance_error)
			end
		end

	handle_oracle_user_error (connection: ORACLE_CONNECTION cursor: ORACLE_CURSOR except, message: STRING)
			-- Handle an Oracle error which is detected by
			-- us rather than by the Oracle Call Interface.
			-- The error occurred on connection `connection' and
			-- involves cursor `cursor' if non-void.
			-- `except' is the exception tag to use if
			-- an exception is raised and `message' is the
			-- message to display.
		require
			connection_exists: connection /= Void
			exception_tag_exists: except /= Void
			message_exists: message /= Void
		deferred
		end

	oci_error_description (connection: ORACLE_CONNECTION cursor: ORACLE_CURSOR): STRING
			-- Return a description of an Oracle Call Interface
			-- error which occurred on connection `connection'
			-- and which involves cursor `cursor' if non-void
		require
			connection_exists: connection /= Void
		deferred
		ensure
			result_exists: Result /= Void
		end

	set_error_display_suppressed (b: BOOLEAN)
			-- Set `error_display_suppressed' to `b'
		do
			error_display_suppressed := b
		end

	error_display_suppressed: BOOLEAN
			-- Should handler suppress display of error messages?
			-- If true, suppresses display of error messages.
			-- If false, error messages are displayed at
			-- handler's discretion.

end
