
indexing
	
	author: "David Hollenberg"
	date:	"December 6, 1995"
	description: "A set of Oracle named placeholders (bind variables)"

class ORACLE_NAMED_BINDSET

inherit
	ORACLE_VALIDITY
		export
			{ANY} is_valid_placeholder_name,
				is_valid_placeholder_value
			{NONE} all
		end
	ORACLE_BINDSET
		rename
			bind_to_integer as generic_bind_to_integer,
			bind_to_real as generic_bind_to_real,
			bind_to_double as generic_bind_to_double,
			bind_to_character as generic_bind_to_character,
			bind_to_boolean as generic_bind_to_boolean,
			bind_to_string as generic_bind_to_string,
			bind_to_string_with_max_size as generic_bind_to_string_with_max_size,
			bind_to_date as generic_bind_to_date,
			bind_to_rowid as generic_bind_to_rowid,
			bind_to_raw as generic_bind_to_raw,
			bind_to_raw_with_max_size as generic_bind_to_raw_with_max_size,
			bind_to_any as generic_bind_to_any
		redefine
			placeholder, placeholder_length
		end

create
	make

feature -- Modification

	bind_to_integer (ph_name: STRING value: INTEGER null: BOOLEAN)
			-- Bind placeholder named `ph_name' to `value' if
			-- `null' is false; otherwise, bind to null
		require
			valid_placeholder_name: is_valid_placeholder_name (ph_name)
		do
			search (ph_name)
			generic_bind_to_integer (value, null)
		end

	bind_to_real (ph_name: STRING value: REAL null: BOOLEAN)
			-- Bind placeholder named `ph_name' to `value' if
			-- `null' is false; otherwise, bind to null
		require
			valid_placeholder_name: is_valid_placeholder_name (ph_name)
		do
			search (ph_name)
			generic_bind_to_real (value, null)
		end

	bind_to_double (ph_name: STRING value: DOUBLE null: BOOLEAN)
			-- Bind placeholder named `ph_name' to `value' if
			-- `null' is false; otherwise, bind to null
		require
			valid_placeholder_name: is_valid_placeholder_name (ph_name)
		do
			search (ph_name)
			generic_bind_to_double (value, null)
		end

	bind_to_character (ph_name: STRING value: CHARACTER null: BOOLEAN)
			-- Bind placeholder named `ph_name' to `value' if
			-- `null' is false; otherwise, bind to null
		require
			valid_placeholder_name: is_valid_placeholder_name (ph_name)
		do
			search (ph_name)
			generic_bind_to_character (value, null)
		end

	bind_to_boolean (ph_name: STRING value: BOOLEAN null: BOOLEAN)
			-- Bind placeholder named `ph_name' to `value' if
			-- `null' is false; otherwise, bind to null
		require
			valid_placeholder_name: is_valid_placeholder_name (ph_name)
		do
			search (ph_name)
			generic_bind_to_boolean (value, null)
		end

	bind_to_string (ph_name: STRING value: STRING null: BOOLEAN)
			-- Bind placeholder named `ph_name' to `value' if
			-- `null' is false; otherwise, bind to null.  If
			-- `value' is void, bind to null regardless of `null'
		require
			valid_placeholder_name: is_valid_placeholder_name (ph_name)
		do
			search (ph_name)
			generic_bind_to_string (value, null)
		end

	bind_to_string_with_max_size (ph_name: STRING value: STRING max_out_len: INTEGER null: BOOLEAN)
			-- Bind placeholder named `ph_name' to `value' if
			-- `null' is false; otherwise, bind to null.  If
			-- `value' is void, bind to null regardless of `null'.
			-- Use `max_out_len' as the maximum length of the
			-- string if it is greater than the actual length
		require
			valid_placeholder_name: is_valid_placeholder_name (ph_name)
			nonnegative_max_length: max_out_len >= 0
		do
			search (ph_name)
			generic_bind_to_string_with_max_size (value, max_out_len, null)
		end

	bind_to_raw (ph_name: STRING value: STRING null: BOOLEAN)
			-- Bind placeholder named `ph_name' to `value' if
			-- `null' is false; otherwise, bind to null.  If
			-- `value' is void, bind to null regardless of `null'
		require
			valid_placeholder_name: is_valid_placeholder_name (ph_name)
		do
			search (ph_name)
			generic_bind_to_raw (value, null)
		end

	bind_to_raw_with_max_size (ph_name: STRING value: STRING max_out_len: INTEGER null: BOOLEAN)
			-- Bind placeholder named `ph_name' to `value' if
			-- `null' is false; otherwise, bind to null.  If
			-- `value' is void, bind to null regardless of `null'.
			-- Use `max_out_len' as the maximum length of the
			-- string if it is greater than the actual length
		require
			valid_placeholder_name: is_valid_placeholder_name (ph_name)
			nonnegative_max_length: max_out_len >= 0
		do
			search (ph_name)
			generic_bind_to_raw_with_max_size (value, max_out_len, null)
		end

	bind_to_date (ph_name: STRING value: ORACLE_DATE null: BOOLEAN)
			-- Bind placeholder named `ph_name' to `value' if
			-- `null' is false; otherwise, bind to null.  If
			-- `value' is void or `value'.`is_null', bind to
			-- null regardless of `null'
		require
			valid_placeholder_name: is_valid_placeholder_name (ph_name)
		do
			search (ph_name)
			generic_bind_to_date (value, null)
		end

	bind_to_rowid (ph_name: STRING value: ORACLE_ROWID null: BOOLEAN)
			-- Bind placeholder named `ph_name' to `value' if
			-- `null' is false; otherwise, bind to null.  If
			-- `value' is void, bind to null regardless of `null'
		require
			valid_placeholder_name: is_valid_placeholder_name (ph_name)
		do
			search (ph_name)
			generic_bind_to_rowid (value, null)
		end

	bind_to_any (ph_name: STRING value: ANY null: BOOLEAN)
			-- Bind placeholder named `ph_name' to `value' if
			-- `null' is false; otherwise, bind to null
		require
			valid_placeholder_name: is_valid_placeholder_name (ph_name)
			value_exists: value /= Void
			valid_value: is_valid_placeholder_value (value)
		do
			search (ph_name)
			generic_bind_to_any (value, null)
		end

feature -- Properties

	is_bound_null (ph_name: STRING): BOOLEAN
			-- Is placeholder named `ph_name' bound to null?
		require
			valid_placeholder_name: is_valid_placeholder_name (ph_name)
		do
			find (ph_name)
			if found then
				Result := placeholder.bound_to_null
			end
		end

	is_bound_integer (ph_name: STRING): BOOLEAN
			-- Is placeholder named `ph_name' bound to an
			-- integer?
		require
			valid_placeholder_name: is_valid_placeholder_name (ph_name)
		do
			find (ph_name)
			if found then
				Result := placeholder.bound_to_integer
			end
		end

	is_bound_real (ph_name: STRING): BOOLEAN
			-- Is placeholder named `ph_name' bound to an
			-- real?
		require
			valid_placeholder_name: is_valid_placeholder_name (ph_name)
		do
			find (ph_name)
			if found then
				Result := placeholder.bound_to_real
			end
		end

	is_bound_double (ph_name: STRING): BOOLEAN
			-- Is placeholder named `ph_name' bound to an
			-- double?
		require
			valid_placeholder_name: is_valid_placeholder_name (ph_name)
		do
			find (ph_name)
			if found then
				Result := placeholder.bound_to_double
			end
		end

	is_bound_character (ph_name: STRING): BOOLEAN
			-- Is placeholder named `ph_name' bound to a
			-- character?
		require
			valid_placeholder_name: is_valid_placeholder_name (ph_name)
		do
			find (ph_name)
			if found then
				Result := placeholder.bound_to_character
			end
		end

	is_bound_boolean (ph_name: STRING): BOOLEAN
			-- Is placeholder named `ph_name' bound to a
			-- boolean?
		require
			valid_placeholder_name: is_valid_placeholder_name (ph_name)
		do
			find (ph_name)
			if found then
				Result := placeholder.bound_to_boolean
			end
		end

	is_bound_string (ph_name: STRING): BOOLEAN
			-- Is placeholder named `ph_name' bound to a
			-- string?
		require
			valid_placeholder_name: is_valid_placeholder_name (ph_name)
		do
			find (ph_name)
			if found then
				Result := placeholder.bound_to_string
			end
		end

	is_bound_raw (ph_name: STRING): BOOLEAN
			-- Is placeholder named `ph_name' bound to a
			-- raw string?
		require
			valid_placeholder_name: is_valid_placeholder_name (ph_name)
		do
			find (ph_name)
			if found then
				Result := placeholder.bound_to_raw
			end
		end

	is_bound_date (ph_name: STRING): BOOLEAN
			-- Is placeholder named `ph_name' bound to a
			-- database date?
		require
			valid_placeholder_name: is_valid_placeholder_name (ph_name)
		do
			find (ph_name)
			if found then
				Result := placeholder.bound_to_date
			end
		end

	is_bound_rowid (ph_name: STRING): BOOLEAN
			-- Is placeholder named `ph_name' bound to a
			-- database rowid?
		require
			valid_placeholder_name: is_valid_placeholder_name (ph_name)
		do
			find (ph_name)
			if found then
				Result := placeholder.bound_to_rowid
			end
		end

	is_bound_any (ph_name: STRING): BOOLEAN
			-- Is placeholder named `ph_name' bound to anything?
		require
			valid_placeholder_name: is_valid_placeholder_name (ph_name)
		do
			find (ph_name)
			Result := found
		end

	integer_value (ph_name: STRING): INTEGER
			-- Value of integer bound to placeholder named
			-- `ph_name'
		require
			valid_placeholder_name: is_valid_placeholder_name (ph_name)
			bound_to_integer: is_bound_integer (ph_name)
		do
			find (ph_name)
			Result := placeholder.integer_value
		end

	real_value (ph_name: STRING): REAL
			-- Value of real bound to placeholder named
			-- `ph_name'
		require
			valid_placeholder_name: is_valid_placeholder_name (ph_name)
			bound_to_real: is_bound_real (ph_name)
		do
			find (ph_name)
			Result := placeholder.real_value
		end

	double_value (ph_name: STRING): DOUBLE
			-- Value of double bound to placeholder named
			-- `ph_name'
		require
			valid_placeholder_name: is_valid_placeholder_name (ph_name)
			bound_to_double: is_bound_double (ph_name)
		do
			find (ph_name)
			Result := placeholder.double_value
		end

	character_value (ph_name: STRING): CHARACTER
			-- Value of character bound to placeholder named
			-- `ph_name'
		require
			valid_placeholder_name: is_valid_placeholder_name (ph_name)
			bound_to_character: is_bound_character (ph_name)
		do
			find (ph_name)
			Result := placeholder.character_value
		end

	boolean_value (ph_name: STRING): BOOLEAN
			-- Value of boolean bound to placeholder named
			-- `ph_name'
		require
			valid_placeholder_name: is_valid_placeholder_name (ph_name)
			bound_to_boolean: is_bound_boolean (ph_name)
		do
			find (ph_name)
			Result := placeholder.boolean_value
		end

	string_value (ph_name: STRING): STRING
			-- Value of string bound to placeholder named
			-- `ph_name'
		require
			valid_placeholder_name: is_valid_placeholder_name (ph_name)
			bound_to_string: is_bound_string (ph_name)
		do
			find (ph_name)
			Result := placeholder.string_value
		end

	raw_value (ph_name: STRING): STRING
			-- Value of raw string bound to placeholder named
			-- `ph_name'
		require
			valid_placeholder_name: is_valid_placeholder_name (ph_name)
			bound_to_raw: is_bound_raw (ph_name)
		do
			find (ph_name)
			Result := placeholder.raw_value
		end

	date_value (ph_name: STRING): ORACLE_DATE
			-- Value of database date bound to placeholder named
			-- `ph_name'
		require
			valid_placeholder_name: is_valid_placeholder_name (ph_name)
			bound_to_date: is_bound_date (ph_name)
		do
			find (ph_name)
			Result := placeholder.date_value
		end

	rowid_value (ph_name: STRING): ORACLE_ROWID
			-- Value of database rowid bound to placeholder named
			-- `ph_name'
		require
			valid_placeholder_name: is_valid_placeholder_name (ph_name)
			bound_to_rowid: is_bound_rowid (ph_name)
		do
			find (ph_name)
			Result := placeholder.rowid_value
		end

	any_value (ph_name: STRING): ANY
			-- Value of placeholder named `ph_name'
		require
			valid_placeholder_name: is_valid_placeholder_name (ph_name)
			is_bound: is_bound_any (ph_name)
		do
			find (ph_name)
			Result := placeholder.any_value
		end

feature {NONE} -- Implementation

	placeholder: ORACLE_NAMED_PLACEHOLDER
			-- Object used to access and modify a piece of C memory
			-- containing a placeholder as an Eiffel object.
			-- Having this as an attribute avoids repeated
			-- creation.

	create_placeholder
			-- Create placeholder object and put in `placeholder'
		do
			create placeholder.set_address (address)
		end

	search (ph_name: STRING)
			-- Find slot for placeholder named `ph_name'
			-- and put in `placeholder'.  If slot is not
			-- already in use, set its name to `ph_name'.
		local
			item: ORACLE_NAMED_PLACEHOLDER
		do
			raw_find (ph_name)
			if not found then
				if count >= capacity then
					grow (capacity + Placeholder_count_increment)
				end
				item := placeholder
				item.set_address (i_th_placeholder (count + 1))
				item.set_name (ph_name)
			end
		end

	find (ph_name: STRING)
			-- Find slot for placeholder named `ph_name'
			-- and put in `placeholder'.
			-- Set `found' to indicate  whether there is an in use
			-- slot with this name which is bound.
		do
			raw_find (ph_name)
			if found and then not placeholder.is_bound then
				found := False
			end
		end

	raw_find (ph_name: STRING)
			-- Find slot for placeholder named `ph_name'
			-- and put in `placeholder'.
			-- Set `found' to indicate  whether an in use
			-- slot with this name exists.
		local
			ph_addr: POINTER
			k, len, name_len, local_count: INTEGER
			item: ORACLE_NAMED_PLACEHOLDER
			slot_found: BOOLEAN
			name_value: POINTER
			area: ANY
		do
			area := ph_name.area
			if (ph_name.item (1) = ':') then
				name_value := offset (to_pointer ($area), 1)
				name_len := ph_name.count - 1
			else
				name_value := to_pointer ($area)
				name_len := ph_name.count
			end
			if name_len > max_placeholder_name_length then
				handle_name_length_error (ph_name)
			end
			from
				ph_addr := address
				k := 1
				local_count := count
				len := placeholder_length
				item := placeholder
				slot_found := False
			until
				k > local_count or slot_found
			loop
				item.set_address (ph_addr)
				check
					placeholder_in_use: item.in_use
				end
				if item.name_length = name_len and
				   memory_equal (name_value, item.name_address, name_len) then
					slot_found := True
				else
					ph_addr := next_placeholder (ph_addr, len)
				end
				k := k + 1
			end
			found := slot_found
		end

	handle_name_length_error (ph_name: STRING)
			-- Handle "placeholder name too long" error on
			-- name `ph_name'
		local
			message: STRING
		do
			create message.make (0)
			message.append ("placeholder name %"")
			message.append (ph_name)
			message.append ("%" has length greater than ")
			message.append_integer (max_placeholder_name_length)
			cursor.handle_oracle_error (dbex.Db_placeholder_too_long,
				message)
		end

	i_th_placeholder (k: INTEGER): POINTER
			-- Address of slot for placeholder number `k'
		do
			Result := phv_i_th_address (address, k - 1)
		end

	Placeholder_count_increment: INTEGER = 10
			-- Number of additional placeholders to
			-- allocate if a new bind is requested and no
			-- empty placeholder slots are available
	
feature {NONE}	-- Externals

	placeholder_length: INTEGER
			-- Length of one placeholder in bytes
		external
			"C macro use %"oracle.h%""
		alias
			"phv_length"
		end

	phv_i_th_address (bvs_addr: POINTER k: INTEGER): POINTER
			-- Address of placeholder `k' in the bind
			-- variable set at address `bvs_addr'
		external
			"C macro use %"oracle.h%""
		end

	memory_equal (addr1, addr2: POINTER num: INTEGER): BOOLEAN
			-- Is memory at addresses `addr1' and `addr2'
			-- equal for `num' bytes?
		external
			"C macro use %"oracle.h%""
		end

end
