
indexing
	
	author: "David Hollenberg"
	date:	"December 5, 1995"
	description: "An Oracle named placeholder (e.g., :design_no)"

class ORACLE_NAMED_PLACEHOLDER

inherit
	ORACLE_PLACEHOLDER
		redefine
			length
		end
	C_MEMORY_ALLOCATION
		export
			{NONE} all
		end
	STRING_FILLER_ACCESS
		export
			{NONE} all
		end

create
	set_address

feature -- Properties

	name_length: INTEGER
			-- Length of placeholder name
		do
			Result := phv_name_length (address)
		ensure
			nonnegative_result: Result >= 0
		end

	name_address: POINTER
			-- Address of placeholder name
		require
			placeholder_in_use: in_use
		do
			Result := phv_name_address (address)
		ensure
			valid_result: Result /= default_pointer
		end

	name: STRING
			-- New string with placeholder name
			-- (e.g., ":design_no")
		local
			len: INTEGER
		do
			len := name_length
			create Result.make (len)
			strfill.fill_string (Result, name_address, len)
		ensure
			valid_result: Result /= Void
		end

	display_name: STRING
			-- Name for display purposes
		do
			Result := name
			if Result.item (1) /= ':' then
				Result.prepend_character (':')
			end
		end

	max_display_name_length: INTEGER
			-- Maximum length of display name
		do
			Result := max_placeholder_name_length
		end

feature -- Modification

	set_name (s: STRING)
			-- Set placeholder name to `s'
		require
			name_exists: s /= Void
		local
			area: ANY
			len: INTEGER
			value: POINTER
		do
			area := s.area
			if s.item (1) = Bind_variable_start_char then
				len := s.count - 1
				value := offset (to_pointer ($area), 1)
			else
				len := s.count
				value := to_pointer ($area)
			end
			phv_set_name_length (address, len)
			phv_set_name (address, value, len)
		ensure
			correct_name_without_colon: s.item (1) /= ':' implies equal (s, name)
			correct_name_with_colon: s.item (1) = ':' implies equal (s.substring (2, s.count), name)
		end
	
	do_oci_bind (c: ORACLE_CURSOR): INTEGER
			-- Bind `Current' via the appropriate Oracle Call
			-- Interface routine using  `c' as the cursor.
			-- Return the return code from this routine.
		do
			c.enter_oci
			Result := ora_bind_placeholder_name (c.address,
				name_address, name_length, data_address,
				data_length, external_datatype,
				std_null_indicator_address)
			c.leave_oci
		end
	
feature {NONE}	-- Externals

	length: INTEGER
			-- Length in bytes
		external
			"C macro use %"oracle.h%""
		alias
			"phv_length"
		end

	phv_name_length (ph_address: POINTER): INTEGER
			-- Length of placeholder name
		external
			"C macro use %"oracle.h%""
		end
	
	phv_name_address (ph_address: POINTER): POINTER
			-- Address of placeholder name
		external
			"C macro use %"oracle.h%""
		end
	
	phv_get_name (ph_address: POINTER dest: POINTER)
			-- Copy placeholder name to area at address `dest',
			-- using length `name_length'
		external
			"C macro use %"oracle.h%""
		end
	
	phv_set_name_length (ph_address: POINTER len: INTEGER)
			-- Set length of placeholder name to `len'
		external
			"C macro use %"oracle.h%""
		end

	phv_set_name (ph_address: POINTER addr: POINTER len: INTEGER)
			-- Set placeholder name from name at address `addr'
			-- with length `len'
		external
			"C macro use %"oracle.h%""
		end

end
