
indexing
	
	author: "David Hollenberg"
	date:	"December 6, 1995"
	description: "A set of Oracle numeric placeholders (bind variables)"

class ORACLE_NUMERIC_BINDSET

inherit
	ORACLE_VALIDITY
	ORACLE_BINDSET
		rename
			bind_to_integer as generic_bind_to_integer,
			bind_to_real as generic_bind_to_real,
			bind_to_double as generic_bind_to_double,
			bind_to_character as generic_bind_to_character,
			bind_to_boolean as generic_bind_to_boolean,
			bind_to_string as generic_bind_to_string,
			bind_to_string_with_max_size as generic_bind_to_string_with_max_size,
			bind_to_date as generic_bind_to_date,
			bind_to_rowid as generic_bind_to_rowid,
			bind_to_raw as generic_bind_to_raw,
			bind_to_raw_with_max_size as generic_bind_to_raw_with_max_size,
			bind_to_any as generic_bind_to_any
		redefine
			placeholder, placeholder_length
		end

create
	make

feature -- Modification

	bind_to_integer (ph_number: INTEGER value: INTEGER null: BOOLEAN)
			-- Bind placeholder number `ph_number' to `value' if
			-- `null' is false; otherwise, bind to null
		require
			valid_placeholder_number: is_valid_placeholder_number (ph_number)
		do
			search (ph_number)
			generic_bind_to_integer (value, null)
		end

	bind_to_real (ph_number: INTEGER value: REAL null: BOOLEAN)
			-- Bind placeholder number `ph_number' to `value' if
			-- `null' is false; otherwise, bind to null
		require
			valid_placeholder_number: is_valid_placeholder_number (ph_number)
		do
			search (ph_number)
			generic_bind_to_real (value, null)
		end

	bind_to_double (ph_number: INTEGER value: DOUBLE null: BOOLEAN)
			-- Bind placeholder number `ph_number' to `value' if
			-- `null' is false; otherwise, bind to null
		require
			valid_placeholder_number: is_valid_placeholder_number (ph_number)
		do
			search (ph_number)
			generic_bind_to_double (value, null)
		end

	bind_to_character (ph_number: INTEGER value: CHARACTER null: BOOLEAN)
			-- Bind placeholder number `ph_number' to `value' if
			-- `null' is false; otherwise, bind to null
		require
			valid_placeholder_number: is_valid_placeholder_number (ph_number)
		do
			search (ph_number)
			generic_bind_to_character (value, null)
		end

	bind_to_boolean (ph_number: INTEGER value: BOOLEAN null: BOOLEAN)
			-- Bind placeholder number `ph_number' to `value' if
			-- `null' is false; otherwise, bind to null
		require
			valid_placeholder_number: is_valid_placeholder_number (ph_number)
		do
			search (ph_number)
			generic_bind_to_boolean (value, null)
		end

	bind_to_string (ph_number: INTEGER value: STRING null: BOOLEAN)
			-- Bind placeholder number `ph_number' to `value' if
			-- `null' is false; otherwise, bind to null.  If
			-- `value' is void, bind to null regardless of `null'
		require
			valid_placeholder_number: is_valid_placeholder_number (ph_number)
		do
			search (ph_number)
			generic_bind_to_string (value, null)
		end

	bind_to_string_with_max_size (ph_number: INTEGER value: STRING max_out_len: INTEGER null: BOOLEAN)
			-- Bind placeholder number `ph_number' to `value' if
			-- `null' is false; otherwise, bind to null.  If
			-- `value' is void, bind to null regardless of `null'.
			-- Use `max_out_len' as the maximum length of the
			-- string if it is greater than the actual length
		require
			valid_placeholder_number: is_valid_placeholder_number (ph_number)
			nonnegative_max_length: max_out_len >= 0
		do
			search (ph_number)
			generic_bind_to_string_with_max_size (value, max_out_len, null)
		end

	bind_to_raw (ph_number: INTEGER value: STRING null: BOOLEAN)
			-- Bind placeholder number `ph_number' to `value' if
			-- `null' is false; otherwise, bind to null.  If
			-- `value' is void, bind to null regardless of `null'
		require
			valid_placeholder_number: is_valid_placeholder_number (ph_number)
		do
			search (ph_number)
			generic_bind_to_raw (value, null)
		end

	bind_to_raw_with_max_size (ph_number: INTEGER value: STRING max_out_len: INTEGER null: BOOLEAN)
			-- Bind placeholder number `ph_number' to `value' if
			-- `null' is false; otherwise, bind to null.  If
			-- `value' is void, bind to null regardless of `null'.
			-- Use `max_out_len' as the maximum length of the
			-- string if it is greater than the actual length
		require
			valid_placeholder_number: is_valid_placeholder_number (ph_number)
			nonnegative_max_length: max_out_len >= 0
		do
			search (ph_number)
			generic_bind_to_raw_with_max_size (value, max_out_len, null)
		end

	bind_to_date (ph_number: INTEGER value: ORACLE_DATE null: BOOLEAN)
			-- Bind placeholder number `ph_number' to
			-- `value' if `null' is false; otherwise, bind
			-- to null.  If `value' is void or
			-- `value'.`is_null', bind to null regardless
			-- of `null'
		require
			valid_placeholder_number: is_valid_placeholder_number (ph_number)
		do
			search (ph_number)
			generic_bind_to_date (value, null)
		end

	bind_to_rowid (ph_number: INTEGER value: ORACLE_ROWID null: BOOLEAN)
			-- Bind placeholder number `ph_number' to `value' if
			-- `null' is false; otherwise, bind to null.  If
			-- `value' is void, bind to null regardless of `null'
		require
			valid_placeholder_number: is_valid_placeholder_number (ph_number)
		do
			search (ph_number)
			generic_bind_to_rowid (value, null)
		end

	bind_to_any (ph_number: INTEGER value: ANY null: BOOLEAN)
			-- Bind placeholder number `ph_number' to `value' if
			-- `null' is false; otherwise, bind to null
		require
			valid_placeholder_number: is_valid_placeholder_number (ph_number)
			value_exists: value /= Void
			valid_value: is_valid_placeholder_value (value)
		do
			search (ph_number)
			generic_bind_to_any (value, null)
		end

feature -- Properties

	is_bound_null (ph_number: INTEGER): BOOLEAN
			-- Is placeholder number `ph_number' bound to null?
		require
			valid_placeholder_number: is_valid_placeholder_number (ph_number)
		do
			find (ph_number)
			if found then
				Result := placeholder.bound_to_null
			end
		end

	is_bound_integer (ph_number: INTEGER): BOOLEAN
			-- Is placeholder number `ph_number' bound to an
			-- integer?
		require
			valid_placeholder_number: is_valid_placeholder_number (ph_number)
		do
			find (ph_number)
			if found then
				Result := placeholder.bound_to_integer
			end
		end

	is_bound_real (ph_number: INTEGER): BOOLEAN
			-- Is placeholder number `ph_number' bound to a real?
		require
			valid_placeholder_number: is_valid_placeholder_number (ph_number)
		do
			find (ph_number)
			if found then
				Result := placeholder.bound_to_real
			end
		end

	is_bound_double (ph_number: INTEGER): BOOLEAN
			-- Is placeholder number `ph_number' bound to a double?
		require
			valid_placeholder_number: is_valid_placeholder_number (ph_number)
		do
			find (ph_number)
			if found then
				Result := placeholder.bound_to_double
			end
		end

	is_bound_character (ph_number: INTEGER): BOOLEAN
			-- Is placeholder number `ph_number' bound to a
			-- character?
		require
			valid_placeholder_number: is_valid_placeholder_number (ph_number)
		do
			find (ph_number)
			if found then
				Result := placeholder.bound_to_character
			end
		end

	is_bound_boolean (ph_number: INTEGER): BOOLEAN
			-- Is placeholder number `ph_number' bound to a
			-- boolean?
		require
			valid_placeholder_number: is_valid_placeholder_number (ph_number)
		do
			find (ph_number)
			if found then
				Result := placeholder.bound_to_boolean
			end
		end

	is_bound_string (ph_number: INTEGER): BOOLEAN
			-- Is placeholder number `ph_number' bound to a
			-- string?
		require
			valid_placeholder_number: is_valid_placeholder_number (ph_number)
		do
			find (ph_number)
			if found then
				Result := placeholder.bound_to_string
			end
		end

	is_bound_raw (ph_number: INTEGER): BOOLEAN
			-- Is placeholder number `ph_number' bound to a
			-- raw string?
		require
			valid_placeholder_number: is_valid_placeholder_number (ph_number)
		do
			find (ph_number)
			if found then
				Result := placeholder.bound_to_raw
			end
		end

	is_bound_date (ph_number: INTEGER): BOOLEAN
			-- Is placeholder number `ph_number' bound to a
			-- datebase date?
		require
			valid_placeholder_number: is_valid_placeholder_number (ph_number)
		do
			find (ph_number)
			if found then
				Result := placeholder.bound_to_date
			end
		end

	is_bound_rowid (ph_number: INTEGER): BOOLEAN
			-- Is placeholder number `ph_number' bound to a
			-- datebase rowid?
		require
			valid_placeholder_number: is_valid_placeholder_number (ph_number)
		do
			find (ph_number)
			if found then
				Result := placeholder.bound_to_rowid
			end
		end

	is_bound_any (ph_number: INTEGER): BOOLEAN
			-- Is placeholder number `ph_number' bound to anything?
		require
			valid_placeholder_number: is_valid_placeholder_number (ph_number)
		do
			find (ph_number)
			Result := found
		end

	integer_value (ph_number: INTEGER): INTEGER
			-- Value of integer bound to placeholder number
			-- `ph_number'
		require
			valid_placeholder_number: is_valid_placeholder_number (ph_number)
			bound_to_integer: is_bound_integer (ph_number)
		do
			find (ph_number)
			Result := placeholder.integer_value
		end

	real_value (ph_number: INTEGER): REAL
			-- Value of real bound to placeholder number
			-- `ph_number'
		require
			valid_placeholder_number: is_valid_placeholder_number (ph_number)
			bound_to_real: is_bound_real (ph_number)
		do
			find (ph_number)
			Result := placeholder.real_value
		end

	double_value (ph_number: INTEGER): DOUBLE
			-- Value of double bound to placeholder number
			-- `ph_number'
		require
			valid_placeholder_number: is_valid_placeholder_number (ph_number)
			bound_to_double: is_bound_double (ph_number)
		do
			find (ph_number)
			Result := placeholder.double_value
		end

	character_value (ph_number: INTEGER): CHARACTER
			-- Value of character bound to placeholder number
			-- `ph_number'
		require
			valid_placeholder_number: is_valid_placeholder_number (ph_number)
			bound_to_character: is_bound_character (ph_number)
		do
			find (ph_number)
			Result := placeholder.character_value
		end

	boolean_value (ph_number: INTEGER): BOOLEAN
			-- Value of boolean bound to placeholder number
			-- `ph_number'
		require
			valid_placeholder_number: is_valid_placeholder_number (ph_number)
			bound_to_boolean: is_bound_boolean (ph_number)
		do
			find (ph_number)
			Result := placeholder.boolean_value
		end

	string_value (ph_number: INTEGER): STRING
			-- Value of string bound to placeholder number
			-- `ph_number'
		require
			valid_placeholder_number: is_valid_placeholder_number (ph_number)
			bound_to_string: is_bound_string (ph_number)
		do
			find (ph_number)
			Result := placeholder.string_value
		end

	raw_value (ph_number: INTEGER): STRING
			-- Value of raw string bound to placeholder number
			-- `ph_number'
		require
			valid_placeholder_number: is_valid_placeholder_number (ph_number)
			bound_to_raw: is_bound_raw (ph_number)
		do
			find (ph_number)
			Result := placeholder.raw_value
		end

	date_value (ph_number: INTEGER): ORACLE_DATE
			-- Value of database date bound to placeholder number
			-- `ph_number'
		require
			valid_placeholder_number: is_valid_placeholder_number (ph_number)
			bound_to_date: is_bound_date (ph_number)
		do
			find (ph_number)
			Result := placeholder.date_value
		end

	rowid_value (ph_number: INTEGER): ORACLE_ROWID
			-- Value of database rowid bound to placeholder number
			-- `ph_number'
		require
			valid_placeholder_number: is_valid_placeholder_number (ph_number)
			bound_to_rowid: is_bound_rowid (ph_number)
		do
			find (ph_number)
			Result := placeholder.rowid_value
		end

	any_value (ph_number: INTEGER): ANY
			-- Value rowid bound to placeholder number `ph_number'
		require
			valid_placeholder_number: is_valid_placeholder_number (ph_number)
			is_bound: is_bound_any (ph_number)
		do
			find (ph_number)
			Result := placeholder.any_value
		end

feature {NONE} -- Implementation

	placeholder: ORACLE_NUMERIC_PLACEHOLDER
			-- Object used to access and modify a piece of C memory
			-- containing a placeholder as an Eiffel object.
			-- Having this as an attribute avoids repeated
			-- creation.

	create_placeholder
			-- Create placeholder object and put in `placeholder'
		do
			create placeholder.set_address (address)
		end

	search (ph_number: INTEGER)
			-- Find slot for placeholder number `ph_number'
			-- and put in `placeholder'.  If slot is not
			-- already in use, set its number to `ph_number'.
		local
			ph: ORACLE_NUMERIC_PLACEHOLDER
		do
			if ph_number > capacity then
				grow (ph_number)
			end
			ph := placeholder
			ph.set_address (i_th_placeholder (ph_number))
			if not ph.in_use then
				ph.set_number (ph_number)
			end
		end

	find (ph_number: INTEGER)
			-- Find slot for placeholder number `ph_number'
			-- and put in `placeholder' if it is in use.
			-- Set `found' to indicate  whether an in use
			-- slot with this number exists.
		local
			ph: ORACLE_NUMERIC_PLACEHOLDER
		do
			if ph_number > capacity then
				found := False
			else
				ph := placeholder
				ph.set_address (i_th_placeholder (ph_number))
				found := ph.in_use and ph.is_bound
			end
		end

	i_th_placeholder (k: INTEGER): POINTER
			-- Address of slot for placeholder number `k'
		do
			Result := phn_i_th_address (address, k - 1)
		end

feature {NONE}	-- Externals

	placeholder_length: INTEGER
			-- Length of one placeholder in bytes
		external
			"C macro use %"oracle.h%""
		alias
			"phn_length"
		end

	phn_i_th_address (bvs_addr: POINTER k: INTEGER): POINTER
			-- Address of placeholder `k' in the bind
			-- variable set at address `bvs_addr'
		external
			"C macro use %"oracle.h%""
		end

end
