
indexing
	
	author: "David Hollenberg"
	date:	"December 11, 1995"
	description: "A rowid in Oracle internal format"
		
class ORACLE_ROWID

inherit
	DB_ROWID
	PLATFORM
		undefine
			out
		end

create
	make_internal, set_null

feature -- Creation
	
	make_internal (part1, part2: DOUBLE)
			-- Create rowid from `part1' and `part2', which
			-- represent the first and second halves of
			-- an Oracle rowid in internal format
		do
			first_half := part1
			second_half := part2
		end

	set_null
			-- Create null rowid
		do
			make_internal (0.0, 0.0)
		end

feature -- Properties
	
        is_null: BOOLEAN
                        -- Is `Current' a null rowid?
                do
			Result := first_half = 0.0 and second_half = 0.0
                end

	length: INTEGER
			-- Length of internal representation
		do
			Result := oracle_rowid_length
		end
	
	hash_code: INTEGER
			-- Hash code value
		do
			get_internal_hash_code ($first_half, $second_half, 8, $Result)
		end
	
	out: STRING
			-- Printable representation of `Current'
		local
			val: STRING
		do
			create Result.make (18)
			Result.append (hex_string (block_number))
			Result.append_character (Rowid_separator)
			val := hex_string (row_number)
			val.keep_tail (4)
			Result.append (val)
			Result.append_character (Rowid_separator)
			val := hex_string (file_number)
			val.keep_tail (4)
			Result.append (val)
		end

	row_number: INTEGER
			-- Row number portion of `Current'
		do
			Result := rowid_row_number ($second_half)
		end

	block_number: INTEGER
			-- Block number portion of `Current'
		do
			Result := rowid_block_number ($second_half)
		end

	file_number: INTEGER
			-- File number portion of `Current'
		do
			Result := rowid_file_number ($first_half)
		end

feature {ORACLE_PLACEHOLDER} -- Representation
	
	first_half: DOUBLE
			-- First half of `Current' in Oracle internal format
	
	second_half: DOUBLE
			-- Second half of `Current' in Oracle internal format
	
feature {NONE} -- Implementation

	Rowid_separator: CHARACTER is '.'
	
	hex_string (num: INTEGER): STRING
			-- Printable hex representation of `num'
		local
			k, number, nibble: INTEGER
			c: CHARACTER
		do
			number := num
			from
				create Result.make (8)
				Result.fill_blank
				k := 0
			until
				k > 7
			loop
				nibble := n_th_nibble (number, k)
				if nibble >= 10 then
					c := 'A' + (nibble - 10)
				else
					c := '0' + nibble
				end
				Result.put (c, 8 - k)
				k := k + 1
			end
		end

	get_internal_hash_code (part1, part2: POINTER nbytes: INTEGER dest: POINTER)
		external
			"C inline"
		alias "[
			{
			EIF_INTEGER result, k;
			result = 0;
			EIF_INTEGER f = 8388593;
			for (k = 0; k < $nbytes; k++) {
				result = ((result % f) << 8) + ((unsigned char *) $part1)[k];
			}
			for (k = 0; k < $nbytes; k++) {
				result = ((result % f) << 8) + ((unsigned char *) $part2)[k];
			}
			if (result < 0) {
				result = 1;
			}
			* ((EIF_INTEGER *) $dest) = result;
			}
]"
		end

feature {NONE} -- Externals
	
	oracle_rowid_length: INTEGER
			-- Length of rowid in Oracle internal format
		external
			"C macro use %"oracle.h%""
		ensure
			nonnegative_result: Result >= 0
		end
	
	n_th_nibble (val: INTEGER n: INTEGER): INTEGER
			-- The `n'-th nibble of val
		external
			"C macro use %"oracle.h%""
		end
	
	rowid_row_number (second_addr: POINTER): INTEGER
			-- Row number portion of rowid specified by
			-- second half of rowid
		external
			"C macro use %"oracle.h%""
		end
	
	rowid_block_number (second_addr: POINTER): INTEGER
			-- Block number portion of rowid specified by
			-- second half of rowid
		external
			"C macro use %"oracle.h%""
		end
	
	rowid_file_number (first_address: POINTER): INTEGER
			-- Database file number portion of rowid specified by
			-- first half of rowid
		external
			"C macro use %"oracle.h%""
		end
	
end
