
indexing
	
	author: "David Hollenberg"
	date:	"December 6, 1995"
	description: "Facilities for validity checking on Oracle database %
		%operations"

class ORACLE_VALIDITY

feature -- Validity

	is_valid_placeholder_number (num: INTEGER): BOOLEAN
			-- Is `num' a valid numeric placeholder number?
		do
			Result := num >= 1 and num <= 255
		end
	
	is_valid_placeholder_name (s: STRING): BOOLEAN
			-- Is `s' a valid placeholder name?
			-- Only the length restrictions are checked here.
			-- All other restrictions are checked by the
			-- database engine.
		do
			Result := s.count >= 1 and
				(s.count <= max_placeholder_name_length - 1 or
				   s.count = max_placeholder_name_length
				   and s.item (1) = Bind_variable_start_char)
		end
	
	Bind_variable_start_char: CHARACTER is ':'
	
	is_valid_placeholder_value (value: ANY): BOOLEAN
			-- Is `value' a valid value for binding to a named
			-- or numeric placeholder?
		local
			integer_ref: INTEGER_REF
			real_ref: REAL_REF
			double_ref: DOUBLE_REF
			boolean_ref: BOOLEAN_REF
			char_ref: CHARACTER_REF
			string: STRING
			date: ORACLE_DATE
			rowid: ORACLE_ROWID
		do
			integer_ref ?= value
			real_ref ?= value
			double_ref ?= value
			boolean_ref ?= value
			char_ref ?= value
			string ?= value
			date ?= value
			rowid ?= value
			Result := value = Void or integer_ref /= Void or
				real_ref /= Void or double_ref /= Void or
				boolean_ref /= Void or char_ref /= Void or
				string /= Void or date /= Void or rowid /= Void
		end
	
	is_valid_select_position (num: INTEGER): BOOLEAN
			-- Is `num' a valid select position for a query?
		do
			Result := num >= 1 and num <= 255
		end
	
	is_valid_string_length (num: INTEGER): BOOLEAN
			-- Is `num' a valid maximum length for a query
			-- select item which is a string or raw string?
		do
			Result := num >= 0 and num <= 65536
		end
	
feature {NONE} -- Externals

	max_placeholder_name_length: INTEGER
			-- Maximum length of placeholder name
		external
			"C macro use %"oracle.h%""
		alias
			"phv_max_name_length"
		end


end
