
indexing
	
	description: "Multi-column list row that has additional information %
		%about the entity it represents that uniquely identifies %
		%the entity and associates it with a customer work order"
	author: "David Hollenberg"
	date: "June 1, 2005"

class ORDER_MULTI_COLUMN_LIST_ROW
inherit	
	MULTI_COLUMN_LIST_ROW
		rename
			make as mcl_make
		end
	HAS_ACCOUNT_NUMBER
		rename
			account_number as cust_acct_no,
			has_account_number as has_acct_no
		undefine
			copy, is_equal, default_create
		end
	HAS_ACCOUNT_STRING
		rename
			account_number as cust_acct_no,
			account_string as cust_acct_string
		undefine
			copy, is_equal, default_create
		end
	HAS_DESIGN_NUMBER
		rename
			design_number as design_no,
			has_design_number as has_design_no
		undefine
			copy, is_equal, default_create
		end
	HAS_RUN_ID
		undefine
			copy, is_equal, default_create
		end
	HAS_DIE_ID
		undefine
			copy, is_equal, default_create
		end
	HAS_PACKAGE_ID
		undefine
			copy, is_equal, default_create
		end
	HAS_DATES_SENT
		undefine
			copy, is_equal, default_create
		end
	DB_OPTIONS_ACCESS
		export
			{NONE} all
		undefine
			copy, is_equal, default_create
		end
	STRING_UTILITIES_ACCESS
		export
			{NONE} all
		undefine
			copy, is_equal, default_create
		end

create
	make

create {ORDER_MULTI_COLUMN_LIST_ROW}
	make_filled

feature {NONE} -- Creation

	make (cwo: CUST_WOS_ROW rep_mode: STRING)
		require
			cwo_not_void: cwo /= Void
			rep_mode_not_void: rep_mode /= Void
		do
			default_create
			cust_wo := cwo
			reply_mode := rep_mode
			enable_visible
		ensure
			is_visible: is_visible
		end

feature -- Properties

	cust_wo: CUST_WOS_ROW
			-- Customer work order row is for

	reply_mode: STRING
			-- Reply mode to look for in M_REPLIES table

	run_id: STRING
			-- run id
		do
			Result := fab_instance.run_id
		end

	die_id: STRING
			-- Die id
		do
			Result := fab_instance.die_id
		end

	package_id: STRING
			-- Package id (actually, comma-separated list of
			-- bonding names)
		do
			if cached_package_id = Void then
				cached_package_id := su.joined_by (cust_wo.final_diagram_bonding_names, ", ")
			end
			Result := cached_package_id
		end

	dates_sent: LIST [DB_DATE]
			-- Date(s) final bonding diagrams prepared
		do
			cache_dates_sent
			Result := cached_dates_sent
		end

	design_no: INTEGER
			-- Design number
		do
			Result := fab_instance.design_no
		end

	cust_acct_no: INTEGER
			-- Cust account number
		do
			Result := cust_wo.cust_acct_no
		end

	cust_acct_string: STRING
			-- Cust account string
		do
			Result := cust_wo.cust_acct.display_name
		end

	unique_identifier: STRING
			-- Value that uniquely identifies row
		do
			Result := die_id
		end

	has_design_no: BOOLEAN
			-- Is there an associated design number?
		do
			Result := fab_instance /= Void
		end

	has_die_id: BOOLEAN
			-- Is there an associated die id?
		do
			Result := fab_instance /= Void and then not fab_instance.is_null_die_id
		end

	has_package_id: BOOLEAN
			-- Is there an associated package id?
		do
			Result := not package_id.is_empty
		end

	has_dates_sent: BOOLEAN
			-- Is there any associated date diagrams prepared?
		do
			cache_dates_sent
			Result := not cached_dates_sent.is_empty
		end

	has_no_dates_sent: BOOLEAN
			-- Is there no associated dates diagrams prepared?
		do
			Result := not has_dates_sent
		end

	has_run_id: BOOLEAN
			-- Is there an associated run id?
		do
			Result := fab_instance /= Void and then not fab_instance.is_null_run_id
		end

	has_acct_no: BOOLEAN = True
			-- Is there an associated cust account number?

	is_pending: BOOLEAN
			-- Is internal front end request for `Current'
			-- pending (still unprocessed)?

	is_not_pending: BOOLEAN
			-- Has internal front end request for `Current'
			-- completed?
		do
			Result := not is_pending
		end

	is_mosis_still_paying: BOOLEAN
			-- Is MOSIS still paying for order?
		do
			Result := cust_wo.is_mosis_still_paying
		end

feature -- Modification

	cache_dates_sent
			-- Cache date(s) final bonding diagrams prepared,
			-- if not already present
		local
			list: LIST [M_REPLIES_ROW]
		do
			if cached_dates_sent = Void then
				list := fab_instance.replies
				create {ARRAYED_LIST [DB_DATE]} cached_dates_sent.make (list.count)
				list.do_if (agent add_date (?, cached_dates_sent), agent {M_REPLIES_ROW}.has_reply_mode (reply_mode))
			end
		end

	uncache_dates_sent
			-- Remove any cached value of `dates_sent'
		do
			cached_dates_sent := Void
			fab_instance.uncache_replies
		end

	set_is_pending (b: BOOLEAN)
			-- Set `is_pending' to `b'
		do
			is_pending := b
		end

feature {NONE} -- Implementation

	fab_instance: IC_FAB_INSTANCES_ROW
			-- Fab instance for work order
		require
			has_m_fab_no: not cust_wo.is_null_m_fab_no
		do
			Result := cust_wo.fab_instance
		end

	add_date (row: M_REPLIES_ROW list: LIST [DB_DATE])
			-- Add date processed in `row' to `list'
		do
			list.extend (row.date_processed)
		end

	cached_package_id: STRING
			-- Package ID (actually, list of bonding names)
			-- cached in memory

	cached_dates_sent: LIST [DB_DATE]
			-- Date(s) final diagrams were prepared and sent
			-- to customers, cached in memory

	new_filled_list (n: INTEGER): like Current
			-- New list with `n' elements.
		do
			create Result.make_filled (n)
		end
		
end
