class OUTGOING_MESSAGE

inherit
	MESSAGE_HEADER

	MESSAGE_SUPPORT

	MIME_SUPPORT

	STRING_UTILITIES_ACCESS		-- su.xxxx

	FILABLE

	UNIX_OS_ACCESS

create
	make,
	make_alt_mailer

feature  -- status

	sent: BOOLEAN		-- has message been sent?

feature  -- attributes

	message_body: STRING;	-- the body

	MIME_message_body: MIME_PART
				-- alternate (or additional) body
	
feature  -- change

	set_header_field(nam,val: STRING) is
	require nam /= Void and val /= Void
	do  header_fields.put(val, nam);
	end;

	set_header_list(nam: STRING;  val: LIST[STRING]) is
	require nam /= Void and val /= Void
	do  set_header_field(nam, folded(val))
	end;

	set_message_body(str: STRING) is
			-- Void allowed
	do  message_body := str;
	end;

	set_MIME_message_body(a_mpart: MIME_PART) is
			-- Void allowed
	do  MIME_message_body := a_mpart;
	    -- and add the MIME-version to the headers
	    if a_mpart /= Void and 
		message_header(msg_MIME_version) = Void
	     then set_header_field(msg_MIME_version, the_MIME_version)
	    end;
	end;

feature  -- list folding

	folded(lst: LIST[STRING]): STRING is
	require lst /= Void
	local len: INTEGER;	-- length of the last line of Result
	      str: STRING;
	do  create Result.make(0);
	    from  lst.start;
	    until lst.after
	    loop  str := lst.item;
		  if Result.count > 0		-- not first item
		   then Result.extend(',');
			if len+str.count > 66
			 then Result.append("%N       "); len := 0;
			 else Result.extend(' ');
			end;
		  end;
		  Result.append(str);
		  len := len + 2 + str.count;
		  lst.forth;
	    end;
	end;

feature  -- send message

	send is
	require	have_addressee: message_header(msg_to) /= Void or
				message_header(msg_cc) /= Void;
	local  mlr: UNIX_PROCESS;  pipe: LARGE_FILE;
	do  -- create a mailer_prog subprocess
	    mlr := unix_os.new_process(mailer_prog);
	    mlr.set_arguments(mailer_args);
	    mlr.set_input_piped;
	    -- FIX_ME: mlr.error and mlr.output ??
	    -- let it rip
	    mlr.spawn_nowait;
	    -- now give it the message
	    pipe := mlr.input_to_child;
	    copy_to_file(pipe);
	    pipe.close;
	    -- and shut down
	    mlr.get_status_block;
	    mailer_status := mlr.status;
	    sent := true;
	ensure
	    sent = true
	end;

feature  -- our FILABLE deferred items

	copy_to_consumer(c: DATA_CONSUMER) is
		-- pump the message into c
	local mbody: MIME_PART
	do  copy_headers(c);
	    if MIME_message_body = Void
	     then c.consume(su.eol);	-- body separator
		  if message_body /= Void
		   then c.consume(message_body)
		  end
	     else if message_body /= Void
		   then mbody := merge_into_single_body
		   else mbody := MIME_message_body
		  end;
		  -- (mbody puts its top-level headers into the
		  --  message header, and then adds a body separator)
		  mbody.copy_to_consumer(c)
	    end;
	end;

	close is
	do  if MIME_message_body /= Void
	     then -- pass it on down the line
		  MIME_message_body.close
		  MIME_message_body := Void
	    end;
	end;

feature {NONE}

	merge_into_single_body: MIME_PART is
		-- have message_body AND MIME_message_body; 
		--  combine into multipart Result
	local prolog: MIME_SIMPLEPART; body: MIME_MULTIPART
	do  -- combine into a new body
	    create prolog.make_string(mtype_is_text, mtext_is_plain,
					message_body)
	    create body.make_mixed
	    body.extend(prolog);
	    body.extend(MIME_message_body)
	    Result := body
	end;

	default_mailer_command: STRING is "/usr/lib/sendmail -t -oi"

	make is
	do  make_alt_mailer(Void);  -- default
	end;

	make_alt_mailer(mail_cmd: STRING) is
	local  mc: STRING; l: LIST[STRING]; i: INTEGER;
	do  -- create empty message header
	    create header_fields.make(0);
	    -- establish mail command
	    mc := mail_cmd;
	    if su.is_empty(mc)
	     then mc := default_mailer_command;
	    end;
	    -- set up mailer_prog & mailer_args from mc (mail_cmd or default)
	    l := su.broken_into_words(mc);
	    l := su.empty_strings_removed(l);
	    l.start;
	    mailer_prog := l.item;
	    create mailer_args.make(1, l.count-1)
	    from  l.forth;		-- step over the mailer_prog
	    until l.after
	    loop  i := i + 1;
		  mailer_args.enter(l.item, i);
		  l.forth;
	    end;
	end;

	mailer_prog: STRING		-- name of mailer program

	mailer_args: ARRAY[STRING]	-- args for mailer program

	mailer_status: INTEGER		-- terminating status for last send

	copy_headers(c: DATA_CONSUMER) is
	local flds: ARRAY[STRING]; i: INTEGER; wrd: STRING;
	do  flds := message_header_field_names;
	    from  i := flds.lower
	    until i >  flds.upper
	    loop  wrd := (flds @ i);
		  c.consume(header_line(wrd,
					folded(message_header(wrd))));
		  i := i + 1;
	    end;
	end;

end  -- OUTGOING_MESSAGE
