indexing
	description:
			"Basic package bonding connections."
	author: 	"Barden E. Smith"
	date:		"August 30, 2004"
	instructions:	"[
		This class provides the canonical representation for a package
		connection.  The orienation consists of three BOOLEAN values and
		two real values.  The symantics of the orientation is to apply
		the mirroring (`is_mirrored_in_x', `is_mirrored_in_y'), and then
		the rotation (`is_rotated_90ccw') followed by a translation
		(`bonding_finger_location_x', `bonding_finger_location_y'). This
		is different from the canonical GDS representation!
		]"

deferred class PACKAGE_CONNECTION

inherit
	BONDING_POINT
		rename
			x as bonding_finger_location_x,
			y as bonding_finger_location_y,
			name as bonding_finger_id
		end

feature -- Essence

	bonding_finger_id: STRING is
		deferred
		end

	pin_id: STRING is
		deferred
		end

	bonding_finger_location_x: DOUBLE is
		deferred
		end

	bonding_finger_location_y: DOUBLE is
		deferred
		end

	is_layout_mirrored_in_x: BOOLEAN is
		deferred
		end

	is_layout_mirrored_in_y: BOOLEAN is
		deferred
		end

	is_layout_rotated_90ccw: BOOLEAN is
		deferred
		end

feature -- Status

	bonding_finger_location_x_mils: DOUBLE is
		do
			Result := bonding_finger_location_x / 25.4
		end

	bonding_finger_location_y_mils: DOUBLE is
		do
			Result := bonding_finger_location_y / 25.4
		end

	orientation: MATRIX is
		do
			create Result.make_mx_my_rot90_tx_ty (
				is_layout_mirrored_in_x,
				is_layout_mirrored_in_y,
				is_layout_rotated_90ccw,
				bonding_finger_location_x,
				bonding_finger_location_y)
		ensure
			result_cif_legal: Result.is_cif_legal
			result_manhattan: Result.is_manhattan
		end

end -- class PACKAGE_CONNECTION