
indexing
	
	description: "Window to provide information about packaging on a run"
	author: "David Hollenberg"
	date: "March 16, 2007"

class PACKAGE_INFO_WINDOW

inherit
	PACKAGE_INFO_IFC
	WINDOW_INTERFACE
		redefine
			enable_active,
			disable_active
		end
	MOSIS_GRAPHICAL_UTILITIES
		export
			{NONE} all
		undefine
			default_create
		end

create
	make

feature {NONE} -- Creation

	make (w: EV_TITLED_WINDOW)
			-- Create interface in window `w'
		require
			window_exists: w /= Void
			window_valid: not w.is_destroyed
		do
			make_interface
			make_proxy (w)
		end

feature -- Operations

	reset
			-- Move focus back into window
		do
			enable_active
		end

feature -- Modification

	enable_active
			-- Make interface active
		do
			precursor
			cancel_button.enable_sensitive
		end

	disable_active
			-- Make interface inactive
			-- (can look but cannot initiate most actions)
		do
			precursor
			cancel_button.disable_sensitive
		end

feature {NONE} -- Implementation

	build_interface
			-- Build interface
		local
			main_vbox: EV_VERTICAL_BOX
			buttons: HORIZONTAL_BUTTON_SET
			box: EV_HORIZONTAL_BOX
		do
			fetch_data
			create main_vbox
			main_vbox.set_padding (user_interface.padding)
			main_vbox.set_border_width (user_interface.border_width)
			build_project_packages

			create box
			box.set_border_width (user_interface.border_width)
			box.extend (project_packages)
			main_vbox.extend (box)
			
			create buttons.make_with_text_and_actions (<< "Cancel" >>,
				<< agent process_cancel >>)
			cancel_button := buttons.i_th (1)
			
			box := buttons.column_set.box
			
			main_vbox.extend (box)
			main_vbox.disable_item_expand (box)
			
			gui.set_box_background_recursive (main_vbox, background_color)
			extend (main_vbox)
			close_request_actions.extend (agent process_cancel)
			gui.add_cancel_accelerator (window, agent process_cancel)
			gui.do_once_on_idle (agent post_display_actions)
			populate_project_packages
		end
		
	fetch_data
			-- Fetch list of designs and their packages and
			-- put in `packagings'
		local
			q: SQL_QUERY
			dno: INTEGER
			waf_etest_id, waf_process, pkg_id, short_id: STRING
			exp_control_type: STRING
			plastic: BOOLEAN
			pkging: DESIGN_AND_PACKAGE
		do
			create {ARRAYED_LIST [DESIGN_AND_PACKAGE]} packagings.make (100)
			q := session.connection.sql_query (designs_and_packages_query)
			q.set_numeric_bind_capacity (1)
			q.bind_num_to_string (1, session.run_id, False)
			q.set_select_count (7)
			q.select_integer (1)
			q.select_string (2, 100)
			q.select_string (3, 100)
			q.select_string (4, 100)
			q.select_string (5, 100)
			q.select_boolean (6)
			q.select_string (7, 100)
			q.execute
			from
				q.fetch
			until
				q.end_of_data
			loop
				dno := q.integer_value (1)
				exp_control_type := q.string_value (2)
				waf_etest_id := q.string_value (3)
				waf_process := q.string_value (4)
				pkg_id := q.string_value (5)
				plastic := q.boolean_value (6)
				short_id := q.string_value (7)
				
				create pkging
				pkging.set_design_no (dno)
				pkging.set_export_control_type (exp_control_type)
				pkging.set_wafer_etest_id (waf_etest_id)
				pkging.set_wafer_process (waf_process)
				pkging.set_package_id (pkg_id)
				pkging.set_is_plastic (plastic)
				pkging.set_manufacturer_short_id (short_id)
				packagings.extend (pkging)
				q.fetch
			end
			q.release
		end

	build_project_packages
			-- Build `project_packages'
		do
			create project_packages
			project_packages.enable_multiple_selection
			project_packages.set_row_to_generic_function (agent row_to_binding_record)
			project_packages.set_visibility_counts_included (False)
			project_packages.build_selection_status_box
			project_packages.set_visible_columns (package_columns)
		end

	populate_project_packages
			-- Populate `project_packages'
		local
			list: LIST [PACKAGE_MULTI_COLUMN_LIST_ROW]
		do
			create {ARRAYED_LIST [PACKAGE_MULTI_COLUMN_LIST_ROW]} list.make (packagings.count)
			packagings.do_all (agent add_package_row (?, list))
			project_packages.set_all_rows (list)
		end
			
	add_package_row (a_package: DESIGN_AND_PACKAGE list: LIST [PACKAGE_MULTI_COLUMN_LIST_ROW])
			-- Add a row for `a_package' to `list'
		local
			row: PACKAGE_MULTI_COLUMN_LIST_ROW
		do
			create row.make (a_package)
			list.extend (row)
		end
			
	post_display_actions
		local
			screen: EV_SCREEN
		do
			reset
			create screen
			gui.do_once_on_idle (agent gui.center_window_with_size (window, screen.width - 200, 500))
		end

	process_cancel
		do
			if is_active then
				disable_active
				cancel_actions.call ([])
			end
		end

	package_columns: like anchored_package_columns
			-- List of columns to display
		do
			create Result.make (4)
			Result.extend (design_no_column)
			Result.extend (export_control_type_column)
			Result.extend (wafer_etest_id_column)
			Result.extend (wafer_process_column)
			Result.extend (package_id_column)
			Result.extend (is_plastic_column)
			Result.extend (package_manufacturer_column)
		end
			
	design_no_column: DESIGN_NO_COLUMN [PACKAGE_MULTI_COLUMN_LIST_ROW]
		do
			create Result
		end
	
	export_control_type_column: EXPORT_CONTROL_TYPE_COLUMN [PACKAGE_MULTI_COLUMN_LIST_ROW]
		do
			create Result
		end
	
	wafer_etest_id_column: WAFER_ETEST_ID_COLUMN [PACKAGE_MULTI_COLUMN_LIST_ROW]
		do
			create Result
		end
	
	wafer_process_column: WAFER_PROCESS_COLUMN [PACKAGE_MULTI_COLUMN_LIST_ROW]
		do
			create Result
		end
	
	package_id_column: PACKAGE_ID_COLUMN [PACKAGE_MULTI_COLUMN_LIST_ROW]
		do
			create Result
		end
	
	is_plastic_column: IS_PLASTIC_COLUMN [PACKAGE_MULTI_COLUMN_LIST_ROW]
		do
			create Result
		end
	
	package_manufacturer_column: PACKAGE_MANUFACTURER_COLUMN [PACKAGE_MULTI_COLUMN_LIST_ROW]
		do
			create Result
		end
	
	row_to_binding_record (row: EV_MULTI_COLUMN_LIST_ROW): PACKAGE_MULTI_COLUMN_LIST_ROW
			-- Function which translates `row' to object of
			-- type PACKAGE_MULTI_COLUMN_LIST_ROW
		do
			Result ?= row
			check
				result_not_void: Result /= Void
			end
		end

	designs_and_packages_query: STRING = "[
select	ifi.design_no,
	cid.export_control_type,
	wp.wafer_etest_id,
	ifip.wafer_process,
	mp.package_id,
	p.leadframe,
	p.manufacturer_short_id
  from	ic_fab_instances ifi,
	ic_fab_instance_processes ifip,
	cust_ic_designs cid,
	wafer_processes wp,
	cust_wos cw,
	cust_wo_line_items cwli,
	m_parts mp,
	packages p
 where	ifi.m_fab_no = ifip.m_fab_no
   and  ifip.wafer_process = wp.wafer_process
   and  ifi.design_no = cid.design_no
   and  ifi.m_fab_no = cw.m_fab_no
   and  cw.m_order_no = cwli.m_order_no
   and  cwli.m_part_no = mp.m_part_no
   and  mp.package_id = p.package_id
   and  mp.part_type = 'PACKAGE'
   and  mp.packaged = 'Y'
   and  ifi.run_id = :1
 order by
	ifip.wafer_process, ifi.design_no, mp.package_id
		]"

	anchored_package_columns: ARRAYED_LIST [MULTI_COLUMN_LIST_COLUMN [PACKAGE_MULTI_COLUMN_LIST_ROW]]
			-- For anchoring only
		do
		ensure
			result_void: Result = Void
		end
	
	packagings: LIST [DESIGN_AND_PACKAGE]
			-- List of designs and their process and package
			-- requirements

	cancel_button: EV_BUTTON
			-- Button to hide window
	
	project_packages: MULTI_COLUMN_LIST [PACKAGE_MULTI_COLUMN_LIST_ROW]
			-- List of fab instances on run and their package
			-- requirements
	
end
