
indexing
	
	description: "Multi-column list row that has information about a %
		%project and one of its packages"
	author: "David Hollenberg"
	date: "April 13, 2007"

class PACKAGE_MULTI_COLUMN_LIST_ROW
inherit	
	MULTI_COLUMN_LIST_ROW
		rename
			make as mcl_make
		end
	HAS_DESIGN_NUMBER
		rename
			design_number as design_no,
			has_design_number as has_design_no
		undefine
			copy, is_equal, default_create
		end
	HAS_WAFER_ETEST_ID
		undefine
			copy, is_equal, default_create
		end
	HAS_EXPORT_CONTROL_TYPE
		undefine
			copy, is_equal, default_create
		end
	HAS_WAFER_PROCESS
		undefine
			copy, is_equal, default_create
		end
	HAS_PACKAGE_ID
		undefine
			copy, is_equal, default_create
		end
	HAS_IS_PLASTIC
		undefine
			copy, is_equal, default_create
		end
	HAS_PACKAGE_MANUFACTURER
		undefine
			copy, is_equal, default_create
		end

create
	make

create {PACKAGE_MULTI_COLUMN_LIST_ROW}
	make_filled

feature {NONE} -- Creation

	make (dp: DESIGN_AND_PACKAGE)
		require
			dp_not_void: dp /= Void
		do
			default_create
			design_package := dp
			enable_visible
		ensure
			is_visible: is_visible
		end

feature -- Properties

	design_package: DESIGN_AND_PACKAGE
			-- Design and associated package in which to package it

	design_no: INTEGER
			-- Design number
		do
			Result := design_package.design_no
		end

	export_control_type: STRING
			-- Export control type for design
		do
			Result := design_package.export_control_type
		end

	wafer_etest_id: STRING
			-- Lot split to be ordered
		do
			Result := design_package.wafer_etest_id
		end

	wafer_process: STRING
			-- Process to be ordered
		do
			Result := design_package.wafer_process
		end

	package_id: STRING
			-- Package to use
		do
			Result := design_package.package_id
		end

	is_plastic: BOOLEAN
			-- Is package a plastic package?
		do
			Result := design_package.is_plastic
		end

	package_manufacturer: STRING
			-- Short ID of package manufacturer
		do
			Result := design_package.manufacturer_short_id
		end

	unique_identifier: STRING
			-- Value that uniquely identifies row
		do
			Result := design_no.out + "%U" + wafer_process + "%U" + package_id
		end

	has_design_no: BOOLEAN = True
			-- Is there an associated design number?

	has_export_control_type: BOOLEAN = True
			-- Is there an associated export control type?

	has_wafer_etest_id: BOOLEAN = True
			-- Is there an associated wafer etest ID?

	has_wafer_process: BOOLEAN = True
			-- Is there an associated wafer process?

	has_package_id: BOOLEAN = True
			-- Is there an associated package ID?

	has_is_plastic: BOOLEAN = True
			-- Is there an associated "is plastic" indicator?

	has_package_manufacturer: BOOLEAN = True
			-- Is there an associated package manufacturer
			-- short ID?

feature {NONE} -- Implementation

	new_filled_list (n: INTEGER): like Current
			-- New list with `n' elements.
		do
			create Result.make_filled (n)
		end
		
end
