
indexing
	
	description: "Window to allow user to create initial packaging lots for %
		%a run"
	author: "David Hollenberg"
	date: "March 1, 2007"

class PACKAGING_LOT_CREATION_WINDOW

inherit
	PACKAGING_LOT_CREATION_IFC
	WINDOW_INTERFACE
	MOSIS_GRAPHICAL_UTILITIES
		export
			{NONE} all
		undefine
			default_create
		end
	RC_POLICIES_ACCESS
		export
			{NONE} all
		undefine
			default_create
		end
		
create
	make

feature {NONE} -- Creation

	make (w: EV_TITLED_WINDOW)
			-- Create interface in window `w'
		require
			window_exists: w /= Void
			window_valid: not w.is_destroyed
		do
			make_interface
			make_proxy (w)
		end

feature -- Operations

	reset
			-- Reset interface
		do
		end

feature {NONE} -- Implementation

	build_interface
			-- Build interface in `window'
		local
			main_vbox: EV_VERTICAL_BOX
			buttons: HORIZONTAL_BUTTON_SET
			box: EV_HORIZONTAL_BOX
		do
			fetch_data
			create main_vbox
			main_vbox.set_border_width (user_interface.border_width)
			-- Build display for packaging lots

			build_packaging_lots
			main_vbox.extend (last_box)
			main_vbox.disable_item_expand (last_box)

			create buttons.make_with_text_and_actions (<< gui.ev_ok, "Info", gui.ev_cancel >>, << agent process_accept, agent process_info, agent process_decline >>)
			info_button := buttons.i_th (2)
			info_button.set_tooltip ("Display information about projects on run and their required packages")
			box := buttons.column_set.box
			
			main_vbox.extend (box)
			main_vbox.disable_item_expand (box)
			gui.set_box_background_recursive (main_vbox, background_color)
			extend (main_vbox)

			close_request_actions.extend (agent process_decline)
			gui.add_cancel_accelerator (window, agent process_decline)
			enable_active
			gui.do_once_on_idle (agent gui.center_window (window))
		end
		
	fetch_data
			-- Fetch needed data from database
		local
			org_names_row: ORG_NAMEXORG_TYPES_ROW
			org_names_rows: LIST [ORG_NAMEXORG_TYPES_ROW]
			org_type: ORG_TYPE_VALUES
			proc_row: WAFER_PROCESSES_ROW
		do
			packaging_order_types := possible_packaging_order_types
			create proc_row.make_template
			processes := proc_row.processes_for_production (cache.production_id, connection, expected_count.At_least_zero)
			virtual_packaging_org_no := policies.virtual_packaging_org_no
			create org_names_row.make_template
			org_names_rows := org_names_row.rows_for_org_type (org_type.Pkg_vendor, connection, expected_count.More_than_one)
			create {ARRAYED_LIST [ORG_NAMEXORG_TYPES_ROW]} packaging_vendors.make (org_names_rows.count)
			org_names_rows.do_all (agent add_org_names_row)
		end

	add_org_names_row (row: ORG_NAMEXORG_TYPES_ROW)
			-- Add `row' to `packaging_vendors' if it is not
			-- the virtual packaging organization.  Otherwise,
			-- set `virtual_packaging_vendor' from `row'
		do
			if row.org_no = virtual_packaging_org_no then
				virtual_packaging_vendor := row
			else
				packaging_vendors.extend (row)
			end
		end

	possible_packaging_order_types: LIST [STRING]
			-- Possible packaging order types
		local
			b: DB_BIND_S
			order_type: PACKAGING_ORDER_TYPE_VALUES
		do
			create b.make (order_type.Virtual)
			Result := connection.selected_strings ("packaging_order_type", "packaging_order_types", "packaging_order_type <> :1", "packaging_order_type", False, b)
			if Result.is_empty then
				handle_hard_error ("no packaging order types that are not " + order_type.Virtual)
			end
		end

	build_packaging_lots
			-- Build portion of window with packaging lots
			-- (one row per wafer lot with a field for every
			-- non-VIRTUAL packaging order type)
		local
			cols: WIDGET_COLUMN_SET
			row: ARRAY [EV_WIDGET]
			k, n: INTEGER
			field: EV_COMBO_BOX
			list: LIST [EV_COMBO_BOX]
			proc: WAFER_PROCESSES_ROW
		do
			n := packaging_order_types.count + 2
			create cols.make (n)
			cols.align_columns_left
			cols.set_border_width (user_interface.border_width)
			cols.set_padding (user_interface.padding)
			create row.make (1, n)
			
			-- Add row with titles

			row.put (gui.label_with_text ("Wafer Lot"), 1)
			row.put (gui.label_with_text ("Ordered Process"), 2)
			from
				k := 3
			until
				k > n
			loop
				row.put (gui.label_with_text (packaging_order_types.i_th (k - 2)), k)
				k := k + 1
			end
			cols.append_row (row)
			
			-- Add row for each distinct process in wafer lots
			
			create process_packaging_vendors.make (processes.count)
			from
				processes.start
			until
				processes.after
			loop
				proc := processes.item
				field := new_packaging_org_box
				row.put (new_field (proc.wafer_etest_id, 15), 1)
				row.put (new_field (proc.wafer_process, 20), 2)
				create {ARRAYED_LIST [EV_COMBO_BOX]} list.make (n - 1)
				from
					k := 3
				until
					k > n
				loop
					field := new_packaging_org_box
					field.set_data (packaging_order_types.i_th (k - 2))
					row.put (field, k)
					list.extend (field)
					k := k + 1
				end
				cols.append_row (row)
				process_packaging_vendors.put (list, proc.wafer_process)
				processes.forth
			end

			cols.enable_expand
			last_box := cols.box
		end
		
	new_packaging_org_box: EV_COMBO_BOX
			-- New combo box with list of possible packaging
			-- vendor organizations
		local
			fact: SINGLE_CONSTRAINED_VALUE_FACTORY
			default_choice: STRING
			vendor: ORG_NAMEXORG_TYPES_ROW
		do
			default_choice := ""
			create fact
			fact.add_choice_and_text (default_choice, "None")
			from
				packaging_vendors.start
			until
				packaging_vendors.after
			loop
				vendor := packaging_vendors.item
				fact.add_choice_and_text (vendor.org_no.out, vendor.short_id)
				packaging_vendors.forth
			end
			fact.set_default_choice (default_choice)
			fact.generate_combo_box
			Result := fact.combo_box
			Result.disable_edit
		end
		
	process_info
			-- Take actions needed when user requests information
			-- about projects and their packages
		local
			w: EV_TITLED_WINDOW
		do
			if is_active then
				disable_active
				if package_info_interface = Void then
					create w
					package_info_interface ?= session.factory.package_info_interface (w, session.run_id, session.title_with_run_id ("Packaged Projects"))
					package_info_interface.set_session (session)
					package_info_interface.build
					package_info_interface.cancel_actions.extend (agent process_info_decline)
				end
				package_info_interface.raise
				enable_active
			end
		end

	process_info_decline
			-- Process a decline request on Info window
		do
			package_info_interface.hide
			package_info_interface.enable_active
		end

	process_accept
			-- Take actions needed when user accepts
			-- current operation
		local
			operation: ANONYMOUS_OPERATION
		do
			if is_active then
				disable_active
				create operation.make (agent process_accept_request)
				operation.enable_abort_caught
				operation.execute
				if operation.aborted then
					session.rollback
					enable_active
				else
					destroy_dependent_interfaces
					success_actions.call ([])
				end
			end
		end

	process_accept_request
			-- Process an accept request
		do
			confirm_order_types
			create_packaging_lots
		end

	confirm_order_types
			-- Tell user how many of each type of packaging
			-- lot will be created and get confirmation
		local
			msg: STRING
			ord_types: LIST [STRING]
			proc: WAFER_PROCESSES_ROW
		do
			create msg.make (200)
			from
				processes.start
			until
				processes.after
			loop
				proc := processes.item
				ord_types := order_types_for_process (proc.wafer_process)
				
				msg.append ("Packaging lots for ")
				msg.append (proc.wafer_etest_id)
				msg.append (":  ")
				msg.append (su.joined_by (ord_types, ", "))
				msg.extend ('%N')
				processes.forth
			end
			msg.append ("%NCreate these packaging lots")
			confirm (msg)
		end
	
	create_packaging_lots
			-- Create one or more packaging lots for each
			-- process
		do
			last_lot_number := 0
			processes.do_all (agent create_lots_for_process)
		end

	create_lots_for_process (a_process: WAFER_PROCESSES_ROW)
			-- Create one or more packaging lots for `a_process'
		local
			list: LIST [EV_COMBO_BOX]
			field: EV_COMBO_BOX
			order_type: PACKAGING_ORDER_TYPE_VALUES
			found: BOOLEAN
			l_process: STRING
		do
			l_process := a_process.wafer_process
			process_packaging_vendors.search (l_process)
			if process_packaging_vendors.found then
				list := process_packaging_vendors.found_item
				from
					list.start
				until
					list.after
				loop
					field := list.item
					if not field_selected_org_no (field).is_empty then
						found := True
						create_one_packaging_lot (l_process, field_order_type (field), field_selected_org_no (field).to_integer, False)
					end
					list.forth
				end
				if not found and not cache.is_dedicated_run then
					create_one_packaging_lot (l_process, order_type.Virtual, virtual_packaging_org_no, True)
				end
			else
				handle_internal_error ("cannot find list of packaging vendor organizations for process " + l_process)
			end
		end

	create_one_packaging_lot (a_process, an_order_type: STRING an_org_no: INTEGER die_form: BOOLEAN)
			-- Create one packaging lot for `a_process' with
			-- `an_order_type' to be sent to organization with
			-- `an_org_no'
		local
			lot: RUN_PACKAGING_LOTS_MROW
		do
			last_lot_number := last_lot_number + 1
			create lot.make_null (connection)
			lot.set_run_id (session.run_id)
			lot.set_packaging_lot_no (last_lot_number)
			lot.set_production_id (cache.production_id)
			lot.set_packaging_org_no (an_org_no)
			lot.set_packaging_order_type (an_order_type)
			lot.set_wafer_process (a_process)
			lot.set_is_die_form (die_form)
			lot.set_is_actual_date_sent (False)
			lot.set_is_actual_date_received (False)
		end

	process_decline
			-- Take actions needed when user declines
			-- current operation
		do
			if is_active then
				disable_active
				destroy_dependent_interfaces
				abort_actions.call ([])
			end
		end

	destroy_dependent_interfaces
			-- Destroy any interfaces which depend on
			-- `Current'
		do
			if package_info_interface /= Void then
				package_info_interface.destroy
			end
		end

	order_types_for_process (proc: STRING): LIST [STRING]
			-- List of packaging order types and short ID of
			-- packaging vendor for which
			-- packaging lots will be created for process `proc'
		local
			list: LIST [EV_COMBO_BOX]
			field: EV_COMBO_BOX
			order_type: PACKAGING_ORDER_TYPE_VALUES
		do
			process_packaging_vendors.search (proc)
			if process_packaging_vendors.found then
				list := process_packaging_vendors.found_item
				create {ARRAYED_LIST [STRING]} Result.make (list.count)
				from
					list.start
				until
					list.after
				loop
					field := list.item
					if not field_selected_org_no (field).is_empty then
						Result.extend (order_description (field_order_type (field), field_selected_short_id (field)))
					end
					list.forth
				end
				if Result.is_empty then
					if not cache.is_dedicated_run then
						Result.extend (order_description (order_type.Virtual, virtual_packaging_vendor.short_id))
					else
						Result.extend ("NONE")
					end
				end
			else
				handle_internal_error ("cannot find list of packaging vendor organizations for process " + proc)
			end
		end

	order_description (ord_type, short_id: STRING): STRING
			-- Description for packaging order of type `ord_type'
			-- to be sent to organization with `short_id'
		do
			create Result.make (30)
			Result.append (ord_type)
			Result.append (" (")
			Result.append (short_id)
			Result.extend (')')
		end

	field_order_type (field: EV_COMBO_BOX): STRING
			-- Packaging order type for `field'
		require
			field_not_void: field /= Void
		do
			Result ?= field.data
			check
				result_not_void: Result /= Void
			end
		end

	field_selected_org_no (field: EV_COMBO_BOX): STRING
			-- Packaging org number for `field', as a string
		require
			field_not_void: field /= Void
		do
			Result ?= field.selected_item.data
			check
				result_not_void: Result /= Void
			end
		end

	field_selected_short_id (field: EV_COMBO_BOX): STRING
			-- Packaging organization short ID selected for
			-- `field'
		require
			field_not_void: field /= Void
		do
			Result := field.selected_item.text
		end

	new_field (s: STRING min_width: INTEGER): EV_TEXT_FIELD
			-- New text field with initial text `s' with
			-- minimum width in characters `min_width' that
			-- is not editable
		do
			create Result.make_with_text (s)
			Result.set_minimum_width_in_characters (min_width)
			Result.disable_edit
		ensure
			result_exists: Result /= Void
		end

	packaging_order_types: LIST [STRING]
			-- Possible packaging order types, excluding
			-- type VIRTUAL

	processes: LIST [WAFER_PROCESSES_ROW]
			-- List of distinct wafer processes among wafer lots
			-- for production

	virtual_packaging_org_no: INTEGER
			-- Org number for organization used for
			-- packaging lot of type VIRTUAL

	packaging_vendors: LIST [ORG_NAMEXORG_TYPES_ROW]
			-- Organizations that are packaging vendors, excluding
			-- the organization whose org_no is
			-- `virtual_packaging_org_no'

	virtual_packaging_vendor: ORG_NAMEXORG_TYPES_ROW
			-- Organization for virtual packaging

	process_packaging_vendors: HASH_TABLE [LIST [EV_COMBO_BOX], STRING]
			-- Lists of packaging vendors and packaging order
			-- types (stored within combo box), indexed by
			-- wafer process name

	last_lot_number: INTEGER
			-- Number of last packaging lot created

	info_button: EV_BUTTON
			-- Button to display information about projects
			-- and their required packages

	package_info_interface: PACKAGE_INFO_WINDOW
			-- Interface to display information about projects
			-- and their required packages

	last_box: EV_HORIZONTAL_BOX
			-- Last box built

end
