
indexing
	
	description: "Utility to generate IC pads and package (cavity) in (CIF) file.";
	author: "Barden E. Smith";
	date: "April 15, 2005"

class
	PACKAGING_PAD_WRITER

inherit
	ANY
	SIMPLE_LAYOUT_IO
		export
			{ANY} the_messages
			{NONE} all
		redefine
			db, dump_the_messages, the_technology
		end

	LIBRARY_SUPPORT
		export
			{NONE} all
		redefine
			db
		end

	DB_OPTIONS_ACCESS
		export
			{NONE} all
		end

create
	make

feature {NONE} -- Creation

	make (icp: IC_PACKAGINGS_ROW conn: DB_CONNECTION) is
		require
			icp_not_void: icp /= Void
			conn_not_void: conn /= Void
		do
			ic_packaging := icp
			connection := conn
			lioopt.set_expand_root_structure(True)
		end

feature -- Access

	build_library is
		require
			has_technology: has_technology
		local
			ifi: IC_FAB_INSTANCES_ROW
			cut_die: CUT_DIE_SITES_ROW
			cid: CUST_IC_DESIGNS_ROW
			pad: CUST_IC_DESIGN_PADS_ROW
		do
			-- fetch data rows
			ifi := ic_packaging.fab_instance
			cid := ifi.cust_ic_design
			cust_ic_design := cid
			create pad.make_template
			pads := pad.rows_for_design_no (ifi.design_no, db, expected_count.At_least_zero)
			create cut_die.make_template
			cut_dies := cut_die.rows_for_ic_packaging (ic_packaging.ic_packaging_no, db, expected_count.At_least_zero)
			if cut_dies.is_empty then
				cut_dies := cut_die.rows_for_die (ifi.run_id, ifi.die_id, db, expected_count.At_least_zero)
			end
			-- build library
			create chip.make (ic_packaging.bonding_name)
			chip.set_body (chip_body)
			-- create a library for it
			create the_library.make (chip.name, one_micron)
			the_library.set_technology (the_technology)
			the_library.add_structure (chip)
			the_library.set_root_structure (ic_packaging.bonding_name)
		end

	has_technology: BOOLEAN is
		do
			Result := the_technology /= Void
		end

	output_as_cif (fname: STRING) is
		require
			have_technology: has_technology
		do
			write_cif_file (fname)
			-- since dump_the_messages has be no-op'd check for and report errors
		end

feature -- Modification

	set_default_technology (conn: DB_CONNECTION) is
		require
			conn_not_void: conn /= Void
			conn_is_open: conn.is_open
		do
			set_technology (default_technology (conn))
		end

	set_technology (t: like the_technology) is
		require
			t_not_void: t /= Void
		do
			the_technology := t
		ensure
			has_technology: has_technology
		end

feature {NONE} -- Implementation

	the_technology: DESIGN_TECHNOLOGY
			-- Design technology for CIF writer

	ic_packaging: IC_PACKAGINGS_ROW
			-- Packaging Row; other rows derived from this

	connection: DB_CONNECTION
			-- Connection to use to access the database

	chip: LIBRARY_STRUCTURE
			-- Place to build the design structure

	cust_ic_design: CUST_IC_DESIGNS_ROW
			-- Customer's IC design row

	cut_dies: LIST [CUT_DIE_SITES_ROW]
			-- All of the cut dies sites

	pads: LIST [CUST_IC_DESIGN_PADS_ROW]
			-- List of pads in design for packaging

	default_technology (conn: DB_CONNECTION): like the_technology is
		local dtv: DESIGN_TECHN_VALUES
		do
			create Result.make_by_key (conn, old_row, dtv.scn_10)
		end

	chip_body: LAYOUT_COLLECTION is
		local
			cid: CUST_IC_DESIGNS_ROW
			die_id_box: CONST_RECTANGLE
			lr: LAYOUT_RECTANGLE
			m0, m: MATRIX
			sf: DOUBLE
			pkg: PACKAGES_ROW
			pad: CUST_IC_DESIGN_PADS_ROW
			pad_layout, die_layout: LAYOUT_COLLECTION
		do
			pkg := ic_packaging.my_package
			cid := ic_packaging.fab_instance.cust_ic_design
			create m.make_translation_xy (-cid.center_x, -cid.center_y)
			if not ic_packaging.is_null_design_package_rotation and
					ic_packaging.design_package_rotation /= 0 then
				m := m.and_then (create {DRW_MATRIX}.make_rotation_degrees
					( - ic_packaging.design_package_rotation))
			end
			if pkg.leadframe then
				if pkg.is_null_autocad_scale_factor then
					sf := 1.0
				else
					sf := pkg.autocad_scale_factor
				end
				create m0.make_magnification (1.0 / (25.4 * sf))
				m := m.and_then (m0)
			else
				create m0.make_identity
			end
			if pads.is_empty then
				create Result.make_layered (2)
			else
				create Result.make_layered (3)
				create pad_layout.make_unlayered (pads.count)
				from
					pads.start
				until
					pads.after
				loop
					pad := pads.item
					create lr.make_by_size_and_center (pad.size_x, pad.size_y, pad.center_x, pad.center_y)
					pad_layout.extend (lr)
					pads.forth
				end
				Result.extend (layer (create {LAYOUT_RELOCATION}.make (pad_layout,m), glass))
			end
			create die_layout.make_layered (2)
			create lr.make_copy (die_size)
			die_layout.extend (layer (lr, metal1))
			-- a 10x5 mil box at the right 1 mil down from the top
			create die_id_box.make_by_size_and_center (
						254.0, 127.0,
						lr.high_x - 127.0, lr.high_y - 88.9) -- 3.5 * 25.4
			die_layout.extend (layer ((text (ic_packaging.die_id)).nicely_fitting_in (die_id_box), contact))
			Result.extend (create {LAYOUT_RELOCATION}.make (die_layout,m))
			create lr.make_copy (pkg.cavity_in_um)
			Result.extend (layer (create {LAYOUT_RELOCATION}.make (lr, m0), metal2))
		end

	die_size: RECTANGLE is
		require
			cid_not_void: cust_ic_design /= Void
			cut_dies_not_void: cut_dies /= Void
		do
			if not cut_dies.is_empty then
				create Result.make_by_size_and_center (
						0, 0,
						cust_ic_design.center_x, cust_ic_design.center_y)
				cut_dies.do_all (Agent stretch_die (?, Result))
			else
				create Result.make_by_size_and_center (
						cust_ic_design.design_size_x, cust_ic_design.design_size_y,
						cust_ic_design.center_x, cust_ic_design.center_y)
			end
		end

	stretch_die (die_row: CUT_DIE_SITES_ROW b: RECTANGLE) is
		local
			bx: RECTANGLE
		do
			create bx.make_by_size_and_center (die_row.die_size_cut_x, die_row.die_size_cut_y,
							 b.center_x, b.center_y)
			b.extend_to_include (bx)
		end

	db: DB_CONNECTION is
		do
			Result := connection
		end

	dump_the_messages is
		do
		-- don't output using io.putstring
		end

end
