
indexing
	
	description: "Packer box as used in ICL"
	author: "David Hollenberg"
	date: "January 26, 2006"

class PACKER_BOX
inherit
	ANY
	STRING_UTILITIES_ACCESS
		export
			{NONE} all
		end
	CONDITION_UTILITIES
		export
			{NONE} all
		end


feature -- Properties (read-only for packer)

	project_type: STRING
			-- Type of project

	is_external: BOOLEAN
			-- Is project external to payload?

	design_name: STRING
			-- Name of design, for customer dies and
			-- test vehicles only

	name: STRING
			-- Project name that uniquely identifies project
			-- within its type

	m_fab_no: INTEGER
			-- Fab number of IC fab instance, for cust projects only

	cust_account_display_name: STRING
			-- Customer account name for display purposes,
			-- for customer projects and test vehicles only

	cust_account_no: INTEGER
			-- Customer account number, for cust projects only

	cust_account_type: STRING
			-- Customer account type, for customer projects only

	cust_account_sponsor: STRING
			-- Customer account sponsor, for customer projects only

	priority: INTEGER
			-- Priority of cust_die for receiving any
			-- extra space on a full wafer run

	min_die_size: CONST_POINT
			-- Minimum die size in microns for project.

	max_die_size: CONST_POINT
			-- Maximum die size in microns for project

	not_rotatable: BOOLEAN
			-- Is packer not allowed to rotate project?

	not_movable: BOOLEAN
			-- Is packer not allowed to move project?

	is_tiny_chip: BOOLEAN
			-- Is this a user project which is a tiny chip?

	requested_number: INTEGER
			-- Number of parts requested over all wafers combined
			-- (if positive) or number per wafer (if negative)
			-- Applies only to projects of type cust_die,
			-- test_vehicle, vendor_geometry or vendor_ghost

	requested_display_name: STRING
			-- Name that client of the packer would like
			-- displayed on screen

	process_name: STRING
			-- Name of wafer process on which project
			-- will be fabricated

	wafer_etest_id: STRING
			-- Wafer etest ID of process on which project
			-- will be fabricated

feature -- Properties (used only inside packer)

	display_name: STRING
			-- Name displayed on screen by packer

	wafer_good_sites: INTEGER
			-- Number of good sites per wafer, for
			-- customer dies and test vehicles only

feature -- Properties (set by packer)

	is_rotated: BOOLEAN
			-- Has die been rotated from original orientation?
			-- Rotation is always clockwise

	die_size: CONST_POINT
			-- Die size in microns for project

	die_id: STRING
			-- Two character identifier

	positions: LIST [PACKER_DIE_LOCATION]
			-- List of the positions at which the project is to be
			-- placed

feature -- Queries

	is_macrocell: BOOLEAN
			-- Is `Current' a macrocell?
		local
			die_type: DIE_TYPE_VALUES
		do
			Result := project_type /= Void and then project_type.is_equal (die_type.Macrocell)
		end

	rotation_in_degrees: INTEGER
			-- Rotation of `Current' in degrees
		do
			if is_rotated then
				Result := 90
			else
				Result := 0
			end
		end

	is_vendor_geometry: BOOLEAN
			-- Is `Current' a vendor geometry?
		local
			die_type: DIE_TYPE_VALUES
		do
			Result := project_type.is_equal (die_type.Vendor_geometry)
		end

	is_vendor_ghost: BOOLEAN
			-- Is `Current' a vendor ghost?
		local
			die_type: DIE_TYPE_VALUES
		do
			Result := project_type.is_equal (die_type.Vendor_ghost)
		end

	is_vendor_geometry_or_ghost: BOOLEAN
			-- Is `Current' a vendor geometry or vendor ghost?
		do
			Result := is_vendor_geometry or is_vendor_ghost
		end

	horizontal_scribe_name: STRING = "HSCRIBE"
			-- Value of "name" field for horizontal scribe

	vertical_scribe_name: STRING = "VSCRIBE"
			-- Value of "name" field for vertical scribe

	is_scribe: BOOLEAN
			-- Is `Current' a scribe?
		do
			Result := project_type.is_equal ("SCRIBE")
		end

	is_horizontal_scribe: BOOLEAN
			-- Is `Current' a horizontal scribe?
		require
			is_scribe: is_scribe
		do
			Result := name /= Void and then su.is_prefix (horizontal_scribe_name, name)
		end

	is_vertical_scribe: BOOLEAN
			-- Is `Current' a vertical scribe?
		require
			is_scribe: is_scribe
		do
			Result := name /= Void and then su.is_prefix (vertical_scribe_name, name)
		end

	scribe_orientation: STRING
			-- Orientation of scribe (horizontal or vertical)
		require
			is_scribe: is_scribe
		local
			orient: ORIENTATION_VALUES
		do
			if is_horizontal_scribe then
				Result := orient.Horizontal
			elseif is_vertical_scribe then
				Result := orient.Vertical
			else
				handle_hard_error ("unknown scribe type " + name)
			end
		end

feature -- Modification

	set_project_type (s: STRING)
			-- Set `project_type' to `s'
		require
			value_exists: s /= Void
		do
			project_type := s
		ensure
			value_set: project_type = s
		end

	set_is_external (b: BOOLEAN)
			-- Set `is_external' to `b'
		do
			is_external := b
		ensure
			value_set: is_external = b
		end

	set_design_name (s: STRING)
			-- Set `design_name' to `s'
		require
			value_exists: s /= Void
		do
			design_name := s
		ensure
			value_set: design_name = s
		end

	set_name (s: STRING)
			-- Set `name' to `s'
		require
			value_exists: s /= Void
		do
			name := s
		ensure
			value_set: name = s
		end

	set_m_fab_no (n: INTEGER)
			-- Set `m_fab_no' to `n'
		do
			m_fab_no := n
		ensure
			value_set: m_fab_no = n
		end

	set_cust_account_display_name (s: STRING)
			-- Set `cust_account_display_name' to `s'
		require
			value_exists: s /= Void
		do
			cust_account_display_name := s
		ensure
			value_set: cust_account_display_name = s
		end

	set_cust_account_no (n: INTEGER)
			-- Set `cust_account_no' to `n'
		do
			cust_account_no := n
		ensure
			value_set: cust_account_no = n
		end

	set_cust_account_type (s: STRING)
			-- Set `cust_account_type' to `s'
		require
			value_exists: s /= Void
		do
			cust_account_type := s
		ensure
			value_set: cust_account_type = s
		end

	set_cust_account_sponsor (s: STRING)
			-- Set `cust_account_sponsor' to `s'
		require
			value_exists: s /= Void
		do
			cust_account_sponsor := s
		ensure
			value_set: cust_account_sponsor = s
		end

	set_priority (n: INTEGER)
			-- Set `priority' to `n'
		do
			priority := n
		ensure
			value_set: priority = n
		end

	set_min_die_size (p: CONST_POINT)
			-- Set `min_die_size' to `p'
		do
			min_die_size := p
		ensure
			value_set: min_die_size = p
		end

	set_max_die_size (p: CONST_POINT)
			-- Set `max_die_size' to `p'
		do
			max_die_size := p
		ensure
			value_set: max_die_size = p
		end

	set_not_rotatable (b: BOOLEAN)
			-- Set `not_rotatable' to `b'
		do
			not_rotatable := b
		ensure
			value_set: not_rotatable = b
		end

	set_not_movable (b: BOOLEAN)
			-- Set `not_movable' to `b'
		do
			not_movable := b
		ensure
			value_set: not_movable = b
		end

	set_is_tiny_chip (b: BOOLEAN)
			-- Set `is_tiny_chip' to `b'
		do
			is_tiny_chip := b
		ensure
			value_set: is_tiny_chip = b
		end

	set_requested_number (n: INTEGER)
			-- Set `requested_number' to `n'
		do
			requested_number := n
		ensure
			value_set: requested_number = n
		end

	set_requested_display_name (s: STRING)
			-- Set `requested_display_name' to `s'
		require
			value_exists: s /= Void
		do
			requested_display_name := s
		ensure
			value_set: requested_display_name = s
		end

	set_process_name (s: STRING)
			-- Set `process_name' to `s'
		require
			value_exists: s /= Void
		do
			process_name := s
		ensure
			value_set: process_name = s
		end

	set_wafer_etest_id (s: STRING)
			-- Set `wafer_etest_id' to `s'
		require
			value_exists: s /= Void
		do
			wafer_etest_id := s
		ensure
			value_set: wafer_etest_id = s
		end

	set_display_name (s: STRING)
			-- Set `display_name' to `s'
		require
			value_exists: s /= Void
		do
			display_name := s
		ensure
			value_set: display_name = s
		end

	set_wafer_good_sites (n: INTEGER)
			-- Set `wafer_good_sites' to `n'
		do
			wafer_good_sites := n
		ensure
			value_set: wafer_good_sites = n
		end

	set_is_rotated (b: BOOLEAN)
			-- Set `is_rotated' to `b'
		do
			is_rotated := b
		ensure
			value_set: is_rotated = b
		end

	set_die_size (p: CONST_POINT)
			-- Set `die_size' to `p'
		do
			die_size := p
		ensure
			value_set: die_size = p
		end

	set_die_id (s: STRING)
			-- Set `die_id' to `s'
		require
			value_exists: s /= Void
		do
			die_id := s
		ensure
			value_set: die_id = s
		end

	set_positions (list: like positions)
			-- Set `positions' to `list'
		require
			value_exists: list /= Void
		do
			positions := list
		ensure
			value_set: positions = list
		end

end

