
indexing
	
	description: "Object that can access current packing version for run"
	author: "David Hollenberg"
	date: "February 27, 2007"

deferred class PACKING_ACCESSOR
inherit
	ANY
	DB_OPTIONS_ACCESS
		export
			{NONE} all
		end
	STRING_UTILITIES_ACCESS
		export
			{NONE} all
		end
	CONDITION_UTILITIES
		export
			{NONE} all
		end

feature -- Properties

	packing: RC_PACKING
			-- Packing to use

	packing_version: PACKING_VERSIONS_MROW
			-- New version of packing

	packing_id: STRING
			-- Packing ID of packing for this run

feature -- Operations

	fetch_packing_selected_version
			-- Fetch selected version of packing for run.
			-- Also, set it to be current version and
			-- detach it from database connection.  Set `packing',
			-- `packing_version' and `packing_id'.  Raise
			-- error if no versions of packing exist
		local
			p: PACKINGS_ROW
			r: RUNS_ROW
		do
			fetch_packing
			if packing = Void then
				handle_hard_error ("could not find packing " + su.double_quoted (cache.runs_row.packing_id))
			end
			p := packing.packings_row
			r := cache.runs_row
			p.check_compatibility_with_run (r)
			if not p.is_compatible_with_run then
				handle_hard_error ("packing " + packing_id + " is not compatible with run " + session.run_id + "%N%N" + p.incompatibilities_with_run)
			end
			packing.refresh_versions
			packing.set_current_version (packing.version_with_number (r.packing_version_no))
			packing_version := packing.current_version
		end

	fetch_packing
			-- Fetch packing to use and put in `packing'.
			-- Set to Void if packing not found
		local
			r: RUNS_ROW
			row: PACKINGS_MROW
		do
			r := cache.runs_row
			if not r.is_null_packing_id then
				packing_id := r.packing_id
				create row.make_by_key (connection, Option.Old_row, packing_id)
				create packing
				packing.set_packings_row (row)
			else
				packing := Void
			end
		end
	
feature {NONE} -- Deferred implementation

	cache: RUN_CACHE
		deferred
		end

	session: RC_RUN_CLOSING_SESSION
		deferred
		end

	connection: DB_CONNECTION
		deferred
		end

end
