
indexing
	
	description: "Object that delegates reporting of packing status to %
		%another object"
	author: "David Hollenberg"
	date: "December 4, 2006"

class PACKING_DELEGATED_STATUS_REPORTER
inherit
	PACKING_STATUS_REPORTER

feature -- Properties

	reporter: PACKING_STATUS_REPORTER
			-- Object to report packer status

feature -- Modification

	set_reporter (a_reporter: like reporter)
			-- Set `reporter' to `a_reporter'
		require
			a_reporter_not_void: a_reporter /= Void
		do
			reporter := a_reporter
		ensure
			reporter_set: reporter = a_reporter
		end

feature -- Operations

	report_start
			-- Report start of an operation
		do
			reporter.report_start
		end

	report_end
			-- Report end of an operation
		do
			reporter.report_end
		end

	report_status (a_message: STRING)
			-- Report transient status described by
			-- `a_message'.  Includes things like "started
			-- operation", "operation in progress",
			-- "waiting for next key", etc
		do
			reporter.report_status (a_message)
		end

	report_progress (a_message: STRING)
			-- Report operation in progress described by
			-- `a_message'
		do
			reporter.report_progress (a_message)
		end

	report_select (a_message: STRING)
			-- Report select operation described by `a_message'
		do
			reporter.report_select (a_message)
		end

	report_view (a_message: STRING)
			-- Report change of how objects are displayed
			-- described by `a_message'
		do
			reporter.report_view (a_message)
		end

	report_lock (a_message: STRING)
			-- Report locking of object property or properties
			-- described by `a_message'
		do
			reporter.report_lock (a_message)
		end

	report_navigation (a_message: STRING)
			-- Report operation that affects what part of
			-- model is displayed, described by `a_message'
		do
			reporter.report_navigation (a_message)
		end

	report_change (a_message: STRING)
			-- Report operation that changes the packing,
			-- described by `a_message'.  Includes things
			-- like move, duplicate, add or remove
			-- projects, etc
		do
			reporter.report_change (a_message)
		end

	report_failure (a_message: STRING)
			-- Report operation failure described by
			-- `a_message'.  This is for the case where operation
			-- failed completely, not where some part of the
			-- operation could be done
		do
			reporter.report_failure (a_message)
		end

	report_cancellation (a_message: STRING)
			-- Report operation cancellation described by
			-- `a_message'
		do
			reporter.report_cancellation (a_message)
		end

	report_validity (a_message: STRING)
			-- Report validity check operation described by
			-- `a_message'
		do
			reporter.report_validity (a_message)
		end

	report_exit (a_message: STRING)
			-- Report exit of packer described by
			-- `a_message'
		do
			reporter.report_exit (a_message)
		end

	report_details (a_message: STRING)
			-- Report details of operation that succeeded or
			-- failed, described by `a_message'
		do
			reporter.report_details (a_message)
		end

	report_internal_error (a_message: STRING)
			-- Report internal error `a_message'
		do
			reporter.report_internal_error (a_message)
		end

end
