
indexing

	author: "David Hollenberg"

class PACKING_DIE_LOCATIONS_ROW
inherit
	PACKING_DIE_LOCATIONS_RAW
	PACKING_QUERIES
	DB_HASHABLE_ROW
		undefine
			initialize_new, initialize_old, fill_in
		end
	HAS_FIELD_POSITION
		rename
			center_x as location_x,
			center_y as location_y
		end
	ROW_COUNT_CHECK_ACCESS
		export
			{NONE} all
		end
	NUMBER_UTILITIES_ACCESS
		export
			{NONE} all
		end
create
	make_template, make_null, make_by_key, make_detached

feature -- Queries

	rows_for_die (a_packing_id: STRING a_version, a_die_no: INTEGER conn: DB_CONNECTION range: ITEM_COUNT_RANGE): LIST [like Current] is
			-- Die locations for `a_packing_id', `a_version' and
			-- `a_die_no', ordered by `site_no'
		require
			a_packing_id_not_void: a_packing_id /= Void
			connection_exists: conn /= Void
			range_exists: range /= Void
		local
			b: DB_BIND_SII
		do
			create b.make (a_packing_id, a_version, a_die_no)
			Result := all_rows_filtered_ordered (conn, "packing_id = :1 and version_no = :2 and die_no = :3", b, "site_no")
			check_row_count (Result, range)
		ensure
			result_exists: Result /= Void
		end

feature -- Properties

	hash_code: INTEGER is
		do
			Result := concatenated_key_columns.hash_code
		end
	
	concatenated_key_columns: STRING is
			-- Key columns concatenated together and
			-- separated by null characters
		do
			create Result.make (packing_id.count + 15)
			Result.append (packing_id)
			Result.extend ('%U')
			Result.append_integer (version_no)
			Result.extend ('%U')
			Result.append_integer (die_no)
			Result.extend ('%U')
			Result.append_integer (site_no)
		end
	
	identification: STRING
			-- Identification of `Current' for error message
		do
			create Result.make (50)
			Result.append ("packing ID ")
			Result.append (packing_id)
			Result.append (", version_no ")
			Result.append_integer (version_no)
			Result.append (", die_no ")
			Result.append_integer (die_no)
			Result.append (", site_no ")
			Result.append_integer (site_no)
		end
	
	location: CONST_POINT is
		do
			create Result.make_xy (location_x, location_y)
		end

	has_field_name: BOOLEAN is True
			-- Does `Current' have an associated field name?

	has_center_x: BOOLEAN is True
			-- Does `Current' have an associated center in X 
			-- dimension?

	has_center_y: BOOLEAN is True
			-- Does `Current' have an associated center in X 
			-- dimension?

	center_x_string: STRING is
			-- Center of associated model element in X dimension
			-- as a string
		do
			Result := nu.formatted_number_or_integer (location_x, scale_location_x)
		end

	center_y_string: STRING is
			-- Center of associated model element in Y dimension
			-- as a string
		do
			Result := nu.formatted_number_or_integer (location_y, scale_location_y)
		end

	max_db_value: DOUBLE is
			-- Maximum value that can fit in the corresponding
			-- database table
		require
			same_x_and_y_precision: precision_location_x = precision_location_y
			same_x_and_y_scale: scale_location_x = scale_location_y
		once
			Result := 10 ^ (precision_location_x - scale_location_x) - 10 ^ -scale_location_x
		end

end
