
indexing
	
	description: "Multi-column list row that has additional information %
		%about the entity it represents that uniquely identifies %
		%the entity and associates it with a packing die"
	author: "David Hollenberg"
	date: "December 6, 2005"

class PACKING_DIE_MULTI_COLUMN_LIST_ROW
inherit	
	MULTI_COLUMN_LIST_ROW
		rename
			make as mcl_make
		end
	HAS_DIE_TYPE
		undefine
			copy, is_equal, default_create
		end
	HAS_DESIGN_NUMBER
		rename
			has_design_number as has_design_no,
			design_number as design_no
		undefine
			copy, is_equal, default_create
		end
	HAS_DATE_FAB_REQUESTED
		undefine
			copy, is_equal, default_create
		end
	HAS_DIE_ID
		undefine
			copy, is_equal, default_create
		end
	HAS_DESIGN_NAME
		undefine
			copy, is_equal, default_create
		end
	HAS_FIELD_NAME
		undefine
			copy, is_equal, default_create
		end
	HAS_IS_ASSIGNED
		undefine
			copy, is_equal, default_create
		end
	HAS_WAFER_ETEST_ID
		undefine
			copy, is_equal, default_create
		end
	HAS_ALT_ETEST_IDS
		undefine
			copy, is_equal, default_create
		end
	HAS_QUANTITY_ORDERED
		undefine
			copy, is_equal, default_create
		end
	HAS_SIZE_X
		undefine
			copy, is_equal, default_create
		end
	HAS_SIZE_Y
		undefine
			copy, is_equal, default_create
		end
	HAS_CENTER_X
		undefine
			copy, is_equal, default_create
		end
	HAS_CENTER_Y
		undefine
			copy, is_equal, default_create
		end
	HAS_MAX_SIZE_X
		undefine
			copy, is_equal, default_create
		end
	HAS_MAX_SIZE_Y
		undefine
			copy, is_equal, default_create
		end
	HAS_ACCOUNT_NUMBER
		rename
			has_account_number as has_cust_acct_no,
			account_number as cust_acct_no
		undefine
			copy, is_equal, default_create
		end
	HAS_ACCOUNT_TYPE
		undefine
			copy, is_equal, default_create
		end
	HAS_OVERHEAD_STYLE
		undefine
			copy, is_equal, default_create
		end
	HAS_ORG_SHORT_ID
		rename
			has_org_short_id as has_organization_short_id,
			org_short_id as organization_short_id
		undefine
			copy, is_equal, default_create
		end
	MOSIS_UTILITIES
		export
			{NONE} all
		undefine
			copy, is_equal, default_create
		end
	NUMBER_UTILITIES_ACCESS
		export
			{NONE} all
		undefine
			copy, is_equal, default_create
		end
	GRIDABLE
		undefine
			copy, is_equal, default_create
		end

create
	make

create {PACKING_DIE_MULTI_COLUMN_LIST_ROW}
	make_filled

feature {NONE} -- Creation

	make (a_packer_die: like packer_die a_packing: like packing)
		require
			a_packer_die_not_void: a_packer_die /= Void
			a_packing_not_void: a_packing /= Void
		do
			default_create
			packer_die := a_packer_die
			packing := a_packing
			enable_visible
		ensure
			is_visible: is_visible
		end

feature -- Properties

	packer_die: RETICLE_DIE
			-- Die and location

	packing: RC_PACKING
			-- Packing

	die: PACKING_DIES_ROW
			-- Packing die for row
		do
			Result := packer_die.die
		end

	die_location: PACKING_DIE_LOCATIONS_ROW
			-- Packing die location for row
		do
			Result := packer_die.die_location
		end

	project: PACKING_PROJECTS_ROW
			-- Packing project this row is for
		require
			is_project: is_project
		do
			Result := die.project
		end

	is_project: BOOLEAN
			-- Is die a project (cust die or test vehicle)?
		do
			Result := die.has_project
		end

	die_type: STRING
			-- Type of die
		do
			Result := die.die_type
		end

	date_fab_requested: DB_DATE
			-- Date fab was requested
		do
			Result := project.date_fab_requested
		end

	design_no: INTEGER
			-- Design number
		do
			Result := project.design_no
		end

	design_name: STRING
			-- Name of design or other identifying name
		do
			Result := packer_die.info_name
		end

	die_id: STRING
			-- Die ID
		do
			Result := packer_die.die_id
		end

	field_name: STRING
			-- Name of associated field of view
		do
			Result := packer_die.field.name
		end

	is_assigned: BOOLEAN
			-- Is there an assigned wafer etest ID?
		do
			Result := not wafer_etest_id.is_empty
		end

	wafer_etest_id: STRING
			-- Assigned wafer etest ID
		do
			Result := packer_die.assigned_wafer_etest_id
		end

	alt_etest_ids: STRING
			-- Alternate etest IDs, other than the one assigned
		do
			Result := su.joined_by (packer_die.unassigned_wafer_etest_ids, ", ")
		end

	quantity_ordered: INTEGER
			-- Quantity of IC parts ordered
		do
			Result := project.quantity_ordered
		end

	size_x: DOUBLE
			-- Size in X dimension
		do
			Result := packer_die.unrotated_size_x
		end

	size_y: DOUBLE
			-- Size in Y dimension
		do
			Result := packer_die.unrotated_size_y
		end

	size_x_string: STRING
			-- Size in X dimension as a string
		do
			Result := grid.formatted_number (size_x)
		end

	size_y_string: STRING
			-- Size in Y dimension as a string
		do
			Result := grid.formatted_number (size_y)
		end

	center_x: DOUBLE
			-- Center in X dimension
		do
			Result := packer_die.field_relative_center_x
		end

	center_y: DOUBLE
			-- Center in Y dimension
		do
			Result := packer_die.field_relative_center_y
		end

	center_x_string: STRING
			-- Center in X dimension as a string
		do
			Result := grid.formatted_number (center_x)
		end

	center_y_string: STRING
			-- Center in Y dimension as a string
		do
			Result := grid.formatted_number (center_y)
		end

	max_size_x: DOUBLE
			-- Maximum size in X dimension
		do
			Result := project.max_die_size.x
		end

	max_size_y: DOUBLE
			-- Maximum size in Y dimension
		do
			Result := project.max_die_size.y
		end

	max_size_x_string: STRING
			-- Maximum size in X dimension as a string
		do
			if has_max_size_x then
				Result := grid.formatted_number (project.max_die_size.x)
			else
				Result := ""
			end
		end

	max_size_y_string: STRING
			-- Maximum size in Y dimension as a string
		do
			if has_max_size_y then
				Result := grid.formatted_number (project.max_die_size.y)
			else
				Result := ""
			end
		end

	cust_acct_no: INTEGER
			-- Account number
		do
			Result := project.cust_acct_no
		end

	account_type: STRING
			-- Account type
		do
			if account.is_null_acct_sponsor_id then
				Result := account.acct_type
			else
				Result := account.acct_sponsor_id + "-" + account.acct_type
			end
		end

	organization_short_id: STRING
			-- Organization short ID
		do
			Result := account.organization.short_id
		end

	overhead_style: STRING
			-- Overhead style for die labels
		do
			if is_project then
				Result := packing.packings_row.effective_overhead_style_description (project.overhead_style)
			else
				Result := ""
			end
		end

	unique_identifier: STRING
			-- Value that uniquely identifies row
		do
			Result := die.die_no.out
		end

	has_die_type: BOOLEAN = True
			-- Is there an associated die type?

	has_design_name: BOOLEAN = True
			-- Is there an associated design name?

	has_is_assigned: BOOLEAN
			-- Is there an associated "wafer etest ID assigned"
			-- indication?
		do
			Result := has_wafer_etest_id
		end

	has_wafer_etest_id: BOOLEAN
			-- Is there an associated wafer etest ID?
		do
			Result := packer_die.has_wafer_etest_id
		end

	has_alt_etest_ids: BOOLEAN
			-- Is there an associated alternate wafer etest ID list?
		do
			Result := packer_die.has_wafer_etest_id
		end

	has_cust_acct_no: BOOLEAN
			-- Is there an associated account number?
		do
			Result := is_project
		end

	has_account_type: BOOLEAN
			-- Is there an associated account type?
		do
			Result := has_cust_acct_no
		end

	has_organization_short_id: BOOLEAN
			-- Is there an associated organization short ID?
		do
			Result := has_cust_acct_no
		end

	has_design_no: BOOLEAN
			-- Is there an associated design number?
		do
			Result := is_project and then not project.is_null_design_no
		end

	has_quantity_ordered: BOOLEAN
			-- Is there an associated quantity ordered?
		do
			Result := is_project and then not project.is_null_quantity_ordered
		end

	has_size_x: BOOLEAN = True
			-- Is there an associated size in X?

	has_size_y: BOOLEAN = True
			-- Is there an associated size in Y?

	has_center_x: BOOLEAN = True
			-- Is there an associated center in X?

	has_center_y: BOOLEAN = True
			-- Is there an associated center in Y?

	has_field_name: BOOLEAN = True
			-- Is there an associated field name?

	has_max_size_x: BOOLEAN
			-- Is there an associated maximum size in X?
		do
			Result := is_project and then project.max_die_size /= Void
		end

	has_max_size_y: BOOLEAN
			-- Is there an associated maximum size in Y?
		do
			Result := is_project and then project.max_die_size /= Void
		end

	has_date_fab_requested: BOOLEAN
			-- Is there an associated date fab requested?
		do
			Result := is_project and then not project.is_null_date_fab_requested
		end

feature {NONE} -- Implementation

	account: like cached_account
			-- Cust account row for `Current'
		require
			has_cust_acct_no: has_cust_acct_no
		do
			if cached_account = Void then
				fetch_account
			end
			Result := cached_account
		end

	fetch_account
		do
			create cached_account.make_by_key (project.any_connection, option.Old_row, cust_acct_no)
		end

	cached_account: CUST_ACCTS_ROW
			-- Cached customer account row

	new_filled_list (n: INTEGER): like Current
			-- New list with `n' elements.
		do
			create Result.make_filled (n)
		end
		
end
