
indexing

	author: "David Hollenberg"

class PACKING_DIES_MROW
inherit
	PACKING_DIES_MRAW
		redefine
			detached_twin, copy_from_row
		end
	PACKING_DIES_ROW
		undefine
			initialize_new,
			initialize_new_by_key
		redefine
			anchored_die_location, anchored_project,
			die_locations, project, set_die_locations, set_project
		end
	DB_DETACHABLE_ROW
		export
			{PACKING_DIES_MROW} detach_actions_internal
		redefine
			detached_twin, copy_from_row
		end

create
	make_template, make_null, make_by_key, make_detached

feature -- Properties

	die_locations: LIST [like anchored_die_location] is
			-- Die locations for `Current'
		do
			if is_detached then
				Result := detached_die_locations
			else
				if cached_die_locations = Void then
					fetch_die_locations
				end
				Result := cached_die_locations
			end
		end

	uniform_white_space_border: DOUBLE
			-- Minimum amount of white space on all sides
			-- of project
		require
			is_project: is_cust_die or is_test_vehicle
		do
			Result := packing.uniform_white_space_border (project.overhead_style)
		end

	project: like anchored_project is
			-- Project for `Current'
		do
			if is_detached then
				Result := detached_project
			else
				if cached_project = Void then
					fetch_project
				end
				Result := cached_project
			end
		end

	detached_twin: like Current is
			-- New object that is the same as `Current' but
			-- is detached from any database connection
		do
			Result := precursor {PACKING_DIES_MRAW}
			call_detach_actions (Result)
		end

	deep_detached_twin: like Current is
			-- Detached copy of `Current' with all
			-- dependent rows detached as well
		do
			Result := detached_twin
			Result.set_die_locations (die_locations.twin)
			if has_project then
				Result.set_project (project)
			end
			Result.uncache_dependents
				-- Just in case: shouldn't be referenced
			Result.detach_dependents_deep
		end

	is_deep_unchanged: BOOLEAN is
			-- Are `Current' and all its dependent rows
			-- unchanged from their original versions?
			-- Returns False if no original version
		local
			dl_pair: DB_ROW_SET_PAIR [like anchored_die_location]
		do
			Result := all_columns_equal_original
				and then die_locations.for_all (agent {like anchored_die_location}.is_deep_unchanged)
			if Result and has_project then
				Result := project.is_deep_unchanged
			end
			if Result then
				create dl_pair.make_from_lists (die_locations, original_die_locations, create {KEY_COLS_EQUALITY_TESTER [like anchored_die_location]})
				Result := dl_pair.is_identical
			end
		end

	last_die_location: like anchored_die_location
			-- Last die location added by `add_die_location'

feature -- Row copy

	copy_from_row (r: like original_row) is
			-- Initialize `Current' from `r'
		do
			precursor {PACKING_DIES_MRAW} (r)
			detach_actions_internal := r.detach_actions_internal
			cached_die_locations := r.die_locations
			cached_project := r.cached_project
			cached_vendor_geometry := r.cached_vendor_geometry
			wafer_process := r.wafer_process
			max_site_no := r.max_site_no
			packing := r.packing
		end

feature -- Modification

	set_attributes_from_project (proj: like anchored_project a_packing: PACKINGS_ROW) is
			-- Copy attributes of `proj', which is part of 
			-- `a_packing', into `Current'
		require
			proj_not_void: proj /= Void
		local
			wt: WAFER_TECHNS_ROW
			size: CONST_POINT
		do
			set_project (proj)
			set_project_no (proj.project_no)
			set_project_version_no (proj.version_no)
			wt := a_packing.wafer_techns_row
			size := wt.die_size_with_overhead (a_packing.effective_overhead_style (proj.overhead_style), a_packing.other_border, proj.design_size)
			set_die_size_printed_x (size.x)
			set_die_size_printed_y (size.y)
			packing := a_packing
		end

	set_die_locations (list: like die_locations) is
			-- Set `die_locations' to `list'
		do
			if is_detached then
				detached_die_locations := list
			else
				precursor (list)
			end
		end

	add_die_location (fld_name: STRING loc_x, loc_y: DOUBLE) is
			-- Add new die location in field `fld_name', whose
			-- center is at (`loc_x', `loc_y') relative to
			-- center of field.  Leave new location in
			-- `last_die_location'
		require
			is_detached: is_detached
		local
			dl: like anchored_die_location
		do
			create dl.make_detached (any_connection)
			dl.set_packing_id (packing_id)
			dl.set_version_no (version_no)
			dl.set_die_no (die_no)
			dl.set_site_no (highest_used_site_number + 1)
			dl.set_field_name (fld_name)
			dl.set_location_x (loc_x)
			dl.set_location_y (loc_y)
			dl.set_default_packer_attributes
			detached_die_locations.extend (dl)
			last_die_location := dl
		end

	add_existing_die_location (a_die_location: like anchored_die_location) is
			-- Add existing die location `dl' (for undo/redo)
		require
			a_die_location_not_void: a_die_location /= Void
		do
			detached_die_locations.extend (a_die_location)
		end

	remove_die_location (loc: PACKING_DIE_LOCATIONS_MROW) is
			-- Remove die location `loc'
		require
			is_detached: is_detached
		do
			detached_die_locations.prune_all (loc)
		end

	set_project (p: like project) is
			-- Set `project' to `p'
		do
			if is_detached then
				detached_project := p
			else
				precursor (p)
			end
		end

	assign_wafer_process is
		require
			is_cust_die: is_cust_die
		do
			set_wafer_process (project.assigned_process)
		end

	assign_wafer_etest_name is
			-- Assign wafer etest name to `Current' if
			-- there is only one possibility
		require
			is_cust_die: is_cust_die
		do
			if project.compatible_wafer_etest_names.count = 1 then
				set_wafer_etest_name (project.compatible_wafer_etest_names.first.wafer_etest_name)
			end
		end

	propagate_assigned_etest_name (table: HASH_TABLE [WAFER_PROCESSES_ROW, STRING]) is
			-- Propagate assigned wafer etest name to the
			-- project associated with `Current', using `table'
			-- to look up wafer processes
		require
			is_cust_die: is_cust_die
		do
			project.set_assigned_wafer_etest_name (assigned_wafer_etest_name, table)
		end

feature -- Operations

	set_rotation (n: INTEGER) is
			-- Set rotation to `n' degrees clockwise
		local
			old_rot: INTEGER
		do
			old_rot := die_wafer_rotation
			set_die_wafer_rotation (n)
			if ((n - old_rot).abs \\ 180) = 90 then
				swap_die_size_x_and_y
			end
		end

	toggle_rotation is
			-- Toggle rotation between 0 and 90 degrees clockwise
		local
			switch: BOOLEAN
		do
			if die_wafer_rotation = 0 then
				switch := True
				set_die_wafer_rotation (90)
			else
				switch := is_rotated_sideways
				set_die_wafer_rotation (0)
			end
			if switch then
				swap_die_size_x_and_y
			end
		end

	rotate_90_clockwise is
			-- Rotate die 90 degrees clockwise
		local
			rot: INTEGER
		do
			rot := (die_wafer_rotation + 90) \\ 360
			set_die_wafer_rotation (rot)
			swap_die_size_x_and_y
		end

	rotate_90_counterclockwise is
			-- Rotate die 90 degrees counterclockwise
		local
			rot: INTEGER
		do
			rot := (die_wafer_rotation + 270) \\ 360
			set_die_wafer_rotation (rot)
			swap_die_size_x_and_y
		end

	update_for_new_version (n: INTEGER) is
			-- Update `Current' with information for new
			-- version `n'
		do
			set_version_no (n)
			update_dependents_for_version (n)
			if has_project then
				set_project_version_no (project.version_no)
			end
		end

	register_deep is
			-- Register `Current' and all dependent rows
			-- on implicit connection
		do
			register_project_deep
			register_on_implicit_connection
			register_dependents_deep
		end

	update_common_from_packer_box (pb: PACKER_BOX) is
			-- Update information in `Current' that is common
			-- to all die types
		require
			pb_not_void: pb /= Void
		local
			old_x: DOUBLE
		do
			set_die_wafer_rotation (pb.rotation_in_degrees)
			if die_wafer_rotation = 90 then
				-- Switch size X and size Y
				old_x := die_size_printed_x
				set_die_size_printed_x (die_size_printed_y)
				set_die_size_printed_y (old_x)
			end
			set_die_id (pb.die_id)
			set_locations_from_packer_box (pb)
		end
	
	set_size_from_packer_box (pb: PACKER_BOX) is
			-- Update size in `Current' from `pb'
		require
			pb_not_void: pb /= Void
		do
			set_die_size_printed_x (pb.die_size.x)
			set_die_size_printed_y (pb.die_size.y)
		end
	
feature {PACKING_DIES_MROW} -- Implementation

	detach_dependents_deep is
			-- Detach all dependent rows
		do
			detach_die_locations_deep
			if has_project then
				set_project (project.deep_detached_twin)
			end
		end

feature {NONE} -- Implementation

	set_locations_from_packer_box (pb: PACKER_BOX) is
			-- Set die locations for `Current' from `pb'
		local
			pb_locs: LIST [PACKER_DIE_LOCATION]
			pdl_list: ARRAYED_LIST [like anchored_die_location]
			pdl: like anchored_die_location
			n: INTEGER
		do
			pb_locs := pb.positions
			if pb_locs = Void then
				create pdl_list.make (0)
			else
				create pdl_list.make (pb_locs.count)
				from
					pb_locs.start
				until
					pb_locs.after
				loop
					n := n + 1
					create pdl.make_detached (any_connection)
					pdl.set_packing_id (packing_id)
					pdl.set_version_no (version_no)
					pdl.set_die_no (die_no)
					pdl.set_site_no (n)
					pdl.set_field_name (pb_locs.item.field_name)
					pdl.set_location_x (pb_locs.item.location.x)
					pdl.set_location_y (pb_locs.item.location.y)
					if pb.is_vendor_geometry_or_ghost then
						if pb.not_movable then
							pdl.set_fixed_vendor_geometry_packer_attributes
						else
							pdl.set_movable_vendor_geometry_packer_attributes (not pb.not_rotatable)
						end
					elseif pb.is_scribe then
						pdl.set_default_packer_attributes
						pdl.set_is_rotatable (False)
					else
						pdl.set_default_packer_attributes
					end
					pdl_list.extend (pdl)
					pb_locs.forth
				end
			end
			set_die_locations (pdl_list)
		end
	
	detach_die_locations_deep is
			-- Detach die locations deeply
		do
			original_die_locations := die_locations.twin
			original_die_locations.wipe_out
			original_die_locations.append (die_locations)
			die_locations.wipe_out
			original_die_locations.do_all (agent append_detached_die_location (?, die_locations))
		end

	append_detached_die_location (dl: like anchored_die_location list: like die_locations) is
			-- Append detached copy of `dl' to `list'
		do
			list.extend (dl.deep_detached_twin)
		end
		
	update_dependents_for_version (n: INTEGER) is
			-- Update all dependent rows prior to save of
			-- new version `n'
		do
			max_site_no := 0
			die_locations.do_all (agent renumber_die_location)
			die_locations.do_all (agent {like anchored_die_location}.update_for_new_version (n))
		end

	highest_used_site_number: INTEGER is
			-- Highest used site number among all die locations
		local
			locs: like die_locations
		do
			locs := die_locations
			from
				locs.start
			until
				locs.after
			loop
				Result := Result.max (locs.item.site_no)
				locs.forth
			end
		end
	
	renumber_die_location (dl: PACKING_DIE_LOCATIONS_MROW) is
		do
			max_site_no := max_site_no + 1
			dl.set_die_no (die_no)
			dl.set_site_no (max_site_no)
		end
	
	register_project_deep is
			-- Register project row (if any) on implicit
			-- connection.  Must be done before `Current'
			-- is registered to avoid foreign key constraint
			-- violation
		do
			-- Move "detached" versions back to "cached" versions
			-- since they will no longer be detached at end
			-- of routine
			
			if has_project then
				if not project.is_deep_unchanged then
					project.register_deep
				end
				cached_project := detached_project
				detached_project := Void
			end
		end

	register_dependents_deep is
			-- Register all dependent rows on implicit
			-- connection
		do
			-- Move "detached" versions back to "cached" versions
			-- since they will no longer be detached at end
			-- of routine
			
			cached_die_locations := detached_die_locations
			detached_die_locations := Void
			die_locations.do_all (agent {like anchored_die_location}.register_deep)
		end

	swap_die_size_x_and_y
			-- Swap `die_size_x' and `die_size_y'
		local
			temp: like die_size_printed_x
		do
			temp := die_size_printed_x
			set_die_size_printed_x (die_size_printed_y)
			set_die_size_printed_y (temp)
		end

	anchored_die_location: PACKING_DIE_LOCATIONS_MROW is
			-- For anchoring only
		do
		end

	anchored_project: PACKING_PROJECTS_MROW is
			-- For anchoring only
		do
		end

	detached_die_locations: like die_locations
			-- List of detached die locations (no explict 
			-- DB connection)

	detached_project: like project
			-- Detached project (no explict DB connection)

	original_die_locations: like die_locations
			-- Original list of die locations before detach

feature {PACKING_DIES_MROW} -- Implementation

	max_site_no: INTEGER
			-- Last site number assigned during renumbering

end
