
indexing

	author: "David Hollenberg"

class PACKING_DIES_ROW
inherit
	PACKING_DIES_RAW
	PACKING_QUERIES
	ABSTRACT_DIE
	HAS_DESIGN_NUMBER
	MASK_SITE_SORTABLE
	DB_HASHABLE_ROW
		undefine
			initialize_new, initialize_old, fill_in
		end
	ROW_COUNT_CHECK_ACCESS
		export
			{NONE} all
		end
	ITEM_COUNT_RANGES_ACCESS
		export
			{NONE} all
		end
	DB_OPTIONS_ACCESS
		export
			{NONE} all
		end
	CONDITION_UTILITIES
		export
			{NONE} all
		end

create
	make_template, make_null, make_by_key, make_detached

feature -- Queries

	rows_for_packing_version (a_packing_id: STRING a_version: INTEGER conn: DB_CONNECTION range: ITEM_COUNT_RANGE): LIST [like Current] is
			-- Dies for `a_packing_id' and `a_version'
		require
			a_packing_id_not_void: a_packing_id /= Void
			connection_exists: conn /= Void
			range_exists: range /= Void
		local
			b: DB_BIND_SI
		do
			create b.make (a_packing_id, a_version)
			Result := all_rows_filtered (conn, "packing_id = :1 and version_no = :2", b)
			check_row_count (Result, range)
		ensure
			result_exists: Result /= Void
		end

feature -- Properties

	hash_code: INTEGER is
		local
			s: STRING
		do
			create s.make (packing_id.count + 10)
			s.append (packing_id)
			s.extend ('%U')
			s.append_integer (version_no)
			s.extend ('%U')
			s.append_integer (die_no)
			Result := s.hash_code
		end
	
	packing: PACKINGS_ROW
			-- Packing of which this die is a part

	die_locations: LIST [like anchored_die_location] is
			-- Die locations for `Current'
		do
			if cached_die_locations = Void then
				fetch_die_locations
			end
			Result := cached_die_locations
		end

	is_cached_die_locations: BOOLEAN is
			-- Are die locations currently cached?
		do
			Result := cached_die_locations /= Void
		end

	has_project: BOOLEAN is
			-- Does `Current' correspond to a project
			-- (customer die or test vehicle)?
		local
			t: DIE_TYPE_VALUES
		do
			Result := die_type.is_equal (t.Cust_die) or die_type.is_equal (t.Test_vehicle)
		end

	is_non_project: BOOLEAN is
			-- Is `Current' not a project
			-- (customer die or test vehicle)?
		do
			Result := not has_project
		end

	has_design_number: BOOLEAN is
			-- Does `Current' have an associated design number?
		do
			Result := has_project and then not project.is_null_design_no
		end

	design_number: INTEGER is
			-- Associated design number
		do
			Result := project.design_no
		end

	project: like anchored_project is
			-- Project for `Current'
		require
			has_project: has_project
		do
			if cached_project = Void then
				fetch_project
			end
			Result := cached_project
		end

	is_cached_project: BOOLEAN is
			-- Is project currently cached?
		require
			has_project: has_project
		do
			Result := cached_project /= Void
		end

	vendor_geometry: like anchored_vendor_geometry is
			-- Vendor geometry for `Current'
		require
			is_vendor_geometry_or_ghost: is_vendor_geometry_or_ghost
		do
			if cached_vendor_geometry = Void then
				fetch_vendor_geometry
			end
			Result := cached_vendor_geometry
		end

	is_cached_vendor_geometry: BOOLEAN is
			-- Is vendor geometry currently cached?
		require
			is_vendor_geometry_or_ghost: is_vendor_geometry_or_ghost
		do
			Result := cached_vendor_geometry /= Void
		end

	is_external_to_payload: BOOLEAN is
			-- Is `Current' external to the payload?
		do
			Result := is_vendor_geometry_or_ghost and then vendor_geometry.external_to_payload
		end

	design_no: INTEGER is
			-- Design number associated with `Current'
		require else
			has_project: has_project
		do
			Result := project.design_no
		end

	design_name: STRING is
			-- Design name associated with `Current'
		require
			has_project: has_project
		do
			Result := project.design_name
		end

	is_deletable: BOOLEAN is
			-- Are all locations marked as being deletable?
		local
			list: LIST [like anchored_die_location]
		do
			Result := True
			list := die_locations
			from
				list.start
			until
				list.after or not Result
			loop
				Result := Result and list.item.is_deletable
				list.forth
			end
		end

	compatible_wafer_etest_names: LIST [WAFER_ETEST_NAMES_ROW] is
			-- Distinct wafer etest names rows for 
			-- `compatible_processes'
		require
			is_cust_die: is_cust_die
		do
			result := project.compatible_wafer_etest_names
		end

	assigned_wafer_etest_name: WAFER_ETEST_NAMES_ROW is
			-- Wafer etest names row for wafer etest name
			-- that has been assigned to cust die or Void 
			-- if none assigned yet
		require
			is_cust_die: is_cust_die
		local
			list: LIST [WAFER_ETEST_NAMES_ROW]
			row: WAFER_ETEST_NAMES_ROW
		do
			if not is_null_wafer_etest_name then
				from
					list := project.compatible_wafer_etest_names
					list.start
				until
					list.after or Result /= Void
				loop
					row := list.item
					if row.wafer_etest_name.is_equal (wafer_etest_name) then
						Result := row
					end
					list.forth
				end
				if Result = Void then
					handle_hard_error ("cannot find row for wafer_etest_name " + wafer_etest_name + " in compatible_wafer_etest_names list")
				end
			end
		end

	assigned_wafer_etest_id: STRING is
			-- Wafer etest ID assigned to cust die or empty
			-- string if none
		require
			is_cust_die: is_cust_die
		local
			row: like assigned_wafer_etest_name
		do
			row := assigned_wafer_etest_name
			if row /= Void then
				Result := row.wafer_etest_id
			else
				Result := ""
			end
		end

	compatible_wafer_etest_ids: LIST [STRING] is
			-- Distinct wafer etest IDs that are compatible
			-- with customer die
		require
			is_cust_die: is_cust_die
		do
			Result := project.compatible_wafer_etest_ids
		end

	unassigned_wafer_etest_ids: LIST [STRING] is
			-- Distinct wafer etest IDs that are compatible
			-- with customer die but are not assigned to it
		require
			is_cust_die: is_cust_die
		local
			list: like compatible_wafer_etest_ids
			l_assigned_id, id: STRING
		do
			l_assigned_id := assigned_wafer_etest_id
			list := compatible_wafer_etest_ids
			if not l_assigned_id.is_empty then
				create {ARRAYED_LIST [STRING]} Result.make (list.count)
				from
					list.start
				until
					list.after
				loop
					id := list.item
					if not id.is_equal (l_assigned_id) then
						Result.extend (id)
					end
					list.forth
				end
			else
				Result := list
			end
		end

	wafer_process: STRING
			-- Actual wafer process assigned after packing is
			-- completed, for customer dies

feature -- Modification

	set_packing (a_packing: like packing)
			-- Set `packing' to `a_packing'
		do
			packing := a_packing
		end

	uncache_die_locations is
			-- Clear die locations from cache
		do
			cached_die_locations := Void
		ensure
			die_locations_not_cached: not is_cached_die_locations
		end

	set_die_locations (list: like die_locations) is
			-- Set `die_locations' to `list'
		require
			list_not_void: list /= Void
		do
			cached_die_locations := list
		ensure
			die_locations_set: die_locations = list
		end

	uncache_project is
			-- Clear project cached copy
		require
			has_project: has_project
		do
			cached_project := Void
		ensure
			project_not_cached: not is_cached_project
		end

	set_project (p: like project) is
			-- Set `project' to `p'
		require
			p_not_void: p /= Void
			has_project: has_project
		do
			cached_project := p
		ensure
			project_set: project = p
		end

	uncache_vendor_geometry is
			-- Clear vendor geometry cached copy
		require
			is_vendor_geometry_or_ghost: is_vendor_geometry_or_ghost
		do
			cached_vendor_geometry := Void
		ensure
			vendor_geometry_not_cached: not is_cached_vendor_geometry
		end

	set_vendor_geometry (g: like vendor_geometry) is
			-- Set `vendor_geometry' to `g'
		do
			cached_vendor_geometry := g
		ensure
			vendor_geometry_set: vendor_geometry = g
		end

	uncache_dependents is
			-- Clear cached dependent rows
		do
			uncache_die_locations
			if has_project then
				uncache_project
			end
		end

	set_wafer_process (a_process: like wafer_process) is
			-- Set `wafer_process' to `a_process'
		do
			wafer_process := a_process
			if a_process = Void then
				wafer_process := ""
			end
		end

feature -- Operations

	extend_with_project (list: LIST [like anchored_project]) is
			-- Extend `list' with project associated with
			-- `Current', if it has one
		do
			if has_project then
				list.extend (project)
			end
		end

feature {NONE} -- Implementation

	fetch_die_locations is
			-- Fetch die_locations for `Current'
		local
			row: like anchored_die_location
		do
			create row.make_template
			cached_die_locations := row.rows_for_die (packing_id, version_no, die_no, any_connection, expected_count.At_least_zero)
		end
		
	anchored_die_location: PACKING_DIE_LOCATIONS_ROW is
			-- For anchoring only
		do
		ensure
			result_void: Result = Void
		end

	fetch_project is
			-- Fetch project for `Current'
		local
			row: like anchored_project
		do
			create row.make_template
			cached_project := row.rows_for_project_version (packing_id, project_no, project_version_no, any_connection, expected_count.Exactly_one).first
		end
		
	anchored_project: PACKING_PROJECTS_ROW is
			-- For anchoring only
		do
		ensure
			result_void: Result = Void
		end

	fetch_vendor_geometry is
			-- Fetch vendor_geometry for `Current'
		do
			create cached_vendor_geometry.make_by_key (any_connection, option.old_row, geometry_id)
		end
		
	anchored_vendor_geometry: GEOMETRIES_ROW is
			-- For anchoring only
		do
		ensure
			result_void: Result = Void
		end

feature {PACKING_DIES_ROW} -- Implementation

	cached_die_locations: like die_locations
			-- Cached list of die_locations
		
	cached_project: like project
			-- Cached project for `Current'
		
	cached_vendor_geometry: like vendor_geometry
			-- Cached vendor geometry for `Current'
		
end
