
indexing

	author: "David Hollenberg"

class PACKING_PROJECT_PACKAGINGS_ROW
inherit
	PACKING_PROJECT_PACKAGINGS_RAW
	PACKING_QUERIES
	DB_HASHABLE_ROW
		undefine
			initialize_new, initialize_old, fill_in
		end
	ROW_COUNT_CHECK_ACCESS
		export
			{NONE} all
		end
	ITEM_COUNT_RANGES_ACCESS
		export
			{NONE} all
		end
	PACKAGING_UTILITIES_INTERFACE
		export
			{NONE} all
		end

create
	make_template, make_null, make_by_key, make_detached

feature -- Queries

	rows_for_project_version (a_packing_id: STRING a_project_no, a_proj_version: INTEGER conn: DB_CONNECTION range: ITEM_COUNT_RANGE): LIST [like Current] is
			-- Packagings for `a_packing_id', `a_project_no' and
			-- `a_proj_version'
		require
			a_packing_id_not_void: a_packing_id /= Void
			connection_exists: conn /= Void
			range_exists: range /= Void
		local
			b: DB_BIND_SII
		do
			create b.make (a_packing_id, a_project_no, a_proj_version)
			Result := all_rows_filtered (conn, "packing_id = :1 and project_no = :2 and version_no = :3", b)
			check_row_count (Result, range)
		ensure
			result_exists: Result /= Void
		end

feature -- Properties

	hash_code: INTEGER is
		local
			s: STRING
		do
			create s.make (packing_id.count + 15)
			s.append (packing_id)
			s.extend ('%U')
			s.append_integer (project_no)
			s.extend ('%U')
			s.append_integer (version_no)
			s.extend ('%U')
			s.append_integer (line_no)
			Result := s.hash_code
		end
	
	max_die_size: CONST_POINT is
			-- Maximum die size dictated by `Current'
			-- or Void if no maximum limit
		do
			if not is_cached_max_die_size then
				fetch_max_die_size
			end
			Result := cached_max_die_size
		end

	is_cached_max_die_size: BOOLEAN
			-- Is `max_die_size' currently cached?

	package: PACKAGES_ROW is
			-- Package for `Current'
		require
			package_id_not_null: not is_null_package_id
		do
			if not is_cached_package then
				fetch_package
			end
			Result := cached_package
		end

	is_cached_package: BOOLEAN is
			-- Is `package' currently cached?
		do
			Result := cached_package /= Void
		end

feature -- Operations

	uncache_max_die_size is
			-- Remove `max_die_size' from cache
		do
			cached_max_die_size := Void
			is_cached_max_die_size := False
		end

	uncache_package is
			-- Remove `package' from cache
		do
			cached_package := Void
		end

feature {NONE} -- Implementation

	fetch_max_die_size is
			-- Fetch maximum die size dictated by `Current'
		local
			p: like max_die_size
		do
			if not is_null_package_id then
				p := package_max_die_size (any_connection, package_id, downbond_location_reqd)
				if design_package_rotation = 90 or design_package_rotation = 270 then
					create p.make_xy (p.y, p.x)
				end
			else
				p := Void
			end
			cached_max_die_size := p
			is_cached_max_die_size := True
		end

	fetch_package is
			-- Fetch package for `Current'
		require
			package_id_not_null: not is_null_package_id
		local
			row: PACKAGES_ROW
		do
			create row.make_template
			cached_package := row.packages_with_id (package_id, any_connection, expected_count.Exactly_one).first
		end

feature {PACKING_PROJECT_PACKAGINGS_ROW} -- Implementation

	cached_max_die_size: CONST_POINT
			-- Maximum die size dictated by `Current',
			-- cached in memory

	cached_package: PACKAGES_ROW
			-- Package for `Current', cached in memory

end
