
indexing

	author: "David Hollenberg"

class PACKING_PROJECTS_MROW
inherit
	PACKING_PROJECTS_MRAW
		redefine
			copy_from_row
		end
	PACKING_PROJECTS_ROW
		undefine
			initialize_new,
			initialize_new_by_key
		redefine
			anchored_packaging, anchored_option,
			anchored_fab_instance, anchored_cust_ic_design,
			packagings, options, set_packagings, set_options
		end

create
	make_template, make_null, make_by_key, make_detached,
	make_cust_project, make_test_vehicle

feature {NONE} -- Creation

	make_cust_project (fno: INTEGER date: DB_DATE conn: DB_CONNECTION) is
			-- Create detached row for customer project
			-- with fab_no `fno', with only non-key
			-- information initialized
		require
			date_not_void: date /= Void
			conn_not_void: conn /= Void
		local
			type: IC_FAB_INSTANCE_TYPE_VALUES
			ifi: IC_FAB_INSTANCES_ROW
		do
			make_detached (conn)
			set_ic_fab_instance_type (type.Customer_fab)
			set_date_selected (date)
			set_date_synchronized (date)
			set_m_fab_no (fno)
			
			ifi := fab_instance
			set_design_no (ifi.design_no)
			set_overhead_style (ifi.overhead_style)
			set_run_type_requested (ifi.run_type_requested)
			set_date_fab_requested (ifi.date_fab_requested)
			set_run_date_requested (ifi.run_date_requested)
			if not ifi.is_null_max_allowed_die_size_x then
				set_max_allowed_die_size_x (ifi.max_allowed_die_size_x)
			end
			if not ifi.is_null_max_allowed_die_size_y then
				set_max_allowed_die_size_y (ifi.max_allowed_die_size_y)
			end
			if not ifi.is_null_allow_oversize_die_on_run then
				set_allow_oversize_die_on_run (ifi.allow_oversize_die_on_run)
			end
			if not ifi.is_null_oversize_die_incl_conditions then
				set_oversize_die_incl_conditions (ifi.oversize_die_incl_conditions)
			end
			set_run_id_copy (ifi.run_id)
			set_quantity_ordered (ifi.work_order.ic_quantity_ordered)
			if not ifi.is_null_quantity_requested then
				set_quantity_requested (ifi.quantity_requested)
			end
			if not ifi.is_null_quantity_unpackaged_requested then
				set_quantity_unpackaged_requested (ifi.quantity_unpackaged_requested)
			end
			if not ifi.is_null_design_wafer_rotation_reqd then
				set_design_wafer_rotation_reqd (ifi.design_wafer_rotation_reqd)
			end
			set_net_address (ifi.net_address)
			set_phone_number (ifi.phone_number)
			copy_ic_design_fields
			set_cust_acct_no (ifi.work_order.cust_acct_no)
			if not ifi.is_null_design_die_rotation then
				set_design_die_rotation (ifi.design_die_rotation)
			else
				set_null_design_die_rotation
			end
		end

	make_test_vehicle (dno: INTEGER date: DB_DATE conn: DB_CONNECTION) is
			-- Create detached row for test vehicle
			-- with design_no `dno' with selection and 
			-- synchronization dates set to `date'.  Initialize
			-- only non-key information
		require
			date_not_void: date /= Void
			conn_not_void: conn /= Void
		local
			type: IC_FAB_INSTANCE_TYPE_VALUES
			style: OVERHEAD_STYLE_VALUES
			l_style: STRING
		do
			make_detached (conn)
			set_ic_fab_instance_type (type.Test_vehicle)
			set_date_selected (date)
			set_date_synchronized (date)
			set_design_no (dno)
			set_test_vehicle_design_no (dno)
			if cust_ic_design.is_null_label_box_size_x then
				l_style := style.External_
			else
				l_style := style.Internal
			end
			set_overhead_style (l_style)
			copy_ic_design_fields
		end

feature -- Properties

	packagings: LIST [like anchored_packaging] is
			-- Packagings for `Current'
		do
			if is_detached then
				Result := detached_packagings
			else
				if cached_packagings = Void then
					fetch_packagings
				end
				Result := cached_packagings
			end
		end

	options: LIST [like anchored_option] is
			-- Options for `Current'
		do
			if is_detached then
				Result := detached_options
			else
				if cached_options = Void then
					fetch_options
				end
				Result := cached_options
			end
		end

	deep_detached_twin: like Current is
			-- Detached copy of `Current' with all
			-- dependent rows detached as well
		do
			Result := detached_twin
			Result.set_packagings (packagings.twin)
			Result.set_options (options.twin)
			Result.uncache_dependents
				-- Just in case: shouldn't be referenced
			Result.detach_dependents_deep
		end

	is_deep_unchanged: BOOLEAN is
			-- Are `Current' and all its dependent rows
			-- unchanged from their original versions?
			-- Returns False if no original version
		local
			pkg_pair: DB_ROW_SET_PAIR [like anchored_packaging]
			opt_pair: DB_ROW_SET_PAIR [like anchored_option]
		do
			Result := all_columns_equal_original
				and then packagings.for_all (agent {like anchored_packaging}.is_deep_unchanged)
				and then options.for_all (agent {like anchored_option}.is_deep_unchanged)
			if Result then
				create pkg_pair.make_from_lists (packagings, original_packagings, create {KEY_COLS_EQUALITY_TESTER [like anchored_packaging]})
				Result := pkg_pair.is_identical
			end
			if Result then
				create opt_pair.make_from_lists (options, original_options, create {KEY_COLS_EQUALITY_TESTER [like anchored_option]})
				Result := opt_pair.is_identical
			end
		end

	new_packagings: LIST [like anchored_packaging] is
			-- Create new list of packagings from database
			-- (assumes project is locked)
		local
			wolis: LIST [CUST_WO_LINE_ITEMS_ROW]
			woli: CUST_WO_LINE_ITEMS_ROW
			pkging: like anchored_packaging
			n: INTEGER
		do
			wolis := fab_instance.work_order.line_items
			create {ARRAYED_LIST [like anchored_packaging]} Result.make (wolis.count)
			from
				n := 0
				wolis.start
			until
				wolis.after
			loop
				woli := wolis.item
				if not woli.is_null_ic_packaging_no then
					create pkging.make_from_line_item (Current, woli)
					n := n + 1
					pkging.set_line_no (n)
					Result.extend (pkging)
				end
				wolis.forth
			end
		end

feature -- Modification

	set_assigned_wafer_etest_name (a_value: like assigned_wafer_etest_name table: HASH_TABLE [WAFER_PROCESSES_ROW, STRING]) is
			-- Set `assigned_wafer_etest_name' to `a_value'
		local
			list: like compatible_processes
		do
			assigned_wafer_etest_name := a_value
			list := compatible_processes
			compatible_processes_with_assigned_etest_name := list.twin
			compatible_processes_with_assigned_etest_name.wipe_out
			list.do_all (agent add_process_if_etest_compatible (?, table))
		end

	set_packagings (list: like packagings) is
			-- Set `packagings' to `list'
		do
			if is_detached then
				detached_packagings := list
			else
				precursor (list)
			end
		end

	set_options (list: like options) is
			-- Set `options' to `list'
		do
			if is_detached then
				detached_options := list
			else
				precursor (list)
			end
		end

	set_packing (p: like packing) is
		do
			cached_packing := p
		end

feature -- Row copy

	copy_from_row (r: like original_row) is
			-- Initialize `Current' from `r'
		do
			precursor (r)
			assigned_process := r.assigned_process
			compatible_processes := r.compatible_processes
			computed_layer_options := r.computed_layer_options
			design_wt_layer_errors := r.design_wt_layer_errors
			cached_packing := r.cached_packing
			cached_packagings := r.cached_packagings
			cached_options := r.cached_options
			cached_cust_account := r.cached_cust_account
			cached_cust_ic_design := r.cached_cust_ic_design
			cached_fab_instance := r.cached_fab_instance
			cached_test_vehicle := r.cached_test_vehicle
			is_selected := r.is_selected
			reason_unselectable := r.reason_unselectable
			compatible_wafer_etest_ids := r.compatible_wafer_etest_ids
			compatible_wafer_etest_names := r.compatible_wafer_etest_names
			assigned_wafer_etest_name := r.assigned_wafer_etest_name
		end

feature -- Operations

	update_for_new_version is
			-- Update `Current' with information for new
			-- project version
		do
			-- FIXME: this is never called.  Should it be called?
			if not is_deep_unchanged then
				set_version_no (highest_used_version + 1)
				update_dependents_with_version (version_no)
			end
		end

	register_deep is
			-- Register `Current' and all dependent rows
			-- on implicit connection
		do
			register_on_implicit_connection
			register_dependents_deep
		end

feature {PACKING_PROJECTS_MROW} -- Implementation

	detach_dependents_deep is
			-- Detach dependent rows from database connection
			-- deeply
		do
			detach_packagings_deep
			detach_options_deep
		end

feature {NONE} -- Implementation

	detach_packagings_deep is
			-- Detach `packagings' from database connection, along
			-- with all dependents
		do
			original_packagings := packagings.twin
			original_packagings.wipe_out
			original_packagings.append (packagings)
			packagings.wipe_out
			original_packagings.do_all (agent append_detached_packaging (?, packagings))
		end

	detach_options_deep is
			-- Detach `options' from database connection, along
			-- with all dependents
		do
			original_options := options.twin
			original_options.wipe_out
			original_options.append (options)
			options.wipe_out
			original_options.do_all (agent append_detached_option (?, options))
		end

	append_detached_packaging (p: like anchored_packaging list: like packagings) is
			-- Append detached copy of `p' to `list'
		do
			list.extend (p.detached_twin)
		end
		
	append_detached_option (o: like anchored_option list: like options) is
			-- Append detached copy of `o' to `list'
		do
			list.extend (o.detached_twin)
		end
		
	update_dependents_with_version (n: INTEGER) is
			-- Update all dependent rows with project_no `n'
			-- prior to save of new version `n'
		do
			last_line_no := 0
			packagings.do_all (agent renumber_packaging)
			packagings.do_all (agent {like anchored_packaging}.set_version_no (n))
			options.do_all (agent {like anchored_option}.set_version_no (n))
		end

	renumber_packaging (p: PACKING_PROJECT_PACKAGINGS_MROW) is
		do
			last_line_no := last_line_no + 1
			p.set_line_no (last_line_no)
		end
	
	register_dependents_deep is
			-- Register all dependent rows on implicit
			-- connection
		do
			-- Move "detached" versions back to "cached" versions
			-- since they will no longer be detached at end
			-- of routine
			
			cached_packagings := detached_packagings
			detached_packagings := Void
			packagings.do_all (agent {like anchored_packaging}.register_deep)
			
			cached_options := detached_options
			detached_options := Void
			options.do_all (agent {like anchored_option}.register_deep)
		end

	add_process_if_etest_compatible (a_process: STRING table: HASH_TABLE [WAFER_PROCESSES_ROW, STRING]) is
		do
			table.search (a_process)
			if table.found then
				if assigned_wafer_etest_id.is_empty or table.found_item.wafer_etest_id.is_equal (assigned_wafer_etest_id) then
					compatible_processes_with_assigned_etest_name.extend (a_process)
				end
			else
				handle_hard_error ("cannot find process " + a_process + " in lookup table to get its wafer etest ID")
			end
		end

	copy_ic_design_fields is
			-- Initialize from info in `cust_ic_design'
		local
			cid: CUST_IC_DESIGNS_ROW
			state: DESIGN_ACCEPT_STATE_VALUES
		do
			cid := cust_ic_design
			set_design_techn_id (cid.design_techn_id)
			set_export_control_type (cid.export_control_type)
			set_design_name (cid.design_name)
			set_design_accept_state (cid.design_accept_state)
			set_cust_acct_no (cid.cust_acct_no)
			if cid.design_accept_state.is_equal (state.Passed) then
				set_is_design_size_estimate (False)
				if not cid.is_null_design_size_x then
					set_design_size_x (cid.design_size_x)
				end
				if not cid.is_null_design_size_y then
					set_design_size_y (cid.design_size_y)
				end
			else
				set_is_design_size_estimate (True)
				if not cid.is_null_est_design_size_x then
					set_design_size_x (cid.est_design_size_x)
				end
				if not cid.is_null_est_design_size_y then
					set_design_size_y (cid.est_design_size_y)
				end
			end
			set_design_die_rotation (0)
		end

	anchored_packaging: PACKING_PROJECT_PACKAGINGS_MROW is
			-- For anchoring only
		do
		end

	anchored_option: PACKING_PROJECT_OPTIONS_MROW is
			-- For anchoring only
		do
		end

	anchored_fab_instance: IC_FAB_INSTANCES_MROW is
			-- For anchoring only
		do
		end

	anchored_cust_ic_design: CUST_IC_DESIGNS_MROW is
			-- For anchoring only
		do
		end

	last_line_no: INTEGER
			-- Last line number assigned during renumbering

	detached_packagings: like packagings
			-- List of detached packagings (no explict DB 
			-- connection)
		
	detached_options: like options
			-- List of detached options (no explict DB connection)
		
	original_packagings: like packagings
			-- Original list of packagings before detach

	original_options: like options
			-- Original list of options before detach


end
