
indexing
	
	description: "Utilities related to a record in PACKINGS table %
		%the PACKINGS table"
	author: "David Hollenberg"
	date: "November 7, 2005"

deferred class PACKING_RECORD_UTILITIES

inherit

	ANY
		undefine
			default_create
		end
	ITEM_COUNT_RANGES_ACCESS
		export
			{NONE} all
		undefine
			default_create
		end
feature -- Operations

	populate_creator (f: EV_COMBO_BOX)
		local
			row: M_OPERATORS_ROW
			closers: LIST [M_OPERATORS_ROW]
		do
			create row.make_template
			closers := row.run_closer_operators_ordered (connection, expected_count.At_least_one)
			closers.do_all (agent add_closer (?, f))
		end

	add_closer (row: M_OPERATORS_ROW f: EV_COMBO_BOX)
		local
			list_item: EV_LIST_ITEM
		do
			create list_item.make_with_text (row.username)
			f.extend (list_item)
			if row.username.is_equal (connection.user_name) then
				list_item.enable_select
			end
		end

	populate_run_type (f: EV_COMBO_BOX)
		local
			row: RUN_TYPES_ROW
			run_types: LIST [RUN_TYPES_ROW]
		do
			create row.make_template
			run_types := row.all_run_types_ordered (connection, expected_count.At_least_one)
			run_types.do_all (agent add_run_type (?, f))
		end

	add_run_type (row: RUN_TYPES_ROW f: EV_COMBO_BOX)
		local
			list_item: EV_LIST_ITEM
		do
			create list_item.make_with_text (row.run_type)
			f.extend (list_item)
		end

	populate_overhead_style (f: EV_COMBO_BOX)
		local
			fact: SINGLE_CONSTRAINED_VALUE_FACTORY
			l_choice, l_text: STRING
			style: OVERHEAD_STYLE_VALUES
		do
			create fact
			fact.enable_choices_prepended
			from
				allowed_overhead_styles.start
			until
				allowed_overhead_styles.after
			loop
				l_choice := allowed_overhead_styles.item
				l_text := overhead_style_descriptions.item (l_choice)
				l_choice := style.null_translated (l_choice)
				fact.add_choice_and_text (l_choice, l_text)
				allowed_overhead_styles.forth
			end
			fact.set_default_choice (style.External_)
			fact.set_combo_box (f)
			fact.fill_in_combo_box
		end

feature -- Properties

	connection: DB_CONNECTION
		deferred
		end

	Border_threshold: DOUBLE = 5.0
			-- Threshold below which user gets warning

	wafer_techns_row: WAFER_TECHNS_ROW
			-- Row for wafer techn
		deferred
		end

	default_other_border: DOUBLE
			-- Default value for other border from wafer techn
		do
			Result := wafer_techns_row.other_border
		end

	minimum_other_border: DOUBLE
			-- Minimum value allowed for other border
		do
			Result := 0
		end

	half_grid_resolution: DOUBLE
			-- Half of grid resolution for wafer techn
		do
			Result := wafer_techns_row.packing_grid_resolution / 2.0
		end

	border_scale: INTEGER
			-- Scale to use for border
		do
			Result := {WAFER_TECHNS_ROW}.scale_top_border
		end

	top_overhead_description (an_overhead_style, an_other_border: STRING): STRING
			-- Description of overhead on top side of design,
			-- based on currently selected values
		local
			val: DOUBLE
			ok: BOOLEAN
		do
			ok := an_other_border.is_double
			if ok then
				val := an_other_border.to_double
				ok := val >= 0.0
			end
			if ok then
				Result := wafer_techns_row.top_overhead_description (an_overhead_style, val)
			else
				Result := ""
			end
		end
	
feature {NONE} -- Implementation

	allowed_overhead_styles: LIST [STRING]
			-- List of allowed overhead styles for run
		local
			style: OVERHEAD_STYLE_VALUES
		do
			if cached_allowed_overhead_styles = Void then
				create {ARRAYED_LIST [STRING]} cached_allowed_overhead_styles.make (3)
				cached_allowed_overhead_styles.compare_objects
				cached_allowed_overhead_styles.extend ("")
				cached_allowed_overhead_styles.extend (style.External_)
				cached_allowed_overhead_styles.extend (style.Internal)
				cached_allowed_overhead_styles.extend (style.None)
			end
			Result := cached_allowed_overhead_styles
		end

	overhead_style_descriptions: HASH_TABLE [STRING, STRING]
			-- Description for each overhead style, to be
			-- displayed to user
		local
			style: OVERHEAD_STYLE_VALUES
		do
			if cached_overhead_style_descriptions = Void then
				create cached_overhead_style_descriptions.make (11)
				cached_overhead_style_descriptions.put ("Die ID inside or above design, depending on design", "")
				cached_overhead_style_descriptions.put ("All projects have die ID in external overhead strip above design", style.External_)
				cached_overhead_style_descriptions.put ("All projects have die ID inside design", style.Internal)
				cached_overhead_style_descriptions.put ("No die ID in any project", style.None)
			end
			Result := cached_overhead_style_descriptions
		end

	cached_allowed_overhead_styles: like allowed_overhead_styles
			-- Field with allowed overhead styles for run,
			-- cached

	cached_overhead_style_descriptions: like overhead_style_descriptions
			-- Field with allowed overhead styles for run,
			-- cached

end
