
indexing
	
	description: "Standard packing status reporter"
	author: "David Hollenberg"
	date: "December 4, 2006"

class PACKING_STD_STATUS_REPORTER
inherit
	PACKING_STATUS_REPORTER
		redefine
			default_create
		end
	GUI_UTILITIES_ACCESS
		export
			{NONE} all
		redefine
			default_create
		end

create
	default_create

feature {NONE} -- Creation

	default_create
		do
			create status_line_update_actions
			create status_window_update_actions
			create event
		end

feature -- Properties

	status_line_update_actions: ACTION_SEQUENCE [TUPLE [STRING]]
			-- Actions to take to update packer status line

	status_window_update_actions: ACTION_SEQUENCE [TUPLE [STRING]]
			-- Actions to take to update packer status window

feature -- Operations

	report_start
			-- Report start of an operation
		do
			append_separator
		end

	report_end
			-- Report end of an operation
		do
			append_separator
		end

	report_status (a_message: STRING)
			-- Report transient status described by
			-- `a_message'.  Includes things like "started
			-- operation", "operation in progress",
			-- "waiting for next key", etc
		do
			set_last_message (a_message)
			-- update_status_line (a_message)
			update_status_line_and_window (a_message)
		end

	report_progress (a_message: STRING)
			-- Report operation in progress described by
			-- `a_message'
		do
			if last_message = Void or else not last_message.is_equal (a_message) then
				-- update_status_line (a_message)
				update_status_line_and_window (a_message)
			end
			set_last_message (a_message)
		end

	report_select (a_message: STRING)
			-- Report select operation described by `a_message'
		do
			set_last_message (a_message)
			update_status_line_and_window (a_message)
			report_end
		end

	report_view (a_message: STRING)
			-- Report change of how objects are displayed
			-- described by `a_message'
		do
			set_last_message (a_message)
			-- update_status_line (a_message)
			update_status_line_and_window (a_message)
			report_end
		end

	report_lock (a_message: STRING)
			-- Report locking of object property or properties
			-- described by `a_message'
		do
			set_last_message (a_message)
			update_status_line_and_window (a_message)
			report_end
		end

	report_navigation (a_message: STRING)
			-- Report operation that affects what part of
			-- model is displayed, described by `a_message'
		do
			set_last_message (a_message)
			-- update_status_line (a_message)
			update_status_line_and_window (a_message)
			report_end
		end

	report_change (a_message: STRING)
			-- Report operation that changes the packing,
			-- described by `a_message'.  Includes things
			-- like move, duplicate, add or remove
			-- projects, etc
		do
			set_last_message (a_message)
			update_status_line_and_window (a_message)
		end

	report_failure (a_message: STRING)
			-- Report operation failure described by
			-- `a_message'.  This is for the case where operation
			-- failed completely, not where some part of the
			-- operation could be done
		do
			set_last_message (a_message)
			update_status_line_and_window (a_message)
			gui.ring_bell_once
		end

	report_cancellation (a_message: STRING)
			-- Report operation cancellation described by
			-- `a_message'
		do
			set_last_message (a_message)
			update_status_line_and_window (a_message)
			report_end
		end

	report_validity (a_message: STRING)
			-- Report validity check operation described by
			-- `a_message'
		do
			set_last_message (a_message)
			update_status_line_and_window (a_message)
		end

	report_exit (a_message: STRING)
			-- Report exit of packer described by
			-- `a_message'
		do
			set_last_message (a_message)
			update_status_line_and_window (a_message)
			report_end
		end

	report_details (a_message: STRING)
			-- Report details of operation that succeeded or
			-- failed, described by `a_message'
		do
			set_last_message (a_message)
			update_status_window (a_message)
		end

	report_internal_error (a_message: STRING)
			-- Report internal error `a_message'
		do
			set_last_message (a_message)
			update_status_line_and_window (a_message)
		end

feature {NONE} -- Implementation

	append_separator
			-- Append separator to status window to separate
			-- text of logical operations
		local
			sep: like separator
		do
			sep := separator
			if last_message = Void or else not last_message.is_equal (sep) then
				update_status_window (sep)
			end
			set_last_message (sep)
		end

	set_last_message (a_message: STRING)
			-- Set `last_message' to `a_message'
		do
			last_message := a_message
		end

	update_status_line (a_message: STRING)
			-- Update status line with `a_message'
		do
			status_line_update_actions.call ([a_message])
			publish_event (a_message)
		end

	update_status_window (a_message: STRING)
			-- Update status window with `a_message'
		do
			status_window_update_actions.call ([a_message])
			publish_event (a_message)
		end

	update_status_line_and_window (a_message: STRING)
			-- Update status line and status window with
			-- `a_message'
		do
			status_line_update_actions.call ([a_message])
			status_window_update_actions.call ([a_message])
			publish_event (a_message)
		end

	publish_event (a_message: STRING)
			-- Publish packer output
		do
			event.set_message (a_message)
			event.publish
		end

	separator: STRING
			-- String that separates text for logical operations
		do
			if separator_internal = Void then
				separator_internal := ""
			end
			Result := separator_internal
		end

	separator_internal: STRING

	last_message: STRING
			-- Last message passed to reporter

	event: PACKING_OUTPUT_EVENT

end
