
indexing

	description: "Target which indicates what packing of a run is destined for"
	author: "David Hollenberg"
	date: "August 28, 2008"

deferred class PACKING_TARGET
inherit
	ANY
	MOSIS_GRAPHICAL_UTILITIES
		export
			{NONE} all
		end
	NUMBER_UTILITIES_ACCESS
		export
			{NONE} all
		end
	PAYLOAD_KERF_FIELD_INTERFACE

feature {NONE} -- Creation

	make (r: like packing_targets_row table: like all_packing_targets sets: like related_field_sets sess: RC_RUN_CLOSING_SESSION)
			-- Create with packing_targets row `r'
		require
			r_not_void: r /= Void
			table_not_void: table /= Void
			sets_not_void: sets /= Void
			sess_not_void: sess /= Void
		local
			res: DOUBLE
			scale: INTEGER
		do
			packing_targets_row := r
			all_packing_targets := table
			run_id := sess.run_id
			related_field_sets := sets
			session := sess
			create {ARRAYED_LIST [FIELD_SETS_MROW]} field_sets.make (10)
			create field_set_table.make (11)

			res := sess.cache.wafer_techns_row.packing_grid_resolution
			scale := {WAFER_TECHNS_ROW}.scale_top_border
			create grid
			grid.set_size_resolution (res)
			grid.set_edge_resolution (res)
			grid.set_center_resolution (res / 2.0)
			grid.set_scale (scale)
		end

feature -- Properties

	session: RC_RUN_CLOSING_SESSION
			-- Run closing session to use

	run_id: STRING
			-- Run ID of run for which packing
			-- target is being selected

	packing_targets_row: PACKING_TARGETS_ROW
			-- Row from table PACKING_TARGETS

	all_packing_targets: HASH_TABLE [PACKING_TARGET, STRING]
			-- All known packing targets, indexed by
			-- target name

	target_name: STRING
			-- Name of packing target
		do
			Result := packing_targets_row.packing_target
		end

	is_reticle: BOOLEAN
			-- Is target a reticle?
		do
			Result := packing_targets_row.is_reticle
		end

	is_custom: BOOLEAN
			-- Is target a custom target?
		deferred
		end

	field_sets: DYNAMIC_LIST [FIELD_SETS_MROW]
			-- List of possible field sets for `Current'

	related_field_sets: DYNAMIC_LIST [FIELD_SETS_ROW]
			-- List of field sets ever associated with `run_id'
			-- or its wafer techn

	has_existing_field_set: BOOLEAN
			-- Are there any existing field sets in database
			-- for `Current'?
		do
			Result := not field_sets.is_empty
		end

	only_field_set: FIELD_SETS_MROW
			-- Only field set for `Current' or Void
			-- if none yet
		require
			is_custom: is_custom
			has_existing_field_set: has_existing_field_set
		do
			if not field_sets.is_empty then
				Result := field_sets.first
			end
		end

	wafer_techn_packing_target: WAFER_TECHN_PACKING_TARGETS_ROW
			-- Row from wafer_techn_packing_targets for
			-- run's wafer techn for `Current'

	standard_name: STRING
			-- Standard name for field set and litho size
		require
			is_custom: is_custom
		do
		end

	is_allowed: BOOLEAN
			-- Is `Current' allowed to be selected
			-- for this run?
		do
			Result := has_enough_data_to_build_interface and is_available
		end

	has_enough_data_to_build_interface: BOOLEAN
			-- Is there enough data from related
			-- field sets to build the interface for `Current'?
		deferred
		end

	is_available: BOOLEAN
			-- Is `Current' marked as being available
			-- for run's wafer techn?
		do
			Result := wafer_techn_packing_target /= Void and then wafer_techn_packing_target.available
		end

	selected_field_set_name: STRING
			-- Name of currently selected field set
		deferred
		end

	field_set_field: EV_TEXT_FIELD
			-- Field for selecting field set

	renaming_style_field: EV_TEXT_FIELD
			-- Field for selecting/displaying pattern file
			-- renaming style

	fab_location_field: EV_TEXT_FIELD
			-- Field for selecting/displaying fab location

	wafer_type_field: EV_TEXT_FIELD
			-- Field for selecting/displaying wafer type

	mask_size_field: EV_TEXT_FIELD
			-- Field for selecting/displaying mask size

	tweezer_margin_field: EV_TEXT_FIELD
			-- Field for setting/displaying tweezer margin

	primary_flat_field: EV_TEXT_FIELD
			-- Field for selecting/displaying primary
			-- flat location

	target_specific_box: EV_HORIZONTAL_BOX
			-- Box with packing target specific contents,
			-- to be displayed in interface
		local
			cols: WIDGET_COLUMN_SET
		do
			if cached_target_specific_box = Void then
				create cols.make (2)
				cols.set_padding (user_interface.padding)
				cols.set_border_width (user_interface.border_width)
				cols.append_row (<< gui.label_with_text ("Field Set"), field_set_field >>)
				if payload_size_x_field /= Void then
					cols.append_row (<< initial_label (Payload_size_x_text), payload_size_x_field >>)
				end
				if payload_size_y_field /= Void then
					cols.append_row (<< initial_label (Payload_size_y_text), payload_size_y_field >>)
				end
				if external_kerf_size_x_field /= Void then
					cols.append_row (<< initial_label (External_kerf_size_x_text), external_kerf_size_x_field >>)
				end
				if external_kerf_size_y_field /= Void then
					cols.append_row (<< initial_label (External_kerf_size_y_text), external_kerf_size_y_field >>)
				end
				if field_size_x_field /= Void then
					cols.append_row (<< initial_label (Field_size_x_text), field_size_x_field >>)
				end
				if field_size_y_field /= Void then
					cols.append_row (<< initial_label (Field_size_y_text), field_size_y_field >>)
				end
				if renaming_style_field /= Void then
					cols.append_row (<< gui.label_with_text (Pattern_file_renaming_style_text), renaming_style_field >>)
				end
				if fab_location_field /= Void then
					cols.append_row (<< gui.label_with_text (Fab_location_text), fab_location_field >>)
				end
				if wafer_type_field /= Void then
					cols.append_row (<< gui.label_with_text (Wafer_type_text), wafer_type_field >>)
				end
				if mask_size_field /= Void then
					cols.append_row (<< gui.label_with_text (Mask_size_text), mask_size_field >>)
				end
				if primary_flat_field /= Void then
					cols.append_row (<< gui.label_with_text (Primary_flat_location_text), primary_flat_field >>)
				end
				if tweezer_margin_field /= Void then
					cols.append_row (<< gui.label_with_text (Tweezer_margin_text), tweezer_margin_field >>)
				end
				cols.disable_column_expand (1)
				cols.align_column_right (1)
				cols.align_column_left (2)
				cached_target_specific_box := cols.box
			end
			Result := cached_target_specific_box
		end

	initial_text (a_text: STRING): STRING
			-- `a_text' prefixed with "Initial" if
			-- `Current' is a custom target
		do
			if is_custom then
				Result := "Initial " + a_text
			else
				Result := a_text
			end
		end

	initial_label (a_text: STRING): EV_LABEL
			-- Label for `a_text', prefixed with "Initial" if
			-- `Current' is a custom target
		do
			Result := gui.label_with_text (initial_text (a_text))
		end

feature -- Modification

	add_field_set (f: FIELD_SETS_MROW)
			-- Add `f' to `field_sets'
		do
			field_sets.extend (f)
			field_set_table.put (f, f.field_set_name)
		end

	set_wafer_techn_packing_target (wtpt: like wafer_techn_packing_target)
			-- Set `wafer_techn_packing_targets' to `wtpt'
		do
			wafer_techn_packing_target := wtpt
		end

	create_text_fields
			-- Create text fields for `Current'
		deferred
		end

feature -- Operations

	validate_input
			-- Validate user input.  Leave any validation
			-- errors in `validation_errors' and set `is_input_valid'
			-- to indicate whether input is valid
		deferred
		ensure
			validation_errors_not_void: validation_errors /= Void
		end

	update_database
			-- Do any needed database inserts or updates
			-- prior to setting field set to one user
			-- has selected
		deferred
		end

feature {NONE} -- Implementation

	new_read_only_text_field (s: STRING min_width: INTEGER): EV_TEXT_FIELD
			-- New text field with initial text `s' with
			-- minimum width in characters `min_width' that
			-- is not editable
		do
			create Result.make_with_text (s)
			Result.set_minimum_width_in_characters (min_width)
			Result.disable_edit
		ensure
			result_not_void: Result /= Void
		end

	new_writable_text_field (s: STRING min_width: INTEGER): EV_TEXT_FIELD
			-- New text field with initial text `s' with
			-- minimum width in characters `min_width' that
			-- is editable
		do
			create Result.make_with_text (s)
			Result.set_minimum_width_in_characters (min_width)
			Result.enable_edit
		ensure
			result_not_void: Result /= Void
		end

	new_not_applicable_text_field (min_width: INTEGER): EV_TEXT_FIELD
			-- New text field with initial text "Not
			-- applicable" with minimum width in pixels
			-- `min_width' that is not editable
		do
			Result := new_read_only_text_field ("Not applicable", min_width)
		ensure
			result_not_void: Result /= Void
		end

	new_std_field_set_field: EV_COMBO_BOX
			-- New non-editable combo box with list of
			-- possible field sets
		local
			fact: SINGLE_CONSTRAINED_VALUE_FACTORY
			fs: FIELD_SETS_ROW
		do
			create fact
			fact.enable_choices_prepended
			from
				field_sets.start
			until
				field_sets.after
			loop
				fs := field_sets.item
				fact.add_choice_and_text (fs.field_set_name, fs.descr)
				field_sets.forth
			end
			fact.set_default_choice (fact.choices.first)
			fact.generate_combo_box
			Result := fact.combo_box
			Result.disable_edit
			Result.change_actions.extend (agent process_field_set_change)
			gui.do_once_on_idle (agent process_field_set_change)
		ensure
			result_not_void: Result /= Void
		end

	new_custom_field_set_field (min_width: INTEGER): EV_TEXT_FIELD
			-- New non-editable text field with
			-- `min_width' displaying name of only
			-- possible custom field set
		require
			is_custom: is_custom
		local
			l_name: STRING
		do
			if has_existing_field_set then
				l_name := only_field_set.field_set_name
			else
				l_name := standard_name
			end
			Result := new_read_only_text_field (l_name, min_width)
		ensure
			result_not_void: Result /= Void
		end

	new_custom_renaming_style_field: EV_COMBO_BOX
			-- New non-editable combo box with
			-- list of possible pattern file renaming styles
		local
			fact: SINGLE_CONSTRAINED_VALUE_FACTORY
			ls: LITHO_SIZES_ROW
			style, default_choice: STRING
		do
			create fact
			fact.disable_choices_prepended
			fact.add_choice_and_text (Null_choice, None_text)
			from
				related_field_sets.start
			until
				related_field_sets.after
			loop
				ls := related_field_sets.item.litho_size
				if not ls.is_null_pattern_file_renaming_style then
			 		style := ls.pattern_file_renaming_style
					if not fact.choices.has (style) then
						fact.add_choice_and_text (style, style)
					end
				end
				related_field_sets.forth
			end
			add_select_one (fact)
			if is_custom and then has_existing_field_set then
				if only_field_set.is_null_pattern_file_renaming_style then
					default_choice := Null_choice
				else
					default_choice := only_field_set.pattern_file_renaming_style
				end
			else
				default_choice := fact.choices.first
			end
			fact.set_default_choice (default_choice)
			fact.generate_combo_box
			Result := fact.combo_box
			Result.disable_edit
		ensure
			result_not_void: Result /= Void
		end

	new_custom_fab_location_field: EV_COMBO_BOX
			-- New non-editable combo box with
			-- list of possible fab locations
		local
			fact: SINGLE_CONSTRAINED_VALUE_FACTORY
			fs: FIELD_SETS_ROW
			fab_loc, default_choice: STRING
		do
			create fact
			fact.disable_choices_prepended
			from
				related_field_sets.start
			until
				related_field_sets.after
			loop
				fs := related_field_sets.item
				if not fs.is_null_fab_location then
			 		fab_loc := fs.fab_location
					if not fact.choices.has (fab_loc) then
						fact.add_choice_and_text (fab_loc, fab_loc)
					end
				end
				related_field_sets.forth
			end
			add_select_one (fact)
			if is_custom and then has_existing_field_set then
				default_choice := only_field_set.fab_location
			else
				default_choice := fact.choices.first
			end
			fact.set_default_choice (default_choice)
			fact.generate_combo_box
			Result := fact.combo_box
			Result.disable_edit
		ensure
			result_not_void: Result /= Void
		end

	new_custom_wafer_type_field: EV_COMBO_BOX
			-- New non-editable combo box with
			-- list of possible wafer types
		local
			fact: SINGLE_CONSTRAINED_VALUE_FACTORY
			ls: LITHO_SIZES_ROW
			waf_type, default_choice: STRING
		do
			create fact
			fact.disable_choices_prepended
			from
				related_field_sets.start
			until
				related_field_sets.after
			loop
				ls := related_field_sets.item.litho_size
				if not ls.is_null_wafer_type_id then
			 		waf_type := ls.wafer_type_id
					if not fact.choices.has (waf_type) then
						fact.add_choice_and_text (waf_type, waf_type)
					end
				end
				related_field_sets.forth
			end
			add_select_one (fact)
			if is_custom and then has_existing_field_set then
				default_choice := only_field_set.wafer_type_id
			else
				default_choice := fact.choices.first
			end
			fact.set_default_choice (default_choice)
			fact.generate_combo_box
			Result := fact.combo_box
			Result.disable_edit
		ensure
			result_not_void: Result /= Void
		end

	new_custom_mask_size_field: EV_COMBO_BOX
			-- New non-editable combo box with
			-- list of possible mask sizes
		local
			fact: SINGLE_CONSTRAINED_VALUE_FACTORY
			ls: LITHO_SIZES_ROW
			mask_size, default_choice: STRING
		do
			create fact
			fact.disable_choices_prepended
			from
				related_field_sets.start
			until
				related_field_sets.after
			loop
				ls := related_field_sets.item.litho_size
				if not ls.is_null_mask_size_id then
			 		mask_size := ls.mask_size_id
					if not fact.choices.has (mask_size) then
						fact.add_choice_and_text (mask_size, mask_size)
					end
				end
				related_field_sets.forth
			end
			add_select_one (fact)
			if is_custom and then has_existing_field_set then
				default_choice := only_field_set.mask_size_id
			else
				default_choice := fact.choices.first
			end
			fact.set_default_choice (default_choice)
			fact.generate_combo_box
			Result := fact.combo_box
			Result.disable_edit
		ensure
			result_not_void: Result /= Void
		end

	new_custom_flat_location_field: EV_COMBO_BOX
			-- New non-editable combo box with
			-- list of possible primary flat location
		local
			fact: SINGLE_CONSTRAINED_VALUE_FACTORY
			ls: LITHO_SIZES_ROW
			flat_loc, default_choice: STRING
		do
			create fact
			fact.disable_choices_prepended
			from
				related_field_sets.start
			until
				related_field_sets.after
			loop
				ls := related_field_sets.item.litho_size
				if not ls.is_null_primary_flat_location then
			 		flat_loc := ls.primary_flat_location
					if not fact.choices.has (flat_loc) then
						fact.add_choice_and_text (flat_loc, flat_loc)
					end
				end
				related_field_sets.forth
			end
			add_select_one (fact)
			if is_custom and then has_existing_field_set then
				default_choice := only_field_set.primary_flat_location
			else
				default_choice := fact.choices.first
			end
			fact.set_default_choice (default_choice)
			fact.generate_combo_box
			Result := fact.combo_box
			Result.disable_edit
		ensure
			result_not_void: Result /= Void
		end

	new_custom_tweezer_margin_field: EV_COMBO_BOX
			-- New editable combo box with
			-- list of possible tweezer margins
		local
			fact: SINGLE_CONSTRAINED_VALUE_FACTORY
			ls: LITHO_SIZES_ROW
			margin, default_choice: STRING
		do
			create fact
			fact.disable_choices_prepended
			from
				related_field_sets.start
			until
				related_field_sets.after
			loop
				ls := related_field_sets.item.litho_size
				if not ls.is_null_tweezer_margin then
			 		margin := formatted_number (ls.tweezer_margin)
					if not fact.choices.has (margin) then
						fact.add_choice_and_text (margin, margin)
					end
				end
				related_field_sets.forth
			end
			if is_custom and then has_existing_field_set then
				default_choice := formatted_number (only_field_set.tweezer_margin)
			else
				default_choice := fact.choices.first
			end
			fact.set_default_choice (default_choice)
			fact.generate_combo_box
			Result := fact.combo_box
			Result.enable_edit
		ensure
			result_not_void: Result /= Void
		end

	process_field_set_change
			-- Process change of field set
		local
			fs_name: STRING
			fs: FIELD_SETS_ROW
			ls: LITHO_SIZES_ROW
		do
			fs_name := field_data (field_set_field)
			fs := field_set_table.item (fs_name)
			ls := fs.litho_size
			if payload_size_x_field /= Void then
				set_text (payload_size_x_field, formatted_number (ls.payload_size_x))
			end
			if payload_size_y_field /= Void then
				set_text (payload_size_y_field, formatted_number (ls.payload_size_y))
			end
			if external_kerf_size_x_field /= Void then
				set_text (external_kerf_size_x_field, formatted_number (ls.external_kerf_x))
			end
			if external_kerf_size_y_field /= Void then
				set_text (external_kerf_size_y_field, formatted_number (ls.external_kerf_y))
			end
			if field_size_x_field /= Void then
				set_text (field_size_x_field, formatted_number (ls.field_size_x))
			end
			if field_size_y_field /= Void then
				set_text (field_size_y_field, formatted_number (ls.field_size_y))
			end
			if renaming_style_field /= Void then
				set_text (renaming_style_field, ls.pattern_file_renaming_style)
			end
			if fab_location_field /= Void then
				set_text (fab_location_field, fs.fab_location)
			end
			if wafer_type_field /= Void then
				set_text (wafer_type_field, ls.wafer_type_id)
			end
			if mask_size_field /= Void then
				set_text (mask_size_field, ls.mask_size_id)
			end
			if tweezer_margin_field /= Void then
				set_text (tweezer_margin_field, formatted_number (ls.tweezer_margin))
			end
			if primary_flat_field /= Void then
				set_text (primary_flat_field, ls.primary_flat_location)
			end
		end

	add_select_one (fact: SINGLE_CONSTRAINED_VALUE_FACTORY)
		do
			if fact.choices.count > 1 then
				fact.prepend_choice_and_text ("", Select_one_text)
			end
		end

	field_data (fld: EV_TEXT_FIELD): STRING
			-- Data associated with currently selected item
			-- of `fld'
		local
			l_combo_box: EV_COMBO_BOX
		do
			l_combo_box ?= fld
			if l_combo_box = Void then
				handle_internal_error ("text field should be a combo box but is not")
			end
			Result ?= l_combo_box.selected_item.data
			if Result = Void then
				handle_internal_error ("no data associated with currently selected item of combo box")
			end
		end

	validate_field (a_fld: EV_TEXT_FIELD fld_name: STRING)
			-- Verify that selected value for `fld' is not
			-- empty and add validation error if not
		local
			val: STRING
		do
			val := field_data (a_fld)
			if val.is_empty then
				add_validation_error (fld_name, "value not selected")
			end
		end

	new_or_existing_litho_size: LITHO_SIZES_MROW
			-- Existing litho sizes row or new one
			-- to be inserted with many initialized or
			-- updated
		local
			val: STRING
			num_val: DOUBLE
		do
			if has_existing_field_set then
				Result := only_field_set.litho_size
			else
				create Result.make_null (session.connection)
				Result.set_litho_size_id (standard_name)
				Result.set_packing_target (target_name)
				Result.set_run_id (session.run_id)
			end

			val := field_data (renaming_style_field)
			if val.is_equal (Null_choice) then
				if not Result.is_null_pattern_file_renaming_style then
					Result.set_null_pattern_file_renaming_style
				end
			elseif not Result.pattern_file_renaming_style.is_equal (val) then
				Result.set_pattern_file_renaming_style (val)
			end

			num_val := payload_size_x_field.text.to_double
			if Result.is_null_payload_size_x or else Result.payload_size_x /= num_val then
				Result.set_payload_size_x (num_val)
			end
			num_val := payload_size_y_field.text.to_double
			if Result.is_null_payload_size_y or else Result.payload_size_y /= num_val then
				Result.set_payload_size_y (num_val)
			end

			if Result.is_null_payload_location_x or else Result.payload_location_x /= 0.0 then
				Result.set_payload_location_x (0.0)
			end
			if Result.is_null_payload_location_y or else Result.payload_location_y /= 0.0 then
				Result.set_payload_location_y (0.0)
			end

			if Result.is_null_merged_pattern_file_location_x or else Result.merged_pattern_file_location_x /= 0.0 then
				Result.set_merged_pattern_file_location_x (0.0)
			end
			if Result.is_null_merged_pattern_file_location_y or else Result.merged_pattern_file_location_y /= 0.0 then
				Result.set_merged_pattern_file_location_y (0.0)
			end

			num_val := payload_size_x_field.text.to_double * session.cache.scale_factor
			if Result.is_null_merged_pattern_file_size_x or else Result.merged_pattern_file_size_x /= num_val then
				Result.set_merged_pattern_file_size_x (num_val)
			end
			num_val := payload_size_y_field.text.to_double * session.cache.scale_factor
			if Result.is_null_merged_pattern_file_size_y or else Result.merged_pattern_file_size_y /= num_val then
				Result.set_merged_pattern_file_size_y (num_val)
			end
		end

	new_or_existing_field_set: FIELD_SETS_MROW
			-- Existing field sets row or new one
			-- to be inserted with most fields initialized
			-- or updated
		local
			val: STRING
		do
			if has_existing_field_set then
				Result := only_field_set
			else
				create Result.make_null (session.connection)
				Result.set_field_set_name (standard_name)
				Result.set_litho_size_id (standard_name)
			end
			val := field_data (fab_location_field)
			if not Result.fab_location.is_equal (val) then
				Result.set_fab_location (val)
			end
		end

	new_or_existing_field_location (ls: LITHO_SIZES_ROW msize_id: STRING): FIELD_LOCATIONS_MROW
			-- Existing field locations row or new one
			-- to be inserted with all values initialized
			-- or updated
		local
			num_val: DOUBLE
			l_name: STRING
			msizes_row: MASK_SIZES_ROW
		do
			if has_existing_field_set then
				create Result.make_by_key (session.connection, option.Old_no_exception_row, only_field_set.field_set_name, Default_field_name, Default_copy_no)
			end
			if Result = Void or else Result.is_empty then
				create Result.make_null (session.connection)
				if has_existing_field_set then
					l_name := only_field_set.field_set_name
				else
					l_name := standard_name
				end
				Result.set_field_set_name (l_name)
				Result.set_field_name (Default_field_name)
				Result.set_copy_no (Default_copy_no)
			end

			create msizes_row.make_template
			msizes_row := msizes_row.mask_sizes_with_name (msize_id, session.connection, expected_count.Exactly_one).first
			num_val := msizes_row.mask_size_x / 2.0
			if Result.is_null_location_x or else Result.location_x /= num_val then
				Result.set_location_x (num_val)
			end

			num_val := msizes_row.mask_size_y / 2.0
			if Result.is_null_location_y or else Result.location_y /= num_val then
				Result.set_location_y (num_val)
			end
		end

	field_set_table: HASH_TABLE [FIELD_SETS_ROW, STRING]
			-- Table of field set rows, indexed by field
			-- set name

	cached_target_specific_box: like target_specific_box
			-- Box with packing target specific contents,
			-- to be displayed in interface

	grid: GRID
			-- Grid used for rounding and converting
			-- numbers

	Pattern_file_renaming_style_text: STRING = "Pattern File Renaming Style"

	Fab_location_text: STRING = "Fab Location"

	Wafer_type_text: STRING = "Wafer Type"

	Mask_size_text: STRING = "Mask Size"

	Primary_flat_location_text: STRING = "Primary Flat Location"

	Tweezer_margin_text: STRING = "Tweezer Margin"

	Select_one_text: STRING = "(Select One)"

	None_text: STRING = "None"

	Null_choice: STRING = "NULL"

	Field_set_width: INTEGER = 20

	Size_width: INTEGER = 10

	Renaming_style_width: INTEGER = 20

	Fab_location_width: INTEGER = 20

	Wafer_type_width: INTEGER = 15

	Mask_size_width: INTEGER = 15

	Tweezer_margin_width: INTEGER = 10

	Primary_flat_width: INTEGER = 15

	Not_applicable_width: INTEGER = 15

	Default_field_name: STRING = "A"

	Default_copy_no: INTEGER = 1

invariant
	grid_not_void: grid /= Void
	packing_targets_row_not_void: packing_targets_row /= Void
	all_packing_targets_not_void: all_packing_targets /= Void
	run_id_not_void: run_id /= Void
	related_field_sets_not_void: related_field_sets /= Void
	field_sets_not_void: field_sets /= Void
	field_set_table_not_void: field_set_table /= Void

end
