
indexing

	author: "David Hollenberg"

class PACKING_VERSIONS_MROW
inherit
	PACKING_VERSIONS_MRAW
		redefine
			detached_twin, copy_from_row
		end
	PACKING_VERSIONS_ROW
		undefine
			initialize_new,
			initialize_new_by_key
		redefine
			anchored_die, anchored_wafer_process, 
			anchored_die_location,
			anchored_cut_pattern, anchored_cut_die_site,
			anchored_project, 
			dies, set_dies, cut_patterns, set_cut_patterns,
			wafer_processes, set_wafer_processes
		end
	DB_DETACHABLE_ROW
		export
			{PACKING_VERSIONS_MROW} detach_actions_internal
		redefine
			detached_twin, copy_from_row
		end
	CONDITION_UTILITIES
		export
			{NONE} all
		end
	STRING_UTILITIES_ACCESS
		export
			{NONE} all
		end

create
	make_template, make_null, make_by_key, make_detached,
	make_initial_version

feature {NONE} -- Creation

	make_initial_version (p: like anchored_packing conn: DB_CONNECTION) is
			-- Create initial version for packing `p' on database
			-- connection `conn'
		require
			p_not_void: p /= Void
			conn_not_void: conn /= Void
		do
			make_detached (conn)
			set_packing_id (p.packing_id)
			set_version_no (0)
			set_null_parent_version_no
			set_date_created (conn.system_date)
			set_creator (conn.user_name)
			set_version_id ("Initial version")
			set_packing (p)
			set_cut_patterns (create {ARRAYED_LIST [like anchored_cut_pattern]}.make (0))
			set_wafer_processes (create {ARRAYED_LIST [like anchored_wafer_process]}.make (0))
			select_internal_vendor_geometry
		end

feature -- Properties

	dies: LIST [like anchored_die] is
			-- Dies for `Current'
		do
			if is_detached then
				Result := detached_dies
			else
				if cached_dies = Void then
					fetch_dies
				end
				Result := cached_dies
			end
		end

	cut_patterns: LIST [like anchored_cut_pattern] is
			-- Cut patterns for `Current'
		do
			if is_detached then
				Result := detached_cut_patterns
			else
				if cached_cut_patterns = Void then
					fetch_cut_patterns
				end
				Result := cached_cut_patterns
			end
		end

	wafer_processes: LIST [like anchored_wafer_process] is
			-- Wafer processes for `Current'
		do
			if is_detached then
				Result := detached_wafer_processes
			else
				if cached_wafer_processes = Void then
					fetch_wafer_processes
				end
				Result := cached_wafer_processes
			end
		end

	detached_twin: like Current is
			-- New object that is the same as `Current' but
			-- is detached from any database connection
		do
			Result := precursor {PACKING_VERSIONS_MRAW}
			call_detach_actions (Result)
		end

	deep_detached_twin: like Current is
			-- Detached copy of `Current' with all
			-- dependent rows detached as well
		do
			Result := detached_twin
			Result.set_dies (dies.twin)
			Result.set_wafer_processes (wafer_processes.twin)
			Result.set_cut_patterns (cut_patterns.twin)
			Result.uncache_dependents
				-- Just in case: shouldn't be referenced
			Result.detach_dependents_deep
			Result.connect_die_sites_to_locations
		end

	is_deep_unchanged: BOOLEAN is
			-- Are `Current' and all its dependent rows
			-- unchanged from their original versions?
			-- Returns False if no original version
		local
			dies_pair: DB_ROW_SET_PAIR [like anchored_die]
			wp_pair: DB_ROW_SET_PAIR [like anchored_wafer_process]
			cp_pair: DB_ROW_SET_PAIR [like anchored_cut_pattern]
		do
			Result := all_columns_equal_original
				and then dies.for_all (agent {like anchored_die}.is_deep_unchanged)
				and then wafer_processes.for_all (agent {like anchored_wafer_process}.is_deep_unchanged)
				and then cut_patterns.for_all (agent {like anchored_cut_pattern}.is_deep_unchanged)
			if Result then
				create dies_pair.make_from_lists (dies, original_dies, create {KEY_COLS_EQUALITY_TESTER [like anchored_die]})
				Result := dies_pair.is_identical
			end
			if Result then
				create wp_pair.make_from_lists (wafer_processes, original_wafer_processes, create {KEY_COLS_EQUALITY_TESTER [like anchored_wafer_process]})
				Result := wp_pair.is_identical
			end
			if Result then
				create cp_pair.make_from_lists (cut_patterns, original_cut_patterns, create {KEY_COLS_EQUALITY_TESTER [like anchored_cut_pattern]})
				Result := cp_pair.is_identical
			end
		end

	new_cust_die (m_fab_no: INTEGER): like anchored_die is
			-- New cust die for fab instance identified by
			-- by `m_fab_no', with selection and synchronization
			-- dates set to `selection_date' (must set it before
			-- this routine is called)
		local
			type: DIE_TYPE_VALUES
			proj: like anchored_project
		do
			proj := new_cust_die_project (m_fab_no)
			Result := new_initialized_die (type.Cust_die)
			Result.set_attributes_from_project (proj, packing)
		end

	new_test_vehicle_die (design_no: INTEGER): like anchored_die is
			-- New test vehicle die for test vehicle identified
			-- by `design_no', with selection and synchronization
			-- dates set to `selection_date' (must set it before
			-- this routine is called)
		local
			type: DIE_TYPE_VALUES
			proj: like anchored_project
		do
			proj := new_test_vehicle_project (design_no)
			Result := new_initialized_die (type.Test_vehicle)
			Result.set_attributes_from_project (proj, packing)
		end

	new_horizontal_scribe: like anchored_die is
			-- New scribe die which is horizontal, without
			-- size or locations
		local
			orient: ORIENTATION_VALUES
		do
			Result := new_scribe_with_orientation (orient.Horizontal)
		end
		
	new_vertical_scribe: like anchored_die is
			-- New scribe die which is vertical, without
			-- size or locations
		local
			orient: ORIENTATION_VALUES
		do
			Result := new_scribe_with_orientation (orient.Vertical)
		end
		
	new_scribe_with_orientation (o: STRING): like anchored_die is
			-- New scribe with orientation `o', without
			-- size or locations
		require
			orientation_not_void: o /= Void
			orientation_valid: is_valid_orientation (o)
		local
			type: DIE_TYPE_VALUES
		do
			Result := new_initialized_die (type.Scribe)
			Result.set_scribe_orientation (o)
		end
		
	is_valid_orientation (s: STRING): BOOLEAN is
			-- Is `s' a valid orientation?
		require
			s_not_void: s /= Void
		local
			orient: ORIENTATION_VALUES
		do
			Result := s.is_equal (orient.Horizontal) or s.is_equal (orient.Vertical)
		end

	new_filler_with_name (s: STRING): like anchored_die is
			-- New filler die with name `s'
		require
			s_not_void: s /= Void
		local
			type: DIE_TYPE_VALUES
		do
			Result := new_initialized_die (type.Filler)
			Result.set_filler_name (s)
		end
		
	new_cust_die_project (m_fab_no: INTEGER): like anchored_project is
			-- New cust die project for project identified
			-- by `m_fab_no', with selection and synchronization
			-- dates set to `selection_date' (must set it before
			-- this routine is called).  Die points to 
			-- project
		local
			opts: LIST [like anchored_option]
			opt: like anchored_option
			explicit_opts: LIST [IC_FAB_INSTANCE_OPTIONS_ROW]
			list: LIST [WAFER_PROCESS_OPTION]
		do
			create Result.make_cust_project (m_fab_no, selection_date, any_connection)
			set_new_project_key (Result, parent.cust_project_highest_version (m_fab_no))
			

			Result.set_packagings (Result.new_packagings)

			create {ARRAYED_LIST [like anchored_option]} opts.make (5)
			explicit_opts := Result.fab_instance.explicit_options
			from
				explicit_opts.start
			until
				explicit_opts.after
			loop
				create opt.make_from_explicit_option (Result, explicit_opts.item)
				opts.extend (opt)
				explicit_opts.forth
			end
			
			Result.fetch_layer_options (packing.wafer_techn)
			from
				list := Result.computed_layer_options
				list.start
			until
				list.after
			loop
				create opt.make_from_layer_option (Result, list.item)
				opts.extend (opt)
				list.forth
			end

			Result.set_options (opts)
			
		end

	new_test_vehicle_project (design_no: INTEGER): like anchored_project is
			-- New test vehicle project for test vehicle identified
			-- by `design_no', with selection and synchronization
			-- dates set to `selection_date' (must set it before
			-- this routine is called).  Die points to 
			-- project
		local
			tv: M_TEST_VEHICLES_ROW
			pkging_list: ARRAYED_LIST [like anchored_packaging]
			pkging: like anchored_packaging
			pkg_id: STRING
		do
			create Result.make_test_vehicle (design_no, selection_date, any_connection)
			set_new_project_key (Result, parent.test_vehicle_highest_version (design_no))
			
			-- Don't have max_allowed_die_size_x and y values
			-- since no fab instance for test vehicle yet.
			-- These are user preference values, not limits
			-- dictated by packaging constraints
		
			-- Don't set project_state or run_id - not meaningful 
			-- for test vehicle
			
			-- Don't set quantity_requested or
			-- quantity_unpackaged_requested - don't have
			-- this info for test vehicle (and may not apply)
			
			tv := Result.test_vehicle
			
			if not tv.is_null_design_wafer_rotation_reqd then
				Result.set_design_wafer_rotation_reqd (tv.design_wafer_rotation_reqd)
			end

			create pkging_list.make (1)
			create pkging.make_detached (any_connection)
			pkging.set_key_from_project (Result)
			pkging.set_line_no (1)
			pkg_id := tv.package_id
			pkging.set_package_id (pkg_id)
			pkging.set_package_requested (pkg_id)
			if not pkging.is_null_package_id then
				pkging.set_packaged (True)
			else
				pkging.set_null_packaged
			end
			pkging_list.extend (pkging)
			Result.set_packagings (pkging_list)
			Result.set_options (create {ARRAYED_LIST [PACKING_PROJECT_OPTIONS_MROW]}.make (0))
		end

	ordered_processes: LIST [STRING]
			-- Names of processes to be ordered computed by
			-- last call to `assign_process_to_projects'

	added_projects: LIST [like anchored_project]
			-- Projects added by last call to 
			-- `add_dies_for_projects'

	added_dies: LIST [like anchored_die]
			-- Dies added by last call to 
			-- `add_dies_for_projects'

	removed_dies: LIST [like anchored_die]
			-- Dies removed by last call to 
			-- `remove_dies_for_projects'

	last_cut_pattern_position: INTEGER
			-- Position of last added cut pattern in cut
			-- pattern list

feature -- Row copy

	copy_from_row (r: like original_row) is
			-- Initialize `Current' from `r'
		do
			precursor {PACKING_VERSIONS_MRAW} (r)
			detach_actions_internal := r.detach_actions_internal
			cached_dies := r.cached_dies
			cached_cut_patterns := r.cached_cut_patterns
			cached_packing := r.cached_packing
			cached_wafer_processes := r.cached_wafer_processes
			die_locations_table := r.die_locations_table
			cust_die_table := r.cust_die_table
			test_vehicle_table := r.test_vehicle_table
			vendor_geometry_table := r.vendor_geometry_table
			selection_date := r.selection_date
			max_die_no := r.max_die_no
			max_cut_pattern_no := r.max_cut_pattern_no
			ordered_processes := r.ordered_processes
			warnings := r.warnings
			cached_family_processes_table := r.cached_family_processes_table
		end

feature -- Modification

	set_dies (list: like dies) is
			-- Set `dies' to `list'
		do
			if is_detached then
				detached_dies := list
			else
				precursor (list)
			end
		end

	set_cut_patterns (list: like cut_patterns) is
			-- Set `cut_patterns' to `list'
		do
			if is_detached then
				detached_cut_patterns := list
			else
				precursor (list)
			end
		end

	set_wafer_processes (list: like wafer_processes) is
			-- Set `wafer_processes' to `list'
		do
			if is_detached then
				detached_wafer_processes := list
			else
				precursor (list)
			end
		end

feature -- Operations

	select_internal_vendor_geometry is
			-- Select vendor geometry which is internal to
			-- payload and add appropriate dependent rows
		local
			row: GEOMETRIES_ROW
			list: LIST [GEOMETRIES_ROW]
		do
			create row.make_template
			list := row.payload_rows (packing.wafer_techn, packing.field_set_name, any_connection, expected_count.At_least_zero)
			if dies = Void then
				set_dies (create {ARRAYED_LIST [like anchored_die]}.make (list.count))
			end
			max_die_no := 0
			list.do_all (agent append_vendor_geometry (?, dies))
		end

	select_external_vendor_geometry is
			-- Select vendor geometry which is external to
			-- payload and add appropriate dependent rows
		local
			row: GEOMETRIES_ROW
			list: LIST [GEOMETRIES_ROW]
		do
			create row.make_template
			list := row.nonpayload_rows (packing.wafer_techn, packing.field_set_name, any_connection, expected_count.At_least_zero)
			if dies = Void then
				set_dies (create {ARRAYED_LIST [like anchored_die]}.make (list.count))
			end
			list.do_all (agent append_vendor_geometry (?, dies))
		end

	register_deep is
			-- Register `Current' and all dependent rows
			-- on implicit connection
		do
			register_on_implicit_connection
			register_dependents_deep
		end

	set_highest_used_numbers is
			-- Set highest used numbers for die, die
			-- locations etc so that allocating a new
			-- entity will pick a number higher than any
			-- previously used number
		do
			set_highest_used_die_no
			set_highest_used_cut_pattern_no
			cut_patterns.do_all (agent {like anchored_cut_pattern}.set_highest_used_cut_no)
		end

	update_for_new_version (a_name: STRING) is
			-- Update `Current' with information for new
			-- version with optional version ID `a_name'
		local
			n: INTEGER
		do
			n := highest_used_version + 1
			if version_no /= 0 then
				set_parent_version_no (version_no)
			else
				set_null_parent_version_no
			end
			set_version_no (n)
			set_date_created (any_connection.transaction_date)
			set_creator (any_connection.user_name)
			set_version_id (a_name)
			if not is_null_version_id then
				set_packing_name (packing_id)
			else
				set_null_packing_name
			end
			update_dependents_for_version (n)
		end

	set_selection_date is
			-- Set selection/synchronization date so that
			-- a whole batch of newly created or updated
			-- projects can get the exact same date
		do
			selection_date := any_connection.system_date
		end

	update_dies_for_projects (list: LIST [PACKING_PROJECTS_ROW]) is
			-- Remove from `dies' all projects in `list'.
			-- Then, add a new die for each project in
			-- list.  This effectively updates information
			-- for each one
		require
			list_not_void: list /= Void
		do
			remove_dies_for_projects (list)
			add_dies_for_projects (list)
		end

	add_dies_for_projects (sel_list: LIST [PACKING_PROJECTS_ROW]) is
			-- Add a die to `dies' for every project in `sel_list'.
			-- Leave list of new projects in `added_projects'
			-- and list of new dies in `added_dies'
		require
			sel_list_not_void: sel_list /= Void
		do
			create {ARRAYED_LIST [like anchored_project]} added_projects.make (sel_list.count)
			create {ARRAYED_LIST [like anchored_die]} added_dies.make (sel_list.count)
			set_selection_date
			parent.setup_for_fast_project_access
			sel_list.do_all (agent add_die_for_project)
			selection_date := Void
		end

	add_die_for_project (p: PACKING_PROJECTS_ROW) is
			-- Add a die to `dies' for `p'
		require
			p_not_void: p /= Void
		local
			die: like anchored_die
		do
			if p.is_customer_project then
				die := new_cust_die (p.m_fab_no)
			else
				die := new_test_vehicle_die (p.design_no)
			end
			dies.extend (die)
			added_dies.extend (die)
			added_projects.extend (die.project)
		end

	remove_dies_for_projects (del_list: LIST [PACKING_PROJECTS_ROW]) is
			-- Remove from `dies' all projects in `del_list'.
			-- Leave list of corresponding dies in `removed_dies'
		require
			del_list_not_void: del_list /= Void
		local
			old_list, new_list: like dies
			die: like anchored_die
			proj_table: HASH_TABLE [INTEGER, INTEGER]
			die_table: HASH_TABLE [INTEGER, INTEGER]
			pno: INTEGER
		do
			create {ARRAYED_LIST [like anchored_die]} removed_dies.make (del_list.count)
			create proj_table.make (del_list.count)
			del_list.do_all (agent add_project_to_table (?, proj_table))
			create die_table.make (del_list.count)

			from
				old_list := dies
				new_list := old_list.twin
				new_list.wipe_out
				old_list.start
			until
				old_list.after
			loop
				die := old_list.item
				if die.has_project then
					pno := die.project_no
					proj_table.search (pno)
					if not proj_table.found then
						new_list.extend (die)
					else
						removed_dies.extend (die)
						die_table.put (1, die.die_no)
					end
				else
					new_list.extend (die)
				end
				old_list.forth
			end
			set_dies (new_list)
			cut_patterns.do_all (agent {like anchored_cut_pattern}.remove_dies_for_projects (die_table))
		end

	remove_dies (del_list: LIST [PACKING_DIES_ROW]) is
			-- Remove from `dies' all dies in `del_list'
		require
			del_list_not_void: del_list /= Void
		local
			old_list, new_list: like dies
			die: like anchored_die
			die_table: HASH_TABLE [INTEGER, INTEGER]
			die_no: INTEGER
		do
			create die_table.make (del_list.count)
			del_list.do_all (agent add_die_to_table (?, die_table))

			from
				old_list := dies
				new_list := old_list.twin
				new_list.wipe_out
				old_list.start
			until
				old_list.after
			loop
				die := old_list.item
				die_no := die.die_no
				die_table.search (die_no)
				if not die_table.found then
					new_list.extend (die)
				end
				old_list.forth
			end
			set_dies (new_list)
			cut_patterns.do_all (agent {like anchored_cut_pattern}.remove_dies_for_projects (die_table))
		end

	remove_dies_for_packer_box_update (pb_list: PACKER_BOX_LIST) is
			-- Remove some dies from `Current' as part of
			-- updating it to make it match `pb_list'.
			-- Remove all scribes and fillers (new dies
			-- will be added later).  Remove cust dies and
			-- test vehicles that have no corresponding
			-- packer box in `pb_list' (were deleted by
			-- run closer during packing).  Signal error
			-- for any vendor geometries/ghosts in
			-- `pb_list' that do not have a corresponding
			-- packer box
		local
			curr_dies, new_dies: LIST [like anchored_die]
			die: like anchored_die
			pb: PACKER_BOX
		do
			curr_dies := dies
			new_dies := curr_dies.twin
			new_dies.wipe_out
			from
				curr_dies.start
			until
				curr_dies.after
			loop
				die := curr_dies.item
				if die.is_scribe or die.is_filler then
					-- Don't add to list
					pb := Void
				elseif die.is_cust_die then
					pb := pb_list.cust_die (die.project.m_fab_no)
				elseif die.is_test_vehicle then
					pb := pb_list.test_vehicle (die.project.design_no)
				elseif die.is_vendor_geometry_or_ghost then
					pb := pb_list.vendor_geometry (die.geometry_id)
					if pb = Void then
						handle_hard_error ("vendor geometry " + die.geometry_id + " should be in packer box list but is not")
					end
				else
					handle_hard_error ("unknown die type " + die.die_type)
					pb := Void
				end
				if pb /= Void then
					new_dies.extend (die)
				end
				dies.forth
			end
			set_dies (new_dies)
		end
	
	update_from_packer_boxes (pb_list: PACKER_BOX_LIST) is
			-- Update `Current' from information
			-- in `pb_list' that is not read-only, using
			-- `wt' as the wafer techn
		do
			setup_for_fast_access
			parent.setup_for_fast_project_access
			set_selection_date
			from
				pb_list.start
			until
				pb_list.after
			loop
				update_from_packer_box (pb_list.item)
				pb_list.forth
			end
			selection_date := Void
				-- Make sure it isn't accidentally used again
		end
	
	add_new_die (a_die: like anchored_die) is
			-- Add `a_die' to `dies'
		require
			a_die_not_void: a_die /= Void
			same_version_or_zero: a_die.version_no = version_no or a_die.version_no = 0
		do
			dies.extend (a_die)
		end

	compute_compatible_processes is
			-- Compute compatible processes for each selected
			-- customer project in `Current'
		do
			compute_compatible_processes_for_list (selected_customer_projects)
		end
	
	compute_compatible_processes_for_list (list: like selected_customer_projects) is
			-- Compute compatible processes for each 
			-- project in `list'
		do
			uncache_family_processes_table
				-- Make sure any added processes will be found
			compute_compatible_processes_for_projects (list, family_processes_table)
		end
	
	assign_process_to_projects is
			-- Assign a process to each customer project,
			-- based on the list of selected projects and
			-- the compatible processes for each of these
			-- projects.  Then propagate the assigned process
			-- to the die for each project.  Leave any
			-- warnings in `warnings'
		local
			die_list: like selected_customer_dies
			proj_list: like selected_customer_projects
		do
			uncache_family_processes_table
				-- Make sure any added processes will be found
			die_list := selected_customer_dies
			proj_list := selected_customer_projects
			die_list.do_all (agent {like anchored_die}.propagate_assigned_etest_name (family_processes_table))
			compute_ordered_processes (proj_list)
			
			proj_list.do_all (agent {like anchored_project}.assign_process (ordered_processes))
			dies.do_if (agent {like anchored_die}.assign_wafer_process, agent {like anchored_die}.is_cust_die)
			dies.do_if (agent {like anchored_die}.assign_wafer_etest_name, agent {like anchored_die}.is_cust_die)
			
			check_ordered_processes (family_processes_table)
		end
	
	report_process_warnings is
			-- Report any warnings from last call to
			-- `assign_process_to_projects'
		local
			op: ANONYMOUS_OPERATION
		do
			if not warnings.is_empty then
				create op.make (agent warn_with_notify ("multiple ordered processes with same wafer etest ID - missing process or conceptual error.%N%N" + su.joined_by (warnings, "%N") + "%N"))
				op.enable_abort_caught
				op.execute
			end
		end

	add_cut_pattern (a_pattern_no: INTEGER a_field_name, a_wafer_etest_name: STRING quant: INTEGER has_quant: BOOLEAN)
			-- Add a new cut pattern `a_pattern_no' for field
			-- `a_field_name' and lot `a_wafer_etest_name'.
			-- Leave position of new cut pattern in 
			-- `last_cut_pattern_position'
		require
			a_field_name_not_void: a_field_name /= Void
			a_field_name_not_empty: not a_field_name.is_empty
			a_wafer_etest_name_not_void: a_wafer_etest_name /= Void
		local
			pat: PACKING_CUT_PATTERNS_MROW
		do
			create pat.make_detached (any_connection)
			pat.set_grid (grid)
			pat.set_packing_id (packing_id)
			pat.set_version_no (version_no)
			pat.set_cut_pattern_no (a_pattern_no)
			pat.set_field_name (a_field_name)
			pat.set_is_completed (False)
			pat.set_wafer_etest_name (a_wafer_etest_name)
			if has_quant then
				pat.set_quantity_wafers_needed (quant)
			else
				pat.set_null_quantity_wafers_needed
			end
			pat.set_cut_locations (create {ARRAYED_LIST [PACKING_CUT_LOCATIONS_MROW]}.make (0))
			pat.set_cut_die_sites (create {ARRAYED_LIST [PACKING_CUT_DIE_SITES_MROW]}.make (0))
			cut_patterns.extend (pat)
				-- FIXME: should probably put cut pattern in
				-- correct position by cut_pattern_no
			last_cut_pattern_position := cut_patterns.count
		end

	remove_cut_pattern (a_pattern_no: INTEGER)
			-- Remove new cut pattern whose number is 
			-- `a_pattern_no'
		local
			list: like cut_patterns
			pat: PACKING_CUT_PATTERNS_MROW
			found: BOOLEAN
		do
			from
				list := cut_patterns
				list.start
			until
				list.after or found
			loop
				pat := list.item
				if pat.cut_pattern_no = a_pattern_no then
					list.remove
					found := True
				else
					list.forth
				end
			end
		end

	remove_cut_patterns
			-- Remove all cut patterns
		do
			cut_patterns.wipe_out
		end

feature {PACKING_VERSIONS_MROW} -- Implementation

	detach_dependents_deep is
			-- Detach dependent rows from database connection
			-- deeply
		do
			detach_dies_deep
			detach_wafer_processes_deep
			detach_cut_patterns_deep
			cut_patterns.do_all (agent {like anchored_cut_pattern}.uncache_isolated_die_locations)
		end

feature {NONE} -- Implementation

	set_highest_used_die_no is
			-- Compute highest used die_no and put in `max_die_no'
		do
			max_die_no := 0
			dies.do_all (agent update_max_die_no)
		end

	set_highest_used_cut_pattern_no is
			-- Compute highest used cut_pattern_no and put in 
			-- `max_cut_pattern_no'
		do
			max_cut_pattern_no := 0
			cut_patterns.do_all (agent update_max_cut_pattern_no)
		end

	compute_compatible_processes_for_projects (list: like selected_customer_projects table: like family_processes_table) is
			-- Compute list of compatible processes for each
			-- project in `list', using `table' to lookup
			-- information about processes
		do
			list.do_all (agent {like anchored_project}.compute_compatible_processes (table))
		end
	
	compute_ordered_processes (proj_list: like selected_customer_projects) is
			-- Compute processes to be ordered, where `proj_list'
			-- is list of all customer projects, and put
			-- it in `ordered_processes'
		local
			set: PROJECTS_AND_PROCESSES_SET
			sets: ARRAYED_LIST [PROJECTS_AND_PROCESSES_SET]
			proj: like anchored_project
			found: BOOLEAN
		do
			create sets.make (10)
			from
				proj_list.start
			until
				proj_list.after
			loop
				proj := proj_list.item
				found := False
				from
					sets.start
				until
					sets.after or found
				loop
					set := sets.item
					found := set.is_compatible (proj)
					sets.forth
				end
				if not found then
					create set.make (10, 20)
					set.processes.append (proj.compatible_processes_with_assigned_etest_name)
					sets.extend (set)
				end
				set.add_project (proj)
				proj_list.forth
			end
			
			create {ARRAYED_LIST [STRING]} ordered_processes.make (sets.count)
			ordered_processes.compare_objects
			sets.do_all (agent {PROJECTS_AND_PROCESSES_SET}.add_cheapest_process (ordered_processes))
		end
	
	check_ordered_processes (proc_table: like family_processes_table) is
			-- Check `ordered_processes' to verify that no two
			-- processes have the same wafer etest ID.
			-- Add a warning to `warnings' for each wafer etest 
			-- ID that corresponds to more than one ordered process
		local
			proc, etest_id: STRING
			table: HASH_TABLE [LIST [STRING], STRING]
			proc_list: LIST [STRING]
			bad_etest_ids: ARRAYED_LIST [STRING]
		do
			create {ARRAYED_LIST [STRING]} warnings.make (0)
			create table.make (10)
			create bad_etest_ids.make (0)
			from
				ordered_processes.start
			until
				ordered_processes.after
			loop
				proc := ordered_processes.item
				proc_table.search (proc)
				if proc_table.found then
					etest_id := proc_table.found_item.wafer_etest_id
					table.search (etest_id)
					if table.found then
						proc_list := table.found_item
						if proc_list.count = 1 then
							bad_etest_ids.extend (etest_id)
						end
					else
						create {ARRAYED_LIST [STRING]} proc_list.make (1)
						table.put (proc_list, etest_id)
					end
					proc_list.extend (proc)
				else
					handle_hard_error ("cannot find ordered wafer process " + proc + " in table of all processes for wafer techn family")
				end
				ordered_processes.forth
			end
			
			if not bad_etest_ids.is_empty then
				from
					bad_etest_ids.start
				until
					bad_etest_ids.after
				loop
					etest_id := bad_etest_ids.item
					proc_list := table.item (etest_id)
					warnings.extend ("Wafer etest ID " + etest_id + " shared by processes " + su.joined_by (proc_list, ", "))
					bad_etest_ids.forth
				end
			end
		end
	
	add_warning (dno: INTEGER msg: STRING) is
			-- Add warning for design number `dno' with message
			-- `msg' to `waraings'
		local
			s: STRING
		do
			s.append ("     ")
			s.append_integer (dno)
			s.append ("     ")
			s.append (msg)
			warnings.extend (s)
		end
	
	update_from_packer_box (pb: PACKER_BOX) is
			-- Update `Current' from information
			-- in `pb' that is not read-only
		local
			proj_type: STRING
			die_type: DIE_TYPE_VALUES
			die: like anchored_die
			sel_on_unix: BOOLEAN
		do
			sel_on_unix := selected_on_unix
			proj_type := pb.project_type
			if proj_type.is_equal (die_type.Cust_die) then
				die := cust_die (pb.m_fab_no)
				if die = Void then 
					if sel_on_unix then
						handle_hard_error ("cannot find cust die with m_fab_no " + pb.m_fab_no.out + " in " + packing_version_name)
					else
				 		die := new_cust_die (pb.m_fab_no)
				 		dies.extend (die)
					end
				end
			elseif proj_type.is_equal (die_type.Test_vehicle) then
				die := test_vehicle (pb.name.to_integer)
				if die = Void then 
					if sel_on_unix then
						handle_hard_error ("cannot find test vehicle with design_no " + pb.name + " in " + packing_version_name)
					else
				 		die := new_test_vehicle_die (pb.name.to_integer)
				 		dies.extend (die)
					end
				end
			elseif proj_type.is_equal (die_type.Vendor_geometry) or proj_type.is_equal (die_type.Vendor_ghost) then
				die := vendor_geometry (pb.name)
				if die = Void then
					-- Error even if selected on Unix
					-- (already synthesized when
					-- dummy VMS version created)
					handle_hard_error ("cannot find vendor geometry/ghost with geometry_id " + pb.name + " in " + packing_version_name)
				end
			elseif proj_type.is_equal (die_type.Scribe) then
				die := new_scribe_with_orientation (pb.scribe_orientation)
				die.set_size_from_packer_box (pb)
				dies.extend (die)
			elseif proj_type.is_equal (die_type.Filler) then
				die := new_filler_with_name (pb.name)
				die.set_size_from_packer_box (pb)
				dies.extend (die)
			else
				handle_hard_error ("unknown die type " + su.double_quoted(proj_type) + " in packer box")
			end
			
			die.update_common_from_packer_box (pb)
		end
	
	set_new_project_key (proj: like anchored_project old_proj: PACKING_PROJECTS_ROW) is
			-- Set key columns in `proj', using `old_proj'
			-- as the existing row if not Void
		local
			proj_no, vers_no: INTEGER
		do
			proj.set_packing_id (packing_id)
			if old_proj /= Void then
				proj_no := old_proj.project_no
				vers_no := old_proj.version_no + 1
			else
				-- Not in any version of packing - allocate
				-- new project_no
				packing.increment_highest_used_project_no
				proj_no := packing.highest_used_project_no
				vers_no := 1
			end
			proj.set_project_no (proj_no)
			proj.set_version_no (vers_no)
		end

	new_vendor_geometry (g: GEOMETRIES_ROW): like anchored_die is
			-- New vendor geometry/ghost die for `g'
		local
			locs: LIST [GEOMETRYXFIXED_LOCATIONS_ROW]
			quants: LIST [GEOMETRY_QUANTITIES_ROW]
		do
			Result := new_initialized_die (g.die_type)
			Result.set_geometry_id (g.geometry_id)
			Result.set_die_size_printed_x (g.non_null_size_x)
			Result.set_die_size_printed_y (g.non_null_size_y)
			Result.set_vendor_geometry (g)
			
			locs := g.fixed_locations (packing.field_set_name)
			quants := g.quantities (packing.field_set_name)
			if not locs.is_empty and not quants.is_empty then
				handle_hard_error ("vendor geometry " + g.geometry_id + " has both fixed locations and a quantity specification")
			elseif not locs.is_empty then
				add_vendor_geometry_locations (locs, Result)
			else
				-- Don't create any locations if just has 
				-- a quantity spec
			end
		end

	append_vendor_geometry (g: GEOMETRIES_ROW list: like dies) is
			-- Append new die for `g' to `list'
		do
			list.extend (new_vendor_geometry (g))
		end
		
	add_vendor_geometry_locations (locs: LIST [GEOMETRYXFIXED_LOCATIONS_ROW] d: like anchored_die) is
			-- Create a list of die locations based on `locs'
			-- and put it in `d'
		local
			loc: GEOMETRYXFIXED_LOCATIONS_ROW
			die_locs: ARRAYED_LIST [PACKING_DIE_LOCATIONS_MROW]
			die_loc: PACKING_DIE_LOCATIONS_MROW
			n: INTEGER
		do
			create die_locs.make (locs.count)
			from
				locs.start
			until
				locs.after
			loop
				loc := locs.item
				n := n + 1
				create die_loc.make_detached (any_connection)
				die_loc.set_packing_id (packing.packing_id)
				die_loc.set_version_no (0)
				die_loc.set_die_no (d.die_no)
				die_loc.set_site_no (n)
				die_loc.set_field_name (loc.field_name)
				-- FIXME: technically, we need to adjust 
				-- location X and Y by offset of payload from 
				-- center of field (doesn't matter for now)
				die_loc.set_location_x (loc.location_x)
				die_loc.set_location_y (loc.location_y)
				die_loc.set_fixed_vendor_geometry_packer_attributes
				die_locs.extend (die_loc)
				locs.forth
			end
			d.set_die_locations (die_locs)
		end
		
	new_initialized_die (dtype: STRING): like anchored_die is
			-- New die of type `dtype' initialized with
			-- key (except version_no, which is set when
			-- save new version) and not rotated
		do
			max_die_no := max_die_no + 1
			create Result.make_detached (any_connection)
			Result.set_packing_id (packing.packing_id)
			Result.set_version_no (0)
			Result.set_die_no (max_die_no)
			Result.set_die_type (dtype)
			Result.set_die_wafer_rotation (0)
			Result.set_die_locations (create {ARRAYED_LIST [like anchored_die_location]}.make (0))
		end
		
	detach_dies_deep is
			-- Detach `dies' from database connection, along
			-- with all dependents
		do
			original_dies := dies.twin
			original_dies.wipe_out
			original_dies.append (dies)
			dies.wipe_out
			original_dies.do_all (agent append_detached_die (?, dies))
		end

	detach_cut_patterns_deep is
			-- Detach `cut_patterns' from database connection,
			-- along with all dependents
		do
			original_cut_patterns := cut_patterns.twin
			original_cut_patterns.wipe_out
			original_cut_patterns.append (cut_patterns)
			cut_patterns.wipe_out
			original_cut_patterns.do_all (agent append_detached_cut_pattern (?, cut_patterns))
		end

	detach_wafer_processes_deep is
			-- Detach `wafer_processes' from database connection,
			-- along with all dependents
		do
			original_wafer_processes := wafer_processes.twin
			original_wafer_processes.wipe_out
			original_wafer_processes.append (wafer_processes)
			wafer_processes.wipe_out
			original_wafer_processes.do_all (agent append_detached_wafer_process (?, wafer_processes))
		end

	append_detached_die (d: like anchored_die list: like dies) is
			-- Append detached copy of `d' to `list'
		do
			list.extend (d.deep_detached_twin)
		end
		
	append_detached_wafer_process (wp: like anchored_wafer_process list: like wafer_processes) is
			-- Append detached copy of `wp' to `list'
		do
			list.extend (wp.deep_detached_twin)
		end
		
	append_detached_cut_pattern (cp: like anchored_cut_pattern list: like cut_patterns) is
			-- Append detached copy of `cp' to `list'
		do
			list.extend (cp.deep_detached_twin)
		end
		
	register_dependents_deep is
			-- Register all dependent rows on implicit
			-- connection
		do
			-- Move "detached" versions back to "cached" versions
			-- since they will no longer be detached at end
			-- of routine
			
			cached_dies := detached_dies
			detached_dies := Void
			dies.do_all (agent {like anchored_die}.register_deep)
			
			cached_wafer_processes := detached_wafer_processes
			detached_wafer_processes := Void
			wafer_processes.do_all (agent {like anchored_wafer_process}.register_deep)
			
			cached_cut_patterns := detached_cut_patterns
			detached_cut_patterns := Void
			cut_patterns.do_all (agent {like anchored_cut_pattern}.register_deep)
		end

	update_dependents_for_version (n: INTEGER) is
			-- Update all dependent rows prior to save of
			-- new version `n'
		do
			max_die_no := 0
			dies.do_all (agent renumber_die)
			dies.do_all (agent {like anchored_die}.update_for_new_version (n))
			wafer_processes.do_all (agent {like anchored_wafer_process}.update_for_new_version (n))
			cut_patterns.do_all (agent {like anchored_cut_pattern}.update_for_new_version (n))
		end

	renumber_die (d: PACKING_DIES_MROW) is
		do
			max_die_no := max_die_no + 1
			d.set_die_no (max_die_no)
		end
	
	update_max_die_no (d: PACKING_DIES_MROW) is
		do
			max_die_no := max_die_no.max (d.die_no)
		end
	
	update_max_cut_pattern_no (cp: like anchored_cut_pattern) is
		do
			max_cut_pattern_no := max_cut_pattern_no.max (cp.cut_pattern_no)
		end
	
	anchored_die: PACKING_DIES_MROW is
			-- For anchoring only
		do
		end
	
	anchored_wafer_process: PACKING_WAFER_PROCESSES_MROW is
			-- For anchoring only
		do
		end
	
	anchored_die_location: PACKING_DIE_LOCATIONS_MROW is
			-- For anchoring only
		do
		end

	anchored_cut_pattern: PACKING_CUT_PATTERNS_MROW is
			-- For anchoring only
		do
		end

	anchored_cut_die_site: PACKING_CUT_DIE_SITES_MROW is
			-- For anchoring only
		do
		end

	anchored_project: PACKING_PROJECTS_MROW is
			-- For anchoring only
		do
		end

	anchored_packaging: PACKING_PROJECT_PACKAGINGS_MROW is
			-- For anchoring only
		do
		end

	anchored_option: PACKING_PROJECT_OPTIONS_MROW is
			-- For anchoring only
		do
		end

	detached_dies: like dies
			-- List of detached dies (no explict DB connection)

	detached_cut_patterns: like cut_patterns
			-- List of detached cut patterns (no explict 
			-- DB connection)
		
	detached_wafer_processes: like wafer_processes
			-- List of detached wafer processes (no explict 
			-- DB connection)
		
	original_dies: like dies
			-- Original list of dies before detach

	original_wafer_processes: like wafer_processes
			-- Original list of wafer processes before detach

	original_cut_patterns: like cut_patterns
			-- Original list of cut patterns before detach

feature {PACKING_VERSIONS_MROW} -- Implementation

	max_die_no: INTEGER
			-- Maximum die number used so far (0 if none used)
			-- in this version

	max_cut_pattern_no: INTEGER
			-- Maximum cut pattern number assigned so far
			-- (0 if none used) in this version

	warnings: LIST [STRING]
			-- Warnings from last call to 
			-- `assign_process_to_projects'

end
