
indexing

	author: "David Hollenberg"

class PACKINGS_ROW
inherit
	PACKINGS_RAW
	PACKING_QUERIES
	ROW_COUNT_CHECK_ACCESS
	ITEM_COUNT_RANGES_ACCESS
	NULL_VALUE_CHECK_ACCESS
		export
			{NONE} all
		end
	DB_OPTIONS_ACCESS
		export
			{NONE} all
		end
create
	make_template, make_null, make_by_key

feature -- Collections

	rows_matching_criteria (a_creator, a_run_type, a_wafer_techn, a_run_id: STRING ord: STRING conn: DB_CONNECTION range: ITEM_COUNT_RANGE): LIST [like Current] is
			-- Packings matching `a_creator', `a_run_type',
			-- `a_wafer_techn' and `a_run_id', ordered by `ord'.
			-- Empty values for these arguments mean "do not
			-- use as a filtering criterion"
		require
			a_creator_not_void: a_creator /= Void
			a_run_type_not_void: a_run_type /= Void
			a_wafer_techn_not_void: a_wafer_techn /= Void
			a_run_id_not_void: a_run_id /= Void
			connection_exists: conn /= Void
			range_exists: range /= Void
		local
			b: DB_BIND_SSSS
		do
			create b.make (a_creator, a_run_type, a_wafer_techn, a_run_id)
			Result := all_rows_filtered_ordered (conn, rows_matching_criteria_filter (a_creator, a_run_type, a_wafer_techn, a_run_id), b, ord)
			check_row_count (Result, range)
		ensure
			result_exists: Result /= Void
		end

	rows_matching_run (a_creator, an_intended_run, a_run_id: STRING ord: STRING conn: DB_CONNECTION range: ITEM_COUNT_RANGE): LIST [like Current] is
			-- Packings matching `a_creator' and
			-- `an_intended_run' that have attributes
			-- matching those of `a_run_id'.  Empty value
			-- for `a_creator' or `an_intended_run' means
			-- "do not use as a filtering criterion"
		require
			a_creator_not_void: a_creator /= Void
			an_intended_run_not_void: an_intended_run /= Void
			a_run_id_not_void: a_run_id /= Void
			connection_exists: conn /= Void
			range_exists: range /= Void
		local
			b: DB_BIND_SSS
		do
			create b.make (a_creator, an_intended_run, a_run_id)
			Result := all_rows_filtered_ordered (conn, rows_matching_run_filter (a_creator, an_intended_run, a_run_id), b, ord)
			check_row_count (Result, range)
		ensure
			result_exists: Result /= Void
		end

	unassigned_packings (conn: DB_CONNECTION range: ITEM_COUNT_RANGE): LIST [like Current] is
			-- List of packings that are not assigned to a run,
			-- ordered by creation date
		require
			connection_exists: conn /= Void
			range_exists: range /= Void
		do
			Result := all_rows_filtered_ordered (conn, "assigned_run_id is null", Void, "date_created")
			check_row_count (Result, range)
		ensure
			result_exists: Result /= Void
		end

	unassigned_compatible_packings (r: RUNS_ROW conn: DB_CONNECTION range: ITEM_COUNT_RANGE): LIST [like Current] is
			-- List of packings that are not assigned to a
			-- run and that are compatible with `r',
			-- ordered by creation date
		require
			runs_row_not_void: r /= Void
			connection_exists: conn /= Void
			range_exists: range /= Void
		local
			list: like unassigned_packings
			p: like Current
		do
			list := unassigned_packings (conn, range)
			create {ARRAYED_LIST [like Current]} Result.make (list.count)
			from
				list.start
			until
				list.after
			loop
				p := list.item
				p.check_compatibility_with_run (r)
				if p.is_compatible_with_run then
					Result.extend (p)
				end
				list.forth
			end
		ensure
			result_exists: Result /= Void
		end

feature -- Operations

	check_compatibility_with_run (r: RUNS_ROW) is
			-- Check whether `Current' is compatible with
			-- run described by `r'.  Set `is_compatible_with_run'
			-- to indicate whether it is compatible and
			-- `incompatibilities_with_run' to a description
			-- of the incompatibilities
		require
			r_not_void: r /= Void
		do
			is_compatible_with_run := True
			create incompatibilities_with_run.make (0)
			if not run_type.is_equal (r.run_type) or r.is_null_run_type then
				add_mismatch ("Run Type", run_type, r.run_type, False, r.is_null_run_type)
			end
			if not educational.is_equal (r.educational) then
				add_mismatch ("Educational", educational.out, r.educational.out, False, False)
			end
			if not wafer_techn.is_equal (r.wafer_techn) or r.is_null_wafer_techn then
				add_mismatch ("Wafer Techn", wafer_techn, r.wafer_techn, False, r.is_null_wafer_techn)
			end
			if sched_run_no /= r.sched_run_no or is_null_sched_run_no /= r.is_null_sched_run_no then
				add_mismatch ("Scheduled Run", sched_run_no.out, r.sched_run_no.out, is_null_sched_run_no, r.is_null_sched_run_no)
			end
			if not field_set_name.is_equal (r.field_set_name) or r.is_null_field_set_name then
				add_mismatch ("Field Set", field_set_name, r.field_set_name, False, r.is_null_field_set_name)
			end
			if uniform_override_border /= r.uniform_override_border or is_null_uniform_override_border /= r.is_null_uniform_override_border then
				add_mismatch ("Override Border", uniform_override_border.out, r.uniform_override_border.out, is_null_uniform_override_border, r.is_null_uniform_override_border)
			end
			if not overhead_style.is_equal (r.overhead_style) or is_null_overhead_style /= r.is_null_overhead_style then
				add_mismatch ("Overhead Style", overhead_style, r.overhead_style, is_null_overhead_style, r.is_null_overhead_style)
			end
			if other_border /= r.other_border or is_null_other_border /= r.is_null_other_border then
				add_mismatch ("Other Border", other_border.out, r.other_border.out, is_null_other_border, r.is_null_other_border)
			end
		end

feature -- Properties

	parent: RC_PACKING
			-- Packing with `Current' is associated

	is_compatible_with_run: BOOLEAN
			-- Is `Current' compatible with last run
			-- passed to `check_compatibility_with_run'?

	incompatibilities_with_run: STRING
			-- Description of incompatibilities between
			-- `Current' and last run passed to 
			-- `check_compatibility_with_run'?

	allowed_pricing_schedules: LIST [STRING] is
			-- Names of pricing schedules a project's order
			-- may have in order to be compatible with `Current'
		do
			if cached_allowed_pricing_schedules = Void then
				fetch_allowed_pricing_schedules
			end
			Result := cached_allowed_pricing_schedules
		ensure
			result_not_void: Result /= Void
		end

	is_cached_allowed_pricing_schedules: BOOLEAN is
			-- Is `allowed_pricing_schedules' currently cached?
		do
			Result := cached_allowed_pricing_schedules /= Void
		end

	wafer_techns_row: WAFER_TECHNS_ROW is
			-- Wafer techn for `Current'
		do
			if cached_wafer_techns_row = Void then
				fetch_wafer_techns_row
			end
			Result := cached_wafer_techns_row
		ensure
			result_not_void: Result /= Void
		end

	is_cached_wafer_techns_row: BOOLEAN is
			-- Is `wafer_techns_row' currently cached?
		do
			Result := cached_wafer_techns_row /= Void
		end

	all_wafer_etest_names: LIST [WAFER_ETEST_NAMES_ROW]
			-- List of all possible wafer etest names for
			-- wafer techn for this packing
		do
			if cached_all_wafer_etest_names = Void then
				fetch_all_wafer_etest_names
			end
			Result := cached_all_wafer_etest_names
		ensure
			result_not_void: Result /= Void
		end

	is_cached_all_wafer_etest_names: BOOLEAN is
			-- Is `all_wafer_etest_names' currently cached?
		do
			Result := cached_all_wafer_etest_names /= Void
		end

	grid_resolution: DOUBLE
			-- Resolution of grid
		do
			Result := wafer_techns_row.packing_grid_resolution
		end

	grid: GRID
			-- Grid used for packing
		do
			if cached_grid = Void then
				set_cached_grid
			end
			Result := cached_grid
		ensure
			result_not_void: Result /= Void
		end

	field_set: FIELD_SETS_ROW is
			-- Field set for `Current'
		do
			if cached_field_set = Void then
				fetch_field_set
			end
			Result := cached_field_set
		ensure
			result_not_void: Result /= Void
		end

	is_cached_field_set: BOOLEAN is
			-- Is `field_set' currently cached?
		do
			Result := cached_field_set /= Void
		end

	litho_sizes: LITHO_SIZES_ROW is
			-- Lithography sizes for `Current'
		do
			Result := field_set.litho_size
		ensure
			result_not_void: Result /= Void
		end

	run: RUNS_ROW is
			-- Run for `Current'
		do
			if cached_run = Void then
				fetch_run
			end
			Result := cached_run
		ensure
			result_not_void: Result /= Void
		end

	is_cached_run: BOOLEAN is
			-- Is `run' currently cached?
		do
			Result := cached_run /= Void
		end

	highest_used_project_no: INTEGER is
			-- Highest project_no that has been used for any
			-- version of packing
		do
			if not is_cached_highest_used_project_no then
				fetch_highest_used_project_no
				is_cached_highest_used_project_no := True
			end
			Result := cached_highest_used_project_no
		end

	is_cached_highest_used_project_no: BOOLEAN
			-- Is `highest_used_project_no' currently cached?

	non_null_assigned_run_id: like assigned_run_id is
			-- Value of `assigned_run_id', with error check 
			-- for nullness
		do
			check_not_null ("assigned_run_id", is_null_assigned_run_id, Current)
			Result := assigned_run_id
		end

	wafer_techn_family_processes: LIST [WAFER_PROCESSES_ROW] is
			-- All wafer processes for packing's wafer techn family
		do
			if cached_wafer_techn_family_processes = Void then
				fetch_wafer_techn_family_processes
			end
			Result := cached_wafer_techn_family_processes
		ensure
			result_not_void: Result /= Void
		end

	is_cached_wafer_techn_family_processes: BOOLEAN is
			-- Is `wafer_techn_family_processes' currently cached?
		do
			Result := cached_wafer_techn_family_processes /= Void
		end

	effective_overhead_style (a_style: STRING): STRING
			-- Effective overhead style for `a_style' (global
			-- value for packing if not null, otherwise `a_style')
		do
			if not is_null_overhead_style then
				Result := overhead_style
			else
				Result := a_style
			end
		end

	effective_overhead_style_description (a_style: STRING): STRING
			-- Description of effective overhead style 
			-- for `a_style'
		do
			Result := effective_overhead_style (a_style)
			if not Result.is_equal (a_style) then
				Result := Result + " (" + a_style + ")"
			end
		end

	uniform_white_space_border (a_style: STRING): DOUBLE
			-- Minimum amount of white space on all sides
			-- of project whose overhead style is `a_style'
		require
			a_style_not_void: a_style /= Void
		local
			l_style: STRING
		do
			l_style := effective_overhead_style (a_style)
			Result := wafer_techns_row.uniform_white_space_border (l_style, other_border)
		end

feature -- Operations

	set_parent (p: like parent) is
		do
			parent := p
		ensure
			parent_set: parent = p
		end

	uncache_all is
			-- Remove all items from all caches, except
			-- `highest_used_project_no' (if projects not
			-- yet saved, uncaching might reuse their 
			-- project_no values)
		do
			uncache_allowed_pricing_schedules
			uncache_wafer_techns_row
			uncache_all_wafer_etest_names
			uncache_field_set
			uncache_wafer_techn_family_processes
			uncache_run
		end

	uncache_allowed_pricing_schedules is
			-- Remove allowed pricing schedules from cache, 
			-- if present
		do
			cached_allowed_pricing_schedules := Void
		ensure
			pricing_scheds_not_cached: not is_cached_allowed_pricing_schedules
		end

	uncache_wafer_techns_row is
			-- Remove wafer techns row from cache, if present
		do
			cached_wafer_techns_row := Void
		ensure
			wafer_techns_row_not_cached: not is_cached_wafer_techns_row
		end

	uncache_all_wafer_etest_names
			-- Remove all wafer etest names list from cache, 
			-- if present
		do
			cached_all_wafer_etest_names := Void
		ensure
			not_cached: not is_cached_all_wafer_etest_names
		end

	uncache_field_set is
			-- Remove field set from cache, if present
		do
			cached_field_set := Void
		ensure
			field_set_not_cached: not is_cached_field_set
		end

	uncache_wafer_techn_family_processes is
			-- Remove wafer techn family processes from cache, 
			-- if present
		do
			cached_wafer_techn_family_processes := Void
		ensure
			not_cached: not is_cached_wafer_techn_family_processes
		end

	uncache_run is
			-- Remove run from cache, if present
		do
			cached_run := Void
		ensure
			run_not_cached: not is_cached_run
		end

	increment_highest_used_project_no is
			-- Add one to `highest_used_project_no'
		do
			cached_highest_used_project_no := highest_used_project_no + 1
		end

	save_highest_used_project_no is
			-- Save current value of `highest_used_project_no',
			-- so that it can be restored after we create
			-- new projects that are used just for display
			-- (not part of packing yet)
		do
			saved_highest_used_project_no := highest_used_project_no
		end

	restore_highest_used_project_no is
			-- Restore saved value of `highest_used_project_no'
		do
			set_highest_used_project_no (saved_highest_used_project_no)
		end

feature {NONE} -- Implementation

	rows_matching_criteria_filter (a_creator, a_run_type, a_wafer_techn, a_run_id: STRING): STRING is
			-- Query filter to match arguments
		do
			create Result.make (100)
			if not a_creator.is_empty then
				Result.append ("creator = :1")
			else
				Result.append (":1 is null")
			end
			Result.append (" and ")
			if not a_run_type.is_empty then
				Result.append ("run_type = :2")
			else
				Result.append (":2 is null")
			end
			Result.append (" and ")
			if not a_wafer_techn.is_empty then
				Result.append ("wafer_techn = :3")
			else
				Result.append (":3 is null")
			end
			Result.append (" and ")
			if not a_run_id.is_empty then
				Result.append ("intended_run = :4")
			else
				Result.append (":4 is null")
			end
		end

	rows_matching_run_filter (a_creator, an_intended_run, a_run_id: STRING): STRING is
			-- Query filter to match `a_creator' and 
			-- `an_intended_run' and attributes of `a_run_id'
		do
			create Result.make (100)
			if not a_creator.is_empty then
				Result.append ("creator = :1")
			else
				Result.append (":1 is null")
			end
			Result.append (" and ")
			if not an_intended_run.is_empty then
				Result.append ("intended_run = :2")
			else
				Result.append (":2 is null")
			end
			Result.append (" and packing_id in %N%
			%	(select packing_id from packings p, runs r %N%
			%	 where %N%
			%	 r.run_type = p.run_type and %N%
			%	 r.educational = p.educational and %N%
			%	 r.wafer_techn = p.wafer_techn and %N%
			%	 r.field_set_name = p.field_set_name and %N%
			%	 ((r.sched_run_no = p.sched_run_no) or (r.sched_run_no is null and p.sched_run_no is null)) and %N%
			%	 ((r.uniform_override_border = p.uniform_override_border) or (r.uniform_override_border is null and p.uniform_override_border is null)) %N%
			%	 and r.run_id = :3)")
		end

	add_mismatch (fld_name, pck_val, run_val: STRING is_null_pck_val, is_null_run_val: BOOLEAN) is
		local
			s: STRING
		do
			s := incompatibilities_with_run
			s.append ("Packing " + fld_name + ":  ")
			if is_null_pck_val then
				s.append ("null")
			else
				s.append (pck_val)
			end
			s.append ("      ")
			s.append ("Run " + fld_name + ":  ")
			if is_null_run_val then
				s.append ("null")
			else
				s.append (run_val)
			end
			s.extend ('%N')
			is_compatible_with_run := False
		end

	fetch_allowed_pricing_schedules is
		local
			type: RUN_TYPE_VALUES
			b: DB_BIND_I
			s: STRING
		do
			if run_type.is_equal (type.Skd) then
				if is_null_sched_run_no then
					s := ""
					b := Void
				else
					s := " and sched_run_no = :1"
					create b.make (sched_run_no)
				end
				cached_allowed_pricing_schedules := any_connection.selected_strings ("pricing_schedule", "pricing_schedules ps, scheduled_run#pricing_groups srpg", "srpg.pricing_group = ps.pricing_group" + s, "pricing_schedule", True, b)
			else
				cached_allowed_pricing_schedules := any_connection.selected_strings ("pricing_schedule", "pricing_schedules", Void, "pricing_schedule", False, Void)
			end
			cached_allowed_pricing_schedules.compare_objects
		end

	cached_allowed_pricing_schedules: LIST [STRING]
			-- Project pricing schedules allowed for `Current',
			-- cached in memory

	fetch_wafer_techns_row is
		local
			row: WAFER_TECHNS_ROW
		do
			create row.make_template
			cached_wafer_techns_row := row.wafer_techns_with_name (wafer_techn, any_connection, expected_count.Exactly_one).first
		end

	fetch_all_wafer_etest_names
		local
			row: WAFER_ETEST_NAMES_ROW
		do
			create row.make_template
			cached_all_wafer_etest_names := row.rows_for_wafer_techn (wafer_techn, any_connection, expected_count.At_least_one)
		end

	cached_wafer_techns_row: WAFER_TECHNS_ROW
			-- Wafer techn for `Current', cached in memory

	cached_all_wafer_etest_names: LIST [WAFER_ETEST_NAMES_ROW]
			-- List of all wafer etest names for Wafer techn 
			-- for `Current', cached in memory

	set_cached_grid
		local
			res: like grid_resolution
		do
			res := grid_resolution
			create cached_grid
			cached_grid.set_size_resolution (res)
			cached_grid.set_edge_resolution (res)
			cached_grid.set_center_resolution (res / 2.0)
			cached_grid.set_scale ({PACKING_DIE_LOCATIONS_ROW}.scale_location_x)
		end

	cached_grid: GRID
			-- Grid for packing (cached)

	fetch_field_set is
		local
			row: FIELD_SETS_ROW
		do
			create row.make_template
			cached_field_set := row.field_sets_with_name (field_set_name, any_connection, expected_count.Exactly_one).first
		end

	cached_field_set: FIELD_SETS_ROW
			-- Field set for `Current', cached in memory

	fetch_wafer_techn_family_processes is
		local
			row: WAFER_PROCESSES_ROW
		do
			create row.make_template
			cached_wafer_techn_family_processes := row.processes_for_techn_family (wafer_techns_row.wafer_techn_family, any_connection, expected_count.At_least_zero)
		end

	cached_wafer_techn_family_processes: LIST [WAFER_PROCESSES_ROW]
			-- Wafer techn family processes for `Current', 
			-- cached in memory

	fetch_run is
		do
			create cached_run.make_by_key (any_connection, option.Old_row, non_null_assigned_run_id)
		end

	cached_run: RUNS_ROW
			-- Run for `Current', cached in memory

	fetch_highest_used_project_no is
		local
			b: DB_BIND_S
		do
			create b.make (packing_id)
			set_highest_used_project_no (any_connection.safe_selected_integer ("max(project_no)", "packing_projects", "packing_id = :1", b))
		end

	set_highest_used_project_no (n: INTEGER) is
			-- Set `cached_highest_used_project_no' to `n'
		do
			cached_highest_used_project_no := n
			is_cached_highest_used_project_no := True
		end

	cached_highest_used_project_no: INTEGER
			-- Highest used project number for `Current', 
			-- cached in memory.  However, the in-memory copy
			-- may be updated as never-previously selected
			-- projects are added, so there is no uncache routine

	saved_highest_used_project_no: INTEGER
			-- Saved value of highest used project number for 
			-- `Current' (can be restored later)

end
