
indexing
	
	description: "Converter that converts PDF to other formats, %
		%such as PostScript"
	author: "David Hollenberg";
	date: "February 16, 2005"

class PDF_CONVERTER
inherit
	ANY
	CONDITION_UTILITIES
		export
			{NONE} all
		end
	UNIX_OS_ACCESS
		export
			{NONE} all
		end

create
	make

feature {NONE} -- Creation

	make (acro_path: STRING) is
			-- Create with `acro_path' as pathname of Acroread
			-- executable
		require
			path_exists: acro_path /= Void
			path_not_empty: not acro_path.is_empty
		do
			acroread_pathname := acro_path
		end

feature -- Properties

	acroread_pathname: STRING
			-- Path name of acroread executable

feature -- Operations

	convert_named_file_to_postscript (pdf_name, ps_name: STRING) is
			-- Convert PDF file `pdf_name' to PostScript
			-- and put in `ps_name'
		require
			pdf_name_not_void: pdf_name /= Void
			pdf_name_not_empty: not pdf_name.is_empty
			ps_name_not_void: ps_name /= Void
			ps_name_not_empty: not ps_name.is_empty
		local
			in_file: LARGE_PLAIN_TEXT_FILE
			out_file: LARGE_RAW_FILE
			open_read_op: FILE_OPEN_READ_OP
			open_write_op: FILE_OPEN_WRITE_OP
			close_op: FILE_CLOSE_OP
		do
			create open_read_op
			create open_write_op
			create close_op
			create in_file.make (pdf_name)
			open_read_op.open (in_file)
			create out_file.make (ps_name)
			open_write_op.open (out_file)
			convert_to_postscript (in_file, out_file)
			close_op.close (in_file)
			close_op.close (out_file)
		end

	convert_to_postscript (in_file: LARGE_PLAIN_TEXT_FILE out_file: LARGE_FILE) is
			-- Convert PDF file `in_file' to PostScript and
			-- write to `out_file'
		require
			in_file_not_void: in_file /= Void
			in_file_readable: in_file.is_open_read
			out_file_not_void: out_file /= Void
			out_file_writable: out_file.is_open_write
		local
			f: LARGE_RAW_FILE
			process: UNIX_PROCESS
			checker: FILE_EXISTENCE_CHECK_OP
			old_count, new_count: INTEGER_64
		do
			create checker
			checker.check_existence (acroread_pathname)
			if not checker.path_exists then
				handle_hard_error (program_description + " not found")
			end
			create f.make (acroread_pathname)
			if not f.is_plain then
				handle_hard_error (program_description + " is not a plain file")
			elseif not f.is_executable then
				handle_hard_error (program_description + " is not executable")
			else
				old_count := out_file.count
				create process.make (acroread_pathname)
				process.set_arguments (<< "-toPostScript", "-size", "letter" >>)
				process.set_input_descriptor (in_file.descriptor)
				process.set_output_descriptor (out_file.descriptor)
				process.set_error_file_name (unix_os.null_file_name)

				process.spawn_wait
				
				if process.is_exit_status (process.status) then
					if process.exit_code (process.status) /= 0 then
						handle_hard_error (program_description + " exited with code " + process.exit_code (process.status).out)
					end
				else
					handle_hard_error (program_description + " terminated or stopped by signal " + process.signal_number (process.status).out)
				end
				
				new_count := out_file.count
				if new_count = old_count then
					handle_hard_error (program_description + " did not convert PDF to PostScript.%NOutput file " + out_file.name + " did not increase in size")
				end
			end
		end

feature {NONE} -- Implementation

	program_description: STRING is
			-- Description of conversion program
		do
			Result := "Acroread program file " + acroread_pathname
		ensure
			result_exists: Result /= Void
		end

end
