
indexing
	
	description: "A run closing step operation that inserts a %
		%FAB POSTED event"
	author: "David Hollenberg"
	date: "June 3, 2005"

class POST_RUN_OP
inherit
	RUN_EVENT_INSERTION_OP

create
	make

feature -- Properties

	input_data_items: LIST [STRING]
			-- Data items required before step can be executed
		do
			Result := array_as_list (<<
				data_item.Tapeout_date,
				data_item.Turnaround_estimates,
				data_item.Sent_foundry_eccn_report
				 >>)
		end

feature {NONE} -- Implementation

	get_final_confirmation (d: DB_DATE)
			-- Get final confirmation that event should really
			-- be inserted and notify appropriate parties
		local
			tapeout_date: DB_DATE
			msg: STRING
		do
			tapeout_date := cache.mask_sets_row.date_tapeout
			msg := "Run will be marked as %"posted%" on " + d.to_date_string + ".%NTapeout date was " + tapeout_date.to_date_string + "."
			if cache.wafer_techn_families_row.foundry_reviews_layout then
				msg := msg + "%NE-mail notifications will be sent to internal MOSIS mailing lists.%N%NProceed"
				confirm (msg)
				session.commit_actions.extend (agent publish_run_posted_event (d, tapeout_date))
			else
				msg := msg + "%N%NProceed"
				confirm (msg)
			end
		end

	publish_run_posted_event (posted_date, tapeout_date: DB_DATE)
			-- Publish a "run posted" event
		require
			posted_date_not_void: posted_date /= Void
			tapeout_date_not_void: tapeout_date /= Void
		local
			event: RUN_POSTED_EVENT
			wt: STRING
		do
			wt := "(" + cache.wafer_techn + ")"
			create event
			event.set_subject (wt + " sent to foundry for review")
			event.set_message ("Run " + session.run_id + " " + wt + " has been posted.%NIt was sent to the foundry for review on " + tapeout_date.to_date_string + ".")
			event.publish
		end

	remove_run_from_reports
			-- Re-run reports so run does not show up on
			-- them any more
		local
			fname: STRING
			open_op: FILE_OPEN_WRITE_OP
			close_op: FILE_CLOSE_OP
			f: LARGE_PLAIN_TEXT_FILE
		do
			execute_all_runs_status_report
			fname := run_status_report_output_file
			create f.make (fname)
			create open_op
			open_op.open (f)
			f.put_string (Empty_html)
			create close_op
			close_op.close (f)
		end

	event_name: STRING
			-- Name of event to be inserted by operation
		local
			n: EVENT_NAME_VALUES
		do
			Result := n.Fab_posted
		end

	date_name: STRING = "Date Fab Posted"
			-- Name of date user is asked to enter

	add_undo_commit_actions
			-- Add any required session commit actions for undo
		do
			session.commit_actions.extend (agent remove_run_from_reports)
		end

	check_date_validity (d: DB_DATE)
			-- Check that `d' is valid for `event_name'
		local
			tapeout_date: DB_DATE
		do
			tapeout_date := cache.mask_sets_row.date_tapeout
			if d < tapeout_date then
				handle_user_error ("Date cannot be before tapeout date (" + tapeout_date.to_date_string + ")")
			end
		end

	max_undo_allowed_days: INTEGER
			-- Maximum age of event in days to allow undo
			-- of step
		do
			Result := policies.post_undo_allowed_interval
		end

	Empty_html: STRING = "[
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN"
    "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
<html>
<head>
<title></title>
<meta http-equiv="Content-Type" content="text/html; charset=iso-8859-1" />
</head>
<body>
</body>
</html>
		]"

end
