
indexing
	
	description: "Converter that converts PostScript to other formats, %
		%such as PDF"
	author: "David Hollenberg";
	date: "January 20, 2005"

class POSTSCRIPT_CONVERTER
inherit
	ANY
	CONDITION_UTILITIES
		export
			{NONE} all
		end
	UNIX_OS_ACCESS
		export
			{NONE} all
		end

create
	make

feature {NONE} -- Creation

	make (gs_path: STRING) is
			-- Create with `gs_path' as pathname of Ghostscript
			-- executable
		require
			path_exists: gs_path /= Void
			path_not_empty: not gs_path.is_empty
		do
			ghostscript_pathname := gs_path
		end

feature -- Properties

	ghostscript_pathname: STRING
			-- Path name of GhostScript executable

feature -- Operations

	convert_named_file_to_pdf (in_fname, out_fname: STRING) is
			-- Convert PostScript file named `in_fname' to PDF and
			-- write to `out_fname'
		require
			in_fname_not_void: in_fname /= Void
			in_fname_not_empty: not in_fname.is_empty
			out_fname_not_void: out_fname /= Void
			out_fname_not_empty: not out_fname.is_empty
		local
			in_file: LARGE_PLAIN_TEXT_FILE
			out_file: LARGE_RAW_FILE
			open_read_op: FILE_OPEN_READ_OP
			open_write_op: FILE_OPEN_WRITE_OP
			close_op: FILE_CLOSE_OP
		do
			create open_read_op
			create open_write_op
			create close_op
			create in_file.make (in_fname)
			open_read_op.open (in_file)
			create out_file.make (out_fname)
			open_write_op.open (out_file)
			convert_file_to_pdf (in_file, out_file)
			close_op.close (in_file)
			close_op.close (out_file)
		end

	convert_file_to_pdf (in_file: LARGE_PLAIN_TEXT_FILE out_file: LARGE_FILE) is
			-- Convert PostScript file `in_file' to PDF and
			-- write to `out_file'
		require
			in_file_not_void: in_file /= Void
			in_file_readable: in_file.is_open_read
			out_file_not_void: out_file /= Void
			out_file_writable: out_file.is_open_write
		do
			convert_file_to_format (in_file, out_file, Pdf_format)
		end

	convert_file_to_format (in_file: LARGE_PLAIN_TEXT_FILE out_file: LARGE_FILE fmt: STRING) is
			-- Convert `in_file' to `out_file', using `fmt'
			-- to determine the output format.  `fmt' must
			-- be one of the available devices listed by
			-- running GhostScript with the -h option (gs -h)
		require
			in_file_not_void: in_file /= Void
			in_file_readable: in_file.is_open_read
			out_file_not_void: out_file /= Void
			out_file_writable: out_file.is_open_write
			format_exists: fmt /= Void
		local
			f: LARGE_RAW_FILE
			process: UNIX_PROCESS
			checker: FILE_EXISTENCE_CHECK_OP
			old_count, new_count: INTEGER_64
		do
			create checker
			checker.check_existence (ghostscript_pathname)
			if not checker.path_exists then
				handle_hard_error ("GhostScript program file " + ghostscript_pathname + " not found")
			end
			create f.make (ghostscript_pathname)
			if not f.is_plain then
				handle_hard_error (program_description + " is not a plain file")
			elseif not f.is_executable then
				handle_hard_error (program_description + " is not executable")
			else
				old_count := out_file.count
				create process.make (ghostscript_pathname)
				process.set_arguments (<< "-sDEVICE=" + fmt, "-sOutputFile=-", "-dAutoRotatePages=/None", "-dNOPAUSE", "-dBATCH", "-dSAFER", "-q", "-" >>)
		    		process.set_input_descriptor (in_file.descriptor)
		    		process.set_output_descriptor (out_file.descriptor)
		    		process.set_error_file_name (unix_os.null_file_name)
				
		    		process.spawn_wait
				
		    		if process.is_exit_status (process.status) then
					if process.exit_code (process.status) /= 0 then
						handle_hard_error (program_description + " exited with code " + process.exit_code (process.status).out)
					end
				else
					handle_hard_error (program_description + " terminated or stopped by signal " + process.signal_number (process.status).out)
				end
				
				new_count := out_file.count
				if new_count = old_count then
					handle_hard_error (program_description + " did not convert PostScript to PDF.%NOutput file " + out_file.name + " did not increase in size")
				end
			end
		end

feature {NONE} -- Implementation

	program_description: STRING is
			-- Description of conversion program
		do
			Result := "GhostScript program file " + ghostscript_pathname
		ensure
			result_exists: Result /= Void
		end

	Pdf_format: STRING is "pdfwrite"
			-- Device name to pass to Ghostscript to make it
			-- produce PDF
end
