
indexing
	
	description: "Action to be taken prior to attempt to do scan"
	author: "David Hollenberg"
	date: "January 3, 2007"

deferred class PRE_SCAN_ACTION
inherit
	VIEW_ELEMENT_ACTION

feature -- Properties

	scan_elements: LIST [RETICLE_VIEW_ELEMENT]
			-- List of elements that will participate in scan

	scan_fields: LIST [RETICLE_FIELD]
			-- List of fields involved in scan

	wrong_orientation_count: INTEGER
			-- Number of scribes that have the wrong orientation
			-- and will participate in scan

	movable_count: INTEGER
			-- Number of elements that will participate in scan
			-- that are movable in the dimension of the scan

feature -- Modification

	set_scan_elements (a_list: like scan_elements)
			-- Set `scan_elements' to `a_list'
		require
			a_list_not_void: a_list /= Void
		do
			scan_elements := a_list
		ensure
			scan_elements_set: scan_elements = a_list
		end

	set_scan_fields (a_list: like scan_fields)
			-- Set `scan_fields' to `a_list'
		require
			a_list_not_void: a_list /= Void
		do
			scan_fields := a_list
		ensure
			scan_fields_set: scan_fields = a_list
		end

	initialize
			-- Initialize before scan
		do
			wrong_orientation_count := 0
			movable_count := 0
		ensure
			wrong_orientation_count_zero: wrong_orientation_count = 0
			movable_count_zero: movable_count = 0
		end

feature -- Operations

	apply (an_element: RETICLE_VIEW_ELEMENT)
			-- Apply action to `an_element'
		local
			included: BOOLEAN
			f: RETICLE_FIELD
		do
			if is_valid_scribe (an_element) then
				if not an_element.intersects_any_field then
					included := False
					an_element.add_warning (an_element.identification_for_message + " excluded from scan - does not intersect any field of view")
				else
					included := is_scribe_included (an_element)
					if not included then
						an_element.add_warning (an_element.identification_for_message + " excluded from scan - center is outside field edges")
					end
				end
			elseif is_invalid_scribe (an_element) then
				included := False
				wrong_orientation_count := wrong_orientation_count + 1
				an_element.add_warning (an_element.identification_for_message + " excluded from scan - wrong orientation")
			else
				included := an_element.is_center_in_field
				if not included then
					an_element.add_warning (an_element.identification_for_message + " excluded from scan - center is not inside any field of view")
				end
			end
			if included then
				scan_elements.extend (an_element)
				f := an_element.reticle_field
				if not scan_fields.has (f) then
					scan_fields.extend (f)
				end
				if is_movable (an_element) then
					movable_count := movable_count + 1
				end
			end
		end

feature {NONE} -- Implementation

	is_valid_scribe (an_element: RETICLE_VIEW_ELEMENT): BOOLEAN
			-- Is `an_element' a scribe of the correct orientation
			-- to be included in this scan?
		deferred
		end

	is_invalid_scribe (an_element: RETICLE_VIEW_ELEMENT): BOOLEAN
			-- Is `an_element' a scribe of the wrong orientation
			-- to be included in this scan?
		do
			Result := an_element.is_scribe and then not is_valid_scribe (an_element)
		end

	is_scribe_included (an_element: RETICLE_VIEW_ELEMENT): BOOLEAN
			-- Should scribe `an_element', which has the correct
			-- orientation, be included in this scan?
		deferred
		end

	is_movable (an_element: RETICLE_VIEW_ELEMENT): BOOLEAN
			-- Is `an_element' movable for scanning purposes
			-- in the direction of the scan?
		deferred
		end

end
