class PRETTY_PRINT_STRING_MODIFIER
		-- the STRING_MODIFIER to indent and wrap long lines

inherit
	OBJECT_MODIFIER[STRING]

	TEXT_FOLDING

create
	make

feature -- the OBJECT_MODIFIER features

	needs_modification(s: STRING): BOOLEAN is
		-- determine whether or not s needs to be fixed
	do  Result := first_indent.count > 0 or s.count > max_count or
			final_suffix.count > 0
	end;

	make_modified_object(s: STRING): STRING is
		-- make a NEW string with the appropriate modifications
		-- reformat txt into a multi-line string, most of whose lines
		--  are no longer than max_count.
		-- first line is prefixed with first_indent
		-- the continuation lines with other_indent
		-- empty lines are removed.
	local lines: DYNAMIC_LIST[STRING];
		lin, nxt, ind: STRING;  length, brk: INTEGER;
	do  lines := su.broken_at(s, '%N');
	    ind := first_indent;	-- just use it once
	    from  lines.start
	    until lines.after
	    loop  lin := lines.item;
		  lin.left_adjust;  lin.right_adjust;
		  length := lin.count;
		  if not lin.is_empty then length := length + ind.count end;
		  if length > max_count
		   then		-- (maybe) break lin up into two entries
			brk := break_somewhere(lin, max_count - ind.count);
			if brk > 0
			 then nxt := lin.substring(brk+1, lin.count);
			      lin.keep_head(brk);
			      lin.right_adjust;
			      lines.put_right(nxt)	-- it will be next
			end
		  end;
		  lin.prepend (ind);
		  ind := other_indent;	-- now switch here
		  lines.forth
	    end;
	    lines := su.empty_strings_removed(lines);
	    Result := unfolded_text(lines);
	    if (Result.count > 0 and then 
		not is_special(Result @ Result.count))
	     then Result.append(final_suffix) end;
	end;

feature -- attributes set at creation

	max_count: INTEGER;
		-- max length of any one line (can be violated)

	first_indent,
	other_indent: STRING
		-- indentation of first and continuation lines

	final_suffix: STRING

feature -- worker bees

	break_somewhere(lin: STRING; max: INTEGER): INTEGER is
		-- maybe pick a point to break up lin, hopefully
		-- somewhere in the first max characters.
	require lin.count > max; max > 0;
	do  Result := break_at_lwsp(lin, max);
	    if Result = 0 then Result := break_after_max(lin, max)  end;
	end;

	break_after_max(lin: STRING; max: INTEGER): INTEGER is
	local cnt: INTEGER
	do  cnt := lin.count - 3;
	    from  Result := max+1
	    until Result >= cnt or else is_lwsp(lin @ (Result+1))
	    loop  Result := Result + 1
	    end;
	    if Result >= cnt then Result := 0 end	-- no fold
	end;

feature -- creation

	make(mlen: INTEGER; ind1, ind_other, suffix: STRING) is
	do  max_count := mlen;
	    first_indent := ind1;
	    other_indent := ind_other;
	    final_suffix := suffix;
	    -- and clean up
	    if max_count <= 0 then max_count := 1000000 end;
	    if first_indent = Void then first_indent := "" end;
	    if other_indent = Void then other_indent := "" end;
	    if final_suffix = Void then final_suffix := "" end;
	end;

end -- PRETTY_PRINT_STRING_MODIFIER
