
indexing
	
	description: "Site on wafer which can be probed or inked"
	author: "David Hollenberg"
	date: "August 30, 2004"

class PROBE_SITE
inherit
	ANY
		redefine
			out
		end
	RC_UTILITIES_ACCESS
		export
			{NONE} all
		redefine
			out
		end
	STRING_UTILITIES_ACCESS
		export
			{NONE} all
		redefine
			out
		end

feature -- Properties

	run_id: STRING
			-- Unique identifier for run

	wafer_site_number: INTEGER
			-- Unique identifying number for site on wafer

	field_name: STRING
			-- Name of field of view containing site

	copy_number: INTEGER
			-- Copy number of field on wafer

	mask_site_number: INTEGER
			-- Site number on mask

	die_id: STRING
			-- Die identifier for test vehicle site is in

	location: CONST_POINT
			-- Location of probe origin for this site in microns,
			-- relative to upper left corner of box enclosing
			-- wafer circle

	step_location: CONST_POINT
			-- Location on wafer of field containing site,
			-- in microns

	object_name: STRING
			-- Name of test object at this site

	object_type: STRING
			-- Type of object at this site (parametric, functional,
			-- ink)

	is_visited: BOOLEAN
			-- Is site marked as being already visited
			-- (for use in reordering sites)?

	is_parametric: BOOLEAN
			-- Is this a parametric test object?
		local
			type: TEST_OBJECT_TYPE_VALUES
		do
			Result := object_type.is_equal (type.Parametric)
		end

	is_functional: BOOLEAN
			-- Is this a functional test object?
		local
			type: TEST_OBJECT_TYPE_VALUES
		do
			Result := object_type.is_equal (type.Functional)
		end

	is_development: BOOLEAN
			-- Is this a development test object?
		local
			type: TEST_OBJECT_TYPE_VALUES
		do
			Result := object_type.is_equal (type.Development)
		end

	is_nsa_object: BOOLEAN
			-- Is this an NSA test object?
		do
			Result := su.wildcard_matches (object_name, "*NSC*")
		end

	description: STRING
			-- Description of site suitable for inclusion
			-- in wafer file header
		do
			create Result.make (100)
			Result.append_integer (wafer_site_number)
			Result.extend ('%T')
			Result.append ("(Field ")
			Result.append (field_name)
			Result.append (", copy ")
			Result.append_integer (copy_number)
			Result.append (", mask_site ")
			Result.append_integer (mask_site_number)
			Result.append (", object ")
			Result.append (object_name)
			Result.extend (')')
		ensure
			result_exists: Result /= Void
		end

	out: STRING
			-- Description of site suitable for inclusion
			-- in wafer file body
		do
			create Result.make (100)
			Result.append (run_id)
			Result.append (name_separator)
			Result.append (die_id)
			Result.append (name_separator)
			Result.append (field_name)
			Result.append (name_separator)
			Result.append (su.integer_zero_padded (copy_number, 2))
			Result.append (name_separator)
			Result.append (su.integer_zero_padded (mask_site_number, 3))
			Result.append (field_separator)
			Result.append (su.left_padded (wafer_site_number.out, 3))
			Result.append (field_separator)
			Result.append (su.left_padded (location.x.rounded.out, 6))
			Result.append (field_separator)
			Result.append (su.left_padded (location.y.rounded.out, 6))
			Result.append (field_separator)
			Result.append (su.right_padded (object_name, 24))
			Result.append (field_separator)
			Result.append (object_type)
		end

feature -- Modification

	set_run_id (s: STRING)
			-- Set `run_id' to `s'
		require
			value_exists: s /= Void
		do
			run_id := s
		ensure
			value_set: run_id = s
		end

	set_wafer_site_number (n: INTEGER)
			-- Set `wafer_site_number' to `n'
		require
			value_positive: n > 0
		do
			wafer_site_number := n
		ensure
			value_set: wafer_site_number = n
		end

	set_field_name (s: STRING)
			-- Set `field_name' to `s'
		require
			value_exists: s /= Void
		do
			field_name := s
		ensure
			value_set: field_name = s
		end

	set_copy_number (n: INTEGER)
			-- Set `copy_number' to `n'
		require
			value_positive: n > 0
		do
			copy_number := n
		ensure
			value_set: copy_number = n
		end

	set_mask_site_number (n: INTEGER)
			-- Set `mask_site_number' to `n'
		require
			value_positive: n > 0
		do
			mask_site_number := n
		ensure
			value_set: mask_site_number = n
		end

	set_die_id (s: STRING)
			-- Set `die_id' to `s'
		require
			value_exists: s /= Void
		do
			die_id := s
		ensure
			value_set: die_id = s
		end

	set_location (p: CONST_POINT)
			-- Set `location' to `p'
		require
			value_exists: p /= Void
		do
			location := p
		ensure
			value_set: location = p
		end

	set_step_location (p: CONST_POINT)
			-- Set `step_location' to `p'
		require
			value_exists: p /= Void
		do
			step_location := p
		ensure
			value_set: step_location = p
		end

	set_object_name (s: STRING)
			-- Set `object_name' to `s'
		require
			value_exists: s /= Void
		do
			object_name := s
		ensure
			value_set: object_name = s
		end

	set_object_type (s: STRING)
			-- Set `object_type' to `s'
		require
			value_exists: s /= Void
		do
			object_type := s
		ensure
			value_set: object_type = s
		end

	set_visited (b: BOOLEAN)
			-- Set `visited' to `b'
		do
			is_visited := b
		ensure
			value_set: is_visited = b
		end

feature {NONE} -- Implementation

	name_separator: STRING = "_"
			-- Separator between components of site name

	field_separator: STRING = "  "
			-- Separator between different attributes of site

end
