class PROCESS_CREATION_ACTION

inherit
	ACTION_ITEM

	SUB_PROCESS_CACHE

--	NFE_PROCESSOR_MODE

	OS_ACCESS export {NONE} all end;

	UNIX_OS_ACCESS export {NONE} all end;

	STRING_UTILITIES_ACCESS export {NONE} all end;

create
	make

feature -- items set up by make and used by do_it

	proc_mode: STRING;

	proc_args: ARRAY[STRING];

	proc_lock: LOCK;

	proc_in_action: UNIX_PROCESS;

	proc_name: STRING;

feature -- set arguments

	set_proc_args(args: ARRAY[STRING]) is
	do
	    proc_args := args;
	end;

feature	-- executor

	do_it is
	local taken: BOOLEAN
	do  -- see if the lock is available (or none required)
	    if proc_lock /= Void
	     then proc_lock.test_lock
		  taken := proc_lock.conflicting_lock_exists
	    end;
	    -- and start up the sucker
	    if not taken
	     then proc_in_action.set_arguments(proc_args)
		  proc_in_action.set_process_group_leader (True)
		  proc_in_action.spawn_nowait;
		  if proc_in_action.in_child
		   then os.die_without_handlers(1) end;	-- BIG OOPS
		  all_my_children.extend(proc_in_action);
--		  glo.log_pid_line("Spawned", proc_name,
--					proc_in_action.process_id);
--	    elseif proc_lock /= Void
--	     then glo.log_line_2("Lock already taken for", proc_mode)
	    end;
	    done := true;
	rescue
	    if proc_in_action.in_child
	     then	-- BIG OOPS; speak quickly and DIE (do not PUNT)
		os.die_without_handlers(2)
	    end;
	end;

	descr: STRING is
	do  
           Result := su.string_of_2("start ", proc_name)
	end;

	add_an_arg(arg: STRING) is
		-- extend proc_args with arg
	local newup: INTEGER;
	do  if proc_args = Void then
	       create proc_args.make(1,1)
	       newup := 1;
	     else
	       newup := proc_args.upper+1
	       proc_args.conservative_resize(1, newup);
	    end;
	    proc_args.enter(arg, newup)
	end;

feature -- creation

	make(program_file, p_name, lock_file_key: STRING) is
	do
	    -- create the process itself
	    proc_in_action := unix_os.new_process(program_file);
	    if su.not_empty(p_name)
	     then proc_in_action.set_process_name(p_name);
		  proc_name := p_name;
	     else proc_name := program_file;
	    end;
	    -- proc_in_action.set_arguments(proc_args)	-- (done in do_it)
	    proc_in_action.set_input_file_name(unix_os.null_file_name);
	    -- FIX_ME: should output and error be closed?
	    proc_in_action.set_output_file_name(unix_os.null_file_name);
	    proc_in_action.set_error_file_name(unix_os.null_file_name);
	    proc_in_action.set_close_nonstandard_files(true);
	    -- maybe create a lock for testing later
	    if su.not_empty(lock_file_key)
	     then proc_lock := unix_os.new_lock(lock_file_key);
		  proc_lock.set_minimize_retained_resources(true);
	    end;
	    -- keep track of him/her and all the siblings
	    check_the_children;
	end;

end -- PROCESS_CREATION_ACTION