
indexing
	description:
		"List of customer projects and a list of all %
		%the wafer processes that are compatible with every project %
		%in the list"
	author: "David Hollenberg"
	date:	"April 26, 2006"

class PROJECTS_AND_PROCESSES_SET
inherit
	ANY
create
	make

feature {NONE} -- Creation

	make (m, n: INTEGER)
			-- Create with space for `m' projects and `n' processes
			-- (will expand automatically if required)
		do
			create {ARRAYED_LIST [PACKING_PROJECTS_ROW]} projects.make (m)
			create {ARRAYED_LIST [STRING]} processes.make (n)
			processes.compare_objects
		end

feature -- Properties

	projects: LIST [PACKING_PROJECTS_ROW]
			-- Projects currently the set

	processes: LIST [STRING]
			-- Processes with which every element of `projects'
			-- is compatible
		
	is_compatible (p: PACKING_PROJECTS_ROW): BOOLEAN
			-- Is `p' compatible with `Current' (is there at
			-- least one process with which `p' is compatible
			-- that is in `processes')?
		local
			cp: LIST [STRING]
		do
			cp := p.compatible_processes_with_assigned_etest_name
			Result := processes.there_exists (agent cp.has)
		end

feature -- Modification

	add_project (p: PACKING_PROJECTS_ROW)
			-- Add `p' to `projects' and remove from
			-- processes any elements with which `p' is not
			-- compatible
		do
			projects.extend (p)
			remove_incompatible_processes (p.compatible_processes)
		end

feature -- Operations

	add_cheapest_process (list: LIST [STRING])
			-- Add cheapest process in `processes' to end of `list'
		require
			list_not_void: list /= Void
		do
			list.extend (processes.first)
		end

feature {NONE} -- Implementation
	
	remove_incompatible_processes (list: like processes)
			-- Remove from `processes' any elements that
			-- do not occur in `list', which is the list
			-- of compatible processes for a new addition
		local
			new_processes: like processes
		do
			new_processes := processes.twin
			new_processes.wipe_out
			processes.do_all (agent add_process (?, list, new_processes))
			processes := new_processes
		end

	add_process (p: STRING compat_list, dest_list: like processes)
			-- Add `p' to `dest_list' if it is in `compat_list'
		do
			if compat_list.has (p) then
				dest_list.extend (p)
			end
		end

end
