
indexing
	
	description: "[
		Manager that allows publication of events and
		notification of subscribers when event is published.
		]"
	author: "David Hollenberg"
	date: "March 12, 2003"

class PUBLICATION_MANAGER
inherit
	ANY
		redefine
			default_create
		end
	EXCEPTIONS_ACCESS
		redefine
			default_create
		end

feature {NONE} -- Creation

	default_create
		do
			create subscribed_events.make (100)
		end

feature -- Properties

	event_processing_enabled: BOOLEAN
			-- Is processing of published events enabled?

feature -- Modification

	enable_event_processing
			-- Enable event processing
		do
			event_processing_enabled := True
		ensure
			processing_enabled: event_processing_enabled
		end

	disable_event_processing
			-- Disable event processing
		do
			event_processing_enabled := False
		ensure
			processing_disabled: not event_processing_enabled
		end

feature -- Publication

	publish (event: APPLICATION_EVENT)
			-- Publish occurrence of `event'
		require
			event_exists: event /= Void
			event_valid: event.is_valid
		local
			tag: STRING
			actions: LIST [PROCEDURE [ANY, TUPLE [APPLICATION_EVENT]]]
			unsubscribed_event: UNSUBSCRIBED_EVENT_EVENT
		do
			if event_processing_enabled then
				tag := event.generating_type
				actions := subscribed_events.item (tag)
				if actions /= Void then
					from
						actions.start
					until
						actions.after
					loop
						actions.item.call ([event])
						actions.forth
					end
				elseif not tag.is_equal (Unsubscribed_event_type) then
					create unsubscribed_event
					unsubscribed_event.set_event (event)
					unsubscribed_event.publish
				else
					exc.raise ("Unsubscribed event")
				end
			end
		end

feature -- Subscription

	subscribe (event: APPLICATION_EVENT action: PROCEDURE [ANY, TUPLE [APPLICATION_EVENT]])
			-- Subscribe to event `s', so that `action' is
			-- called when event occurs
		require
			event_exists: event /= Void
			action_exists: action /= Void
		local
			tag: STRING
			actions: LIST [PROCEDURE [ANY, TUPLE [APPLICATION_EVENT]]]
		do
			tag := event.generating_type
			actions := subscribed_events.item (tag)
			if actions = Void then
				create {ARRAYED_LIST [PROCEDURE [ANY, TUPLE [APPLICATION_EVENT]]]} actions.make (5)
				actions.compare_objects
				subscribed_events.put (actions, tag)
			end
			actions.extend (action)
		end

	unsubscribe (event: APPLICATION_EVENT action: PROCEDURE [ANY, TUPLE [APPLICATION_EVENT]])
			-- Remove subscription to event `s', so that `action' 
			-- is no longer called when event occurs
		require
			event_exists: event /= Void
			action_exists: action /= Void
		local
			tag: STRING
			actions: LIST [PROCEDURE [ANY, TUPLE [APPLICATION_EVENT]]]
		do
			tag := event.generating_type
			actions := subscribed_events.item (tag)
			if actions /= Void then
				actions.prune_all (action)
			end
		end

feature {NONE} -- Implementation

	subscribed_events: HASH_TABLE [LIST [PROCEDURE [ANY, TUPLE [APPLICATION_EVENT]]], STRING]
			-- Events that have subscribers

	unsubscribed_event_type: STRING = "UNSUBSCRIBED_EVENT_EVENT"
			-- Generating type of "unsubscribed event" event

end
