class RATIONAL
	-- a rational number, in canonical form

inherit
	MORE_DOUBLE_MATH
	redefine  out
	end;

	NUMBER_UTILITIES_ACCESS
	redefine  out
	end;

create
	make,
	make_from_double

feature
	numerator,
	denominator: INTEGER

	in_error: BOOLEAN
		-- indicates that make_from_double failed

feature	-- status

	is_reduced: BOOLEAN is
	do  Result := nu.gcd(numerator, denominator) = 1
	end;

	reduce is
		-- reduce 
	local  d: INTEGER
	do  d := nu.gcd(numerator, denominator);
	    if d /= 1
	     then numerator := numerator // d
		  denominator := denominator // d
	    end;
	ensure is_reduced
	end;

	double: DOUBLE is
		-- my double value
	do  Result := numerator
	    Result := Result / denominator
	end;

	out: STRING is
		-- "<numerator>/<denominator>"
	do  Result := numerator.out;
	    Result.extend('/');
	    Result.append(denominator.out)
	end;

feature -- creation

	make(num, den: INTEGER) is
	require den /= 0
	do  if den > 0
	     then numerator := num;
		  denominator := den
	     else numerator := -num;
		  denominator := -den
	    end
	end;

	make_from_double(val: DOUBLE; err: INTEGER) is
		-- a rational number ~ val.
		-- err is maximum relative error allowed as multiple of
		--  (MORE_DOUBLE_MATH) epsilon
		-- for now, limit denominator to powers of 10
	require err > 0
	local v, eps: DOUBLE
	do  v := val.abs;
	    eps := err * epsilon * v;
	    from  denominator := 1
		  numerator := v.rounded
	    until (double - v).abs <= eps or
			-- OK
		  (denominator = 1000000000 or numerator > 200000000)
			-- won't make it around another time!!
	    loop  denominator := denominator * 10
		  numerator := (v*denominator).rounded
	    end;
	    in_error := (double - v).abs > eps;
	    if val < 0 then numerator := - numerator end;
	end;

invariant
	denominator > 0

end -- RATIONAL
