
indexing
	
	description: "Window to allow viewing of bonding diagrams for %
		%a specific run or for a design"
	author: "David Hollenberg"
	date: "February 23, 2005"

class RC_BONDING_WINDOW

inherit
	RC_BONDING_IFC
		redefine
			print_interface
		end
	WINDOW_INTERFACE
		redefine
			enable_active,
			disable_active
		end
	MOSIS_GRAPHICAL_UTILITIES
		export
			{NONE} all
		undefine
			default_create
		end
	RC_ENVIRONMENT_ACCESS
		export
			{NONE} all
		undefine
			default_create
		end
	MEMORY
		export
			{NONE} all
		undefine
			default_create
		end
		
create
	make

feature {NONE} -- Creation

	make (w: EV_TITLED_WINDOW)
			-- Create interface in window `w'
		require
			window_exists: w /= Void
			window_valid: not w.is_destroyed
		do
			make_interface
			make_proxy (w)
		end

feature -- Operations

	reset
			-- Move focus back into window
		do
			enable_active
			-- center_pointer
		end

feature -- Modification

	enable_active
			-- Make interface active
		do
			precursor
			cancel_button.enable_sensitive
		end

	disable_active
			-- Make interface inactive
			-- (can look but cannot initiate most actions)
		do
			precursor
			cancel_button.disable_sensitive
		end

feature {NONE} -- Implementation

	display_button: EV_BUTTON
			-- Button to query database for list of packagings
			-- for which to display diagrams and display first one
	
	clear_button: EV_BUTTON
			-- Button to clear all fields
	
	draw_button: EV_BUTTON
			-- Button to redraw current diagram (not zoomed)
	
	zoom_in_button: EV_BUTTON
			-- Button to zoom in
	
	zoom_out_button: EV_BUTTON
			-- Button to zoom out
	
	prev_button: EV_BUTTON
			-- Button to draw previous diagram
	
	next_button: EV_BUTTON
			-- Button to next next diagram
	
	cancel_button: EV_BUTTON
			-- Button to end session
	
	build_interface
			-- Build interface
		local
			buttons: HORIZONTAL_BUTTON_SET
			main_vbox: EV_VERTICAL_BOX
			box: EV_BOX
			frame: EV_FRAME
		do
			add_menu_bar

			create main_vbox
			main_vbox.set_border_width (user_interface.border_width)
			build_fields
			main_vbox.extend (fields_box)
			main_vbox.disable_item_expand (fields_box)

			create buttons.make_with_text_and_actions (<<
				"Display", "Clear", "Draw", "Zoom In", "Zoom Out", "Prev", "Next", "Cancel" >>,
				<< agent process_display, agent process_clear, agent process_draw, agent process_zoom_in, agent process_zoom_out, agent process_prev, agent process_next, agent process_cancel >>)
			display_button := buttons.i_th (1)
			clear_button := buttons.i_th (2)
			draw_button := buttons.i_th (3)
			zoom_in_button := buttons.i_th (4)
			zoom_out_button := buttons.i_th (5)
			prev_button := buttons.i_th (6)
			next_button := buttons.i_th (7)
			cancel_button := buttons.i_th (8)
			
			display_button.disable_sensitive
			draw_button.disable_sensitive
			zoom_in_button.disable_sensitive
			zoom_out_button.disable_sensitive
			prev_button.disable_sensitive
			next_button.disable_sensitive
			
			box := buttons.column_set.box
			
			main_vbox.extend (box)
			main_vbox.disable_item_expand (box)
			
			create diagram_area
			diagram_area.set_background_color (gui.White)
			set_new_pixmap (Initial_pixmap_size_x, Initial_pixmap_size_y)
			create frame
			frame.extend (diagram_area)
			main_vbox.extend (frame)
			
			gui.set_box_background_recursive (main_vbox, background_color)
			extend (main_vbox)
			gui.add_cancel_accelerator (window, agent process_cancel)
			gui.do_once_on_idle (agent post_display_actions)
		end
		
	add_menu_bar
			-- Add menu bar to window
		local
			file_menu: EV_MENU
			exit_menu_item: EV_MENU_ITEM
			close_menu_item: EV_MENU_ITEM
			print_menu_item: EV_MENU_ITEM
			a_menu_bar: EV_MENU_BAR
		do	
			create file_menu.make_with_text ("File")
			
			create print_menu_item.make_with_text ("Print")
			print_menu_item.select_actions.extend (agent process_print (print_interface))
			file_menu.extend (print_menu_item)
			
			create close_menu_item.make_with_text ("Close")
			close_menu_item.select_actions.extend (agent process_cancel)
			file_menu.extend (close_menu_item)
			
			create exit_menu_item.make_with_text ("Exit")
			exit_menu_item.select_actions.extend (gui.destroy_application_agent)
			file_menu.extend (exit_menu_item)
			
			create a_menu_bar
			a_menu_bar.extend (file_menu)
			set_menu_bar (a_menu_bar)
		end

	build_fields
			-- Build portion of window with fields
			-- design number, run id and die id and leave
			-- in `fields_box'
		local
			cols: WIDGET_COLUMN_SET
		do
			create cols.make (12)
			cols.set_padding (user_interface.padding)
			cols.set_border_width (user_interface.border_width)
			
			create design_no_field
			design_no_field.set_minimum_width_in_characters (6)
			design_no_field.change_actions.extend (agent process_change)
			design_no_field.return_actions.extend (agent process_return)
			
			create run_id_field
			run_id_field.enable_uppercase_only
			run_id_field.set_minimum_width_in_characters (4)
			run_id_field.change_actions.extend (agent process_change)
			run_id_field.return_actions.extend (agent process_return)
			
			create production_id_field
			production_id_field.enable_uppercase_only
			production_id_field.set_minimum_width_in_characters (9)
			production_id_field.change_actions.extend (agent process_change)
			production_id_field.return_actions.extend (agent process_return)
			
			create die_id_field
			die_id_field.enable_uppercase_only
			die_id_field.set_minimum_width_in_characters (2)
			die_id_field.change_actions.extend (agent process_change)
			die_id_field.return_actions.extend (agent process_return)
			
			
			create package_id_field
			package_id_field.enable_uppercase_only
			package_id_field.set_minimum_width_in_characters (10)
			package_id_field.change_actions.extend (agent process_change)
			package_id_field.return_actions.extend (agent process_return)
			
			create bonding_id_field
			bonding_id_field.enable_uppercase_only
			bonding_id_field.set_minimum_width_in_characters (2)
			bonding_id_field.change_actions.extend (agent process_change)
			bonding_id_field.return_actions.extend (agent process_return)
			
			cols.append_row (<< gui.label_with_text ("Design No"), design_no_field, gui.label_with_text ("Run ID"), run_id_field, gui.label_with_text ("Prod ID"), production_id_field, gui.label_with_text ("Die ID"), die_id_field, gui.label_with_text ("Package ID"), package_id_field, gui.label_with_text ("Bonding ID"), bonding_id_field >>)
			cols.disable_expand
			cols.align_center
			fields_box := cols.box
		end
		
	set_new_pixmap (size_x, size_y: INTEGER)
			-- Create `pixmap' of size (`size_x', `size_y')
			-- and put it in `diagram_area'
		do
			create pixmap.make_with_size (size_x, size_y)
			diagram_area.replace (pixmap)
		end

	post_display_actions
		do
			gui.do_once_on_idle (agent gui.center_window_with_margin (window, 100, 0))
			reset
		end

	process_change
		local
			dno, run, prod: STRING
		do
			if not text_changed_by_program then
				dno := design_no_field.text
				run := run_id_field.text
				prod := production_id_field.text
				if (run.is_empty and prod.is_empty and dno.is_empty) or (not dno.is_empty and then not dno.is_integer) then
					display_button.disable_sensitive
				else
					display_button.enable_sensitive
				end
				draw_button.disable_sensitive
				zoom_in_button.disable_sensitive
				zoom_out_button.disable_sensitive
				prev_button.disable_sensitive
				next_button.disable_sensitive
			end
		end

	process_return
		do
			if display_button.is_sensitive then
				process_display
			end
		end

	display_current_packaging
			-- Display the current packaging
		do
			set_button_sensitivity
			fill_in_fields
			generate_bonding_diagram
			display_current_diagram
		end

	set_button_sensitivity
			-- Set sensitivity of Next and Prev buttons
		do
			if position = 1 then
				prev_button.disable_sensitive
			else
				prev_button.enable_sensitive
			end
			if position = packagings.count then
				next_button.disable_sensitive
			else
				next_button.enable_sensitive
			end
		end

	fill_in_fields
			-- Fill in fields with information for
			-- the current packaging
		local
			ifi: IC_FAB_INSTANCES_ROW
		do
			text_changed_by_program := True
			create ifi.make_by_key (session.connection, option.Old_no_exception_row, packaging.m_fab_no)
			if not ifi.is_empty then
				design_no_field.set_text (ifi.design_no.out)
				run_id_field.set_text (ifi.run_id)
				production_id_field.set_text (packaging.production_id)
				die_id_field.set_text (ifi.die_id)
			else
				design_no_field.set_text ("")
				run_id_field.set_text ("")
				production_id_field.set_text ("")
				die_id_field.set_text ("")
			end
			package_id_field.set_text (packaging.package_id)
			bonding_id_field.set_text (packaging.bonding_id)
			text_changed_by_program := False
		end

	generate_bonding_diagram
			-- Generate bonding diagram for current packaging
		local
			gen: BONDING_DIAGRAM_GENERATOR
			diagram: BONDING_DIAGRAM
		do
			cached_diagrams.search (packaging.unique_id)
			if cached_diagrams.found then
				diagram := cached_diagrams.found_item
			else
				create gen.make (session.connection)
				diagram := gen.vendor_bonding_diagram (packaging.ic_packagings_row, packaging.production_id)
				diagram.set_drawing_rotatable (False)
				cached_diagrams.put (diagram, packaging.unique_id)
			end
			if diagram.last_document = Void then
				diagram.generate_document
			end
			current_document := diagram.last_document
		end

	display_current_diagram
			-- Display bonding diagram for current packaging
		local
			projector: DRW_EV_PROJECTOR
		do
			create projector.make (pixmap)
			projector.project (current_document)
		end

	process_display
		do
			if is_active then
				disable_active
				validate_fields
				if is_valid then
					fetch_packagings
				end
				if is_valid then
					create cached_diagrams.make (packagings.count)
					packagings.start
					position := 1
					packaging := packagings.item
					set_new_pixmap (Initial_pixmap_size_x, Initial_pixmap_size_y)
					zero_zoom_count
					display_current_packaging
					draw_button.enable_sensitive
				end
				enable_active
			end
		end

	process_draw
		do
			if is_active then
				disable_active
				if zoom_count /= 0 then
					set_new_pixmap (Initial_pixmap_size_x, Initial_pixmap_size_y)
					zero_zoom_count
				end
				display_current_packaging
				enable_active
			end
		end

	process_zoom_in
		local
			x, y, w, h: INTEGER
		do
			if is_active then
				disable_active
				save_displayed_position
				x := diagram_area.x_offset
				y := diagram_area.y_offset
				w := pixmap.width
				h := pixmap.height
				increment_zoom_count
				set_new_pixmap (zoomed_pixmap_x, zoomed_pixmap_y)
				collect
				display_current_diagram
				restore_displayed_position
				enable_active
			end
		end

	process_zoom_out
		do
			if is_active then
				disable_active
				save_displayed_position
				decrement_zoom_count
				set_new_pixmap (zoomed_pixmap_x, zoomed_pixmap_y)
				collect
				display_current_diagram
				restore_displayed_position
				enable_active
			end
		end

	process_clear
		do
			if is_active then
				disable_active
				design_no_field.set_text ("")
				run_id_field.set_text ("")
				production_id_field.set_text ("")
				die_id_field.set_text ("")
				package_id_field.set_text ("")
				bonding_id_field.set_text ("")
				pixmap.clear
				enable_active
			end
		end

	process_prev
		do
			if is_active then
				disable_active
				packagings.back
				position := position - 1
				packaging := packagings.item
				display_current_packaging
				enable_active
			end
		end

	process_next
		do
			if is_active then
				disable_active
				packagings.forth
				position := position + 1
				packaging := packagings.item
				display_current_packaging
				enable_active
			end
		end

	process_cancel
		do
			if is_active then
				disable_active
				cancel_actions.call ([])
			end
		end

	process_print (ifc: BOND_DIAGRAM_LIST_PRINT_WINDOW)
			-- Process request to print diagrams via
			-- interface `ifc'
		require
			ifc_not_void: ifc /= Void
		local
			list: like packagings
		do
			if packagings /= Void then
				list := packagings
			else
				create {ARRAYED_LIST [IC_PACKAGING_AND_WO]} list.make (0)
			end
			ifc.set_packagings (list)
			if not ifc.is_built then
				ifc.build
				ifc.cancel_actions.extend (agent process_print_cancel (ifc))
				ifc.show
			elseif not ifc.is_displayed then
				ifc.refresh
				ifc.show
			elseif ifc.is_minimized then
				ifc.refresh
				ifc.restore
			else
				ifc.raise
			end
		end

	process_print_cancel (ifc: RC_MAP_PRINT_WINDOW)
			-- Process a cancel request on print interface window
		require
			ifc_not_void: ifc /= Void
		do
			ifc.hide
			ifc.enable_active
		end

	validate_fields
			-- Check whether fields are valid, individually
			-- and in combination.
			-- Set `is_valid' to reflect the answer and
			-- Also, report user error if not valid
		local
			dno, run, prod, die, pkg, bnd, prod_run_id: STRING
			the_run_id: STRING
			cid_row: CUST_IC_DESIGNS_ROW
			runs_row: RUNS_ROW
			prods_row: PRODUCTIONS_ROW
			pkgs_row: PACKAGES_ROW
			ifi_row: IC_FAB_INSTANCES_ROW
			dno_val: INTEGER
			cid_rows: LIST [CUST_IC_DESIGNS_ROW]
			pkgs: LIST [PACKAGES_ROW]
			ifi_rows: LIST [IC_FAB_INSTANCES_ROW]
		do
			is_valid := True
			dno := design_no_field.text
			run := run_id_field.text
			prod := production_id_field.text
			die := die_id_field.text
			pkg := package_id_field.text
			bnd := bonding_id_field.text
			
			if not dno.is_empty then
				dno_val := dno.to_integer
				create cid_row.make_template
				cid_rows := cid_row.rows_with_design_no (dno_val, session.connection, expected_count.At_most_one)
				is_valid := not cid_rows.is_empty
				if not is_valid then
					inform ("Design number " + dno + " not found")
				end
			end
			
			if is_valid and not run.is_empty then
				create runs_row.make_by_key (session.connection, option.Old_no_exception_row, run)
				is_valid := not runs_row.is_empty
				if not is_valid then
					inform ("Run " + run + " not found")
				end
			end
			
			if is_valid and not prod.is_empty then
				create prods_row.make_by_key (session.connection, option.Old_no_exception_row, prod)
				is_valid := not prods_row.is_empty
				if not is_valid then
					inform ("Production " + prod + " not found")
				else
					prod_run_id := prods_row.run_id
				end
			end
			
			if is_valid and not pkg.is_empty then
				create pkgs_row.make_template
				pkgs := pkgs_row.packages_with_id (pkg, session.connection, expected_count.At_most_one)
				is_valid := not pkgs.is_empty
				if not is_valid then
					inform ("Package " + pkg + " not found")
				end
			end
			
			if is_valid then
				if not run.is_empty then
					the_run_id := run
				elseif not prod.is_empty then
					the_run_id := prod_run_id
				else
					the_run_id := ""
				end
			end
			
			if is_valid and not run.is_empty and not prod.is_empty then
				is_valid := prod_run_id.is_equal (run)
				if not is_valid then
					inform ("Production " + prod + " exists, but it is for run " + prod_run_id + " (not " + run + ")")
				end
			end
			
			if is_valid and (not run.is_empty or not prod.is_empty) and not die.is_empty then
				create ifi_row.make_template
				ifi_row := ifi_row.row_for_run_and_die (the_run_id, die, session.connection, expected_count.At_most_one)
				is_valid := ifi_row /= Void
				if not is_valid then
					inform ("Run " + the_run_id + " exists, but die " + die + " not found")
				end
			end
			
			if is_valid and not dno.is_empty and (not run.is_empty or not prod.is_empty) then
				dno_val := dno.to_integer
				create ifi_row.make_template
				ifi_rows := ifi_row.rows_for_design_no (dno_val, session.connection, expected_count.At_least_zero)
				is_valid := ifi_rows.there_exists (agent has_run_id (?, the_run_id))
				if not is_valid then
					inform ("Design number " + dno + " is not on run " + the_run_id)
				elseif not die.is_empty then
					is_valid := ifi_rows.there_exists (agent has_run_and_die (?, the_run_id, die))
					if not is_valid then
						inform ("Design number " + dno + " is on run " + the_run_id + " but is not die " + die)
					end
				end
				
			end
		end

	fetch_packagings
			-- Fetch IC packagings that meet user's
			-- constraints and put in `packagings'.  Set
			-- `is_valid' to indicate whether there are
			-- any and report error if none
		local
			dno, run, prod, die, pkg, bnd: STRING
			pkgings_row: IC_PACKAGINGS_ROW
		do
			is_valid := True
			dno := design_no_field.text
			run := run_id_field.text
			prod := production_id_field.text
			die := die_id_field.text
			pkg := package_id_field.text
			bnd := bonding_id_field.text
			
			create pkgings_row.make_template
			ic_packaging_rows := pkgings_row.rows_for_complex_criteria (dno, run, prod, die, pkg, bnd, session.connection, expected_count.At_least_zero)
			build_packagings
			filter_packagings
			is_valid := not packagings.is_empty
			if not is_valid then
				inform ("No packagings meet your criteria")
			end
		end

	build_packagings
			-- Build `packagings' from `ic_packaging_rows',
			-- using constraining production ID, if any
		local
			row: IC_PACKAGINGS_ROW
			wo: CUST_WOS_ROW
			wos: LIST [CUST_WOS_ROW]
			p: IC_PACKAGING_AND_WO
		do
			create {ARRAYED_LIST [IC_PACKAGING_AND_WO]} packagings.make (ic_packaging_rows.count)
			from
				ic_packaging_rows.start
			until
				ic_packaging_rows.after
			loop
				row := ic_packaging_rows.item
				create wo.make_template
				wos := wo.orders_for_fab_no_prod_and_pkging_no (row.m_fab_no, production_id_field.text, row.ic_packaging_no, session.connection, expected_count.At_least_zero)
				from
					wos.start
				until
					wos.after
				loop
					create p.make (row, wos.item)
					packagings.extend (p)
					wos.forth
				end
				ic_packaging_rows.forth
			end
		end

	filter_packagings
			-- Remove from `packagings' all rows which
			-- have a null package id and for which there
			-- is another row with the same m_fab_no and
			-- production ID that has a non-null package
			-- id
		local
			row: IC_PACKAGING_AND_WO
			table: HASH_TABLE [BOOLEAN, STRING]
			list: like packagings
			ok: BOOLEAN
		do
			create table.make (packagings.count)
			from
				packagings.start
			until
				packagings.after
			loop
				row := packagings.item
				if not row.is_null_package_id then
					table.force (True, row.fab_no_prod_id)
				end
				packagings.forth
			end
			
			list := packagings.twin
			list.wipe_out
	
			from
				packagings.start
			until
				packagings.after
			loop
				row := packagings.item
				ok := not row.is_null_package_id or not table.has (row.fab_no_prod_id)
				if ok then
					list.extend (row)
				end
				packagings.forth
			end
			
			packagings := list
		end

	has_run_id (row: IC_FAB_INSTANCES_ROW run: STRING): BOOLEAN
			-- Is `row' on run id `run'?
		do
			Result := row.run_id.is_equal (run)
		end

	has_run_and_die (row: IC_FAB_INSTANCES_ROW run, die: STRING): BOOLEAN
			-- Is `row' on run id `run' as die id `die'?
		do
			Result := row.run_id.is_equal (run) and row.die_id.is_equal (die)
		end

	zero_zoom_count
			-- Set `zoom_count' to 0
		do
			zoom_count := 0
			zoom_in_button.enable_sensitive
			zoom_out_button.enable_sensitive
		end

	increment_zoom_count
			-- Increment `zoom_count'
		do
			if zoom_count = Min_zoom_count then
				zoom_out_button.enable_sensitive
			end
			if zoom_count < Max_zoom_count then
				zoom_count := zoom_count + 1
			end
			if zoom_count = Max_zoom_count then
				zoom_in_button.disable_sensitive
			end
		end

	decrement_zoom_count
			-- Decrement `zoom_count'
		do
			if zoom_count = Max_zoom_count then
				zoom_in_button.enable_sensitive
			end
			if zoom_count > Min_zoom_count then
				zoom_count := zoom_count - 1
			end
			if zoom_count = Min_zoom_count then
				zoom_out_button.disable_sensitive
			end
		end

	save_displayed_position
		do
			saved_x_offset := diagram_area.x_offset
			saved_y_offset := diagram_area.y_offset
			saved_pixmap_width := pixmap.width
			saved_pixmap_height := pixmap.height
			saved_area_client_width := diagram_area.client_width
			saved_area_client_height := diagram_area.client_height
		end

	restore_displayed_position
		local
			f: DOUBLE
			half_w, half_h: DOUBLE
		do
			if pixmap.width > saved_area_client_width then
				half_w := saved_area_client_width / 2.0
				f := (saved_x_offset + half_w) / saved_pixmap_width
				diagram_area.set_x_offset ((f * pixmap.width - half_w).truncated_to_integer)
			end
			if pixmap.height > saved_area_client_height then
				half_h := saved_area_client_height / 2.0
				f := (saved_y_offset + half_h) / saved_pixmap_height
				diagram_area.set_y_offset ((f * pixmap.height - half_h).truncated_to_integer)
			end
		end

	zoomed_pixmap_x: INTEGER
			-- X size of pixmap to use for `zoom_count'
		do
			Result := (Initial_pixmap_size_x * (Zoom_scale_factor ^ zoom_count)).truncated_to_integer
		end

	zoomed_pixmap_y: INTEGER
			-- Y size of pixmap to use for `zoom_count'
		do
			Result := (Initial_pixmap_size_y * (Zoom_scale_factor ^ zoom_count)).truncated_to_integer
		end

	is_valid: BOOLEAN
			-- Do fields specify an existing design number?

	ic_packaging_rows: LIST [IC_PACKAGINGS_ROW]
			-- Packagings that meet last valid user-specified
			-- criteria (without associated cust work order)

	packagings: LIST [IC_PACKAGING_AND_WO]
			-- Packagings that meet last valid user-specified
			-- criteria (with associated cust work order)

	packaging: IC_PACKAGING_AND_WO
			-- Current packaging to display

	production_id: STRING
			-- Current production ID for which to display packaging

	position: INTEGER
			-- Position of current packaging in `packagings'
			-- (changed by Next and Prev)

	text_changed_by_program: BOOLEAN
			-- Was last change to text field done by program
			-- rather than by the user?

	fields_box: EV_BOX
			-- Box with fields for run id, etc.

	design_no_field: TEXT_FIELD
			-- Field for constraining design number

	run_id_field: TEXT_FIELD
			-- Field for constraining run id

	production_id_field: TEXT_FIELD
			-- Field for constraining production id

	die_id_field: TEXT_FIELD
			-- Field for constraining die id

	package_id_field: TEXT_FIELD
			-- Field for constraining package id

	bonding_id_field: TEXT_FIELD
			-- Field for constraining bonding id

	diagram_area: EV_SCROLLABLE_AREA
			-- Scrollable area for displaying bonding diagrams
		
	pixmap: EV_PIXMAP
			-- Pixmap for displaying bonding diagrams

	label: EV_LABEL
			-- For construction of labels

	cached_diagrams: HASH_TABLE [BONDING_DIAGRAM, STRING]
			-- Cache of bonding diagrams, indexed by
			-- unique ID (combination of packaging_no and
			-- production_id)

	current_document: DRW_DOCUMENT
			-- Document generated for current bonding diagram

	Initial_pixmap_size_x: INTEGER = 1000
			-- Initialize size of `pixmap' in X

	Initial_pixmap_size_y: INTEGER = 1000
			-- Initialize size of `pixmap' in X

	Zoom_scale_factor: DOUBLE = 1.2
			-- Amount to scale by for zoom

	zoom_count: INTEGER
			-- Number of times user zoomed in (if positive)
			-- or out (if negative) from initial size

	Min_zoom_count: INTEGER = -20
			-- Minimum allowed value for `zoom_count'

	Max_zoom_count: INTEGER = 10
			-- Minimum allowed value for `zoom_count'

	saved_x_offset: INTEGER
			-- Last saved value of `x_offset' for `diagram_area'

	saved_y_offset: INTEGER
			-- Last saved value of `y_offset' for `diagram_area'

	saved_pixmap_width: INTEGER
			-- Last saved value of `pixmap'.`width'

	saved_pixmap_height: INTEGER
			-- Last saved value of `pixmap'.`height'

	saved_area_client_width: INTEGER
			-- Last saved value of `diagram_area'.`client_width'

	saved_area_client_height: INTEGER
			-- Last saved value of `diagram_area'.`client_height'

	print_interface: BOND_DIAGRAM_LIST_PRINT_WINDOW
			-- Window to print bonding diagrams

end
