
indexing
	
	description: "Configuration parameters for run closing"
	author: "David Hollenberg"
	date: "June 20, 2002"

class RC_CONFIGURATION_VARS

inherit
	ANY
		undefine
			is_equal, copy
		end
	HASHED_CONFIGURATION_VARS
		export
			{ANY} is_equal, copy, configuration_errors,
				initialize_from_args, item
			{HASHED_CONFIGURATION_VARS} all
		end
create
	make

feature -- Parameters

	fixed_display_name: STRING
			-- Set DISPLAY environment variable to this
			-- value if not empty
		once
			Result := item (Parm_fixed_display_name)
		end
	
	db_conn_str: STRING
			-- Database connect string (service name)
		once
			Result := item (Parm_db_conn_str)
		end
	
	oracle_home: STRING
			-- Oracle home directory
		once
			Result := item (Parm_oracle_home)
		end
	
	oracle_reports_home: STRING
			-- Oracle home directory for running Oracle
			-- report writer
		once
			Result := item (Parm_oracle_reports_home)
		end
	
	tns_admin: STRING
			-- TNS (Transparent Network Susbtrate) admin directory.
			-- Used to locate tnsnames.ora file that describes
			-- services
		once
			Result := item (Parm_tns_admin)
		end
	
	sqlplus_exe_file: STRING
			-- Name of executable file to run SQL*Plus
		once
			Result := item (Parm_sqlplus_exe_file)
		end
	
	melt_path: STRING
			-- Directory where Eiffel file with melted code
			-- (for a frozen application) is located
		once
			Result := item (Parm_melt_path)
		end
	
	real_run_dir_base: STRING
			-- Directory containing real run directories
		once
			Result := item (Parm_real_run_dir_base)
		end
	
	real_probing_dir_base: STRING
			-- Directory containing real run directories
			-- for probing purposes
		once
			Result := item (Parm_probing_run_dir_base)
		end
	
	trial_run_dir_base: STRING
			-- Directory containing trial run directories
		once
			Result := item (Parm_trial_run_dir_base)
		end
	
	trial_probing_dir_base: STRING
			-- Directory containing trial run directories
			-- for probing purposes
		once
			Result := item (Parm_trial_probing_dir_base)
		end
	
	lock_dir: STRING
			-- Directory for run lock files
		once
			Result := item (Parm_lock_dir)
		end
	
	delta_lock_root: STRING
			-- Root name of lock file that allows running of
			-- Delta, without directory or file extension
		once
			Result := item (Parm_delta_lock_root)
		end
	
	shared_lock_root: STRING
			-- Root name of shared lock file, without directory
			-- or file extension
		once
			Result := item (Parm_shared_lock_root)
		end
	
	lock_suffix: STRING
			-- Suffix of lock file for access control
		once
			Result := item (Parm_lock_suffix)
		end
	
	exclusive_delta_lock: BOOLEAN
			-- Should we get lock on running Delta in exclusive
			-- mode, to lock out all other executions of Delta?
		once
			Result := as_boolean (item (Parm_exclusive_delta_lock))
		end
	
	log_dir: STRING
			-- Directory for log files (including error logs)
		once
			Result := item (Parm_log_dir)
		end
	
	log_suffix: STRING
			-- Suffix of file for logging information
		once
			Result := item (Parm_log_suffix)
		end
	
	error_suffix: STRING
			-- Suffix of file for recording errors
		once
			Result := item (Parm_error_suffix)
		end
	
	temp_dir: STRING
			-- Directory for temporary files
		once
			Result := item (Parm_temp_dir)
		end
	
	delta_version_file: STRING
			-- Name of file that contains information about
			-- latest version of Delta
		once
			Result := item (Parm_delta_version_file)
		end
	
	std_ghostscript_program: STRING
			-- Name of file with GhostScript program in
			-- standard location
		once
			Result := item (Parm_std_ghostscript_program)
		end
	
	alt_ghostscript_program: STRING
			-- Name of file with GhostScript program in
			-- alternate location
		once
			Result := item (Parm_alt_ghostscript_program)
		end
	
	std_acroread_program: STRING
			-- Name of file with Acrobat reader program
			-- in standard location
		once
			Result := item (Parm_std_acroread_program)
		end
	
	alt_acroread_program: STRING
			-- Name of file with Acrobat reader program
			-- in alternate location
		once
			Result := item (Parm_alt_acroread_program)
		end
	
	print_program: STRING
			-- Name of file with program to print files
		once
			Result := item (Parm_print_program)
		end
	
	cat_program: STRING
			-- Name of file with program to cat files
		once
			Result := item (Parm_cat_program)
		end
	
	rshell_program: STRING
			-- Name of file with program to rshell to another host
		once
			Result := item (Parm_rshell_program)
		end
	
	phone_program: STRING
			-- Name of file with program to initiate a
			-- telephone call
		once
			Result := item (Parm_phone_program)
		end
	
	cif_to_dxf_program: STRING
			-- Name of file that converts CIF to DXF
		once
			Result := item (Parm_cif_to_dxf_program)
		end
	
	cif_to_dxf_host: STRING
			-- Name of host on which CIF to DXF conversion
			-- program must be run
		once
			Result := item (Parm_cif_to_dxf_host)
		end
	
	run_as_user_program: STRING
			-- Name of file that can run a program as
			-- a different user
		once
			Result := item (Parm_run_as_user_program)
		end
	
	front_end_user: STRING
			-- Name of operating system user front end
			-- runs under
		once
			Result := item (Parm_front_end_user)
		end
	
	printer_environment_var: STRING
			-- Name of environment variable with default
			-- printer for maps
		once
			Result := item (Parm_printer_environment_var)
		end
	
	reportwriter_program: STRING
			-- Name of file with program to run ReportWriter
			-- report
		once
			Result := item (Parm_reportwriter_program)
		end
	
	gds_merge_parm_file: STRING
			-- Name of parameter file for program to create merged GDS
			-- file for one die (that is a project) or one field
		once
			Result := item (Parm_gds_merge_parm_file)
		end
	
	delta_program: STRING
			-- Name of file with program to run Delta
			-- (possibly a shell script)
		once
			Result := item (Parm_delta_program)
		end
	
	gds_merge_program: STRING
			-- Name of file with program to create merged GDS
			-- file for one die (that is a project) or one field
		once
			Result := item (Parm_gds_merge_program)
		end
	
	level_merge_program: STRING
			-- Name of file with program to merge
			-- pattern files for one level
		once
			Result := item (Parm_level_merge_program)
		end
	
	level_fill_program: STRING
			-- Name of file with program to fill
			-- merged pattern file for one level
		once
			Result := item (Parm_level_fill_program)
		end
	
	fill_selector_program: STRING
			-- Name of file with program to create fill
			-- selector file for runs that need fill added
		once
			Result := item (Parm_fill_selector_program)
		end
	
	probing_files_program: STRING
			-- Name of file with program to create additional
			-- probing files (.pts files)
		once
			Result := item (Parm_probing_files_program)
		end
	
	wafer_techn_survey_script: STRING
			-- Name of SQL*Plus script file that produces a survey
			-- of a single wafer techn
		once
			Result := item (Parm_wafer_techn_survey_script)
		end
	
	project_summary_report: STRING
			-- Name of report module that produces project summary
			-- report
		once
			Result := item (Parm_project_summary_report)
		end
	
	line_item_report: STRING
			-- Name of report module that produces line item
			-- report
		once
			Result := item (Parm_line_item_report)
		end
	
	payload_and_charges_report: STRING
			-- Name of report module that produces payload and
			-- charges report
		once
			Result := item (Parm_payload_and_charges_report)
		end
	
	costs_and_margin_report: STRING
			-- Name of report module that produces costs and
			-- margin report
		once
			Result := item (Parm_costs_and_margin_report)
		end
	
	sales_order_report: STRING
			-- Name of report module that produces sales order
			-- report
		once
			Result := item (Parm_sales_order_report)
		end
	
	all_runs_status_report: STRING
			-- Name of report module that produces web page
			-- with links to status for all runs
		once
			Result := item (Parm_all_runs_status_report)
		end
	
	run_status_report: STRING
			-- Name of report module that produces web page
			-- that shows status of one run
		once
			Result := item (Parm_run_status_report)
		end
	
	foundry_eccn_report: STRING
			-- Name of report module that produces Export
			-- Control report to be sent to foundry
		once
			Result := item (Parm_foundry_eccn_report)
		end
	
	mailer_command: STRING
			-- Command to send mail, including any options
		once
			Result := item (Parm_mailer_command)
		end

	from_line: STRING
			-- From line to be included in header when send e-mail
		once
			Result := item (Parm_from_line)
		end

	crash_msg_to_list: STRING
			-- Address list for crash message
			-- (empty means don't send mail)
		once
			Result := item (Parm_crash_msg_to_list)
		end
	
	unused_options_to_list: STRING
			-- Address list for warnings about unused options
			-- in wafer processes for a run
		once
			Result := item (Parm_unused_options_to_list)
		end
	
	unresolved_issue_to_list: STRING
			-- Address list for unresolved issues with projects,
			-- such as policy violations (e.g., projects without
			-- funding allocation) and customer satisifaction
			-- issues (e.g., order doesn't match what customer
			-- requested)
		once
			Result := item (Parm_unresolved_issue_to_list)
		end
	
	level_density_to_list: STRING
			-- Address list for messages that list density
			-- (both before and after fill) of levels with
			-- a minimum density requirement
		once
			Result := item (Parm_level_density_to_list)
		end
	
	probing_files_to_list: STRING
			-- Address list for messages that report
			-- information about generated probing files
		once
			Result := item (Parm_probing_files_to_list)
		end
	
	test_vehicles_to_list: STRING
			-- Address list for messages that report
			-- test vehicles on the run after packing
		once
			Result := item (Parm_test_vehicles_to_list)
		end
	
	export_control_to_list: STRING
			-- Address list for messages that report
			-- things related to export control
		once
			Result := item (Parm_export_control_to_list)
		end
	
	gds_merge_to_list: STRING
			-- Address list for messages that report
			-- details of files created by GDS merge
		once
			Result := item (Parm_gds_merge_to_list)
		end
	
	gds_merge_error_to_list: STRING
			-- Address list for messages that report
			-- possible problems with GDS merge
		once
			Result := item (Parm_gds_merge_error_to_list)
		end
	
	scmos_conversion_to_list: STRING
			-- Address list for messages that report
			-- on projects that need to be converted
			-- from SCMOS rules to native rules prior
			-- to GDS merge
		once
			Result := item (Parm_scmos_conversion_to_list)
		end
	
	run_posted_to_list: STRING
			-- Address list for messages that report
			-- that run has been posted (layout files sent
			-- to foundry for review)
		once
			Result := item (Parm_run_posted_to_list)
		end
	
	run_rtm_to_list: STRING
			-- Address list for messages that report
			-- that run has been released to manufacturing
		once
			Result := item (Parm_run_rtm_to_list)
		end
	
	mask_invoice_to_list: STRING
			-- Address list for messages that report
			-- that masks should be invoiced for run
		once
			Result := item (Parm_mask_invoice_to_list)
		end
	
	is_production: BOOLEAN
			-- Is this production (as opposed to test)?
		once
			Result := as_boolean (item (Parm_is_production))
		end
	
	is_graphical: BOOLEAN
			-- Are we running as a graphical application?
		once
			Result := as_boolean (item (Parm_is_graphical))
		end
	
	background_color: STRING
			-- Requested background color for most windows
			-- (should be 3 real numbers in range 0.0 to 1.0,
			-- separated by white space)
		once
			Result := item (Parm_background_color)
		end
	
	cache_auto_flush_interval: INTEGER
			-- Interval in seconds for automatic flush of
			-- run cache when there is no step activity,
			-- if auto-flush is enabled
		once
			Result := as_integer (cache_auto_flush_interval_string)
		end
	
	cache_auto_flush_interval_string: STRING
			-- Interval in seconds for automatic flush of
			-- run cache when there is no step activity,
			-- if auto-flush is enabled, as a string
		once
			Result := item (Parm_cache_auto_flush_interval)
		end
	
	db_access_control: STRING
			-- Password for database roles
		once
			Result := item (Parm_db_access_control)
		end
	
	db_default_role: STRING
			-- Default role for database access
		once
			Result := item (Parm_db_default_role)
		end
	
	db_run_closer_role: STRING
			-- Database role for run closing
		once
			Result := item (Parm_db_run_closer_role)
		end
	
	db_run_packager_role: STRING
			-- Database role for packaging-related tasks
		once
			Result := item (Parm_db_run_packager_role)
		end
	
	step_done_icon: STRING
			-- Name of file with "step done" icon
		once
			Result := item (Parm_step_done_icon)
		end
	
	step_inapplicable_icon: STRING
			-- Name of file with "step not applicable" icon
		once
			Result := item (Parm_step_inapplicable_icon)
		end
	
	step_undone_icon: STRING
			-- Name of file with "step undone" icon
		once
			Result := item (Parm_step_undone_icon)
		end
	
	doing_step_icon: STRING
			-- Name of file with "doing step" icon
		once
			Result := item (Parm_doing_step_icon)
		end
	
	undoing_step_icon: STRING
			-- Name of file with "undoing step" icon
		once
			Result := item (Parm_undoing_step_icon)
		end
	
	step_saved_state_icon: STRING
			-- Name of file with "step undone but has saved state"
			-- icon
		once
			Result := item (Parm_step_saved_state_icon)
		end
	
feature {NONE} -- Parameter names

	Parm_fixed_display_name: STRING = "FIXED_DISPLAY_NAME"
	
	Parm_db_conn_str: STRING = "DB_CONN_STR"
	
	Parm_oracle_home: STRING = "ORACLE_HOME"
	
	Parm_oracle_reports_home: STRING = "ORACLE_REPORTS_HOME"
	
	Parm_tns_admin: STRING = "TNS_ADMIN"
	
	Parm_sqlplus_exe_file: STRING = "SQLPLUS_EXE_FILE"
	
	Parm_melt_path: STRING = "MELT_PATH"
	
	Parm_real_run_dir_base: STRING = "REAL_RUN_DIR_BASE"
	
	Parm_probing_run_dir_base: STRING = "REAL_PROBING_DIR_BASE"
	
	Parm_trial_run_dir_base: STRING = "TRIAL_RUN_DIR_BASE"
	
	Parm_trial_probing_dir_base: STRING = "TRIAL_PROBING_DIR_BASE"
	
	Parm_lock_dir: STRING = "LOCK_DIR"
	
	Parm_delta_lock_root: STRING = "DELTA_LOCK_ROOT"
	
	Parm_shared_lock_root: STRING = "SHARED_LOCK_ROOT"
	
	Parm_lock_suffix: STRING = "LOCK_SUFFIX"
	
	Parm_exclusive_delta_lock: STRING = "EXCLUSIVE_DELTA_LOCK"
	
	Parm_log_dir: STRING = "LOG_DIR"
	
	Parm_log_suffix: STRING = "LOG_SUFFIX"
	
	Parm_error_suffix: STRING = "ERROR_SUFFIX"
	
	Parm_temp_dir: STRING = "TEMP_DIR"
	
	Parm_delta_version_file: STRING = "DELTA_VERSION_FILE"
	
	Parm_std_ghostscript_program: STRING = "STD_GHOSTSCRIPT_PROGRAM"
	
	Parm_alt_ghostscript_program: STRING = "ALT_GHOSTSCRIPT_PROGRAM"
	
	Parm_std_acroread_program: STRING = "STD_ACROREAD_PROGRAM"
	
	Parm_alt_acroread_program: STRING = "ALT_ACROREAD_PROGRAM"
	
	Parm_print_program: STRING = "PRINT_PROGRAM"
	
	Parm_cat_program: STRING = "CAT_PROGRAM"
	
	Parm_rshell_program: STRING = "RSHELL_PROGRAM"
	
	Parm_phone_program: STRING = "PHONE_PROGRAM"
	
	Parm_cif_to_dxf_program: STRING = "CIF_TO_DXF_PROGRAM"
	
	Parm_cif_to_dxf_host: STRING = "CIF_TO_DXF_HOST"
	
	Parm_run_as_user_program: STRING = "RUN_AS_USER_PROGRAM"
	
	Parm_front_end_user: STRING = "FRONT_END_USER"
	
	Parm_printer_environment_var: STRING = "PRINTER_ENVIRONMENT_VAR"
	
	Parm_reportwriter_program: STRING = "REPORTWRITER_PROGRAM"
	
	Parm_gds_merge_parm_file: STRING = "GDS_MERGE_PARM_FILE"
	
	Parm_delta_program: STRING = "DELTA_PROGRAM"
	
	Parm_gds_merge_program: STRING = "GDS_MERGE_PROGRAM"
	
	Parm_level_merge_program: STRING = "LEVEL_MERGE_PROGRAM"
	
	Parm_level_fill_program: STRING = "LEVEL_FILL_PROGRAM"
	
	Parm_fill_selector_program: STRING = "FILL_SELECTOR_PROGRAM"
	
	Parm_probing_files_program: STRING = "PROBING_FILES_PROGRAM"
	
	Parm_wafer_techn_survey_script: STRING = "WAFER_TECHN_SURVEY_SCRIPT"
	
	Parm_project_summary_report: STRING = "PROJECT_SUMMARY_REPORT"
	
	Parm_line_item_report: STRING = "LINE_ITEM_REPORT"
	
	Parm_payload_and_charges_report: STRING = "PAYLOAD_AND_CHARGES_REPORT"
	
	Parm_costs_and_margin_report: STRING = "COSTS_AND_MARGIN_REPORT"
	
	Parm_sales_order_report: STRING = "SALES_ORDER_REPORT"
	
	Parm_all_runs_status_report: STRING = "ALL_RUNS_STATUS_REPORT"
	
	Parm_run_status_report: STRING = "RUN_STATUS_REPORT"
	
	Parm_foundry_eccn_report: STRING = "FOUNDRY_ECCN_REPORT"
	
	Parm_mailer_command: STRING = "MAILER_COMMAND"

	Parm_from_line: STRING = "FROM_LINE"

	Parm_crash_msg_to_list: STRING = "CRASH_MSG_TO_LIST"
	
	Parm_unused_options_to_list: STRING = "UNUSED_OPTIONS_TO_LIST"
	
	Parm_unresolved_issue_to_list: STRING = "UNRESOLVED_ISSUE_TO_LIST"
	
	Parm_level_density_to_list: STRING = "LEVEL_DENSITY_TO_LIST"
	
	Parm_probing_files_to_list: STRING = "PROBING_FILES_TO_LIST"
	
	Parm_test_vehicles_to_list: STRING = "TEST_VEHICLES_TO_LIST"
	
	Parm_export_control_to_list: STRING = "EXPORT_CONTROL_TO_LIST"
	
	Parm_gds_merge_to_list: STRING = "GDS_MERGE_TO_LIST"
	
	Parm_gds_merge_error_to_list: STRING = "GDS_MERGE_ERROR_TO_LIST"
	
	Parm_scmos_conversion_to_list: STRING = "SCMOS_CONVERSION_TO_LIST"
	
	Parm_run_posted_to_list: STRING = "RUN_POSTED_TO_LIST"
	
	Parm_run_rtm_to_list: STRING = "RUN_RTM_TO_LIST"
	
	Parm_mask_invoice_to_list: STRING = "MASK_INVOICE_TO_LIST"
	
	Parm_is_production: STRING = "IS_PRODUCTION"
	
	Parm_is_graphical: STRING = "IS_GRAPHICAL"
	
	Parm_background_color: STRING = "BACKGROUND_COLOR"
	
	Parm_cache_auto_flush_interval: STRING = "CACHE_AUTO_FLUSH_INTERVAL"
	
	Parm_db_access_control: STRING = "DB_ACCESS_CONTROL"
	
	Parm_db_default_role: STRING = "DB_DEFAULT_ROLE"
	
	Parm_db_run_closer_role: STRING = "DB_RUN_CLOSER_ROLE"
	
	Parm_db_run_packager_role: STRING = "DB_RUN_PACKAGER_ROLE"
	
	Parm_step_done_icon: STRING = "STEP_DONE_ICON"
	
	Parm_step_inapplicable_icon: STRING = "STEP_INAPPLICABLE_ICON"
	
	Parm_step_undone_icon: STRING = "STEP_UNDONE_ICON"
	
	Parm_doing_step_icon: STRING = "DOING_STEP_ICON"
	
	Parm_undoing_step_icon: STRING = "UNDOING_STEP_ICON"
	
	Parm_step_saved_state_icon: STRING = "STEP_SAVED_STATE_ICON"
	
end
