
indexing
	
	description: "Interface to obtain information needed to create a new %
		%cut pattern"
	author: "David Hollenberg"
	date: "June 27, 2007"

deferred class RC_CUT_PATTERN_PROPERTIES_IFC
inherit
	ABSTRACT_INTERFACE

feature {NONE} -- Creation

	make_interface
			-- Create interface
		do
			create accept_actions
			create cancel_actions
		end

feature -- Properties

	cut_pattern_numbers: LIST [INTEGER]
			-- List of valid cut pattern numbers for new cut
			-- pattern

	fields: LIST [RETICLE_FIELD]
			-- List of possible fields for new cut pattern

	default_field_name: STRING
			-- Name of field to offer as default

	wafer_etest_names: LIST [WAFER_ETEST_NAMES_ROW]
			-- List of possible wafer etest names for new cut
			-- pattern

	default_etest_name: STRING
			-- Wafer etest name to offer as default, if not Void

	default_wafer_quantity: INTEGER
			-- Wafer etest name to offer as default, if not 
			-- negative (-1 means no default)

	is_pattern_number_changeable: BOOLEAN
			-- Is user allowed to change the cut pattern number?

	is_field_name_changeable: BOOLEAN
			-- Is user allowed to change the field name?

	all_values_required: BOOLEAN
			-- Must user specify a valid value for every item?

	accept_actions: ACTION_SEQUENCE [TUPLE [INTEGER, STRING, STRING, INTEGER]]
			-- Action to be invoked when user accepts
			-- current values

	cancel_actions: CLOSED_ACTION_SEQUENCE
			-- Action to be invoked when user clicks on Cancel

feature -- Modification

	set_cut_pattern_numbers (a_list: like cut_pattern_numbers)
			-- Set `cut_pattern_numbers' to `a_list'
		require
			a_list_not_void: a_list /= Void
			a_list_not_empty: not a_list.is_empty
		do
			cut_pattern_numbers := a_list
		ensure
			cut_pattern_numbers_set: cut_pattern_numbers = a_list
		end

	set_fields (a_list: like fields)
			-- Set `fields' to `a_list'
		require
			a_list_not_void: a_list /= Void
			a_list_not_empty: not a_list.is_empty
		do
			fields := a_list
		ensure
			fields_set: fields = a_list
		end

	set_default_field_name (a_name: like default_field_name)
			-- Set `default_field_name' to `a_name'
		do
			default_field_name := a_name
		ensure
			default_field_name_set: default_field_name = a_name
		end

	set_wafer_etest_names (a_list: like wafer_etest_names)
			-- Set `wafer_etest_names' to `a_list'
		require
			a_list_not_void: a_list /= Void
		do
			wafer_etest_names := a_list
		ensure
			wafer_etest_names_set: wafer_etest_names = a_list
		end

	set_default_etest_name (a_name: like default_etest_name)
			-- Set `default_etest_name' to `a_name'
		do
			default_etest_name := a_name
		ensure
			default_etest_name_set: default_etest_name = a_name
		end

	set_default_wafer_quantity (a_value: like default_wafer_quantity)
			-- Set `default_wafer_quantity' to `a_value'
		do
			default_wafer_quantity := a_value
		ensure
			default_wafer_quantity_set: default_wafer_quantity = a_value
		end

	set_is_pattern_number_changeable (a_value: like is_pattern_number_changeable)
			-- Set `is_pattern_number_changeable' to `a_value'
		do
			is_pattern_number_changeable := a_value
		ensure
			value_set: is_pattern_number_changeable = a_value
		end

	set_is_field_name_changeable (a_value: like is_field_name_changeable)
			-- Set `is_field_name_changeable' to `a_value'
		do
			is_field_name_changeable := a_value
		ensure
			value_set: is_field_name_changeable = a_value
		end

	set_all_values_required (a_value: like all_values_required)
			-- Set `all_values_required' to `a_value'
		do
			all_values_required := a_value
		ensure
			value_set: all_values_required = a_value
		end

end
