
indexing
	
	description: "Window to obtain information needed to create a new %
		%cut pattern"
	author: "David Hollenberg"
	date: "June 27, 2007"

class RC_CUT_PATTERN_PROPERTIES_WINDOW

inherit
	RC_CUT_PATTERN_PROPERTIES_IFC
	WINDOW_INTERFACE
		redefine
			enable_active,
			disable_active
		end
	MOSIS_GRAPHICAL_UTILITIES
		export
			{NONE} all
		undefine
			default_create
		end

create
	make

feature {NONE} -- Creation

	make (w: EV_TITLED_WINDOW)
			-- Create interface in window `w'
		require
			window_exists: w /= Void
			window_valid: not w.is_destroyed
		do
			make_interface
			make_proxy (w)
		end

feature -- Operations

	reset
			-- Move focus back into window
		do
			enable_active
		end

feature -- Modification

	enable_active
			-- Make interface active
		do
			precursor
			ok_button.enable_sensitive
			cancel_button.enable_sensitive
		end

	disable_active
			-- Make interface inactive
			-- (can look but cannot initiate most actions)
		do
			precursor
			ok_button.disable_sensitive
			cancel_button.disable_sensitive
		end

feature {NONE} -- Implementation

	build_interface
			-- Build interface
		local
			cols: WIDGET_COLUMN_SET
			buttons: HORIZONTAL_BUTTON_SET
			main_vbox: EV_VERTICAL_BOX
			box: EV_BOX
		do
			create main_vbox
			main_vbox.set_padding (user_interface.padding)
			main_vbox.set_border_width (user_interface.border_width)
			create cols.make (2)
			cols.set_padding (user_interface.padding)
			
			build_cut_pattern_list
			build_field_list
			build_wafer_etest_id_list
			build_wafer_quantity_field
			cols.append_row (<< gui.label_with_text ("Cut Pattern Number"), cut_pattern_field >>)
			cols.append_row (<< gui.label_with_text ("Field Name"), field_name_field >>)
			cols.append_row (<< gui.label_with_text ("Lot Split"), wafer_etest_id_field >>)
			cols.append_row (<< gui.label_with_text ("Number of Wafers to Cut"), wafer_quantity_field >>)

			cols.disable_column_expand (1)
			cols.align_column_right (1)
			box := cols.box
			
			main_vbox.extend (box)
			main_vbox.disable_item_expand (box)
			
			create buttons.make_with_text_and_actions (<<
				"OK", "Cancel" >>,
				<< agent process_ok, agent process_cancel >>)
			ok_button := buttons.i_th (1)
			cancel_button := buttons.i_th (2)
			
			box := buttons.column_set.box
			
			main_vbox.extend (box)
			main_vbox.disable_item_expand (box)
			
			gui.set_box_background_recursive (main_vbox, background_color)
			extend (main_vbox)
			close_request_actions.extend (agent process_cancel)
			gui.add_cancel_accelerator (window, agent process_cancel)
			gui.do_once_on_idle (agent post_display_actions)
		end
		
	build_cut_pattern_list
			-- Build combo box with list of cut pattern numbers
			-- put in `cut_pattern_field'
		local
			fact: SINGLE_CONSTRAINED_VALUE_FACTORY
			n: INTEGER
		do
			create fact
			from
				cut_pattern_numbers.start
			until
				cut_pattern_numbers.after
			loop
				n := cut_pattern_numbers.item
				fact.add_choice_and_text (n.out, n.out)
				cut_pattern_numbers.forth
			end
			fact.set_default_choice (cut_pattern_numbers.first.out)
			fact.generate_combo_box
			cut_pattern_field := fact.combo_box
			if not is_pattern_number_changeable then
				cut_pattern_field.disable_sensitive
			end
		end
		
	build_field_list
			-- Build combo box with list of fields and
			-- put in `field_name_field'
		local
			fact: SINGLE_CONSTRAINED_VALUE_FACTORY
			l_field, l_default: STRING
		do
			create fact
			from
				fields.start
			until
				fields.after
			loop
				l_field := fields.item.name
				fact.add_choice_and_text (l_field, l_field)
				fields.forth
			end
			if default_field_name /= Void then
				l_default := default_field_name
			else
				l_default := fields.first.name
			end
			fact.set_default_choice (l_default)
			fact.generate_combo_box
			field_name_field := fact.combo_box
			if not is_field_name_changeable then
				field_name_field.disable_sensitive
			end
		end
		
	build_wafer_etest_id_list
			-- Build combo box with list of wafer etest IDs and
			-- put in `wafer_etest_id_field'
		local
			fact: SINGLE_CONSTRAINED_VALUE_FACTORY
			row: WAFER_ETEST_NAMES_ROW
			dflt: STRING
		do
			create fact
			if wafer_etest_names.count /= 1 then
				fact.add_choice_and_text ("", Unspecified_etest_id)
			end
			from
				wafer_etest_names.start
			until
				wafer_etest_names.after
			loop
				row := wafer_etest_names.item
				fact.add_choice_and_text (row.wafer_etest_name, row.wafer_etest_id)
				wafer_etest_names.forth
			end
			if wafer_etest_names.count = 1 then
				-- Only one possibility
				dflt := wafer_etest_names.first.wafer_etest_name
			elseif default_etest_name /= Void then
				-- More than one choice, but default available
				dflt := default_etest_name
			else
				dflt := ""
			end
			fact.set_default_choice (dflt)
			fact.generate_combo_box
			wafer_etest_id_field := fact.combo_box
		end
		
	build_wafer_quantity_field
			-- Build text field with number of wafers to cut
			-- and put in `wafer_quantity_field'
		do
			create wafer_quantity_field
			wafer_quantity_field.return_actions.extend (agent process_ok)
			if default_wafer_quantity >= 0 then
				wafer_quantity_field.set_text (default_wafer_quantity.out)
			end
		end
		
	post_display_actions
		do
			reset
		end

	process_ok
		do
			if is_active then
				disable_active
				selected_etest_name := data_value (wafer_etest_id_field.selected_item)
				check_validity
				if is_valid then
					accept_actions.call ([cut_pattern_field.selected_item.text.to_integer, field_name_field.selected_item.text.to_string_8, selected_etest_name, wafer_quantity])
				else
					inform (last_message)
				end
				enable_active
			end
		end

	check_validity
			-- Check validity of values user specified.  Set
			-- `is_valid' to indicate whether everything is valid
			-- and if not valid put explanation in `last_message' 
		local
			quant: STRING
			n, l_max: INTEGER
		do
			is_valid := True
			create last_message.make (100)
			quant := wafer_quantity_field.text
			if quant.is_empty then
				if all_values_required then
					add_message ("Number of wafers to cut must be specified")
				else
					wafer_quantity := -1
				end
			elseif not quant.is_integer then
				add_message ("Number of wafers to cut must be an integer")
			else
				n := quant.to_integer
				l_max := (10 ^ {PACKING_CUT_PATTERNS_ROW}.precision_quantity_wafers_needed).rounded - 1
				if n < 0 or n > l_max then
					add_message ("Number of wafers to cut must be in range 0 - " + l_max.out)
				else
					wafer_quantity := n
					is_valid := True
				end
			end
			if all_values_required and selected_etest_name.is_empty then
				add_message ("Lot split must be specified")
			end
		end

	data_value (a_list_item: EV_LIST_ITEM): STRING
			-- Data associated with `a_list_item', as a string
		do
			Result ?= a_list_item.data
			check
				result_not_void: Result /= Void
			end
		end

	add_message (a_text: STRING)
			-- Append `a_text' to `last_message'
		do
			is_valid := False
			if not last_message.is_empty then
				last_message.extend ('%N')
			end
			last_message.append (a_text)
		end

	process_cancel
		do
			if is_active then
				disable_active
				cancel_actions.call ([])
			end
		end

	cut_pattern_field: EV_COMBO_BOX
			-- Field with list of possible cut pattern numbers
	
	field_name_field: EV_COMBO_BOX
			-- Field with list of field names
	
	wafer_etest_id_field: EV_COMBO_BOX
			-- Field with list of possible wafer etest IDs
	
	wafer_quantity_field: EV_TEXT_FIELD
			-- Field with number of wafers to cut for pattern
	
	ok_button: EV_BUTTON
			-- Button to accept values
	
	cancel_button: EV_BUTTON
			-- Button to close window
	
	Unspecified_etest_id: STRING is "Unspecified"
			-- Text for unspecified wafer etest ID

	selected_etest_name: STRING
			-- Wafer etest name for wafer etest ID user selected

	wafer_quantity: INTEGER
			-- Number of wafers to cut entered by user

	is_valid: BOOLEAN
			-- Was wafer quantity valid?

	last_message: STRING
			-- Message for last validity check

end
