
indexing
	
	description: "Window to allow user to set die overhead parameters for a run"
	author: "David Hollenberg"
	date: "March 1, 2007"

class RC_DIE_OVERHEAD_PARAMETERS_WINDOW

inherit
	RC_DIE_OVERHEAD_PARAMETERS_IFC
	WINDOW_INTERFACE
	MOSIS_GRAPHICAL_UTILITIES
		export
			{NONE} all
		undefine
			default_create
		end
	PACKING_RECORD_UTILITIES
		export
			{NONE} all
		end
	NUMBER_UTILITIES_ACCESS
		export
			{NONE} all
		undefine
			default_create
		end
		
create
	make

feature {NONE} -- Creation

	make (w: EV_TITLED_WINDOW)
			-- Create interface in window `w'
		require
			window_exists: w /= Void
			window_valid: not w.is_destroyed
		do
			make_interface
			make_proxy (w)
		end

feature -- Operations

	reset
			-- Reset interface
		do
		end

feature {NONE} -- Implementation

	build_interface
			-- Build interface in `window'
		local
			main_vbox: EV_VERTICAL_BOX
			buttons: HORIZONTAL_BUTTON_SET
			box: EV_BOX
			cols: WIDGET_COLUMN_SET
		do
			create main_vbox
			main_vbox.set_border_width (user_interface.border_width)
			build_overhead_style
			build_other_border
			build_top_overhead
			
			create cols.make (2)
			cols.set_border_width (user_interface.border_width)
			cols.set_padding (user_interface.padding)
			cols.append_row (<< gui.label_with_text ("Die Labeling Style"), overhead_style_field >>)
			cols.append_row (<< gui.label_with_text ("Other Side Overhead (White Space)"), other_border_field >>)
			cols.append_row (<< gui.label_with_text ("Top Side Overhead"), top_overhead_field >>)
			cols.align_column_right (1)
			cols.disable_column_expand (1)
			box := cols.box
			main_vbox.extend (box)
			main_vbox.disable_item_expand (box)

			create cols.make (1)
			cols.set_border_width (user_interface.border_width)
			cols.set_padding (user_interface.padding)
			cols.append_row (<< gui.label_with_text ("Other Side White Space Overhead is the amount of white space (in microns) added on all but the top side of a project.%NTop Side Overhead is the amount of overhead added on the top side of the project (computed automatically).") >>)
			cols.align_column_left (1)
			cols.disable_column_expand (1)
			box := cols.box
			main_vbox.extend (box)
			main_vbox.disable_item_expand (box)
			
			create buttons.make_with_text_and_actions (<< gui.ev_ok, gui.ev_cancel >>, << agent process_ok, agent process_cancel >>)
			box := buttons.column_set.box
			
			main_vbox.extend (box)
			main_vbox.disable_item_expand (box)
			gui.set_box_background_recursive (main_vbox, background_color)
			extend (main_vbox)

			close_request_actions.extend (agent process_cancel)
			gui.add_cancel_accelerator (window, agent process_cancel)
			enable_active
			gui.do_once_on_idle (agent gui.center_window (window))
		end
		
	build_overhead_style
			-- Build combo box for overhead style for run
		local
			fact: SINGLE_CONSTRAINED_VALUE_FACTORY
			l_choice, l_text, default_choice: STRING
			style: OVERHEAD_STYLE_VALUES
		do
			create fact
			fact.enable_choices_prepended
			from
				allowed_overhead_styles.start
			until
				allowed_overhead_styles.after
			loop
				l_choice := allowed_overhead_styles.item
				l_text := overhead_style_descriptions.item (l_choice)
				l_choice := style.null_translated (l_choice)
				fact.add_choice_and_text (l_choice, l_text)
				allowed_overhead_styles.forth
			end
			if state /= Void and then state.overhead_style /= Void and then allowed_overhead_styles.has (state.overhead_style) then
				default_choice := style.null_translated (state.overhead_style)
			else
				default_choice := style.External_
			end
			fact.set_default_choice (default_choice)
			fact.generate_combo_box
			overhead_style_field := fact.combo_box
			overhead_style_field.select_actions.extend (agent process_style_selection)
		end
		
	build_other_border
			-- Build text field for other border (for non-overhead
			-- strip sides of die)
		local
			val: DOUBLE
			val_string: STRING
		do
			if state /= Void then
				val := state.other_border
			else
				val := default_other_border
			end
			val_string := nu.formatted_number_or_integer (val, border_scale)
			create other_border_field.make_with_text (val_string)
			other_border_field.enable_edit
			other_border_field.change_actions.extend (agent process_other_border_change)
			other_border_field.return_actions.extend (agent process_ok)
		end
		
	build_top_overhead
			-- Build text field for overhead on top side of design
		do
			create top_overhead_field.make_with_text (current_top_overhead)
			top_overhead_field.disable_sensitive
			
		end
		
	process_style_selection
			-- Take actions needed when user selects a die
			-- labeling style
		local
			l_style: STRING
			style: OVERHEAD_STYLE_VALUES
		do
			l_style := current_overhead_style
			if l_style.is_equal (style.External_) then
				other_border_field.set_text (nu.formatted_number_or_integer (default_other_border, border_scale))
			end
			top_overhead_field.set_text (current_top_overhead)
		end

	process_other_border_change
			-- Take actions needed when value of 
			-- `other_border_field' changes
		do
			top_overhead_field.set_text (current_top_overhead)
		end

	process_ok
			-- Take actions needed when user accepts
			-- current operation
		local
			operation: ANONYMOUS_OPERATION
		do
			if is_active then
				disable_active
				create operation.make (agent process_accept_request)
				operation.enable_abort_caught
				operation.execute
				if operation.aborted then
					session.rollback
					enable_active
				else
					success_actions.call ([])
				end
			end
		end

	process_accept_request
			-- Process an accept request
		local
			l_style, border_val: STRING
			style: OVERHEAD_STYLE_VALUES
		do
			border_val := other_border_field.text
			l_style := current_overhead_style
			check_valid_border (border_val)
			check_overhead_style (l_style)
			
			fetch_and_modify_run
			border_val := nu.formatted_number_or_integer (nu.rounded_to_resolution (border_val.to_double, half_grid_resolution), border_scale)
			session.add_commit_action (agent publish_new_overhead_style_value (style.null_translated (l_style)))
			session.add_commit_action (agent publish_new_other_border_value (border_val))
			session.add_commit_action (agent cache.uncache_run)
		end

	check_valid_border (val: STRING)
			-- Check that `val' represents a valid other
			-- border value
		local
			value, half_grid_res: DOUBLE
			scale: INTEGER
		do
			if not val.is_double then
				handle_user_error ("Value %"" + val + "%" is not a valid real number")
			else
				half_grid_res := half_grid_resolution
				scale := border_scale
				value := val.to_double
				if value < 0.0 then
					handle_user_error ("Value (" + val + ") is negative")
				elseif value < minimum_other_border then
					handle_user_error ("Value (" + val + ") is smaller than minimum value " + nu.formatted_number_or_integer (minimum_other_border, scale))
				elseif value = 0.0 then
					-- OK
				elseif not nu.is_multiple_of_resolution (value, half_grid_res, scale) then
					handle_user_error ("Value (" + val + ") is not a multiple of half-grid resolution " + nu.formatted_number_or_integer (half_grid_res, scale))
				elseif value < Border_threshold then
					confirm ("Value (" + val + ") is less than threshold (" + Border_threshold.out + " microns) but is not 0%NProceed anyway")
				end
			end
		end

	check_overhead_style (val: STRING)
			-- Issue warning if overhead style is NONE
		local
			style: OVERHEAD_STYLE_VALUES
		do
			if val.is_equal (style.None) then
				warn ("Selected die labeling style is " + style.None + ".%NNone of the dies will have die IDs, which may make it difficult%Nto identify individual dies.%N%NProceed anyway?")
			end
		end

	wafer_techns_row: WAFER_TECHNS_ROW
			-- Row for wafer techn for run
		do
			Result := cache.wafer_techns_row
		end

	fetch_and_modify_run
			-- Fetch runs row, lock it and set overhead style and
			-- other border
		local
			run: RUNS_MROW
			val: DOUBLE
			l_style: STRING
			style: OVERHEAD_STYLE_VALUES
		do
			run := cache.runs_mrow
			session.lock_row (run)
			l_style := current_overhead_style
			val := connection.string_to_double (other_border_field.text)
			run.set_overhead_style (l_style)
			run.set_other_border (val)
			if l_style.is_equal (style.None) then
				run.set_uniform_override_border (val)
			end
		end

	publish_new_overhead_style_value (s: STRING)
			-- Publish new overhead style value
		require
			value_exists: s /= Void
		local
			e: OVERHEAD_STYLE_CHANGE_EVENT
		do
			create e
			e.set_overhead_style (s)
			e.publish
		end

	publish_new_other_border_value (s: STRING)
			-- Publish new value of other border
		require
			value_exists: s /= Void
		local
			e: OTHER_BORDER_CHANGE_EVENT
		do
			create e
			e.set_other_border (s)
			e.publish
		end

	current_overhead_style: STRING
			-- Overhead style currently selected
		local
			style: OVERHEAD_STYLE_VALUES
		do
			Result ?= overhead_style_field.selected_item.data
			check
				result_not_void: Result /= Void
			end
			Result := style.project_specific_translated (Result)
		end
	
	current_top_overhead: STRING
			-- Description of overhead on top side of design,
			-- based on currently selected values
		do
			Result := top_overhead_description (current_overhead_style, other_border_field.text)
		end
	
	process_cancel
			-- Take actions needed when user cancels
			-- current operation
		do
			if is_active then
				disable_active
				abort_actions.call ([])
			end
		end

	overhead_style_field: EV_COMBO_BOX
			-- Field with allowed overhead styles for run

	other_border_field: EV_TEXT_FIELD
			-- Field for white space border on non-overhead
			-- strip sides of die

	top_overhead_field: EV_TEXT_FIELD
			-- Field to display amount of overhead on top side
			-- of design

end
