
indexing
	
	description: "An environment for run closing.  Includes configuration %
		%parameters and other information that is not specific to %
		%one session"
	author: "David Hollenberg"
	date: "September 17, 2002"

class RC_ENVIRONMENT

inherit
	RC_CONFIGURATION_VARS
		rename
			configuration_errors as std_configuration_errors
		redefine
			make
		end
	UNIX_OS_ACCESS
		export
			{NONE} all
		undefine
			is_equal, copy
		end
	OS_ACCESS
		export
			{NONE} all
		undefine
			is_equal, copy
		end

create
	make

feature {NONE} -- Creation

	make (n: INTEGER)
			-- Create environment with initial space for
			-- `n' configuration parameters
		do
			precursor (n)
			initialize
		end

feature -- Properties

	user_name: STRING
			-- Name of operating system user running application,
			-- in lower case (from real user id)

	upper_user_name: STRING
			-- Name of operating system user running application,
			-- in upper case (from real user id)

	user_email_address: STRING
			-- E-mail address of user running application

	process_id: INTEGER
			-- Process id of current process

	host_name: STRING
			-- Short name of host on which process is executing,
			-- in lower case

	upper_host_name: STRING
			-- Short name of host on which process is executing,
			-- in upper case

	session_identifier: STRING
			-- Identifier for this session.  Unique among active
			-- sessions and probably over all time (but not
			-- guaranteed)
		local
			sep: STRING
		do
			sep := "   "
			Result := process_id.out + sep + user_name + sep + host_name + sep + unix_os.current_date_string
		end

	general_log_filename: STRING
			-- Name of file for logging before a
			-- session-specific file has been determined
		do
			Result := unix_os.full_file_name (log_dir, "delta" + log_suffix)
		end

	general_error_filename: STRING
			-- Name of file for recording errors that
			-- occur before a session-specific file has been
			-- determined
		do
			Result := unix_os.full_file_name (log_dir, "delta" + error_suffix)
		end

	general_log_file: LARGE_PLAIN_TEXT_FILE
			-- File for logging before a session-specific
			-- file has been determined

	general_error_file: LARGE_PLAIN_TEXT_FILE
			-- File for recording errors that occur before
			-- a session-specific file has been determined

	run_directory_for_real_run (s: STRING): STRING
			-- Name of run directory for real run `s'
		require
			run_id_exists: s /= Void
		do
			Result := os.full_file_name (real_run_dir_base, s.as_lower)
		ensure
			result_exists: Result /= Void
		end

	probing_directory_for_real_run (s: STRING): STRING
			-- Name of run directory for probing purposes for
			-- real run `s'
		require
			run_id_exists: s /= Void
		do
			Result := os.full_file_name (real_probing_dir_base, s.as_lower)
		ensure
			result_exists: Result /= Void
		end

	run_directory_for_trial_run (s: STRING): STRING
			-- Name of run directory for trial run `s'
		require
			run_id_exists: s /= Void
		do
			Result := os.full_file_name (trial_run_dir_base, s.as_lower)
		ensure
			result_exists: Result /= Void
		end

	probing_directory_for_trial_run (s: STRING): STRING
			-- Name of run directory for probing purposes for
			-- trial run `s'
		require
			run_id_exists: s /= Void
		do
			Result := os.full_file_name (trial_probing_dir_base, s.as_lower)
		ensure
			result_exists: Result /= Void
		end

	log_name_for_run (s: STRING): STRING
			-- Name of log file for run `s'
		require
			run_id_exists: s /= Void
		do
			Result := os.full_file_name (log_dir, s.as_lower + log_suffix)
		ensure
			result_exists: Result /= Void
		end

	error_name_for_run (s: STRING): STRING
			-- Name of error file for run `s'
		require
			run_id_exists: s /= Void
		do
			Result := os.full_file_name (log_dir, s.as_lower + error_suffix)
		ensure
			result_exists: Result /= Void
		end

	lock_name_for_run (s: STRING): STRING
			-- Name of lock file for run `s'
		require
			run_id_exists: s /= Void
		do
			Result := os.full_file_name (lock_dir, s.as_lower + lock_suffix)
		ensure
			result_exists: Result /= Void
		end

	log_name_for_packing (s: STRING): STRING
			-- Name of log file for packing `s'
		require
			packing_id_exists: s /= Void
		do
			Result := os.full_file_name (log_dir, Packing_prefix + s.as_lower + log_suffix)
		ensure
			result_exists: Result /= Void
		end

	error_name_for_packing (s: STRING): STRING
			-- Name of error file for packing `s'
		require
			packing_id_exists: s /= Void
		do
			Result := os.full_file_name (log_dir, Packing_prefix + s.as_lower + error_suffix)
		ensure
			result_exists: Result /= Void
		end

	lock_name_for_packing (s: STRING): STRING
			-- Name of lock file for packing `s'
		require
			packing_id_exists: s /= Void
		do
			Result := os.full_file_name (lock_dir, Packing_prefix + s.as_lower + lock_suffix)
		ensure
			result_exists: Result /= Void
		end

	delta_lock_name: STRING
			-- Full name of lock file that allows running of Delta
		do
			Result := os.full_file_name (lock_dir, delta_lock_root + lock_suffix)
		ensure
			result_exists: Result /= Void
		end

	shared_lock_name: STRING
			-- Full name of shared lock file
		do
			Result := os.full_file_name (lock_dir, shared_lock_root + lock_suffix)
		ensure
			result_exists: Result /= Void
		end

	ghostscript_program: STRING
			-- Full name of GhostScript program file
		local
			f: LARGE_RAW_FILE
		do
			create f.make (std_ghostscript_program)
			if f.exists then
				Result := std_ghostscript_program
			else
				Result := alt_ghostscript_program
			end
		ensure
			result_exists: Result /= Void
		end

	acroread_program: STRING
			-- Full name of acroread program file
		local
			f: LARGE_RAW_FILE
		do
			create f.make (std_acroread_program)
			if f.exists then
				Result := std_acroread_program
			else
				Result := alt_acroread_program
			end
		ensure
			result_exists: Result /= Void
		end

	open_errors: STRING
			-- Errors opening general log and error files,
			-- if not Void or empty
	
	configuration_errors: STRING
			-- Errors in configurations
		local
			req_errors: STRING
		do
			create Result.make (0)
			if su.not_empty (std_configuration_errors) then
				Result.append (std_configuration_errors)
				if (Result.count > 0 and then Result.item (Result.count) /= '%N') then
					Result.extend ('%N')
				end
			end
			req_errors := required_parameter_errors
			if su.not_empty (req_errors) then
				Result.append (req_errors)
			end
		end

	required_parameter_errors: STRING
			-- Errors for required parameters that were
			-- not specified or have empty values
		do
			create parameter_errors.make (0)
			check_parameter (Parm_db_conn_str, db_conn_str)
			check_parameter (Parm_oracle_home, oracle_home)
			check_parameter (Parm_oracle_reports_home, oracle_reports_home)
			check_parameter (Parm_tns_admin, tns_admin)
			check_parameter (Parm_sqlplus_exe_file, sqlplus_exe_file)
			check_parameter (Parm_melt_path, melt_path)
			check_parameter (Parm_real_run_dir_base, real_run_dir_base)
			check_parameter (Parm_probing_run_dir_base, real_probing_dir_base)
			check_parameter (Parm_trial_run_dir_base, trial_run_dir_base)
			check_parameter (Parm_trial_probing_dir_base, trial_probing_dir_base)
			check_parameter (Parm_lock_dir, lock_dir)
			check_parameter (Parm_delta_lock_root, delta_lock_root)
			check_parameter (Parm_shared_lock_root, shared_lock_root)
			check_parameter (Parm_lock_suffix, lock_suffix)
			check_parameter (Parm_log_dir, log_dir)
			check_parameter (Parm_log_suffix, log_suffix)
			check_parameter (Parm_error_suffix, error_suffix)
			check_parameter (Parm_temp_dir, temp_dir)
			check_parameter (Parm_delta_version_file, delta_version_file)
			check_parameter (Parm_std_ghostscript_program, std_ghostscript_program)
			check_parameter (Parm_alt_ghostscript_program, alt_ghostscript_program)
			check_parameter (Parm_std_acroread_program, std_acroread_program)
			check_parameter (Parm_alt_acroread_program, alt_acroread_program)
			check_parameter (Parm_print_program, print_program)
			check_parameter (Parm_cat_program, cat_program)
			check_parameter (Parm_rshell_program, rshell_program)
			check_parameter (Parm_phone_program, phone_program)
			check_parameter (Parm_cif_to_dxf_program, cif_to_dxf_program)
			check_parameter (Parm_cif_to_dxf_host, cif_to_dxf_host)
			check_parameter (Parm_run_as_user_program, run_as_user_program)
			check_parameter (Parm_front_end_user, front_end_user)
			check_parameter (Parm_printer_environment_var, printer_environment_var)
			check_parameter (Parm_reportwriter_program, reportwriter_program)
			check_parameter (Parm_gds_merge_parm_file, gds_merge_parm_file)
			check_parameter (Parm_delta_program, delta_program)
			check_parameter (Parm_gds_merge_program, gds_merge_program)
			check_parameter (Parm_level_merge_program, level_merge_program)
			check_parameter (Parm_level_fill_program, level_fill_program)
			check_parameter (Parm_fill_selector_program, fill_selector_program)
			check_parameter (Parm_probing_files_program, probing_files_program)
			check_parameter (Parm_wafer_techn_survey_script, wafer_techn_survey_script)
			check_parameter (Parm_project_summary_report, project_summary_report)
			check_parameter (Parm_line_item_report, line_item_report)
			check_parameter (Parm_payload_and_charges_report, payload_and_charges_report)
			check_parameter (Parm_costs_and_margin_report, costs_and_margin_report)
			check_parameter (Parm_sales_order_report, sales_order_report)
			check_parameter (Parm_all_runs_status_report, all_runs_status_report)
			check_parameter (Parm_run_status_report, run_status_report)
			check_parameter (Parm_foundry_eccn_report, foundry_eccn_report)
			check_parameter (Parm_mailer_command, mailer_command)
			check_parameter (Parm_from_line, from_line)
			check_parameter (Parm_crash_msg_to_list, crash_msg_to_list)
			check_parameter (Parm_unused_options_to_list, unused_options_to_list)
			check_parameter (Parm_unresolved_issue_to_list, unresolved_issue_to_list)
			check_parameter (Parm_level_density_to_list, level_density_to_list)
			check_parameter (Parm_probing_files_to_list, probing_files_to_list)
			check_parameter (Parm_test_vehicles_to_list, test_vehicles_to_list)
			check_parameter (Parm_gds_merge_to_list, gds_merge_to_list)
			check_parameter (Parm_gds_merge_error_to_list, gds_merge_error_to_list)
			check_parameter (Parm_scmos_conversion_to_list, scmos_conversion_to_list)
			check_parameter (Parm_export_control_to_list, export_control_to_list)
			check_parameter (Parm_run_posted_to_list, run_posted_to_list)
			check_parameter (Parm_run_rtm_to_list, run_rtm_to_list)
			check_parameter (Parm_mask_invoice_to_list, mask_invoice_to_list)
			check_parameter (Parm_background_color, background_color)
			check_parameter (Parm_cache_auto_flush_interval, cache_auto_flush_interval_string)
			check_parameter (Parm_db_access_control, db_access_control)
			check_parameter (Parm_db_default_role, db_default_role)
			check_parameter (Parm_db_run_closer_role, db_run_closer_role)
			check_parameter (Parm_db_run_packager_role, db_run_packager_role)
			check_parameter (Parm_step_done_icon, step_done_icon)
			check_parameter (Parm_step_inapplicable_icon, step_inapplicable_icon)
			check_parameter (Parm_step_undone_icon, step_undone_icon)
			check_parameter (Parm_doing_step_icon, doing_step_icon)
			check_parameter (Parm_undoing_step_icon, undoing_step_icon)
			check_parameter (Parm_step_saved_state_icon, step_saved_state_icon)
			Result := parameter_errors
		end

feature -- Operations

	set_user_email_address (s: STRING)
			-- Set `user_email_address' to `s'
		require
			address_exists: s /= Void
			address_not_empty: not s.is_empty
		do
			user_email_address := s
		end

	open_log_and_error_files
			-- Open `general_log_file' and `general_error_file'
			-- for append.  Set `open_errors' if
			-- there are any errors
		local
			log_name, err_name: STRING
			errs: STRING
		do
			log_name := general_log_filename
			err_name := general_error_filename
			general_log_file := fu.opened_for_append (log_name)
			general_error_file := fu.opened_for_append (err_name)
			create errs.make (0)
			if general_log_file = Void then
				errs.append ("Cannot open log file " + log_name + " for append%N")
			end
			if general_error_file = Void then
				errs.append ("Cannot open error file " + err_name + " for append%N")
			end
			open_errors := errs
		end

	log_operation (descr: STRING)
			-- Log operation whose description is `descr'
			-- to `general_log_file'
		require
			file_not_void: general_log_file /= Void
			file_open_write: general_log_file.is_open_write
			description_exists: descr /= Void
		do
			log_one_line (general_log_file, descr)
		end

	log_operation_to_file (f: LARGE_PLAIN_TEXT_FILE descr: STRING)
			-- Log operation whose description is `descr'
			-- to `f'
		require
			file_not_void: f /= Void
			file_open_write: f.is_open_write
			description_exists: descr /= Void
		do
			log_one_line (f, descr)
		end

feature {NONE} -- Implementation

	initialize
			-- Initialize `Current'
		do
			process_id := unix_os.my_process_id
			user_name := unix_os.user_name_for_id (unix_os.real_user_id)
			upper_user_name := user_name.twin
			upper_user_name.to_upper
			host_name := unix_os.local_host_short_name
			upper_host_name := host_name.twin
			upper_host_name.to_upper
		end

	check_parameter (name, val: STRING)
			-- Check parameter `name' value `val'.
			-- If it is Void or empty, append an error description
			-- to `parameter_errors'
		require
			name_exists: name /= Void
			name_not_empty: not name.is_empty
			parameter_errors_exists: parameter_errors /= Void
		do
			if su.is_empty (val) then
				parameter_errors.append ("No value for parameter " + name + "%N")
			end
		end

	log_one_line (f: LARGE_PLAIN_TEXT_FILE descr: STRING)
			-- Log operation to `f' with description `descr'
		require
			file_not_void: f /= Void
			file_open_write: f.is_open_write
			description_exists: descr /= Void
		local
			s, temp: STRING
			lines: LIST [STRING]
		do
			create s.make (100)
			create temp.make (10)
			unix_os.append_current_date_string (s)
			s.append (Spacer)
			temp.keep_head (0)
			temp.append (user_name)
			su.right_pad (temp, Max_user_length)
			s.append (temp)
			s.append (Spacer)
			temp.keep_head (0)
			temp.append (host_name)
			su.right_pad (temp, Max_host_length)
			s.append (temp)
			s.append (Spacer)
			temp.keep_head (0)
			temp.append_integer (process_id)
			su.left_pad (temp, Max_process_id_length)
			s.append (temp)
			s.append (Spacer)
			lines := descr.split ('%N')
			from
				lines.start
			until
				lines.after
			loop
				s.append (lines.item)
				lines.forth
				if not lines.after then
					s.extend (' ')
				end
			end
			f.put_string (s)
			f.new_line
			f.flush
		end

	parameter_errors: STRING

	Start_tag: STRING = "Start"

	End_tag: STRING = "End"

	No_tag: STRING = ""

	Spacer: STRING = "  "

	Packing_prefix: STRING = "packing_"

	Max_user_length: INTEGER = 8
			-- Maximum expected length of user name

	Max_host_length: INTEGER = 8
			-- Maximum expected length of host name
	
	Max_process_id_length: INTEGER = 5
			-- Maximum expected length of process id

	Max_tag_length: INTEGER = 5
			-- Maximum expected length of tag

invariant
	process_id_valid: process_id > 0
	user_name_exists: user_name /= Void
	user_name_not_empty: not user_name.is_empty
	upper_user_name_exists: upper_user_name /= Void
	upper_user_name_not_empty: not upper_user_name.is_empty
	host_name_exists: host_name /= Void
	host_name_not_empty: not host_name.is_empty
	upper_host_name_exists: upper_host_name /= Void
	upper_host_name_not_empty: not upper_host_name.is_empty
end
