
indexing
	
	description: "A file created by run closing"
	author: "David Hollenberg"
	date: "May 5, 2003"

class RC_FILE
inherit
	RC_FILE_CATEGORY
		rename
			file_pathname as category_file_pathname
		export
			{NONE} category_file_pathname
		redefine
			can_compute_file_pathnames
		end
	FILE_UTILITIES_ACCESS
		export
			{NONE} all
		end
	STRING_UTILITIES_ACCESS
		export
			{NONE} all
		end
	OS_ACCESS
		export
			{NONE} all
		end
create
	make

feature {NONE} -- Creation

feature -- Properties

	can_compute_file_pathnames: BOOLEAN
			-- May category be used to compute pathnames?
		do
			Result := can_compute_dir_pathnames and file_name /= Void
		end

	file_name: STRING
			-- Name of file, without directory

	file_pathname: STRING
			-- Full pathname of `Current'
		require
			file_name_exists: file_name /= Void
		do
			Result := category_file_pathname (file_name)
		ensure
			result_exists: Result /= Void
		end

feature -- Modification

	set_file_name (s: STRING)
			-- Set `file_name' to `s'
		require
			value_exists: s /= Void
		do
			file_name := s
		ensure
			file_name_set: file_name = s
		end

feature -- Operations

	delete_if_exists
			-- Delete `Current' if it exists
		local
			checker: FILE_EXISTENCE_CHECK_OP
			delete_op: FILE_DELETE_OP
			p: STRING
		do
			p := file_pathname
			create checker
			checker.check_existence (p)
			if checker.path_exists then
				create delete_op
				delete_op.delete (p)
			end
		end

	delete_matching_files
			-- Delete any files matching name of `Current'
			-- in same directory.  Assumes that name has
			-- a wildcard pattern that may contain "*"
			-- (matches zero or more characters) and "%"
			-- (matches exactly one character)
		local
			open_op: DIR_OPEN_READ_OP
			close_op: DIR_CLOSE_OP
			delete_op: FILE_DELETE_OP
			dir: LARGE_DIRECTORY
			fname, pat: STRING
		do
			pat := file_name
			create open_op
			create close_op
			create delete_op
			create dir.make (directory_pathname)
			open_op.open (dir)
			from
                                dir.start
                                dir.readentry
                        until
                                dir.lastentry = Void
                        loop
				fname := dir.lastentry
				if su.wildcard_matches (fname, pat) then
					delete_op.delete (os.full_file_name (dir.name, fname))
				end
                                dir.readentry
                        end

			close_op.close (dir)
		end

end
