
indexing
	
	description: "Logical group of files created by run closing"
	author: "David Hollenberg"
	date: "September 16, 2002"

class RC_FILE_CATEGORY
inherit
	FILE_CATEGORY
	FILE_UTILITIES_ACCESS
	OS_ACCESS
		export
			{NONE} all
		end
create
	make

feature {NONE} -- Creation

	make (the_name, the_dir, the_group, dir_perm_string, file_perm_string: STRING)
			-- Create file category with category name `the_name',
			-- associated directory `the_dir', group_name
			-- `the_group', directory permission string
			-- `dir_perm_string' and file permission string
			-- `file_perm_string'
		require
			name_exists: the_name /= Void
			directory_exists: the_dir /= Void
			group_exists: the_group /= Void
			dir_perm_string_exists: dir_perm_string /= Void
			valid_dir_perm_string: fu.is_valid_permission_string (dir_perm_string)
			file_perm_string_exists: file_perm_string /= Void
			valid_file_perm_string: fu.is_valid_permission_string (file_perm_string)
		do
			name := the_name
			sub_directory_name := the_dir
			group_name := the_group
			directory_permission_string := dir_perm_string
			file_permission_string := file_perm_string
		end

feature -- Properties

	name: STRING
			-- Name of category
	
	can_compute_dir_pathnames: BOOLEAN
			-- May category be used to compute directory pathnames?
		do
			Result := run_id /= Void and run_directory /= Void
		end

	can_compute_file_pathnames: BOOLEAN
			-- May category be used to compute file pathnames?
		do
			Result := False
		end

	run_id: STRING
			-- Name of run to use in pathname computations

	run_directory: STRING
			-- Name of run directory to use in pathname
			-- computations

	sub_directory_name: STRING
			-- Name of sub-directory in which files
			-- are placed

	group_name: STRING
			-- Name of group under which directory
			-- and files in directory should be
			-- created

	directory_permission_string: STRING
			-- Permissions to assign to directory

	file_permission_string: STRING
			-- Permissions to assign to files in directory

	directory_pathname: STRING
			-- Full pathname of directory for category
		do
			if sub_directory_name.is_empty then
				Result := run_directory
			else
				Result := os.full_file_name (run_directory, sub_directory_name)
			end
		end

	file_pathname (fname: STRING): STRING
			-- Full pathname of file named `fname'
			-- that is in `Current'
		do
			if fname.is_empty then
				Result := directory_pathname
			else
				Result := os.full_file_name (directory_pathname, fname)
			end
		end

feature -- Modification

	set_run_id (s: STRING)
			-- Set `run_id' to `s'
		require
			value_exists: s /= Void
			value_lowercase: equal (s, s.as_lower)
		do
			run_id := s
		end

	set_run_directory (s: STRING)
			-- Set `run_directory' to `s'
		require
			value_exists: s /= Void
		do
			run_directory := s
		ensure
			run_directory_set: run_directory = s
		end

end
