
indexing
	
	description: "An object that generates files and file categories"
	author: "David Hollenberg"
	date: "May 5, 2003"

class RC_FILE_GENERATOR
inherit
	ANY
	RC_FILE_CATEGORY_NAMES
		export
			{NONE} all
		end

	CONDITION_UTILITIES
		export	
			{NONE} all	
		end

create
	make


feature {NONE} -- Creation

	make (run_dir_name, probe_run_dir_name, a_run_id: STRING)
		require
			run_dir_name_exists: run_dir_name /= Void
			probe_run_dir_name_exists: probe_run_dir_name /= Void
			a_run_id: a_run_id /= Void
		do
			run_directory_name := run_dir_name
			probing_directory_name := probe_run_dir_name
			run_id := a_run_id
			initialize_file_categories
		end

feature -- Properties

	run_id: STRING
			-- Name of run to use in pathname computations

	run_directory_name: STRING
			-- Name of run directory to use in pathname
			-- computations

	probing_directory_name: STRING
			-- Name of run directory for probing purposes

	run_directory: RC_FILE_CATEGORY
			-- Category for run
		do
			Result := file_categories.item (Category_run)
		end
		
	probing_directory: RC_FILE_CATEGORY
			-- Category for probing files
		do
			Result := file_categories.item (Category_probing)
		end
		
	state_directory: RC_FILE_CATEGORY
			-- Category for state information
		do
			Result := file_categories.item (Category_state)
		end
		
	state_file (step: STRING version: INTEGER): RC_FILE
			-- State file for `step' with version number `version'
		require
			step_exists: step /= Void
			nonnegative_version: version >= 0
		do
			Result := file_categories.item (Category_state).twin
			Result.set_file_name (step.as_lower + "." + version.out)
		end
		
	site_numbers_map_ps_file: RC_FILE
			-- PostScript file with mask site numbers map
		do
			Result := postscript_name (Site_numbers_root)
		end
		
	site_numbers_map_pdf_file: RC_FILE
			-- PDF file with mask site numbers map
		do
			Result := pdf_name (Site_numbers_root)
		end
		
	organizations_map_ps_file: RC_FILE
			-- PostScript file with organizations and sponsors map
		do
			Result := postscript_name (Organizations_root)
		end
		
	organizations_map_pdf_file: RC_FILE
			-- PDF file with organizations and sponsors map
		do
			Result := pdf_name (Organizations_root)
		end
		
	design_names_map_ps_file: RC_FILE
			-- PostScript file with design names map
		do
			Result := postscript_name (Design_names_root)
		end
		
	design_names_map_pdf_file: RC_FILE
			-- PDF file with design names map
		do
			Result := pdf_name (Design_names_root)
		end
		
	design_numbers_map_ps_file: RC_FILE
			-- PostScript file with design numbers map
		do
			Result := postscript_name (Design_numbers_root)
		end
		
	design_numbers_map_pdf_file: RC_FILE
			-- PDF file with design numbers map
		do
			Result := pdf_name (Design_numbers_root)
		end
		
	step_pattern_map_ps_file: RC_FILE
			-- PostScript file with field step pattern map
		do
			Result := postscript_name (Step_pattern_root)
		end
		
	step_pattern_map_pdf_file: RC_FILE
			-- PDF file with field step pattern map
		do
			Result := pdf_name (Step_pattern_root)
		end
		
	detailed_step_pattern_map_ps_file: RC_FILE
			-- PostScript file with detailed field step pattern map
		do
			Result := postscript_name (Detailed_step_pattern_root)
		end
		
	detailed_step_pattern_map_pdf_file: RC_FILE
			-- PDF file with detailed field step pattern map
		do
			Result := pdf_name (Detailed_step_pattern_root)
		end
		
	cut_patterns_map_ps_file (lot: RUN_PACKAGING_LOTS_ROW short_org, etest_id: STRING): RC_FILE
			-- PostScript file with cut patterns map for `lot'
		require
			short_org_not_void: short_org /= Void
			etest_id_not_void: etest_id /= Void
		do
			if lot /= Void then
				Result := postscript_name (cut_patterns_base (Cut_patterns_root, lot, short_org, etest_id))
			else
				Result := postscript_name (Cut_patterns_root + Unassigned_suffix)
			end
		end
		
	cut_patterns_map_pdf_file (lot: RUN_PACKAGING_LOTS_ROW short_org, etest_id: STRING): RC_FILE
			-- PDF file with cut patterns map for `lot'
		require
			short_org_not_void: short_org /= Void
			etest_id_not_void: etest_id /= Void
		do
			if lot /= Void then
				Result := pdf_name (cut_patterns_base (Cut_patterns_root, lot, short_org, etest_id))
			else
				Result := pdf_name (Cut_patterns_root + Unassigned_suffix)
			end
		end
		
	cut_patterns_ps_wildcard: RC_FILE
			-- Wildcard for PostScript cut pattern files
		do
			Result := postscript_name (Cut_patterns_root + "*")
		end
		
	cut_patterns_pdf_wildcard: RC_FILE
			-- Wildcard for PDF cut pattern files
		do
			Result := pdf_name (Cut_patterns_root + "*")
		end
		
	bond_diagrams_map_ps_file (lot: RUN_PACKAGING_LOTS_ROW short_org, etest_id: STRING): RC_FILE
			-- PostScript file with bond diagrams map for `lot'
		require
			short_org_not_void: short_org /= Void
			etest_id_not_void: etest_id /= Void
		do
			if lot /= Void then
				Result := postscript_name (bond_diagrams_base (bond_diagrams_root, lot, short_org, etest_id))
			else
				Result := postscript_name (Bond_diagrams_root + Unassigned_suffix)
			end
		end
		
	bond_diagrams_map_pdf_file (lot: RUN_PACKAGING_LOTS_ROW short_org, etest_id: STRING): RC_FILE
			-- PDF file with bond diagrams map for `lot'
		require
			short_org_not_void: short_org /= Void
			etest_id_not_void: etest_id /= Void
		do
			if lot /= Void then
				Result := pdf_name (bond_diagrams_base (bond_diagrams_root, lot, short_org, etest_id))
			else
				Result := pdf_name (Bond_diagrams_root + Unassigned_suffix)
			end
		end
		
	bond_diagram_ps_file (icp: IC_PACKAGINGS_ROW): RC_FILE
			-- PostScript file to generate for `icp' that needs
			-- diagram in an individual file
		require
			icp_not_void: icp /= Void
		do
			Result := file_categories.item (Category_run).twin
			Result.set_file_name (icp.final_diagram_rundir_name (PostScript_ext))
		end
		
	bond_diagram_pdf_file (icp: IC_PACKAGINGS_ROW): RC_FILE
			-- PDF file to generate for `icp' that needs
			-- diagram in an individual file
		require
			icp_not_void: icp /= Void
		do
			Result := file_categories.item (Category_run).twin
			Result.set_file_name (icp.final_diagram_rundir_name (Pdf_ext))
		end
		
	manual_bond_diagram_cif_file (icp: IC_PACKAGINGS_ROW): RC_FILE
			-- CIF file to generate for `icp' that needs
			-- manual bonding
		require
			icp_not_void: icp /= Void
		do
			Result := file_categories.item (Category_run).twin
			Result.set_file_name (manual_bond_file_name (icp, Cif_ext))
		end
		
	manual_bond_diagram_dxf_file (icp: IC_PACKAGINGS_ROW): RC_FILE
			-- DXF file to generate for `icp' that needs
			-- a manual bonding
		require
			icp_not_void: icp /= Void
		do
			Result := file_categories.item (Category_run).twin
			Result.set_file_name (manual_bond_file_name (icp, Dxf_ext))
		end
		
	manual_bond_diagram_ps_file (icp: IC_PACKAGINGS_ROW): RC_FILE
			-- PostScript file to generate for `icp' that needs
			-- a manual bonding
		require
			icp_not_void: icp /= Void
		do
			Result := file_categories.item (Category_run).twin
			Result.set_file_name (manual_bond_file_name (icp, Postscript_ext))
		end
		
	manual_bond_diagram_pdf_file (icp: IC_PACKAGINGS_ROW): RC_FILE
			-- PDF file to generate for `icp' that needs
			-- a manual bonding
		require
			icp_not_void: icp /= Void
		do
			Result := file_categories.item (Category_run).twin
			Result.set_file_name (manual_bond_file_name (icp, Pdf_ext))
		end
		
	bond_diagrams_run_ps_file: RC_FILE
			-- PostScript file with bond diagrams for entire run
		do
			Result := postscript_name (Bond_diagrams_root + All_suffix)
		end
		
	bond_diagrams_run_pdf_file: RC_FILE
			-- PDF file with bond diagrams for entire run
		do
			Result := pdf_name (Bond_diagrams_root + All_suffix)
		end
		
	bond_diagrams_ps_wildcard: RC_FILE
			-- Wildcard for PostScript bond diagram files
		do
			Result := postscript_name (Bond_diagrams_root + "*")
		end
		
	bond_diagrams_pdf_wildcard: RC_FILE
			-- Wildcard for PDF bond diagram files
		do
			Result := pdf_name (Bond_diagrams_root + "*")
		end
		
	bond_diagrams_cif_wildcard: RC_FILE
			-- Wildcard for CIF bond diagram files
		do
			Result := cif_name (Autocad_files_wildcard)
		end
		
	bond_diagrams_dxf_wildcard: RC_FILE
			-- Wildcard for DXF bond diagram files
		do
			Result := dxf_name (Autocad_files_wildcard)
		end
		
	individual_diagrams_ps_wildcard: RC_FILE
			-- Wildcard for PostScript bond diagram
			-- in individual files
		do
			Result := postscript_name (Individual_bond_diagrams_wildcard)
		end
		
	individual_diagrams_pdf_wildcard: RC_FILE
			-- Wildcard for PDF bond diagram files
			-- in individual files
		do
			Result := pdf_name (Individual_bond_diagrams_wildcard)
		end
		
	main_wafer_file (prober_id: STRING): RC_FILE
			-- Main wafer file used by probing software for
			-- prober identified by `prober_id'
		require
			prober_id_exists: prober_id /= Void
		do
			Result := file_categories.item (Category_probing).twin
			Result.set_file_name (run_id + "_" + prober_id + ".waf")
		end
		
	aux_wafer_file (prober_id: STRING rot: INTEGER): RC_FILE
			-- Auxiliary wafer file used by probing software for
			-- prober identified by `prober_id', which contains
			-- test vehicles whose rotation is `rot' degrees
			-- clockwise
		require
			prober_id_exists: prober_id /= Void
		do
			Result := file_categories.item (Category_probing).twin
			Result.set_file_name (run_id + "_" + rot.out + "_" + prober_id + ".waf")
		end
		
	merge_job_file: RC_FILE
			-- Job file used only to merge individual pattern
			-- files into one pattern file per mask level/field
			-- combination
		do
			Result := file_categories.item (Category_run).twin
			Result.set_file_name (run_id + "_full.jb")
		end
		
	final_job_file: RC_FILE
			-- Final job file to be sent to mask shop
		do
			Result := file_categories.item (Category_run).twin
			Result.set_file_name (run_id + ".job")
		end
		
	wafer_techn_survey_log_file: RC_FILE
			-- Log file with output from survey of wafer techn
			-- for run
		do
			Result := file_categories.item (Category_run).twin
			Result.set_file_name (run_id + "_wafer_techn_survey.rpt")
		end
		
	fill_exclusion_file: RC_FILE
			-- File with commands to CATS to build solid blocks
			-- covering projects and parts of vendor frame that
			-- are not to be filled
		do
			Result := file_categories.item (Category_run).twin
			Result.set_file_name (run_id + "_fill_exclusion.cinc")
		end
		
	die_scribe_layout_file (die_id: STRING): RC_FILE
			-- Layout file for scribe with ID `die_id'
		require
			die_id_exists: die_id /= Void
		do
			Result := file_categories.item (Category_run).twin
			Result.set_file_name (run_id + die_id.as_lower + Cif_ext)
		end
		
	die_filler_layout_file (die_id: STRING): RC_FILE
			-- Layout file for filler with ID `die_id'
		require
			die_id_exists: die_id /= Void
		do
			Result := file_categories.item (Category_run).twin
			Result.set_file_name (run_id + die_id.as_lower + Cif_ext)
		end
		
	die_overhead_layout_file (die_id: STRING): RC_FILE
			-- Layout file for die with ID `die_id'
		require
			die_id_exists: die_id /= Void
		do
			Result := file_categories.item (Category_run).twin
			Result.set_file_name (run_id + die_id.as_lower + "_ovrhd" + Cif_ext)
		end
		
	die_merged_gds_layout_file (die_id: STRING): RC_FILE
			-- Layout file for merged GDS for die with `die_id'
		require
			die_id_not_void: die_id /= Void
			die_id_not_empty: not die_id.is_empty
		do
			Result := file_categories.item (Category_run).twin
			Result.set_file_name (run_id + die_id.as_lower + Gds_ext)
		end
		
	die_merged_gds_log_file (die_id: STRING): RC_FILE
			-- Log file for GDS merge for die with `die_id'
		require
			die_id_not_void: die_id /= Void
			die_id_not_empty: not die_id.is_empty
		do
			Result := file_categories.item (Category_run).twin
			Result.set_file_name (run_id + die_id.as_lower + Log_ext)
		end
		
	field_merged_gds_layout_file (a_field_name: STRING): RC_FILE
			-- Layout file for merged GDS for field named
			-- `a_field_name'
		require
			a_field_name_not_void: a_field_name /= Void
			a_field_name_not_empty: not a_field_name.is_empty
		do
			Result := file_categories.item (Category_run).twin
			Result.set_file_name (run_id + a_field_name.as_lower + Gds_ext)
		end
		
	field_merged_gds_log_file (a_field_name: STRING): RC_FILE
			-- Log file for GDS merge for field name `a_field_name'
		require
			a_field_name_not_void: a_field_name /= Void
			a_field_name_not_empty: not a_field_name.is_empty
		do
			Result := file_categories.item (Category_run).twin
			Result.set_file_name (run_id + a_field_name.as_lower + Log_ext)
		end
		
	die_fracture_script_file (die_id: STRING): RC_FILE
			-- Script file for fracturing `die_id'
		require
			die_id_exists: die_id /= Void
		do
			Result := file_categories.item (Category_run).twin
			Result.set_file_name (run_id + die_id.as_lower + ".csh")
		end
		
	die_fracture_log_file (die_id: STRING): RC_FILE
			-- Log file for output of script that fractures
			-- `die_id'
		require
			die_id_exists: die_id /= Void
		do
			Result := file_categories.item (Category_run).twin
			Result.set_file_name (run_id + die_id.as_lower + Log_ext)
		end
		
	die_fracture_done_file (die_id: STRING): RC_FILE
			-- The "is_done" file created by execution of script
			-- that fractures `die_id'
		require
			die_id_exists: die_id /= Void
		do
			Result := file_categories.item (Category_run).twin
			Result.set_file_name (run_id + die_id.as_lower + ".is_done")
		end
		
	die_fracture_pattern_file (die_id: STRING a_rotation: INTEGER overetch: BOOLEAN ext: STRING): RC_FILE
			-- The pattern file for `die_id' where
			-- `overetch' indicates whether mask techn for
			-- run overetches, `a_rotation' is the
			-- rotation of the die in degrees clockwise
			-- and `ext' is the pattern file extension
		require
			die_id_exists: die_id /= Void
			a_rotation_valid: a_rotation = 0 or a_rotation = 90 or a_rotation = 180 or a_rotation = 270
			extension_exists: ext /= Void
		local
			overetch_text, rotated_text: STRING
		do
			if overetch then
				overetch_text := "x"
			else
				overetch_text := "0"
			end
			inspect a_rotation
			when 0 then
				rotated_text := "0"
			when 90 then
				rotated_text := "r"
			when 180 then
				rotated_text := "u"
			when 270 then
				rotated_text := "l"
			end
			Result := file_categories.item (Category_run).twin
			Result.set_file_name (run_id + die_id.as_lower + "0" + overetch_text + rotated_text + "." + ext.as_lower)
		end
		
	level_merge_log_file (mebes_ext: STRING): RC_FILE
			-- Log file for merging level whose
			-- MEBES extension is `mebes_ext'
		require
			mebes_extension_exists: mebes_ext /= Void
		do
			Result := file_categories.item (Category_run).twin
			Result.set_file_name (run_id + "_merge_" + mebes_ext.as_lower + Log_ext)
		end
		
	level_merge_done_file (mebes_ext: STRING): RC_FILE
			-- The "is_done" file created by execution of script
			-- that merges level whose MEBES extension is
			-- `mebes_ext'
		require
			mebes_extension_exists: mebes_ext /= Void
		do
			Result := file_categories.item (Category_run).twin
			Result.set_file_name (run_id + "_merge_" + mebes_ext.as_lower + ".is_done")
		end
		
	level_merge_pattern_file (n, field_count: INTEGER mebes_ext: STRING): RC_FILE
			-- The pattern file field `n' of `field_count' fields
			-- with extension `mebes_ext'
		require
			field_count_positive: field_count >= 1
			n_positive: n >= 1
			n_small_enough: n <= field_count
			mebes_extension_exists: mebes_ext /= Void
		do
			Result := level_merge_root_name (n, field_count)
			Result.set_file_name (Result.file_name + "." + mebes_ext.as_lower)
		end
		
	level_merge_root_name (n, field_count: INTEGER): RC_FILE
			-- The root name of the merged pattern file for
			-- field `n' of `field_count' fields, with no extension
		require
			field_count_positive: field_count >= 1
			n_positive: n >= 1
			n_small_enough: n <= field_count
		local
			name: STRING
		do
			name := field_name_part (n, field_count)
			Result := file_categories.item (Category_run).twin
			Result.set_file_name (run_id + "00" + name)
		end
		
	level_fill_log_file (mebes_ext: STRING): RC_FILE
			-- Log file for filling level whose
			-- MEBES extension is `mebes_ext'
		require
			mebes_extension_exists: mebes_ext /= Void
		do
			Result := file_categories.item (Category_run).twin
			Result.set_file_name (run_id + "_fill_" + mebes_ext.as_lower + Log_ext)
		end
		
	level_fill_done_file (mebes_ext: STRING): RC_FILE
			-- The "is_done" file created by execution of script
			-- that fill level whose MEBES extension is
			-- `mebes_ext'
		require
			mebes_extension_exists: mebes_ext /= Void
		do
			Result := file_categories.item (Category_run).twin
			Result.set_file_name (run_id + "_fill_" + mebes_ext.as_lower + ".is_done")
		end
		
	level_fill_pattern_file (n, field_count: INTEGER mebes_ext: STRING): RC_FILE
			-- The pattern file field `n' of `field_count' fields
			-- with extension `mebes_ext'
		require
			field_count_positive: field_count >= 1
			n_positive: n >= 1
			n_small_enough: n <= field_count
			mebes_extension_exists: mebes_ext /= Void
		do
			Result := filled_mebes_root_name (n, field_count)
			Result.set_file_name (Result.file_name + "." + mebes_ext.as_lower)
		end
		
	filled_mebes_root_name (n, field_count: INTEGER): RC_FILE
			-- The root name of the filled pattern file
			-- for field `n' of `field_count' fields, with
			-- no extension
		require
			field_count_positive: field_count >= 1
			n_positive: n >= 1
			n_small_enough: n <= field_count
		local
			name: STRING
		do
			name := field_name_part (n, field_count)
			Result := file_categories.item (Category_run).twin
			Result.set_file_name (run_id + "99" + name)
		end
		
	fill_selector_log_file: RC_FILE
			-- Log file for job that creates fill selector file
		do
			Result := file_categories.item (Category_run).twin
			Result.set_file_name (run_id + "_fill_selector" + Log_ext)
		end
		
	fill_selector_done_file: RC_FILE
			-- The "is_done" file created by execution of script
			-- that creates fill selector file
		do
			Result := file_categories.item (Category_run).twin
			Result.set_file_name (run_id + "_fill_selector.is_done")
		end
		
	fill_exclusion_zone_file: RC_FILE
			-- The .cflt file with the total fill exclusion
			-- zone (includes projects that are excluded from fill
			-- plus areas that cannot be filled because
			-- of exclusion mask based on OR of exclusion levels)
		do
			Result := file_categories.item (Category_run).twin
			Result.set_file_name (run_id + "_fill_xx.cflt")
		end
		
	fill_selector_file: RC_FILE
			-- The .cflt file created to select areas to be filled
		do
			Result := file_categories.item (Category_run).twin
			Result.set_file_name (run_id + "_fill_se.cflt")
		end
		
	fill_reset_file: RC_FILE
			-- The .cinc file created to setup CATS for fill
		do
			Result := file_categories.item (Category_run).twin
			Result.set_file_name (run_id + "_fill_reset.cinc")
		end
		
	project_summary_ps_file: RC_FILE
			-- PostScript file with project summary report
		do
			Result := postscript_name (Project_summary_root)
		end
		
	project_summary_pdf_file: RC_FILE
			-- PDF file with project summary report
		do
			Result := pdf_name (Project_summary_root)
		end
		
	line_item_report_output_file: RC_FILE
			-- Output file from running the line item report
		do
			Result := file_categories.item (Category_run).twin
			Result.set_file_name (run_id + "_line_item.rpt")
		end
		
	payload_charges_report_output_file: RC_FILE
			-- Output file from running the payload and
			-- charges report
		do
			Result := file_categories.item (Category_run).twin
			Result.set_file_name (run_id + "_payload_and_charges.rpt")
		end
		
	costs_and_margin_report_output_file: RC_FILE
			-- Output file from running the costs and
			-- margin report
		do
			Result := file_categories.item (Category_run).twin
			Result.set_file_name (run_id + "_costs_and_margin.rpt")
		end
		
	sales_order_report_output_file: RC_FILE
			-- Output file from running the sales order
			-- report
		do
			Result := file_categories.item (Category_run).twin
			Result.set_file_name (run_id + "_sales_order.rpt")
		end
		
	eccn_report_output_file: RC_FILE
			-- Output file from running the ECCN report
		do
			Result := file_categories.item (Category_run).twin
			Result.set_file_name (run_id + "_eccn.rpt")
		end
		
	temp_packer_box_file: RC_FILE
			-- Temporary file to which to write packer boxes
			-- for old ICL code on VMS (will be renamed to
			-- `final_packer_box_file' later)
		do
			Result := file_categories.item (Category_run).twin
			Result.set_file_name (run_id + "_temporary.boxes")
		end
		
	final_packer_box_file: RC_FILE
			-- Final file to which to write packer boxes
		do
			Result := file_categories.item (Category_run).twin
			Result.set_file_name (run_id + "_selected.boxes")
		end

	
	dynamic_vendor_geometry_file(g: GEOMETRIES_ROW sps: SUBSTITUTION_PARAMETER_SET): RC_FILE
		-- create an RC_FILE for g, a (dynamic) vendor geometry item
		-- that is, a file in the run directory
		do
			Result := file_categories.item (Category_run).twin
			Result.set_file_name (g.final_geometry_filename (sps))
		end
		
feature {NONE} -- Implementation
	
	file_categories: HASH_TABLE [RC_FILE, STRING]
			-- Table of known file categories

	initialize_file_categories
			-- Initialize table of known file categories
		require
			run_directory_name_exists: run_directory_name /= Void
			run_id_exists: run_id /= Void
		local
			table: like file_categories
		do
			create table.make (29)
			add_std_category (Category_run, run_directory_name, "", table)
			add_std_category (Category_probing, probing_directory_name, "", table)
			add_std_category (Category_state, run_directory_name, "state", table)
			file_categories := table
		end

	add_std_category (cat_name, run_dir, subdir: STRING table: HASH_TABLE [FILE_CATEGORY, STRING])
		require
			category_name_exists: cat_name /= Void
			run_dir_exists: run_dir /= Void
			subdir_exists: subdir /= Void
			table_exists: table /= Void
		local
			cat: RC_FILE
		do
			create cat.make (cat_name, subdir, Std_group, Std_dir_perm, Std_file_perm)
			cat.set_run_id (run_id)
			cat.set_run_directory (run_dir)
			table.put (cat, cat_name)
		end

	field_name_part (n, field_count: INTEGER): STRING
			-- Field name part of file name for
			-- field `n' of `field_count' fields
		require
			field_count_positive: field_count >= 1
			n_positive: n >= 1
			n_small_enough: n <= field_count
		do
			if field_count = 1 then
				Result := "fld"
			elseif n < 10 then
				Result := "f0" + n.out
			else
				Result := "f" + n.out
			end
		end
		
	cut_patterns_base (root: STRING lot: RUN_PACKAGING_LOTS_ROW short_org, etest_id: STRING): STRING
			-- Base file name (without directory or extension)
			-- for cut pattern file
		require
			root_not_void: root /= Void
			lot_not_void: lot /= Void
			short_org_not_void: short_org /= Void
			etest_id_not_void: etest_id /= Void
		do
			Result := Cut_patterns_root + lot.packaging_lot_no.out + "_" + short_org.as_lower + "_" + etest_id.as_lower
		end
		
	bond_diagrams_base (root: STRING lot: RUN_PACKAGING_LOTS_ROW short_org, etest_id: STRING): STRING
			-- Base file name (without directory or extension)
			-- for bonding diagrams file
		require
			root_not_void: root /= Void
			lot_not_void: lot /= Void
			short_org_not_void: short_org /= Void
			etest_id_not_void: etest_id /= Void
		do
			Result := Bond_diagrams_root + lot.packaging_lot_no.out + "_" + short_org.as_lower + "_" + etest_id.as_lower
		end
		
	postscript_name (root: STRING): RC_FILE
			-- Name of Postscript file for `run_id' with `root'
			-- in run directory
		require
			root_exists: root /= Void
			root_non_empty: not root.is_empty
		do
			Result := file_categories.item (Category_run).twin
			Result.set_file_name (run_id + "_" + root + Postscript_ext)
		end
		
	pdf_name (root: STRING): RC_FILE
			-- Name of PDF file for `run_id' with `root'
			-- in run directory
		require
			root_exists: root /= Void
			root_non_empty: not root.is_empty
		do
			Result := file_categories.item (Category_run).twin
			Result.set_file_name (run_id + "_" + root + Pdf_ext)
		end
		
	cif_name (root: STRING): RC_FILE
			-- Name of CIF file for `run_id' with `root'
			-- in run directory
		require
			root_exists: root /= Void
			root_non_empty: not root.is_empty
		do
			Result := file_categories.item (Category_run).twin
			Result.set_file_name (run_id + "_" + root + Cif_ext)
		end
		
	dxf_name (root: STRING): RC_FILE
			-- Name of DXF file for `run_id' with `root'
			-- in run directory
		require
			root_exists: root /= Void
			root_non_empty: not root.is_empty
		do
			Result := file_categories.item (Category_run).twin
			Result.set_file_name (run_id + "_" + root + Dxf_ext)
		end
		
	manual_bond_file_name (icp: IC_PACKAGINGS_ROW ext: STRING): STRING
			-- File name of manual bonding-related file for `icp',
			-- with extension `ext'
		require
			icp_not_void: icp /= Void
			ext_not_void: ext /= Void
		do
			Result := run_id + "_" + icp.die_id.as_lower + "_" + icp.design_no.out + "_"+ icp.bonding_name.as_lower + "_chip" + ext
		end
		
	Std_group: STRING = "vlsi"

	Std_file_perm: STRING = "rw-rw----"

	Std_dir_perm: STRING = "rwxrwx---"

	Postscript_ext: STRING = ".ps"

	Log_ext: STRING = ".log"

	Pdf_ext: STRING = ".pdf"

	Cif_ext: STRING = ".cif"

	Gds_ext: STRING = ".gds"

	Dxf_ext: STRING = ".dxf"

	Site_numbers_root: STRING = "site_numbers_map"

	Organizations_root: STRING = "organizations_map"

	Design_names_root: STRING = "design_names_map"

	Design_numbers_root: STRING = "design_numbers_map"

	Step_pattern_root: STRING = "step_pattern_map"

	Detailed_step_pattern_root: STRING = "detailed_step_pattern_map"

	Cut_patterns_root: STRING = "cut_patterns_"

	Bond_diagrams_root: STRING = "bond_diagrams_"

	Project_summary_root: STRING = "project_summary"

	Unassigned_suffix: STRING = "unassigned"

	All_suffix: STRING = "all"

	Autocad_files_wildcard: STRING = "*_*_chip"

	Individual_bond_diagrams_wildcard: STRING = "*_*_diagram"

invariant
	file_categories_exists: file_categories /= Void
end
