
indexing
	
	description: "Window to allow printing of maps for %
		%a specific run"
	author: "David Hollenberg"
	date: "March 10, 2005"

deferred class RC_MAP_PRINT_WINDOW

inherit
	RC_MAP_PRINT_IFC
	WINDOW_INTERFACE
		redefine
			enable_active,
			disable_active
		end
	MOSIS_GRAPHICAL_UTILITIES
		export
			{NONE} all
		undefine
			default_create
		end
	RC_ENVIRONMENT_ACCESS
		export
			{NONE} all
		undefine
			default_create
		end
	UNIX_OS_ACCESS
		export
			{NONE} all
		undefine
			default_create
		end
		
feature {NONE} -- Creation

	make (w: EV_TITLED_WINDOW)
			-- Create interface in window `w'
		require
			window_exists: w /= Void
			window_valid: not w.is_destroyed
		do
			make_interface
			make_proxy (w)
		end

feature -- Operations

	reset
			-- Move focus back into window
		do
			enable_active
			-- center_pointer
		end

	refresh
			-- Refresh list of files
		do
			process_zoom_out
		end

feature -- Modification

	enable_active
			-- Make interface active
		do
			precursor
			cancel_button.enable_sensitive
		end

	disable_active
			-- Make interface inactive
			-- (can look but cannot initiate most actions)
		do
			precursor
			cancel_button.disable_sensitive
		end

feature -- Properties

	is_built: BOOLEAN
			-- Has interface been built?

	has_file_name_items: BOOLEAN
			-- Is the data associated with each list item
			-- that can be printed a file name?  If not,
			-- the associated data is a data source that
			-- provides the data to be printed
		deferred
		end

feature {NONE} -- Implementation

	refresh_button: EV_BUTTON
			-- Button to refresh list of maps
	
	zoom_button: EV_BUTTON
			-- Button to zoom in on individual pages of
			-- a PostScript file or zoom back out
	
	cancel_button: EV_BUTTON
			-- Button to end session
	
	build_interface
			-- Build interface
		local
			buttons: HORIZONTAL_BUTTON_SET
			main_vbox: EV_VERTICAL_BOX
			box: EV_BOX
			cols: WIDGET_COLUMN_SET
		do
			create maps
			maps.enable_multiple_selection
			maps.select_actions.extend (agent select_one_file)
			maps.deselect_actions.extend (agent deselect_one_file)
			
			create printer_combo_box
			printer_combo_box.set_minimum_width (200)
			printer_combo_box.disable_edit
			create status_label
			
			fetch_data
			add_menu_bar

			create main_vbox
			main_vbox.set_border_width (user_interface.border_width)
			create cols.make (2)
			cols.set_padding (user_interface.padding)
			cols.set_border_width (user_interface.border_width)
			cols.append_row (<< gui.label_with_text ("Printer"), printer_combo_box >>)
			cols.disable_expand
			cols.align_center
			box := cols.box
			main_vbox.extend (box)
			main_vbox.disable_item_expand (box)
			
			main_vbox.extend (maps)
			main_vbox.extend (status_label)
			main_vbox.disable_item_expand (status_label)

			create buttons.make_with_text_and_actions (<< "All", "None", "Invert", "Refresh", "Zoom In" >>, << agent process_select_all, agent process_select_none, agent process_select_invert, agent process_refresh, agent process_zoom >>)
			refresh_button := buttons.i_th (4)
			zoom_button := buttons.i_th (5)
			zoom_button.disable_sensitive
			box := buttons.column_set.box
			
			main_vbox.extend (box)
			main_vbox.disable_item_expand (box)
			
			create buttons.make_with_text_and_actions (<< "Print", "Save", "Cancel" >>, << agent process_print, agent process_save, agent process_cancel >>)
			
			cancel_button := buttons.i_th (2)
			
			box := buttons.column_set.box
			
			main_vbox.extend (box)
			main_vbox.disable_item_expand (box)
			
			gui.set_box_background_recursive (main_vbox, background_color)
			extend (main_vbox)
			gui.add_cancel_accelerator (window, agent process_cancel)
			gui.do_once_on_idle (agent post_display_actions)
			refresh
			is_built := True
		end
		
	fetch_data
			-- Fetch needed data
		do
			set_default_printer
			fetch_printers
			populate_printers
		end

	add_menu_bar
			-- Add menu bar to window
		local
			file_menu, select_menu: EV_MENU
			menu_item, exit_menu_item: EV_MENU_ITEM
			close_menu_item: EV_MENU_ITEM
			a_menu_bar: EV_MENU_BAR
		do	
			create file_menu.make_with_text ("File")
			
			create close_menu_item.make_with_text ("Close")
			close_menu_item.select_actions.extend (agent process_cancel)
			file_menu.extend (close_menu_item)
			
			create exit_menu_item.make_with_text ("Exit")
			exit_menu_item.select_actions.extend (gui.destroy_application_agent)
			file_menu.extend (exit_menu_item)
			
			create select_menu.make_with_text ("Select")
			
			create menu_item.make_with_text ("All")
			menu_item.select_actions.extend (agent process_select_all)
			select_menu.extend (menu_item)
			
			create menu_item.make_with_text ("None")
			menu_item.select_actions.extend (agent process_select_none)
			select_menu.extend (menu_item)
			
			create menu_item.make_with_text ("Invert")
			menu_item.select_actions.extend (agent process_select_invert)
			select_menu.extend (menu_item)
			
			add_extra_select_menu_items (select_menu)
			
			create a_menu_bar
			a_menu_bar.extend (file_menu)
			a_menu_bar.extend (select_menu)
			set_menu_bar (a_menu_bar)
		end

	add_extra_select_menu_items (select_menu: EV_MENU)
		require
			select_menu_not_void: select_menu /= Void
		deferred
		end

	set_default_printer
			-- Set device id of default printer from
			-- environment variable.  Set to empty if environment
			-- variable not defined
		do
			default_printer := exe.get (session.environment.printer_environment_var)
		end

	fetch_printers
			-- Fetch list of printers
		local
			row: M_DEVICES_ROW
			env: OPERATING_ENVIRONMENT_VALUES
			device_class: DEVICE_CLASS_VALUES
			medium_type: MEDIUM_TYPE_VALUES
		do
			create row.make_template
			printers := row.available_devices (env.Unix, device_class.Printer, medium_type.Paper_1_sided, session.connection, expected_count.At_least_one)
			printers.append (row.available_devices (env.Unix, device_class.Printer, medium_type.Paper_2_sided, session.connection, expected_count.At_least_zero))
		end

	populate_printers
			-- Populate `printer_combo_box'
		do
			printer_combo_box.wipe_out
			printer_combo_box.enable_edit
			default_printer_item := Void
			printers.do_all (agent add_printer)
			printer_combo_box.disable_edit
			if default_printer_item /= Void then
				default_printer_item.enable_select
			else
				printer_combo_box.first.enable_select
			end
		end

	add_printer (p: M_DEVICES_ROW)
			-- Add item for `p' to `printer_combo_box'
		require
			p_not_void: p /= Void
		local
			list_item: EV_LIST_ITEM
			s: STRING
		do
			create s.make (20)
			s.append (p.device_id)
			if not p.is_null_device_location then
				s.append ("    " + p.device_location)
			end
			create list_item.make_with_text (s)
			list_item.set_data (p)
			printer_combo_box.extend (list_item)
			if default_printer /= Void and then p.device_id.is_equal (default_printer) then
				default_printer_item := list_item
			end
		end

	populate_maps
			-- Populate list of maps that can be printed
		deferred
		end

	populate_map_pages (s: DATA_SOURCE)
			-- Populate `maps' with list of pages in `s'
		require
			s_not_void: s /= Void
		local
			parser: POSTSCRIPT_PAGE_PARSER
		do
			maps.wipe_out
			create parser
			parser.parse_source (s)
			preamble := parser.preamble
			parser.pages.do_all (agent add_one_page)
			postamble := parser.postamble
		end

	add_one_page (page: POSTSCRIPT_PAGE)
			-- Add `page' to `maps'
		require
			page_not_void: page /= Void
		local
			list_item: EV_LIST_ITEM
			s: STRING
		do
			s := page.label.twin
			s.prune_all ('%R')
			create list_item.make_with_text (s)
			list_item.set_data (page)
			maps.extend (list_item)
		end

	add_map (map_name: STRING file: RC_FILE)
			-- Add map with symbolic name `map_name' for
			-- `file' to `maps' if `file' exists
		require
			map_name_not_void: map_name /= Void
			file_not_void: file /= Void
		local
			list_item: EV_LIST_ITEM
			fname: STRING
		do
			fname := file.file_pathname
			if session.path_exists (fname) then
				create list_item.make_with_text (map_name)
				list_item.set_data (fname)
				maps.extend (list_item)
			else
				missing_map_count := missing_map_count + 1
			end
		end

	post_display_actions
		do
			gui.do_once_on_idle (agent gui.center_window_with_size (window, 600, 700))
			reset
		end

	process_select_all
			-- Process "select all" request
		do
			if is_active then
				disable_active
				maps.do_all (agent {EV_LIST_ITEM}.enable_select)
				enable_active
			end
		end

	process_select_none
			-- Process "select none" request
		do
			if is_active then
				disable_active
				maps.do_all (agent {EV_LIST_ITEM}.disable_select)
				enable_active
			end
		end

	process_select_invert
			-- Process "invert selection" request
		do
			if is_active then
				disable_active
				maps.do_all (agent {EV_LIST_ITEM}.toggle)
				enable_active
			end
		end

	process_print
			-- Process "print" request
		do
			if is_active then
				disable_active
				is_print_to_file := False
				print_selected_items
				enable_active
			end
		end

	process_save
			-- Process "save" request (print to file instead
			-- of printer)
		do
			if is_active then
				disable_active
				is_print_to_file := True
				create operation.make (agent print_selected_items)
				operation.enable_abort_caught
				operation.execute
				enable_active
			end
		end

	print_selected_items
			-- Print files or pages that are currently selected
		do
			if is_zoomed_in then
				print_selected_map_pages
			else
				print_selected_maps
			end
		end

	print_selected_maps
			-- Print files or data sources for maps that
			-- are currently selected
		do
			if has_file_name_items then
				create files.make (maps.count)
				maps.do_if (agent add_file, agent {EV_LIST_ITEM}.is_selected)
				if files.is_empty then
					inform ("No items selected")
				else
					print_selected_files
				end
			else
				create sources.make (maps.count)
				maps.do_if (agent add_source, agent {EV_LIST_ITEM}.is_selected)
				if sources.is_empty then
					inform ("No items selected")
				else
					create sources_table.make (maps.count)
					page_count := 0
					parse_first_postscript_source (sources.first)
					sources.do_all (agent parse_postscript_source)
					print_selected_sources
				end
			end
		end

	parse_first_postscript_source (s: DATA_SOURCE)
			-- Parse `s', which is the first data source
			-- in a list of sources the contain
			-- PostScript, and set `preamble' and
			-- `postamble'
		require
			s_not_void: s /= Void
		local
			parser: POSTSCRIPT_PAGE_PARSER
		do
			create parser
			parser.parse_source (s)
			preamble := parser.preamble
			create {ARRAYED_LIST [POSTSCRIPT_PAGE]} pages.make (50)
			postamble := parser.postamble
		end

	parse_postscript_source (s: DATA_SOURCE)
			-- Parse `s', which contains PostScript, and
			-- append its pages to `pages'
		require
			s_not_void: s /= Void
		local
			parser: POSTSCRIPT_PAGE_PARSER
			list: LIST [POSTSCRIPT_PAGE]
		do
			create parser
			parser.parse_source (s)
			list := parser.pages
			pages.append (list)
			list.do_all (agent set_page_source (?, s))
		end

	set_page_source (p: POSTSCRIPT_PAGE s: DATA_SOURCE)
			-- Set source for `p' to `s'
		require
			p_not_void: p /= Void
			s_not_void: s /= Void
		do
			page_count := page_count + 1
			sources_table.put (s, page_count)
		end

	append_pages (s: DATA_SOURCE)
			-- Append to `pages' the PostScript pages in `s'
		require
			s_not_void: s /= Void
		local
			parser: POSTSCRIPT_PAGE_PARSER
		do
			maps.wipe_out
			create parser
			parser.parse_source (s)
			preamble := parser.preamble
			parser.pages.do_all (agent add_one_page)
			postamble := parser.postamble
		end

	print_selected_map_pages
			-- Print pages of current map file that are selected
		do
			create {ARRAYED_LIST [POSTSCRIPT_PAGE]} pages.make (maps.count)
			maps.do_if (agent add_page, agent {EV_LIST_ITEM}.is_selected)
			if pages.is_empty then
				inform ("No pages selected")
			else
				print_selected_pages
			end
		end

	print_selected_files
			-- Print files in `files'
		local
			list: LIST [STRING]
		do
			create_print_process
			list := standard_argument_list (files.count)
			list.append (files)
			process.set_arguments (su.list_as_array (list))
			process.set_input_file_name (unix_os.null_file_name)
			process.spawn_wait
			check_process_status (files.count, su.number (files.count, "file", "files"))
		end
	
	print_selected_sources
			-- Print data sources in `pages'
		local
			list: LIST [STRING]
			k: INTEGER
		do
			create_print_process
			list := standard_argument_list (0)
			process.set_arguments (su.list_as_array (list))
			process.set_input_piped
			process.spawn_nowait
			current_source := sources.first
			write_page (preamble)
			from
				k := 1
				pages.start
			until
				pages.after
			loop
				current_source := sources_table.item (k)
				write_page (pages.item)
				sources.forth
				k := k + 1
				pages.forth
			end
			current_source := sources.first
			write_page (postamble)
			if not process.input_to_child.is_closed then
				process.input_to_child.close
			end
			process.get_status_block
			check_process_status (pages.count, su.number (pages.count, "page", "pages"))
		end
	
	print_selected_pages
			-- Print files in `pages'
		local
			list: LIST [STRING]
		do
			create_print_process
			list := standard_argument_list (0)
			process.set_arguments (su.list_as_array (list))
			process.set_input_piped
			process.spawn_nowait
			write_page (preamble)
			pages.do_all (agent write_page)
			write_page (postamble)
			if not process.input_to_child.is_closed then
				process.input_to_child.close
			end
			process.get_status_block
			check_process_status (pages.count, su.number (pages.count, "page", "pages"))
		end
	
	create_print_process
			-- Create process to do printing and put in `process'
		local
			f: LARGE_RAW_FILE
		do
			if is_print_to_file then
				print_program := session.environment.cat_program
				prompt_for_file_names
			else
				print_program := session.environment.print_program
			end
			create f.make (print_program)
			if not f.exists then
				handle_hard_error ("Program " + print_program + " not found")
			elseif not f.is_executable then
				handle_hard_error ("Program " + print_program + " is not executable")
			else
				create process.make (print_program)
				if is_print_to_file then
					process.set_output_file_name (Postscript_output_name)
					process.set_error_file_name (unix_os.null_file_name)
				else
					process.set_output_file_name (unix_os.null_file_name)
					process.set_error_same_as_output
				end
				-- Rest of setup of `process' done by caller
			end
		end
	
	prompt_for_file_names
			-- Get names of PostScript and PDF files to write
		do
			create file_dialog
			file_dialog.cancel_actions.extend (agent process_file_dialog_cancel)
			file_dialog.save_actions.extend (agent process_file_dialog_save)
			file_dialog.show_modal_to_window (window)
		end
	
	process_file_dialog_cancel
			-- Process Cancel request in file save dialog
		do
			file_dialog.destroy
			operation.abort
		end

	process_file_dialog_save
			-- Process Save request in file save dialog
		local
			s: STRING
		do
			s := file_dialog.file_name
			if su.wildcard_matches (s, "%%*.pdf") then
				s.keep_head (s.count - 4)
			elseif su.wildcard_matches (s, "%%*.ps") then
				s.keep_head (s.count - 3)
			end
			postscript_output_name := s + ".ps"
			pdf_output_name := s + ".pdf"
			if ok_to_write (postscript_output_name) and then ok_to_write (pdf_output_name) then
				file_dialog.destroy
				gui.process_events
			else
				operation.abort
			end
		end

	ok_to_write (fname: STRING): BOOLEAN
			-- Check whether it is OK to write `fname'
		local
			op: FILE_EXISTENCE_CHECK_OP
		do
			create op
			op.check_existence (fname)
			if op.path_exists then
				Result := question ("File " + fname + " exists.  Overwrite?")
			else
				Result := True
			end
		end

	standard_argument_list (n: INTEGER): LIST [STRING]
			-- Standard argument list for printing, with
			-- space for `n' files
		do
			if is_print_to_file then
				create {ARRAYED_LIST [STRING]} Result.make (0)
			else
				create {ARRAYED_LIST [STRING]} Result.make (n + 4)
				Result.extend ("-P")
				Result.extend (current_printer_queue_name)
				Result.extend ("-J")
				Result.extend (job_name)
			end
		end

	write_page (p: POSTSCRIPT_PAGE)
			-- Write text of `p' to standard input of `process'
		require
			p_not_void: p /= void
		local
			full_count: INTEGER_64
			amt, blk_size: INTEGER
		do
			current_source.go (p.start_offset)
			from
				full_count := p.count
				blk_size := 8192
			until
				full_count <= 0
			loop
				if full_count >= blk_size then
					amt := blk_size
				else
					amt := full_count.to_integer
				end
				current_source.read_stream (amt)
				write_block (current_source.last_string)
				full_count := full_count - amt
			end
		end

	write_block (s: STRING)
			-- Write `s' to standard input of `process' if
			-- it is still open.  Close our end of pipe
			-- if get I/O error
		require
			s_not_void: s /= void
		local
			f: LARGE_RAW_FILE
			tried: BOOLEAN
		do
			if not tried then
				f := process.input_to_child
				if not f.is_closed then
					f.put_string (s)
				end
			else
				f.close
			end
		rescue
			tried := True
			retry
		end

	check_process_status (n: INTEGER what: STRING)
			-- Check exit status of `process', which was trying
			-- to print `n' items of type `what'
		local
			converter: POSTSCRIPT_CONVERTER
		do
			if process.is_exit_status (process.status) then
				if process.exit_code (process.status) = 0 then
					if is_print_to_file then
						create converter.make (session.environment.ghostscript_program)
						converter.convert_named_file_to_pdf (Postscript_output_name, Pdf_output_name)
						inform (n.out + " " + what + " saved in " + Postscript_output_name + "%N" + n.out + " " + what + " saved in " + Pdf_output_name)
					else
						inform (n.out + " " + what + " queued for printing on " + current_printer_queue_name + " in room " + current_printer_location)
					end
				else
					warn_with_notify ("Program " + print_program + " exited with code " + process.exit_code (process.status).out)
				end
			else
				warn_with_notify ("Program " + print_program + " terminated or stopped by signal " + process.signal_number (process.status).out)
			end
		end
	
	job_name: STRING
			-- Name to use for print job
		deferred
		ensure
			result_not_void: Result /= Void
			result_not_empty: not Result.is_empty
		end
	
	add_file (list_item: EV_LIST_ITEM)
			-- Add file for `list_item' to `files'
		require
			list_item_not_void: list_item /= Void
		do
			files.extend (list_item_file_name (list_item))
		end

	add_source (list_item: EV_LIST_ITEM)
			-- Add data source for `list_item' to `sources'
		require
			list_item_not_void: list_item /= Void
		do
			sources.extend (list_item_data_source (list_item))
		end

	add_page (list_item: EV_LIST_ITEM)
			-- Add page for `list_item' to `pages'
		require
			list_item_not_void: list_item /= Void
		do
			pages.extend (list_item_page (list_item))
		end

	list_item_file_name (list_item: EV_LIST_ITEM): STRING
			-- Name of file associated with `list_item'
		require
			list_item_not_void: list_item /= Void
		do
			Result ?= list_item.data
			check
				result_not_void: Result /= Void
			end
		end

	list_item_data_source (list_item: EV_LIST_ITEM): DATA_SOURCE
			-- Data source associated with `list_item'
		require
			list_item_not_void: list_item /= Void
		local
			s: DATA_SOURCEABLE
		do
			s ?= list_item.data
			check
				s_not_void: s /= Void
			end
			Result := s.data_source
		end

	list_item_page (list_item: EV_LIST_ITEM): POSTSCRIPT_PAGE
			-- Page associated with `list_item'
		require
			list_item_not_void: list_item /= Void
		do
			Result ?= list_item.data
			check
				result_not_void: Result /= Void
			end
		end

	set_missing_files_status
			-- Set text of `status_label' to indicate number
			-- of missing files
		local
			s: STRING
		do
			if missing_map_count /= 0 then
				s := "Missing files: " + missing_map_count.out
			else
				s := ""
			end
			status_label.set_text (s)
		end

	current_printer_queue_name: STRING
			-- Queue name of currently selected printer
		local
			list_item: EV_LIST_ITEM
			dev: M_DEVICES_ROW
		do
			list_item := printer_combo_box.selected_item
			dev ?= list_item.data
			check
				dev_not_void: dev /= Void
			end
			Result := dev.queue_name
		end

	current_printer_location: STRING
			-- Location of currently selected printer
		local
			list_item: EV_LIST_ITEM
			dev: M_DEVICES_ROW
		do
			list_item := printer_combo_box.selected_item
			dev ?= list_item.data
			check
				dev_not_void: dev /= Void
			end
			Result := dev.device_location
		end

	process_refresh
			-- Process "refresh" request
		do
			if is_active then
				disable_active
				refresh
				enable_active
			end
		end

	process_zoom
			-- Process "zoom" request
		do
			if is_active then
				disable_active
				if is_zoomed_in then
					process_zoom_out
				else
					process_zoom_in
				end
				enable_active
			end
		end

	process_zoom_in
			-- Process "zoom in" request
		local
			an_item: EV_LIST_ITEM
			open_op: FILE_OPEN_READ_OP
			fname: STRING
			f: LARGE_PLAIN_TEXT_FILE
		do
			an_item := maps.selected_item
			if has_file_name_items then
				fname := list_item_file_name (an_item)
				create f.make (fname)
				create open_op
				open_op.open (f)
				create {DATA_SOURCE_FILE} current_source.make_entire (f)
			else
				current_source := list_item_data_source (an_item)
			end
			populate_map_pages (current_source)
			zoom_button.set_text ("Zoom Out")
			zoom_button.enable_sensitive
			refresh_button.disable_sensitive
			status_label.set_text (an_item.text)
			is_zoomed_in := True
		end

	process_zoom_out
			-- Process "zoom out" request
		local
			-- FIXME
			-- close_op: FILE_CLOSE_OP
		do
			zoom_button.set_text ("Zoom In")
			refresh_button.enable_sensitive
			is_zoomed_in := False
			if current_source /= Void then
				-- FIXME: need to close current source
				-- with error checking and logging
				-- create close_op
				-- close_op.close (current_file)
				current_source.close
				current_source := Void
			end
			populate_maps
			set_selected_count (0)
		end

	process_cancel
			-- Process cancel request
		do
			if is_active then
				disable_active
				cancel_actions.call ([])
			end
		end

	select_one_file
			-- Take actions needed when an item in `maps'
			-- is selected
		do
			set_selected_count (selected_count + 1)
		end

	deselect_one_file
			-- Take actions needed when an item in `maps'
			-- is deselected
		do
			set_selected_count (selected_count - 1)
		end

	set_selected_count (n: INTEGER)
			-- Set `selected_count' to `n'
		require
			n_nonnegative: n >= 0
		do
			selected_count := n
			if not is_zoomed_in then
				if selected_count = 1 then
					enable_zoom_in
				else
					disable_zoom_in
				end
			end
		ensure
			selected_count_set: selected_count = n
		end

	enable_zoom_in
		do
			zoom_button.enable_sensitive
		end

	disable_zoom_in
		do
			if zoom_button.is_sensitive then
				zoom_button.disable_sensitive
			end
		end

	
	default_printer: STRING
			-- Device id of default printer, if any

	default_printer_item: EV_LIST_ITEM
			-- Item in printer combo box list whose device
			-- id matches `default_printer', if any

	printers: LIST [M_DEVICES_ROW]
			-- Available Unix printers

	printer_combo_box: EV_COMBO_BOX
			-- Combo box for selecting printer

	status_label: EV_LABEL
			-- Label with text that gives status (number of
			-- missing maps when zoomed out, symbolic name
			-- of zoomed file when zoomed in)

	maps: EV_LIST
			-- List of maps that exist and could be selected
			-- for printing

	missing_map_count: INTEGER
			-- Number of maps whose file does not exist

	selected_count: INTEGER
			-- Number of maps currently selected

	files: ARRAYED_LIST [STRING]
			-- Files user asked to print

	sources: ARRAYED_LIST [DATA_SOURCE]
			-- Data sources user asked to print

	pages: LIST [POSTSCRIPT_PAGE]
			-- Pages within current file user asked to print
			-- (when zoomed in on one file)

	page_count: INTEGER
			-- Number of pages added to `sources_table' so far

	sources_table: HASH_TABLE [DATA_SOURCE, INTEGER]
			-- Data source for each PostScript page, indexed
			-- by page number

	preamble: POSTSCRIPT_PAGE
			-- Preamble of current file when zoomed in.  Only the
			-- offset and count for page are applicable
			-- since this isn't really a PostScript page

	postamble: POSTSCRIPT_PAGE
			-- Postamble of current file when zoomed in.  Only the
			-- offset and count for page are applicable
			-- since this isn't really a PostScript page

	is_zoomed_in: BOOLEAN
			-- Are we zoomed in on the individual pages
			-- within one PostScript file?

	is_print_to_file: BOOLEAN
			-- Did user request Save (print to file) rather
			-- than Print (print to printer)?

	current_source: DATA_SOURCE
			-- Data source on which we are zoomed in, if any

	print_program: STRING
			-- Full pathname of print program

	process: UNIX_PROCESS
			-- Process to do printing

	postscript_output_name: STRING
			-- Name of file where Postscript will be written
			-- for Save

	pdf_output_name: STRING
			-- Name of file where PDF will be written
			-- for Save

	file_dialog: EV_FILE_SAVE_DIALOG
	
	operation: ANONYMOUS_OPERATION
	
end
