
indexing
	
	description: "A named packing.  Used to select and pack projects. "
	author: "David Hollenberg"
	date: "November 14, 2005"

class RC_PACKING
inherit
	ANY
	DB_OPTIONS_ACCESS
		export
			{NONE} all
		end
	ITEM_COUNT_RANGES_ACCESS
		export
			{NONE} all
		end
	CONDITION_UTILITIES
		export
			{NONE} all
		end

feature -- Properties

	packing_id: STRING
			-- Packing unique identifer
		do
			Result := packings_row.packing_id
		end

	production_id: STRING
			-- Production ID if editing cut patterns for
			-- specific production.  Void otherwise

	connection: DB_CONNECTION
			-- Database connection
		do
			Result := packings_row.any_connection
		end

	wafer_techn: STRING
			-- Wafer techn for packing
		do
			Result := packings_row.wafer_techn
		end

	wafer_techns_row: WAFER_TECHNS_ROW
			-- Wafer techns row for packing's wafer techn
		do
			Result := packings_row.wafer_techns_row
		end

	field_set: FIELD_SETS_ROW
			-- Field set for packing
		do
			Result := packings_row.field_set
		end

	litho_sizes: LITHO_SIZES_ROW
			-- Lithography sizes for packing
		do
			Result := packings_row.litho_sizes
		end

	wafer_techn_family_processes: LIST [WAFER_PROCESSES_ROW]
			-- All wafer processes for packing's wafer techn family
		do
			Result := packings_row.wafer_techn_family_processes
		end

	run_type: STRING
			-- Run type for packing
		do
			Result := packings_row.run_type
		end

	packings_row: PACKINGS_MROW
			-- Row describing packing

	versions: LIST [PACKING_VERSIONS_MROW]
			-- Versions of packing

	current_version: PACKING_VERSIONS_MROW
			-- Currently selected version of packing

	version_with_number (n: INTEGER): PACKING_VERSIONS_MROW
			-- Version of `Current' with whose version number
			-- is `n'
		local
			v: PACKING_VERSIONS_MROW
			found: BOOLEAN
		do
			from
				versions.start
			until
				versions.after or found
			loop
				v := versions.item
				if v.version_no = n then
					Result := v
					found := True
				end
				versions.forth
			end
			if not found then
				handle_internal_error ("cannot find version " + n.out + " in list of versions for packing " + packing_id)
			end
		end

	is_current_version_modified: BOOLEAN
			-- Has currently selected version of packing
			-- been modified?
		do
			Result := not current_version.is_deep_unchanged
		end

	is_version_id_used (s: STRING): BOOLEAN
			-- Has `s' already been used as the version ID
			-- in an existing version?
		do
			Result := versions.there_exists (agent {PACKING_VERSIONS_MROW}.has_version_id (s))
		end

	highest_used_version: INTEGER
			-- Highest version previously used
		do
			Result := current_version.highest_used_version
		end

	selected_projects: LIST [PACKING_PROJECTS_MROW]
			-- List of all projects currently selected
		do
			Result := current_version.selected_projects
			Result.do_all (agent {PACKING_PROJECTS_MROW}.set_selected (True))
		end

	selected_unlocked_projects: LIST [PACKING_PROJECTS_MROW]
			-- List of all projects currently selected that
			-- are not locked
		do
			uncache_fab_instances
			Result := current_version.selected_unlocked_projects
			set_selected (Result)
		end

	selected_locked_projects: LIST [PACKING_PROJECTS_MROW]
			-- List of all projects currently selected that
			-- are locked
		do
			uncache_fab_instances
			Result := current_version.selected_locked_projects
			set_selected (Result)
		end

	selected_unlocked_or_unassigned_projects (a_run_id: STRING): LIST [PACKING_PROJECTS_MROW]
			-- List of all customer projects currently selected
			-- that are not locked or are not assigned to
			-- `a_run_id'
		require
			a_run_id_not_void: a_run_id /= Void
		do
			uncache_fab_instances
			Result := current_version.selected_unlocked_or_unassigned_projects (a_run_id)
			set_selected (Result)
		end

	selected_locked_unassigned_projects (a_run_id: STRING): LIST [PACKING_PROJECTS_MROW]
			-- List of all projects currently selected that
			-- are locked and not assigned to `a_run_id'
		require
			a_run_id_not_void: a_run_id /= Void
		do
			uncache_fab_instances
			Result := current_version.selected_locked_unassigned_projects (a_run_id)
			set_selected (Result)
		end

	selected_locked_assigned_projects (a_run_id: STRING): LIST [PACKING_PROJECTS_MROW]
			-- List of all projects currently selected that
			-- are locked and are assigned to `a_run_id'
		require
			a_run_id_not_void: a_run_id /= Void
		do
			uncache_fab_instances
			Result := current_version.selected_locked_assigned_projects (a_run_id)
			set_selected (Result)
		end

	selected_customer_projects: LIST [PACKING_PROJECTS_MROW]
			-- List of all projects currently selected that
			-- are customer projects
		do
			uncache_fab_instances
			Result := current_version.selected_customer_projects
			set_selected (Result)
		end

	selected_projects_matching_filter (f: FUNCTION [ANY, TUPLE [PACKING_PROJECTS_MROW], BOOLEAN]): LIST [PACKING_PROJECTS_MROW]
			-- List of all projects that are currently selected
			-- and match filter `f'
		do
			Result := current_version.selected_projects_matching_filter (f)
			Result.do_all (agent {PACKING_PROJECTS_MROW}.set_selected (True))
		end

	unselected_assigned_fab_instances (a_run_id: STRING): LIST [IC_FAB_INSTANCES_ROW]
			-- List of all fab instances that are not included
			-- in the current packing version, but are assigned
			-- to `a_run_id'
		require
			a_run_id_not_void: a_run_id /= Void
		do
			Result := current_version.unselected_assigned_fab_instances (a_run_id)
		end

	uncache_fab_instances
			-- Uncache fab instances for all selected projects
		do
			selected_projects.do_all (agent {PACKING_PROJECTS_MROW}.uncache_fab_instance)
		end

	set_selected (list: LIST [PACKING_PROJECTS_MROW])
			-- Mark each project in `list' as "selected"
		do
			list.do_all (agent {PACKING_PROJECTS_MROW}.set_selected (True))
		end

	deselected_projects: LIST [PACKING_PROJECTS_MROW]
			-- List of all projects which were selected previously
			-- but are now deselected
		local
			pair: DB_ROW_SET_PAIR [PACKING_PROJECTS_MROW]
			tester: PACKING_PROJECT_EQUALITY_TESTER [PACKING_PROJECTS_MROW]
		do
			create tester
			create pair.make_from_lists (all_projects, current_version.selected_projects, tester)
			Result := pair.subtraction
			Result.do_all (agent {PACKING_PROJECTS_MROW}.set_selected (False))
		end

	deselected_projects_matching_filter (f: FUNCTION [ANY, TUPLE [PACKING_PROJECTS_MROW], BOOLEAN]): LIST [PACKING_PROJECTS_MROW]
			-- List of all projects which were selected previously
			-- but are now deselected and which match filter `f'
		local
			list: like deselected_projects
		do
			list := deselected_projects
			Result := list.twin
			Result.wipe_out
			list.do_if (agent Result.extend, f)
			Result.do_all (agent {PACKING_PROJECTS_MROW}.set_selected (False))
		end

	all_projects: LIST [PACKING_PROJECTS_MROW]
			-- List of all projects for packing (highest version
			-- only, for each one)
		local
			row: PACKING_PROJECTS_MROW
		do
			create row.make_template
			Result := row.highest_version_rows_for_packing (packing_id, connection, expected_count.At_least_zero)
		end

	cust_project_highest_version (mfn: INTEGER): like anchored_project
			-- Highest version of cust project whose
			-- fab number is `mfn', among all versions of
			-- packing.  Void if not in any versions.
			-- Must have previously called
			-- `setup_for_fast_project_access'
		do
			Result := cust_project_table.item (mfn)
		end

	test_vehicle_highest_version (dno: INTEGER): like anchored_project
			-- Highest version of test vehicle project whose
			-- design number is `dno', among all versions of
			-- packing.  Void if not in any versions.
			-- Must have previously called
			-- `setup_for_fast_project_access'
		do
			Result := test_vehicle_project_table.item (dno)
		end

feature -- Modification

	set_packings_row (p: like packings_row)
			-- Set `packings_row' to `p'
		do
			packings_row := p
			packings_row.set_parent (Current)
		end

	set_production_id (an_id: like production_id)
			-- Set `production_id' to `an_id'
		do
			production_id := an_id
		end

	set_current_version (pv: like current_version)
			-- Set `current_version' to `pv' and detach it
		require
			pv_not_void: pv /= Void
		do
			current_version := pv
			current_version.set_packing (packings_row)
			current_version.set_highest_used_numbers
			detach_current_version
			current_version.compute_compatible_processes
			current_version.assign_process_to_projects
			current_version.report_process_warnings
		end

feature -- Operations

	refresh_versions
			-- Refresh `versions'
		local
			row: PACKING_VERSIONS_MROW
		do
			create row.make_template
			versions := row.rows_for_packing (packing_id, "date_created desc, version_no desc", connection, expected_count.At_least_zero)
		end

	create_initial_version
			-- Create the initial version for a packing
			-- which has no versions
		do
			create current_version.make_initial_version (packings_row, connection)
		end

	detach_current_version
			-- Replace `current_version' with detached copy
			-- of itself
		do
			current_version := current_version.deep_detached_twin
		end

	save_new_version (a_name: STRING r: RUNS_MROW)
			-- Save contents of `current_version' as a new
			-- packing version under `a_name' (under no
			-- name if Void).  Update `r' if not Void
		local
			e: PACKING_VERSION_CREATED_EVENT
		do
			current_version.update_for_new_version (a_name)
			current_version.register_deep
			if r /= Void then
				r.set_packing_id (current_version.packing_id)
				r.set_packing_version_no (current_version.version_no)
				if r.is_registered then
					-- Change RUNS table last to avoid foreign
					-- key constraint violation to packing version
					r.reregister_last
				end
			end
			connection.commit
			create e
			e.set_version (current_version)
			e.publish
		end

	save_new_dummy_vms_version (r: RUNS_MROW)
			-- Save contents of `current_version' as a new
			-- dummy VMS packing version (projects were selected
			-- on VMS, not Unix).  Update `r' if not Void
		do
			save_new_version (current_version.vms_selection_id, r)
		end

	update_current_version_from_packer_box_file (fname: STRING)
			-- Update current version of packing from
			-- packer box file whose Unix filename is `fname'
		require
			fname_not_void: fname /= Void
			has_current_version: current_version /= Void
		local
			pb_list: PACKER_BOX_LIST
		do
			create pb_list.make_from_named_file (fname)
			
			pb_list := pb_list.minus_external_projects_and_macrocells
			pb_list.setup_for_fast_access
			current_version.remove_dies_for_packer_box_update (pb_list)
			current_version.update_from_packer_boxes (pb_list)
			current_version.compute_compatible_processes
			current_version.assign_process_to_projects
			current_version.report_process_warnings
		end

	setup_for_fast_project_access
			-- Do setup which allows quickly finding out
			-- whether a particular cust die or test
			-- vehicle project has ever been part of a
			-- packing version, so that we can assign it
			-- the same project_no
		local
			list: LIST [like anchored_project]
		do
			list := all_projects
			create cust_project_table.make (list.count)
			create test_vehicle_project_table.make (list.count)
			list.do_all (agent add_project_to_table)
		end

	save_new_scribe_widths (old_horiz_width, old_vert_width, new_horiz_width, new_vert_width, def_width: DOUBLE)
			-- Save new scribe widths in the database,
			-- if they changed, so that they will be
			-- preserved over sessions
		local
			horiz_changed, vert_changed: BOOLEAN
		do
			horiz_changed := old_horiz_width /= new_horiz_width
			vert_changed := old_vert_width /= new_vert_width
			if horiz_changed or vert_changed then
				if horiz_changed then
					if new_horiz_width /= def_width then
						packings_row.set_horizontal_scribe_width (new_horiz_width)
					else
						packings_row.set_null_horizontal_scribe_width
					end
				end
				if vert_changed then
					if new_vert_width /= def_width then
						packings_row.set_vertical_scribe_width (new_vert_width)
					else
						packings_row.set_null_vertical_scribe_width
					end
				end
				connection.commit
			end
		end

feature {NONE} -- Implementation

	add_project_to_table (proj: like anchored_project)
			-- Add `proj' to appropriate table, based on
			-- fab instance type
		do
			if proj.is_customer_project then
				cust_project_table.put (proj, proj.m_fab_no)
				if not cust_project_table.inserted then
					handle_internal_error ("duplicate m_fab_no among distinct cust die projects for all packing versions")
				end
			elseif proj.is_test_vehicle then
				test_vehicle_project_table.put (proj, proj.design_no)
				if not test_vehicle_project_table.inserted then
					handle_internal_error ("duplicate design_no among distinct test vehicles for all packing versions")
				end
			else
				handle_internal_error ("project is not customer project or test vehicle - design_no is " + proj.design_no.out)
			end
		end

	cust_project_table: HASH_TABLE [like anchored_project, INTEGER]
			-- Packer boxes of type cust_die indexed by m_fab_no

	test_vehicle_project_table: HASH_TABLE [like anchored_project, INTEGER]
			-- Packer boxes of type test_vehicle indexed by
			-- design_no

	anchored_project: PACKING_PROJECTS_ROW
			-- For anchoring only
		do
		ensure
			result_void: Result = Void
		end

end

