
indexing
	
	description: "Window to view and set application colors in packer"
	author: "David Hollenberg"
	date: "April 18, 2007"

class RC_PACKING_APP_COLORS_WINDOW

inherit
	RC_PACKING_APP_COLORS_IFC
	WINDOW_INTERFACE
		redefine
			enable_active,
			disable_active
		end
	MOSIS_GRAPHICAL_UTILITIES
		export
			{NONE} all
		undefine
			default_create
		end
	APPLICATION_COLOR_TRANSLATION
		rename
			to_ev_color as app_color_to_ev_color
		export
			{NONE} all
		undefine
			default_create
		end

create
	make

feature {NONE} -- Creation

	make (w: EV_TITLED_WINDOW)
			-- Create interface in window `w'
		require
			window_exists: w /= Void
			window_valid: not w.is_destroyed
		do
			make_interface
			make_proxy (w)
		end

feature -- Operations

	reset
			-- Move focus back into window
		do
			enable_active
		end

feature -- Modification

	enable_active
			-- Make interface active
		do
			precursor
			ok_button.enable_sensitive
			apply_button.enable_sensitive
			reset_button.enable_sensitive
			cancel_button.enable_sensitive
		end

	disable_active
			-- Make interface inactive
			-- (can look but cannot initiate most actions)
		do
			precursor
			ok_button.disable_sensitive
			apply_button.disable_sensitive
			reset_button.disable_sensitive
			cancel_button.disable_sensitive
		end

feature {NONE} -- Implementation

	build_interface
			-- Build interface
		local
			cols: WIDGET_COLUMN_SET
			main_vbox: EV_VERTICAL_BOX
			buttons: HORIZONTAL_BUTTON_SET
			box: EV_BOX
			color_row: APPLICATION_COLORS_ROW
			color_role: STRING
			name_box: EV_COMBO_BOX
			custom_button: EV_BUTTON
			color_area: EV_DRAWING_AREA
			frame: EV_FRAME
			scroll_area: EV_SCROLLABLE_AREA
			l_color: EV_COLOR
			l_color_name: STRING
		do
			create reset_actions
			build_application_colors_table
			build_named_colors_table
			clear_new_application_colors_table
			create main_vbox
			main_vbox.set_padding (user_interface.padding)
			main_vbox.set_border_width (user_interface.border_width)
			create cols.make (4)
			cols.set_padding (user_interface.padding)
			from
				application_colors.start
			until
				application_colors.after
			loop
				color_row := application_colors.item
				color_role := color_row.color_role
				create color_area
				color_area.set_minimum_width (50)
				name_box := new_color_name_box (color_role, color_area)
				l_color := to_ev_color (color_row)
				l_color_name := name_for_color (color_row)
				color_area.set_background_color (l_color)
				name_box.set_text (l_color_name)
				reset_actions.extend (agent set_name_and_color (name_box, color_area, l_color_name, l_color))
				color_area.expose_actions.extend (agent color_area.clear_rectangle)
				name_box.disable_edit
				create frame
				frame.extend (color_area)
				create custom_button.make_with_text_and_action ("Custom", agent show_custom_color_dialog (color_role, name_box, color_area))
				custom_button.set_minimum_width (user_interface.minimum_button_width)
				cols.append_row (<< gui.label_with_text (color_role), frame, name_box, custom_button >>)
				
				application_colors.forth
			end

			cols.disable_expand
			cols.enable_column_expand (3)
			cols.align_column_right (1)
			box := cols.box
			
			create scroll_area
			scroll_area.extend (box)
			
			main_vbox.extend (scroll_area)
			
			create buttons.make_with_text_and_actions (<<
				"OK", "Apply", "Reset", "Cancel" >>,
				<< agent process_ok, agent process_apply, agent process_reset, agent process_cancel >>)
			ok_button := buttons.i_th (1)
			apply_button := buttons.i_th (2)
			reset_button := buttons.i_th (3)
			cancel_button := buttons.i_th (4)
			
			box := buttons.column_set.box
			
			main_vbox.extend (box)
			main_vbox.disable_item_expand (box)
			
			gui.set_box_background_recursive (main_vbox, background_color)
			extend (main_vbox)
			close_request_actions.extend (agent process_cancel)
			gui.add_cancel_accelerator (window, agent process_cancel)
			gui.do_once_on_idle (agent post_display_actions)
			set_minimum_width (800)
		end
		
	post_display_actions
		do
			reset
		end

	build_application_colors_table
			-- Build table that maps color role to the
			-- database row that assigns the role a color
		do
			create application_colors_table.make (application_colors.count)
			application_colors.do_all (agent add_application_color)
		end

	add_application_color (row: APPLICATION_COLORS_MROW)
			-- Add `row' to `application_colors_table'
		do
			application_colors_table.put (row, row.color_role)
		end

	build_named_colors_table
			-- Build table that maps color name to a named color
		do
			create named_colors_table.make (named_colors.count)
			named_colors.do_all (agent add_named_color)
		end

	add_named_color (row: NAMED_COLORS_ROW)
			-- Add `row' to `named_colors_table'
		do
			named_colors_table.put (row, row.color_name)
		end

	to_ev_color (row: APPLICATION_COLORS_ROW): EV_COLOR
			-- `row' as an EV_COLOR
		do
			if row.is_null_color_name then
				create Result.make_with_rgb (row.red_value, row.green_value, row.blue_value)
			else
				named_colors_table.search (row.color_name)
				if named_colors_table.found then
					Result := gui.to_ev_color (named_colors_table.found_item)
				else
					handle_hard_error ("Color name " + row.color_name + " not found in named_colors_table")
				end
			end
		end

	name_for_color (row: APPLICATION_COLORS_ROW): STRING
			-- Color name for `row'
		do
			if not row.is_null_color_name then
				Result := row.color_name
			else
				Result := Custom_name + "  " + gui.color_name_8_bit (app_color_to_ev_color (row))
			end
		end

	new_color_name_box (a_color_role: STRING a_color_area: EV_DRAWING_AREA): EV_COMBO_BOX
			-- New combo box with all known named colors in
			-- its pulldown list for `a_color_role'
		require
			a_color_role_not_void: a_color_role /= Void
			a_color_area_not_void: a_color_area /= Void
		local
			list_item: EV_LIST_ITEM
			l_color: NAMED_COLORS_ROW
		do
			is_select_blocked := True
			create Result.make_with_text ("")
			from
				named_colors.start
			until
				named_colors.after
			loop
				l_color := named_colors.item
				create list_item.make_with_text (l_color.color_name)
				list_item.select_actions.extend (agent assign_named_color (a_color_role, l_color, a_color_area))
				Result.extend (list_item)
				named_colors.forth
			end
			Result.set_minimum_width (250)
			is_select_blocked := False
		end

	set_name_and_color (a_name_box: EV_COMBO_BOX a_color_area: EV_DRAWING_AREA a_color_name: STRING a_color: EV_COLOR)
			-- Set `a_color_area' and `a_name_box' text from
			-- information in `a_color_row' and to `a_color'
		do
			a_color_area.set_background_color (a_color)
			a_color_area.clear
			a_name_box.enable_edit
			a_name_box.set_text (a_color_name)
			a_name_box.disable_edit
		end

	show_custom_color_dialog (a_color_role: STRING a_name_box: EV_COMBO_BOX a_color_area: EV_DRAWING_AREA)
			-- Display dialog to allow user to select
			-- custom color
		require
			a_color_role_not_void: a_color_role /= Void
			a_name_box_not_void: a_name_box /= Void
			a_color_area_not_void: a_color_area /= Void
		local
			l_dialog: EV_COLOR_DIALOG
		do
			create l_dialog
			l_dialog.set_title ("Set Color for " + a_color_role)
			l_dialog.set_color (a_color_area.background_color)
			l_dialog.ok_actions.extend (agent assign_custom_color (a_color_role, l_dialog, a_name_box, a_color_area))
			l_dialog.show_modal_to_window (window)
		end

	assign_named_color (a_color_role: STRING a_color_row: NAMED_COLORS_ROW a_color_area: EV_DRAWING_AREA)
			-- Assign color in `a_color_row', which is displayed
			-- in `a_color_area' to `a_color_role'
		require
			a_color_role_not_void: a_color_role /= Void
			a_color_row_not_void: a_color_row /= Void
			a_color_area_not_void: a_color_area /= Void
		local
			row: APPLICATION_COLORS_MROW
		do
			if not is_select_blocked then
				create row.make_detached (session.connection)
				row.set_color_role (a_color_role)
				row.set_color_name (a_color_row.color_name)
				new_application_colors_table.force (row, a_color_role)
				a_color_area.set_background_color (gui.to_ev_color (a_color_row))
				a_color_area.clear
			end
		end

	assign_custom_color (a_color_role: STRING a_dialog: EV_COLOR_DIALOG a_name_box: EV_COMBO_BOX a_color_area: EV_DRAWING_AREA)
			-- Assign color selected in `a_dialog' to
			-- `a_color_role' and display it in `a_color_area'
		require
			a_color_role_not_void: a_color_role /= Void
			a_dialog_not_void: a_dialog /= Void
			a_name_box_not_void: a_name_box /= Void
			a_color_area_not_void: a_color_area /= Void
		local
			row: APPLICATION_COLORS_MROW
			l_color: EV_COLOR
		do
			l_color := a_dialog.color
			create row.make_detached (session.connection)
			row.set_color_role (a_color_role)
			row.set_red_value (l_color.red)
			row.set_green_value (l_color.green)
			row.set_blue_value (l_color.blue)
			new_application_colors_table.force (row, a_color_role)
			
			a_name_box.enable_edit
			a_name_box.set_text (Custom_name + "  " + gui.color_name_8_bit (l_color))
			a_name_box.disable_edit
			a_color_area.set_background_color (l_color)
			a_color_area.clear
		end

	process_ok
		do
			if is_active then
				disable_active
				set_new_application_colors
				accept_actions.call ([new_application_colors])
				clear_new_application_colors_table
			end
		end

	process_apply
		do
			if is_active then
				disable_active
				set_new_application_colors
				if colors_changed then
					apply_actions.call ([new_application_colors])
					clear_new_application_colors_table
				end
				enable_active
			end
		end

	process_reset
		do
			if is_active then
				disable_active
				apply_actions.call ([application_colors])
				reset_actions.call ([])
				clear_new_application_colors_table
				enable_active
			end
		end

	clear_new_application_colors_table
		do
			create new_application_colors_table.make (application_colors.count)
		end

	process_cancel
		do
			if is_active then
				disable_active
				cancel_actions.call ([])
			end
		end

	set_new_application_colors
			-- Compute new value for `application_colors'
			-- and leave it in `new_application_colors'.
		do
			new_application_colors := new_application_colors_table.linear_representation
			colors_changed := not new_application_colors.is_empty
		end

	ok_button: EV_BUTTON
			-- Button to accept current colors and hide window
	
	apply_button: EV_BUTTON
			-- Button to apply current colors
	
	reset_button: EV_BUTTON
			-- Button to reset colors to default colors
	
	cancel_button: EV_BUTTON
			-- Button to cancel setting of colors
	
	application_colors_table: HASH_TABLE [APPLICATION_COLORS_MROW, STRING]
	
			-- Table of existing application color
			-- rows indexed by color role
	
	new_application_colors: LIST [APPLICATION_COLORS_MROW]
			-- New colors for packing application
	
	new_application_colors_table: HASH_TABLE [APPLICATION_COLORS_MROW, STRING]
			-- Table of new application color assignment rows
			-- indexed by color role
	
	named_colors_table: HASH_TABLE [NAMED_COLORS_ROW, STRING]
			-- Table of named color rows indexed
			-- by color name
	
	is_select_blocked: BOOLEAN
			-- Is selection of a named color blocked?
	
	colors_changed: BOOLEAN
			-- Were any application colors changed?
	
	Custom_name: STRING = "Custom"
			-- Name for color that is not a known named color
	
	reset_actions: CLOSED_ACTION_SEQUENCE
			-- Action to be invoked when user clicks on Reset

end
