
indexing
	
	description: "Window to allow creation of new packings"
	author: "David Hollenberg"
	date: "November 7, 2005"

class RC_PACKING_CREATION_WINDOW

inherit
	RC_PACKING_RECORD_WINDOW
		redefine
			build_interface
		end

create
	make

feature -- Modification

feature {NONE} -- Implementation

	build_interface
			-- Build interface
		do
			precursor
			enable_change_processing
			process_run_type_change
		end
		
	enable_buttons_sensitive
			-- Make buttons sensitive
		do
			create_button.enable_sensitive
			cancel_button.enable_sensitive
		end

	disable_buttons_sensitive
			-- Make buttons insensitive
		do
			create_button.disable_sensitive
			cancel_button.disable_sensitive
		end

	populate_fields
			-- Populate some fields
		do
			populate_creator (creator_field)
			populate_run_type (run_type_field)
			populate_overhead_style (overhead_style_field)
		end
		
	set_field_properties
			-- Set properties of fields
		do
			creator_field.disable_edit
			run_type_field.disable_edit
			run_type_field.change_actions.extend (agent process_run_type_change)
			educational_field.select_actions.extend (agent process_educational_change)
			wafer_techn_field.change_actions.extend (agent process_wafer_techn_change)
			field_set_field.change_actions.extend (agent process_field_set_change)
			overhead_style_field.change_actions.extend (agent process_overhead_style_change)
			other_border_field.change_actions.extend (agent process_other_border_change)
			top_overhead_field.disable_sensitive
		end
		
	add_buttons
			-- Add buttons to end of `main_vox'
		local
			box: EV_BOX
			buttons: HORIZONTAL_BUTTON_SET
		do
			create buttons.make_with_text_and_actions (<<
				"Create", "Cancel" >>,
				<< agent process_create, agent process_cancel >>)
			create_button := buttons.i_th (1)
			cancel_button := buttons.i_th (2)
			
			box := buttons.column_set.box
			
			main_vbox.extend (box)
			main_vbox.disable_item_expand (box)
			
		end
		
	populate_wafer_techn
		local
			row: WAFER_TECHNS_ROW
			wafer_techns: LIST [WAFER_TECHNS_ROW]
			design_type: DESIGN_TYPE_VALUES
			old_wt: STRING
		do
			old_wt ?= selected_data (wafer_techn_field)
			create row.make_template
			wafer_techns := row.possible_wafer_techns_ordered (design_type.Ic, selected_text (run_type_field), connection, expected_count.At_least_zero)
			previous_wafer_techn ?= selected_data (wafer_techn_field)
			wafer_techn_field.wipe_out
			wafer_techn_field.enable_edit
			wafer_techn_field.set_text ("")
			wafer_techn_field.disable_edit
			wafer_techns.do_all (agent add_wafer_techn)
			wafer_techn ?= selected_data (wafer_techn_field)
			wafer_techn_changed := not equal (wafer_techn, old_wt)
			if wafer_techn_changed then
				populate_field_set
				populate_other_border
			end
		end

	add_wafer_techn (row: WAFER_TECHNS_ROW)
		local
			list_item: EV_LIST_ITEM
		do
			create list_item.make_with_text (row.wafer_techn + "   " + row.descr)
			list_item.set_data (row.wafer_techn)
			wafer_techn_field.extend (list_item)
			if previous_wafer_techn /= Void and then row.wafer_techn.is_equal (previous_wafer_techn) then
				list_item.enable_select
			end
		end

	populate_field_set
		local
			row: FIELD_SETS_ROW
			field_sets: LIST [FIELD_SETS_ROW]
			wt, old_fs: STRING
		do
			old_fs ?= selected_data (field_set_field)
			wt := su.empty_value (wafer_techn, "")
			create row.make_template
			field_sets := row.available_field_sets_ordered ("NO_RUN_ID", wt, connection, expected_count.at_least_zero)
			previous_field_set ?= selected_data (field_set_field)
			field_set_field.wipe_out
			field_set_field.enable_edit
			field_set_field.set_text ("")
			field_set_field.disable_edit
			field_sets.do_all (agent add_field_set)
			field_set ?= selected_data (field_set_field)
			field_set_changed := not equal (field_set, old_fs)
			if field_set_changed then
				populate_step_pattern
			end
		end

	add_field_set (row: FIELD_SETS_ROW)
		local
			list_item: EV_LIST_ITEM
		do
			create list_item.make_with_text (row.field_set_name + "   " + row.descr)
			list_item.set_data (row.field_set_name)
			field_set_field.extend (list_item)
			if previous_field_set /= Void and then row.field_set_name.is_equal (previous_field_set) then
				list_item.enable_select
			end
		end

	populate_step_pattern
		local
			row: FIELD_STEP_PATTERNS_ROW
			step_patterns: LIST [FIELD_STEP_PATTERNS_ROW]
			fs, old_sp: STRING
		do
			old_sp ?= selected_data (step_pattern_field)
			create row.make_template
			fs := su.empty_value (field_set, "")
			step_patterns := row.available_step_patterns_ordered (fs, connection, expected_count.at_least_zero)
			previous_step_pattern := old_sp
			step_pattern_field.wipe_out
			step_pattern_field.enable_edit
			step_pattern_field.set_text ("")
			step_pattern_field.disable_edit
			step_patterns.do_all (agent add_step_pattern)
		end

	add_step_pattern (row: FIELD_STEP_PATTERNS_ROW)
		local
			list_item: EV_LIST_ITEM
		do
			create list_item.make_with_text (row.field_step_pattern + "   " + row.descr)
			list_item.set_data (row.field_step_pattern)
			step_pattern_field.extend (list_item)
			if previous_step_pattern /= Void and then row.field_step_pattern.is_equal (previous_step_pattern) then
				list_item.enable_select
			end
		end

	populate_sched_run
		local
			row: SCHEDULED_RUNS_EROW
			sched_runs: LIST [SCHEDULED_RUNS_EROW]
		do
			create row.make_template
			sched_runs := row.possible_scheduled_runs_ordered (su.empty_value (wafer_techn, ""), is_educational, 180, 30, connection, expected_count.at_least_zero)
			previous_sched_run ?= selected_data (sched_run_field)
			sched_run_field.wipe_out
			sched_run_field.enable_edit
			sched_run_field.set_text ("")
			sched_run_field.disable_edit
			sched_runs.do_all (agent add_sched_run)
			add_no_sched_run
		end

	add_sched_run (row: SCHEDULED_RUNS_EROW)
		local
			list_item: EV_LIST_ITEM
		do
			create list_item.make_with_text (row.date_scheduled.to_date_string)
			list_item.set_data (row.sched_run_no)
			sched_run_field.extend (list_item)
			if previous_sched_run /= Void and then row.sched_run_no= previous_sched_run then
				list_item.enable_select
			end
		end

	add_no_sched_run
			-- Add scheduled run "None", which means do
			-- not associate a scheduled run with this
			-- run
		local
			list_item: EV_LIST_ITEM
		do
			create list_item.make_with_text ("None")
			list_item.set_data (No_sched_run_no)
			sched_run_field.extend (list_item)
			if previous_sched_run /= Void and then previous_sched_run = No_sched_run_no then
				list_item.enable_select
			end
		end

	populate_other_border
		local
			wt_row: WAFER_TECHNS_ROW
			style: OVERHEAD_STYLE_VALUES
			l_other_border: DOUBLE
		do
			if current_overhead_style.is_equal (style.External_) then
				create wt_row.make_template
				l_other_border := wt_row.wafer_techns_with_name (wafer_techn, connection, expected_count.Exactly_one).first.other_border
				other_border_field.disable_sensitive
				other_border_field.set_text (nu.formatted_number_or_integer (l_other_border, border_scale))
			else
				other_border_field.enable_sensitive
			end
			populate_top_overhead
		end

	populate_top_overhead
		do
			top_overhead_field.set_text (top_overhead_description (current_overhead_style, other_border_field.text))
		end

	process_run_type_change
		local
			rt: STRING
			run_type_val: RUN_TYPE_VALUES
			is_sched, is_ded: BOOLEAN
		do
			if change_processing_enabled then
				disable_change_processing
				rt := run_type_field.text
				if run_type = Void or else not run_type.is_equal (rt) then
					-- Run type changed
					run_type := rt
					is_sched := rt.is_equal (run_type_val.Skd)
					is_ded := rt.is_equal (run_type_val.Ded)
					if is_sched then
						educational_field.enable_sensitive
						sched_run_field.enable_sensitive
					else
						educational_field.disable_sensitive
						educational_field.disable_select
						sched_run_field.wipe_out
						sched_run_field.enable_edit
						sched_run_field.set_text ("")
						sched_run_field.disable_edit
						sched_run_field.disable_sensitive
					end
					is_educational := False
					populate_wafer_techn
					if wafer_techn_changed or is_sched then
						populate_sched_run
					end
					if wafer_techn_changed then
						populate_other_border
					end
					enable_change_processing
				end
			end
		end

	process_educational_change
		do
			if change_processing_enabled then
				disable_change_processing
				if is_educational /= educational_field.is_selected then
					is_educational := educational_field.is_selected
					populate_sched_run
				end
				enable_change_processing
			end
		end

	process_wafer_techn_change
		local
			wt: STRING
		do
			if change_processing_enabled then
				disable_change_processing
				wt ?= selected_data (wafer_techn_field)
				if wafer_techn = Void or else not wafer_techn.is_equal (wt) then
					wafer_techn := wt
					populate_sched_run
					populate_field_set
					populate_other_border
				end
				enable_change_processing
			end
		end

	process_field_set_change
		local
			fs: STRING
		do
			if change_processing_enabled then
				disable_change_processing
				fs ?= selected_data (field_set_field)
				if field_set = Void or else not field_set.is_equal (fs) then
					field_set := fs
					populate_step_pattern
				end
				enable_change_processing
			end
		end

	process_overhead_style_change
		local
			os: STRING
		do
			if change_processing_enabled then
				disable_change_processing
				os ?= selected_data (overhead_style_field)
				if overhead_style = Void or else not overhead_style.is_equal (os) then
					overhead_style := os
					populate_other_border
				end
				enable_change_processing
			end
		end

	process_other_border_change
		do
			if change_processing_enabled then
				disable_change_processing
				populate_top_overhead
				enable_change_processing
			end
		end

	process_create
		local
			role_setter: ROLE_SET_OP
		do
			if is_active then
				disable_active
				create role_setter
				role_setter.enable_abort_caught
				role_setter.set_role (connection, role, session.environment.db_access_control)
				if not role_setter.aborted then
					packing_id := packing_id_field.text
					packing_id.left_adjust
					packing_id.right_adjust
					validate_packing
					if is_valid then
						insert_packing
						session.commit
						inform ("Packing " + su.double_quoted (packing_id) + " created")
					end
				end
				enable_active
			end
		end

	validate_packing
		local
			type: RUN_TYPE_VALUES
			border: STRING
			p: PACKINGS_ROW
			val: DOUBLE
		do
			is_valid := True
			if packing_id.is_empty then
				missing_field ("Packing ID")
				is_valid := False
			elseif packing_id.has (operating_environment.Directory_separator) then
				invalid_string_field ("Packing ID", operating_environment.Directory_separator)
			end
			if is_valid and su.is_empty (wafer_techn) then
				missing_field ("Wafer Techn")
			end
			if is_valid and su.is_empty (field_set) then
				missing_field ("Field Set")
			end
			if is_valid and run_type.is_equal (type.Skd) and sched_run_field.text.is_empty then
				missing_field ("Scheduled Run")
			end
			if is_valid then
				border := other_border_field.text
				if border.is_empty then
					missing_field ("Other Side Overhead")
				elseif not border.is_double then
					invalid_real_field ("Other Side Overhead")
				else
					val := border.to_double
					if val = 0 then
						-- OK
					elseif val < 0 then
						negative_field ("Other Side Overhead")
					elseif not nu.is_multiple_of_resolution (val, half_grid_resolution, border_scale) then
						inform ("Other Side Overhead value (" + border + ") is not a multiple of half-grid resolution " + nu.formatted_number_or_integer (half_grid_resolution, border_scale))
						is_valid := False
					end
				end
			end
			if is_valid then
				create p.make_by_key (connection, option.Old_no_exception_row, packing_id)
				if not p.is_empty then
					inform ("Packing " + su.double_quoted (packing_id) + " already exists")
					is_valid := False
				end
			end
		end

	wafer_techns_row: WAFER_TECHNS_ROW
			-- Row for wafer techn
		do
			check
				wafer_techn_not_void: wafer_techn /= Void
				wafer_techn_not_empty: not wafer_techn.is_empty
			end
			create Result.make_template
			Result := Result.wafer_techns_with_name (wafer_techn, connection, expected_count.Exactly_one).first
		end

	missing_field (fld: STRING)
		do
			inform (fld + " is required")
			is_valid := False
		end

	invalid_string_field (fld: STRING c: CHARACTER)
		do
			inform (fld + " has invalid character " + su.double_quoted (c.out))
			is_valid := False
		end

	invalid_real_field (fld: STRING)
		do
			inform (fld + " is not a valid real number")
			is_valid := False
		end

	negative_field (fld: STRING)
		do
			inform (fld + " cannot be negative")
			is_valid := False
		end

	selected_text (b: EV_COMBO_BOX): STRING
			-- Text for selected item of `b' or empty string if none
		local
			i: EV_LIST_ITEM
		do
			i := b.selected_item
			if i /= Void then
				Result := i.text
			else
				Result := ""
			end
		end

	selected_data (b: EV_COMBO_BOX): ANY
			-- Data for selected item of `b' or Void if none
		local
			i: EV_LIST_ITEM
		do
			i := b.selected_item
			if i /= Void then
				Result := i.data
			end
		end

	current_overhead_style: STRING
			-- Currently selected overhead style
		local
			style: OVERHEAD_STYLE_VALUES
		do
			Result ?= selected_data (overhead_style_field)
			Result := style.project_specific_translated (Result)
		end

	current_top_overhead: STRING
			-- Description of overhead on top side of design,
			-- based on currently selected values
		do
			Result := top_overhead_description (current_overhead_style, other_border_field.text)
		end
	
	insert_packing
		local
			p: PACKINGS_MROW
			wt_row: WAFER_TECHNS_ROW
			sr_no: INTEGER_REF
			wt, wtf, fs, fsp: STRING
		do
			create p.make_null (connection)
			p.set_packing_id (packing_id)
			p.set_creator (creator_field.text)
			p.set_date_created (connection.transaction_date)
			p.set_run_type (run_type_field.text)
			p.set_is_taxi (taxi_field.is_selected)
			p.set_educational (educational_field.is_selected)
			wt ?= selected_data (wafer_techn_field)
			p.set_wafer_techn (wt)
			create wt_row.make_template
			wtf := wt_row.wafer_techns_with_name (wt, connection, expected_count.Exactly_one).first.wafer_techn_family
			p.set_wafer_techn_family (wtf)
			sr_no ?= selected_data (sched_run_field)
			if sr_no /= Void then
				if sr_no.item /= No_sched_run_no then
					p.set_sched_run_no (sr_no.item)
				else
					p.set_null_sched_run_no
				end
			else
				p.set_null_sched_run_no
			end
			fs ?= selected_data (field_set_field)
			p.set_field_set_name (fs)
			fsp ?= selected_data (step_pattern_field)
			p.set_field_step_pattern (fsp)
			p.set_overhead_style (current_overhead_style)
			p.set_other_border (other_border_field.text.to_double)
			p.set_intended_run (intended_run_field.text)
			p.set_descr (description_field.text)
		end

	enable_change_processing
		do
			change_processing_enabled := True
		end

	disable_change_processing
		do
			change_processing_enabled := False
		end

	create_button: EV_BUTTON
			-- Button to create new packing
	
	cancel_button: EV_BUTTON
			-- Button to end session
	
	run_type: STRING
			-- Last selected run type

	is_educational: BOOLEAN
			-- Last selected "educational" indicator for run

	wafer_techn: STRING
			-- Last selected wafer techn

	field_set: STRING
			-- Last selected field set

	overhead_style: STRING
			-- Last selected overhead style

	change_processing_enabled: BOOLEAN
			-- Should changes to fields trigger processing
			-- of the corresponding change actions?

	wafer_techn_changed: BOOLEAN
			-- Was value of wafer techn changed by last
			-- call to `populate_wafer_techn'?

	field_set_changed: BOOLEAN
			-- Was value of field seet changed by last
			-- call to `populate_field_set'?

	previous_wafer_techn: STRING
			-- Previously selected wafer techn, if any

	previous_field_set: STRING
			-- Previously selected field set, if any

	previous_step_pattern: STRING
			-- Previously selected field step pattern, if any

	previous_sched_run: INTEGER_REF
			-- Previously selected scheduled run number, if any

	is_valid: BOOLEAN
			-- Are values for new packing valid?

	no_sched_run_no: INTEGER = -1
			-- Scheduled run number indicating "no scheduled run"

end
