
indexing
	
	description: "Window to allow user to specify disposition of %
		%projects whose last copy is deleted during packing"
	author: "David Hollenberg"
	date: "November 29, 2005"

class RC_PACKING_DELETE_WINDOW

inherit
	RC_PACKING_DELETE_IFC
	WINDOW_INTERFACE
		redefine
			enable_active,
			disable_active
		end
	MOSIS_GRAPHICAL_UTILITIES
		export
			{NONE} all
		undefine
			default_create
		end

create
	make

feature {NONE} -- Creation

	make (w: EV_TITLED_WINDOW)
			-- Create interface in window `w'
		require
			window_exists: w /= Void
			window_valid: not w.is_destroyed
		do
			make_interface
			make_proxy (w)
		end

feature -- Operations

	reset
			-- Move focus back into window
		do
			enable_active
		end

feature -- Modification

	enable_active
			-- Make interface active
		do
			precursor
			ok_button.enable_sensitive
			cancel_button.enable_sensitive
		end

	disable_active
			-- Make interface inactive
			-- (can look but cannot initiate most actions)
		do
			precursor
			ok_button.disable_sensitive
			cancel_button.disable_sensitive
		end

feature {NONE} -- Implementation

	build_interface
			-- Build interface
		local
			main_vbox: EV_VERTICAL_BOX
			label: EV_LABEL
			buttons: HORIZONTAL_BUTTON_SET
			box: EV_BOX
			n: INTEGER
			text: EV_TEXT
		do
			create main_vbox
			main_vbox.set_padding (user_interface.padding)
			main_vbox.set_border_width (user_interface.border_width)
			n := deleted_projects.count
			create label.make_with_text ("The last copy is about to be deleted for " + n.out + " " + su.number (n, "project", "projects") + ".%NIf you proceed the projects will be removed from the packing and the change history list will be cleared.")
			label.align_text_left
			main_vbox.extend (label)
			main_vbox.disable_item_expand (label)
			
			create text
			text.set_text (deleted_projects_description)
			text.disable_edit
			main_vbox.extend (text)

			create unlock_button.make_with_text ("Unlock projects")
			create deassign_button.make_with_text ("Deassign projects from run")
			unlock_button.enable_select
			deassign_button.enable_select
			unlock_button.select_actions.extend (agent process_unlock_change)
			deassign_button.select_actions.extend (agent process_deassign_change)
			main_vbox.extend (unlock_button)
			main_vbox.disable_item_expand (unlock_button)
			main_vbox.extend (deassign_button)
			main_vbox.disable_item_expand (deassign_button)
			
			create buttons.make_with_text_and_actions (<<
				"OK", "Cancel" >>,
				<< agent process_ok, agent process_cancel >>)
			ok_button := buttons.i_th (1)
			cancel_button := buttons.i_th (2)
			
			box := buttons.column_set.box
			
			main_vbox.extend (box)
			main_vbox.disable_item_expand (box)
			
			gui.set_box_background_recursive (main_vbox, background_color)
			extend (main_vbox)
			close_request_actions.extend (agent process_cancel)
			gui.add_cancel_accelerator (window, agent process_cancel)
			gui.do_once_on_idle (agent post_display_actions)
			set_minimum_height (350)
		end
		
	post_display_actions
		do
			reset
		end

	process_ok
		do
			if is_active then
				disable_active
				accept_actions.call ([unlock_button.is_selected, deassign_button.is_selected])
			end
		end

	process_cancel
		do
			if is_active then
				disable_active
				cancel_actions.call ([])
			end
		end

	deleted_projects_description: STRING
			-- Description of `deleted_projects'
		local
			p: PACKING_PROJECTS_ROW
		do
			create Result.make (deleted_projects.count * 50)
			from
				deleted_projects.start
			until
				deleted_projects.after
			loop
				p := deleted_projects.item
				Result.append_integer (p.design_no)
				Result.append ("    ")
				Result.append (p.design_name)
				Result.extend ('%N')
				deleted_projects.forth
			end
		end

	process_unlock_change
			-- Process change to `unlock_button'
		do
			if unlock_button.is_selected and not deassign_button.is_selected then
				deassign_button.enable_select
			end
		end

	process_deassign_change
			-- Process change to `deassign_button'
		do
			if not deassign_button.is_selected and unlock_button.is_selected then
				unlock_button.disable_select
			end
		end

	unlock_button: EV_CHECK_BUTTON
			-- Button to indicate whether to unlock projects

	deassign_button: EV_CHECK_BUTTON
			-- Button to indicate whether to deassign projects
			-- from run

	ok_button: EV_BUTTON
			-- Button to hide window
	
	cancel_button: EV_BUTTON
			-- Button to hide window
	
end
