
indexing
	
	description: "Interface to allow setting of new filler properties"
	author: "David Hollenberg"
	date: "November 7, 2006"

deferred class RC_PACKING_FILLER_IFC
inherit
	ABSTRACT_INTERFACE
	GRIDABLE

feature {NONE} -- Creation

	make_interface
			-- Create interface
		do
			create accept_actions
			create cancel_actions
		end

feature -- Modification

	set_session (sess: like session)
			-- Set `session' to `sess'
		require
			sess_exists: sess /= Void
		do
			session := sess
		ensure
			session_set: session = sess
		end

feature -- Properties

	default_filler_name: STRING
			-- Default name for new filler

	default_overhead_style: STRING
			-- Default overhead style for new filler

	default_filler_width: DOUBLE
			-- Default width for new filler

	default_filler_height: DOUBLE
			-- Default height for new filler

	is_size_clear_requested: BOOLEAN
			-- Should die and project heights be cleared
			-- initially?

	used_filler_names: LIST [STRING]
			-- List of uppercased filler names that have been
			-- used previously

	maximum_filler_size: DOUBLE
			-- Maximum width and height filler

	die_to_project_size_agent: FUNCTION [ANY, TUPLE [STRING, CONST_POINT], CONST_POINT]
			-- Agent that translates overhead style and
			-- die size to size of maximum project that
			-- can fit in die

	project_to_die_size_agent: FUNCTION [ANY, TUPLE [STRING, CONST_POINT], CONST_POINT]
			-- Agent that translates overhead style and project 
			-- size to size of die for project

	session: RC_ABSTRACT_PACKING_SESSION
			-- Session used to do packing

	accept_actions: ACTION_SEQUENCE [TUPLE [STRING, DOUBLE, DOUBLE]]
			-- Action to be invoked when user accepts
			-- current filler properties

	cancel_actions: CLOSED_ACTION_SEQUENCE
			-- Action to be invoked when user clicks on Cancel

feature -- Modification

	set_default_filler_name (a_name: like default_filler_name)
			-- Set `default_filler_name' to `a_name'
		require
			a_name_not_void: a_name /= Void
		do
			default_filler_name := a_name
		ensure
			default_filler_name_set: default_filler_name = a_name
		end

	set_default_overhead_style (a_style: like default_overhead_style)
			-- Set `default_overhead_style' to `a_style'
		require
			a_style_not_void: a_style /= Void
		do
			default_overhead_style := a_style
		ensure
			default_overhead_style_set: default_overhead_style = a_style
		end

	set_default_filler_width (a_width: like default_filler_width)
			-- Set `default_filler_width' to `a_width'
		require
			a_width_positive: a_width > 0.0
			a_width_on_grid: grid.is_size_on_grid (a_width)
		do
			default_filler_width := a_width
		ensure
			default_filler_width_set: default_filler_width = a_width
		end

	set_default_filler_height (a_height: like default_filler_height)
			-- Set `default_filler_height' to `a_height'
		require
			a_height_positive: a_height > 0.0
			a_height_on_grid: grid.is_size_on_grid (a_height)
		do
			default_filler_height := a_height
		ensure
			default_filler_height_set: default_filler_height = a_height
		end

	set_size_clear_requested (a_value: like is_size_clear_requested)
			-- Set `is_size_clear_requested' to `a_value'
		do
			 is_size_clear_requested := a_value
		end

	set_used_filler_names (a_name_list: like used_filler_names)
			-- Set `used_filler_names' to `a_name_list'
		require
			a_name_list_not_void: a_name_list /= Void
		do
			used_filler_names := a_name_list
		ensure
			used_filler_names_set: used_filler_names = a_name_list
		end

	set_maximum_filler_size (a_size: like maximum_filler_size)
			-- Set `maximum_filler_size' to `a_size'
		require
			a_size_positive: a_size > 0.0
			a_size_on_grid: grid.is_size_on_grid (a_size)
		do
			maximum_filler_size := a_size
		ensure
			maximum_filler_size_set: maximum_filler_size = a_size
		end

	set_project_to_die_size_agent (an_agent: like project_to_die_size_agent)
			-- Set `project_to_die_size_agent' to `an_agent'
		require
			an_agent_not_void: an_agent /= Void
		do
			project_to_die_size_agent := an_agent
		ensure
			project_to_die_size_agent_set: project_to_die_size_agent = an_agent
		end

	set_die_to_project_size_agent (an_agent: like die_to_project_size_agent)
			-- Set `die_to_project_size_agent' to `an_agent'
		require
			an_agent_not_void: an_agent /= Void
		do
			die_to_project_size_agent := an_agent
		ensure
			die_to_project_size_agent_set: die_to_project_size_agent = an_agent
		end

end
