
indexing
	
	description: "Window to allow selection of existing packing"
	author: "David Hollenberg"
	date: "November 7, 2005"

class RC_PACKING_FILLER_WINDOW

inherit
	RC_PACKING_FILLER_IFC
		undefine
			default_create
		end
	WINDOW_INTERFACE
		redefine
			enable_active,
			disable_active
		end
	MOSIS_GRAPHICAL_UTILITIES
		export
			{NONE} all
		undefine
			default_create
		end
	NUMBER_UTILITIES_ACCESS
		export
			{NONE} all
		undefine
			default_create
		end
	STRING_UTILITIES_ACCESS
		export
			{NONE} all
		undefine
			default_create
		end

create
	make

feature {NONE} -- Creation

	make (w: EV_TITLED_WINDOW)
			-- Create interface in window `w'
		require
			window_exists: w /= Void
			window_valid: not w.is_destroyed
		do
			make_interface
			make_proxy (w)
		end

feature -- Operations

	reset
			-- Move focus back into window
		do
			enable_active
		end

	refresh
			-- Refresh window with current values
		do
		end

feature -- Modification

	enable_active
			-- Make interface active
		do
			precursor
			ok_button.enable_sensitive
			clear_button.enable_sensitive
			cancel_button.enable_sensitive
		end

	disable_active
			-- Make interface inactive
			-- (can look but cannot initiate most actions)
		do
			precursor
			ok_button.disable_sensitive
			clear_button.disable_sensitive
			cancel_button.disable_sensitive
		end

feature {NONE} -- Implementation

	build_interface
			-- Build interface
		local
			cols: WIDGET_COLUMN_SET
			buttons: HORIZONTAL_BUTTON_SET
			main_vbox: EV_VERTICAL_BOX
			box: EV_BOX
			list_item: EV_LIST_ITEM
			l_text: STRING
			proj_size: CONST_POINT
			label: EV_LABEL
			hbox: EV_HORIZONTAL_BOX
			style: OVERHEAD_STYLE_VALUES
		do
			create main_vbox
			main_vbox.set_padding (user_interface.padding)
			main_vbox.set_border_width (user_interface.border_width)
			create cols.make (2)
			cols.set_padding (user_interface.padding)
			
			create name_field.make_with_text (default_filler_name)
			name_field.return_actions.extend (agent process_ok)
			name_field.disable_edit
			cols.append_row (<< gui.label_with_text ("Filler Name"), name_field >>)
			
			build_overhead_style
			cols.append_row (<< gui.label_with_text ("Die Labeling Style"), overhead_style_field >>)
			
			l_text := grid.formatted_number (default_filler_width)
			create die_width_field.make_with_text (l_text)
			create list_item.make_with_text (l_text)
			list_item.select_actions.extend (agent die_width_field.set_text (l_text))
			die_width_field.extend (list_item)
			die_width_field.return_actions.extend (agent process_ok)
			die_width_field.change_actions.extend (agent process_die_width_change)
			cols.append_row (<< gui.label_with_text ("Die Width"), die_width_field >>)
			
			l_text := grid.formatted_number (default_filler_height)
			create die_height_field.make_with_text (l_text)
			create list_item.make_with_text (l_text)
			list_item.select_actions.extend (agent die_height_field.set_text (l_text))
			die_height_field.extend (list_item)
			die_height_field.return_actions.extend (agent process_ok)
			die_height_field.change_actions.extend (agent process_die_height_change)
			cols.append_row (<< gui.label_with_text ("Die Height"), die_height_field >>)
			
			proj_size := die_to_project_size_agent.item ([style.External_, create {POINT}.make_xy (default_filler_width, default_filler_height)])

			create project_width_field.make_with_text (grid.formatted_number (proj_size.x))
			project_width_field.return_actions.extend (agent process_ok)
			project_width_field.change_actions.extend (agent process_project_width_change)
			cols.append_row (<< gui.label_with_text ("Project Width"), project_width_field >>)
			
			create project_height_field.make_with_text (grid.formatted_number (proj_size.y))
			project_height_field.return_actions.extend (agent process_ok)
			project_height_field.change_actions.extend (agent process_project_height_change)
			cols.append_row (<< gui.label_with_text ("Project Height"), project_height_field >>)
			
			cols.disable_column_expand (1)
			cols.align_column_right (1)
			box := cols.box
			
			main_vbox.extend (box)
			main_vbox.disable_item_expand (box)
			
			create custom_names_button.make_with_text ("Allow custom filler names")
			custom_names_button.select_actions.extend (agent process_custom_name_change)
			main_vbox.extend (custom_names_button)
			main_vbox.disable_item_expand (custom_names_button)
			
			create label.make_with_text ("Size resolution is " + grid.size_resolution_text + " microns")
			create hbox
			hbox.extend (label)
			hbox.disable_item_expand (label)
			main_vbox.extend (hbox)
			main_vbox.disable_item_expand (hbox)
			
			create buttons.make_with_text_and_actions (<<
				"OK", "Clear", "Cancel" >>,
				<< agent process_ok, agent process_clear, agent process_cancel >>)
			ok_button := buttons.i_th (1)
			clear_button := buttons.i_th (2)
			cancel_button := buttons.i_th (3)
			
			box := buttons.column_set.box
			
			main_vbox.extend (box)
			main_vbox.disable_item_expand (box)
			
			gui.set_box_background_recursive (main_vbox, background_color)
			extend (main_vbox)
			close_request_actions.extend (agent process_cancel)
			gui.add_cancel_accelerator (window, agent process_cancel)
			gui.do_once_on_idle (agent post_display_actions)
			set_minimum_width (500)
			if is_size_clear_requested then
				process_clear
			end
		end
		
	build_overhead_style
			-- Build `overhead_style_field'
		local
			fact: SINGLE_CONSTRAINED_VALUE_FACTORY
			style: OVERHEAD_STYLE_VALUES
		do
			create fact
			fact.add_choice_and_text (style.External_, style.External_)
			fact.add_choice_and_text (style.Internal, style.Internal)
			fact.add_choice_and_text (style.None, style.None)
			if default_overhead_style /= Void and then not default_overhead_style.is_empty then
				fact.set_default_choice (default_overhead_style)
			else
				fact.set_default_choice (style.External_)
			end
			fact.generate_combo_box
			overhead_style_field := fact.combo_box
			overhead_style_field.select_actions.extend (agent process_overhead_style_change)
		end

	post_display_actions
		do
			reset
		end

	set_name_field (a_name: STRING)
			-- Set text displayed in `name_field' to `a_name'
		local
			b: BOOLEAN
		do
			b := name_field.is_editable
			if not b then
				name_field.enable_edit
			end
			name_field.set_text (a_name)
			if not b then
				name_field.disable_edit
			end
		end

	process_custom_name_change
			-- Process state change for `custom_names_button'
		do
			if custom_names_button.is_selected then
				name_field.enable_edit
			else
				name_field.disable_edit
			end
			update_name
		end

	process_overhead_style_change
			-- Process change for `overhead_style_field'
		do
			if not change_actions_disabled then
				change_actions_disabled := True
				update_project_width (project_width_field, die_width_field)
				update_project_height (project_height_field, die_height_field)
				change_actions_disabled := False
			end
		end

	process_die_width_change
			-- Process change of text in `die_width_field'
		do
			if not change_actions_disabled then
				change_actions_disabled := True
				update_name
				update_project_width (project_width_field, die_width_field)
				change_actions_disabled := False
			end
		end

	process_die_height_change
			-- Process change of text in `die_height_field'
		do
			if not change_actions_disabled then
				change_actions_disabled := True
				update_name
				update_project_height (project_height_field, die_height_field)
				change_actions_disabled := False
			end
		end

	process_project_width_change
			-- Process change of text in `project_width_field'
		do
			if not change_actions_disabled then
				change_actions_disabled := True
				update_die_width (die_width_field, project_width_field)
				update_name
				change_actions_disabled := False
			end
		end

	process_project_height_change
			-- Process change of text in `project_height_field'
		do
			if not change_actions_disabled then
				change_actions_disabled := True
				update_die_height (die_height_field, project_height_field)
				update_name
				change_actions_disabled := False
			end
		end

	update_name
			-- Update text in `name_field' if not using
			-- custom names
		do
			if not custom_names_button.is_selected then
				set_name_field (formatted_field_value (die_width_field.text) + "_x_" + formatted_field_value (die_height_field.text))
			end
		end

	update_project_width (a_project_field, a_die_field: EV_TEXT_FIELD)
			-- Update text in `a_project_field' based on text in
			-- `a_die_field'
		local
			val, new_val: STRING
			d: DOUBLE
		do
			val := a_die_field.text
			if val.is_double and then val.to_double > 0 then
				d := val.to_double
				new_val := grid.formatted_number (die_to_project_size_agent.item ([current_overhead_style, create {CONST_POINT}.make_xy (d, d)]).x)
			else
				new_val := ""
			end
			a_project_field.set_text (new_val)
		end

	update_project_height (a_project_field, a_die_field: EV_TEXT_FIELD)
			-- Update text in `a_project_field' based on text in
			-- `a_die_field'
		local
			val, new_val: STRING
			d: DOUBLE
		do
			val := a_die_field.text
			if val.is_double and then val.to_double > 0 then
				d := val.to_double
				new_val := grid.formatted_number (die_to_project_size_agent.item ([current_overhead_style, create {CONST_POINT}.make_xy (d, d)]).y)
			else
				new_val := ""
			end
			a_project_field.set_text (new_val)
		end

	update_die_width (a_die_field, a_project_field: EV_TEXT_FIELD)
			-- Update text in `a_die_field' based on text in
			-- `a_project_field'
		local
			val, new_val: STRING
			d: DOUBLE
		do
			val := a_project_field.text
			if val.is_double and then val.to_double > 0 then
				d := val.to_double
				new_val := grid.formatted_number (project_to_die_size_agent.item ([current_overhead_style, create {CONST_POINT}.make_xy (d, d)]).x)
			else
				new_val := ""
			end
			a_die_field.set_text (new_val)
		end

	update_die_height (a_die_field, a_project_field: EV_TEXT_FIELD)
			-- Update text in `a_die_field' based on text in
			-- `a_project_field'
		local
			val, new_val: STRING
			d: DOUBLE
		do
			val := a_project_field.text
			if val.is_double and then val.to_double > 0 then
				d := val.to_double
				new_val := grid.formatted_number (project_to_die_size_agent.item ([current_overhead_style, create {CONST_POINT}.make_xy (d, d)]).y)
			else
				new_val := ""
			end
			a_die_field.set_text (new_val)
		end

	formatted_field_value (a_value: STRING): STRING
			-- `a_value' formatted nicely
		do
			if not a_value.is_double or else a_value.to_double <= 0.0 then
				Result := "???"
			else
				Result := grid.formatted_number (a_value.to_double)
			end
		end

	process_clear
			-- Process Clear button press
		do
			die_width_field.set_text ("")
			die_height_field.set_text ("")
		end

	process_ok
		local
			ok: BOOLEAN
			name_string, width_string, height_string: STRING
			upper_name_string: STRING
			maxlen: INTEGER
			w, h: DOUBLE
		do
			if is_active then
				disable_active
				name_string := name_field.text
				width_string := die_width_field.text
				height_string := die_height_field.text
				name_string.left_adjust
				name_string.right_adjust
				upper_name_string := name_string.as_upper
				create last_message.make (50)
				ok := True
				maxlen := {PACKING_DIES_ROW}.Maxlen_filler_name
				if name_string.is_empty then
					last_message.append ("Filler name is empty%N")
					ok := False
				elseif name_string.count > maxlen then
					last_message.append ("Filler name exceeds maximum length of " + maxlen.out + "%N")
					ok := False
				elseif used_filler_names.has (upper_name_string) then
					last_message.append ("Filler name " + su.double_quoted (name_string) + " already used by another filler%N")
					ok := False
				end
				check_dimension ("Die width", width_string, maximum_filler_size)
				ok := ok and is_valid
				w := last_dimension
				check_dimension ("Die height", height_string, maximum_filler_size)
				ok := ok and is_valid
				h := last_dimension
				if ok then
					accept_actions.call ([name_string, w, h])
				else
					inform (last_message)
				end
				enable_active
			end
		end

	check_dimension (a_dimension, a_value: STRING a_max: DOUBLE)
			-- Check that `a_dimension' with `a_value'
			-- whose maximum value is `a_max' is valid.
			-- Append error to `last_message' if not
		require
			a_dimension_not_void: a_dimension /= Void
			a_value_not_void: a_value /= Void
		local
			val: DOUBLE
		do
			is_valid := False
			if not a_value.is_double then
				last_message.append (a_dimension + " " + su.double_quoted (a_value) + " is not a valid real number%N")
			else
				val := a_value.to_double
				if val < grid.size_resolution then
					last_message.append (a_dimension + " (" + a_value + ") is less than minimum allowed value (" + grid.size_resolution_text + ")%N")
				elseif val > a_max then
					last_message.append (a_dimension + " (" + a_value + ") is greater than maximum allowed value (" + grid.formatted_number (a_max) + ")%N")
				elseif not grid.is_size_on_grid (val) then
					last_message.append (a_dimension + " (" + a_value + ") is not a multiple of size resolution " + grid.size_resolution_text + "%N")
				else
					is_valid := True
					last_dimension := grid.rounded_size (val)
				end
			end
		end

	process_cancel
		do
			if is_active then
				disable_active
				cancel_actions.call ([])
			end
		end

	current_overhead_style: STRING
			-- Currently selected overhead style
		do
			Result ?= overhead_style_field.selected_item.data
			check
				result_not_void: Result /= Void
			end
		end

	name_field: EV_TEXT_FIELD
			-- Field to set name of filler
	
	overhead_style_field: EV_COMBO_BOX
			-- Field to allow selection of overhead style
			-- to use when computing project and die sizes
	
	die_width_field: EV_COMBO_BOX
			-- Field to set width of filler die
	
	die_height_field: EV_COMBO_BOX
			-- Field to set height of filler die
	
	project_width_field: EV_TEXT_FIELD
			-- Field to set width of filler project
	
	project_height_field: EV_TEXT_FIELD
			-- Field to set height of filler project
	
	custom_names_button: EV_CHECK_BUTTON
			-- Button to allow user-specified custom filler names
			-- instead of just standard names
	
	ok_button: EV_BUTTON
			-- Button to hide window
	
	clear_button: EV_BUTTON
			-- Button to clear die/project widths and heights
	
	cancel_button: EV_BUTTON
			-- Button to hide window
	
	is_valid: BOOLEAN
			-- Was last value passed to `check_dimension' valid?
	
	last_message: STRING
			-- Error message
	
	last_dimension: DOUBLE
			-- Last valid dimension found by `check_dimension'
	
	change_actions_disabled: BOOLEAN
			-- Are actions to be executed when text field or combo
			-- box text changes currently disable?
	

end
