
indexing
	
	description: "Window to provide help in packer"
	author: "David Hollenberg"
	date: "March 16, 2007"

class RC_PACKING_HELP_WINDOW

inherit
	RC_PACKING_HELP_IFC
	WINDOW_INTERFACE
		redefine
			enable_active,
			disable_active
		end
	MOSIS_GRAPHICAL_UTILITIES
		export
			{NONE} all
		undefine
			default_create
		end

create
	make

feature {NONE} -- Creation

	make (w: EV_TITLED_WINDOW)
			-- Create interface in window `w'
		require
			window_exists: w /= Void
			window_valid: not w.is_destroyed
		do
			make_interface
			make_proxy (w)
		end

feature -- Operations

	reset
			-- Move focus back into window
		do
			enable_active
		end

feature -- Modification

	enable_active
			-- Make interface active
		do
			precursor
			save_button.enable_sensitive
			cancel_button.enable_sensitive
		end

	disable_active
			-- Make interface inactive
			-- (can look but cannot initiate most actions)
		do
			precursor
			save_button.disable_sensitive
			cancel_button.disable_sensitive
		end

feature {NONE} -- Implementation

	build_interface
			-- Build interface
		local
			main_vbox: EV_VERTICAL_BOX
			buttons: HORIZONTAL_BUTTON_SET
			box: EV_HORIZONTAL_BOX
			cols: WIDGET_COLUMN_SET
		do
			create main_vbox
			main_vbox.set_padding (user_interface.padding)
			main_vbox.set_border_width (user_interface.border_width)
			create cols.make (6)
			cols.set_padding (user_interface.padding)
			cols.set_border_width (user_interface.border_width)
			
			create filter_field.make_with_text ("")
			filter_field.change_actions.extend (agent process_change)
			set_binding_types
			build_binding_type_field
			set_categories
			build_category_field
			
			cols.append_row (<< gui.label_with_text ("Binding Type"), binding_type_field, gui.label_with_text ("Category"), category_field, gui.label_with_text ("Filter"), filter_field >>)
			cols.disable_expand
			cols.enable_column_expand (6)
			box := cols.box
			main_vbox.extend (box)
			main_vbox.disable_item_expand (box)
			
			build_packer_bindings

			create box
			box.set_border_width (user_interface.border_width)
			box.extend (packer_bindings)
			main_vbox.extend (box)
			
			create cols.make (1)
			cols.set_padding (user_interface.padding)
			cols.set_border_width (5)
			create include_unbound_ops_button.make_with_text ("Include unbound operations")
			include_unbound_ops_button.select_actions.extend (agent process_include_unbound_ops_change)
			cols.append_row (<< include_unbound_ops_button >>)
			cols.disable_expand
			cols.align_column_right (1)
			box := cols.box
			main_vbox.extend (box)
			main_vbox.disable_item_expand (box)
			
			create buttons.make_with_text_and_actions (<< "List", "Save", "Cancel" >>,
				<< agent process_list, agent process_save, agent process_cancel >>)
			list_button := buttons.i_th (1)
			save_button := buttons.i_th (2)
			cancel_button := buttons.i_th (3)
			
			list_button.set_tooltip ("Write text representation of list of all bindings to Output window")
			save_button.set_tooltip ("Write text representation of currently selected rows (in current order) and currently visible columns to Output window")
			
			box := buttons.column_set.box
			
			main_vbox.extend (box)
			main_vbox.disable_item_expand (box)
			
			gui.set_box_background_recursive (main_vbox, background_color)
			extend (main_vbox)
			close_request_actions.extend (agent process_cancel)
			gui.add_cancel_accelerator (window, agent process_cancel)
			gui.do_once_on_idle (agent post_display_actions)
			populate_packer_bindings
		end
		
	set_binding_types
			-- Set `binding_types' to list of distinct binding
			-- types
		local
			l_sorter: LIST_QUICK_SORTER [STRING]
			comparator: STRING_COMPARATOR
		do
			create {ARRAYED_LIST [STRING]} binding_types.make (5)
			binding_types.compare_objects
			bindings.do_all (agent add_binding_type)
			create l_sorter
			create comparator
			binding_types := l_sorter.sorted (binding_types, comparator)
		end

	add_binding_type (a_binding: MODIFIED_BINDING)
			-- Add binding_type for `a_binding' to `binding_types'
			-- if not already present
		local
			bt: STRING
		do
			bt := a_binding.type
			if not binding_types.has (bt) then
				binding_types.extend (bt)
			end
		end

	build_binding_type_field
			-- Build `binding_type_field'
		local
			fact: SINGLE_CONSTRAINED_VALUE_FACTORY
			bt, default_choice: STRING
		do
			create fact
			default_choice := ""
			fact.add_choice_and_text (default_choice, "Any")
			from
				binding_types.start
			until
				binding_types.after
			loop
				bt := binding_types.item
				fact.add_choice_and_text (bt, bt)
				binding_types.forth
			end
			fact.set_default_choice (default_choice)
			fact.generate_combo_box
			binding_type_field := fact.combo_box
			binding_type_field.disable_edit
			binding_type_field.select_actions.extend (agent process_change)
		end
		
	set_categories
			-- Set `categories' to list of distinct categories
		local
			l_sorter: LIST_QUICK_SORTER [STRING]
			comparator: STRING_COMPARATOR
		do
			create {ARRAYED_LIST [STRING]} categories.make (30)
			categories.compare_objects
			bindings.do_all (agent add_category)
			create l_sorter
			create comparator
			categories := l_sorter.sorted (categories, comparator)
		end

	add_category (a_binding: MODIFIED_BINDING)
			-- Add category for `a_binding' to `categories' if
			-- not already present
		local
			cat: STRING
		do
			cat := a_binding.category
			if not categories.has (cat) then
				categories.extend (cat)
			end
		end

	build_category_field
			-- Build `category_field'
		local
			fact: SINGLE_CONSTRAINED_VALUE_FACTORY
			cat, default_choice: STRING
		do
			create fact
			default_choice := ""
			fact.add_choice_and_text (default_choice, "Any")
			from
				categories.start
			until
				categories.after
			loop
				cat := categories.item
				fact.add_choice_and_text (cat, cat)
				categories.forth
			end
			fact.set_default_choice (default_choice)
			fact.generate_combo_box
			category_field := fact.combo_box
			category_field.disable_edit
			category_field.select_actions.extend (agent process_change)
		end
		
	build_packer_bindings
			-- Build `packer_bindings'
		do
			create packer_bindings
			packer_bindings.enable_multiple_selection
			packer_bindings.set_row_to_generic_function (agent row_to_binding_record)
			packer_bindings.set_visibility_counts_included (False)
			packer_bindings.build_selection_status_box
			packer_bindings.set_visible_columns (binding_columns)
		end

	populate_packer_bindings
			-- Populate `packer_bindings'
		local
			list: LIST [BINDING_MULTI_COLUMN_LIST_ROW]
		do
			create {ARRAYED_LIST [BINDING_MULTI_COLUMN_LIST_ROW]} list.make (bindings.count)
			bindings.do_all (agent add_binding_row (?, list))
			packer_bindings.set_all_rows (list)
		end
			
	add_binding_row (a_binding: MODIFIED_BINDING list: LIST [BINDING_MULTI_COLUMN_LIST_ROW])
			-- Add a row for `a_binding' to `list' if it is
			-- bound or if user wants unbound operations
		local
			row: BINDING_MULTI_COLUMN_LIST_ROW
		do
			if a_binding.is_bound or include_unbound_ops_button.is_selected then
				create row.make (a_binding)
				list.extend (row)
			end
		end
			
	process_include_unbound_ops_change
			-- Process change of state of `include_unbound_ops_button'
		do
			populate_packer_bindings
			process_change
		end

	post_display_actions
		local
			screen: EV_SCREEN
		do
			reset
			create screen
			gui.do_once_on_idle (agent gui.center_window_with_size (window, screen.width - 200, 500))
		end

	process_list
			-- Save text representation of list of all bound
			-- operations, in fixed order and with fixed
			-- columns
		do
			if is_active then
				disable_active
				compute_all_bindings
				save_actions.call ([separated_by_category (formatted_text (all_bindings, list_columns))])
				enable_active
			end
		end

	compute_all_bindings
			-- Compute list of all bindings that are bound to
			-- something, sorted properly for List operation
		local
			sorter: LIST_BUBBLE_SORTER [BINDING_MULTI_COLUMN_LIST_ROW]
			comp1: OPERATION_COMPARATOR [BINDING_MULTI_COLUMN_LIST_ROW]
			comp2: CATEGORY_COMPARATOR [BINDING_MULTI_COLUMN_LIST_ROW]
		do
			create {ARRAYED_LIST [BINDING_MULTI_COLUMN_LIST_ROW]} all_bindings.make (bindings.count)
			bindings.do_if (agent add_binding (?, all_bindings), agent {MODIFIED_BINDING}.is_bound)
			create sorter
			create comp1
			create comp2
			all_bindings := sorter.sorted (all_bindings, comp1)
			all_bindings := sorter.sorted (all_bindings, comp2)
		end
			
	add_binding (a_binding: MODIFIED_BINDING list: LIST [BINDING_MULTI_COLUMN_LIST_ROW])
			-- Add a row for `a_binding' to `list'
		local
			row: BINDING_MULTI_COLUMN_LIST_ROW
		do
			create row.make (a_binding)
			list.extend (row)
		end
			
	formatted_text (a_row_list: LIST [BINDING_MULTI_COLUMN_LIST_ROW] a_column_list: LIST [MULTI_COLUMN_LIST_COLUMN [BINDING_MULTI_COLUMN_LIST_ROW]]): STRING
			-- Nicely formatted text representing contents
			-- of `a_row_list', using just the columns in
			-- `a_column_list' in order
			-- in the Output window
		local
			l_formatter: MULTI_COLUMN_LIST_TEXT_FORMATTER [BINDING_MULTI_COLUMN_LIST_ROW]
		do
			create l_formatter
			Result := l_formatter.formatted_text (a_row_list, a_column_list, "  ", True, True)
		end

	separated_by_category (a_text: STRING): STRING
			-- `a_text' with a blank line inserted after every
			-- category change
		local
			list, words: LIST [STRING]
			line, prev_word: STRING
		do
			create Result.make (a_text.count)
			prev_word := ""
			list := su.broken_at (a_text, '%N')
			from
				list.start
			until
				list.after
			loop
				line := list.item
				words := su.broken_at (line, ' ')
				if not words.is_empty and then not words.first.is_empty and then not words.first.is_equal (prev_word) then
					if not prev_word.is_empty then
						Result.extend ('%N')
					end
					prev_word := words.first
				end
				Result.append (line)
				Result.extend ('%N')
				list.forth
			end
		end

	process_save
			-- Save text representation of list of all selected
			-- rows (in current order) and all visible columns
		local
			list: LIST [BINDING_MULTI_COLUMN_LIST_ROW]
		do
			if is_active then
				disable_active
				packer_bindings.rebuild_selected_rows
				list := packer_bindings.selected_rows
				if list.is_empty then
					inform ("No rows selected")
				else
					save_actions.call ([formatted_text (list, packer_bindings.visible_columns)])
				end
				enable_active
			end
		end

	process_cancel
		do
			if is_active then
				disable_active
				cancel_actions.call ([])
			end
		end

	process_change
			-- Process change of category field, binding type
			-- field or filter field
		local
			bt, cat, filt: STRING
		do
			if is_active then
				bt ?= binding_type_field.selected_item.data
				cat ?= category_field.selected_item.data
				filt := filter_field.text.as_lower
				if bt.is_empty and cat.is_empty and filt.is_empty then
					packer_bindings.process_view_all_rows
				else
					packer_bindings.process_view_rows_filtered (agent {BINDING_MULTI_COLUMN_LIST_ROW}.matches_type_category_and_op (bt, cat, filt))
				end
			end
		end

	binding_columns: like anchored_binding_columns
			-- List of columns for `packer_bindings'
		do
			create Result.make (4)
			Result.extend (binding_type_column)
			Result.extend (binding_column)
			Result.extend (category_column)
			Result.extend (operation_column)
		end
			
	list_columns: like anchored_binding_columns
			-- List of columns for List operation
		do
			create Result.make (3)
			Result.extend (category_column)
			Result.extend (binding_column)
			Result.extend (operation_column)
		end
			
	binding_type_column: BINDING_TYPE_COLUMN [BINDING_MULTI_COLUMN_LIST_ROW]
		do
			create Result
		end
	
	binding_column: BINDING_COLUMN [BINDING_MULTI_COLUMN_LIST_ROW]
		do
			create Result
		end
	
	category_column: CATEGORY_COLUMN [BINDING_MULTI_COLUMN_LIST_ROW]
		do
			create Result
		end
	
	operation_column: OPERATION_COLUMN [BINDING_MULTI_COLUMN_LIST_ROW]
		do
			create Result
		end
	
	row_to_binding_record (row: EV_MULTI_COLUMN_LIST_ROW): BINDING_MULTI_COLUMN_LIST_ROW
			-- Function which translates `row' to object of
			-- type BINDING_MULTI_COLUMN_LIST_ROW
		do
			Result ?= row
			check
				result_not_void: Result /= Void
			end
		end

	anchored_binding_columns: ARRAYED_LIST [MULTI_COLUMN_LIST_COLUMN [BINDING_MULTI_COLUMN_LIST_ROW]]
			-- For anchoring only
		do
		ensure
			result_void: Result = Void
		end
	
	binding_types: LIST [STRING]
			-- Sorted list of distinct binding types among all
			-- binding

	categories: LIST [STRING]
			-- Sorted list of distinct categories among all
			-- bindings

	list_button: EV_BUTTON
			-- Button to save text representation of all bindings
			-- (excludes unbound operations)
	
	save_button: EV_BUTTON
			-- Button to save text representation of currently
			-- visible and selected rows
	
	cancel_button: EV_BUTTON
			-- Button to hide window
	
	binding_type_field: EV_COMBO_BOX
			-- Field to filter rows displayed by binding type
	
	category_field: EV_COMBO_BOX
			-- Field to filter rows displayed by category
	
	filter_field: EV_TEXT_FIELD
			-- Field for string to filter rows displayed
	
	include_unbound_ops_button: EV_CHECK_BUTTON
			-- Button to control whether unbound operations
			-- are displayed
	
	packer_bindings: MULTI_COLUMN_LIST [BINDING_MULTI_COLUMN_LIST_ROW]
			-- List of key/pointer bindings
	
	all_bindings: LIST [BINDING_MULTI_COLUMN_LIST_ROW]

end
