
indexing
	
	description: "Interface to allow selection and packing projects"
	author: "David Hollenberg"
	date: "November 7, 2005"

deferred class RC_PACKING_IFC

inherit
	ABSTRACT_INTERFACE
		
feature {NONE} -- Creation

	make_interface
			-- Create interface
		do
			create edit_actions
			create success_actions
			create cancel_actions
		end

feature -- Modification

	set_session (sess: like session)
			-- Set `session' to `sess'
		require
			sess_exists: sess /= Void
		do
			session := sess
		ensure
			session_set: session = sess
		end

	set_run_id (s: like run_id)
			-- Set `run_id' to `s'
		do
			run_id := s
		ensure
			run_id_set: run_id = s
		end

	set_production_id (s: like production_id)
			-- Set `production_id' to `s'
		do
			production_id := s
		ensure
			production_id_set: production_id = s
		end

	set_packing_id (s: like packing_id)
			-- Set `packing_id' to `s'
		do
			packing_id := s
		ensure
			packing_id_set: packing_id = s
		end

	set_is_read_only (b: like is_read_only)
			-- Set `is_read_only' to `b'
		do
			is_read_only := b
		ensure
			read_only_set: is_read_only = b
		end

	set_is_cut_editor_only (b: like is_cut_editor_only)
			-- Set `is_cut_editor_only' to `b'
		do
			is_cut_editor_only := b
		ensure
			cut_editor_only_set: is_cut_editor_only = b
		end

	set_packing (a_packing: like packing)
			-- Set `packing' to `a_packing'
		do
			packing := a_packing
		ensure
			packing_set: packing = a_packing
		end

	set_packings_row (a_row: like packings_row)
			-- Set `packings_row' to `a_row'
		do
			packings_row := a_row
		ensure
			packings_row_set: packings_row = a_row
		end

	set_official_packing_version (a_packing_version: like official_packing_version)
			-- Set `official_packing_version' to `a_packing_version'
		do
			official_packing_version := a_packing_version
		ensure
			official_packing_version_set: official_packing_version = a_packing_version
		end

	set_role (r: like role)
			-- Set `role' to `r'
		do
			role := r
		ensure
			role_set: role = r
		end

	set_packing_session (sess: like packing_session)
			-- Set `packing_session' to `sess'
		require
			sess_exists: sess /= Void
		do
			packing_session := sess
		ensure
			packing_session_set: packing_session = sess
		end

	set_version_selection_interface (ifc: like version_selection_interface)
			-- Set `version_selection_interface' to `ifc'
		require
			ifc_exists: ifc /= Void
		do
			version_selection_interface := ifc
		ensure
			interface_set: version_selection_interface = ifc
		end

feature -- Properties

	session: RC_SESSION
			-- Session used to select and pack projects

	packing_session: RC_ABSTRACT_PACKING_SESSION
			-- Session used to do packing

	is_read_only: BOOLEAN
			-- Should packer run in read-only mode?

	is_cut_editor_only: BOOLEAN
			-- Should packer only allow editing in Cut mode,
			-- and not in Pack mode?

	packing: RC_PACKING
			-- Packing with packings rows synthesized from
			-- official database tables.  Void unless
			-- `is_cut_editor_only'

	packings_row: PACKINGS_MROW
			-- Packings row synthesized from
			-- official database tables.  Void unless
			-- `is_cut_editor_only'

	official_packing_version: PACKING_VERSIONS_MROW
			-- Version of packing synthesized from
			-- official database tables.  Void unless
			-- `is_cut_editor_only'

	role: STRING
			-- Required role

	run_id: STRING
			-- Run ID (Void if not created in context of a run)

	production_id: STRING
			-- Production ID (Void if packing initial run)

	packing_id: STRING
			-- Packing ID

	has_run_context: BOOLEAN
			-- Is packing being done in context of
			-- a specific run?
		do
			Result := run_id /= Void
		end
	
	has_packing_context: BOOLEAN
			-- Has a specific packing been selected
			-- for editing?
		do
			Result := packing_id /= Void
		end
	
	version_selection_interface: RC_PACKING_VERSIONS_IFC
			-- Interface to show packing versions and
			-- let user select one

	project_management_interface: RC_PACKING_PROJECTS_IFC
			-- Interface to manage projects while packing

	success_actions: CLOSED_ACTION_SEQUENCE
			-- Actions to be executed when user finishes
			-- editing packing

	cancel_actions: CLOSED_ACTION_SEQUENCE
			-- Actions to be executed when user clicks on Cancel
			-- button

	edit_actions: CLOSED_ACTION_SEQUENCE
			-- Actions to be executed when user confirms
			-- a packing to be edited

end
