
indexing
	
	description: "Window to display information about selected projects in packer"
	author: "David Hollenberg"
	date: "November 7, 2005"

class RC_PACKING_INFO_WINDOW

inherit
	RC_PACKING_INFO_IFC
		undefine
			default_create
		redefine
			set_dies
		end
	WINDOW_INTERFACE
		redefine
			enable_active,
			disable_active
		end
	MOSIS_GRAPHICAL_UTILITIES
		export
			{NONE} all
		undefine
			default_create
		end
	UNIX_OS_ACCESS
		export
			{NONE} all
		undefine
			default_create
		end

create
	make

feature {NONE} -- Creation

	make (w: EV_TITLED_WINDOW)
			-- Create interface in window `w'
		require
			window_exists: w /= Void
			window_valid: not w.is_destroyed
		do
			make_interface
			make_proxy (w)
		end

feature -- Operations

	reset
			-- Move focus back into window
		do
			enable_active
		end

feature -- Modification

	set_dies (a_list: like dies)
			-- Set `dies' to `a_list'
		local
			l_text: STRING
		do
			precursor (a_list)
			l_text := "Snapshot at "
			unix_os.append_current_date_string (l_text)
			date_label.set_text (l_text)
			build_list (packer_dies, dies)
			packer_dies.sort_on_column (2)
		end

	enable_active
			-- Make interface active
		do
			precursor
			save_button.enable_sensitive
			cancel_button.enable_sensitive
		end

	disable_active
			-- Make interface inactive
			-- (can look but cannot initiate most actions)
		do
			precursor
			save_button.disable_sensitive
			cancel_button.disable_sensitive
		end

feature {NONE} -- Implementation

	build_interface
			-- Build interface
		local
			cols: WIDGET_COLUMN_SET
			main_vbox: EV_VERTICAL_BOX
			box, status_box: EV_BOX
		do
			add_menu_bar
			create main_vbox
			main_vbox.set_padding (user_interface.padding)
			main_vbox.set_border_width (user_interface.border_width)
			create cols.make (2)
			cols.set_padding (user_interface.padding)
			cols.set_border_width (5)
			create include_scribes_button.make_with_text ("Display scribes")
			include_scribes_button.select_actions.extend (agent process_include_scribes_change)
			date_label := gui.label_with_text ("Date")
			cols.append_row (<< date_label, include_scribes_button >>)
			cols.disable_expand
			cols.enable_column_expand (1)
			cols.align_column_left (1)
			box := cols.box
			main_vbox.extend (box)
			main_vbox.disable_item_expand (box)
			
			create packer_dies
			packer_dies.enable_multiple_selection
			packer_dies.set_row_to_generic_function (agent row_to_record)
			packer_dies.set_visibility_counts_included (False)
			packer_dies.build_selection_status_box
			packer_dies.set_visible_columns (standard_columns)
			
			main_vbox.extend (packer_dies)
			
			status_box := separated_box (packer_dies.selection_status_box)
			create cols.make (2)
			cols.set_padding (user_interface.padding)
			cols.set_border_width (5)
			create save_button.make_with_text_and_action ("Save", agent process_save)
			save_button.set_minimum_width (user_interface.minimum_button_width)
			create cancel_button.make_with_text_and_action ("Cancel", agent process_cancel)
			cancel_button.set_minimum_width (user_interface.minimum_button_width)
			cols.append_row (<< save_button, cancel_button >>)
			cols.enable_expand
			cols.align_center
			box := cols.box
			main_vbox.extend (box)
			main_vbox.disable_item_expand (box)
			
			gui.set_box_background_recursive (main_vbox, background_color)
			main_vbox.start
			main_vbox.forth
			main_vbox.put_right (status_box)
			main_vbox.disable_item_expand (status_box)
			
			extend (main_vbox)
			close_request_actions.extend (agent process_cancel)
			gui.add_cancel_accelerator (window, agent process_cancel)
			gui.do_once_on_idle (agent post_display_actions)
			set_minimum_width (500)
		end
		
	add_menu_bar
			-- Add menu bar to window
		local
			a_menu_bar: EV_MENU_BAR
		do	
			build_file_menu
			build_select_menu
			build_view_menu
			create a_menu_bar
			a_menu_bar.extend (file_menu)
			a_menu_bar.extend (select_menu)
			a_menu_bar.extend (view_menu)
			set_menu_bar (a_menu_bar)
		end

	build_file_menu
			-- Build the File menu
		do	
			create file_menu.make_with_text ("File")
			add_menu_item (file_menu, "Close", agent process_cancel)
		end

	build_select_menu
			-- Build the Select menu
		do	
			create select_menu.make_with_text ("Select")
			add_menu_item (select_menu, "All", agent process_select_all)
			add_menu_item (select_menu, "None", agent process_select_none)
			add_menu_item (select_menu, "Invert", agent process_invert_selection)
		end

	process_select_all
			-- Process request to select all items in list
		do
			if is_active then
				disable_active
				packer_dies.process_select_all
				enable_active
			end
		end

	process_select_none
			-- Process request to unselect all items in list
		do
			if is_active then
				disable_active
				packer_dies.process_select_none
				enable_active
			end
		end

	process_invert_selection
			-- Process request to invert selection in list
		do
			if is_active then
				disable_active
				packer_dies.process_select_invert
				enable_active
			end
		end

	build_view_menu
			-- Build the View menu
		do	
			create View_menu.make_with_text ("View")
			add_menu_item (view_menu, "Basic", agent set_visible_columns (basic_columns))
			add_menu_item (view_menu, "Size", agent set_visible_columns (size_columns))
			add_menu_item (view_menu, "Lot", agent set_visible_columns (lot_columns))
			add_menu_item (view_menu, "Submitted", agent set_visible_columns (submitted_columns))
			add_menu_item (view_menu, "Full", agent set_visible_columns (standard_columns))
			add_menu_item (view_menu, "Full Plus Centers", agent set_visible_columns (full_plus_center_columns))
		end

	add_menu_item (a_menu: EV_MENU a_text: STRING an_action: PROCEDURE [ANY, TUPLE])
			-- Add to `a_menu' new menu item with `a_text'
			-- that invokes `an_action' when selected.
			-- Leave new item in `last_menu_item'
		require
			a_menu_not_void: a_menu /= Void
			a_text_not_void: a_text /= Void
			an_action_not_void: an_action /= Void
		do
			create last_menu_item.make_with_text (a_text)
			last_menu_item.select_actions.extend (an_action)
			a_menu.extend (last_menu_item)
		end

	post_display_actions
		do
			reset
		end

	process_include_scribes_change
			-- Process change of state of `include_scribes_button'
		do
			build_list (packer_dies, dies)
		end

	process_save
		local
			list: LIST [PACKING_DIE_MULTI_COLUMN_LIST_ROW]
		do
			if is_active then
				disable_active
				packer_dies.rebuild_selected_rows
				list := packer_dies.selected_rows
				if list.is_empty then
					inform ("No rows selected")
				else
					save_actions.call ([packer_dies.selected_rows, packer_dies.visible_columns])
				end
				enable_active
			end
		end

	process_cancel
		do
			if is_active then
				disable_active
				cancel_actions.call ([])
			end
		end

	set_visible_columns (a_column_list: like anchored_columns)
		do
			packer_dies.set_visible_columns (a_column_list)
		end

	build_list (a_die_list: like packer_dies rows: LIST [RETICLE_DIE])
			-- Build multi-column list `a_die_list' from `rows'
		local
			list: ARRAYED_LIST [PACKING_DIE_MULTI_COLUMN_LIST_ROW]
		do
			create list.make (rows.count)
			rows.do_all (agent add_row (?, list, a_die_list))
			a_die_list.set_all_rows (list)
		end
			
	add_row (d: RETICLE_DIE list: LIST [PACKING_DIE_MULTI_COLUMN_LIST_ROW] die_list: like packer_dies)
			-- Add a row for `d' to `list', which will be
			-- the list of rows for `die_list'
		local
			row: PACKING_DIE_MULTI_COLUMN_LIST_ROW
		do
			if include_scribes_button.is_selected or not d.is_scribe then
				create row.make (d, packing)
				row.set_grid (grid)
				row.select_actions.extend (agent die_list.process_select_one (row))
				row.deselect_actions.extend (agent die_list.process_deselect_one (row))
				list.extend (row)
			end
		end
			
	separated_box (box: EV_BOX): EV_BOX
			-- `box' with horizontal separators above and below it
		local
			sep: EV_HORIZONTAL_SEPARATOR
		do
			create {EV_VERTICAL_BOX} Result
			create sep
			Result.extend (sep)
			Result.disable_item_expand (sep)
			Result.extend (box)
			create sep
			Result.extend (sep)
			Result.disable_item_expand (sep)
		end

	row_to_record (row: EV_MULTI_COLUMN_LIST_ROW): PACKING_DIE_MULTI_COLUMN_LIST_ROW
			-- Function which translates `row' to object of
			-- type PACKING_DIE_MULTI_COLUMN_LIST_ROW
		do
			Result ?= row
			check
				result_not_void: Result /= Void
			end
		end

	anchored_columns: ARRAYED_LIST [MULTI_COLUMN_LIST_COLUMN [PACKING_DIE_MULTI_COLUMN_LIST_ROW]]
			-- For anchoring only
		do
		ensure
			result_void: Result = Void
		end
	
	standard_columns: like anchored_columns
			-- Standard list of columns for `packer_dies'
		do
			create Result.make (15)
			Result.extend (die_type_column)
			Result.extend (design_no_column)
			Result.extend (die_id_column)
			Result.extend (wafer_etest_id_column)
			Result.extend (alt_etest_ids_column)
			Result.extend (quantity_ordered_column)
			Result.extend (size_x_column)
			Result.extend (size_y_column)
			Result.extend (max_size_x_column)
			Result.extend (max_size_y_column)
			Result.extend (overhead_style_column)
			Result.extend (date_time_fab_requested_column)
			Result.extend (acct_no_column)
			Result.extend (account_type_column)
			Result.extend (org_short_id_column)
			Result.extend (design_name_column)
			Result.extend (empty_column)
		end
			
	basic_columns: like anchored_columns
			-- Basic list of columns for `packer_dies'
		do
			create Result.make (15)
			Result.extend (die_type_column)
			Result.extend (design_no_column)
			Result.extend (die_id_column)
			Result.extend (quantity_ordered_column)
			Result.extend (size_x_column)
			Result.extend (size_y_column)
			Result.extend (max_size_x_column)
			Result.extend (max_size_y_column)
			Result.extend (overhead_style_column)
			Result.extend (acct_no_column)
			Result.extend (account_type_column)
			Result.extend (org_short_id_column)
			Result.extend (empty_column)
		end
			
	size_columns: like anchored_columns
			-- List of columns for `packer_dies' with size info
		do
			create Result.make (15)
			Result.extend (die_type_column)
			Result.extend (design_no_column)
			Result.extend (die_id_column)
			Result.extend (size_x_column)
			Result.extend (size_y_column)
			Result.extend (max_size_x_column)
			Result.extend (max_size_y_column)
			Result.extend (empty_column)
		end
			
	lot_columns: like anchored_columns
			-- List of columns for `packer_dies' with lot info
		do
			create Result.make (15)
			Result.extend (die_type_column)
			Result.extend (design_no_column)
			Result.extend (die_id_column)
			Result.extend (wafer_etest_id_column)
			Result.extend (alt_etest_ids_column)
			Result.extend (empty_column)
		end
			
	submitted_columns: like anchored_columns
			-- List of columns for `packer_dies' with lot info
		do
			create Result.make (15)
			Result.extend (die_type_column)
			Result.extend (design_no_column)
			Result.extend (die_id_column)
			Result.extend (date_time_fab_requested_column)
			Result.extend (acct_no_column)
			Result.extend (account_type_column)
			Result.extend (org_short_id_column)
			Result.extend (empty_column)
		end
			
	full_plus_center_columns: like anchored_columns
			-- List of all columns for `packer_dies'
		do
			create Result.make (15)
			Result.extend (die_type_column)
			Result.extend (design_no_column)
			Result.extend (die_id_column)
			Result.extend (wafer_etest_id_column)
			Result.extend (alt_etest_ids_column)
			Result.extend (quantity_ordered_column)
			Result.extend (size_x_column)
			Result.extend (size_y_column)
			Result.extend (center_x_column)
			Result.extend (center_y_column)
			Result.extend (field_name_column)
			Result.extend (max_size_x_column)
			Result.extend (max_size_y_column)
			Result.extend (overhead_style_column)
			Result.extend (date_time_fab_requested_column)
			Result.extend (acct_no_column)
			Result.extend (account_type_column)
			Result.extend (org_short_id_column)
			Result.extend (design_name_column)
			Result.extend (empty_column)
		end
			
	die_type_column: DIE_TYPE_COLUMN [PACKING_DIE_MULTI_COLUMN_LIST_ROW]
		do
			create Result
		end

	design_no_column: DESIGN_NO_COLUMN [PACKING_DIE_MULTI_COLUMN_LIST_ROW]
		do
			create Result
		end

	die_id_column: DIE_ID_COLUMN [PACKING_DIE_MULTI_COLUMN_LIST_ROW]
		do
			create Result
		end

	design_name_column: DESIGN_NAME_COLUMN [PACKING_DIE_MULTI_COLUMN_LIST_ROW]
		do
			create Result
		end

	wafer_etest_id_column: WAFER_ETEST_ID_COLUMN [PACKING_DIE_MULTI_COLUMN_LIST_ROW]
		do
			create Result
		end

	alt_etest_ids_column: ALT_ETEST_IDS_COLUMN [PACKING_DIE_MULTI_COLUMN_LIST_ROW]
		do
			create Result
		end

	acct_no_column: ACCT_NO_COLUMN [PACKING_DIE_MULTI_COLUMN_LIST_ROW]
		do
			create Result
		end

	account_type_column: ACCOUNT_TYPE_COLUMN [PACKING_DIE_MULTI_COLUMN_LIST_ROW]
		do
			create Result
		end

	overhead_style_column: OVERHEAD_STYLE_COLUMN [PACKING_DIE_MULTI_COLUMN_LIST_ROW]
		do
			create Result
		end

	org_short_id_column: ORG_SHORT_ID_COLUMN [PACKING_DIE_MULTI_COLUMN_LIST_ROW]
		do
			create Result
		end

	quantity_ordered_column: QUANTITY_ORDERED_COLUMN [PACKING_DIE_MULTI_COLUMN_LIST_ROW]
		do
			create Result
		end

	date_time_fab_requested_column: DATE_TIME_FAB_REQUESTED_COLUMN [PACKING_DIE_MULTI_COLUMN_LIST_ROW]
		do
			create Result
		end

	size_x_column: SIZE_X_COLUMN [PACKING_DIE_MULTI_COLUMN_LIST_ROW]
		do
			create Result
		end

	size_y_column: SIZE_Y_COLUMN [PACKING_DIE_MULTI_COLUMN_LIST_ROW]
		do
			create Result
		end

	max_size_x_column: MAX_SIZE_X_COLUMN [PACKING_DIE_MULTI_COLUMN_LIST_ROW]
		do
			create Result
		end

	max_size_y_column: MAX_SIZE_Y_COLUMN [PACKING_DIE_MULTI_COLUMN_LIST_ROW]
		do
			create Result
		end

	center_x_column: CENTER_X_COLUMN [PACKING_DIE_MULTI_COLUMN_LIST_ROW]
		do
			create Result
		end

	center_y_column: CENTER_Y_COLUMN [PACKING_DIE_MULTI_COLUMN_LIST_ROW]
		do
			create Result
		end

	field_name_column: FIELD_NAME_COLUMN [PACKING_DIE_MULTI_COLUMN_LIST_ROW]
		do
			create Result
		end

	empty_column: EMPTY_COLUMN [PACKING_DIE_MULTI_COLUMN_LIST_ROW]
		do
			create Result
		end

	date_label: EV_LABEL
			-- Label with date the information was sent to window
	
	include_scribes_button: EV_CHECK_BUTTON
			-- Button to control whether scribes are displayed
	
	save_button: EV_BUTTON
			-- Button to save text for selected rows
	
	cancel_button: EV_BUTTON
			-- Button to hide window
	
	packer_dies: MULTI_COLUMN_LIST [PACKING_DIE_MULTI_COLUMN_LIST_ROW]
			-- List of dies and locations
	
	file_menu: EV_MENU
			-- Menu for file operations

	select_menu: EV_MENU
			-- Menu to allow user to select/deselect items

	view_menu: EV_MENU
			-- Menu to allow user to change which columns are
			-- displayed

	last_menu_item: EV_MENU_ITEM
			-- Last menu item created

end
