
indexing
	
	description: "Window to allow user to assign a lot to projects %
		%during packing"
	author: "David Hollenberg"
	date: "December 18, 2006"

class RC_PACKING_LOT_ASSIGN_WINDOW

inherit
	RC_PACKING_LOT_ASSIGN_IFC
	WINDOW_INTERFACE
		redefine
			enable_active,
			disable_active
		end
	MOSIS_GRAPHICAL_UTILITIES
		export
			{NONE} all
		undefine
			default_create
		end

create
	make

feature {NONE} -- Creation

	make (w: EV_TITLED_WINDOW)
			-- Create interface in window `w'
		require
			window_exists: w /= Void
			window_valid: not w.is_destroyed
		do
			make_interface
			make_proxy (w)
		end

feature -- Operations

	reset
			-- Move focus back into window
		do
			enable_active
		end

feature -- Modification

	enable_active
			-- Make interface active
		do
			precursor
			ok_button.enable_sensitive
			cancel_button.enable_sensitive
		end

	disable_active
			-- Make interface inactive
			-- (can look but cannot initiate most actions)
		do
			precursor
			ok_button.disable_sensitive
			cancel_button.disable_sensitive
		end

feature {NONE} -- Implementation

	build_interface
			-- Build interface
		local
			cols: WIDGET_COLUMN_SET
			main_vbox: EV_VERTICAL_BOX
			label: EV_LABEL
			buttons: HORIZONTAL_BUTTON_SET
			box: EV_BOX
			n: INTEGER
			text: EV_TEXT
		do
			build_compatible_lots
			create main_vbox
			main_vbox.set_padding (user_interface.padding)
			main_vbox.set_border_width (user_interface.border_width)
			n := projects.count
			create label.make_with_text ("Assign lot for " + n.out + " " + su.number (n, "project", "projects"))
			main_vbox.extend (label)
			main_vbox.disable_item_expand (label)
			
			create text
			text.set_text (projects_description)
			text.disable_edit
			main_vbox.extend (text)

			create cols.make (2)
			cols.set_padding (user_interface.padding)
			cols.set_border_width (10)
			build_wafer_etest_id_field
			cols.append_row (<< gui.label_with_text ("Assigned Lot"), wafer_etest_id_field >>)
			cols.disable_expand
			cols.enable_column_expand (2)
			cols.align_column_right (1)
			box := cols.box
			main_vbox.extend (box)
			main_vbox.disable_item_expand (box)

			create buttons.make_with_text_and_actions (<<
				"OK", "Cancel" >>,
				<< agent process_ok, agent process_cancel >>)
			ok_button := buttons.i_th (1)
			cancel_button := buttons.i_th (2)
			
			box := buttons.column_set.box
			
			main_vbox.extend (box)
			main_vbox.disable_item_expand (box)
			
			gui.set_box_background_recursive (main_vbox, background_color)
			extend (main_vbox)
			close_request_actions.extend (agent process_cancel)
			gui.add_cancel_accelerator (window, agent process_cancel)
			gui.do_once_on_idle (agent post_display_actions)
			set_minimum_width (600)
			set_minimum_height (300)
		end
		
	post_display_actions
		do
			reset
		end

	process_ok
		do
			if is_active then
				disable_active
				accept_actions.call ([selected_lot])
			end
		end

	process_cancel
		do
			if is_active then
				disable_active
				cancel_actions.call ([])
			end
		end

	projects_description: STRING
			-- Description of `projects'
		local
			elem: PACKING_DIES_MROW
		do
			create Result.make (projects.count * 50)
			from
				projects.start
			until
				projects.after
			loop
				elem := projects.item
				Result.append_integer (elem.design_no)
				Result.append ("    ")
				Result.append (elem.design_name)
				Result.extend ('%N')
				projects.forth
			end
		end

	build_compatible_lots
			-- Build list of lots with which all
			-- projects in `projects' are compatible
		local
			elem: PACKING_DIES_MROW
			lots: like compatible_lots
			lot: WAFER_ETEST_NAMES_ROW
			names: ARRAYED_LIST [STRING]
			all_lots: like compatible_lots
			sorter: LIST_BUBBLE_SORTER [WAFER_ETEST_NAMES_ROW]
			comparator: WAFER_ETEST_ID_COMPARATOR [WAFER_ETEST_NAMES_ROW]
			found, compat_with_all: BOOLEAN
		do
			create {ARRAYED_LIST [WAFER_ETEST_NAMES_ROW]} all_lots.make (10)
			create names.make (10)
			names.compare_objects
			from
				projects.start
			until
				projects.after
			loop
				elem := projects.item
				lots := elem.compatible_wafer_etest_names
				from
					lots.start
				until
					lots.after
				loop
					lot := lots.item
					if not names.has (lot.wafer_etest_name) then
						names.extend (lot.wafer_etest_name)
						all_lots.extend (lot)
					end
					lots.forth
				end
				projects.forth
			end
			
			-- All_lots now has distinct list of all lots
			-- with which any of the projects are
			-- compatible.  Create list of lots with which
			-- all projects are compatible
			
			create {ARRAYED_LIST [WAFER_ETEST_NAMES_ROW]} compatible_lots.make (10)
			from
				all_lots.start
			until
				all_lots.after
			loop
				lot := all_lots.item
				compat_with_all := True
				from
					projects.start
				until
					projects.after or not compat_with_all
				loop
					lots := projects.item.compatible_wafer_etest_names
					found := False
					from
						lots.start
					until
						lots.after or found
					loop
						if lots.item.wafer_etest_name.is_equal (lot.wafer_etest_name) then
							found := True
						end
						lots.forth
					end
					if not found then
						compat_with_all := False
					end
					projects.forth
				end
				if compat_with_all then
					compatible_lots.extend (lot)
				end
				all_lots.forth
			end
			

			create sorter
			create comparator
			compatible_lots := sorter.sorted (compatible_lots, comparator)
		end

	build_wafer_etest_id_field
			-- Build `wafer_etest_id_field'
		local
			lot: WAFER_ETEST_NAMES_ROW
			list_item: EV_LIST_ITEM
		do
			create wafer_etest_id_field
			create list_item.make_with_text ("None")
			list_item.select_actions.extend (agent set_lot (Void))
			wafer_etest_id_field.extend (list_item)
			from
				compatible_lots.start
			until
				compatible_lots.after
			loop
				lot := compatible_lots.item
				create list_item.make_with_text (lot.wafer_etest_id)
				list_item.select_actions.extend (agent set_lot (lot))
				wafer_etest_id_field.extend (list_item)
				compatible_lots.forth
			end
			wafer_etest_id_field.disable_edit
		end

	set_lot (a_lot: WAFER_ETEST_NAMES_ROW)
		do
			selected_lot := a_lot
		end

	wafer_etest_id_field: EV_COMBO_BOX
			-- Field for displaying list of wafer etest IDs
			-- compatible with all projects in `projects'
	
	ok_button: EV_BUTTON
			-- Button to accept current lot
	
	cancel_button: EV_BUTTON
			-- Button to cancel lot assignment
	
	compatible_lots: LIST [WAFER_ETEST_NAMES_ROW]
			-- List of lots with which every project in
			-- `projects' is compatible
	
	selected_lot: WAFER_ETEST_NAMES_ROW
			-- Currently selected lot to assign (Void indicates
			-- remove any previous assignment)
	
end
