
indexing
	
	description: "Window to display summary of project/lot compatibility"
	author: "David Hollenberg"
	date: "December 18, 2006"

class RC_PACKING_LOT_SUMMARY_WINDOW

inherit
	RC_PACKING_LOT_SUMMARY_IFC
	WINDOW_INTERFACE
		redefine
			enable_active,
			disable_active
		end
	MOSIS_GRAPHICAL_UTILITIES
		export
			{NONE} all
		undefine
			default_create
		end

create
	make

feature {NONE} -- Creation

	make (w: EV_TITLED_WINDOW)
			-- Create interface in window `w'
		require
			window_exists: w /= Void
			window_valid: not w.is_destroyed
		do
			make_interface
			make_proxy (w)
		end

feature -- Operations

	reset
			-- Move focus back into window
		do
			enable_active
		end

	refresh
			-- Refresh contents of interface
		local
			sorter: LIST_BUBBLE_SORTER [PACKING_DIES_MROW]
			comparator: DESIGN_NO_COMPARATOR [PACKING_DIES_MROW]
		do
			if is_active then
				disable_active
				create sorter
				create comparator
				projects := sorter.sorted (projects_agent.item ([]), comparator)
				build_all_lots
				build_packer_compatible_lots
				build_packer_assigned_lots
				build_packer_projects
				enable_active
			end
		end

feature -- Modification

	enable_active
			-- Make interface active
		do
			precursor
			refresh_button.enable_sensitive
			send_button.enable_sensitive
			cancel_button.enable_sensitive
		end

	disable_active
			-- Make interface inactive
			-- (can look but cannot initiate most actions)
		do
			precursor
			refresh_button.disable_sensitive
			send_button.disable_sensitive
			cancel_button.disable_sensitive
		end

feature {NONE} -- Implementation

	build_interface
			-- Build interface
		local
			main_vbox: EV_VERTICAL_BOX
			buttons: HORIZONTAL_BUTTON_SET
			box: EV_BOX
		do
			create main_vbox
			main_vbox.set_padding (user_interface.padding)
			main_vbox.set_border_width (user_interface.border_width)
			build_notebook
			main_vbox.extend (notebook)

			create buttons.make_with_text_and_actions (<<
				"Refresh", "Send", "Cancel" >>,
				<< agent refresh, agent process_send, agent process_cancel >>)
			refresh_button := buttons.i_th (1)
			send_button := buttons.i_th (2)
			cancel_button := buttons.i_th (3)
			refresh_button.set_tooltip ("Refresh lot summary with current information")
			send_button.set_tooltip ("Send data in current notebook tab to Output window")
			
			box := buttons.column_set.box
			
			main_vbox.extend (box)
			main_vbox.disable_item_expand (box)
			
			gui.set_box_background_recursive (main_vbox, background_color)
			extend (main_vbox)
			close_request_actions.extend (agent process_cancel)
			gui.add_cancel_accelerator (window, agent process_cancel)
			gui.do_once_on_idle (agent post_display_actions)
		end
		
	build_notebook
			-- Build notebook with tabs for various views
			-- of lot summary
		local
			box: EV_VERTICAL_BOX
		do
			create notebook
			
			create packer_compatible_lots
			packer_compatible_lots.enable_multiple_selection
			packer_compatible_lots.set_row_to_generic_function (agent row_to_lot_record)
			packer_compatible_lots.set_visibility_counts_included (False)
			packer_compatible_lots.build_selection_status_box
			packer_compatible_lots.set_visible_columns (compatibility_columns)
			
			create box
			box.set_border_width (user_interface.border_width)
			box.extend (packer_compatible_lots)
			box.set_data (packer_compatible_lots)
			notebook.extend (box)
			notebook.set_item_text (box, compatibility_overview_text)
			
			create packer_assigned_lots
			packer_assigned_lots.enable_multiple_selection
			packer_assigned_lots.set_row_to_generic_function (agent row_to_lot_record)
			packer_assigned_lots.set_visibility_counts_included (False)
			packer_assigned_lots.build_selection_status_box
			packer_assigned_lots.set_visible_columns (assignment_columns)
			
			create box
			box.set_border_width (user_interface.border_width)
			box.extend (packer_assigned_lots)
			box.set_data (packer_assigned_lots)
			notebook.extend (box)
			notebook.set_item_text (box, lot_assignments_text)
			
			create packer_projects
			packer_projects.enable_multiple_selection
			packer_projects.set_row_to_generic_function (agent row_to_project_record)
			packer_projects.set_visibility_counts_included (False)
			packer_projects.build_selection_status_box
			packer_projects.set_visible_columns (initial_project_columns)
			
			create box
			box.set_border_width (user_interface.border_width)
			box.extend (packer_projects)
			box.set_data (packer_projects)
			notebook.extend (box)
			notebook.set_item_text (box, project_lot_matrix_text)
		end
		
	post_display_actions
		do
			reset
			gui.do_once_on_idle (agent gui.center_window_with_margin (window, 200, 200))
		end

	process_send
			-- Send nicely formatted text of list of projects
			-- in current notebook tab to observers
		local
			list1: like current_tab_lot_summary_list
			list2: like current_tab_project_list
			list: MULTI_COLUMN_LIST [MULTI_COLUMN_LIST_ROW]
			descr: STRING
		do
			if is_active then
				disable_active
				list1 := current_tab_lot_summary_list
				list2 := current_tab_project_list
				if list1 /= Void then
					list := list1
					if list1 = packer_compatible_lots then
						descr := compatibility_overview_text
					elseif list1 = packer_assigned_lots then
						descr := lot_assignments_text
					else
						handle_hard_error ("Unknown lot summary list")
					end
				elseif list2 /= Void then
					list := list2
					if list2 = packer_projects then
						descr := project_lot_matrix_text
					else
						handle_hard_error ("Unknown project list")
					end
				else
					handle_hard_error ("Unknown list in notebook tab")
				end
				send_actions.call ([descr, formatted_text (list.all_rows, list.visible_columns)])
				enable_active
			end
		end

	process_cancel
		do
			if is_active then
				disable_active
				cancel_actions.call ([])
			end
		end

	formatted_text (a_row_list: LIST [MULTI_COLUMN_LIST_ROW] a_column_list: LIST [MULTI_COLUMN_LIST_COLUMN [MULTI_COLUMN_LIST_ROW]]): STRING
			-- Nicely formatted text representing contents
			-- of `a_row_list', using just the columns in
			-- `a_column_list' in order
			-- in the Output window
		local
			l_formatter: MULTI_COLUMN_LIST_TEXT_FORMATTER [MULTI_COLUMN_LIST_ROW]
		do
			create l_formatter
			Result := l_formatter.formatted_text (a_row_list, a_column_list, "  ", True, True)
		end

	build_all_lots
			-- Build distinct list of lots with which at least
			-- one project in `projects' is compatible
		local
			elem: PACKING_DIES_MROW
			lots: like compatible_lots
			lot: WAFER_ETEST_NAMES_ROW
			names: ARRAYED_LIST [STRING]
			sorter: LIST_BUBBLE_SORTER [WAFER_ETEST_NAMES_ROW]
			comparator: WAFER_ETEST_ID_COMPARATOR [WAFER_ETEST_NAMES_ROW]
		do
			create {ARRAYED_LIST [WAFER_ETEST_NAMES_ROW]} compatible_lots.make (10)
			create names.make (10)
			names.compare_objects
			from
				projects.start
			until
				projects.after
			loop
				elem := projects.item
				lots := elem.compatible_wafer_etest_names
				from
					lots.start
				until
					lots.after
				loop
					lot := lots.item
					if not names.has (lot.wafer_etest_name) then
						names.extend (lot.wafer_etest_name)
						compatible_lots.extend (lot)
					end
					lots.forth
				end
				projects.forth
			end
			

			create sorter
			create comparator
			compatible_lots := sorter.sorted (compatible_lots, comparator)
		end

	build_packer_compatible_lots
			-- Build `packer_compatible_lots'
		do
			build_lot_list (packer_compatible_lots)
		end
			
	build_packer_assigned_lots
			-- Build `packer_assigned_lots'
		local
			list: LIST [LOT_SUMMARY_MULTI_COLUMN_LIST_ROW]
			row: LOT_SUMMARY_MULTI_COLUMN_LIST_ROW
		do
			list := new_lot_list
			create row.make ("None")
			row.set_total_assigned_count (unassigned_count)
			list.extend (row)
			packer_assigned_lots.set_all_rows (list)
		end
			
	build_lot_list (lots: MULTI_COLUMN_LIST [LOT_SUMMARY_MULTI_COLUMN_LIST_ROW])
			-- Build multi-column list `packer_compatible_lots'
		require
			lots_not_void: lots /= Void
		do
			lots.set_all_rows (new_lot_list)
		end
			
	new_lot_list: LIST [LOT_SUMMARY_MULTI_COLUMN_LIST_ROW]
			-- New list with one row for each element of
			-- `compatible_lots'
		do
			create {ARRAYED_LIST [LOT_SUMMARY_MULTI_COLUMN_LIST_ROW]} Result.make (compatible_lots.count)
			compatible_lots.do_all (agent add_lot_row (?, Result))
		end
			
	add_lot_row (lot: WAFER_ETEST_NAMES_ROW list: LIST [LOT_SUMMARY_MULTI_COLUMN_LIST_ROW])
			-- Add a row for `lot' to `list'
		local
			row: LOT_SUMMARY_MULTI_COLUMN_LIST_ROW
			an_etest_id: STRING
		do
			an_etest_id := lot.wafer_etest_id
			compute_counts (an_etest_id)
			create row.make (an_etest_id)
			row.set_required_count (required_count)
			row.set_incompatible_count (forbidden_count)
			row.set_assigned_count (assigned_count)
			row.set_assignable_count (assignable_count)
			row.set_total_assigned_count (required_count + assigned_count)
			row.set_total_count (total_count)
			list.extend (row)
		end
			
	compute_counts (an_etest_id: STRING)
			-- Compute number of projects that require, forbid,
			-- assign currently and could assign `an_etest_id'
		local
			p: PACKING_DIES_ROW
			ids: LIST [STRING]
			curr_id: STRING
			has_id: BOOLEAN
		do
			required_count := 0
			forbidden_count := 0
			assigned_count := 0
			assignable_count := 0
			total_count := projects.count
			from
				projects.start
			until
				projects.after
			loop
				p := projects.item
				ids := p.compatible_wafer_etest_ids
				curr_id := p.assigned_wafer_etest_id
				has_id := ids.has (an_etest_id)
				if ids.count = 1 and has_id then
					required_count := required_count + 1
				elseif not has_id then
					forbidden_count := forbidden_count + 1
				elseif curr_id.is_equal (an_etest_id) then
					-- Currently assigned `an_etest_id'
					assigned_count := assigned_count + 1
				else
					-- Could be  assigned `an_etest_id'
					-- but currently is not
					assignable_count := assignable_count + 1
				end
				projects.forth
			end
		end
			
	unassigned_count: INTEGER
			-- Number of projects that do not have an etest ID
			-- assigned yet
		local
			p: PACKING_DIES_ROW
		do
			from
				projects.start
			until
				projects.after
			loop
				p := projects.item
				if p.assigned_wafer_etest_id.is_empty then
					Result := Result + 1
				end
				projects.forth
			end
		end
			
	row_to_lot_record (row: EV_MULTI_COLUMN_LIST_ROW): LOT_SUMMARY_MULTI_COLUMN_LIST_ROW
			-- Function which translates `row' to object of
			-- type LOT_SUMMARY_MULTI_COLUMN_LIST_ROW
		do
			Result ?= row
			check
				result_not_void: Result /= Void
			end
		end

	row_to_project_record (row: EV_MULTI_COLUMN_LIST_ROW): PACKING_PROJ_MULTI_COLUMN_LIST_ROW
			-- Function which translates `row' to object of
			-- type PACKING_PROJ_MULTI_COLUMN_LIST_ROW
		do
			Result ?= row
			check
				result_not_void: Result /= Void
			end
		end

	build_packer_projects
			-- Build `packer_projects'
		local
			list: LIST [PACKING_PROJ_MULTI_COLUMN_LIST_ROW]
		do
			create {ARRAYED_LIST [PACKING_PROJ_MULTI_COLUMN_LIST_ROW]} list.make (projects.count)
			projects.do_all (agent add_project_row (?, list))
			packer_projects.set_all_rows (list)
			packer_projects.set_visible_columns (project_columns)
		end
			
	add_project_row (die: PACKING_DIES_MROW list: LIST [PACKING_PROJ_MULTI_COLUMN_LIST_ROW])
			-- Add a row for `lot' to `list'
		local
			row: PACKING_PROJ_MULTI_COLUMN_LIST_ROW
		do
			create row.make (die.project, packing)
			row.set_die (die)
			row.set_lot_compatibility_status (compatibility_status (die))
			list.extend (row)
		end
			
	compatibility_status (die: PACKING_DIES_ROW): LIST [STRING]
			-- List of strings indicating compatibility of
			-- `die' with compatible lots
		local
			lot: WAFER_ETEST_NAMES_ROW
			ids: LIST [STRING]
			curr_id: STRING
			has_id: BOOLEAN
		do
			create {ARRAYED_LIST [STRING]} Result.make (compatible_lots.count)
			from
				compatible_lots.start
			until
				compatible_lots.after
			loop
				lot := compatible_lots.item
				ids := die.compatible_wafer_etest_ids
				curr_id := die.assigned_wafer_etest_id
				has_id := ids.has (lot.wafer_etest_id)
				if ids.count = 1 and has_id then
					Result.extend (Only_status)
				elseif not has_id then
					Result.extend (Incompatible_status)
				elseif curr_id.is_equal (lot.wafer_etest_id) then
					-- Currently assigned lot's etest ID
					Result.extend (Assigned_status)
				else
					-- Could be assigned lot's etest ID
					-- but currently is not
					Result.extend (Allowed_status)
				end
				compatible_lots.forth
			end
		end
			
	current_tab_lot_summary_list: MULTI_COLUMN_LIST [LOT_SUMMARY_MULTI_COLUMN_LIST_ROW]
			-- Lot summary multi-column list in currently 
			-- selected notebook, if any
		do
			Result ?= notebook.selected_item.data
		end

	current_tab_project_list: MULTI_COLUMN_LIST [PACKING_PROJ_MULTI_COLUMN_LIST_ROW]
			-- Project multi-column list in currently 
			-- selected notebook, if any
		do
			Result ?= notebook.selected_item.data
		end

	anchored_lot_columns: ARRAYED_LIST [MULTI_COLUMN_LIST_COLUMN [LOT_SUMMARY_MULTI_COLUMN_LIST_ROW]]
			-- For anchoring only
		do
		ensure
			result_void: Result = Void
		end
	
	anchored_project_columns: ARRAYED_LIST [MULTI_COLUMN_LIST_COLUMN [PACKING_PROJ_MULTI_COLUMN_LIST_ROW]]
			-- For anchoring only
		do
		ensure
			result_void: Result = Void
		end
	
	compatibility_columns: like anchored_lot_columns
			-- List of columns for `packer_compatible_lots'
		do
			create Result.make (6)
			Result.extend (wafer_etest_id_column)
			Result.extend (required_count_column)
			Result.extend (incompatible_count_column)
			Result.extend (assigned_count_column)
			Result.extend (assignable_count_column)
			Result.extend (empty_lot_column)
		end
			
	assignment_columns: like anchored_lot_columns
			-- List of columns for `packer_assigned_lots'
		do
			create Result.make (6)
			Result.extend (wafer_etest_id_column)
			Result.extend (total_assigned_count_column)
			Result.extend (empty_lot_column)
		end
			
	initial_project_columns: like anchored_project_columns
			-- List of columns for `packer_projects', before
			-- we know list of compatible wafer lots
		do
			create Result.make (6)
			Result.extend (design_no_column)
			Result.extend (is_lot_assigned_column)
			Result.extend (empty_project_column)
		end
			
	project_columns: like anchored_project_columns
			-- List of columns for `packer_projects', after
			-- we know list of compatible wafer lots
		local
			lot: WAFER_ETEST_NAMES_ROW
			n: INTEGER
		do
			create Result.make (6)
			Result.extend (design_no_column)
			Result.extend (is_lot_assigned_column)
			from
				compatible_lots.start
			until
				compatible_lots.after
			loop
				lot := compatible_lots.item
				n := n + 1
				Result.extend (create {LOT_COMPATIBILITY_COLUMN [PACKING_PROJ_MULTI_COLUMN_LIST_ROW]}.make (lot.wafer_etest_id, n))
				compatible_lots.forth
			end
			Result.extend (empty_project_column)
		end
			
	wafer_etest_id_column: WAFER_ETEST_ID_COLUMN [LOT_SUMMARY_MULTI_COLUMN_LIST_ROW]
		do
			create Result
		end

	required_count_column: REQUIRED_COUNT_COLUMN [LOT_SUMMARY_MULTI_COLUMN_LIST_ROW]
		do
			create Result
		end

	incompatible_count_column: INCOMPATIBLE_COUNT_COLUMN [LOT_SUMMARY_MULTI_COLUMN_LIST_ROW]
		do
			create Result
		end

	assigned_count_column: ASSIGNED_COUNT_COLUMN [LOT_SUMMARY_MULTI_COLUMN_LIST_ROW]
		do
			create Result
		end

	assignable_count_column: ASSIGNABLE_COUNT_COLUMN [LOT_SUMMARY_MULTI_COLUMN_LIST_ROW]
		do
			create Result
		end

	total_assigned_count_column: TOTAL_ASSIGNED_COUNT_COLUMN [LOT_SUMMARY_MULTI_COLUMN_LIST_ROW]
		do
			create Result
		end

	total_count_column: TOTAL_COUNT_COLUMN [LOT_SUMMARY_MULTI_COLUMN_LIST_ROW]
		do
			create Result
		end

	empty_lot_column: EMPTY_COLUMN [LOT_SUMMARY_MULTI_COLUMN_LIST_ROW]
		do
			create Result
		end

	design_no_column: DESIGN_NO_COLUMN [PACKING_PROJ_MULTI_COLUMN_LIST_ROW]
		do
			create Result
		end

	is_lot_assigned_column: IS_LOT_ASSIGNED_COLUMN [PACKING_PROJ_MULTI_COLUMN_LIST_ROW]
		do
			create Result
		end

	empty_project_column: EMPTY_COLUMN [PACKING_PROJ_MULTI_COLUMN_LIST_ROW]
		do
			create Result
		end

	refresh_button: EV_BUTTON
			-- Button to refresh lot summary information
	
	send_button: EV_BUTTON
			-- Button to send lot summary information from
			-- current notebook tab to observers
	
	cancel_button: EV_BUTTON
			-- Button to cancel lot summary window display
	
	projects: LIST [PACKING_DIES_MROW]
			-- List of distinct dies for current customer
			-- projects

	compatible_lots: LIST [WAFER_ETEST_NAMES_ROW]
			-- List of lots with which every project in
			-- `projects' is compatible
	
	packer_compatible_lots: MULTI_COLUMN_LIST [LOT_SUMMARY_MULTI_COLUMN_LIST_ROW]
			-- List of lots (one per wafer etest name) with
			-- compatilibility counts
	
	packer_assigned_lots: MULTI_COLUMN_LIST [LOT_SUMMARY_MULTI_COLUMN_LIST_ROW]
			-- List of lots (one per wafer etest name) with
			-- counts of projects assigned to each lot
	
	packer_projects: MULTI_COLUMN_LIST [PACKING_PROJ_MULTI_COLUMN_LIST_ROW]
			-- List of projects with one column for each possible
			-- wafer lot showing whether the project must be,
			-- must not be, could be but is not or could be but
			-- is assigned to that lot
	
	required_count: INTEGER
			-- Number of projects that required wafer etest ID
			-- for last call to `compute_counts'
	
	forbidden_count: INTEGER
			-- Number of projects that were incompatible
			-- with wafer etest ID for last call to
			-- `compute_counts'
	
	assigned_count: INTEGER
			-- Number of projects that were assigned wafer
			-- etest ID but had other choices, for last
			-- call to `compute_counts'
	
	assignable_count: INTEGER
			-- Number of projects that could have been assigned
			-- wafer etest ID but are currently not
	
	total_count: INTEGER
			-- Total number of projects
	
	notebook: EV_NOTEBOOK
			-- Notebook to hold various kinds of lot summary
			-- information
	
	Only_status: STRING = "Only"
			-- Status indicating project can only go on this lot
	
	Incompatible_status: STRING = "Incompatible"
			-- Status indicating project cannot go on this lot
	
	Assigned_status: STRING = "Selected"
			-- Status indicating project is compatible with more
			-- than one lot but has been assigned to this one
	
	allowed_status: STRING = "Allowed"
			-- Status indicating project is compatible with this
			-- lot and at least one other lot but has not been
			-- assigned to this lot
	
	compatibility_overview_text: STRING = "Compatibility Overview"
			-- Label for compatibility overview notebook tab
	
	lot_assignments_text: STRING = "Lot Assignments"
			-- Label for lot assignments notebook tab
	
	project_lot_matrix_text: STRING = "Project/Lot Matrix"
			-- Label for project lot matrix notebook tab
	
end
