
indexing
	
	description: "Window to allow specification of field and location %
		%for absolute move operation in packer"
	author: "David Hollenberg"
	date: "November 7, 2005"

class RC_PACKING_MOVE_ABS_WINDOW

inherit
	RC_PACKING_MOVE_ABS_IFC
		undefine
			default_create
		redefine
			set_element,
			set_fields
		end
	WINDOW_INTERFACE
		redefine
			enable_active,
			disable_active
		end
	MOSIS_GRAPHICAL_UTILITIES
		export
			{NONE} all
		undefine
			default_create
		end

create
	make

feature {NONE} -- Creation

	make (w: EV_TITLED_WINDOW)
			-- Create interface in window `w'
		require
			window_exists: w /= Void
			window_valid: not w.is_destroyed
		do
			make_interface
			make_proxy (w)
		end

feature -- Modification

	set_element (an_element: like element)
			-- Set `element' to `an_element'
		do
			precursor (an_element)
			set_element_odd_even
		end

	set_fields (a_list: like fields)
			-- Set `fields' to `a_list'
		do
			precursor (a_list)
			set_field_odd_even
		end

feature -- Operations

	reset
			-- Move focus back into window
		do
			enable_active
		end

feature -- Modification

	enable_active
			-- Make interface active
		do
			precursor
			ok_button.enable_sensitive
			cancel_button.enable_sensitive
		end

	disable_active
			-- Make interface inactive
			-- (can look but cannot initiate most actions)
		do
			precursor
			ok_button.disable_sensitive
			cancel_button.disable_sensitive
		end

feature {NONE} -- Implementation

	build_interface
			-- Build interface
		local
			cols: WIDGET_COLUMN_SET
			buttons: HORIZONTAL_BUTTON_SET
			main_vbox: EV_VERTICAL_BOX
			box: EV_BOX
			label: EV_LABEL
			hbox: EV_HORIZONTAL_BOX
		do
			create main_vbox
			main_vbox.set_padding (user_interface.padding)
			main_vbox.set_border_width (user_interface.border_width)
			create cols.make (2)
			cols.set_padding (user_interface.padding)
			
			build_field_list
			cols.append_row (<< gui.label_with_text ("Field Name"), field_name_field >>)

			create location_x_field.make_with_text ("")
			location_x_field.return_actions.extend (agent process_ok)
			cols.append_row (<< gui.label_with_text ("Absolute Center X"), location_x_field >>)
			
			create location_y_field.make_with_text ("")
			location_y_field.return_actions.extend (agent process_ok)
			cols.append_row (<< gui.label_with_text ("Absolute Center Y"), location_y_field >>)
			
			cols.disable_column_expand (1)
			cols.align_column_right (1)
			box := cols.box
			
			main_vbox.extend (box)
			main_vbox.disable_item_expand (box)
			
			create label.make_with_text ("Center resolution is " + grid.center_resolution_text + " microns")
			create hbox
			hbox.extend (label)
			hbox.disable_item_expand (label)
			main_vbox.extend (hbox)
			main_vbox.disable_item_expand (hbox)
			
			create buttons.make_with_text_and_actions (<<
				"OK", "Cancel" >>,
				<< agent process_ok, agent process_cancel >>)
			ok_button := buttons.i_th (1)
			cancel_button := buttons.i_th (2)
			
			box := buttons.column_set.box
			
			main_vbox.extend (box)
			main_vbox.disable_item_expand (box)
			
			gui.set_box_background_recursive (main_vbox, background_color)
			extend (main_vbox)
			close_request_actions.extend (agent process_cancel)
			gui.add_cancel_accelerator (window, agent process_cancel)
			gui.do_once_on_idle (agent post_display_actions)
		end
		
	set_element_odd_even
			-- Set indicators that tell whether size of `element'
			-- is odd or even number of grid units in X and Y
		local
			x, y: DOUBLE
		do
			x := element.size_x
			y := element.size_y
			is_element_even_in_x := grid.rounded_size_units (x) \\ 2 = 0
			is_element_even_in_y := grid.rounded_size_units (y) \\ 2 = 0
		end

	set_field_odd_even
			-- Set indicators that tell whether size of
			-- first element of `fields' is odd or even
			-- number of grid units in X and Y
		local
			x, y: DOUBLE
		do
			x := fields.first.size_x
			y := fields.first.size_y
			is_field_even_in_x := grid.rounded_size_units (x) \\ 2 = 0
			is_field_even_in_y := grid.rounded_size_units (y) \\ 2 = 0
		end

	build_field_list
			-- Build combo box with list of fields and
			-- put in `field_name_field'
		local
			fact: SINGLE_CONSTRAINED_VALUE_FACTORY
			l_field: RETICLE_FIELD
		do
			create fact
			from
				fields.start
			until
				fields.after
			loop
				l_field := fields.item
				fact.add_choice_and_text (l_field.name, l_field.name)
				fields.forth
			end
			fact.set_default_choice (fields.first.name)
			fact.generate_combo_box
			field_name_field := fact.combo_box
		end
		
	post_display_actions
		do
			reset
		end

	process_ok
		local
			ok: BOOLEAN
			x_string, y_string: STRING
			l_x, l_y: DOUBLE
		do
			if is_active then
				disable_active
				x_string := location_x_field.text
				y_string := location_y_field.text
				create last_message.make (50)
				ok := True
				check_dimension ("Absolute center X", x_string, max_db_value, True)
				ok := ok and is_valid
				l_x := last_dimension
				check_dimension ("Absolute center Y", y_string, max_db_value, False)
				ok := ok and is_valid
				l_y := last_dimension
				if ok then
					accept_actions.call ([field_name_field.text.to_string_8, l_x, l_y])
				else
					inform (last_message)
				end
				enable_active
			end
		end

	check_dimension (a_dimension, a_value: STRING a_max: DOUBLE is_x: BOOLEAN)
			-- Check that `a_dimension' with `a_value'
			-- whose maximum value is `a_max' is valid.
			-- Append error to `last_message' if not
		require
			a_dimension_not_void: a_dimension /= Void
			a_value_not_void: a_value /= Void
		local
			val: DOUBLE
		do
			is_valid := False
			if not a_value.is_double then
				last_message.append (a_dimension + " " + su.double_quoted (a_value) + " is not a valid real number%N")
			else
				val := a_value.to_double
				if val > a_max or val < -a_max then
					last_message.append (a_dimension + " (" + a_value + ") has absolute value greater than maximum allowed value (" + grid.formatted_number (a_max) + ")%N")
				elseif not grid.is_center_on_grid (val) then
					last_message.append (a_dimension + " (" + a_value + ") is not a multiple of center resolution " + grid.center_resolution_text + "%N")
				elseif is_x and ((is_field_even_in_x = is_element_even_in_x) /= grid.is_size_on_grid (val)) or not is_x and ((is_field_even_in_y = is_element_even_in_y) /= grid.is_size_on_grid (val)) then
					last_message.append (a_dimension + " (" + a_value + ") must be changed by 1/2 grid unit (" + grid.center_resolution_text + ") to put element edges on grid%N")
				else
					is_valid := True
					last_dimension := grid.rounded_center (val)
				end
			end
		end

	process_cancel
		do
			if is_active then
				disable_active
				cancel_actions.call ([])
			end
		end

	field_name_field: EV_COMBO_BOX
			-- Field with list of field names
	
	location_x_field: EV_TEXT_FIELD
			-- Field to set absolute location in X dimension
	
	location_y_field: EV_TEXT_FIELD
			-- Field to set absolute location in Y dimension
	
	ok_button: EV_BUTTON
			-- Button to accept values
	
	cancel_button: EV_BUTTON
			-- Button to close window
	
	is_element_even_in_x: BOOLEAN
			-- Is element size in X an even number of grid units?
	
	is_element_even_in_y: BOOLEAN
			-- Is element size in Y an even number of grid units?
	
	is_field_even_in_x: BOOLEAN
			-- Is field size in X an even number of grid units?
	
	is_field_even_in_y: BOOLEAN
			-- Is field size in Y an even number of grid units?
	
	is_valid: BOOLEAN
			-- Was last value passed to `check_dimension' valid?
	
	last_message: STRING
			-- Error message
	
	last_dimension: DOUBLE
			-- Last valid dimension found by `check_dimension'
	
end
