
indexing
	
	description: "Window to allow specification of location %
		%for relative move operation in packer"
	author: "David Hollenberg"
	date: "November 7, 2005"

class RC_PACKING_MOVE_REL_WINDOW

inherit
	RC_PACKING_MOVE_REL_IFC
		undefine
			default_create
		end
	WINDOW_INTERFACE
		redefine
			enable_active,
			disable_active
		end
	MOSIS_GRAPHICAL_UTILITIES
		export
			{NONE} all
		undefine
			default_create
		end

create
	make

feature {NONE} -- Creation

	make (w: EV_TITLED_WINDOW)
			-- Create interface in window `w'
		require
			window_exists: w /= Void
			window_valid: not w.is_destroyed
		do
			make_interface
			make_proxy (w)
		end

feature -- Operations

	reset
			-- Move focus back into window
		do
			enable_active
		end

feature -- Modification

	enable_active
			-- Make interface active
		do
			precursor
			ok_button.enable_sensitive
			cancel_button.enable_sensitive
		end

	disable_active
			-- Make interface inactive
			-- (can look but cannot initiate most actions)
		do
			precursor
			ok_button.disable_sensitive
			cancel_button.disable_sensitive
		end

feature {NONE} -- Implementation

	build_interface
			-- Build interface
		local
			cols: WIDGET_COLUMN_SET
			buttons: HORIZONTAL_BUTTON_SET
			main_vbox: EV_VERTICAL_BOX
			box: EV_BOX
			label: EV_LABEL
			hbox: EV_HORIZONTAL_BOX
		do
			create main_vbox
			main_vbox.set_padding (user_interface.padding)
			main_vbox.set_border_width (user_interface.border_width)
			create cols.make (2)
			cols.set_padding (user_interface.padding)
			
			create distance_x_field.make_with_text ("")
			distance_x_field.return_actions.extend (agent process_ok)
			cols.append_row (<< gui.label_with_text ("Distance X"), distance_x_field >>)
			
			create distance_y_field.make_with_text ("")
			distance_y_field.return_actions.extend (agent process_ok)
			cols.append_row (<< gui.label_with_text ("Distance Y"), distance_y_field >>)
			
			cols.disable_column_expand (1)
			cols.align_column_right (1)
			box := cols.box
			
			main_vbox.extend (box)
			main_vbox.disable_item_expand (box)
			
			create label.make_with_text ("Distance resolution is " + grid.edge_resolution_text + " microns")
			create hbox
			hbox.extend (label)
			hbox.disable_item_expand (label)
			main_vbox.extend (hbox)
			main_vbox.disable_item_expand (hbox)
			
			create buttons.make_with_text_and_actions (<<
				"OK", "Cancel" >>,
				<< agent process_ok, agent process_cancel >>)
			ok_button := buttons.i_th (1)
			cancel_button := buttons.i_th (2)
			
			box := buttons.column_set.box
			
			main_vbox.extend (box)
			main_vbox.disable_item_expand (box)
			
			gui.set_box_background_recursive (main_vbox, background_color)
			extend (main_vbox)
			close_request_actions.extend (agent process_cancel)
			gui.add_cancel_accelerator (window, agent process_cancel)
			gui.do_once_on_idle (agent post_display_actions)
		end
		
	post_display_actions
		do
			reset
		end

	process_ok
		local
			ok: BOOLEAN
			x_string, y_string: STRING
			l_x, l_y: DOUBLE
		do
			if is_active then
				disable_active
				x_string := distance_x_field.text
				y_string := distance_y_field.text
				create last_message.make (50)
				ok := True
				check_dimension ("Distance X", x_string)
				ok := ok and is_valid
				l_x := last_dimension
				check_dimension ("Distance Y", y_string)
				ok := ok and is_valid
				l_y := last_dimension
				if ok then
					accept_actions.call ([l_x, l_y])
				else
					inform (last_message)
				end
				enable_active
			end
		end

	check_dimension (a_dimension, a_value: STRING)
			-- Check that `a_dimension' with `a_value'
			-- is valid.  Append error to `last_message' if not
		require
			a_dimension_not_void: a_dimension /= Void
			a_value_not_void: a_value /= Void
		local
			val: DOUBLE
		do
			is_valid := False
			if not a_value.is_double then
				last_message.append (a_dimension + " " + su.double_quoted (a_value) + " is not a valid real number%N")
			else
				val := a_value.to_double
				if not grid.is_edge_on_grid (val) then
					last_message.append (a_dimension + " (" + a_value + ") is not a multiple of resolution " + grid.edge_resolution_text + "%N")
				else
					is_valid := True
					last_dimension := grid.rounded_edge (val)
				end
			end
		end

	process_cancel
		do
			if is_active then
				disable_active
				cancel_actions.call ([])
			end
		end

	distance_x_field: EV_TEXT_FIELD
			-- Field to set relative distance in X dimension
	
	distance_y_field: EV_TEXT_FIELD
			-- Field to set relative distance in Y dimension
	
	ok_button: EV_BUTTON
			-- Button to accept values
	
	cancel_button: EV_BUTTON
			-- Button to close window
	
	is_valid: BOOLEAN
			-- Was last value passed to `check_dimension' valid?
	
	last_message: STRING
			-- Error message
	
	last_dimension: DOUBLE
			-- Last valid dimension found by `check_dimension'
	
end
