
indexing
	
	description: "Window to display output from packer operations"
	author: "David Hollenberg"
	date: "November 7, 2005"

class RC_PACKING_OUTPUT_WINDOW

inherit
	RC_PACKING_OUTPUT_IFC
	WINDOW_INTERFACE
		redefine
			enable_active,
			disable_active
		end
	MOSIS_GRAPHICAL_UTILITIES
		export
			{NONE} all
		undefine
			default_create
		end

create
	make

feature {NONE} -- Creation

	make (w: EV_TITLED_WINDOW)
			-- Create interface in window `w'
		require
			window_exists: w /= Void
			window_valid: not w.is_destroyed
		do
			make_interface
			make_proxy (w)
			create recent_keys.make (200)
			create font
			create string_size_and_font.make ("", font)
			keys_text_width := string_width (Keys_text)
		end

feature -- Operations

	reset
			-- Move focus back into window
		do
			enable_active
		end

	process_key_press (a_key: MODIFIED_KEY)
			-- Take actions needed when `a_key' is pressed
		do
			recent_keys.append (a_key.description)
			recent_keys.extend (' ')
			adjust_recent_keys
			keys_label.set_text (Keys_text + recent_keys)
		end

	process_info_added (a_text: STRING)
			-- Take actions needed when informational output
			-- `a_text' is generated
		do
			text_area.append_text (a_text)
			text_area.append_text (Newline)
			text_area.scroll_to_end
		end

feature -- Modification

	enable_active
			-- Make interface active
		do
			precursor
		end

	disable_active
			-- Make interface inactive
			-- (can look but cannot initiate most actions)
		do
			precursor
		end

feature {NONE} -- Implementation

	build_interface
			-- Build interface
		local
			main_vbox, status_box: EV_VERTICAL_BOX
			status_line_box: EV_HORIZONTAL_BOX
		do
			create main_vbox
			main_vbox.set_padding (user_interface.padding)
			main_vbox.set_border_width (5)
			create text_area
			text_area.disable_edit
			text_area.disable_word_wrapping
			main_vbox.extend (text_area)
			
			create keys_label.make_with_text (Keys_text)
			keys_label.align_text_left
			keys_label.set_minimum_width (100)

			create status_line_box
			status_line_box.extend (keys_label)
			
			create status_box
			status_box.extend (create {EV_HORIZONTAL_SEPARATOR})
			status_box.extend (status_line_box)
			status_box.extend (create {EV_HORIZONTAL_SEPARATOR})

			lower_bar.extend (status_box)

			gui.set_box_background_recursive (main_vbox, background_color)
			extend (main_vbox)
			close_request_actions.extend (agent process_cancel)
			gui.add_cancel_accelerator (window, agent process_cancel)
			gui.do_once_on_idle (agent post_display_actions)
		end
		
	post_display_actions
		do
			reset
			gui.do_once_on_idle (agent position_window)
		end

	position_window
		do
			if not is_destroyed then
				gui.abut_window_bottom_left_with_size (window, 800, 200)
			end
		end

	process_cancel
		do
			if is_active then
				disable_active
				cancel_actions.call ([])
			end
		end

	adjust_recent_keys
			-- Adjust `recent_keys' to make it fit in
			-- `keys_label' by removing leading "words"
			-- (where words are separated by spaces)
		local
			max_width, pos: INTEGER
			done: BOOLEAN
		do
			max_width := keys_label.width - Keys_text_width - Keys_right_margin
			from
			until
				string_width (recent_keys) <= max_width or done
			loop
				pos := recent_keys.index_of (' ', 1)
				if pos > 0 then
					recent_keys.keep_tail (recent_keys.count - pos)
				else
					done := False
				end
			end
		end

	string_width (s: STRING): INTEGER
			-- Width of `s' in pixels
		do
			string_size_and_font.update (s)
			Result := string_size_and_font.width
		end

	text_area: EV_TEXT
			-- Area where packer output is displayed

	keys_label: EV_LABEL
			-- Label to display recent keystrokes that
			-- were directed to drawing area

	recent_keys: STRING
			-- Recent keystrokes that were directed to
			-- drawing area

	Keys_text: STRING = "Keys:  "
			-- Text for line with recent keystrokes that
			-- were directed to drawing area

	Keys_text_width: INTEGER
			-- Width of `keys_text' in default font

	Keys_right_margin: INTEGER = 20
			-- Number of pixels of white space at end of
			-- recent keys

	font: EV_FONT
			-- Default font

	string_size_and_font: STRING_SIZE_AND_FONT
			-- For text width calculations

	Newline: STRING = "%N"

end
