
indexing
	
	description: "Window to manage projects during packing"
	author: "David Hollenberg"
	date: "November 7, 2005"

class RC_PACKING_PROJECTS_WINDOW

inherit
	RC_PACKING_PROJECTS_IFC
	WINDOW_INTERFACE
		redefine
			enable_active,
			disable_active
		end
	MOSIS_GRAPHICAL_UTILITIES
		export
			{NONE} all
		undefine
			default_create
		end
	UNIX_OS_ACCESS
		export
			{NONE} all
		undefine
			default_create
		end
		
create
	make

feature {NONE} -- Creation

	make (w: EV_TITLED_WINDOW)
			-- Create interface in window `w'
		require
			window_exists: w /= Void
			window_valid: not w.is_destroyed
		do
			make_interface
			make_proxy (w)
			create projects_added_actions
			create projects_removed_actions
			create projects_updated_actions
		end

feature -- Properties

	projects_added_actions:  ACTION_SEQUENCE [TUPLE [LIST [PACKING_DIES_MROW]]]
			-- Actions to be performed when projects are
			-- added to packing

	projects_removed_actions:  ACTION_SEQUENCE [TUPLE [LIST [PACKING_DIES_MROW]]]
			-- Actions to be performed when projects are
			-- removed from packing

	projects_updated_actions:  ACTION_SEQUENCE [TUPLE [LIST [PACKING_DIES_MROW]]]
			-- Actions to be performed when projects are
			-- updated

	removed_dies: LIST [PACKING_DIES_MROW]
			-- Dies for projects that were deselected by
			-- last call to `deselect_project_list'

feature -- Operations

	enable_read_only
			-- Make window read-only (not able to add, remove,
			-- update, lock projects, etc)
		do
			selection_select_button.disable_sensitive
			deselection_deselect_button.disable_sensitive
			lock_lock_button.disable_sensitive
			unlock_unlock_button.disable_sensitive
			assignment_assign_button.disable_sensitive
			deassignment_deassign_button.disable_sensitive
			update_update_button.disable_sensitive
		end

	disable_read_only
			-- Make window allow changes (add, remove,
			-- update, lock projects, etc)
		do
			selection_select_button.enable_sensitive
			deselection_deselect_button.enable_sensitive
			lock_lock_button.enable_sensitive
			unlock_unlock_button.enable_sensitive
			assignment_assign_button.enable_sensitive
			deassignment_deassign_button.enable_sensitive
			update_update_button.enable_sensitive
		end

	reset
			-- Move focus back into window
		do
			enable_active
		end

	refresh_top_page
			-- Refresh contents of current topmost page
		local
			an_agent: PROCEDURE [ANY, TUPLE]
			old_style: EV_POINTER_STYLE
		do
			an_agent := current_tab_refresh_agent
			if an_agent /= Void then
				old_style := pointer_style
				set_pointer_style (gui.Wait_cursor)
				gui.process_events
				an_agent.call ([])
				set_pointer_style (old_style)
				gui.process_events
			end
		rescue
                        if old_style /= Void then
                                set_pointer_style (old_style)
                        end
		end

	deselect_project_list (projs: LIST [PACKING_PROJECTS_MROW] wants_unlock, wants_deassign: BOOLEAN)
			-- Confirm deselection of `projs' and then try
			-- to deselect them.  Display results of
			-- deselection attempt
		require
			projs_not_void: projs /= Void
		local
			dies, the_locked_dies: LIST [PACKING_DIES_MROW]
			l_projects: like projs
		do
			dies := current_version.dies_for_projects (projs)
			the_locked_dies := locked_dies (dies)
			confirm_delete_locked_dies (the_locked_dies)
			l_projects := current_version.projects_for_dies (unlocked_dies (dies))
			confirm_deselection (l_projects, wants_unlock, wants_deassign)
			try_to_deselect_projects (l_projects, wants_unlock, wants_deassign)
			if ok_count > 0 then
				unlockable_projects_changed := True
				lockable_projects_changed := True
				assignable_projects_changed := True
				deassignable_projects_changed := True
				status_projects_changed := True
				deselectable_projects_changed := True
				selection_projects_changed := True
				refresh_deselectable_rows
			end
			display_operation_results
		end

feature -- Modification

	subscribe_events
			-- Subscribe to events of interest
		do
			(create {PACKING_VERSION_CREATED_EVENT}).subscribe (agent handle_version_created)
			(create {PACKING_VERSION_SELECTED_EVENT}).subscribe (agent handle_version_selected)
		end

	unsubscribe_events
			-- Unsubscribe from events of interest
		do
			(create {PACKING_VERSION_CREATED_EVENT}).unsubscribe (agent handle_version_created)
			(create {PACKING_VERSION_SELECTED_EVENT}).unsubscribe (agent handle_version_selected)
		end

	enable_active
			-- Make interface active
		do
			precursor
		end

	disable_active
			-- Make interface inactive
			-- (can look but cannot initiate most actions)
		do
			precursor
		end

feature {NONE} -- Implementation

	build_interface
			-- Build interface
		local
			main_vbox: EV_VERTICAL_BOX
		do
			add_menu_bar

			create main_vbox
			main_vbox.set_padding (user_interface.padding)
			main_vbox.set_border_width (user_interface.border_width)
			build_notebook
			main_vbox.extend (notebook)

			gui.set_box_background_recursive (main_vbox, background_color)
			extend (main_vbox)
			close_request_actions.extend (agent process_close)
			show_actions.extend (agent refresh_top_page)
			restore_actions.extend (agent refresh_top_page)
			gui.add_cancel_accelerator (window, agent process_close)
			gui.do_once_on_idle (agent post_display_actions)
		end
		
	build_notebook
			-- Build notebook with tabs for status, select, etc
		do
			create notebook
			build_status_box
			notebook.extend (status_box)
			notebook.set_item_text (status_box, "Status")
			build_selection_box
			notebook.extend (selection_box)
			notebook.set_item_text (selection_box, "Add")
			build_deselection_box
			notebook.extend (deselection_box)
			notebook.set_item_text (deselection_box, "Remove")
			build_update_box
			notebook.extend (update_box)
			notebook.set_item_text (update_box, "Update")
			build_lock_box
			notebook.extend (lock_box)
			notebook.set_item_text (lock_box, "Lock")
			build_unlock_box
			notebook.extend (unlock_box)
			notebook.set_item_text (unlock_box, "Unlock")
			build_assignment_box
			notebook.extend (assignment_box)
			notebook.set_item_text (assignment_box, "Assign")
			build_deassignment_box
			notebook.extend (deassignment_box)
			notebook.set_item_text (deassignment_box, "Deassign")

			notebook.selection_actions.extend (agent refresh_top_page)
			if not has_run_context then
				assignment_box.disable_sensitive
				deassignment_box.disable_sensitive
			end
		end
		
	build_status_box
			-- Build box for Status tab of notebook
		local
			cols: WIDGET_COLUMN_SET
			box: EV_BOX
			hbox: EV_HORIZONTAL_BOX
			buttons: HORIZONTAL_BUTTON_SET
			the_data: PACKING_PROJ_NOTEBOOK_TAB_DATA
		do
			create status_box
			
			create cols.make (3)
			cols.set_padding (user_interface.padding)
			cols.set_border_width (Std_border_width)
			create status_cust_die_button.make_with_text ("Customer projects")
			create status_test_vehicle_button.make_with_text ("Test vehicles")
			status_cust_die_button.enable_select
			status_test_vehicle_button.enable_select
			cols.append_row (<< gui.label_with_text ("Filter By Type:"),  status_cust_die_button, status_test_vehicle_button >>)
			cols.disable_expand
			cols.align_left
			box := cols.box
			status_box.extend (box)
			status_box.disable_item_expand (box)
			
			create cols.make (3)
			cols.set_padding (user_interface.padding)
			cols.set_border_width (Std_border_width)
			create status_included_button.make_with_text ("Included in packing")
			create status_not_included_button.make_with_text ("Removed from packing but previously included")
			status_included_button.enable_select
			cols.append_row (<< gui.label_with_text ("Filter By Status:"),  status_included_button, status_not_included_button >>)
			cols.disable_expand
			cols.align_left
			box := cols.box
			status_box.extend (box)
			status_box.disable_item_expand (box)
			
			create buttons.make_with_text_and_actions (<<
				"Filter", "Send", "Close" >>,
				<< agent refresh_status, agent process_send, agent process_close >>)
			status_filter_button := buttons.i_th (1)
			status_send_button := buttons.i_th (2)
			status_close_button := buttons.i_th (3)
			status_filter_button.set_tooltip ("Display only items that meet the filter criteria")
			status_send_button.set_tooltip ("Send list of currently displayed projects to Output window")
			box := buttons.column_set.box
			status_box.extend (box)
			status_box.disable_item_expand (box)
			
			create status_label
			status_label.set_font (list_title_font)
			create hbox
			hbox.set_border_width (Std_border_width)
			hbox.extend (status_label)
			status_box.extend (hbox)
			status_box.disable_item_expand (hbox)
			
			status_projects := new_multi_column_list
			status_projects.disable_multiple_selection
			status_projects.set_visible_columns (full_status_columns)
			status_box.extend (status_projects)
			box := separated_box (status_projects.selection_status_box)
			status_box.extend (box)
			status_box.disable_item_expand (box)
			
			create the_data
			the_data.set_category ("included")
			the_data.set_refresh_agent (agent refresh_status_rows)
			the_data.set_selection_list (status_projects)
			the_data.add_list_formatter (Basic_column_format, status_projects, agent basic_standard_columns)
			the_data.add_list_formatter (Contact_column_format, status_projects, agent contact_standard_columns)
			the_data.add_list_formatter (Full_column_format, status_projects, agent full_status_columns)
			status_box.set_data (the_data)
			status_projects_changed := True
		end
		
	build_selection_box
			-- Build box for Select tab of notebook
		local
			cols: WIDGET_COLUMN_SET
			box: EV_BOX
			hbox: EV_HORIZONTAL_BOX
			buttons: HORIZONTAL_BUTTON_SET
			sel_box, unsel_box: EV_VERTICAL_BOX
			label: EV_LABEL
			the_data: PACKING_PROJ_NOTEBOOK_TAB_DATA
		do
			create selection_box
			
			create cols.make (3)
			cols.set_padding (user_interface.padding)
			cols.set_border_width (Std_border_width)
			create selection_cust_die_button.make_with_text ("Customer projects")
			create selection_test_vehicle_button.make_with_text ("Test vehicles")
			selection_cust_die_button.enable_select
			selection_test_vehicle_button.enable_select
			cols.append_row (<< gui.label_with_text ("Filter By Type:"),  selection_cust_die_button, selection_test_vehicle_button >>)
			cols.disable_expand
			cols.align_left
			box := cols.box
			selection_box.extend (box)
			selection_box.disable_item_expand (box)
			
			create cols.make (3)
			cols.set_padding (user_interface.padding)
			cols.set_border_width (Std_border_width)
			create selection_lock_button.make_with_text ("Lock projects")
			create selection_assign_button.make_with_text ("Assign projects to run")
			selection_lock_button.enable_select
			if has_run_context then
				selection_assign_button.enable_select
			else
				selection_assign_button.disable_select
				selection_assign_button.disable_sensitive
			end
			selection_lock_button.select_actions.extend (agent process_select_lock_change)
			selection_assign_button.select_actions.extend (agent process_select_assign_change)
			label := gui.label_with_text ("Options:")
			cols.append_row (<< label,  selection_lock_button, selection_assign_button >>)
			cols.disable_expand
			cols.align_left
			box := cols.box
			selection_box.extend (box)
			selection_box.disable_item_expand (box)
			
			create cols.make (7)
			cols.set_padding (user_interface.padding)
			cols.set_border_width (Std_border_width)
			create selection_bad_run_type_button.make_with_text ("Wrong run type")
			create selection_wrong_foundry_button.make_with_text ("Wrong foundry")
			create selection_no_access_button.make_with_text ("Access not granted")
			create selection_already_assigned_button.make_with_text ("Assigned to another run")
			create selection_not_charged_button.make_with_text ("Not charged")
			create selection_incompat_test_vehicle_button.make_with_text ("Incompatible test vehicles")
			if not has_run_context then
				selection_already_assigned_button.disable_select
				selection_already_assigned_button.disable_sensitive
			end
			label := gui.label_with_text ("Make Available:")
			cols.append_row (<< label,  selection_bad_run_type_button, selection_wrong_foundry_button, selection_no_access_button, selection_already_assigned_button, selection_not_charged_button, selection_incompat_test_vehicle_button >>)
			cols.disable_expand
			cols.align_left
			box := cols.box
			selection_box.extend (box)
			selection_box.disable_item_expand (box)
			
			create buttons.make_with_text_and_actions (<<
				"Add", "Filter", "Refresh", "Send", "Close" >>,
				<< agent select_projects, agent process_selection_filter, agent refresh_selection, agent process_send, agent process_close >>)
			selection_select_button := buttons.i_th (1)
			selection_filter_button := buttons.i_th (2)
			selection_refresh_button := buttons.i_th (3)
			selection_send_button := buttons.i_th (4)
			if is_read_only then
				selection_select_button.disable_sensitive
			end
			selection_select_button.set_tooltip ("Add the projects selected in the Available Projects list to packing and modify their status as specified by Options")
			selection_filter_button.set_tooltip ("Display only items that meet the filter criteria")
			selection_refresh_button.set_tooltip ("Refresh lists of Available and Unavailable Projects by re-querying fab queue.  Applies filter conditions too.")
			selection_send_button.set_tooltip ("Send list of currently displayed projects to Output window")
			box := buttons.column_set.box
			selection_box.extend (box)
			selection_box.disable_item_expand (box)
			
			selectable_projects := new_multi_column_list
			selectable_projects.set_visible_columns (full_selectable_columns)
			
			create selectable_label
			selectable_label.set_font (list_title_font)
			create hbox
			hbox.set_border_width (Std_border_width)
			hbox.extend (selectable_label)

			create sel_box
			sel_box.extend (hbox)
			sel_box.disable_item_expand (hbox)
			sel_box.extend (selectable_projects)
			box := separated_box (selectable_projects.selection_status_box)
			sel_box.extend (box)
			sel_box.disable_item_expand (box)
			
			unselectable_projects := new_multi_column_list
			unselectable_projects.set_visible_columns (full_unselectable_columns)
			
			create unselectable_label
			unselectable_label.set_font (list_title_font)
			create hbox
			hbox.set_border_width (Std_border_width)
			hbox.extend (unselectable_label)

			create unsel_box
			unsel_box.extend (hbox)
			unsel_box.disable_item_expand (hbox)
			unsel_box.extend (unselectable_projects)
			box := separated_box (unselectable_projects.selection_status_box)
			unsel_box.extend (box)
			unsel_box.disable_item_expand (box)
			
			
			create selection_split_area
			selection_split_area.set_first (sel_box)
			selection_split_area.set_second (unsel_box)
			
			selection_box.extend (selection_split_area)
			
			create the_data
			the_data.set_category ("selectable")
			the_data.set_refresh_agent (agent refresh_selection_rows (True))
			the_data.set_selection_list (selectable_projects)
			the_data.add_list_formatter (Basic_column_format, selectable_projects, agent basic_standard_columns)
			the_data.add_list_formatter (Basic_column_format, unselectable_projects, agent basic_standard_columns)
			the_data.add_list_formatter (Contact_column_format, selectable_projects, agent contact_standard_columns)
			the_data.add_list_formatter (Contact_column_format, unselectable_projects, agent contact_standard_columns)
			the_data.add_list_formatter (Full_column_format, selectable_projects, agent full_selectable_columns)
			the_data.add_list_formatter (Full_column_format, unselectable_projects, agent full_unselectable_columns)
			selection_box.set_data (the_data)
			selection_projects_changed := True
		end
		
	build_deselection_box
			-- Build box for Deselect tab of notebook
		local
			cols: WIDGET_COLUMN_SET
			box: EV_BOX
			hbox: EV_HORIZONTAL_BOX
			buttons: HORIZONTAL_BUTTON_SET
			label: EV_LABEL
			the_data: PACKING_PROJ_NOTEBOOK_TAB_DATA
		do
			create deselection_box
			
			create cols.make (3)
			cols.set_padding (user_interface.padding)
			cols.set_border_width (Std_border_width)
			create deselection_cust_die_button.make_with_text ("Customer projects")
			create deselection_test_vehicle_button.make_with_text ("Test vehicles")
			deselection_cust_die_button.enable_select
			deselection_test_vehicle_button.enable_select
			cols.append_row (<< gui.label_with_text ("Filter By Type:"),  deselection_cust_die_button, deselection_test_vehicle_button >>)
			cols.disable_expand
			cols.align_left
			box := cols.box
			deselection_box.extend (box)
			deselection_box.disable_item_expand (box)
			
			create cols.make (3)
			cols.set_padding (user_interface.padding)
			cols.set_border_width (Std_border_width)
			create deselection_unlock_button.make_with_text ("Unlock projects")
			create deselection_deassign_button.make_with_text ("Deassign projects from run")
			deselection_unlock_button.enable_select
			deselection_deassign_button.enable_select
			if not has_run_context then
				deselection_deassign_button.disable_sensitive
			end
			deselection_unlock_button.select_actions.extend (agent process_deselect_unlock_change)
			deselection_deassign_button.select_actions.extend (agent process_deselect_deassign_change)
			label := gui.label_with_text ("Options:")
			cols.append_row (<< label,  deselection_unlock_button, deselection_deassign_button >>)
			cols.disable_expand
			cols.align_left
			box := cols.box
			deselection_box.extend (box)
			deselection_box.disable_item_expand (box)
			
			create buttons.make_with_text_and_actions (<<
				"Remove", "Filter", "Send", "Close" >>,
				<< agent deselect_projects, agent refresh_deselection, agent process_send, agent process_close >>)
			deselection_deselect_button := buttons.i_th (1)
			deselection_filter_button := buttons.i_th (2)
			deselection_send_button := buttons.i_th (3)
			deselection_close_button := buttons.i_th (4)
			if is_read_only then
				deselection_deselect_button.disable_sensitive
			end
			deselection_deselect_button.set_tooltip ("Remove the projects selected in Currently Included Projects from packing and modify their status as specified by Options")
			deselection_filter_button.set_tooltip ("Display only items that meet the filter criteria")
			deselection_send_button.set_tooltip ("Send list of currently displayed projects to Output window")
			box := buttons.column_set.box
			deselection_box.extend (box)
			deselection_box.disable_item_expand (box)
			
			create deselectable_label
			deselectable_label.set_font (list_title_font)
			create hbox
			hbox.set_border_width (Std_border_width)
			hbox.extend (deselectable_label)
			deselection_box.extend (hbox)
			deselection_box.disable_item_expand (hbox)
			
			deselectable_projects := new_multi_column_list
			deselectable_projects.set_visible_columns (full_standard_columns)
			deselection_box.extend (deselectable_projects)
			box := separated_box (deselectable_projects.selection_status_box)
			deselection_box.extend (box)
			deselection_box.disable_item_expand (box)

			create the_data
			the_data.set_category ("deselectable")
			the_data.set_refresh_agent (agent refresh_deselectable_rows)
			the_data.set_selection_list (deselectable_projects)
			the_data.add_list_formatter (Basic_column_format, deselectable_projects, agent basic_standard_columns)
			the_data.add_list_formatter (Contact_column_format, deselectable_projects, agent contact_standard_columns)
			the_data.add_list_formatter (Full_column_format, deselectable_projects, agent full_standard_columns)
			deselection_box.set_data (the_data)
			deselectable_projects_changed := True
		end
		
	build_lock_box
			-- Build box for Lock tab of notebook
		local
			cols: WIDGET_COLUMN_SET
			box: EV_BOX
			buttons: HORIZONTAL_BUTTON_SET
			label: EV_LABEL
			the_data: PACKING_PROJ_NOTEBOOK_TAB_DATA
		do
			create lock_box
			
			create cols.make (2)
			cols.set_padding (user_interface.padding)
			cols.set_border_width (Std_border_width)
			create lock_assign_button.make_with_text ("Assign projects to run")
			if has_run_context then
				lock_assign_button.enable_select
			else
				lock_assign_button.disable_select
				lock_assign_button.disable_sensitive
			end
			label := gui.label_with_text ("Options:")
			cols.append_row (<< label,  lock_assign_button >>)
			cols.disable_expand
			cols.align_left
			box := cols.box
			lock_box.extend (box)
			lock_box.disable_item_expand (box)
			
			create buttons.make_with_text_and_actions (<<
				"Lock", "Refresh", "Send", "Close" >>,
				<< agent lock_projects, agent refresh_locks, agent process_send, agent process_close >>)
			lock_lock_button := buttons.i_th (1)
			lock_refresh_button := buttons.i_th (2)
			lock_send_button := buttons.i_th (3)
			if is_read_only then
				lock_lock_button.disable_sensitive
			end
			lock_lock_button.set_tooltip ("Lock the projects selected in the Included But Unlocked Projects list, preventing subsequent customer update and cancel operations, and modify their status as specified by Options.")
			lock_refresh_button.set_tooltip ("Refresh list of Included But Unlocked Projects")
			lock_send_button.set_tooltip ("Send list of currently displayed projects to Output window")
			box := buttons.column_set.box
			lock_box.extend (box)
			lock_box.disable_item_expand (box)
			
			lockable_projects := new_multi_column_list
			lockable_projects.set_visible_columns (full_standard_columns)
			lockable_projects.sort_on_column (Design_no_position)
			
			create cols.make (1)
			cols.set_padding (user_interface.padding)
			cols.set_border_width (Std_border_width)
			label := gui.label_with_text ("Included But Unlocked Projects")
			label.set_font (list_title_font)
			cols.append_row (<< label >>)
			cols.align_center
			box := cols.box
			
			lock_box.extend (box)
			lock_box.disable_item_expand (box)
			lock_box.extend (lockable_projects)
			box := separated_box (lockable_projects.selection_status_box)
			lock_box.extend (box)
			lock_box.disable_item_expand (box)
			
			create the_data
			the_data.set_category ("lockable")
			the_data.set_refresh_agent (agent refresh_lockable_rows)
			the_data.set_selection_list (lockable_projects)
			the_data.add_list_formatter (Basic_column_format, lockable_projects, agent basic_standard_columns)
			the_data.add_list_formatter (Contact_column_format, lockable_projects, agent contact_standard_columns)
			the_data.add_list_formatter (Full_column_format, lockable_projects, agent full_standard_columns)
			lock_box.set_data (the_data)
			lockable_projects_changed := True
		end
		
	build_unlock_box
			-- Build box for Unlock tab of notebook
		local
			cols: WIDGET_COLUMN_SET
			box: EV_BOX
			buttons: HORIZONTAL_BUTTON_SET
			label: EV_LABEL
			the_data: PACKING_PROJ_NOTEBOOK_TAB_DATA
		do
			create unlock_box
			
			create buttons.make_with_text_and_actions (<<
				"Unlock", "Refresh", "Send", "Close" >>,
				<< agent unlock_projects, agent refresh_unlocks, agent process_send, agent process_close >>)
			unlock_unlock_button := buttons.i_th (1)
			unlock_refresh_button := buttons.i_th (2)
			unlock_send_button := buttons.i_th (3)
			unlock_close_button := buttons.i_th (4)
			if is_read_only then
				unlock_unlock_button.disable_sensitive
			end
			unlock_unlock_button.set_tooltip ("Unlock the projects selected in the Included and Locked Projects list, allowing subsequent customer update and cancel operations.  Deassign them from run, if assigned to a run.")
			unlock_refresh_button.set_tooltip ("Refresh list of Included and Locked Projects")
			unlock_send_button.set_tooltip ("Send list of currently displayed projects to Output window")
			box := buttons.column_set.box
			unlock_box.extend (box)
			unlock_box.disable_item_expand (box)
			
			unlockable_projects := new_multi_column_list
			unlockable_projects.set_visible_columns (full_standard_columns)
			
			create cols.make (1)
			cols.set_padding (user_interface.padding)
			cols.set_border_width (Std_border_width)
			label := gui.label_with_text ("Included and Locked Projects")
			label.set_font (list_title_font)
			cols.append_row (<< label >>)
			cols.align_center
			box := cols.box
			
			unlock_box.extend (box)
			unlock_box.disable_item_expand (box)
			unlock_box.extend (unlockable_projects)
			box := separated_box (unlockable_projects.selection_status_box)
			unlock_box.extend (box)
			unlock_box.disable_item_expand (box)
			
			create the_data
			the_data.set_category ("unlockable")
			the_data.set_refresh_agent (agent refresh_unlockable_rows)
			the_data.set_selection_list (unlockable_projects)
			the_data.add_list_formatter (Basic_column_format, unlockable_projects, agent basic_standard_columns)
			the_data.add_list_formatter (Contact_column_format, unlockable_projects, agent contact_standard_columns)
			the_data.add_list_formatter (Full_column_format, unlockable_projects, agent full_standard_columns)
			unlock_box.set_data (the_data)
			unlockable_projects_changed := True
		end
		
	build_assignment_box
			-- Build box for Assign tab of notebook
		local
			cols: WIDGET_COLUMN_SET
			box: EV_BOX
			buttons: HORIZONTAL_BUTTON_SET
			label: EV_LABEL
			the_data: PACKING_PROJ_NOTEBOOK_TAB_DATA
		do
			create assignment_box
			
			create buttons.make_with_text_and_actions (<<
				"Assign", "Refresh", "Send", "Close" >>,
				<< agent assign_projects, agent refresh_assignments, agent process_send, agent process_close >>)
			assignment_assign_button := buttons.i_th (1)
			assignment_refresh_button := buttons.i_th (2)
			assignment_send_button := buttons.i_th (3)
			assignment_close_button := buttons.i_th (4)
			if is_read_only then
				assignment_assign_button.disable_sensitive
			end
			assignment_assign_button.set_tooltip ("Assign the projects selected in the Locked But Unassigned Projects list to this run, preventing them from being assigned to another run")
			assignment_refresh_button.set_tooltip ("Refresh list of Locked But Unassigned Projects")
			assignment_send_button.set_tooltip ("Send list of currently displayed projects to Output window")
			box := buttons.column_set.box
			assignment_box.extend (box)
			assignment_box.disable_item_expand (box)
			
			assignable_projects := new_multi_column_list
			assignable_projects.set_visible_columns (full_standard_columns)
			
			create cols.make (1)
			cols.set_padding (user_interface.padding)
			cols.set_border_width (Std_border_width)
			label := gui.label_with_text ("Locked But Unassigned Projects")
			label.set_font (list_title_font)
			cols.append_row (<< label >>)
			cols.align_center
			box := cols.box
			
			assignment_box.extend (box)
			assignment_box.disable_item_expand (box)
			assignment_box.extend (assignable_projects)
			box := separated_box (assignable_projects.selection_status_box)
			assignment_box.extend (box)
			assignment_box.disable_item_expand (box)
			
			create the_data
			the_data.set_category ("assignable")
			the_data.set_refresh_agent (agent refresh_assignable_rows)
			the_data.set_selection_list (assignable_projects)
			the_data.add_list_formatter (Basic_column_format, assignable_projects, agent basic_standard_columns)
			the_data.add_list_formatter (Contact_column_format, assignable_projects, agent contact_standard_columns)
			the_data.add_list_formatter (Full_column_format, assignable_projects, agent full_standard_columns)
			assignment_box.set_data (the_data)
			assignable_projects_changed := True
		end
		
	build_deassignment_box
			-- Build box for Deassign tab of notebook
		local
			cols: WIDGET_COLUMN_SET
			box: EV_BOX
			buttons: HORIZONTAL_BUTTON_SET
			label: EV_LABEL
			the_data: PACKING_PROJ_NOTEBOOK_TAB_DATA
		do
			create deassignment_box
			
			create buttons.make_with_text_and_actions (<<
				"Deassign", "Refresh", "Send", "Close" >>,
				<< agent deassign_projects, agent refresh_deassignments, agent process_send, agent process_close >>)
			deassignment_deassign_button := buttons.i_th (1)
			deassignment_refresh_button := buttons.i_th (2)
			deassignment_send_button := buttons.i_th (3)
			deassignment_close_button := buttons.i_th (4)
			if is_read_only then
				deassignment_deassign_button.disable_sensitive
			end
			deassignment_deassign_button.set_tooltip ("Deassign the projects selected in the Projects Currently Assigned to Run list from run, allowing them to be assigned to another run")
			deassignment_refresh_button.set_tooltip ("Refresh list of Projects Currently Assigned to Run")
			deassignment_send_button.set_tooltip ("Send list of currently displayed projects to Output window")
			box := buttons.column_set.box
			deassignment_box.extend (box)
			deassignment_box.disable_item_expand (box)
			
			deassignable_projects := new_multi_column_list
			deassignable_projects.set_visible_columns (full_standard_columns)
			
			create cols.make (1)
			cols.set_padding (user_interface.padding)
			cols.set_border_width (Std_border_width)
			label := gui.label_with_text ("Projects Currently Assigned to Run")
			label.set_font (list_title_font)
			cols.append_row (<< label >>)
			cols.align_center
			box := cols.box
			
			deassignment_box.extend (box)
			deassignment_box.disable_item_expand (box)
			deassignment_box.extend (deassignable_projects)
			box := separated_box (deassignable_projects.selection_status_box)
			deassignment_box.extend (box)
			deassignment_box.disable_item_expand (box)
			
			create the_data
			the_data.set_category ("deassignable")
			the_data.set_refresh_agent (agent refresh_deassignable_rows)
			the_data.set_selection_list (deassignable_projects)
			the_data.add_list_formatter (Basic_column_format, deassignable_projects, agent basic_standard_columns)
			the_data.add_list_formatter (Contact_column_format, deassignable_projects, agent contact_standard_columns)
			the_data.add_list_formatter (Full_column_format, deassignable_projects, agent full_standard_columns)
			deassignment_box.set_data (the_data)
			deassignable_projects_changed := True
		end
		
	build_update_box
			-- Build box for Update tab of notebook
		local
			cols: WIDGET_COLUMN_SET
			box: EV_BOX
			buttons: HORIZONTAL_BUTTON_SET
			hbox: EV_HORIZONTAL_BOX
			the_data: PACKING_PROJ_NOTEBOOK_TAB_DATA
		do
			create update_box
			
			create cols.make (4)
			cols.set_padding (user_interface.padding)
			cols.set_border_width (Std_border_width)
			create update_locked_cust_button.make_with_text ("Locked customer projects")
			create update_unlocked_cust_button.make_with_text ("Unlocked customer projects")
			create update_test_vehicle_button.make_with_text ("Test vehicles")
			update_locked_cust_button.enable_select
			update_unlocked_cust_button.enable_select
			update_test_vehicle_button.enable_select
			cols.append_row (<< gui.label_with_text ("Filter By Type:"),  update_unlocked_cust_button, update_locked_cust_button, update_test_vehicle_button >>)
			cols.disable_expand
			cols.align_left
			box := cols.box
			update_box.extend (box)
			update_box.disable_item_expand (box)
			
			create buttons.make_with_text_and_actions (<<
				"Update", "Compare", "Filter", "Send", "Close" >>,
				<< agent update_projects, agent compare_projects, agent refresh_update, agent process_send, agent process_close >>)
			update_update_button := buttons.i_th (1)
			update_compare_button := buttons.i_th (2)
			update_filter_button := buttons.i_th (3)
			update_send_button := buttons.i_th (4)
			update_close_button := buttons.i_th (5)
			if is_read_only then
				update_update_button.disable_sensitive
			end
			update_filter_button.set_tooltip ("Display only items that meet the filter criteria")
			update_compare_button.set_tooltip ("Compare the snapshots of the projects selected in the Included Projects list to their current data and display any differences")
			update_update_button.set_tooltip ("Update the snapshots of the projects selected in the Included Projects list with the current data")
			update_send_button.set_tooltip ("Send list of currently displayed projects to Output window")
			box := buttons.column_set.box
			update_box.extend (box)
			update_box.disable_item_expand (box)
			
			create updatable_label
			updatable_label.set_font (list_title_font)
			create hbox
			hbox.set_border_width (Std_border_width)
			hbox.extend (updatable_label)
			update_box.extend (hbox)
			update_box.disable_item_expand (hbox)
			
			updatable_projects := new_multi_column_list
			updatable_projects.set_visible_columns (full_standard_columns)
			update_box.extend (updatable_projects)
			box := separated_box (updatable_projects.selection_status_box)
			update_box.extend (box)
			update_box.disable_item_expand (box)
			
			create the_data
			the_data.set_category ("updatable")
			the_data.set_refresh_agent (agent refresh_updatable_rows)
			the_data.set_selection_list (updatable_projects)
			the_data.add_list_formatter (Basic_column_format, updatable_projects, agent basic_standard_columns)
			the_data.add_list_formatter (Contact_column_format, updatable_projects, agent contact_standard_columns)
			the_data.add_list_formatter (Full_column_format, updatable_projects, agent full_standard_columns)
			update_box.set_data (the_data)
			updatable_projects_changed := True
		end
		
	add_menu_bar
			-- Add menu bar to window
		local
			file_menu, select_menu, view_menu: EV_MENU
			all_menu_item, none_menu_item, invert_menu_item, close_menu_item: EV_MENU_ITEM
			flush_menu_item: EV_MENU_ITEM
			basic_menu_item, contact_menu_item: EV_MENU_ITEM
			phone_menu_item: EV_MENU_ITEM
			a_menu_bar: EV_MENU_BAR
		do	
			create file_menu.make_with_text ("File")
			
			create flush_menu_item.make_with_text ("Flush project cache")
			file_menu.extend (flush_menu_item)
			flush_menu_item.select_actions.extend (agent process_flush)
			create phone_menu_item.make_with_text ("Telephone selected project")
			phone_menu_item.select_actions.extend (agent process_phone_project)
			file_menu.extend (phone_menu_item)
			create close_menu_item.make_with_text ("Close")
			file_menu.extend (close_menu_item)
			close_menu_item.select_actions.extend (agent process_close)
			
			create select_menu.make_with_text ("Select")
			
			create all_menu_item.make_with_text ("All")
			all_menu_item.select_actions.extend (agent process_select_all)
			select_menu.extend (all_menu_item)
			create none_menu_item.make_with_text ("None")
			none_menu_item.select_actions.extend (agent process_select_none)
			select_menu.extend (none_menu_item)
			create invert_menu_item.make_with_text ("Invert")
			invert_menu_item.select_actions.extend (agent process_invert_selection)
			select_menu.extend (invert_menu_item)
			
			create view_menu.make_with_text ("View")
			
			create basic_menu_item.make_with_text ("Short Format")
			basic_menu_item.select_actions.extend (agent process_view_columns (Basic_column_format))
			view_menu.extend (basic_menu_item)
			
			create all_menu_item.make_with_text ("Long Format")
			all_menu_item.select_actions.extend (agent process_view_columns (Full_column_format))
			view_menu.extend (all_menu_item)
			
			create contact_menu_item.make_with_text ("Contact Info")
			contact_menu_item.select_actions.extend (agent process_view_columns (Contact_column_format))
			view_menu.extend (contact_menu_item)
			
			create a_menu_bar
			a_menu_bar.extend (file_menu)
			a_menu_bar.extend (select_menu)
			a_menu_bar.extend (view_menu)
			set_menu_bar (a_menu_bar)
		end

	post_display_actions
		do
			gui.do_once_on_idle (agent gui.center_window_with_margin (window, 100, 100))
			reset
		end

	handle_version_selected (e: PACKING_VERSION_SELECTED_EVENT)
			-- Handle selection of a new version of packing
		do
			handle_version_switch
		end

	handle_version_created (e: PACKING_VERSION_CREATED_EVENT)
			-- Handle creation and switch to a new version
			-- of packing
		do
			handle_version_switch
		end

	process_close
		do
			if is_active then
				disable_active
				cancel_actions.call ([])
			end
		end

	process_flush
		do
			if is_active then
				disable_active
				set_all_changed
				if is_displayed then
					refresh_top_page
				end
				enable_active
			end
		end

	process_phone_project
			-- Process request to call telephone number
			-- of selected project
		do
			do_operation_if_active (agent phone_project_request)
		end

	phone_project_request
			-- Call telephone number of selected project if
			-- only one project is selected
		local
			list: like current_tab_selection_list
			rows: LIST [PACKING_PROJ_MULTI_COLUMN_LIST_ROW]
			phone_rows: LIST [PACKING_PROJECTS_ROW]
			phone_row: PACKING_PROJECTS_ROW
			dno: INTEGER
			phone_num: STRING
		do
			list := current_tab_selection_list
			if list /= Void then
				list.rebuild_selected_rows
				rows := list.selected_rows
				if rows.count /= 1 then
					report_need_one_row
				else
					phone_rows := project_rows (rows)
					phone_row := phone_rows.first
					dno := phone_row.design_no
					if phone_row.phone_number.is_empty then
						inform ("No phone number for project " + dno.out)
					else
						confirm ("Call project " + dno.out + " at " + phone_row.phone_number)
						phone_num := su.reduced_to_numerics (phone_row.phone_number)
						dial_phone_number (phone_num)
					end
				end
			end
		end

	dial_phone_number (s: STRING)
			-- Execute phone program to dial phone number `s'
		local
			p: UNIX_PROCESS
		do
			create p.make (session.environment.phone_program)
			p.set_arguments (<< s >>)
			p.set_output_file_name (unix_os.null_file_name)
			p.set_error_same_as_output
			p.spawn_nowait
			p.get_status_block
			if not p.is_success_exit then
				-- Display p.status_description (p.status)
			end
		end

	process_select_all
			-- Process request to select all items in
			-- list on current notebook tab
		local
			list: like current_tab_selection_list
		do
			if is_active then
				disable_active
				list := current_tab_selection_list
				if list /= Void then
					list.process_select_all
				end
				enable_active
			end
		end

	process_select_none
			-- Process request to unselect all items in
			-- list on current notebook tab
		local
			list: like current_tab_selection_list
		do
			if is_active then
				disable_active
				list := current_tab_selection_list
				if list /= Void then
					list.process_select_none
				end
				enable_active
			end
		end

	process_invert_selection
			-- Process request to invert selection in
			-- on current notebook tab
		local
			list: like current_tab_selection_list
		do
			if is_active then
				disable_active
				list := current_tab_selection_list
				if list /= Void then
					list.process_select_invert
				end
				enable_active
			end
		end

	process_view_columns (fmt: STRING)
			-- Display only columns selected by `fmt'
			-- in multi-column lists in current notebook tab
		do
			if is_active then
				disable_active
				current_tab_data.set_format (fmt)
				enable_active
			end
		end

	refresh_status
			-- Refresh contents of Status tab of notebook
		do
			if is_active then
				disable_active
				status_projects_changed := True
					-- Force refresh
				refresh_status_rows
				enable_active
			end
		end

	refresh_status_rows
			-- Refresh contents of Status tab of notebook
		local
			f: FUNCTION [ANY, TUPLE [PACKING_PROJECTS_MROW], BOOLEAN]
			wants_cust, wants_tv: BOOLEAN
			wants_selected, wants_deselected: BOOLEAN
			sel_rows, desel_rows: like status_rows
		do
			if status_projects_changed then
				wants_selected := status_included_button.is_selected
				wants_deselected := status_not_included_button.is_selected
				wants_cust := status_cust_die_button.is_selected
				wants_tv := status_test_vehicle_button.is_selected
				status_label.set_text (status_label_text (wants_selected, wants_deselected, wants_cust, wants_tv))
				f := cust_test_vehicle_selector (wants_cust, wants_tv)
				if wants_selected then
					sel_rows := packing.selected_projects_matching_filter (f)
				else
					sel_rows := empty_project_list (0)
				end
				if wants_deselected then
					desel_rows := packing.deselected_projects_matching_filter (f)
				else
					desel_rows := empty_project_list (0)
				end
				status_rows := sel_rows
				status_rows.append (desel_rows)
				build_list (status_projects, status_rows)
				status_projects.sort_on_column (Design_no_position)
				status_projects_changed := False
			end
		end

	status_label_text (wants_selected, wants_deselected, wants_cust, wants_tv: BOOLEAN): STRING
			-- Text for status label based on value of arguments
		local
			sel, type: STRING
			none: BOOLEAN
		do
			sel := selected_deselected_text (wants_selected, wants_deselected)
			if sel.is_empty then
				none := True
			end
				
			type := cust_test_vehicle_text (wants_cust, wants_tv)
			if type.is_empty then
				none := True
			end
			
			if none then
				Result := "No Projects Requested (Adjust Conditions and Re-Filter)"
			else
				Result := sel + " " + type
			end
		end
			
	selection_label_text (kind: STRING wants_cust, wants_tv: BOOLEAN): STRING
			-- Text for selectable projects label
		local
			s: STRING
		do
			s := cust_test_vehicle_text (wants_cust, wants_tv)
			if not s.is_empty then
				Result := kind + " " + s
			else
				Result := kind + " Projects (None Requested - Adjust Conditions and Re-Filter)"
			end
		end
			
	updatable_label_text (kind: STRING wants_locked_cust, wants_unlocked_cust, wants_tv: BOOLEAN): STRING
			-- Text for updatable projects label
		local
			locked, type: STRING
			wants_cust, none: BOOLEAN
		do
			wants_cust := wants_locked_cust or wants_unlocked_cust
			locked := locked_unlocked_text (wants_locked_cust, wants_unlocked_cust)
			type := cust_test_vehicle_text (wants_cust, wants_tv)
			if type.is_empty then
				none := True
			end
			
			if none then
				Result := "No Projects Requested (Adjust Conditions and Re-Filter)"
			elseif wants_tv and not wants_cust then
				Result := kind + " " + type
			else
				Result := kind + " " + locked + " " + type
			end
		end
			
	selected_deselected_text (wants_selected, wants_deselected: BOOLEAN): STRING
			-- Text for label based on value of arguments
		do
			if wants_selected and wants_deselected then
				Result := "Included and Removed"
			elseif wants_selected then
				Result := "Included"
			elseif wants_deselected then
				Result := "Removed"
			else
				Result := ""
			end
		end
			
	locked_unlocked_text (wants_locked, wants_unlocked: BOOLEAN): STRING
			-- Text for label based on value of arguments
		do
			if wants_locked and wants_unlocked then
				Result := "Locked and Unlocked"
			elseif wants_locked then
				Result := "Locked"
			elseif wants_unlocked then
				Result := "Unlocked"
			else
				Result := ""
			end
		end
			
	cust_test_vehicle_text (wants_cust, wants_tv: BOOLEAN): STRING
			-- Text for label based on value of arguments
		do
			if wants_cust and wants_tv then
				Result := "Customer Projects and Test Vehicles"
			elseif wants_cust then
				Result := "Customer Projects"
			elseif wants_tv then
				Result := "Test vehicles"
			else
				Result := ""
			end
		end
			
	cust_test_vehicle_selector (wants_cust, wants_tv: BOOLEAN): FUNCTION [ANY, TUPLE [PACKING_PROJECTS_MROW], BOOLEAN]
			-- Agent to select customer projects and/or test
			-- vehicles, based on `wants_cust' and `wants_tv'
		do
			if wants_cust and wants_tv then
				Result := agent {PACKING_PROJECTS_MROW}.is_customer_project_or_test_vehicle
			elseif wants_cust then
				Result := agent {PACKING_PROJECTS_MROW}.is_customer_project
			elseif wants_tv then
				Result := agent {PACKING_PROJECTS_MROW}.is_test_vehicle
			else
				Result := agent {PACKING_PROJECTS_MROW}.is_neither_customer_project_nor_test_vehicle
			end
		end
			
	build_list (proj_list: like status_projects rows: LIST [PACKING_PROJECTS_ROW])
			-- Build multi-column list `proj_list' from `rows'.
			-- Uncache foreign keys for each project in `rows'
			-- so that database information changed outside of
			-- Delta will be visible
		local
			list: ARRAYED_LIST [PACKING_PROJ_MULTI_COLUMN_LIST_ROW]
		do
			create list.make (rows.count)
			rows.do_all (agent {PACKING_PROJECTS_ROW}.uncache_foreign_keys)
			rows.do_all (agent add_row (?, list, proj_list))
			proj_list.set_all_rows (list)
			if has_run_context then
				proj_list.all_rows.do_all (agent {PACKING_PROJ_MULTI_COLUMN_LIST_ROW}.set_run_id (run_id))
			end
		end
			
	add_row (p: PACKING_PROJECTS_ROW list: LIST [PACKING_PROJ_MULTI_COLUMN_LIST_ROW] proj_list: like status_projects)
			-- Add a row for `p' to `list', which will be
			-- the list of rows for `proj_list'
		local
			row: PACKING_PROJ_MULTI_COLUMN_LIST_ROW
		do
			create row.make (p, packing)
			row.set_data (p)
			row.select_actions.extend (agent proj_list.process_select_one (row))
			row.deselect_actions.extend (agent proj_list.process_deselect_one (row))
			list.extend (row)
		end
			
	refresh_selection
			-- Refresh contents of Select tab of notebook
		do
			if is_active then
				disable_active
				selection_projects_changed := True
					-- Force refresh
				refresh_selection_rows (True)
				enable_active
			end
		end

	refresh_selection_rows (refresh_cust: BOOLEAN)
			-- Refresh contents of Select tab of notebook.
			-- Refresh customer projects if `refresh_cust'
		require
			virgin_transaction: not connection.is_changed
		do
			if selection_projects_changed then
				-- Ensure we get read-consistent view of
				-- database so information we show user
				-- is consistent
				connection.rollback
				connection.set_transaction_read_only
				current_version.setup_for_fast_access
				if refresh_cust then
					fetch_fab_pool
					remove_included_cust_projects
				end
				fetch_test_vehicles
				remove_included_test_vehicles
				if not split_area_initialized then
					selection_split_area.set_proportion (0.75)
					split_area_initialized := True
				end
				process_selection_filter
				connection.rollback
			end
		end

	refresh_deselection
			-- Refresh contents of Deselect tab of notebook
		do
			if is_active then
				disable_active
				deselectable_projects_changed := True
					-- Force refresh
				refresh_deselectable_rows
				enable_active
			end
		end

	refresh_deselectable_rows
			-- Refresh contents of Deselect tab of notebook
		local
			wants_cust, wants_tv: BOOLEAN
		do
			if deselectable_projects_changed then
				wants_cust := deselection_cust_die_button.is_selected
				wants_tv := deselection_test_vehicle_button.is_selected
				deselectable_label.set_text (selection_label_text ("Currently Included", wants_cust, wants_tv))
				
				deselectable_rows := packing.selected_projects_matching_filter (cust_test_vehicle_selector (wants_cust, wants_tv))
				build_list (deselectable_projects, deselectable_rows)
				deselectable_projects.sort_on_column (Design_no_position)
				deselectable_projects_changed := False
			end
		end

	select_projects
			-- Try to select projects
		do
			do_operation_if_active (agent select_projects_request)
		end

	select_projects_request
		local
			rows: LIST [PACKING_PROJ_MULTI_COLUMN_LIST_ROW]
			wants_lock, wants_assign: BOOLEAN
			old_style: EV_POINTER_STYLE
		do
			selectable_projects.rebuild_selected_rows
			rows := selectable_projects.selected_rows
			if rows.is_empty then
				report_no_rows_selected
			else
				wants_lock := selection_lock_button.is_selected
				wants_assign := selection_assign_button.is_selected
				confirm_selection (rows, wants_lock, wants_assign)
				old_style := pointer_style
				set_pointer_style (gui.Wait_cursor)
				gui.process_events
				try_to_select_projects (project_rows (rows), wants_lock, wants_assign)
				if ok_count > 0 then
					unlockable_projects_changed := True
						-- Even if user didn't ask for
						-- Lock, projects might already
						-- be locked
					if not wants_lock then
						lockable_projects_changed := True
					end
					deassignable_projects_changed := True
						-- Even if user didn't ask for
						-- Assign, projects might
						-- already be assigned
					if not wants_assign then
						assignable_projects_changed := True
					end
					status_projects_changed := True
					deselectable_projects_changed := True
					selection_projects_changed := True
					refresh_selection_rows (cust_project_count > 0)
				end
				set_pointer_style (old_style)
				display_operation_results
			end
		rescue
			if old_style /= Void then
				set_pointer_style (old_style)
			end
		end

	confirm_selection (rows: LIST [PACKING_PROJ_MULTI_COLUMN_LIST_ROW] wants_lock, wants_assign: BOOLEAN)
			-- Get user confirmation for Select request
		local
			msg: STRING
		do
			msg := rows.count.out + " " + su.number (rows.count, "project", "projects") + " will be added to packing"
			if wants_assign then
				msg.append (", locked against customer update and assigned to this run")
			elseif wants_lock then
				if has_run_context then
					msg.append (" and locked against customer update but will not be assigned to this run")
				else
					msg.append (" and locked against customer update")
				end
			else
				msg.append (", but will not be locked or assigned to this run.%N(Note that projects might already be locked or assigned to run - this status won't be changed)")
			end
			msg.append (Proceed_text)
			confirm (msg)
		end

	try_to_select_projects (list: LIST [PACKING_PROJECTS_MROW] wants_lock, wants_assign: BOOLEAN)
			-- Try to select the projects in `list'.
			-- Leave description of results in `messages'
		local
			l_locked_projects: like list
		do
			create {ARRAYED_LIST [PACKING_PROJECTS_MROW]} l_locked_projects.make (list.count)
			initialize_counts
			list.do_all (agent try_to_lock_project_rows (?, l_locked_projects))
			if cust_project_count > 0 then
				fetch_fab_pool_and_setup_table
			end
			create {ARRAYED_LIST [PACKING_PROJECTS_ROW]} selected_projects.make (list.count)
			try_to_do_action (l_locked_projects, agent try_to_modify_project (?, "available", wants_lock, wants_assign, False, selected_projects), agent finish_select, "added")
		end

	finish_select
			-- Do actions needed to finish Select operation
		do
			if not selected_projects.is_empty then
				current_version.add_dies_for_projects (selected_projects)
				compute_compatible_processes (current_version.added_projects)
				assign_processes_if_needed (selected_projects)
				projects_added_actions.call ([current_version.added_dies])
			end
		end

	try_to_lock_project_rows (p: PACKING_PROJECTS_MROW list: like selected_projects)
			-- Try to lock cust IC design for `p' (and fab
			-- instance for cust project).  Add `p' to
			-- `list' if it is locked.  Append description
			-- of results to `errors' it lock failure.
		require
			proj_not_void: p /= Void
			list_not_void: list /= Void
		do
			p.uncache_foreign_keys
			if p.is_customer_project then
				cust_project_count := cust_project_count + 1
				try_to_lock_cid_and_ifi (p)
			else
				try_to_lock_cust_ic_design (p)
			end
			if last_lock_ok then
				list.extend (p)
			end
		end

	try_to_modify_project (p: PACKING_PROJECTS_MROW what: STRING wants_lock, wants_assign, tech_sel: BOOLEAN list: like selected_projects)
			-- Try to modify project for which cust IC
			-- design and fab instance (if applicable)
			-- have been locked.  Modify it as indicated
			-- by `what', `wants_lock' and `wants_assign'.
			-- Add `p' to `list' if it is successfully
			-- modified.  Append description of results to
			-- `errors', `warnings' and `notes'
		require
			proj_not_void: p /= Void
			what_not_void: what /= Void
			list_not_void: list /= Void
		do
			if p.is_customer_project then
				try_to_modify_cust_project (p, what, wants_lock, wants_assign, tech_sel, list)
			else
				try_to_modify_test_vehicle (p, what, list)
			end
		end

	try_to_modify_cust_project (p: PACKING_PROJECTS_MROW what: STRING wants_lock, wants_assign, tech_sel: BOOLEAN list: like selected_projects)
			-- Try to select `p', locking it if
			-- `wants_lock' and assigning it to run if
			-- `wants_assign'.  Cust IC design and fab instance
			-- (if applicable) must already be locked.
			-- If `tech_sel', then only
			-- check technical selectability (compatible
			-- with wafer techn, can find a process);
			-- otherwise check both technical and policy
			-- selectability reasons.  Add `p' to `list'
			-- if it is successfully locked and modified.
			-- Append description of results to `errors',
			-- `warnings' and `notes'
		require
			is_cust_project: p.is_customer_project
		local
			proj: FAB_POOL_W_WAFER_TECHNS_ROW
			cid: CUST_IC_DESIGNS_MROW
			ifi: IC_FAB_INSTANCES_MROW
		do
			last_ok := False
			create proj.make_template
			fab_pool_projects.search (p.m_fab_no)
			if fab_pool_projects.found then
				proj := fab_pool_projects.found_item
				if tech_sel then
					proj.check_technical_selectability (packing.packings_row)
				else
					check_project_full_selectability (proj)
				end
				if proj.is_selectable then
					-- Check selectability with
					-- no overrides
					proj.check_policy_selectability (packing.packings_row, run_id_or_void, False, False, False, False, False)
					if not proj.is_selectable then
						add_warning (p.design_no, Void, proj.reason_unselectable)
					end
					last_ok := True
					add_ok (p.design_no)
					list.extend (p)
					check_overhead_viable (p)
					cid := p.cust_ic_design
					ifi := p.fab_instance
					if wants_lock then
						ifi.freeze
						cid.freeze
					end
					if wants_assign then
						if ifi.can_be_assigned_to_run (run_id) then
							ifi.assign_to_run_if_unassigned (run_id)
						else
							add_note (p.design_no, Void, "not assigned to this run - already assigned to run " + ifi.run_id)
						end
					end
				else
					add_error (p.design_no, Void, "no longer " + what + ".  " + proj.reason_unselectable)
				end
			else
				add_error (p.design_no, Void, "no longer in fab pool for " + packing.wafer_techn)
			end
		end

	try_to_modify_test_vehicle (p: PACKING_PROJECTS_MROW what: STRING list: like selected_projects)
			-- Add `p' to `list' if it is successfully
			-- selected.  Append description of results to
			-- `errors', `warnings' and `notes'
		require
			is_test_vehicle: p.is_test_vehicle
		local
			tvs: LIST [M_TEST_VEHICLES_ROW]
			tv: M_TEST_VEHICLES_ROW
		do
			last_ok := False
			create tv.make_template
			tvs := tv.test_vehicle_with_design_no (p.design_no, packing.wafer_techn, connection, expected_count.At_most_one)

			if not tvs.is_empty then
				tv := tvs.first
				if is_test_vehicle_selectable (tv) then
					last_ok := True
					add_ok (p.design_no)
					list.extend (p)
					check_overhead_viable (p)
				elseif tv.may_be_fabricated then
					add_error (p.design_no, Void, "test vehicle no longer " + what + ".  " + tv.wafer_techn_layer_error_text)
				else
					add_error (p.design_no, Void, "test vehicle no longer " + what + ": status is now " + tv.test_vehicle_status)
				end
			else
				add_error (p.design_no, Void, "test vehicle has been retired or no longer exists")
			end
		end

	check_overhead_viable (p: PACKING_PROJECTS_MROW)
			-- Verify that we can generate overhead
			-- for `p', if requested.  If we can't, add a warning
		local
			cid: CUST_IC_DESIGNS_ROW
			style: OVERHEAD_STYLE_VALUES
		do
			cid := p.cust_ic_design
			if not cid.is_overhead_viable (packing.packings_row.effective_overhead_style (p.overhead_style)) then
				add_warning (p.design_no, Void, "Overhead generation will fail for this project (effective overhead style is " + style.External_ + ")")
			end
		end

	process_selection_filter
			-- Apply filter criteria and re-display selectable
			-- and unselectable projects in Select tab
			-- NOTE: when this routine is called directly
			-- (because user clicked Filter in Select tab)
			-- transaction is not read-only.  However, we
			-- shouldn't be retrieving anything from the database
			-- in that case - everything should be cached in
			-- memory due to a prior "refresh" operation
		local
			wants_cust, wants_tv: BOOLEAN
		do
			wants_cust := selection_cust_die_button.is_selected
			wants_tv := selection_test_vehicle_button.is_selected
			save_user_selection_preferences
			selectable_label.set_text (selection_label_text ("Available", wants_cust, wants_tv))
			unselectable_label.set_text (selection_label_text ("Unavailable", wants_cust, wants_tv))
			if wants_cust then
				split_fab_pool
			end
			if wants_tv then
				split_test_vehicles
			end
			
			packing.setup_for_fast_project_access
			current_version.set_selection_date
			packing.packings_row.save_highest_used_project_no
			set_selectable_rows (wants_cust, wants_tv)
			set_unselectable_rows (wants_cust, wants_tv)
			packing.packings_row.restore_highest_used_project_no
			
			build_list (selectable_projects, selectable_rows)
			build_list (unselectable_projects, unselectable_rows)
			selectable_projects.sort_on_column (Design_no_position)
			unselectable_projects.sort_on_column (Design_no_position)
			selection_projects_changed := False
		end

	save_user_selection_preferences
			-- Save user preferences for which projects and
			-- test vehicles should be made selectable, even
			-- though they are incompatible (wrong run type, etc)
		do
			allow_wrong_type := selection_bad_run_type_button.is_selected
			allow_wrong_foundry := selection_wrong_foundry_button.is_selected
			allow_no_access := selection_no_access_button.is_selected
			allow_assigned_to_run := selection_already_assigned_button.is_selected
			allow_uncharged := selection_not_charged_button.is_selected
			allow_incompat_test_vehicles := selection_incompat_test_vehicle_button.is_selected
		end

	set_selectable_rows (wants_cust, wants_tv: BOOLEAN)
			-- Set `selectable_rows' based on `selectable_fab_pool'
			-- and `selectable_test_vehicles'
		local
			list: like selectable_rows
			n: INTEGER
		do
			if wants_cust then
				n := selectable_fab_pool.count
			end
			if wants_tv then
				n := n + selectable_test_vehicles.count
			end
			create {ARRAYED_LIST [PACKING_PROJECTS_MROW]} list.make (n)
			if wants_cust then
				selectable_fab_pool.do_all (agent append_cust_project (?, list, True))
			end
			if wants_tv then
				selectable_test_vehicles.do_all (agent append_test_vehicle (?, list, True))
			end
			selectable_rows := list
		end

	set_unselectable_rows (wants_cust, wants_tv: BOOLEAN)
			-- Set `unselectable_rows' based on
			-- `unselectable_fab_pool' and
			-- `unselectable_test_vehicles'
		local
			list: like unselectable_rows
			n: INTEGER
		do
			if wants_cust then
				n := unselectable_fab_pool.count
			end
			if wants_tv then
				n := n + unselectable_test_vehicles.count
			end
			create {ARRAYED_LIST [PACKING_PROJECTS_MROW]} list.make (n)
			if wants_cust then
				unselectable_fab_pool.do_all (agent append_cust_project (?, list, False))
			end
			if wants_tv then
				unselectable_test_vehicles.do_all (agent append_test_vehicle (?, list, False))
			end
			unselectable_rows := list
		end

	append_cust_project (proj: FAB_POOL_W_WAFER_TECHNS_ROW list: like selectable_rows sel: BOOLEAN)
			-- Append project for `proj' to `list', where `sel'
			-- indicates whether project is selectable
		local
			p: PACKING_PROJECTS_MROW
		do
			p := current_version.new_cust_die_project (proj.m_fab_no)
			if not sel then
				p.set_reason_unselectable (proj.reason_unselectable)
			end
			list.extend (p)
		end

	append_test_vehicle (tv: M_TEST_VEHICLES_ROW list: like selectable_rows sel: BOOLEAN)
			-- Append project for `tv' to `list', where `sel'
			-- indicates whether project is selectable
		local
			p: PACKING_PROJECTS_MROW
		do
			p := current_version.new_test_vehicle_project (tv.design_no)
			if not sel then
				p.set_reason_unselectable (tv.wafer_techn_layer_error_text)
			end
			list.extend (p)
		end

	deselect_projects
			-- Try to deselect projects
		do
			do_operation_if_active (agent deselect_projects_request)
		end

	deselect_projects_request
		local
			rows: LIST [PACKING_PROJ_MULTI_COLUMN_LIST_ROW]
			wants_unlock, wants_deassign: BOOLEAN
			projs: like project_rows
		do
			deselectable_projects.rebuild_selected_rows
			rows := deselectable_projects.selected_rows
			if rows.is_empty then
				report_no_rows_selected
			else
				wants_unlock := deselection_unlock_button.is_selected
				wants_deassign := deselection_deassign_button.is_selected
				projs := project_rows (rows)
				deselect_project_list (projs, wants_unlock, wants_deassign)
			end
		end

	locked_dies (list: LIST [PACKING_DIES_MROW]): LIST [PACKING_DIES_MROW]
			-- Dies in `list' that have one or more locations
			-- that are delete-locked
		local
			die: PACKING_DIES_MROW
		do
			create {ARRAYED_LIST [PACKING_DIES_MROW]} Result.make (list.count)
			from
				list.start
			until
				list.after
			loop
				die := list.item
				if not die.is_deletable then
					Result.extend (die)
				end
				list.forth
			end
		end

	unlocked_dies (list: LIST [PACKING_DIES_MROW]): LIST [PACKING_DIES_MROW]
			-- Dies in `list' that have do not have any locations
			-- that are delete-locked
		local
			die: PACKING_DIES_MROW
		do
			create {ARRAYED_LIST [PACKING_DIES_MROW]} Result.make (list.count)
			from
				list.start
			until
				list.after
			loop
				die := list.item
				if die.is_deletable then
					Result.extend (die)
				end
				list.forth
			end
		end

	confirm_delete_locked_dies (dies: LIST [PACKING_DIES_ROW])
			-- Get user confirmation for Deselect request
			-- for `dies', which have at least one location
			-- that is delete-locked
		local
			msg: STRING
			die: PACKING_DIES_ROW
			n: INTEGER
		do
			if not dies.is_empty then
				msg := ""
				from
					dies.start
				until
					dies.after
				loop
					die := dies.item
					n := n + 1
					msg.append ("   " + die.design_no.out + "   " + die.design_name + "%N")
					dies.forth
				end
				msg := "The following projects will not be removed because they are delete-locked.%N%N" + msg
				msg.append ("%N" + n.out + " " + su.number (n, "project", "projects") + " with at least one copy that is delete-locked")
				msg.append (Proceed_text)
				confirm (msg)
			end
		end

	confirm_deselection (rows: LIST [PACKING_PROJECTS_ROW] wants_unlock, wants_deassign: BOOLEAN)
			-- Get user confirmation for Deselect request
		local
			msg: STRING
		do
			msg := rows.count.out + " " + su.number (rows.count, "project", "projects") + " will be removed from packing"
			if wants_unlock then
				msg.append (", deassigned from this run and unlocked to allow customer update")
			elseif wants_deassign then
				msg.append (" and deassigned from this run but will not be unlocked")
			else
				msg.append (", but the %"locked%" and %"assigned%" status will not be affected")
			end
			msg.append (Proceed_text)
			confirm (msg)
		end

	try_to_deselect_projects (list: LIST [PACKING_PROJECTS_MROW] wants_unlock, wants_deassign: BOOLEAN)
			-- Try to deselect the projects in `list'.
			-- Leave description of results in `messages'
		do
			create {ARRAYED_LIST [PACKING_PROJECTS_MROW]} deselected_projects.make (list.count)
			initialize_counts
			try_to_do_action (list, agent try_to_deselect_project (?, wants_unlock, wants_deassign), agent finish_deselect, "removed")
		end

	finish_deselect
		do
			if not deselected_projects.is_empty then
				current_version.remove_dies_for_projects (deselected_projects)
				assign_processes_if_needed (deselected_projects)
				projects_removed_actions.call ([current_version.removed_dies])
				removed_dies := current_version.removed_dies
			else
				create {ARRAYED_LIST [PACKING_DIES_MROW]} removed_dies.make (0)
			end
		end

	try_to_deselect_project (p: PACKING_PROJECTS_MROW wants_unlock, wants_deassign: BOOLEAN)
			-- Try to deselect `p', unlocking it if `wants_unlock'
			-- and deassigning it from run if `wants_deassign'.
			-- Add `p' to `deselected_projects' if it is
			-- successfully deselected.
			-- Append description of results to
			-- `errors', `warnings' and `notes'
		do
			if p.is_customer_project then
				if wants_unlock then
					try_to_unlock_project (p)
				elseif wants_deassign then
					try_to_deassign_project (p)
				else
					add_ok (p.design_no)
					last_ok := True
				end
			else
				add_ok (p.design_no)
				last_ok := True
			end
			if last_ok then
				deselected_projects.extend (p)
			end
		end

	lock_projects
			-- Try to lock projects
		do
			inform ("The Lock function is temporarily disabled until it is fixed.%NAs a workaround, Remove the project(s) you want to lock and then Add them back with lock requested.%NNote that the projects will lose their positions if you do this.")
			-- FIXME: add this back in when Lock plays nicely
			-- with graphical packer
			-- do_operation_if_active (agent lock_projects_request)
		end

	lock_projects_request
		local
			rows: LIST [PACKING_PROJ_MULTI_COLUMN_LIST_ROW]
			wants_assign: BOOLEAN
		do
			lockable_projects.rebuild_selected_rows
			rows := lockable_projects.selected_rows
			if rows.is_empty then
				report_no_rows_selected
			else
				wants_assign := lock_assign_button.is_selected
				confirm_lock (rows, wants_assign)
				try_to_lock_projects (project_rows (rows), wants_assign)
				if ok_count > 0 then
					if wants_assign then
						deassignable_projects_changed := True
					end
					assignable_projects_changed := True
					status_projects_changed := True
					unlockable_projects_changed := True
					lockable_projects_changed := True
					refresh_lockable_rows
				end
				display_operation_results
			end
		end

	confirm_lock (rows: LIST [PACKING_PROJ_MULTI_COLUMN_LIST_ROW] wants_assign: BOOLEAN)
			-- Get user confirmation for Lock request
		local
			msg: STRING
		do
			msg := rows.count.out + " " + su.number (rows.count, "project", "projects") + " will be locked against customer update"
			if wants_assign then
				msg.append (" and assigned to this run")
			elseif has_run_context then
				msg.append (" but will not be assigned to this run")
			else
				msg.append (" but will not be assigned to any run")
			end
			msg.append (Proceed_text)
			confirm (msg)
		end

	try_to_lock_projects (list: LIST [PACKING_PROJECTS_MROW] wants_assign: BOOLEAN)
			-- Try to lock the projects in `list'.
			-- Leave description of results in `messages'
		local
			l_locked_projects: like list
		do
			create {ARRAYED_LIST [PACKING_PROJECTS_MROW]} l_locked_projects.make (list.count)
			initialize_counts
			list.do_all (agent try_to_lock_project_rows (?, l_locked_projects))
			fetch_fab_pool_and_setup_table
			create {ARRAYED_LIST [PACKING_PROJECTS_ROW]} locked_projects.make (list.count)
			try_to_do_action (l_locked_projects, agent try_to_modify_project (?, "lockable", True, wants_assign, True, locked_projects), agent finish_lock, "locked")
		end

	finish_lock
		do
			if not locked_projects.is_empty then
				current_version.update_dies_for_projects (locked_projects)
				compute_compatible_processes (current_version.added_projects)
				assign_processes_if_needed (locked_projects)
				-- FIXME: should be current_version.updated_dies
				projects_updated_actions.call ([current_version.added_dies])
			end
		end

	refresh_locks
			-- Refresh contents of Lock tab of notebook
		do
			if is_active then
				disable_active
				lockable_projects_changed := True
					-- Force refresh
				refresh_lockable_rows
				enable_active
			end
		end

	refresh_lockable_rows
			-- Refresh contents of Lock tab of notebook
		do
			if lockable_projects_changed then
				lockable_rows := packing.selected_unlocked_projects
				build_list (lockable_projects, lockable_rows)
				lockable_projects.sort_on_column (lockable_projects.last_sort_position)
				lockable_projects_changed := False
			end
		end

	unlock_projects
			-- Try to unlock projects
		do
			do_operation_if_active (agent unlock_projects_request)
		end

	unlock_projects_request
		local
			rows: LIST [PACKING_PROJ_MULTI_COLUMN_LIST_ROW]
		do
			unlockable_projects.rebuild_selected_rows
			rows := unlockable_projects.selected_rows
			if rows.is_empty then
				report_no_rows_selected
			else
				confirm_unlock (rows)
				try_to_unlock_projects (project_rows (rows))
				if ok_count > 0 then
					unlockable_projects_changed := True
					lockable_projects_changed := True
					deassignable_projects_changed := True
					assignable_projects_changed := True
					status_projects_changed := True
					refresh_unlockable_rows
				end
				display_operation_results
			end
		end

	confirm_unlock (rows: LIST [PACKING_PROJ_MULTI_COLUMN_LIST_ROW])
			-- Get user confirmation for Unlock request
		local
			msg: STRING
		do
			msg := rows.count.out + " " + su.number (rows.count, "project", "projects") + " will be unlocked and deassigned from run%Nbut will remain included in the packing"
			msg.append (Proceed_text)
			confirm (msg)
		end

	try_to_unlock_projects (list: LIST [PACKING_PROJECTS_MROW])
			-- Try to unlock the projects in `list'.
			-- Leave description of results in `messages'
		do
			initialize_counts
			try_to_do_action (list, agent try_to_unlock_project, Void, "unlocked")
		end

	try_to_unlock_project (p: PACKING_PROJECTS_MROW)
			-- Try to unlock `p'.
			-- Append description of results to
			-- `errors', `warnings' and `notes'
		local
			ifi: IC_FAB_INSTANCES_MROW
			cid: CUST_IC_DESIGNS_MROW
			dno: INTEGER
			ok: BOOLEAN
		do
			p.uncache_foreign_keys
			dno := p.design_no
			ok := False
			try_to_lock_cid_and_ifi (p)
			if last_lock_ok then
				cid := p.cust_ic_design
				ifi := p.fab_instance
				if has_run_context then
					if ifi.is_null_run_id then
						ok := True
					elseif ifi.run_id.is_equal (run_id) then
						if ifi.is_reworkable then
							add_error (dno, Void, "in rework")
						else
							ok := True
						end
					else
						add_error (dno, Void, "assigned to run " + ifi.run_id)
					end
				else
					if ifi.is_null_run_id then
						ok := True
					else
						add_error (dno, Void, "assigned to run " + ifi.run_id)
					end
				end
			end
			if ok then
				add_ok (dno)
				ifi.clear_pending_for_run
				ifi.clear_run_id
				cid.unfreeze
			end
			last_ok := ok
		end

	refresh_unlocks
			-- Refresh contents of Unlock tab of notebook
		do
			if is_active then
				disable_active
				unlockable_projects_changed := True
					-- Force refresh
				refresh_unlockable_rows
				enable_active
			end
		end

	refresh_unlockable_rows
			-- Refresh contents of Unlock tab of notebook
		do
			if unlockable_projects_changed then
				unlockable_rows := packing.selected_locked_projects
				build_list (unlockable_projects, unlockable_rows)
				unlockable_projects.sort_on_column (unlockable_projects.last_sort_position)
				unlockable_projects_changed := False
			end
		end

	assign_projects
			-- Try to assign projects to run
		do
			do_operation_if_active (agent assign_projects_request)
		end

	assign_projects_request
		local
			rows: LIST [PACKING_PROJ_MULTI_COLUMN_LIST_ROW]
		do
			assignable_projects.rebuild_selected_rows
			rows := assignable_projects.selected_rows
			if rows.is_empty then
				report_no_rows_selected
			else
				confirm_assign (rows)
				try_to_assign_projects (project_rows (rows))
				if ok_count > 0 then
					assignable_projects_changed := True
					deassignable_projects_changed := True
					status_projects_changed := True
					refresh_assignable_rows
				end
				display_operation_results
			end
		end

	confirm_assign (rows: LIST [PACKING_PROJ_MULTI_COLUMN_LIST_ROW])
			-- Get user confirmation for Assign request
		local
			msg: STRING
		do
			msg := rows.count.out + " " + su.number (rows.count, "project", "projects") + " which " + su.number (rows.count, "is", "are") + " already included in packing and locked against customer update%Nwill be assigned to this run"
			msg.append (Proceed_text)
			confirm (msg)
		end

	try_to_assign_projects (list: LIST [PACKING_PROJECTS_MROW])
			-- Try to assign the projects in `list'.
			-- Leave description of results in `messages'
		do
			initialize_counts
			try_to_do_action (list, agent try_to_assign_project, Void, "assigned to run")
		end

	try_to_assign_project (p: PACKING_PROJECTS_MROW)
			-- Try to assign `p' to run
			-- Append description of results to
			-- `errors', `warnings' and `notes'
		local
			ifi: IC_FAB_INSTANCES_MROW
			dno: INTEGER
		do
			p.uncache_foreign_keys
			dno := p.design_no
			try_to_lock_fab_instance (p)
			if last_lock_ok then
				ifi := p.fab_instance
				if ifi.can_be_assigned_to_run (run_id) then
					ifi.assign_to_run_if_unassigned (run_id)
					add_ok (dno)
					last_ok := True
				else
					add_error (dno, Void, "assigned to run " + ifi.run_id)
					last_ok := False
				end
			end
		end

	refresh_assignments
			-- Refresh contents of Assign tab of notebook
		do
			if is_active then
				disable_active
				assignable_projects_changed := True
					-- Force refresh
				refresh_assignable_rows
				enable_active
			end
		end

	refresh_assignable_rows
			-- Refresh contents of Assign tab of notebook
		do
			if assignable_projects_changed and has_run_context then
				assignable_rows := packing.selected_locked_unassigned_projects (run_id)
				build_list (assignable_projects, assignable_rows)
				assignable_projects.sort_on_column (assignable_projects.last_sort_position)
				assignable_projects_changed := False
			end
		end

	deassign_projects
			-- Try to deassign projects from run
		do
			do_operation_if_active (agent deassign_projects_request)
		end

	deassign_projects_request
		local
			rows: LIST [PACKING_PROJ_MULTI_COLUMN_LIST_ROW]
		do
			deassignable_projects.rebuild_selected_rows
			rows := deassignable_projects.selected_rows
			if rows.is_empty then
				report_no_rows_selected
			else
				confirm_deassign (rows)
				try_to_deassign_projects (project_rows (rows))
				if ok_count > 0 then
					deassignable_projects_changed := True
					assignable_projects_changed := True
					status_projects_changed := True
					refresh_deassignable_rows
				end
				display_operation_results
			end
		end

	confirm_deassign (rows: LIST [PACKING_PROJ_MULTI_COLUMN_LIST_ROW])
			-- Get user confirmation for Deassign request
		local
			msg: STRING
		do
			msg := rows.count.out + " " + su.number (rows.count, "project", "projects") + " will be deassigned from this run%Nbut will still be included in the packing and locked against customer update"
			msg.append (Proceed_text)
			confirm (msg)
		end

	try_to_deassign_projects (list: LIST [PACKING_PROJECTS_MROW])
			-- Try to deassign the projects in `list'.
			-- Leave description of results in `messages'
		do
			initialize_counts
			try_to_do_action (list, agent try_to_deassign_project, Void, "deassigned from run")
		end

	try_to_deassign_project (p: PACKING_PROJECTS_MROW)
			-- Try to deassign `p' from run.
			-- Append description of results to
			-- `errors', `warnings' and `notes'
		local
			ifi: IC_FAB_INSTANCES_MROW
			dno: INTEGER
		do
			p.uncache_foreign_keys
			dno := p.design_no
			try_to_lock_fab_instance (p)
			if last_lock_ok then
				ifi := p.fab_instance
				if ifi.is_null_run_id then
					add_ok (dno)
					last_ok := True
				elseif ifi.run_id.is_equal (run_id) then
					if ifi.is_reworkable then
						add_error (dno, Void, "in rework")
					else
						add_ok (dno)
						ifi.clear_run_id
						last_ok := True
					end
				else
					add_error (dno, Void, "assigned to run " + ifi.run_id)
					last_ok := False
				end
			end
		end

	refresh_deassignments
			-- Refresh contents of Deassign tab of notebook
		do
			if is_active then
				disable_active
				deassignable_projects_changed := True
					-- Force refresh
				refresh_deassignable_rows
				enable_active
			end
		end

	refresh_deassignable_rows
			-- Refresh contents of Deassign tab of notebook
		do
			if deassignable_projects_changed and has_run_context then
				deassignable_rows := packing.selected_locked_assigned_projects (run_id)
				build_list (deassignable_projects, deassignable_rows)
				deassignable_projects.sort_on_column (deassignable_projects.last_sort_position)
				deassignable_projects_changed := False
			end
		end

	compare_projects
			-- Compare snapshots of chosen projects to
			-- their current data and report differences
		do
			if is_active then
				disable_active
				-- FIXME: implement
				inform ("Project comparison not implemented yet")
				enable_active
			end
		end

	update_projects
			-- Update snapshots of chosen projects from
			-- their current data
		do
			inform ("The Update function is temporarily disabled until it is fixed.%NAs a workaround, Remove the project(s) you want to update and then Add them back.%NNote that the projects will lose their positions if you do this.")
			-- FIXME: add this back in when Update plays nicely
			-- with graphical packer
			-- do_operation_if_active (agent update_projects_request)
		end

	update_projects_request
		local
			rows: LIST [PACKING_PROJ_MULTI_COLUMN_LIST_ROW]
		do
			updatable_projects.rebuild_selected_rows
			rows := updatable_projects.selected_rows
			if rows.is_empty then
				report_no_rows_selected
			else
				confirm_update (rows)
				try_to_update_projects (project_rows (rows))
				if ok_count > 0 then
					status_projects_changed := True
					deselectable_projects_changed := True
					selection_projects_changed := True
					lockable_projects_changed := True
					unlockable_projects_changed := True
					assignable_projects_changed := True
					deassignable_projects_changed := True
					updatable_projects_changed := True
					refresh_updatable_rows
				end
				display_operation_results
			end
		end

	confirm_update (rows: LIST [PACKING_PROJ_MULTI_COLUMN_LIST_ROW])
			-- Get user confirmation for Update request
		local
			msg: STRING
		do
			msg := rows.count.out + " " + su.number (rows.count, "project", "projects") + " will be updated with current information.%NThe %"locked%" and %"assigned%" status will not be affected"
			msg.append (Proceed_text)
			confirm (msg)
		end

	try_to_update_projects (list: LIST [PACKING_PROJECTS_MROW])
			-- Try to update the projects in `list' with
			-- current information.
			-- Leave description of results in `messages'
		local
			save: BOOLEAN
			l_locked_projects: like list
		do
			create {ARRAYED_LIST [PACKING_PROJECTS_MROW]} l_locked_projects.make (list.count)
			initialize_counts
			list.do_all (agent try_to_lock_project_rows (?, l_locked_projects))
			fetch_fab_pool_and_setup_table
			create {ARRAYED_LIST [PACKING_PROJECTS_ROW]} updated_projects.make (list.count)
			save := allow_incompat_test_vehicles
			allow_incompat_test_vehicles := True
			try_to_do_action (l_locked_projects, agent try_to_modify_project (?, "updatable", False, False, True, updated_projects), agent finish_update, "updated")
			allow_incompat_test_vehicles := save
		end

	finish_update
		do
			if not updated_projects.is_empty then
				current_version.update_dies_for_projects (updated_projects)
				compute_compatible_processes (current_version.added_projects)
				assign_processes_if_needed (updated_projects)
				-- FIXME: should be current_version.updated_dies
				projects_updated_actions.call ([current_version.added_dies])
			end
		end

	refresh_update
			-- Refresh contents of Status tab of notebook
		do
			if is_active then
				disable_active
				updatable_projects_changed := True
					-- Force refresh
				refresh_updatable_rows
				enable_active
			end
		end

	refresh_updatable_rows
			-- Refresh contents of Update tab of notebook
		local
			wants_locked_cust, wants_unlocked_cust, wants_tv: BOOLEAN
			locked_cust_rows, unlocked_cust_rows, tv_rows: like updatable_rows
		do
			if updatable_projects_changed then
				wants_locked_cust := update_locked_cust_button.is_selected
				wants_unlocked_cust := update_unlocked_cust_button.is_selected
				wants_tv := update_test_vehicle_button.is_selected
				updatable_label.set_text (updatable_label_text ("Currently Included", wants_locked_cust, wants_unlocked_cust, wants_tv))
				if wants_locked_cust then
					locked_cust_rows := packing.selected_projects_matching_filter (agent {PACKING_PROJECTS_MROW}.is_locked_customer_project)
				else
					locked_cust_rows := empty_mod_project_list (0)
				end
				if wants_unlocked_cust then
					unlocked_cust_rows := packing.selected_projects_matching_filter (agent {PACKING_PROJECTS_MROW}.is_unlocked_customer_project)
				else
					unlocked_cust_rows := empty_mod_project_list (0)
				end
				if wants_tv then
					tv_rows := packing.selected_projects_matching_filter (agent {PACKING_PROJECTS_MROW}.is_test_vehicle)
				else
					tv_rows := empty_mod_project_list (0)
				end
				updatable_rows := empty_mod_project_list (locked_cust_rows.count + unlocked_cust_rows.count + tv_rows.count)
				updatable_rows.append (locked_cust_rows)
				updatable_rows.append (unlocked_cust_rows)
				updatable_rows.append (tv_rows)
				build_list (updatable_projects, updatable_rows)
				updatable_projects.sort_on_column (updatable_projects.last_sort_position)
				updatable_projects_changed := False
			end
		end

	compute_compatible_processes (list: LIST [PACKING_PROJECTS_MROW])
			-- Compute list of compatible processes for every
			-- customer project in `list'
		local
			cust_list: like list
		do
			create {ARRAYED_LIST [PACKING_PROJECTS_MROW]} cust_list.make (list.count)
			list.do_if (agent add_cust_project (?, cust_list), agent {PACKING_PROJECTS_MROW}.is_customer_project)
			current_version.compute_compatible_processes_for_list (cust_list)
		end

	add_cust_project (p: PACKING_PROJECTS_MROW list: like selected_projects)
			-- Add `p' to `list'
		do
			list.extend (p)
		end

	assign_processes_if_needed (list: like selected_projects)
			-- If there are any customer projects in `list',
			-- then the set of customer projects (or the
			-- properties of a customer project) has changed,
			-- so assign a process to each customer project
		do
			if list.there_exists (agent {PACKING_PROJECTS_MROW}.is_customer_project) then
				current_version.assign_process_to_projects
				processes_were_assigned := True
			end
		end

	row_to_record (row: EV_MULTI_COLUMN_LIST_ROW): PACKING_PROJ_MULTI_COLUMN_LIST_ROW
			-- Function which translates `row' to object of
			-- type PACKING_PROJ_MULTI_COLUMN_LIST_ROW
		do
			Result ?= row
			check
				result_not_void: Result /= Void
			end
		end

	current_tab_data: PACKING_PROJ_NOTEBOOK_TAB_DATA
			-- Data associated with currently selected notebook tab
		do
			Result ?= notebook.selected_item.data
			check
				result_not_void: Result /= Void
			end
		ensure
			result_not_void: Result /= Void
		end

	current_tab_refresh_agent: PROCEDURE [ANY, TUPLE]
			-- Agent to refresh rows in currently selected
			-- notebook tab
		do
			Result := current_tab_data.refresh_agent
		end

	current_tab_selection_list: MULTI_COLUMN_LIST [PACKING_PROJ_MULTI_COLUMN_LIST_ROW]
			-- Multi-column list in currently selected notebook
			-- tab, to which Select menu items apply (if any)
		do
			Result := current_tab_data.selection_list
		end

	current_tab_category: STRING
			-- Category of currently selected notebook tab
		do
			Result := current_tab_data.category
		end

	empty_project_list (n: INTEGER): LIST [PACKING_PROJECTS_ROW]
			-- Empty list of packing projects with capacity `n'
		do
			create {ARRAYED_LIST [PACKING_PROJECTS_ROW]} Result.make (n)
		end

	empty_mod_project_list (n: INTEGER): LIST [PACKING_PROJECTS_MROW]
			-- Empty list of modifiable packing projects
			-- with capacity `n'
		do
			create {ARRAYED_LIST [PACKING_PROJECTS_MROW]} Result.make (n)
		end

	project_rows (list: LIST [PACKING_PROJ_MULTI_COLUMN_LIST_ROW]): LIST [PACKING_PROJECTS_MROW]
			-- Project rows associated with elements of `list'
		do
			create {ARRAYED_LIST [PACKING_PROJECTS_MROW]} Result.make (list.count)
			list.do_all (agent add_project_row (?, Result))
		end

	add_project_row (p: PACKING_PROJ_MULTI_COLUMN_LIST_ROW list: LIST [PACKING_PROJECTS_ROW])
			-- Add project associated with `p' to `list'
		local
			proj: PACKING_PROJECTS_ROW
		do
			proj ?= p.data
			check
				proj_not_void: proj /= Void
			end
			list.extend (proj)
		end

	process_select_lock_change
			-- Process change to `selection_lock_button'
		do
			if not selection_lock_button.is_selected and selection_assign_button.is_selected then
				selection_assign_button.disable_select
			end
		end

	process_select_assign_change
			-- Process change to `selection_assign_button'
		do
			if selection_assign_button.is_selected and not selection_lock_button.is_selected then
				selection_lock_button.enable_select
			end
		end

	process_deselect_unlock_change
			-- Process change to `deselection_unlock_button'
		do
			if deselection_unlock_button.is_selected and not deselection_deassign_button.is_selected then
				deselection_deassign_button.enable_select
			elseif not deselection_unlock_button.is_selected and not has_run_context and deselection_deassign_button.is_selected then
				deselection_deassign_button.disable_select
			end
		end

	process_deselect_deassign_change
			-- Process change to `deselection_deassign_button'
		do
			if not deselection_deassign_button.is_selected and deselection_unlock_button.is_selected then
				deselection_unlock_button.disable_select
			end
		end

	report_no_rows_selected
		do
			inform ("You must select one or more rows first")
		end

	report_need_one_row
		do
			inform ("You must select exactly one row first")
		end

	anchored_columns: ARRAYED_LIST [MULTI_COLUMN_LIST_COLUMN [PACKING_PROJ_MULTI_COLUMN_LIST_ROW]]
			-- For anchoring only
		do
		ensure
			result_void: Result = Void
		end
	
	full_status_columns: like anchored_columns
			-- Full list of columns for Status tab
		do
			create Result.make (15)
			Result.extend (fab_instance_type_column)
			Result.extend (design_no_column)
			Result.extend (design_name_column)
			Result.extend (export_control_type_column)
			Result.extend (acct_no_column)
			Result.extend (account_type_column)
			Result.extend (org_short_id_column)
			Result.extend (date_time_fab_requested_column)
			Result.extend (size_x_column)
			Result.extend (size_y_column)
			Result.extend (overhead_style_column)
			Result.extend (is_included_column)
			Result.extend (is_locked_column)
			if has_run_context then
				Result.extend (is_assigned_column)
			end
			Result.extend (assigned_run_id_column)
			Result.extend (telephone_column)
		end
			
	full_selectable_columns: like anchored_columns
			-- Full list of columns for selectable list in
			-- Select tab
		do
			create Result.make (15)
			Result.extend (fab_instance_type_column)
			Result.extend (design_no_column)
			Result.extend (design_name_column)
			Result.extend (assigned_run_id_column)
			Result.extend (export_control_type_column)
			Result.extend (acct_no_column)
			Result.extend (account_type_column)
			Result.extend (org_short_id_column)
			Result.extend (run_date_requested_column)
			Result.extend (date_time_fab_requested_column)
			Result.extend (size_x_column)
			Result.extend (size_y_column)
			Result.extend (overhead_style_column)
			Result.extend (telephone_column)
		end
			
	full_unselectable_columns: like anchored_columns
			-- Full list of columns for unselectable list in
			-- Select tab
		do
			create Result.make (15)
			Result.extend (fab_instance_type_column)
			Result.extend (design_no_column)
			Result.extend (design_name_column)
			Result.extend (assigned_run_id_column)
			Result.extend (export_control_type_column)
			Result.extend (run_type_column)
			Result.extend (acct_no_column)
			Result.extend (account_type_column)
			Result.extend (org_short_id_column)
			-- Result.extend (run_date_requested_column)
			-- Result.extend (date_time_fab_requested_column)
			-- Result.extend (size_x_column)
			-- Result.extend (size_y_column)
			-- Result.extend (telephone_column)
			Result.extend (reason_unselectable_column)
		end
			
	full_standard_columns: like anchored_columns
			-- Full list of columns for standard list (used
			-- by most multi-column lists)
		do
			create Result.make (12)
			Result.extend (fab_instance_type_column)
			Result.extend (design_no_column)
			Result.extend (design_name_column)
			Result.extend (assigned_run_id_column)
			Result.extend (export_control_type_column)
			Result.extend (acct_no_column)
			Result.extend (account_type_column)
			Result.extend (org_short_id_column)
			Result.extend (date_time_fab_requested_column)
			Result.extend (size_x_column)
			Result.extend (size_y_column)
			Result.extend (overhead_style_column)
			Result.extend (telephone_column)
		end
			
	basic_standard_columns: like anchored_columns
			-- Basic list of columns for standard list (used
			-- by most multi-column lists)
		do
			create Result.make (6)
			Result.extend (design_no_column)
			Result.extend (acct_no_column)
			Result.extend (account_type_column)
			Result.extend (org_short_id_column)
			Result.extend (size_x_column)
			Result.extend (size_y_column)
			Result.extend (overhead_style_column)
			Result.extend (is_included_column)
			Result.extend (is_locked_column)
		end
	
	contact_standard_columns: like anchored_columns
			-- List of columns for contacting designer
		do
			create Result.make (6)
			Result.extend (design_no_column)
			Result.extend (design_name_column)
			Result.extend (acct_no_column)
			Result.extend (account_type_column)
			Result.extend (org_short_id_column)
			Result.extend (telephone_column)
			Result.extend (admin_liaison_name_column)
			Result.extend (admin_liaison_net_address_column)
		end
	
	fab_instance_type_column: FAB_INSTANCE_TYPE_COLUMN [PACKING_PROJ_MULTI_COLUMN_LIST_ROW]
		do
			create Result
		end

	design_no_column: DESIGN_NO_COLUMN [PACKING_PROJ_MULTI_COLUMN_LIST_ROW]
		do
			create Result
		end

	export_control_type_column: EXPORT_CONTROL_TYPE_COLUMN [PACKING_PROJ_MULTI_COLUMN_LIST_ROW]
		do
			create Result
		end

	run_type_column: RUN_TYPE_COLUMN [PACKING_PROJ_MULTI_COLUMN_LIST_ROW]
		do
			create Result
		end

	acct_no_column: ACCT_NO_COLUMN [PACKING_PROJ_MULTI_COLUMN_LIST_ROW]
		do
			create Result
		end

	account_type_column: ACCOUNT_TYPE_COLUMN [PACKING_PROJ_MULTI_COLUMN_LIST_ROW]
		do
			create Result
		end

	org_short_id_column: ORG_SHORT_ID_COLUMN [PACKING_PROJ_MULTI_COLUMN_LIST_ROW]
		do
			create Result
		end

	run_date_requested_column: RUN_DATE_REQUESTED_COLUMN [PACKING_PROJ_MULTI_COLUMN_LIST_ROW]
		do
			create Result
		end

	date_time_fab_requested_column: DATE_TIME_FAB_REQUESTED_COLUMN [PACKING_PROJ_MULTI_COLUMN_LIST_ROW]
		do
			create Result
		end

	overhead_style_column: OVERHEAD_STYLE_COLUMN [PACKING_PROJ_MULTI_COLUMN_LIST_ROW]
		do
			create Result
		end

	size_x_column: SIZE_X_COLUMN [PACKING_PROJ_MULTI_COLUMN_LIST_ROW]
		do
			create Result
		end

	size_y_column: SIZE_Y_COLUMN [PACKING_PROJ_MULTI_COLUMN_LIST_ROW]
		do
			create Result
		end

	telephone_column: TELEPHONE_COLUMN [PACKING_PROJ_MULTI_COLUMN_LIST_ROW]
		do
			create Result
		end

	reason_unselectable_column: REASON_UNSELECTABLE_COLUMN [PACKING_PROJ_MULTI_COLUMN_LIST_ROW]
		do
			create Result
		end

	admin_liaison_name_column: ADMIN_LIAISON_NAME_COLUMN [PACKING_PROJ_MULTI_COLUMN_LIST_ROW]
		do
			create Result
		end

	admin_liaison_net_address_column: ADMIN_LIAISON_NET_ADDRESS_COLUMN [PACKING_PROJ_MULTI_COLUMN_LIST_ROW]
		do
			create Result
		end

	design_name_column: DESIGN_NAME_COLUMN [PACKING_PROJ_MULTI_COLUMN_LIST_ROW]
		do
			create Result
		end

	assigned_run_id_column: RUN_ID_COLUMN [PACKING_PROJ_MULTI_COLUMN_LIST_ROW]
		do
			create Result
		end

	is_included_column: IS_INCLUDED_COLUMN [PACKING_PROJ_MULTI_COLUMN_LIST_ROW]
		do
			create Result
		end

	is_locked_column: IS_LOCKED_COLUMN [PACKING_PROJ_MULTI_COLUMN_LIST_ROW]
		do
			create Result
		end

	is_assigned_column: IS_ASSIGNED_COLUMN [PACKING_PROJ_MULTI_COLUMN_LIST_ROW]
		do
			create Result
		end

	has_run_context: BOOLEAN
			-- Are we in the context of a run?
		do
			Result := packing_interface.has_run_context
		end

	run_id: STRING
			-- Run ID of associated run
		require
			has_run_context: has_run_context
		do
			Result := packing_interface.run_id
		end

	run_id_or_void: STRING
			-- Run ID of associated run or Void if none
		do
			if has_run_context then
				Result := run_id
			else
				Result := Void
			end
		end

	list_title_font: EV_FONT
			-- Font for titles above multi-column lists
		do
			create Result
			Result.set_height (Bigger_font_height)
		end

	do_operation_if_active (p: PROCEDURE [ANY, TUPLE])
			-- If active, create operation to do `p', catching
			-- aborts
		do
			if is_active then
				disable_active
				create operation.make (p)
				operation.enable_abort_caught
				operation.execute
				enable_active
			end
		end

	try_to_do_action (list: LIST [PACKING_PROJECTS_MROW] item_action: PROCEDURE [ANY, TUPLE [PACKING_PROJECTS_MROW]] final_action: PROCEDURE [ANY, TUPLE] descr: STRING)
			-- Try to do `item_action' to every item of
			-- list, where action is described by `descr'.
			-- Then do `final_action' if not Void.  Leave
			-- description of results in `messages' Do the
			-- action with condition handler which first
			-- does database rollback and then calls
			-- original condition handler (so no locks
			-- will be held while waiting for user to
			-- respond)
		require
			list_not_void: list /= Void
			item_action_not_void: item_action /= Void
			descr_not_void: descr /= Void
		local
			old_handler: like condition_handler
			new_handler: DYNAMIC_CONDITION_HANDLER
		do
			old_handler := condition_handler
			create new_handler.make_with_action (agent database_rollback_handler (old_handler, ?))
			condition_handler_cell.put (new_handler)
			list.do_all (item_action)
			build_messages (descr)
			if final_action /= Void then
				connection.post
					-- Flush changes to fab instance and
					-- cust IC design to database so
					-- `final_action' will see them
				final_action.call ([])
			end
			connection.commit
			condition_handler_cell.put (old_handler)
			if processes_were_assigned then
				current_version.report_process_warnings
			end
		rescue
			if old_handler /= Void then
				condition_handler_cell.put (old_handler)
			end
			connection.rollback
		end

	database_rollback_handler (old_handler: like condition_handler cond: APPLICATION_CONDITION)
		do
			connection.rollback
			old_handler.handle (cond)
		end

	initialize_counts
		do
			cust_project_count := 0
			ok_count := 0
			processes_were_assigned := False
			create errors.make (5)
			create warnings.make (5)
			create notes.make (5)
		end

	fetch_fab_pool
			-- Fetch all projects in fab pool for packing's
			-- wafer techn whose design accept state is PASSED
			-- and put in `fab_pool'
		local
			row: FAB_POOL_W_WAFER_TECHNS_ROW
		do
			create row.make_template
			fab_pool := row.passed_projects_for_wafer_techn (packing.wafer_techn, connection, expected_count.At_least_zero)
		end

	fetch_fab_pool_and_setup_table
			-- Fetch all projects in fab pool for packing's
			-- wafer techn whose design accept state is PASSED
			-- and put in `fab_pool'.  Put projects in hash
			-- table for fast lookup
		do
			fetch_fab_pool
			create fab_pool_projects.make (fab_pool.count)
			fab_pool.do_all (agent add_fab_pool_project)
		end

	add_fab_pool_project (p: FAB_POOL_W_WAFER_TECHNS_ROW)
			-- Add `p' to `fab_pool_projects'
		do
			fab_pool_projects.put (p, p.m_fab_no)
		end

	remove_included_cust_projects
			-- Remove from `fab_pool' all projects that are
			-- already included in the current packing version
		local
			list: like fab_pool
		do
			list := fab_pool.twin
			list.wipe_out
			fab_pool.do_all (agent add_unselected_cust_project (?, list))
			fab_pool := list
		end

	add_unselected_cust_project (proj: FAB_POOL_W_WAFER_TECHNS_ROW list: like fab_pool)
			-- Add `proj' to `list' if it is not already
			-- in current version of packing (lookup by m_fab_no)
		do
			if not current_version.has_cust_die (proj.m_fab_no) then
				list.extend (proj)
			end
		end

	fetch_test_vehicles
			-- Fetch all test vehicles which are compatible
			-- with wafer techn and put in `test_vehicles'
		local
			row: M_TEST_VEHICLES_ROW
		do
			create row.make_template
			test_vehicles := row.test_vehicles_for_wafer_techn (packing.wafer_techn, connection, expected_count.At_least_zero)
		end

	remove_included_test_vehicles
			-- Remove from `test_vehicles' all test vehicles that
			-- are  already included in the current packing
			-- version
		local
			list: like test_vehicles
		do
			list := test_vehicles.twin
			list.wipe_out
			test_vehicles.do_all (agent add_unselected_test_vehicle (?, list))
			test_vehicles := list
		end

	add_unselected_test_vehicle (tv: M_TEST_VEHICLES_ROW list: like test_vehicles)
			-- Add `tv' to `list' if it is not already
			-- in current version of packing (lookup by design_no)
		do
			if not current_version.has_test_vehicle (tv.design_no) then
				list.extend (tv)
			end
		end

	split_fab_pool
			-- Split the rows in `fab_pool' into
			-- `selectable_fab_pool' and `unselectable_fab_pool'
			-- based on whether or not they are selectable, as
			-- defined by the user's selection criteria
		require
			fab_pool_not_void: fab_pool /= Void
		local
		do
			create {ARRAYED_LIST [FAB_POOL_W_WAFER_TECHNS_ROW]} selectable_fab_pool.make (fab_pool.count)
			create {ARRAYED_LIST [FAB_POOL_W_WAFER_TECHNS_ROW]} unselectable_fab_pool.make (fab_pool.count)
			fab_pool.do_all (agent classify_fab_pool_project)
		end

	classify_fab_pool_project (p: FAB_POOL_W_WAFER_TECHNS_ROW)
			-- Append `p' to `selectable_fab_pool' if it is
			-- compatible with user-defined criteria and to
			-- `unselectable_fab_pool' otherwise
		require
			p_not_void: p /= Void
		do
			check_project_full_selectability (p)
			if p.is_selectable then
				selectable_fab_pool.extend (p)
			else
				unselectable_fab_pool.extend (p)
			end
		end

	check_project_full_selectability (p: FAB_POOL_W_WAFER_TECHNS_ROW)
			-- Check whether customer project `p' is fully
			-- selectable (including layer
			-- incompatibilities and lack of compatible
			-- process).
		require
			p_not_void: p /= Void
		do
			p.check_full_selectability (packing.packings_row, run_id_or_void, allow_wrong_type, allow_wrong_foundry, allow_no_access, allow_assigned_to_run, allow_uncharged)
		end

	split_test_vehicles
			-- Split the rows in `test_vehicles' into
			-- `selectable_test_vehicles' and
			-- `unselectable_test_vehicles'
			-- based on whether or not they have any layer
			-- incompatibilities
		require
			test_vehicles_not_void: test_vehicles /= Void
		do
			create {ARRAYED_LIST [M_TEST_VEHICLES_ROW]} selectable_test_vehicles.make (test_vehicles.count)
			create {ARRAYED_LIST [M_TEST_VEHICLES_ROW]} unselectable_test_vehicles.make (test_vehicles.count)
			test_vehicles.do_all (agent classify_test_vehicle)
		end

	classify_test_vehicle (tv: M_TEST_VEHICLES_ROW)
			-- Append `tv' to `selectable_test_vehicles' if it
			-- either has no layer incompatibilities with wafer
			-- techn or if `allow_incompat_test_vehicles' is true.
			-- Append it to `unselectable_test_vehicles' otherwise
		require
			tv_not_void: tv /= Void
		do
			-- Make sure wafer techn layer errors referenced first
			-- to ensure they are cached and won't have to be
			-- computed for a subsequent Filter operation
			if is_test_vehicle_selectable (tv) then
				selectable_test_vehicles.extend (tv)
			else
				unselectable_test_vehicles.extend (tv)
			end
		end

	is_test_vehicle_selectable (tv: M_TEST_VEHICLES_ROW): BOOLEAN
			-- Is test vehicle `tv' currently selectable?
		require
			tv_not_void: tv /= Void
		do
			Result := tv.is_selectable or else allow_incompat_test_vehicles
		end

	try_to_lock_cid_and_ifi (p: PACKING_PROJECTS_MROW)
			-- Try to lock both cust IC design row and
			-- fab instance for `p'.
			-- Append any errors to `errors' and set
			-- `last_lock_ok' to indicate whether lock succeeded
		do
			try_to_lock_cust_ic_design (p)
			if last_lock_ok then
				try_to_lock_fab_instance (p)
			end
		end

	try_to_lock_fab_instance (p: PACKING_PROJECTS_MROW)
			-- Try to lock fab instance for `p'
			-- Append any errors to `errors' and set
			-- `last_lock_ok' to indicate whether lock succeeded
		local
			ifi: IC_FAB_INSTANCES_MROW
			what: STRING
		do
			ifi := p.fab_instance
			what := "fab instance (" + p.m_fab_no.out + ")"
			last_lock_ok := False
			if ifi.is_old then
				try_to_lock_row (ifi, ifi.design_no, what)
			else
				add_error (p.design_no, what, "no longer exists")
			end
		end

	try_to_lock_cust_ic_design (p: PACKING_PROJECTS_MROW)
			-- Try to lock cust IC design row for `p'
			-- Append any errors to `errors' and set
			-- `last_lock_ok' to indicate whether lock succeeded
		local
			cid: CUST_IC_DESIGNS_MROW
		do
			cid := p.cust_ic_design
			try_to_lock_row (cid, cid.design_no, "cust IC design")
		end

	try_to_lock_row (row: DB_LOCKABLE_ROW dno: INTEGER descr: STRING)
			-- Try to lock `row' for design number `dno',
			-- described by `descr'.  Append any errors to
			-- `errors' and set `last_lock_ok' to
			-- indicate whether lock succeeded
		do
			last_lock_ok := False
			row.try_to_lock_nowait
			if row.is_locked_and_unchanged then
				last_lock_ok := True
			elseif row.is_locked_but_changed then
				add_error (dno, descr, "changed by another process after fetch")
			elseif row.is_busy then
				add_error (dno, descr, "locked by another process")
			elseif row.is_gone then
				add_error (dno, descr, "no longer exists")
			else
				add_error (dno, descr, "internal code error")
			end
		end

	add_ok (dno: INTEGER)
		do
			ok_count := ok_count + 1
		end

	add_error (dno: INTEGER what, msg: STRING)
			-- Add `msg' about `dno' when doing `what' to
			-- `errors'
		require
			msg_not_void: msg /= Void
		do
			add_message  (dno, what, msg, errors)
		end

	add_warning (dno: INTEGER what, msg: STRING)
			-- Add `msg' about `dno' when doing `what' to
			-- `warnings'
		require
			msg_not_void: msg /= Void
		do
			add_message  (dno, what, msg, warnings)
		end

	add_note (dno: INTEGER what, msg: STRING)
			-- Add `msg' about `dno' when doing `what' to `notes'
		require
			msg_not_void: msg /= Void
		do
			add_message (dno, what, msg, notes)
		end

	add_message (dno: INTEGER what, msg: STRING list: LIST [STRING])
			-- Add `msg' about `dno' when doing `what' to `list'
		require
			msg_not_void: msg /= Void
			list_not_void: list /= Void
		local
			s, descr: STRING
		do
			create s.make (50)
			s.append ("     ")
			s.append_integer (dno)
			s.append ("     ")
			create descr.make (50)
			if what /= Void then
				descr.append (what + " ")
			end
			descr.append (msg)
			if not descr.is_empty then
				descr.put (descr.item (1).as_upper, 1)
			end
			s.append (descr)
			list.extend (s)
		end

	build_messages (op: STRING)
			-- Build `messages' (to be shown to user).
			-- Add counts of successful and unsuccessful operations
		local
			list: like errors
			n: INTEGER
		do
			create list.make (errors.count + warnings.count + notes.count + 10)
			list.extend (ok_count.out + " " + su.number (ok_count, "project", "projects") + " " + op)
			if not errors.is_empty then
				n := errors.count
				list.extend (n.out + " " + su.number (n, "error", "errors"))
				list.extend ("")
				list.extend ("Errors:")
				list.extend ("")
				list.append (errors)
			end
			if not warnings.is_empty then
				list.extend ("")
				list.extend ("Warnings:")
				list.extend ("")
				list.append (warnings)
			end
			if not notes.is_empty then
				list.extend ("")
				list.extend ("Notes:")
				list.extend ("")
				list.append (notes)
			end
			list.extend ("")
			messages := su.joined_by (list, "%N")
		end

	display_operation_results
		do
			inform (messages)
		end

	handle_version_switch
			-- Handle change to a different packing version
		do
			set_all_changed
			if is_displayed then
				refresh_top_page
			end
		end

	set_all_changed
			-- Set "is changed" indicator for every
			-- notebook tab to true
		do
			status_projects_changed := True
			selection_projects_changed := True
			deselectable_projects_changed := True
			lockable_projects_changed := True
			unlockable_projects_changed := True
			assignable_projects_changed := True
			deassignable_projects_changed := True
			updatable_projects_changed := True
		end

	process_send
			-- Send nicely formatted text of list of projects
			-- in current notebook tab to observers
		local
			list: like current_tab_selection_list
		do
			if is_active then
				disable_active
				list := current_tab_selection_list
				send_actions.call (["list of " + current_tab_category + " projects", formatted_text (list.all_rows, list.visible_columns)])
				enable_active
			end
		end

	formatted_text (a_row_list: LIST [PACKING_PROJ_MULTI_COLUMN_LIST_ROW] a_column_list: LIST [MULTI_COLUMN_LIST_COLUMN [PACKING_PROJ_MULTI_COLUMN_LIST_ROW]]): STRING
			-- Nicely formatted text representing contents
			-- of `a_row_list', using just the columns in
			-- `a_column_list' in order
			-- in the Output window
		local
			l_formatter: MULTI_COLUMN_LIST_TEXT_FORMATTER [PACKING_PROJ_MULTI_COLUMN_LIST_ROW]
		do
			create l_formatter
			Result := l_formatter.formatted_text (a_row_list, a_column_list, "  ", True, True)
		end

	separated_box (box: EV_BOX): EV_BOX
			-- `box' with horizontal separators above and below it
		local
			sep: EV_HORIZONTAL_SEPARATOR
		do
			create {EV_VERTICAL_BOX} Result
			create sep
			Result.extend (sep)
			Result.disable_item_expand (sep)
			Result.extend (box)
			create sep
			Result.extend (sep)
			Result.disable_item_expand (sep)
		end

	new_multi_column_list: like status_projects
		do
			create Result
			Result.enable_multiple_selection
			Result.set_row_to_generic_function (agent row_to_record)
			Result.set_visibility_counts_included (False)
			Result.build_selection_status_box
		end

	current_version: PACKING_VERSIONS_MROW
			-- Current version of packing
		do
			Result := packing.current_version
		end

	connection: DB_CONNECTION
			-- Database connection to use
		do
			Result := session.connection
		end

	Design_no_position: INTEGER = 2
			-- Position of "design number" column in
			-- `status_projects', so we can sort on it

	notebook: EV_NOTEBOOK
			-- Notebook with tabs for various operations
			-- (select projects, deselect projects, lock projects,
			-- status, etc)

	status_box: EV_VERTICAL_BOX
			-- Box for Status tab of notebook

	selection_box: EV_VERTICAL_BOX
			-- Box for Select tab of notebook

	deselection_box: EV_VERTICAL_BOX
			-- Box for Deselect tab of notebook

	lock_box: EV_VERTICAL_BOX
			-- Box for Lock tab of notebook

	unlock_box: EV_VERTICAL_BOX
			-- Box for Unlock tab of notebook

	assignment_box: EV_VERTICAL_BOX
			-- Box for Assign tab of notebook

	deassignment_box: EV_VERTICAL_BOX
			-- Box for Deassign tab of notebook

	update_box: EV_VERTICAL_BOX
			-- Box for Update tab of notebook

	status_cust_die_button: EV_CHECK_BUTTON
			-- Check button to display customer projects
			-- in Status tab
	
	status_test_vehicle_button: EV_CHECK_BUTTON
			-- Check button to display test vehicles
			-- in Status tab
	
	status_included_button: EV_CHECK_BUTTON
			-- Check button to display projects included in packing
			-- in Status tab
	
	status_not_included_button: EV_CHECK_BUTTON
			-- Check button to display projects that were
			-- previously included in packing but are not
			-- currently included, in Status tab
	
	status_filter_button: EV_BUTTON
			-- Button to refresh Status tab contents
	
	status_send_button: EV_BUTTON
			-- Button to send tab contents to observers
	
	status_close_button: EV_BUTTON
			-- Button to close window in Status tab
	
	status_label: EV_LABEL
			-- Label to say what is displayed in Status tab
	
	selection_cust_die_button: EV_CHECK_BUTTON
			-- Check button to display customer projects
			-- in Select tab
	
	selection_test_vehicle_button: EV_CHECK_BUTTON
			-- Check button to display test vehicles
			-- in Select tab
	
	selection_lock_button: EV_CHECK_BUTTON
			-- Check button to lock selected projects
			-- in Select tab
	
	selection_assign_button: EV_CHECK_BUTTON
			-- Check button to assign selected projects to run
			-- in Select tab
	
	selection_bad_run_type_button: EV_CHECK_BUTTON
			-- Check button to include projects with incompatible
			-- run type in selectable list
	
	selection_wrong_foundry_button: EV_CHECK_BUTTON
			-- Check button to include projects that have an
			-- incompatible foundry restriction in selectable list
	
	selection_no_access_button: EV_CHECK_BUTTON
			-- Check button to include projects with access
			-- to wafer techn not granted in selectable list
	
	selection_already_assigned_button: EV_CHECK_BUTTON
			-- Check button to include projects already assigned
			-- to another run (but with no die id) in selectable
			-- list
	
	selection_not_charged_button: EV_CHECK_BUTTON
			-- Check button to include projects that have not
			-- yet been charged in selectable list
	
	selection_incompat_test_vehicle_button: EV_CHECK_BUTTON
			-- Check button to include test vehicles that have
			-- layer incompatibilities in the selectable list
	
	selection_select_button: EV_BUTTON
			-- Button to select projects in Select tab
	
	selection_filter_button: EV_BUTTON
			-- Button to filter projects in Select tab
	
	selection_send_button: EV_BUTTON
			-- Button to send list of selectable projects to 
			-- observers
	
	selection_refresh_button: EV_BUTTON
			-- Button to refresh Select tab contents
	
	selection_split_area: EV_VERTICAL_SPLIT_AREA
			-- Area to hold selectable and unselectable
			-- project lists in Select tab

	selectable_label: EV_LABEL
			-- Label to say what is displayed in Selectable
			-- area of Select tab
	
	unselectable_label: EV_LABEL
			-- Label to say what is displayed in Unselectable
			-- area of Select tab
	deselection_cust_die_button: EV_CHECK_BUTTON
			-- Check button to display customer projects
			-- in Deselect tab
	
	deselection_test_vehicle_button: EV_CHECK_BUTTON
			-- Check button to display test vehicles
			-- in Deselect tab
	
	deselection_unlock_button: EV_CHECK_BUTTON
			-- Check button to unlock selected projects
			-- in Deselect tab
	
	deselection_deassign_button: EV_CHECK_BUTTON
			-- Check button to deassign selected projects from run
			-- in Deselect tab
	
	deselection_deselect_button: EV_BUTTON
			-- Button to deselect projects in Deselect tab
	
	deselection_filter_button: EV_BUTTON
			-- Button to filter Deselect tab contents
	
	deselection_send_button: EV_BUTTON
			-- Button to send list of deselectable projects
			-- to observers
	
	deselection_close_button: EV_BUTTON
			-- Button to close window in Deselect tab
	
	deselectable_label: EV_LABEL
			-- Label to say what is displayed in
			-- Deselect tab
	
	lock_assign_button: EV_CHECK_BUTTON
			-- Check button to assign selected projects to run
			-- in Lock tab
	
	lock_lock_button: EV_BUTTON
			-- Button to lock projects in Lock tab
	
	lock_refresh_button: EV_BUTTON
			-- Button to refresh list of lockable projects in
			-- Lock tab
	
	lock_send_button: EV_BUTTON
			-- Button to send list of lockable projects to
			-- observers
	
	lock_close_button: EV_BUTTON
			-- Button to close window in Lock tab
	
	unlock_unlock_button: EV_BUTTON
			-- Button to unlock projects in Unlock tab
	
	unlock_refresh_button: EV_BUTTON
			-- Button to refresh list of currently locked projects
			-- in Unlock tab
	
	unlock_send_button: EV_BUTTON
			-- Button to send list of unlockable projects to
			-- observers
	
	unlock_close_button: EV_BUTTON
			-- Button to close window in Unlock tab
	
	assignment_assign_button: EV_BUTTON
			-- Button to assign projects to run in Assign tab
	
	assignment_refresh_button: EV_BUTTON
			-- Button to refresh list of currently assignable
			-- in Assign tab
	
	assignment_send_button: EV_BUTTON
			-- Button to send list of assignable projects to
			-- observers
	
	assignment_close_button: EV_BUTTON
			-- Button to close window in Assign tab
	
	deassignment_deassign_button: EV_BUTTON
			-- Button to deassign projects from run in Deassign tab
	
	deassignment_refresh_button: EV_BUTTON
			-- Button to refresh list of projects currently
			-- assigned to run in Deassign tab
	
	deassignment_send_button: EV_BUTTON
			-- Button to send list of deassignable projects to
			-- observers
	
	deassignment_close_button: EV_BUTTON
			-- Button to close window in Deassign tab
	
	update_locked_cust_button: EV_CHECK_BUTTON
			-- Check button to display locked customer projects
			-- in Update tab
	
	update_unlocked_cust_button: EV_CHECK_BUTTON
			-- Check button to display unlocked customer projects
			-- in Update tab
	
	update_test_vehicle_button: EV_CHECK_BUTTON
			-- Check button to display test vehicles
			-- in Update tab
	
	update_filter_button: EV_BUTTON
			-- Button to filter Update tab contents
	
	update_compare_button: EV_BUTTON
			-- Button to compare projects in Update tab
	
	update_update_button: EV_BUTTON
			-- Button to update snapshot of projects from
			-- current data in Update tab
	
	update_send_button: EV_BUTTON
			-- Button to send list of updatable projects to
			-- observers
	
	update_close_button: EV_BUTTON
			-- Button to close window in Update tab
	
	updatable_label: EV_LABEL
			-- Label to say what is displayed in Update tab
	
	status_projects: MULTI_COLUMN_LIST [PACKING_PROJ_MULTI_COLUMN_LIST_ROW]
			-- List of projects for Status tab of notebook,
			-- displayed in multi-column list
	
	selectable_projects: like status_projects
			-- List of selectable projects for Select tab
			-- of notebook, displayed in multi-column list
	
	unselectable_projects: like status_projects
			-- List of unselectable projects for Select
			-- tab of notebook, displayed in multi-column
			-- list
	
	deselectable_projects: like status_projects
			-- List of deselectable projects for Deselect
			-- tab of notebook, displayed in multi-column
			-- list
	
	lockable_projects: like status_projects
			-- List of lockable projects for Lock
			-- tab of notebook, displayed in multi-column
			-- list
	
	unlockable_projects: like status_projects
			-- List of projects which may be unlocked for Lock
			-- tab of notebook, displayed in multi-column
			-- list
	
	assignable_projects: like status_projects
			-- List of projects which may be assigned to
			-- run for Assign tab of notebook, displayed
			-- in multi-column list
	
	deassignable_projects: like status_projects
			-- List of projects which may be deassigned for Assign
			-- tab of notebook, displayed in multi-column
			-- list
	
	updatable_projects: like status_projects
			-- List of projects which may be compared and
			-- updated for Update tab of notebook,
			-- displayed in multi-column list
	
	status_rows: LIST [PACKING_PROJECTS_ROW]
			-- Rows for projects to be displayed in Status tab
	
	selectable_rows: LIST [PACKING_PROJECTS_MROW]
			-- Rows for projects to be displayed in Select tab
			-- as selectable
	
	unselectable_rows: LIST [PACKING_PROJECTS_MROW]
			-- Rows for projects to be displayed in Select tab
			-- as unselectable
	
	deselectable_rows: LIST [PACKING_PROJECTS_MROW]
			-- Rows for projects to be displayed in Deselect tab
	
	lockable_rows: LIST [PACKING_PROJECTS_MROW]
			-- Rows for projects to be displayed in Lock tab
	
	unlockable_rows: LIST [PACKING_PROJECTS_MROW]
			-- Rows for projects to be displayed in Unlock tab
	
	assignable_rows: LIST [PACKING_PROJECTS_MROW]
			-- Rows for projects to be displayed in Assign tab
	
	deassignable_rows: LIST [PACKING_PROJECTS_MROW]
			-- Rows for projects to be displayed in Deassign tab
	
	updatable_rows: LIST [PACKING_PROJECTS_MROW]
			-- Rows for projects to be displayed in Update tab
	
	status_projects_changed: BOOLEAN
			-- Have any changes that may affect projects shown
			-- in Status tab been made since last refresh?

	selection_projects_changed: BOOLEAN
			-- Have any changes that may affect projects shown
			-- in Select tab been made since last refresh?

	deselectable_projects_changed: BOOLEAN
			-- Have any changes that may affect projects shown
			-- in Deselect tab been made since last refresh?

	lockable_projects_changed: BOOLEAN
			-- Have any changes that may affect projects shown
			-- in Lock tab been made since last refresh?

	unlockable_projects_changed: BOOLEAN
			-- Have any changes that may affect projects shown
			-- in Unlock tab been made since last refresh?

	assignable_projects_changed: BOOLEAN
			-- Have any changes that may affect projects shown
			-- in Assign tab been made since last refresh?

	deassignable_projects_changed: BOOLEAN
			-- Have any changes that may affect projects shown
			-- in Deassign tab been made since last refresh?

	updatable_projects_changed: BOOLEAN
			-- Have any changes that may affect projects shown
			-- in Update tab been made since last refresh?

	allow_wrong_type: BOOLEAN
			-- Did user request that projects with wrong run
			-- type be made selectable anyway, in last Filter

	allow_wrong_foundry: BOOLEAN
			-- Did user request that projects whose foundry
			-- restriction does not include wafer techn's foundry
			-- be made selectable anyway, in last Filter

	allow_no_access: BOOLEAN
			-- Did user request that projects whose account has
			-- not been granted fab access to wafer techn
			-- be made selectable anyway, in last Filter

	allow_assigned_to_run: BOOLEAN
			-- Did user request that projects which are
			-- already assigned to another run be made
			-- selectable anyway, in last Filter

	allow_uncharged: BOOLEAN
			-- Did user request that projects which are
			-- not yet charged be made selectable anyway, in last
			-- Filter

	allow_incompat_test_vehicles: BOOLEAN
			-- Did user request that test vehicles which
			-- have layer incompatibilities with wafer
			-- techn be made selectable anyway, in last
			-- Filter

	fab_pool: LIST [FAB_POOL_W_WAFER_TECHNS_ROW]
			-- Projects in fab pool, as of last refresh

	selectable_fab_pool: like fab_pool
			-- Projects in fab pool as of last refresh
			-- that appear to be currently selectable

	unselectable_fab_pool: like fab_pool
			-- Projects in fab pool as of last refresh
			-- that appear to be currently unselectable

	fab_pool_projects: HASH_TABLE [FAB_POOL_W_WAFER_TECHNS_ROW, INTEGER]
			-- Projects in fab pool, indexed by m_fab_no

	test_vehicles: LIST [M_TEST_VEHICLES_ROW]
			-- Test vehicles compatible with wafer techn,
			-- as of last refresh

	selectable_test_vehicles: like test_vehicles
			-- Test vehicles that appear to be currently
			-- selectable as of last refresh

	unselectable_test_vehicles: like test_vehicles
			-- Test vehicles that appear to be currently
			-- unselectable as of last refresh

	Bigger_font_height: INTEGER = 20
			-- Height of bigger font

	Std_border_width: INTEGER = 5
			-- Standard width of border for rows with
			-- filters and options

	operation: ANONYMOUS_OPERATION

	messages: STRING
			-- All messages to be displayed to user after
			-- trying an operation (such as lock) for
			-- the chosen set of projects

	errors: ARRAYED_LIST [STRING]
			-- Error messages to be displayed to user after
			-- trying an operation

	warnings: ARRAYED_LIST [STRING]
			-- Warning messages to be displayed to user after
			-- trying an operation

	notes: ARRAYED_LIST [STRING]
			-- Informational messages to be displayed to user after
			-- trying an operation

	cust_project_count: INTEGER
			-- Number of customer projects involved in operation
			-- (for project selection only)

	ok_count: INTEGER
			-- Number of projects for which operation
			-- was successful

	fail_count: INTEGER
			-- Number of projects for which operation
			-- failed

	processes_were_assigned: BOOLEAN
			-- Were processes assigned to projects in last
			-- call to `try_to_do_action'?

	last_lock_ok: BOOLEAN
			-- Did last call to `try_to_lock_row'
			-- lock the row (and row is unchanged)?

	last_ok: BOOLEAN
			-- Did last call to routine that tried to
			-- change a project (e.g., lock, unlock, select etc)
			-- succeed?

	split_area_initialized: BOOLEAN
			-- Has split area in Select tab been initialized?

	selected_projects: LIST [PACKING_PROJECTS_ROW]
			-- Projects that were selected by last call to
			-- `try_to_select_projects'

	deselected_projects: LIST [PACKING_PROJECTS_ROW]
			-- Projects that were deselected by last call to
			-- `try_to_deselect_projects'

	locked_projects: like selected_projects
			-- Projects that were locked by last call to
			-- `try_to_lock_projects'

	updated_projects: like selected_projects
			-- Projects that were update by last call to
			-- `try_to_update_projects'

	Basic_column_format: STRING = "Basic"
			-- Used to display only a basic set of columns

	Contact_column_format: STRING = "Contact"
			-- Used to display columns useful when contacting
			-- designer

	Full_column_format: STRING = "Full"
			-- Used to display a full set of columns

	Explanation_for_select: STRING = "Selecting a project just includes it in the packing.%N"

	Explanation_for_lock: STRING = "Locking a project prevents the customer from updating it or cancelling fabrication.%N"

	Explanation_for_assign: STRING = "Assigning a project to a run prevents another run from claiming it.%N"

	Explanation_for_deselect: STRING = "Deselecting a project just removes it from the packing.%N"

	Explanation_for_deassign: STRING = "Deassigning a project from a run allows another run to claim it.%N"

	Explanation_for_unlock: STRING = "Unlocking a project allows the customer to update it or cancel fabrication.%N"

	Explanation_for_update: STRING = "Updating a project gets current information about a project.%N"

	Proceed_text: STRING = ".%N%NProceed"

end
