
indexing
	
	description: "Window that has most of the fields for a record from %
		%the PACKINGS table"
	author: "David Hollenberg"
	date: "November 7, 2005"

deferred class RC_PACKING_RECORD_WINDOW

inherit
	RC_PACKING_IFC
	WINDOW_INTERFACE
		redefine
			enable_active,
			disable_active
		end
	MOSIS_GRAPHICAL_UTILITIES
		export
			{NONE} all
		undefine
			default_create
		end
	PACKING_RECORD_UTILITIES
		export
			{NONE} all
		end
	NUMBER_UTILITIES_ACCESS
		export
			{NONE} all
		undefine
			default_create
		end
		
feature {NONE} -- Creation

	make (w: EV_TITLED_WINDOW)
			-- Create interface in window `w'
		require
			window_exists: w /= Void
			window_valid: not w.is_destroyed
		do
			make_interface
			make_proxy (w)
		end

feature -- Operations

	reset
			-- Move focus back into window
		do
			enable_active
		end

feature -- Modification

	enable_active
			-- Make interface active
		do
			precursor
			enable_buttons_sensitive
		end

	disable_active
			-- Make interface inactive
			-- (can look but cannot initiate most actions)
		do
			precursor
			disable_buttons_sensitive
		end

feature {NONE} -- Implementation

	enable_buttons_sensitive
			-- Make buttons sensitive
		deferred
		end

	disable_buttons_sensitive
			-- Make buttons insensitive
		deferred
		end

	build_interface
			-- Build interface
		do
			build_fields
			populate_fields
			set_field_properties
			add_menu_bar
			initialize_main_vbox
			add_buttons
			gui.set_box_background_recursive (main_vbox, background_color)
		end
		
	initialize_main_vbox
			-- Initialize `main_vbox'
		do
			create main_vbox
			main_vbox.set_border_width (user_interface.border_width)
			main_vbox.extend (fields_box)
			main_vbox.disable_item_expand (fields_box)

			extend (main_vbox)
			gui.add_cancel_accelerator (window, agent process_cancel)
			gui.do_once_on_idle (agent post_display_actions)
		end
		
	add_menu_bar
			-- Add menu bar to window
		local
			file_menu: EV_MENU
			exit_menu_item: EV_MENU_ITEM
			close_menu_item: EV_MENU_ITEM
			a_menu_bar: EV_MENU_BAR
		do	
			create file_menu.make_with_text ("File")
			
			create close_menu_item.make_with_text ("Close")
			close_menu_item.select_actions.extend (agent process_cancel)
			file_menu.extend (close_menu_item)
			
			create exit_menu_item.make_with_text ("Exit")
			exit_menu_item.select_actions.extend (gui.destroy_application_agent)
			file_menu.extend (exit_menu_item)
			
			create a_menu_bar
			a_menu_bar.extend (file_menu)
			set_menu_bar (a_menu_bar)
		end

	build_fields
			-- Build portion of window with fields
			-- in `fields_box'
		local
			cols: WIDGET_COLUMN_SET
		do
			create cols.make (2)
			cols.set_padding (user_interface.padding)
			cols.set_border_width (user_interface.border_width)
			
			create packing_id_field
			packing_id_field.set_capacity ({PACKINGS_ROW}.maxlen_packing_id)
			packing_id_field.set_minimum_width_in_characters (20)
			
			create creator_field
			creator_field.set_minimum_width_in_characters (10)
			
			create run_type_field
			run_type_field.set_minimum_width_in_characters (3)
			
			create educational_field
			create taxi_field
			create wafer_techn_field
			wafer_techn_field.set_minimum_width_in_characters (40)
			
			create sched_run_field
			sched_run_field.set_minimum_width_in_characters (10)
			
			create field_set_field
			field_set_field.set_minimum_width_in_characters (40)
			
			create step_pattern_field
			step_pattern_field.set_minimum_width_in_characters (40)
			
			create overhead_style_field
			overhead_style_field.set_minimum_width_in_characters (40)
			create other_border_field
			other_border_field.set_minimum_width_in_characters (10)
			create top_overhead_field
			top_overhead_field.set_minimum_width_in_characters (22)
			create intended_run_field
			intended_run_field.set_capacity ({PACKINGS_ROW}.maxlen_intended_run)
			intended_run_field.set_minimum_width_in_characters (10)
			
			create description_field
			description_field.set_capacity ({PACKINGS_ROW}.maxlen_descr)
			description_field.set_minimum_width_in_characters (40)
			
			
			cols.append_row (<< gui.label_with_text ("Packing ID"), packing_id_field >>)
			cols.append_row (<< gui.label_with_text ("Created By"), creator_field >>)
			cols.append_row (<< gui.label_with_text ("Run Type"), run_type_field >>)
			cols.append_row (<< gui.label_with_text ("Educational Run"), educational_field >>)
			cols.append_row (<< gui.label_with_text ("Taxi Run"), taxi_field >>)
			cols.append_row (<< gui.label_with_text ("Wafer Techn"), wafer_techn_field >>)
			cols.append_row (<< gui.label_with_text ("Scheduled Run"), sched_run_field >>)
			cols.append_row (<< gui.label_with_text ("Field Set"), field_set_field >>)
			cols.append_row (<< gui.label_with_text ("Field Step Pattern"), step_pattern_field >>)
			cols.append_row (<< gui.label_with_text ("Die Labeling Style"), overhead_style_field >>)
			cols.append_row (<< gui.label_with_text ("Other Side Overhead (White Space)"), other_border_field >>)
			cols.append_row (<< gui.label_with_text ("Top Side Overhead"), top_overhead_field >>)
			cols.append_row (<< gui.label_with_text ("Intended Run"), intended_run_field >>)
			cols.append_row (<< gui.label_with_text ("Description"), description_field >>)
			
			cols.disable_expand
			cols.align_column_right (1)
			cols.align_column_left (2)
			fields_box := cols.box
		end
		
	set_field_properties
			-- Set properties of fields
		deferred
		end
		
	populate_fields
			-- Populate some fields
		deferred
		end
		
	add_buttons
			-- Add buttons to `main_vbox'
		deferred
		end
		
	connection: DB_CONNECTION
		do
			Result := session.connection
		end

	post_display_actions
		do
			reset
		end

	process_cancel
		do
			if is_active then
				disable_active
				cancel_actions.call ([])
			end
		end

	main_vbox: EV_VERTICAL_BOX
			-- Main box for window

	fields_box: EV_BOX
			-- Box with fields for run id, etc.

	packing_id_field: TEXT_FIELD
			-- Field for packing ID

	creator_field: EV_COMBO_BOX
			-- Field for creator of packing

	run_type_field: EV_COMBO_BOX
			-- Field for run type

	taxi_field: EV_CHECK_BUTTON
			-- Field for indicating whether packing is targeted to
			-- a taxi run

	educational_field: EV_CHECK_BUTTON
			-- Field for indicating whether packing is targeted to
			-- an educational

	wafer_techn_field: EV_COMBO_BOX
			-- Field for wafer techn

	sched_run_field: EV_COMBO_BOX
			-- Field for scheduled run packing is targeted to

	field_set_field: EV_COMBO_BOX
			-- Field for name of field set to use

	step_pattern_field: EV_COMBO_BOX
			-- Field for name of field step pattern touse

	overhead_style_field: EV_COMBO_BOX
			-- Field for overhead style

	other_border_field: TEXT_FIELD
			-- Field for size of other border

	top_overhead_field: TEXT_FIELD
			-- Field for amount of overhead on top side

	intended_run_field: TEXT_FIELD
			-- Field for run ID for which packing is
			-- intended

	description_field: TEXT_FIELD
			-- Field for description of packing

end
