
indexing
	
	description: "Interface to allow setting of scribe properties"
	author: "David Hollenberg"
	date: "November 7, 2006"

deferred class RC_PACKING_SCRIBES_IFC
inherit
	ABSTRACT_INTERFACE
	GRIDABLE

feature {NONE} -- Creation

	make_interface
			-- Create interface
		do
			create accept_actions
			create cancel_actions
		end

feature -- Modification

	set_session (sess: like session)
			-- Set `session' to `sess'
		require
			sess_exists: sess /= Void
		do
			session := sess
		ensure
			session_set: session = sess
		end

feature -- Properties

	current_horizontal_scribe_width: DOUBLE
			-- Current width of horizontal scribes
		do
			Result := current_horizontal_scribe_width_agent.item ([])
		end

	current_vertical_scribe_width: DOUBLE
			-- Current width of vertical scribes
		do
			Result := current_vertical_scribe_width_agent.item ([])
		end

	current_horizontal_scribe_width_agent: FUNCTION [ANY, TUPLE, DOUBLE]
			-- Agent that returns current width of horizontal
			-- scribes

	current_vertical_scribe_width_agent: FUNCTION [ANY, TUPLE, DOUBLE]
			-- Agent that returns current width of horizontal
			-- scribes

	minimum_scribe_width: DOUBLE
			-- Minimum width of scribes

	maximum_horizontal_scribe_width: DOUBLE
			-- Maximum width of horizontal scribes

	maximum_vertical_scribe_width: DOUBLE
			-- Maximum width of vertical scribes

	default_scribe_width: DOUBLE
			-- Default width of scribes

	horizontal_external_scribe_width: DOUBLE
			-- Width of horizontal external scribes
			-- (outside of payload)

	vertical_external_scribe_width: DOUBLE
			-- Width of vertical external scribes
			-- (outside of payload)

	session: RC_ABSTRACT_PACKING_SESSION
			-- Session used to do packing

	accept_actions: ACTION_SEQUENCE [TUPLE [DOUBLE, DOUBLE]]
			-- Action to be invoked when user accepts
			-- current scribe properties

	cancel_actions: CLOSED_ACTION_SEQUENCE
			-- Action to be invoked when user clicks on Cancel

feature -- Modification

	set_minimum_scribe_width (a_width: like minimum_scribe_width)
			-- Set `minimum_scribe_width' to `a_width'
		require
			a_width_positive: a_width > 0.0
		do
			minimum_scribe_width := a_width
		ensure
			minimum_scribe_width_set: minimum_scribe_width = a_width
		end

	set_maximum_horizontal_scribe_width (a_width: like maximum_horizontal_scribe_width)
			-- Set `maximum_horizontal_scribe_width' to `a_width'
		require
			a_width_positive: a_width > 0.0
			a_width_on_grid: grid.is_size_on_grid (a_width)
		do
			maximum_horizontal_scribe_width := a_width
		ensure
			maximum_scribe_width_set: maximum_horizontal_scribe_width = a_width
		end

	set_maximum_vertical_scribe_width (a_width: like maximum_vertical_scribe_width)
			-- Set `maximum_vertical_scribe_width' to `a_width'
		require
			a_width_positive: a_width > 0.0
			a_width_on_grid: grid.is_size_on_grid (a_width)
		do
			maximum_vertical_scribe_width := a_width
		ensure
			maximum_scribe_width_set: maximum_vertical_scribe_width = a_width
		end

	set_default_scribe_width (a_width: like default_scribe_width)
			-- Set `default_scribe_width' to `a_width'
		require
			a_width_positive: a_width > 0.0
			a_width_on_grid: grid.is_size_on_grid (a_width)
		do
			default_scribe_width := a_width
		ensure
			default_scribe_width_set: default_scribe_width = a_width
		end

	set_horizontal_external_scribe_width (a_width: like horizontal_external_scribe_width)
			-- Set `horizontal_external_scribe_width' to `a_width'
		require
			a_width_nonnegative: a_width >= 0.0
			a_width_on_grid: grid.is_size_on_grid (a_width)
		do
			horizontal_external_scribe_width := a_width
		ensure
			horizontal_external_scribe_width_set: horizontal_external_scribe_width = a_width
		end

	set_vertical_external_scribe_width (a_width: like vertical_external_scribe_width)
			-- Set `vertical_external_scribe_width' to `a_width'
		require
			a_width_nonnegative: a_width >= 0.0
			a_width_on_grid: grid.is_size_on_grid (a_width)
		do
			vertical_external_scribe_width := a_width
		ensure
			vertical_external_scribe_width_set: vertical_external_scribe_width = a_width
		end

	set_current_horizontal_scribe_width_agent (an_agent: like current_horizontal_scribe_width_agent)
			-- Set `current_horizontal_scribe_width_agent' to
			-- `an_agent'
		require
			an_agent_not_void: an_agent /= Void
		do
			current_horizontal_scribe_width_agent := an_agent
		ensure
			agent_set: current_horizontal_scribe_width_agent = an_agent
		end

	set_current_vertical_scribe_width_agent (an_agent: like current_vertical_scribe_width_agent)
			-- Set `current_vertical_scribe_width_agent' to
			-- `an_agent'
		require
			an_agent_not_void: an_agent /= Void
		do
			current_vertical_scribe_width_agent := an_agent
		ensure
			agent_set: current_vertical_scribe_width_agent = an_agent
		end

end
