
indexing
	
	description: "Window to allow setting of scribe properties"
	author: "David Hollenberg"
	date: "November 7, 2005"

class RC_PACKING_SCRIBES_WINDOW

inherit
	RC_PACKING_SCRIBES_IFC
		undefine
			default_create
		end
	WINDOW_INTERFACE
		redefine
			enable_active,
			disable_active
		end
	MOSIS_GRAPHICAL_UTILITIES
		export
			{NONE} all
		undefine
			default_create
		end
	NUMBER_UTILITIES_ACCESS
		export
			{NONE} all
		undefine
			default_create
		end
	STRING_UTILITIES_ACCESS
		export
			{NONE} all
		undefine
			default_create
		end

create
	make

feature {NONE} -- Creation

	make (w: EV_TITLED_WINDOW)
			-- Create interface in window `w'
		require
			window_exists: w /= Void
			window_valid: not w.is_destroyed
		do
			make_interface
			make_proxy (w)
		end

feature -- Operations

	reset
			-- Move focus back into window
		do
			enable_active
		end

	refresh
			-- Refresh window with current width values
		do
			width_field.set_text (formatted_current_horizontal_width)
			horizontal_width_field.set_text (formatted_current_horizontal_width)
			vertical_width_field.set_text (formatted_current_vertical_width)
			if current_horizontal_scribe_width = current_vertical_scribe_width then
				if allow_different_widths_button.is_selected then
					allow_different_widths_button.disable_select
				end
			else
				if not allow_different_widths_button.is_selected then
					allow_different_widths_button.enable_select
				end
			end
		end

feature -- Modification

	enable_active
			-- Make interface active
		do
			precursor
			ok_button.enable_sensitive
			cancel_button.enable_sensitive
		end

	disable_active
			-- Make interface inactive
			-- (can look but cannot initiate most actions)
		do
			precursor
			ok_button.disable_sensitive
			cancel_button.disable_sensitive
		end

feature {NONE} -- Implementation

	build_interface
			-- Build interface
		local
			cols: WIDGET_COLUMN_SET
			buttons: HORIZONTAL_BUTTON_SET
			main_vbox: EV_VERTICAL_BOX
			box: EV_BOX
			list_item: EV_LIST_ITEM
			label: EV_LABEL
			hbox: EV_HORIZONTAL_BOX
		do
			create main_vbox
			main_vbox.set_padding (user_interface.padding)
			main_vbox.set_border_width (user_interface.border_width)
			create cols.make (2)
			cols.set_padding (user_interface.padding)
			
			create horizontal_width_field.make_with_text (formatted_current_horizontal_width)
			create list_item.make_with_text (grid.formatted_number (default_scribe_width) + " (default width)")
			list_item.select_actions.extend (agent horizontal_width_field.set_text (grid.formatted_number (default_scribe_width)))
			horizontal_width_field.extend (list_item)
			if horizontal_external_scribe_width > 0 then
				create list_item.make_with_text (grid.formatted_number (horizontal_external_scribe_width) + " (external width)")
				list_item.select_actions.extend (agent horizontal_width_field.set_text (grid.formatted_number (horizontal_external_scribe_width)))
				horizontal_width_field.extend (list_item)
			end
			horizontal_width_field.return_actions.extend (agent process_ok)
			cols.append_row (<< gui.label_with_text ("Horizontal Scribe Width"), horizontal_width_field >>)
			
			create vertical_width_field.make_with_text (formatted_current_vertical_width)
			create list_item.make_with_text (grid.formatted_number (default_scribe_width) + " (default width)")
			list_item.select_actions.extend (agent vertical_width_field.set_text (grid.formatted_number (default_scribe_width)))
			vertical_width_field.extend (list_item)
			if vertical_external_scribe_width > 0 then
				create list_item.make_with_text (grid.formatted_number (vertical_external_scribe_width) + " (external width)")
				list_item.select_actions.extend (agent vertical_width_field.set_text (grid.formatted_number (vertical_external_scribe_width)))
				vertical_width_field.extend (list_item)
			end
			vertical_width_field.return_actions.extend (agent process_ok)
			cols.append_row (<< gui.label_with_text ("Vertical Scribe Width"), vertical_width_field >>)
			
			cols.disable_column_expand (1)
			cols.align_column_right (1)
			different_widths_box := cols.box
			gui.set_box_background_recursive (different_widths_box, background_color)
			
			create cols.make (2)
			cols.set_padding (user_interface.padding)
			
			create width_field.make_with_text (formatted_current_horizontal_width)
			create list_item.make_with_text (grid.formatted_number (default_scribe_width) + " (default width)")
			list_item.select_actions.extend (agent width_field.set_text (grid.formatted_number (default_scribe_width)))
			width_field.extend (list_item)
			width_field.return_actions.extend (agent process_ok)
			cols.append_row (<< gui.label_with_text ("Scribe Width"), width_field >>)
			cols.disable_column_expand (1)
			cols.align_column_right (1)
			same_widths_box := cols.box
			gui.set_box_background_recursive (same_widths_box, background_color)
			
			create widths_box
			widths_box.extend (same_widths_box)

			main_vbox.extend (widths_box)
			main_vbox.disable_item_expand (widths_box)
			
			create allow_different_widths_button.make_with_text ("Allow different widths for horizontal and vertical scribes")
			allow_different_widths_button.select_actions.extend (agent process_widths_switch)
			main_vbox.extend (allow_different_widths_button)
			main_vbox.disable_item_expand (allow_different_widths_button)

			create label.make_with_text ("Size resolution is " + grid.size_resolution_text + " microns")
			create hbox
			hbox.extend (label)
			hbox.disable_item_expand (label)
			main_vbox.extend (hbox)
			main_vbox.disable_item_expand (hbox)
			
			create buttons.make_with_text_and_actions (<<
				"OK", "Cancel" >>,
				<< agent process_ok, agent process_cancel >>)
			ok_button := buttons.i_th (1)
			cancel_button := buttons.i_th (2)
			
			box := buttons.column_set.box
			
			main_vbox.extend (box)
			main_vbox.disable_item_expand (box)
			
			gui.set_box_background_recursive (main_vbox, background_color)
			extend (main_vbox)
			close_request_actions.extend (agent process_cancel)
			gui.add_cancel_accelerator (window, agent process_cancel)
			gui.do_once_on_idle (agent post_display_actions)
			set_minimum_width (500)
		end
		
	post_display_actions
		do
			reset
		end

	process_widths_switch
			-- Process click on `different_widths_check_button'
		do
			widths_box.wipe_out
			if allow_different_widths_button.is_selected then
				horizontal_width_field.set_text (formatted_current_horizontal_width)
				vertical_width_field.set_text (formatted_current_vertical_width)
				widths_box.extend (different_widths_box)
			else
				width_field.set_text (formatted_current_horizontal_width)
				widths_box.extend (same_widths_box)
			end
		end

	formatted_current_horizontal_width: STRING
			-- `current_horizontal_scribe_width' rounded
			-- to `grid_scale' and formatted nicely
		do
			Result := grid.formatted_number (current_horizontal_scribe_width)
		end

	formatted_current_vertical_width: STRING
			-- `current_vertical_scribe_width' rounded
			-- to `grid_scale' and formatted nicely
		do
			Result := grid.formatted_number (current_vertical_scribe_width)
		end

	process_ok
		local
			horiz, vert, both: STRING
			h, v: DOUBLE
			ok: BOOLEAN
		do
			if is_active then
				disable_active
				if allow_different_widths_button.is_selected then
					horiz := horizontal_width_field.text
					vert := vertical_width_field.text
					create last_message.make (50)
					ok := True
					check_dimension ("Horizontal scribe", horiz, maximum_horizontal_scribe_width)
					ok := ok and is_valid
					h := last_width
					check_dimension ("Vertical scribe", vert, maximum_vertical_scribe_width)
					ok := ok and is_valid
					v := last_width
				else
					both := width_field.text
					create last_message.make (50)
					check_dimension ("Scribe", both, maximum_horizontal_scribe_width.min (maximum_vertical_scribe_width))
					ok := is_valid
					h := last_width
					v := last_width
				end
				if ok then
					accept_actions.call ([h, v])
				else
					inform (last_message)
				end
				enable_active
			end
		end

	check_dimension (a_type, a_value: STRING a_max: DOUBLE)
			-- Check that dimension `a_type' with
			-- `a_value' whose maximum value is `a_max' is
			-- valid.  Append error to `last_message' if
			-- not
		require
			a_type_not_void: a_type /= Void
			a_value_not_void: a_value /= Void
		local
			val: DOUBLE
		do
			is_valid := False
			if not a_value.is_double then
				last_message.append (a_type + " width " + su.double_quoted (a_value) + " is not a valid real number%N")
			else
				val := a_value.to_double
				if val < minimum_scribe_width then
					last_message.append (a_type + " width (" + a_value + ") is less than minimum allowed width (" + grid.formatted_number (minimum_scribe_width) + ")%N")
				elseif val > a_max then
					last_message.append (a_type + " width (" + a_value + ") is greater than maximum allowed width (" + grid.formatted_number (a_max) + ")%N")
				elseif not grid.is_size_on_grid (val) then
					last_message.append (a_type + " width (" + a_value + ") is not a multiple of size resolution " + grid.size_resolution_text + "%N")
				else
					is_valid := True
					last_width := grid.rounded_size (val)
				end
			end
		end

	process_cancel
		do
			if is_active then
				disable_active
				cancel_actions.call ([])
			end
		end

	widths_box: EV_VERTICAL_BOX
			-- Box to hold width fields
	
	different_widths_box: EV_BOX
			-- Box to hold width fields when horizontal and
			-- vertical scribes may have different widths
	
	same_widths_box: EV_BOX
			-- Box to hold width fields when horizontal and
			-- vertical scribes must have same width
	
	width_field: EV_COMBO_BOX
			-- Field to set width of scribes
	
	horizontal_width_field: EV_COMBO_BOX
			-- Field to set width of horizontal scribes
	
	vertical_width_field: EV_COMBO_BOX
			-- Field to set width of vertical scribes
	
	allow_different_widths_button: EV_CHECK_BUTTON
			-- Button to allow user to set different widths
			-- for horizontal and vertical scribes
	
	ok_button: EV_BUTTON
			-- Button to hide window
	
	cancel_button: EV_BUTTON
			-- Button to hide window
	
	is_valid: BOOLEAN
			-- Was last value passed to `check_dimension' valid?
	
	last_message: STRING
			-- Error message
	
	last_width: DOUBLE
			-- Last valid width found by `check_dimension'
	

end
