
indexing
	
	description: "Window to allow searching for projects in packer"
	author: "David Hollenberg"
	date: "December 11, 2006"

class RC_PACKING_SEARCH_WINDOW

inherit
	RC_PACKING_SEARCH_IFC
	WINDOW_INTERFACE
		redefine
			enable_active,
			disable_active
		end
	MOSIS_GRAPHICAL_UTILITIES
		export
			{NONE} all
		undefine
			default_create
		end
	STRING_UTILITIES_ACCESS
		export
			{NONE} all
		undefine
			default_create
		end

create
	make

feature {NONE} -- Creation

	make (w: EV_TITLED_WINDOW)
			-- Create interface in window `w'
		require
			window_exists: w /= Void
			window_valid: not w.is_destroyed
		do
			make_interface
			make_proxy (w)
		end

feature -- Operations

	reset
			-- Move focus back into window
		do
			enable_active
		end

feature -- Modification

	enable_active
			-- Make interface active
		do
			precursor
			search_button.enable_sensitive
			cancel_button.enable_sensitive
		end

	disable_active
			-- Make interface inactive
			-- (can look but cannot initiate most actions)
		do
			precursor
			search_button.disable_sensitive
			cancel_button.disable_sensitive
		end

feature {NONE} -- Implementation

	build_interface
			-- Build interface
		local
			cols: WIDGET_COLUMN_SET
			main_vbox: EV_VERTICAL_BOX
			box: EV_BOX
			buttons: HORIZONTAL_BUTTON_SET
		do
			create main_vbox
			main_vbox.set_padding (user_interface.padding)
			main_vbox.set_border_width (user_interface.border_width)
			create cols.make (2)
			cols.set_padding (user_interface.padding)
			create design_number_field
			design_number_field.return_actions.extend (agent process_search)
			cols.append_row (<< gui.label_with_text ("Design Numbers"), design_number_field >>)
			cols.disable_column_expand (1)
			cols.align_column_right (1)
			box := cols.box
			main_vbox.extend (box)
			main_vbox.disable_item_expand (box)
			
			create select_matches_button.make_with_text ("Select projects")
			select_matches_button.enable_select
			main_vbox.extend (select_matches_button)
			main_vbox.disable_item_expand (select_matches_button)

			create buttons.make_with_text_and_actions (<<
				"Search", "Cancel" >>,
				<< agent process_search, agent process_cancel >>)
			search_button := buttons.i_th (1)
			cancel_button := buttons.i_th (2)
			
			box := buttons.column_set.box
			
			main_vbox.extend (box)
			main_vbox.disable_item_expand (box)
			
			gui.set_box_background_recursive (main_vbox, background_color)
			extend (main_vbox)
			close_request_actions.extend (agent process_cancel)
			gui.add_cancel_accelerator (window, agent process_cancel)
			gui.do_once_on_idle (agent post_display_actions)
			set_minimum_width (500)
		end
		
	post_display_actions
		do
			reset
		end

	process_search
		do
			if is_active then
				disable_active
				build_valid_design_numbers
				build_design_number_list
				if not design_number_list.is_empty then
					search_actions.call ([design_number_list, select_matches_button.is_selected])
				end
				enable_active
			end
		end

	build_valid_design_numbers
			-- Build list of valid design numbers
		local
			list: LIST [RETICLE_VIEW_ELEMENT]
			elem: RETICLE_VIEW_ELEMENT
		do
			list := view_elements_agent.item ([])
			create {ARRAYED_LIST [INTEGER]} valid_design_numbers.make (list.count)
			from
				list.start
			until
				list.after
			loop
				elem := list.item
				if elem.is_project then
					valid_design_numbers.extend (elem.design_number)
				end
				list.forth
			end
		end

	build_design_number_list
			-- Build list of design numbers to search for
			-- and put it in `design_number_list'
		local
			list: LIST [STRING]
			dn: STRING
			msg: STRING
			n: INTEGER
		do
			list := su.broken_into_words (design_number_field.text)
			create {ARRAYED_LIST [INTEGER]} design_number_list.make (list.count)
			create msg.make (0)
			from
				list.start
			until
				list.after
			loop
				dn := list.item
				if dn.is_integer then
					n := dn.to_integer
					if valid_design_numbers.has (n) then
						design_number_list.extend (n)
					else
						msg.append ("Design number " + dn.out + " not found in packing%N")
					end
				else
					msg.append ("Value " + su.double_quoted (dn) + " is not a valid integer%N")
				end
				list.forth
			end
			if not msg.is_empty then
				inform (msg)
			end
		end

	process_cancel
		do
			if is_active then
				disable_active
				cancel_actions.call ([])
			end
		end

	design_number_field: EV_TEXT_FIELD
			-- Field for entering design numbers to search for
	
	search_button: EV_BUTTON
			-- Button to search for projects
	
	select_matches_button: EV_CHECK_BUTTON
			-- Button to indicate whether matching projects
			-- should be selected, as well as displayed differently
	
	cancel_button: EV_BUTTON
			-- Button to hide window
	
	valid_design_numbers: LIST [INTEGER]
			-- List of valid design numbers

	design_number_list: LIST [INTEGER]
			-- List of design numbers to search for

end
