
indexing
	
	description: "Window to allow selection of existing packing"
	author: "David Hollenberg"
	date: "November 7, 2005"

class RC_PACKING_SELECTION_WINDOW

inherit
	RC_PACKING_IFC
	WINDOW_INTERFACE
		redefine
			enable_active,
			disable_active
		end
	GUI_UTILITIES_ACCESS
		export
			{NONE} all
		undefine
			default_create
		end
	USER_INTERFACE_ACCESS
		export
			{NONE} all
		undefine
			default_create
		end
	CONDITION_UTILITIES
		export
			{NONE} all
		undefine
			default_create
		end
	STRING_UTILITIES_ACCESS
		export
			{NONE} all
		undefine
			default_create
		end
	DB_OPTIONS_ACCESS
		export
			{NONE} all
		undefine
			default_create
		end
	RC_SESSION_MANAGER_ACCESS
		export
			{NONE} all
		undefine
			default_create
		end
	PACKING_RECORD_UTILITIES
		export
			{NONE} all
		redefine
			populate_creator, populate_run_type,
			add_run_type
		end
		
create
	make

feature {NONE} -- Creation

	make (w: EV_TITLED_WINDOW)
			-- Create interface in window `w'
		require
			window_exists: w /= Void
			window_valid: not w.is_destroyed
		do
			make_interface
			make_proxy (w)
		end

feature -- Operations

	reset
			-- Move focus back into window
		do
			enable_active
		end

feature -- Modification

	enable_active
			-- Make interface active
		do
			precursor
			ok_button.enable_sensitive
			filter_button.enable_sensitive
			cancel_button.enable_sensitive
		end

	disable_active
			-- Make interface inactive
			-- (can look but cannot initiate most actions)
		do
			precursor
			ok_button.disable_sensitive
			filter_button.disable_sensitive
			cancel_button.disable_sensitive
		end

feature {NONE} -- Implementation

	build_interface
			-- Build interface
		local
			buttons: HORIZONTAL_BUTTON_SET
			main_vbox: EV_VERTICAL_BOX
			box: EV_BOX
		do
			add_menu_bar

			create main_vbox
			main_vbox.set_padding (user_interface.padding)
			main_vbox.set_border_width (user_interface.border_width)
			create buttons.make_with_text_and_actions (<<
				"OK", "Filter", "Cancel" >>,
				<< agent process_ok, agent process_filter, agent process_cancel >>)
			ok_button := buttons.i_th (1)
			filter_button := buttons.i_th (2)
			cancel_button := buttons.i_th (3)
			
			box := buttons.column_set.box
			
			build_fields
			populate_fields
			set_field_properties
			main_vbox.extend (fields_box)
			main_vbox.disable_item_expand (fields_box)

			main_vbox.extend (box)
			main_vbox.disable_item_expand (box)
			
			create packings
			packings.set_row_to_generic_function (agent row_to_record)
			packings.build_selection_status_box
			set_visible_columns
			main_vbox.extend (packings)

			-- main_vbox.extend (box)
			-- main_vbox.disable_item_expand (box)
			
			gui.set_box_background_recursive (main_vbox, background_color)
			extend (main_vbox)
			gui.add_cancel_accelerator (window, agent process_cancel)
			gui.do_once_on_idle (agent post_display_actions)
		end
		
	add_menu_bar
			-- Add menu bar to window
		local
			file_menu: EV_MENU
			exit_menu_item: EV_MENU_ITEM
			close_menu_item: EV_MENU_ITEM
			a_menu_bar: EV_MENU_BAR
		do	
			create file_menu.make_with_text ("File")
			
			create close_menu_item.make_with_text ("Close")
			close_menu_item.select_actions.extend (agent process_cancel)
			file_menu.extend (close_menu_item)
			
			create exit_menu_item.make_with_text ("Exit")
			exit_menu_item.select_actions.extend (gui.destroy_application_agent)
			file_menu.extend (exit_menu_item)
			
			create a_menu_bar
			a_menu_bar.extend (file_menu)
			set_menu_bar (a_menu_bar)
		end

	build_fields
			-- Build portion of window with fields
			-- in `fields_box'
		local
			cols: WIDGET_COLUMN_SET
			frame: EV_FRAME
		do
			create fields_box

			create cols.make (2)
			cols.set_padding (user_interface.padding)
			cols.set_border_width (user_interface.border_width)
			
			create packing_id_field
			packing_id_field.set_capacity ({PACKINGS_ROW}.maxlen_packing_id)
			packing_id_field.set_minimum_width_in_characters (30)
			
			cols.append_row (<< gui.label_with_text ("Packing ID"), packing_id_field >>)
			cols.disable_expand
			cols.align_column_right (1)
			cols.align_column_left (2)
			fields_box.extend (cols.box)
			
			create cols.make (2)
			cols.set_padding (user_interface.padding)
			cols.set_border_width (user_interface.border_width)
			
			create creator_field
			creator_field.set_minimum_width_in_characters (10)
			
			create intended_run_field
			intended_run_field.enable_uppercase_only
			intended_run_field.set_capacity ({PACKINGS_ROW}.maxlen_intended_run)
			intended_run_field.set_minimum_width_in_characters (4)
			
			create wafer_techn_field
			wafer_techn_field.set_minimum_width_in_characters (40)
			
			create run_type_field
			run_type_field.set_minimum_width_in_characters (3)
			
			cols.append_row (<< gui.label_with_text ("Created By"), creator_field >>)
			cols.append_row (<< gui.label_with_text ("Intended Run"), intended_run_field >>)
			cols.append_row (<< gui.label_with_text ("Run Type"), run_type_field >>)
			cols.append_row (<< gui.label_with_text ("Wafer Techn"), wafer_techn_field >>)
			if has_run_context then
				create match_run_field
				match_run_field.enable_select
				cols.append_row (<< gui.label_with_text ("Match Run Attributes"), match_run_field >>)
			end
			cols.disable_expand
			cols.align_column_right (1)
			cols.align_column_left (2)
			create frame.make_with_text ("Filter Criteria")
			frame.extend (cols.box)
			fields_box.extend (frame)
		end
		
	fetch_runs_row
			-- Fetch row in RUNS table
		do
			create runs_row.make_by_key (connection, option.Old_row, run_id)
		end
		
	set_field_properties
			-- Set properties of fields
		do
			packing_id_field.return_actions.extend (agent process_ok)
			creator_field.disable_edit
			run_type_field.disable_edit
			wafer_techn_field.disable_edit
			if has_run_context then
				process_match_run_change
				match_run_field.select_actions.extend (agent process_match_run_change)
			end
		end
		
	populate_fields
			-- Populate some fields
		do
			if has_run_context then
				fetch_runs_row
			end
			populate_creator (creator_field)
			populate_run_type (run_type_field)
			populate_wafer_techn (wafer_techn_field)
		end
		
	populate_creator (f: EV_COMBO_BOX)
		local
			list_item: EV_LIST_ITEM
		do
			create list_item.make_with_text ("")
			f.extend (list_item)
			precursor (f)
		end

	populate_run_type (f: EV_COMBO_BOX)
		local
			list_item: EV_LIST_ITEM
		do
			create list_item.make_with_text ("")
			f.extend (list_item)
			precursor (f)
		end

	add_run_type (row: RUN_TYPES_ROW f: EV_COMBO_BOX)
		do
			precursor (row, f)
			if has_run_context then
				if f.last.text.is_equal (runs_row.run_type) then
					f.last.enable_select
				end
			end
		end

	populate_wafer_techn (f: EV_COMBO_BOX)
		local
			row: WAFER_TECHNS_ROW
			wafer_techns: LIST [WAFER_TECHNS_ROW]
			design_type: DESIGN_TYPE_VALUES
			list_item: EV_LIST_ITEM
		do
			create list_item.make_with_text ("")
			list_item.set_data ("")
			f.extend (list_item)
			create row.make_template
			wafer_techns := row.current_wafer_techns_ordered (design_type.Ic, connection, expected_count.At_least_zero)
			wafer_techns.do_all (agent add_wafer_techn)
		end

	add_wafer_techn (row: WAFER_TECHNS_ROW)
		local
			list_item: EV_LIST_ITEM
			wt: STRING
		do
			create list_item.make_with_text (row.wafer_techn + "   " + row.descr)
			list_item.set_data (row.wafer_techn)
			wafer_techn_field.extend (list_item)
			if has_run_context then
				wt ?= list_item.data
				if wt.is_equal (runs_row.wafer_techn) then
					list_item.enable_select
				end
			end
		end

	post_display_actions
		do
			gui.do_once_on_idle (agent gui.center_window_with_margin (window, 100, 100))
			gui.do_once_on_idle (agent process_filter)
			reset
		end

	process_match_run_change
		do
			if match_run_field.is_selected then
				run_type_field.disable_sensitive
				wafer_techn_field.disable_sensitive
			else
				run_type_field.enable_sensitive
				wafer_techn_field.enable_sensitive
			end
		end

	process_ok
		local
			ifc: RC_PACKING_IFC
			w: EV_TITLED_WINDOW
			a_title: STRING
		do
			if is_active then
				disable_active
				validate_selection
				if is_valid then
					packing_session.set_packing_id (packing_id)
					create w
					a_title := "Confirm Packing Selection"
					if run_id /= Void then
						a_title.append (" for " + run_id)
					end
					ifc := session.factory.packing_confirmation_interface (w, run_id, packing_id, a_title)
					ifc.set_session (packing_session)
					ifc.set_role (role)
					ifc.edit_actions.extend (agent process_edit)
					ifc.edit_actions.extend (agent w.destroy)
					ifc.cancel_actions.extend (agent w.destroy)
					ifc.cancel_actions.extend (agent enable_active)
					ifc.build
					w.show
					confirmation_interface := ifc
				else
					enable_active
				end
			end
		end

	validate_selection
			-- Validate selected packing (make sure it exists
			-- and is compatible with run if have run context)
		local
			p: PACKINGS_ROW
		do
			is_valid := False
			packing_id := packing_id_field.text
			packing_id.left_adjust
			packing_id.right_adjust
			if packing_id.is_empty then
				inform ("Required field Packing ID is empty")
			else
				create p.make_by_key (connection, option.Old_no_exception_row, packing_id)
				if p.is_empty then
					inform ("Packing " + su.double_quoted (packing_id) + " not found")
				elseif has_run_context then
					p.check_compatibility_with_run (runs_row)
					if not p.is_compatible_with_run then
						inform ("Packing " + su.double_quoted (packing_id) + " is not compatible with run " + run_id + "%N%N" + p.incompatibilities_with_run)
					else
						is_valid := True
					end
				else
					is_valid := True
				end
			end
		end

	process_filter
		do
			if is_active then
				disable_active
				fetch_matching_packings
				build_all_rows
				packings.reverse_sort_on_column (Date_created_position)
				gui.do_once_on_idle (agent packings.resize_all_columns_to_content)
				enable_active
			end
		end

	process_cancel
		do
			if is_active then
				disable_active
				cancel_actions.call ([])
			end
		end

	process_edit
		local
			ifc: RC_PACKING_IFC
		do
			ifc := session.factory.packing_interface (Void, run_id, packing_id, packing_session.title_with_run_and_packing_id ("Edit"))
			ifc.set_session (packing_session)
			ifc.set_packing_session (packing_session)
			ifc.cancel_actions.extend (agent packing_session.terminate)
			ifc.cancel_actions.append (cancel_actions)
			ifc.build
			session_manager.add_session (packing_session)
			session_manager.set_current_session (packing_session)
		end

	row_to_record (row: EV_MULTI_COLUMN_LIST_ROW): PACKING_MULTI_COLUMN_LIST_ROW
			-- Function which translates `row' to object of
			-- type PACKING_MULTI_COLUMN_LIST_ROW
		do
			Result ?= row
			check
				result_not_void: Result /= Void
			end
		end

	set_visible_columns
			-- Set visible columns in `packings'
		local
			cols: ARRAYED_LIST [MULTI_COLUMN_LIST_COLUMN [PACKING_MULTI_COLUMN_LIST_ROW]]
		do
			create cols.make (10)
			cols.extend (create {PACKING_ID_COLUMN [PACKING_MULTI_COLUMN_LIST_ROW]})
			cols.extend (create {CREATOR_COLUMN [PACKING_MULTI_COLUMN_LIST_ROW]})
			cols.extend (create {DATE_CREATED_COLUMN [PACKING_MULTI_COLUMN_LIST_ROW]})
			cols.extend (create {RUN_TYPE_COLUMN [PACKING_MULTI_COLUMN_LIST_ROW]})
			cols.extend (create {WAFER_TECHN_COLUMN [PACKING_MULTI_COLUMN_LIST_ROW]})
			cols.extend (create {FIELD_SET_COLUMN [PACKING_MULTI_COLUMN_LIST_ROW]})
			cols.extend (create {RUN_ID_COLUMN [PACKING_MULTI_COLUMN_LIST_ROW]})
			packings.set_visible_columns (cols)
		end
			
	fetch_matching_packings
			-- Fetch packings that match selection criteria
		do
			if has_run_context and then match_run_field.is_selected then
				fetch_packings_matching_run
			else
				fetch_packings_matching_criteria
			end
		end
			
	fetch_packings_matching_criteria
			-- Fetch packings that match selection criteria
		local
			row: PACKINGS_ROW
			wt: STRING
		do
			if wafer_techn_field.selected_item /= Void then
				wt ?= wafer_techn_field.selected_item.data
			else
				wt := ""
			end
			create row.make_template
			packing_rows := row.rows_matching_criteria (creator_field.text, run_type_field.text, wt, intended_run_field.text, "date_created desc", connection, expected_count.At_least_zero)
		end
			
	fetch_packings_matching_run
			-- Fetch packings that match attributes of run
		require
			has_run_context: has_run_context
		local
			row: PACKINGS_ROW
		do
			create row.make_template
			packing_rows := row.rows_matching_run (creator_field.text, intended_run_field.text, run_id, "date_created desc", connection, expected_count.At_least_zero)
		end
			
	build_all_rows
			-- Build all rows and put in `packings'
		local
			list: ARRAYED_LIST [PACKING_MULTI_COLUMN_LIST_ROW]
		do
			create list.make (packing_rows.count)
			packing_rows.do_all (agent add_row (?, list))
			packings.set_all_rows (list)
		end
			
	add_row (p: PACKINGS_ROW list: LIST [PACKING_MULTI_COLUMN_LIST_ROW])
			-- Add a row for `p' to `list'
		local
			row: PACKING_MULTI_COLUMN_LIST_ROW
		do
			create row.make (p)
			list.extend (row)
			row.select_actions.extend (agent packings.process_select_one (row))
			row.select_actions.extend (agent set_packing_id_text (row))
			row.pointer_double_press_actions.force_extend (agent process_ok)
			row.deselect_actions.extend (agent packings.process_deselect_one (row))
			row.deselect_actions.extend (agent packing_id_field.set_text (""))
		end
			
	set_packing_id_text (p: PACKING_MULTI_COLUMN_LIST_ROW)
			-- Set text in `packing_id_field' to packing id in `p'
		do
			packing_id_field.set_text (p.packing_id)
		end
			
	wafer_techns_row: WAFER_TECHNS_ROW
			-- Row for wafer techn
		do
			if has_run_context then
				Result := runs_row.wafer_techn_row
			end
		end

	ok_button: EV_BUTTON
			-- Button to select packing named in `packing_id_field'
	
	filter_button: EV_BUTTON
			-- Button to filter packings according to selection
			-- criteria
	
	cancel_button: EV_BUTTON
			-- Button to end session
	
	fields_box: EV_VERTICAL_BOX
			-- Box with fields for run id, etc.

	packing_id_field: TEXT_FIELD
			-- Field for constraining design number

	creator_field: EV_COMBO_BOX
			-- Field for constraining creator

	intended_run_field: TEXT_FIELD
			-- Field for constraining intended run

	run_type_field: EV_COMBO_BOX
			-- Field for constraining run type

	wafer_techn_field: EV_COMBO_BOX
			-- Field for constraining wafer techn

	match_run_field: EV_CHECK_BUTTON
			-- Field for specifying whether to constrain
			-- packings to those matching attributes of
			-- run, if have run context

	connection: DB_CONNECTION
		do
			Result := session.connection
		end

	packing_rows: LIST [PACKINGS_ROW]
			-- List of packings that meet filtering criteria

	runs_row: RUNS_ROW
			-- Row for run, if has run context

	packings: MULTI_COLUMN_LIST [PACKING_MULTI_COLUMN_LIST_ROW]
			-- List of packings that meet filtering criteria,
			-- displayed in multi-column list

	confirmation_interface: RC_PACKING_IFC
			-- Interface to confirm user's choice of
			-- packing to edit

	Date_created_position: INTEGER = 3
			-- Position of "date created" column in
			-- `packings', so we can reverse sort on it

	is_valid: BOOLEAN
			-- Is selected packing valid?

end
