
indexing
	
	description: "Window to view and set lot split colors %
		%in packer"
	author: "David Hollenberg"
	date: "December 13, 2005"

class RC_PACKING_SPLIT_COLORS_WINDOW

inherit
	RC_PACKING_SPLIT_COLORS_IFC
	WINDOW_INTERFACE
		redefine
			enable_active,
			disable_active
		end
	MOSIS_GRAPHICAL_UTILITIES
		export
			{NONE} all
		undefine
			default_create
		end
	ITEM_COUNT_RANGES_ACCESS
		export
			{NONE} all
		undefine
			default_create
		end

create
	make

feature {NONE} -- Creation

	make (w: EV_TITLED_WINDOW)
			-- Create interface in window `w'
		require
			window_exists: w /= Void
			window_valid: not w.is_destroyed
		do
			make_interface
			make_proxy (w)
		end

feature -- Operations

	reset
			-- Move focus back into window
		do
			enable_active
		end

feature -- Modification

	enable_active
			-- Make interface active
		do
			precursor
			ok_button.enable_sensitive
			cancel_button.enable_sensitive
		end

	disable_active
			-- Make interface inactive
			-- (can look but cannot initiate most actions)
		do
			precursor
			ok_button.disable_sensitive
			cancel_button.disable_sensitive
		end

feature {NONE} -- Implementation

	build_interface
			-- Build interface
		local
			cols: WIDGET_COLUMN_SET
			main_vbox: EV_VERTICAL_BOX
			buttons: HORIZONTAL_BUTTON_SET
			box: EV_BOX
			names_row: WAFER_ETEST_NAMES_ROW
			etest_name: STRING
			name_box: EV_COMBO_BOX
			custom_button: EV_BUTTON
			color_area: EV_DRAWING_AREA
			frame: EV_FRAME
		do
			build_split_colors_table
			build_named_colors_table
			create new_split_colors_table.make (wafer_etest_names.count)
			create main_vbox
			main_vbox.set_padding (user_interface.padding)
			main_vbox.set_border_width (user_interface.border_width)
			create cols.make (4)
			cols.set_padding (user_interface.padding)
			from
				wafer_etest_names.start
			until
				wafer_etest_names.after
			loop
				names_row := wafer_etest_names.item
				etest_name := names_row.wafer_etest_name
				create color_area
				color_area.set_minimum_width (50)
				name_box := new_color_name_box (etest_name, color_area)
				split_colors_table.search (etest_name)
				if split_colors_table.found then
					color_area.set_background_color (to_ev_color (split_colors_table.found_item))
					name_box.set_text (name_for_color (split_colors_table.found_item))
				else
					color_area.set_background_color (unassigned_color)
					name_box.set_text (Unassigned_name)
				end
				color_area.expose_actions.extend (agent color_area.clear_rectangle)
				name_box.disable_edit
				create frame
				frame.extend (color_area)
				create custom_button.make_with_text_and_action ("Custom", agent show_custom_color_dialog (names_row, name_box, color_area))
				custom_button.set_minimum_width (user_interface.minimum_button_width)
				cols.append_row (<< gui.label_with_text (names_row.wafer_etest_id), frame, name_box, custom_button >>)
				
				wafer_etest_names.forth
			end

			cols.disable_expand
			cols.enable_column_expand (3)
			cols.align_column_right (1)
			box := cols.box
			
			main_vbox.extend (box)
			main_vbox.disable_item_expand (box)
			
			create buttons.make_with_text_and_actions (<<
				"OK", "Cancel" >>,
				<< agent process_ok, agent process_cancel >>)
			ok_button := buttons.i_th (1)
			cancel_button := buttons.i_th (2)
			
			box := buttons.column_set.box
			
			main_vbox.extend (box)
			main_vbox.disable_item_expand (box)
			
			gui.set_box_background_recursive (main_vbox, background_color)
			extend (main_vbox)
			close_request_actions.extend (agent process_cancel)
			gui.add_cancel_accelerator (window, agent process_cancel)
			gui.do_once_on_idle (agent post_display_actions)
			set_minimum_width (600)
		end
		
	post_display_actions
		do
			reset
		end

	fetch_split_colors
			-- Fetch existing lot split color assignments
			-- from database
		local
			row: PACKING_ETEST_COLORS_MROW
		do
			create row.make_template
			split_colors := row.rows_for_packing_id (packing.packing_id, packing.any_connection, expected_count.At_least_zero)
		end

	build_split_colors_table
			-- Build table that maps wafer etest ID to the
			-- database row that assigns it a color
		do
			create split_colors_table.make (split_colors.count)
			split_colors.do_all (agent add_split_color)
		end

	add_split_color (row: PACKING_ETEST_COLORS_MROW)
			-- Add `row' to `split_colors_table'
		do
			split_colors_table.put (row, row.wafer_etest_name)
		end

	build_named_colors_table
			-- Build table that maps color name to a named color
		do
			create named_colors_table.make (named_colors.count)
			named_colors.do_all (agent add_named_color)
		end

	add_named_color (row: NAMED_COLORS_ROW)
			-- Add `row' to `named_colors_table'
		do
			named_colors_table.put (row, row.color_name)
		end

	to_ev_color (row: PACKING_ETEST_COLORS_ROW): EV_COLOR
			-- `row' as an EV_COLOR
		do
			if row.is_null_color_name then
				create Result.make_with_rgb (row.red_value, row.green_value, row.blue_value)
			else
				named_colors_table.search (row.color_name)
				if named_colors_table.found then
					Result := gui.to_ev_color (named_colors_table.found_item)
				else
					handle_hard_error ("Color name " + row.color_name + " not found in named_colors_table")
				end
			end
		end

	name_for_color (row: PACKING_ETEST_COLORS_ROW): STRING
			-- Color name for `row'
		do
			if not row.is_null_color_name then
				Result := row.color_name
			else
				Result := Custom_name + "  " + gui.color_name_8_bit (row.to_ev_color)
			end
		end

	new_color_name_box (an_etest_name: STRING a_color_area: EV_DRAWING_AREA): EV_COMBO_BOX
			-- New combo box with all known named colors in
			-- its pulldown list for `an_etest_name'
		require
			an_etest_name_not_void: an_etest_name /= Void
			a_color_area_not_void: a_color_area /= Void
		local
			list_item: EV_LIST_ITEM
			l_color: NAMED_COLORS_ROW
		do
			is_select_blocked := True
			create Result.make_with_text ("")
			from
				named_colors.start
			until
				named_colors.after
			loop
				l_color := named_colors.item
				create list_item.make_with_text (l_color.color_name)
				list_item.select_actions.extend (agent assign_named_color (an_etest_name, l_color, a_color_area))
				Result.extend (list_item)
				named_colors.forth
			end
			is_select_blocked := False
		end

	show_custom_color_dialog (a_names_row: WAFER_ETEST_NAMES_ROW a_name_box: EV_COMBO_BOX a_color_area: EV_DRAWING_AREA)
			-- Display dialog to allow user to select
			-- custom color
		require
			a_names_row_not_void: a_names_row /= Void
			a_name_box_not_void: a_name_box /= Void
			a_color_area_not_void: a_color_area /= Void
		local
			l_dialog: EV_COLOR_DIALOG
		do
			create l_dialog
			l_dialog.set_title ("Set Color for Lot " + a_names_row.wafer_etest_id)
			l_dialog.set_color (a_color_area.background_color)
			l_dialog.ok_actions.extend (agent assign_custom_color (a_names_row.wafer_etest_name, l_dialog, a_name_box, a_color_area))
			l_dialog.show_modal_to_window (window)
		end

	assign_named_color (an_etest_name: STRING a_color_row: NAMED_COLORS_ROW a_color_area: EV_DRAWING_AREA)
			-- Assign color in `a_color_row', which is displayed
			-- in `a_color_area' to `an_etest_name'
		require
			an_etest_name_not_void: an_etest_name /= Void
			a_color_row_not_void: a_color_row /= Void
			a_color_area_not_void: a_color_area /= Void
		local
			row: PACKING_ETEST_COLORS_MROW
		do
			if not is_select_blocked then
				create row.make_detached (session.connection)
				row.set_packing_id (packing.packing_id)
				row.set_wafer_etest_name (an_etest_name)
				row.set_color_name (a_color_row.color_name)
				new_split_colors_table.force (row, an_etest_name)
				a_color_area.set_background_color (gui.to_ev_color (a_color_row))
				a_color_area.clear
			end
		end

	assign_custom_color (an_etest_name: STRING a_dialog: EV_COLOR_DIALOG a_name_box: EV_COMBO_BOX a_color_area: EV_DRAWING_AREA)
			-- Assign color selected in `a_dialog' to
			-- `an_etest_name' and display it in `a_color_area'
		require
			an_etest_name_not_void: an_etest_name /= Void
			a_dialog_not_void: a_dialog /= Void
			a_name_box_not_void: a_name_box /= Void
			a_color_area_not_void: a_color_area /= Void
		local
			row: PACKING_ETEST_COLORS_MROW
			l_color: EV_COLOR
		do
			l_color := a_dialog.color
			create row.make_detached (session.connection)
			row.set_packing_id (packing.packing_id)
			row.set_wafer_etest_name (an_etest_name)
			row.set_red_value (l_color.red)
			row.set_green_value (l_color.green)
			row.set_blue_value (l_color.blue)
			new_split_colors_table.force (row, an_etest_name)
			
			a_name_box.enable_edit
			a_name_box.set_text (Custom_name + "  " + gui.color_name_8_bit (l_color))
			a_name_box.disable_edit
			a_color_area.set_background_color (l_color)
			a_color_area.clear
		end

	process_ok
		do
			if is_active then
				disable_active
				set_new_split_colors
				accept_actions.call ([new_split_colors, colors_changed])
			end
		end

	process_cancel
		do
			if is_active then
				disable_active
				cancel_actions.call ([])
			end
		end

	set_new_split_colors
			-- Compute new value for `split_colors' and leave it
			-- in `new_split_colors'.  If anything changed,
			-- update the database and commit
		local
			list: like wafer_etest_names
			name: STRING
			has_old, has_new: BOOLEAN
			old_row, new_row: PACKING_ETEST_COLORS_MROW
		do
			colors_changed := False
			list := wafer_etest_names
			create {ARRAYED_LIST [PACKING_ETEST_COLORS_MROW]} new_split_colors.make (list.count)
			from
				list.start
			until
				list.after
			loop
				name := list.item.wafer_etest_name
				split_colors_table.search (name)
				has_old := split_colors_table.found
				if has_old then
					old_row := split_colors_table.found_item
				end
				new_split_colors_table.search (name)
				has_new := new_split_colors_table.found
				if has_new then
					new_row := new_split_colors_table.found_item
					colors_changed := True
				end
				
				if has_old and has_new then
					-- Update `old_row' from `new_row'
					old_row.set_non_key_fields (new_row)
					new_split_colors.extend (old_row)
				elseif has_old then
					-- Was assigned and is unchanged
					new_split_colors.extend (old_row)
				elseif has_new then
					-- Was unassigned but now assigned
					-- Insert `new_row'
					new_row.register_on_implicit_connection
					new_split_colors.extend (new_row)
				else
					-- Still no split color assigned
				end
				list.forth
			end
			if colors_changed then
				session.connection.commit
				fetch_split_colors
				new_split_colors := split_colors
			end
		end

	ok_button: EV_BUTTON
			-- Button to accept current colors
	
	cancel_button: EV_BUTTON
			-- Button to cancel setting of colors
	
	split_colors_table: HASH_TABLE [PACKING_ETEST_COLORS_MROW, STRING]
			-- Table of existing split color assignment
			-- rows indexed by wafer etest name
	
	new_split_colors: LIST [PACKING_ETEST_COLORS_MROW]
			-- New colors for lot splits
	
	new_split_colors_table: HASH_TABLE [PACKING_ETEST_COLORS_MROW, STRING]
			-- Table of new split color assignment rows indexed
			-- by wafer etest name
	
	named_colors_table: HASH_TABLE [NAMED_COLORS_ROW, STRING]
			-- Table of named color rows indexed
			-- by color name
	
	is_select_blocked: BOOLEAN
			-- Is selection of a named color blocked?
	
	colors_changed: BOOLEAN
			-- Were any lot split colors changed?
	
	Custom_name: STRING = "Custom"
			-- Name for color that is not a known named color
	
	Unassigned_name: STRING = "Not Assigned"
			-- Name for color that has not been assigned yet
			-- and so is the unassigned color
	
end
