
indexing
	
	description: "Window to allow selection of existing packing"
	author: "David Hollenberg"
	date: "November 7, 2005"

class RC_PACKING_VERSIONS_WINDOW

inherit
	RC_PACKING_VERSIONS_IFC
	WINDOW_INTERFACE
		redefine
			enable_active,
			disable_active
		end
	MOSIS_GRAPHICAL_UTILITIES
		export
			{NONE} all
		undefine
			default_create
		end

create
	make

feature {NONE} -- Creation

	make (w: EV_TITLED_WINDOW)
			-- Create interface in window `w'
		require
			window_exists: w /= Void
			window_valid: not w.is_destroyed
		do
			make_interface
			make_proxy (w)
		end

feature -- Properties

	is_read_only: BOOLEAN
			-- Is window read-only (cannot select a version
			-- to edit)?

feature -- Operations

	reset
			-- Move focus back into window
		do
			enable_active
		end

feature -- Modification

	subscribe_events
			-- Subscribe to events of interest
		do
			(create {PACKING_VERSION_CREATED_EVENT}).subscribe (agent handle_version_created)
		end

	unsubscribe_events
			-- Unsubscribe from events of interest
		do
			(create {PACKING_VERSION_CREATED_EVENT}).unsubscribe (agent handle_version_created)
		end

	enable_active
			-- Make interface active
		do
			precursor
			select_button.enable_sensitive
			refresh_button.enable_sensitive
			send_button.enable_sensitive
			cancel_button.enable_sensitive
		end

	disable_active
			-- Make interface inactive
			-- (can look but cannot initiate most actions)
		do
			precursor
			select_button.disable_sensitive
			refresh_button.disable_sensitive
			send_button.disable_sensitive
			cancel_button.disable_sensitive
		end

	enable_read_only
			-- Make window read-only (not able to select a
			-- new version to edit)
		do
			is_read_only := True
			select_button.disable_sensitive
		end

	disable_read_only
			-- Make window allow selection of new version
			-- to edit
		do
			is_read_only := False
			select_button.enable_sensitive
		end

feature {NONE} -- Implementation

	build_interface
			-- Build interface
		local
			buttons: HORIZONTAL_BUTTON_SET
			main_vbox: EV_VERTICAL_BOX
			box: EV_BOX
		do
			add_menu_bar

			create main_vbox
			main_vbox.set_padding (user_interface.padding)
			main_vbox.set_border_width (user_interface.border_width)
			create versions
			versions.set_row_to_generic_function (agent row_to_record)
			versions.build_selection_status_box
			set_visible_columns
			main_vbox.extend (versions)

			create buttons.make_with_text_and_actions (<<
				"Select", "Refresh", "Send", "Cancel" >>,
				<< agent process_ok, agent display_versions, agent process_send, agent process_cancel >>)
			select_button := buttons.i_th (1)
			refresh_button := buttons.i_th (2)
			send_button := buttons.i_th (3)
			cancel_button := buttons.i_th (4)
			select_button.set_tooltip ("Select the currently highlighted packing version to edit or view")
			refresh_button.set_tooltip ("Refresh list of packing versions (only needed if new version created by another session)")
			send_button.set_tooltip ("Send list of packing versions to Output window")
			
			box := buttons.column_set.box
			
			main_vbox.extend (box)
			main_vbox.disable_item_expand (box)
			
			gui.set_box_background_recursive (main_vbox, background_color)
			extend (main_vbox)
			close_request_actions.extend (agent process_cancel)
			gui.add_cancel_accelerator (window, agent process_cancel)
			gui.do_once_on_idle (agent post_display_actions)
		end
		
	add_menu_bar
			-- Add menu bar to window
		local
			file_menu: EV_MENU
			close_menu_item: EV_MENU_ITEM
			a_menu_bar: EV_MENU_BAR
		do	
			create file_menu.make_with_text ("File")
			
			create close_menu_item.make_with_text ("Close")
			close_menu_item.select_actions.extend (agent process_cancel)
			file_menu.extend (close_menu_item)
			
			create a_menu_bar
			a_menu_bar.extend (file_menu)
			set_menu_bar (a_menu_bar)
		end

	post_display_actions
		do
			gui.do_once_on_idle (agent gui.center_window_with_size (window, 700, 600))
			display_versions
			reset
		end

	process_ok
		local
			p: PACKING_VERSIONS_MROW
		do
			if is_active and not is_read_only then
				disable_active
				if selected_version /= Void then
					create p.make_by_key (session.connection, option.Old_row, selected_version.packing_id, selected_version.version_no)
					select_actions.call ([p])
				else
					inform ("You must select a version first")
				end
				enable_active
			end
		end

	handle_version_created (e: PACKING_VERSION_CREATED_EVENT)
		do
			display_versions
		end

	display_versions
		do
				fetch_packing_versions
				build_all_rows
				versions.reverse_sort_on_column (Date_created_position)
				gui.do_once_on_idle (agent versions.resize_all_columns_to_content)
		end

	process_send
			-- Send nicely formatted text of list of versions
			-- to observers
		do
			if is_active then
				disable_active
				send_actions.call (["info", formatted_text (versions.all_rows, versions.visible_columns)])
				enable_active
			end
		end

	formatted_text (a_row_list: LIST [MULTI_COLUMN_LIST_ROW] a_column_list: LIST [MULTI_COLUMN_LIST_COLUMN [MULTI_COLUMN_LIST_ROW]]): STRING
			-- Nicely formatted text representing contents
			-- of `a_row_list', using just the columns in
			-- `a_column_list' in order
			-- in the Output window
		local
			l_formatter: MULTI_COLUMN_LIST_TEXT_FORMATTER [MULTI_COLUMN_LIST_ROW]
		do
			create l_formatter
			Result := l_formatter.formatted_text (a_row_list, a_column_list, "  ", True, True)
		end

	process_cancel
		do
			if is_active then
				disable_active
				cancel_actions.call ([])
			end
		end

	row_to_record (row: EV_MULTI_COLUMN_LIST_ROW): PACKING_VERSION_MULTI_COLUMN_LIST_ROW
			-- Function which translates `row' to object of
			-- type PACKING_VERSION_MULTI_COLUMN_LIST_ROW
		do
			Result ?= row
			check
				result_not_void: Result /= Void
			end
		end

	set_visible_columns
			-- Set visible columns in `packings'
		local
			cols: ARRAYED_LIST [MULTI_COLUMN_LIST_COLUMN [PACKING_VERSION_MULTI_COLUMN_LIST_ROW]]
		do
			create cols.make (10)
			cols.extend (create {VERSION_NO_COLUMN [PACKING_VERSION_MULTI_COLUMN_LIST_ROW]})
			cols.extend (create {DATE_TIME_CREATED_COLUMN [PACKING_VERSION_MULTI_COLUMN_LIST_ROW]})
			cols.extend (create {CREATOR_COLUMN [PACKING_VERSION_MULTI_COLUMN_LIST_ROW]})
			cols.extend (create {VERSION_ID_COLUMN [PACKING_VERSION_MULTI_COLUMN_LIST_ROW]})
			cols.extend (create {PARENT_VERSION_NO_COLUMN [PACKING_VERSION_MULTI_COLUMN_LIST_ROW]})
			cols.extend (create {EMPTY_COLUMN [PACKING_VERSION_MULTI_COLUMN_LIST_ROW]})
			versions.set_visible_columns (cols)
		end
			
	fetch_packing_versions
			-- Fetch all versions of packing
		local
			row: PACKING_VERSIONS_ROW
		do
			create row.make_template
			packing_version_rows := row.rows_for_packing (session.packing_id, "date_created", session.connection, expected_count.At_least_zero)
		end
			
	build_all_rows
			-- Build all rows and put in `packings'
		local
			list: ARRAYED_LIST [PACKING_VERSION_MULTI_COLUMN_LIST_ROW]
		do
			create list.make (packing_version_rows.count)
			packing_version_rows.do_all (agent add_row (?, list))
			versions.set_all_rows (list)
		end
			
	add_row (p: PACKING_VERSIONS_ROW list: LIST [PACKING_VERSION_MULTI_COLUMN_LIST_ROW])
			-- Add a row for `p' to `list'
		local
			row: PACKING_VERSION_MULTI_COLUMN_LIST_ROW
		do
			create row.make (p)
			list.extend (row)
			row.select_actions.extend (agent versions.process_select_one (row))
			row.select_actions.extend (agent select_version (p))
			row.pointer_double_press_actions.force_extend (agent process_ok)
			row.deselect_actions.extend (agent versions.process_deselect_one (row))
			row.deselect_actions.extend (agent deselect_version)
		end
			
	select_version (p: PACKING_VERSIONS_ROW)
			-- Select packing version whose row is `p'
		do
			selected_version := p
		end
			
	deselect_version
			-- Select packing version whose row is `p'
		do
			selected_version := Void
		end
			
	select_button: EV_BUTTON
			-- Button to edit packing version that is selected
	
	refresh_button: EV_BUTTON
			-- Button to refresh list of packing versions
			-- displayed (needed only for read-only session to
			-- see a new version saved by another session)
	
	send_button: EV_BUTTON
			-- Button to send info on packing versions
			-- to observers
	
	cancel_button: EV_BUTTON
			-- Button to hide window
	
	packing_version_rows: LIST [PACKING_VERSIONS_ROW]
			-- List of versions for packing

	versions: MULTI_COLUMN_LIST [PACKING_VERSION_MULTI_COLUMN_LIST_ROW]
			-- List of versions for packing,
			-- displayed in multi-column list

	selected_version: PACKING_VERSIONS_ROW
			-- Row for version currently selected by user

	Date_created_position: INTEGER = 2
			-- Position of "date created" column in
			-- `packings', so we can reverse sort on it

end
