
indexing
	
	description: "Window to allow selection and packing projects"
	author: "David Hollenberg"
	date: "November 7, 2005"

class RC_PACKING_WINDOW

inherit
	RC_PACKING_IFC
		redefine
			version_selection_interface,
			project_management_interface
		end
	WINDOW_INTERFACE
		redefine
			enable_active,
			disable_active
		end
	MOSIS_GRAPHICAL_UTILITIES
		export
			{NONE} all
		undefine
			default_create
		end
	MOSIS_EV_STOCK_COLORS_ACCESS
		export
			{NONE} all
		undefine
			default_create
		end
	RC_POLICIES_ACCESS
		export
			{NONE} all
		undefine
			default_create
		end
	EV_FRAME_CONSTANTS
		export
			{NONE} all
		undefine
			default_create
		end
	DOUBLE_MATH
		export
			{NONE} all
		undefine
			default_create
		end
	NUMBER_UTILITIES_ACCESS
		export
			{NONE} all
		undefine
			default_create
		end
	NAMED_VIEW_NAMES
		export
			{NONE} all
		undefine
			default_create
		end
	APPLICATION_COLOR_TRANSLATION
		export
			{NONE} all
		undefine
			default_create
		end
		
create
	make

feature {NONE} -- Creation

	make (w: EV_TITLED_WINDOW)
			-- Create interface in window `w'
		require
			window_exists: w /= Void
			window_valid: not w.is_destroyed
		do
			make_interface
			make_proxy (w)
		end

feature -- Operations

	reset
			-- Move focus back into window
		do
			enable_active
		end

	close
			-- Close window and dependent windows, without
			-- any confirmation and without saving any changes
		do
			destroy_dependent_interfaces
			if not close_request_actions.is_empty then
				close_request_actions.start
				close_request_actions.remove
			end
			cancel_actions.call ([])
		end

	show_packer_output_window
			-- Build packer output window if it does
			-- not already exist.  Otherwise, just show it
		do
			build_packer_output_window
			packer_output_interface.raise
		end

	show_project_management_window
			-- Build project management window if it does
			-- not already exist.  Otherwise, just show it
		do
			build_project_management_window
			project_management_interface.raise
		end

	show_application_colors_window
			-- Build application colors window if it does
			-- not already exist.  Otherwise, just show it
		do
			build_application_colors_window
			application_colors_interface.raise
		end

	show_version_selector
			-- Process request to show version selection window
		do
			is_version_selector_shown := not is_cut_editor_only
			if is_version_selector_shown then
				show_version_selection_window
			end
		end

	display_info_on_dies (a_list: LIST [RETICLE_DIE])
			-- Process request to display information about
			-- dies (and locations) in `a_list'
		do
			build_packer_info_window
			packer_info_interface.set_dies (a_list)
			show_packer_info_window
		end
	
	show_scribe_properties_window
			-- Process request to show scribe properties
			-- window.  Create window if it does not
			-- already exist.  Otherwise, just show it
		do
			if not scribe_properties_built then
				build_scribe_properties_window
				scribe_properties_interface.show
			else
				scribe_properties_interface.refresh
				scribe_properties_interface.raise
			end
		end

	show_payload_and_kerf_sizes_window
			-- Process request to show payload and
			-- kerf sizes window.  Create window if it does not
			-- already exist.  Otherwise, just show it
		do
			if not payload_sizes_built then
				build_payload_sizes_window
				payload_sizes_interface.show
			else
				payload_sizes_interface.refresh
				payload_sizes_interface.raise
			end
		end

	show_move_absolute_window (an_element: RETICLE_VIEW_ELEMENT fields: LIST [RETICLE_FIELD] max_db_value: DOUBLE)
			-- Process request to show Move Absolute
			-- window to specify where to move `an_element'
		require
			an_element_not_void: an_element /= Void
			fields_not_void: fields /= Void
		local
			w: EV_DIALOG
		do
			create w
			move_absolute_interface ?= session.factory.packing_move_absolute_interface (w, packing_session.run_id, packing_session.packing_id, packing_session.title_with_run_and_packing_id ("Move Absolute for"))
			move_absolute_interface.set_grid (grid)
			move_absolute_interface.set_element (an_element)
			move_absolute_interface.set_fields (fields)
			move_absolute_interface.set_max_db_value (max_db_value)
			move_absolute_interface.build
			move_absolute_interface.accept_actions.extend (agent move_absolute)
			move_absolute_interface.cancel_actions.extend (agent cancel_move_absolute)
			w.show_modal_to_window (window)
		end

	show_move_relative_window
			-- Process request to show Move Relative
			-- window
		local
			w: EV_DIALOG
		do
			create w
			move_relative_interface ?= session.factory.packing_move_relative_interface (w, packing_session.run_id, packing_session.packing_id, packing_session.title_with_run_and_packing_id ("Move Relative for"))
			move_relative_interface.set_grid (grid)
			move_relative_interface.build
			move_relative_interface.accept_actions.extend (agent move_relative)
			move_relative_interface.cancel_actions.extend (agent cancel_move_relative)
			w.show_modal_to_window (window)
		end

	show_filler_properties_window (a_name: STRING a_width, a_height: DOUBLE a_clear_sizes: BOOLEAN a_name_list: LIST [STRING])
			-- Process request to show filler properties
			-- window, with default values `a_name',
			-- `a_width' and `a_height'.  Clear die and
			-- project sizes if `a_clear_sizes' is true.
			-- `a_name_list' has the list of filler names
			-- (uppercased) that have already been used
		require
			a_name_not_void: a_name /= Void
			a_name_list_not_void: a_name_list /= Void
			a_width_positive: a_width > 0.0
			a_height_positive: a_height > 0.0
		local
			w: EV_DIALOG
		do
			create w
			filler_properties_interface ?= session.factory.packing_filler_interface (w, packing_session.run_id, packing_session.packing_id, packing_session.title_with_run_and_packing_id ("Set Filler Properties for"))
			filler_properties_interface.set_session (packing_session)
			filler_properties_interface.set_grid (grid)
			filler_properties_interface.set_maximum_filler_size (maximum_filler_size)
			filler_properties_interface.set_project_to_die_size_agent (agent project_to_die_size)
			filler_properties_interface.set_die_to_project_size_agent (agent die_to_project_size)
			filler_properties_interface.set_default_filler_name (a_name)
			filler_properties_interface.set_default_overhead_style (packings_row.overhead_style)
			filler_properties_interface.set_default_filler_width (a_width)
			filler_properties_interface.set_default_filler_height (a_height)
			filler_properties_interface.set_size_clear_requested (a_clear_sizes)
			filler_properties_interface.set_used_filler_names (a_name_list)
			filler_properties_interface.build
			filler_properties_interface.accept_actions.extend (agent set_filler_properties)
			filler_properties_interface.cancel_actions.extend (agent cancel_filler_creation)
			w.show_modal_to_window (window)
		end

	show_lot_split_colors_window
			-- Process request to show window for viewing and
			-- setting lot split colors
		local
			w: EV_DIALOG
		do
			create w
			lot_split_colors_interface ?= session.factory.packing_lot_split_colors_interface (w, packing_session.run_id, packing_session.packing_id, packing_session.title_with_run_and_packing_id ("Set Lot Split Colors for"))
			lot_split_colors_interface.set_session (packing_session)
			lot_split_colors_interface.set_packing (packings_row)
			lot_split_colors_interface.set_split_colors (split_colors)
			lot_split_colors_interface.set_named_colors (named_colors)
			lot_split_colors_interface.set_unassigned_color (palette.no_lot_color_assigned_color)
			lot_split_colors_interface.set_wafer_etest_names (view.possible_wafer_etest_names)
			lot_split_colors_interface.build
			lot_split_colors_interface.accept_actions.extend (agent set_lot_split_colors)
			lot_split_colors_interface.cancel_actions.extend (agent destroy_lot_split_color_window)
			w.show_modal_to_window (window)
		end

	show_lot_assignment_window (a_list: LIST [PACKING_DIES_MROW])
			-- Process request to show window for assigning a
			-- lot to selected customer projects that have
			-- more than one compatible lot
		require
			a_list_not_void: a_list /= Void
		local
			w: EV_DIALOG
		do
			create w
			lot_assignment_interface ?= session.factory.packing_lot_assign_interface (w, packing_session.run_id, packing_session.packing_id, packing_session.title_with_run_and_packing_id ("Assign Lot for"))
			lot_assignment_interface.set_session (packing_session)
			lot_assignment_interface.set_projects (a_list)
			lot_assignment_interface.build
			lot_assignment_interface.accept_actions.extend (agent assign_lot)
			lot_assignment_interface.cancel_actions.extend (agent destroy_lot_assignment_window)
			w.show_modal_to_window (window)
		end

	show_lot_summary_window
			-- Process request to show lot summary window.
			-- Create window if it does not
			-- already exist.  Otherwise, just show it
		do
			build_lot_summary_window
			lot_summary_interface.refresh
			lot_summary_interface.raise
		end

	show_search_window
			-- Process request to show search window.
			-- Create window if it does not
			-- already exist.  Otherwise, just show it
		do
			build_search_window
			search_interface.raise
		end

	show_help_window
			-- Process request to show help window.
			-- Create window if it does not
			-- already exist.  Otherwise, just show it
		do
			build_help_window
			help_interface.raise
		end

	show_cut_pattern_properties_window (a_number_list: LIST [INTEGER] fields: LIST [RETICLE_FIELD] a_default_field: STRING etest_names: LIST [WAFER_ETEST_NAMES_ROW] a_default_etest_name: STRING a_default_waf_quantity: INTEGER a_pattern_no_changeable, a_field_name_changeable, all_values_required: BOOLEAN)
			-- Process request to show window to set cut pattern
			-- properties for new or existing cut pattern
		require
			a_number_list_not_void: a_number_list /= Void
			a_number_list_not_empty: not a_number_list.is_empty
			fields_not_void: fields /= Void
			fields_not_empty: not fields.is_empty
			etest_names_not_void: etest_names /= Void
		local
			w: EV_DIALOG
		do
			create w
			cut_pattern_properties_interface ?= session.factory.cut_pattern_properties_interface (w, packing_session.run_id, packing_session.packing_id, packing_session.title_with_run_and_packing_id ("Set Cut Pattern Properties for"))
			cut_pattern_properties_interface.set_cut_pattern_numbers (a_number_list)
			cut_pattern_properties_interface.set_fields (fields)
			cut_pattern_properties_interface.set_default_field_name (a_default_field)
			cut_pattern_properties_interface.set_wafer_etest_names (etest_names)
			cut_pattern_properties_interface.set_default_etest_name (a_default_etest_name)
			cut_pattern_properties_interface.set_default_wafer_quantity (a_default_waf_quantity)
			cut_pattern_properties_interface.set_is_pattern_number_changeable (a_pattern_no_changeable)
			cut_pattern_properties_interface.set_is_field_name_changeable (a_field_name_changeable)
			cut_pattern_properties_interface.set_all_values_required (all_values_required)
			cut_pattern_properties_interface.build
			cut_pattern_properties_interface.accept_actions.extend (agent add_cut_pattern)
			cut_pattern_properties_interface.cancel_actions.extend (agent cancel_cut_pattern_properties)
			w.show_modal_to_window (window)
		end

	toggle_pointer_coordinates_display
			-- Toggle display of pointer coordinates
		do
			if is_coordinates_line_displayed then
				hide_pointer_coordinates
			else
				show_pointer_coordinates
			end
		end

	show_delete_window (a_list: LIST [PACKING_PROJECTS_MROW])
			-- Process request to show delete window, where
			-- `a_list' contains the projects whose last copy
			-- is about to be deleted
		require
			a_list_not_void: a_list /= Void
		local
			w: EV_DIALOG
		do
				create w
				project_delete_interface ?= session.factory.packing_delete_interface (w, packing_session.run_id, packing_session.packing_id, packing_session.title_with_run_and_packing_id ("Set Deleted Projects Disposition for"))
				project_delete_interface.set_session (packing_session)
				project_delete_interface.set_deleted_projects (a_list)
				project_delete_interface.build
				project_delete_interface.accept_actions.extend (agent delete_projects)
				project_delete_interface.cancel_actions.extend (agent cancel_project_deletion)
				w.show_modal_to_window (window)
		end

	delete_project_list (a_list: LIST [PACKING_PROJECTS_MROW] wants_unlock, wants_deassign: BOOLEAN)
			-- Try to delete projects in `a_list', using
			-- `wants_unlock' and `wants_deassign'
		do
			if not project_manager_built then
				build_project_management_window
			end
			create operation.make (agent project_management_interface.deselect_project_list (a_list, wants_unlock, wants_deassign))
			operation.enable_abort_caught
			operation.execute
			if operation.aborted then
				controller.cancel_delete
			else
				controller.set_deleted_dies (project_management_interface.removed_dies)
			end
		end

	delete_non_project_list (a_list: LIST [PACKING_DIES_MROW])
			-- Delete dies in `a_list' from packing.  The dies
			-- must not represent projects since then the
			-- project management interface would need to know
			-- about it
		do
			packing.current_version.remove_dies (a_list)
		end

	process_save (force: BOOLEAN)
			-- Handle request to save new version.  If `force',
			-- save even if there are no changes
		do
			disable_active
			is_new_version_saved := False
			if is_cut_editor_only then
				inform ("Save not supported when editing cut patterns for a production")
			elseif not force and then not packing.is_current_version_modified then
				inform ("No changes to save")
			else
				get_new_version_name
				if new_version_name /= Void then
					if has_run_context then
						fetch_runs_row_by_id
					else
						runs_mrow := Void
					end
					save_new_version_with_wait_cursor (new_version_name)
					reset_title
					inform ("Saved " + packing.current_version.version_title)
					packing.detach_current_version
					packing.refresh_versions
					model.process_save_notification
					view.process_save_notification
					is_new_version_saved := True
				end
			end
			enable_active
		end

	process_cancel
		do
			is_quit_completed := False
			if close_request_actions.call_is_underway then
				close_request_actions.abort
			end
			if is_active then
				disable_active
				create operation.make (agent process_cancel_request)
				operation.enable_abort_caught
				operation.execute
				if operation.aborted then
					enable_active
				else
			 		close
					is_quit_completed := True
				end
			end
		end

	process_finish
			-- Handle request to finish packing step, with
			-- aborts caught
		do
			is_finish_completed := False
			create operation.make (agent process_finish_request)
			operation.enable_abort_caught
			operation.execute
			if operation.aborted then
				is_finish_completed := False
				enable_active
			end
		end

	update_coordinates (a_field_list: LIST [RETICLE_FIELD] a_point: CONST_POINT)
			-- Update field-relative coordinates in pointer
			-- coordinates line for each field in `a_field_list'
			-- and new model coordinates `a_point' of
			-- mouse pointer
		local
			l_label: EV_LABEL
			hbox: EV_HORIZONTAL_BOX
			f: RETICLE_FIELD
			s: STRING
			l_x, l_y: DOUBLE
			n: INTEGER
		do
			if is_coordinates_line_displayed then
				if coordinates_labels = Void or else coordinates_labels.count /= a_field_list.count then
					coordinates_labels_box.wipe_out
					create {ARRAYED_LIST [EV_LABEL]} coordinates_labels.make (a_field_list.count)
					from
						a_field_list.start
					until
						a_field_list.after
					loop
						create l_label
						l_label.align_text_left
						coordinates_labels.extend (l_label)
						create hbox
						hbox.set_minimum_width (50)
						hbox.extend (l_label)
						hbox.disable_item_expand (l_label)
						coordinates_labels_box.extend (hbox)
						a_field_list.forth
					end
				end
				from
					a_field_list.start
					n := 1
				until
					a_field_list.after
				loop
					create s.make (20)
					f := a_field_list.item
					l_label := coordinates_labels.i_th (n)
					l_x := a_point.x - f.center_x
					l_y := a_point.y - f.center_y
					s.append ("Field ")
					s.append (f.name)
					s.append (": ")
					s.append (nu.fixed_formatted_number (l_x, Pointer_coordinate_scale))
					s.append (", ")
					s.append (nu.fixed_formatted_number (l_y, Pointer_coordinate_scale))
					l_label.set_text (s)
					n := n + 1
					a_field_list.forth
				end
			end
		end

	switch_to_pack_mode
			-- Do actions needed when switch to Pack mode
		do
			set_interface_for_pack_mode
			view.switch_to_pack_mode
		end

	process_switch_to_cut_mode
			-- Handle request to switch to Cut mode, with
			-- aborts caught
		do
			if is_active then
				disable_active
				is_cut_mode := False
				create operation.make (agent switch_to_cut_mode)
				operation.enable_abort_caught
				operation.execute
				if operation.aborted then
					is_cut_mode := False
				end
				enable_active
			end
		end

feature -- Properties

	is_new_version_saved: BOOLEAN
			-- Did last call to `process_save' save a new version?

	is_version_selector_shown: BOOLEAN
			-- Did last call to `show_version_selector' display 
			-- the version selection window?

	is_quit_completed: BOOLEAN
			-- Did last call to `process_cancel' successfully
			-- quit the packing session?

	is_finish_completed: BOOLEAN
			-- Did last call to `process_finish' successfully
			-- exit the packing session?

	is_cut_mode: BOOLEAN
			-- Did last call to `switch_to_cut_mode' 
			-- successfully switch to Cut mode?

	is_pack_mode: BOOLEAN
			-- Are we in Pack mode, as opposed to Cut mode?
		do
			Result := not view.is_cut_mode
		end

	is_coordinates_line_displayed: BOOLEAN
			-- Is line with field-relative pointer coordinates
			-- currently displayed?
		do
			Result := not coordinates_box.is_empty
		end

feature -- Modification

	enable_active
			-- Make interface active
		do
			precursor
		end

	disable_active
			-- Make interface inactive
			-- (can look but cannot initiate most actions)
		do
			precursor
		end

feature {NONE} -- Implementation

	build_interface
			-- Build interface
		do
			if is_cut_editor_only then
				-- `packing', `packings_row' and
				-- `official_packing_version' already
				-- set by caller
				build_packing_interface
			else
				if has_packing_context then
					fetch_packing_row_by_id
				elseif has_run_context then
					fetch_current_packing_row
				end
				if has_packing_context then
					create packing
					packing.set_packings_row (packings_row)
					build_packing_interface
				else
					-- Has run context but no packing selected
					build_selection_interface
				end
			end
			move_actions.extend (agent handle_window_move)
			resize_actions.extend (agent handle_window_resize)
		end
		
	build_selection_interface
			-- Build interface to select a packing
		local
			ifc: RC_PACKING_IFC
		do
			ifc := session.factory.packing_selection_interface ("Select Packing for " + run_id)
			ifc.set_session (session)
			ifc.set_run_id (run_id)
			create {RC_PACKING_SESSION} packing_session.make_from_session (session)
			packing_session.set_run_id (run_id)
			ifc.set_packing_session (packing_session)
			ifc.cancel_actions.append (cancel_actions)
			ifc.build
		end

	build_packing_interface
			-- Build interface when packing has already
			-- been selected
		local
			main_vbox: EV_VERTICAL_BOX
			toolbar_box: EV_HORIZONTAL_BOX
			frame: EV_FRAME
		do
			is_resize_to_minimum_requested := False
			build_status_box

			lower_bar.extend (status_box)
			
			create_model_view_controller
			create main_vbox
			create toolbar_box
			add_button (toolbar_box, "Projects", "Select, deselect, lock and unlock projects", agent process_manage_projects)
			projects_button := last_button
			add_button (toolbar_box, "Versions", "Select packing version", agent show_version_selector)
			versions_button := last_button
			if is_cut_editor_only then
				versions_button.disable_sensitive
			end

			toolbar_box.extend (create {EV_HORIZONTAL_BOX})
			add_toggle_button (toolbar_box, "Fixed", "Prevent fixed items from moving for horizontal/vertical scan but allow them to be moved manually or by other commands (original meaning of fixed)", agent set_fixed_mode, True)
			fixed_button := last_toggle_button

			add_toggle_button (toolbar_box, "Locked", "Lock fixed items in position and do not allow them to move in the locked dimension for any reason", agent set_locked_mode, False)
			locked_button := last_toggle_button

			toolbar_box.extend (create {EV_HORIZONTAL_BOX})
			add_button (toolbar_box, "Back", "Go back to previous displayed area in history list", agent controller.show_previous_area)
			back_button := last_button

			add_button (toolbar_box, "Forward", "Go forward to next displayed area in history list", agent controller.show_next_area)
			forward_button := last_button

			toolbar_box.extend (create {EV_HORIZONTAL_BOX})
			add_change_button (toolbar_box, "Undo", "Undo previous change in change history list", agent controller.undo_one_change)
			undo_button := last_button

			add_change_button (toolbar_box, "Redo", "Redo next change in change history list", agent controller.redo_one_change)
			redo_button := last_button

			create all_view_buttons.make (11)

			toolbar_box.extend (create {EV_HORIZONTAL_BOX})
			add_toggle_button (toolbar_box, "Standard", "Switch to Standard view, where elements are labeled with design number and colored by fixed (or locked) status", agent set_standard_view, True)
			standard_view_button := last_toggle_button
			all_view_buttons.put (standard_view_button, Standard_fixed_view_name)
			all_view_buttons.put (standard_view_button, Standard_locked_view_name)

			add_toggle_button (toolbar_box, "Acct Type", "Switch to Account Type view, where elements are labeled and colored by account type", agent set_account_type_view, False)
			account_type_view_button := last_toggle_button
			all_view_buttons.put (account_type_view_button, Account_type_view_name)

			add_toggle_button (toolbar_box, "Max Size", "Switch to Max Die Size view, where elements are labeled and colored by whether they have a maximum die size", agent set_max_die_size_view, False)
			max_die_size_view_button := last_toggle_button
			all_view_buttons.put (max_die_size_view_button, Max_die_view_name)

			add_toggle_button (toolbar_box, "Lot Split", "Switch to Lot Split view, where elements are labeled and colored by lot split", agent set_lot_split_view, False)
			lot_split_view_button := last_toggle_button
			all_view_buttons.put (lot_split_view_button, Lot_split_view_name)

			add_toggle_button (toolbar_box, "Packaging", "Switch to Packaging view, where elements are labeled by package name and colored by packaging vendor type", agent set_packaging_view, False)
			packaging_view_button := last_toggle_button
			all_view_buttons.put (packaging_view_button, Packaging_view_name)

			add_toggle_button (toolbar_box, "Validity", "Switch to Validity Check view, where elements are colored according to whether any validity errors or warnings were found during last check", agent set_validity_check_view, False)
			validity_check_view_button := last_toggle_button
			all_view_buttons.put (validity_check_view_button, Validity_check_view_name)

			add_toggle_button (toolbar_box, "Cut", "Switch to Cut view, where elements are colored by lot split and label by design number", agent set_cut_view, False)
			cut_view_button := last_toggle_button
			all_view_buttons.put (cut_view_button, Cut_view_name)

			toolbar_box.extend (create {EV_HORIZONTAL_BOX})
			add_change_button (toolbar_box, "Save", "Save a new version of packing", agent controller.save_new_version)
			save_button := last_button

			if has_run_context then
				add_change_button (toolbar_box, "Finish", "Finish packing and continue on to next step", agent controller.finish)
				finish_button := last_button
			end
			upper_bar.extend (toolbar_box)
			
			add_menu_bar
			set_interface_for_pack_mode
			drawing_area_event_handler.set_is_fix_mapped_to_lock (False)
			enable_fixed_menus
			create frame
			frame.extend (drawing_area)
			main_vbox.set_border_width (Main_border_width)
			main_vbox.extend (frame)
			gui.set_box_background_recursive (main_vbox, background_color)
			extend (main_vbox)
			
			close_request_actions.put_front (agent controller.quit)
			key_press_actions.extend (agent drawing_area_event_handler.handle_key_press)
			key_release_actions.extend (agent drawing_area_event_handler.handle_key_release)
			gui.center_window_with_margin (window, 100, 50)
			gui.do_once_on_idle (agent post_display_actions)
			saved_pointer_style := pointer_style
			set_pointer_style (gui.Wait_cursor)
		end
		
	build_status_box
			-- Build box with elements that display status
			-- information and leave in `status_box'
		local
			selected_count_frame, area_frame: EV_FRAME
			named_view_frame: EV_FRAME
			status_line_box: EV_HORIZONTAL_BOX
		do
			create status_label.make_with_text ("")
			status_label.align_text_left
			status_label.set_minimum_width (100)

			create selected_count_label.make_with_text ("")
			selected_count_frame := new_frame_with_label (selected_count_label)

			create area_label.make_with_text ("")
			area_frame := new_frame_with_label (area_label)
			
			create named_view_label.make_with_text ("")
			named_view_frame := new_frame_with_label (named_view_label)
			
			create status_line_box
			status_line_box.extend (status_label)
			status_line_box.extend (named_view_frame)
			status_line_box.disable_item_expand (named_view_frame)
			status_line_box.extend (area_frame)
			status_line_box.disable_item_expand (area_frame)
			status_line_box.extend (selected_count_frame)
			status_line_box.disable_item_expand (selected_count_frame)
			
			create coordinates_box
			create coordinates_labels_box
			create coordinates_separator
			create status_box
			status_box.extend (coordinates_box)
			status_box.extend (create {EV_HORIZONTAL_SEPARATOR})
			status_box.extend (status_line_box)
			status_box.extend (create {EV_HORIZONTAL_SEPARATOR})
		end
		
	new_frame_with_label (a_label: EV_LABEL): EV_FRAME
			-- Frame with `a_label' inside it
		local
			label_box: EV_HORIZONTAL_BOX
			hbox: EV_HORIZONTAL_BOX
		do
			create label_box
			label_box.set_padding_width (5)
			create hbox
			label_box.extend (hbox)
			label_box.disable_item_expand (hbox)
			label_box.extend (a_label)
			create hbox
			label_box.extend (hbox)
			label_box.disable_item_expand (hbox)
			create Result
			Result.extend (label_box)
			Result.set_style (Ev_frame_lowered)
		end
		
	create_model_view_controller
			-- Create model, view and controller
		do
			create status_reporter
			status_reporter.status_line_update_actions.extend (agent display_in_status_line)
			status_reporter.status_window_update_actions.extend (agent display_in_status_window)
			create drawing_area
			create model.make (100)
			create_view
			create_palette
			view.set_palette (palette)
			create controller.make (model, view)
			controller.set_main_window (Current)
			controller.set_reporter (status_reporter)
			controller.set_is_read_only (is_read_only)
			controller.initialize
			create drawing_area_event_handler.make (drawing_area, controller)
			drawing_area_event_handler.set_reporter (status_reporter)
			drawing_area_event_handler.set_main_window (Current)
			drawing_area_event_handler.set_is_read_only (is_read_only)
			drawing_area_event_handler.bind
			drawing_area_event_handler.disable_key_press_handling
		end
		
	create_view
			-- Create view
		local
			min_sep, dummy: DOUBLE
		do
			create view.make (model, drawing_area)
			view.disable_redraw
			view.set_reporter (status_reporter)
			view.set_maximum_pixels_per_grid_unit_from_dist (policies.max_inches_per_grid_unit)
			view.set_maximum_model_units_per_pixel (policies.max_model_units_per_inch, 1.0)
			view.set_minimum_zoom_dimension_in_pixels (policies.min_zoom_dimension_in_pixels)
			view.set_minimum_selection_dimension_in_pixels (policies.min_select_dimension_in_pixels)
			view.set_minimum_scribe_dimension_in_pixels (policies.min_scribe_dimension_in_pixels)
			view.set_minimum_dup_distance_in_pixels (policies.min_dup_distance_in_pixels)
			view.set_saved_area_capacity (policies.max_saved_areas)
			view.set_displayed_area_history_capacity (policies.displayed_area_hist_capacity)
			view.set_reticle_change_history_capacity (policies.reticle_change_hist_capacity)
			view.set_corner_snap_limit_in_pixels_from_dist (policies.corner_snap_limit_in_inches)
			view.set_measure_marker_size_in_pixels_from_dist (policies.measure_marker_size_in_inches)
			view.set_measure_marker_line_width (policies.measure_marker_line_width)
			view.selected_count_change_actions.extend (agent update_selected_count)
			view.displayed_dimensions_change_actions.extend (agent update_dimensions)
			view.named_view_change_actions.extend (agent update_view)
			view.locked_mode_change_actions.extend (agent update_locked_mode)
			view.area_history_list_position_actions.extend (agent update_area_history_positions)
			view.change_history_list_position_actions.extend (agent update_change_history_positions)
		end
		
	set_initial_scribe_widths
			-- Set initial widths for newly created
			-- scribes (default value or saved value from
			-- a previous session)
		local
			def_width, horiz_width, vert_width: DOUBLE
		do
			def_width := default_scribe_width
			horiz_width := def_width
			vert_width := def_width
			if not packings_row.is_null_horizontal_scribe_width then
				-- Override default horizontal width
				-- with saved value
				horiz_width := packings_row.horizontal_scribe_width
			end
			if not packings_row.is_null_vertical_scribe_width then
				-- Override default vertical width
				-- with saved value
				vert_width := packings_row.vertical_scribe_width
			end

			-- Check that initial scribe widths are valid

			if horiz_width <= 0.0 then
				handle_hard_error ("initial horizontal scribe width " + grid.formatted_number (horiz_width) + " is not positive")
			elseif not grid.is_size_on_grid (horiz_width) then
				handle_hard_error ("initial horizontal scribe width " + grid.formatted_number (horiz_width) + " is not a multiple of size resolution " + grid.size_resolution_text)
			end
			if vert_width <= 0.0 then
				handle_hard_error ("initial vertical scribe width " + grid.formatted_number (vert_width) + " is not positive")
			elseif not grid.is_size_on_grid (vert_width) then
				handle_hard_error ("initial vertical scribe width " + grid.formatted_number (vert_width) + " is not a multiple of size resolution " + grid.size_resolution_text)
			end
			
			view.set_custom_horizontal_scribe_width (horiz_width)
			view.set_custom_vertical_scribe_width (vert_width)
		end
		
	create_palette
			-- Create color palette
		do
			create palette
			palette.set_standard_background_color (gui.to_ev_color (policies.standard_background_color))
			palette.set_current_background_color (palette.standard_background_color)
			palette.set_unselectable_color (gui.to_ev_color (policies.unselectable_color))
			palette.set_default_border_color (gui.to_ev_color (policies.default_border_color))
			palette.set_default_label_color (gui.to_ev_color (policies.default_label_color))
			palette.set_default_fill_color (gui.to_ev_color (policies.default_fill_color))
			palette.set_fully_zoomed_in_background_color (gui.to_ev_color (policies.fully_zoomed_background_color))
			palette.set_not_fixed_in_x_or_y_color (gui.to_ev_color (policies.not_fixed_in_x_or_y_color))
			palette.set_fixed_in_x_only_color (gui.to_ev_color (policies.fixed_in_x_only_color))
			palette.set_fixed_in_y_only_color (gui.to_ev_color (policies.fixed_in_y_only_color))
			palette.set_fixed_in_x_and_y_color (gui.to_ev_color (policies.fixed_in_x_and_y_color))
			palette.set_not_locked_in_x_or_y_color (gui.to_ev_color (policies.not_locked_in_x_or_y_color))
			palette.set_locked_in_x_only_color (gui.to_ev_color (policies.locked_in_x_only_color))
			palette.set_locked_in_y_only_color (gui.to_ev_color (policies.locked_in_y_only_color))
			palette.set_locked_in_x_and_y_color (gui.to_ev_color (policies.locked_in_x_and_y_color))
			palette.set_has_max_die_size_color (gui.to_ev_color (policies.has_max_die_size_color))
			palette.set_tight_fit_color (gui.to_ev_color (policies.tight_fit_color))
			palette.set_no_max_die_size_color (gui.to_ev_color (policies.no_max_die_size_color))
			palette.set_no_packaging_color (gui.to_ev_color (policies.no_packaging_color))
			palette.set_ceramic_packaging_color (gui.to_ev_color (policies.ceramic_packaging_color))
			palette.set_plastic_packaging_color (gui.to_ev_color (policies.plastic_packaging_color))
			palette.set_ceramic_and_plastic_packaging_color (gui.to_ev_color (policies.ceramic_and_plastic_packaging_color))
			palette.set_search_match_color (gui.to_ev_color (policies.search_match_color))
			palette.set_search_no_match_color (gui.to_ev_color (policies.search_no_match_color))
			palette.set_search_match_fill_color (gui.to_ev_color (policies.search_match_fill_color))
			palette.set_highlighted_selected_color (gui.to_ev_color (policies.highlighted_selected_color))
			palette.set_highlighted_unselected_color (gui.to_ev_color (policies.highlighted_unselected_color))
			palette.set_highlighted_selected_fill_color (gui.to_ev_color (policies.highlighted_selected_fill_color))
			palette.set_commercial_project_color (gui.to_ev_color (policies.commercial_project_color))
			palette.set_mep_project_color (gui.to_ev_color (policies.mep_project_color))
			palette.set_other_project_color (gui.to_ev_color (policies.other_project_color))
			palette.set_non_project_color (gui.to_ev_color (policies.non_project_color))
			palette.set_external_overhead_color (gui.to_ev_color (policies.external_overhead_color))
			palette.set_internal_overhead_color (gui.to_ev_color (policies.internal_overhead_color))
			palette.set_none_overhead_color (gui.to_ev_color (policies.none_overhead_color))
			palette.set_no_overhead_color (gui.to_ev_color (policies.no_overhead_color))
			palette.set_cut_color (gui.to_ev_color (policies.cut_color))
			palette.set_selected_fill_color (gui.to_ev_color (policies.selected_fill_color))
			palette.set_unselected_fill_color (gui.to_ev_color (policies.unselected_fill_color))
			palette.set_locked_in_x_fill_color (gui.to_ev_color (policies.locked_in_x_fill_color))
			palette.set_locked_in_y_fill_color (gui.to_ev_color (policies.locked_in_y_fill_color))
			palette.set_locked_in_x_and_y_fill_color (gui.to_ev_color (policies.locked_in_x_and_y_fill_color))
			palette.set_no_max_die_size_fill_color (gui.to_ev_color (policies.no_max_die_size_fill_color))
			palette.set_standard_rubber_box_color (gui.to_ev_color (policies.standard_rubber_box_color))
			palette.set_too_small_rubber_box_color (gui.to_ev_color (policies.too_small_rubber_box_color))
			palette.set_horizontal_scribe_rubber_box_color (gui.to_ev_color (policies.horizontal_scribe_rubber_box_color))
			palette.set_vertical_scribe_rubber_box_color (gui.to_ev_color (policies.vertical_scribe_rubber_box_color))
			palette.set_standard_move_box_color (gui.to_ev_color (policies.standard_move_box_color))
			palette.set_standard_duplicate_box_color (gui.to_ev_color (policies.standard_duplicate_box_color))
			palette.set_no_lot_color_assigned_color (gui.to_ev_color (policies.no_lot_color_assigned_color))
			palette.set_no_lot_assigned_color (gui.to_ev_color (policies.no_lot_assigned_color))
			palette.set_no_validity_issues_color (gui.to_ev_color (policies.no_validity_issues_color))
			palette.set_multiple_validity_issues_color (gui.to_ev_color (policies.multiple_validity_issues_color))
			palette.set_unlocked_or_unassigned_color (gui.to_ev_color (policies.unlocked_or_unassigned_color))
			palette.set_outside_all_fields_color (gui.to_ev_color (policies.outside_all_fields_color))
			palette.set_overlaps_field_edge_color (gui.to_ev_color (policies.overlaps_field_edge_color))
			palette.set_overlaps_another_object_color (gui.to_ev_color (policies.overlaps_another_object_color))
			palette.set_below_minimum_size_color (gui.to_ev_color (policies.below_minimum_size_color))
			palette.set_needs_lot_assignment_color (gui.to_ev_color (policies.needs_lot_assignment_color))
			palette.set_floating_scribe_color (gui.to_ev_color (policies.floating_scribe_color))
			palette.set_misaligned_scribe_color (gui.to_ev_color (policies.misaligned_scribe_color))
			palette.set_too_close_to_object_color (gui.to_ev_color (policies.too_close_to_object_color))
			palette.set_too_close_to_edge_color (gui.to_ev_color (policies.too_close_to_edge_color))
			palette.set_small_but_legal_color (gui.to_ev_color (policies.small_but_legal_color))
			palette.set_cut_die_too_big_color (gui.to_ev_color (policies.cut_die_too_big_color))
			palette.set_not_isolated_in_cut_die_color (gui.to_ev_color (policies.not_isolated_in_cut_die_color))
			palette.set_not_centered_in_cut_die_color (gui.to_ev_color (policies.not_centered_in_cut_die_color))
			palette.set_measurement_marker_color (gui.to_ev_color (policies.measurement_marker_color))
			palette.set_isolated_in_curr_pattern_fill_color (gui.to_ev_color (policies.isolated_in_curr_pattern_fill_color))
			palette.set_isolated_in_other_pattern_fill_color (gui.to_ev_color (policies.isolated_in_other_pattern_fill_color))
			palette.set_invalid_isolated_cut_die_color (gui.to_ev_color (policies.invalid_isolated_cut_die_color))
			palette.set_wrong_lot_color (gui.to_ev_color (policies.wrong_lot_color))
			palette.set_non_default_cut_die_size_color (gui.to_ev_color (policies.non_default_cut_die_size_color))
			palette.set_shared_cut_die_color (gui.to_ev_color (policies.shared_cut_die_color))
			palette.set_extra_cut_color (gui.to_ev_color (policies.extra_cut_color))
		
			fetch_split_colors
			palette.set_split_colors (split_colors)
		end
		
	fetch_split_colors
			-- Fetch existing lot split color assignments
			-- from database
		local
			row: PACKING_ETEST_COLORS_MROW
		do
			create row.make_template
			split_colors := row.rows_for_packing_id (packing.packing_id, packing.connection, expected_count.At_least_zero)
		end

	add_button (a_box: EV_BOX a_text, a_tooltip: STRING an_action: PROCEDURE [ANY, TUPLE])
			-- Add to `a_box' a new button that cannot
			-- expand, with `a_text' and `a_tooltip' that
			-- calls `an_action' when clicked.  The action
			-- must not change the packing.  Leave new
			-- button in `last_button'
		do
			create last_button.make_with_text_and_action (a_text, an_action)
			last_button.set_tooltip (a_tooltip)
			a_box.extend (last_button)
			a_box.disable_item_expand (last_button)
		end

	add_change_button (a_box: EV_BOX a_text, a_tooltip: STRING an_action: PROCEDURE [ANY, TUPLE])
			-- Add to `a_box' a new button that cannot
			-- expand, with `a_text' and `a_tooltip' that
			-- calls `an_action' when clicked.  The action
			-- may potentially change the packing if packer is
			-- not in read-only mode.  Leave new
			-- button in `last_button'
		do
			create last_button.make_with_text_and_action (a_text, an_action)
			last_button.set_tooltip (a_tooltip)
			a_box.extend (last_button)
			a_box.disable_item_expand (last_button)
			if is_read_only then
				last_button.disable_sensitive
			end
		end

	create_toggle_button (a_text, a_tooltip: STRING an_action: PROCEDURE [ANY, TUPLE] selected: BOOLEAN)
			-- Create new toggle button with `a_text' and
			-- `a_tooltip' that calls `an_action' when
			-- clicked and is selected if `selected'.
			-- Leave new button in `last_toggle_button'
		do
			create last_toggle_button.make_with_text_and_action (a_text, an_action)
			if selected then
				last_toggle_button.enable_select
			else
				last_toggle_button.disable_select
			end
			last_toggle_button.set_tooltip (a_tooltip)
		end

	add_toggle_button (a_box: EV_BOX a_text, a_tooltip: STRING an_action: PROCEDURE [ANY, TUPLE] selected: BOOLEAN)
			-- Add to `a_box' a new toggle button that cannot
			-- expand, with `a_text' and `a_tooltip' that
			-- calls `an_action' when clicked and is selected if
			-- `selected'.  Leave new
			-- button in `last_toggle_button'
		do
			create_toggle_button (a_text, a_tooltip, an_action, selected)
			a_box.extend (last_toggle_button)
			a_box.disable_item_expand (last_toggle_button)
		end

	add_menu_bar
			-- Add menu bar to window, starting in Pack mode
		local
			a_menu_bar: EV_MENU_BAR
		do	
			build_file_menu

			-- Build Edit menus for Cut and Pack mode

			build_pack_edit_menu
			build_cut_edit_menu

			build_select_menu
			build_view_menu
			build_go_to_menu
			build_pack_check_menu
			build_cut_check_menu
			build_help_menu
			create a_menu_bar
			a_menu_bar.extend (file_menu)
			a_menu_bar.extend (create {EV_MENU})
				-- Dummy Edit menu, which will be replaced
				-- by Pack mode or Cut mode Edit menu
			a_menu_bar.extend (select_menu)
			a_menu_bar.extend (view_menu)
			a_menu_bar.extend (go_to_menu)
			a_menu_bar.extend (create {EV_MENU})
				-- Dummy Check menu, which will be replaced
				-- by Pack mode or Cut mode Check menu
			a_menu_bar.extend (help_menu)
			set_menu_bar (a_menu_bar)
		end

	add_menu_item (a_menu: EV_MENU a_text: STRING an_action: PROCEDURE [ANY, TUPLE])
			-- Add to `a_menu' new menu item with `a_text'
			-- that invokes `an_action' when selected,
			-- where the action does not change the packing
			-- (is safe to call when packer in read-only mode).
			-- Leave new item in `last_menu_item'
		require
			a_menu_not_void: a_menu /= Void
			a_text_not_void: a_text /= Void
			an_action_not_void: an_action /= Void
		do
			create last_menu_item.make_with_text (a_text)
			last_menu_item.select_actions.extend (an_action)
			a_menu.extend (last_menu_item)
		end

	add_change_menu_item (a_menu: EV_MENU a_text: STRING an_action: PROCEDURE [ANY, TUPLE])
			-- Add to `a_menu' new menu item with `a_text'
			-- that invokes `an_action' when selected, where
			-- the action may change the packing.
			-- Leave new item in `last_menu_item'
		require
			a_menu_not_void: a_menu /= Void
			a_text_not_void: a_text /= Void
			an_action_not_void: an_action /= Void
		do
			add_menu_item (a_menu, a_text, an_action)
			if is_read_only then
				last_menu_item.disable_sensitive
			end
		end

	menu_text (a_text, an_accelerator_text: STRING): STRING
			-- Text for menu item with `a_text' and accelerator
			-- an_accelerator_text
		require
			a_text_not_void: a_text /= Void
			an_accelerator_text_not_void: an_accelerator_text /= Void
		do
			Result := a_text + "%T" + an_accelerator_text
		end

	build_file_menu
			-- Build the File menu
		do	
			create file_menu.make_with_text ("File")
			add_change_menu_item (file_menu, menu_text ("Save", "G"), agent controller.save_new_version)
			if is_cut_editor_only then
				last_menu_item.disable_sensitive
			end
			add_menu_item (file_menu, menu_text ("Manage Projects", "D P"), agent process_manage_projects)
			add_menu_item (file_menu, menu_text ("Select Version", "D V"), agent show_version_selector)
			if is_cut_editor_only then
				last_menu_item.disable_sensitive
			end
			add_menu_item (file_menu, menu_text ("Show Output Window", "D O"), agent show_packer_output_window)
			add_menu_item (file_menu, menu_text ("Show Pointer Coordinates", "Ctrl+M"), agent controller.toggle_pointer_coordinates_display)
			pointer_coords_menu_item := last_menu_item
			add_menu_item (file_menu, menu_text ("Close", "Q"), agent controller.quit)
		end

	build_pack_edit_menu
			-- Build the Edit menu for Pack mode
		local
			n: INTEGER
		do	
			create pack_edit_menu.make_with_text (Edit_text)
			create pack_fix_menu.make_with_text (Fix_text)
			build_fix_menu (pack_fix_menu)
			create pack_unfix_menu.make_with_text (Unfix_text)
			build_unfix_menu (pack_unfix_menu)
			create pack_lock_menu.make_with_text (Lock_text)
			build_lock_menu (pack_lock_menu)
			create pack_unlock_menu.make_with_text (Unlock_text)
			build_unlock_menu (pack_unlock_menu)
			pack_edit_menu.extend (pack_fix_menu)
			pack_edit_menu.extend (pack_unfix_menu)
			pack_edit_menu.extend (pack_lock_menu)
			pack_edit_menu.extend (pack_unlock_menu)
			pack_edit_menu.extend (create {EV_MENU_SEPARATOR})
			add_menu_item (pack_edit_menu, menu_text ("Measure in X", "M X"), agent controller.measure_selected_items_in_x)
			add_menu_item (pack_edit_menu, menu_text ("Measure in Y", "M Y"), agent controller.measure_selected_items_in_y)
			pack_edit_menu.extend (create {EV_MENU_SEPARATOR})
			add_change_menu_item (pack_edit_menu, menu_text ("Horizontal Scan", "H"), agent controller.horizontal_scan_selected_items)
			add_change_menu_item (pack_edit_menu, menu_text ("Vertical Scan", "V"), agent controller.vertical_scan_selected_items)
			pack_edit_menu.extend (create {EV_MENU_SEPARATOR})
			add_change_menu_item (pack_edit_menu, menu_text ("Move Absolute", "Shift+M A"), agent controller.move_absolute_selected_items)
			add_change_menu_item (pack_edit_menu, menu_text ("Move Relative", "Shift+M R"), agent controller.move_relative_selected_items)
			pack_edit_menu.extend (create {EV_MENU_SEPARATOR})
			add_change_menu_item (pack_edit_menu, menu_text ("Center All", "C A"), agent controller.center_all_projects_in_cut_die)
			add_change_menu_item (pack_edit_menu, menu_text ("Center Selected", "C S"), agent controller.center_selected_projects_in_cut_die)
			pack_edit_menu.extend (create {EV_MENU_SEPARATOR})
			add_change_menu_item (pack_edit_menu, menu_text ("Delete", "K"), agent controller.delete_selected_items)
			add_change_menu_item (pack_edit_menu, menu_text ("Delete Scribes", "Shift+K"), agent controller.delete_selected_scribes)
			pack_edit_menu.extend (create {EV_MENU_SEPARATOR})
			add_change_menu_item (pack_edit_menu, menu_text ("Toggle Rotation", "R"), agent controller.toggle_rotation_of_selected_items)
			add_change_menu_item (pack_edit_menu, menu_text ("Rotate Clockwise", "Shift+R"), agent controller.rotate_selected_items_90_clockwise)
			add_change_menu_item (pack_edit_menu, menu_text ("Assign Lot", "Shift+L"), agent controller.assign_lot)
			add_menu_item (pack_edit_menu, "Compute Cut Dies", agent controller.compute_cut_dies)
			pack_edit_menu.extend (create {EV_MENU_SEPARATOR})
			add_change_menu_item (pack_edit_menu, menu_text ("Undo", "Ctrl+Z"), agent controller.undo_one_change)
			pack_undo_menu_item := last_menu_item
			add_change_menu_item (pack_edit_menu, menu_text ("Redo", "Ctrl+Y"), agent controller.redo_one_change)
			pack_redo_menu_item := last_menu_item
			pack_edit_menu.extend (create {EV_MENU_SEPARATOR})
			add_change_menu_item (pack_edit_menu, menu_text ("Join Scribes", "J"), agent controller.join_scribes)
			add_change_menu_item (pack_edit_menu, menu_text ("Trim All Scribes", "Ctrl+T A"), agent controller.trim_all_scribes)
			add_change_menu_item (pack_edit_menu, menu_text ("Trim Selected Scribes", "Ctrl+T S"), agent controller.trim_selected_scribes)
			add_change_menu_item (pack_edit_menu, menu_text ("Separate All Scribes in Multiple Fields", "Ctrl+B A"), agent controller.separate_all_scribes)
			add_change_menu_item (pack_edit_menu, menu_text ("Separate Selected Scribes in Multiple Fields", "Ctrl+B S"), agent controller.separate_selected_scribes)
			add_change_menu_item (pack_edit_menu, menu_text ("Extend All Scribes", "Ctrl+E A"), agent controller.extend_all_scribes)
			add_change_menu_item (pack_edit_menu, menu_text ("Extend Selected Scribes", "Ctrl+E S"), agent controller.extend_selected_scribes)
			add_change_menu_item (pack_edit_menu, menu_text ("Unoverlap All Scribes", "Ctrl+U A"), agent controller.unoverlap_all_scribes)
			add_change_menu_item (pack_edit_menu, menu_text ("Unoverlap Selected Scribes", "Ctrl+U S"), agent controller.unoverlap_selected_scribes)
			pack_edit_menu.extend (create {EV_MENU_SEPARATOR})
			add_menu_item (pack_edit_menu, menu_text ("Display Die Information", "T or D D"), agent controller.display_info_on_selected_items)
			add_menu_item (pack_edit_menu, menu_text ("Display Lot Summary", "D L"), agent controller.display_lot_summary)
			add_menu_item (pack_edit_menu, menu_text ("Find", "Ctrl+F"), agent controller.search_packing)
			pack_edit_menu.extend (create {EV_MENU_SEPARATOR})
			add_menu_item (pack_edit_menu, menu_text ("Set Payload and External Kerf Sizes", "D S"), agent controller.set_payload_and_kerf_sizes)
			add_change_menu_item (pack_edit_menu, menu_text ("Set Scribe Widths", "W or D W"), agent controller.set_scribe_widths)
			add_change_menu_item (pack_edit_menu, menu_text ("Set Lot Split Colors", "D C"), agent controller.set_lot_split_colors)
			add_menu_item (pack_edit_menu, menu_text ("Set Application Colors", "D A"), agent controller.set_application_colors)
			pack_edit_menu.extend (create {EV_MENU_SEPARATOR})
			add_menu_item (pack_edit_menu, "Refresh", agent controller.refresh_drawing_area)
			add_menu_item (pack_edit_menu, "Redraw", agent controller.redraw_drawing_area)
			pack_edit_menu.extend (create {EV_MENU_SEPARATOR})
			add_menu_item (pack_edit_menu, menu_text ("Switch to Cut Mode", "Shift+C"), agent controller.set_cut_mode)
			pack_edit_menu.extend (create {EV_MENU_SEPARATOR})
			from
				n := 1
			until
				n > view.saved_area_capacity
			loop
				add_menu_item (pack_edit_menu, menu_text ("Set Saved Area " + n.out, "Ctrl+" + n.out), agent controller.set_saved_area (n))
				n := n + 1
			end
		end

	build_cut_edit_menu
			-- Build the Edit menu for cut mode
		local
			n: INTEGER
		do	
			create cut_edit_menu.make_with_text (Edit_text)
			add_change_menu_item (cut_edit_menu, menu_text ("Add Cut Pattern", "A"), agent controller.add_cut_pattern)
			add_change_menu_item (cut_edit_menu, menu_text ("Remove Cut Pattern", "R"), agent controller.delete_cut_pattern)
			add_change_menu_item (cut_edit_menu, menu_text ("Modify Completed Cut Pattern", "Shift+M"), agent controller.modify_cut_pattern)
			add_change_menu_item (cut_edit_menu, menu_text ("Save Completed Cut Pattern", "S"), agent controller.save_cut_pattern)
			add_change_menu_item (cut_edit_menu, menu_text ("Set Cut Pattern Properties", "P"), agent controller.set_cut_pattern_properties)
			cut_edit_menu.extend (create {EV_MENU_SEPARATOR})
			add_change_menu_item (cut_edit_menu, menu_text ("Create Cut Through Every Scribe", "B"), agent controller.synthesize_all_cuts)
			add_change_menu_item (cut_edit_menu, menu_text ("Delete Selected Cuts", "K"), agent controller.delete_selected_cuts)
			add_change_menu_item (cut_edit_menu, menu_text ("Delete Intersecting Cuts", "Shift+K"), agent controller.delete_intersecting_cuts)
			cut_edit_menu.extend (create {EV_MENU_SEPARATOR})
			add_change_menu_item (cut_edit_menu, menu_text ("Compute Default Isolation", "Shift+I"), agent controller.compute_default_isolation)
			add_change_menu_item (cut_edit_menu, menu_text ("Toggle Isolation", "I"), agent controller.toggle_isolation_of_selected_projects)
			cut_edit_menu.extend (create {EV_MENU_SEPARATOR})
			add_change_menu_item (cut_edit_menu, menu_text ("Switch to Next Field", "N"), agent controller.switch_to_next_field)
			cut_edit_menu.extend (create {EV_MENU_SEPARATOR})
			add_menu_item (cut_edit_menu, menu_text ("Switch to Pack Mode", "Shift+C"), agent controller.set_pack_mode)
			if is_cut_editor_only then
				last_menu_item.disable_sensitive
			end
			cut_edit_menu.extend (create {EV_MENU_SEPARATOR})
			create cut_fix_menu.make_with_text (Fix_text)
			build_fix_menu (cut_fix_menu)
			cut_fix_menu.disable_sensitive
			create cut_unfix_menu.make_with_text (Unfix_text)
			build_unfix_menu (cut_unfix_menu)
			cut_unfix_menu.disable_sensitive
			create cut_lock_menu.make_with_text (Lock_text)
			build_lock_menu (cut_lock_menu)
			cut_lock_menu.disable_sensitive
			create cut_unlock_menu.make_with_text (Unlock_text)
			build_unlock_menu (cut_unlock_menu)
			cut_unlock_menu.disable_sensitive
			cut_edit_menu.extend (cut_fix_menu)
			cut_edit_menu.extend (cut_unfix_menu)
			cut_edit_menu.extend (cut_lock_menu)
			cut_edit_menu.extend (cut_unlock_menu)
			cut_edit_menu.extend (create {EV_MENU_SEPARATOR})
			add_menu_item (cut_edit_menu, menu_text ("Measure in X", "M X"), agent controller.measure_selected_items_in_x)
			add_menu_item (cut_edit_menu, menu_text ("Measure in Y", "M Y"), agent controller.measure_selected_items_in_y)
			cut_edit_menu.extend (create {EV_MENU_SEPARATOR})
			add_change_menu_item (cut_edit_menu, menu_text ("Undo", "Ctrl+Z"), agent controller.undo_one_change)
			cut_undo_menu_item := last_menu_item
			add_change_menu_item (cut_edit_menu, menu_text ("Redo", "Ctrl+Y"), agent controller.redo_one_change)
			cut_redo_menu_item := last_menu_item
			cut_edit_menu.extend (create {EV_MENU_SEPARATOR})
			add_menu_item (cut_edit_menu, menu_text ("Display Die Information", "T or D D"), agent controller.display_info_on_selected_items)
			add_menu_item (cut_edit_menu, menu_text ("Display Lot Summary", "D L"), agent controller.display_lot_summary)
			add_menu_item (cut_edit_menu, menu_text ("Find", "Ctrl+F"), agent controller.search_packing)
			cut_edit_menu.extend (create {EV_MENU_SEPARATOR})
			add_change_menu_item (cut_edit_menu, menu_text ("Set Scribe Widths", "W or D W"), agent controller.set_scribe_widths)
			add_change_menu_item (cut_edit_menu, menu_text ("Set Lot Split Colors", "D C"), agent controller.set_lot_split_colors)
			add_menu_item (cut_edit_menu, menu_text ("Set Application Colors", "D A"), agent controller.set_application_colors)
			cut_edit_menu.extend (create {EV_MENU_SEPARATOR})
			add_menu_item (cut_edit_menu, "Refresh", agent controller.refresh_drawing_area)
			add_menu_item (cut_edit_menu, "Redraw", agent controller.redraw_drawing_area)
			cut_edit_menu.extend (create {EV_MENU_SEPARATOR})
			from
				n := 1
			until
				n > view.saved_area_capacity
			loop
				add_menu_item (cut_edit_menu, menu_text ("Set Saved Area " + n.out, "Ctrl+" + n.out), agent controller.set_saved_area (n))
				n := n + 1
			end
		end

	build_fix_menu (a_menu: EV_MENU)
			-- Build Fix menu to allow user to fix items in
			-- X and/or Y
		do	
			add_menu_item (a_menu, menu_text (In_x_text, "F X"), agent controller.fix_selected_items_in_x)
			add_menu_item (a_menu, menu_text (In_y_text, "F Y"), agent controller.fix_selected_items_in_y)
			add_menu_item (a_menu, In_x_and_y_text, agent controller.fix_selected_items_in_x_and_y)
			
		end

	build_unfix_menu (a_menu: EV_MENU)
			-- Build Unfix menu to allow user to unfix items in
			-- X and/or Y
		do	
			add_menu_item (a_menu, menu_text (In_x_text, "U X"), agent controller.unfix_selected_items_in_x)
			add_menu_item (a_menu, menu_text (In_y_text, "U Y"), agent controller.unfix_selected_items_in_y)
			add_menu_item (a_menu, In_x_and_y_text, agent controller.unfix_selected_items_in_x_and_y)
			
		end

	build_lock_menu (a_menu: EV_MENU)
			-- Build Lock menu to allow user to lock
			-- various attributes of items
		do	
			build_lock_all_menu
			build_lock_selected_menu
			build_lock_unselected_menu
			a_menu.disable_sensitive
			a_menu.extend (lock_all_menu)
			a_menu.extend (lock_selected_menu)
			a_menu.extend (lock_unselected_menu)
			
		end

	build_lock_all_menu
			-- Build Lock All menu to allow user to lock
			-- all items
		do	
			create lock_all_menu.make_with_text (All_text)
			add_menu_item (lock_all_menu, In_x_text, agent controller.lock_all_items_in_x)
			add_menu_item (lock_all_menu, In_y_text, agent controller.lock_all_items_in_y)
			add_menu_item (lock_all_menu, In_x_and_y_text, agent controller.lock_all_items_in_x_and_y)
			add_menu_item (lock_all_menu, Orientation_text, agent controller.lock_all_items_orientation)
			add_menu_item (lock_all_menu, To_prevent_duplication_text, agent controller.lock_all_items_to_prevent_duplication)
			add_menu_item (lock_all_menu, To_prevent_deletion_text, agent controller.lock_all_items_to_prevent_deletion)
			add_menu_item (lock_all_menu, Almost_completely_text, agent controller.lock_all_items_almost_completely)
			add_menu_item (lock_all_menu, Completely_text, agent controller.lock_all_items_completely)
			
		end

	build_lock_selected_menu
			-- Build Lock Selected menu to allow user to lock
			-- selected selected items
		do	
			create lock_selected_menu.make_with_text (Selected_text)
			add_menu_item (lock_selected_menu, menu_text (In_x_text, "F X"), agent controller.lock_selected_items_in_x)
			add_menu_item (lock_selected_menu, menu_text (In_y_text, "F Y"), agent controller.lock_selected_items_in_y)
			add_menu_item (lock_selected_menu, In_x_and_y_text, agent controller.lock_selected_items_in_x_and_y)
			add_menu_item (lock_selected_menu, Orientation_text, agent controller.lock_selected_items_orientation)
			add_menu_item (lock_selected_menu, To_prevent_duplication_text, agent controller.lock_selected_items_to_prevent_duplication)
			add_menu_item (lock_selected_menu, To_prevent_deletion_text, agent controller.lock_selected_items_to_prevent_deletion)
			add_menu_item (lock_selected_menu, Almost_completely_text, agent controller.lock_selected_items_almost_completely)
			add_menu_item (lock_selected_menu, Completely_text, agent controller.lock_selected_items_completely)
		end

	build_lock_unselected_menu
			-- Build Lock Unselected menu to allow user to lock
			-- unselected
		do	
			create lock_unselected_menu.make_with_text (Unselected_text)
			add_menu_item (lock_unselected_menu, In_x_text, agent controller.lock_unselected_items_in_x)
			add_menu_item (lock_unselected_menu, In_y_text, agent controller.lock_unselected_items_in_y)
			add_menu_item (lock_unselected_menu, In_x_and_y_text, agent controller.lock_unselected_items_in_x_and_y)
			add_menu_item (lock_unselected_menu, Orientation_text, agent controller.lock_unselected_items_orientation)
			add_menu_item (lock_unselected_menu, To_prevent_duplication_text, agent controller.lock_unselected_items_to_prevent_duplication)
			add_menu_item (lock_unselected_menu, To_prevent_deletion_text, agent controller.lock_unselected_items_to_prevent_deletion)
			add_menu_item (lock_unselected_menu, Almost_completely_text, agent controller.lock_unselected_items_almost_completely)
			add_menu_item (lock_unselected_menu, Completely_text, agent controller.lock_unselected_items_completely)
		end

	build_unlock_menu (a_menu: EV_MENU)
			-- Build UnLock menu to allow user to unlock items
		do	
			build_unlock_all_menu
			build_unlock_selected_menu
			build_unlock_unselected_menu
			a_menu.disable_sensitive
			a_menu.extend (unlock_all_menu)
			a_menu.extend (unlock_selected_menu)
			a_menu.extend (unlock_unselected_menu)
			
		end

	build_unlock_all_menu
			-- Build Unlock All menu to allow user to
			-- unlock all items
		do	
			create unlock_all_menu.make_with_text (All_text)
			add_menu_item (unlock_all_menu, In_x_text, agent controller.unlock_all_items_in_x)
			add_menu_item (unlock_all_menu, In_y_text, agent controller.unlock_all_items_in_y)
			add_menu_item (unlock_all_menu, In_x_and_y_text, agent controller.unlock_all_items_in_x_and_y)
			add_menu_item (unlock_all_menu, Orientation_text, agent controller.unlock_all_items_orientation)
			add_menu_item (unlock_all_menu, To_allow_duplication_text, agent controller.unlock_all_items_to_allow_duplication)
			add_menu_item (unlock_all_menu, To_allow_deletion_text, agent controller.unlock_all_items_to_allow_deletion)
			add_menu_item (unlock_all_menu, Almost_completely_text, agent controller.unlock_all_items_almost_completely)
			add_menu_item (unlock_all_menu, Completely_text, agent controller.unlock_all_items_completely)
		end

	build_unlock_selected_menu
			-- Build Unlock Selected menu to allow user to
			-- unlock selected items
		do	
			create unlock_selected_menu.make_with_text (Selected_text)
			add_menu_item (unlock_selected_menu, menu_text (In_x_text, "U X"), agent controller.unlock_selected_items_in_x)
			add_menu_item (unlock_selected_menu, menu_text (In_y_text, "U Y"), agent controller.unlock_selected_items_in_y)
			add_menu_item (unlock_selected_menu, In_x_and_y_text, agent controller.unlock_selected_items_in_x_and_y)
			add_menu_item (unlock_selected_menu, Orientation_text, agent controller.unlock_selected_items_orientation)
			add_menu_item (unlock_selected_menu, To_allow_duplication_text, agent controller.unlock_selected_items_to_allow_duplication)
			add_menu_item (unlock_selected_menu, To_allow_deletion_text, agent controller.unlock_selected_items_to_allow_deletion)
			add_menu_item (unlock_selected_menu, Almost_completely_text, agent controller.unlock_selected_items_almost_completely)
			add_menu_item (unlock_selected_menu, Completely_text, agent controller.unlock_selected_items_completely)
		end

	build_unlock_unselected_menu
			-- Build Unlock Unselected menu to allow user
			-- to unlock position of unselected items in X
			-- and/or Y
		do	
			create unlock_unselected_menu.make_with_text (Unselected_text)
			add_menu_item (unlock_unselected_menu, In_x_text, agent controller.unlock_unselected_items_in_x)
			add_menu_item (unlock_unselected_menu, In_y_text, agent controller.unlock_unselected_items_in_y)
			add_menu_item (unlock_unselected_menu, In_x_and_y_text, agent controller.unlock_unselected_items_in_x_and_y)
			add_menu_item (unlock_unselected_menu, Orientation_text, agent controller.unlock_unselected_items_orientation)
			add_menu_item (unlock_unselected_menu, To_allow_duplication_text, agent controller.unlock_unselected_items_to_allow_duplication)
			add_menu_item (unlock_unselected_menu, To_allow_deletion_text, agent controller.unlock_unselected_items_to_allow_deletion)
			add_menu_item (unlock_unselected_menu, Almost_completely_text, agent controller.unlock_unselected_items_almost_completely)
			add_menu_item (unlock_unselected_menu, Completely_text, agent controller.unlock_unselected_items_completely)
		end

	build_view_menu
			-- Build View menu to allow user to change appearance
			-- of items
		do	
			create view_menu.make_with_text ("View")
			add_menu_item (view_menu, menu_text ("Standard (Design Number)", "Ctrl+V S"), agent standard_view_button.enable_select)
			add_menu_item (view_menu, menu_text ("Design Name", "Ctrl+V N"), agent set_design_name_view)
			add_menu_item (view_menu, menu_text ("Die ID", "Ctrl+V I"), agent set_die_id_view)
			view_menu.extend (create {EV_MENU_SEPARATOR})
			add_menu_item (view_menu, menu_text ("Account Type", "Ctrl+V A"), agent account_type_view_button.enable_select)
			add_menu_item (view_menu, menu_text ("Account Number", "Ctrl+V C"), agent set_account_number_view)
			add_menu_item (view_menu, menu_text ("Organization Short ID", "Ctrl+V O"), agent set_org_short_id_view)
			view_menu.extend (create {EV_MENU_SEPARATOR})
			add_menu_item (view_menu, menu_text ("Maximum Die Size", "Ctrl+V M"), agent max_die_size_view_button.enable_select)
			add_menu_item (view_menu, menu_text ("Lot Split", "Ctrl+V L"), agent lot_split_view_button.enable_select)
			add_menu_item (view_menu, menu_text ("Packaging", "Ctrl+V P"), agent packaging_view_button.enable_select)
			add_menu_item (view_menu, menu_text ("Validity Check", "Ctrl+V V"), agent validity_check_view_button.enable_select)
			add_menu_item (view_menu, menu_text ("Die Labelling Style", "Ctrl+V B"), agent set_overhead_style_view)
			add_menu_item (view_menu, menu_text ("Cut", "Ctrl+V T"), agent cut_view_button.enable_select)
			view_menu.extend (create {EV_MENU_SEPARATOR})
			add_menu_item (view_menu, menu_text ("Search Results", "Ctrl+V F"), agent set_search_results_view)
			add_menu_item (view_menu, menu_text ("Highlighted Selection", "Ctrl+V H"), agent set_highlighted_selection_view)
			view_menu.extend (create {EV_MENU_SEPARATOR})
			add_menu_item (view_menu, menu_text ("Rotation Locked", "Ctrl+V R"), agent set_rotation_locked_view)
			add_menu_item (view_menu, menu_text ("Duplication Locked", "Ctrl+V U"), agent set_duplication_locked_view)
			add_menu_item (view_menu, menu_text ("Deletion Locked", "Ctrl+V D"), agent set_deletion_locked_view)
		end

	build_old_view_menu
			-- Build old View menu to allow user to change
			-- appearance of items (old version of
			-- `build_view_menu' - not currently used)
		do	
			build_named_view_menu
			build_label_menu
			build_color_menu
			build_fill_menu
			build_set_opacity_menu
			create view_menu.make_with_text ("View")
			view_menu.extend (named_view_menu)
			view_menu.extend (label_menu)
			view_menu.extend (color_menu)
			view_menu.extend (fill_menu)
			view_menu.extend (set_opacity_menu)
		end

	build_named_view_menu
			-- Build Named View menu to allow user to choose
			-- a named view, which determines how items are
			-- labeled, colored, filled and whether they are
			-- transparent or opaque
		do	
			create named_view_menu.make_with_text ("Named View")
			add_menu_item (named_view_menu, "Standard (Design Number)", agent standard_view_button.enable_select)
			add_menu_item (named_view_menu, "Account Type", agent account_type_view_button.enable_select)
			add_menu_item (named_view_menu, "Maximum Die Size", agent max_die_size_view_button.enable_select)
			add_menu_item (named_view_menu, "Lot Split", agent lot_split_view_button.enable_select)
			add_menu_item (named_view_menu, "Packaging", agent packaging_view_button.enable_select)
			add_menu_item (named_view_menu, "Validity Check", agent validity_check_view_button.enable_select)
			add_menu_item (named_view_menu, "Cut", agent cut_view_button.enable_select)
			add_menu_item (named_view_menu, "Search Results", agent set_search_results_view)
			add_menu_item (named_view_menu, "Design Name", agent set_design_name_view)
			add_menu_item (named_view_menu, "Account Number", agent set_account_number_view)
			add_menu_item (named_view_menu, "Organization Short ID", agent set_org_short_id_view)
			add_menu_item (named_view_menu, "Rotation Locked", agent set_rotation_locked_view)
			add_menu_item (named_view_menu, "Duplication Locked", agent set_duplication_locked_view)
			add_menu_item (named_view_menu, "Deletion Locked", agent set_deletion_locked_view)
		end

	build_label_menu
			-- Build Label menu to allow user to change labels
			-- for items
		do	
			build_label_all_menu
			build_label_selected_menu
			build_label_unselected_menu
			create label_menu.make_with_text ("Label")
			label_menu.extend (label_all_menu)
			label_menu.extend (label_selected_menu)
			label_menu.extend (label_unselected_menu)
		end

	build_label_all_menu
			-- Build Label All menu to allow user to label
			-- all items
		do	
			create label_all_menu.make_with_text (All_text)
			add_menu_item (label_all_menu, By_design_number_text, agent controller.label_all_items_by_design_number)
			add_menu_item (label_all_menu, By_design_name_text, agent controller.label_all_items_by_design_name)
			add_menu_item (label_all_menu, By_account_type_text, agent controller.label_all_items_by_account_type)
			add_menu_item (label_all_menu, By_account_number_text, agent controller.label_all_items_by_account_number)
			add_menu_item (label_all_menu, By_lot_split_text, agent controller.label_all_items_by_wafer_etest_id)
			add_menu_item (label_all_menu, By_organization_text, agent controller.label_all_items_by_org_short_id)
			
		end

	build_label_selected_menu
			-- Build Label Selected menu to allow user to label
			-- only selected items
		do	
			create label_selected_menu.make_with_text (Selected_text)
			add_menu_item (label_selected_menu, By_design_number_text, agent controller.label_selected_items_by_design_number)
			add_menu_item (label_selected_menu, By_design_name_text, agent controller.label_selected_items_by_design_name)
			add_menu_item (label_selected_menu, By_account_type_text, agent controller.label_selected_items_by_account_type)
			add_menu_item (label_selected_menu, By_account_number_text, agent controller.label_selected_items_by_account_number)
			add_menu_item (label_selected_menu, By_lot_split_text, agent controller.label_selected_items_by_wafer_etest_id)
			add_menu_item (label_selected_menu, By_organization_text, agent controller.label_selected_items_by_org_short_id)
			
		end

	build_label_unselected_menu
			-- Build Label Unselected menu to allow user to label
			-- only unselected items
		do	
			create label_unselected_menu.make_with_text (Unselected_text)
			add_menu_item (label_unselected_menu, By_design_number_text, agent controller.label_unselected_items_by_design_number)
			add_menu_item (label_unselected_menu, By_design_name_text, agent controller.label_unselected_items_by_design_name)
			add_menu_item (label_unselected_menu, By_account_type_text, agent controller.label_unselected_items_by_account_type)
			add_menu_item (label_unselected_menu, By_account_number_text, agent controller.label_unselected_items_by_account_number)
			add_menu_item (label_unselected_menu, By_lot_split_text, agent controller.label_unselected_items_by_wafer_etest_id)
			add_menu_item (label_unselected_menu, By_organization_text, agent controller.label_unselected_items_by_org_short_id)
			
		end

	build_color_menu
			-- Build Color menu to allow user to change color
			-- of items
		do	
			build_color_all_menu
			build_color_selected_menu
			build_color_unselected_menu
			create color_menu.make_with_text ("Color")
			color_menu.extend (color_all_menu)
			color_menu.extend (color_selected_menu)
			color_menu.extend (color_unselected_menu)
		end

	build_color_all_menu
			-- Build Color All menu to allow user to color
			-- all items
		do	
			create color_all_menu.make_with_text (All_text)
			add_menu_item (color_all_menu, By_fixed_status_text, agent controller.color_all_items_by_fixed_status)
			add_menu_item (color_all_menu, By_locked_status_text, agent controller.color_all_items_by_locked_status)
			add_menu_item (color_all_menu, By_account_type_text, agent controller.color_all_items_by_account_type)
		end

	build_color_selected_menu
			-- Build Color Selected menu to allow user to color
			-- selected items
		do	
			create color_selected_menu.make_with_text (Selected_text)
			add_menu_item (color_selected_menu, By_fixed_status_text, agent controller.color_selected_items_by_fixed_status)
			add_menu_item (color_selected_menu, By_locked_status_text, agent controller.color_selected_items_by_locked_status)
			add_menu_item (color_selected_menu, By_account_type_text, agent controller.color_selected_items_by_account_type)
		end

	build_color_unselected_menu
			-- Build Color Unselected menu to allow user to color
			-- unselected items
		do	
			create color_unselected_menu.make_with_text (Unselected_text)
			add_menu_item (color_unselected_menu, By_fixed_status_text, agent controller.color_unselected_items_by_fixed_status)
			add_menu_item (color_unselected_menu, By_locked_status_text, agent controller.color_unselected_items_by_locked_status)
			add_menu_item (color_unselected_menu, By_account_type_text, agent controller.color_unselected_items_by_account_type)
		end

	build_fill_menu
			-- Build Fill menu to allow user to interior filling
			-- for items
		do	
			create fill_menu.make_with_text ("Fill")
			add_menu_item (fill_menu, None_text, agent controller.fill_no_items)
			add_menu_item (fill_menu, Selected_text, agent controller.fill_selected_items)
			add_menu_item (fill_menu, Unselected_text, agent controller.fill_unselected_items)
			add_menu_item (fill_menu, Selected_and_unselected_text, agent controller.fill_selected_and_unselected_items)
			add_menu_item (fill_menu, By_fixed_status_text, agent controller.fill_by_fixed_status)
			add_menu_item (fill_menu, By_locked_status_text, agent controller.fill_by_locked_status)
		end

	build_set_opacity_menu
			-- Build Set Opacity menu to allow user to change
			-- opacity of items
		do	
			build_set_opacity_all_menu
			build_set_opacity_selected_menu
			build_set_opacity_unselected_menu
			create set_opacity_menu.make_with_text ("Set Opacity")
			set_opacity_menu.extend (set_opacity_all_menu)
			set_opacity_menu.extend (set_opacity_selected_menu)
			set_opacity_menu.extend (set_opacity_unselected_menu)
		end

	build_set_opacity_all_menu
			-- Build Set Opacity All menu to allow user to set
			-- opacity of all items
		do	
			create set_opacity_all_menu.make_with_text (All_text)
			add_menu_item (set_opacity_all_menu, Transparent_text, agent controller.set_transparent_all_items)
			add_menu_item (set_opacity_all_menu, Opaque_text, agent controller.set_opaque_all_items)
		end

	build_set_opacity_selected_menu
			-- Build Set Opacity Selected menu to allow user to set
			-- opacity of selected items
		do	
			create set_opacity_selected_menu.make_with_text (Selected_text)
			add_menu_item (set_opacity_selected_menu, Transparent_text, agent controller.set_transparent_selected_items)
			add_menu_item (set_opacity_selected_menu, Opaque_text, agent controller.set_opaque_selected_items)
		end

	build_set_opacity_unselected_menu
			-- Build Set Opacity Unselected menu to allow user to
			-- set opacity of unselected items
		do	
			create set_opacity_unselected_menu.make_with_text (Unselected_text)
			add_menu_item (set_opacity_unselected_menu, Transparent_text, agent controller.set_transparent_unselected_items)
			add_menu_item (set_opacity_unselected_menu, Opaque_text, agent controller.set_opaque_unselected_items)
		end

	build_select_menu
			-- Build Select menu to allow user to change which
			-- items are selected
		do	
			build_set_selection_menu
			build_add_selection_menu
			build_remove_selection_menu
			create select_menu.make_with_text ("Select")
			select_menu.extend (set_selection_menu)
			select_menu.extend (add_selection_menu)
			select_menu.extend (remove_selection_menu)
			select_menu.extend (create {EV_MENU_SEPARATOR})
			add_menu_item (select_menu, menu_text (All_text, "Ctrl+S A"), agent controller.select_all_items)
			add_menu_item (select_menu, menu_text (None_text, "Ctrl+S Z"), agent controller.unselect_all_items)
			add_menu_item (select_menu, menu_text (Invert_text, "Ctrl+S I"), agent controller.invert_selection)
		end

	build_set_selection_menu
			-- Build Set Selection menu to allow user to set
			-- the list of selected items
		do	
			create set_selection_menu.make_with_text ("Set Selection")
			add_menu_item (set_selection_menu, menu_text (Customer_projects_text, "Ctrl+S C"), agent controller.set_selection_to_cust_project_items)
			add_menu_item (set_selection_menu, menu_text (Test_vehicles_text, "Ctrl+S T"), agent controller.set_selection_to_test_vehicle_items)
			add_menu_item (set_selection_menu, menu_text (Cust_and_test_vehicles_text, "Ctrl+S B"), agent controller.set_selection_to_project_items)
			add_menu_item (set_selection_menu, menu_text (Fillers_text, "Ctrl+S L"), agent controller.set_selection_to_filler_items)
			add_menu_item (set_selection_menu, menu_text (Vendor_geometries_text, "Ctrl+S G"), agent controller.set_selection_to_vendor_geometry_items)
			add_menu_item (set_selection_menu, menu_text (Cuts_text, "Ctrl+S U"), agent controller.set_selection_to_cut_items)
			set_selection_menu.extend (create {EV_MENU_SEPARATOR})
			add_menu_item (set_selection_menu, menu_text (All_scribes_text, "Ctrl+S S"), agent controller.set_selection_to_scribe_items)
			add_menu_item (set_selection_menu,  menu_text (Horizontal_scribes_text, "Ctrl+S H"), agent controller.set_selection_to_horizontal_scribe_items)
			add_menu_item (set_selection_menu, menu_text (Vertical_scribes_text, "Ctrl+S V"), agent controller.set_selection_to_vertical_scribe_items)
			add_menu_item (set_selection_menu, menu_text (Non_scribes_text, "Ctrl+S N"), agent controller.set_selection_to_non_scribe_items)
			add_menu_item (set_selection_menu, Default_width_scribes_text, agent controller.set_selection_to_default_width_scribe_items)
			add_menu_item (set_selection_menu, Custom_width_scribes_text, agent controller.set_selection_to_custom_width_scribe_items)
			add_menu_item (set_selection_menu, menu_text (Floating_scribes_text, "Ctrl+S F"), agent controller.set_selection_to_floating_scribe_items)
			add_menu_item (set_selection_menu, Misaligned_scribes_text, agent controller.set_selection_to_misaligned_scribe_items)
			add_menu_item (set_selection_menu, Small_scribes_and_fillers_text, agent controller.set_selection_to_small_scribes_and_fillers)
			set_selection_menu.extend (create {EV_MENU_SEPARATOR})
			add_menu_item (set_selection_menu, menu_text (Commercial_projects_text, "Ctrl+S Shift+C"), agent controller.set_selection_to_commercial_account_items)
			add_menu_item (set_selection_menu, menu_text (Mep_projects_text, "Ctrl+S Shift+M"), agent controller.set_selection_to_mep_account_items)
			add_menu_item (set_selection_menu, menu_text (Other_projects_text, "Ctrl+S Shift+O"), agent controller.set_selection_to_other_account_items)
			set_selection_menu.extend (create {EV_MENU_SEPARATOR})
			add_menu_item (set_selection_menu, menu_text (Partially_selected_items_text, "Ctrl+S P"), agent controller.set_selection_to_partially_selected_items)
			set_selection_menu.extend (create {EV_MENU_SEPARATOR})
			add_menu_item (set_selection_menu, menu_text (Outside_fields_items_text, "Ctrl+S E"), agent controller.set_selection_to_outside_fields_items)
			add_menu_item (set_selection_menu, Below_minimum_size_items_text, agent controller.set_selection_to_too_small_items)
			add_menu_item (set_selection_menu, Overlapping_field_edge_text, agent controller.set_selection_to_overlapping_field_items)
			add_menu_item (set_selection_menu, Overlapping_object_text, agent controller.set_selection_to_overlapping_object_items)
			add_menu_item (set_selection_menu, Too_close_to_field_edge_text, agent controller.set_selection_to_close_edge_items)
			add_menu_item (set_selection_menu, Too_close_to_object_text, agent controller.set_selection_to_close_object_items)
			set_selection_menu.extend (create {EV_MENU_SEPARATOR})
			add_menu_item (set_selection_menu, Cut_die_too_big_text, agent controller.set_selection_to_too_big_items)
			add_menu_item (set_selection_menu, Not_isolated_in_cut_die_text, agent controller.set_selection_to_unisolated_items)
			add_menu_item (set_selection_menu, Not_centered_in_cut_die_text, agent controller.set_selection_to_uncentered_items)
			set_selection_menu.extend (create {EV_MENU_SEPARATOR})
			add_menu_item (set_selection_menu, Lot_not_assigned_text, agent controller.set_selection_to_unassigned_lot_items)
			add_menu_item (set_selection_menu, Unlocked_or_unassigned_text, agent controller.set_selection_to_unlocked_or_unassigned)
		end

	build_add_selection_menu
			-- Build Add to Selection menu to allow user to add
			-- to the list of selected items
		do	
			create add_selection_menu.make_with_text ("Add to Selection")
			add_menu_item (add_selection_menu, menu_text (Customer_projects_text, "Ctrl+A C"), agent controller.add_to_selection_cust_project_items)
			add_menu_item (add_selection_menu, menu_text (Test_vehicles_text, "Ctrl+A T"), agent controller.add_to_selection_test_vehicle_items)
			add_menu_item (add_selection_menu, menu_text (Cust_and_test_vehicles_text, "Ctrl+A B"), agent controller.add_to_selection_project_items)
			add_menu_item (add_selection_menu, menu_text (Fillers_text, "Ctrl+A L"), agent controller.add_to_selection_filler_items)
			add_menu_item (add_selection_menu, menu_text (Vendor_geometries_text, "Ctrl+A G"), agent controller.add_to_selection_vendor_geometry_items)
			add_menu_item (add_selection_menu, menu_text (Cuts_text, "Ctrl+A U"), agent controller.add_to_selection_cut_items)
			add_selection_menu.extend (create {EV_MENU_SEPARATOR})
			add_menu_item (add_selection_menu, menu_text (All_scribes_text, "Ctrl+A S"), agent controller.add_to_selection_scribe_items)
			add_menu_item (add_selection_menu, menu_text (Horizontal_scribes_text, "Ctrl+A H"), agent controller.add_to_selection_horizontal_scribe_items)
			add_menu_item (add_selection_menu, menu_text (Vertical_scribes_text, "Ctrl+A V"), agent controller.add_to_selection_vertical_scribe_items)
			add_menu_item (add_selection_menu, menu_text (Non_scribes_text, "Ctrl+A N"), agent controller.add_to_selection_non_scribe_items)
			add_menu_item (add_selection_menu, Default_width_scribes_text, agent controller.add_to_selection_default_width_scribe_items)
			add_menu_item (add_selection_menu, Custom_width_scribes_text, agent controller.add_to_selection_custom_width_scribe_items)
			add_menu_item (add_selection_menu, menu_text (Floating_scribes_text, "Ctrl+A F"), agent controller.add_to_selection_floating_scribe_items)
			add_menu_item (add_selection_menu, Misaligned_scribes_text, agent controller.add_to_selection_misaligned_scribe_items)
			add_menu_item (add_selection_menu, Small_scribes_and_fillers_text, agent controller.add_to_selection_small_scribes_and_fillers)
			add_selection_menu.extend (create {EV_MENU_SEPARATOR})
			add_menu_item (add_selection_menu, menu_text (Commercial_projects_text, "Ctrl+A Shift+C"), agent controller.add_to_selection_commercial_account_items)
			add_menu_item (add_selection_menu, menu_text (Mep_projects_text, "Ctrl+A Shift+M"), agent controller.add_to_selection_mep_account_items)
			add_menu_item (add_selection_menu, menu_text (Other_projects_text, "Ctrl+A Shift+O"), agent controller.add_to_selection_other_account_items)
			add_selection_menu.extend (create {EV_MENU_SEPARATOR})
			add_menu_item (add_selection_menu, menu_text (Partially_selected_items_text, "Ctrl+A P"), agent controller.add_to_selection_partially_selected_items)
			add_selection_menu.extend (create {EV_MENU_SEPARATOR})
			add_menu_item (add_selection_menu, menu_text (Outside_fields_items_text, "Ctrl+A E"), agent controller.add_to_selection_outside_fields_items)
			add_menu_item (add_selection_menu, Below_minimum_size_items_text, agent controller.add_to_selection_too_small_items)
			add_menu_item (add_selection_menu, Overlapping_field_edge_text, agent controller.add_to_selection_overlapping_field_items)
			add_menu_item (add_selection_menu, Overlapping_object_text, agent controller.add_to_selection_overlapping_object_items)
			add_menu_item (add_selection_menu, Too_close_to_field_edge_text, agent controller.add_to_selection_close_edge_items)
			add_menu_item (add_selection_menu, Too_close_to_object_text, agent controller.add_to_selection_close_object_items)
			add_selection_menu.extend (create {EV_MENU_SEPARATOR})
			add_menu_item (add_selection_menu, Cut_die_too_big_text, agent controller.add_to_selection_too_big_items)
			add_menu_item (add_selection_menu, Not_isolated_in_cut_die_text, agent controller.add_to_selection_unisolated_items)
			add_menu_item (add_selection_menu, Not_centered_in_cut_die_text, agent controller.add_to_selection_uncentered_items)
			add_selection_menu.extend (create {EV_MENU_SEPARATOR})
			add_menu_item (add_selection_menu, Lot_not_assigned_text, agent controller.add_to_selection_unassigned_lot_items)
			add_menu_item (add_selection_menu, Unlocked_or_unassigned_text, agent controller.add_to_selection_unlocked_or_unassigned)
		end

	build_remove_selection_menu
			-- Build Remove from Selection menu to allow
			-- user to remove items from the list of
			-- selected items
		do	
			create remove_selection_menu.make_with_text ("Remove from Selection")
			add_menu_item (remove_selection_menu, menu_text (Customer_projects_text, "Ctrl+R C"), agent controller.remove_from_selection_cust_project_items)
			add_menu_item (remove_selection_menu, menu_text (Test_vehicles_text, "Ctrl+R T"), agent controller.remove_from_selection_test_vehicle_items)
			add_menu_item (remove_selection_menu, menu_text (Cust_and_test_vehicles_text, "Ctrl+R B"), agent controller.remove_from_selection_project_items)
			add_menu_item (remove_selection_menu, menu_text (Fillers_text, "Ctrl+R L"), agent controller.remove_from_selection_filler_items)
			add_menu_item (remove_selection_menu, menu_text (Vendor_geometries_text, "Ctrl+R G"), agent controller.remove_from_selection_vendor_geometry_items)
			add_menu_item (remove_selection_menu, menu_text (Cuts_text, "Ctrl+R U"), agent controller.remove_from_selection_cut_items)
			remove_selection_menu.extend (create {EV_MENU_SEPARATOR})
			add_menu_item (remove_selection_menu, menu_text (All_scribes_text, "Ctrl+R S"), agent controller.remove_from_selection_scribe_items)
			add_menu_item (remove_selection_menu, menu_text (Horizontal_scribes_text, "Ctrl+R H"), agent controller.remove_from_selection_horizontal_scribe_items)
			add_menu_item (remove_selection_menu, menu_text (Vertical_scribes_text, "Ctrl+R V"), agent controller.remove_from_selection_vertical_scribe_items)
			add_menu_item (remove_selection_menu, menu_text (Non_scribes_text, "Ctrl+R N"), agent controller.remove_from_selection_non_scribe_items)
			add_menu_item (remove_selection_menu, Default_width_scribes_text, agent controller.remove_from_selection_default_width_scribe_items)
			add_menu_item (remove_selection_menu, Custom_width_scribes_text, agent controller.remove_from_selection_custom_width_scribe_items)
			add_menu_item (remove_selection_menu, menu_text (Floating_scribes_text, "Ctrl+R F"), agent controller.remove_from_selection_floating_scribe_items)
			add_menu_item (remove_selection_menu, Misaligned_scribes_text, agent controller.remove_from_selection_misaligned_scribe_items)
			add_menu_item (remove_selection_menu, Small_scribes_and_fillers_text, agent controller.remove_from_selection_small_scribes_and_fillers)
			remove_selection_menu.extend (create {EV_MENU_SEPARATOR})
			add_menu_item (remove_selection_menu, menu_text (Commercial_projects_text, "Ctrl+R Shift+C"), agent controller.remove_from_selection_commercial_account_items)
			add_menu_item (remove_selection_menu, menu_text (Mep_projects_text, "Ctrl+R Shift+M"), agent controller.remove_from_selection_mep_account_items)
			add_menu_item (remove_selection_menu, menu_text (Other_projects_text, "Ctrl+R Shift+O"), agent controller.remove_from_selection_other_account_items)
			remove_selection_menu.extend (create {EV_MENU_SEPARATOR})
			add_menu_item (remove_selection_menu, menu_text (Partially_selected_items_text, "Ctrl+R P"), agent controller.remove_from_selection_partially_selected_items)
			remove_selection_menu.extend (create {EV_MENU_SEPARATOR})
			add_menu_item (remove_selection_menu, menu_text (Outside_fields_items_text, "Ctrl+R E"), agent controller.remove_from_selection_outside_fields_items)
			add_menu_item (remove_selection_menu, Below_minimum_size_items_text, agent controller.remove_from_selection_too_small_items)
			add_menu_item (remove_selection_menu, Overlapping_field_edge_text, agent controller.remove_from_selection_overlapping_field_items)
			add_menu_item (remove_selection_menu, Overlapping_object_text, agent controller.remove_from_selection_overlapping_object_items)
			add_menu_item (remove_selection_menu, Too_close_to_field_edge_text, agent controller.remove_from_selection_close_edge_items)
			add_menu_item (remove_selection_menu, Too_close_to_object_text, agent controller.remove_from_selection_close_object_items)
			remove_selection_menu.extend (create {EV_MENU_SEPARATOR})
			add_menu_item (remove_selection_menu, Cut_die_too_big_text, agent controller.remove_from_selection_too_big_items)
			add_menu_item (remove_selection_menu, Not_isolated_in_cut_die_text, agent controller.remove_from_selection_unisolated_items)
			add_menu_item (remove_selection_menu, Not_centered_in_cut_die_text, agent controller.remove_from_selection_uncentered_items)
			remove_selection_menu.extend (create {EV_MENU_SEPARATOR})
			add_menu_item (remove_selection_menu, Lot_not_assigned_text, agent controller.remove_from_selection_unassigned_lot_items)
			add_menu_item (remove_selection_menu, Unlocked_or_unassigned_text, agent controller.remove_from_selection_unlocked_or_unassigned)
		end

	build_go_to_menu
			-- Build Go To menu to allow user to display a
			-- specified part of the model
		local
			fnames: LIST [STRING]
			n: INTEGER
		do	
			create go_to_menu.make_with_text ("Go To")
			add_menu_item (go_to_menu, menu_text ("Entire Packing", "="), agent controller.show_entire_model_in_view)
			fnames := packing.field_set.field_names
			create {ARRAYED_LIST [EV_MENU_ITEM]} go_to_field_menu_items.make (fnames.count)
			from
				n := 1
				fnames.start
			until
				fnames.after
			loop
				add_menu_item (go_to_menu, menu_text ("Field " + fnames.item, "Shift+" + n.out), agent controller.show_field_by_name (fnames.item))
				go_to_field_menu_items.extend (last_menu_item)
				n := n + 1
				fnames.forth
			end
			go_to_menu.extend (create {EV_MENU_SEPARATOR})
			add_menu_item (go_to_menu, menu_text ("Back", "Ctrl+P"), agent controller.show_previous_area)
			back_menu_item := last_menu_item
			add_menu_item (go_to_menu, menu_text ("Forward", "Ctrl+N"), agent controller.show_next_area)
			forward_menu_item := last_menu_item
			go_to_menu.extend (create {EV_MENU_SEPARATOR})
			add_menu_item (go_to_menu, menu_text ("Next Cut Pattern", "Right"), agent controller.show_next_cut_pattern)
			next_pattern_menu_item := last_menu_item
			next_pattern_menu_item.disable_sensitive
			add_menu_item (go_to_menu, menu_text ("Previous Cut Pattern", "Left"), agent controller.show_previous_cut_pattern)
			previous_pattern_menu_item := last_menu_item
			previous_pattern_menu_item.disable_sensitive
			go_to_menu.extend (create {EV_MENU_SEPARATOR})
			from
				n := 1
			until
				n > view.saved_area_capacity
			loop
				add_menu_item (go_to_menu, menu_text ("Saved Area " + n.out, n.out), agent controller.show_saved_area (n))
				n := n + 1
			end
		end

	build_pack_check_menu
			-- Build Check menu to allow user to do various
			-- kinds of validity checks in Pack mode
		do	
			create pack_check_menu.make_with_text ("Check")
			add_menu_item (pack_check_menu, menu_text ("Everything", "L or Ctrl+C A"), agent controller.check_all_pack_validity)
			add_menu_item (pack_check_menu,  menu_text ("Almost Everything (Skips Objects Outside Fields)", "Ctrl+C Shift+A"), agent controller.check_almost_all_validity)
			add_menu_item (pack_check_menu, menu_text ("Before Join Scribes", "Ctrl+C B"), agent controller.check_before_join_scribes_validity)
			pack_check_menu.extend (create {EV_MENU_SEPARATOR})
			add_menu_item (pack_check_menu, menu_text (Outside_fields_items_text, "Ctrl+C E"), agent controller.check_outside_fields_items)
			add_menu_item (pack_check_menu, menu_text (Overlapping_field_edge_text, "Ctrl+C Shift+O"), agent controller.check_overlapping_field_edge_items)
			add_menu_item (pack_check_menu, menu_text (Overlapping_object_text, "Ctrl+C O"), agent controller.check_overlapping_object_items)
			add_menu_item (pack_check_menu, menu_text (Below_minimum_size_items_text, "Ctrl+C T"), agent controller.check_below_minimum_size_items)
			add_menu_item (pack_check_menu, menu_text (Too_close_to_field_edge_text, "Ctrl+C Shift+G"), agent controller.check_close_to_field_edge_items)
			add_menu_item (pack_check_menu, menu_text (Too_close_to_object_text, "Ctrl+C G"), agent controller.check_close_to_object_items)
			pack_check_menu.extend (create {EV_MENU_SEPARATOR})
			add_menu_item (pack_check_menu, menu_text (Custom_width_scribes_text, "Ctrl+C W"), agent controller.check_custom_width_scribes)
			add_menu_item (pack_check_menu, menu_text (Floating_scribes_text, "Ctrl+C F"), agent controller.check_floating_scribes)
			add_menu_item (pack_check_menu, menu_text (Misaligned_scribes_text, "Ctrl+C M"), agent controller.check_misaligned_scribes)
			add_menu_item (pack_check_menu, menu_text (Small_scribes_and_fillers_text, "Ctrl+C Shift+T"), agent controller.check_small_scribes_and_fillers)
			add_menu_item (pack_check_menu, menu_text (Test_vehicles_text, "Ctrl+C V"), agent controller.check_test_vehicles)
			pack_check_menu.extend (create {EV_MENU_SEPARATOR})
			add_menu_item (pack_check_menu, menu_text (Cut_die_too_big_text, "Ctrl+C S"), agent controller.check_scribe_cut_die_too_big_items)
			add_menu_item (pack_check_menu, menu_text (Not_isolated_in_cut_die_text, "Ctrl+C I"), agent controller.check_unisolated_in_cut_die_items)
			add_menu_item (pack_check_menu, menu_text (Not_centered_in_cut_die_text, "Ctrl+C C"), agent controller.check_uncentered_in_cut_die_items)
			pack_check_menu.extend (create {EV_MENU_SEPARATOR})
			add_menu_item (pack_check_menu, menu_text (Lot_not_assigned_text, "Ctrl+C L"), agent controller.check_lot_not_assigned_items)
			add_menu_item (pack_check_menu, menu_text (Unlocked_or_unassigned_text, "Ctrl+C U"), agent controller.check_unlocked_or_unassigned_projects)
		end

	build_cut_check_menu
			-- Build Check menu to allow user to do various
			-- kinds of validity checks in cut mode
		do	
			create cut_check_menu.make_with_text ("Check")
			add_menu_item (cut_check_menu, menu_text ("Everything", "L or Ctrl+C A"), agent controller.check_all_cut_validity)
			cut_check_menu.extend (create {EV_MENU_SEPARATOR})
			add_menu_item (cut_check_menu, menu_text (Invalid_isolated_cut_die_text, "Ctrl+C C"), agent controller.check_invalid_cut_die_items)
			cut_check_menu.extend (create {EV_MENU_SEPARATOR})
			add_menu_item (cut_check_menu, menu_text (Wrong_lot_text, "Ctrl+C L"), agent controller.check_wrong_lot_items)
			add_menu_item (cut_check_menu, menu_text (Isolated_cut_die_too_big_text, "Ctrl+C S"), agent controller.check_cut_cut_die_too_big_items)
			add_menu_item (cut_check_menu, menu_text (Non_default_cut_die_text, "Ctrl+C N"), agent controller.check_non_default_cut_die_size_items)
			add_menu_item (cut_check_menu, menu_text (Shared_cut_die_text, "Ctrl+C O"), agent controller.check_shared_cut_die_items)
			add_menu_item (cut_check_menu, menu_text (Extra_cut_text, "Ctrl+C E"), agent controller.check_extra_cut_items)
		end

	build_help_menu
			-- Build Help menu
		do	
			create help_menu.make_with_text ("Help")
			add_menu_item (help_menu, menu_text ("Show Key Bindings", "Ctrl+H"), agent controller.display_help)
		end

	set_fixed_mode
			-- Set Fixed mode, where Fix in X/Y commands
			-- are interpreted as fixing position of item
			-- only for horizontal/vertical scanning
			-- purposes.  Convert all locked items to fixed items.
		do
			if not in_fixed_or_locked_action then
				in_fixed_or_locked_action := True
				if fixed_button.is_selected then
					controller.convert_locked_items_to_fixed
					enable_fixed_functions
					locked_button.disable_select
				else
					fixed_button.enable_select
				end
				in_fixed_or_locked_action := False
			end
		end

	set_locked_mode
			-- Set Locked mode, where Fix in X/Y commands
			-- are interpreted as fixing position of items for all
			-- purposes (i.e., locking their position).
			-- Convert all fixed items to locked items
		do
			if not in_fixed_or_locked_action then
				in_fixed_or_locked_action := True
				if locked_button.is_selected then
					controller.convert_fixed_items_to_locked
					enable_locked_functions
					fixed_button.disable_select
				else
					locked_button.enable_select
				end
				in_fixed_or_locked_action := False
			end
		end

	enable_locked_functions
			-- Enable "lock" commands and treat
			-- Fix/Unfix key sequences as Lock/Unlock.
			-- Convert all currently fixed items in X or Y
			-- to locked in that dimension
		do
			drawing_area_event_handler.set_is_fix_mapped_to_lock (True)
			enable_locked_menus
		end

	enable_locked_menus
			-- Enable the Lock/Unlock menus and disable the
			-- Fix/Unfix menus
		do
			fix_menu.disable_sensitive
			unfix_menu.disable_sensitive
			lock_menu.enable_sensitive
			unlock_menu.enable_sensitive
		end

	enable_fixed_functions
			-- Disable "lock" commands and treat Fix/Unfix
			-- key sequences as Fix/Unfix.  Convert all
			-- currently non-stationary items locked in X
			-- or Y as fixed in that dimension
		do
			drawing_area_event_handler.set_is_fix_mapped_to_lock (False)
			enable_fixed_menus
		end

	enable_fixed_menus
			-- Enable the Fix/Unfix menus and disable the
			-- Lock/Unlock menus
		do
			lock_menu.disable_sensitive
			unlock_menu.disable_sensitive
			fix_menu.enable_sensitive
			unfix_menu.enable_sensitive
		end

	set_pack_mode
			-- Do actions needed when packing version just
			-- read in should be in Pack mode (version does
			-- not have any cut patterns)
		do
			set_interface_for_pack_mode
			view.set_pack_mode
		end

	set_interface_for_pack_mode
			-- Set interface to the one for Pack mode
		do
			menu_bar.go_i_th (2)
			if menu_bar.item /= pack_edit_menu then
				menu_bar.replace (pack_edit_menu)
			end
			menu_bar.go_i_th (6)
			if menu_bar.item /= pack_check_menu then
				menu_bar.replace (pack_check_menu)
			end
			if is_read_only then
				save_button.disable_sensitive
			end
			if has_run_context then
				finish_button.disable_sensitive
			end
			fix_menu := pack_fix_menu
			unfix_menu := pack_unfix_menu
			lock_menu := pack_lock_menu
			unlock_menu := pack_unlock_menu
			undo_menu_item := pack_undo_menu_item
			redo_menu_item := pack_redo_menu_item
			go_to_field_menu_items.do_all (agent {EV_MENU_ITEM}.enable_sensitive)
			fixed_button.enable_sensitive
			locked_button.enable_sensitive
			build_project_management_window
			build_version_selection_window
			if is_read_only then
				project_management_interface.enable_read_only
			else
				project_management_interface.disable_read_only
			end
			drawing_area_event_handler.set_is_cut_mode (False)
			if payload_sizes_built then
				payload_sizes_interface.set_is_pack_mode (True)
			end
		end

	set_cut_mode
			-- Do actions needed when switch to Cut mode
		do
			set_interface_for_cut_mode
			view.set_cut_mode
		end

	set_interface_for_cut_mode
			-- Set interface to the one for Cut mode
		do
			menu_bar.go_i_th (2)
			if menu_bar.item /= cut_edit_menu then
				menu_bar.replace (cut_edit_menu)
			end
			menu_bar.go_i_th (6)
			if menu_bar.item /= cut_check_menu then
				menu_bar.replace (cut_check_menu)
			end
			if is_read_only then
				save_button.disable_sensitive
				if has_run_context then
					finish_button.disable_sensitive
				end
			elseif is_cut_editor_only then
				save_button.disable_sensitive
				if has_run_context then
					finish_button.enable_sensitive
				end
			else
				save_button.enable_sensitive
				if has_run_context then
					finish_button.enable_sensitive
				end
			end
			fix_menu := cut_fix_menu
			unfix_menu := cut_unfix_menu
			lock_menu := cut_lock_menu
			unlock_menu := cut_unlock_menu
			undo_menu_item := cut_undo_menu_item
			redo_menu_item := cut_redo_menu_item
			fixed_button.disable_sensitive
			locked_button.disable_sensitive
			go_to_field_menu_items.do_all (agent {EV_MENU_ITEM}.disable_sensitive)
			build_project_management_window
			build_version_selection_window
			project_management_interface.enable_read_only
			drawing_area_event_handler.set_is_cut_mode (True)
			if payload_sizes_built then
				payload_sizes_interface.set_is_pack_mode (False)
			end
		end

	set_standard_view
			-- Set Standard view, where items are labeled by
			-- design number
		do
			if not in_set_view_action then
				in_set_view_action := True
				if standard_view_button.is_selected then
					controller.set_standard_view
					unselect_all_view_buttons
				end
				standard_view_button.enable_select
				in_set_view_action := False
			end
		end

	set_account_type_view
			-- Set Account Type view, where items are labeled by
			-- account type
		do
			if not in_set_view_action then
				in_set_view_action := True
				if account_type_view_button.is_selected then
					controller.set_account_type_view
					unselect_all_view_buttons
				end
				account_type_view_button.enable_select
				in_set_view_action := False
			end
		end

	set_max_die_size_view
			-- Set Max Die Size view, where items are labeled by
			-- maximum die size
		do
			if not in_set_view_action then
				in_set_view_action := True
				if max_die_size_view_button.is_selected then
					controller.set_max_die_size_view
					unselect_all_view_buttons
				end
				max_die_size_view_button.enable_select
				in_set_view_action := False
			end
		end

	set_lot_split_view
			-- Set Lot Split view, where items are labeled by
			-- wafer etest ID(s)
		do
			if not in_set_view_action then
				in_set_view_action := True
				if lot_split_view_button.is_selected then
					controller.set_lot_split_view
					unselect_all_view_buttons
				end
				lot_split_view_button.enable_select
				in_set_view_action := False
			end
		end

	set_packaging_view
			-- Set Packaging view, where items are labeled by
			-- package name and packaging vendor type
		do
			if not in_set_view_action then
				in_set_view_action := True
				if packaging_view_button.is_selected then
					controller.set_packaging_view
					unselect_all_view_buttons
				end
				packaging_view_button.enable_select
				in_set_view_action := False
			end
		end

	set_validity_check_view
			-- Set Validity Check view, where items are colored
			-- by which validity errors/warnings they have
		do
			if not in_set_view_action then
				in_set_view_action := True
				if validity_check_view_button.is_selected then
					controller.set_validity_check_view
					unselect_all_view_buttons
				end
				validity_check_view_button.enable_select
				in_set_view_action := False
			end
		end

	set_cut_view
			-- Set Cut view, where items are colored
			-- by lot split and labeled by design number
		do
			if not in_set_view_action then
				in_set_view_action := True
				if cut_view_button.is_selected then
					controller.set_cut_view
					unselect_all_view_buttons
				end
				cut_view_button.enable_select
				in_set_view_action := False
			end
		end

	set_design_name_view
			-- Set Design Name view, where items are labeled by
			-- design name
		do
			in_set_view_action := True
			controller.set_design_name_view
			unselect_all_view_buttons
			in_set_view_action := False
		end

	set_die_id_view
			-- Set Die ID view, where items are labeled by
			-- die ID
		do
			in_set_view_action := True
			controller.set_die_id_view
			unselect_all_view_buttons
			in_set_view_action := False
		end

	set_account_number_view
			-- Set Design Number view, where items are labeled by
			-- account number
		do
			in_set_view_action := True
			controller.set_account_number_view
			unselect_all_view_buttons
			in_set_view_action := False
		end

	set_overhead_style_view
			-- Set Overhead Style view, where items are labeled by
			-- overhead style
		do
			in_set_view_action := True
			controller.set_overhead_style_view
			unselect_all_view_buttons
			in_set_view_action := False
		end

	set_search_results_view
			-- Set Search Results view, where items that matched
			-- last search criteria are displayed differently
		do
			in_set_view_action := True
			controller.set_search_results_view
			unselect_all_view_buttons
			in_set_view_action := False
		end

	set_highlighted_selection_view
			-- Set Highlighted SElection view, where items that
			-- are selected are displayed differently
		do
			in_set_view_action := True
			controller.set_highlighted_selection_view
			unselect_all_view_buttons
			in_set_view_action := False
		end

	set_org_short_id_view
			-- Set Organization Short ID view, where items are
			-- labeled by organization short ID
		do
			in_set_view_action := True
			controller.set_org_short_id_view
			unselect_all_view_buttons
			in_set_view_action := False
		end

	set_rotation_locked_view
			-- Set Rotation Locked view, where
			-- rotation-locked items are displayed
			-- differently
		do
			in_set_view_action := True
			controller.set_rotation_locked_view
			unselect_all_view_buttons
			in_set_view_action := False
		end

	set_duplication_locked_view
			-- Set Duplication Locked view, where
			-- duplication-locked items are displayed
			-- differently
		do
			in_set_view_action := True
			controller.set_duplication_locked_view
			unselect_all_view_buttons
			in_set_view_action := False
		end

	set_deletion_locked_view
			-- Set Deletion Locked view, where
			-- deletion-locked items are displayed
			-- differently
		do
			in_set_view_action := True
			controller.set_deletion_locked_view
			unselect_all_view_buttons
			in_set_view_action := False
		end

	unselect_all_view_buttons
			-- Set all buttons that set a named view to be
			-- unselected
		do
			standard_view_button.disable_select
			account_type_view_button.disable_select
			max_die_size_view_button.disable_select
			lot_split_view_button.disable_select
			packaging_view_button.disable_select
			validity_check_view_button.disable_select
			cut_view_button.disable_select
		end

	unselect_all_mode_buttons
			-- Set all buttons that select a mode (Fixed or
			-- Locked) to be unselected
		do
			fixed_button.disable_select
			locked_button.disable_select
		end

	update_selected_count (a_new_count: INTEGER)
			-- Update displayed count of selected items to
			-- `a_new_count'
		do
			selected_count_label.set_text (a_new_count.out)
		end

	update_dimensions (a_width, a_height: DOUBLE)
			-- Update width and height (in model units)
			-- of currently displayed area
		do
			area_label.set_text (formatted_dimension (a_width) + " x " + formatted_dimension (a_height))
		end

	update_locked_mode (a_locked_mode: BOOLEAN)
			-- Set Locked mode `a_locked_mode', Fixed mode
			-- otherwise.  Do not convert any objects to
			-- the new mode - just update the interface
		do
			if not in_fixed_or_locked_action then
				in_fixed_or_locked_action := True
				unselect_all_mode_buttons
				if a_locked_mode then
					locked_button.enable_select
					enable_locked_functions
				else
					fixed_button.enable_select
					enable_fixed_functions
				end
				in_fixed_or_locked_action := False
			end
		end

	update_view (a_name: STRING)
			-- Update name of currently displayed Named View
		require
			a_name_not_void: a_name /= Void
		do
			named_view_label.set_text (a_name)
			if not in_set_view_action and all_view_buttons /= Void then
				-- Only do if interface has been built
				in_set_view_action := True
				unselect_all_view_buttons
				all_view_buttons.search (a_name)
				if all_view_buttons.found then
					all_view_buttons.found_item.enable_select
				end
				in_set_view_action := False
			end
		end

	update_area_history_positions (at_start, at_end: BOOLEAN)
			-- Update interface items to take account of
			-- whether area history list is now `at_start' or
			-- `at_end'
		do
			if at_start then
				back_menu_item.disable_sensitive
				back_button.disable_sensitive
			else
				back_menu_item.enable_sensitive
				back_button.enable_sensitive
			end
			if at_end then
				forward_menu_item.disable_sensitive
				forward_button.disable_sensitive
			else
				forward_menu_item.enable_sensitive
				forward_button.enable_sensitive
			end
		end

	update_change_history_positions (at_start, at_end: BOOLEAN)
			-- Update interface items to take account of
			-- whether change history list is now `at_start' or
			-- `at_end'
		do
			if not is_read_only then
				if at_start then
					undo_menu_item.disable_sensitive
					undo_button.disable_sensitive
				else
					undo_menu_item.enable_sensitive
					undo_button.enable_sensitive
				end
				if at_end then
					redo_menu_item.disable_sensitive
					redo_button.disable_sensitive
				else
					redo_menu_item.enable_sensitive
					redo_button.enable_sensitive
				end
			end
		end

	formatted_dimension (a_value: DOUBLE): STRING
			-- Dimension `a_value' formatted in a useful way
		local
			e: DOUBLE
			n: INTEGER
		do
			e := log10 (a_value.abs)
			if a_value = 0.0 then
				Result := a_value.out
			elseif e >= 1.0 then
				Result := nu.rounded_to_integer_64 (a_value).out
			elseif e >= 0.0 then
				Result := nu.formatted_number (nu.rounded_to_scale (a_value, 1), 1)
			else
				n := e.abs.ceiling
				Result := nu.formatted_number (nu.rounded_to_scale (a_value, n + 1), n + 1)
			end
		end

	handle_window_move (a_x, a_y, a_width, a_height: INTEGER)
		do
			if view /= Void then
				view.set_current_window_position (a_x, a_y)
			end
		end

	handle_window_resize (a_x, a_y, a_width, a_height: INTEGER)
		do
			if view /= Void then
				view.save_current_window_position
				view.set_current_window_position (a_x, a_y)
			end
		end

	post_display_actions
		do
			gui.do_once_on_idle (agent resize_window)
			gui.do_once_on_idle (agent post_post_display_actions)
		end

	resize_window
			-- Resize window to initial size
		do
			gui.center_window_with_margin (window, 100, 50)
		end

	post_post_display_actions
		do
			gui.do_once_on_idle (agent select_starting_version_protected)
			show_packer_output_window
			drawing_area_event_handler.key_press_translated_actions.extend (agent packer_output_interface.process_key_press)
			reset
		end

	fetch_current_packing_row
			-- Fetch packing row for packing currently selected for
			-- this run
		do
			fetch_runs_row_by_id
			if not runs_mrow.is_null_packing_id then
				packing_id := runs_mrow.packing_id
				fetch_packing_row_by_id
			else
				packing_id := Void
				packings_row := Void
			end
		end

	fetch_packing_row_by_id
			-- Fetch packing row for `packing_id'
		do
			create packings_row.make_by_key (connection, option.Old_row, packing_id)
		end

	fetch_runs_row_by_id
			-- Fetch runs row for `run_id'
		do
			create runs_mrow.make_by_key (connection, option.Old_row, run_id)
		end

	select_starting_version_protected
			-- Select the version of the packing to edit or view
			-- and read it in.  Close window if aborted
		do
			create operation.make (agent select_starting_version)
			operation.enable_abort_caught
			operation.execute
			if operation.aborted then
				close
			end
		end

	select_starting_version
			-- Select the version of the packing to edit or view
			-- and read it in
		local
			n: INTEGER
			wt_row: WAFER_TECHNS_ROW
			res, def_width, min_sep, dummy: DOUBLE
		do
			wt_row := wafer_techns_row
			res := grid_resolution
			min_sep := minimum_die_separation
			create grid
			grid.set_size_resolution (res)
			grid.set_edge_resolution (res)
			grid.set_center_resolution (res / 2.0)
			grid.set_scale ({PACKING_DIE_LOCATIONS_ROW}.scale_location_x)
			def_width := default_scribe_width
			model.set_run_id (run_id)
			model.set_global_overhead_style (packings_row.overhead_style)
			model.set_grid (grid)
			if def_width <= 0.0 then
				handle_hard_error ("default scribe width " + grid.formatted_number (def_width) + " for wafer techn " + wt_row.wafer_techn + " is not positive")
			elseif not grid.is_size_on_grid (def_width) then
				handle_hard_error ("default scribe width " + grid.formatted_number (def_width) + " for wafer techn " + wt_row.wafer_techn + " is not a multiple of size resolution " + grid.size_resolution_text)
			end
			model.set_default_scribe_width (grid.rounded_size (def_width))
			if not grid.is_size_on_grid (min_sep) then
				handle_hard_error ("minimum object separation " + grid.formatted_number (min_sep) + " for wafer techn " + wt_row.wafer_techn + " is not a multiple of size resolution " + grid.size_resolution_text)
			end
			model.set_minimum_separation (min_sep)
			if not grid.is_size_on_grid (horizontal_external_scribe_width) then
				inform ("Horizontal external scribe width " + grid.formatted_number (horizontal_external_scribe_width) + " is not a multiple of size resolution " + grid.size_resolution_text + "%NUsing " + grid.formatted_number (rounded_horizontal_external_scribe_width) + " instead.")
			end
			if not grid.is_size_on_grid (vertical_external_scribe_width) then
				inform ("vertical external scribe width " + grid.formatted_number (vertical_external_scribe_width) + " is not a multiple of size resolution " + grid.size_resolution_text + "%NUsing " + grid.formatted_number (rounded_vertical_external_scribe_width) + " instead.")
			end
			model.set_created_small_object_threshold (policies.created_small_object_threshold)
			model.set_cut_handle_width_fraction (policies.cut_handle_width_fraction)
			model.set_cut_handle_height_fraction (policies.cut_handle_height_fraction)
			model.set_cut_extension_fraction (policies.cut_extension_fraction)
			model.set_cut_separation_fraction (policies.cut_separation_fraction)
			
			-- Set initial horizontal and vertical scribe
			-- widths
			set_initial_scribe_widths
			
			if has_run_context then
				fetch_runs_row_by_id
			end
			dummy := minimum_scribe_width
				-- Causes early reporting of any error
			set_model_fields
			model.set_is_setup_completed (True)

			packing.refresh_versions
			n := packing.versions.count
			if has_run_context then
				fetch_runs_row_by_id
				if is_cut_editor_only then
					select_version (official_packing_version)
				elseif runs_mrow.is_null_packing_version_no then
					packing.create_initial_version
					select_version (packing.current_version)
				else
					-- Pick version indicated by RUNS
					select_version (packing.version_with_number (runs_mrow.packing_version_no))
				end
			else
				if n = 0 then
					packing.create_initial_version
					select_version (packing.current_version)
				elseif n = 1 then
					-- Only one version, so just select it
					select_version (packing.versions.first)
				else
					-- Select newest version
					select_version (packing.versions.first)
				end
			end
		end

	save_new_version_with_wait_cursor (a_version_name: STRING)
			-- Save a new version of packing with
			-- `a_version_name' with
			-- cursor changed to the wait cursor to avoid
			-- user confusion.  `runs_mrow' must have previously
			-- been set appropriately
		local
			old_style: EV_POINTER_STYLE
		do
			old_style := pointer_style
			set_pointer_style (gui.Wait_cursor)
			gui.process_events
			if view.is_cut_mode then
				-- Make sure cut die site sizes and centers
				-- are accurate, to avoid confusion
				view.recompute_isolated_site_cut_dies
			end
			packing.save_new_version (a_version_name, runs_mrow)
			set_pointer_style (old_style)
		rescue
			if old_style /= Void then
				set_pointer_style (old_style)
			end
		end

	show_pointer_coordinates
			-- Show line that displays field-relative coordinates
			-- of mouse pointer position
		do
			coordinates_box.extend (coordinates_separator)
			coordinates_box.extend (coordinates_labels_box)
			pointer_coords_menu_item.set_text ("Hide Pointer Coordinates")
		end

	hide_pointer_coordinates
			-- Hide line that displays field-relative coordinates
			-- of mouse pointer position
		do
			coordinates_box.wipe_out
			pointer_coords_menu_item.set_text ("Show Pointer Coordinates")
		end

	display_in_status_line (s: STRING)
			-- Display `s' in status line
		do
			status_label.set_text (s)
		end

	display_in_status_window (s: STRING)
			-- Display `s' in status window
		do
			if not packer_output_built then
				build_packer_output_window
			end
			packer_output_interface.process_info_added (s)
		end

	wafer_techns_row: WAFER_TECHNS_ROW
		do
			Result := packing.wafer_techns_row
		end

	grid_resolution: DOUBLE
		do
			Result := wafer_techns_row.packing_grid_resolution
		end

	grid_scale: INTEGER
		do
			Result := {PACKING_DIE_LOCATIONS_ROW}.scale_location_x
		end

	minimum_die_separation: DOUBLE
			-- Minimum separation between dies
		do
			Result := wafer_techns_row.packing_minimum_separation
		end

	current_payload_size: CONST_POINT
			-- Current size of payload
		do
			create Result.make_xy (packing.litho_sizes.payload_size_x, packing.litho_sizes.payload_size_y)
		end

	current_field_size: CONST_POINT
			-- Current size of field
		do
			create Result.make_xy (packing.litho_sizes.field_size_x, packing.litho_sizes.field_size_y)
		end

	current_horizontal_scribe_width: DOUBLE
			-- Current width for newly created horizontal scribes
		do
			Result := view.horizontal_scribe_width
		end

	current_vertical_scribe_width: DOUBLE
			-- Current width for newly created vertical scribes
		do
			Result := view.vertical_scribe_width
		end

	default_scribe_width: DOUBLE
			-- Default width for scribes
		do
			Result := wafer_techns_row.max_scribe_width
		end

	minimum_scribe_width: DOUBLE
			-- Minimum width for scribes
		do
			if wafer_techns_row.is_null_design_techn_scribe then
				handle_hard_error ("cannot compute minimum scribe width because DESIGN_TECHN_SCRIBE is null for wafer techn " + wafer_techns_row.wafer_techn)
			end
			Result := wafer_techns_row.minimum_scribe_width
		end

	maximum_horizontal_scribe_width: DOUBLE
		do
			Result := packing.litho_sizes.payload_size_y
		end

	maximum_vertical_scribe_width: DOUBLE
		do
			Result := packing.litho_sizes.payload_size_x
		end

	horizontal_external_scribe_width: DOUBLE
		do
			Result := packing.litho_sizes.external_kerf_y
		end

	vertical_external_scribe_width: DOUBLE
		do
			Result := packing.litho_sizes.external_kerf_x
		end

	rounded_horizontal_external_scribe_width: DOUBLE
			-- External horizontal scribe width rounded to grid
		do
			Result := grid.rounded_size (horizontal_external_scribe_width)
		end

	rounded_vertical_external_scribe_width: DOUBLE
			-- External vertical scribe width rounded to grid
		do
			Result := grid.rounded_size (vertical_external_scribe_width)
		end

	maximum_filler_size: DOUBLE
			-- Maximum width and height of filler
		do
			Result := packing.litho_sizes.payload_size_x.max (packing.litho_sizes.payload_size_y)
		end

	set_model_fields
			-- Set fields in `model'
		local
			names: LIST [STRING]
			ls: LITHO_SIZES_ROW
			fs: FIELD_SETS_ROW
			ret_field: RETICLE_FIELD
			ret_fields: ARRAYED_LIST [RETICLE_FIELD]
			ret_field_label: RETICLE_FIELD_LABEL
			ret_field_labels: ARRAYED_LIST [RETICLE_FIELD_LABEL]
			x, y, sep: DOUBLE
			pay_size: CONST_POINT
			size, label_size, loc: POINT
		do
			fs := packing.field_set
			names := fs.field_names
			if names.is_empty then
				handle_hard_error ("No fields found for field set " + fs.field_set_name)
			end
			ls := fs.litho_size
			pay_size := ls.payload_size
			if not grid.is_size_on_grid (pay_size.x) or not grid.is_size_on_grid (pay_size.y) then
				handle_hard_error ("Payload size (" + grid.formatted_number (pay_size.x) + ", " + grid.formatted_number (pay_size.y) + ") not a multiple of size resolution " + grid.size_resolution_text + " in X and/or Y")
			end
			-- Now put size exactly on grid
			create size.make_xy (pay_size.x, pay_size.y)
			round_size_to_grid (size)
			sep := grid.rounded_edge (size.y * Field_spacing_fraction).max (grid.size_resolution)
			-- Set coordinates for field lower left corner
			x := 0.0
			y := grid.rounded_edge ((names.count - 1) * (size.y + sep))
			create ret_fields.make (5)
			create ret_field_labels.make (5)
			from
				names.start
			until
				names.after
			loop
				create loc.make_xy (x, y)
				round_edge_to_grid (loc)
				loc.move_by (size.x / 2.0, size.y / 2.0)
					-- Convert lower left to center
				create ret_field.make (fs.field_set_name, names.item, size, loc, Field_spacing_fraction, model)
				ret_fields.extend (ret_field)
				create label_size.make_xy (size.x, (size.y * Field_label_fraction).max (grid.size_resolution))
				round_size_to_grid (label_size)
				create loc.make_xy (x, y - label_size.y)
				round_edge_to_grid (loc)
				loc.move_by (label_size.x / 2.0, label_size.y / 2.0)
					-- Convert lower left to center
				create ret_field_label.make (ret_field, label_size, loc, model)
				ret_field_labels.extend (ret_field_label)
				ret_field.set_label (ret_field_label)
				y := grid.rounded_edge (y - (size.y + sep))
				names.forth
			end
			model.set_fields (ret_fields)
			model.set_field_labels (ret_field_labels)
			drawing_area_event_handler.set_field_names (names)
			drawing_area_event_handler.add_late_key_bindings
		end

	round_size_to_grid (p: POINT)
		do
			p.make_xy (grid.rounded_size (p.x), grid.rounded_size (p.y))
		end

	round_edge_to_grid (p: POINT)
		do
			p.make_xy (grid.rounded_edge (p.x), grid.rounded_edge (p.y))
		end

	check_changes_and_select_version (p: PACKING_VERSIONS_MROW)
			-- Check whether there are any unsaved changes.
			-- If so, get confirmation from user and then
			-- select version of packing whose row is `p'
		do
			create operation.make (agent process_select_version_request (p))
			operation.enable_abort_caught
			operation.execute
			if operation.aborted then
				version_selection_interface.enable_active
			else
				select_version (p)
			end
		end

	process_select_version_request (p: PACKING_VERSIONS_MROW)
			-- Process request to select version of
			-- packing whose row is `p'
		do
			if not is_read_only and packing.is_current_version_modified then
				confirm ("There are unsaved changes.%NEdit " + p.version_title + " and lose all changes since last save")
			end
		end

	select_version (p: PACKING_VERSIONS_MROW)
			-- Select version of packing whose row is `p'
		local
			e: PACKING_VERSION_SELECTED_EVENT
			has_fixed, has_locked: BOOLEAN
		do
			packing.set_current_version (p)
			model.set_from_version (packing.current_version)
			controller.set_initial_model_elements (model.elements, False)
			has_fixed := view.has_fixed_element
			has_locked := view.has_locked_element
			if has_fixed and has_locked then
				handle_hard_error ("packing version " + packing.current_version.version_no.out + " has both fixed and locked objects")
			elseif has_fixed and not fixed_button.is_selected then
				fixed_button.enable_select
			elseif has_locked and not locked_button.is_selected then
				locked_button.enable_select
			end
			gui.do_once_on_idle (agent display_initial_version (p))
			reset_title
			create e
			e.set_version (p)
			e.publish
		end

	display_initial_version (p: PACKING_VERSIONS_ROW)
			-- Display initial version `p'
		do
			display_version (p)
			drawing_area_event_handler.enable_key_press_handling
			set_pointer_style (saved_pointer_style)
		end

	display_version (p: PACKING_VERSIONS_ROW)
			-- Display version `p'
		do
			view.show_entire_model_without_redraw
			view.set_final_pack_mode_view
			view.enable_redraw
			-- Set mode according to whether there are any
			-- cut patterns
			if is_cut_editor_only or model.has_cut_patterns then
				set_cut_mode
				check_for_unlocked_or_unassigned_projects
			else
				set_pack_mode
				view.redraw
			end
			controller.report_selected_version (p)
		end

	add_new_projects (list: LIST [PACKING_DIES_MROW])
			-- Add new projects in `list' to model and view
		do
			model.add_dies (list)
			controller.add_model_elements (model.new_elements)
		end

	remove_deleted_projects (list: LIST [PACKING_DIES_MROW])
			-- Remove projects in `list' from model and view
		do
			controller.remove_model_elements (list)
		end

	update_projects (list: LIST [PACKING_DIES_MROW])
			-- Update in `list' with new information
		do
			controller.update_model_elements (list)
		end

	reset_title
			-- Reset window title to reflect current version
			-- being edited
		local
			verb, l_title: STRING
		do
			if is_read_only then
				verb := "View"
			else
				verb := "Edit"
			end
			l_title := verb + " "
			if is_cut_editor_only then
				l_title := l_title + "Cut Patterns for " + packing.current_version.version_id
			else
				l_title := packing_session.title_with_run_and_packing_id (l_title + packing.current_version.version_title + " of ")
			end
			set_title (l_title)
		end

	show_version_selection_window
			-- Create version selection window if it does
			-- not already exist.  Otherwise, just show it
		do
			build_version_selection_window
			version_selection_interface.raise
		end

	build_version_selection_window
			-- Create version selection window
		local
			w: EV_TITLED_WINDOW
		do
			if not version_selector_built then
				create w
				version_selection_interface ?= session.factory.packing_versions_interface (w, packing_session.run_id, packing_session.packing_id, packing_session.title_with_run_and_packing_id ("Select Version for"))
				version_selection_interface.set_session (packing_session)
				version_selection_interface.build
				version_selection_interface.select_actions.extend (agent check_changes_and_select_version)
				version_selection_interface.send_actions.extend (agent process_version_send)
				version_selection_interface.cancel_actions.extend (agent process_version_decline)
				version_selection_interface.subscribe_events
				version_selector_built := True
			end
		end

	process_version_decline
			-- Process a decline request on version selection window
		do
			version_selection_interface.hide
			version_selection_interface.enable_active
		end

	process_manage_projects
			-- Handle request to manage projects
		do
			show_project_management_window
		end

	build_project_management_window
			-- Build project management window if it does
			-- not already exist
		local
			w: EV_TITLED_WINDOW
		do
			if not project_manager_built then
				create w
				project_management_interface ?= session.factory.packing_projects_interface (w, packing_session.run_id, packing_session.packing_id, packing_session.title_with_run_and_packing_id ("Manage Projects for"))
				project_management_interface.set_packing_interface (Current)
				project_management_interface.set_session (packing_session)
				project_management_interface.set_packing (packing)
				project_management_interface.set_is_read_only (is_read_only)
				project_management_interface.build
				project_management_interface.projects_added_actions.extend (agent add_new_projects)
				project_management_interface.projects_removed_actions.extend (agent remove_deleted_projects)
				project_management_interface.projects_updated_actions.extend (agent update_projects)
				project_management_interface.send_actions.extend (agent process_project_send)
				project_management_interface.cancel_actions.extend (agent process_project_decline)
				project_management_interface.subscribe_events
				project_manager_built := True
			end
		end

	process_project_send (what, a_text: STRING)
			-- Process a Send request for `what' from Project 
			-- Management interface with `a_text'
		require
			what_not_void: what /= Void
			a_text_not_void: a_text /= Void
		do
			controller.save_project_info ("Project Management", what, a_text)
		end

	process_version_send (what, a_text: STRING)
			-- Process a Send request for `what' from Packing
			-- Versions interface with `a_text'
		require
			what_not_void: what /= Void
			a_text_not_void: a_text /= Void
		do
			controller.save_project_info ("Packing Versions", what, a_text)
		end

	process_lot_summary_send (what, a_text: STRING)
			-- Process a Send request for `what' from Lot Summary 
			-- interface with `a_text'
		require
			what_not_void: what /= Void
			a_text_not_void: a_text /= Void
		do
			controller.save_project_info ("Lot Summary", what, a_text)
		end

	process_project_decline
			-- Process a decline request on project management
			-- window
		do
			project_management_interface.hide
			project_management_interface.enable_active
		end

	build_application_colors_window
			-- Build application colors window if it does
			-- not already exist
		local
			w: EV_TITLED_WINDOW
		do
			if not application_colors_built then
				create w
				application_colors_interface ?= session.factory.packing_application_colors_interface (w, packing_session.run_id, packing_session.packing_id, packing_session.title_with_run_and_packing_id ("Set Application Colors for"))
				application_colors_interface.set_session (packing_session)
				application_colors_interface.set_named_colors (named_colors)
				application_colors_interface.set_application_colors (application_colors)
				application_colors_interface.build
				application_colors_interface.accept_actions.extend (agent set_application_colors_and_hide)
				application_colors_interface.apply_actions.extend (agent set_application_colors)
				application_colors_interface.cancel_actions.extend (agent process_application_colors_decline)
				application_colors_built := True
			end
		end

	process_application_colors_decline
			-- Process a decline request on packer application
			-- colors window
		do
			application_colors_interface.hide
			application_colors_interface.enable_active
		end

	set_application_colors_and_hide (a_list: LIST [APPLICATION_COLORS_ROW])
			-- Process an accept request on packer application
			-- colors window
		require
			a_list_not_void: a_list /= Void
		do
			set_application_colors (a_list)
			process_application_colors_decline
		end

	set_application_colors (a_list: LIST [APPLICATION_COLORS_ROW])
			-- Process an Apply request on packer application
			-- colors window
		require
			a_list_not_void: a_list /= Void
		local
			row: APPLICATION_COLORS_ROW
		do
			from
				a_list.start
			until
				a_list.after
			loop
				row := a_list.item
				palette.set_color_by_role (row.color_role, to_ev_color (row))
				a_list.forth
			end
			view.redraw
		end

	build_scribe_properties_window
			-- Build window to set scribe properties
		local
			w: EV_TITLED_WINDOW
		do
			if not scribe_properties_built then
				create w
				scribe_properties_interface ?= session.factory.packing_scribes_interface (w, packing_session.run_id, packing_session.packing_id, packing_session.title_with_run_and_packing_id ("Set Scribe Widths for"))
				scribe_properties_interface.set_session (packing_session)
				scribe_properties_interface.set_grid (grid)
				scribe_properties_interface.set_minimum_scribe_width (minimum_scribe_width)
				scribe_properties_interface.set_maximum_horizontal_scribe_width (maximum_horizontal_scribe_width)
				scribe_properties_interface.set_maximum_vertical_scribe_width (maximum_vertical_scribe_width)
				scribe_properties_interface.set_default_scribe_width (default_scribe_width)
				scribe_properties_interface.set_horizontal_external_scribe_width (rounded_horizontal_external_scribe_width)
				scribe_properties_interface.set_vertical_external_scribe_width (rounded_vertical_external_scribe_width)
				scribe_properties_interface.set_current_horizontal_scribe_width_agent (agent current_horizontal_scribe_width)
				scribe_properties_interface.set_current_vertical_scribe_width_agent (agent current_vertical_scribe_width)
				scribe_properties_interface.build
				scribe_properties_interface.accept_actions.extend (agent set_scribe_widths)
				scribe_properties_interface.cancel_actions.extend (agent process_scribes_decline)
				scribe_properties_built := True
			end
		end

	build_payload_sizes_window
			-- Build window to set payload and
			-- kerf sizes
		local
			w: EV_TITLED_WINDOW
		do
			if not payload_sizes_built then
				create w
				payload_sizes_interface ?= session.factory.packing_payload_sizes_interface (w, packing_session.run_id, packing_session.packing_id, packing_session.title_with_run_and_packing_id ("Set Payload and Kerf Sizes for"))
				payload_sizes_interface.set_session (packing_session)
				payload_sizes_interface.set_grid (grid)
				payload_sizes_interface.set_is_read_only (is_read_only)
				payload_sizes_interface.set_is_pack_mode (not view.is_cut_mode)
				payload_sizes_interface.set_current_payload_size_agent (agent current_payload_size)
				payload_sizes_interface.set_current_field_size_agent (agent current_field_size)
				payload_sizes_interface.build
				payload_sizes_interface.accept_actions.extend (agent set_payload_and_field_sizes)
				payload_sizes_interface.cancel_actions.extend (agent process_payload_sizes_decline)
				payload_sizes_built := True
			end
		end

	build_lot_summary_window
			-- Build the lot summary window
		local
			w: EV_DIALOG
		do
			if not lot_summary_interface_built then
				create w
				lot_summary_interface ?= session.factory.packing_lot_summary_interface (w, packing_session.run_id, packing_session.packing_id, packing_session.title_with_run_and_packing_id ("Lot Summary for"))
				lot_summary_interface.set_session (packing_session)
				lot_summary_interface.set_packing (packing)
				lot_summary_interface.set_projects_agent (agent current_customer_projects)
				lot_summary_interface.build
				lot_summary_interface.send_actions.extend (agent process_lot_summary_send)
				lot_summary_interface.cancel_actions.extend (agent process_lot_summary_decline)
				lot_summary_interface.enable_active
				lot_summary_interface_built := True
			end
		end

	current_customer_projects: LIST [PACKING_DIES_MROW]
			-- Current list of distinct dies for customer projects
		do
			Result := view.customer_projects
		end

	build_search_window
			-- Build window to allow searching of packing
		local
			w: EV_TITLED_WINDOW
		do
			if not search_interface_built then
				create w
				search_interface ?= session.factory.packing_search_interface (w, packing_session.run_id, packing_session.packing_id, packing_session.title_with_run_and_packing_id ("Search"))
				search_interface.set_session (packing_session)
				search_interface.set_view_elements_agent (agent current_view_elements)
				search_interface.build
				search_interface.search_actions.extend (agent search_for_design_numbers)
				search_interface.cancel_actions.extend (agent process_search_decline)
				search_interface_built := True
			end
		end

	build_help_window
			-- Build window to provide help while packing
		local
			w: EV_TITLED_WINDOW
		do
			if not help_interface_built then
				create w
				help_interface ?= session.factory.packing_help_interface (w, packing_session.run_id, packing_session.packing_id, packing_session.title_with_run_and_packing_id ("Key and Pointer Bindings for"))
				help_interface.set_bindings (drawing_area_event_handler.bindings)
				help_interface.build
				help_interface.save_actions.extend (agent controller.save_help_info)
				help_interface.cancel_actions.extend (agent process_help_decline)
				help_interface_built := True
			end
		end

	current_view_elements: LIST [RETICLE_VIEW_ELEMENT]
			-- Current list of view elements
		do
			Result := view.elements
		end

	build_packer_output_window
			-- Build packer output window if it does
			-- not already exist
		local
			w: EV_TITLED_WINDOW
		do
			if not packer_output_built then
				create w
				packer_output_interface ?= session.factory.packing_output_interface (w, packing_session.run_id, packing_session.packing_id, packing_session.title_with_run_and_packing_id ("Packer Output for"))
				packer_output_interface.set_session (packing_session)
				packer_output_interface.build
				packer_output_interface.cancel_actions.extend (agent process_output_decline)
				packer_output_built := True
			end
		end

	process_output_decline
			-- Process a decline request on packer output
			-- window
		do
			packer_output_interface.hide
			packer_output_interface.enable_active
		end

	show_packer_info_window
			-- Build packer information window if it does
			-- not already exist.  Otherwise, just show it
		do
			if not packer_info_built then
				build_packer_info_window
			end
			packer_info_interface.raise
		end

	build_packer_info_window
			-- Build packer info window if it does
			-- not already exist
		local
			w: EV_TITLED_WINDOW
		do
			if not packer_info_built then
				create w
				packer_info_interface ?= session.factory.packing_info_interface (w, packing_session.run_id, packing_session.packing_id, packing_session.title_with_run_and_packing_id ("Die Information for"))
				packer_info_interface.set_session (packing_session)
				packer_info_interface.set_packing (packing)
				packer_info_interface.set_grid (grid)
				packer_info_interface.build
				packer_info_interface.save_actions.extend (agent controller.save_die_info)
				packer_info_interface.cancel_actions.extend (agent process_info_decline)
				gui.do_once_on_idle (agent position_packer_info_window)
				packer_info_built := True
			end
		end

	position_packer_info_window
		local
			l_screen: EV_SCREEN
		do
			create l_screen
			packer_info_interface.set_size (l_screen.width, 300)
			packer_info_interface.set_position (0, 100)
			create l_screen
			
		end

	process_info_decline
			-- Process a decline request on packer information
			-- window
		do
			packer_info_interface.hide
			packer_info_interface.enable_active
		end

	process_lot_summary_decline
			-- Process a decline request on packer lot summary
			-- window
		do
			lot_summary_interface.hide
			lot_summary_interface.enable_active
		end

	search_for_design_numbers (a_list: LIST [INTEGER] a_select_also: BOOLEAN)
			-- Process a request to search packing for
			-- design numbers in `a_list'.  If `a_select_also',
			-- then select these projects too
		do
			controller.search_for_design_numbers (a_list, a_select_also)
			set_search_results_view
		end

	process_search_decline
			-- Process a decline request on search window
		do
			search_interface.hide
			search_interface.enable_active
		end

	process_help_decline
			-- Process a decline request on help window
		do
			help_interface.hide
			help_interface.enable_active
		end

	set_payload_and_field_sizes (a_payload_size_x, a_payload_size_y, a_field_size_x, a_field_size_y: DOUBLE)
			-- Process a request to set payload
			-- and field sizes
		local
			kerf_x, kerf_y: DOUBLE
		do
			-- FIXME
			-- view.set_payload_size (a_payload_size_x, a_payload_size_y)
			kerf_x := a_field_size_x - a_payload_size_x
			kerf_y := a_field_size_y - a_payload_size_y
			payload_sizes_interface.hide
			payload_sizes_interface.enable_active
			status_reporter.report_change ("Payload size set to " + grid.formatted_number (a_payload_size_x) + " by " + grid.formatted_number (a_payload_size_y) + ", external kerf set to " + grid.formatted_number (kerf_x) + " by " + grid.formatted_number (kerf_y) + ", field size set to " + grid.formatted_number (a_field_size_x) + " by " + grid.formatted_number (a_field_size_y))
		end

	process_payload_sizes_decline
			-- Process a decline request on payload sizes
			-- window
		do
			payload_sizes_interface.hide
			payload_sizes_interface.enable_active
			status_reporter.report_cancellation ("Payload size change cancelled")
		end

	set_scribe_widths (a_horizontal_width, a_vertical_width: DOUBLE)
			-- Process a request to set horizontal scribe
			-- width to `a_horizontal_width' and vertical
			-- scribe width to `a_vertical_width'
		local
			old_horiz_width, old_vert_width, def_width: DOUBLE
		do
			old_horiz_width := view.horizontal_scribe_width
			old_vert_width := view.vertical_scribe_width
			def_width := view.default_scribe_width
			view.set_custom_scribe_widths (a_horizontal_width, a_vertical_width, packing)
			packing.save_new_scribe_widths (old_horiz_width, old_vert_width, a_horizontal_width, a_vertical_width, def_width)
			scribe_properties_interface.hide
			scribe_properties_interface.enable_active
			status_reporter.report_change ("Scribe widths set to Horizontal = " + grid.formatted_number (a_horizontal_width) + " and Vertical = " + grid.formatted_number (a_vertical_width))
		end

	process_scribes_decline
			-- Process a decline request on scribe properties
			-- window
		do
			scribe_properties_interface.hide
			scribe_properties_interface.enable_active
			status_reporter.report_cancellation ("Scribe width change cancelled")
		end

	project_to_die_size (an_overhead_style: STRING a_size: CONST_POINT): CONST_POINT
			-- Die size for project size `a_size' using
			-- `an_overhead_style'
		do
			Result := wafer_techns_row.die_size_with_overhead (an_overhead_style, packing.packings_row.other_border, a_size)
		end

	die_to_project_size (an_overhead_style: STRING a_size: CONST_POINT): CONST_POINT
			-- Maximum project size for die size `a_size'
			-- using `an_overhead_style'
		do
			Result := wafer_techns_row.project_size_without_overhead (an_overhead_style, packing.packings_row.other_border, a_size)
		end

	set_filler_properties (a_name: STRING a_width, a_height: DOUBLE)
			-- Process a request to set filler name
			-- to `a_name' and size `a_width' by `a_height'
		do
			controller.set_filler_properties (a_name, a_width, a_height)
			filler_properties_interface.destroy
		end

	cancel_filler_creation
			-- Process a decline request on filler properties
			-- window
		do
			controller.cancel_filler_creation
			filler_properties_interface.destroy
		end

	move_absolute (a_field_name: STRING a_x, a_y: DOUBLE)
			-- Process a request to move single selected object
			-- to (`a_x', `a_y') relative to center of
			-- `a_field_name'
		do
			controller.move_absolute (a_field_name, a_x, a_y)
			move_absolute_interface.destroy
		end

	cancel_move_absolute
			-- Process a decline request on Move Absolute
			-- window
		do
			move_absolute_interface.destroy
			controller.cancel_move_absolute
		end

	move_relative (a_x, a_y: DOUBLE)
			-- Process a request to move selected objects
			-- by (`a_x', `a_y') relative to their current
			-- position
		do
			move_relative_interface.destroy
			controller.move_relative (a_x, a_y)
		end

	cancel_move_relative
			-- Process a decline request on Move Relative
			-- window
		do
			move_relative_interface.destroy
			controller.cancel_move_relative
		end

	set_lot_split_colors (a_color_list: LIST [PACKING_ETEST_COLORS_MROW] changed: BOOLEAN)
			-- Take actions needed when user accepts lot split
			-- colors `a_color_list' where `changed' indicates
			-- whether any changes were made
		require
			a_color_list_not_void: a_color_list /= Void
		do
			lot_split_colors_interface.destroy
			split_colors := a_color_list
			palette.set_split_colors (split_colors)
			view.notify_split_colors_changed
		end

	assign_lot (a_wafer_etest_row: WAFER_ETEST_NAMES_ROW)
			-- Take actions needed when user accepts lot
			-- indicated by `a_wafer_etest_row' for
			-- selected customer projects that have more
			-- than one compatible lot
		do
			lot_assignment_interface.destroy
			controller.set_lot_etest_row (a_wafer_etest_row)
		end

	destroy_lot_split_color_window
			-- Destroy the window used to set lot split colors
		do
			lot_split_colors_interface.destroy
		end

	destroy_lot_assignment_window
			-- Destroy the window used to assign lots
		do
			lot_assignment_interface.destroy
			controller.cancel_lot_assignment
		end

	delete_projects (wants_unlock, wants_deassign: BOOLEAN)
			-- Process a request to delete projects
			-- with `wants_unlock' and `wants_deassign'
		do
			controller.set_delete_properties (wants_unlock, wants_deassign)
			project_delete_interface.destroy
		end

	cancel_project_deletion
			-- Process a decline request on project delete window
		do
			controller.cancel_delete
			project_delete_interface.destroy
		end

	get_new_version_name
			-- Show dialog to allow user to optionally enter name
			-- of new packing version or to cancel
		do
			build_version_name_dialog
			version_name_dialog.show_modal_to_window (window)
		end

	build_version_name_dialog
			-- Build version name dialog
		local
			cols: WIDGET_COLUMN_SET
			buttons: HORIZONTAL_BUTTON_SET
			main_vbox: EV_VERTICAL_BOX
			box: EV_BOX
		do
			create version_name_dialog
			create main_vbox
			create cols.make (2)
			cols.set_padding (user_interface.padding)
			cols.set_border_width (user_interface.border_width)
			create version_id_field
			version_id_field.set_capacity ({PACKING_VERSIONS_ROW}.maxlen_version_id)
			version_id_field.set_minimum_width_in_characters (20)
			cols.append_row (<< gui.label_with_text ("Optional version name"), version_id_field >>)
			cols.disable_expand
			cols.align_column_right (1)
			cols.align_column_left (2)
			box := cols.box
			main_vbox.extend (box)
			main_vbox.disable_item_expand (box)
			create buttons.make_with_text_and_actions (<<
				"OK", "Cancel" >>,
				<< agent process_name_ok, agent process_name_cancel >>)
			version_id_field.return_actions.extend (agent process_name_ok)
			box := buttons.column_set.box
			main_vbox.extend (box)
			main_vbox.disable_item_expand (box)
			gui.set_box_background_recursive (main_vbox, background_color)
			version_name_dialog.extend (main_vbox)
			version_name_dialog.set_title ("Save New Version")
		end

	process_name_ok
		do
			new_version_name := version_id_field.text
			new_version_name.left_adjust
			new_version_name.right_adjust
			if not new_version_name.is_empty and then packing.is_version_id_used (new_version_name) then
				inform ("Version name " + su.double_quoted (new_version_name) + " has already been used")
			else
				version_name_dialog.destroy
			end
		end

	process_name_cancel
		do
			new_version_name := Void
			version_name_dialog.destroy
		end

	
	add_cut_pattern (a_pattern_no: INTEGER a_field_name, a_wafer_etest_name: STRING a_wafer_quantity: INTEGER)
			-- Process a request to add a new cut pattern with
			-- `a_pattern_no' for `a_field_name'.  New pattern
			-- is intended to isolate projects whose split is 
			-- `a_wafer_etest_name' and number of wafers to cut
			-- is `a_wafer_quantity' (negative if not specified)
		do
			controller.create_cut_pattern (a_pattern_no, a_field_name, a_wafer_etest_name, a_wafer_quantity)
			cut_pattern_properties_interface.destroy
		end

	cancel_cut_pattern_properties
			-- Process a decline request on window to set cut
			-- pattern properties window
		do
			cut_pattern_properties_interface.destroy
			controller.cancel_cut_pattern_properties
		end

	check_for_unlocked_or_unassigned_projects
			-- Warn user if there are any unlocked or unassigned
			-- projects in Cut mode.  Set 
			-- `has_unlocked_or_unassigned_projects'
		local
			unlocked_list, unassigned_list: LIST [PACKING_PROJECTS_ROW]
		do
			unlocked_list := packing.selected_unlocked_projects
			unassigned_list := packing.selected_locked_unassigned_projects (packing_session.run_id)
			has_unlocked_or_unassigned_projects := not unlocked_list.is_empty or not unassigned_list.is_empty
			if has_unlocked_or_unassigned_projects then
				report_unlocked_or_unassigned_projects ("finish cut patterns", unlocked_list, unassigned_list)
			end
		end

	switch_to_cut_mode
			-- Do actions needed when switch to Cut mode
		local
			err_count, warn_count, n: INTEGER
			rot_180_count, rot_270_count: INTEGER
			ok: BOOLEAN
			incomp, msg: STRING
			proj_list, unlocked_list, unassigned_list: LIST [PACKING_PROJECTS_ROW]
			assigned_list: LIST [IC_FAB_INSTANCES_ROW]
			fp_row: FAB_POOL_W_WAFER_TECHNS_ROW
			fab_pool_list: LIST [FAB_POOL_W_WAFER_TECHNS_ROW]
			old_style: EV_POINTER_STYLE
		do
			ok := True
			old_style := pointer_style
			set_pointer_style (gui.Wait_cursor)
			gui.process_events
			
			-- Check for unlocked or unassigned projects

			if ok then
				unlocked_list := packing.selected_unlocked_projects
				unassigned_list := packing.selected_locked_unassigned_projects (packing_session.run_id)
				if not unlocked_list.is_empty or not unassigned_list.is_empty then
					report_unlocked_or_unassigned_projects ("switch to Cut mode", unlocked_list, unassigned_list)
					ok := False
				end
			end
				
			-- Check for projects assigned to run but not included
			-- in packing

			if ok then
				assigned_list := packing.unselected_assigned_fab_instances (packing_session.run_id)
				if not assigned_list.is_empty then
					report_unselected_assigned_projects (assigned_list)
					ok := False
				end
			end
				
			-- Check for project incompatibilities with packing
			-- (policy violations like SKD project on DED run)

			if ok then				
				proj_list := packing.selected_customer_projects
				create fp_row.make_template
				fab_pool_list := fp_row.passed_projects_for_wafer_techn (packing.wafer_techn, connection, expected_count.At_least_zero)
				incomp := project_packing_incompatibilities (proj_list, fab_pool_list)
				if not incomp.is_empty then
					inform ("Cannot switch to Cut mode due to the following unresolved project incompatibilities with run:%N%N" + incomp)
					ok := False
				end
			end

			-- Run all validity checks if new packer enabled

			if ok then
				view.set_final_pack_mode_view
				controller.check_all_pack_validity
				err_count := controller.validity_error_count
				warn_count := controller.validity_warning_count
				if err_count > 0 then
					inform ("Cannot switch to Cut mode due to " + err_count.out + " validity " + su.number (err_count, "error", "errors"))
					ok := False
				elseif warn_count > 0 then
					warn (warn_count.out + " validity " + su.number (warn_count, "warning", "warnings") + " listed in Output window.%NSwitch to Cut mode anyway?")
				end
			end

			-- Warn user if there are multiple distinct
			-- scribe widths

			if ok then
				view.check_multiple_scribe_widths
				if not view.warning_messages.is_empty then
					warn (view.warning_messages.first + "%N%NProceed anyway?")
				end
			end

			-- Warn user if there are any fillers still included
			-- in packing

			if ok then
				n := view.fillers.count
				if n > 0 then
					warn (n.out + " " + su.number (n, "filler", "fillers") + " included in packing%N%NProceed anyway?")
				end
			end

			-- Warn user if there are any objects with unusual
			-- rotations in packing

			if ok then
				rot_180_count := view.rotated_180_elements.count
				rot_270_count := view.rotated_270_elements.count
				n := rot_180_count + rot_270_count
				if n > 0 then
					msg := "Packing includes " + n.out + " " + su.number (n, "die", "dies") + " with non-standard rotation:%N%N"
					if rot_180_count > 0 then
						msg.append ("   " + rot_180_count.out + " " + su.number (rot_180_count, "die", "dies") + " rotated 180 degrees clockwise%N")
					end
					if rot_270_count > 0 then
						msg.append ("   " + rot_270_count.out + " " + su.number (rot_270_count, "die", "dies") + " rotated 270 degrees clockwise%N")
					end
					msg.append ("%NProceed anyway?")
					warn (msg)
				end
			end

			-- Tell user how many projects are assigned to each
			-- lot split

			if ok then
				confirm ("Number of customer projects on each lot split:%N%N" + view.lot_split_distribution_text + "%NProceed")
			end

			-- Save new version if packing modified and not in
			-- read-only mode

			if ok and then packing.is_current_version_modified and not is_read_only then
				controller.save_new_version
				if is_new_version_saved then
					if packing.is_current_version_modified then
						set_pointer_style (old_style)
						handle_internal_error ("new version saved but newest version is still modified")
					end
				else
					ok := False
				end
			end

			if ok then
				set_cut_mode
			end

			is_cut_mode := ok
			set_pointer_style (old_style)
		rescue
			if old_style /= Void then
				set_pointer_style (old_style)
			end
		end

	process_finish_request
			-- Handle request to finish packing step
		local
			curr_vers: INTEGER
			ok: BOOLEAN
		do
			disable_active
			ok := True
			
			-- Make sure there are no projects that are unlocked
			-- or not assigned to this run

			if ok then
				check_for_unlocked_or_unassigned_projects
				ok := not has_unlocked_or_unassigned_projects
			end
			
			-- Save new version if packing modified

			if ok and then packing.is_current_version_modified then
				controller.save_new_version
				disable_active
				if is_new_version_saved then
					if packing.is_current_version_modified then
						handle_internal_error ("new version saved but newest version is still modified")
					end
				else
					ok := False
				end
			end
			
			-- Update RUNS table to point to current packing
			-- version

			if ok then
				-- User saved new version or there were
				-- no changes to save
				fetch_runs_row_by_id
				curr_vers := packing.current_version.version_no
				if runs_mrow.packing_version_no /= curr_vers then
					-- User selected older version but
					-- didn't modify it - act like it was
					-- just saved
					runs_mrow.set_packing_version_no (curr_vers)
				end
				if not packings_row.packed_with_new_packer then
					-- Packing has now been packed
					-- with Unix packer (not old ICL VMS packer)
					packings_row.set_packed_with_new_packer (True)
				end
				destroy_dependent_interfaces
				-- Don't publish final test vehicle list here
				-- (moved to a later step so that it is only
				-- sent when data is in official DB tables)
				success_actions.call ([])
			end
			is_finish_completed := ok
			enable_active
		end

	report_unlocked_or_unassigned_projects (what: STRING unlocked_list, unassigned_list: LIST [PACKING_PROJECTS_ROW])
			-- Tell user `what' cannot be done because of
			-- projects in `unlocked_list' that are
			-- selected but are not locked and/or because
			-- of projects in `unassigned_list' that are
			-- locked but are not assigned to run
		require
			unlocked_list_not_void: unlocked_list /= Void
			unassigned_list_not_void: unassigned_list /= Void
		local
			msg, status: STRING
			unlocked_count, unassigned_count: INTEGER
			proj: PACKING_PROJECTS_ROW
			list: like unlocked_list
		do
			create msg.make (200)
			unlocked_count := unlocked_list.count
			unassigned_count := unassigned_list.count
			if unlocked_count > 0 then
				msg.append (unlocked_count.out + " " + su.number (unlocked_count, "project", "projects") + " included in packing but not locked%N")
			end
			if unassigned_count > 0 then
				msg.append (unassigned_count.out + " " + su.number (unassigned_count, "project", "projects") + " included in packing and locked but not assigned to this run%N")
			end
			msg.extend ('%N')
				
			list := unlocked_list.twin
			list.append (unassigned_list)
			from
				list.start
			until
				list.after
			loop
				proj := list.item
				status := proj.locked_and_assigned_status
				if not status.is_empty then
					status.put (status.item (1).as_upper, 1)
				end
				msg.append ("   " + proj.design_no.out + "     " + status + "%N")
				list.forth
			end
			
			inform (msg + "%NCannot " + what + " until above projects have either been removed from packing%Nor locked and assigned to this run.")
		end

	report_unselected_assigned_projects (assigned_list: LIST [IC_FAB_INSTANCES_ROW])
			-- Tell user Finish cannot complete because of
			-- projects in `assigned_list' that are not included
			-- in packing but are assigned to run
		require
			assigned_list_not_void: assigned_list /= Void
		local
			msg: STRING
			assigned_count: INTEGER
			ifi: IC_FAB_INSTANCES_ROW
			list: like assigned_list
		do
			create msg.make (200)
			assigned_count := assigned_list.count
			if assigned_count > 0 then
				msg.append (assigned_count.out + " " + su.number (assigned_count, "project", "projects") + " not included in packing but assigned to run%N")
			end
			msg.extend ('%N')
				
			list := assigned_list
			from
				list.start
			until
				list.after
			loop
				ifi := list.item
				msg.append ("   " + ifi.design_no.out + "     " + "Assigned to run but not included in packing" + "%N")
				list.forth
			end
			
			inform (msg + "%NCannot switch to Cut mode until above projects have either been added to packing%Nor deassigned from this run.")
		end

	project_packing_incompatibilities (proj_list: LIST [PACKING_PROJECTS_ROW] fp_list: LIST [FAB_POOL_W_WAFER_TECHNS_ROW]): STRING
			-- Text describing unresolved incompatibilities between
			-- projects in `proj_list' and packing, as determined
			-- by information about project found in `fp_list'
		require
			proj_list_not_void: proj_list /= Void
			fp_list_not_void: fp_list /= Void
		local
			table: HASH_TABLE [FAB_POOL_W_WAFER_TECHNS_ROW, INTEGER]
		do
			create Result.make (100)
			create table.make (fp_list.count)
			fp_list.do_all (agent add_fab_pool_row (?, table))
			proj_list.do_all (agent check_project_compatibility (?, table, Result))
		end

	add_fab_pool_row (r: FAB_POOL_W_WAFER_TECHNS_ROW table: HASH_TABLE [FAB_POOL_W_WAFER_TECHNS_ROW, INTEGER])
			-- Add `r' to `table' stored under its m_fab_no
		require
			r_not_void: r /= Void
			table_not_void: table /= Void
		do
			table.force (r, r.m_fab_no)
		end

	check_project_compatibility (p: PACKING_PROJECTS_ROW table: HASH_TABLE [FAB_POOL_W_WAFER_TECHNS_ROW, INTEGER] errs: STRING)
			-- Check compatibility of `p' with packing, using
			-- `table' to find fab pool row.  Append any
			-- compatilility errors to `errs'
		require
			p_not_void: p /= Void
			table_not_void: table /= Void
			errs_not_void: errs /= Void
		local
			mfn: INTEGER
			fp_row: FAB_POOL_W_WAFER_TECHNS_ROW
		do
			mfn := p.m_fab_no
			table.search (mfn)
			if table.found then
				fp_row := table.found_item
				fp_row.check_policy_selectability (packing.packings_row, packing_session.run_id, False, False, False, True, False)
				if not fp_row.is_selectable then
					errs.append ("   " + p.design_no.out + "   " + fp_row.reason_unselectable + "%N")
				end
			else
				handle_hard_error ("cannot find fab pool row for m_fab_no " + mfn.out + " and wafer techn " + packing.wafer_techn)
			end
		end

	destroy_dependent_interfaces
			-- Destroy any interfaces which depend on
			-- `Current'
		do
			if version_selection_interface /= Void then
				version_selection_interface.destroy
			end
			if payload_sizes_interface /= Void then
				payload_sizes_interface.destroy
			end
			if scribe_properties_interface /= Void then
				scribe_properties_interface.destroy
			end
			if packer_output_interface /= Void then
				packer_output_interface.destroy
			end
			if packer_info_interface /= Void then
				packer_info_interface.destroy
			end
			if lot_summary_interface /= Void then
				lot_summary_interface.destroy
			end
			if search_interface /= Void then
				search_interface.destroy
			end
			if help_interface /= Void then
				help_interface.destroy
			end
			if project_management_interface /= Void then
				project_management_interface.unsubscribe_events
				project_management_interface.destroy
			end
			if application_colors_interface /= Void then
				application_colors_interface.destroy
			end
		end

	process_cancel_request
		do
			if not is_read_only and packing.is_current_version_modified then
				confirm ("There are unsaved changes.%NExit and lose all changes since last save")
			end
		end

	connection: DB_CONNECTION
			-- Connection for session
		do
			Result := session.connection
		end
	
	named_colors: LIST [NAMED_COLORS_ROW]
			-- List of supported named colors
		local
			row: NAMED_COLORS_ROW
		do
			if cached_named_colors = Void then
				create row.make_template
				cached_named_colors := row.all_rows_ordered (connection, "color_name")
			end
			Result := cached_named_colors
		end
	
	application_colors: LIST [APPLICATION_COLORS_MROW]
			-- List of application colors for this application
		local
			row: APPLICATION_COLORS_MROW
		do
			if cached_application_colors = Void then
				create row.make_template
				cached_application_colors := row.colors_for_application ("RUN_CLOSING", connection, expected_count.At_least_zero)
			end
			Result := cached_application_colors
		end
	
	cached_named_colors: LIST [NAMED_COLORS_ROW]
			-- List of supported named colors, cached in memory
			
	cached_application_colors: LIST [APPLICATION_COLORS_MROW]
			-- List of colors for this application, cached in
			-- memory
			
	runs_mrow: RUNS_MROW
			-- Modificable row for current run, if any

	last_menu_item: EV_MENU_ITEM
			-- Last menu item created

	file_menu: EV_MENU
			-- Menu for file operations

	pointer_coords_menu_item: EV_MENU_ITEM
			-- Menu item to show/hide line with field-relative
			-- coordinates of current mouse pointer position

	edit_menu: EV_MENU
			-- Current menu to allow user to do editing
			-- operations on drawing area

	pack_edit_menu: EV_MENU
			-- Menu to allow user to do editing
			-- operations on drawing area in Pack mode

	cut_edit_menu: EV_MENU
			-- Menu to allow user to do editing
			-- operations on drawing area in Cut mode

	fix_menu: EV_MENU
			-- Current menu to allow user to fix projects
			-- in X and/or Y

	pack_fix_menu: EV_MENU
			-- Menu to allow user to fix projects
			-- in X and/or Y in Pack mode

	cut_fix_menu: EV_MENU
			-- Menu to allow user to fix projects
			-- in X and/or Y in Cut mode

	unfix_menu: EV_MENU
			-- Current menu to allow user to unfix
			-- projects in X and/or Y

	pack_unfix_menu: EV_MENU
			-- Menu to allow user to unfix
			-- projects in X and/or Y in Pack mode

	cut_unfix_menu: EV_MENU
			-- Menu to allow user to unfix
			-- projects in X and/or Y in Cut mode

	lock_menu: EV_MENU
			-- Current menu to allow user to lock various
			-- attributes of projects

	pack_lock_menu: EV_MENU
			-- Menu to allow user to lock various
			-- attributes of projects in Pack mode

	cut_lock_menu: EV_MENU
			-- Menu to allow user to lock various
			-- attributes of projects in cut mode

	lock_all_menu: EV_MENU
			-- Menu to allow user to lock all projects

	lock_selected_menu: EV_MENU
			-- Menu to allow user to lock selected projects

	lock_unselected_menu: EV_MENU
			-- Menu to allow user to lock unselected projects

	unlock_menu: EV_MENU
			-- Current menu to allow user to unlock
			-- various attributes of projects

	pack_unlock_menu: EV_MENU
			-- Menu to allow user to unlock various
			-- attributes of projects in Pack mode

	cut_unlock_menu: EV_MENU
			-- Menu to allow user to unlock various
			-- attributes of projects in cut mode

	unlock_all_menu: EV_MENU
			-- Menu to allow user to unlock all projects

	unlock_selected_menu: EV_MENU
			-- Menu to allow user to unlock selected projects

	unlock_unselected_menu: EV_MENU
			-- Menu to allow user to unlock unselected projects

	view_menu: EV_MENU
			-- Menu to allow user to change the appearance
			-- of drawing area

	named_view_menu: EV_MENU
			-- Menu to allow user to choose standard named view
			-- of items in drawing area

	label_menu: EV_MENU
			-- Menu to allow user to change labeling of projects

	label_all_menu: EV_MENU
			-- Menu to allow user to change labeling of all
			-- projects

	label_selected_menu: EV_MENU
			-- Menu to allow user to change labeling of selected
			-- projects

	label_unselected_menu: EV_MENU
			-- Menu to allow user to change labeling of unselected
			-- projects

	color_menu: EV_MENU
			-- Menu to allow user to change coloring of projects

	color_all_menu: EV_MENU
			-- Menu to allow user to change coloring of all
			-- projects

	color_selected_menu: EV_MENU
			-- Menu to allow user to change coloring of selected
			-- projects

	color_unselected_menu: EV_MENU
			-- Menu to allow user to change coloring of unselected
			-- projects

	set_opacity_menu: EV_MENU
			-- Menu to allow user to change opacity of projects

	set_opacity_all_menu: EV_MENU
			-- Menu to allow user to change opacity of all
			-- projects

	set_opacity_selected_menu: EV_MENU
			-- Menu to allow user to change opacity of selected
			-- projects

	set_opacity_unselected_menu: EV_MENU
			-- Menu to allow user to change opacity of unselected
			-- projects

	fill_menu: EV_MENU
			-- Menu to allow user to change interior filling
			-- of projects

	select_menu: EV_MENU
			-- Menu to allow user to select/deselect items

	set_selection_menu: EV_MENU
			-- Menu to allow user to set selection to a list of
			-- items

	add_selection_menu: EV_MENU
			-- Menu to allow user to add items to selection

	remove_selection_menu: EV_MENU
			-- Menu to allow user to remove items from selection

	go_to_menu: EV_MENU
			-- Menu to allow user to display a specific area of
			-- drawing area

	check_menu: EV_MENU
			-- Menu to allow user to do various kinds of validity
			-- checks

	pack_check_menu: EV_MENU
			-- Menu to allow user to do various kinds of validity
			-- checks in Pack mode

	cut_check_menu: EV_MENU
			-- Menu to allow user to do various kinds of validity
			-- checks in Cut mode

	help_menu: EV_MENU
			-- Menu to provide help

	back_menu_item: EV_MENU_ITEM
			-- Menu item to allow user to go back to previous
			-- displayed area in history list

	forward_menu_item: EV_MENU_ITEM
			-- Menu item to allow user to go forward to next
			-- displayed area in history list

	next_pattern_menu_item: EV_MENU_ITEM
			-- Menu item to allow user to go forward to next
			-- cut pattern, in Cut mode

	previous_pattern_menu_item: EV_MENU_ITEM
			-- Menu item to allow user to go back to previous
			-- cut pattern, in Cut mode

	undo_menu_item: EV_MENU_ITEM
			-- Current menu item to allow user to undo
			-- previous change in history list

	pack_undo_menu_item: EV_MENU_ITEM
			-- Menu item to allow user to undo previous change
			-- in history list in Pack mode

	cut_undo_menu_item: EV_MENU_ITEM
			-- Menu item to allow user to undo previous change
			-- in history list in Cut mode

	redo_menu_item: EV_MENU_ITEM
			-- Current menu item to allow user to redo
			-- next change in history list

	pack_redo_menu_item: EV_MENU_ITEM
			-- Menu item to allow user to redo
			-- next change in history list in Pack mode

	cut_redo_menu_item: EV_MENU_ITEM
			-- Menu item to allow user to redo
			-- next change in history list in cut mode

	go_to_field_menu_items: LIST [EV_MENU_ITEM]
			-- Menu items to allow navigation to a specific field

	last_button: EV_BUTTON
			-- Last button created by `add_button'

	projects_button: EV_BUTTON
			-- Toolbar button to manage projects

	versions_button: EV_BUTTON
			-- Toolbar button to select a new version of packing

	back_button: EV_BUTTON
			-- Toolbar button to go to previous displayed area
			-- in history list

	forward_button: EV_BUTTON
			-- Toolbar button to go to next displayed area
			-- in history list

	undo_button: EV_BUTTON
			-- Toolbar button to undo previous change in
			-- history list

	redo_button: EV_BUTTON
			-- Toolbar button to redo next change in
			-- history list

	save_button: EV_BUTTON
			-- Toolbar button to save a new version of packing

	finish_button: EV_BUTTON
			-- Toolbar button to finish packing

	last_toggle_button: EV_TOGGLE_BUTTON
			-- Last button created by `add_toggle_button'

	fixed_button: EV_TOGGLE_BUTTON
			-- Toolbar button to select to make Fix/Unfix
			-- operations do old-style fix operation (fixed only
			-- for horizontal/vertical scanning purposes)
			-- lock in position operation (fixed for all purposes)

	locked_button: EV_TOGGLE_BUTTON
			-- Toolbar button to select to make Fix/Unfix
			-- operations do lock in position operation
			-- (fixed for all purposes)

	in_fixed_or_locked_action: BOOLEAN
			-- Are we currently in an action to set Fixed
			-- or Locked mode?

	standard_view_button: EV_TOGGLE_BUTTON
			-- Toolbar button to switch to standard view,
			-- where items are labeled by design number or
			-- standard identifying name if no design number

	account_type_view_button: EV_TOGGLE_BUTTON
			-- Toolbar button to switch to account type view,
			-- where items are labeled by account type

	max_die_size_view_button: EV_TOGGLE_BUTTON
			-- Toolbar button to switch to max die size view,
			-- where items are labeled by maximum die size

	lot_split_view_button: EV_TOGGLE_BUTTON
			-- Toolbar button to switch to lot split view,
			-- where items are labeled by wafer etest ID

	packaging_view_button: EV_TOGGLE_BUTTON
			-- Toolbar button to switch to Packaging view,
			-- where items are labeled by package ID and
			-- packaging vendor type

	validity_check_view_button: EV_TOGGLE_BUTTON
			-- Toolbar button to switch to Validity Check view,
			-- where items are colored by which validity
			-- errors/warnings they have

	cut_view_button: EV_TOGGLE_BUTTON
			-- Toolbar button to switch to Cut view,
			-- where items are colored by lot split and labeled
			-- by design number

	all_view_buttons: HASH_TABLE [EV_TOGGLE_BUTTON, STRING]
			-- Table of buttons to select named views, indexed
			-- by associated view name

	in_set_view_action: BOOLEAN
			-- Are we currently in an action to set a named view?

	version_selection_interface: RC_PACKING_VERSIONS_WINDOW
			-- Interface to select packing version

	version_selector_built: BOOLEAN
			-- Has version selection interface been built yet?

	payload_sizes_interface: RC_PACKING_PAYLOAD_SIZES_WINDOW
			-- Interface to set payload and field
			-- sizes for a run with a custom packing target

	payload_sizes_built: BOOLEAN
			-- Has payload sizes interface been built yet?

	scribe_properties_interface: RC_PACKING_SCRIBES_WINDOW
			-- Interface to set scribe properties version

	scribe_properties_built: BOOLEAN
			-- Has scribe properties interface been built yet?

	lot_summary_interface: RC_PACKING_LOT_SUMMARY_WINDOW
			-- Interface to display summary of project/lot
			-- compatibility

	lot_summary_interface_built: BOOLEAN
			-- Has lot summary interface been built yet?

	search_interface: RC_PACKING_SEARCH_WINDOW
			-- Interface to allow searching packing

	search_interface_built: BOOLEAN
			-- Has search interface been built yet?

	help_interface: RC_PACKING_HELP_WINDOW
			-- Interface to provide help during packing

	help_interface_built: BOOLEAN
			-- Has help interface been built yet?

	packer_output_interface: RC_PACKING_OUTPUT_WINDOW
			-- Interface to display output from packer operations

	packer_output_built: BOOLEAN
			-- Has packer output interface been built yet?

	packer_info_interface: RC_PACKING_INFO_WINDOW
			-- Interface to display information about selected
			-- projects

	packer_info_built: BOOLEAN
			-- Has packer information interface been built yet?

	lot_split_colors_interface: RC_PACKING_SPLIT_COLORS_WINDOW
			-- Interface to set colors used for lot splits
			-- in Lot Split view

	lot_assignment_interface: RC_PACKING_LOT_ASSIGN_WINDOW
			-- Interface to assign lot to selected customer
			-- projects that have more than one compatible lot

	filler_properties_interface: RC_PACKING_FILLER_WINDOW
			-- Interface to set properties of new filler

	move_absolute_interface: RC_PACKING_MOVE_ABS_WINDOW
			-- Interface to allow user to specify field and
			-- coordinates for Move Absolute operation

	move_relative_interface: RC_PACKING_MOVE_REL_WINDOW
			-- Interface to allow user to specify relative
			-- amount for Move Relative operation

	project_delete_interface: RC_PACKING_DELETE_WINDOW
			-- Interface to allow user to specify disposition of
			-- projects whose last copy is about to be deleted

	project_management_interface: RC_PACKING_PROJECTS_WINDOW
			-- Interface to manage projects (select, deselect, etc)

	project_manager_built: BOOLEAN
			-- Has project management interface been built yet?

	application_colors_interface: RC_PACKING_APP_COLORS_WINDOW
			-- Interface to set application colors in packer

	application_colors_built: BOOLEAN
			-- Has application color management interface
			-- been built yet?

	cut_pattern_properties_interface: RC_CUT_PATTERN_PROPERTIES_WINDOW
			-- Interface to obtain information about the properties
			-- for a new cut or existing cut pattern

	has_unlocked_or_unassigned_projects: BOOLEAN
			-- Were there any projects in packing that were
			-- unlocked or not assigned to this run as of last
			-- call to `check_for_unlocked_or_unassigned_projects'?

	version_id_field: EV_TEXT_FIELD
			-- Field for entering version name

	new_version_name: STRING
			-- Name of new version to be saved (entered by user)
			-- Void means "cancel this save" and empty string
			-- means optional name was not supplied

	version_name_dialog: EV_DIALOG
			-- Dialog for user to enter optional version name

	drawing_area_event_handler: RETICLE_MAIN_VIEW_EVENT_HANDLER
			-- Handler for events on `drawing_area'

	drawing_area: EV_DRAWING_AREA
			-- Area where graphical representation of packing
			-- is displayed

	model: RETICLE_MODEL
			-- Model of a reticle, which is manipulated during
			-- packing

	view: RETICLE_VIEW
			-- View of a model of a reticle

	controller: RETICLE_MAIN_MODEL_VIEW_CONTROLLER
			-- Controller that manipulates model and updates view

	grid: GRID
			-- Grid to use for model

	palette: RETICLE_VIEW_COLORS
			-- Color palette to use

	split_colors: LIST [PACKING_ETEST_COLORS_MROW]
			-- Colors for lot splits
	
	status_reporter: PACKING_STD_STATUS_REPORTER
			-- Handler for status messages from packing

	status_box: EV_VERTICAL_BOX
			-- Box that displays status information

	coordinates_box: EV_VERTICAL_BOX
			-- Box that displays coordinates of current pointer
			-- position relative to center of all fields

	coordinates_labels_box: EV_HORIZONTAL_BOX
			-- Box that displays coordinates of current pointer
			-- position relative to center of all fields

	coordinates_labels: LIST [EV_LABEL]
			-- Labels that displays coordinate of current pointer
			-- position relative to center of each fields

	coordinates_separator: EV_HORIZONTAL_SEPARATOR
			-- Separator above `coordinates_label'

	status_label: EV_LABEL
			-- Shows current status of reticle packer

	selected_count_label: EV_LABEL
			-- Shows number of items currently selected

	area_label: EV_LABEL
			-- Shows width and height of of currently displayed
			-- area in model units

	named_view_label: EV_LABEL
			-- Shows abbreviation for name of currently selected
			-- Named View

	saved_pointer_style: EV_POINTER_STYLE
			-- Pointer style saved when interface was first built

	Field_spacing_fraction: DOUBLE = 0.15
			-- Spacing between one field of view and the next
			-- is this fraction of the height of one field

	Field_label_fraction: DOUBLE = 0.1
			-- Height of field of view label is this
			-- fraction of the height of the field

	Main_border_width: INTEGER = 10
			-- Width of border for main box holding drawing area

	Pointer_coordinate_scale: INTEGER = 3
			-- Number of digits to display after decimal point
			-- for pointer coordinates

	operation: ANONYMOUS_OPERATION

	Edit_text: STRING = "Edit"

	Fix_text: STRING = "Fix"

	Unfix_text: STRING = "Unfix"

	Lock_text: STRING = "Lock"

	Unlock_text: STRING = "Unlock"

	All_text: STRING = "All"

	None_text: STRING = "None"

	Invert_text: STRING = "Invert"

	Selected_text: STRING = "Selected"

	Unselected_text: STRING = "Unselected"

	Selected_and_unselected_text: STRING = "Selected and Unselected"

	Partially_selected_items_text: STRING = "Partially Selected Objects"

	Outside_fields_items_text: STRING = "External Objects (Outside All Fields)"

	Customer_projects_text: STRING = "Customer Projects"

	Test_vehicles_text: STRING = "Test Vehicles"

	Cust_and_test_vehicles_text: STRING = "Both Customer Projects and Test Vehicles"

	All_scribes_text: STRING = "All Scribes"

	Horizontal_scribes_text: STRING = "Horizontal Scribes"

	Vertical_scribes_text: STRING = "Vertical Scribes"

	Floating_scribes_text: STRING = "Floating Scribes"

	Misaligned_scribes_text: STRING = "Misaligned Scribes"

	Default_width_scribes_text: STRING = "Default Width Scribes"

	Custom_width_scribes_text: STRING = "Custom Width Scribes"

	Non_scribes_text: STRING = "Non-Scribes (Ignores Cuts)"

	Below_minimum_size_items_text: STRING = "Objects Below Minimum Size"

	Small_scribes_and_fillers_text: STRING
		do
			Result := "Small Scribes and Fillers (< " + nu.formatted_number_or_integer (policies.created_small_object_threshold, grid_scale) + ")"
		end

	Overlapping_field_edge_text: STRING = "Objects Overlapping Field Edge"

	Overlapping_object_text: STRING = "Objects Overlapping Another Object"

	Too_close_to_field_edge_text: STRING = "Objects Too Close to Field Edge"

	Too_close_to_object_text: STRING = "Objects Too Close to Another Object"

	Cut_die_too_big_text: STRING = "Projects with Cut Die Size Too Big"

	Not_centered_in_cut_die_text: STRING = "Projects Not Centered in Cut Die"

	Not_isolated_in_cut_die_text: STRING = "Projects Not Isolated in Cut Die"

	Lot_not_assigned_text: STRING = "Lot Not Assigned Projects"

	Unlocked_or_unassigned_text: STRING = "Unlocked or Not Assigned to Run Projects"

	Invalid_isolated_cut_die_text: STRING = "Isolated Sites with Invalid Cut Die"

	Isolated_cut_die_too_big_text: STRING = "Isolated Sites with Cut Die Size Too Big"

	Non_default_cut_die_text: STRING = "Isolated Sites Whose Cut Die has Non-default Size"

	Wrong_lot_text: STRING = "Isolated Sites with Wrong Lot"

	Shared_cut_die_text: STRING = "Isolated Sites that Share Cut Die with Others"

	Extra_cut_text: STRING = "Extra Cuts that Do Not Help Isolate a Site"

	Fillers_text: STRING = "Fillers"

	Vendor_geometries_text: STRING = "Vendor Geometries"

	Cuts_text: STRING = "Cuts"

	Commercial_projects_text: STRING = "Commercial Projects"

	Mep_projects_text: STRING = "MEP Projects"

	Other_projects_text: STRING = "Other Projects"

	In_x_text: STRING = "In X"

	In_y_text: STRING = "In Y"

	In_x_and_y_text: STRING = "In X and Y"

	Orientation_text: STRING = "Orientation"

	To_prevent_duplication_text: STRING = "To Prevent Duplication"

	To_prevent_deletion_text: STRING = "To Prevent Deletion"

	Almost_completely_text: STRING = "Almost Completely (position, orientation, deletion)"

	Completely_text: STRING = "Completely (position, orientation, duplication, deletion)"

	To_allow_duplication_text: STRING = "To Allow Duplication"

	To_allow_deletion_text: STRING = "To Allow Deletion"

	By_fixed_status_text: STRING = "By Fixed Status"

	By_locked_status_text: STRING = "By Locked Status"

	By_account_type_text: STRING = "By Account Type"

	By_design_number_text: STRING = "By Design Number"

	By_design_name_text: STRING = "By Design Name"

	By_account_number_text: STRING = "By Account Number"

	By_lot_split_text: STRING = "By Lot Split"

	By_organization_text: STRING = "By Organization"

	Transparent_text: STRING = "Transparent"

	Opaque_text: STRING = "Opaque"

end
