class RC_POLICIES

inherit
	ANY
	GENERIC_APPLICATION_PARAMETERS
	export {NONE} all end

create
	make	-- (in GENERIC_APPLICATION_PARAMETERS)

feature	-- RC specific policy items

	cats_script_prolog: STRING
		-- opening lines for each die fracture script
	do  Result := as_string("CATS_SCRIPT_PROLOG")
	end

	cats_script_epilog: STRING
		-- closing lines for each die fracture script
	do  Result := as_string("CATS_SCRIPT_EPILOG")
	end

	cats_op_reset: STRING
		-- official reset .cinc file for each step
	do  Result := as_string("CATS_OP_RESET")
	end

	use_cats_expr_optimizer: BOOLEAN
		-- should we apply the optimizer to our CATS_STAEMENTs?
	do  Result := as_boolean("USE_CATS_EXPR_OPTIMIZER")
	end

	rtm_undo_allowed_interval: INTEGER
			-- Number of days after RELEASED TO MANUFACTURING
			-- event was inserted until undo of step that
			-- inserted the event is disallowed
		do
			Result := as_integer ("RTM_UNDO_ALLOWED_INTERVAL")
		end

	post_undo_allowed_interval: INTEGER
			-- Number of days after FAB POSTED
			-- event was inserted until undo of step that
			-- inserted the event is disallowed
		do
			Result := as_integer ("POST_UNDO_ALLOWED_INTERVAL")
		end

	cust_quote_t_and_c_part: INTEGER
			-- Part number for Terms and Conditions pseudo-part
		do
			Result := as_integer ("CUST_QUOTE_T_AND_C_PART")
		end

	trusted_foundry_fab_part: INTEGER
			-- FAB RUN part number used for charging TRF runs
		do
			Result := as_integer ("TRUSTED_FOUNDRY_FAB_PART")
		end

	virtual_packaging_org_no: INTEGER
			-- Org number for organization used for
			-- packaging lot of type VIRTUAL
		do
			Result := as_integer ("VIRTUAL_PACKAGING_ORG_NO")
		end

	reference_extra_tweezer_margin: DOUBLE
			-- Number of microns to add to tweezer margin for
			-- purposes of selecting reference site for probing
		do
			Result := as_double ("REFERENCE_EXTRA_TWEEZER_MARGIN")
		end

feature	-- Packer policy items

	max_inches_per_grid_unit: DOUBLE
			-- Maximum number of inches on screen one grid
			-- unit can be (to keep from zooming in too far)
		do
			Result := as_double ("MAX_INCHES_PER_GRID_UNIT")
		end

	max_model_units_per_inch: DOUBLE
			-- Maximum number of model units one inch
			-- can represent on screen (to keep from zooming out
			-- too far)
		do
			Result := as_double ("MAX_MODEL_UNITS_PER_INCH")
		end

	created_small_object_threshold: DOUBLE
			-- Size in either dimension (length dimension only
			-- for scribes) below which objects created by the
			-- user during packing (scribes and fillers) are
			-- considered small
		do
			Result := as_double ("CREATED_SMALL_OBJECT_THRESHOLD")
		end

	min_zoom_dimension_in_pixels: INTEGER
			-- Minimum width and height of zoom box in pixels
			-- for it to be interpreted as a zoom
		do
			Result := as_integer ("MIN_ZOOM_DIMENSION_IN_PIXELS")
		end

	min_select_dimension_in_pixels: INTEGER
			-- Minimum width or height of selection box
			-- in pixels for it to be interpreted as an
			-- area selection rather than a point
			-- selection
		do
			Result := as_integer ("MIN_SELECT_DIMENSION_IN_PIXELS")
		end

	min_scribe_dimension_in_pixels: INTEGER
			-- Minimum width or height of scribe creation
			-- box in pixels for it to be valid (to
			-- prevent accidental creation of very small scribes)
		do
			Result := as_integer ("MIN_SCRIBE_DIMENSION_IN_PIXELS")
		end

	min_dup_distance_in_pixels: INTEGER
			-- Minimum distance in pixels Duplicate operation
			-- must move from original position for it to be
			-- valid (to prevent accidental duplication)
		do
			Result := as_integer ("MIN_DUP_DISTANCE_IN_PIXELS")
		end

	max_saved_areas: INTEGER
			-- Maximum number of saved areas supported
		do
			Result := as_integer ("MAX_SAVED_AREAS")
		end

	displayed_area_hist_capacity: INTEGER
			-- Maximum number of displayed areas of model
			-- in history list
		do
			Result := as_integer ("DISPLAYED_AREA_HIST_CAPACITY")
		end

	reticle_change_hist_capacity: INTEGER
			-- Maximum number of changes in reticle packer history
			-- list
		do
			Result := as_integer ("RETICLE_CHANGE_HIST_CAPACITY")
		end

	corner_snap_limit_in_inches: DOUBLE
			-- Maximum number of inches from edge to corner
			-- to allow snapping user-selected point for distance
			-- measurement to nearest visible corner
		do
			Result := as_double ("CORNER_SNAP_LIMIT_IN_INCHES")
		end

	measure_marker_size_in_inches: DOUBLE
			-- Size in inches of marker displayed to show a
			-- measurement point graphically
		do
			Result := as_double ("MEASURE_MARKER_SIZE_IN_INCHES")
		end

	measure_marker_line_width: INTEGER
			-- Width of lines used to draw marker displayed
			-- to show a measurement point graphically,
			-- in pixels
		do
			Result := as_integer ("MEASURE_MARKER_LINE_WIDTH")
		end

	tight_packing_fit: INTEGER
			-- Maximum difference between minimum and
			-- maximum die size for die to be considered a
			-- tight fit for packing purposes
		do
			Result := as_integer ("TIGHT_PACKING_FIT")
		end

	cut_handle_width_fraction: DOUBLE
			-- Fraction of maximum field dimension for width
			-- of cut handle box
		do
			Result := as_double ("CUT_HANDLE_WIDTH_FRACTION")
		end

	cut_handle_height_fraction: DOUBLE
			-- Fraction of maximum field dimension for height 
			-- of cut handle box
		do
			Result := as_double ("CUT_HANDLE_HEIGHT_FRACTION")
		end

	cut_extension_fraction: DOUBLE
			-- Fraction of maximum field dimension for distance
			-- cut extends beyond field edge
		do
			Result := as_double ("CUT_EXTENSION_FRACTION")
		end

	cut_separation_fraction: DOUBLE
			-- Fraction of maximum field dimension for distance
			-- between bottom of vertical cut and field label
		do
			Result := as_double ("CUT_SEPARATION_FRACTION")
		end

	standard_background_color: RGB_COLOR
			-- Standard background color for packer
		do
			Result := as_color (standard_background_name)
		end

	standard_background_name: STRING = "STANDARD_BACKGROUND"
			-- Name of role for standard background color

	fully_zoomed_background_color: RGB_COLOR
			-- Background color for packer when fully zoomed in
		do
			Result := as_color (fully_zoomed_background_name)
		end

	fully_zoomed_background_name: STRING = "FULLY_ZOOMED_BACKGROUND"
			-- Name of role for fully zoomed in background color

	unselectable_color: RGB_COLOR
			-- Foreground color to use for items that are not
			-- selectable in packer
		do
			Result := as_color (unselectable_name)
		end

	unselectable_name: STRING = "UNSELECTABLE"
			-- Name of role for unselectable item color

	default_border_color: RGB_COLOR
			-- Default foreground color to use for border
			-- for items that are selectable in packer
		do
			Result := as_color (default_border_name)
		end

	default_border_name: STRING = "DEFAULT_BORDER"
			-- Name of role for default border color

	default_label_color: RGB_COLOR
			-- Default foreground color to use for label
			-- for items that are selectable in packer
		do
			Result := as_color (default_label_name)
		end

	default_label_name: STRING = "DEFAULT_LABEL"
			-- Name of role for default label color

	default_fill_color: RGB_COLOR
			-- Default fill color to use for items
			-- that are selectable and are filled
		do
			Result := as_color (default_fill_name)
		end

	default_fill_name: STRING = "DEFAULT_FILL"
			-- Name of role for default fill color

	not_fixed_in_x_or_y_color: RGB_COLOR
			-- Foreground color to use for selectable
			-- items that are not fixed in X or Y in packer
		do
			Result := as_color (not_fixed_in_x_or_y_name)
		end

	not_fixed_in_x_or_y_name: STRING = "NOT_FIXED_IN_X_OR_Y"
			-- Name of role for items that are not fixed in X or Y

	fixed_in_x_only_color: RGB_COLOR
			-- Foreground color to use for selectable
			-- items that are fixed in X but not fixed in
			-- Y
		do
			Result := as_color (fixed_in_x_only_name)
		end

	fixed_in_x_only_name: STRING = "FIXED_IN_X_ONLY"
			-- Name of role for items that are fixed in X only

	fixed_in_y_only_color: RGB_COLOR
			-- Foreground color to use for selectable
			-- items that are fixed in Y but not fixed in
			-- X
		do
			Result := as_color (fixed_in_y_only_name)
		end

	fixed_in_y_only_name: STRING = "FIXED_IN_Y_ONLY"
			-- Name of role for items that are fixed in Y only

	fixed_in_x_and_y_color: RGB_COLOR
			-- Foreground color to use for selectable
			-- items that are fixed in both X and Y in packer
		do
			Result := as_color (fixed_in_x_and_y_name)
		end

	fixed_in_x_and_y_name: STRING = "FIXED_IN_X_AND_Y"
			-- Name of role for items that are fixed in X and Y

	not_locked_in_x_or_y_color: RGB_COLOR
			-- Foreground color to use for selectable
			-- items that are not locked in X or Y in packer
		do
			Result := as_color (not_locked_in_x_or_y_name)
		end

	not_locked_in_x_or_y_name: STRING = "NOT_LOCKED_IN_X_OR_Y"
			-- Name of role for items that are not locked in X
			-- or Y

	locked_in_x_only_color: RGB_COLOR
			-- Foreground color to use for selectable
			-- items that are locked in X but not locked in
			-- Y
		do
			Result := as_color (locked_in_x_only_name)
		end

	locked_in_x_only_name: STRING = "LOCKED_IN_X_ONLY"
			-- Name of role for items that are locked in X only

	locked_in_y_only_color: RGB_COLOR
			-- Foreground color to use for selectable
			-- items that are locked in Y but not locked in
			-- X
		do
			Result := as_color (locked_in_y_only_name)
		end

	locked_in_y_only_name: STRING = "LOCKED_IN_Y_ONLY"
			-- Name of role for items that are locked in Y only

	locked_in_x_and_y_color: RGB_COLOR
			-- Foreground color to use for selectable
			-- items that are locked in both X and Y in packer
		do
			Result := as_color (locked_in_x_and_y_name)
		end

	locked_in_x_and_y_name: STRING = "LOCKED_IN_X_AND_Y"
			-- Name of role for items that are locked in X and Y

	has_max_die_size_color: RGB_COLOR
			-- Foreground color to use for selectable
			-- items that have a maximum die size and
			-- the maximum is not too close to the min die size
		do
			Result := as_color (has_max_die_size_name)
		end

	has_max_die_size_name: STRING = "HAS_MAX_DIE_SIZE"
			-- Name of role for items that have a maximum die size
			-- and maximum is not too close to min die size

	tight_fit_color: RGB_COLOR
			-- Foreground color to use for selectable
			-- items that have a maximum die size and
			-- the maximum is close to the min die size
		do
			Result := as_color (tight_fit_name)
		end

	tight_fit_name: STRING = "TIGHT_FIT"
			-- Name of role for items that have a maximum die size
			-- and maximum is close to min die size

	no_max_die_size_color: RGB_COLOR
			-- Foreground color to use for selectable
			-- items that do not have a maximum die size
		do
			Result := as_color (no_max_die_size_name)
		end

	no_max_die_size_name: STRING = "NO_MAX_DIE_SIZE"
			-- Name of role for items that do not have a
			-- maximum die size

	no_packaging_color: RGB_COLOR
			-- Foreground color to use for selectable
			-- items that do not have any packaging
		do
			Result := as_color (no_packaging_name)
		end

	no_packaging_name: STRING = "NO_PACKAGING"
			-- Name of role for items that do not have any
			-- packaging

	ceramic_packaging_color: RGB_COLOR
			-- Foreground color to use for selectable
			-- items that have only ceramic packaging
		do
			Result := as_color (ceramic_packaging_name)
		end

	ceramic_packaging_name: STRING = "CERAMIC_PACKAGING"
			-- Name of role for items that have only ceramic
			-- packaging

	plastic_packaging_color: RGB_COLOR
			-- Foreground color to use for selectable
			-- items that have only plastic packaging
		do
			Result := as_color (plastic_packaging_name)
		end

	plastic_packaging_name: STRING = "PLASTIC_PACKAGING"
			-- Name of role for items that have only plastic
			-- packaging

	ceramic_and_plastic_packaging_color: RGB_COLOR
			-- Foreground color to use for selectable
			-- items that have both ceramic and plastic packaging
		do
			Result := as_color (ceramic_and_plastic_packaging_name)
		end

	ceramic_and_plastic_packaging_name: STRING = "CERAMIC_AND_PLASTIC_PACKAGING"
			-- Name of role for items that have only both
			-- ceramic and plastic packaging

	search_match_color: RGB_COLOR
			-- Foreground color to use for items that matched
			-- search criteria for last search
		do
			Result := as_color (search_match_name)
		end

	search_match_name: STRING = "SEARCH_MATCH"
			-- Name of role for items that matched search criteria

	search_no_match_color: RGB_COLOR
			-- Foreground color to use for items that did not
			-- match search criteria for last search
		do
			Result := as_color (search_no_match_name)
		end

	search_no_match_name: STRING = "SEARCH_NO_MATCH"
			-- Name of role for items that did not match
			-- search criteria

	search_match_fill_color: RGB_COLOR
			-- Color to use to fill elements that matched search
			-- criteria for last search
		do
			Result := as_color (search_match_fill_name)
		end

	search_match_fill_name: STRING = "SEARCH_MATCH_FILL"
			-- Name of role for fill color for items that
			-- matched search criteria

	highlighted_selected_color: RGB_COLOR
			-- Foreground color to use for items that are
			-- selected when selected items are highlighted
		do
			Result := as_color (highlighted_selected_name)
		end

	highlighted_selected_name: STRING = "HIGHLIGHTED_SELECTED"
			-- Name of role for color for items to be
			-- highlighted when selected

	highlighted_unselected_color: RGB_COLOR
			-- Foreground color to use for items that are
			-- not selected when selected items are highlighted
		do
			Result := as_color (highlighted_unselected_name)
		end

	highlighted_unselected_name: STRING = "HIGHLIGHTED_UNSELECTED"
			-- Name of role for color for items not to be
			-- highlighted when selected

	highlighted_selected_fill_color: RGB_COLOR
			-- Color to use to fill elements that are selected
			-- when selected items are highlighted
		do
			Result := as_color (highlighted_selected_fill_name)
		end

	highlighted_selected_fill_name: STRING = "HIGHLIGHTED_SELECTED_FILL"
			-- Name of role for color to fill elements that are
			-- selected when selected items are highlighted

	commercial_project_color: RGB_COLOR
			-- Foreground color to use for commercial projects
			-- in packer
		do
			Result := as_color (commercial_project_name)
		end

	commercial_project_name: STRING = "COMMERCIAL_PROJECT"
			-- Name of role for color to use for commercial
			-- projects

	mep_project_color: RGB_COLOR
			-- Foreground color to use for MEP projects
			-- in packer
		do
			Result := as_color ("MEP_PROJECT")
		end

	mep_project_name: STRING = "MEP_PROJECT"
			-- Name of role for color to use for MEP
			-- projects

	other_project_color: RGB_COLOR
			-- Foreground color to use for projects that are
			-- not commercial or MEP
		do
			Result := as_color (other_project_name)
		end

	other_project_name: STRING = "OTHER_PROJECT"
			-- Name of role for color to use for
			-- projects that are not commercial or MEP

	non_project_color: RGB_COLOR
			-- Foreground color to use for elements that are
			-- not projects (e.g., scribes, fillers, etc)
		do
			Result := as_color (non_project_name)
		end

	non_project_name: STRING = "NON_PROJECT"
			-- Name of role for color to use for
			-- non-projects

	external_overhead_color: RGB_COLOR
			-- Foreground color to use for elements that have
			-- external overhead
		do
			Result := as_color (external_overhead_name)
		end

	external_overhead_name: STRING = "EXTERNAL_OVERHEAD"
			-- Name of role for color to use for
			-- elements with external overhead

	internal_overhead_color: RGB_COLOR
			-- Foreground color to use for elements that have
			-- internal overhead
		do
			Result := as_color (internal_overhead_name)
		end

	internal_overhead_name: STRING = "INTERNAL_OVERHEAD"
			-- Name of role for color to use for
			-- elements with internal overhead

	none_overhead_color: RGB_COLOR
			-- Foreground color to use for elements that have
			-- no overhead (but for which overhead makes sense)
		do
			Result := as_color (none_overhead_name)
		end

	none_overhead_name: STRING = "NONE_OVERHEAD"
			-- Name of role for color to use for
			-- elements with no overhead, but for which
			-- overhead makes sense

	no_overhead_color: RGB_COLOR
			-- Foreground color to use for elements that have
			-- no overhead (and for which overhead does not make sense)
		do
			Result := as_color (no_overhead_name)
		end

	no_overhead_name: STRING = "NO_OVERHEAD"
			-- Name of role for color to use for
			-- elements with no overhead, and for which
			-- overhead does not make sense

	cut_color: RGB_COLOR
			-- Foreground color to use for elements that are cuts
		do
			Result := as_color (cut_name)
		end

	cut_name: STRING = "CUT"
			-- Name of role for color to use for cuts

	selected_fill_color: RGB_COLOR
			-- Color to use to fill elements that are selected,
			-- when fill of selected elements is requested
		do
			Result := as_color (selected_fill_name)
		end

	selected_fill_name: STRING = "SELECTED_FILL"
			-- Name of role for color to use to fill elements
			-- that are selected

	unselected_fill_color: RGB_COLOR
			-- Color to use to fill elements that are not selected,
			-- when fill of unselected elements is requested
		do
			Result := as_color (unselected_fill_name)
		end

	unselected_fill_name: STRING = "UNSELECTED_FILL"
			-- Name of role for color to use to fill elements
			-- that are not selected

	locked_in_x_fill_color: RGB_COLOR
			-- Color to use to fill elements that are locked in X,
			-- when fill of these elements is requested
		do
			Result := as_color (locked_in_x_fill_name)
		end

	locked_in_x_fill_name: STRING = "LOCKED_IN_X_FILL"
			-- Name of role for color to use to fill elements
			-- that are locked in X

	locked_in_y_fill_color: RGB_COLOR
			-- Color to use to fill elements that are locked in Y,
			-- when fill of these elements is requested
		do
			Result := as_color (locked_in_y_fill_name)
		end

	locked_in_y_fill_name: STRING = "LOCKED_IN_Y_FILL"
			-- Name of role for color to use to fill elements
			-- that are locked in Y

	locked_in_x_and_y_fill_color: RGB_COLOR
			-- Color to use to fill elements that are
			-- locked in X and Y, when fill of these
			-- elements is requested
		do
			Result := as_color (locked_in_x_and_y_fill_name)
		end

	locked_in_x_and_y_fill_name: STRING = "LOCKED_IN_X_AND_Y_FILL"
			-- Name of role for color to use to fill elements
			-- that are locked in X and Y

	no_max_die_size_fill_color: RGB_COLOR
			-- Color to use to fill elements that have no
			-- maximum die size, when fill of these
			-- elements is requested
		do
			Result := as_color (no_max_die_size_fill_name)
		end

	no_max_die_size_fill_name: STRING = "NO_MAX_DIE_SIZE_FILL"
			-- Name of role for color to use to fill elements
			-- that have no maximum die size

	standard_rubber_box_color: RGB_COLOR
			-- Standard foreground color to use when drawing
			-- rubber boxes
		do
			Result := as_color (standard_rubber_box_name)
		end

	standard_rubber_box_name: STRING = "STANDARD_RUBBER_BOX"
			-- Name of role for color to use when drawing
			-- standard rubber boxes

	too_small_rubber_box_color: RGB_COLOR
			-- Foreground color to use when drawing rubber
			-- boxes that are smaller than allowed for the
			-- operation in progress
		do
			Result := as_color (too_small_rubber_box_name)
		end

	too_small_rubber_box_name: STRING = "TOO_SMALL_RUBBER_BOX"
			-- Name of role for color to use when drawing
			-- rubber boxes that are smaller than allowed

	horizontal_scribe_rubber_box_color: RGB_COLOR
			-- Foreground color to use when drawing rubber
			-- boxes for creation of horizontal scribes
		do
			Result := as_color (horizontal_scribe_rubber_box_name)
		end

	horizontal_scribe_rubber_box_name: STRING = "HORIZONTAL_SCRIBE_RUBBER_BOX"
			-- Name of role for color to use when drawing
			-- rubber boxes for creation of horizontal scribes

	vertical_scribe_rubber_box_color: RGB_COLOR
			-- Foreground color to use when drawing rubber
			-- boxes for creation of vertical scribes
		do
			Result := as_color ("VERTICAL_SCRIBE_RUBBER_BOX")
		end

	vertical_scribe_rubber_box_name: STRING = "VERTICAL_SCRIBE_RUBBER_BOX"
			-- Name of role for color to use when drawing
			-- rubber boxes for creation of vertical scribes

	standard_move_box_color: RGB_COLOR
			-- Foreground color to use when drawing box
			-- that shows outline of bounding box for items
			-- to be moved
		do
			Result := as_color (standard_move_box_name)
		end

	standard_move_box_name: STRING = "STANDARD_MOVE_BOX"
			-- Name of role for color to use when drawing
			-- box for move

	standard_duplicate_box_color: RGB_COLOR
			-- Foreground color to use when drawing box
			-- that shows outline of bounding box for items
			-- to be duplicated
		do
			Result := as_color (standard_duplicate_box_name)
		end

	standard_duplicate_box_name: STRING = "STANDARD_DUPLICATE_BOX"
			-- Name of role for color to use when drawing
			-- box for duplicate

	no_lot_color_assigned_color: RGB_COLOR
			-- Foreground color to use in Lot Split view
			-- for projects whose lot has not yet had a color
			-- assigned by user
		do
			Result := as_color (no_lot_color_assigned_name)
		end

	no_lot_color_assigned_name: STRING = "NO_LOT_COLOR_ASSIGNED"
			-- Name of role for color to use in Lot Split view
			-- for projects whose lot has not yet had a color
			-- box for duplicate

	no_lot_assigned_color: RGB_COLOR
			-- Foreground color to use in Lot Split view
			-- for projects which have not yet been assigned
			-- a lot
		do
			Result := as_color (no_lot_assigned_name)
		end

	no_lot_assigned_name: STRING = "NO_LOT_ASSIGNED"
			-- Name of role for color to use in Lot Split view
			-- for projects which have not yet been assigned a lot

	no_validity_issues_color: RGB_COLOR
			-- Foreground color to use in Validity Check view
			-- for objects which have no validity issues
		do
			Result := as_color (no_validity_issues_name)
		end

	no_validity_issues_name: STRING = "NO_VALIDITY_ISSUES"
			-- Name of role for color to use in Validity Check view
			-- for objects which have no validity issues

	multiple_validity_issues_color: RGB_COLOR
			-- Foreground color to use in Validity Check view
			-- for objects which have multiple validity issues
		do
			Result := as_color (multiple_validity_issues_name)
		end

	multiple_validity_issues_name: STRING = "MULTIPLE_VALIDITY_ISSUES"
			-- Name of role for color to use in Validity Check view
			-- for objects which have multiple validity issues

	unlocked_or_unassigned_color: RGB_COLOR
			-- Foreground color to use in Validity Check view
			-- for customer projects which are not locked against
			-- customer update or are not assigned to run
			-- being closed
		do
			Result := as_color (unlocked_or_unassigned_name)
		end

	unlocked_or_unassigned_name: STRING = "UNLOCKED_OR_UNASSIGNED"
			-- Name of role for color to use in Validity Check view
			-- for customer projects which are not locked against
			-- customer update or are not assigned to run
			-- being closed

	outside_all_fields_color: RGB_COLOR
			-- Foreground color to use in Validity Check view
			-- for objects which are completely outside all
			-- fields of view
		do
			Result := as_color (outside_all_fields_name)
		end

	outside_all_fields_name: STRING = "OUTSIDE_ALL_FIELDS"
			-- Name of role for color to use in Validity Check view
			-- for objects which are completely outside all
			-- fields of view

	overlaps_field_edge_color: RGB_COLOR
			-- Foreground color to use in Validity Check view
			-- for objects which overlap a field of view edge
		do
			Result := as_color (overlaps_field_edge_name)
		end

	overlaps_field_edge_name: STRING = "OVERLAPS_FIELD_EDGE"
			-- Name of role for color to use in Validity Check view
			-- for objects which overlap a field of view edge

	overlaps_another_object_color: RGB_COLOR
			-- Foreground color to use in Validity Check view
			-- for objects which overlap another object
		do
			Result := as_color (overlaps_another_object_name)
		end

	overlaps_another_object_name: STRING = "OVERLAPS_ANOTHER_OBJECT"
			-- Name of role for color to use in Validity Check view
			-- for objects which overlap another object

	below_minimum_size_color: RGB_COLOR
			-- Foreground color to use in Validity Check view
			-- for objects which are below the minimum size
			-- in either dimension
		do
			Result := as_color (below_minimum_size_name)
		end

	below_minimum_size_name: STRING = "BELOW_MINIMUM_SIZE"
			-- Name of role for color to use in Validity Check view
			-- for objects which are below the minimum size
			-- in either dimension

	needs_lot_assignment_color: RGB_COLOR
			-- Foreground color to use in Validity Check view
			-- for projects which do not have a specific
			-- wafer etest ID assigned
		do
			Result := as_color (needs_lot_assignment_name)
		end

	needs_lot_assignment_name: STRING = "NEEDS_LOT_ASSIGNMENT"
			-- Name of role for color to use in Validity Check view
			-- for projects which do not have a specific
			-- wafer etest ID assigned

	floating_scribe_color: RGB_COLOR
			-- Foreground color to use in Validity Check
			-- view for floating scribes (either end does
			-- not abut field of view edge or scribe of
			-- opposite orientation)
		do
			Result := as_color (floating_scribe_name)
		end

	floating_scribe_name: STRING = "FLOATING_SCRIBE"
			-- Name of role for color to use in Validity Check view
			-- for floating scribes (either end does
			-- not abut field of view edge or scribe of
			-- opposite orientation)

	misaligned_scribe_color: RGB_COLOR
			-- Foreground color to use in Validity Check
			-- view for misaligned scribes (not quite lined up)
		do
			Result := as_color (misaligned_scribe_name)
		end

	misaligned_scribe_name: STRING = "MISALIGNED_SCRIBE"
			-- Name of role for color to use in Validity Check view
			-- for misaligned scribes (not quite lined up)

	too_close_to_object_color: RGB_COLOR
			-- Foreground color to use in Validity Check
			-- view for objects which are too close to
			-- another object
		do
			Result := as_color (too_close_to_object_name)
		end

	too_close_to_object_name: STRING = "TOO_CLOSE_TO_OBJECT"
			-- Name of role for color to use in Validity Check
			-- view for objects which are too close to
			-- another object

	too_close_to_edge_color: RGB_COLOR
			-- Foreground color to use in Validity Check
			-- view for objects which are too close to
			-- a field of view edge
		do
			Result := as_color (too_close_to_edge_name)
		end

	too_close_to_edge_name: STRING = "TOO_CLOSE_TO_EDGE"
			-- Name of role for color to use in Validity Check
			-- view for objects which are too close to
			-- a field of view edge

	small_but_legal_color: RGB_COLOR
			-- Foreground color to use in Validity Check
			-- view for objects which are above minimum size
			-- but are small and so might be errors
		do
			Result := as_color (small_but_legal_name)
		end

	small_but_legal_name: STRING = "SMALL_BUT_LEGAL"
			-- Name of role for color to use in Validity Check
			-- view for objects which are above minimum size
			-- but are small and so might be errors

	cut_die_too_big_color: RGB_COLOR
			-- Foreground color to use in Validity Check
			-- view for projects whose cut die size exceeds
			-- their maximum die size
		do
			Result := as_color (cut_die_too_big_name)
		end

	cut_die_too_big_name: STRING = "CUT_DIE_TOO_BIG"
			-- Name of role for color to use in Validity Check
			-- view for projects whose cut die size exceeds
			-- their maximum die size

	not_isolated_in_cut_die_color: RGB_COLOR
			-- Foreground color to use in Validity Check
			-- view for projects which are not isolated in
			-- their cut die
		do
			Result := as_color (not_isolated_in_cut_die_name)
		end

	not_isolated_in_cut_die_name: STRING = "NOT_ISOLATED_IN_CUT_DIE"
			-- Name of role for color to use in Validity Check
			-- view for projects which are not isolated in
			-- their cut die

	not_centered_in_cut_die_color: RGB_COLOR
			-- Foreground color to use in Validity Check
			-- view for projects which are not centered in
			-- their cut die
		do
			Result := as_color (not_centered_in_cut_die_name)
		end

	not_centered_in_cut_die_name: STRING = "NOT_CENTERED_IN_CUT_DIE"
			-- Name of role for color to use in Validity Check
			-- view for projects which are not centered in
			-- their cut die

	measurement_marker_color: RGB_COLOR
			-- Foreground color to use for marking locations
			-- when measuring distances
		do
			Result := as_color (measurement_marker_name)
		end

	measurement_marker_name: STRING = "MEASUREMENT_MARKER"
			-- Name of role for color to use for marking locations
			-- when measuring distances

	isolated_in_curr_pattern_fill_color: RGB_COLOR
			-- Color to use to fill elements that are isolated
			-- in current cut pattern, when fill of these
			-- elements is requested
		do
			Result := as_color (isolated_in_curr_pattern_fill_name)
		end

	isolated_in_curr_pattern_fill_name: STRING = "ISOLATED_IN_CURR_PATTERN_FILL"
			-- Name of role for color to use to fill elements
			-- that are isolated in current cut pattern

	isolated_in_other_pattern_fill_color: RGB_COLOR
			-- Color to use to fill elements that are
			-- isolated in another cut pattern (not the
			-- current one), when fill of these elements
			-- is requested
		do
			Result := as_color (isolated_in_other_pattern_fill_name)
		end

	isolated_in_other_pattern_fill_name: STRING = "ISOLATED_IN_OTHER_PATTERN_FILL"
			-- Name of role for color to use to fill elements
			-- that are isolated in another cut pattern

	invalid_isolated_cut_die_color: RGB_COLOR
			-- Foreground color to use in Validity Check
			-- view for projects that are isolated in a cut
			-- pattern but are not surrounded by cuts or have
			-- a cut through the printed die
		do
			Result := as_color (invalid_isolated_cut_die_name)
		end

	invalid_isolated_cut_die_name: STRING = "INVALID_ISOLATED_CUT_DIE"
			-- Name of role for color to use in Validity Check
			-- view for projects that are isolated in a cut
			-- pattern but are not surrounded by cuts or have
			-- a cut through the printed die

	wrong_lot_color: RGB_COLOR
			-- Foreground color to use in Validity Check view
			-- for projects that are isolated in a cut pattern
			-- but are on a different lot than the one for the
			-- cut pattern
		do
			Result := as_color (wrong_lot_name)
		end

	wrong_lot_name: STRING = "WRONG_LOT"
			-- Name of role for color to use in Validity Check view
			-- for projects that are isolated in a cut pattern
			-- but are on a different lot than the one for the
			-- cut pattern

	non_default_cut_die_size_color: RGB_COLOR
			-- Foreground color to use in Validity Check view
			-- for projects whose cut die determined by cuts
			-- is different from the default cut die determined
			-- by surrounding scribes
		do
			Result := as_color (non_default_cut_die_size_name)
		end

	non_default_cut_die_size_name: STRING = "NON_DEFAULT_CUT_DIE_SIZE"
			-- Name of role for color to use in Validity Check view
			-- for projects whose cut die determined by cuts
			-- is different from the default cut die determined
			-- by surrounding scribes

	shared_cut_die_color: RGB_COLOR
			-- Foreground color to use in Validity Check view
			-- for projects whose cut die determined by cuts
			-- is shared by other non-scribe dies
		do
			Result := as_color (shared_cut_die_name)
		end

	shared_cut_die_name: STRING = "SHARED_CUT_DIE"
			-- Name of role for color to use in Validity Check view
			-- for projects whose cut die determined by cuts
			-- is shared by other non-scribe dies

	extra_cut_color: RGB_COLOR
			-- Foreground color to use in Validity Check
			-- view for extra cuts that are not an isolating cut
			-- for any isolated sites
		do
			Result := as_color (extra_cut_name)
		end

	extra_cut_name: STRING = "EXTRA_CUT"
			-- Name of role for color to use in Validity Check
			-- view for extra cuts that are not an isolating cut
			-- for any isolated sites

end
