
indexing
	
	description: "A set of preferences for run closing.  These %
		%preferences affect the way run closing works and can %
		%be changed by the user running Delta."
	author: "David Hollenberg"
	date: "February 25, 2004"

class RC_PREFERENCE_SET
inherit
	ANY
		redefine
			default_create
		end

feature {NONE} -- Creation

	default_create
		do
			set_defaults
		end

feature -- Properties

	is_autosubmit_enabled: BOOLEAN
			-- When starting a step that submits jobs,
			-- should we automatically select all rows that
			-- are submittable and then monitor job progress?

	is_cache_auto_flush_enabled: BOOLEAN
			-- Should run cache be flushed automatically
			-- when there has been no step activity (do, undo,
			-- redo) for an application-determined period of time?

	is_unattended_mode_enabled: BOOLEAN
			-- Should program assume that there is no one
			-- watching its progress and modify its behavior
			-- accordingly?  An unattended program won't
			-- warn about unresolved issues via a warning
			-- dialog (but it will still send e-mail about
			-- such issues)

	is_unattended_mode_disabled: BOOLEAN
			-- Should program assume there is an operator
			-- watching its progress and modify its behavior
			-- accordingly?
		do
			Result := not is_unattended_mode_enabled
		end

feature -- Modification

	set_defaults
			-- Set default preferences
		do
			enable_autosubmit
			enable_cache_auto_flush
			disable_unattended_mode
		ensure
			autosubmit_enabled: is_autosubmit_enabled
			unattended_mode_disabled: is_unattended_mode_disabled
		end

	enable_autosubmit
			-- Set `is_autosubmit_enabled' to True
		do
			is_autosubmit_enabled := True
		ensure
			autosubmit_enabled: is_autosubmit_enabled
		end

	disable_autosubmit
			-- Set `is_autosubmit_enabled' to False
		do
			is_autosubmit_enabled := False
		ensure
			autosubmit_disabled: not is_autosubmit_enabled
		end

	enable_cache_auto_flush
			-- Set `is_cache_auto_flush_enabled' to True
		do
			is_cache_auto_flush_enabled := True
		ensure
			cache_auto_flush_enabled: is_cache_auto_flush_enabled
		end

	disable_cache_auto_flush
			-- Set `is_cache_auto_flush_enabled' to False
		do
			is_cache_auto_flush_enabled := False
		ensure
			cache_auto_flush_disabled: not is_cache_auto_flush_enabled
		end

	enable_unattended_mode
			-- Set `is_unattended_mode_enabled' to True
		do
			is_unattended_mode_enabled := True
		ensure
			unattended_mode_enabled: is_unattended_mode_enabled
		end

	disable_unattended_mode
			-- Set `is_unattended_mode_enabled' to False
		do
			is_unattended_mode_enabled := False
		ensure
			unattended_mode_disabled: is_unattended_mode_disabled
		end

end
