
indexing
	
	description: "An operation that executes a primitive run closing step"
	author: "David Hollenberg"
	date: "April 23, 2003"

deferred class RC_PRIMITIVE_STEP_OP
inherit
	RC_STEP_OP
		rename
			do_step as restore_and_do_step
		end
	STRING_UTILITIES_ACCESS
		export
			{NONE} all
		undefine
			default_create
		end
	DB_OPTIONS_ACCESS
		export
			{NONE} all
		undefine
			default_create
		end
	REPORT_EXECUTION
		export
			{NONE} all
		undefine
			default_create
		end

feature {NONE} -- Creation

	make (s: like step sess: RC_RUN_CLOSING_SESSION)
			-- Create operation to execute step `s' in context
			-- of `session'
		require
			step_exists: s /= Void
			sess_exists: sess /= Void
		do
			rc_step_make (sess)
			step := s
		end

feature -- Properties

	step: RC_PRIMITIVE_STEP
			-- Step for which this is an implementation

	steps: LIST [RC_PRIMITIVE_STEP]
			-- List of all steps that were selected for
			-- current "do" or "undo" operation

	input_data_items: LIST [STRING]
			-- Data items step requires before step can
			-- be executed
		deferred
		ensure
			result_exists: Result /= Void
		end

	is_applicable_to_run: BOOLEAN
			-- Is `step' applicable to the run for `session'?
		do
			if not is_step_applicable_computed then
				is_step_applicable := is_applicable
				is_step_applicable_computed := True
			end
			Result := is_step_applicable
		end

	has_user_interface: BOOLEAN
			-- Does `Current' have its own user interface?
			-- If not, default progress interface is used
		deferred
		end

	creates_external_state: BOOLEAN
			-- Does `Current' create state outside the database?
		deferred
		end

	state: ANY
			-- State saved by soft undo

	is_currently_undoable: BOOLEAN
			-- Is `Current' currently in a state where it can be
			-- undone?
		do
			Result := True
		end

	not_undoable_reason: STRING
			-- Reason why `Current' cannot be undone now
		do
			Result := ""
		ensure
			result_not_void: Result /= Void
		end

feature -- Modification

	set_steps (list: like steps)
		require
			list_not_void: list /= Void
		do
			steps := list
		ensure
			steps_set: steps = list
		end

feature -- Operations

	do_inapplicable_step
			-- Take actions needed to do step that is
			-- not applicable.  May be redefined in descendant
		do
		end

	undo_inapplicable_step
			-- Take actions needed to undo step that is
			-- not applicable.  May be redefined in descendant
		do
		end

feature {NONE} -- Implementation
	
	is_step_applicable: BOOLEAN
			-- Is `step' applicable to run for session?
			-- Saved here so it won't have to be recomputed

	is_step_applicable_computed: BOOLEAN
			-- Has `is_step_applicable' been computed yet for
			-- this step?

	is_applicable: BOOLEAN
			-- Is step applicable to run for session?
		deferred
		end

	data_item: DATA_ITEM_NAME_VALUES
			-- Access to data item names

	array_as_list (a: ARRAY [STRING]): LIST [STRING]
			-- `a' as a list
		do
			create {ARRAYED_LIST [STRING]} Result.make (a.count)
			Result.fill (a)
		end

	restore_and_do_step
			-- Restore state if available and execute step
		do
			restore_state
			do_step
		end

	undo_and_discard_state
			-- Undo execution of step.  Do not save state
			-- information
		do
			undo_step
		end

	undo_and_save_state
			-- Undo execution of step and save state information
			-- so that step might be redone with less work
		do
			save_state
			undo_step
		end

	do_step
			-- Execute step
		deferred
		end

	undo_step
			-- Undo execution of step
		deferred
		end

	save_state
			-- Save state before undoing execution of step
		local
			file: RC_FILE
			f: LARGE_RAW_FILE
		do
			set_state
			if state /= Void then
				file := session.next_state_file (step)
				create f.make (file.file_pathname)
				session.open_file_for_write (f, file)
				f.independent_store (state)
				session.close_file (f)
			end
		end

	restore_state
			-- Restore state, if any, before doing step
		local
			f: LARGE_RAW_FILE
			fname, item_type: STRING
			run_step: RUN_PRIMITIVE_STEPS_ROW
			retriever: FILE_RETRIEVE_OP
			item: ANY
		do
			state := Void
			run_step := step.run_step
			if run_step /= Void then
				fname := step.run_step.saved_state_filename
			end
			if fname /= Void and then not fname.is_empty then
				create f.make (fname)
				session.open_file_for_read (f)
				create retriever
				retriever.retrieve (f)
				item := retriever.item
				state ?= item
				session.close_file (f)
				if state = Void then
					if item /= Void then
						item_type := retriever.item.generating_type
					else
						item_type := "Void"
					end
					warn_with_notify ("Object structure with saved state retrieved successfully from file%N" + fname + "%Nbut object has wrong type " + item_type)
				end
			end
		end

	set_state
			-- Set `state'.  Leave `state' Void if no state
		require
			no_state: state = Void
		deferred
		end

	call_safely_with_string (s: STRING action: PROCEDURE [like Current, TUPLE [STRING]] abort_action: PROCEDURE [ANY, TUPLE])
			-- Call `action' with aborts caught and with
			-- `abort_actions' replaced by `abort_action'
		do
			set_implementation (agent action.call ([s]))
			enable_abort_caught
			abort_actions.wipe_out
			abort_actions.extend (abort_action)
			execute
		end

	reset (ifc: ABSTRACT_INTERFACE)
			-- Reset `ifc' and rollback session
		do
			session.rollback
			ifc.reset
		end

	run_one_report (module, parms, dest: STRING)
			-- Run ReportWriter report `module' with `parms'
			-- and put output in `dest'
		require
			module_exists: module /= Void
			parms_exists: parms /= Void
			dest_exists: dest /= Void
		do
			run_report (connection, environment.reportwriter_program, environment.oracle_reports_home, environment.temp_dir, module, parms, dest, environment.is_production, Void)
		end

invariant
	step_exists: step /= Void
end
