
indexing

	description: "An operation that executes a list of primitive run %
		%closing steps, in order"
	author: "David Hollenberg"
	date: "April 23, 2003"

class RC_PRIMITIVE_STEPS_OP
inherit
	RC_STEP_OP
	GUI_UTILITIES_ACCESS
		export
			{NONE} all
		undefine
			default_create
		end

create
	make

feature {NONE} -- Creation

	make (list: like steps sess: RC_RUN_CLOSING_SESSION)
			-- Create operation to execute `list' in context
			-- of `sess'
		require
			list_exists: list /= Void
			list_nonempty: not list.is_empty
			sess_exists: sess /= Void
		do
			rc_step_make (sess)
			steps := list
		end

feature -- Properties

	steps: LIST [RC_PRIMITIVE_STEP]
			-- List of steps for this operation

	selected_step: RC_STEP
			-- Primitive or composite step user selected

feature -- Modification

	set_selected_step (s: RC_STEP)
			-- Set `selected_step' to `s'
		require
			step_exists: s /= Void
		do
			selected_step := s
		ensure
			selected_step_set: selected_step = s
		end

	set_modify
		do
			set_implementation (agent modify_step)
		end

	set_redo
		do
			set_implementation (agent redo_steps)
		end

feature -- Operations

	do_step
			-- Execute `steps'
		do
			check_partially_undone_steps
				-- Issue error if any partially undone steps
			gui.do_once_on_idle (agent switch_windows)
			session.clear_abort_requested
			queue_up_do_operations (step_do_operations (steps))
		end

	modify_step
			-- Undo implicitly selected steps and save state,
			-- then undo "done" indication for the single
			-- explicit step (but don't undo step itself),
			-- so that step can be modified
		do
			is_redo := False
			undo_steps_and_modify (True)
		end

	redo_steps
			-- Undo `steps' and save state, then do `steps'
			-- in reverse order
		do
			is_redo := True
			check_partially_undone_steps
			confirm ("Are you sure you want to redo step %"" + selected_step.presentation_text + "%"")
			check_for_close_run
			undo_steps (True)
		end

	undo_and_discard_state
			-- Undo execution of step.  Do not save state
			-- information
		do
			is_redo := False
			confirm ("A hard undo does NOT save state information to make it easy to do step again.%N%NAre you sure you want to undo step %"" + selected_step.presentation_text + "%"")
			check_for_close_run
			undo_steps (False)
		end

	undo_and_save_state
			-- Undo execution of step and save state information
			-- so that step might be redone with less work
		do
			is_redo := False
			confirm ("Are you sure you want to undo step %"" + selected_step.presentation_text + "%"")
			check_for_close_run
			undo_steps (True)
		end

feature {NONE} -- Implementation

	check_for_close_run
			-- Issue stronger warning if user asked to
			-- undo step "Close run" or if steps to be
			-- undone include "Complete run assembly" step
		local
			step: STEP_NAME_VALUES
			prim_step: PRIMITIVE_STEP_NAME_VALUES
		do
			if selected_step.step_name.is_equal (step.Close_run) then
				confirm ("Undoing step %"" + selected_step.presentation_text + "%" will COMPLETELY remove all run steps and their output%Nfrom the database and the file system (but will not delete the run or the run directory).%N%NAre you ABSOLUTELY sure you want to do this")
			end
			steps.do_if (agent warn_about_completed_assembly, agent {RC_PRIMITIVE_STEP}.has_step_name (prim_step.Complete_run_assembly))
		end

	warn_about_completed_assembly (s: RC_PRIMITIVE_STEP)
			-- Issue warning if user asked to
			-- undo step "Complete run assembly"
		require
			s_not_void: s /= Void
		do
			confirm ("The set of steps to be undone includes step %"" + s.presentation_text + "%".%NThis step was completed on " + s.run_step.date_completed.to_date_string + "%Nand should not be undone if any masks have already been made.%N%NAre you ABSOLUTELY sure you want to do this")
		end

	check_partially_undone_steps
			-- Check whether any of the steps to be done
			-- is partially undone.  If so, issue error
		local
			partial_steps: LIST [RC_PRIMITIVE_STEP]
		do
			partial_steps := partially_undone_steps
			if not partial_steps.is_empty then
				handle_user_error ("The set of primitive steps selected includes the following steps that have been%Npartially undone.  You must finish undoing these steps before they can be done (or redone):%N%N" + text_for_steps (partial_steps))
			end
		end

	step_do_operations (list: LIST [RC_PRIMITIVE_STEP]):  LIST [RC_PRIMITIVE_STEP_OP]
			-- List of operations to do steps in `list'
		local
			step: RC_PRIMITIVE_STEP
			op: RC_PRIMITIVE_STEP_OP
		do
			create {ARRAYED_LIST [RC_PRIMITIVE_STEP_OP]} Result.make (list.count)
			from
				list.start
			until
				list.after
			loop
				step := list.item
				op := session.primitive_step_implementation (step)
				op.set_steps (steps)
				op.set_do
				op.enable_abort_caught
				op.abort_actions.extend (agent session.abort_step (step))
				op.abort_actions.append (abort_actions)
				Result.extend (op)
				list.forth
			end
		end

	step_do_operation (step: RC_PRIMITIVE_STEP):  LIST [RC_PRIMITIVE_STEP_OP]
			-- List of operations to do `step'
		local
			list: ARRAYED_LIST [RC_PRIMITIVE_STEP]
		do
			create list.make (1)
			list.extend (step)
			Result := step_do_operations (list)
		end

	reversed (list:  LIST [RC_PRIMITIVE_STEP_OP]): like list
			-- `list' in reverse order
		do
			create {ARRAYED_LIST [RC_PRIMITIVE_STEP_OP]} Result.make (list.count)
			from
				list.finish
			until
				list.before
			loop
				Result.extend (list.item)
				list.back
			end
		end

	queue_up_do_operations (ops: LIST [RC_PRIMITIVE_STEP_OP])
			-- Queue the "do" operations `ops' for
			-- execution
		local
			curr_op, next_op: RC_PRIMITIVE_STEP_OP
		do
			from
				ops.start
				curr_op := ops.item
				ops.forth
			until
				ops.after
			loop
				next_op := ops.item
				curr_op.completion_actions.extend (agent gui.do_once_on_idle (agent do_next_step_safely (curr_op, next_op)))
				curr_op := next_op
				ops.forth
			end
			curr_op.completion_actions.extend (agent gui.do_once_on_idle (agent do_next_step_safely (curr_op, Void)))
			gui.do_once_on_idle (agent do_next_step_safely (Void, ops.first))
		end

	undo_steps (save: BOOLEAN)
			-- Undo execution of `steps'.  Save state if `save'
			-- is true
		local
			implicit_steps: LIST [RC_PRIMITIVE_STEP]
		do
			implicit_steps := implicitly_selected_steps
			if not implicit_steps.is_empty then
				confirm ("In addition to the step(s) you selected explicitly, the following primitive steps%Nmust also be undone:%N%N" +
				text_for_steps (implicit_steps) +
				"%NProceed with operation anyway")
			end
			gui.do_once_on_idle (agent switch_windows)
			gui.do_once_on_idle (agent show_undo_progress)
			session.clear_abort_requested
			queue_up_undo_operations (step_undo_operations (steps, save), save, Void)
		end

	undo_steps_and_modify (save: BOOLEAN)
			-- Undo execution of implicitly selected steps in
			-- `steps', saving state.  Then remove the "done"
			-- indicator for the single explicitly selected
			-- step, which must be last in the list, so that
			-- user can modify it
			-- If `save' is true, then use a soft undo.
			-- Otherwise, use a hard undo
		local
			implicit_steps: LIST [RC_PRIMITIVE_STEP]
			mod_step: RC_PRIMITIVE_STEP
		do
			check
				last_step_explicit: not steps.is_empty and then steps.last.is_explicit
			end
			mod_step := steps.last
			implicit_steps := implicitly_selected_steps
			if not implicit_steps.is_empty then
				confirm ("The following primitive steps must be undone first:%N%N" +
				text_for_steps (implicit_steps) +
				"%NProceed with operation anyway")
				gui.do_once_on_idle (agent switch_windows)
				gui.do_once_on_idle (agent show_undo_progress)
				session.clear_abort_requested
				queue_up_undo_operations (step_undo_operations (implicit_steps, save), save, mod_step)
			else
				session.reenter_completed_step (mod_step)
				gui.do_once_on_idle (agent switch_windows)
				session.clear_abort_requested
				queue_up_do_operations (step_do_operation (mod_step))
			end
		end

	step_undo_operations (list: LIST [RC_PRIMITIVE_STEP] save: BOOLEAN): LIST [RC_PRIMITIVE_STEP_OP]
			-- List of operations to undo steps in `list'.
			-- If `save' is true, then this is a soft undo.
			-- Otherwise, it is a hard undo
		local
			step: RC_PRIMITIVE_STEP
			op: RC_PRIMITIVE_STEP_OP
		do
			create {ARRAYED_LIST [RC_PRIMITIVE_STEP_OP]} Result.make (steps.count)
			from
				list.start
			until
				list.after
			loop
				step := list.item
				op := session.primitive_step_implementation (step)
				op.set_steps (steps)
				op.enable_abort_caught
				if save then
					op.set_undo_and_save_state
					op.abort_actions.extend (agent session.abort_soft_undo_step (step))
				else
					op.set_undo_and_discard_state
					op.abort_actions.extend (agent session.abort_hard_undo_step (step))
				end
				op.abort_actions.append (abort_actions)
				Result.extend (op)
				list.forth
			end
		end

	queue_up_undo_operations (ops: LIST [RC_PRIMITIVE_STEP_OP] save: BOOLEAN mod_step: RC_PRIMITIVE_STEP)
			-- Queue the "undo" operations `ops' for
			-- execution.  Save state if `save' is true.
			-- If `mod_step' is not Void, re-open it after
			-- last undo completes
		local
			curr_op, next_op: RC_PRIMITIVE_STEP_OP
		do
			step_to_modify := mod_step
			from
				ops.start
				curr_op := ops.item
				ops.forth
			until
				ops.after
			loop
				next_op := ops.item
				curr_op.completion_actions.extend (agent gui.do_once_on_idle (agent undo_next_step_safely (curr_op, next_op, save)))
				curr_op := next_op
				ops.forth
			end
			curr_op.completion_actions.extend (agent gui.do_once_on_idle (agent undo_next_step_safely (curr_op, Void, save)))
			gui.do_once_on_idle (agent undo_next_step_safely (Void, ops.first, save))
		end

	do_next_step_safely (prev_op, op: RC_PRIMITIVE_STEP_OP)
			-- Do next step, whose operation is `op'.
			-- `prev_op' is the operation for the
			-- previous step.  Handle aborts correctly
		do
			set_implementation (agent do_next_step (prev_op, op))
			execute
		end

	do_next_step (prev_op, op: RC_PRIMITIVE_STEP_OP)
			-- Do next step, whose operation is `op'.
			-- `prev_op' is the operation for the
			-- previous step.  Both may be Void
		local
			doing_text: STRING
		do
			if prev_op /= Void then
				-- Finish off previous step
				abort_actions.put_front (agent session.abort_step (prev_op.step))
				if prev_op.is_applicable_to_run then
					session.finish_applicable_step (prev_op)
				else
					session.finish_inapplicable_step (prev_op)
				end
				abort_actions.start
				abort_actions.remove
			end
			application.raise_auxiliary_window
			if op /= Void then
				-- Start next step
				abort_actions.put_front (agent session.abort_step (op.step))
				if not op.has_user_interface or not op.is_applicable_to_run then
					process_pending_abort
					show_do_progress
					if op.is_applicable_to_run then
						doing_text := "Doing step: "
					else
						doing_text := "Doing inapplicable step: "
					end
					progress_ifc.set_label (doing_text + op.step.presentation_text)
					process_pending_abort
				end
				if op.is_applicable_to_run then
					session.start_applicable_step (op)
					gui.do_once_on_idle (agent op.execute)
				else
					session.start_inapplicable_step (op)
					gui.do_once_on_idle (agent do_inapplicable_step (op))
				end
				abort_actions.start
				abort_actions.remove
			else
				-- No next step
				call_completion_actions
			end
		end

	undo_next_step_safely (prev_op, op: RC_PRIMITIVE_STEP_OP save: BOOLEAN)
			-- Undo next step, whose operation is `op'.
			-- `prev_op' is the operation for the
			-- previous step.  Handle aborts correctly
		do
			set_implementation (agent undo_next_step (prev_op, op, save))
			execute
		end

	undo_next_step (prev_op, op: RC_PRIMITIVE_STEP_OP save: BOOLEAN)
			-- Undo next step, whose operation is `op'.
			-- `prev_op' is the operation for the
			-- previous step.  Both may be Void
			-- If `save', then this is a soft undo that saves
			-- state
		local
			applicable: BOOLEAN
		do
			if prev_op /= Void then
				-- Finish undo of previous step
				applicable := prev_op.step.was_applicable_to_run
				if save then
					abort_actions.put_front (agent session.abort_soft_undo_step (prev_op.step))
					session.finish_soft_undo_step (prev_op, applicable)
				else
					abort_actions.put_front (agent session.abort_hard_undo_step (prev_op.step))
					session.finish_hard_undo_step (prev_op, applicable)
				end
				abort_actions.start
				abort_actions.remove
			end
			application.raise_auxiliary_window
			if op /= Void then
				-- Start undo of next step
				process_pending_abort
				progress_ifc.set_label ("Undoing step: " + op.step.presentation_text)
				process_pending_abort
				applicable := op.step.was_applicable_to_run
				if save then
					abort_actions.put_front (agent session.abort_soft_undo_step (op.step))
					session.start_soft_undo_step (op, applicable)
				else
					abort_actions.put_front (agent session.abort_hard_undo_step (op.step))
					session.start_hard_undo_step (op, applicable)
				end
				abort_actions.start
				abort_actions.remove
				if not op.step.is_undone then
					if applicable then
						gui.do_once_on_idle (agent op.execute)
					else
						gui.do_once_on_idle (agent undo_inapplicable_step (op))
					end
				elseif op.step.is_soft_undone then
					gui.do_once_on_idle (agent op.call_completion_actions)
				end
			elseif is_redo then
				queue_up_do_operations (reversed (step_do_operations (steps)))
			else
				-- No next step
				if step_to_modify /= Void then
					session.reenter_completed_step (step_to_modify)
					queue_up_do_operations (step_do_operation (step_to_modify))
				else
					call_completion_actions
				end
			end
		end

	do_inapplicable_step (op: RC_PRIMITIVE_STEP_OP)
			-- Do inapplicable step with operation `op'
		require
			operation_exists: op /= Void
		do
			op.do_inapplicable_step
			op.call_completion_actions
		end

	undo_inapplicable_step (op: RC_PRIMITIVE_STEP_OP)
			-- Undo inapplicable step with operation `op'
		require
			operation_exists: op /= Void
		do
			op.undo_inapplicable_step
			op.call_completion_actions
		end

	text_for_steps (step_list: LIST [RC_STEP]): STRING
			-- Text for `step_list' that is suitable
			-- for inclusion in an error or confirmation dialog
		require
			list_exists: step_list /= Void
		local
			s: RC_STEP
		do
			create Result.make (0)
			from
				step_list.start
			until
				step_list.after
			loop
				s := step_list.item
				Result.append ("    ")
				Result.append (s.presentation_text)
				Result.extend ('%N')
				step_list.forth
			end
		ensure
			result_exists: Result /= Void
		end

	partially_undone_steps: LIST [RC_PRIMITIVE_STEP]
		local
			s: RC_PRIMITIVE_STEP
		do
			create {ARRAYED_LIST [RC_PRIMITIVE_STEP]} Result.make (steps.count)
			from
				steps.start
			until
				steps.after
			loop
				s := steps.item
				if s.is_partially_undone then
					Result.extend (s)
				end
				steps.forth
			end
		ensure
			result_exists: Result /= Void
		end

	implicitly_selected_steps: LIST [RC_PRIMITIVE_STEP]
		local
			s: RC_PRIMITIVE_STEP
		do
			create {ARRAYED_LIST [RC_PRIMITIVE_STEP]} Result.make (steps.count)
			from
				steps.start
			until
				steps.after
			loop
				s := steps.item
				if s.is_implicit then
					Result.extend (s)
				end
				steps.forth
			end
		ensure
			result_exists: Result /= Void
		end

	switch_windows
			-- Switch to auxiliary window
		do
			application.switch_to_auxiliary_window
			application.show_current_window
		end

	show_do_progress
			-- Show progress of do operation, with abort button
		do
			progress_ifc := factory.progress_interface (session.run_id, session.title_with_run_id ("Do Progress"))
			progress_ifc.build
			progress_ifc.decline_actions.extend (agent handle_user_abort)
		end

	show_undo_progress
			-- Show progress of undo operation, with abort button
		do
			progress_ifc := factory.progress_interface (session.run_id, session.title_with_run_id ("Undo Progress"))
			progress_ifc.build
			progress_ifc.decline_actions.extend (agent handle_user_abort)
		end

	handle_user_abort
			-- Handle user abort request
		do
			session.set_abort_requested
		end

	is_redo: BOOLEAN
			-- Is this a redo, rather than just an undo?

	step_to_modify: RC_PRIMITIVE_STEP
			-- Step which is to be re-opened, without undoing it

	progress_ifc: PROGRESS_IFC

	abort_handled: BOOLEAN
			-- Has abort already been handled?

invariant
	steps_exists: steps /= Void
end
