
indexing
	
	description: "Manager for run closing sessions"
	author: "David Hollenberg"
	date: "September 17, 2002"

class RC_SESSION_MANAGER
create
	make

feature {NONE} -- Creation

	make
		do
			create sessions.make (3)
			sessions.compare_references
		end

feature -- Properties

	current_session: RC_SESSION
			-- Currently active session or Void if none

	is_active (s: RC_SESSION): BOOLEAN
			-- Is `s' still an active session (not terminated)?
		do
			Result := sessions.has (s)
		end

feature -- Allocation

	new_session: RC_SESSION
			-- Newly created session
		do
			create {RC_TYPELESS_SESSION} Result.make
			sessions.extend (Result)
		ensure
			result_exists: Result /= Void
		end

feature -- Modification

	set_current_session (s: RC_SESSION)
			-- Set `current_session' to `s'
		require
			session_exists: s /= Void
			session_active: is_active (s)
		do
			current_session := s
		ensure
			current_session_set: current_session = s
		end

	add_session (s: RC_SESSION)
			-- Add `s' to list of active sessions
		require
			session_exists: s /= Void
		do
			sessions.extend (s)
		ensure
			session_active: is_active (s)
		end

	terminate_session (s: RC_SESSION)
			-- Terminate `s'
		require
			session_exists: s /= Void
			session_active: is_active (s)
		do
			sessions.prune (s)
			if current_session = s then
				if not sessions.is_empty then
					current_session := sessions.last
				else
					current_session := Void
				end
			end
		ensure
			session_terminated: not is_active (s)
		end

feature {NONE} -- Implementation

	sessions: ARRAYED_LIST [RC_SESSION]
			-- All run closing sessions

invariant
	sessions_exists: sessions /= Void
	curr_session_active: current_session /= Void implies is_active (current_session)
end
