
indexing
	
	description: "Window to show status of run"
	author: "David Hollenberg"
	date: "May 27, 2003"

class RC_STATUS_WINDOW

inherit
	RC_STATUS_IFC
		redefine
			set_primitive_steps
		end
	WINDOW_INTERFACE
	MOSIS_GRAPHICAL_UTILITIES
		export
			{NONE} all
		undefine
			default_create
		end
	NUMBER_UTILITIES_ACCESS
		export
			{NONE} all
		undefine
			default_create
		end
		
create
	make

feature {NONE} -- Creation

	make (w: EV_TITLED_WINDOW)
			-- Create interface in window `w'
		require
			window_exists: w /= Void
			window_valid: not w.is_destroyed
		do
			make_interface
			make_proxy (w)
		end

feature -- Modification

	set_primitive_steps (list: LIST [RC_PRIMITIVE_STEP])
			-- Set `primitive_steps' to `list'
		local
			step: RC_PRIMITIVE_STEP
		do
			precursor (list)
			create step_table.make (list.count)
			from
				list.start
			until
				list.after
			loop
				step := list.item
				step_table.put (step, step.step_name)
				list.forth
			end
		end

feature -- Operations

	reset
			-- Reset interface
		do
		end

feature -- Modification

	subscribe_events
			-- Subscribe to events of interest
		do
			(create {RC_STEP_DO_ENDED_EVENT}).subscribe (agent handle_step_do_ended)
			(create {RC_STEP_UNDO_ENDED_EVENT}).subscribe (agent handle_step_undo_ended)
			
			(create {RUN_CLERK_CHANGE_EVENT}).subscribe (agent handle_run_clerk_change)
			(create {DESIGN_ENGINEER_CHANGE_EVENT}).subscribe (agent handle_design_engineer_change)
			(create {TEST_ENGINEER_CHANGE_EVENT}).subscribe (agent handle_test_engineer_change)
			(create {RUN_CLOSER_CHANGE_EVENT}).subscribe (agent handle_run_closer_change)
			(create {RUN_TYPE_CHANGE_EVENT}).subscribe (agent handle_run_type_change)
			(create {OVERHEAD_RUN_ID_CHANGE_EVENT}).subscribe (agent handle_overhead_run_id_change)
			(create {EDUCATIONAL_STATUS_CHANGE_EVENT}).subscribe (agent handle_educational_status_change)
			(create {TAXI_STATUS_CHANGE_EVENT}).subscribe (agent handle_taxi_status_change)
			(create {WAFER_TECHN_CHANGE_EVENT}).subscribe (agent handle_wafer_techn_change)
			(create {MASK_TECHN_CHANGE_EVENT}).subscribe (agent handle_mask_techn_change)
			(create {FIELD_SET_CHANGE_EVENT}).subscribe (agent handle_field_set_change)
			(create {CLOSE_DATE_CHANGE_EVENT}).subscribe (agent handle_close_date_change)
			(create {SCHED_RUN_NO_CHANGE_EVENT}).subscribe (agent handle_sched_run_no_change)
			(create {OTHER_BORDER_CHANGE_EVENT}).subscribe (agent handle_other_border_change)
			(create {OVERHEAD_STYLE_CHANGE_EVENT}).subscribe (agent handle_overhead_style_change)
			(create {WAFER_PART_ID_SRC_CHANGE_EVENT}).subscribe (agent handle_wafer_part_id_src_change)
			(create {WAFER_PART_ID_CHANGE_EVENT}).subscribe (agent handle_wafer_part_id_change)
			(create {MASK_PART_ID_CHANGE_EVENT}).subscribe (agent handle_mask_part_id_change)
		end

	unsubscribe_events
			-- Unsubscribe from events of interest
		do
			(create {RC_STEP_DO_ENDED_EVENT}).unsubscribe (agent handle_step_do_ended)
			(create {RC_STEP_UNDO_ENDED_EVENT}).unsubscribe (agent handle_step_undo_ended)
			
			(create {RUN_CLERK_CHANGE_EVENT}).unsubscribe (agent handle_run_clerk_change)
			(create {DESIGN_ENGINEER_CHANGE_EVENT}).unsubscribe (agent handle_design_engineer_change)
			(create {TEST_ENGINEER_CHANGE_EVENT}).unsubscribe (agent handle_test_engineer_change)
			(create {RUN_CLOSER_CHANGE_EVENT}).unsubscribe (agent handle_run_closer_change)
			(create {RUN_TYPE_CHANGE_EVENT}).unsubscribe (agent handle_run_type_change)
			(create {OVERHEAD_RUN_ID_CHANGE_EVENT}).unsubscribe (agent handle_overhead_run_id_change)
			(create {EDUCATIONAL_STATUS_CHANGE_EVENT}).unsubscribe (agent handle_educational_status_change)
			(create {TAXI_STATUS_CHANGE_EVENT}).unsubscribe (agent handle_taxi_status_change)
			(create {WAFER_TECHN_CHANGE_EVENT}).unsubscribe (agent handle_wafer_techn_change)
			(create {MASK_TECHN_CHANGE_EVENT}).unsubscribe (agent handle_mask_techn_change)
			(create {FIELD_SET_CHANGE_EVENT}).unsubscribe (agent handle_field_set_change)
			(create {CLOSE_DATE_CHANGE_EVENT}).unsubscribe (agent handle_close_date_change)
			(create {SCHED_RUN_NO_CHANGE_EVENT}).unsubscribe (agent handle_sched_run_no_change)
			(create {OTHER_BORDER_CHANGE_EVENT}).unsubscribe (agent handle_other_border_change)
			(create {OVERHEAD_STYLE_CHANGE_EVENT}).unsubscribe (agent handle_overhead_style_change)
			(create {WAFER_PART_ID_SRC_CHANGE_EVENT}).unsubscribe (agent handle_wafer_part_id_src_change)
			(create {WAFER_PART_ID_CHANGE_EVENT}).unsubscribe (agent handle_wafer_part_id_change)
			(create {MASK_PART_ID_CHANGE_EVENT}).unsubscribe (agent handle_mask_part_id_change)
		end

	refresh
			-- Refresh information in interface so that
			-- it is up-to-date
		local
			edu_text, date_text, sched_text, border_text: STRING
			taxi_text, overhead_text, part_id_text: STRING
			part_id_source_text: STRING
			step: RC_PRIMITIVE_STEP
			step_name: PRIMITIVE_STEP_NAME_VALUES
			events: LIST [RUN_EVENTS_ROW]
			style: OVERHEAD_STYLE_VALUES
		do
			run := cache.db_runs_row
			production := cache.db_productions_row
			mask_set := cache.db_mask_sets_row
			create run_events_row.make_template
			events := run_events_row.run_closed_events (session.run_id, connection, expected_count.At_most_one)
			if events.is_empty then
				run_events_row := Void
			else
				run_events_row := events.first
			end
			
			run_id_field.set_text (run.run_id)
			run_type_field.set_text (run.run_type)
			
			step := step_with_name (step_name.set_educational_status)
			if not step.is_done then
				edu_text := ""
			elseif run.educational then
				edu_text := Yes
			else
				edu_text := No
			end
			
			educational_field.set_text (edu_text)
			
			step := step_with_name (step_name.set_taxi_status)
			if not step.is_done then
				taxi_text := ""
			elseif production.is_taxi then
				taxi_text := Yes
			else
				taxi_text := No
			end
			
			taxi_field.set_text (taxi_text)
			
			wafer_techn_field.set_text (run.wafer_techn)
			mask_techn_field.set_text (mask_set.mask_techn)
			field_set_field.set_text (run.field_set_name)
			
			if run_events_row = Void then
				date_text := ""
			else
				date_text := run_events_row.date_event.to_date_string
			end
			close_date_field.set_text (date_text)
			
			if run.is_null_sched_run_no then
				sched_text := ""
			else
				sched_text := sched_run_description (run.sched_run_no.out)
			end
			sched_run_field.set_text (sched_text)
			
			step := step_with_name (step_name.set_die_overhead_parameters)
			
			if not step.is_done then
				overhead_text := ""
			else
				overhead_text := style.null_translated (run.overhead_style)
			end
			
			overhead_style_field.set_text (overhead_text)
			
			border_text := number_as_string (run.other_border, run.is_null_other_border)
			other_border_field.set_text (border_text)
			top_overhead_field.set_text (current_top_overhead)
			
			step := step_with_name (step_name.set_wafer_part_id_source)
			part_id_source_text := value_or_not_required (production.wafer_part_id_source, step)
			wafer_part_id_src_field.set_text (part_id_source_text)
			
			step := step_with_name (step_name.set_wafer_part_id)
			part_id_text := value_or_not_required (production.wafer_part_id, step)
			wafer_part_id_field.set_text (part_id_text)
			
			step := step_with_name (step_name.set_mask_part_id)
			part_id_text := value_or_not_required (run.mask_part_id, step)
			mask_part_id_field.set_text (part_id_text)
			
			set_field_sizes
			set_wafer_techn_sizes

			closer_field.set_text (run.closer)
			clerk_field.set_text (production.clerk)
			design_engineer_field.set_text (run.design_engineer)
			test_engineer_field.set_text (production.test_engineer)
			
			overhead_run_id_field.set_text (run.run_id_for_overhead)
			
			refill_notebook_boxes
		end
		

feature {NONE} -- Implementation

	build_interface
			-- Build interface in `window'
		local
			main_vbox: EV_VERTICAL_BOX
			notebook: EV_NOTEBOOK
			buttons: HORIZONTAL_BUTTON_SET
			box: EV_BOX
			step_name: PRIMITIVE_STEP_NAME_VALUES
		do
			create notebook

			create general_fields.make (10)
			create run_id_field
			add_step_field (general_fields, run_id_field,
				"Run ID", Void)
			create overhead_run_id_field
			add_step_field (general_fields, overhead_run_id_field,
				"Overhead Run ID", step_name.Set_overhead_run_id)
			create run_type_field
			add_step_field (general_fields, run_type_field,
				"Run Type", step_name.Set_run_type)
			create educational_field
			add_step_field (general_fields, educational_field,
				"Educational", step_name.Set_educational_status)
			create taxi_field
			add_step_field (general_fields, taxi_field,
				"Taxi", step_name.Set_taxi_status)
			create wafer_techn_field
			add_step_field (general_fields, wafer_techn_field,
				"Wafer Technology", step_name.Set_wafer_techn)
			create mask_techn_field
			add_step_field (general_fields, mask_techn_field,
				"Mask Technology", step_name.Set_mask_techn)
			create field_set_field
			add_step_field (general_fields, field_set_field,
				"Field Set", step_name.Set_field_set)
			create close_date_field
			add_step_field (general_fields, close_date_field,
				"Close Date", step_name.Set_close_date)
			create sched_run_field
			add_step_field (general_fields, sched_run_field,
				"Scheduled Run", step_name.Set_sched_run_no)
			create overhead_style_field
			add_step_field (general_fields, overhead_style_field,
				"Die Labeling Style", step_name.Set_die_overhead_parameters)
			general_fields.last.set_always_displayed (True)
			create top_overhead_field
			add_step_field (general_fields, top_overhead_field,
				"Top Side Overhead", step_name.Set_die_overhead_parameters)
			general_fields.last.set_always_displayed (True)
			create other_border_field
			add_step_field (general_fields, other_border_field,
				"Other Side Overhead (White Space)", step_name.Set_die_overhead_parameters)
			general_fields.last.set_always_displayed (True)
			create wafer_part_id_src_field
			add_step_field (general_fields, wafer_part_id_src_field,
				"Wafer Part Id Source", step_name.Set_wafer_part_id_source)
			general_fields.last.set_always_displayed (True)

			create wafer_part_id_field
			add_step_field (general_fields, wafer_part_id_field,
				"Wafer Part Id", step_name.Set_wafer_part_id)
			general_fields.last.set_always_displayed (True)

			create mask_part_id_field
			add_step_field (general_fields, mask_part_id_field,
				"Mask Part Id", step_name.Set_mask_part_id)
			general_fields.last.set_always_displayed (True)

			create general_vbox
			general_vbox.set_padding (user_interface.padding)
			fill_notebook_box (general_vbox, general_fields)
			notebook.extend (general_vbox)
			notebook.set_item_text (general_vbox, "General")
			
			create technical_fields.make (10)
			create packing_target_field
			add_step_field (technical_fields, packing_target_field,
				"Packing Target", step_name.Set_field_set)
			create payload_size_x_field
			add_step_field (technical_fields, payload_size_x_field,
				"Payload Size X", step_name.Set_field_set)
			create payload_size_y_field
			add_step_field (technical_fields, payload_size_y_field,
				"Payload Size Y", step_name.Set_field_set)
			create external_kerf_x_field
			add_step_field (technical_fields, external_kerf_x_field,
				"External Kerf X", step_name.Set_field_set)
			create external_kerf_y_field
			add_step_field (technical_fields, external_kerf_y_field,
				"External Kerf Y", step_name.Set_field_set)
			create field_size_x_field
			add_step_field (technical_fields, field_size_x_field,
				"Field Size X", step_name.Set_field_set)
			create field_size_y_field
			add_step_field (technical_fields, field_size_y_field,
				"Field Size Y", step_name.Set_field_set)
			create merged_size_x_field
			add_step_field (technical_fields, merged_size_x_field,
				"Merged Size X", step_name.Set_field_set)
			create merged_size_y_field
			add_step_field (technical_fields, merged_size_y_field,
				"Merged Size Y", step_name.Set_field_set)
			create grid_resolution_field
			add_step_field (technical_fields, grid_resolution_field,
				"Grid Resolution", step_name.Set_wafer_techn)
			create min_separation_field
			add_step_field (technical_fields, min_separation_field,
				"Min Separation", step_name.Set_wafer_techn)
			create mask_size_field
			add_step_field (technical_fields, mask_size_field,
				"Mask Size", step_name.Set_field_set)
			create wafer_type_field
			add_step_field (technical_fields, wafer_type_field,
				"Wafer Type", step_name.Set_field_set)
			create primary_flat_field
			add_step_field (technical_fields, primary_flat_field,
				"Primary Flat Location", step_name.Set_field_set)
			create tweezer_margin_field
			add_step_field (technical_fields, tweezer_margin_field,
				"Tweezer Margin", step_name.Set_field_set)
			
			create technical_vbox
			technical_vbox.set_padding (user_interface.padding)
			
			fill_notebook_box (technical_vbox, technical_fields)
			notebook.extend (technical_vbox)
			notebook.set_item_text (technical_vbox, "Technical")
			
			create roles_fields.make (10)
			create closer_field
			add_step_field (roles_fields, closer_field,
				"Run Closer", step_name.Set_closer)
			create clerk_field
			add_step_field (roles_fields, clerk_field,
				"Run Clerk", step_name.Set_clerk)
			create design_engineer_field
			add_step_field (roles_fields, design_engineer_field,
				"Design Engineer", step_name.Set_design_engineer)
			create test_engineer_field
			add_step_field (roles_fields, test_engineer_field,
				"Test Engineer", step_name.Set_test_engineer)
			
			create roles_vbox
			roles_vbox.set_padding (user_interface.padding)
			
			fill_notebook_box (roles_vbox, roles_fields)
			notebook.extend (roles_vbox)
			notebook.set_item_text (roles_vbox, "Roles")
			
			create main_vbox
			main_vbox.extend (notebook)
			main_vbox.disable_item_expand (notebook)
			
			create buttons.make_with_text_and_actions (<< gui.ev_cancel >>, << agent process_decline >>)
			box := buttons.column_set.box
			
			main_vbox.extend (box)
			main_vbox.disable_item_expand (box)
			gui.set_box_background_recursive (main_vbox, background_color)
			extend (main_vbox)

			close_request_actions.extend (agent hide)
			gui.add_cancel_accelerator (window, agent process_decline)
			enable_active
			refresh
		end
		
	process_decline
			-- Take actions needed when user declines
			-- current operation
		do
			if is_active then
				disable_active
				decline_actions.call ([])
			end
		end

	step_with_name (s: STRING): RC_PRIMITIVE_STEP
			-- Primitive step with name `s'
		require
			name_exists: s /= Void
		do
			Result := step_table.item (s)
		end
		
	add_step_field (list: LIST [RC_STEP_TEXT_FIELD] field: EV_TEXT_FIELD label: STRING step_name: STRING)
			-- Add new step field for `step' with `label' to `list'
		require
			list_exists: list /= Void
			label_exists: label /= Void
			field_exists: field /= Void
		local
			step_field: RC_STEP_TEXT_FIELD
			step: RC_PRIMITIVE_STEP
		do
			if step_name /= Void then
				step := step_with_name (step_name)
			else
				step := Void
			end
			create step_field.make (field, label, step, session)
			list.extend (step_field)
		end
		
	fill_notebook_box (vbox: EV_VERTICAL_BOX fields: LIST [RC_STEP_TEXT_FIELD])
			-- Fill `vbox' with `fields'.  Do not include
			-- fields that are inapplicable
		require
			vbox_exists: vbox /= Void
			fields_exists: fields /= Void
		local
			field: RC_STEP_TEXT_FIELD
			cols: WIDGET_COLUMN_SET
			box: EV_BOX
		do
			vbox.wipe_out
			vbox.set_border_width (user_interface.border_width)
			create cols.make (2)
			cols.set_padding (user_interface.padding)
			
			from
				fields.start
			until
				fields.after
			loop
				field := fields.item
				if field.is_display_requested then
					prepare_field (field.text_field)
					cols.append_row (<< gui.label_with_text (field.label), field.text_field >>)
				end
				fields.forth
			end
			cols.disable_column_expand (1)
			cols.align_column_right (1)
			box := cols.box
			vbox.extend (box)
			vbox.disable_item_expand (box)
		end

	refill_notebook_boxes
		do
			refill_notebook_box (general_vbox, general_fields)
			refill_notebook_box (technical_vbox, technical_fields)
			refill_notebook_box (roles_vbox, roles_fields)
		end

	refill_notebook_box (vbox: EV_VERTICAL_BOX fields: LIST [RC_STEP_TEXT_FIELD])
			-- Refill `vbox' with `fields', if applicability
			-- of any fields has changed
		require
			vbox_exists: vbox /= Void
			fields_exists: fields /= Void
		local
			field: RC_STEP_TEXT_FIELD
			old_display, new_display, changed: BOOLEAN
			old_done, new_done: BOOLEAN
		do
			from
				fields.start
			until
				fields.after
			loop
				field := fields.item
				old_done := field.all_applicability_steps_done
				field.compute_all_applicability_steps_done
				new_done := field.all_applicability_steps_done
				if old_done /= new_done then
					old_display := field.is_display_requested
					field.compute_applicable
					new_display := field.is_display_requested
					if old_display /= new_display then
						changed := True
					end
				end
				fields.forth
			end
			if changed then
				fill_notebook_box (vbox, fields)
			end
		end

	prepare_field (f: EV_TEXT_FIELD)
			-- Prepare field `f'
		require
			field_exists: f /= Void
		do
			f.set_minimum_width (Minimum_field_width)
			f.disable_edit
			if f.parent /= Void then
				f.parent.destroy
			end
		end
		
	current_top_overhead: STRING
			-- Description of top overhead
		local
			l_style: STRING
			wt: WAFER_TECHNS_ROW
		do
			run := cache.runs_row
			if cache.is_nonempty_string (cache.wafer_techn) then
				wt := cache.wafer_techns_row
				l_style := run.overhead_style
				if not run.is_null_other_border then
					Result := wt.top_overhead_description (l_style, run.other_border)
				else
					Result := ""
				end
			else
				Result := ""
			end
		end

	handle_step_do_ended (event: RC_STEP_DO_ENDED_EVENT)
		do
			refill_notebook_boxes
		end

	handle_step_undo_ended (event: RC_STEP_UNDO_ENDED_EVENT)
		do
			refill_notebook_boxes
		end

	handle_run_clerk_change (event: RUN_CLERK_CHANGE_EVENT)
		do
			clerk_field.set_text (event.run_clerk)
		end

	handle_design_engineer_change (event: DESIGN_ENGINEER_CHANGE_EVENT)
		do
			design_engineer_field.set_text (event.design_engineer)
		end

	handle_test_engineer_change (event: TEST_ENGINEER_CHANGE_EVENT)
		do
			test_engineer_field.set_text (event.test_engineer)
		end

	handle_run_closer_change (event: RUN_CLOSER_CHANGE_EVENT)
		do
			closer_field.set_text (event.run_closer)
		end

	handle_run_type_change (event: RUN_TYPE_CHANGE_EVENT)
		do
			run_type_field.set_text (event.run_type)
		end

	handle_overhead_run_id_change (event: OVERHEAD_RUN_ID_CHANGE_EVENT)
		do
			overhead_run_id_field.set_text (event.overhead_run_id)
		end

	handle_educational_status_change (event: EDUCATIONAL_STATUS_CHANGE_EVENT)
		do
			educational_field.set_text (event.is_educational)
		end

	handle_taxi_status_change (event: TAXI_STATUS_CHANGE_EVENT)
		do
			taxi_field.set_text (event.is_taxi)
		end

	handle_wafer_techn_change (event: WAFER_TECHN_CHANGE_EVENT)
		do
			wafer_techn_field.set_text (event.wafer_techn)
			set_wafer_techn_sizes
		end

	handle_mask_techn_change (event: MASK_TECHN_CHANGE_EVENT)
		do
			mask_techn_field.set_text (event.mask_techn)
		end

	handle_field_set_change (event: FIELD_SET_CHANGE_EVENT)
		do
			field_set_field.set_text (event.field_set)
			set_field_sizes
		end

	handle_close_date_change (event: CLOSE_DATE_CHANGE_EVENT)
		do
			close_date_field.set_text (event.close_date)
		end

	handle_sched_run_no_change (event: SCHED_RUN_NO_CHANGE_EVENT)
		do
			sched_run_field.set_text (sched_run_description (event.sched_run_no))
		end

	handle_other_border_change (event: OTHER_BORDER_CHANGE_EVENT)
		do
			other_border_field.set_text (event.other_border)
			top_overhead_field.set_text (current_top_overhead)
		end

	handle_overhead_style_change (event: OVERHEAD_STYLE_CHANGE_EVENT)
		do
			overhead_style_field.set_text (event.overhead_style)
			top_overhead_field.set_text (current_top_overhead)
		end

	handle_wafer_part_id_src_change (event: WAFER_PART_ID_SRC_CHANGE_EVENT)
		local
			step: RC_PRIMITIVE_STEP
			text: STRING
			step_name: PRIMITIVE_STEP_NAME_VALUES
		do
			step := step_with_name (step_name.set_wafer_part_id_source)
			text := value_or_not_required (event.wafer_part_id_source, step)
			wafer_part_id_src_field.set_text (text)
		end

	handle_wafer_part_id_change (event: WAFER_PART_ID_CHANGE_EVENT)
		local
			step: RC_PRIMITIVE_STEP
			text: STRING
			step_name: PRIMITIVE_STEP_NAME_VALUES
		do
			step := step_with_name (step_name.set_wafer_part_id)
			text := value_or_not_required (event.wafer_part_id, step)
			wafer_part_id_field.set_text (text)
		end

	handle_mask_part_id_change (event: MASK_PART_ID_CHANGE_EVENT)
		local
			step: RC_PRIMITIVE_STEP
			text: STRING
			step_name: PRIMITIVE_STEP_NAME_VALUES
		do
			step := step_with_name (step_name.set_mask_part_id)
			text := value_or_not_required (event.mask_part_id, step)
			mask_part_id_field.set_text (text)
		end

	set_wafer_techn_sizes
			-- Set values of wafer techn parameters
		local
			step: RC_PRIMITIVE_STEP
			step_name: PRIMITIVE_STEP_NAME_VALUES
			wt: WAFER_TECHNS_ROW
		do
			step := step_with_name (step_name.set_wafer_techn)
			if step.is_done then
				wt := cache.wafer_techns_row
				grid_resolution_field.set_text (number_as_string (wt.packing_grid_resolution, wt.is_null_packing_grid_resolution))
				min_separation_field.set_text (number_as_string (wt.packing_minimum_separation, wt.is_null_packing_minimum_separation))
			else
				grid_resolution_field.set_text ("")
				min_separation_field.set_text ("")
			end
		end

	set_field_sizes
			-- Set values of fields sizes
		local
			step: RC_PRIMITIVE_STEP
			step_name: PRIMITIVE_STEP_NAME_VALUES
			ls: like litho_sizes_row
		do
			step := step_with_name (step_name.set_field_set)
			if step.is_done then
				litho_sizes_row := cache.litho_sizes_row
				ls := litho_sizes_row
				scale_factor := cache.scale_factor
				packing_target_field.set_text (ls.packing_target)
				payload_size_x_field.set_text (number_as_string (ls.payload_size_x, ls.is_null_payload_size_x))
				payload_size_y_field.set_text (number_as_string (ls.payload_size_y, ls.is_null_payload_size_y))
				field_size_x_field.set_text (number_as_string (ls.field_size_x, ls.is_null_field_size_x))
				field_size_y_field.set_text (number_as_string (ls.field_size_y, ls.is_null_field_size_y))
				external_kerf_x_field.set_text (number_as_string (ls.external_kerf_x, ls.is_null_field_size_x or ls.is_null_payload_size_x))
				external_kerf_y_field.set_text (number_as_string (ls.external_kerf_y, ls.is_null_field_size_y or ls.is_null_payload_size_y))
				merged_size_x_field.set_text (number_as_string (ls.merged_pattern_file_size_x / scale_factor, ls.is_null_merged_pattern_file_size_x))
				merged_size_y_field.set_text (number_as_string (ls.merged_pattern_file_size_y / scale_factor, ls.is_null_merged_pattern_file_size_y))
				mask_size_field.set_text (ls.mask_size_id)
				wafer_type_field.set_text (ls.wafer_type_id)
				primary_flat_field.set_text (ls.primary_flat_location)
				tweezer_margin_field.set_text (number_as_string (ls.tweezer_margin, ls.is_null_tweezer_margin))
			else
				packing_target_field.set_text ("")
				payload_size_x_field.set_text ("")
				payload_size_y_field.set_text ("")
				field_size_x_field.set_text ("")
				field_size_y_field.set_text ("")
				external_kerf_x_field.set_text ("")
				external_kerf_y_field.set_text ("")
				merged_size_x_field.set_text ("")
				merged_size_y_field.set_text ("")
				mask_size_field.set_text ("")
				wafer_type_field.set_text ("")
				primary_flat_field.set_text ("")
				tweezer_margin_field.set_text ("")
			end
		end

	value_or_not_required (text: STRING step: RC_PRIMITIVE_STEP): STRING
			-- Value of `text', or "not required" if `text'
			-- is empty and step is done and was not applicable
			-- to run
		require
			text_exists: text /= Void
			step_exists: step /= Void
		do
			if text.is_empty and step.is_done and then not step.was_applicable_to_run then
				Result := Not_required
			else
				Result := text
			end
		ensure
			result_exists: Result /= Void
		end

	sched_run_description (sched_run_no: STRING): STRING
			-- Description for `sched_run_no'
		local
			row: SCHEDULED_RUNS_ROW
		do
			if sched_run_no.is_integer then
				create row.make_by_key (session.connection, option.Old_row, sched_run_no.to_integer)
				Result := row.date_scheduled.to_date_string + "    (" + sched_run_no + ")"
			else
				Result := ""
			end
		end

	number_as_string (n: DOUBLE is_null:  BOOLEAN): STRING
			-- Value of `n' as a string suitable for display
		do
			if not is_null then
				Result := nu.formatted_number_or_integer (n, {WAFER_TECHNS_ROW}.scale_top_border)
			else
				Result := ""
			end
		end

	run: RUNS_ROW
			-- Row from RUNS table

	production: PRODUCTIONS_ROW
			-- Row from PRODUCTIONS table

	mask_set: MASK_SETS_ROW
			-- Row from MASK_SETS table

	run_events_row: RUN_EVENTS_ROW
			-- Row for "run closed" event

	litho_sizes_row: LITHO_SIZES_ROW
			-- Row for litho size

	scale_factor: DOUBLE
			-- Scale factor for first wafer techn level

	step_table: HASH_TABLE [RC_PRIMITIVE_STEP, STRING]
			-- Primitive step lookup table, with each
			-- step stored under its name

	general_fields: ARRAYED_LIST [RC_STEP_TEXT_FIELD]
			-- Fields in General tab of notebook

	technical_fields: ARRAYED_LIST [RC_STEP_TEXT_FIELD]
			-- Fields in Technical tab of notebook

	roles_fields: ARRAYED_LIST [RC_STEP_TEXT_FIELD]
			-- Fields in Roles tab of notebook

	general_vbox, technical_vbox, roles_vbox: EV_VERTICAL_BOX
			-- Boxes for field in each notebook tab

	label_box, text_box: EV_VERTICAL_BOX

	run_id_field: EV_TEXT_FIELD
	run_type_field: EV_TEXT_FIELD
	overhead_run_id_field, educational_field, wafer_techn_field: EV_TEXT_FIELD
	taxi_field: EV_TEXT_FIELD
	mask_techn_field, field_set_field, close_date_field: EV_TEXT_FIELD
	sched_run_field, overhead_style_field, top_overhead_field, other_border_field: EV_TEXT_FIELD
	wafer_part_id_src_field: EV_TEXT_FIELD
	wafer_part_id_field, mask_part_id_field: EV_TEXT_FIELD

	packing_target_field: EV_TEXT_FIELD
	mask_size_field, wafer_type_field: EV_TEXT_FIELD
	primary_flat_field, tweezer_margin_field: EV_TEXT_FIELD
	payload_size_x_field, payload_size_y_field: EV_TEXT_FIELD
	field_size_x_field, field_size_y_field: EV_TEXT_FIELD
	external_kerf_x_field, external_kerf_y_field: EV_TEXT_FIELD
	merged_size_x_field, merged_size_y_field: EV_TEXT_FIELD
	grid_resolution_field, min_separation_field: EV_TEXT_FIELD
	
	closer_field, clerk_field: EV_TEXT_FIELD
	design_engineer_field, test_engineer_field: EV_TEXT_FIELD

	Minimum_field_width: INTEGER = 200
			-- Minimum text field width in pixels

	Not_required: STRING = "Not Required"

end
