
indexing
	
	description: "A run closing step (primitive or composite)"
	author: "David Hollenberg"
	date: "April 16, 2003"

deferred class RC_STEP
inherit
	MARKABLE

feature -- Initialization

	initialize (sess: RC_RUN_CLOSING_SESSION)
			-- Initialize step that will be executed in context
			-- of `sess'
		require
			sess_exists: sess /= Void
		do
			session := sess
			create {ARRAYED_LIST [RC_COMPOSITE_STEP]} parents.make (5)
		end

feature -- Properties

	session: RC_RUN_CLOSING_SESSION
			-- Session context for step

	parents: DYNAMIC_LIST [RC_COMPOSITE_STEP]
			-- Composite steps that include this step
			-- as a component

	is_top_level: BOOLEAN
			-- Is this a top-level step (one with no parents)?
		require
			parents_exists: parents /= Void
		do
			Result := parents.is_empty
		end

	step_name: STRING
			-- Name of step
		deferred
		end
	
	type_name: STRING
			-- Name for step type (primitive or composite)
		deferred
		end
	
	presentation_text: STRING
			-- Text to be presented in user interface
		deferred
		end
	
	description: STRING
			-- Description of step suitable for logging
		do
			Result := type_name + " step " + step_name
		end
	
	implementation: RC_STEP_OP
			-- Operation that implements step
	
	is_primitive: BOOLEAN
			-- Is this a primitive step?
		deferred
		end
	
	is_composite: BOOLEAN
			-- Is this a composite step?
		do
			Result := not is_primitive
		end
	
	component_steps: LIST [RC_STEP]
			-- Sub-steps of this step, if any
		deferred
		ensure
			result_exists: Result /= Void
		end

	primitive_steps: LIST [RC_PRIMITIVE_STEP]
			-- Primitive steps included in this step,
			-- either directly or indirectly, in order
		deferred
		ensure
			result_exists: result /= Void
		end

	was_applicable_to_run: BOOLEAN
			-- Was step applicable to run?
		deferred
		end

	is_done: BOOLEAN
			-- Has step been fully completed?
		deferred
		end

	is_undone: BOOLEAN
			-- Is step fully uncompleted (not even started)?
		deferred
		end

	is_soft_undone: BOOLEAN
			-- Is step uncompleted, but was previously
			-- done and then undone with a "soft undo",
			-- possibly saving some state?
		deferred
		end

	is_partially_done: BOOLEAN
			-- Is step partially completed?
		deferred
		end

	is_partially_undone: BOOLEAN
			-- Has step been partially undone?
		deferred
		end

	has_saved_state: BOOLEAN
			-- Does step have saved state from a previous
			-- execution?
		deferred
		end

feature -- Modification

	set_implementation (i: RC_STEP_OP)
			-- Set `implementation' to `i'
		do
			implementation := i
		ensure
			value_set: implementation = i
		end

end
