
indexing
	
	description: "A operation that does a run closing step"
	author: "David Hollenberg"
	date: "April 16, 2003"

deferred class RC_STEP_OP
inherit
	DYNAMIC_OPERATION
		rename
			try_operation as try_step
		end
	RC_SESSIONABLE
		undefine
			cache
		redefine
			connection
		end

feature {NONE} -- Creation

	rc_step_make (sess: RC_RUN_CLOSING_SESSION)
			-- Create implementation with `sess' as session context
		require
			sess_exists: sess /= Void
		do
			default_create
			cache := sess.cache
			session := sess
			factory := sess.factory
			application:= sess.application
			file_generator := sess.file_generator
			preferences := sess.preferences
			connection := session.connection
			create completion_actions
		end

feature -- Properties

	cache: RUN_CACHE
			-- Run-specific cache

	connection: DB_CONNECTION
			-- Database connection to use

	environment: RC_ENVIRONMENT
			-- Environment for run closing
		do
			Result := session.environment
		end

	factory: RC_INTERFACE_FACTORY
			-- Factory from which to get interface elements

	application: RC_APPLICATION
			-- Application control

	file_generator: RC_FILE_GENERATOR
			-- Generator that produces file category and
			-- file objects

	preferences: RC_PREFERENCE_SET
			-- User preferences for session

	completion_actions: CLOSED_ACTION_SEQUENCE
			-- Actions to be executed when do or undo
			-- operation on step completes

feature -- Modification

	set_do
		do
			set_implementation (agent do_step)
		end

	set_undo_and_discard_state
		do
			set_implementation (agent undo_and_discard_state)
		end

	set_undo_and_save_state
		do
			set_implementation (agent undo_and_save_state)
		end

feature -- Operations

	process_pending_abort
			-- Process pending abort request from user, if any
		do
			session.process_pending_abort
		end

feature {RC_PRIMITIVE_STEPS_OP} -- Operations

	call_completion_actions
			-- Call `completion_actions'
		do
			completion_actions.call ([])
		end

feature {NONE} -- Implementation

	do_step
			-- Execute step
		deferred
		end

	undo_and_discard_state
			-- Undo execution of step.  Do not save state
			-- information
		deferred
		end

	undo_and_save_state
			-- Undo execution of step and save state information
			-- so that step might be redone with less work
		deferred
		end

	add_commit_action (p: PROCEDURE [ANY, TUPLE])
			-- Append `p' to session's commit actions
		do
			session.commit_actions.extend (p)
		end

invariant
	session_exists: session /= Void
	factory_exists: factory /= Void
	completion_actions_exists: completion_actions /= Void
end
