
indexing
	
	description: "Interface to obtain information about the set of run %
		%closing steps user selected and what kind of action user %
		%wants to apply (execute, undo, undo and save state)"
	author: "David Hollenberg"
	date: "April 17, 2003"

deferred class RC_STEP_SELECTION_IFC

inherit
	ABSTRACT_INTERFACE
		
feature {NONE} -- Creation

	make_interface
			-- Create interface
		do
			create do_actions
			create modify_actions
			create redo_actions
			create undo_actions
			create hard_undo_actions
			create cancel_actions
		end

feature -- Modification

	set_session (sess: RC_RUN_CLOSING_SESSION)
			-- Set `session' to `sess'
		require
			sess_exists: sess /= Void
		do
			session := sess
		ensure
			session_set: session = sess
		end

	set_status_interface (ifc: like status_interface)
			-- Set `status_interface' to `ifc'
		require
			ifc_exists: ifc /= Void
		do
			status_interface := ifc
		ensure
			interface_set: status_interface = ifc
		end

	set_wafer_map_print_interface (ifc: like wafer_map_print_interface)
			-- Set `wafer_map_print_interface' to `ifc'
		require
			ifc_exists: ifc /= Void
		do
			wafer_map_print_interface := ifc
		ensure
			interface_set: wafer_map_print_interface = ifc
		end

	set_bond_diagram_print_interface (ifc: like bond_diagram_print_interface)
			-- Set `bond_diagram_print_interface' to `ifc'
		require
			ifc_exists: ifc /= Void
		do
			bond_diagram_print_interface := ifc
		ensure
			interface_set: bond_diagram_print_interface = ifc
		end

	set_job_file_print_interface (ifc: like job_file_print_interface)
			-- Set `job_file_print_interface' to `ifc'
		require
			ifc_exists: ifc /= Void
		do
			job_file_print_interface := ifc
		ensure
			interface_set: job_file_print_interface = ifc
		end

	set_preferences_interface (ifc: like preferences_interface)
			-- Set `preferences_interface' to `ifc'
		require
			ifc_exists: ifc /= Void
		do
			preferences_interface := ifc
		ensure
			interface_set: preferences_interface = ifc
		end

	set_primitive_steps (list: LIST [RC_PRIMITIVE_STEP])
			-- Set `primitive_steps' to `list'
		require
			list_exists: list /= Void
		do
			primitive_steps := list
			set_all_steps
		ensure
			value_set: primitive_steps = list
		end

	set_composite_steps (list: LIST [RC_COMPOSITE_STEP])
			-- Set `composite_steps' to `list'
		require
			list_exists: list /= Void
		do
			composite_steps := list
			set_all_steps
		ensure
			value_set: composite_steps = list
		end

	subscribe_events
			-- Subscribe to events of interest
		deferred
		end

	unsubscribe_events
			-- Unsubscribe from events of interest
		deferred
		end

feature -- Properties

	session: RC_RUN_CLOSING_SESSION
			-- Session used to close run

	status_interface: RC_STATUS_IFC
			-- Interface to show run status

	wafer_map_print_interface: RC_MAP_PRINT_IFC
			-- Interface to print wafer maps

	bond_diagram_print_interface: RC_MAP_PRINT_IFC
			-- Interface to print bonding diagrams

	job_file_print_interface: RC_MAP_PRINT_IFC
			-- Interface to print job files

	preferences_interface: EDIT_PREFERENCES_IFC
			-- Interface to allow user to edit preferences

	primitive_steps: LIST [RC_PRIMITIVE_STEP]
			-- Primitive run closing steps

	composite_steps: LIST [RC_COMPOSITE_STEP]
			-- Composite run closing steps

	all_steps: LIST [RC_STEP]
			-- All run closing steps, both composite and primitive

	do_actions: ACTION_SEQUENCE [TUPLE [RC_STEP]]
			-- Actions to be executed when user clicks on Do
			-- button

	redo_actions: ACTION_SEQUENCE [TUPLE [RC_STEP]]
			-- Actions to be executed when user clicks on Redo
			-- button

	modify_actions: ACTION_SEQUENCE [TUPLE [RC_STEP]]
			-- Actions to be executed when user clicks on Modify
			-- button

	undo_actions: ACTION_SEQUENCE [TUPLE [RC_STEP]]
			-- Actions to be executed when user clicks on Undo
			-- button

	hard_undo_actions: ACTION_SEQUENCE [TUPLE [RC_STEP]]
			-- Actions to be executed when user clicks on Hard Undo
			-- button

	cancel_actions: CLOSED_ACTION_SEQUENCE
			-- Actions to be executed when user clicks on Cancel
			-- button

feature {NONE} -- Implementation
	
	set_all_steps
			-- Set `all_steps' to concatenation of
			-- `composite_steps' and `primitive_steps'
		local
			count1, count2: INTEGER
		do
			if primitive_steps /= Void then
				count1 := primitive_steps.count
			end
			if composite_steps /= Void then
				count2 := composite_steps.count
			end
			create {ARRAYED_LIST [RC_STEP]} all_steps.make (count1 + count2)
			
			if composite_steps /= Void then
				all_steps.append (composite_steps)
			end
			if primitive_steps /= Void then
				all_steps.append (primitive_steps)
			end
		ensure
			all_steps_exists: all_steps /= Void
		end

end
