
indexing
	
	description: "Window to present a forest of run %
		%closing steps and obtain the user's selection, as well as %
		%what action is to be applied (execute, undo, redo)"
	author: "David Hollenberg"
	date: "April 17, 2003"

class RC_STEP_SELECTION_WINDOW

inherit
	RC_STEP_SELECTION_IFC
		redefine
			status_interface,
			wafer_map_print_interface,
			bond_diagram_print_interface,
			job_file_print_interface,
			preferences_interface
		end
	WINDOW_INTERFACE
		redefine
			enable_active,
			disable_active
		end
	MOSIS_GRAPHICAL_UTILITIES
		export
			{NONE} all
		undefine
			default_create
		end
	RC_ENVIRONMENT_ACCESS
		export
			{NONE} all
		undefine
			default_create
		end
	DELTA_VERSION_CHECKER_ACCESS
		export
			{NONE} all
		undefine
			default_create
		end
		
create
	make

feature {NONE} -- Creation

	make (w: EV_TITLED_WINDOW)
			-- Create interface in window `w'
		require
			window_exists: w /= Void
			window_valid: not w.is_destroyed
		do
			make_interface
			make_proxy (w)
		end

feature -- Operations

	reset
			-- Move focus back into window
		do
			enable_active
			-- center_pointer
		end

feature -- Modification

	set_step_done_icon (p: EV_PIXMAP)
		do
			step_done_icon := p
		end

	set_step_inapplicable_icon (p: EV_PIXMAP)
		do
			step_inapplicable_icon := p
		end

	set_step_undone_icon (p: EV_PIXMAP)
		do
			step_undone_icon := p
		end

	set_doing_step_icon (p: EV_PIXMAP)
		do
			doing_step_icon := p
		end

	set_undoing_step_icon (p: EV_PIXMAP)
		do
			undoing_step_icon := p
		end

	set_step_saved_state_icon (p: EV_PIXMAP)
		do
			step_saved_state_icon := p
		end
	
	enable_active
			-- Make interface active
		do
			precursor
			if not session.is_read_only and tree.selected_item /= Void then
				select_tree_item
			end
			cancel_button.enable_sensitive
			flush_menu_item.enable_sensitive
			view_packing_menu_item.enable_sensitive
			close_menu_item.enable_sensitive
			exit_menu_item.enable_sensitive
		end

	disable_active
			-- Make interface inactive
			-- (can look but cannot initiate most actions)
		do
			precursor
			do_button.disable_sensitive
			modify_button.disable_sensitive
			redo_button.disable_sensitive
			undo_button.disable_sensitive
			hard_undo_button.disable_sensitive
			cancel_button.disable_sensitive
			flush_menu_item.disable_sensitive
			view_packing_menu_item.disable_sensitive
			close_menu_item.disable_sensitive
			exit_menu_item.disable_sensitive
		end

	subscribe_events
			-- Subscribe to events of interest
		do
			(create {RC_STEP_STATE_CHANGE_EVENT}).subscribe (agent handle_step_state_change)
			(create {WAFER_TECHN_CHANGE_EVENT}).subscribe (agent handle_wafer_techn_change)
			(create {RUN_TYPE_CHANGE_EVENT}).subscribe (agent handle_run_type_change)
			(create {RUN_CLOSER_CHANGE_EVENT}).subscribe (agent handle_run_closer_change)
		end

	unsubscribe_events
			-- Unsubscribe from events of interest
		do
			(create {RC_STEP_STATE_CHANGE_EVENT}).unsubscribe (agent handle_step_state_change)
		end

feature {NONE} -- Implementation

	tree: EV_TREE
			-- Tree of run closing steps
	
	do_button: EV_BUTTON
			-- Button to do selected steps that are not done
	
	modify_button: EV_BUTTON
			-- Button to modify selected step that is done
			-- (undoes all steps that depend on it first)
	
	redo_button: EV_BUTTON
			-- Button to redo selected steps that are done or
			-- partially done
	
	undo_button: EV_BUTTON
			-- Button to undo selected steps that are done or
			-- partially done and save state information
	
	hard_undo_button: EV_BUTTON
			-- Button to undo selected steps that are done or
			-- partially done and discard state information
	
	cancel_button: EV_BUTTON
			-- Button to end session for this run
	
	last_step: RC_STEP
			-- Step for last selected tree item
	
	build_interface
			-- Build interface
		local
			tree_item: EV_TREE_ITEM
			step: RC_STEP
			buttons: HORIZONTAL_BUTTON_SET
			separator: EV_HORIZONTAL_SEPARATOR
			main_vbox: EV_VERTICAL_BOX
			box: EV_BOX
		do
			if session.is_read_only then
				-- FIXME: use fetch that issues error instead
				-- of dying if session is_read_only
				fetch_run
			else
				fetch_run
			end
			add_menu_bar
			create separator
			upper_bar.extend (separator)
			initialize_status_box
			upper_bar.extend (status_box)
			create separator
			upper_bar.extend (separator)
			initialize_toolbar_box
			upper_bar.extend (toolbar_box)
			create separator
			upper_bar.extend (separator)

			create main_vbox
			
			create tree
			
			from
				all_steps.start
			until
				all_steps.after
			loop
				step := all_steps.item
				if step.is_top_level then
					create tree_item.make_with_text (step.presentation_text)
					tree_item.set_data (step)
					tree.extend (tree_item)
					if step.is_composite then
						fill_tree_item (tree_item, step)
					end
				end
				all_steps.forth
			end
			if tree.count = 1 and then tree.first.is_expandable then
				tree.first.expand
				tree.first.enable_select
			end

			update_all_icons

			if not session.is_read_only then
				tree.select_actions.extend (agent select_tree_item)
				tree.deselect_actions.extend (agent deselect_tree_item)
			end

			main_vbox.extend (tree)
			
			create buttons.make_with_text_and_actions (<< "Do", "Modify", "Redo", "Undo", "Hard Undo", "Cancel" >>, << agent process_do, agent process_modify, agent process_redo, agent process_undo, agent process_hard_undo, agent process_cancel >>)
			do_button := buttons.i_th (1)
			modify_button := buttons.i_th (2)
			redo_button := buttons.i_th (3)
			undo_button := buttons.i_th (4)
			hard_undo_button := buttons.i_th (5)
			cancel_button := buttons.i_th (6)
			
			do_button.disable_sensitive
			modify_button.disable_sensitive
			redo_button.disable_sensitive
			undo_button.disable_sensitive
			hard_undo_button.disable_sensitive
			
			do_button.set_tooltip ("Do the selected steps or resume them if they have already been started")
			modify_button.set_tooltip ("Re-enter the selected step so that you can modify it")
			redo_button.set_tooltip ("Undo the selected steps and save state information, then do the steps that were just undone")
			undo_button.set_tooltip ("Undo the selected steps but save state information to allow them to be redone easily")
			hard_undo_button.set_tooltip ("Undo the selected steps permanently and leave no trace of them in file system or database")
			
			box := buttons.column_set.box
			
			main_vbox.extend (box)
			main_vbox.disable_item_expand (box)
			
			gui.set_box_background_recursive (main_vbox, background_color)
			extend (main_vbox)
			gui.add_cancel_accelerator (window, agent process_cancel)
			gui.do_once_on_idle (agent post_display_actions)
		end
		
	add_menu_bar
			-- Add menu bar to window
		local
			file_menu, edit_menu, help_menu: EV_MENU
			window_menu, print_menu: EV_MENU
			about_menu_item: EV_MENU_ITEM
			prefs_menu_item: EV_MENU_ITEM
			raise_menu_item: EV_MENU_ITEM
			maps_menu_item: EV_MENU_ITEM
			a_menu_bar: EV_MENU_BAR
		do	
			create file_menu.make_with_text ("File")
			create edit_menu.make_with_text ("Edit")
			create window_menu.make_with_text ("Window")
			create help_menu.make_with_text ("Help")
			
			create print_menu.make_with_text ("Print")
			file_menu.extend (print_menu)
			
			create maps_menu_item.make_with_text ("Wafer Maps")
			maps_menu_item.select_actions.extend (agent process_print_maps (wafer_map_print_interface))
			print_menu.extend (maps_menu_item)
			
			create maps_menu_item.make_with_text ("Bonding Diagrams")
			maps_menu_item.select_actions.extend (agent process_print_maps (bond_diagram_print_interface))
			print_menu.extend (maps_menu_item)
			
			create maps_menu_item.make_with_text ("Job Files")
			maps_menu_item.select_actions.extend (agent process_print_maps (job_file_print_interface))
			print_menu.extend (maps_menu_item)
			
			create view_packing_menu_item.make_with_text ("View Packing")
			view_packing_menu_item.select_actions.extend (agent process_view_packing)
			file_menu.extend (view_packing_menu_item)
			
			create flush_menu_item.make_with_text ("Flush Cache")
			flush_menu_item.select_actions.extend (agent process_flush_cache)
			file_menu.extend (flush_menu_item)
			
			create close_menu_item.make_with_text ("Close")
			close_menu_item.select_actions.extend (agent process_cancel)
			file_menu.extend (close_menu_item)
			
			create exit_menu_item.make_with_text ("Exit")
			exit_menu_item.select_actions.extend (gui.destroy_application_agent)
			file_menu.extend (exit_menu_item)
			
			create prefs_menu_item.make_with_text ("Preferences")
			prefs_menu_item.select_actions.extend (agent process_edit_preferences)
			edit_menu.extend (prefs_menu_item)
			
			create raise_menu_item.make_with_text ("Raise Other Window")
			raise_menu_item.select_actions.extend (agent process_raise_other_window)
			window_menu.extend (raise_menu_item)
			
			create about_menu_item.make_with_text ("About")
			about_menu_item.select_actions.extend (agent show_version_dialog)
			help_menu.extend (about_menu_item)
			
			create a_menu_bar
			a_menu_bar.extend (file_menu)
			a_menu_bar.extend (edit_menu)
			a_menu_bar.extend (window_menu)
			a_menu_bar.extend (help_menu)
			set_menu_bar (a_menu_bar)
		end

	show_version_dialog
			-- Create and display the version dialog
		local
			version_dialog: DELTA_VERSION_DIALOG
		do
			create version_dialog.make
			version_dialog.show_modal_to_window (window)
		end
			
	initialize_toolbar_box
			-- Create and initialize box for toolbar
		local
			buttons: HORIZONTAL_BUTTON_SET
		do
			if session.is_read_only then
				create buttons.make_with_text_and_actions (<< Status_text, Refresh_text >>, << agent show_status_window, agent refresh >>)
			else
				create buttons.make_with_text_and_actions (<< Status_text, Refresh_text >>, << agent show_status_window, agent refresh_without_icons >>)
			end
			toolbar_box := buttons.column_set.box
			gui.set_box_background_recursive (toolbar_box, background_color)
		end

	show_status_window
			-- Create status window if it does not already
			-- exist.  Otherwise, just show it
		do
			if not status_built then
				status_interface.build
				status_interface.decline_actions.extend (agent process_status_decline)
				status_interface.subscribe_events
				status_interface.show
				status_built := True
			elseif status_interface.is_minimized then
				status_interface.restore
			elseif not status_interface.is_displayed then
				status_interface.show
			else
				status_interface.raise
			end
		end

	process_status_decline
			-- Process a decline request on run status window
		do
			status_interface.hide
			status_interface.enable_active
		end

	process_print_cancel (ifc: RC_MAP_PRINT_WINDOW)
			-- Process a cancel request on print interface window
		require
			ifc_not_void: ifc /= Void
		do
			ifc.hide
			ifc.enable_active
		end

	initialize_status_box
			-- Create and initialize box with run status info
		do
			create status_box
			create run_id_box
			create label.make_with_text (session.run_id)
			run_id_box.extend (label)
			create wafer_techn_box
			create label.make_with_text (run.wafer_techn)
			wafer_techn_box.extend (label)
			create run_type_box
			create label.make_with_text (run.run_type)
			run_type_box.extend (label)
			create run_closer_box
			create label.make_with_text (run.closer)
			run_closer_box.extend (label)
			create step_count_box
			refresh_step_count
			status_box.extend (run_id_box)
			status_box.extend (wafer_techn_box)
			status_box.extend (run_type_box)
			status_box.extend (run_closer_box)
			status_box.extend (step_count_box)
		end

	fill_tree_item (tree_item: EV_TREE_ITEM step: RC_STEP)
		require
			tree_item_exists: tree_item /= Void
			step_exists: step /= Void
		local
			list: LIST [RC_STEP]
			sub_item: EV_TREE_ITEM
			next_step: RC_STEP
		do
			from
				list := step.component_steps
				list.start
			until
				list.after
			loop
				next_step := list.item
				create sub_item.make_with_text (next_step.presentation_text)
				sub_item.set_data (next_step)
				tree_item.extend (sub_item)
				if step.is_composite then
					fill_tree_item (sub_item, next_step)
				end
				list.forth
			end
		end

	handle_step_state_change (event: RC_STEP_STATE_CHANGE_EVENT)
			-- Handle `event'
		require
			tree_exists: tree /= Void
			event_exists: event /= Void
		do
			update_all_icons
			refresh_step_count
		end

	update_all_icons
			-- Update icons for all nodes in `tree'
		require
			tree_exists: tree /= Void
		do
			from
				tree.start
			until
				tree.after
			loop
				update_all_icons_for_tree_node (tree.item)
				tree.forth
			end
		end

	update_all_icons_for_tree_node (node: EV_TREE_NODE)
			-- Update icons for everything in subtree
			-- starting at `node'
		require
			node_exists: node /= Void
		local
			step: RC_STEP
		do
			step ?= node.data
			check
				step_exists: step /= Void
			end
			set_pixmap (node, step)
			from
				node.start
			until
				node.after
			loop
				update_all_icons_for_tree_node (node.item)
				node.forth
			end
		end

	set_pixmap (tree_node: EV_TREE_NODE step: RC_STEP)
			-- Set appropriate pixmap for `tree_node' depending
			-- on state of `step'
		require
			tree_node_exists: tree_node /= Void
			step_exists: step /= Void
		do
			if step.is_done then
				if step.was_applicable_to_run then
					tree_node.set_pixmap (step_done_icon)
				else
					tree_node.set_pixmap (step_inapplicable_icon)
				end
			elseif step.is_undone then
				if step.is_soft_undone then
					tree_node.set_pixmap (step_saved_state_icon)
				else
					tree_node.set_pixmap (step_undone_icon)
				end
			elseif step.is_partially_undone then
				tree_node.set_pixmap (undoing_step_icon)
			elseif step.is_partially_done then
				tree_node.set_pixmap (doing_step_icon)
			else
			end
		end

	select_tree_item
			-- Action to be performed when tree item is selected
		local
			sel_item: EV_TREE_NODE
			step: RC_STEP
			prim_step: RC_PRIMITIVE_STEP
			is_done, is_undone, is_partially_undone: BOOLEAN
			is_soft_undone: BOOLEAN
		do
			if is_active then
				sel_item := tree.selected_item
				step ?= sel_item.data
				check
					step_exists: step /= Void
				end
				is_done := step.is_done
				is_undone := step.is_undone
				is_partially_undone := step.is_partially_undone
				is_soft_undone := step.is_soft_undone
				if not is_done and not is_partially_undone then
					do_button.enable_sensitive
				else
					do_button.disable_sensitive
				end
				prim_step ?= step
				if is_done and prim_step /= Void and then prim_step.is_modifiable and then prim_step.run_step.is_applicable then
					modify_button.enable_sensitive
				else
					modify_button.disable_sensitive
				end
				if not is_undone then
					redo_button.enable_sensitive
					undo_button.enable_sensitive
				else
					redo_button.disable_sensitive
					undo_button.disable_sensitive
				end
				if not is_undone or is_soft_undone then
					hard_undo_button.enable_sensitive
				else
					hard_undo_button.disable_sensitive
				end
				set_last_step
			end
		end

	deselect_tree_item
			-- Action to be performed when tree item is deselected
		do
			do_button.disable_sensitive
			modify_button.disable_sensitive
			redo_button.disable_sensitive
			undo_button.disable_sensitive
			hard_undo_button.disable_sensitive
		end

	refresh
			-- Re-read primitive step status information
			-- and update icons in tree to show current
			-- step statuses
		do
			session.set_primitive_step_status
			update_all_icons
			refresh_without_icons
		end

	refresh_without_icons
			-- Update status information, except for icons
			-- showing status of each step
		do
			fetch_run
			process_wafer_techn_change (run.wafer_techn)
			process_run_type_change (run.run_type)
			process_run_closer_change (run.closer)
			refresh_step_count
			if status_built then
				status_interface.refresh
			end
		end

	post_display_actions
		local
			screen: EV_SCREEN
		do
			create screen
			gui.do_once_on_idle (agent gui.center_window_with_size (window, 600, screen.height - 200))
			reset
		end

	set_last_step
			-- Set `last_step' to currently selected tree item's data
		local
			s: RC_STEP
		do
			s ?= tree.selected_item.data
			check
				step_exists: s /= Void
			end
			last_step := s
		end

	unfinished_step_count: INTEGER
			-- Number of steps that have not been finished
		local
			step: RC_PRIMITIVE_STEP
		do
			Result := primitive_steps.count
			from
				primitive_steps.start
			until
				primitive_steps.after
			loop
				step := primitive_steps.item
				if step.is_done then
					Result := Result - 1
				end
				primitive_steps.forth
			end
		end

	refresh_step_count
			-- Refresh count of steps remaining to be done
		do
			create label.make_with_text ("Steps left:  " + unfinished_step_count.out)
			step_count_box.wipe_out
			step_count_box.extend (label)
		end

	publish_selection
			-- Publish event that indicates user selected
			-- a particular choice
		local
			choice: SELECTED_CHOICE_EVENT
		do
			if is_active then
				create choice
				choice.set_value (tree.selected_item.text)
				choice.publish
			end
		end

	process_do
		do
			if is_active then
				disable_active
				publish_selection
				do_actions.call ([last_step])
			end
		end

	process_redo
		do
			if is_active then
				disable_active
				publish_selection
				redo_actions.call ([last_step])
			end
		end

	process_modify
		do
			if is_active then
				disable_active
				publish_selection
				modify_actions.call ([last_step])
			end
		end

	process_undo
		do
			if is_active then
				disable_active
				publish_selection
				undo_actions.call ([last_step])
			end
		end

	process_hard_undo
		do
			if is_active then
				disable_active
				publish_selection
				hard_undo_actions.call ([last_step])
			end
		end

	process_cancel
		do
			if is_active then
				disable_active
				publish_selection
				unsubscribe_events
				destroy_dependent_interfaces
				cancel_actions.call ([])
			end
		end

	destroy_dependent_interfaces
			-- Destroy any interfaces which depend on
			-- `Current'
		do
			if status_interface /= Void then
				status_interface.unsubscribe_events
				status_interface.destroy
			end
			if wafer_map_print_interface /= Void then
				wafer_map_print_interface.destroy
			end
			if bond_diagram_print_interface /= Void then
				bond_diagram_print_interface.destroy
			end
			if job_file_print_interface /= Void then
				job_file_print_interface.destroy
			end
			if preferences_interface /= Void then
				preferences_interface.destroy
			end
			if packing_interface_built then
				packing_interface.close
				destroy_packing_window
			end
		end

	process_print_maps (ifc: RC_MAP_PRINT_WINDOW)
			-- Process request to print maps via interface `ifc'
		require
			ifc_not_void: ifc /= Void
		do
			delta_version_checker.check_for_new_version
			if delta_version_checker.abort_requested then
				-- Nothing
			elseif not ifc.is_built then
				ifc.build
				ifc.cancel_actions.extend (agent process_print_cancel (ifc))
				ifc.show
			elseif ifc.is_minimized then
				ifc.refresh
				ifc.restore
			elseif not ifc.is_displayed then
				ifc.refresh
				ifc.show
			else
				ifc.raise
			end
		end

	process_view_packing
			-- Process request to view the packing for this run
			-- (read-only)
		local
			runs_row: RUNS_ROW
		do
			delta_version_checker.check_for_new_version
			if not delta_version_checker.abort_requested then
				session.cache.uncache_run
				runs_row := session.cache.runs_row
				if runs_row.is_null_packing_id or runs_row.is_null_packing_version_no then
					inform ("Cannot view packing because no packing is associated with run%Nor packing has no versions yet")
				else
					if not packing_interface_built then
						build_packing_interface
					end
					packing_interface.raise
				end
			end
		end

	build_packing_interface
			-- Build interface to view packing (read-only)
		local
			ifc: RC_PACKING_IFC
			pack_id: STRING
			l_window: EV_TITLED_WINDOW
		do
			pack_id := session.cache.runs_row.packing_id
			session.set_packing_id (pack_id)
			create l_window
			ifc := session.factory.packing_interface (l_window, session.run_id, session.packing_id, session.title_with_run_and_packing_id ("View"))
			ifc.set_session (session)
			ifc.set_packing_session (session)
			ifc.set_run_id (session.run_id)
			ifc.set_packing_id (pack_id)
			ifc.set_is_read_only (True)
			ifc.success_actions.extend (agent destroy_packing_window)
			ifc.cancel_actions.extend (agent destroy_packing_window)
			ifc.build
			packing_interface ?= ifc
			packing_interface_built := True
		end

	destroy_packing_window
			-- Destroy window used to view a packing
		do
			packing_interface.destroy
			packing_interface_built := False
		end

	process_flush_cache
			-- Process request to flush cache for session
		local
			op: ANONYMOUS_OPERATION
		do
			delta_version_checker.check_for_new_version
			if not delta_version_checker.abort_requested then
				create op.make (agent process_flush_cache_request)
				op.enable_abort_caught
				op.execute
			end
		end

	process_flush_cache_request
			-- Process request to flush cache for session
		do
			if question ("Clear database row cache?") then
				session.cache.uncache_all
				inform ("Database row cache cleared")
			end
		end

	process_edit_preferences
			-- Create preferences window if it does not already
			-- exist.  Otherwise, just show it
		do
			if not preferences_built then
				preferences_interface.build
				preferences_interface.enable_active
				preferences_interface.show
				preferences_built := True
			elseif preferences_interface.is_minimized then
				preferences_interface.enable_active
				preferences_interface.restore
			elseif not preferences_interface.is_displayed then
				preferences_interface.enable_active
				preferences_interface.refresh
				preferences_interface.show
			else
				preferences_interface.enable_active
				preferences_interface.raise
			end
		end

	process_raise_other_window
			-- Raise auxiliary window if it exists
		do
			session.application.raise_auxiliary_window
		end

	handle_wafer_techn_change (event: WAFER_TECHN_CHANGE_EVENT)
		do
			process_wafer_techn_change (event.wafer_techn)
		end

	handle_run_type_change (event: RUN_TYPE_CHANGE_EVENT)
		do
			process_run_type_change (event.run_type)
		end

	handle_run_closer_change (event: RUN_CLOSER_CHANGE_EVENT)
		do
			process_run_closer_change (event.run_closer)
		end

	process_wafer_techn_change (wt: STRING)
		do
			create label.make_with_text (wt)
			wafer_techn_box.wipe_out
			wafer_techn_box.extend (label)
			set_status_background
		end

	process_run_type_change (rt: STRING)
		do
			create label.make_with_text (rt)
			run_type_box.wipe_out
			run_type_box.extend (label)
			set_status_background
		end

	process_run_closer_change (closer: STRING)
		do
			create label.make_with_text (closer)
			run_closer_box.wipe_out
			run_closer_box.extend (label)
			set_status_background
		end

	fetch_run
			-- Fetch RUNS row from database (do not use cache)
		do
			run := session.cache.db_runs_row
		end

	set_status_background
		do
			-- FIXME: add back in?
			-- gui.set_box_background_recursive (upper_bar, background_color)
		end

	run: RUNS_ROW
			-- Row from RUNS table

	step_done_icon: EV_PIXMAP
			-- Icon that indicates step has been completed

	step_inapplicable_icon: EV_PIXMAP
			-- Icon that indicates step was not applicable to run

	step_undone_icon: EV_PIXMAP
			-- Icon that indicates step is not even started

	doing_step_icon: EV_PIXMAP
			-- Icon that indicates we have started doing step

	undoing_step_icon: EV_PIXMAP
			-- Icon that indicates we have started undoing step

	step_saved_state_icon: EV_PIXMAP
			-- Icon that indicates that step has been completely
			-- undone, but we have saved state from the last
			-- time we did it

	toolbar_box: EV_HORIZONTAL_BOX
			-- Box for toolbar

	status_box: EV_HORIZONTAL_BOX
			-- Box to hold basic run status information

	run_id_box: EV_HORIZONTAL_BOX
			-- Box to hold run id

	wafer_techn_box: EV_HORIZONTAL_BOX
			-- Box to hold wafer_techn

	run_type_box: EV_HORIZONTAL_BOX
			-- Box to hold run type

	run_closer_box: EV_HORIZONTAL_BOX
			-- Box to hold run closer

	step_count_box: EV_HORIZONTAL_BOX
			-- Box to hold number of steps left to do

	label: EV_LABEL
			-- For construction of labels

	status_interface: RC_STATUS_WINDOW
			-- Window to show run status

	status_built: BOOLEAN
			-- Has run status interface been built yet?

	wafer_map_print_interface: RC_MAP_PRINT_WINDOW
			-- Window to print wafer maps

	bond_diagram_print_interface: RC_MAP_PRINT_WINDOW
			-- Window to print bonding diagrams

	job_file_print_interface: RC_MAP_PRINT_WINDOW
			-- Window to print job files

	preferences_interface: EDIT_PREFERENCES_WINDOW
			-- Window to allow user to edit preferences for session

	preferences_built: BOOLEAN
			-- Has preferences interface been built yet?

	packing_interface: RC_PACKING_WINDOW
			-- Window to allow user to view packing for run

	packing_interface_built: BOOLEAN
			-- Has packing interface been built yet?

	exit_menu_item: EV_MENU_ITEM
			-- File/Exit menu item
	
	close_menu_item: EV_MENU_ITEM
			-- File/Close menu item
	
	flush_menu_item: EV_MENU_ITEM
			-- File/Flush Cache menu item
	
	view_packing_menu_item: EV_MENU_ITEM
			-- File/View Packing menu item
	
	Refresh_text: STRING = "Refresh"

	Status_text: STRING = "Status"

end
