
indexing
	
	description: "Text field, with a descriptive label, that is %
		% associated with a specific run closing primitive step"
	author: "David Hollenberg"
	date: "May 8, 2003"

class RC_STEP_TEXT_FIELD

create
	make

feature {NONE} -- Creation

	make (f: EV_TEXT_FIELD a_label: STRING s: RC_PRIMITIVE_STEP sess: RC_RUN_CLOSING_SESSION)
			-- Create with `a_label' and associated step `s',
			-- using `f' as text field
		require
			field_exists: f /= Void
			a_label_exists: a_label /= Void
			session_exists: sess /= Void
		do
			text_field := f
			label := a_label
			step := s
			session := sess
			compute_applicable
			compute_all_applicability_steps_done
		end

feature -- Properties

	session: RC_RUN_CLOSING_SESSION
			-- Session used to close run
	
	text_field: EV_TEXT_FIELD
			-- Text field
	
	label: STRING
			-- Text that describes contents of field

	step: RC_PRIMITIVE_STEP
			-- Step whose execution sets contents of `text_field',
			-- if any

	is_applicable: BOOLEAN
			-- Is text field set by step that is known
			-- to be applicable?

	is_always_displayed: BOOLEAN
			-- Should text field always be displayed?

	is_display_requested: BOOLEAN
			-- Should text field be displayed?
		do
			Result := is_applicable or is_always_displayed
		end

	all_applicability_steps_done: BOOLEAN
			-- Are all steps required to determine applicability
			-- of `step' done?

feature -- Modification

	set_always_displayed (b: BOOLEAN)
			-- Set `is_always_displayed' to `b'
		do
			is_always_displayed := b
		ensure
			value_set: is_always_displayed = b
		end

feature -- Operations

	compute_applicable
			-- Compute whether step is applicable
		local
			op: RC_PRIMITIVE_STEP_OP
		do
			if step = Void then
				is_applicable := True
			elseif not step.all_applicability_steps_done then
				is_applicable := False
			else
				op := session.primitive_step_implementation (step)
				is_applicable := op.is_applicable_to_run
			end
		end

	compute_all_applicability_steps_done
			-- Compute whether all steps required to determine
			-- applicability of `step' are done
		do
			if step /= Void then
				 all_applicability_steps_done := step.all_applicability_steps_done
			else
				all_applicability_steps_done := True
			end
		end

end
