
indexing
	
	description: "Run closing utilities"
	author: "David Hollenberg"
	date: "March 7, 2003"

class RC_UTILITIES
inherit
	STRING_UTILITIES_ACCESS
		export
			{NONE} all
		end
	CONDITION_UTILITIES
	DOUBLE_MATH
		export
			{NONE} all
		end

feature -- Queries

	Initial_mask_revision: CHARACTER = 'A'
			-- Mask revision for first set of masks for run
	
	is_possible_real_run_id (s: STRING): BOOLEAN
			-- May `s' be used as a real run id?
		require
			run_id_exists: s /= Void
		do
			Result := s.count = ({RUNS_ROW}.Maxlen_run_id) and then s.item (1) /= Trial_run_first_char and su.is_identifier (s)
		end
	
	is_possible_trial_run_id (s: STRING): BOOLEAN
			-- May `s' be used as a trial run id?
		require
			run_id_exists: s /= Void
		local
			real_run: STRING
		do
			if not s.is_empty then
				real_run := s.twin
				real_run.put ('A', 1)
			end
			Result := s.count = ({RUNS_ROW}.Maxlen_run_id) and then s.item (1) = Trial_run_first_char and su.is_identifier (real_run)
		end
	
	is_standard_real_run_id (s: STRING): BOOLEAN
			-- Does `s' conform to standard MOSIS naming
			-- conventions for a real run id?
		require
			run_id_exists: s /= Void
		local
			c1, c2, c3, c4: CHARACTER
		do
			if s.count = 4 then
				c1 := s.item (1)
				c2 := s.item (2)
				c3 := s.item (3)
				c4 := s.item (4)
			end
			Result := s.count = 4 and then
			  (c1 = 'T' and c2.is_digit and
			  (c3.is_digit or c3 = 'A' or c3 = 'B' or c3 = 'C') and
			   c4.is_alpha and c4 /= 'I' and c4 /= 'O' and
			   c4 /= 'N')
		end
	
	is_standard_trial_run_id (s: STRING): BOOLEAN
			-- Does `s' conform to standard MOSIS naming
			-- conventions for a trial run id?
		require
			run_id_exists: s /= Void
		local
			c1, c2, c3, c4: CHARACTER
		do
			if s.count = 4 then
				c1 := s.item (1)
				c2 := s.item (2)
				c3 := s.item (3)
				c4 := s.item (4)
			end
			Result := s.count = 4 and then
			  (c1 = '0' and c2.is_digit and
			  (c3.is_digit or c3 = 'A' or c3 = 'B' or c3 = 'C') and
			   c4.is_alpha and c4 /= 'I' and c4 /= 'O' and
			   c4 /= 'N')
		end
	
	looks_like_trial_run_id (s: STRING): BOOLEAN
			-- Does `s' look like a run id for a trial run?
		require
			run_id_exists: s /= Void
		do
			Result := not s.is_empty and then s.item (1) = Trial_run_first_char
		end
	
	least_common_multiple_list (list: LIST [DOUBLE]): DOUBLE
			-- Least common multiple of number in `list'
			-- Result has the property that each number in the
			-- list times some integer is 'close to' result
        		-- and there is no number less than Result which
			-- has this same property.
		require
			list_exists: list /= Void
			list_nonempty: not list.is_empty
			-- All elements of list are positive
		do
			Result := list.first
			from
				list.start
			until
				list.after
			loop
				Result := least_common_multiple (Result, list.item)
				list.forth
			end
		end

	least_common_multiple (d1, d2: DOUBLE): DOUBLE
			-- Least common multiple of `d1' and `d2'
		require
			first_number_positive: d1 > 0.0
			second_number_positive: d2 > 0.0
		local
			k: INTEGER
		do
			Result := d1
			from
			 	k := 1
			until
				k > 100000 or close_to_integer (Result / d2)
			loop
				k := k + 1
				Result := k * d1
			end
			if k > 100000 then
				handle_hard_error ("cannot find LCM of " + d1.out + " and " + d2.out)
			end
		end

	close_to_integer (d: DOUBLE): BOOLEAN
			-- Is `d' "close to" an integer?
		do
			if d = 0.0 then
				Result := True
			else
				Result := (d.rounded - d).abs < Double_threshold
			end
		end

feature {NONE} -- Implementation

	Trial_run_first_char: CHARACTER = '0'

	Double_threshold: DOUBLE = 1.0E-6

end
