
indexing
	
	description: "A run closing step operation that inserts a %
		%RELEASED TO MANUFACTURING event"
	author: "David Hollenberg"
	date: "June 3, 2005"

class RELEASE_TO_MANUFACTURING_OP
inherit
	RUN_EVENT_INSERTION_OP

create
	make

feature -- Properties

	input_data_items: LIST [STRING]
			-- Data items required before step can be executed
		do
			Result := array_as_list (<<
				data_item.Sent_submission_messages
				 >>)
		end

feature {NONE} -- Implementation

	get_final_confirmation (d: DB_DATE)
			-- Get final confirmation that event should really
			-- be inserted and notify appropriate parties
		do
			confirm ("Run will be marked as %"released to manufacturing%" on " + d.to_date_string + "%Nand e-mail notifications will be sent to internal MOSIS mailing lists.%N%NProceed")
			session.commit_actions.extend (agent publish_run_rtm_event (d))
		end

	publish_run_rtm_event (d: DB_DATE)
			-- Publish a "run released to manufacturing" event
		require
			d_not_void: d /= Void
		local
			event: RUN_RTM_EVENT
			wt, subj_wt: STRING
			inv_event: DED_MASK_INVOICE_EVENT
			acct_string: STRING
		do
			wt := "(" + cache.wafer_techn + ")"
			create event
			event.set_subject (wt + " released to manufacturing")
			event.set_message ("Run " + session.run_id + " " + wt + " was released to manufacturing on " + d.to_date_string + ".")
			event.publish
			
			if cache.is_dedicated_run then
			-- we should really check and only issue this message if there are line items
			-- that needs invoicing.
				acct_string := fab_run_paying_accounts
				subj_wt := "(" + cache.wafer_techn + " - " + acct_string + ")"
				create inv_event
				inv_event.set_subject (subj_wt + " - invoice for masks")
				inv_event.set_message ("Dedicated run " + session.run_id + " " + wt + " was released to manufacturing on " + d.to_date_string + ".%NPlease invoice account " + acct_string + " for the mask line items (Initial Invoice).")
				inv_event.publish
			end
		end

	fab_run_paying_accounts: STRING
			-- Comma-separated list of account strings of
			-- accounts which have an order on this production
			-- and also have a line item for a FAB RUN part
		local
			list: LIST [STRING]
			b: DB_BIND_SS
			part_type: PART_TYPE_VALUES
		do
			create b.make (session.run_id, part_type.Fab_run)
			list := connection.selected_strings ("m_lookup.cust_acct_display_name(cwo.cust_acct_no)",
			 "cust_wos cwo, cust_wo_line_items woli, m_parts mp",
			 "cwo.m_order_no = woli.m_order_no and %
			 % woli.m_part_no = mp.m_part_no and %
			 % cwo.production_id = :1 and mp.part_type = :2",
			 Void, True, b)
			Result := su.joined_by (list, ", ")
			if Result.is_empty then
				Result := "(undetermined account)"
			end
		end

	event_name: STRING
			-- Name of event to be inserted by operation
		local
			n: EVENT_NAME_VALUES
		do
			Result := n.Released_to_manufacturing
		end

	date_name: STRING = "Date Released to Manufacturing"
			-- Name of date user is asked to enter

	add_undo_commit_actions
			-- Add any required session commit actions for undo
		do
		end

	check_date_validity (d: DB_DATE)
			-- Check that `d' is valid for `event_name'
		local
			tapeout_date: DB_DATE
		do
			tapeout_date := cache.mask_sets_row.date_tapeout
			if d < tapeout_date then
				handle_user_error ("Date cannot be before tapeout date (" + tapeout_date.to_date_string + ")")
			end
		end

	max_undo_allowed_days: INTEGER
			-- Maximum age of event in days to allow undo
			-- of step
		do
			Result := policies.rtm_undo_allowed_interval
		end

end
