
indexing
	
	description: "A run closing step operation that renames %
		%MEBES files to be sent to the mask shop to the names %
		%required by the mask shop"
	author: "David Hollenberg"
	date: "September 30, 2004"

class RENAME_MEBES_OP
inherit
	RC_PRIMITIVE_STEP_OP
	NO_USER_INTERFACE
	CREATES_EXTERNAL_STATE
	OS_ACCESS
		export
			{NONE} all
		undefine
			default_create
		end
	UNIX_OS_ACCESS
		export
			{NONE} all
		undefine
			default_create
		end
	FILE_UTILITIES_ACCESS
		export
			{NONE} all
		undefine
			default_create
		end

create
	make

feature -- Properties

	input_data_items: LIST [STRING]
			-- Data items required before step can be executed
		do
			Result := array_as_list (<<
				data_item.Filled_mebes_files
				>>)
		end

feature {NONE} -- Implementation

	is_applicable: BOOLEAN
			-- Is step applicable to run for session?
		do
			Result := not cache.litho_sizes_row.is_null_pattern_file_renaming_style
		end

	do_step
		local
			style: PATTERN_FILE_RENAMING_STYLE_VALUES
		do
			renaming_style := cache.pattern_file_renaming_style
			if renaming_style.is_equal (style.Ami_part_nos) then
				process_ami_rename
			else
				handle_hard_error ("unknown pattern file renaming style " + renaming_style)
			end
		end

	undo_step
			-- Undo execution of step
		local
			pf_mrows: like pattern_file_mrows
			lock_op: ROW_LOCK_OP
		do
			set_file_directory
			pf_mrows := pattern_file_mrows
			create lock_op
			lock_op.lock_list_nowait (pf_mrows)
			pf_mrows.do_all (agent undo_rename_ami_pattern_file)
			add_commit_action (agent publish_new_value (""))
			call_completion_actions
		end

	process_ami_rename
			-- Process AMI part numbers style MEBES file rename
		do
			cache.check_files_merged
			compute_ami_file_root
			rename_ami_pattern_files
			add_commit_action (agent publish_new_value (file_root))
			call_completion_actions
		end

	set_file_directory
			-- Set `file_directory'
		do
			file_directory := file_generator.level_merge_pattern_file (1, 1, "XX").directory_pathname
		end

	compute_ami_file_root
			-- Compute new root name for pattern files to
			-- be sent to mask shop for AMI part numbers renaming
			-- style
		local
			part_id: STRING
			part_no: INTEGER
		do
			part_id := cache.wafer_part_id
			if part_id.is_empty then
				handle_hard_error ("wafer part id is empty")
			elseif not part_id.is_integer then
				handle_hard_error ("wafer part id " + su.double_quoted (part_id) + " is not an integer")
			else
				part_no := part_id.to_integer
				if part_no < 19000 or part_no >= 21000 then
					handle_hard_error ("wafer part id " + su.double_quoted (part_id) + " not in range (19000, 20999).%NDon't know how to compute new root file name")
				else
					if part_no < 20000 then
						file_root := "J"
					elseif part_no < 21000 then
						file_root := "K"
					else
						handle_internal_error ("bug in routine `compute_file_root'")
					end
					file_root.append (part_id.substring (3, 5))
					file_root.append ("000A5")
					file_root.to_lower
				end
			end
			
		end

	rename_ami_pattern_files
			-- Rename final pattern files to be sent to mask
			-- shop for AMI part numbers style renaming
		local
			pf_mrows: like pattern_file_mrows
			lock_op: ROW_LOCK_OP
		do
			set_file_directory
			pf_mrows := pattern_file_mrows
			create lock_op
			lock_op.lock_list_nowait (pf_mrows)
			pf_mrows.do_all (agent rename_ami_pattern_file)
		end

	rename_ami_pattern_file (row: PATTERN_FILES_MROW)
			-- Rename final pattern file for `row'
			-- using AMI part numbers style renaming
		local
			symlink_op: SYMLINK_CREATE_OP
			fname: STRING
		do
			row.set_renamed_pattern_filename_from_root (file_root)
			fname := os.full_file_name (file_directory, row.renamed_pattern_filename)
			remove_link_if_exists (fname)
			
			create symlink_op
			symlink_op.create_symlink (row.pattern_filename, fname)
			set_internal_mebes_file_name (fname)
		end

	undo_rename_ami_pattern_file (row: PATTERN_FILES_MROW)
			-- Undo rename of final pattern file for `row'
			-- (delete symbolic link and reset name inside file
			-- to original name)
		local
			fname, orig_fname: STRING
		do
			if not row.is_null_renamed_pattern_filename then
				fname := os.full_file_name (file_directory, row.renamed_pattern_filename)
				remove_link_if_exists (fname)
				
				orig_fname := os.full_file_name (file_directory, row.pattern_filename)
				set_internal_mebes_file_name (orig_fname)
				row.set_null_renamed_pattern_filename
			end
		end

	remove_link_if_exists (fname: STRING)
			-- Remove `fname' if it exists and is a symbolic link.
			-- If it exists, but is not a symbolic link, signal
			-- an error
		local
			unlink_op: LINK_DELETE_OP
			checker: FILE_EXISTENCE_CHECK_OP
		do
			create checker
			checker.check_existence (fname)
			if checker.path_exists then
				if unix_os.is_symbolic_link (fname) then
					create unlink_op
					unlink_op.delete_link (fname)
				else
					handle_hard_error ("file " + fname + " expected to be a symbolic link but is not")
				end
			end
		end

	set_internal_mebes_file_name (fname: STRING)
			-- Set internal name in MEBES file `fname' based
			-- on `fname'
		local
			file: LARGE_RAW_FILE
			open_op: FILE_OPEN_READ_WRITE_OP
			close_op: FILE_CLOSE_OP
		do
			create file.make (fname)
			create open_op
			open_op.open (file)
			fu.update_name_in_open_mebes_header (file, Void)
			create close_op
			close_op.close (file)
		end

	pattern_file_mrows: LIST [PATTERN_FILES_MROW]
			-- Rows in pattern files table for files to be
			-- sent to mask shop
		local
			row: PATTERN_FILES_MROW
		do
			create row.make_template
			Result := row.final_rows_for_run (session.run_id, connection, expected_count.At_least_zero)
		end

	set_state
			-- Set `state'
		do
			state := Void
		end

	publish_new_value (s: STRING)
			-- Publish new value of MEBES file name root
		require
			root_exists: s /= Void
		local
			e: FILE_ROOT_CHANGE_EVENT
		do
			create e
			e.set_root (s)
			e.publish
		end

	renaming_style: STRING
			-- Pattern file renaming style for run

	file_root: STRING
			-- Requested root name for final MEBES files

	file_directory: STRING
			-- Directory where original and renamed pattern
			-- files reside.

end
