
indexing
	
	description: "Utilities for executing ReportWriter reports"
	author: "David Hollenberg"
	date: "May 26, 2004"

class REPORT_EXECUTION
inherit
	ANY
	CONDITION_UTILITIES
		export
			{NONE} all
		end
	STRING_UTILITIES_ACCESS
		export
			{NONE} all
		end
	UNIX_OS_ACCESS
		export
			{NONE} all
		end

feature -- Operations

	run_report_with_umask (conn: DB_CONNECTION mask: INTEGER prog, ora_home, temp_dir, module, parms, dest: STRING recomp: BOOLEAN disp_name: STRING)
			-- Run ReportWriter program `prog' for report
			-- `module' with `parms' and put output in
			-- `dest', using `ora_home' as the ORACLE_HOME
			-- if not Void.  Use the login information
			-- (username, password, service name) from
			-- `conn'.  If `recomp', then recompile report
			-- if needed before execution.  If `disp_name' is
			-- not Void, set DISPLAY environment variable from
			-- its value.  Set umask to
			-- `mask' before report is run and reset to
			-- original value afterwards
		require
			connection_exists: conn /= Void
			program_exists: prog /= Void
			program_not_empty: not prog.is_empty
			ora_home_void_or_nonempty: ora_home = Void or else not ora_home.is_empty
			temp_dir_exists: temp_dir /= Void
			temp_dir_not_empty: temp_dir /= Void
			module_exists: module /= Void
			parms_exists: parms /= Void
			dest_exists: dest /= Void
			dest_not_empty: not dest.is_empty
		local
			old_mask, new_mask: INTEGER
		do
			new_mask := mask
			old_mask := unix_os.set_file_creation_mask (new_mask)
			run_report (conn, prog, ora_home, temp_dir, module, parms, dest, recomp, disp_name)
			new_mask := unix_os.set_file_creation_mask (old_mask)
		rescue
			new_mask := unix_os.set_file_creation_mask (old_mask)
		end

	run_report (conn: DB_CONNECTION prog, ora_home, temp_dir, module, parms, dest: STRING recomp: BOOLEAN disp_name: STRING)
			-- Run ReportWriter program `prog' for report
			-- `module' with `parms' and put output in
			-- `dest', using `ora_home' as the ORACLE_HOME
			-- if not Void.  Use the login information
			-- (username, password, service name) from
			-- `conn'.  If `recomp', then recompile report
			-- if needed before execution.  If `disp_name' is
			-- not Void, set DISPLAY environment variable from
			-- its value
		require
			connection_exists: conn /= Void
			program_exists: prog /= Void
			program_not_empty: not prog.is_empty
			ora_home_void_or_nonempty: ora_home = Void or else not ora_home.is_empty
			temp_dir_exists: temp_dir /= Void
			temp_dir_not_empty: temp_dir /= Void
			module_exists: module /= Void
			parms_exists: parms /= Void
			dest_exists: dest /= Void
			dest_not_empty: not dest.is_empty
		local
			report: REPORTWRITER_REPORT
		do
			create report
			report.set_reportwriter_file (prog)
			report.set_oracle_home (ora_home)
			report.set_username (conn.user_name)
			report.set_password (conn.password)
			report.set_service_name (conn.connect_string)
			report.set_destination_type_file
			report.set_batch (True)
			report.set_use_recursive_load (recomp)
			report.set_parameter_form_requested (False)
			report.set_print_dialog_requested (False)
			report.set_temp_directory (temp_dir)
			report.set_module_name (module)
			report.set_parameters (parms)
			report.set_destination_name (dest)
			report.set_display_name (disp_name)
			report.execute
			if not report.ok then
				handle_hard_error ("Errors encountered while running report " + module + "%N" + su.joined_by (report.errors, "%N"))
			end
		end

end
