
indexing
	
	description: "An executable Oracle Reportwriter report, which can %
		%be run to produce an output file "
	author: "David Hollenberg"
	date: "April 6, 2004"

class REPORTWRITER_REPORT
inherit
	ANY
	STRING_UTILITIES_ACCESS
		export
			{NONE} all
		end
	UNIX_OS_ACCESS
		export
			{NONE} all
		end
	EXECUTION_ENVIRONMENT_ACCESS
		export
			{NONE} all
		end
	STAT_CONSTANTS
		export
			{NONE} all
		end

create
	default_create

feature -- Properties

	reportwriter_file: STRING
			-- Name of file with Reportwriter executable

	oracle_home: STRING
			-- Value for ORACLE_HOME environment variable
			-- needed to run Reportwriter.  If Void,
			-- environment variable value is not changed
			-- before running report

	module_name: STRING
			-- Name of module to be executed

	username: STRING
			-- Oracle username for login

	password: STRING
			-- Oracle password for login

	service_name: STRING
			-- Oracle service name for login

	parameters: STRING
			-- Parameters specific to this report

	parameter_form_requested: BOOLEAN
			-- Should report bring up form to enter parameters?

	print_dialog_requested: BOOLEAN
			-- Should print dialog be brought up?

	destination_type: STRING
			-- Type of destination for report (file or screen)

	destination_name: STRING
			-- Name of destination for report (determined by
			-- report itself if Void)

	temp_directory: STRING
			-- Name of directory for temporary file with
			-- parameters to run report writer

	is_batch: BOOLEAN
			-- Should report be run in batch mode?

	use_recursive_load: BOOLEAN
			-- Should we use recursive load (undocumented
			-- parameter)?

	display_name: STRING
			-- Value to set for DISPLAY environment variable,
			-- if not Void (needed due to broken way report
			-- writer works)

	ok: BOOLEAN
			-- Was last call to `execute' successful?
		do
			Result := errors /= Void and then errors.is_empty
		end

	errors: LIST [STRING]
			-- Errors found by last call to `execute'

feature -- Modification

	set_reportwriter_file (s: STRING)
			-- Set `reportwriter_file' to `s'
		require
			value_exists: s /= Void
			value_not_empty: not s.is_empty
		do
			reportwriter_file := s
		end

	set_oracle_home (s: STRING)
			-- Set `oracle_home' to `s'
		require
			value_void_or_nonempty: s = Void or else not s.is_empty
		do
			oracle_home := s
		end

	set_module_name (s: STRING)
			-- Set `module_name' to `s'
		require
			value_exists: s /= Void
			value_not_empty: not s.is_empty
		do
			module_name := s
		end

	set_username (s: STRING)
			-- Set `username' to `s'
		require
			value_exists: s /= Void
			value_not_empty: not s.is_empty
		do
			username := s
		end

	set_password (s: STRING)
			-- Set `password' to `s'
		require
			value_exists: s /= Void
			value_not_empty: not s.is_empty
		do
			password := s
		end

	set_service_name (s: STRING)
			-- Set `service_name' to `s'
		require
			value_exists: s /= Void
			value_not_empty: not s.is_empty
		do
			service_name := s
		end

	set_parameters (s: STRING)
			-- Set `parameters' to `s'
		do
			parameters := s
		end

	set_parameter_form_requested (b: BOOLEAN)
			-- Set `parameter_form_requested' to `b'
		do
			parameter_form_requested := b
		end

	set_print_dialog_requested (b: BOOLEAN)
			-- Set `print_dialog_requested' to `b'
		do
			print_dialog_requested := b
		end

	set_destination_type (s: STRING)
			-- Set `destination_type' to `s'
		do
			destination_type := s
		end

	set_destination_type_file
			-- Set `destination_type' to "file"
		do
			set_destination_type ("file")
		end

	set_destination_type_screen
			-- Set `destination_type' to "screen"
		do
			set_destination_type ("screen")
		end

	set_destination_name (s: STRING)
			-- Set `destination_name' to `s'
		do
			destination_name := s
		end

	set_temp_directory (s: STRING)
			-- Set `temp_directory' to `s'
		do
			temp_directory := s
		end

	set_batch (b: BOOLEAN)
			-- Set `is_batch' to `b'
		do
			is_batch := b
		end

	set_use_recursive_load (b: BOOLEAN)
			-- Set `use_recursive_load' to `b'
		do
			use_recursive_load := b
		end

	set_display_name (s: STRING)
			-- Set `display_name' to `s'
		do
			display_name := s
		end

feature -- Operations

	execute
			-- Execute report.  Set DISPLAY environment
			-- variable to value of `display_name', if not Void
		require
			username_exists: username /= Void
			password_exists: password /= Void
			service_name_exists: service_name /= Void
			reportwriter_exists: reportwriter_file /= Void
			reportwriter_not_empty: not reportwriter_file.is_empty
			module_name_exists: module_name /= Void
			parameters_exists: parameters /= Void
			temp_dir_exists: temp_directory /= Void
		local
			fname, output, old_ora_home: STRING
			f: LARGE_RAW_FILE
			process: UNIX_PROCESS
			delete_op: FILE_DELETE_OP
			checker: FILE_EXISTENCE_CHECK_OP
			out_file: LARGE_RAW_FILE
			exists: BOOLEAN
			old_mod_time, new_mod_time: INTEGER
		do
			if display_name /= Void then
				exe.put (display_name, Display_env_name)
			end
			
			fname := unix_os.full_file_name (temp_directory, "report_parameters_" + unix_os.my_process_id.out + ".txt")
			create delete_op
			delete_op.delete_if_exists (fname)
			if destination_name /= Void and then not destination_name.is_empty then
				create checker
				checker.check_existence (destination_name)
				exists := checker.path_exists
				if exists then
					create f.make (destination_name)
					old_mod_time := f.date
				end
			end

			create_parameter_file (fname)
			create {ARRAYED_LIST [STRING]} errors.make (0)
			create f.make (reportwriter_file)
			if not f.exists then
				errors.extend ("File " + reportwriter_file + " not found")
			elseif not f.is_executable then
				errors.extend ("File " + reportwriter_file + " is not executable")
			else
				create process.make (reportwriter_file)
				process.set_process_name (module_name)
				process.set_arguments (<< "cmdfile=" + fname >>)
		    		process.set_output_piped
		    		process.set_error_same_as_output
				old_ora_home := exe.get (Oracle_home_env_name)
				exe.put (oracle_home, Oracle_home_env_name)
		    		process.spawn_nowait
				if old_ora_home /= Void then
					exe.put (old_ora_home, Oracle_home_env_name)
				end
				
				-- Read all output
				from
					out_file := process.output_from_child
					create output.make (0)
				until
					out_file.end_of_file
				loop
					out_file.read_stream (8192)
					output.append (out_file.last_string)
				end

		    		process.get_status_block
		    		if not process.is_success_exit then
					errors.extend ("Reportwriter " + process.status_description (process.status))
				end
				
				if destination_name /= Void and then not destination_name.is_empty then
					create checker
					checker.check_existence (destination_name)
					if not checker.path_exists then
						errors.extend ("Output report file not found: " + destination_name)
					elseif exists then
						create f.make (destination_name)
						new_mod_time := f.date
						if new_mod_time = old_mod_time then
							errors.extend ("Output report file modification date did not change: " + destination_name)
						end
					end
				end
				if not output.is_empty then
					errors.extend ("Reportwriter output not empty.  Output was:%N" + output)
				end
			end
			delete_op.delete_if_exists (fname)
		rescue
			if old_ora_home /= Void then
				exe.put (old_ora_home, Oracle_home_env_name)
			end
		end

feature {NONE} -- Implementation

	create_parameter_file (fname: STRING)
			-- Create file `fname' with parameters to run report
		require
			fname_exists: fname /= Void
		local
			old_mask, new_mask: INTEGER
			file_opener: FILE_OPEN_WRITE_OP
			file_closer: FILE_CLOSE_OP
			f: LARGE_PLAIN_TEXT_FILE
		do
			new_mask := (S_irusr | S_iwusr).bit_not
			old_mask := unix_os.set_file_creation_mask (new_mask)
			create f.make (fname)
			create file_opener
			file_opener.open (f)
			f.put_string ("userid=" + username + "/" + password + "@" + service_name + "%N")
			f.put_string ("module=" + module_name + "%N")
			f.put_string ("batch=" + su.boolean_as_yes_no (is_batch) + "%N")
			f.put_string ("recursive_load=" + su.boolean_as_yes_no (use_recursive_load) + "%N")
			if destination_type /= Void then
				f.put_string ("destype=" + destination_type + "%N")
			end
			if destination_name /= Void then
				f.put_string ("desname=" + destination_name + "%N")
			end
			f.put_string ("paramform=" + su.boolean_as_yes_no (parameter_form_requested) + "%N")
			f.put_string ("printjob=" + su.boolean_as_yes_no (print_dialog_requested) + "%N")
			if parameters /= Void and then not parameters.is_empty then
				f.put_string (parameters + "%N")
			end
			create file_closer
			file_closer.close (f)
			new_mask := unix_os.set_file_creation_mask (old_mask)
		rescue
			new_mask := unix_os.set_file_creation_mask (old_mask)
		end

	Oracle_home_env_name: STRING = "ORACLE_HOME"
			-- Name of Oracle home environment variable
	
	Display_env_name: STRING = "DISPLAY"
			-- Name of DISPLAY environment variable
end

