
indexing
	
	description: "Facilities for dealing with Unix resource limits"
	author: "David Hollenberg"
	date: "March 7, 2002"

class RESOURCE_LIMITS

inherit
	RESOURCE_LIMIT_CONSTANTS
	UNIX_OS_ACCESS
		export
			{NONE} all
		end

feature

	set_soft_resource_limit (code, val: INTEGER)
			-- Set soft resource limit for resource with code
			-- `code' to `val'
		do
			unix_os.set_soft_resource_limit (code, val)
		end

	set_soft_resource_limit_to_max (code: INTEGER)
			-- Set soft resource limit for resource with code
			-- `code' to `val'
		do
			unix_os.set_soft_resource_limit_to_max (code)
		end

	set_all_soft_resource_limits_to_max
			-- Set soft resource limit for all resources to
			-- maximum value (except set maximum size of core file
			-- to 0)
		do
			unix_os.set_soft_resource_limit (Rlimit_core, 0)
			unix_os.set_soft_resource_limit_to_max (Rlimit_cpu)
			unix_os.set_soft_resource_limit_to_max (Rlimit_data)
			unix_os.set_soft_resource_limit_to_max (Rlimit_fsize)
			unix_os.set_soft_resource_limit_to_max (Rlimit_nofile)
			unix_os.set_soft_resource_limit_to_max (Rlimit_stack)
			unix_os.set_soft_resource_limit_to_max (Rlimit_vmem)
		end

	set_hard_resource_limit (code, val: INTEGER)
			-- Set hard resource limit for resource with code
			-- `code' to `val'
		do
			unix_os.set_hard_resource_limit (code, val)
		end

feature -- Properties

	soft_resource_limit (code: INTEGER): INTEGER
			-- Soft resource limit for resource with code
			-- `code'
		do
			Result := unix_os.soft_resource_limit (code)
		end

	hard_resource_limit (code: INTEGER): INTEGER
			-- Hard resource limit for resource with code
			-- `code'
		do
			Result := unix_os.hard_resource_limit (code)
		end

end
