
indexing
	
	description: "Change applied to reticle during packing that creates %
		%or deletes elements that are dies"
	author: "David Hollenberg"
	date: "January 25, 2007"

deferred class RETICLE_CREATION_OR_DELETION_CHANGE
inherit
	RETICLE_CHANGE
		redefine
			undo_simple_change, redo_simple_change
		end
	
feature -- Operations

	add_fully_deleted_die (a_die: PACKING_DIES_MROW)
		do
			dies_table.search (a_die)
			if not dies_table.found then
				dies_table.put (1, a_die)
				fully_deleted_dies.extend (a_die)
			end
		end

	add_fully_deleted_element (an_element: RETICLE_VIEW_ELEMENT)
		do
			fully_deleted_elements.extend (an_element)
		end

feature {NONE} -- Implementation

	undo_simple_change (a_change: RETICLE_SIMPLE_CHANGE)
			-- Undo `a_change'
		do
			a_change.set_parent (Current)
			precursor (a_change)
		end

	redo_simple_change (a_change: RETICLE_SIMPLE_CHANGE)
			-- Redo `a_change'
		do
			a_change.set_parent (Current)
			precursor (a_change)
		end

	start_delete
			-- Take actions needed before starting a delete
		do
			create {ARRAYED_LIST [PACKING_DIES_MROW]} fully_deleted_dies.make (50)
			create {ARRAYED_LIST [RETICLE_VIEW_ELEMENT]} fully_deleted_elements.make (50)
			create dies_table.make (50)
			view.unmark_all
		end

	finish_delete
			-- Take actions needed after finishing a delete
		do
			if not fully_deleted_dies.is_empty then
				model.remove_dies (fully_deleted_dies)
			end
			if not fully_deleted_elements.is_empty then
				view.update_copies_table_for_delete (fully_deleted_elements)
			end
			view.remove_marked_elements
			model.set_from_element_list (view.model_elements)
		end

	fully_deleted_dies: LIST [PACKING_DIES_MROW]
			-- Dies all of whose copies were deleted

	fully_deleted_elements: LIST [RETICLE_VIEW_ELEMENT]
			-- Elements for dies whose copies were all deleted

	dies_table: HASH_TABLE [INTEGER, PACKING_DIES_MROW]
			-- Table of distinct dies

end
