
indexing
	
	description: "A cut line through a scribe in reticle model"
	author: "David Hollenberg"
	date: "July 28, 2006"

class RETICLE_CUT
inherit
	RETICLE_MODEL_ELEMENT
		redefine
			is_cut, is_horizontal_cut, is_vertical_cut,
			cut_location, has_sub_elements, sub_elements,
			participates_in_measure,
			has_zero_size_in_x, has_zero_size_in_y,
			identification_for_message
		end

create
	make

feature {NONE} -- Creation

	make (a_cut_location: like cut_location a_model: like anchored_model a_field_name: STRING a_cut_size_x, a_cut_size_y, a_handle_size_x, a_handle_size_y: DOUBLE)
		require
			a_cut_location_not_void: a_cut_location /= Void
			a_model_not_void: a_model /= Void
			a_field_name_not_void: a_field_name /= Void
		do
			cut_location := a_cut_location
			model := a_model
			associated_field_name := a_field_name
			cut_size_x := a_cut_size_x
			cut_size_y := a_cut_size_y
			handle_size_x := a_handle_size_x
			handle_size_y := a_handle_size_y
			create {ARRAYED_LIST [like Current]} all_copies.make (1)
			all_copies.extend (Current)
			cut_location.detach_actions.extend (agent update_cut_location)
			set_sub_elements
		end

feature -- Properties

	cut_location: PACKING_CUT_LOCATIONS_MROW
			-- Cut location row

	identification: STRING = ""
			-- Identifying name of element

	identification_for_message: STRING = "CUT"
			-- Identifying name of element for displaying
			-- error and warning messages

	is_rotatable_internal: BOOLEAN = False
			-- May rotation of object currently be changed?

	is_movable: BOOLEAN = False
			-- May object be moved?

	is_selectable: BOOLEAN = True
			-- May object be selected?

	rotation: INTEGER = 0
			-- Rotation of object in degrees clockwise

	is_rotated_sideways: BOOLEAN = False
			-- Is object rotated by 90 degrees clockwise or
			-- 90 degrees counterclockwise?

	can_be_fixed_in_x: BOOLEAN = False
			-- Can object be temporarily fixed in the X dimension
			-- so that it cannot move in that dimension?

	can_be_fixed_in_y: BOOLEAN = False
			-- Can object be temporarily fixed in the Y dimension
			-- so that it cannot move in that dimension?

	is_fixed_in_x_internal: BOOLEAN = False
			-- Is object currently fixed in the Y dimension
			-- for vertical scanning purposes?

	is_fixed_in_y_internal: BOOLEAN = False
			-- Is object currently fixed in the Y dimension
			-- for vertical scanning purposes?

	is_movable_in_x_internal: BOOLEAN = False
			-- May object currently be moved in the X dimension?

	is_movable_in_y_internal: BOOLEAN = False
			-- May object currently be moved in the X dimension?

	is_duplicatable_internal: BOOLEAN = False
			-- May object currently be duplicated?

	is_deletable_internal: BOOLEAN = True
			-- May object currently be deleted?

	is_stationary_in_x: BOOLEAN = True
			-- Can object never be moved in the X dimension?

	is_stationary_in_y: BOOLEAN = True
			-- Can object never be moved in the Y dimension?

	is_stationary_orientation: BOOLEAN = True
			-- Can orientation of object never be changed?

	is_never_duplicatable: BOOLEAN = True
			-- Can object never be duplicated?

	is_never_deletable: BOOLEAN = False
			-- Can object never be deleted?

	is_cut: BOOLEAN = True
			-- Is object a cut line?

	is_horizontal_cut: BOOLEAN
			-- Is element a horizontal cut line?
		do
			Result := cut_location.is_horizontal
		end

	is_vertical_cut: BOOLEAN
			-- Is element a vertical cut line?
		do
			Result := cut_location.is_vertical
		end

	has_zero_size_in_x: BOOLEAN
			-- Does `Current' actually have zero size in X
			-- dimension?
		do
			if is_vertical_cut then
				Result := True
			else
				Result := precursor
			end
		end
		
	has_zero_size_in_y: BOOLEAN
			-- Does `Current' actually have zero size in Y
			-- dimension?
		do
			if is_horizontal_cut then
				Result := True
			else
				Result := precursor
			end
		end
		
	has_sub_elements: BOOLEAN = True
			-- Is element composed of more than one sub-element,
			-- each of which has its own size and center?

	sub_elements: LIST [like anchored_model_sub_element]
			-- Sub-elements of `Current'

	cut_size_x: DOUBLE
			-- Size of cut in X dimension (not including handle
			-- boxes at each end)

	cut_size_y: DOUBLE
			-- Size of cut in Y dimension (not including handle
			-- boxes at each end)

	handle_size_x: DOUBLE
			-- Size of handle box at each end of cut in X 
			-- dimension

	handle_size_y: DOUBLE
			-- Size of handle box at each end of cut in Y 
			-- dimension

	size_x: DOUBLE
			-- Size of minimum bounding box in X dimension
			-- of reticle field of view
		do
			if is_horizontal_cut then
				Result := cut_size_x + 2.0 * handle_size_x
			else
				Result := cut_size_x.max (handle_size_x)
			end
		end

	size_y: DOUBLE
			-- Size of minimum bounding box in Y dimension
			-- of reticle field of view
		do
			if is_vertical_cut then
				Result := cut_size_y + 2.0 * handle_size_y
			else
				Result := cut_size_y.max (handle_size_y)
			end
		end

	center_x: DOUBLE
			-- Center of minimum bounding box in X dimension
		do
			Result := field.center_x + field_relative_center_x
		end

	center_y: DOUBLE
			-- Center of minimum bounding box in Y dimension
		do
			Result := field.center_y + field_relative_center_y
		end

	field_relative_center_x: DOUBLE
			-- Center of minimum bounding box in X dimension,
			-- relative to center of field with which element
			-- is associated
		do
			if is_horizontal_cut then
				Result := 0.0
			else
				Result := cut_location.location
			end
		end

	field_relative_center_y: DOUBLE
			-- Center of minimum bounding box in Y dimension,
			-- relative to center of field with which element
			-- is associated
		do
			if is_vertical_cut then
				Result := 0.0
			else
				Result := cut_location.location
			end
		end

	db_maximum_move_up: DOUBLE = 0.0
			-- Maximum distance element can move up and still
			-- fit in database (inapplicable)

	db_maximum_move_down: DOUBLE = 0.0
			-- Maximum distance element can move down and still
			-- fit in database (inapplicable)

	db_maximum_move_left: DOUBLE = 0.0
			-- Maximum distance element can move left and still
			-- fit in database (inapplicable)

	db_maximum_move_right: DOUBLE = 0.0
			-- Maximum distance element can move right and still
			-- fit in database (inapplicable)

	die: PACKING_DIES_MROW
			-- Die for element that is a die
		do
			-- Inapplicable
		end

	die_location: PACKING_DIE_LOCATIONS_MROW
			-- Die location for element that is a die
		do
			-- Inapplicable
		end

	max_die_size: CONST_POINT
			-- Maximum die size, as constrained by package
			-- cavity sizes and user-specified limits.
			-- Void indicates no maximum
		do
			-- Inapplicable
		end

	is_oversize_die_always_allowed: BOOLEAN
			-- Is a die that is bigger than `max_die_size'
			-- allowed unconditionally?
		do
			-- Inapplicable
		end

	is_oversize_die_possibly_allowed: BOOLEAN
			-- Is a die that is bigger than `max_die_size'
			-- allowed but only under specific conditions?
		do
			-- Inapplicable
		end

	oversize_die_inclusion_conditions: STRING
			-- Description of conditions under which 
			-- die that is bigger than `max_die_size'
			-- allowed
		do
			-- Inapplicable
		end

	participates_in_measure: BOOLEAN = False
			-- Does `Current' participate in measure in X/Y
			-- operations?

feature -- Modification

	set_is_fixed_in_x (b: BOOLEAN)
			-- Set `is_fixed_in_x' to `b'
		do
			-- Inapplicable
		end

	set_is_fixed_in_y (b: BOOLEAN)
			-- Set `is_fixed_in_y' to `b'
		do
			-- Inapplicable
		end

	set_is_movable_in_x (b: BOOLEAN)
			-- Set `is_movable_in_x' to `b'
		do
			-- Inapplicable
		end

	set_is_movable_in_y (b: BOOLEAN)
			-- Set `is_movable_in_y' to `b'
		do
			-- Inapplicable
		end

	set_is_rotatable (b: BOOLEAN)
			-- Set `is_rotatable' to `b'
		do
			-- Inapplicable
		end

	set_is_duplicatable (b: BOOLEAN)
			-- Set `is_duplicatable' to `b'
		do
			-- Inapplicable
		end

	set_is_deletable (b: BOOLEAN)
			-- Set `is_deletable' to `b'
		do
			-- Inapplicable
		end

	set_rotation (n: INTEGER)
			-- Set rotation of `Current' to `n' degrees clockwise
		do
			-- Inapplicable
		end

	toggle_rotation
			-- Toggle `is_rotated'
		do
			-- Inapplicable
		end

	rotate_90_clockwise
			-- Rotate object 90 degrees clockwise
		do
			-- Inapplicable
		end

	rotate_90_counterclockwise
			-- Rotate object 90 degrees counterclockwise
		do
			-- Inapplicable
		end

	move_by (a_distance: CONST_POINT)
			-- Move `Current' by `a_distance'
		do
			-- Inapplicable
		end

	round_edge_to_grid
			-- Move lower left corner of `Current' to nearest
			-- grid point.  Used to prevent accumulation of
			-- errors from multiple moves
		do
			-- Inapplicable
		end

	duplicate
			-- Duplicate element and add it to model
		do
			-- Inapplicable
		end

	move_left_edge (a_distance: DOUBLE)
			-- Move left edge of `Current' by `a_distance'
		do
			-- Inapplicable
		end

	move_right_edge (a_distance: DOUBLE)
			-- Move right edge of `Current' by `a_distance'
		do
			-- Inapplicable
		end

	move_top_edge (a_distance: DOUBLE)
			-- Move top edge of `Current' by `a_distance'
		do
			-- Inapplicable
		end

	move_bottom_edge (a_distance: DOUBLE)
			-- Move bottom edge of `Current' by `a_distance'
		do
			-- Inapplicable
		end

feature {NONE} -- Implementation

	set_sub_elements
			-- Set sub-elements that comprise cut (cut line plus
			-- handle box at each end)
		local
			l_cut_line: RETICLE_CUT_LINE
			l_first_cut_handle, l_second_cut_handle: RETICLE_CUT_HANDLE
			l_offset: DOUBLE
		do
			create {ARRAYED_LIST [like anchored_model_sub_element]} sub_elements.make (3)
			create l_cut_line.make (cut_size_x, cut_size_y, center_x, center_y)
			sub_elements.extend (l_cut_line)
			if is_horizontal_cut then
				l_offset := (cut_size_x + handle_size_x) / 2.0
				create l_first_cut_handle.make (handle_size_x, handle_size_y, center_x - l_offset, center_y)
				create l_second_cut_handle.make (handle_size_x, handle_size_y, center_x + l_offset, center_y)
			else
				l_offset := (cut_size_y + handle_size_y) / 2.0
				create l_first_cut_handle.make (handle_size_x, handle_size_y, center_x, center_y - l_offset)
				create l_second_cut_handle.make (handle_size_x, handle_size_y, center_x, center_y + l_offset)
			end
			sub_elements.extend (l_first_cut_handle)
			sub_elements.extend (l_second_cut_handle)
		end

	update_cut_location (an_original, a_replacement: like cut_location)
			-- If `cut_location' equals `an_original', replace it
			-- with `a_replacement'
		do
			if cut_location = an_original then
				cut_location := a_replacement
			end
		end

	associated_field_name: STRING
			-- Name of field with which element is associated
	
end
