
indexing
	
	description: "Change applied to reticle during packing that creates %
		%or deletes elements that are cuts"
	author: "David Hollenberg"
	date: "January 25, 2007"

deferred class RETICLE_CUT_CREATION_OR_DELETION_CHANGE
inherit
	RETICLE_CHANGE
		redefine
			undo_simple_change, redo_simple_change
		end
	
feature -- Properties

	cut_pattern: RETICLE_CUT_PATTERN
			-- Cut pattern of which cut is part

feature -- Modification

	set_cut_pattern (a_pattern: like cut_pattern)
			-- Set `cut_pattern' to `a_pattern'
		do
			cut_pattern := a_pattern
		end

feature -- Operations

	add_deleted_cut (a_cut: PACKING_CUT_LOCATIONS_MROW)
		do
			deleted_cuts.extend (a_cut)
		end

	add_deleted_element (an_element: RETICLE_VIEW_ELEMENT)
		do
			deleted_elements.extend (an_element)
		end

feature {NONE} -- Implementation

	undo_simple_change (a_change: RETICLE_SIMPLE_CHANGE)
			-- Undo `a_change'
		do
			a_change.set_parent (Current)
			precursor (a_change)
		end

	redo_simple_change (a_change: RETICLE_SIMPLE_CHANGE)
			-- Redo `a_change'
		do
			a_change.set_parent (Current)
			precursor (a_change)
		end

	start_delete
			-- Take actions needed before starting a delete
		do
			create {ARRAYED_LIST [PACKING_CUT_LOCATIONS_MROW]} deleted_cuts.make (10)
			create {ARRAYED_LIST [RETICLE_VIEW_ELEMENT]} deleted_elements.make (10)
			view.unmark_all
		end

	finish_delete
			-- Take actions needed after finishing a delete
		do
			if not deleted_cuts.is_empty then
				model.remove_cuts (deleted_cuts, cut_pattern)
			end
			view.remove_marked_elements
			model.set_from_element_list (view.model_elements)
		end

	deleted_cuts: LIST [PACKING_CUT_LOCATIONS_MROW]
			-- Cuts that were deleted

	deleted_elements: LIST [RETICLE_VIEW_ELEMENT]
			-- Elements that were deleted

end
