
indexing
	
	description: "A cut pattern in reticle model"
	author: "David Hollenberg"
	date: "July 28, 2006"

class RETICLE_CUT_PATTERN

create
	make

feature {NONE} -- Creation

	make (a_pattern: like pattern a_model: RETICLE_MODEL)
		require
			a_pattern_not_void: a_pattern /= Void
			a_model_not_void: a_model /= Void
		do
			pattern := a_pattern
			model := a_model
			pattern.detach_actions.extend (agent update_pattern)
		end

feature -- Properties

	pattern: PACKING_CUT_PATTERNS_MROW
			-- Cut pattern row

	cut_pattern_no: INTEGER
			-- Number assigned to cut pattern
		do
			Result := pattern.cut_pattern_no
		end

	field_name: STRING
			-- Name of field to which cut pattern applies
		do
			Result := pattern.field_name
		end

	has_wafer_etest_name: BOOLEAN
			-- Is there a wafer etest name for cut pattern?
		do
			Result := not pattern.is_null_wafer_etest_name
		end

	wafer_etest_name: STRING
			-- Wafer etest name for cut pattern
		do
			Result := pattern.wafer_etest_name
		end

	wafer_etest_id: STRING
			-- Wafer etest ID for cut pattern
		do
			Result := pattern.wafer_etest_id
		end

	has_quantity_wafers_needed: BOOLEAN
			-- Is there a number wafers to cut for pattern?
		do
			Result := not pattern.is_null_quantity_wafers_needed
		end

	quantity_wafers_needed: INTEGER
			-- Number of wafers to cut for pattern or -1 if
			-- not yet specified
		do
			if has_quantity_wafers_needed then
				Result := pattern.quantity_wafers_needed
			else
				Result := -1
			end
		end

	is_completed: BOOLEAN
			-- Is cut pattern completed (not under construction)?
		do
			Result := pattern.is_completed
		end

	has_cuts: BOOLEAN
			-- Are there any cuts for pattern?
		do
			Result := pattern.has_cuts
		end

	has_horizontal_cut_at (a_location: DOUBLE): BOOLEAN
			-- Does pattern include a horizontal cut at
			-- `a_location'?
		do
			Result := pattern.has_horizontal_cut_at (a_location)
		end

	has_vertical_cut_at (a_location: DOUBLE): BOOLEAN
			-- Does pattern include a vertical cut at
			-- `a_location'?
		do
			Result := pattern.has_vertical_cut_at (a_location)
		end

	has_isolated_sites: BOOLEAN
			-- Are there any isolated sites for pattern?
		do
			Result := pattern.has_isolated_sites
		end

	isolates (a_location: PACKING_DIE_LOCATIONS_MROW): BOOLEAN
			-- Does pattern isolate `a_location'?
		do
			Result := pattern.isolates_location (a_location)
		end

	isolated_cut_die (an_element: RETICLE_MODEL_ELEMENT): RECTANGLE
			-- Cut die for isolated element `an_element' in 
			-- cut pattern, determined from its cut die site
		require
			an_element_not_void: an_element /= Void
			an_element_is_die: an_element.is_die
		local
			rect: RECTANGLE
		do
			rect := pattern.isolated_cut_die (an_element.die_location)
			Result := an_element.unoriented_model_relative_rectangle (rect)
		end

	isolated_location_count: INTEGER
			-- Number of die locations isolated by cut pattern
		do
			Result := pattern.isolated_location_count
		end

	description: STRING
			-- Description of cut pattern for messages to user
		do
			Result := "pattern " + cut_pattern_no.out + ", field " + field_name
		end

	status: STRING
			-- Status of cut pattern
		do
			Result := pattern.status
		end

	model: RETICLE_MODEL
			-- Model containing cut pattern

feature -- Modification

	set_is_completed (a_value: BOOLEAN)
			-- Set `is_completed' to `a_value'
		do
			pattern.set_is_completed (a_value)
		ensure
			completed_set: is_completed = a_value
		end

	set_wafer_etest_name (a_name: STRING)
			-- Set `wafer_etest_name' to `a_name'
		do
			pattern.set_wafer_etest_name (a_name)
		ensure
			name_set: wafer_etest_name.is_equal (a_name)
		end

	set_quantity_wafers_needed (a_value: INTEGER has_value: BOOLEAN)
			-- Set `quantity_wafers_needed' to `a_value' where
			-- `has_value' indicates whether there really is a
			-- value
		do
			if has_value then
				pattern.set_quantity_wafers_needed (a_value)
			else
				pattern.set_null_quantity_wafers_needed
			end
		end

feature -- Operations

	remove_isolated_sites
			-- Remove all isolated sites for cut pattern
		do
			pattern.remove_isolated_sites
		end

	isolate (an_element: RETICLE_MODEL_ELEMENT)
			-- Isolate `an_element' in cut pattern
		require
			an_element_not_void: an_element /= Void
		do
			pattern.isolate (an_element.die_location, an_element.oriented_field_relative_cut_cut_die)
		end

	unisolate (an_element: RETICLE_MODEL_ELEMENT)
			-- Unisolate `an_element' in cut pattern
		require
			an_element_not_void: an_element /= Void
		do
			pattern.unisolate (an_element.die_location)
		end

	set_isolated_site_cut_die (an_element: RETICLE_MODEL_ELEMENT)
			-- Set cut die size and center from `an_element'
		require
			an_element_not_void: an_element /= Void
		do
			pattern.set_isolated_site_cut_die (an_element.die_location, an_element.oriented_field_relative_cut_cut_die)
		end

feature {NONE} -- Implementation

	update_pattern (an_original, a_replacement: like pattern)
			-- If `pattern' equals `an_original', replace it
			-- with `a_replacement'
		do
			if pattern = an_original then
				pattern := a_replacement
			end
		end

end
