
indexing
	
	description: "Controller for a model and main view of a reticle"
	author: "David Hollenberg"
	date: "July 28, 2006"

class RETICLE_MAIN_MODEL_VIEW_CONTROLLER
inherit
	MODEL_VIEW_CONTROLLER
		redefine
			model, view, make, anchored_model_element
		end
	PACKING_DELEGATED_STATUS_REPORTER
		export
			{ANY} reporter, set_reporter
			{NONE} all
		end
	CONDITION_UTILITIES
		export
			{NONE} all
		end
	GUI_UTILITIES_ACCESS
		export
			{NONE} all
		end
	ASCII
		export
			{NONE} all
		end
	STRING_UTILITIES_ACCESS
		export
			{NONE} all
		end

create
	make

feature {NONE} -- Creation

	make (a_model: like model a_view: like view)
			-- Create with `a_model' and `a_view'
		do
			precursor (a_model, a_view)
			create operation_start_location
			create operation_current_location
			create operation_end_location
			create operation_current_rectangle
			application := (create {EV_ENVIRONMENT}).application
		end

feature -- Properties

	model: RETICLE_MODEL
			-- Model of a reticle that can be manipulated
			-- and displayed

	view: RETICLE_VIEW
			-- View of a reticle model

	main_window: RC_PACKING_WINDOW
			-- Main packing window

	is_read_only: BOOLEAN
			-- Is packer being run in read-only mode?

	saved_area_capacity: INTEGER
			-- Maximum number of saved areas supported by view
		do
			Result := view.saved_area_capacity
		end

	is_scribe_resize_valid: BOOLEAN
			-- Was last scribe resize operation user tried to
			-- start valid?

	validity_error_count: INTEGER
			-- Number of validity errors for last call to any
			-- validity check routine

	validity_warning_count: INTEGER
			-- Number of validity warnings for last call to any
			-- validity check routine

feature -- Modification

	set_main_window (a_window: like main_window)
		require
			a_window_not_void: a_window /= Void
		do
			main_window := a_window
		ensure
			main_window_set: main_window = a_window
		end

	set_is_read_only (a_value: like is_read_only)
		do
			is_read_only := a_value
		ensure
			is_read_only_set: is_read_only = a_value
		end

feature -- Operations

	set_initial_model_elements (list: LIST [like anchored_model_element] with_redraw: BOOLEAN)
			-- Handle setting of initial list of model elements
			-- to `list'.  Redraw model if `with_redraw'
		do
			view.set_initial_model_elements (list, with_redraw)
		end

	add_model_elements (list: LIST [like anchored_model_element])
			-- Handle addition of model elements in `list'
		local
			n: INTEGER
		do
			view.add_model_elements (list, True)
			n := list.count
			report_change ("Added " + n.out + " " + su.number (n, "project", "projects"))
			report_end
		end

	remove_model_elements (list: LIST [PACKING_DIES_MROW])
			-- Remove model elements in `list' from model and view
		local
			n, objs: INTEGER
		do
			if not is_remote_delete_disabled then
				save_count (view.selectable_count)
				view.remove_model_elements (list)
				n := list.count
				objs := old_count - view.selectable_count
				report_change ("Removed " + n.out + " " + su.number (n, "project", "projects") + " (" + objs.out + " " + su.number (objs, "object", "objects") + ")")
				report_end
			end
		end

	update_model_elements (list: LIST [PACKING_DIES_MROW])
			-- Handle update of model elements in `list'
		local
			n: INTEGER
		do
			view.update_model_elements (list)
			n := list.count
			report_change ("Updated " + n.out + " " + su.number (n, "project", "projects"))
			report_end
		end

	redraw_rectangle (a_x, a_y, a_width, a_height: INTEGER)
			-- Redraw the area of the canvas whose upper left
			-- corner is at (`a_x', `a_y') and whose size
			-- is `a_width' by `a_height'
		do
			view.redraw_canvas_rectangle (a_x, a_y, a_width, a_height)
		end

	handle_canvas_resize (a_x, a_y, a_width, a_height: INTEGER)
			-- Handle "canvas resized" event, where new upper left
			-- corner is at (`a_x', `a_y') and new size
			-- is `a_width' by `a_height'
		do
			view.handle_canvas_resize (a_x, a_y, a_width, a_height)
		end

	select_all_items
			-- Select all items that can be selected
		do
			view.select_all_items
			report_select ("Selected all objects" + selected_text)
		end
	
	unselect_all_items
			-- Unselect all items, so that no items are selected
		do
			view.unselect_all_items
			report_select ("Unselected all objects" + selected_text)
		end
	
	invert_selection
			-- Unselect all items that are selected and select
			-- all items that are not selected and that can be
			-- selected
		do
			view.invert_selection
			report_select ("Inverted selection" + selected_text)
		end
	
	set_selection_to_cust_project_items
			-- Set selection to all items which are
			-- customer projects
		do
			view.set_selection_to_cust_project_items
			report_select ("Set selection to all customer projects" + selected_text)
		end
	
	set_selection_to_test_vehicle_items
			-- Set selection to all items which are test vehicles
		do
			view.set_selection_to_test_vehicle_items
			report_select ("Set selection to all test vehicles" + selected_text)
		end
	
	set_selection_to_project_items
			-- Set selection to all items which are projects
		do
			view.set_selection_to_project_items
			report_select ("Set selection to all projects" + selected_text)
		end
	
	set_selection_to_scribe_items
			-- Set selection to all items which are scribes
		do
			view.set_selection_to_scribe_items
			report_select ("Set selection to all scribes" + selected_text)
		end
	
	set_selection_to_horizontal_scribe_items
			-- Set selection to all items which are
			-- horizontal scribes
		do
			view.set_selection_to_horizontal_scribe_items
			report_select ("Set selection to all horizontal scribes" + selected_text)
		end
	
	set_selection_to_vertical_scribe_items
			-- Set selection to all items which are
			-- vertical scribes
		do
			view.set_selection_to_vertical_scribe_items
			report_select ("Set selection to all vertical scribes" + selected_text)
		end
	
	set_selection_to_floating_scribe_items
			-- Set selection to all items which are
			-- floating scribes (either end not on edge of scribe
			-- of opposite orientation or field of view edge)
		do
			view.set_selection_to_floating_scribe_items
			report_select ("Set selection to all floating scribes" + selected_text)
		end
	
	set_selection_to_misaligned_scribe_items
			-- Set selection to all items which are
			-- misaligned scribes (not quite lined up)
		do
			view.set_selection_to_misaligned_scribe_items
			report_select ("Set selection to all misaligned scribes" + selected_text)
		end
	
	set_selection_to_default_width_scribe_items
			-- Set selection to all items which are
			-- scribes with the default width
		do
			view.set_selection_to_default_width_scribe_items
			report_select ("Set selection to all default width scribes" + selected_text)
		end
	
	set_selection_to_custom_width_scribe_items
			-- Set selection to all items which are
			-- scribes with non-default width
		do
			view.set_selection_to_custom_width_scribe_items
			report_select ("Set selection to all custom width scribes" + selected_text)
		end
	
	set_selection_to_non_scribe_items
			-- Set selection to all items which are not scribes
		do
			view.set_selection_to_non_scribe_items
			report_select ("Set selection to all non-scribes" + selected_text)
		end
	
	set_selection_to_filler_items
			-- Set selection_to all items which are fillers
		do
			view.set_selection_to_filler_items
			report_select ("Set selection to all fillers" + selected_text)
		end
	
	set_selection_to_vendor_geometry_items
			-- Set selection to all items which are vendor
			-- geometries/ghosts
		do
			view.set_selection_to_vendor_geometry_items
			report_select ("Set selection to all vendor geometries/ghosts" + selected_text)
		end
	
	set_selection_to_cut_items
			-- Set selection to all items which are cuts
		do
			view.set_selection_to_cut_items
			report_select ("Set selection to all cuts" + selected_text)
		end
	
	set_selection_to_commercial_account_items
			-- Set selection to all items whose account
			-- type is commercial
		do
			view.set_selection_to_commercial_account_items
			report_select ("Set selection to all commercial projects" + selected_text)
		end
	
	set_selection_to_mep_account_items
			-- Set selection to all items whose account
			-- type is MEP
		do
			view.set_selection_to_mep_account_items
			report_select ("Set selection to all MEP projects" + selected_text)
		end
	
	set_selection_to_other_account_items
			-- Set selection to all items whose account
			-- type is other than commercial or MEP
		do
			view.set_selection_to_other_account_items
			report_select ("Set selection to all non-commercial and non-MEP projects" + selected_text)
		end
	
	set_selection_to_partially_selected_items
			-- Set selection to all items that are partially
			-- selected (have at least one copy selected but
			-- not all copies)
		do
			view.set_selection_to_partially_selected_items
			report_select ("Set selection to all objects that are partially selected" + selected_text)
		end
	
	set_selection_to_outside_fields_items
			-- Set selection to all items that are completely
			-- outside all fields of view
		do
			view.set_selection_to_outside_fields_items
			report_select ("Set selection to all objects that are outside all fields" + selected_text)
		end
	
	set_selection_to_too_small_items
			-- Set selection to all objects whose size is below
			-- the minimum size in either dimension
		do
			view.set_selection_to_too_small_items
			report_select ("Set selection to all objects that are below minimum size" + selected_text)
		end
	
	set_selection_to_small_scribes_and_fillers
			-- Set selection to all scribes and fillers whose
			-- size is below the small object threshold in either
			-- dimension (just length for scribes)
		do
			view.set_selection_to_small_scribes_and_fillers
			report_select ("Set selection to all small scribes and fillers" + selected_text)
		end
	
	set_selection_to_overlapping_field_items
			-- Set selection to all objects which intersect
			-- at least one field of view but are not enclosed
			-- by any field
		do
			view.set_selection_to_overlapping_field_items
			report_select ("Set selection to all objects overlapping field edge" + selected_text)
		end
	
	set_selection_to_overlapping_object_items
			-- Set selection to all objects which intersect
			-- at least one field of view and overlap another
			-- object
		do
			view.set_selection_to_overlapping_object_items
			report_select ("Set selection to all objects overlapping another object" + selected_text)
		end
	
	set_selection_to_close_edge_items
			-- Set selection to all objects which intersect
			-- at least one field of view and have at least one
			-- edge which is too close to field of view edge
		do
			view.set_selection_to_close_edge_items
			report_select ("Set selection to all objects too close to field edge" + selected_text)
		end
	
	set_selection_to_close_object_items
			-- Set selection to all objects which intersect
			-- at least one field of view and are too close to
			-- another object that intersects at least one field
		do
			view.set_selection_to_close_object_items
			report_select ("Set selection to all objects too close to another object" + selected_text)
		end
	
	set_selection_to_too_big_items
			-- Set selection to all projects which have a die
			-- size that exceeds their maximum die size
		do
			view.set_selection_to_too_big_items
			report_select ("Set selection to all projects whose die size is too big" + selected_text)
		end
	
	set_selection_to_uncentered_items
			-- Set selection to all projects which are not
			-- centered in their cut die
		do
			view.set_selection_to_uncentered_items
			report_select ("Set selection to all projects not centered in cut die" + selected_text)
		end
	
	set_selection_to_unisolated_items
			-- Set selection to all projects which are not
			-- isolated in their cut die
		do
			view.set_selection_to_unisolated_items
			report_select ("Set selection to all projects not isolated in cut die" + selected_text)
		end
	
	set_selection_to_unassigned_lot_items
			-- Set selection to all objects which have associated
			-- wafer etest ID(s) but have not been assigned a
			-- specific one
		do
			view.set_selection_to_unassigned_lot_items
			report_select ("Set selection to all projects with no lot assigned" + selected_text)
		end
	
	set_selection_to_unlocked_or_unassigned
			-- Set selection to all projects which are not
			-- locked against customer update or are not assigned
			-- to run being closed
		do
			view.set_selection_to_unlocked_or_unassigned
			report_select ("Set selection to all unlocked or unassigned to run projects" + selected_text)
		end
	
	set_selection_all_copies_by_point (a_coordinate: EV_COORDINATE)
			-- Set selected items to all copies of every
			-- selectable item that contains `a_coordinate'
		require
			a_coordinate_not_void: a_coordinate /= Void
		do
			view.set_selection_all_copies_by_point (a_coordinate)
			report_select ("Set selection to all copies of objects containing mouse pointer" + selected_text)
		end
	
	add_to_selection_no_items
			-- Add to selection no items
		do
			save_selected_count
			report_select ("Added nothing to selection" + added_selected_text)
		end
	
	add_to_selection_cust_project_items
			-- Add to selection all items which are
			-- customer projects
		do
			save_selected_count
			view.add_to_selection_cust_project_items
			report_select ("Added all customer projects to selection" + added_selected_text)
		end
	
	add_to_selection_test_vehicle_items
			-- Add to selection all items which are test
			-- vehicles
		do
			save_selected_count
			view.add_to_selection_test_vehicle_items
			report_select ("Added all test vehicles to selection" + added_selected_text)
		end
	
	add_to_selection_project_items
			-- Add to selection all items which are projects
		do
			save_selected_count
			view.add_to_selection_project_items
			report_select ("Added all projects to selection" + added_selected_text)
		end
	
	add_to_selection_scribe_items
			-- Add to selection all items which are scribes
		do
			save_selected_count
			view.add_to_selection_scribe_items
			report_select ("Added all scribes to selection" + added_selected_text)
		end
	
	add_to_selection_horizontal_scribe_items
			-- Add to selection all items which are
			-- horizontal scribes
		do
			save_selected_count
			view.add_to_selection_horizontal_scribe_items
			report_select ("Added all horizontal scribes to selection" + added_selected_text)
		end
	
	add_to_selection_vertical_scribe_items
			-- Add to selection to all items which are
			-- vertical scribes
		do
			save_selected_count
			view.add_to_selection_vertical_scribe_items
			report_select ("Added all vertical scribes to selection" + added_selected_text)
		end
	
	add_to_selection_floating_scribe_items
			-- Add to selection all items which are
			-- floating scribes (either end not on edge of
			-- scribe of opposite orientation or field of
			-- view edge)
		do
			save_selected_count
			view.add_to_selection_floating_scribe_items
			report_select ("Added all floating scribes to selection" + added_selected_text)
		end
	
	add_to_selection_misaligned_scribe_items
			-- Add to selection all items which are
			-- misaligned scribes (not quite lined up)
		do
			save_selected_count
			view.add_to_selection_misaligned_scribe_items
			report_select ("Added all misaligned scribes to selection" + added_selected_text)
		end
	
	add_to_selection_default_width_scribe_items
			-- Add to selection all items which are
			-- scribes with the default width
		do
			save_selected_count
			view.add_to_selection_default_width_scribe_items
			report_select ("Added all default width scribes to selection" + added_selected_text)
		end
	
	add_to_selection_custom_width_scribe_items
			-- Add to selection all items which are
			-- scribes with non-default width
		do
			save_selected_count
			view.add_to_selection_custom_width_scribe_items
			report_select ("Added all custom width scribes to selection" + added_selected_text)
		end
	
	add_to_selection_non_scribe_items
			-- Add to selection all items which are not scribes
		do
			save_selected_count
			view.add_to_selection_non_scribe_items
			report_select ("Added all non-scribes to selection" + added_selected_text)
		end
	
	add_to_selection_filler_items
			-- Add to selection all items which are fillers
		do
			save_selected_count
			view.add_to_selection_filler_items
			report_select ("Added all fillers to selection" + added_selected_text)
		end
	
	add_to_selection_vendor_geometry_items
			-- Add to selection all items which are vendor
			-- geometries/ghosts
		do
			save_selected_count
			view.add_to_selection_vendor_geometry_items
			report_select ("Added all vendor geometries/ghosts to selection" + added_selected_text)
		end
	
	add_to_selection_cut_items
			-- Add to selection all items which are cuts
		do
			save_selected_count
			view.add_to_selection_cut_items
			report_select ("Added all cuts to selection" + added_selected_text)
		end
	
	add_to_selection_commercial_account_items
			-- Add to selection all items whose account
			-- type is commercial
		do
			save_selected_count
			view.add_to_selection_commercial_account_items
			report_select ("Added all commercial projects to selection" + added_selected_text)
		end
	
	add_to_selection_mep_account_items
			-- Add to selection all items whose account
			-- type is MEP
		do
			save_selected_count
			view.add_to_selection_mep_account_items
			report_select ("Added all MEP projects to selection" + added_selected_text)
		end
	
	add_to_selection_other_account_items
			-- Add to selection all items whose account
			-- type is other than commercial or MEP
		do
			save_selected_count
			view.add_to_selection_other_account_items
			report_select ("Added all non-commercial and non-MEP projects to selection" + added_selected_text)
		end
	
	add_to_selection_partially_selected_items
			-- Add to selection all unselected copies of items
			-- that have any copies currently selected
		do
			save_selected_count
			view.add_to_selection_partially_selected_items
			report_select ("Added all copies of selected objects to selection" + added_selected_text)
		end
	
	add_to_selection_outside_fields_items
			-- Add to selection all objects that are outside
			-- all fields of view
		do
			save_selected_count
			view.add_to_selection_outside_fields_items
			report_select ("Added objects outside all fields to selection" + added_selected_text)
		end
	
	add_to_selection_too_small_items
			-- Add to selection all objects whose size is below
			-- the minimum size in either dimension
		do
			save_selected_count
			view.add_to_selection_too_small_items
			report_select ("Added objects that are below minimum size to selection" + added_selected_text)
		end
	
	add_to_selection_small_scribes_and_fillers
			-- Add to selection all scribes and fillers whose
			-- size is below the small object threshold in either
			-- dimension (just length for scribes)
		do
			save_selected_count
			view.add_to_selection_small_scribes_and_fillers
			report_select ("Added small scribes and fillers to selection" + added_selected_text)
		end
	
	add_to_selection_overlapping_field_items
			-- Add to selection all objects which intersect
			-- at least one field of view but are not enclosed
			-- by any field
		do
			save_selected_count
			view.add_to_selection_overlapping_field_items
			report_select ("Added objects overlapping field edge to selection" + added_selected_text)
		end
	
	add_to_selection_overlapping_object_items
			-- Add to selection all objects which intersect
			-- at least one field of view and overlap another
			-- object
		do
			save_selected_count
			view.add_to_selection_overlapping_object_items
			report_select ("Added objects overlapping another object to selection" + added_selected_text)
		end
	
	add_to_selection_close_edge_items
			-- Add to selection all objects which intersect
			-- at least one field of view and have at least one
			-- edge which is too close to field of view edge
		do
			save_selected_count
			view.add_to_selection_close_edge_items
			report_select ("Added objects too close to field edge to selection" + added_selected_text)
		end
	
	add_to_selection_close_object_items
			-- Add to selection all objects which intersect
			-- at least one field of view and are too close to
			-- another object that intersects at least one field
		do
			save_selected_count
			view.add_to_selection_close_object_items
			report_select ("Added objects too close to another object to selection" + added_selected_text)
		end
	
	add_to_selection_too_big_items
			-- Add to selection all projects which have a die
			-- size that exceeds their maximum die size
		do
			save_selected_count
			view.add_to_selection_too_big_items
			report_select ("Added projects whose die size is too big to selection" + added_selected_text)
		end
	
	add_to_selection_uncentered_items
			-- Add to selection all projects which are not
			-- centered in their cut die
		do
			save_selected_count
			view.add_to_selection_uncentered_items
			report_select ("Added projects not centered in cut die to selection" + added_selected_text)
		end
	
	add_to_selection_unisolated_items
			-- Add to selection all projects which are not
			-- isolated in their cut die
		do
			save_selected_count
			view.add_to_selection_unisolated_items
			report_select ("Added projects not isolated in cut die to selection" + added_selected_text)
		end
	
	add_to_selection_unassigned_lot_items
			-- Add to selection to all objects which have
			-- associated wafer etest ID(s) but have not
			-- been assigned a specific one
		do
			save_selected_count
			view.add_to_selection_unassigned_lot_items
			report_select ("Added projects with no lot assigned to selection" + added_selected_text)
		end
	
	add_to_selection_unlocked_or_unassigned
			-- Add to selection all projects which are not
			-- locked against customer update or are not assigned
			-- to run being closed
		do
			save_selected_count
			view.add_to_selection_unlocked_or_unassigned
			report_select ("Added unlocked or unassigned to run projects to selection" + added_selected_text)
		end
	
	add_to_selection_all_copies_by_point (a_coordinate: EV_COORDINATE)
			-- Add to selection all copies of every selectable
			-- item that contains `a_coordinate'
		require
			a_coordinate_not_void: a_coordinate /= Void
		do
			save_selected_count
			view.add_to_selection_all_copies_by_point (a_coordinate)
			report_select ("Added all copies of objects containing mouse pointer to selection" + added_selected_text)
		end

	remove_from_selection_no_items
			-- Remove from selection no items
		do
			save_selected_count
			report_select ("Removed nothing from selection" + removed_selected_text)
		end
	
	remove_from_selection_cust_project_items
			-- Remove from selection all items which are
			-- customer projects
		do
			save_selected_count
			view.remove_from_selection_cust_project_items
			report_select ("Removed all customer projects from selection" + removed_selected_text)
		end
	
	remove_from_selection_test_vehicle_items
			-- Remove from selection all items which are
			-- test vehicles
		do
			save_selected_count
			view.remove_from_selection_test_vehicle_items
			report_select ("Removed all test vehicles from selection" + removed_selected_text)
		end
	
	remove_from_selection_project_items
			-- Remove from selection all items which are projects
		do
			save_selected_count
			view.remove_from_selection_project_items
			report_select ("Removed all projects from selection" + removed_selected_text)
		end
	
	remove_from_selection_scribe_items
			-- Remove from selection all items which are scribes
		do
			save_selected_count
			view.remove_from_selection_scribe_items
			report_select ("Removed all scribes from selection" + removed_selected_text)
		end
	
	remove_from_selection_horizontal_scribe_items
			-- Removed from selection all items which are
			-- horizontal scribes
		do
			save_selected_count
			view.remove_from_selection_horizontal_scribe_items
			report_select ("Removed all horizontal scribes from selection" + removed_selected_text)
		end
	
	remove_from_selection_vertical_scribe_items
			-- Remove from selection all items which are
			-- vertical scribes
		do
			save_selected_count
			view.remove_from_selection_vertical_scribe_items
			report_select ("Removed all vertical scribes from selection" + removed_selected_text)
		end
	
	remove_from_selection_floating_scribe_items
			-- Remove from selection all items which are
			-- floating scribes (either end not on edge of scribe
			-- of opposite orientation or field of view edge)
		do
			save_selected_count
			view.remove_from_selection_floating_scribe_items
			report_select ("Removed all floating scribes from selection" + removed_selected_text)
		end
	
	remove_from_selection_misaligned_scribe_items
			-- Remove from selection all items which are
			-- misaligned scribes (not quite lined up)
		do
			save_selected_count
			view.remove_from_selection_misaligned_scribe_items
			report_select ("Removed all misaligned scribes from selection" + removed_selected_text)
		end
	
	remove_from_selection_default_width_scribe_items
			-- Remove from selection all items which are
			-- scribes with the default width
		do
			save_selected_count
			view.remove_from_selection_default_width_scribe_items
			report_select ("Removed all default width scribes from selection" + removed_selected_text)
		end
	
	remove_from_selection_custom_width_scribe_items
			-- Remove from selection all items which are
			-- scribes with non-default width
		do
			save_selected_count
			view.remove_from_selection_custom_width_scribe_items
			report_select ("Removed all custom width scribes from selection" + removed_selected_text)
		end
	
	remove_from_selection_non_scribe_items
			-- Remove from selection all items which are not
			-- scribes
		do
			save_selected_count
			view.remove_from_selection_non_scribe_items
			report_select ("Removed all non-scribes from selection" + removed_selected_text)
		end
	
	remove_from_selection_filler_items
			-- Remove from selection all items which are fillers
		do
			view.remove_from_selection_filler_items
			report_select ("Removed all fillers from selection" + removed_selected_text)
		end
	
	remove_from_selection_vendor_geometry_items
			-- Remove from selection all items which are vendor
			-- geometries/ghosts
		do
			save_selected_count
			view.remove_from_selection_vendor_geometry_items
			report_select ("Removed all vendor geometries/ghosts from selection" + removed_selected_text)
		end
	
	remove_from_selection_cut_items
			-- Remove from selection all cuts
		do
			save_selected_count
			view.remove_from_selection_cut_items
			report_select ("Removed all cuts from selection" + removed_selected_text)
		end
	
	remove_from_selection_commercial_account_items
			-- Remove from selection all items whose account
			-- type is commercial
		do
			save_selected_count
			view.remove_from_selection_commercial_account_items
			report_select ("Removed all commercial projects from selection" + removed_selected_text)
		end
	
	remove_from_selection_mep_account_items
			-- Remove from selection all items whose account
			-- type is MEP
		do
			save_selected_count
			view.remove_from_selection_mep_account_items
			report_select ("Removed all MEP projects from selection" + removed_selected_text)
		end
	
	remove_from_selection_other_account_items
			-- Rmove from selection all items whose account
			-- type is other than commercial or MEP
		do
			save_selected_count
			view.remove_from_selection_other_account_items
			report_select ("Removed all non-commercial and non-MEP projects from selection" + removed_selected_text)
		end
	
	remove_from_selection_partially_selected_items
			-- Remove from selection all items which do not
			-- have all copies currently selected
		do
			save_selected_count
			view.remove_from_selection_partially_selected_items
			report_select ("Removed all partially selected objects from selection" + removed_selected_text)
		end
	
	remove_from_selection_outside_fields_items
			-- Remove from selection all items which are outside
			-- all fields of view
		do
			save_selected_count
			view.remove_from_selection_outside_fields_items
			report_select ("Removed objects outside all fields from selection" + removed_selected_text)
		end
	
	remove_from_selection_too_small_items
			-- Remove from selection all objects whose
			-- size is below the minimum size in either
			-- dimension
		do
			save_selected_count
			view.remove_from_selection_too_small_items
			report_select ("Removed objects that are below minimum size from selection" + removed_selected_text)
		end

	remove_from_selection_small_scribes_and_fillers
			-- Remove from selection all scribes and fillers whose
			-- size is below the small object threshold in either
			-- dimension (just length for scribes)
		do
			save_selected_count
			view.remove_from_selection_small_scribes_and_fillers
			report_select ("Removed small scribes and fillers from selection" + removed_selected_text)
		end
	
	remove_from_selection_overlapping_field_items
			-- Remove from selection all objects which intersect
			-- at least one field of view but are not enclosed
			-- by any field
		do
			save_selected_count
			view.remove_from_selection_overlapping_field_items
			report_select ("Removed objects overlapping field edge from selection" + removed_selected_text)
		end
	
	remove_from_selection_overlapping_object_items
			-- Remove from selection all objects which intersect
			-- at least one field of view and overlap another
			-- object
		do
			save_selected_count
			view.remove_from_selection_overlapping_object_items
			report_select ("Removed objects overlapping another object from selection" + removed_selected_text)
		end
	
	remove_from_selection_close_edge_items
			-- Remove from selection all objects which intersect
			-- at least one field of view and have at least one
			-- edge which is too close to field of view edge
		do
			save_selected_count
			view.remove_from_selection_close_edge_items
			report_select ("Removed objects too close to field edge from selection" + removed_selected_text)
		end
	
	remove_from_selection_close_object_items
			-- Remove from selection all objects which intersect
			-- at least one field of view and are too close to
			-- another object that intersects at least one field
		do
			save_selected_count
			view.remove_from_selection_close_object_items
			report_select ("Removed objects too close to another object from selection" + removed_selected_text)
		end
	
	remove_from_selection_too_big_items
			-- Remove from selection all projects which have a die
			-- size that exceeds their maximum die size
		do
			save_selected_count
			view.remove_from_selection_too_big_items
			report_select ("Removed projects whose die size is too big from selection" + removed_selected_text)
		end
	
	remove_from_selection_uncentered_items
			-- Remove from selection all projects which are not
			-- centered in their cut die
		do
			save_selected_count
			view.remove_from_selection_uncentered_items
			report_select ("Removed projects not centered in cut die from selection" + removed_selected_text)
		end
	
	remove_from_selection_unisolated_items
			-- Remove from selection all projects which are not
			-- isolated in their cut die
		do
			save_selected_count
			view.remove_from_selection_unisolated_items
			report_select ("Removed projects not isolated in cut die from selection" + removed_selected_text)
		end
	
	remove_from_selection_unassigned_lot_items
			-- Remove from selection all objects which
			-- have associated wafer etest ID(s) but have
			-- not been assigned a specific one
		do
			save_selected_count
			view.remove_from_selection_unassigned_lot_items
			report_select ("Removed projects with no lot assigned from selection" + removed_selected_text)
		end
	
	remove_from_selection_unlocked_or_unassigned
			-- Remove from selection all projects which are not
			-- locked against customer update or are not assigned
			-- to run being closed
		do
			save_selected_count
			view.remove_from_selection_unlocked_or_unassigned
			report_select ("Removed unlocked or unassigned to run projects from selection" + removed_selected_text)
		end
	
	remove_from_selection_all_copies_by_point (a_coordinate: EV_COORDINATE)
			-- Remove from selection all copies of every
			-- selectable item any of whose copies contains `a_coordinate'
		require
			a_coordinate_not_void: a_coordinate /= Void
		do
			save_selected_count
			view.remove_from_selection_all_copies_by_point (a_coordinate)
			report_select ("Removed all copies of objects containing mouse pointer from selection" + removed_selected_text)
		end

	fix_selected_items_in_x
			-- Mark the currently selected items as fixed in
			-- the X dimension, for horizontal scanning purposes
		do
			view.fix_selected_items_in_x
			report_lock ("Fixed selected objects in X" + selected_changed_count_text)
		end

	fix_selected_items_in_y
			-- Mark the currently selected items as fixed in
			-- the Y dimension, for vertical scanning purposes
		do
			view.fix_selected_items_in_y
			report_lock ("Fixed selected objects in Y" + selected_changed_count_text)
		end

	fix_selected_items_in_x_and_y
			-- Mark the currently selected items as fixed in
			-- both X and Y dimensions, for horizontal and
			-- vertical scanning purposes
		do
			view.fix_selected_items_in_x_and_y
			report_lock ("Fixed selected objects in X and Y" + selected_changed_count_text)
		end

	unfix_selected_items_in_x
			-- Mark the currently selected items as not fixed in
			-- the X dimension, for horizontal scanning purposes
		do
			view.unfix_selected_items_in_x
			report_lock ("Unfixed selected objects in X" + selected_changed_count_text)
		end

	unfix_selected_items_in_y
			-- Mark the currently selected items as not fixed in
			-- the Y dimension, for vertical scanning purposes
		do
			view.unfix_selected_items_in_y
			report_lock ("Unfixed selected objects in Y" + selected_changed_count_text)
		end

	unfix_selected_items_in_x_and_y
			-- Mark the currently selected items as not fixed in
			-- the X or Y dimension, for horizontal and vertical
			--  scanning purposes
		do
			view.unfix_selected_items_in_x_and_y
			report_lock ("Unfixed selected objects in X and Y" + selected_changed_count_text)
		end

	lock_all_items_in_x
			-- Mark all selectable items as locked in position
			-- in the X dimension
		do
			view.lock_all_items_in_x
			report_lock ("Locked all objects in X" + selectable_changed_count_text)
		end

	lock_all_items_in_y
			-- Mark all selectable items as locked in position
			-- the Y dimension
		do
			view.lock_all_items_in_y
			report_lock ("Locked all objects in Y" + selectable_changed_count_text)
		end

	lock_all_items_in_x_and_y
			-- Mark all selectable items as locked in position
			-- in the X and Y dimensions
		do
			view.lock_all_items_in_x_and_y
			report_lock ("Locked all objects in X and Y" + selectable_changed_count_text)
		end

	lock_all_items_orientation
			-- Mark all selectable items as locked in current
			-- orientation
		do
			view.lock_all_items_orientation
			report_lock ("Locked all objects in current orientation" + selectable_changed_count_text)
		end

	lock_all_items_to_prevent_duplication
			-- Mark all selectable items as locked to prevent
			-- duplication
		do
			view.lock_all_items_to_prevent_duplication
			report_lock ("Locked all objects to prevent duplication" + selectable_changed_count_text)
		end

	lock_all_items_to_prevent_deletion
			-- Mark all selectable items as locked to prevent
			-- deletion
		do
			view.lock_all_items_to_prevent_deletion
			report_lock ("Locked all objects to prevent deletion" + selectable_changed_count_text)
		end

	lock_all_items_almost_completely
			-- Mark all selectable items as almost completely
			-- locked (everything except duplication)
		do
			view.lock_all_items_almost_completely
			report_lock ("Locked all objects completely except for duplication" + selectable_changed_count_text)
		end

	lock_all_items_completely
			-- Mark all selectable items as completely locked
		do
			view.lock_all_items_completely
			report_lock ("Locked all objects completely" + selectable_changed_count_text)
		end

	lock_selected_items_in_x
			-- Mark selected items as locked in position
			-- in the X dimension
		do
			view.lock_selected_items_in_x
			report_lock ("Locked selected objects in X" + selected_changed_count_text)
		end

	lock_selected_items_in_y
			-- Mark selected items as locked in position
			-- the Y dimension
		do
			view.lock_selected_items_in_y
			report_lock ("Locked selected objects in Y" + selected_changed_count_text)
		end

	lock_selected_items_in_x_and_y
			-- Mark selected items as locked in position
			-- in the X and Y dimensions
		do
			view.lock_selected_items_in_x_and_y
			report_lock ("Locked selected objects in X and Y" + selected_changed_count_text)
		end

	lock_selected_items_orientation
			-- Mark selected items as locked in current
			-- orientation
		do
			view.lock_selected_items_orientation
			report_lock ("Locked selected objects in current orientation" + selected_changed_count_text)
		end

	lock_selected_items_to_prevent_duplication
			-- Mark selected items as locked to prevent
			-- duplication
		do
			view.lock_selected_items_to_prevent_duplication
			report_lock ("Locked selected objects to prevent duplication" + selected_changed_count_text)
		end

	lock_selected_items_to_prevent_deletion
			-- Mark selected items as locked to prevent
			-- deletion
		do
			view.lock_selected_items_to_prevent_deletion
			report_lock ("Locked selected objects to prevent deletion" + selected_changed_count_text)
		end

	lock_selected_items_almost_completely
			-- Mark selected items as locked almost completely
			-- (everything but duplication)
		do
			view.lock_selected_items_almost_completely
			report_lock ("Locked selected objects completely except for duplication" + selected_changed_count_text)
		end

	lock_selected_items_completely
			-- Mark selected items as locked completely
		do
			view.lock_selected_items_completely
			report_lock ("Locked selected objects completely" + selected_changed_count_text)
		end

	lock_unselected_items_in_x
			-- Mark unselected items as locked in position
			-- in the X dimension
		do
			view.lock_unselected_items_in_x
			report_lock ("Locked unselected objects in X" + unselected_changed_count_text)
		end

	lock_unselected_items_in_y
			-- Mark unselected items as locked in position
			-- the Y dimension
		do
			view.lock_unselected_items_in_y
			report_lock ("Locked unselected objects in Y" + unselected_changed_count_text)
		end

	lock_unselected_items_in_x_and_y
			-- Mark unselected items as locked in position
			-- in the X and Y dimensions
		do
			view.lock_unselected_items_in_x_and_y
			report_lock ("Locked unselected objects in X and Y" + unselected_changed_count_text)
		end

	lock_unselected_items_orientation
			-- Mark unselected items as locked in current
			-- orientation
		do
			view.lock_unselected_items_orientation
			report_lock ("Locked unselected objects in current orientation" + unselected_changed_count_text)
		end

	lock_unselected_items_to_prevent_duplication
			-- Mark unselected items as locked to prevent
			-- duplication
		do
			view.lock_unselected_items_to_prevent_duplication
			report_lock ("Locked unselected objects to prevent duplication" + unselected_changed_count_text)
		end

	lock_unselected_items_to_prevent_deletion
			-- Mark unselected items as locked to prevent
			-- deletion
		do
			view.lock_unselected_items_to_prevent_deletion
			report_lock ("Locked unselected objects to prevent deletion" + unselected_changed_count_text)
		end

	lock_unselected_items_almost_completely
			-- Mark unselected items as locked almost completely
			-- (everything but duplication)
		do
			view.lock_unselected_items_almost_completely
			report_lock ("Locked unselected objects completely except for duplication" + unselected_changed_count_text)
		end

	lock_unselected_items_completely
			-- Mark unselected items as locked completely
		do
			view.lock_unselected_items_completely
			report_lock ("Locked unselected objects completely" + unselected_changed_count_text)
		end

	unlock_all_items_in_x
			-- Mark all non-stationary items as unlocked
			-- in position in the X dimension
		do
			view.unlock_all_items_in_x
			report_lock ("Unlocked all objects in X" + selectable_changed_count_text)
		end

	unlock_all_items_in_y
			-- Mark all non-stationary items as unlocked
			-- in position the Y dimension
		do
			view.unlock_all_items_in_y
			report_lock ("Unlocked all objects unlocked in Y" + selectable_changed_count_text)
		end

	unlock_all_items_in_x_and_y
			-- Mark all non-stationary items as unlocked
			-- in position in the X and Y dimensions
		do
			view.unlock_all_items_in_x_and_y
			report_lock ("Unlocked all objects in X and Y" + selectable_changed_count_text)
		end

	unlock_all_items_orientation
			-- Mark all items whose orientation is not stationary
			-- as unlocked in orientation
		do
			view.unlock_all_items_orientation
			report_lock ("Unlocked all objects orientation" + selectable_changed_count_text)
		end

	unlock_all_items_to_allow_duplication
			-- Mark all items as unlocked to allow duplication
		do
			view.unlock_all_items_to_allow_duplication
			report_lock ("Unlocked all objects to allow duplication" + selectable_changed_count_text)
		end

	unlock_all_items_to_allow_deletion
			-- Mark all items as unlocked to allow deletion
		do
			view.unlock_all_items_to_allow_deletion
			report_lock ("Unlocked all objects to allow deletion" + selectable_changed_count_text)
		end

	unlock_all_items_almost_completely
			-- Mark all items as unlocked almost completely
			-- (everything but duplication)
		do
			view.unlock_all_items_almost_completely
			report_lock ("Unlocked all objects completely except for duplication" + selectable_changed_count_text)
		end

	unlock_all_items_completely
			-- Mark all items as unlocked completely
		do
			view.unlock_all_items_completely
			report_lock ("Unlocked all objects completely" + selectable_changed_count_text)
		end

	unlock_selected_items_in_x
			-- Mark selected non-stationary items as
			-- unlocked in position in the X dimension
		do
			view.unlock_selected_items_in_x
			report_lock ("Unlocked selected objects in X" + selected_changed_count_text)
		end

	unlock_selected_items_in_y
			-- Mark selected non-stationary items as
			-- unlocked in position the Y dimension
		do
			view.unlock_selected_items_in_y
			report_lock ("Unlocked selected objects in Y" + selected_changed_count_text)
		end

	unlock_selected_items_in_x_and_y
			-- Mark selected non-stationary items as
			-- unlocked in position in the X and Y
			-- dimensions
		do
			view.unlock_selected_items_in_x_and_y
			report_lock ("Unlocked selected objects in X and Y" + selected_changed_count_text)
		end

	unlock_selected_items_orientation
			-- Mark selected items whose orientation is
			-- not stationary as unlocked in orientation
		do
			view.unlock_selected_items_orientation
			report_lock ("Unlocked selected objects orientation" + selected_changed_count_text)
		end

	unlock_selected_items_to_allow_duplication
			-- Mark selected items as unlocked to allow
			-- duplication
		do
			view.unlock_selected_items_to_allow_duplication
			report_lock ("Unlocked selected objects to allow duplication" + selected_changed_count_text)
		end

	unlock_selected_items_to_allow_deletion
			-- Mark selected items as unlocked to allow deletion
		do
			view.unlock_selected_items_to_allow_deletion
			report_lock ("Unlocked selected objects to allow deletion" + selected_changed_count_text)
		end

	unlock_selected_items_almost_completely
			-- Mark selected items as unlocked almost completely
			-- (everything but duplication)
		do
			view.unlock_selected_items_almost_completely
			report_lock ("Unlocked selected objects completely except for duplication" + selected_changed_count_text)
		end

	unlock_selected_items_completely
			-- Mark selected items as unlocked completely
		do
			view.unlock_selected_items_completely
			report_lock ("Unlocked selected objects completely" + selected_changed_count_text)
		end

	unlock_unselected_items_in_x
			-- Mark unselected non-stationary items as
			-- unlocked in position in the X dimension
		do
			view.unlock_unselected_items_in_x
			report_lock ("Unlocked unselected objects in X" + unselected_changed_count_text)
		end

	unlock_unselected_items_in_y
			-- Mark unselected non-stationary items as
			-- unlocked in position the Y dimension
		do
			view.unlock_unselected_items_in_y
			report_lock ("Unlocked unselected objects in Y" + unselected_changed_count_text)
		end

	unlock_unselected_items_in_x_and_y
			-- Mark unselected non-stationary items as
			-- unlocked in position in the X and Y
			-- dimensions
		do
			view.unlock_unselected_items_in_x_and_y
			report_lock ("Unlocked unselected objects in X and Y" + unselected_changed_count_text)
		end

	unlock_unselected_items_orientation
			-- Mark unselected items whose orientation is
			-- not stationary as unlocked in orientation
		do
			view.unlock_unselected_items_orientation
			report_lock ("Unlocked unselected objects orientation" + unselected_changed_count_text)
		end

	unlock_unselected_items_to_allow_duplication
			-- Mark unselected items as unlocked to allow
			-- duplication
		do
			view.unlock_unselected_items_to_allow_duplication
			report_lock ("Unlocked unselected objects to allow duplication" + unselected_changed_count_text)
		end

	unlock_unselected_items_to_allow_deletion
			-- Mark unselected items as unlocked to allow deletion
		do
			view.unlock_unselected_items_to_allow_deletion
			report_lock ("Unlocked unselected objects to allow deletion" + unselected_changed_count_text)
		end

	unlock_unselected_items_almost_completely
			-- Mark unselected items as unlocked almost completely
			-- (everything but duplication)
		do
			view.unlock_unselected_items_almost_completely
			report_lock ("Unlocked unselected objects completely except for duplication" + unselected_changed_count_text)
		end

	unlock_unselected_items_completely
			-- Mark unselected items as unlocked completely
		do
			view.unlock_unselected_items_completely
			report_lock ("Unlocked unselected objects completely" + unselected_changed_count_text)
		end

	convert_fixed_items_to_locked
			-- Change any items that are fixed in X to instead be
			-- locked in X and similarly for the Y dimension
		do
			view.convert_fixed_items_to_locked
			report_lock ("Converted all fixed objects to locked objects" + selectable_changed_count_text)
		end

	convert_locked_items_to_fixed
			-- Change any items that are locked in X to instead be
			-- fixed in X and similarly for the Y dimension
		do
			view.convert_locked_items_to_fixed
			report_lock ("Converted all locked objects to fixed objects" + selectable_changed_count_text)
		end

	measure_selected_items_in_x
			-- Display sum of widths of selected objects,
			-- width of payload and available space in X
		local
			field_x, used, avail, half: DOUBLE
			spacer: STRING
		do
			view.measure_selected_items
			if not view.fields.is_empty then
				field_x := view.fields.first.size_x
				used := view.measured_sum_x
				avail := field_x - used
				half := avail / 2.0
				spacer := "  "
				report_status ("Payload X = " + grid.formatted_number (field_x) + spacer + "Used X = " + grid.formatted_number (used) + spacer + "Available X = " + grid.formatted_number (avail) + spacer + "(half = " + grid.formatted_number (half) + ")")
			else
				report_failure ("Measure in X failed - no fields included in view")
			end
			report_end
		end

	measure_selected_items_in_y
			-- Display sum of heights of selected objects,
			-- height of payload and available space in y
		local
			field_y, used, avail, half: DOUBLE
			spacer: STRING
		do
			view.measure_selected_items
			if not view.fields.is_empty then
				field_y := view.fields.first.size_y
				used := view.measured_sum_y
				avail := field_y - used
				half := avail / 2.0
				spacer := "  "
				report_status ("Payload Y = " + grid.formatted_number (field_y) + spacer + "Used Y = " + grid.formatted_number (used) + spacer + "Available Y = " + grid.formatted_number (avail) + spacer + "(half = " + grid.formatted_number (half) + ")")
			else
				report_failure ("Measure in Y failed - no fields included in view")
			end
			report_end
		end

	finish
			-- Finish packing.  Check that everything is legal
			-- and warn about any possible problems first
		local
			op: ANONYMOUS_OPERATION
			n: INTEGER
		do
			n := view.under_construction_cut_pattern_count
			if n > 0 then
				report_failure ("Cannot Finish - " + n.out + " cut " + su.number (n, "pattern", "patterns") + " still Under Construction")
			else
				report_status ("Processing Finish command")
				create op.make (agent confirm_finish)
				op.enable_abort_caught
				op.execute
				if op.aborted then
					report_cancellation ("Finish command aborted")
				else
					main_window.process_finish
					if main_window.is_finish_completed then
						report_exit ("Finish command completed")
					else
						report_cancellation ("Finish command aborted")
					end
				end
			end
			report_end
		end

	quit
			-- Quit packing without finishing.  Warn about
			-- unsaved changes before quitting
		local
			op: ANONYMOUS_OPERATION
		do
			report_status ("Processing Quit command")
			create op.make (agent confirm_quit)
			op.enable_abort_caught
			op.execute
			if op.aborted then
				report_cancellation ("Quit command aborted")
			else
				main_window.process_cancel
				if not main_window.is_quit_completed then
					report_cancellation ("Quit command aborted")
				end
			end
			report_end
		end

	save_new_version
			-- Save current packing state as a new version
		do
			report_status ("Saving new version of packing")
			main_window.process_save (False)
			if main_window.is_new_version_saved then
				report_change ("Saved new version")
			else
				report_cancellation ("Save cancelled by user or not supported or no changes to save")
			end
			report_end
		end

	undo_one_change
			-- Undo previous history list change made to packing
		do
			view.undo_one_change
			if not view.had_error then
				report_change ("Undid " + view.last_change.description)
			else
				report_failure ("Undo failed - " + view.error_messages.first)
			end
			report_end
		end

	redo_one_change
			-- Redo next history list change made to packing
		do
			view.redo_one_change
			if not view.had_error then
				report_change ("Redid " + view.last_change.description)
			else
				report_failure ("Redo failed - " + view.error_messages.first)
			end
			report_end
		end

	undo_last_change
			-- Undo last change made to packing.  If at end
			-- of change history list, undo one change.  Otherwise,
			-- redo one change.  Report error if history list is
			-- empty
		local
			hist_list: LIST [RETICLE_CHANGE]
		do
			hist_list := view.reticle_change_history_list
			if hist_list.is_empty then
				report_failure ("Undo failed - change history list is empty")
			elseif hist_list.after then
				undo_one_change
			else
				redo_one_change
			end
			report_end
		end

	horizontal_scan_selected_items
			-- Scan currently selected items and space them out
			-- evenly in the X dimension, taking into account items
			-- that are fixed in X and the field of view edges
		local
			n: INTEGER
		do
			if view.selected_count > 0 then
				view.horizontal_scan_selected_items
				if view.is_scan_valid then
					if view.had_warning then
						n := view.warning_messages.count
						report_failure ("Scanned selected objects horizontally (" + n.out + " " + su.number (n, "warning", "warnings") + ")")
					else
						report_change ("Scanned selected objects horizontally")
					end
					display_messages
				else
					report_scan_error_summary ("Horizontal Scan")
					display_warnings
				end
			else
				report_failure ("Horizontal Scan not allowed - no objects selected")
			end
			report_end
		end

	vertical_scan_selected_items
			-- Scan currently selected items and space them out
			-- evenly in the Y dimension, taking into account items
			-- that are fixed in Y and the field of view edges
		local
			n: INTEGER
		do
			if view.selected_count > 0 then
				view.vertical_scan_selected_items
				if view.is_scan_valid then
					if view.had_warning then
						n := view.warning_messages.count
						report_failure ("Scanned selected objects vertically (" + n.out + " " + su.number (n, "warning", "warnings") + ")")
					else
						report_change ("Scanned selected objects vertically")
					end
					display_messages
				else
					report_scan_error_summary ("Vertical Scan")
					display_warnings
				end
			else
				report_failure ("Vertical Scan not allowed - no objects selected")
			end
			report_end
		end

	move_absolute_selected_items
			-- Move currently selected items to absolute location
			-- relative to center of a field
		local
			n: INTEGER
		do
			n := view.selected_count
			if n = 1 then
			 	view.set_selected_element
				main_window.show_move_absolute_window (view.selected_element, view.reticle_fields, view.max_db_value)
				if is_last_move_valid then
					view.move_absolute_selected_items (last_field_name, last_move_x, last_move_y)
					if not view.had_error then
						report_change ("Moved " + view.selected_element.packing_name + " by " + last_move_distance_text + " to " + grid.formatted_point (create {CONST_POINT}.make_xy (last_move_x, last_move_y)) + " from field " + last_field_name + " center")
					else
						report_failure ("Move Absolute not allowed - " + view.error_messages.first)
					end
				else
					report_cancellation ("Move Absolute cancelled")
				end
			elseif n = 0 then
				report_failure ("Move Absolute not allowed - no objects selected")
			else
				report_failure ("Move Absolute not allowed - more than one object selected")
			end
			report_end
		end

	move_absolute (a_field_name: STRING a_x, a_y: DOUBLE)
			-- Set information for Move Absolute operation
			-- to be done
		do
			last_field_name := a_field_name
			last_move_x := a_x
			last_move_y := a_y
			is_last_move_valid := True
		end

	cancel_move_absolute
			-- Cancel Move Absolute operation when user clicks
			-- Cancel in dialog
		do
			is_last_move_valid := False
		end

	move_relative_selected_items
			-- Move currently selected items by amount relative
			-- to their current positions
		local
			n: INTEGER
		do
			n := view.selected_count
			if n > 0 then
				main_window.show_move_relative_window
				if is_last_move_valid then
					view.move_relative_selected_items (last_move_x, last_move_y)
					if not view.had_error then
						report_change ("Moved " + n.out + " " + su.number (n, "object", "objects") + " by " + last_move_distance_text)
					else
						report_failure ("Move Relative not allowed - " + view.error_messages.first)
					end
				else
					report_cancellation ("Move Relative cancelled")
				end
			else
				report_failure ("Move Relative not allowed - no objects selected")
			end
			report_end
		end

	move_relative (a_x, a_y: DOUBLE)
			-- Set information for Move Relataive operation
			-- to be done
		do
			last_move_x := a_x
			last_move_y := a_y
			is_last_move_valid := True
		end

	cancel_move_relative
			-- Cancel Move Relative operation when user clicks
			-- Cancel in dialog
		do
			is_last_move_valid := False
		end

	center_all_projects_in_cut_die
			-- Center all projects in their cut die, ignoring
			-- projects which are not isolated in their
			-- cut die
		local
			n: INTEGER
		do
			view.center_all_projects_in_cut_die
			if not view.had_error then
				n := view.centered_count
				report_change ("Centered " + n.out + " " + su.number (n, "object", "objects") + " in cut die")
			else
				report_failure ("Center All not allowed - " + view.error_messages.first)
			end
			report_end
		end

	center_selected_projects_in_cut_die
			-- Center selected projects in their cut die,
			-- ignoring projects which are not isolated in
			-- their cut die
		local
			n: INTEGER
		do
			if view.selected_count > 0 then
				view.center_selected_projects_in_cut_die
				n := view.centered_count
				if not view.had_error then
					report_change ("Centered " + n.out + " " + su.number (n, "object", "objects") + " in cut die")
				else
					report_failure ("Center Selected not allowed - " + view.error_messages.first)
				end
			else
				report_failure ("Center Selected not allowed - no objects selected")
			end
			report_end
		end

	join_scribes
			-- Trim scribes at field of view edge, separate
			-- scribes that span multiple fields into one scribe
			-- per field, extend scribes to next meetable edge
			-- and remove scribe overlaps without changing area
			-- covered by scribes
		local
			trimmed_count, separated_count, extended_count: INTEGER
		do
			view.check_scribe_join_validity
			if view.is_scribe_join_valid then
				report_status ("Join Scribes started")
				view.start_join
				view.trim_all_scribes (False)
				trimmed_count := view.changed_scribes.count
				view.separate_all_scribes (False)
				separated_count := view.changed_scribes.count
				view.extend_all_scribes (False)
				extended_count := view.changed_scribes.count
				view.unoverlap_all_scribes (False)
				view.end_join
				report_join_summary (trimmed_count, separated_count, extended_count)
			else
				report_join_error_summary
			end
			report_end
		end

	delete_selected_items
			-- Delete from packing all currently selected items
			-- that are deletable
		do
			delete_selected (False)
		end

	delete_selected_scribes
			-- Delete from packing all currently selected scribes
			-- that are deletable.  Ignore selected items that
			-- are not scribes
		do
			delete_selected (True)
		end

	set_delete_properties (wants_unlock, wants_deassign: BOOLEAN)
			-- Set disposition of projects whose last copy is to
			-- be deleted from `wants_unlock' and `wants_deassign'
		do
			is_unlock_requested := wants_unlock
			is_deassign_requested := wants_deassign
			is_last_delete_valid := True
		end

	set_deleted_dies (a_list: like deleted_dies)
			-- Set `deleted_dies' to `a_list'
		do
			deleted_dies := a_list
		end

	cancel_delete
			-- Cancel deletion when user clicks Cancel
			-- in delete projects dialog
		do
			is_last_delete_valid := False
		end

	trim_all_scribes
			-- Resize all scribes that intersect at least
			-- one field of view so that ends which are outside
			-- any field of view are "trimmed" back to field of
			-- view edge
		do
			view.trim_all_scribes (True)
			report_trim_error_summary
			display_messages
		end

	trim_selected_scribes
			-- Resize selected scribes that intersect at least
			-- one field of view so that ends which are outside
			-- any field of view are "trimmed" back to field of
			-- view edge
		do
			view.trim_selected_scribes (True)
			report_trim_error_summary
			display_messages
		end

	separate_all_scribes
			-- Replace every scribe that intersects more
			-- than one field of view with a set of
			-- scribes that represents the intersection of
			-- the scribe with each field
		do
			view.separate_all_scribes (True)
			report_separate_error_summary
			display_messages
		end

	separate_selected_scribes
			-- Replace selected scribes that intersect more
			-- than one field of view with a set of
			-- scribes that represents the intersection of
			-- the scribes with each field
		do
			view.separate_selected_scribes (True)
			report_separate_error_summary
			display_messages
		end

	extend_all_scribes
			-- Extend every scribe to meet a scribe of different
			-- orientation or field of view edge
		do
			view.extend_all_scribes (True)
			report_extend_error_summary
			display_messages
		end

	extend_selected_scribes
			-- Extend selected scribe to meet a scribe of different
			-- orientation or field of view edge
		do
			view.extend_selected_scribes (True)
			report_extend_error_summary
			display_messages
		end

	unoverlap_all_scribes
			-- Remove overlaps of all scribes with other scribes
			-- of the opposite orientation
		do
			view.unoverlap_all_scribes (True)
			report_unoverlap_error_summary
			display_messages
		end

	unoverlap_selected_scribes
			-- Remove overlaps of selected scribes with
			-- other selected scribes of the opposite
			-- orientation
		do
			view.unoverlap_selected_scribes (True)
			report_unoverlap_error_summary
			display_messages
		end

	check_all_pack_validity
			-- Check that packing is valid and report any errors
			-- in Pack mode
		local
			old_style: EV_POINTER_STYLE
		do
			old_style := main_window.pointer_style
			main_window.set_pointer_style (gui.Wait_cursor)
			gui.process_events
			view.check_all_pack_validity
			report_validity_check_summary ("Checked for all validity issues")
			main_window.set_pointer_style (old_style)
		end

	check_almost_all_validity
			-- Check that packing is valid and report any errors.
			-- Skip check for objects outside all fields
		do
			view.check_almost_all_pack_validity
			report_validity_check_summary ("Checked for all validity issues except objects outside fields")
		end

	check_before_join_scribes_validity
			-- Check that packing is valid and report any errors.
			-- Skip check for objects outside all fields and
			-- all checks for problems that are likely to be
			-- fixed by a Join Scribes operation
		do
			view.check_before_join_scribes_validity
			report_validity_check_summary ("Checked for validity issues not likely to be fixed by Join Scribes")
		end

	check_all_cut_validity
			-- Check that packing is valid and report any errors
			-- in Cut mode
		local
			old_style: EV_POINTER_STYLE
		do
			if view.has_current_cut_pattern then
				old_style := main_window.pointer_style
				main_window.set_pointer_style (gui.Wait_cursor)
				gui.process_events
				view.check_all_cut_validity
				report_validity_check_summary ("Checked for all validity issues")
				main_window.set_pointer_style (old_style)
			else
				report_failure ("Cannot check all validity issues" + no_cut_pattern_text)
			end
		end

	check_outside_fields_items
			-- Check for objects that are completely outside all
			-- fields of view
		do
			view.check_outside_fields_items
			report_validity_check_summary ("Checked for objects outside all fields")
		end

	check_overlapping_field_edge_items
			-- Check for objects that intersect at least one
			-- field of view but are not enclosed by any field
		do
			view.check_overlapping_field_edge_items
			report_validity_check_summary ("Checked for objects overlapping field edge")
		end

	check_overlapping_object_items
			-- Check for objects that intersect at least one
			-- field of view and overlap another object that
			-- intersects at least one field of view
		do
			view.check_overlapping_object_items
			report_validity_check_summary ("Checked for objects overlapping another object")
		end

	check_below_minimum_size_items
			-- Check for objects that are below minimum legal
			-- size in either dimension
		do
			view.check_below_minimum_size_items
			report_validity_check_summary ("Checked for objects below minimum size of " + grid.formatted_number (view.minimum_die_size))
		end

	check_lot_not_assigned_items
			-- Check for objects that are customer projects for
			-- which a specific lot has not been assigned
		do
			view.check_lot_not_assigned_items
			report_validity_check_summary ("Checked for projects with no lot assigned")
		end

	check_unlocked_or_unassigned_projects
			-- Check for projects that are not locked against
			-- customer update or are not assigned to run being
			-- closed
		do
			view.check_unlocked_or_unassigned_projects
			report_validity_check_summary ("Checked for unlocked or not assigned to run projects")
		end

	check_close_to_field_edge_items
			-- Check for objects that intersect at least one
			-- field of view and have at least one edge that
			-- is too close to the corresponding field edge
		do
			view.check_close_to_field_edge_items
			report_validity_check_summary ("Checked for objects too close to field edge")
		end

	check_close_to_object_items
			-- Check for objects that intersect at least one
			-- field of view and are too close to another object
			-- that also intersects at least one field
		do
			view.check_close_to_object_items
			report_validity_check_summary ("Checked for objects too close to another object")
		end

	check_scribe_cut_die_too_big_items
			-- Check for projects that intersect at least
			-- one field of view and have a cut die size
			-- determined by surrounding scribes that
			-- exceeds their maximum die size
		do
			view.check_scribe_cut_die_too_big_items
			report_validity_check_summary ("Checked for projects whose cut die size is too big")
		end

	check_unisolated_in_cut_die_items
			-- Check for projects that are not isolated in their
			-- cut dies
		do
			view.check_unisolated_in_cut_die_items
			report_validity_check_summary ("Checked for projects not isolated in cut die")
		end

	check_uncentered_in_cut_die_items
			-- Check for projects that are not centered in their
			-- cut dies, using edges of surrounding scribes instead
			-- of centers
		do
			view.check_uncentered_in_cut_die_items
			report_validity_check_summary ("Checked for projects not centered in cut die")
		end

	check_floating_scribes
			-- Check for floating scribes (ones that do not abut
			-- either the field of view edge or a scribe of
			-- opposite orientation at either end)
		do
			view.check_floating_scribes
			report_validity_check_summary ("Checked for floating scribes")
		end

	check_misaligned_scribes
			-- Check for misaligned scribes (ones that do not
			-- quite line up)
		do
			view.check_misaligned_scribes
			report_validity_check_summary ("Checked for misaligned scribes")
		end

	check_small_scribes_and_fillers
			-- Check for scribes and fillers that are legal
			-- but are very small and so possibly erroneous
		do
			view.check_small_scribes_and_fillers
			report_validity_check_summary ("Checked for small scribes and fillers")
		end

	check_test_vehicles
			-- Check for test vehicle issues with packing
		do
			view.check_test_vehicles
			report_validity_check_summary ("Checked for test vehicle issues")
		end

	check_custom_width_scribes
			-- Check for custom width scribes and warn if any
			-- present
		do
			view.check_custom_width_scribes
			report_validity_check_summary ("Checked for custom width scribes")
		end

	check_invalid_cut_die_items
			-- Check for projects that are isolated in cut pattern
			-- but cut die determined by surrounding cuts is
			-- invalid (not surrounded by cuts on all sides or
			-- has cut through the printed die)
		local
			old_style: EV_POINTER_STYLE
		do
			if view.has_current_cut_pattern then
				old_style := main_window.pointer_style
				main_window.set_pointer_style (gui.Wait_cursor)
				gui.process_events
				view.check_invalid_cut_die_items
				report_validity_check_summary ("Checked for isolated sites with invalid cut dies")
				main_window.set_pointer_style (old_style)
			else
				report_failure ("Cannot check for isolated sites with invalid cut dies" + no_cut_pattern_text)
			end
		end

	check_wrong_lot_items
			-- Check for projects that have the wrong lot
			-- (different than the one for the cut pattern)
		local
			old_style: EV_POINTER_STYLE
		do
			if view.has_current_cut_pattern then
				old_style := main_window.pointer_style
				main_window.set_pointer_style (gui.Wait_cursor)
				gui.process_events
				view.check_wrong_lot_items
				report_validity_check_summary ("Checked for isolated sites with wrong lot")
				main_window.set_pointer_style (old_style)
			else
				report_failure ("Cannot check for isolated sites with wrong lot" + no_cut_pattern_text)
			end
		end

	check_cut_cut_die_too_big_items
			-- Check for projects that intersect at least
			-- one field of view and have a cut die size
			-- determined by surrounding cuts that
			-- exceeds their maximum die size
		local
			old_style: EV_POINTER_STYLE
		do
			if view.has_current_cut_pattern then
				old_style := main_window.pointer_style
				main_window.set_pointer_style (gui.Wait_cursor)
				gui.process_events
				view.check_cut_cut_die_too_big_items
				report_validity_check_summary ("Checked for isolated sites whose cut die determined by cuts is too big")
				main_window.set_pointer_style (old_style)
			else
				report_failure ("Cannot check for isolated sites with cut die size too big" + no_cut_pattern_text)
			end
		end

	check_non_default_cut_die_size_items
			-- Check for isolated elements that have a die size
			-- determined by surrounding cuts that is different
			-- than the die size determined by surrounding scribes
		local
			old_style: EV_POINTER_STYLE
		do
			if view.has_current_cut_pattern then
				old_style := main_window.pointer_style
				main_window.set_pointer_style (gui.Wait_cursor)
				gui.process_events
				view.check_non_default_cut_die_size_items
				report_validity_check_summary ("Checked for isolated sites whose cut die determined by cuts is different from cut die determined by scribes")
				main_window.set_pointer_style (old_style)
			else
				report_failure ("Cannot check for isolated sites with non-default cut die size" + no_cut_pattern_text)
			end
		end

	check_shared_cut_die_items
			-- Check for isolated elements that have a cut
			-- die determined by surrounding cuts that is
			-- shared by partially or fully by one or more
			-- other non-scribe die elements
		local
			old_style: EV_POINTER_STYLE
		do
			if view.has_current_cut_pattern then
				old_style := main_window.pointer_style
				main_window.set_pointer_style (gui.Wait_cursor)
				gui.process_events
				view.check_shared_cut_die_items
				report_validity_check_summary ("Checked for isolated sites whose cut die includes all or part of another die")
				main_window.set_pointer_style (old_style)
			else
				report_failure ("Cannot check for isolated sites that share cut die with others" + no_cut_pattern_text)
			end
		end

	check_extra_cut_items
			-- Check for cuts that are not an isolating cut
			-- for any isolated site
		local
			old_style: EV_POINTER_STYLE
		do
			if view.has_current_cut_pattern then
				old_style := main_window.pointer_style
				main_window.set_pointer_style (gui.Wait_cursor)
				gui.process_events
				view.check_extra_cut_items
				report_validity_check_summary ("Checked for extra cuts that do not help isolate any isolated site")
				main_window.set_pointer_style (old_style)
			else
				report_failure ("Cannot check for extra cuts" + no_cut_pattern_text)
			end
		end

	toggle_rotation_of_selected_items
			-- Toggle rotation of all currently selected
			-- items between unrotated (0 degrees) and
			-- rotated (rotated 90 degrees clockwise).
			-- Issue error if any unrotatable items are
			-- selected or if not all copies of an item
			-- are selected
		do
			view.toggle_rotation_of_selected_items
			report_rotate_summary ("Rotated")
			display_messages
		end

	rotate_selected_items_90_clockwise
			-- Rotate all currently selected items 90
			-- degrees clockwise
		do
			view.rotate_selected_items_90_clockwise
			report_rotate_summary ("Rotated Clockwise")
			display_messages
		end

	spread_selected_items_horizontally_by_asc_width (a_coordinate: EV_COORDINATE)
			-- Spread all currently selected items apart
			-- so that they do not overlap, arrange them
			-- horizontally sorted by ascending width and
			-- place lower left corner of bounding box at
			-- view coordinates `a_coordinate'
		require
			a_coordinate_not_void: a_coordinate /= Void
		do
			if view.selected_count > 0 then
				view.spread_selected_items_horizontally_by_asc_width (a_coordinate)
				if view.is_move_valid then
					report_change ("Spread selected objects horizontally by ascending width" + selected_count_text)
				else
					report_spread_error_summary ("Horizontal Spread By Ascending Width")
				end
			else
				report_failure ("Horizontal Spread By Ascending Width not allowed - no objects selected")
			end
			report_end
		end

	spread_selected_items_horizontally_by_desc_width (a_coordinate: EV_COORDINATE)
			-- Spread all currently selected items apart
			-- so that they do not overlap, arrange them
			-- horizontally sorted by descending width and
			-- place lower left corner of bounding box at
			-- view coordinates `a_coordinate'
		require
			a_coordinate_not_void: a_coordinate /= Void
		do
			if view.selected_count > 0 then
				view.spread_selected_items_horizontally_by_desc_width (a_coordinate)
				if view.is_move_valid then
					report_change ("Spread selected objects horizontally by descending width" + selected_count_text)
				else
					report_spread_error_summary ("Horizontal Spread By Descending Width")
				end
			else
				report_failure ("Horizontal Spread By Descending Width not allowed - no objects selected")
			end
			report_end
		end

	spread_selected_items_horizontally_by_asc_height (a_coordinate: EV_COORDINATE)
			-- Spread all currently selected items apart
			-- so that they do not overlap, arrange them
			-- horizontally sorted by ascending height and
			-- place lower left corner of bounding box at
			-- view coordinates `a_coordinate'
		require
			a_coordinate_not_void: a_coordinate /= Void
		do
			if view.selected_count > 0 then
				view.spread_selected_items_horizontally_by_asc_height (a_coordinate)
				if view.is_move_valid then
					report_change ("Spread selected objects horizontally by ascending height" + selected_count_text)
				else
					report_spread_error_summary ("Horizontal Spread By Ascending Height")
				end
			else
				report_failure ("Horizontal Spread By Ascending Height not allowed - no objects selected")
			end
			report_end
		end

	spread_selected_items_horizontally_by_desc_height (a_coordinate: EV_COORDINATE)
			-- Spread all currently selected items apart
			-- so that they do not overlap, arrange them
			-- horizontally sorted by descending height and
			-- place lower left corner of bounding box at
			-- view coordinates `a_coordinate'
		require
			a_coordinate_not_void: a_coordinate /= Void
		do
			if view.selected_count > 0 then
				view.spread_selected_items_horizontally_by_desc_height (a_coordinate)
				if view.is_move_valid then
					report_change ("Spread selected objects horizontally by descending height" + selected_count_text)
				else
					report_spread_error_summary ("Horizontal Spread By Descending Height")
				end
			else
				report_failure ("Horizontal Spread By Descending Height not allowed - no objects selected")
			end
			report_end
		end

	spread_selected_items_vertically_by_asc_width (a_coordinate: EV_COORDINATE)
			-- Spread all currently selected items apart
			-- so that they do not overlap, arrange them
			-- vertically sorted by ascending width and
			-- place lower left corner of bounding box at
			-- view coordinates `a_coordinate'
		require
			a_coordinate_not_void: a_coordinate /= Void
		do
			if view.selected_count > 0 then
				view.spread_selected_items_vertically_by_asc_width (a_coordinate)
				if view.is_move_valid then
					report_change ("Spread selected objects vertically by ascending width" + selected_count_text)
				else
					report_spread_error_summary ("Vertical Spread By Ascending Width")
				end
			else
				report_failure ("Vertical Spread By Ascending Width not allowed - no objects selected")
			end
			report_end
		end

	spread_selected_items_vertically_by_desc_width (a_coordinate: EV_COORDINATE)
			-- Spread all currently selected items apart
			-- so that they do not overlap, arrange them
			-- vertically sorted by descending width and
			-- place lower left corner of bounding box at
			-- view coordinates `a_coordinate'
		require
			a_coordinate_not_void: a_coordinate /= Void
		do
			if view.selected_count > 0 then
				view.spread_selected_items_vertically_by_desc_width (a_coordinate)
				if view.is_move_valid then
					report_change ("Spread selected objects vertically by descending width" + selected_count_text)
				else
					report_spread_error_summary ("Vertical Spread By Descending Width")
				end
			else
				report_failure ("Vertical Spread By Descending Width not allowed - no objects selected")
			end
			report_end
		end

	spread_selected_items_vertically_by_asc_height (a_coordinate: EV_COORDINATE)
			-- Spread all currently selected items apart so that
			-- they do not overlap, arrange them vertically
			-- sorted by ascending height and place lower left corner of
			-- bounding box at view coordinates `a_coordinate'
		require
			a_coordinate_not_void: a_coordinate /= Void
		do
			if view.selected_count > 0 then
				view.spread_selected_items_vertically_by_asc_height (a_coordinate)
				if view.is_move_valid then
					report_change ("Spread selected objects vertically by ascending height" + selected_count_text)
				else
					report_spread_error_summary ("Vertical Spread By Ascending Height")
				end
			else
				report_failure ("Vertical Spread By Ascending Height not allowed - no objects selected")
			end
			report_end
		end

	spread_selected_items_vertically_by_desc_height (a_coordinate: EV_COORDINATE)
			-- Spread all currently selected items apart so that
			-- they do not overlap, arrange them vertically
			-- sorted by descending height and place lower left corner of
			-- bounding box at view coordinates `a_coordinate'
		require
			a_coordinate_not_void: a_coordinate /= Void
		do
			if view.selected_count > 0 then
				view.spread_selected_items_vertically_by_desc_height (a_coordinate)
				if view.is_move_valid then
					report_change ("Spread selected objects vertically by descending height" + selected_count_text)
				else
					report_spread_error_summary ("Vertical Spread By Descending Height")
				end
			else
				report_failure ("Vertical Spread By Descending Height not allowed - no objects selected")
			end
			report_end
		end

	spread_selected_items_horizontally_by_asc_x_position (a_coordinate: EV_COORDINATE)
			-- Spread all currently selected items apart
			-- so that they do not overlap, arrange them
			-- horizontally sorted by ascending center in X and
			-- place lower left corner of bounding box at
			-- view coordinates `a_coordinate'
		require
			a_coordinate_not_void: a_coordinate /= Void
		do
			if view.selected_count > 0 then
				view.spread_selected_items_horizontally_by_asc_x_position (a_coordinate)
				if view.is_move_valid then
					report_change ("Spread selected objects horizontally by ascending X position" + selected_count_text)
				else
					report_spread_error_summary ("Horizontal Spread By Ascending X Position")
				end
			else
				report_failure ("Horizontal Spread By Ascending X Position not allowed - no objects selected")
			end
			report_end
		end

	spread_selected_items_horizontally_by_desc_x_position (a_coordinate: EV_COORDINATE)
			-- Spread all currently selected items apart
			-- so that they do not overlap, arrange them
			-- horizontally sorted by descending center in X and
			-- place lower left corner of bounding box at
			-- view coordinates `a_coordinate'
		require
			a_coordinate_not_void: a_coordinate /= Void
		do
			if view.selected_count > 0 then
				view.spread_selected_items_horizontally_by_desc_x_position (a_coordinate)
				if view.is_move_valid then
					report_change ("Spread selected objects horizontally by descending X position" + selected_count_text)
				else
					report_spread_error_summary ("Horizontal Spread By Descending X Position")
				end
			else
				report_failure ("Horizontal Spread By Descending X Position not allowed - no objects selected")
			end
			report_end
		end

	spread_selected_items_horizontally_by_asc_y_position (a_coordinate: EV_COORDINATE)
			-- Spread all currently selected items apart
			-- so that they do not overlap, arrange them
			-- horizontally sorted by ascending center in Y and
			-- place lower left corner of bounding box at
			-- view coordinates `a_coordinate'
		require
			a_coordinate_not_void: a_coordinate /= Void
		do
			if view.selected_count > 0 then
				view.spread_selected_items_horizontally_by_asc_y_position (a_coordinate)
				if view.is_move_valid then
					report_change ("Spread selected objects horizontally by ascending Y position" + selected_count_text)
				else
					report_spread_error_summary ("Horizontal Spread By Ascending Y Position")
				end
			else
				report_failure ("Horizontal Spread By Ascending Y Position not allowed - no objects selected")
			end
			report_end
		end

	spread_selected_items_horizontally_by_desc_y_position (a_coordinate: EV_COORDINATE)
			-- Spread all currently selected items apart
			-- so that they do not overlap, arrange them
			-- horizontally sorted by descending center in Y and
			-- place lower left corner of bounding box at
			-- view coordinates `a_coordinate'
		require
			a_coordinate_not_void: a_coordinate /= Void
		do
			if view.selected_count > 0 then
				view.spread_selected_items_horizontally_by_desc_y_position (a_coordinate)
				if view.is_move_valid then
					report_change ("Spread selected objects horizontally by descending Y position" + selected_count_text)
				else
					report_spread_error_summary ("Horizontal Spread By Descending Y Position")
				end
			else
				report_failure ("Horizontal Spread By Descending Y Position not allowed - no objects selected")
			end
			report_end
		end

	spread_selected_items_vertically_by_asc_x_position (a_coordinate: EV_COORDINATE)
			-- Spread all currently selected items apart
			-- so that they do not overlap, arrange them
			-- vertically sorted by ascending center in X and place
			-- lower left corner of bounding box at view
			-- coordinates `a_coordinate'
		require
			a_coordinate_not_void: a_coordinate /= Void
		do
			if view.selected_count > 0 then
				view.spread_selected_items_vertically_by_asc_x_position (a_coordinate)
				if view.is_move_valid then
					report_change ("Spread selected objects vertically by ascending X position" + selected_count_text)
				else
					report_spread_error_summary ("Vertical Spread By Ascending X Position")
				end
			else
				report_failure ("Vertical Spread By Ascending X Position not allowed - no objects selected")
			end
			report_end
		end

	spread_selected_items_vertically_by_desc_x_position (a_coordinate: EV_COORDINATE)
			-- Spread all currently selected items apart
			-- so that they do not overlap, arrange them
			-- vertically sorted by descending center in X and place
			-- lower left corner of bounding box at view
			-- coordinates `a_coordinate'
		require
			a_coordinate_not_void: a_coordinate /= Void
		do
			if view.selected_count > 0 then
				view.spread_selected_items_vertically_by_desc_x_position (a_coordinate)
				if view.is_move_valid then
					report_change ("Spread selected objects vertically by descending X position" + selected_count_text)
				else
					report_spread_error_summary ("Vertical Spread By Descending X Position")
				end
			else
				report_failure ("Vertical Spread By Descending X Position not allowed - no objects selected")
			end
			report_end
		end

	spread_selected_items_vertically_by_asc_y_position (a_coordinate: EV_COORDINATE)
			-- Spread all currently selected items apart
			-- so that they do not overlap, arrange them
			-- vertically sorted by ascending center in Y and place
			-- lower left corner of bounding box at view
			-- coordinates `a_coordinate'
		require
			a_coordinate_not_void: a_coordinate /= Void
		do
			if view.selected_count > 0 then
				view.spread_selected_items_vertically_by_asc_y_position (a_coordinate)
				if view.is_move_valid then
					report_change ("Spread selected objects vertically by ascending Y position" + selected_count_text)
				else
					report_spread_error_summary ("Vertical Spread By Ascending Y Position")
				end
			else
				report_failure ("Vertical Spread By Ascending Y Position not allowed - no objects selected")
			end
			report_end
		end

	spread_selected_items_vertically_by_desc_y_position (a_coordinate: EV_COORDINATE)
			-- Spread all currently selected items apart
			-- so that they do not overlap, arrange them
			-- vertically sorted by descending center in Y and place
			-- lower left corner of bounding box at view
			-- coordinates `a_coordinate'
		require
			a_coordinate_not_void: a_coordinate /= Void
		do
			if view.selected_count > 0 then
				view.spread_selected_items_vertically_by_desc_y_position (a_coordinate)
				if view.is_move_valid then
					report_change ("Spread selected objects vertically by descending Y position" + selected_count_text)
				else
					report_spread_error_summary ("Vertical Spread By Descending Y Position")
				end
			else
				report_failure ("Vertical Spread By Descending Y Position not allowed - no objects selected")
			end
			report_end
		end

	spread_selected_items_horizontally_by_asc_name (a_coordinate: EV_COORDINATE)
			-- Spread all currently selected items apart
			-- so that they do not overlap, arrange them
			-- horizontally sorted by ascending packing name and
			-- place lower left corner of bounding box at
			-- view coordinates `a_coordinate'
		require
			a_coordinate_not_void: a_coordinate /= Void
		do
			if view.selected_count > 0 then
				view.spread_selected_items_horizontally_by_asc_name (a_coordinate)
				if view.is_move_valid then
					report_change ("Spread selected objects horizontally by ascending name" + selected_count_text)
				else
					report_spread_error_summary ("Horizontal Spread By Ascending Name")
				end
			else
				report_failure ("Horizontal Spread By Ascending Name not allowed - no objects selected")
			end
			report_end
		end

	spread_selected_items_horizontally_by_desc_name (a_coordinate: EV_COORDINATE)
			-- Spread all currently selected items apart
			-- so that they do not overlap, arrange them
			-- horizontally sorted by descending packing name and
			-- place lower left corner of bounding box at
			-- view coordinates `a_coordinate'
		require
			a_coordinate_not_void: a_coordinate /= Void
		do
			if view.selected_count > 0 then
				view.spread_selected_items_horizontally_by_desc_name (a_coordinate)
				if view.is_move_valid then
					report_change ("Spread selected objects horizontally by descending name" + selected_count_text)
				else
					report_spread_error_summary ("Horizontal Spread By Descending Name")
				end
			else
				report_failure ("Horizontal Spread By Descending Name not allowed - no objects selected")
			end
			report_end
		end

	spread_selected_items_vertically_by_asc_name (a_coordinate: EV_COORDINATE)
			-- Spread all currently selected items apart
			-- so that they do not overlap, arrange them
			-- vertically sorted by ascending packing name and place
			-- lower left corner of bounding box at view
			-- coordinates `a_coordinate'
		require
			a_coordinate_not_void: a_coordinate /= Void
		do
			if view.selected_count > 0 then
				view.spread_selected_items_vertically_by_asc_name (a_coordinate)
				if view.is_move_valid then
					report_change ("Spread selected objects vertically by ascending name" + selected_count_text)
				else
					report_spread_error_summary ("Vertical Spread By Ascending Name")
				end
			else
				report_failure ("Vertical Spread By Ascending Name not allowed - no objects selected")
			end
			report_end
		end

	spread_selected_items_vertically_by_desc_name (a_coordinate: EV_COORDINATE)
			-- Spread all currently selected items apart
			-- so that they do not overlap, arrange them
			-- vertically sorted by descending packing name and place
			-- lower left corner of bounding box at view
			-- coordinates `a_coordinate'
		require
			a_coordinate_not_void: a_coordinate /= Void
		do
			if view.selected_count > 0 then
				view.spread_selected_items_vertically_by_desc_name (a_coordinate)
				if view.is_move_valid then
					report_change ("Spread selected objects vertically by descending name" + selected_count_text)
				else
					report_spread_error_summary ("Vertical Spread By Descending Name")
				end
			else
				report_failure ("Vertical Spread By Descending Name not allowed - no objects selected")
			end
			report_end
		end

	abut_selected_item (a_coordinate: EV_COORDINATE)
			-- Abut the currently selected item (can only be one)
			-- to the object indicated by the mouse pointer
		require
			a_coordinate_not_void: a_coordinate /= Void
		local
			sel: INTEGER
		do
			sel := view.selected_count
			if sel = 1 then
				view.abut_selected_item (a_coordinate)
				if not view.had_error then
					report_change ("Abut completed")
				else
					report_abut_error_summary
				end
			elseif sel = 0 then
				report_failure ("Abut not allowed - no object selected")
			else
				report_failure ("Abut not allowed - more than one object selected")
			end
			report_end
		end

	align_left_selected_items (a_coordinate: EV_COORDINATE)
			-- Align the currently selected items to the left
			-- edge of the object containing the mouse pointer
		require
			a_coordinate_not_void: a_coordinate /= Void
		do
			view.align_left_selected_items (a_coordinate)
			report_align_results ("Align Left")
		end

	align_right_selected_items (a_coordinate: EV_COORDINATE)
			-- Align the currently selected items to the right
			-- edge of the object containing the mouse pointer
		require
			a_coordinate_not_void: a_coordinate /= Void
		do
			view.align_right_selected_items (a_coordinate)
			report_align_results ("Align Right")
		end

	align_top_selected_items (a_coordinate: EV_COORDINATE)
			-- Align the currently selected items to the top
			-- edge of the object containing the mouse pointer
		require
			a_coordinate_not_void: a_coordinate /= Void
		do
			view.align_top_selected_items (a_coordinate)
			report_align_results ("Align Top")
		end

	align_bottom_selected_items (a_coordinate: EV_COORDINATE)
			-- Align the currently selected items to the bottom
			-- edge of the object containing the mouse pointer
		require
			a_coordinate_not_void: a_coordinate /= Void
		do
			view.align_bottom_selected_items (a_coordinate)
			report_align_results ("Align Bottom")
		end

	align_center_x_selected_items (a_coordinate: EV_COORDINATE)
			-- Align the currently selected items to the center
			-- of the object containing the mouse pointer in X
		require
			a_coordinate_not_void: a_coordinate /= Void
		do
			view.align_center_x_selected_items (a_coordinate)
			report_align_results ("Align Center in X")
		end

	align_center_y_selected_items (a_coordinate: EV_COORDINATE)
			-- Align the currently selected items to the center
			-- of the object containing the mouse pointer in Y
		require
			a_coordinate_not_void: a_coordinate /= Void
		do
			view.align_center_y_selected_items (a_coordinate)
			report_align_results ("Align Center in Y")
		end

	display_info_on_selected_items
			-- Display information about all currently selected
			-- items
		do
			main_window.display_info_on_dies (view.selected_dies)
			report_status ("Showed Die Information window")
			report_end
		end

	save_die_info (a_row_list: LIST [PACKING_DIE_MULTI_COLUMN_LIST_ROW] a_column_list: LIST [MULTI_COLUMN_LIST_COLUMN [PACKING_DIE_MULTI_COLUMN_LIST_ROW]])
			-- Create nicely formatted text representing contents
			-- of `a_row_list', using just the columns in
			-- `a_column_list' in order.  Display this text
			-- in the Output window
		local
			l_formatter: MULTI_COLUMN_LIST_TEXT_FORMATTER [PACKING_DIE_MULTI_COLUMN_LIST_ROW]
			s: STRING
		do
			create l_formatter
			s := l_formatter.formatted_text (a_row_list, a_column_list, "  ", True, True)
			report_status ("Wrote text for rows selected in Die Information window to Output window")
			report_details (s)
			report_end
		end

	save_help_info (a_text: STRING)
			-- Display Help info `a_text' in Output window
		do
			report_status ("Wrote Help text to Output window")
			report_details (a_text)
			report_end
		end

	save_project_info (an_interface_type, what, a_text: STRING)
			-- Display info `what' from `an_interface_type'
			-- interface with `a_text' in Output window
		require
			what_not_void: what /= Void
			a_text_not_void: a_text /= Void
		do
			report_status ("Wrote " + an_interface_type + " " + what + " to Output window")
			report_details (a_text)
			report_end
		end

	set_payload_and_kerf_sizes
			-- Set size of payload and external kerf
			-- for a run whose packing target is a custom
			-- target
		do
			main_window.show_payload_and_kerf_sizes_window
			report_status ("Showed Payload and Kerf Sizes window")
			report_end
		end

	set_scribe_widths
			-- Set width of any new scribes created after
			-- this command executes successfully
		do
			main_window.show_scribe_properties_window
			report_status ("Showed Scribe Properties window")
			report_end
		end

	set_lot_split_colors
			-- Set colors to be used for wafer etest IDs in
			-- Lot Split view
		do
			main_window.show_lot_split_colors_window
			report_status ("Showed Lot Split Colors window")
			report_end
		end

	set_application_colors
			-- Set colors to be used for application, other than
			-- lot split colors
		do
			main_window.show_application_colors_window
			report_status ("Showed Application Colors window")
			report_end
		end

	compute_cut_dies
			-- Compute cut dies for all projects
		do
			view.compute_cut_dies
			view.redraw_if_max_size_view
			report_status ("Computed cut dies for all projects")
			report_end
		end

	assign_lot
			-- Assign lot to selected items
		local
			list: LIST [PACKING_DIES_MROW]
			n: INTEGER
		do
			list := view.selected_customer_projects_with_multiple_etest_ids
			if not list.is_empty then
				report_status ("Showed Lot Assignment window")
				main_window.show_lot_assignment_window (list)
				if is_last_lot_assignment_valid then
					view.assign_lot_to_selected_projects (wafer_etest_row)
					n := list.count
					if wafer_etest_row /= Void then
						report_change ("Assigned lot " + wafer_etest_row.wafer_etest_id + " to " + n.out + " " + su.number (n, "project", "projects"))
					else
						report_change ("Removed lot assignment for " + n.out + " " + su.number (n, "project", "projects"))
					end
				else
					report_cancellation ("Lot assignment cancelled")
				end
			elseif view.selected_count > 0 then
				report_failure ("Lot assignment failed - none of the selected objects are customer projects with more than one compatible lot")
			else
				report_failure ("Lot assignment failed - no objects selected")
			end
			report_end
		end

	set_lot_etest_row (a_wafer_etest_row: WAFER_ETEST_NAMES_ROW)
			-- Set wafer etest name chosen by user for projects
		do
			wafer_etest_row := a_wafer_etest_row
			is_last_lot_assignment_valid := True
		end

	cancel_lot_assignment
			-- Cancel deletion when user clicks Cancel
			-- in delete projects dialog
		do
			is_last_lot_assignment_valid := False
		end

	display_lot_summary
			-- Display summary of project/lot compatibility
		do
			main_window.show_lot_summary_window
			report_status ("Showed Lot Summary window")
			report_end
		end

	display_packer_output_window
			-- Display packer output window
		do
			main_window.show_packer_output_window
			report_status ("Showed Output window")
			report_end
		end

	display_project_management_window
			-- Display window to manage projects (add, remove, etc)
		do
			main_window.show_project_management_window
			report_status ("Showed Project Management window")
			report_end
		end

	display_version_selector
			-- Display version selection window
		do
			main_window.show_version_selector
			if main_window.is_version_selector_shown then
				report_status ("Showed Version Selection window")
			else
				report_failure ("Cannot select a different version when editing cut patterns for a production")
			end
			report_end
		end

	display_help
			-- Display window which shows key and pointer button
			-- bindings
		do
			main_window.show_help_window
			report_status ("Showed Help window")
			report_end
		end

	toggle_pointer_coordinates_display
			-- Toggle display of pointer coordinates
		local
			p: EV_COORDINATE
		do
			main_window.toggle_pointer_coordinates_display
			p := view.pointer_position
			handle_pointer_motion_at_view_coords (p.x, p.y)
			report_status ("Toggled display of mouse pointer coordinates")
			report_end
		end

	search_packing
			-- Search packing for particular objects
			-- and highlight them
		do
			main_window.show_search_window
			report_status ("Showed Search window")
			report_end
		end

	search_for_design_numbers (a_list: LIST [INTEGER] a_select_also: BOOLEAN)
			-- Search element list for elemnts that have design
			-- numbers in `a_list'.  Mark them as "found"
			-- so that they will be displayed differently in
			-- Search Results view.  If `a_select_also' then
			-- select these projects too
		local
			n: INTEGER
		do
			view.search_for_design_numbers (a_list, a_select_also)
			n := view.found_elements.count
			report_status ("Found " + n.out + " " + su.number (n, "object", "objects"))
			report_end
		end

	show_entire_model_in_view
			-- Reset the area of the model the view covers to
			-- include the entire model, then redraw the view
		do
			view.show_entire_model
			report_navigation ("Showed entire packing")
		end

	refresh_drawing_area
			-- Refresh the drawing area from the pixmap
		do
			view.refresh_drawing_area
			report_view ("Refreshed drawing area")
		end

	redraw_drawing_area
			-- Redraw the model onto the pixmap and then
			-- draw the pixmap onto the drawing area
		do
			view.redraw
			report_view ("Redrew drawing area")
		end

	show_field_by_name (a_field_name: STRING)
			-- Reset the area of the model the view covers to
			-- include just field named `a_field_name' and its
			-- associated label, then redraw the view
		require
			a_field_name_not_void: a_field_name /= Void
		do
			view.show_field_by_name (a_field_name)
			report_navigation ("Showed field " + a_field_name)
		end

	show_saved_area (an_area_number: INTEGER)
			-- Reset the area of the model the view covers to
			-- the previously saved area `an_area_number'
			-- then redraw the view
		require
			an_area_number_positive: an_area_number > 0
			an_area_number_small_enough: an_area_number <= saved_area_capacity
		do
			view.show_saved_area (an_area_number)
			if not view.had_error then
				report_navigation ("Showed saved area " + an_area_number.out)
			else
				report_failure (view.error_messages.first)
			end
			display_messages
		end

	set_saved_area (an_area_number: INTEGER)
			-- Set saved area `an_area_number' to the current
			-- displayed area of model
		require
			an_area_number_positive: an_area_number > 0
			an_area_number_small_enough: an_area_number <= saved_area_capacity
		do
			view.set_saved_area (an_area_number)
			report_navigation ("Set saved area number " + an_area_number.out + " to current displayed area")
		end

	show_previous_area
			-- Reset the area of the model the view covers to
			-- the previously displayed area relative to
			-- the currently displayed area
		do
			view.show_previous_area
			if not view.had_error then
				report_navigation ("Showed previous area")
			else
				report_failure (view.error_messages.first)
			end
			display_messages
		end

	show_next_area
			-- Reset the area of the model the view covers to
			-- the next displayed area relative to
			-- the currently displayed area
		do
			view.show_next_area
			if not view.had_error then
				report_navigation ("Showed next area")
			else
				report_failure (view.error_messages.first)
			end
			display_messages
		end

	label_all_items_by_design_number
			-- Label all items with design number
		do
			view.label_all_items_by_design_number
			report_view ("Labeled all objects by design number" + selectable_count_text)
		end

	label_selected_items_by_design_number
			-- Label selected items with design number
		do
			view.label_selected_items_by_design_number
			report_view ("Labeled selected objects by design number" + selected_count_text)
		end

	label_unselected_items_by_design_number
			-- Label unselected items with design number
		do
			view.label_unselected_items_by_design_number
			report_view ("Labeled unselected objects by design number" + unselected_count_text)
		end

	label_all_items_by_design_name
			-- Label all items with design name
		do
			view.label_all_items_by_design_name
			report_view ("Labeled all objects by design name" + selectable_count_text)
		end

	label_selected_items_by_design_name
			-- Label selected items with design name
		do
			view.label_selected_items_by_design_name
			report_view ("Labeled selected objects by design name" + selected_count_text)
		end

	label_unselected_items_by_design_name
			-- Label unselected items with design name
		do
			view.label_unselected_items_by_design_name
			report_view ("Labeled unselected objects by design name" + unselected_count_text)
		end

	label_all_items_by_account_type
			-- Label all items with account type
		do
			view.label_all_items_by_account_type
			report_view ("Labeled all objects by account type" + selectable_count_text)
		end

	label_selected_items_by_account_type
			-- Label selected items with account type
		do
			view.label_selected_items_by_account_type
			report_view ("Labeled selected objects by account type" + selected_count_text)
		end

	label_unselected_items_by_account_type
			-- Label unselected items with account type
		do
			view.label_unselected_items_by_account_type
			report_view ("Labeled unselected objects by account type" + unselected_count_text)
		end

	label_all_items_by_account_number
			-- Label all items with account number
		do
			view.label_all_items_by_account_number
			report_view ("Labeled all objects by account number" + selectable_count_text)
		end

	label_selected_items_by_account_number
			-- Label selected items with account number
		do
			view.label_selected_items_by_account_number
			report_view ("Labeled selected objects by account number" + selected_count_text)
		end

	label_unselected_items_by_account_number
			-- Label unselected items with account number
		do
			view.label_unselected_items_by_account_number
			report_view ("Labeled unselected objects by account number" + unselected_count_text)
		end

	label_all_items_by_wafer_etest_id
			-- Label all items with lot split
		do
			view.label_all_items_by_wafer_etest_id
			report_view ("Labeled all objects by lot split" + selectable_count_text)
		end

	label_selected_items_by_wafer_etest_id
			-- Label selected items with lot split
		do
			view.label_selected_items_by_wafer_etest_id
			report_view ("Labeled selected objects by lot split" + selected_count_text)
		end

	label_unselected_items_by_wafer_etest_id
			-- Label unselected items with lot split
		do
			view.label_unselected_items_by_wafer_etest_id
			report_view ("Labeled unselected objects by lot split" + unselected_count_text)
		end

	label_all_items_by_org_short_id
			-- Label all items with organization short ID
		do
			view.label_all_items_by_org_short_id
			report_view ("Labeled all objects by organization short ID" + selectable_count_text)
		end

	label_selected_items_by_org_short_id
			-- Label selected items with organization short ID
		do
			view.label_selected_items_by_org_short_id
			report_view ("Labeled selected objects by organization short ID" + selected_count_text)
		end

	label_unselected_items_by_org_short_id
			-- Label unselected items with organization short ID
		do
			view.label_unselected_items_by_org_short_id
			report_view ("Labeled unselected objects by organization short ID" + unselected_count_text)
		end

	color_all_items_by_fixed_status
			-- Color all items by fixed in X and Y status
		do
			view.color_all_items_by_fixed_status
			report_view ("Colored all objects by fixed in X and Y status" + selectable_count_text)
		end

	color_selected_items_by_fixed_status
			-- Color selected items by fixed in X and Y status
		do
			view.color_selected_items_by_fixed_status
			report_view ("Colored selected objects by fixed in X and Y status" + selected_count_text)
		end

	color_unselected_items_by_fixed_status
			-- Color unselected items by fixed in X and Y status
		do
			view.color_unselected_items_by_fixed_status
			report_view ("Colored unselected objects by fixed in X and Y status" + unselected_count_text)
		end

	color_all_items_by_locked_status
			-- Color all items by locked in position in X and Y
			-- status
		do
			view.color_all_items_by_locked_status
			report_view ("Colored all objects by locked in X and Y status" + selectable_count_text)
		end

	color_selected_items_by_locked_status
			-- Color selected items by locked in position
			-- in X and Y status
		do
			view.color_selected_items_by_locked_status
			report_view ("Colored selected objects by locked in X and Y status" + selected_count_text)
		end

	color_unselected_items_by_locked_status
			-- Color unselected items by locked in position
			-- in X and Y status
		do
			view.color_unselected_items_by_locked_status
			report_view ("Colored unselected objects by locked in X and Y status" + unselected_count_text)
		end

	color_all_items_by_account_type
			-- Color all items by account type
		do
			view.color_all_items_by_account_type
			report_view ("Colored all objects by account type" + selectable_count_text)
		end

	color_selected_items_by_account_type
			-- Color selected items by account type
		do
			view.color_selected_items_by_account_type
			report_view ("Colored selected objects by account type" + selected_count_text)
		end

	color_unselected_items_by_account_type
			-- Color unselected items by account type
		do
			view.color_unselected_items_by_account_type
			report_view ("Colored unselected objects by account type" + unselected_count_text)
		end

	fill_no_items
			-- Do not fill interior of any items
		do
			view.fill_no_items
			report_view ("Fill disabled for all objects" + selectable_count_text)
		end

	fill_selected_items
			-- Fill interior of selected items only
		do
			view.fill_selected_items
			report_view ("Filled all objects by selection status" + selectable_count_text)
		end

	fill_unselected_items
			-- Fill interior of unselected items only
		do
			view.fill_unselected_items
			report_view ("Filled unselected objects by selection status" + unselected_count_text)
		end

	fill_selected_and_unselected_items
			-- Fill interior of both selected and unselected items
		do
			view.fill_selected_and_unselected_items
			report_view ("Filled all objects by selection status" + selectable_count_text)
		end

	fill_by_fixed_status
			-- Fill interior of items by their fixed in X and Y
			-- status
		do
			view.fill_by_fixed_status
			report_view ("Filled all objects by fixed status" + selectable_count_text)
		end

	fill_by_locked_status
			-- Fill interior of items by their locked in X and Y
			-- status
		do
			view.fill_by_locked_status
			report_view ("Filled all objects by locked status" + selectable_count_text)
		end

	set_transparent_all_items
			-- Set all items to be transparent
		do
			view.set_transparent_all_items
			report_view ("Made all objects transparent" + selectable_count_text)
		end

	set_opaque_all_items
			-- Set all items to be opaque
		do
			view.set_opaque_all_items
			report_view ("Made all objects opaque" + selectable_count_text)
		end

	set_transparent_selected_items
			-- Set selected items to be transparent
		do
			view.set_transparent_selected_items
			report_view ("Made selected objects transparent" + selected_count_text)
		end

	set_opaque_selected_items
			-- Set selected items to be opaque
		do
			view.set_opaque_selected_items
			report_view ("Made selected objects opaque" + selected_count_text)
		end

	set_transparent_unselected_items
			-- Set unselected items to be transparent
		do
			view.set_transparent_unselected_items
			report_view ("Made unselected objects transparent" + unselected_count_text)
		end

	set_opaque_unselected_items
			-- Set unselected items to be opaque
		do
			view.set_opaque_unselected_items
			report_view ("Made unselected objects opaque" + unselected_count_text)
		end

	set_standard_view
			-- Switch to Standard view where items are labeled
			-- by design number
		do
			view.set_standard_view
			report_view ("Switched to Standard view")
		end

	set_account_type_view
			-- Switch to Account Type view where items are labeled
			-- by account type
		do
			view.set_account_type_view
			report_view ("Switched to Account Type view")
		end

	set_lot_split_view
			-- Switch to Lot Split view where items are labeled
			-- by wafer etest ID(s)
		do
			view.set_lot_split_view
			report_view ("Switched to Lot Split view")
		end

	set_packaging_view
			-- Switch to Packaging view where items are labeled
			-- by package name and packaging vendor type
		do
			view.set_packaging_view
			report_view ("Switched to Packaging view")
		end

	set_validity_check_view
			-- Switch to Validity Check view where items
			-- are colored by which errors/warnings they
			-- have
		do
			view.set_validity_check_view
			report_view ("Switched to Validity Check view")
		end

	set_cut_view
			-- Switch to Cut view where items are colored by
			-- lot split and labeled by design number
		do
			view.set_cut_view
			report_view ("Switched to Cut view")
		end

	set_search_results_view
			-- Switch to Search Results view where items found
			-- by last search are displayed differently
		do
			view.set_search_results_view
			report_view ("Switched to Search Results view")
		end

	set_highlighted_selection_view
			-- Switch to Highlighted Selection view where items
			-- that are selected are displayed differently
		do
			view.set_highlighted_selection_view
			report_view ("Switched to Highlighted Selection view")
		end

	set_design_name_view
			-- Switch to Design Name view where items are labeled
			-- by design name
		do
			view.set_design_name_view
			report_view ("Switched to Design Name view")
		end

	set_die_id_view
			-- Switch to Die ID view where items are labeled
			-- by die ID
		do
			view.set_die_id_view
			report_view ("Switched to Die ID view")
		end

	set_account_number_view
			-- Switch to Account Number view where items are
			-- labeled by account number
		do
			view.set_account_number_view
			report_view ("Switched to Account Number view")
		end

	set_org_short_id_view
			-- Switch to Organization Short ID view where items
			-- are  labeled by organization short ID
		do
			view.set_org_short_id_view
			report_view ("Switched to Organization Short ID view")
		end

	set_max_die_size_view
			-- Switch to Max Die Size view where items
			-- are labeled by maximum die size
		do
			view.set_max_die_size_view
			report_view ("Switched to Max Die Size view")
		end

	set_overhead_style_view
			-- Switch to Overhead Style view where items are
			-- labeled by overhead style
		do
			view.set_overhead_style_view
			report_view ("Switched to Die Labelling Style view")
		end

	set_rotation_locked_view
			-- Switch to Rotation Locked view where items
			-- that are rotation-locked are displayed differently
		do
			view.set_rotation_locked_view
			report_view ("Switched to Rotation Locked view")
		end

	set_duplication_locked_view
			-- Switch to Duplication Locked view where
			-- items that are duplication-locked are
			-- displayed differently
		do
			view.set_duplication_locked_view
			report_view ("Switched to Duplication Locked view")
		end

	set_deletion_locked_view
			-- Switch to Deletion Locked view where
			-- items that are deletion-locked are
			-- displayed differently
		do
			view.set_deletion_locked_view
			report_view ("Switched to Deletion Locked view")
		end

	start_set_selection_at_view_coords (a_x, a_y: INTEGER)
			-- Start setting a new selection at view coordinates
			-- (`a_x', `a_y')
		do
			set_operation_start_location (a_x, a_y)
			redraw_selection_style_rubber_box
			report_status ("Set selection started")
		end

	continue_set_selection_at_view_coords (a_x, a_y: INTEGER)
			-- Continue setting a new selection at view coordinates
			-- (`a_x', `a_y')
		do
			set_operation_current_location (a_x, a_y)
			redraw_selection_style_rubber_box
			report_progress ("Set selection in progress")
		end

	end_set_selection_at_view_coords (a_x, a_y: INTEGER)
			-- End setting new selection at view coordinates
			-- (`a_x', `a_y')
		local
			min_sel: INTEGER
			r: EV_RECTANGLE
		do
			set_operation_end_location (a_x, a_y)
			erase_rubber_box
			compute_current_rectangle
			r := operation_current_rectangle
			min_sel := minimum_selection_dimension
			if r.width >= min_sel or r.height >= min_sel then
				view.set_selection_by_area (r)
			else
				view.set_selection_by_point (operation_start_location)
			end
			report_select ("Set selection completed" + selected_text)
		end

	cancel_set_selection_at_view_coords (a_x, a_y: INTEGER)
			-- Cancel setting new selection at view coordinates
			-- (`a_x', `a_y')
		do
			set_operation_end_location (a_x, a_y)
			erase_rubber_box
			report_cancellation ("Set selection cancelled")
		end

	start_set_selection_by_intersection (a_x, a_y: INTEGER)
			-- Start setting a new selection by
			-- intersection at view coordinates (`a_x', `a_y')
		do
			set_operation_start_location (a_x, a_y)
			redraw_selection_style_rubber_box
			report_status ("Set selection by intersection started")
		end

	continue_set_selection_by_intersection (a_x, a_y: INTEGER)
			-- Continue setting a new selection by
			-- intersection at view coordinates (`a_x', `a_y')
		do
			set_operation_current_location (a_x, a_y)
			redraw_selection_style_rubber_box
			report_progress ("Set selection by intersection in progress")
		end

	end_set_selection_by_intersection (a_x, a_y: INTEGER)
			-- End setting new selection by intersection at
			-- view coordinates (`a_x', `a_y')
		local
			r: EV_RECTANGLE
		do
			set_operation_end_location (a_x, a_y)
			erase_rubber_box
			compute_current_rectangle
			r := operation_current_rectangle
			view.set_selection_by_intersection (r)
			report_select ("Set selection by intersection completed" + selected_text)
		end

	cancel_set_selection_by_intersection (a_x, a_y: INTEGER)
			-- Cancel setting new selection by intersection
			-- at view coordinates (`a_x', `a_y')
		do
			set_operation_end_location (a_x, a_y)
			erase_rubber_box
			report_cancellation ("Set selection by intersection cancelled")
		end

	start_add_selection_at_view_coords (a_x, a_y: INTEGER)
			-- Start an "add to selection" operation at
			-- view coordinates (`a_x', `a_y')
		do
			set_operation_start_location (a_x, a_y)
			redraw_selection_style_rubber_box
			report_status ("Add to selection started")
		end

	continue_add_selection_at_view_coords (a_x, a_y: INTEGER)
			-- Continue an "add to selection" operation at
			-- view coordinates (`a_x', `a_y')
		do
			set_operation_current_location (a_x, a_y)
			redraw_selection_style_rubber_box
			report_progress ("Add to selection in progress")
		end

	end_add_selection_at_view_coords (a_x, a_y: INTEGER)
			-- End an "add to selection" operation at
			-- view coordinates (`a_x', `a_y')
		local
			min_sel: INTEGER
			r: EV_RECTANGLE
		do
			save_selected_count
			set_operation_end_location (a_x, a_y)
			erase_rubber_box
			compute_current_rectangle
			r := operation_current_rectangle
			min_sel := minimum_selection_dimension
			if r.width >= min_sel or r.height >= min_sel then
				view.add_to_selection_by_area (r)
			else
				view.add_to_selection_by_point (operation_start_location)
			end
			report_select ("Add to selection completed" + added_selected_text)
		end

	cancel_add_selection_at_view_coords (a_x, a_y: INTEGER)
			-- Cancel an "add to selection" operation at
			-- view coordinates (`a_x', `a_y')
		do
			set_operation_end_location (a_x, a_y)
			erase_rubber_box
			report_cancellation ("Add to selection cancelled")
		end

	start_add_selection_by_intersection (a_x, a_y: INTEGER)
			-- Start an "add to selection" by intersection
			-- operation at view coordinates (`a_x', `a_y')
		do
			set_operation_start_location (a_x, a_y)
			redraw_selection_style_rubber_box
			report_status ("Add to selection by intersection started")
		end

	continue_add_selection_by_intersection (a_x, a_y: INTEGER)
			-- Continue an "add to selection" by intersection
			-- operation at view coordinates (`a_x', `a_y')
		do
			set_operation_current_location (a_x, a_y)
			redraw_selection_style_rubber_box
			report_progress ("Add to selection by intersection in progress")
		end

	end_add_selection_by_intersection (a_x, a_y: INTEGER)
			-- End an "add to selection" by intersection
			-- operation at view coordinates (`a_x', `a_y')
		local
			r: EV_RECTANGLE
		do
			save_selected_count
			set_operation_end_location (a_x, a_y)
			erase_rubber_box
			compute_current_rectangle
			r := operation_current_rectangle
			view.add_to_selection_by_intersection (r)
			report_select ("Add to selection by intersection completed" + added_selected_text)
		end

	cancel_add_selection_by_intersection (a_x, a_y: INTEGER)
			-- Cancel an "add to selection" by intersection
			-- operation at view coordinates (`a_x', `a_y')
		do
			set_operation_end_location (a_x, a_y)
			erase_rubber_box
			report_cancellation ("Add to selection by intersection cancelled")
		end

	start_remove_selection_at_view_coords (a_x, a_y: INTEGER)
			-- Start a "remove from selection" operation at
			-- view coordinates (`a_x', `a_y')
		do
			set_operation_start_location (a_x, a_y)
			redraw_selection_style_rubber_box
			report_status ("Remove from selection started")
		end

	continue_remove_selection_at_view_coords (a_x, a_y: INTEGER)
			-- Start a "remove from selection" operation at
			-- view coordinates (`a_x', `a_y')
		do
			set_operation_current_location (a_x, a_y)
			redraw_selection_style_rubber_box
			report_progress ("Remove from selection in progress")
		end

	end_remove_selection_at_view_coords (a_x, a_y: INTEGER)
			-- End a "remove from selection" operation at
			-- view coordinates (`a_x', `a_y')
		local
			min_sel: INTEGER
			r: EV_RECTANGLE
		do
			save_selected_count
			set_operation_end_location (a_x, a_y)
			erase_rubber_box
			compute_current_rectangle
			r := operation_current_rectangle
			min_sel := minimum_selection_dimension
			if r.width >= min_sel or r.height >= min_sel then
				view.remove_from_selection_by_area (r)
			else
				view.remove_from_selection_by_point (operation_start_location)
			end
			report_select ("Remove from selection completed" + removed_selected_text)
		end

	cancel_remove_selection_at_view_coords (a_x, a_y: INTEGER)
			-- Cancel a "remove from selection" operation at
			-- view coordinates (`a_x', `a_y')
		do
			set_operation_end_location (a_x, a_y)
			erase_rubber_box
			report_cancellation ("Remove from selection cancelled")
		end

	start_remove_selection_by_intersection (a_x, a_y: INTEGER)
			-- Start a "remove from selection" by intersection
			-- operation at view coordinates (`a_x', `a_y')
		do
			set_operation_start_location (a_x, a_y)
			redraw_selection_style_rubber_box
			report_status ("Remove from selection by intersection started")
		end

	continue_remove_selection_by_intersection (a_x, a_y: INTEGER)
			-- Start a "remove from selection" by intersection
			-- operation at view coordinates (`a_x', `a_y')
		do
			set_operation_current_location (a_x, a_y)
			redraw_selection_style_rubber_box
			report_progress ("Remove from selection by intersection in progress")
		end

	end_remove_selection_by_intersection (a_x, a_y: INTEGER)
			-- End a "remove from selection" by intersection
			-- operation at view coordinates (`a_x', `a_y')
		local
			r: EV_RECTANGLE
		do
			save_selected_count
			set_operation_end_location (a_x, a_y)
			erase_rubber_box
			compute_current_rectangle
			r := operation_current_rectangle
			view.remove_from_selection_by_intersection (r)
			report_select ("Remove from selection by intersection completed" + removed_selected_text)
		end

	cancel_remove_selection_by_intersection (a_x, a_y: INTEGER)
			-- Cancel a "remove from selection" by intersection
			-- operation at view coordinates (`a_x', `a_y')
		do
			set_operation_end_location (a_x, a_y)
			erase_rubber_box
			report_cancellation ("Remove from selection by intersection cancelled")
		end

	start_move_selected_at_view_coords (a_x, a_y: INTEGER)
			-- Start an operation to move all selected items
			-- at view coordinates (`a_x', `a_y')
		do
			if view.selected_count > 0 then
				set_operation_start_location (a_x, a_y)
				view.start_move_selected_at_view_coords (a_x, a_y)
				report_status ("Move started" + selected_count_text)
			else
				report_no_move_items
			end
		end

	continue_move_selected_at_view_coords (a_x, a_y: INTEGER)
			-- Continue an operation to move all selected items
			-- at view coordinates (`a_x', `a_y')
		do
			if view.selected_count > 0 then
				set_operation_current_location (a_x, a_y)
				view.continue_move_selected_at_view_coords (a_x, a_y)
				report_progress ("Move in progress" + selected_count_text)
			else
				report_progress ("Move not allowed - no objects selected")
			end
		end

	end_move_selected_at_view_coords (a_x, a_y: INTEGER)
			-- End an operation to move all selected items
			-- at view coordinates (`a_x', `a_y')
		local
			n: INTEGER
		do
			if view.selected_count > 0 then
				set_operation_end_location (a_x, a_y)
				view.end_move_selected_at_view_coords (a_x, a_y)
				n := view.selected_count
				report_change ("Moved " + n.out + " " + su.number (n, "object", "objects") + " by " + last_move_distance_text)
			else
				report_progress ("Move failed - nothing is selected")
			end
			report_end
		end

	cancel_move_selected_at_view_coords (a_x, a_y: INTEGER)
			-- Cancel an operation to move all selected items
			-- at view coordinates (`a_x', `a_y')
		do
			set_operation_end_location (a_x, a_y)
			erase_rubber_box
			report_cancellation ("Move cancelled")
		end

	start_duplicate_selected_at_view_coords (a_x, a_y: INTEGER)
			-- Start an operation to duplicate all selected
			-- items at view coordinates (`a_x', `a_y')
		do
			if view.selected_count > 0 then
				set_operation_start_location (a_x, a_y)
				view.start_duplicate_selected_at_view_coords (a_x, a_y)
				if view.is_duplicate_valid then
					report_status ("Duplicate started" + selected_count_text)
				else
					report_failure (duplicate_error_summary)
					display_messages
				end
			else
				report_no_duplicate_items
			end
		end

	continue_duplicate_selected_at_view_coords (a_x, a_y: INTEGER)
			-- Continue an operation to duplicate all selected
			-- items at view coordinates (`a_x', `a_y')
		do
			if view.selected_count > 0 then
				set_operation_current_location (a_x, a_y)
				view.continue_duplicate_selected_at_view_coords (a_x, a_y)
				if view.is_duplicate_valid then
					report_progress ("Duplicate in progress" + selected_count_text)
				else
					report_progress (duplicate_error_summary)
				end
			else
				report_progress ("Duplicate not allowed - no objects selected")
			end
		end

	end_duplicate_selected_at_view_coords (a_x, a_y: INTEGER)
			-- End an operation to duplicate all selected
			-- items at view coordinates (`a_x', `a_y')
		local
			n: INTEGER
		do
			if view.selected_count > 0 then
				set_operation_end_location (a_x, a_y)
				view.end_duplicate_selected_at_view_coords (a_x, a_y)
				if view.is_duplicate_valid then
					if view.is_duplicate_far_enough then
						n := view.selected_count
						report_change ("Duplicated " + n.out + " " + su.number (n, "object", "objects") + " and moved by " + last_move_distance_text)
					else
						report_failure ("Duplicate cancelled - must move at least " + view.minimum_dup_distance_in_pixels.out + " pixels from original position")
					end
				else
					report_progress (duplicate_error_summary)
				end
			else
				report_progress ("Duplicate not allowed - no objects selected")
			end
			report_end
		end

	cancel_duplicate_selected_at_view_coords (a_x, a_y: INTEGER)
			-- Cancel an operation to duplicate all selected
			-- items at view coordinates (`a_x', `a_y')
		do
			set_operation_end_location (a_x, a_y)
			erase_rubber_box
			report_cancellation ("Duplicate cancelled")
		end

	start_scribe_creation_at_view_coords (a_x, a_y: INTEGER)
			-- Start creation of a scribe at view
			-- coordinates (`a_x', `a_y')
		do
			set_operation_start_location (a_x, a_y)
			redraw_scribe_style_rubber_box
			report_status ("Scribe creation started")
		end

	continue_scribe_creation_at_view_coords (a_x, a_y: INTEGER)
			-- Continue creation of a scribe at view
			-- coordinates (`a_x', `a_y')
		do
			set_operation_current_location (a_x, a_y)
			redraw_scribe_style_rubber_box
			report_progress ("Scribe creation in progress")
		end

	end_scribe_creation_at_view_coords (a_x, a_y: INTEGER)
			-- End creation of a scribe at view
			-- coordinates (`a_x', `a_y')
		local
			r: like operation_current_rectangle
			is_horiz: BOOLEAN
			l_type: STRING
		do
			set_operation_end_location (a_x, a_y)
			erase_rubber_box
			compute_current_rectangle
			r := operation_current_rectangle
			if r.width >= minimum_scribe_dimension or r.height >= minimum_scribe_dimension then
				view.create_scribe (r)
				if view.had_error then
					report_failure ("Scribe creation cancelled - scribe size or center too big")
				else
					is_horiz := view.is_last_scribe_horizontal
					if is_horiz then
						l_type := "horizontal"
					else
						l_type := "vertical"
					end
					report_change ("Created " + l_type + " scribe" + last_scribe_size_text (is_horiz))
				end
				display_messages
			else
				report_failure ("Scribe creation cancelled - box width or height must be at least " + minimum_scribe_dimension.out + " pixels")
			end
			report_end
		end

	cancel_scribe_creation_at_view_coords (a_x, a_y: INTEGER)
			-- Cancel creation of a scribe at view
			-- coordinates (`a_x', `a_y')
		do
			set_operation_end_location (a_x, a_y)
			erase_rubber_box
			report_cancellation ("Scribe creation cancelled")
		end

	start_zoom_in_at_view_coords (a_x, a_y: INTEGER)
			-- Start "zoom in" operation at view coordinates
			-- (`a_x', `a_y')
		do
			set_operation_start_location (a_x, a_y)
			redraw_zoom_style_rubber_box
			report_status ("Zoom in started")
		end

	continue_zoom_in_at_view_coords (a_x, a_y: INTEGER)
			-- Continue "zoom in" operation at view coordinates
			-- (`a_x', `a_y')
		do
			set_operation_current_location (a_x, a_y)
			redraw_zoom_style_rubber_box
			report_progress ("Zoom in in progress")
		end

	end_zoom_in_at_view_coords (a_x, a_y: INTEGER)
			-- End "zoom in" operation at view coordinates
			-- (`a_x', `a_y')
		local
			r: EV_RECTANGLE
		do
			set_operation_end_location (a_x, a_y)
			erase_rubber_box
			compute_current_rectangle
			r := operation_current_rectangle
			if r.width >= minimum_zoom_dimension and r.height >= minimum_zoom_dimension then
				view.zoom_in (r)
				report_navigation ("Zoom in completed")
			else
				report_failure ("Zoom in cancelled - box width or height must be at least " + minimum_zoom_dimension.out + " pixels")
			end
			report_end
		end

	cancel_zoom_in_at_view_coords (a_x, a_y: INTEGER)
			-- Cancel "zoom in" operation at view coordinates
			-- (`a_x', `a_y')
		do
			set_operation_end_location (a_x, a_y)
			erase_rubber_box
			report_cancellation ("Zoom in cancelled")
		end

	start_zoom_out_at_view_coords (a_x, a_y: INTEGER)
			-- Start "zoom out" operation at view coordinates
			-- (`a_x', `a_y')
		do
			set_operation_start_location (a_x, a_y)
			redraw_zoom_style_rubber_box
			report_status ("Zoom out started")
		end

	continue_zoom_out_at_view_coords (a_x, a_y: INTEGER)
			-- Continue "zoom out" operation at view coordinates
			-- (`a_x', `a_y')
		do
			set_operation_current_location (a_x, a_y)
			redraw_zoom_style_rubber_box
			report_progress ("Zoom out in progress")
		end

	end_zoom_out_at_view_coords (a_x, a_y: INTEGER)
			-- End "zoom out" operation at view coordinates
			-- (`a_x', `a_y')
		local
			r: EV_RECTANGLE
		do
			set_operation_end_location (a_x, a_y)
			erase_rubber_box
			compute_current_rectangle
			r := operation_current_rectangle
			if r.width >= minimum_zoom_dimension and r.height >= minimum_zoom_dimension then
				view.zoom_out (r)
				report_navigation ("Zoom out completed")
			else
				report_failure ("Zoom out cancelled - box width or height less than " + minimum_zoom_dimension.out + " pixels")
			end
			report_end
		end

	cancel_zoom_out_at_view_coords (a_x, a_y: INTEGER)
			-- Cancel "zoom out" operation at view coordinates
			-- (`a_x', `a_y')
		do
			set_operation_end_location (a_x, a_y)
			erase_rubber_box
			report_cancellation ("Zoom out cancelled")
		end

	start_pan_at_view_coords (a_x, a_y: INTEGER)
			-- Start pan operation at view coordinates
			-- (`a_x', `a_y')
		do
			set_operation_start_location (a_x, a_y)
			view.start_pan
			report_status ("Pan started")
		end

	continue_pan_at_view_coords (a_x, a_y: INTEGER)
			-- Continue pan operation at view coordinates
			-- (`a_x', `a_y')
		do
			set_operation_current_location (a_x, a_y)
			report_progress ("Pan in progress")
		end

	end_pan_at_view_coords (a_x, a_y: INTEGER)
			-- End pan operation at view coordinates
			-- (`a_x', `a_y')
		do
			set_operation_end_location (a_x, a_y)
			view.end_pan
			view.pan_by (operation_end_location.x - operation_start_location.x, operation_end_location.y - operation_start_location.y)
			report_navigation ("Pan completed")
		end

	cancel_pan_at_view_coords (a_x, a_y: INTEGER)
			-- Cancel pan operation at view coordinates
			-- (`a_x', `a_y')
		do
			set_operation_end_location (a_x, a_y)
			view.cancel_pan
			report_cancellation ("Pan cancelled")
		end

	handle_pointer_motion_at_view_coords (a_x, a_y: INTEGER)
			-- Handle pointer motion at view coordinates
			-- (`a_x', `a_y')
		do
			if main_window.is_coordinates_line_displayed then
				main_window.update_coordinates (view.reticle_fields, view.pixel_center_model_coords (a_x, a_y))
			end
		end

	start_filler_creation_at_view_coords (a_x, a_y: INTEGER)
			-- Start creation of filler at view coordinates
			-- (`a_x', `a_y')
		do
			set_operation_start_location (a_x, a_y)
			redraw_filler_style_rubber_box
			report_status ("Filler creation started")
		end

	continue_filler_creation_at_view_coords (a_x, a_y: INTEGER)
			-- Continue creation of filler at view coordinates
			-- (`a_x', `a_y')
		do
			set_operation_current_location (a_x, a_y)
			redraw_filler_style_rubber_box
			report_progress ("Filler creation in progress")
		end

	end_filler_creation_at_view_coords (a_x, a_y: INTEGER)
			-- End creation of filler at view coordinates
			-- (`a_x', `a_y')
		local
			r: like operation_current_rectangle
			rect: CONST_RECTANGLE
			def_name: STRING
			clear: BOOLEAN
			l_size_x, l_size_y: DOUBLE
		do
			set_operation_end_location (a_x, a_y)
			erase_rubber_box
			compute_current_rectangle
			r := operation_current_rectangle
			clear := r.width = 1 and r.height = 1
			rect := view.filler_model_coordinates (r)
			if model.center_fits_in_database (rect.center) then
				l_size_x := rect.size_x.max (grid.size_resolution)
				l_size_y := rect.size_y.max (grid.size_resolution)
				def_name := view.default_filler_name (l_size_x, l_size_y)
				report_status ("Showed Filler Properties window")
				main_window.show_filler_properties_window (def_name, l_size_x, l_size_y, clear, model.upper_case_filler_names)
				if is_last_filler_valid then
					create rect.make_by_size_and_center (last_filler_width, last_filler_height, rect.center_x, rect.center_y)
					view.create_filler (last_filler_name, rect)
					report_change ("Filler created" + last_filler_size_text)
				else
					report_cancellation ("Filler creation cancelled")
				end
			else
				report_failure ("Filler creation cancelled - filler center too big")
			end
			report_end
		end

	set_filler_properties (a_name: STRING a_width, a_height: DOUBLE)
			-- Set properties of filler to be created to `a_name'
			-- with size `a_width' by `a_height'
		do
			last_filler_name := a_name
			last_filler_width := a_width
			last_filler_height := a_height
			is_last_filler_valid := True
		end

	cancel_filler_creation
			-- Cancel creation of filler when user clicks Cancel
			-- in filler properties dialog
		do
			is_last_filler_valid := False
		end

	cancel_filler_creation_at_view_coords (a_x, a_y: INTEGER)
			-- Cancel creation of filler at view coordinates
			-- (`a_x', `a_y')
		do
			set_operation_end_location (a_x, a_y)
			erase_rubber_box
			report_cancellation ("Filler creation cancelled")
		end

	start_scribe_resize_at_view_coords (a_x, a_y: INTEGER)
			-- Start resizing of scribe at view coordinates
			-- (`a_x', `a_y')
		do
			view.start_scribe_resize_at_view_coords (a_x, a_y)
			is_scribe_resize_valid := not view.had_error
			if is_scribe_resize_valid then
				report_status ("Scribe resize started")
			else
				report_failure ("Scribe resize not allowed - " + view.error_messages.first)
			end
		end

	continue_scribe_resize_at_view_coords (a_x, a_y: INTEGER)
			-- Continue resizing of scribe at view coordinates
			-- (`a_x', `a_y')
		do
			view.continue_scribe_resize_at_view_coords (a_x, a_y)
			report_progress ("Scribe resize in progress")
		end

	end_scribe_resize_at_view_coords (a_x, a_y: INTEGER)
			-- End resizing of scribe at view coordinates
			-- (`a_x', `a_y')
		do
			view.end_scribe_resize_at_view_coords (a_x, a_y)
			report_change ("Scribe resized")
			report_end
		end

	cancel_scribe_resize_at_view_coords (a_x, a_y: INTEGER)
			-- Cancel resizing of scribe at view coordinates
			-- (`a_x', `a_y')
		do
			set_operation_end_location (a_x, a_y)
			erase_rubber_box
			report_cancellation ("Scribe resize cancelled")
		end

	measure_distance_at_view_coords (a_x, a_y: INTEGER)
			-- Measure distance from last point to
			-- edge (or corner if close enough to a corner)
			-- closest to (`a_x', `a_y') that is currently visible
		local
			const_dist: CONST_POINT
			dist: POINT
		do
			view.measure_distance_at_view_coords (a_x, a_y)
			if not view.had_error then
				const_dist := view.second_measure_point - view.first_measure_point
				create dist.make_xy (const_dist.x.abs, const_dist.y.abs)
				report_status ("Distance is X = " + grid.formatted_number (dist.x) + ", Y = " + grid.formatted_number (dist.y))
			else
				report_failure (view.error_messages.first)
			end
			report_end
		end

	add_cut_pattern
			-- Add a new "under construction" cut pattern
		do
			main_window.show_cut_pattern_properties_window (view.possible_new_cut_pattern_numbers, view.reticle_fields, view.current_cut_pattern_field, view.assigned_wafer_etest_names, Void, -1, True, True, False)
			if is_last_cut_pattern_valid then
				view.add_cut_pattern (last_cut_pattern_no, last_field_name, last_wafer_etest_name, last_wafer_quantity, last_wafer_quantity >= 0)
				report_change ("Added cut pattern " + last_cut_pattern_no.out + " for field " + last_field_name)
			else
				report_cancellation ("Cut pattern creation cancelled")
			end
			report_end
		end

	create_cut_pattern (a_pattern_no: INTEGER a_field_name, a_wafer_etest_name: STRING a_wafer_quantity: INTEGER)
			-- Set information for Add Cut Pattern operation
			-- to be done
		do
			last_cut_pattern_no := a_pattern_no
			last_field_name := a_field_name
			last_wafer_etest_name := a_wafer_etest_name
			last_wafer_quantity := a_wafer_quantity
			is_last_cut_pattern_valid := True
		end

	cancel_cut_pattern_properties
			-- Cancel operation to set cut pattern properties 
			-- when user clicks Cancel in dialog
		do
			is_last_cut_pattern_valid := False
		end

	delete_cut_pattern
			-- Delete current cut pattern
		local
			n: INTEGER
			op: ANONYMOUS_OPERATION
			status: STRING
		do
			if view.has_current_cut_pattern then
				report_status ("Processing Delete Cut Pattern command")
				n := view.current_cut_pattern.cut_pattern_no
				status := view.current_cut_pattern.status
				create op.make (agent confirm ("Delete " + status + " cut pattern " + n.out + " (cannot be undone)"))
				op.enable_abort_caught
				op.execute
				if op.aborted then
					report_cancellation ("Delete Cut Pattern command aborted")
				else
					view.delete_current_cut_pattern
					report_change ("Deleted cut pattern " + n.out)
				end
			else
				report_failure ("No current cut pattern to delete")
			end
			report_end
		end

	modify_cut_pattern
			-- Make current completed cut pattern modifiable
		local
			n: INTEGER
		do
			if view.has_current_cut_pattern then
				n := view.current_cut_pattern.cut_pattern_no
				if view.current_cut_pattern.is_completed then
					view.modify_current_cut_pattern
					report_change ("Cut pattern " + n.out + " status changed to Under Construction")
				else
					report_failure ("Status of cut pattern " + n.out + " is already Under Construction")
				end
			else
				report_failure ("Cannot change cut pattern status to Under Construction" + no_cut_pattern_text)
			end
			report_end
		end

	save_cut_pattern
			-- Save current cut pattern
		local
			n, err_count, warn_count: INTEGER
			pat: RETICLE_VIEW_CUT_PATTERN
			l_etest_name: STRING
			l_has_quant, ok: BOOLEAN
			pat_nums: ARRAYED_LIST [INTEGER]
			op: ANONYMOUS_OPERATION
		do
			if view.has_current_cut_pattern then
				pat := view.current_cut_pattern
				n := pat.cut_pattern_no
				if not pat.is_completed then
					l_etest_name := pat.wafer_etest_name
					l_has_quant := pat.has_quantity_wafers_needed
					if l_etest_name.is_empty or not l_has_quant then
						create pat_nums.make (1)
						pat_nums.extend (n)
						main_window.show_cut_pattern_properties_window (pat_nums, view.reticle_fields, pat.field_name, view.assigned_wafer_etest_names, pat.wafer_etest_name, pat.quantity_wafers_needed, False, False, True)
						if is_last_cut_pattern_valid then
							ok := True
							view.set_cut_pattern_properties (last_cut_pattern_no, last_field_name, last_wafer_etest_name, last_wafer_quantity, last_wafer_quantity >= 0)
						else
							ok := False
							report_cancellation ("Cut pattern property setting cancelled")
						end
					else
						ok := True
					end
					
					if ok then
						check_all_cut_validity
						err_count := view.error_count
						warn_count := view.warning_count
						if err_count > 0 then
							inform ("Cannot change cut pattern status to Completed due to " + err_count.out + " validity " + su.number (err_count, "error", "errors"))
							ok := False
						elseif warn_count > 0 then
							create op.make (agent confirm_cut_pattern_save)
							op.enable_abort_caught
							op.execute
							ok := not op.aborted
						end
						if ok then
							view.recompute_isolated_site_cut_dies_body
								-- Just call the body
								-- since cut dies were
								-- already computed
							view.complete_current_cut_pattern
							report_change ("Cut pattern " + n.out + " status changed to Completed")
						else
							report_cancellation ("Cut pattern completion cancelled")
						end
					end
				else
					report_failure ("Status of cut pattern " + n.out + " is already Completed")
				end
			else
				report_failure ("Cannot change cut pattern status to Completed" + no_cut_pattern_text)
			end
			report_end
		end

	set_cut_pattern_properties
			-- Change the properties of the current cut
			-- pattern
		local
			n: INTEGER
			pat_nums: ARRAYED_LIST [INTEGER]
			pat: RETICLE_VIEW_CUT_PATTERN
		do
			check_cut_operation_allowed ("set cut pattern properties")
			if is_cut_operation_allowed then
				pat := view.current_cut_pattern
				n := pat.cut_pattern_no
				create pat_nums.make (1)
				pat_nums.extend (n)
				main_window.show_cut_pattern_properties_window (pat_nums, view.reticle_fields, pat.field_name, view.assigned_wafer_etest_names, pat.wafer_etest_name, pat.quantity_wafers_needed, False, False, False)
				if is_last_cut_pattern_valid then
					view.set_cut_pattern_properties (last_cut_pattern_no, last_field_name, last_wafer_etest_name, last_wafer_quantity, last_wafer_quantity >= 0)
					report_change ("Set properties of cut pattern " + last_cut_pattern_no.out)
				else
					report_cancellation ("Cut pattern property setting cancelled")
				end
			end
			report_end
		end

	show_next_cut_pattern
			-- Display the next cut pattern in the view
		do
			if view.at_last_cut_pattern then
				report_failure ("No next cut pattern to display")
			else
				view.show_next_cut_pattern
				report_status ("Showed next cut pattern (" + view.current_cut_pattern.description + ")")
			end
			report_end
		end

	show_previous_cut_pattern
			-- Display the previous cut pattern in the view
		do
			if view.at_first_cut_pattern then
				report_failure ("No previous cut pattern to display")
			else
				view.show_previous_cut_pattern
				report_status ("Showed previous cut pattern (" + view.current_cut_pattern.description + ")")
			end
			report_end
		end

	synthesize_all_cuts
			-- Synthesize one cut through each distinct horizontal
			-- and vertical scribe center in current cut pattern
			-- that does not already have a cut at that position
		do
			check_cut_operation_allowed ("create all cuts")
			if is_cut_operation_allowed then
				view.synthesize_all_cuts
				report_change ("Created " + view.last_cut_text)
			end
			report_end
		end

	add_cuts_by_point (a_coordinate: EV_COORDINATE)
			-- Add a cut through the center of every
			-- scribe that contains `a_coordinate' or through
			-- appropriate implicit scribe(s) outside field
			-- of view edge if `a_coordinate' outside field
		do
			check_cut_operation_allowed ("add cut")
			if is_cut_operation_allowed then
				view.add_cuts_by_point (a_coordinate)
				if not view.had_error then
					report_change ("Created " + view.last_cut_text)
				else
					report_failure (view.error_messages.first)
				end
				display_messages
			end
			report_end
		end

	delete_selected_cuts
			-- Delete from packing all currently selected
			-- cuts.  Ignore selected items that are not cuts
		local
			n: INTEGER
		do
			check_cut_operation_allowed ("delete selected cuts")
			if is_cut_operation_allowed then
				n := view.selected_cuts.count
				if n > 0 then
					view.delete_selected_cuts
					report_change ("Deleted " + view.last_cut_text)
				else
					report_failure ("Delete failed - no cuts selected")
				end
			end
			report_end
		end

	delete_intersecting_cuts
			-- Delete from packing all cuts that intersect
			-- the cut die of any selected project
		local
			n: INTEGER
		do
			check_cut_operation_allowed ("delete cuts intersecting selected projects")
			if is_cut_operation_allowed then
				n := view.selected_projects.count
				if n > 0 then
					view.delete_intersecting_cuts
					report_change ("Deleted " + view.last_cut_text)
				else
					report_failure ("Delete failed - no projects selected")
				end
			end
			report_end
		end

	compute_default_isolation
			-- Compute default set of isolated projects,
			-- based on current cut pattern's cuts and
			-- wafer etest ID
		local
			n: INTEGER
		do
			check_cut_operation_allowed ("compute default set of isolated projects")
			if is_cut_operation_allowed then
				if view.current_cut_pattern.has_wafer_etest_name then
					view.compute_default_isolation
					n := view.last_isolated_count
					report_change ("Computed default set of isolated projects (" + n.out + " isolated)")
				else
					report_failure ("Cannot compute default set of isolated projects because cut pattern lot has not been specified")
				end
			end
			report_end
		end

	toggle_isolation_of_selected_projects
			-- Toggle the "isolated" status of every selected
			-- project
		local
			n: INTEGER
		do
			check_cut_operation_allowed ("toggle isolation of selected projects")
			if is_cut_operation_allowed then
				if view.selected_projects.count > 0 then
					view.toggle_isolation_of_selected_projects
					n := view.last_isolation_failure_count
					if n = 0 then
						report_change ("Toggled isolation of selected projects (" + view.last_isolated_text + ")")
					else
						report_failure ("Toggled isolation of selected projects (" + view.last_isolated_text + ")")
						display_messages
					end
				else
					report_failure ("Toggle Isolation failed - no projects are selected")
				end
			end
			report_end
		end

	switch_to_next_field
			-- Switch to the next field of view (if any)
			-- for current cut pattern
		local
			op: ANONYMOUS_OPERATION
			ok: BOOLEAN
			pat: RETICLE_VIEW_CUT_PATTERN
		do
			check_cut_operation_allowed ("switch cut pattern to next field")
			if is_cut_operation_allowed then
				if view.reticle_fields.count = 1 then
					report_failure ("Cannot switch cut pattern to next field because there is only one field of view")
				else
					if view.current_cut_pattern.has_cuts or view.current_cut_pattern.has_isolated_sites or not view.reticle_change_history_list.is_empty then
						report_status ("Processing Switch to Next Field command")
						create op.make (agent confirm ("Switching cut pattern to next field will discard all cuts,%Ndie isolation information and change history for this cut pattern.%N%NProceed"))
						op.enable_abort_caught
						op.execute
						ok := not op.aborted
					else
						ok := True
					end
					if ok then
						view.switch_to_next_field
						pat := view.current_cut_pattern
						report_change ("Switched cut pattern " + pat.cut_pattern_no.out + " to field " + pat.field_name)
					else
						report_cancellation ("Switch to Next Field command aborted")
					end
				end
			end
			report_end
		end

	set_pack_mode
			-- Switch to Pack mode
		local
			op: ANONYMOUS_OPERATION
		do
			if main_window.is_cut_editor_only then
				report_failure ("Cannot switch to Pack mode when editing cut patterns for a production")
			else
				report_status ("Processing Switch to Pack Mode command")
				create op.make (agent confirm_pack_mode_switch)
				op.enable_abort_caught
				op.execute
				if op.aborted then
					report_cancellation ("Switch to Pack Mode command aborted")
				else
					main_window.switch_to_pack_mode
					report_status ("Switched to Pack mode")
				end
			end
			report_end
		end

	set_cut_mode
			-- Switch to Cut mode.  Check that everything
			-- is legal and warn about any possible problems
			-- first
		local
			op: ANONYMOUS_OPERATION
		do
			report_status ("Processing Switch to Cut Mode command")
			create op.make (agent confirm_cut_mode_switch)
			op.enable_abort_caught
			op.execute
			if op.aborted then
				report_cancellation ("Switch to Cut Mode command aborted")
			else
				main_window.process_switch_to_cut_mode
				if main_window.is_cut_mode then
					report_status ("Switched to Cut Mode")
				else
					report_cancellation ("Switch to Cut Mode command aborted")
				end
			end
			report_end
		end

	report_selected_version (p: PACKING_VERSIONS_ROW)
			-- Report selection of packing version `n'
		local
			l_name, l_rest: STRING
		do
			l_name := p.version_id
			if l_name /= Void and then not l_name.is_empty then
				l_rest := " (" + l_name + ")"
			else
				l_rest := ""
			end
			report_status ("Selected packing version " + p.version_no.out + l_rest)
			report_end
		end

feature {NONE} -- Implementation

	delete_selected (scribes_only: BOOLEAN)
			-- Delete from packing all currently selected items
			-- that are deletable.  If `scribes_only', then
			-- consider only scribes for deletion.  Otherwise,
			-- consider all selected objects for deletion
		local
			proj_list: LIST [PACKING_PROJECTS_MROW]
			non_proj_list: LIST [PACKING_DIES_MROW]
			n: INTEGER
			undoable: BOOLEAN
		do
			n := view.selected_count
			if n > 0 then
				save_selected_count
				view.compute_lists_for_delete (scribes_only)
				proj_list := view.projects_to_delete
				non_proj_list := view.non_project_dies_to_delete
				if not proj_list.is_empty then
					report_status ("Showed Deleted Projects Disposition window")
					main_window.show_delete_window (proj_list)
					if is_last_delete_valid then
						is_remote_delete_disabled := True
						main_window.delete_project_list (proj_list, is_unlock_requested, is_deassign_requested)
						is_remote_delete_disabled := False
					end
				else
					is_last_delete_valid := True
					create {ARRAYED_LIST [PACKING_DIES_MROW]} deleted_dies.make (0)
				end
						
				if is_last_delete_valid then
					main_window.delete_non_project_list (non_proj_list)
					undoable := deleted_dies.is_empty
						-- Undoable if no projects deleted
					deleted_dies.append (non_proj_list)
					view.delete_selected_items (deleted_dies, scribes_only, undoable)
					report_delete_error_summary (scribes_only)
					display_messages
				else
					report_cancellation ("Delete cancelled")
				end
			else
				report_failure ("Delete failed - no objects selected")
			end
			report_end
		end

	set_operation_start_location (a_x, a_y: INTEGER)
			-- Set `operation_start_location' to (`a_x', `a_y')
		do
			operation_start_location.set_x (a_x)
			operation_start_location.set_y (a_y)
			set_operation_current_location (a_x, a_y)
		end

	set_operation_current_location (a_x, a_y: INTEGER)
			-- Set `operation_current_location' to (`a_x', `a_y')
		do
			operation_current_location.set_x (a_x)
			operation_current_location.set_y (a_y)
		end

	set_operation_end_location (a_x, a_y: INTEGER)
			-- Set `operation_end_location' to (`a_x', `a_y')
		do
			operation_end_location.set_x (a_x)
			operation_end_location.set_y (a_y)
		end

	compute_current_rectangle
			-- Set `operation_current_rectangle' from
			-- `operation_current_location' and
			-- `operation_start_location'
		local
			a_x, a_y, a_width, a_height: INTEGER
		do
			a_x := operation_current_location.x.min (operation_start_location.x)
			a_y := operation_current_location.y.min (operation_start_location.y)
			a_width := (operation_current_location.x - operation_start_location.x).abs + 1
			a_height := (operation_current_location.y - operation_start_location.y).abs + 1
			operation_current_rectangle.move_and_resize (a_x, a_y, a_width, a_height)
		end

	redraw_rubber_box
			-- Tell view to redraw rubber box on canvas
		local
			r: like operation_current_rectangle
		do
			compute_current_rectangle
			r := operation_current_rectangle
			view.redraw_rubber_box (r.x, r.y, r.width, r.height)
		end

	redraw_zoom_style_rubber_box
			-- Tell view to redraw zoom-style rubber box on canvas
		local
			r: like operation_current_rectangle
		do
			compute_current_rectangle
			r := operation_current_rectangle
			view.redraw_zoom_style_rubber_box (r.x, r.y, r.width, r.height)
		end

	redraw_selection_style_rubber_box
			-- Tell view to redraw selection-style rubber box
			-- on canvas
		local
			r: like operation_current_rectangle
		do
			compute_current_rectangle
			r := operation_current_rectangle
			view.redraw_selection_style_rubber_box (r.x, r.y, r.width, r.height)
		end

	redraw_scribe_style_rubber_box
			-- Tell view to redraw style of rubber box for
			-- scribe creation on canvas
		local
			r: like operation_current_rectangle
		do
			compute_current_rectangle
			r := operation_current_rectangle
			view.redraw_scribe_style_rubber_box (r.x, r.y, r.width, r.height)
		end

	redraw_scribe_resize_style_rubber_box
			-- Tell view to redraw style of rubber box for
			-- scribe resize on canvas
		local
			r: like operation_current_rectangle
		do
			compute_current_rectangle
			r := operation_current_rectangle
			view.redraw_scribe_style_rubber_box (r.x, r.y, r.width, r.height)
		end

	redraw_filler_style_rubber_box
			-- Tell view to redraw style of rubber box for
			-- filler creation on canvas
		local
			r: like operation_current_rectangle
		do
			compute_current_rectangle
			r := operation_current_rectangle
			view.redraw_filler_style_rubber_box (r.x, r.y, r.width, r.height)
		end

	erase_rubber_box
			-- Tell view to erase rubber box on canvas
		do
			view.erase_rubber_box
		end

	half_truncated_to_grid (val: DOUBLE): DOUBLE
			-- Half of `val', truncated to be on grid
		local
			n: INTEGER_64
		do
			n := grid.rounded_size_units (val)
			if n \\ 2 = 0 then
				Result := val / 2.0
			else
				Result := (val - grid.size_resolution / 2.0) / 2.0
			end
		end

	report_no_move_items
			-- Report "no objects selected" error for Move operation
		do
			report_failure ("Move not allowed - no objects selected")
		end

	report_no_duplicate_items
			-- Report "no objects selected" error for Duplicate
			-- operation
		do
			report_failure ("Duplicate not allowed - no objects selected")
		end

	report_rotate_summary (what: STRING)
			-- Report summary of results of rotation operation 
			-- `what'
		local
			partial_count, locked_count, fixed_rot_count: INTEGER
			selected_count, ok_count, err_count: INTEGER
			msg, errs: STRING
		do
			selected_count := view.selected_count
			partial_count := view.partially_selected_elements.count
			locked_count := view.rotation_locked_elements.count
			fixed_rot_count := view.fixed_orientation_elements.count
			err_count := partial_count + locked_count + fixed_rot_count
			ok_count := selected_count - err_count
			msg := what + " " + ok_count.out + " " + su.number (ok_count, "object", "objects")
			if err_count > 0 then
				msg.append ("   (" + err_count.out + " " + su.number (err_count, "failure", "failures") + ": ")
				create errs.make (50)
				if partial_count > 0 then
					errs.append (partial_count.out + " partially selected")
				end
				if locked_count > 0 then
					if not errs.is_empty then
						errs.append (", ")
					end
					errs.append (locked_count.out + " rotation-locked")
				end
				if fixed_rot_count > 0 then
					if not errs.is_empty then
						errs.append (", ")
					end
					errs.append (fixed_rot_count.out + " fixed orientation")
				end
				msg.append (errs)
				msg.append (")")
				report_failure (msg)
			else
				report_change (msg)
			end
		end

	duplicate_error_summary: STRING
			-- Summary of errors for Duplicate operation
		local
			locked_count, never_dup_count, err_count: INTEGER
			msg, errs: STRING
		do
			locked_count := view.duplication_locked_elements.count
			never_dup_count := view.never_duplicatable_elements.count
			err_count := locked_count + never_dup_count
			msg := "Duplicate not allowed   ("
			create errs.make (50)
			if locked_count > 0 then
				errs.append (locked_count.out + " duplication-locked")
			end
			if never_dup_count > 0 then
				if not errs.is_empty then
					errs.append (", ")
				end
				errs.append (never_dup_count.out + " never duplicatable")
			end
			msg.append (errs)
			msg.append (")")
			Result := msg
		end

	report_spread_error_summary (a_type: STRING)
			-- Report summary of errors for Spread operation
		require
			a_type_not_void: a_type /= Void
		local
			no_move_count: INTEGER
			msg: STRING
		do
			no_move_count := view.unmovable_elements.count
			msg := a_type + " not allowed "
			if no_move_count > 0 then
				msg.append (" (" + no_move_count.out + " position-locked)")
			else
				msg := view.error_messages.first
			end
			report_failure (msg)
		end

	report_scan_error_summary (a_type: STRING)
			-- Report summary of errors for Scan operation
		require
			a_type_not_void: a_type /= Void
		do
			report_failure (a_type + " failed - " + view.error_messages.first)
		end

	report_delete_error_summary (scribes_only: BOOLEAN)
			-- Report summary of errors for Delete operation
		local
			err_count, fail_count, locked_count, ok_count: INTEGER
			msg, errs: STRING
		do
			fail_count := view.delete_failed_elements.count
			locked_count := view.delete_locked_elements.count
			err_count := fail_count + locked_count
			ok_count := view.deleted_elements.count
			msg := "Deleted " + ok_count.out + " "
			if scribes_only then
				msg.append (su.number (ok_count, "scribe", "scribes"))
			else
				msg.append (su.number (ok_count, "object", "objects"))
			end
			if err_count > 0 then
				msg.append ("   (" + err_count.out + " " + su.number (err_count, "failure", "failures") + ": ")
				create errs.make (50)
				if locked_count > 0 then
					errs.append (locked_count.out + " delete-locked")
				end
				if fail_count > 0 then
					if not errs.is_empty then
						errs.append (", ")
					end
					errs.append (fail_count.out + " in rework or database row locked")
				end
				msg.append (errs)
				msg.append (")")
				report_failure (msg)
			else
				report_change (msg)
			end
		end

	report_trim_error_summary
			-- Report summary of errors for Trim Scribes operation
		do
			report_scribe_error_summary ("Trimmed")
		end

	report_separate_error_summary
			-- Report summary of errors for Separate Scribes
			-- operation
		do
			report_scribe_error_summary ("Separated")
		end

	report_extend_error_summary
			-- Report summary of errors for Extend Scribes
			-- operation
		do
			report_scribe_error_summary ("Extended")
		end

	report_unoverlap_error_summary
			-- Report summary of errors for Unoverlap Scribes
			-- operation
		local
			ok_count, resize_err_count, delete_err_count: INTEGER
			tee_count, cross_count, enclosed_count: INTEGER
			err_count: INTEGER
			msg, details: STRING
		do
			ok_count := view.overlap_removed_count
			tee_count := view.overlap_tee_count
			cross_count := view.overlap_cross_count
			enclosed_count := view.overlap_enclosed_count
			resize_err_count := view.overlap_resize_failed_count
			delete_err_count := view.overlap_delete_failed_count
			err_count := resize_err_count + delete_err_count
			msg := "Removed " + ok_count.out + " scribe "
			msg.append (su.number (ok_count, "overlap", "overlaps"))
			create details.make (40)
			if tee_count > 0 then
				details.append (tee_count.out + " tee")
			end
			if cross_count > 0 then
				if not details.is_empty then
					details.append (", ")
				end
				details.append (cross_count.out + " cross")
			end
			if enclosed_count > 0 then
				if not details.is_empty then
					details.append (", ")
				end
				details.append (enclosed_count.out + " enclosed")
			end
			if not details.is_empty then
				msg.append (" - " + details)
			end
			if err_count > 0 then
				msg.append ("   (" + err_count.out + " " + su.number (err_count, "failure", "failures") + ": ")
				if resize_err_count > 0 then
					msg.append (resize_err_count.out + " locked in length dimension")
				end
				if delete_err_count > 0 then
					if resize_err_count > 0 then
						msg.append (", ")
					end
					msg.append (delete_err_count.out + " delete-locked")
				end
				msg.append (")")
				report_failure (msg)
			else
				report_change (msg)
			end
		end

	report_join_error_summary
			-- Report summary of errors for Join Scribes
			-- operation
		local
			locked_err_count, delete_err_count: INTEGER
			msg, errs: STRING
		do
			locked_err_count := view.locked_scribes.count
			delete_err_count := view.undeletable_scribes.count
			msg := "Join Scribes not allowed ("
			create errs.make (40)
			if locked_err_count > 0 then
				errs.append (locked_err_count.out + " " + su.number (locked_err_count, "scribe", "scribes") + " locked in length dimension")
			end
			if delete_err_count > 0 then
				if not errs.is_empty then
					errs.append (", ")
				end
				errs.append (delete_err_count.out + " " + su.number (delete_err_count, "scribe", "scribes") + " delete-locked")
			end
			msg.append (errs)
			msg.append (")")
			report_failure (msg)
		end

	report_join_summary (trimmed_count, separated_count, extended_count: INTEGER)
			-- Report summary for Join Scribes operation
		local
			msg, details, overlap_msg: STRING
			unoverlapped_count, tee_count, cross_count, enclosed_count: INTEGER
			changed_count: INTEGER
		do
			unoverlapped_count := view.overlap_removed_count
			changed_count := trimmed_count + separated_count + extended_count + unoverlapped_count
			tee_count := view.overlap_tee_count
			cross_count := view.overlap_cross_count
			enclosed_count := view.overlap_enclosed_count
			
			msg := "Joined Scribes (" + changed_count.out + " " + su.number (changed_count, "change", "changes")
			create details.make (100)
			if trimmed_count > 0 then
				details.append (trimmed_count.out + " trimmed")
			end
			if separated_count > 0 then
				if not details.is_empty then
					details.append (", ")
				end
				details.append (separated_count.out + " separated")
			end
			if extended_count > 0 then
				if not details.is_empty then
					details.append (", ")
				end
				details.append (extended_count.out + " extended")
			end
			if unoverlapped_count > 0 then
				if not details.is_empty then
					details.append (", ")
				end
				details.append (unoverlapped_count.out + " " + su.number (unoverlapped_count, "overlap", "overlaps") + " removed")
				details.append (" [")
				create overlap_msg.make (30)
				if tee_count > 0 then
					 overlap_msg.append (tee_count.out + " tee")
				end
				if cross_count > 0 then
					if not overlap_msg.is_empty then
						overlap_msg.append (", ")
					end
					overlap_msg.append (cross_count.out + " cross")
				end
				if enclosed_count > 0 then
					if not overlap_msg.is_empty then
						overlap_msg.append (", ")
					end
					overlap_msg.append (enclosed_count.out + " enclosed")
				end
				details.append (overlap_msg)
				details.extend (']')
			end
			if not details.is_empty then
				msg.append (": ")
				msg.append (details)
			end
			msg.append (")")
			report_change (msg)
		end

	report_scribe_error_summary (a_verb: STRING)
			-- Report summary of errors for operation to trim,
			-- separate, extend, or unoverlap scribes
		local
			err_count, ok_count: INTEGER
			msg: STRING
		do
			err_count := view.locked_scribes.count
			ok_count := view.changed_scribes.count
			msg := a_verb + " " + ok_count.out + " "
			msg.append (su.number (ok_count, "scribe", "scribes"))
			if err_count > 0 then
				msg.append ("   (" + err_count.out + " " + su.number (err_count, "failure", "failures") + ": ")
				msg.append (err_count.out + " locked in length dimension)")
				report_failure (msg)
			else
				report_change (msg)
			end
		end

	report_abut_error_summary
			-- Report summary of errors for Abut operation
		local
			msg: STRING
		do
			msg := "Abut not allowed - "
			msg.append (view.error_messages.first)
			report_failure (msg)
		end

	report_align_results (a_type: STRING)
			-- Report results of Align operation described by
			-- `a_type'
		require
			a_type_not_void: a_type /= Void
		do
			if not view.had_error then
				report_change (a_type + " completed" + selected_count_text)
			else
				report_align_error_summary (a_type)
			end
			report_end
		end

	report_align_error_summary (a_type: STRING)
			-- Report summary of errors for Align operation
		require
			a_type_not_void: a_type /= Void
		local
			msg: STRING
		do
			msg := a_type + " not allowed - "
			msg.append (view.error_messages.first)
			report_failure (msg)
		end

	report_validity_check_summary (a_description: STRING)
			-- Report results of validity check operation
		require
			a_description_not_void: a_description /= Void
		local
			msg, errs: STRING
			total_count, err_count, warn_count: INTEGER
		do
			err_count := view.error_count
			warn_count := view.warning_count
			total_count := err_count + warn_count
			msg := a_description
			if total_count > 0 then
				create errs.make (40)
				if err_count > 0 then
					errs.append (err_count.out + " " + su.number (err_count, "error", "errors"))
				end
				if warn_count > 0 then
					if not errs.is_empty then
						errs.append (", ")
					end
					errs.append (warn_count.out + " " + su.number (warn_count, "warning", "warnings"))
				end
				errs.prepend (" (")
				errs.extend (')')
				msg.append (errs)
				report_failure (msg)
			else
				report_validity (msg + " (no problems found)")
			end
			display_messages
			validity_error_count := err_count
			validity_warning_count := warn_count
		end

	confirm_quit
			-- Get confirmation from user before quitting
			-- packing session
		do
			if is_read_only then
				confirm ("Do you want to quit viewing packing")
			else
				confirm ("Do you want to quit packing")
			end
		end

	confirm_cut_pattern_save
			-- Get confirmation from user before saving completed
			-- cut pattern when there are validity warnings
		local
			warn_count: INTEGER
		do
			warn_count := view.warning_count
			warn (warn_count.out + " validity " + su.number (warn_count, "warning", "warnings") + " listed in Output window.%NChange cut pattern status to Completed anyway?")
		end
	
	confirm_finish
			-- Get confirmation from user before finishing
			-- packing session
		local
			n: INTEGER
			list: LIST [PACKING_DIES_ROW]
			msg: STRING
			proj: PACKING_DIES_ROW
		do
			n := view.completed_cut_pattern_count
			confirm ("There " + su.number (n, "is", "are") + " " + n.out + " cut " + su.number (n, "pattern", "patterns") + ".%N%NDo you want to finish packing and proceed to next step")
			list := view.never_isolated_projects
			if not list.is_empty then
				n := list.count
				create msg.make (30 * n)
				msg.append (n.out + " " + su.number (n, "project is", "projects are") + " not isolated by any cut pattern:%N%N")
				from
					list.start
				until
					list.after
				loop
					proj := list.item
					msg.append ("    ")
					msg.append_integer (proj.design_number)
					msg.append ("    ")
					msg.append (proj.design_name)
					msg.extend ('%N')
					list.forth
				end
				msg.append ("%NProceed anyway?")
				warn (msg)
			end
		end

	confirm_pack_mode_switch
			-- Get confirmation from user before switching
			-- to Pack mode
		do
			confirm ("Do you want to switch back to Pack mode")
			if view.has_cut_patterns then
				confirm ("All cut patterns will be discarded.%N%NProceed anyway")
			end
		end

	confirm_cut_mode_switch
			-- Get confirmation from user before switching
			-- to Cut mode
		do
			confirm ("Do you want to switch to Cut mode")
		end

	display_messages
			-- Display error and warning messages from last
			-- operation on view
		do
			display_errors
			display_warnings
			report_end
		end

	display_errors
			-- Display error messages from last operation
			-- on view
		do
			if view.had_error then
				report_details ("Errors:%N" + su.joined_by (view.error_messages, "%N"))
			end
		end

	display_warnings
			-- Display warning messages from last
			-- operation on view
		do
			if view.had_warning then
				report_details ("Warnings:%N" + su.joined_by (view.warning_messages, "%N"))
			end
		end

	save_selected_count
			-- Save current value of view.selected_count
		do
			save_count (view.selected_count)
		end

	save_count (a_value: INTEGER)
			-- Save `a_value' in `old_count'
		do
			old_count := a_value
		end

	selected_text: STRING
			-- Text that describes how many objects are now
			-- selected
		do
			Result := " (" + view.selected_count.out + " selected)"
		end

	added_selected_text: STRING
			-- Text that describes how many objects were added
			-- to selection and how many are now selected
		local
			old_val, new_val, diff: INTEGER
		do
			old_val := old_count
			new_val := view.selected_count
			diff := new_val - old_val
			Result := " (" + diff.out + " added, " + new_val.out + " selected)"
		end

	removed_selected_text: STRING
			-- Text that describes how many objects were removed
			-- from selection and how many are now selected
		local
			old_val, new_val, diff: INTEGER
		do
			old_val := old_count
			new_val := view.selected_count
			diff := old_val - new_val
			Result := " (" + diff.out + " removed, " + new_val.out + " selected)"
		end

	selected_count_text: STRING
			-- Text that describes how many objects were affected
			-- by operation (all selected objects)
		local
			val: INTEGER
		do
			val := view.selected_count
			Result := " (" + val.out + " " + su.number (val, "object", "objects") + ")"
		end

	unselected_count_text: STRING
			-- Text that describes how many objects were affected
			-- by operation (all unselected objects)
		local
			val: INTEGER
		do
			val := view.unselected_count
			Result := " (" + val.out + " " + su.number (val, "object", "objects") + ")"
		end

	selectable_count_text: STRING
			-- Text that describes how many objects were affected
			-- by operation (all selectable objects)
		local
			val: INTEGER
		do
			val := view.selectable_count
			Result := " (" + val.out + " " + su.number (val, "object", "objects") + ")"
		end

	selected_changed_count_text: STRING
			-- Text that describes how many objects an operation
			-- was applied to (all selected objects) and how
			-- many changes resulted
		local
			sel, chg: INTEGER
		do
			sel := view.selected_count
			chg := view.last_change_count
			Result := " (" + sel.out + " " + su.number (sel, "object", "objects") + ", " + chg.out + " changed)"
		end

	unselected_changed_count_text: STRING
			-- Text that describes how many objects an operation
			-- was applied to (all unselected objects) and how
			-- many changes resulted
		local
			sel, chg: INTEGER
		do
			sel := view.unselected_count
			chg := view.last_change_count
			Result := " (" + sel.out + " " + su.number (sel, "object", "objects") + ", " + chg.out + " changed)"
		end

	selectable_changed_count_text: STRING
			-- Text that describes how many objects an operation
			-- was applied to (all selectable objects) and how
			-- many changes resulted
		local
			sel, chg: INTEGER
		do
			sel := view.selectable_count
			chg := view.last_change_count
			Result := " (" + sel.out + " " + su.number (sel, "object", "objects") + ", " + chg.out + " changed)"
		end

	last_move_distance_text: STRING
			-- Text describing distance of last move or duplicate
		do
			Result := grid.formatted_point (view.move_distance)
		end

	last_scribe_size_text (is_horiz: BOOLEAN): STRING
			-- Text describing size of last scribe created,
			-- where `is_horiz' indicates whether it was a
			-- horizontal scribe
		local
			l_size: CONST_POINT
			wid, len: DOUBLE
		do
			l_size := view.last_size
			if is_horiz then
				len := l_size.x
				wid := l_size.y
			else
				len := l_size.y
				wid := l_size.x
			end
			Result := " (length = " + grid.formatted_number (len) + " and width = " + grid.formatted_number (wid) + ")"
		end

	last_filler_size_text: STRING
			-- Text describing size of last filler created
		local
			l_size: CONST_POINT
		do
			l_size := view.last_size
			Result := " (width = " + grid.formatted_number (l_size.x) + " and height = " + grid.formatted_number (l_size.y) + ")"
		end

	check_cut_operation_allowed (an_error_text: STRING)
			-- Check whether cut operation is currently allowed
			-- and set `is_cut_operation_allowed'.  Report
			-- error if not allowed
		require
			an_error_text_not_void: an_error_text /= Void
		local
			pat: RETICLE_VIEW_CUT_PATTERN
			n: INTEGER
		do
			is_cut_operation_allowed := False
			if view.has_current_cut_pattern then
				pat := view.current_cut_pattern
				n := pat.cut_pattern_no
				if not pat.is_completed then
					is_cut_operation_allowed := True
				else
					report_failure ("Cannot " + an_error_text + " because cut pattern " + n.out + " status is Completed")
				end
			else
				report_failure ("Cannot " + an_error_text + no_cut_pattern_text)
			end
		end

	operation_start_location: EV_COORDINATE
			-- View coordinates where last operation started

	operation_current_location: EV_COORDINATE
			-- View coordinates where operation in progress
			-- is currently

	operation_end_location: EV_COORDINATE
			-- View coordinates where last operation end

	operation_current_rectangle: EV_RECTANGLE
			-- Rectangle for current operation (used to
			-- draw rubber box)

	old_count: INTEGER
			-- Last value passed to `save_count'

	last_filler_name: STRING
			-- Name of last valid filler to be created

	last_filler_width: DOUBLE
			-- Width of last valid filler to be created

	last_filler_height: DOUBLE
			-- Height of last valid filler to be created

	is_last_filler_valid: BOOLEAN
			-- Did user accept valid filler properties in
			-- last filler creation dialog?

	is_remote_delete_disabled: BOOLEAN
			-- Should deletes triggered remotely (from outside the
			-- main drawing window) be processed?

	is_last_move_valid: BOOLEAN
			-- Did user accept move in last Move Absolute or
			-- Move Relative dialog?

	last_field_name: STRING
			-- Name of field for last Move Absolute or Add Cut
			-- Pattern operation

	last_move_x: DOUBLE
			-- X value requested for last Move Absolute or
			-- Move Relative operation

	last_move_y: DOUBLE
			-- Y value requested for last Move Absolute or
			-- Move Relative operation

	is_last_delete_valid: BOOLEAN
			-- Did user accept project disposition in
			-- last project deletion dialog?

	is_unlock_requested: BOOLEAN
			-- Did user request that projects whose last copy is
			-- to be deleted be unlocked?

	is_deassign_requested: BOOLEAN
			-- Did user request that projects whose last copy is
			-- to be deleted be deassigned from run?

	deleted_dies: LIST [PACKING_DIES_MROW]
			-- List of dies for projects which were
			-- successfully deleted (last copy was being
			-- deleted) for Delete operation

	is_last_lot_assignment_valid: BOOLEAN
			-- Did user accept a particular wafer etest ID
			-- in last lot assignment dialog?

	is_last_cut_pattern_valid: BOOLEAN
			-- Did user accept values in cut pattern creation
			-- dialog?

	last_cut_pattern_no: INTEGER
			-- Cut pattern number selected in last Set Cut Pattern
			-- Properties operation

	last_wafer_etest_name: STRING
			-- Wafer etest name selected in last Set Cut Pattern
			-- Properties operation

	last_wafer_quantity: INTEGER
			-- Number of wafers to cut entered in last Set Cut 
			-- Pattern Properties operation

	wafer_etest_row: WAFER_ETEST_NAMES_ROW
			-- Wafer etest names row for lot selected by
			-- user (to be assigned to projects)

	is_cut_operation_allowed: BOOLEAN
			-- Is last checked cut operation currently allowed?

	application: EV_APPLICATION
			-- Access to application functions

	minimum_zoom_dimension: INTEGER
			-- Minimum width and height of zoom box in pixels
			-- for it to be interpreted as a zoom
		do
			Result := view.minimum_zoom_dimension_in_pixels
		end

	minimum_scribe_dimension: INTEGER
			-- Minimum width or height of scribe creation box
			-- in pixels for it to be valid
		do
			Result := view.minimum_scribe_dimension_in_pixels
		end

	minimum_selection_dimension: INTEGER
			-- Minimum width and height of selection box in pixels
			-- for it to be interpreted as an area selection rather
			-- than a point selection
		do
			Result := view.minimum_selection_dimension_in_pixels
		end

	grid: GRID
			-- Grid for object sizes and edges
		do
			Result := view.grid
		end

	anchored_model_element: RETICLE_MODEL_ELEMENT
			-- Type of elements in model
		do
		end

	no_cut_pattern_text: STRING = " because there is no current cut pattern"

end
