
indexing
	
	description: "Object to handle the events for reticle packer main %
		%view"
	author: "David Hollenberg"
	date: "July 28, 2006"

class RETICLE_MAIN_VIEW_EVENT_HANDLER
inherit
	VIEW_EVENT_HANDLER
		redefine
			make,
			controller,
			handle_expose,
			handle_resize,
			handle_key_press,
			handle_key_release,
			handle_pointer_button_press,
			handle_pointer_button_release,
			handle_pointer_motion,
			handle_pointer_enter
		end
	EV_KEY_CONSTANTS
		export
			{NONE} all
		end
	MODIFIER_KEY_CONSTANTS
		export
			{NONE} all
		end
	STRING_UTILITIES_ACCESS
		export
			{NONE} all
		end
	GUI_UTILITIES_ACCESS
		export
			{NONE} all
		end
	PACKING_DELEGATED_STATUS_REPORTER
		export
			{ANY} reporter, set_reporter
			{NONE} all
		end

create
	make

feature {NONE} -- Implementation

	make (a_view: like view a_controller: MODEL_VIEW_CONTROLLER)
			-- Create with `a_view' and `a_controller'
		do
			precursor (a_view, a_controller)
			state := Ready
			end_operation
			application := (create {EV_ENVIRONMENT}).application
			create_key_bindings
			current_key_bindings := pack_first_key_bindings
			enable_key_press_handling
		end

feature -- Properties

	controller: RETICLE_MAIN_MODEL_VIEW_CONTROLLER
			-- Controller to be used to handle events

	is_fix_mapped_to_lock: BOOLEAN
			-- Should keys to fix and unfix items be mapped
			-- to lock and unlock?

	is_cut_mode: BOOLEAN
			-- Are we in Cut mode instead of than Pack mode?

	main_window: RC_PACKING_WINDOW
			-- Main packing window

	field_names: LIST [STRING]
			-- Sorted list of names of fields of view

	bindings: LIST [MODIFIED_BINDING]
			-- Key and pointer button bindings
		do
			if cached_bindings = Void then
				set_cached_bindings
			end
			Result := cached_bindings
		end

	is_key_press_handling_enabled: BOOLEAN
			-- Is key press handling currently enabled?

feature -- Modifications

	set_is_fix_mapped_to_lock (b: BOOLEAN)
			-- Set `is_fix_mapped_to_lock' to `b'
		do
			is_fix_mapped_to_lock := b
		ensure
			value_set: is_fix_mapped_to_lock = b
		end

	set_is_cut_mode (b: BOOLEAN)
			-- Set `is_cut_modt' to `b'
		do
			is_cut_mode := b
			set_bindings_for_mode
		ensure
			value_set: is_cut_mode = b
		end

	set_main_window (a_window: like main_window)
		require
			a_window_not_void: a_window /= Void
		do
			main_window := a_window
		ensure
			main_window_set: main_window = a_window
		end

	set_field_names (a_list: like field_names)
			-- Set `field_names' to `a_list'
		do
			field_names := a_list
		ensure
			value_set: field_names = a_list
		end

	enable_key_press_handling
			-- Enable key press handling
		do
			is_key_press_handling_enabled := True
		ensure
			key_press_handling_enabled: is_key_press_handling_enabled
		end

	disable_key_press_handling
			-- Disable key press handling
		do
			is_key_press_handling_enabled := False
		ensure
			key_press_handling_disabled: not is_key_press_handling_enabled
		end

feature -- Operations

	bind
			-- Bind handler to view by extending the
			-- action sequence for events to be handled
		do
			bind_expose_events
			bind_resize_events
			bind_pointer_button_press_events
			bind_pointer_button_release_events
			bind_pointer_motion_events
		end

	handle_expose (a_x, a_y, a_width, a_height: INTEGER)
			-- Handle expose event on `view'
		local
			s: STRING
		do
			debug ("packer_events")
				create s.make (40)
				s.append ("Expose: X = ")
				s.append_integer (a_x)
				s.append (" Y = ")
				s.append_integer (a_y)
				s.append (" W = ")
				s.append_integer (a_width)
				s.append (" H = ")
				s.append_integer (a_height)
				print (s)
				io.new_line
			end
			controller.redraw_rectangle (a_x, a_y, a_width, a_height)
		end

	handle_resize (a_x, a_y, a_width, a_height: INTEGER)
			-- Handle resize event on `view'
		local
			s: STRING
		do
			debug ("packer_events")
				create s.make (35)
				s.append ("Resize: X = ")
				s.append_integer (a_x)
				s.append (" Y = ")
				s.append_integer (a_y)
				s.append (" W = ")
				s.append_integer (a_width)
				s.append (" H = ")
				s.append_integer (a_height)
				print (s)
				io.new_line
			end
			controller.handle_canvas_resize (a_x, a_y, a_width, a_height)
		end

	handle_key_press (some_key: EV_KEY)
			-- Handle press of `some_key on `view'
		local
			s: STRING
			a_key: MODIFIED_KEY
		do
			create a_key.make_with_code_and_application (some_key.code, application)
			debug ("packer_events")
				create s.make (35)
				s.append ("Key press: ")
				s.append (a_key.description)
				print (s)
				io.new_line
			end
			
			if a_key.is_caps_lock then
				is_caps_lock_on := True
			elseif a_key.is_modifier or not is_key_press_handling_enabled then
				-- Ignore
			else
				key_press_translated_actions.call ([a_key])
				inspect state
				when Ready then
					if not is_operation_in_progress or is_operation_started_by_key then
						translate_key_press (a_key)
					else
						-- Don't think we can get here
						report_ignored_key_press (a_key, "end of current operation in progress")
					end
				when Waiting_for_button_release then
					if a_key.is_unmodified_escape then
						translate_key_press (a_key)
					else
						report_ignored_key_press (a_key, "release of " + last_button_pressed.unmodified_description)
					end
				end
			end
		end

	handle_key_release (some_key: EV_KEY)
			-- Handle release of `some_key' on `view'
		local
			s: STRING
			a_key: MODIFIED_KEY
		do
			create a_key.make_with_code_and_application (some_key.code, application)
			
			debug ("packer_events")
				create s.make (35)
				s.append ("Key release: ")
				s.append (a_key.description)
				print (s)
				io.new_line
			end
			
			if a_key.is_caps_lock then
				is_caps_lock_on := False
			end
		end

	handle_pointer_button_press (a_x, a_y, some_button: INTEGER a_x_tilt, a_y_tilt, a_pressure: DOUBLE a_screen_x, a_screen_y: INTEGER)
			-- Handle press of pointer button `some_button'
			-- event on `view'
		local
			s: STRING
			a_button: MODIFIED_POINTER_BUTTON
		do
			create a_button.make_with_number_and_application (some_button, application)
			debug ("pointer_button_events")
				create s.make (80)
				s.append ("Press pointer button ")
				s.append (pointer_description (a_x, a_y, a_button, a_x_tilt, a_y_tilt, a_pressure, a_screen_x, a_screen_y))
				print (s)
				io.new_line
			end
				
			inspect state
			when Ready then
				translate_button_press (a_button, a_x, a_y)
			when Waiting_for_button_release then
				report_ignored_button_press (a_button, "release of " + last_button_pressed.unmodified_description)
			end
		end

	handle_pointer_button_release (a_x, a_y, some_button: INTEGER a_x_tilt, a_y_tilt, a_pressure: DOUBLE a_screen_x, a_screen_y: INTEGER)
			-- Handle release of pointer button `some_button'
			-- on `view'
		local
			s: STRING
			a_button: MODIFIED_POINTER_BUTTON
		do
			create a_button.make_with_number_and_application (some_button, application)
			debug ("pointer_button_events")
				create s.make (80)
				s.append ("Release pointer button ")
				s.append (pointer_description (a_x, a_y, a_button, a_x_tilt, a_y_tilt, a_pressure, a_screen_x, a_screen_y))
				print (s)
				io.new_line
			end
			
			inspect state
			when Ready then
				report_ignored_button_release (a_button, Void)
			when Waiting_for_button_release then
				if a_button.number = last_button_pressed.number then
					if not is_operation_cancelled then
						translate_button_release (a_button, a_x, a_y)
					else
						end_operation
					end
					state := Ready
				else
					report_ignored_button_release (a_button, "release of " + last_button_pressed.unmodified_description)
				end
			end
		end

	handle_pointer_motion (a_x, a_y: INTEGER a_x_tilt, a_y_tilt, a_pressure: DOUBLE a_screen_x, a_screen_y: INTEGER)
			-- Handle pointer motion event on `view'
		local
			s: STRING
		do
			debug ("pointer_button_events")
				create s.make (80)
				s.append ("Pointer motion ")
				s.append (pointer_description (a_x, a_y, Void, a_x_tilt, a_y_tilt, a_pressure, a_screen_x, a_screen_y))
				print (s)
				io.new_line
			end
			
			translate_pointer_motion (a_x, a_y)
			controller.handle_pointer_motion_at_view_coords (a_x, a_y)
		end

	handle_pointer_enter
			-- Handle pointer enter event on `view'
		do
			debug ("packer_events")
				print ("Enter event")
				io.new_line
			end
			-- FIXME: remove when fix focus issues
			-- view.set_focus
		end

	add_late_key_bindings
			-- Initialize key bindings that are only
			-- known when list of field names is known
		require
			field_names_not_void: field_names /= Void
		local
			kb: like pack_first_key_bindings
			cut_kb: like cut_first_key_bindings
			n, l_code: INTEGER
		do
			kb := pack_first_key_bindings
			cut_kb := cut_first_key_bindings
			from
				field_names.start
				n := 1
			until
				field_names.after
			loop
				if is_single_digit (n) then
					l_code := key_code_for_digit (n)
					kb.bind_shift (l_code, agent controller.show_field_by_name (field_names.item))
					cut_kb.bind_shift (l_code, agent report_pack_mode_only_key ("Show Field " + n.out, ?))
				end
				n := n + 1
				field_names.forth
			end
		end

feature {NONE} -- Implementation

	create_key_bindings
			-- Initialize key bindings
		do
			create_pack_first_key_bindings
			create_cut_first_key_bindings
			create_set_selection_key_bindings
			create_add_to_selection_key_bindings
			create_remove_from_selection_key_bindings
			create_display_key_bindings
			create_switch_to_view_key_bindings
			create_horizontal_spread_key_bindings
			create_vertical_spread_key_bindings
			create_move_key_bindings
			create_center_key_bindings
			create_align_key_bindings
			create_trim_key_bindings
			create_break_key_bindings
			create_extend_key_bindings
			create_unoverlap_key_bindings
			create_pack_check_key_bindings
			create_cut_check_key_bindings
			create_fix_key_bindings
			create_unfix_key_bindings
			create_measure_key_bindings
			create_make_filler_bindings
			create_resize_scribe_bindings
			create_miscellaneous_bindings
		end

	create_pack_first_key_bindings
			-- Initialize first key bindings for Pack mode
		local
			kb: like pack_first_key_bindings
		do
			create pack_first_key_bindings.make (agent report_unknown_key_press, agent report_read_only_key_error)
			kb := pack_first_key_bindings
			set_common_first_key_bindings (kb)
			kb.bind_unmodified_change (Key_a, agent abut_selected)
			kb.bind_unmodified (Key_b, agent report_cut_mode_only_key ("Create All Cuts", ?))
			kb.bind_unmodified_change (Key_c, agent start_center)
			kb.bind_unmodified (Key_e, agent report_cut_mode_only_key ("Finish", ?))
			kb.bind_unmodified (Key_f, agent start_fix)
			kb.bind_unmodified_change (Key_h, agent controller.horizontal_scan_selected_items)
			kb.bind_unmodified (Key_i, agent report_cut_mode_only_key ("Toggle Isolation", ?))
			kb.bind_unmodified_change (Key_j, agent controller.join_scribes)
			kb.bind_unmodified_change (Key_k, agent controller.delete_selected_items)
			kb.bind_unmodified (Key_l, agent controller.check_all_pack_validity)
			kb.bind_unmodified (Key_n, agent report_cut_mode_only_key ("Switch to Next Field", ?))
			kb.bind_unmodified_change (Key_p, agent start_filler_creation)
			kb.bind_unmodified_change (Key_r, agent controller.toggle_rotation_of_selected_items)
			kb.bind_unmodified_change (Key_s, agent start_horizontal_spread)
			kb.bind_unmodified (Key_u, agent start_unfix)
			kb.bind_unmodified_change (Key_v, agent controller.vertical_scan_selected_items)
			kb.bind_unmodified (Key_x, agent report_key_too_early (?, "%"F%" or %"U%""))
			kb.bind_unmodified (Key_y, agent report_key_too_early (?, "%"F%" or %"U%""))
			kb.bind_unmodified_change (Key_z, agent start_scribe_resize)
			kb.bind_unmodified (Key_left, agent report_cut_mode_only_key ("Show Previous Cut Pattern", ?))
			kb.bind_unmodified (Key_right, agent report_cut_mode_only_key ("Show Next Cut Pattern", ?))
			kb.bind_control_change (Key_b, agent start_break)
			kb.bind_control (Key_c, agent start_pack_check)
			kb.bind_control_change (Key_e, agent start_extend)
			kb.bind_control_change (Key_t, agent start_trim)
			kb.bind_control_change (Key_u, agent start_unoverlap)
			kb.bind_shift_change (Key_a, agent start_align)
			kb.bind_shift (Key_c, agent controller.set_cut_mode)
			kb.bind_shift (Key_i, agent report_cut_mode_only_key ("Compute Default Isolation", ?))
			kb.bind_shift_change (Key_k, agent controller.delete_selected_scribes)
			kb.bind_shift_change (Key_l, agent controller.assign_lot)
			kb.bind_shift_change (Key_m, agent start_move)
			kb.bind_shift_change (Key_r, agent controller.rotate_selected_items_90_clockwise)
			kb.bind_shift_change (Key_s, agent start_vertical_spread)
		end

	create_cut_first_key_bindings
			-- Initialize first key bindings for Cut mode
		local
			kb: like cut_first_key_bindings
		do
			create cut_first_key_bindings.make (agent report_unknown_key_press, agent report_read_only_key_error)
			kb := cut_first_key_bindings
			set_common_first_key_bindings (kb)
			kb.bind_unmodified_change (Key_a, agent controller.add_cut_pattern)
			kb.bind_unmodified_change (Key_b, agent controller.synthesize_all_cuts)
			kb.bind_unmodified_change (Key_c, agent add_cuts_by_point)
			kb.bind_unmodified_change (Key_e, agent controller.finish)
			kb.bind_unmodified (Key_f, agent report_pack_mode_only_key ("Fix", ?))
			kb.bind_unmodified (Key_h, agent report_pack_mode_only_key ("Horizontal Scan", ?))
			kb.bind_unmodified_change (Key_i, agent controller.toggle_isolation_of_selected_projects)
			kb.bind_unmodified (Key_j, agent report_pack_mode_only_key ("Join Scribes", ?))
			kb.bind_unmodified_change (Key_k, agent controller.delete_selected_cuts)
			kb.bind_unmodified (Key_l, agent controller.check_all_cut_validity)
			kb.bind_unmodified_change (Key_n, agent controller.switch_to_next_field)
			kb.bind_unmodified_change (Key_p, agent controller.set_cut_pattern_properties)
			kb.bind_unmodified_change (Key_r, agent controller.delete_cut_pattern)
			kb.bind_unmodified_change (Key_s, agent controller.save_cut_pattern)
			kb.bind_unmodified (Key_u, agent report_pack_mode_only_key ("Unfix", ?))
			kb.bind_unmodified (Key_v, agent report_pack_mode_only_key ("Vertical Scan", ?))
			kb.bind_unmodified (Key_z, agent report_pack_mode_only_key ("Resize Scribe", ?))
			kb.bind_unmodified (Key_left, agent controller.show_previous_cut_pattern)
			kb.bind_unmodified (Key_right, agent controller.show_next_cut_pattern)
			kb.bind_control (Key_b, agent report_pack_mode_only_key ("Break Scribes", ?))
			kb.bind_control (Key_c, agent start_cut_check)
			kb.bind_control (Key_e, agent report_pack_mode_only_key ("Extend Scribes", ?))
			kb.bind_control (Key_t, agent report_pack_mode_only_key ("Trim Scribes", ?))
			kb.bind_control (Key_u, agent report_pack_mode_only_key ("Unoverlap Scribes", ?))
			kb.bind_shift (Key_a, agent report_pack_mode_only_key ("Align", ?))
			kb.bind_shift (Key_c, agent controller.set_pack_mode)
			kb.bind_shift_change (Key_i, agent controller.compute_default_isolation)
			kb.bind_shift_change (Key_k, agent controller.delete_intersecting_cuts)
			kb.bind_shift (Key_p, agent report_pack_mode_only_key ("Assign Lot", ?))
			kb.bind_shift (Key_r, agent report_pack_mode_only_key ("Rotate Clockwise", ?))
			kb.bind_shift_change (Key_m, agent controller.modify_cut_pattern)
			kb.bind_shift (Key_s, agent report_pack_mode_only_key ("Spread", ?))
		end

	set_common_first_key_bindings (kb: like pack_first_key_bindings)
			-- Initialize `kb' with key bindings common
			-- to both Pack and Cut mode
		local
			n, max_saved_areas, l_code: INTEGER
		do
			kb.bind_unmodified (Key_d, agent start_display)
			kb.bind_unmodified_change (Key_g, agent controller.save_new_version)
			kb.bind_unmodified (Key_m, agent start_measure)
			kb.bind_unmodified (Key_q, agent controller.quit)
			kb.bind_unmodified (Key_t, agent controller.display_info_on_selected_items)
			kb.bind_unmodified (Key_w, agent controller.set_scribe_widths)
			kb.bind_unmodified (Key_x, agent report_key_too_early (?, "%"F%" or %"U%""))
			kb.bind_unmodified (Key_y, agent report_key_too_early (?, "%"F%" or %"U%""))
			kb.bind_unmodified (Key_equal, agent controller.show_entire_model_in_view)
			kb.bind_unmodified_change (Key_delete, agent controller.undo_last_change)
			kb.bind_unmodified (Key_escape, agent abort_operation_if_needed)
			kb.bind_control (Key_a, agent start_add_to_selection)
			kb.bind_control (Key_f, agent controller.search_packing)
			kb.bind_control (Key_h, agent controller.display_help)
			kb.bind_control (Key_m, agent controller.toggle_pointer_coordinates_display)
			kb.bind_control (Key_n, agent controller.show_next_area)
			kb.bind_control (Key_p, agent controller.show_previous_area)
			kb.bind_control (Key_r, agent start_remove_from_selection)
			kb.bind_control (Key_s, agent start_set_selection)
			kb.bind_control (Key_v, agent start_switch_to_view)
			kb.bind_control (Key_x, agent start_miscellaneous)
			kb.bind_control_change (Key_y, agent controller.redo_one_change)
			kb.bind_control_change (Key_z, agent controller.undo_one_change)
			max_saved_areas := controller.view.saved_area_capacity
			from
				n := 1
			until
				n > max_saved_areas
			loop
				if is_single_digit (n) then
					l_code := key_code_for_digit (n)
					kb.bind_unmodified (l_code, agent controller.show_saved_area (n))
					kb.bind_control (l_code, agent controller.set_saved_area (n))
				end
				n := n + 1
			end
		end

	create_set_selection_key_bindings
			-- Initialize Set Selection key bindings
		local
			kb: like set_selection_key_bindings
		do
			create set_selection_key_bindings.make (agent report_invalid_named_key_sequence (?, "Set Selection"), agent report_read_only_key_error)
			kb := set_selection_key_bindings
			kb.set_post_call_action (agent process_after_second_key)
			kb.bind_unmodified (Key_a, agent controller.select_all_items)
			kb.bind_unmodified (Key_b, agent controller.set_selection_to_project_items)
			kb.bind_unmodified (Key_c, agent controller.set_selection_to_cust_project_items)
			kb.bind_unmodified (Key_e, agent controller.set_selection_to_outside_fields_items)
			kb.bind_unmodified (Key_f, agent controller.set_selection_to_floating_scribe_items)
			kb.bind_unmodified (Key_g, agent controller.set_selection_to_vendor_geometry_items)
			kb.bind_unmodified (Key_h, agent controller.set_selection_to_horizontal_scribe_items)
			kb.bind_unmodified (Key_i, agent controller.invert_selection)
			kb.bind_unmodified (Key_l, agent controller.set_selection_to_filler_items)
			kb.bind_unmodified (Key_m, agent select_all_copies_by_point)
			kb.bind_unmodified (Key_n, agent controller.set_selection_to_non_scribe_items)
			kb.bind_unmodified (Key_p, agent controller.set_selection_to_partially_selected_items)
			kb.bind_unmodified (Key_s, agent controller.set_selection_to_scribe_items)
			kb.bind_unmodified (Key_t, agent controller.set_selection_to_test_vehicle_items)
			kb.bind_unmodified (Key_u, agent controller.set_selection_to_cut_items)
			kb.bind_unmodified (Key_v, agent controller.set_selection_to_vertical_scribe_items)
			kb.bind_unmodified (Key_z, agent controller.unselect_all_items)
			kb.bind_unmodified (Key_Escape, agent report_operation_cancellation ("Set Selection"))
			kb.bind_shift (Key_c, agent controller.set_selection_to_commercial_account_items)
			kb.bind_shift (Key_m, agent controller.set_selection_to_mep_account_items)
			kb.bind_shift (Key_o, agent controller.set_selection_to_other_account_items)
		end

	create_add_to_selection_key_bindings
			-- Initialize Add to Selection key bindings
		local
			kb: like add_to_selection_key_bindings
		do
			create add_to_selection_key_bindings.make (agent report_invalid_named_key_sequence (?, "Add to Selection"), agent report_read_only_key_error)
			kb := add_to_selection_key_bindings
			kb.set_post_call_action (agent process_after_second_key)
			kb.bind_unmodified (Key_a, agent controller.select_all_items)
			kb.bind_unmodified (Key_b, agent controller.add_to_selection_project_items)
			kb.bind_unmodified (Key_c, agent controller.add_to_selection_cust_project_items)
			kb.bind_unmodified (Key_e, agent controller.add_to_selection_outside_fields_items)
			kb.bind_unmodified (Key_f, agent controller.add_to_selection_floating_scribe_items)
			kb.bind_unmodified (Key_g, agent controller.add_to_selection_vendor_geometry_items)
			kb.bind_unmodified (Key_h, agent controller.add_to_selection_horizontal_scribe_items)
			kb.bind_unmodified (Key_i, agent controller.select_all_items)
			kb.bind_unmodified (Key_l, agent controller.add_to_selection_filler_items)
			kb.bind_unmodified (Key_m, agent add_to_selection_all_copies_by_point)
			kb.bind_unmodified (Key_n, agent controller.add_to_selection_non_scribe_items)
			kb.bind_unmodified (Key_p, agent controller.add_to_selection_partially_selected_items)
			kb.bind_unmodified (Key_s, agent controller.add_to_selection_scribe_items)
			kb.bind_unmodified (Key_t, agent controller.add_to_selection_test_vehicle_items)
			kb.bind_unmodified (Key_u, agent controller.add_to_selection_cut_items)
			kb.bind_unmodified (Key_v, agent controller.add_to_selection_vertical_scribe_items)
			kb.bind_unmodified (Key_z, agent controller.add_to_selection_no_items)
			kb.bind_unmodified (Key_Escape, agent report_operation_cancellation ("Add to Selection"))
			kb.bind_shift (Key_c, agent controller.add_to_selection_commercial_account_items)
			kb.bind_shift (Key_m, agent controller.add_to_selection_mep_account_items)
			kb.bind_shift (Key_o, agent controller.add_to_selection_other_account_items)
		end

	create_remove_from_selection_key_bindings
			-- Initialize Remove from Selection key bindings
		local
			kb: like remove_from_selection_key_bindings
		do
			create remove_from_selection_key_bindings.make (agent report_invalid_named_key_sequence (?, "Remove from Selection"), agent report_read_only_key_error)
			kb := remove_from_selection_key_bindings
			kb.set_post_call_action (agent process_after_second_key)
			kb.bind_unmodified (Key_a, agent controller.unselect_all_items)
			kb.bind_unmodified (Key_b, agent controller.remove_from_selection_project_items)
			kb.bind_unmodified (Key_c, agent controller.remove_from_selection_cust_project_items)
			kb.bind_unmodified (Key_e, agent controller.remove_from_selection_outside_fields_items)
			kb.bind_unmodified (Key_f, agent controller.remove_from_selection_floating_scribe_items)
			kb.bind_unmodified (Key_g, agent controller.remove_from_selection_vendor_geometry_items)
			kb.bind_unmodified (Key_h, agent controller.remove_from_selection_horizontal_scribe_items)
			kb.bind_unmodified (Key_i, agent controller.remove_from_selection_no_items)
			kb.bind_unmodified (Key_l, agent controller.remove_from_selection_filler_items)
			kb.bind_unmodified (Key_m, agent remove_from_selection_all_copies_by_point)
			kb.bind_unmodified (Key_n, agent controller.remove_from_selection_non_scribe_items)
			kb.bind_unmodified (Key_p, agent controller.remove_from_selection_partially_selected_items)
			kb.bind_unmodified (Key_s, agent controller.remove_from_selection_scribe_items)
			kb.bind_unmodified (Key_t, agent controller.remove_from_selection_test_vehicle_items)
			kb.bind_unmodified (Key_u, agent controller.remove_from_selection_cut_items)
			kb.bind_unmodified (Key_v, agent controller.remove_from_selection_vertical_scribe_items)
			kb.bind_unmodified (Key_z, agent controller.remove_from_selection_no_items)
			kb.bind_unmodified (Key_Escape, agent report_operation_cancellation ("Remove from Selection"))
			kb.bind_shift (Key_c, agent controller.remove_from_selection_commercial_account_items)
			kb.bind_shift (Key_m, agent controller.remove_from_selection_mep_account_items)
			kb.bind_shift (Key_o, agent controller.remove_from_selection_other_account_items)
		end

	create_display_key_bindings
			-- Initialize Display key bindings
		local
			kb: like display_key_bindings
		do
			create display_key_bindings.make (agent report_invalid_named_key_sequence (?, "Display"), agent report_read_only_key_error)
			kb := display_key_bindings
			kb.set_post_call_action (agent process_after_second_key)
			kb.bind_unmodified (Key_a, agent controller.set_application_colors)
			kb.bind_unmodified_change (Key_c, agent controller.set_lot_split_colors)
			kb.bind_unmodified (Key_d, agent controller.display_info_on_selected_items)
			kb.bind_unmodified (Key_l, agent controller.display_lot_summary)
			kb.bind_unmodified (Key_o, agent controller.display_packer_output_window)
			kb.bind_unmodified (Key_p, agent controller.display_project_management_window)
			kb.bind_unmodified (Key_s, agent controller.set_payload_and_kerf_sizes)
			kb.bind_unmodified (Key_v, agent controller.display_version_selector)
			kb.bind_unmodified (Key_w, agent controller.set_scribe_widths)
			kb.bind_unmodified (Key_Escape, agent report_operation_cancellation ("Display"))
		end

	create_switch_to_view_key_bindings
			-- Initialize Switch to View key bindings
		local
			kb: like switch_to_view_key_bindings
		do
			create switch_to_view_key_bindings.make (agent report_invalid_named_key_sequence (?, "Switch to View"), agent report_read_only_key_error)
			kb := switch_to_view_key_bindings
			kb.set_post_call_action (agent process_after_second_key)
			kb.bind_unmodified (Key_a, agent controller.set_account_type_view)
			kb.bind_unmodified (Key_b, agent controller.set_overhead_style_view)
			kb.bind_unmodified (Key_c, agent controller.set_account_number_view)
			kb.bind_unmodified (Key_d, agent controller.set_deletion_locked_view)
			kb.bind_unmodified (Key_f, agent controller.set_search_results_view)
			kb.bind_unmodified (Key_h, agent controller.set_highlighted_selection_view)
			kb.bind_unmodified (Key_i, agent controller.set_die_id_view)
			kb.bind_unmodified (Key_l, agent controller.set_lot_split_view)
			kb.bind_unmodified (Key_m, agent controller.set_max_die_size_view)
			kb.bind_unmodified (Key_n, agent controller.set_design_name_view)
			kb.bind_unmodified (Key_o, agent controller.set_org_short_id_view)
			kb.bind_unmodified (Key_p, agent controller.set_packaging_view)
			kb.bind_unmodified (Key_r, agent controller.set_rotation_locked_view)
			kb.bind_unmodified (Key_s, agent controller.set_standard_view)
			kb.bind_unmodified (Key_t, agent controller.set_cut_view)
			kb.bind_unmodified (Key_u, agent controller.set_duplication_locked_view)
			kb.bind_unmodified (Key_v, agent controller.set_validity_check_view)
			kb.bind_unmodified (Key_Escape, agent report_operation_cancellation ("Switch to View"))
		end

	create_horizontal_spread_key_bindings
			-- Initialize Horizontal Spread key bindings
		local
			kb: like horizontal_spread_key_bindings
		do
			create horizontal_spread_key_bindings.make (agent report_invalid_named_key_sequence (?, "Horizontal Spread"), agent report_read_only_key_error)
			kb := horizontal_spread_key_bindings
			kb.set_post_call_action (agent process_after_second_key)
			kb.bind_unmodified_change (Key_h, agent spread_horizontally_by_desc_height)
			kb.bind_unmodified_change (Key_n, agent spread_horizontally_by_asc_name)
			kb.bind_unmodified_change (Key_s, agent spread_horizontally_by_desc_width)
			kb.bind_unmodified_change (Key_w, agent spread_horizontally_by_desc_width)
			kb.bind_unmodified_change (Key_x, agent spread_horizontally_by_asc_x_position)
			kb.bind_unmodified_change (Key_y, agent spread_horizontally_by_asc_y_position)
			kb.bind_shift_change (Key_h, agent spread_horizontally_by_asc_height)
			kb.bind_shift_change (Key_n, agent spread_horizontally_by_desc_name)
			kb.bind_shift_change (Key_s, agent spread_horizontally_by_asc_width)
			kb.bind_shift_change (Key_w, agent spread_horizontally_by_asc_width)
			kb.bind_shift_change (Key_x, agent spread_horizontally_by_desc_x_position)
			kb.bind_shift_change (Key_y, agent spread_horizontally_by_desc_y_position)
			kb.bind_unmodified (Key_Escape, agent report_operation_cancellation ("Horizontal Spread"))
		end

	create_vertical_spread_key_bindings
			-- Initialize Vertical Spread key bindings
		local
			kb: like vertical_spread_key_bindings
		do
			create vertical_spread_key_bindings.make (agent report_invalid_named_key_sequence (?, "Vertical Spread"), agent report_read_only_key_error)
			kb := vertical_spread_key_bindings
			kb.set_post_call_action (agent process_after_second_key)
			kb.bind_unmodified_change (Key_h, agent spread_vertically_by_asc_height)
			kb.bind_unmodified_change (Key_n, agent spread_vertically_by_desc_name)
			kb.bind_unmodified_change (Key_s, agent spread_vertically_by_asc_height)
			kb.bind_unmodified_change (Key_w, agent spread_vertically_by_asc_width)
			kb.bind_unmodified_change (Key_x, agent spread_vertically_by_desc_x_position)
			kb.bind_unmodified_change (Key_y, agent spread_vertically_by_asc_y_position)
			kb.bind_shift_change (Key_h, agent spread_vertically_by_desc_height)
			kb.bind_shift_change (Key_n, agent spread_vertically_by_asc_name)
			kb.bind_shift_change (Key_s, agent spread_vertically_by_desc_height)
			kb.bind_shift_change (Key_w, agent spread_vertically_by_desc_width)
			kb.bind_shift_change (Key_x, agent spread_vertically_by_asc_x_position)
			kb.bind_shift_change (Key_y, agent spread_vertically_by_desc_y_position)
			kb.bind_unmodified (Key_Escape, agent report_operation_cancellation ("Vertical Spread"))
		end

	create_move_key_bindings
			-- Initialize Move key bindings
		local
			kb: like move_key_bindings
		do
			create move_key_bindings.make (agent report_invalid_named_key_sequence (?, "Move"), agent report_read_only_key_error)
			kb := move_key_bindings
			kb.set_post_call_action (agent process_after_second_key)
			kb.bind_unmodified_change (Key_a, agent controller.move_absolute_selected_items)
			kb.bind_unmodified_change (Key_r, agent controller.move_relative_selected_items)
			kb.bind_unmodified (Key_Escape, agent report_operation_cancellation ("Move"))
		end

	create_center_key_bindings
			-- Initialize Center key bindings
		local
			kb: like center_key_bindings
		do
			create center_key_bindings.make (agent report_invalid_named_key_sequence (?, "Center"), agent report_read_only_key_error)
			kb := center_key_bindings
			kb.set_post_call_action (agent process_after_second_key)
			kb.bind_unmodified_change (Key_a, agent controller.center_all_projects_in_cut_die)
			kb.bind_unmodified_change (Key_s, agent controller.center_selected_projects_in_cut_die)
			kb.bind_unmodified (Key_Escape, agent report_operation_cancellation ("Center"))
		end

	create_align_key_bindings
			-- Initialize Align key bindings
		local
			kb: like align_key_bindings
		do
			create align_key_bindings.make (agent report_invalid_named_key_sequence (?, "Align"), agent report_read_only_key_error)
			kb := align_key_bindings
			kb.set_post_call_action (agent process_after_second_key)
			kb.bind_unmodified_change (Key_b, agent align_bottom)
			kb.bind_unmodified_change (Key_l, agent align_left)
			kb.bind_unmodified_change (Key_r, agent align_right)
			kb.bind_unmodified_change (Key_t, agent align_top)
			kb.bind_unmodified_change (Key_x, agent align_center_x)
			kb.bind_unmodified_change (Key_y, agent align_center_y)
			kb.bind_unmodified (Key_Escape, agent report_operation_cancellation ("Align"))
		end

	create_trim_key_bindings
			-- Initialize Trim Scribes key bindings
		local
			kb: like trim_key_bindings
		do
			create trim_key_bindings.make (agent report_invalid_named_key_sequence (?, "Trim Scribes"), agent report_read_only_key_error)
			kb := trim_key_bindings
			kb.set_post_call_action (agent process_after_second_key)
			kb.bind_unmodified_change (Key_a, agent controller.trim_all_scribes)
			kb.bind_unmodified_change (Key_s, agent controller.trim_selected_scribes)
			kb.bind_unmodified (Key_Escape, agent report_operation_cancellation ("Trim Scribes"))
		end

	create_break_key_bindings
			-- Initialize Break Scribes key bindings
		local
			kb: like break_key_bindings
		do
			create break_key_bindings.make (agent report_invalid_named_key_sequence (?, "Break Scribes"), agent report_read_only_key_error)
			kb := break_key_bindings
			kb.set_post_call_action (agent process_after_second_key)
			kb.bind_unmodified_change (Key_a, agent controller.separate_all_scribes)
			kb.bind_unmodified_change (Key_s, agent controller.separate_selected_scribes)
			kb.bind_unmodified (Key_Escape, agent report_operation_cancellation ("Separate Scribes"))
		end

	create_extend_key_bindings
			-- Initialize Extend Scribes key bindings
		local
			kb: like extend_key_bindings
		do
			create extend_key_bindings.make (agent report_invalid_named_key_sequence (?, "Extend Scribes"), agent report_read_only_key_error)
			kb := extend_key_bindings
			kb.set_post_call_action (agent process_after_second_key)
			kb.bind_unmodified_change (Key_a, agent controller.extend_all_scribes)
			kb.bind_unmodified_change (Key_s, agent controller.extend_selected_scribes)
			kb.bind_unmodified (Key_Escape, agent report_operation_cancellation ("Extend Scribes"))
		end

	create_unoverlap_key_bindings
			-- Initialize Unoverlap Scribes key bindings
		local
			kb: like unoverlap_key_bindings
		do
			create unoverlap_key_bindings.make (agent report_invalid_named_key_sequence (?, "Unoverlap Scribes"), agent report_read_only_key_error)
			kb := unoverlap_key_bindings
			kb.set_post_call_action (agent process_after_second_key)
			kb.bind_unmodified_change (Key_a, agent controller.unoverlap_all_scribes)
			kb.bind_unmodified_change (Key_s, agent controller.unoverlap_selected_scribes)
			kb.bind_unmodified (Key_Escape, agent report_operation_cancellation ("Unoverlap Scribes"))
		end

	create_pack_check_key_bindings
			-- Initialize Check Validity key bindings for
			-- Pack mode
		local
			kb: like pack_check_key_bindings
		do
			create pack_check_key_bindings.make (agent report_invalid_named_key_sequence (?, "Check Validity"), agent report_read_only_key_error)
			kb := pack_check_key_bindings
			kb.set_post_call_action (agent process_after_second_key)
			kb.bind_unmodified (Key_a, agent controller.check_all_pack_validity)
			kb.bind_unmodified (Key_b, agent controller.check_before_join_scribes_validity)
			kb.bind_unmodified (Key_c, agent controller.check_uncentered_in_cut_die_items)
			kb.bind_unmodified (Key_e, agent controller.check_outside_fields_items)
			kb.bind_unmodified (Key_f, agent controller.check_floating_scribes)
			kb.bind_unmodified (Key_g, agent controller.check_close_to_object_items)
			kb.bind_unmodified (Key_i, agent controller.check_unisolated_in_cut_die_items)
			kb.bind_unmodified (Key_l, agent controller.check_lot_not_assigned_items)
			kb.bind_unmodified (Key_m, agent controller.check_misaligned_scribes)
			kb.bind_unmodified (Key_o, agent controller.check_overlapping_object_items)
			kb.bind_unmodified (Key_s, agent controller.check_scribe_cut_die_too_big_items)
			kb.bind_unmodified (Key_t, agent controller.check_below_minimum_size_items)
			kb.bind_unmodified (Key_u, agent controller.check_unlocked_or_unassigned_projects)
			kb.bind_unmodified (Key_v, agent controller.check_test_vehicles)
			kb.bind_unmodified (Key_w, agent controller.check_custom_width_scribes)
			kb.bind_shift (Key_a, agent controller.check_almost_all_validity)
			kb.bind_shift (Key_g, agent controller.check_close_to_field_edge_items)
			kb.bind_shift (Key_o, agent controller.check_overlapping_field_edge_items)
			kb.bind_shift (Key_t, agent controller.check_small_scribes_and_fillers)
			kb.bind_unmodified (Key_Escape, agent report_operation_cancellation ("Check Validity"))
		end

	create_cut_check_key_bindings
			-- Initialize Check Validity key bindings for
			-- Cut mode
		local
			kb: like cut_check_key_bindings
		do
			create cut_check_key_bindings.make (agent report_invalid_named_key_sequence (?, "Check Validity"), agent report_read_only_key_error)
			kb := cut_check_key_bindings
			kb.set_post_call_action (agent process_after_second_key)
			kb.bind_unmodified (Key_a, agent controller.check_all_cut_validity)
			kb.bind_unmodified (Key_c, agent controller.check_invalid_cut_die_items)
			kb.bind_unmodified (Key_e, agent controller.check_extra_cut_items)
			kb.bind_unmodified (Key_l, agent controller.check_wrong_lot_items)
			kb.bind_unmodified (Key_n, agent controller.check_non_default_cut_die_size_items)
			kb.bind_unmodified (Key_o, agent controller.check_shared_cut_die_items)
			kb.bind_unmodified (Key_s, agent controller.check_cut_cut_die_too_big_items)
			kb.bind_unmodified (Key_Escape, agent report_operation_cancellation ("Check Validity"))
		end

	create_fix_key_bindings
			-- Initialize Fix operation key bindings
		local
			kb: like fix_key_bindings
		do
			create fix_key_bindings.make (agent report_invalid_fix_key_sequence, agent report_read_only_key_error)
			kb := fix_key_bindings
			kb.set_post_call_action (agent process_after_second_key)
			kb.bind_unmodified (Key_x, agent fix_in_x)
			kb.bind_unmodified (Key_y, agent fix_in_y)
			kb.bind_unmodified (Key_Escape, agent cancel_fix_operation)
		end

	create_unfix_key_bindings
			-- Initialize Unfix operation key bindings
		local
			kb: like unfix_key_bindings
		do
			create unfix_key_bindings.make (agent report_invalid_unfix_key_sequence, agent report_read_only_key_error)
			kb := unfix_key_bindings
			kb.set_post_call_action (agent process_after_second_key)
			kb.bind_unmodified (Key_x, agent unfix_in_x)
			kb.bind_unmodified (Key_y, agent unfix_in_y)
			kb.bind_unmodified (Key_Escape, agent cancel_unfix_operation)
		end

	create_measure_key_bindings
			-- Initialize Measure operation key bindings
		local
			kb: like measure_key_bindings
		do
			create measure_key_bindings.make (agent report_invalid_named_key_sequence (?, ""), agent report_read_only_key_error)
			kb := measure_key_bindings
			kb.set_post_call_action (agent process_after_second_key)
			kb.bind_unmodified (Key_x, agent controller.measure_selected_items_in_x)
			kb.bind_unmodified (Key_y, agent controller.measure_selected_items_in_y)
			kb.bind_unmodified (Key_Escape, agent report_operation_cancellation ("measure"))
		end

	create_make_filler_bindings
			-- Initialize Create Filler operation key bindings
		local
			kb: like create_filler_key_bindings
		do
			create create_filler_key_bindings.make (agent report_invalid_named_key_sequence (?, ""), agent report_read_only_key_error)
			kb := create_filler_key_bindings
			kb.set_post_call_action (agent process_after_second_key)
			kb.bind_unmodified_change (Key_d, agent end_filler_creation)
			kb.bind_unmodified (Key_Escape, agent cancel_operation_at_coords)
		end

	create_resize_scribe_bindings
			-- Initialize Resize Scribe operation key bindings
		local
			kb: like resize_scribe_key_bindings
		do
			create resize_scribe_key_bindings.make (agent report_invalid_named_key_sequence (?, ""), agent report_read_only_key_error)
			kb := resize_scribe_key_bindings
			kb.set_post_call_action (agent process_after_second_key)
			kb.bind_unmodified_change (Key_z, agent end_scribe_resize)
			kb.bind_unmodified (Key_Escape, agent cancel_operation_at_coords)
		end

	create_miscellaneous_bindings
			-- Initialize Miscellaneous operation key bindings
		local
			kb: like miscellaneous_key_bindings
		do
			create miscellaneous_key_bindings.make (agent report_invalid_named_key_sequence (?, "miscellaneous"), agent report_read_only_key_error)
			kb := miscellaneous_key_bindings
			kb.set_post_call_action (agent process_after_second_key)
			kb.bind_unmodified (Key_q, agent controller.redraw_drawing_area)
			kb.bind_unmodified (Key_Escape, agent report_operation_cancellation ("miscellaneous"))
		end

	 process_after_second_key
			-- Do processing needed after second key of a
			-- sequence
		do
			if not is_operation_started_by_key then
				end_multi_key_sequence
			end
		end

	 key_code_for_digit (n: INTEGER): INTEGER
			-- Key code for digit whose number is `n'
		require
			is_single_digit: is_single_digit (n)
		do
			inspect n
			when 0 then
				Result := Key_0
			when 1 then
				Result := Key_1
			when 2 then
				Result := Key_2
			when 3 then
				Result := Key_3
			when 4 then
				Result := Key_4
			when 5 then
				Result := Key_5
			when 6 then
				Result := Key_6
			when 7 then
				Result := Key_7
			when 8 then
				Result := Key_8
			when 9 then
				Result := Key_9
			end
		end

	 is_single_digit (n: INTEGER): BOOLEAN
			-- Does `n' represent a single digit number?
		do
			Result := n >= 0 and n <= 9
		end

	pointer_description (a_x, a_y: INTEGER a_button: MODIFIED_POINTER_BUTTON a_x_tilt, a_y_tilt, a_pressure: DOUBLE a_screen_x, a_screen_y: INTEGER): STRING
		do
			create Result.make (60)
			if a_button /= Void then
				Result.append (a_button.description)
			end
			Result.append (" at (")
			Result.append_integer (a_x)
			Result.append (", ")
			Result.append_integer (a_y)
			Result.append (") with tilt (")
			Result.append_double (a_x_tilt)
			Result.append (", ")
			Result.append_double (a_y_tilt)
			Result.append (") and pressure ")
			Result.append_double (a_pressure)
			Result.append (" at screen (")
			Result.append_integer (a_screen_x)
			Result.append (", ")
			Result.append_integer (a_screen_y)
			Result.append (")")
		end

	translate_key_press (a_key: MODIFIED_KEY)
			-- Translate key press into call to controller
			-- or report error if not valid
		require
			a_key_not_void: a_key /= Void
		do
			current_key_bindings.call (a_key, is_read_only)
		end

	abut_selected
		do
			controller.abut_selected_item (view.pointer_position)
		end

	start_sequence (a_key: MODIFIED_KEY a_key_bindings: MODIFIED_KEY_BIND_SET a_command: STRING)
			-- Start multi-key sequence whose first key is `a_key',
			-- switching to `a_key_bindings' for next key, for
			-- `a_command'
		require
			a_key_not_void: a_key /= Void
			a_key_bindings_not_void: a_key_bindings /= Void
			a_command_not_void: a_command /= Void
		do
			start_multi_key_sequence (a_key, a_key_bindings)
			report_start
			report_status (a_command + " command started - waiting for next character")
		end

	start_set_selection (a_key: MODIFIED_KEY)
		require
			a_key_not_void: a_key /= Void
		do
			start_sequence (a_key, set_selection_key_bindings, "Set Selection")
		end

	start_add_to_selection (a_key: MODIFIED_KEY)
		require
			a_key_not_void: a_key /= Void
		do
			start_sequence (a_key, add_to_selection_key_bindings, "Add to Selection")
		end

	start_remove_from_selection (a_key: MODIFIED_KEY)
		require
			a_key_not_void: a_key /= Void
		do
			start_sequence (a_key, remove_from_selection_key_bindings, "Remove from Selection")
		end

	start_display (a_key: MODIFIED_KEY)
		require
			a_key_not_void: a_key /= Void
		do
			start_sequence (a_key, display_key_bindings, "Display")
		end

	start_switch_to_view (a_key: MODIFIED_KEY)
		require
			a_key_not_void: a_key /= Void
		do
			start_sequence (a_key, switch_to_view_key_bindings, "Switch to View")
		end

	start_horizontal_spread (a_key: MODIFIED_KEY)
		require
			a_key_not_void: a_key /= Void
		do
			start_sequence (a_key, horizontal_spread_key_bindings, "Horizontal Spread")
		end

	start_vertical_spread (a_key: MODIFIED_KEY)
		require
			a_key_not_void: a_key /= Void
		do
			start_sequence (a_key, vertical_spread_key_bindings, "Vertical Spread")
		end

	start_align (a_key: MODIFIED_KEY)
		require
			a_key_not_void: a_key /= Void
		do
			start_sequence (a_key, align_key_bindings, "Align")
		end

	start_move (a_key: MODIFIED_KEY)
		require
			a_key_not_void: a_key /= Void
		do
			start_sequence (a_key, move_key_bindings, "Move")
		end

	start_center (a_key: MODIFIED_KEY)
		require
			a_key_not_void: a_key /= Void
		do
			start_sequence (a_key, center_key_bindings, "Center")
		end

	start_trim (a_key: MODIFIED_KEY)
		require
			a_key_not_void: a_key /= Void
		do
			start_sequence (a_key, trim_key_bindings, "Trim Scribes")
		end

	start_break (a_key: MODIFIED_KEY)
		require
			a_key_not_void: a_key /= Void
		do
			start_sequence (a_key, break_key_bindings, "Break Scribes")
		end

	start_extend (a_key: MODIFIED_KEY)
		require
			a_key_not_void: a_key /= Void
		do
			start_sequence (a_key, extend_key_bindings, "Extend Scribes")
		end

	start_unoverlap (a_key: MODIFIED_KEY)
		require
			a_key_not_void: a_key /= Void
		do
			start_sequence (a_key, unoverlap_key_bindings, "Unoverlap Scribes")
		end

	start_pack_check (a_key: MODIFIED_KEY)
		require
			a_key_not_void: a_key /= Void
		do
			start_sequence (a_key, pack_check_key_bindings, "Check Validity")
		end

	start_cut_check (a_key: MODIFIED_KEY)
		require
			a_key_not_void: a_key /= Void
		do
			start_sequence (a_key, cut_check_key_bindings, "Check Validity")
		end

	start_fix (a_key: MODIFIED_KEY)
		require
			a_key_not_void: a_key /= Void
		do
			start_sequence (a_key, fix_key_bindings, capitalized_name_for_fix_operation)
		end

	start_measure (a_key: MODIFIED_KEY)
		require
			a_key_not_void: a_key /= Void
		do
			start_sequence (a_key, measure_key_bindings, "Measure")
		end

	start_filler_creation (a_key: MODIFIED_KEY)
		require
			a_key_not_void: a_key /= Void
		local
			p: EV_COORDINATE
		do
			start_multi_key_sequence (a_key, create_filler_key_bindings)
			start_operation (Create_filler)
			p := view.pointer_position
			controller.start_filler_creation_at_view_coords (p.x, p.y)
		end

	start_unfix (a_key: MODIFIED_KEY)
		require
			a_key_not_void: a_key /= Void
		do
			start_sequence (a_key, unfix_key_bindings, "Un" + name_for_fix_operation)
		end

				
	spread_horizontally_by_asc_width
		do
			controller.spread_selected_items_horizontally_by_asc_width (view.pointer_position)
		end
	
	spread_horizontally_by_desc_width
		do
			controller.spread_selected_items_horizontally_by_desc_width (view.pointer_position)
		end
	
	spread_horizontally_by_asc_height
		do
			controller.spread_selected_items_horizontally_by_asc_height (view.pointer_position)
		end
	
	spread_horizontally_by_desc_height
		do
			controller.spread_selected_items_horizontally_by_desc_height (view.pointer_position)
		end
	
	spread_vertically_by_asc_width
		do
			controller.spread_selected_items_vertically_by_asc_width (view.pointer_position)
		end
				
	spread_vertically_by_desc_width
		do
			controller.spread_selected_items_vertically_by_desc_width (view.pointer_position)
		end
				
	spread_vertically_by_asc_height
		do
			controller.spread_selected_items_vertically_by_asc_height (view.pointer_position)
		end
				
	spread_vertically_by_desc_height
		do
			controller.spread_selected_items_vertically_by_desc_height (view.pointer_position)
		end
				
	spread_horizontally_by_asc_x_position
		do
			controller.spread_selected_items_horizontally_by_asc_x_position (view.pointer_position)
		end
	
	spread_horizontally_by_desc_x_position
		do
			controller.spread_selected_items_horizontally_by_desc_x_position (view.pointer_position)
		end
	
	spread_horizontally_by_asc_y_position
		do
			controller.spread_selected_items_horizontally_by_asc_y_position (view.pointer_position)
		end
	
	spread_horizontally_by_desc_y_position
		do
			controller.spread_selected_items_horizontally_by_desc_y_position (view.pointer_position)
		end
	
	spread_vertically_by_asc_x_position
		do
			controller.spread_selected_items_vertically_by_asc_x_position (view.pointer_position)
		end
				
	spread_vertically_by_desc_x_position
		do
			controller.spread_selected_items_vertically_by_desc_x_position (view.pointer_position)
		end
				
	spread_vertically_by_asc_y_position
		do
			controller.spread_selected_items_vertically_by_asc_y_position (view.pointer_position)
		end
				
	spread_vertically_by_desc_y_position
		do
			controller.spread_selected_items_vertically_by_desc_y_position (view.pointer_position)
		end
				
	spread_horizontally_by_asc_name
		do
			controller.spread_selected_items_horizontally_by_asc_name (view.pointer_position)
		end
				
	spread_horizontally_by_desc_name
		do
			controller.spread_selected_items_horizontally_by_desc_name (view.pointer_position)
		end
				
	spread_vertically_by_asc_name
		do
			controller.spread_selected_items_vertically_by_asc_name (view.pointer_position)
		end
				
	spread_vertically_by_desc_name
		do
			controller.spread_selected_items_vertically_by_desc_name (view.pointer_position)
		end
				
	align_left
		do
			controller.align_left_selected_items (view.pointer_position)
		end
				
	align_right
		do
			controller.align_right_selected_items (view.pointer_position)
		end
				
	align_top
		do
			controller.align_top_selected_items (view.pointer_position)
		end
				
	align_bottom
		do
			controller.align_bottom_selected_items (view.pointer_position)
		end
				
	align_center_x
		do
			controller.align_center_x_selected_items (view.pointer_position)
		end
				
	align_center_y
		do
			controller.align_center_y_selected_items (view.pointer_position)
		end
				
	abort_operation_if_needed (a_key: MODIFIED_KEY)
		require
			a_key_not_void: a_key /= Void
		local
			p: EV_COORDINATE
		do
			if is_operation_in_progress then
				p := view.pointer_position
				translate_operation_cancellation (p.x, p.y)
				cancel_operation
			else
				report_no_operation_to_cancel (a_key)
			end
		end

	fix_in_x
		do
			if is_fix_mapped_to_lock then
				controller.lock_selected_items_in_x
			else
				controller.fix_selected_items_in_x
			end
		end

	fix_in_y
		do
			if is_fix_mapped_to_lock then
				controller.lock_selected_items_in_y
			else
				controller.fix_selected_items_in_y
			end
		end

	cancel_fix_operation
		do
			report_operation_cancellation (name_for_fix_operation)
		end

	unfix_in_x
		do
			if is_fix_mapped_to_lock then
				controller.unlock_selected_items_in_x
			else
				controller.unfix_selected_items_in_x
			end
		end

	unfix_in_y
		do
			if is_fix_mapped_to_lock then
				controller.unlock_selected_items_in_y
			else
				controller.unfix_selected_items_in_y
			end
		end

	cancel_unfix_operation
		do
			report_operation_cancellation ("un" + name_for_fix_operation)
		end

	end_filler_creation
		local
			p: EV_COORDINATE
		do
			p := view.pointer_position
			end_operation
			controller.end_filler_creation_at_view_coords (p.x, p.y)
		end

	cancel_operation_at_coords
		local
			p: EV_COORDINATE
		do
			p := view.pointer_position
			translate_operation_cancellation (p.x, p.y)
			cancel_operation
		end

	start_scribe_resize (a_key: MODIFIED_KEY)
		require
			a_key_not_void: a_key /= Void
		local
			p: EV_COORDINATE
		do
			start_multi_key_sequence (a_key, resize_scribe_key_bindings)
			start_operation (Resize_scribe)
			p := view.pointer_position
			controller.start_scribe_resize_at_view_coords (p.x, p.y)
			if not controller.is_scribe_resize_valid then
				end_operation
				end_multi_key_sequence
			end
		end

	end_scribe_resize
		local
			p: EV_COORDINATE
		do
			p := view.pointer_position
			end_operation
			controller.end_scribe_resize_at_view_coords (p.x, p.y)
		end
	
	start_miscellaneous (a_key: MODIFIED_KEY)
		require
			a_key_not_void: a_key /= Void
		do
			start_sequence (a_key, miscellaneous_key_bindings, "Miscellaneous")
		end

	select_all_copies_by_point
		do
			controller.set_selection_all_copies_by_point (view.pointer_position)
		end

	add_to_selection_all_copies_by_point
		do
			controller.add_to_selection_all_copies_by_point (view.pointer_position)
		end

	remove_from_selection_all_copies_by_point
		do
			controller.remove_from_selection_all_copies_by_point (view.pointer_position)
		end

	add_cuts_by_point
		do
			controller.add_cuts_by_point (view.pointer_position)
		end

	translate_button_press (a_button: MODIFIED_POINTER_BUTTON a_x, a_y: INTEGER)
			-- Translate press of `a_button' at view coordinates
			-- (`a_x', `a_y')
		local
			is_button_known: BOOLEAN
		do
			if in_multi_key_sequence then
				report_ignored_button_press (a_button, "next key after " + previous_key.quoted_description)
			else
				is_button_known := True
				inspect a_button.number
				when 1 then
					inspect a_button.modifier_code
					when Unmodified_code then
						start_operation (Set_selection)
						controller.start_set_selection_at_view_coords (a_x, a_y)
					when Ctrl_code then
						start_operation (Add_to_selection)
						controller.start_add_selection_at_view_coords (a_x, a_y)
					when Shift_code then
						start_operation (Remove_from_selection)
						controller.start_remove_selection_at_view_coords (a_x, a_y)
					when Ctrl_alt_shift_code then
						start_operation (Set_selection_by_intersection)
						controller.start_set_selection_by_intersection (a_x, a_y)
					when Ctrl_alt_code then
						start_operation (Add_to_selection_by_intersection)
						controller.start_add_selection_by_intersection (a_x, a_y)
					when Alt_shift_code then
						start_operation (Remove_from_selection_by_intersection)
						controller.start_remove_selection_by_intersection (a_x, a_y)
					else
						is_button_known := False
					end
				when 2 then
					inspect a_button.modifier_code
					when Unmodified_code then
						if is_cut_mode then
							handle_cut_mode_button_error (a_button, "Move")
						elseif is_read_only then
							handle_read_only_button_error (a_button, "Move")
						else
							start_operation (Move_selected)
							controller.start_move_selected_at_view_coords (a_x, a_y)
						end
					when Ctrl_code then
						if is_cut_mode then
							handle_cut_mode_button_error (a_button, "Duplicate")
						elseif is_read_only then
							handle_read_only_button_error (a_button, "Duplicate")
						else
							start_operation (Duplicate_selected)
							controller.start_duplicate_selected_at_view_coords (a_x, a_y)
						end
					when Shift_code then
						if is_cut_mode then
							handle_cut_mode_button_error (a_button, "Create Scribe")
						elseif is_read_only then
							handle_read_only_button_error (a_button, "Create Scribe")
						else
							start_operation (Create_scribe)
							controller.start_scribe_creation_at_view_coords (a_x, a_y)
						end
					else
						is_button_known := False
					end
				when 3 then
					inspect a_button.modifier_code
					when Unmodified_code then
						start_operation (Zoom_in)
						controller.start_zoom_in_at_view_coords (a_x, a_y)
					when Ctrl_code then
						start_operation (Zoom_out)
						controller.start_zoom_out_at_view_coords (a_x, a_y)
					when Shift_code then
						start_operation (Pan)
						controller.start_pan_at_view_coords (a_x, a_y)
					when Ctrl_shift_code then
						start_operation (Measure_distance)
						controller.measure_distance_at_view_coords (a_x, a_y)
					else
						is_button_known := False
					end
				else
					is_button_known := False
				end
				if is_button_known then
					last_button_pressed := a_button
					state := Waiting_for_button_release
				else
					report_unknown_button_press (a_button)
				end
			end
		end

	translate_pointer_motion (a_x, a_y: INTEGER)
			-- Translate pointer motion to view coordinates
			-- (`a_x', `a_y')
		do
			inspect operation
			when No_operation then
			when Set_selection then
				controller.continue_set_selection_at_view_coords (a_x, a_y)
			when Add_to_selection then
				controller.continue_add_selection_at_view_coords (a_x, a_y)
			when Remove_from_selection then
				controller.continue_remove_selection_at_view_coords (a_x, a_y)
			when Set_selection_by_intersection then
				controller.continue_set_selection_by_intersection (a_x, a_y)
			when Add_to_selection_by_intersection then
				controller.continue_add_selection_by_intersection (a_x, a_y)
			when Remove_from_selection_by_intersection then
				controller.continue_remove_selection_by_intersection (a_x, a_y)
			when Move_selected then
				controller.continue_move_selected_at_view_coords (a_x, a_y)
			when Duplicate_selected then
				controller.continue_duplicate_selected_at_view_coords (a_x, a_y)
			when Create_scribe then
				controller.continue_scribe_creation_at_view_coords (a_x, a_y)
			when Zoom_in then
				controller.continue_zoom_in_at_view_coords (a_x, a_y)
			when Zoom_out then
				controller.continue_zoom_out_at_view_coords (a_x, a_y)
			when Pan then
				controller.continue_pan_at_view_coords (a_x, a_y)
			when Create_filler then
				controller.continue_filler_creation_at_view_coords (a_x, a_y)
			when Resize_scribe then
				controller.continue_scribe_resize_at_view_coords (a_x, a_y)
			when Measure_distance then
				-- Nothing to do
			else
				report_internal_error ("Unknown operation code " + operation.out)
			end
		end

	translate_button_release (a_button: MODIFIED_POINTER_BUTTON a_x, a_y: INTEGER)
			-- Translate release of `a_button' at view coordinates
			-- (`a_x', `a_y')
		do
			inspect operation
			when No_operation then
				report_internal_error ("Release of " + a_button.unmodified_description + " when no operation in progress")
			when Set_selection then
				controller.end_set_selection_at_view_coords (a_x, a_y)
			when Add_to_selection then
				controller.end_add_selection_at_view_coords (a_x, a_y)
			when Remove_from_selection then
				controller.end_remove_selection_at_view_coords (a_x, a_y)
			when Set_selection_by_intersection then
				controller.end_set_selection_by_intersection (a_x, a_y)
			when Add_to_selection_by_intersection then
				controller.end_add_selection_by_intersection (a_x, a_y)
			when Remove_from_selection_by_intersection then
				controller.end_remove_selection_by_intersection (a_x, a_y)
			when Move_selected then
				controller.end_move_selected_at_view_coords (a_x, a_y)
			when Duplicate_selected then
				controller.end_duplicate_selected_at_view_coords (a_x, a_y)
			when Create_scribe then
				controller.end_scribe_creation_at_view_coords (a_x, a_y)
			when Zoom_in then
				controller.end_zoom_in_at_view_coords (a_x, a_y)
			when Zoom_out then
				controller.end_zoom_out_at_view_coords (a_x, a_y)
			when Pan then
				controller.end_pan_at_view_coords (a_x, a_y)
			when Create_filler then
				controller.end_filler_creation_at_view_coords (a_x, a_y)
			when Resize_scribe then
				controller.end_scribe_resize_at_view_coords (a_x, a_y)
			when Measure_distance then
				-- Nothing to do
			else
				report_internal_error ("Unknown operation code " + operation.out)
			end
			end_operation
		end

	translate_operation_cancellation (a_x, a_y: INTEGER)
			-- Translate cancellation of current operation into
			-- at view coordinates (`a_x', `a_y') into call
			-- to controller
		do
			inspect operation
			when No_operation then
				report_internal_error ("Cancel when no operation in progress")
			when Set_selection then
				controller.cancel_set_selection_at_view_coords (a_x, a_y)
			when Add_to_selection then
				controller.cancel_add_selection_at_view_coords (a_x, a_y)
			when Remove_from_selection then
				controller.cancel_remove_selection_at_view_coords (a_x, a_y)
			when Set_selection_by_intersection then
				controller.cancel_set_selection_by_intersection (a_x, a_y)
			when Add_to_selection_by_intersection then
				controller.cancel_add_selection_by_intersection (a_x, a_y)
			when Remove_from_selection_by_intersection then
				controller.cancel_remove_selection_by_intersection (a_x, a_y)
			when Move_selected then
				controller.cancel_move_selected_at_view_coords (a_x, a_y)
			when Duplicate_selected then
				controller.cancel_duplicate_selected_at_view_coords (a_x, a_y)
			when Create_scribe then
				controller.cancel_scribe_creation_at_view_coords (a_x, a_y)
			when Zoom_in then
				controller.cancel_zoom_in_at_view_coords (a_x, a_y)
			when Zoom_out then
				controller.cancel_zoom_out_at_view_coords (a_x, a_y)
			when Pan then
				controller.cancel_pan_at_view_coords (a_x, a_y)
			when Create_filler then
				controller.cancel_filler_creation_at_view_coords (a_x, a_y)
			when Resize_scribe then
				controller.cancel_scribe_resize_at_view_coords (a_x, a_y)
			when Measure_distance then
				-- Nothing to do
			else
				report_internal_error ("Unknown operation code " + operation.out)
			end
		end

	start_multi_key_sequence (a_key: MODIFIED_KEY a_bindset: MODIFIED_KEY_BIND_SET)
			-- Record information about `a_key' which was
			-- pressed and is first in a multi-key
			-- sequence.  Switch current key bindings to
			-- `a_bindset'
		require
			a_key_not_void: a_key /= Void
			a_bindset_not_void: a_bindset /= Void
		do
			previous_key := a_key
			in_multi_key_sequence := True
			current_key_bindings := a_bindset
		end

	end_multi_key_sequence
			-- Record end of multi-key sequence
		do
			previous_key := Void
			in_multi_key_sequence := False
			set_bindings_for_mode
		end

	set_bindings_for_mode
			-- Set key bindings based on whether we're in
			-- Pack mode or Cut mode
		do
			if is_cut_mode then
				current_key_bindings := cut_first_key_bindings
			else
				current_key_bindings := pack_first_key_bindings
			end
		end

	start_operation (op: INTEGER)
			-- Indicate start of operation whose code is `op'
		require
			no_operation_in_progress: not is_operation_in_progress
		do
			operation := op
			is_operation_cancelled := False
			report_start
		ensure
			
		end

	end_operation
			-- Indicate end of current operation, if any
		do
			operation := No_operation
			is_operation_cancelled := False
		end

	cancel_operation
			-- Indicate cancellation of current operation
		do
			operation := No_operation
			is_operation_cancelled := True
		end

	handle_read_only_button_error (a_button: MODIFIED_POINTER_BUTTON op: STRING)
			-- Handle a user error where user tried to do
			-- an operation using a pointer button that is not
			-- allowed in read-only mode
		do
			end_operation
			report_read_only_button_error (a_button, op)
		end

	handle_cut_mode_button_error (a_button: MODIFIED_POINTER_BUTTON op: STRING)
			-- Handle a user error where user tried to do
			-- an operation using a pointer button that is not
			-- allowed in Cut mode and we're currently in
			-- Cut mode
		do
			end_operation
			report_cut_mode_button_error (a_button, op)
		end

	report_unknown_key_press (a_key: MODIFIED_KEY)
			-- Report press of unknown key `a_key'
		require
			a_key_not_void: a_key /= Void
		do
			report_status ("Unknown key " + a_key.quoted_description + " ignored")
			gui.ring_bell_once
		end

	report_read_only_key_error (a_key: MODIFIED_KEY)
			-- Report press of key `a_key' that invokes a
			-- non read-only action when in read-only mode
		require
			a_key_not_void: a_key /= Void
		do
			report_status ("Operation for key " + a_key.quoted_description + " not allowed in read-only mode")
			gui.ring_bell_once
		end

	report_ignored_key_press (a_key: MODIFIED_KEY what: STRING)
			-- Report key press of `a_key' that was ignored
			-- during `what'
		require
			a_key_not_void: a_key /= Void
			what_not_void: what /= Void
		do
			report_status ("Key " + a_key.quoted_description + " ignored while waiting for " + what)
			gui.ring_bell_once
		end

	report_unimplemented_key (an_operation: STRING a_key: MODIFIED_KEY)
			-- Report press of known key `a_key' that is
			-- invokes `an_operation', which is not yet
			-- implemented
		require
			an_operation_not_void: an_operation /= Void
			a_key_not_void: a_key /= Void
		do
			report_status ("Operation " + an_operation + " not yet implemented - ignored key " + a_key.quoted_description)
			gui.ring_bell_once
		end

	report_unknown_first_key (a_key: MODIFIED_KEY)
			-- Report internal error involving unexpected
			-- first key of a multi-key sequence
		require
			a_key_not_void: a_key /= Void
		do
			gui.ring_bell_once
			report_internal_error ("Key " + a_key.quoted_description + " not recognized as first key of multi-key sequence")
		end

	report_invalid_fix_key_sequence (a_key: MODIFIED_KEY)
			-- Report press of invalid second key for Fix operation
		require
			a_key_not_void: a_key /= Void
		do
			report_invalid_key_sequence (previous_key, a_key, name_for_fix_operation)
		end

	report_invalid_unfix_key_sequence (a_key: MODIFIED_KEY)
			-- Report press of invalid second key for Unfix
			-- operation
		require
			a_key_not_void: a_key /= Void
		do
			report_invalid_key_sequence (previous_key, a_key, "un" + name_for_fix_operation)
		end

	report_invalid_named_key_sequence (a_key: MODIFIED_KEY an_op_name: STRING)
			-- Report press of invalid second key for named
			-- operation
		require
			a_key_not_void: a_key /= Void
			an_op_name_not_void: an_op_name /= Void
		do
			report_invalid_key_sequence (previous_key, a_key, an_op_name)
		end

	report_invalid_key_sequence (a_first_key, a_key: MODIFIED_KEY a_name: STRING)
			-- Report press of invalid second key `a_key' for
			-- multi-key sequence starting with `a_first_key'
			-- and report that `a_name' was cancelled if it is
			-- not empty
		require
			a_first_key_not_void: a_first_key /= Void
			a_key_not_void: a_key /= Void
			a_name_not_void: a_name /= Void
		local
			msg: STRING
		do
			msg := "Invalid key " + a_key.quoted_description + " following key " + a_first_key.quoted_description
			if not a_name.is_empty then
				msg.append (" - cancelled " + a_name)
			end
			report_status (msg)
			gui.ring_bell_once
		end

	report_key_too_early (a_key: MODIFIED_KEY what: STRING)
			-- Report press of invalid first key `a_key', which
			-- must follow `what'
		require
			a_key_not_void: a_key /= Void
			what_not_void: what /= Void
		do
			report_status ("You must type " + what + " before " + a_key.quoted_description)
			gui.ring_bell_once
		end

	report_pack_mode_only_key (an_operation: STRING a_key: MODIFIED_KEY)
			-- Report press of invalid first key `a_key',
			-- which invokes `an_operation' that is valid
			-- only in Pack mode but we are currently in
			-- Cut mode
		require
			a_key_not_void: a_key /= Void
		do
			report_status ("Operation " + an_operation + " invoked by key " + a_key.quoted_description + " is only allowed in Pack mode")
			gui.ring_bell_once
		end

	report_cut_mode_only_key (an_operation: STRING a_key: MODIFIED_KEY)
			-- Report press of invalid first key `a_key',
			-- which invokes `an_operation' that is valid
			-- only in Cut mode but we are currently in
			-- Pack mode
		require
			an_operation_not_void: an_operation /= Void
			a_key_not_void: a_key /= Void
		do
			report_status ("Operation " + an_operation + " invoked by key " + a_key.quoted_description + " is only allowed in Cut mode")
			gui.ring_bell_once
		end

	report_no_operation_to_cancel (a_key: MODIFIED_KEY)
			-- Report press of cancellation key `a_key',
			-- when there is no operation in progress to
			-- cancel
		require
			a_key_not_void: a_key /= Void
		do
			report_status ("No operation in progress to cancel via " + a_key.description)
			gui.ring_bell_once
		end

	report_ignored_button_press (a_button: MODIFIED_POINTER_BUTTON what: STRING)
			-- Report press of `a_button' that was ignored
			-- while waiting for `what'
		require
			a_button_not_void: a_button /= Void
			what_not_void: what /= Void
		do
			report_status ("Press of " + a_button.unmodified_description + " ignored while waiting for " + what)
			gui.ring_bell_once
		end

	report_ignored_button_release (a_button: MODIFIED_POINTER_BUTTON what: STRING)
			-- Report release of `a_button' that was ignored
			-- while waiting for `what'
		require
			a_button_not_void: a_button /= Void
		local
			s: STRING		
		do
			create s.make (60)
			s.append ("Release of " + a_button.unmodified_description + " ignored")
			if what /= Void then
				s.append (" while waiting for " + what)
			end
			report_status (s)
			gui.ring_bell_once
		end

	report_read_only_button_error (a_button: MODIFIED_POINTER_BUTTON op: STRING)
			-- Report press of button `a_button' that is not
			-- valid in read-only mode
		require
			a_button_not_void: a_button /= Void
			op_not_void: op /= Void
		do
			report_status ("Press of " + a_button.description + " ignored - " + op + " not allowed in read-only mode")
			gui.ring_bell_once
		end

	report_cut_mode_button_error (a_button: MODIFIED_POINTER_BUTTON op: STRING)
			-- Report press of button `a_button' that is not
			-- valid in Cut mode
		require
			a_button_not_void: a_button /= Void
			op_not_void: op /= Void
		do
			report_status ("Press of " + a_button.description + " ignored - " + op + " not allowed in Cut mode")
			gui.ring_bell_once
		end

	report_unknown_button_press (a_button: MODIFIED_POINTER_BUTTON)
			-- Report press of unknown button `a_button'
		require
			a_button_not_void: a_button /= Void
		do
			report_status ("Press of " + a_button.description + " ignored")
			gui.ring_bell_once
		end

	report_operation_cancellation (what: STRING)
			-- Report cancellation of `what'
		require
			what_not_void: what /= Void
		do
			report_cancellation ("Cancelled " + what)
		end

	name_for_fix_operation: STRING
			-- Current name for "fix" operation
		do
			if is_fix_mapped_to_lock then
				Result := "lock"
			else
				Result := "fix"
			end
		end

	capitalized_name_for_fix_operation: STRING
			-- Current name for "fix" operation, with first
			-- character in upper case
		do
			create Result.make (4)
			su.append_capitalized (Result, name_for_fix_operation)
		end

	set_cached_bindings
			-- Set `cached_bindings' with key and pointer button
			-- bindings
		do
			create {ARRAYED_LIST [MODIFIED_BINDING]} cached_bindings.make (200)
			add_key (unmod_key (Key_r), Category_rotate, "Toggle rotation of selected objects")
			add_key (shift_key (Key_r), Category_rotate, "Rotate selected objects 90 degrees clockwise")
			
			add_keys (unmod_key (Key_f), unmod_key (Key_x), Category_lock, "Lock selected objects in X (Fix in X if in Fixed mode)")
			add_keys (unmod_key (Key_f), unmod_key (Key_y), Category_lock, "Lock selected objects in Y (Fix in Y if in Fixed mode)")
			add_keys (unmod_key (Key_u), unmod_key (Key_x), Category_lock, "Unlock selected objects in X (Unfix in X if in Fixed Mode)")
			add_keys (unmod_key (Key_u), unmod_key (Key_y), Category_lock, "Unlock selected objects in Y (Unfix in Y if in Fixed Mode)")
			
			add_unbound (Category_lock, "Switch to Fixed mode")
			add_unbound (Category_lock, "Switch to Locked mode")
			
			add_unbound (Category_lock, "Lock orientation of all objects")
			add_unbound (Category_lock, "Lock orientation of selected objects")
			add_unbound (Category_lock, "Lock orientation of unselected objects")
			add_unbound (Category_lock, "Lock to prevent duplication all objects")
			add_unbound (Category_lock, "Lock to prevent duplication selected objects")
			add_unbound (Category_lock, "Lock to prevent duplication unselected objects")
			add_unbound (Category_lock, "Lock to prevent deletion all objects")
			add_unbound (Category_lock, "Lock to prevent deletion selected objects")
			add_unbound (Category_lock, "Lock to prevent deletion unselected objects")
			add_unbound (Category_lock, "Lock almost completely all objects (doesn't affect duplication)")
			add_unbound (Category_lock, "Lock almost completely selected objects (doesn't affect duplication)")
			add_unbound (Category_lock, "Lock almost completely unselected objects (doesn't affect duplication)")
			add_unbound (Category_lock, "Lock completely all objects")
			add_unbound (Category_lock, "Lock completely selected objects")
			add_unbound (Category_lock, "Lock completely unselected objects")

			add_unbound (Category_lock, "Unlock orientation of all objects")
			add_unbound (Category_lock, "Unlock orientation of selected objects")
			add_unbound (Category_lock, "Unlock orientation of unselected objects")
			add_unbound (Category_lock, "Unlock to allow duplication all objects")
			add_unbound (Category_lock, "Unlock to allow duplication selected objects")
			add_unbound (Category_lock, "Unlock to allow duplication unselected objects")
			add_unbound (Category_lock, "Unlock to allow deletion all objects")
			add_unbound (Category_lock, "Unlock to allow deletion selected objects")
			add_unbound (Category_lock, "Unlock to allow deletion unselected objects")
			add_unbound (Category_lock, "Unlock almost completely all objects (doesn't affect duplication)")
			add_unbound (Category_lock, "Unlock almost completely selected objects (doesn't affect duplication)")
			add_unbound (Category_lock, "Unlock almost completely unselected objects (doesn't affect duplication)")
			add_unbound (Category_lock, "Unlock completely all objects")
			add_unbound (Category_lock, "Unlock completely selected objects")
			add_unbound (Category_lock, "Unlock completely unselected objects")
			
			add_key (unmod_key (Key_equal), Category_navigate, "Show all objects")
			add_button (unmod_button (3), Category_navigate, "Zoom in to area defined by box")
			add_button (ctrl_button (3), Category_navigate, "Zoom out to fit displayed area into a box")
			add_button (shift_button (3), Category_navigate, "Pan (move displayed position of objects)")
			add_key (ctrl_key (Key_n), Category_navigate, "Show next area in displayed area history list")
			add_key (ctrl_key (Key_p), Category_navigate, "Show previous area in displayed area history list")
			add_key (shift_key (Key_1), Category_navigate, "Show field 1")
			add_key (shift_key (Key_2), Category_navigate, "Show field 2 (if more than one field)")
			add_key (shift_key (Key_3), Category_navigate, "Show field 3 (if more than two fields)")
			add_key (unmod_key (Key_1), Category_navigate, "Show saved area 1")
			add_key (unmod_key (Key_2), Category_navigate, "Show saved area 2")
			add_key (unmod_key (Key_3), Category_navigate, "Show saved area 3")
			add_key (unmod_key (Key_4), Category_navigate, "Show saved area 4")
			add_key (ctrl_key (Key_1), Category_navigate, "Set saved area 1 to currently displayed area")
			add_key (ctrl_key (Key_2), Category_navigate, "Set saved area 2 to currently displayed area")
			add_key (ctrl_key (Key_3), Category_navigate, "Set saved area 3 to currently displayed area")
			add_key (ctrl_key (Key_4), Category_navigate, "Set saved area 4 to currently displayed area")
			
			add_button (unmod_button (1), Category_select, "Select objects containing mouse pointer (click) or inside box (drag)")
			add_button (ctrl_button (1), Category_select, "Add to selection objects containing mouse pointer (click) or inside box (drag)")
			add_button (shift_button (1), Category_select, "Remove from selection objects containing mouse pointer (click) or inside box (drag)")
			add_button (ctrl_alt_shift_button (1), Category_select, "Select objects that intersect drawn box")
			add_button (ctrl_alt_button (1), Category_select, "Add to selection objects that intersect drawn box")
			add_button (alt_shift_button (1), Category_select, "Remove from selection objects that intersect drawn box")
			
			add_keys (ctrl_key (Key_s), unmod_key (Key_a), Category_select, "Select all objects")
			add_keys (ctrl_key (Key_s), unmod_key (Key_z), Category_select, "Select no objects (unselect all objects)")
			add_keys (ctrl_key (Key_s), unmod_key (Key_i), Category_select, "Select objects that are not selected (invert selection)")
			add_keys (ctrl_key (Key_s), unmod_key (Key_c), Category_select, "Select all customer projects")
			add_keys (ctrl_key (Key_s), unmod_key (Key_t), Category_select, "Select all test vehicles")
			add_keys (ctrl_key (Key_s), unmod_key (Key_b), Category_select, "Select all projects (both customer projects and test vehicles)")
			add_keys (ctrl_key (Key_s), unmod_key (Key_g), Category_select, "Select all vendor geometries/ghosts")
			add_keys (ctrl_key (Key_s), unmod_key (Key_l), Category_select, "Select all fillers")
			add_keys (ctrl_key (Key_s), unmod_key (Key_s), Category_select, "Select all scribes")
			add_keys (ctrl_key (Key_s), unmod_key (Key_h), Category_select, "Select all horizontal scribes")
			add_keys (ctrl_key (Key_s), unmod_key (Key_u), Category_select, "Select all cuts")
			add_keys (ctrl_key (Key_s), unmod_key (Key_v), Category_select, "Select all vertical scribes")
			add_keys (ctrl_key (Key_s), unmod_key (Key_f), Category_select, "Select all floating scribes")
			add_keys (ctrl_key (Key_s), unmod_key (Key_n), Category_select, "Select all non-scribes (ignores cuts)")
			add_keys (ctrl_key (Key_s), shift_key (Key_c), Category_select, "Select all Commercial projects")
			add_keys (ctrl_key (Key_s), shift_key (Key_m), Category_select, "Select all MEP projects")
			add_keys (ctrl_key (Key_s), shift_key (Key_o), Category_select, "Select all Other projects (not Commercial or MEP)")
			add_keys (ctrl_key (Key_s), unmod_key (Key_p), Category_select, "Select all partially selected objects")
			add_keys (ctrl_key (Key_s), unmod_key (Key_m), Category_select, "Select all objects containing mouse pointer")
			add_keys (ctrl_key (Key_s), unmod_key (Key_e), Category_select, "Select all external objects (outside all fields)")
			add_unbound (Category_select, "Select all misaligned scribes")
			
			add_keys (ctrl_key (Key_a), unmod_key (Key_a), Category_select, "Add to selection all objects")
			add_keys (ctrl_key (Key_a), unmod_key (Key_z), Category_select, "Add to selection no objects (leave selection unchanged)")
			add_keys (ctrl_key (Key_a), unmod_key (Key_i), Category_select, "Add to selection inversion of selected objects (select all objects)")
			add_keys (ctrl_key (Key_a), unmod_key (Key_c), Category_select, "Add to selection all customer projects")
			add_keys (ctrl_key (Key_a), unmod_key (Key_t), Category_select, "Add to selection all test vehicles")
			add_keys (ctrl_key (Key_a), unmod_key (Key_b), Category_select, "Add to selection all projects (both customer projects and test vehicles)")
			add_keys (ctrl_key (Key_a), unmod_key (Key_g), Category_select, "Add to selection all vendor geometries/ghosts")
			add_keys (ctrl_key (Key_a), unmod_key (Key_l), Category_select, "Add to selection all fillers")
			add_keys (ctrl_key (Key_a), unmod_key (Key_s), Category_select, "Add to selection all scribes")
			add_keys (ctrl_key (Key_a), unmod_key (Key_h), Category_select, "Add to selection all horizontal scribes")
			add_keys (ctrl_key (Key_a), unmod_key (Key_u), Category_select, "Add to selection all cuts")
			add_keys (ctrl_key (Key_a), unmod_key (Key_v), Category_select, "Add to selection all vertical scribes")
			add_keys (ctrl_key (Key_a), unmod_key (Key_f), Category_select, "Add to selection all floating scribes")
			add_keys (ctrl_key (Key_a), unmod_key (Key_n), Category_select, "Add to selection all non-scribes (ignores cuts)")
			add_keys (ctrl_key (Key_a), shift_key (Key_c), Category_select, "Add to selection all Commercial projects")
			add_keys (ctrl_key (Key_a), shift_key (Key_m), Category_select, "Add to selection all MEP projects")
			add_keys (ctrl_key (Key_a), shift_key (Key_o), Category_select, "Add to selection all Other projects (not Commercial or MEP)")
			add_keys (ctrl_key (Key_a), unmod_key (Key_p), Category_select, "Add to selection all partially selected objects")
			add_keys (ctrl_key (Key_a), unmod_key (Key_m), Category_select, "Add to selection all objects containing mouse pointer")
			add_keys (ctrl_key (Key_a), unmod_key (Key_e), Category_select, "Add to selection all external objects (outside all fields)")
			add_unbound (Category_select, "Add to selection misaligned scribes")
			
			add_keys (ctrl_key (Key_r), unmod_key (Key_a), Category_select, "Remove from selection all objects")
			add_keys (ctrl_key (Key_r), unmod_key (Key_z), Category_select, "Remove from selection no objects (leave selection unchanged)")
			add_keys (ctrl_key (Key_r), unmod_key (Key_i), Category_select, "Remove from selection inversion of selected objects (leave selection unchanged)")
			add_keys (ctrl_key (Key_r), unmod_key (Key_c), Category_select, "Remove from selection all customer projects")
			add_keys (ctrl_key (Key_r), unmod_key (Key_t), Category_select, "Remove from selection all test vehicles")
			add_keys (ctrl_key (Key_r), unmod_key (Key_b), Category_select, "Remove from selection all projects (both customer projects and test vehicles)")
			add_keys (ctrl_key (Key_r), unmod_key (Key_g), Category_select, "Remove from selection all vendor geometries/ghosts")
			add_keys (ctrl_key (Key_r), unmod_key (Key_l), Category_select, "Remove from selection all fillers")
			add_keys (ctrl_key (Key_r), unmod_key (Key_s), Category_select, "Remove from selection all scribes")
			add_keys (ctrl_key (Key_r), unmod_key (Key_h), Category_select, "Remove from selection all horizontal scribes")
			add_keys (ctrl_key (Key_r), unmod_key (Key_u), Category_select, "Remove from selection all cuts")
			add_keys (ctrl_key (Key_r), unmod_key (Key_v), Category_select, "Remove from selection all vertical scribes")
			add_keys (ctrl_key (Key_r), unmod_key (Key_f), Category_select, "Remove from selection all floating scribes")
			add_keys (ctrl_key (Key_r), unmod_key (Key_n), Category_select, "Remove from selection all non-scribes (ignores cuts)")
			add_keys (ctrl_key (Key_r), shift_key (Key_c), Category_select, "Remove from selection all Commercial projects")
			add_keys (ctrl_key (Key_r), shift_key (Key_m), Category_select, "Remove from selection all MEP projects")
			add_keys (ctrl_key (Key_r), shift_key (Key_o), Category_select, "Remove from selection all Other projects (not Commercial or MEP)")
			add_keys (ctrl_key (Key_r), unmod_key (Key_p), Category_select, "Remove from selection all partially selected objects")
			add_keys (ctrl_key (Key_r), unmod_key (Key_m), Category_select, "Remove from selection all objects containing mouse pointer")
			add_keys (ctrl_key (Key_r), unmod_key (Key_e), Category_select, "Remove from selection all external objects (outside all fields)")
			add_unbound (Category_select, "Remove from selection misaligned scribes")
			
			add_button (ctrl_button (2), Category_duplicate, "Duplicate selected objects and place where button is released")
			
			add_button (shift_button (2), Category_create, "Create scribe defined by box")
			add_keys (unmod_key (Key_p), unmod_key (Key_d), Category_create, "Create filler")
			
			add_key (unmod_key (Key_k), Category_delete, "Delete selected objects")
			add_key (shift_key (Key_k), Category_delete, "Delete selected scribes")
			
			add_button (unmod_button (2), Category_move, "Move selected objects")
			add_keys (unmod_key (Key_s), unmod_key (Key_s), Category_move, "Spread selected objects horizontally by decreasing size (width)")
			add_keys (unmod_key (Key_s), unmod_key (Key_w), Category_move, "Spread selected objects horizontally by decreasing width")
			add_keys (unmod_key (Key_s), unmod_key (Key_h), Category_move, "Spread selected objects horizontally by decreasing height")
			add_keys (unmod_key (Key_s), unmod_key (Key_n), Category_move, "Spread selected objects horizontally by increasing name (design number)")
			add_keys (unmod_key (Key_s), unmod_key (Key_x), Category_move, "Spread selected objects horizontally by increasing X position")
			add_keys (unmod_key (Key_s), unmod_key (Key_y), Category_move, "Spread selected objects horizontally by increasing Y position")
			add_keys (unmod_key (Key_s), shift_key (Key_s), Category_move, "Spread selected objects horizontally by increasing size (width)")
			add_keys (unmod_key (Key_s), shift_key (Key_w), Category_move, "Spread selected objects horizontally by increasing width")
			add_keys (unmod_key (Key_s), shift_key (Key_h), Category_move, "Spread selected objects horizontally by increasing height")
			add_keys (unmod_key (Key_s), shift_key (Key_n), Category_move, "Spread selected objects horizontally by decreasing name (design number)")
			add_keys (unmod_key (Key_s), shift_key (Key_x), Category_move, "Spread selected objects horizontally by decreasing X position")
			add_keys (unmod_key (Key_s), shift_key (Key_y), Category_move, "Spread selected objects horizontally by decreasing Y position")
			add_keys (shift_key (Key_s), unmod_key (Key_s), Category_move, "Spread selected objects vertically by decreasing size (height)")
			add_keys (shift_key (Key_s), unmod_key (Key_w), Category_move, "Spread selected objects vertically by decreasing width")
			add_keys (shift_key (Key_s), unmod_key (Key_h), Category_move, "Spread selected objects vertically by decreasing height")
			add_keys (shift_key (Key_s), unmod_key (Key_n), Category_move, "Spread selected objects vertically by increasing name")
			add_keys (shift_key (Key_s), unmod_key (Key_x), Category_move, "Spread selected objects vertically by increasing X position")
			add_keys (shift_key (Key_s), unmod_key (Key_y), Category_move, "Spread selected objects vertically by increasing Y position")
			add_keys (shift_key (Key_s), shift_key (Key_s), Category_move, "Spread selected objects vertically by increasing size (height)")
			add_keys (shift_key (Key_s), shift_key (Key_w), Category_move, "Spread selected objects vertically by increasing width")
			add_keys (shift_key (Key_s), shift_key (Key_h), Category_move, "Spread selected objects vertically by increasing height")
			add_keys (shift_key (Key_s), shift_key (Key_n), Category_move, "Spread selected objects vertically by decreasing name")
			add_keys (shift_key (Key_s), shift_key (Key_x), Category_move, "Spread selected objects vertically by decreasing X position")
			add_keys (shift_key (Key_s), shift_key (Key_y), Category_move, "Spread selected objects vertically by decreasing Y position")
			add_key (unmod_key (Key_a), Category_move, "Abut selected object to object containing pointer (or field edge if outside fields)")
			add_keys (shift_key (Key_a), unmod_key (Key_l), Category_move, "Align left side of selected objects to left side of object containing pointer")
			add_keys (shift_key (Key_a), unmod_key (Key_r), Category_move, "Align right side selected objects to right side of object containing pointer")
			add_keys (shift_key (Key_a), unmod_key (Key_t), Category_move, "Align top side selected objects to top side of object containing pointer")
			add_keys (shift_key (Key_a), unmod_key (Key_b), Category_move, "Align bottom side selected objects to bottom side of object containing pointer")
			add_keys (shift_key (Key_a), unmod_key (Key_x), Category_move, "Align center in X of selected objects to center in X of object containing pointer")
			add_keys (shift_key (Key_a), unmod_key (Key_y), Category_move, "Align center in Y of selected objects to center in Y of object containing pointer")
			add_key (unmod_key (Key_h), Category_move, "Horizontal scan of selected objects")
			add_key (unmod_key (Key_v), Category_move, "Vertical scan of selected objects")
			add_keys (shift_key (Key_m), unmod_key (Key_a), Category_move, "Move selected object to an absolute location from center of a specified field")
			add_keys (shift_key (Key_m), unmod_key (Key_r), Category_move, "Move selected objects by a distance relative to their current location")
			add_keys (unmod_key (Key_c), unmod_key (Key_a), Category_move, "Center all projects in their cut die (ignores unisolated projects)")
			add_keys (unmod_key (Key_c), unmod_key (Key_s), Category_move, "Center selected projects in their cut die (ignores unisolated projects)")
			
			add_keys (unmod_key (Key_z), unmod_key (Key_z), Category_resize, "Resize scribe (moves end nearest mouse pointer)")
			add_key (unmod_key (Key_j), Category_resize, "Join scribes (trim, separate, extend, unoverlap)")
			add_keys (ctrl_key (Key_t), unmod_key (Key_a), Category_resize, "Trim all scribes at field of view edge")
			add_keys (ctrl_key (Key_t), unmod_key (Key_s), Category_resize, "Trim selected scribes at field of view edge")
			add_keys (ctrl_key (Key_b), unmod_key (Key_a), Category_resize, "Separate (break) all scribes that span fields into one scribe per field")
			add_keys (ctrl_key (Key_b), unmod_key (Key_s), Category_resize, "Separate (break) selected scribes that span fields into one scribe per field")
			add_keys (ctrl_key (Key_e), unmod_key (Key_a), Category_resize, "Extend all scribes to next scribe of opposite orientation")
			add_keys (ctrl_key (Key_e), unmod_key (Key_s), Category_resize, "Extend selected scribes to next scribe of opposite orientation")
			add_keys (ctrl_key (Key_u), unmod_key (Key_a), Category_resize, "Unoverlap all scribes")
			add_keys (ctrl_key (Key_u), unmod_key (Key_s), Category_resize, "Unoverlap selected scribes")
			
			add_keys (ctrl_key (Key_v), unmod_key (Key_s), Category_view, "Switch to Standard view")
			add_keys (ctrl_key (Key_v), unmod_key (Key_a), Category_view, "Switch to Account Type view")
			add_keys (ctrl_key (Key_v), unmod_key (Key_b), Category_view, "Switch to Die Labeling Style view")
			add_keys (ctrl_key (Key_v), unmod_key (Key_m), Category_view, "Switch to Max Die Size view")
			add_keys (ctrl_key (Key_v), unmod_key (Key_l), Category_view, "Switch to Lot Split view")
			add_keys (ctrl_key (Key_v), unmod_key (Key_p), Category_view, "Switch to Packaging view")
			add_keys (ctrl_key (Key_v), unmod_key (Key_c), Category_view, "Switch to Account Number view")
			add_keys (ctrl_key (Key_v), unmod_key (Key_n), Category_view, "Switch to Design Name view")
			add_keys (ctrl_key (Key_v), unmod_key (Key_i), Category_view, "Switch to Die ID view")
			add_keys (ctrl_key (Key_v), unmod_key (Key_o), Category_view, "Switch to Organization Short ID view")
			add_keys (ctrl_key (Key_v), unmod_key (Key_r), Category_view, "Switch to Rotation Locked view")
			add_keys (ctrl_key (Key_v), unmod_key (Key_u), Category_view, "Switch to Duplication Locked view")
			add_keys (ctrl_key (Key_v), unmod_key (Key_d), Category_view, "Switch to Deletion Locked view")
			add_keys (ctrl_key (Key_v), unmod_key (Key_f), Category_view, "Switch to Search Results view")
			add_keys (ctrl_key (Key_v), unmod_key (Key_h), Category_view, "Switch to Highlighted Selection view")
			add_keys (ctrl_key (Key_v), unmod_key (Key_v), Category_view, "Switch to Validity Check view")
			add_keys (ctrl_key (Key_v), unmod_key (Key_t), Category_view, "Switch to Cut view")
			
			add_key (unmod_key (Key_l), Category_check, "Check all validity constraints")
			add_keys (ctrl_key (Key_c), unmod_key (Key_a), Category_check, "Check all validity constraints")
			add_keys (ctrl_key (Key_c), shift_key (Key_a), Category_check, "Check almost all validity constraints (skips objects outside fields)")
			add_keys (ctrl_key (Key_c), unmod_key (Key_b), Category_check, "Check validity constraints before Join Scribes (skips some scribe checks)")
			add_keys (ctrl_key (Key_c), unmod_key (Key_e), Category_check, "Check for external objects (outside all fields)")
			add_keys (ctrl_key (Key_c), shift_key (Key_o), Category_check, "Check for overlapping field edge objects")
			add_keys (ctrl_key (Key_c), unmod_key (Key_o), Category_check, "Check for overlaps of another object")
			add_keys (ctrl_key (Key_c), unmod_key (Key_t), Category_check, "Check for below minimum size objects (t is for Tiny)")
			add_keys (ctrl_key (Key_c), unmod_key (Key_l), Category_check, "Check for lot not assigned projects (in Pack mode)")
			add_keys (ctrl_key (Key_c), unmod_key (Key_u), Category_check, "Check for unlocked or not assigned to run projects")
			add_keys (ctrl_key (Key_c), shift_key (Key_G), Category_check, "Check for too close to field edge objects (g is for Gap)")
			add_keys (ctrl_key (Key_c), unmod_key (Key_g), Category_check, "Check for too close to another object (g is for Gap)")
			add_keys (ctrl_key (Key_c), unmod_key (Key_s), Category_check, "Check for cut die size too big projects")
			add_keys (ctrl_key (Key_c), unmod_key (Key_i), Category_check, "Check for not isolated in cut die projects")
			add_keys (ctrl_key (Key_c), unmod_key (Key_c), Category_check, "Check for not centered in cut die projects (in Pack mode)")
			add_keys (ctrl_key (Key_c), unmod_key (Key_f), Category_check, "Check for floating scribes")
			add_keys (ctrl_key (Key_c), shift_key (Key_t), Category_check, "Check for small scribes and fillers (t is for Tiny)")
			add_keys (ctrl_key (Key_c), unmod_key (Key_v), Category_check, "Check for test vehicle issues")
			add_keys (ctrl_key (Key_c), unmod_key (Key_w), Category_check, "Check for custom width scribes")
			add_keys (ctrl_key (Key_c), unmod_key (Key_m), Category_check, "Check for misaligned scribes")
			add_keys (ctrl_key (Key_c), unmod_key (Key_c), Category_check, "Check for isolated sites with invalid cut die (in Cut mode)")
			add_keys (ctrl_key (Key_c), unmod_key (Key_l), Category_check, "Check for isolated sites with wrong lot (in Cut mode)")
			add_keys (ctrl_key (Key_c), unmod_key (Key_n), Category_check, "Check for isolated sites with non-default cut die size (in Cut mode)")
			add_keys (ctrl_key (Key_c), unmod_key (Key_o), Category_check, "Check for isolated sites that share cut die with others (in Cut mode)")
			add_keys (ctrl_key (Key_c), unmod_key (Key_e), Category_check, "Check for extra cuts that do not help isolate any isolated sites(in Cut mode)")
			
			add_key (ctrl_key (Key_f), Category_search, "Find design numbers and switch to Search Results view")
			
			add_key (unmod_key (Key_t), Category_display, "Display Die Info window (with information on selected objects)")
			add_key (unmod_key (Key_w), Category_display, "Display Scribe Width window (for viewing/setting width of newly created scribes)")
			add_keys (unmod_key (Key_d), unmod_key (Key_d), Category_display, "Display Die Info window (with information on selected objects)")
			add_keys (unmod_key (Key_d), unmod_key (Key_l), Category_display, "Display Lot Assignment window (for viewing lot assignments)")
			add_keys (unmod_key (Key_d), unmod_key (Key_o), Category_display, "Display Output window")
			add_keys (unmod_key (Key_d), unmod_key (Key_a), Category_display, "Display Application Color window (for viewing/setting application colors)")
			add_keys (unmod_key (Key_d), unmod_key (Key_c), Category_display, "Display Color window (for viewing/setting lot split colors)")
			add_keys (unmod_key (Key_d), unmod_key (Key_v), Category_display, "Display Version Selection window (for selecting version of packing to edit)")
			add_keys (unmod_key (Key_d), unmod_key (Key_p), Category_display, "Display Project Management window (to add, remove, update or lock projects)")
			add_keys (unmod_key (Key_d), unmod_key (Key_w), Category_display, "Display Scribe Width window (for viewing/setting width of newly created scribes)")
			add_keys (unmod_key (Key_d), unmod_key (Key_s), Category_display, "Display Payload and Kerf Sizes window (for viewing/setting size of payload and external kerf)")
			
			add_key (shift_key (Key_l), Category_lot, "Assign lot to selected projects")
			
			add_key (unmod_key (Key_escape), Category_abort, "Abort partially specified operation (before second key typed or button released)")
			
			add_key (unmod_key (Key_e), Category_exit, "Exit packer and finish step")
			add_key (unmod_key (Key_q), Category_exit, "Quit packer without finishing step")
			
			add_key (unmod_key (Key_g), Category_save, "Save a new version of packing")
			
			add_unbound (Category_refresh, "Refresh drawing area from pixmap (clears measurement markers)")
			add_unbound (Category_refresh, "Redraw displayed area onto pixmap and copy pixmap to drawing area (for confirming display bugs only)")
			
			add_key (unmod_key (Key_delete), Category_undo, "Undo last change (toggle)")
			add_key (ctrl_key (Key_z), Category_undo, "Undo one change")
			add_key (ctrl_key (Key_y), Category_undo, "Redo one change")
			
			add_key (unmod_key (Key_a), Category_cut, "Add a new %"under construction%" cut pattern")
			add_key (unmod_key (Key_b), Category_cut, "Put a cut through every scribe in current cut pattern")
			add_key (unmod_key (Key_c), Category_cut, "Add a cut through scribe or field edge containing mouse pointer")
			add_key (unmod_key (Key_i), Category_cut, "Toggle isolation of selected projects")
			add_key (unmod_key (Key_k), Category_cut, "Delete selected objects that are cuts")
			add_key (unmod_key (Key_n), Category_cut, "Switch cut pattern to next field in sequence")
			add_key (unmod_key (Key_p), Category_cut, "Change properties of current cut pattern")
			add_key (unmod_key (Key_r), Category_cut, "Remove current cut pattern")
			add_key (unmod_key (Key_s), Category_cut, "Save completed cut pattern")
			add_key (unmod_key (Key_left), Category_cut, "Show previous cut pattern")
			add_key (unmod_key (Key_right), Category_cut, "Show next cut pattern")
			add_key (shift_key (Key_c), Category_cut, "Switch to Cut mode (switch back to Pack mode if in Cut mode)")
			add_key (shift_key (Key_i), Category_cut, "Compute default set of isolated projects for current cut pattern")
			add_key (shift_key (Key_k), Category_cut, "Delete cuts that intersect the cut die of any selected project")
			add_key (shift_key (Key_m), Category_cut, "Modify current completed cut pattern")
			

			add_key (ctrl_key (Key_h), Category_help, "Show help window with key and pointer button bindings")
			
			add_keys (unmod_key (Key_m), unmod_key (Key_x), Category_measure, "Measure sum of widths of selected objects")
			add_keys (unmod_key (Key_m), unmod_key (Key_y), Category_measure, "Measure sum of heights of selected objects")
			add_button (ctrl_shift_button (3), Category_measure, "Measure distance from previous point to point where button is clicked")
			
			last_binding_number := 0
			cached_bindings.do_all (agent assign_binding_number)
		end

	assign_binding_number (a_binding: MODIFIED_BINDING)
			-- Assign a binding number to `a_binding'
		require
			a_binding_not_void: a_binding /= Void
		do
			last_binding_number := last_binding_number + 1
			a_binding.set_number (last_binding_number)
		end

	add_key (a_key: MODIFIED_KEY a_category, an_operation: STRING)
			-- Add a binding of `a_key' to `an_operation' in
			-- `a_category'
		require
			a_key_not_void: a_key /= Void
			a_category_not_void: a_category /= Void
			an_operation_not_void: an_operation /= Void
		do
			cached_bindings.extend (create {MODIFIED_BINDING}.make_with_key (a_key, a_category, an_operation))
		end

	add_keys (a_first_key, a_second_key: MODIFIED_KEY a_category, an_operation: STRING)
			-- Add a binding of `a_first_key' followed by
			-- `a_second_key' to `an_operation' in
			-- `a_category'
		require
			a_first_key_not_void: a_first_key /= Void
			a_second_key_not_void: a_second_key /= Void
			a_category_not_void: a_category /= Void
			an_operation_not_void: an_operation /= Void
		do
			cached_bindings.extend (create {MODIFIED_BINDING}.make_with_keys (a_first_key, a_second_key, a_category, an_operation))
		end

	add_unbound (a_category, an_operation: STRING)
			-- Add unbound operation `an_operation' in `a_category'
		require
			a_category_not_void: a_category /= Void
			an_operation_not_void: an_operation /= Void
		do
			cached_bindings.extend (create {MODIFIED_BINDING}.make_unbound (a_category, an_operation))
		end

	add_button (a_button: MODIFIED_POINTER_BUTTON a_category, an_operation: STRING)
			-- Add a binding of `a_button' to `an_operationi'
			-- in `a_category'
		require
			a_button_not_void: a_button /= Void
			a_category_not_void: a_category /= Void
			an_operation_not_void: an_operation /= Void
		do
			cached_bindings.extend (create {MODIFIED_BINDING}.make_with_button (a_button, a_category, an_operation))
		end

	unmod_key (a_code: INTEGER): MODIFIED_KEY
			-- Key with `a_code' and no modifiers
		require
			valid_key_code: valid_key_code (a_code)
		do
			create Result.make_unmodified (a_code)
		ensure
			result_not_void: Result /= Void
		end

	ctrl_key (a_code: INTEGER): MODIFIED_KEY
			-- Key with `a_code' and Ctrl modifier
		require
			valid_key_code: valid_key_code (a_code)
		do
			create Result.make_control (a_code)
		ensure
			result_not_void: Result /= Void
		end

	alt_key (a_code: INTEGER): MODIFIED_KEY
			-- Key with `a_code' and Alt modifier
		require
			valid_key_code: valid_key_code (a_code)
		do
			create Result.make_alt (a_code)
		ensure
			result_not_void: Result /= Void
		end

	shift_key (a_code: INTEGER): MODIFIED_KEY
			-- Key with `a_code' and Shift modifier
		require
			valid_key_code: valid_key_code (a_code)
		do
			create Result.make_shift (a_code)
		ensure
			result_not_void: Result /= Void
		end

	unmod_button (a_button_number: INTEGER): MODIFIED_POINTER_BUTTON
			-- Button with `a_button_number' and no modifiers
		do
			create Result.make_unmodified (a_button_number)
		ensure
			result_not_void: Result /= Void
		end

	ctrl_button (a_button_number: INTEGER): MODIFIED_POINTER_BUTTON
			-- Button with `a_button_number' and Ctrl modifier
		do
			create Result.make_control (a_button_number)
		ensure
			result_not_void: Result /= Void
		end

	alt_button (a_button_number: INTEGER): MODIFIED_POINTER_BUTTON
			-- Button with `a_button_number' and Alt modifier
		do
			create Result.make_alt (a_button_number)
		ensure
			result_not_void: Result /= Void
		end

	shift_button (a_button_number: INTEGER): MODIFIED_POINTER_BUTTON
			-- Button with `a_button_number' and Shift modifier
		do
			create Result.make_shift (a_button_number)
		ensure
			result_not_void: Result /= Void
		end

	ctrl_shift_button (a_button_number: INTEGER): MODIFIED_POINTER_BUTTON
			-- Button with `a_button_number' and Control and
			-- Shift modifiers
		do
			create Result.make_control_shift (a_button_number)
		ensure
			result_not_void: Result /= Void
		end

	ctrl_alt_button (a_button_number: INTEGER): MODIFIED_POINTER_BUTTON
			-- Button with `a_button_number' and Control and
			-- Alt modifiers
		do
			create Result.make_control_alt (a_button_number)
		ensure
			result_not_void: Result /= Void
		end

	alt_shift_button (a_button_number: INTEGER): MODIFIED_POINTER_BUTTON
			-- Button with `a_button_number' and Alt and
			-- shift modifiers
		do
			create Result.make_alt_shift (a_button_number)
		ensure
			result_not_void: Result /= Void
		end

	ctrl_alt_shift_button (a_button_number: INTEGER): MODIFIED_POINTER_BUTTON
			-- Button with `a_button_number' and Ctrl, Alt and
			-- shift modifiers
		do
			create Result.make_control_alt_shift (a_button_number)
		ensure
			result_not_void: Result /= Void
		end

	Category_rotate: STRING = "ROTATE"

	Category_lock: STRING = "LOCK"

	Category_navigate: STRING = "NAVIGATE"

	Category_select: STRING = "SELECT"

	Category_create: STRING = "CREATE"

	Category_delete: STRING = "DELETE"

	Category_duplicate: STRING = "DUPLICATE"

	Category_move: STRING = "MOVE"

	Category_resize: STRING = "RESIZE"

	Category_view: STRING = "VIEW"

	Category_check: STRING = "CHECK"

	Category_search: STRING = "SEARCH"

	Category_display: STRING = "DISPLAY"

	Category_refresh: STRING = "REFRESH"

	Category_lot: STRING = "LOT"

	Category_save: STRING = "SAVE"

	Category_abort: STRING = "ABORT"

	Category_exit: STRING = "EXIT"

	Category_undo: STRING = "UNDO"

	Category_help: STRING = "HELP"

	Category_cut: STRING = "CUT"

	Category_measure: STRING = "MEASURE"

	state: INTEGER
			-- State of simple finite state machine

	Ready: INTEGER = 1
			-- Code for ready state
	
	Waiting_for_button_release: INTEGER = 2
			-- Code for waiting for button release state
	
	operation: INTEGER
			-- Operation currently in progress

	No_operation: INTEGER = 0
			-- Code for no operation
	
	Set_selection: INTEGER = 1
			-- Code for set selection operation
	
	Add_to_selection: INTEGER = 2
			-- Code for add to selection operation
	
	Remove_from_selection: INTEGER = 3
			-- Code for remove from selection operation
	
	Move_selected: INTEGER = 4
			-- Code for move operation
	
	Duplicate_selected: INTEGER = 5
			-- Code for duplicate operation
	
	Create_scribe: INTEGER = 6
			-- Code for create scribe operation
	
	Zoom_in: INTEGER = 7
			-- Code for zoom in operation
	
	Zoom_out: INTEGER = 8
			-- Code for zoom out operation
	
	Pan: INTEGER = 9
			-- Code for pan operation
	
	Create_filler: INTEGER = 10
			-- Code for create filler operation
	
	Resize_scribe: INTEGER = 11
			-- Code for resize scribe operation
	
	Measure_distance: INTEGER = 12
			-- Code for measure distance operation
	
	Set_selection_by_intersection: INTEGER = 13
			-- Code for set selection by intersection operation
	
	Add_to_selection_by_intersection: INTEGER = 14
			-- Code for add to selection by intersection operation
	
	Remove_from_selection_by_intersection: INTEGER = 15
			-- Code for remove from selection by intersection
			-- operation
	
	is_operation_in_progress: BOOLEAN
			-- Is an operation currently in progress?
		do
			Result := operation /= No_operation
		end

	is_operation_started_by_key: BOOLEAN
			-- Is an operation which was started by a key press
			-- currently in progress?
		do
			inspect operation
			when Create_filler, Resize_scribe then
				Result := True
			else
				Result := False
			end
		end

	is_operation_cancelled: BOOLEAN
			-- Was last started operation cancelled?

	is_caps_lock_on: BOOLEAN
			-- Is Caps Lock currently on?

	in_multi_key_sequence: BOOLEAN
			-- Have we seen the first key of a multi-key
			-- sequence?

	previous_key: MODIFIED_KEY
			-- Previous key typed that is part of
			-- a multi-key sequence that maps to one controller
			-- action

	last_button_pressed: MODIFIED_POINTER_BUTTON
			-- Last button pressed that caused some
			-- action (was not ignored), with modifier keys

	application: EV_APPLICATION
			-- Application

	cached_bindings: like bindings

	current_key_bindings: MODIFIED_KEY_BIND_SET
			-- Current bindings of modified keys to actions

	pack_first_key_bindings: MODIFIED_KEY_BIND_SET
			-- Bindings for first key of sequence in Pack mode

	cut_first_key_bindings: MODIFIED_KEY_BIND_SET
			-- Bindings for first key of sequence in Cut mode

	set_selection_key_bindings: MODIFIED_KEY_BIND_SET
			-- Bindings for Set Selection operation

	add_to_selection_key_bindings: MODIFIED_KEY_BIND_SET
			-- Bindings for Add to Selection operation

	remove_from_selection_key_bindings: MODIFIED_KEY_BIND_SET
			-- Bindings for Remove from Selection operation

	display_key_bindings: MODIFIED_KEY_BIND_SET
			-- Bindings for Display operation

	switch_to_view_key_bindings: MODIFIED_KEY_BIND_SET
			-- Bindings for Switch to View operation

	horizontal_spread_key_bindings: MODIFIED_KEY_BIND_SET
			-- Bindings for Horizontal Spread operation

	vertical_spread_key_bindings: MODIFIED_KEY_BIND_SET
			-- Bindings for Vertical Spread operation

	align_key_bindings: MODIFIED_KEY_BIND_SET
			-- Bindings for Align operation

	move_key_bindings: MODIFIED_KEY_BIND_SET
			-- Bindings for Move operation

	center_key_bindings: MODIFIED_KEY_BIND_SET
			-- Bindings for Center operation

	trim_key_bindings: MODIFIED_KEY_BIND_SET
			-- Bindings for Trim Scribes operation

	break_key_bindings: MODIFIED_KEY_BIND_SET
			-- Bindings for Break Scribes operation

	extend_key_bindings: MODIFIED_KEY_BIND_SET
			-- Bindings for Extend Scribes operation

	unoverlap_key_bindings: MODIFIED_KEY_BIND_SET
			-- Bindings for Unoverlap Scribes operation

	pack_check_key_bindings: MODIFIED_KEY_BIND_SET
			-- Bindings for Check operation in Pack mode

	cut_check_key_bindings: MODIFIED_KEY_BIND_SET
			-- Bindings for Check operation in Cut mode

	fix_key_bindings: MODIFIED_KEY_BIND_SET
			-- Bindings for Fix operation

	unfix_key_bindings: MODIFIED_KEY_BIND_SET
			-- Bindings for Unfix operation

	measure_key_bindings: MODIFIED_KEY_BIND_SET
			-- Bindings for Measure operation

	create_filler_key_bindings: MODIFIED_KEY_BIND_SET
			-- Bindings for Create Filler operation

	resize_scribe_key_bindings: MODIFIED_KEY_BIND_SET
			-- Bindings for Resize Scribe operation

	miscellaneous_key_bindings: MODIFIED_KEY_BIND_SET
			-- Bindings for Miscellaneous operation

	last_binding_number: INTEGER
			-- Last binding number assigned

end
