
indexing
	
	description: "Object that is an element of a model of a reticle"
	author: "David Hollenberg"
	date: "July 28, 2006"

deferred class RETICLE_MODEL_ELEMENT
inherit
	MODEL_ELEMENT
		redefine
			anchored_model, anchored_model_element,
			anchored_model_sub_element
		end
	PACKER_UTILITIES
		export
			{NONE} all
		end
	HAS_DESIGN_NAME
	HAS_DESIGN_NUMBER

feature -- Properties

	identification: STRING
			-- Identifying name of element
		deferred
		end

	identification_for_message: STRING
			-- Identifying name of element for displaying
			-- error and warning messages
		do
			Result := identification
		end

	location_description: STRING
			-- Description of location of element, for use in
			-- error and warning messages
		do
			Result := "at (" + grid.formatted_number (field_relative_center_x) + ", " + grid.formatted_number (field_relative_center_y) + ") from field " + associated_field_name + " center"
		end

	has_zero_size_in_x: BOOLEAN
			-- Does `Current' actually have zero size in X
			-- dimension?
		do
			Result := size_x = 0.0
		end
		
	has_zero_size_in_y: BOOLEAN
			-- Does `Current' actually have zero size in Y
			-- dimension?
		do
			Result := size_y = 0.0
		end
		
	size_description: STRING
			-- Description of size of element, for use in
			-- error and warning messages
		do
			Result := grid.formatted_number (size_x) + " x " + grid.formatted_number (size_y)
		end

	scribe_cut_size_description: STRING
			-- Description of cut die size of element
			-- determined by surrounding scribes, for use
			-- in error and warning messages
		do
			Result := grid.formatted_number (scribe_cut_die_size_x) + " x " + grid.formatted_number (scribe_cut_die_size_y)
		end

	cut_cut_size_description: STRING
			-- Description of cut die size of element determined
			-- by surrounding cuts, for use in
			-- error and warning messages
		do
			Result := grid.formatted_number (cut_cut_die_size_x) + " x " + grid.formatted_number (cut_cut_die_size_y)
		end

	offset_cut_center_description: STRING
			-- Description of offset of die center from cut
			-- die center, for use in error and warning message
		do
			Result := grid.formatted_number (offset_cut_center_x) + " x " + grid.formatted_number (offset_cut_center_y)
		end

	packing_name: STRING
			-- Identifying name of element for packing
			-- sort purposes
		do
			Result := identification_for_message
		end

	has_design_number: BOOLEAN
			-- Does `Current' have an associated design number?
		do
			Result := is_project
		end

	design_number: INTEGER
			-- Design number of element
		do
			-- Inapplicable
		end

	has_design_name: BOOLEAN = True
			-- Does `Current' have a associated design name?

	design_name: STRING
			-- Design name of element or identifying name if
			-- no associated design name
		do
			Result := identification
		end

	die_id: STRING
			-- Die ID of element or empty string if
			-- no associated die ID
		do
			Result := ""
		end

	info_name: STRING
			-- Name of element for packer information window
		do
			Result := identification_for_message
		end

	account_number: STRING
			-- Account number of element or identifying name if
			-- no associated account number
		do
			Result := identification
		end

	account_type: STRING
			-- Account type of element or identifying name if
			-- no associated account type
		do
			Result := identification
		end

	effective_overhead_style: STRING
			-- Effective overhead style for element
		do
			Result := ""
		end

	effective_overhead_style_description: STRING
			-- Description of effective overhead 
			-- style for element
		do
			Result := ""
		end

	organization_short_id: STRING
			-- Short name of organization type for element or
			-- identifying name if no associated organization
		do
			Result := identification
		end

	wafer_etest_id: STRING
			-- Wafer etest ID for element or identifying
			-- name if no associated wafer etest ID
		do
			Result := identification
		end

	wafer_etest_name: STRING
			-- Wafer etest name for element
		require
			has_wafer_etest_id: has_wafer_etest_id
		do
		end

	assigned_wafer_etest_id: STRING
			-- Wafer etest ID assigned to element or empty string
			-- if none assigned yet or if not applicable
		require
			has_wafer_etest_id: has_wafer_etest_id
		do
			Result := ""
		end

	assigned_wafer_etest_name: WAFER_ETEST_NAMES_ROW
			-- Wafer etest name assigned to element or Void
			-- if none assigned yet
		require
			has_wafer_etest_id: has_wafer_etest_id
		do
		end

	unassigned_wafer_etest_ids: LIST [STRING]
			-- Wafer etest IDs not assigned to element or
			-- empty list if no unassigned etest IDs or
			-- if not applicable
		require
			has_wafer_etest_id: has_wafer_etest_id
		do
			create {ARRAYED_LIST [STRING]} Result.make (0)
		end

	has_wafer_etest_id: BOOLEAN
			-- Does object have associated wafer etest ID(s)?
		do
			Result := False
		end

	is_wafer_etest_id_assigned: BOOLEAN
			-- Does object with associated wafer etest ID(s)
			-- have a specific one assigned to it?
		require
			has_wafer_etest_id: has_wafer_etest_id
		do
			Result := not assigned_wafer_etest_id.is_empty
		end

	is_wafer_etest_id_missing: BOOLEAN
			-- Does object require that a wafer etest ID be
			-- assigned but not have a specific one assigned?
		do
			Result := has_wafer_etest_id and then not is_wafer_etest_id_assigned
		end

	compatible_wafer_etest_ids: LIST [STRING]
			-- Compatible wafer etest IDs for element or
			-- empty list if not applicable to this element
		require
			has_wafer_etest_id: has_wafer_etest_id
		do
			create {ARRAYED_LIST [STRING]} Result.make (0)
		end

	compatible_wafer_etest_names: LIST [WAFER_ETEST_NAMES_ROW]
			-- Compatible wafer etest names rows for element
		require
			has_wafer_etest_id: has_wafer_etest_id
		do
			create {ARRAYED_LIST [WAFER_ETEST_NAMES_ROW]} Result.make (0)
		end

	packaging_description: STRING
			-- Description of packaging for element
		do
			Result := ""
		end

	has_ceramic_packaging: BOOLEAN
			-- Will element be packaged in at least one ceramic
			-- package?
		do
			Result := False
		end

	has_plastic_packaging: BOOLEAN
			-- Will element be packaged in at least one plastic
			-- package?
		do
			Result := False
		end

	is_commercial_project: BOOLEAN
			-- Is element a commercial project?
		do
			Result := False
		end

	is_mep_project: BOOLEAN
			-- Is element an MEP project?
		do
			Result := False
		end

	is_other_project: BOOLEAN
			-- Is element a project whose account type is not
			-- commercial or MEP?
		do
			Result := False
		end

	is_project: BOOLEAN
			-- Is element a project?
		do
			Result := False
		end

	is_virtual: BOOLEAN
			-- Is element virtual (label, field of view edge)
			-- rather than something that can potentially be
			-- manipulated (selected, moved, etc)?
		do
			Result := False
		end

	is_field_part: BOOLEAN
			-- Is element one of the parts of a field of view?
		do
			Result := False
		end

	is_field: BOOLEAN
			-- Is element a field of view?
		do
			Result := False
		end

	is_field_label: BOOLEAN
			-- Is element a field of view label?
		do
			Result := False
		end

	is_non_field_label: BOOLEAN
			-- Is element a label that does not label a specific
			-- field of view?
		do
			Result := False
		end

	is_cut_pattern_label: BOOLEAN
			-- Is element a label for a cut pattern?
		do
			Result := False
		end

	is_die: BOOLEAN
			-- Is element a die?
		do
			Result := False
		end

	is_cut: BOOLEAN
			-- Is element a cut line?
		do
			Result := False
		end

	is_horizontal_cut: BOOLEAN
			-- Is element a horizontal cut line?
		do
			Result := False
		end

	is_vertical_cut: BOOLEAN
			-- Is element a vertical cut line?
		do
			Result := False
		end

	is_isolated_in_cut_pattern (a_pattern: RETICLE_CUT_PATTERN): BOOLEAN
			-- Is element isolated in `a_pattern'?
		require
			is_die: is_die
		do
			Result := a_pattern.isolates (die_location)
		end

	cut_position: DOUBLE
			-- Position of cut, in model coordinates (not
			-- relative to center of field)
		require
			is_cut: is_cut
		do
			if is_horizontal_cut then
				Result := center_y
			else
				Result := center_x
			end
		end

	is_cust_project: BOOLEAN
			-- Is element a customer project?
		do
			Result := False
		end

	is_unlocked_project: BOOLEAN
			-- Is element a customer project that is not
			-- locked against customer update?
		do
			Result := is_cust_project and then not die.project.is_currently_locked
		end

	is_unassigned_project: BOOLEAN
			-- Is element a customer project that is not
			-- not assigned to this run?
		do
			Result := is_cust_project and model.run_id /= Void and then not die.project.is_assigned_to_run (model.run_id)
		end

	is_test_vehicle: BOOLEAN
			-- Is element a test vehicle?
		do
			Result := False
		end

	is_vendor_geometry: BOOLEAN
			-- Is element a vendor geometry/ghost?
		do
			Result := False
		end

	is_scribe: BOOLEAN
			-- Is element a scribe line?
		do
			Result := False
		end

	is_horizontal_scribe: BOOLEAN
			-- Is element a horizontal scribe line?
		do
			Result := False
		end

	is_vertical_scribe: BOOLEAN
			-- Is element a vertical scribe line?
		do
			Result := False
		end

	is_filler: BOOLEAN
			-- Is element a filler?
		do
			Result := False
		end

	is_below_minimum_size: BOOLEAN
			-- Is element size below the minimum allowed die size
			-- in either dimension?
		local
			l_min_size: DOUBLE
		do
			if is_die then
				l_min_size := model.minimum_die_size
				Result := size_x < l_min_size or size_y < l_min_size
			else
				Result := False
			end
		end

	is_small_scribe_or_filler: BOOLEAN
			-- Is element a scribe or filler whose size is
			-- below the small object threshold in either
			-- dimension (just length for scribes)?
		local
			l_x, l_y, limit: DOUBLE
		do
			l_x := size_x
			l_y := size_y
			limit := model.created_small_object_threshold
			Result := (is_horizontal_scribe and l_x < limit) or (is_vertical_scribe and l_y < limit) or (is_filler and (l_x < limit or l_y < limit))
		end

	uniform_white_space_border: DOUBLE
			-- Uniform amount of white space around element
		do
			Result := 0.0
		end

	minimum_separation: DOUBLE
			-- Minimum separation between object that is a
			-- project (contains circuitry) and other objects
			-- or field of view edge
		do
			Result := model.minimum_separation
		end

	has_field_edge_restriction: BOOLEAN
			-- Does element have a restriction on how close it
			-- can be to edge of field?
		do
			Result := True
		end

	needs_min_sep_white_space: BOOLEAN
			-- Does element need "minimum separation" amount of
			-- white space around it?
		do
			Result := False
		end

	minimum_gap: DOUBLE
			-- Gridded minimum gap needed between object and field
			-- of view edge or another object (zero gap is also
			-- allowed if not `needs_min_sep_white_space')
		do
			if needs_min_sep_white_space then
				Result := needed_white_space
			else
				Result := minimum_separation
			end
		ensure
			result_nonnegative: Result >= 0.0
		end

	minimum_separation_from (an_element: RETICLE_MODEL_ELEMENT): DOUBLE
			-- Minimum separation needed between `Current'
			-- and `an_element' (zero is also allowed if neither
			-- element `needs_min_sep_white_space')
		do
			Result := minimum_separation - (uniform_white_space_border + an_element.uniform_white_space_border)
		end

	participates_in_measure: BOOLEAN
			-- Does `Current' participate in measure in X/Y
			-- operations?
		do
			Result := True
		end

	can_be_isolated: BOOLEAN
			-- Is `Current' allowed to be isolated, based on its
			-- type?
		do
			Result := False
		end

	offset_cut_center_x: DOUBLE
			-- Offset of center of die from center of cut die
			-- in X dimension, using closest edges of surrounding
			-- scribes instead of centers
		require
			scribe_cut_die_not_void: scribe_cut_die /= Void
		local
			l_cut_die: like edge_cut_die
		do
			l_cut_die := edge_cut_die
			Result := (grid.rounded_edge (left_x - l_cut_die.low_x) - grid.rounded_edge (l_cut_die.high_x - right_x)) / 2.0
			if grid.rounded_center_units (Result) \\ 2 = 0 then
				Result := grid.rounded_edge (Result)
			else
				Result := grid.truncated_edge (Result)
			end
		end

	offset_cut_center_y: DOUBLE
			-- Offset of center of die from center of cut die
			-- in X dimension, using closest edges of surrounding
			-- scribes instead of centers
		require
			scribe_cut_die_not_void: scribe_cut_die /= Void
		local
			l_cut_die: like edge_cut_die
		do
			l_cut_die := edge_cut_die
			Result := (grid.rounded_edge (bottom_y - l_cut_die.low_y) - grid.rounded_edge (l_cut_die.high_y - top_y)) / 2.0
			if grid.rounded_center_units (Result) \\ 2 = 0 then
				Result := grid.rounded_edge (Result)
			else
				Result := grid.truncated_edge (Result)
			end
		end

	field_relative_center_x: DOUBLE
			-- Center of minimum bounding box in X dimension,
			-- relative to center of field with which element
			-- is associated
		deferred
		end

	field_relative_center_y: DOUBLE
			-- Center of minimum bounding box in Y dimension,
			-- relative to center of field with which element
			-- is associated
		deferred
		end

	field_relative_left_x: DOUBLE
			-- Left edge of minimum bounding box in X dimension,
			-- relative to center of field with which element
			-- is associated
		do
			Result := grid.rounded_center (field_relative_center_x - size_x / 2.0)
		end

	field_relative_right_x: DOUBLE
			-- Right edge of minimum bounding box in X dimension,
			-- relative to center of field with which element
			-- is associated
		do
			Result := grid.rounded_center (field_relative_center_x + size_x / 2.0)
		end

	field_relative_top_y: DOUBLE
			-- Top edge of minimum bounding box in X dimension,
			-- relative to center of field with which element
			-- is associated
		do
			Result := grid.rounded_center (field_relative_center_y + size_y / 2.0)
		end

	field_relative_bottom_y: DOUBLE
			-- Bottom edge of minimum bounding box in X dimension,
			-- relative to center of field with which element
			-- is associated
		do
			Result := grid.rounded_center (field_relative_center_y - size_y / 2.0)
		end

	field: RETICLE_FIELD
			-- Field of view with which element is associated
		do
			if cached_field = Void then
				fetch_field
			end
			Result := cached_field
		end

	is_center_in_field: BOOLEAN
			-- Is center of element inside field of view associated
			-- with element (closest field)?
		do
			Result := field.bounding_box.contains (create {CONST_POINT}.make_xy (center_x, center_y))
		end

	is_center_x_in_field: BOOLEAN
			-- Is center of element in X inside field of view
			-- associated with element (closest field)?
		do
			Result := field.bounding_box.contains (create {CONST_POINT}.make_xy (center_x, field.bounding_box.center_y))
		end

	is_center_y_in_field: BOOLEAN
			-- Is center of element in Y inside field of view
			-- associated with element (closest field)?
		do
			Result := field.bounding_box.contains (create {CONST_POINT}.make_xy (field.bounding_box.center_x, center_y))
		end

	intersects_any_field: BOOLEAN
			-- Does element bounding box intersect any
			-- field of view?
		local
			list: like all_fields
			mbb: like bounding_box
		do
			list := all_fields
			mbb := bounding_box
			from
				list.start
			until
				list.after or Result
			loop
				Result := list.item.bounding_box.intersects (mbb)
				list.forth
			end
		end

	intersects_more_than_one_field: BOOLEAN
			-- Does element bounding box intersect more than one
			-- field of view?
		local
			list: like all_fields
			mbb: like bounding_box
			l_count: INTEGER
		do
			list := all_fields
			mbb := bounding_box
			from
				list.start
			until
				list.after or l_count > 1
			loop
				if list.item.bounding_box.intersects (mbb) then
					l_count := l_count + 1
				end
				list.forth
			end
			Result := l_count > 1
		end

	intersection_with_fields: LIST [RECTANGLE]
			-- Rectangles that represent intersection of element
			-- bounding box with fields of view
		local
			list: like all_fields
			mbb: like bounding_box
			fbb, r: RECTANGLE
			f: RETICLE_FIELD
			lx, ly, hx, hy: DOUBLE
		do
			list := all_fields
			create {ARRAYED_LIST [RECTANGLE]} Result.make (list.count)
			mbb := bounding_box
			from
				list.start
			until
				list.after
			loop
				f := list.item
				create fbb.make_copy (f.bounding_box)
				if fbb.intersects (mbb) then
					r := fbb.intersection_with (mbb)
					lx := grid.rounded_edge (r.low_x)
					ly := grid.rounded_edge (r.low_y)
					hx := grid.rounded_edge (r.high_x)
					hy := grid.rounded_edge (r.high_y)
					r.make_by_corners (lx, ly, hx, hy)
					Result.extend (r)
					
				end
				list.forth
			end
		end

	intersecting_fields: LIST [RETICLE_FIELD]
			-- Fields that element bounding box intersects
		local
			list: like all_fields
			mbb: like bounding_box
			f: RETICLE_FIELD
		do
			list := all_fields
			create {ARRAYED_LIST [RETICLE_FIELD]} Result.make (list.count)
			mbb := bounding_box
			from
				list.start
			until
				list.after
			loop
				f := list.item
				if f.bounding_box.intersects (mbb) then
					Result.extend (f)
				end
				list.forth
			end
		end

	enclosed_by_one_field: BOOLEAN
			-- Does element bounding box completely enclosed by
			-- exactly one field of view (edges may touch field
			-- edges)?
		local
			list: like all_fields
			mbb: like bounding_box
		do
			list := all_fields
			mbb := bounding_box
			from
				list.start
			until
				list.after or Result
			loop
				if list.item.bounding_box.encloses (mbb) then
					Result := True
				end
				list.forth
			end
		end

	partially_outside_fields: BOOLEAN
			-- Is element bounding box partially outside fields
			-- of view (intersects at least one field but is not
			-- completely enclosed by any field)?
		do
			Result := intersects_any_field and not enclosed_by_one_field
		end

	is_too_close_to_field_edge: BOOLEAN
			-- Is at least one edge of element too close to
			-- corresponding edge of its associated field?
		local
			dist, l, r, t, b: DOUBLE
		do
			if has_field_edge_restriction and then intersects_any_field then
				if needs_min_sep_white_space then
					dist := needed_white_space
					if dist > 0.0 then
						l := left_to_field_edge
						r := right_to_field_edge
						t := top_to_field_edge
						b := bottom_to_field_edge
						Result := (l >= 0.0 and l < dist) or (r >= 0.0 and r < dist) or (t >= 0.0 and t < dist) or (b >= 0.0 and b < dist)
					end
				else
					dist := minimum_separation
					l := left_to_field_edge
					r := right_to_field_edge
					t := top_to_field_edge
					b := bottom_to_field_edge
					Result := (l > 0.0 and l < dist) or (r > 0.0 and r < dist) or (t > 0.0 and t < dist) or (b > 0.0 and b < dist)
				end
			end
		end

	is_edge_too_close_to_field_edge (dist: DOUBLE): BOOLEAN
			-- Is edge of element that is `dist' away from
			-- corresponding edge of its associated field
			-- too close?
		local
			required_dist: DOUBLE
		do
			if has_field_edge_restriction and then intersects_any_field then
				if needs_min_sep_white_space then
					required_dist := needed_white_space
					if required_dist > 0.0 then
						Result := dist >= 0.0 and dist < required_dist
					end
				else
					required_dist := minimum_separation
					Result := dist > 0.0 and dist < required_dist
				end
			end
		end

	distance_in_x_from (an_element: like Current): DOUBLE
			-- Gridded distance of element from
			-- `an_element' in X dimension.  Result is
			-- zero if elements overlap in X
		local
			l, r: DOUBLE
			l2, r2: DOUBLE
		do
			l := grid.rounded_edge (left_x)
			r := grid.rounded_edge (right_x)
			l2 := grid.rounded_edge (an_element.left_x)
			r2 := grid.rounded_edge (an_element.right_x)
			if l >= r2 then
				Result := l - r2
			elseif r <= l2 then
				Result := l2 - r
			else
				Result := 0.0
			end
			Result := grid.rounded_edge (Result)
		end

	distance_in_y_from (an_element: like Current): DOUBLE
			-- Gridded distance of element from
			-- `an_element' in Y dimension.  Result is
			-- zero if elements overlap in Y
		local
			t, b: DOUBLE
			t2, b2: DOUBLE
		do
			t := grid.rounded_edge (top_y)
			b := grid.rounded_edge (bottom_y)
			t2 := grid.rounded_edge (an_element.top_y)
			b2 := grid.rounded_edge (an_element.bottom_y)
			if b >= t2 then
				Result := b - t2
			elseif t <= b2 then
				Result := b2 - t
			else
				Result := 0.0
			end
			Result := grid.rounded_edge (Result)
		end

	left_to_field_edge: DOUBLE
			-- Gridded distance from left edge of element to left
			-- edge of its associated field (positive if
			-- element edge is to right of field edge)
		do
			Result := grid.rounded_edge (field_relative_left_x - field.field_relative_left_x)
		end

	right_to_field_edge: DOUBLE
			-- Gridded distance from right edge of element to right
			-- edge of its associated field (positive if
			-- element edge is to left of field edge)
		do
			Result := grid.rounded_edge (field.field_relative_right_x - field_relative_right_x)
		end

	top_to_field_edge: DOUBLE
			-- Gridded distance from top edge of element to top
			-- edge of its associated field (positive if
			-- element edge is below field edge)
		do
			Result := grid.rounded_edge (field.field_relative_top_y - field_relative_top_y)
		end

	bottom_to_field_edge: DOUBLE
			-- Gridded distance from bottom edge of
			-- element to bottom edge of its associated
			-- field (positive if element edge is above
			-- field edge)
		do
			Result := grid.rounded_edge (field_relative_bottom_y - field.field_relative_bottom_y)
		end

	field_containing_top_edge: RETICLE_FIELD
			-- Field that truly contains at least part of
			-- top edge of element bounding box or Void if
			-- none
		local
			mbb: like bounding_box
		do
			mbb := bounding_box
			Result := field_containing_either_point (mbb.ul, mbb.ur)
		end

	field_containing_bottom_edge: RETICLE_FIELD
			-- Field that truly contains at least part of
			-- bottom edge of element bounding box or Void if
			-- none
		local
			mbb: like bounding_box
		do
			mbb := bounding_box
			Result := field_containing_either_point (mbb.ll, mbb.lr)
		end

	field_containing_left_edge: RETICLE_FIELD
			-- Field that truly contains at least part of
			-- left edge of element bounding box or Void if
			-- none
		local
			mbb: like bounding_box
		do
			mbb := bounding_box
			Result := field_containing_either_point (mbb.ll, mbb.ul)
		end

	field_containing_right_edge: RETICLE_FIELD
			-- Field that truly contains at least part of
			-- right edge of element bounding box or Void if
			-- none
		local
			mbb: like bounding_box
		do
			mbb := bounding_box
			Result := field_containing_either_point (mbb.lr, mbb.ur)
		end

	x_outside_all_fields (a_value: DOUBLE): BOOLEAN
			-- Is `a_value' outside all fields of view in X
			-- dimension or on field edge?
		local
			list: like all_fields
			p: POINT
			mbb: CONST_RECTANGLE
		do
			Result := True
			list := all_fields
			from
				list.start
			until
				list.after or not Result
			loop
				mbb := list.item.bounding_box
				create p.make_xy (a_value, mbb.center_y)
				Result := not mbb.truly_contains (p)
				list.forth
			end
		end

	y_outside_all_fields (a_value: DOUBLE): BOOLEAN
			-- Is `a_value' outside all fields of view in Y
			-- dimension or on field edge?
		local
			list: like all_fields
			p: POINT
			mbb: CONST_RECTANGLE
		do
			Result := True
			list := all_fields
			from
				list.start
			until
				list.after or not Result
			loop
				mbb := list.item.bounding_box
				create p.make_xy (mbb.center_x, a_value)
				Result := not mbb.truly_contains (p)
				list.forth
			end
		end

	closest_field_edge_above (a_value: DOUBLE): DOUBLE
			-- Y coordinate of field lower edge which is
			-- closest to and above or equal to `a_value'
		local
			list: like all_fields
			f: RETICLE_FIELD
			edge: DOUBLE
		do
			Result := Infinity
			list := all_fields
			from
				list.start
			until
				list.after
			loop
				f := list.item
				edge := grid.rounded_edge (f.bottom_y)
				if edge >= a_value and edge < Result then
					Result := edge
				end
				list.forth
			end
		end

	closest_field_edge_below (a_value: DOUBLE): DOUBLE
			-- Y coordinate of field upper edge which is closest to
			-- and below or equal to `a_value'
		local
			list: like all_fields
			f: RETICLE_FIELD
			edge: DOUBLE
		do
			Result := -Infinity
			list := all_fields
			from
				list.start
			until
				list.after
			loop
				f := list.item
				edge := grid.rounded_edge (f.top_y)
				if edge <= a_value and edge > Result then
					Result := edge
				end
				list.forth
			end
		end

	closest_field_edge_to_left (a_value: DOUBLE): DOUBLE
			-- Y coordinate of field right edge which is
			-- closest to and left of or equal to
			-- `a_value'
		local
			list: like all_fields
			f: RETICLE_FIELD
			edge: DOUBLE
		do
			Result := -Infinity
			list := all_fields
			from
				list.start
			until
				list.after
			loop
				f := list.item
				edge := grid.rounded_edge (f.right_x)
				if edge <= a_value and edge > Result then
					Result := edge
				end
				list.forth
			end
		end

	closest_field_edge_to_right (a_value: DOUBLE): DOUBLE
			-- Y coordinate of field left edge which is
			-- closest to and right of or equal to
			-- `a_value'
		local
			list: like all_fields
			f: RETICLE_FIELD
			edge: DOUBLE
		do
			Result := Infinity
			list := all_fields
			from
				list.start
			until
				list.after
			loop
				f := list.item
				edge := grid.rounded_edge (f.left_x)
				if edge >= a_value and edge < Result then
					Result := edge
				end
				list.forth
			end
		end

	all_fields: LIST [RETICLE_FIELD]
			-- All fields of view with which element could be
			-- associated
		do
			Result := model.fields
		end

	die: PACKING_DIES_MROW
			-- Die for element that is a die
		require
			is_die: is_die
		deferred
		ensure
			result_not_void: Result /= Void
		end

	die_location: PACKING_DIE_LOCATIONS_MROW
			-- Die location for element that is a die
		require
			is_die: is_die
		deferred
		ensure
			result_not_void: Result /= Void
		end

	cut_location: PACKING_CUT_LOCATIONS_MROW
			-- Cut location for element that is a cut
		require
			is_cut: is_cut
		do
		ensure
			result_not_void: Result /= Void
		end

	project: PACKING_PROJECTS_MROW
			-- Project for element that is a project
		require
			is_project: is_project
		do
			Result := die.project
		ensure
			result_not_void: Result /= Void
		end

	max_die_size: CONST_POINT
			-- Maximum die size, as constrained by package
			-- cavity sizes and user-specified limits.
			-- Void indicates no maximum
		require
			is_die: is_die
		deferred
		end

	is_oversize_die_always_allowed: BOOLEAN
			-- Is a die that is bigger than `max_die_size'
			-- allowed unconditionally?
		require
			is_die: is_die
		deferred
		end

	is_oversize_die_possibly_allowed: BOOLEAN
			-- Is a die that is bigger than `max_die_size'
			-- allowed but only under specific conditions?
		require
			is_die: is_die
		deferred
		end

	oversize_die_inclusion_conditions: STRING
			-- Description of conditions under which 
			-- die that is bigger than `max_die_size'
			-- allowed
		require
			is_die: is_die
			oversize_die_possibly_allowed: is_oversize_die_possibly_allowed
		deferred
		end

	has_sub_elements: BOOLEAN
			-- Is element composed of more than one sub-element,
			-- each of which has its own size and center?
		do
			Result := False
		end

	sub_elements: LIST [like anchored_model_sub_element]
			-- Sub-elements of `Current'
		do
		end

	last_new_element: RETICLE_MODEL_ELEMENT
			-- Last new element created by call to `duplicate'
		do
			Result := model.last_new_element
		end

	all_copies: LIST [RETICLE_MODEL_ELEMENT]
			-- All copies of this element

	scribe_cut_die: CONST_RECTANGLE
			-- Last computed cut die for a project, using
			-- centers of surrounding scribes for edges of cut die

	oriented_scribe_cut_die: CONST_RECTANGLE
			-- Last computed cut die for a project,
			-- oriented the same as the die (i.e., rotated
			-- if die is rotated), using centers of
			-- surrounding scribes for edges of cut die
		do
			if scribe_cut_die /= Void then
				create Result.make_by_size_and_center (scribe_cut_die_size_x, scribe_cut_die_size_y, scribe_cut_die.center_x, scribe_cut_die.center_y)
			else
				Result := Void
			end
		end

	scribe_cut_die_size_x: DOUBLE
			-- Size of cut die in X dimension, using centers
			-- of surrounding scribes for edges of cut die
		do
			if is_rotated_sideways then
				Result := scribe_cut_die.size_y
			else
				Result := scribe_cut_die.size_x
			end
		end

	scribe_cut_die_size_y: DOUBLE
			-- Size of cut die in Y dimension, using centers
			-- of surrounding scribes for edges of cut die
		do
			if is_rotated_sideways then
				Result := scribe_cut_die.size_x
			else
				Result := scribe_cut_die.size_y
			end
		end

	edge_cut_die: CONST_RECTANGLE
			-- Last computed cut die for a project, using
			-- closest edges of surrounding scribes for edges
			-- of cut die

	cut_cut_die: CONST_RECTANGLE
			-- Last computed cut die for a project, using
			-- cuts for edges of cut die

	cut_cut_die_size_x: DOUBLE
			-- Size of cut die in X dimension, using centers
			-- of surrounding cuts for edges of cut die
		do
			if is_rotated_sideways then
				Result := cut_cut_die.size_y
			else
				Result := cut_cut_die.size_x
			end
		end

	cut_cut_die_size_y: DOUBLE
			-- Size of cut die in Y dimension, using centers
			-- of surrounding cut for edges of cut die
		do
			if is_rotated_sideways then
				Result := cut_cut_die.size_x
			else
				Result := cut_cut_die.size_y
			end
		end

	oriented_field_relative_cut_cut_die: CONST_RECTANGLE
			-- Last computed cut die for a project,
			-- oriented the same as the die (i.e., rotated
			-- if die is rotated), using surrounding cuts for
			-- for edges of cut die and with location relative
			-- to center of field
		do
			if cut_cut_die /= Void then
				create Result.make_by_size_and_center (cut_cut_die_size_x, cut_cut_die_size_y, cut_cut_die.center_x - field.center_x, cut_cut_die.center_y - field.center_y)
			else
				Result := Void
			end
		end

	unoriented_model_relative_rectangle (a_rectangle: CONST_RECTANGLE): RECTANGLE
			-- Transformation of oriented (rotated if die
			-- is rotated) field-relative rectangle `a_rectangle' 
			-- to an unoriented model-relative rectangle
		
		local
			l_size_x, l_size_y: DOUBLE
		do
			if a_rectangle /= Void then
				if is_rotated_sideways then
					l_size_x := a_rectangle.size_y
					l_size_y := a_rectangle.size_x
				else
					l_size_x := a_rectangle.size_x
					l_size_y := a_rectangle.size_y
				end
				create Result.make_by_size_and_center (l_size_x, l_size_y, a_rectangle.center_x + field.center_x, a_rectangle.center_y + field.center_y)
			else
				Result := Void
			end
		end

	anchored_model: RETICLE_MODEL
			-- Type of model element is part of
		do
		end

	anchored_model_element: RETICLE_MODEL_ELEMENT
			-- Type of model element of which `Current' is
			-- one part (inapplicable)
		do
		end

	anchored_model_sub_element: RETICLE_MODEL_SUB_ELEMENT
			-- Type of sub-elements if model element has
			-- sub-elements
		do
		end

	is_reworkable: BOOLEAN
			-- Is element currently reworkable?
		do
			Result := False
		end

feature -- Modification

	set_all_copies (a_list: like all_copies)
		require
			a_list_not_void: a_list /= Void
		do
			all_copies := a_list
		ensure
			all_copies_set: all_copies = a_list
		end

	set_scribe_cut_die (a_cut_die: like scribe_cut_die)
			-- Set `scribe_cut_die' to `a_cut_die'
		do
			scribe_cut_die := a_cut_die
		end

	set_edge_cut_die (a_cut_die: like edge_cut_die)
			-- Set `edge_cut_die' to `a_cut_die'
		do
			edge_cut_die := a_cut_die
		end

	set_cut_cut_die (a_cut_die: like cut_cut_die)
			-- Set `cut_cut_die' to `a_cut_die'
		do
			cut_cut_die := a_cut_die
		end

feature -- Operations

	uncache_field
			-- Remove `field' from cache, if present
		do
			cached_field := Void
		end

	duplicate
			-- Duplicate element and add it to model.
			-- Leave new element in `last_new_element'
		require
			is_die: is_die
		deferred
		end

	move_left_edge (a_distance: DOUBLE)
			-- Move left edge of `Current' by `a_distance'
		require
			is_die: is_die
		deferred
		end

	move_right_edge (a_distance: DOUBLE)
			-- Move right edge of `Current' by `a_distance'
		require
			is_die: is_die
		deferred
		end

	move_top_edge (a_distance: DOUBLE)
			-- Move top edge of `Current' by `a_distance'
		require
			is_die: is_die
		deferred
		end

	move_bottom_edge (a_distance: DOUBLE)
			-- Move bottom edge of `Current' by `a_distance'
		require
			is_die: is_die
		deferred
		end

feature {NONE} -- Implementation

	needed_white_space: DOUBLE
			-- Gridded amount of extra white space needed between
			-- object that is a project (contains circuitry) and
			-- other objects or field of view edge
		local
			val: DOUBLE
		do
			val := minimum_separation - uniform_white_space_border
			if not grid.is_edge_on_grid (val) and then grid.is_center_on_grid (val) then
				-- Uniform white space border may be on half
				-- grid, so leave `va' on half grid if so
				Result := grid.rounded_center (val)
			else
				Result := grid.rounded_edge (val)
			end
		end

	field_containing_either_point (p1, p2: CONST_POINT): RETICLE_FIELD
			-- Field that truly contains either `p1' or
			-- `p2' or both or Void if none
		local
			list: like all_fields
			f: RETICLE_FIELD
			mbb: CONST_RECTANGLE
		do
			list := all_fields
			from
				list.start
			until
				list.after or Result /= Void
			loop
				f := list.item
				mbb := f.bounding_box
				if mbb.truly_contains (p1) or mbb.truly_contains (p2) then
					Result := f
				end
				list.forth
			end
		end

	fetch_field
			-- Fetch field with which die is associated, based
			-- on its name, and put in `cached_field'
		do
			cached_field := model.field_with_name (associated_field_name)
		end

	associated_field_name: STRING
			-- Name of field with which element is associated
		deferred
		ensure
			result_not_void: Result /= Void
		end
	
	cached_field: RETICLE_FIELD
			-- Field with which die is associated, cached
			-- from lookup
	
end
