
indexing
	
	description: "Scribe element of reticle model"
	author: "David Hollenberg"
	date: "July 28, 2006"

class RETICLE_SCRIBE
inherit
	RETICLE_DIE
		redefine
			is_scribe, is_stationary_orientation,
			is_horizontal_scribe, is_vertical_scribe,
			identification_for_message, duplicate
		end

create
	make

feature -- Properties

	identification: STRING = ""
			-- Identifying name of element

	identification_for_message: STRING
			-- Identifying name of element for displaying
			-- error and warning messages
		do
			if is_horizontal_scribe then
				Result := horizontal_scribe_name
			else
				Result := vertical_scribe_name
			end
		end

	is_scribe: BOOLEAN = True
			-- Is element a scribe line?

	is_horizontal_scribe: BOOLEAN
			-- Is scribe horizontal?
		do
			Result := die.is_horizontal_scribe
		end

	is_vertical_scribe: BOOLEAN
			-- Is scribe vertical?
		do
			Result := die.is_vertical_scribe
		end

	is_stationary_orientation: BOOLEAN = True
			-- Can orientation of object never be changed?

feature -- Contract support

	is_valid_for_creation (a_die: like die): BOOLEAN
			-- Is `a_die' have a die type that is valid
			-- for creation of `Current'?
		do
			Result := a_die.is_scribe
		end

feature -- Modification

	duplicate
			-- Duplicate element and add it to model.  For scribes,
			-- we create a new die with a single location to
			-- ensure that each scribe has only one location
		local
			d: like die
			dl: like die_location
		do
			model.add_new_scribe_with_orientation (die.scribe_orientation)
			d := model.last_die
			d.set_die_size_printed_x (die.die_size_printed_x)
			d.set_die_size_printed_y (die.die_size_printed_y)
			d.add_die_location (die_location.field_name, die_location.location_x, die_location.location_y)
			dl := d.last_die_location
			dl.set_is_rotatable (False)
			model.add_die_location (d, dl)
		end

feature {NONE} -- Implementation

	horizontal_scribe_name: STRING = "HORIZONTAL SCRIBE"
			-- Name for horizontal scribe in messages

	vertical_scribe_name: STRING = "VERTICAL SCRIBE"
			-- Name for vertical scribe in messages

end
