
indexing
	
	description: "Vendor geometry/ghost element of reticle model"
	author: "David Hollenberg"
	date: "July 28, 2006"

class RETICLE_VENDOR_GEOMETRY
inherit
	RETICLE_DIE
		redefine
			is_vendor_geometry, is_stationary_in_x,
			is_stationary_in_y, is_stationary_orientation,
			is_never_duplicatable, is_never_deletable,
			has_field_edge_restriction,
			needs_min_sep_white_space
		end

create
	make

feature -- Properties

	identification: STRING
			-- Identifying name of element
		do
			Result := die.geometry_id
		end

	is_vendor_geometry: BOOLEAN = True
			-- Is element a vendor geometry/ghost?

	is_stationary_in_x: BOOLEAN
			-- Can object never be moved in the X dimension?
		do
			Result := has_fixed_locations
		end

	is_stationary_in_y: BOOLEAN
			-- Can object never be moved in the Y dimension?
		do
			Result := has_fixed_locations
		end

	is_stationary_orientation: BOOLEAN
			-- Can orientation of object never be changed?
		local
			vg: GEOMETRIES_ROW
		do
			vg := die.vendor_geometry
			if vg.has_quantities (field.field_set_name) then
				Result := not vg.quantities (field.field_set_name).first.is_rotatable
			else
				Result := True
			end
		end

	is_never_duplicatable: BOOLEAN
			-- Can object never be duplicated?
		do
			Result := has_fixed_locations
		end

	is_never_deletable: BOOLEAN
			-- Can object never be deleted?
		do
			Result := has_fixed_locations
		end

	has_field_edge_restriction: BOOLEAN
			-- Does element have a restriction on how close it
			-- can be to edge of field?
		do
			Result := not has_fixed_locations
		end

	needs_min_sep_white_space: BOOLEAN
			-- Does element need "minimum separation" amount of
			-- white space around it?
		do
			Result := not has_fixed_locations
		end

feature -- Contract support

	is_valid_for_creation (a_die: like die): BOOLEAN
			-- Does `a_die' have a die type that is valid
			-- for creation of `Current'?
		do
			Result := a_die.is_vendor_geometry_or_ghost
		end

feature {NONE} -- Implementation

	has_fixed_locations: BOOLEAN
			-- Does vendor geometry have fixed locations rather
			-- than a quantity requirement?
		do
			Result := die.vendor_geometry.has_fixed_locations (field.field_set_name)
		end

end
