
indexing
	
	description: "A cut pattern in reticle view"
	author: "David Hollenberg"
	date: "July 28, 2006"

class RETICLE_VIEW_CUT_PATTERN

create
	make

feature {NONE} -- Creation

	make (a_pattern: like pattern a_view: RETICLE_VIEW)
		require
			a_pattern_not_void: a_pattern /= Void
			a_view_not_void: a_view /= Void
		do
			pattern := a_pattern
			view := a_view
		end

feature -- Properties

	pattern: RETICLE_CUT_PATTERN
			-- Cut pattern in model

	cut_pattern_no: INTEGER
			-- Number assigned to cut pattern
		do
			Result := pattern.cut_pattern_no
		end

	cut_patterns_row: PACKING_CUT_PATTERNS_MROW
			-- Row in PACKING_CUT_PATTERNS to which this cut 
			-- pattern corresponds
		do
			Result := pattern.pattern
		end

	field_name: STRING
			-- Name of field to which cut pattern applies
		do
			Result := pattern.field_name
		end

	has_wafer_etest_name: BOOLEAN
			-- Is there a wafer etest name for cut pattern?
		do
			Result := pattern.has_wafer_etest_name
		end

	wafer_etest_name: STRING
			-- Wafer etest name for cut pattern
		do
			Result := pattern.wafer_etest_name
		end

	wafer_etest_id: STRING
			-- Wafer etest ID for cut pattern
		do
			Result := pattern.wafer_etest_id
		end

	has_quantity_wafers_needed: BOOLEAN
			-- Is there a number wafers to cut for pattern?
		do
			Result := pattern.has_quantity_wafers_needed
		end

	quantity_wafers_needed: INTEGER
			-- Number of wafers to cut for pattern or -1 if
			-- not yet specified
		do
			Result := pattern.quantity_wafers_needed
		end

	is_completed: BOOLEAN
			-- Is cut pattern completed (not under construction)?
		do
			Result := pattern.is_completed
		end

	has_cuts: BOOLEAN
			-- Are there any cuts for pattern?
		do
			Result := pattern.has_cuts
		end

	has_horizontal_cut_at (a_location: DOUBLE): BOOLEAN
			-- Does pattern include a horizontal cut at
			-- `a_location'?
		do
			Result := pattern.has_horizontal_cut_at (a_location)
		end

	has_vertical_cut_at (a_location: DOUBLE): BOOLEAN
			-- Does pattern include a vertical cut at
			-- `a_location'?
		do
			Result := pattern.has_vertical_cut_at (a_location)
		end

	has_isolated_sites: BOOLEAN
			-- Are there any isolated sites for pattern?
		do
			Result := pattern.has_isolated_sites
		end

	isolated_cut_die (an_element: RETICLE_VIEW_ELEMENT): CONST_RECTANGLE
			-- Cut die for isolated element `an_element' in 
			-- cut pattern, determined from its cut die site
		require
			an_element_not_void: an_element /= Void
		do
			Result := pattern.isolated_cut_die (an_element.model_element)
		end

	isolated_location_count: INTEGER
			-- Number of die locations isolated by cut pattern
		do
			Result := pattern.isolated_location_count
		end

	description: STRING
			-- Description of cut pattern for messages to user
		do
			Result := pattern.description
		end

	status: STRING
			-- Status of cut pattern
		do
			Result := pattern.status
		end

	view: RETICLE_VIEW
			-- View containing cut pattern

	context: RETICLE_VIEW_CONTEXT
			-- Context to be restored when view displays
			-- this cut pattern

feature -- Modification

	set_context (a_context: like context)
			-- Set `context' to `a_context'
		do
			context := a_context
		ensure
			context_set: context = a_context
		end

	set_is_completed (a_value: BOOLEAN)
			-- Set `is_completed' to `a_value'
		do
			pattern.set_is_completed (a_value)
		ensure
			completed_set: is_completed = a_value
		end

	set_wafer_etest_name (a_name: STRING)
			-- Set `wafer_etest_name' to `a_name'
		do
			pattern.set_wafer_etest_name (a_name)
		ensure
			name_set: wafer_etest_name.is_equal (a_name)
		end

	set_quantity_wafers_needed (a_value: INTEGER has_value: BOOLEAN)
			-- Set `quantity_wafers_needed' to `a_value' where
			-- `has_value' indicates whether there really is a
			-- value
		do
			pattern.set_quantity_wafers_needed (a_value, has_value)
		end

feature -- Operations

	clear_change_history_list
			-- Clear the change history list for `Current'
		do
			context.reticle_change_history_list.wipe_out
		end

	remove_isolated_sites
			-- Remove all isolated sites for cut pattern
		do
			pattern.remove_isolated_sites
		end

	isolate (an_element: RETICLE_VIEW_ELEMENT)
			-- Isolate `an_element' in cut pattern
		require
			an_element_not_void: an_element /= Void
		do
			pattern.isolate (an_element.model_element)
		end

	unisolate (an_element: RETICLE_VIEW_ELEMENT)
			-- Unisolate `an_element' in cut pattern
		require
			an_element_not_void: an_element /= Void
		do
			pattern.unisolate (an_element.model_element)
		end

	set_isolated_site_cut_die (an_element: RETICLE_VIEW_ELEMENT)
			-- Set cut die size and center from `an_element'
		require
			an_element_not_void: an_element /= Void
		do
			pattern.set_isolated_site_cut_die (an_element.model_element)
		end

end
