
indexing
	
	description: "Object that is an element of a view of a reticle"
	author: "David Hollenberg"
	date: "July 25, 2006"

class RETICLE_VIEW_ELEMENT
inherit
	VIEW_ELEMENT
		redefine
			view, anchored_model_element,anchored_view_sub_element,
			draw_without_fill, draw_fill_only, contains_point,
			intersects_area, enclosed_by_area
		end
	HAS_SIZE_X
	HAS_SIZE_Y
	HAS_GRIDDED_SIZE_X
	HAS_GRIDDED_SIZE_Y
	HAS_CENTER_X
	HAS_CENTER_Y
	HAS_LEFT_X
	HAS_BOTTOM_Y
	RETICLE_SCANNABLE_X
	RETICLE_SCANNABLE_Y
	SPREADABLE_BY_WIDTH
	SPREADABLE_BY_HEIGHT
	SPREADABLE_BY_X_POSITION
	SPREADABLE_BY_Y_POSITION
	HAS_PACKING_NAME
	PACKER_UTILITIES
		export
			{NONE} all
		end
	CONDITION_UTILITIES
		export
			{NONE} all
		end
	STRING_UTILITIES_ACCESS
		export
			{NONE} all
		end

feature {NONE} -- Creation

	make (a_model_element: like anchored_model_element a_view: like view)
		require
			a_model_element_not_void: a_model_element /= Void
			a_view_not_void: a_view /= Void
		do
			model_element := a_model_element
			view := a_view
			create bounding_box.make (0, 0, 0, 0)
			create {VIEW_ELEMENT_STANDARD_DRAWER} drawer
			set_label_generator (view.current_label_generator)
			set_color_generator (view.current_color_generator)
			set_fill_generator (view.current_fill_generator)
			set_opacity_generator (view.current_opacity_generator)
		end

feature -- Properties

	view: RETICLE_VIEW
			-- View of which this is one element

	max_db_value: DOUBLE
			-- Maximum value that can fit in the database
		do
			Result := view.max_db_value
		end

	grid: GRID
			-- Grid for object sizes and edges
		do
			Result := view.grid
		end

	palette: RETICLE_VIEW_COLORS
			-- Palette of colors to be used
		do
			Result := view.palette
		end

	label_generator: RETICLE_VIEW_ELEMENT_LABEL_GENERATOR
			-- Generator for text to display to label this element

	is_label_generator_changed: BOOLEAN
			-- Did label generator change on last call to
			-- to `set_label_generator'?

	color_generator: RETICLE_VIEW_ELEMENT_COLOR_GENERATOR
			-- Generator for colors to use to display this element

	is_color_generator_changed: BOOLEAN
			-- Did color generator change on last call to
			-- to `set_color_generator'?

	fill_generator: RETICLE_VIEW_ELEMENT_FILL_GENERATOR
			-- Generator for "filled" indicator to use to
			-- display this element

	is_fill_generator_changed: BOOLEAN
			-- Did fill generator change on last call to
			-- to `set_fill_generator'?

	opacity_generator: RETICLE_VIEW_ELEMENT_OPACITY_GENERATOR
			-- Generator for opaqueness or transparency
			-- indicator to use to display this element

	is_opacity_generator_changed: BOOLEAN
			-- Did opacity generator change on last call to
			-- to `set_opacity_generator'?

	line_width: INTEGER
			-- Width of line to draw for border, in pixels
		do
			if is_selected then
				Result := 3
			else
				Result := 1
			end
		end

	label: STRING
			-- Current label to display
		do
			Result := label_generator.label (Current)
		end

	label_rotation: INTEGER
			-- Rotation of label in degrees clockwise
		do
			Result := rotation
		end

	border_color: EV_COLOR
			-- Color to use to display border
		do
			Result := color_generator.border_color (Current)
		end

	label_color: EV_COLOR
			-- Color to use to display label
		do
			Result := color_generator.label_color (Current)
		end

	is_interior_filled: BOOLEAN
			-- Should interior of element be filled when displayed?
		do
			Result := fill_generator.is_interior_filled (Current)
		end

	interior_color: EV_COLOR
			-- Color to use to fill interior, if filled or opaque
		do
			if is_interior_filled then
				Result := fill_generator.interior_color (Current)
			else
				Result := non_filled_interior_color
			end
		end

	non_filled_interior_color: EV_COLOR
			-- Color to use to fill interior, if not filled or 
			-- opaque
		do
			Result := palette.current_background_color
		end

	is_opaque: BOOLEAN
			-- Should element be displayed as opaque, so that
			-- it completely obscures the part of any element
			-- that is below it in stacking order?
		do
			Result := opacity_generator.is_opaque (Current)
		end

	is_transparent: BOOLEAN
			-- Should element be displayed as transparent, so that
			-- we attempt to draw parts of elements that fall
			-- below it in stacking order so that they are visible?
		do
			Result := opacity_generator.is_transparent (Current)
		end

	is_selected: BOOLEAN
			-- Is element currently selected?

	is_selected_changed: BOOLEAN
			-- Did last call to `set_is_selected' change the
			-- value of `is_selected'?

	included_in_changed_region: BOOLEAN
			-- Is `Current' already included in the region which
			-- has been changed by the current operation?

	matched_last_search: BOOLEAN
			-- Did `Current' match the search criteria for last
			-- Search operation?

	matched_last_search_changed: BOOLEAN
			-- Did value of `matched_last_search' change for
			-- last Search operation?

	identification: STRING
			-- Identifying name of element for displaying
			-- in Standard view
		do
			Result := model_element.identification
		end

	identification_for_message: STRING
			-- Identifying name of element for displaying
			-- error and warning messages
		do
			Result := model_element.identification_for_message
		end

	location_description: STRING
			-- Description of location of element, for use in
			-- error and warning messages
		do
			Result := model_element.location_description
		end

	size_description: STRING
			-- Description of size of element, for use in
			-- error and warning messages
		do
			Result := model_element.size_description
		end

	scribe_cut_size_description: STRING
			-- Description of cut die size of element
			-- determined by surrounding scribes, for use
			-- in error and warning messages
		do
			Result := model_element.scribe_cut_size_description
		end

	cut_cut_size_description: STRING
			-- Description of cut die size of element determined
			-- by surrounding cuts, for use in
			-- error and warning messages
		do
			Result := model_element.cut_cut_size_description
		end

	offset_cut_center_description: STRING
			-- Description of offset of die center from cut
			-- die center, for use in error and warning message
		do
			Result := model_element.offset_cut_center_description
		end

	design_number: INTEGER
			-- Design number of element
		require
			is_project: is_project
		do
			Result := model_element.design_number
		end

	design_name: STRING
			-- Design name of element or identifying name if
			-- no associated design name
		do
			Result := model_element.design_name
		end

	die_id: STRING
			-- Die ID of element or identifying name if
			-- no associated die ID
		do
			Result := model_element.die_id
		end

	account_number: STRING
			-- Account number of element or identifying name if
			-- no associated account number
		do
			Result := model_element.account_number
		end

	account_type: STRING
			-- Account type of element or identifying name if
			-- no associated account type
		do
			Result := model_element.account_type
		end

	organization_short_id: STRING
			-- Short name of organization type for element or
			-- identifying name if no associated organization
		do
			Result := model_element.organization_short_id
		end

	effective_overhead_style: STRING
			-- Effective overhead style for element
		do
			Result := model_element.effective_overhead_style
		end

	effective_overhead_style_description: STRING
			-- Description of effective overhead style for
			-- element
		do
			Result := model_element.effective_overhead_style_description
		end

	wafer_etest_id: STRING
			-- Wafer etest ID for element or identifying
			-- name if no associated wafer etest ID
		do
			Result := model_element.wafer_etest_id
		end

	wafer_etest_name: STRING
			-- Wafer etest name for element
		require
			has_wafer_etest_id: has_wafer_etest_id
		do
			Result := model_element.wafer_etest_name
		end

	assigned_wafer_etest_id: STRING
			-- Wafer etest ID assigned to element or empty string
			-- if none assigned yet or if not applicable
		require
			has_wafer_etest_id: has_wafer_etest_id
		do
			Result := model_element.assigned_wafer_etest_id
		end

	assigned_wafer_etest_name: WAFER_ETEST_NAMES_ROW
			-- Wafer etest name assigned to element or Void
			-- if none assigned yet
		require
			has_wafer_etest_id: has_wafer_etest_id
		do
			Result := model_element.assigned_wafer_etest_name
		end

	unassigned_wafer_etest_ids: LIST [STRING]
			-- Wafer etest IDs not assigned to element or
			-- empty string if no unassigned etest IDs or
			-- if not applicable
		require
			has_wafer_etest_id: has_wafer_etest_id
		do
			Result := model_element.unassigned_wafer_etest_ids
		end

	has_wafer_etest_id: BOOLEAN
			-- Does object have associated wafer etest ID(s)?
		do
			Result := model_element.has_wafer_etest_id
		end

	is_wafer_etest_id_assigned: BOOLEAN
			-- Does object with associated wafer etest ID(s)
			-- have a specific one assigned to it?
		require
			has_wafer_etest_id: has_wafer_etest_id
		do
			Result := model_element.is_wafer_etest_id_assigned
		end

	is_wafer_etest_id_missing: BOOLEAN
			-- Does object require that a wafer etest ID be
			-- assigned but not have a specific one assigned?
		do
			Result := model_element.is_wafer_etest_id_missing
		end

	compatible_wafer_etest_ids: LIST [STRING]
			-- Compatible wafer etest IDs for element or
			-- empty list if not applicable to this element
		require
			has_wafer_etest_id: has_wafer_etest_id
		do
			Result := model_element.compatible_wafer_etest_ids
		end

	compatible_wafer_etest_names: LIST [WAFER_ETEST_NAMES_ROW]
			-- Compatible wafer etest names rows for element
		do
			Result := model_element.compatible_wafer_etest_names
		end

	packaging_description: STRING
			-- Description of packaging for element
		do
			Result := model_element.packaging_description
		end

	has_ceramic_packaging: BOOLEAN
			-- Will element be packaged in at least one ceramic
			-- package?
		do
			Result := model_element.has_ceramic_packaging
		end

	has_plastic_packaging: BOOLEAN
			-- Will element be packaged in at least one plastic
			-- package?
		do
			Result := model_element.has_plastic_packaging
		end

	is_commercial_project: BOOLEAN
			-- Is element a commercial project?
		do
			Result := model_element.is_commercial_project
		end

	is_mep_project: BOOLEAN
			-- Is element an MEP project?
		do
			Result := model_element.is_mep_project
		end

	is_other_project: BOOLEAN
			-- Is element a project whose account type is not
			-- commercial or MEP?
		do
			Result := model_element.is_other_project
		end

	is_die: BOOLEAN
			-- Is element a die?
		do
			Result := model_element.is_die
		end

	is_cut: BOOLEAN
			-- Is element a cut?
		do
			Result := model_element.is_cut
		end

	is_horizontal_cut: BOOLEAN
			-- Is element a horizontal cut line?
		do
			Result := model_element.is_horizontal_cut
		end

	is_vertical_cut: BOOLEAN
			-- Is element a vertical cut line?
		do
			Result := model_element.is_vertical_cut
		end

	cut_position: DOUBLE
			-- Position of cut, in model coordinates (not
			-- relative to center of field)
		require
			is_cut: is_cut
		do
			Result := model_element.cut_position
		end

	gridded_cut_position: DOUBLE
			-- Position of cut, in model coordinates (not
			-- relative to center of field), on grid
		require
			is_cut: is_cut
		do
			Result := grid.rounded_center (cut_position)
		end

	is_cut_pattern_label: BOOLEAN
			-- Is element a cut pattern label?
		do
			Result := model_element.is_cut_pattern_label
		end

	is_field: BOOLEAN
			-- Is element a field of view?
		do
			Result := model_element.is_field
		end

	is_field_label: BOOLEAN
			-- Is element a field of view label?
		do
			Result := model_element.is_field_label
		end

	is_project: BOOLEAN
			-- Is element a project?
		do
			Result := model_element.is_project
		end

	is_cust_project: BOOLEAN
			-- Is element a customer project?
		do
			Result := model_element.is_cust_project
		end

	is_unlocked_or_unassigned_cust_project: BOOLEAN
			-- Is element a customer project that is not
			-- locked against customer update or is not assigned
			-- to this run?
		do
			Result := is_cust_project and is_unlocked_project or is_unassigned_project
		end

	is_unlocked_project: BOOLEAN
			-- Is element a customer project that is not
			-- locked against customer update?
		do
			Result := model_element.is_unlocked_project
		end

	is_unassigned_project: BOOLEAN
			-- Is element a customer project that is not
			-- not assigned to this run?
		do
			Result := model_element.is_unassigned_project
		end

	is_uncentered_in_cut_die_project: BOOLEAN
			-- Is element a customer project or test vehicle
			-- that is isolated in its cut die but not centered
			-- in its cut die?
		do
			Result := is_project and then not is_unisolated_in_scribe_cut_die and then is_uncentered_in_cut_die
		end

	is_test_vehicle: BOOLEAN
			-- Is element a customer project?
		do
			Result := model_element.is_test_vehicle
		end

	is_scribe: BOOLEAN
			-- Is element a scribe line?
		do
			Result := model_element.is_scribe
		end
	
	is_horizontal_scribe: BOOLEAN
			-- Is element a horizontal scribe line?
		do
			Result := model_element.is_horizontal_scribe
		end

	is_vertical_scribe: BOOLEAN
			-- Is element a vertical scribe line?
		do
			Result := model_element.is_vertical_scribe
		end

	is_default_width_scribe: BOOLEAN
			-- Is element a scribe with the default width?
		do
			Result := (is_horizontal_scribe and gridded_size_y = default_scribe_width) or (is_vertical_scribe and gridded_size_x = default_scribe_width)
		end

	is_custom_width_scribe: BOOLEAN
			-- Is element a scribe with custom (non-default) width?
		do
			Result := (is_horizontal_scribe and gridded_size_y /= view.default_scribe_width) or (is_vertical_scribe and gridded_size_x /= view.default_scribe_width)
		end

	scribe_width: DOUBLE
			-- Width of element which is a scribe
		require
			is_scribe: is_scribe
		do
			if is_horizontal_scribe then
				Result := size_y
			else
				Result := size_x
			end
		end

	default_scribe_width: DOUBLE
			-- Default width of a scribe
		do
			Result := view.default_scribe_width
		end

	is_non_scribe: BOOLEAN
			-- Is element a die that is not a scribe line?
		do
			Result := is_die and not is_scribe
		end
	
	is_meetable: BOOLEAN
			-- Can scribe be extended to meet the edge of
			-- this element?
		do
			Result := is_scribe or is_field
		end
	
	is_movable_in_length_dimension: BOOLEAN
			-- Can scribe be extended to meet the edge of
			-- this element?
		require
			is_scribe: is_scribe
		do
			if is_horizontal_scribe then
				Result := is_movable_in_x
			else
				Result := is_movable_in_y
			end
		end
	
	is_filler: BOOLEAN
			-- Is element a filler?
		do
			Result := model_element.is_filler
		end

	is_vendor_geometry: BOOLEAN
			-- Is element a vendor geometry or ghost?
		do
			Result := model_element.is_vendor_geometry
		end

	is_below_minimum_size: BOOLEAN
			-- Is element size below the minimum allowed die size
			-- in either dimension?
		do
			Result := model_element.is_below_minimum_size
		end

	is_small_scribe_or_filler: BOOLEAN
			-- Is element a scribe or filler whose size is
			-- below the small object threshold in either
			-- dimension (just length for scribes)?
		do
			Result := model_element.is_small_scribe_or_filler
		end

	uniform_white_space_border: DOUBLE
			-- Uniform amount of white space around element
		do
			Result := model_element.uniform_white_space_border
		end

	minimum_separation: DOUBLE
			-- Minimum separation between object that is a
			-- project (contains circuitry) and other objects
			-- or field of view edge
		do
			Result := model_element.minimum_separation
		end

	needs_min_sep_white_space: BOOLEAN
			-- Does element need "minimum separation" amount of
			-- white space around it?
		do
			Result := model_element.needs_min_sep_white_space
		end

	minimum_gap: DOUBLE
			-- Gridded minimum gap needed between object and field
			-- of view edge or another object (zero gap is also
			-- allowed if not `needs_min_sep_white_space')
		do
			Result := model_element.minimum_gap
		end

	minimum_separation_from (an_element: RETICLE_VIEW_ELEMENT): DOUBLE
			-- Gridded minimum separation needed between `Current'
			-- and `an_element' (zero is also allowed if neither
			-- element `needs_min_sep_white_space')
		do
			Result := model_element.minimum_separation_from (an_element.model_element)
		end

	fields: LIST [RETICLE_VIEW_ELEMENT]
			-- List of possible fields for reticle
		do
			Result := view.fields
		end

	field: RETICLE_VIEW_ELEMENT
			-- Field of view with which element is associated
		local
			l_fields: like fields
			f: RETICLE_VIEW_ELEMENT
		do
			l_fields := fields
			from
				l_fields.start
			until
				l_fields.after or Result /= Void
			loop
				f := l_fields.item
				if f.model_element.field = model_element.field then
					Result := f
				end
				l_fields.forth
			end
		end

	reticle_field: RETICLE_FIELD
			-- Field of view with which element is associated,
			-- as a RETICLE_FIELD
		do
			Result := model_element.field
		end

	is_center_in_field: BOOLEAN
			-- Is center of element inside field of view associated
			-- with element (closest field)?
		do
			Result := model_element.is_center_in_field
		end

	is_center_x_in_field: BOOLEAN
			-- Is center of element in X inside field of
			-- view associated with element (closest
			-- field)?
		do
			Result := model_element.is_center_x_in_field
		end

	is_center_y_in_field: BOOLEAN
			-- Is center of element in Y inside field of
			-- view associated with element (closest
			-- field)?
		do
			Result := model_element.is_center_y_in_field
		end

	intersects_any_field: BOOLEAN
			-- Does element bounding box intersect any
			-- field of view?
		do
			Result := model_element.intersects_any_field
		end

	intersects_more_than_one_field: BOOLEAN
			-- Does element bounding box intersect more than one
			-- field of view?
		do
			Result := model_element.intersects_more_than_one_field
		end

	intersection_with_fields: LIST [RECTANGLE]
			-- Rectangles that represent intersection of element
			-- bounding box with fields of view
		do
			Result := model_element.intersection_with_fields
		end

	intersecting_fields: LIST [RETICLE_FIELD]
			-- Fields that element bounding box intersects
		do
			Result := model_element.intersecting_fields
		end

	enclosed_by_one_field: BOOLEAN
			-- Does element bounding box completely enclosed by
			-- exactly one field of view (edges may touch field
			-- edges)?
		do
			Result := model_element.enclosed_by_one_field
		end

	partially_outside_fields: BOOLEAN
			-- Is element bounding box partially outside fields
			-- of view (intersects at least one field but is not
			-- completely enclosed by any field)?
		do
			Result := model_element.partially_outside_fields
		end

	is_too_close_to_field_edge: BOOLEAN
			-- Is at least one edge of element too close to
			-- corresponding edge of its associated field?
		do
			Result := model_element.is_too_close_to_field_edge
		end

	is_edge_too_close_to_field_edge (dist: DOUBLE): BOOLEAN
			-- Is edge of element that is `dist' away from
			-- corresponding edge of its associated field
			-- too close?
		do
			Result := model_element.is_edge_too_close_to_field_edge (dist)
		end

	distance_in_x_from (an_element: RETICLE_VIEW_ELEMENT): DOUBLE
			-- Gridded distance of element from
			-- `an_element' in X dimension.  Result is
			-- zero if elements overlap in X
		do
			Result := model_element.distance_in_x_from (an_element.model_element)
		end

	distance_in_y_from (an_element: RETICLE_VIEW_ELEMENT): DOUBLE
			-- Gridded distance of element from
			-- `an_element' in Y dimension.  Result is
			-- zero if elements overlap in Y
		do
			Result := model_element.distance_in_y_from (an_element.model_element)
		end

	left_to_field_edge: DOUBLE
			-- Gridded distance from left edge of element to left
			-- edge of its associated field (positive if
			-- element edge is to right of field edge)
		do
			Result := model_element.left_to_field_edge
		end

	right_to_field_edge: DOUBLE
			-- Gridded distance from right edge of element to right
			-- edge of its associated field (positive if
			-- element edge is to left of field edge)
		do
			Result := model_element.right_to_field_edge
		end

	top_to_field_edge: DOUBLE
			-- Gridded distance from top edge of element to top
			-- edge of its associated field (positive if
			-- element edge is below field edge)
		do
			Result := model_element.top_to_field_edge
		end

	bottom_to_field_edge: DOUBLE
			-- Gridded distance from bottom edge of
			-- element to bottom edge of its associated
			-- field (positive if element edge is above
			-- field edge)
		do
			Result := model_element.bottom_to_field_edge
		end

	field_containing_top_edge: RETICLE_FIELD
			-- Field that truly contains at least part of
			-- top edge of element bounding box or Void if
			-- none
		do
			Result := model_element.field_containing_top_edge
		end

	field_containing_bottom_edge: RETICLE_FIELD
			-- Field that truly contains at least part of
			-- bottom edge of element bounding box or Void if
			-- none
		do
			Result := model_element.field_containing_bottom_edge
		end

	field_containing_left_edge: RETICLE_FIELD
			-- Field that truly contains at least part of
			-- left edge of element bounding box or Void if
			-- none
		do
			Result := model_element.field_containing_left_edge
		end

	field_containing_right_edge: RETICLE_FIELD
			-- Field that truly contains at least part of
			-- right edge of element bounding box or Void if
			-- none
		do
			Result := model_element.field_containing_right_edge
		end

	x_outside_all_fields (a_value: DOUBLE): BOOLEAN
			-- Is `a_value' outside all fields of view in X
			-- dimension or on field edge?
		do
			Result := model_element.x_outside_all_fields (a_value)
		end

	y_outside_all_fields (a_value: DOUBLE): BOOLEAN
			-- Is `a_value' outside all fields of view in Y
			-- dimension or on field edge?
		do
			Result := model_element.y_outside_all_fields (a_value)
		end

	closest_field_edge_above (a_value: DOUBLE): DOUBLE
			-- Y coordinate of field lower edge which is
			-- closest to and above or equal to `a_value'
		do
			Result := model_element.closest_field_edge_above (a_value)
		end

	closest_field_edge_below (a_value: DOUBLE): DOUBLE
			-- Y coordinate of field upper edge which is closest to
			-- and below or equal to `a_value'
		do
			Result := model_element.closest_field_edge_below (a_value)
		end

	closest_field_edge_to_left (a_value: DOUBLE): DOUBLE
			-- Y coordinate of field right edge which is
			-- closest to and left of or equal to
			-- `a_value'
		do
			Result := model_element.closest_field_edge_to_left (a_value)
		end

	closest_field_edge_to_right (a_value: DOUBLE): DOUBLE
			-- Y coordinate of field left edge which is
			-- closest to and right of or equal to
			-- `a_value'
		do
			Result := model_element.closest_field_edge_to_right (a_value)
		end

	intersects (an_element: like Current): BOOLEAN
			-- Is intersection of `an_element' and `Current'
			-- non-empty?
		do
			Result := model_bounding_box.intersects (an_element.model_bounding_box)
		end

	encloses (an_element: like Current): BOOLEAN
			-- Does model bounding box of `Current' enclose
			-- that of `an_element'?
		do
			Result := model_bounding_box.encloses (an_element.model_bounding_box)
		end

	model_bounding_box: CONST_RECTANGLE
			-- Bounding box of element in model (not in view)
		do
			Result := model_element.bounding_box
		end

	size_x: DOUBLE
			-- Size of associated model element in X dimension
		do
			Result := model_element.size_x
		end

	size_y: DOUBLE
			-- Size of associated model element in X dimension
		do
			Result := model_element.size_y
		end

	has_zero_size_in_x: BOOLEAN
			-- Does `Current' actually have zero size in X
			-- dimension?
		do
			Result := model_element.has_zero_size_in_x
		end
		
	has_zero_size_in_y: BOOLEAN
			-- Does `Current' actually have zero size in Y
			-- dimension?
		do
			Result := model_element.has_zero_size_in_y
		end
		
	has_size_x: BOOLEAN = True
			-- Does `Current' have an associated size in X
			-- dimension?

	has_size_y: BOOLEAN = True
			-- Does `Current' have an associated size in Y
			-- dimension?

	size_x_string: STRING
			-- Size of associated model element in X dimension
			-- as a string
		do
			Result := grid.formatted_number (size_x)
		end

	size_y_string: STRING
			-- Size of associated model element in Y dimension
			-- as a string
		do
			Result := grid.formatted_number (size_y)
		end

	gridded_size_x: DOUBLE
			-- Size of associated model element in X dimension,
			-- rounded to grid
		do
			Result := grid.rounded_size (size_x)
		end

	gridded_size_y: DOUBLE
			-- Size of associated model element in X dimension,
			-- rounded to grid
		do
			Result := grid.rounded_size (size_y)
		end

	has_gridded_size_x: BOOLEAN = True
			-- Does `Current' have an associated gridded size in X
			-- dimension?

	has_gridded_size_y: BOOLEAN = True
			-- Does `Current' have an associated gridded size in Y
			-- dimension?

	gridded_size_x_string: STRING
			-- Gridded size of associated model element in
			-- X dimension as a string
		do
			Result := grid.formatted_number (gridded_size_x)
		end

	gridded_size_y_string: STRING
			-- Gridded size of associated model element in
			-- Y dimension as a string
		do
			Result := grid.formatted_number (gridded_size_y)
		end

	slop_x: DOUBLE
			-- Difference between maximum die size and minimum
			-- die size in X dimension
		require
			has_max_die_size: has_max_die_size
		do
			Result := max_die_size.x - size_x
		end

	slop_y: DOUBLE
			-- Difference between maximum die size and minimum
			-- die size in X dimension
		require
			has_max_die_size: has_max_die_size
		do
			Result := max_die_size.y - size_y
		end

	left_x: DOUBLE
			-- Left edge of minimum bounding box of associated
			-- model element in X dimension
		do
			Result := model_element.left_x
		end

	right_x: DOUBLE
			-- Right edge of minimum bounding box of associated
			-- model element in X dimension
		do
			Result := model_element.right_x
		end

	top_y: DOUBLE
			-- Top edge of minimum bounding box of associated
			-- model element in Y dimension
		do
			Result := model_element.top_y
		end

	bottom_y: DOUBLE
			-- Bottom edge of minimum bounding box of associated
			-- model element in X dimension
		do
			Result := model_element.bottom_y
		end

	gridded_left_x: DOUBLE
			-- Left edge of minimum bounding box of associated
			-- model element in X dimension, placed on grid
		do
			Result := grid.rounded_edge (left_x)
		end

	gridded_right_x: DOUBLE
			-- Right edge of minimum bounding box of associated
			-- model element in X dimension, placed on grid
		do
			Result := grid.rounded_edge (right_x)
		end

	gridded_top_y: DOUBLE
			-- Top edge of minimum bounding box of associated
			-- model element in Y dimension, placed on grid
		do
			Result := grid.rounded_edge (top_y)
		end

	gridded_bottom_y: DOUBLE
			-- Bottom edge of minimum bounding box of associated
			-- model element in X dimension, placed on grid
		do
			Result := grid.rounded_edge (bottom_y)
		end

	center_x: DOUBLE
			-- Center of minimum bounding box of associated
			-- model element in X dimension
		do
			Result := model_element.center_x
		end

	center_y: DOUBLE
			-- Center of minimum bounding box of associated
			-- model element in Y dimension
		do
			Result := model_element.center_y
		end

	field_relative_center_x: DOUBLE
			-- Center of minimum bounding box in X dimension,
			-- relative to center of field with which element
			-- is associated
		do
			Result := model_element.field_relative_center_x
		end

	field_relative_center_y: DOUBLE
			-- Center of minimum bounding box in Y dimension,
			-- relative to center of field with which element
			-- is associated
		do
			Result := model_element.field_relative_center_y
		end

	field_relative_left_x: DOUBLE
			-- Left edge of minimum bounding box in X dimension,
			-- relative to center of field with which element
			-- is associated
		do
			Result := model_element.field_relative_left_x
		end

	field_relative_right_x: DOUBLE
			-- Right edge of minimum bounding box in X dimension,
			-- relative to center of field with which element
			-- is associated
		do
			Result := model_element.field_relative_right_x
		end

	field_relative_top_y: DOUBLE
			-- Top edge of minimum bounding box in X dimension,
			-- relative to center of field with which element
			-- is associated
		do
			Result := model_element.field_relative_top_y
		end

	field_relative_bottom_y: DOUBLE
			-- Bottom edge of minimum bounding box in X dimension,
			-- relative to center of field with which element
			-- is associated
		do
			Result := model_element.field_relative_bottom_y
		end

	has_left_x: BOOLEAN = True
			-- Does `Current' have an associated left edge in X
			-- dimension?

	has_bottom_y: BOOLEAN = True
			-- Does `Current' have an associated bottom edge in Y
			-- dimension?

	has_center_x: BOOLEAN = True
			-- Does `Current' have an associated center in X
			-- dimension?

	has_center_y: BOOLEAN = True
			-- Does `Current' have an associated center in Y
			-- dimension?

	left_x_string: STRING
			-- Left edge of associated model element in X dimension
			-- as a string
		do
			Result := grid.formatted_number (left_x)
		end

	bottom_y_string: STRING
			-- Bottom edge of associated model element in
			-- Y dimension as a string
		do
			Result := grid.formatted_number (bottom_y)
		end

	center_x_string: STRING
			-- Center of associated model element in X dimension
			-- as a string
		do
			Result := grid.formatted_number (center_x)
		end

	center_y_string: STRING
			-- Center of associated model element in Y dimension
			-- as a string
		do
			Result := grid.formatted_number (center_y)
		end

	gridded_center_x: DOUBLE
			-- Center of minimum bounding box of associated
			-- model element in X dimension, placed on grid
		do
			Result := grid.rounded_center (center_x)
		end

	gridded_center_y: DOUBLE
			-- Center of minimum bounding box of associated
			-- model element in Y dimension, placed on grid
		do
			Result := grid.rounded_center (center_y)
		end

	has_gridded_center_x: BOOLEAN = True
			-- Does `Current' have an associated gridded
			-- center in X dimension?

	has_gridded_center_y: BOOLEAN = True
			-- Does `Current' have an associated gridded center
			-- in Y dimension?

	gridded_center_x_string: STRING
			-- Gridded center of associated model element
			-- in X dimension as a string
		do
			Result := grid.formatted_number (gridded_center_x)
		end

	gridded_center_y_string: STRING
			-- Gridded center of associated model element
			-- in Y dimension as a string
		do
			Result := grid.formatted_number (gridded_center_y)
		end

	has_fixed_in_x: BOOLEAN = True
			-- Does `Current' have an associated "fixed in X"
			-- attribute?

	has_fixed_in_y: BOOLEAN = True
			-- Does `Current' have an associated "fixed in Y"
			-- attribute?

	has_max_die_size: BOOLEAN
			-- Does `Current' have a maximum die size?
		do
			Result := max_die_size /= Void
		end

	max_die_size: CONST_POINT
			-- Maximum die size, as constrained by package
			-- cavity sizes and user-specified limits.
			-- Void indicates no maximum
		require
			is_die: is_die
		do
			Result := model_element.max_die_size
		end

	is_oversize_die_always_allowed: BOOLEAN
			-- Is a die that is bigger than `max_die_size'
			-- allowed unconditionally?
		require
			is_die: is_die
		do
			Result := model_element.is_oversize_die_always_allowed
		end

	is_oversize_die_possibly_allowed: BOOLEAN
			-- Is a die that is bigger than `max_die_size'
			-- allowed but only under specific conditions?
		require
			is_die: is_die
		do
			Result := model_element.is_oversize_die_possibly_allowed
		end

	oversize_die_inclusion_conditions: STRING
			-- Description of conditions under which 
			-- die that is bigger than `max_die_size'
			-- allowed
		require
			is_die: is_die
			oversize_die_possibly_allowed: is_oversize_die_possibly_allowed
		do
			Result := model_element.oversize_die_inclusion_conditions
		end

	has_sub_elements: BOOLEAN
			-- Is element composed of more than one sub-element,
			-- each of which has its own size and center?
		do
			Result := model_element.has_sub_elements
		end

	sub_elements: LIST [like anchored_view_sub_element]
			-- Sub-elements of `Current'

	packing_name: STRING
			-- Packing name for element
		do
			Result := model_element.packing_name
		end

	has_packing_name: BOOLEAN = True
			-- Does `Current' have an associated packing name?

	die: PACKING_DIES_MROW
			-- Die for element that is a die
		require
			is_die: is_die
		do
			Result := model_element.die
		end

	die_location: PACKING_DIE_LOCATIONS_MROW
			-- Die location for element that is a die
		require
			is_die: is_die
		do
			Result := model_element.die_location
		end

	project: PACKING_PROJECTS_MROW
			-- Project for element that is a project
		require
			is_project: is_project
		do
			Result := model_element.project
		end

	all_copies: RETICLE_VIEW_ELEMENT_LIST
			-- All copies of this element

	all_copies_selected: BOOLEAN
			-- Are all copies of element currently selected?
		do
			Result := all_copies.all_elements_selected
		end

	any_copies_selected: BOOLEAN
			-- Are any copies of element currently selected?
		do
			Result := all_copies.any_elements_selected
		end

	is_partially_selected: BOOLEAN
			-- Is element partially selected (some but not all
			-- copies selected)?
		do
			Result := any_copies_selected and not all_copies_selected
		end

	intersects_area (a_rectangle: EV_RECTANGLE): BOOLEAN
			-- Does element intersect `a_rectangle'?
		do
			if has_sub_elements then
				from
					sub_elements.start
				until
					sub_elements.after
				loop
					if sub_elements.item.intersects_area (a_rectangle) then
						Result := True
					end
					sub_elements.forth
				end
			else
				Result := precursor (a_rectangle)
			end
		end

	enclosed_by_area (a_rectangle: EV_RECTANGLE): BOOLEAN
			-- Is element enclosed by `a_rectangle'?
		do
			if has_sub_elements then
				from
					sub_elements.start
				until
					sub_elements.after
				loop
					if sub_elements.item.enclosed_by_area (a_rectangle) then
						Result := True
					end
					sub_elements.forth
				end
			else
				Result := precursor (a_rectangle)
			end
		end

	contains_point (a_coordinate: EV_COORDINATE): BOOLEAN
			-- Does element contain `a_coordinate'?
		do
			if has_sub_elements then
				from
					sub_elements.start
				until
					sub_elements.after
				loop
					if sub_elements.item.contains_point (a_coordinate) then
						Result := True
					end
					sub_elements.forth
				end
			else
				Result := precursor (a_coordinate)
			end
		end

	any_copies_contain (a_coordinate: EV_COORDINATE): BOOLEAN
			-- Do any copies of element contain `a_coordinate'?
		require
			a_coordinate_not_void: a_coordinate /= Void
		do
			Result := all_copies.any_elements_contain (a_coordinate)
		end

	any_copies_stationary_orientation: BOOLEAN
			-- Do any copies of element have a stationary
			-- orientation, which cannot be changed
		do
			Result := all_copies.any_elements_stationary_orientation
		end

	any_copies_rotation_locked: BOOLEAN
			-- Are any copies of element locked to prevent
			-- changed of orientation?
		do
			Result := all_copies.any_elements_rotation_locked
		end

	is_processed: BOOLEAN
			-- Have any copies of element been processed?
		do
			Result := all_copies.is_processed
		end

	participates_in_measure: BOOLEAN
			-- Does `Current' participate in measure in X/Y
			-- operations?
		do
			Result := model_element.participates_in_measure
		end

	is_misaligned_with (an_element: RETICLE_VIEW_ELEMENT): BOOLEAN
			-- Is element misaligned with `an_element'?
		require
			is_scribe: is_scribe
			an_element_is_scribe: an_element.is_scribe
			same_orientation_if_horizontal: is_horizontal_scribe implies an_element.is_horizontal_scribe
			same_orientation_if_vertical: is_vertical_scribe implies an_element.is_vertical_scribe
		local
			low, high, other_low, other_high: DOUBLE
		do
			if is_horizontal_scribe then
				low := gridded_bottom_y
				high := gridded_top_y
				other_low := an_element.gridded_bottom_y
				other_high := an_element.gridded_top_y
			else
				low := gridded_left_x
				high := gridded_right_x
				other_low := an_element.gridded_left_x
				other_high := an_element.gridded_right_x
			end
			Result := low <= other_high and high >= other_low and (low /= other_low or high /= other_high)
		end

	misalignment_amount (an_element: RETICLE_VIEW_ELEMENT): DOUBLE
			-- Amount by which `an_element' is misaligned with
			-- `Current'
		require
			is_scribe: is_scribe
			an_element_is_scribe: an_element.is_scribe
			same_orientation_if_horizontal: is_horizontal_scribe implies an_element.is_horizontal_scribe
			same_orientation_if_vertical: is_vertical_scribe implies an_element.is_vertical_scribe
		local
			low, high, other_low, other_high: DOUBLE
			low_diff, high_diff: DOUBLE
		do
			if is_horizontal_scribe then
				low := gridded_bottom_y
				high := gridded_top_y
				other_low := an_element.gridded_bottom_y
				other_high := an_element.gridded_top_y
			else
				low := gridded_left_x
				high := gridded_right_x
				other_low := an_element.gridded_left_x
				other_high := an_element.gridded_right_x
			end
			low_diff := grid.rounded_edge ((low - other_low).abs)
			high_diff := grid.rounded_edge ((high - other_high).abs)
			if low_diff = 0.0 then
				Result := high_diff
			elseif high_diff = 0.0 then
				Result := low_diff
			else
				Result := low_diff.min (high_diff)
			end
		end

	is_marked: BOOLEAN
			-- Has element been marked as being special in some
			-- way?  Meaning of marked depends on caller

	invalid_elements: LIST [RETICLE_VIEW_ELEMENT]
			-- List of invalid elements, with which `Current'
			-- has some kind of validity problem, such as
			-- overlap.  List is used to avoid issuing duplicate
			-- error messages for objects that fall into more
			-- than one area and hence may be compared more than
			-- once with `Current' for validity problems

	scribe_cut_die: CONST_RECTANGLE
			-- Last computed cut die for a project, using
			-- centers of surrounding scribes for edges of cut die
		do
			Result := model_element.scribe_cut_die
		end

	oriented_scribe_cut_die: CONST_RECTANGLE
			-- Last computed cut die for a project,
			-- oriented the same as the die (i.e., rotated
			-- if die is rotated), using centers of
			-- surrounding scribes for edges of cut die
		do
			Result := model_element.oriented_scribe_cut_die
		end

	scribe_cut_die_size_x: DOUBLE
			-- Size of cut die in X dimension
		do
			Result := model_element.scribe_cut_die_size_x
		end

	scribe_cut_die_size_y: DOUBLE
			-- Size of cut die in Y dimension
		do
			Result := model_element.scribe_cut_die_size_y
		end

	edge_cut_die: CONST_RECTANGLE
			-- Last computed cut die for a project, using
			-- closest edges of surrounding scribes for edges
			-- of cut die
		do
			Result := model_element.edge_cut_die
		end

	cut_cut_die: CONST_RECTANGLE
			-- Last computed cut die for a project, using
			-- cuts for edges of cut die
		do
			Result := model_element.cut_cut_die
		end

	cut_or_scribe_cut_die: CONST_RECTANGLE
			-- Last computed cut die for a project, using
			-- `cut_cut_die' if not Void and otherwise
			-- `scribe_cut_die'
		do
			Result := cut_cut_die
			if Result = Void then
				Result := scribe_cut_die
			end
		end

	cut_cut_die_size_x: DOUBLE
			-- Size of cut die in X dimension, using centers
			-- of surrounding cuts for edges of cut die
		do
			Result := model_element.cut_cut_die_size_x
		end

	cut_cut_die_size_y: DOUBLE
			-- Size of cut die in Y dimension, using centers
			-- of surrounding cut for edges of cut die
		do
			Result := model_element.cut_cut_die_size_y
		end

	has_surrounding_cuts: BOOLEAN
			-- Were there cuts on all four sides of project on
			-- last computation of cut die

	isolated_cut_die: CONST_RECTANGLE
			-- Cut die for an isolated project, determined
			-- from its cut die site
		do
			Result := view.current_cut_pattern.isolated_cut_die (Current)
		end

	is_in_current_cut_pattern_lot: BOOLEAN
			-- Does `Current' have the same lot (same 
			-- wafer_etest_name) as the lot for the current cut
			-- pattern?
		do
			Result := not has_wafer_etest_id or else view.current_cut_pattern.wafer_etest_name.is_equal (wafer_etest_name)
		end

	is_floating_scribe: BOOLEAN
			-- Is element a floating scribe?  Only valid
			-- after being computed for certain validity
			-- check operations

	is_misaligned_scribe: BOOLEAN
			-- Is element a scribe misaligned with another
			-- scribe of the same orientation (both must
			-- intersect at least one field of view)?  Value
			-- is the one computed for last validity check
			-- operation that checked for misaligned scribes

	last_duplicated_element: RETICLE_VIEW_ELEMENT
			-- Last new element create by call to `duplicate'

	center_fits_in_database (a_center: CONST_POINT): BOOLEAN
			-- Will model location `a_center' fit in database
			-- (in PACKING_DIE_LOCATIONS table)?  `a_center'
			-- is a location in model coordinates, not a
			-- field-relative center
		require
			a_center_not_void: a_center /= Void
		do
			Result := view.center_fits_in_database (a_center)
		end

	intersecting_mbb_areas: LIST [RETICLE_MODEL_AREA]
			-- Areas of model this element's bounding box
			-- intersects.  Used
			-- to speed up validity checking

	intersecting_scribe_cut_die_areas: LIST [RETICLE_MODEL_AREA]
			-- Areas of model this element's cut die
			-- (determined by surrounding scribes)
			-- intersects.  Used to speed up validity
			-- checking

	intersecting_cut_cut_die_areas: LIST [RETICLE_MODEL_AREA]
			-- Areas of model this element's cut die
			-- (determined by surrounding cuts)
			-- intersects.  Used to speed up validity
			-- checking

	intersecting_hscribe_areas: LIST [RETICLE_MODEL_AREA]
			-- Areas of model this horizontal scribe's
			-- bounding box intersects.  Used to speed up
			-- validity checking

	intersecting_vscribe_areas: LIST [RETICLE_MODEL_AREA]
			-- Areas of model this vertical scribe's
			-- bounding box intersects.  Used to speed up
			-- validity checking

	intersecting_scribe_areas: LIST [RETICLE_MODEL_AREA]
			-- Areas of model this scribe's bounding box
			-- intersects
		require
			is_scribe: is_scribe
		do
			if is_horizontal_scribe then
				Result := intersecting_hscribe_areas
			else
				Result := intersecting_vscribe_areas
			end
		end

	is_overlapping_another_object: BOOLEAN
			-- Does element overlap another element (both must
			-- intersect at least one field of view)?  Value
			-- is the one computed for last validity check
			-- operation that checked for overlapping objects

	is_too_close_to_another_object: BOOLEAN
			-- Is element too close to another element but
			-- not overlapping it (both must intersect at
			-- least one field of view)?  Value is the one
			-- computed for last validity check operation
			-- that checked for "too close" objects

	is_unisolated_in_scribe_cut_die: BOOLEAN
			-- Is element not isolated in its cut die determined
			-- by surrounding scribes?
			-- Value is the one computed for last validity
			-- check operation that checked for unisolated
			-- projects

	is_unisolated_in_cut_cut_die: BOOLEAN
			-- Is element not isolated in its cut die determined
			-- by surrounding cuts?
			-- Value is the one computed for last validity
			-- check operation that checked for unisolated
			-- projects

	is_isolating_cut: BOOLEAN
			-- Is element a cut that helps isolate at least one
			-- isolate site?

	misaligned_with_other_scribe: BOOLEAN
			-- Is element which is a scribe misaligned with
			-- another scribe of the same orientation?
		require
			is_scribe: is_scribe
		local
			areas: LIST [RETICLE_MODEL_AREA]
			area: RETICLE_MODEL_AREA
			elems: LIST [RETICLE_VIEW_ELEMENT]
			elem: RETICLE_VIEW_ELEMENT
		do
			if intersects_any_field then
				areas := intersecting_scribe_areas
				from
					areas.start
				until
					areas.after or Result
				loop
					area := areas.item
					elems := area.intersecting_elements
					from
						elems.start
					until
						elems.after or Result
					loop
						elem := elems.item
						if elem /= Current and is_misaligned_with (elem) then
							Result := True
						end
						elems.forth
					end
					areas.forth
				end
			end
		end

	overlaps_other_object: BOOLEAN
			-- Does element overlap another object?  Considers
			-- only dies
		local
			areas: LIST [RETICLE_MODEL_AREA]
			mbb: CONST_RECTANGLE
			area: RETICLE_MODEL_AREA
			elems: LIST [RETICLE_VIEW_ELEMENT]
			elem: RETICLE_VIEW_ELEMENT
		do
			if intersects_any_field then
				areas := intersecting_mbb_areas
				mbb := model_bounding_box
				from
					areas.start
				until
					areas.after or Result
				loop
					area := areas.item
					elems := area.intersecting_elements
					from
						elems.start
					until
						elems.after or Result
					loop
						elem := elems.item
						if elem /= Current and then mbb.intersects (elem.model_bounding_box) then
							Result := True
						end
						elems.forth
					end
					areas.forth
				end
			end
		end

	is_too_close_to_object: BOOLEAN
			-- Is element too close to another object?  Considers
			-- only dies
		local
			areas: LIST [RETICLE_MODEL_AREA]
			mbb: CONST_RECTANGLE
			bigger_mbb, other_mbb: RECTANGLE
			gap, other_grow, dist_x, dist_y: DOUBLE
			area: RETICLE_MODEL_AREA
			elems: LIST [RETICLE_VIEW_ELEMENT]
			elem: RETICLE_VIEW_ELEMENT
		do
			if intersects_any_field then
				areas := intersecting_mbb_areas
				gap := minimum_gap
				if gap > 0.0 then
					mbb := model_bounding_box
					create bigger_mbb.make_copy (mbb)
					bigger_mbb.size_by (2.0 * gap, 2.0 * gap)
					from
						areas.start
					until
						areas.after or Result
					loop
						area := areas.item
						elems := area.intersecting_elements
						from
							elems.start
						until
							elems.after or Result
						loop
							elem := elems.item
							create other_mbb.make_copy (elem.model_bounding_box)
							if elem /= Current and then not mbb.intersects (other_mbb) then
								other_grow := -2.0 * elem.uniform_white_space_border
								other_mbb.size_by (other_grow, other_grow)
								dist_x := distance_in_x_from (elem)
								dist_y := distance_in_y_from (elem)
								Result := bigger_mbb.intersects (other_mbb) and (needs_min_sep_white_space or elem.needs_min_sep_white_space or dist_x /= 0.0 or dist_y /= 0.0)
							end
							elems.forth
						end
						areas.forth
					end
				end
			end
		end

	is_scribe_cut_die_too_big: BOOLEAN
			-- Does die size of element determined by surrounding
			-- scribes exceed its maximum size?
		local
			l_max: CONST_POINT
			sx, sy: DOUBLE
		do
			if is_die then
				l_max := max_die_size
				if l_max /= Void and scribe_cut_die /= Void then
					sx := grid.rounded_size (scribe_cut_die_size_x)
					sy := grid.rounded_size (scribe_cut_die_size_y)
					Result := sx > l_max.x or sy > l_max.y
				end
			end
		end

	is_cut_cut_die_too_big: BOOLEAN
			-- Does die size of element determined by surrounding
			-- cuts exceed its maximum size?
		local
			l_max: CONST_POINT
			sx, sy: DOUBLE
		do
			if is_isolated_in_current_cut_pattern then
				l_max := max_die_size
				if l_max /= Void and cut_cut_die /= Void then
					sx := grid.rounded_center (cut_cut_die_size_x)
					sy := grid.rounded_center (cut_cut_die_size_y)
					Result := sx > l_max.x or sy > l_max.y
				end
			end
		end

	is_cut_cut_die_invalid: BOOLEAN
			-- Is die size of element determined by surrounding
			-- cuts invalid?
		do
			Result := cut_cut_die = Void and is_isolated_in_current_cut_pattern 
		end

	has_wrong_lot_for_cut_pattern: BOOLEAN
			-- Is element one that is isolated by cut pattern but 
			-- has an assigned lot that is different from the
			-- one for the cut pattern?
		do
			Result := view.current_cut_pattern.has_wafer_etest_name and then not is_in_current_cut_pattern_lot and is_isolated_in_current_cut_pattern
		end

	has_non_default_cut_die_size: BOOLEAN
			-- Is element one that is isolated by cut pattern and
			-- has a cut die size determined by surrounding cuts
			-- that is different from the cut die size determined
			-- by surrounding scribes?
		local
			l_cut_cut_die: like cut_cut_die
		do
			l_cut_cut_die := cut_cut_die
			Result := l_cut_cut_die /= Void and then not l_cut_cut_die.is_same_box (scribe_cut_die) and is_isolated_in_current_cut_pattern
		end

	shares_cut_die_with_others: BOOLEAN
			-- Is element one that is isolated by cut pattern and
			-- shares its cut die determined by surrounding cuts
			-- either partially or fully with another non-scribe
			-- die?
		do
			Result := is_unisolated_in_cut_cut_die
		end

	is_extra_cut: BOOLEAN
			-- Is element an extra cut that does not help isolate
			-- any isolated sites?
		do
			Result := is_cut and not is_isolating_cut
		end

	offset_cut_center_x: DOUBLE
			-- Offset of center of die from center of cut die
			-- in X dimension, using closest edges of surrounding
			-- scribes instead of centers
		require
			scribe_cut_die_not_void: scribe_cut_die /= Void
		do
			Result := model_element.offset_cut_center_x
		end

	offset_cut_center_y: DOUBLE
			-- Offset of center of die from center of cut die
			-- in X dimension, using closest edges of surrounding
			-- scribes instead of centers
		require
			scribe_cut_die_not_void: scribe_cut_die /= Void
		do
			Result := model_element.offset_cut_center_y
		end

	is_uncentered_in_cut_die: BOOLEAN
			-- Is element not centered in its cut die (using
			-- closest edges of surrounding scribes, instead
			-- of centers)?
		local
			l_cut_die: like edge_cut_die
			diff_x, diff_y: DOUBLE
		do
			l_cut_die := edge_cut_die
			if is_project and l_cut_die /= Void then
				diff_x := (grid.rounded_edge (left_x - l_cut_die.low_x) - grid.rounded_edge (l_cut_die.high_x - right_x)).abs
				diff_x := grid.rounded_edge (diff_x)
				diff_y := (grid.rounded_edge (bottom_y - l_cut_die.low_y) - grid.rounded_edge (l_cut_die.high_y - top_y)).abs
				diff_y := grid.rounded_edge (diff_y)
				Result := diff_x > grid.edge_resolution or diff_y > grid.edge_resolution
			end
		end

	unisolated_in_scribe_cut_die: BOOLEAN
			-- Is element not isolated in its cut die, determined
			-- by surrounding scribes?
		local
			areas: LIST [RETICLE_MODEL_AREA]
			mbb: CONST_RECTANGLE
			area: RETICLE_MODEL_AREA
			elems: LIST [RETICLE_VIEW_ELEMENT]
			elem: RETICLE_VIEW_ELEMENT
		do
			if is_project and intersects_any_field then
				areas := intersecting_scribe_cut_die_areas
				mbb := scribe_cut_die
				from
					areas.start
				until
					areas.after or Result
				loop
					area := areas.item
					elems := area.intersecting_elements
					from
						elems.start
					until
						elems.after or Result
					loop
						elem := elems.item
						if elem /= Current and then mbb.intersects (elem.scribe_cut_die) then
							Result := True
						end
						elems.forth
					end
					areas.forth
				end
			end
		end

	unisolated_in_cut_cut_die: BOOLEAN
			-- Does element share its cut die, determined
			-- by surrounding cuts, with other non-scribe
			-- dies?
		local
			areas: LIST [RETICLE_MODEL_AREA]
			mbb: CONST_RECTANGLE
			area: RETICLE_MODEL_AREA
			elems: LIST [RETICLE_VIEW_ELEMENT]
			elem: RETICLE_VIEW_ELEMENT
		do
			if can_be_isolated then
				areas := intersecting_cut_cut_die_areas
				mbb := cut_cut_die
				if mbb /= Void then
					from
						areas.start
					until
						areas.after or Result
					loop
						area := areas.item
						elems := area.intersecting_elements
						from
							elems.start
						until
							elems.after or Result
						loop
							elem := elems.item
							if elem /= Current and then mbb.intersects (elem.scribe_cut_die) then
								Result := True
							end
							elems.forth
						end
						areas.forth
					end
				end
			end
		end

	can_be_isolated: BOOLEAN
			-- Is element allowed to be isolated, based on its
			-- type?
		do
			Result := model_element.can_be_isolated
		end

	is_isolated_in_any_cut_pattern: BOOLEAN
			-- Is element isolated in any cut pattern?
		do
			Result := can_be_isolated and view.is_isolated_in_any_cut_pattern (Current)
		end

	is_isolated_in_current_cut_pattern: BOOLEAN
			-- Is element isolated in current cut pattern?
		do
			Result := can_be_isolated and view.is_isolated_in_current_cut_pattern (Current)
		end

	appears_isolated: BOOLEAN
			-- Does element appear to be isolated in current 
			-- cut pattern (is scribe cut die same as cut cut die)?
		local
			l_cut_cut_die: like cut_cut_die
		do
			l_cut_cut_die := cut_cut_die
			Result := l_cut_cut_die /= Void and then l_cut_cut_die.is_same_box (scribe_cut_die)
		end

	is_reworkable: BOOLEAN
			-- Is element currently reworkable?
		do
			Result := is_cust_project and then model_element.is_reworkable
		end

feature -- Modification

	set_model_element (a_model_element: like model_element)
			-- Set `model_element' to `a_model_element'
		require
			a_model_element_not_void: a_model_element /= Void
		do
			model_element := a_model_element
		ensure
			model_element_set: model_element = a_model_element
		end

	set_label_generator (a_generator: like label_generator)
			-- Set `label_generator' to `a_generator'.  Set
			-- `is_label_generator_changed' to indicate whether
			-- the label generator changed
		require
			a_generator_not_void: a_generator /= Void
		do
			is_label_generator_changed := label_generator /= a_generator
			label_generator := a_generator
		ensure
			label_generator_set: label_generator = a_generator
		end

	set_color_generator (a_generator: like color_generator)
			-- Set `color_generator' to `a_generator'.  Set
			-- `is_color_generator_changed' to indicate whether
			-- the color generator changed
		require
			a_generator_not_void: a_generator /= Void
		do
			is_color_generator_changed := color_generator /= a_generator
			color_generator := a_generator
		ensure
			color_generator_set: color_generator = a_generator
		end

	set_fill_generator (a_generator: like fill_generator)
			-- Set `fill_generator' to `a_generator'.  Set
			-- `is_fill_generator_changed' to indicate whether
			-- the fill generator changed
		require
			a_generator_not_void: a_generator /= Void
		do
			is_fill_generator_changed := fill_generator /= a_generator
			fill_generator := a_generator
		ensure
			fill_generator_set: fill_generator = a_generator
		end

	set_opacity_generator (a_generator: like opacity_generator)
			-- Set `opacity_generator' to `a_generator'.  Set
			-- `is_opacity_generator_changed' to indicate whether
			-- the opacity generator changed
		require
			a_generator_not_void: a_generator /= Void
		do
			is_opacity_generator_changed := opacity_generator /= a_generator
			opacity_generator := a_generator
		ensure
			opacity_generator_set: opacity_generator = a_generator
		end

	set_is_selected (b: BOOLEAN)
			-- Set `is_selected' to `b'.  Set `is_selected_changed'
			-- to indicate whether the value of `is_selected'
			-- was changed
		require
			is_selectable: is_selectable
		do
			is_selected_changed := is_selected /= b
			is_selected := b
		ensure
			value_set: is_selected = b
		end

	set_included_in_changed_region (b: BOOLEAN)
			-- Set `included_in_changed_region' to `b'
		do
			included_in_changed_region := b
		ensure
			value_set: included_in_changed_region = b
		end

	set_matched_last_search (b: BOOLEAN)
			-- Set `matched_last_search' to `b'
		do
			matched_last_search_changed := matched_last_search /= b
			matched_last_search := b
		ensure
			value_set: matched_last_search = b
		end

	set_all_copies (a_list: like all_copies)
			-- Set `all_copies' to `a_list'
		require
			a_list_not_void: a_list /= Void
		do
			all_copies := a_list
		ensure
			all_copies_set: all_copies = a_list
		end

	remove_from_all_copies
			-- Remove `Current' from `all_copies'
		do
			all_copies.remove (Current)
		end

	set_is_overlapping_another_object (b: BOOLEAN)
			-- Set `is_overlapping_another_object' to `b'
		do
			is_overlapping_another_object := b
		end

	set_is_too_close_to_another_object (b: BOOLEAN)
			-- Set `is_too_close_to_another_object' to `b'
		do
			is_too_close_to_another_object := b
		end

	set_is_unisolated_in_scribe_cut_die (b: BOOLEAN)
			-- Set `is_unisolated_in_scribe_cut_die' to `b'
		do
			is_unisolated_in_scribe_cut_die := b
		end

	set_is_unisolated_in_cut_cut_die (b: BOOLEAN)
			-- Set `is_unisolated_in_cut_cut_die' to `b'
		do
			is_unisolated_in_cut_cut_die := b
		end

	set_is_isolating_cut (b: BOOLEAN)
			-- Set `is_isolating_cut' to `b'
		do
			is_isolating_cut := b
		end

	set_scribe_cut_die (a_cut_die: like scribe_cut_die)
			-- Set `scribe_cut_die' to `a_cut_die'
		do
			model_element.set_scribe_cut_die (a_cut_die)
		end

	set_edge_cut_die (a_cut_die: like edge_cut_die)
			-- Set `edge_cut_die' to `a_cut_die'
		do
			model_element.set_edge_cut_die (a_cut_die)
		end

	set_cut_cut_die (a_cut_die: like cut_cut_die)
			-- Set `cut_cut_die' to `a_cut_die'
		do
			model_element.set_cut_cut_die (a_cut_die)
		end

	set_has_surrounding_cuts (a_value: like has_surrounding_cuts)
			-- Set `has_surrounding_cuts' to `a_value'
		do
			has_surrounding_cuts := a_value
		ensure
			has_surrounding_cuts_set: has_surrounding_cuts = a_value
		end

feature -- Operations

	set_view_coordinates
			-- Set `width', `height', `x', `y' and
			-- `bounding_box' based on size, location and
			-- rotation of `model_element' and its
			-- position on `view'
		do
			if has_sub_elements then
				from
					sub_elements.start
				until
					sub_elements.after
				loop
					sub_elements.item.set_view_coordinates
					sub_elements.forth
				end
			end
			view.set_view_rectangle_from_model_rectangle (bounding_box, Maximum_view_coordinate.as_integer_32, model_element.left_x, model_element.top_y, model_element.right_x, model_element.bottom_y)
			unload_view_coordinates
		end

	draw_without_fill (a_drawable: EV_DRAWABLE a_palette: VIEW_COLORS)
			-- Draw `Current' onto `a_drawable' using
			-- `a_palette', not including any needed interior fill
		do
			if has_sub_elements then
				from
					sub_elements.start
				until
					sub_elements.after
				loop
					sub_elements.item.draw_without_fill (a_drawable, a_palette)
					sub_elements.forth
				end
			else
				precursor (a_drawable, a_palette)
			end
		end

	draw_fill_only (a_drawable: EV_DRAWABLE a_palette: VIEW_COLORS)
			-- Draw only any needed interior fill for
			-- `Current' onto `a_drawable' using
			-- `a_palette'
		do
			if has_sub_elements then
				from
					sub_elements.start
				until
					sub_elements.after
				loop
					sub_elements.item.draw_fill_only (a_drawable, a_palette)
					sub_elements.forth
				end
			else
				precursor (a_drawable, a_palette)
			end
		end

	set_selected (a_new_value, a_filter_matches: BOOLEAN)
			-- Set "selected" status of `Current' to
			-- `a_new_value' if `a_filter_matches' and
			-- item is selectable.  Update changed region
			-- and add `Current' to appropriate temporary
			-- list for maintaining element stacking order
		do
			if is_selectable and a_filter_matches then
				set_is_selected (a_new_value)
				add_to_region_if_selected_changed
			end
			if not is_selectable then
				unselectable_elements.extend (Current)
			elseif not is_selected then
				unselected_elements.extend (Current)
			elseif a_filter_matches then
				newly_selected_elements.extend (Current)
			else
				previously_selected_elements.extend (Current)
			end
		end

	add_to_new_element_list
			-- Add new element `Current' to list of new
			-- elements to be added to the view
		do
			view.add_to_new_element_list (Current)
		end

	add_to_region_if_selected_changed
			-- Add `Current' to the region to be updated if
			-- its "selected" status changed
		do
			if is_selected_changed then
				add_to_changed_region
			end
		end

	add_to_changed_region
			-- Add rectangle on canvas for element's bounding
			-- box to changed region, if it is not already there
		do
			if not included_in_changed_region then
				view.add_to_changed_region (bounding_box)
				included_in_changed_region := True
			end
		end

	add_old_and_new_to_changed_region
			-- Add rectangle on canvas for element's current
			-- bounding box to changed region, recompute its
			-- new bounding box (after move or rotation) and
			-- add a rectangle for new bounding box to changed
			-- region
		do
			if not included_in_changed_region then
				view.add_to_changed_region (bounding_box)
				set_view_coordinates
				view.add_to_changed_region (bounding_box)
			end
		end

	add_change (a_change: RETICLE_SIMPLE_CHANGE)
			-- Add `a_change' to current changes
		require
			a_change_not_void: a_change /= Void
		do
			view.add_change (a_change)
		end

	try_to_toggle_rotation
			-- Toggle rotation of element, if possible.
			-- Otherwise, add element to appropriate list
		local
			old_rot, new_rot: INTEGER
		do
			verify_rotation_allowed
			if last_rotation_ok then
				if not is_processed then
					old_rot := rotation
					toggle_rotation
					new_rot := rotation
					set_processed
					add_change (create {ORIENTATION_CHANGE}.make (Current, old_rot, new_rot))
				end
				add_old_and_new_to_changed_region
			end
			include_in_selection_bounding_box
		end

	try_to_rotate_90_clockwise
			-- Rotate element 90 degrees clockwise, if possible.
			-- Otherwise, add element to appropriate list
		local
			old_rot, new_rot: INTEGER
		do
			verify_rotation_allowed
			if last_rotation_ok then
				if not is_processed then
					old_rot := rotation
					rotate_90_clockwise
					new_rot := rotation
					set_processed
					add_change (create {ORIENTATION_CHANGE}.make (Current, old_rot, new_rot))
				end
				add_old_and_new_to_changed_region
			end
			include_in_selection_bounding_box
		end

	add_delete_error
			-- Add error for element that cannot be deleted
		do
			if not is_deletable then
				view.delete_locked_elements.extend (Current)
				add_error ("Cannot delete " + identification_for_message + " because it is delete-locked")
			else
				view.delete_failed_elements.extend (Current)
				if is_reworkable then
					add_error ("Cannot delete " + identification_for_message + " because it is in rework")
				else
					add_error ("Cannot delete " + identification_for_message + " due to a database row lock failure")
				end
			end
		end

	update_move_limits
			-- Update view distance limits for Move operation
		local
			up, down, left, right: DOUBLE
		do
			if is_locked_in_x then
				left := 0.0
				right := 0.0
			else
				left := model_element.db_maximum_move_left
				right := model_element.db_maximum_move_right
			end
			if is_locked_in_y then
				up := 0.0
				down := 0.0
			else
				up := model_element.db_maximum_move_up
				down := model_element.db_maximum_move_down
			end
			view.update_distance_limits (up, down, left, right)
		end

	update_duplicate_limits
			-- Update view distance limits for Duplicate operation
		local
			up, down, left, right: DOUBLE
		do
			if not is_duplicatable then
				left := 0.0
				right := 0.0
				up := 0.0
				down := 0.0
			else
				left := model_element.db_maximum_move_left
				right := model_element.db_maximum_move_right
				up := model_element.db_maximum_move_up
				down := model_element.db_maximum_move_down
			end
			view.update_distance_limits (up, down, left, right)
		end

	update_duplicate_lists
			-- Update view lists with elements that could not
			-- be duplicated for various reasons
		do
			if is_never_duplicatable then
				view.add_never_duplicatable_element (Current)
				add_warning ("Cannot duplicate " + identification_for_message + " because it is never duplicatable")
			elseif not is_duplicatable then
				view.add_duplication_locked_element (Current)
				add_warning ("Cannot duplicate " + identification_for_message + " because it is duplication-locked")
			end
		end

	duplicate
			-- Duplicate `Current' and put new copy in
			-- `last_duplicated_element'
		require
			is_die: is_die
		do
			model_element.duplicate
			last_duplicated_element := view.new_view_element (model_element.last_new_element)
			last_duplicated_element.copy_display_attributes (Current)
		end

	update_scribe_resize_limits (left_or_bottom: BOOLEAN)
			-- Update view distance limits for Resize Scribe
			-- operation, where `left_or_bottom' indicates whether
			-- we are resizing the left or bottom of the scribe
			-- (depending on whether scribe is horizontal or
			-- vertical) rather than the right or top
		local
			up, down, left, right, db_max: DOUBLE
		do
			up := 0.0
			down := 0.0
			left := 0.0
			right := 0.0
			db_max := max_db_value
			if is_horizontal_scribe then
				if left_or_bottom then
					left := (2.0 * model_element.db_maximum_move_left).min (db_max - model_element.size_x)
					right := (model_element.size_x - grid.size_resolution).min ((2.0 * db_max - model_element.field_relative_right_x - model_element.field_relative_left_x).abs)
				else
					left := (model_element.size_x - grid.size_resolution).min ((2.0 * -db_max - model_element.field_relative_left_x - model_element.field_relative_right_x).abs)
					right := (2.0 * model_element.db_maximum_move_right).min (db_max - model_element.size_x)
				end
			else
				if left_or_bottom then
					up := (model_element.size_y - grid.size_resolution).min ((2.0 * db_max - model_element.field_relative_top_y - model_element.field_relative_bottom_y).abs)
					down := (2.0 * model_element.db_maximum_move_down).min (db_max - model_element.size_y)
				else
					up := (2.0 * model_element.db_maximum_move_up).min (db_max - model_element.size_y)
					down := (model_element.size_y - grid.size_resolution).min ((2.0 * -db_max - model_element.field_relative_top_y - model_element.field_relative_bottom_y).abs)
				end
			end
			view.update_distance_limits (up, down, left, right)
		end

	resize_by (a_resize_amount: CONST_POINT is_left_or_bottom: BOOLEAN)
			-- Resize `Current' by `a_resize_amount', where
			-- `is_left_or_bottom' indicates whether the left or
			-- bottom edge is to be moved by `a_resize_amount'
			-- (as opposed to right or top edge)
		require
			a_resize_amount_not_void: a_resize_amount /= Void
			is_scribe: is_scribe
		do
			if is_horizontal_scribe then
				if is_left_or_bottom then
					model_element.move_left_edge (a_resize_amount.x)
				else
					model_element.move_right_edge (a_resize_amount.x)
				end
			else
				if is_left_or_bottom then
					model_element.move_bottom_edge (a_resize_amount.y)
				else
					model_element.move_top_edge (a_resize_amount.y)
				end
			end
		end

	delete
			-- Delete `Current' when not all copies are selected
			-- for a Delete operation
		require
			is_die: is_die
		do
			remove_from_all_copies
			die.remove_die_location (die_location)
		end

	isolate
			-- Isolate `Current' in its cut die in current
			-- cut pattern
		require
			is_die: is_die
		do
			view.isolate (Current)
		end

	unisolate
			-- Remove `Current' from list of isolated die
			-- locations in current cut pattern
		require
			is_die: is_die
		do
			view.unisolate (Current)
		end

	set_isolated_site_cut_die
			-- Set cut die size and center for `an_element'
			-- in current cut pattern
		require
			is_die: is_die
		do
			view.set_isolated_site_cut_die (Current)
		end

	copy_display_attributes (an_element: like Current)
			-- Copy attributes which affect how `Current' is
			-- displayed from `an_element'
		do
			set_label_generator (an_element.label_generator)
			set_color_generator (an_element.color_generator)
			set_fill_generator (an_element.fill_generator)
			set_opacity_generator (an_element.opacity_generator)
			set_view_coordinates
		end

	prepare_for_horizontal_spread
			-- Take actions needed before a horizontal spread
			-- operation
		do
			view.add_horizontal_spread_element (Current)
		end

	prepare_for_vertical_spread
			-- Take actions needed before a vertical spread
			-- operation
		do
			view.add_vertical_spread_element (Current)
		end

	check_move_restrictions (a_distance: POINT)
			-- Check that element can be moved `a_distance'
		require
			a_distance_not_void: a_distance /= Void
		do
			if (is_locked_in_x and a_distance.x /= 0.0) or (is_locked_in_y and a_distance.y /= 0.0) then
				view.unmovable_elements.extend (Current)
			end
		end

	check_outside_fields
			-- Check whether element is completely outside any
			-- field of view
		do
			if not intersects_any_field then
				add_error (identification_for_message + " is outside all fields of view " + location_description)
			end
		end

	check_overlapping_field (scribes_included: BOOLEAN)
			-- Check whether element intersects at least one
			-- field of view but is not enclosed by any field
		do
			if partially_outside_fields and (scribes_included or is_non_scribe) then
				add_error (identification_for_message + " overlaps field edge " + location_description)
			end
		end

	check_overlapping_object (scribes_included: BOOLEAN)
			-- Check whether element intersects at least one
			-- field of view and overlaps any other object that
			-- intersects at least one field of view.
			-- Set `is_overlapping_another_object' for Validity
			-- Check view coloring
		local
			areas: LIST [RETICLE_MODEL_AREA]
			mbb: CONST_RECTANGLE
			area: RETICLE_MODEL_AREA
			elems: LIST [RETICLE_VIEW_ELEMENT]
			elem: RETICLE_VIEW_ELEMENT
		do
			if intersects_any_field then
				areas := intersecting_mbb_areas
				mbb := model_bounding_box
				from
					areas.start
				until
					areas.after
				loop
					area := areas.item
					elems := area.intersecting_elements
					from
						elems.start
					until
						elems.after
					loop
						elem := elems.item
						if elem /= Current and then mbb.intersects (elem.model_bounding_box) and then (scribes_included or is_non_scribe or elem.is_non_scribe) then
							is_overlapping_another_object := True
							if not elem.is_marked and not invalid_elements.has (elem) then
								invalid_elements.extend (elem)
								add_error (identification_for_message + " overlaps " + elem.identification_for_message + "%N" + "    " + identification_for_message + " is " + location_description + "%N" + "    " + elem.identification_for_message + " is " + elem.location_description)
							end
						end
						elems.forth
					end
					areas.forth
				end
				mark
			end
		end

	check_too_close_to_object (scribes_included: BOOLEAN)
			-- Check whether element intersects at least one
			-- field of view and is too close to (but does not
			-- overlap) any other object that
			-- intersects at least one field of view.
			-- Set `is_too_close_to_other_object' for Validity
			-- Check view coloring
		local
			areas: LIST [RETICLE_MODEL_AREA]
			mbb: CONST_RECTANGLE
			bigger_mbb, other_mbb: RECTANGLE
			gap, other_grow, dist_x, dist_y: DOUBLE
			area: RETICLE_MODEL_AREA
			elems: LIST [RETICLE_VIEW_ELEMENT]
			elem: RETICLE_VIEW_ELEMENT
		do
			if intersects_any_field then
				areas := intersecting_mbb_areas
				gap := minimum_gap
				if gap > 0.0 then
					mbb := model_bounding_box
					create bigger_mbb.make_copy (mbb)
					bigger_mbb.size_by (2.0 * gap, 2.0 * gap)
					from
						areas.start
					until
						areas.after
					loop
						area := areas.item
						elems := area.intersecting_elements
						from
							elems.start
						until
							elems.after
						loop
							elem := elems.item
							create other_mbb.make_copy (elem.model_bounding_box)
							if elem /= Current and then not mbb.intersects (other_mbb) and (scribes_included or is_non_scribe or elem.is_non_scribe) then
								other_grow := -2.0 * elem.uniform_white_space_border
								other_mbb.size_by (other_grow, other_grow)
								dist_x := distance_in_x_from (elem)
								dist_y := distance_in_y_from (elem)
								if bigger_mbb.intersects (other_mbb) and (needs_min_sep_white_space or elem.needs_min_sep_white_space or dist_x /= 0.0 or dist_y /= 0.0) then
									is_too_close_to_another_object := True
									if not elem.is_marked and not invalid_elements.has (elem) then
										invalid_elements.extend (elem)
										report_too_close_to_object (elem, dist_x, dist_y, minimum_separation_from (elem), not needs_min_sep_white_space and not elem.needs_min_sep_white_space)
									end
								end
							end
							elems.forth
						end
						areas.forth
					end
				end
				mark
			end
		end

	check_misaligned_scribe
			-- Check whether element intersects at least
			-- one field of view and is misaligned with
			-- another scribe of same orientation that
			-- intersects at least one field of view.  Set
			-- `is_misaligned_scribe' for Validity Check
			-- view coloring
		local
			areas: LIST [RETICLE_MODEL_AREA]
			area: RETICLE_MODEL_AREA
			elems: LIST [RETICLE_VIEW_ELEMENT]
			elem: RETICLE_VIEW_ELEMENT
			diff: DOUBLE
		do
			if intersects_any_field then
				areas := intersecting_scribe_areas
				from
					areas.start
				until
					areas.after
				loop
					area := areas.item
					elems := area.intersecting_elements
					from
						elems.start
					until
						elems.after
					loop
						elem := elems.item
						if elem /= Current and then is_misaligned_with (elem) then
							is_misaligned_scribe := True
							if not elem.is_marked and not invalid_elements.has (elem) then
								invalid_elements.extend (elem)
								diff := misalignment_amount (elem)
								add_warning (identification_for_message + " is misaligned with " + elem.identification_for_message + " by " + grid.formatted_number (diff) + "%N" + "    " + identification_for_message + " is " + location_description + "%N" + "    " + elem.identification_for_message + " is " + elem.location_description)
							end
						end
						elems.forth
					end
					areas.forth
				end
				mark
			end
		end

	check_below_minimum_size
			-- Check whether element is below minimum valid size
			-- in either dimension
		do
			if is_below_minimum_size and not is_processed then
				set_processed
				add_error (identification_for_message + " size " + size_description + " is below minimum size of " + grid.formatted_number (view.minimum_die_size) + " in at least one dimension%N    " + identification_for_message + " is " + location_description)
			end
		end

	check_no_lot_assigned
			-- Check whether element is a project with no lot
			-- assigned
		do
			if is_wafer_etest_id_missing and not is_processed then
				set_processed
				add_error (identification_for_message + " has not been assigned to a lot split")
			end
		end

	check_unlocked_or_unassigned
			-- Check whether element is a project which is not
			-- locked against customer update or not assigned to
			-- run being closed
		local
			l_unlocked, l_unassigned: BOOLEAN
			what: STRING
		do
			if is_cust_project and not is_processed then
				l_unlocked := is_unlocked_project
				l_unassigned := is_unassigned_project
				if l_unlocked or l_unassigned then
					set_processed
					if l_unlocked and l_unassigned then
						what := "is not locked against customer update and not assigned to run"
					elseif l_unlocked then
						what := "is not locked against customer update"
					else
						what := "is not assigned to run"
					end
					add_error (identification_for_message + " " + what)
				end
			end
		end

	check_too_close_to_edge (scribes_included: BOOLEAN)
			-- Check whether element is too close to a field edge
		local
			l, r, t, b: DOUBLE
			l_bad, r_bad, t_bad, b_bad: BOOLEAN
		do
			if scribes_included or is_non_scribe then
				l := left_to_field_edge
				r := right_to_field_edge
				t := top_to_field_edge
				b := bottom_to_field_edge
				l_bad := is_edge_too_close_to_field_edge (l)
				r_bad := is_edge_too_close_to_field_edge (r)
				t_bad := is_edge_too_close_to_field_edge (t)
				b_bad := is_edge_too_close_to_field_edge (b)
				if l_bad then
					report_too_close_to_edge ("left", l, minimum_gap, not needs_min_sep_white_space)
				end
				if r_bad then
					report_too_close_to_edge ("right", r, minimum_gap, not needs_min_sep_white_space)
				end
				if t_bad then
					report_too_close_to_edge ("top", t, minimum_gap, not needs_min_sep_white_space)
				end
				if b_bad then
					report_too_close_to_edge ("bottom", b, minimum_gap, not needs_min_sep_white_space)
				end
			end
		end

	check_scribe_cut_die_too_big
			-- Check whether element has a cut die size
			-- determined by surrounding scribes that
			-- exceeds its maximum die size
		local
			l_max: like max_die_size
			msg: STRING
		do
			if is_scribe_cut_die_too_big then
				l_max := max_die_size
				msg := identification_for_message + " cut die size " + scribe_cut_size_description + " exceeds maximum size of " + grid.formatted_number (l_max.x) + " x " + grid.formatted_number (l_max.y) + "%N    " + identification_for_message + " is " + location_description
				if is_oversize_die_always_allowed then
					add_warning (msg)
				elseif is_oversize_die_possibly_allowed then
					add_warning (msg + oversize_inclusion_text)
				else
					add_error (msg)
				end
			end
		end

	check_unisolated_in_scribe_cut_die
			-- Check whether element is not isolated in its cut die
			-- determined by surrounding scribes
		do
			if unisolated_in_scribe_cut_die then
				is_unisolated_in_scribe_cut_die := True
				add_warning (identification_for_message + " is not isolated in its cut die " + location_description)
			end
		end

	compute_unisolated_in_scribe_cut_die
			-- Compute whether element is not isolated in its
			-- cut die determined by surrounding scribes
		do
			if unisolated_in_scribe_cut_die then
				is_unisolated_in_scribe_cut_die := True
			end
		end

	check_uncentered_in_cut_die
			-- Check whether element is not centered in its cut die
		do
			if is_uncentered_in_cut_die then
				add_warning (identification_for_message + " is offset from cut die center by " + offset_cut_center_description + " " + location_description)
			end
		end

	check_invalid_cut_die
			-- Check whether element is isolated in current cut
			-- pattern but has an invalid cut die determined by
			-- surrounding cuts
		local
			what: STRING
		do
			if is_cut_cut_die_invalid then
				if has_surrounding_cuts then
					what := "is isolated but has a cut through printed die"
				else
					what := "is isolated but is not surrounded by cuts on all sides"
				end
				add_error (identification_for_message + " " + what + " " + location_description)
			end
		end

	check_cut_cut_die_too_big
			-- Check whether element has a cut die size
			-- determined by surrounding cuts that
			-- exceeds its maximum die size
		local
			l_max: like max_die_size
			msg: STRING
		do
			if is_cut_cut_die_too_big then
				l_max := max_die_size
				msg := identification_for_message + " cut die size " + cut_cut_size_description + " exceeds maximum size of " + grid.formatted_number (l_max.x) + " x " + grid.formatted_number (l_max.y) + "%N    " + identification_for_message + " is " + location_description
				if is_oversize_die_always_allowed then
					add_warning (msg)
				elseif is_oversize_die_possibly_allowed then
					add_warning (msg + oversize_inclusion_text)
				else
					add_error (msg)
				end
			end
		end

	check_wrong_lot
			-- Check whether element is isolated and has the
			-- wrong lot (different from the one for the cut
			-- pattern)
		do
			if has_wrong_lot_for_cut_pattern then
				add_warning (identification_for_message + " has wrong lot " + assigned_wafer_etest_id + " - cut pattern is for lot " + view.current_cut_pattern.wafer_etest_id + "%N    " + identification_for_message + " is " + location_description)
			end
		end

	check_non_default_cut_die_size
			-- Check whether element that is isolated has
			-- an cut die size determined by surrounding
			-- cuts that is different from its default cut
			-- die size determined by surrounding scribes
		do
			if has_non_default_cut_die_size then
				add_warning (identification_for_message + " has cut die that differs from cut die determined by surrounding scribes" + "%N    " + identification_for_message + " is " + location_description)
			end
		end

	check_shared_cut_die
			-- Check whether element that is isolated
			-- shares its cut die partially or fully with
			-- one or more other non-scribe die elements
		do
			if is_isolated_in_current_cut_pattern and unisolated_in_cut_cut_die then
				is_unisolated_in_cut_cut_die := True
				add_warning (identification_for_message + " shares in its cut die with others " + location_description)
			end
		end

	check_extra_cut
			-- Check whether element is an extra cut that does
			-- not help isolate any isolated sites
			-- one or more other non-scribe die elements
		do
			if is_extra_cut then
				add_warning (identification_for_message + " is an extra cut that does not help isolate any isolated sites " + location_description)
			end
		end

	set_floating_scribe
			-- Set element to be a floating scribe and add
			-- warning
		do
			set_is_floating_scribe (True)
			add_warning (identification_for_message + " is a floating scribe " + location_description)
		end

	set_misaligned_scribe
			-- Set element to be a misaligned scribe and add
			-- warning
		do
			set_is_misaligned_scribe (True)
			add_warning (identification_for_message + " is a misaligned scribe " + location_description)
		end

	report_small_but_legal (scribes_included: BOOLEAN)
			-- Report element as small but legal
		do
			if scribes_included or is_non_scribe then
				-- Report all locations 
				-- (ignore is_processed indicator)
				set_processed
				add_warning (identification_for_message + " size " + size_description + " is legal but small " + location_description)
			end
		end

	add_field
			-- Add `Current' to list of fields
		do
			view.add_field (Current)
		end

	add_error (a_message: STRING)
			-- Add a new error described by `a_message'
		require
			a_message_not_void: a_message /= Void
		do
			view.add_error (a_message)
		end

	add_warning (a_message: STRING)
			-- Add a new warning described by `a_message'
		require
			a_message_not_void: a_message /= Void
		do
			view.add_warning (a_message)
		end

	set_processed
			-- Set indicator that a copy of element has been
			-- processed
		do
			all_copies.set_processed
		end

	clear_processed
			-- Clear indicator that a copy of element has been
			-- processed
		do
			all_copies.clear_processed
		end

	uncache_all_copies_selected
			-- Remove from cache value computed for
			-- `all_copies_selected'
		do
			all_copies.uncache_all_elements_selected
		end

	uncache_any_copies_selected
			-- Remove from cache value computed for
			-- `any_copies_selected'
		do
			all_copies.uncache_any_elements_selected
		end

	uncache_any_copies_stationary_orientation
			-- Remove from cache value computed for
			-- `any_copies_stationary_orientation'
		do
			all_copies.uncache_any_elements_stationary_orientation
		end

	uncache_any_copies_rotation_locked
			-- Remove from cache value computed for
			-- `any_copies_rotation_locked'
		do
			all_copies.uncache_any_elements_rotation_locked
		end

	uncache_any_copies_contain
			-- Remove from cache value computed for
			-- `any_copies_contain'
		do
			all_copies.uncache_any_elements_contain
		end

	include_in_selection_bounding_box
			-- Include element in current selection bounding box
		do
			view.include_in_selection_bounding_box (Current)
		end

	mark
			-- Set "marked" indicator
		do
			is_marked := True
		end

	unmark
			-- Clear "marked" indicator
		do
			is_marked := False
		end

	clear_invalid_elements
			-- Clear list of invalid elements with which
			-- `Current' has some kind of validity problem
		do
			create {ARRAYED_LIST [RETICLE_VIEW_ELEMENT]} invalid_elements.make (5)
			invalid_elements.compare_references
		end

	set_is_floating_scribe (b: BOOLEAN)
			-- Set `is_floating_scribe' to `b'
		do
			is_floating_scribe := b
		ensure
			value_set: is_floating_scribe = b
		end

	set_is_misaligned_scribe (b: BOOLEAN)
			-- Set `is_misaligned_scribe' to `b'
		do
			is_misaligned_scribe := b
		ensure
			value_set: is_misaligned_scribe = b
		end

	initialize_intersecting_mbb_areas (n: INTEGER)
			-- Create `intersecting_mbb_areas' with capacity `n'
		do
			create {ARRAYED_LIST [RETICLE_MODEL_AREA]} intersecting_mbb_areas.make (n)
		end

	initialize_intersecting_scribe_cut_die_areas (n: INTEGER)
			-- Create `intersecting_scribe_cut_die_areas' with capacity `n'
		do
			create {ARRAYED_LIST [RETICLE_MODEL_AREA]} intersecting_scribe_cut_die_areas.make (n)
		end

	initialize_intersecting_cut_cut_die_areas (n: INTEGER)
			-- Create `intersecting_cut_cut_die_areas' with capacity `n'
		do
			create {ARRAYED_LIST [RETICLE_MODEL_AREA]} intersecting_cut_cut_die_areas.make (n)
		end

	initialize_intersecting_hscribe_areas (n: INTEGER)
			-- Create `intersecting_hscribe_areas' with capacity `n'
		do
			create {ARRAYED_LIST [RETICLE_MODEL_AREA]} intersecting_hscribe_areas.make (n)
		end

	initialize_intersecting_vscribe_areas (n: INTEGER)
			-- Create `intersecting_vscribe_areas' with capacity `n'
		do
			create {ARRAYED_LIST [RETICLE_MODEL_AREA]} intersecting_vscribe_areas.make (n)
		end

	populate_intersecting_mbb_areas
			-- Populate `intersecting_mbb_areas' and add element
			-- to intersecting elements list of every model
			-- area its bounding box intersects
		do
			populate_intersecting_areas (view.mbb_model_areas, model_bounding_box, intersecting_mbb_areas)
		end

	populate_intersecting_scribe_cut_die_areas
			-- Populate `intersecting_scribe_cut_die_areas' and
			-- add element to intersecting elements list
			-- of every model area its cut die (determined by
			-- surrounding scribes) intersects
		do
			populate_intersecting_areas (view.scribe_cut_die_model_areas, scribe_cut_die, intersecting_scribe_cut_die_areas)
		end

	populate_intersecting_cut_cut_die_areas
			-- Populate `intersecting_cut_cut_die_areas' and
			-- add element to intersecting elements list
			-- of every model area its cut die (determined by
			-- surrounding cuts) intersects.  Use cut die
			-- determined by surrounding scribes if cut cut
			-- die is Void
		do
			populate_intersecting_areas (view.cut_cut_die_model_areas, cut_or_scribe_cut_die, intersecting_cut_cut_die_areas)
		end

	populate_intersecting_hscribe_areas
			-- Populate `intersecting_hscribe_areas' and
			-- add element to intersecting elements list
			-- of every model area its bounding box
			-- intersects
		do
			populate_intersecting_areas (view.horizontal_scribe_model_areas, model_bounding_box, intersecting_hscribe_areas)
		end

	populate_intersecting_vscribe_areas
			-- Populate `intersecting_vscribe_areas' and
			-- add element to intersecting elements list
			-- of every model area its bounding box
			-- intersects
		do
			populate_intersecting_areas (view.vertical_scribe_model_areas, model_bounding_box, intersecting_vscribe_areas)
		end

	populate_intersecting_areas (areas: LIST [RETICLE_MODEL_AREA]  mbb: CONST_RECTANGLE intersecting_areas: like intersecting_mbb_areas)
			-- Populate `intersecting_areas' by adding
			-- element to intersecting elements list of
			-- every model area in `areas' that `mbb'
			-- intersects
		local
			area: RETICLE_MODEL_AREA
		do
			if intersects_any_field then
				if mbb /= Void then
					from
						areas.start
					until
						areas.after
					loop
						area := areas.item
						if mbb.intersects (area.rectangle) then
							area.intersecting_elements.extend (Current)
							intersecting_areas.extend (area)
						end
						areas.forth
					end
				end
			end
		end

	increment_centered_count
			-- Increment count of projects centered in cut die
			-- by Center operation
		do
			view.increment_centered_count
		end

	report_isolation_failure
			-- Add error message when toggling isolation of
			-- project is not allowed
		do
			if has_surrounding_cuts then
				add_error (identification_for_message + " has a cut through the printed die " + location_description)
			else
				add_error (identification_for_message + " is not surrounded by cuts on all sides " + location_description)
			end
			view.increment_last_isolation_failure_count
		end

feature {NONE} -- Implementation

	verify_rotation_allowed
			-- Verify that element can be rotated and add
			-- warning if not.  Set `last_rotation_ok' to
			-- indicate whether rotation is allowed
		do
			last_rotation_ok := False
			if any_copies_rotation_locked then
				view.add_rotation_locked_element (Current)
				add_rotate_warning ("Cannot rotate " + identification_for_message + " because at least one copy is rotation-locked")
			elseif is_stationary_orientation then
				view.add_fixed_orientation_element (Current)
				add_rotate_warning ("Cannot rotate " + identification_for_message + " because its orientation is fixed")
			elseif not is_rotatable then
				handle_internal_error ("element with identification " + identification_for_message + " is not stationary in orientation or rotation-locked but is not rotatable")
			elseif not all_copies_selected then
				view.add_partially_selected_element (Current)
				add_rotate_warning ("Cannot rotate " + identification_for_message + " because not all copies are selected")
			else
				last_rotation_ok := True
			end
		end

	add_rotate_warning (a_message: STRING)
			-- Add warning about rotation that failed
		do
			if not is_processed then
				add_warning (a_message)
				set_processed
			end
		end

	last_rotation_ok: BOOLEAN
			-- Was last attempt to rotate element allowed?

	anchored_model_element: RETICLE_MODEL_ELEMENT
			-- Type of elements in model
		do
		end

	anchored_view_sub_element: RETICLE_VIEW_SUB_ELEMENT
			-- Type of sub-elements in view
		do
		end

	unselectable_elements: LIST [RETICLE_VIEW_ELEMENT]
			-- List of items that cannot be selected
		do
			Result := view.unselectable_elements
		end

	unselected_elements: LIST [RETICLE_VIEW_ELEMENT]
			-- List of items that are not selected but can be
			-- selected
		do
			Result := view.unselected_elements
		end

	previously_selected_elements: LIST [RETICLE_VIEW_ELEMENT]
			-- List of items that are selected but which
			-- did not match the current filter that determines
			-- which items have their selection status changed
		do
			Result := view.previously_selected_elements
		end

	newly_selected_elements: LIST [RETICLE_VIEW_ELEMENT]
			-- List of items that are selected and which
			-- matched the current filter that determines
			-- which items have their selection status changed
		do
			Result := view.newly_selected_elements
		end

	report_too_close_to_edge (an_edge: STRING a_distance, a_minimum_gap: DOUBLE a_zero_ok: BOOLEAN)
			-- Add warning about element `an_edge' edge which
			-- is `a_distance' away from corresponding field
			-- edge (which is too close)
		local
			zero: STRING
		do
			if a_zero_ok then
				zero := "gap = 0 or "
			else
				zero := ""
			end
			add_warning (identification_for_message + " is too close to field " + an_edge + " edge (gap = " + grid.formatted_number (a_distance) + ", requires " + zero + " gap >= " + grid.formatted_number (a_minimum_gap) + ") " + location_description)
		end

	report_too_close_to_object (an_element: RETICLE_VIEW_ELEMENT a_distance_x, a_distance_y, a_minimum_gap: DOUBLE a_zero_ok: BOOLEAN)
			-- Add warning about element `an_element' edge which
			-- is `a_distance_x' and `a_distance_y' away from
			-- `Current', which is too close
		local
			zero: STRING
			dist: DOUBLE
		do
			if a_zero_ok then
				zero := "gap = 0 or "
			else
				zero := ""
			end
			if a_distance_x = 0.0 then
				dist := a_distance_y
			elseif a_distance_y = 0.0 then
				dist := a_distance_x
			else
				dist := a_distance_x.min (a_distance_y)
			end

			add_warning (identification_for_message + " is too close to " + an_element.identification_for_message + " (gap = " + grid.formatted_number (dist) + ", requires " + zero + " gap >= " + grid.formatted_number (a_minimum_gap) + ")%N" + "    " + identification_for_message + " is " + location_description + "%N" + "    " + an_element.identification_for_message + " is " + an_element.location_description)
		end

	oversize_inclusion_text: STRING
			-- Text to add to warning message if oversize
			-- die is to be included on run but only under
			-- certain conditions
		local
			folder: TEXT_FOLDING
		do
			create folder
			Result := "%N    Inclusion Conditions:%N" + su.joined_by (su.strings_prefixed_by (folder.folded_text (oversize_die_inclusion_conditions, Max_inclusion_length), "        "), "%N")
		end

	Max_inclusion_length: INTEGER = 100
			-- Maximum length of lines in `oversize_inclusion_text'

invariant
	label_generator_not_void: label_generator /= Void
	color_generator_not_void: color_generator /= Void
	fill_generator_not_void: fill_generator /= Void
	opacity_generator_not_void: opacity_generator /= Void
	
end
