
indexing
	
	description: "Object that is a sub-element of an element of a view of a reticle"
	author: "David Hollenberg"
	date: "July 25, 2006"

class RETICLE_VIEW_SUB_ELEMENT
inherit
	VIEW_SUB_ELEMENT
		redefine
			view
		end
	PACKER_UTILITIES
		export
			{NONE} all
		end

feature {NONE} -- Creation

	make (a_model_sub_element: like model_sub_element a_view_element: like view_element a_view: like view)
		require
			a_model_sub_element_not_void: a_model_sub_element /= Void
			a_view_not_void: a_view /= Void
		do
			model_sub_element := a_model_sub_element
			view_element := a_view_element
			view := a_view
			create bounding_box.make (0, 0, 0, 0)
			create {VIEW_ELEMENT_STANDARD_DRAWER} drawer
		end

feature -- Properties

	view: RETICLE_VIEW
			-- View of which this is one sub-element of an element

	model_sub_element: RETICLE_MODEL_SUB_ELEMENT
			-- Sub-element of model to which `Current' corresponds

	view_element: RETICLE_VIEW_ELEMENT
			-- View element of which `Current' is one part

	label_generator: RETICLE_VIEW_ELEMENT_LABEL_GENERATOR
			-- Generator for text to display to label this element
		do
			Result := view_element.label_generator
		end

	color_generator: RETICLE_VIEW_ELEMENT_COLOR_GENERATOR
			-- Generator for colors to use to display this element
		do
			Result := view_element.color_generator
		end

	fill_generator: RETICLE_VIEW_ELEMENT_FILL_GENERATOR
			-- Generator for "filled" indicator to use to
			-- display this element
		do
			Result := view_element.fill_generator
		end

	opacity_generator: RETICLE_VIEW_ELEMENT_OPACITY_GENERATOR
			-- Generator for opaqueness or transparency
			-- indicator to use to display this element
		do
			Result := view_element.opacity_generator
		end

	line_width: INTEGER
			-- Width of line to draw for border, in pixels
		do
			Result := view_element.line_width
		end

	label: STRING
			-- Current label to display
		do
			Result := label_generator.label (view_element)
		end

	label_rotation: INTEGER
			-- Rotation of label in degrees clockwise
		do
			Result := view_element.label_rotation
		end

	border_color: EV_COLOR
			-- Color to use to display border
		do
			Result := color_generator.border_color (view_element)
		end

	label_color: EV_COLOR
			-- Color to use to display label
		do
			Result := color_generator.label_color (view_element)
		end

	is_interior_filled: BOOLEAN
			-- Should interior of element be filled when displayed?
		do
			Result := fill_generator.is_interior_filled (view_element)
		end

	interior_color: EV_COLOR
			-- Color to use to fill interior, if filled or opaque
		do
			if is_interior_filled then
				Result := fill_generator.interior_color (view_element)
			else
				Result := view_element.non_filled_interior_color
			end
		end

	is_opaque: BOOLEAN
			-- Should element be displayed as opaque, so that
			-- it completely obscures the part of any element
			-- that is below it in stacking order?
		do
			Result := opacity_generator.is_opaque (view_element)
		end

	is_transparent: BOOLEAN
			-- Should element be displayed as transparent, so that
			-- we attempt to draw parts of elements that fall
			-- below it in stacking order so that they are visible?
		do
			Result := opacity_generator.is_transparent (view_element)
		end

	is_cut_line: BOOLEAN
			-- Is element a cut line?
		do
			Result := model_sub_element.is_cut_line
		end

	is_cut_handle: BOOLEAN
			-- Is element a cut handle?
		do
			Result := model_sub_element.is_cut_handle
		end

	size_x: DOUBLE
			-- Size of associated model element in X dimension
		do
			Result := model_sub_element.size_x
		end

	size_y: DOUBLE
			-- Size of associated model element in X dimension
		do
			Result := model_sub_element.size_y
		end

	left_x: DOUBLE
			-- Left edge of minimum bounding box of associated
			-- model element in X dimension
		do
			Result := model_sub_element.left_x
		end

	right_x: DOUBLE
			-- Right edge of minimum bounding box of associated
			-- model element in X dimension
		do
			Result := model_sub_element.right_x
		end

	top_y: DOUBLE
			-- Top edge of minimum bounding box of associated
			-- model element in Y dimension
		do
			Result := model_sub_element.top_y
		end

	bottom_y: DOUBLE
			-- Bottom edge of minimum bounding box of associated
			-- model element in X dimension
		do
			Result := model_sub_element.bottom_y
		end

	center_x: DOUBLE
			-- Center of minimum bounding box of associated
			-- model element in X dimension
		do
			Result := model_sub_element.center_x
		end

	center_y: DOUBLE
			-- Center of minimum bounding box of associated
			-- model element in Y dimension
		do
			Result := model_sub_element.center_y
		end

feature -- Modification

	set_model_sub_element (a_model_sub_element: like model_sub_element)
			-- Set `model_sub_element' to `a_model_sub_element'
		require
			a_model_sub_element_not_void: a_model_sub_element /= Void
		do
			model_sub_element := a_model_sub_element
		ensure
			model_sub_element_set: model_sub_element = a_model_sub_element
		end

feature -- Operations

	set_view_coordinates
			-- Set `width', `height', `x', `y' and
			-- `bounding_box' based on size, location and
			-- rotation of `model_element' and its
			-- position on `view'
		do
			view.set_view_rectangle_from_model_rectangle (bounding_box, Maximum_view_coordinate.as_integer_32, model_sub_element.left_x, model_sub_element.top_y, model_sub_element.right_x, model_sub_element.bottom_y)
			unload_view_coordinates
		end

feature {NONE} -- Implementation

	anchored_model_element: RETICLE_MODEL_ELEMENT
			-- Type of elements in model
		do
		end

invariant
	label_generator_not_void: label_generator /= Void
	color_generator_not_void: color_generator /= Void
	fill_generator_not_void: fill_generator /= Void
	opacity_generator_not_void: opacity_generator /= Void
	
end
