
indexing
	
	description: "Virtual element of reticle model.  Used to show field %
		%boundaries, labels, etc."
	author: "David Hollenberg"
	date: "October 4, 2006"

deferred class RETICLE_VIRTUAL_ELEMENT
inherit
	RETICLE_MODEL_ELEMENT
		redefine
			is_virtual, has_field_edge_restriction
		end

feature {NONE} -- Initialization

	initialize (a_size, a_center: CONST_POINT a_model: like anchored_model)
			-- Initialize newly created object
		require
			a_size_not_void: a_size /= Void
			a_center_not_void: a_center /= Void
			a_size_x_positive: a_size.x > 0.0
			a_size_y_positive: a_size.y > 0.0
			a_model_not_void: a_model /= Void
		do
			size_x := a_size.x
			size_y := a_size.y
			center_x := a_center.x
			center_y := a_center.y
			model := a_model
			create {ARRAYED_LIST [like Current]} all_copies.make (1)
			all_copies.extend (Current)
		end

feature -- Properties

	is_virtual: BOOLEAN = True
			-- Is element virtual (label, field of view edge)
			-- rather than something that can potentially be
			-- manipulated (selected, moved, etc)?

	size_x: DOUBLE
			-- Size of minimum bounding box in X dimension

	size_y: DOUBLE
			-- Size of minimum bounding box in Y dimension

	center_x: DOUBLE
			-- Location of center of minimum bounding box in
			-- X dimension

	center_y: DOUBLE
			-- Location of center of minimum bounding box in
			-- Y dimension

	has_field_edge_restriction: BOOLEAN = False
			-- Does element have a restriction on how close it
			-- can be to edge of field?

	is_selectable: BOOLEAN = False
			-- May object be selected?

	rotation: INTEGER = 0
			-- Rotation of object in degrees clockwise

	is_rotated_sideways: BOOLEAN = False
			-- Is object rotated by 90 degrees clockwise or
			-- 90 degrees counterclockwise?

	is_fixed_in_x_internal: BOOLEAN = False
			-- Is object temporarily fixed in the X dimension
			-- so that it cannot move in that dimension?

	is_fixed_in_y_internal: BOOLEAN = False
			-- Is object temporarily fixed in the Y dimension
			-- so that it cannot move in that dimension?

	can_be_fixed_in_x: BOOLEAN = False
			-- Can object be temporarily fixed in the X dimension
			-- so that it cannot move in that dimension?

	can_be_fixed_in_y: BOOLEAN = False
			-- Can object be temporarily fixed in the Y dimension
			-- so that it cannot move in that dimension?

	is_stationary_in_x: BOOLEAN = True
			-- Can object never be moved in the X dimension?

	is_stationary_in_y: BOOLEAN = True
			-- Can object never be moved in the Y dimension?

	is_stationary_orientation: BOOLEAN = True
			-- Can orientation of object never be changed?

	is_never_duplicatable: BOOLEAN = True
			-- Can object never be duplicated?

	is_never_deletable: BOOLEAN = True
			-- Can object never be deleted?

feature {NONE} -- Modification

	set_is_fixed_in_x (b: BOOLEAN)
			-- Set `is_fixed_in_x' to `b'
		do
			-- Inapplicable
		end

	set_is_fixed_in_y (b: BOOLEAN)
			-- Set `is_fixed_in_y' to `b'
		do
			-- Inapplicable
		end

	set_is_movable_in_x (b: BOOLEAN)
			-- Set `is_movable_in_x' to `b'
		do
			-- Inapplicable
		end

	set_is_movable_in_y (b: BOOLEAN)
			-- Set `is_movable_in_y' to `b'
		do
			-- Inapplicable
		end

	set_is_rotatable (b: BOOLEAN)
			-- Set `is_rotatable' to `b'
		do
			-- Inapplicable
		end

	set_is_duplicatable (b: BOOLEAN)
			-- Set `is_duplicatable' to `b'
		do
			-- Inapplicable
		end

	set_is_deletable (b: BOOLEAN)
			-- Set `is_deletable' to `b'
		do
			-- Inapplicable
		end

	set_rotation (n: INTEGER)
			-- Set rotation of `Current' to `n' degrees clockwise
		do
			-- Inapplicable
		end

	toggle_rotation
			-- Toggle `is_rotated'
		do
			-- Inapplicable
		end

	rotate_90_clockwise
			-- Rotate `Current' 90 degrees clockwise
		do
			-- Inapplicable
		end

	rotate_90_counterclockwise
			-- Rotate `Current' 90 degrees counterclockwise
		do
			-- Inapplicable
		end

	db_maximum_move_up: DOUBLE
			-- Maximum distance element can move up and still
			-- fit in database
		do
			-- Inapplicable
		end

	db_maximum_move_down: DOUBLE
			-- Maximum distance element can move down and still
			-- fit in database
		do
			-- Inapplicable
		end

	db_maximum_move_left: DOUBLE
			-- Maximum distance element can move left and still
			-- fit in database
		do
			-- Inapplicable
		end

	db_maximum_move_right: DOUBLE
			-- Maximum distance element can move right and still
			-- fit in database
		do
			-- Inapplicable
		end

	move_by (a_distance: CONST_POINT)
			-- Move `Current' by `a_distance'
		do
			-- Inapplicable
		end

	round_edge_to_grid
			-- Move lower left corner of `Current' to nearest
			-- grid point.  Used to prevent accumulation of
			-- errors from multiple moves
		do
			-- Inapplicable
		end

	duplicate
			-- Duplicate element and add it to model
		do
			-- Inapplicable
		end

	move_left_edge (a_distance: DOUBLE)
			-- Move left edge of `Current' by `a_distance'
		do
			-- Inapplicable
		end

	move_right_edge (a_distance: DOUBLE)
			-- Move right edge of `Current' by `a_distance'
		do
			-- Inapplicable
		end

	move_top_edge (a_distance: DOUBLE)
			-- Move top edge of `Current' by `a_distance'
		do
			-- Inapplicable
		end

	move_bottom_edge (a_distance: DOUBLE)
			-- Move bottom edge of `Current' by `a_distance'
		do
			-- Inapplicable
		end

	die: PACKING_DIES_MROW
			-- Die for element that is a die
		do
			-- Inapplicable
		end

	die_location: PACKING_DIE_LOCATIONS_MROW
			-- Die location for element that is a die
		do
			-- Inapplicable
		end

	max_die_size: CONST_POINT
			-- Maximum die size, as constrained by package
			-- cavity sizes and user-specified limits.
			-- Void indicates no maximum
		do
			-- Inapplicable
		end

	is_oversize_die_always_allowed: BOOLEAN
			-- Is a die that is bigger than `max_die_size'
			-- allowed unconditionally?
		do
			-- Inapplicable
		end

	is_oversize_die_possibly_allowed: BOOLEAN
			-- Is a die that is bigger than `max_die_size'
			-- allowed but only under specific conditions?
		do
			-- Inapplicable
		end

	oversize_die_inclusion_conditions: STRING
			-- Description of conditions under which 
			-- die that is bigger than `max_die_size'
			-- allowed
		do
			-- Inapplicable
		end

feature {NONE} -- Implementation

	is_movable_in_x_internal: BOOLEAN = False
			-- May object currently be moved in the X dimension?

	is_movable_in_y_internal: BOOLEAN = False
			-- May object currently be moved in the Y dimension?

	is_rotatable_internal: BOOLEAN = False
			-- May rotation of object currently be changed?

	is_duplicatable_internal: BOOLEAN = False
			-- May object currently be duplicated?

	is_deletable_internal: BOOLEAN = False
			-- May object currently be deleted?

end
