
indexing
	
	description: "An operation that sets a database role"
	author: "David Hollenberg"
	date: "March 12, 2003"

class ROLE_SET_OP
inherit
	OPERATION
		rename
			try_operation as try_to_set_role
		end

create
	default_create

feature -- Operations

	set_role (conn: DB_CONNECTION r: STRING pass: STRING)
		require
			connection_exists: conn /= Void
			connection_open: conn.is_open
			connection_readable: conn.has_query_access
			role_exists: r /= Void
		do
			connection := conn
			role := r
			password := pass
			execute
		end

feature {NONE} -- Implementation

	connection: DB_CONNECTION
			-- Connection for which to set role

	role: STRING
			-- Role to try to set

	password: STRING
			-- Role password, if not Void or empty

	try_to_set_role
			-- Try to set role to `role' with `password'
		local
			the_role: STRING
			error: ROLE_SET_ERROR
		do
			if user_has_role then
				the_role := role.twin
				if password /= Void and then not password.is_empty then
					the_role.append (" identified by ")
					the_role.append (password)
				end
				connection.set_role (the_role)
				publish_ok_event
			else
				publish_failed_event
				create error
				error.set_hard_error
				error.set_role (role)
				error.set_user_name (connection.user_name)
				error.handle
			end
		end

	user_has_role: BOOLEAN
		local
			b: DB_BIND_S
		do
			create b.make (role)
			Result := connection.selected_count ("user_role_privs", "granted_role = :1", b) > 0
		end

	publish_ok_event
		local
			ok_event: ROLE_SET_OK_EVENT
		do
			create ok_event
			ok_event.set_role (role)
			ok_event.publish
		end

	publish_failed_event
		local
			failed_event: ROLE_SET_FAILED_EVENT
		do
			create failed_event
			failed_event.set_role (role)
			failed_event.publish
		end

end
