
indexing
	
	description: "An operation that locks a database row"
	author: "David Hollenberg"
	date: "April 10, 2003"

class ROW_LOCK_OP
inherit
	OPERATION
		rename
			try_operation as try_to_lock_row
		end

create
	default_create

feature -- Operations

	lock_list_nowait (list: LIST [DB_LOCKABLE_ROW])
			-- Lock every row in `list'
		require
			list_exists: list /= Void
		do
			from
				list.start
			until
				list.after
			loop
				lock_nowait (list.item)
				list.forth
			end
		end

	lock_nowait (r: DB_LOCKABLE_ROW)
			-- Lock `r'
		require
			row_exists: r /= Void
		do
			row := r
			execute
		end

feature {NONE} -- Implementation

	row: DB_LOCKABLE_ROW
			-- Row to try to lock

	try_to_lock_row
			-- Try to lock `row'
		local
			error: ROW_LOCK_ERROR
		do
			if not row.is_locked then
				row.try_to_lock_nowait
				if row.is_locked_and_unchanged then
					publish_ok_event
				else
					publish_failed_event
					create error
					error.set_hard_error
					if row.is_busy then
						error.enable_retry
					else
						error.disable_retry
					end
					error.set_row (row)
					error.handle
				end
			end
		end

	publish_ok_event
		local
			ok_event: ROW_LOCK_OK_EVENT
		do
			create ok_event
			ok_event.set_row (row)
			ok_event.publish
		end

	publish_failed_event
		local
			failed_event: ROW_LOCK_FAILED_EVENT
		do
			create failed_event
			failed_event.set_row (row)
			failed_event.publish
		end

end
