
indexing
	
	description: "A run closing step operation that runs %
		%accounting reports for the run"
	author: "David Hollenberg"
	date: "April 6, 2004"

class RUN_ACCOUNTING_REPORTS_OP
inherit
	RC_PRIMITIVE_STEP_OP
	RC_APPLICABLE_STEP
	NO_USER_INTERFACE
	CREATES_EXTERNAL_STATE

create
	make

feature -- Properties

	input_data_items: LIST [STRING]
			-- Data items required before step can be executed
		do
			Result := array_as_list (<<
					data_item.Tapeout_date,
					data_item.Turnaround_estimates,
					data_item.Locked_production_charges
				 >>)
		end

feature {NONE} -- Implementation

	do_step
			-- Execute step
		local
			row: RUN_EVENTS_ROW
			event_name: EVENT_NAME_VALUES
		do
			run_one_report (environment.line_item_report, std_parameters + " p_production_id=" + session.run_id, line_item_report_output_file.file_pathname)
			process_pending_abort
			run_one_report (environment.sales_order_report, std_parameters, sales_order_report_output_file.file_pathname)
			process_pending_abort
			check_payload_report_empty
			run_one_report (environment.payload_and_charges_report, std_parameters, payload_charges_report_output_file.file_pathname)
			process_pending_abort
			run_one_report (environment.costs_and_margin_report, std_parameters + " p_production_id=" + session.run_id + " p_command_line_flag=Y", costs_and_margin_report_output_file.file_pathname)
			if not payload_and_charges_ok then
				inform ("Reports were run, but step cannot complete until payload and charges%Nreport is not empty.")
				abort
			end
			create row.make_by_key (connection, option.Old_no_exception_row, session.run_id, event_name.Fab_posted)
			if not row.is_old then
				inform ("Reports were run, but step cannot complete until " + event_name.Fab_posted + " event exists")
				abort
			end
			call_completion_actions
		end

	undo_step
			-- Undo execution of step
		do
			line_item_report_output_file.delete_if_exists
			payload_charges_report_output_file.delete_if_exists
			costs_and_margin_report_output_file.delete_if_exists
			sales_order_report_output_file.delete_if_exists
			call_completion_actions
		end

	set_state
			-- Set `state'
		do
			state := Void
		end

	check_payload_report_empty
			-- Check whether payload and charges report will
			-- be empty because there is no vendor work order
			-- with a line item whose service is of the
			-- appropriate type.  Issue a warning if report
			-- will be empty
		local
			service: VENDOR_SERVICE_VALUES
		do
			payload_and_charges_ok := cache.productions_row.is_payload_and_charges_nonempty
			if not payload_and_charges_ok then
				warn_with_notify ("no vendor work orders with line item of type " + service.Wafer + " or " + service.Foundryshr + ".%NPayload and charges report will be empty.%N")
			end
		end

	line_item_report_output_file: RC_FILE
			-- Output file from running the line item report
		do
			Result := file_generator.line_item_report_output_file
		ensure
			result_exists: Result /= Void
		end

	payload_charges_report_output_file: RC_FILE
			-- Output file from running the payload and
			-- charges report
		do
			Result := file_generator.payload_charges_report_output_file
		ensure
			result_exists: Result /= Void
		end

	costs_and_margin_report_output_file: RC_FILE
			-- Output file from running the costs and
			-- margin report
		do
			Result := file_generator.costs_and_margin_report_output_file
		ensure
			result_exists: Result /= Void
		end

	sales_order_report_output_file: RC_FILE
			-- Output file from running the sales order
			-- report
		do
			Result := file_generator.sales_order_report_output_file
		ensure
			result_exists: Result /= Void
		end

	std_parameters: STRING
			-- Standard parameters passed to all accounting
			-- reports
		do
			Result := "runid=" + session.run_id
		ensure
			result_exists: Result /= Void
		end

	payload_and_charges_ok: BOOLEAN
			-- Will the payload and charges report produce
			-- a non-empty result?

end
