
indexing

	description: "A database cache for a specific run"
	author: "David Hollenberg"
	date: "January 27, 2004"

class RUN_CACHE
inherit
	ANY
	ITEM_COUNT_RANGES_ACCESS
		export
			{NONE} all
		end
	DB_OPTIONS_ACCESS
		export
			{NONE} all
		end
	DB_QUERY_OPTIONS
		export
			{NONE} all
		end
	RC_UTILITIES_ACCESS
		export
			{NONE} all
		end
	NUMBER_UTILITIES_ACCESS
		export
			{NONE} all
		end
	CONDITION_UTILITIES
		export
			{NONE} all
		end
create
	make

feature {NONE} -- Creation

	make (s: STRING conn: DB_CONNECTION gen: RC_FILE_GENERATOR)
		require
			string_exists: s /= Void
			conn_exists: conn /= Void
			conn_open: conn.is_open
			generator_exists: gen /= Void
		do
			run_id := s
			connection := conn
			file_generator := gen
		ensure
			run_id_set: run_id = s
			connection_set: connection = conn
			generator_set: file_generator = gen
		end

feature -- Properties

	run_id: STRING
			-- Run id of run to which cache applies

	production_id: STRING
			-- Production id to which cache applies
		do
			Result := productions_row.production_id
		end

	connection: DB_CONNECTION
			-- Database connection to use

	file_generator: RC_FILE_GENERATOR
			-- File generator for run

	run_id_for_overhead: STRING
			-- Run id to be placed in overhead around each die
		do
			Result := runs_row.run_id_for_overhead
		ensure
			result_exists: Result /= Void
		end

	mask_techn: STRING
			-- Mask techn
		do
			Result := mask_sets_row.mask_techn
		ensure
			result_exists: Result /= Void
		end

	wafer_techn: STRING
			-- Wafer techn
		do
			Result := runs_row.wafer_techn
		ensure
			result_exists: Result /= Void
		end

	field_set_name: STRING
			-- Name of field set
		do
			Result := runs_row.field_set_name
		ensure
			result_exists: Result /= Void
		end

	closer: STRING
			-- Run closer
		do
			Result := runs_row.closer
		ensure
			result_exists: Result /= Void
		end

	clerk: STRING
			-- Production clerk
		do
			Result := productions_row.clerk
		ensure
			result_exists: Result /= Void
		end

	design_engineer: STRING
			-- Design engineer for run
		do
			Result := runs_row.design_engineer
		ensure
			result_exists: Result /= Void
		end

	test_engineer: STRING
			-- Test engineer for run
		do
			Result := productions_row.test_engineer
		ensure
			result_exists: Result /= Void
		end

	is_run_complete: BOOLEAN
			-- Is INITIAL_RUN production for run complete?
		do
			Result := productions_row.is_completed
		end

	run_type: STRING
			-- Run type
		do
			Result := runs_row.run_type
		ensure
			result_exists: Result /= Void
		end

	is_educational_run: BOOLEAN
			-- Is this an educational run?
		do
			Result := runs_row.educational
		end

	is_taxi_run: BOOLEAN
			-- Is this a taxi run?
		do
			Result := productions_row.is_taxi
		end

	is_scheduled_run: BOOLEAN
			-- Is this a scheduled run?
		require
			run_type_valid: is_nonempty_string (run_type)
		do
			Result := runs_row.is_scheduled_run
		end

	is_dedicated_run: BOOLEAN
			-- Is this a dedicated run?
		require
			run_type_valid: is_nonempty_string (run_type)
		do
			Result := runs_row.is_dedicated_run
		end

	is_trusted_foundry_run: BOOLEAN
			-- Is this a trusted foundry run?
		require
			run_type_valid: is_nonempty_string (run_type)
		do
			Result := runs_row.is_trusted_foundry_run
		end

	is_development_run: BOOLEAN
			-- Is this a development run?
		require
			run_type_valid: is_nonempty_string (run_type)
		do
			Result := runs_row.is_development_run
		end

	wafer_part_id_source: STRING
			-- Source for wafer part id
		do
			Result := productions_row.wafer_part_id_source
		end

	wafer_part_id: STRING
			-- Wafer part id
		do
			Result := productions_row.wafer_part_id
		end

	wafer_techn_family: STRING
			-- Name of wafer techn family
		require
			wafer_techn_valid: is_nonempty_string (wafer_techn)
		do
			Result := wafer_techns_row.wafer_techn_family
		ensure
			result_exists: Result /= Void
		end

	run_techn: STRING
			-- Name of run techn
		require
			wafer_techn_valid: is_nonempty_string (wafer_techn)
		do
			Result := wafer_techns_row.run_techn
		ensure
			result_exists: Result /= Void
		end

	process_vdd: DOUBLE
			-- Vdd for all processes offered for wafer techn
		require
			wafer_techn_valid: is_nonempty_string (wafer_techn)
		local
			list: LIST [DOUBLE]
			b: DB_BIND_S
		do
			create b.make (wafer_techn)
			list := connection.selected_doubles ("process_vdd",
				"wafer_techns wt, wafer_processes wtp, wafer_techn_offered_processes wtop",
				"wt.wafer_techn_family = wtop.wafer_techn_family and wt.wafer_techn_id = wtop.wafer_techn_id and wtop.wafer_techn_family = wtp.wafer_techn_family and wtop.wafer_process_id = wtp.wafer_process_id and wt.wafer_techn = :1", Void, True, b)
			if list.count = 1 then
				Result := list.first
			else
				handle_hard_error (list.count.out + " distinct values of process_vdd for processes offered for wafer techn " + wafer_techn)
				Result := 0.0
			end
		end

	litho_size: STRING
			-- Lithography size identifier
		require
			field_set_name_valid: is_nonempty_string (field_set_name)
		do
			Result := field_sets_row.litho_size_id
		ensure
			result_exists: Result /= Void
		end

	litho_type: STRING
			-- Lithography type identifier
		require
			wafer_techn_valid: is_nonempty_string (wafer_techn)
		do
			Result := wafer_techns_row.litho_type
		ensure
			result_exists: Result /= Void
		end

	packing_target: STRING
			-- Packing target
		require
			field_set_name_valid: is_nonempty_string (field_set_name)
		do
			Result := litho_sizes_row.packing_target
		ensure
			result_exists: Result /= Void
		end

	is_custom_packing_target: BOOLEAN
			-- Is packing target a custom target?
		require
			field_set_name_valid: is_nonempty_string (field_set_name)
		do
			Result := litho_sizes_row.is_custom_target
		end

	is_reticle_packing_target: BOOLEAN
			-- Is packing target a (standard or custom) reticle?
		require
			field_set_name_valid: is_nonempty_string (field_set_name)
		do
			Result := litho_sizes_row.is_reticle
		end

	is_die_merge: BOOLEAN
			-- Is this a die merge (merge overhead with
			-- each individual die only)?
		require
			needs_gds_merge: run_needs_merged_gds_files
		do
			Result := litho_sizes_row.is_die_merge
		end

	is_field_merge: BOOLEAN
			-- Is this a field merge (merge all layout in
			-- each field)?
		require
			needs_gds_merge: run_needs_merged_gds_files
		do
			Result := litho_sizes_row.is_field_merge
		end

	has_mask_size_id: BOOLEAN
			-- Does litho size for run have a mask size ID?
		require
			field_set_name_valid: is_nonempty_string (field_set_name)
		do
			Result := not litho_sizes_row.is_null_mask_size_id
		end

	mask_size_id: STRING
			-- Mask size identifier
		require
			field_set_name_valid: is_nonempty_string (field_set_name)
		do
			Result := litho_sizes_row.mask_size_id
		ensure
			result_exists: Result /= Void
		end

	wafer_type_id: STRING
			-- Wafer type identifier
		require
			field_set_name_valid: is_nonempty_string (field_set_name)
		do
			Result := litho_sizes_row.non_null_wafer_type_id
		ensure
			result_exists: Result /= Void
		end

	pattern_file_renaming_style: STRING
			-- Style of pattern file renaming required by
			-- mask shop
		require
			field_set_name_valid: is_nonempty_string (field_set_name)
		do
			Result := litho_sizes_row.pattern_file_renaming_style
		ensure
			result_exists: Result /= Void
		end

	primary_flat_location: STRING
			-- Location of primary flat(s)
		require
			field_set_name_valid: is_nonempty_string (field_set_name)
		do
			Result := litho_sizes_row.non_null_primary_flat_location
		ensure
			result_exists: Result /= Void
		end

	wafer_primary_flat_box: CONST_RECTANGLE
			-- Rectangle enclosing wafer with flat location(s)
			-- taken into account
		require
			field_set_name_valid: is_nonempty_string (field_set_name)
		local
			r, indent, low_x, low_y, high_x, high_y: DOUBLE
			lsr: LITHO_SIZES_ROW
		do
			r := wafer_radius
			indent := wafer_types_row.max_primary_flat_indent
			lsr := litho_sizes_row
			low_x := -r
			if lsr.has_left_flat then
				low_x := low_x + indent
			end
			low_y := -r
			if lsr.has_bottom_flat then
				low_y := low_y + indent
			end
			high_x := r
			if lsr.has_right_flat then
				high_x := high_x - indent
			end
			high_y := r
			if lsr.has_top_flat then
				high_y := high_y - indent
			end
			create Result.make_by_corners (low_x, low_y, high_x, high_y)
		ensure
			result_exists: Result /= Void
		end

	tweezer_primary_flat_box: CONST_RECTANGLE
			-- Rectangle enclosing wafer with flat location(s)
			-- taken into account, with tweezer margin subtracted
		require
			field_set_name_valid: is_nonempty_string (field_set_name)
		local
			rect: CONST_RECTANGLE
			margin: CONST_POINT
			tm: DOUBLE
		do
			rect := wafer_primary_flat_box
			tm := tweezer_margin
			create margin.make_xy (tm, tm)
			create Result.make_corners (rect.ll + margin, rect.ur - margin)
		ensure
			result_exists: Result /= Void
		end

	wafer_diameter: DOUBLE
			-- Wafer diameter
		require
			field_set_name_valid: is_nonempty_string (field_set_name)
		do
			Result := wafer_types_row.non_null_wafer_diameter
		end

	wafer_radius: DOUBLE
			-- Wafer radius
		require
			field_set_name_valid: is_nonempty_string (field_set_name)
		do
			Result := wafer_diameter / 2.0
		end

	tweezer_margin: DOUBLE
			-- Tweezer margin
		require
			field_set_name_valid: is_nonempty_string (field_set_name)
		do
			Result := litho_sizes_row.tweezer_margin
		end

	tweezer_radius: DOUBLE
			-- Wafer radius minus tweezer margin
		require
			field_set_name_valid: is_nonempty_string (field_set_name)
		do
			Result := wafer_radius - tweezer_margin
		end

	designated_design_techn: STRING
			-- Name of designated design technology
		do
			Result := runs_row.designated_design_techn
		ensure
			result_exists: Result /= Void
		end

	substitution_parameter_set: SUBSTITUTION_PARAMETER_SET
			-- Substitution parameters for run, used to
			-- translate parameterized text strings into
			-- final text strings with parameters replaced
			-- by their values
		require
			wafer_part_id_exists: wafer_part_id /= Void
			mask_techn_valid: is_nonempty_string (mask_techn)
		do
			create Result.make (13)
			Result.bind ("RUN", run_id)
			Result.bind ("PARTNO", wafer_part_id)
			Result.bind ("MASKID", mask_techns_row.title_short_name)
		ensure
			result_exists: Result /= Void
		end

	slice_code: INTEGER
			-- Slice code for run
		require
			field_set_name_valid: is_nonempty_string (field_set_name)
			mask_techn_valid: is_nonempty_string (mask_techn)
		do
			Result := job_deck_slice_codes_row.slice_code
		end

	virtual_slice_code: INTEGER
			-- Slice code for run whose litho size does not have
			-- a mask_size_id
		require
			field_set_name_valid: is_nonempty_string (field_set_name)
			no_mask_size_id: not has_mask_size_id
		local
			row: like cached_job_deck_slice_codes_row
			type: EBEAM_TYPE_VALUES
		do
			create row.make_template
			Result := row.slice_code_for_size_and_type (virtual_mask_size_id, type.Mebes, connection, expected_count.exactly_one).first.slice_code
		end

	virtual_mask_size_id: STRING
			-- Mask size ID for run whose litho size does not have
			-- a mask_size_id
		local
			msize: MASK_SIZE_ID_VALUES
		do
			Result := msize.Size_6_inch
		end

	mask_title_orientations: LIST [STRING]
			-- Mask title orientations for run
		require
			wafer_techn_family_valid: is_nonempty_string (wafer_techn_family)
		local
			rows: like mask_title_orientation_rows
		do
			rows := mask_title_orientation_rows
			create {ARRAYED_LIST [STRING]} Result.make (rows.count)
			mask_title_orientation_rows.do_all (agent add_title_orientation (Result, ?))
		ensure
			result_exists: Result /= Void
		end

	run_needs_merged_gds_files: BOOLEAN
			-- Do we need to produce merged GDS files for run?
		require
			mask_techn_valid: is_nonempty_string (mask_techn)
			wafer_techn_valid: is_nonempty_string (wafer_techn)
		do
			Result := mask_techns_row.needs_gds_files and wafer_techns_row.needs_merged_gds_files
		end

	run_needs_mebes_files: BOOLEAN
			-- Do we need to produce MEBES files for run?
		require
			mask_techn_valid: is_nonempty_string (mask_techn)
		do
			Result := mask_techns_row.needs_mebes_files
		end

	run_needs_merge: BOOLEAN
			-- Do we need to merge the MEBES or GDS files for run?
		require
			mask_techn_valid: is_nonempty_string (mask_techn)
			wafer_techn_valid: is_nonempty_string (wafer_techn)
		do
			Result := run_needs_mebes_files and wafer_mask_techns_row.one_pattern_file_per_field
		end

	run_needs_fill: BOOLEAN
			-- Do we need to fill the merged MEBES files for run?
		require
			mask_techn_valid: is_nonempty_string (mask_techn)
			wafer_techn_valid: is_nonempty_string (wafer_techn)
		do
			Result := run_needs_merge and wafer_techns_row.mosis_fills
		end

	status_directory: STRING
			-- Directory where run status report should
			-- be generated
		local
			b: DB_BIND_S
		do
			create b.make (run_id)
			Result := connection.selected_string ("m_lookup.status_location(:1)", "dual", Void, b)
		ensure
			result_exists: Result /= Void
		end

	admin_application_parameter_string (sect, app, parm: STRING): STRING
			-- String value for admin application parameter
			-- `parm' for application `app' in section `sect'
		require
			app_exists: app /= Void
			parm_exists: parm /= Void
			sect_exists: sect /= Void
		local
			row: ADMIN_APPLICATION_PARAMETERS_ROW
		do
			create row.make_template
			Result := row.admin_parameter_value (sect, app, parm, connection, expected_count.Exactly_one).first.char_value
		ensure
			result_exists: Result /= Void
		end

	distinct_commercial_funding_allocs: LIST [INTEGER]
			-- List of distinct funding allocations for work
			-- orders on run for commercial accounts
		require
			projects_selected: -- Projects have been selected
		local
			b: DB_BIND_SSS
			ic_fab_instance_type: IC_FAB_INSTANCE_TYPE_VALUES
			cust_acct_type: CUST_ACCT_TYPE_VALUES
		do
			create b.make (run_id, ic_fab_instance_type.Customer_fab, cust_acct_type.Com)
			Result := connection.selected_integers ("m_funding_alloc_no", "cust_wos", "production_id = :1 and m_fab_no in (select m_fab_no from ic_fab_instances where ic_fab_instance_type = :2) and cust_acct_no in (select cust_acct_no from cust_accts where acct_type = :3)", Void, True, b)
		ensure
			result_exists: Result /= Void
		end

	commercial_project_count: INTEGER
			-- Number of projects on run which are being
			-- paid for by a commercial account
		require
			projects_selected: -- Projects have been selected
		local
			b: DB_BIND_SSS
			ic_fab_instance_type: IC_FAB_INSTANCE_TYPE_VALUES
			cust_acct_type: CUST_ACCT_TYPE_VALUES
		do
			create b.make (run_id, ic_fab_instance_type.Customer_fab, cust_acct_type.Com)
			Result := connection.selected_integers ("m_fab_no", "cust_wos", "production_id = :1 and m_fab_no in (select m_fab_no from ic_fab_instances where ic_fab_instance_type = :2) and cust_acct_no in (select cust_acct_no from cust_accts where acct_type = :3)", Void, True, b).count
		ensure
			nonnegative_result: Result >= 0
		end

	db_runs_row: RUNS_ROW
			-- Read-only row from RUNS table, fetched from database
		do
			Result := db_runs_mrow
		ensure
			result_exists: Result /= Void
		end

	db_runs_mrow: RUNS_MROW
			-- Modifiable row from RUNS table, fetched from
			-- database
		do
			create Result.make_by_key (connection, option.Old_row, run_id)
		ensure
			result_exists: Result /= Void
		end

	runs_row: RUNS_ROW
			-- Read-only row from RUNS table, possibly from cache
		do
			Result := runs_mrow
		ensure
			result_exists: Result /= Void
		end

	runs_mrow: RUNS_MROW
			-- Modifiable row from RUNS table, possibly from cache
		do
			if is_invalid_row (cached_runs_mrow) then
				cached_runs_mrow := db_runs_mrow
			end
			Result := cached_runs_mrow
		ensure
			result_exists: Result /= Void
		end

	db_productions_row: PRODUCTIONS_ROW
			-- Read-only row from PRODUCTIONS table, fetched
			-- from database
		do
			Result := db_productions_mrow
		ensure
			result_exists: Result /= Void
		end

	productions_row: PRODUCTIONS_ROW
			-- Read-only row from PRODUCTIONS table,
			-- possibly from cache
		do
			Result := productions_mrow
		ensure
			result_exists: Result /= Void
		end

	productions_mrow: PRODUCTIONS_MROW
			-- Modifiable row from PRODUCTIONS table,
			-- possibly from cache
		do
			if is_invalid_row (cached_productions_mrow) then
				cached_productions_mrow := db_productions_mrow
			end
			Result := cached_productions_mrow
		ensure
			result_exists: Result /= Void
		end

	db_mask_sets_row: MASK_SETS_ROW
			-- Read-only row from MASK_SETS table, fetched
			-- from database
		do
			Result := db_mask_sets_mrow
		ensure
			result_exists: Result /= Void
		end

	mask_sets_row: MASK_SETS_ROW
			-- Read-only row from MASK_SETS table,
			-- possibly from cache
		do
			Result := mask_sets_mrow
		ensure
			result_exists: Result /= Void
		end

	mask_sets_mrow: MASK_SETS_MROW
			-- Modifiable row from MASK_SETS table,
			-- possibly from cache
		do
			if is_invalid_row (cached_mask_sets_mrow) then
				cached_mask_sets_mrow := db_mask_sets_mrow
			end
			Result := cached_mask_sets_mrow
		ensure
			result_exists: Result /= Void
		end

	db_commercial_cust_wo_rows: LIST [CUST_WOS_ROW]
			-- List of read-only rows for commercial customer
			-- work orders for run, fetched from database
		require
			projects_selected: -- Projects have been selected
		local
			row: CUST_WOS_ROW
		do
			create row.make_template
			Result := row.commercial_work_orders_for_run (run_id, connection, expected_count.At_least_zero)
		ensure
			result_exists: Result /= Void
		end

	wafer_techns_row: WAFER_TECHNS_ROW
			-- Read-only row from WAFER_TECHNS table, possibly
			-- from cache
		require
			wafer_techn_valid: is_nonempty_string (wafer_techn)
		do
			if cached_wafer_techns_row = Void then
				cached_wafer_techns_row := db_wafer_techns_row
			end
			Result := cached_wafer_techns_row
		ensure
			result_exists: Result /= Void
		end

	wafer_techn_families_row: WAFER_TECHN_FAMILIES_ROW
			-- Read-only row from WAFER_TECHN_FAMILIES table,
			-- possibly from cache
		require
			wafer_techn_valid: is_nonempty_string (wafer_techn)
		do
			if cached_wafer_techn_families_row = Void then
				cached_wafer_techn_families_row := db_wafer_techn_families_row
			end
			Result := cached_wafer_techn_families_row
		ensure
			result_exists: Result /= Void
		end

	wafer_techn_run_types_row: WAFER_TECHNXRUN_TYPES_ROW
			-- Read-only row from WAFER_TECHN#RUN_TYPES table,
			-- possibly from cache
		require
			wafer_techn_valid: is_nonempty_string (wafer_techn)
			run_type_valid: is_nonempty_string (run_type)
		do
			if cached_wafer_techn_run_types_row = Void then
				cached_wafer_techn_run_types_row := db_wafer_techn_run_types_row
			end
			Result := cached_wafer_techn_run_types_row
		ensure
			result_exists: Result /= Void
		end

	wt_organizations_row: ORGANIZATIONS_ROW
			-- Read-only row from ORGANIZATIONS table for
			-- organization for wafer techn, possibly
			-- from cache
		require
			wafer_techn_valid: is_nonempty_string (wafer_techn)
		do
			if cached_wt_organizations_row = Void then
				cached_wt_organizations_row := db_wt_organizations_row
			end
			Result := cached_wt_organizations_row
		ensure
			result_exists: Result /= Void
		end

	mask_techns_row: MASK_TECHNS_ROW
			-- Read-only row from MASK_TECHNS table, possibly
			-- from cache
		require
			mask_techn_valid: is_nonempty_string (mask_techn)
		do
			if cached_mask_techns_row = Void then
				cached_mask_techns_row := db_mask_techns_row
			end
			Result := cached_mask_techns_row
		ensure
			result_exists: Result /= Void
		end

	mt_organizations_row: ORGANIZATIONS_ROW
			-- Read-only row from ORGANIZATIONS table for
			-- organization for mask techn, possibly
			-- from cache
		require
			mask_techn_valid: is_nonempty_string (mask_techn)
		do
			if cached_mt_organizations_row = Void then
				cached_mt_organizations_row := db_mt_organizations_row
			end
			Result := cached_mt_organizations_row
		ensure
			result_exists: Result /= Void
		end

	wafer_mask_techns_row: WAFER_TECHNXMASK_TECHNS_ROW
			-- Read-only row from WAFER_TECHN#MASK_TECHNS table,
			-- possibly from cache
		require
			wafer_techn_valid: is_nonempty_string (wafer_techn)
			mask_techn_valid: is_nonempty_string (mask_techn)
		do
			if cached_wafer_mask_techns_row = Void then
				cached_wafer_mask_techns_row := db_wafer_mask_techns_row
			end
			Result := cached_wafer_mask_techns_row
		ensure
			result_exists: Result /= Void
		end

	packaging_vendor_org_rows: LIST [ORGANIZATIONS_ROW]
			-- List of read-only rows from ORGANIZATIONS table for
			-- organization that are packaging vendors, bump
			-- vendors or test vendors, possibly from cache
		do
			if cached_packaging_vendor_org_rows = Void then
				cached_packaging_vendor_org_rows := db_packaging_vendor_org_rows
			end
			Result := cached_packaging_vendor_org_rows
		ensure
			result_exists: Result /= Void
		end

	organizations_row_for_org_no (org_no: INTEGER): ORGANIZATIONS_ROW
			-- Read-only row from ORGANIZATIONS table for
			-- organization identified by `org_no',
			-- possibly from cache
		local
			row: ORGANIZATIONS_ROW
			table: like organizations_org_table
		do
			table := organizations_org_table
			table.search (org_no)
			if table.found then
				Result := table.found_item
			else
				create row.make_template
				Result := row.rows_with_org_no (org_no, connection, expected_count.Exactly_one).first
				table.put (Result, org_no)
				organizations_short_id_table.put (Result, Result.short_id)
			end
		ensure
			result_exists: Result /= Void
		end

	organizations_row_for_short_id (short_id: STRING): ORGANIZATIONS_ROW
			-- Read-only row from ORGANIZATIONS table for
			-- organization identified by `short_id',
			-- possibly from cache
		require
			short_id_exists: short_id /= Void
		local
			row: ORGANIZATIONS_ROW
			table: like organizations_short_id_table
		do
			table := organizations_short_id_table
			table.search (short_id)
			if table.found then
				Result := table.found_item
			else
				create row.make_template
				Result := row.rows_with_short_id (short_id, connection, expected_count.Exactly_one).first
				table.put (Result, short_id)
				organizations_org_table.put (Result, Result.org_no)
			end
		ensure
			result_exists: Result /= Void
		end

	field_sets_row: FIELD_SETS_ROW
			-- Read-only row from FIELD_SETS table,
			-- possibly from cache
		require
			field_set_name_valid: is_nonempty_string (field_set_name)
		do
			if cached_field_sets_row = Void then
				cached_field_sets_row := db_field_sets_row
			end
			Result := cached_field_sets_row
		ensure
			result_exists: Result /= Void
		end

	field_step_patterns_row: FIELD_STEP_PATTERNS_ROW
			-- Read-only row from FIELD_STEP_PATTERNS
			-- table for this run (as determined from the
			-- run_wafer_lots), possibly from cache.  Only
			-- uses wafer lots for INITIAL_RUN production
		local
			waf_lots: like wafer_lot_rows
			waf_lot: RUN_WAFER_LOTS_ROW
			step_pat_names: LIST [STRING]
			pat_name: STRING
			list: LIST [FIELD_STEP_PATTERNS_ROW]
		do
			waf_lots := wafer_lot_rows
			create {ARRAYED_LIST [FIELD_STEP_PATTERNS_ROW]} list.make (1)
			create {ARRAYED_LIST [STRING]} step_pat_names.make (1)
			step_pat_names.compare_objects
			from
				waf_lots.start
			until
				waf_lots.after
			loop
				waf_lot := waf_lots.item
				pat_name := waf_lot.field_step_pattern
				if waf_lot.run_id.is_equal (waf_lot.production_id) -- Initial run production
				   and not pat_name.is_empty and not step_pat_names.has (pat_name) then
					step_pat_names.extend (pat_name)
					list.extend (waf_lot.field_step_patterns_row)
				end
				waf_lots.forth
			end
			if list.is_empty then
				handle_hard_error ("no field step patterns for " + run_id)
			elseif list.count > 1 then
				handle_unsupported ("more than one field step pattern (" + list.count.out + ") for " + run_id)
			end
			Result := list.first
		ensure
			result_not_void: Result /= Void
		end

	litho_sizes_row: LITHO_SIZES_ROW
			-- Read-only row from LITHO_SIZES table,
			-- possibly from cache
		require
			field_set_name_valid: is_nonempty_string (field_set_name)
		do
			if cached_litho_sizes_row = Void then
				cached_litho_sizes_row := db_litho_sizes_row
			end
			Result := cached_litho_sizes_row
		ensure
			result_exists: Result /= Void
		end

	wafer_types_row: WAFER_TYPES_ROW
			-- Read-only row from WAFER_TYPES table,
			-- possibly from cache
		require
			field_set_name_valid: is_nonempty_string (field_set_name)
		do
			if cached_wafer_types_row = Void then
				cached_wafer_types_row := db_wafer_types_row
			end
			Result := cached_wafer_types_row
		ensure
			result_exists: Result /= Void
		end

	mask_sizes_row: MASK_SIZES_ROW
			-- Read-only row from MASK_SIZES table,
			-- possibly from cache
		require
			field_set_name_valid: is_nonempty_string (field_set_name)
		do
			if cached_mask_sizes_row = Void then
				cached_mask_sizes_row := db_mask_sizes_row
			end
			Result := cached_mask_sizes_row
		ensure
			result_exists: Result /= Void
		end

	virtual_mask_sizes_row: MASK_SIZES_ROW
			-- Read-only row from MASK_SIZES table, for
			-- run whose litho size does not have a mask
			-- size ID, possibly from cache
		require
			field_set_name_valid: is_nonempty_string (field_set_name)
			no_mask_size_id: not has_mask_size_id
		do
			if cached_virtual_mask_sizes_row = Void then
				cached_virtual_mask_sizes_row := db_virtual_mask_sizes_row
			end
			Result := cached_virtual_mask_sizes_row
		ensure
			result_exists: Result /= Void
		end

	job_deck_slice_codes_row: JOB_DECK_SLICE_CODES_ROW
			-- Read-only row from JOB_DECK_SLICE_CODES table,
			-- possibly from cache
		require
			field_set_name_valid: is_nonempty_string (field_set_name)
			mask_techn_valid: is_nonempty_string (mask_techn)
		do
			if cached_job_deck_slice_codes_row = Void then
				cached_job_deck_slice_codes_row := db_job_deck_slice_codes_row
			end
			Result := cached_job_deck_slice_codes_row
		ensure
			result_exists: Result /= Void
		end

	mask_title_orientation_rows: LIST [MASK_TITLE_ORIENTATIONS_ROW]
			-- List of read-only rows from MASK_TITLE_ORIENTATIONS
			-- table for wafer techn family for run,
			-- possibly from cache
		require
			wafer_techn_family_valid: is_nonempty_string (wafer_techn_family)
		do
			if cached_mask_title_orientation_rows = Void then
				cached_mask_title_orientation_rows := db_mask_title_orientation_rows
			end
			Result := cached_mask_title_orientation_rows
		ensure
			result_exists: Result /= Void
		end

	mask_title_rows: LIST [MASK_TITLES_ROW]
			-- List of read-only rows from MASK_TITLES
			-- table for wafer techn family for run and levels
			-- used by run, possibly from cache
		require
			wafer_techn_family_valid: is_nonempty_string (wafer_techn_family)
			processes_assigned: -- Processes assigned to projects
		do
			if cached_mask_title_rows = Void then
				cached_mask_title_rows := db_mask_title_rows
			end
			Result := cached_mask_title_rows
		ensure
			result_exists: Result /= Void
		end

	mask_techn_prolog_lines: LIST [STRING]
			-- List of prolog lines for mask techn and field set,
			-- possibly from cache
		require
			field_set_name_valid: is_nonempty_string (field_set_name)
			mask_techn_valid: is_nonempty_string (mask_techn)
		do
			if cached_mask_techn_prolog_lines = Void then
				cached_mask_techn_prolog_lines := db_mask_techn_prolog_lines
			end
			Result := cached_mask_techn_prolog_lines
		ensure
			result_exists: Result /= Void
		end

	mask_techn_epilog_lines: LIST [STRING]
			-- List of epilog lines for mask techn and field set,
			-- possibly from cache
		require
			field_set_name_valid: is_nonempty_string (field_set_name)
			mask_techn_valid: is_nonempty_string (mask_techn)
		do
			if cached_mask_techn_epilog_lines = Void then
				cached_mask_techn_epilog_lines := db_mask_techn_epilog_lines
			end
			Result := cached_mask_techn_epilog_lines
		ensure
			result_exists: Result /= Void
		end

	fill_patterns_row: FILL_PATTERNS_ROW
			-- Read-only row from FILL_PATTERNS table for pattern
			-- for wafer techn for run, possibly from cache
		require
			wafer_techn_valid: is_nonempty_string (wafer_techn)
		do
			if cached_fill_patterns_row = Void then
				cached_fill_patterns_row := db_fill_patterns_row
			end
			Result := cached_fill_patterns_row
		ensure
			result_exists: Result /= Void
		end

	mask_litho_types_row_for_name (name: STRING): MASK_LITHO_TYPES_ROW
			-- Read-only row from MASK_LITHO_TYPES table for
			-- `name', possibly from cache
		require
			name_valid: is_nonempty_string (name)
		do
			Result := mask_litho_types_table.item (name)
			if Result = Void then
				handle_hard_error ("cannot find MASK_LITHO_TYPES row for " + name)
			end
		ensure
			result_exists: Result /= Void
		end

	field_location_rows: LIST [FIELD_LOCATIONS_ROW]
			-- List of read-only rows from FIELD_LOCATIONS table
			-- for field set for run, possibly from cache
		require
			field_set_name_valid: is_nonempty_string (field_set_name)
		do
			if cached_field_location_rows = Void then
				cached_field_location_rows := db_field_location_rows
			end
			Result := cached_field_location_rows
		ensure
			result_exists: Result /= Void
		end

	field_location_rows_for_field (fld_name: STRING): LIST [FIELD_LOCATIONS_ROW]
			-- List of read-only rows from FIELD_LOCATIONS table
			-- for field set for run and field named `fld_name',
			-- possibly from cache
		require
			field_set_name_valid: is_nonempty_string (field_set_name)
			field_name_valid: is_nonempty_string (fld_name)
		do
			Result := field_locations_table.item (fld_name)
			if Result = Void then
				handle_hard_error ("cannot find FIELD_LOCATIONS rows for field " + fld_name)
			end
		ensure
			result_exists: Result /= Void
		end

	field_step_location_rows (pat_name: STRING): LIST [FIELD_STEP_PATTERNXFIELD_LOCS_ROW]
			-- List of read-only rows from
			-- FIELD_STEP_PATTERN#FIELD_LOCS table for step pattern
			-- named `pat_name', possibly from cache
		require
			pat_name_valid: is_nonempty_string (pat_name)
		local
			table: like field_step_locations_table
			row: FIELD_STEP_PATTERNXFIELD_LOCS_ROW
		do
			table := field_step_locations_table
			table.search (pat_name)
			if table.found then
				Result := table.found_item
			else
				create row.make_template
				Result := row.locations_for_step_pattern (pat_name, connection, expected_count.At_least_one)
				table.put (Result, pat_name)
			end
			if Result.is_empty then
				handle_hard_error ("cannot find FIELD_STEP_PATTERN#FIELD_LOCS rows for field step pattern " + pat_name)
			end
		ensure
			result_exists: Result /= Void
		end

	field_name_rows: LIST [FIELD_NAMES_ROW]
			-- List of read-only rows from FIELD_NAMES table
			-- for field set for run ordered by field name,
			-- possibly from cache
		require
			field_set_name_valid: is_nonempty_string (field_set_name)
		do
			if cached_field_name_rows = Void then
				cached_field_name_rows := db_field_name_rows
			end
			Result := cached_field_name_rows
		ensure
			result_exists: Result /= Void
		end

	field_names: LIST [STRING]
			-- List of field names for run, ordered by field name
		require
			field_set_name_valid: is_nonempty_string (field_set_name)
		local
			rows: like field_name_rows
		do
			rows := field_name_rows
			create {ARRAYED_LIST [STRING]} Result.make (rows.count)
			from
				rows.start
			until
				rows.after
			loop
				Result.extend (rows.item.field_name)
				rows.forth
			end
		ensure
			result_exists: Result /= Void
		end

	field_count: INTEGER
			-- Number of fields in field set for run
		require
			field_set_name_valid: is_nonempty_string (field_set_name)
		do
			Result := field_name_rows.count
		ensure
			result_positive: Result >= 1
		end

	design_technology_for_dt (dt: STRING): DESIGN_TECHNOLOGY
			-- DESIGN_TECHNOLOGY for design techn `dt', possibly from cache
		require
			design_techn_valid: is_nonempty_string (dt)
		local
			table: like design_technologies_table
		do
			table := design_technologies_table
			table.search (dt)
			if table.found then
				Result := table.found_item
			else
				create Result.make_by_key(connection, old_row, dt)
				table.put (Result, dt)
			end
		ensure
			result_exists: Result /= Void
		end

	xform_level_rows_for_dt (dt: STRING): LIST [DESIGN_TECHN_XFORM_LEVELS_ROW]
			-- List of read-only rows from
			-- DESIGN_TECHN_XFORM_LEVELS table for wafer
			-- techn family for run, design techn `dt', possibly from cache
		require
			wafer_techn_valid: is_nonempty_string (wafer_techn)
			design_techn_valid: is_nonempty_string (dt)
		local
			table: like xform_levels_table
			dtxl_row: DESIGN_TECHN_XFORM_LEVELS_ROW
			rows: like xform_level_rows_for_dt
		do
			table := xform_levels_table
			table.search (dt)
			if table.found then
				Result := table.found_item
			else
				create dtxl_row.make_template
				rows := dtxl_row.levels_for_wtf_and_dt (wafer_techn_family, dt, connection, expected_count.At_least_one)
				table.put (rows, dt)
				Result := rows
			end
		ensure
			result_exists: Result /= Void
		end

	xform_level_row_for_dt_level (dt: STRING n: INTEGER): DESIGN_TECHN_XFORM_LEVELS_ROW
			-- Read-only row from
			-- DESIGN_TECHN_XFORM_LEVELS table for wafer
			-- techn family for run, design techn `dt'
			-- and level number `n', possibly from cache
		require
			wafer_techn_valid: is_nonempty_string (wafer_techn)
			design_techn_valid: is_nonempty_string (dt)
		local
			rows: like xform_level_rows_for_dt
		do
			rows := xform_level_rows_for_dt (dt)
			from
				rows.start
			until
				rows.after or Result /= Void
			loop
				if rows.item.level_no = n then
					Result := rows.item
				end
				rows.forth
			end
			if Result = Void then
				handle_hard_error ("cannot find DESIGN_TECHN_XFORM_LEVEL rows for design techn " + dt + ", level " + n.out)
			end
		ensure
			result_exists: Result /= Void
		end

	die_merged_gds_layout_file (rgf: RUN_GDS_FILES_ROW): RC_FILE
			-- Layout file for merged GDS for die for 
			-- row `rgf'
		require
			rgf_not_void: rgf /= Void
		do
			Result := file_generator.die_merged_gds_layout_file (rgf.die_id)
		end
		
	die_merged_gds_log_file (rgf: RUN_GDS_FILES_ROW): RC_FILE
			-- Log file for GDS merge for die for row `rgf
		require
			rgf_not_void: rgf /= Void
		do
			Result := file_generator.die_merged_gds_log_file (rgf.die_id)
		end
		
	field_merged_gds_layout_file (rgf: RUN_GDS_FILES_ROW): RC_FILE
			-- Layout file for merged GDS for field for
			-- row `rgf'
		require
			rgf_not_void: rgf /= Void
		do
			Result := file_generator.field_merged_gds_layout_file (rgf.field_name)
		end
		
	field_merged_gds_log_file (rgf: RUN_GDS_FILES_ROW): RC_FILE
			-- Log file for GDS merge for field for row `rgf'
		require
			rgf_not_void: rgf /= Void
		do
			Result := file_generator.field_merged_gds_log_file (rgf.field_name)
		end
		
	die_fracture_script_file (die: DIES_ROW): RC_FILE
			-- Script file to fracture `die'
		require
			die_exists: die /= Void
		do
			Result := file_generator.die_fracture_script_file (die.die_id)
		ensure
			result_exists: Result /= Void
		end

	die_fracture_log_file (die: DIES_ROW): RC_FILE
			-- Log file for fracture of `die'
		require
			die_exists: die /= Void
		do
			Result := file_generator.die_fracture_log_file (die.die_id)
		ensure
			result_exists: Result /= Void
		end

	die_fracture_done_file (die: DIES_ROW): RC_FILE
			-- Done file for fracture of `die'
		require
			die_exists: die /= Void
		do
			Result := file_generator.die_fracture_done_file (die.die_id)
		ensure
			result_exists: Result /= Void
		end

	die_fracture_pattern_file (die: DIES_ROW mext: STRING): RC_FILE
			-- Pattern file for level with extension mext
		require
			die_exists: die /= Void
			wafer_techn_valid: is_nonempty_string (wafer_techn)
			mask_techn_valid: is_nonempty_string (mask_techn)
		do
			Result := file_generator.die_fracture_pattern_file (die.die_id, die.die_wafer_rotation, mask_techns_row.overetch, mext)
		end

	die_fracture_pattern_files (die: DIES_ROW): LIST [RC_FILE]
			-- Pattern files created by fracture of `die'
		require
			die_exists: die /= Void
			wafer_techn_valid: is_nonempty_string (wafer_techn)
			mask_techn_valid: is_nonempty_string (mask_techn)
			processes_assigned: -- Processes assigned to projects
		local
			levels: like run_level_rows
			f: RC_FILE
		do
			levels := run_level_rows
			create {ARRAYED_LIST [RC_FILE]} Result.make (levels.count)
			from
				levels.start
			until
				levels.after
			loop
				f := die_fracture_pattern_file (die, levels.item.non_null_mebes_extension)
				Result.extend (f)
				levels.forth
			end
		ensure
			result_exists: Result /= Void
		end

	level_merge_log_file (level: ABSTRACT_LEVEL): RC_FILE
			-- Log file for merge of `level'
		require
			level_exists: level /= Void
		do
			Result := file_generator.level_merge_log_file (level.non_null_mebes_extension)
		ensure
			result_exists: Result /= Void
		end

	level_merge_done_file (level: ABSTRACT_LEVEL): RC_FILE
			-- Done file for merge of `level'
		require
			level_exists: level /= Void
		do
			Result := file_generator.level_merge_done_file (level.non_null_mebes_extension)
		ensure
			result_exists: Result /= Void
		end

	level_merge_pattern_files (level: ABSTRACT_LEVEL): LIST [RC_FILE]
			-- Pattern files created by merge of `level'
		require
			level_exists: level /= Void
			field_set_name_valid: is_nonempty_string (field_set_name)
		local
			fcount: INTEGER
			f: RC_FILE
			k: INTEGER
		do
			fcount := field_count
			create {ARRAYED_LIST [RC_FILE]} Result.make (fcount)
			from
				k := 1
			until
				k > fcount
			loop
				f := file_generator.level_merge_pattern_file (k, fcount, level.non_null_mebes_extension)
				Result.extend (f)
				k := k + 1
			end
		ensure
			result_exists: Result /= Void
		end

	level_fill_log_file (level: ABSTRACT_LEVEL): RC_FILE
			-- Log file for fill of `level'
		require
			level_exists: level /= Void
		do
			Result := file_generator.level_fill_log_file (level.non_null_mebes_extension)
		ensure
			result_exists: Result /= Void
		end

	level_fill_done_file (level: ABSTRACT_LEVEL): RC_FILE
			-- Done file for fill of `level'
		require
			level_exists: level /= Void
		do
			Result := file_generator.level_fill_done_file (level.non_null_mebes_extension)
		ensure
			result_exists: Result /= Void
		end

	level_fill_pattern_files (level: ABSTRACT_LEVEL): LIST [RC_FILE]
			-- Pattern files created by fill job for `level'
		require
			level_exists: level /= Void
			level_filled: level.filled_layer
		do
			create {ARRAYED_LIST [RC_FILE]} Result.make (1)
			Result.extend (file_generator.level_fill_pattern_file (1, 1, level.non_null_mebes_extension))
				-- Assumes we don't fill in cases where
				-- more than one field
		ensure
			result_exists: Result /= Void
		end

	fill_selector_log_file: RC_FILE
			-- Log file for fill selector job
		do
			Result := file_generator.fill_selector_log_file
		ensure
			result_exists: Result /= Void
		end

	fill_selector_done_file: RC_FILE
			-- Done file for fill selector job
		do
			Result := file_generator.fill_selector_done_file
		ensure
			result_exists: Result /= Void
		end

	fill_selector_created_files: LIST [RC_FILE]
			-- Files created by fill selector job
		do
			create {ARRAYED_LIST [RC_FILE]} Result.make (2)
			Result.extend (file_generator.fill_selector_file)
			Result.extend (file_generator.fill_exclusion_zone_file)
			Result.extend (file_generator.fill_reset_file)
		ensure
			result_exists: Result /= Void
		end

	design_die_offset: CONST_POINT
			-- Offset of center of design from center of die
		obsolete "Use `external_design_die_offset' or `internal_design_die_offset' instead"
		require
			wafer_techn_valid: is_nonempty_string (wafer_techn)
		local
			wt: like wafer_techns_row
		do
			if not runs_row.is_null_uniform_override_border then
				create Result.make_xy (0.0, 0.0)
			else
				wt := wafer_techns_row
				create Result.make_xy (wt.design_die_offset_x, wt.design_die_offset_y)
			end
		ensure
			result_exists: Result /= Void
		end

	external_design_die_offset: CONST_POINT
			-- Offset of center of design from center of die
			-- for dies whose effective overhead style is EXTERNAL
		require
			wafer_techn_valid: is_nonempty_string (wafer_techn)
		do
			Result := runs_row.external_design_die_offset
		ensure
			result_not_void: Result /= Void
		end

	internal_design_die_offset: CONST_POINT
			-- Offset of center of design from center of die
			-- for dies whose effective overhead style is 
			-- not EXTERNAL
		require
			wafer_techn_valid: is_nonempty_string (wafer_techn)
		do
			Result := runs_row.internal_design_die_offset
		ensure
			result_not_void: Result /= Void
		end

	field_size: CONST_POINT
			-- Size of field for run, in wafer coordinates
		require
			field_set_name_valid: is_nonempty_string (field_set_name)
		do
			Result := litho_sizes_row.field_size
		ensure
			result_exists: Result /= Void
		end

	payload_size: CONST_POINT
			-- Size of payload for run, in wafer coordinates
		require
			field_set_name_valid: is_nonempty_string (field_set_name)
		do
			Result := litho_sizes_row.payload_size
		ensure
			result_exists: Result /= Void
		end

	payload_center: CONST_POINT
			-- Location of center of payload for run relative
			-- to center of field, in wafer coordinates
		require
			field_set_name_valid: is_nonempty_string (field_set_name)
		do
			Result := litho_sizes_row.payload_center
		ensure
			result_exists: Result /= Void
		end

	payload_box: CONST_RECTANGLE
			-- Rectangle that defines extent of payload in
			-- microns relative to center of field, in
			-- wafer coordinates
		require
			field_set_name_valid: is_nonempty_string (field_set_name)
		do
			Result := litho_sizes_row.payload_box
		ensure
			result_exists: Result /= Void
		end

	merged_size: CONST_POINT
			-- Size of merged pattern files for run, in mask
			-- coordinates
		require
			field_set_name_valid: is_nonempty_string (field_set_name)
		do
			Result := litho_sizes_row.merged_size
		ensure
			result_exists: Result /= Void
		end

	merged_center: CONST_POINT
			-- Location of center of merged pattern files for
			-- run relative to center of field, in mask coordinates
		require
			field_set_name_valid: is_nonempty_string (field_set_name)
		do
			Result := litho_sizes_row.merged_center
		ensure
			result_exists: Result /= Void
		end

	merged_box: CONST_RECTANGLE
			-- Rectangle that defines extent of merged
			-- pattern files in microns, relative to
			-- center of field in mask coordinates
		require
			field_set_name_valid: is_nonempty_string (field_set_name)
		do
			Result := litho_sizes_row.merged_box
		ensure
			result_exists: Result /= Void
		end

	merged_box_at_1x: CONST_RECTANGLE
			-- Rectangle that defines extent of merged
			-- pattern files in microns, relative to
			-- center of field at 1X
		require
			wafer_techn_valid: is_nonempty_string (wafer_techn)
			field_set_name_valid: is_nonempty_string (field_set_name)
		local
			row: like litho_sizes_row
			size, center: CONST_POINT
			factor: DOUBLE
		do
			factor := scale_factor
			row := litho_sizes_row
			size := row.merged_size / factor
			center := row.merged_center / factor
			create Result.make_size_and_center (size, center)
		ensure
			result_exists: Result /= Void
		end

	merge_limit_size_for_level (n: INTEGER): CONST_POINT
			-- Size of merge limit box for wafer techn level
			-- number `n'.  Uses designated design techn and
			-- lambda to do the computation
		require
			wafer_techn_valid: is_nonempty_string (wafer_techn)
			designated_dt_valid: is_nonempty_string (designated_design_techn)
			field_set_name_valid: is_nonempty_string (field_set_name)
			processes_assigned: -- Processes assigned to projects
		local
			wt_level: ABSTRACT_LEVEL
			dt_level: DESIGN_TECHN_XFORM_LEVELS_ROW
			mtl_row: MASK_LITHO_TYPES_ROW
			bias, designated_size: DOUBLE
			quotient: INTEGER
			odd: BOOLEAN
		do
			Result := litho_sizes_row.merged_size
			wt_level := run_level_row_for_level (n)
			dt_level := xform_level_row_for_dt_level (designated_design_techn, n)
			mtl_row := mask_litho_types_row_for_name (wt_level.non_null_mask_litho_type)
			designated_size := dt_level.mebes_spot_size
			
			bias := dt_level.total_nominal_bias
			if mask_techns_row.overetch then
				bias := bias + dt_level.overetch_bias
			end
			quotient := (bias.abs / (designated_size / mtl_row.scale_factor)).rounded
                	odd := (quotient \\ 2) = 1
			if odd then
				Result := Result + create {CONST_POINT}.make_xy (designated_size, designated_size)
			end
		ensure
			result_exists: Result /= Void
		end

	merge_segment_count_for_level (n: INTEGER): DOUBLE
			-- Number of segments in merged pattern file
			-- for level number `n'.  Uses designated
			-- design techn and lambda to do the
			-- computation
		require
			wafer_techn_valid: is_nonempty_string (wafer_techn)
			designated_dt_valid: is_nonempty_string (designated_design_techn)
			field_set_name_valid: is_nonempty_string (field_set_name)
			processes_assigned: -- Processes assigned to projects
		local
			wt_level: ABSTRACT_LEVEL
		do
			wt_level := run_level_row_for_level (n)
			Result := level_segment_count (n, wt_level.non_null_merged_pattern_file_spot_size)
		ensure
			result_positive: Result > 0.0
		end

	level_segment_count (n: INTEGER res: DOUBLE): DOUBLE
			-- Number of segments in merged or filled
			-- pattern file for level number `n', whose
			-- spot size is `res'.  Uses designated design
			-- techn and lambda to do the computation
		require
			resolution_positive: res > 0.0
			wafer_techn_valid: is_nonempty_string (wafer_techn)
			designated_dt_valid: is_nonempty_string (designated_design_techn)
			field_set_name_valid: is_nonempty_string (field_set_name)
			processes_assigned: -- Processes assigned to projects
		local
			size: CONST_POINT
		do
			size := merge_limit_size_for_level (n)
			Result := size.x / (res * Address_units_per_segment)
		ensure
			result_positive: Result > 0.0
		end

	mebes_format_for_merged_level (n: INTEGER): STRING
			-- MEBES format to use (MEBES or MEBES/4) for
			-- merged pattern file for level number `n'.
			-- Uses designated design techn and lambda to
			-- do the computation
		require
			wafer_techn_valid: is_nonempty_string (wafer_techn)
			designated_dt_valid: is_nonempty_string (designated_design_techn)
			field_set_name_valid: is_nonempty_string (field_set_name)
			processes_assigned: -- Processes assigned to projects
		do
			Result := mebes_format_for_segment_count (merge_segment_count_for_level (n))
		ensure
			result_exists: Result /= Void
		end

	level_mebes_format (n: INTEGER res: DOUBLE): STRING
			-- MEBES format to use (MEBES or MEBES/4) for
			-- pattern file for level number `n', whose spot
			-- size is `res'
		require
			resolution_positive: res > 0.0
			wafer_techn_valid: is_nonempty_string (wafer_techn)
			designated_dt_valid: is_nonempty_string (designated_design_techn)
			field_set_name_valid: is_nonempty_string (field_set_name)
			processes_assigned: -- Processes assigned to projects
		do
			Result := mebes_format_for_segment_count (level_segment_count (n, res))
		ensure
			result_exists: Result /= Void
		end

	mebes_format_for_segment_count (n: DOUBLE): STRING
			-- MEBES format to use for pattern file with
			-- `n' segments
		require
			segment_count_positive: n > 0.0
		do
			if n > Max_mebes3_segments then
				Result := "MEBES/4"
			else
				Result := "MEBES"
			end
		ensure
			result_exists: Result /= Void
		end

	scale_factor: DOUBLE
			-- Scale factor for all mask levels
			-- (assuming only one scale factor)
		require
			wafer_techn_valid: is_nonempty_string (wafer_techn)
		do
			Result := wafer_techn_level_rows.first.scale_factor
		end

	wafer_techn_family_level_rows: LIST [WAFER_TECHN_FAMILY_LEVELS_ROW]
			-- List of read-only rows for wafer techn family levels
			-- for run's wafer techn ordered by level number,
			-- possibly from cache
		require
			wafer_techn_valid: is_nonempty_string (wafer_techn)
		do
			if cached_wafer_techn_family_level_rows = Void then
				cached_wafer_techn_family_level_rows := db_wafer_techn_family_level_rows
			end
			Result := cached_wafer_techn_family_level_rows
		ensure
			result_exists: Result /= Void
		end

	wafer_techn_level_rows: LIST [WAFER_TECHN_LEVELS_ROW]
			-- List of read-only rows for wafer techn levels
			-- for run's wafer techn ordered by level number,
			-- possibly from cache
		require
			wafer_techn_valid: is_nonempty_string (wafer_techn)
		do
			if cached_wafer_techn_level_rows = Void then
				cached_wafer_techn_level_rows := db_wafer_techn_level_rows
			end
			Result := cached_wafer_techn_level_rows
		ensure
			result_exists: Result /= Void
		end

	wafer_techn_level_row_for_level (lev: INTEGER): WAFER_TECHN_LEVELS_ROW
			-- Read-only row for wafer techn level number `lev'
			-- for run, possibly from cache
		require
			wafer_techn_valid: is_nonempty_string (wafer_techn)
		local
			list: like wafer_techn_level_rows
		do
			list := wafer_techn_level_rows
			from
				list.start
			until
				list.after or Result /= Void
			loop
				if list.item.level_no = lev then
					Result := list.item
				end
				list.forth
			end
			if Result = Void then
				handle_hard_error ("cannot find WAER_TECHN_LEVELS row for level number " + lev.out)
			end
		ensure
			result_exists: Result /= Void
		end

	run_process_rows: LIST [WAFER_PROCESSES_ROW]
			-- List of read-only rows for wafer processes
			-- that are used on this run (as determined from
			-- the run_wafer_lots)
		require
			wafer_techn_valid: is_nonempty_string (wafer_techn)
		local
			wprs: like wafer_process_rows
			wlots: like wafer_lot_rows
			wp: WAFER_PROCESSES_ROW
		do
			wprs := wafer_process_rows
			wlots := wafer_lot_rows
			create {LINKED_LIST[WAFER_PROCESSES_ROW]}Result.make
			from	wprs.start
			until	wprs.after
			loop	wp := wprs.item
				if wp.is_found_in(wlots) then
					Result.extend(wp)
				end
				wprs.forth
			end
		end

	wafer_process_rows: LIST [WAFER_PROCESSES_ROW]
			-- List of read-only rows for wafer processes
			-- for run's wafer techn family, possibly from
			-- cache
		require
			wafer_techn_valid: is_nonempty_string (wafer_techn)
		do
			if cached_wafer_process_rows = Void then
				cached_wafer_process_rows := db_wafer_process_rows
			end
			Result := cached_wafer_process_rows
		ensure
			result_exists: Result /= Void
		end

	run_level_rows: LIST [RUN_LEVELS_ROW]
			-- List of read-only rows for wafer techn levels
			-- for run ordered by level number, possibly from cache
		require
			wafer_techn_valid: is_nonempty_string (wafer_techn)
			processes_assigned: -- Processes assigned to projects
		do
			if cached_run_level_rows = Void then
				cached_run_level_rows := db_run_level_rows
			end
			Result := cached_run_level_rows
		ensure
			result_exists: Result /= Void
		end

	run_level_row_for_level (lev: INTEGER): RUN_LEVELS_ROW
			-- Read-only row for wafer techn level number `lev'
			-- for run, possibly from cache
		require
			wafer_techn_valid: is_nonempty_string (wafer_techn)
			processes_assigned: -- Processes assigned to projects
		local
			list: like run_level_rows
		do
			list := run_level_rows
			from
				list.start
			until
				list.after or Result /= Void
			loop
				if list.item.level_no = lev then
					Result := list.item
				end
				list.forth
			end
			if Result = Void then
				handle_hard_error ("cannot find RUN_LEVELS row for level number " + lev.out)
			end
		ensure
			result_exists: Result /= Void
		end

	level_lcm_resolution: DOUBLE
			-- Least common multiple of all merged pattern
			-- file resolutions for levels for run that
			-- are involved in fill (filled level or exclusion
			-- level)
		require
			wafer_techn_valid: is_nonempty_string (wafer_techn)
			processes_assigned: -- Processes assigned to projects
		local
			list: like run_level_rows
			resolutions: ARRAYED_LIST [DOUBLE]
			row: RUN_LEVELS_ROW
		do
			list := run_level_rows
			create resolutions.make (list.count)
			from
				list.start
			until
				list.after
			loop
				row := list.item
				if row.exclusion_layer or row.filled_layer then
					resolutions.extend (row.non_null_merged_pattern_file_spot_size)
				end
				list.forth
			end
			Result := rc.least_common_multiple_list (resolutions)
		end

	die_rows: LIST [DIES_ROW]
			-- List of read-only rows for dies for run,
			-- possibly from cache
		do
			Result := die_mxrows
		ensure
			result_exists: Result /= Void
		end

	die_xrows: LIST [DIES_XROW]
			-- List of read-only xrows for dies for run,
			-- possibly from cache
		do
			Result := die_mxrows
		ensure
			result_exists: Result /= Void
		end

	die_mrows: LIST [DIES_MROW]
			-- List of modifiable rows for dies for run,
			-- possibly from cache
		do
			Result := die_mxrows
		ensure
			result_exists: Result /= Void
		end

	die_mxrows: LIST [DIES_MXROW]
			-- List of modifiable xrows for dies for run,
			-- possibly from cache
		do
			if is_invalid_list (cached_die_mrows) then
				cached_die_mrows := db_die_mrows
			end
			Result := cached_die_mrows
		ensure
			result_exists: Result /= Void
		end

	die_attribute_rows: LIST [DIE_ATTRIBUTES_ROW]
			-- List of read-only rows for die attributes for run,
			-- possibly from cache
		do
			if cached_die_attribute_rows = Void then
				cached_die_attribute_rows := db_die_attribute_rows
			end
			Result := cached_die_attribute_rows
		ensure
			result_exists: Result /= Void
		end

	non_ghost_die_attribute_rows: LIST [DIE_ATTRIBUTES_ROW]
			-- List of read-only rows for all die attributes on run
			-- that are not for vendor ghost dies, possibly from
			-- cache
		local
			list: like die_attribute_rows
			item: DIE_ATTRIBUTES_ROW
		do
			list := die_attribute_rows
			Result := list.twin
			Result.wipe_out
			from
				list.start
			until
				list.after
			loop
				item := list.item
				if not item.is_vendor_ghost then
					Result.extend (list.item)
				end
				list.forth
			end
		ensure
			result_exists: Result /= Void
		end

	die_location_rows: LIST [DIE_LOCATIONS_ROW]
			-- List of read-only rows for die locations for run,
			-- possibly from cache
		do
			Result := die_location_mrows
		ensure
			result_exists: Result /= Void
		end

	die_location_rows_for_die_id (die_id: STRING): LIST [DIE_LOCATIONS_ROW]
			-- List of read-only rows for die locations for
			-- `die_id', possibly from cache
		require
			die_id_valid: is_nonempty_string (die_id)
		do
			Result := die_locations_table.item (die_id)
			if Result = Void then
				handle_hard_error ("cannot find DIE_LOCATIONS rows for die id " + die_id)
			end
		ensure
			result_exists: Result /= Void
		end

	extended_die_locations_table: like cached_extended_die_locations_table
			-- Lists of extended die location rows for run and
			-- field, indexed by field name
		local
			rows, list: like die_location_erows
			table: like extended_die_locations_table
			location: DIE_LOCATIONS_EROW
			fld_name: STRING
		do
			if cached_extended_die_locations_table = Void then
				rows := die_location_erows
				create table.make (field_names.count)
				table.compare_objects
				from
					rows.start
				until
					rows.after
				loop
					location := rows.item
					fld_name := location.field_name
					table.search (fld_name)
					if table.found then
						table.found_item.extend (location)
					else
						list := rows.twin
						list.wipe_out
						list.extend (location)
						table.put (list, fld_name)
					end
					rows.forth
				end
				cached_extended_die_locations_table := table
			end
			Result := cached_extended_die_locations_table
		ensure
			result_exists: Result /= Void
		end

	all_mask_locations_for_die (die: ABSTRACT_DIE): LIST [CONST_POINT]
			-- List of locations on mask for `die',
			-- sorted by Y and by X within Y
		require
			die_exists: die /= Void
			wafer_techn_valid: is_nonempty_string (wafer_techn)
			field_set_name_valid: is_nonempty_string (field_set_name)
			projects_packed: -- Projects have been packed
		do
			Result := mask_locations (die.die_id, die_location_rows_for_die_id (die.die_id))
		ensure
			result_exists: Result /= Void
		end

	all_unmerged_mask_locations_for_die (die: ABSTRACT_DIE): LIST [CONST_POINT]
			-- List of locations on mask for `die_id' which are
			-- not inside merge box, sorted by Y and by X within Y
		require
			die_exists: die /= Void
			wafer_techn_valid: is_nonempty_string (wafer_techn)
			field_set_name_valid: is_nonempty_string (field_set_name)
			projects_packed: -- Projects have been packed
		local
			factor: DOUBLE
			loc_rows, unmerged_loc_rows: LIST [DIE_LOCATIONS_ROW]
			loc_row: DIE_LOCATIONS_ROW
			die_box, mrg_box: CONST_RECTANGLE
			inside, outside: BOOLEAN
		do
			factor := scale_factor
				-- Assuming one scale factor for all levels
			mrg_box := merged_box_at_1x
			loc_rows := die_location_rows_for_die_id (die.die_id)
			create {ARRAYED_LIST [DIE_LOCATIONS_ROW]} unmerged_loc_rows.make (loc_rows.count)
			
			from
				loc_rows.start
			until
				loc_rows.after
			loop
				loc_row := loc_rows.item
				create die_box.make_size_and_center (die.printed_size_on_wafer, loc_row.center)
				inside := mrg_box.encloses (die_box)
				outside := not mrg_box.intersects (die_box)
				if not inside and not outside then
					handle_hard_error ("die box straddles merge box for die id " + die.die_id + ", field " + loc_row.field_name + ",%Nposition (" + loc_row.center.out + ")")
				elseif outside then
					unmerged_loc_rows.extend (loc_row)
				end
				loc_rows.forth
			end
			Result := mask_locations (die.die_id, unmerged_loc_rows)
		ensure
			result_exists: Result /= Void
		end

	all_merged_mask_locations_for_field (fld_name: STRING): LIST [CONST_POINT]
			-- List of locations on mask for field `fld_name'
		require
			fld_name_valid: is_nonempty_string (fld_name)
			field_set_name_valid: is_nonempty_string (field_set_name)
			projects_packed: -- Projects have been packed
		local
			field_locs: like field_location_rows_for_field
			mrg_center: like merged_center
			sorter: LIST_BUBBLE_SORTER [CONST_POINT]
		do
			mrg_center := merged_center
			field_locs := field_location_rows_for_field (fld_name)
			create {ARRAYED_LIST [CONST_POINT]} Result.make (field_locs.count)
			from
				field_locs.start
			until
				field_locs.after
			loop
				Result.extend (field_locs.item.center + mrg_center)
				field_locs.forth
			end
			create sorter
			Result := sorter.sorted (Result, create {POINT_YX_COMPARATOR})
		ensure
			result_exists: Result /= Void
		end

	die_location_mrows: LIST [DIE_LOCATIONS_MROW]
			-- List of modifiable rows for die locations for run,
			-- possibly from cache
		do
			if is_invalid_list (cached_die_location_mrows) then
				cached_die_location_mrows := db_die_location_mrows
			end
			Result := cached_die_location_mrows
		ensure
			result_exists: Result /= Void
		end

	die_location_erows: LIST [DIE_LOCATIONS_EROW]
			-- List of read-only rows for extended die locations
			-- for run (include extra attributes),
			-- possibly from cache
		do
			if cached_die_location_erows = Void then
				cached_die_location_erows := db_die_location_erows
			end
			Result := cached_die_location_erows
		ensure
			result_exists: Result /= Void
		end

	db_wafer_lot_rows: LIST [RUN_WAFER_LOTS_ROW]
			-- List of read-only rows for wafer lots for run,
			-- fetched from database
		do
			Result := db_wafer_lot_mrows
		ensure
			result_exists: Result /= Void
		end

	wafer_lot_rows: LIST [RUN_WAFER_LOTS_ROW]
			-- List of read-only rows for wafer lots for run,
			-- possibly from cache
		do
			Result := wafer_lot_mrows
		ensure
			result_exists: Result /= Void
		end

	wafer_lot_mrows: LIST [RUN_WAFER_LOTS_MROW]
			-- List of modifiable rows for wafer lots for run,
			-- possibly from cache
		do
			if is_invalid_list (cached_wafer_lot_mrows) then
				cached_wafer_lot_mrows := db_wafer_lot_mrows
			end
			Result := cached_wafer_lot_mrows
		ensure
			result_exists: Result /= Void
		end

	first_wafer_lot_row: RUN_WAFER_LOTS_ROW
			-- Read-only rows for lowest numbered wafer lot for
			-- run, possibly from cache
		local
			list: like wafer_lot_rows
		do
			list := wafer_lot_rows
			if list.is_empty then
				handle_hard_error ("no rows in table RUN_WAFER_LOTS for run")
			end
			Result := list.first
		ensure
			result_exists: Result /= Void
		end

	field_step_pattern_name: STRING
			-- Name of field step pattern for lowest numbered
			-- wafer lot for run, possibly from cache
		do
			Result := first_wafer_lot_row.field_step_pattern
		ensure
			result_exists: Result /= Void
		end

	db_packaging_lot_rows: LIST [RUN_PACKAGING_LOTS_ROW]
			-- List of read-only rows for packaging lots for run,
			-- fetched from database
		do
			Result := db_packaging_lot_mrows
		ensure
			result_exists: Result /= Void
		end

	packaging_lot_rows: LIST [RUN_PACKAGING_LOTS_ROW]
			-- List of read-only rows for packaging lots for run,
			-- possibly from cache
		do
			Result := packaging_lot_mrows
		ensure
			result_exists: Result /= Void
		end

	packaging_lot_mrows: LIST [RUN_PACKAGING_LOTS_MROW]
			-- List of modifiable rows for packaging lots for run,
			-- possibly from cache
		do
			if is_invalid_list (cached_packaging_lot_mrows) then
				cached_packaging_lot_mrows := db_packaging_lot_mrows
			end
			Result := cached_packaging_lot_mrows
		ensure
			result_exists: Result /= Void
		end

	db_packaging_lot_mrows: LIST [RUN_PACKAGING_LOTS_MROW]
			-- List of modifiable rows for packaging lots for run,
			-- fetched from database
		local
			row: RUN_PACKAGING_LOTS_MROW
		do
			create row.make_template
			Result := row.lots_for_run (run_id, connection, expected_count.At_least_zero)
		ensure
			result_exists: Result /= Void
		end

	cut_pattern_rows: LIST [RUN_CUT_PATTERNS_ROW]
			-- List of read-only rows for cut patterns for run,
			-- possibly from cache
		do
			Result := cut_pattern_mrows
		ensure
			result_exists: Result /= Void
		end

	cut_pattern_mrows: LIST [RUN_CUT_PATTERNS_MROW]
			-- List of modifiable rows for cut patterns for run,
			-- possibly from cache
		do
			if is_invalid_list (cached_cut_pattern_mrows) then
				cached_cut_pattern_mrows := db_cut_pattern_mrows
			end
			Result := cached_cut_pattern_mrows
		ensure
			result_exists: Result /= Void
		end

	db_cut_pattern_mrows: LIST [RUN_CUT_PATTERNS_MROW]
			-- List of modifiable rows for cut patterns for run,
			-- fetched from database
		local
			row: RUN_CUT_PATTERNS_MROW
		do
			create row.make_template
			Result := row.rows_for_run (run_id, connection, expected_count.At_least_zero)
		ensure
			result_exists: Result /= Void
		end

	cut_location_rows: LIST [CUT_LOCATIONS_ROW]
			-- List of read-only rows for cut location for run,
			-- possibly from cache
		do
			Result := cut_location_mrows
		ensure
			result_exists: Result /= Void
		end

	cut_location_mrows: LIST [CUT_LOCATIONS_MROW]
			-- List of modifiable rows for cut location for run,
			-- possibly from cache
		do
			if is_invalid_list (cached_cut_location_mrows) then
				cached_cut_location_mrows := db_cut_location_mrows
			end
			Result := cached_cut_location_mrows
		ensure
			result_exists: Result /= Void
		end

	db_cut_location_mrows: LIST [CUT_LOCATIONS_MROW]
			-- List of modifiable rows for cut patterns for run,
			-- fetched from database
		local
			row: CUT_LOCATIONS_MROW
		do
			create row.make_template
			Result := row.rows_for_run (run_id, connection, expected_count.At_least_zero)
		ensure
			result_exists: Result /= Void
		end

	cut_die_site_rows: LIST [CUT_DIE_SITES_ROW]
			-- List of read-only rows for cut die sites for run,
			-- possibly from cache
		do
			Result := cut_die_site_mrows
		ensure
			result_exists: Result /= Void
		end

	cut_die_site_mrows: LIST [CUT_DIE_SITES_MROW]
			-- List of modifiable rows for cut die sites for run,
			-- possibly from cache
		do
			if is_invalid_list (cached_cut_die_site_mrows) then
				cached_cut_die_site_mrows := db_cut_die_site_mrows
			end
			Result := cached_cut_die_site_mrows
		ensure
			result_exists: Result /= Void
		end

	db_cut_die_site_mrows: LIST [CUT_DIE_SITES_MROW]
			-- List of modifiable rows for cut die sites for run,
			-- fetched from database
		local
			row: CUT_DIE_SITES_MROW
		do
			create row.make_template
			Result := row.rows_for_run (run_id, connection, expected_count.At_least_zero)
		ensure
			result_exists: Result /= Void
		end

	cust_die_die_rows: LIST [DIES_ROW]
			-- List of read-only rows for dies for customer dies
			-- on run, possibly from cache
		local
			list: like die_mrows
			item: DIES_MROW
		do
			list := die_mrows
			Result := list.twin
			Result.wipe_out
			from
				list.start
			until
				list.after
			loop
				item := list.item
				if item.is_cust_die then
					Result.extend (list.item)
				end
				list.forth
			end
		ensure
			result_exists: Result /= Void
		end

	test_vehicle_die_rows: LIST [DIES_ROW]
			-- List of read-only rows for dies for test vehicles
			-- on run, possibly from cache
		local
			list: like die_mrows
			item: DIES_MROW
		do
			list := die_mrows
			Result := list.twin
			Result.wipe_out
			from
				list.start
			until
				list.after
			loop
				item := list.item
				if item.is_test_vehicle then
					Result.extend (list.item)
				end
				list.forth
			end
		ensure
			result_exists: Result /= Void
		end

	probeable_die_rows: LIST [DIES_ROW]
			-- List of read-only rows for dies that
			-- can be probed, possibly from cache
		local
			list: like die_mrows
			item: DIES_MROW
		do
			list := die_mrows
			Result := list.twin
			Result.wipe_out
			from
				list.start
			until
				list.after
			loop
				item := list.item
				if item.is_probeable then
					Result.extend (list.item)
				end
				list.forth
			end
		ensure
			result_exists: Result /= Void
		end

	run_wafer_level_job_merge_rows: LIST [RUN_WAFER_LEVEL_JOBS_ROW]
			-- List of read-only rows for wafer level jobs for run
			-- for merge, possibly from cache
		do
			Result := run_wafer_level_job_merge_mrows
		ensure
			result_exists: Result /= Void
		end

	run_wafer_level_job_fill_rows: LIST [RUN_WAFER_LEVEL_JOBS_ROW]
			-- List of read-only rows for wafer level jobs for run
			-- for fill, possibly from cache
		do
			Result := run_wafer_level_job_fill_mrows
		ensure
			result_exists: Result /= Void
		end

	run_wafer_level_job_merge_mrows: LIST [RUN_WAFER_LEVEL_JOBS_MROW]
			-- List of modifiable rows for wafer level jobs for
			-- run for merge, possibly from cache
		do
			if is_invalid_list (cached_run_wafer_level_job_merge_mrows) then
				cached_run_wafer_level_job_merge_mrows := db_run_wafer_level_job_merge_mrows
			end
			Result := cached_run_wafer_level_job_merge_mrows
		ensure
			result_exists: Result /= Void
		end

	run_wafer_level_job_fill_mrows: LIST [RUN_WAFER_LEVEL_JOBS_MROW]
			-- List of modifiable rows for wafer level jobs for
			-- run for fill, possibly from cache
		do
			if is_invalid_list (cached_run_wafer_level_job_fill_mrows) then
				cached_run_wafer_level_job_fill_mrows := db_run_wafer_level_job_fill_mrows
			end
			Result := cached_run_wafer_level_job_fill_mrows
		ensure
			result_exists: Result /= Void
		end

	run_fill_selector_job_rows: LIST [RUN_JOBS_ROW]
			-- List of read-only rows for fill selector jobs
			-- for run, possibly from cache
		do
			Result := run_fill_selector_job_mrows
		ensure
			result_exists: Result /= Void
		end

	run_fill_selector_job_mrows: LIST [RUN_JOBS_MROW]
			-- List of modifiable rows for fill selector jobs
			-- for run, possibly from cache
		do
			if is_invalid_list (cached_run_fill_selector_job_mrows) then
				cached_run_fill_selector_job_mrows := db_run_fill_selector_job_mrows
			end
			Result := cached_run_fill_selector_job_mrows
		ensure
			result_exists: Result /= Void
		end

	run_gds_file_rows: LIST [RUN_GDS_FILES_ROW]
			-- List of read-only rows for GDS merge jobs 
			-- for run, possibly from cache
		do
			Result := run_gds_file_mrows
		ensure
			result_exists: Result /= Void
		end

	run_gds_file_mrows: LIST [RUN_GDS_FILES_MROW]
			-- List of modifiable rows for GDS merge jobs
			-- for run, possibly from cache
		do
			if is_invalid_list (cached_run_gds_file_mrows) then
				cached_run_gds_file_mrows := db_run_gds_file_mrows
			end
			Result := cached_run_gds_file_mrows
		ensure
			result_exists: Result /= Void
		end

	run_gds_file_layer_rows: LIST [RUN_GDS_FILE_LAYERS_ROW]
			-- List of read-only rows for layers GDS merge jobs 
			-- for run, possibly from cache
		do
			Result := run_gds_file_layer_mrows
		ensure
			result_exists: Result /= Void
		end

	run_gds_file_layer_mrows: LIST [RUN_GDS_FILE_LAYERS_MROW]
			-- List of modifiable rows for layers for GDS merge jobs
			-- for run, possibly from cache
		do
			if is_invalid_list (cached_run_gds_file_layer_mrows) then
				cached_run_gds_file_layer_mrows := db_run_gds_file_layer_mrows
			end
			Result := cached_run_gds_file_layer_mrows
		ensure
			result_exists: Result /= Void
		end

	run_gds_merge_result_rows: LIST [RUN_GDS_MERGE_RESULTS_ROW]
			-- List of read-only rows for GDS merge results
			-- for run, possibly from cache
		do
			Result := run_gds_merge_result_mrows
		ensure
			result_exists: Result /= Void
		end

	run_gds_merge_result_mrows: LIST [RUN_GDS_MERGE_RESULTS_MROW]
			-- List of modifiable rows for GDS merge results
			-- for run, possibly from cache
		do
			if is_invalid_list (cached_run_gds_merge_result_mrows) then
				cached_run_gds_merge_result_mrows := db_run_gds_merge_result_mrows
			end
			Result := cached_run_gds_merge_result_mrows
		ensure
			result_exists: Result /= Void
		end

feature -- Contract support

	is_nonempty_string (s: STRING): BOOLEAN
			-- Is `s' non-Void and non-empty?
		do
			Result := s /= Void and then not s.is_empty
		end

feature -- Modification

	uncache_all
			-- Clear entire cache
		do
			uncache_miscellaneous
			uncache_run
			uncache_mask_techn
			uncache_wafer_techn
			uncache_field_set
			uncache_dies
			uncache_die_locations
			uncache_run_wafer_level_jobs
			uncache_run_fill_selector_jobs
			uncache_run_gds_files
			uncache_run_gds_file_layers
			uncache_run_gds_merge_results
			uncache_wafer_lots
			uncache_packaging_lots
			uncache_cut_patterns
		end

	cache_run (r: RUNS_MROW)
			-- Set `runs_mrow' to `r'
		require
			row_exists: r /= Void
		do
			cached_runs_mrow := r
		ensure
			cached: runs_row = r
		end

	uncache_run
			-- Clear cached rows due to a change of run
		do
			cached_runs_mrow := Void
			cached_productions_mrow := Void
			cached_mask_sets_mrow := Void
		end

	uncache_mask_techn
			-- Clear cached rows due to a change of mask techn
		do
			cached_mask_techns_row := Void
			cached_mt_organizations_row := Void
			cached_wafer_mask_techns_row := Void
			cached_job_deck_slice_codes_row := Void
			cached_mask_techn_prolog_lines := Void
			cached_mask_techn_epilog_lines := Void
			uncache_run_gds_files
			uncache_run_gds_file_layers
			uncache_run_gds_merge_results
		end

	uncache_wafer_techn
			-- Clear cached rows due to a change of wafer techn
		do
			cached_wafer_techns_row := Void
			cached_wafer_techn_families_row := Void
			cached_wafer_techn_run_types_row := Void
			cached_wt_organizations_row := Void
			cached_wafer_mask_techns_row := Void
			cached_wafer_techn_family_level_rows := Void
			cached_wafer_techn_level_rows := Void
			cached_wafer_process_rows := Void
			cached_run_level_rows := Void
			cached_xform_levels_table := Void
			cached_mask_title_orientation_rows := Void
			cached_mask_title_rows := Void
			cached_fill_patterns_row := Void
			clear_dies_data	-- (dies know xform_levels)
			if cached_runs_mrow /= Void then
				cached_runs_mrow.uncache_wafer_techn
			end
			uncache_run_gds_files
			uncache_run_gds_file_layers
			uncache_run_gds_merge_results
		end

	uncache_field_set
			-- Clear cached rows due to a change of field set
		do
			cached_field_sets_row := Void
			cached_litho_sizes_row := Void
			cached_wafer_types_row := Void
			cached_mask_sizes_row := Void
			cached_virtual_mask_sizes_row := Void
			cached_job_deck_slice_codes_row := Void
			cached_field_name_rows := Void
			cached_field_location_rows := Void
			cached_field_locations_table := Void
			cached_mask_techn_prolog_lines := Void
			cached_mask_techn_epilog_lines := Void
		end

	uncache_wafer_lots
			-- Clear cached rows for wafer lots
		do
			cached_wafer_lot_mrows := Void
		end

	uncache_packaging_lots
			-- Clear cached rows for packaging lots
		do
			cached_packaging_lot_mrows := Void
		end

	uncache_cut_patterns
			-- Clear cached rows for cut patterns
		do
			cached_cut_pattern_mrows := Void
			cached_cut_location_mrows := Void
			cached_cut_die_site_mrows := Void
		end

	uncache_miscellaneous
			-- Clear miscellaneous cached rows due to an
			-- uncache request
		do
			cached_packaging_vendor_org_rows := Void
			cached_mask_litho_types_table := Void
			cached_field_step_locations_table := Void
			cached_organizations_org_table := Void
			cached_organizations_short_id_table := Void
			cached_design_technologies_table := Void
			clear_dies_data	-- (dies know design_technologies)
		end

	cache_dies (list: LIST [DIES_MXROW])
			-- Set `die_mrows' to `list'
		require
			list_exists: list /= Void
		do
			cached_die_mrows := list
		ensure
			cached: die_mrows = list
		end

	uncache_dies
			-- Clear cached rows for dies
		do
			cached_die_mrows := Void
			cached_die_attribute_rows := Void
		end

	clear_dies_data
			-- Clear out the cached data in the dies
		do
			if cached_die_mrows /= Void then
			   -- (don't fetch the dies just to clear them!!)
				cached_die_mrows.do_all
					(agent {DIES_MXROW}.clear_xrow_data)
			end
		end

	uncache_die_locations
			-- Clear cached rows for die locations
		do
			cached_die_location_mrows := Void
			cached_die_location_erows := Void
			cached_die_locations_table := Void
			cached_extended_die_locations_table := Void
		end

	cache_run_wafer_level_merge_jobs (list: LIST [RUN_WAFER_LEVEL_JOBS_MROW])
			-- Set `run_wafer_level_jobs_merge_mrows' to `list'
		require
			list_exists: list /= Void
		do
			cached_run_wafer_level_job_merge_mrows := list
		ensure
			cached: cached_run_wafer_level_job_merge_mrows = list
		end

	cache_run_wafer_level_fill_jobs (list: LIST [RUN_WAFER_LEVEL_JOBS_MROW])
			-- Set `run_wafer_level_jobs_fill_mrows' to `list'
		require
			list_exists: list /= Void
		do
			cached_run_wafer_level_job_fill_mrows := list
		ensure
			cached: cached_run_wafer_level_job_fill_mrows = list
		end

	uncache_run_wafer_level_jobs
			-- Clear cached rows for run wafer level jobs
			-- for both merge and fill
		do
			cached_run_wafer_level_job_merge_mrows := Void
			cached_run_wafer_level_job_fill_mrows := Void
		end

	cache_run_fill_selector_jobs (list: LIST [RUN_JOBS_MROW])
			-- Set `run_fill_selector_jobs_mrows' to `list'
		require
			list_exists: list /= Void
		do
			cached_run_fill_selector_job_mrows := list
		ensure
			cached: cached_run_fill_selector_job_mrows = list
		end

	uncache_run_fill_selector_jobs
			-- Clear cached rows for run fill selector jobs
		do
			cached_run_fill_selector_job_mrows := Void
		end

	cache_run_gds_files (list: LIST [RUN_GDS_FILES_MROW])
			-- Set `run_gds_file_mrows' to `list'
		require
			list_exists: list /= Void
		do
			cached_run_gds_file_mrows := list
		ensure
			cached: cached_run_gds_file_mrows = list
		end

	cache_run_gds_file_layers (list: LIST [RUN_GDS_FILE_LAYERS_MROW])
			-- Set `run_gds_file_layer_mrows' to `list'
		require
			list_exists: list /= Void
		do
			cached_run_gds_file_layer_mrows := list
		ensure
			cached: cached_run_gds_file_layer_mrows = list
		end

	cache_run_gds_merge_results (list: LIST [RUN_GDS_MERGE_RESULTS_MROW])
			-- Set `run_gds_merge_result_mrows' to `list'
		require
			list_exists: list /= Void
		do
			cached_run_gds_merge_result_mrows := list
		ensure
			cached: cached_run_gds_merge_result_mrows = list
		end

	uncache_run_gds_files
			-- Clear cached rows for run merged GDS files
		do
			cached_run_gds_file_mrows := Void
		end

	uncache_run_gds_file_layers
			-- Clear cached rows for layers for run merged GDS files
		do
			cached_run_gds_file_layer_mrows := Void
		end

	uncache_run_gds_merge_results
			-- Clear cached rows for run GDS merge results
		do
			cached_run_gds_merge_result_mrows := Void
		end

feature -- Operations

	check_one_scale_factor
			-- Report unsupported condition if there is more
			-- than one scale factor among levels for run
		require
			wafer_techn_exists: wafer_techn /= Void
			wafer_techn_nonempty: not wafer_techn.is_empty
		do
			if not same_scale_factor (wafer_techn_level_rows) then
				handle_unsupported ("not all levels for wafer_techn " + wafer_techn + " have the same scale factor")
			end
		end

	check_origin_payload_center
			-- Report unsupported condition if the payload
			-- center is not the origin (0, 0)
		require
			field_set_name_valid: is_nonempty_string (field_set_name)
		local
			center: CONST_POINT
		do
			center := payload_center
			if center.x /= 0.0 or center.y /= 0.0 then
				handle_unsupported ("center of payload relative to center of field is not (0, 0)")
			end
		end

	check_one_field
			-- Report unsupported condition if there is more
			-- than one field for run
		require
			field_set_name_valid: is_nonempty_string (field_set_name)
		do
			if field_name_rows.count > 1 then
				handle_unsupported ("more than one field for run")
			end
		end

	check_valid_ebeam_type
			-- Report unsupported condition if ebeam type is
			-- VARIAN
		require
			mask_techn_valid: is_nonempty_string (mask_techn)
		local
			ebeam_type: EBEAM_TYPE_VALUES
		do
			if mask_techns_row.ebeam_type = ebeam_type.Varian then
				handle_unsupported ("mask techn has ebeam type " + ebeam_type.Varian)
			end
		end

	check_valid_litho_type
			-- Report unsupported condition if litho type is
			-- FULL_WAFER
		require
			wafer_techn_valid: is_nonempty_string (wafer_techn)
		local
			type: LITHO_TYPE_VALUES
		do
			if litho_type = type.Full_wafer then
				handle_unsupported ("wafer techn has litho type " + type.Full_wafer)
			end
		end

	check_fill_pattern_sizings
			-- Report error if the remove or merge sizings
			-- for the fill pattern are odd multiple of
			-- `level_lcm_resolution'
		require
			wafer_techn_valid: is_nonempty_string (wafer_techn)
			processes_assigned: -- Processes assigned to projects
		local
			res, factor: DOUBLE
			fp_row: FILL_PATTERNS_ROW
		do
			res := level_lcm_resolution
			factor := scale_factor
				-- Assuming one scale factor for all levels
			fp_row := fill_patterns_row
			check_fill_pattern_sizing (fp_row.cell_remove_sizing, res, factor, "cell remove")
			check_fill_pattern_sizing (fp_row.cell_merge_sizing, res, factor, "cell merge")
		end

	check_files_merged
			-- Report unsupported condition if run does not
			-- generate merged MEBES files
		require
			mask_techn_valid: is_nonempty_string (mask_techn)
			wafer_techn_valid: is_nonempty_string (wafer_techn)
		do
			if not run_needs_merge then
				handle_unsupported ("no merged pattern files generated for run")
			end
		end

feature {NONE} -- Implementation

	mask_locations (die_id: STRING loc_rows: LIST [DIE_LOCATIONS_ROW]): LIST [CONST_POINT]
			-- List of locations on mask for `die_id' and
			-- `loc_rows', sorted by Y and by X within Y
		require
			die_id_valid: is_nonempty_string (die_id)
			loc_rows_exists: loc_rows /= Void
			wafer_techn_valid: is_nonempty_string (wafer_techn)
			field_set_name_valid: is_nonempty_string (field_set_name)
			projects_packed: -- Projects have been packed
		local
			factor: DOUBLE
			loc_row: DIE_LOCATIONS_ROW
			die_location: CONST_POINT
			field_locs: like field_location_rows_for_field
			sorter: LIST_BUBBLE_SORTER [CONST_POINT]
		do
			factor := scale_factor
				-- Assuming one scale factor for all levels
			create {ARRAYED_LIST [CONST_POINT]} Result.make (loc_rows.count * 2)
				-- Size is rough guess
			from
				loc_rows.start
			until
				loc_rows.after
			loop
				loc_row := loc_rows.item
				die_location := loc_row.center * factor
				field_locs := field_location_rows_for_field (loc_row.field_name)
				from
					field_locs.start
				until
					field_locs.after
				loop
					Result.extend (field_locs.item.center + die_location)
					field_locs.forth
				end
				loc_rows.forth
			end
			create sorter
			Result := sorter.sorted (Result, create {POINT_YX_COMPARATOR})
		ensure
			result_exists: Result /= Void
		end

	db_productions_mrow: PRODUCTIONS_MROW
			-- Modifiable row from PRODUCTIONS table, fetched from
			-- database
		local
			row: like cached_productions_mrow
			prod: PRODUCTION_TYPE_VALUES
		do
			create row.make_template
			Result := row.productions_for_run_and_type (run_id, prod.Initial_run, connection, expected_count.exactly_one).first
		ensure
			result_exists: Result /= Void
		end

	db_mask_sets_mrow: MASK_SETS_MROW
			-- Modifiable row from MASK_SETS table, fetched from
			-- database
		local
			row: like cached_mask_sets_mrow
		do
			create row.make_template
			Result := row.mask_sets_for_run_and_revision (run_id, rc.Initial_mask_revision, connection, expected_count.exactly_one).first
		ensure
			result_exists: Result /= Void
		end

	db_wafer_techns_row: WAFER_TECHNS_ROW
			-- Read-only row for wafer techn for run, fetched
			-- from database
		require
			wafer_techn_valid: is_nonempty_string (wafer_techn)
		local
			row: like cached_wafer_techns_row
		do
			create row.make_template
			Result := row.wafer_techns_with_name (wafer_techn, connection, expected_count.exactly_one).first
		ensure
			result_exists: Result /= Void
		end

	db_wafer_techn_families_row: WAFER_TECHN_FAMILIES_ROW
			-- Read-only row for wafer techn family for run,
			-- fetched from database
		require
			wafer_techn_valid: is_nonempty_string (wafer_techn)
		local
			row: like cached_wafer_techn_families_row
		do
			create row.make_template
			Result := row.wafer_techn_families_for_wt (wafer_techn, connection, expected_count.exactly_one).first
		ensure
			result_exists: Result /= Void
		end

	db_wafer_techn_run_types_row: WAFER_TECHNXRUN_TYPES_ROW
			-- Read-only row for wafer techn and run type 
			-- for run, fetched from database
		require
			wafer_techn_valid: is_nonempty_string (wafer_techn)
			run_type_valid: is_nonempty_string (run_type)
		local
			row: like cached_wafer_techn_run_types_row
		do
			create row.make_template
			Result := row.rows_for_wt_and_run_type (wafer_techn, run_type, connection, expected_count.exactly_one).first
		ensure
			result_exists: Result /= Void
		end

	db_wt_organizations_row: ORGANIZATIONS_ROW
			-- Read-only row for organization for wafer techn
			-- for run, fetched from database
		require
			wafer_techn_valid: is_nonempty_string (wafer_techn)
		local
			row: like cached_wt_organizations_row
		do
			create row.make_template
			Result := row.rows_with_org_no (wafer_techns_row.org_no, connection, expected_count.exactly_one).first
		ensure
			result_exists: Result /= Void
		end

	db_mask_techns_row: MASK_TECHNS_ROW
			-- Read-only row for mask techn for run, fetched
			-- from database
		require
			mask_techn_valid: is_nonempty_string (mask_techn)
		local
			row: like cached_mask_techns_row
		do
			create row.make_template
			Result := row.mask_techns_with_name (mask_techn, connection, expected_count.exactly_one).first
		ensure
			result_exists: Result /= Void
		end

	db_mt_organizations_row: ORGANIZATIONS_ROW
			-- Read-only row for organization for mask techn
			-- for run, fetched from database
		require
			mask_techn_valid: is_nonempty_string (mask_techn)
		local
			row: like cached_mt_organizations_row
		do
			create row.make_template
			Result := row.rows_with_org_no (mask_techns_row.org_no, connection, expected_count.exactly_one).first
		ensure
			result_exists: Result /= Void
		end

	db_wafer_mask_techns_row: WAFER_TECHNXMASK_TECHNS_ROW
			-- Read-only row for wafer/mask techn combination
			-- for run, fetched from database
		require
			mask_techn_valid: is_nonempty_string (mask_techn)
			wafer_techn_valid: is_nonempty_string (wafer_techn)
		local
			row: like cached_wafer_mask_techns_row
		do
			create row.make_template
			Result := row.wafer_mask_techns_with_name (wafer_techn, mask_techn, connection, expected_count.exactly_one).first
		ensure
			result_exists: Result /= Void
		end

	db_field_sets_row: FIELD_SETS_ROW
			-- Read-only row for field set for run,
			-- fetched from database
		require
			field_set_name_valid: is_nonempty_string (field_set_name)
		local
			row: like cached_field_sets_row
		do
			create row.make_template
			Result := row.field_sets_with_name (field_set_name, connection, expected_count.exactly_one).first
		ensure
			result_exists: Result /= Void
		end

	db_litho_sizes_row: LITHO_SIZES_ROW
			-- Read-only row for litho size for run,
			-- fetched from database
		require
			field_set_name_valid: is_nonempty_string (field_set_name)
		local
			row: like cached_litho_sizes_row
		do
			create row.make_template
			Result := row.litho_sizes_with_name (litho_size, connection, expected_count.exactly_one).first
		ensure
			result_exists: Result /= Void
		end

	db_wafer_types_row: WAFER_TYPES_ROW
			-- Read-only row for wafer type for run,
			-- fetched from database
		require
			field_set_name_valid: is_nonempty_string (field_set_name)
		local
			row: like cached_wafer_types_row
		do
			create row.make_template
			Result := row.wafer_types_with_name (wafer_type_id, connection, expected_count.exactly_one).first
		ensure
			result_exists: Result /= Void
		end

	db_mask_sizes_row: MASK_SIZES_ROW
			-- Read-only row for mask size for run,
			-- fetched from database
		require
			field_set_name_valid: is_nonempty_string (field_set_name)
		local
			row: like cached_mask_sizes_row
		do
			create row.make_template
			Result := row.mask_sizes_with_name (mask_size_id, connection, expected_count.exactly_one).first
		ensure
			result_exists: Result /= Void
		end

	db_virtual_mask_sizes_row: MASK_SIZES_ROW
			-- Read-only row for virtual mask size for run
			-- whose litho size does not have a mask size
			-- ID, fetched from database
		require
			field_set_name_valid: is_nonempty_string (field_set_name)
			no_mask_size_id: not has_mask_size_id
		local
			row: like cached_virtual_mask_sizes_row
		do
			create row.make_template
			Result := row.mask_sizes_with_name (virtual_mask_size_id, connection, expected_count.exactly_one).first
		ensure
			result_exists: Result /= Void
		end

	db_job_deck_slice_codes_row: JOB_DECK_SLICE_CODES_ROW
			-- Read-only row for job deck slice code for run,
			-- fetched from database
		require
			field_set_name_valid: is_nonempty_string (field_set_name)
			mask_techn_valid: is_nonempty_string (mask_techn)
		local
			row: like cached_job_deck_slice_codes_row
		do
			create row.make_template
			Result := row.slice_code_for_size_and_type (mask_size_id, mask_techns_row.ebeam_type, connection, expected_count.exactly_one).first
		ensure
			result_exists: Result /= Void
		end

	db_packaging_vendor_org_rows: LIST [ORGANIZATIONS_ROW]
			-- List of read-only rows from ORGANIZATIONS table for
			-- organization that are packaging vendors, bump
			-- vendors or test vendors, fetched from database
		local
			row: ORGANIZATIONS_ROW
		do
			create row.make_template
			Result := row.packaging_vendor_rows (connection, expected_count.At_least_one)
		ensure
			result_exists: Result /= Void
		end

	db_mask_title_orientation_rows: LIST [MASK_TITLE_ORIENTATIONS_ROW]
			-- List of read-only rows from MASK_TITLE_ORIENTATIONS
			-- table for wafer techn family for run,
			-- fetched from database
		require
			wafer_techn_family_valid: is_nonempty_string (wafer_techn_family)
		local
			row: MASK_TITLE_ORIENTATIONS_ROW
		do
			create row.make_template
			Result := row.title_orientations_for_wt_family (wafer_techn_family, connection, expected_count.At_least_zero)
		ensure
			result_exists: Result /= Void
		end

	db_mask_title_rows: LIST [MASK_TITLES_ROW]
			-- List of read-only rows from MASK_TITLE_ORIENTATIONS
			-- table for wafer techn family for run,
			-- fetched from database
		require
			wafer_techn_family_valid: is_nonempty_string (wafer_techn_family)
			processes_assigned: -- Processes assigned to projects
		local
			row: MASK_TITLES_ROW
		do
			create row.make_template
			Result := row.titles_for_wt_family (wafer_techn_family, run_id, connection, expected_count.At_least_zero)
		ensure
			result_exists: Result /= Void
		end

	db_mask_techn_prolog_lines: LIST [STRING]
			-- List of prolog lines for mask techn and field set,
			-- fetched from database
		require
			field_set_name_valid: is_nonempty_string (field_set_name)
			mask_techn_valid: is_nonempty_string (mask_techn)
		local
			row: MASK_TECHN_CANNED_LINES_ROW
		do
			create row.make_template
			Result := row.prolog_lines (mask_techn, mask_size_id, connection, expected_count.At_least_zero)
		ensure
			result_exists: Result /= Void
		end

	db_mask_techn_epilog_lines: LIST [STRING]
			-- List of epilog lines for mask techn and field set,
			-- fetched from database
		require
			field_set_name_valid: is_nonempty_string (field_set_name)
			mask_techn_valid: is_nonempty_string (mask_techn)
		local
			row: MASK_TECHN_CANNED_LINES_ROW
		do
			create row.make_template
			Result := row.epilog_lines (mask_techn, mask_size_id, connection, expected_count.At_least_zero)
		ensure
			result_exists: Result /= Void
		end

	db_fill_patterns_row: FILL_PATTERNS_ROW
			-- Read-only row for fill pattern for run,
			-- fetched from database
		require
			wafer_techn_valid: is_nonempty_string (wafer_techn)
		local
			row: like cached_fill_patterns_row
		do
			create row.make_template
			Result := row.fill_patterns_with_name (wafer_techns_row.fill_pattern_id, connection, expected_count.exactly_one).first
		ensure
			result_exists: Result /= Void
		end

	db_field_name_rows: LIST [FIELD_NAMES_ROW]
			-- List of read-only rows from FIELD_NAMES table
			-- for field set for run, fetched from database
		require
			field_set_name_valid: is_nonempty_string (field_set_name)
		local
			row: FIELD_NAMES_ROW
		do
			create row.make_template
			Result := row.field_names_for_set (field_set_name, connection, expected_count.At_least_one)
		ensure
			result_exists: Result /= Void
		end

	db_field_location_rows: LIST [FIELD_LOCATIONS_ROW]
			-- List of read-only rows from FIELD_LOCATIONS table
			-- for field set for run, fetched from database
		require
			field_set_name_valid: is_nonempty_string (field_set_name)
		local
			row: FIELD_LOCATIONS_ROW
		do
			create row.make_template
			Result := row.locations_for_field_set (field_set_name, connection, expected_count.At_least_one)
		ensure
			result_exists: Result /= Void
		end

	db_wafer_techn_family_level_rows: LIST [WAFER_TECHN_FAMILY_LEVELS_ROW]
			-- List of read-only rows for wafer techn family levels
			-- for run's wafer techn ordered by level number,
			-- fetched from database
		require
			wafer_techn_valid: is_nonempty_string (wafer_techn)
		local
			row: WAFER_TECHN_FAMILY_LEVELS_ROW
			b: DB_BIND_S
		do
			create row.make_template
			create b.make (wafer_techn_family)
			Result := row.all_rows_filtered_ordered
					(connection, "wafer_techn_family = :1", b, "level_no")
		ensure
			result_exists: Result /= Void
		end

	db_wafer_techn_level_rows: LIST [WAFER_TECHN_LEVELS_ROW]
			-- List of read-only rows for wafer techn levels
			-- for run's wafer techn ordered by level number,
			-- fetched from database
		require
			wafer_techn_valid: is_nonempty_string (wafer_techn)
		local
			row: WAFER_TECHN_LEVELS_ROW
		do
			create row.make_template
			Result := row.levels_for_wafer_techn (wafer_techn, connection, expected_count.At_least_one)
		ensure
			result_exists: Result /= Void
		end

	db_wafer_process_rows: LIST [WAFER_PROCESSES_ROW]
			-- List of read-only rows for wafer processes
			-- for run's wafer techn family,
			-- fetched from database
		require
			wafer_techn_valid: is_nonempty_string (wafer_techn)
		local
			row: WAFER_PROCESSES_ROW
		do
			create row.make_template
			Result := row.processes_for_techn_family (wafer_techn_family, connection, expected_count.At_least_one)
		ensure
			result_exists: Result /= Void
		end

	db_run_level_rows: LIST [RUN_LEVELS_ROW]
			-- List of read-only rows for wafer techn levels
			-- for run ordered by level number, fetched
			-- from database
		require
			wafer_techn_valid: is_nonempty_string (wafer_techn)
			processes_assigned: -- Processes assigned to projects
		local
			row: RUN_LEVELS_ROW
		do
			create row.make_template
			Result := row.levels_for_run (run_id, connection, expected_count.At_least_one)
		ensure
			result_exists: Result /= Void
		end

	db_die_mrows: LIST [DIES_MXROW]
			-- List of modifiable rows for dies for run,
			-- fetched from database
		local
			row: DIES_MXROW
		do
			create row.make_template
			Result := row.dies_for_run (run_id, connection, expected_count.At_least_zero)
		ensure
			result_exists: Result /= Void
		end

	db_die_attribute_rows: LIST [DIE_ATTRIBUTES_ROW]
			-- List of modifiable rows for die attributes for run,
			-- fetched from database
		local
			row: DIE_ATTRIBUTES_ROW
		do
			create row.make_template
			Result := row.dies_for_run (run_id, connection, expected_count.At_least_zero)
		ensure
			result_exists: Result /= Void
		end

	db_die_location_mrows: LIST [DIE_LOCATIONS_MROW]
			-- List of modifiable rows for die locations for run,
			-- fetched from database
		local
			row: DIE_LOCATIONS_MROW
		do
			create row.make_template
			Result := row.die_locations_for_run (run_id, connection, expected_count.At_least_zero)
		ensure
			result_exists: Result /= Void
		end

	db_die_location_erows: LIST [DIE_LOCATIONS_EROW]
			-- List of read-only rows for extended die locations
			-- for run (include extra attributes),
			-- fetched from database
		local
			row: DIE_LOCATIONS_EROW
		do
			create row.make_template
			Result := row.die_locations_for_run (run_id, connection, expected_count.At_least_zero)
		ensure
			result_exists: Result /= Void
		end

	db_wafer_lot_mrows: LIST [RUN_WAFER_LOTS_MROW]
			-- List of modifiable rows for wafer lots for run,
			-- fetched from database
		local
			row: RUN_WAFER_LOTS_MROW
		do
			create row.make_template
			Result := row.lots_for_run_and_prod (run_id, production_id, connection, expected_count.At_least_zero)
		ensure
			result_exists: Result /= Void
		end

	db_run_wafer_level_job_merge_mrows: LIST [RUN_WAFER_LEVEL_JOBS_MROW]
			-- List of modifiable rows for wafer level jobs
			-- for run for merge, fetched from database
		local
			row: RUN_WAFER_LEVEL_JOBS_MROW
			job_type: CATS_JOB_TYPE_VALUES
		do
			create row.make_template
			Result := row.level_jobs_for_run_and_type (run_id, job_type.Merge, connection, expected_count.At_least_zero)
		ensure
			result_exists: Result /= Void
		end

	db_run_wafer_level_job_fill_mrows: LIST [RUN_WAFER_LEVEL_JOBS_MROW]
			-- List of modifiable rows for wafer level jobs
			-- for run for fill, fetched from database
		local
			row: RUN_WAFER_LEVEL_JOBS_MROW
			job_type: CATS_JOB_TYPE_VALUES
		do
			create row.make_template
			Result := row.level_jobs_for_run_and_type (run_id, job_type.Fill, connection, expected_count.At_least_zero)
		ensure
			result_exists: Result /= Void
		end

	db_run_fill_selector_job_mrows: LIST [RUN_JOBS_MROW]
			-- List of modifiable rows for fill selector jobs
			-- for run, fetched from database
		local
			row: RUN_JOBS_MROW
		do
			create row.make_template
			Result := row.fill_selector_jobs_for_run (run_id, connection, expected_count.At_most_one)
		ensure
			result_exists: Result /= Void
		end

	db_run_gds_file_mrows: LIST [RUN_GDS_FILES_MROW]
			-- List of modifiable rows for GDS merge
			-- for run, fetched from database
		local
			row: RUN_GDS_FILES_MROW
		do
			create row.make_template
			Result := row.rows_for_run (run_id, connection, expected_count.At_least_zero)
		ensure
			result_exists: Result /= Void
		end

	db_run_gds_file_layer_mrows: LIST [RUN_GDS_FILE_LAYERS_MROW]
			-- List of modifiable rows for layers for GDS merge
			-- for run, fetched from database
		local
			row: RUN_GDS_FILE_LAYERS_MROW
		do
			create row.make_template
			Result := row.rows_for_run (run_id, connection, expected_count.At_least_zero)
		ensure
			result_exists: Result /= Void
		end

	db_run_gds_merge_result_mrows: LIST [RUN_GDS_MERGE_RESULTS_MROW]
			-- List of modifiable rows for GDS merge
			-- results for run, fetched from database
		local
			row: RUN_GDS_MERGE_RESULTS_MROW
		do
			create row.make_template
			Result := row.rows_for_run (run_id, connection, expected_count.At_least_zero)
		ensure
			result_exists: Result /= Void
		end

	field_locations_table: like cached_field_locations_table
			-- Lists of field location rows for run and field name,
			-- indexed by field name
		require
			field_set_name_valid: is_nonempty_string (field_set_name)
		local
			rows, list: like field_location_rows
			table: like field_locations_table
			fname: STRING
		do
			if cached_field_locations_table = Void then
				rows := field_location_rows
				create table.make (field_count)
				table.compare_objects
				from
					rows.start
				until
					rows.after
				loop
					fname := rows.item.field_name
					table.search (fname)
					if table.found then
						table.found_item.extend (rows.item)
					else
						list := rows.twin
						list.wipe_out
						list.extend (rows.item)
						table.put (list, fname)
					end
					rows.forth
				end
				cached_field_locations_table := table
			end
			Result := cached_field_locations_table
		ensure
			result_exists: Result /= Void
		end

	field_step_locations_table: like cached_field_step_locations_table
			-- Lists of field step location rows for run and field
			-- step pattern name, indexed by step pattern name
		do
			if cached_field_step_locations_table = Void then
				create cached_field_step_locations_table.make (10)
				cached_field_step_locations_table.compare_objects
			end
			Result := cached_field_step_locations_table
		ensure
			result_exists: Result /= Void
		end

	die_locations_table: like cached_die_locations_table
			-- Lists of die location rows for run,
			-- indexed by die id
		local
			rows, list: like die_location_rows
			table: like die_locations_table
			location: DIE_LOCATIONS_ROW
			die_id: STRING
		do
			if cached_die_locations_table = Void then
				rows := die_location_rows
				create table.make (die_rows.count)
				table.compare_objects
				from
					rows.start
				until
					rows.after
				loop
					location := rows.item
					die_id := location.die_id
					table.search (die_id)
					if table.found then
						table.found_item.extend (location)
					else
						list := rows.twin
						list.wipe_out
						list.extend (location)
						table.put (list, die_id)
					end
					rows.forth
				end
				cached_die_locations_table := table
			end
			Result := cached_die_locations_table
		ensure
			result_exists: Result /= Void
		end

	mask_litho_types_table: like cached_mask_litho_types_table
			-- Lists of all known mask litho types,
			-- indexed by name
		local
			mtl_row: MASK_LITHO_TYPES_ROW
			rows: LIST [MASK_LITHO_TYPES_ROW]
			table: like mask_litho_types_table
			name: STRING
		do
			if cached_mask_litho_types_table = Void then
				create mtl_row.make_template
				rows := mtl_row.all_mask_litho_types (connection, expected_count.At_least_one)
				create table.make (rows.count)
				table.compare_objects
				from
					rows.start
				until
					rows.after
				loop
					name := rows.item.mask_litho_type
					table.put (rows.item, name)
					rows.forth
				end
				cached_mask_litho_types_table := table
			end
			Result := cached_mask_litho_types_table
		ensure
			result_exists: Result /= Void
		end

	design_technologies_table: like cached_design_technologies_table
			-- Lists of design_technologies by
			-- design techn/lambda combination, indexed by
			-- key created from design techn and lambda
		do
			if cached_design_technologies_table = Void then
				create cached_design_technologies_table.make (10)
				cached_design_technologies_table.compare_objects
			end
			Result := cached_design_technologies_table
		ensure
			result_exists: Result /= Void
		end

	xform_levels_table: like cached_xform_levels_table
			-- Lists of xform levels for run's wafer techn and
			-- a design techn/lambda combination, indexed by
			-- key created from design techn and lambda
		require
			wafer_techn_valid: is_nonempty_string (wafer_techn)
		do
			if cached_xform_levels_table = Void then
				create cached_xform_levels_table.make (10)
				cached_xform_levels_table.compare_objects
			end
			Result := cached_xform_levels_table
		ensure
			result_exists: Result /= Void
		end

	organizations_org_table: like cached_organizations_org_table
			-- Organization rows, indexed by org_no
		do
			if cached_organizations_org_table = Void then
				create cached_organizations_org_table.make (10)
				cached_organizations_org_table.compare_objects
			end
			Result := cached_organizations_org_table
		ensure
			result_exists: Result /= Void
		end

	organizations_short_id_table: like cached_organizations_short_id_table
			-- Organization rows, indexed by short id
		do
			if cached_organizations_short_id_table = Void then
				create cached_organizations_short_id_table.make (10)
				cached_organizations_short_id_table.compare_objects
			end
			Result := cached_organizations_short_id_table
		ensure
			result_exists: Result /= Void
		end

	add_title_orientation (list: LIST [STRING] row: MASK_TITLE_ORIENTATIONS_ROW)
			-- Add text from `row' to `list'
		require
			list_exists: list /= Void
			row_exists: row /= Void
		do
			list.extend (row.text)
		end

	same_scale_factor (levels: LIST [WAFER_TECHN_LEVELS_ROW]): BOOLEAN
			-- Do all elements of `levels' have the same scale
			-- factor?
		require
			levels_exists: levels /= Void
		local
			factor: DOUBLE
		do
			Result := True
			if not levels.is_empty then
				factor := levels.first.scale_factor
			end
			from
				levels.start
			until
				levels.after or not Result
			loop
				if levels.item.scale_factor /= factor then
					Result := False
				end
				levels.forth
			end
		end

	is_invalid_list (list: LIST [DB_IDENTIFIED_ROW]): BOOLEAN
			-- Is list nonexistent or are any rows in
			-- `list' invalid (nonexistent, corrupted or
			-- deleted)?
		local
			c: CURSOR
		do
			if list /= Void then
				c := list.cursor
				from
					list.start
				until
					list.after or Result
				loop
					Result := Result or is_invalid_row (list.item)
					list.forth
				end
				list.go_to (c)
			else
				Result := True
			end
		end
			
	is_invalid_row (row: DB_IDENTIFIED_ROW): BOOLEAN
			-- Is `row' invalid (nonexistent, corrupted or
			-- deleted)?
		do
			Result := row = Void or else (row.is_corrupted or row.is_deleted)
		end
			
	dt_lambda_key (dt: STRING lambda: STRING): STRING
			-- Key corresponding to design techn `dt' and `lambda'
		do
			Result := dt + "%U" + lambda
		end
			
	check_fill_pattern_sizing (sizing, res, factor: DOUBLE name: STRING)
			-- Report error if 1X `sizing'
			-- is an odd multiple of resolution `res' divided
			-- by `factor' ,
			-- where `name' is the name of the sizing value
		require
			name_exists: name /= Void
			resolution_positive: res > 0.0
			factor_positive: factor > 0.0
		local
			quotient: INTEGER
			odd: BOOLEAN
		do
			quotient := (sizing.abs / (res / factor)).rounded
                	odd := (quotient \\ 2) = 1
			if odd then
				handle_hard_error ("fill pattern " + name + " sizing value " + nu.std_formatted_number (sizing) + " is an odd multiple of LCM of level resolutions (" + nu.std_formatted_number (res) + ")%Ndivided by scale factor (" + nu.std_formatted_number (factor)+ ")")
			end
		end

	cached_runs_mrow: RUNS_MROW
			-- Runs row for `Current' cached in memory

	cached_productions_mrow: PRODUCTIONS_MROW
			-- Productions row for `Current' cached in memory

	cached_mask_sets_mrow: MASK_SETS_MROW
			-- Productions row for `Current' cached in memory

	cached_wafer_techns_row: WAFER_TECHNS_ROW
			-- Wafer techns row for `Current' cached in memory

	cached_wafer_techn_families_row: WAFER_TECHN_FAMILIES_ROW
			-- Wafer techn families row for `Current' cached
			-- in memory

	cached_wafer_techn_run_types_row: WAFER_TECHNXRUN_TYPES_ROW
			-- Wafer techn run types row for `Current' cached
			-- in memory

	cached_wt_organizations_row: ORGANIZATIONS_ROW
			-- Organizations row for wafer techn for `Current'
			-- cached in memory

	cached_mask_techns_row: MASK_TECHNS_ROW
			-- Mask techns row for `Current' cached in memory

	cached_mt_organizations_row: ORGANIZATIONS_ROW
			-- Organizations row for mask techn for `Current'
			-- cached in memory

	cached_wafer_mask_techns_row: WAFER_TECHNXMASK_TECHNS_ROW
			-- Wafer mask techns row for `Current' cached in memory

	cached_field_sets_row: FIELD_SETS_ROW
			-- Field sets row for `Current' cached in memory

	cached_litho_sizes_row: LITHO_SIZES_ROW
			-- Litho sizes row for `Current' cached in memory

	cached_mask_sizes_row: MASK_SIZES_ROW
			-- Mask sizes row for `Current' cached in memory

	cached_virtual_mask_sizes_row: MASK_SIZES_ROW
			-- Virtual mask sizes row for `Current' (if it
			-- is for run whose litho size has no mask
			-- size ID), cached in memory

	cached_wafer_types_row: WAFER_TYPES_ROW
			-- Wafer types row for `Current' cached in memory

	cached_job_deck_slice_codes_row: JOB_DECK_SLICE_CODES_ROW
			-- Job deck slice codes row for `Current' cached
			-- in memory

	cached_packaging_vendor_org_rows: LIST [ORGANIZATIONS_ROW]
			-- Rows for organizations that are packaging
			-- vendors, bump vendors or test vendors,
			-- cached in memory

	cached_mask_title_orientation_rows: LIST [MASK_TITLE_ORIENTATIONS_ROW]
			-- Mask title orientation rows for `Current' cached
			-- in memory

	cached_mask_title_rows: LIST [MASK_TITLES_ROW]
			-- Mask title rows for `Current' cached in memory

	cached_mask_techn_prolog_lines: LIST [STRING]
			-- Prolog canned lines for mask techn and field set
			-- for `Current' cached in memory

	cached_mask_techn_epilog_lines: LIST [STRING]
			-- Epilog canned lines for mask techn and field set
			-- for `Current' cached in memory

	cached_fill_patterns_row: FILL_PATTERNS_ROW
			-- Fill patterns row for `Current' cached in memory

	cached_field_name_rows: LIST [FIELD_NAMES_ROW]
			-- Field name rows for `Current' cached in memory

	cached_field_location_rows: LIST [FIELD_LOCATIONS_ROW]
			-- Field location rows for `Current' cached in memory

	cached_wafer_techn_family_level_rows: LIST [WAFER_TECHN_FAMILY_LEVELS_ROW]
			-- Wafer techn family level rows for wafer techn of `Current'
			-- cached in memory

	cached_wafer_techn_level_rows: LIST [WAFER_TECHN_LEVELS_ROW]
			-- Wafer techn level rows for wafer techn of `Current'
			-- cached in memory

	cached_wafer_process_rows: LIST [WAFER_PROCESSES_ROW]
			-- Wafer process rows for wafer techn family of
			-- `Current' cached in memory

	cached_run_level_rows: LIST [RUN_LEVELS_ROW]
			-- Wafer techn level rows for `Current' that are
			-- included on this run (based on processes assigned),
			-- cached in memory

	cached_die_mrows: LIST [DIES_MXROW]
			-- Modifiable die rows for `Current' cached in memory

	cached_die_attribute_rows: LIST [DIE_ATTRIBUTES_ROW]
			-- Read-only die attribute rows for `Current'
			-- cached in memory

	cached_die_location_mrows: LIST [DIE_LOCATIONS_MROW]
			-- Modifiable die location rows for `Current'
			-- cached in memory

	cached_die_location_erows: LIST [DIE_LOCATIONS_EROW]
			-- Read-only extended die location rows for `Current'
			-- cached in memory

	cached_wafer_lot_mrows: LIST [RUN_WAFER_LOTS_MROW]
			-- Modifiable wafer lot rows for `Current' cached
			-- in memory

	cached_packaging_lot_mrows: LIST [RUN_PACKAGING_LOTS_MROW]
			-- Modifiable packaging lot rows for `Current' cached
			-- in memory

	cached_cut_pattern_mrows: LIST [RUN_CUT_PATTERNS_MROW]
			-- Modifiable cut pattern rows for `Current' cached
			-- in memory

	cached_cut_location_mrows: LIST [CUT_LOCATIONS_MROW]
			-- Modifiable cut location rows for `Current' cached
			-- in memory

	cached_cut_die_site_mrows: LIST [CUT_DIE_SITES_MROW]
			-- Modifiable cut die site rows for `Current' cached
			-- in memory

	cached_run_wafer_level_job_merge_mrows: LIST [RUN_WAFER_LEVEL_JOBS_MROW]
			-- Modifiable run wafer level job rows for merge jobs
			-- for `Current' cached in memory

	cached_run_wafer_level_job_fill_mrows: LIST [RUN_WAFER_LEVEL_JOBS_MROW]
			-- Modifiable run wafer level job rows for fill jobs
			-- for `Current' cached in memory

	cached_run_fill_selector_job_mrows: LIST [RUN_JOBS_MROW]
			-- Modifiable run fill selector job rows for
			-- `Current' cached in memory

	cached_run_gds_file_mrows: LIST [RUN_GDS_FILES_MROW]
			-- Modifiable merged GDS file rows for
			-- `Current' cached in memory

	cached_run_gds_file_layer_mrows: LIST [RUN_GDS_FILE_LAYERS_MROW]
			-- Modifiable layers for merged GDS file rows for
			-- `Current' cached in memory

	cached_run_gds_merge_result_mrows: LIST [RUN_GDS_MERGE_RESULTS_MROW]
			-- Modifiable merged GDS merge result rows for
			-- `Current' cached in memory

	cached_field_locations_table: HASH_TABLE [LIST [FIELD_LOCATIONS_ROW], STRING]
			-- Lists of field location rows for run and field name,
			-- indexed by field name and cached in memory

	cached_field_step_locations_table: HASH_TABLE [LIST [FIELD_STEP_PATTERNXFIELD_LOCS_ROW], STRING]
			-- Lists of field step location rows for run
			-- and field step pattern name, indexed by
			-- field step pattern name and cached in
			-- memory

	cached_die_locations_table: HASH_TABLE [LIST [DIE_LOCATIONS_ROW], STRING]
			-- Lists of die location rows for run and die id,
			-- indexed by die id and cached in memory

	cached_extended_die_locations_table: HASH_TABLE [LIST [DIE_LOCATIONS_EROW], STRING]
			-- Lists of extended die location rows for run and
			-- field, indexed by field name and cached in memory

	cached_design_technologies_table: HASH_TABLE [DESIGN_TECHNOLOGY, STRING]
			-- design_technologies, indexed by
			-- key created from design techn and lambda
			-- and cached in memory

	cached_xform_levels_table: HASH_TABLE [LIST [DESIGN_TECHN_XFORM_LEVELS_ROW], STRING]
			-- Lists of xform levels for run's wafer techn and
			-- a design techn/lambda combination, indexed by
			-- indexed by key created from design techn and lambda
			-- and cached in memory

	cached_mask_litho_types_table: HASH_TABLE [MASK_LITHO_TYPES_ROW, STRING]
			-- List of all known mask litho types,
			-- indexed by name and cached in memory

	cached_organizations_org_table: HASH_TABLE [ORGANIZATIONS_ROW, INTEGER]
			-- Cache of organizations, indexed org_no

	cached_organizations_short_id_table: HASH_TABLE [ORGANIZATIONS_ROW, STRING]
			-- Cache of organizations, indexed short_id

	Address_units_per_segment: INTEGER = 32768
			-- Number of address units in width of a MEBES file
			-- segment
			--
	Max_mebes3_segments: INTEGER = 50
			-- Maximum number of segments in MEBES/3 file

end
