
indexing
	
	description: "An operation that closes a run"
	author: "David Hollenberg"
	date: "March 12, 2003"

class RUN_CLOSE_OP
inherit
	OPERATION
		rename
			try_operation as try_to_close_run
		end
	RC_SESSIONABLE
		export
			{NONE} set_session
		end
	RC_ENVIRONMENT_ACCESS
		export
			{NONE} all
		undefine
			default_create
		end
	RUN_QUERIES
		export
			{NONE} all
		end
	DB_OPTIONS_ACCESS
		export
			{NONE} all
		undefine
			default_create
		end

create
	default_create

feature -- Operations

	close_run (s: STRING sess: RC_RUN_CLOSING_SESSION)
			-- Close run with run id `s', using `sess'
		require
			name_exists: s /= Void
			session_exists: sess /= Void
			session_has_connection: sess.connection /= Void
			session_connection_open: sess.connection.is_open
		do
			session := sess
			run_id := s
			execute
		end

feature {NONE} -- Implementation

	run_id: STRING
			-- Run id for which to try to close run

	try_to_close_run
			-- Try to close run
		local
			validator: OPERATOR_VALIDATION_OP
			step_reader: RC_STEP_READ_OP
			prods_mrow: PRODUCTIONS_MROW
			the_closer: STRING
			run_dir: RC_FILE_CATEGORY
		do
			if not session.is_read_only then
				create validator
				validator.validate_user (connection, connection.user_name)
			end
			session.check_valid_run_id (run_id)

			if session.is_read_only then
				session.lock_shared
			else
				session.lock
			end
			
			-- Check whether run exists

			if not is_existing_run (run_id, connection) then
				handle_user_error ("Run does not exist: " + run_id)
			end
				
			-- Check whether production is marked complete.
			-- If it is, convert session to read-only

			if not session.is_read_only then
				if cache.productions_row.is_completed then
					session.set_read_only
					inform ("Run " + run_id + " is marked COMPLETE - converting to read-only session")
				elseif cache.productions_row.not_mosis then
					session.set_run_closer_role
					prods_mrow := cache.productions_mrow
					if prods_mrow.is_null_date_production_closed then
						prods_mrow.set_date_production_closed (connection.transaction_date)
					end
					if not prods_mrow.is_completed then
						prods_mrow.set_is_completed (True)
					end
					session.commit
					session.set_read_only
					inform ("Run " + run_id + " is a non-MOSIS run - converting to read-only session")
				end
			end

			the_closer := cache.closer
			if not session.is_read_only and then validator.is_run_closer and not the_closer.is_empty and not connection.user_name.is_equal (the_closer) then
				warn ("Official run closer for " + run_id + " is " + cache.closer + "%NContinue anyway?")
			end
	
			if not session.is_read_only then
				session.open_log
			end
				
			-- Create run directory if run was
			-- initially created via Omega instead of Delta
			
			run_dir := session.file_generator.run_directory
			if not session.path_exists (run_dir.directory_pathname) then
				session.set_run_closer_role
				session.ensure_directory_exists (run_dir)
			end

			-- Read and check info about run closing steps
			
			create step_reader
			step_reader.read_steps (session)
			
			publish_start_close_event
		end

	publish_start_close_event
		local
			close_event: RUN_START_CLOSE_EVENT
		do
			create close_event
			close_event.set_run_id (run_id)
			close_event.publish
		end

end
