
indexing

	author: "Generated"
	date: "Thu Mar 11 15:52:47 PST 2004"

class RUN_CUT_PATTERNS_ROW
inherit
	RUN_CUT_PATTERNS_RAW
	ROW_COUNT_CHECK_ACCESS
	DB_OPTIONS_ACCESS
		export
			{NONE} all
		end

create
	make_template, make_null, make_by_key

feature -- Properties

	rows_for_run (a_run_id: STRING conn: DB_CONNECTION range: ITEM_COUNT_RANGE): LIST [like Current] is
			-- All rows for `a_run_id', ordered by field name and
			-- by cut pattern number within field name
		require
			a_run_id_exists: a_run_id /= Void
			connection_exists: conn /= Void
			connection_open: conn.is_open
			range_exists: range /= Void
		local
			b: DB_BIND_S
		do
			create b.make (a_run_id)
			Result := all_rows_filtered_ordered (conn, "run_id = :1", b, "field_name, cut_pattern_no")
			check_row_count (Result, range)
		ensure
			result_exists: Result /= Void
		end

	rows_for_production (a_prod_id: STRING conn: DB_CONNECTION range: ITEM_COUNT_RANGE): LIST [like Current] is
			-- All rows for production `a_prod_id', ordered by
			-- cut pattern number
		require
			a_prod_id_not_void: a_prod_id /= Void
			connection_not_void: conn /= Void
			connection_open: conn.is_open
			range_exists: range /= Void
		local
			b: DB_BIND_S
		do
			create b.make (a_prod_id)
			Result := all_rows_filtered_ordered (conn, "(run_id, packaging_lot_no) in (select run_id, packaging_lot_no from run_packaging_lots where production_id = :1)", b, "cut_pattern_no")
			check_row_count (Result, range)
		ensure
			result_exists: Result /= Void
		end

	rows_for_packaging_lot (a_run_id: STRING lot_no: INTEGER conn: DB_CONNECTION range: ITEM_COUNT_RANGE): LIST [like Current] is
			-- All rows for `a_run_id' that are assigned to
			-- packaging lot `lot_no'
		require
			a_run_id_exists: a_run_id /= Void
			connection_exists: conn /= Void
			connection_open: conn.is_open
			range_exists: range /= Void
		local
			b: DB_BIND_SI
		do
			create b.make (a_run_id, lot_no)
			Result := all_rows_filtered (conn, "run_id = :1 and packaging_lot_no = :2", b)
			check_row_count (Result, range)
		ensure
			result_exists: Result /= Void
		end

	is_unassigned: BOOLEAN is
			-- Is lot not assigned to any packaging lot?
		do
			Result := is_null_packaging_lot_no
		end

	assigned_to_lot (lot: RUN_PACKAGING_LOTS_ROW): BOOLEAN is
			-- Is `Current' assigned to `lot' (or is it unassigned
			-- to any lot if `lot' is Void)?
		do
			if lot /= Void then
				Result := not is_null_packaging_lot_no and packaging_lot_no = lot.packaging_lot_no
			else
				Result := is_unassigned
			end
		end

	packaging_lot: RUN_PACKAGING_LOTS_ROW
			-- Packaging lot to which `Current' is assigned
			-- or Void if not assigned to a lot yet
		do
			if not is_cached_packaging_lot then
				fetch_packaging_lot
			end
			Result := cached_packaging_lot
		end

	is_cached_packaging_lot: BOOLEAN
			-- Is `packaging_lot' cached?

	cuts: LIST [CUT_LOCATIONS_ROW]
			-- Cuts that define `Current'
		do
			if not is_cached_cuts then
				fetch_cuts
			end
			Result := cached_cuts
		end

	is_cached_cuts: BOOLEAN
			-- Is `cuts' cached?

	cut_die_sites: LIST [CUT_DIE_SITES_ROW]
			-- Cut die sites isolated by `Current'
		do
			if not is_cached_cut_die_sites then
				fetch_cut_die_sites
			end
			Result := cached_cut_die_sites
		end

	is_cached_cut_die_sites: BOOLEAN
			-- Is `cut_die_sites' cached?

	isolated_sites: HASH_TABLE [CUT_DIE_SITES_ROW, INTEGER]
			-- Sites that are isolated by `Current', indexed
			-- by mask site number

feature -- Modification

	clear_cuts_and_sites is
			-- Set `cuts' and `isolated_sites' to empty lists
		do
			create {ARRAYED_LIST [CUT_LOCATIONS_ROW]} cached_cuts.make (10)
			is_cached_cuts := True
			create isolated_sites.make (10)
		end
	
	set_packaging_lot (lot: like packaging_lot) is
			-- Set `packaging_lot' to `lot'
		do
			cached_packaging_lot := lot
			is_cached_packaging_lot := True
		ensure
			value_set: packaging_lot = lot
		end

	uncache_packaging_lot
			-- Remove `packaging_lot' from cache
		do
			cached_packaging_lot := Void
			is_cached_packaging_lot := False
		ensure
			packaging_lot_not_cached: not is_cached_packaging_lot
		end

	uncache_cuts
			-- Remove `cuts' from cache
		do
			is_cached_cuts := False
		ensure
			cuts_not_cached: not is_cached_cuts
		end

	uncache_cut_die_sites
			-- Remove `cut_die_sites' from cache
		do
			is_cached_cut_die_sites := False
		ensure
			cut_die_sites_not_cached: not is_cached_cut_die_sites
		end

feature {NONE} -- Implementation

	fetch_packaging_lot
			-- Fetch packaging lot for `Current', if any,
			-- and put it in `cached_packaging_lot'
		do
			if not is_null_packaging_lot_no then
				create cached_packaging_lot.make_by_key (connection, option.Old_no_exception_row, run_id, packaging_lot_no)
			 	if cached_packaging_lot.is_empty then
					-- No row retrieved
					cached_packaging_lot := Void
				end
			else
				cached_packaging_lot := Void
			end
			is_cached_packaging_lot := True
		end

	fetch_cuts
			-- Fetch cuts for `Current', and put in
			-- `cached_cuts'
		local
			row: CUT_LOCATIONS_ROW
		do
			create row.make_template
			cached_cuts := row.rows_for_run_and_pattern (run_id, cut_pattern_no, connection, expected_count.At_least_zero)
			is_cached_cuts := True
		end

	fetch_cut_die_sites
			-- Fetch cut die sites for `Current' and put
			-- in `cached_cut_die_sites'
		local
			row: CUT_DIE_SITES_ROW
		do
			create row.make_template
			cached_cut_die_sites := row.rows_for_run_and_pattern (run_id, cut_pattern_no, connection, expected_count.At_least_zero)
			is_cached_cut_die_sites := True
		end

	cached_packaging_lot: like packaging_lot
			-- Packaging lot for `Current', cached in memory

	cached_cuts: like cuts
			-- Cuts for `Current', cached in memory

	cached_cut_die_sites: like cut_die_sites
			-- Cut die sites for `Current', cached in memory

end
