
indexing
	
	description: "An operation that deletes a run"
	author: "David Hollenberg"
	date: "March 12, 2003"

class RUN_DELETION_OP
inherit
	OPERATION
		rename
			try_operation as try_to_delete_run
		end
	DB_QUERY_OPTIONS
		export
			{NONE} all
		undefine
			default_create
		end
	DB_OPTIONS_ACCESS
		export
			{NONE} all
		undefine
			default_create
		end
	ITEM_COUNT_RANGES_ACCESS
		export
			{NONE} all
		undefine
			default_create
		end
	RC_UTILITIES_ACCESS
		export
			{NONE} all
		end
	STRING_UTILITIES_ACCESS
		export
			{NONE} all
		undefine
			default_create
		end

create
	default_create

feature -- Operations

	delete_run (s: STRING sess: RC_RUN_CLOSING_SESSION a_role: STRING)
			-- Delete run with run id `s', using `sess' and
			-- with `a_role' as the only database role
		require
			name_exists: s /= Void
			session_exists: sess /= Void
			session_has_connection: sess.connection /= Void
			session_connection_open: sess.connection.is_open
			a_role_exists: a_role /= Void
		do
			session := sess
			run_id := s
			connection := sess.connection
			role := a_role
			execute
		end

feature {NONE} -- Implementation

	session: RC_RUN_CLOSING_SESSION
			-- Session to use

	connection: DB_CONNECTION
			-- Database connection to use

	run_id: STRING
			-- Run id for which to try to create run

	role: STRING
			-- Role to enable while deleting run

	try_to_delete_run
			-- Try to delete run
		local
			b: DB_BIND_S
			runs_row: RUNS_MROW
			productions_row: PRODUCTIONS_MROW
			prod_list: LIST [PRODUCTIONS_MROW]
			mask_sets_row: MASK_SETS_MROW
			mask_sets_list: LIST [MASK_SETS_MROW]
			ok_to_delete: BOOLEAN
			count: INTEGER
			production_type: PRODUCTION_TYPE_VALUES
			cut_die_sites_row: PACKING_CUT_DIE_SITES_MROW
			cut_locations_row: PACKING_CUT_LOCATIONS_MROW
			cut_patterns_row: PACKING_CUT_PATTERNS_MROW
			proj_options_row: PACKING_PROJECT_OPTIONS_MROW
			proj_packagings_row: PACKING_PROJECT_PACKAGINGS_MROW
			projs_row: PACKING_PROJECTS_MROW
			die_locations_row: PACKING_DIE_LOCATIONS_MROW
			dies_row: PACKING_DIES_MROW
			etest_colors_row: PACKING_ETEST_COLORS_MROW
			wafer_processes_row: PACKING_WAFER_PROCESSES_MROW
			versions_row: PACKING_VERSIONS_MROW
			packings_row: PACKINGS_MROW
			step_pat_row: FIELD_STEP_PATTERNS_MROW
			step_pat_loc_row: FIELD_STEP_PATTERNXFIELD_LOCS_MROW
			field_locs_row: FIELD_LOCATIONS_MROW
			field_sets_row: FIELD_SETS_MROW
			litho_sizes_row: LITHO_SIZES_MROW
			wlspsi_row: WAFER_LITHO_SIZE_PART_SEQ_IDS_MROW

			cut_die_sites:  LIST [DB_DELETABLE_ROW]
			cut_locations:  LIST [DB_DELETABLE_ROW]
			cut_patterns:  LIST [DB_DELETABLE_ROW]
			proj_options:  LIST [DB_DELETABLE_ROW]
			proj_packagings:  LIST [DB_DELETABLE_ROW]
			projs:  LIST [DB_DELETABLE_ROW]
			die_locations:  LIST [DB_DELETABLE_ROW]
			dies:  LIST [DB_DELETABLE_ROW]
			etest_colors:  LIST [DB_DELETABLE_ROW]
			wafer_processes:  LIST [DB_DELETABLE_ROW]
			versions:  LIST [DB_DELETABLE_ROW]
			packings:  LIST [DB_DELETABLE_ROW]
			step_pat_locs: LIST [DB_DELETABLE_ROW]
			step_pats: LIST [DB_DELETABLE_ROW]
			field_locs: LIST [DB_DELETABLE_ROW]
			field_sets: LIST [DB_DELETABLE_ROW]
			litho_sizes: LIST [DB_DELETABLE_ROW]
			waf_litho_size_part_seq_ids: LIST [DB_DELETABLE_ROW]
			lock_op: ROW_LOCK_OP
		do
			session.set_role (role)
			session.lock
			
			-- Check whether run already exists

			create runs_row.make_by_key (connection, Old_no_exception_row, run_id)
			if runs_row.is_empty then
				handle_user_error ("Run does not exist: " + run_id)
			end
			
			-- Verify that there are no primitive steps for run

			create b.make (run_id)
			count := connection.selected_count ("run_primitive_steps", "run_id = :1", b)
			ok_to_delete := count = 0
			if not ok_to_delete then
				handle_user_error ("Cannot delete run " + run_id + " - you must hard undo " + count.out + " " + su.number (count, "step", "steps") + " first")
			end
			
			-- Verify that there are no vendor work orders for run

			create b.make (run_id)
			count := connection.selected_count ("vendor_wos", "production_id = :1", b)
			ok_to_delete := count = 0
			if not ok_to_delete then
				handle_user_error ("Cannot delete run " + run_id + " because it has " + count.out + " vendor work " + su.number (count, "order", "orders"))
			end
			
			-- Verify that there is at most one production and
			-- one mask set

			create productions_row.make_template
			prod_list := productions_row.productions_for_run (run_id, connection, expected_count.At_least_zero)
			create mask_sets_row.make_template
			mask_sets_list := mask_sets_row.mask_sets_for_run (run_id, connection, expected_count.At_least_zero)
			if prod_list.count > 1 then
				handle_user_error ("Cannot delete run " + run_id + " because it has " + prod_list.count.out + " productions")
			end
			
			if mask_sets_list.count > 1 then
				handle_user_error ("Cannot delete run " + run_id + " because it has " + mask_sets_list.count.out + " mask sets")
			end
			
			confirm ("Deleting a run removes it completely from the database and %Ndeletes the entire run directory tree.%N%NAre you sure you want to delete run " + run_id)
			
			-- Fetch productions row and mask sets row

			create productions_row.make_template
			prod_list := productions_row.productions_for_run_and_type (run_id, production_type.Initial_run, connection, expected_count.At_least_zero)
			create mask_sets_row.make_template
			mask_sets_list := mask_sets_row.mask_sets_for_run_and_revision (run_id, rc.Initial_mask_revision, connection, expected_count.At_least_zero)
			if not prod_list.is_empty then
				productions_row := prod_list.first
			else
				productions_row := Void
			end
			
			if not mask_sets_list.is_empty then
				mask_sets_row := mask_sets_list.first
			else
				mask_sets_row := Void
			end
			
			-- Fetch all packings and associated rows intended
			-- for this run

			create cut_die_sites_row.make_template
			cut_die_sites := cut_die_sites_row.rows_for_run (run_id, connection, expected_count.At_least_zero)
			create cut_locations_row.make_template
			cut_locations := cut_locations_row.rows_for_run (run_id, connection, expected_count.At_least_zero)
			create cut_patterns_row.make_template
			cut_patterns := cut_patterns_row.rows_for_run (run_id, connection, expected_count.At_least_zero)
			create die_locations_row.make_template
			die_locations := die_locations_row.rows_for_run (run_id, connection, expected_count.At_least_zero)
			create dies_row.make_template
			dies := dies_row.rows_for_run (run_id, connection, expected_count.At_least_zero)
			create proj_options_row.make_template
			proj_options := proj_options_row.rows_for_run (run_id, connection, expected_count.At_least_zero)
			create proj_packagings_row.make_template
			proj_packagings := proj_packagings_row.rows_for_run (run_id, connection, expected_count.At_least_zero)
			create projs_row.make_template
			projs := projs_row.rows_for_run (run_id, connection, expected_count.At_least_zero)
			create etest_colors_row.make_template
			etest_colors := etest_colors_row.rows_for_run (run_id, connection, expected_count.At_least_zero)
			create wafer_processes_row.make_template
			wafer_processes := wafer_processes_row.rows_for_run (run_id, connection, expected_count.At_least_zero)
			create versions_row.make_template
			versions := versions_row.rows_for_run (run_id, connection, expected_count.At_least_zero)
			create packings_row.make_template
			packings := packings_row.rows_for_run (run_id, connection, expected_count.At_least_zero)

			-- Fetch all field sets, field set locations
			-- litho sizes, field step patterns and field
			-- step pattern locations that are custom ones for this run

			create step_pat_loc_row.make_template
			step_pat_locs := step_pat_loc_row.custom_step_patterns_locs_for_run (run_id, connection, expected_count.At_least_zero)
			create step_pat_row.make_template
			step_pats := step_pat_row.custom_step_patterns_for_run (run_id, connection, expected_count.At_least_zero)
			create field_locs_row.make_template
			field_locs := field_locs_row.custom_locations_for_run (run_id, connection, expected_count.At_least_zero)
			create field_sets_row.make_template
			field_sets := field_sets_row.custom_field_sets_for_run (run_id, connection, expected_count.At_least_zero)
			create litho_sizes_row.make_template
			litho_sizes := litho_sizes_row.custom_litho_sizes_for_run (run_id, connection, expected_count.At_least_zero)
			create wlspsi_row.make_template
			waf_litho_size_part_seq_ids := wlspsi_row.rows_for_custom_litho_sizes_for_run (run_id, connection, expected_count.At_least_zero)
			
			-- Lock RUNS table row and all rows that depend on it via
			-- foreign key relationship
	
			session.open_log
			session.lock_row (runs_row)
			if productions_row /= Void then
				session.lock_row (productions_row)
			end
			if mask_sets_row /= Void then
				session.lock_row (mask_sets_row)
			end
			create lock_op
			lock_op.lock_list_nowait (cut_die_sites)
			lock_op.lock_list_nowait (cut_locations)
			lock_op.lock_list_nowait (cut_patterns)
			lock_op.lock_list_nowait (die_locations)
			lock_op.lock_list_nowait (dies)
			lock_op.lock_list_nowait (proj_options)
			lock_op.lock_list_nowait (proj_packagings)
			lock_op.lock_list_nowait (projs)
			lock_op.lock_list_nowait (etest_colors)
			lock_op.lock_list_nowait (wafer_processes)
			lock_op.lock_list_nowait (versions)
			lock_op.lock_list_nowait (packings)
			lock_op.lock_list_nowait (step_pat_locs)
			lock_op.lock_list_nowait (step_pats)
			lock_op.lock_list_nowait (field_locs)
			lock_op.lock_list_nowait (field_sets)
			lock_op.lock_list_nowait (waf_litho_size_part_seq_ids)
			lock_op.lock_list_nowait (litho_sizes)
			
			-- Delete rows in correct order to avoid
			-- referential integrity violations
	
			runs_row.set_null_field_set_name
			runs_row.set_null_packing_id
			runs_row.set_null_packing_version_no
			session.post
			cut_die_sites.do_all (agent {DB_DELETABLE_ROW}.delete)
			cut_locations.do_all (agent {DB_DELETABLE_ROW}.delete)
			cut_patterns.do_all (agent {DB_DELETABLE_ROW}.delete)
			die_locations.do_all (agent {DB_DELETABLE_ROW}.delete)
			dies.do_all (agent {DB_DELETABLE_ROW}.delete)
			proj_options.do_all (agent {DB_DELETABLE_ROW}.delete)
			proj_packagings.do_all (agent {DB_DELETABLE_ROW}.delete)
			projs.do_all (agent {DB_DELETABLE_ROW}.delete)
			etest_colors.do_all (agent {DB_DELETABLE_ROW}.delete)
			wafer_processes.do_all (agent {DB_DELETABLE_ROW}.delete)
			versions.do_all (agent {DB_DELETABLE_ROW}.delete)
			packings.do_all (agent {DB_DELETABLE_ROW}.delete)
			step_pat_locs.do_all (agent {DB_DELETABLE_ROW}.delete)
			step_pats.do_all (agent {DB_DELETABLE_ROW}.delete)
			field_locs.do_all (agent {DB_DELETABLE_ROW}.delete)
			field_sets.do_all (agent {DB_DELETABLE_ROW}.delete)
			waf_litho_size_part_seq_ids.do_all (agent {DB_DELETABLE_ROW}.delete)
			litho_sizes.do_all (agent {DB_DELETABLE_ROW}.delete)
			if mask_sets_row /= Void then
				mask_sets_row.delete
			end
			if productions_row /= Void then
				productions_row.delete
			end
			runs_row.delete
			session.commit
			publish_deleted_event
			delete_run_dir
			inform ("Run " + run_id + " deleted")
			session.terminate
		end

	delete_run_dir
			-- Delete the run directory
		local
			dir_name: STRING
			remover: DIR_TREE_REMOVER
		do
			dir_name := session.run_directory
			if session.path_exists (dir_name) then
				create remover.make (dir_name)
				remover.set_directory_symlinks_followed (False)
				remover.remove
				if session.path_exists (dir_name) then
					warn_with_notify ("cannot remove entire directory tree " + dir_name + "%N" + remover.error_count.out + " errors encountered%N%NPlease remove this directory tree manually")
				end
			end
		end

	publish_deleted_event
		local
			deleted_event: RUN_DELETED_EVENT
		do
			create deleted_event
			deleted_event.set_run_id (run_id)
			deleted_event.publish
		end

end
