
indexing
	
	description: "A run closing step operation that inserts an %
		%event for the run"
	author: "David Hollenberg"
	date: "June 6, 2005"

deferred class RUN_EVENT_INSERTION_OP
inherit
	FAB_POSTING_OP
		redefine
			state, is_currently_undoable, not_undoable_reason
		end
	RC_APPLICABLE_STEP
	HAS_USER_INTERFACE
	NO_EXTERNAL_STATE
	DATE_VALIDITY
		export
			{NONE} all
		undefine
			default_create
		end
	RC_POLICIES_ACCESS
		export
			{NONE} all
		undefine
			default_create
		end

feature -- Properties

	state: DB_DATE
			-- State saved by soft undo

feature {NONE} -- Implementation

	is_currently_undoable: BOOLEAN
			-- Is `Current' currently in a state where it can be
			-- undone?
		local
			diff: DOUBLE
		do
			fetch_run_event
			diff := connection.date_difference (connection.system_date, step.run_step.date_completed)
			Result := diff <= max_undo_allowed_days
		end

	not_undoable_reason: STRING
			-- Reason step cannot currently be undone
		do
			Result := event_name + " event was inserted more than " + max_undo_allowed_days.out + " days ago"
		end

	do_step
		local
			ifc: SINGLE_FREE_VALUE_IFC
			ask: BOOLEAN
			def_date: STRING
		do
			fetch_run_event
			if run_events_row /= Void then
				ask := question ("There is already a " + event_name + " event dated " + run_events_row.date_event.to_date_string + ".%NDo you want to change the date?")
				def_date := run_events_row.date_event.to_date_string
			else
				ask := True
				if state /= Void then
					def_date := state.to_date_string
				else
					def_date := connection.transaction_date.to_date_string
				end
			end
			if ask then
				ifc := factory.single_free_value_interface (session.title_with_run_id ("Select " + date_name))
				ifc.set_label (date_name)
				ifc.set_minimum_width_in_chars (11)
				ifc.enable_uppercase_only
				ifc.set_default_value (def_date)
				ifc.build
				ifc.accept_actions.extend (agent ifc.execute_action (agent call_safely_with_string (?, agent set_run_event_date, agent reset (ifc))))
				ifc.decline_actions.append (abort_actions)
			else
				get_final_confirmation (run_events_row.date_event)
				call_completion_actions
			end
		end

	undo_step
			-- Undo execution of step
		do
			fetch_run_event
			if run_events_row /= Void then
				run_events_row.delete
			end
			add_undo_commit_actions
			call_completion_actions
		end

	fetch_run_event
			-- Fetch the event (if any), put it in
			-- `run_events_row' and lock it
		local
			lock_op: ROW_LOCK_OP
		do
			create run_events_row.make_by_key (connection, option.Old_no_exception_row, session.run_id, event_name)
			if run_events_row.is_old then
				create lock_op
				lock_op.lock_nowait (run_events_row)
			else
				run_events_row := Void
			end
		end

	set_run_event_date (s: STRING)
			-- Insert event with date `s' if
			-- it is a valid date and is in the appropriate
			-- date range
		require
			string_exists: s /= Void
		local
			d: DB_DATE
		do
			if not valid_date (s) then
				handle_user_error ("Date " + su.double_quoted (s) + " is not a valid date")
			end
			d := connection.date (s)
			if d > connection.system_date.truncated_to_midnight then
				handle_user_error ("Date cannot be in the future")
			end
			check_date_validity (d)
			get_final_confirmation (d)
			insert_event (d)
			call_completion_actions
		end

	insert_event (d: DB_DATE)
			-- Insert event for run
		require
			d_not_void: d /= Void
		local
			row: RUN_EVENTS_MROW
		do
			if run_events_row /= Void then
				run_events_row.set_date_event (d)
			else
				create row.make_by_key (connection, option.New_row, session.run_id, event_name)
				row.set_date_event (d)
				row.set_significant (True)
				row.set_actual_date (True)
			end
		end

	get_final_confirmation (d: DB_DATE)
			-- Get final confirmation that event should really
			-- be inserted and notify appropriate parties
		require
			d_not_void: d /= Void
		deferred
		end

	set_state
			-- Set `state'
		do
			fetch_run_event
			if run_events_row /= Void then
				state := run_events_row.date_event
			else
				state := Void
			end
		end

	run_events_row: RUN_EVENTS_MROW

	event_name: STRING
			-- Name of event to be inserted by operation
		deferred
		ensure
			result_not_void: Result /= Void
		end

	date_name: STRING
			-- Name of date user is asked to enter
		deferred
		ensure
			result_not_void: Result /= Void
		end

	add_undo_commit_actions
			-- Add any required session commit actions for undo
		deferred
		end

	check_date_validity (d: DB_DATE)
			-- Check that `d' is valid for `event_name'
		require
			d_not_void: d /= Void
		deferred
		end

	max_undo_allowed_days: INTEGER
			-- Maximum age of event in days to allow undo
			-- of step
		deferred
		end

end
