class RUN_GDS_FILES_ROW

inherit

	RUN_GDS_FILES_RAW
	ROW_COUNT_CHECK_ACCESS
	CONST_BATCH_JOB_REQUESTOR
	ITEM_COUNT_RANGES_ACCESS
		export
			{NONE} all
		end

create

        make_template, make_null, make_detached, make_by_key,
	make_by_die_id, make_by_field_name

feature {NONE} -- alternate creation routines

	make_by_die_id(conn: DB_CONNECTION; opt: DB_QUERY_OPTION;
				rid, did: STRING)
	local b: DB_BIND_SS
	do  create b.make(rid, did);
	    make_by_alternate_key(conn, opt, 
				"run_id = :1 and die_id = :2", b)
	end;

	make_by_field_name(conn: DB_CONNECTION; opt: DB_QUERY_OPTION;
				rid, fnam: STRING)
	local b: DB_BIND_SS
	do  create b.make(rid, fnam);
	    make_by_alternate_key(conn, opt,
				"run_id = :1 and field_name = :2", b)
	end;

feature -- Collections

	rows_for_run (run: STRING conn: DB_CONNECTION range: ITEM_COUNT_RANGE): DYNAMIC_LIST [like Current] is
			-- All rows for run `run', ordered by file_no
		require
			run_exists: run /= Void
			connection_exists: conn /= Void
			connection_open: conn.is_open
			range_exists: range /= Void
		local
			b: DB_BIND_S
		do
			create b.make (run)
			Result := all_rows_filtered_ordered (conn, "run_id = :1", b, "file_no")
			check_row_count (Result, range)
		ensure
			result_exists: Result /= Void
		end

feature -- Properties

	merge_result: like anchored_merge_result
			-- Merge result for `Current'
		require
			connection_open: connection.is_open
		local
			list: LIST [like anchored_merge_result]
			template_row: like anchored_merge_result
		do
			create template_row.make_template
			list := template_row.rows_for_run_and_file (run_id, file_no, connection, expected_count.At_most_one)
			if not list.is_empty then
				Result := list.first
			end
		end

	merge_layers: DYNAMIC_LIST [like anchored_merge_layer]
			-- Merge result for `Current'
		require
			connection_open: connection.is_open
		local
			template_row: like anchored_merge_layer
		do
			create template_row.make_template
			Result := template_row.rows_for_run_and_file (run_id, file_no, connection, expected_count.At_least_zero)
		end

feature {NONE} -- Implementation

	anchored_merge_result: RUN_GDS_MERGE_RESULTS_ROW
			-- For anchoring only
		do
		ensure
			result_void: Result = Void
		end

	anchored_merge_layer: RUN_GDS_FILE_LAYERS_ROW
			-- For anchoring only
		do
		ensure
			result_void: Result = Void
		end

end -- RUN_GDS_FILES_ROW
