
class RUN_GDS_MERGE_RESULTS_ROW

inherit
	RUN_GDS_MERGE_RESULTS_RAW
	ROW_COUNT_CHECK_ACCESS
	NULL_VALUE_CHECK_ACCESS

create

        make_template, make_null, make_detached, make_by_key,
	make_by_die_id, make_by_field_name

feature {NONE} -- Alternate creation routines

	make_by_die_id(conn: DB_CONNECTION; opt: DB_QUERY_OPTION;
				rid, did: STRING)
	local b: DB_BIND_SS
	do  create b.make(rid, did);
	    make_by_alternate_key(conn, opt, 
		"(run_id, file_no) in (select run_id, file_no from run_gds_files where run_id = :1 and die_id = :2)", b)
	end;

	make_by_field_name(conn: DB_CONNECTION; opt: DB_QUERY_OPTION;
				rid, fnam: STRING)
	local b: DB_BIND_SS
	do  create b.make(rid, fnam);
	    make_by_alternate_key(conn, opt,
		"(run_id, file_no) in (select run_id, file_no from run_gds_files where run_id = :1 and field_name = :2)", b)
	end;

feature -- Collections

	rows_for_run (run: STRING conn: DB_CONNECTION range: ITEM_COUNT_RANGE): DYNAMIC_LIST [like Current] is
			-- All rows for run `run', ordered by file_no
		require
			run_exists: run /= Void
			connection_exists: conn /= Void
			connection_open: conn.is_open
			range_exists: range /= Void
		local
			b: DB_BIND_S
		do
			create b.make (run)
			Result := all_rows_filtered_ordered (conn, "run_id = :1", b, "file_no")
			check_row_count (Result, range)
		ensure
			result_exists: Result /= Void
		end

	rows_for_run_and_file (run: STRING n: INTEGER conn: DB_CONNECTION range: ITEM_COUNT_RANGE): DYNAMIC_LIST [like Current] is
			-- All rows for run `run' and file_no `n'
		require
			run_exists: run /= Void
			connection_exists: conn /= Void
			connection_open: conn.is_open
			range_exists: range /= Void
		local
			b: DB_BIND_SI
		do
			create b.make (run, n)
			Result := all_rows_filtered (conn, "run_id = :1 and file_no = :2", b)
			check_row_count (Result, range)
		ensure
			result_exists: Result /= Void
		end

feature -- Properties

	non_null_top_structure: like top_structure is
			-- Value of `top_structure', with error check 
			-- for nullness
		do
			check_not_null ("top_structure", is_null_top_structure, Current)
			Result := top_structure
		end

	non_null_size_x: like size_x is
			-- Value of `size_x', with error check 
			-- for nullness
		do
			check_not_null ("size_x", is_null_size_x, Current)
			Result := size_x
		end

	non_null_size_y: like size_y is
			-- Value of `size_y', with error check 
			-- for nullness
		do
			check_not_null ("size_y", is_null_size_y, Current)
			Result := size_y
		end

	non_null_center_x: like center_x is
			-- Value of `center_x', with error check 
			-- for nullness
		do
			check_not_null ("center_x", is_null_center_x, Current)
			Result := center_x
		end

	non_null_center_y: like center_y is
			-- Value of `center_y', with error check 
			-- for nullness
		do
			check_not_null ("center_y", is_null_center_y, Current)
			Result := center_y
		end

end

