class SALES_TAX_LOOKUP
		-- determine sales tax rate by zip code (and date)

inherit
	DB_ROW_LIST[ZIP_TAXES_ROW]
	rename make as make_list
	end

	DB_ACCESS
	undefine copy, is_equal
	end

create
	make

create {SALES_TAX_LOOKUP}
	make_filled

feature -- lookup

	percentage(zip: INTEGER; d: DB_DATE): DOUBLE is
		-- sales tax rate for zip code zip as of time d
		-- NOTE: value is in % points
		--   (i.e., 8.25% tax rate returns 8.25, not 0.0825)
	local  f, r: ZIP_TAXES_ROW; frange, rrange: INTEGER
	do  from  start
	    until after
	    loop  r := item;
		  rrange := r.zip_2 - r.zip_1		-- how broad??
		  if r.zip_1 <= zip and zip <= r.zip_2	-- right zip
		     and r.date_effective <= d		-- right date
		     and (f = Void or else		-- first hit
			  rrange < frange or		-- or more specific
			  (rrange = frange and		-- or more recent
			   r.date_effective > f.date_effective))
		   then f := r			-- f is best find (so far)
			frange := rrange
		  end
		  forth
	    end;
	    if f /= Void then Result := f.tax end;
	end;

	current_percentage(zip: INTEGER): DOUBLE is
	do  Result := percentage(zip, db.system_date)
	end;

feature {NONE} -- creation

	make is
		-- get the zip list
	local  q: ANONYMOUS_QUERY[ZIP_TAXES_ROW]; r: ZIP_TAXES_ROW;
	do  create r.make_template;
	    create q.make(db, r);
	    make_by_query(q);	-- get all the rows
	end;

end -- SALES_TAX_LOOKUP
