
indexing
	
	description: "Action to build slots for scan"
	author: "David Hollenberg"
	date: "January 3, 2007"

deferred class SCAN_ACTION
inherit
	VIEW_ELEMENT_ACTION
		redefine
			default_create
		end
	VIEW_ELEMENT_ACTIONS
		export
			{NONE} all
		redefine
			default_create
		end
	GRIDABLE
		redefine
			default_create
		end
	STRING_UTILITIES_ACCESS
		export
			{NONE} all
		redefine
			default_create
		end

feature {NONE} -- Creation

	default_create
		do
			create move_distance.make_xy (0, 0)
		end

feature -- Properties

	scan_slots: LIST [SCAN_SLOT]
			-- Slots into which movable elements will be placed
			-- (to be built)

	field_lower_limit: DOUBLE
			-- Coordinate of left or bottom edge of field

	field_upper_limit: DOUBLE
			-- Coordinate of right or top edge of field

feature -- Modification

	set_scan_slots (a_list: like scan_slots)
			-- Set `scan_slots' to `a_list'
		require
			a_list_not_void: a_list /= Void
		do
			scan_slots := a_list
		ensure
			scan_slots_set: scan_slots = a_list
		end

	set_field_limits (f: RETICLE_FIELD)
			-- Set `field_lower_limit' and `field_upper_limit'
			-- from `f'
		deferred
		end

feature -- Operations

	start
			-- Take actions needed before any calls to `apply'
		do
			last_obstacle_center := field_lower_limit
			create current_slot.make
			current_slot.set_lower_limit (field_lower_limit)
			current_slot.set_lower_edge (field_lower_limit)
		end

	apply (an_element: RETICLE_VIEW_ELEMENT)
			-- Apply action to `an_element'
		local
			l_center: DOUBLE
		do
			if is_obstacle (an_element) then
				-- Not movable
				l_center := gridded_center (an_element)
				if l_center /= last_obstacle_center then
					last_obstacle_center := l_center
					current_slot.set_upper_limit (l_center)
					current_slot.set_upper_edge (gridded_lower_limit (an_element))
					scan_slots.extend (current_slot)
					-- Create new slot
					create current_slot.make
					current_slot.set_lower_limit (l_center)
					current_slot.set_lower_edge (gridded_upper_limit (an_element))
				elseif l_center = field_lower_limit then
					-- Obstacle is centered exactly on
					-- field edge
					current_slot.set_lower_edge (gridded_upper_limit (an_element))
				else
					-- Current obstacle is at same location
					-- as previous one and is not centered
					-- exactly on field edge, so ignore it
				end
			else
				-- Movable
				current_slot.elements.extend (an_element)
			end
			last_element := an_element
		end

	finish
			-- Take actions needed after all calls to `apply'
		do
			current_slot.set_upper_limit (field_upper_limit)
			current_slot.set_upper_edge (field_upper_limit)
			scan_slots.extend (current_slot)
			check_movable_scribes_by_field_edge
		end

	do_scan
			-- Process `scan_slots' to place items and
			-- generate warnings
		local
			n: INTEGER
		do
			from
				scan_slots.start
			until
				scan_slots.after
			loop
				n := n + 1
				scan_one_slot (scan_slots.item, n)
				scan_slots.forth
			end
		end

feature {NONE} -- Implementation

	check_movable_scribes_by_field_edge
			-- Check whether there is a movable scribe adjacent to
			-- field of view edge and add error if so
		local
			first, last: SCAN_SLOT
			first_elems, last_elems: LIST [RETICLE_VIEW_ELEMENT]
			first_elem, last_elem: RETICLE_VIEW_ELEMENT
			bad_first, bad_last: BOOLEAN
			msg: STRING
		do
			first := scan_slots.first
			last := scan_slots.last
			first_elems := first.elements
			last_elems := last.elements
			bad_first := False
			bad_last := False
			if not first_elems.is_empty then
				first_elem := first_elems.first
				if first_elem.is_scribe and then not is_obstacle (first_elem) then
					bad_first := True
				end
			end
			if not last_elems.is_empty then
				last_elem := last_elems.last
				if last_elem.is_scribe and then not is_obstacle (last_elem) then
					bad_last := True
				end
			end
			if bad_first or bad_last then
				msg := "no selected project between movable scribe and field of view "
				if bad_first and bad_last then
					msg.append (lower_edge_name)
					msg.append (" and ")
					msg.append (upper_edge_name)
					msg.append (" edges")
				elseif bad_first then
					msg.append (lower_edge_name)
					msg.append (" edge")
				else
					msg.append (upper_edge_name)
					msg.append (" edge")
				end
				last_element.add_error (msg)
			end
		end

	scan_one_slot (a_slot: SCAN_SLOT a_slot_number: INTEGER)
			-- Scan `a_slot' which is `a_slot_number' in the list
		require
			a_slot_not_void: a_slot /= Void
		local
			list: LIST [RETICLE_VIEW_ELEMENT]
			elem: RETICLE_VIEW_ELEMENT
			l_available_space, exact_spacing, spacing, edge: DOUBLE
			extra: DOUBLE
			l_available_grid_units: INTEGER_64
			k, n, num_extra_used, extra_grid_units: INTEGER_64
			spacing_in_grid_units: INTEGER_64
		do
			available_width := grid.rounded_size (a_slot.width)
			needed_width := slot_needed_width (a_slot)
			list := a_slot.elements
			
			if needed_width <= available_width then
				-- Movable objects assigned to slot fit -
				-- space them out evenly in available space
				
				n := list.count + 1
				l_available_space := grid.rounded_size (available_width - needed_width)
				l_available_grid_units := grid.rounded_size_units (l_available_space)
				exact_spacing := l_available_space / n
				spacing_in_grid_units := l_available_grid_units // n
				extra_grid_units := l_available_grid_units \\ n
				spacing := grid.rounded_size (spacing_in_grid_units * grid.size_resolution)
				
				num_extra_used := 0
				k := 1
				edge := a_slot.lower_edge
				from
					list.start
				until
					list.after
				loop
					elem := list.item
					if (k * extra_grid_units) >= (num_extra_used + 1) * n then
						num_extra_used := num_extra_used + 1
						extra := grid.size_resolution
					else
						extra := 0
					end
					edge := grid.rounded_edge (edge + spacing + extra)
					set_move_distance (elem, edge)
					move_action.set_move_distance (move_distance)
					move_action.apply (elem)
					edge := grid.rounded_edge (edge + gridded_width (elem))
					k := k + 1
					list.forth
				end
				
			else
				add_slot_too_small_warnings (a_slot, a_slot_number)
			end
		end

	slot_needed_width (a_slot: SCAN_SLOT): DOUBLE
			-- Width required for movable objects assigned
			-- to `a_slot'
		require
			a_slot_not_void: a_slot /= Void
		local
			list: LIST [RETICLE_VIEW_ELEMENT]
		do
			list := a_slot.elements
			from
				list.start
			until
				list.after
			loop
				Result := grid.rounded_size (Result + gridded_width (list.item))
				list.forth
			end
			
		end

	add_slot_too_small_warnings (a_slot: SCAN_SLOT a_slot_number: INTEGER)
			-- Add warnings that say `a_slot' which is
			-- `a_slot_number' in the list is too small to
			-- hold the assigned objects, plus details about the
			-- objects
		require
			a_slot_not_void: a_slot /= Void
		local
			list: LIST [RETICLE_VIEW_ELEMENT]
			elem: RETICLE_VIEW_ELEMENT
			n: INTEGER
		do
			list := a_slot.elements
			n := list.count
			if n > 0 then
				last_element.add_warning ("Scan slot " + a_slot_number.out + ":  available space is " + grid.formatted_number (available_width) + " but need " + grid.formatted_number (needed_width) + " for " + n.out + " " + su.number (n, "object", "objects") + " (short by " + grid.formatted_number (grid.rounded_size (needed_width - available_width)) + ")")
			else
				last_element.add_warning ("Scan slot " + a_slot_number.out + ":  unmovable object overlaps adjacent unmovable object or field of view edge by " + grid.formatted_number (-available_width))
			end
			from
				list.start
			until
				list.after
			loop
				elem := list.item
				last_element.add_warning ("     " + elem.identification_for_message + " " + dimension_name + " is " + grid.formatted_number (gridded_width (elem)))

				list.forth
			end
		end

	is_obstacle (an_element: RETICLE_VIEW_ELEMENT): BOOLEAN
			-- Is `an_element' an obstacle (won't be moved by
			-- scan and helps define limits of a slot)?
		deferred
		end

	gridded_center (an_element: RETICLE_VIEW_ELEMENT): DOUBLE
			-- Location of element center, placed on grid
		deferred
		end

	gridded_lower_limit (an_element: RETICLE_VIEW_ELEMENT): DOUBLE
			-- Left or bottom edge of element, placed on grid
		deferred
		end

	gridded_upper_limit (an_element: RETICLE_VIEW_ELEMENT): DOUBLE
			-- Right or top edge of element, placed on grid
		deferred
		end

	gridded_width (an_element: RETICLE_VIEW_ELEMENT): DOUBLE
			-- Wdith of element, rounded to grid
		deferred
		end

	set_move_distance (an_element: RETICLE_VIEW_ELEMENT a_lower_edge: DOUBLE)
			-- Set `move_distance' for `an_element' to move its
			-- lower edge to `a_lower_edge'
		deferred
		end

	lower_edge_name: STRING
			-- Name of lower edge of field of view
		deferred
		ensure
			result_not_void: Result /= Void
		end

	upper_edge_name: STRING
			-- Name of upper edge of field of view
		deferred
		ensure
			result_not_void: Result /= Void
		end

	dimension_name: STRING
			-- Name of dimension of objects that matters for scan
		deferred
		ensure
			result_not_void: Result /= Void
		end

	current_slot: SCAN_SLOT
			-- Slot being built

	last_element: RETICLE_VIEW_ELEMENT
			-- Last element encountered during scan

	last_obstacle_center: DOUBLE
			-- Center of last obstacle encountered during scan

	available_width: DOUBLE
			-- Width available in slot being scanned
	
	needed_width: DOUBLE
			-- Width needed by movable objects assigned to
			-- slot being scanned
	
	move_distance: POINT
			-- Distance to move next object

end
