class SEARCH_PATH
	-- implement our version of a (Unix style) search path

inherit
	ARRAYED_LIST[STRING]
	-- where we keep the search path entries

	STRING_UTILITIES_ACCESS
	undefine  copy, is_equal
	end

	UNIX_OS_ACCESS
	undefine  copy, is_equal
	end

create
	make, 
	make_from_string

create {SEARCH_PATH}
	make_filled

feature

	make_from_string(eval: STRING; dict: DICTIONARY) is
		-- eval is of the form "path1:path2:...:pathn"
		-- if any path begins with a '$', it is taken as a
		--  variable, looked up in dict, and expanded (recursively)
	require eval /= Void;
		-- dict can be Void iff it is not needed;
	local l: LIST[STRING]; p: STRING; c: CHARACTER;
	do  l := su.broken_at(eval,':');
	    l := su.empty_strings_removed(l);
	    if lower = 0 then make(l.count) end;  -- just make me once
	    c := operating_environment.directory_separator;
	    from  l.start
	    until l.after
	    loop  p := l.item
		  if (p @ 1) = '$'
		   then p.remove(1);		-- remove it
			p := dict.item(p);
			make_from_string(p, dict) -- and put into the list
		   else -- put p into the list (but add a final dir_sep)
			if (p @ p.count) /= c
			 then p.extend(c)
			end;
			extend(p)
		  end;
		  l.forth
	    end;
	end;

	find_file(fnam: STRING): STRING is
		-- returns full file name if found on path
		-- returns Void if not found
	require fnam /= Void
	local f: LARGE_RAW_FILE; fullnam: STRING
	do  create f.make(fnam);	-- (cannot create f without a name)
	    from  start
	    until after or Result /= Void
	    loop  fullnam := unix_os.full_file_name(item, fnam)
		  f.make(fullnam);		-- and give it a try
		  if f.exists then Result := fullnam end
		  forth
	    end;
	end;

	all_exist: BOOLEAN is
		-- check list to see if all directories exist
		-- return TRUE iff they do; otherwise, item is first bad one
	local f: LARGE_RAW_FILE;
	do  from  start;  create f.make(item);  Result := true;
	    until after or not Result
	    loop  f.make(item);
		  Result := (f.exists and then f.is_directory)
		  if Result then forth end
	    end
	end;

end -- SEARCH_PATH
