
indexing
	
	description: "A run closing step operation that allows runs closer %
		%to select and deselect projects and test vehicles, pack %
		%them into the reticle and create cut patterns"
	author: "David Hollenberg"
	date: "October 30, 2005"

class SELECT_AND_PACK_PROJECTS_OP
inherit
	RC_PRIMITIVE_STEP_OP
	RC_APPLICABLE_STEP
	HAS_USER_INTERFACE
	NO_EXTERNAL_STATE
	PACKING_ACCESSOR
		export
			{NONE} all
		undefine
			default_create
		end
	UNIX_OS_ACCESS
		export
			{NONE} all
		undefine
			default_create
		end
	FILE_UTILITIES_ACCESS
		export
			{NONE} all
		undefine
			default_create
		end

create
	make

feature -- Properties

	input_data_items: LIST [STRING]
			-- Data items required before step can be executed
		do
			Result := array_as_list (<<
					data_item.Assigned_packing
				 >>)
		end

feature {NONE} -- Implementation

	do_step
			-- Execute step
		do
			fetch_packing
			if packing = Void or else not packing.packings_row.packed_with_new_packer then
				-- No packing or packed by old VMS
				-- ICL packer
				create_packing_version_from_box_file
			end
			select_and_pack_projects
		end

	select_and_pack_projects
			-- Select projects
		local
			ifc: RC_PACKING_IFC
			pack_id: STRING
		do
			pack_id := cache.runs_row.packing_id
			session.set_packing_id (pack_id)
			ifc := factory.packing_interface (Void, session.run_id, session.packing_id, session.title_with_run_and_packing_id ("Edit"))
			ifc.set_session (session)
			ifc.set_packing_session (session)
			ifc.set_run_id (session.run_id)
			ifc.set_packing_id (pack_id)
			ifc.set_is_read_only (False)
			ifc.success_actions.extend (agent call_completion_actions)
			ifc.cancel_actions.append (abort_actions)
			ifc.build
		end

	undo_step
			-- Undo execution of step
		do
			call_completion_actions
		end

	set_state
			-- Set `state'
		do
			state := Void
		end

	create_packing_version_from_box_file
			-- Create new version of packing from contents of
			-- packer box file indicated by RUNS table.
			-- Use packing which has same name as upper case
			-- run ID if have to create a new packing
		local
			unix_pbf_name: STRING
		do
			fetch_or_create_packing_selected_version
			unix_pbf_name := version_adjusted_name (cache.runs_row.packer_box_full_filename)
			packing.update_current_version_from_packer_box_file (unix_pbf_name)
			if not packing.packings_row.packed_with_new_packer then
				-- Once we save new version below, we will have a
				-- valid packing constructed from packer
				-- box file.  Do this right before the
				-- save to avoid holding locks for a long time
				packing.packings_row.set_packed_with_new_packer (True)
			end
			packing.save_new_version (computed_version_id (unix_pbf_name), cache.runs_mrow)
		end

	fetch_or_create_packing_selected_version
			-- Fetch selected version of packing for run
			-- and put it in `packing_version'.  If no
			-- versions of packing exist, create a new
			-- version and then fetch it
		local
			p: PACKINGS_MROW
		do
			fetch_packing
			if packing = Void then
				create p.make_null (connection)
				p.set_packing_id (session.run_id)
				p.initialize_from_run (cache.runs_row)
				p.set_packed_with_new_packer (False)
				create packing
				packing.set_packings_row (p)
				packing.create_initial_version
				packing.save_new_dummy_vms_version (cache.runs_mrow)
			end
			fetch_packing_selected_version
		end

	version_adjusted_name (fname: STRING): STRING
			-- `fname' with version number removed if it
			-- has a version number and version is 1 and
			-- there are no versions of file in directory
		require
			fname_not_void: fname /= Void
		local
			dir_name, file_name, vers: STRING
			name_without_vers: STRING
			del: BOOLEAN
			pos, n: INTEGER
		do
			del := False
			dir_name := fu.directory_part (fname)
			file_name := fu.file_name_part (fname)
			pos := file_name.last_index_of (version_separator, file_name.count)
			if pos > 0 then
				name_without_vers := file_name.substring (1, pos - 1)
				vers := file_name.substring (pos + 1, file_name.count)
				if vers.is_integer then
					n := vers.to_integer
					if n = 1 then
						if not fu.file_exists (fname) and then not directory_has_versions (dir_name, name_without_vers) then
							del := True
						end
					end
				end
			end
			
			if del then
				Result := unix_os.full_file_name (dir_name, name_without_vers)
			else
				Result := fname
			end
		end

	directory_has_versions (dir_name, fname: STRING): BOOLEAN
			-- Does directory `dir_name' contain any explicit
			-- versions (ending with ";<N>") of file
			-- `fname'?
		require
			dir_name_not_void: dir_name /= Void
			fname_not_void: fname /= Void
		local
			dir: LARGE_DIRECTORY
			open_op: DIR_OPEN_READ_OP
			close_op: DIR_CLOSE_OP
			curr_name: STRING
		do
			create dir.make (dir_name)
			create open_op
			open_op.open (dir)

			from
				Result := False
				dir.readentry
			until
				dir.lastentry = Void or Result
			loop
				curr_name := dir.lastentry
				split_name (curr_name)
				if last_name.is_equal (fname) and last_version > 0 then
					Result := True
				end
				dir.readentry
			end

			create close_op
			close_op.close (dir)
		end

	split_name (s: STRING)
			-- Split `s' into file name and version number and
			-- put in `last_name' and `last_version'
		require
			s_not_void: s /= Void
		local
			pos: INTEGER
			version_string: STRING

		do
			pos := s.last_index_of (Version_separator, s.count)
			if pos > 0 then
				version_string := s.substring (pos + 1, s.count)
			else
				version_string := Void
			end
			if version_string /= Void and version_string.is_integer and then version_string.to_integer > 0 then
				last_name := s.substring (1, pos - 1)
				last_version := version_string.to_integer
			else
				last_name := s
				last_version := 0
			end
		end

	computed_version_id (fname: STRING): STRING
			-- Version ID computed for packer box file
			-- named `fname'
		local
			row: PACKING_VERSIONS_ROW
			n: INTEGER
			not_used: BOOLEAN
		do
			create row.make_template
			n := row.rows_with_version_id_prefix (packing_id, fname, connection, expected_count.At_least_zero).count
			if n = 0 then
				Result := fname
			else
				from
				until
					not_used
				loop
					n := n + 1
					Result := fname + " (" + n.out + ")"
					not_used := not packing.is_version_id_used (Result)
				end
			end
		end

	last_name: STRING
			-- File name (without version number) from last
			-- call to `split_name'

	last_version: INTEGER
			-- File version number from last
			-- call to `split_name' (0 if no version number)

	version_separator: CHARACTER = ';'
			-- Character that separates version number from
			-- file name

end
