
indexing
	
	description: "A run closing step operation that sends %
		%the Export Control report to the foundry.  May ask user %
		%whether report has been sent rather than actually sending it."
	author: "David Hollenberg"
	date: "May 4, 2005"

class SEND_FOUNDRY_ECCN_REPORT_OP
inherit
	FOUNDRY_ECCN_REPORT_OP
	NO_EXTERNAL_STATE
	DB_UTILITIES
		export
			{NONE} all
		undefine
			default_create
		end

create
	make

feature -- Properties

	input_data_items: LIST [STRING]
			-- Data items required before step can be executed
		do
			Result := array_as_list (<<
					data_item.Foundry_eccn_report
				 >>)
		end

feature {NONE} -- Implementation

	do_step
			-- Execute step
		do
			if question ("Has ECCN report been reviewed, approved and sent to the foundry?") then
				fetch_eccn_reported_event
				insert_or_update_event
			else
				abort
			end
			call_completion_actions
		end

	undo_step
			-- Undo execution of step
		local
			step_name: PRIMITIVE_STEP_NAME_VALUES
		do
			if not steps.there_exists (agent {RC_PRIMITIVE_STEP}.has_step_name (step_name.Gen_foundry_eccn_report)) then
				session.commit_actions.extend (agent inform ("Undoing this step (" + step.step_name + ")%Ndoes not allow customers to update their ECCN questionnaires.%NYou must undo the step that generates the ECCN report to allow these updates."))
			end
			call_completion_actions
		end

	insert_or_update_event
			-- Insert new ECCN REPORTED event or update
			-- existing row
		local
			e: RUN_EVENTS_MROW
			event_name: EVENT_NAME_VALUES
			lock_op: ROW_LOCK_OP
		do
			if eccn_reported_event = Void then
				create e.make_null (connection)
				e.set_run_id (session.run_id)
				e.set_event_name (event_name.Eccn_reported)
				e.set_comments ("Inserted by Delta")
			elseif event_needs_update then
				e := eccn_reported_event
				create lock_op
				lock_op.lock_nowait (e)
				e.set_comments ("Updated by Delta")
			end
			if e /= Void then
				e.set_date_event (connection.transaction_date)
				e.set_actual_date (True)
				e.set_significant (False)
			end
		end

	event_needs_update: BOOLEAN
			-- Does ECCN REPORTED event need to be updated?
		local
			f: LARGE_PLAIN_TEXT_FILE
			d: DB_DATE
		do
			create f.make (eccn_report_output_file.file_pathname)
			d := db_date_from_unix_time (f.date)
			Result := d >= eccn_reported_event.date_event
		end
end
