
indexing
	
	description: "A run closing step operation that sets run clerk"
	author: "David Hollenberg"
	date: "May 19, 2003"

class SET_CLERK_OP
inherit
	RC_PRIMITIVE_STEP_OP
		redefine
			state
		end
	RC_APPLICABLE_STEP
	NO_INPUT_DATA_ITEMS
	HAS_USER_INTERFACE
	NO_EXTERNAL_STATE

create
	make

feature -- Properties

	state: STRING
			-- State saved by soft undo

feature {NONE} -- Implementation

	do_step
		local
			ifc: SINGLE_CONSTRAINED_VALUE_IFC
			clerks: LIST [M_OPERATORS_ROW]
			clerk: M_OPERATORS_ROW
			curr_clerk, default_choice: STRING
		do
			ifc := factory.single_constrained_value_interface (session.title_with_run_id ("Select Run Clerk"))
			ifc.set_label ("Run Clerk")
			ifc.enable_choices_prepended
			clerks := possible_clerks
			from
				clerks.start
			until
				clerks.after
			loop
				clerk := clerks.item
				ifc.add_choice_and_text (clerk.username, clerk.full_name)
				clerks.forth
			end
			curr_clerk := cache.db_productions_row.clerk
			if not curr_clerk.is_empty then
				default_choice := curr_clerk
			elseif state /= Void and then clerks.there_exists (agent matches_saved_value) then
				default_choice := state
			else
				default_choice := next_default_clerk
				if default_choice = Void then
					default_choice := clerks.first.username
				end
			end
			ifc.set_default_choice (default_choice)
			ifc.build
			ifc.accept_actions.extend (agent ifc.execute_action (agent call_safely_with_string (?, agent set_clerk, agent reset (ifc))))
			ifc.decline_actions.append (abort_actions)
		end

	possible_clerks: LIST [M_OPERATORS_ROW]
			-- Possible run clerks
		local
			template_row: M_OPERATORS_ROW
		do
			create template_row.make_template
			Result := template_row.run_clerk_operators_ordered (connection, expected_count.at_least_one)
		end

	next_default_clerk: STRING
			-- Username of next clerk to use for default, based
			-- on username of last preferred clerk to be
			-- assigned to a run
		local
			opers_row: M_OPERATORS_ROW
			prods_row: PRODUCTIONS_ROW
			prods: LIST [PRODUCTIONS_ROW]
			step_name: PRIMITIVE_STEP_NAME_VALUES
			l_clerk, l_last_clerk: STRING
			pref_clerks: LIST [M_OPERATORS_ROW]
			found: BOOLEAN
		do
			create opers_row.make_template
			pref_clerks := opers_row.preferred_clerk_operators_ordered (connection, expected_count.At_least_zero)
			create prods_row.make_template
			prods := prods_row.productions_for_latest_preferred_clerk (step_name.Set_clerk, connection, expected_count.At_least_zero)
			if pref_clerks.is_empty then
				-- No preferred clerks
				Result := Void
			elseif prods.is_empty then
				 -- No preferred clerks assigned to runs
				Result := pref_clerks.first.username
			else
				l_last_clerk := prods.first.clerk
				from
					pref_clerks.start
				until
					pref_clerks.after or Result /= Void
				loop
					l_clerk := pref_clerks.item.username
					if found then
						Result := l_clerk
					elseif l_clerk.is_equal (l_last_clerk) then
						found := True
					end
					pref_clerks.forth
				end
				if Result = Void then
					Result := pref_clerks.first.username
				end
			end
		end

	matches_saved_value (r: M_OPERATORS_ROW): BOOLEAN
			-- Does `r' have same clerk as one saved in
			-- `state'?
		require
			row_exists: r /= Void
		do
			Result := equal (r.username, state)
		end

	set_clerk (s: STRING)
			-- Set clerk to `s'
		require
			string_exists: s /= Void
			string_not_empty: not s.is_empty
		do
			fetch_and_modify_production (s)
			add_commit_action (agent publish_new_value (s))
			call_completion_actions
		end

	undo_step
			-- Undo execution of step
		do
			fetch_and_modify_production (Void)
			add_commit_action (agent publish_new_value (""))
			call_completion_actions
		end

	set_state
			-- Set `state'
		do
			state := cache.clerk
		end

	fetch_and_modify_production (s: STRING)
			-- Fetch productions row, lock it and set clerk to `s'
		local
			prod: PRODUCTIONS_MROW
		do
			prod := cache.productions_mrow
			session.lock_row (prod)
			prod.set_clerk (s)
		end

	publish_new_value (c: STRING)
			-- Publish new value of clerk
		require
			clerk_exists: c /= Void
		local
			e: RUN_CLERK_CHANGE_EVENT
		do
			create e
			e.set_run_clerk (c)
			e.publish
		end

end
