
indexing
	
	description: "A run closing step operation that sets run closer"
	author: "David Hollenberg"
	date: "May 16, 2003"

class SET_CLOSER_OP
inherit
	RC_PRIMITIVE_STEP_OP
		redefine
			state
		end
	RC_APPLICABLE_STEP
	NO_INPUT_DATA_ITEMS
	NO_USER_INTERFACE
	NO_EXTERNAL_STATE
	STRING_UTILITIES_ACCESS
		export
			{NONE} all
		undefine
			default_create
		end

create
	make

feature -- Properties

	state: STRING
			-- State saved by soft undo

feature {NONE} -- Implementation

	do_step
		local
			closer: STRING
		do
			closer := connection.user_name.as_upper
			confirm ("Set run closer to " + closer)
			fetch_and_modify_run (closer)
			add_commit_action (agent publish_new_value (closer))
			call_completion_actions
		end

	undo_step
			-- Undo execution of step
		do
			fetch_and_modify_run (Void)
			add_commit_action (agent publish_new_value (""))
			call_completion_actions
		end

	set_state
			-- Set `state'
		do
			state := cache.closer
		end

	fetch_and_modify_run (s: STRING)
			-- Fetch runs row, lock it and set closer to `s'
		local
			run: RUNS_MROW
		do
			run := cache.runs_mrow
			session.lock_row (run)
			run.set_closer (s)
		end

	publish_new_value (rc: STRING)
			-- Publish new value of run closer
		require
			rc_exists: rc /= Void
		local
			e: RUN_CLOSER_CHANGE_EVENT
		do
			create e
			e.set_run_closer (rc)
			e.publish
		end

end
