
indexing
	
	description: "A run closing step operation that sets design engineer"
	author: "David Hollenberg"
	date: "March 9, 2005"

class SET_DESIGN_ENGINEER_OP
inherit
	RC_PRIMITIVE_STEP_OP
		redefine
			state
		end
	RC_APPLICABLE_STEP
	NO_INPUT_DATA_ITEMS
	HAS_USER_INTERFACE
	NO_EXTERNAL_STATE

create
	make

feature -- Properties

	state: STRING
			-- State saved by soft undo

feature {NONE} -- Implementation

	do_step
		local
			ifc: SINGLE_CONSTRAINED_VALUE_IFC
			engineers: LIST [M_OPERATORS_ROW]
			engineer: M_OPERATORS_ROW
			curr_engineer, default_choice: STRING
		do
			ifc := factory.single_constrained_value_interface (session.title_with_run_id ("Select Design Engineer"))
			ifc.set_label ("Design Engineer")
			ifc.enable_choices_prepended
			ifc.add_choice_and_text ("", "")
			engineers := possible_engineers
			from
				engineers.start
			until
				engineers.after
			loop
				engineer := engineers.item
				ifc.add_choice_and_text (engineer.username, engineer.full_name)
				engineers.forth
			end
			curr_engineer := cache.db_runs_row.design_engineer
			if not curr_engineer.is_empty then
				default_choice := curr_engineer
			elseif state /= Void and then engineers.there_exists (agent matches_saved_value) then
				default_choice := state
			else
				default_choice := ""
			end
			ifc.set_default_choice (default_choice)
			ifc.build
			ifc.accept_actions.extend (agent ifc.execute_action (agent call_safely_with_string (?, agent set_design_engineer, agent reset (ifc))))
			ifc.decline_actions.append (abort_actions)
		end

	possible_engineers: LIST [M_OPERATORS_ROW]
			-- Possible design engineers
		local
			template_row: M_OPERATORS_ROW
		do
			create template_row.make_template
			Result := template_row.design_engineer_operators_ordered (connection, expected_count.at_least_one)
		end

	matches_saved_value (r: M_OPERATORS_ROW): BOOLEAN
			-- Does `r' have same design engineer as one saved in
			-- `state'?
		require
			row_exists: r /= Void
		do
			Result := equal (r.username, state)
		end

	set_design_engineer (s: STRING)
			-- Set design engineer to `s'
		require
			string_exists: s /= Void
		do
			fetch_and_modify_run (s)
			add_commit_action (agent publish_new_value (s))
			call_completion_actions
		end

	undo_step
			-- Undo execution of step
		do
			fetch_and_modify_run (Void)
			add_commit_action (agent publish_new_value (""))
			call_completion_actions
		end

	set_state
			-- Set `state'
		do
			state := cache.design_engineer
		end

	fetch_and_modify_run (s: STRING)
			-- Fetch runs row, lock it and set design engineer
			-- to `s'
		local
			run: RUNS_MROW
		do
			run := cache.runs_mrow
			session.lock_row (run)
			run.set_design_engineer (s)
		end

	publish_new_value (s: STRING)
			-- Publish new value of design engineer
		require
			engineer_exists: s /= Void
		local
			e: DESIGN_ENGINEER_CHANGE_EVENT
		do
			create e
			e.set_design_engineer (s)
			e.publish
		end

end
