
indexing
	
	description: "A run closing step operation that sets designated %
		%design technology (has the most design area)"
	author: "David Hollenberg"
	date: "July 13, 2005"

class SET_DESIGNATED_DESIGN_TECHN_OP
inherit
	RC_PRIMITIVE_STEP_OP
	NO_USER_INTERFACE
	NO_EXTERNAL_STATE

create
	make

feature -- Properties

	input_data_items: LIST [STRING]
			-- Data items required before step can be executed
		do
			Result := array_as_list (<<
				data_item.Assigned_die_ids
				>>)
		end

feature {NONE} -- Implementation

	is_applicable: BOOLEAN
			-- Is step applicable to run for session?
		do
			Result := True
		end

	do_step
			-- Execute of step
		local
			run: RUNS_MROW
		do
			select_designated_dt
			run := cache.runs_mrow
			run.set_designated_design_techn (designated_dt)
			add_commit_action (agent publish_new_value)
			call_completion_actions
		end

	undo_step
			-- Undo execution of step
		local
			run: RUNS_MROW
		do
			run := cache.runs_mrow
			run.set_null_designated_design_techn
			designated_dt := ""
			add_commit_action (agent publish_new_value)
			call_completion_actions
		end

	select_designated_dt
			-- Select designated design technology
			-- and put in `designated_dt'
		local
			all_dies, list: LIST [DIES_ROW]
			wt: WAFER_TECHNS_ROW
			table: HASH_TABLE [DOUBLE, STRING]
		do
			all_dies := cache.die_rows
			list := all_dies.twin
			list.wipe_out
			all_dies.do_if (agent list.extend, agent {DIES_ROW}.selects_designated_dt)
			if list.is_empty then
				-- No dies that help select designated design
				-- techn, so just use scribe design techn
				wt := cache.wafer_techns_row
				designated_dt := wt.non_null_design_techn_scribe
			else
				maximum_area := 0.0
				create table.make (list.count)
				list.do_all (agent add_area (?, table))
			end
		end

	add_area (r: DIES_ROW table: HASH_TABLE [DOUBLE, STRING])
			-- Add area of layout for `r' to entry in
			-- `table' for its wafer design techn.
			
		require
			r_not_void: r /= Void
			table_not_void: table /= Void
		local
			area: DOUBLE
			dt: STRING
			attr: DIE_ATTRIBUTES_ROW
		do
			area := r.ic_fab_instance.cust_ic_design.layout_area
			attr := r.attributes
			dt := attr.non_null_wafer_design_techn
			table.search (dt)
			if table.found then
				area := area + table.found_item
			end
			table.force (area, dt)
			if area > maximum_area then
				maximum_area := area
				designated_dt := dt
			end
		end

	set_state
			-- Set `state'
		do
			state := Void
		end

	publish_new_value
			-- Publish new designated design techn
		local
			event: DESIGNATED_DT_CHANGE_EVENT
		do
			create event
			event.set_designated_design_techn (designated_dt)
			event.publish
		end
	
	maximum_area: DOUBLE
			-- Area of design techn with most
			-- area so far

	designated_dt: STRING
			-- Designated design techn

end
