
indexing
	
	description: "A run closing step operation that sets overhead parameters%
		%(die labeling and white space border on non-overhead strip sides)"
	author: "David Hollenberg"
	date: "September 24, 2007"

class SET_DIE_OVERHEAD_PARAMETERS_OP
inherit
	RC_PRIMITIVE_STEP_OP
		redefine
			state
		end
	RC_APPLICABLE_STEP
	HAS_USER_INTERFACE
	NO_EXTERNAL_STATE
	BOOLEAN_STRINGS
	NUMBER_UTILITIES_ACCESS
		export
			{NONE} all
		undefine
			default_create
		end

create
	make

feature -- Properties

	input_data_items: LIST [STRING]
			-- Data items required before step can be executed
		do
			Result := array_as_list (<<
				data_item.Run_type,
				data_item.Wafer_techn
				 >>)
		end

	state: DIE_OVERHEAD_PARAMETERS_STATE
			-- State saved by soft undo

feature {NONE} -- Implementation

	Border_threshold: DOUBLE = 5.0
			-- Threshold below which user gets warning

	do_step
		local
			ifc: RC_DIE_OVERHEAD_PARAMETERS_IFC
		do
			ifc := factory.die_overhead_parameters_interface (session.run_id, session.title_with_run_id ("Set Die Overhead Parameters"))
			ifc.set_session (session)
			ifc.set_state (state)
			ifc.success_actions.extend (agent call_completion_actions)
			ifc.abort_actions.append (abort_actions)
			ifc.build
		end

	undo_step
			-- Undo execution of step
		do
			fetch_and_modify_run
			add_commit_action (agent publish_new_other_border_value (""))
			add_commit_action (agent publish_new_overhead_style_value (""))
			add_commit_action (agent cache.uncache_run)
			call_completion_actions
		end

	set_state
			-- Set `state'
		local
			run: RUNS_ROW
		do
			run := cache.runs_row
			create state
			state.set_overhead_style (run.overhead_style)
			state.set_other_border (run.other_border)
		end

	half_grid_resolution: DOUBLE
			-- Half of grid resolution for wafer techn
		do
			Result := cache.wafer_techns_row.packing_grid_resolution / 2.0
		end

	border_scale: INTEGER
			-- Scale to use for border
		do
			Result := {WAFER_TECHNS_ROW}.scale_top_border
		end

	fetch_and_modify_run
			-- Fetch runs row, lock it and set overhead style and
			-- other border to null
		local
			run: RUNS_MROW
		do
			run := cache.runs_mrow
			session.lock_row (run)
			run.set_null_uniform_override_border
			run.set_null_other_border
			run.set_null_overhead_style
		end

	publish_new_other_border_value (s: STRING)
			-- Publish new value of other border
		require
			value_exists: s /= Void
		local
			e: OTHER_BORDER_CHANGE_EVENT
		do
			create e
			e.set_other_border (s)
			e.publish
		end

	publish_new_overhead_style_value (s: STRING)
			-- Publish new overhead style value
		require
			value_exists: s /= Void
		local
			e: OVERHEAD_STYLE_CHANGE_EVENT
		do
			create e
			e.set_overhead_style (s)
			e.publish
		end

end
