
indexing
	
	description: "A run closing step operation that sets whether run %
		%is an educational run (with donated wafers)"
	author: "David Hollenberg"
	date: "May 13, 2003"

class SET_EDUCATIONAL_STATUS_OP
inherit
	RC_PRIMITIVE_STEP_OP
		redefine
			state
		end
	RC_APPLICABLE_STEP
	HAS_USER_INTERFACE
	NO_EXTERNAL_STATE
	BOOLEAN_STRINGS

create
	make

feature -- Properties

	input_data_items: LIST [STRING]
			-- Data items required before step can be executed
		do
			Result := array_as_list (<<
				data_item.Run_type
				 >>)
		end

	state: BOOLEAN_REF
			-- State saved by soft undo

feature {NONE} -- Implementation

	do_step
		local
			ifc: SINGLE_CONSTRAINED_VALUE_IFC
			default_choice: STRING
			curr_edu: BOOLEAN
		do
			if cache.is_scheduled_run then
				ifc := factory.single_constrained_value_interface (session.title_with_run_id ("Select Educational Status"))
				ifc.set_label ("Is this an educational run (donated wafer run)?")
				ifc.add_choice_and_text (Yes, Yes)
				ifc.add_choice_and_text (No, No)
				curr_edu := cache.db_runs_row.educational
				if curr_edu then
					default_choice := Yes
				elseif state /= Void then
					if state.item then
						default_choice := Yes
					else
						default_choice := No
					end
				else
					default_choice := No
				end
				ifc.set_default_choice (default_choice)
				ifc.build
				ifc.accept_actions.extend (agent ifc.execute_action (agent call_safely_with_string (?, agent set_educational_status, agent reset(ifc))))
				ifc.decline_actions.append (abort_actions)
			else
				set_educational_status (No)
			end
		end

	set_educational_status (s: STRING)
			-- Set educational status to `s'
		require
			string_exists: s /= Void
			string_yes_or_no: is_valid_boolean_string (s)
		do
			fetch_and_modify_run (as_boolean (s))
			add_commit_action (agent publish_new_value (s))
			call_completion_actions
		end

	undo_step
			-- Undo execution of step
		do
			fetch_and_modify_run (False)
			add_commit_action (agent publish_new_value (""))
			call_completion_actions
		end

	set_state
			-- Set `state'
		do
			state := cache.is_educational_run
		end

	fetch_and_modify_run (b: BOOLEAN)
			-- Fetch runs row, lock it and set education status
			-- to `b'
		local
			run: RUNS_MROW
		do
			run := cache.runs_mrow
			session.lock_row (run)
			run.set_educational (b)
		end

	publish_new_value (s: STRING)
			-- Publish new value of educational status
		require
			status_exists: s /= Void
		local
			e: EDUCATIONAL_STATUS_CHANGE_EVENT
		do
			create e
			e.set_is_educational (s)
			e.publish
		end

end
