
indexing
	
	description: "A run closing step operation that sets field set"
	author: "David Hollenberg"
	date: "May 14, 2003"

class SET_FIELD_SET_OP
inherit
	RC_PRIMITIVE_STEP_OP
		redefine
			state
		end
	RC_APPLICABLE_STEP
	HAS_USER_INTERFACE
	NO_EXTERNAL_STATE

create
	make

feature -- Properties

	input_data_items: LIST [STRING]
			-- Data items required before step can be executed
		do
			Result := array_as_list (<<
				data_item.Wafer_techn
				 >>)
		end

	state: STRING
			-- State saved by soft undo

feature {NONE} -- Implementation

	do_step
		local
			ifc: FIELD_SET_IFC
		do
			ifc := factory.field_set_interface (session.run_id, session.title_with_run_id ("Select Field Set"))
			ifc.set_session (session)
			ifc.set_state (state)
			ifc.success_actions.extend (agent finish_step (ifc))
			ifc.abort_actions.append (abort_actions)
			ifc.build
		end

	finish_step (ifc: FIELD_SET_IFC)
			-- Finish execution of step after
			-- user selects field set
		local
			fs_name: STRING
		do
			fs_name := ifc.field_set_name
			fetch_and_modify_run (fs_name)
			add_commit_action (agent cache.uncache_field_set)
			add_commit_action (agent publish_new_value (fs_name))
			call_completion_actions
		end

	undo_step
			-- Undo execution of step
		do
			fetch_and_modify_run (Void)
			add_commit_action (agent cache.uncache_field_set)
			add_commit_action (agent publish_new_value (""))
			call_completion_actions
		end

	set_state
			-- Set `state'
		do
			state := cache.field_set_name
		end

	fetch_and_modify_run (s: STRING)
			-- Fetch runs row, lock it and set field set to `s'
		local
			run: RUNS_MROW
		do
			run := cache.runs_mrow
			session.lock_row (run)
			run.set_field_set_name (s)
		end

	publish_new_value (fs: STRING)
			-- Publish new value of field set
		require
			fs_exists: fs /= Void
		local
			e: FIELD_SET_CHANGE_EVENT
		do
			create e
			e.set_field_set (fs)
			e.publish
		end

end
