
indexing
	
	description: "A run closing step operation that sets mask %
		%part id "
	author: "David Hollenberg"
	date: "June 3, 2003"

class SET_MASK_PART_ID_OP
inherit
	SET_PART_ID_OP

create
	make

feature -- Properties

	input_data_items: LIST [STRING]
			-- Data items required before step can be executed
		do
			Result := array_as_list (<<
				data_item.Mask_techn
				 >>)
		end

feature {NONE} -- Implementation

	is_applicable: BOOLEAN
			-- Is step applicable to run for session?
		do
			Result := not cache.mask_techns_row.is_null_vendor_part_sequence_id
		end

	do_step
		do
			sequence_name := cache.mask_techns_row.vendor_part_sequence_id
			fetch_next_part_id
			set_part_id
			if part_id_row /= Void then
				set_part_id
			else
				report_out_of_parts
			end
		end

	set_part_id
			-- Mark part id assigned and update RUNS table
		do
			session.lock_row (part_id_row)
			part_id_row.set_assigned (True)
			set_mask_part_id (part_id_row.vendor_part_id_source)
			call_completion_actions
		end

	set_mask_part_id (s: STRING)
			-- Set mask part id source to `s'
		require
			string_exists: s /= Void
			string_not_empty: not s.is_empty
		do
			fetch_and_modify_run (s)
			add_commit_action (agent publish_new_value (s))
		end

	undo_step
			-- Undo execution of step
		do
			fetch_and_modify_run (Void)
			add_commit_action (agent publish_new_value (""))
			call_completion_actions
		end

	fetch_and_modify_run (s: STRING)
			-- Fetch runs row, lock it and set mask part id
			-- to `s'
		local
			run: RUNS_MROW
		do
			run := cache.runs_mrow
			session.lock_row (run)
			run.set_mask_part_id (s)
		end


	publish_new_value (s: STRING)
			-- Publish new value of mask part id
		local
			e: MASK_PART_ID_CHANGE_EVENT
		do
			create e
			e.set_mask_part_id (s)
			e.publish
		end

end
