
indexing
	
	description: "A run closing step operation that sets mask %
		%technology"
	author: "David Hollenberg"
	date: "May 14, 2003"

class SET_MASK_TECHN_OP
inherit
	RC_PRIMITIVE_STEP_OP
		redefine
			state
		end
	RC_APPLICABLE_STEP
	HAS_USER_INTERFACE
	NO_EXTERNAL_STATE

create
	make

feature -- Properties

	input_data_items: LIST [STRING]
			-- Data items required before step can be executed
		do
			Result := array_as_list (<<
				data_item.Wafer_techn
				 >>)
		end

	state: STRING
			-- State saved by soft undo

feature {NONE} -- Implementation

	do_step
		local
			ifc: SINGLE_CONSTRAINED_VALUE_IFC
			mask_techns: LIST [MASK_TECHNS_EROW]
			mask_techn: MASK_TECHNS_EROW
			default_choice: STRING
		do
			ifc := factory.single_constrained_value_interface (session.title_with_run_id ("Select Mask Technology"))
			ifc.set_label ("Mask Technology")
			ifc.enable_choices_prepended
			mask_techns := possible_mask_techns
			from
				mask_techns.start
			until
				mask_techns.after
			loop
				mask_techn := mask_techns.item
				ifc.add_choice_and_text (mask_techn.mask_techn, mask_techn.descr)
				mask_techns.forth
			end
			if state /= Void and then mask_techns.there_exists (agent matches_saved_value) then
				default_choice := state
			else
				default_choice := mask_techns.first.mask_techn
			end
			ifc.set_default_choice (default_choice)
			ifc.build
			ifc.accept_actions.extend (agent ifc.execute_action (agent call_safely_with_string (?, agent set_mask_techn, agent reset (ifc))))
			ifc.decline_actions.append (abort_actions)
		end

	possible_mask_techns: LIST [MASK_TECHNS_EROW]
			-- Possible mask techns for this run
		local
			template_row: MASK_TECHNS_EROW
		do
			create template_row.make_template
			Result := template_row.available_mask_techns_ordered (cache.wafer_techn, connection, expected_count.at_least_zero)
			if Result.is_empty then
				handle_soft_error ("no available mask techns for wafer techn " + cache.wafer_techn)
				Result := template_row.available_mask_techns_ordered (cache.wafer_techn, connection, expected_count.at_least_one)
			end
		end

	matches_saved_value (r: MASK_TECHNS_EROW): BOOLEAN
			-- Does `r' have same mask techn as one saved in
			-- `state'?
		require
			row_exists: r /= Void
		do
			Result := equal (r.mask_techn, state)
		end

	set_mask_techn (s: STRING)
			-- Set mask techn to `s'
		require
			string_exists: s /= Void
			string_not_empty: not s.is_empty
		do
			fetch_and_modify_run (s)
			add_commit_action (agent publish_new_value (s))
			call_completion_actions
		end

	undo_step
			-- Undo execution of step
		do
			fetch_and_modify_run (Void)
			add_commit_action (agent cache.uncache_mask_techn)
			add_commit_action (agent publish_new_value (""))
			call_completion_actions
		end

	set_state
			-- Set `state'
		do
			state := cache.mask_techn
		end

	fetch_and_modify_run (s: STRING)
			-- Fetch mask sets row, lock it and set mask
			-- techn to `s'
		local
			mask_set: MASK_SETS_MROW
		do
			mask_set := cache.mask_sets_mrow
			session.lock_row (mask_set)
			mask_set.set_mask_techn (s)
		end

	publish_new_value (mt: STRING)
			-- Publish new value of mask techn
		require
			mt_exists: mt /= Void
		local
			e: MASK_TECHN_CHANGE_EVENT
		do
			create e
			e.set_mask_techn (mt)
			e.publish
		end

end
