
indexing
	
	description: "A run closing step operation that sets run id used %
		%in overhead placed around each project"
	author: "David Hollenberg"
	date: "May 13, 2003"

class SET_OVERHEAD_RUN_ID_OP
inherit
	RC_PRIMITIVE_STEP_OP
		redefine
			state
		end
	RC_APPLICABLE_STEP
	HAS_USER_INTERFACE
	NO_INPUT_DATA_ITEMS
	NO_EXTERNAL_STATE
	STRING_UTILITIES_ACCESS
		export
			{NONE} all
		undefine
			default_create
		end

create
	make

feature -- Properties

	state: STRING
			-- State saved by soft undo

feature {NONE} -- Implementation

	do_step
		do
			set_overhead_run_id (session.run_id)
		end

	select_overhead_run_id
		local
			ifc: SINGLE_FREE_VALUE_IFC
		do
			ifc := factory.single_free_value_interface (session.title_with_run_id ("Select Overhead Run ID"))
			ifc.set_label ("Overhead Run ID")
			ifc.set_default_value (session.run_id)
			ifc.enable_uppercase_only
			if state /= Void then
				ifc.set_default_value (state)
			end
			ifc.build
			
			ifc.accept_actions.extend (agent ifc.execute_action (agent call_safely_with_string (?, agent set_overhead_run_id, agent reset (ifc))))
			ifc.decline_actions.append (abort_actions)
		end

	set_overhead_run_id (s: STRING)
			-- Set overhead run id to `overhead_run_id'
		require
			overhead_run_id_exists: s /= Void
		do
			overhead_run_id := s
			check_short_enough
			fetch_and_modify_run (overhead_run_id)
			add_commit_action (agent publish_new_value (overhead_run_id))
			call_completion_actions
		end

	check_short_enough
			-- Issue error if `overhead_run_id' won't fit in column
			-- in RUNS table
		require
			overhead_run_id_exists: overhead_run_id /= Void
		local
			len, max_len: INTEGER
		do
			len := overhead_run_id.count
			max_len := {RUNS_ROW}.Maxlen_run_id_for_overhead
			if len > max_len then
				handle_hard_error ("Overhead run id length (" + len.out + ") exceeds maximum length (" + max_len.out + ")")
			end
		end

	undo_step
			-- Undo execution of step
		do
			fetch_and_modify_run (Void)
			add_commit_action (agent publish_new_value (""))
			call_completion_actions
		end

	set_state
			-- Set `state'
		do
			state := cache.run_id_for_overhead
		end

	fetch_and_modify_run (s: STRING)
			-- Fetch runs row, lock it and set overhead run id
			-- to `s'
		local
			run: RUNS_MROW
		do
			run := cache.runs_mrow
			session.lock_row (run)
			run.set_run_id_for_overhead (s)
		end

	overhead_run_id: STRING
			-- Last overhead run id entered by user
	
	publish_new_value (s: STRING)
			-- Publish new value of overhead run id
		local
			e: OVERHEAD_RUN_ID_CHANGE_EVENT
		do
			create e
			e.set_overhead_run_id (s)
			e.publish
		end
	
end
