
indexing
	
	description: "A run closing step operation that sets a part id"
	author: "David Hollenberg"
	date: "June 18, 2003"

deferred class SET_PART_ID_OP
inherit
	RC_PRIMITIVE_STEP_OP
		redefine
			do_inapplicable_step, undo_inapplicable_step
		end
	NO_USER_INTERFACE
	NO_EXTERNAL_STATE
	VENDOR_PART_ID_QUERIES
		export
			{NONE} all
		end

feature -- Operations

	do_inapplicable_step
			-- Take actions needed to do step that is
			-- not applicable
		do
			add_commit_action (agent publish_new_value (""))
		end

	undo_inapplicable_step
			-- Take actions needed to undo step that is
			-- not applicable
		do
			add_commit_action (agent publish_new_value (""))
		end

feature {NONE} -- Implementation

	fetch_part_sequence
			-- Fetch row for sequence from which part id
			-- should be obtained
		local
			template_row: VENDOR_PART_SEQUENCE_IDS_ROW
		do
			create template_row.make_template
			sequence_row := template_row.vendor_part_sequence_ids_rows (sequence_name, connection, expected_count.exactly_one).first
		end

	fetch_next_part_id
			-- Fetch next part id from `sequence_name' and put
			-- row in `part_id_row'.  Set to Void if no
			-- more parts in sequence
		require
			sequence_name_exists: sequence_name /= Void
		local
			list: LIST [VENDOR_PART_IDS_MROW]
			template_row: VENDOR_PART_IDS_MROW
		do
			create template_row.make_template
			list := template_row.next_vendor_part_ids (sequence_name, connection, expected_count.at_most_one)
			if not list.is_empty then
				part_id_row := list.first
			else
				part_id_row := Void
			end
		end

	set_state
			-- Set `state'
		do
		end

	report_out_of_parts
			-- Tell user we're out of part ids
		local
			row: VENDOR_PART_IDS_ROW
		do
			create row.make_template
			handle_hard_error ("No more parts in table " + row.table_name + "%Nfor sequence " + sequence_name)
		end

	warn_if_low_on_parts (seq: STRING)
			-- Warn user if number of parts left in `seq'
			-- is below threshold
		require
			sequence_exists: seq /= Void
		local
			remaining: INTEGER
			template_row: VENDOR_PART_SEQUENCE_IDS_ROW
		do
			create template_row.make_template
			remaining := template_row.number_of_parts_left (seq, connection)
			if remaining >= 0 then
				inform ("Sequence " + seq + " has " + remaining.out + " unused part ids left")
			end
		end

	publish_new_value (s: STRING)
			-- Publish new value of part id
		require
			value_exists: s /= Void
		deferred
		end

	sequence_row: VENDOR_PART_SEQUENCE_IDS_ROW
			-- Row for sequence to be used to select part id
	
	sequence_name: STRING
			-- Name of sequence to be used to select part id
	
	part_id_row: VENDOR_PART_IDS_MROW
			-- Row with part id to be assigned
	
end
