
indexing
	
	description: "A run closing step operation that sets run close date"
	author: "David Hollenberg"
	date: "May 14, 2003"

class SET_RUN_CLOSE_DATE_OP
inherit
	RC_PRIMITIVE_STEP_OP
		redefine
			state
		end
	RC_APPLICABLE_STEP
	NO_INPUT_DATA_ITEMS
	HAS_USER_INTERFACE
	NO_EXTERNAL_STATE
	DB_OPTIONS_ACCESS
		export
			{NONE} all
		undefine
			default_create
		end
	STRING_UTILITIES_ACCESS
		export
			{NONE} all
		undefine
			default_create
		end
	DATE_UTILITIES
		export
			{NONE} all
		undefine
			default_create
		end

create
	make

feature -- Properties

	state: DB_DATE
			-- State saved by soft undo

feature {NONE} -- Implementation

	do_step
		local
			ifc: SINGLE_FREE_VALUE_IFC
			init_date: DB_DATE
		do
			ifc := factory.single_free_value_interface (session.title_with_run_id ("Select Run Close Date"))
			ifc.set_label ("Run Close Date")
			ifc.enable_uppercase_only
			ifc.set_minimum_width_in_chars (11)
			if state /= Void then
				init_date := state
			else
				init_date := connection.system_date
			end
			ifc.set_default_value (init_date.to_date_string)
			ifc.build
			ifc.accept_actions.extend (agent ifc.execute_action (agent call_safely_with_string (?, agent set_close_date, agent reset (ifc))))
			ifc.decline_actions.append (abort_actions)
		end

	set_close_date (s: STRING)
			-- Set close date to `close_date'
		require
			close_date_exists: s /= Void
		do
			close_date := s
			check_date_valid (close_date)
			check_date_in_range (close_date, connection, Past_interval, Future_interval)
			insert_run_events_row
			add_commit_action (agent publish_new_value (close_date))
			call_completion_actions
		end

	check_date_valid (date: STRING)
			-- Check that `date' is a valid date and issue
			-- error if not
		require
			date_exists: date /= Void
		do
			if not valid_date (date) then
				handle_user_error ("Date %"" + date + "%" is not valid.%NPlease enter a date in format DD-MON-YYYY")
			end
		end

	undo_step
			-- Undo execution of step
		local
			run_events_row: RUN_EVENTS_MROW
		do
			create run_events_row.make_template
			run_events_row := run_events_row.run_closed_events (session.run_id, connection, expected_count.Exactly_one).first
			session.lock_row (run_events_row)
			run_events_row.delete
			add_commit_action (agent publish_new_value (""))
			call_completion_actions
		end

	set_state
			-- Set `state'
		local
			run_events_row: RUN_EVENTS_MROW
		do
			create run_events_row.make_template
			run_events_row := run_events_row.run_closed_events (session.run_id, connection, expected_count.Exactly_one).first
			state := run_events_row.date_event
		end

	insert_run_events_row
			-- Insert row for "run closed" event
		local
			event_names: EVENT_NAME_VALUES
			run_events_row: RUN_EVENTS_MROW
		do
			create run_events_row.make_null (connection)
			run_events_row.set_run_id (session.run_id)
			run_events_row.set_event_name (event_names.Run_closed)
			run_events_row.set_date_event (connection.date (close_date))
			run_events_row.set_significant (True)
			run_events_row.set_actual_date (True)
		end

	close_date: STRING
			-- Close date entered by user
	
	publish_new_value (s: STRING)
			-- Publish new value of close date
		require
			date_exists: s /= Void
		local
			e: CLOSE_DATE_CHANGE_EVENT
		do
			create e
			e.set_close_date (s)
			e.publish
		end

	Past_interval: INTEGER = -14
			-- Issue warning if entered run close date is more
			-- than this many days before today's date

	Future_interval: INTEGER = 0
			-- Issue warning if entered run close date is more
			-- than this many days after today's date

end
