
indexing
	
	description: "A run closing step operation that sets run type"
	author: "David Hollenberg"
	date: "May 9, 2003"

class SET_RUN_TYPE_OP
inherit
	RC_PRIMITIVE_STEP_OP
		redefine
			state
		end
	RC_APPLICABLE_STEP
	NO_INPUT_DATA_ITEMS
	HAS_USER_INTERFACE
	NO_EXTERNAL_STATE

create
	make

feature -- Properties

	state: STRING
			-- State saved by soft undo

feature {NONE} -- Implementation

	do_step
		local
			ifc: SINGLE_CONSTRAINED_VALUE_IFC
			run_types: LIST [RUN_TYPES_ROW]
			run_type, template_row: RUN_TYPES_ROW
			curr_run_type, default_choice: STRING
			type: RUN_TYPE_VALUES
		do
			ifc := factory.single_constrained_value_interface (session.title_with_run_id ("Select Run Type"))
			ifc.set_label ("Run Type")
			create template_row.make_template
			run_types := template_row.all_run_types_ordered (connection, expected_count.at_least_one)
			from
				run_types.start
			until
				run_types.after
			loop
				run_type := run_types.item
				if run_type.run_type.is_equal (type.Skd) then
					default_choice := run_type.run_type
				end
				ifc.add_choice_and_text (run_type.run_type, capitalized (run_type.run_type_word))
				run_types.forth
			end
			curr_run_type := cache.db_runs_row.run_type
			if not curr_run_type.is_empty then
				default_choice := curr_run_type
			elseif state /= Void and then run_types.there_exists (agent matches_saved_value) then
				default_choice := state
			elseif default_choice = Void then
				default_choice := run_types.last.run_type
			end
			ifc.set_default_choice (default_choice)
			ifc.build
			ifc.accept_actions.extend (agent ifc.execute_action (agent call_safely_with_string (?, agent set_run_type, agent reset (ifc))))
			ifc.decline_actions.append (abort_actions)
		end

	capitalized (s: STRING): STRING
			-- New string with first character converted to
			-- upper case
		require
			string_exists: s /= Void
		do
			if s.is_empty then
				Result := s.twin
			else
				create Result.make (s.count)
				Result.extend (s.item (1).as_upper)
				Result.append (s.substring (2, s.count))
			end
		end

	matches_saved_value (r: RUN_TYPES_ROW): BOOLEAN
			-- Does `r' have same run type as one saved in
			-- `state'?
		require
			row_exists: r /= Void
		do
			Result := equal (r.run_type, state)
		end

	set_run_type (s: STRING)
			-- Set run type to `s'
		require
			string_exists: s /= Void
			string_not_empty: not s.is_empty
		do
			fetch_and_modify_run (s)
			add_commit_action (agent publish_new_value (s))
			call_completion_actions
		end

	undo_step
			-- Undo execution of step
		do
			fetch_and_modify_run (Void)
			add_commit_action (agent publish_new_value (""))
			call_completion_actions
		end

	set_state
			-- Set `state'
		do
			state := cache.run_type
		end

	fetch_and_modify_run (s: STRING)
			-- Fetch runs row, lock it and set run type to `s'
		local
			run: RUNS_MROW
			type: RUN_TYPE_VALUES
		do
			run := cache.runs_mrow
			session.lock_row (run)
			run.set_run_type (s)
			if s = Void or else not s.is_equal (type.Skd) then
				run.set_null_sched_run_no
			end
		end

	publish_new_value (rt: STRING)
			-- Publish new value of run type
		require
			rt_exists: rt /= Void
		local
			e: RUN_TYPE_CHANGE_EVENT
		do
			create e
			e.set_run_type (rt)
			e.publish
		end

end
