
indexing
	
	description: "A run closing step operation that sets scheduled %
		%run number for a scheduled run"
	author: "David Hollenberg"
	date: "May 14, 2003"

class SET_SCHED_RUN_NO_OP
inherit
	RC_PRIMITIVE_STEP_OP
		redefine
			state
		end
	HAS_USER_INTERFACE
	NO_EXTERNAL_STATE
	STRING_UTILITIES_ACCESS
		export
			{NONE} all
		undefine
			default_create
		end

create
	make

feature -- Properties

	input_data_items: LIST [STRING]
			-- Data items required before step can be executed
		do
			Result := array_as_list (<<
				data_item.Run_type,
				data_item.Is_educational,
				data_item.Wafer_techn
				 >>)
		end

	state: INTEGER_REF
			-- State saved by soft undo

feature {NONE} -- Implementation

	is_applicable: BOOLEAN
			-- Is step applicable to run for session?
		do
			Result := cache.is_scheduled_run
		end

	do_step
		local
			ifc: SINGLE_CONSTRAINED_VALUE_IFC
			sched_runs: LIST [SCHEDULED_RUNS_EROW]
			sched_run: SCHEDULED_RUNS_EROW
			choice, default_choice, descr, prev_runs: STRING
		do
			ifc := factory.single_constrained_value_interface (session.title_with_run_id ("Select Scheduled Run Date"))
			ifc.set_label ("Scheduled Run Date")
			sched_runs := possible_sched_runs
			from
				sched_runs.start
			until
				sched_runs.after
			loop
				sched_run := sched_runs.item
				choice := sched_run.sched_run_no.out
				descr := sched_run.date_scheduled.to_date_string
				prev_runs := assigned_runs (sched_run.sched_run_no)
				if not prev_runs.is_empty then
					descr := descr + Spacer + prev_runs
				end
				if not ifc.has_choice (choice) then
					ifc.add_choice_and_text (choice, descr)
				end
				sched_runs.forth
			end
			ifc.add_choice_and_text ("", "None")
				-- For case where user doesn't want to
				-- associate any scheduled run with this run
			
			if not cache.runs_row.is_null_sched_run_no then
				-- Override any saved state from previous
				-- undo with value in database
				state := Void
				set_state
			end
			if state /= Void and then sched_runs.there_exists (agent matches_saved_value) then
				default_choice := state.out
			elseif not sched_runs.is_empty then
				default_choice := sched_runs.first.sched_run_no.out
			else
				default_choice := ""
			end
			ifc.set_default_choice (default_choice)
			ifc.build
			ifc.accept_actions.extend (agent ifc.execute_action (agent call_safely_with_string (?, agent set_sched_run, agent reset (ifc))))
			ifc.decline_actions.append (abort_actions)
		end

	possible_sched_runs: LIST [SCHEDULED_RUNS_EROW]
			-- Possible scheduled runs for this run
		local
			template_row: SCHEDULED_RUNS_EROW
		do
			create template_row.make_template
			Result := template_row.possible_scheduled_runs_ordered (cache.wafer_techn, cache.is_educational_run, 180, 30, connection, expected_count.at_least_zero)
		end

	assigned_runs (n: INTEGER): STRING
			-- Space-separated list of runs assigned to
			-- scheduled run number `n'
		local
			template_row: RUNS_ROW
			list: LIST [RUNS_ROW]
			strings: ARRAYED_LIST [STRING]
		do
			create template_row.make_template
			list := template_row.runs_assigned_to_sched_run (n, session.is_trial_run, connection, expected_count.at_least_zero)
			create strings.make (list.count)
			from
				list.start
			until
				list.after
			loop
				strings.extend (list.item.run_id)
				list.forth
			end
			Result := su.joined_by (strings, ", ")
		ensure
			result_exists: Result /= Void
		end

	matches_saved_value (r: SCHEDULED_RUNS_EROW): BOOLEAN
			-- Does `r' have same scheduled run number as
			-- one saved in `state'?
		require
			row_exists: r /= Void
		do
			Result := equal (r.sched_run_no, state.item)
		end

	set_sched_run (s: STRING)
			-- Set scheduled run number to `s'
		require
			string_exists: s /= Void
			string_valid: s.is_integer or s.is_empty
		local
			sr_no: STRING
		do
			if s.is_empty then
				sr_no := Void
			else
				sr_no := s
			end
			fetch_and_modify_run (sr_no)
			add_commit_action (agent publish_new_value (s))
			call_completion_actions
		end

	undo_step
			-- Undo execution of step
		do
			fetch_and_modify_run (Void)
			add_commit_action (agent publish_new_value (""))
			call_completion_actions
		end

	set_state
			-- Set `state'
		do
			state := cache.runs_row.sched_run_no
		end

	fetch_and_modify_run (s: STRING)
			-- Fetch runs row, lock it and set scheduled run
			-- number to `s'
		require
			integer_if_nonvoid: s /= Void implies s.is_integer
		local
			run: RUNS_MROW
		do
			run := cache.runs_mrow
			session.lock_row (run)
			if s /= Void then
				run.set_sched_run_no (s.to_integer)
			else
				run.set_null_sched_run_no
			end
		end


	Spacer: STRING = "      "

	publish_new_value (s: STRING)
			-- Publish new value of scheduled run number
		require
			value_exists: s /= Void
		local
			e: SCHED_RUN_NO_CHANGE_EVENT
		do
			create e
			e.set_sched_run_no (s)
			e.publish
		end

end
