
indexing
	
	description: "A run closing step operation that sets whether run %
		%is a taxi run"
	author: "David Hollenberg"
	date: "January 29, 2008"

class SET_TAXI_STATUS_OP
inherit
	RC_PRIMITIVE_STEP_OP
		redefine
			state
		end
	RC_APPLICABLE_STEP
	HAS_USER_INTERFACE
	NO_EXTERNAL_STATE
	BOOLEAN_STRINGS

create
	make

feature -- Properties

	input_data_items: LIST [STRING]
			-- Data items required before step can be executed
		do
			Result := array_as_list (<<
				data_item.Run_type
				 >>)
		end

	state: BOOLEAN_REF
			-- State saved by soft undo

feature {NONE} -- Implementation

	do_step
		local
			ifc: SINGLE_CONSTRAINED_VALUE_IFC
			default_choice: STRING
			curr_taxi: BOOLEAN
		do
			if cache.is_scheduled_run then
				ifc := factory.single_constrained_value_interface (session.title_with_run_id ("Select Taxi Status"))
				ifc.set_label ("Is this a taxi run?")
				ifc.add_choice_and_text (Yes, Yes)
				ifc.add_choice_and_text (No, No)
				curr_taxi := cache.db_productions_row.is_taxi
				if curr_taxi then
					default_choice := Yes
				elseif state /= Void then
					if state.item then
						default_choice := Yes
					else
						default_choice := No
					end
				else
					default_choice := No
				end
				ifc.set_default_choice (default_choice)
				ifc.build
				ifc.accept_actions.extend (agent ifc.execute_action (agent call_safely_with_string (?, agent set_taxi_status, agent reset(ifc))))
				ifc.decline_actions.append (abort_actions)
			else
				set_taxi_status (No)
			end
		end

	set_taxi_status (s: STRING)
			-- Set taxi status to `s'
		require
			string_exists: s /= Void
			string_yes_or_no: is_valid_boolean_string (s)
		do
			fetch_and_modify_production (as_boolean (s))
			add_commit_action (agent publish_new_value (s))
			call_completion_actions
		end

	undo_step
			-- Undo execution of step
		do
			fetch_and_modify_production (False)
			add_commit_action (agent publish_new_value (""))
			call_completion_actions
		end

	set_state
			-- Set `state'
		do
			state := cache.is_taxi_run
		end

	fetch_and_modify_production (b: BOOLEAN)
			-- Fetch productions row, lock it and taxi education 
			-- status to `b'
		local
			prod: PRODUCTIONS_MROW
		do
			prod := cache.productions_mrow
			session.lock_row (prod)
			prod.set_is_taxi (b)
		end

	publish_new_value (s: STRING)
			-- Publish new value of taxi status
		require
			status_exists: s /= Void
		local
			e: TAXI_STATUS_CHANGE_EVENT
		do
			create e
			e.set_is_taxi (s)
			e.publish
		end

end
