
indexing
	
	description: "A run closing step operation that sets test engineer"
	author: "David Hollenberg"
	date: "March 9, 2005"

class SET_TEST_ENGINEER_OP
inherit
	RC_PRIMITIVE_STEP_OP
		redefine
			state
		end
	RC_APPLICABLE_STEP
	NO_INPUT_DATA_ITEMS
	HAS_USER_INTERFACE
	NO_EXTERNAL_STATE

create
	make

feature -- Properties

	state: STRING
			-- State saved by soft undo

feature {NONE} -- Implementation

	do_step
		local
			ifc: SINGLE_CONSTRAINED_VALUE_IFC
			engineers: LIST [M_OPERATORS_ROW]
			engineer: M_OPERATORS_ROW
			curr_engineer, default_choice: STRING
		do
			ifc := factory.single_constrained_value_interface (session.title_with_run_id ("Select Test Engineer"))
			ifc.set_label ("Test Engineer")
			ifc.enable_choices_prepended
			engineers := possible_engineers
			from
				engineers.start
			until
				engineers.after
			loop
				engineer := engineers.item
				ifc.add_choice_and_text (engineer.username, engineer.full_name)
				engineers.forth
			end
			curr_engineer := cache.db_productions_row.test_engineer
			if not curr_engineer.is_empty then
				default_choice := curr_engineer
			elseif state /= Void and then engineers.there_exists (agent matches_saved_value) then
				default_choice := state
			else
				default_choice := next_default_test_engineer
				if default_choice = Void then
					default_choice := engineers.first.username
				end
			end
			ifc.set_default_choice (default_choice)
			ifc.build
			ifc.accept_actions.extend (agent ifc.execute_action (agent call_safely_with_string (?, agent set_test_engineer, agent reset (ifc))))
			ifc.decline_actions.append (abort_actions)
		end

	possible_engineers: LIST [M_OPERATORS_ROW]
			-- Possible test engineers
		local
			template_row: M_OPERATORS_ROW
		do
			create template_row.make_template
			Result := template_row.test_engineer_operators_ordered (connection, expected_count.at_least_one)
		end

	next_default_test_engineer: STRING
			-- Username of next test engineer to use for
			-- default, based on username of last
			-- preferred test engineer to be assigned to a run
		local
			opers_row: M_OPERATORS_ROW
			prods_row: PRODUCTIONS_ROW
			prods: LIST [PRODUCTIONS_ROW]
			step_name: PRIMITIVE_STEP_NAME_VALUES
			l_engineer, l_last_engineer: STRING
			pref_engineers: LIST [M_OPERATORS_ROW]
			found: BOOLEAN
		do
			create opers_row.make_template
			pref_engineers := opers_row.preferred_test_engineers_ordered (connection, expected_count.At_least_zero)
			create prods_row.make_template
			prods := prods_row.productions_for_latest_preferred_test_engineer (step_name.Set_test_engineer, connection, expected_count.At_least_zero)
			if pref_engineers.is_empty then
				-- No preferred engineers
				Result := Void
			elseif prods.is_empty then
				-- No preferred test engineers assigned to 
				-- runs
				Result := pref_engineers.first.username
			else
				l_last_engineer := prods.first.test_engineer
				from
					pref_engineers.start
				until
					pref_engineers.after or Result /= Void
				loop
					l_engineer := pref_engineers.item.username
					if found then
						Result := l_engineer
					elseif l_engineer.is_equal (l_last_engineer) then
						found := True
					end
					pref_engineers.forth
				end
				if Result = Void then
					Result := pref_engineers.first.username
				end
			end
		end

	matches_saved_value (r: M_OPERATORS_ROW): BOOLEAN
			-- Does `r' have same test engineer as one saved in
			-- `state'?
		require
			row_exists: r /= Void
		do
			Result := equal (r.username, state)
		end

	set_test_engineer (s: STRING)
			-- Set test engineer to `s'
		require
			string_exists: s /= Void
		do
			fetch_and_modify_production (s)
			add_commit_action (agent publish_new_value (s))
			call_completion_actions
		end

	undo_step
			-- Undo execution of step
		do
			fetch_and_modify_production (Void)
			add_commit_action (agent publish_new_value (""))
			call_completion_actions
		end

	set_state
			-- Set `state'
		do
			state := cache.test_engineer
		end

	fetch_and_modify_production (s: STRING)
			-- Fetch productions row, lock it and set test engineer
			-- to `s'
		local
			prod: PRODUCTIONS_MROW
		do
			prod := cache.productions_mrow
			session.lock_row (prod)
			prod.set_test_engineer (s)
		end

	publish_new_value (s: STRING)
			-- Publish new value of test engineer
		require
			engineer_exists: s /= Void
		local
			e: TEST_ENGINEER_CHANGE_EVENT
		do
			create e
			e.set_test_engineer (s)
			e.publish
		end

end
