
indexing
	
	description: "A run closing step operation that sets tapeout date and %
		%initial estimated turaround dates for run"
	author: "David Hollenberg"
	date: "March 29, 2004"

class SET_TURNAROUND_DATES_OP
inherit
	RC_PRIMITIVE_STEP_OP
		redefine
			state
		end
	RC_APPLICABLE_STEP
	HAS_USER_INTERFACE
	NO_EXTERNAL_STATE
	DATE_VALIDITY
		export
			{NONE} all
		undefine
			default_create
		end

create
	make

feature -- Properties

	input_data_items: LIST [STRING]
			-- Data items required before step can be executed
		do
			Result := array_as_list (<<
				data_item.Assembled_run
				 >>)
		end

	state: TURNAROUND_DATES_STATE
			-- State saved by soft undo

feature {NONE} -- Implementation

	do_step
		local
			ifc: TURNAROUND_DATES_IFC
		do
			ifc := factory.turnaround_dates_interface (session.run_id, session.title_with_run_id ("Set Tapeout and Turnaround Dates"))
			ifc.set_session (session)
			ifc.set_state (state)
			ifc.success_actions.extend (agent call_completion_actions)
			ifc.abort_actions.append (abort_actions)
			ifc.build
		end

	undo_step
			-- Undo execution of step
		do
			remove_data
			call_completion_actions
		end

	remove_data
			-- Remove data added to database by step execution
		local
			runs_mrow: RUNS_MROW
			productions_mrow: PRODUCTIONS_MROW
			mask_sets_mrow: MASK_SETS_MROW
			wafer_lots: LIST [RUN_WAFER_LOTS_MROW]
			packaging_lots: LIST [RUN_PACKAGING_LOTS_MROW]
			lock_op: ROW_LOCK_OP
		do
			runs_mrow := cache.runs_mrow
			productions_mrow := cache.productions_mrow
			mask_sets_mrow := cache.mask_sets_mrow
			wafer_lots := cache.wafer_lot_mrows
			packaging_lots := cache.packaging_lot_mrows
			
			create lock_op
			lock_op.lock_nowait (runs_mrow)
			lock_op.lock_nowait (productions_mrow)
			lock_op.lock_nowait (mask_sets_mrow)
			lock_op.lock_list_nowait (wafer_lots)
			lock_op.lock_list_nowait (packaging_lots)
			
			mask_sets_mrow.set_null_date_tapeout
			mask_sets_mrow.set_null_date_masks_out
			productions_mrow.set_null_data_entry_username
			productions_mrow.set_null_date_ta_dates_modified
			productions_mrow.set_null_date_production_closed
			
			wafer_lots.do_all (agent {RUN_WAFER_LOTS_MROW}.set_null_date_received)
			packaging_lots.do_all (agent {RUN_PACKAGING_LOTS_MROW}.clear_dates)
		end

	set_state
			-- Set `state'
		local
			runs_row: RUNS_ROW
			mask_sets_row: MASK_SETS_ROW
			wafer_lots: LIST [RUN_WAFER_LOTS_ROW]
			packaging_lots: LIST [RUN_PACKAGING_LOTS_ROW]
			waf_lots: ARRAYED_LIST [WAFER_LOT_STATE]
			pkg_lots: ARRAYED_LIST [PACKAGING_LOT_STATE]
		do
			runs_row := cache.db_runs_row
			mask_sets_row := cache.db_mask_sets_row
			wafer_lots := cache.db_wafer_lot_rows
			packaging_lots := cache.db_packaging_lot_rows
			
			create waf_lots.make (wafer_lots.count)
			create pkg_lots.make (packaging_lots.count)
			
			wafer_lots.do_all (agent add_wafer_lot (waf_lots, ?))
			packaging_lots.do_all (agent add_packaging_lot (pkg_lots, ?))
			create state
			state.set_tapeout_date (mask_sets_row.date_tapeout)
			state.set_wafer_lots (waf_lots)
			state.set_packaging_lots (pkg_lots)
		end

	add_wafer_lot (list: LIST [WAFER_LOT_STATE] lot: RUN_WAFER_LOTS_ROW)
			-- Add `lot' to `list'
		require
			list_exists: list /= Void
			lot_exists: lot /= Void
		local
			new_lot: WAFER_LOT_STATE
		do
			create new_lot
			new_lot.set_wafer_process (lot.wafer_process)
			new_lot.set_date_received (lot.date_received)
			list.extend (new_lot)
		end

	add_packaging_lot (list: LIST [PACKAGING_LOT_STATE] lot: RUN_PACKAGING_LOTS_ROW)
			-- Add `lot' to `list'
		require
			list_exists: list /= Void
			lot_exists: lot /= Void
		local
			new_lot: PACKAGING_LOT_STATE
		do
			create new_lot
			new_lot.set_wafer_process (lot.wafer_process)
			new_lot.set_packaging_org_no (lot.packaging_org_no)
			new_lot.set_packaging_order_type (lot.packaging_order_type)
			new_lot.set_date_received (lot.date_received)
			new_lot.set_date_sent (lot.date_sent)
			list.extend (new_lot)
		end

end
