
indexing
	
	description: "A run closing step operation that sets vendor %
		%part sequence"
	author: "David Hollenberg"
	date: "September 23, 2008"

class SET_VENDOR_PART_SEQUENCE_OP
inherit
	RC_PRIMITIVE_STEP_OP
		redefine
			state, do_inapplicable_step, undo_inapplicable_step
		end
	HAS_USER_INTERFACE
	NO_EXTERNAL_STATE
	ITEM_COUNT_RANGES_ACCESS
		export
			{NONE} all
		undefine
			default_create
		end
	DB_OPTIONS_ACCESS
		export
			{NONE} all
		undefine
			default_create
		end

create
	make

feature -- Properties

	input_data_items: LIST [STRING]
			-- Data items required before step can be executed
		do
			Result := array_as_list (<<
				data_item.Wafer_techn,
				data_item.Field_set
				 >>)
		end

	state: STRING
			-- State saved by soft undo

feature -- Properties

	do_inapplicable_step
			-- Take actions needed to do step that is
			-- not applicable
		do
			add_commit_action (agent publish_new_value (""))
		end

	undo_inapplicable_step
			-- Take actions needed to undo step that is
			-- not applicable
		do
			add_commit_action (agent publish_new_value (""))
		end

feature {NONE} -- Implementation

	is_applicable: BOOLEAN
			-- Is step applicable to run for session?
		do
			Result := cache.is_custom_packing_target
		end

	do_step
		local
			ifc: SINGLE_CONSTRAINED_VALUE_IFC
			list: LIST [WAFER_LITHO_SIZE_PART_SEQ_IDS_ROW]
			wls_part_seq_id, template_row: WAFER_LITHO_SIZE_PART_SEQ_IDS_ROW
			s, default_choice: STRING
			entity: LABELED_ENTITY_VALUES
		do
			ifc := factory.single_constrained_value_interface (session.title_with_run_id ("Select Vendor Part Sequence"))
			ifc.set_label ("Vendor Part Sequence")
			ifc.disable_choices_prepended
			ifc.add_choice_and_text ("", "None")
			create template_row.make_template
			list := template_row.rows_for_wt_and_entity (cache.wafer_techn, entity.Run, connection, expected_count.at_least_zero)
			from
				list.start
			until
				list.after
			loop
				wls_part_seq_id := list.item
				s := wls_part_seq_id.vendor_part_sequence_id
				if not ifc.has_choice (s) then
					ifc.add_choice_and_text (s, s)
				end
				list.forth
			end
			if state /= Void and then ifc.has_choice (state) then
				default_choice := state
			else
				default_choice := ifc.choices.first
			end
			ifc.set_default_choice (default_choice)
			ifc.build
			ifc.accept_actions.extend (agent ifc.execute_action (agent call_safely_with_string (?, agent set_vendor_part_sequence, agent reset(ifc))))
			ifc.decline_actions.append (abort_actions)
		end

	set_vendor_part_sequence (vps: STRING)
			-- Set vendor part sequence by inserting row
			-- into wafer_litho_size_part_seq_ids
		require
			vps_not_void: vps /= Void
		local
			row: like wafer_litho_size_part_seq_ids_row
			entity: LABELED_ENTITY_VALUES
		do
			-- Retrieve any existing row
			row := wafer_litho_size_part_seq_ids_row
			if not vps.is_empty then
				if row.is_empty then  -- No existing row found
					create row.make_null (connection)
					row.set_wafer_techn (cache.wafer_techn)
					row.set_litho_size_id (cache.litho_size)
					row.set_labelled_entity (entity.Run)
				end
				if not row.vendor_part_sequence_id.is_equal (vps) then
					row.set_vendor_part_sequence_id (vps)
				end
			else
				if not row.is_empty then -- Existing row found
					session.lock_row (row)
					row.delete
				end
			end
			add_commit_action (agent publish_new_value (vps))
			call_completion_actions
		end

	undo_step
			-- Undo execution of step
		local
			row: like wafer_litho_size_part_seq_ids_row
		do
			row := wafer_litho_size_part_seq_ids_row
			if not row.is_empty then
				session.lock_row (row)
				row.delete
			end
			add_commit_action (agent publish_new_value (""))
			call_completion_actions
		end

	wafer_litho_size_part_seq_ids_row: WAFER_LITHO_SIZE_PART_SEQ_IDS_MROW
			-- Row for run (will be is_empty if none)
		local
			entity: LABELED_ENTITY_VALUES
		do
			create Result.make_by_key (connection, option.old_no_exception_row, cache.wafer_techn, cache.litho_size, entity.Run)
		end

	set_state
			-- Set `state'
		local
			row: WAFER_LITHO_SIZE_PART_SEQ_IDS_ROW
		do
			row := wafer_litho_size_part_seq_ids_row
			if not row.is_empty then
				state := row.vendor_part_sequence_id
			else
				state := ""
			end
		end

	publish_new_value (s: STRING)
			-- Publish new value of vendor part sequence
		require
			value_exists: s /= Void
		local
			e: VENDOR_PART_SEQUENCE_CHANGE_EVENT
		do
			create e
			e.set_vendor_part_sequence (s)
			e.publish
		end

end
