
indexing
	
	description: "Action that sets view element appearance according to %
		%a view (combination of label, color, fill, opacity)"
	author: "David Hollenberg"
	date: "October 1, 2006"

class SET_VIEW_ACTION
inherit
	VIEW_ELEMENT_ACTION

feature -- Properties

	label_generator: RETICLE_VIEW_ELEMENT_LABEL_GENERATOR
			-- Generator for text to display to label this element

	color_generator: RETICLE_VIEW_ELEMENT_COLOR_GENERATOR
			-- Generator for colors to display this element

	fill_generator: RETICLE_VIEW_ELEMENT_FILL_GENERATOR
			-- Generator for interior fill to display this element

	opacity_generator: RETICLE_VIEW_ELEMENT_OPACITY_GENERATOR
			-- Generator for opaque/transparent indicator
			-- to display this element

feature -- Modification

	set_label_generator (a_generator: like label_generator)
			-- Set `label_generator' to `a_generator'
		require
			a_generator_not_void: a_generator /= Void
		do
			label_generator := a_generator
		ensure
			label_generator_set: label_generator = a_generator
		end

	set_color_generator (a_generator: like color_generator)
			-- Set `color_generator' to `a_generator'
		require
			a_generator_not_void: a_generator /= Void
		do
			color_generator := a_generator
		ensure
			color_generator_set: color_generator = a_generator
		end

	set_fill_generator (a_generator: like fill_generator)
			-- Set `fill_generator' to `a_generator'
		require
			a_generator_not_void: a_generator /= Void
		do
			fill_generator := a_generator
		ensure
			fill_generator_set: fill_generator = a_generator
		end

	set_opacity_generator (a_generator: like opacity_generator)
			-- Set `opacity_generator' to `a_generator'
		require
			a_generator_not_void: a_generator /= Void
		do
			opacity_generator := a_generator
		ensure
			opacity_generator_set: opacity_generator = a_generator
		end

feature -- Operations

	apply (an_element: RETICLE_VIEW_ELEMENT)
			-- Apply action to `an_element'
		local
			changed: BOOLEAN
		do
			an_element.set_label_generator (label_generator)
			changed := changed or an_element.is_label_generator_changed
			an_element.set_color_generator (color_generator)
			changed := changed or an_element.is_color_generator_changed
			an_element.set_fill_generator (fill_generator)
			changed := changed or an_element.is_fill_generator_changed
			an_element.set_opacity_generator (opacity_generator)
			changed := changed or an_element.is_opacity_generator_changed
			if changed then
				an_element.add_to_changed_region
			end
		end

invariant
	label_generator_not_void: label_generator /= Void
	color_generator_not_void: color_generator /= Void
	fill_generator_not_void: fill_generator /= Void
	opacity_generator_not_void: opacity_generator /= Void
end
