
indexing
	
	description: "A run closing step operation that sets wafer %
		%part id"
	author: "David Hollenberg"
	date: "June 18, 2003"

class SET_WAFER_PART_ID_OP
inherit
	RC_PRIMITIVE_STEP_OP
		redefine
			do_inapplicable_step, undo_inapplicable_step
		end
	NO_USER_INTERFACE
	NO_EXTERNAL_STATE

create
	make

feature -- Properties

	input_data_items: LIST [STRING]
			-- Data items required before step can be executed
		do
			Result := array_as_list (<<
				data_item.Wafer_techn,
				data_item.Wafer_part_id_source
				 >>)
		end

feature -- Properties

	do_inapplicable_step
			-- Take actions needed to do step that is
			-- not applicable
		do
			add_commit_action (agent publish_new_value (""))
		end

	undo_inapplicable_step
			-- Take actions needed to undo step that is
			-- not applicable
		do
			add_commit_action (agent publish_new_value (""))
		end

feature {NONE} -- Implementation

	is_applicable: BOOLEAN
			-- Is step applicable to run for session?
		do
			Result := not cache.productions_row.is_null_wafer_part_id_source
		end

	do_step
		local
			source, wtf, part_id: STRING
			family: WAFER_TECHN_FAMILY
			len, max_len: INTEGER
		do
			source := cache.wafer_part_id_source
			wtf := cache.wafer_techn_family
			family := session.wafer_techn_family_for_wtf (wtf)
			if family = Void then
				handle_hard_error ("Cannot find object that knows how to handle wafer techn family " + wtf)
			end
			part_id := family.wafer_part_id_from_source (source)
			len := part_id.count
			max_len := {PRODUCTIONS_ROW}.maxlen_wafer_part_id
			if len > max_len then
				handle_hard_error ("Wafer part id " + part_id + " has length (" + len.out + ") that exceeds maximum length (" + max_len.out + ")")
			end
			set_wafer_part_id (part_id)
			call_completion_actions
		end

	set_wafer_part_id (s: STRING)
			-- Set wafer part id to `s'
		require
			string_exists: s /= Void
			string_not_empty: not s.is_empty
		do
			fetch_and_modify_run (s)
			add_commit_action (agent publish_new_value (s))
		end

	undo_step
			-- Undo execution of step
		do
			fetch_and_modify_run (Void)
			add_commit_action (agent publish_new_value (""))
			call_completion_actions
		end

	set_state
			-- Set `state'
		do
		end

	fetch_and_modify_run (s: STRING)
			-- Fetch runs row, lock it and set wafer part id
			-- to `s'
		local
			production: PRODUCTIONS_MROW
		do
			production := cache.productions_mrow
			session.lock_row (production)
			production.set_wafer_part_id (s)
		end

	publish_new_value (s: STRING)
			-- Publish new value of wafer part id
		require
			value_exists: s /= Void
		local
			e: WAFER_PART_ID_CHANGE_EVENT
		do
			create e
			e.set_wafer_part_id (s)
			e.publish
		end

end
