
indexing
	
	description: "A run closing step operation that sets the wafer %
		%process for each customer die on a run"
	author: "David Hollenberg"
	date: "February 27, 2007"

class SET_WAFER_PROCESSES_OP
inherit
	RC_PRIMITIVE_STEP_OP
	RC_APPLICABLE_STEP
	NO_USER_INTERFACE
	NO_EXTERNAL_STATE
	PACKING_ACCESSOR
		export
			{NONE} all
		undefine
			default_create
		end

create
	make

feature -- Properties

	input_data_items: LIST [STRING]
			-- Data items required before step can be executed
		do
			Result := array_as_list (<<
				data_item.Completed_packing
				 >>)
		end

feature {NONE} -- Implementation

	do_step
			-- Execute of step
		do
			cache.uncache_all
			set_wafer_processes
			add_commit_action (agent warn_about_unused_options)
			call_completion_actions
		end

	undo_step
			-- Undo execution of step
		do
			remove_data
			call_completion_actions
		end

	set_state
			-- Set `state'
		do
			state := Void
		end

	set_wafer_processes
			-- Insert row for each customer fab instance into
			-- IC_FAB_INSTANCE_PROCESSES
		do
			fetch_packing_selected_version
			packing_version.dies.do_if (agent insert_wafer_process, agent {PACKING_DIES_MROW}.is_cust_die)
		end

	 insert_wafer_process (die: PACKING_DIES_MROW)
			-- Insert row for fab instance for `die' into
			-- IC_FAB_INSTANCE_PROCESSES
		require
			die_not_void: die /= Void
			is_cust_die: die.is_cust_die
		local
			r: IC_FAB_INSTANCE_PROCESSES_MROW
			ifi: IC_FAB_INSTANCES_ROW
			proc: STRING
		do
			proc := die.wafer_process
			ifi := die.project.fab_instance
			if proc.is_empty then
				handle_hard_error ("no wafer process assigned to design number " + ifi.design_no.out)
			end
			create r.make_null (connection)
			r.set_m_fab_no (ifi.m_fab_no)
			r.set_wafer_process (proc)
			r.set_quantity_ordered (ifi.work_order.ic_quantity_ordered)
		end

	remove_data
			-- Remove data added to database by step execution
		local
			lock_op: ROW_LOCK_OP
		do
			fetch_fab_processes
			create lock_op
			lock_op.lock_list_nowait (fab_processes)
			fab_processes.do_all (agent {IC_FAB_INSTANCE_PROCESSES_MROW}.delete)
		end

	warn_about_unused_options
			-- Send e-mail warning about unused options, if any
		local
			n: INTEGER
			waf_process, proc, msg: STRING
			opt_ids: LIST [STRING]
		do
			build_unused_options
			from
				wafer_processes.start
			until
				wafer_processes.after
			loop
				n := n + 1
				waf_process := wafer_processes.item
				opt_ids := unused_options.item (waf_process)
				if n = 1 then
					create msg.make (200)
					msg.append ("Wafer Process                             Unused Options%N")
					msg.append ("----------------------------------------  --------------%N%N")
				end
				proc := waf_process.twin
				su.right_pad (proc, 40)
				msg.append (proc + "  " + su.joined_by (opt_ids, ", ") + "%N")
				wafer_processes.forth
			end
			if n > 0 then
				msg := n.out + " " + su.number (n, "process", "processes") + " with unused options on run " + session.run_id + "%N%N" + msg
				publish_unused_options ("unused options", msg)
			end
		end

	build_unused_options
			-- Build list of processes and hash table that
			-- maps each process to its list of unused options
		local
			q: SQL_QUERY
			waf_process, opt_id: STRING
			opt_list: ARRAYED_LIST [STRING]
		do
			create {ARRAYED_LIST [STRING]} wafer_processes.make (10)
			wafer_processes.compare_objects
			create unused_options.make (13)
			q := connection.sql_query (unused_options_query)
			q.set_numeric_bind_capacity (1)
			q.bind_num_to_string (1, session.run_id, False)
			q.set_select_count (2)
			q.select_string (1, 255)
			q.select_string (2, 255)
			q.execute
			from
				q.fetch
			until
				q.end_of_data
			loop
				waf_process := q.string_value (1)
				opt_id := q.string_value (2)
				if not wafer_processes.has (waf_process) then
					wafer_processes.extend (waf_process)
				end
				unused_options.search (waf_process)
				if unused_options.found then
					unused_options.found_item.extend (opt_id)
				else
					create opt_list.make (10)
					opt_list.extend (opt_id)
					unused_options.put (opt_list, waf_process)
				end
				q.fetch
			end
			q.release
		end

	publish_unused_options (subj, msg: STRING)
			-- Publish message `msg' warning about unused options
			-- for wafer processes, with `subj' in case e-mail
			-- is sent
		require
			subject_exists: subj /= Void
			message_exists: msg /= Void
		local
			e: UNUSED_OPTIONS_EVENT
		do
			create e
			e.set_subject (subj)
			e.set_message (msg)
			e.publish
		end

	unused_options_query: STRING =
			-- Query to find unused options
			-- requested and ordered quantities differ
		"[
  select
	wp.wafer_process, wpo.option_id
  from	option_ids oid,
	wafer_techn_family_options wtfo,
	wafer_process_options wpo,
	wafer_processes wp
  where	oid.option_id = wtfo.option_id
   and	oid.user_selectable = 'N'
   and	wtfo.wafer_techn_family = wpo.wafer_techn_family
   and	wtfo.option_id = wpo.option_id
   and	wtfo.base = 'N'
   and	wpo.wafer_techn_family = wp.wafer_techn_family
   and	wpo.wafer_process_id = wp.wafer_process_id
   and	wp.wafer_process in	-- every process for this run
	(select distinct ifip.wafer_process 
	  from	ic_fab_instance_processes ifip,
		ic_fab_instances ifi
	 where	ifip.m_fab_no = ifi.m_fab_no
	   and	ifi.run_id = :1)
   and	not exists	-- worq-based need for this option
	(select 1
	  from	wafer_option_requirements worq,
		cust_ic_design_attributes cida,
		ic_fab_instance_processes ifip,
		ic_fab_instances ifi
	  where	worq.wafer_techn_family = wp.wafer_techn_family
	   and	worq.design_techn_id = cida.final_design_techn_id
	   and	worq.option_id = wpo.option_id
	   and	worq.option_in_process = 'Y'
	   and	wafer_option_utilities.design_has_layer_error_YN
			(ifi.design_no, worq.layer_error_id) = 'Y'
	   and  cida.design_no = ifi.design_no
	   and	ifip.m_fab_no = ifi.m_fab_no
	   and	ifip.wafer_process = wp.wafer_process
	   and	ifi.run_id = :1)
   and	not exists	-- explicit option need for this option
	(select 1
	   from	ic_fab_instance_options ifio,
		ic_fab_instance_processes ifip,
		ic_fab_instances ifi
	  where	ifio.m_fab_no = ifi.m_fab_no
	    and	ifio.option_id = wpo.option_id
	    and ifip.m_fab_no = ifi.m_fab_no
	    and	ifip.wafer_process = wp.wafer_process
	    and	ifi.run_id = :1)
  order by
	wp.wafer_process, wpo.option_id
		]"

	fetch_fab_processes
			-- Remove data added to database by step execution
		local
			template_row: IC_FAB_INSTANCE_PROCESSES_MROW
		do
			create template_row.make_template
			fab_processes := template_row.rows_for_run (session.run_id, session.connection, expected_count.At_least_zero)
		end

	fab_processes: LIST [IC_FAB_INSTANCE_PROCESSES_MROW]
			-- Processes to order for each fab instance

	wafer_processes: LIST [STRING]
			-- Wafer processes with unused options

	unused_options: HASH_TABLE [LIST [STRING], STRING]
			-- Lists of unused options, indexed by wafer process

end
