
indexing
	
	description: "A run closing step operation that sets wafer %
		%technology and wafer technology org_no"
	author: "David Hollenberg"
	date: "May 13, 2003"

class SET_WAFER_TECHN_OP
inherit
	RC_PRIMITIVE_STEP_OP
		redefine
			state
		end
	RC_APPLICABLE_STEP
	HAS_USER_INTERFACE
	NO_EXTERNAL_STATE

create
	make

feature -- Properties

	input_data_items: LIST [STRING]
			-- Data items required before step can be executed
		do
			Result := array_as_list (<<
				data_item.Run_type
				 >>)
		end

	state: STRING
			-- State saved by soft undo

feature {NONE} -- Implementation

	original_abort_actions: CLOSED_ACTION_SEQUENCE
			-- Saved value of `abort_actions' when
			-- `do_step' first called

	do_step
		local
			ifc: SINGLE_CONSTRAINED_VALUE_IFC
			design_types: LIST [DESIGN_TYPES_ROW]
			design_type, template_row: DESIGN_TYPES_ROW
			default_choice: STRING
		do
			create original_abort_actions
			original_abort_actions.append (abort_actions)
			ifc := factory.single_constrained_value_interface (session.title_with_run_id ("Select Design Type"))
			ifc.set_label ("Design Type")
			ifc.enable_choices_prepended
			create template_row.make_template
			design_types := template_row.all_design_types_ordered (connection, expected_count.at_least_one)
			from
				design_types.start
			until
				design_types.after
			loop
				design_type := design_types.item
				ifc.add_choice_and_text (design_type.design_type, design_type.descr)
				design_types.forth
			end
			default_choice := design_types.first.design_type
			ifc.set_default_choice (default_choice)
			ifc.build
			ifc.accept_actions.extend (agent ifc.execute_action (agent call_safely_with_string (?, agent select_wafer_techn, agent reset(ifc))))
			ifc.decline_actions.append (abort_actions)
		end

	select_wafer_techn (dt: STRING)
			-- Select wafer techn for design type `dt'
		local
			ifc: SINGLE_CONSTRAINED_VALUE_IFC
			wafer_techns: LIST [WAFER_TECHNS_ROW]
			wafer_techn: WAFER_TECHNS_ROW
			curr_wafer_techn, default_choice: STRING
			design_type: DESIGN_TYPE_VALUES
		do
			if not dt.is_equal (design_type.ic) and not dt.is_equal (design_type.mems) then
				handle_hard_error ("Design type " + dt + " is not supported")
			end
			ifc := factory.single_constrained_value_interface (session.title_with_run_id ("Select Wafer Technology"))
			ifc.set_label ("Wafer Technology")
			ifc.enable_choices_prepended
			wafer_techns := possible_wafer_techns (dt)
			from
				wafer_techns.start
			until
				wafer_techns.after
			loop
				wafer_techn := wafer_techns.item
				ifc.add_choice_and_text (wafer_techn.wafer_techn, wafer_techn.descr)
				wafer_techns.forth
			end
			curr_wafer_techn := cache.db_runs_row.wafer_techn
			if not curr_wafer_techn.is_empty and wafer_techns.there_exists (agent {WAFER_TECHNS_ROW}.has_wafer_techn (curr_wafer_techn)) then
				default_choice := curr_wafer_techn
			elseif state /= Void and then wafer_techns.there_exists (agent {WAFER_TECHNS_ROW}.has_wafer_techn (state)) then
				default_choice := state
			else
				default_choice := wafer_techns.first.wafer_techn
			end
			ifc.set_default_choice (default_choice)
			ifc.build
			ifc.accept_actions.extend (agent ifc.execute_action (agent call_safely_with_string (?, agent set_wafer_techn, agent reset (ifc))))
			ifc.decline_actions.append (original_abort_actions)
		end

	possible_wafer_techns (dt: STRING): LIST [WAFER_TECHNS_ROW]
			-- Possible wafer techns for this run for design
			-- type `dt'
		require
			design_type_exists: dt /= Void
		local
			template_row: WAFER_TECHNS_ROW
		do
			create template_row.make_template
			Result := template_row.possible_wafer_techns_ordered (dt, cache.run_type, connection, expected_count.at_least_one)
		ensure
			result_exists: Result /= Void
		end

	set_wafer_techn (wt: STRING)
			-- Set wafer techn to `wt'
		require
			wafer_techn_exists: wt /= Void
			wafer_type_not_empty: not wt.is_empty
		do
			fetch_and_modify_run (wt)
			add_commit_action (agent publish_new_value (wt))
			call_completion_actions
		end

	undo_step
			-- Undo execution of step
		do
			fetch_and_modify_run (Void)
			add_commit_action (agent cache.uncache_wafer_techn)
			add_commit_action (agent publish_new_value (""))
			call_completion_actions
		end

	set_state
			-- Set `state'
		do
			state := cache.wafer_techn
		end

	fetch_and_modify_run (wt: STRING)
			-- Fetch runs row, lock it and set wafer techn to `wt'
			-- and set wafer_techn org_no to the org_no
			-- for `wt'
		local
			wt_row: WAFER_TECHNS_ROW
			run: RUNS_MROW
		do
			run := cache.runs_mrow
			session.lock_row (run)
			if wt = Void then
				run.set_null_wafer_techn
				run.set_null_wafer_techn_org_no
			else
				create wt_row.make_template
				wt_row := wt_row.wafer_techns_with_name (wt,
				   connection, expected_count.exactly_one).first
				run.set_wafer_techn (wt)
				run.set_wafer_techn_org_no (wt_row.org_no)
			end
		end

	publish_new_value (wt: STRING)
			-- Publish new value of wafer techn
		require
			wt_exists: wt /= Void
		local
			e: WAFER_TECHN_CHANGE_EVENT
		do
			create e
			e.set_wafer_techn (wt)
			e.publish
		end

end
