class SEVERITY_LEVELS
	-- level characters and values for response_item objects
	-- for use with the SEVERITY attribute in the database

feature -- constants

	severity_is_null: INTEGER is 0;
		-- a placeholder; logically Void

	severity_is_info: INTEGER is 1;
		-- just FYI

	severity_is_warn: INTEGER is 2;
		-- something to consider, but possibly OK		

	severity_is_urgent: INTEGER is 3;
		-- warning, OK for now but must be corrected

	severity_is_error: INTEGER is 4;
		-- definitely a problem here; operation not done

	severity_is_fatal: INTEGER is 5;
		-- a show stopper!

feature -- validation and conversion

	is_severity_level(l: INTEGER): BOOLEAN is
	do  Result := severity_is_null <= l and l <= severity_is_fatal
	end;

	severity_char(l: INTEGER): CHARACTER is
	require is_severity_level(l)
	do  inspect l
	    when severity_is_null   then Result := 'N'
	    when severity_is_info   then Result := 'I'
	    when severity_is_warn   then Result := 'W'
	    when severity_is_urgent then Result := 'U'
	    when severity_is_error  then Result := 'E'
	    when severity_is_fatal  then Result := 'F'
	    end
	end;

	severity_name(l: INTEGER): STRING is
	require is_severity_level(l)
	do  inspect l
	    when severity_is_null   then Result := "Null"
	    when severity_is_info   then Result := "Info"
	    when severity_is_warn   then Result := "Warning"
	    when severity_is_urgent then Result := "Urgent Warning"
	    when severity_is_error  then Result := "Error"
	    when severity_is_fatal  then Result := "Fatal Error"
	    end
	end;

	is_severity_char(c: CHARACTER): BOOLEAN is
	do  Result := (c='N') or (c='I') or (c='W') or (c='U') or (c='E') or (c='F');
	end;

	severity_level(c: CHARACTER): INTEGER is
	require is_severity_char(c);
	do  inspect c
	    when 'N' then Result := severity_is_null
	    when 'I' then Result := severity_is_info
	    when 'W' then Result := severity_is_warn
	    when 'U' then Result := severity_is_urgent
	    when 'E' then Result := severity_is_error
	    when 'F' then Result := severity_is_fatal
	    end
	end;

end -- SEVERITY_LEVELS
