class SIMPLE_LAYOUT_IO
	-- simple cif and gds input and output routines
	-- between the_library and a file
	-- caller must set any non-default option values before
	--  using a reader or a writer

inherit
	LAYOUT_SUPPORT

	LIBRARY_SUPPORT

	LAYOUT_IO_OPTIONS_ACCESS

feature	-- the_library

  --	the_library: LAYOUT_LIBRARY	-- is already in LIBRARY_SUPPORT

feature -- result holders

	the_messages: RESPONSE_COLLECTOR
		-- error and warning messages

	the_unknown_layers: SIMPLE_LAYER_SET
		-- unknown layers (in case one needs to pass them on)

feature -- helpers

	safe_ds_open_read(fnam: STRING): DATA_SOURCE_FILE is
		-- open a data_source_file safely or return Void
	local not_found: BOOLEAN; fullnam: STRING;
	do  if not_found
	     then Result := Void
	     else fullnam := fu.shell_tilde_expanded(fnam)
		  create Result.make_from_filename(fullnam)
	    end
	rescue
	    not_found := true
	end

	dump_the_messages is
	do  io.putstring(the_messages.all_plain_text
			("Errors:", "Urgents:", "Warnings:", "Tidbits:"))
	end;

	prepare_for_caching(rdr: LAYOUT_IO_SUPPORT)
	local his_lib: LAYOUT_LIBRARY
	do  if su.not_empty(the_cache_name) then
		create his_lib.make("ANON", 0);
		his_lib.cache_bodies_to_files(the_cache_name);
		rdr.set_the_lib(his_lib)
	    end;
	end;

	the_cache_name: STRING
		-- feed to library after creation in readers

feature -- quick cif input and output

	read_cif_file(fnam: STRING) is
		-- read cif file, leaving results in
		-- the_library (which is always created anew), and
		-- the_messages (any errors or warnings encountered)
	require su.not_empty(fnam);
		the_technology /= Void
	local dsf: DATA_SOURCE_FILE; cfil: CIF_FILE_READER;
	do  dsf := safe_ds_open_read(fnam);
	    if dsf = Void
	     then io.putstring("Cannot find/read file ");
		  io.putstring(fnam);
		  io.new_line;
	     else create cfil.make(dsf);
		  cfil.set_the_techn(the_technology);
		  prepare_for_caching(cfil);
		  -- and go do it
		  cfil.read_library;
		  -- retrieve output and go bye-bye
		  the_library := cfil.the_lib;
		  the_messages := cfil.the_messages;
		  the_unknown_layers := cfil.unknown_layers
		  dsf.close;
		  dump_the_messages
	    end;
	end;

	write_cif_into_file(dcf: DATA_CONSUMER) is
		-- write cif into the argument from the_library
		-- dcf must be intiialized beforehand and completed afterward
	require dcf /= Void
		the_library /= Void
	local cfil: CIF_FILE_WRITER;
	do  create cfil.make(dcf);
	    cfil.put_library(the_library);
	    the_messages := cfil.the_messages;
	    the_unknown_layers := cfil.unknown_layers
	    dump_the_messages
	end;

	write_cif_file(fnam: STRING) is
		-- write a complete cif file from the_library
	require su.not_empty(fnam);
		the_library /= Void
	local dcf: DATA_CONSUMER_FILE;
	do  create dcf.make(fu.shell_tilde_expanded(fnam));
	    dcf.initialize;
	    write_cif_into_file(dcf);
	    dcf.consume_eof
	end;

feature -- gds reader and writer

	read_gds_file(fnam: STRING) is
		-- read gds file, leaving results in
		-- the_library (which is always a new library)
		-- the_messages (any errors or warnings encountered)
	require su.not_empty(fnam);
		the_technology /= Void
	local gr: GDS_TO_LAYOUT; ds: DATA_SOURCE_FILE;
	do  ds := safe_ds_open_read(fnam);
	    if ds = Void
	     then io.putstring("Cannot find/read file ");
		  io.putstring(fnam);
		  io.new_line;
	     else create gr.make(ds);
		  gr.set_the_techn(the_technology);
		  prepare_for_caching(gr);
		  -- if we have a forced library unit, pre-create the_lib
		  -- and go do it
		  gr.read_gds_library;
		  -- extract results
		  the_library := gr.the_lib;
		  the_messages := gr.the_messages;
		  the_unknown_layers := gr.unknown_layers;
		  ds.close
		  dump_the_messages
	    end;
	end;

	write_gds_file(fnam: STRING) is
		-- write a gds file from the_library
	require su.not_empty(fnam);
		the_library /= Void
	local gw: GDS_FILE_WRITER; dc: DATA_CONSUMER_FILE;
	do  create dc.make(fu.shell_tilde_expanded(fnam));	-- the consumer
	    dc.initialize
	    create gw.make(dc);	-- the library writer
	    gw.write_gds_from_library(the_library);
	    the_messages := gw.the_messages;
	    the_unknown_layers := gw.unknown_layers;
	    dc.consume_eof
	    dump_the_messages;
	end;

feature -- library instantiation with error reporting

	key_is_val:      STRING is "VAL";
	error_is_unkref: STRING is "UNKREF";

	instantiate_the_library(ext: LAYOUT_LIBRARY): LAYOUT is
		-- ext is an optional external library
		-- any errors (unkref) are added to the_messages
	require the_library/= Void; su.not_empty(the_library.root_structure);
		the_library.has_structure(the_library.root_structure);
		the_messages /= Void;
	local foo: LIST[STRING];
	do  the_library.instantiate_from_root(ext);
	    foo := the_library.uninstantiated_references;
	    from  foo.start
	    until foo.after
	    loop  the_messages.update_rvalue(key_is_val, foo.item);
		  the_messages.add_response_item(error_is_unkref, false)
		  foo.forth
	    end;
	    Result := the_library.root_layout
	end;

end -- SIMPLE_LAYOUT_IO
