
indexing
	
	description: "Window to present a single labelled combo box field with editing disabled and let user select one of the allowed choices"
	author: "David Hollenberg"
	date: "April 3, 2003"

class SINGLE_CONSTRAINED_VALUE_WINDOW

inherit
	SINGLE_CONSTRAINED_VALUE_IFC
	WINDOW_INTERFACE
	MOSIS_GRAPHICAL_UTILITIES
		export
			{NONE} all
		undefine
			default_create
		end

create
	make

feature {NONE} -- Creation

	make (w: EV_TITLED_WINDOW)
			-- Create interface in window `w'
		require
			window_exists: w /= Void
			window_valid: not w.is_destroyed
		do
			make_interface
			make_proxy (w)
		end

feature -- Operations

	reset
			-- Move pointer back into window
		do
			enable_active
			-- combo_box.center_pointer
		end

feature {NONE} -- Implementation

	timeout: EV_TIMEOUT

	build_interface
			-- Build interface in `window'
		local
			main_vbox: EV_VERTICAL_BOX
			cols: WIDGET_COLUMN_SET
			box: EV_BOX
			buttons: HORIZONTAL_BUTTON_SET
		do
			if title = Void or else title.is_empty then
				set_title ("Select One")
			end
			
			generate_combo_box
			
			create cols.make (2)
			cols.set_padding (user_interface.padding)
			cols.set_border_width (user_interface.border_width)
			cols.append_row (<< gui.label_with_text (label), combo_box >>)
			cols.disable_column_expand (1)
			cols.disable_column_expand (2)

			create main_vbox
			main_vbox.set_padding (user_interface.padding)
			main_vbox.set_border_width (user_interface.border_width)
			box := cols.box
			main_vbox.extend (box)
			main_vbox.disable_item_expand (box)
			
			create buttons.make_with_text_and_actions (<<gui.ev_ok, gui.ev_cancel >>, << agent process_accept, agent process_decline >>)
			box := buttons.column_set.box
			
			main_vbox.extend (box)
			main_vbox.disable_item_expand (box)
			
			gui.set_box_background_recursive (main_vbox, background_color)
			extend (main_vbox)
			gui.add_cancel_accelerator (window, agent process_decline)
			gui.do_once_on_idle (agent post_display_actions)
		end
		
	post_display_actions
		do
			reset
		end

	process_accept
			-- Take actions needed when user accepts
			-- current operation
		local
			input_value: LABELED_INPUT_VALUE_EVENT
			list_item: EV_LIST_ITEM
			value, a_label: STRING
		do
			if is_active then
				disable_active
				list_item := combo_box.selected_item
				if not free_form_entry_allowed or list_item /= Void and then list_item.text.is_equal (combo_box.text) then
					value ?= list_item.data
					check
						value_exists: value /= Void
					end
					a_label := list_item.text
				else
					value := combo_box.text
					a_label := value
				end
				entered_value := value
				create input_value
				input_value.set_value (value)
				input_value.set_label (a_label)
				input_value.publish
				accept_actions.call ([])
			end
		end

	process_decline
			-- Take actions needed when user declines
			-- current operation
		do
			if is_active then
				disable_active
				decline_actions.call ([])
			end
		end

end
