
indexing
	
	description: "Window to present a fixed (and fairly short) set of choices and obtain user's selection"
	author: "David Hollenberg"
	date: "April 3, 2003"

class SINGLE_FIXED_SELECTION_WINDOW

inherit
	SINGLE_SELECTION_IFC
	WINDOW_INTERFACE
	MOSIS_GRAPHICAL_UTILITIES
		export
			{NONE} all
		undefine
			default_create
		end
		
create
	make

feature {NONE} -- Creation

	make (w: EV_TITLED_WINDOW)
			-- Create interface in window `w'
		require
			window_exists: w /= Void
			window_valid: not w.is_destroyed
		do
			make_interface
			make_proxy (w)
		end

feature -- Operations

	reset
			-- Move focus back into window
		do
			enable_active
			-- default_button.center_pointer
		end

feature {NONE} -- Implementation

	build_interface
			-- Build interface in `window'
		local
			main_vbox: EV_VERTICAL_BOX
			choice: STRING
			button: EV_RADIO_BUTTON
			frame: EV_FRAME
			cols: WIDGET_COLUMN_SET
			box: EV_BOX
			buttons: HORIZONTAL_BUTTON_SET
		do
			if title = Void or else title.is_empty then
				set_title ("Select Choice")
			end
			
			create cols.make (1)
			cols.set_padding (user_interface.padding)
			cols.set_border_width (user_interface.border_width)
			from
				choices.start
			until
				choices.after
			loop
				choice := choices.item
				create button.make_with_text (choice)
				button.select_actions.extend (agent select_choice (choice))
				cols.append_row (<< button >>)
				if default_button = Void then
					default_button := button
				end
				if choice.is_equal (default_choice) then
					button.enable_select
					default_button := button
				end
				choices.forth
			end
			
			create main_vbox
			main_vbox.set_padding (user_interface.padding)
			main_vbox.set_border_width (user_interface.border_width)
			
			create frame
			frame.extend (cols.box)
			
			main_vbox.extend (frame)
			main_vbox.disable_item_expand (frame)
			
			create buttons.make_with_text_and_actions (<<gui.ev_ok, gui.ev_cancel >>, << agent process_accept, agent process_decline >>)
			box := buttons.column_set.box
			
			main_vbox.extend (box)
			main_vbox.disable_item_expand (box)
			
			gui.set_box_background_recursive (main_vbox, background_color)
			extend (main_vbox)
			
			select_choice (default_choice)
			gui.add_ok_accelerator (window, agent process_accept)
			gui.add_cancel_accelerator (window, agent process_decline)
			gui.do_once_on_idle (agent post_display_actions)
		end
		
	post_display_actions
		do
			reset
		end

	select_choice (s: STRING)
			-- Set choice `s' user has selected
		require
			choice_exists: s /= Void
		do
			selected_choice := s
			selected_action := actions.item (s)
		end
	
	process_accept
			-- Take actions needed when user accepts
			-- current operation
		local
			choice: SELECTED_CHOICE_EVENT
		do
			if is_active then
				disable_active
				create choice
				choice.set_value (selected_choice)
				choice.publish
				accept_actions.call ([])
			end
		end

	process_decline
			-- Take actions needed when user declines
			-- current operation
		do
			if is_active then
				disable_active
				decline_actions.call ([])
			end
		end

	default_button: EV_RADIO_BUTTON

end
