
indexing
	
	description: "Window to present a single labelled text field and let user enter a value"
	author: "David Hollenberg"
	date: "April 3, 2003"

class SINGLE_FREE_VALUE_WINDOW

inherit
	SINGLE_FREE_VALUE_IFC
	WINDOW_INTERFACE
	MOSIS_GRAPHICAL_UTILITIES
		export
			{NONE} all
		undefine
			default_create
		end
		
create
	make

feature {NONE} -- Creation

	make (w: EV_TITLED_WINDOW)
			-- Create interface in window `w'
		require
			window_exists: w /= Void
			window_valid: not w.is_destroyed
		do
			make_interface
			make_proxy (w)
			enable_mixed_case
			default_value := ""
		end

feature -- Operations

	reset
			-- Move focus back into window
		do
			enable_active
			only_field.set_focus
			-- only_field.center_pointer
		end

feature {NONE} -- Implementation

	build_interface
			-- Build interface in `window'
		local
			cols: WIDGET_COLUMN_SET
			main_vbox: EV_VERTICAL_BOX
			box: EV_BOX
			buttons: HORIZONTAL_BUTTON_SET
			field: TEXT_FIELD
		do
			if title = Void or else title.is_empty then
				set_title ("Enter Value")
			end
			
			if is_uppercase_only then
				create field.make_with_text (default_value)
				field.enable_uppercase_only
				only_field := field
			elseif is_lowercase_only then
				create field.make_with_text (default_value)
				field.enable_lowercase_only
				only_field := field
			else
				-- Mixed case
				create only_field.make_with_text (default_value)
			end
			if minimum_width_in_chars > 0 then
				only_field.set_minimum_width_in_characters (minimum_width_in_chars)
			end

			only_field.return_actions.extend (agent process_accept)
			
			create cols.make (2)
			cols.set_padding (user_interface.padding)
			cols.set_border_width (user_interface.border_width)
			cols.append_row (<< gui.label_with_text (label), only_field >>)
			cols.disable_column_expand (1)
			cols.align_column_right (1)
			box := cols.box
			
			create main_vbox
			main_vbox.set_padding (user_interface.padding)
			main_vbox.set_border_width (user_interface.border_width)
			main_vbox.extend (box)
			main_vbox.disable_item_expand (box)
			
			create buttons.make_with_text_and_actions (<<gui.ev_ok, gui.ev_cancel >>, << agent process_accept, agent process_decline >>)
			box := buttons.column_set.box
			
			main_vbox.extend (box)
			main_vbox.disable_item_expand (box)
			
			gui.set_box_background_recursive (main_vbox, background_color)
			extend (main_vbox)
			gui.add_cancel_accelerator (window, agent process_decline)
			gui.do_once_on_idle (agent post_display_actions)
		end
		
	post_display_actions
		do
			reset
		end

	process_accept
			-- Take actions needed when user accepts
			-- current operation
		local
			input_value: INPUT_VALUE_EVENT
		do
			if is_active then
				disable_active
				entered_value := only_field.text
				create input_value
				input_value.set_value (entered_value)
				input_value.publish
				accept_actions.call ([])
			end
		end

	process_decline
			-- Take actions needed when user declines
			-- current operation
		do
			if is_active then
				disable_active
				decline_actions.call ([])
			end
		end

	only_field: EV_TEXT_FIELD
	
end
