
indexing
	
	description: "Interface to present a set of choices and obtain user's selection"
	author: "David Hollenberg"
	date: "April 3, 2003"

deferred class SINGLE_SELECTION_IFC

inherit
	ABSTRACT_INTERFACE
		
feature {NONE} -- Creation

	make_interface
			-- Create interface
		do
			create {ARRAYED_LIST [STRING]} choices.make (10)
			create actions.make (10)
			create accept_actions
			create decline_actions
		end

feature -- Properties

	accept_actions: CLOSED_ACTION_SEQUENCE
			-- Actions to be performed when user selects choice
			-- The text of the selected choice will be passed

	decline_actions: CLOSED_ACTION_SEQUENCE
			-- Actions to be performed when user declines to
			-- seleect a choice


	choices: LIST [STRING]
			-- Choices to present to user, in order

	default_choice: STRING
			-- Default choice.  If Void, first choice
			-- the default

	selected_choice: STRING
			-- Text of choice selected by user

	selected_action: PROCEDURE [ANY, TUPLE]
			-- Action associated with choice selected by user

feature -- Modification

	add_choice (s: STRING)
			-- Add `s' to list of choices to present to user,
			-- with no associated action
		require
			choice_exists: s /= Void
		do
			choices.extend (s)
			actions.put (Void, s)
		end

	add_choice_and_action (s: STRING a: PROCEDURE [ANY, TUPLE])
			-- Add `s' to list of choices to present to user
			-- and associate action `a' with it
		require
			choice_exists: s /= Void
			action_exists: a /= Void
		do
			choices.extend (s)
			actions.put (a, s)
		end

	add_default_choice (s: STRING)
			-- Add `s' to list of choices to present to user,
			-- with no associated action and also make it
			-- the default choice
		require
			choice_exists: s /= Void
		do
			choices.extend (s)
			actions.put (Void, s)
			default_choice := s
		end

	add_default_choice_and_action (s: STRING a: PROCEDURE [ANY, TUPLE])
			-- Add `s' to list of choices to present to user
			-- and associate action `a' with it, and also make
			-- it the default choice
		require
			choice_exists: s /= Void
			action_exists: a /= Void
		do
			choices.extend (s)
			actions.put (a, s)
			default_choice := s
		end

feature -- Operations
	
	execute_selected_action
			-- Execute action associated with selected choice
		require
			choice_has_action: selected_action /= Void
		do
			selected_action.call ([])
		end

	execute_action (action: PROCEDURE [ANY, TUPLE [STRING]])
			-- Execute `action' and pass it the text of
			-- selected choice
		require
			action_exists: action /= Void
		do
			action.call ([selected_choice])
		end

feature {NONE} -- Implementation

	actions: HASH_TABLE [PROCEDURE [ANY, TUPLE], STRING]
			-- Action associated with each choice, if any

end
