
indexing
	
	description: "Object that can handle a condition (error, warning, %
		%information, etc) in a standard way"
	author: "David Hollenberg"
	date: "October 26, 2003"

class STD_CONDITION_HANDLER
inherit
	CONDITION_HANDLER

feature -- Properties

	handlers: CONDITION_HANDLER_SET

feature -- Modification

	set_handlers (s: CONDITION_HANDLER_SET)
		do
			handlers := s
		ensure
			handlers_set: handlers = s
		end

feature -- Operations

	handle (cond: APPLICATION_CONDITION)
			-- Handle condition `cond'
		do
			depth := depth + 1
			if depth > 2 then
				handle_recursion_too_deep (cond)
			elseif cond.is_retryable then
				if cond.is_hard_error then
					handlers.retryable_hard_error_handler.handle (cond)
				elseif cond.is_soft_error then
					handlers.retryable_soft_error_handler.handle (cond)
				elseif cond.is_warning then
					handlers.retryable_warning_handler.handle (cond)
				elseif cond.is_question then
					handlers.question_handler.handle (cond)
				elseif cond.is_confirmation then
					handlers.confirmation_handler.handle (cond)
				elseif cond.is_information then
					handlers.information_handler.handle (cond)
				end
			else
				if cond.is_hard_error then
					handlers.hard_error_handler.handle (cond)
				elseif cond.is_soft_error then
					handlers.soft_error_handler.handle (cond)
				elseif cond.is_warning then
					handlers.warning_handler.handle (cond)
				elseif cond.is_question then
					handlers.question_handler.handle (cond)
				elseif cond.is_confirmation then
					handlers.confirmation_handler.handle (cond)
				elseif cond.is_information then
					handlers.information_handler.handle (cond)
				end
			end
			depth := depth - 1
		rescue
			depth := depth - 1
		end

end
