
indexing
	
	description: "Map that shows fields of view stepped across wafer"
	author: "David Hollenberg"
	date: "January 21, 2005"

class STEP_PATTERN_MAP
inherit
	WAFER_MAP
		redefine
			set_stroke_and_fill_colors
		end
create
	make

feature -- Operations

feature -- Properties

	title: STRING = "Step Pattern"
			-- Title to include on map
	
feature {NONE} -- Implementation

	add_stepped_field (fsize, psize: CONST_POINT step_loc: FIELD_STEP_PATTERNXFIELD_LOCS_ROW)
			-- <Precursor>
		local
			step_center: CONST_POINT
			box: DRW_RECTANGLE
		do
			step_center := step_loc.center + step_pattern_offset
			create box.make (fsize.x, fsize.y, step_center.x, step_center.y)
			box.set_stroke_width (field_stroke_width)
			box.set_stroke (field_step_stroke)
			if step_loc.partially_processed and partially_processed_or_edge_fill /= Void then
				box.set_fill (partially_processed_or_edge_fill)
			end
			last_drawing.extend (box)
		end

	generate_drawing_specific_part
			-- Generate part of drawing that is specific
			-- to a particular map
		local
			step_loc: FIELD_STEP_PATTERNXFIELD_LOCS_ROW
			step_center: CONST_POINT
			box: DRW_BASIC_RECTANGLE
			fsize: CONST_POINT
			desc1, desc2: STRING
		do
			fsize := litho_sizes_row.field_size
			from
				step_locations.start
			until
				step_locations.after
			loop
				step_loc := step_locations.item
				step_center := step_loc.center + step_pattern_offset
				create box.make (fsize.x, fsize.y, step_center.x, step_center.y)
				desc1 := primary_description (step_loc)
				desc2 := secondary_description (step_loc)
				last_drawing.extend (optional_text_pair_in_box (desc1, desc2, box, 0))
				step_locations.forth
			end
		end

	primary_description (step_loc: FIELD_STEP_PATTERNXFIELD_LOCS_ROW): STRING
			-- Primary description of `step_loc' to display on map
		do
			if has_single_field then
				Result := step_loc.copy_no.out
			else
				Result := "Field " + step_loc.field_name
				if step_loc.partially_processed then
					Result.append ("   ")
					Result.append (step_loc.copy_no.out)
				end
			end
		end

	secondary_description (step_loc: FIELD_STEP_PATTERNXFIELD_LOCS_ROW): STRING
			-- Secondary description of `step_loc' to display
			-- on map
		do
			if has_single_field then
				if step_loc.partially_processed then
					Result := Partial_label
				else
					Result := ""
				end
			else
				if step_loc.partially_processed then
					Result := Partial_label
				else
					Result := step_loc.copy_no.out
				end
			end
		end

	set_stroke_and_fill_colors
			-- Set colors used for stroke and fill
		do
			precursor
			create field_step_stroke.make_with_rgb (0.0, 0.0, 0.0)
		end

	Partial_label: STRING = "Edge"
			-- Label indicating that field is only
			-- partially processed

end
