deferred class STORED_HUNK_OF_TEXT
	-- common id and storage elements of incoming_communications,
	--  m_requests, and m_replies

inherit
	NFE_SUPPORT
		export
			{NONE} all
		undefine
			is_equal
		end

	HTML_TABLE_CONSTANTS
		export
			{NONE} all
		undefine
			is_equal
		end

	OS_ACCESS
		export
			{NONE} all
		undefine
			is_equal
		end

	COMPARABLE
		rename
			max		as more_recently_processed,
			min		as less_recently_processed
		undefine
			out
		end

feature -- Comparison

	infix "<" (other: like Current): BOOLEAN is
	    do
		if date_processed = Void then
		    Result := other.date_processed /= Void
		elseif other.date_processed = Void then
		    Result := date_processed /= Void
		elseif not date_processed.is_equal(other.date_processed) then
		    Result := date_processed < other.date_processed
		elseif not (m_communication_no = other.m_communication_no) then
		    Result := m_communication_no < other.m_communication_no
		elseif not (request_no = other.request_no) then
		    Result := request_no < other.request_no
		else
		    Result := reply_no < other.reply_no
		end
	    end


feature	-- ID attributes of the row

	m_communication_no: INTEGER is
	deferred
	end;

	request_no: INTEGER is
		-- (0 for incoming_communications)
	deferred
	end;

	reply_no: INTEGER is
		-- (0 for incoming_communications and m_requests)
	deferred
	end;

feature -- storage attributes of the row

	storage_filename: STRING is
	deferred
	end;

	storage_segment_offset: INTEGER is
	deferred
	end;

	storage_segment_length: INTEGER is
	deferred
	end;

	is_null_storage_segment_length: BOOLEAN is
	deferred
	end;

	actual_storage_segment_length: INTEGER is
		-- `storage_segment_length' if non-null, otherwise
		-- actual length of segment in storage file from 
		-- `storage_segment_offset' to the end.  If cannot get
		-- file length, returns 0
	do
		if is_null_storage_segment_length then
			Result := fu.current_size (storage_filename).to_integer
			Result := Result - storage_segment_offset
			if Result < 0 then
				Result := 0
			end
		else
			Result := storage_segment_length
		end
	end;

	set_storage_filename(str: STRING) is
	deferred
	end;

	set_storage_segment_offset(i: INTEGER) is
	deferred
	end;

	set_storage_segment_length(i: INTEGER) is
	deferred
	end;

	set_null_storage_filename is
	deferred
	end;

	set_null_storage_segment_offset is
	deferred
	end;

	set_null_storage_segment_length is
	deferred
	end;

feature -- other attributres of the row

	content_type_id: STRING is
	deferred
	end;

	date_processed: DB_DATE is
	deferred
	end;

	first_response_line: STRING is
	deferred
	end;

	header_subject: STRING is
	deferred
	end;

	sent_to: STRING is
	deferred
	end;

	security_type: STRING is
	deferred
	end;

	delete is
	deferred
	end;

feature -- viewer support

	is_viewer_table_row: BOOLEAN is
	    do
		Result := viewer_row_select = viewer_table_row
	    end

	is_viewer_status_row: BOOLEAN is
	    do
		Result := viewer_row_select = viewer_status_row
	    end

	set_viewer_table_row is
	    do
		viewer_row_select := viewer_table_row
	    end

	set_viewer_status_row is
	    do
		viewer_row_select := viewer_status_row
	    end

	viewer_anchor_base: STRING

	set_viewer_anchor_base(anchor_base: STRING) is
	    do
		viewer_anchor_base := anchor_base
	    end

	viewer_row: ARRAY [STRING] is
	    -- array of strings for display in the communication viewer.
	    -- viewer_anchor_base may be set for inclusion in anchor if needed.
	    deferred
	    end;


feature	-- the common code for the three

	is_html: BOOLEAN is
	do  Result := equal(content_type_id, glo.content_is_html)
	end;

	my_handle: STRING is
		-- my unique handle
	do  Result := su.integer_zero_padded(m_communication_no, 8);
	    Result.extend('-');
	    Result.append(su.integer_zero_padded(request_no, 3));
	    Result.extend('-');
	    Result.append(su.integer_zero_padded(reply_no, 3));
	end;

	my_standard_filename: STRING is
		-- my own special file
	do  Result := my_handle;
	    Result.prepend("arc");
	    Result.precede(operating_environment.directory_separator);
	    Result.prepend(vars.nfe_archival_dir);
	    Result.append(".txt");
	end;

	is_stored: BOOLEAN is
		-- have I been stored away?
	do  Result := not storage_filename.is_empty
	end;

	store_me_away(src: FILABLE) is
		-- do it (in my own special file) and record the result
	require not is_stored
	local  anam: STRING; afil: LARGE_PLAIN_TEXT_FILE;
	do  anam := my_standard_filename;
	    create afil.make_open_write(anam);
	    set_new_file_perm(afil);
	    src.copy_to_file(afil);
	    set_storage_filename(anam);
	    set_storage_segment_offset(0);
	    set_storage_segment_length(afil.position.to_integer);
	    os.flush_file_to_disk (afil)
	    afil.close;
	    glo.delete_on_rollback(anam)	-- just in case 
	ensure is_stored
	end;	

	get_me_back: DATA_SOURCE is
		-- build a new text producer to replay my archive
		-- Void if not is_stored or if file not found (??)
	local  afil: LARGE_PLAIN_TEXT_FILE; tp: DATA_SOURCE_FILE;
	do  if is_stored
	     then afil := fu.opened_for_read(storage_filename)
	    end;
	    if afil /= Void
	     then create tp.make_partial(afil, storage_segment_offset,
					  actual_storage_segment_length);
		  tp.reset;
		  Result := tp
	    end
	end;

	move_to_new_file(nam: STRING) is
		-- move me from my current file to the end of nam
	require is_stored; nam /= Void
	local  offset: INTEGER  afil: LARGE_PLAIN_TEXT_FILE; moi: DATA_SOURCE;
	do  moi := get_me_back;
	    if moi /= Void
	     then -- move it
		  create afil.make_open_append(nam);
		  offset := afil.position.to_integer
		  moi.copy_to_file(afil);
		  moi.close;
		  os.flush_file_to_disk (afil)
		  afil.close;
		  -- update my records
		  set_storage_filename(nam);
		  set_storage_segment_offset(offset);
		  set_storage_segment_length(actual_storage_segment_length)
	    end
	end;

	
feature {NONE} -- Common utilities and impliementation

	viewer_row_select: INTEGER

	viewer_table_row: INTEGER is 1

	viewer_status_row: INTEGER is 2

	std_html_fixer: HTML_STRING_MODIFIER is
	    once
		create Result
	    end


end -- STORED_HUNK_OF_TEXT
