
indexing
	
	description: "Font and size of a particular string in that font"
	author: "David Hollenberg"
	date: "July 25, 2006"

class STRING_SIZE_AND_FONT

create
	make

feature {NONE} -- Creation

	make (a_text: STRING a_font: EV_FONT)
			-- Create with `a_text' and `a_font'
		require
			a_text_not_void: a_text /= Void
			a_font_not_void: a_font /= Void
		local
			vals: TUPLE [INTEGER, INTEGER, INTEGER, INTEGER]
			w, h: INTEGER
		do
			text := a_text
			font := a_font
			vals := font.string_size (text)
			w := vals.integer_item (1)
			h := vals.integer_item (2)
			left_offset := vals.integer_item (3)
			right_offset := vals.integer_item (4)
			width := w + -left_offset + right_offset
			height := h
		end

feature -- Properties

	text: STRING
			-- Text whose width and height in `font' are recorded
		
	font: EV_FONT
			-- Font used for width and height calculations for
			-- `text'
		
	width: INTEGER
			-- Logical width in pixels of `text' rendered
			-- in `font'
		
	height: INTEGER
			-- Logical height in pixels of `text' rendered
			-- in `font'
		
	left_offset: INTEGER
			-- Left offset to where first "ink" is actually drawn
		
	right_offset: INTEGER
			-- Right offset to where first "ink" is actually drawn
		
feature -- Operations

	update (a_text: STRING)
			-- Update to set properties for `a_text'
		do
			make (a_text, font)
		end

invariant
	text_not_void: text /= Void
	font_not_void: font /= Void
	width_nonnegative: width >= 0
	height_nonnegative: height >= 0

end
