
indexing
	
	description: "Font factory that can provide the appropriate font and %
		%the size of a string in that font for a particular string"
	author: "David Hollenberg"
	date: "July 28, 2006"

class STRING_SIZE_FONT_FACTORY
inherit
	ANY
	CONDITION_UTILITIES
		export
			{NONE} all
		end

create
	make

feature  {NONE} -- Creation

	make
		do
			create table.make (100)
		end

feature  -- Access

	fonts: LIST [EV_FONT]
			-- Fonts to consider (in order) when calculating best
			-- font to use for a string

	best_size_and_font (a_text: STRING a_width, a_height: INTEGER): STRING_SIZE_AND_FONT
			-- Best font and size of string in that font for
			-- `a_text' which has rendered width and height that
			-- do not exceed `a_width' and `a_height'.
			-- Void if no such available font
		require
			a_text_not_void: a_text /= Void
		local
			ssf: STRING_SIZE_AND_FONT
			list: LIST [STRING_SIZE_AND_FONT]
			too_small: BOOLEAN
		do
			table.search (a_text)
			if not table.found then
				add_text (a_text)
				table.search (a_text)
				if not table.found then
					handle_internal_error ("cannot find string %"" + a_text + "%" after adding it to font table")
				end
			end
			list := table.found_item
			
			from
				list.start
			until
				list.after or too_small
			loop
				ssf := list.item
				if ssf.width <= a_width and ssf.height <= a_height then
					Result := ssf
				else
					too_small := True
				end
				list.forth
			end
		end

feature  -- Modification

	set_fonts (a_list: like fonts)
			-- Set `fonts' to `a_list'
		require
			a_list_not_void: a_list /= Void
		do
			fonts := a_list
		ensure
			fonts_set: fonts = a_list
		end

feature {NONE} -- Implementation

	add_text (a_text: STRING)
			-- Add entry for `a_text' to `table'
		require
			a_text_not_void: a_text /= Void
		local
			list: ARRAYED_LIST [STRING_SIZE_AND_FONT]
			a_font: EV_FONT
			ssf: STRING_SIZE_AND_FONT
		do
			create list.make (fonts.count)
			from
				fonts.start
			until
				fonts.after
			loop
				a_font := fonts.item
				create ssf.make (a_text, a_font)
				list.extend (ssf)
				fonts.forth
			end
			table.put (list, a_text)
		end

	table: HASH_TABLE [ARRAYED_LIST [STRING_SIZE_AND_FONT], STRING]
			-- Table of lists of font and size, indexed by string

end
