
indexing
	
	description: "An operation that creates a symbolic link"
	author: "David Hollenberg"
	date: "April 23, 2003"

class SYMLINK_CREATE_OP
inherit
	OPERATION
		rename
			try_operation as try_symlink_create
		end
	EXCEPTIONS_ACCESS
		export
			{NONE} all
		undefine
			default_create
		end
	UNIX_OS_ACCESS
		export
			{NONE} all
		undefine
			default_create
		end

create
	default_create

feature -- Operations

	create_symlink (old_path, new_path: STRING)
			-- Create symbolic link `new_path' to existing file
			-- or directory `old_path'
		require
			old_path_not_void: old_path /= Void
			old_path_not_empty: not old_path.is_empty
			new_path_not_void: new_path /= Void
			new_path_not_empty: not new_path.is_empty
		do
			old_path_name := old_path
			new_path_name := new_path
			execute
		end

feature {NONE} -- Implementation

	try_symlink_create
			-- Try symbolic link creation
		local
			error: SYMLINK_CREATE_ERROR
		do
			try_to_create_symlink_safely
			if is_created then
				publish_ok_event
			else
				publish_failed_event
				create error
				error.set_hard_error
				error.set_old_path_name (old_path_name)
				error.set_new_path_name (new_path_name)
				error.handle
			end
		end

	try_to_create_symlink_safely
			-- Try to create symbolic link.  Do not raise
			-- exception if creation fails, unless the
			-- exception is due to an assertion violation
		local
			tried: BOOLEAN
		do
			if not tried then
				try_to_create_symlink
				is_created := True
			else
				is_created := False
			end
		rescue
			if not exc.assertion_violation then
				tried := True
				retry
			end
		end

	try_to_create_symlink
		do
			unix_os.make_symbolic_link (old_path_name, new_path_name)
		end

	old_path_name: STRING
			-- Name of existing path to link to

	new_path_name: STRING
			-- Name of new symbolic link to create

	is_created: BOOLEAN
			-- Was symbolic link successfully created?

	publish_ok_event
		local
			ok_event: SYMLINK_CREATE_OK_EVENT
		do
			create ok_event
			ok_event.set_old_path_name (old_path_name)
			ok_event.set_new_path_name (new_path_name)
			ok_event.publish
		end

	publish_failed_event
		local
			failed_event: SYMLINK_CREATE_FAILED_EVENT
		do
			create failed_event
			failed_event.set_old_path_name (old_path_name)
			failed_event.set_new_path_name (new_path_name)
			failed_event.publish
		end

end
