
indexing
	
	author: "David Hollenberg"
	date: "August 21, 1998"
	description: "An operating system interval timer that counts down in %
		%real time"

deferred class SYSTEM_INTERVAL_TIMER

feature -- Properties

	is_set: BOOLEAN
			-- Is timer set?
		deferred
		end
	
	seconds_remaining: DOUBLE
			-- Number of seconds remaining until timer expires
			-- NOTE: number of seconds remaining can be
			-- zero shortly before the timer expires.  Also,
			-- intervals may be rounded to the resolution of
			-- the system clock, which can cause the time
			-- interval to appear to increase right after it
			-- is set
		deferred
		ensure
			nonnegative_result: Result >= 0.0
		end

feature -- Validity

	valid_interval (d: DOUBLE): BOOLEAN
			-- Is `d' a valid number of seconds for a
			-- timer interval?
		deferred
		end
	
feature -- Modification

	set_seconds (d: DOUBLE)
			-- Set timer to expire in `d' seconds.  Raise
			-- a Sigalrm signal exception after it expires
		require
			valid_interval: valid_interval (d)
			timer_not_set: not is_set
		deferred
		end
	
	clear
		deferred
		ensure
			timer_not_set: not is_set
		end
	
end
