class TECHN_GROUPS_ROW

inherit

	TECHN_GROUPS_RAW

	MOSIS_AREA_CALCULATOR

	DB_QUERY_OPTIONS

	STRING_UTILITIES_ACCESS

create

	make_template, make_null, make_by_key

feature -- supplemental

	min_area: DOUBLE is
		-- the max_area of the guy at tier_no - 1
	local tgs: like Current
	do  create tgs.make_by_key(connection, old_no_exception_row,
				techn_group_family, tg_family_tier_no - 1);
	    Result := tgs.max_area_sqmm_sss
		-- (returns 0 if no lower tier)
	end;

	max_area: DOUBLE is
	do
		Result := max_area_sqmm_sss
	end;

feature

	can_contain(X, Y: DOUBLE): BOOLEAN is
		-- can this techn group contain a layout of size X x Y ?
		-- round X, Y to nearest micron (11-Mar-2002, BES)
	local A: DOUBLE; iX, iY: INTEGER;
	do  A := chargeable_IC_area(connection, X, Y);
	    iX := X.rounded; iY := Y.rounded;
	    Result := A <= max_area_sqmm_sss and
		      ((iX <= max_design_size_x and iY <= max_design_size_y) or
		       (iX <= max_design_size_y and iY <= max_design_size_x))
	end

	max_size_description: STRING
		-- a textual description of my max size
		-- (Void if effectively unlimited)
	local fd2: FORMAT_DOUBLE
	do  if max_design_size_x < 100000 or
		max_design_size_y < 100000
	     then -- absolute size limited
		  create fd2.make(20,2);
		  fd2.no_justify;
		  Result := su.string_of_4(fd2.formatted(max_design_size_x/1000),
						" x ",
					   fd2.formatted(max_design_size_y/1000),
						" mm")
	    elseif max_area_sqmm_sss < 1000
	     then -- area limited
		  Result := max_area_sqmm_sss.out + " sq mm"
	    else  -- unlimited
		  Result := Void
	    end;
	end;

feature -- my parts

	ic_parts: LIST[PART_AND_PRICE] is
		-- all IC parts for this techn_group
	local ptv: PART_TYPE_VALUES
	do  if cached_ic_parts = Void
	     then cached_ic_parts := get_parts_of_type(ptv.ic)
	    end
	    Result := cached_ic_parts
	ensure Result /= Void
	end;

	ic_prep_parts: LIST[PART_AND_PRICE] is
		-- all IC_PREP parts for this techn_group
	local ptv: PART_TYPE_VALUES
	do  if cached_prep_parts = Void
	     then cached_prep_parts := get_parts_of_type(ptv.ic_prep)
	    end
	    Result := cached_prep_parts
	ensure Result /= Void
	end;

feature {NONE} -- effectors for parts

	cached_ic_parts: LIST[PART_AND_PRICE]
	cached_prep_parts: LIST[PART_AND_PRICE]

	get_parts_of_type(a_type: STRING): LIST[PART_AND_PRICE] is
	local b: DB_BIND_SIS; r: PART_AND_PRICE;
	do  create b.make(techn_group_family, tg_family_tier_no, a_type);
	    create r.make_template;
	    Result := r.all_rows_filtered
				(connection,
				 "techn_group_family = :1 and %
				 %tg_family_tier_no = :2 and %
				 %part_type = :3", b)
	end;

end -- TECHN_GROUPS_ROW
