class TEXT_FOLDING
	-- fold a long string into a bunch of lines and vice versa

inherit
	STRING_UTILITIES_ACCESS

	RFC822_PARSERS

feature -- text folding

	unfolded_text(ftxt: LIST[STRING]): STRING is
		-- unfold ftxt into a long string with '%N' as the separator
	require ftxt /= Void
	do  create Result.make(40*ftxt.count);
	    from  ftxt.start
	    until ftxt.after
	    loop  if not Result.is_empty then Result.extend('%N') end
		  Result.append(ftxt.item)
		  ftxt.forth
	    end
	ensure Result /= Void
	end;

	folded_text(txt: STRING; max: INTEGER): DYNAMIC_LIST[STRING] is
		-- fold txt into a series of strings, none of which are
		-- longer than max.
		-- first we fold at all line breaks (which are removed)
		-- and then we break any remaining long lines at white
		-- space if possible
		-- the resulting strings are all right_adjusted and empty
		-- lines are removed.
	require txt /= Void; max > 0;
	local lin, nxt: STRING;  brk: INTEGER;
	do  Result := su.broken_at(txt, '%N');
	    from  Result.start
	    until Result.after
	    loop  lin := Result.item;
		  lin.right_adjust;
		  if lin.count > max
		   then		-- break lin up into two entries
			brk := break_after(lin, max);
			nxt := lin.substring(brk+1, lin.count);
			nxt.left_adjust;
			lin.keep_head(brk);
			lin.right_adjust;
			Result.put_right(nxt)
		  end;
		  Result.forth
	    end;
	    Result := su.empty_strings_removed(Result);
	end;

	break_after(lin: STRING; max: INTEGER): INTEGER is
		-- pick a point to break up lin somewhere in the first
		-- max characters.
	require lin.count > max; max > 0;
	do  Result := break_at_lwsp(lin, max);
	    if Result = 0 then Result := break_at_special(lin, max) end;
	    if Result = 0 then Result := max end;
	ensure Result <= max; Result > 0
	end;

feature {NONE}

	break_at_lwsp(lin: STRING; max: INTEGER): INTEGER is
	do  from  Result := max;
	    until Result = 0 or else is_lwsp(lin @ (Result+1))
	    loop  Result := Result - 1
	    end
	end;

	break_at_special(lin: STRING; max: INTEGER): INTEGER is
	do  from  Result := max;
	    until Result = 0 or else is_special(lin @ Result)
	    loop  Result := Result - 1
	    end
	end;

end -- TEXT_FOLDING

