deferred class THREE_CHECKSUM_FLAVORS
	-- insert or extract any of the cif, binary_crc, or text_crc
	--  checksum from a database object that has all three
	-- NOTE: this is an _MROW parent; 
	--  there is no read_only version as yet.

inherit
	THREE_CHECKSUMS

feature -- the deferred database attributes for remapping

	cif_checksum: DOUBLE		is deferred end
	cif_byte_count: DOUBLE		is deferred end
	is_null_cif_checksum: BOOLEAN	is deferred end
	set_cif_checksum(d: DOUBLE)	is deferred end
	set_cif_byte_count(d: DOUBLE)	is deferred end
	set_null_cif_checksum		is deferred end
	set_null_cif_byte_count		is deferred end

	binary_crc_checksum: DOUBLE	is deferred end
	binary_crc_byte_count: DOUBLE	is deferred end
	is_null_binary_crc_checksum: BOOLEAN	is deferred end
	set_binary_crc_checksum(d: DOUBLE)	is deferred end
	set_binary_crc_byte_count(d: DOUBLE)	is deferred end
	set_null_binary_crc_checksum	is deferred end
	set_null_binary_crc_byte_count	is deferred end

	text_crc_checksum: DOUBLE	is deferred end
	text_crc_byte_count: DOUBLE	is deferred end
	is_null_text_crc_checksum: BOOLEAN	is deferred end
	set_text_crc_checksum(d: DOUBLE)	is deferred end
	set_text_crc_byte_count(d: DOUBLE)	is deferred end
	set_null_text_crc_checksum	is deferred end
	set_null_text_crc_byte_count	is deferred end

feature -- checksum inserters; each sets to null if argument is Void

	set_cif_sum(s: CHECKSUM_PAIR) is
	do  if s = Void
	     then set_null_cif_checksum;
		  set_null_cif_byte_count
	     else set_cif_checksum(s.checksum);
		  set_cif_byte_count(s.byte_count)
	    end
	end;

	set_binary_crc_sum(s: CHECKSUM_PAIR) is
	do  if s = Void
	     then set_null_binary_crc_checksum;
		  set_null_binary_crc_byte_count
	     else set_binary_crc_checksum(s.checksum);
		  set_binary_crc_byte_count(s.byte_count)
	    end
	end;

	set_text_crc_sum(s: CHECKSUM_PAIR) is
	do  if s = Void
	     then set_null_text_crc_checksum;
		  set_null_text_crc_byte_count
	     else set_text_crc_checksum(s.checksum);
		  set_text_crc_byte_count(s.byte_count)
	    end
	end;

feature -- checksum extractors; each returns Void if the values are null

	cif_sum: CHECKSUM_PAIR is
	do  if not is_null_cif_checksum
	     then create Result.make(cif_checksum, cif_byte_count)
	    end
	end;

	binary_crc_sum: CHECKSUM_PAIR is
	do  if not is_null_binary_crc_checksum
	     then create Result.make(binary_crc_checksum, binary_crc_byte_count)
	    end
	end;

	text_crc_sum: CHECKSUM_PAIR is
	do  if not is_null_text_crc_checksum
	     then create Result.make(text_crc_checksum, text_crc_byte_count)
	    end
	end;

	number_of_sums: INTEGER is
		-- how many checksums do we have here?
	do  if not is_null_cif_checksum
	     then Result := Result + 1 end;
	    if not is_null_binary_crc_checksum
	     then Result := Result + 1 end;
	    if not is_null_text_crc_checksum
	     then Result := Result + 1 end;
	ensure 0 <= Result and Result <= 3
	end;

	set_named_sum(s: CHECKSUM_PAIR; n: STRING) is
	require valid_name: valid_sum_name(n)
	do  if n = cif_sum_name
	     then set_cif_sum(s)
	    elseif n = binary_crc_sum_name
	     then set_binary_crc_sum(s)
	     else set_text_crc_sum(s)
	    end
	end;

feature	-- checksum selector: pick (non-Void) sum and its name

	a_non_Void_sum: CHECKSUM_PAIR is
		-- returns Void iff all three are Void
	do  if not is_null_binary_crc_checksum
	     then Result := binary_crc_sum
	    elseif not is_null_text_crc_checksum
	     then Result := text_crc_sum
	    elseif not is_null_cif_checksum
	     then Result := cif_sum
	    end
	end;

	a_non_Void_sum_name: STRING is
		-- returns name for the sum returned above
		-- (even if there is more than one)
	do  if not is_null_binary_crc_checksum
	     then Result := binary_crc_sum_name
	    elseif not is_null_text_crc_checksum
	     then Result := text_crc_sum_name
	    elseif not is_null_cif_checksum
	     then Result := cif_sum_name
	    end
	end;

feature	-- checksum realignment

	move_checksum_to_matching_type(v: THREE_CHECKSUMS) is
		-- our checksum (may be) is of the wrong type;
		-- move the numbers to the one that matches in the 
		-- argument checksum set
	require v /= Void
	local csum: CHECKSUM_PAIR; ctyp, ctyp2: STRING;
	do  csum := a_non_Void_sum;
	    ctyp := a_non_Void_sum_name;
	    if csum = Void
	     then ctyp2 := ctyp		-- guarantee no move
	    elseif equal(csum, v.cif_sum)
	     then ctyp2 := cif_sum_name
	    elseif equal(csum, v.text_crc_sum)
	     then ctyp2 := text_crc_sum_name
	    elseif equal(csum, v.binary_crc_sum)
	     then ctyp2 := binary_crc_sum_name
	     else ctyp2 := ctyp		-- guarantee no move
	    end;
	    -- now move it if necessary
	    if ctyp /= ctyp2
	     then set_named_sum(Void, ctyp)	-- erase old entry
		  if ctyp2 /= Void
		   then set_named_sum(csum, ctyp2)
		  end
	    end;
	end;

end -- THREE_CHECKSUM_FLAVORS
