
indexing
	
	description: "Action that trims a scribe that intersects at least one %
		%field of view and that has an end that is not in any field %
		%of view by trimming that end back to the field of view edge"
	author: "David Hollenberg"
	date: "October 31, 2006"

class TRIM_SCRIBE_ACTION
inherit
	SCRIBE_MULTIPLE_CHANGE_ACTION
		redefine
			default_create
		end
	GRIDABLE
		redefine
			default_create
		end

feature {NONE} -- Creation

	default_create
		do
			create resize_amount.make_xy (0, 0)
		end

feature -- Operations

	apply (an_element: RETICLE_VIEW_ELEMENT)
			-- Apply action to `an_element'
		local
			left_outside, right_outside, top_outside, bottom_outside: BOOLEAN
			left, right, top, bottom: DOUBLE
			dist_left, dist_right, dist_up, dist_down: DOUBLE
		do
			if an_element.is_scribe and then an_element.intersects_any_field then
				left := grid.rounded_edge (an_element.left_x)
				right := grid.rounded_edge (an_element.right_x)
				top := grid.rounded_edge (an_element.top_y)
				bottom := grid.rounded_edge (an_element.bottom_y)
				dist_left := an_element.closest_field_edge_to_left (right) - right
				dist_right := an_element.closest_field_edge_to_right (left) - left
				dist_up := an_element.closest_field_edge_above (bottom) - bottom
				dist_down := an_element.closest_field_edge_below (top) - top
				left_outside := an_element.x_outside_all_fields (left) and dist_right /= 0.0
				right_outside := an_element.x_outside_all_fields (right) and dist_left /= 0.0
				top_outside := an_element.y_outside_all_fields (top) and dist_down /= 0.0
				bottom_outside := an_element.y_outside_all_fields (bottom) and dist_up /= 0.0
				
				if an_element.is_horizontal_scribe and (left_outside or right_outside) then
					if an_element.is_movable_in_x then
						create {ARRAYED_LIST [RETICLE_SIMPLE_CHANGE]} change_list.make (2)
						if left_outside then
							resize_one_end (an_element, True, dist_right, 0.0)
						end
						if right_outside then
							resize_one_end (an_element, False, dist_left, 0.0)
						end
						an_element.view.changed_scribes.extend (an_element)
						add_change (an_element)
					else
						an_element.view.locked_scribes.extend (an_element)
					end
				elseif an_element.is_vertical_scribe and (top_outside or bottom_outside) then
					if an_element.is_movable_in_y then
						create {ARRAYED_LIST [RETICLE_SIMPLE_CHANGE]} change_list.make (2)
						if bottom_outside then
							resize_one_end (an_element, True, 0.0, dist_up)
						end
						if top_outside then
							resize_one_end (an_element, False, 0.0, dist_down)
						end
						an_element.view.changed_scribes.extend (an_element)
						add_change (an_element)
					else
						an_element.view.locked_scribes.extend (an_element)
					end
				end
			end
			if an_element.is_selected then
				an_element.include_in_selection_bounding_box
			end
		end

feature {NONE} -- Implementation

	resize_one_end (an_element: RETICLE_VIEW_ELEMENT is_left_or_bottom: BOOLEAN dist_x, dist_y: DOUBLE)
		local
			l_x, l_y: DOUBLE
		do
			l_x := grid.rounded_edge (dist_x)
			l_y := grid.rounded_edge (dist_y)
			if l_x /= 0.0 or l_y /= 0.0 then
				resize_amount.make_xy (l_x, l_y)
				an_element.resize_by (resize_amount, is_left_or_bottom)
				an_element.add_old_and_new_to_changed_region
				change_list.extend (create {SCRIBE_END_POSITION_CHANGE}.make (an_element, resize_amount.x, resize_amount.y, is_left_or_bottom))
			end
		end
	
	resize_amount: POINT
			-- Temporary to avoid object creation

end
