indexing
	
	description: "TSMC_018 wafer techn family"
	author: "David Hollenberg";
	date: "June 18, 2003"

class TSMC_018

inherit
	WAFER_TECHN_FAMILY
		redefine
			add_explicit_cats_statements
		end;

feature -- Properties

	name: STRING is "TSMC_018"

	native_design_techn: STRING is
			-- name of the "native" design_techn
	do  Result := dtv.tsmc18
	end;

feature	-- statements for Cats fracture script

	add_explicit_cats_statements (d: DIES_XROW) is
	do	-- establish cached data
		nwel_leaf := leaf (source_layer_name (d, lvl.n_well));
		act_leaf  := leaf (source_layer_name (d, lvl.thin_oxide));
		od2_leaf  := leaf (source_layer_name (d, lvl.thick_oxide));
		poly_leaf := leaf (source_layer_name (d, lvl.poly));
		cont_leaf := leaf (source_layer_name (d, lvl.contact));
		-- break it up into chunks for greater digestibility
		well_and_active(d);
		n_and_p_select(d);
		poly_mask(d);
		ldd_masks(d);	-- (option tests are inside)
		if option_is_present (oid.rpo) then
			rpo_mask (d)
		end;
		if option_is_present (oid.vtm_n) then
			vtm_n_mask (d)
		end;
		if option_is_present (oid.vtm_p) then
			vtm_p_mask (d)
		end;
		if option_is_present(oid.ctm2) or
		   option_is_present(oid.ctm3) or
		   option_is_present(oid.ctm4) then
			use_alternate_ctm(d)
		end;
		if option_is_present (oid.esd) then
			esd_mask(d)
		end;
		if option_is_present (oid.hri) then
			hri_mask(d)
		end;
		if option_is_present (oid.pm) then
			polyimide_mask(d)
		end;
		if option_is_present (oid.hv) then
			handle_hard_error ("TSMC_018 does not support HV mask fracturing.")
		end;
		-- metal fill and slot logic is provided automagically
		--  by CATS_LAYER_SOURCES
	end;

feature {NONE} -- cached data

	nwel_leaf, act_leaf, od2_leaf, poly_leaf, cont_leaf: CATS_EXPRESSION
		-- we use these all over the place; stash here once
		-- (we assume that _every_ design_techn has these layers)

	nsel_leaf, psel_leaf: CATS_EXPRESSION
		-- nominally, the drawn layers, but we do have exceptions

feature {NONE} -- the specifics

	rlppdmy_layer(d: DIES_ROW): STRING is
		-- for SCMOS, hi_res_poly is both the HRI mask source
		--  AND the rlppdmy equivalent
	do  if is_scmos_die(d) then
		Result := lyr.hi_res_poly
	    else
		Result := lyr.rlppdmy
	    end;
	end;

	well_and_active (d: DIES_XROW) is
	local pw, odx, tox: CATS_EXPRESSION
	do
	    -- nwell
	    add_clevel_with_merge (nwel_leaf, lvl.n_well, 0.56, d)
	    -- pwell: source is n_well, but then maybe modified
	    pw := nwel_leaf
	    if d.is_present (lyr.nt_n) then
		pw := pw #or# leaf (lyr.nt_n)	-- native rules only
	    elseif d.is_present (lyr.expl_field_implant) then
		pw := pw #xor# leaf (lyr.expl_field_implant)	-- SCMOS only
	    end;
	    add_clevel_with_merge (pw, lvl.p_well, 0.56, d)
	    -- OD: for SCMOS, enlarge contact dogbone
	    odx := act_leaf;		-- usual active source
	    if is_scmos_die(d) then
		add_ctemp(odx #or#	-- add 0.13 um active dogbone
			  ((odx #and# cont_leaf) 
					#sized# 0.38 #sized# -0.12),
				 tmp_scmos_act, 0.005);
		odx := leaf (tmp_scmos_act);	-- SCMOS active source
		add_clevel_with_bias(odx, lvl.thin_oxide, d);
	    end;
	    -- ODR (trench): source is thin_oxide
	    tox := odx	#sized#	(bias_differential(d, lvl.thin_oxide) - 0.8)
			   #sized# 0.4;
	    add_clevel (tox, lvl.trench, d)
	end;

	tmp_scmos_act: STRING is "SCMOS_ACTIVE"

	n_and_p_select(d: DIES_XROW) is
	local nnam, pnam, rnam: STRING; hn, hp: BOOLEAN; 
		nmask, pmask: CATS_EXPRESSION
	do  nnam := source_layer_name (d, lvl.n_plus_select);
	    pnam := source_layer_name (d, lvl.p_plus_select);
	    hn := d.is_present(nnam);
	    hp := d.is_present(pnam);
	    -- derivation of missing select from the other one
	    if hp and not hn then
		nsel_leaf := #not# (leaf (pnam) #sized# 0.16)
	    else
		nsel_leaf := leaf (nnam)
	    end;
	    if hn and not hp then
		psel_leaf := #not# (leaf (nnam) #sized# 0.16)
	    else
		psel_leaf := leaf (pnam)
	    end;
	    -- now do the NPnew and PPnew derivation
	    add_ctemp (nsel_leaf #sized# 0.16,	tmp_np_08, 0.005);
	    add_ctemp (((((((#not# leaf (tmp_np_08)) #or# psel_leaf)
				#sized# -0.42) #sized# 0.84) #sized# -0.42)
				#minus# leaf (tmp_np_08)) 
			#or# psel_leaf,		tmp_pp_big, 0.005);
	    add_ctemp ( (((act_leaf #minus# nsel_leaf) #minus# psel_leaf)
				#sized# -0.10) #sized# 0.30, 
						tmp_od_bare, 0.005);
	    nmask := (((((((#not# (leaf (tmp_pp_big) #sized# 0.16)) #or# nsel_leaf)
			#sized# -0.42) #sized# 0.84) #sized# -0.42)
			#minus# (leaf (tmp_pp_big) #sized# 0.16)) #or# nsel_leaf)
			#minus# leaf (tmp_od_bare);
	    pmask := leaf (tmp_pp_big) #minus# leaf (tmp_od_bare);
	    rnam := rlppdmy_layer(d);
	    if d.is_present (rnam) then
		pmask := pmask #minus# leaf (rnam)
	    end;
	    add_clevel(select_sizing(nmask), lvl.n_plus_select, d);
	    add_clevel(select_sizing(pmask), lvl.p_plus_select, d);
	end;

	tmp_np_08: STRING is "NP_08"
	tmp_pp_big: STRING is "PP_BIG"
	tmp_od_bare: STRING is "OD_BARE"

	select_sizing(src: CATS_EXPRESSION): CATS_EXPRESSION is
		-- apply the np/pp sizing dance specified in the MB
		-- (the net sizing is zero, BTW)
	do  Result := src #sized# 0.30 #sized# -0.60
			  #sized# 0.50 #sized# -0.40
			  #sized# 0.30 #sized# -0.20
			  #sized# 0.10
	end;

	poly_mask(d: DIES_XROW) is
	local wdt: STRING; mask: CATS_EXPRESSION; 
	do  -- make mods only for SCMOS
	    if is_scmos_die(d) then
		wdt := d.wafer_design_techn;
		-- poly gate endcap extension for vanilla and subm
		if equal(wdt, dtv.scn6m_012) then
			mask := extend_poly_endcap(0.24, 0.01)
		elseif equal(wdt, dtv.scn6m_subm_01) then
			mask := extend_poly_endcap(0.20, 0.03)
		else
			mask := poly_leaf
		end;
		-- contact dogbones for subm and deep
		if equal(wdt, dtv.scn6m_subm_01) or
		   equal(wdt, dtv.scn6m_deep_009) then
			mask := mask #or#
				((poly_leaf #and# cont_leaf) #sized# 0.36 #sized# -0.12);
		end;
		-- fracture it
		add_clevel_with_bias(mask, lvl.poly, d)
	    end;
	end;

	extend_poly_endcap(dr_ext, xtra_ext: DOUBLE): CATS_EXPRESSION is
		-- the drawn poly overlap of the gate is dr_ext;
		-- extend the overlap by another xtra_ext
	do  -- fracture the channel
	    add_ctemp (poly_leaf #and# act_leaf, tmp_channel, 0.005);
	    -- slowly glom all poly w/in dr_ext of the channel
	    Result := (leaf (tmp_channel) #sized# (dr_ext/2)) #and# poly_leaf;
	    Result := (Result #sized# (dr_ext/2)) #and# poly_leaf;
	    Result := (Result #sized# (dr_ext/2)) #and# poly_leaf;
	    Result := (Result #sized# (dr_ext/2)) #and# poly_leaf;
	    -- now grow the poly by the extra_ext
	    Result := (Result #sized# (2*xtra_ext));
	    -- remove the growth in poly width, leaving just endcaps
	    Result := (Result #minus#
			  (leaf (tmp_channel) #sized# (2*dr_ext)));
	    -- add to the poly and trim fragments
	    Result := (Result #or# poly_leaf) #sized# (-xtra_ext) #sized# xtra_ext
	end;

	tmp_channel: STRING is "SCN_CHANNEL"

	ldd_masks(d: DIES_XROW) is
		-- (check both options before we go)
	local n2v, p2v, n3v, p3v: CATS_EXPRESSION; rnam: STRING
	do  -- generate all 4 source expressions; decide later which
	    -- are needed, and which are tossed
	    n2v := (nsel_leaf #minus# nwel_leaf) #minus# (od2_leaf #sized# -0.06);
	    p2v := (psel_leaf #and# nwel_leaf) #minus# (od2_leaf #sized# -0.06);
	    n3v := (nsel_leaf #minus# nwel_leaf) #and# (od2_leaf #sized# 0.06);
	    p3v := (psel_leaf #and# nwel_leaf) #and# (od2_leaf #sized# 0.06)
	    -- factor in dmn2v and dmp2v if they are present
	    if d.is_present (lyr.dmn2v) then
		n2v := n2v #or# leaf (lyr.dmn2v);
		n3v := n3v #minus# leaf (lyr.dmn2v);
		p3v := p3v #minus# leaf (lyr.dmn2v);
	    end;
	    if d.is_present (lyr.dmp2v) then
		p2v := p2v #or# leaf (lyr.dmp2v);
		n3v := n3v #minus# leaf (lyr.dmp2v);
		p3v := p3v #minus# leaf (lyr.dmp2v);
	    end;
	    -- apply the ldd_sizing dance to each
	    n2v := ldd_sizing (n2v);
	    p2v := ldd_sizing (p2v);
	    n3v := ldd_sizing (n3v);
	    p3v := ldd_sizing (p3v);
	    -- factor in vardmy and rlppdmy, if they are present
	    if d.is_present (lyr.vardmy) then
		n2v := n2v #minus# leaf (lyr.vardmy);
		p2v := p2v #minus# leaf (lyr.vardmy);
		n3v := n3v #minus# leaf (lyr.vardmy);
		p3v := p3v #minus# leaf (lyr.vardmy);
	    end;
	    rnam := rlppdmy_layer(d);
	    if d.is_present (rnam) then
		p2v := p2v #minus# leaf (rnam);
		p3v := p3v #minus# leaf (rnam);
	    end;
	    -- now put each one out if option is present and layer not drawn
	    if option_is_present (oid.x1_8ldd) then
		if not d.is_present (source_layer_name (d, lvl.nldd_2v)) then
			add_clevel(n2v, lvl.nldd_2v, d)
		end;
		if not d.is_present (source_layer_name (d, lvl.pldd_2v)) then
			add_clevel(p2v, lvl.pldd_2v, d)
		end;
	    end;
	    if option_is_present (oid.x3_3v) then
		if not d.is_present (source_layer_name (d, lvl.nldd_3v)) then
			add_clevel(n3v, lvl.nldd_3v, d)
		end;
		if not d.is_present (source_layer_name (d, lvl.pldd_3v)) then
			add_clevel(p3v, lvl.pldd_3v, d)
		end;
	    end;
	end;

	ldd_sizing(src: CATS_EXPRESSION): CATS_EXPRESSION is
		-- apply the ldd sizing dance specified in the MB
		-- (the net sizing is zero, BTW)
	do  Result := src #sized# 0.42 #sized# -0.84
			  #sized# 0.72 #sized# -0.60
			  #sized# 0.50 #sized# -0.40
			  #sized# 0.30 #sized# -0.20
			  #sized# 0.10
	end;

	rpo_mask(d: DIES_XROW) is
	local rpo_leaf, mask: CATS_EXPRESSION
	do  rpo_leaf := leaf (source_layer_name (d, lvl.rpo));
	    mask := 
-- this is a direct transcription from T-018-MM-MB-001, rev 2.4
--  except that the sizings have all been doubled (1-sided vs 2-sided)
(((((rpo_leaf #or# ((((act_leaf #minus# poly_leaf) #and# rpo_leaf) #sized#
0.64) #minus# ((act_leaf #minus# rpo_leaf) #sized# 0.30))) #sized# -0.34)
#sized# 0.34) #or# ((((((((rpo_leaf #or# ((((((poly_leaf #minus# (act_leaf
#sized# 0.44)) #and# rpo_leaf) #sized# 0.64) #sized# -0.4) #sized# 0.4)
#minus# ((poly_leaf #minus# rpo_leaf) #sized# 0.30))) #sized# -0.34)
#sized# 0.34) #sized# 0.4) #sized# -0.4) #minus# ((act_leaf #sized# 0.30)
#or# ((((poly_leaf #minus# act_leaf) #minus# rpo_leaf) #sized# 0.30)
#minus# (((rpo_leaf #or# ((((((poly_leaf #minus# (act_leaf #sized# 0.44))
#and# rpo_leaf) #sized# 0.64) #sized# -0.4) #sized# 0.4) #minus#
((poly_leaf #minus# rpo_leaf) #sized# 0.30))) #sized# -0.34) #sized#
0.34)))) #sized# -0.4) #sized# 0.4)) #or# ((((((((((((poly_leaf #and#
act_leaf) #and# od2_leaf) #and# nsel_leaf) #minus# nwel_leaf) #sized# 3.0)
#and# (rpo_leaf #sized# 2.0)) #minus# (poly_leaf #sized# -0.10)) #minus#
(cont_leaf #sized# 0.6)) #or# rpo_leaf) #sized# -1.49) #sized# 1.49)
#or# rpo_leaf));
-- end transcription
	    add_clevel(mask, lvl.rpo, d);
	end;

	vtm_n_mask(d: DIES_XROW) is
		-- NOTE: always derived; never empty
	local vtmn: STRING; mask: CATS_EXPRESSION
	do  mask := nwel_leaf;
	    vtmn := source_layer_name (d, lvl.vtm_n)
	    if d.is_present (vtmn) then
		mask := mask #or# leaf (vtmn)
	    end;
	    if d.is_present (lyr.nt_n) then
		mask := mask #or# leaf (lyr.nt_n)
	    end;
	    add_clevel_with_bias (mask, lvl.vtm_n, d)
	end;

	vtm_p_mask(d: DIES_XROW) is
		-- NOTE: always derived; never empty
	local vtmp: STRING; mask: CATS_EXPRESSION
	do  mask := nwel_leaf;
	    vtmp := source_layer_name (d, lvl.vtm_p);
	    if d.is_present (vtmp) then
		mask := mask #minus# leaf (vtmp)
	    end;
	    add_clevel_with_bias (mask, lvl.vtm_p, d)
	end;

	use_alternate_ctm(d: DIES_XROW) is
		-- if this die has an alternate ctm layer, 
		--  fracture it instead of std CTM (aka, ctm5)
		-- (there can only be one CTM layer in a project)
	local alt_ctm: STRING
	do  -- look for an alternate ctm layer in the layout
	    if d.is_present (lyr.ctm2) then
		alt_ctm := lyr.ctm2
	    elseif d.is_present (lyr.ctm3) then
		alt_ctm := lyr.ctm3
	    elseif d.is_present (lyr.ctm4) then
		alt_ctm := lyr.ctm4
	    end;
	    -- use in place of lyr.ctm on ctm mask
	    if alt_ctm /= Void then
		add_clevel_with_bias (leaf (alt_ctm), lvl.cap_top_metal, d)
	    end
	end;

	esd_mask(d: DIES_XROW) is
	local esd: STRING; mask: CATS_EXPRESSION
	do  esd := source_layer_name(d, lvl.esd_implant);
	    if d.is_present(esd) then
		mask := (((((((act_leaf #sized# -0.4) #and# nsel_leaf)
			 #and# leaf (lyr.rpo)) #minus# nwel_leaf)
			 #minus# (poly_leaf #sized# 0.4))
			 #sized# -1.6) #sized# 1.2) #and# leaf (esd);
		add_clevel (mask, lvl.esd_implant, d)
	    end
	end;

	hri_mask(d: DIES_XROW) is
		-- SCMOS only special
	local hri: STRING; mask: CATS_EXPRESSION; lamb: DOUBLE
	do  hri := source_layer_name(d, lvl.hri)
	    if is_scmos_die(d) and d.is_present(hri) then
		lamb := d.design_techn.design_lambda.to_double
		mask := (leaf (hri) #and# poly_leaf) #sized# (6*lamb);
		mask := (mask #and# poly_leaf) #sized# (6*lamb);
		mask := (mask #and# poly_leaf) #sized# (6*lamb);
		mask := (mask #and# poly_leaf) #sized# (6*lamb);
		mask := (mask #and# poly_leaf) #sized# (4*lamb);
		add_clevel (mask, lvl.hri, d)
	    end
	end;

	polyimide_mask(d: DIES_XROW) is
		-- we are making a PM mask; use passivation if not drawn
	local pm: STRING; mask: CATS_EXPRESSION
	do  pm := source_layer_name(d, lvl.polyimide);
	    if not d.is_present(pm) then
		mask := (leaf (source_layer_name(d, lvl.glass))
				#sized# 60) #sized# -20;
		add_clevel (mask, lvl.polyimide, d)
	    end
	end;

end
