
indexing
	
	description: "TSMC_025 wafer techn family"
	author: "David Hollenberg";
	date: "June 18, 2003"

class TSMC_025
inherit
	WAFER_TECHN_FAMILY
		redefine	add_explicit_cats_statements
		end;

feature -- Properties

	name: STRING is "TSMC_025"

	native_design_techn: STRING is
			-- name of the "native" design_techn
	do  Result := dtv.tsmc25
	end;

	add_explicit_cats_statements (d: DIES_XROW) is
	do	-- establish cached data
		wafer_dt := d.wafer_design_techn;
		act_layer := source_layer_name (d, lvl.thin_oxide);
		-- same routines for all design_technologies
		well_and_active (d);
		n_and_p_select (d);
		poly_mask (d);
		if option_is_present (oid.x3_3v) then
			thick_ox_and_nldd (d)
		end;
		if option_is_present (oid.rpo) then
			rpo_mask (d)
		end;
		if option_is_present (oid.ctm2) or 
		   option_is_present (oid.ctm3) then
			use_alternate_ctm (d)
		end;
		if option_is_present (oid.vtm_n) then
			vtm_n_mask (d)
		end;
		if option_is_present (oid.vtm_p) then
			vtm_p_mask (d)
		end;
	end;

feature {NONE} -- cached data

	act_layer: STRING
		-- we want this one all over the place; stash here once

	wafer_dt: STRING
		-- wafer design_techn_id

feature {NONE} -- statement generators

	well_and_active (d: DIES_XROW) is
	local nw, pw, tox: CATS_EXPRESSION
	do
	    -- nwell
	    nw := leaf (source_layer_name (d, lvl.n_well))
	    add_clevel_with_merge (nw, lvl.n_well, 0.56, d)
	    -- pwell: source is n_well, but then maybe modified
	    pw := nw
	    if d.is_present (lyr.nt_n) then
		pw := pw #or# leaf (lyr.nt_n)	-- native rules only
	    elseif d.is_present (lyr.expl_field_implant) then
		pw := pw #xor# leaf (lyr.expl_field_implant)	-- SCMOS only
	    end;
	    add_clevel_with_merge (pw, lvl.p_well, 0.56, d)
	    -- trench: source is thin_oxide
	    tox := (leaf (act_layer)
			#sized#	(bias_differential(d, lvl.thin_oxide) - 0.8))
			   #sized# 0.4;
	    add_clevel (tox, lvl.trench, d)
	end;

	tmp_raw_nplus: STRING is "RAW_NPLUS"
	tmp_raw_pplus: STRING is "RAW_PPLUS"
	tmp_final_nplus: STRING is "FINAL_NPLUS"
	tmp_intr_poly: STRING is "INTRINSIC_POLY"

	n_and_p_select (d: DIES_XROW) is
	local pp: CATS_EXPRESSION
	do
	    -- first we build the "raw" nplus and pplus
	    -- (they are later used again for the derived ldd implants)
	    if is_scmos_die(d) then
		build_scmos_raw_selects (d)
	    else
		build_native_raw_selects (d)
	    end;
	    -- save "final" nplus for other future use
	    add_ctemp (merge_X2 (leaf (tmp_raw_nplus), 0.4),
				tmp_final_nplus, 0.01)
	    -- fracture n_plus and p_plus
	    add_clevel (leaf (tmp_final_nplus), lvl.n_plus_select, d)
	    pp := leaf (tmp_raw_pplus);
	    if d.is_present (lyr.rwdmy) or d.is_present (lyr.rlppdmy) then
		pp := pp #minus# (leaf (lyr.rwdmy) #or# leaf (lyr.rlppdmy))
	    end;
	    add_clevel_with_merge_x2(pp, lvl.p_plus_select, 0.4, d)
	end;

	build_scmos_raw_selects (d: DIES_XROW) is
	local hn, hp: BOOLEAN; 
	do  -- check for presence
	    hn := d.is_present (lyr.n_plus_select);
	    hp := d.is_present (lyr.p_plus_select);
	    if hn and not hp then	-- use n_plus for both
		add_ctemp (leaf (lyr.n_plus_select), 
						tmp_raw_nplus, 0.01);
		add_ctemp (derive_missing_select (lyr.n_plus_select, d),
						tmp_raw_pplus, 0.01)
	    elseif hp and not hn then	-- use p_plus for both
		add_ctemp (leaf (lyr.p_plus_select),
						tmp_raw_pplus, 0.01);
		add_ctemp (derive_missing_select (lyr.p_plus_select, d),
						tmp_raw_nplus, 0.01)
	    else			-- both drawn
		-- find all the intrinsic poly and cover it
		add_ctemp ((leaf (lyr.poly) #sized# 0.8) #minus#
				(leaf (lyr.p_plus_select) #or# leaf (lyr.n_plus_select)),
						tmp_intr_poly, 0.01);
		add_ctemp (leaf (lyr.n_plus_select) #or#
				(leaf (tmp_intr_poly) #minus# leaf (lyr.n_well)),
						tmp_raw_nplus, 0.01);
		add_ctemp (leaf (lyr.p_plus_select) #or#
				(leaf (tmp_intr_poly) #and# leaf (lyr.n_well)),
						tmp_raw_pplus, 0.01);
	    end;
	end;

	derive_missing_select (other: STRING; d: DIES_XROW) : CATS_EXPRESSION is
		-- expression for the missing select from other
		-- other is lyr.n_plus_select or lyr.p_plus_select
		-- leave 0.1 um gap from other except around active
	local sel_o_act: DOUBLE; act_region, x_other: CATS_EXPRESSION
	do  if d.design_techn.is_scalable then
		sel_o_act := 4 * d.design_techn.lambda	-- 2 per side
	    else
		sel_o_act := 0.52			-- 0.26 per side
	    end;
	    act_region := (leaf (act_layer) #sized# sel_o_act) #minus# leaf (other);
	    x_other := #not# (leaf (other) #sized# 0.2);
	    Result := act_region #or# x_other
	end;

	tmp_fringe_nplus: STRING is "FRINGE_NPLUS"
	tmp_fringe_pplus: STRING is "FRINGE_PPLUS"

	build_native_raw_selects (d: DIES_XROW) is
	do  if d.is_present (lyr.n_plus_select) and d.is_present (lyr.p_plus_select) then
		-- for TSMC (and CS022), we deal with the special mask op needed
		-- if active plugs are closer than 0.4 um to a well edge
		-- compute the "fringe" select regions
		add_ctemp ((((leaf (act_layer) #and# leaf (lyr.n_plus_select)) #and# leaf (lyr.n_well))
				#sized# 0.3) #minus# (leaf (lyr.n_well) #sized# -0.5),
						tmp_fringe_nplus, 0.01)
		add_ctemp ((((leaf (act_layer) #and# leaf (lyr.p_plus_select)) #minus# leaf (lyr.n_well))
				#sized# 0.3) #and# (leaf (lyr.n_well) #sized# 0.5),
						tmp_fringe_pplus, 0.01)
		-- now add them to the drawn selects to form the raw selects
		add_ctemp ((((leaf (lyr.n_plus_select) #or# leaf (tmp_fringe_nplus))
				#sized# 0.42) #sized# -0.42)
				#minus# (leaf (tmp_fringe_pplus) #sized# 0.2),
						tmp_raw_nplus, 0.01)
		add_ctemp ((((leaf (lyr.p_plus_select) #or# leaf (tmp_fringe_pplus))
				#sized# 0.42) #sized# -0.42)
				#minus# (leaf (tmp_fringe_nplus) #sized# 0.2),
						tmp_raw_pplus, 0.01)
	    else	-- abandon this stuff and use SCMOS style
		build_scmos_raw_selects(d)
	    end
	end;

	tmp_channel: STRING is "CHANNEL"

	thick_ox_and_nldd (d: DIES_XROW) is
	local thox_lyr, n2v_lyr, n3v_lyr: STRING; 
		n2v_src, n3v_src, pldd_src: CATS_EXPRESSION;
	do  -- the layers of interest
	    -- NOTE: all of our design techns have a thick_oxide source layer
	    --	but only the native rules have n2v and n3v source layers
	    thox_lyr := source_layer_name (d, lvl.thick_oxide)
	    n2v_lyr := source_layer_name (d, lvl.nldd_2v)
	    n3v_lyr := source_layer_name (d, lvl.nldd_3v)
	    -- the nldd implant sources: as drawn or derived
	    if su.not_empty (n2v_lyr) and then d.is_present (n2v_lyr) then
		n2v_src := leaf (n2v_lyr)
	    else
		n2v_src := leaf (tmp_raw_nplus) #minus# leaf (thox_lyr);
		if d.is_present (lyr.vardummy) then 
			n2v_src := n2v_src #minus# leaf (lyr.vardummy)
		end;
	    end;
	    if su.not_empty (n3v_lyr) and then d.is_present (n3v_lyr) then
		n3v_src := leaf (n3v_lyr)
	    else
		n3v_src := (leaf (thox_lyr) #minus# leaf (tmp_raw_pplus)) #minus# leaf (lyr.n_well);
	    end;
	    -- now toss in the PLDD (25E process) perturbations
	    if option_is_present (oid.pldd) then
		add_ctemp (leaf (lyr.poly) #and# leaf (act_layer), tmp_channel, 0.01);
		n2v_src := ((n2v_src #and# leaf (tmp_channel)) #sized# 0.7) #or# n2v_src
		n2v_src := (n2v_src #sized# 0.36) #sized# -0.36
		pldd_src := ((leaf (tmp_raw_pplus) #and# leaf (tmp_channel)) #sized# 0.7)
				#or# leaf (tmp_raw_pplus);
		add_clevel (merge_X2 (pldd_src, 0.36), lvl.pldd, d)
	    end;
	    -- finally, put out n2v and n3v
	    add_clevel (merge_X2 (n2v_src, 0.36), lvl.nldd_2v, d)
	    add_clevel (merge_X2 (n3v_src, 0.36), lvl.nldd_3v, d)
	end;

	tmp_poly_3vg: STRING is "POLY_3VG"
	tmp_poly_other: STRING is "POLY_OTHER"
	tmp_poly_tsmc: STRING is "POLY_TSMC"

	poly_mask (d: DIES_XROW) is
	local tox_lyr, thox_lyr, poly_src: STRING;
		toxbias, polbias, grobias, threel, almost_3l: DOUBLE
	do
	    tox_lyr := act_layer;
	    thox_lyr := source_layer_name (d, lvl.thick_oxide);
	    toxbias := bias_differential (d, lvl.thin_oxide) + 0.2
			-- toxbias is used to extract that part of poly that is considered
			-- "gate poly", which is poly over drawn thin oxide and 0.1 um beyond
	    polbias := bias_differential (d, lvl.poly)
	    -- bias the non-native designs to create a pseudo-native poly
	    if is_scmos_die (d) and d.is_present (thox_lyr) then
		-- SCMOS w/ thick active: do a differential bias of the 3.3V gates
		--  to make a 3 lambda gate be TSMC minimal
		threel := 3 * d.design_techn.lambda	-- the SCMOS drawn 3.3V minimum gate
		almost_3l := threel - 0.02		-- filter for undersized gates
		grobias := (threel + polbias - 0.30 - 0.01).max (0)
			-- amount of overlap to create between poly_3vg and poly_other
			-- to guarantee that we do not create any poly gaps below
		-- split poly into 3vg: 3.3V gates that are at least 3 lambda wide
		--  and          other: all the rest, but with a grobias overlap
		add_ctemp ((((leaf (lyr.poly) #and# leaf (thox_lyr)) #and#
				(leaf (tox_lyr) #sized# (toxbias+grobias)))
				#sized# (-almost_3l)) #sized# almost_3l,	-- remove small
						tmp_poly_3vg, 0.005);
		add_ctemp (((leaf (lyr.poly) #minus# leaf (tmp_poly_3vg)) #sized# grobias)
				#and# leaf (lyr.poly),
						tmp_poly_other, 0.005);
		-- now size differentially and merge back together
		add_ctemp ((leaf (tmp_poly_other)
			    #or#
			    ((leaf (tmp_poly_3vg) #and# leaf (lyr.n_well))
				#sized# (0.30 - (threel + polbias)))	-- 3vg p-channel gates
			    #or#
			    ((leaf (tmp_poly_3vg) #minus# leaf (lyr.n_well))
				#sized# (0.35 - (threel + polbias)))	-- 3vg n-channel gates
			   ) #sized# polbias,
						tmp_poly_tsmc, 0.005);
		poly_src := tmp_poly_tsmc
	    elseif polbias /= 0 then
		add_ctemp (leaf (lyr.poly) #sized# polbias,
						tmp_poly_tsmc, 0.005);
		poly_src := tmp_poly_tsmc
	    else
		poly_src := lyr.poly
	    end;
	    -- now apply the native logic operation, which is a bloat of all field poly
	    -- (although TSMC calls it a poly endcap operation)
	    add_clevel (leaf (poly_src) #or#
			((leaf (poly_src) #minus# (leaf (tox_lyr) #sized# toxbias))
				#sized# 0.04), lvl.poly, d)
	end;

	rpo_mask (d: DIES_XROW) is
	local rpo_lyr: STRING; rpo_xtra: CATS_EXPRESSION
	do  rpo_lyr := source_layer_name(d, lvl.rpo)
	    if su.not_empty (rpo_lyr) and then d.is_present (rpo_lyr) then
		-- generate a little bit of extra RPO per TSMC logic op
		-- NOTE: the old ICL tsmc_025.family had a completely different RPO derivation;
		--	I cannot find any justification for it.
		rpo_xtra := ((leaf (act_layer) #and# leaf (lyr.poly)) #sized# 3.0)
				#and# (leaf (rpo_lyr) #sized# 2.0);
		rpo_xtra := rpo_xtra #minus# (leaf (lyr.poly) #sized# bias_differential (d, lvl.poly));
		rpo_xtra := rpo_xtra #minus# (leaf (lyr.contact) #sized# 0.6);
		rpo_xtra := ((rpo_xtra #or# leaf (rpo_lyr)) #sized# -1.49) #sized# 1.49;
		add_clevel_with_bias (leaf (rpo_lyr) #or# rpo_xtra, lvl.rpo, d)
	    end;
	end;

	use_alternate_ctm(d: DIES_XROW) is
		-- if this die has an alternate ctm layer, 
		--  fracture it instead of std CTM (aka, ctm4)
		-- (there can only be one CTM layer in a project)
	local alt_ctm: STRING
	do  -- look for an alternate ctm layer in the layout
	    if d.is_present (lyr.ctm2) then
		alt_ctm := lyr.ctm2
	    elseif d.is_present (lyr.ctm3) then
		alt_ctm := lyr.ctm3
	    end;
	    -- use in place of lyr.ctm on ctm mask
	    if alt_ctm /= Void then
		add_clevel_with_bias (leaf (alt_ctm), lvl.cap_top_metal, d)
	    end
	end;

	vtm_n_mask (d: DIES_XROW) is
	local vtmn_src: CATS_EXPRESSION
	do  -- start with n_well for everyone
	    vtmn_src := leaf (lyr.n_well)
	    if equal (wafer_dt, native_design_techn) then
		vtmn_src := vtmn_src #or# 
			    ((leaf (lyr.vtm_n) #or# leaf (lyr.vtd_n)) #or# leaf (lyr.nt_n))
	    end;
	    add_clevel_with_bias (vtmn_src, lvl.vtm_n, d)
	end;

	vtm_p_mask (d: DIES_XROW) is
	local vtmp_src: CATS_EXPRESSION
	do  -- start with n_well for everyone
	    vtmp_src := leaf (lyr.n_well)
	    if equal (wafer_dt, native_design_techn) then
		vtmp_src := vtmp_src #minus# 
			    (leaf (lyr.vtm_p) #or# leaf (lyr.vtd_p))
	    end;
	    add_clevel_with_bias (vtmp_src, lvl.vtm_p, d)
	end;

end
