
indexing
	
	description: "TSMC_035 wafer techn family"
	author: "David Hollenberg";
	date: "June 18, 2003"

class TSMC_035
inherit
	WAFER_TECHN_FAMILY
		redefine
			add_explicit_cats_statements
		end;

feature -- Properties

	name: STRING is "TSMC_035"

	native_design_techn: STRING is
			-- name of the "native" design_techn
	do  Result := dtv.tsmc35_p2_new
	end;

feature	-- statements for Cats fracture script

	add_explicit_cats_statements (d: DIES_XROW) is
	do	-- establish cached data
		act_layer := source_layer_name (d, lvl.active);	-- OD
		act_bdiff := bias_differential (d, lvl.active)
		-- same routines for all design_technologies
		well_merge (d);
		passport_active (d);
		n_and_p_select (d);
		poly_mask (d);
		if option_is_present (oid.x5v) then
			thick_ox_and_nldd (d)
		end;
		if option_is_present (oid.po2l) then
			po2l_mask (d)
		end;
	end;

feature {NONE} -- cached data

	act_layer: STRING
		-- we want this one all over the place; stash here once
	act_bdiff: DOUBLE
		-- bias differential for act_layer

feature {NONE} -- statement generators

	well_merge (d: DIES_XROW) is
	local nw, pw: CATS_EXPRESSION
	do
	    -- nwell
	    nw := leaf (source_layer_name (d, lvl.n_well))
	    add_clevel_with_merge (nw, lvl.n_well, 0.95, d)
	    -- pwell: source is n_well, but then maybe modified
	    pw := nw
	    if d.is_present (lyr.expl_field_implant) then
		pw := pw #xor# leaf (lyr.expl_field_implant)	-- SCMOS only
	    end;
	    add_clevel_with_merge (pw, lvl.p_well, 0.95, d)
	end;

	passport_active (d: DIES_XROW) is
	do  -- passport has separate n_diffusion and p_diffusion layers
	    if equal (d.wafer_design_techn, dtv.tsmc35_passport) then
		add_clevel_with_bias (leaf (lyr.n_diffusion) #or#
					leaf (lyr.p_diffusion),
					lvl.active, d)
	    end;
	end;

	n_and_p_select (d: DIES_XROW) is
	local lnp, lpp: STRING hnp, hpp: BOOLEAN; xnp, xpp: CATS_EXPRESSION
	do  -- get select layer names
	    lnp := source_layer_name (d, lvl.n_plus_select);
	    lpp := source_layer_name (d, lvl.p_plus_select);
	    -- look for presence in layout
	    hnp := d.is_present (lnp);
	    hpp := d.is_present (lpp);
	    -- possible synthesis of missing select from the other one
	    if hpp and not hnp then
		xnp := #not# leaf (lpp)
	    else
		xnp := leaf (lnp)
	    end;
	    if hnp and not hpp then
		xpp := #not# leaf (lnp)
	    else
		xpp := leaf (lpp)
	    end;
	    -- fracture them
	    add_clevel_with_merge_X2(xnp, lvl.n_plus_select, 0.55, d);
	    add_clevel_with_merge_X2(xpp, lvl.p_plus_select, 0.55, d);
	end;

	poly_mask (d: DIES_XROW) is
	local lpoly: STRING; mpoly, xpoly: CATS_EXPRESSION;
	do  lpoly := source_layer_name (d, lvl.poly);
	    mpoly := leaf (lpoly);
	    -- for SCMOS, we first resize the 5V poly gates
	    if is_scmos_die (d) then
		mpoly := scmos_5V_poly(mpoly, d)
	    end;
	    -- now, for everyone, we create poly "endcaps" (bloat field poly)
	    -- from mpoly (the modified drawn poly)
	    xpoly := mpoly
	    if d.is_present (lyr.esexcl) then
		-- region excluded from endcap creation
		xpoly := mpoly #minus# leaf (lyr.esexcl)
	    end;
	    add_clevel_with_bias (mpoly #or#
				  ((xpoly #minus# (leaf (act_layer) #sized# (0.05 + act_bdiff)))
				   #sized# 0.05), lvl.poly, d)
	end;

	tmp_poly_5vg: STRING is "POLY_5VG"
	tmp_poly_xxx: STRING is "POLY_XXX"
	tmp_poly_scmos: STRING is "POLY_SCMOS"

	scmos_5V_poly (mpoly: CATS_EXPRESSION;
			d: DIES_XROW): CATS_EXPRESSION is
		-- take drawn mpoly and rebias the 5V gates
		-- (we want to take a 3 lambda 5V gate to the TSMC 5V minimum
		--  gate size of 0.50 um)
	require is_scmos: is_scmos_die (d)
	local poly_bdiff, threel, almost_3l, ovrlap: DOUBLE;
		xxx: CATS_EXPRESSION
	do  
	    if d.is_present (lyr.thick_active) then
		poly_bdiff := bias_differential (d, lvl.poly);
		threel := 3 * d.design_techn.design_lambda.to_double;
		almost_3l := threel - 0.05;
		ovrlap := (threel - 0.50) + 0.05;
		-- extract the 5V gate poly
		xxx := mpoly #and# leaf (lyr.thick_active)
			#and# (leaf (act_layer) #sized# (ovrlap + act_bdiff));
		-- eliminate the stuff smaller than 3 lambda (shouldn't be any)
		add_ctemp (xxx #sized# (-almost_3l) #sized# almost_3l,
			   tmp_poly_5vg, 0.0125);
		-- now extract the rest of the poly, with overlap to avoid gapping
		add_ctemp (((mpoly #minus# leaf (tmp_poly_5vg)) #sized# ovrlap)
			   #and# mpoly,
			   tmp_poly_xxx, 0.0125);
		-- and put them back together
		add_ctemp (leaf (tmp_poly_xxx) #or#
			   (leaf (tmp_poly_5vg) #sized# (0.50 - threel)),
			   tmp_poly_scmos, 0.0125);
		Result := leaf (tmp_poly_scmos)
	    else
		Result := mpoly
	    end
	end;

	thick_ox_and_nldd (d: DIES_XROW) is
	local od2_lyr, n3v_lyr, n5v_lyr, npl_lyr: STRING
	do  od2_lyr := source_layer_name (d, lvl.active_5v);
	    n3v_lyr := source_layer_name (d, lvl.nldd_3v);
	    n5v_lyr := source_layer_name (d, lvl.nldd_5v);
	    npl_lyr := source_layer_name (d, lvl.n_plus_select);
	    if su.is_empty(od2_lyr) then
		-- technology is not aware of OD2; default statements OK
	    else
		-- technology is OD2 aware; synthesize nldd masks if not drawn
		if not d.is_present (n3v_lyr) then
			add_clevel_with_bias (leaf (npl_lyr) #minus# leaf (od2_lyr),
					  lvl.nldd_3v, d)
		end;
		if not d.is_present (n5v_lyr) then
			add_clevel_with_bias (leaf (npl_lyr) #and# leaf (od2_lyr),
					  lvl.nldd_5v, d)
		end;
	    end
	end;

	po2l_mask (d: DIES_XROW) is
	local po2l: CATS_EXPRESSION
	do  if d.is_present (lyr.po2l) then
		-- just use it
		po2l := leaf (lyr.po2l)
	    elseif d.is_present (lyr.poly2) then
		-- lo res guy needs po2l over all poly2
		po2l := leaf (lyr.poly2) #sized# 0.9
		if d.is_present (lyr.hi_res_implant) then
			-- except hi_res_implant is inverse of po2l
			po2l := (po2l #minus# leaf (lyr.hi_res_implant))
					#sized# -0.4 #sized# 0.4
		end
	    else
		-- single poly guy needs nothing
		po2l := emptiness
	    end;
	    add_clevel_with_merge (po2l, lvl.po2l, 0.8, d)
	end;

end
