
indexing
	
	description: "Window to allow user to set tapeout date and estimate %
		%turnaround dates for a run"
	author: "David Hollenberg"
	date: "March 29, 2004"

class TURNAROUND_DATES_WINDOW

inherit
	TURNAROUND_DATES_IFC
	WINDOW_INTERFACE
	MOSIS_GRAPHICAL_UTILITIES
		export
			{NONE} all
		undefine
			default_create
		end
		
create
	make

feature {NONE} -- Creation

	make (w: EV_TITLED_WINDOW)
			-- Create interface in window `w'
		require
			window_exists: w /= Void
			window_valid: not w.is_destroyed
		do
			make_interface
			make_proxy (w)
		end

feature -- Operations

	reset
			-- Reset interface
		do
		end

feature {NONE} -- Implementation

	build_interface
			-- Build interface in `window'
		local
			main_vbox: EV_VERTICAL_BOX
			buttons: HORIZONTAL_BUTTON_SET
			box: EV_HORIZONTAL_BOX
			sep: EV_HORIZONTAL_SEPARATOR
		do
			fetch_data
			create main_vbox
			main_vbox.set_border_width (user_interface.border_width)
			
			-- Build display for tapeout date

			build_tapeout_date
			main_vbox.extend (last_box)
			main_vbox.disable_item_expand (last_box)
			create sep
			main_vbox.extend (sep)
			main_vbox.disable_item_expand (sep)
			
			-- Build display for wafer lots

			build_wafer_lots
			main_vbox.extend (last_box)
			main_vbox.disable_item_expand (last_box)
			create sep
			main_vbox.extend (sep)
			main_vbox.disable_item_expand (sep)

			-- Build display for packaging lots

			build_packaging_lots
			main_vbox.extend (last_box)
			main_vbox.disable_item_expand (last_box)

			
			create buttons.make_with_text_and_actions (<< gui.ev_ok, gui.ev_cancel >>, << agent process_accept, agent process_decline >>)
			box := buttons.column_set.box
			
			main_vbox.extend (box)
			main_vbox.disable_item_expand (box)
			gui.set_box_background_recursive (main_vbox, background_color)
			extend (main_vbox)

			close_request_actions.extend (agent process_decline)
			gui.add_cancel_accelerator (window, agent process_decline)
			gui.do_once_on_idle (agent process_date_tapeout_change)
			enable_active
		end
		
	fetch_data
			-- Fetch needed data from database
		do
			if state /= Void then
				tapeout_date := state.tapeout_date
			else
				tapeout_date := cache.mask_sets_row.date_tapeout
				if tapeout_date.is_null then
					tapeout_date := connection.system_date
				end
				tapeout_date := tapeout_date.truncated_to_midnight
			end
			mask_turn_days := cache.wafer_mask_techns_row.non_null_nominal_turnaround_days
			masks_out_date := aligned_to_work_day (connection, tapeout_date, mask_turn_days)
			wafer_lots := cache.wafer_lot_mrows
			packaging_lots := cache.packaging_lot_mrows
		end

	build_tapeout_date
			-- Build portion of window with tapeout date
			-- and leave in `last_box'
		local
			cols: WIDGET_COLUMN_SET
		do
			create cols.make (2)
			cols.set_padding (user_interface.padding)
			cols.set_border_width (user_interface.border_width)
			create tapeout_date_field.make_with_text (tapeout_date.to_date_string)
			tapeout_date_field.enable_uppercase_only
			tapeout_date_field.set_minimum_width_in_characters (Max_date_width)
			tapeout_date_field.change_actions.extend (agent process_date_tapeout_change)
			cols.append_row (<< gui.label_with_text ("Date tapeout"), tapeout_date_field >>)
			cols.disable_expand
			last_box := cols.box
		end
		
	build_wafer_lots
			-- Build portion of window with wafer lots
			-- and leave in `last_box'
		local
			cols: WIDGET_COLUMN_SET
			wafer_lot: RUN_WAFER_LOTS_ROW
			process_field, date_field, turn_field: EV_TEXT_FIELD
			action: PROCEDURE [ANY, TUPLE]
			date: DB_DATE
			process_name: STRING
		do
			create wafer_lot_actions
			create wafer_lot_received_dates.make (5)
			wafer_lot_received_dates.compare_objects
			create wafer_lot_sent_dates.make (5)
			wafer_lot_sent_dates.compare_objects
			create {ARRAYED_LIST [EV_TEXT_FIELD]} wafer_lot_date_fields.make (5)
			create cols.make (3)
			cols.align_columns_left
			cols.set_border_width (user_interface.border_width)
			cols.set_padding (user_interface.padding)
			cols.append_row (<< gui.label_with_text ("Turnaround Calendar Days"), gui.label_with_text ("Process"), gui.label_with_text ("Est Date Wafers Received") >>)
			from
				wafer_lots.start
			until
				wafer_lots.after
			loop
				wafer_lot := wafer_lots.item
				process_name := wafer_lot.wafer_process
				turn_field := new_field (process_turn_days (process_name).out, 3)
				turn_field.enable_edit
				turn_field.set_data (wafer_lot)
				process_field := new_field (process_name, 22)
				date_field := new_date_field
				date_field.set_data (wafer_lot)
				action := agent process_wafer_turn_change (turn_field, date_field)
				turn_field.change_actions.extend (action)
				wafer_lot_actions.extend (action)
				cols.append_row (<< turn_field, process_field, date_field >>)
				date := connection.null_date
				wafer_lot_received_dates.put (date, process_name)
				date := connection.null_date
				wafer_lot_sent_dates.put (date, process_name)
				wafer_lot_date_fields.extend (date_field)
				wafer_lots.forth
			end
			cols.disable_expand
			cols.disable_column_expand (1)
			cols.disable_column_expand (3)
			last_box := cols.box
		end
		
	build_packaging_lots
			-- Build portion of window with packaging lots
			-- and leave in `last_box'
		local
			cols: WIDGET_COLUMN_SET
			packaging_lot: RUN_PACKAGING_LOTS_ROW
			process_field, date_field, turn_field: EV_TEXT_FIELD
			packager_field, order_type_field: EV_TEXT_FIELD
			process_name: STRING
			action: PROCEDURE [ANY, TUPLE]
			list: ACTION_SEQUENCE [TUPLE]
		do
			create packaging_lot_actions.make (5)
			packaging_lot_actions.compare_objects
			create {ARRAYED_LIST [EV_TEXT_FIELD]} packaging_lot_date_fields.make (5)
			create cols.make (5)
			cols.align_columns_left
			cols.set_border_width (user_interface.border_width)
			cols.set_padding (user_interface.padding)
			cols.append_row (<< gui.label_with_text ("Turnaround Calendar Days"), gui.label_with_text ("Packager"), gui.label_with_text ("Order Type"), gui.label_with_text ("Process"), gui.label_with_text ("Est Date Lot Received") >>)
			from
				packaging_lots.start
			until
				packaging_lots.after
			loop
				packaging_lot := packaging_lots.item
				process_name := packaging_lot.non_null_wafer_process
				turn_field := new_field (packaging_type_turn_days (packaging_lot.packaging_org_no, packaging_lot.packaging_order_type, process_name).out, 3)
				turn_field.enable_edit
				turn_field.set_data (packaging_lot)
				packager_field := new_field (short_id (packaging_lot.packaging_org_no), 5)
				order_type_field := new_field (packaging_lot.packaging_order_type, 8)
				process_field := new_field (process_name, 22)
				date_field := new_date_field
				date_field.set_data (packaging_lot)
				action := agent process_packaging_turn_change (turn_field, date_field)
				packaging_lot_actions.search (process_name)
				if packaging_lot_actions.found then
					packaging_lot_actions.found_item.extend (action)
				else
					create list
					list.extend (action)
					packaging_lot_actions.put (list, process_name)
				end
				turn_field.change_actions.extend (action)
				cols.append_row (<< turn_field, packager_field, order_type_field, process_field, date_field >>)
				packaging_lot_date_fields.extend (date_field)
				packaging_lots.forth
			end
			
			-- Add null packaging lot action for every process
			-- that does not have any packaging lots

			from
				wafer_lots.start
			until
				wafer_lots.after
			loop
				process_name := wafer_lots.item.wafer_process

				packaging_lot_actions.search (process_name)
				if not packaging_lot_actions.found then
					create list
					list.extend (agent do_nothing)
					packaging_lot_actions.put (list, process_name)
				end
				wafer_lots.forth
			end

			cols.disable_expand
			last_box := cols.box
		end
		
	process_date_tapeout_change
			-- Take actions needed when text in
			-- `tapeout_date_field' changes
		do
			if valid_date (tapeout_date_field.text) then
				tapeout_date := connection.date (tapeout_date_field.text)
				masks_out_date := aligned_to_work_day (connection, tapeout_date, mask_turn_days)
			else
				tapeout_date.set_null
				masks_out_date.set_null
			end
			wafer_lot_actions.call ([])
				-- Update received dates for wafer lots
				-- and their packaging lots
		end

	process_wafer_turn_change (turn_field, date_field: EV_TEXT_FIELD)
			-- Take actions needed when text in
			-- wafer lot turnaround days field `turn_field'
			-- changes, where `date_field' has the corresponding
			-- date
		local
			val, process_name: STRING
			received_date, sent_date: DB_DATE
			wafer_lot: RUN_WAFER_LOTS_ROW
			operation: ANONYMOUS_OPERATION
		do
			wafer_lot ?= turn_field.data
			check
				wafer_lot_exists: wafer_lot /= Void
			end
			process_name := wafer_lot.wafer_process
			received_date := date_plus_days (tapeout_date, turn_field.text)
			sent_date := date_plus_days (received_date, "1")
			if not received_date.is_null then
				val := received_date.to_date_string
			else
				val := ""
			end
			date_field.set_text (val)
			wafer_lot_received_dates.force (received_date, process_name)
			wafer_lot_sent_dates.force (sent_date, process_name)
			packaging_lot_actions.search (process_name)
			if packaging_lot_actions.found then
				packaging_lot_actions.found_item.call ([])
			else
				create operation.make (agent handle_hard_error ("cannot find packaging lot actions for process " + process_name))
				operation.enable_abort_caught
				operation.execute
			end
		end

	process_packaging_turn_change (turn_field, date_field: EV_TEXT_FIELD)
			-- Take actions needed when text in
			-- packaging lot turnaround days field `turn_field'
			-- changes, where `date_field' has the corresponding
			-- date
		local
			val, process_name: STRING
			wafers_sent_date, date: DB_DATE
			packaging_lot: RUN_PACKAGING_LOTS_ROW
			operation: ANONYMOUS_OPERATION
		do
			packaging_lot ?= turn_field.data
			check
				packaging_lot_exists: packaging_lot /= Void
			end
			process_name := packaging_lot.non_null_wafer_process
			wafer_lot_sent_dates.search (process_name)
			if wafer_lot_sent_dates.found then
				wafers_sent_date := wafer_lot_sent_dates.found_item
				date := date_plus_days (wafers_sent_date, turn_field.text)
				if not date.is_null then
					val := date.to_date_string
				else
					val := ""
				end
				date_field.set_text (val)
			else
				create operation.make (agent handle_hard_error ("cannot find wafer lot sent date for process " + process_name))
				operation.enable_abort_caught
				operation.execute
			end
		end

	process_accept
			-- Take actions needed when user accepts
			-- current operation
		local
			operation: ANONYMOUS_OPERATION
		do
			if is_active then
				disable_active
				create operation.make (agent process_accept_request)
				operation.enable_abort_caught
				operation.execute
				if operation.aborted then
					session.rollback
					enable_active
				else
					success_actions.call ([])
				end
			end
		end

	process_accept_request
			-- Process an accept request
		do
			check_dates_valid
			warn_about_extreme_dates
			set_dates
		end

	check_dates_valid
			-- Check that tapeout date, all wafer lot received
			-- dates and all packaging lot received dates
			-- are present and valid
		do
			check_date_valid (tapeout_date_field, "Tapeout date")
			wafer_lot_date_fields.do_all (agent check_wafer_lot_date_valid)
			packaging_lot_date_fields.do_all (agent check_packaging_lot_date_valid)
		end

	check_wafer_lot_date_valid (field: EV_TEXT_FIELD)
			-- Check that `field' contains a valid wafer lot date.
			-- Issue error if not
		require
			field_exists: field /= Void
		local
			wafer_lot: RUN_WAFER_LOTS_ROW
			received_date, accept_date: DB_DATE
		do
			wafer_lot ?= field.data
			check
				wafer_lot_exists: wafer_lot /= Void
			end
			check_date_valid (field, "Wafers received date for " + wafer_lot.wafer_process)
			-- Received date is valid.  Now make sure it is
			-- not later than date accepted
		
			if not wafer_lot.is_null_date_accepted then
				received_date := connection.date (field.text).truncated_to_midnight
				accept_date := wafer_lot.date_accepted.truncated_to_midnight
				if accept_date < received_date then
				
					handle_user_error ("Wafers received date " + received_date.to_date_string + " is after accept date " + accept_date.to_date_string)
				end
			end
		end

	check_packaging_lot_date_valid (field: EV_TEXT_FIELD)
			-- Check that `field' contains a valid packaging
			-- lot date.  Issue error if not
		require
			field_exists: field /= Void
		local
			packaging_lot: RUN_PACKAGING_LOTS_ROW
		do
			packaging_lot ?= field.data
			check
				packaging_lot_exists: packaging_lot /= Void
			end
			check_date_valid (field, "Packaging lot received date for " + packaging_lot.packaging_order_type + " lot for " +  packaging_lot.non_null_wafer_process)
		end


	check_date_valid (date: EV_TEXT_FIELD name: STRING)
			-- Check that `date' which is labeled `name'
			-- is present and valid.  Issue error if not
		require
			date_exists: date /= Void
			name_exists: name /= Void
		local
			s: STRING
		do
			s := date.text
			if not valid_date (s) then
				handle_user_error (name + " is not valid")
			end
		end

	warn_about_extreme_dates
			-- Warn about any dates that are out of the
			-- expected range
		do
			check_date_range (tapeout_date_field, 0, 0)
			wafer_lot_date_fields.do_all (agent check_wafer_lot_date_range)
			packaging_lot_date_fields.do_all (agent check_packaging_lot_date_range)
		end

	check_wafer_lot_date_range (field: EV_TEXT_FIELD)
			-- Issue warning if date in `field' is out of
			-- expected range
		require
			field_exists: field /= Void
		local
			wafer_lot: RUN_WAFER_LOTS_ROW
		do
			wafer_lot ?= field.data
			check
				wafer_lot_exists: wafer_lot /= Void
			end
			check_date_range (field, 0, Max_days)
		end

	check_packaging_lot_date_range (field: EV_TEXT_FIELD)
			-- Issue warning if date in `field' is out of
			-- expected range
		require
			field_exists: field /= Void
		local
			packaging_lot: RUN_PACKAGING_LOTS_ROW
		do
			packaging_lot ?= field.data
			check
				packaging_lot_exists: packaging_lot /= Void
			end
			check_date_range (field, 0, Max_days)
		end

	check_date_range (date: EV_TEXT_FIELD start_days, end_days: INTEGER)
			-- Check that `date' is in the range
			-- [today + `start_days', today + `end_days']
			-- Issue warning if not
		require
			date_exists: date /= Void
		do
			check_date_in_range (date.text, connection, start_days, end_days)
		end

	set_dates
			-- Update dates in database
		local
			lock_op: ROW_LOCK_OP
			list: like wafer_lot_date_fields
			field: EV_TEXT_FIELD
			wafer_lot: RUN_WAFER_LOTS_MROW
			packaging_lot: RUN_PACKAGING_LOTS_MROW
			runs_row: RUNS_MROW
			mask_sets_row: MASK_SETS_MROW
			productions_row: PRODUCTIONS_MROW
			process_name: STRING
			event_row: RUN_EVENTS_ROW
		do
			runs_row := cache.runs_mrow
			productions_row := cache.productions_mrow
			mask_sets_row := cache.mask_sets_mrow
			create lock_op
			lock_op.lock_nowait (runs_row)
			lock_op.lock_nowait (productions_row)
			lock_op.lock_nowait (mask_sets_row)
			lock_op.lock_list_nowait (wafer_lots)
			lock_op.lock_list_nowait (packaging_lots)
			
			mask_sets_row.set_date_tapeout (tapeout_date)
			mask_sets_row.set_null_date_masks_in
			mask_sets_row.set_date_masks_out (masks_out_date)
			productions_row.set_data_entry_username (connection.user_name)
			productions_row.set_date_ta_dates_modified (connection.transaction_date)
			create event_row.make_template
			event_row := event_row.run_closed_events (session.run_id, connection, expected_count.Exactly_one).first
			productions_row.set_date_production_closed (event_row.date_event)
			
			from
				list := wafer_lot_date_fields
				list.start
			until
				list.after
			loop
				field := list.item
				wafer_lot ?= field.data
				check
					wafer_lot_exists: wafer_lot /= Void
				end
				process_name := wafer_lot.wafer_process
				wafer_lot.set_date_received (wafer_lot_received_dates.item (process_name))
				list.forth
			end
			
			from
				list := packaging_lot_date_fields
				list.start
			until
				list.after
			loop
				field := list.item
				packaging_lot ?= field.data
				check
					packaging_lot_exists: packaging_lot /= Void
				end
				process_name := packaging_lot.non_null_wafer_process
				packaging_lot.set_date_sent (wafer_lot_sent_dates.item (process_name))
				packaging_lot.set_is_actual_date_sent (False)
				packaging_lot.set_date_received (connection.date (field.text))
				packaging_lot.set_is_actual_date_received (False)
				list.forth
			end
		end

	process_decline
			-- Take actions needed when user declines
			-- current operation
		do
			if is_active then
				disable_active
				abort_actions.call ([])
			end
		end

	new_field (s: STRING min_width: INTEGER): EV_TEXT_FIELD
			-- New text field with initial text `s' with
			-- minimum width in characters `min_width' that
			-- is not editable
		do
			create Result.make_with_text (s)
			Result.set_minimum_width_in_characters (min_width)
			Result.disable_edit
		ensure
			result_exists: Result /= Void
		end

	new_date_field: EV_TEXT_FIELD
			-- New text field that can hold a date
		do
			Result := new_field ("", Max_date_width)
		ensure
			result_exists: Result /= Void
		end

	short_id (org_no: INTEGER): STRING
			-- Short id for organization whose number is `org_no'
		local
			b: DB_BIND_I
		do
			create b.make (org_no)
			Result := connection.selected_string ("short_id", "organizations", "org_no = :1", b)
		end

	date_plus_days (date: DB_DATE n: STRING): DB_DATE
			-- `date' plus `n' days, aligned with work day.
			-- If `n' is not an integer or `date' is null,
			-- returns null date
		require
			date_exists: date /= Void
			days_exists: n /= Void
		local
			day_count: INTEGER
		do
			if  not date.is_null and n.is_integer then
				day_count := n.to_integer
				if is_valid_turn_days (day_count) then
					Result := aligned_to_work_day (connection, date, day_count)
				else
					Result := connection.null_date
				end
			else
				Result := connection.null_date
			end
		ensure
			result_exists: Result /= Void
		end

	is_valid_turn_days (n: INTEGER): BOOLEAN
			-- Is `n' a valid number of turnaround days?
		do
			Result := n >= 0 and n < 1000
				-- If it is too big, it is certainly wrong.
				-- If it is really big, Oracle will get
				-- sick over it
		end

	process_turn_days (proc_name: STRING): INTEGER
			-- Nominal number of calendar days required to
			-- produce wafers from tapeout for `proc_name'
		require
			process_name_exists: proc_name /= Void
		local
			b: DB_BIND_S
			found: BOOLEAN
			list: LIST [WAFER_LOT_STATE]
			lot: WAFER_LOT_STATE
		do
			if state /= Void then
				from
					list := state.wafer_lots
					list.start
				until
					list.after or found
				loop
					lot := list.item
					if lot.wafer_process.is_equal (proc_name) then
						found := True
						Result := connection.date_difference (lot.date_received, tapeout_date).rounded
					end
					list.forth
				end
			end
			if not found then
				-- No saved state or no wafer lot for `proc_name'
				create b.make (proc_name)
				Result := connection.selected_integer ("nominal_turnaround_days", "wafer_processes", "wafer_process = :1", b) + mask_turn_days
			end
		end

	packaging_type_turn_days (org_no: INTEGER order_type, proc_name: STRING): INTEGER
			-- Nominal number of calendar days required to
			-- package wafers for an `order_type' packaging order
			-- for `proc_name' with packager identified by `org_no'
		require
			process_name_exists: proc_name /= Void
			order_type_exists: order_type /= Void
		local
			b: DB_BIND_S
			found: BOOLEAN
			list: LIST [PACKAGING_LOT_STATE]
			lot: PACKAGING_LOT_STATE
		do
			if state /= Void then
				from
					list := state.packaging_lots
					list.start
				until
					list.after or found
				loop
					lot := list.item
					if lot.wafer_process.is_equal (proc_name) and lot.packaging_order_type.is_equal (order_type) and lot.packaging_org_no = org_no then
						found := True
						Result := connection.date_difference (lot.date_received, lot.date_sent).rounded
					end
					list.forth
				end
			end
			if not found then
				-- No saved state or no packaging lot for `proc_name'
				create b.make (order_type)
				Result := connection.selected_integer ("nominal_turnaround_days", "packaging_order_types", "packaging_order_type = :1", b)
			end
		end

	wafer_lots: LIST [RUN_WAFER_LOTS_MROW]
			-- List of wafer lots for run

	packaging_lots: LIST [RUN_PACKAGING_LOTS_MROW]
			-- List of packaging lots for run
			
	tapeout_date_field: TEXT_FIELD
			-- Text field for tapeout date

	mask_turn_days: INTEGER
			-- Nominal number of calendar days required to produce
			-- masks for run

	tapeout_date: DB_DATE
			-- Date of tapeout (files sent to mask shop0

	masks_out_date: DB_DATE
			-- Estimated (theoretical) date masks went out
			-- from MOSIS to foundry

	wafer_lot_received_dates: HASH_TABLE [DB_DATE, STRING]
			-- Estimated date wafers received for wafer lots,
			-- indexed by process name (assuming only one lot
			-- per process)

	wafer_lot_sent_dates: HASH_TABLE [DB_DATE, STRING]
			-- Estimated date wafers sent for wafer lots,
			-- indexed by process name (assuming only one lot
			-- per process)

	wafer_lot_actions: ACTION_SEQUENCE [TUPLE]
			-- Actions to be take to update all wafer lot
			-- received dates (and their packaging lot received
			-- dates)

	packaging_lot_actions: HASH_TABLE [ACTION_SEQUENCE [TUPLE], STRING]
			-- Lists of actions to update packaging lot received
			-- dates, indexed by wafer process

	wafer_lot_date_fields: LIST [EV_TEXT_FIELD]
			-- List of date received fields for wafer lots

	packaging_lot_date_fields: LIST [EV_TEXT_FIELD]
			-- List of date received fields for packaging lots

	last_box: EV_HORIZONTAL_BOX
			-- Last box built

	Max_date_width: INTEGER = 11
			-- Maximum width of date in characters

	Max_days: INTEGER = 180
			-- Maximum number of days from today until
			-- date without a warning

end
