
indexing
	
	author: "David Hollenberg"
	date: "October 8, 1997"
	description: "An unnamed Unix pipe used for interprocess communication"

class UNIX_PIPE

inherit
	ANY
	MEMORY
		export
			{NONE} all
		redefine
			dispose
		end
	UNIX_OS
		export
			{ANY} valid_file_descriptor
			{NONE} all
		end
create
	make

feature {UNIX_OS} -- Creation

	make (read_fd, write_fd: INTEGER)
			-- Create a pipe object which represents the
			-- pipe with read file descriptor `read_fd' and
			-- write file descriptor `write_fd'
		require
			valid_read_desc: valid_file_descriptor (read_fd)
			valid_write_desc: valid_file_descriptor (write_fd)
		do
			read_descriptor := read_fd
			write_descriptor := write_fd
		ensure
			read_desc_set: read_descriptor = read_fd
			write_desc_set: write_descriptor = write_fd
		end

feature -- Attributes

	read_descriptor: INTEGER
			-- Descriptor to be used for reading from pipe

	write_descriptor: INTEGER
			-- Descriptor to be used for writing to pipe

feature -- Modification

	close_read_descriptor
			-- Close descriptor to be used for reading from pipe,
			-- if it is open
		do
			if read_descriptor /= Invalid_file_descriptor then
				close_file_descriptor (read_descriptor)
				erase_read_descriptor
			end
		end

	close_write_descriptor
			-- Close descriptor to be used for write to pipe,
			-- if it is open
		do
			if write_descriptor /= Invalid_file_descriptor then
				close_file_descriptor (write_descriptor)
				erase_write_descriptor
			end
		end

	erase_read_descriptor
			-- Set `read_descriptor' to an invalid value
			-- so it won't be closed if object is GC'ed.
			-- Caller must take responsiblity for ensuring
			-- that descriptor is closed when no longer needed
		do
			read_descriptor := Invalid_file_descriptor
		end

	erase_write_descriptor
			-- Set `write_descriptor' to an invalid value
			-- so it won't be closed if object is GC'ed.
			-- Caller must take responsiblity for ensuring
			-- that descriptor is closed when no longer needed
		do
			write_descriptor := Invalid_file_descriptor
		end

feature {NONE} -- Cleanup

	dispose
		do
			if read_descriptor /= Invalid_file_descriptor then
				close_file_descriptor_without_exception (read_descriptor)
				erase_read_descriptor
			end
			if write_descriptor /= Invalid_file_descriptor then
				close_file_descriptor_without_exception (write_descriptor)
				erase_write_descriptor
			end
		end

end
