class VALUE_INTERVAL [G -> COMPARABLE]
	-- a closed range of values, possibly empty, of G

create
	make_empty, make_interval

feature	-- the emptiness of the range

	is_empty: BOOLEAN

feature	-- the range, as defined by its extrema
	-- (meaningless if is_empty)

	low,
	high: G

feature	-- inclusion test

	contains(val: G): BOOLEAN is
		-- is val within the interval?
	require have_val: val /= Void
	do  Result := (not is_empty) and then
			low <= val and val <= high
	end;

feature	-- extension

	extend(val: G) is
		-- extend interval (minimally) to include val
	require have_val: val /= Void
	do  if is_empty
	     then make_interval(val, val)
	    elseif val < low
	     then low := val
	    elseif val > high
	     then high := val
	    end
	ensure extended: contains(val)
	end;

feature	-- creation

	make_empty is
		-- make an empty interval (contains nothing)
	do  is_empty := true
	ensure is_empty: is_empty
	end;

	make_interval(val1, val2: G) is
		-- make a true interval (contains at least one value)
		-- of all values between val1 and val2
	require have_interval: val1 /= Void and val2 /= Void
	do  is_empty := false;
	    low := val1.min(val2);
	    high := val1.max(val2)
	ensure extended: contains(val1) and contains(val2)
	end;

invariant
	proper_range: is_empty or else (high >= low)

end -- INTERVAL
