
indexing

	author: "Generated"
	date: "Thu Jun 12 13:50:27 PDT 2003"

class VENDOR_PART_IDS_ROW
inherit
	VENDOR_PART_IDS_RAW
	ROW_COUNT_CHECK_ACCESS
create
	make_template, make_null, make_by_key

feature -- Properties
	
	row_for_seq_and_part (seq, part: STRING conn: DB_CONNECTION range: ITEM_COUNT_RANGE): DYNAMIC_LIST [like Current] is
			-- Row for next vendor part id from sequence `seq'
			-- that has part id source `part'
		require
			sequence_exists: seq /= Void
			part_exists: part /= Void
			connection_exists: conn /= Void
			connection_open: conn.is_open
			range_exists: range /= Void
		local
			b: DB_BIND_SS
		do
			create b.make (seq, part)
			Result := all_rows_filtered (conn, "vendor_part_sequence_id = :1 and vendor_part_id_source = :2", b)
			check_row_count (Result, range)
		ensure
			result_exists: Result /= Void
		end

	next_vendor_part_ids (seq: STRING conn: DB_CONNECTION range: ITEM_COUNT_RANGE): DYNAMIC_LIST [like Current] is
			-- Row for next unassigned vendor part id
			-- from sequence `seq' (empty list if no unassigned
			-- part ids left in sequence)
		require
			sequence_exists: seq /= Void
			connection_exists: conn /= Void
			connection_open: conn.is_open
			range_exists: range /= Void
		local
			b: DB_BIND_SB
		do
			create b.make (seq, False)
			Result := all_rows_filtered (conn, "vendor_part_sequence_id = :1 and vpi.row_no = %N  (select min(vpi1.row_no) from vendor_part_ids vpi1%N   where vpi1.assigned = :2 and vpi1.vendor_part_sequence_id = :1)", b)
			check_row_count (Result, range)
		ensure
			result_exists: Result /= Void
		end

	assigned_part_id_rows (seq: STRING conn: DB_CONNECTION range: ITEM_COUNT_RANGE): DYNAMIC_LIST [like Current] is
			-- All rows for part ids in `seq' which are
			-- marked "assigned"
		require
			sequence_exists: seq /= Void
			connection_exists: conn /= Void
			connection_open: conn.is_open
			range_exists: range /= Void
		do
			Result := part_id_rows_for_seq (seq, True, conn, range)
		ensure
			result_exists: Result /= Void
		end

	unassigned_part_id_rows (seq: STRING conn: DB_CONNECTION range: ITEM_COUNT_RANGE): DYNAMIC_LIST [like Current] is
			-- All rows for part ids in `seq' which are
			-- marked "unassigned"
		require
			sequence_exists: seq /= Void
			connection_exists: conn /= Void
			connection_open: conn.is_open
			range_exists: range /= Void
		do
			Result := part_id_rows_for_seq (seq, False, conn, range)
		ensure
			result_exists: Result /= Void
		end

	assigned_part_ids (seq: STRING conn: DB_CONNECTION range: ITEM_COUNT_RANGE): DYNAMIC_LIST [STRING] is
			-- All part ids in `seq' which are marked
			-- "assigned"
		require
			sequence_exists: seq /= Void
			connection_exists: conn /= Void
			connection_open: conn.is_open
			range_exists: range /= Void
		local
			list: like assigned_part_id_rows
		do
			list := assigned_part_id_rows (seq, conn, range)
			Result := part_list_from_row_list (list)
		ensure
			result_exists: Result /= Void
		end

	unassigned_part_ids (seq: STRING conn: DB_CONNECTION range: ITEM_COUNT_RANGE): DYNAMIC_LIST [STRING] is
			-- All part ids in `seq' which are marked
			-- "unassigned"
		require
			sequence_exists: seq /= Void
			connection_exists: conn /= Void
			connection_open: conn.is_open
			range_exists: range /= Void
		local
			list: like unassigned_part_id_rows
		do
			list := unassigned_part_id_rows (seq, conn, range)
			Result := part_list_from_row_list (list)
		ensure
			result_exists: Result /= Void
		end

	sequence_part_id_rows (seq, part_id: STRING conn: DB_CONNECTION range: ITEM_COUNT_RANGE): DYNAMIC_LIST [like Current] is
			-- Rows for `part_id', which is part of `seq'
		require
			sequence_exists: seq /= Void
			part_id_exists: part_id /= Void
			connection_exists: conn /= Void
			connection_open: conn.is_open
			range_exists: range /= Void
		local
			b: DB_BIND_SS
		do
			create b.make (seq, part_id)
			Result := all_rows_filtered (conn, "vendor_part_sequence_id = :1 and vendor_part_id_source = :2", b)
			check_row_count (Result, range)
		ensure
			result_exists: Result /= Void
		end

feature {NONE} -- Implementation

	part_list_from_row_list (list: LIST [VENDOR_PART_IDS_ROW]): DYNAMIC_LIST [STRING] is
			-- List of part ids from rows in `list'
		require
			list_exists: list /= Void
		do
			create {ARRAYED_LIST [STRING]} Result.make (list.count)
			from
				list.start
			until
				list.after
			loop
				Result.extend (list.item.vendor_part_id_source)
				list.forth
			end
		ensure
			result_exists: Result /= Void
		end

	part_id_rows_for_seq (seq: STRING used: BOOLEAN conn: DB_CONNECTION range: ITEM_COUNT_RANGE): DYNAMIC_LIST [like Current] is
			-- All rows for part ids in `seq' which are
			-- marked according to `used'
		require
			sequence_exists: seq /= Void
			connection_exists: conn /= Void
			connection_open: conn.is_open
			range_exists: range /= Void
		local
			b: DB_BIND_SB
		do
			create b.make (seq, used)
			Result := all_rows_filtered_ordered (conn, "vendor_part_sequence_id = :1 and assigned = :2", b, "vendor_part_id_source")
			check_row_count (Result, range)
		ensure
			result_exists: Result /= Void
		end


end
