
indexing
	
	description: "Object that can draw a view element onto a drawable %
		%in a simple way"
	author: "David Hollenberg"
	date: "July 25, 2006"

class VIEW_ELEMENT_STANDARD_DRAWER
inherit
	VIEW_ELEMENT_DRAWER
	SHARED_STRING_SIZE_FONT_FACTORY
		export
			{NONE} all
		end
	EV_ANY_HANDLER
		export
			{NONE} all
		end
	DOUBLE_MATH
		export
			{NONE} all
		end

feature -- Operations

	draw_without_fill (a_view_element: VIEW_GRAPHICAL_ELEMENT a_drawable: EV_DRAWABLE a_palette: RETICLE_VIEW_COLORS)
			-- Draw `a_view_element' onto `a_drawable' using
			-- `a_palette', not including any needed interior fill
		local
			elem: like a_view_element
			a_text: STRING
			x, y, w, h: INTEGER
			line_width, margin: INTEGER
			rot: INTEGER
			ssf: STRING_SIZE_AND_FONT
		do
			elem := a_view_element
			
			line_width := elem.line_width
			
			debug ("packer_drawing")
				print ("Before: drawing rectangle at (" + elem.x.out + ", " + elem.y.out + ") size " + elem.width.out + " by " + elem.height.out + "%N")
			end
			
			if elem.is_visible then
				-- Visible in view
				
				compute_visible_rectangle (elem, line_width)
				if visible_rectangle.intersects (elem.view.changed_bounding_box) then
					x := visible_x
					y := visible_y
					w := visible_width
					h := visible_height

					debug ("packer_drawing")
						print ("After: drawing rectangle at (" + x.out + ", " + y.out + ") size " + w.out + " by " + h.out + "%N")
					end

					-- Work around bug in 5.6 Vision2 library,
					-- which doesn't work right for width or height
					-- of zero
					if w > 0 and h > 0 then
						if elem.is_border_drawn then
							a_drawable.set_foreground_color (elem.border_color)
							draw_rectangle_with_line_width (a_drawable, x, y, w, h, line_width)
						end
						a_text := elem.label
						if not a_text.is_empty then
							
							margin := line_width + Minimum_separation
							rot := elem.label_rotation
							if rot = 0 or rot = 180 then
								ssf := string_size_font_factory.best_size_and_font (a_text, w - 2 * margin, h - 2 * margin)
							else
								ssf := string_size_font_factory.best_size_and_font (a_text, h - 2 * margin, w - 2 * margin)
							end
							if ssf /= Void then
								-- Found a font where text will fit
								a_drawable.set_font (ssf.font)
								a_drawable.set_foreground_color (elem.label_color)
								if rot = 0 then
									a_drawable.draw_text_top_left (x - ssf.left_offset + (w - ssf.width) // 2, y + (h - ssf.height) // 2, a_text)
								else
									elem.set_clip_area
									inspect rot
									when 90 then
										a_drawable.implementation.draw_rotated_text (x + (w - ssf.height) // 2 + ssf.height - ssf.font.ascent, y - ssf.left_offset + (h - ssf.width) // 2, 3.0 * Pi / 2.0, a_text)
									when 180 then
										a_drawable.implementation.draw_rotated_text (x + (w - ssf.width) // 2 + ssf.width + ssf.left_offset, y + (h - ssf.height) // 2 + ssf.height - ssf.font.ascent, Pi, a_text)
									when 270 then
										a_drawable.implementation.draw_rotated_text (x + (w - ssf.height) // 2 + ssf.font.ascent, y + ssf.width + ssf.left_offset + (h - ssf.width) // 2, Pi / 2.0, a_text)
									end
									elem.remove_clip_area
								end
							end
						end
					end
				end
			end
		end

	draw_fill_only (a_view_element: VIEW_GRAPHICAL_ELEMENT a_drawable: EV_DRAWABLE a_palette: RETICLE_VIEW_COLORS)
			-- Draw only any needed interior fill for
			-- `a_view_element' onto `a_drawable' using
			-- `a_palette'
		local
			elem: like a_view_element
		do
			elem := a_view_element
			if (elem.is_interior_filled or elem.is_opaque) and elem.is_visible then
				compute_visible_rectangle (elem, 1)
				if visible_rectangle.intersects (elem.view.changed_bounding_box) and visible_width > 0 and visible_height > 0 then

					a_drawable.set_foreground_color (elem.interior_color)
					a_drawable.set_line_width (1)
					a_drawable.fill_rectangle (visible_x, visible_y, visible_width, visible_height)
				end
			end
		end

feature {NONE} -- Implementation

	draw_rectangle_with_line_width (a_drawable: EV_DRAWABLE a_x, a_y, a_width, a_height, a_line_width: INTEGER)
			-- Draw a rectangle on `a_drawable' with upper left
			-- corner at (`a_x', `a_y') and dimensions `a_width'
			-- by `a_height'.  Make the outline of the rectangle
			-- `a_line_width' pixels wide, with all of the line
			-- width inside the rectangle (line gets thicker only
			-- toward interior of rectangle)
		local
			x, y, w, h, line_width: INTEGER
		do
			x := a_x
			y := a_y
			w := a_width
			h := a_height
			line_width := a_line_width
			from
			until
				w <= 0 or h <= 0 or line_width <= 0
			loop
				a_drawable.set_line_width (1)
				a_drawable.draw_rectangle (x, y, w, h)
				w := w - 2
				h := h - 2
				x := x + 1
				y := y + 1
				line_width := line_width - 1
			end
		end
	
	compute_visible_rectangle (a_view_element: VIEW_GRAPHICAL_ELEMENT a_line_width: INTEGER)
			-- Compute `visible_x', `visible_y', `visible_width'
			-- and `visible_height' for `a_view_element' using
			-- `a_line_width'
		local
			elem: like a_view_element
		do
			elem := a_view_element
			elem.view.set_visible_rectangle (visible_rectangle, elem.view.drawable, elem.x, elem.y, elem.width, elem.height, a_line_width)
			visible_x := visible_rectangle.x
			visible_y := visible_rectangle.y
			visible_width := visible_rectangle.width
			visible_height := visible_rectangle.height
		end
	
	visible_x: INTEGER
			-- X coordinate of upper left corner of visible
			-- bounding box

	visible_y: INTEGER
			-- Y coordinate of upper left corner of visible
			-- bounding box

	visible_width: INTEGER
			-- Width of visible bounding box

	visible_height: INTEGER
			-- Width of visible bounding box

	visible_rectangle: EV_RECTANGLE
		once
			create Result.make (0, 0, 0, 0)
		end

	Minimum_separation: INTEGER = 1
			-- Minimum separation between drawn edge of rectangle
			-- and text label

end


