
indexing
	
	description: "Object which will handle some or all of the events %
		%on a view (key presses, pointer button up/down, etc)."
	author: "David Hollenberg"
	date: "July 28, 2006"

deferred class VIEW_EVENT_HANDLER

feature {NONE} -- Creation

	make (a_view: like view a_controller: MODEL_VIEW_CONTROLLER)
			-- Create with `a_view' and `a_controller'
		require
			a_view_not_void: a_view /= Void
			a_controller_not_void: a_controller /= Void
		do
			view := a_view
			controller := a_controller
			create key_press_translated_actions
		ensure
			view_set: view = a_view
			controller_set: controller = a_controller
		end

feature -- Properties

	view: EV_DRAWING_AREA
			-- View of model for which to handle events

	controller: MODEL_VIEW_CONTROLLER
			-- Controller to be used to handle events

	key_press_translated_actions: ACTION_SEQUENCE [TUPLE [MODIFIED_KEY]]
			-- Actions to be performed when a key press is
			-- translated into the appropriate action or error
			-- message

	is_read_only: BOOLEAN
			-- Should handler only process events that do
			-- not change the model?

feature -- Modification

	set_is_read_only (b: like is_read_only)
			-- Set `is_read_only' to `b'
		do
			is_read_only := b
		ensure
			read_only_set: is_read_only = b
		end

feature -- Operations

	bind
			-- Bind handler to view by extending the
			-- action sequence for events to be handled
		deferred
		end

	bind_expose_events
			-- Make handler handle expose events on view
		do
			view.expose_actions.extend (agent handle_expose)
		end

	bind_resize_events
			-- Make handler handle resize events on view
		do
			view.resize_actions.extend (agent handle_resize)
		end

	bind_key_press_events
			-- Make handler handle key press events on view
		do
			view.key_press_actions.extend (agent handle_key_press)
		end

	bind_key_release_events
			-- Make handler handle key release events on view
		do
			view.key_release_actions.extend (agent handle_key_release)
		end

	bind_pointer_button_press_events
			-- Make handler handle pointer button press events
			-- on view
		do
			view.pointer_button_press_actions.extend (agent handle_pointer_button_press)
		end

	bind_pointer_button_release_events
			-- Make handler handle pointer button release events
			-- on view
		do
			view.pointer_button_release_actions.extend (agent handle_pointer_button_release)
		end

	bind_pointer_motion_events
			-- Make handler handle pointer motion events
			-- on view
		do
			view.pointer_motion_actions.extend (agent handle_pointer_motion)
		end

	bind_pointer_enter_events
			-- Make handler handle pointer enter events
			-- on view
		do
			view.pointer_enter_actions.extend (agent handle_pointer_enter)
		end

	handle_expose (a_x, a_y, a_width, a_height: INTEGER)
			-- Handle expose event on `view'
		do
		end

	handle_resize (a_x, a_y, a_width, a_height: INTEGER)
			-- Handle resize event on `view'
		do
		end

	handle_key_press (a_key: EV_KEY)
			-- Handle key press event on `view'
		require
			a_key_not_void: a_key /= Void
		do
		end

	handle_key_release (a_key: EV_KEY)
			-- Handle key release event on `view'
		require
			a_key_not_void: a_key /= Void
		do
		end

	handle_pointer_button_press (a_x, a_y, a_button: INTEGER a_x_tilt, a_y_tilt, a_pressure: DOUBLE a_screen_x, a_screen_y: INTEGER)
			-- Handle pointer button press event on `view'
		do
		end

	handle_pointer_button_release (a_x, a_y, a_button: INTEGER a_x_tilt, a_y_tilt, a_pressure: DOUBLE a_screen_x, a_screen_y: INTEGER)
			-- Handle pointer button release event on `view'
		do
		end

	handle_pointer_motion (a_x, a_y: INTEGER a_x_tilt, a_y_tilt, a_pressure: DOUBLE a_screen_x, a_screen_y: INTEGER)
			-- Handle pointer motion event on `view'
		do
		end

	handle_pointer_enter
			-- Handle pointer enter event on `view'
		do
		end

invariant
	view_not_void: view /= Void
	controller_not_void: controller /= Void
end
