deferred class VIRTUAL_GDS_PAIR
		-- just a pair of integers for a gds layer
		-- (or a layer and a null datatype to mean all datatypes)

inherit
	ANY
		redefine	out
		end;

feature	-- the gds layer and datatype

	layer: INTEGER is
	deferred
	end;

	datatype: INTEGER is
	deferred
	end;

	is_null_datatype: BOOLEAN is
	deferred
	end;

feature	-- layer and datatype validity

	min_layer: INTEGER is 0;
	max_layer: INTEGER is 255;
	min_datatype: INTEGER is 0;
	max_datatype: INTEGER is 255;

	is_valid_layer(l: INTEGER): BOOLEAN is
	do  Result := min_layer <= l and l <= max_layer
	end;

	is_valid_datatype(dt: INTEGER): BOOLEAN is
	do  Result := min_datatype <= dt and dt <= max_datatype
	end;

feature	-- as Cats string

	out: STRING is
		-- represent it Cats style
	do  if is_null_datatype
	     then Result := layer.out
	     else create Result.make(7);
		  Result.append(layer.out);
		  Result.extend(';');
		  Result.append(datatype.out)
	    end
	end;

	list_as_Cats_string(lst: LIST[VIRTUAL_GDS_PAIR]): STRING is
		-- represent list as Cats style
		-- (if list is layer sorted, output will be shorter)
	require lst /= Void
	local prev, curr: VIRTUAL_GDS_PAIR
	do  create Result.make(8*lst.count)	-- worst case
	    from  lst.start
	    until lst.after
	    loop  prev := curr;
		  curr := lst.item
		  if prev = Void then
			Result := curr.out
		  elseif prev.layer = curr.layer then
			Result.extend(',');
			Result.append(curr.datatype.out)
		  else
			Result.extend(' ');
			Result.append(curr.out)
		  end;
		  lst.forth
	    end
	end;

feature	-- single integer representation

	as_integer: INTEGER is
		-- returns the value of a make_integer creation arg
	do  Result := 256 * datatype + layer
	end;

feature	-- comparison

	three_way_comparison (other: VIRTUAL_GDS_PAIR): INTEGER is
	require other_exists: other /= Void
	do  Result := (layer - other.layer).sign;
	    if Result = 0 then		-- same layer; break the tie
		if is_null_datatype then
			if not other.is_null_datatype then Result := -1 end
		elseif other.is_null_datatype then Result := 1
		else Result := (datatype - other.datatype).sign
		end
	    end;
	end;

end -- VIRTUAL_GDS_PAIR