
indexing
	
	description: "Window to allow user to create initial wafer lots for %
		%a run"
	author: "David Hollenberg"
	date: "March 1, 2007"

class WAFER_LOT_CREATION_WINDOW

inherit
	WAFER_LOT_CREATION_IFC
	WINDOW_INTERFACE
	MOSIS_GRAPHICAL_UTILITIES
		export
			{NONE} all
		undefine
			default_create
		end
		
create
	make

feature {NONE} -- Creation

	make (w: EV_TITLED_WINDOW)
			-- Create interface in window `w'
		require
			window_exists: w /= Void
			window_valid: not w.is_destroyed
		do
			make_interface
			make_proxy (w)
		end

feature -- Operations

	reset
			-- Reset interface
		do
		end

feature {NONE} -- Implementation

	build_interface
			-- Build interface in `window'
		local
			main_vbox: EV_VERTICAL_BOX
			buttons: HORIZONTAL_BUTTON_SET
			box: EV_HORIZONTAL_BOX
			sep: EV_HORIZONTAL_SEPARATOR
		do
			fetch_data
			create main_vbox
			main_vbox.set_border_width (user_interface.border_width)
			if cache.is_reticle_packing_target then
				build_step_pattern
				main_vbox.extend (last_box)
				main_vbox.disable_item_expand (last_box)
				create sep
				main_vbox.extend (sep)
				main_vbox.disable_item_expand (sep)
			end
			
			-- Build display for wafer lots

			build_wafer_lots
			main_vbox.extend (last_box)
			main_vbox.disable_item_expand (last_box)

			create buttons.make_with_text_and_actions (<< gui.ev_ok, gui.ev_cancel >>, << agent process_accept, agent process_decline >>)
			box := buttons.column_set.box
			
			main_vbox.extend (box)
			main_vbox.disable_item_expand (box)
			gui.set_box_background_recursive (main_vbox, background_color)
			extend (main_vbox)

			close_request_actions.extend (agent process_decline)
			gui.add_cancel_accelerator (window, agent process_decline)
			enable_active
			is_resize_to_minimum_requested := False
			gui.do_once_on_idle (agent gui.center_window_with_horizontal_margin (window, 250))
		end
		
	fetch_data
			-- Fetch needed data from database
		local
			row: WAFER_PROCESSES_ROW
		do
			create row.make_template
			processes := row.processes_for_run_fab_instances (session.run_id, connection, expected_count.At_least_zero)
			if cache.is_reticle_packing_target then
				step_patterns := possible_field_step_patterns
			end
		end

	possible_field_step_patterns: LIST [FIELD_STEP_PATTERNS_ROW]
			-- Possible field step patterns for this run
		local
			template_row: FIELD_STEP_PATTERNS_ROW
		do
			create template_row.make_template
			Result := template_row.available_step_patterns_ordered (cache.field_set_name, connection, expected_count.at_least_zero)
			if Result.is_empty then
				handle_soft_error ("no available field step patterns for field set " + cache.field_set_name)
				Result := template_row.available_step_patterns_ordered (cache.field_set_name, connection, expected_count.at_least_one)
			end
		end

	build_step_pattern
			-- Build portion of window with field step pattern
			-- and leave in `last_box'
		local
			cols: WIDGET_COLUMN_SET
			fact: SINGLE_CONSTRAINED_VALUE_FACTORY
			step_pattern: FIELD_STEP_PATTERNS_ROW
			default_choice: STRING
		do
			create fact
			fact.enable_choices_prepended
			from
				step_patterns.start
			until
				step_patterns.after
			loop
				step_pattern := step_patterns.item
				fact.add_choice_and_text (step_pattern.field_step_pattern, step_pattern.descr)
				step_patterns.forth
			end
			if fact.choices.count /= 1 then
				fact.prepend_choice_and_text ("", "(Select One)")
			end
			if state /= Void and then state.has_one_step_pattern and then step_patterns.there_exists (agent matches_saved_value (?, state.step_pattern)) then
				default_choice := state.step_pattern
			else
				default_choice := fact.choices.first
			end
			fact.set_default_choice (default_choice)
			fact.generate_combo_box
			step_pattern_field := fact.combo_box
			
			create cols.make (2)
			cols.align_column_right (1)
			cols.align_column_none (2)
			cols.set_border_width (user_interface.border_width)
			cols.set_padding (user_interface.padding)
			cols.append_row (<< gui.label_with_text ("Field Step Pattern"), step_pattern_field >>)
			cols.enable_expand
			cols.disable_column_expand (1)
			last_box := cols.box
		end
		
	matches_saved_value (r: FIELD_STEP_PATTERNS_ROW saved_pattern: STRING): BOOLEAN
			-- Does `r' have `saved_pattern' as its field step
			-- pattern?
		require
			row_exists: r /= Void
		do
			Result := equal (r.field_step_pattern, saved_pattern)
		end

	has_required_processes: BOOLEAN
			-- Are there any processes which must be ordered
			-- (i.e., are there any customer projects on the run)?
		do
			Result := not processes.is_empty
		end
		
	build_wafer_lots
			-- Build portion of window with wafer lots
			-- and leave in `last_box'
		do
			if has_required_processes then
				build_wafer_lots_for_processes
			else
				build_single_wafer_lot
			end
		end
		
	build_single_wafer_lot
			-- Build portion of window with wafer lots
			-- when there are no customer projects
			-- leave in `last_box'.  In this case, the wafer
			-- process is a single pull down list with
			-- all possible processes for the run's wafer techn
		local
			cols: WIDGET_COLUMN_SET
			fact: SINGLE_CONSTRAINED_VALUE_FACTORY
			proc: WAFER_PROCESSES_ROW
			wt_processes: LIST [WAFER_PROCESSES_ROW]
			default_choice: STRING
		do
			create proc.make_template
			wt_processes := proc.processes_for_wafer_techn (cache.wafer_techn, connection, expected_count.At_least_one)
			create fact
			fact.add_choice_and_text ("", "None")
			from
				wt_processes.start
			until
				wt_processes.after
			loop
				proc := wt_processes.item
				fact.add_choice_and_text (proc.wafer_process, proc.name_for_wafer_lot_creation)
				wt_processes.forth
			end
			default_choice := ""
			fact.set_default_choice (default_choice)
			fact.generate_combo_box
			single_process_field := fact.combo_box
			
			create cols.make (2)
			cols.align_columns_none
			cols.set_border_width (user_interface.border_width)
			cols.set_padding (user_interface.padding)
			cols.append_row (<< gui.label_with_text ("Process"), gui.label_with_text ("Number of Wafers to Order") >>)
				
			single_quantity_field := new_quantity_field
			cols.append_row (<< single_process_field, single_quantity_field >>)
			cols.enable_expand
			last_box := cols.box
		end
		
	build_wafer_lots_for_processes
			-- Build portion of window with wafer lots
			-- when there is at least one ordered process
			-- (i.e., at least one customer project) and
			-- leave in `last_box'
		local
			cols: WIDGET_COLUMN_SET
			lot_field, process_field, quantity_field: EV_TEXT_FIELD
			proc: WAFER_PROCESSES_ROW
		do
			create {ARRAYED_LIST [EV_TEXT_FIELD]} wafer_lot_quantity_fields.make (5)
			create cols.make (3)
			cols.align_columns_none
			cols.set_border_width (user_interface.border_width)
			cols.set_padding (user_interface.padding)
			cols.append_row (<< gui.label_with_text ("Wafer Lot"), gui.label_with_text ("Ordered Process"), gui.label_with_text ("Wafers to Order") >>)
			from
				processes.start
			until
				processes.after
			loop
				proc := processes.item
				lot_field := new_field (proc.wafer_etest_id)
				lot_field.disable_edit
				process_field := new_field (proc.name_for_wafer_lot_creation)
				process_field.disable_edit
				quantity_field := new_quantity_field
				quantity_field.set_data (proc)
				if state /= Void and then state.has_lot_for_process (proc.wafer_process) then
					quantity_field.set_text (state.quantity_for_process (proc.wafer_process).out)
				end
				cols.append_row (<< lot_field, process_field, quantity_field >>)
				wafer_lot_quantity_fields.extend (quantity_field)
				processes.forth
			end
			cols.disable_column_expand (3)
			last_box := cols.box
		end
		
	process_accept
			-- Take actions needed when user accepts
			-- current operation
		local
			operation: ANONYMOUS_OPERATION
		do
			if is_active then
				disable_active
				create operation.make (agent process_accept_request)
				operation.enable_abort_caught
				operation.execute
				if operation.aborted then
					session.rollback
					enable_active
				else
					success_actions.call ([])
				end
			end
		end

	process_accept_request
			-- Process an accept request
		do
			if cache.is_reticle_packing_target then
				check_step_pattern_valid
			end
			check_quantities_valid
			create_wafer_lots
		end

	check_step_pattern_valid
			-- Check that field step pattern has been selected
		do
			if field_pattern (step_pattern_field.selected_item).is_empty then
				handle_user_error ("No field step pattern selected")
			end
		end

	check_quantities_valid
			-- Check that all wafer lot quantities
			-- are present and valid
		local
			list_item: EV_LIST_ITEM
			proc_name: STRING
		do
			if has_required_processes then
				wafer_lot_quantity_fields.do_all (agent check_wafer_lot_quantity_valid)
			else
				list_item := single_process_field.selected_item
				proc_name ?= list_item.data
				if proc_name.is_empty then
					handle_user_error ("No wafer process selected")
				end
				check_quantity_valid (single_quantity_field, "Wafer quantity for process " + list_item.text)
			end
		end

	field_process (field: EV_TEXT_FIELD): WAFER_PROCESSES_ROW
			-- Wafer process associated with `field'
		require
			field_not_void: field /= Void
		do
			Result ?= field.data
			check
				result_not_void: Result /= Void
			end
		end
	
	field_pattern (list_item: EV_LIST_ITEM): STRING
			-- Step pattern associated with `field'
		require
			list_item_not_void: list_item /= Void
		do
			Result ?= list_item.data
			check
				result_not_void: Result /= Void
			end
		end
	
	check_wafer_lot_quantity_valid (field: EV_TEXT_FIELD)
			-- Check that `field' contains a valid wafer lot
			-- quantity.  Issue error if not
		require
			field_not_void: field /= Void
		do
			check_quantity_valid (field, "Wafer quantity for process " + field_process (field).name_for_wafer_lot_creation)
		end

	check_quantity_valid (quantity: EV_TEXT_FIELD name: STRING)
			-- Check that `quantity' which is labeled `name'
			-- is present and valid.  Issue error if not
		require
			quantity_exists: quantity /= Void
			name_exists: name /= Void
		local
			s: STRING
			n, the_max: INTEGER
		do
			s := quantity.text
			if s.is_empty then
				handle_user_error (name + " is empty")
			elseif not s.is_integer then
				handle_user_error (name + " is not a valid integer")
			else
				n := s.to_integer
				the_max := max_quantity_wafers_ordered
				if n < 0 then
					handle_user_error (name + " is negative")
				elseif n > the_max then
					handle_user_error (name + " exceeds maximum value " + the_max.out)
				end
			end
		end

	max_quantity_wafers_ordered: INTEGER
			-- Maximum number that will fit in quantity wafers
			-- ordered database field for wafer lot
		local
			row: RUN_WAFER_LOTS_ROW
		do
			create row.make_template
			Result := row.maximum_quantity_wafers_ordered
		end

	create_wafer_lots
			-- Insert initial wafer lots into database
		local
			list: like wafer_lot_quantity_fields
			field: EV_TEXT_FIELD
			proc: WAFER_PROCESSES_ROW
			n: INTEGER
			proc_name, pat: STRING
		do
			if cache.is_reticle_packing_target then
				pat := field_pattern (step_pattern_field.selected_item)
			else
				pat := ""
			end
			if has_required_processes then
				from
					list := wafer_lot_quantity_fields
					list.start
				until
					list.after
				loop
					field := list.item
					proc := field_process (field)
					n := n + 1
					create_one_lot (n, proc.wafer_process, pat, field)
					list.forth
				end
			else
				proc_name ?= single_process_field.selected_item.data
				create_one_lot (1, proc_name, pat, single_quantity_field)
			end
		end

	create_one_lot (n: INTEGER proc_name, pat: STRING field: EV_TEXT_FIELD)
			-- Create one row for wafer lot `n' with process
			-- name `proc_name and step pattern `pat', with
			-- quantity specified by `field'
		local
			wafer_lot: RUN_WAFER_LOTS_MROW
			quant: INTEGER
		do
			quant := field.text.to_integer
			create wafer_lot.make_null (connection)
			wafer_lot.set_run_id (session.run_id)
			wafer_lot.set_wafer_lot_no (n)
			wafer_lot.set_production_id (cache.production_id)
			wafer_lot.set_wafer_process (proc_name)
			wafer_lot.set_field_step_pattern (pat)
			if not pat.is_empty then
				wafer_lot.set_field_step_pattern_offset_x (0.0)
				wafer_lot.set_field_step_pattern_offset_y (0.0)
			else
				wafer_lot.set_null_field_step_pattern_offset_x
				wafer_lot.set_null_field_step_pattern_offset_y
			end
			wafer_lot.set_quantity_wafers_ordered (quant)
			wafer_lot.set_quantity_wafers_ordered_wo_dfc (quant)
		end

	process_decline
			-- Take actions needed when user declines
			-- current operation
		do
			if is_active then
				disable_active
				abort_actions.call ([])
			end
		end

	new_field (s: STRING): EV_TEXT_FIELD
			-- New text field with initial text `s' with
			-- minimum width in characters `min_width' that
			-- is editable
		do
			create Result.make_with_text (s)
			Result.enable_edit
		ensure
			result_exists: Result /= Void
		end

	new_quantity_field: EV_TEXT_FIELD
			-- New text field that can hold a wafer quantity
		do
			Result := new_field ("")
			Result.return_actions.extend (agent process_accept)
		ensure
			result_exists: Result /= Void
		end

	processes: LIST [WAFER_PROCESSES_ROW]
			-- List of distinct wafer processes (to be ordered)

	step_patterns: LIST [FIELD_STEP_PATTERNS_ROW]
			-- Possible field step patterns for this run

	step_pattern_field: EV_COMBO_BOX
			-- Field that offers choice of step patterns

	single_process_field: EV_COMBO_BOX
			-- Field that offers choice of wafer processes
			-- offered for run's wafer techn, in case where there
			-- are no customer projects on run

	single_quantity_field: EV_TEXT_FIELD
			-- Field for entering quantity in case where there are
			-- no customer projects on run

	wafer_lot_quantity_fields: LIST [EV_TEXT_FIELD]
			-- List of quantity fields for wafer lots

	last_box: EV_BOX
			-- Last box built

end
