
indexing
	
	description: "State saved by step that creates initial wafer lots"
	author: "David Hollenberg"
	date: "March 1, 2007"

class WAFER_LOTS_STATE

feature -- Properties

	wafer_lots: LIST [WAFER_PROCESS_QUANTITY_STATE]
			-- Saved information about wafer lots

	has_lot_for_process (a_process: STRING): BOOLEAN
			-- Is there a saved lot for `a_process'?
		require
			a_process_not_void: a_process /= Void
		do
			Result := not wafer_lots.is_empty and then wafer_lots.there_exists (agent {WAFER_PROCESS_QUANTITY_STATE}.is_wafer_process_equal (a_process))
		end

	quantity_for_process (a_process: STRING): INTEGER
			-- Number of wafers ordered for `a_process'
		require
			a_process_not_void: a_process /= Void
		local
			found: BOOLEAN
			lot: WAFER_PROCESS_QUANTITY_STATE
		do
			from
				wafer_lots.start
			until
				wafer_lots.after or found
			loop
				lot := wafer_lots.item
				if lot.wafer_process.is_equal (a_process) then
					Result := lot.quantity
					found := True
				end
				wafer_lots.forth
			end
		end

	has_one_step_pattern: BOOLEAN
			-- Do all lots specify the same field step pattern?
		do
			Result := not wafer_lots.is_empty and then not wafer_lots.there_exists (agent {WAFER_PROCESS_QUANTITY_STATE}.step_pattern_not_equal (wafer_lots.first.step_pattern))
		end

	step_pattern: STRING
			-- Name of field step pattern used by all lots
		require
			has_one_step_pattern: has_one_step_pattern
		do
			Result := wafer_lots.first.step_pattern
		end

feature -- Modification

	set_wafer_lots (list: like wafer_lots)
			-- Set `wafer_lots' to `list'
		do
			wafer_lots := list
		end

end
