
indexing
	
	description: "Window to allow printing of wafer maps for %
		%a specific run"
	author: "David Hollenberg"
	date: "March 10, 2005"

class WAFER_MAP_PRINT_WINDOW

inherit
	RC_RUN_PRINT_WINDOW
		redefine
			process_zoom_in,
			process_zoom_out
		end
	CUT_MAP_UTILITIES
		export
			{NONE} all
		undefine
			default_create
		end
		
create
	make

feature {NONE} -- Implementation

	add_extra_select_menu_items (select_menu: EV_MENU)
		local
			menu_item: EV_MENU_ITEM
		do
			create menu_item.make_with_text ("Probing maps")
			menu_item.select_actions.extend (agent process_select_probing_maps)
			select_menu.extend (menu_item)
			probing_maps_menu_item := menu_item
			
			create menu_item.make_with_text ("Cut patterns")
			menu_item.select_actions.extend (agent process_select_cut_patterns)
			select_menu.extend (menu_item)
			cut_patterns_menu_item := menu_item
		end

	populate_maps
			-- Populate list of maps that can be printed
		local
			fg: RC_FILE_GENERATOR
		do
			maps.wipe_out
			missing_map_count := 0
			fetch_cut_pattern_data
			fg := file_generator
			add_map (Site_map_name, fg.site_numbers_map_ps_file)
			add_map (Organizations_map_name, fg.organizations_map_ps_file)
			add_map (Design_numbers_map_name, fg.design_numbers_map_ps_file)
			add_map (Design_names_map_name, fg.design_names_map_ps_file)
			add_map (Project_summary_name, fg.project_summary_ps_file)
			if cache.is_reticle_packing_target then
				add_map (Step_pattern_map_name, fg.step_pattern_map_ps_file)
				add_map (Detailed_step_pattern_map_name, fg.detailed_step_pattern_map_ps_file)
			end
			cache.packaging_lot_rows.do_all (agent add_cut_map)
			if cache.cut_pattern_rows.there_exists (agent {RUN_CUT_PATTERNS_ROW}.is_unassigned) then
				add_cut_map (Void)
			end
			set_missing_files_status
		end

	add_cut_map (lot: RUN_PACKAGING_LOTS_ROW)
			-- Add cut map for `lot' to `maps' if file exists
		local
			sym_name: STRING
			need_map: BOOLEAN
		do
			if lot /= Void then
				sym_name := lot.symbolic_cut_patterns_map_name
				need_map := not lot.cut_patterns.is_empty
			else
				sym_name := "Cut Patterns not Assigned to a Packaging Lot"
				need_map := True
			end
			if need_map then
				add_map (sym_name, postscript_file_for_lot (lot))
			end
		end

	process_select_probing_maps
			-- Process "select probing maps" request
		do
			if is_active then
				disable_active
				maps.do_all (agent select_probing_map)
				enable_active
			end
		end

	select_probing_map (list_item: EV_LIST_ITEM)
			-- Select `list_item' if it is for a probing map
			-- and deselect it if it is not
		require
			list_item_not_void: list_item /= Void
		local
			s: STRING
		do
			s := list_item.text
			if s.is_equal (Step_pattern_map_name) or s.is_equal (Detailed_step_pattern_map_name) then
				list_item.enable_select
			else
				list_item.disable_select
			end
		end

	process_select_cut_patterns
			-- Process "select cut patterns" request
		do
			if is_active then
				disable_active
				maps.do_all (agent select_cut_pattern)
				enable_active
			end
		end

	select_cut_pattern (list_item: EV_LIST_ITEM)
			-- Select `list_item' if it is for a cut pattern
			-- and deselect it if it is not
		require
			list_item_not_void: list_item /= Void
		local
			s: STRING
		do
			s := list_item.text
			if s.as_lower.substring_index ("cut pattern", 1) > 0 then
				list_item.enable_select
			else
				list_item.disable_select
			end
		end

	process_zoom_in
			-- Process "zoom in" request
		do
			precursor
			probing_maps_menu_item.disable_sensitive
			cut_patterns_menu_item.disable_sensitive
		end

	process_zoom_out
			-- Process "zoom out" request
		do
			precursor
			probing_maps_menu_item.enable_sensitive
			cut_patterns_menu_item.enable_sensitive
		end

	job_name: STRING
			-- Name to use for print job
		do
			Result := session.run_id + "_wafer_maps"
		end
	
	probing_maps_menu_item: EV_MENU_ITEM

	cut_patterns_menu_item: EV_MENU_ITEM

	Site_map_name: STRING = "Site Number Map"

	Organizations_map_name: STRING = "Organizations and Sponsors Map"

	Design_numbers_map_name: STRING = "Design Numbers Map"

	Design_names_map_name: STRING = "Design Names Map"

	Project_summary_name: STRING = "Project Summary"

	Step_pattern_map_name: STRING = "Step Pattern Map"

	Detailed_step_pattern_map_name: STRING = "Detailed Step Pattern Map"

end
