
indexing
	
	description: "A wafer techn family.  Specific wafer techn families %
		%inherit from this class"
	author: "David Hollenberg"
	date: "June 18, 2003"

deferred class WAFER_TECHN_FAMILY

inherit
	RUN_CACHABLE		-- cache and set_cache

	CONDITION_UTILITIES

	CATS_EXPRESSION_SUPPORT

	RC_POLICIES_ACCESS
	export {NONE} all end

	MOSIS_PROD_VALUES
	export {NONE} all end

	CATS_CONSTANTS
	export {NONE} all end

	SEVERITY_LEVELS
	export {NONE} all end

feature -- Properties

	name: STRING
			-- Name of this wafer_techn_family
		deferred
		ensure
			result_exists: Result /= Void
		end

	native_design_techn: STRING
			-- name of the "native" design_techn
	deferred
	ensure	result_exists: Result /= Void
	end

feature	-- wafer_part_id derivation (redefine as needed)

	wafer_part_id_from_source (s: STRING): STRING
			-- Wafer part id derived from wafer part id source `s'
		require
			valid_source: is_valid_source (s)
		do
			Result := s
		ensure
			result_exists: Result /= Void
		end

	is_valid_source (s: STRING): BOOLEAN
			-- Is `s' a valid source for family?
		do
			Result := s /= Void
		end

feature	-- dynamic vendor geometry support

	is_dynamic_vendor_geometry(d: DIES_XROW): BOOLEAN
		-- indicates whether this die's geometry item is static (files
		--  already in place) or dynamic (files generated per run)
	require d.has_geometry
	do
	    Result := false	-- redefine if there are any
	end

	generate_dynamic_vendor_geometry(d: DIES_XROW)
		-- generates the layout for this dynamic vendor geometry
		-- item for this run, and writes it into the appropriate file
	require is_dynamic: is_dynamic_vendor_geometry(d)
	local g: GEOMETRIES_ROW; dt: DESIGN_TECHNOLOGY; body: LAYOUT; cname: STRING
		ltc: LAYOUT_TO_CIF; resp: RESPONSE_COLLECTOR
	do  d.extend_xrow_data (cache)
	    g := d.geometries_row
	    dt := d.design_techn
	    -- build a single-cell layout
	    body := build_vendor_geometry(g, dt, d.fracture_limits)
	    -- and write it out per g
	    cname := vendor_geometry_file(g).file_pathname
	    if equal (g.geometry_format, fmt_is_cif) then
		create ltc
		ltc.layout_to_cif_file (body, dt, cname)
		resp := ltc.the_messages
		if resp /= Void and then resp.is_in_error then
			handle_hard_error ("cif writer error: " +
					   resp.i_th_at_severity(1, severity_is_error).text)
		end
	    else
		handle_hard_error ("Unsupported format " + g.geometry_format +
				   " for dynamic geometry item " + g.geometry_id)
	    end
	end

feature {NONE} -- (redefine along with is_dynamic_vendor_geometry)

	build_vendor_geometry(	g: GEOMETRIES_ROW
				dt: DESIGN_TECHNOLOGY
				lim: CONST_RECTANGLE): LAYOUT
		-- build a single-celled layout for g
	do
	end

feature	-- fracture script status

	is_mebes_checking_performed: BOOLEAN
		-- do we (potentially) use the script facility to generate
		--  checks (cassert statements) in the fracture script?
	do  Result := false	-- redefine if true
	end

	is_fracture_script_needed: BOOLEAN
		-- is a fracture script needed at all (either for checking or
		--  for generating mebes files)
	do  Result := cache.run_needs_mebes_files or
			is_mebes_checking_performed
	end

feature	-- fracture script writer

	write_fracture_script (d: DIES_XROW)
		-- write out the complete fracture script for die d into
		--  a shell script file
		-- the cache's file_generator is used to obtain all permanent
		-- file names
		-- (all temporary file names, aka $TED/ files, are invented
		--  here and are cleaned up by the script; they are outside
		--  the ken of the RC_FILE_GENERATOR)
	require have_die: d /= Void
	local	sfile: LARGE_PLAIN_TEXT_FILE
		opw: FILE_OPEN_WRITE_OP
		cls: FILE_CLOSE_OP
		cenv: CATS_COMPILATION_ENVIRONMENT
	do
		-- setup extended stuff in d
		d.extend_xrow_data (cache)
		-- initialize shared variables
		ted_filename_prefix :=
			su.string_of_3(ted_path, d.fab_id_lc, su.underscore)
		die_fracture_pf_root :=	cache.die_fracture_pattern_file(d, "")
		overetching := cache.mask_techns_row.overetch
		create cats_layer_source.make(d.design_techn)
		native_dt := cache.design_technology_for_dt(native_design_techn)
		native_dtxls := cache.xform_level_rows_for_dt(native_design_techn)
		-- create compilation environment
		cenv := compilation_environment_setup (d)
		-- compile fracture of d into the environment
		compile_die_fracture(d, cenv)
		-- add writefile steps for final fracture
		cenv.add_writefile_steps (die_fracture_pf_root.file_pathname, true)
		-- create and open the die fracture script file
		create sfile.make (cache.die_fracture_script_file(d).file_pathname)
		create opw
		opw.open(sfile)
		sfile.add_permission("u", "rx")
		sfile.add_permission("g", "rx")
		sfile.put_string(policies.cats_script_prolog)
		sfile.put_string(cleanup_ted)
		-- everything else
		cenv.write_shell_script(sfile, true)
		-- and the is_done file creation
		sfile.put_string(cleanup_ted)
		sfile.put_string(create_is_done(d))
		-- close script file
		sfile.put_string(policies.cats_script_epilog)
		create cls
		cls.close(sfile)
	end

feature {NONE} -- compilation environment setup

	compilation_environment_setup (d: DIES_XROW): CATS_COMPILATION_ENVIRONMENT
		-- create and initialize a compilation environment for d
	local	cid: CUST_IC_DESIGNS_ROW
		cidf: CUST_IC_DESIGN_FILES_ROW
		lnam, llib,	-- layout source and library (clib) file names
		onam, olib,	-- overhead source and library (clib)
		clay,		-- clay file name
		gfmt,		-- layout geometry format
		root: STRING	-- layout root structure (gds only)
		claystp: CATS_CLAYFILE_STEP
	do  create Result.make
	    Result.set_design_technology (d.design_techn)
	    Result.set_operation_template (standard_rc_op_template)
	    -- set up the leaf input template(s) (layout and maybe overhead)
	    cid := d.cust_ic_design
	    if cid /= Void then		-- cust die or test vehicle
		cidf := cid.layout_of_interest
		lnam := cidf.full_filename
		llib := ted_filename("layout", dot_clib)
		gfmt := cid.geometry_format
		root := cidf.comp_top_structure	-- (only applies to gds)
		-- and its associated overhead file
		onam := fgen.die_overhead_layout_file(d.die_id).file_pathname
		olib := ted_filename("overhead", dot_clib)
	    elseif d.is_scribe or d.is_filler then
		-- scribe, filler; these are always cif files
		lnam := fgen.die_scribe_layout_file(d.die_id).file_pathname
		llib := ted_filename("layout", dot_clib)
		gfmt := fmt_is_cif
	    elseif d.is_vendor_geometry then
		gfmt := d.geometries_row.geometry_format
		if equal(gfmt, fmt_is_mebes) then
			llib := locate_vendor_geometry(d)	-- direct input
		else
			lnam := locate_vendor_geometry(d)
			llib := ted_filename("layout", dot_clib)
			root := d.geometries_row.top_structure_name
		end
	    else --
	    end
	    Result.set_leaf_input_templates
			(leaf_input(llib, gfmt, root, d),
			 leaf_input(olib, fmt_is_cif, Void, d))
	    -- establish prefixes for generated file name pools
	    Result.set_final_fracture_prefix(su.string_of_3
			(ted_path, die_fracture_pf_root.file_name.substring(1,9), "_"))
	    Result.set_generated_temp_prefix(ted_filename	("T_", Void))
    	    Result.set_explicit_temp_prefix(ted_filename("X_", Void))
	    -- FIX_ME: anything else to establish ??
	    -- now stick in the file setup steps too
	    -- create clay file (if have any cif)
	    if equal(gfmt, fmt_is_cif) or olib /= Void then
		clay := ted_filename("clay", dot_clay)
		claystp := clayfile_step (d.design_techn, clay)
		Result.add_a_step (claystp)
	    end
	    -- create source clib (unless source was mebes)
	    if lnam /= Void then
		Result.add_a_step (input_to_clib (lnam, llib, clay, gfmt, claystp))
	    end
	    -- create overhead clib (if have overhead)
	    if onam /= Void then
		Result.add_a_step (input_to_clib (onam, olib, clay, fmt_is_cif, claystp))
	    end
	end

	locate_vendor_geometry(d: DIES_XROW): STRING
		-- given a vendor geometry dies row, find the location of the
		-- file (or file set) and return the full path name
		-- for mebes, up to and including the trailing '.' without the extension
	local g: GEOMETRIES_ROW; rcf: RC_FILE
		rname, mext: STRING; srch: RC_SEARCH_PATH
	do  g := d.geometries_row
	    rcf := vendor_geometry_file(g)
	    if is_dynamic_vendor_geometry(d) then
		-- assume it is (or will be) in the run directory
		Result := rcf.file_pathname
	    else -- search for it along our standard search path
		rname := rcf.file_name.twin	-- (ignoring the path in rcf)
		-- (if mebes, just look for one member)
		if equal(g.geometry_format, fmt_is_mebes) then
		    mext := cache.run_level_rows.first.mebes_extension.as_lower
		    rname := su.string_of_3 (rname, ".", mext)
		end
		create srch.make_rc_path (cache)
		Result := srch.find_file (rname)
		if su.is_empty(Result) then
			handle_hard_error(su.string_of_5
					("Vendor geometry file ", rname, " for ",
					 g.geometry_id, " not found."))
		end
		if mext /= Void then
		    Result.keep_head(Result.count-mext.count)	-- knock off the mebes extension
		end
	    end
	end

	vendor_geometry_file(g: GEOMETRIES_ROW): RC_FILE
		-- return the rc_file for g
		-- (the rc_file has the correct file_name for ANY vendor geometry,
		--  but the path is likely to be wrong for static geometry!)
	do  Result := fgen.dynamic_vendor_geometry_file (g, cache.substitution_parameter_set)
	end

	standard_rc_op_template: CATS_OPERATION
		-- the standard operations template for the environment
	do  create Result.m_standard
	    Result.set_initial_cinc_file (policies.cats_op_reset)
	    -- FIX_ME: anything else to change??
	end

	leaf_input(flib, gfmt, root: STRING d: DIES_XROW): CATS_OP_FILE
		-- build a CATS_OP_FILE template for this guy
	do  if flib = Void then
		-- Result := Void (no overhead)
	    elseif equal(gfmt, fmt_is_cif) then
		create Result.make_hierarchical(flib, cif_root_structure_name, d.fracture_limits)
		Result.set_cif_layers
	    elseif equal(gfmt, fmt_is_gds) then
		create Result.make_hierarchical(flib, root, d.fracture_limits)
		Result.set_gds_layers
	    elseif equal(gfmt, fmt_is_mebes) then
		create Result.make_flat_set(flib,
					d.design_techn.input_scale_factor)
	    else
		handle_internal_error("unsupported format: " + gfmt)
	    end
	    -- now add orientation
	    if Result = Void or d.is_null_die_wafer_rotation or d.die_wafer_rotation = 0 then
		-- OK
	    elseif d.die_wafer_rotation = 90 then	-- die_wafer_rotation is CW
		Result.set_orientation (rot270)		-- matrix (& Cats) rotation is CCW
	    elseif d.die_wafer_rotation = 180 then
		Result.set_orientation (rot180)
	    elseif d.die_wafer_rotation = 270 then
		Result.set_orientation (rot90)
	    else
		handle_internal_error("unsupported die_wafer_rotation of " + d.die_wafer_rotation.out)
	    end
	end

	clayfile_step (dtech: DESIGN_TECHNOLOGY clay: STRING): CATS_CLAYFILE_STEP
		-- step to create a clay (cif layer) file for dtech
	do  create Result.make (dtech, clay)
	end

	input_to_clib (lnam, llib, clay, gfmt: STRING claystp: CATS_SCRIPT_STEP): CATS_SCRIPT_STEP
		-- step to convert layout file lnam to clib file llib
	local ccs: CATS_CIFLIB_STEP
	do  if equal(gfmt, fmt_is_cif) then
		create ccs.make (clay, lnam, llib)
		ccs.set_my_producer(claystp)
		Result := ccs
	    else
		create {CATS_READGDS_STEP} Result.make (lnam, llib)
	    end
	    Result.set_output_is_temp(false)
	end

feature {NONE} -- statement generation and compilation

	compile_die_fracture (d: DIES_XROW
			cenv: CATS_COMPILATION_ENVIRONMENT)
		-- create the cats statements for the fracture of d and
		-- compile them into cenv
	local opt: CATS_EXPR_OPTIMIZER
	do  -- start afresh for each die
	    create all_stmts.make(100)
	    -- generate the Cats statements for d
	    add_checking_cats_statements(d)
	    if cache.run_needs_mebes_files then
		add_explicit_cats_statements(d)
		add_default_cats_statements(d)
	    end
	    -- extract common subexpressions (modifies all_stmts in place)
	    if policies.use_cats_expr_optimizer then
		create opt.perform_optimization(all_stmts)
		cenv.add_a_comment("CSE optimizer stats: " + opt.summary)
	    end
	    -- and compile
	    from  all_stmts.start
	    until all_stmts.after
	    loop  all_stmts.item.compile(cenv)
		  all_stmts.forth
	    end
	    -- and dispose
	    all_stmts := Void
	end

	add_checking_cats_statements (d: DIES_XROW)
		-- compute all checking (cassert) Cats statements for d
		--  and add them onto all_stmts
	do	-- redefine (together with is_mebes_checking_performed)
	end

	add_explicit_cats_statements (d: DIES_XROW)
		-- compute all of the explicitly-specified Cats statements required to fracture d
		--  and add them onto all_stmts.
		-- (redefine per wafer_techn that actually does Cats fracture)
	do  handle_internal_error ("add_explicit_cats_statements not defined for " + name)
	end

	add_default_cats_statements (d: DIES_XROW)
		-- add missing statements based on design_techn_xform_levels
		--  and add them onto stmts.
	local lvls: LIST[RUN_LEVELS_ROW]
		a_lvl: RUN_LEVELS_ROW
		dtxl: DESIGN_TECHN_XFORM_LEVELS_ROW
		expr: CATS_EXPRESSION
	do  lvls := cache.run_level_rows
	    from  lvls.start
	    until lvls.after
	    loop  a_lvl := lvls.item
		  if not have_statement_for_level (a_lvl.mebes_extension) then
			-- create statement with default expression for lvl
			dtxl := xform_for_level(d.xform_levels, a_lvl)
			if dtxl.is_null_total_nominal_bias then
				-- no default provided
				handle_hard_error("no default Cats statement data for "
						  + a_lvl.level_name + " in " + dtxl.design_techn)
			elseif (not dtxl.is_null_source_layer_name) then
				-- normal default case: named layer and any bias
				expr := leaf (dtxl.source_layer_name) #sized# dtxl.total_nominal_bias
			elseif dtxl.total_nominal_bias = 0 then
				-- special default case: no layer & 0 bias
				expr := emptiness
			else
				handle_hard_error("improper default Cats statement data for "
						  + a_lvl.level_name + " in " + dtxl.design_techn)
			end
			all_stmts.extend (final_clevel(expr, d, a_lvl, dtxl))
		  end
		  lvls.forth
	    end
	end

	have_statement_for_level(mext: STRING): BOOLEAN
		-- does all_stmts have a statement to fracture lvl?
	local stmt: CATS_LEVEL_FRACTURE
	do  from  all_stmts.start
	    until all_stmts.after or Result
	    loop  stmt ?= all_stmts.item
		  Result := stmt /= Void and then
				equal(stmt.mebes_ext, mext)
		  all_stmts.forth
	    end
	end

	cleanup_ted: STRING
		-- script line to clean up $TED
	do  Result := su.string_of_3 ("rm ", ted_filename_prefix, "*%N")
	end

	create_is_done (d: DIES_XROW): STRING
		-- script lines to create is_done file for d
	local fname: STRING
	do  fname := cache.die_fracture_done_file(d).file_pathname
	    Result := su.string_of_3 ("cat << EOF >! ", fname, "%NEOF%N")
	end

feature {NONE} -- attributes set up once per die, for shared use

	die_fracture_pf_root: RC_FILE
		-- final die_fracture_pattern_file set name (through the '.')
		-- (needed for both the final writefile and set_final_fracture_prefix)

	overetching: BOOLEAN
		-- is this (mask job) using deliberate overetch data?

	ted_filename_prefix: STRING
		-- opening for all our temp files ($TED/<fab-id>_)

	cats_layer_source: CATS_LAYER_SOURCES
		-- remapping of leaf layer names to assembly expressions
		-- NOTE: in addition to this class' descendants, there
		--	 are also technology-specific operations buried
		--	 in this class (CATS_LAYER_SOURCES)!!!

	native_dt: DESIGN_TECHNOLOGY
		-- design_technology for (native_dt)
	native_dtxls: LIST[DESIGN_TECHN_XFORM_LEVELS_ROW]
		-- and its xforms

	all_stmts: ARRAYED_LIST[CATS_STATEMENT]
		-- collector for each compile_die_fracture execution

feature {NONE} -- temporary ($TED) filename stuff

	ted_filename(suf1, suf2: STRING): STRING
		-- generate a temporary filename
	do  Result := su.string_of_3(ted_filename_prefix, suf1, suf2)
	end

feature {NONE} -- helper functions for the descendants

	fgen: RC_FILE_GENERATOR
		-- shortcut to cache.file_generator
	do  Result := cache.file_generator
	end

	oid: OPTION_ID_VALUES
		-- values for various option_ids

	lyr: LAYER_NAME_VALUES
		-- values for all the layer_names

	lvl: LEVEL_NAME_VALUES
		-- values for all the level_names

	dtid: DESIGN_TECHN_ID_VALUES
		-- values for various design_techn_ids

	dtv: DESIGN_TECHN_VALUES
		-- values for various design_techns

	gids: GEOMETRY_ID_VALUES
		-- values for various geometry_ids

	is_scmos(a_dt: STRING): BOOLEAN
		-- kludge, but it still holds true
	do  Result := a_dt.count >= 2 and then
			((a_dt @ 1) = 'S' and (a_dt @ 2) = 'C')
	end

	is_scmos_die(d: DIES_ROW): BOOLEAN
	do  Result := is_scmos(d.design_techn)
	end

feature {NONE} -- access to useful things

	check_level_by_name(lnam: STRING): WAFER_TECHN_FAMILY_LEVELS_ROW
		-- look up a level by level_name; Void if not a level name
	do
		Result := wtfllu.first_item_passing_filter
				(cache.wafer_techn_family_level_rows,
				 agent {WAFER_TECHN_FAMILY_LEVELS_ROW}.is_my_level_name(lnam))
	end

	level_by_name(lnam: STRING): WAFER_TECHN_FAMILY_LEVELS_ROW
		-- look up a level by level_name; error if not a level name
	do
		Result := check_level_by_name (lnam)
		if Result = Void then
			handle_internal_error
				("unknown level name in family: " + lnam)
		end
	end

	level_no(lnam: STRING): INTEGER
		-- convert LEVEL_NAME to LEVEL_NO
		do
			Result := level_by_name(lnam).level_no
		end

	xform_for_level(dtxls: LIST[DESIGN_TECHN_XFORM_LEVELS_ROW]
			a_lvl: ABSTRACT_LEVEL): DESIGN_TECHN_XFORM_LEVELS_ROW
		-- DESIGN_TECHN_XFORM_LEVEL from dtxl for a_lvl
		-- Error if not found
	local	lno: INTEGER
	do
		lno := a_lvl.level_no
		Result := dtxllu.first_item_passing_filter
				(dtxls,
				 agent {DESIGN_TECHN_XFORM_LEVELS_ROW}.is_my_level_no(lno))
		if Result = Void then
			handle_internal_error
				("missing xform for level " + a_lvl.level_name +
				 " in dtid " + dtxls.first.design_techn)
		end
	end

	xform_for_die_level(d: DIES_XROW lnam: STRING): DESIGN_TECHN_XFORM_LEVELS_ROW
		-- DESIGN_TECHN_XFORM_LEVEL for die and level lnam
		require	have_die: d /= Void
			have_lnam: lnam /= Void
		do	Result := xform_for_level(d.xform_levels, level_by_name(lnam))
		end

	source_layer_name(d: DIES_XROW lnam: STRING): STRING
		-- source_layer_name for die d and level lnam
		do	Result := xform_for_die_level(d, lnam).source_layer_name
		end

	nominal_bias(d: DIES_XROW lnam: STRING): DOUBLE
		-- nominal_bias for die d and level lnam
		do	Result := xform_for_die_level(d, lnam).total_nominal_bias
		end

	bias_differential(d: DIES_XROW lnam: STRING): DOUBLE
		-- difference between d's nominal bias for lnam and the native nominal bias
	local a_lvl: ABSTRACT_LEVEL
	do  if d.design_techn_name = native_design_techn then
		Result := 0	-- save your breath
	    else
		a_lvl := level_by_name(lnam)
		Result := xform_for_level(d.xform_levels, a_lvl).total_nominal_bias -
			  xform_for_level(native_dtxls, a_lvl).total_nominal_bias
	    end
	end

	option_is_present(an_oid: STRING): BOOLEAN
		-- is option an_oid present on this run
		-- (i.e., is an_oid part of some wafer_process of a wafer lot)
		require	option_present: an_oid /= Void
		do
			if cache.wafer_techn_families_row.is_offered_option(an_oid) then
				Result := cache.run_process_rows.there_exists
						(agent {WAFER_PROCESSES_ROW}.has_option(an_oid))
			else
				handle_internal_error
					("family specifies unknown option " + an_oid)
			end
		end

	wtfllu: LIST_UTILITIES[WAFER_TECHN_FAMILY_LEVELS_ROW]
	once create Result
	end

	dtxllu: LIST_UTILITIES[DESIGN_TECHN_XFORM_LEVELS_ROW]
	once create Result
	end

feature {NONE} -- synthesis

	leaf(lnam: STRING): CATS_EXPRESSION
		-- lnam is a layer name (may be a ctemp, but that's OK);
		-- see if it has a source expression or is truly itself
	do
		Result := cats_layer_source.source_expr(lnam)
	end

	final_clevel(expr: CATS_EXPRESSION d: DIES_XROW a_lvl: ABSTRACT_LEVEL dtxl:DESIGN_TECHN_XFORM_LEVELS_ROW): CATS_LEVEL_FRACTURE
		-- generate clevel statement for a_lvl from expr
		-- augmented by the overetch_bias, if applicable,
		--  and appropriate scale factor
	local scale: DOUBLE
		fexpr: CATS_EXPRESSION
	do  fexpr := expr
	    -- add in overetch bias if appropriate, and then scaling
	    if overetching then
		fexpr := fexpr #sized# dtxl.overetch_bias
	    end
	    scale := a_lvl.scale_factor / d.design_techn.input_scale_factor
	    if scale /= 1 then
		fexpr := fexpr #scaled# scale
	    end
	    -- and create the fracture statement
	    Result := clevel(fexpr, a_lvl.mebes_extension, dtxl.mebes_spot_size)
	end

feature {NONE} -- the five common statement builders for the descendents

	add_clevel(expr: CATS_EXPRESSION lnam: STRING d: DIES_XROW)
		-- add overetch and scaling to expr, and put it out as the fracture
		-- statement for lnam
	local a_lvl: ABSTRACT_LEVEL
		dtxl: DESIGN_TECHN_XFORM_LEVELS_ROW
	do  a_lvl := level_by_name(lnam)
	    dtxl  := xform_for_level(d.xform_levels, a_lvl)
	    -- finalize and add
	    all_stmts.extend (final_clevel(expr, d, a_lvl, dtxl))
	end

	add_clevel_with_bias(expr: CATS_EXPRESSION lnam: STRING d: DIES_XROW)
		-- like add_clevel, but first augment expr with
		--  the total_nominal_bias
	local a_lvl: ABSTRACT_LEVEL
		dtxl: DESIGN_TECHN_XFORM_LEVELS_ROW
		fexpr: CATS_EXPRESSION
	do  a_lvl := level_by_name(lnam)
	    dtxl  := xform_for_level(d.xform_levels, a_lvl)
	    -- now generate the expression
	    fexpr := expr #sized# dtxl.total_nominal_bias
	    -- finalize and add
	    all_stmts.extend (final_clevel(fexpr, d, a_lvl, dtxl))
	end

	add_clevel_with_merge(expr: CATS_EXPRESSION lnam: STRING merge_dim: DOUBLE d: DIES_XROW)
		-- like add_clevel, but first augment expr with
		-- merge_dim (um) bloat/shrink to merge nearby features
		-- plus the total_nominal_bias
	require positive: merge_dim > 0
	local a_lvl: ABSTRACT_LEVEL
		dtxl: DESIGN_TECHN_XFORM_LEVELS_ROW
		fexpr: CATS_EXPRESSION
	do  a_lvl := level_by_name(lnam)
	    dtxl  := xform_for_level(d.xform_levels, a_lvl)
	    -- now generate the expression
	    fexpr := expr #sized# (merge_dim)		-- first step
	    fexpr := fexpr #sized# (dtxl.total_nominal_bias-merge_dim)
							-- second step
	    -- finalize and add
	    all_stmts.extend (final_clevel(fexpr, d, a_lvl, dtxl))
	end

	add_clevel_with_merge_X2(expr: CATS_EXPRESSION lnam: STRING merge_dim: DOUBLE d: DIES_XROW)
		-- like add_clevel, but first augment expr with 2 way
		-- merge_dim (um) bloat/shrink/bloat to get rid of slivers and gaps
		-- plus the total_nominal_bias
	local a_lvl: ABSTRACT_LEVEL
		dtxl: DESIGN_TECHN_XFORM_LEVELS_ROW
		fexpr: CATS_EXPRESSION
	do  a_lvl := level_by_name(lnam)
	    dtxl  := xform_for_level(d.xform_levels, a_lvl)
	    -- now generate the 2-way merge
	    fexpr := expr #sized# (merge_dim)		-- first step
	    fexpr := fexpr #sized# (-2*merge_dim)	-- second step
	    fexpr := fexpr #sized# (merge_dim + dtxl.total_nominal_bias)
							-- third step
	    -- final mods and turn into statement
	    all_stmts.extend (final_clevel(fexpr, d, a_lvl, dtxl))
	end

	add_ctemp(expr: CATS_EXPRESSION lnam: STRING res: DOUBLE)
		-- add a temporary fracture to the statement list.
		-- lnam must NOT match a layer name (or another ctemp)
		-- it becomes a pseudo-layer that can be
		--  used as a leaf in subsequent expressions
	do  all_stmts.extend (ctemp (expr, lnam, res))
	end

feature {NONE} -- misc helper functions

	merge_X2(inp: CATS_EXPRESSION merge_dim: DOUBLE): CATS_EXPRESSION
		-- 2 way bloat/shrink/bloat to eliminate slivers and gaps
		-- that are of size merge_dim or smaller
	do  Result := ((inp #sized# merge_dim) #sized# (-2*merge_dim)) #sized# merge_dim
	end

	unknown_design_techn(a_dt: STRING)
		-- fell through all known design technologies
		do
			handle_internal_error
				("unsupported design technology " + a_dt)
		end

end
