
indexing

	author: "Generated"
	date: "Thu Jun 12 13:50:27 PDT 2003"

class WAFER_TECHNS_ROW
inherit
	WAFER_TECHNS_RAW
	ROW_COUNT_CHECK_ACCESS
		export 
			{NONE} all
		end
	NULL_VALUE_CHECK_ACCESS
		export 
			{NONE} all
		end
	ITEM_COUNT_RANGES_ACCESS
		export 
			{NONE} all
		end
	NUMBER_UTILITIES_ACCESS
		export
			{NONE} all
		end
	CONDITION_UTILITIES
		export
			{NONE} all
		end

create
	make_template, make_null

feature -- Properties

	wafer_techns_with_name (wt_name: STRING conn: DB_CONNECTION range: ITEM_COUNT_RANGE): LIST [like Current] is
			-- Wafer techns with name `wt_name'
		require
			wt_name_exists: wt_name /= Void
			connection_exists: conn /= Void
			connection_open: conn.is_open
			range_exists: range /= Void
		local
			b: DB_BIND_S
		do
			create b.make (wt_name)
			Result := all_rows_filtered (conn, "wafer_techn = :1", b)
			check_row_count (Result, range)
		ensure
			result_exists: Result /= Void
		end

	possible_wafer_techns_ordered (dt, run_type: STRING conn: DB_CONNECTION range: ITEM_COUNT_RANGE): LIST [like Current] is
			-- Current wafer techns compatible with
			-- design type `dt' and `run_type'
		require
			design_type_exists: dt /= Void
			run_type_exists: run_type /= Void
			connection_exists: conn /= Void
			range_exists: range /= Void
		local
			b: DB_BIND_SSS
			wt_status: WAFER_TECHN_STATUS_VALUES
		do
			create b.make (wt_status.Cur, dt, run_type)
			Result := all_rows_filtered_ordered (conn, "wafer_techn_status = :1 and design_type = :2 and %N  exists %N  (select 1 from wafer_techn#run_types wtrt%N   where wtrt.wafer_techn = wt.wafer_techn and wtrt.run_type = :3)", b, "wafer_techn")
			check_row_count (Result, range)
		ensure
			result_exists: Result /= Void
		end

	current_wafer_techns_ordered (dt: STRING conn: DB_CONNECTION range: ITEM_COUNT_RANGE): LIST [like Current] is
			-- Current wafer techns compatible with
			-- design type `dt'
		require
			design_type_exists: dt /= Void
			connection_exists: conn /= Void
			range_exists: range /= Void
		local
			b: DB_BIND_SS
			wt_status: WAFER_TECHN_STATUS_VALUES
		do
			create b.make (wt_status.Cur, dt)
			Result := all_rows_filtered_ordered (conn, "wafer_techn_status = :1 and design_type = :2", b, "wafer_techn")
			check_row_count (Result, range)
		ensure
			result_exists: Result /= Void
		end

	has_wafer_techn (s: STRING): BOOLEAN is
			-- Does `Current' have `s' as its wafer techn?
		require
			s_not_void: s /= Void
		do
			Result := wafer_techn.is_equal (s)
		end

	non_null_design_techn_scribe: like design_techn_scribe is
			-- Value of `design_techn_scribe', with error check 
			-- for nullness
		do
			check_not_null ("design_techn_scribe", is_null_design_techn_scribe, Current)
			Result := design_techn_scribe
		end

	external_design_die_offset (an_other_border: DOUBLE): CONST_POINT
			-- Offset of center of design from center of die
			-- for dies whose effective overhead style is EXTERNAL,
			-- using `an_other_border' instead of `other_border'
		do
			create Result.make_xy (0.0, (an_other_border - (top_border.min (an_other_border) + overhead_strip_height + design_strip_separation)) / 2.0)
		ensure
			result_not_void: Result /= Void
		end

	minimum_scribe_width: DOUBLE is
			-- Minimum width of scribes for wafer techn
		require
			has_design_techn_scribe: not is_null_design_techn_scribe
		local
			layer: DESIGN_TECHN_LAYERS_ROW
			found: BOOLEAN
		do
			if not is_cached_scribe_layers then
				fetch_scribe_layers
			end
			from
				Result := 0
				found := False
				cached_scribe_layers.start
			until
				cached_scribe_layers.after
			loop
				layer := cached_scribe_layers.item
				if not layer.is_null_near_edge then
					found := True
					Result := Result.max (layer.near_edge + layer.min_width)
						-- Guarantee that no scribe layer
						-- can disappear or go below 
						-- minimum width
				end
				cached_scribe_layers.forth
			end
			if not found then
				handle_hard_error ("cannot compute minimum scribe width for wafer techn " + wafer_techn + " - no scribe layers with non-null near_edge value")
			elseif Result <= 0 then
				handle_hard_error ("cannot compute minimum scribe width for wafer techn " + wafer_techn + "%NNear_edge + min_width <= 0 for every scribe layer with non-null near_edge value")
			end
		end

	top_overhead_for_other_border (an_other_border: DOUBLE): DOUBLE
			-- Amount of overhead added to top side of design,
			-- using `an_other_border' instead of `other_border'
			-- in `Current'.  Applicable only when die overhead
			-- style is EXTERNAL
		do
			Result := design_strip_separation + overhead_strip_height + an_other_border.min (top_border)
		end

	top_overhead_description (an_overhead_style: STRING an_other_border: DOUBLE): STRING
			-- Description of top overhead
		local
			style: OVERHEAD_STYLE_VALUES
			ext_val, int_val: STRING
		do
			if an_overhead_style.is_equal (style.External_) then
				Result := number_as_string (top_overhead_for_other_border (an_other_border))
			elseif an_overhead_style.is_equal (style.None) or an_overhead_style.is_equal (style.Internal) then
				Result := number_as_string (an_other_border)
			elseif an_overhead_style.is_equal ("") then
				ext_val := number_as_string (top_overhead_for_other_border (an_other_border))
				int_val := number_as_string (an_other_border)
				create Result.make (30)
				if not int_val.is_empty then
					Result.append (int_val)
					Result.append (" (internal)")
				end
				if not ext_val.is_empty then
					Result.append ("   ")
					Result.append (ext_val)
					Result.append (" (external)")
				end
			else
				Result := "(unknown style " + an_overhead_style + ")"
			end
		end

	project_overhead (an_overhead_style: STRING an_other_border: DOUBLE): CONST_POINT is
			-- Amount of overhead in X and Y dimensions that
			-- is added around a cust project or test vehicle
			-- using `an_overhead_style' and `an_other_border'
		require
			an_overhead_style_not_void: an_overhead_style /= Void
		local
			style: OVERHEAD_STYLE_VALUES
		do
			if an_overhead_style.is_equal (style.External_) then
				create Result.make_xy (2 * an_other_border, top_overhead_for_other_border (an_other_border) + an_other_border)
			else
				create Result.make_xy (2 * an_other_border, 2 * an_other_border)
			end
		end

	uniform_white_space_border (an_overhead_style: STRING an_other_border: DOUBLE): DOUBLE
			-- Amount of white space added around a cust
			-- project or test vehicle using `an_overhead_style'
			-- and `an_other_border'
		local
			style: OVERHEAD_STYLE_VALUES
		do
			if an_overhead_style.is_equal (style.External_) then
				Result := an_other_border.min (top_border)
			else
				Result := an_other_border
			end
		end

	die_size_with_overhead (an_overhead_style: STRING an_other_border: DOUBLE size: CONST_POINT): CONST_POINT is
			-- Size of die for project whose design size
			-- is `size' using `an_overhead_style' and
			-- `an_other_border'.  Result is rounded up to
			-- multiple of packing grid resolution
		require
			size_not_void: size /= Void
			an_overhead_style_not_void: an_overhead_style /= Void
		local
			res: DOUBLE
		do
			res := packing_grid_resolution
			Result := size.twin
			Result := Result + project_overhead (an_overhead_style, an_other_border)
			create Result.make_xy (ceiling_multiple_of_res (Result.x, res), ceiling_multiple_of_res (Result.y, res))
		end

	project_size_without_overhead (an_overhead_style: STRING an_other_border: DOUBLE size: CONST_POINT): CONST_POINT is
			-- Maximum size of project for die whose size
			-- is `size' using `an_overhead_style' and
			-- `an_other_border'
		require
			size_not_void: size /= Void
			an_overhead_style_not_void: an_overhead_style /= Void
		local
			res: DOUBLE
		do
			res := packing_grid_resolution
			Result := size.twin
			Result := Result - project_overhead (an_overhead_style, an_other_border)
			create Result.make_xy (floor_multiple_of_res (Result.x, res), floor_multiple_of_res (Result.y, res))
		end

	needs_merged_gds_files: BOOLEAN
			-- Does `Current' require that we generate
			-- merged GDS files?
		do
			Result := not is_null_gds_merge_mode
		end

feature {NONE} -- Implementation

	ceiling_multiple_of_res (size: DOUBLE n: DOUBLE): DOUBLE is
			-- Size increased (if necessary) to multiple of n
		require
			size_positive: size >= 0.0
			n_positive: n > 0.0
		do
			if nu.is_multiple_of_resolution (size, n, scale_top_border) then
				Result := size
			else
				Result := nu.ceiling_real_64 (size / n) * n
			end
		end
	
	floor_multiple_of_res (size: DOUBLE n: DOUBLE): DOUBLE is
			-- Size decreased (if necessary) to multiple of n
		require
			n_positive: n > 0.0
		do
			if nu.is_multiple_of_resolution (size, n, scale_top_border) then
				Result := size
			else
				Result := nu.floor_real_64 (size / n) * n
			end
		end
	
	fetch_scribe_layers is
			-- Fetch layers for scribe design techn
			-- and put in `cached_scribe_layers'
		local
			row: DESIGN_TECHN_LAYERS_ROW
		do
			create row.make_template
			cached_scribe_layers := row.layers_for_dt (design_techn_scribe, connection, expected_count.At_least_one)
		end

	number_as_string (n: DOUBLE): STRING
			-- Value of `n' as a string suitable for display,
			-- using `scale_top_border' as the scale
		do
			Result := nu.formatted_number_or_integer (n, Scale_top_border)
		end

	is_cached_scribe_layers: BOOLEAN
			-- Are layers for scribe design techn
			-- currently cached?

	cached_scribe_layers: LIST [DESIGN_TECHN_LAYERS_ROW]
			-- Layers for scribe design techn
			-- cached in memory
	
	
end
