
indexing
	
	description: "Column of widgets with associated layout properties"
	author: "David Hollenberg";
	date: "November 5, 2003"

class WIDGET_COLUMN
inherit
	ARRAYED_LIST [EV_WIDGET]
		undefine
			is_equal, copy
		redefine
			make
		end
	WIDGET_ALIGNMENT
		export
			{NONE} all
		end

create
	make

create {WIDGET_COLUMN}
	make_filled

feature {NONE} -- Creation

	make (n: INTEGER) is
		do
			precursor (n)
			is_expanded := True
			align_none
		end

feature -- Properties

	padding: INTEGER
			-- Padding between widgets

	border_width: INTEGER
			-- Width of border around column

	is_expanded: BOOLEAN
			-- Should items in list expand to occupy available
			-- spare space?

	is_unaligned: BOOLEAN is
			-- Should items in list not be aligned?
		do
			Result := alignment = No_alignment
		end
		
	is_left_aligned: BOOLEAN is
			-- Should items in list be left aligned?
		do
			Result := alignment = Ev_text_alignment_left
		end
		
	is_center_aligned: BOOLEAN is
			-- Should items in list be center aligned?
		do
			Result := alignment = Ev_text_alignment_center
		end

	is_right_aligned: BOOLEAN is
			-- Should items in list be right aligned?
		do
			Result := alignment = Ev_text_alignment_right
		end

	box: EV_VERTICAL_BOX is
			-- Box with widgets in column laid out as requested
		local
			widget: EV_WIDGET
			next_item: like item
		do
			create Result
			Result.set_padding (padding)
			Result.set_border_width (border_width)
			from
				start
			until
				after
			loop
				next_item := item
				if next_item = Void then
					create {EV_HORIZONTAL_BOX} widget
				else
					widget := aligned_widget (next_item)
				end
				Result.extend (widget)
				forth
			end
			
		end

feature -- Modification

	set_padding (n: INTEGER) is
			-- Set `padding' to `n'
		require
			value_nonnegative: n >= 0
		do
			padding := n
		ensure
			padding_set: padding = n
		end

	set_border_width (n: INTEGER) is
			-- Set `border_width' to `n'
		require
			value_nonnegative: n >= 0
		do
			border_width := n
		ensure
			border_width_set: border_width = n
		end

	enable_expand is
			-- Make items in list expand to occupy available
			-- spare space
		do
			is_expanded := True
		end

	disable_expand is
			-- Make items in list not expand to occupy available
			-- spare space
		do
			is_expanded := False
		end

	align_none is
			-- Do not align list items
		do
			alignment := No_alignment
		end

	align_left is
			-- Left-align list items
		do
			alignment := Ev_text_alignment_left
		end

	align_center is
			-- Center-align list items
		do
			alignment := Ev_text_alignment_center
		end

	align_right is
			-- Right-align list items
		do
			alignment := Ev_text_alignment_right
		end

feature {NONE} -- Implementation

end
