
indexing
	
	description: "View of a reticle model, which can be displayed on a %
		%drawable canvas"
	author: "David Hollenberg"
	date: "July 28, 2006"

class RETICLE_VIEW
inherit
	MODEL_VIEW
		redefine
			anchored_view_element,
			anchored_model_element,
			anchored_model,
			palette,
			redraw,
			make,
			set_initial_model_elements
		end
	PACKING_DELEGATED_STATUS_REPORTER
		export
			{ANY} reporter, set_reporter
			{NONE} all
		end
	CONDITION_UTILITIES
		export
			{NONE} all
		end
	SHARED_RETICLE_LABEL_GENERATORS
		export
			{NONE} all
		end
	SHARED_RETICLE_OPACITY_GENERATORS
		export
			{NONE} all
		end
	MOSIS_EV_STOCK_COLORS_ACCESS
		export
			{NONE} all
		end
	VIEW_ELEMENT_FILTERS
		export
			{NONE} all
		end
	VIEW_ELEMENT_ACTIONS
		export
			{NONE} all
		end
	STRING_UTILITIES_ACCESS
		export
			{NONE} all
		end
	RC_POLICIES_ACCESS
		export
			{NONE} all
		end
	NAMED_VIEW_NAMES
		export
			{NONE} all
		end

create
	make

feature {NONE} -- Creation
	
	make (a_model: like anchored_model a_drawable: EV_DRAWING_AREA)
			-- Create with `a_model' and `a_drawable'
		do
			precursor (a_model, a_drawable)
			create named_view_change_actions
			create locked_mode_change_actions
			create_copies_table
			create sorter
			create quick_sorter
			create size_x_desc_comparator
			create size_y_desc_comparator
			create spread_by_width_asc_comparator
			create spread_by_width_desc_comparator
			create spread_by_height_asc_comparator
			create spread_by_height_desc_comparator
			create spread_by_x_position_asc_comparator
			create spread_by_x_position_desc_comparator
			create spread_by_y_position_asc_comparator
			create spread_by_y_position_desc_comparator
			create packing_name_asc_comparator
			create packing_name_desc_comparator
			create scan_x_comparator
			create scan_y_comparator
			create abut_move_distance.make_xy (0.0, 0.0)
			create reticle_change_history_list.make
			create change_history_list_position_actions
			create {ARRAYED_LIST [like anchored_view_element]} partially_selected_elements.make (0)
			create {ARRAYED_LIST [like anchored_view_element]} rotation_locked_elements.make (0)
			create {ARRAYED_LIST [like anchored_view_element]} fixed_orientation_elements.make (0)
			create {ARRAYED_LIST [like anchored_view_element]} duplication_locked_elements.make (0)
			create {ARRAYED_LIST [like anchored_view_element]} never_duplicatable_elements.make (0)
			create {ARRAYED_LIST [like anchored_view_element]} unmovable_elements.make (0)
			create {ARRAYED_LIST [like anchored_view_element]} new_elements.make (0)
			create {ARRAYED_LIST [like anchored_view_element]} spread_elements.make (0)
			create {ARRAYED_LIST [like anchored_view_element]} containing_elements.make (0)
			initialize_selection_lists
		end

feature -- Properties

	max_db_value: DOUBLE
			-- Maximum value that can fit in the database
		do
			Result := model.max_db_value
		end

	palette: RETICLE_VIEW_COLORS
			-- Palette of colors to use for drawing elements

	current_label_generator: RETICLE_VIEW_ELEMENT_LABEL_GENERATOR
			-- Generator for text to display to label elements,
			-- set by last call to set a named view

	current_color_generator: RETICLE_VIEW_ELEMENT_COLOR_GENERATOR
			-- Generator for colors to use to display elements,
			-- set by last call to set a named view

	current_fill_generator: RETICLE_VIEW_ELEMENT_FILL_GENERATOR
			-- Generator for "filled" indicator to use to
			-- display elements, set by last call to set a
			-- named view

	current_opacity_generator: RETICLE_VIEW_ELEMENT_OPACITY_GENERATOR
			-- Generator for opaqueness or transparency
			-- indicator to use to display elements,
			-- set by last call to set a named view

	current_view: RETICLE_NAMED_VIEW
			-- Currently displayed named view

	current_view_name: STRING
			-- Name of currently displayed named view
		do
			Result := current_view.name
		end

	selected_element: like anchored_view_element
			-- The single selected element for certain operations
			-- (e.g., Resize Scribe)

	partially_selected_elements: LIST [like anchored_view_element]
			-- Elements to which last Rotate operation
			-- could not be applied because they were
			-- partially selected

	rotation_locked_elements: LIST [like anchored_view_element]
			-- Elements to which last Rotate operation
			-- could not be applied because they were
			-- rotation locked

	fixed_orientation_elements: LIST [like anchored_view_element]
			-- Elements to which last Rotate operation
			-- could not be applied because they were
			-- rotation locked

	duplication_locked_elements: LIST [like anchored_view_element]
			-- Elements to which last Duplicate operation
			-- could not be applied because they were
			-- locked against duplication

	never_duplicatable_elements: LIST [like anchored_view_element]
			-- Elements to which last Duplicate operation
			-- could not be applied because they were
			-- never duplicatable

	unmovable_elements: LIST [like anchored_view_element]
			-- Elements to which last Spread operation could
			-- not be applied because they were locked in
			-- X or Y

	new_elements: LIST [like anchored_view_element]
			-- New elements added by last Duplicate operation

	found_elements: LIST [like anchored_view_element]
			-- Elements which were found by last
			-- Search operation

	deleted_elements: LIST [like anchored_view_element]
			-- Elements which were deleted by last
			-- Delete operation

	delete_locked_elements: LIST [like anchored_view_element]
			-- Elements which could not be deleted by last
			-- Delete operation because they were delete-locked

	delete_failed_elements: LIST [like anchored_view_element]
			-- Elements which could not be deleted by last
			-- Delete operation because they are projects for
			-- which delete failed (due to database lock failure)

	projects_to_delete: LIST [PACKING_PROJECTS_MROW]
			-- Projects whose last copy is about to be deleted,
			-- set by last call to `compute_projects_to_delete'

	non_project_dies_to_delete: LIST [PACKING_DIES_MROW]
			-- Dies for projects whose last copy is about
			-- to be deleted, set by last call to
			-- `compute_projects_to_delete'

	changed_scribes: LIST [like anchored_view_element]
			-- Scribes which were changed by last scribe
			-- Trim/Separate/etc operation

	locked_scribes: LIST [like anchored_view_element]
			-- Scribes which could not be changed by last
			-- scribe Trim/Separate/etc operation because
			-- they were locked in the length dimension

	undeletable_scribes: LIST [like anchored_view_element]
			-- Scribes which were not deletable by last
			-- Join Scribes operation

	overlap_tee_count: INTEGER
			-- Number of overlaps removed by last Unoverlap
			-- operation which were tee intersections

	overlap_cross_count: INTEGER
			-- Number of overlaps removed by last Unoverlap
			-- operation which were cross intersections

	overlap_enclosed_count: INTEGER
			-- Number of overlaps removed by last Unoverlap
			-- operation where one of the scribes was completely
			-- contained in the other

	overlap_removed_count: INTEGER
			-- Number of overlaps removed by last Unoverlap
			-- operation

	overlap_resize_failed_count: INTEGER
			-- Number of overlaps retained by last Unoverlap
			-- operation because the scribe to be resized was
			-- locked in length dimension

	overlap_delete_failed_count: INTEGER
			-- Number of overlaps retained by last Unoverlap
			-- operation because the scribes to be deleted
			-- was delete-locked

	deleted_scribes: LIST [PACKING_DIES_MROW]
			-- Dies for scribes deleted during last scribe
			-- Separate operation

	never_isolated_projects: LIST [PACKING_DIES_ROW]
			-- Projects that were never isolated in any
			-- cut pattern
		require
			is_cut_mode: is_cut_mode
		local
			l_sorter: LIST_QUICK_SORTER [PACKING_DIES_ROW]
			l_comparator: DESIGN_NO_COMPARATOR [PACKING_DIES_ROW]
		do
			Result := model.never_isolated_projects
			create l_sorter
			create l_comparator
			Result := l_sorter.sorted (Result, l_comparator)
		end

	selected_dies: LIST [RETICLE_DIE]
			-- Dies for all selected objects that are dies
		do
			create {ARRAYED_LIST [RETICLE_DIE]} Result.make (selected_count)
			add_die_action.set_die_list (Result)
			apply_action_with_filter (selected_filter, add_die_action)
		end

	selected_customer_projects: LIST [RETICLE_DIE]
			-- Dies for all selected objects that are customer
			-- projects
		do
			create {ARRAYED_LIST [RETICLE_DIE]} Result.make (selected_count)
			add_die_action.set_die_list (Result)
			apply_action_with_filter (cust_project_filter, add_die_action)
		end

	selected_projects: LIST [like anchored_view_element]
			-- Selected objects that are projects (customer dies
			-- or test vehicles)
		do
			create {ARRAYED_LIST [like anchored_view_element]} Result.make (selected_count)
			add_action.set_elements (Result)
			apply_action_with_filter (selected_project_filter, add_action)
		end

	selected_cuts: LIST [like anchored_view_element]
			-- Selected objects that are cuts
		do
			create {ARRAYED_LIST [like anchored_view_element]} Result.make (selected_count)
			add_action.set_elements (Result)
			apply_action_with_filter (selected_cut_filter, add_action)
		end

	customer_projects: LIST [PACKING_DIES_MROW]
			-- List of distinct dies for all objects
			-- that are customer projects
		local
			l_saved_elements: like elements
		do
			create {ARRAYED_LIST [PACKING_DIES_MROW]} Result.make (elements.count)
			add_unique_die_action.set_die_list (Result)
			if is_cut_mode then
				l_saved_elements := elements
				elements := pack_mode_context.elements
				apply_action_with_filter (cust_project_filter, add_unique_die_action)
				elements := l_saved_elements
			else
				apply_action_with_filter (cust_project_filter, add_unique_die_action)
			end
		rescue
			if l_saved_elements /= Void then
				elements := l_saved_elements
			end
		end

	selected_customer_projects_with_multiple_etest_ids: LIST [PACKING_DIES_MROW]
			-- List of distinct dies for all selected
			-- objects that are customer projects and that
			-- have more than one compatible wafer etest
			-- ID
		do
			create {ARRAYED_LIST [PACKING_DIES_MROW]} Result.make (selected_count)
			add_unique_die_action.set_die_list (Result)
			apply_action_with_filter (selected_cust_project_etest_filter, add_unique_die_action)
		end

	fillers: LIST [PACKING_DIES_MROW]
			-- List of distinct dies for fillers
		do
			create {ARRAYED_LIST [PACKING_DIES_MROW]} Result.make (elements.count)
			add_unique_die_action.set_die_list (Result)
			apply_action_with_filter (filler_filter, add_unique_die_action)
		end

	rotated_180_elements: LIST [PACKING_DIES_MROW]
			-- List of distinct dies for objects that are
			-- rotated 180 degrees clockwise
		do
			create {ARRAYED_LIST [PACKING_DIES_MROW]} Result.make (elements.count)
			add_unique_die_action.set_die_list (Result)
			apply_action_with_filter (rotated_180_filter, add_unique_die_action)
		end

	rotated_270_elements: LIST [PACKING_DIES_MROW]
			-- List of distinct dies for objects that are
			-- rotated 270 degrees clockwise
		do
			create {ARRAYED_LIST [PACKING_DIES_MROW]} Result.make (elements.count)
			add_unique_die_action.set_die_list (Result)
			apply_action_with_filter (rotated_270_filter, add_unique_die_action)
		end

	is_duplicate_valid: BOOLEAN
			-- Were there any elements which could not be
			-- duplicated for last request to start a Duplicate
			-- operation?
		do
			Result := duplication_locked_elements.is_empty and never_duplicatable_elements.is_empty
		end

	is_duplicate_far_enough: BOOLEAN
			-- Did mouse pointer move at least
			-- `minimum_dup_distance_in_pixels' in X or Y dimension
			-- from original position for a Duplicate operation?

	is_move_valid: BOOLEAN
			-- Were there any elements which could not be
			-- moved for last request to start a Spread, Align
			-- Move Absolute or Move Relative operation?
		do
			Result := unmovable_elements.is_empty and not had_error
		end

	move_distance: CONST_POINT
			-- Distance moved in X and Y for last move operation

	is_scan_valid: BOOLEAN
			-- Was last horizontal or vertical scan successful?
		do
			Result := not had_error
		end

	is_scribe_join_valid: BOOLEAN
			-- Was last Join Scribes operation valid?
		do
			Result := locked_scribes.is_empty and undeletable_scribes.is_empty
		end

	last_size: CONST_POINT
			-- Size of last filler or scribe created

	is_last_scribe_horizontal: BOOLEAN
			-- Was last scribe created horizontal, rather than
			-- vertical?

	last_horizontal_cut_count: INTEGER
			-- Number of horizontal cuts created or deleted

	last_vertical_cut_count: INTEGER
			-- Number of vertical cuts created or deleted

	last_cut_text: STRING
			-- Text that describes number of horizontal and
			-- vertical cuts affected by last cut creation or
			-- deletion operation
		local
			h, v: INTEGER
		do
			h := last_horizontal_cut_count
			v := last_vertical_cut_count
			if h > 0 and v > 0 then
				Result := h.out + " horizontal " + su.number (h, "cut", "cuts") + " and " + v.out + " vertical " + su.number (v, "cut", "cuts")
			elseif h > 0 then
				Result := h.out + " horizontal " + su.number (h, "cut", "cuts")
			elseif v > 0 then
				Result := v.out + " vertical " + su.number (v, "cut", "cuts")
			else
				Result := "0 cuts"
			end
		end

	last_isolated_count: INTEGER
			-- Number of projects isolated by last operation

	last_unisolated_count: INTEGER
			-- Number of projects unisolated by last operation

	last_isolation_failure_count: INTEGER
			-- Number of projects whose isolation could not
			-- be toggled

	last_isolated_text: STRING
			-- Text describing number of projects isolated
			-- and unisolated by last operation and number
			-- of failures
		local
			l_isolated, l_unisolated, l_failures: INTEGER
		do
			create Result.make (30)
			l_isolated := last_isolated_count
			l_unisolated := last_unisolated_count
			l_failures := last_isolation_failure_count
			if l_isolated > 0 or l_failures > 0 then
				Result.append_integer (l_isolated)
				Result.append (" isolated")
			end
			if l_unisolated > 0 or l_failures > 0 then
				if not Result.is_empty then
					Result.append (", ")
				end
				Result.append_integer (l_unisolated)
				Result.append (" unisolated")
			end
			if l_failures > 0 then
				if not Result.is_empty then
					Result.append (", ")
				end
				Result.append_integer (l_failures)
				Result.extend (' ')
				Result.append (su.number (l_failures, "failure", "failures"))
			end
		end

	default_scribe_width: DOUBLE
			-- Default width for horizontal and vertical scribes
		do
			Result := model.default_scribe_width
		end

	minimum_separation: DOUBLE
			-- Minimum separation between object that is a
			-- project (contains circuitry) and other objects
			-- or field of view edge
		do
			Result := model.minimum_separation
		end

	minimum_die_size: DOUBLE
			-- Minimum valid size of object that is a die
		do
			Result := model.minimum_die_size
		end

	created_small_object_threshold: DOUBLE
			-- Size in either dimension (length dimension only
			-- for scribes) below which objects created by the
			-- user during packing (scribes and fillers) are
			-- considered small
		do
			Result := model.created_small_object_threshold
		end

	horizontal_scribe_width: DOUBLE
			-- Current width for newly created horizontal scribes
		do
			if has_custom_horizontal_scribe_width then
				Result := custom_horizontal_scribe_width
			else
				Result := default_scribe_width
			end
		end

	vertical_scribe_width: DOUBLE
			-- Current width for newly created vertical scribes
		do
			if has_custom_vertical_scribe_width then
				Result := custom_vertical_scribe_width
			else
				Result := default_scribe_width
			end
		end

	custom_horizontal_scribe_width: DOUBLE
			-- Last custom width set for newly created horizontal
			-- scribes

	custom_vertical_scribe_width: DOUBLE
			-- Last custom width set for newly created vertical
			-- scribes

	has_custom_horizontal_scribe_width: BOOLEAN
			-- Should the value of `custom_horizontal_scribe_width'
			-- be used for new scribes instead of
			-- `default_scribe_width'?

	has_custom_vertical_scribe_width: BOOLEAN
			-- Should the value of `custom_vertical_scribe_width'
			-- be used for new scribes instead of
			-- `default_scribe_width'?

	minimum_scribe_dimension_in_pixels: INTEGER
			-- Minimum width or height of scribe creation
			-- box in pixels for it to be valid (to
			-- prevent accidental creation of very small scribes)

	center_fits_in_database (a_center: CONST_POINT): BOOLEAN
			-- Will model location `a_center' fit in database
			-- (in PACKING_DIE_LOCATIONS table)?  `a_center'
			-- is a location in model coordinates, not a
			-- field-relative center
		require
			a_center_not_void: a_center /= Void
		do
			Result := model.center_fits_in_database (a_center)
		end

	fields: LIST [like anchored_view_element]
			-- Fields for reticle
		do
			if cached_fields = Void then
				fetch_fields
			end
			Result := cached_fields
		end

	reticle_fields: LIST [RETICLE_FIELD]
			-- Fields for reticle, as list of RETICLE_FIELD
		do
			Result := model.fields
		end

	fields_bounding_box: CONST_RECTANGLE
			-- Bounding box of fields for reticle
		do
			Result := model.fields_bounding_box
		end

	has_fixed_element: BOOLEAN
			-- Are there any die elements that are fixed in
			-- X and/or Y?
		do
			compute_fixed_and_locked_action.initialize
			apply_action_with_filter (die_filter, compute_fixed_and_locked_action)
			Result := compute_fixed_and_locked_action.has_fixed_element
		end

	has_locked_element: BOOLEAN
			-- Are there any die elements that are locked in
			-- X and/or Y?
		do
			compute_fixed_and_locked_action.initialize
			apply_action_with_filter (die_filter, compute_fixed_and_locked_action)
			Result := compute_fixed_and_locked_action.has_locked_element
		end

	model_elements: LIST [like anchored_model_element]
			-- Model elements for view
		do
			create {ARRAYED_LIST [like anchored_model_element]} Result.make (elements.count)
			from
				elements.start
			until
				elements.after
			loop
				Result.extend (elements.item.model_element)
				elements.forth
			end
		end

	named_view_change_actions: ACTION_SEQUENCE [TUPLE [STRING]]
			-- Actions to be performed when currently
			-- displayed Named View changes.  New view name
			-- is passed as argument

	locked_mode_change_actions: ACTION_SEQUENCE [TUPLE [BOOLEAN]]
			-- Actions to be performed when mode changes
			-- between Fixed and Locked (only called when
			-- this change is the result of an undo/redo).
			-- New locked mode value is passed as argument

	selectable_count: INTEGER
			-- Number of items that are selectable
		do
			count_action.set_count (0)
			apply_action_with_filter (selectable_filter, count_action)
			Result := count_action.count
		end

	possible_wafer_etest_ids: LIST [STRING]
			-- List of wafer etest IDs that are compatible with
			-- at least one element of view
		local
			l_sorter: LIST_BUBBLE_SORTER [STRING]
			l_comparator: STRING_COMPARATOR
		do
			compute_possible_wafer_etest_names
			create l_sorter
			create l_comparator
			Result := l_sorter.sorted (compute_wafer_etest_ids_action.wafer_etest_ids, l_comparator)
		end

	possible_wafer_etest_names: LIST [WAFER_ETEST_NAMES_ROW]
			-- List of wafer etest names rows that are
			-- compatible with at least one element of
			-- view
		local
			l_sorter: LIST_BUBBLE_SORTER [WAFER_ETEST_NAMES_ROW]
			l_comparator: WAFER_ETEST_ID_COMPARATOR [WAFER_ETEST_NAMES_ROW]
		do
			compute_possible_wafer_etest_names
			create l_sorter
			create l_comparator
			Result := l_sorter.sorted (compute_wafer_etest_ids_action.wafer_etest_names, l_comparator)
		end

	assigned_wafer_etest_names: LIST [WAFER_ETEST_NAMES_ROW]
			-- List of wafer etest names rows that are
			-- assigned to at least one element of
			-- view.  If no customer projects,  returns
			-- list of all wafer etest name rows for wafer techn
		require
			is_cut_mode: is_cut_mode
		local
			l_sorter: LIST_BUBBLE_SORTER [WAFER_ETEST_NAMES_ROW]
			l_comparator: WAFER_ETEST_ID_COMPARATOR [WAFER_ETEST_NAMES_ROW]
			l_names: like assigned_wafer_etest_names
		do
			compute_assigned_wafer_etest_names
			l_names := compute_assigned_etest_ids_action.wafer_etest_names
			if l_names.is_empty then
				l_names := model.all_wafer_etest_names
			end
			create l_sorter
			create l_comparator
			Result := l_sorter.sorted (l_names, l_comparator)
		end

	possible_new_cut_pattern_numbers: LIST [INTEGER]
			-- List of valid cut pattern numbers for a new cut
			-- pattern (holes plus max+1)
		local
			c: CURSOR
			list: ARRAYED_LIST [INTEGER]
			n, low, high: INTEGER
		do
			-- Make list of all currently used cut pattern numbers,
			-- without moving cursor

			create list.make (cut_patterns.count)
			c := cut_patterns.cursor
			from
				cut_patterns.start
			until
				cut_patterns.after
			loop
				list.extend (cut_patterns.item.cut_pattern_no)
				cut_patterns.forth
			end
			cut_patterns.go_to (c)
			
			-- Add unused cut pattern numbers to Result

			create {ARRAYED_LIST [INTEGER]} Result.make (10)
			from
				low := 1
				list.start
			until
				list.after
			loop
				high := list.item - 1
				from
					n := low
				until
					n > high
				loop
					Result.extend (n)
					n := n + 1
				end
				low := list.item + 1
				list.forth
			end
			
			-- Add max+1 to Result
			
			if list.is_empty then
				n := 0
			else
				n := list.last
			end
			Result.extend (n + 1)
		end

	reticle_change_history_list: TWO_WAY_LIST [RETICLE_CHANGE]
			-- History list of changes that can be undone/redone

	reticle_change_history_capacity: INTEGER
			-- Maximum number of changes in reticle packer history
			-- list
			
	last_change: RETICLE_CHANGE
			-- Last change undone or redone

	last_change_count: INTEGER
			-- Number of simple changes actually made for
			-- last operation that made changes that go
			-- into history list (as one changes)
		do
			Result := current_changes.count
		end
			
	change_history_list_position_actions: ACTION_SEQUENCE [TUPLE [BOOLEAN, BOOLEAN]]
			-- Actions to be performed when current position in
			-- `reticle_change_history_list' may have changed.
			-- Values passed are whether position is at start
			-- and at end of list

	corner_snap_limit_in_pixels: INTEGER
			-- Maximum number of pixels from edge to corner
			-- to allow snapping user-selected point for distance
			-- measurement to nearest visible corner

	measure_marker_size_in_pixels: INTEGER
			-- Width and height in pixels of marker drawn to
			-- show location of measurement point

	measure_marker_line_width: INTEGER
			-- Width of lines used to draw marker displayed
			-- to show a measurement point graphically,
			-- in pixels

	measured_sum_x: DOUBLE
			-- Sum of widths of selected objects for last
			-- call to `measure_selected_items'

	measured_sum_y: DOUBLE
			-- Sum of heights of selected objects for last
			-- call to `measure_selected_items'

	first_measure_point: POINT
			-- First point for Measure Distance operation

	second_measure_point: POINT
			-- Second point for Measure Distance operation

	centered_count: INTEGER
			-- Number of projects centered in cut die (only
			-- counting the ones actually moved) by last
			-- Center operation

	current_context_copy: RETICLE_VIEW_CONTEXT
			-- New object with copy of current context
		do
			set_initial_selected_elements
			create Result
			Result.set_elements (elements)
			Result.set_selected_elements (initial_selected_elements)
			Result.set_copies_table (copies_table)
			Result.set_view (current_view)
			Result.set_displayed_model_area (displayed_model_area.twin)
			Result.set_displayed_area_history_list (displayed_area_history_list)
			Result.set_reticle_change_history_list (reticle_change_history_list)
			Result.set_model_context (model.current_context_copy)
		ensure
			result_not_void: Result /= Void
		end

	is_isolated_in_any_cut_pattern (an_element: like anchored_view_element): BOOLEAN
			-- Is `an_element' isolated in any cut pattern?
		local
			c: CURSOR
		do
			if is_cut_mode and then an_element.is_die and has_cut_patterns then
				c := cut_patterns.cursor
				from
					cut_patterns.start
				until
					cut_patterns.after or Result
				loop
					Result := Result or an_element.model_element.is_isolated_in_cut_pattern (cut_patterns.item.pattern)
					cut_patterns.forth
				end
				cut_patterns.go_to (c)
			end
		end

	is_isolated_in_current_cut_pattern (an_element: like anchored_view_element): BOOLEAN
			-- Is `an_element' isolated in current cut pattern?
		do
			if is_cut_mode and then an_element.is_die and has_current_cut_pattern then
				Result := an_element.model_element.is_isolated_in_cut_pattern (current_cut_pattern.pattern)
			end
		end

	is_cut_mode: BOOLEAN
			-- Are we in Cut mode?
		do
			Result := pack_mode_context /= Void
		end

	has_cut_patterns: BOOLEAN
			-- Are there any cut patterns?
		do
			Result := not cut_patterns.is_empty
		end

	cut_pattern_count: INTEGER
			-- Total number of cut patterns
		do
			Result := cut_patterns.count
		end

	completed_cut_pattern_count: INTEGER
			-- Number of cut patterns that are Completed
		local
			c: CURSOR
		do
			c := cut_patterns.cursor
			from
				cut_patterns.start
			until
				cut_patterns.after
			loop
				if cut_patterns.item.is_completed then
					Result := Result + 1
				end
				cut_patterns.forth
			end
			cut_patterns.go_to (c)
		end

	under_construction_cut_pattern_count: INTEGER
			-- Number of cut patterns that are Under Construction
		do
			Result := cut_pattern_count - completed_cut_pattern_count
		end

	at_first_cut_pattern: BOOLEAN
			-- Is the first cut pattern (or no cut pattern)
			-- currently displayed?
		do
			Result := not has_current_cut_pattern or cut_patterns.isfirst
		end

	at_last_cut_pattern: BOOLEAN
			-- Is the last cut pattern (or no cut pattern)
			-- currently displayed?
		do
			Result := not has_current_cut_pattern or cut_patterns.islast
		end

	has_current_cut_pattern: BOOLEAN
			--  Is there a current cut pattern?
		do
			Result := not cut_patterns.off
		end

	current_cut_pattern: RETICLE_VIEW_CUT_PATTERN
			--  Current cut pattern, if not Void
		require
			has_current_cut_pattern: has_current_cut_pattern
		do
			Result := cut_patterns.item
		end

	current_cut_pattern_field: STRING
			-- Name of field for current cut pattern or Void if
			-- no current cut pattern
		do
			if has_current_cut_pattern then 
				Result := current_cut_pattern.field_name
			end
		end

	current_scribe_count: INTEGER
			-- Current number of scribes (both horizontal and
			-- vertical)
		do
			count_action.set_count (0)
			apply_action_with_filter (scribe_filter, count_action)
			Result := count_action.count
		end

	lot_split_distribution_text: STRING
			-- Text describing number of projects assigned to
			-- each lot split (wafer etest ID)
		do
			Result := model.lot_split_distribution_text
		end

feature -- Modification

	set_reticle_change_history_capacity (a_value: like reticle_change_history_capacity)
			-- Set `reticle_change_history capacity' to `a_value'
		require
			a_value_nonnegative: a_value >= 0
		do
			reticle_change_history_capacity := a_value
		end

	set_corner_snap_limit_in_pixels_from_dist (a_value: DOUBLE)
			-- Set `corner_snap_limit_in_pixels' from distance
			-- `a_value' inches
		require
			a_value_nonnegative: a_value >= 0.0
		do
			corner_snap_limit_in_pixels := inches_to_pixels (a_value)
		end

	set_measure_marker_size_in_pixels_from_dist (a_value: DOUBLE)
			-- Set `measure_marker_size_in_pixels' from distance
			-- `a_value' inches
		require
			a_value_nonnegative: a_value >= 0.0
		do
			measure_marker_size_in_pixels := inches_to_pixels (a_value)
		end

	set_measure_marker_line_width (a_value: INTEGER)
			-- Set `measure_marker_line_width' to `a_value'
		require
			a_value_positive: a_value > 0
		do
			measure_marker_line_width := a_value
		ensure
			line_width_set: measure_marker_line_width = a_value
		end

	set_custom_scribe_widths (a_horizontal_width: like custom_horizontal_scribe_width a_vertical_width: like custom_vertical_scribe_width a_packing: RC_PACKING)
			-- Set `custom_horizontal_scribe_width' to
			-- `a_horizontal_width' and
			-- `custom_vertical_scribe_width' to
			-- `a_vertical_width'.  Add change to history
			-- list if either width changed
		require
			a_packing_not_void: a_packing /= Void
		local
			old_horiz_width, old_vert_width: DOUBLE
			new_horiz_width, new_vert_width: DOUBLE
		do
			initialize_change
			old_horiz_width := horizontal_scribe_width
			old_vert_width := vertical_scribe_width
			set_custom_horizontal_scribe_width (a_horizontal_width)
			set_custom_vertical_scribe_width (a_vertical_width)
			new_horiz_width := horizontal_scribe_width
			new_vert_width := vertical_scribe_width
			if old_horiz_width /= new_horiz_width or old_vert_width /= new_vert_width then
				add_change (create {CURRENT_SCRIBE_WIDTHS_CHANGE}.make (Current, old_horiz_width, old_vert_width, new_horiz_width, new_vert_width, default_scribe_width, a_packing))
				set_final_selected_elements_to_initial
				add_change_to_history_list (new_standard_change ("Set Scribe Widths"))
			end
		end

	set_custom_horizontal_scribe_width (a_width: like custom_horizontal_scribe_width)
			-- Set `custom_horizontal_scribe_width' to `a_width'
		do
			custom_horizontal_scribe_width := a_width
			has_custom_horizontal_scribe_width := custom_horizontal_scribe_width /= default_scribe_width
		ensure
			width_set: custom_horizontal_scribe_width = a_width
		end

	set_custom_vertical_scribe_width (a_width: like custom_vertical_scribe_width)
			-- Set `custom_vertical_scribe_width' to `a_width'
		do
			custom_vertical_scribe_width := a_width
			has_custom_vertical_scribe_width := custom_vertical_scribe_width /= default_scribe_width
		ensure
			width_set: custom_vertical_scribe_width = a_width
		end

	increment_overlap_tee_count
		do
			overlap_tee_count := overlap_tee_count + 1
		end

	increment_overlap_cross_count
		do
			overlap_cross_count := overlap_cross_count + 1
		end

	increment_overlap_enclosed_count
		do
			overlap_enclosed_count := overlap_enclosed_count + 1
		end

	increment_overlap_removed_count
		do
			overlap_removed_count := overlap_removed_count + 1
		end

	increment_overlap_resize_failed_count
		do
			overlap_resize_failed_count := overlap_resize_failed_count + 1
		end

	increment_overlap_delete_failed_count
		do
			overlap_delete_failed_count := overlap_delete_failed_count + 1
		end

	increment_centered_count
		do
			centered_count := centered_count + 1
		end

	uncache_fields
			-- Uncache `fields'
		do
			cached_fields := Void
		end

	set_context (a_context: RETICLE_VIEW_CONTEXT)
			-- Set context from `a_context'
		require
			a_context_not_void: a_context /= Void
		do
			set_partial_context (a_context)
			set_view_with_filter (all_filter, a_context.view)
			uncache_fields
		end

	set_partial_context (a_context: RETICLE_VIEW_CONTEXT)
			-- Set context from `a_context', excluding named
			-- view and label, color, fill and opacity generators
		require
			a_context_not_void: a_context /= Void
		do
			elements := a_context.elements
			copies_table := a_context.copies_table
			displayed_area_history_list := a_context.displayed_area_history_list
			reticle_change_history_list := a_context.reticle_change_history_list
			model.set_context (a_context.model_context)
			set_selected_elements_from_list (a_context.selected_elements)
			displayed_model_area := a_context.displayed_model_area
			set_adjusted_displayed_model_area_without_history (displayed_model_area)
			call_area_history_list_actions
			call_change_history_list_actions
		end

feature -- Operations

	initialize
			-- Initialize view
		do
			create font
			create rubber_box
			create operation_start_location
			create selection_bounding_box.make_by_corners (0, 0, 0, 0)
			create temp_rectangle.make_by_corners (0, 0, 0, 0)
			create temp_ev_rectangle.make (0, 0, 0, 0)
			drawable.set_background_color (stock_colors.White)
			create_new_pixmap (drawable.width, drawable.height)
			set_current_view (standard_fixed_view)
		end

	set_minimum_scribe_dimension_in_pixels (a_value: like minimum_scribe_dimension_in_pixels)
			-- Set `minimum_scribe_dimension_in_pixels' to
			-- `a_value'
		require
			a_value_nonnegative: a_value >= 0
		do
			minimum_scribe_dimension_in_pixels := a_value
		end

	set_initial_model_elements (list: LIST [like anchored_model_element] with_redraw: BOOLEAN)
			-- Set initial list of model elements
			-- to `list'.  Redraw model if `with_redraw'
		do
			create_copies_table
			clear_change_history_list
			uncache_fields
			precursor (list, with_redraw)
		end

	process_save_notification
			-- Take actions needed when a Save operation has
			-- completed and new version has been detached
		do
			rebuild_copies_table
			if is_cut_mode then
				clear_cut_pattern_change_histories
			end
			clear_change_history_list
		end

	clear_change_history_list
			-- Clear the undo/redo history list and notify
			-- observers
		do
			reticle_change_history_list.wipe_out
			call_change_history_list_actions
		end

	redraw
			-- Clear pixmap, draw view on pixmap and draw
			-- pixmap on drawable (assumes any necessary
			-- clearing of drawable has been done already)
		do
			if is_redraw_enabled then
				set_background_color_for_zoom
				create changed_bounding_box.make (0, 0, pixmap.width, pixmap.height)
				precursor
			end
		end

	redraw_rubber_box (a_x, a_y, a_width, a_height: INTEGER)
			-- Erase current rubber box, if any, and draw a
			-- new rubber box on the canvas whose upper left
			-- corner is at (`a_x', `a_y') and whose size
			-- is `a_width' by `a_height'.  Set new current
			-- rubber box to the one just drawn
		require
			a_width_nonnegative: a_width >= 0
			a_height_nonnegative: a_height >= 0
		do
			internal_redraw_rubber_box (a_x, a_y, a_width, a_height, 1, palette.standard_rubber_box_color)
		end
	
	redraw_zoom_style_rubber_box (a_x, a_y, a_width, a_height: INTEGER)
			-- Erase current rubber box, if any, and draw a
			-- new rubber box on the canvas whose upper left
			-- corner is at (`a_x', `a_y') and whose size
			-- is `a_width' by `a_height'.  Set new current
			-- rubber box to the one just drawn.  Draw rubber
			-- box using style for a zoom operation
		require
			a_width_nonnegative: a_width >= 0
			a_height_nonnegative: a_height >= 0
		local
			a_color: EV_COLOR
		do
			if a_width >= minimum_zoom_dimension_in_pixels and a_height >= minimum_zoom_dimension_in_pixels then
				a_color := palette.standard_rubber_box_color
			else
				a_color := palette.too_small_rubber_box_color
			end
			internal_redraw_rubber_box (a_x, a_y, a_width, a_height, 1, a_color)
		end

	redraw_selection_style_rubber_box (a_x, a_y, a_width, a_height: INTEGER)
			-- Erase current rubber box, if any, and draw a
			-- new rubber box on the canvas whose upper left
			-- corner is at (`a_x', `a_y') and whose size
			-- is `a_width' by `a_height'.  Set new current
			-- rubber box to the one just drawn.  Draw rubber
			-- box using style for a selection operation
		require
			a_width_nonnegative: a_width >= 0
			a_height_nonnegative: a_height >= 0
		do
			if a_width >= minimum_selection_dimension_in_pixels or a_height >= minimum_selection_dimension_in_pixels then
				internal_redraw_rubber_box (a_x, a_y, a_width, a_height, 1, palette.standard_rubber_box_color)
			else
				erase_rubber_box
			end
		end

	redraw_scribe_style_rubber_box (a_x, a_y, a_width, a_height: INTEGER)
			-- Erase current rubber box, if any, and draw a
			-- new rubber box on the canvas whose upper left
			-- corner is at (`a_x', `a_y') and whose size
			-- is `a_width' by `a_height'.  Set new current
			-- rubber box to the one just drawn.  Draw rubber
			-- box using style for a scribe creation operation
		require
			a_width_nonnegative: a_width >= 0
			a_height_nonnegative: a_height >= 0
		local
			l_color: EV_COLOR
		do
			if a_width >= a_height then
				l_color := palette.horizontal_scribe_rubber_box_color
			else
				l_color := palette.vertical_scribe_rubber_box_color
				
			end
			internal_redraw_rubber_box (a_x, a_y, a_width, a_height, 1, l_color)
		end

	redraw_scribe_resize_style_rubber_box (a_x, a_y, a_width, a_height: INTEGER)
			-- Erase current rubber box, if any, and draw a
			-- new rubber box on the canvas whose upper left
			-- corner is at (`a_x', `a_y') and whose size
			-- is `a_width' by `a_height'.  Set new current
			-- rubber box to the one just drawn.  Draw rubber
			-- box using style for a scribe resize operation
		require
			a_width_nonnegative: a_width >= 0
			a_height_nonnegative: a_height >= 0
		do
			redraw_scribe_style_rubber_box (a_x, a_y, a_width, a_height)
		end

	redraw_filler_style_rubber_box (a_x, a_y, a_width, a_height: INTEGER)
			-- Erase current rubber box, if any, and draw a
			-- new rubber box on the canvas whose upper left
			-- corner is at (`a_x', `a_y') and whose size
			-- is `a_width' by `a_height'.  Set new current
			-- rubber box to the one just drawn.  Draw rubber
			-- box using style for a filler creation operation
		require
			a_width_nonnegative: a_width >= 0
			a_height_nonnegative: a_height >= 0
		do
			internal_redraw_rubber_box (a_x, a_y, a_width, a_height, 1, palette.standard_rubber_box_color)
		end

	redraw_move_style_rubber_box (a_x, a_y, a_width, a_height: INTEGER)
			-- Erase current rubber box, if any, and draw a
			-- new rubber box on the canvas whose upper left
			-- corner is at (`a_x', `a_y') and whose size
			-- is `a_width' by `a_height'.  Set new current
			-- rubber box to the one just drawn.  Draw rubber
			-- box using style for a move operation
		require
			a_width_nonnegative: a_width >= 0
			a_height_nonnegative: a_height >= 0
		do
			internal_redraw_rubber_box (a_x, a_y, a_width, a_height, 1, palette.standard_move_box_color)
		end

	redraw_duplicate_style_rubber_box (a_x, a_y, a_width, a_height: INTEGER)
			-- Erase current rubber box, if any, and draw a
			-- new rubber box on the canvas whose upper left
			-- corner is at (`a_x', `a_y') and whose size
			-- is `a_width' by `a_height'.  Set new current
			-- rubber box to the one just drawn.  Draw rubber
			-- box using style for a duplicate operation
		require
			a_width_nonnegative: a_width >= 0
			a_height_nonnegative: a_height >= 0
		do
			internal_redraw_rubber_box (a_x, a_y, a_width, a_height, 1, palette.standard_duplicate_box_color)
		end

	erase_rubber_box
			-- Erase current rubber box, if any
		local
			a_x, a_y, a_width, a_height: INTEGER
			-- p_width, p_height: INTEGER
			-- temp: like temp_ev_rectangle
		do
			if rubber_box.width > 0 or rubber_box.height > 0 then
				
				a_x := rubber_box.x
				a_y := rubber_box.y
				a_width := rubber_box.width
				a_height := rubber_box.height
				
				rectangle.move_and_resize (a_x, a_y, a_width, 1)
				-- FIXME: work around Vision2 bug
				-- drawable.draw_sub_pixmap (a_x, a_y, pixmap, rectangle)
				draw_sub_pixmap (drawable, a_x, a_y, pixmap, rectangle)
				rectangle.move_and_resize (a_x, a_y, 1, a_height)
				-- FIXME: work around Vision2 bug
				-- drawable.draw_sub_pixmap (a_x, a_y, pixmap, rectangle)
				draw_sub_pixmap (drawable, a_x, a_y, pixmap, rectangle)
				rectangle.move_and_resize (a_x, a_y + (a_height - 1), a_width, 1)
				-- FIXME: work around Vision2 bug
				-- drawable.draw_sub_pixmap (a_x, a_y + (a_height - 1), pixmap, rectangle)
				draw_sub_pixmap (drawable, a_x, a_y + (a_height - 1), pixmap, rectangle)
				rectangle.move_and_resize (a_x + (a_width - 1), a_y, 1, a_height)
				-- FIXME: work around Vision2 bug
				-- drawable.draw_sub_pixmap (a_x + (a_width - 1), a_y, pixmap, rectangle)
				draw_sub_pixmap (drawable, a_x + (a_width - 1), a_y, pixmap, rectangle)
				rubber_box.move_and_resize (0, 0, 0, 0)
			end
		end

	handle_canvas_resize (a_x, a_y, a_width, a_height: INTEGER)
			-- Handle "canvas resized" event, where new upper left
			-- corner is at (`a_x', `a_y') and new size
			-- is `a_width' by `a_height'
		local
			ul_x, ul_y, lr_x, lr_y: DOUBLE
			diff_x, diff_y: INTEGER
		do
			create_new_pixmap (a_width, a_height)
			diff_x := current_window_position.x - previous_window_position.x
			diff_y := current_window_position.y - previous_window_position.y
			ul_x := upper_left_model_coords.x + model_units_per_pixel * diff_x
			ul_y := upper_left_model_coords.y - model_units_per_pixel * diff_y
			lr_x := ul_x + model_units_per_pixel * a_width
			lr_y := ul_y - model_units_per_pixel * a_height
			
			if model.is_setup_completed and a_width >= Resize_threshold and a_height >= Resize_threshold then
				set_displayed_model_area (ul_x, ul_y, lr_x, lr_y)
				redraw
			end
		end

	show_field_by_name (a_field_name: STRING)
			-- Display field named `a_field_name'
		require
			a_field_name_not_void: a_field_name /= Void
		do
			set_adjusted_displayed_model_area (enlarged_field_bounding_box (a_field_name))
			redraw
		end

	zoom_in (r: EV_RECTANGLE)
			-- Zoom in to show area defined by `r' in view
		do
			set_adjusted_displayed_model_area (model_area_for_view_rectangle (r))
			redraw
		end

	zoom_out (r: EV_RECTANGLE)
			-- Zoom out to show currently displayed area in
			-- area defined by `r'
		do
			set_adjusted_displayed_model_area (zoomed_out (model_area_for_view_rectangle (r)))
			redraw
		end

	start_pan
			-- Start pan operation
		do
			drawable.set_pointer_style (stock_pixmaps.Crosshair_cursor)
		end

	end_pan
			-- End pan operation
		do
			drawable.set_pointer_style (stock_pixmaps.Standard_cursor)
		end

	pan_by (a_x, a_y: INTEGER)
			-- Pan by (`a_x', `a_y') in view coordinates
		local
			x_chg, y_chg: DOUBLE
		do
			x_chg := -a_x * model_units_per_pixel
			y_chg := a_y * model_units_per_pixel
			set_displayed_model_area (upper_left_model_coords.x + x_chg, upper_left_model_coords.y + y_chg, lower_right_model_coords.x + x_chg, lower_right_model_coords.y + y_chg)
			redraw
		end

	cancel_pan
			-- Cancel pan operation
		do
			drawable.set_pointer_style (stock_pixmaps.Standard_cursor)
		end

	start_move_selected_at_view_coords (a_x, a_y: INTEGER)
			-- Start an operation to move all selected items
			-- at view coordinates (`a_x', `a_y')
		require
			selected_count_positive: selected_count > 0
		do
			set_operation_start_location (a_x, a_y)
			set_limits_infinite
			apply_action_with_filter (selected_filter, compute_move_limits_action)
			set_pixel_limits
			redraw_move_box (0, 0)
		end

	continue_move_selected_at_view_coords (a_x, a_y: INTEGER)
			-- Continue an operation to move all selected items
			-- at view coordinates (`a_x', `a_y')
		require
			selected_count_positive: selected_count > 0
		local
			offset_x, offset_y: INTEGER
		do
			offset_x := horizontally_limited (a_x - operation_start_location.x)
			offset_y := vertically_limited (a_y - operation_start_location.y)
			redraw_move_box (offset_x, offset_y)
		end

	end_move_selected_at_view_coords (a_x, a_y: INTEGER)
			-- End an operation to move all selected items
			-- at view coordinates (`a_x', `a_y')
		require
			selected_count_positive: selected_count > 0
		local
			p: POINT
			offset_x, offset_y: INTEGER
		do
			initialize_change
			erase_rubber_box
			set_selection_bounding_box_empty
			offset_x := horizontally_limited (a_x - operation_start_location.x)
			offset_y := vertically_limited (a_y - operation_start_location.y)
			create p.make_xy (grid.rounded_edge (offset_x * model_units_per_pixel), grid.rounded_edge (-offset_y * model_units_per_pixel))
			move_action.set_move_distance (p)
			apply_action_with_filter_and_redraw (selected_filter, move_action)
			move_distance := p
			set_final_selected_elements_to_initial
			add_change_to_history_list (new_standard_change ("Move"))
		end

	start_duplicate_selected_at_view_coords (a_x, a_y: INTEGER)
			-- Start an operation to duplicate all selected items
			-- at view coordinates (`a_x', `a_y')
		require
			selected_count_positive: selected_count > 0
		do
			set_operation_start_location (a_x, a_y)
			initialize_status
			initialize_duplicate_element_lists
			set_limits_infinite
			apply_action_with_filter (selected_filter, compute_duplicate_limits_action)
			set_pixel_limits
			redraw_duplicate_box (0, 0)
		end

	continue_duplicate_selected_at_view_coords (a_x, a_y: INTEGER)
			-- Continue an operation to duplicate all
			-- selected items at view coordinates (`a_x',
			-- `a_y')
		require
			selected_count_positive: selected_count > 0
		local
			offset_x, offset_y: INTEGER
		do
			offset_x := horizontally_limited (a_x - operation_start_location.x)
			offset_y := vertically_limited (a_y - operation_start_location.y)
			redraw_duplicate_box (offset_x, offset_y)
		end

	end_duplicate_selected_at_view_coords (a_x, a_y: INTEGER)
			-- End an operation to duplicate all selected items
			-- at view coordinates (`a_x', `a_y')
		require
			selected_count_positive: selected_count > 0
		local
			p: POINT
			offset_x, offset_y: INTEGER
		do
			erase_rubber_box
			offset_x := horizontally_limited (a_x - operation_start_location.x)
			offset_y := vertically_limited (a_y - operation_start_location.y)
			is_duplicate_far_enough := offset_x.abs >= minimum_dup_distance_in_pixels or offset_y.abs >= minimum_dup_distance_in_pixels
			if is_duplicate_far_enough then
				initialize_change
				set_selection_bounding_box_empty
				create p.make_xy (grid.rounded_edge (offset_x * model_units_per_pixel), grid.rounded_edge (-offset_y * model_units_per_pixel))
				duplicate_action.set_move_distance (p)
				new_elements.wipe_out
				set_changed_region_empty
				apply_action_with_filter (selected_filter, duplicate_action)
				elements.append (new_elements)
				redraw_changed_region
				move_distance := p
				set_final_selected_elements
				add_change_to_history_list (new_creation_change ("Duplicate"))
			end
		end

	move_relative_selected_items (a_x, a_y: DOUBLE)
			-- Move selected objects by (`a_x', `a_y')
		require
			a_x_on_grid: grid.is_edge_on_grid (a_x)
			a_y_on_grid: grid.is_edge_on_grid (a_y)
		do
			move_selected_items_by ("Move Relative", a_x, a_y)
		end

	move_absolute_selected_items (a_field_name: STRING a_x, a_y: DOUBLE)
			-- Move selected objects to (`a_x', `a_y') relative
			-- to center of field with name `a_field_name'
		require
			a_field_name_not_void: a_field_name /= Void
		local
			l_x, l_y: DOUBLE
			l_field: RETICLE_FIELD
		do
			l_field := model.field_with_name (a_field_name)
			l_x := grid.rounded_edge (l_field.center_x + a_x - selected_element.center_x)
			l_y := grid.rounded_edge (l_field.center_y + a_y - selected_element.center_y)
			move_selected_items_by ("Move Absolute", l_x, l_y)
		end

	center_all_projects_in_cut_die
			-- Center all projects in their cut die, ignoring
			-- projects which are not isolated in their
			-- cut die
		do
			center_projects_in_cut_die (uncentered_projects_filter)
		end
	
	center_selected_projects_in_cut_die
			-- Center selected projects in their cut die,
			-- ignoring projects which are not isolated in
			-- their cut die
		do
			center_projects_in_cut_die (selected_uncentered_project_filter)
		end

	spread_selected_items_horizontally_by_asc_width (a_coordinate: EV_COORDINATE)
			-- Spread all currently selected items apart
			-- so that they do not overlap, arrange them
			-- horizontally sorted by ascending width and
			-- place lower left corner of bounding box at
			-- view coordinates `a_coordinate'
		require
			a_coordinate_not_void: a_coordinate /= Void
			selected_count_positive: selected_count > 0
		do
			spread_selected_items_horizontally ("Horizontal Spread By Ascending Width", a_coordinate, spread_by_width_asc_comparator)
		end

	spread_selected_items_horizontally_by_desc_width (a_coordinate: EV_COORDINATE)
			-- Spread all currently selected items apart
			-- so that they do not overlap, arrange them
			-- horizontally sorted by descending width and
			-- place lower left corner of bounding box at
			-- view coordinates `a_coordinate'
		require
			a_coordinate_not_void: a_coordinate /= Void
			selected_count_positive: selected_count > 0
		do
			spread_selected_items_horizontally ("Horizontal Spread By Descending Width", a_coordinate, spread_by_width_desc_comparator)
		end

	spread_selected_items_horizontally_by_asc_height (a_coordinate: EV_COORDINATE)
			-- Spread all currently selected items apart
			-- so that they do not overlap, arrange them
			-- horizontally sorted by ascending height and
			-- place lower left corner of bounding box at
			-- view coordinates `a_coordinate'
		require
			a_coordinate_not_void: a_coordinate /= Void
			selected_count_positive: selected_count > 0
		do
			spread_selected_items_horizontally ("Horizontal Spread By Ascending Height", a_coordinate, spread_by_height_asc_comparator)
		end

	spread_selected_items_horizontally_by_desc_height (a_coordinate: EV_COORDINATE)
			-- Spread all currently selected items apart
			-- so that they do not overlap, arrange them
			-- horizontally sorted by descending height and
			-- place lower left corner of bounding box at
			-- view coordinates `a_coordinate'
		require
			a_coordinate_not_void: a_coordinate /= Void
			selected_count_positive: selected_count > 0
		do
			spread_selected_items_horizontally ("Horizontal Spread By Descending Height", a_coordinate, spread_by_height_desc_comparator)
		end

	spread_selected_items_vertically_by_asc_width (a_coordinate: EV_COORDINATE)
			-- Spread all currently selected items apart so that
			-- they do not overlap, arrange them vertically
			-- sorted by ascending width and place lower left corner of
			-- bounding box at view coordinates `a_coordinate'
		require
			a_coordinate_not_void: a_coordinate /= Void
			selected_count_positive: selected_count > 0
		do
			spread_selected_items_vertically ("Vertical Spread By Ascending Width", a_coordinate, spread_by_width_asc_comparator)
		end

	spread_selected_items_vertically_by_desc_width (a_coordinate: EV_COORDINATE)
			-- Spread all currently selected items apart so that
			-- they do not overlap, arrange them vertically
			-- sorted by descending width and place lower left corner of
			-- bounding box at view coordinates `a_coordinate'
		require
			a_coordinate_not_void: a_coordinate /= Void
			selected_count_positive: selected_count > 0
		do
			spread_selected_items_vertically ("Vertical Spread By Descending Width", a_coordinate, spread_by_width_desc_comparator)
		end

	spread_selected_items_vertically_by_asc_height (a_coordinate: EV_COORDINATE)
			-- Spread all currently selected items apart
			-- so that they do not overlap, arrange them
			-- vertically sorted by ascending height and
			-- place lower left corner of bounding box at
			-- view coordinates `a_coordinate'
		require
			a_coordinate_not_void: a_coordinate /= Void
			selected_count_positive: selected_count > 0
		do
			spread_selected_items_vertically ("Vertical Spread By Ascending Height", a_coordinate, spread_by_height_asc_comparator)
		end

	spread_selected_items_vertically_by_desc_height (a_coordinate: EV_COORDINATE)
			-- Spread all currently selected items apart
			-- so that they do not overlap, arrange them
			-- vertically sorted by descending height and
			-- place lower left corner of bounding box at
			-- view coordinates `a_coordinate'
		require
			a_coordinate_not_void: a_coordinate /= Void
			selected_count_positive: selected_count > 0
		do
			spread_selected_items_vertically ("Vertical Spread By Descending Height", a_coordinate, spread_by_height_desc_comparator)
		end

	spread_selected_items_horizontally_by_asc_x_position (a_coordinate: EV_COORDINATE)
			-- Spread all currently selected items apart
			-- so that they do not overlap, arrange them
			-- horizontally sorted by ascending center in X and
			-- place lower left corner of bounding box at
			-- view coordinates `a_coordinate'
		require
			a_coordinate_not_void: a_coordinate /= Void
			selected_count_positive: selected_count > 0
		do
			spread_selected_items_horizontally ("Horizontal Spread By Ascending X Position", a_coordinate, spread_by_x_position_asc_comparator)
		end

	spread_selected_items_horizontally_by_desc_x_position (a_coordinate: EV_COORDINATE)
			-- Spread all currently selected items apart
			-- so that they do not overlap, arrange them
			-- horizontally sorted by descending center in X and
			-- place lower left corner of bounding box at
			-- view coordinates `a_coordinate'
		require
			a_coordinate_not_void: a_coordinate /= Void
			selected_count_positive: selected_count > 0
		do
			spread_selected_items_horizontally ("Horizontal Spread By Descending X Position", a_coordinate, spread_by_x_position_desc_comparator)
		end

	spread_selected_items_horizontally_by_asc_y_position (a_coordinate: EV_COORDINATE)
			-- Spread all currently selected items apart
			-- so that they do not overlap, arrange them
			-- horizontally sorted by ascending center in Y and
			-- place lower left corner of bounding box at
			-- view coordinates `a_coordinate'
		require
			a_coordinate_not_void: a_coordinate /= Void
			selected_count_positive: selected_count > 0
		do
			spread_selected_items_horizontally ("Horizontal Spread By Ascending Y Position", a_coordinate, spread_by_y_position_asc_comparator)
		end

	spread_selected_items_horizontally_by_desc_y_position (a_coordinate: EV_COORDINATE)
			-- Spread all currently selected items apart
			-- so that they do not overlap, arrange them
			-- horizontally sorted by descending center in Y and
			-- place lower left corner of bounding box at
			-- view coordinates `a_coordinate'
		require
			a_coordinate_not_void: a_coordinate /= Void
			selected_count_positive: selected_count > 0
		do
			spread_selected_items_horizontally ("Horizontal Spread By Descending Y Position", a_coordinate, spread_by_y_position_desc_comparator)
		end

	spread_selected_items_vertically_by_asc_x_position (a_coordinate: EV_COORDINATE)
			-- Spread all currently selected items apart
			-- so that they do not overlap, arrange them
			-- vertically sorted by ascending center in X
			-- and place lower left corner of bounding box
			-- at view coordinates `a_coordinate'
		require
			a_coordinate_not_void: a_coordinate /= Void
			selected_count_positive: selected_count > 0
		do
			spread_selected_items_vertically ("Vertical Spread By Ascending X Position", a_coordinate, spread_by_x_position_asc_comparator)
		end

	spread_selected_items_vertically_by_desc_x_position (a_coordinate: EV_COORDINATE)
			-- Spread all currently selected items apart
			-- so that they do not overlap, arrange them
			-- vertically sorted by descending center in X
			-- and place lower left corner of bounding box
			-- at view coordinates `a_coordinate'
		require
			a_coordinate_not_void: a_coordinate /= Void
			selected_count_positive: selected_count > 0
		do
			spread_selected_items_vertically ("Vertical Spread By Descending X Position", a_coordinate, spread_by_x_position_desc_comparator)
		end

	spread_selected_items_vertically_by_asc_y_position (a_coordinate: EV_COORDINATE)
			-- Spread all currently selected items apart
			-- so that they do not overlap, arrange them
			-- vertically sorted by ascending center in Y
			-- and place lower left corner of bounding box
			-- at view coordinates `a_coordinate'
		require
			a_coordinate_not_void: a_coordinate /= Void
			selected_count_positive: selected_count > 0
		do
			spread_selected_items_vertically ("Vertical Spread By Ascending Y Position", a_coordinate, spread_by_y_position_asc_comparator)
		end

	spread_selected_items_vertically_by_desc_y_position (a_coordinate: EV_COORDINATE)
			-- Spread all currently selected items apart
			-- so that they do not overlap, arrange them
			-- vertically sorted by descending center in Y
			-- and place lower left corner of bounding box
			-- at view coordinates `a_coordinate'
		require
			a_coordinate_not_void: a_coordinate /= Void
			selected_count_positive: selected_count > 0
		do
			spread_selected_items_vertically ("Vertical Spread By Descending Y Position", a_coordinate, spread_by_y_position_desc_comparator)
		end

	spread_selected_items_horizontally_by_asc_name (a_coordinate: EV_COORDINATE)
			-- Spread all currently selected items apart
			-- so that they do not overlap, arrange them
			-- horizontally sorted by ascending packing
			-- name and place lower left corner of
			-- bounding box at view coordinates
			-- `a_coordinate'
		require
			a_coordinate_not_void: a_coordinate /= Void
		do
			spread_selected_items_horizontally ("Horizontal Spread By Ascending Name", a_coordinate, packing_name_asc_comparator)
		end

	spread_selected_items_horizontally_by_desc_name (a_coordinate: EV_COORDINATE)
			-- Spread all currently selected items apart
			-- so that they do not overlap, arrange them
			-- horizontally sorted by descending packing
			-- name and place lower left corner of
			-- bounding box at view coordinates
			-- `a_coordinate'
		require
			a_coordinate_not_void: a_coordinate /= Void
		do
			spread_selected_items_horizontally ("Horizontal Spread By Descending Name", a_coordinate, packing_name_desc_comparator)
		end

	spread_selected_items_vertically_by_asc_name (a_coordinate: EV_COORDINATE)
			-- Spread all currently selected items apart
			-- so that they do not overlap, arrange them
			-- vertically sorted by ascending packing name
			-- and place lower left corner of bounding box
			-- at view coordinates `a_coordinate'
		require
			a_coordinate_not_void: a_coordinate /= Void
		do
			spread_selected_items_vertically ("Vertical Spread By Ascending Name", a_coordinate, packing_name_asc_comparator)
		end

	spread_selected_items_vertically_by_desc_name (a_coordinate: EV_COORDINATE)
			-- Spread all currently selected items apart
			-- so that they do not overlap, arrange them
			-- vertically sorted by descending packing name
			-- and place lower left corner of bounding box
			-- at view coordinates `a_coordinate'
		require
			a_coordinate_not_void: a_coordinate /= Void
		do
			spread_selected_items_vertically ("Vertical Spread By Descending Name", a_coordinate, packing_name_desc_comparator)
		end

	abut_selected_item (a_coordinate: EV_COORDINATE)
			-- Abut the currently selected item (can only be one)
			-- to the object indicated by the mouse pointer
		require
			a_coordinate_not_void: a_coordinate /= Void
			one_item_selected: selected_count = 1
		local
			list: like new_elements
			p: POINT
		do
			initialize_status
			create {ARRAYED_LIST [like anchored_view_element]} list.make (1)
			containing_elements.wipe_out
			pre_abut_action.set_selection_point (a_coordinate)
			pre_abut_action.set_no_target_field
			pre_abut_action.set_selected_elements (list)
			pre_abut_action.set_containing_elements (containing_elements)
			apply_action_with_filter (all_filter, pre_abut_action)
			target_field := pre_abut_action.target_field
			abut_source := list.first
			pick_abut_target
			if not had_error then
				set_abut_move_distance (a_coordinate)
			end
			if not had_error then
				check_abut_lock_restrictions
			end
			if not had_error then
				create p.make_xy (abut_source.center_x, abut_source.center_y)
				p := p + abut_move_distance
				if not model.center_fits_in_database (p) then
					add_error ("new center too big to fit in database")
				end
			end
			if not had_error then
				if abut_move_distance.x /= 0.0 or abut_move_distance.y /= 0.0 then
					initialize_change
					set_selection_bounding_box_empty
					move_action.set_move_distance (abut_move_distance)
					apply_action_with_filter_and_redraw (selected_filter, move_action)
					set_final_selected_elements_to_initial
					add_change_to_history_list (new_standard_change ("Abut"))
				end
			end
		end

	align_left_selected_items (a_coordinate: EV_COORDINATE)
			-- Align the currently selected items to the left
			-- edge of the object containing the mouse pointer
		require
			a_coordinate_not_void: a_coordinate /= Void
		do
			align_selected_items ("Align Left", a_coordinate, align_left_action)
		end

	align_right_selected_items (a_coordinate: EV_COORDINATE)
			-- Align the currently selected items to the right
			-- edge of the object containing the mouse pointer
		require
			a_coordinate_not_void: a_coordinate /= Void
		do
			align_selected_items ("Align Right", a_coordinate, align_right_action)
		end

	align_top_selected_items (a_coordinate: EV_COORDINATE)
			-- Align the currently selected items to the top
			-- edge of the object containing the mouse pointer
		require
			a_coordinate_not_void: a_coordinate /= Void
		do
			align_selected_items ("Align Top", a_coordinate, align_top_action)
		end

	align_bottom_selected_items (a_coordinate: EV_COORDINATE)
			-- Align the currently selected items to the bottom
			-- edge of the object containing the mouse pointer
		require
			a_coordinate_not_void: a_coordinate /= Void
		do
			align_selected_items ("Align Bottom", a_coordinate, align_bottom_action)
		end

	align_center_x_selected_items (a_coordinate: EV_COORDINATE)
			-- Align the currently selected items to the center
			-- of the object containing the mouse pointer in X
		require
			a_coordinate_not_void: a_coordinate /= Void
		do
			align_selected_items ("Align Center X", a_coordinate, align_center_x_action)
		end

	align_center_y_selected_items (a_coordinate: EV_COORDINATE)
			-- Align the currently selected items to the center
			-- of the object containing the mouse pointer in Y
		require
			a_coordinate_not_void: a_coordinate /= Void
		do
			align_selected_items ("Align Center Y", a_coordinate, align_center_y_action)
		end

	horizontal_scan_selected_items
			-- Scan currently selected items and space them out
			-- evenly in the X dimension, taking into account items
			-- that are fixed in X and the field of view edges
		require
			selected_count_nonnegative: selected_count > 0
		do
			scan_selected_items ("Horizontal Scan", False, "horizontal", pre_horizontal_scan_action, horizontal_scan_action, scan_x_comparator)
		end

	vertical_scan_selected_items
			-- Scan currently selected items and space them out
			-- evenly in the Y dimension, taking into account items
			-- that are fixed in Y and the field of view edges
		require
			selected_count_nonnegative: selected_count > 0
		do
			scan_selected_items ("Vertical Scan", True, "vertical", pre_vertical_scan_action, vertical_scan_action, scan_y_comparator)
		end

	set_selected_element
			-- Set `selected_element' to the single selected
			-- element
		require
			one_selected_element: selected_count = 1
		do
			find_selected_element_action.set_void_selected_element
			apply_action_with_filter (selected_filter, find_selected_element_action)
			selected_element := find_selected_element_action.selected_element
		end

	start_scribe_resize_at_view_coords (a_x, a_y: INTEGER)
			-- Start resizing of scribe at view coordinates
			-- (`a_x', `a_y')
		do
			initialize_status
			if selected_count = 0 then
				add_error ("no object selected")
			elseif selected_count > 1 then
				add_error (selected_count.out + " objects selected")
			else
				set_selected_element
				if not selected_element.is_scribe then
					add_error ("selected object is not a scribe")
				elseif selected_element.is_horizontal_scribe then
					if selected_element.is_locked_in_x then
						add_error ("selected scribe is locked in X")
					end
					is_left_or_bottom_resize := (selected_element.x - a_x).abs < (selected_element.x + selected_element.width - a_x).abs
				else -- Vertical scribe
					if selected_element.is_locked_in_y then
						add_error ("selected scribe is locked in Y")
					end
					is_left_or_bottom_resize := (selected_element.y - a_y).abs >= (selected_element.y + selected_element.height - a_y).abs
				end
			end
			if not had_error then
				set_operation_start_location (a_x, a_y)
				set_limits_infinite
				selected_element.update_scribe_resize_limits (is_left_or_bottom_resize)
				set_pixel_limits
				redraw_scribe_resize_box (0, 0)
			end
		end

	continue_scribe_resize_at_view_coords (a_x, a_y: INTEGER)
			-- Continue resizing of scribe at view coordinates
			-- (`a_x', `a_y')
		local
			offset_x, offset_y: INTEGER
		do
			offset_x := horizontally_limited (a_x - operation_start_location.x)
			offset_y := vertically_limited (a_y - operation_start_location.y)
			redraw_scribe_resize_box (offset_x, offset_y)
		end

	end_scribe_resize_at_view_coords (a_x, a_y: INTEGER)
			-- End resizing of scribe at view coordinates
			-- (`a_x', `a_y')
		local
			offset_x, offset_y: INTEGER
			p: CONST_POINT
		do
			initialize_change
			erase_rubber_box
			set_selection_bounding_box_empty
			offset_x := horizontally_limited (a_x - operation_start_location.x)
			offset_y := vertically_limited (a_y - operation_start_location.y)
			create p.make_xy (grid.rounded_edge (offset_x * model_units_per_pixel), grid.rounded_edge (-offset_y * model_units_per_pixel))
			resize_scribe_action.set_resize_amount (p)
			resize_scribe_action.set_is_left_or_bottom_resize (is_left_or_bottom_resize)
			apply_action_with_filter_and_redraw (selected_filter, resize_scribe_action)
			set_final_selected_elements_to_initial
			add_change_to_history_list (new_standard_change ("Resize Scribe"))
		end

	create_scribe (a_rectangle: EV_RECTANGLE)
			-- Create scribe whose size and position is
			-- dictated by view rectangle `a_rectangle'
		require
			a_rectangle_not_void: a_rectangle /= Void
		local
			r: RECTANGLE
			low_x, low_y, high_x, high_y: DOUBLE
			is_horiz, size_fits, center_fits: BOOLEAN
		do
			initialize_status
			is_horiz := a_rectangle.width >= a_rectangle.height
			create r.make_copy (model_area_for_view_rectangle (a_rectangle))
			if is_horiz then
				low_x := grid.rounded_edge (r.low_x)
				high_x := grid.rounded_edge (r.high_x)
				low_y := grid.rounded_edge (r.center_y - horizontal_scribe_width / 2.0)
				high_y := grid.rounded_edge (low_y + horizontal_scribe_width)
			else
				low_x := grid.rounded_edge (r.center_x - vertical_scribe_width / 2.0)
				high_x := grid.rounded_edge (low_x + vertical_scribe_width)
				low_y := grid.rounded_edge (r.low_y)
				high_y := grid.rounded_edge (r.high_y)
				
			end
			create r.make_by_size_and_center (grid.rounded_size (high_x - low_x), grid.rounded_size (high_y - low_y), (low_x + high_x) / 2.0, (low_y + high_y) / 2.0)
			size_fits := model.size_fits_in_database (r.size)
			center_fits := model.center_fits_in_database (r.center)
			if size_fits and center_fits then
				initialize_change_without_status
				if is_horiz then
					model.add_new_horizontal_scribe (r)
				else
					model.add_new_vertical_scribe (r)
				end
				add_and_select_new_element (model.last_new_element)
				add_change (create {CREATION_CHANGE}.make (last_new_element))
				set_final_selected_elements
				add_change_to_history_list (new_creation_change ("Create Scribe"))
				last_size := r.size
				is_last_scribe_horizontal := is_horiz
			elseif not size_fits and not center_fits then
				add_error ("Could not create scribe because its size and center would not fit in the database")
			elseif not size_fits then
				add_error ("Could not create scribe because its size would not fit in the database")
			else
				add_error ("Could not create scribe because its center would not fit in the database")
			end
		end

	filler_model_coordinates (a_rectangle: EV_RECTANGLE): RECTANGLE
			-- Model coordinates for filler whose size and
			-- position is dictated by view rectangle
			-- `a_rectangle'
		require
			a_rectangle_not_void: a_rectangle /= Void
		local
			r: RECTANGLE
			low_x, low_y, high_x, high_y: DOUBLE
		do
			create r.make_copy (model_area_for_view_rectangle (a_rectangle))
			low_x := grid.rounded_edge (r.low_x)
			high_x := grid.rounded_edge (r.high_x)
			low_y := grid.rounded_edge (r.low_y)
			high_y := grid.rounded_edge (r.high_y)
			create Result.make_by_size_and_center (grid.rounded_size (high_x - low_x), grid.rounded_size (high_y - low_y), (low_x + high_x) / 2.0, (low_y + high_y) / 2.0)
		end

	default_filler_name (a_size_x, a_size_y: DOUBLE): STRING
			-- Default name for filler whose size is `a_size_x'
			-- by `a_size_y'
		require
			a_size_x_positive: a_size_x > 0.0
			a_size_y_positive: a_size_y > 0.0
		local
			w, h: STRING
		do
			w := grid.formatted_number (a_size_x)
			h := grid.formatted_number (a_size_y)
			Result := w + "_x_" + h
		end

	create_filler (a_name: STRING a_rectangle: CONST_RECTANGLE)
			-- Create filler with `a_name' whose size and
			-- position is dictated by model rectangle
			-- `a_rectangle'
		require
			a_name_not_void: a_name /= Void
			a_rectangle_not_void: a_rectangle /= Void
		local
			r: RECTANGLE
			l_x, l_y, diff_x, diff_y: DOUBLE
			low_x, low_y, high_x, high_y: DOUBLE
		do
			initialize_change
			create r.make_copy (a_rectangle)
			l_x := r.low_x
			l_y := r.low_y
			diff_x := grid.rounded_edge (l_x) - l_x
			diff_y := grid.rounded_edge (l_y) - l_y
			r.move_by (diff_x, diff_y)
				-- Put rectangle edges on grid, so that size
				-- calculation works correctly
			
			low_x := grid.rounded_edge (r.low_x)
			high_x := grid.rounded_edge (r.high_x)
			low_y := grid.rounded_edge (r.low_y)
			high_y := grid.rounded_edge (r.high_y)
			create r.make_by_size_and_center (grid.rounded_size (high_x - low_x), grid.rounded_size (high_y - low_y), (low_x + high_x) / 2.0, (low_y + high_y) / 2.0)
			model.add_new_filler (a_name, r)
			add_and_select_new_element (model.last_new_element)
			add_change (create {CREATION_CHANGE}.make (last_new_element))
			set_final_selected_elements
			add_change_to_history_list (new_creation_change ("Create Filler"))
			last_size := r.size
		end

	create_cut (is_horiz: BOOLEAN a_location: DOUBLE)
			-- Create cut and add it to `new_elements' if there is
			-- not already a cut with same orientation at
			-- `a_location' (relative to center of field)
		local
			elem: like anchored_view_element
			pat: like current_cut_pattern
			ok: BOOLEAN
		do
			pat := current_cut_pattern
			if is_horiz then
				ok := not pat.has_horizontal_cut_at (a_location)
				if ok then
					model.add_new_horizontal_cut (current_cut_pattern.pattern, a_location)
					last_horizontal_cut_count := last_horizontal_cut_count + 1
				end
			else
				ok := not pat.has_vertical_cut_at (a_location)
				if ok then
					model.add_new_vertical_cut (current_cut_pattern.pattern, a_location)
					last_vertical_cut_count := last_vertical_cut_count + 1
				end
			end
			if ok then
				elem := new_view_element (model.last_new_element)
				elem.set_view_coordinates
				elem.add_to_changed_region
				add_to_new_element_list (elem)
				add_change (create {CUT_CREATION_CHANGE}.make (elem))
			end
		end

	delete_cut (an_element: like anchored_view_element)
			-- Delete `an_element' which is a cut
		require
			an_element_not_void: an_element /= Void
			an_element_is_cut: an_element.is_cut
		local
			pat: like current_cut_pattern
		do
			pat := current_cut_pattern
			model.remove_cut (an_element.model_element.cut_location, pat.pattern)
			deleted_elements.extend (an_element)
			if an_element.is_horizontal_cut then
				last_horizontal_cut_count := last_horizontal_cut_count + 1
			else
				last_vertical_cut_count := last_vertical_cut_count + 1
			end
		end

	add_and_select_new_element (a_model_element: like anchored_model_element)
			-- Add a new view element for `a_model_element' to
			-- `elements' and select it.  Leave it in
			-- `last_new_element'
		do
			add_new_element (a_model_element)
			set_selection_to_element_action.set_only_element (last_new_element)
			apply_select_action_and_redraw (set_selection_to_element_action)
		end

	add_new_element (a_model_element: like anchored_model_element)
			-- Add a new view element for `a_model_element' to
			-- `elements'.  Leave it in `last_new_element'
		local
			elem: like anchored_view_element
		do
			elem := new_view_element (a_model_element)
			elem.set_view_coordinates
			elements.extend (elem)
			last_new_element := elem
		end

	new_scribe (an_element: like anchored_model_element a_view_element: like anchored_view_element): like anchored_view_element
			-- New view element for scribe
			-- `an_element'.  Set its selection status and
			-- other properties from `a_view_element'
			-- (setting selection status will cause it to
			-- be added to element list when elements are
			-- rebuilt).  If not called from within a
			-- "select" action, caller must add scribe to view
		do
			Result := new_view_element (an_element)
			Result.set_view_coordinates
			if a_view_element.is_fixed_in_x then
				Result.fix_in_x
			end
			if a_view_element.is_fixed_in_y then
				Result.fix_in_y
			end
			if a_view_element.is_locked_in_x then
				Result.lock_in_x
			end
			if a_view_element.is_locked_in_y then
				Result.lock_in_y
			end
			Result.set_is_duplicatable (a_view_element.is_duplicatable)
			Result.set_is_deletable (a_view_element.is_deletable)
			Result.set_selected (a_view_element.is_selected, True)
		end

	delete_selected_items (a_deleted_list: LIST [PACKING_DIES_MROW] scribes_only, undoable: BOOLEAN)
			-- Delete from packing all currently selected items,
			-- where `a_deleted_list' is a list of the dies
			-- for elements which had all copies selected
			-- and none of the copies were delete-locked and
			-- which have already been deleted from packing.
			-- If `scribes_only' then ignore selected items that
			-- are not scribes.  If `undoable', then there were
			-- no projects whose last copy was deleted
		require
			selected_count_positive: selected_count > 0
		local
			l_partially_deleted_elements, l_fully_deleted_elements: like deleted_elements
			l_name: STRING
		do
			initialize_change
			create {ARRAYED_LIST [like anchored_view_element]} delete_locked_elements.make (selected_count)
			create {ARRAYED_LIST [like anchored_view_element]} delete_failed_elements.make (selected_count)
			create {ARRAYED_LIST [like anchored_view_element]} l_partially_deleted_elements.make (selected_count)
			create {ARRAYED_LIST [like anchored_view_element]} l_fully_deleted_elements.make (selected_count)
			create {ARRAYED_LIST [like anchored_view_element]} deleted_elements.make (selected_count)
			delete_action.set_dies (a_deleted_list)
			delete_action.set_partially_deleted_elements (l_partially_deleted_elements)
			delete_action.set_fully_deleted_elements (l_fully_deleted_elements)
			delete_action.set_scribes_only (scribes_only)
			apply_select_action_and_redraw (delete_action)
			deleted_elements.append (l_partially_deleted_elements)
			deleted_elements.append (l_fully_deleted_elements)
			if not l_fully_deleted_elements.is_empty then
				update_copies_table_for_delete (l_fully_deleted_elements)
			end
			if not deleted_elements.is_empty then
				model.set_from_element_list (model_elements)
			end
			if undoable then
				if scribes_only then
					l_name := "Delete"
				else
					l_name := "Delete Scribes"
				end
				set_final_selected_elements
				add_change_to_history_list (new_deletion_change (l_name))
			else
				clear_change_history_list
			end
		end

	delete_selected_cuts
			-- Delete from packing all currently selected items
			-- that are cuts
		do
			initialize_change
			initialize_cut_counts
			create {ARRAYED_LIST [like anchored_view_element]} deleted_elements.make (selected_count)
			apply_select_action_and_redraw (delete_selected_cut_action)
			if not deleted_elements.is_empty then
				model.set_from_element_list (model_elements)
			end
			set_final_selected_elements
			add_change_to_history_list (new_cut_deletion_change ("Delete Cuts"))
		end

	delete_intersecting_cuts
			-- Delete from packing all cuts that intersect
			-- the cut die of any selected project
		local
			l_cut_dies: LIST [CONST_RECTANGLE]
		do
			initialize_change
			initialize_cut_counts
			compute_cut_dies
			create {ARRAYED_LIST [like anchored_view_element]} deleted_elements.make (selected_count)
			create {ARRAYED_LIST [CONST_RECTANGLE]} l_cut_dies.make (selected_count)
			add_cut_die_action.set_cut_dies (l_cut_dies)
			apply_action_with_filter (selected_isolatable_filter, add_cut_die_action)
			delete_intersecting_cut_action.set_cut_dies (l_cut_dies)
			delete_intersecting_cut_action.set_grid (grid)
			apply_select_action_and_redraw (delete_intersecting_cut_action)
			if not deleted_elements.is_empty then
				model.set_from_element_list (model_elements)
			end
			set_final_selected_elements
			add_change_to_history_list (new_cut_deletion_change ("Delete Intersecting Cuts"))
		end

	compute_lists_for_delete (scribes_only: BOOLEAN)
			-- Compute various lists from selected items.
			-- Leave them in `projects_to_delete' and
			-- `non_project_dies_to_delete'.  If `scribes_only'
			-- then ignore selected items which are not scribes
		do
			create {ARRAYED_LIST [PACKING_PROJECTS_MROW]} projects_to_delete.make (selected_count)
			create {ARRAYED_LIST [PACKING_DIES_MROW]} non_project_dies_to_delete.make (selected_count)
			compute_delete_lists_action.set_project_list (projects_to_delete)
			compute_delete_lists_action.set_non_project_list (non_project_dies_to_delete)
			compute_delete_lists_action.set_scribes_only (scribes_only)
			apply_action_with_filter (all_copies_filter, compute_delete_lists_action)
		end

	compute_default_isolation
			-- Compute default set of isolated projects,
			-- based on current cut pattern's cuts and
			-- wafer etest ID
		require
			has_current_cut_pattern: has_current_cut_pattern
		do
			initialize_change
			set_changed_region_empty
			compute_cut_dies
			apply_action_with_filter (isolatable_filter, compute_default_isolation_action)
			redraw_changed_region
			set_final_selected_elements
			add_change_to_history_list (new_standard_change ("Compute Default Isolation Set"))
			last_isolated_count := current_cut_pattern.isolated_location_count
		end

	toggle_isolation_of_selected_projects
			-- Toggle the "isolated" status of every selected
			-- isolatable element
		require
			has_current_cut_pattern: has_current_cut_pattern
		do
			initialize_change
			set_changed_region_empty
			compute_cut_dies
			last_isolated_count := 0
			last_unisolated_count := 0
			last_isolation_failure_count := 0
			apply_action_with_filter (selected_isolatable_filter, toggle_isolation_action)
			redraw_changed_region
			set_final_selected_elements
			add_change_to_history_list (new_standard_change ("Toggle Isolation"))
		end

	trim_all_scribes (a_redraw_requested: BOOLEAN)
			-- Resize all scribes that intersect at least
			-- one field of view so that ends which are
			-- outside any field of view are "trimmed"
			-- back to field of view edge.  Redraw the
			-- result if `a_redraw_requested'
		do
			trim_scribes (all_filter, a_redraw_requested)
		end

	trim_selected_scribes (a_redraw_requested: BOOLEAN)
			-- Resize selected scribes that intersect at
			-- least one field of view so that ends which
			-- are outside any field of view are "trimmed"
			-- back to field of view edge.  Redraw the
			-- result if `a_redraw_requested'
		do
			trim_scribes (selected_filter, a_redraw_requested)
		end

	separate_all_scribes (a_redraw_requested: BOOLEAN)
			-- Replace every scribe that intersects more
			-- than one field of view with a set of
			-- scribes that represents the intersection of
			-- the scribe with each field.  Redraw the
			-- result if `a_redraw_requested'
		do
			separate_scribes (all_filter, a_redraw_requested)
		end

	separate_selected_scribes (a_redraw_requested: BOOLEAN)
			-- Replace selected scribes that intersect
			-- more than one field of view with a set of
			-- scribes that represents the intersection of
			-- the scribe with each field.  Redraw the
			-- result if `a_redraw_requested'
		do
			separate_scribes (selected_filter, a_redraw_requested)
		end

	extend_all_scribes (a_redraw_requested: BOOLEAN)
			-- Extend every scribe to meet a scribe of
			-- different orientation or field of view
			-- edge.  Redraw the result if
			-- `a_redraw_requested'
		do
			extend_scribes (False, a_redraw_requested)
		end

	extend_selected_scribes (a_redraw_requested: BOOLEAN)
			-- Extend selected scribe to meet a scribe of
			-- different orientation or field of view
			-- edge.  Redraw the result if
			-- `a_redraw_requested'
		do
			extend_scribes (True, a_redraw_requested)
		end

	unoverlap_all_scribes (a_redraw_requested: BOOLEAN)
			-- Remove overlaps of all scribes with other
			-- scribes of the opposite orientation.
			-- Redraw the result if `a_redraw_requested'
		do
			unoverlap_scribes (False, a_redraw_requested)
		end

	unoverlap_selected_scribes (a_redraw_requested: BOOLEAN)
			-- Remove overlaps of selected scribes with
			-- other selected scribes of the opposite
			-- orientation.  Redraw the result if
			-- `a_redraw_requested'
		do
			unoverlap_scribes (True, a_redraw_requested)
		end

	check_scribe_join_validity
			-- Check whether it is valid to join scribes.
			-- Every scribe that intersects any field of view
			-- (the ones possibly affected by the join) must
			-- be resizable in length dimension and deletable.
			-- Set `is_scribe_join_valid'
		do
			initialize_status
			create {ARRAYED_LIST [like anchored_view_element]} locked_scribes.make (elements.count)
			create {ARRAYED_LIST [like anchored_view_element]} undeletable_scribes.make (elements.count)
			apply_action_with_filter (scribe_filter, check_join_validity_action)
		end

	start_join
			-- Take actions needs before Join Scribes operation
		do
			initialize_change
			set_final_selected_elements
			add_change_to_history_list (new_creation_and_deletion_change ("Separate Scribe"))
		end

	end_join
			-- Take actions needs after Join Scribes operation
		do
			redraw
			set_final_selected_elements
			add_change_to_history_list (new_creation_and_deletion_change ("Join Scribes"))
		end

	start_pack_validity_check
			-- Take actions needed before starting validity
			-- check with multiple parts in Pack mode
		do
			initialize_status
			compute_cut_dies
			build_mbb_model_areas
			build_scribe_cut_die_model_areas
			build_scribe_model_areas
		end

	end_pack_validity_check (scribes_included: BOOLEAN)
			-- Take actions needed after finish validity
			-- check with multiple parts in Pack mode
		do
			all_pack_validity_color_generator.set_scribes_included (scribes_included)
			set_validity_color_generator (all_pack_validity_color_generator)
			show_validity_view_or_redraw
		end

	start_cut_short_validity_check
			-- Take actions needed before starting short validity
			-- in Cut mode
		do
			initialize_status
		end

	start_cut_validity_check
			-- Take actions needed before starting validity
			-- check with multiple parts in Cut mode
		do
			initialize_status
			compute_cut_dies
			build_cut_cut_die_model_areas
		end

	end_cut_validity_check
			-- Take actions needed after finish validity
			-- check with multiple parts in Cut mode
		do
			set_validity_color_generator (all_cut_validity_color_generator)
			show_validity_view_or_redraw
		end

	undo_one_change
			-- Undo previous history list change made to packing
		local
			list: like reticle_change_history_list
		do
			initialize_status
			list := reticle_change_history_list
			if list.is_empty then
				add_error ("change history list is empty")
			elseif list.isfirst then
				add_error ("no previous change in history list")
			else
				list.back
				last_change := list.item
				last_change.undo
				restore_state_and_redraw (last_change, True)
			end
			call_change_history_list_actions
		end

	redo_one_change
			-- Redo next history list change made to packing
		local
			list: like reticle_change_history_list
		do
			initialize_status
			list := reticle_change_history_list
			if list.is_empty then
				add_error ("change history list is empty")
			elseif list.after then
				add_error ("no next change in history list")
			else
				last_change := list.item
				last_change.redo
				restore_state_and_redraw (last_change, False)
				list.forth
			end
			call_change_history_list_actions
		end

	check_all_pack_validity
			-- Check packing for validity issues in Pack mode.
			-- Do all validity checks
		do
			start_pack_validity_check
			check_outside_fields_body
			check_overlapping_field_edge_body (True)
			check_overlapping_objects_body (True)
			check_below_minimum_size_body
			check_lot_not_assigned_body
			check_unlocked_or_unassigned_projects_body
			check_close_to_field_edge_body (True)
			check_close_to_object_body (True)
			check_scribe_cut_die_too_big_body
			check_unisolated_in_cut_die_body
			check_uncentered_in_cut_die_body
			check_floating_scribes_body
			check_misaligned_scribes_body
			check_small_scribes_and_fillers_body (True)
			check_test_vehicles_body
			check_custom_width_scribes_body
			set_scribe_includable_filters (True)
			sort_elements_by_filters (all_pack_validity_sort_filters)
			end_pack_validity_check (True)
		end

	check_almost_all_pack_validity
			-- Check packing for validity issues in Pack mode.
			-- Skip check for objects outside all fields
		do
			start_pack_validity_check
			check_overlapping_field_edge_body (True)
			check_overlapping_objects_body (True)
			check_below_minimum_size_body
			check_lot_not_assigned_body
			check_unlocked_or_unassigned_projects_body
			check_close_to_field_edge_body (True)
			check_close_to_object_body (True)
			check_scribe_cut_die_too_big_body
			check_unisolated_in_cut_die_body
			check_uncentered_in_cut_die_body
			check_floating_scribes_body
			check_misaligned_scribes_body
			check_small_scribes_and_fillers_body (True)
			check_test_vehicles_body
			check_custom_width_scribes_body
			set_scribe_includable_filters (True)
			sort_elements_by_filters (almost_all_pack_validity_sort_filters)
			end_pack_validity_check (True)
		end

	check_before_join_scribes_validity
			-- Check packing for validity issues in Pack mode.
			-- Skip check for objects outside all fields and
			-- all checks for problems that are likely to be
			-- fixed by a Join Scribes operation
		do
			start_pack_validity_check
			check_overlapping_field_edge_body (False)
			check_overlapping_objects_body (False)
			check_below_minimum_size_body
			check_lot_not_assigned_body
			check_unlocked_or_unassigned_projects_body
			check_close_to_field_edge_body (False)
			check_close_to_object_body (False)
			check_scribe_cut_die_too_big_body
			check_unisolated_in_cut_die_body
			check_uncentered_in_cut_die_body
			clear_floating_scribes_body
			check_misaligned_scribes_body
			check_small_scribes_and_fillers_body (False)
			check_test_vehicles_body
			check_custom_width_scribes_body
			set_scribe_includable_filters (False)
			sort_elements_by_filters (before_join_scribes_validity_sort_filters)
			end_pack_validity_check (False)
		end

	check_all_cut_validity
			-- Check packing for validity issues in Cut mode
		do
			start_cut_short_validity_check
			check_has_isolated_sites_body
			if error_count = 0 then
				start_cut_validity_check
				check_invalid_cut_die_body
				check_wrong_lot_body
				check_cut_cut_die_too_big_body
				check_non_default_cut_die_size_body
				check_shared_cut_die_body
				check_extra_cut_body
				sort_elements_by_filters (<< cut_cut_die_invalid_filter, wrong_lot_filter, cut_cut_die_too_big_filter, shares_cut_die_filter, non_default_cut_size_filter, extra_cut_filter >>)
				end_cut_validity_check
			end
		end

	check_outside_fields_items
			-- Check for objects that are completely outside all
			-- fields of view
		do
			initialize_status
			check_outside_fields_body
			sort_elements_by_filters (<< outside_fields_filter >>)
			show_validity_view_or_redraw
		end

	check_outside_fields_body
			-- Check for objects that are completely outside all
			-- fields of view, without initialization or redraw
		do
			set_validity_color_generator (outside_fields_color_generator)
			apply_action_with_filter (die_filter, check_outside_fields_action)
		end

	check_overlapping_field_edge_items
			-- Check for objects that intersect at least one
			-- field of view but are not enclosed by any field
		do
			initialize_status
			check_overlapping_field_edge_body (True)
			partially_outside_fields_alt_filter.set_scribes_included (True)
			sort_elements_by_filters (<< partially_outside_fields_alt_filter >>)
			show_validity_view_or_redraw
		end

	check_overlapping_field_edge_body (scribes_included: BOOLEAN)
			-- Check for objects that intersect at least one
			-- field of view but are not enclosed by any field,
			-- without initialization or redraw
		do
			overlapping_field_color_generator.set_scribes_included (scribes_included)
			set_validity_color_generator (overlapping_field_color_generator)
			check_overlapping_field_action.set_scribes_included (scribes_included)
			apply_action_with_filter (die_filter, check_overlapping_field_action)
		end

	check_overlapping_object_items
			-- Check for objects that intersect at least one
			-- field of view and overlap another object that
			-- intersects at least one field of view
		do
			initialize_status
			build_mbb_model_areas
			check_overlapping_objects_body (True)
			sort_elements_by_filters (<< overlaps_other_alt_filter >>)
			show_validity_view_or_redraw
		end

	check_overlapping_objects_body (scribes_included: BOOLEAN)
			-- Check for objects that intersect at least one
			-- field of view and overlap another object that
			-- intersects at least one field of view,
			-- without initialization or redraw
		do
			unmark_and_clear_invalid_all
			apply_action_with_filter (all_filter, clear_overlapping_object_action)
			set_validity_color_generator (overlapping_object_color_generator)
			check_overlapping_object_action.set_scribes_included (scribes_included)
			apply_action_with_filter (die_filter, check_overlapping_object_action)
		end

	check_below_minimum_size_items
			-- Check for objects that are below minimum legal
			-- size in either dimension
		do
			initialize_status
			check_below_minimum_size_body
			sort_elements_by_filters (<< size_too_small_filter >>)
			show_validity_view_or_redraw
		end

	check_below_minimum_size_body
			-- Check for objects that are below minimum
			-- legal size in either dimension, without
			-- initialization or redraw
		do
			clear_processed_indicators
			set_validity_color_generator (below_min_size_color_generator)
			apply_action_with_filter (die_filter, check_below_min_size_action)
		end

	check_lot_not_assigned_items
			-- Check for objects that are customer projects for
			-- which a specific lot has not been assigned
		do
			initialize_status
			check_lot_not_assigned_body
			sort_elements_by_filters (<< wafer_etest_id_unassigned_filter >>)
			show_validity_view_or_redraw
		end

	check_lot_not_assigned_body
			-- Check for objects that are customer projects for
			-- which a specific lot has not been assigned
			-- without initialization or redraw
		do
			clear_processed_indicators
			set_validity_color_generator (needs_lot_assignment_color_generator)
			apply_action_with_filter (die_filter, check_no_lot_assigned_action)
		end

	check_unlocked_or_unassigned_projects
			-- Check for projects that are not locked against
			-- customer update or are not assigned to run being
			-- closed
		do
			initialize_status
			check_unlocked_or_unassigned_projects_body
			sort_elements_by_filters (<< unlocked_or_unassigned_filter >>)
			show_validity_view_or_redraw
		end

	check_unlocked_or_unassigned_projects_body
			-- Check for projects that are not locked against
			-- customer update or are not assigned to run being
			-- closed, without initialization or redraw
		do
			clear_processed_indicators
			set_validity_color_generator (unlocked_unassigned_color_generator)
			apply_action_with_filter (die_filter, check_unlocked_or_unassigned_action)
		end

	check_close_to_field_edge_items
			-- Check for objects that intersect at least one
			-- field of view and have at least one edge that
			-- is too close to the corresponding field edge
		do
			initialize_status
			check_close_to_field_edge_body (True)
			too_close_to_edge_alt_filter.set_scribes_included (True)
			sort_elements_by_filters (<< too_close_to_edge_alt_filter >>)
			show_validity_view_or_redraw
		end

	check_close_to_field_edge_body (scribes_included: BOOLEAN)
			-- Check for objects that intersect at least one
			-- field of view and have at least one edge that
			-- is too close to the corresponding field edge
			-- without initialization or redraw
		do
			
			close_to_edge_color_generator.set_scribes_included (scribes_included)
			set_validity_color_generator (close_to_edge_color_generator)
			check_too_close_to_edge_action.set_scribes_included (scribes_included)
			apply_action_with_filter (die_filter, check_too_close_to_edge_action)
		end

	check_close_to_object_items
			-- Check for objects that intersect at least one
			-- field of view and are too close to another object
			-- that also intersects at least one field
		do
			initialize_status
			build_mbb_model_areas
			check_close_to_object_body (True)
			sort_elements_by_filters (<< too_close_to_object_alt_filter >>)
			show_validity_view_or_redraw
		end

	check_close_to_object_body (scribes_included: BOOLEAN)
			-- Check for objects that intersect at least one
			-- field of view and are too close to another object
			-- that also intersects at least one field,
			-- without initialization or redraw
		do
			unmark_and_clear_invalid_all
			apply_action_with_filter (all_filter, clear_close_to_object_action)
			set_validity_color_generator (close_to_object_color_generator)
			check_too_close_to_object_action.set_scribes_included (scribes_included)
			apply_action_with_filter (die_filter, check_too_close_to_object_action)
		end

	check_scribe_cut_die_too_big_items
			-- Check for projects that intersect at least
			-- one field of view and have a cut die size
			-- determined by surrounding scribes that
			-- exceeds their maximum die size
		do
			initialize_status
			compute_cut_dies
			check_scribe_cut_die_too_big_body
			sort_elements_by_filters (<< scribe_cut_die_too_big_filter >>)
			show_validity_view_or_redraw
		end

	check_scribe_cut_die_too_big_body
			-- Check for projects that intersect at least
			-- one field of view and have a cut die size
			-- determined by surrounding scribes that
			-- exceeds their maximum die size, without
			-- initialization or redraw
		do
			set_validity_color_generator (scribe_cut_die_too_big_color_generator)
			apply_action_with_filter (die_filter, check_scribe_cut_die_too_big_action)
		end

	check_unisolated_in_cut_die_items
			-- Check for projects that are not isolated in their
			-- cut dies
		do
			initialize_status
			compute_cut_dies
			build_scribe_cut_die_model_areas
			check_unisolated_in_cut_die_body
			sort_elements_by_filters (<< unisolated_in_cut_die_alt_filter >>)
			show_validity_view_or_redraw
		end

	check_unisolated_in_cut_die_body
			-- Check for projects that are not isolated in
			-- their cut dies, without initialization or
			-- redraw
		do
			apply_action_with_filter (all_filter, clear_unisolated_in_scribe_cut_die_action)
			set_validity_color_generator (not_isolated_in_cut_die_color_generator)
			apply_action_with_filter (isolatable_filter, check_unisolated_in_scribe_cut_die_action)
		end

	check_uncentered_in_cut_die_items
			-- Check for projects that are not centered in their
			-- cut dies, using edges of surrounding scribes instead
			-- of centers
		do
			initialize_status
			compute_cut_dies
			check_uncentered_in_cut_die_body
			sort_elements_by_filters (<< uncentered_in_cut_die_filter >>)
			show_validity_view_or_redraw
		end

	check_uncentered_in_cut_die_body
			-- Check for projects that are not centered in their
			-- cut dies, using edges of surrounding scribes instead
			-- of centers, without initialization or redraw
		do
			set_validity_color_generator (not_centered_in_cut_die_color_generator)
			apply_action_with_filter (die_filter, check_uncentered_in_cut_die_action)
		end

	check_floating_scribes
			-- Check for floating scribes (ones that do not abut
			-- either the field of view edge or a scribe of
			-- opposite orientation at either end)
		do
			initialize_status
			check_floating_scribes_body
			sort_elements_by_filters (<< floating_scribe_alt_filter >>)
			show_validity_view_or_redraw
		end

	check_floating_scribes_body
			-- Check for floating scribes (ones that do
			-- not abut either the field of view edge or a
			-- scribe of opposite orientation at either
			-- end), without initialization or redraw
		do
			initialize_floating_scribe_filter
			set_validity_color_generator (floating_scribe_color_generator)
			apply_action_with_filter (all_filter, clear_floating_scribe_action)
			apply_action_with_filter (floating_scribe_filter, set_floating_scribe_action)
		end

	clear_floating_scribes_body
			-- Clear floating scribe indicator for all objects
		do
			apply_action_with_filter (all_filter, clear_floating_scribe_action)
		end

	check_misaligned_scribes
			-- Check for misaligned scribes (ones that do not
			-- quite line up)
		do
			initialize_status
			build_scribe_model_areas
			check_misaligned_scribes_body
			sort_elements_by_filters (<< misaligned_scribe_alt_filter >>)
			show_validity_view_or_redraw
		end

	check_misaligned_scribes_body
			-- Check for misaligned scribes (ones that do
			-- not quite line up), without initialization
			-- or redraw
		do
			unmark_and_clear_invalid_all
			apply_action_with_filter (all_filter, clear_misaligned_scribe_action)
			set_validity_color_generator (misaligned_scribe_color_generator)
			apply_action_with_filter (scribe_filter, check_misaligned_scribe_action)
		end

	check_small_scribes_and_fillers
			-- Check for scribes and fillers that are legal
			-- but are very small and so possibly erroneous
		do
			initialize_status
			check_small_scribes_and_fillers_body (True)
			small_scribe_and_filler_alt_filter.set_scribes_included (True)
			sort_elements_by_filters (<< small_scribe_and_filler_alt_filter >>)
			show_validity_view_or_redraw
		end

	check_small_scribes_and_fillers_body (scribes_included: BOOLEAN)
			-- Check for scribes and fillers that are legal
			-- but are very small and so possibly erroneous,
			-- without initialization or redraw
		do
			clear_processed_indicators
			small_but_legal_color_generator.set_scribes_included (scribes_included)
			set_validity_color_generator (small_but_legal_color_generator)
			small_but_legal_action.set_scribes_included (scribes_included)
			apply_action_with_filter (small_scribe_and_filler_filter, small_but_legal_action)
		end

	check_test_vehicles
			-- Check for test vehicle issues with packing
		do
			initialize_status
			check_test_vehicles_body
		end

	check_test_vehicles_body
			-- Check for test vehicle issues with packing,
			-- without initialization or redraw
		local
			rot_0, rot_90, rot_180, rot_270, rot_count: INTEGER
		do
			clear_processed_indicators
			check_test_vehicle_action.initialize
			apply_action_with_filter (test_vehicle_filter, check_test_vehicle_action)
			rot_0 := check_test_vehicle_action.rotated_0_count
			rot_90 := check_test_vehicle_action.rotated_90_count
			rot_180 := check_test_vehicle_action.rotated_180_count
			rot_270 := check_test_vehicle_action.rotated_270_count
			rot_count := 0
			if rot_0 > 0 then
				rot_count := rot_count + 1
			end
			if rot_90 > 0 then
				rot_count := rot_count + 1
			end
			if rot_180 > 0 then
				rot_count := rot_count + 1
			end
			if rot_270 > 0 then
				rot_count := rot_count + 1
			end
			if rot_0 = 0 and rot_90 = 0 and rot_180 = 0 and rot_270 = 0 then
				add_warning ("No test vehicles included in packing")
			elseif rot_count > 1 then
				add_warning ("Packing includes test vehicles with different rotations (requires extra probing)")
			end
		end

	check_multiple_scribe_widths
			-- Check for multiple different scribe
			-- widths and set `warning_messages' if found
		local
			msg: STRING
		do
			-- Check for custom width scribes and
			-- multiple scribe widths
			check_custom_width_scribes
			
			-- Discard any custom width scribe
			-- warnings
			initialize_status

			msg := check_scribe_width_action.multiple_widths_warning
			if not msg.is_empty then
				warning_messages.extend (msg)
			end
		end

	check_custom_width_scribes
			-- Check for custom width scribes and warn if any
			-- present
		do
			initialize_status
			check_custom_width_scribes_body
		end

	check_custom_width_scribes_body
			-- Check for custom width scribes and warn if any
			-- present, without initialization or redraw
		do
			check_scribe_width_action.initialize
			apply_action_with_filter (scribe_filter, check_scribe_width_action)
			check_scribe_width_action.finish
		end

	check_has_isolated_sites_body
			-- Check that current cut pattern has at least one
			-- isolated site
		do
			if not current_cut_pattern.has_isolated_sites then
				add_error ("Cut pattern does not have any isolated sites")
			end
		end

	check_invalid_cut_die_items
			-- Check for projects that are isolated in cut pattern
			-- but cut die determined by surrounding cuts is
			-- invalid (not surrounded by cuts on all sides or
			-- has cut through the printed die)
		do
			initialize_status
			compute_cut_dies
			check_invalid_cut_die_body
			sort_elements_by_filters (<< cut_cut_die_invalid_filter >>)
			show_validity_view_or_redraw
		end

	check_invalid_cut_die_body
			-- Check for projects that are isolated in cut
			-- pattern but cut die determined by
			-- surrounding cuts is invalid (not surrounded
			-- by cuts on all sides or has cut through the
			-- printed die), without initialization or
			-- redraw
		do
			set_validity_color_generator (invalid_cut_die_color_generator)
			apply_action_with_filter (die_filter, check_invalid_cut_die_action)
		end

	check_cut_cut_die_too_big_items
			-- Check for projects that have a cut die size
			-- determined by surrounding cuts that exceeds
			-- their maximum die size
		do
			initialize_status
			compute_cut_dies
			check_cut_cut_die_too_big_body
			sort_elements_by_filters (<< cut_cut_die_too_big_filter >>)
			show_validity_view_or_redraw
		end

	check_cut_cut_die_too_big_body
			-- Check for projects that have a cut die size
			-- determined by surrounding cuts that exceeds
			-- their maximum die size, without
			-- initialization or redraw
		do
			set_validity_color_generator (cut_cut_die_too_big_color_generator)
			apply_action_with_filter (die_filter, check_cut_cut_die_too_big_action)
		end

	check_wrong_lot_items
			-- Check for projects that have the wrong lot
			-- (different than the one for the cut pattern)
		do
			initialize_status
			check_wrong_lot_body
			sort_elements_by_filters (<< wrong_lot_filter >>)
			show_validity_view_or_redraw
		end

	check_wrong_lot_body
			-- Check for isolated projects that have the wrong lot
			-- (different than the one for the cut pattern),
			-- without initialization or redraw
		do
			set_validity_color_generator (wrong_lot_color_generator)
			apply_action_with_filter (die_filter, check_wrong_lot_action)
			if not current_cut_pattern.has_wafer_etest_name then
				add_warning ("Cannot check isolated sites for wrong lot because cut pattern lot has not been specified")
			end
		end

	check_non_default_cut_die_size_items
			-- Check for isolated elements that have a die size
			-- determined by surrounding cuts that is different
			-- than the die size determined by surrounding scribes
		do
			initialize_status
			compute_cut_dies
			check_non_default_cut_die_size_body
			sort_elements_by_filters (<< non_default_cut_size_filter >>)
			show_validity_view_or_redraw
		end

	check_non_default_cut_die_size_body
			-- Check for isolated elements that have a die size
			-- determined by surrounding cuts that is different
			-- than the die size determined by surrounding scribes,
			-- without initialization or redraw
		do
			set_validity_color_generator (non_default_cut_die_size_color_generator)
			apply_action_with_filter (die_filter, check_non_default_cut_die_size_action)
		end

	check_shared_cut_die_items
			-- Check for isolated elements that have a cut
			-- die determined by surrounding cuts that is
			-- shared by partially or fully by one or more
			-- other non-scribe die elements
		do
			initialize_status
			compute_cut_dies
			build_cut_cut_die_model_areas
			check_shared_cut_die_body
			sort_elements_by_filters (<< shares_cut_die_filter >>)
			show_validity_view_or_redraw
		end

	check_shared_cut_die_body
			-- Check for isolated elements that have a cut
			-- die determined by surrounding cuts that is
			-- shared partially or fully by one or more
			-- other non-scribe die elements, without
			-- initialization or redraw
		do
			apply_action_with_filter (all_filter, clear_unisolated_in_cut_cut_die_action)
			set_validity_color_generator (shared_cut_die_color_generator)
			apply_action_with_filter (die_filter, check_shared_cut_die_action)
		end

	check_extra_cut_items
			-- Check for cuts that are not an isolating cut
			-- for any isolated site
		do
			initialize_status
			compute_cut_dies
			check_extra_cut_body
			sort_elements_by_filters (<< extra_cut_filter >>)
			show_validity_view_or_redraw
		end

	check_extra_cut_body
			-- Check for cuts that are not an isolating
			-- cut for any isolated site, without
			-- initialization or redraw
		do
			set_validity_color_generator (extra_cut_color_generator)
			apply_action_with_filter (cut_filter, check_extra_cut_action)
		end

	set_validity_color_generator (a_generator: RETICLE_VIEW_ELEMENT_COLOR_GENERATOR)
			-- Set current color generator used by Validity
			-- Check color generator
		do
			validity_check_color_generator.set_current_generator (a_generator)
		end

	sort_elements_by_filters (a_filter_list: ARRAY [VIEW_ELEMENT_FILTER])
			-- Sort `elements' according to
			-- `a_filter_list'.  First element is drawn
			-- first.  Put unselectable elements first,
			-- then elements that are selectable but do not
			-- match any of the supplied filters, then
			-- elements that match the last filter in the
			-- list, etc.  An element that matches
			-- multiple filters is included only in the
			-- first filter in `a_filter_list' that it
			-- matches.  Also, set to "selected" only those
			-- elements that have at least one validity
			-- issue (others are set to "not selected")
		local
			k, lower, upper, unsel_count, nomatch_count: INTEGER
			sel_count: INTEGER
			match_counts: ARRAY [INTEGER]
			elem: like anchored_view_element
			elems: like elements
			matched: BOOLEAN
			l_filter: VIEW_ELEMENT_FILTER
			unsel_list, nomatch_list, temp_list: ARRAYED_LIST [like anchored_view_element]
			match_lists: ARRAY [ARRAYED_LIST [like anchored_view_element]]
			temp_elements: ARRAYED_LIST [like anchored_view_element]
		do	
			create match_counts.make (a_filter_list.lower, a_filter_list.upper)
			match_counts.clear_all
			
			-- Compute `unsel_count', `nomatch_count' and `match_counts'

			elems := elements
			lower := a_filter_list.lower
			upper := a_filter_list.upper
			from
				elems.start
			until
				elems.after
			loop
			 	elem := elems.item
				if not elem.is_selectable then
					unsel_count := unsel_count + 1
				else
					matched := False
					from
						k := lower
					until
						k > upper or matched
					loop
						l_filter := a_filter_list.item (k)
						if l_filter.selects (elem) then
							match_counts.put (match_counts.item (k) + 1, k)
							matched := True
						end
						k := k + 1
					end
					if not matched then
						nomatch_count := nomatch_count + 1
					end
				end
				elems.forth
			end
		
			-- Allocate lists of correct length

			create unsel_list.make (unsel_count)
			create nomatch_list.make (nomatch_count)
			create match_lists.make (lower, upper)
			from
				k := lower
			until
				k > upper
			loop
				create temp_list.make (match_counts.item (k))
				match_lists.put (temp_list, k)
				k := k + 1
			end
		
			-- Scan `elements' again and put each element in correct list.
			-- Also, compute count of elements with validity issues
			-- and put in `sel_count'

			sel_count := 0
			from
				elems.start
			until
				elems.after
			loop
			 	elem := elems.item
				if not elem.is_selectable then
					unsel_list.extend (elem)
				else
					matched := False
					from
						k := lower
					until
						k > upper or matched
					loop
						l_filter := a_filter_list.item (k)
						if l_filter.selects (elem) then
							elem.set_is_selected (True)
							sel_count := sel_count + 1
							match_lists.item (k).extend (elem)
							matched := True
						end
						k := k + 1
					end
					if not matched then
						elem.set_is_selected (False)
						nomatch_list.extend (elem)
					end
				end
				elems.forth
			end
		
			-- Reorder `elements' so that `unsel_list' is first,
			-- then `no_match_list', then lists in `match_lists'
			-- from last to first (first ones have highest priority,
			-- so they appear last in final list, which means they
			-- are drawn on top of other elements)

			create temp_elements.make (elements.count)
			temp_elements.append (unsel_list)
			temp_elements.append (nomatch_list)
			from
				k := upper
			until
				k < lower
			loop
				temp_elements.append (match_lists.item (k))
				k := k - 1
			end

			elements := temp_elements
			
			-- Now make sure the selection_bounding box and
			-- number of selected elements is correct
			
			set_changed_region_empty
			set_selection_bounding_box_empty
			apply_action_with_filter (all_filter, selection_changed_action)
			set_selected_count (sel_count)
		end

	show_validity_view_or_redraw
			-- Switch to Validity Check view if it is not
			-- the current named view.  Otherwise, just
			-- redraw everything
		do
			if current_view /= validity_check_view then
				set_validity_check_view
			else
				redraw
			end
		end

	redraw_if_max_size_view
			-- If current named view is the Max Die Size view,
			-- then redraw it
		do
			if current_view = max_die_size_view then
				redraw
			end
		end

	add_model_elements (list: LIST [like anchored_model_element] with_redraw: BOOLEAN)
			-- Add model elements in `list' to view.  Redraw
			-- model if `with_redraw'
		local
			elem: like anchored_model_element
			view_elem: like anchored_view_element
		do
			set_changed_region_empty
			from
				list.start
			until
				list.after
			loop
				elem := list.item
				view_elem := new_view_element (elem)
				view_elem.set_view_coordinates
				view_elem.add_to_changed_region
				elements.extend (view_elem)
				list.forth
			end
			if with_redraw then
				redraw_changed_region
			end
			clear_change_history_list
		end

	remove_model_elements (list: LIST [PACKING_DIES_MROW])
			-- Remove model elements corresponding to dies
			-- in `list' from model and view
		local
			del_list: ARRAYED_LIST [like anchored_view_element]
		do
			create del_list.make (list.count)
			delete_dies_action.set_dies (list)
			delete_dies_action.set_deleted_elements (del_list)
			apply_select_action_and_redraw (delete_dies_action)
			update_copies_table_for_delete (del_list)
			model.set_from_element_list (model_elements)
			clear_change_history_list
		end

	update_model_elements (list: LIST [PACKING_DIES_MROW])
			-- Redraw model elements in `list' to display them
			-- properly
		do
			update_dies_action.set_dies (list)
			apply_select_action_and_redraw (update_dies_action)
			clear_change_history_list
		end

	assign_lot_to_selected_projects (a_wafer_etest_row: WAFER_ETEST_NAMES_ROW)
			-- Assign lot indicated by `a_wafer_etest_row' to
			-- all selected customer projects that have more
			-- than one compatible wafer etest ID.  If row is
			-- Void, then remove any existing assignment for
			-- these projects
		do
			initialize_change
			assign_lot_action.set_wafer_etest_row (a_wafer_etest_row)
			if current_view = lot_split_view then
				apply_action_with_filter_and_redraw (cust_project_etest_filter, assign_lot_action)
			else
				set_changed_region_empty
				apply_action_with_filter (cust_project_etest_filter, assign_lot_action)
			end
			set_final_selected_elements_to_initial
			add_change_to_history_list (new_standard_change ("Assign Lot"))
		end

	compute_cut_dies
			-- Compute scribe cut die (using centers of surrounding
			-- scribes), edge cut die (using closest edges of
			-- surrounding scribes) and cut cut die (using
			-- surrounding cuts) for all projects
		local
			horiz_scribes: LIST [like anchored_view_element]
			vert_scribes: LIST [like anchored_view_element]
			horiz_cuts: LIST [like anchored_view_element]
			vert_cuts: LIST [like anchored_view_element]
			l_sorter: LIST_QUICK_SORTER [like anchored_view_element]
			x_comparator: CENTER_X_COMPARATOR [like anchored_view_element]
			y_comparator: CENTER_Y_COMPARATOR [like anchored_view_element]
		do
			-- Make lists of horizontal and vertical scribes
			-- and cuts

			create {ARRAYED_LIST [like anchored_view_element]} horiz_scribes.make (elements.count)
			create {ARRAYED_LIST [like anchored_view_element]} vert_scribes.make (elements.count)
			create {ARRAYED_LIST [like anchored_view_element]} horiz_cuts.make (elements.count)
			create {ARRAYED_LIST [like anchored_view_element]} vert_cuts.make (elements.count)
			
			extend_list_action.set_list (horiz_scribes)
			apply_action_with_filter (int_horizontal_scribe_filter, extend_list_action)
			extend_list_action.set_list (vert_scribes)
			apply_action_with_filter (int_vertical_scribe_filter, extend_list_action)
			extend_list_action.set_list (horiz_cuts)
			apply_action_with_filter (horizontal_cut_filter, extend_list_action)
			extend_list_action.set_list (vert_cuts)
			apply_action_with_filter (vertical_cut_filter, extend_list_action)

			-- Sort lists of horizontal and vertical scribes
			-- by center in X and Y, respectively
			
			create l_sorter
			create x_comparator
			create y_comparator
			horiz_scribes := l_sorter.sorted (horiz_scribes, y_comparator)
			vert_scribes := l_sorter.sorted (vert_scribes, x_comparator)
			horiz_cuts := l_sorter.sorted (horiz_cuts, y_comparator)
			vert_cuts := l_sorter.sorted (vert_cuts, x_comparator)
			compute_cut_die_action.set_grid (grid)
			compute_cut_die_action.set_default_scribe_width (default_scribe_width)
			compute_cut_die_action.set_horizontal_scribes (horiz_scribes)
			compute_cut_die_action.set_vertical_scribes (vert_scribes)
			compute_cut_die_action.set_horizontal_cuts (horiz_cuts)
			compute_cut_die_action.set_vertical_cuts (vert_cuts)
			apply_action_with_filter (all_filter, clear_isolating_cut_action)
			apply_action_with_filter (isolatable_filter, compute_cut_die_action)
			
		end

	recompute_isolated_site_cut_dies
			-- Recompute the cut die size and center for all
			-- isolated sites, to guarantee that values are
			-- correct for a completed cut pattern
		do
			compute_cut_dies
			recompute_isolated_site_cut_dies_body
		end

	recompute_isolated_site_cut_dies_body
			-- Recompute the cut die size and center for all
			-- isolated sites, to guarantee that values are
			-- correct for a completed cut pattern
		do
			apply_action_with_filter (isolatable_filter, set_isolated_site_cut_die_action)
		end

	select_all_items
			-- Set all selectable items to be selected
		do
			set_selection (all_filter)
		end

	unselect_all_items
			-- Set all items to be unselected
		do
			set_selection (none_filter)
		end

	invert_selection
			-- Set all items that are selected to be unselected and
			-- all items that are unselected and can be selected
			-- to selected
		do
			set_selection (unselected_filter)
		end

	set_selection_to_cust_project_items
			-- Set selection to all items which are
			-- customer projects
		do
			set_selection (cust_project_filter)
		end
	
	set_selection_to_test_vehicle_items
			-- Set selection to all items which are test vehicles
		do
			set_selection (test_vehicle_filter)
		end
	
	set_selection_to_project_items
			-- Set selection to all items which are projects
		do
			set_selection (project_filter)
		end
	
	set_selection_to_scribe_items
			-- Set selection to all items which are scribes
		do
			set_selection (scribe_filter)
		end
	
	set_selection_to_horizontal_scribe_items
			-- Set selection to all items which are
			-- horizontal scribes
		do
			set_selection (horizontal_scribe_filter)
		end
	
	set_selection_to_vertical_scribe_items
			-- Set selection to all items which are
			-- vertical scribes
		do
			set_selection (vertical_scribe_filter)
		end
	
	set_selection_to_floating_scribe_items
			-- Set selection to all items which are
			-- floating scribes (either end not on edge of scribe
			-- of opposite orientation or field of view edge)
		do
			initialize_floating_scribe_filter
			set_selection (floating_scribe_filter)
		end
	
	set_selection_to_misaligned_scribe_items
			-- Set selection to all items which are
			-- misaligned scribes (not quite lined up)
		do
			build_scribe_model_areas
			set_selection (misaligned_scribe_filter)
		end
	
	set_selection_to_default_width_scribe_items
			-- Set selection to all items which are
			-- scribes with the default width
		do
			set_selection (default_width_scribe_filter)
		end
	
	set_selection_to_custom_width_scribe_items
			-- Set selection to all items which are
			-- scribes with non-default width
		do
			set_selection (custom_width_scribe_filter)
		end
	
	set_selection_to_non_scribe_items
			-- Set selection to all items which are not scribes
		do
			set_selection (non_scribe_filter)
		end
	
	set_selection_to_filler_items
			-- Set selection_to all items which are fillers
		do
			set_selection (filler_filter)
		end
	
	set_selection_to_vendor_geometry_items
			-- Set selection to all items which are vendor
			-- geometries/ghosts
		do
			set_selection (vendor_geometry_filter)
		end
	
	set_selection_to_cut_items
			-- Set selection to all items which are cuts
		do
			set_selection (cut_filter)
		end
	
	set_selection_to_commercial_account_items
			-- Set selection to all items whose account
			-- type is commercial
		do
			set_selection (commercial_project_filter)
		end
	
	set_selection_to_mep_account_items
			-- Set seleection to all items whose account
			-- type is MEP
		do
			set_selection (mep_project_filter)
		end
	
	set_selection_to_other_account_items
			-- Set selection to all items whose account
			-- type is other than commercial or MEP
		do
			set_selection (other_project_filter)
		end
	
	set_selection_to_partially_selected_items
			-- Set selection to all items that are partially
			-- selected (have at least one copy selected but
			-- not all copies)
		do
			set_selection (partially_selected_filter)
		end
	
	set_selection_to_outside_fields_items
			-- Set selection to all items that are completely
			-- outside all fields of view
		do
			set_selection (outside_fields_filter)
		end
	
	set_selection_to_too_small_items
			-- Set selection to all objects whose size is below
			-- the minimum size in either dimension
		do
			set_selection (size_too_small_filter)
		end
	
	set_selection_to_small_scribes_and_fillers
			-- Set selection to all scribes and fillers whose
			-- size is below the small object threshold in either
			-- dimension (just length for scribes)
		do
			set_selection (small_scribe_and_filler_filter)
		end
	
	set_selection_to_overlapping_field_items
			-- Set selection to all objects which intersect
			-- at least one field of view by are not enclosed
			-- by any field
		do
			set_selection (partially_outside_fields_filter)
		end
	
	set_selection_to_overlapping_object_items
			-- Set selection to all objects which intersect
			-- at least one field of view and overlap another
			-- object
		do
			build_mbb_model_areas
			set_selection (overlaps_other_filter)
		end
	
	set_selection_to_close_edge_items
			-- Set selection to all objects which intersect
			-- at least one field of view and have at least one
			-- edge which is too close to field of view edge
		do
			set_selection (too_close_to_edge_filter)
		end
	
	set_selection_to_close_object_items
			-- Set selection to all objects which intersect
			-- at least one field of view and are too close to
			-- another object that intersects at least one field
		do
			build_mbb_model_areas
			set_selection (too_close_to_object_filter)
		end
	
	set_selection_to_too_big_items
			-- Set selection to all projects which have a die
			-- size that exceeds their maximum die size
		do
			compute_cut_dies
			set_selection (scribe_cut_die_too_big_filter)
		end
	
	set_selection_to_uncentered_items
			-- Set selection to all projects which are not
			-- centered in their cut die
		do
			compute_cut_dies
			set_selection (uncentered_in_cut_die_filter)
		end
	
	set_selection_to_unisolated_items
			-- Set selection to all projects which are not
			-- isolated in their cut die
		do
			compute_cut_dies
			build_scribe_cut_die_model_areas
			set_selection (unisolated_in_cut_die_filter)
		end
	
	set_selection_to_unassigned_lot_items
			-- Set selection to all objects which have associated
			-- wafer etest ID(s) but have not been assigned a
			-- specific one
		do
			set_selection (wafer_etest_id_unassigned_filter)
		end
	
	set_selection_to_unlocked_or_unassigned
			-- Set selection to all projects which are not
			-- locked against customer update or are not assigned
			-- to run being closed
		do
			set_selection (unlocked_or_unassigned_filter)
		end
	
	add_to_selection_cust_project_items
			-- Add to selection to all items which are
			-- customer projects
		do
			add_to_selection (cust_project_filter)
		end
	
	add_to_selection_test_vehicle_items
			-- Added to selection to all items which are
			-- test vehicles
		do
			add_to_selection (test_vehicle_filter)
		end
	
	add_to_selection_project_items
			-- Add to selection all items which are projects
		do
			add_to_selection (project_filter)
		end
	
	add_to_selection_scribe_items
			-- Add to selection to all items which are scribes
		do
			add_to_selection (scribe_filter)
		end
	
	add_to_selection_horizontal_scribe_items
			-- Added to selection to all items which are
			-- horizontal scribes
		do
			add_to_selection (horizontal_scribe_filter)
		end
	
	add_to_selection_vertical_scribe_items
			-- Add to selection to all items which are
			-- vertical scribes
		do
			add_to_selection (vertical_scribe_filter)
		end
	
	add_to_selection_floating_scribe_items
			-- Add to selection to all items which are
			-- floating scribes (either end not on edge of scribe
			-- of opposite orientation or field of view edge)
		do
			initialize_floating_scribe_filter
			add_to_selection (floating_scribe_filter)
		end
	
	add_to_selection_misaligned_scribe_items
			-- Add to selection to all items which are
			-- misaligned scribes (not quite lined up)
		do
			build_scribe_model_areas
			add_to_selection (misaligned_scribe_filter)
		end
	
	add_to_selection_default_width_scribe_items
			-- Add to selection to all items which are
			-- scribes with the default width
		do
			add_to_selection (default_width_scribe_filter)
		end
	
	add_to_selection_custom_width_scribe_items
			-- Add to selection to all items which are
			-- scribes with non-default width
		do
			add_to_selection (custom_width_scribe_filter)
		end
	
	add_to_selection_non_scribe_items
			-- Add to selection to all items which are not scribes
		do
			add_to_selection (non_scribe_filter)
		end
	
	add_to_selection_filler_items
			-- Add to selection all items which are fillers
		do
			add_to_selection (filler_filter)
		end
	
	add_to_selection_vendor_geometry_items
			-- Add to selection all items which are vendor
			-- geometries/ghosts
		do
			add_to_selection (vendor_geometry_filter)
		end
	
	add_to_selection_cut_items
			-- Add to selection all items which are cuts
		do
			add_to_selection (cut_filter)
		end
	
	add_to_selection_commercial_account_items
			-- Add to selection all items whose account
			-- type is commercial
		do
			add_to_selection (commercial_project_filter)
		end
	
	add_to_selection_mep_account_items
			-- Add to selection all items whose account
			-- type is MEP
		do
			add_to_selection (mep_project_filter)
		end
	
	add_to_selection_other_account_items
			-- Add to selection all items whose account
			-- type is other than commercial or MEP
		do
			add_to_selection (other_project_filter)
		end
	
	add_to_selection_partially_selected_items
			-- Add to selection all unselected copies of items
			-- that have any copies currently selected
		do
			add_to_selection (partially_selected_filter)
		end
	
	add_to_selection_outside_fields_items
			-- Add to selection all objects that are outside
			-- all fields of view
		do
			add_to_selection (outside_fields_filter)
		end
	
	add_to_selection_too_small_items
			-- Add to selection all objects whose size is below
			-- the minimum size in either dimension
		do
			add_to_selection (size_too_small_filter)
		end
	
	add_to_selection_small_scribes_and_fillers
			-- Add to selection all scribes and fillers whose
			-- size is below the small object threshold in either
			-- dimension (just length for scribes)
		do
			add_to_selection (small_scribe_and_filler_filter)
		end
	
	add_to_selection_overlapping_field_items
			-- Add to selection all objects which intersect
			-- at least one field of view by are not enclosed
			-- by any field
		do
			add_to_selection (partially_outside_fields_filter)
		end
	
	add_to_selection_overlapping_object_items
			-- Add to selection all objects which intersect
			-- at least one field of view and overlap another
			-- object
		do
			build_mbb_model_areas
			add_to_selection (overlaps_other_filter)
		end
	
	add_to_selection_close_edge_items
			-- Add to selection all objects which intersect
			-- at least one field of view and have at least one
			-- edge which is too close to field of view edge
		do
			add_to_selection (too_close_to_edge_filter)
		end
	
	add_to_selection_close_object_items
			-- Add to selection all objects which intersect
			-- at least one field of view and are too close to
			-- another object that intersects at least one field
		do
			build_mbb_model_areas
			add_to_selection (too_close_to_object_filter)
		end
	
	add_to_selection_too_big_items
			-- Add to selection all projects which have a die
			-- size that exceeds their maximum die size
		do
			compute_cut_dies
			add_to_selection (scribe_cut_die_too_big_filter)
		end
	
	add_to_selection_uncentered_items
			-- Add to selection all projects which are not
			-- centered in their cut die
		do
			compute_cut_dies
			add_to_selection (uncentered_in_cut_die_filter)
		end
	
	add_to_selection_unisolated_items
			-- Add to selection all projects which are not
			-- isolated in their cut die
		do
			compute_cut_dies
			build_scribe_cut_die_model_areas
			add_to_selection (unisolated_in_cut_die_filter)
		end
	
	add_to_selection_unassigned_lot_items
			-- Add to selection to all objects which have
			-- associated wafer etest ID(s) but have not
			-- been assigned a specific one
		do
			add_to_selection (wafer_etest_id_unassigned_filter)
		end
	
	add_to_selection_unlocked_or_unassigned
			-- Add to selection all projects which are not
			-- locked against customer update or are not assigned
			-- to run being closed
		do
			add_to_selection (unlocked_or_unassigned_filter)
		end
	
	remove_from_selection_cust_project_items
			-- Remove from selection all items which are
			-- customer projects
		do
			remove_from_selection (cust_project_filter)
		end
	
	remove_from_selection_test_vehicle_items
			-- Remove from selection all items which are
			-- test vehicles
		do
			remove_from_selection (test_vehicle_filter)
		end
	
	remove_from_selection_project_items
			-- Remove from selection all items which are projects
		do
			remove_from_selection (project_filter)
		end
	
	remove_from_selection_scribe_items
			-- Remove from selection all items which are scribes
		do
			remove_from_selection (scribe_filter)
		end
	
	remove_from_selection_horizontal_scribe_items
			-- Removed from selection all items which are
			-- horizontal scribes
		do
			remove_from_selection (horizontal_scribe_filter)
		end
	
	remove_from_selection_vertical_scribe_items
			-- Remove from selection all items which are
			-- vertical scribes
		do
			remove_from_selection (vertical_scribe_filter)
		end
	
	remove_from_selection_floating_scribe_items
			-- Remove from selection all items which are
			-- floating scribes (either end not on edge of scribe
			-- of opposite orientation or field of view edge)
		do
			initialize_floating_scribe_filter
			remove_from_selection (floating_scribe_filter)
		end
	
	remove_from_selection_misaligned_scribe_items
			-- Remove from selection all items which are
			-- misaligned scribes (not quite lined up)
		do
			build_scribe_model_areas
			remove_from_selection (misaligned_scribe_filter)
		end
	
	remove_from_selection_default_width_scribe_items
			-- Remove from selection all items which are
			-- scribes with the default width
		do
			remove_from_selection (default_width_scribe_filter)
		end
	
	remove_from_selection_custom_width_scribe_items
			-- Remove from selection all items which are
			-- scribes with non-default width
		do
			remove_from_selection (custom_width_scribe_filter)
		end
	
	remove_from_selection_non_scribe_items
			-- Remove from selection all items which are not
			-- scribes
		do
			remove_from_selection (non_scribe_filter)
		end
	
	remove_from_selection_filler_items
			-- Remove from selection all items which are fillers
		do
			remove_from_selection (filler_filter)
		end
	
	remove_from_selection_vendor_geometry_items
			-- Remove from selection all items which are vendor
			-- geometries/ghosts
		do
			remove_from_selection (vendor_geometry_filter)
		end
	
	remove_from_selection_cut_items
			-- Remove from selection all items which are cuts
		do
			remove_from_selection (cut_filter)
		end
	
	remove_from_selection_commercial_account_items
			-- Remove from selection all items whose account
			-- type is commercial
		do
			remove_from_selection (commercial_project_filter)
		end
	
	remove_from_selection_mep_account_items
			-- Remove from selection all items whose account
			-- type is MEP
		do
			remove_from_selection (mep_project_filter)
		end
	
	remove_from_selection_other_account_items
			-- Rmove from selection all items whose account
			-- type is other than commercial or MEP
		do
			remove_from_selection (other_project_filter)
		end
	
	remove_from_selection_partially_selected_items
			-- Remove from selection all items which do not
			-- have all copies currently selected
		do
			remove_from_selection (partially_selected_filter)
		end
	
	remove_from_selection_outside_fields_items
			-- Remove from selection all items which are outside
			-- all fields of view
		do
			remove_from_selection (outside_fields_filter)
		end
	
	remove_from_selection_too_small_items
			-- Remove from selection all objects whose
			-- size is below the minimum size in either
			-- dimension
		do
			remove_from_selection (size_too_small_filter)
		end
	
	remove_from_selection_small_scribes_and_fillers
			-- Remove from selection all scribes and fillers whose
			-- size is below the small object threshold in either
			-- dimension (just length for scribes)
		do
			remove_from_selection (small_scribe_and_filler_filter)
		end
	
	remove_from_selection_overlapping_field_items
			-- Remove from selection all objects which intersect
			-- at least one field of view by are not enclosed
			-- by any field
		do
			remove_from_selection (partially_outside_fields_filter)
		end
	
	remove_from_selection_overlapping_object_items
			-- Remove from selection all objects which intersect
			-- at least one field of view and overlap another
			-- object
		do
			build_mbb_model_areas
			remove_from_selection (overlaps_other_filter)
		end
	
	remove_from_selection_close_edge_items
			-- Remove from selection all objects which intersect
			-- at least one field of view and have at least one
			-- edge which is too close to field of view edge
		do
			remove_from_selection (too_close_to_edge_filter)
		end
	
	remove_from_selection_close_object_items
			-- Remove from selection all objects which intersect
			-- at least one field of view and are too close to
			-- another object that intersects at least one field
		do
			build_mbb_model_areas
			remove_from_selection (too_close_to_object_filter)
		end
	
	remove_from_selection_too_big_items
			-- Remove from selection all projects which have a die
			-- size that exceeds their maximum die size
		do
			compute_cut_dies
			remove_from_selection (scribe_cut_die_too_big_filter)
		end
	
	remove_from_selection_uncentered_items
			-- Remove from selection all projects which are not
			-- centered in their cut die
		do
			compute_cut_dies
			remove_from_selection (uncentered_in_cut_die_filter)
		end
	
	remove_from_selection_unisolated_items
			-- Remove from selection all projects which are not
			-- isolated in their cut die
		do
			compute_cut_dies
			build_scribe_cut_die_model_areas
			remove_from_selection (unisolated_in_cut_die_filter)
		end
	
	remove_from_selection_unassigned_lot_items
			-- Remove from selection all objects which
			-- have associated wafer etest ID(s) but have
			-- not been assigned a specific one
		do
			remove_from_selection (wafer_etest_id_unassigned_filter)
		end
	
	remove_from_selection_unlocked_or_unassigned
			-- Remove from selection all projects which are not
			-- locked against customer update or are not assigned
			-- to run being closed
		do
			remove_from_selection (unlocked_or_unassigned_filter)
		end
	
	set_selection_by_area (r: EV_RECTANGLE)
			-- Set selected items to selectable items that are
			-- completely contained inside `r'
		do
			inside_view_area_filter.set_rectangle (r)
			set_selection (inside_view_area_filter)
		end

	set_selection_by_intersection (r: EV_RECTANGLE)
			-- Set selected items to selectable items that
			-- intersect `r'
		do
			intersects_view_area_filter.set_rectangle (r)
			set_selection (intersects_view_area_filter)
		end

	set_selection_by_point (p: EV_COORDINATE)
			-- Set selected items to all selectable items that
			-- contain `p'
		do
			contains_view_point_filter.set_point (p)
			set_selection (contains_view_point_filter)
		end

	set_selection_all_copies_by_point (a_coordinate: EV_COORDINATE)
			-- Set selected items to all copies of every
			-- selectable item that contains `a_coordinate'
		require
			a_coordinate_not_void: a_coordinate /= Void
		do
			apply_action_with_filter (all_filter, all_copies_by_point_action)
			any_copy_contains_view_point_filter.set_point (a_coordinate)
			set_selection (any_copy_contains_view_point_filter)
		end

	add_to_selection_by_area (r: EV_RECTANGLE)
			-- Add to selection all selectable items that are
			-- completely contained inside `r'
		do
			inside_view_area_filter.set_rectangle (r)
			add_to_selection (inside_view_area_filter)
		end

	add_to_selection_by_intersection (r: EV_RECTANGLE)
			-- Add to selection all selectable items that
			-- intersect `r'
		do
			intersects_view_area_filter.set_rectangle (r)
			add_to_selection (intersects_view_area_filter)
		end

	add_to_selection_by_point (p: EV_COORDINATE)
			-- Add to selection all selectable items that
			-- contain `p'
		do
			contains_view_point_filter.set_point (p)
			add_to_selection (contains_view_point_filter)
		end

	add_to_selection_all_copies_by_point (a_coordinate: EV_COORDINATE)
			-- Add to selection all copies of every selectable
			-- item that contains `a_coordinate'
		require
			a_coordinate_not_void: a_coordinate /= Void
		do
			apply_action_with_filter (all_filter, all_copies_by_point_action)
			any_copy_contains_view_point_filter.set_point (a_coordinate)
			add_to_selection (any_copy_contains_view_point_filter)
		end

	remove_from_selection_by_area (r: EV_RECTANGLE)
			-- Remove from selection all selectable items that are
			-- completely contained inside `r'
		do
			inside_view_area_filter.set_rectangle (r)
			remove_from_selection (inside_view_area_filter)
		end

	remove_from_selection_by_intersection (r: EV_RECTANGLE)
			-- Remove from selection all selectable items that
			-- intersect `r'
		do
			intersects_view_area_filter.set_rectangle (r)
			remove_from_selection (intersects_view_area_filter)
		end

	remove_from_selection_by_point (p: EV_COORDINATE)
			-- Remove from selection all selectable
			-- items that contains `p'
		do
			contains_view_point_filter.set_point (p)
			remove_from_selection (contains_view_point_filter)
		end

	remove_from_selection_all_copies_by_point (a_coordinate: EV_COORDINATE)
			-- Remove from selection all copies of every
			-- selectable item any of whose copies contains `a_coordinate'
		require
			a_coordinate_not_void: a_coordinate /= Void
		do
			apply_action_with_filter (all_filter, all_copies_by_point_action)
			any_copy_contains_view_point_filter.set_point (a_coordinate)
			remove_from_selection (any_copy_contains_view_point_filter)
		end

	label_all_items_by_design_number
			-- Label all items with design number
		do
			set_label_with_filter_and_redraw (all_filter, design_number_label_generator)
		end

	label_selected_items_by_design_number
			-- Label selected items with design number
		do
			set_label_with_filter_and_redraw (selected_filter, design_number_label_generator)
		end

	label_unselected_items_by_design_number
			-- Label unselected items with design number
		do
			set_label_with_filter_and_redraw (unselected_filter, design_number_label_generator)
		end

	label_all_items_by_design_name
			-- Label all items with design name
		do
			set_label_with_filter_and_redraw (all_filter, design_name_label_generator)
		end

	label_selected_items_by_design_name
			-- Label selected items with design name
		do
			set_label_with_filter_and_redraw (selected_filter, design_name_label_generator)
		end

	label_unselected_items_by_design_name
			-- Label unselected items with design name
		do
			set_label_with_filter_and_redraw (unselected_filter, design_name_label_generator)
		end

	label_all_items_by_account_type
			-- Label all items with account type
		do
			set_label_with_filter_and_redraw (all_filter, account_type_label_generator)
		end

	label_selected_items_by_account_type
			-- Label selected items with account type
		do
			set_label_with_filter_and_redraw (selected_filter, account_type_label_generator)
		end

	label_unselected_items_by_account_type
			-- Label unselected items with account type
		do
			set_label_with_filter_and_redraw (unselected_filter, account_type_label_generator)
		end

	label_all_items_by_account_number
			-- Label all items with account number
		do
			set_label_with_filter_and_redraw (all_filter, account_number_label_generator)
		end

	label_selected_items_by_account_number
			-- Label selected items with account number
		do
			set_label_with_filter_and_redraw (selected_filter, account_number_label_generator)
		end

	label_unselected_items_by_account_number
			-- Label unselected items with account number
		do
			set_label_with_filter_and_redraw (unselected_filter, account_number_label_generator)
		end

	label_all_items_by_wafer_etest_id
			-- Label all items with lot split
		do
			set_label_with_filter_and_redraw (all_filter, wafer_etest_id_label_generator)
		end

	label_selected_items_by_wafer_etest_id
			-- Label selected items with lot split
		do
			set_label_with_filter_and_redraw (selected_filter, wafer_etest_id_label_generator)
		end

	label_unselected_items_by_wafer_etest_id
			-- Label unselected items with lot split
		do
			set_label_with_filter_and_redraw (unselected_filter, wafer_etest_id_label_generator)
		end

	label_all_items_by_org_short_id
			-- Label all items with organization short ID
		do
			set_label_with_filter_and_redraw (all_filter, org_short_id_label_generator)
		end

	label_selected_items_by_org_short_id
			-- Label selected items with organization short ID
		do
			set_label_with_filter_and_redraw (selected_filter, org_short_id_label_generator)
		end

	label_unselected_items_by_org_short_id
			-- Label unselected items with organization short ID
		do
			set_label_with_filter_and_redraw (unselected_filter, org_short_id_label_generator)
		end

	color_all_items_by_fixed_status
			-- Color all items by fixed in X and Y status
		do
			set_color_with_filter_and_redraw (all_filter, fixed_status_color_generator)
		end

	color_selected_items_by_fixed_status
			-- Color selected items by fixed in X and Y status
		do
			set_color_with_filter_and_redraw (selected_filter, fixed_status_color_generator)
		end

	color_unselected_items_by_fixed_status
			-- Color unselected items by fixed in X and Y status
		do
			set_color_with_filter_and_redraw (unselected_filter, fixed_status_color_generator)
		end

	color_all_items_by_locked_status
			-- Color all items by locked in position in X and Y
			-- status
		do
			set_color_with_filter_and_redraw (all_filter, locked_status_color_generator)
		end

	color_selected_items_by_locked_status
			-- Color selected items by locked in position
			-- in X and Y status
		do
			set_color_with_filter_and_redraw (selected_filter, locked_status_color_generator)
		end

	color_unselected_items_by_locked_status
			-- Color unselected items by locked in position
			-- in X and Y status
		do
			set_color_with_filter_and_redraw (unselected_filter, locked_status_color_generator)
		end

	color_all_items_by_account_type
			-- Color all items by account type
		do
			set_color_with_filter_and_redraw (all_filter, account_type_color_generator)
		end

	color_selected_items_by_account_type
			-- Color selected items by account type
		do
			set_color_with_filter_and_redraw (selected_filter, account_type_color_generator)
		end

	color_unselected_items_by_account_type
			-- Color unselected items by account type
		do
			set_color_with_filter_and_redraw (unselected_filter, account_type_color_generator)
		end

	fill_no_items
			-- Do not fill interior of any items
		do
			set_fill_with_filter_and_redraw (selectable_filter, none_fill_generator)
		end

	fill_selected_items
			-- Fill interior of selected items only
		do
			set_fill_with_filter_and_redraw (selectable_filter, selected_fill_generator)
		end

	fill_unselected_items
			-- Fill interior of unselected items only
		do
			set_fill_with_filter_and_redraw (selectable_filter, unselected_fill_generator)
		end

	fill_selected_and_unselected_items
			-- Fill interior of both selected and unselected items
		do
			set_fill_with_filter_and_redraw (selectable_filter, all_fill_generator)
		end

	fill_by_fixed_status
			-- Fill interior of items that are fixed in X or
			-- Y or both
		do
			set_fill_with_filter_and_redraw (selectable_filter, fixed_status_fill_generator)
		end

	fill_by_locked_status
			-- Fill interior of items that are locked in X or
			-- Y or both
		do
			set_fill_with_filter_and_redraw (selectable_filter, locked_status_fill_generator)
		end

	set_transparent_all_items
			-- Set all items to be transparent
		do
			set_opacity_with_filter_and_redraw (selectable_filter, transparent_opacity_generator)
		end

	set_opaque_all_items
			-- Set all items to be opaque
		do
			set_opacity_with_filter_and_redraw (selectable_filter, opaque_opacity_generator)
		end

	set_transparent_selected_items
			-- Set selected items to be transparent
		do
			set_opacity_with_filter_and_redraw (selected_filter, transparent_opacity_generator)
		end

	set_opaque_selected_items
			-- Set selected items to be opaque
		do
			set_opacity_with_filter_and_redraw (selected_filter, opaque_opacity_generator)
		end

	set_transparent_unselected_items
			-- Set unselected items to be transparent
		do
			set_opacity_with_filter_and_redraw (unselected_filter, transparent_opacity_generator)
		end

	set_opaque_unselected_items
			-- Set unselected items to be opaque
		do
			set_opacity_with_filter_and_redraw (unselected_filter, opaque_opacity_generator)
		end

	set_standard_view
			-- Switch to standard view where items are labeled
			-- by design number
		do
			if is_locked_mode then
				set_view_with_filter_and_redraw (all_filter, standard_locked_view)
			else
				set_view_with_filter_and_redraw (all_filter, standard_fixed_view)
			end
		end

	set_account_type_view
			-- Switch to Account Type view where items are labeled
			-- by account type
		do
			set_view_with_filter_and_redraw (all_filter, account_type_view)
		end

	set_lot_split_view
			-- Switch to Lot Split view where items are labeled
			-- by wafer etest ID(s)
		do
			set_view_with_filter_and_redraw (all_filter, lot_split_view)
		end

	set_packaging_view
			-- Switch to Packaging view where items are labeled
			-- by package name and packaging vendor type
		do
			set_view_with_filter_and_redraw (all_filter, packaging_view)
		end

	set_validity_check_view
			-- Switch to Validity Check view where items
			-- are colored by which errors/warnings they
			-- have
		do
			set_view_with_filter_and_redraw (all_filter, validity_check_view)
		end

	set_cut_view
			-- Switch to Cut view where items are colored by
			-- lot split and labeled by design number
		do
			set_view_with_filter_and_redraw (all_filter, cut_view)
		end

	set_overhead_style_view
			-- Switch to Overhead Style view where items are
			-- labeled by overhead style
		do
			set_view_with_filter_and_redraw (all_filter, overhead_style_view)
		end

	set_search_results_view
			-- Switch to Search Results view where items found
			-- by last search are displayed differently
		do
			set_view_with_filter_and_redraw (all_filter, search_results_view)
		end

	set_highlighted_selection_view
			-- Switch to Highlighted Selection view where items
			-- that are selected are displayed differently
		do
			set_view_with_filter_and_redraw (all_filter, highlighted_selection_view)
		end

	set_design_name_view
			-- Switch to Design Name view where items are labeled
			-- by design name
		do
			set_view_with_filter_and_redraw (all_filter, design_name_view)
		end

	set_die_id_view
			-- Switch to Die ID view where items are labeled
			-- by die ID
		do
			set_view_with_filter_and_redraw (all_filter, die_id_view)
		end

	set_account_number_view
			-- Switch to Account Number view where items are
			-- labeled by account number
		do
			set_view_with_filter_and_redraw (all_filter, account_number_view)
		end

	set_org_short_id_view
			-- Switch to Organization Short ID view where items
			-- are labeled by organization short ID
		do
			set_view_with_filter_and_redraw (all_filter, org_short_id_view)
		end

	set_max_die_size_view
			-- Switch to Max Die Size view where items
			-- are labeled by maximum die size
		do
			compute_cut_dies
			set_view_with_filter_and_redraw (all_filter, max_die_size_view)
		end

	set_rotation_locked_view
			-- Switch to Rotation Locked view where items
			-- that are rotation-locked are displayed differently
		do
			set_view_with_filter_and_redraw (all_filter, rotation_locked_view)
		end

	set_duplication_locked_view
			-- Switch to Duplication Locked view where
			-- items that are duplication-locked are
			-- displayed differently
		do
			set_view_with_filter_and_redraw (all_filter, duplication_locked_view)
		end

	set_deletion_locked_view
			-- Switch to Deltion Locked view where
			-- items that are deletion-locked are
			-- displayed differently
		do
			set_view_with_filter_and_redraw (all_filter, deletion_locked_view)
		end

	notify_split_colors_changed
			-- Lot split colors may have changed, so redraw
			-- view if Lot Split view is currently selected
		do
			if current_view = lot_split_view or current_view = cut_view then
				redraw
			end
		end

	apply_action (an_action: VIEW_ELEMENT_ACTION a_list: LIST [like anchored_view_element])
			-- Apply `an_action' to all elements of `a_list'
		require
			an_action_not_void: an_action /= Void
			a_list_not_void: a_list /= Void
		local
			e: like anchored_view_element
		do
			from
				a_list.start
			until
				a_list.after
			loop
				e := a_list.item
				an_action.apply (e)
				a_list.forth
			end
		end

	apply_action_with_filter (a_filter: VIEW_ELEMENT_FILTER an_action: VIEW_ELEMENT_ACTION)
			-- Apply `an_action' to all elements of view selected
			-- by `a_filter'
		require
			a_filter_not_void: a_filter /= Void
			an_action_not_void: an_action /= Void
		local
			e: like anchored_view_element
		do
			from
				elements.start
			until
				elements.after
			loop
				e := elements.item
				if a_filter.selects (e) then
					an_action.apply (e)
				end
				elements.forth
			end
		end

	do_nonselect_filtered_change (a_name: STRING a_filter: VIEW_ELEMENT_FILTER an_action: VIEW_ELEMENT_ACTION)
			-- Apply history list change operation
			-- `a_name', which does not change set of selected
			-- objects or current named view, to elements
			-- matching `a_filter',
			-- using `an_action' to do it
		require
			a_name_not_void: a_name /= Void
			a_filter_not_void: a_filter /= Void
			an_action_not_void: an_action /= Void
		do
			initialize_change
			apply_action_with_filter_and_redraw (a_filter, an_action)
			set_final_selected_elements_to_initial
			add_change_to_history_list (new_standard_change (a_name))
		end

	apply_select_action_and_redraw (an_action: VIEW_ELEMENT_ACTION)
			-- Apply `an_action' to all elements of view.
			-- Action must call
			-- {RETICLE_VIEW_ELEMENT}.`set_selected' on
			-- every element of view.  Then rebuild
			-- `elements' in the correct stacking order
			-- and redraw the changed region of the view
		require
			an_action_not_void: an_action /= Void
		do
			initialize_selection_lists
			set_changed_region_empty
			set_selection_bounding_box_empty
			apply_action_with_filter (all_filter, an_action)
			apply_action_with_filter (all_filter, selection_changed_action)
			rebuild_elements_in_stacking_order
			set_selected_count (previously_selected_elements.count + newly_selected_elements.count)
			redraw_changed_region
		end

	apply_select_action (an_action: VIEW_ELEMENT_ACTION)
			-- Apply `an_action' to all elements of view.
			-- Action must call
			-- {RETICLE_VIEW_ELEMENT}.`set_selected' on
			-- every element of view.  Then rebuild
			-- `elements' in the correct stacking order.
			-- Do not initialize changed region to empty
			-- and do not redraw the changed region
			-- (caller must do so)
		require
			an_action_not_void: an_action /= Void
		do
			initialize_selection_lists
			set_selection_bounding_box_empty
			apply_action_with_filter (all_filter, an_action)
			apply_action_with_filter (all_filter, selection_changed_action)
			rebuild_elements_in_stacking_order
			set_selected_count (previously_selected_elements.count + newly_selected_elements.count)
		end

	apply_action_with_filter_and_redraw (a_filter: VIEW_ELEMENT_FILTER an_action: VIEW_ELEMENT_ACTION)
			-- Apply `an_action' to all elements of view selected
			-- by `a_filter'.  Then redraw the view
		require
			a_filter_not_void: a_filter /= Void
			an_action_not_void: an_action /= Void
		do
			set_changed_region_empty
			apply_action_with_filter (a_filter, an_action)
			redraw_changed_region
		end

	set_view_with_filter (a_filter: VIEW_ELEMENT_FILTER a_named_view: RETICLE_NAMED_VIEW)
			-- Set appearance of all elements of view selected
			-- by `a_filter' as specified by `a_named_view'
		require
			a_filter_not_void: a_filter /= Void
			a_named_view_not_void: a_named_view /= Void
		do
			set_named_view_action (a_named_view)
			set_current_view (a_named_view)
			apply_action_with_filter (a_filter, set_view_action)
		end

	set_named_view_action (a_named_view: RETICLE_NAMED_VIEW)
			-- Initialize `set_view_action' from `a_named_view'
		require
			a_named_view_not_void: a_named_view /= Void
		do
			set_view_action.set_label_generator (a_named_view.label_generator)
			set_view_action.set_color_generator (a_named_view.color_generator)
			set_view_action.set_fill_generator (a_named_view.fill_generator)
			set_view_action.set_opacity_generator (a_named_view.opacity_generator)
		end

	set_view_with_filter_and_redraw (a_filter: VIEW_ELEMENT_FILTER a_named_view: RETICLE_NAMED_VIEW)
			-- Set appearance of all elements of view selected
			-- by `a_filter' as specified by `a_named_view'
		require
			a_filter_not_void: a_filter /= Void
			a_named_view_not_void: a_named_view /= Void
		do
			set_named_view_action (a_named_view)
			set_current_view (a_named_view)
			apply_action_with_filter_and_redraw (a_filter, set_view_action)
		end

	set_label_with_filter (a_filter: VIEW_ELEMENT_FILTER a_label_generator: RETICLE_VIEW_ELEMENT_LABEL_GENERATOR)
			-- Set label generator to `a_label_generator'
			-- for all elements of view selected by
			-- `a_filter'
		require
			a_filter_not_void: a_filter /= Void
			a_label_generator_not_void: a_label_generator /= Void
		do
			set_label_generator_action.set_label_generator (a_label_generator)
			apply_action_with_filter (a_filter, set_label_generator_action)
		end

	set_label_with_filter_and_redraw (a_filter: VIEW_ELEMENT_FILTER a_label_generator: RETICLE_VIEW_ELEMENT_LABEL_GENERATOR)
			-- Set label generator to `a_label_generator'
			-- for all elements of view selected by
			-- `a_filter'.  Then redraw the view
		require
			a_filter_not_void: a_filter /= Void
			a_label_generator_not_void: a_label_generator /= Void
		do
			set_label_generator_action.set_label_generator (a_label_generator)
			apply_action_with_filter_and_redraw (a_filter, set_label_generator_action)
		end

	set_color_with_filter (a_filter: VIEW_ELEMENT_FILTER a_color_generator: RETICLE_VIEW_ELEMENT_COLOR_GENERATOR)
			-- Set color generator to `a_color_generator'
			-- for all elements of view selected by
			-- `a_filter'
		require
			a_filter_not_void: a_filter /= Void
			a_color_generator_not_void: a_color_generator /= Void
		do
			set_color_generator_action.set_color_generator (a_color_generator)
			apply_action_with_filter (a_filter, set_color_generator_action)
		end

	set_color_with_filter_and_redraw (a_filter: VIEW_ELEMENT_FILTER a_color_generator: RETICLE_VIEW_ELEMENT_COLOR_GENERATOR)
			-- Set color generator to `a_color_generator'
			-- for all elements of view selected by
			-- `a_filter'.  Then redraw the view
		require
			a_filter_not_void: a_filter /= Void
			a_color_generator_not_void: a_color_generator /= Void
		do
			set_color_generator_action.set_color_generator (a_color_generator)
			apply_action_with_filter_and_redraw (a_filter, set_color_generator_action)
		end

	set_fill_with_filter (a_filter: VIEW_ELEMENT_FILTER a_fill_generator: RETICLE_VIEW_ELEMENT_FILL_GENERATOR)
			-- Set fill generator to `a_fill_generator'
			-- for all elements of view selected by
			-- `a_filter'
		require
			a_filter_not_void: a_filter /= Void
			a_fill_generator_not_void: a_fill_generator /= Void
		do
			set_fill_generator_action.set_fill_generator (a_fill_generator)
			apply_action_with_filter (a_filter, set_fill_generator_action)
		end

	set_fill_with_filter_and_redraw (a_filter: VIEW_ELEMENT_FILTER a_fill_generator: RETICLE_VIEW_ELEMENT_FILL_GENERATOR)
			-- Set fill generator to `a_fill_generator'
			-- for all elements of view selected by
			-- `a_filter'.  Then redraw the view
		require
			a_filter_not_void: a_filter /= Void
			a_fill_generator_not_void: a_fill_generator /= Void
		do
			set_fill_generator_action.set_fill_generator (a_fill_generator)
			apply_action_with_filter_and_redraw (a_filter, set_fill_generator_action)
		end

	set_opacity_with_filter (a_filter: VIEW_ELEMENT_FILTER an_opacity_generator: RETICLE_VIEW_ELEMENT_OPACITY_GENERATOR)
			-- Set opacity generator to `an_opacity_generator'
			-- for all elements of view selected by
			-- `a_filter'
		require
			a_filter_not_void: a_filter /= Void
			an_opacity_generator_not_void: an_opacity_generator /= Void
		do
			set_opacity_generator_action.set_opacity_generator (an_opacity_generator)
			apply_action_with_filter (a_filter, set_opacity_generator_action)
		end

	set_opacity_with_filter_and_redraw (a_filter: VIEW_ELEMENT_FILTER an_opacity_generator: RETICLE_VIEW_ELEMENT_OPACITY_GENERATOR)
			-- Set opacity generator to `an_opacity_generator'
			-- for all elements of view selected by
			-- `a_filter'.  Then redraw the view
		require
			a_filter_not_void: a_filter /= Void
			an_opacity_generator_not_void: an_opacity_generator /= Void
		do
			set_opacity_generator_action.set_opacity_generator (an_opacity_generator)
			apply_action_with_filter_and_redraw (a_filter, set_opacity_generator_action)
		end

	switch_color_with_filter (a_filter: VIEW_ELEMENT_FILTER an_old_generator, a_new_generator: RETICLE_VIEW_ELEMENT_COLOR_GENERATOR)
			-- Switch color generator from
			-- `old_color_generator' to
			-- `new_color_generator' for all elements of
			-- view selected by `a_filter'
		require
			a_filter_not_void: a_filter /= Void
			an_old_generator_not_void: an_old_generator /= Void
			a_new_generator_not_void: a_new_generator /= Void
		do
			switch_color_generator_action.set_old_color_generator (an_old_generator)
			switch_color_generator_action.set_new_color_generator (a_new_generator)
			apply_action_with_filter (a_filter, switch_color_generator_action)
		end

	switch_fill_with_filter (a_filter: VIEW_ELEMENT_FILTER an_old_generator, a_new_generator: RETICLE_VIEW_ELEMENT_FILL_GENERATOR)
			-- Switch fill generator from
			-- `old_fill_generator' to
			-- `new_fill_generator' for all elements of
			-- view selected by `a_filter'
		require
			a_filter_not_void: a_filter /= Void
			an_old_generator_not_void: an_old_generator /= Void
			a_new_generator_not_void: a_new_generator /= Void
		do
			switch_fill_generator_action.set_old_fill_generator (an_old_generator)
			switch_fill_generator_action.set_new_fill_generator (a_new_generator)
			apply_action_with_filter (a_filter, switch_fill_generator_action)
		end

	fix_selected_items_in_x
			-- Mark the currently selected items as fixed in
			-- the X dimension, for horizontal scanning purposes
		do
			do_nonselect_filtered_change ("Fix in X", selected_filter, fix_in_x_action)
		end

	fix_selected_items_in_y
			-- Mark the currently selected items as fixed in
			-- the Y dimension, for vertical scanning purposes
		do
			do_nonselect_filtered_change ("Fix in Y", selected_filter, fix_in_y_action)
		end

	fix_selected_items_in_x_and_y
			-- Mark the currently selected items as fixed
			-- in the X and Y dimension, for horizontal
			-- and vertical scanning purposes
		do
			do_nonselect_filtered_change ("Fix in X and Y", selected_filter, fix_in_x_and_y_action)
		end

	unfix_selected_items_in_x
			-- Mark the currently selected items as not fixed in
			-- the X dimension, for horizontal scanning purposes
		do
			do_nonselect_filtered_change ("Unfix in X", selected_filter, unfix_in_x_action)
		end

	unfix_selected_items_in_y
			-- Mark the currently selected items as not fixed in
			-- the Y dimension, for vertical scanning purposes
		do
			do_nonselect_filtered_change ("Unfix in Y", selected_filter, unfix_in_y_action)
		end

	unfix_selected_items_in_x_and_y
			-- Mark the currently selected items as not fixed in
			-- the X or Y dimension, for horizontal and vertical
			-- scanning purposes
		do
			do_nonselect_filtered_change ("Unfix in X and Y", selected_filter, unfix_in_x_and_y_action)
		end

	lock_all_items_in_x
			-- Mark all selectable items as locked in position
			-- in the X dimension
		do
			do_nonselect_filtered_change ("Lock in X", selectable_filter, lock_in_x_action)
		end

	lock_all_items_in_y
			-- Mark all selectable items as locked in position
			-- in the Y dimension
		do
			do_nonselect_filtered_change ("Lock in Y", selectable_filter, lock_in_y_action)
		end

	lock_all_items_in_x_and_y
			-- Mark all selectable items as locked in position
			-- in the X and Y dimensions
		do
			do_nonselect_filtered_change ("Lock in X and Y", selectable_filter, lock_in_x_and_y_action)
		end

	lock_all_items_orientation
			-- Mark all selectable items as locked in current
			-- orientation
		do
			do_nonselect_filtered_change ("Lock Orientation", selectable_filter, lock_orientation_action)
		end

	lock_all_items_to_prevent_duplication
			-- Mark all selectable items as locked to prevent
			-- duplication
		do
			do_nonselect_filtered_change ("Lock Duplication", selectable_filter, lock_duplication_action)
		end

	lock_all_items_to_prevent_deletion
			-- Mark all selectable items as locked to prevent
			-- deletion
		do
			do_nonselect_filtered_change ("Lock Deletion", selectable_filter, lock_deletion_action)
		end

	lock_all_items_almost_completely
			-- Mark all selectable items as almost completely
			-- locked (everything except duplication)
		do
			do_nonselect_filtered_change ("Lock Almost Completely", selectable_filter, lock_almost_completely_action)
		end

	lock_all_items_completely
			-- Mark all selectable items as completely locked
		do
			do_nonselect_filtered_change ("Lock Completely", selectable_filter, lock_completely_action)
		end

	lock_selected_items_in_x
			-- Mark selected items as locked in position
			-- in the X dimension
		do
			do_nonselect_filtered_change ("Lock in X", selected_filter, lock_in_x_action)
		end

	lock_selected_items_in_y
			-- Mark selected items as locked in position
			-- the Y dimension
		do
			do_nonselect_filtered_change ("Lock in Y", selected_filter, lock_in_y_action)
		end

	lock_selected_items_in_x_and_y
			-- Mark selected items as locked in position
			-- in the X and Y dimensions
		do
			do_nonselect_filtered_change ("Lock in X and Y", selected_filter, lock_in_x_and_y_action)
		end

	lock_selected_items_orientation
			-- Mark selected items as locked in current
			-- orientation
		do
			do_nonselect_filtered_change ("Lock Orientation", selected_filter, lock_orientation_action)
		end

	lock_selected_items_to_prevent_duplication
			-- Mark selected items as locked to prevent
			-- duplication
		do
			do_nonselect_filtered_change ("Lock Duplication", selected_filter, lock_duplication_action)
		end

	lock_selected_items_to_prevent_deletion
			-- Mark selected items as locked to prevent
			-- deletion
		do
			do_nonselect_filtered_change ("Lock Deletion", selected_filter, lock_deletion_action)
		end

	lock_selected_items_almost_completely
			-- Mark selected items as locked almost completely
			-- (everything but duplication)
		do
			do_nonselect_filtered_change ("Lock Almost Completely", selected_filter, lock_almost_completely_action)
		end

	lock_selected_items_completely
			-- Mark selected items as locked completely
		do
			do_nonselect_filtered_change ("Lock Completely", selected_filter, lock_completely_action)
		end

	lock_unselected_items_in_x
			-- Mark unselected items as locked in position
			-- in the X dimension
		do
			do_nonselect_filtered_change ("Lock in X", unselected_filter, lock_in_x_action)
		end

	lock_unselected_items_in_y
			-- Mark unselected items as locked in position
			-- the Y dimension
		do
			do_nonselect_filtered_change ("Lock in Y", unselected_filter, lock_in_y_action)
		end

	lock_unselected_items_in_x_and_y
			-- Mark unselected items as locked in position
			-- in the X and Y dimensions
		do
			do_nonselect_filtered_change ("Lock in X and Y", unselected_filter, lock_in_x_and_y_action)
		end

	lock_unselected_items_orientation
			-- Mark unselected items as locked in current
			-- orientation
		do
			do_nonselect_filtered_change ("Lock Orientation", unselected_filter, lock_orientation_action)
		end

	lock_unselected_items_to_prevent_duplication
			-- Mark unselected items as locked to prevent
			-- duplication
		do
			do_nonselect_filtered_change ("Lock Duplication", unselected_filter, lock_duplication_action)
		end

	lock_unselected_items_to_prevent_deletion
			-- Mark unselected items as locked to prevent
			-- deletion
		do
			do_nonselect_filtered_change ("Lock Deletion", unselected_filter, lock_deletion_action)
		end

	lock_unselected_items_almost_completely
			-- Mark unselected items as locked almost completely
			-- (everything but duplication)
		do
			do_nonselect_filtered_change ("Lock Almost Completely", unselected_filter, lock_almost_completely_action)
		end

	lock_unselected_items_completely
			-- Mark unselected items as locked completely
		do
			do_nonselect_filtered_change ("Lock Completely", unselected_filter, lock_completely_action)
		end

	unlock_all_items_in_x
			-- Mark all non-stationary items as unlocked
			-- in position in the X dimension
		do
			do_nonselect_filtered_change ("Unlock in X", selectable_filter, unlock_in_x_action)
		end

	unlock_all_items_in_y
			-- Mark all non-stationary items as unlocked
			-- in position the Y dimension
		do
			do_nonselect_filtered_change ("Unlock in Y", selectable_filter, unlock_in_y_action)
		end

	unlock_all_items_in_x_and_y
			-- Mark all non-stationary items as unlocked
			-- in position in the X and Y dimensions
		do
			do_nonselect_filtered_change ("Unlock in X and Y", selectable_filter, unlock_in_x_and_y_action)
		end

	unlock_all_items_orientation
			-- Mark all items whose orientation is not stationary
			-- as unlocked in orientation
		do
			do_nonselect_filtered_change ("Unlock Orientation", selectable_filter, unlock_orientation_action)
		end

	unlock_all_items_to_allow_duplication
			-- Mark all items as unlocked to allow duplication
		do
			do_nonselect_filtered_change ("Unlock Duplication", selectable_filter, unlock_duplication_action)
		end

	unlock_all_items_to_allow_deletion
			-- Mark all items as unlocked to allow deletion
		do
			do_nonselect_filtered_change ("Unlock Deletion", selectable_filter, unlock_deletion_action)
		end

	unlock_all_items_almost_completely
			-- Mark all items as unlocked almost completely
			-- (everything but duplication)
		do
			do_nonselect_filtered_change ("Unlock Almost Completely", selectable_filter, unlock_almost_completely_action)
		end

	unlock_all_items_completely
			-- Mark all items as unlocked completely
		do
			do_nonselect_filtered_change ("Unlock Completely", selectable_filter, unlock_completely_action)
		end

	unlock_selected_items_in_x
			-- Mark selected non-stationary items as
			-- unlocked in position in the X dimension
		do
			do_nonselect_filtered_change ("Unlock in X", selected_filter, unlock_in_x_action)
		end

	unlock_selected_items_in_y
			-- Mark selected non-stationary items as
			-- unlocked in position the Y dimension
		do
			do_nonselect_filtered_change ("Unlock in Y", selected_filter, unlock_in_y_action)
		end

	unlock_selected_items_in_x_and_y
			-- Mark selected non-stationary items as
			-- unlocked in position in the X and Y
			-- dimensions
		do
			do_nonselect_filtered_change ("Unlock in X and Y", selected_filter, unlock_in_x_and_y_action)
		end

	unlock_selected_items_orientation
			-- Mark selected items whose orientation is
			-- not stationary as unlocked in orientation
		do
			do_nonselect_filtered_change ("Unlock Orientation", selected_filter, unlock_orientation_action)
		end

	unlock_selected_items_to_allow_duplication
			-- Mark selected items as unlocked to allow
			-- duplication
		do
			do_nonselect_filtered_change ("Unlock Duplication", selected_filter, unlock_duplication_action)
		end

	unlock_selected_items_to_allow_deletion
			-- Mark selected items as unlocked to allow deletion
		do
			do_nonselect_filtered_change ("Unlock Deletion", selected_filter, unlock_deletion_action)
		end

	unlock_selected_items_almost_completely
			-- Mark selected items as unlocked almost completely
			-- (everything but duplication)
		do
			do_nonselect_filtered_change ("Unlock Almost Completely", selected_filter, unlock_almost_completely_action)
		end

	unlock_selected_items_completely
			-- Mark selected items as unlocked completely
		do
			do_nonselect_filtered_change ("Unlock Completely", selected_filter, unlock_completely_action)
		end

	unlock_unselected_items_in_x
			-- Mark unselected non-stationary items as
			-- unlocked in position in the X dimension
		do
			do_nonselect_filtered_change ("Unlock in X", unselected_filter, unlock_in_x_action)
		end

	unlock_unselected_items_in_y
			-- Mark unselected non-stationary items as
			-- unlocked in position the Y dimension
		do
			do_nonselect_filtered_change ("Unlock in Y", unselected_filter, unlock_in_y_action)
		end

	unlock_unselected_items_in_x_and_y
			-- Mark unselected non-stationary items as
			-- unlocked in position in the X and Y
			-- dimensions
		do
			do_nonselect_filtered_change ("Unlock in X and Y", unselected_filter, unlock_in_x_and_y_action)
		end

	unlock_unselected_items_orientation
			-- Mark unselected items whose orientation is
			-- not stationary as unlocked in orientation
		do
			do_nonselect_filtered_change ("Unlock Orientation", unselected_filter, unlock_orientation_action)
		end

	unlock_unselected_items_to_allow_duplication
			-- Mark unselected items as unlocked to allow
			-- duplication
		do
			do_nonselect_filtered_change ("Unlock Duplication", unselected_filter, unlock_duplication_action)
		end

	unlock_unselected_items_to_allow_deletion
			-- Mark unselected items as unlocked to allow deletion
		do
			do_nonselect_filtered_change ("Unlock Deletion", unselected_filter, unlock_deletion_action)
		end

	unlock_unselected_items_almost_completely
			-- Mark unselected items as unlocked almost completely
			-- (everything but duplication)
		do
			do_nonselect_filtered_change ("Unlock Almost Completely", unselected_filter, unlock_almost_completely_action)
		end

	unlock_unselected_items_completely
			-- Mark unselected items as unlocked completely
		do
			do_nonselect_filtered_change ("Unlock Completely", unselected_filter, unlock_completely_action)
		end

	convert_fixed_items_to_locked
			-- Change any items that are fixed in X to instead be
			-- locked in X and similarly for the Y dimension
		do
			initialize_change
			set_changed_region_empty
			switch_color_with_filter (all_filter, fixed_status_color_generator, locked_status_color_generator)
			switch_fill_with_filter (all_filter, fixed_status_fill_generator, locked_status_fill_generator)
			apply_action_with_filter (selectable_filter, convert_fixed_to_locked_action)
			redraw_changed_region
			if current_view = standard_fixed_view then
				set_current_view (standard_locked_view)
			end
			is_locked_mode := True
			set_final_selected_elements_to_initial
			add_change_to_history_list (new_standard_change ("Convert Fixed to Locked"))
		end

	convert_locked_items_to_fixed
			-- Change any items that are locked in X to instead be
			-- fixed in X and similarly for the Y dimension
		do
			initialize_change
			set_changed_region_empty
			switch_color_with_filter (all_filter, locked_status_color_generator, fixed_status_color_generator)
			switch_fill_with_filter (all_filter, locked_status_fill_generator, fixed_status_fill_generator)
			apply_action_with_filter (selectable_filter, convert_locked_to_fixed_action)
			redraw_changed_region
			if current_view = standard_locked_view then
				set_current_view (standard_fixed_view)
			end
			is_locked_mode := False
			set_final_selected_elements_to_initial
			add_change_to_history_list (new_standard_change ("Convert Locked to Fixed"))
		end

	measure_selected_items
			-- Compute sum of widths and sum of heights of
			-- selected objects
		do
			measure_sum_action.initialize
			measure_sum_action.set_grid (grid)
			apply_action_with_filter (selected_filter, measure_sum_action)
			measured_sum_x := measure_sum_action.total_x
			measured_sum_y := measure_sum_action.total_y
		end

	measure_distance_at_view_coords (a_x, a_y: INTEGER)
			-- Measure distance from last point to
			-- edge (or corner if close enough to a corner)
			-- closest to (`a_x', `a_y') that is currently visible
		local
			p: POINT
		do
			initialize_status
			compute_measure_point_action.set_is_first_pass (True)
			compute_measure_point_action.set_view_coordinates (a_x, a_y)
			compute_measure_point_action.set_limits (width - 1, height - 1)
			compute_measure_point_action.set_corner_threshold (corner_snap_limit_in_pixels)
			compute_measure_point_action.initialize
			apply_action_with_filter (non_field_label_filter, compute_measure_point_action)
			if compute_measure_point_action.found_visible_edge then
				-- Compute best point along closest edge
				compute_measure_point_action.set_is_first_pass (False)
				compute_measure_point_action.initialize
				apply_action_with_filter (non_field_label_filter, compute_measure_point_action)
				compute_measure_point_action.finish
				p := compute_measure_point_action.measure_point
				create p.make_xy (grid.rounded_edge (p.x), grid.rounded_edge (p.y))
				redraw_canvas
				if first_measure_point = Void and second_measure_point = Void then
					second_measure_point := p
					draw_std_measurement_marker (second_measure_point)
					add_error ("Cannot compute distance because no previous point is available")
				else
					first_measure_point := second_measure_point
					second_measure_point := p
					draw_std_measurement_marker (first_measure_point)
					draw_std_measurement_marker (second_measure_point)
				end
			else
				add_error ("Cannot set distance measurement point because there are no visible object edges")
			end
		end

	toggle_rotation_of_selected_items
			-- Toggle rotation of all currently selected
			-- items between unrotated (0 degrees) and
			-- rotated (rotated 90 degrees clockwise)
		do
			initialize_change
			initialize_rotate_element_lists
			set_selection_bounding_box_empty
			apply_action_with_filter (all_filter, pre_rotate_action)
			apply_action_with_filter_and_redraw (selected_filter, toggle_rotation_action)
			set_final_selected_elements_to_initial
			add_change_to_history_list (new_standard_change ("Rotate"))
		end

	rotate_selected_items_90_clockwise
			-- Rotate all currently selected items 90
			-- degrees clockwise
		do
			initialize_change
			initialize_rotate_element_lists
			set_selection_bounding_box_empty
			apply_action_with_filter (all_filter, pre_rotate_action)
			apply_action_with_filter_and_redraw (selected_filter, rotate_clockwise_action)
			set_final_selected_elements_to_initial
			add_change_to_history_list (new_standard_change ("Rotate Clockwise"))
		end

	search_for_design_numbers (a_list: LIST [INTEGER] a_select_also: BOOLEAN)
			-- Search element list for elemnts that have design
			-- numbers in `a_list'.  Mark them as "found"
			-- so that they will be displayed differently in
			-- Search Results view.  If `a_select_also',
			-- then select these projects too
		do
			initialize_status
			create {ARRAYED_LIST [like anchored_view_element]} found_elements.make (20)
			search_design_number_action.set_design_number_list (a_list)
			search_design_number_action.set_select_matching_elements (a_select_also)
			set_view_action.set_label_generator (design_number_label_generator)
			set_view_action.set_color_generator (search_results_color_generator)
			set_view_action.set_fill_generator (search_results_fill_generator)
			set_view_action.set_opacity_generator (transparent_opacity_generator)
			if a_select_also then
				apply_select_action_and_redraw (search_design_number_action)
			else
				apply_action_with_filter_and_redraw (project_filter, search_design_number_action)
			end
		end

	add_to_changed_region (a_rectangle: EV_RECTANGLE)
			-- Union `a_rectangle' into `changed_region'
		local
			a_new_region: EV_REGION
			rect: EV_RECTANGLE
		do
			if a_rectangle.intersects (pixmap_rectangle) then
				rect := a_rectangle.twin
				intersect (rect, pixmap_rectangle)
				create a_new_region.make_with_rectangle (rect)
				if changed_region = Void then
					changed_region := a_new_region
					changed_bounding_box := a_rectangle.twin
				else
					changed_region := changed_region.union (a_new_region)
					changed_bounding_box.merge (a_rectangle)
				end
				changed_region_count := changed_region_count + 1
			end
		end

	set_cut_mode
			-- Switch to Cut mode
		local
			n, l_count: INTEGER
		do
			set_validity_color_generator (no_validity_color_generator)
					-- Don't display anything as having
					-- validity errors/warnings in Validity
					-- view
			pack_mode_context := current_context_copy
			pack_mode_context.set_view (final_pack_mode_view)
			set_current_view (cut_view)
			build_no_cut_patterns
			create {ARRAYED_LIST [RETICLE_VIEW_CUT_PATTERN]} cut_patterns.make (20)
			from
				l_count := model.cut_pattern_count
				n := 1
			until
				n > l_count
			loop
				build_cut_pattern (n)
				cut_patterns.extend (last_cut_pattern)
				n := n + 1
			end
			if not cut_patterns.is_empty then
				-- Move cursor to last cut pattern, which is
				-- the one that will be displayed
				cut_patterns.go_i_th (cut_patterns.count)
			end
			set_view_with_filter (all_filter, cut_view)
			show_entire_model
		end

	switch_to_pack_mode
			-- Switch to Pack mode
		do
			if pack_mode_context /= Void then
				remove_cut_patterns
				set_context (pack_mode_context)
				pack_mode_context := Void
				create_copies_table
				set_all_copies
					-- Reset list of all copies for die
					-- elements
				set_validity_color_generator (no_validity_color_generator)
					-- Don't display anything as having
					-- validity errors/warnings in Validity
					-- view
				redraw
			end
		end

	set_pack_mode
			-- Set Pack mode after a new version is read in
		do
			pack_mode_context := Void
		end

	set_final_pack_mode_view
			-- Set `final_pack_mode_view' to current view
			-- when about to enter Cut mode
		do
			final_pack_mode_view := current_view
		end

	add_cut_pattern (a_pattern_no: INTEGER a_field_name, a_wafer_etest_name: STRING quant: INTEGER has_quant: BOOLEAN)
			-- Add a new cut pattern with `a_pattern_no'
			-- for field `a_field_name' for lot identified
			-- by `a_wafer_etest_name'.  Position in
			-- `cut_patterns' is determined by `a_pattern_no'
		require
			is_cut_mode: is_cut_mode
			a_field_name_not_void: a_field_name /= Void
			a_field_name_not_empty: not a_field_name.is_empty
			a_wafer_etest_name_not_void: a_wafer_etest_name /= Void
		do
			set_validity_color_generator (no_validity_color_generator)
			if has_current_cut_pattern then
				update_current_cut_pattern_context
			end
			create displayed_model_area.make_by_corners (0, 0, 0, 0)
			set_current_view (cut_view)

			model.add_cut_pattern (a_pattern_no, a_field_name, a_wafer_etest_name, quant, has_quant)
			build_cut_pattern (model.last_cut_pattern_position)
			insert_cut_pattern (last_cut_pattern)
			set_view_with_filter (all_filter, cut_view)
			show_entire_model
		end

	set_cut_pattern_properties (a_pattern_no: INTEGER a_field_name, a_wafer_etest_name: STRING quant: INTEGER has_quant: BOOLEAN)
			-- Set properties of current cut pattern
		require
			is_cut_mode: is_cut_mode
			a_field_name_not_void: a_field_name /= Void
			a_field_name_not_empty: not a_field_name.is_empty
			a_wafer_etest_name_not_void: a_wafer_etest_name /= Void
		local
			pat: like current_cut_pattern
			old_etest_name, new_etest_name: STRING
			old_quant, new_quant: INTEGER
			old_has_quant, new_has_quant: BOOLEAN
		do
			initialize_change
			pat := current_cut_pattern
			
			old_etest_name := pat.wafer_etest_name
			old_quant := pat.quantity_wafers_needed
			old_has_quant := pat.has_quantity_wafers_needed
			new_etest_name := a_wafer_etest_name
			new_quant := quant
			new_has_quant := has_quant
			
			pat.set_wafer_etest_name (new_etest_name)
			pat.set_quantity_wafers_needed (new_quant, has_quant)
			
			add_change (create {CUT_PATTERN_PROPERTY_CHANGE}.make (Current, old_etest_name, old_quant, old_has_quant, new_etest_name, new_quant, new_has_quant))
			set_final_selected_elements_to_initial
			add_change_to_history_list (new_standard_change ("Set Cut Pattern Properties"))
			redraw_cut_pattern_label
		end

	show_previous_cut_pattern
			-- Display previous cut pattern
		require
			is_cut_mode: is_cut_mode
			not_at_first_pattern: not at_first_cut_pattern
		do
			set_validity_color_generator (no_validity_color_generator)
			update_current_cut_pattern_context
			cut_patterns.back
			show_current_cut_pattern
		end

	show_next_cut_pattern
			-- Display next cut pattern
		require
			is_cut_mode: is_cut_mode
			not_at_last_pattern: not at_last_cut_pattern
		do
			set_validity_color_generator (no_validity_color_generator)
			update_current_cut_pattern_context
			cut_patterns.forth
			show_current_cut_pattern
		end

	delete_current_cut_pattern
			-- Delete the current cut pattern
		require
			is_cut_mode: is_cut_mode
			has_current_cut_pattern: has_current_cut_pattern
		do
			delete_current_cut_pattern_without_redraw
			if has_current_cut_pattern then
				show_current_cut_pattern
			else
				show_no_cut_patterns
			end
		end

	modify_current_cut_pattern
			-- Change status of current cut pattern back to
			-- Under Construction so that it can be modified
		require
			is_cut_mode: is_cut_mode
			has_current_cut_pattern: has_current_cut_pattern
		local
			old_completed, new_completed: BOOLEAN
		do
			initialize_change
			old_completed := current_cut_pattern.is_completed
			current_cut_pattern.set_is_completed (False)
			new_completed := current_cut_pattern.is_completed
			add_change (create {CUT_PATTERN_STATUS_CHANGE}.make (Current, old_completed, new_completed))
			set_final_selected_elements_to_initial
			add_change_to_history_list (new_standard_change ("Modify Cut Pattern"))
			redraw_cut_pattern_label
		end

	complete_current_cut_pattern
			-- Change status of current cut pattern to Completed
			-- after doing validity checks
		require
			is_cut_mode: is_cut_mode
			has_current_cut_pattern: has_current_cut_pattern
		local
			old_completed, new_completed: BOOLEAN
		do
			initialize_change
			old_completed := current_cut_pattern.is_completed
			current_cut_pattern.set_is_completed (True)
			new_completed := current_cut_pattern.is_completed
			add_change (create {CUT_PATTERN_STATUS_CHANGE}.make (Current, old_completed, new_completed))
			set_final_selected_elements_to_initial
			add_change_to_history_list (new_standard_change ("Complete Cut Pattern"))
			redraw_cut_pattern_label
		end

	switch_to_next_field
			-- Switch to the next field of view (if any)
			-- for current cut pattern
		require
			is_cut_mode: is_cut_mode
			has_current_cut_pattern: has_current_cut_pattern
		local
			pat: like current_cut_pattern
			pat_no, waf_count: INTEGER
			has_waf_count: BOOLEAN
			fld_name, new_field_name, etest_name: STRING
		do
			-- Save info about current cut pattern

			pat := current_cut_pattern
			pat_no := pat.cut_pattern_no
			fld_name := pat.field_name
			etest_name := pat.wafer_etest_name
			has_waf_count := pat.has_quantity_wafers_needed
			waf_count := pat.quantity_wafers_needed
			
			-- Delete current cut pattern
			
			delete_current_cut_pattern_without_redraw
		
			-- Ensure that there is no current cut pattern
		
			cut_patterns.finish
			if not cut_patterns.after then
				cut_patterns.forth
			end
	
			-- Find name of next field in sequence (wrap if needed)
			
			new_field_name := next_field_name (fld_name)

			-- Add new cut pattern with same cut pattern number
			-- and wafer etest name but new field name

			add_cut_pattern (pat_no, new_field_name, etest_name, waf_count, has_waf_count)
		end

	synthesize_all_cuts
			-- Synthesize one cut through each distinct horizontal
			-- and vertical scribe center in current cut pattern
			-- that does not already have a cut at that position
		require
			is_cut_mode: is_cut_mode
			has_current_cut_pattern: has_current_cut_pattern
		local
			l_field: like anchored_view_element
			half_width: DOUBLE
		do
			initialize_change
			new_elements.wipe_out
			initialize_cut_counts
			set_changed_region_empty
			apply_action_with_filter (scribe_filter, create_cut_action)
			l_field := elements.first.field
			half_width := default_scribe_width / 2.0
			create_cut (True, l_field.field_relative_bottom_y - half_width)
			create_cut (True, l_field.field_relative_top_y + half_width)
			create_cut (False, l_field.field_relative_left_x - half_width)
			create_cut (False, l_field.field_relative_right_x + half_width)
			elements.append (new_elements)
			show_entire_model
			set_final_selected_elements_to_initial
			add_change_to_history_list (new_cut_creation_change ("Synthesize All Cuts"))
		end

	add_cuts_by_point (a_coordinate: EV_COORDINATE)
			-- Add a cut through the center of every
			-- scribe that contains `a_coordinate' or through
			-- appropriate implicit scribe(s) outside field
			-- of view edge if `a_coordinate' outside field
		require
			is_cut_mode: is_cut_mode
			has_current_cut_pattern: has_current_cut_pattern
		local
			l_field: like anchored_view_element
			l_scribes: LIST [like anchored_view_element]
			half_width: DOUBLE
			field_mbb: EV_RECTANGLE
		do
			initialize_change
			new_elements.wipe_out
			initialize_cut_counts
			set_changed_region_empty
			l_field := elements.first.field
			half_width := default_scribe_width / 2.0
			field_mbb := l_field.bounding_box
			if field_mbb.has (a_coordinate) then
				create {ARRAYED_LIST [like anchored_view_element]} l_scribes.make (2)
				pre_add_cuts_action.set_selection_point (a_coordinate)
				pre_add_cuts_action.set_containing_elements (l_scribes)
				apply_action_with_filter (scribe_filter, pre_add_cuts_action)
				if not l_scribes.is_empty then
					apply_action (create_cut_action, l_scribes)
				else
					add_error ("Add Cuts failed - mouse pointer must be inside a scribe or outside the field")
				end
			else
				if a_coordinate.x < field_mbb.left then
					create_cut (False, l_field.field_relative_left_x - half_width)
				end
				if a_coordinate.x > field_mbb.right then
					create_cut (False, l_field.field_relative_right_x + half_width)
				end
				if a_coordinate.y > field_mbb.bottom then
					create_cut (True, l_field.field_relative_bottom_y - half_width)
				end
				if a_coordinate.y < field_mbb.top then
					create_cut (True, l_field.field_relative_top_y + half_width)
				end
			end
			elements.append (new_elements)
			redraw_changed_region
			set_final_selected_elements_to_initial
			add_change_to_history_list (new_cut_creation_change ("Add Cuts"))
		end

feature {RETICLE_VIEW_ELEMENT} -- Implementation

	unselectable_elements: LIST [like anchored_view_element]
			-- List of items that cannot be selected
	
	unselected_elements: LIST [like anchored_view_element]
			-- List of items that are not selected but can be
			-- selected

	previously_selected_elements: LIST [like anchored_view_element]
			-- List of items that are selected but which
			-- did not match the current filter that determines
			-- which items have their selection status changed

	newly_selected_elements: LIST [like anchored_view_element]
			-- List of items that are selected and which
			-- matched the current filter that determines
			-- which items have their selection status changed

	add_change (a_change: RETICLE_SIMPLE_CHANGE)
			-- Add `a_change' to end of `current_changes'
		require
			a_change_not_void: a_change /= Void
		do
			current_changes.extend (a_change)
		end

	changed_region: EV_REGION
			-- Region which has changed for current operation,
			-- so that it will need to be redrawn

	changed_region_count: INTEGER
			-- Number of rectangles in changed region

	include_in_selection_bounding_box (an_element: RETICLE_VIEW_ELEMENT)
			-- Include `an_element' in current selection
			-- bounding box
		local
			elem: like anchored_model_element
		do
			elem := an_element.model_element
			temp_rectangle.make_by_corners (elem.left_x, elem.bottom_y, elem.right_x, elem.top_y)
			if selection_bounding_box.size_x = 0.0 or selection_bounding_box.size_y = 0.0 then
				selection_bounding_box.make_copy (temp_rectangle)
			else
				selection_bounding_box.extend_to_include (temp_rectangle)
			end
		end

	update_distance_limits (an_up_limit, a_down_limit, a_left_limit, a_right_limit: DOUBLE)
			-- Update limits for an operation that moves
			-- or duplicates elements from `an_up_limit',
			-- `a_down_limit', `a_left_limit' and
			-- `a_right_limit'
		require
			an_up_limit_nonnegative: an_up_limit >= 0.0
			a_down_limit_nonnegative: a_down_limit >= 0.0
			a_left_limit_nonnegative: a_left_limit >= 0.0
			a_right_limit_nonnegative: a_right_limit >= 0.0
		do
			up_limit := up_limit.min (grid.truncated_edge (an_up_limit))
			down_limit := down_limit.min (grid.truncated_edge (a_down_limit))
			left_limit := left_limit.min (grid.truncated_edge (a_left_limit))
			right_limit := right_limit.min (grid.truncated_edge (a_right_limit))
		end

	set_pixel_limits
			-- Set pixel limits from absolute distance limits
		do
			up_pixel_limit := pixel_limit (up_limit)
			down_pixel_limit := pixel_limit (down_limit)
			left_pixel_limit := pixel_limit (left_limit)
			right_pixel_limit := pixel_limit (right_limit)
		end

	pixel_limit (a_limit: DOUBLE): INTEGER
			-- `a_limit' in model units converted to pixels
		do
			Result  := (a_limit / model_units_per_pixel).truncated_to_integer_64.min ({INTEGER}.Max_value).max ({INTEGER}.Min_value).to_integer
		end

	horizontally_limited (a_value: INTEGER): INTEGER
			-- `a_value' adjusted to fall between
			-- `-left_pixel_limit' and `right_pixel_limit'
		do
			Result := a_value.max (-left_pixel_limit).min (right_pixel_limit)
		end

	vertically_limited (a_value: INTEGER): INTEGER
			-- `a_value' adjusted to fall between
			-- `up_pixel_limit' and `down_pixel_limit'
		do
			Result := a_value.max (-up_pixel_limit).min (down_pixel_limit)
		end

	add_partially_selected_element (an_element: like anchored_view_element)
			-- Add `an_element' to `partially_selected_elements'
		require
			an_element_not_void: an_element /= Void
		do
			partially_selected_elements.extend (an_element)
		end

	add_rotation_locked_element (an_element: like anchored_view_element)
			-- Add `an_element' to `rotation_locked_elements'
		require
			an_element_not_void: an_element /= Void
		do
			rotation_locked_elements.extend (an_element)
		end

	add_fixed_orientation_element (an_element: like anchored_view_element)
			-- Add `an_element' to `fixed_orientation_elements'
		require
			an_element_not_void: an_element /= Void
		do
			fixed_orientation_elements.extend (an_element)
		end

	add_duplication_locked_element (an_element: like anchored_view_element)
			-- Add `an_element' to `duplication_locked_elements'
		require
			an_element_not_void: an_element /= Void
		do
			duplication_locked_elements.extend (an_element)
		end

	add_never_duplicatable_element (an_element: like anchored_view_element)
			-- Add `an_element' to `never_duplicatable_elements'
		require
			an_element_not_void: an_element /= Void
		do
			never_duplicatable_elements.extend (an_element)
		end

	add_to_new_element_list (an_element: like anchored_view_element)
			-- Add `an_element' to `new_elements'
		require
			an_element_not_void: an_element /= Void
		do
			new_elements.extend (an_element)
		end

	add_horizontal_spread_element (an_element: like anchored_view_element)
			-- Add `an_element' to `spread_elements' as
			-- part of a horizontal Spread operation
		require
			an_element_not_void: an_element /= Void
		do
			spread_elements.extend (an_element)
			largest_spread_dimension := largest_spread_dimension.max (an_element.model_element.size_y)
			if an_element.is_locked_in_x or an_element.is_locked_in_y then
				unmovable_elements.extend (an_element)
			end
		end

	add_vertical_spread_element (an_element: like anchored_view_element)
			-- Add `an_element' to `spread_elements' as
			-- part of a vertical Spread operation
		require
			an_element_not_void: an_element /= Void
		do
			spread_elements.extend (an_element)
			largest_spread_dimension := largest_spread_dimension.max (an_element.model_element.size_x)
			if an_element.is_locked_in_x or an_element.is_locked_in_y then
				unmovable_elements.extend (an_element)
			end
		end

	add_field (an_element: like anchored_view_element)
			-- Add `an_element' to `cached_fields'
		require
			an_element_not_void: an_element /= Void
		do
			cached_fields.extend (an_element)
		end

	isolate (an_element: like anchored_view_element)
			-- Isolate `an_element' in current cut pattern
		require
			an_element_not_void: an_element /= Void
		do
			current_cut_pattern.isolate (an_element)
			last_isolated_count := last_isolated_count + 1
		end

	unisolate (an_element: like anchored_view_element)
			-- Unisolate `an_element' in current cut pattern
		require
			an_element_not_void: an_element /= Void
		do
			current_cut_pattern.unisolate (an_element)
			last_unisolated_count := last_unisolated_count + 1
		end

	set_isolated_site_cut_die (an_element: like anchored_view_element)
			-- Set cut die size and center for `an_element'
			-- in current cut pattern
		require
			an_element_not_void: an_element /= Void
		do
			current_cut_pattern.set_isolated_site_cut_die (an_element)
		end

	increment_last_isolation_failure_count
			-- Increment `last_isolation_failure_count'
		do
			last_isolation_failure_count := last_isolation_failure_count + 1
		end

	new_view_element (a_model_element: like anchored_model_element): like anchored_view_element
			-- New view element for `a_model_element'
		local
			elem: like a_model_element
		do
			elem := a_model_element
			if elem.is_die then
				if elem.is_cust_project then
					create {RETICLE_VIEW_CUST_DIE} Result.make (elem, Current)
				elseif elem.is_test_vehicle then
					create {RETICLE_VIEW_TEST_VEHICLE} Result.make (elem, Current)
				elseif elem.is_vendor_geometry then
					create {RETICLE_VIEW_VENDOR_GEOMETRY} Result.make (elem, Current)
				elseif elem.is_scribe then
					create {RETICLE_VIEW_SCRIBE} Result.make (elem, Current)
				elseif elem.is_filler then
					create {RETICLE_VIEW_FILLER} Result.make (elem, Current)
				else
					handle_internal_error ("Unknown element type for model die")
				end
				set_all_copies_list (Result)
			elseif elem.is_field then
				create {RETICLE_VIEW_FIELD} Result.make (elem, Current)
			elseif elem.is_field_label then
				create {RETICLE_VIEW_FIELD_LABEL} Result.make (elem, Current)
			elseif elem.is_non_field_label then
				create {RETICLE_VIEW_NON_FIELD_LABEL} Result.make (elem, Current)
			elseif elem.is_cut_pattern_label then
				create {RETICLE_VIEW_CUT_PATTERN_LABEL} Result.make (elem, Current)
			elseif elem.is_cut then
				create {RETICLE_VIEW_CUT} Result.make (elem, Current)
			else
				handle_internal_error ("Unknown element type for model element - not a die, field or cut")
			end
		end

	mbb_model_areas: LIST [RETICLE_MODEL_AREA]
			-- Areas of model and the elements whose bounding box
			-- intersects them

	scribe_cut_die_model_areas: like mbb_model_areas
			-- Areas of model and the elements whose cut die
			-- determined by surrounding scribes intersects them

	cut_cut_die_model_areas: like mbb_model_areas
			-- Areas of model and the elements whose cut die
			-- determined by surrounding cuts intersects them

	horizontal_scribe_model_areas: LIST [RETICLE_MODEL_AREA]
			-- Areas of model and the elements that are
			-- horizontal scribes whose bounding box
			-- intersects them

	vertical_scribe_model_areas: LIST [RETICLE_MODEL_AREA]
			-- Areas of model and the elements that are
			-- vertical scribes whose bounding box
			-- intersects them

feature {VIEW_ELEMENT_DRAWER} -- Implementation
	
	changed_bounding_box: EV_RECTANGLE
			-- Bounding box of changed region

feature {NONE} -- Named views

	standard_fixed_view: RETICLE_NAMED_VIEW
			-- Standard view in "fixed" mode
		do
			if standard_fixed_view_internal = Void then
				create standard_fixed_view_internal.make (Standard_fixed_view_name, design_number_label_generator, fixed_status_color_generator, fixed_status_fill_generator, transparent_opacity_generator)
			end
			Result := standard_fixed_view_internal
		ensure
			result_not_void: Result /= Void
		end

	standard_locked_view: RETICLE_NAMED_VIEW
			-- Standard view in "locked" mode
		do
			if standard_locked_view_internal = Void then
				create standard_locked_view_internal.make (Standard_locked_view_name, design_number_label_generator, locked_status_color_generator, locked_status_fill_generator, transparent_opacity_generator)
			end
			Result := standard_locked_view_internal
		ensure
			result_not_void: Result /= Void
		end

	account_type_view: RETICLE_NAMED_VIEW
			-- Account type view
		do
			if account_type_view_internal = Void then
				create account_type_view_internal.make (Account_type_view_name, account_type_label_generator, account_type_color_generator, none_fill_generator, transparent_opacity_generator)
			end
			Result := account_type_view_internal
		ensure
			result_not_void: Result /= Void
		end

	lot_split_view: RETICLE_NAMED_VIEW
			-- Lot split view
		do
			if lot_split_view_internal = Void then
				create lot_split_view_internal.make (Lot_split_view_name, wafer_etest_id_label_generator, wafer_etest_id_color_generator, none_fill_generator, transparent_opacity_generator)
			end
			Result := lot_split_view_internal
		ensure
			result_not_void: Result /= Void
		end

	packaging_view: RETICLE_NAMED_VIEW
			-- Packaging view
		do
			if packaging_view_internal = Void then
				create packaging_view_internal.make (Packaging_view_name, packaging_label_generator, packaging_color_generator, none_fill_generator, transparent_opacity_generator)
			end
			Result := packaging_view_internal
		ensure
			result_not_void: Result /= Void
		end

	validity_check_view: RETICLE_NAMED_VIEW
			-- Validity Check view
		do
			if validity_check_view_internal = Void then
				create validity_check_view_internal.make (Validity_check_view_name,design_number_label_generator, validity_check_color_generator, none_fill_generator, transparent_opacity_generator)
			end
			Result := validity_check_view_internal
		ensure
			result_not_void: Result /= Void
		end

	search_results_view: RETICLE_NAMED_VIEW
			-- Search Results view
		do
			if search_results_view_internal = Void then
				create search_results_view_internal.make (Search_results_view_name, design_number_label_generator, search_results_color_generator, search_results_fill_generator, transparent_opacity_generator)
			end
			Result := search_results_view_internal
		ensure
			result_not_void: Result /= Void
		end

	highlighted_selection_view: RETICLE_NAMED_VIEW
			-- Highlighted Selection view
		do
			if highlighted_selection_view_internal = Void then
				create highlighted_selection_view_internal.make (highlighted_selection_view_name, design_number_label_generator, highlighted_selection_color_generator, highlighted_selection_fill_generator, transparent_opacity_generator)
			end
			Result := highlighted_selection_view_internal
		ensure
			result_not_void: Result /= Void
		end

	design_name_view: RETICLE_NAMED_VIEW
			-- Design name view
		do
			if design_name_view_internal = Void then
				create design_name_view_internal.make (Design_name_view_name, design_name_label_generator, default_color_generator, none_fill_generator, transparent_opacity_generator)
			end
			Result := design_name_view_internal
		ensure
			result_not_void: Result /= Void
		end

	die_id_view: RETICLE_NAMED_VIEW
			-- Die ID view
		do
			if die_id_view_internal = Void then
				create die_id_view_internal.make (Die_id_view_name, die_id_label_generator, default_color_generator, none_fill_generator, transparent_opacity_generator)
			end
			Result := die_id_view_internal
		ensure
			result_not_void: Result /= Void
		end

	account_number_view: RETICLE_NAMED_VIEW
			-- Account number view
		do
			if account_number_view_internal = Void then
				create account_number_view_internal.make (Account_number_view_name, account_number_label_generator, default_color_generator, none_fill_generator, transparent_opacity_generator)
			end
			Result := account_number_view_internal
		ensure
			result_not_void: Result /= Void
		end

	org_short_id_view: RETICLE_NAMED_VIEW
			-- Organization short ID view
		do
			if org_short_id_view_internal = Void then
				create org_short_id_view_internal.make (Org_short_id_view_name, org_short_id_label_generator, default_color_generator, none_fill_generator, transparent_opacity_generator)
			end
			Result := org_short_id_view_internal
		ensure
			result_not_void: Result /= Void
		end

	overhead_style_view: RETICLE_NAMED_VIEW
			-- Overhead style view
		do
			if overhead_style_view_internal = Void then
				create overhead_style_view_internal.make (Overhead_style_view_name, overhead_style_label_generator, overhead_style_color_generator, none_fill_generator, transparent_opacity_generator)
			end
			Result := overhead_style_view_internal
		ensure
			result_not_void: Result /= Void
		end

	max_die_size_view: RETICLE_NAMED_VIEW
			-- Max Die Size view
		do
			if max_die_size_view_internal = Void then
				max_die_size_color_generator.set_tight_packing_fit (policies.tight_packing_fit)
				max_die_size_fill_generator.set_tight_packing_fit (policies.tight_packing_fit)
				create max_die_size_view_internal.make (Max_die_view_name, max_die_size_label_generator, max_die_size_color_generator, max_die_size_fill_generator, transparent_opacity_generator)
			end
			Result := max_die_size_view_internal
		ensure
			result_not_void: Result /= Void
		end

	rotation_locked_view: RETICLE_NAMED_VIEW
			-- Rotation Locked view
		do
			if rotation_locked_view_internal = Void then
				create rotation_locked_view_internal.make (Rotation_locked_view_name, design_number_label_generator, default_color_generator, rotation_locked_fill_generator, transparent_opacity_generator)
			end
			Result := rotation_locked_view_internal
		ensure
			result_not_void: Result /= Void
		end

	duplication_locked_view: RETICLE_NAMED_VIEW
			-- Duplication Locked view
		do
			if duplication_locked_view_internal = Void then
				create duplication_locked_view_internal.make (Duplication_locked_view_name, design_number_label_generator, default_color_generator, duplication_locked_fill_generator, transparent_opacity_generator)
			end
			Result := duplication_locked_view_internal
		ensure
			result_not_void: Result /= Void
		end

	deletion_locked_view: RETICLE_NAMED_VIEW
			-- Deletion Locked view
		do
			if deletion_locked_view_internal = Void then
				create deletion_locked_view_internal.make (Deletion_locked_view_name, design_number_label_generator, default_color_generator, deletion_locked_fill_generator, transparent_opacity_generator)
			end
			Result := deletion_locked_view_internal
		ensure
			result_not_void: Result /= Void
		end

	cut_view: RETICLE_NAMED_VIEW
			-- Cut view
		do
			if cut_view_internal = Void then
				create cut_view_internal.make (Cut_view_name, design_number_label_generator, wafer_etest_id_color_generator, isolation_fill_generator, transparent_opacity_generator)
			end
			Result := cut_view_internal
		ensure
			result_not_void: Result /= Void
		end

	standard_fixed_view_internal: RETICLE_NAMED_VIEW

	standard_locked_view_internal: RETICLE_NAMED_VIEW

	account_type_view_internal: RETICLE_NAMED_VIEW

	lot_split_view_internal: RETICLE_NAMED_VIEW

	packaging_view_internal: RETICLE_NAMED_VIEW

	validity_check_view_internal: RETICLE_NAMED_VIEW

	search_results_view_internal: RETICLE_NAMED_VIEW

	highlighted_selection_view_internal: RETICLE_NAMED_VIEW

	design_name_view_internal: RETICLE_NAMED_VIEW

	die_id_view_internal: RETICLE_NAMED_VIEW

	account_number_view_internal: RETICLE_NAMED_VIEW

	org_short_id_view_internal: RETICLE_NAMED_VIEW

	overhead_style_view_internal: RETICLE_NAMED_VIEW

	max_die_size_view_internal: RETICLE_NAMED_VIEW

	rotation_locked_view_internal: RETICLE_NAMED_VIEW

	duplication_locked_view_internal: RETICLE_NAMED_VIEW

	deletion_locked_view_internal: RETICLE_NAMED_VIEW

	cut_view_internal: RETICLE_NAMED_VIEW

feature {NONE} -- Colors generators

	default_color_generator: RETICLE_VIEW_ELEMENT_COLOR_GENERATOR
		do
			if default_color_generator_internal = Void then
				create default_color_generator_internal.make (palette)
			end
			Result := default_color_generator_internal
		ensure
			result_not_void: Result /= Void
		end

	fixed_status_color_generator: RETICLE_VIEW_ELEMENT_COLOR_GENERATOR
		do
			if fixed_status_color_generator_internal = Void then
				create fixed_status_color_generator_internal.make (palette)
			end
			Result := fixed_status_color_generator_internal
		ensure
			result_not_void: Result /= Void
		end

	locked_status_color_generator: RETICLE_VIEW_ELEMENT_COLOR_GENERATOR
		do
			if locked_status_color_generator_internal = Void then
				create locked_status_color_generator_internal.make (palette)
			end
			Result := locked_status_color_generator_internal
		ensure
			result_not_void: Result /= Void
		end

	account_type_color_generator: RETICLE_VIEW_ELEMENT_COLOR_GENERATOR
		do
			if account_type_color_generator_internal = Void then
				create account_type_color_generator_internal.make (palette)
			end
			Result := account_type_color_generator_internal
		ensure
			result_not_void: Result /= Void
		end

	overhead_style_color_generator: RETICLE_VIEW_ELEMENT_COLOR_GENERATOR
		do
			if overhead_style_color_generator_internal = Void then
				create overhead_style_color_generator_internal.make (palette)
			end
			Result := overhead_style_color_generator_internal
		ensure
			result_not_void: Result /= Void
		end

	max_die_size_color_generator: MAX_DIE_SIZE_COLOR_GENERATOR
		do
			if max_die_size_color_generator_internal = Void then
				create max_die_size_color_generator_internal.make (palette)
			end
			Result := max_die_size_color_generator_internal
		ensure
			result_not_void: Result /= Void
		end

	packaging_color_generator: PACKAGING_COLOR_GENERATOR
		do
			if packaging_color_generator_internal = Void then
				create packaging_color_generator_internal.make (palette)
			end
			Result := packaging_color_generator_internal
		ensure
			result_not_void: Result /= Void
		end

	validity_check_color_generator: VALIDITY_CHECK_COLOR_GENERATOR
		do
			if validity_check_color_generator_internal = Void then
				create validity_check_color_generator_internal.make (palette)
				validity_check_color_generator_internal.set_current_generator (no_validity_color_generator)
			end
			Result := validity_check_color_generator_internal
		ensure
			result_not_void: Result /= Void
		end

	wafer_etest_id_color_generator: RETICLE_VIEW_ELEMENT_COLOR_GENERATOR
		do
			if wafer_etest_id_color_generator_internal = Void then
				create wafer_etest_id_color_generator_internal.make (palette)
			end
			Result := wafer_etest_id_color_generator_internal
		ensure
			result_not_void: Result /= Void
		end

	search_results_color_generator: SEARCH_RESULTS_COLOR_GENERATOR
		do
			if search_results_color_generator_internal = Void then
				create search_results_color_generator_internal.make (palette)
			end
			Result := search_results_color_generator_internal
		ensure
			result_not_void: Result /= Void
		end

	highlighted_selection_color_generator: HIGHLIGHTED_SELECTION_COLOR_GENERATOR
		do
			if highlighted_selection_color_generator_internal = Void then
				create highlighted_selection_color_generator_internal.make (palette)
			end
			Result := highlighted_selection_color_generator_internal
		ensure
			result_not_void: Result /= Void
		end

	no_validity_color_generator: RETICLE_VIEW_ELEMENT_COLOR_GENERATOR
		do
			if no_validity_color_generator_internal = Void then
				create no_validity_color_generator_internal.make (palette)
			end
			Result := no_validity_color_generator_internal
		ensure
			result_not_void: Result /= Void
		end

	all_pack_validity_color_generator: ALL_PACK_VALIDITY_COLOR_GENERATOR
		do
			if all_pack_validity_color_generator_internal = Void then
				create all_pack_validity_color_generator_internal.make (palette)
			end
			Result := all_pack_validity_color_generator_internal
		ensure
			result_not_void: Result /= Void
		end

	all_cut_validity_color_generator: ALL_CUT_VALIDITY_COLOR_GENERATOR
		do
			if all_cut_validity_color_generator_internal = Void then
				create all_cut_validity_color_generator_internal.make (palette)
			end
			Result := all_cut_validity_color_generator_internal
		ensure
			result_not_void: Result /= Void
		end

	outside_fields_color_generator: RETICLE_VIEW_ELEMENT_COLOR_GENERATOR
		do
			if outside_fields_color_generator_internal = Void then
				create outside_fields_color_generator_internal.make (palette)
			end
			Result := outside_fields_color_generator_internal
		ensure
			result_not_void: Result /= Void
		end

	overlapping_field_color_generator: SCRIBES_INCLUDABLE_COLOR_GENERATOR
		do
			if overlapping_field_color_generator_internal = Void then
				create overlapping_field_color_generator_internal.make (palette)
			end
			Result := overlapping_field_color_generator_internal
		ensure
			result_not_void: Result /= Void
		end

	overlapping_object_color_generator: RETICLE_VIEW_ELEMENT_COLOR_GENERATOR
		do
			if overlapping_object_color_generator_internal = Void then
				create overlapping_object_color_generator_internal.make (palette)
			end
			Result := overlapping_object_color_generator_internal
		ensure
			result_not_void: Result /= Void
		end

	below_min_size_color_generator: RETICLE_VIEW_ELEMENT_COLOR_GENERATOR
		do
			if below_min_size_color_generator_internal = Void then
				create below_min_size_color_generator_internal.make (palette)
			end
			Result := below_min_size_color_generator_internal
		ensure
			result_not_void: Result /= Void
		end

	needs_lot_assignment_color_generator: RETICLE_VIEW_ELEMENT_COLOR_GENERATOR
		do
			if needs_lot_assignment_color_generator_internal = Void then
				create needs_lot_assignment_color_generator_internal.make (palette)
			end
			Result := needs_lot_assignment_color_generator_internal
		ensure
			result_not_void: Result /= Void
		end

	unlocked_unassigned_color_generator: RETICLE_VIEW_ELEMENT_COLOR_GENERATOR
		do
			if unlocked_unassigned_color_generator_internal = Void then
				create unlocked_unassigned_color_generator_internal.make (palette)
			end
			Result := unlocked_unassigned_color_generator_internal
		ensure
			result_not_void: Result /= Void
		end

	close_to_edge_color_generator: SCRIBES_INCLUDABLE_COLOR_GENERATOR
		do
			if close_to_edge_color_generator_internal = Void then
				create close_to_edge_color_generator_internal.make (palette)
			end
			Result := close_to_edge_color_generator_internal
		ensure
			result_not_void: Result /= Void
		end

	close_to_object_color_generator: RETICLE_VIEW_ELEMENT_COLOR_GENERATOR
		do
			if close_to_object_color_generator_internal = Void then
				create close_to_object_color_generator_internal.make (palette)
			end
			Result := close_to_object_color_generator_internal
		ensure
			result_not_void: Result /= Void
		end

	scribe_cut_die_too_big_color_generator: RETICLE_VIEW_ELEMENT_COLOR_GENERATOR
		do
			if scribe_cut_die_too_big_color_generator_internal = Void then
				create scribe_cut_die_too_big_color_generator_internal.make (palette)
			end
			Result := scribe_cut_die_too_big_color_generator_internal
		ensure
			result_not_void: Result /= Void
		end

	not_isolated_in_cut_die_color_generator: RETICLE_VIEW_ELEMENT_COLOR_GENERATOR
		do
			if not_isolated_in_cut_die_color_generator_internal = Void then
				create not_isolated_in_cut_die_color_generator_internal.make (palette)
			end
			Result := not_isolated_in_cut_die_color_generator_internal
		ensure
			result_not_void: Result /= Void
		end

	not_centered_in_cut_die_color_generator: RETICLE_VIEW_ELEMENT_COLOR_GENERATOR
		do
			if not_centered_in_cut_die_color_generator_internal = Void then
				create not_centered_in_cut_die_color_generator_internal.make (palette)
			end
			Result := not_centered_in_cut_die_color_generator_internal
		ensure
			result_not_void: Result /= Void
		end

	floating_scribe_color_generator: RETICLE_VIEW_ELEMENT_COLOR_GENERATOR
		do
			if floating_scribe_color_generator_internal = Void then
				create floating_scribe_color_generator_internal.make (palette)
			end
			Result := floating_scribe_color_generator_internal
		ensure
			result_not_void: Result /= Void
		end

	misaligned_scribe_color_generator: RETICLE_VIEW_ELEMENT_COLOR_GENERATOR
		do
			if misaligned_scribe_color_generator_internal = Void then
				create misaligned_scribe_color_generator_internal.make (palette)
			end
			Result := misaligned_scribe_color_generator_internal
		ensure
			result_not_void: Result /= Void
		end

	small_but_legal_color_generator: SCRIBES_INCLUDABLE_COLOR_GENERATOR
		do
			if small_but_legal_color_generator_internal = Void then
				create small_but_legal_color_generator_internal.make (palette)
			end
			Result := small_but_legal_color_generator_internal
		ensure
			result_not_void: Result /= Void
		end

	invalid_cut_die_color_generator: RETICLE_VIEW_ELEMENT_COLOR_GENERATOR
		do
			if invalid_cut_die_color_generator_internal = Void then
				create invalid_cut_die_color_generator_internal.make (palette)
			end
			Result := invalid_cut_die_color_generator_internal
		ensure
			result_not_void: Result /= Void
		end

	cut_cut_die_too_big_color_generator: RETICLE_VIEW_ELEMENT_COLOR_GENERATOR
		do
			if cut_cut_die_too_big_color_generator_internal = Void then
				create cut_cut_die_too_big_color_generator_internal.make (palette)
			end
			Result := cut_cut_die_too_big_color_generator_internal
		ensure
			result_not_void: Result /= Void
		end

	wrong_lot_color_generator: RETICLE_VIEW_ELEMENT_COLOR_GENERATOR
		do
			if wrong_lot_color_generator_internal = Void then
				create wrong_lot_color_generator_internal.make (palette)
			end
			Result := wrong_lot_color_generator_internal
		ensure
			result_not_void: Result /= Void
		end

	non_default_cut_die_size_color_generator: RETICLE_VIEW_ELEMENT_COLOR_GENERATOR
		do
			if non_default_cut_die_size_color_generator_internal = Void then
				create non_default_cut_die_size_color_generator_internal.make (palette)
			end
			Result := non_default_cut_die_size_color_generator_internal
		ensure
			result_not_void: Result /= Void
		end

	shared_cut_die_color_generator: RETICLE_VIEW_ELEMENT_COLOR_GENERATOR
		do
			if shared_cut_die_color_generator_internal = Void then
				create shared_cut_die_color_generator_internal.make (palette)
			end
			Result := shared_cut_die_color_generator_internal
		ensure
			result_not_void: Result /= Void
		end

	extra_cut_color_generator: RETICLE_VIEW_ELEMENT_COLOR_GENERATOR
		do
			if extra_cut_color_generator_internal = Void then
				create extra_cut_color_generator_internal.make (palette)
			end
			Result := extra_cut_color_generator_internal
		ensure
			result_not_void: Result /= Void
		end

	default_color_generator_internal: DEFAULT_COLOR_GENERATOR

	fixed_status_color_generator_internal: FIXED_STATUS_COLOR_GENERATOR

	locked_status_color_generator_internal: LOCKED_STATUS_COLOR_GENERATOR

	account_type_color_generator_internal: ACCOUNT_TYPE_COLOR_GENERATOR

	overhead_style_color_generator_internal: OVERHEAD_STYLE_COLOR_GENERATOR

	max_die_size_color_generator_internal: MAX_DIE_SIZE_COLOR_GENERATOR

	packaging_color_generator_internal: PACKAGING_COLOR_GENERATOR

	validity_check_color_generator_internal: VALIDITY_CHECK_COLOR_GENERATOR

	wafer_etest_id_color_generator_internal: WAFER_ETEST_ID_COLOR_GENERATOR

	search_results_color_generator_internal: SEARCH_RESULTS_COLOR_GENERATOR

	highlighted_selection_color_generator_internal: HIGHLIGHTED_SELECTION_COLOR_GENERATOR

	no_validity_color_generator_internal: NO_VALIDITY_COLOR_GENERATOR

	all_pack_validity_color_generator_internal: ALL_PACK_VALIDITY_COLOR_GENERATOR

	all_cut_validity_color_generator_internal: ALL_CUT_VALIDITY_COLOR_GENERATOR

	outside_fields_color_generator_internal: OUTSIDE_FIELDS_COLOR_GENERATOR

	overlapping_field_color_generator_internal: OVERLAPPING_FIELD_COLOR_GENERATOR

	overlapping_object_color_generator_internal: OVERLAPPING_OBJECT_COLOR_GENERATOR

	below_min_size_color_generator_internal: BELOW_MIN_SIZE_COLOR_GENERATOR

	needs_lot_assignment_color_generator_internal: NEEDS_LOT_ASSIGNMENT_COLOR_GENERATOR

	unlocked_unassigned_color_generator_internal: UNLOCKED_UNASSIGNED_COLOR_GENERATOR

	close_to_edge_color_generator_internal: CLOSE_TO_EDGE_COLOR_GENERATOR

	close_to_object_color_generator_internal: CLOSE_TO_OBJECT_COLOR_GENERATOR

	scribe_cut_die_too_big_color_generator_internal: SCRIBE_CUT_DIE_TOO_BIG_COLOR_GENERATOR

	not_centered_in_cut_die_color_generator_internal: NOT_CENTERED_IN_CUT_DIE_COLOR_GENERATOR

	not_isolated_in_cut_die_color_generator_internal: NOT_ISOLATED_IN_CUT_DIE_COLOR_GENERATOR

	floating_scribe_color_generator_internal: FLOATING_SCRIBE_COLOR_GENERATOR

	misaligned_scribe_color_generator_internal: MISALIGNED_SCRIBE_COLOR_GENERATOR

	small_but_legal_color_generator_internal: SMALL_BUT_LEGAL_COLOR_GENERATOR

	invalid_cut_die_color_generator_internal: INVALID_CUT_DIE_COLOR_GENERATOR

	cut_cut_die_too_big_color_generator_internal: CUT_CUT_DIE_TOO_BIG_COLOR_GENERATOR

	wrong_lot_color_generator_internal: WRONG_LOT_COLOR_GENERATOR

	non_default_cut_die_size_color_generator_internal: NON_DEFAULT_CUT_DIE_SIZE_COLOR_GENERATOR

	shared_cut_die_color_generator_internal: SHARED_CUT_DIE_COLOR_GENERATOR

	extra_cut_color_generator_internal: EXTRA_CUT_COLOR_GENERATOR

feature {NONE} -- Fill generators

	all_fill_generator: RETICLE_VIEW_ELEMENT_FILL_GENERATOR
		do
			if all_fill_generator_internal = Void then
				create all_fill_generator_internal.make (palette)
			end
			Result := all_fill_generator_internal
		ensure
			result_not_void: Result /= Void
		end

	none_fill_generator: RETICLE_VIEW_ELEMENT_FILL_GENERATOR
		do
			if none_fill_generator_internal = Void then
				create none_fill_generator_internal.make (palette)
			end
			Result := none_fill_generator_internal
		ensure
			result_not_void: Result /= Void
		end

	selected_fill_generator: RETICLE_VIEW_ELEMENT_FILL_GENERATOR
		do
			if selected_fill_generator_internal = Void then
				create selected_fill_generator_internal.make (palette)
			end
			Result := selected_fill_generator_internal
		ensure
			result_not_void: Result /= Void
		end

	unselected_fill_generator: RETICLE_VIEW_ELEMENT_FILL_GENERATOR
		do
			if unselected_fill_generator_internal = Void then
				create unselected_fill_generator_internal.make (palette)
			end
			Result := unselected_fill_generator_internal
		ensure
			result_not_void: Result /= Void
		end

	fixed_status_fill_generator: RETICLE_VIEW_ELEMENT_FILL_GENERATOR
		do
			if fixed_status_fill_generator_internal = Void then
				create fixed_status_fill_generator_internal.make (palette)
			end
			Result := fixed_status_fill_generator_internal
		ensure
			result_not_void: Result /= Void
		end

	locked_status_fill_generator: RETICLE_VIEW_ELEMENT_FILL_GENERATOR
		do
			if locked_status_fill_generator_internal = Void then
				create locked_status_fill_generator_internal.make (palette)
			end
			Result := locked_status_fill_generator_internal
		ensure
			result_not_void: Result /= Void
		end

	max_die_size_fill_generator: MAX_DIE_SIZE_FILL_GENERATOR
		do
			if max_die_size_fill_generator_internal = Void then
				create max_die_size_fill_generator_internal.make (palette)
			end
			Result := max_die_size_fill_generator_internal
		ensure
			result_not_void: Result /= Void
		end

	search_results_fill_generator: RETICLE_VIEW_ELEMENT_FILL_GENERATOR
		do
			if search_results_fill_generator_internal = Void then
				create search_results_fill_generator_internal.make (palette)
			end
			Result := search_results_fill_generator_internal
		ensure
			result_not_void: Result /= Void
		end

	highlighted_selection_fill_generator: RETICLE_VIEW_ELEMENT_FILL_GENERATOR
		do
			if highlighted_selection_fill_generator_internal = Void then
				create highlighted_selection_fill_generator_internal.make (palette)
			end
			Result := highlighted_selection_fill_generator_internal
		ensure
			result_not_void: Result /= Void
		end

	rotation_locked_fill_generator: RETICLE_VIEW_ELEMENT_FILL_GENERATOR
		do
			if rotation_locked_fill_generator_internal = Void then
				create rotation_locked_fill_generator_internal.make (palette)
			end
			Result := rotation_locked_fill_generator_internal
		ensure
			result_not_void: Result /= Void
		end

	duplication_locked_fill_generator: RETICLE_VIEW_ELEMENT_FILL_GENERATOR
		do
			if duplication_locked_fill_generator_internal = Void then
				create duplication_locked_fill_generator_internal.make (palette)
			end
			Result := duplication_locked_fill_generator_internal
		ensure
			result_not_void: Result /= Void
		end

	deletion_locked_fill_generator: RETICLE_VIEW_ELEMENT_FILL_GENERATOR
		do
			if deletion_locked_fill_generator_internal = Void then
				create deletion_locked_fill_generator_internal.make (palette)
			end
			Result := deletion_locked_fill_generator_internal
		ensure
			result_not_void: Result /= Void
		end

	isolation_fill_generator: RETICLE_VIEW_ELEMENT_FILL_GENERATOR
		do
			if isolation_fill_generator_internal = Void then
				create isolation_fill_generator_internal.make (palette)
			end
			Result := isolation_fill_generator_internal
		ensure
			result_not_void: Result /= Void
		end

	all_fill_generator_internal: ALL_FILL_GENERATOR

	none_fill_generator_internal: NONE_FILL_GENERATOR

	selected_fill_generator_internal: SELECTED_FILL_GENERATOR

	unselected_fill_generator_internal: UNSELECTED_FILL_GENERATOR

	fixed_status_fill_generator_internal: FIXED_STATUS_FILL_GENERATOR

	locked_status_fill_generator_internal: LOCKED_STATUS_FILL_GENERATOR

	max_die_size_fill_generator_internal: MAX_DIE_SIZE_FILL_GENERATOR

	search_results_fill_generator_internal: SEARCH_RESULTS_FILL_GENERATOR

	highlighted_selection_fill_generator_internal: HIGHLIGHTED_SELECTION_FILL_GENERATOR

	rotation_locked_fill_generator_internal: ROTATION_LOCKED_FILL_GENERATOR

	duplication_locked_fill_generator_internal: DUPLICATION_LOCKED_FILL_GENERATOR

	deletion_locked_fill_generator_internal: DELETION_LOCKED_FILL_GENERATOR

	isolation_fill_generator_internal: ISOLATION_FILL_GENERATOR

feature {RETICLE_CHANGE, RETICLE_SIMPLE_CHANGE} -- Implementation

	set_all_copies_list (an_element: like anchored_view_element)
		do
			find_copies (an_element)
			last_element_list.extend (an_element)
			an_element.set_all_copies (last_element_list)
		end

	update_copies_table_for_delete (a_list: LIST [like anchored_view_element])
			-- Delete entries for every element in `a_list'
			-- from `copies_table'
		require
			a_list_not_void: a_list /= Void
		do
			from
				a_list.start
			until
				a_list.after
			loop
				delete_from_copies_table (a_list.item)
				a_list.forth
			end
		end

	delete_from_copies_table (an_element: like anchored_view_element)
			-- Delete entry for `an_element' from `copies_table',
			-- if present
		require
			an_element_not_void: an_element /= Void
		do
			copies_table.search (an_element.die)
			if copies_table.found then
				copies_table.remove (an_element.die)
			end
		end

	unmark_all
			-- Remove "marked" indicator from all elements
		do
			apply_action_with_filter (all_filter, unmark_action)
		end

	unmark_and_clear_invalid_all
			-- Remove "marked" indicator from all elements
			-- and clear invalid element list for every
			-- element
		do
			apply_action_with_filter (all_filter, unmark_and_clear_invalid_action)
		end

	remove_marked_elements
			-- Remove elements that are marked
		local
			list: ARRAYED_LIST [like anchored_view_element]
		do
			create list.make (elements.count)
			extend_list_action.set_list (list)
			apply_action_with_filter (unmarked_filter, extend_list_action)
			elements := list
		end

feature {NONE} -- Implementation

	set_selection (a_filter: VIEW_ELEMENT_FILTER)
			-- Set selection to all items which match filter
		require
			a_filter_not_void: a_filter /= Void
		do
			set_selection_action.set_filter (a_filter)
			apply_select_action_and_redraw (set_selection_action)
		end
	
	add_to_selection (a_filter: VIEW_ELEMENT_FILTER)
			-- Add to selection all items which match filter
		require
			a_filter_not_void: a_filter /= Void
		do
			add_to_selection_action.set_filter (a_filter)
			apply_select_action_and_redraw (add_to_selection_action)
		end
	
	remove_from_selection (a_filter: VIEW_ELEMENT_FILTER)
			-- Remove from selection all items which match filter
		require
			a_filter_not_void: a_filter /= Void
		do
			remove_from_selection_action.set_filter (a_filter)
			apply_select_action_and_redraw (remove_from_selection_action)
		end
	
	initialize_change
			-- Do initialize required before starting operation
			-- operation that could generate changes that go
			-- on history list
		do
			initialize_status
			initialize_change_without_status
		end

	initialize_change_without_status
			-- Do initialize required before starting operation
			-- operation that could generate changes that go
			-- on history list, but do not initialize errors
			-- and warnings
		do
			set_change_list_empty
			set_initial_selected_elements
			initial_locked_mode := is_locked_mode
			initial_view := current_view
		end

	set_change_list_empty
			-- Set list of changes made by current operations
			-- to be the empty list.  Only needed for operations
			-- which can make changes that go into the history
			-- list (not needed for operations that change
			-- selection, displayed area or how things are
			-- displayed)
		do
			create current_changes.make (elements.count)
		end

	set_initial_selected_elements
			-- Set `initial_selected_elements' to list of
			-- currently selected elements
		do
			create initial_selected_elements.make (selected_count.max (0))
			extend_list_action.set_list (initial_selected_elements)
			apply_action_with_filter (selected_filter, extend_list_action)
		end

	set_final_selected_elements
			-- Set `final_selected_elements' to list of
			-- currently selected elements
		do
			if is_history_change_needed then
				create final_selected_elements.make (selected_count.max (0))
				extend_list_action.set_list (final_selected_elements)
				apply_action_with_filter (selected_filter, extend_list_action)
			end
		end

	set_final_selected_elements_to_initial
			-- Set `final_selected_elements' to
			-- `initial_selected_elements'
		do
			final_selected_elements := initial_selected_elements
		end

	new_standard_change (a_name: STRING): RETICLE_CHANGE
		do
			create {RETICLE_STANDARD_CHANGE} Result.make (a_name)
		end

	new_creation_change (a_name: STRING): RETICLE_CHANGE
		do
			create {RETICLE_CREATION_CHANGE} Result.make (a_name)
		end

	new_deletion_change (a_name: STRING): RETICLE_CHANGE
		do
			create {RETICLE_DELETION_CHANGE} Result.make (a_name)
		end

	new_creation_and_deletion_change (a_name: STRING): RETICLE_CHANGE
		do
			create {RETICLE_CREATION_AND_DELETION_CHANGE} Result.make (a_name)
		end

	new_cut_creation_change (a_name: STRING): RETICLE_CHANGE
		local
			chg: RETICLE_CUT_CREATION_CHANGE
		do
			create chg.make (a_name)
			chg.set_cut_pattern (current_cut_pattern.pattern)
			Result := chg
		end

	new_cut_deletion_change (a_name: STRING): RETICLE_CHANGE
		local
			chg: RETICLE_CUT_DELETION_CHANGE
		do
			create chg.make (a_name)
			chg.set_cut_pattern (current_cut_pattern.pattern)
			Result := chg
		end

	add_change_to_history_list (a_change: RETICLE_CHANGE)
			-- Add `a_change' made by last operation to
			-- the history list as a single change that
			-- can be undone/redone, if there are any simple
			-- changes
		require
			a_change_not_void: a_change /= Void
		local
			list: like reticle_change_history_list
		do
			if is_history_change_needed then
				-- Truncate portion of list after cursor
				-- (if not at end, due to an undo)
				
				list := reticle_change_history_list
				from
				until
					list.off
				loop
					list.remove
				end
				
				-- Fill in state information in change

				a_change.set_view (Current)
				a_change.set_changes (current_changes)
				a_change.set_displayed_area (create {CONST_RECTANGLE}.make_copy (displayed_model_area))
				a_change.set_initial_locked_mode (initial_locked_mode)
				a_change.set_final_locked_mode (is_locked_mode)
				a_change.set_initial_named_view (initial_view)
				a_change.set_final_named_view (current_view)
				a_change.set_initial_selected_elements (initial_selected_elements)
				a_change.set_final_selected_elements (final_selected_elements)

				-- Add entry to end of history list and move
				-- cursor past it

				list.extend (a_change)
				list.finish
				list.forth

				-- Notify others
				call_change_history_list_actions
			end
		end

	is_history_change_needed: BOOLEAN
			-- Should a change be added to the history change list
			-- after an operation which may have created a
			-- history change?
		do
			Result := reticle_change_history_capacity > 0 and not current_changes.is_empty
		end

	initialize_cut_counts
			-- Set `last_horizontal_cut_count' and
			-- `last_vertical_cut_count' to zero
		do
			last_horizontal_cut_count := 0
			last_vertical_cut_count := 0
		end

	clear_processed_indicators
			-- Clear "processed" indicator for every element,
			-- to indicate that no copies of any element have
			-- been processed
		do
			apply_action_with_filter (all_filter, clear_processed_action)
		end

	create_new_pixmap (a_width, a_height: INTEGER)
			-- Create `pixmap' of size `a_width' by `a_height'
		do
			create pixmap.make_with_size (a_width, a_height)
			pixmap.set_background_color (palette.current_background_color)
			pixmap_rectangle.move_and_resize (0, 0, pixmap.width, pixmap.height)
		end

	enlarged_model_bounding_box: CONST_RECTANGLE
			-- Minimum bounding box of model enlarged a bit
			-- so that there is some empty space around it
		local
			rect: RECTANGLE
		do
			create rect.make_copy (model.bounding_box)
			rect.scale_by (1.0 + Model_bounding_box_extra_fraction)
			Result := rect
		end

	enlarged_field_bounding_box (a_field_name: STRING): CONST_RECTANGLE
			-- Minimum bounding box of field named
			-- `a_field_name' enlarged a bit so that there
			-- is some empty space around it
		require
			a_field_name_not_void: a_field_name /= Void
		local
			rect: RECTANGLE
		do
			create rect.make_copy (model.field_with_name (a_field_name).bounding_box_including_label)
			rect.scale_by (1.0 + Model_bounding_box_extra_fraction)
			Result := rect
		end

	set_background_color_for_zoom
			-- Set current background color according
			-- to whether view is fully zoomed in or not
		do
			if is_fully_zoomed_in then
				palette.set_current_background_color (palette.fully_zoomed_in_background_color)
			else
				palette.set_current_background_color (palette.standard_background_color)
			end
			pixmap.set_background_color (palette.current_background_color)
		end

	internal_redraw_rubber_box (a_x, a_y, a_width, a_height: INTEGER a_line_width: INTEGER a_color: EV_COLOR)
			-- Erase current rubber box, if any, and draw a
			-- new rubber box on the canvas whose upper left
			-- corner is at (`a_x', `a_y') and whose size
			-- is `a_width' by `a_height'.  Draw box `a_line_width'
			-- pixels thick using `a_color'.  Set new current
			-- rubber box to the one just drawn
		require
			a_width_nonnegative: a_width >= 0
			a_height_nonnegative: a_height >= 0
			a_line_width_positive: a_line_width > 0
			a_color_not_void: a_color /= Void
		do
			erase_rubber_box
			drawable.set_line_width (a_line_width)
			drawable.set_foreground_color (a_color)
			set_visible_rectangle (rectangle, drawable, a_x, a_y, a_width, a_height, a_line_width)
			drawable.draw_rectangle (rectangle.x, rectangle.y, rectangle.width, rectangle.height)
			rubber_box.move_and_resize (rectangle.x, rectangle.y, rectangle.width, rectangle.height)
		end

	draw_std_measurement_marker (a_point: CONST_POINT)
			-- Draw a measurement marker (cross) on the
			-- canvas whose center is at model location
			-- (`a_point'), using standard arm length,
			-- line width and color
		local
			coords: like model_point_view_coords
			l_x, l_y: INTEGER_64
		do
			coords := model_point_view_coords (a_point.x, a_point.y)
			l_x := coords.integer_64_item (1)
			l_y := coords.integer_64_item (2)
			if l_x >= 0 and l_x < width and l_y >= 0 and l_y < height then
				draw_measurement_marker (l_x.to_integer_32, l_y.to_integer_32, measure_marker_size_in_pixels, measure_marker_line_width, palette.measurement_marker_color)
			end
		end

	draw_measurement_marker (a_x, a_y, a_length: INTEGER a_line_width: INTEGER a_color: EV_COLOR)
			-- Draw a measurement marker (cross) on the canvas
			-- whose center is at (`a_x', `a_y') and whose arms
			-- are `a_length' long.  Draw it with lines
			-- `a_line_width' pixels thick using `a_color'
		require
			a_length_nonnegative: a_length >= 0
			a_line_width_positive: a_line_width > 0
			a_color_not_void: a_color /= Void
		do
			drawable.set_line_width (a_line_width)
			drawable.set_foreground_color (a_color)
			drawable.draw_segment (a_x - a_length, a_y, a_x + a_length, a_y)
			drawable.draw_segment (a_x, a_y - a_length, a_x, a_y + a_length)
		end

	initialize_selection_lists
			-- Make empty temporary lists which will be used
			-- to rebuild `elements' in stacking order
		local
			n: INTEGER
		do
			n := elements.count
			create {ARRAYED_LIST [like anchored_view_element]} unselectable_elements.make (n)
			create {ARRAYED_LIST [like anchored_view_element]} unselected_elements.make (n)
			create {ARRAYED_LIST [like anchored_view_element]} previously_selected_elements.make (n)
			create {ARRAYED_LIST [like anchored_view_element]} newly_selected_elements.make (n)
		end

	rebuild_elements_in_stacking_order
			-- Rebuild `elements' from temporary lists so that
			-- it is in stacking order
		do
			create {ARRAYED_LIST [like anchored_view_element]} elements.make (elements.count)
			elements.append (unselectable_elements)
			elements.append (unselected_elements)
			elements.append (previously_selected_elements)
			elements.append (newly_selected_elements)
		end

	set_selected_count (n: INTEGER)
			-- Set `selected_count' to `n'
		local
			old_count: INTEGER
		do
			old_count := selected_count
			selected_count := n
			if old_count /= selected_count then
				selected_count_change_actions.call ([selected_count])
			end
		end

	set_selection_bounding_box_empty
			-- Make `selection_bounding_box' be empty
		do
			selection_bounding_box.make_by_corners (0, 0, 0, 0)
		end

	set_changed_region_empty
			-- Make `changed_region' be empty.  Set items
			-- to not be included in the changed region
			-- prior to an operation that may change
			-- appearance or location of some items of
			-- view
		do
			initialize_changed_region
			apply_action_with_filter (all_filter, clear_in_changed_region_action)
		end

	initialize_changed_region
			-- Initialize `changed_region' and `changed_bounding_box'
		do
			changed_region := Void
			changed_region_count := 0
			changed_bounding_box := Void
		end

	redraw_cut_pattern_label
			-- Redraw only the cut label(s)
		do
			set_changed_region_empty
			apply_action_with_filter (cut_label_filter, add_to_changed_region_action)
			redraw_changed_region
		end

	redraw_changed_region
			-- Clear changed region on pixmap, draw view on
			-- pixmap (drawing only on changed region) and draw
			-- changed region of pixmap on drawable
		do
			if changed_region /= Void and then is_redraw_enabled then
				set_background_color_for_zoom
				if changed_region_count <= Changed_region_threshold then
					pixmap.set_clip_region (changed_region)
					is_clipping_set := True
					pixmap.clear
					draw_view_on_pixmap
					drawable.set_clip_region (changed_region)
					drawable.draw_pixmap (0, 0, pixmap)
					pixmap.remove_clipping
					drawable.remove_clipping
					is_clipping_set := False
				else
					redraw
				end
			end
		end

	create_copies_table
			-- Create `copies_table'
		do
			create copies_table.make (200)
			copies_table.compare_references
		end

	set_all_copies
			-- Set `all_copies' for every die in `elements'.
			-- `copies_table' is assumed to have been previously
			-- created
		local
			elem: like anchored_view_element
		do
			from
				elements.start
			until
				elements.after
			loop
				elem := elements.item
				if elem.is_die then
					find_copies (elem)
					last_element_list.extend (elem)
					elem.set_all_copies (last_element_list)
				end
				elements.forth
			end
		end

	rebuild_copies_table
			-- Rebuild `copies_table' with new detached dies
			-- after a Save, which also have new hash code values
		local
			elem: like anchored_view_element
			d: PACKING_DIES_MROW
		do
			create_copies_table
			from
				elements.start
			until
				elements.after
			loop
				elem := elements.item
				if elem.is_die then
					d := elem.die
					copies_table.search (d)
					if copies_table.found then
						if copies_table.found_item /= elem.all_copies then
							handle_internal_error ("different view elements for same die have different lists of all copies of die")
						end
					else
						copies_table.put (elem.all_copies, d)
					end
				end
				elements.forth
			end
		end

	find_copies (an_element: like anchored_view_element)
			-- Find list with all copies of `an_element' (may be
			-- only partially populated) and put in
			-- `last_element_list'.  If no list yet, create
			-- an empty one
		require
			an_element_not_void: an_element /= Void
		do
			copies_table.search (an_element.die)
			if copies_table.found then
				last_element_list := copies_table.found_item
			else
				create last_element_list.make (5)
				copies_table.put (last_element_list, an_element.die)
			end
		ensure
			last_element_list_not_void: last_element_list /= Void
		end

	set_current_view (a_named_view: RETICLE_NAMED_VIEW)
			-- Set currently displayed Named View
			-- to `a_named_view'
		do
			current_view := a_named_view
			current_label_generator := a_named_view.label_generator
			current_color_generator := a_named_view.color_generator
			current_fill_generator := a_named_view.fill_generator
			current_opacity_generator := a_named_view.opacity_generator
			named_view_change_actions.call ([a_named_view.name])
		end

	restore_state_and_redraw (a_change: RETICLE_CHANGE is_undo: BOOLEAN)
			-- Restore state at start or end of `a_change', where
			-- `is_undo' indicates whether this is an undo or a
			-- redo
		local
			sel_list: LIST [RETICLE_VIEW_ELEMENT]
			l_named_view: RETICLE_NAMED_VIEW
			l_locked_mode: BOOLEAN
			l_locked_mode_changed: BOOLEAN
		do
			if is_undo then
				sel_list := a_change.initial_selected_elements
				l_locked_mode := a_change.initial_locked_mode
				l_named_view := a_change.initial_named_view
			else
				sel_list := a_change.final_selected_elements
				l_locked_mode := a_change.final_locked_mode
				l_named_view := a_change.final_named_view
			end
			set_adjusted_displayed_model_area_without_history (a_change.displayed_area)
			l_locked_mode_changed := is_locked_mode /= l_locked_mode
			is_locked_mode := l_locked_mode
			if l_locked_mode_changed then
				locked_mode_change_actions.call ([is_locked_mode])
			end
			if l_named_view /= current_view then
				set_current_view (l_named_view)
				set_named_view_action (current_view)
				set_changed_region_empty
				apply_action_with_filter (all_filter, set_view_action)
			end
			set_selected_elements_from_list (sel_list)
			redraw
		end

	set_selected_elements_from_list (a_list: LIST [like anchored_view_element])
			-- Set selected elements to only those in `a_list'
		require
			a_list_not_void: a_list /= void
		do
			set_changed_region_empty
			apply_action_with_filter (all_filter, unselect_action)
			set_changed_region_empty
			apply_action (select_action, a_list)
			set_changed_region_empty
			apply_select_action (unchanged_selection_action)
		end

	set_operation_start_location (a_x, a_y: INTEGER)
			-- Set `operation_start_location' to (`a_x', `a_y')
		do
			operation_start_location.set_x (a_x)
			operation_start_location.set_y (a_y)
		end

	redraw_move_box (a_x, a_y: INTEGER)
			-- Redraw box for move of selected items at
			-- offset (`a_x', `a_y') in view coordinates
			-- from original position of selection bounding
			-- box
		local
			t: like temp_ev_rectangle
		do
			t := temp_ev_rectangle
			compute_move_box (t, a_x, a_y)
			redraw_move_style_rubber_box (t.x, t.y, t.width, t.height)
		end

	redraw_duplicate_box (a_x, a_y: INTEGER)
			-- Redraw box for duplication of selected items at
			-- offset (`a_x', `a_y') in view coordinates
			-- from original position of selection bounding
			-- box
		local
			t: like temp_ev_rectangle
		do
			t := temp_ev_rectangle
			compute_move_box (t, a_x, a_y)
			redraw_duplicate_style_rubber_box (t.x, t.y, t.width, t.height)
		end

	redraw_scribe_resize_box (a_x, a_y: INTEGER)
			-- Redraw box for resize of selected item
			-- (which is a scribe) at offset (`a_x',
			-- `a_y') in view coordinates from original
			-- position of mouse pointer when resize
			-- started
		local
			t: like temp_ev_rectangle
		do
			t := temp_ev_rectangle
			compute_scribe_resize_box (t, a_x, a_y)
			redraw_scribe_resize_style_rubber_box (t.x, t.y, t.width, t.height)
		end

	compute_move_box (a_rectangle: EV_RECTANGLE a_x, a_y: INTEGER)
			-- Set `a_rectangle' to rectangle for move or
			-- duplicate of selected items at offset
			-- (`a_x', `a_y') in view coordinates from
			-- original position of selection bounding box
		require
			a_rectangle_not_void: a_rectangle /= Void
		local
			r: like selection_bounding_box
			t: like a_rectangle
		do
			r := selection_bounding_box
			t := a_rectangle
			set_view_rectangle_from_model_rectangle (t, 2 * (pixmap.width.max (pixmap.height) + 1), r.low_x, r.high_y, r.high_x, r.low_y)
			t.move (t.x + a_x, t.y + a_y)
		end

	compute_scribe_resize_box (a_rectangle: EV_RECTANGLE a_x, a_y: INTEGER)
			-- Set `a_rectangle' to rectangle for resize of
			-- selected scribe at offset
			-- (`a_x', `a_y') in view coordinates from
			-- original position of mouse pointer when resize
			-- started
		require
			a_rectangle_not_void: a_rectangle /= Void
			selected_element_not_void: selected_element /= Void
			selected_element_is_scribe: selected_element.is_scribe
		local
			t: like a_rectangle
			diff_x, diff_y, diff_w, diff_h: INTEGER
		do
			t := selected_element.bounding_box
			if selected_element.is_horizontal_scribe then
				if is_left_or_bottom_resize then
					diff_x := a_x
					diff_w := -a_x
				else
					diff_w := a_x
				end
			else
				if is_left_or_bottom_resize then
					diff_h := a_y
				else
					diff_y := a_y
					diff_h := -a_y
				end
			end
			a_rectangle.move_and_resize (t.x + diff_x, t.y + diff_y, t.width + diff_w, t.height + diff_h)
		end

	set_limits_infinite
			-- Set distance limits for current operation to
			-- maximum values
		do
			up_limit := Infinity
			down_limit := Infinity
			left_limit := Infinity
			right_limit := Infinity
		end

	initialize_rotate_element_lists
			-- Set lists of elements which could not be rotated
			-- for various reasons
		do
			partially_selected_elements.wipe_out
			rotation_locked_elements.wipe_out
			fixed_orientation_elements.wipe_out
		end

	initialize_duplicate_element_lists
			-- Set lists of elements which could not be duplicated
			-- for various reasons
		do
			duplication_locked_elements.wipe_out
			never_duplicatable_elements.wipe_out
		end

	move_selected_items_by (a_move_type: STRING a_x, a_y: DOUBLE)
			-- Move selected objects by (`a_x', `a_y')
		require
			a_move_type_not_void: a_move_type /= Void
			a_x_on_grid: grid.is_edge_on_grid (a_x)
			a_y_on_grid: grid.is_edge_on_grid (a_y)
		local
			p: POINT
			n: INTEGER
		do
			initialize_status
			unmovable_elements.wipe_out
			create p.make_xy (a_x, a_y)
			move_relative_action.set_grid (grid)
			move_relative_action.set_requested_move_distance (p)
			move_relative_action.set_valid (True)
			move_relative_action.set_validate_only (True)
			apply_action_with_filter (selected_filter, move_relative_action)
			if not unmovable_elements.is_empty then
				n := unmovable_elements.count
				add_error (n.out + " " + su.number (n, "element", "elements") + " position-locked")
			elseif not move_relative_action.is_valid then
				add_error ("new object center would be too big to fit in database")
			end
			
			if not had_error then
				initialize_change
				set_selection_bounding_box_empty
				move_relative_action.set_validate_only (False)
				apply_action_with_filter_and_redraw (selected_filter, move_relative_action)
				move_distance := p
				set_final_selected_elements_to_initial
				add_change_to_history_list (new_standard_change (a_move_type))
			end
		end

	center_projects_in_cut_die (a_filter: VIEW_ELEMENT_FILTER)
			-- Center projects selected by `a_filter' in
			-- their cut die
		local
			n: INTEGER
		do
			initialize_status
			centered_count := 0
			compute_cut_dies
			build_scribe_cut_die_model_areas
			apply_action_with_filter (all_filter, clear_unisolated_in_scribe_cut_die_action)
			apply_action_with_filter (project_filter, compute_unisolated_in_scribe_cut_die_action)
			unmovable_elements.wipe_out
			center_in_cut_die_action.set_grid (grid)
			center_in_cut_die_action.set_valid (True)
			center_in_cut_die_action.set_validate_only (True)
			apply_action_with_filter (a_filter, center_in_cut_die_action)
			if not unmovable_elements.is_empty then
				n := unmovable_elements.count
				add_error (n.out + " " + su.number (n, "element", "elements") + " position-locked")
			elseif not center_in_cut_die_action.is_valid then
				add_error ("new object center would be too big to fit in database")
			end
			
			if not had_error then
				initialize_change
				set_selection_bounding_box_empty
				center_in_cut_die_action.set_validate_only (False)
				apply_action_with_filter_and_redraw (a_filter, center_in_cut_die_action)
				compute_selection_bounding_box
				centered_count := current_changes.count
				set_final_selected_elements_to_initial
				add_change_to_history_list (new_standard_change ("Center"))
			end
		end

	initialize_for_spread
			-- Do initialization before Spread operation
		do
			initialize_status
			set_selection_bounding_box_empty
			largest_spread_dimension := 0
			spread_elements.wipe_out
			unmovable_elements.wipe_out
		end

	spread_selected_items_horizontally (a_name: STRING a_coordinate: EV_COORDINATE a_comparator: KL_COMPARATOR [like anchored_view_element])
			-- Spread all currently selected items horizontally
			-- so that they do not overlap, sorted by
			-- `a_comparator'
		require
			a_name_not_void: a_name /= Void
			a_coordinate_not_void: a_coordinate /= Void
			a_comparator_not_void: a_comparator /= Void
		do
			spread_selected_items (a_name, a_coordinate, pre_horizontal_spread_action, horizontal_spread_action, a_comparator, vertical_scribe_width)
		end

	spread_selected_items_vertically (a_name: STRING a_coordinate: EV_COORDINATE a_comparator: KL_COMPARATOR [like anchored_view_element])
			-- Spread all currently selected items
			-- horizontally so that they do not overlap,
			-- sorted by `a_comparator' with lower left
			-- corner of bounding box at view coordinates
			-- `a_coordinate'
		require
			a_name_not_void: a_name /= Void
			a_coordinate_not_void: a_coordinate /= Void
			a_comparator_not_void: a_comparator /= Void
		do
			spread_selected_items (a_name, a_coordinate, pre_vertical_spread_action, vertical_spread_action, a_comparator, horizontal_scribe_width)
		end

	spread_selected_items (a_name: STRING a_coordinate: EV_COORDINATE a_pre_action: VIEW_ELEMENT_ACTION a_spread_action: SPREAD_ACTION a_comparator: KL_COMPARATOR [like anchored_view_element] a_spacing: DOUBLE)
			-- Spread all currently selected items apart
			-- so that they do not overlap sorted by width
			-- and place lower left corner of bounding box
			-- at view coordinates `a_coordinate'
		require
			selected_count_positive: selected_count > 0
			a_name_not_void: a_name /= Void
			a_coordinate_not_void: a_coordinate /= Void
			a_pre_action_not_void: a_pre_action /= Void
			a_spread_action_not_void: a_spread_action /= Void
			a_comparator_not_void: a_comparator /= Void
		local
			p: CONST_POINT
			start_point: POINT
		do
			initialize_for_spread
			apply_action_with_filter (selected_filter, a_pre_action)
			if is_move_valid then
				spread_elements := quick_sorter.sorted (spread_elements, a_comparator)
				p := pixel_upper_left_model_coords (a_coordinate.x, a_coordinate.y)
				create start_point.make_xy (p.x, p.y)
				a_spread_action.set_grid (grid)
				a_spread_action.set_spacing (a_spacing)
				a_spread_action.set_largest_dimension (largest_spread_dimension)
				a_spread_action.set_next_point (start_point.twin)
				a_spread_action.set_valid (True)
				a_spread_action.set_validate_only (True)
				apply_action (a_spread_action, spread_elements)
				if a_spread_action.is_valid then
					initialize_change
					set_changed_region_empty
					a_spread_action.set_next_point (start_point.twin)
					a_spread_action.set_validate_only (False)
					apply_action (a_spread_action, spread_elements)
					redraw_changed_region
					set_final_selected_elements_to_initial
					add_change_to_history_list (new_standard_change (a_name))
				else
					add_error ("Spread not allowed - new object center would be too big to fit in database")
				end
			end
		end

	scan_selected_items (a_name: STRING is_vert: BOOLEAN an_orientation: STRING a_pre_scan_action: PRE_SCAN_ACTION a_scan_action: SCAN_ACTION a_comparator: KL_COMPARATOR [like anchored_view_element])
			-- Scan currently selected items and space them out
			-- evenly in the appropriate dimension, taking into
			-- account items that are fixed in that dimension
			-- and the field of view edges
		require
			a_name_not_void: a_name /= Void
			an_orientation_not_void: an_orientation /= Void
			a_pre_scan_action_not_void: a_pre_scan_action /= Void
			a_comparator_not_void: a_comparator /= Void
		local
			list: LIST [like anchored_view_element]
			l_fields: LIST [RETICLE_FIELD]
			f: RETICLE_FIELD
			slots: LIST [SCAN_SLOT]
			n, move_count: INTEGER
		do
			initialize_status
			create {ARRAYED_LIST [like anchored_view_element]} list.make (selected_count)
			create {ARRAYED_LIST [RETICLE_FIELD]} l_fields.make (fields.count)
			a_pre_scan_action.initialize
			a_pre_scan_action.set_scan_elements (list)
			a_pre_scan_action.set_scan_fields (l_fields)
			apply_action_with_filter (selected_filter, a_pre_scan_action)
			n := a_pre_scan_action.wrong_orientation_count
			if n > 0 then
				add_error (n.out + " " + an_orientation + " " + su.number (n, "scribe", "scribes") + " included in scan")
			elseif list.is_empty then
				add_warning ("No objects included in " + an_orientation + " scan")
			else
				move_count := a_pre_scan_action.movable_count
				if move_count = 0 then
					add_warning ("No movable objects included in " + an_orientation + " scan")
				else
					if is_vert then
						n := a_pre_scan_action.scan_fields.count
						if n /= 1 then
							add_error ("included scan objects are in " + n.out + " different fields of view for a " + an_orientation + " scan")
						
						else
							f := a_pre_scan_action.scan_fields.first
						end
					else
						f := reticle_fields.first
					end
				end
			end
			if not had_error and move_count > 0 then
				list := sorter.sorted (list, a_comparator)
				create {ARRAYED_LIST [SCAN_SLOT]} slots.make (list.count)
				a_scan_action.set_grid (grid)
				a_scan_action.set_scan_slots (slots)
				a_scan_action.set_field_limits (f)
				a_scan_action.start
				apply_action (a_scan_action, list)
				a_scan_action.finish
			end
			if not had_error and move_count > 0 then
				-- Do the scan, using `slots'
				initialize_change_without_status
				set_changed_region_empty
				a_scan_action.do_scan
				redraw_changed_region
				compute_selection_bounding_box
				set_final_selected_elements_to_initial
				add_change_to_history_list (new_standard_change (a_name))
			end
		end

	align_selected_items (a_name: STRING a_coordinate: EV_COORDINATE an_align_action: ALIGN_ACTION)
			-- Align the currently selected items to the
			-- object containing the mouse pointer, using
			-- `a_name' as the name of the operation for history
			-- list
		require
			a_name_not_void: a_name /= Void
			a_coordinate_not_void: a_coordinate /= Void
		local
			n: INTEGER
		do
			initialize_status
			if selected_count = 0 then
				add_error ("no objects selected")
			end
			if not had_error then
				containing_elements.wipe_out
				pre_align_action.set_selection_point (a_coordinate)
				pre_align_action.set_containing_elements (containing_elements)
				apply_action_with_filter (all_filter, pre_align_action)
				n := containing_elements.count
				if n = 0 then
					add_error ("no align target (mouse pointer not inside any objects)")
				elseif n > 1 then
					add_error ("align target is ambiguous (mouse pointer inside " + n.out + " objects)")
				else
					align_target := containing_elements.first
				end
			end
			
			if not had_error then
				unmovable_elements.wipe_out
				an_align_action.set_align_target (align_target)
				an_align_action.set_grid (grid)
				an_align_action.set_valid (True)
				an_align_action.set_validate_only (True)
				apply_action_with_filter (selected_filter, an_align_action)
				if not unmovable_elements.is_empty then
					n := unmovable_elements.count
					add_error (n.out + " " + su.number (n, "element", "elements") + " position-locked")
				elseif not an_align_action.is_valid then
					add_error ("new object center would be too big to fit in database")
				end
			end
			if not had_error then
				initialize_change
				set_selection_bounding_box_empty
				an_align_action.set_validate_only (False)
				apply_action_with_filter_and_redraw (selected_filter, an_align_action)
				set_final_selected_elements_to_initial
				add_change_to_history_list (new_standard_change (a_name))
			end
		end

	pick_abut_target
			-- Pick `abut_target' based on `abut_source',
			-- `containing_elements' and `target_field'
		require
			abut_source_not_void: abut_source /= Void
			containing_elements_not_void: containing_elements /= Void
		local
			l_target: like abut_target
			n: INTEGER
		do
			abut_target := Void
			n := containing_elements.count
			if n = 1 then
				l_target := containing_elements.first
			elseif n > 1 then
				if containing_elements.has (abut_source) then
					containing_elements.prune (abut_source)
					l_target := containing_elements.first
					n := n - 1
				end
			end
			if abut_source.is_center_in_field then
				if target_field /= Void and then abut_source.field = target_field then
					-- Mouse pointer was inside same field
					-- as `abut_source'
					if n = 0 then
						add_error ("pointer position not inside valid abut target")
					elseif n > 1 then
						add_error ("abut target is ambiguous (pointer in " + containing_elements.count.out + " objects)")
					elseif abut_source = l_target then
						add_error ("cannot abut object to itself")
					else
						abut_target := l_target
					end
				else
					-- Mouse pointer was outside field
					-- containing `abut_source' - abut
					-- to source field
					abut_target := abut_source.field
				end
			else
				if target_field /= Void then
					-- Mouse pointer was inside a field
					abut_target := target_field
				elseif n = 0 then
					add_error ("pointer position not inside valid abut target")
				elseif n > 1 then
					add_error ("abut target is ambiguous (pointer in " + containing_elements.count.out + " objects)")
				elseif abut_source = l_target then
					add_error ("cannot abut object to itself")
				else
					abut_target := l_target
				end
			end
		end

	set_abut_move_distance (a_coordinate: EV_COORDINATE)
			-- Set `abut_move_distance' to distance in X
			-- and Y `abut_source' needs to be moved,
			-- where `a_coordinate' is mouse pointer
			-- coordinates when Abut operation was
			-- requested.  Add error if not valid
		require
			a_coordinate_not_void: a_coordinate /= Void
			abut_source_not_void: abut_source /= Void
			abut_target_not_void: abut_target /= Void
		do
			if abut_target.is_field then
				set_abut_move_distance_for_field (a_coordinate)
			else
				set_abut_move_distance_for_die
			end
		end

	set_abut_move_distance_for_field (a_coordinate: EV_COORDINATE)
			-- Set `abut_move_distance' to distance in X
			-- and Y `abut_source' needs to be moved,
			-- where `a_coordinate' is mouse pointer
			-- coordinates when Abut operation was
			-- requested and `abut_target' is a field.
			-- Add error if not valid
		require
			a_coordinate_not_void: a_coordinate /= Void
			abut_source_not_void: abut_source /= Void
			abut_target_not_void: abut_target /= Void
			abut_target_is_field: abut_target.is_field
		local
			p: like a_coordinate
			field_mbb, source_mbb: EV_RECTANGLE
			l_x, l_y: DOUBLE
		do
			field_mbb := abut_target.bounding_box
			if abut_source.is_center_in_field then
				-- Use pointer position to decide which
				-- side(s) of field to abut to
				p := a_coordinate
			else
				-- Use position of center of
				-- `abut_source' to decide which
				-- side(s) of field to abut to
				source_mbb := abut_source.bounding_box
				create p.make_precise ((source_mbb.right + source_mbb.left) / 2.0, (source_mbb.top + source_mbb.bottom) / 2.0)
			end

			if p.x < field_mbb.left then
				-- Abut source to field left side
				l_x := abut_target.left_x - abut_source.left_x
			elseif p.x > field_mbb.right then
				-- Abut source to field right side
				l_x := abut_target.right_x - abut_source.right_x
			else
				-- Do not abut source in X
				l_x := 0
			end
			if p.y < field_mbb.top then
				-- Abut source to field top side
				l_y := abut_target.top_y - abut_source.top_y
			elseif p.y > field_mbb.bottom then
				-- Abut source to field bottom side
				l_y := abut_target.bottom_y - abut_source.bottom_y
			else
				-- Do not abut source in Y
				l_y := 0
			end
			abut_move_distance.make_xy (grid.rounded_edge (l_x), grid.rounded_edge (l_y))
		end

	set_abut_move_distance_for_die
			-- Set `abut_move_distance' to distance in X
			-- and Y `abut_source' needs to be moved
			-- when `abut_target' is a die.
			-- Add error if not valid
		require
			abut_source_not_void: abut_source /= Void
			abut_target_not_void: abut_target /= Void
			abut_target_not_field: not abut_target.is_field
		local
			l_x, l_y: DOUBLE
			overlap_x, overlap_y: BOOLEAN
			overlap_x_subset, overlap_y_subset: BOOLEAN
			abut_x_allowed, abut_y_allowed: BOOLEAN
			source_left, source_right, source_top, source_bottom: DOUBLE
			target_left, target_right, target_top, target_bottom: DOUBLE
			source_center_x, source_center_y: DOUBLE
			target_center_x, target_center_y: DOUBLE
			is_src_hscribe, is_src_vscribe, is_src_die: BOOLEAN
			is_targ_hscribe, is_targ_vscribe, is_targ_die: BOOLEAN
			to_left_right, to_top_bottom: BOOLEAN
		do
			source_left := abut_source.left_x
			source_right := abut_source.right_x
			source_top := abut_source.top_y
			source_bottom := abut_source.bottom_y
			source_center_x := abut_source.center_x
			source_center_y := abut_source.center_y
			
			target_left := abut_target.left_x
			target_right := abut_target.right_x
			target_top := abut_target.top_y
			target_bottom := abut_target.bottom_y
			target_center_x := abut_target.center_x
			target_center_y := abut_target.center_y
			
			is_src_hscribe := abut_source.is_horizontal_scribe
			is_src_vscribe := abut_source.is_vertical_scribe
			is_src_die := not is_src_hscribe and not is_src_vscribe
			is_targ_hscribe := abut_target.is_horizontal_scribe
			is_targ_vscribe := abut_target.is_vertical_scribe
			is_targ_die := not is_targ_hscribe and not is_targ_vscribe

			overlap_x := source_left < target_right and source_right > target_left
			overlap_y := source_bottom < target_top and source_top > target_bottom

			overlap_x_subset := is_subinterval (source_left, source_right, target_left, target_right)
			overlap_y_subset := is_subinterval (source_bottom, source_top, target_bottom, target_top)

			abut_x_allowed := not ((is_src_hscribe and is_targ_die) or (is_src_die and is_targ_hscribe) or (is_src_vscribe and is_targ_vscribe))
			abut_y_allowed := not ((is_src_vscribe and is_targ_die) or (is_src_die and is_targ_vscribe) or (is_src_hscribe and is_targ_hscribe))

			to_left_right := False
			to_top_bottom := False
			if abut_x_allowed and abut_y_allowed then
				if overlap_x and overlap_y then
					if overlap_x_subset and not overlap_y_subset then
						-- Abut source to top/bottom of target
						to_top_bottom := True
					elseif overlap_y_subset and not overlap_x_subset then
						-- Abut source to left/right of target
						to_left_right := True
					else
						add_error ("abut target side is ambiguous")
					end
				elseif overlap_x then
					-- Abut source to top/bottom of target
					to_top_bottom := True
				elseif overlap_y then
					-- Abut source to left/right of target
					to_left_right := True
				else
					add_error ("abut target side is ambiguous")
				end
			elseif abut_x_allowed then
				-- Abut source to left/right of target
				to_left_right := True
			elseif abut_y_allowed then
				-- Abut source to top/bottom of target
				to_top_bottom := True
			else
				handle_internal_error ("abut not allowed in X or Y for die to die abut operation")
			end

			l_x := 0.0
			l_y := 0.0
			if not had_error then
				if to_left_right then
					if source_center_x < target_center_x then
						l_x := target_left - source_right
					else
						l_x := target_right - source_left
					end
				elseif to_top_bottom then
					if source_center_y < target_center_y then
						l_y := target_bottom - source_top
					else
						l_y := target_top - source_bottom
					end
				else
					handle_internal_error ("abut allowed but not to left/right or top/bottom")
				end
			end
			abut_move_distance.make_xy (grid.rounded_edge (l_x), grid.rounded_edge (l_y))
		end
	
	check_abut_lock_restrictions
			-- Issue error if source is locked in dimension
			-- that Abut wants to move it
		local
			locked_x, locked_y: BOOLEAN
			dist_x, dist_y: DOUBLE
		do
			locked_x := abut_source.is_locked_in_x
			locked_y := abut_source.is_locked_in_y
			dist_x := abut_move_distance.x
			dist_y := abut_move_distance.y
			if locked_x and locked_y then
				if dist_x /= 0.0 or dist_y /= 0.0 then
					add_error ("source object is locked in X and Y")
				end
			elseif locked_x then
				if dist_x /= 0.0 then
					add_error ("source object is locked in X")
				end
			elseif locked_y then
				if dist_y /= 0.0 then
					add_error ("source object is locked in Y")
				end
			end
		end
	
	trim_scribes (a_filter: VIEW_ELEMENT_FILTER a_redraw_requested: BOOLEAN)
			-- Resize objects selected by `a_filter' that
			-- are scribes that intersect at least one
			-- field of view so that ends which are
			-- outside any field of view are "trimmed"
			-- back to field of view edge.
			-- Redraw the result if `a_redraw_requested'
		do
			if a_redraw_requested then
				initialize_change
			end
			set_selection_bounding_box_empty
			create {ARRAYED_LIST [like anchored_view_element]} changed_scribes.make (elements.count)
			create {ARRAYED_LIST [like anchored_view_element]} locked_scribes.make (elements.count)
			trim_scribe_action.set_grid (grid)
			if a_redraw_requested then
				apply_action_with_filter_and_redraw (a_filter, trim_scribe_action)
				set_final_selected_elements_to_initial
				add_change_to_history_list (new_standard_change ("Trim Scribe"))
			else
				set_changed_region_empty
				apply_action_with_filter (a_filter, trim_scribe_action)
			end
		end

	separate_scribes (a_filter: VIEW_ELEMENT_FILTER a_redraw_requested: BOOLEAN)
			-- Replace every scribe selected by `a_filter'
			-- that intersects more than one field of view
			-- with a set of scribes that represents the
			-- intersection of the scribe with each field.
			-- Redraw the result if `a_redraw_requested'
		do
			if a_redraw_requested then
				initialize_change
			end
			create {ARRAYED_LIST [like anchored_view_element]} changed_scribes.make (elements.count)
			create {ARRAYED_LIST [like anchored_view_element]} locked_scribes.make (elements.count)
			create {ARRAYED_LIST [like anchored_view_element]} deleted_elements.make (elements.count)
			create {ARRAYED_LIST [PACKING_DIES_MROW]} deleted_scribes.make (elements.count)
			separate_scribe_action.set_filter (a_filter)
			if a_redraw_requested then
				apply_select_action_and_redraw (separate_scribe_action)
				set_final_selected_elements
				add_change_to_history_list (new_creation_and_deletion_change ("Separate Scribe"))
			else
				set_changed_region_empty
				apply_select_action (separate_scribe_action)
			end
			remove_deleted_scribes
		end

	remove_deleted_scribes
			-- Remove any deleted scribes from model
		do
			if not deleted_elements.is_empty then
				model.remove_dies (deleted_scribes)
				update_copies_table_for_delete (deleted_elements)
				model.set_from_element_list (model_elements)
			end
		end

	extend_scribes (selected_only: BOOLEAN a_redraw_requested: BOOLEAN)
			-- Extend either selected scribes (if
			-- `selected_only') or every scribe to meet a
			-- scribe of different orientation or field of
			-- view edge.  Redraw the result if
			-- `a_redraw_requested'.  Only does one
			-- iteration
		do
			if a_redraw_requested then
				initialize_change
			end
			create {ARRAYED_LIST [like anchored_view_element]} changed_scribes.make (elements.count)
			create {ARRAYED_LIST [like anchored_view_element]} locked_scribes.make (elements.count)
			
			-- Extend horizontal scribes
			
			build_field_edge_lists
			extend_scribe_action.initialize
			extend_scribe_action.set_selected_only (selected_only)
			extend_scribe_action.set_is_horizontal (True)
			extend_scribe_action.set_grid (grid)
			extend_scribe_action.set_field_list (fields_and_edges)
			apply_action_with_filter (horizontal_scribe_filter, extend_scribe_action)
			
			-- Extend vertical scribes scribes
			
			build_field_edge_lists
			extend_scribe_action.set_is_horizontal (False)
			extend_scribe_action.set_field_list (fields_and_edges)
			apply_action_with_filter (vertical_scribe_filter, extend_scribe_action)
			compute_selection_bounding_box
			if a_redraw_requested then
				redraw
				set_final_selected_elements_to_initial
				add_change_to_history_list (new_standard_change ("Extend Scribe"))
			end
		end

	unoverlap_scribes (selected_only: BOOLEAN a_redraw_requested: BOOLEAN)
			-- Unoverlap either selected scribes (if
			-- `selected_only') or every scribe by pulling
			-- scribes back to edge of another scribe of
			-- opposite orientation, deleting scribe if
			-- contained in another of opposite
			-- orientation or breaking a scribe into two
			-- scribes and then pulling back each one.
			-- Redraw the result if `a_redraw_requested'
		local
			horiz_scribes: LIST [like anchored_view_element]
			vert_scribes: LIST [like anchored_view_element]
			h_filter, v_filter: VIEW_ELEMENT_FILTER
			l_sorter: LIST_QUICK_SORTER [like anchored_view_element]
			x_comparator: LEFT_X_COMPARATOR [like anchored_view_element]
			y_comparator: BOTTOM_Y_COMPARATOR [like anchored_view_element]
		do
			if a_redraw_requested then
				initialize_change
			end
			clear_overlap_counts
			create {ARRAYED_LIST [like anchored_view_element]} changed_scribes.make (elements.count)
			create {ARRAYED_LIST [like anchored_view_element]} locked_scribes.make (elements.count)
			create {ARRAYED_LIST [like anchored_view_element]} deleted_elements.make (elements.count)
			create {ARRAYED_LIST [PACKING_DIES_MROW]} deleted_scribes.make (elements.count)
			create {ARRAYED_LIST [like anchored_view_element]} new_elements.make (elements.count)
			build_field_edge_lists
			
			-- Create lists of horizontal and vertical scribes
			-- (all or selected)
			
			create {ARRAYED_LIST [like anchored_view_element]} horiz_scribes.make (elements.count)
			create {ARRAYED_LIST [like anchored_view_element]} vert_scribes.make (elements.count)
			
			if selected_only then
				h_filter := selected_int_horizontal_scribe_filter
				v_filter := selected_int_vertical_scribe_filter
			else
				h_filter := int_horizontal_scribe_filter
				v_filter := int_vertical_scribe_filter
			end
			extend_list_action.set_list (horiz_scribes)
			apply_action_with_filter (h_filter, extend_list_action)
			extend_list_action.set_list (vert_scribes)
			apply_action_with_filter (v_filter, extend_list_action)

			-- Sort lists of horizontal and vertical scribes
			-- by bottom edge and left edge, respectively
			
			create l_sorter
			create x_comparator
			create y_comparator
			horiz_scribes := l_sorter.sorted (horiz_scribes, y_comparator)
			vert_scribes := l_sorter.sorted (vert_scribes, x_comparator)
			
			unmark_all
			
			unoverlap_scribe_action.set_grid (grid)
			unoverlap_scribe_action.set_field_list (fields_and_edges)
			unoverlap_scribe_action.set_vertical_scribes (vert_scribes)
			apply_action (unoverlap_scribe_action, horiz_scribes)
			
			elements.append (new_elements)
			apply_select_action (remove_marked_with_select_action)
			remove_deleted_scribes
			if a_redraw_requested then
				redraw
				set_final_selected_elements
				add_change_to_history_list (new_creation_and_deletion_change ("Unoverlap Scribe"))
			end
		end

	clear_overlap_counts
			-- Clear counts for Unoverlap operation
		do
			overlap_tee_count := 0
			overlap_cross_count := 0
			overlap_enclosed_count := 0
			overlap_removed_count := 0
			overlap_resize_failed_count := 0
			overlap_delete_failed_count := 0
		end
	
	initialize_floating_scribe_filter
			-- Build list of scribe and field edges and put it
			-- in floating scribe filter
		do
			build_field_edge_lists
			floating_scribe_filter.set_grid (grid)
			floating_scribe_filter.set_field_list (fields_and_edges)
		end
	
	build_field_edge_lists
			-- Build sorted lists of horizontal and vertical object
			-- edges for each field of view
		local
			edges: FIELD_ELEMENT_EDGES
			l_fields: like reticle_fields
		do
			l_fields := reticle_fields
			create {ARRAYED_LIST [FIELD_ELEMENT_EDGES]} fields_and_edges.make (l_fields.count)
			from
				l_fields.start
			until
				l_fields.after
			loop
				create edges.make (l_fields.item, elements.count * 2, grid)
				edges.set_grid (grid)
				fields_and_edges.extend (edges)
				l_fields.forth
			end
			build_edge_list_action.set_grid (grid)
			build_edge_list_action.set_field_list (fields_and_edges)
			apply_action_with_filter (all_filter, build_edge_list_action)
			-- Sort edge lists for each field

			from
				fields_and_edges.start
			until
				fields_and_edges.after
			loop
				fields_and_edges.item.sort_edges
				fields_and_edges.forth
			end
		end

	compute_selection_bounding_box
			-- Compute bounding box of selected objects
		do
			set_selection_bounding_box_empty
			apply_action_with_filter (all_filter, selection_changed_action)
		end

	call_change_history_list_actions
			-- Call `change_history_list_position_actions'
		local
			list: like reticle_change_history_list
		do
			list := reticle_change_history_list
			change_history_list_position_actions.call ([list.is_empty or list.before or list.isfirst, list.is_empty or list.after])
		end

	between (a, b, c: DOUBLE): BOOLEAN
			-- Is `a' between `b' and `c'?
		do
			Result := a >= b and a <= c
		end

	is_subinterval (a, b, c, d: DOUBLE): BOOLEAN
			-- Is interval (`a', `b') a sub-interval of (`c', `d')
			-- or vice versa?
		do
			Result := (between (a, c, d) and between (b, c, d)) or
				(between (c, a, b) and between (d, a, b))
		end

	compute_possible_wafer_etest_names
			-- Compute lists of wafer etest IDs and wafer etest
			-- names rows that are compatible with
			-- at least one element of view
		local
			ids: ARRAYED_LIST [STRING]
			names: ARRAYED_LIST [WAFER_ETEST_NAMES_ROW]
		do
			create ids.make (5)
			create names.make (5)
			compute_wafer_etest_ids_action.set_wafer_etest_ids (ids)
			compute_wafer_etest_ids_action.set_wafer_etest_names (names)
			if is_cut_mode then
				apply_action (compute_wafer_etest_ids_action, pack_mode_context.elements)
			else
				apply_action_with_filter (all_filter, compute_wafer_etest_ids_action)
			end
		end

	compute_assigned_wafer_etest_names
			-- Compute lists of wafer etest IDs and wafer
			-- etest names rows that are assigned to at
			-- least one element of view
		local
			ids: ARRAYED_LIST [STRING]
			names: ARRAYED_LIST [WAFER_ETEST_NAMES_ROW]
		do
			create ids.make (5)
			create names.make (5)
			compute_assigned_etest_ids_action.set_wafer_etest_ids (ids)
			compute_assigned_etest_ids_action.set_wafer_etest_names (names)
			if is_cut_mode then
				apply_action (compute_assigned_etest_ids_action, pack_mode_context.elements)
			else
				apply_action_with_filter (all_filter, compute_assigned_etest_ids_action)
			end
		end

	areas_per_side: INTEGER
			-- Number of areas to divide model into per side,
			-- to speed up validity checking operations by
			-- only comparing objects that intersect same area.
			-- Total number of areas is square of this value
		do
			-- FIXME: implement better based on element count
			Result := 5
		end

	strips_per_side: INTEGER
			-- Number of strip to divide model into on one side,
			-- to speed up validity checking operations by
			-- only comparing objects that intersect same area.
			-- Total number of areas is this value
		do
			-- FIXME: implement better based on element count
			Result := 10
		end

	build_mbb_model_areas
			-- Build areas of model and the elements that
			-- intersect them, to speed up validity
			-- checking operations.  Also, for each element
			-- build list of areas it intersects
		local
			num_per_side, area_count: INTEGER
		do
			num_per_side := areas_per_side
			area_count := num_per_side * num_per_side
			create {ARRAYED_LIST [RETICLE_MODEL_AREA]} mbb_model_areas.make (area_count)
			build_model_areas (num_per_side, num_per_side, 2 * minimum_separation, mbb_model_areas)

			clear_intersecting_mbb_areas_action.set_capacity (4)
			apply_action_with_filter (all_filter, clear_intersecting_mbb_areas_action)
			apply_action_with_filter (die_filter, populate_intersecting_mbb_areas_action)
		end

	build_scribe_cut_die_model_areas
			-- Build areas of model and the elements whose
			-- cut die (determined by surrounding scribes)
			-- intersects them, to speed up validity
			-- checking operations.  Also, for each
			-- element build list of areas its scribe cut die
			-- intersects
		local
			num_per_side, area_count: INTEGER
		do
			num_per_side := areas_per_side
			area_count := num_per_side * num_per_side
			create {ARRAYED_LIST [RETICLE_MODEL_AREA]} scribe_cut_die_model_areas.make (area_count)
			build_model_areas (num_per_side, num_per_side, 2 * minimum_separation, scribe_cut_die_model_areas)

			clear_intersecting_scribe_cut_die_areas_action.set_capacity (4)
			apply_action_with_filter (all_filter, clear_intersecting_scribe_cut_die_areas_action)
			apply_action_with_filter (die_filter, populate_intersecting_scribe_cut_die_areas_action)
		end

	build_cut_cut_die_model_areas
			-- Build areas of model and the elements whose
			-- cut die (determined by surrounding cuts)
			-- intersects them, to speed up validity
			-- checking operations.  Also, for each
			-- element build list of areas its cut cut die
			-- intersects
		local
			num_per_side, area_count: INTEGER
		do
			num_per_side := areas_per_side
			area_count := num_per_side * num_per_side
			create {ARRAYED_LIST [RETICLE_MODEL_AREA]} cut_cut_die_model_areas.make (area_count)
			build_model_areas (num_per_side, num_per_side, 2 * minimum_separation, cut_cut_die_model_areas)

			clear_intersecting_cut_cut_die_areas_action.set_capacity (4)
			apply_action_with_filter (all_filter, clear_intersecting_cut_cut_die_areas_action)
			apply_action_with_filter (non_scribe_filter, populate_intersecting_cut_cut_die_areas_action)
		end

	build_scribe_model_areas
			-- Build areas of model and elements that are
			-- horizontal or vertical scribes that intersect
			-- them
		do
			build_horizontal_scribe_model_areas
			build_vertical_scribe_model_areas
		end

	build_horizontal_scribe_model_areas
			-- Build areas of model and the elements that
			-- are horizontal scribes that intersect them,
			-- to speed up validity checking operations.
			-- Also, for each element build list of areas
			-- it intersects
		local
			row_count, col_count, area_count: INTEGER
		do
			row_count := strips_per_side
			col_count := 1
			area_count := row_count
			create {ARRAYED_LIST [RETICLE_MODEL_AREA]} horizontal_scribe_model_areas.make (area_count)
			build_model_areas (row_count, col_count, 2 * minimum_separation, horizontal_scribe_model_areas)

			clear_intersecting_hscribe_areas_action.set_capacity (2)
			apply_action_with_filter (all_filter, clear_intersecting_hscribe_areas_action)
			apply_action_with_filter (horizontal_scribe_filter, populate_intersecting_hscribe_areas_action)
		end

	build_vertical_scribe_model_areas
			-- Build areas of model and the elements that
			-- are vertical scribes that intersect them,
			-- to speed up validity checking operations.
			-- Also, for each element build list of areas
			-- it intersects
		local
			row_count, col_count, area_count: INTEGER
		do
			row_count := 1
			col_count := strips_per_side
			area_count := col_count
			create {ARRAYED_LIST [RETICLE_MODEL_AREA]} vertical_scribe_model_areas.make (area_count)
			build_model_areas (row_count, col_count, 2 * minimum_separation, vertical_scribe_model_areas)

			clear_intersecting_vscribe_areas_action.set_capacity (2)
			apply_action_with_filter (all_filter, clear_intersecting_vscribe_areas_action)
			apply_action_with_filter (vertical_scribe_filter, populate_intersecting_vscribe_areas_action)
		end

	build_model_areas (row_count, col_count: INTEGER extra_per_side: DOUBLE areas: like mbb_model_areas)
			-- Build areas of model in `areas'
		require
			row_count_positive: row_count > 0
			col_count_positive: col_count > 0
			areas_not_void: areas /= void
		local
			n, k, area_count: INTEGER
			mbb: CONST_RECTANGLE
			dist_x, dist_y, l, r, t, b: DOUBLE
			base: DOUBLE
			area: RETICLE_MODEL_AREA
			rect: RECTANGLE
		do
			area_count := row_count * col_count
			mbb := fields_bounding_box
			dist_x := mbb.size_x / col_count
			dist_y := mbb.size_y / row_count
			from
				n := 1
			until
				n > col_count
			loop
				base := mbb.low_x + (n - 1) * dist_x
				l := base - extra_per_side
				r := base + dist_x + extra_per_side
				from
					k := 1
				until
					k > row_count
				loop
					base := mbb.low_y + (k - 1) * dist_y
					b := base - extra_per_side
					t := base + dist_y + extra_per_side
					create rect.make_by_corners (l, b, r, t)
					create area.make (rect, 2 * elements.count // area_count)
					areas.extend (area)
					k := k + 1
				end
				n := n + 1
			end
		end

	fetch_fields
			-- Compute list of fields
		do
			create {ARRAYED_LIST [like anchored_view_element]} cached_fields.make (5)
			apply_action_with_filter (all_filter, compute_fields_action)
		end

	build_no_cut_patterns
			-- Setup view when switch into Cut mode
			-- (no cut patterns)
		require
			is_cut_mode: is_cut_mode
		do
			model.build_no_cut_patterns
			create {ARRAYED_LIST [like anchored_view_element]} elements.make (1)
			create displayed_area_history_list.make
			create reticle_change_history_list.make
			set_initial_model_elements (model.elements, False)
			no_cut_patterns_context := current_context_copy
		end

	build_cut_pattern (n: INTEGER)
			-- Build cut pattern `n' and leave in 
			-- `last_cut_pattern'
		local
			pat: RETICLE_VIEW_CUT_PATTERN
		do
			model.build_cut_pattern (n, pack_mode_context.model_context.elements)
			create {ARRAYED_LIST [like anchored_view_element]} elements.make (model.elements.count)
			create displayed_area_history_list.make
			create reticle_change_history_list.make
			set_initial_model_elements (model.elements, False)
			create pat.make (model.current_cut_pattern, Current)
			pat.set_context (current_context_copy)
			last_cut_pattern := pat
		end

	insert_cut_pattern (a_pattern: like last_cut_pattern)
			-- Insert `a_pattern' at correct point in 
			-- `cut_patterns', based on its cut_pattern_no
		local
			pat_no: INTEGER
		do
			pat_no := a_pattern.cut_pattern_no
			from
				cut_patterns.start
			until
				cut_patterns.after or else cut_patterns.item.cut_pattern_no > pat_no
			loop
				cut_patterns.forth
			end
			if cut_patterns.after then
				cut_patterns.extend (a_pattern)
			else
				cut_patterns.put_left (a_pattern)
				cut_patterns.back
			end
		end

	delete_current_cut_pattern_without_redraw
			-- Delete the current cut pattern but do not redraw
			-- the new current cut pattern (or no cut patterns
			-- label)
		require
			is_cut_mode: is_cut_mode
			has_current_cut_pattern: has_current_cut_pattern
		do
			set_validity_color_generator (no_validity_color_generator)
			model.remove_cut_pattern (current_cut_pattern.cut_pattern_no)
			cut_patterns.remove
			if not cut_patterns.is_empty and cut_patterns.after then
				cut_patterns.back
			end
		end

	remove_cut_patterns
			-- Remove all cut patterns
		do
			cut_patterns := Void
			model.remove_cut_patterns
		end

	update_current_cut_pattern_context
			-- Update context for current cut pattern so that
			-- its state will be restored correctly if user
			-- switches back to it
		require
			is_cut_mode: is_cut_mode
			has_current_cut_pattern: has_current_cut_pattern
		local
			l_context: RETICLE_VIEW_CONTEXT
		do
			l_context := current_cut_pattern.context
			l_context.set_view (current_view)
			set_initial_selected_elements
			l_context.set_elements (elements)
			l_context.set_selected_elements (initial_selected_elements)
			l_context.set_displayed_model_area (displayed_model_area.twin)
			l_context.set_model_context (model.current_context_copy)
		end

	show_current_cut_pattern
			-- Display current cut pattern
		require
			is_cut_mode: is_cut_mode
			has_current_cut_pattern: has_current_cut_pattern
		do
			set_context (current_cut_pattern.context)
			redraw
		end

	show_no_cut_patterns
			-- Display "no cut patterns"
		require
			is_cut_mode: is_cut_mode
		do
			set_context (no_cut_patterns_context)
			redraw
		end

	next_field_name (a_field_name: STRING): STRING
			-- Name of next field in sequence after `a_field_name'
			-- (first field name if `a_field_name' is last)
		require
			a_field_name_not_void: a_field_name /= Void
		local
			list: like reticle_fields
		do
			list := reticle_fields
			from
				list.start
			until
				list.after or else list.item.name.is_equal (a_field_name)
			loop
				list.forth
			end
			if list.islast then
				Result := list.first.name
			else
				list.forth
				Result := list.item.name
			end
		end

	clear_cut_pattern_change_histories
			-- Clear the undo/redo history list for every cut
			-- pattern
		local
			c: CURSOR
		do
			c := cut_patterns.cursor
			from
				cut_patterns.start
			until
				cut_patterns.after
			loop
				cut_patterns.item.clear_change_history_list
				cut_patterns.forth
			end
			cut_patterns.go_to (c)
		end

	set_scribe_includable_filters (a_value: BOOLEAN)
			-- Set `scribes_included' in filters where can
			-- included or exclude scribes to `a_value'.
			-- Need since some validity checking variants
			-- want to ignore certain validity issues for
			-- scribes
		do
			partially_outside_fields_alt_filter.set_scribes_included (a_value)
			too_close_to_edge_alt_filter.set_scribes_included (a_value)
			small_scribe_and_filler_alt_filter.set_scribes_included (a_value)
		end

	all_pack_validity_sort_filters: ARRAY [VIEW_ELEMENT_FILTER]
			-- Filters used to sort elements to affect
			-- display order (which elements appear on
			-- top of others).  Elements matching earlier
			-- filters are displayed on top of elements
			-- matching only a later filter in list
		once
			Result := <<
				outside_fields_filter,
				partially_outside_fields_alt_filter,
				overlaps_other_alt_filter,
				size_too_small_filter,
				wafer_etest_id_unassigned_filter,
				unlocked_or_unassigned_filter,
				too_close_to_edge_alt_filter,
				too_close_to_object_alt_filter,
				scribe_cut_die_too_big_filter,
				unisolated_in_cut_die_alt_filter,
				uncentered_in_cut_die_filter,
				floating_scribe_alt_filter,
				misaligned_scribe_alt_filter,
				small_scribe_and_filler_alt_filter
				>>
		end

	almost_all_pack_validity_sort_filters: ARRAY [VIEW_ELEMENT_FILTER]
			-- Filters used to sort elements to affect
			-- display order (which elements appear on
			-- top of others).  Omits filter for objects
			-- outside all fields.  Elements matching earlier
			-- filters are displayed on top of elements
			-- matching only a later filter in list
		once
			Result := <<
				partially_outside_fields_alt_filter,
				overlaps_other_alt_filter,
				size_too_small_filter,
				wafer_etest_id_unassigned_filter,
				unlocked_or_unassigned_filter,
				too_close_to_edge_alt_filter,
				too_close_to_object_alt_filter,
				scribe_cut_die_too_big_filter,
				unisolated_in_cut_die_alt_filter,
				uncentered_in_cut_die_filter,
				floating_scribe_alt_filter,
				misaligned_scribe_alt_filter,
				small_scribe_and_filler_alt_filter
				>>
		end

	before_join_scribes_validity_sort_filters: ARRAY [VIEW_ELEMENT_FILTER]
			-- Filters used to sort elements to affect
			-- display order (which elements appear on top
			-- of others).  Omits filter for objects
			-- outside all fields and filter for floating
			-- scribes.  Elements matching earlier filters
			-- are displayed on top of elements matching
			-- only a later filter in list
		once
			Result := <<
				partially_outside_fields_alt_filter,
				overlaps_other_alt_filter,
				size_too_small_filter,
				wafer_etest_id_unassigned_filter,
				unlocked_or_unassigned_filter,
				too_close_to_edge_alt_filter,
				too_close_to_object_alt_filter,
				scribe_cut_die_too_big_filter,
				unisolated_in_cut_die_alt_filter,
				uncentered_in_cut_die_filter,
				misaligned_scribe_alt_filter,
				small_scribe_and_filler_alt_filter
				>>
		end

	cut_patterns: DYNAMIC_LIST [RETICLE_VIEW_CUT_PATTERN]
			-- List of cut patterns, in Cut mode

	last_cut_pattern: RETICLE_VIEW_CUT_PATTERN
			-- Last cut pattern built

	pack_mode_context: RETICLE_VIEW_CONTEXT
			-- Context saved when switch out of Pack mode and
			-- into Cut mode

	no_cut_patterns_context: RETICLE_VIEW_CONTEXT
			-- Context when there are no cut patterns in
			-- Cut mode

	final_pack_mode_view: RETICLE_NAMED_VIEW
			-- Current named view when leaving Pack mode and
			-- switching to Cut mode

	cached_fields: LIST [like anchored_view_element]
			-- List of fields of view

	is_locked_mode: BOOLEAN
			-- Is view in Locked mode, where Fix is treated like
			-- Lock and items should be displayed by Locked
			-- status rather than Fixed status?

	copies_table: HASH_TABLE [RETICLE_VIEW_ELEMENT_LIST, PACKING_DIES_MROW]
			-- Table that maps a packing die to the list
			-- of view elements for that die (each
			-- element is one location of die)

	spread_elements: LIST [like anchored_view_element]
			-- List of elements to which Spread operation
			-- should be applied

	containing_elements: LIST [like anchored_view_element]
			-- List of elements which contain mouse pointer
			-- position (for Abut operation)

	target_field: like anchored_view_element
			-- Field which contains mouse pointer position for
			-- Abut operation or Void if none

	abut_source: like anchored_view_element
			-- Source object for Abut operation (to be moved
			-- to abut it to target)

	abut_target: like anchored_view_element
			-- Target object for Abut operation (source is moved
			-- to abut to target), if target is not abutted to
			-- field of view

	abut_move_distance: POINT
			-- Distance to move selected object for Abut operation
			-- if no errors

	align_target: like anchored_view_element
			-- Target object for align operation

	is_left_or_bottom_resize: BOOLEAN
			-- Does resize of scribe move the left edge
			-- (for horizontal scribe) or bottom edge (for
			-- vertical scribe)?  If not, it moves right or
			-- top edge

	largest_spread_dimension: DOUBLE
			-- Width (or height) of tallest (or widest)
			-- element to which vertical (or horizontal)
			-- Spread operation is about to be applied

	fields_and_edges: LIST [FIELD_ELEMENT_EDGES]
			-- List of fields and the sorted lists of
			-- horizontal and vertical edges of objects that
			-- intersect each field

	last_element_list: RETICLE_VIEW_ELEMENT_LIST

	last_new_element: like anchored_view_element
			-- Last new element added by `add_new_element'

	current_changes: ARRAYED_LIST [RETICLE_SIMPLE_CHANGE]
			-- List of changes made so far by last operation
			-- that makes changes that go on the history list

	initial_locked_mode: BOOLEAN
			-- Value of `is_locked_mode' at start of "change"
			-- command, which goes into history list

	initial_view: RETICLE_NAMED_VIEW
			-- Named view displayed at start of "change" command,
			-- which goes into history list

	initial_selected_elements: ARRAYED_LIST [like anchored_view_element]
			-- List of elements selected at start of operation
			-- that makes changes that go on the history list

	final_selected_elements: ARRAYED_LIST [like anchored_view_element]
			-- List of elements selected at end of operation
			-- that makes changes that go on the history list

	Model_bounding_box_extra_fraction: DOUBLE = 0.05
			-- Fraction by which to enlarge bounding box of
			-- model when displaying entire model or one
			-- field of model

	font: EV_FONT
			-- Reusable default font.  Avoids need to create new
			-- objects repeatedly

	rubber_box: EV_RECTANGLE
			-- Last rubber box drawn on canvas, if width and
			-- height are > 0

	operation_start_location: EV_COORDINATE
			-- View coordinates where last operation started

	selection_bounding_box: RECTANGLE
			-- Minimum bounding box in model coordinates of
			-- all selected elements

	up_limit: DOUBLE
			-- Maximum distance selected items (or a copy of them)
			-- can be moved up from current position, in model
			-- units

	down_limit: DOUBLE
			-- Maximum distance selected items (or a copy of them)
			-- can be moved down from current position, in model
			-- units

	left_limit: DOUBLE
			-- Maximum distance selected items (or a copy of them)
			-- can be moved left from current position, in model
			-- units

	right_limit: DOUBLE
			-- Maximum distance selected items (or a copy of them)
			-- can be moved right from current position, in model
			-- units

	up_pixel_limit: INTEGER
			-- Maximum distance selected items (or a copy of them)
			-- can be moved up from current position, in pixels

	down_pixel_limit: INTEGER
			-- Maximum distance selected items (or a copy of them)
			-- can be moved down from current position, in pixels

	left_pixel_limit: INTEGER
			-- Maximum distance selected items (or a copy of them)
			-- can be moved left from current position, in pixels

	right_pixel_limit: INTEGER
			-- Maximum distance selected items (or a copy of them)
			-- can be moved right from current position, in pixels

	sorter: LIST_BUBBLE_SORTER [like anchored_view_element]
			-- Sorter for view elements

	quick_sorter: LIST_QUICK_SORTER [like anchored_view_element]
			-- Quick sorter for view elements

	size_x_desc_comparator: SIZE_X_DESC_COMPARATOR [like anchored_view_element]
			-- Comparator to sort view elements by descending width

	size_y_desc_comparator: SIZE_Y_DESC_COMPARATOR [like anchored_view_element]
			-- Comparator to sort view elements by descending
			-- height

	scan_x_comparator: RETICLE_SCANNABLE_X_COMPARATOR [like anchored_view_element]
			-- Comparator to sort view elements by gridded center
			-- in X, with objects that are fixed in X sorting
			-- before those that are not in case of ties

	scan_y_comparator: RETICLE_SCANNABLE_Y_COMPARATOR [like anchored_view_element]
			-- Comparator to sort view elements by gridded center
			-- in Y, with objects that are fixed in Y sorting
			-- before those that are not in case of ties

	packing_name_asc_comparator: PACKING_NAME_ASC_COMPARATOR [like anchored_view_element]
			-- Comparator to sort view elements by
			-- ascending packing name

	packing_name_desc_comparator: PACKING_NAME_DESC_COMPARATOR [like anchored_view_element]
			-- Comparator to sort view elements by descending
			-- packing name

	spread_by_width_asc_comparator: SPREADABLE_BY_WIDTH_ASC_COMPARATOR [like anchored_view_element]
			-- Comparator to sort view elements by
			-- major-to-minor gridded size in X
			-- ascending, gridded size in Y ascending
			-- and packing name

	spread_by_width_desc_comparator: SPREADABLE_BY_WIDTH_DESC_COMPARATOR [like anchored_view_element]
			-- Comparator to sort view elements by
			-- major-to-minor gridded size in X
			-- descending, gridded size in Y descending
			-- and packing name

	spread_by_height_asc_comparator: SPREADABLE_BY_HEIGHT_ASC_COMPARATOR [like anchored_view_element]
			-- Comparator to sort view elements by
			-- major-to-minor gridded size in y
			-- ascending, gridded size in X ascending
			-- and packing name

	spread_by_height_desc_comparator: SPREADABLE_BY_HEIGHT_DESC_COMPARATOR [like anchored_view_element]
			-- Comparator to sort view elements by
			-- major-to-minor gridded size in y
			-- descending, gridded size in X descending
			-- and packing name

	spread_by_x_position_asc_comparator: SPREADABLE_BY_X_POSITION_ASC_COMPARATOR [like anchored_view_element]
			-- Comparator to sort view elements by
			-- major-to-minor gridded center in X ascending, 
			-- gridded center in Y  ascending and packing name

	spread_by_x_position_desc_comparator: SPREADABLE_BY_X_POSITION_DESC_COMPARATOR [like anchored_view_element]
			-- Comparator to sort view elements by
			-- major-to-minor gridded center in X
			-- descending, gridded center in Y descending
			-- and packing name

	spread_by_y_position_asc_comparator: SPREADABLE_BY_Y_POSITION_ASC_COMPARATOR [like anchored_view_element]
			-- Comparator to sort view elements by
			-- major-to-minor gridded center in y ascending,
			-- gridded center in X ascending and packing name

	spread_by_y_position_desc_comparator: SPREADABLE_BY_Y_POSITION_DESC_COMPARATOR [like anchored_view_element]
			-- Comparator to sort view elements by
			-- major-to-minor gridded center in y
			-- descending, gridded center in X descending
			-- and packing name

	temp_rectangle: RECTANGLE
			-- Rectangle used for temporary purposes, to avoid
			-- creation of extra objects

	temp_ev_rectangle: EV_RECTANGLE
			-- Rectangle used for temporary purposes, to avoid
			-- creation of extra objects

	anchored_view_element: RETICLE_VIEW_ELEMENT
			-- Type of elements in view
		do
		end

	anchored_model_element: RETICLE_MODEL_ELEMENT
			-- Type of elements in model
		do
		end

	anchored_model: RETICLE_MODEL
			-- Type of model
		do
		end

	Changed_region_threshold: INTEGER = 25
			-- Redraw just changed region if number if
			-- `changed_region_count' does not exceed this value.
			-- Otherwise, redraw everything

	Resize_threshold: INTEGER = 3
			-- Ignore window resizes if the new drawing area
			-- width or height is less than this value

end
