
indexing
	
	description: "A map for a specific run which shows the payload area %
		%and includes details of dies inside the payload"
	author: "David Hollenberg"
	date: "January 24, 2005"

deferred class DETAILED_PAYLOAD_MAP
inherit
	PAYLOAD_MAP
		export
			{ANY} last_page, last_title, last_drawing
		end
	DIE_LOCATIONS_MAP
	
feature {NONE} -- Implementation

	generate_drawing_common_part
			-- Generate part of drawing that is common
			-- to a group of maps
		local
			rect: DRW_RECTANGLE
			pb: CONST_RECTANGLE
			locs: LIST [DIE_LOCATIONS_EROW]
			loc: DIE_LOCATIONS_EROW
			size: CONST_POINT
			color: DRW_COLOR
		do
			pb := payload_box
			create rect.make (pb.size_x, pb.size_y, pb.center_x, pb.center_y)
			rect.set_stroke_width (thick_width)
			last_drawing.extend (rect)
			
			locs := die_locations.item (field_name)
			if locs /= Void then
				from
					locs.start
				until
					locs.after
				loop
					loc := locs.item
					if loc.included_in_payload_maps then
						size := loc.printed_size_on_wafer
						create rect.make (size.x, size.y, loc.location_x, loc.location_y)
						rect.set_stroke_width (thick_width)
						color := location_fill_color (loc)
						if color /= Void then
							rect.set_fill (color)
						end
						last_drawing.extend (rect)
					end
					locs.forth
				end
			end
		end

	generate_drawing_specific_part
			-- Generate part of drawing that is specific
			-- to a particular map
			
		local
			locs: LIST [DIE_LOCATIONS_EROW]
			loc: DIE_LOCATIONS_EROW
		do
			locs := die_locations.item (field_name)
			if locs /= Void then
				from
					locs.start
				until
					locs.after
				loop
					loc := locs.item
					if loc.included_in_payload_maps then
						generate_drawing_for_location (loc)
					end
					locs.forth
				end
			end
			add_top_label
		end

	generate_drawing_for_location (loc: DIE_LOCATIONS_EROW)
			-- Generate part of drawing for `loc' that is specific
			-- to a particular map
		require
			loc_exists: loc /= Void
		deferred
		end

	current_page_label: STRING
			-- Descriptive label for current page of map
			-- (alternative to page number)
		do
			Result := "Field " + field_name
		end

	add_top_label
			-- Add label to drawing indicating top of reticle
		local
			b, box: DRW_BASIC_RECTANGLE
			max_hw, h: DOUBLE
		do
			b := last_drawing.bounding_box
			max_hw := b.width.max (b.height)
			h := max_hw * label_fraction
			create box.make (b.width, h, b.x, b.upper_left.y + h)
			last_drawing.extend (text_in_box (Top_label, box))
		end

	location_fill_color (loc: DIE_LOCATIONS_EROW): DRW_COLOR
			-- Fill color to be used for interior of `loc'
		require
			loc_not_void: loc /= Void
		deferred
		end

	set_stroke_and_fill_colors
			-- Set colors used for stroke and fill
		do
			create test_vehicle_fill.make_with_rgb (0.5, 1.0, 1.0)
			create scribe_fill.make_with_rgb (1.0, 0.5, 0.5)
			create vendor_geometry_fill.make_with_rgb (1.0, 0.75, 1.0)
		end

	test_vehicle_fill: DRW_COLOR
			-- Fill for interior of test vehicles on some maps

	vendor_geometry_fill: DRW_COLOR
			-- Fill for interior of vendor geometry and vendor
			-- ghosts on some maps

	scribe_fill: DRW_COLOR
			-- Fill for scribes on some maps

	field_name: STRING
			-- Name of field for which page is being generated

	Top_label: STRING = "Top"
			-- Label indicating top of drawing

end
