
indexing
	
	description: "Interface to allow viewing/setting of payload and %
		%field sizes while packing"
	author: "David Hollenberg"
	date: "September 30, 2008"

deferred class RC_PACKING_PAYLOAD_SIZES_IFC
inherit
	ABSTRACT_INTERFACE
	GRIDABLE

feature {NONE} -- Creation

	make_interface
			-- Create interface
		do
			create accept_actions
			create cancel_actions
		end

feature -- Modification

	set_session (sess: like session)
			-- Set `session' to `sess'
		require
			sess_exists: sess /= Void
		do
			session := sess
		ensure
			session_set: session = sess
		end

feature -- Properties

	is_read_only: BOOLEAN
			-- Is interface read-only (user can view current
			-- data but cannot change it)?

	is_pack_mode: BOOLEAN
			-- Are we in Pack mode rather than Cut mode?

	current_payload_size: CONST_POINT
			-- Current size of payload
		do
			Result := current_payload_size_agent.item ([])
		end

	current_field_size: CONST_POINT
			-- Current size of field
		do
			Result := current_field_size_agent.item ([])
		end

	current_external_kerf_size: CONST_POINT
			-- Current size of external kerf
		local
		do
			Result := current_field_size - current_payload_size
		end

	current_payload_size_agent: FUNCTION [ANY, TUPLE, CONST_POINT]
			-- Agent that returns current size of payload

	current_field_size_agent: FUNCTION [ANY, TUPLE, CONST_POINT]
			-- Agent that returns current size of field

	session: RC_ABSTRACT_PACKING_SESSION
			-- Session used to do packing

	accept_actions: ACTION_SEQUENCE [TUPLE [DOUBLE, DOUBLE, DOUBLE, DOUBLE]]
			-- Action to be invoked when user accepts
			-- current payload and field sizes

	cancel_actions: CLOSED_ACTION_SEQUENCE
			-- Action to be invoked when user clicks on Cancel

feature -- Modification

	set_is_read_only (b: like is_read_only)
			-- Set `is_read_only' to `b'
		do
			is_read_only := b
		end

	set_is_pack_mode (b: like is_pack_mode)
			-- Set `is_pack_mode' to `b'
		do
			is_pack_mode := b
		end

	set_current_payload_size_agent (an_agent: like current_payload_size_agent)
			-- Set `current_payload_size_agent' to
			-- `an_agent'
		require
			an_agent_not_void: an_agent /= Void
		do
			current_payload_size_agent := an_agent
		ensure
			agent_set: current_payload_size_agent = an_agent
		end

	set_current_field_size_agent (an_agent: like current_field_size_agent)
			-- Set `current_field_size_agent' to
			-- `an_agent'
		require
			an_agent_not_void: an_agent /= Void
		do
			current_field_size_agent := an_agent
		ensure
			agent_set: current_field_size_agent = an_agent
		end

end
