
indexing
	
	description: "Window to allow viewing/setting of payload and %
		%field sizes while packing"
	author: "David Hollenberg"
	date: "September 30, 2008"

class RC_PACKING_PAYLOAD_SIZES_WINDOW

inherit
	RC_PACKING_PAYLOAD_SIZES_IFC
		undefine
			default_create
		redefine
			set_is_pack_mode
		end
	PAYLOAD_KERF_FIELD_INTERFACE
		undefine
			default_create
		end
	WINDOW_INTERFACE
		redefine
			enable_active,
			disable_active
		end
	MOSIS_GRAPHICAL_UTILITIES
		export
			{NONE} all
		undefine
			default_create
		end
	NUMBER_UTILITIES_ACCESS
		export
			{NONE} all
		undefine
			default_create
		end
	STRING_UTILITIES_ACCESS
		export
			{NONE} all
		undefine
			default_create
		end

create
	make

feature {NONE} -- Creation

	make (w: EV_TITLED_WINDOW)
			-- Create interface in window `w'
		require
			window_exists: w /= Void
			window_valid: not w.is_destroyed
		do
			make_interface
			make_proxy (w)
		end

feature -- Operations

	reset
			-- Move focus back into window
		do
			enable_active
		end

	refresh
			-- Refresh window with current width values
		do
			payload_size_x_field.set_text (formatted_current_payload_size_x)
			payload_size_y_field.set_text (formatted_current_payload_size_y)
			field_size_x_field.set_text (formatted_current_field_size_x)
			field_size_y_field.set_text (formatted_current_field_size_y)
			external_kerf_size_x_field.set_text (formatted_current_external_kerf_size_x)
			external_kerf_size_y_field.set_text (formatted_current_external_kerf_size_y)
		end

feature -- Modification

	set_is_pack_mode (b: like is_pack_mode)
			-- Set `is_pack_mode' to `b'
		do
			precursor (b)
			if is_active then
				if not is_read_only and is_pack_mode then
					ok_button.enable_sensitive
				else
					ok_button.disable_sensitive
				end
			end
		end

	enable_active
			-- Make interface active
		do
			precursor
			if not is_read_only and is_pack_mode then
				ok_button.enable_sensitive
			else
				ok_button.disable_sensitive
			end
			cancel_button.enable_sensitive
		end

	disable_active
			-- Make interface inactive
			-- (can look but cannot initiate most actions)
		do
			precursor
			ok_button.disable_sensitive
			cancel_button.disable_sensitive
		end

feature {NONE} -- Implementation

	build_interface
			-- Build interface
		local
			cols: WIDGET_COLUMN_SET
			buttons: HORIZONTAL_BUTTON_SET
			main_vbox: EV_VERTICAL_BOX
			box: EV_BOX
			label: EV_LABEL
			hbox: EV_HORIZONTAL_BOX
		do
			create main_vbox
			main_vbox.set_padding (user_interface.padding)
			main_vbox.set_border_width (user_interface.border_width)
			create cols.make (2)
			cols.set_padding (user_interface.padding)
			
			create payload_size_x_field.make_with_text (formatted_current_payload_size_x)
			cols.append_row (<< gui.label_with_text (Payload_size_x_text), payload_size_x_field >>)
			
			create payload_size_y_field.make_with_text (formatted_current_payload_size_y)
			cols.append_row (<< gui.label_with_text (Payload_size_y_text), payload_size_y_field >>)
			
			create external_kerf_size_x_field.make_with_text (formatted_current_external_kerf_size_x)
			cols.append_row (<< gui.label_with_text (External_kerf_size_x_text), external_kerf_size_x_field >>)
			
			create external_kerf_size_y_field.make_with_text (formatted_current_external_kerf_size_y)
			cols.append_row (<< gui.label_with_text (External_kerf_size_y_text), external_kerf_size_y_field >>)
			
			create field_size_x_field.make_with_text (formatted_current_field_size_x)
			cols.append_row (<< gui.label_with_text (Field_size_x_text), field_size_x_field >>)
			field_size_x_field.disable_edit
			
			create field_size_y_field.make_with_text (formatted_current_field_size_y)
			cols.append_row (<< gui.label_with_text (Field_size_y_text), field_size_y_field >>)
			field_size_y_field.disable_edit
	
			if is_read_only then
				disable_field_editing
			else
				add_change_actions
			end
			cols.disable_column_expand (1)
			cols.align_column_right (1)
			box := cols.box
			main_vbox.extend (box)
			main_vbox.disable_item_expand (box)
			
			create label.make_with_text ("Size resolution is " + grid.size_resolution_text + " microns")
			create hbox
			hbox.extend (label)
			hbox.disable_item_expand (label)
			main_vbox.extend (hbox)
			main_vbox.disable_item_expand (hbox)
			
			create buttons.make_with_text_and_actions (<<
				"OK", "Cancel" >>,
				<< agent process_ok, agent process_cancel >>)
			ok_button := buttons.i_th (1)
			cancel_button := buttons.i_th (2)
			if is_read_only then
				ok_button.disable_sensitive
			end
			box := buttons.column_set.box
			
			main_vbox.extend (box)
			main_vbox.disable_item_expand (box)
			
			gui.set_box_background_recursive (main_vbox, background_color)
			extend (main_vbox)
			close_request_actions.extend (agent process_cancel)
			gui.add_cancel_accelerator (window, agent process_cancel)
			gui.do_once_on_idle (agent post_display_actions)
			set_minimum_width (400)
		end
		
	post_display_actions
		do
			reset
		end

	process_ok
		do
			if is_active then
				disable_active
				clear_validation_errors
				if is_pack_mode then
					validate_payload_and_kerf_fields
					if is_input_valid then
						accept_actions.call ([payload_size_x_field.text.to_double, payload_size_y_field.text.to_double, field_size_x_field.text.to_double, field_size_y_field.text.to_double])
					else
						inform ("Some input values are missing or invalid.%N%N" + validation_errors)
					end
				else
					inform ("Cannot change payload and kerf sizes in Cut mode")
				end
				enable_active
			end
		end

	process_cancel
		do
			if is_active then
				disable_active
				cancel_actions.call ([])
			end
		end

	formatted_current_payload_size_x: STRING
			-- `current_payload_size' X component rounded
			-- to `grid_scale' and formatted nicely
		do
			Result := grid.formatted_number (current_payload_size.x)
		end

	formatted_current_payload_size_y: STRING
			-- `current_payload_size' Y component rounded
			-- to `grid_scale' and formatted nicely
		do
			Result := grid.formatted_number (current_payload_size.y)
		end

	formatted_current_field_size_x: STRING
			-- `current_field_size' X component rounded
			-- to `grid_scale' and formatted nicely
		do
			Result := grid.formatted_number (current_field_size.x)
		end

	formatted_current_field_size_y: STRING
			-- `current_field_size' Y component rounded
			-- to `grid_scale' and formatted nicely
		do
			Result := grid.formatted_number (current_field_size.y)
		end

	formatted_current_external_kerf_size_x: STRING
			-- `current_external_kerf_size' X component rounded
			-- to `grid_scale' and formatted nicely
		do
			Result := grid.formatted_number (current_external_kerf_size.x)
		end

	formatted_current_external_kerf_size_y: STRING
			-- `current_external_kerf_size' Y component rounded
			-- to `grid_scale' and formatted nicely
		do
			Result := grid.formatted_number (current_external_kerf_size.y)
		end

	initial_text (a_text: STRING): STRING
			-- `a_text' possibly prefixed with "Initial"
		do
			Result := a_text
		end

	ok_button: EV_BUTTON
			-- Button to hide window
	
	cancel_button: EV_BUTTON
			-- Button to hide window
	
end
